// Copyright (c) 1994-2009 Nokia Corporation and/or its subsidiary(-ies).
// All rights reserved.
// This component and the accompanying materials are made available
// under the terms of the License "Eclipse Public License v1.0"
// which accompanies this distribution, and is available
// at the URL "http://www.eclipse.org/legal/epl-v10.html".
//
// Initial Contributors:
// Nokia Corporation - initial contribution.
//
// Contributors:
//
// Description:
// e32test\buffer\t_collate.cpp
// Overview:
// Test Unicode collations.
// API Information:
// CompareC, TCollationMethod, TCollationKeyTable.
// Details:
// - Check the collation for characters, accents, different cases and standard 
// alphabetical ordering is as expected.
// - Check collation using standard collation method, overriding the collation tables, 
// collation keys is as expected.
// - Check collation for simplified Chinese characters is as expected.
// - Check collation and compare the Chinese ideological characters to other 
// characters which are not included in the Chinese collation table are as expected.
// - Check collation and compare Chinese to English alphabets.
// - Check collation and compare Chinese to Hiragana characters.
// - Check collation and compare Chinese to Katakana characters.
// - Check collation and compare Hiragana to a Katakana characters.
// - Check collation and compare full width and half width digits and letters.
// - Set the current collation key table to standard table, collation keys to default keys,
// constants to specified values, compare data and check it is as expected.
// Platforms/Drives/Compatibility:
// All 
// Assumptions/Requirement/Pre-requisites:
// Failures and causes:
// Base Port information:
// 
//

#include <e32test.h>
#include <collate.h>	  
#include "../../../kernel/eka/euser/unicode/collateimp.h"
#include "u32std.h"

#ifdef __VC32__
    // Solve compilation problem caused by non-English locale
    #pragma setlocale("english")
#endif

LOCAL_D RTest test(_L("T_COLLATE"));


#ifndef _DEBUG
#pragma warning (disable :4710) //Function not expanded 
#pragma warning (disable :4700) //Local variable used without being initialized
#endif

///***************** copied from locale euser source code ***********************
static const TCollationMethod TheCollationMethod[] =
	{
		{
		KUidBasicCollationMethod,				// this is the standard unlocalised method
		NULL,									// null means use the standard table
		NULL,									// there's no override table
		0										// the flags are standard
		}
	};
static const TCollationDataSet TheCollationDataSet =
	{
	TheCollationMethod,
	1
	};
const LCharSet TheCharSet =
	{
	NULL,
	&TheCollationDataSet
	};
const LCharSet* GetLocaleCharSet()
	{
	return &TheCharSet;
	}
///*******************************************************************************




void test_unicode_collation(const TDesC& x,const TDesC& y,TInt desired_order,
							TInt level = 3,const TCollationMethod* method = NULL)
	{
	TInt order = x.CompareC(y,level,method);

	// Normalise order to -1, 0 or 1.
	if (order < 0)
		order = -1;
	else if (order > 0)
		order = 1;

	test(order == desired_order);
	}

#undef USE_SWEDISH_TABLE
#ifdef USE_SWEDISH_TABLE
static const TUint32 TheSwedishKey[] = 
	{
	0x8f60109,0x8f70109,0x8f80109,0x8f60121,0x8f70121,0x8f80121,0x8dd0109,0x8dd0121,
	0x8c50121,0x8c50109,
	};

static const TUint32 TheSwedishIndex[] = 
	{
	0x570008,0x770009,
	};

static const TUint16 TheSwedishStringElement[] = 
	{
	0x2,0x61,0x30a,0x2,0x61,0x308,0x2,0x6f,
	0x308,0x2,0x41,0x30a,0x2,0x41,0x308,0x2,
	0x4f,0x308,0x2,0x75,0x308,0x2,0x55,0x308,
	};

static const TUint32 TheSwedishStringIndex[] = 
	{
	0xc0004,0x90003,0xf0005,0x150007,0x30001,0x0,0x60002,0x120006,
	};

static const TCollationKeyTable TheSwedishTable = 
	{ TheSwedishKey, TheSwedishIndex, 2, TheSwedishStringElement, TheSwedishStringIndex, 8 };

static const TCollationMethod TheSwedishMethod =
	{ 0, NULL, &TheSwedishTable, 0 };
#endif

static const TCollationMethod TheSwapCaseMethod =
	{ 0, NULL, NULL, TCollationMethod::ESwapCase };

static const TUint32 TheKanaKey[] = 
	{
	0x102c013c,0xc949,0x103f013c,0xc949,0x1044013c,0xc949,0x1049013c,0xc949,
	0x104e013c,0xc949,0x1053013c,0xc949,0x1058013c,0xc949,0x105d013c,0xc949,
	0x1062013c,0xc949,0x1067013c,0xc949,0x106c013c,0xc949,0x1071013c,0xc949,
	0x1076013c,0xc949,0x107b013c,0xc949,0x1083013c,0xc949,0x1088013c,0xc949,
	0x109c013c,0xc949,0x10a3013c,0xc949,0x10aa013c,0xc949,0x10b1013c,0xc949,
	0x10b8013c,0xc949,0x109c013c,0xca49,0x10a3013c,0xca49,0x10aa013c,0xca49,
	0x10b1013c,0xca49,0x10b8013c,0xca49,0x10200134,0x10200135,0x10260134,0x10260135,
	0x102c0134,0x102c0135,0x10330134,0x10330135,0x10390134,0x10390135,0x103f0134,0x10200135,
	0x10440134,0x10260135,0x10490134,0x102c0135,0x104e0134,0x10330135,0x10530134,0x10390135,
	0x103f0134,0xc948,0x10200135,0x10440134,0xc948,0x10260135,0x10490134,0xc948,
	0x102c0135,0x104e0134,0xc948,0x10330135,0x10530134,0xc948,0x10390135,0x10580134,
	0x10200135,0x105d0134,0x10260135,0x10620134,0x102c0135,0x10670134,0x10330135,0x106c0134,
	0x10390135,0x10580134,0xc948,0x10200135,0x105d0134,0xc948,0x10260135,0x10620134,
	0xc948,0x102c0135,0x10670134,0xc948,0x10330135,0x106c0134,0xc948,0x10390135,
	0x10710134,0x10200135,0x10760134,0x10260135,0x107b0134,0x102c0135,0x10830134,0x10330135,
	0x10880134,0x10390135,0x10710134,0xc948,0x10200135,0x10760134,0xc948,0x10260135,
	0x107b0134,0xc948,0x102c0135,0x10830134,0xc948,0x10330135,0x10880134,0xc948,
	0x10390135,0x108d0134,0x10200135,0x10900134,0x10260135,0x10930134,0x102c0135,0x10960134,
	0x10330135,0x10990134,0x10390135,0x109c0134,0x10200135,0x10a30134,0x10260135,0x10aa0134,
	0x102c0135,0x10b10134,0x10330135,0x10b80134,0x10390135,0x109c0134,0xc948,0x10200135,
	0x10a30134,0xc948,0x10260135,0x10aa0134,0xc948,0x102c0135,0x10b10134,0xc948,
	0x10330135,0x10b80134,0xc948,0x10390135,0x109c0134,0xca08,0x10200135,0x10a30134,
	0xca08,0x10260135,0x10aa0134,0xca08,0x102c0135,0x10b10134,0xca08,0x10330135,
	0x10b80134,0xca08,0x10390135,0x10bf0134,0x10200135,0x10c20134,0x10260135,0x10c50134,
	0x102c0135,0x10c80134,0x10330135,0x10cb0134,0x10390135,0x10ce0134,0x10200135,0x10d40134,
	0x102c0135,0x10da0134,0x10390135,0x10e00134,0x10200135,0x10e30134,0x10260135,0x10e60134,
	0x102c0135,0x10e90134,0x10330135,0x10ec0134,0x10390135,0x10ef0134,0x10200135,0x103f0144,
	0x10200145,0x10440144,0x10260145,0x10490144,0x102c0145,0x104e0144,0x10330145,0x10530144,
	0x10390145,0x103f0144,0xc948,0x10200145,0x10440144,0xc948,0x10260145,0x10490144,
	0xc948,0x102c0145,0x104e0144,0xc948,0x10330145,0x10530144,0xc948,0x10390145,
	0x10580144,0x10200145,0x105d0144,0x10260145,0x10620144,0x102c0145,0x10670144,0x10330145,
	0x106c0144,0x10390145,0x10580144,0xc948,0x10200145,0x105d0144,0xc948,0x10260145,
	0x10620144,0xc948,0x102c0145,0x10670144,0xc948,0x10330145,0x106c0144,0xc948,
	0x10390145,0x10710144,0x10200145,0x10760144,0x10260145,0x107b0144,0x102c0145,0x10830144,
	0x10330145,0x10880144,0x10390145,0x10710144,0xc948,0x10200145,0x10760144,0xc948,
	0x10260145,0x107b0144,0xc948,0x102c0145,0x10830144,0xc948,0x10330145,0x10880144,
	0xc948,0x10390145,0x108d0144,0x10200145,0x10900144,0x10260145,0x10930144,0x102c0145,
	0x10960144,0x10330145,0x10990144,0x10390145,0x109c0144,0x10200145,0x10a30144,0x10260145,
	0x10aa0144,0x102c0145,0x10b10144,0x10330145,0x10b80144,0x10390145,0x109c0144,0xc948,
	0x10200145,0x10a30144,0xc948,0x10260145,0x10aa0144,0xc948,0x102c0145,0x10b10144,
	0xc948,0x10330145,0x10b80144,0xc948,0x10390145,0x109c0144,0xca08,0x10200145,
	0x10a30144,0xca08,0x10260145,0x10aa0144,0xca08,0x102c0145,0x10b10144,0xca08,
	0x10330145,0x10b80144,0xca08,0x10390145,0x10bf0144,0x10200145,0x10c20144,0x10260145,
	0x10c50144,0x102c0145,0x10c80144,0x10330145,0x10cb0144,0x10390145,0x10ce0144,0x10200145,
	0x10d40144,0x102c0145,0x10da0144,0x10390145,0x10e00144,0x10200145,0x10e30144,0x10260145,
	0x10e60144,0x102c0145,0x10e90144,0x10330145,0x10ec0144,0x10390145,0x10ef0144,0x10200145,
	0x10f40144,0x10260145,0x10f60144,0x10330145,0x10f80144,0x10390145,0x10ef0144,0xc948,
	0x10200145,0x10f40144,0xc948,0x10260145,0x102c0144,0xc948,0x102c0145,0x10f60144,
	0xc948,0x10330145,0x10f80144,0xc948,0x10390145,0x103f013c,0x1020013d,0x1044013c,
	0x1026013d,0x1049013c,0x102c013d,0x104e013c,0x1033013d,0x1053013c,0x1039013d,0x103f013c,
	0xc948,0x1020013d,0x1044013c,0xc948,0x1026013d,0x1049013c,0xc948,0x102c013d,
	0x104e013c,0xc948,0x1033013d,0x1053013c,0xc948,0x1039013d,0x1058013c,0x1020013d,
	0x105d013c,0x1026013d,0x1062013c,0x102c013d,0x1067013c,0x1033013d,0x106c013c,0x1039013d,
	0x1058013c,0xc948,0x1020013d,0x105d013c,0xc948,0x1026013d,0x1062013c,0xc948,
	0x102c013d,0x1067013c,0xc948,0x1033013d,0x106c013c,0xc948,0x1039013d,0x1071013c,
	0x1020013d,0x1076013c,0x1026013d,0x107b013c,0x102c013d,0x1083013c,0x1033013d,0x1088013c,
	0x1039013d,0x1071013c,0xc948,0x1020013d,0x1076013c,0xc948,0x1026013d,0x107b013c,
	0xc948,0x102c013d,0x1083013c,0xc948,0x1033013d,0x1088013c,0xc948,0x1039013d,
	0x108d013c,0x1020013d,0x1090013c,0x1026013d,0x1093013c,0x102c013d,0x1096013c,0x1033013d,
	0x1099013c,0x1039013d,0x109c013c,0x1020013d,0x10a3013c,0x1026013d,0x10aa013c,0x102c013d,
	0x10b1013c,0x1033013d,0x10b8013c,0x1039013d,0x109c013c,0xc948,0x1020013d,0x10a3013c,
	0xc948,0x1026013d,0x10aa013c,0xc948,0x102c013d,0x10b1013c,0xc948,0x1033013d,
	0x10b8013c,0xc948,0x1039013d,0x109c013c,0xca48,0x1020013d,0x10a3013c,0xca48,
	0x1026013d,0x10aa013c,0xca48,0x102c013d,0x10b1013c,0xca48,0x1033013d,0x10b8013c,
	0xca48,0x1039013d,0x10bf013c,0x1020013d,0x10c2013c,0x1026013d,0x10c5013c,0x102c013d,
	0x10c8013c,0x1033013d,0x10cb013c,0x1039013d,0x10ce013c,0x1020013d,0x10d4013c,0x102c013d,
	0x10da013c,0x1039013d,0x10e0013c,0x1020013d,0x10e3013c,0x1026013d,0x10e6013c,0x102c013d,
	0x10e9013c,0x1033013d,0x10ec013c,0x1039013d,0x10ef013c,0x1020013d,0x10ef013c,0xc948,
	0x1020013d,0xca09,0x2010f,0x20113,0x4010b,0x4010f,0x5010b,0x5010f,
	0x8010b,0x8010f,0xa010f,0xa014b,0xc010b,0xc010f,0xe010b,0xe010f,
	0xe013f,0x10010b,0x10010f,0x12010b,0x12010f,0x14c90b,0x14c90f,0x16ca0b,
	0x16ca0f,0x17010b,0x18030f,0x1a060b,0x1a060f,0x1c160f,0x1e0d0b,0x1e0d0f,
	0x20290f,0x22020b,0x22020f,0x24010d,0x28010d,0x2c010d,0x2e010d,0x30010d,
	0x32010d,0x34010d,0x36010b,0x36010f,0x38010f,0x3a010f,0x3c0109,0x3c010d,
	0x3e0109,0x3e010d,0x40010d,0x400109,0x42010d,0x440109,0x44010d,0x46010d,
	0x48010d,0x4a010f,0x4c010f,0x4e010b,0x4f010b,0x50010b,0x50010f,0x52010b,
	0x52010f,0x54010b,0x56010b,0x58010b,0x58010f,0x5a010b,0x5a010f,0x5c010b,
	0x5c010f,0x5e010b,0x5e010f,0x60010b,0x62010b,0x64010b,0x66010b,0x68010f,
	0x68010b,0x6a010f,0x6a010b,0x6c010b,0x6e010b,0x6f010b,0x70010b,0x72010b,
	0x72010f,0x74010b,0x74010f,0x76010f,0x78010f,0x7a010f,0x7c010b,0x7c010f,
	0x80010b,0x80010f,0x82010b,0x82010f,0x84010f,0x86010f,0x88010f,0x8a010f,
	0x8c010b,0x8e010f,0x90010f,0x92010f,0x94010d,0x960109,0x96010d,0x980109,
	0x98010d,0x9a010d,0x9c0109,0x9c010d,0x9e010b,0x9e010f,0x100010b,0x100010f,
	0x102010b,0x102010f,0x104010b,0x104010f,0x106010b,0x106010f,0x107010f,0x108010f,
	0x109010f,0x10a010f,0x10b010f,0x10c010f,0x10d010f,0x10e010f,0x10f010f,0x110010f,
	0x111010f,0x112010f,0x113010f,0x114010f,0x115010f,0x116010f,0x117010f,0x118010f,
	0x119010f,0x11a010f,0x11b010f,0x11c010f,0x11d010f,0x11e010f,0x11f010f,0x120010f,
	0x121010f,0x122010f,0x123010f,0x124010f,0x125010f,0x126010f,0x127010f,0x128010f,
	0x129010f,0x12a010f,0x12b010f,0x12c010f,0x12d010f,0x12e010f,0x12f010f,0x130010f,
	0x131010f,0x132010f,0x133010f,0x134010f,0x135010f,0x136010f,0x137010f,0x138010f,
	0x139010f,0x13b010f,0x13c010d,0x13d010d,0x13e010f,0x13f010f,0x140010f,0x141010f,
	0x142010f,0x143010d,0x144010d,0x145010d,0x146010d,0x147010d,0x148010d,0x149010d,
	0x14a010d,0x14b010d,0x14c010d,0x14d010d,0x14e010d,0x14f010d,0x150010d,0x151010d,
	0x152010d,0x153010d,0x154010d,0x155010d,0x156010d,0x157010d,0x158010d,0x159010d,
	0x15a010d,0x15b010d,0x15c010d,0x15d010d,0x15e010d,0x15f010d,0x160010f,0x161010f,
	0x162010f,0x163010f,0x164010f,0x165010f,0x166010f,0x167010f,0x168010f,0x169010f,
	0x16a010f,0x16b010f,0x16c010f,0x16d010f,0x16e010f,0x16f010f,0x170010f,0x171010f,
	0x172010f,0x174010f,0x175010f,0x176010f,0x177010f,0x178010f,0x179010f,0x17a010d,
	0x17b010d,0x17c010d,0x17d0119,0x17e0119,0x17f0119,0x1800119,0x1810119,0x182010d,
	0x183010d,0x184010d,0x185010d,0x186010d,0x187010d,0x189010f,0x18a010f,0x18b010f,
	0x18c010f,0x18d0111,0x18e0111,0x18f0111,0x1900111,0x1910111,0x1920111,0x1930111,
	0x1940111,0x1950111,0x1960111,0x197010f,0x198010f,0x199010b,0x199010f,0x6c50109,
	0x6c60109,0x6c70109,0x6c80109,0x6c90109,0x6ca0109,0x6cb0109,0x6cc0109,0x6cd0109,
	0x6ce0109,0x6cf0109,0x6cf010d,0x6cf0121,0x6cf0125,0x6e30109,0x6e3010d,0x6e30121,
	0x6e30125,0x6fb0109,0x6fb010d,0x6fb0121,0x6fb0125,0x7070109,0x707010d,0x7070121,
	0x7070125,0x72f0109,0x72f010d,0x72f0121,0x72f0125,0x75b0109,0x75b010d,0x75b0121,
	0x75b0125,0x7630109,0x763010d,0x7630121,0x7630125,0x7830109,0x783010d,0x7830121,
	0x7830125,0x79d0109,0x79d010d,0x79d0121,0x79d0125,0x7b10109,0x7b1010d,0x7b10121,
	0x7b10125,0x7c10109,0x7c1010d,0x7c10121,0x7c10125,0x7cd0109,0x7cd010d,0x7cd0121,
	0x7cd0125,0x7f50109,0x7f5010d,0x7f50121,0x7f50125,0x7fd0109,0x7fd010d,0x7fd0121,
	0x7fd0125,0x8150109,0x815010d,0x8150121,0x8150125,0x82d0109,0x82d010d,0x82d0121,
	0x82d0125,0x8390109,0x839010d,0x8390121,0x8390125,0x8450109,0x845010d,0x8450121,
	0x8450125,0x86d0109,0x86d010d,0x86d0121,0x86d0125,0x8890109,0x889010d,0x8890121,
	0x8890125,0x8ad0109,0x8ad010d,0x8ad0121,0x8ad0125,0x8c50109,0x8c5010d,0x8c50121,
	0x8c50125,0x8d10109,0x8d1010d,0x8d10121,0x8d10125,0x8d90109,0x8d9010d,0x8d90121,
	0x8d90125,0x8dd0109,0x8dd010d,0x8dd0121,0x8dd0125,0x8e90109,0x8e9010d,0x8e90121,
	0x8e90125,0x10200139,0x10200149,0x10200141,0x10260139,0x10260149,0x10260141,0x102c0139,
	0x102c0149,0x102c0141,0x10330139,0x10330149,0x10330141,0x10390139,0x10390149,0x10390141,
	0x103f0135,0x103f0145,0x103f013d,0x10440135,0x10440145,0x1044013d,0x10490135,0x10490145,
	0x1049013d,0x104e0135,0x104e0145,0x104e013d,0x10530135,0x10530145,0x1053013d,0x10580135,
	0x10580145,0x1058013d,0x105d0135,0x105d0145,0x105d013d,0x10620135,0x10620145,0x1062013d,
	0x10670135,0x10670145,0x1067013d,0x106c0135,0x106c0145,0x106c013d,0x10710135,0x10710145,
	0x1071013d,0x10760135,0x10760145,0x1076013d,0x107b0139,0x107b0135,0x107b0149,0x107b0141,
	0x107b0145,0x107b013d,0x10830135,0x10830145,0x1083013d,0x10880135,0x10880145,0x1088013d,
	0x108d0135,0x108d0145,0x108d013d,0x10900135,0x10900145,0x1090013d,0x10930135,0x10930145,
	0x1093013d,0x10960135,0x10960145,0x1096013d,0x10990135,0x10990145,0x1099013d,0x109c0135,
	0x109c0145,0x109c013d,0x10a30135,0x10a30145,0x10a3013d,0x10aa0135,0x10aa0145,0x10aa013d,
	0x10b10135,0x10b10145,0x10b1013d,0x10b80135,0x10b80145,0x10b8013d,0x10bf0135,0x10bf0145,
	0x10bf013d,0x10c20135,0x10c20145,0x10c2013d,0x10c50135,0x10c50145,0x10c5013d,0x10c80135,
	0x10c80145,0x10c8013d,0x10cb0135,0x10cb0145,0x10cb013d,0x10ce0139,0x10ce0135,0x10ce0149,
	0x10ce0141,0x10ce0145,0x10ce013d,0x10d40139,0x10d40135,0x10d40149,0x10d40141,0x10d40145,
	0x10d4013d,0x10da0139,0x10da0135,0x10da0149,0x10da0141,0x10da0145,0x10da013d,0x10e00135,
	0x10e00145,0x10e0013d,0x10e30135,0x10e30145,0x10e3013d,0x10e60135,0x10e60145,0x10e6013d,
	0x10e90135,0x10e90145,0x10e9013d,0x10ec0135,0x10ec0145,0x10ec013d,0x10ef0139,0x10ef0135,
	0x10ef0149,0x10ef0145,0x10ef013d,0x10f40135,0x10f40145,0x10f60135,0x10f60145,0x10f80135,
	0x10f80145,0x10f8013d,0x10fb0135,0x10fb0145,
	};

static const TUint32 TheKanaIndex[] = 
	{
	0x210233,0x22032d,0x230296,0x24028f,0x250294,0x260298,0x270239,0x28025d,
	0x29025f,0x2a029a,0x2b0277,0x2c0226,0x2d0279,0x2e0228,0x2f024e,0x30032f,
	0x310330,0x320331,0x330332,0x340333,0x350334,0x360335,0x370336,0x380337,
	0x390338,0x3a022c,0x3b022e,0x3c0280,0x3d027e,0x3e0282,0x3f0231,0x40029c,
	0x41033b,0x42033f,0x430343,0x440347,0x45034b,0x46034f,0x470353,0x480357,
	0x49035b,0x4a035f,0x4b0363,0x4c0367,0x4d036b,0x4e036f,0x4f0373,0x500377,
	0x51037b,0x52037f,0x530383,0x540387,0x55038b,0x56038f,0x570393,0x580397,
	0x59039b,0x5a039f,0x5b0263,0x5c0250,0x5d0265,0x5e023e,0x5f0241,0x60023b,
	0x610339,0x62033d,0x630341,0x640345,0x650349,0x66034d,0x670351,0x680355,
	0x690359,0x6a035d,0x6b0361,0x6c0365,0x6d0369,0x6e036d,0x6f0371,0x700375,
	0x710379,0x72037d,0x730381,0x740385,0x750389,0x76038d,0x770391,0x780395,
	0x790399,0x7a039d,0x7b0267,0x7c0255,0x7d0269,0x7e0253,0xa20291,0xa30292,
	0xa5028d,0xa7029e,0xa8023d,0xb00289,0xb1027b,0xb4023a,0xb602d7,0xd7027c,
	0xf7027d,0x2010024d,0x2015024c,0x20180259,0x2019025a,0x201c025b,0x201d025c,0x202002d5,
	0x202102d6,0x20250258,0x20260257,0x203002d1,0x2032028a,0x2033028b,0x203b02ac,0x2103028c,
	0x2116030f,0x21210311,0x216002ec,0x216102ed,0x216202ee,0x216302ef,0x216402f0,0x216502f1,
	0x216602f2,0x216702f3,0x216802f4,0x216902f5,0x21700321,0x21710322,0x21720323,0x21730324,
	0x21740325,0x21750326,0x21760327,0x21770328,0x21780329,0x2179032a,0x219002af,0x219102b0,
	0x219202ae,0x219302b1,0x21d202be,0x21d402bf,0x220002c0,0x220202c5,0x220302c1,0x220702c6,
	0x220802b3,0x220b02b4,0x2211031e,0x2212027a,0x221a02cb,0x221d02cd,0x221e0286,0x221f031f,
	0x222002c2,0x22250254,0x222702bb,0x222802bc,0x222902ba,0x222a02b9,0x222b02cf,0x222c02d0,
	0x222e031d,0x22340286,0x223502ce,0x223d02cc,0x225202c8,0x226102c7,0x22660284,0x22670285,
	0x226a02c9,0x226b02ca,0x228202b7,0x228302b8,0x228602b5,0x228702b6,0x22a502c3,0x22bf0320,
	0x231202c4,0x246002d9,0x246102da,0x246202db,0x246302dc,0x246402dd,0x246502de,0x246602df,
	0x246702e0,0x246802e1,0x246902e2,0x246a02e3,0x246b02e3,0x246c02e4,0x246d02e5,0x246e02e6,
	0x246f02e7,0x247002e8,0x247102e9,0x247202ea,0x247302eb,0x25a002a7,0x25a102a6,0x25b202a9,
	0x25b302a8,0x25bc02ab,0x25bd02aa,0x25c602a5,0x25c702a4,0x25cb02a1,0x25ce02a3,0x25cf02a2,
	0x25ef02d8,0x260502a0,0x2606029f,0x26400288,0x26420287,0x266a02d4,0x266d02d3,0x266f02d2,
	0x30010223,0x30020225,0x30030245,0x30050247,0x30060248,0x30070249,0x3008026b,0x3009026c,
	0x300a026d,0x300b026e,0x300c026f,0x300d0271,0x300e0273,0x300f0274,0x30100275,0x30110276,
	0x301202ad,0x301302b2,0x30140261,0x30150262,0x301c0252,0x301d030d,0x301f030e,0x304103a1,
	0x304200d6,0x304303a4,0x304400ce,0x304503a7,0x304600d0,0x304703aa,0x304800d2,0x304903ad,
	0x304a00d4,0x304b03b0,0x304d03b3,0x304f03b6,0x305103b9,0x305303bc,0x305503bf,0x305703c2,
	0x305903c5,0x305b03c8,0x305d03cb,0x305f03ce,0x306103d1,0x306303d4,0x306403d5,0x306603da,
	0x306803dd,0x306a03e0,0x306b03e3,0x306c03e6,0x306d03e9,0x306e03ec,0x306f03ef,0x307203f2,
	0x307503f5,0x307803f8,0x307b03fb,0x307e03fe,0x307f0401,0x30800404,0x30810407,0x3082040a,
	0x3083040d,0x3084040e,0x30850413,0x30860414,0x30870419,0x3088041a,0x3089041f,0x308a0422,
	0x308b0425,0x308c0428,0x308d042b,0x308e042e,0x308f042f,0x30900433,0x30910435,0x30920437,
	0x3093043a,0x30990029,0x309a0221,0x309b0236,0x309c0238,0x309d0244,0x30a103a2,0x30a20178,
	0x30a303a5,0x30a4017b,0x30a503a8,0x30a6017e,0x30a703ab,0x30a80181,0x30a903ae,0x30aa0184,
	0x30ab03b1,0x30ad03b4,0x30af03b7,0x30b103ba,0x30b303bd,0x30b503c0,0x30b703c3,0x30b903c6,
	0x30bb03c9,0x30bd03cc,0x30bf03cf,0x30c103d2,0x30c303d6,0x30c403d8,0x30c603db,0x30c803de,
	0x30ca03e1,0x30cb03e4,0x30cc03e7,0x30cd03ea,0x30ce03ed,0x30cf03f0,0x30d203f3,0x30d503f6,
	0x30d803f9,0x30db03fc,0x30de03ff,0x30df0402,0x30e00405,0x30e10408,0x30e2040b,0x30e3040f,
	0x30e40411,0x30e50415,0x30e60417,0x30e7041b,0x30e8041d,0x30e90420,0x30ea0423,0x30eb0426,
	0x30ec0429,0x30ed042c,0x30ee0430,0x30ef0431,0x30f00434,0x30f10436,0x30f20438,0x30f3043b,
	0x30fb022a,0x30fc024b,0x30fd0243,0x32310317,0x32320318,0x32390319,0x32a40312,0x32a50313,
	0x32a60314,0x32a70315,0x32a80316,0x330302fc,0x330d0300,0x331402f7,0x331802fa,0x332202f8,
	0x33230302,0x33260301,0x332702fb,0x332b0303,0x333602fd,0x333b0305,0x334902f6,0x334a0304,
	0x334d02f9,0x335102fe,0x335702ff,0x337b030c,0x337c031c,0x337d031b,0x337e031a,0x338f0309,
	0x339c0306,0x339d0307,0x339e0308,0x33a1030b,0x33c4030a,0x33cd0310,0x4edd0246,0xfe540230,
	0xff010234,0xff02032e,0xff030297,0xff040290,0xff050295,0xff060299,0xff07032c,0xff08025e,
	0xff090260,0xff0a029b,0xff0b0278,0xff0c0227,0xff0d027a,0xff0e0229,0xff0f024f,0xff1a022d,
	0xff1b022f,0xff1c0281,0xff1d027f,0xff1e0283,0xff1f0232,0xff20029d,0xff21033c,0xff220340,
	0xff230344,0xff240348,0xff25034c,0xff260350,0xff270354,0xff280358,0xff29035c,0xff2a0360,
	0xff2b0364,0xff2c0368,0xff2d036c,0xff2e0370,0xff2f0374,0xff300378,0xff31037c,0xff320380,
	0xff330384,0xff340388,0xff35038c,0xff360390,0xff370394,0xff380398,0xff39039c,0xff3a03a0,
	0xff3b0264,0xff3c0251,0xff3d0266,0xff3e023f,0xff3f0242,0xff40023c,0xff41033a,0xff42033e,
	0xff430342,0xff440346,0xff45034a,0xff46034e,0xff470352,0xff480356,0xff49035a,0xff4a035e,
	0xff4b0362,0xff4c0366,0xff4d036a,0xff4e036e,0xff4f0372,0xff500376,0xff51037a,0xff52037e,
	0xff530382,0xff540386,0xff55038a,0xff56038e,0xff570392,0xff580396,0xff59039a,0xff5a039e,
	0xff5b0268,0xff5c0256,0xff5d026a,0xff5e0252,0xff610224,0xff620270,0xff630272,0xff640222,
	0xff65022b,0xff660439,0xff6703a3,0xff6803a6,0xff6903a9,0xff6a03ac,0xff6b03af,0xff6c0410,
	0xff6d0416,0xff6e041c,0xff6f03d7,0xff70024a,0xff710220,0xff720215,0xff730217,0xff740219,
	0xff75021b,0xff7603b2,0xff7703b5,0xff7803b8,0xff7903bb,0xff7a03be,0xff7b03c1,0xff7c03c4,
	0xff7d03c7,0xff7e03ca,0xff7f03cd,0xff8003d0,0xff8103d3,0xff8203d9,0xff8303dc,0xff8403df,
	0xff8503e2,0xff8603e5,0xff8703e8,0xff8803eb,0xff8903ee,0xff8a03f1,0xff8b03f4,0xff8c03f7,
	0xff8d03fa,0xff8e03fd,0xff8f0400,0xff900403,0xff910406,0xff920409,0xff93040c,0xff940412,
	0xff950418,0xff96041e,0xff970421,0xff980424,0xff990427,0xff9a042a,0xff9b042d,0xff9c0432,
	0xff9e0235,0xff9f0237,0xffe00291,0xffe10293,0xffe202bd,0xffe30240,0xffe4032b,0xffe5028e,
	};

static const TUint16 TheKanaStringElement[] = 
	{
	0x2,0xff73,0xff9e,0x2,0xff76,0xff9e,0x2,0xff77,
	0xff9e,0x2,0xff78,0xff9e,0x2,0xff79,0xff9e,0x2,
	0xff7a,0xff9e,0x2,0xff7b,0xff9e,0x2,0xff7c,0xff9e,
	0x2,0xff7d,0xff9e,0x2,0xff7e,0xff9e,0x2,0xff7f,
	0xff9e,0x2,0xff80,0xff9e,0x2,0xff81,0xff9e,0x2,
	0xff82,0xff9e,0x2,0xff83,0xff9e,0x2,0xff84,0xff9e,
	0x2,0xff8a,0xff9e,0x2,0xff8b,0xff9e,0x2,0xff8c,
	0xff9e,0x2,0xff8d,0xff9e,0x2,0xff8e,0xff9e,0x2,
	0xff8a,0xff9f,0x2,0xff8b,0xff9f,0x2,0xff8c,0xff9f,
	0x2,0xff8d,0xff9f,0x2,0xff8e,0xff9f,0x2,0x3042,
	0x30fc,0x2,0x3044,0x30fc,0x2,0x3046,0x30fc,0x2,
	0x3048,0x30fc,0x2,0x304a,0x30fc,0x2,0x304b,0x30fc,
	0x2,0x304d,0x30fc,0x2,0x304f,0x30fc,0x2,0x3051,
	0x30fc,0x2,0x3053,0x30fc,0x3,0x304b,0x3099,0x30fc,
	0x3,0x304d,0x3099,0x30fc,0x3,0x304f,0x3099,0x30fc,
	0x3,0x3051,0x3099,0x30fc,0x3,0x3053,0x3099,0x30fc,
	0x2,0x3055,0x30fc,0x2,0x3057,0x30fc,0x2,0x3059,
	0x30fc,0x2,0x305b,0x30fc,0x2,0x305d,0x30fc,0x3,
	0x3055,0x3099,0x30fc,0x3,0x3057,0x3099,0x30fc,0x3,
	0x3059,0x3099,0x30fc,0x3,0x305b,0x3099,0x30fc,0x3,
	0x305d,0x3099,0x30fc,0x2,0x305f,0x30fc,0x2,0x3061,
	0x30fc,0x2,0x3064,0x30fc,0x2,0x3066,0x30fc,0x2,
	0x3068,0x30fc,0x3,0x305f,0x3099,0x30fc,0x3,0x3061,
	0x3099,0x30fc,0x3,0x3064,0x3099,0x30fc,0x3,0x3066,
	0x3099,0x30fc,0x3,0x3068,0x3099,0x30fc,0x2,0x306a,
	0x30fc,0x2,0x306b,0x30fc,0x2,0x306c,0x30fc,0x2,
	0x306d,0x30fc,0x2,0x306e,0x30fc,0x2,0x306f,0x30fc,
	0x2,0x3072,0x30fc,0x2,0x3075,0x30fc,0x2,0x3078,
	0x30fc,0x2,0x307b,0x30fc,0x3,0x306f,0x3099,0x30fc,
	0x3,0x3072,0x3099,0x30fc,0x3,0x3075,0x3099,0x30fc,
	0x3,0x3078,0x3099,0x30fc,0x3,0x307b,0x3099,0x30fc,
	0x3,0x306f,0x309a,0x30fc,0x3,0x3072,0x309a,0x30fc,
	0x3,0x3075,0x309a,0x30fc,0x3,0x3078,0x309a,0x30fc,
	0x3,0x307b,0x309a,0x30fc,0x2,0x307e,0x30fc,0x2,
	0x307f,0x30fc,0x2,0x3080,0x30fc,0x2,0x3081,0x30fc,
	0x2,0x3082,0x30fc,0x2,0x3084,0x30fc,0x2,0x3086,
	0x30fc,0x2,0x3088,0x30fc,0x2,0x3089,0x30fc,0x2,
	0x308a,0x30fc,0x2,0x308b,0x30fc,0x2,0x308c,0x30fc,
	0x2,0x308d,0x30fc,0x2,0x308f,0x30fc,0x2,0x30ab,
	0x30fc,0x2,0x30ad,0x30fc,0x2,0x30af,0x30fc,0x2,
	0x30b1,0x30fc,0x2,0x30b3,0x30fc,0x3,0x30ab,0x3099,
	0x30fc,0x3,0x30ad,0x3099,0x30fc,0x3,0x30af,0x3099,
	0x30fc,0x3,0x30b1,0x3099,0x30fc,0x3,0x30b3,0x3099,
	0x30fc,0x2,0x30b5,0x30fc,0x2,0x30b7,0x30fc,0x2,
	0x30b9,0x30fc,0x2,0x30bb,0x30fc,0x2,0x30bd,0x30fc,
	0x3,0x30b5,0x3099,0x30fc,0x3,0x30b7,0x3099,0x30fc,
	0x3,0x30b9,0x3099,0x30fc,0x3,0x30bb,0x3099,0x30fc,
	0x3,0x30bd,0x3099,0x30fc,0x2,0x30bf,0x30fc,0x2,
	0x30c1,0x30fc,0x2,0x30c4,0x30fc,0x2,0x30c6,0x30fc,
	0x2,0x30c8,0x30fc,0x3,0x30bf,0x3099,0x30fc,0x3,
	0x30c1,0x3099,0x30fc,0x3,0x30c4,0x3099,0x30fc,0x3,
	0x30c6,0x3099,0x30fc,0x3,0x30c8,0x3099,0x30fc,0x2,
	0x30ca,0x30fc,0x2,0x30cb,0x30fc,0x2,0x30cc,0x30fc,
	0x2,0x30cd,0x30fc,0x2,0x30ce,0x30fc,0x2,0x30cf,
	0x30fc,0x2,0x30d2,0x30fc,0x2,0x30d5,0x30fc,0x2,
	0x30d8,0x30fc,0x2,0x30db,0x30fc,0x3,0x30cf,0x3099,
	0x30fc,0x3,0x30d2,0x3099,0x30fc,0x3,0x30d5,0x3099,
	0x30fc,0x3,0x30d8,0x3099,0x30fc,0x3,0x30db,0x3099,
	0x30fc,0x3,0x30cf,0x309a,0x30fc,0x3,0x30d2,0x309a,
	0x30fc,0x3,0x30d5,0x309a,0x30fc,0x3,0x30d8,0x309a,
	0x30fc,0x3,0x30db,0x309a,0x30fc,0x2,0x30de,0x30fc,
	0x2,0x30df,0x30fc,0x2,0x30e0,0x30fc,0x2,0x30e1,
	0x30fc,0x2,0x30e2,0x30fc,0x2,0x30e4,0x30fc,0x2,
	0x30e6,0x30fc,0x2,0x30e8,0x30fc,0x2,0x30e9,0x30fc,
	0x2,0x30ea,0x30fc,0x2,0x30eb,0x30fc,0x2,0x30ec,
	0x30fc,0x2,0x30ed,0x30fc,0x2,0x30ef,0x30fc,0x2,
	0x30f0,0x30fc,0x2,0x30f1,0x30fc,0x2,0x30f2,0x30fc,
	0x3,0x30ef,0x3099,0x30fc,0x3,0x30f0,0x3099,0x30fc,
	0x3,0x30ef,0x3099,0x30fc,0x3,0x30f1,0x3099,0x30fc,
	0x3,0x30f2,0x3099,0x30fc,0x2,0xff76,0xff70,0x2,
	0xff77,0xff70,0x2,0xff78,0xff70,0x2,0xff79,0xff70,
	0x2,0xff7a,0xff70,0x3,0xff76,0xff9e,0xff70,0x3,
	0xff77,0xff9e,0xff70,0x3,0xff78,0xff9e,0xff70,0x3,
	0xff79,0xff9e,0xff70,0x3,0xff7a,0xff9e,0xff70,0x2,
	0xff7b,0xff70,0x2,0xff7c,0xff70,0x2,0xff7d,0xff70,
	0x2,0xff7e,0xff70,0x2,0xff7f,0xff70,0x3,0xff7b,
	0xff9e,0xff70,0x3,0xff7c,0xff9e,0xff70,0x3,0xff7d,
	0xff9e,0xff70,0x3,0xff7e,0xff9e,0xff70,0x3,0xff7f,
	0xff9e,0xff70,0x2,0xff80,0xff70,0x2,0xff81,0xff70,
	0x2,0xff82,0xff70,0x2,0xff83,0xff70,0x2,0xff84,
	0xff70,0x3,0xff80,0xff9e,0xff70,0x3,0xff81,0xff9e,
	0xff70,0x3,0xff82,0xff9e,0xff70,0x3,0xff83,0xff9e,
	0xff70,0x3,0xff84,0xff9e,0xff70,0x2,0xff85,0xff70,
	0x2,0xff86,0xff70,0x2,0xff87,0xff70,0x2,0xff88,
	0xff70,0x2,0xff89,0xff70,0x2,0xff8a,0xff70,0x2,
	0xff8b,0xff70,0x2,0xff8c,0xff70,0x2,0xff8d,0xff70,
	0x2,0xff8e,0xff70,0x3,0xff8a,0xff9e,0xff70,0x3,
	0xff8b,0xff9e,0xff70,0x3,0xff8c,0xff9e,0xff70,0x3,
	0xff8d,0xff9e,0xff70,0x3,0xff8e,0xff9e,0xff70,0x3,
	0xff8a,0xff9f,0xff70,0x3,0xff8b,0xff9f,0xff70,0x3,
	0xff8c,0xff9f,0xff70,0x3,0xff8d,0xff9f,0xff70,0x3,
	0xff8e,0xff9f,0xff70,0x2,0xff8f,0xff70,0x2,0xff90,
	0xff70,0x2,0xff91,0xff70,0x2,0xff92,0xff70,0x2,
	0xff93,0xff70,0x2,0xff94,0xff70,0x2,0xff95,0xff70,
	0x2,0xff96,0xff70,0x2,0xff97,0xff70,0x2,0xff98,
	0xff70,0x2,0xff99,0xff70,0x2,0xff9a,0xff70,0x2,
	0xff9b,0xff70,0x2,0xff9c,0xff70,0x3,0xff9c,0xff9e,
	0xff70,
	};

static const TUint32 TheKanaStringIndex[] = 
	{
	0x4e0034,0x510036,0x540038,0x57003a,0x5a003c,0x6c0048,0x5d003e,0x70004b,
	0x600040,0x74004e,0x630042,0x780051,0x660044,0x7c0054,0x690046,0x8f0061,
	0x800057,0x930064,0x830059,0x970067,0x86005b,0x9b006a,0x89005d,0x9f006d,
	0x8c005f,0xb2007a,0xa30070,0xb6007d,0xa60072,0xba0080,0xa90074,0xbe0083,
	0xac0076,0xc20086,0xaf0078,0xc60089,0xc9008b,0xcc008d,0xcf008f,0xd20091,
	0xe4009d,0xf800ac,0xd50093,0xe800a0,0xfc00af,0xd80095,0xec00a3,0x10000b2,
	0xdb0097,0xf000a6,0x10400b5,0xde0099,0xf400a9,0x10800b8,0xe1009b,0x10c00bb,
	0x10f00bd,0x11200bf,0x11500c1,0x11800c3,0x11b00c5,0x11e00c7,0x12100c9,0x12400cb,
	0x12700cd,0x12a00cf,0x12d00d1,0x13000d3,0x13300d5,0x14500e1,0x13600d7,0x14900e4,
	0x13900d9,0x14d00e7,0x13c00db,0x15100ea,0x13f00dd,0x15500ed,0x14200df,0x16800fa,
	0x15900f0,0x16c00fd,0x15c00f2,0x1700100,0x15f00f4,0x1740103,0x16200f6,0x1780106,
	0x16500f8,0x18b0113,0x17c0109,0x18f0116,0x17f010b,0x1930119,0x182010d,0x197011c,
	0x185010f,0x19b011f,0x1880111,0x19f0122,0x1a20124,0x1a50126,0x1a80128,0x1ab012a,
	0x1bd0136,0x1d10145,0x1ae012c,0x1c10139,0x1d50148,0x1b1012e,0x1c5013c,0x1d9014b,
	0x1b40130,0x1c9013f,0x1dd014e,0x1b70132,0x1cd0142,0x1e10151,0x1ba0134,0x1e50154,
	0x1e80156,0x1eb0158,0x1ee015a,0x1f1015c,0x1f4015e,0x1f70160,0x1fa0162,0x1fd0164,
	0x2000166,0x2030168,0x206016a,0x209016c,0x2180176,0x220017c,0x20c016e,0x21c0179,
	0x20f0170,0x224017f,0x2120172,0x2280182,0x2150174,0x0,0x22c0185,0x30002,
	0x23b018f,0x22f0187,0x60004,0x23f0192,0x2320189,0x90006,0x2430195,0x235018b,
	0xc0008,0x2470198,0x238018d,0xf000a,0x24b019b,0x24f019e,0x12000c,0x25e01a8,
	0x25201a0,0x15000e,0x26201ab,0x25501a2,0x180010,0x26601ae,0x25801a4,0x1b0012,
	0x26a01b1,0x25b01a6,0x1e0014,0x26e01b4,0x27201b7,0x210016,0x28101c1,0x27501b9,
	0x240018,0x28501c4,0x27801bb,0x27001a,0x28901c7,0x27b01bd,0x2a001c,0x28d01ca,
	0x27e01bf,0x2d001e,0x29101cd,0x29501d0,0x29801d2,0x29b01d4,0x29e01d6,0x2a101d8,
	0x2a401da,0x300020,0x2b301e4,0x3f002a,0x2c701f3,0x2a701dc,0x330022,0x2b701e7,
	0x42002c,0x2cb01f6,0x2aa01de,0x360024,0x2bb01ea,0x45002e,0x2cf01f9,0x2ad01e0,
	0x390026,0x2bf01ed,0x480030,0x2d301fc,0x2b001e2,0x3c0028,0x2c301f0,0x4b0032,
	0x2d701ff,0x2db0202,0x2de0204,0x2e10206,0x2e40208,0x2e7020a,0x2ea020c,0x2ed020e,
	0x2f00210,0x2f30212,0x2f60214,0x2f90216,0x2fc0218,0x2ff021a,0x302021c,0x305021e,
	};

static const TCollationKeyTable TheKanaTable = 
	{ TheKanaKey, TheKanaIndex, 560, TheKanaStringElement, TheKanaStringIndex, 232 };


static const TCollationMethod TheKanaMethod =
	{ 0, NULL, &TheKanaTable, 0 };

static const TCollationMethod TheSwapKanaMethod =
	{ 0, NULL, &TheKanaTable, TCollationMethod::ESwapKana };

// Simplified Chinese. The data is extracted from loce32\Is_unic_template_chinese_simplified.cpp
static const TUint32 TheChineseKey[] = 
	{
	0x4e01001d,0x4e02001d,0x4e03001d,0x4e04001d,0x4e05001d,0x4e06001d,0x4e07001d,0x4e08001d,
	0x4e09001d,0x4e0a001d,0x4e0b001d,0x4e0c001d,0x4e0d001d,0x4e0e001d,0x4e0f001d,0x4e10001d,
	0x4e11001d,0x4e12001d,0x4e13001d,0x4e14001d,0x4e15001d,0x4e16001d,0x4e17001d,0x4e18001d,
	0x4e19001d,0x4e1a001d,0x4e1b001d,0x4e1c001d,0x4e1d001d,0x4e1e001d,0x4e1f001d,0x4e20001d,
	0x4e21001d,0x4e22001d,0x4e23001d,0x4e24001d,0x4e25001d,0x4e26001d,0x4e27001d,0x4e28001d,
	0x4e29001d,0x4e2a001d,0x4e2b001d,0x4e2c001d,0x4e2d001d,0x4e2e001d,0x4e2f001d,0x4e30001d,
	0x4e31001d,0x4e32001d,0x4e33001d,0x4e34001d,0x4e35001d,0x4e36001d,0x4e37001d,0x4e38001d,
	0x4e39001d,0x4e3a001d,0x4e3b001d,0x4e3c001d,0x4e3d001d,0x4e3e001d,0x4e3f001d,0x4e40001d,
	0x4e41001d,0x4e42001d,0x4e43001d,0x4e44001d,0x4e45001d,0x4e46001d,0x4e47001d,0x4e48001d,
	0x4e49001d,0x4e4a001d,0x4e4b001d,0x4e4c001d,0x4e4d001d,0x4e4e001d,0x4e4f001d,0x4e50001d,
	0x4e51001d,0x4e52001d,0x4e53001d,0x4e54001d,0x4e55001d,0x4e56001d,0x4e57001d,0x4e58001d,
	0x4e59001d,0x4e5a001d,0x4e5b001d,0x4e5c001d,0x4e5d001d,0x4e5e001d,0x4e5f001d,0x4e60001d,
	0x4e61001d,0x4e62001d,0x4e63001d,0x4e64001d,0x4e65001d,0x4e66001d,0x4e67001d,0x4e68001d,
	0x4e69001d,0x4e6a001d,0x4e6b001d,0x4e6c001d,0x4e6d001d,0x4e6e001d,0x4e6f001d,0x4e70001d,
	0x4e71001d,0x4e72001d,0x4e73001d,0x4e74001d,0x4e75001d,0x4e76001d,0x4e77001d,0x4e78001d,
	0x4e79001d,0x4e7a001d,0x4e7b001d,0x4e7c001d,0x4e7d001d,0x4e7e001d,0x4e7f001d,0x4e80001d,
	0x4e81001d,0x4e82001d,0x4e83001d,0x4e84001d,0x4e85001d,0x4e86001d,0x4e87001d,0x4e88001d,
	0x4e89001d,0x4e8a001d,0x4e8b001d,0x4e8c001d,0x4e8d001d,0x4e8e001d,0x4e8f001d,0x4e90001d,
	0x4e91001d,0x4e92001d,0x4e93001d,0x4e94001d,0x4e95001d,0x4e96001d,0x4e97001d,0x4e98001d,
	0x4e99001d,0x4e9a001d,0x4e9b001d,0x4e9c001d,0x4e9d001d,0x4e9e001d,0x4e9f001d,0x4ea0001d,
	0x4ea1001d,0x4ea2001d,0x4ea3001d,0x4ea4001d,0x4ea5001d,0x4ea6001d,0x4ea7001d,0x4ea8001d,
	0x4ea9001d,0x4eaa001d,0x4eab001d,0x4eac001d,0x4ead001d,0x4eae001d,0x4eaf001d,0x4eb0001d,
	0x4eb1001d,0x4eb2001d,0x4eb3001d,0x4eb4001d,0x4eb5001d,0x4eb6001d,0x4eb7001d,0x4eb8001d,
	0x4eb9001d,0x4eba001d,0x4ebb001d,0x4ebc001d,0x4ebd001d,0x4ebe001d,0x4ebf001d,0x4ec0001d,
	0x4ec1001d,0x4ec2001d,0x4ec3001d,0x4ec4001d,0x4ec5001d,0x4ec6001d,0x4ec7001d,0x4ec8001d,
	0x4ec9001d,0x4eca001d,0x4ecb001d,0x4ecc001d,0x4ecd001d,0x4ece001d,0x4ecf001d,0x4ed0001d,
	0x4ed1001d,0x4ed2001d,0x4ed3001d,0x4ed4001d,0x4ed5001d,0x4ed6001d,0x4ed7001d,0x4ed8001d,
	0x4ed9001d,0x4eda001d,0x4edb001d,0x4edc001d,0x4edd001d,0x4ede001d,0x4edf001d,0x4ee0001d,
	0x4ee1001d,0x4ee2001d,0x4ee3001d,0x4ee4001d,0x4ee5001d,0x4ee6001d,0x4ee7001d,0x4ee8001d,
	0x4ee9001d,0x4eea001d,0x4eeb001d,0x4eec001d,0x4eed001d,0x4eee001d,0x4eef001d,0x4ef0001d,
	0x4ef1001d,0x4ef2001d,0x4ef3001d,0x4ef4001d,0x4ef5001d,0x4ef6001d,0x4ef7001d,0x4ef8001d,
	0x4ef9001d,0x4efa001d,0x4efb001d,0x4efc001d,0x4efd001d,0x4efe001d,0x4eff001d,0x4f00001d,
	0x4f01001d,0x4f02001d,0x4f03001d,0x4f04001d,0x4f05001d,0x4f06001d,0x4f07001d,0x4f08001d,
	0x4f09001d,0x4f0a001d,0x4f0b001d,0x4f0c001d,0x4f0d001d,0x4f0e001d,0x4f0f001d,0x4f10001d,
	0x4f11001d,0x4f12001d,0x4f13001d,0x4f14001d,0x4f15001d,0x4f16001d,0x4f17001d,0x4f18001d,
	0x4f19001d,0x4f1a001d,0x4f1b001d,0x4f1c001d,0x4f1d001d,0x4f1e001d,0x4f1f001d,0x4f20001d,
	0x4f21001d,0x4f22001d,0x4f23001d,0x4f24001d,0x4f25001d,0x4f26001d,0x4f27001d,0x4f28001d,
	0x4f29001d,0x4f2a001d,0x4f2b001d,0x4f2c001d,0x4f2d001d,0x4f2e001d,0x4f2f001d,0x4f30001d,
	0x4f31001d,0x4f32001d,0x4f33001d,0x4f34001d,0x4f35001d,0x4f36001d,0x4f37001d,0x4f38001d,
	0x4f39001d,0x4f3a001d,0x4f3b001d,0x4f3c001d,0x4f3d001d,0x4f3e001d,0x4f3f001d,0x4f40001d,
	0x4f41001d,0x4f42001d,0x4f43001d,0x4f44001d,0x4f45001d,0x4f46001d,0x4f47001d,0x4f48001d,
	0x4f49001d,0x4f4a001d,0x4f4b001d,0x4f4c001d,0x4f4d001d,0x4f4e001d,0x4f4f001d,0x4f50001d,
	0x4f51001d,0x4f52001d,0x4f53001d,0x4f54001d,0x4f55001d,0x4f56001d,0x4f57001d,0x4f58001d,
	0x4f59001d,0x4f5a001d,0x4f5b001d,0x4f5c001d,0x4f5d001d,0x4f5e001d,0x4f5f001d,0x4f60001d,
	0x4f61001d,0x4f62001d,0x4f63001d,0x4f64001d,0x4f65001d,0x4f66001d,0x4f67001d,0x4f68001d,
	0x4f69001d,0x4f6a001d,0x4f6b001d,0x4f6c001d,0x4f6d001d,0x4f6e001d,0x4f6f001d,0x4f70001d,
	0x4f71001d,0x4f72001d,0x4f73001d,0x4f74001d,0x4f75001d,0x4f76001d,0x4f77001d,0x4f78001d,
	0x4f79001d,0x4f7a001d,0x4f7b001d,0x4f7c001d,0x4f7d001d,0x4f7e001d,0x4f7f001d,0x4f80001d,
	0x4f81001d,0x4f82001d,0x4f83001d,0x4f84001d,0x4f85001d,0x4f86001d,0x4f87001d,0x4f88001d,
	0x4f89001d,0x4f8a001d,0x4f8b001d,0x4f8c001d,0x4f8d001d,0x4f8e001d,0x4f8f001d,0x4f90001d,
	0x4f91001d,0x4f92001d,0x4f93001d,0x4f94001d,0x4f95001d,0x4f96001d,0x4f97001d,0x4f98001d,
	0x4f99001d,0x4f9a001d,0x4f9b001d,0x4f9c001d,0x4f9d001d,0x4f9e001d,0x4f9f001d,0x4fa0001d,
	0x4fa1001d,0x4fa2001d,0x4fa3001d,0x4fa4001d,0x4fa5001d,0x4fa6001d,0x4fa7001d,0x4fa8001d,
	0x4fa9001d,0x4faa001d,0x4fab001d,0x4fac001d,0x4fad001d,0x4fae001d,0x4faf001d,0x4fb0001d,
	0x4fb1001d,0x4fb2001d,0x4fb3001d,0x4fb4001d,0x4fb5001d,0x4fb6001d,0x4fb7001d,0x4fb8001d,
	0x4fb9001d,0x4fba001d,0x4fbb001d,0x4fbc001d,0x4fbd001d,0x4fbe001d,0x4fbf001d,0x4fc0001d,
	0x4fc1001d,0x4fc2001d,0x4fc3001d,0x4fc4001d,0x4fc5001d,0x4fc6001d,0x4fc7001d,0x4fc8001d,
	0x4fc9001d,0x4fca001d,0x4fcb001d,0x4fcc001d,0x4fcd001d,0x4fce001d,0x4fcf001d,0x4fd0001d,
	0x4fd1001d,0x4fd2001d,0x4fd3001d,0x4fd4001d,0x4fd5001d,0x4fd6001d,0x4fd7001d,0x4fd8001d,
	0x4fd9001d,0x4fda001d,0x4fdb001d,0x4fdc001d,0x4fdd001d,0x4fde001d,0x4fdf001d,0x4fe0001d,
	0x4fe1001d,0x4fe2001d,0x4fe3001d,0x4fe4001d,0x4fe5001d,0x4fe6001d,0x4fe7001d,0x4fe8001d,
	0x4fe9001d,0x4fea001d,0x4feb001d,0x4fec001d,0x4fed001d,0x4fee001d,0x4fef001d,0x4ff0001d,
	0x4ff1001d,0x4ff2001d,0x4ff3001d,0x4ff4001d,0x4ff5001d,0x4ff6001d,0x4ff7001d,0x4ff8001d,
	0x4ff9001d,0x4ffa001d,0x4ffb001d,0x4ffc001d,0x4ffd001d,0x4ffe001d,0x4fff001d,0x5000001d,
	0x5001001d,0x5002001d,0x5003001d,0x5004001d,0x5005001d,0x5006001d,0x5007001d,0x5008001d,
	0x5009001d,0x500a001d,0x500b001d,0x500c001d,0x500d001d,0x500e001d,0x500f001d,0x5010001d,
	0x5011001d,0x5012001d,0x5013001d,0x5014001d,0x5015001d,0x5016001d,0x5017001d,0x5018001d,
	0x5019001d,0x501a001d,0x501b001d,0x501c001d,0x501d001d,0x501e001d,0x501f001d,0x5020001d,
	0x5021001d,0x5022001d,0x5023001d,0x5024001d,0x5025001d,0x5026001d,0x5027001d,0x5028001d,
	0x5029001d,0x502a001d,0x502b001d,0x502c001d,0x502d001d,0x502e001d,0x502f001d,0x5030001d,
	0x5031001d,0x5032001d,0x5033001d,0x5034001d,0x5035001d,0x5036001d,0x5037001d,0x5038001d,
	0x5039001d,0x503a001d,0x503b001d,0x503c001d,0x503d001d,0x503e001d,0x503f001d,0x5040001d,
	0x5041001d,0x5042001d,0x5043001d,0x5044001d,0x5045001d,0x5046001d,0x5047001d,0x5048001d,
	0x5049001d,0x504a001d,0x504b001d,0x504c001d,0x504d001d,0x504e001d,0x504f001d,0x5050001d,
	0x5051001d,0x5052001d,0x5053001d,0x5054001d,0x5055001d,0x5056001d,0x5057001d,0x5058001d,
	0x5059001d,0x505a001d,0x505b001d,0x505c001d,0x505d001d,0x505e001d,0x505f001d,0x5060001d,
	0x5061001d,0x5062001d,0x5063001d,0x5064001d,0x5065001d,0x5066001d,0x5067001d,0x5068001d,
	0x5069001d,0x506a001d,0x506b001d,0x506c001d,0x506d001d,0x506e001d,0x506f001d,0x5070001d,
	0x5071001d,0x5072001d,0x5073001d,0x5074001d,0x5075001d,0x5076001d,0x5077001d,0x5078001d,
	0x5079001d,0x507a001d,0x507b001d,0x507c001d,0x507d001d,0x507e001d,0x507f001d,0x5080001d,
	0x5081001d,0x5082001d,0x5083001d,0x5084001d,0x5085001d,0x5086001d,0x5087001d,0x5088001d,
	0x5089001d,0x508a001d,0x508b001d,0x508c001d,0x508d001d,0x508e001d,0x508f001d,0x5090001d,
	0x5091001d,0x5092001d,0x5093001d,0x5094001d,0x5095001d,0x5096001d,0x5097001d,0x5098001d,
	0x5099001d,0x509a001d,0x509b001d,0x509c001d,0x509d001d,0x509e001d,0x509f001d,0x50a0001d,
	0x50a1001d,0x50a2001d,0x50a3001d,0x50a4001d,0x50a5001d,0x50a6001d,0x50a7001d,0x50a8001d,
	0x50a9001d,0x50aa001d,0x50ab001d,0x50ac001d,0x50ad001d,0x50ae001d,0x50af001d,0x50b0001d,
	0x50b1001d,0x50b2001d,0x50b3001d,0x50b4001d,0x50b5001d,0x50b6001d,0x50b7001d,0x50b8001d,
	0x50b9001d,0x50ba001d,0x50bb001d,0x50bc001d,0x50bd001d,0x50be001d,0x50bf001d,0x50c0001d,
	0x50c1001d,0x50c2001d,0x50c3001d,0x50c4001d,0x50c5001d,0x50c6001d,0x50c7001d,0x50c8001d,
	0x50c9001d,0x50ca001d,0x50cb001d,0x50cc001d,0x50cd001d,0x50ce001d,0x50cf001d,0x50d0001d,
	0x50d1001d,0x50d2001d,0x50d3001d,0x50d4001d,0x50d5001d,0x50d6001d,0x50d7001d,0x50d8001d,
	0x50d9001d,0x50da001d,0x50db001d,0x50dc001d,0x50dd001d,0x50de001d,0x50df001d,0x50e0001d,
	0x50e1001d,0x50e2001d,0x50e3001d,0x50e4001d,0x50e5001d,0x50e6001d,0x50e7001d,0x50e8001d,
	0x50e9001d,0x50ea001d,0x50eb001d,0x50ec001d,0x50ed001d,0x50ee001d,0x50ef001d,0x50f0001d,
	0x50f1001d,0x50f2001d,0x50f3001d,0x50f4001d,0x50f5001d,0x50f6001d,0x50f7001d,0x50f8001d,
	0x50f9001d,0x50fa001d,0x50fb001d,0x50fc001d,0x50fd001d,0x50fe001d,0x50ff001d,0x5100001d,
	0x5101001d,0x5102001d,0x5103001d,0x5104001d,0x5105001d,0x5106001d,0x5107001d,0x5108001d,
	0x5109001d,0x510a001d,0x510b001d,0x510c001d,0x510d001d,0x510e001d,0x510f001d,0x5110001d,
	0x5111001d,0x5112001d,0x5113001d,0x5114001d,0x5115001d,0x5116001d,0x5117001d,0x5118001d,
	0x5119001d,0x511a001d,0x511b001d,0x511c001d,0x511d001d,0x511e001d,0x511f001d,0x5120001d,
	0x5121001d,0x5122001d,0x5123001d,0x5124001d,0x5125001d,0x5126001d,0x5127001d,0x5128001d,
	0x5129001d,0x512a001d,0x512b001d,0x512c001d,0x512d001d,0x512e001d,0x512f001d,0x5130001d,
	0x5131001d,0x5132001d,0x5133001d,0x5134001d,0x5135001d,0x5136001d,0x5137001d,0x5138001d,
	0x5139001d,0x513a001d,0x513b001d,0x513c001d,0x513d001d,0x513e001d,0x513f001d,0x5140001d,
	0x5141001d,0x5142001d,0x5143001d,0x5144001d,0x5145001d,0x5146001d,0x5147001d,0x5148001d,
	0x5149001d,0x514a001d,0x514b001d,0x514c001d,0x514d001d,0x514e001d,0x514f001d,0x5150001d,
	0x5151001d,0x5152001d,0x5153001d,0x5154001d,0x5155001d,0x5156001d,0x5157001d,0x5158001d,
	0x5159001d,0x515a001d,0x515b001d,0x515c001d,0x515d001d,0x515e001d,0x515f001d,0x5160001d,
	0x5161001d,0x5162001d,0x5163001d,0x5164001d,0x5165001d,0x5166001d,0x5167001d,0x5168001d,
	0x5169001d,0x516a001d,0x516b001d,0x516c001d,0x516d001d,0x516e001d,0x516f001d,0x5170001d,
	0x5171001d,0x5172001d,0x5173001d,0x5174001d,0x5175001d,0x5176001d,0x5177001d,0x5178001d,
	0x5179001d,0x517a001d,0x517b001d,0x517c001d,0x517d001d,0x517e001d,0x517f001d,0x5180001d,
	0x5181001d,0x5182001d,0x5183001d,0x5184001d,0x5185001d,0x5186001d,0x5187001d,0x5188001d,
	0x5189001d,0x518a001d,0x518b001d,0x518c001d,0x518d001d,0x518e001d,0x518f001d,0x5190001d,
	0x5191001d,0x5192001d,0x5193001d,0x5194001d,0x5195001d,0x5196001d,0x5197001d,0x5198001d,
	0x5199001d,0x519a001d,0x519b001d,0x519c001d,0x519d001d,0x519e001d,0x519f001d,0x51a0001d,
	0x51a1001d,0x51a2001d,0x51a3001d,0x51a4001d,0x51a5001d,0x51a6001d,0x51a7001d,0x51a8001d,
	0x51a9001d,0x51aa001d,0x51ab001d,0x51ac001d,0x51ad001d,0x51ae001d,0x51af001d,0x51b0001d,
	0x51b1001d,0x51b2001d,0x51b3001d,0x51b4001d,0x51b5001d,0x51b6001d,0x51b7001d,0x51b8001d,
	0x51b9001d,0x51ba001d,0x51bb001d,0x51bc001d,0x51bd001d,0x51be001d,0x51bf001d,0x51c0001d,
	0x51c1001d,0x51c2001d,0x51c3001d,0x51c4001d,0x51c5001d,0x51c6001d,0x51c7001d,0x51c8001d,
	0x51c9001d,0x51ca001d,0x51cb001d,0x51cc001d,0x51cd001d,0x51ce001d,0x51cf001d,0x51d0001d,
	0x51d1001d,0x51d2001d,0x51d3001d,0x51d4001d,0x51d5001d,0x51d6001d,0x51d7001d,0x51d8001d,
	0x51d9001d,0x51da001d,0x51db001d,0x51dc001d,0x51dd001d,0x51de001d,0x51df001d,0x51e0001d,
	0x51e1001d,0x51e2001d,0x51e3001d,0x51e4001d,0x51e5001d,0x51e6001d,0x51e7001d,0x51e8001d,
	0x51e9001d,0x51ea001d,0x51eb001d,0x51ec001d,0x51ed001d,0x51ee001d,0x51ef001d,0x51f0001d,
	0x51f1001d,0x51f2001d,0x51f3001d,0x51f4001d,0x51f5001d,0x51f6001d,0x51f7001d,0x51f8001d,
	0x51f9001d,0x51fa001d,0x51fb001d,0x51fc001d,0x51fd001d,0x51fe001d,0x51ff001d,0x5200001d,
	0x5201001d,0x5202001d,0x5203001d,0x5204001d,0x5205001d,0x5206001d,0x5207001d,0x5208001d,
	0x5209001d,0x520a001d,0x520b001d,0x520c001d,0x520d001d,0x520e001d,0x520f001d,0x5210001d,
	0x5211001d,0x5212001d,0x5213001d,0x5214001d,0x5215001d,0x5216001d,0x5217001d,0x5218001d,
	0x5219001d,0x521a001d,0x521b001d,0x521c001d,0x521d001d,0x521e001d,0x521f001d,0x5220001d,
	0x5221001d,0x5222001d,0x5223001d,0x5224001d,0x5225001d,0x5226001d,0x5227001d,0x5228001d,
	0x5229001d,0x522a001d,0x522b001d,0x522c001d,0x522d001d,0x522e001d,0x522f001d,0x5230001d,
	0x5231001d,0x5232001d,0x5233001d,0x5234001d,0x5235001d,0x5236001d,0x5237001d,0x5238001d,
	0x5239001d,0x523a001d,0x523b001d,0x523c001d,0x523d001d,0x523e001d,0x523f001d,0x5240001d,
	0x5241001d,0x5242001d,0x5243001d,0x5244001d,0x5245001d,0x5246001d,0x5247001d,0x5248001d,
	0x5249001d,0x524a001d,0x524b001d,0x524c001d,0x524d001d,0x524e001d,0x524f001d,0x5250001d,
	0x5251001d,0x5252001d,0x5253001d,0x5254001d,0x5255001d,0x5256001d,0x5257001d,0x5258001d,
	0x5259001d,0x525a001d,0x525b001d,0x525c001d,0x525d001d,0x525e001d,0x525f001d,0x5260001d,
	0x5261001d,0x5262001d,0x5263001d,0x5264001d,0x5265001d,0x5266001d,0x5267001d,0x5268001d,
	0x5269001d,0x526a001d,0x526b001d,0x526c001d,0x526d001d,0x526e001d,0x526f001d,0x5270001d,
	0x5271001d,0x5272001d,0x5273001d,0x5274001d,0x5275001d,0x5276001d,0x5277001d,0x5278001d,
	0x5279001d,0x527a001d,0x527b001d,0x527c001d,0x527d001d,0x527e001d,0x527f001d,0x5280001d,
	0x5281001d,0x5282001d,0x5283001d,0x5284001d,0x5285001d,0x5286001d,0x5287001d,0x5288001d,
	0x5289001d,0x528a001d,0x528b001d,0x528c001d,0x528d001d,0x528e001d,0x528f001d,0x5290001d,
	0x5291001d,0x5292001d,0x5293001d,0x5294001d,0x5295001d,0x5296001d,0x5297001d,0x5298001d,
	0x5299001d,0x529a001d,0x529b001d,0x529c001d,0x529d001d,0x529e001d,0x529f001d,0x52a0001d,
	0x52a1001d,0x52a2001d,0x52a3001d,0x52a4001d,0x52a5001d,0x52a6001d,0x52a7001d,0x52a8001d,
	0x52a9001d,0x52aa001d,0x52ab001d,0x52ac001d,0x52ad001d,0x52ae001d,0x52af001d,0x52b0001d,
	0x52b1001d,0x52b2001d,0x52b3001d,0x52b4001d,0x52b5001d,0x52b6001d,0x52b7001d,0x52b8001d,
	0x52b9001d,0x52ba001d,0x52bb001d,0x52bc001d,0x52bd001d,0x52be001d,0x52bf001d,0x52c0001d,
	0x52c1001d,0x52c2001d,0x52c3001d,0x52c4001d,0x52c5001d,0x52c6001d,0x52c7001d,0x52c8001d,
	0x52c9001d,0x52ca001d,0x52cb001d,0x52cc001d,0x52cd001d,0x52ce001d,0x52cf001d,0x52d0001d,
	0x52d1001d,0x52d2001d,0x52d3001d,0x52d4001d,0x52d5001d,0x52d6001d,0x52d7001d,0x52d8001d,
	0x52d9001d,0x52da001d,0x52db001d,0x52dc001d,0x52dd001d,0x52de001d,0x52df001d,0x52e0001d,
	0x52e1001d,0x52e2001d,0x52e3001d,0x52e4001d,0x52e5001d,0x52e6001d,0x52e7001d,0x52e8001d,
	0x52e9001d,0x52ea001d,0x52eb001d,0x52ec001d,0x52ed001d,0x52ee001d,0x52ef001d,0x52f0001d,
	0x52f1001d,0x52f2001d,0x52f3001d,0x52f4001d,0x52f5001d,0x52f6001d,0x52f7001d,0x52f8001d,
	0x52f9001d,0x52fa001d,0x52fb001d,0x52fc001d,0x52fd001d,0x52fe001d,0x52ff001d,0x5300001d,
	0x5301001d,0x5302001d,0x5303001d,0x5304001d,0x5305001d,0x5306001d,0x5307001d,0x5308001d,
	0x5309001d,0x530a001d,0x530b001d,0x530c001d,0x530d001d,0x530e001d,0x530f001d,0x5310001d,
	0x5311001d,0x5312001d,0x5313001d,0x5314001d,0x5315001d,0x5316001d,0x5317001d,0x5318001d,
	0x5319001d,0x531a001d,0x531b001d,0x531c001d,0x531d001d,0x531e001d,0x531f001d,0x5320001d,
	0x5321001d,0x5322001d,0x5323001d,0x5324001d,0x5325001d,0x5326001d,0x5327001d,0x5328001d,
	0x5329001d,0x532a001d,0x532b001d,0x532c001d,0x532d001d,0x532e001d,0x532f001d,0x5330001d,
	0x5331001d,0x5332001d,0x5333001d,0x5334001d,0x5335001d,0x5336001d,0x5337001d,0x5338001d,
	0x5339001d,0x533a001d,0x533b001d,0x533c001d,0x533d001d,0x533e001d,0x533f001d,0x5340001d,
	0x5341001d,0x5342001d,0x5343001d,0x5344001d,0x5345001d,0x5346001d,0x5347001d,0x5348001d,
	0x5349001d,0x534a001d,0x534b001d,0x534c001d,0x534d001d,0x534e001d,0x534f001d,0x5350001d,
	0x5351001d,0x5352001d,0x5353001d,0x5354001d,0x5355001d,0x5356001d,0x5357001d,0x5358001d,
	0x5359001d,0x535a001d,0x535b001d,0x535c001d,0x535d001d,0x535e001d,0x535f001d,0x5360001d,
	0x5361001d,0x5362001d,0x5363001d,0x5364001d,0x5365001d,0x5366001d,0x5367001d,0x5368001d,
	0x5369001d,0x536a001d,0x536b001d,0x536c001d,0x536d001d,0x536e001d,0x536f001d,0x5370001d,
	0x5371001d,0x5372001d,0x5373001d,0x5374001d,0x5375001d,0x5376001d,0x5377001d,0x5378001d,
	0x5379001d,0x537a001d,0x537b001d,0x537c001d,0x537d001d,0x537e001d,0x537f001d,0x5380001d,
	0x5381001d,0x5382001d,0x5383001d,0x5384001d,0x5385001d,0x5386001d,0x5387001d,0x5388001d,
	0x5389001d,0x538a001d,0x538b001d,0x538c001d,0x538d001d,0x538e001d,0x538f001d,0x5390001d,
	0x5391001d,0x5392001d,0x5393001d,0x5394001d,0x5395001d,0x5396001d,0x5397001d,0x5398001d,
	0x5399001d,0x539a001d,0x539b001d,0x539c001d,0x539d001d,0x539e001d,0x539f001d,0x53a0001d,
	0x53a1001d,0x53a2001d,0x53a3001d,0x53a4001d,0x53a5001d,0x53a6001d,0x53a7001d,0x53a8001d,
	0x53a9001d,0x53aa001d,0x53ab001d,0x53ac001d,0x53ad001d,0x53ae001d,0x53af001d,0x53b0001d,
	0x53b1001d,0x53b2001d,0x53b3001d,0x53b4001d,0x53b5001d,0x53b6001d,0x53b7001d,0x53b8001d,
	0x53b9001d,0x53ba001d,0x53bb001d,0x53bc001d,0x53bd001d,0x53be001d,0x53bf001d,0x53c0001d,
	0x53c1001d,0x53c2001d,0x53c3001d,0x53c4001d,0x53c5001d,0x53c6001d,0x53c7001d,0x53c8001d,
	0x53c9001d,0x53ca001d,0x53cb001d,0x53cc001d,0x53cd001d,0x53ce001d,0x53cf001d,0x53d0001d,
	0x53d1001d,0x53d2001d,0x53d3001d,0x53d4001d,0x53d5001d,0x53d6001d,0x53d7001d,0x53d8001d,
	0x53d9001d,0x53da001d,0x53db001d,0x53dc001d,0x53dd001d,0x53de001d,0x53df001d,0x53e0001d,
	0x53e1001d,0x53e2001d,0x53e3001d,0x53e4001d,0x53e5001d,0x53e6001d,0x53e7001d,0x53e8001d,
	0x53e9001d,0x53ea001d,0x53eb001d,0x53ec001d,0x53ed001d,0x53ee001d,0x53ef001d,0x53f0001d,
	0x53f1001d,0x53f2001d,0x53f3001d,0x53f4001d,0x53f5001d,0x53f6001d,0x53f7001d,0x53f8001d,
	0x53f9001d,0x53fa001d,0x53fb001d,0x53fc001d,0x53fd001d,0x53fe001d,0x53ff001d,0x5400001d,
	0x5401001d,0x5402001d,0x5403001d,0x5404001d,0x5405001d,0x5406001d,0x5407001d,0x5408001d,
	0x5409001d,0x540a001d,0x540b001d,0x540c001d,0x540d001d,0x540e001d,0x540f001d,0x5410001d,
	0x5411001d,0x5412001d,0x5413001d,0x5414001d,0x5415001d,0x5416001d,0x5417001d,0x5418001d,
	0x5419001d,0x541a001d,0x541b001d,0x541c001d,0x541d001d,0x541e001d,0x541f001d,0x5420001d,
	0x5421001d,0x5422001d,0x5423001d,0x5424001d,0x5425001d,0x5426001d,0x5427001d,0x5428001d,
	0x5429001d,0x542a001d,0x542b001d,0x542c001d,0x542d001d,0x542e001d,0x542f001d,0x5430001d,
	0x5431001d,0x5432001d,0x5433001d,0x5434001d,0x5435001d,0x5436001d,0x5437001d,0x5438001d,
	0x5439001d,0x543a001d,0x543b001d,0x543c001d,0x543d001d,0x543e001d,0x543f001d,0x5440001d,
	0x5441001d,0x5442001d,0x5443001d,0x5444001d,0x5445001d,0x5446001d,0x5447001d,0x5448001d,
	0x5449001d,0x544a001d,0x544b001d,0x544c001d,0x544d001d,0x544e001d,0x544f001d,0x5450001d,
	0x5451001d,0x5452001d,0x5453001d,0x5454001d,0x5455001d,0x5456001d,0x5457001d,0x5458001d,
	0x5459001d,0x545a001d,0x545b001d,0x545c001d,0x545d001d,0x545e001d,0x545f001d,0x5460001d,
	0x5461001d,0x5462001d,0x5463001d,0x5464001d,0x5465001d,0x5466001d,0x5467001d,0x5468001d,
	0x5469001d,0x546a001d,0x546b001d,0x546c001d,0x546d001d,0x546e001d,0x546f001d,0x5470001d,
	0x5471001d,0x5472001d,0x5473001d,0x5474001d,0x5475001d,0x5476001d,0x5477001d,0x5478001d,
	0x5479001d,0x547a001d,0x547b001d,0x547c001d,0x547d001d,0x547e001d,0x547f001d,0x5480001d,
	0x5481001d,0x5482001d,0x5483001d,0x5484001d,0x5485001d,0x5486001d,0x5487001d,0x5488001d,
	0x5489001d,0x548a001d,0x548b001d,0x548c001d,0x548d001d,0x548e001d,0x548f001d,0x5490001d,
	0x5491001d,0x5492001d,0x5493001d,0x5494001d,0x5495001d,0x5496001d,0x5497001d,0x5498001d,
	0x5499001d,0x549a001d,0x549b001d,0x549c001d,0x549d001d,0x549e001d,0x549f001d,0x54a0001d,
	0x54a1001d,0x54a2001d,0x54a3001d,0x54a4001d,0x54a5001d,0x54a6001d,0x54a7001d,0x54a8001d,
	0x54a9001d,0x54aa001d,0x54ab001d,0x54ac001d,0x54ad001d,0x54ae001d,0x54af001d,0x54b0001d,
	0x54b1001d,0x54b2001d,0x54b3001d,0x54b4001d,0x54b5001d,0x54b6001d,0x54b7001d,0x54b8001d,
	0x54b9001d,0x54ba001d,0x54bb001d,0x54bc001d,0x54bd001d,0x54be001d,0x54bf001d,0x54c0001d,
	0x54c1001d,0x54c2001d,0x54c3001d,0x54c4001d,0x54c5001d,0x54c6001d,0x54c7001d,0x54c8001d,
	0x54c9001d,0x54ca001d,0x54cb001d,0x54cc001d,0x54cd001d,0x54ce001d,0x54cf001d,0x54d0001d,
	0x54d1001d,0x54d2001d,0x54d3001d,0x54d4001d,0x54d5001d,0x54d6001d,0x54d7001d,0x54d8001d,
	0x54d9001d,0x54da001d,0x54db001d,0x54dc001d,0x54dd001d,0x54de001d,0x54df001d,0x54e0001d,
	0x54e1001d,0x54e2001d,0x54e3001d,0x54e4001d,0x54e5001d,0x54e6001d,0x54e7001d,0x54e8001d,
	0x54e9001d,0x54ea001d,0x54eb001d,0x54ec001d,0x54ed001d,0x54ee001d,0x54ef001d,0x54f0001d,
	0x54f1001d,0x54f2001d,0x54f3001d,0x54f4001d,0x54f5001d,0x54f6001d,0x54f7001d,0x54f8001d,
	0x54f9001d,0x54fa001d,0x54fb001d,0x54fc001d,0x54fd001d,0x54fe001d,0x54ff001d,0x5500001d,
	0x5501001d,0x5502001d,0x5503001d,0x5504001d,0x5505001d,0x5506001d,0x5507001d,0x5508001d,
	0x5509001d,0x550a001d,0x550b001d,0x550c001d,0x550d001d,0x550e001d,0x550f001d,0x5510001d,
	0x5511001d,0x5512001d,0x5513001d,0x5514001d,0x5515001d,0x5516001d,0x5517001d,0x5518001d,
	0x5519001d,0x551a001d,0x551b001d,0x551c001d,0x551d001d,0x551e001d,0x551f001d,0x5520001d,
	0x5521001d,0x5522001d,0x5523001d,0x5524001d,0x5525001d,0x5526001d,0x5527001d,0x5528001d,
	0x5529001d,0x552a001d,0x552b001d,0x552c001d,0x552d001d,0x552e001d,0x552f001d,0x5530001d,
	0x5531001d,0x5532001d,0x5533001d,0x5534001d,0x5535001d,0x5536001d,0x5537001d,0x5538001d,
	0x5539001d,0x553a001d,0x553b001d,0x553c001d,0x553d001d,0x553e001d,0x553f001d,0x5540001d,
	0x5541001d,0x5542001d,0x5543001d,0x5544001d,0x5545001d,0x5546001d,0x5547001d,0x5548001d,
	0x5549001d,0x554a001d,0x554b001d,0x554c001d,0x554d001d,0x554e001d,0x554f001d,0x5550001d,
	0x5551001d,0x5552001d,0x5553001d,0x5554001d,0x5555001d,0x5556001d,0x5557001d,0x5558001d,
	0x5559001d,0x555a001d,0x555b001d,0x555c001d,0x555d001d,0x555e001d,0x555f001d,0x5560001d,
	0x5561001d,0x5562001d,0x5563001d,0x5564001d,0x5565001d,0x5566001d,0x5567001d,0x5568001d,
	0x5569001d,0x556a001d,0x556b001d,0x556c001d,0x556d001d,0x556e001d,0x556f001d,0x5570001d,
	0x5571001d,0x5572001d,0x5573001d,0x5574001d,0x5575001d,0x5576001d,0x5577001d,0x5578001d,
	0x5579001d,0x557a001d,0x557b001d,0x557c001d,0x557d001d,0x557e001d,0x557f001d,0x5580001d,
	0x5581001d,0x5582001d,0x5583001d,0x5584001d,0x5585001d,0x5586001d,0x5587001d,0x5588001d,
	0x5589001d,0x558a001d,0x558b001d,0x558c001d,0x558d001d,0x558e001d,0x558f001d,0x5590001d,
	0x5591001d,0x5592001d,0x5593001d,0x5594001d,0x5595001d,0x5596001d,0x5597001d,0x5598001d,
	0x5599001d,0x559a001d,0x559b001d,0x559c001d,0x559d001d,0x559e001d,0x559f001d,0x55a0001d,
	0x55a1001d,0x55a2001d,0x55a3001d,0x55a4001d,0x55a5001d,0x55a6001d,0x55a7001d,0x55a8001d,
	0x55a9001d,0x55aa001d,0x55ab001d,0x55ac001d,0x55ad001d,0x55ae001d,0x55af001d,0x55b0001d,
	0x55b1001d,0x55b2001d,0x55b3001d,0x55b4001d,0x55b5001d,0x55b6001d,0x55b7001d,0x55b8001d,
	0x55b9001d,0x55ba001d,0x55bb001d,0x55bc001d,0x55bd001d,0x55be001d,0x55bf001d,0x55c0001d,
	0x55c1001d,0x55c2001d,0x55c3001d,0x55c4001d,0x55c5001d,0x55c6001d,0x55c7001d,0x55c8001d,
	0x55c9001d,0x55ca001d,0x55cb001d,0x55cc001d,0x55cd001d,0x55ce001d,0x55cf001d,0x55d0001d,
	0x55d1001d,0x55d2001d,0x55d3001d,0x55d4001d,0x55d5001d,0x55d6001d,0x55d7001d,0x55d8001d,
	0x55d9001d,0x55da001d,0x55db001d,0x55dc001d,0x55dd001d,0x55de001d,0x55df001d,0x55e0001d,
	0x55e1001d,0x55e2001d,0x55e3001d,0x55e4001d,0x55e5001d,0x55e6001d,0x55e7001d,0x55e8001d,
	0x55e9001d,0x55ea001d,0x55eb001d,0x55ec001d,0x55ed001d,0x55ee001d,0x55ef001d,0x55f0001d,
	0x55f1001d,0x55f2001d,0x55f3001d,0x55f4001d,0x55f5001d,0x55f6001d,0x55f7001d,0x55f8001d,
	0x55f9001d,0x55fa001d,0x55fb001d,0x55fc001d,0x55fd001d,0x55fe001d,0x55ff001d,0x5600001d,
	0x5601001d,0x5602001d,0x5603001d,0x5604001d,0x5605001d,0x5606001d,0x5607001d,0x5608001d,
	0x5609001d,0x560a001d,0x560b001d,0x560c001d,0x560d001d,0x560e001d,0x560f001d,0x5610001d,
	0x5611001d,0x5612001d,0x5613001d,0x5614001d,0x5615001d,0x5616001d,0x5617001d,0x5618001d,
	0x5619001d,0x561a001d,0x561b001d,0x561c001d,0x561d001d,0x561e001d,0x561f001d,0x5620001d,
	0x5621001d,0x5622001d,0x5623001d,0x5624001d,0x5625001d,0x5626001d,0x5627001d,0x5628001d,
	0x5629001d,0x562a001d,0x562b001d,0x562c001d,0x562d001d,0x562e001d,0x562f001d,0x5630001d,
	0x5631001d,0x5632001d,0x5633001d,0x5634001d,0x5635001d,0x5636001d,0x5637001d,0x5638001d,
	0x5639001d,0x563a001d,0x563b001d,0x563c001d,0x563d001d,0x563e001d,0x563f001d,0x5640001d,
	0x5641001d,0x5642001d,0x5643001d,0x5644001d,0x5645001d,0x5646001d,0x5647001d,0x5648001d,
	0x5649001d,0x564a001d,0x564b001d,0x564c001d,0x564d001d,0x564e001d,0x564f001d,0x5650001d,
	0x5651001d,0x5652001d,0x5653001d,0x5654001d,0x5655001d,0x5656001d,0x5657001d,0x5658001d,
	0x5659001d,0x565a001d,0x565b001d,0x565c001d,0x565d001d,0x565e001d,0x565f001d,0x5660001d,
	0x5661001d,0x5662001d,0x5663001d,0x5664001d,0x5665001d,0x5666001d,0x5667001d,0x5668001d,
	0x5669001d,0x566a001d,0x566b001d,0x566c001d,0x566d001d,0x566e001d,0x566f001d,0x5670001d,
	0x5671001d,0x5672001d,0x5673001d,0x5674001d,0x5675001d,0x5676001d,0x5677001d,0x5678001d,
	0x5679001d,0x567a001d,0x567b001d,0x567c001d,0x567d001d,0x567e001d,0x567f001d,0x5680001d,
	0x5681001d,0x5682001d,0x5683001d,0x5684001d,0x5685001d,0x5686001d,0x5687001d,0x5688001d,
	0x5689001d,0x568a001d,0x568b001d,0x568c001d,0x568d001d,0x568e001d,0x568f001d,0x5690001d,
	0x5691001d,0x5692001d,0x5693001d,0x5694001d,0x5695001d,0x5696001d,0x5697001d,0x5698001d,
	0x5699001d,0x569a001d,0x569b001d,0x569c001d,0x569d001d,0x569e001d,0x569f001d,0x56a0001d,
	0x56a1001d,0x56a2001d,0x56a3001d,0x56a4001d,0x56a5001d,0x56a6001d,0x56a7001d,0x56a8001d,
	0x56a9001d,0x56aa001d,0x56ab001d,0x56ac001d,0x56ad001d,0x56ae001d,0x56af001d,0x56b0001d,
	0x56b1001d,0x56b2001d,0x56b3001d,0x56b4001d,0x56b5001d,0x56b6001d,0x56b7001d,0x56b8001d,
	0x56b9001d,0x56ba001d,0x56bb001d,0x56bc001d,0x56bd001d,0x56be001d,0x56bf001d,0x56c0001d,
	0x56c1001d,0x56c2001d,0x56c3001d,0x56c4001d,0x56c5001d,0x56c6001d,0x56c7001d,0x56c8001d,
	0x56c9001d,0x56ca001d,0x56cb001d,0x56cc001d,0x56cd001d,0x56ce001d,0x56cf001d,0x56d0001d,
	0x56d1001d,0x56d2001d,0x56d3001d,0x56d4001d,0x56d5001d,0x56d6001d,0x56d7001d,0x56d8001d,
	0x56d9001d,0x56da001d,0x56db001d,0x56dc001d,0x56dd001d,0x56de001d,0x56df001d,0x56e0001d,
	0x56e1001d,0x56e2001d,0x56e3001d,0x56e4001d,0x56e5001d,0x56e6001d,0x56e7001d,0x56e8001d,
	0x56e9001d,0x56ea001d,0x56eb001d,0x56ec001d,0x56ed001d,0x56ee001d,0x56ef001d,0x56f0001d,
	0x56f1001d,0x56f2001d,0x56f3001d,0x56f4001d,0x56f5001d,0x56f6001d,0x56f7001d,0x56f8001d,
	0x56f9001d,0x56fa001d,0x56fb001d,0x56fc001d,0x56fd001d,0x56fe001d,0x56ff001d,0x5700001d,
	0x5701001d,0x5702001d,0x5703001d,0x5704001d,0x5705001d,0x5706001d,0x5707001d,0x5708001d,
	0x5709001d,0x570a001d,0x570b001d,0x570c001d,0x570d001d,0x570e001d,0x570f001d,0x5710001d,
	0x5711001d,0x5712001d,0x5713001d,0x5714001d,0x5715001d,0x5716001d,0x5717001d,0x5718001d,
	0x5719001d,0x571a001d,0x571b001d,0x571c001d,0x571d001d,0x571e001d,0x571f001d,0x5720001d,
	0x5721001d,0x5722001d,0x5723001d,0x5724001d,0x5725001d,0x5726001d,0x5727001d,0x5728001d,
	0x5729001d,0x572a001d,0x572b001d,0x572c001d,0x572d001d,0x572e001d,0x572f001d,0x5730001d,
	0x5731001d,0x5732001d,0x5733001d,0x5734001d,0x5735001d,0x5736001d,0x5737001d,0x5738001d,
	0x5739001d,0x573a001d,0x573b001d,0x573c001d,0x573d001d,0x573e001d,0x573f001d,0x5740001d,
	0x5741001d,0x5742001d,0x5743001d,0x5744001d,0x5745001d,0x5746001d,0x5747001d,0x5748001d,
	0x5749001d,0x574a001d,0x574b001d,0x574c001d,0x574d001d,0x574e001d,0x574f001d,0x5750001d,
	0x5751001d,0x5752001d,0x5753001d,0x5754001d,0x5755001d,0x5756001d,0x5757001d,0x5758001d,
	0x5759001d,0x575a001d,0x575b001d,0x575c001d,0x575d001d,0x575e001d,0x575f001d,0x5760001d,
	0x5761001d,0x5762001d,0x5763001d,0x5764001d,0x5765001d,0x5766001d,0x5767001d,0x5768001d,
	0x5769001d,0x576a001d,0x576b001d,0x576c001d,0x576d001d,0x576e001d,0x576f001d,0x5770001d,
	0x5771001d,0x5772001d,0x5773001d,0x5774001d,0x5775001d,0x5776001d,0x5777001d,0x5778001d,
	0x5779001d,0x577a001d,0x577b001d,0x577c001d,0x577d001d,0x577e001d,0x577f001d,0x5780001d,
	0x5781001d,0x5782001d,0x5783001d,0x5784001d,0x5785001d,0x5786001d,0x5787001d,0x5788001d,
	0x5789001d,0x578a001d,0x578b001d,0x578c001d,0x578d001d,0x578e001d,0x578f001d,0x5790001d,
	0x5791001d,0x5792001d,0x5793001d,0x5794001d,0x5795001d,0x5796001d,0x5797001d,0x5798001d,
	0x5799001d,0x579a001d,0x579b001d,0x579c001d,0x579d001d,0x579e001d,0x579f001d,0x57a0001d,
	0x57a1001d,0x57a2001d,0x57a3001d,0x57a4001d,0x57a5001d,0x57a6001d,0x57a7001d,0x57a8001d,
	0x57a9001d,0x57aa001d,0x57ab001d,0x57ac001d,0x57ad001d,0x57ae001d,0x57af001d,0x57b0001d,
	0x57b1001d,0x57b2001d,0x57b3001d,0x57b4001d,0x57b5001d,0x57b6001d,0x57b7001d,0x57b8001d,
	0x57b9001d,0x57ba001d,0x57bb001d,0x57bc001d,0x57bd001d,0x57be001d,0x57bf001d,0x57c0001d,
	0x57c1001d,0x57c2001d,0x57c3001d,0x57c4001d,0x57c5001d,0x57c6001d,0x57c7001d,0x57c8001d,
	0x57c9001d,0x57ca001d,0x57cb001d,0x57cc001d,0x57cd001d,0x57ce001d,0x57cf001d,0x57d0001d,
	0x57d1001d,0x57d2001d,0x57d3001d,0x57d4001d,0x57d5001d,0x57d6001d,0x57d7001d,0x57d8001d,
	0x57d9001d,0x57da001d,0x57db001d,0x57dc001d,0x57dd001d,0x57de001d,0x57df001d,0x57e0001d,
	0x57e1001d,0x57e2001d,0x57e3001d,0x57e4001d,0x57e5001d,0x57e6001d,0x57e7001d,0x57e8001d,
	0x57e9001d,0x57ea001d,0x57eb001d,0x57ec001d,0x57ed001d,0x57ee001d,0x57ef001d,0x57f0001d,
	0x57f1001d,0x57f2001d,0x57f3001d,0x57f4001d,0x57f5001d,0x57f6001d,0x57f7001d,0x57f8001d,
	0x57f9001d,0x57fa001d,0x57fb001d,0x57fc001d,0x57fd001d,0x57fe001d,0x57ff001d,0x5800001d,
	0x5801001d,0x5802001d,0x5803001d,0x5804001d,0x5805001d,0x5806001d,0x5807001d,0x5808001d,
	0x5809001d,0x580a001d,0x580b001d,0x580c001d,0x580d001d,0x580e001d,0x580f001d,0x5810001d,
	0x5811001d,0x5812001d,0x5813001d,0x5814001d,0x5815001d,0x5816001d,0x5817001d,0x5818001d,
	0x5819001d,0x581a001d,0x581b001d,0x581c001d,0x581d001d,0x581e001d,0x581f001d,0x5820001d,
	0x5821001d,0x5822001d,0x5823001d,0x5824001d,0x5825001d,0x5826001d,0x5827001d,0x5828001d,
	0x5829001d,0x582a001d,0x582b001d,0x582c001d,0x582d001d,0x582e001d,0x582f001d,0x5830001d,
	0x5831001d,0x5832001d,0x5833001d,0x5834001d,0x5835001d,0x5836001d,0x5837001d,0x5838001d,
	0x5839001d,0x583a001d,0x583b001d,0x583c001d,0x583d001d,0x583e001d,0x583f001d,0x5840001d,
	0x5841001d,0x5842001d,0x5843001d,0x5844001d,0x5845001d,0x5846001d,0x5847001d,0x5848001d,
	0x5849001d,0x584a001d,0x584b001d,0x584c001d,0x584d001d,0x584e001d,0x584f001d,0x5850001d,
	0x5851001d,0x5852001d,0x5853001d,0x5854001d,0x5855001d,0x5856001d,0x5857001d,0x5858001d,
	0x5859001d,0x585a001d,0x585b001d,0x585c001d,0x585d001d,0x585e001d,0x585f001d,0x5860001d,
	0x5861001d,0x5862001d,0x5863001d,0x5864001d,0x5865001d,0x5866001d,0x5867001d,0x5868001d,
	0x5869001d,0x586a001d,0x586b001d,0x586c001d,0x586d001d,0x586e001d,0x586f001d,0x5870001d,
	0x5871001d,0x5872001d,0x5873001d,0x5874001d,0x5875001d,0x5876001d,0x5877001d,0x5878001d,
	0x5879001d,0x587a001d,0x587b001d,0x587c001d,0x587d001d,0x587e001d,0x587f001d,0x5880001d,
	0x5881001d,0x5882001d,0x5883001d,0x5884001d,0x5885001d,0x5886001d,0x5887001d,0x5888001d,
	0x5889001d,0x588a001d,0x588b001d,0x588c001d,0x588d001d,0x588e001d,0x588f001d,0x5890001d,
	0x5891001d,0x5892001d,0x5893001d,0x5894001d,0x5895001d,0x5896001d,0x5897001d,0x5898001d,
	0x5899001d,0x589a001d,0x589b001d,0x589c001d,0x589d001d,0x589e001d,0x589f001d,0x58a0001d,
	0x58a1001d,0x58a2001d,0x58a3001d,0x58a4001d,0x58a5001d,0x58a6001d,0x58a7001d,0x58a8001d,
	0x58a9001d,0x58aa001d,0x58ab001d,0x58ac001d,0x58ad001d,0x58ae001d,0x58af001d,0x58b0001d,
	0x58b1001d,0x58b2001d,0x58b3001d,0x58b4001d,0x58b5001d,0x58b6001d,0x58b7001d,0x58b8001d,
	0x58b9001d,0x58ba001d,0x58bb001d,0x58bc001d,0x58bd001d,0x58be001d,0x58bf001d,0x58c0001d,
	0x58c1001d,0x58c2001d,0x58c3001d,0x58c4001d,0x58c5001d,0x58c6001d,0x58c7001d,0x58c8001d,
	0x58c9001d,0x58ca001d,0x58cb001d,0x58cc001d,0x58cd001d,0x58ce001d,0x58cf001d,0x58d0001d,
	0x58d1001d,0x58d2001d,0x58d3001d,0x58d4001d,0x58d5001d,0x58d6001d,0x58d7001d,0x58d8001d,
	0x58d9001d,0x58da001d,0x58db001d,0x58dc001d,0x58dd001d,0x58de001d,0x58df001d,0x58e0001d,
	0x58e1001d,0x58e2001d,0x58e3001d,0x58e4001d,0x58e5001d,0x58e6001d,0x58e7001d,0x58e8001d,
	0x58e9001d,0x58ea001d,0x58eb001d,0x58ec001d,0x58ed001d,0x58ee001d,0x58ef001d,0x58f0001d,
	0x58f1001d,0x58f2001d,0x58f3001d,0x58f4001d,0x58f5001d,0x58f6001d,0x58f7001d,0x58f8001d,
	0x58f9001d,0x58fa001d,0x58fb001d,0x58fc001d,0x58fd001d,0x58fe001d,0x58ff001d,0x5900001d,
	0x5901001d,0x5902001d,0x5903001d,0x5904001d,0x5905001d,0x5906001d,0x5907001d,0x5908001d,
	0x5909001d,0x590a001d,0x590b001d,0x590c001d,0x590d001d,0x590e001d,0x590f001d,0x5910001d,
	0x5911001d,0x5912001d,0x5913001d,0x5914001d,0x5915001d,0x5916001d,0x5917001d,0x5918001d,
	0x5919001d,0x591a001d,0x591b001d,0x591c001d,0x591d001d,0x591e001d,0x591f001d,0x5920001d,
	0x5921001d,0x5922001d,0x5923001d,0x5924001d,0x5925001d,0x5926001d,0x5927001d,0x5928001d,
	0x5929001d,0x592a001d,0x592b001d,0x592c001d,0x592d001d,0x592e001d,0x592f001d,0x5930001d,
	0x5931001d,0x5932001d,0x5933001d,0x5934001d,0x5935001d,0x5936001d,0x5937001d,0x5938001d,
	0x5939001d,0x593a001d,0x593b001d,0x593c001d,0x593d001d,0x593e001d,0x593f001d,0x5940001d,
	0x5941001d,0x5942001d,0x5943001d,0x5944001d,0x5945001d,0x5946001d,0x5947001d,0x5948001d,
	0x5949001d,0x594a001d,0x594b001d,0x594c001d,0x594d001d,0x594e001d,0x594f001d,0x5950001d,
	0x5951001d,0x5952001d,0x5953001d,0x5954001d,0x5955001d,0x5956001d,0x5957001d,0x5958001d,
	0x5959001d,0x595a001d,0x595b001d,0x595c001d,0x595d001d,0x595e001d,0x595f001d,0x5960001d,
	0x5961001d,0x5962001d,0x5963001d,0x5964001d,0x5965001d,0x5966001d,0x5967001d,0x5968001d,
	0x5969001d,0x596a001d,0x596b001d,0x596c001d,0x596d001d,0x596e001d,0x596f001d,0x5970001d,
	0x5971001d,0x5972001d,0x5973001d,0x5974001d,0x5975001d,0x5976001d,0x5977001d,0x5978001d,
	0x5979001d,0x597a001d,0x597b001d,0x597c001d,0x597d001d,0x597e001d,0x597f001d,0x5980001d,
	0x5981001d,0x5982001d,0x5983001d,0x5984001d,0x5985001d,0x5986001d,0x5987001d,0x5988001d,
	0x5989001d,0x598a001d,0x598b001d,0x598c001d,0x598d001d,0x598e001d,0x598f001d,0x5990001d,
	0x5991001d,0x5992001d,0x5993001d,0x5994001d,0x5995001d,0x5996001d,0x5997001d,0x5998001d,
	0x5999001d,0x599a001d,0x599b001d,0x599c001d,0x599d001d,0x599e001d,0x599f001d,0x59a0001d,
	0x59a1001d,0x59a2001d,0x59a3001d,0x59a4001d,0x59a5001d,0x59a6001d,0x59a7001d,0x59a8001d,
	0x59a9001d,0x59aa001d,0x59ab001d,0x59ac001d,0x59ad001d,0x59ae001d,0x59af001d,0x59b0001d,
	0x59b1001d,0x59b2001d,0x59b3001d,0x59b4001d,0x59b5001d,0x59b6001d,0x59b7001d,0x59b8001d,
	0x59b9001d,0x59ba001d,0x59bb001d,0x59bc001d,0x59bd001d,0x59be001d,0x59bf001d,0x59c0001d,
	0x59c1001d,0x59c2001d,0x59c3001d,0x59c4001d,0x59c5001d,0x59c6001d,0x59c7001d,0x59c8001d,
	0x59c9001d,0x59ca001d,0x59cb001d,0x59cc001d,0x59cd001d,0x59ce001d,0x59cf001d,0x59d0001d,
	0x59d1001d,0x59d2001d,0x59d3001d,0x59d4001d,0x59d5001d,0x59d6001d,0x59d7001d,0x59d8001d,
	0x59d9001d,0x59da001d,0x59db001d,0x59dc001d,0x59dd001d,0x59de001d,0x59df001d,0x59e0001d,
	0x59e1001d,0x59e2001d,0x59e3001d,0x59e4001d,0x59e5001d,0x59e6001d,0x59e7001d,0x59e8001d,
	0x59e9001d,0x59ea001d,0x59eb001d,0x59ec001d,0x59ed001d,0x59ee001d,0x59ef001d,0x59f0001d,
	0x59f1001d,0x59f2001d,0x59f3001d,0x59f4001d,0x59f5001d,0x59f6001d,0x59f7001d,0x59f8001d,
	0x59f9001d,0x59fa001d,0x59fb001d,0x59fc001d,0x59fd001d,0x59fe001d,0x59ff001d,0x5a00001d,
	0x5a01001d,0x5a02001d,0x5a03001d,0x5a04001d,0x5a05001d,0x5a06001d,0x5a07001d,0x5a08001d,
	0x5a09001d,0x5a0a001d,0x5a0b001d,0x5a0c001d,0x5a0d001d,0x5a0e001d,0x5a0f001d,0x5a10001d,
	0x5a11001d,0x5a12001d,0x5a13001d,0x5a14001d,0x5a15001d,0x5a16001d,0x5a17001d,0x5a18001d,
	0x5a19001d,0x5a1a001d,0x5a1b001d,0x5a1c001d,0x5a1d001d,0x5a1e001d,0x5a1f001d,0x5a20001d,
	0x5a21001d,0x5a22001d,0x5a23001d,0x5a24001d,0x5a25001d,0x5a26001d,0x5a27001d,0x5a28001d,
	0x5a29001d,0x5a2a001d,0x5a2b001d,0x5a2c001d,0x5a2d001d,0x5a2e001d,0x5a2f001d,0x5a30001d,
	0x5a31001d,0x5a32001d,0x5a33001d,0x5a34001d,0x5a35001d,0x5a36001d,0x5a37001d,0x5a38001d,
	0x5a39001d,0x5a3a001d,0x5a3b001d,0x5a3c001d,0x5a3d001d,0x5a3e001d,0x5a3f001d,0x5a40001d,
	0x5a41001d,0x5a42001d,0x5a43001d,0x5a44001d,0x5a45001d,0x5a46001d,0x5a47001d,0x5a48001d,
	0x5a49001d,0x5a4a001d,0x5a4b001d,0x5a4c001d,0x5a4d001d,0x5a4e001d,0x5a4f001d,0x5a50001d,
	0x5a51001d,0x5a52001d,0x5a53001d,0x5a54001d,0x5a55001d,0x5a56001d,0x5a57001d,0x5a58001d,
	0x5a59001d,0x5a5a001d,0x5a5b001d,0x5a5c001d,0x5a5d001d,0x5a5e001d,0x5a5f001d,0x5a60001d,
	0x5a61001d,0x5a62001d,0x5a63001d,0x5a64001d,0x5a65001d,0x5a66001d,0x5a67001d,0x5a68001d,
	0x5a69001d,0x5a6a001d,0x5a6b001d,0x5a6c001d,0x5a6d001d,0x5a6e001d,0x5a6f001d,0x5a70001d,
	0x5a71001d,0x5a72001d,0x5a73001d,0x5a74001d,0x5a75001d,0x5a76001d,0x5a77001d,0x5a78001d,
	0x5a79001d,0x5a7a001d,0x5a7b001d,0x5a7c001d,0x5a7d001d,0x5a7e001d,0x5a7f001d,0x5a80001d,
	0x5a81001d,0x5a82001d,0x5a83001d,0x5a84001d,0x5a85001d,0x5a86001d,0x5a87001d,0x5a88001d,
	0x5a89001d,0x5a8a001d,0x5a8b001d,0x5a8c001d,0x5a8d001d,0x5a8e001d,0x5a8f001d,0x5a90001d,
	0x5a91001d,0x5a92001d,0x5a93001d,0x5a94001d,0x5a95001d,0x5a96001d,0x5a97001d,0x5a98001d,
	0x5a99001d,0x5a9a001d,0x5a9b001d,0x5a9c001d,0x5a9d001d,0x5a9e001d,0x5a9f001d,0x5aa0001d,
	0x5aa1001d,0x5aa2001d,0x5aa3001d,0x5aa4001d,0x5aa5001d,0x5aa6001d,0x5aa7001d,0x5aa8001d,
	0x5aa9001d,0x5aaa001d,0x5aab001d,0x5aac001d,0x5aad001d,0x5aae001d,0x5aaf001d,0x5ab0001d,
	0x5ab1001d,0x5ab2001d,0x5ab3001d,0x5ab4001d,0x5ab5001d,0x5ab6001d,0x5ab7001d,0x5ab8001d,
	0x5ab9001d,0x5aba001d,0x5abb001d,0x5abc001d,0x5abd001d,0x5abe001d,0x5abf001d,0x5ac0001d,
	0x5ac1001d,0x5ac2001d,0x5ac3001d,0x5ac4001d,0x5ac5001d,0x5ac6001d,0x5ac7001d,0x5ac8001d,
	0x5ac9001d,0x5aca001d,0x5acb001d,0x5acc001d,0x5acd001d,0x5ace001d,0x5acf001d,0x5ad0001d,
	0x5ad1001d,0x5ad2001d,0x5ad3001d,0x5ad4001d,0x5ad5001d,0x5ad6001d,0x5ad7001d,0x5ad8001d,
	0x5ad9001d,0x5ada001d,0x5adb001d,0x5adc001d,0x5add001d,0x5ade001d,0x5adf001d,0x5ae0001d,
	0x5ae1001d,0x5ae2001d,0x5ae3001d,0x5ae4001d,0x5ae5001d,0x5ae6001d,0x5ae7001d,0x5ae8001d,
	0x5ae9001d,0x5aea001d,0x5aeb001d,0x5aec001d,0x5aed001d,0x5aee001d,0x5aef001d,0x5af0001d,
	0x5af1001d,0x5af2001d,0x5af3001d,0x5af4001d,0x5af5001d,0x5af6001d,0x5af7001d,0x5af8001d,
	0x5af9001d,0x5afa001d,0x5afb001d,0x5afc001d,0x5afd001d,0x5afe001d,0x5aff001d,0x5b00001d,
	0x5b01001d,0x5b02001d,0x5b03001d,0x5b04001d,0x5b05001d,0x5b06001d,0x5b07001d,0x5b08001d,
	0x5b09001d,0x5b0a001d,0x5b0b001d,0x5b0c001d,0x5b0d001d,0x5b0e001d,0x5b0f001d,0x5b10001d,
	0x5b11001d,0x5b12001d,0x5b13001d,0x5b14001d,0x5b15001d,0x5b16001d,0x5b17001d,0x5b18001d,
	0x5b19001d,0x5b1a001d,0x5b1b001d,0x5b1c001d,0x5b1d001d,0x5b1e001d,0x5b1f001d,0x5b20001d,
	0x5b21001d,0x5b22001d,0x5b23001d,0x5b24001d,0x5b25001d,0x5b26001d,0x5b27001d,0x5b28001d,
	0x5b29001d,0x5b2a001d,0x5b2b001d,0x5b2c001d,0x5b2d001d,0x5b2e001d,0x5b2f001d,0x5b30001d,
	0x5b31001d,0x5b32001d,0x5b33001d,0x5b34001d,0x5b35001d,0x5b36001d,0x5b37001d,0x5b38001d,
	0x5b39001d,0x5b3a001d,0x5b3b001d,0x5b3c001d,0x5b3d001d,0x5b3e001d,0x5b3f001d,0x5b40001d,
	0x5b41001d,0x5b42001d,0x5b43001d,0x5b44001d,0x5b45001d,0x5b46001d,0x5b47001d,0x5b48001d,
	0x5b49001d,0x5b4a001d,0x5b4b001d,0x5b4c001d,0x5b4d001d,0x5b4e001d,0x5b4f001d,0x5b50001d,
	0x5b51001d,0x5b52001d,0x5b53001d,0x5b54001d,0x5b55001d,0x5b56001d,0x5b57001d,0x5b58001d,
	0x5b59001d,0x5b5a001d,0x5b5b001d,0x5b5c001d,0x5b5d001d,0x5b5e001d,0x5b5f001d,0x5b60001d,
	0x5b61001d,0x5b62001d,0x5b63001d,0x5b64001d,0x5b65001d,0x5b66001d,0x5b67001d,0x5b68001d,
	0x5b69001d,0x5b6a001d,0x5b6b001d,0x5b6c001d,0x5b6d001d,0x5b6e001d,0x5b6f001d,0x5b70001d,
	0x5b71001d,0x5b72001d,0x5b73001d,0x5b74001d,0x5b75001d,0x5b76001d,0x5b77001d,0x5b78001d,
	0x5b79001d,0x5b7a001d,0x5b7b001d,0x5b7c001d,0x5b7d001d,0x5b7e001d,0x5b7f001d,0x5b80001d,
	0x5b81001d,0x5b82001d,0x5b83001d,0x5b84001d,0x5b85001d,0x5b86001d,0x5b87001d,0x5b88001d,
	0x5b89001d,0x5b8a001d,0x5b8b001d,0x5b8c001d,0x5b8d001d,0x5b8e001d,0x5b8f001d,0x5b90001d,
	0x5b91001d,0x5b92001d,0x5b93001d,0x5b94001d,0x5b95001d,0x5b96001d,0x5b97001d,0x5b98001d,
	0x5b99001d,0x5b9a001d,0x5b9b001d,0x5b9c001d,0x5b9d001d,0x5b9e001d,0x5b9f001d,0x5ba0001d,
	0x5ba1001d,0x5ba2001d,0x5ba3001d,0x5ba4001d,0x5ba5001d,0x5ba6001d,0x5ba7001d,0x5ba8001d,
	0x5ba9001d,0x5baa001d,0x5bab001d,0x5bac001d,0x5bad001d,0x5bae001d,0x5baf001d,0x5bb0001d,
	0x5bb1001d,0x5bb2001d,0x5bb3001d,0x5bb4001d,0x5bb5001d,0x5bb6001d,0x5bb7001d,0x5bb8001d,
	0x5bb9001d,0x5bba001d,0x5bbb001d,0x5bbc001d,0x5bbd001d,0x5bbe001d,0x5bbf001d,0x5bc0001d,
	0x5bc1001d,0x5bc2001d,0x5bc3001d,0x5bc4001d,0x5bc5001d,0x5bc6001d,0x5bc7001d,0x5bc8001d,
	0x5bc9001d,0x5bca001d,0x5bcb001d,0x5bcc001d,0x5bcd001d,0x5bce001d,0x5bcf001d,0x5bd0001d,
	0x5bd1001d,0x5bd2001d,0x5bd3001d,0x5bd4001d,0x5bd5001d,0x5bd6001d,0x5bd7001d,0x5bd8001d,
	0x5bd9001d,0x5bda001d,0x5bdb001d,0x5bdc001d,0x5bdd001d,0x5bde001d,0x5bdf001d,0x5be0001d,
	0x5be1001d,0x5be2001d,0x5be3001d,0x5be4001d,0x5be5001d,0x5be6001d,0x5be7001d,0x5be8001d,
	0x5be9001d,0x5bea001d,0x5beb001d,0x5bec001d,0x5bed001d,0x5bee001d,0x5bef001d,0x5bf0001d,
	0x5bf1001d,0x5bf2001d,0x5bf3001d,0x5bf4001d,0x5bf5001d,0x5bf6001d,0x5bf7001d,0x5bf8001d,
	0x5bf9001d,0x5bfa001d,0x5bfb001d,0x5bfc001d,0x5bfd001d,0x5bfe001d,0x5bff001d,0x5c00001d,
	0x5c01001d,0x5c02001d,0x5c03001d,0x5c04001d,0x5c05001d,0x5c06001d,0x5c07001d,0x5c08001d,
	0x5c09001d,0x5c0a001d,0x5c0b001d,0x5c0c001d,0x5c0d001d,0x5c0e001d,0x5c0f001d,0x5c10001d,
	0x5c11001d,0x5c12001d,0x5c13001d,0x5c14001d,0x5c15001d,0x5c16001d,0x5c17001d,0x5c18001d,
	0x5c19001d,0x5c1a001d,0x5c1b001d,0x5c1c001d,0x5c1d001d,0x5c1e001d,0x5c1f001d,0x5c20001d,
	0x5c21001d,0x5c22001d,0x5c23001d,0x5c24001d,0x5c25001d,0x5c26001d,0x5c27001d,0x5c28001d,
	0x5c29001d,0x5c2a001d,0x5c2b001d,0x5c2c001d,0x5c2d001d,0x5c2e001d,0x5c2f001d,0x5c30001d,
	0x5c31001d,0x5c32001d,0x5c33001d,0x5c34001d,0x5c35001d,0x5c36001d,0x5c37001d,0x5c38001d,
	0x5c39001d,0x5c3a001d,0x5c3b001d,0x5c3c001d,0x5c3d001d,0x5c3e001d,0x5c3f001d,0x5c40001d,
	0x5c41001d,0x5c42001d,0x5c43001d,0x5c44001d,0x5c45001d,0x5c46001d,0x5c47001d,0x5c48001d,
	0x5c49001d,0x5c4a001d,0x5c4b001d,0x5c4c001d,0x5c4d001d,0x5c4e001d,0x5c4f001d,0x5c50001d,
	0x5c51001d,0x5c52001d,0x5c53001d,0x5c54001d,0x5c55001d,0x5c56001d,0x5c57001d,0x5c58001d,
	0x5c59001d,0x5c5a001d,0x5c5b001d,0x5c5c001d,0x5c5d001d,0x5c5e001d,0x5c5f001d,0x5c60001d,
	0x5c61001d,0x5c62001d,0x5c63001d,0x5c64001d,0x5c65001d,0x5c66001d,0x5c67001d,0x5c68001d,
	0x5c69001d,0x5c6a001d,0x5c6b001d,0x5c6c001d,0x5c6d001d,0x5c6e001d,0x5c6f001d,0x5c70001d,
	0x5c71001d,0x5c72001d,0x5c73001d,0x5c74001d,0x5c75001d,0x5c76001d,0x5c77001d,0x5c78001d,
	0x5c79001d,0x5c7a001d,0x5c7b001d,0x5c7c001d,0x5c7d001d,0x5c7e001d,0x5c7f001d,0x5c80001d,
	0x5c81001d,0x5c82001d,0x5c83001d,0x5c84001d,0x5c85001d,0x5c86001d,0x5c87001d,0x5c88001d,
	0x5c89001d,0x5c8a001d,0x5c8b001d,0x5c8c001d,0x5c8d001d,0x5c8e001d,0x5c8f001d,0x5c90001d,
	0x5c91001d,0x5c92001d,0x5c93001d,0x5c94001d,0x5c95001d,0x5c96001d,0x5c97001d,0x5c98001d,
	0x5c99001d,0x5c9a001d,0x5c9b001d,0x5c9c001d,0x5c9d001d,0x5c9e001d,0x5c9f001d,0x5ca0001d,
	0x5ca1001d,0x5ca2001d,0x5ca3001d,0x5ca4001d,0x5ca5001d,0x5ca6001d,0x5ca7001d,0x5ca8001d,
	0x5ca9001d,0x5caa001d,0x5cab001d,0x5cac001d,0x5cad001d,0x5cae001d,0x5caf001d,0x5cb0001d,
	0x5cb1001d,0x5cb2001d,0x5cb3001d,0x5cb4001d,0x5cb5001d,0x5cb6001d,0x5cb7001d,0x5cb8001d,
	0x5cb9001d,0x5cba001d,0x5cbb001d,0x5cbc001d,0x5cbd001d,0x5cbe001d,0x5cbf001d,0x5cc0001d,
	0x5cc1001d,0x5cc2001d,0x5cc3001d,0x5cc4001d,0x5cc5001d,0x5cc6001d,0x5cc7001d,0x5cc8001d,
	0x5cc9001d,0x5cca001d,0x5ccb001d,0x5ccc001d,0x5ccd001d,0x5cce001d,0x5ccf001d,0x5cd0001d,
	0x5cd1001d,0x5cd2001d,0x5cd3001d,0x5cd4001d,0x5cd5001d,0x5cd6001d,0x5cd7001d,0x5cd8001d,
	0x5cd9001d,0x5cda001d,0x5cdb001d,0x5cdc001d,0x5cdd001d,0x5cde001d,0x5cdf001d,0x5ce0001d,
	0x5ce1001d,0x5ce2001d,0x5ce3001d,0x5ce4001d,0x5ce5001d,0x5ce6001d,0x5ce7001d,0x5ce8001d,
	0x5ce9001d,0x5cea001d,0x5ceb001d,0x5cec001d,0x5ced001d,0x5cee001d,0x5cef001d,0x5cf0001d,
	0x5cf1001d,0x5cf2001d,0x5cf3001d,0x5cf4001d,0x5cf5001d,0x5cf6001d,0x5cf7001d,0x5cf8001d,
	0x5cf9001d,0x5cfa001d,0x5cfb001d,0x5cfc001d,0x5cfd001d,0x5cfe001d,0x5cff001d,0x5d00001d,
	0x5d01001d,0x5d02001d,0x5d03001d,0x5d04001d,0x5d05001d,0x5d06001d,0x5d07001d,0x5d08001d,
	0x5d09001d,0x5d0a001d,0x5d0b001d,0x5d0c001d,0x5d0d001d,0x5d0e001d,0x5d0f001d,0x5d10001d,
	0x5d11001d,0x5d12001d,0x5d13001d,0x5d14001d,0x5d15001d,0x5d16001d,0x5d17001d,0x5d18001d,
	0x5d19001d,0x5d1a001d,0x5d1b001d,0x5d1c001d,0x5d1d001d,0x5d1e001d,0x5d1f001d,0x5d20001d,
	0x5d21001d,0x5d22001d,0x5d23001d,0x5d24001d,0x5d25001d,0x5d26001d,0x5d27001d,0x5d28001d,
	0x5d29001d,0x5d2a001d,0x5d2b001d,0x5d2c001d,0x5d2d001d,0x5d2e001d,0x5d2f001d,0x5d30001d,
	0x5d31001d,0x5d32001d,0x5d33001d,0x5d34001d,0x5d35001d,0x5d36001d,0x5d37001d,0x5d38001d,
	0x5d39001d,0x5d3a001d,0x5d3b001d,0x5d3c001d,0x5d3d001d,0x5d3e001d,0x5d3f001d,0x5d40001d,
	0x5d41001d,0x5d42001d,0x5d43001d,0x5d44001d,0x5d45001d,0x5d46001d,0x5d47001d,0x5d48001d,
	0x5d49001d,0x5d4a001d,0x5d4b001d,0x5d4c001d,0x5d4d001d,0x5d4e001d,0x5d4f001d,0x5d50001d,
	0x5d51001d,0x5d52001d,0x5d53001d,0x5d54001d,0x5d55001d,0x5d56001d,0x5d57001d,0x5d58001d,
	0x5d59001d,0x5d5a001d,0x5d5b001d,0x5d5c001d,0x5d5d001d,0x5d5e001d,0x5d5f001d,0x5d60001d,
	0x5d61001d,0x5d62001d,0x5d63001d,0x5d64001d,0x5d65001d,0x5d66001d,0x5d67001d,0x5d68001d,
	0x5d69001d,0x5d6a001d,0x5d6b001d,0x5d6c001d,0x5d6d001d,0x5d6e001d,0x5d6f001d,0x5d70001d,
	0x5d71001d,0x5d72001d,0x5d73001d,0x5d74001d,0x5d75001d,0x5d76001d,0x5d77001d,0x5d78001d,
	0x5d79001d,0x5d7a001d,0x5d7b001d,0x5d7c001d,0x5d7d001d,0x5d7e001d,0x5d7f001d,0x5d80001d,
	0x5d81001d,0x5d82001d,0x5d83001d,0x5d84001d,0x5d85001d,0x5d86001d,0x5d87001d,0x5d88001d,
	0x5d89001d,0x5d8a001d,0x5d8b001d,0x5d8c001d,0x5d8d001d,0x5d8e001d,0x5d8f001d,0x5d90001d,
	0x5d91001d,0x5d92001d,0x5d93001d,0x5d94001d,0x5d95001d,0x5d96001d,0x5d97001d,0x5d98001d,
	0x5d99001d,0x5d9a001d,0x5d9b001d,0x5d9c001d,0x5d9d001d,0x5d9e001d,0x5d9f001d,0x5da0001d,
	0x5da1001d,0x5da2001d,0x5da3001d,0x5da4001d,0x5da5001d,0x5da6001d,0x5da7001d,0x5da8001d,
	0x5da9001d,0x5daa001d,0x5dab001d,0x5dac001d,0x5dad001d,0x5dae001d,0x5daf001d,0x5db0001d,
	0x5db1001d,0x5db2001d,0x5db3001d,0x5db4001d,0x5db5001d,0x5db6001d,0x5db7001d,0x5db8001d,
	0x5db9001d,0x5dba001d,0x5dbb001d,0x5dbc001d,0x5dbd001d,0x5dbe001d,0x5dbf001d,0x5dc0001d,
	0x5dc1001d,0x5dc2001d,0x5dc3001d,0x5dc4001d,0x5dc5001d,0x5dc6001d,0x5dc7001d,0x5dc8001d,
	0x5dc9001d,0x5dca001d,0x5dcb001d,0x5dcc001d,0x5dcd001d,0x5dce001d,0x5dcf001d,0x5dd0001d,
	0x5dd1001d,0x5dd2001d,0x5dd3001d,0x5dd4001d,0x5dd5001d,0x5dd6001d,0x5dd7001d,0x5dd8001d,
	0x5dd9001d,0x5dda001d,0x5ddb001d,0x5ddc001d,0x5ddd001d,0x5dde001d,0x5ddf001d,0x5de0001d,
	0x5de1001d,0x5de2001d,0x5de3001d,0x5de4001d,0x5de5001d,0x5de6001d,0x5de7001d,0x5de8001d,
	0x5de9001d,0x5dea001d,0x5deb001d,0x5dec001d,0x5ded001d,0x5dee001d,0x5def001d,0x5df0001d,
	0x5df1001d,0x5df2001d,0x5df3001d,0x5df4001d,0x5df5001d,0x5df6001d,0x5df7001d,0x5df8001d,
	0x5df9001d,0x5dfa001d,0x5dfb001d,0x5dfc001d,0x5dfd001d,0x5dfe001d,0x5dff001d,0x5e00001d,
	0x5e01001d,0x5e02001d,0x5e03001d,0x5e04001d,0x5e05001d,0x5e06001d,0x5e07001d,0x5e08001d,
	0x5e09001d,0x5e0a001d,0x5e0b001d,0x5e0c001d,0x5e0d001d,0x5e0e001d,0x5e0f001d,0x5e10001d,
	0x5e11001d,0x5e12001d,0x5e13001d,0x5e14001d,0x5e15001d,0x5e16001d,0x5e17001d,0x5e18001d,
	0x5e19001d,0x5e1a001d,0x5e1b001d,0x5e1c001d,0x5e1d001d,0x5e1e001d,0x5e1f001d,0x5e20001d,
	0x5e21001d,0x5e22001d,0x5e23001d,0x5e24001d,0x5e25001d,0x5e26001d,0x5e27001d,0x5e28001d,
	0x5e29001d,0x5e2a001d,0x5e2b001d,0x5e2c001d,0x5e2d001d,0x5e2e001d,0x5e2f001d,0x5e30001d,
	0x5e31001d,0x5e32001d,0x5e33001d,0x5e34001d,0x5e35001d,0x5e36001d,0x5e37001d,0x5e38001d,
	0x5e39001d,0x5e3a001d,0x5e3b001d,0x5e3c001d,0x5e3d001d,0x5e3e001d,0x5e3f001d,0x5e40001d,
	0x5e41001d,0x5e42001d,0x5e43001d,0x5e44001d,0x5e45001d,0x5e46001d,0x5e47001d,0x5e48001d,
	0x5e49001d,0x5e4a001d,0x5e4b001d,0x5e4c001d,0x5e4d001d,0x5e4e001d,0x5e4f001d,0x5e50001d,
	0x5e51001d,0x5e52001d,0x5e53001d,0x5e54001d,0x5e55001d,0x5e56001d,0x5e57001d,0x5e58001d,
	0x5e59001d,0x5e5a001d,0x5e5b001d,0x5e5c001d,0x5e5d001d,0x5e5e001d,0x5e5f001d,0x5e60001d,
	0x5e61001d,0x5e62001d,0x5e63001d,0x5e64001d,0x5e65001d,0x5e66001d,0x5e67001d,0x5e68001d,
	0x5e69001d,0x5e6a001d,0x5e6b001d,0x5e6c001d,0x5e6d001d,0x5e6e001d,0x5e6f001d,0x5e70001d,
	0x5e71001d,0x5e72001d,0x5e73001d,0x5e74001d,0x5e75001d,0x5e76001d,0x5e77001d,0x5e78001d,
	0x5e79001d,0x5e7a001d,0x5e7b001d,0x5e7c001d,0x5e7d001d,0x5e7e001d,0x5e7f001d,0x5e80001d,
	0x5e81001d,0x5e82001d,0x5e83001d,0x5e84001d,0x5e85001d,0x5e86001d,0x5e87001d,0x5e88001d,
	0x5e89001d,0x5e8a001d,0x5e8b001d,0x5e8c001d,0x5e8d001d,0x5e8e001d,0x5e8f001d,0x5e90001d,
	0x5e91001d,0x5e92001d,0x5e93001d,0x5e94001d,0x5e95001d,0x5e96001d,0x5e97001d,0x5e98001d,
	0x5e99001d,0x5e9a001d,0x5e9b001d,0x5e9c001d,0x5e9d001d,0x5e9e001d,0x5e9f001d,0x5ea0001d,
	0x5ea1001d,0x5ea2001d,0x5ea3001d,0x5ea4001d,0x5ea5001d,0x5ea6001d,0x5ea7001d,0x5ea8001d,
	0x5ea9001d,0x5eaa001d,0x5eab001d,0x5eac001d,0x5ead001d,0x5eae001d,0x5eaf001d,0x5eb0001d,
	0x5eb1001d,0x5eb2001d,0x5eb3001d,0x5eb4001d,0x5eb5001d,0x5eb6001d,0x5eb7001d,0x5eb8001d,
	0x5eb9001d,0x5eba001d,0x5ebb001d,0x5ebc001d,0x5ebd001d,0x5ebe001d,0x5ebf001d,0x5ec0001d,
	0x5ec1001d,0x5ec2001d,0x5ec3001d,0x5ec4001d,0x5ec5001d,0x5ec6001d,0x5ec7001d,0x5ec8001d,
	0x5ec9001d,0x5eca001d,0x5ecb001d,0x5ecc001d,0x5ecd001d,0x5ece001d,0x5ecf001d,0x5ed0001d,
	0x5ed1001d,0x5ed2001d,0x5ed3001d,0x5ed4001d,0x5ed5001d,0x5ed6001d,0x5ed7001d,0x5ed8001d,
	0x5ed9001d,0x5eda001d,0x5edb001d,0x5edc001d,0x5edd001d,0x5ede001d,0x5edf001d,0x5ee0001d,
	0x5ee1001d,0x5ee2001d,0x5ee3001d,0x5ee4001d,0x5ee5001d,0x5ee6001d,0x5ee7001d,0x5ee8001d,
	0x5ee9001d,0x5eea001d,0x5eeb001d,0x5eec001d,0x5eed001d,0x5eee001d,0x5eef001d,0x5ef0001d,
	0x5ef1001d,0x5ef2001d,0x5ef3001d,0x5ef4001d,0x5ef5001d,0x5ef6001d,0x5ef7001d,0x5ef8001d,
	0x5ef9001d,0x5efa001d,0x5efb001d,0x5efc001d,0x5efd001d,0x5efe001d,0x5eff001d,0x5f00001d,
	0x5f01001d,0x5f02001d,0x5f03001d,0x5f04001d,0x5f05001d,0x5f06001d,0x5f07001d,0x5f08001d,
	0x5f09001d,0x5f0a001d,0x5f0b001d,0x5f0c001d,0x5f0d001d,0x5f0e001d,0x5f0f001d,0x5f10001d,
	0x5f11001d,0x5f12001d,0x5f13001d,0x5f14001d,0x5f15001d,0x5f16001d,0x5f17001d,0x5f18001d,
	0x5f19001d,0x5f1a001d,0x5f1b001d,0x5f1c001d,0x5f1d001d,0x5f1e001d,0x5f1f001d,0x5f20001d,
	0x5f21001d,0x5f22001d,0x5f23001d,0x5f24001d,0x5f25001d,0x5f26001d,0x5f27001d,0x5f28001d,
	0x5f29001d,0x5f2a001d,0x5f2b001d,0x5f2c001d,0x5f2d001d,0x5f2e001d,0x5f2f001d,0x5f30001d,
	0x5f31001d,0x5f32001d,0x5f33001d,0x5f34001d,0x5f35001d,0x5f36001d,0x5f37001d,0x5f38001d,
	0x5f39001d,0x5f3a001d,0x5f3b001d,0x5f3c001d,0x5f3d001d,0x5f3e001d,0x5f3f001d,0x5f40001d,
	0x5f41001d,0x5f42001d,0x5f43001d,0x5f44001d,0x5f45001d,0x5f46001d,0x5f47001d,0x5f48001d,
	0x5f49001d,0x5f4a001d,0x5f4b001d,0x5f4c001d,0x5f4d001d,0x5f4e001d,0x5f4f001d,0x5f50001d,
	0x5f51001d,0x5f52001d,0x5f53001d,0x5f54001d,0x5f55001d,0x5f56001d,0x5f57001d,0x5f58001d,
	0x5f59001d,0x5f5a001d,0x5f5b001d,0x5f5c001d,0x5f5d001d,0x5f5e001d,0x5f5f001d,0x5f60001d,
	0x5f61001d,0x5f62001d,0x5f63001d,0x5f64001d,0x5f65001d,0x5f66001d,0x5f67001d,0x5f68001d,
	0x5f69001d,0x5f6a001d,0x5f6b001d,0x5f6c001d,0x5f6d001d,0x5f6e001d,0x5f6f001d,0x5f70001d,
	0x5f71001d,0x5f72001d,0x5f73001d,0x5f74001d,0x5f75001d,0x5f76001d,0x5f77001d,0x5f78001d,
	0x5f79001d,0x5f7a001d,0x5f7b001d,0x5f7c001d,0x5f7d001d,0x5f7e001d,0x5f7f001d,0x5f80001d,
	0x5f81001d,0x5f82001d,0x5f83001d,0x5f84001d,0x5f85001d,0x5f86001d,0x5f87001d,0x5f88001d,
	0x5f89001d,0x5f8a001d,0x5f8b001d,0x5f8c001d,0x5f8d001d,0x5f8e001d,0x5f8f001d,0x5f90001d,
	0x5f91001d,0x5f92001d,0x5f93001d,0x5f94001d,0x5f95001d,0x5f96001d,0x5f97001d,0x5f98001d,
	0x5f99001d,0x5f9a001d,0x5f9b001d,0x5f9c001d,0x5f9d001d,0x5f9e001d,0x5f9f001d,0x5fa0001d,
	0x5fa1001d,0x5fa2001d,0x5fa3001d,0x5fa4001d,0x5fa5001d,0x5fa6001d,0x5fa7001d,0x5fa8001d,
	0x5fa9001d,0x5faa001d,0x5fab001d,0x5fac001d,0x5fad001d,0x5fae001d,0x5faf001d,0x5fb0001d,
	0x5fb1001d,0x5fb2001d,0x5fb3001d,0x5fb4001d,0x5fb5001d,0x5fb6001d,0x5fb7001d,0x5fb8001d,
	0x5fb9001d,0x5fba001d,0x5fbb001d,0x5fbc001d,0x5fbd001d,0x5fbe001d,0x5fbf001d,0x5fc0001d,
	0x5fc1001d,0x5fc2001d,0x5fc3001d,0x5fc4001d,0x5fc5001d,0x5fc6001d,0x5fc7001d,0x5fc8001d,
	0x5fc9001d,0x5fca001d,0x5fcb001d,0x5fcc001d,0x5fcd001d,0x5fce001d,0x5fcf001d,0x5fd0001d,
	0x5fd1001d,0x5fd2001d,0x5fd3001d,0x5fd4001d,0x5fd5001d,0x5fd6001d,0x5fd7001d,0x5fd8001d,
	0x5fd9001d,0x5fda001d,0x5fdb001d,0x5fdc001d,0x5fdd001d,0x5fde001d,0x5fdf001d,0x5fe0001d,
	0x5fe1001d,0x5fe2001d,0x5fe3001d,0x5fe4001d,0x5fe5001d,0x5fe6001d,0x5fe7001d,0x5fe8001d,
	0x5fe9001d,0x5fea001d,0x5feb001d,0x5fec001d,0x5fed001d,0x5fee001d,0x5fef001d,0x5ff0001d,
	0x5ff1001d,0x5ff2001d,0x5ff3001d,0x5ff4001d,0x5ff5001d,0x5ff6001d,0x5ff7001d,0x5ff8001d,
	0x5ff9001d,0x5ffa001d,0x5ffb001d,0x5ffc001d,0x5ffd001d,0x5ffe001d,0x5fff001d,0x6000001d,
	0x6001001d,0x6002001d,0x6003001d,0x6004001d,0x6005001d,0x6006001d,0x6007001d,0x6008001d,
	0x6009001d,0x600a001d,0x600b001d,0x600c001d,0x600d001d,0x600e001d,0x600f001d,0x6010001d,
	0x6011001d,0x6012001d,0x6013001d,0x6014001d,0x6015001d,0x6016001d,0x6017001d,0x6018001d,
	0x6019001d,0x601a001d,0x601b001d,0x601c001d,0x601d001d,0x601e001d,0x601f001d,0x6020001d,
	0x6021001d,0x6022001d,0x6023001d,0x6024001d,0x6025001d,0x6026001d,0x6027001d,0x6028001d,
	0x6029001d,0x602a001d,0x602b001d,0x602c001d,0x602d001d,0x602e001d,0x602f001d,0x6030001d,
	0x6031001d,0x6032001d,0x6033001d,0x6034001d,0x6035001d,0x6036001d,0x6037001d,0x6038001d,
	0x6039001d,0x603a001d,0x603b001d,0x603c001d,0x603d001d,0x603e001d,0x603f001d,0x6040001d,
	0x6041001d,0x6042001d,0x6043001d,0x6044001d,0x6045001d,0x6046001d,0x6047001d,0x6048001d,
	0x6049001d,0x604a001d,0x604b001d,0x604c001d,0x604d001d,0x604e001d,0x604f001d,0x6050001d,
	0x6051001d,0x6052001d,0x6053001d,0x6054001d,0x6055001d,0x6056001d,0x6057001d,0x6058001d,
	0x6059001d,0x605a001d,0x605b001d,0x605c001d,0x605d001d,0x605e001d,0x605f001d,0x6060001d,
	0x6061001d,0x6062001d,0x6063001d,0x6064001d,0x6065001d,0x6066001d,0x6067001d,0x6068001d,
	0x6069001d,0x606a001d,0x606b001d,0x606c001d,0x606d001d,0x606e001d,0x606f001d,0x6070001d,
	0x6071001d,0x6072001d,0x6073001d,0x6074001d,0x6075001d,0x6076001d,0x6077001d,0x6078001d,
	0x6079001d,0x607a001d,0x607b001d,0x607c001d,0x607d001d,0x607e001d,0x607f001d,0x6080001d,
	0x6081001d,0x6082001d,0x6083001d,0x6084001d,0x6085001d,0x6086001d,0x6087001d,0x6088001d,
	0x6089001d,0x608a001d,0x608b001d,0x608c001d,0x608d001d,0x608e001d,0x608f001d,0x6090001d,
	0x6091001d,0x6092001d,0x6093001d,0x6094001d,0x6095001d,0x6096001d,0x6097001d,0x6098001d,
	0x6099001d,0x609a001d,0x609b001d,0x609c001d,0x609d001d,0x609e001d,0x609f001d,0x60a0001d,
	0x60a1001d,0x60a2001d,0x60a3001d,0x60a4001d,0x60a5001d,0x60a6001d,0x60a7001d,0x60a8001d,
	0x60a9001d,0x60aa001d,0x60ab001d,0x60ac001d,0x60ad001d,0x60ae001d,0x60af001d,0x60b0001d,
	0x60b1001d,0x60b2001d,0x60b3001d,0x60b4001d,0x60b5001d,0x60b6001d,0x60b7001d,0x60b8001d,
	0x60b9001d,0x60ba001d,0x60bb001d,0x60bc001d,0x60bd001d,0x60be001d,0x60bf001d,0x60c0001d,
	0x60c1001d,0x60c2001d,0x60c3001d,0x60c4001d,0x60c5001d,0x60c6001d,0x60c7001d,0x60c8001d,
	0x60c9001d,0x60ca001d,0x60cb001d,0x60cc001d,0x60cd001d,0x60ce001d,0x60cf001d,0x60d0001d,
	0x60d1001d,0x60d2001d,0x60d3001d,0x60d4001d,0x60d5001d,0x60d6001d,0x60d7001d,0x60d8001d,
	0x60d9001d,0x60da001d,0x60db001d,0x60dc001d,0x60dd001d,0x60de001d,0x60df001d,0x60e0001d,
	0x60e1001d,0x60e2001d,0x60e3001d,0x60e4001d,0x60e5001d,0x60e6001d,0x60e7001d,0x60e8001d,
	0x60e9001d,0x60ea001d,0x60eb001d,0x60ec001d,0x60ed001d,0x60ee001d,0x60ef001d,0x60f0001d,
	0x60f1001d,0x60f2001d,0x60f3001d,0x60f4001d,0x60f5001d,0x60f6001d,0x60f7001d,0x60f8001d,
	0x60f9001d,0x60fa001d,0x60fb001d,0x60fc001d,0x60fd001d,0x60fe001d,0x60ff001d,0x6100001d,
	0x6101001d,0x6102001d,0x6103001d,0x6104001d,0x6105001d,0x6106001d,0x6107001d,0x6108001d,
	0x6109001d,0x610a001d,0x610b001d,0x610c001d,0x610d001d,0x610e001d,0x610f001d,0x6110001d,
	0x6111001d,0x6112001d,0x6113001d,0x6114001d,0x6115001d,0x6116001d,0x6117001d,0x6118001d,
	0x6119001d,0x611a001d,0x611b001d,0x611c001d,0x611d001d,0x611e001d,0x611f001d,0x6120001d,
	0x6121001d,0x6122001d,0x6123001d,0x6124001d,0x6125001d,0x6126001d,0x6127001d,0x6128001d,
	0x6129001d,0x612a001d,0x612b001d,0x612c001d,0x612d001d,0x612e001d,0x612f001d,0x6130001d,
	0x6131001d,0x6132001d,0x6133001d,0x6134001d,0x6135001d,0x6136001d,0x6137001d,0x6138001d,
	0x6139001d,0x613a001d,0x613b001d,0x613c001d,0x613d001d,0x613e001d,0x613f001d,0x6140001d,
	0x6141001d,0x6142001d,0x6143001d,0x6144001d,0x6145001d,0x6146001d,0x6147001d,0x6148001d,
	0x6149001d,0x614a001d,0x614b001d,0x614c001d,0x614d001d,0x614e001d,0x614f001d,0x6150001d,
	0x6151001d,0x6152001d,0x6153001d,0x6154001d,0x6155001d,0x6156001d,0x6157001d,0x6158001d,
	0x6159001d,0x615a001d,0x615b001d,0x615c001d,0x615d001d,0x615e001d,0x615f001d,0x6160001d,
	0x6161001d,0x6162001d,0x6163001d,0x6164001d,0x6165001d,0x6166001d,0x6167001d,0x6168001d,
	0x6169001d,0x616a001d,0x616b001d,0x616c001d,0x616d001d,0x616e001d,0x616f001d,0x6170001d,
	0x6171001d,0x6172001d,0x6173001d,0x6174001d,0x6175001d,0x6176001d,0x6177001d,0x6178001d,
	0x6179001d,0x617a001d,0x617b001d,0x617c001d,0x617d001d,0x617e001d,0x617f001d,0x6180001d,
	0x6181001d,0x6182001d,0x6183001d,0x6184001d,0x6185001d,0x6186001d,0x6187001d,0x6188001d,
	0x6189001d,0x618a001d,0x618b001d,0x618c001d,0x618d001d,0x618e001d,0x618f001d,0x6190001d,
	0x6191001d,0x6192001d,0x6193001d,0x6194001d,0x6195001d,0x6196001d,0x6197001d,0x6198001d,
	0x6199001d,0x619a001d,0x619b001d,0x619c001d,0x619d001d,0x619e001d,0x619f001d,0x61a0001d,
	0x61a1001d,0x61a2001d,0x61a3001d,0x61a4001d,0x61a5001d,0x61a6001d,0x61a7001d,0x61a8001d,
	0x61a9001d,0x61aa001d,0x61ab001d,0x61ac001d,0x61ad001d,0x61ae001d,0x61af001d,0x61b0001d,
	0x61b1001d,0x61b2001d,0x61b3001d,0x61b4001d,0x61b5001d,0x61b6001d,0x61b7001d,0x61b8001d,
	0x61b9001d,0x61ba001d,0x61bb001d,0x61bc001d,0x61bd001d,0x61be001d,0x61bf001d,0x61c0001d,
	0x61c1001d,0x61c2001d,0x61c3001d,0x61c4001d,0x61c5001d,0x61c6001d,0x61c7001d,0x61c8001d,
	0x61c9001d,0x61ca001d,0x61cb001d,0x61cc001d,0x61cd001d,0x61ce001d,0x61cf001d,0x61d0001d,
	0x61d1001d,0x61d2001d,0x61d3001d,0x61d4001d,0x61d5001d,0x61d6001d,0x61d7001d,0x61d8001d,
	0x61d9001d,0x61da001d,0x61db001d,0x61dc001d,0x61dd001d,0x61de001d,0x61df001d,0x61e0001d,
	0x61e1001d,0x61e2001d,0x61e3001d,0x61e4001d,0x61e5001d,0x61e6001d,0x61e7001d,0x61e8001d,
	0x61e9001d,0x61ea001d,0x61eb001d,0x61ec001d,0x61ed001d,0x61ee001d,0x61ef001d,0x61f0001d,
	0x61f1001d,0x61f2001d,0x61f3001d,0x61f4001d,0x61f5001d,0x61f6001d,0x61f7001d,0x61f8001d,
	0x61f9001d,0x61fa001d,0x61fb001d,0x61fc001d,0x61fd001d,0x61fe001d,0x61ff001d,0x6200001d,
	0x6201001d,0x6202001d,0x6203001d,0x6204001d,0x6205001d,0x6206001d,0x6207001d,0x6208001d,
	0x6209001d,0x620a001d,0x620b001d,0x620c001d,0x620d001d,0x620e001d,0x620f001d,0x6210001d,
	0x6211001d,0x6212001d,0x6213001d,0x6214001d,0x6215001d,0x6216001d,0x6217001d,0x6218001d,
	0x6219001d,0x621a001d,0x621b001d,0x621c001d,0x621d001d,0x621e001d,0x621f001d,0x6220001d,
	0x6221001d,0x6222001d,0x6223001d,0x6224001d,0x6225001d,0x6226001d,0x6227001d,0x6228001d,
	0x6229001d,0x622a001d,0x622b001d,0x622c001d,0x622d001d,0x622e001d,0x622f001d,0x6230001d,
	0x6231001d,0x6232001d,0x6233001d,0x6234001d,0x6235001d,0x6236001d,0x6237001d,0x6238001d,
	0x6239001d,0x623a001d,0x623b001d,0x623c001d,0x623d001d,0x623e001d,0x623f001d,0x6240001d,
	0x6241001d,0x6242001d,0x6243001d,0x6244001d,0x6245001d,0x6246001d,0x6247001d,0x6248001d,
	0x6249001d,0x624a001d,0x624b001d,0x624c001d,0x624d001d,0x624e001d,0x624f001d,0x6250001d,
	0x6251001d,0x6252001d,0x6253001d,0x6254001d,0x6255001d,0x6256001d,0x6257001d,0x6258001d,
	0x6259001d,0x625a001d,0x625b001d,0x625c001d,0x625d001d,0x625e001d,0x625f001d,0x6260001d,
	0x6261001d,0x6262001d,0x6263001d,0x6264001d,0x6265001d,0x6266001d,0x6267001d,0x6268001d,
	0x6269001d,0x626a001d,0x626b001d,0x626c001d,0x626d001d,0x626e001d,0x626f001d,0x6270001d,
	0x6271001d,0x6272001d,0x6273001d,0x6274001d,0x6275001d,0x6276001d,0x6277001d,0x6278001d,
	0x6279001d,0x627a001d,0x627b001d,0x627c001d,0x627d001d,0x627e001d,0x627f001d,0x6280001d,
	0x6281001d,0x6282001d,0x6283001d,0x6284001d,0x6285001d,0x6286001d,0x6287001d,0x6288001d,
	0x6289001d,0x628a001d,0x628b001d,0x628c001d,0x628d001d,0x628e001d,0x628f001d,0x6290001d,
	0x6291001d,0x6292001d,0x6293001d,0x6294001d,0x6295001d,0x6296001d,0x6297001d,0x6298001d,
	0x6299001d,0x629a001d,0x629b001d,0x629c001d,0x629d001d,0x629e001d,0x629f001d,0x62a0001d,
	0x62a1001d,0x62a2001d,0x62a3001d,0x62a4001d,0x62a5001d,0x62a6001d,0x62a7001d,0x62a8001d,
	0x62a9001d,0x62aa001d,0x62ab001d,0x62ac001d,0x62ad001d,0x62ae001d,0x62af001d,0x62b0001d,
	0x62b1001d,0x62b2001d,0x62b3001d,0x62b4001d,0x62b5001d,0x62b6001d,0x62b7001d,0x62b8001d,
	0x62b9001d,0x62ba001d,0x62bb001d,0x62bc001d,0x62bd001d,0x62be001d,0x62bf001d,0x62c0001d,
	0x62c1001d,0x62c2001d,0x62c3001d,0x62c4001d,0x62c5001d,0x62c6001d,0x62c7001d,0x62c8001d,
	0x62c9001d,0x62ca001d,0x62cb001d,0x62cc001d,0x62cd001d,0x62ce001d,0x62cf001d,0x62d0001d,
	0x62d1001d,0x62d2001d,0x62d3001d,0x62d4001d,0x62d5001d,0x62d6001d,0x62d7001d,0x62d8001d,
	0x62d9001d,0x62da001d,0x62db001d,0x62dc001d,0x62dd001d,0x62de001d,0x62df001d,0x62e0001d,
	0x62e1001d,0x62e2001d,0x62e3001d,0x62e4001d,0x62e5001d,0x62e6001d,0x62e7001d,0x62e8001d,
	0x62e9001d,0x62ea001d,0x62eb001d,0x62ec001d,0x62ed001d,0x62ee001d,0x62ef001d,0x62f0001d,
	0x62f1001d,0x62f2001d,0x62f3001d,0x62f4001d,0x62f5001d,0x62f6001d,0x62f7001d,0x62f8001d,
	0x62f9001d,0x62fa001d,0x62fb001d,0x62fc001d,0x62fd001d,0x62fe001d,0x62ff001d,0x6300001d,
	0x6301001d,0x6302001d,0x6303001d,0x6304001d,0x6305001d,0x6306001d,0x6307001d,0x6308001d,
	0x6309001d,0x630a001d,0x630b001d,0x630c001d,0x630d001d,0x630e001d,0x630f001d,0x6310001d,
	0x6311001d,0x6312001d,0x6313001d,0x6314001d,0x6315001d,0x6316001d,0x6317001d,0x6318001d,
	0x6319001d,0x631a001d,0x631b001d,0x631c001d,0x631d001d,0x631e001d,0x631f001d,0x6320001d,
	0x6321001d,0x6322001d,0x6323001d,0x6324001d,0x6325001d,0x6326001d,0x6327001d,0x6328001d,
	0x6329001d,0x632a001d,0x632b001d,0x632c001d,0x632d001d,0x632e001d,0x632f001d,0x6330001d,
	0x6331001d,0x6332001d,0x6333001d,0x6334001d,0x6335001d,0x6336001d,0x6337001d,0x6338001d,
	0x6339001d,0x633a001d,0x633b001d,0x633c001d,0x633d001d,0x633e001d,0x633f001d,0x6340001d,
	0x6341001d,0x6342001d,0x6343001d,0x6344001d,0x6345001d,0x6346001d,0x6347001d,0x6348001d,
	0x6349001d,0x634a001d,0x634b001d,0x634c001d,0x634d001d,0x634e001d,0x634f001d,0x6350001d,
	0x6351001d,0x6352001d,0x6353001d,0x6354001d,0x6355001d,0x6356001d,0x6357001d,0x6358001d,
	0x6359001d,0x635a001d,0x635b001d,0x635c001d,0x635d001d,0x635e001d,0x635f001d,0x6360001d,
	0x6361001d,0x6362001d,0x6363001d,0x6364001d,0x6365001d,0x6366001d,0x6367001d,0x6368001d,
	0x6369001d,0x636a001d,0x636b001d,0x636c001d,0x636d001d,0x636e001d,0x636f001d,0x6370001d,
	0x6371001d,0x6372001d,0x6373001d,0x6374001d,0x6375001d,0x6376001d,0x6377001d,0x6378001d,
	0x6379001d,0x637a001d,0x637b001d,0x637c001d,0x637d001d,0x637e001d,0x637f001d,0x6380001d,
	0x6381001d,0x6382001d,0x6383001d,0x6384001d,0x6385001d,0x6386001d,0x6387001d,0x6388001d,
	0x6389001d,0x638a001d,0x638b001d,0x638c001d,0x638d001d,0x638e001d,0x638f001d,0x6390001d,
	0x6391001d,0x6392001d,0x6393001d,0x6394001d,0x6395001d,0x6396001d,0x6397001d,0x6398001d,
	0x6399001d,0x639a001d,0x639b001d,0x639c001d,0x639d001d,0x639e001d,0x639f001d,0x63a0001d,
	0x63a1001d,0x63a2001d,0x63a3001d,0x63a4001d,0x63a5001d,0x63a6001d,0x63a7001d,0x63a8001d,
	0x63a9001d,0x63aa001d,0x63ab001d,0x63ac001d,0x63ad001d,0x63ae001d,0x63af001d,0x63b0001d,
	0x63b1001d,0x63b2001d,0x63b3001d,0x63b4001d,0x63b5001d,0x63b6001d,0x63b7001d,0x63b8001d,
	0x63b9001d,0x63ba001d,0x63bb001d,0x63bc001d,0x63bd001d,0x63be001d,0x63bf001d,0x63c0001d,
	0x63c1001d,0x63c2001d,0x63c3001d,0x63c4001d,0x63c5001d,0x63c6001d,0x63c7001d,0x63c8001d,
	0x63c9001d,0x63ca001d,0x63cb001d,0x63cc001d,0x63cd001d,0x63ce001d,0x63cf001d,0x63d0001d,
	0x63d1001d,0x63d2001d,0x63d3001d,0x63d4001d,0x63d5001d,0x63d6001d,0x63d7001d,0x63d8001d,
	0x63d9001d,0x63da001d,0x63db001d,0x63dc001d,0x63dd001d,0x63de001d,0x63df001d,0x63e0001d,
	0x63e1001d,0x63e2001d,0x63e3001d,0x63e4001d,0x63e5001d,0x63e6001d,0x63e7001d,0x63e8001d,
	0x63e9001d,0x63ea001d,0x63eb001d,0x63ec001d,0x63ed001d,0x63ee001d,0x63ef001d,0x63f0001d,
	0x63f1001d,0x63f2001d,0x63f3001d,0x63f4001d,0x63f5001d,0x63f6001d,0x63f7001d,0x63f8001d,
	0x63f9001d,0x63fa001d,0x63fb001d,0x63fc001d,0x63fd001d,0x63fe001d,0x63ff001d,0x6400001d,
	0x6401001d,0x6402001d,0x6403001d,0x6404001d,0x6405001d,0x6406001d,0x6407001d,0x6408001d,
	0x6409001d,0x640a001d,0x640b001d,0x640c001d,0x640d001d,0x640e001d,0x640f001d,0x6410001d,
	0x6411001d,0x6412001d,0x6413001d,0x6414001d,0x6415001d,0x6416001d,0x6417001d,0x6418001d,
	0x6419001d,0x641a001d,0x641b001d,0x641c001d,0x641d001d,0x641e001d,0x641f001d,0x6420001d,
	0x6421001d,0x6422001d,0x6423001d,0x6424001d,0x6425001d,0x6426001d,0x6427001d,0x6428001d,
	0x6429001d,0x642a001d,0x642b001d,0x642c001d,0x642d001d,0x642e001d,0x642f001d,0x6430001d,
	0x6431001d,0x6432001d,0x6433001d,0x6434001d,0x6435001d,0x6436001d,0x6437001d,0x6438001d,
	0x6439001d,0x643a001d,0x643b001d,0x643c001d,0x643d001d,0x643e001d,0x643f001d,0x6440001d,
	0x6441001d,0x6442001d,0x6443001d,0x6444001d,0x6445001d,0x6446001d,0x6447001d,0x6448001d,
	0x6449001d,0x644a001d,0x644b001d,0x644c001d,0x644d001d,0x644e001d,0x644f001d,0x6450001d,
	0x6451001d,0x6452001d,0x6453001d,0x6454001d,0x6455001d,0x6456001d,0x6457001d,0x6458001d,
	0x6459001d,0x645a001d,0x645b001d,0x645c001d,0x645d001d,0x645e001d,0x645f001d,0x6460001d,
	0x6461001d,0x6462001d,0x6463001d,0x6464001d,0x6465001d,0x6466001d,0x6467001d,0x6468001d,
	0x6469001d,0x646a001d,0x646b001d,0x646c001d,0x646d001d,0x646e001d,0x646f001d,0x6470001d,
	0x6471001d,0x6472001d,0x6473001d,0x6474001d,0x6475001d,0x6476001d,0x6477001d,0x6478001d,
	0x6479001d,0x647a001d,0x647b001d,0x647c001d,0x647d001d,0x647e001d,0x647f001d,0x6480001d,
	0x6481001d,0x6482001d,0x6483001d,0x6484001d,0x6485001d,0x6486001d,0x6487001d,0x6488001d,
	0x6489001d,0x648a001d,0x648b001d,0x648c001d,0x648d001d,0x648e001d,0x648f001d,0x6490001d,
	0x6491001d,0x6492001d,0x6493001d,0x6494001d,0x6495001d,0x6496001d,0x6497001d,0x6498001d,
	0x6499001d,0x649a001d,0x649b001d,0x649c001d,0x649d001d,0x649e001d,0x649f001d,0x64a0001d,
	0x64a1001d,0x64a2001d,0x64a3001d,0x64a4001d,0x64a5001d,0x64a6001d,0x64a7001d,0x64a8001d,
	0x64a9001d,0x64aa001d,0x64ab001d,0x64ac001d,0x64ad001d,0x64ae001d,0x64af001d,0x64b0001d,
	0x64b1001d,0x64b2001d,0x64b3001d,0x64b4001d,0x64b5001d,0x64b6001d,0x64b7001d,0x64b8001d,
	0x64b9001d,0x64ba001d,0x64bb001d,0x64bc001d,0x64bd001d,0x64be001d,0x64bf001d,0x64c0001d,
	0x64c1001d,0x64c2001d,0x64c3001d,0x64c4001d,0x64c5001d,0x64c6001d,0x64c7001d,0x64c8001d,
	0x64c9001d,0x64ca001d,0x64cb001d,0x64cc001d,0x64cd001d,0x64ce001d,0x64cf001d,0x64d0001d,
	0x64d1001d,0x64d2001d,0x64d3001d,0x64d4001d,0x64d5001d,0x64d6001d,0x64d7001d,0x64d8001d,
	0x64d9001d,0x64da001d,0x64db001d,0x64dc001d,0x64dd001d,0x64de001d,0x64df001d,0x64e0001d,
	0x64e1001d,0x64e2001d,0x64e3001d,0x64e4001d,0x64e5001d,0x64e6001d,0x64e7001d,0x64e8001d,
	0x64e9001d,0x64ea001d,0x64eb001d,0x64ec001d,0x64ed001d,0x64ee001d,0x64ef001d,0x64f0001d,
	0x64f1001d,0x64f2001d,0x64f3001d,0x64f4001d,0x64f5001d,0x64f6001d,0x64f7001d,0x64f8001d,
	0x64f9001d,0x64fa001d,0x64fb001d,0x64fc001d,0x64fd001d,0x64fe001d,0x64ff001d,0x6500001d,
	0x6501001d,0x6502001d,0x6503001d,0x6504001d,0x6505001d,0x6506001d,0x6507001d,0x6508001d,
	0x6509001d,0x650a001d,0x650b001d,0x650c001d,0x650d001d,0x650e001d,0x650f001d,0x6510001d,
	0x6511001d,0x6512001d,0x6513001d,0x6514001d,0x6515001d,0x6516001d,0x6517001d,0x6518001d,
	0x6519001d,0x651a001d,0x651b001d,0x651c001d,0x651d001d,0x651e001d,0x651f001d,0x6520001d,
	0x6521001d,0x6522001d,0x6523001d,0x6524001d,0x6525001d,0x6526001d,0x6527001d,0x6528001d,
	0x6529001d,0x652a001d,0x652b001d,0x652c001d,0x652d001d,0x652e001d,0x652f001d,0x6530001d,
	0x6531001d,0x6532001d,0x6533001d,0x6534001d,0x6535001d,0x6536001d,0x6537001d,0x6538001d,
	0x6539001d,0x653a001d,0x653b001d,0x653c001d,0x653d001d,0x653e001d,0x653f001d,0x6540001d,
	0x6541001d,0x6542001d,0x6543001d,0x6544001d,0x6545001d,0x6546001d,0x6547001d,0x6548001d,
	0x6549001d,0x654a001d,0x654b001d,0x654c001d,0x654d001d,0x654e001d,0x654f001d,0x6550001d,
	0x6551001d,0x6552001d,0x6553001d,0x6554001d,0x6555001d,0x6556001d,0x6557001d,0x6558001d,
	0x6559001d,0x655a001d,0x655b001d,0x655c001d,0x655d001d,0x655e001d,0x655f001d,0x6560001d,
	0x6561001d,0x6562001d,0x6563001d,0x6564001d,0x6565001d,0x6566001d,0x6567001d,0x6568001d,
	0x6569001d,0x656a001d,0x656b001d,0x656c001d,0x656d001d,0x656e001d,0x656f001d,0x6570001d,
	0x6571001d,0x6572001d,0x6573001d,0x6574001d,0x6575001d,0x6576001d,0x6577001d,0x6578001d,
	0x6579001d,0x657a001d,0x657b001d,0x657c001d,0x657d001d,0x657e001d,0x657f001d,0x6580001d,
	0x6581001d,0x6582001d,0x6583001d,0x6584001d,0x6585001d,0x6586001d,0x6587001d,0x6588001d,
	0x6589001d,0x658a001d,0x658b001d,0x658c001d,0x658d001d,0x658e001d,0x658f001d,0x6590001d,
	0x6591001d,0x6592001d,0x6593001d,0x6594001d,0x6595001d,0x6596001d,0x6597001d,0x6598001d,
	0x6599001d,0x659a001d,0x659b001d,0x659c001d,0x659d001d,0x659e001d,0x659f001d,0x65a0001d,
	0x65a1001d,0x65a2001d,0x65a3001d,0x65a4001d,0x65a5001d,0x65a6001d,0x65a7001d,0x65a8001d,
	0x65a9001d,0x65aa001d,0x65ab001d,0x65ac001d,0x65ad001d,0x65ae001d,0x65af001d,0x65b0001d,
	0x65b1001d,0x65b2001d,0x65b3001d,0x65b4001d,0x65b5001d,0x65b6001d,0x65b7001d,0x65b8001d,
	0x65b9001d,0x65ba001d,0x65bb001d,0x65bc001d,0x65bd001d,0x65be001d,0x65bf001d,0x65c0001d,
	0x65c1001d,0x65c2001d,0x65c3001d,0x65c4001d,0x65c5001d,0x65c6001d,0x65c7001d,0x65c8001d,
	0x65c9001d,0x65ca001d,0x65cb001d,0x65cc001d,0x65cd001d,0x65ce001d,0x65cf001d,0x65d0001d,
	0x65d1001d,0x65d2001d,0x65d3001d,0x65d4001d,0x65d5001d,0x65d6001d,0x65d7001d,0x65d8001d,
	0x65d9001d,0x65da001d,0x65db001d,0x65dc001d,0x65dd001d,0x65de001d,0x65df001d,0x65e0001d,
	0x65e1001d,0x65e2001d,0x65e3001d,0x65e4001d,0x65e5001d,0x65e6001d,0x65e7001d,0x65e8001d,
	0x65e9001d,0x65ea001d,0x65eb001d,0x65ec001d,0x65ed001d,0x65ee001d,0x65ef001d,0x65f0001d,
	0x65f1001d,0x65f2001d,0x65f3001d,0x65f4001d,0x65f5001d,0x65f6001d,0x65f7001d,0x65f8001d,
	0x65f9001d,0x65fa001d,0x65fb001d,0x65fc001d,0x65fd001d,0x65fe001d,0x65ff001d,0x6600001d,
	0x6601001d,0x6602001d,0x6603001d,0x6604001d,0x6605001d,0x6606001d,0x6607001d,0x6608001d,
	0x6609001d,0x660a001d,0x660b001d,0x660c001d,0x660d001d,0x660e001d,0x660f001d,0x6610001d,
	0x6611001d,0x6612001d,0x6613001d,0x6614001d,0x6615001d,0x6616001d,0x6617001d,0x6618001d,
	0x6619001d,0x661a001d,0x661b001d,0x661c001d,0x661d001d,0x661e001d,0x661f001d,0x6620001d,
	0x6621001d,0x6622001d,0x6623001d,0x6624001d,0x6625001d,0x6626001d,0x6627001d,0x6628001d,
	0x6629001d,0x662a001d,0x662b001d,0x662c001d,0x662d001d,0x662e001d,0x662f001d,0x6630001d,
	0x6631001d,0x6632001d,0x6633001d,0x6634001d,0x6635001d,0x6636001d,0x6637001d,0x6638001d,
	0x6639001d,0x663a001d,0x663b001d,0x663c001d,0x663d001d,0x663e001d,0x663f001d,0x6640001d,
	0x6641001d,0x6642001d,0x6643001d,0x6644001d,0x6645001d,0x6646001d,0x6647001d,0x6648001d,
	0x6649001d,0x664a001d,0x664b001d,0x664c001d,0x664d001d,0x664e001d,0x664f001d,0x6650001d,
	0x6651001d,0x6652001d,0x6653001d,0x6654001d,0x6655001d,0x6656001d,0x6657001d,0x6658001d,
	0x6659001d,0x665a001d,0x665b001d,0x665c001d,0x665d001d,0x665e001d,0x665f001d,0x6660001d,
	0x6661001d,0x6662001d,0x6663001d,0x6664001d,0x6665001d,0x6666001d,0x6667001d,0x6668001d,
	0x6669001d,0x666a001d,0x666b001d,0x666c001d,0x666d001d,0x666e001d,0x666f001d,0x6670001d,
	0x6671001d,0x6672001d,0x6673001d,0x6674001d,0x6675001d,0x6676001d,0x6677001d,0x6678001d,
	0x6679001d,0x667a001d,0x667b001d,0x667c001d,0x667d001d,0x667e001d,0x667f001d,0x6680001d,
	0x6681001d,0x6682001d,0x6683001d,0x6684001d,0x6685001d,0x6686001d,0x6687001d,0x6688001d,
	0x6689001d,0x668a001d,0x668b001d,0x668c001d,0x668d001d,0x668e001d,0x668f001d,0x6690001d,
	0x6691001d,0x6692001d,0x6693001d,0x6694001d,0x6695001d,0x6696001d,0x6697001d,0x6698001d,
	0x6699001d,0x669a001d,0x669b001d,0x669c001d,0x669d001d,0x669e001d,0x669f001d,0x66a0001d,
	0x66a1001d,0x66a2001d,0x66a3001d,0x66a4001d,0x66a5001d,0x66a6001d,0x66a7001d,0x66a8001d,
	0x66a9001d,0x66aa001d,0x66ab001d,0x66ac001d,0x66ad001d,0x66ae001d,0x66af001d,0x66b0001d,
	0x66b1001d,0x66b2001d,0x66b3001d,0x66b4001d,0x66b5001d,0x66b6001d,0x66b7001d,0x66b8001d,
	0x66b9001d,0x66ba001d,0x66bb001d,0x66bc001d,0x66bd001d,0x66be001d,0x66bf001d,0x66c0001d,
	0x66c1001d,0x66c2001d,0x66c3001d,0x66c4001d,0x66c5001d,0x66c6001d,0x66c7001d,0x66c8001d,
	0x66c9001d,0x66ca001d,0x66cb001d,0x66cc001d,0x66cd001d,0x66ce001d,0x66cf001d,0x66d0001d,
	0x66d1001d,0x66d2001d,0x66d3001d,0x66d4001d,0x66d5001d,0x66d6001d,0x66d7001d,0x66d8001d,
	0x66d9001d,0x66da001d,0x66db001d,0x66dc001d,0x66dd001d,0x66de001d,0x66df001d,0x66e0001d,
	0x66e1001d,0x66e2001d,0x66e3001d,0x66e4001d,0x66e5001d,0x66e6001d,0x66e7001d,0x66e8001d,
	0x66e9001d,0x66ea001d,0x66eb001d,0x66ec001d,0x66ed001d,0x66ee001d,0x66ef001d,0x66f0001d,
	0x66f1001d,0x66f2001d,0x66f3001d,0x66f4001d,0x66f5001d,0x66f6001d,0x66f7001d,0x66f8001d,
	0x66f9001d,0x66fa001d,0x66fb001d,0x66fc001d,0x66fd001d,0x66fe001d,0x66ff001d,0x6700001d,
	0x6701001d,0x6702001d,0x6703001d,0x6704001d,0x6705001d,0x6706001d,0x6707001d,0x6708001d,
	0x6709001d,0x670a001d,0x670b001d,0x670c001d,0x670d001d,0x670e001d,0x670f001d,0x6710001d,
	0x6711001d,0x6712001d,0x6713001d,0x6714001d,0x6715001d,0x6716001d,0x6717001d,0x6718001d,
	0x6719001d,0x671a001d,0x671b001d,0x671c001d,0x671d001d,0x671e001d,0x671f001d,0x6720001d,
	0x6721001d,0x6722001d,0x6723001d,0x6724001d,0x6725001d,0x6726001d,0x6727001d,0x6728001d,
	0x6729001d,0x672a001d,0x672b001d,0x672c001d,0x672d001d,0x672e001d,0x672f001d,0x6730001d,
	0x6731001d,0x6732001d,0x6733001d,0x6734001d,0x6735001d,0x6736001d,0x6737001d,0x6738001d,
	0x6739001d,0x673a001d,0x673b001d,0x673c001d,0x673d001d,0x673e001d,0x673f001d,0x6740001d,
	0x6741001d,0x6742001d,0x6743001d,0x6744001d,0x6745001d,0x6746001d,0x6747001d,0x6748001d,
	0x6749001d,0x674a001d,0x674b001d,0x674c001d,0x674d001d,0x674e001d,0x674f001d,0x6750001d,
	0x6751001d,0x6752001d,0x6753001d,0x6754001d,0x6755001d,0x6756001d,0x6757001d,0x6758001d,
	0x6759001d,0x675a001d,0x675b001d,0x675c001d,0x675d001d,0x675e001d,0x675f001d,0x6760001d,
	0x6761001d,0x6762001d,0x6763001d,0x6764001d,0x6765001d,0x6766001d,0x6767001d,0x6768001d,
	0x6769001d,0x676a001d,0x676b001d,0x676c001d,0x676d001d,0x676e001d,0x676f001d,0x6770001d,
	0x6771001d,0x6772001d,0x6773001d,0x6774001d,0x6775001d,0x6776001d,0x6777001d,0x6778001d,
	0x6779001d,0x677a001d,0x677b001d,0x677c001d,0x677d001d,0x677e001d,0x677f001d,0x6780001d,
	0x6781001d,0x6782001d,0x6783001d,0x6784001d,0x6785001d,0x6786001d,0x6787001d,0x6788001d,
	0x6789001d,0x678a001d,0x678b001d,0x678c001d,0x678d001d,0x678e001d,0x678f001d,0x6790001d,
	0x6791001d,0x6792001d,0x6793001d,0x6794001d,0x6795001d,0x6796001d,0x6797001d,0x6798001d,
	0x6799001d,0x679a001d,0x679b001d,0x679c001d,0x679d001d,0x679e001d,0x679f001d,0x67a0001d,
	0x67a1001d,0x67a2001d,0x67a3001d,0x67a4001d,0x67a5001d,0x67a6001d,0x67a7001d,0x67a8001d,
	0x67a9001d,0x67aa001d,0x67ab001d,0x67ac001d,0x67ad001d,0x67ae001d,0x67af001d,0x67b0001d,
	0x67b1001d,0x67b2001d,0x67b3001d,0x67b4001d,0x67b5001d,0x67b6001d,0x67b7001d,0x67b8001d,
	0x67b9001d,0x67ba001d,0x67bb001d,0x67bc001d,0x67bd001d,0x67be001d,0x67bf001d,0x67c0001d,
	0x67c1001d,0x67c2001d,0x67c3001d,0x67c4001d,0x67c5001d,0x67c6001d,0x67c7001d,0x67c8001d,
	0x67c9001d,0x67ca001d,0x67cb001d,0x67cc001d,0x67cd001d,0x67ce001d,0x67cf001d,0x67d0001d,
	0x67d1001d,0x67d2001d,0x67d3001d,0x67d4001d,0x67d5001d,0x67d6001d,0x67d7001d,0x67d8001d,
	0x67d9001d,0x67da001d,0x67db001d,0x67dc001d,0x67dd001d,0x67de001d,0x67df001d,0x67e0001d,
	0x67e1001d,0x67e2001d,0x67e3001d,0x67e4001d,0x67e5001d,0x67e6001d,0x67e7001d,0x67e8001d,
	0x67e9001d,0x67ea001d,0x67eb001d,0x67ec001d,0x67ed001d,0x67ee001d,0x67ef001d,0x67f0001d,
	0x67f1001d,0x67f2001d,0x67f3001d,0x67f4001d,0x67f5001d,0x67f6001d,0x67f7001d,0x67f8001d,
	0x67f9001d,0x67fa001d,0x67fb001d,0x67fc001d,0x67fd001d,0x67fe001d,0x67ff001d,0x6800001d,
	0x6801001d,0x6802001d,0x6803001d,0x6804001d,0x6805001d,0x6806001d,0x6807001d,0x6808001d,
	0x6809001d,0x680a001d,0x680b001d,0x680c001d,0x680d001d,0x680e001d,0x680f001d,0x6810001d,
	0x6811001d,0x6812001d,0x6813001d,0x6814001d,0x6815001d,0x6816001d,0x6817001d,0x6818001d,
	0x6819001d,0x681a001d,0x681b001d,0x681c001d,0x681d001d,0x681e001d,0x681f001d,0x6820001d,
	0x6821001d,0x6822001d,0x6823001d,0x6824001d,0x6825001d,0x6826001d,0x6827001d,0x6828001d,
	0x6829001d,0x682a001d,0x682b001d,0x682c001d,0x682d001d,0x682e001d,0x682f001d,0x6830001d,
	0x6831001d,0x6832001d,0x6833001d,0x6834001d,0x6835001d,0x6836001d,0x6837001d,0x6838001d,
	0x6839001d,0x683a001d,0x683b001d,0x683c001d,0x683d001d,0x683e001d,0x683f001d,0x6840001d,
	0x6841001d,0x6842001d,0x6843001d,0x6844001d,0x6845001d,0x6846001d,0x6847001d,0x6848001d,
	0x6849001d,0x684a001d,0x684b001d,0x684c001d,0x684d001d,0x684e001d,0x684f001d,0x6850001d,
	0x6851001d,0x6852001d,0x6853001d,0x6854001d,0x6855001d,0x6856001d,0x6857001d,0x6858001d,
	0x6859001d,0x685a001d,0x685b001d,0x685c001d,0x685d001d,0x685e001d,0x685f001d,0x6860001d,
	0x6861001d,0x6862001d,0x6863001d,0x6864001d,0x6865001d,0x6866001d,0x6867001d,0x6868001d,
	0x6869001d,0x686a001d,0x686b001d,0x21e010f,0x234010f,0x269010f,0x2c4010d,0x26a010f,
	0x268010f,0x22d010f,0x23e010f,0x23f010f,0x263010f,0x369010f,0x214010f,0x20c010f,
	0x226010f,0x265010f,0x6c5010d,0x6c6010d,0x6c7010d,0x6c8010d,0x6c9010d,0x6ca010d,
	0x6cb010d,0x6cc010d,0x6cd010d,0x6ce010d,0x21c010f,0x21a010f,0x36d010f,0x36e010f,
	0x36f010f,0x221010f,0x262010f,0x6cf0125,0x6e30125,0x6f70125,0x7030125,0x71f0125,
	0x74b0125,0x7530125,0x7730125,0x78d0125,0x7a10125,0x7b10125,0x7bd0125,0x7e50125,
	0x7ed0125,0x8050125,0x81d0125,0x8290125,0x8350125,0x85d0125,0x8750125,0x88d0125,
	0x8a50125,0x8b10125,0x8b90125,0x8bd0125,0x8c90125,0x240010f,0x241010f,0x2091d0f,
	0x209020f,0x209160f,0x6cf010d,0x6e3010d,0x6f7010d,0x703010d,0x71f010d,0x74b010d,
	0x753010d,0x773010d,0x78d010d,0x7a1010d,0x7b1010d,0x7bd010d,0x7e5010d,0x7ed010d,
	0x805010d,0x81d010d,0x829010d,0x835010d,0x85d010d,0x875010d,0x88d010d,0x8a5010d,
	0x8b1010d,0x8b9010d,0x8bd010d,0x8c9010d,0x242010f,0x371010f,0x243010f,0x374010f,
	0x228010b,0x219010b,0x22e010b,0x22f010b,0x235010b,
	};

static const TUint32 TheChineseIndex[] = 
	{
	0x20181aca,0x20191acb,0x201c1acc,0x30011ac9,0x30021ac8,0x4e0016ba,0x4e0103eb,0x4e030f2d,
	0x4e0713de,0x4e0818d0,0x4e0910e0,0x4e0a113a,0x4e0b14db,0x4e0c0f3c,0x4e0d0176,0x4e0e17d4,
	0x4e100575,0x4e110284,0x4e1319d8,0x4e140fd3,0x4e150ea7,0x4e1611b0,0x4e181014,0x4e190143,
	0x4e1a16ad,0x4e1b02ea,0x4e1c03ff,0x4e1d122e,0x4e1e0232,0x4e2203fd,0x4e240b87,0x4e25163c,
	0x4e2710e8,0x4e28067b,0x4e2a05cd,0x4e2b160f,0x4e2c0e65,0x4e2d197e,0x4e3004fe,0x4e3202af,
	0x4e340bb1,0x4e3619bb,0x4e3813c9,0x4e390348,0x4e3a13fb,0x4e3b19bc,0x4e3d0b3f,0x4e3e098c,
	0x4e3f0ee9,0x4e430dab,0x4e450968,0x4e47139f,0x4e480ccf,0x4e4916ea,0x4e4b1930,0x4e4c144e,
	0x4e4d189f,0x4e4e071f,0x4e4f049b,0x4e500af8,0x4e520ef6,0x4e530e6e,0x4e540fbe,0x4e560642,
	0x4e580239,0x4e5916df,0x4e5c0d41,0x4e5d0967,0x4e5e0f5b,0x4e5f16aa,0x4e6014b3,0x4e61150d,
	0x4e6611dd,0x4e6907db,0x4e700c92,0x4e710c49,0x4e7310bb,0x4e7e0f8f,0x4e860afd,0x4e8817b4,
	0x4e89191c,0x4e8b11b5,0x4e8c0498,0x4e8d029b,0x4e8e17b3,0x4e8f0a7e,0x4e911836,0x4e92073d,
	0x4e930f3d,0x4e941460,0x4e950941,0x4e9805d6,0x4e9a1623,0x4e9b1547,0x4e9f0801,0x4ea01377,
	0x4ea113e1,0x4ea20a08,0x4ea408ba,0x4ea5069c,0x4ea616f1,0x4ea701e1,0x4ea806f7,0x4ea90d8c,
	0x4eab151d,0x4eac0932,0x4ead1355,0x4eae0b89,0x4eb20fdd,0x4eb3015b,0x4eb5155e,0x4eba1086,
	0x4ebb1087,0x4ebf16eb,0x4ec0116f,0x4ec11088,0x4ec20af7,0x4ec303ec,0x4ec41883,0x4ec5091b,
	0x4ec60f14,0x4ec70278,0x4ec918cd,0x4eca0910,0x4ecb0904,0x4ecd1098,0x4ece02e9,0x4ed10c4e,
	0x4ed30198,0x4ed41a0c,0x4ed511b1,0x4ed612ae,0x4ed718d1,0x4ed80554,0x4ed914df,0x4edd1363,
	0x4ede108f,0x4edf0f78,0x4ee105bd,0x4ee30337,0x4ee40bdc,0x4ee516e1,0x4ee810d4,0x4eea16c7,
	0x4eeb0d91,0x4eec0cee,0x4ef0167d,0x4ef2198b,0x4ef30ec3,0x4ef51462,0x4ef6088e,0x4ef7085b,
	0x4efb1090,0x4efd04f6,0x4eff04c8,0x4f010f5c,0x4f090a09,0x4f0a16bc,0x4f0d1463,0x4f0e081d,
	0x4f0f0520,0x4f10049c,0x4f11159b,0x4f17198c,0x4f181784,0x4f1907c9,0x4f1a07a6,0x4f1b17d5,
	0x4f1e10e3,0x4f1f140b,0x4f2002a8,0x4f221617,0x4f241131,0x4f2501eb,0x4f260c4f,0x4f270199,
	0x4f2a140c,0x4f2b19c3,0x4f2f0156,0x4f30060d,0x4f320de1,0x4f340078,0x4f360bc6,0x4f381164,
	0x4f3a1241,0x4f3c1242,0x4f3d083c,0x4f4303c0,0x4f460358,0x4f4d141d,0x4f4e0390,0x4f4f19c4,
	0x4f501a60,0x4f5117a5,0x4f531325,0x4f5506dc,0x4f5713a5,0x4f581155,0x4f5917b5,0x4f5a16f4,
	0x4f5b0513,0x4f5c1a61,0x4f5d05f9,0x4f5e0e16,0x4f5f1365,0x4f600de2,0x4f63176b,0x4f6413be,
	0x4f650f7d,0x4f6709e6,0x4f690e8c,0x4f6c0aee,0x4f6f1677,0x4f700062,0x4f73083f,0x4f740497,
	0x4f760802,0x4f7b133d,0x4f7c08cd,0x4f7e16fa,0x4f7f11a8,0x4f8309fd,0x4f841942,0x4f88025b,
	0x4f890a5e,0x4f8b0b46,0x4f8d11b6,0x4f8f19a6,0x4f9117a6,0x4f940d85,0x4f970409,0x4f9b05e9,
	0x4f9d16bf,0x4fa014cf,0x4fa30c72,0x4fa508ce,0x4fa618fe,0x4fa701a8,0x4fa80fbf,0x4fa90a65,
	0x4faa01ca,0x4fac0e21,0x4fae1469,0x4faf0710,0x4fb50fde,0x4fbf0115,0x4fc302f4,0x4fc4046c,
	0x4fc51020,0x4fca09db,0x4fce1a4c,0x4fcf0fcb,0x4fd00b4b,0x4fd1177d,0x4fd7126e,0x4fd8052b,
	0x4fda0b2f,0x4fdc0ef7,0x4fdd0096,0x4fde17bb,0x4fdf0f5a,0x4fe1157b,0x4fe317da,0x4fe60279,
	0x4fe8164d,0x4fe90b63,0x4fea0b4c,0x4fed0879,0x4fee159c,0x4fef0549,0x4ff1099b,0x4ff30e56,
	0x4ff80512,0x4ffa0020,0x4ffe00d7,0x500c0649,0x500d00b1,0x500f11e4,0x50120374,0x501409c2,
	0x501812f7,0x5019071c,0x501a16e7,0x501c1328,0x501f090b,0x50210202,0x50250a41,0x502609b2,
	0x5028099c,0x50290f9d,0x502a0ddc,0x502c19f9,0x502d143f,0x502e0c60,0x503a18ab,0x503c1944,
	0x503e0ff6,0x5043164f,0x5047085a,0x5048082c,0x504c10d1,0x504e13f1,0x504f0ed1,0x5055154f,
	0x505a1a6a,0x505c1358,0x50650895,0x506c1a37,0x50760e46,0x50771376,0x507b0c08,0x507e04f9,
	0x507f01f6,0x50800a8f,0x5085055e,0x50880b5c,0x508d0085,0x50a30336,0x50a512f9,0x50a70132,
	0x50a80296,0x50a90e32,0x50ac0304,0x50b20040,0x50ba0269,0x50bb110a,0x50cf1527,0x50d614a4,
	0x50da0b92,0x50e60977,0x50e710fd,0x50ec08c6,0x50ed08a1,0x50ee136d,0x50f3127c,0x50f508aa,
	0x50fb0ece,0x51060947,0x510715d3,0x510b0351,0x511210b2,0x51210b0b,0x513f048c,0x51401470,
	0x5141183d,0x5143180b,0x51441594,0x5145026b,0x514618e0,0x514814e0,0x51490657,0x514b0a2e,
	0x514d0d26,0x51510442,0x51541389,0x51551243,0x5156164b,0x515a0365,0x515c0413,0x5162093e,
	0x516510bd,0x5168104e,0x516b0045,0x516c05e6,0x516d0bf4,0x516e1485,0x51700ac1,0x517105f6,
	0x51730646,0x5174157d,0x51750142,0x51760f43,0x51770998,0x517803bb,0x51791a0e,0x517b167e,
	0x517c0868,0x517d11d8,0x51800838,0x518101e6,0x5182095a,0x51850dd3,0x51880593,0x51891074,
	0x518c01a7,0x518d1858,0x51920cc6,0x51950d2a,0x51960d16,0x519710a8,0x51991557,0x519b09d3,
	0x519c0e20,0x51a00648,0x51a21989,0x51a41807,0x51a50d5b,0x51ab0140,0x51ac0400,0x51af050b,
	0x51b00141,0x51b1073f,0x51b2026c,0x51b309bc,0x51b50a78,0x51b616ab,0x51b70b14,0x51bb0408,
	0x51bc14f7,0x51bd0ba4,0x51c0094a,0x51c40f31,0x51c619f6,0x51c7124f,0x51c90b7f,0x51cb03d1,
	0x51cc0bce,0x51cf087e,0x51d102ed,0x51db0bbd,0x51dd0e14,0x51e007f8,0x51e104a9,0x51e40510,
	0x51eb0521,0x51ed0efa,0x51ef09ec,0x51f0077e,0x51f3038b,0x51f50a00,0x51f61593,0x51f8137c,
	0x51f9002e,0x51fa0287,0x51fb07d8,0x51fc0367,0x51fd06a8,0x51ff186b,0x5200036d,0x520103cf,
	0x5202036e,0x5203108d,0x520604ea,0x52070fd1,0x520816ed,0x520a09f6,0x520d028a,0x520e1432,
	0x52111583,0x5212074b,0x5216182a,0x52170ba2,0x52180be3,0x52191878,0x521a0594,0x521b02b6,
	0x521d0288,0x52201114,0x52240e67,0x52280e79,0x52290b40,0x522b012e,0x522d0943,0x522e0638,
	0x52300378,0x52330a50,0x5236195e,0x5237120a,0x52381061,0x52391101,0x523a02df,0x523b0a2f,
	0x523d0671,0x523f0672,0x524009ed,0x52410463,0x52420824,0x52431327,0x524a15e5,0x524c0aaf,
	0x524d0f89,0x5250063c,0x52510891,0x52541319,0x52560f11,0x525c13c5,0x525e07e4,0x5261164a,
	0x52650150,0x5267099d,0x5269118c,0x526a087f,0x526f055d,0x527205ba,0x527d0edb,0x527f08d9,
	0x52810fb8,0x528209c8,0x52880eae,0x529007c3,0x52931718,0x529b0b2a,0x529d1060,0x529e0076,
	0x529f05e7,0x52a0083d,0x52a11472,0x52a20c94,0x52a30ba3,0x52a80407,0x52a919c5,0x52aa0e2a,
	0x52ab08f2,0x52ac1039,0x52ad114a,0x52b10b41,0x52b20925,0x52b30ae7,0x52be06dd,0x52bf11b7,
	0x52c3015a,0x52c7177e,0x52c90d28,0x52cb15f2,0x52d00cfc,0x52d20afb,0x52d615c1,0x52d809f7,
	0x52df0d98,0x52e40fe6,0x52f01553,0x52f9008b,0x52fa1145,0x52fe05f8,0x52ff1471,0x53001837,
	0x5305008c,0x530602e2,0x53081595,0x530d0f1a,0x530f0e7f,0x53100535,0x531500d1,0x53160753,
	0x531700aa,0x531911ce,0x531a04bc,0x531d184c,0x532008b3,0x53210a6e,0x532314ce,0x5326066b,
	0x532a04db,0x532e0a91,0x53390ec1,0x533a102b,0x533b16be,0x533e010d,0x533f0de7,0x53411199,
	0x53430f77,0x534510d7,0x5347117e,0x53481461,0x534907a4,0x534a0077,0x534e074d,0x534f154b,
	0x535100a4,0x53521a46,0x535319fd,0x53550349,0x53560c97,0x53570db6,0x535a015f,0x535c0171,
	0x535e0110,0x535f0172,0x536018b6,0x536109e5,0x53620c18,0x5363179b,0x53640c24,0x5366063e,
	0x53671447,0x536908ee,0x536b141b,0x536e1932,0x536f0cc0,0x53701740,0x537113ef,0x537307ff,
	0x53741065,0x53750c48,0x537709b1,0x5378155b,0x537a091c,0x537f0ff7,0x538201fb,0x53840475,
	0x53851350,0x53860b3b,0x53890b3c,0x538b1610,0x538c1659,0x538d1158,0x539501a9,0x53980b17,
	0x539a0719,0x539d031f,0x539f1811,0x53a21511,0x53a31650,0x53a509c7,0x53a6110f,0x53a8028c,
	0x53a90973,0x53ae1237,0x53b6122c,0x53bb1048,0x53bf1500,0x53c110e1,0x53c2018d,0x53c817a2,
	0x53c901b2,0x53ca07fa,0x53cb1799,0x53cc1216,0x53cd04b3,0x53d1049a,0x53d411e1,0x53d61044,
	0x53d711d6,0x53d80116,0x53d915be,0x53db0e69,0x53df1265,0x53e003e5,0x53e30a4a,0x53e4061e,
	0x53e50993,0x53e60be0,0x53e8036f,0x53e90a4b,0x53ea193f,0x53eb08dd,0x53ec18df,0x53ed0047,
	0x53ee03ed,0x53ef0a2b,0x53f012bf,0x53f10261,0x53f211a5,0x53f317a3,0x53f50f09,0x53f616ae,
	0x53f706cf,0x53f8122f,0x53f912e2,0x53fb0af9,0x53fc03d0,0x53fd07d9,0x540115b4,0x54030246,
	0x540405ce,0x54061687,0x540806db,0x540907fb,0x540a03d6,0x540c1364,0x540d0d57,0x540e0718,
	0x540f0b3e,0x54101387,0x54111523,0x5412189d,0x541314dc,0x54150c71,0x5416160e,0x54170c85,
	0x541b09d4,0x541d0bc1,0x541e1398,0x541f172d,0x542004e1,0x542100d3,0x54230ff0,0x54260515,
	0x54270049,0x54280447,0x542904eb,0x542b06a6,0x542c1352,0x542d0a3d,0x542e121f,0x542f0f60,
	0x54311935,0x5432173a,0x54341459,0x5435020e,0x54381488,0x543902b8,0x543b1433,0x543c0717,
	0x543e145a,0x54401611,0x54430476,0x54460333,0x54480234,0x544a05ae,0x544b0517,0x54500daa,
	0x54520543,0x545316f5,0x54540334,0x54550e45,0x54560b42,0x545700bd,0x5458180c,0x545913c1,
	0x545b0fa2,0x545c1452,0x54620dd1,0x54640bdd,0x54661787,0x54681992,0x5471061b,0x54721a0b,
	0x5473141e,0x547506d7,0x54760dc3,0x547714cb,0x54780e83,0x547b1166,0x547c0721,0x547d0d62,
	0x5480098a,0x5482184d,0x54840457,0x54860e7a,0x548b189e,0x548c06de,0x548e096f,0x548f177b,
	0x54900558,0x5492199a,0x549409e4,0x5495060e,0x549609e2,0x54990bf7,0x549a0401,0x549b0e0f,
	0x549d1231,0x54a30658,0x54a418a1,0x54a616cf,0x54a70ba1,0x54a81a0f,0x54a90d42,0x54aa0d05,
	0x54ab1954,0x54ac169a,0x54ad07e3,0x54af09e8,0x54b1185c,0x54b30a2a,0x54b40793,0x54b814ef,
	0x54bb159d,0x54bd162b,0x54bf16c0,0x54c00004,0x54c10ef1,0x54c2116c,0x54c40700,0x54c60458,
	0x54c713b7,0x54c80694,0x54c91853,0x54cc0e5a,0x54cd151e,0x54ce0003,0x54cf05d5,0x54d00a70,
	0x54d11620,0x54d20324,0x54d3152c,0x54d400e1,0x54d51828,0x54d7074e,0x54d90a67,0x54da045f,
	0x54dc0826,0x54dd0e22,0x54de0d82,0x54df1769,0x54e505b6,0x54e60e3f,0x54e70247,0x54e8114d,
	0x54e90b62,0x54ea0da2,0x54ed0a52,0x54ee1542,0x54f218eb,0x54f31890,0x54fa0174,0x54fc06f8,
	0x54fd05de,0x54ff05cb,0x5501165c,0x5506129e,0x550702c4,0x55090005,0x550f148f,0x551012e9,
	0x55111a68,0x55140c82,0x551b0c8d,0x55200af2,0x552212a8,0x55231873,0x5524076d,0x552707e5,
	0x552a050f,0x552c073b,0x552e11d9,0x552f1405,0x5530120b,0x55310204,0x55330b57,0x5537176a,
	0x553c110c,0x553e13b5,0x553f0725,0x5541198e,0x55430a3b,0x55441a05,0x55461133,0x55490ace,
	0x554a0001,0x55500309,0x55551309,0x5556035a,0x555c02cc,0x556104d0,0x55640eba,0x55651109,
	0x55660ab4,0x5567187d,0x556a0e4c,0x556c10f8,0x556d19dc,0x556e0e05,0x5575016c,0x557603f9,
	0x55770ad8,0x55781546,0x557b0268,0x557c131e,0x557e0964,0x558009e3,0x55811778,0x55821424,
	0x55830db8,0x55841127,0x55870aae,0x558808ea,0x55890711,0x558a06af,0x558b03e2,0x558f0e34,
	0x5591172b,0x55940e3e,0x559802ae,0x559907b2,0x559c14be,0x559d06d8,0x559f0a92,0x55a715ce,
	0x55b10b1d,0x55b31891,0x55b50d30,0x55b70e92,0x55b90a86,0x55bb17f6,0x55bd0c09,0x55be0a59,
	0x55c4110b,0x55c515ab,0x55c91278,0x55cc0014,0x55cd129d,0x55d10a28,0x55d20327,0x55d310e9,
	0x55d4021d,0x55d6125d,0x55dc11c9,0x55dd05c5,0x55df08eb,0x55e1143a,0x55e3124b,0x55e4024c,
	0x55e506c8,0x55e612ac,0x55e80695,0x55ea0fe7,0x55eb0e06,0x55ec06d9,0x55ef0d9f,0x55f203b5,
	0x55f3000c,0x55f51362,0x55f70031,0x55fd1269,0x55fe1266,0x56000395,0x56010f38,0x560801a1,
	0x5609084a,0x560c0edf,0x560e056a,0x560f062a,0x561815b1,0x561b0c8f,0x561e0b10,0x561f041f,
	0x562300c8,0x56241747,0x56270d20,0x562c1a5d,0x562d0e99,0x563119c0,0x5632020c,0x56341a53,
	0x56361239,0x56390b94,0x563b14a8,0x563f06f2,0x564c01af,0x564d08e3,0x564e16a5,0x56540385,
	0x56570f19,0x565809b9,0x56590fe9,0x565c0c16,0x56620e3d,0x56640931,0x56680f70,0x56690486,
	0x566a1875,0x566b16c5,0x566c11cc,0x567115e8,0x5676056d,0x567b10dd,0x567c0eaf,0x568510b3,
	0x568606c4,0x568e06ca,0x568f132f,0x5693017f,0x56a3153a,0x56af07d4,0x56b7107a,0x56bc08cb,
	0x56ca0dbe,0x56d40dbd,0x56d713fa,0x56da101b,0x56db123e,0x56dd0876,0x56de079c,0x56df157a,
	0x56e01721,0x56e10db4,0x56e2138d,0x56e4044e,0x56eb0729,0x56ed180d,0x56f00aa4,0x56f102e1,
	0x56f413fe,0x56f50c50,0x56f90bc8,0x56fa062d,0x56fd068a,0x56fe137f,0x56ff17a7,0x57030f23,
	0x570417dd,0x57061812,0x5708104d,0x570917de,0x570a0ff8,0x571c0768,0x571f1386,0x57231189,
	0x57281859,0x572913fd,0x572a05b3,0x572c144f,0x572d065f,0x572e0ec4,0x572f16c8,0x573003a9,
	0x57331914,0x57390a76,0x573a01fa,0x573b0f40,0x573e07dc,0x57401950,0x57420071,0x574709d5,
	0x574a04bf,0x574c00c4,0x574d12ce,0x574e09fc,0x574f075c,0x57501a62,0x57510a3e,0x57570a63,
	0x575a0863,0x575b12d4,0x575c0b43,0x575d0058,0x575e146e,0x575f04f0,0x576019ee,0x57610f03,
	0x57640a9a,0x576612de,0x576813a7,0x5769057f,0x576a0efb,0x576b03c3,0x576d0dd9,0x576f0eab,
	0x5773003e,0x57760d94,0x57770a17,0x577b03a3,0x577c0216,0x578202ba,0x57830aaa,0x57840c04,
	0x57850c05,0x57860c1b,0x578b1588,0x578c040a,0x57920b08,0x57930572,0x579b0460,0x57a0172e,
	0x57a1049d,0x57a20607,0x57a3180f,0x57a403df,0x57a60a39,0x57a71137,0x57a9047a,0x57ab03c5,
	0x57ad1626,0x57ae0a5d,0x57b209ee,0x57b40dc9,0x57b81821,0x57c205df,0x57c30006,0x57cb0c90,
	0x57ce0238,0x57cf1638,0x57d20ba6,0x57d40f24,0x57d5023a,0x57d811a3,0x57d915f3,0x57da0683,
	0x57dd0df7,0x57df17f2,0x57e0017c,0x57e40ebb,0x57ed0342,0x57ef0021,0x57f41945,0x57f8170a,
	0x57f90e87,0x57fa07ea,0x57fd10f4,0x58000a53,0x580212ea,0x5806043f,0x58070919,0x580b0e9b,
	0x580d138a,0x58110f9e,0x58150464,0x5819172c,0x581e03e3,0x5820071d,0x58210098,0x58240392,
	0x582a09f9,0x58301662,0x5835042b,0x58440b11,0x584c12b2,0x584d023e,0x58511279,0x585412b4,
	0x585812ec,0x585e10db,0x586505c6,0x586b1336,0x586c1817,0x587e11f2,0x58800257,0x58810ca3,
	0x58830956,0x58851207,0x58891771,0x58921135,0x58930d99,0x58990fad,0x589a0b84,0x589e1889,
	0x589f15b3,0x58a80d7c,0x58a90449,0x58bc07f4,0x58c100f7,0x58c51773,0x58d106f0,0x58d506cb,
	0x58e4107b,0x58eb11ad,0x58ec1089,0x58ee19e6,0x58f01180,0x58f30a29,0x58f6072d,0x58f916c2,
	0x5902194d,0x5904029a,0x590700ae,0x590d055b,0x590f14dd,0x59140a8d,0x59151484,0x591613c3,
	0x5919126f,0x591a0456,0x591c16b2,0x591f0608,0x59241734,0x592507cb,0x59270332,0x59291330,
	0x592a12c7,0x592b0516,0x592d1686,0x592e166c,0x592f06bd,0x5931118f,0x59341378,0x593716c9,
	0x59380a5c,0x5939083e,0x593a045b,0x593c0a74,0x59410b64,0x5942076b,0x5944164c,0x59470f44,
	0x59480daf,0x59490511,0x594b04f7,0x594e0a83,0x594f1a41,0x59510f6c,0x595400be,0x59551701,
	0x595608af,0x5957130e,0x595819e5,0x595a1490,0x596003ca,0x5962114f,0x59650041,0x59730e39,
	0x59740e27,0x59760dac,0x59780861,0x597912ad,0x597d06cd,0x59811224,0x598210af,0x598304ce,
	0x598413ea,0x598619e1,0x59870555,0x59880c83,0x598a1092,0x598d163d,0x5992042f,0x5993081f,
	0x59961688,0x59970926,0x59990d3d,0x599e0e18,0x59a300d4,0x59a417b6,0x59a513b0,0x59a804c4,
	0x59a91464,0x59aa17e8,0x59ab0660,0x59ae0dd7,0x59af1995,0x59b2032a,0x59b90ce5,0x59bb0f2f,
	0x59be0fd5,0x59c60d8e,0x59ca1a25,0x59cb11a9,0x59d00902,0x59d1060f,0x59d21244,0x59d3158e,
	0x59d4140f,0x59d71117,0x59d80eeb,0x59da168e,0x59dc08a5,0x59dd11e3,0x59e308bc,0x59e50aef,
	0x59e816d0,0x59ec07e6,0x59f901c5,0x59fb1723,0x59ff1a10,0x5a0113f0,0x5a0313bc,0x5a040c07,
	0x5a051627,0x5a061081,0x5a0708bd,0x5a080c3f,0x5a090ef5,0x5a0c0b30,0x5a11129f,0x5a131413,
	0x5a180df8,0x5a1c0da6,0x5a1f09aa,0x5a201169,0x5a2303ac,0x5a25046d,0x5a290d29,0x5a3117bf,
	0x5a3213ba,0x5a3414f1,0x5a361045,0x5a3c01ed,0x5a40046a,0x5a460f06,0x5a4913d4,0x5a4a0128,
	0x5a5508f8,0x5a5a07ba,0x5a6200e8,0x5a670951,0x5a6a0ac5,0x5a741745,0x5a7501d5,0x5a761174,
	0x5a771359,0x5a7a147c,0x5a7f15c5,0x5a920cd7,0x5a9a0ce8,0x5a9b1822,0x5aaa003c,0x5ab20ecd,
	0x5ab314b7,0x5ab51768,0x5ab8024d,0x5abe0609,0x5ac1085e,0x5ac210f3,0x5ac907f7,0x5acc14f6,
	0x5ad20015,0x5ad40eee,0x5ad60ee1,0x5ad80aff,0x5adc18c6,0x5ae00b22,0x5ae1039f,0x5ae31637,
	0x5ae601f9,0x5ae90dd4,0x5aeb0d67,0x5af10fae,0x5b0914a9,0x5b1600f8,0x5b17112b,0x5b320dfd,
	0x5b34175d,0x5b370d65,0x5b401218,0x5b501a24,0x5b5108ef,0x5b5309bb,0x5b540a44,0x5b551841,
	0x5b571a2c,0x5b580313,0x5b591295,0x5b5a0522,0x5b5b00bc,0x5b5c1a0d,0x5b5d1541,0x5b5f0d03,
	0x5b62008d,0x5b630825,0x5b640610,0x5b650e28,0x5b6615ea,0x5b690696,0x5b6a0c40,0x5b6c0dc2,
	0x5b7011f0,0x5b7101d7,0x5b731a16,0x5b75051d,0x5b7a10b4,0x5b7d0e0b,0x5b800d22,0x5b810e0e,
	0x5b8312af,0x5b840668,0x5b8518a9,0x5b8717d6,0x5b8811d2,0x5b890019,0x5b8b1259,0x5b8c13cc,
	0x5b8f0706,0x5b930d19,0x5b950368,0x5b971a30,0x5b980656,0x5b99199b,0x5b9a03f8,0x5b9b13d0,
	0x5b9c16cc,0x5b9d0094,0x5b9e119e,0x5ba00274,0x5ba11171,0x5ba20a30,0x5ba315cc,0x5ba411bb,
	0x5ba517a8,0x5ba6076c,0x5baa1506,0x5bab05eb,0x5bb01855,0x5bb3069e,0x5bb4165d,0x5bb5152f,
	0x5bb60844,0x5bb80224,0x5bb910a0,0x5bbd0a6b,0x5bbe0130,0x5bbf1275,0x5bc2082d,0x5bc4082e,
	0x5bc51730,0x5bc60d1d,0x5bc70a4d,0x5bcc055f,0x5bd00ce9,0x5bd206ac,0x5bd317f7,0x5bdd0fef,
	0x5bde0d78,0x5bdf01bd,0x5be1063d,0x5be41480,0x5be50b93,0x5be818ad,0x5bee0b95,0x5bf00765,
	0x5bf80315,0x5bf90441,0x5bfa123f,0x5bfb15f9,0x5bfc0372,0x5bff11d5,0x5c010502,0x5c04115b,
	0x5c0608a6,0x5c091422,0x5c0a1a58,0x5c0f153e,0x5c111149,0x5c140490,0x5c15056e,0x5c160862,
	0x5c18021e,0x5c1a113b,0x5c1c056c,0x5c1d01f5,0x5c22178a,0x5c24178b,0x5c250b9e,0x5c27168c,
	0x5c2c056f,0x5c310975,0x5c340586,0x5c38118e,0x5c391738,0x5c3a025a,0x5c3b0a0e,0x5c3c0dd8,
	0x5c3d0924,0x5c3e13f9,0x5c3f0dfe,0x5c400987,0x5c410ecb,0x5c4201b0,0x5c450979,0x5c481030,
	0x5c491326,0x5c4a0907,0x5c4b1454,0x5c4e11ab,0x5c4f0efd,0x5c5007e7,0x5c51155c,0x5c5518b8,
	0x5c590469,0x5c5e11fb,0x5c601384,0x5c610c76,0x5c6314c0,0x5c650c7a,0x5c6609a6,0x5c6e0214,
	0x5c6f139a,0x5c711112,0x5c7916f2,0x5c7a0f5d,0x5c7f17d7,0x5c81128c,0x5c820f5e,0x5c881618,
	0x5c8c07fc,0x5c8d0f7e,0x5c900f41,0x5c9101ad,0x5c9401c3,0x5c96102d,0x5c97059a,0x5c981501,
	0x5c99003f,0x5c9a0ac2,0x5c9b0373,0x5c9c004a,0x5ca20a2c,0x5ca305ff,0x5ca91640,0x5cab15a6,
	0x5cac0854,0x5cad0bde,0x5cb10338,0x5cb3182b,0x5cb50742,0x5cb70d48,0x5cb80025,0x5cbd0402,
	0x5cbf0a7f,0x5cc10cc1,0x5cc416fb,0x5ccb15fd,0x5cd2040b,0x5cd91965,0x5ce114d1,0x5ce40fbd,
	0x5ce5191f,0x5ce60c41,0x5ce8046e,0x5cea17ee,0x5ced0fcd,0x5cf00505,0x5cfb09dd,0x5d020ae9,
	0x5d030ab6,0x5d060a42,0x5d070273,0x5d0e0f4a,0x5d140303,0x5d16161c,0x5d1b09c3,0x5d1e0685,
	0x5d24153c,0x5d26162f,0x5d271250,0x5d2900c6,0x5d2d18ba,0x5d2e0630,0x5d3413ee,0x5d3d1856,
	0x5d3e169d,0x5d4707f9,0x5d4a118d,0x5d4b0cd8,0x5d4c0f9f,0x5d5810a1,0x5d5b17c7,0x5d5d0c0f,
	0x5d691253,0x5d6b1a17,0x5d6c1409,0x5d6f031a,0x5d740817,0x5d8218d7,0x5d990bb5,0x5d9d038c,
	0x5db716da,0x5dc503b9,0x5dcd13f8,0x5ddb02a4,0x5ddd02a5,0x5dde198f,0x5de115fa,0x5de2020a,
	0x5de505e4,0x5de61a5f,0x5de70fc9,0x5de80994,0x5de905f2,0x5deb1453,0x5dee01c6,0x5def1025,
	0x5df10811,0x5df216e0,0x5df3123d,0x5df40046,0x5df71524,0x5dfd160c,0x5dfe090f,0x5e0100db,
	0x5e0211b2,0x5e030177,0x5e051210,0x5e0604a4,0x5e081190,0x5e0c1489,0x5e0f13ff,0x5e1018d2,
	0x5e1112f6,0x5e140e8d,0x5e150e53,0x5e16134e,0x5e180b66,0x5e19195f,0x5e1a1998,0x5e1b0158,
	0x5e1c1960,0x5e1d03ab,0x5e26033c,0x5e27190c,0x5e2d14b4,0x5e2e007e,0x5e31027a,0x5e371406,
	0x5e3801f7,0x5e3b187e,0x5e3c068b,0x5e3d0cca,0x5e420d1e,0x5e441448,0x5e45053c,0x5e4c0791,
	0x5e540ca4,0x5e550d9a,0x5e5b18d8,0x5e5e0540,0x5e6104a6,0x5e6202b4,0x5e72057b,0x5e730ef8,
	0x5e740ded,0x5e76014a,0x5e78158f,0x5e7a1685,0x5e7b076a,0x5e7c17a4,0x5e7d1788,0x5e7f065b,
	0x5e800ec2,0x5e8419e2,0x5e861007,0x5e8700df,0x5e8a02b3,0x5e8b066a,0x5e8f15bd,0x5e900c19,
	0x5e911465,0x5e930a57,0x5e941743,0x5e9503a4,0x5e960e7b,0x5e9703c4,0x5e990d3e,0x5e9a05da,
	0x5e9c0546,0x5e9e0e71,0x5e9f04e2,0x5ea0151a,0x5ea5159e,0x5ea60432,0x5ea71a69,0x5ead1356,
	0x5eb300a9,0x5eb5001c,0x5eb61204,0x5eb70a04,0x5eb8176f,0x5eb913b1,0x5ebe17df,0x5ec90b6c,
	0x5eca0adb,0x5ed10920,0x5ed20032,0x5ed30aa9,0x5ed60b9c,0x5edb01dc,0x5ee81563,0x5eea0bbe,
	0x5ef41737,0x5ef6163a,0x5ef71354,0x5efa088f,0x5efe05f1,0x5eff0df5,0x5f0009e9,0x5f010111,
	0x5f0216f3,0x5f030f69,0x5f040e25,0x5f081702,0x5f0a00f3,0x5f0b16ec,0x5f0f11b4,0x5f1111c6,
	0x5f1305e5,0x5f151739,0x5f17051f,0x5f180704,0x5f1b0251,0x5f1f03aa,0x5f2018c3,0x5f250d07,
	0x5f2614ed,0x5f27072a,0x5f290e2b,0x5f2a094b,0x5f2d0d13,0x5f2f13c4,0x5f3110d2,0x5f39035b,
	0x5f3a0fac,0x5f3c00eb,0x5f40060a,0x5f5015ed,0x5f52065e,0x5f530362,0x5f550c2d,0x5f561390,
	0x5f5707af,0x5f581972,0x5f5d16db,0x5f611113,0x5f621586,0x5f641366,0x5f66165a,0x5f690188,
	0x5f6a011c,0x5f6c0131,0x5f6d0e9c,0x5f7018c7,0x5f711764,0x5f730260,0x5f770e70,0x5f7916f6,
	0x5f7b0215,0x5f7c00d5,0x5f8013e4,0x5f81191d,0x5f8202f2,0x5f84094c,0x5f85033d,0x5f871609,
	0x5f8806f4,0x5f891679,0x5f8a0758,0x5f8b0c7d,0x5f8c071a,0x5f9015b5,0x5f921380,0x5f950aba,
	0x5f97037f,0x5f980e57,0x5f9914bc,0x5f9c01f8,0x5fa117f8,0x5fa80781,0x5faa1602,0x5fad1693,
	0x5fae13f5,0x5fb51926,0x5fb70381,0x5fbc08db,0x5fbd079a,0x5fc3156d,0x5fc4156c,0x5fc500dc,
	0x5fc616ee,0x5fc90370,0x5fcc0820,0x5fcd108a,0x5fcf01e8,0x5fd012dd,0x5fd1130f,0x5fd21310,
	0x5fd60314,0x5fd7195b,0x5fd813eb,0x5fd90cac,0x5fdd1338,0x5fe0197f,0x5fe1026d,0x5fe41466,
	0x5fe71785,0x5fea124d,0x5feb0a64,0x5fed0112,0x5fee195c,0x5ff10220,0x5ff50df6,0x5ff80e1a,
	0x5ffb156e,0x5ffd0722,0x5ffe09f5,0x5fff04f8,0x60000757,0x600112c9,0x60021254,0x60031467,
	0x60040e49,0x60050200,0x600602b7,0x600a0206,0x600d1a64,0x600e1886,0x600f1681,0x60120e2d,
	0x6014191e,0x60150e54,0x60160179,0x60190743,0x601b032b,0x601c0b67,0x601d1232,0x6020033e,
	0x602116cd,0x60250803,0x60260e95,0x60271590,0x6028181f,0x60290dda,0x602a0645,0x602b0525,
	0x602f0fd6,0x6035029c,0x603b1a36,0x603c0443,0x603f16fc,0x60410dd5,0x604215fe,0x604311bc,
	0x604b0b79,0x604d078e,0x60500a45,0x605206f9,0x60551203,0x60591682,0x605a07ac,0x605d084f,
	0x60620794,0x60631a22,0x606415bf,0x60670e3b,0x606806f6,0x60690489,0x606a0a31,0x606b040c,
	0x606c1333,0x606d05ec,0x606f1491,0x60700f74,0x60730a3a,0x6076047b,0x60781374,0x6079162c,
	0x607a09ef,0x607b01aa,0x607c0dca,0x607d1844,0x607f1780,0x60830aa1,0x60840fc8,0x60891494,
	0x608c1329,0x608d06b3,0x60921706,0x609407a1,0x609600b2,0x609a1255,0x609b104c,0x609d0a80,
	0x609f1479,0x60a01789,0x60a30771,0x60a6182c,0x60a80e0d,0x60ab1066,0x60ac15d6,0x60ad0f81,
	0x60af0d53,0x60b104dd,0x60b200a6,0x60b4030a,0x60b8082f,0x60bb1592,0x60bc0379,0x60c50ffc,
	0x60c6027b,0x60ca0937,0x60cb13d5,0x60d107d1,0x60d5132c,0x60d813e7,0x60da0726,0x60dc1495,
	0x60dd01fd,0x60df1407,0x60e007b3,0x60e603c8,0x60e7099e,0x60e80193,0x60e90242,0x60eb00b4,
	0x60ec0fd9,0x60ed0192,0x60ee035c,0x60ef0650,0x60f00466,0x60f31521,0x60f419f0,0x60f60782,
	0x60f91084,0x60fa157f,0x61000fca,0x6101027e,0x61060f84,0x610817fc,0x610917c8,0x610d0d55,
	0x610e00ec,0x610f170e,0x61150480,0x611a17cc,0x611f058b,0x61201847,0x61230b15,0x612404fa,
	0x61260a93,0x61270a94,0x612b127a,0x613f1825,0x614802d7,0x614a0f99,0x614c077b,0x614e117b,
	0x6151115e,0x61550d9c,0x615d1313,0x61620ca5,0x616707b5,0x616809f1,0x61701428,0x61751772,
	0x61770a05,0x618b012b,0x618e188a,0x61940fc3,0x619d0445,0x61a70270,0x61a806a3,0x61a90f71,
	0x61ac0948,0x61b702a0,0x61be06b9,0x61c20406,0x61c81564,0x61ca0043,0x61cb0cce,0x61d10cf1,
	0x61d20ad5,0x61d40bbf,0x61e60e37,0x61f50d01,0x61ff1720,0x6206059e,0x620805b2,0x620a1473,
	0x620b0860,0x620c15ac,0x620d11fd,0x620e109a,0x620f14c3,0x62100233,0x62111444,0x62120905,
	0x62150fa4,0x621607cd,0x62170fa5,0x621818bd,0x621a0f34,0x621b0850,0x621f0816,0x622109fa,
	0x62220806,0x62240579,0x62250389,0x622a08fc,0x622c0888,0x622e0c37,0x623302cb,0x62340345,
	0x6237073e,0x623d0744,0x623e0b47,0x623f04c5,0x624012a7,0x6241010b,0x6243095b,0x62471126,
	0x62480747,0x624904d3,0x624b11d0,0x624c11d1,0x624d0183,0x624e1896,0x62510f17,0x62520048,
	0x62530330,0x62541097,0x625813a0,0x625b0a07,0x62630a4c,0x62660f7a,0x62671941,0x62690aa5,
	0x626a0cec,0x626b10f2,0x626c1672,0x626d0e1b,0x626e0079,0x626f0213,0x62701082,0x62730068,
	0x62760523,0x62790ea8,0x627c0477,0x627e18dd,0x627f0235,0x62800821,0x62840205,0x628909be,
	0x628a0055,0x629116f7,0x629211df,0x629319d5,0x62951379,0x62960417,0x62970a0a,0x629818e9,
	0x629a0544,0x629b0e77,0x629f138e,0x62a00a47,0x62a10c4d,0x62a20fb2,0x62a40740,0x62a5009b,
	0x62a80e96,0x62ab0eac,0x62ac12c1,0x62b1009c,0x62b503a5,0x62b90d70,0x62bb021a,0x62bc1612,
	0x62bd0276,0x62bf0d50,0x62c20526,0x62c419bd,0x62c5034a,0x62c601c7,0x62c70d8f,0x62c80deb,
	0x62c90aab,0x62ca0547,0x62cc007a,0x62cd0e55,0x62ce0bae,0x62d00644,0x62d20996,0x62d313b3,
	0x62d40050,0x62d613a1,0x62d70e1f,0x62d8097a,0x62d919f8,0x62da0e66,0x62db18db,0x62dc005f,
	0x62df0de3,0x62e20c06,0x62e30877,0x62e5176c,0x62e60ac3,0x62e70e10,0x62e8014d,0x62e91879,
	0x62ec0aa6,0x62ed11bd,0x62ee08f4,0x62ef1927,0x62f105f4,0x62f31053,0x62f41213,0x62f6184e,
	0x62f70a10,0x62fc0eec,0x62fd19d7,0x62fe119f,0x62ff0da0,0x63010256,0x63020640,0x63071955,
	0x63080fd8,0x63090026,0x630e0a5f,0x6311133e,0x631613b8,0x631a1968,0x631b0c42,0x631d143e,
	0x631e12b8,0x631f154e,0x63200dc4,0x63210364,0x632208cf,0x63231929,0x63240813,0x63250795,
	0x63280007,0x632a0e31,0x632b0320,0x632f1917,0x633212a0,0x63391707,0x633a135d,0x633d13d1,
	0x6342146a,0x634309de,0x63451371,0x63460aa2,0x634919fa,0x634b0c7b,0x634c004d,0x634d06b4,
	0x634e113e,0x634f0e00,0x635009ab,0x63550175,0x635e0ae6,0x635f1299,0x6361087c,0x6362076e,
	0x63630375,0x63670ea5,0x63690ba8,0x636d0064,0x636e099f,0x63710009,0x637602bc,0x637708f9,
	0x637a0da9,0x637b0df1,0x638014e6,0x638203b6,0x6387045a,0x638811da,0x638903d9,0x638a0f13,
	0x638c18cf,0x638e0815,0x638f1301,0x63900f72,0x63920e58,0x639616b5,0x639809c4,0x63a00c4a,
	0x63a212e4,0x63a30217,0x63a508e8,0x63a70a46,0x63a81391,0x63a91651,0x63aa0321,0x63ac097f,
	0x63ad133c,0x63ae0f90,0x63b0005e,0x63b30c26,0x63b40643,0x63b7196e,0x63b80354,0x63ba01d3,
	0x63bc0651,0x63be1823,0x63c417c9,0x63c60a87,0x63c910aa,0x63cd1a42,0x63ce15cf,0x63cf0d32,
	0x63d0131f,0x63d201b4,0x63d616c3,0x63de0023,0x63e0162a,0x63e11449,0x63e301cd,0x63e909ea,
	0x63ea0965,0x63ed08ec,0x63f21153,0x63f41814,0x63f616a8,0x63f81892,0x63fd0ad0,0x63ff0ff2,
	0x640001d4,0x640105bb,0x64020c10,0x640508d7,0x640b01c9,0x640c18bb,0x640f0162,0x6410029e,
	0x64130316,0x641410ed,0x641b086f,0x641c125e,0x641e05a8,0x64201229,0x642110ea,0x64260e35,
	0x642a12ed,0x642c006d,0x642d0326,0x64340f87,0x643a1552,0x643d01ba,0x643f05c7,0x6441048b,
	0x6444115f,0x644511eb,0x64460065,0x64471694,0x6448013b,0x644a12d0,0x6452014c,0x6454120e,
	0x645818a8,0x645e0c6b,0x64670305,0x64690d6c,0x646d194b,0x64780d64,0x64790d69,0x647a18ef,
	0x64820ba0,0x64841748,0x648509ba,0x64870ee7,0x64910230,0x649210d5,0x6495123a,0x649606b8,
	0x64991a5c,0x649e19e8,0x64a40218,0x64a90b8d,0x64ac0fcf,0x64ad0154,0x64ae0318,0x64b019de,
	0x64b50df3,0x64b71554,0x64b80c17,0x64ba02fd,0x64bc06ba,0x64c0058e,0x64c20b05,0x64c5112c,
	0x64cd019e,0x64ce0fff,0x64d00778,0x64d20fea,0x64d70ec6,0x64d8016a,0x64de126a,0x64e21a08,
	0x64e4158c,0x64e60180,0x65000e5f,0x650907c5,0x6512185e,0x6518107c,0x65251a52,0x652b09d2,
	0x652e0dc1,0x652f1931,0x65340f15,0x65350f16,0x653611cf,0x65381786,0x65390574,0x653b05e8,
	0x653e04cc,0x653f192e,0x6545062e,0x65481543,0x65490d14,0x654c039a,0x654f0d54,0x65510974,
	0x65550267,0x65560030,0x655908e0,0x655b0b72,0x655d00e9,0x655e01fe,0x6562058a,0x656310e6,
	0x65660448,0x656b08da,0x656c0953,0x65701205,0x65720fb9,0x65741928,0x6577051e,0x6587142c,
	0x658b18a7,0x658c0133,0x659004d7,0x6591006c,0x65930acb,0x65970416,0x65990b9f,0x659b072e,
	0x659c1550,0x659f1905,0x65a1144c,0x65a40911,0x65a50262,0x65a70548,0x65a918b7,0x65ab1a00,
	0x65ad0439,0x65af1234,0x65b01574,0x65b904bd,0x65bc17b1,0x65bd1193,0x65c10e73,0x65c318b1,
	0x65c40cba,0x65c50c73,0x65c60e8e,0x65cb15d7,0x65cc0938,0x65ce0de4,0x65cf1a47,0x65d20be9,
	0x65d616e9,0x65d70f54,0x65e01457,0x65e20827,0x65e51099,0x65e60357,0x65e7096d,0x65e8194f,
	0x65e9186c,0x65ec15fb,0x65ed15bc,0x65ee0569,0x65ef0aad,0x65f0058f,0x65f106b2,0x65f6119c,
	0x65f70a79,0x65fa13ec,0x6600183a,0x6602002c,0x66031884,0x66060a9b,0x660a06d0,0x660c01ec,
	0x660e0d58,0x660f07b8,0x661316fd,0x6614148a,0x66151571,0x661912d5,0x661d185d,0x661f157e,
	0x66201766,0x662502c0,0x66270ce6,0x66281a5e,0x662d18dc,0x662f11be,0x663117ec,0x66340cc3,
	0x66350de5,0x663601fc,0x663c199d,0x663e14f8,0x66410209,0x6643078f,0x664b092a,0x664c1138,
	0x664f165e,0x66521111,0x6653153f,0x665416b3,0x66551845,0x66560796,0x665706a9,0x665a13d2,
	0x665f118a,0x6661016f,0x6664147a,0x666607b0,0x66680225,0x666e0f26,0x666f0946,0x6670149b,
	0x66740ffd,0x6676093a,0x6677066f,0x667a1973,0x667e0b8c,0x66821860,0x668415d1,0x668714d6,
	0x668c0a89,0x669111f4,0x66960e2e,0x66970029,0x669d0d5e,0x66a70017,0x66a80832,0x66ae0d9d,
	0x66b400a0,0x66b914eb,0x66be1399,0x66d911fa,0x66db15f7,0x66dc16a1,0x66dd00a2,0x66e614b1,
	0x66e90dc0,0x66f01826,0x66f2102c,0x66f316af,0x66f405d9,0x66f706e0,0x66f901a0,0x66fc0ca2,
	0x66fe1888,0x66ff132d,0x67001a54,0x67081829,0x6709179a,0x670a10c4,0x670b0e9a,0x670d0527,
	0x6710103a,0x67141226,0x67151918,0x67170ae1,0x671b13ed,0x671d020b,0x671f0f36,0x67260cf8,
	0x67280d90,0x672a141c,0x672b0d71,0x672c00c1,0x672d1897,0x672f11fc,0x673119a5,0x67340f22,
	0x6735045e,0x673a07dd,0x673d15a4,0x674010fe,0x6742184f,0x6743104f,0x6746057d,0x674801b3,
	0x67491115,0x674c1475,0x674e0b2d,0x674f158d,0x67500184,0x67510311,0x67531144,0x675618d3,
	0x675c0430,0x675e0f61,0x675f11fe,0x6760059c,0x67611340,0x67650ab5,0x67681675,0x67690c8b,
	0x676a0d35,0x676d06be,0x676f00a5,0x677008f3,0x677205a7,0x67731699,0x67750294,0x67770e4e,
	0x677c19c7,0x677e124e,0x677f0072,0x67810800,0x67840604,0x67870eb5,0x678913e5,0x678b04c1,
	0x6790148b,0x6795190e,0x67970bb0,0x679810c9,0x679a0cd2,0x679c068e,0x679d1936,0x679e02e4,
	0x67a211e2,0x67a3186d,0x67a50b48,0x67a70878,0x67a80236,0x67aa0fa6,0x67ab0501,0x67ad152b,
	0x67af0a51,0x67b00efe,0x67b31940,0x67b5152d,0x67b6085d,0x67b70841,0x67b80602,0x67c10468,
	0x67c30bcb,0x67c40146,0x67cf0063,0x67d00d89,0x67d10582,0x67d20f30,0x67d31076,0x67d410a9,
	0x67d818f7,0x67d914d2,0x67da17a1,0x67dc0673,0x67dd13b4,0x67de1a65,0x67e00e12,0x67e203a6,
	0x67e501b7,0x67e90971,0x67ec087a,0x67ef0a19,0x67f00db0,0x67f119cb,0x67f30bef,0x67f401cb,
	0x67fd022e,0x67ff11bf,0x6800193a,0x680518a2,0x6807011d,0x680818be,0x68091966,0x680a0bfa,
	0x680b040d,0x680c0c1e,0x680e0b4d,0x680f0ac4,0x68111201,0x68131214,0x68160f32,0x68170b52,
	0x681d063b,0x68211544,0x682915b8,0x682a19ab,0x68320a11,0x68330af0,0x68371683,0x683806e2,
	0x683905d3,0x683c05c0,0x683d1854,0x683e0c43,0x684008f7,0x684106fa,0x68420676,0x68431307,
	0x68440659,0x68451403,0x68460a7c,0x68480027,0x6849001a,0x684a09b3,0x684c19fb,0x684e1969,
	0x68501368,0x685110e7,0x68530762,0x68540901,0x68550972,0x68601614,0x68611080,0x68621901,
	0x6863036b,0x68640f33,0x68650fc1,0x68660756,0x68670679,0x686808b0,0x686919e3,0x686b12a1,
	0x68740536,0x68761372,0x687709c5,0x68810b80,0x6883135e,0x68850cd6,0x6886007f,0x688f0631,
	0x68931a29,0x689705e2,0x68a2113f,0x68a60d04,0x68a7145c,0x68a80b18,0x68ad12a2,0x68af131a,
	0x68b0155d,0x68b311e6,0x68b504bb,0x68c00880,0x68c20bd1,0x68c90d25,0x68cb0f4f,0x68cd0682,
	0x68d20086,0x68d51a32,0x68d80807,0x68da0e9d,0x68e012eb,0x68e303b0,0x68ee10fc,0x68f002bd,
	0x68f10b12,0x68f50a1f,0x68f918e4,0x68fa064a,0x68fc04f2,0x6901068f,0x690516e8,0x690b0b81,
	0x690d1947,0x690e19ec,0x69100980,0x691208c2,0x691f0425,0x69200fa0,0x69240c5a,0x692d13b2,
	0x693016a4,0x6934043b,0x6939117c,0x693d02ac,0x693f02c1,0x69421894,0x69541548,0x6957089a,
	0x695a0298,0x695d0b7c,0x695e0b13,0x69600db9,0x69630cdb,0x696615e2,0x696b080a,0x696e0297,
	0x697102ee,0x697709f3,0x69781018,0x69791758,0x697c0c0b,0x69800ef2,0x6982057a,0x69840ad2,
	0x698617cd,0x6987022b,0x69880c70,0x6989098f,0x698d1561,0x69940add,0x699510a4,0x69980990,
	0x699b1908,0x699c0083,0x69a704de,0x69a818a6,0x69ab129c,0x69ad1562,0x69b10306,0x69b40beb,
	0x69b7106c,0x69bb12bb,0x69c105aa,0x69ca122b,0x69cc02be,0x69ce01bc,0x69d0075a,0x69d405a2,
	0x69db09fb,0x69df0136,0x69e019b1,0x69ed0f64,0x69f20735,0x69fd01a3,0x69ff0922,0x6a0a04ad,
	0x6a170289,0x6a1812f0,0x6a1f18ca,0x6a210d6a,0x6a2814ab,0x6a2a06fc,0x6a2f0fb0,0x6a31174b,
	0x6a350fc5,0x6a3d1a5a,0x6a3e1832,0x6a44058d,0x6a470fbb,0x6a5013ae,0x6a580989,0x6a590241,
	0x6a5b09cc,0x6a611528,0x6a6519b3,0x6a710291,0x6a790c28,0x6a7c181b,0x6a8012dc,0x6a8414b9,
	0x6a8e0feb,0x6a901649,0x6a910b01,0x6a97016b,0x6aa01000,0x6aa90bc0,0x6aab01bf,0x6aac0cf9,
	0x6b200f9a,0x6b2102de,0x6b22075d,0x6b231572,0x6b2417b7,0x6b270e41,0x6b3217f3,0x6b371496,
	0x6b390f3b,0x6b3a0f37,0x6b3e0a6d,0x6b43110d,0x6b461575,0x6b471549,0x6b490fa1,0x6b4c05bc,
	0x6b591162,0x6b62194e,0x6b63192a,0x6b6402dd,0x6b650178,0x6b661468,0x6b670f45,0x6b6a13c2,
	0x6b790335,0x6b7b123c,0x6b7c0864,0x6b810d72,0x6b8202f3,0x6b83166e,0x6b841339,0x6b86033f,
	0x6b871132,0x6b89160b,0x6b8a11e5,0x6b8b0190,0x6b8d0ee3,0x6b921840,0x6b930b7a,0x6b961948,
	0x6b9a034f,0x6b9b0808,0x6ba1013c,0x6baa1719,0x6bb311de,0x6bb40e42,0x6bb50438,0x6bb71728,
	0x6bbf03cb,0x6bc107a2,0x6bc2061c,0x6bc51715,0x6bcb1458,0x6bcd0d8b,0x6bcf0ce1,0x6bd20422,
	0x6bd31800,0x6bd400d2,0x6bd500dd,0x6bd600e2,0x6bd70eb6,0x6bd900e5,0x6bdb0cb6,0x6be118b0,
	0x6bea0d8a,0x6beb06c7,0x6bef12e1,0x6bf3030d,0x6bf510e2,0x6bf911ec,0x6bfd089b,0x6c0501ff,
	0x6c060f29,0x6c070c2a,0x6c0d1041,0x6c0f11ae,0x6c100394,0x6c110d47,0x6c130cb1,0x6c140f65,
	0x6c150ee6,0x6c160dae,0x6c180371,0x6c1914e2,0x6c1a02a6,0x6c1b04ee,0x6c1f052c,0x6c210403,
	0x6c220ff4,0x6c241729,0x6c26069f,0x6c27167f,0x6c28001b,0x6c291629,0x6c2a0a33,0x6c2e0360,
	0x6c2f0c81,0x6c300ffe,0x6c321835,0x6c34121c,0x6c35121b,0x6c381779,0x6c3d139e,0x6c401351,
	0x6c411933,0x6c42101d,0x6c4602fc,0x6c4707a5,0x6c4906b0,0x6c4a01c2,0x6c501486,0x6c540f67,
	0x6c551124,0x6c5706b1,0x6c5b1607,0x6c5c1240,0x6c5d10ba,0x6c5e05f3,0x6c5f08a4,0x6c600252,
	0x6c611450,0x6c6412e6,0x6c680d18,0x6c69061f,0x6c6a13e0,0x6c7012c8,0x6c7207fd,0x6c740113,
	0x6c761437,0x6c791597,0x6c7d0f6a,0x6c7e04f1,0x6c810ff1,0x6c8216ca,0x6c831445,0x6c85180e,
	0x6c8606c2,0x6c881170,0x6c890221,0x6c8c044f,0x6c8f0f2e,0x6c900d93,0x6c9312b7,0x6c940d27,
	0x6c9910ff,0x6c9b0e8b,0x6c9f05fa,0x6ca10cd1,0x6ca30500,0x6ca40e4a,0x6ca50b44,0x6ca60c51,
	0x6ca7019a,0x6ca91400,0x6caa0741,0x6cab0d73,0x6cad11ff,0x6cae098b,0x6cb113a8,0x6cb213a3,
	0x6cb306df,0x6cb804e3,0x6cb9178f,0x6cbb1961,0x6cbc18de,0x6cbd0611,0x6cbe18af,0x6cbf1641,
	0x6cc41558,0x6cc5101f,0x6cc91051,0x6cca0159,0x6ccc0d1a,0x6cd00afa,0x6cd30708,0x6cd40580,
	0x6cd504a1,0x6cd60cc2,0x6cd71246,0x6cdb04b6,0x6cde0e17,0x6ce00bc9,0x6ce10e81,0x6ce2014e,
	0x6ce30f6b,0x6ce50ddb,0x6ce819c8,0x6cea0b0d,0x6ceb15dc,0x6cee0e68,0x6cef0d51,0x6cf012cc,
	0x6cf1166d,0x6cf3177c,0x6cf500ca,0x6cf615eb,0x6cf70bf8,0x6cf80c1c,0x6cfa0c64,0x6cfb1559,
	0x6cfc0f04,0x6cfd187a,0x6cfe0933,0x6d0108f5,0x6d04079d,0x6d071724,0x6d0b167a,0x6d0c0ba5,
	0x6d0e0828,0x6d1210d6,0x6d1714ba,0x6d1919a9,0x6d1a08b5,0x6d1b0c65,0x6d1e040e,0x6d250914,
	0x6d271412,0x6d2a0709,0x6d2b15c0,0x6d2e1305,0x6d310493,0x6d321993,0x6d3310be,0x6d3515ff,
	0x6d390761,0x6d3b07c6,0x6d3c13b9,0x6d3d0f75,0x6d3e0e5b,0x6d410be5,0x6d430842,0x6d450f94,
	0x6d4608a8,0x6d4708be,0x6d4818ff,0x6d4a1a01,0x6d4b01ab,0x6d4d07a8,0x6d4e0829,0x6d4f0be4,
	0x6d5107bc,0x6d52073a,0x6d530e23,0x6d541600,0x6d5918f8,0x6d5a09df,0x6d5c0080,0x6d5e1a02,
	0x6d601492,0x6d63076f,0x6d660f25,0x6d6906d1,0x6d6a0ae4,0x6d6e0531,0x6d6f145d,0x6d7417ef,
	0x6d770699,0x6d78092b,0x6d7c0ce3,0x6d821381,0x6d850e02,0x6d881530,0x6d89115c,0x6d8c177f,
	0x6d8e14f0,0x6d911272,0x6d9309ac,0x6d9401ae,0x6d95132a,0x6d9b12ff,0x6d9d0af3,0x6d9e0ab7,
	0x6d9f0b68,0x6da01404,0x6da11440,0x6da30770,0x6da4039b,0x6da610cf,0x6da70896,0x6da818ce,
	0x6da910f7,0x6daa0537,0x6dab064c,0x6dae1215,0x6daf161d,0x6db216b6,0x6db506aa,0x6db806e5,
	0x6dbf19fc,0x6dc003c9,0x6dc41a13,0x6dc51497,0x6dc6153d,0x6dc70f4b,0x6dcb0bb2,0x6dcc12f8,
	0x6dd111e7,0x6dd60dce,0x6dd8130a,0x6dd902eb,0x6ddd04d9,0x6dde1251,0x6de00ecc,0x6de1035d,
	0x6de417af,0x6de60591,0x6deb1731,0x6dec030b,0x6dee0759,0x6df1116b,0x6df302c6,0x6df707c0,
	0x6df91630,0x6dfb1331,0x6dfc0d38,0x6e050ff9,0x6e0a180a,0x6e0c0c30,0x6e0d1a2e,0x6e0e0424,
	0x6e100898,0x6e111184,0x6e1417c3,0x6e161175,0x6e17117a,0x6e1a19be,0x6e1d17b2,0x6e20103c,
	0x6e210433,0x6e231893,0x6e240160,0x6e25144a,0x6e29142a,0x6e2b155f,0x6e2d1425,0x6e2f059b,
	0x6e3215e1,0x6e340a2d,0x6e381795,0x6e3a0d39,0x6e430e5c,0x6e440cd9,0x6e4d138c,0x6e4e0d2b,
	0x6e530e94,0x6e54086c,0x6e56072f,0x6e581512,0x6e5b18c1,0x6e5f0783,0x6e6b08d8,0x6e6e1634,
	0x6e7e13c6,0x6e7f1195,0x6e830a95,0x6e85089c,0x6e8615c6,0x6e890578,0x6e8f12ee,0x6e901818,
	0x6e980a36,0x6e9c0be1,0x6e9f0d5d,0x6ea2170f,0x6ea50f27,0x6ea70b16,0x6eaa14a1,0x6eaf127b,
	0x6eb10fe8,0x6eb2125f,0x6eb415aa,0x6eb610a2,0x6eb707c1,0x6eba0de8,0x6ebb12b3,0x6ebd10bf,
	0x6ec1028d,0x6ec20e6f,0x6ec703b7,0x6ecb1a18,0x6ecf054e,0x6ed10751,0x6ed31a2b,0x6ed41302,
	0x6ed51317,0x6ed700ee,0x6eda067f,0x6ede1974,0x6edf1666,0x6ee01160,0x6ee10ca0,0x6ee21759,
	0x6ee40c7c,0x6ee50ad7,0x6ee60c46,0x6ee80134,0x6ee912d1,0x6ef40393,0x6ef90727,0x6f020edc,
	0x6f060f39,0x6f090c36,0x6f0f0c13,0x6f130b1f,0x6f141656,0x6f1501a2,0x6f200d79,0x6f240ad3,
	0x6f2915d8,0x6f2a16c4,0x6f2b0ca6,0x6f2d0cb3,0x6f2f0c6c,0x6f311208,0x6f3318c8,0x6f360776,
	0x6f3e1684,0x6f46175b,0x6f471536,0x6f4b0b7d,0x6f4d140a,0x6f580e5e,0x6f5c0f92,0x6f5e0c39,
	0x6f620786,0x6f660b8e,0x6f6d12db,0x6f6e020d,0x6f72114e,0x6f7419b2,0x6f78111e,0x6f7a01dd,
	0x6f7c136e,0x6f840240,0x6f880219,0x6f89058c,0x6f8c123b,0x6f8d1209,0x6f8e0ea1,0x6f9c0ac9,
	0x6fa1186f,0x6fa70b35,0x6fb30042,0x6fb601e0,0x6fb912d3,0x6fc007f5,0x6fc20b6e,0x6fc91287,
	0x6fd10abe,0x6fd20138,0x6fde00fc,0x6fe006cc,0x6fe110b5,0x6fee0f20,0x6fef1a09,0x70110f2c,
	0x701a06bc,0x701b175f,0x70231569,0x703504fd,0x70391834,0x704c0653,0x704f06d5,0x705e005d,
	0x706b07c8,0x706c07c7,0x706d0d43,0x706f0383,0x70700792,0x70750bc7,0x70761871,0x70780969,
	0x707c19fe,0x707e1851,0x707f0195,0x70801676,0x70850674,0x70890c1d,0x708a02b9,0x708e1642,
	0x7092020f,0x70941062,0x70950a0c,0x70960450,0x70991962,0x709c1410,0x709d0fb5,0x70ab15dd,
	0x70ac0999,0x70ad12e3,0x70ae0e7d,0x70af095d,0x70b112c3,0x70b30147,0x70b719cc,0x70b818a3,
	0x70b903bc,0x70bb11a0,0x70bc0b78,0x70bd0265,0x70c00723,0x70c11225,0x70c20ad6,0x70c31353,
	0x70c80ba7,0x70ca167b,0x70d80702,0x70d90af4,0x70db19b6,0x70df162d,0x70e40a12,0x70e604ac,
	0x70e71140,0x70e816b4,0x70e907ad,0x70eb12fd,0x70ec092c,0x70ed1085,0x70ef1498,0x70f713cf,
	0x70f90e98,0x70fd0506,0x71091631,0x710a06b5,0x7110147b,0x711306ab,0x71150772,0x71160cf0,
	0x7118037a,0x711900b5,0x711a04f3,0x712608c3,0x712f19f7,0x71301663,0x71311664,0x71361071,
	0x7145043c,0x714a15d2,0x714c0785,0x714e0870,0x715c17fd,0x715e1108,0x71640cdc,0x716615c8,
	0x716718e5,0x716813f6,0x716e19bf,0x71720090,0x71730732,0x71780106,0x717a1395,0x717d111d,
	0x718414a5,0x718a159a,0x718f15f4,0x719410a5,0x71980be2,0x719914a6,0x719f11f3,0x71a01716,
	0x71a8184a,0x71ac002f,0x71b30ca9,0x71b51136,0x71b914ac,0x71c31073,0x71ce0b98,0x71d404af,
	0x71d5166a,0x71e01803,0x71e51876,0x71e71292,0x71ee1568,0x71f914ff,0x720600a1,0x721d09d1,
	0x72280302,0x722a19d6,0x722c0e4f,0x72301810,0x72310011,0x723509cd,0x72360552,0x723716a6,
	0x72380059,0x723903dc,0x723b168b,0x723d1219,0x723f0e60,0x72470ed9,0x72480073,0x724c0e59,
	0x724d0426,0x725203e6,0x7256179f,0x72591616,0x725b0e19,0x725d0ef3,0x725f0d84,0x72610d8d,
	0x72620ae8,0x72660cb8,0x72670d95,0x72691477,0x726e0892,0x726f0623,0x72721181,0x72750f80,
	0x72791311,0x727a1493,0x727e146b,0x727f0632,0x7280149c,0x72810b1c,0x728407eb,0x728a0427,
	0x728b09a1,0x728d086d,0x728f0ed2,0x72920a14,0x729f08b8,0x72ac105d,0x72ad105c,0x72af04b5,
	0x72b0101c,0x72b40024,0x72b619e7,0x72b7065c,0x72b80c87,0x72b9178d,0x72c1183e,0x72c20a72,
	0x72c30e1c,0x72c40397,0x72c800ac,0x72cd0e7c,0x72ce14d0,0x72d0072b,0x72d204e4,0x72d70600,
	0x72d9097b,0x72de0e11,0x72e006f5,0x72e108d0,0x72e8109c,0x72e911d7,0x72ec0429,0x72ed14d3,
	0x72ee1194,0x72ef0a68,0x72f01920,0x72f117ed,0x72f21296,0x72f317c0,0x72f400e6,0x72f709b4,
	0x72f80b19,0x72fa172f,0x72fb1280,0x72fc0ada,0x73010b53,0x730314fa,0x730a0dde,0x730e0ba9,
	0x73130689,0x73150d0a,0x731601ee,0x731716dc,0x731b0cfd,0x731c0182,0x731d02f5,0x731e1150,
	0x73210c56,0x73220730,0x73251416,0x73291580,0x732a19ae,0x732b0cb5,0x732c1426,0x732e150a,
	0x73310dc7,0x73340712,0x73371797,0x73380cda,0x733901bb,0x733e0752,0x733f1819,0x734d0957,
	0x735018c9,0x73520033,0x735709ca,0x73600b96,0x736c1565,0x736d12b5,0x736f15f5,0x737e075e,
	0x738415d4,0x73870c7f,0x738917e4,0x738b13e2,0x738e03ee,0x739107de,0x7396096a,0x739b0c88,
	0x739f0d49,0x73a2013a,0x73a913cd,0x73ab0cd3,0x73ae1411,0x73af075f,0x73b01503,0x73b20bcc,
	0x73b30340,0x73b703c6,0x73ba14bb,0x73bb014f,0x73c00f0d,0x73c20a1a,0x73c80843,0x73c90d4b,
	0x73ca111a,0x73cd1900,0x73cf09bf,0x73d004a3,0x73d10bfb,0x73d905f5,0x73de0c69,0x73e019ac,
	0x73e50495,0x73e71690,0x73e906fb,0x73ed0069,0x73f20797,0x74031023,0x74050adc,0x74060b32,
	0x74090be7,0x740a161a,0x740f0b73,0x741012aa,0x741a0981,0x741b021c,0x74221a06,0x7425073c,
	0x74260f50,0x74280a9c,0x742a0f51,0x742c13d9,0x742e02ec,0x74301654,0x74330bb3,0x74340fe4,
	0x74350ebc,0x74360e51,0x743c1012,0x74410ccb,0x745514d7,0x74571824,0x74590dcc,0x745a0733,
	0x745b1746,0x745c17ce,0x745e10cc,0x745f10fa,0x746d12ef,0x74700666,0x74761696,0x74770018,
	0x747e0923,0x74800307,0x748102e7,0x74830b23,0x748715d9,0x748b18cb,0x748e174c,0x74900c3a,
	0x749c0787,0x749e0f1f,0x74a700ff,0x74a80197,0x74a9103f,0x74ba1438,0x74d21863,0x74dc0637,
	0x74de03e0,0x74e00748,0x74e20ee0,0x74e3007c,0x74e41078,0x74e613bd,0x74ee143c,0x74ef0e43,
	0x74f40bcd,0x74f60eff,0x74f702d6,0x74ff017d,0x75041906,0x750d0cf6,0x750f00cc,0x7511188e,
	0x75130ecf,0x7518057c,0x75190339,0x751a1178,0x751c1335,0x751f117f,0x75251183,0x75281783,
	0x7529120f,0x752b0545,0x752c177a,0x752d00c9,0x752f0e15,0x75301332,0x7531178c,0x75320855,
	0x75331163,0x753503bf,0x75370db5,0x753803c1,0x753a03f4,0x753b0754,0x753e1852,0x754000e0,
	0x75450201,0x754804ba,0x754b1334,0x754c0908,0x754e105e,0x754f141f,0x75540e6b,0x75590be6,
	0x755a00c3,0x755b1910,0x755c02a3,0x75650c4b,0x75660f4c,0x756a04a5,0x75721152,0x7574027d,
	0x757807ee,0x757913db,0x757f07f1,0x7583138f,0x758608ad,0x758b0ec8,0x758f11e9,0x759116d9,
	0x75920dcf,0x759403ef,0x759608e6,0x75970b90,0x759905b5,0x759a0970,0x759d1125,0x759f0e2f,
	0x75a00b49,0x75a11678,0x75a31790,0x75a4004c,0x75a50909,0x75ab1703,0x75ac0b4e,0x75ae02b1,
	0x75af0503,0x75b019ce,0x75b10e82,0x75b20eb7,0x75b30584,0x75b40a1d,0x75b502d1,0x75b80353,
	0x75b91911,0x75bc1314,0x75bd097e,0x75be0805,0x75c20845,0x75c315d5,0x75c418a4,0x75c5014b,
	0x75c7192f,0x75c8176d,0x75c9094f,0x75ca1055,0x75cd16d5,0x75d21680,0x75d4196f,0x75d506f3,
	0x75d61621,0x75d8041d,0x75db1375,0x75de0ec5,0x75e20b5d,0x75e31975,0x75e4031b,0x75e6147d,
	0x75e71105,0x75e80aea,0x75ea0774,0x75eb14f4,0x75f012d8,0x75f104e7,0x75f4024e,0x75f900ef,
	0x75fc0634,0x75ff1417,0x760017b0,0x7601030e,0x760319b9,0x76050356,0x760a0713,0x760c0ab1,
	0x761017e0,0x76150859,0x76171713,0x76180c14,0x761910f5,0x761b026a,0x761f142b,0x7620080d,
	0x7622006e,0x76240bec,0x762501cf,0x762611dc,0x7629032d,0x762a012d,0x762b12d2,0x762d0122,
	0x76300c62,0x76330277,0x763418d9,0x763518ae,0x76381064,0x763c0d7d,0x763e173e,0x763f1765,
	0x7640078a,0x76430c01,0x764c000b,0x764d006f,0x7654171e,0x76560ec7,0x765c03cd,0x765e0abf,
	0x766315db,0x766b03ba,0x766f1042,0x7678066d,0x767b0384,0x767d0060,0x767e0061,0x76821872,
	0x76840382,0x768608e7,0x7687077d,0x76880663,0x768b059f,0x768e08d3,0x7691000a,0x769306d3,
	0x769613da,0x769914a2,0x76a40f08,0x76ae0eb3,0x76b119a0,0x76b209d7,0x76b40312,0x76bf0d4e,
	0x76c217b8,0x76c51981,0x76c60e93,0x76c81752,0x76ca1708,0x76cd06e3,0x76ce002d,0x76cf18b9,
	0x76d01644,0x76d10869,0x76d206e6,0x76d40a81,0x76d60577,0x76d7037b,0x76d80e61,0x76db118b,
	0x76df0cf5,0x76e50652,0x76ee0d92,0x76ef03f0,0x76f115ad,0x76f20cae,0x76f41943,0x76f8150f,
	0x76f9044c,0x76fc0e6a,0x76fe0451,0x77011186,0x77040d2e,0x77070d36,0x7708034b,0x77090cd4,
	0x770b0a01,0x770d0a49,0x771916d3,0x771a1187,0x771f1902,0x77200d23,0x77221808,0x77261a2f,
	0x7728189b,0x772915df,0x772d067a,0x772f0d06,0x7735024a,0x77360a7d,0x773709b7,0x77380d86,
	0x773a1348,0x773c1652,0x774018fb,0x77411922,0x774312a3,0x774703b1,0x77500abc,0x77510882,
	0x775a161e,0x775b093c,0x7761121e,0x77621286,0x77630420,0x77650101,0x77660d9b,0x77680de9,
	0x776b08fb,0x776c0189,0x7779042d,0x777d0a8b,0x777e05a3,0x777f10cd,0x77800ccc,0x77840d33,
	0x77850285,0x778c0a24,0x778d1267,0x778e14cd,0x77910d5f,0x77920c9c,0x779f0ee4,0x77a00231,
	0x77a20cf7,0x77a50ee8,0x77a70fc6,0x77a919c2,0x77aa038d,0x77ac1222,0x77b00a03,0x77b3136f,
	0x77b50bb9,0x77bb18b5,0x77bd062c,0x77bf1038,0x77cd09d0,0x77d702a2,0x77db0cb7,0x77dc0915,
	0x77e211a6,0x77e316e3,0x77e51937,0x77e71172,0x77e9098d,0x77eb08d4,0x77ec031c,0x77ed0437,
	0x77ee000d,0x77f3119b,0x77f607e1,0x77f80587,0x77fd148c,0x77fe04aa,0x77ff0a7a,0x78000369,
	0x78010c89,0x78021102,0x7809074c,0x780c0f6d,0x780d09fe,0x78111628,0x78120ead,0x78141643,
	0x781619d9,0x78170212,0x78180452,0x781a165b,0x781c0504,0x781d04a2,0x781f189c,0x782313aa,
	0x782503a7,0x782618ac,0x78271903,0x78290532,0x782c0b38,0x782d0103,0x78300e97,0x78340f0e,
	0x7837116a,0x78381850,0x78390012,0x783a0b54,0x783b0bfd,0x783c1369,0x783e0b55,0x78400295,
	0x78450665,0x78470dc5,0x784c05d0,0x784e1589,0x78500411,0x78521499,0x78551228,0x785614d4,
	0x78570fb6,0x785d1534,0x786a144b,0x786b0be8,0x786c1767,0x786d0cb0,0x786e1068,0x78770883,
	0x787c0e9e,0x788703fb,0x788903d3,0x788c0c34,0x788d0016,0x788e1290,0x789100a7,0x78930444,
	0x789713dc,0x789803bd,0x789a00b7,0x789b0f6f,0x789c0227,0x789f03e7,0x78a119a4,0x78a308fd,
	0x78a5010e,0x78a700f4,0x78b00ea6,0x78b10889,0x78b203b4,0x78b312e5,0x78b401be,0x78b915e3,
	0x78be0df4,0x78c102d9,0x78c50089,0x78c910eb,0x78ca0b09,0x78cb0317,0x78d00e62,0x78d418f0,
	0x78d50a25,0x78d90680,0x78e80d6d,0x78ec1009,0x78f2103e,0x78f4038e,0x78f70bba,0x78fa078b,
	0x790108c9,0x7905044a,0x791308ac,0x791e0cfa,0x79240181,0x79340167,0x793a11b3,0x793b11af,
	0x793c0b2c,0x793e115a,0x79401245,0x79410f3e,0x794614e3,0x79480f46,0x79491953,0x7953052d,
	0x79561a4d,0x7957193b,0x795a1a66,0x795b1031,0x795c0745,0x795d19cd,0x795e116e,0x795f128d,
	0x796002d3,0x79620d08,0x7965151b,0x7967133f,0x79680ee5,0x796d0830,0x796f1904,0x79770376,
	0x797807d0,0x797a0f52,0x79800149,0x7981091a,0x79840c33,0x798501d8,0x798a14ca,0x798f053d,
	0x799a1a07,0x79a714c2,0x79b31077,0x79b917db,0x79ba17bc,0x79bb0b1a,0x79bd0fe5,0x79be06da,
	0x79c015a5,0x79c11230,0x79c3137d,0x79c60588,0x79c90145,0x79cb1016,0x79cd1988,0x79d10a1b,
	0x79d20d37,0x79d500d6,0x79d80d1c,0x79df1a43,0x79e30d76,0x79e40245,0x79e60fe3,0x79e7166f,
	0x79e9196a,0x79eb11ef,0x79ed1a26,0x79ef07e8,0x79f0022d,0x79f808e9,0x79fb16d6,0x79fd07b1,
	0x7a00149d,0x7a020ade,0x7a03051b,0x7a060c74,0x7a0b023c,0x7a0d1141,0x7a0e121d,0x7a14108c,
	0x7a170067,0x7a1a1978,0x7a1e0a21,0x7a20027f,0x7a23126d,0x7a331435,0x7a370836,0x7a391913,
	0x7a3b037d,0x7a3c085f,0x7a3d07f2,0x7a3f05ab,0x7a460d9e,0x7a5110fb,0x7a571293,0x7a701079,
	0x7a7415e9,0x7a760960,0x7a77100e,0x7a78148d,0x7a79100f,0x7a7a0a40,0x7a7f02a7,0x7a8019f4,
	0x7a81137e,0x7a830fd7,0x7a8418aa,0x7a86010c,0x7a88169b,0x7a8d0fce,0x7a911691,0x7a921970,
	0x7a951347,0x7a9608e1,0x7a9702b2,0x7a98095e,0x7a9c0300,0x7a9d1442,0x7a9f0a54,0x7aa00a22,
	0x7aa50a82,0x7aa6041e,0x7aa8173f,0x7aac17d1,0x7aad09a4,0x7ab317e1,0x7abf0c02,0x7acb0b3d,
	0x7ad61202,0x7ad918bf,0x7ade0950,0x7adf0952,0x7ae018c4,0x7ae309e1,0x7ae5136b,0x7ae61257,
	0x7aed08fe,0x7aef0436,0x7af919b4,0x7afa19b5,0x7afd17bd,0x7aff0583,0x7b03042a,0x7b0407e9,
	0x7b06004e,0x7b080804,0x7b0a18e3,0x7b0b129a,0x7b0f0746,0x7b111545,0x7b1400d8,0x7b15087d,
	0x7b191182,0x7b1b039d,0x7b1e024b,0x7b200b58,0x7b241342,0x7b251249,0x7b260538,0x7b2800c5,
	0x7b2a0331,0x7b2b1a28,0x7b2c03ae,0x7b2e187f,0x7b310603,0x7b330846,0x7b380f0b,0x7b3a086a,
	0x7b3c0bfe,0x7b3e0104,0x7b4514fc,0x7b471011,0x7b490388,0x7b4b0917,0x7b4c1057,0x7b4f04a0,
	0x7b500a71,0x7b5119d1,0x7b521373,0x7b54032c,0x7b5601ac,0x7b580a4e,0x7b5a00ed,0x7b5b1110,
	0x7b5d1924,0x7b6009d9,0x7b620e52,0x7b6e11ca,0x7b711540,0x7b721142,0x7b751646,0x7b770a6a,
	0x7b790280,0x7b7b059d,0x7b7e0f85,0x7b800886,0x7b8500f5,0x7b8d061a,0x7b901008,0x7b940163,
	0x7b9507f0,0x7b971283,0x7b9c0a43,0x7b9d0f91,0x7ba1064e,0x7ba213dd,0x7ba61881,0x7ba70fda,
	0x7ba813b6,0x7ba90c5c,0x7baa0350,0x7bab1537,0x7bac10d3,0x7bad08a2,0x7bb11515,0x7bb41909,
	0x7bb819d3,0x7bc10788,0x7bc619df,0x7bc70ed3,0x7bcc0714,0x7bd10a98,0x7bd30c11,0x7bd905a5,
	0x7bda04e0,0x7bdd05fd,0x7be10301,0x7be50b61,0x7be600f9,0x7bea0259,0x7bee0acc,0x7bf10b26,
	0x7bf70ea2,0x7bfc0415,0x7bfe0d45,0x7c0702f9,0x7c0b0670,0x7c0c127f,0x7c0f0c3b,0x7c16043e,
	0x7c1f03ce,0x7c260386,0x7c27078c,0x7c2a185b,0x7c380169,0x7c3f017e,0x7c4019a3,0x7c410ac0,
	0x7c4d080f,0x7c730d12,0x7c740398,0x7c7b0b0e,0x7c7c14e4,0x7c7d1a23,0x7c8904f5,0x7c91004f,
	0x7c920b59,0x7c950f0f,0x7c9702f1,0x7c9818b2,0x7c9c1349,0x7c9d0b5a,0x7c9e149e,0x7c9f1276,
	0x7ca21a19,0x7ca41830,0x7ca51994,0x7caa04fb,0x7cae0b82,0x7cb10b83,0x7cb20196,0x7cb3093d,
	0x7cb9030f,0x7cbc0bb4,0x7cbd1a39,0x7cbe093f,0x7cc110e4,0x7cc510ab,0x7cc70715,0x7cc815b9,
	0x7cca0728,0x7ccc185a,0x7ccd02dc,0x7cd505a6,0x7cd612f2,0x7cd7102a,0x7cd9019f,0x7cdc0d0d,
	0x7cdf186a,0x7ce00a06,0x7ce808b9,0x7cef0e38,0x7cf80d17,0x7cfb14c4,0x7d0a1434,0x7d201273,
	0x7d2212a9,0x7d27091d,0x7d2b1a2a,0x7d2f0b04,0x7d6e15c7,0x7d771949,0x7da60f55,0x7dae1006,
	0x7e3b0d0e,0x7e4104b0,0x7e471697,0x7e821a51,0x7e9b037e,0x7e9f122d,0x7ea0095f,0x7ea117ad,
	0x7ea20705,0x7ea31999,0x7ea414e1,0x7ea505b4,0x7ea61827,0x7ea707fe,0x7ea813ca,0x7ea90a77,
	0x7eaa081e,0x7eab1091,0x7eac140d,0x7ead1838,0x7eaf02c3,0x7eb00ea9,0x7eb11100,0x7eb20595,
	0x7eb30da4,0x7eb51a38,0x7eb60c52,0x7eb704ec,0x7eb81951,0x7eb9142d,0x7eba04ca,0x7ebd0e1d,
	0x7ebe11e0,0x7ebf1504,0x7ec00590,0x7ec1155a,0x7ec20528,0x7ec30b77,0x7ec41a4b,0x7ec51167,
	0x7ec614c6,0x7ec71938,0x7ec81980,0x7ec9199c,0x7eca007b,0x7ecb0529,0x7ecc029d,0x7ecd114c,
	0x7ece16fe,0x7ecf0934,0x7ed0033a,0x7ed10082,0x7ed2109d,0x7ed308f6,0x7ed40a58,0x7ed51083,
	0x7ed706bf,0x7ed807a9,0x7ed905d2,0x7eda15de,0x7edb08b6,0x7edc0c66,0x7edd09c0,0x7ede08d1,
	0x7edf1370,0x7ee005e0,0x7ee11531,0x7ee209b5,0x7ee315a7,0x7ee51288,0x7ee61300,0x7ee7082a,
	0x7ee8131d,0x7ee9081a,0x7eea15c2,0x7eeb0bd2,0x7eed15c3,0x7eee0f63,0x7eef04d1,0x7ef002ce,
	0x7ef1113c,0x7ef2067d,0x7ef31185,0x7ef41408,0x7ef50d24,0x7ef611db,0x7ef700c7,0x7ef8027c,
	0x7efa0bf0,0x7efb105f,0x7efc1a31,0x7efd18c0,0x7efe13d6,0x7eff0c80,0x7f0019ef,0x7f011a14,
	0x7f020a35,0x7f031513,0x7f04086e,0x7f050d2c,0x7f060ad1,0x7f071320,0x7f080d3a,0x7f0907ec,
	0x7f0b07b4,0x7f0c1235,0x7f0d0461,0x7f0e043a,0x7f0f0117,0x7f1105fc,0x7f1219f1,0x7f130769,
	0x7f1403b2,0x7f150c77,0x7f160105,0x7f170d4c,0x7f181815,0x7f19092e,0x7f1a0561,0x7f1b10c0,
	0x7f1c1912,0x7f1d050d,0x7f1f05a9,0x7f2001da,0x7f210b20,0x7f221710,0x7f230871,0x7f240135,
	0x7f250ee2,0x7f260ca7,0x7f270b00,0x7f281749,0x7f2912a6,0x7f2a0d3f,0x7f2b10ef,0x7f2c1555,
	0x7f2d0b97,0x7f2e112a,0x7f2f188b,0x7f3008ab,0x7f310f98,0x7f320fbc,0x7f330766,0x7f3408dc,
	0x7f351a50,0x7f360514,0x7f380597,0x7f3a1063,0x7f42174a,0x7f44100a,0x7f4514de,0x7f500655,
	0x7f5113e3,0x7f5413e6,0x7f5506ae,0x7f570c55,0x7f58052e,0x7f5a049e,0x7f5f0625,0x7f610599,
	0x7f62005a,0x7f681655,0x7f6918e6,0x7f6a1a55,0x7f6e1979,0x7f710ad4,0x7f7211f6,0x7f740ebe,
	0x7f790b27,0x7f7e188c,0x7f8107f6,0x7f8a1673,0x7f8c0fa3,0x7f8e0ce2,0x7f9405a0,0x7f9a0bd3,
	0x7f9d0391,0x7f9e159f,0x7f9f0fb3,0x7fa11509,0x7fa4106f,0x7fa712a4,0x7faf0900,0x7fb012e7,
	0x7fb214ad,0x7fb80b03,0x7fb905dd,0x7fbc01ea,0x7fbd17d8,0x7fbf1704,0x7fc11439,0x7fc50266,
	0x7fca170b,0x7fcc170c,0x7fce0bd4,0x7fd4151c,0x7fd5149f,0x7fd80fc7,0x7fdf03a0,0x7fe00310,
	0x7fe104df,0x7fe519d4,0x7fe6088a,0x7fe90ed4,0x7fee06eb,0x7ff006bb,0x7ff10035,0x7ff3171b,
	0x7ffb04a7,0x7ffc171c,0x800016a2,0x80010aed,0x80030a0f,0x80040cc8,0x800518f2,0x80060f47,
	0x800b03e4,0x800c048d,0x800d120c,0x80100db1,0x80120b06,0x80141a27,0x801505db,0x80160210,
	0x801706d2,0x8018183c,0x80190e50,0x801c124a,0x802007c2,0x80220af5,0x802512fa,0x80260e47,
	0x80270c0c,0x80280e26,0x802908b2,0x802a0e75,0x80310d81,0x80330491,0x803503f2,0x803616a7,
	0x80370325,0x80381256,0x803b025d,0x803d034c,0x803f05e1,0x80420e03,0x8043034e,0x80460bd5,
	0x804a0b91,0x804b0bff,0x804c1946,0x804d0e13,0x80520686,0x80540b6a,0x80580ef4,0x805a09a5,
	0x80690a99,0x806a02e8,0x80710036,0x807f17e6,0x808017e3,0x80831271,0x80841711,0x8086124c,
	0x808718e7,0x808910ae,0x808b0b0c,0x808c07df,0x80930779,0x8096153b,0x80981997,0x809a0431,
	0x809b0596,0x809c109b,0x809d057e,0x809f1446,0x80a001f3,0x80a10622,0x80a21939,0x80a40518,
	0x80a504d8,0x80a90866,0x80aa04c6,0x80ab19f3,0x80ad0da5,0x80ae002b,0x80af0a38,0x80b105ea,
	0x80b217ea,0x80b4168d,0x80b70f95,0x80ba04e5,0x80bc0944,0x80bd12ca,0x80be1177,0x80bf1987,
	0x80c018d4,0x80c1154d,0x80c21179,0x80c31420,0x80c4199e,0x80c60352,0x80cc00af,0x80cd0639,
	0x80ce12be,0x80d60e76,0x80d7190b,0x80d91a67,0x80da0e84,0x80db0856,0x80dc1188,0x80dd193c,
	0x80de008f,0x80e1072c,0x80e41742,0x80e515ae,0x80e70bfc,0x80e8040f,0x80e909e7,0x80ea0c1f,
	0x80eb094e,0x80ec0e2c,0x80ed162e,0x80ef0a60,0x80f016d4,0x80f1065a,0x80f2069a,0x80f305b7,
	0x80f40410,0x80f608c1,0x80f81598,0x80fa0028,0x80fc0ed6,0x80fd0dd6,0x8102193d,0x81060308,
	0x81090c98,0x810a0814,0x810d0a69,0x810e10d9,0x810f1867,0x81100f48,0x81110dcb,0x81120d15,
	0x81130e24,0x81140c45,0x81160155,0x811813d7,0x811a08d5,0x811e031e,0x812c0e78,0x812f054b,
	0x813113a2,0x81320dff,0x81360c57,0x81380b74,0x813e0ebd,0x8146133a,0x8148093b,0x814a0ab0,
	0x814b16b8,0x814c1635,0x8150054f,0x8151054d,0x815304da,0x81540fa8,0x815513df,0x81591a33,
	0x815a03fa,0x816002ef,0x81651581,0x81671206,0x81690dbb,0x816d0483,0x816e10dc,0x81701689,
	0x8171089d,0x817417ca,0x81790562,0x817a150b,0x817b0dea,0x817c0d2d,0x817d13c0,0x817e1315,
	0x817f1393,0x81800084,0x81820c78,0x818805c8,0x818a0164,0x818f05a4,0x8191013d,0x81980121,
	0x819b12f1,0x819c0d6b,0x819d14aa,0x81a3197b,0x81a60bc4,0x81a80ea3,0x81aa01d1,0x81b3112d,
	0x81ba174e,0x81bb111f,0x81c0139d,0x81c10b6f,0x81c200fd,0x81c31775,0x81c6171d,0x81ca10f0,
	0x81cc062b,0x81e3021f,0x81e71865,0x81ea1a2d,0x81ec0e04,0x81ed0286,0x81f3195a,0x81f4196b,
	0x81fb190a,0x81fc096e,0x81fe17b9,0x8200169c,0x820117be,0x8202026f,0x820414c8,0x82050976,
	0x820617d2,0x820c1154,0x820d1157,0x821011c2,0x821211ea,0x8214133b,0x821b02ad,0x821c1221,
	0x821e146d,0x821f1990,0x822102a9,0x8222111b,0x822316e5,0x82280075,0x822a06c0,0x822b04cb,
	0x822c006a,0x822d00d9,0x822f1983,0x82300897,0x8231019c,0x823319b8,0x82341880,0x82350465,
	0x8236015e,0x823714f2,0x823805cc,0x823902aa,0x823b0c21,0x823e14a0,0x82441143,0x82471360,
	0x82491418,0x824b0cff,0x824f11d4,0x82581263,0x825a01a4,0x825f0271,0x82680cfb,0x826e05d7,
	0x826f0b7e,0x82700867,0x827210f6,0x8273165f,0x82740539,0x8279003d,0x827a16ef,0x827d101a,
	0x827e0010,0x827f0dad,0x828208f0,0x828413cb,0x82880d40,0x828a0f7b,0x828b17e7,0x828d1146,
	0x828e1596,0x828f042e,0x82910f5f,0x82920cad,0x8297150e,0x82980eb4,0x82990524,0x829c145b,
	0x829d1934,0x829f1116,0x82a10f9b,0x82a40a48,0x82a50906,0x82a60c1a,0x82a807e0,0x82a90fe1,
	0x82aa0f42,0x82ab163e,0x82ac04ed,0x82ad004b,0x82ae10c8,0x82af156f,0x82b00822,0x82b1074a,
	0x82b304c0,0x82b41476,0x82b71952,0x82b81839,0x82b90fe2,0x82bd1619,0x82be04e9,0x82c102e3,
	0x82c40114,0x82c7140e,0x82c80b45,0x82ca0478,0x82cb1502,0x82cc01f4,0x82cd019b,0x82ce19c6,
	0x82cf126b,0x82d1181e,0x82d21075,0x82d30bca,0x82d412c2,0x82d51147,0x82d70d31,0x82d81002,
	0x82db0a18,0x82dc0d96,0x82de008e,0x82df0601,0x82e00d4a,0x82e116e4,0x82e30997,0x82e40eea,
	0x82e510d0,0x82e60a56,0x82eb1120,0x82ef00c2,0x82f11760,0x82f4097c,0x82f70581,0x82f90efc,
	0x82fb052a,0x830119ff,0x83020cc5,0x830304b8,0x83040fd2,0x83050cb9,0x83060cbf,0x83070051,
	0x830802d4,0x83090d74,0x830c0255,0x830e0935,0x830f0bf9,0x83110dfb,0x83141751,0x83151010,
	0x83170d5a,0x831a1741,0x831b05d8,0x831c0f9c,0x8327087b,0x832802d5,0x832b0caf,0x832c01b8,
	0x832d08bf,0x832f052f,0x833119aa,0x833308a7,0x8334079e,0x83351725,0x833601b9,0x8338109e,
	0x833910b0,0x833a026e,0x833c1367,0x83401601,0x83431052,0x83460936,0x83471591,0x834901a6,
	0x834f108b,0x83500893,0x835116d1,0x8352077a,0x83540b4f,0x835a084e,0x835b107e,0x835c00e3,
	0x835e0fc0,0x835f07aa,0x8360083b,0x8361036a,0x8363109f,0x836407b9,0x8365158a,0x83660c67,
	0x83671753,0x83680f8a,0x83690929,0x836a1297,0x836b1726,0x836c0c93,0x836d070a,0x836e199f,
	0x836f169e,0x837706e4,0x837800cf,0x837b039c,0x837c1382,0x837d1285,0x83850b56,0x83860f1b,
	0x83890b39,0x838e1103,0x8392098e,0x83930cd5,0x8398116d,0x839b1357,0x839c1791,0x839e13d3,
	0x83a0179d,0x83a80ae2,0x83a90533,0x83aa046f,0x83ab0d77,0x83b009ff,0x83b10ab8,0x83b20b69,
	0x83b311ac,0x83b41441,0x83b614e5,0x83b707cf,0x83b81792,0x83b91754,0x83ba1744,0x83bc02c5,
	0x83bd0cb2,0x83c013d8,0x83c10939,0x83c5086b,0x83c70614,0x83ca0988,0x83cc09d8,0x83cf06e7,
	0x83d4053a,0x83d601ef,0x83d81252,0x83dc018b,0x83dd0052,0x83df138b,0x83e00153,0x83e106b6,
	0x83e5149a,0x83e90f1c,0x83ea036c,0x83f00615,0x83f10bd6,0x83f204d2,0x83f81632,0x83f91a44,
	0x83fd11e8,0x84010f4d,0x8403030c,0x8404130b,0x840600ea,0x840b0f35,0x840c0cf4,0x840d0f00,
	0x840e1415,0x840f035e,0x84110763,0x84180db2,0x841c134c,0x841d0c58,0x84241755,0x84251756,
	0x84261757,0x84271533,0x842810da,0x843115d0,0x843817c4,0x843c0481,0x843d0c6a,0x84460099,
	0x84510507,0x845719cf,0x84591514,0x845a1176,0x845b05c2,0x845c0f73,0x84610f1d,0x84630405,
	0x84690e4d,0x846b0731,0x846c1868,0x846d0848,0x847102e5,0x847313f4,0x84750a88,0x8476135a,
	0x847814bf,0x847a0f6e,0x848203b3,0x848701e5,0x848809f2,0x84890a96,0x848b08b1,0x848c0c0a,
	0x848e0e5d,0x84970ae5,0x84990cf2,0x849c1282,0x84a10088,0x84af0a62,0x84b20f1e,0x84b4122a,
	0x84b81925,0x84b90872,0x84ba080b,0x84bd048a,0x84bf06c3,0x84c11907,0x84c415c9,0x84c910a3,
	0x84ca143b,0x84cd1196,0x84d010c1,0x84d112a5,0x84d300b8,0x84d600f0,0x84dd0ac7,0x84df0831,
	0x84e00b21,0x84e317fe,0x84e5175a,0x84e60d7a,0x84ec0e9f,0x84f014c1,0x84fc0b9b,0x84ff15ca,
	0x850c127d,0x85110d44,0x85130ca8,0x851718f9,0x851a1427,0x851f02f7,0x8521018c,0x852b0dec,
	0x852c11ee,0x85370faf,0x85380414,0x85390b76,0x853a0bc3,0x853b0a4f,0x853c000e,0x853d00f6,
	0x854304ae,0x8548160d,0x854908c8,0x854a10c7,0x8556103d,0x855907b6,0x855e1a56,0x856410c6,
	0x856809cb,0x85720f57,0x8574184b,0x8579143d,0x857a080e,0x857b070e,0x857e0b0a,0x85840166,
	0x858506c5,0x858713f7,0x858f171a,0x859b15e7,0x859c00fa,0x85a41566,0x85a80703,0x85aa1576,
	0x85ae1268,0x85af11f9,0x85b015f6,0x85b710b6,0x85b912c6,0x85c105ad,0x85c9090e,0x85cf019d,
	0x85d00d3b,0x85d314fe,0x85d50e48,0x85dc0b28,0x85e41318,0x85e904a8,0x85fb1870,0x85ff07d5,
	0x860506fe,0x86110d6e,0x86160e0c,0x86271040,0x862904b2,0x863818c2,0x863c0d11,0x864d0720,
	0x864e0739,0x864f0c25,0x86500e30,0x86510c7e,0x86540f8c,0x865a15b0,0x865e17cf,0x8662068c,
	0x866b0272,0x866c101e,0x866e0812,0x86711191,0x8679070b,0x867a07a0,0x867b0cf3,0x867c05cf,
	0x867d1284,0x867e14cc,0x867f01ce,0x868011a1,0x868116e6,0x86820c8a,0x868a142f,0x868b10ca,
	0x868c0081,0x868d0eb8,0x8693173c,0x86950191,0x869c161b,0x869d06c6,0x86a305ed,0x86a4186e,
	0x86a7090c,0x86a80534,0x86a90248,0x86aa041a,0x86ac14fb,0x86af1017,0x86b01794,0x86b118a5,
	0x86b417aa,0x86b50a16,0x86b606a1,0x86ba1070,0x86c019d0,0x86c40616,0x86c61032,0x86c71156,
	0x86c90bd7,0x86ca0627,0x86cb035f,0x86ce0b5b,0x86cf022f,0x86d01034,0x86d10d83,0x86d4079f,
	0x86d8167c,0x86d913bb,0x86db19b0,0x86de0aa7,0x86df08c4,0x86e405c3,0x86e91013,0x86ed1976,
	0x86ee0c9a,0x86f018ed,0x86f10852,0x86f20dc8,0x86f31236,0x86f40f53,0x86f8152a,0x86f91781,
	0x86fe0472,0x870011f8,0x87020509,0x8703117d,0x870718ea,0x8708145e,0x8709053e,0x870a0b3a,
	0x870d028f,0x87121647,0x8713135b,0x87151396,0x87171443,0x8718193e,0x871a04d4,0x871c0d21,
	0x871e0f56,0x87210ab2,0x87220d00,0x87230fa9,0x872514a7,0x87291344,0x872e1801,0x87310ebf,
	0x87341714,0x87371058,0x873b0ffa,0x873e0690,0x873f13c7,0x8747175c,0x87480688,0x874901db,
	0x874c0a26,0x874e154a,0x875317d3,0x87570789,0x87590107,0x87600541,0x87631798,0x87641028,
	0x87650cbd,0x876e0565,0x87700a8c,0x87740736,0x877603e8,0x877b0dbc,0x877c0c0d,0x877d02c2,
	0x877e10a6,0x87820ae0,0x87830e74,0x878514ae,0x8788181c,0x878b1264,0x878d10a7,0x87930fec,
	0x879712f3,0x879f0d60,0x87a80ca1,0x87ab11cd,0x87ac01a5,0x87ad024f,0x87af0037,0x87b312f4,
	0x87b50ede,0x87ba0c5f,0x87bd1986,0x87c01211,0x87c60c8e,0x87ca0cbe,0x87cb14af,0x87d118cc,
	0x87d20cb4,0x87d31529,0x87db0ea4,0x87e00e64,0x87e5077c,0x87ea07b7,0x87ee112f,0x87f9156a,
	0x87fe01de,0x88030c63,0x880a0b71,0x88130d02,0x881510b8,0x881607d6,0x881b0d46,0x88210b2b,
	0x882202c9,0x883209af,0x88390435,0x883c1043,0x884015f0,0x88440e3c,0x8845157c,0x884c1584,
	0x884d164e,0x885414f3,0x885708ed,0x8859161f,0x886106fd,0x88621047,0x886316bd,0x886416bb,
	0x88650173,0x88680127,0x886901c1,0x886b1118,0x886c0228,0x886e067c,0x8870120d,0x88720da7,
	0x88771984,0x887d1096,0x887e0fe0,0x887f0916,0x88811813,0x88820ce7,0x8884003b,0x88850dfc,
	0x88880847,0x888b0343,0x888d0e7e,0x889212df,0x889615a8,0x889c13bf,0x88a20e6c,0x88a40cc9,
	0x88ab00b3,0x88ad14b5,0x88b1053b,0x88b7084c,0x88bc05b8,0x88c10186,0x88c20baa,0x88c519e4,
	0x88c60363,0x88c90a3c,0x88ce023d,0x88d20f12,0x88d41712,0x88d517f9,0x88d81027,0x88d9106e,
	0x88df1106,0x88e20b6b,0x88e30b75,0x88e40a5a,0x88e50884,0x88e800f1,0x88f00459,0x88f10129,
	0x88f3113d,0x88f40e8a,0x88f80c61,0x88f90691,0x88fc132e,0x88fe0983,0x89020641,0x890a010f,
	0x891006ed,0x89120093,0x8913009a,0x891900b9,0x891a0299,0x891b0c79,0x89210328,0x892510c2,
	0x892a1397,0x892b025f,0x89300f88,0x89340aca,0x893618f5,0x89410fb4,0x89441516,0x895e0100,
	0x895f0918,0x896610b7,0x897b0e6d,0x897f1487,0x8981169f,0x89830fed,0x89860567,0x89c1088d,
	0x89c20647,0x89c40662,0x89c50d1b,0x89c611b8,0x89c701d2,0x89c80acf,0x89c909c1,0x89ca082b,
	0x89cb14b6,0x89cc039e,0x89ce17d0,0x89cf060c,0x89d00930,0x89d1104a,0x89d208cc,0x89d609c6,
	0x89da0617,0x89dc1a1b,0x89de1134,0x89e30903,0x89e505f0,0x89e6029f,0x89eb127e,0x89ef197c,
	0x89f30738,0x8a00163f,0x8a070701,0x8a3e1a21,0x8a480b5e,0x8a7918b3,0x8a8917ff,0x8a8a1316,
	0x8a9311cb,0x8b07088b,0x8b261005,0x8b660949,0x8b6c0ed0,0x8ba01639,0x8ba1081b,0x8ba203f7,
	0x8ba30553,0x8ba4108e,0x8ba507d7,0x8ba608f1,0x8ba7070f,0x8ba8130d,0x8ba9107d,0x8baa1123,
	0x8bab0f66,0x8bad1605,0x8bae16f0,0x8baf1606,0x8bb0081c,0x8bb208ae,0x8bb307a7,0x8bb40e40,
	0x8bb50995,0x8bb61624,0x8bb70dd0,0x8bb815b6,0x8bb9046b,0x8bba0c54,0x8bbc1258,0x8bbd050e,
	0x8bbe1159,0x8bbf04c9,0x8bc009bd,0x8bc1192b,0x8bc20620,0x8bc306d6,0x8bc40ef9,0x8bc51a49,
	0x8bc6119d,0x8bc818a0,0x8bc91270,0x8bca190d,0x8bcb03a1,0x8bcc1991,0x8bcd02d2,0x8bce102e,
	0x8bcf18e1,0x8bd116f8,0x8bd216cb,0x8bd30a6f,0x8bd40b07,0x8bd511b9,0x8bd6063f,0x8bd71192,
	0x8bd80810,0x8bd907a3,0x8bda0237,0x8bdb19a7,0x8bdc1168,0x8bdd0755,0x8bde0359,0x8bdf0605,
	0x8be01050,0x8be1066c,0x8be215fc,0x8be316ff,0x8be4192c,0x8be50570,0x8be61519,0x8be701c4,
	0x8be807bf,0x8be915b7,0x8beb090a,0x8bec1455,0x8bed17dc,0x8bee0fcc,0x8bef1478,0x8bf005af,
	0x8bf117a9,0x8bf207ab,0x8bf30a73,0x8bf41223,0x8bf5125a,0x8bf60488,0x8bf71004,0x8bf819ad,
	0x8bf91a3c,0x8bfa0e33,0x8bfb0423,0x8bfc1a03,0x8bfd04dc,0x8bfe0a32,0x8bff1414,0x8c0017c1,
	0x8c01121a,0x8c021173,0x8c0303d8,0x8c0401e2,0x8c050b8a,0x8c0619f5,0x8c07128e,0x8c0812d6,
	0x8c0a1709,0x8c0b0d87,0x8c0c0226,0x8c0d03e1,0x8c0e0790,0x8c0f0899,0x8c101551,0x8c1115f1,
	0x8c1216b7,0x8c131423,0x8c14047d,0x8c1517f4,0x8c1615cd,0x8c1701d6,0x8c181a15,0x8c19001d,
	0x8c1a1661,0x8c1b03af,0x8c1c0d0b,0x8c1d0ed8,0x8c1f0d66,0x8c200366,0x8c211277,0x8c221560,
	0x8c231692,0x8c240087,0x8c2511c7,0x8c260f83,0x8c270d1f,0x8c28091e,0x8c290c9f,0x8c2a18ee,
	0x8c2b0887,0x8c2c0d63,0x8c2d12da,0x8c2e1887,0x8c2f0fc2,0x8c300ac8,0x8c310f28,0x8c3209c9,
	0x8c331668,0x8c340f97,0x8c3518b4,0x8c36022c,0x8c370621,0x8c4107c4,0x8c46041b,0x8c4708a9,
	0x8c49025e,0x8c4c13c8,0x8c5511a7,0x8c5a139c,0x8c611526,0x8c620775,0x8c6a06c9,0x8c6b1802,
	0x8c730139,0x8c78195d,0x8c79009d,0x8c7a01cc,0x8c8203d2,0x8c8515a1,0x8c8906e9,0x8c8a0d7b,
	0x8c8c0ccd,0x8c940eca,0x8c980d80,0x8d1d00ab,0x8d1e18fc,0x8d1f0556,0x8d2105f7,0x8d220185,
	0x8d23187b,0x8d2414ee,0x8d250066,0x8d2618d5,0x8d2707ce,0x8d281963,0x8d2904b9,0x8d2a12cf,
	0x8d2b0eed,0x8d2c010a,0x8d2d0606,0x8d2e19c9,0x8d2f064f,0x8d300499,0x8d310894,0x8d3200bf,
	0x8d3311c0,0x8d34134b,0x8d350675,0x8d360a7b,0x8d370341,0x8d380cc7,0x8d3904e6,0x8d3a06ec,
	0x8d3b16d2,0x8d3c1885,0x8d3d196c,0x8d3e0857,0x8d3f07ae,0x8d401a11,0x8d410bc2,0x8d420c2e,
	0x8d431864,0x8d441a12,0x8d450573,0x8d46092d,0x8d471024,0x8d481919,0x8d490abb,0x8d4a1151,
	0x8d4b0560,0x8d4c042c,0x8d4d07ed,0x8d4e11f1,0x8d4f1139,0x8d5002e0,0x8d5305dc,0x8d540e88,
	0x8d550355,0x8d560abd,0x8d5819f2,0x8d590564,0x8d5a19dd,0x8d5b10df,0x8d5c1882,0x8d5d166b,
	0x8d5e1861,0x8d60188f,0x8d61112e,0x8d62175e,0x8d630592,0x8d640263,0x8d66115d,0x8d670dba,
	0x8d6b06ee,0x8d6d18f4,0x8d701a40,0x8d730962,0x8d74055c,0x8d7518e2,0x8d760589,0x8d770f62,
	0x8d81022a,0x8d840fd4,0x8d850208,0x8d8a1831,0x8d8b1035,0x8d911a1c,0x8d940bab,0x8d9f12fe,
	0x8da3104b,0x8db1185f,0x8db31a45,0x8db40e4b,0x8db5009e,0x8db8044d,0x8dba0519,0x8dbc0881,
	0x8dbe1957,0x8dbf12b0,0x8dc3182f,0x8dc40fa7,0x8dc612c4,0x8dcb0053,0x8dcc03dd,0x8dce13ac,
	0x8dcf0849,0x8dd10e80,0x8dd6194a,0x8dd7051c,0x8dda111c,0x8ddb0168,0x8ddd09a0,0x8dde0b5f,
	0x8ddf05d4,0x8de314fd,0x8de408c5,0x8de80a61,0x8dea0677,0x8deb100b,0x8dec0a90,0x8def0c35,
	0x8df3134a,0x8df5089e,0x8df70fb7,0x8df800f2,0x8df914e7,0x8dfa0467,0x8dfb07ef,0x8dfd0833,
	0x8e0515ec,0x8e090b85,0x8e0a1782,0x8e0c0282,0x8e0f12bc,0x8e1402ca,0x8e1d075b,0x8e1e09a7,
	0x8e1f0258,0x8e22131c,0x8e230165,0x8e29018a,0x8e2a1a34,0x8e2c197d,0x8e2e03be,0x8e2f194c,
	0x8e31045d,0x8e35198a,0x8e3901d0,0x8e3a08a3,0x8e3d0992,0x8e4003e9,0x8e410ed7,0x8e4210ac,
	0x8e441323,0x8e47088c,0x8e480377,0x8e490319,0x8e4a1483,0x8e4b12bd,0x8e510e0a,0x8e520e63,
	0x8e5902fa,0x8e6600cd,0x8e69012f,0x8e6c0387,0x8e6d01b1,0x8e6f04b1,0x8e700293,0x8e72044b,
	0x8e7402fb,0x8e7609cf,0x8e7c0f2b,0x8e7f02ff,0x8e811877,0x8e8519ba,0x8e870292,0x8e8f0bc5,
	0x8e900bac,0x8e9401df,0x8e9c1a4e,0x8e9e156b,0x8eab1165,0x8eac05ee,0x8eaf1033,0x8eb20462,
	0x8eba12fb,0x8ece1421,0x8f660211,0x8f671898,0x8f680669,0x8f6915cb,0x8f6b1093,0x8f6c19db,
	0x8f6d0479,0x8f6e0c53,0x8f6f10c5,0x8f7006ff,0x8f710612,0x8f720a1c,0x8f730c3e,0x8f741996,
	0x8f751956,0x8f761705,0x8f770724,0x8f78190f,0x8f790b50,0x8f7a168f,0x8f7b0ff5,0x8f7c11c3,
	0x8f7d1857,0x8f7e196d,0x8f7f08de,0x8f811054,0x8f820c2f,0x8f8308df,0x8f8418ec,0x8f85054c,
	0x8f860b8b,0x8f870df2,0x8f8800b6,0x8f890798,0x8f8a067e,0x8f8b13e8,0x8f8d02cf,0x8f8e1a1a,
	0x8f8f02f0,0x8f90053f,0x8f91080c,0x8f9311ed,0x8f940e90,0x8f95181a,0x8f9614d8,0x8f9718bc,
	0x8f980c38,0x8f9918f1,0x8f9a0bb7,0x8f9b1570,0x8f9c0618,0x8f9e02d8,0x8f9f0ec9,0x8fa30ab3,
	0x8fa80119,0x8fa9011a,0x8fab011b,0x8fb00222,0x8fb110bc,0x8fb602cd,0x8fb90102,0x8fbd0b8f,
	0x8fbe0329,0x8fc10f7c,0x8fc217ae,0x8fc40f68,0x8fc51608,0x8fc70692,0x8fc80c95,0x8fce1750,
	0x8fd01842,0x8fd10927,0x8fd31625,0x8fd404b4,0x8fd5146f,0x8fd80698,0x8fd918f6,0x8fdb0928,
	0x8fdc181d,0x8fdd1401,0x8fde0b65,0x8fdf0254,0x8fe21341,0x8fe416dd,0x8fe5095c,0x8fe60840,
	0x8fe8033b,0x8fe90492,0x8fea0399,0x8feb0f0c,0x8fed03de,0x8fee187c,0x8ff01200,0x8ff3094d,
	0x8ff70d09,0x8ff800cb,0x8ff90819,0x8ffd19ea,0x90001394,0x9001125b,0x900211c1,0x90031306,
	0x90040e72,0x9005071b,0x90060de6,0x900915da,0x900a160a,0x900b016d,0x900d1532,0x900f137b,
	0x901019b7,0x90111022,0x901203ad,0x90141383,0x9016132b,0x9017041c,0x901a1361,0x901b065d,
	0x901d11c4,0x901e0243,0x901f1274,0x90201874,0x9021106d,0x9022050c,0x90260b31,0x902d0773,
	0x902e0344,0x902f0c31,0x90350a84,0x903613f2,0x9038170d,0x903b0c59,0x903c00ce,0x903e17cb,
	0x90410455,0x9042128f,0x904402ab,0x904717fa,0x904d0118,0x904f0482,0x905014d5,0x90510784,
	0x90521026,0x9053037c,0x905716d7,0x9058060b,0x905b0bf2,0x906212ba,0x90630f96,0x90651695,
	0x90680034,0x906d1869,0x906e18e8,0x90740bb6,0x90751a59,0x907d09a8,0x907f00fb,0x9080168a,
	0x90821567,0x90831294,0x90880d3c,0x908b0aac,0x909116f9,0x9093038a,0x9095176e,0x909706a5,
	0x90990cab,0x909b100d,0x909d0a75,0x90a104be,0x90a21585,0x90a30da3,0x90a6007d,0x90aa154c,
	0x90ac1451,0x90ae178e,0x90af06a7,0x90b012c0,0x90b11015,0x90b30eaa,0x90b40144,0x90b5114b,
	0x90b600ad,0x90b803a2,0x90b91a3a,0x90ba16b1,0x90bb0baf,0x90be19a8,0x90c117eb,0x90c40fdc,
	0x90c51964,0x90c70760,0x90ca08bb,0x90ce0ad9,0x90cf084d,0x90d00a66,0x90d1192d,0x90d31843,
	0x90d7148e,0x90db0530,0x90dc05b0,0x90dd06ce,0x90e109dc,0x90e21761,0x90e60b51,0x90e7183b,
	0x90e8017b,0x90eb0eb9,0x90ed0684,0x90ef12d7,0x90f4021b,0x90f8034d,0x90fd0412,0x90fe1653,
	0x9102047e,0x910409b8,0x911900da,0x911e1733,0x91221636,0x912318c5,0x912f1129,0x91310f07,
	0x91391a3e,0x91430bdb,0x9146050a,0x9149179c,0x914a03f3,0x914b1021,0x914c1a04,0x914d0e8f,
	0x914e19a1,0x914f16de,0x91500585,0x9152096c,0x915715c4,0x915a04ef,0x915d1846,0x915e12cd,
	0x916113ad,0x916202f6,0x916306a2,0x91640619,0x9165126c,0x91690d61,0x916a0af6,0x916c0281,
	0x916e136c,0x916f1959,0x917014e8,0x917108b7,0x9172023f,0x91741385,0x917508e2,0x91760cdd,
	0x91770a5b,0x91781281,0x91790b0f,0x917d1667,0x917e1197,0x917f0df9,0x91850e85,0x918702c8,
	0x91891a57,0x918b02f8,0x918c0a9f,0x918d1324,0x91900737,0x919115ba,0x9192158b,0x919a0d0c,
	0x919b1059,0x91a2069b,0x91a312f5,0x91aa0aec,0x91ad0170,0x91ae08e4,0x91af14b0,0x91b40b36,
	0x91b509a9,0x91ba15f8,0x91c70187,0x91c917ab,0x91ca11c8,0x91cc0b2e,0x91cd198d,0x91ce16ac,
	0x91cf0b86,0x91d10913,0x91dc054a,0x9274089f,0x928e100c,0x92ae0c47,0x92c81482,0x933e1862,
	0x936a0d88,0x938f0bee,0x93ca0044,0x93d60039,0x943e00bb,0x946b1578,0x94850912,0x9486056b,
	0x948716e2,0x948818fd,0x948903f1,0x948a18da,0x948b0f02,0x948c0b9a,0x948d1388,0x948e0f7f,
	0x948f02b0,0x94901119,0x949204ab,0x949303d7,0x94940ced,0x94950e3a,0x949701c8,0x94990576,
	0x949a017a,0x949b12cb,0x949c099a,0x949d0453,0x949e0207,0x949f1982,0x94a00da8,0x94a100b0,
	0x94a20598,0x94a30074,0x94a40f8b,0x94a516a3,0x94a60fdf,0x94a709d6,0x94a81456,0x94a905fb,
	0x94aa0a0d,0x94ab04c2,0x94ac07ca,0x94ad0418,0x94ae0e1e,0x94af0056,0x94b017f0,0x94b10f8d,
	0x94b21921,0x94b30f8e,0x94b40626,0x94b50151,0x94b60a1e,0x94b70f0a,0x94b8016e,0x94b9015c,
	0x94ba182d,0x94bb1a4f,0x94bc0d97,0x94bd12e0,0x94be0858,0x94bf03c7,0x94c01793,0x94c1134d,
	0x94c2015d,0x94c30bcf,0x94c41227,0x94c50f82,0x94c60cc4,0x94c811c5,0x94c915e0,0x94ca12b1,
	0x94cb00e7,0x94cc0ddd,0x94cd0eb2,0x94ce045c,0x94d00a13,0x94d10af1,0x94d20496,0x94d5179e,
	0x94d6023b,0x94d70851,0x94d816a9,0x94d90dc6,0x94db0361,0x94dc136a,0x94dd0c75,0x94de03da,
	0x94df172a,0x94e009f0,0x94e1189a,0x94e219af,0x94e314bd,0x94e4135f,0x94e503fe,0x94e70750,
	0x94e81056,0x94e91104,0x94ea0693,0x94eb03db,0x94ec05d1,0x94ed0d5c,0x94ee1923,0x94ef10f9,
	0x94f008d6,0x94f116c1,0x94f201e3,0x94f30275,0x94f412fc,0x94f50022,0x94f61732,0x94f710b1,
	0x94f819d2,0x94f90aeb,0x94fa0f18,0x94fc0ab9,0x94fd1312,0x94fe0b7b,0x94ff0a3f,0x95001535,
	0x950112ab,0x95020b33,0x9503188d,0x9504028e,0x95050687,0x950605b1,0x95070470,0x950815a9,
	0x95090322,0x950a0c4c,0x950b0508,0x950c1573,0x950d0bf1,0x950e09eb,0x950f0885,0x951010cb,
	0x9511131b,0x95120adf,0x95130fee,0x95140982,0x95150002,0x95160faa,0x951718f3,0x95180e36,
	0x95190323,0x951a0cbb,0x951b00c0,0x951d0380,0x951e0a37,0x951f0a9d,0x952114a3,0x95220635,
	0x95230c5b,0x952402bf,0x952519ed,0x9526091f,0x952814e9,0x952909b0,0x952a07cc,0x952b0e89,
	0x952c12d9,0x952d03fc,0x952e08a0,0x952f09a3,0x95300cfe,0x95311a1d,0x95320fdb,0x953409f4,
	0x95350fab,0x95361238,0x95370484,0x953801b6,0x95390fba,0x953a1985,0x953b043d,0x953c1262,
	0x953e0764,0x953f0008,0x95400434,0x95410ce4,0x95420c15,0x954404e8,0x95450cde,0x95460d7e,
	0x9547191a,0x954905c9,0x954a0e07,0x954c09ae,0x954d0e08,0x954e0da1,0x954f0bed,0x955005ac,
	0x9551008a,0x95521717,0x9553084b,0x95540137,0x95560123,0x955712e8,0x95580caa,0x95590c5e,
	0x955b1774,0x955c0959,0x955d0396,0x955e1a48,0x955f15e4,0x95611579,0x956209ce,0x95630b9d,
	0x95640f21,0x95650c29,0x95660446,0x95670acd,0x95680f2a,0x956902fe,0x956a0fb1,0x956b038f,
	0x956c07d3,0x956d0b02,0x956f1a0a,0x95700b70,0x9571171f,0x957201c0,0x95730126,0x95761518,
	0x957f01f2,0x95e80ceb,0x95e91212,0x95ea1121,0x95eb163b,0x95ed00de,0x95ee1436,0x95ef02b5,
	0x95f010ce,0x95f11402,0x95f214ec,0x95f30707,0x95f40865,0x95f50d4f,0x95f60a0b,0x95f70cef,
	0x95f81899,0x95f90dcd,0x95fa0664,0x95fb142e,0x95fc12b9,0x95fd0d52,0x95fe0c6f,0x9600049f,
	0x960105be,0x960206e1,0x96030aa3,0x96040963,0x9605182e,0x96060ae3,0x960817f5,0x96091633,
	0x960a01f0,0x960b14c7,0x960c1430,0x960d07bb,0x960e1645,0x960f047f,0x961001e4,0x96110ac6,
	0x96121049,0x96140aa8,0x96151069,0x961606ea,0x96171337,0x9619106a,0x961a0a02,0x961c0559,
	0x961d0551,0x961f0440,0x96210f79,0x96221474,0x962a0070,0x962e10c3,0x96310942,0x963204c3,
	0x96331674,0x96341722,0x96351915,0x963608e5,0x963b1a4a,0x963c1a63,0x963d03c2,0x963f0000,
	0x964013a6,0x964200a3,0x96440557,0x96450823,0x96460c2c,0x96470c03,0x96480223,0x96491587,
	0x964b0c12,0x964c0d75,0x964d08b4,0x96501505,0x96540571,0x96551122,0x965b00e4,0x965f1967,
	0x96610419,0x96621820,0x9664028b,0x96670e01,0x9668183f,0x966914f9,0x966a0e86,0x966c1a3d,
	0x967202bb,0x96740eb1,0x96750bd0,0x96761308,0x96771507,0x968517c5,0x96860c00,0x968813f3,
	0x968b1289,0x968d077f,0x968f128a,0x9690173d,0x969405c4,0x96970a8e,0x96980013,0x969914c9,
	0x969c18d6,0x96a71291,0x96b014b8,0x96b3079b,0x96b60b4a,0x96b919e9,0x96bc129b,0x96bd09b6,
	0x96be0db7,0x96c01067,0x96c11665,0x96c41599,0x96c51622,0x96c60809,0x96c70633,0x96c9197a,
	0x96cc02da,0x96cd1770,0x96ce0984,0x96cf0290,0x96d20c6d,0x96d503d4,0x96e00283,0x96e817d9,
	0x96e917c6,0x96ea15ee,0x96ef1431,0x96f30b60,0x96f60bd8,0x96f70afe,0x96f90092,0x96fe147f,
	0x970015b2,0x97010834,0x97041538,0x9706135c,0x9707191b,0x97080e91,0x97090ce0,0x970d07d2,
	0x970e110e,0x970f04d5,0x97130ddf,0x97160bb8,0x971c1217,0x971e14d9,0x972a1736,0x972d000f,
	0x9730150c,0x97320c2b,0x9738005c,0x97390eb0,0x973e0c91,0x97520ff3,0x97530954,0x97560955,
	0x97590958,0x975b03cc,0x975e04cf,0x97600a15,0x97610d10,0x97620d2f,0x976516b9,0x976905bf,
	0x9773092f,0x977415e6,0x97760057,0x977c032e,0x97851671,0x978b1556,0x978d001f,0x9791032f,
	0x97920fc4,0x97940c9d,0x97980fd0,0x97a00985,0x97a310ad,0x97ab0986,0x97ad0109,0x97af0875,
	0x97b205fe,0x97b400ba,0x97e613fc,0x97e71094,0x97e906ad,0x97ea1419,0x97eb1848,0x97ec1303,
	0x97ed096b,0x97f31727,0x97f51849,0x97f61148,0x987516b0,0x987603f5,0x98771003,0x987806a0,
	0x98791525,0x987a1220,0x987b15af,0x987c15bb,0x987d13ce,0x987e062f,0x987f0454,0x98800f49,
	0x9881006b,0x9882125c,0x988306c1,0x988417f1,0x98850c22,0x98860bdf,0x98870f05,0x98880945,
	0x988908fa,0x988a0853,0x988c06e8,0x988d1762,0x988f0a20,0x989016d8,0x98910eef,0x98931392,
	0x989406b7,0x98961763,0x98970a23,0x98981322,0x989a0474,0x989b19da,0x989c1648,0x989d0473,
	0x989e0e09,0x989f0c99,0x98a003b8,0x98a110ec,0x98a206d4,0x98a401e9,0x98a510b9,0x98a60ef0,
	0x98a7105b,0x98ce04ff,0x98d1011e,0x98d210d8,0x98d309a2,0x98d51261,0x98d80edd,0x98d90124,
	0x98da0125,0x98de04cd,0x98df11a2,0x98e71298,0x98e81520,0x990d1669,0x9910018f,0x992e134f,
	0x99541777,0x99551304,0x9963119a,0x996507da,0x99671582,0x9968139b,0x996914c5,0x996a1095,
	0x996b17e9,0x996c0264,0x996d04b7,0x996e173b,0x996f0890,0x997011ba,0x99710095,0x99721247,
	0x997416ce,0x99750494,0x9976107f,0x9977151f,0x997a08d2,0x997c0148,0x997d0152,0x997f047c,
	0x998017c2,0x99810dd2,0x998407bd,0x99851508,0x9986064d,0x998701b5,0x99880a97,0x998a1260,
	0x998b01d9,0x998d0d68,0x998f0bf3,0x999015a2,0x99910921,0x99920c9b,0x999310e5,0x999419e0,
	0x99950dbf,0x999611d3,0x99970a85,0x9998068d,0x99991510,0x99a50568,0x99a81577,0x9a6c0c86,
	0x9a6d17e5,0x9a6e13a4,0x9a6f1604,0x9a700253,0x9a71102f,0x9a730157,0x9a740c6e,0x9a751866,
	0x9a7611aa,0x9a771248,0x9a78055a,0x9a79097d,0x9a7a1a3b,0x9a7b19ca,0x9a7c13a9,0x9a7d0e29,
	0x9a7e085c,0x9a7f1700,0x9a800347,0x9a81152e,0x9a820c8c,0x9a8408c0,0x9a85074f,0x9a860c68,
	0x9a87069d,0x9a880ed5,0x9a8a0b1b,0x9a8b0244,0x9a8c1660,0x9a8f09e0,0x9a900f4e,0x9a910f3a,
	0x9a920a34,0x9a9319eb,0x9a96018e,0x9a970eda,0x9a981977,0x9a9a10ee,0x9a9b147e,0x9a9c003a,
	0x9a9d0bea,0x9a9e0f86,0x9a9f1128,0x9aa00120,0x9aa10c5d,0x9aa202e6,0x9aa301e7,0x9aa419a2,
	0x9aa5083a,0x9aa71517,0x9aa80624,0x9ab0137a,0x9ab1090d,0x9ab603a8,0x9ab70a55,0x9ab80697,
	0x9aba0716,0x9abc05ca,0x9ac000fe,0x9ac10a27,0x9ac20f76,0x9ac50c0e,0x9acb0a6c,0x9acc013e,
	0x9ad10421,0x9ad3128b,0x9ad805a1,0x9adf011f,0x9ae10a9e,0x9ae60cbc,0x9aeb1345,0x9aed1a1f,
	0x9aef1072,0x9af915a3,0x9afb0839,0x9b031a35,0x9b08105a,0x9b0f0966,0x9b13013f,0x9b1f0767,
	0x9b230bad,0x9b2f0203,0x9b3205c1,0x9b3b1805,0x9b3c066e,0x9b410a8a,0x9b4207be,0x9b430054,
	0x9b440f10,0x9b450cea,0x9b471657,0x9b481539,0x9b490b88,0x9b4d13e9,0x9b4f1429,0x9b510250,
	0x9b540d6f,0x9c7c17ba,0x9c7f1796,0x9c810c27,0x9c8204c7,0x9c85005b,0x9c860f01,0x9c870dee,
	0x9c880c23,0x9c8b0563,0x9c8d009f,0x9c8e071e,0x9c9012c5,0x9c910667,0x9c9208ff,0x9c94141a,
	0x9c95048f,0x9c9a0835,0x9c9b08c7,0x9c9c14ea,0x9c9e1522,0x9c9f1603,0x9ca005e3,0x9ca10b24,
	0x9ca20b6d,0x9ca30873,0x9ca40b34,0x9ca511a4,0x9ca61346,0x9ca70681,0x9ca81107,0x9ca90777,
	0x9cab0837,0x9cad0ffb,0x9cae0bda,0x9cb01a3f,0x9cb104d6,0x9cb20aa0,0x9cb301f1,0x9cb40636,
	0x9cb50de0,0x9cb60def,0x9cb703d5,0x9cb80940,0x9cba1198,0x9cbb1a20,0x9cbc04fc,0x9cbd03ea,
	0x9cc310de,0x9cc40487,0x9cc51019,0x9cc60566,0x9cc7078d,0x9cca0108,0x9ccb10f1,0x9ccc0038,
	0x9ccd0f58,0x9cce12b6,0x9ccf064b,0x9cd01698,0x9cd30afc,0x9cd4012a,0x9cd515ef,0x9cd6012c,
	0x9cd70c9e,0x9cd80d56,0x9cd91776,0x9cdc0678,0x9cdd1130,0x9cde0bbb,0x9cdf1a5b,0x9ce20b37,
	0x9e1f0dfa,0x9e200961,0x9e2107e2,0x9e221806,0x9e230d59,0x9e250e44,0x9e261613,0x9e280097,
	0x9e291916,0x9e2a0613,0x9e2b0404,0x9e2c0c20,0x9e2d1615,0x9e2f1670,0x9e310249,0x9e32103b,
	0x9e331809,0x9e3513ab,0x9e361233,0x9e371971,0x9e38048e,0x9e39063a,0x9e3a15a0,0x9e3d05b9,
	0x9e3e0c44,0x9e3f070c,0x9e410161,0x9e420b1e,0x9e4309ad,0x9e440628,0x9e450471,0x9e4617fb,
	0x9e4714f5,0x9e481321,0x9e49146c,0x9e4a106b,0x9e4b0d34,0x9e4c001e,0x9e4e00a8,0x9e4f0ea0,
	0x9e5102c7,0x9e550734,0x9e570485,0x9e58061d,0x9e5a02db,0x9e5b0cdf,0x9e5c1481,0x9e5e16a0,
	0x9e630874,0x9e6406ef,0x9e66174d,0x9e6718fa,0x9e680bf5,0x9e690b99,0x9e6a08ca,0x9e6b0978,
	0x9e6c1804,0x9e6d0c3c,0x9e70174f,0x9e710749,0x9e730654,0x9e7e031d,0x9e7f0c32,0x9e820818,
	0x9e8709da,0x9e8819c1,0x9e8b0d0f,0x9e920f59,0x9e930c3d,0x9e9d1161,0x9e9f0bbc,0x9ea60c96,
	0x9eb41036,0x9eb8051a,0x9ebb0c84,0x9ebd0cd0,0x9ebe0799,0x9ec40780,0x9ec9070d,0x9ecd11f5,
	0x9ece0b25,0x9ecf0df0,0x9ed106f1,0x9ed40f93,0x9ed80d7f,0x9edb0346,0x9edc02a1,0x9edd17a0,
	0x9edf16c6,0x9ee014da,0x9ee21037,0x9ee51001,0x9ee70b29,0x9ee90428,0x9eea0194,0x9eef002a,
	0x9ef91958,0x9efb0542,0x9efc0550,0x9efe0d4d,0x9f0b1816,0x9f0d13af,0x9f0e03f6,0x9f100db3,
	0x9f130629,0x9f17130c,0x9f190ec0,0x9f2011f7,0x9f2204f4,0x9f2c17ac,0x9f2f145f,0x9f3714b2,
	0x9f391658,0x9f3b00d0,0x9f3d1029,0x9f3e06a4,0x9f441895,0x9f500f3f,0x9f5107f3,0x9f7f025c,
	0x9f800229,0x9f830991,0x9f840bd9,0x9f850091,0x9f861343,0x9f871a1e,0x9f881735,0x9f8917e2,
	0x9f8a02d0,0x9f8b1046,0x9f8c144d,0x9f990bf6,0x9f9a05ef,0x9f9b09f8,0x9f9f0661,0x9fa01833,
	0xff011a6b,0xff021a6c,0xff031a6d,0xff041a6e,0xff051a6f,0xff061a70,0xff071a71,0xff081a72,
	0xff091a73,0xff0a1a74,0xff0b1a75,0xff0c1a76,0xff0d1a77,0xff0e1a78,0xff0f1a79,0xff101a7a,
	0xff111a7b,0xff121a7c,0xff131a7d,0xff141a7e,0xff151a7f,0xff161a80,0xff171a81,0xff181a82,
	0xff191a83,0xff1a1a84,0xff1b1a85,0xff1c1a86,0xff1d1a87,0xff1e1a88,0xff1f1a89,0xff201a8a,
	0xff211a8b,0xff221a8c,0xff231a8d,0xff241a8e,0xff251a8f,0xff261a90,0xff271a91,0xff281a92,
	0xff291a93,0xff2a1a94,0xff2b1a95,0xff2c1a96,0xff2d1a97,0xff2e1a98,0xff2f1a99,0xff301a9a,
	0xff311a9b,0xff321a9c,0xff331a9d,0xff341a9e,0xff351a9f,0xff361aa0,0xff371aa1,0xff381aa2,
	0xff391aa3,0xff3a1aa4,0xff3b1aa5,0xff3d1aa6,0xff3e1aa7,0xff3f1aa8,0xff401aa9,0xff411aaa,
	0xff421aab,0xff431aac,0xff441aad,0xff451aae,0xff461aaf,0xff471ab0,0xff481ab1,0xff491ab2,
	0xff4a1ab3,0xff4b1ab4,0xff4c1ab5,0xff4d1ab6,0xff4e1ab7,0xff4f1ab8,0xff501ab9,0xff511aba,
	0xff521abb,0xff531abc,0xff541abd,0xff551abe,0xff561abf,0xff571ac0,0xff581ac1,0xff591ac2,
	0xff5a1ac3,0xff5b1ac4,0xff5c1ac5,0xff5d1ac6,0xff5e1ac7,
	};

static const TCollationKeyTable TheChineseTable = 
	{ TheChineseKey, TheChineseIndex, 6861, NULL, NULL, 0 };

static const TCollationMethod TheChineseMethod =
	{ 0, NULL, &TheChineseTable, 0 };

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
// surrogate
// generated from gb18030_allkeys.txt.
// generated by coltab.exe.


static const TUint32 TheGB18030Key[] = 
	{	 // 133770 keys
	0x4e01001d,0x4e02001d,0x4e03001d,0x4e04001d,0x4e05001d,0x4e06001d,0x4e07001d,0x4e08001d,
	0x4e09001d,0x4e0a001d,0x4e0b001d,0x4e0c001d,0x4e0d001d,0x4e0e001d,0x4e0f001d,0x4e10001d,
	0x4e11001d,0x4e12001d,0x4e13001d,0x4e14001d,0x4e15001d,0x4e16001d,0x4e17001d,0x4e18001d,
	0x4e19001d,0x4e1a001d,0x4e1b001d,0x4e1c001d,0x4e1d001d,0x4e1e001d,0x4e1f001d,0x4e20001d,
	0x4e21001d,0x4e22001d,0x4e23001d,0x4e24001d,0x4e25001d,0x4e26001d,0x4e27001d,0x4e28001d,
	0x4e29001d,0x4e2a001d,0x4e2b001d,0x4e2c001d,0x4e2d001d,0x4e2e001d,0x4e2f001d,0x4e30001d,
	0x4e31001d,0x4e32001d,0x4e33001d,0x4e34001d,0x4e35001d,0x4e36001d,0x4e37001d,0x4e38001d,
	0x4e39001d,0x4e3a001d,0x4e3b001d,0x4e3c001d,0x4e3d001d,0x4e3e001d,0x4e3f001d,0x4e40001d,
	0x4e41001d,0x4e42001d,0x4e43001d,0x4e44001d,0x4e45001d,0x4e46001d,0x4e47001d,0x4e48001d,
	0x4e49001d,0x4e4a001d,0x4e4b001d,0x4e4c001d,0x4e4d001d,0x4e4e001d,0x4e4f001d,0x4e50001d,
	0x4e51001d,0x4e52001d,0x4e53001d,0x4e54001d,0x4e55001d,0x4e56001d,0x4e57001d,0x4e58001d,
	0x4e59001d,0x4e5a001d,0x4e5b001d,0x4e5c001d,0x4e5d001d,0x4e5e001d,0x4e5f001d,0x4e60001d,
	0x4e61001d,0x4e62001d,0x4e63001d,0x4e64001d,0x4e65001d,0x4e66001d,0x4e67001d,0x4e68001d,
	0x4e69001d,0x4e6a001d,0x4e6b001d,0x4e6c001d,0x4e6d001d,0x4e6e001d,0x4e6f001d,0x4e70001d,
	0x4e71001d,0x4e72001d,0x4e73001d,0x4e74001d,0x4e75001d,0x4e76001d,0x4e77001d,0x4e78001d,
	0x4e79001d,0x4e7a001d,0x4e7b001d,0x4e7c001d,0x4e7d001d,0x4e7e001d,0x4e7f001d,0x4e80001d,
	0x4e81001d,0x4e82001d,0x4e83001d,0x4e84001d,0x4e85001d,0x4e86001d,0x4e87001d,0x4e88001d,
	0x4e89001d,0x4e8a001d,0x4e8b001d,0x4e8c001d,0x4e8d001d,0x4e8e001d,0x4e8f001d,0x4e90001d,
	0x4e91001d,0x4e92001d,0x4e93001d,0x4e94001d,0x4e95001d,0x4e96001d,0x4e97001d,0x4e98001d,
	0x4e99001d,0x4e9a001d,0x4e9b001d,0x4e9c001d,0x4e9d001d,0x4e9e001d,0x4e9f001d,0x4ea0001d,
	0x4ea1001d,0x4ea2001d,0x4ea3001d,0x4ea4001d,0x4ea5001d,0x4ea6001d,0x4ea7001d,0x4ea8001d,
	0x4ea9001d,0x4eaa001d,0x4eab001d,0x4eac001d,0x4ead001d,0x4eae001d,0x4eaf001d,0x4eb0001d,
	0x4eb1001d,0x4eb2001d,0x4eb3001d,0x4eb4001d,0x4eb5001d,0x4eb6001d,0x4eb7001d,0x4eb8001d,
	0x4eb9001d,0x4eba001d,0x4ebb001d,0x4ebc001d,0x4ebd001d,0x4ebe001d,0x4ebf001d,0x4ec0001d,
	0x4ec1001d,0x4ec2001d,0x4ec3001d,0x4ec4001d,0x4ec5001d,0x4ec6001d,0x4ec7001d,0x4ec8001d,
	0x4ec9001d,0x4eca001d,0x4ecb001d,0x4ecc001d,0x4ecd001d,0x4ece001d,0x4ecf001d,0x4ed0001d,
	0x4ed1001d,0x4ed2001d,0x4ed3001d,0x4ed4001d,0x4ed5001d,0x4ed6001d,0x4ed7001d,0x4ed8001d,
	0x4ed9001d,0x4eda001d,0x4edb001d,0x4edc001d,0x4edd001d,0x4ede001d,0x4edf001d,0x4ee0001d,
	0x4ee1001d,0x4ee2001d,0x4ee3001d,0x4ee4001d,0x4ee5001d,0x4ee6001d,0x4ee7001d,0x4ee8001d,
	0x4ee9001d,0x4eea001d,0x4eeb001d,0x4eec001d,0x4eed001d,0x4eee001d,0x4eef001d,0x4ef0001d,
	0x4ef1001d,0x4ef2001d,0x4ef3001d,0x4ef4001d,0x4ef5001d,0x4ef6001d,0x4ef7001d,0x4ef8001d,
	0x4ef9001d,0x4efa001d,0x4efb001d,0x4efc001d,0x4efd001d,0x4efe001d,0x4eff001d,0x4f00001d,
	0x4f01001d,0x4f02001d,0x4f03001d,0x4f04001d,0x4f05001d,0x4f06001d,0x4f07001d,0x4f08001d,
	0x4f09001d,0x4f0a001d,0x4f0b001d,0x4f0c001d,0x4f0d001d,0x4f0e001d,0x4f0f001d,0x4f10001d,
	0x4f11001d,0x4f12001d,0x4f13001d,0x4f14001d,0x4f15001d,0x4f16001d,0x4f17001d,0x4f18001d,
	0x4f19001d,0x4f1a001d,0x4f1b001d,0x4f1c001d,0x4f1d001d,0x4f1e001d,0x4f1f001d,0x4f20001d,
	0x4f21001d,0x4f22001d,0x4f23001d,0x4f24001d,0x4f25001d,0x4f26001d,0x4f27001d,0x4f28001d,
	0x4f29001d,0x4f2a001d,0x4f2b001d,0x4f2c001d,0x4f2d001d,0x4f2e001d,0x4f2f001d,0x4f30001d,
	0x4f31001d,0x4f32001d,0x4f33001d,0x4f34001d,0x4f35001d,0x4f36001d,0x4f37001d,0x4f38001d,
	0x4f39001d,0x4f3a001d,0x4f3b001d,0x4f3c001d,0x4f3d001d,0x4f3e001d,0x4f3f001d,0x4f40001d,
	0x4f41001d,0x4f42001d,0x4f43001d,0x4f44001d,0x4f45001d,0x4f46001d,0x4f47001d,0x4f48001d,
	0x4f49001d,0x4f4a001d,0x4f4b001d,0x4f4c001d,0x4f4d001d,0x4f4e001d,0x4f4f001d,0x4f50001d,
	0x4f51001d,0x4f52001d,0x4f53001d,0x4f54001d,0x4f55001d,0x4f56001d,0x4f57001d,0x4f58001d,
	0x4f59001d,0x4f5a001d,0x4f5b001d,0x4f5c001d,0x4f5d001d,0x4f5e001d,0x4f5f001d,0x4f60001d,
	0x4f61001d,0x4f62001d,0x4f63001d,0x4f64001d,0x4f65001d,0x4f66001d,0x4f67001d,0x4f68001d,
	0x4f69001d,0x4f6a001d,0x4f6b001d,0x4f6c001d,0x4f6d001d,0x4f6e001d,0x4f6f001d,0x4f70001d,
	0x4f71001d,0x4f72001d,0x4f73001d,0x4f74001d,0x4f75001d,0x4f76001d,0x4f77001d,0x4f78001d,
	0x4f79001d,0x4f7a001d,0x4f7b001d,0x4f7c001d,0x4f7d001d,0x4f7e001d,0x4f7f001d,0x4f80001d,
	0x4f81001d,0x4f82001d,0x4f83001d,0x4f84001d,0x4f85001d,0x4f86001d,0x4f87001d,0x4f88001d,
	0x4f89001d,0x4f8a001d,0x4f8b001d,0x4f8c001d,0x4f8d001d,0x4f8e001d,0x4f8f001d,0x4f90001d,
	0x4f91001d,0x4f92001d,0x4f93001d,0x4f94001d,0x4f95001d,0x4f96001d,0x4f97001d,0x4f98001d,
	0x4f99001d,0x4f9a001d,0x4f9b001d,0x4f9c001d,0x4f9d001d,0x4f9e001d,0x4f9f001d,0x4fa0001d,
	0x4fa1001d,0x4fa2001d,0x4fa3001d,0x4fa4001d,0x4fa5001d,0x4fa6001d,0x4fa7001d,0x4fa8001d,
	0x4fa9001d,0x4faa001d,0x4fab001d,0x4fac001d,0x4fad001d,0x4fae001d,0x4faf001d,0x4fb0001d,
	0x4fb1001d,0x4fb2001d,0x4fb3001d,0x4fb4001d,0x4fb5001d,0x4fb6001d,0x4fb7001d,0x4fb8001d,
	0x4fb9001d,0x4fba001d,0x4fbb001d,0x4fbc001d,0x4fbd001d,0x4fbe001d,0x4fbf001d,0x4fc0001d,
	0x4fc1001d,0x4fc2001d,0x4fc3001d,0x4fc4001d,0x4fc5001d,0x4fc6001d,0x4fc7001d,0x4fc8001d,
	0x4fc9001d,0x4fca001d,0x4fcb001d,0x4fcc001d,0x4fcd001d,0x4fce001d,0x4fcf001d,0x4fd0001d,
	0x4fd1001d,0x4fd2001d,0x4fd3001d,0x4fd4001d,0x4fd5001d,0x4fd6001d,0x4fd7001d,0x4fd8001d,
	0x4fd9001d,0x4fda001d,0x4fdb001d,0x4fdc001d,0x4fdd001d,0x4fde001d,0x4fdf001d,0x4fe0001d,
	0x4fe1001d,0x4fe2001d,0x4fe3001d,0x4fe4001d,0x4fe5001d,0x4fe6001d,0x4fe7001d,0x4fe8001d,
	0x4fe9001d,0x4fea001d,0x4feb001d,0x4fec001d,0x4fed001d,0x4fee001d,0x4fef001d,0x4ff0001d,
	0x4ff1001d,0x4ff2001d,0x4ff3001d,0x4ff4001d,0x4ff5001d,0x4ff6001d,0x4ff7001d,0x4ff8001d,
	0x4ff9001d,0x4ffa001d,0x4ffb001d,0x4ffc001d,0x4ffd001d,0x4ffe001d,0x4fff001d,0x5000001d,
	0x5001001d,0x5002001d,0x5003001d,0x5004001d,0x5005001d,0x5006001d,0x5007001d,0x5008001d,
	0x5009001d,0x500a001d,0x500b001d,0x500c001d,0x500d001d,0x500e001d,0x500f001d,0x5010001d,
	0x5011001d,0x5012001d,0x5013001d,0x5014001d,0x5015001d,0x5016001d,0x5017001d,0x5018001d,
	0x5019001d,0x501a001d,0x501b001d,0x501c001d,0x501d001d,0x501e001d,0x501f001d,0x5020001d,
	0x5021001d,0x5022001d,0x5023001d,0x5024001d,0x5025001d,0x5026001d,0x5027001d,0x5028001d,
	0x5029001d,0x502a001d,0x502b001d,0x502c001d,0x502d001d,0x502e001d,0x502f001d,0x5030001d,
	0x5031001d,0x5032001d,0x5033001d,0x5034001d,0x5035001d,0x5036001d,0x5037001d,0x5038001d,
	0x5039001d,0x503a001d,0x503b001d,0x503c001d,0x503d001d,0x503e001d,0x503f001d,0x5040001d,
	0x5041001d,0x5042001d,0x5043001d,0x5044001d,0x5045001d,0x5046001d,0x5047001d,0x5048001d,
	0x5049001d,0x504a001d,0x504b001d,0x504c001d,0x504d001d,0x504e001d,0x504f001d,0x5050001d,
	0x5051001d,0x5052001d,0x5053001d,0x5054001d,0x5055001d,0x5056001d,0x5057001d,0x5058001d,
	0x5059001d,0x505a001d,0x505b001d,0x505c001d,0x505d001d,0x505e001d,0x505f001d,0x5060001d,
	0x5061001d,0x5062001d,0x5063001d,0x5064001d,0x5065001d,0x5066001d,0x5067001d,0x5068001d,
	0x5069001d,0x506a001d,0x506b001d,0x506c001d,0x506d001d,0x506e001d,0x506f001d,0x5070001d,
	0x5071001d,0x5072001d,0x5073001d,0x5074001d,0x5075001d,0x5076001d,0x5077001d,0x5078001d,
	0x5079001d,0x507a001d,0x507b001d,0x507c001d,0x507d001d,0x507e001d,0x507f001d,0x5080001d,
	0x5081001d,0x5082001d,0x5083001d,0x5084001d,0x5085001d,0x5086001d,0x5087001d,0x5088001d,
	0x5089001d,0x508a001d,0x508b001d,0x508c001d,0x508d001d,0x508e001d,0x508f001d,0x5090001d,
	0x5091001d,0x5092001d,0x5093001d,0x5094001d,0x5095001d,0x5096001d,0x5097001d,0x5098001d,
	0x5099001d,0x509a001d,0x509b001d,0x509c001d,0x509d001d,0x509e001d,0x509f001d,0x50a0001d,
	0x50a1001d,0x50a2001d,0x50a3001d,0x50a4001d,0x50a5001d,0x50a6001d,0x50a7001d,0x50a8001d,
	0x50a9001d,0x50aa001d,0x50ab001d,0x50ac001d,0x50ad001d,0x50ae001d,0x50af001d,0x50b0001d,
	0x50b1001d,0x50b2001d,0x50b3001d,0x50b4001d,0x50b5001d,0x50b6001d,0x50b7001d,0x50b8001d,
	0x50b9001d,0x50ba001d,0x50bb001d,0x50bc001d,0x50bd001d,0x50be001d,0x50bf001d,0x50c0001d,
	0x50c1001d,0x50c2001d,0x50c3001d,0x50c4001d,0x50c5001d,0x50c6001d,0x50c7001d,0x50c8001d,
	0x50c9001d,0x50ca001d,0x50cb001d,0x50cc001d,0x50cd001d,0x50ce001d,0x50cf001d,0x50d0001d,
	0x50d1001d,0x50d2001d,0x50d3001d,0x50d4001d,0x50d5001d,0x50d6001d,0x50d7001d,0x50d8001d,
	0x50d9001d,0x50da001d,0x50db001d,0x50dc001d,0x50dd001d,0x50de001d,0x50df001d,0x50e0001d,
	0x50e1001d,0x50e2001d,0x50e3001d,0x50e4001d,0x50e5001d,0x50e6001d,0x50e7001d,0x50e8001d,
	0x50e9001d,0x50ea001d,0x50eb001d,0x50ec001d,0x50ed001d,0x50ee001d,0x50ef001d,0x50f0001d,
	0x50f1001d,0x50f2001d,0x50f3001d,0x50f4001d,0x50f5001d,0x50f6001d,0x50f7001d,0x50f8001d,
	0x50f9001d,0x50fa001d,0x50fb001d,0x50fc001d,0x50fd001d,0x50fe001d,0x50ff001d,0x5100001d,
	0x5101001d,0x5102001d,0x5103001d,0x5104001d,0x5105001d,0x5106001d,0x5107001d,0x5108001d,
	0x5109001d,0x510a001d,0x510b001d,0x510c001d,0x510d001d,0x510e001d,0x510f001d,0x5110001d,
	0x5111001d,0x5112001d,0x5113001d,0x5114001d,0x5115001d,0x5116001d,0x5117001d,0x5118001d,
	0x5119001d,0x511a001d,0x511b001d,0x511c001d,0x511d001d,0x511e001d,0x511f001d,0x5120001d,
	0x5121001d,0x5122001d,0x5123001d,0x5124001d,0x5125001d,0x5126001d,0x5127001d,0x5128001d,
	0x5129001d,0x512a001d,0x512b001d,0x512c001d,0x512d001d,0x512e001d,0x512f001d,0x5130001d,
	0x5131001d,0x5132001d,0x5133001d,0x5134001d,0x5135001d,0x5136001d,0x5137001d,0x5138001d,
	0x5139001d,0x513a001d,0x513b001d,0x513c001d,0x513d001d,0x513e001d,0x513f001d,0x5140001d,
	0x5141001d,0x5142001d,0x5143001d,0x5144001d,0x5145001d,0x5146001d,0x5147001d,0x5148001d,
	0x5149001d,0x514a001d,0x514b001d,0x514c001d,0x514d001d,0x514e001d,0x514f001d,0x5150001d,
	0x5151001d,0x5152001d,0x5153001d,0x5154001d,0x5155001d,0x5156001d,0x5157001d,0x5158001d,
	0x5159001d,0x515a001d,0x515b001d,0x515c001d,0x515d001d,0x515e001d,0x515f001d,0x5160001d,
	0x5161001d,0x5162001d,0x5163001d,0x5164001d,0x5165001d,0x5166001d,0x5167001d,0x5168001d,
	0x5169001d,0x516a001d,0x516b001d,0x516c001d,0x516d001d,0x516e001d,0x516f001d,0x5170001d,
	0x5171001d,0x5172001d,0x5173001d,0x5174001d,0x5175001d,0x5176001d,0x5177001d,0x5178001d,
	0x5179001d,0x517a001d,0x517b001d,0x517c001d,0x517d001d,0x517e001d,0x517f001d,0x5180001d,
	0x5181001d,0x5182001d,0x5183001d,0x5184001d,0x5185001d,0x5186001d,0x5187001d,0x5188001d,
	0x5189001d,0x518a001d,0x518b001d,0x518c001d,0x518d001d,0x518e001d,0x518f001d,0x5190001d,
	0x5191001d,0x5192001d,0x5193001d,0x5194001d,0x5195001d,0x5196001d,0x5197001d,0x5198001d,
	0x5199001d,0x519a001d,0x519b001d,0x519c001d,0x519d001d,0x519e001d,0x519f001d,0x51a0001d,
	0x51a1001d,0x51a2001d,0x51a3001d,0x51a4001d,0x51a5001d,0x51a6001d,0x51a7001d,0x51a8001d,
	0x51a9001d,0x51aa001d,0x51ab001d,0x51ac001d,0x51ad001d,0x51ae001d,0x51af001d,0x51b0001d,
	0x51b1001d,0x51b2001d,0x51b3001d,0x51b4001d,0x51b5001d,0x51b6001d,0x51b7001d,0x51b8001d,
	0x51b9001d,0x51ba001d,0x51bb001d,0x51bc001d,0x51bd001d,0x51be001d,0x51bf001d,0x51c0001d,
	0x51c1001d,0x51c2001d,0x51c3001d,0x51c4001d,0x51c5001d,0x51c6001d,0x51c7001d,0x51c8001d,
	0x51c9001d,0x51ca001d,0x51cb001d,0x51cc001d,0x51cd001d,0x51ce001d,0x51cf001d,0x51d0001d,
	0x51d1001d,0x51d2001d,0x51d3001d,0x51d4001d,0x51d5001d,0x51d6001d,0x51d7001d,0x51d8001d,
	0x51d9001d,0x51da001d,0x51db001d,0x51dc001d,0x51dd001d,0x51de001d,0x51df001d,0x51e0001d,
	0x51e1001d,0x51e2001d,0x51e3001d,0x51e4001d,0x51e5001d,0x51e6001d,0x51e7001d,0x51e8001d,
	0x51e9001d,0x51ea001d,0x51eb001d,0x51ec001d,0x51ed001d,0x51ee001d,0x51ef001d,0x51f0001d,
	0x51f1001d,0x51f2001d,0x51f3001d,0x51f4001d,0x51f5001d,0x51f6001d,0x51f7001d,0x51f8001d,
	0x51f9001d,0x51fa001d,0x51fb001d,0x51fc001d,0x51fd001d,0x51fe001d,0x51ff001d,0x5200001d,
	0x5201001d,0x5202001d,0x5203001d,0x5204001d,0x5205001d,0x5206001d,0x5207001d,0x5208001d,
	0x5209001d,0x520a001d,0x520b001d,0x520c001d,0x520d001d,0x520e001d,0x520f001d,0x5210001d,
	0x5211001d,0x5212001d,0x5213001d,0x5214001d,0x5215001d,0x5216001d,0x5217001d,0x5218001d,
	0x5219001d,0x521a001d,0x521b001d,0x521c001d,0x521d001d,0x521e001d,0x521f001d,0x5220001d,
	0x5221001d,0x5222001d,0x5223001d,0x5224001d,0x5225001d,0x5226001d,0x5227001d,0x5228001d,
	0x5229001d,0x522a001d,0x522b001d,0x522c001d,0x522d001d,0x522e001d,0x522f001d,0x5230001d,
	0x5231001d,0x5232001d,0x5233001d,0x5234001d,0x5235001d,0x5236001d,0x5237001d,0x5238001d,
	0x5239001d,0x523a001d,0x523b001d,0x523c001d,0x523d001d,0x523e001d,0x523f001d,0x5240001d,
	0x5241001d,0x5242001d,0x5243001d,0x5244001d,0x5245001d,0x5246001d,0x5247001d,0x5248001d,
	0x5249001d,0x524a001d,0x524b001d,0x524c001d,0x524d001d,0x524e001d,0x524f001d,0x5250001d,
	0x5251001d,0x5252001d,0x5253001d,0x5254001d,0x5255001d,0x5256001d,0x5257001d,0x5258001d,
	0x5259001d,0x525a001d,0x525b001d,0x525c001d,0x525d001d,0x525e001d,0x525f001d,0x5260001d,
	0x5261001d,0x5262001d,0x5263001d,0x5264001d,0x5265001d,0x5266001d,0x5267001d,0x5268001d,
	0x5269001d,0x526a001d,0x526b001d,0x526c001d,0x526d001d,0x526e001d,0x526f001d,0x5270001d,
	0x5271001d,0x5272001d,0x5273001d,0x5274001d,0x5275001d,0x5276001d,0x5277001d,0x5278001d,
	0x5279001d,0x527a001d,0x527b001d,0x527c001d,0x527d001d,0x527e001d,0x527f001d,0x5280001d,
	0x5281001d,0x5282001d,0x5283001d,0x5284001d,0x5285001d,0x5286001d,0x5287001d,0x5288001d,
	0x5289001d,0x528a001d,0x528b001d,0x528c001d,0x528d001d,0x528e001d,0x528f001d,0x5290001d,
	0x5291001d,0x5292001d,0x5293001d,0x5294001d,0x5295001d,0x5296001d,0x5297001d,0x5298001d,
	0x5299001d,0x529a001d,0x529b001d,0x529c001d,0x529d001d,0x529e001d,0x529f001d,0x52a0001d,
	0x52a1001d,0x52a2001d,0x52a3001d,0x52a4001d,0x52a5001d,0x52a6001d,0x52a7001d,0x52a8001d,
	0x52a9001d,0x52aa001d,0x52ab001d,0x52ac001d,0x52ad001d,0x52ae001d,0x52af001d,0x52b0001d,
	0x52b1001d,0x52b2001d,0x52b3001d,0x52b4001d,0x52b5001d,0x52b6001d,0x52b7001d,0x52b8001d,
	0x52b9001d,0x52ba001d,0x52bb001d,0x52bc001d,0x52bd001d,0x52be001d,0x52bf001d,0x52c0001d,
	0x52c1001d,0x52c2001d,0x52c3001d,0x52c4001d,0x52c5001d,0x52c6001d,0x52c7001d,0x52c8001d,
	0x52c9001d,0x52ca001d,0x52cb001d,0x52cc001d,0x52cd001d,0x52ce001d,0x52cf001d,0x52d0001d,
	0x52d1001d,0x52d2001d,0x52d3001d,0x52d4001d,0x52d5001d,0x52d6001d,0x52d7001d,0x52d8001d,
	0x52d9001d,0x52da001d,0x52db001d,0x52dc001d,0x52dd001d,0x52de001d,0x52df001d,0x52e0001d,
	0x52e1001d,0x52e2001d,0x52e3001d,0x52e4001d,0x52e5001d,0x52e6001d,0x52e7001d,0x52e8001d,
	0x52e9001d,0x52ea001d,0x52eb001d,0x52ec001d,0x52ed001d,0x52ee001d,0x52ef001d,0x52f0001d,
	0x52f1001d,0x52f2001d,0x52f3001d,0x52f4001d,0x52f5001d,0x52f6001d,0x52f7001d,0x52f8001d,
	0x52f9001d,0x52fa001d,0x52fb001d,0x52fc001d,0x52fd001d,0x52fe001d,0x52ff001d,0x5300001d,
	0x5301001d,0x5302001d,0x5303001d,0x5304001d,0x5305001d,0x5306001d,0x5307001d,0x5308001d,
	0x5309001d,0x530a001d,0x530b001d,0x530c001d,0x530d001d,0x530e001d,0x530f001d,0x5310001d,
	0x5311001d,0x5312001d,0x5313001d,0x5314001d,0x5315001d,0x5316001d,0x5317001d,0x5318001d,
	0x5319001d,0x531a001d,0x531b001d,0x531c001d,0x531d001d,0x531e001d,0x531f001d,0x5320001d,
	0x5321001d,0x5322001d,0x5323001d,0x5324001d,0x5325001d,0x5326001d,0x5327001d,0x5328001d,
	0x5329001d,0x532a001d,0x532b001d,0x532c001d,0x532d001d,0x532e001d,0x532f001d,0x5330001d,
	0x5331001d,0x5332001d,0x5333001d,0x5334001d,0x5335001d,0x5336001d,0x5337001d,0x5338001d,
	0x5339001d,0x533a001d,0x533b001d,0x533c001d,0x533d001d,0x533e001d,0x533f001d,0x5340001d,
	0x5341001d,0x5342001d,0x5343001d,0x5344001d,0x5345001d,0x5346001d,0x5347001d,0x5348001d,
	0x5349001d,0x534a001d,0x534b001d,0x534c001d,0x534d001d,0x534e001d,0x534f001d,0x5350001d,
	0x5351001d,0x5352001d,0x5353001d,0x5354001d,0x5355001d,0x5356001d,0x5357001d,0x5358001d,
	0x5359001d,0x535a001d,0x535b001d,0x535c001d,0x535d001d,0x535e001d,0x535f001d,0x5360001d,
	0x5361001d,0x5362001d,0x5363001d,0x5364001d,0x5365001d,0x5366001d,0x5367001d,0x5368001d,
	0x5369001d,0x536a001d,0x536b001d,0x536c001d,0x536d001d,0x536e001d,0x536f001d,0x5370001d,
	0x5371001d,0x5372001d,0x5373001d,0x5374001d,0x5375001d,0x5376001d,0x5377001d,0x5378001d,
	0x5379001d,0x537a001d,0x537b001d,0x537c001d,0x537d001d,0x537e001d,0x537f001d,0x5380001d,
	0x5381001d,0x5382001d,0x5383001d,0x5384001d,0x5385001d,0x5386001d,0x5387001d,0x5388001d,
	0x5389001d,0x538a001d,0x538b001d,0x538c001d,0x538d001d,0x538e001d,0x538f001d,0x5390001d,
	0x5391001d,0x5392001d,0x5393001d,0x5394001d,0x5395001d,0x5396001d,0x5397001d,0x5398001d,
	0x5399001d,0x539a001d,0x539b001d,0x539c001d,0x539d001d,0x539e001d,0x539f001d,0x53a0001d,
	0x53a1001d,0x53a2001d,0x53a3001d,0x53a4001d,0x53a5001d,0x53a6001d,0x53a7001d,0x53a8001d,
	0x53a9001d,0x53aa001d,0x53ab001d,0x53ac001d,0x53ad001d,0x53ae001d,0x53af001d,0x53b0001d,
	0x53b1001d,0x53b2001d,0x53b3001d,0x53b4001d,0x53b5001d,0x53b6001d,0x53b7001d,0x53b8001d,
	0x53b9001d,0x53ba001d,0x53bb001d,0x53bc001d,0x53bd001d,0x53be001d,0x53bf001d,0x53c0001d,
	0x53c1001d,0x53c2001d,0x53c3001d,0x53c4001d,0x53c5001d,0x53c6001d,0x53c7001d,0x53c8001d,
	0x53c9001d,0x53ca001d,0x53cb001d,0x53cc001d,0x53cd001d,0x53ce001d,0x53cf001d,0x53d0001d,
	0x53d1001d,0x53d2001d,0x53d3001d,0x53d4001d,0x53d5001d,0x53d6001d,0x53d7001d,0x53d8001d,
	0x53d9001d,0x53da001d,0x53db001d,0x53dc001d,0x53dd001d,0x53de001d,0x53df001d,0x53e0001d,
	0x53e1001d,0x53e2001d,0x53e3001d,0x53e4001d,0x53e5001d,0x53e6001d,0x53e7001d,0x53e8001d,
	0x53e9001d,0x53ea001d,0x53eb001d,0x53ec001d,0x53ed001d,0x53ee001d,0x53ef001d,0x53f0001d,
	0x53f1001d,0x53f2001d,0x53f3001d,0x53f4001d,0x53f5001d,0x53f6001d,0x53f7001d,0x53f8001d,
	0x53f9001d,0x53fa001d,0x53fb001d,0x53fc001d,0x53fd001d,0x53fe001d,0x53ff001d,0x5400001d,
	0x5401001d,0x5402001d,0x5403001d,0x5404001d,0x5405001d,0x5406001d,0x5407001d,0x5408001d,
	0x5409001d,0x540a001d,0x540b001d,0x540c001d,0x540d001d,0x540e001d,0x540f001d,0x5410001d,
	0x5411001d,0x5412001d,0x5413001d,0x5414001d,0x5415001d,0x5416001d,0x5417001d,0x5418001d,
	0x5419001d,0x541a001d,0x541b001d,0x541c001d,0x541d001d,0x541e001d,0x541f001d,0x5420001d,
	0x5421001d,0x5422001d,0x5423001d,0x5424001d,0x5425001d,0x5426001d,0x5427001d,0x5428001d,
	0x5429001d,0x542a001d,0x542b001d,0x542c001d,0x542d001d,0x542e001d,0x542f001d,0x5430001d,
	0x5431001d,0x5432001d,0x5433001d,0x5434001d,0x5435001d,0x5436001d,0x5437001d,0x5438001d,
	0x5439001d,0x543a001d,0x543b001d,0x543c001d,0x543d001d,0x543e001d,0x543f001d,0x5440001d,
	0x5441001d,0x5442001d,0x5443001d,0x5444001d,0x5445001d,0x5446001d,0x5447001d,0x5448001d,
	0x5449001d,0x544a001d,0x544b001d,0x544c001d,0x544d001d,0x544e001d,0x544f001d,0x5450001d,
	0x5451001d,0x5452001d,0x5453001d,0x5454001d,0x5455001d,0x5456001d,0x5457001d,0x5458001d,
	0x5459001d,0x545a001d,0x545b001d,0x545c001d,0x545d001d,0x545e001d,0x545f001d,0x5460001d,
	0x5461001d,0x5462001d,0x5463001d,0x5464001d,0x5465001d,0x5466001d,0x5467001d,0x5468001d,
	0x5469001d,0x546a001d,0x546b001d,0x546c001d,0x546d001d,0x546e001d,0x546f001d,0x5470001d,
	0x5471001d,0x5472001d,0x5473001d,0x5474001d,0x5475001d,0x5476001d,0x5477001d,0x5478001d,
	0x5479001d,0x547a001d,0x547b001d,0x547c001d,0x547d001d,0x547e001d,0x547f001d,0x5480001d,
	0x5481001d,0x5482001d,0x5483001d,0x5484001d,0x5485001d,0x5486001d,0x5487001d,0x5488001d,
	0x5489001d,0x548a001d,0x548b001d,0x548c001d,0x548d001d,0x548e001d,0x548f001d,0x5490001d,
	0x5491001d,0x5492001d,0x5493001d,0x5494001d,0x5495001d,0x5496001d,0x5497001d,0x5498001d,
	0x5499001d,0x549a001d,0x549b001d,0x549c001d,0x549d001d,0x549e001d,0x549f001d,0x54a0001d,
	0x54a1001d,0x54a2001d,0x54a3001d,0x54a4001d,0x54a5001d,0x54a6001d,0x54a7001d,0x54a8001d,
	0x54a9001d,0x54aa001d,0x54ab001d,0x54ac001d,0x54ad001d,0x54ae001d,0x54af001d,0x54b0001d,
	0x54b1001d,0x54b2001d,0x54b3001d,0x54b4001d,0x54b5001d,0x54b6001d,0x54b7001d,0x54b8001d,
	0x54b9001d,0x54ba001d,0x54bb001d,0x54bc001d,0x54bd001d,0x54be001d,0x54bf001d,0x54c0001d,
	0x54c1001d,0x54c2001d,0x54c3001d,0x54c4001d,0x54c5001d,0x54c6001d,0x54c7001d,0x54c8001d,
	0x54c9001d,0x54ca001d,0x54cb001d,0x54cc001d,0x54cd001d,0x54ce001d,0x54cf001d,0x54d0001d,
	0x54d1001d,0x54d2001d,0x54d3001d,0x54d4001d,0x54d5001d,0x54d6001d,0x54d7001d,0x54d8001d,
	0x54d9001d,0x54da001d,0x54db001d,0x54dc001d,0x54dd001d,0x54de001d,0x54df001d,0x54e0001d,
	0x54e1001d,0x54e2001d,0x54e3001d,0x54e4001d,0x54e5001d,0x54e6001d,0x54e7001d,0x54e8001d,
	0x54e9001d,0x54ea001d,0x54eb001d,0x54ec001d,0x54ed001d,0x54ee001d,0x54ef001d,0x54f0001d,
	0x54f1001d,0x54f2001d,0x54f3001d,0x54f4001d,0x54f5001d,0x54f6001d,0x54f7001d,0x54f8001d,
	0x54f9001d,0x54fa001d,0x54fb001d,0x54fc001d,0x54fd001d,0x54fe001d,0x54ff001d,0x5500001d,
	0x5501001d,0x5502001d,0x5503001d,0x5504001d,0x5505001d,0x5506001d,0x5507001d,0x5508001d,
	0x5509001d,0x550a001d,0x550b001d,0x550c001d,0x550d001d,0x550e001d,0x550f001d,0x5510001d,
	0x5511001d,0x5512001d,0x5513001d,0x5514001d,0x5515001d,0x5516001d,0x5517001d,0x5518001d,
	0x5519001d,0x551a001d,0x551b001d,0x551c001d,0x551d001d,0x551e001d,0x551f001d,0x5520001d,
	0x5521001d,0x5522001d,0x5523001d,0x5524001d,0x5525001d,0x5526001d,0x5527001d,0x5528001d,
	0x5529001d,0x552a001d,0x552b001d,0x552c001d,0x552d001d,0x552e001d,0x552f001d,0x5530001d,
	0x5531001d,0x5532001d,0x5533001d,0x5534001d,0x5535001d,0x5536001d,0x5537001d,0x5538001d,
	0x5539001d,0x553a001d,0x553b001d,0x553c001d,0x553d001d,0x553e001d,0x553f001d,0x5540001d,
	0x5541001d,0x5542001d,0x5543001d,0x5544001d,0x5545001d,0x5546001d,0x5547001d,0x5548001d,
	0x5549001d,0x554a001d,0x554b001d,0x554c001d,0x554d001d,0x554e001d,0x554f001d,0x5550001d,
	0x5551001d,0x5552001d,0x5553001d,0x5554001d,0x5555001d,0x5556001d,0x5557001d,0x5558001d,
	0x5559001d,0x555a001d,0x555b001d,0x555c001d,0x555d001d,0x555e001d,0x555f001d,0x5560001d,
	0x5561001d,0x5562001d,0x5563001d,0x5564001d,0x5565001d,0x5566001d,0x5567001d,0x5568001d,
	0x5569001d,0x556a001d,0x556b001d,0x556c001d,0x556d001d,0x556e001d,0x556f001d,0x5570001d,
	0x5571001d,0x5572001d,0x5573001d,0x5574001d,0x5575001d,0x5576001d,0x5577001d,0x5578001d,
	0x5579001d,0x557a001d,0x557b001d,0x557c001d,0x557d001d,0x557e001d,0x557f001d,0x5580001d,
	0x5581001d,0x5582001d,0x5583001d,0x5584001d,0x5585001d,0x5586001d,0x5587001d,0x5588001d,
	0x5589001d,0x558a001d,0x558b001d,0x558c001d,0x558d001d,0x558e001d,0x558f001d,0x5590001d,
	0x5591001d,0x5592001d,0x5593001d,0x5594001d,0x5595001d,0x5596001d,0x5597001d,0x5598001d,
	0x5599001d,0x559a001d,0x559b001d,0x559c001d,0x559d001d,0x559e001d,0x559f001d,0x55a0001d,
	0x55a1001d,0x55a2001d,0x55a3001d,0x55a4001d,0x55a5001d,0x55a6001d,0x55a7001d,0x55a8001d,
	0x55a9001d,0x55aa001d,0x55ab001d,0x55ac001d,0x55ad001d,0x55ae001d,0x55af001d,0x55b0001d,
	0x55b1001d,0x55b2001d,0x55b3001d,0x55b4001d,0x55b5001d,0x55b6001d,0x55b7001d,0x55b8001d,
	0x55b9001d,0x55ba001d,0x55bb001d,0x55bc001d,0x55bd001d,0x55be001d,0x55bf001d,0x55c0001d,
	0x55c1001d,0x55c2001d,0x55c3001d,0x55c4001d,0x55c5001d,0x55c6001d,0x55c7001d,0x55c8001d,
	0x55c9001d,0x55ca001d,0x55cb001d,0x55cc001d,0x55cd001d,0x55ce001d,0x55cf001d,0x55d0001d,
	0x55d1001d,0x55d2001d,0x55d3001d,0x55d4001d,0x55d5001d,0x55d6001d,0x55d7001d,0x55d8001d,
	0x55d9001d,0x55da001d,0x55db001d,0x55dc001d,0x55dd001d,0x55de001d,0x55df001d,0x55e0001d,
	0x55e1001d,0x55e2001d,0x55e3001d,0x55e4001d,0x55e5001d,0x55e6001d,0x55e7001d,0x55e8001d,
	0x55e9001d,0x55ea001d,0x55eb001d,0x55ec001d,0x55ed001d,0x55ee001d,0x55ef001d,0x55f0001d,
	0x55f1001d,0x55f2001d,0x55f3001d,0x55f4001d,0x55f5001d,0x55f6001d,0x55f7001d,0x55f8001d,
	0x55f9001d,0x55fa001d,0x55fb001d,0x55fc001d,0x55fd001d,0x55fe001d,0x55ff001d,0x5600001d,
	0x5601001d,0x5602001d,0x5603001d,0x5604001d,0x5605001d,0x5606001d,0x5607001d,0x5608001d,
	0x5609001d,0x560a001d,0x560b001d,0x560c001d,0x560d001d,0x560e001d,0x560f001d,0x5610001d,
	0x5611001d,0x5612001d,0x5613001d,0x5614001d,0x5615001d,0x5616001d,0x5617001d,0x5618001d,
	0x5619001d,0x561a001d,0x561b001d,0x561c001d,0x561d001d,0x561e001d,0x561f001d,0x5620001d,
	0x5621001d,0x5622001d,0x5623001d,0x5624001d,0x5625001d,0x5626001d,0x5627001d,0x5628001d,
	0x5629001d,0x562a001d,0x562b001d,0x562c001d,0x562d001d,0x562e001d,0x562f001d,0x5630001d,
	0x5631001d,0x5632001d,0x5633001d,0x5634001d,0x5635001d,0x5636001d,0x5637001d,0x5638001d,
	0x5639001d,0x563a001d,0x563b001d,0x563c001d,0x563d001d,0x563e001d,0x563f001d,0x5640001d,
	0x5641001d,0x5642001d,0x5643001d,0x5644001d,0x5645001d,0x5646001d,0x5647001d,0x5648001d,
	0x5649001d,0x564a001d,0x564b001d,0x564c001d,0x564d001d,0x564e001d,0x564f001d,0x5650001d,
	0x5651001d,0x5652001d,0x5653001d,0x5654001d,0x5655001d,0x5656001d,0x5657001d,0x5658001d,
	0x5659001d,0x565a001d,0x565b001d,0x565c001d,0x565d001d,0x565e001d,0x565f001d,0x5660001d,
	0x5661001d,0x5662001d,0x5663001d,0x5664001d,0x5665001d,0x5666001d,0x5667001d,0x5668001d,
	0x5669001d,0x566a001d,0x566b001d,0x566c001d,0x566d001d,0x566e001d,0x566f001d,0x5670001d,
	0x5671001d,0x5672001d,0x5673001d,0x5674001d,0x5675001d,0x5676001d,0x5677001d,0x5678001d,
	0x5679001d,0x567a001d,0x567b001d,0x567c001d,0x567d001d,0x567e001d,0x567f001d,0x5680001d,
	0x5681001d,0x5682001d,0x5683001d,0x5684001d,0x5685001d,0x5686001d,0x5687001d,0x5688001d,
	0x5689001d,0x568a001d,0x568b001d,0x568c001d,0x568d001d,0x568e001d,0x568f001d,0x5690001d,
	0x5691001d,0x5692001d,0x5693001d,0x5694001d,0x5695001d,0x5696001d,0x5697001d,0x5698001d,
	0x5699001d,0x569a001d,0x569b001d,0x569c001d,0x569d001d,0x569e001d,0x569f001d,0x56a0001d,
	0x56a1001d,0x56a2001d,0x56a3001d,0x56a4001d,0x56a5001d,0x56a6001d,0x56a7001d,0x56a8001d,
	0x56a9001d,0x56aa001d,0x56ab001d,0x56ac001d,0x56ad001d,0x56ae001d,0x56af001d,0x56b0001d,
	0x56b1001d,0x56b2001d,0x56b3001d,0x56b4001d,0x56b5001d,0x56b6001d,0x56b7001d,0x56b8001d,
	0x56b9001d,0x56ba001d,0x56bb001d,0x56bc001d,0x56bd001d,0x56be001d,0x56bf001d,0x56c0001d,
	0x56c1001d,0x56c2001d,0x56c3001d,0x56c4001d,0x56c5001d,0x56c6001d,0x56c7001d,0x56c8001d,
	0x56c9001d,0x56ca001d,0x56cb001d,0x56cc001d,0x56cd001d,0x56ce001d,0x56cf001d,0x56d0001d,
	0x56d1001d,0x56d2001d,0x56d3001d,0x56d4001d,0x56d5001d,0x56d6001d,0x56d7001d,0x56d8001d,
	0x56d9001d,0x56da001d,0x56db001d,0x56dc001d,0x56dd001d,0x56de001d,0x56df001d,0x56e0001d,
	0x56e1001d,0x56e2001d,0x56e3001d,0x56e4001d,0x56e5001d,0x56e6001d,0x56e7001d,0x56e8001d,
	0x56e9001d,0x56ea001d,0x56eb001d,0x56ec001d,0x56ed001d,0x56ee001d,0x56ef001d,0x56f0001d,
	0x56f1001d,0x56f2001d,0x56f3001d,0x56f4001d,0x56f5001d,0x56f6001d,0x56f7001d,0x56f8001d,
	0x56f9001d,0x56fa001d,0x56fb001d,0x56fc001d,0x56fd001d,0x56fe001d,0x56ff001d,0x5700001d,
	0x5701001d,0x5702001d,0x5703001d,0x5704001d,0x5705001d,0x5706001d,0x5707001d,0x5708001d,
	0x5709001d,0x570a001d,0x570b001d,0x570c001d,0x570d001d,0x570e001d,0x570f001d,0x5710001d,
	0x5711001d,0x5712001d,0x5713001d,0x5714001d,0x5715001d,0x5716001d,0x5717001d,0x5718001d,
	0x5719001d,0x571a001d,0x571b001d,0x571c001d,0x571d001d,0x571e001d,0x571f001d,0x5720001d,
	0x5721001d,0x5722001d,0x5723001d,0x5724001d,0x5725001d,0x5726001d,0x5727001d,0x5728001d,
	0x5729001d,0x572a001d,0x572b001d,0x572c001d,0x572d001d,0x572e001d,0x572f001d,0x5730001d,
	0x5731001d,0x5732001d,0x5733001d,0x5734001d,0x5735001d,0x5736001d,0x5737001d,0x5738001d,
	0x5739001d,0x573a001d,0x573b001d,0x573c001d,0x573d001d,0x573e001d,0x573f001d,0x5740001d,
	0x5741001d,0x5742001d,0x5743001d,0x5744001d,0x5745001d,0x5746001d,0x5747001d,0x5748001d,
	0x5749001d,0x574a001d,0x574b001d,0x574c001d,0x574d001d,0x574e001d,0x574f001d,0x5750001d,
	0x5751001d,0x5752001d,0x5753001d,0x5754001d,0x5755001d,0x5756001d,0x5757001d,0x5758001d,
	0x5759001d,0x575a001d,0x575b001d,0x575c001d,0x575d001d,0x575e001d,0x575f001d,0x5760001d,
	0x5761001d,0x5762001d,0x5763001d,0x5764001d,0x5765001d,0x5766001d,0x5767001d,0x5768001d,
	0x5769001d,0x576a001d,0x576b001d,0x576c001d,0x576d001d,0x576e001d,0x576f001d,0x5770001d,
	0x5771001d,0x5772001d,0x5773001d,0x5774001d,0x5775001d,0x5776001d,0x5777001d,0x5778001d,
	0x5779001d,0x577a001d,0x577b001d,0x577c001d,0x577d001d,0x577e001d,0x577f001d,0x5780001d,
	0x5781001d,0x5782001d,0x5783001d,0x5784001d,0x5785001d,0x5786001d,0x5787001d,0x5788001d,
	0x5789001d,0x578a001d,0x578b001d,0x578c001d,0x578d001d,0x578e001d,0x578f001d,0x5790001d,
	0x5791001d,0x5792001d,0x5793001d,0x5794001d,0x5795001d,0x5796001d,0x5797001d,0x5798001d,
	0x5799001d,0x579a001d,0x579b001d,0x579c001d,0x579d001d,0x579e001d,0x579f001d,0x57a0001d,
	0x57a1001d,0x57a2001d,0x57a3001d,0x57a4001d,0x57a5001d,0x57a6001d,0x57a7001d,0x57a8001d,
	0x57a9001d,0x57aa001d,0x57ab001d,0x57ac001d,0x57ad001d,0x57ae001d,0x57af001d,0x57b0001d,
	0x57b1001d,0x57b2001d,0x57b3001d,0x57b4001d,0x57b5001d,0x57b6001d,0x57b7001d,0x57b8001d,
	0x57b9001d,0x57ba001d,0x57bb001d,0x57bc001d,0x57bd001d,0x57be001d,0x57bf001d,0x57c0001d,
	0x57c1001d,0x57c2001d,0x57c3001d,0x57c4001d,0x57c5001d,0x57c6001d,0x57c7001d,0x57c8001d,
	0x57c9001d,0x57ca001d,0x57cb001d,0x57cc001d,0x57cd001d,0x57ce001d,0x57cf001d,0x57d0001d,
	0x57d1001d,0x57d2001d,0x57d3001d,0x57d4001d,0x57d5001d,0x57d6001d,0x57d7001d,0x57d8001d,
	0x57d9001d,0x57da001d,0x57db001d,0x57dc001d,0x57dd001d,0x57de001d,0x57df001d,0x57e0001d,
	0x57e1001d,0x57e2001d,0x57e3001d,0x57e4001d,0x57e5001d,0x57e6001d,0x57e7001d,0x57e8001d,
	0x57e9001d,0x57ea001d,0x57eb001d,0x57ec001d,0x57ed001d,0x57ee001d,0x57ef001d,0x57f0001d,
	0x57f1001d,0x57f2001d,0x57f3001d,0x57f4001d,0x57f5001d,0x57f6001d,0x57f7001d,0x57f8001d,
	0x57f9001d,0x57fa001d,0x57fb001d,0x57fc001d,0x57fd001d,0x57fe001d,0x57ff001d,0x5800001d,
	0x5801001d,0x5802001d,0x5803001d,0x5804001d,0x5805001d,0x5806001d,0x5807001d,0x5808001d,
	0x5809001d,0x580a001d,0x580b001d,0x580c001d,0x580d001d,0x580e001d,0x580f001d,0x5810001d,
	0x5811001d,0x5812001d,0x5813001d,0x5814001d,0x5815001d,0x5816001d,0x5817001d,0x5818001d,
	0x5819001d,0x581a001d,0x581b001d,0x581c001d,0x581d001d,0x581e001d,0x581f001d,0x5820001d,
	0x5821001d,0x5822001d,0x5823001d,0x5824001d,0x5825001d,0x5826001d,0x5827001d,0x5828001d,
	0x5829001d,0x582a001d,0x582b001d,0x582c001d,0x582d001d,0x582e001d,0x582f001d,0x5830001d,
	0x5831001d,0x5832001d,0x5833001d,0x5834001d,0x5835001d,0x5836001d,0x5837001d,0x5838001d,
	0x5839001d,0x583a001d,0x583b001d,0x583c001d,0x583d001d,0x583e001d,0x583f001d,0x5840001d,
	0x5841001d,0x5842001d,0x5843001d,0x5844001d,0x5845001d,0x5846001d,0x5847001d,0x5848001d,
	0x5849001d,0x584a001d,0x584b001d,0x584c001d,0x584d001d,0x584e001d,0x584f001d,0x5850001d,
	0x5851001d,0x5852001d,0x5853001d,0x5854001d,0x5855001d,0x5856001d,0x5857001d,0x5858001d,
	0x5859001d,0x585a001d,0x585b001d,0x585c001d,0x585d001d,0x585e001d,0x585f001d,0x5860001d,
	0x5861001d,0x5862001d,0x5863001d,0x5864001d,0x5865001d,0x5866001d,0x5867001d,0x5868001d,
	0x5869001d,0x586a001d,0x586b001d,0x586c001d,0x586d001d,0x586e001d,0x586f001d,0x5870001d,
	0x5871001d,0x5872001d,0x5873001d,0x5874001d,0x5875001d,0x5876001d,0x5877001d,0x5878001d,
	0x5879001d,0x587a001d,0x587b001d,0x587c001d,0x587d001d,0x587e001d,0x587f001d,0x5880001d,
	0x5881001d,0x5882001d,0x5883001d,0x5884001d,0x5885001d,0x5886001d,0x5887001d,0x5888001d,
	0x5889001d,0x588a001d,0x588b001d,0x588c001d,0x588d001d,0x588e001d,0x588f001d,0x5890001d,
	0x5891001d,0x5892001d,0x5893001d,0x5894001d,0x5895001d,0x5896001d,0x5897001d,0x5898001d,
	0x5899001d,0x589a001d,0x589b001d,0x589c001d,0x589d001d,0x589e001d,0x589f001d,0x58a0001d,
	0x58a1001d,0x58a2001d,0x58a3001d,0x58a4001d,0x58a5001d,0x58a6001d,0x58a7001d,0x58a8001d,
	0x58a9001d,0x58aa001d,0x58ab001d,0x58ac001d,0x58ad001d,0x58ae001d,0x58af001d,0x58b0001d,
	0x58b1001d,0x58b2001d,0x58b3001d,0x58b4001d,0x58b5001d,0x58b6001d,0x58b7001d,0x58b8001d,
	0x58b9001d,0x58ba001d,0x58bb001d,0x58bc001d,0x58bd001d,0x58be001d,0x58bf001d,0x58c0001d,
	0x58c1001d,0x58c2001d,0x58c3001d,0x58c4001d,0x58c5001d,0x58c6001d,0x58c7001d,0x58c8001d,
	0x58c9001d,0x58ca001d,0x58cb001d,0x58cc001d,0x58cd001d,0x58ce001d,0x58cf001d,0x58d0001d,
	0x58d1001d,0x58d2001d,0x58d3001d,0x58d4001d,0x58d5001d,0x58d6001d,0x58d7001d,0x58d8001d,
	0x58d9001d,0x58da001d,0x58db001d,0x58dc001d,0x58dd001d,0x58de001d,0x58df001d,0x58e0001d,
	0x58e1001d,0x58e2001d,0x58e3001d,0x58e4001d,0x58e5001d,0x58e6001d,0x58e7001d,0x58e8001d,
	0x58e9001d,0x58ea001d,0x58eb001d,0x58ec001d,0x58ed001d,0x58ee001d,0x58ef001d,0x58f0001d,
	0x58f1001d,0x58f2001d,0x58f3001d,0x58f4001d,0x58f5001d,0x58f6001d,0x58f7001d,0x58f8001d,
	0x58f9001d,0x58fa001d,0x58fb001d,0x58fc001d,0x58fd001d,0x58fe001d,0x58ff001d,0x5900001d,
	0x5901001d,0x5902001d,0x5903001d,0x5904001d,0x5905001d,0x5906001d,0x5907001d,0x5908001d,
	0x5909001d,0x590a001d,0x590b001d,0x590c001d,0x590d001d,0x590e001d,0x590f001d,0x5910001d,
	0x5911001d,0x5912001d,0x5913001d,0x5914001d,0x5915001d,0x5916001d,0x5917001d,0x5918001d,
	0x5919001d,0x591a001d,0x591b001d,0x591c001d,0x591d001d,0x591e001d,0x591f001d,0x5920001d,
	0x5921001d,0x5922001d,0x5923001d,0x5924001d,0x5925001d,0x5926001d,0x5927001d,0x5928001d,
	0x5929001d,0x592a001d,0x592b001d,0x592c001d,0x592d001d,0x592e001d,0x592f001d,0x5930001d,
	0x5931001d,0x5932001d,0x5933001d,0x5934001d,0x5935001d,0x5936001d,0x5937001d,0x5938001d,
	0x5939001d,0x593a001d,0x593b001d,0x593c001d,0x593d001d,0x593e001d,0x593f001d,0x5940001d,
	0x5941001d,0x5942001d,0x5943001d,0x5944001d,0x5945001d,0x5946001d,0x5947001d,0x5948001d,
	0x5949001d,0x594a001d,0x594b001d,0x594c001d,0x594d001d,0x594e001d,0x594f001d,0x5950001d,
	0x5951001d,0x5952001d,0x5953001d,0x5954001d,0x5955001d,0x5956001d,0x5957001d,0x5958001d,
	0x5959001d,0x595a001d,0x595b001d,0x595c001d,0x595d001d,0x595e001d,0x595f001d,0x5960001d,
	0x5961001d,0x5962001d,0x5963001d,0x5964001d,0x5965001d,0x5966001d,0x5967001d,0x5968001d,
	0x5969001d,0x596a001d,0x596b001d,0x596c001d,0x596d001d,0x596e001d,0x596f001d,0x5970001d,
	0x5971001d,0x5972001d,0x5973001d,0x5974001d,0x5975001d,0x5976001d,0x5977001d,0x5978001d,
	0x5979001d,0x597a001d,0x597b001d,0x597c001d,0x597d001d,0x597e001d,0x597f001d,0x5980001d,
	0x5981001d,0x5982001d,0x5983001d,0x5984001d,0x5985001d,0x5986001d,0x5987001d,0x5988001d,
	0x5989001d,0x598a001d,0x598b001d,0x598c001d,0x598d001d,0x598e001d,0x598f001d,0x5990001d,
	0x5991001d,0x5992001d,0x5993001d,0x5994001d,0x5995001d,0x5996001d,0x5997001d,0x5998001d,
	0x5999001d,0x599a001d,0x599b001d,0x599c001d,0x599d001d,0x599e001d,0x599f001d,0x59a0001d,
	0x59a1001d,0x59a2001d,0x59a3001d,0x59a4001d,0x59a5001d,0x59a6001d,0x59a7001d,0x59a8001d,
	0x59a9001d,0x59aa001d,0x59ab001d,0x59ac001d,0x59ad001d,0x59ae001d,0x59af001d,0x59b0001d,
	0x59b1001d,0x59b2001d,0x59b3001d,0x59b4001d,0x59b5001d,0x59b6001d,0x59b7001d,0x59b8001d,
	0x59b9001d,0x59ba001d,0x59bb001d,0x59bc001d,0x59bd001d,0x59be001d,0x59bf001d,0x59c0001d,
	0x59c1001d,0x59c2001d,0x59c3001d,0x59c4001d,0x59c5001d,0x59c6001d,0x59c7001d,0x59c8001d,
	0x59c9001d,0x59ca001d,0x59cb001d,0x59cc001d,0x59cd001d,0x59ce001d,0x59cf001d,0x59d0001d,
	0x59d1001d,0x59d2001d,0x59d3001d,0x59d4001d,0x59d5001d,0x59d6001d,0x59d7001d,0x59d8001d,
	0x59d9001d,0x59da001d,0x59db001d,0x59dc001d,0x59dd001d,0x59de001d,0x59df001d,0x59e0001d,
	0x59e1001d,0x59e2001d,0x59e3001d,0x59e4001d,0x59e5001d,0x59e6001d,0x59e7001d,0x59e8001d,
	0x59e9001d,0x59ea001d,0x59eb001d,0x59ec001d,0x59ed001d,0x59ee001d,0x59ef001d,0x59f0001d,
	0x59f1001d,0x59f2001d,0x59f3001d,0x59f4001d,0x59f5001d,0x59f6001d,0x59f7001d,0x59f8001d,
	0x59f9001d,0x59fa001d,0x59fb001d,0x59fc001d,0x59fd001d,0x59fe001d,0x59ff001d,0x5a00001d,
	0x5a01001d,0x5a02001d,0x5a03001d,0x5a04001d,0x5a05001d,0x5a06001d,0x5a07001d,0x5a08001d,
	0x5a09001d,0x5a0a001d,0x5a0b001d,0x5a0c001d,0x5a0d001d,0x5a0e001d,0x5a0f001d,0x5a10001d,
	0x5a11001d,0x5a12001d,0x5a13001d,0x5a14001d,0x5a15001d,0x5a16001d,0x5a17001d,0x5a18001d,
	0x5a19001d,0x5a1a001d,0x5a1b001d,0x5a1c001d,0x5a1d001d,0x5a1e001d,0x5a1f001d,0x5a20001d,
	0x5a21001d,0x5a22001d,0x5a23001d,0x5a24001d,0x5a25001d,0x5a26001d,0x5a27001d,0x5a28001d,
	0x5a29001d,0x5a2a001d,0x5a2b001d,0x5a2c001d,0x5a2d001d,0x5a2e001d,0x5a2f001d,0x5a30001d,
	0x5a31001d,0x5a32001d,0x5a33001d,0x5a34001d,0x5a35001d,0x5a36001d,0x5a37001d,0x5a38001d,
	0x5a39001d,0x5a3a001d,0x5a3b001d,0x5a3c001d,0x5a3d001d,0x5a3e001d,0x5a3f001d,0x5a40001d,
	0x5a41001d,0x5a42001d,0x5a43001d,0x5a44001d,0x5a45001d,0x5a46001d,0x5a47001d,0x5a48001d,
	0x5a49001d,0x5a4a001d,0x5a4b001d,0x5a4c001d,0x5a4d001d,0x5a4e001d,0x5a4f001d,0x5a50001d,
	0x5a51001d,0x5a52001d,0x5a53001d,0x5a54001d,0x5a55001d,0x5a56001d,0x5a57001d,0x5a58001d,
	0x5a59001d,0x5a5a001d,0x5a5b001d,0x5a5c001d,0x5a5d001d,0x5a5e001d,0x5a5f001d,0x5a60001d,
	0x5a61001d,0x5a62001d,0x5a63001d,0x5a64001d,0x5a65001d,0x5a66001d,0x5a67001d,0x5a68001d,
	0x5a69001d,0x5a6a001d,0x5a6b001d,0x5a6c001d,0x5a6d001d,0x5a6e001d,0x5a6f001d,0x5a70001d,
	0x5a71001d,0x5a72001d,0x5a73001d,0x5a74001d,0x5a75001d,0x5a76001d,0x5a77001d,0x5a78001d,
	0x5a79001d,0x5a7a001d,0x5a7b001d,0x5a7c001d,0x5a7d001d,0x5a7e001d,0x5a7f001d,0x5a80001d,
	0x5a81001d,0x5a82001d,0x5a83001d,0x5a84001d,0x5a85001d,0x5a86001d,0x5a87001d,0x5a88001d,
	0x5a89001d,0x5a8a001d,0x5a8b001d,0x5a8c001d,0x5a8d001d,0x5a8e001d,0x5a8f001d,0x5a90001d,
	0x5a91001d,0x5a92001d,0x5a93001d,0x5a94001d,0x5a95001d,0x5a96001d,0x5a97001d,0x5a98001d,
	0x5a99001d,0x5a9a001d,0x5a9b001d,0x5a9c001d,0x5a9d001d,0x5a9e001d,0x5a9f001d,0x5aa0001d,
	0x5aa1001d,0x5aa2001d,0x5aa3001d,0x5aa4001d,0x5aa5001d,0x5aa6001d,0x5aa7001d,0x5aa8001d,
	0x5aa9001d,0x5aaa001d,0x5aab001d,0x5aac001d,0x5aad001d,0x5aae001d,0x5aaf001d,0x5ab0001d,
	0x5ab1001d,0x5ab2001d,0x5ab3001d,0x5ab4001d,0x5ab5001d,0x5ab6001d,0x5ab7001d,0x5ab8001d,
	0x5ab9001d,0x5aba001d,0x5abb001d,0x5abc001d,0x5abd001d,0x5abe001d,0x5abf001d,0x5ac0001d,
	0x5ac1001d,0x5ac2001d,0x5ac3001d,0x5ac4001d,0x5ac5001d,0x5ac6001d,0x5ac7001d,0x5ac8001d,
	0x5ac9001d,0x5aca001d,0x5acb001d,0x5acc001d,0x5acd001d,0x5ace001d,0x5acf001d,0x5ad0001d,
	0x5ad1001d,0x5ad2001d,0x5ad3001d,0x5ad4001d,0x5ad5001d,0x5ad6001d,0x5ad7001d,0x5ad8001d,
	0x5ad9001d,0x5ada001d,0x5adb001d,0x5adc001d,0x5add001d,0x5ade001d,0x5adf001d,0x5ae0001d,
	0x5ae1001d,0x5ae2001d,0x5ae3001d,0x5ae4001d,0x5ae5001d,0x5ae6001d,0x5ae7001d,0x5ae8001d,
	0x5ae9001d,0x5aea001d,0x5aeb001d,0x5aec001d,0x5aed001d,0x5aee001d,0x5aef001d,0x5af0001d,
	0x5af1001d,0x5af2001d,0x5af3001d,0x5af4001d,0x5af5001d,0x5af6001d,0x5af7001d,0x5af8001d,
	0x5af9001d,0x5afa001d,0x5afb001d,0x5afc001d,0x5afd001d,0x5afe001d,0x5aff001d,0x5b00001d,
	0x5b01001d,0x5b02001d,0x5b03001d,0x5b04001d,0x5b05001d,0x5b06001d,0x5b07001d,0x5b08001d,
	0x5b09001d,0x5b0a001d,0x5b0b001d,0x5b0c001d,0x5b0d001d,0x5b0e001d,0x5b0f001d,0x5b10001d,
	0x5b11001d,0x5b12001d,0x5b13001d,0x5b14001d,0x5b15001d,0x5b16001d,0x5b17001d,0x5b18001d,
	0x5b19001d,0x5b1a001d,0x5b1b001d,0x5b1c001d,0x5b1d001d,0x5b1e001d,0x5b1f001d,0x5b20001d,
	0x5b21001d,0x5b22001d,0x5b23001d,0x5b24001d,0x5b25001d,0x5b26001d,0x5b27001d,0x5b28001d,
	0x5b29001d,0x5b2a001d,0x5b2b001d,0x5b2c001d,0x5b2d001d,0x5b2e001d,0x5b2f001d,0x5b30001d,
	0x5b31001d,0x5b32001d,0x5b33001d,0x5b34001d,0x5b35001d,0x5b36001d,0x5b37001d,0x5b38001d,
	0x5b39001d,0x5b3a001d,0x5b3b001d,0x5b3c001d,0x5b3d001d,0x5b3e001d,0x5b3f001d,0x5b40001d,
	0x5b41001d,0x5b42001d,0x5b43001d,0x5b44001d,0x5b45001d,0x5b46001d,0x5b47001d,0x5b48001d,
	0x5b49001d,0x5b4a001d,0x5b4b001d,0x5b4c001d,0x5b4d001d,0x5b4e001d,0x5b4f001d,0x5b50001d,
	0x5b51001d,0x5b52001d,0x5b53001d,0x5b54001d,0x5b55001d,0x5b56001d,0x5b57001d,0x5b58001d,
	0x5b59001d,0x5b5a001d,0x5b5b001d,0x5b5c001d,0x5b5d001d,0x5b5e001d,0x5b5f001d,0x5b60001d,
	0x5b61001d,0x5b62001d,0x5b63001d,0x5b64001d,0x5b65001d,0x5b66001d,0x5b67001d,0x5b68001d,
	0x5b69001d,0x5b6a001d,0x5b6b001d,0x5b6c001d,0x5b6d001d,0x5b6e001d,0x5b6f001d,0x5b70001d,
	0x5b71001d,0x5b72001d,0x5b73001d,0x5b74001d,0x5b75001d,0x5b76001d,0x5b77001d,0x5b78001d,
	0x5b79001d,0x5b7a001d,0x5b7b001d,0x5b7c001d,0x5b7d001d,0x5b7e001d,0x5b7f001d,0x5b80001d,
	0x5b81001d,0x5b82001d,0x5b83001d,0x5b84001d,0x5b85001d,0x5b86001d,0x5b87001d,0x5b88001d,
	0x5b89001d,0x5b8a001d,0x5b8b001d,0x5b8c001d,0x5b8d001d,0x5b8e001d,0x5b8f001d,0x5b90001d,
	0x5b91001d,0x5b92001d,0x5b93001d,0x5b94001d,0x5b95001d,0x5b96001d,0x5b97001d,0x5b98001d,
	0x5b99001d,0x5b9a001d,0x5b9b001d,0x5b9c001d,0x5b9d001d,0x5b9e001d,0x5b9f001d,0x5ba0001d,
	0x5ba1001d,0x5ba2001d,0x5ba3001d,0x5ba4001d,0x5ba5001d,0x5ba6001d,0x5ba7001d,0x5ba8001d,
	0x5ba9001d,0x5baa001d,0x5bab001d,0x5bac001d,0x5bad001d,0x5bae001d,0x5baf001d,0x5bb0001d,
	0x5bb1001d,0x5bb2001d,0x5bb3001d,0x5bb4001d,0x5bb5001d,0x5bb6001d,0x5bb7001d,0x5bb8001d,
	0x5bb9001d,0x5bba001d,0x5bbb001d,0x5bbc001d,0x5bbd001d,0x5bbe001d,0x5bbf001d,0x5bc0001d,
	0x5bc1001d,0x5bc2001d,0x5bc3001d,0x5bc4001d,0x5bc5001d,0x5bc6001d,0x5bc7001d,0x5bc8001d,
	0x5bc9001d,0x5bca001d,0x5bcb001d,0x5bcc001d,0x5bcd001d,0x5bce001d,0x5bcf001d,0x5bd0001d,
	0x5bd1001d,0x5bd2001d,0x5bd3001d,0x5bd4001d,0x5bd5001d,0x5bd6001d,0x5bd7001d,0x5bd8001d,
	0x5bd9001d,0x5bda001d,0x5bdb001d,0x5bdc001d,0x5bdd001d,0x5bde001d,0x5bdf001d,0x5be0001d,
	0x5be1001d,0x5be2001d,0x5be3001d,0x5be4001d,0x5be5001d,0x5be6001d,0x5be7001d,0x5be8001d,
	0x5be9001d,0x5bea001d,0x5beb001d,0x5bec001d,0x5bed001d,0x5bee001d,0x5bef001d,0x5bf0001d,
	0x5bf1001d,0x5bf2001d,0x5bf3001d,0x5bf4001d,0x5bf5001d,0x5bf6001d,0x5bf7001d,0x5bf8001d,
	0x5bf9001d,0x5bfa001d,0x5bfb001d,0x5bfc001d,0x5bfd001d,0x5bfe001d,0x5bff001d,0x5c00001d,
	0x5c01001d,0x5c02001d,0x5c03001d,0x5c04001d,0x5c05001d,0x5c06001d,0x5c07001d,0x5c08001d,
	0x5c09001d,0x5c0a001d,0x5c0b001d,0x5c0c001d,0x5c0d001d,0x5c0e001d,0x5c0f001d,0x5c10001d,
	0x5c11001d,0x5c12001d,0x5c13001d,0x5c14001d,0x5c15001d,0x5c16001d,0x5c17001d,0x5c18001d,
	0x5c19001d,0x5c1a001d,0x5c1b001d,0x5c1c001d,0x5c1d001d,0x5c1e001d,0x5c1f001d,0x5c20001d,
	0x5c21001d,0x5c22001d,0x5c23001d,0x5c24001d,0x5c25001d,0x5c26001d,0x5c27001d,0x5c28001d,
	0x5c29001d,0x5c2a001d,0x5c2b001d,0x5c2c001d,0x5c2d001d,0x5c2e001d,0x5c2f001d,0x5c30001d,
	0x5c31001d,0x5c32001d,0x5c33001d,0x5c34001d,0x5c35001d,0x5c36001d,0x5c37001d,0x5c38001d,
	0x5c39001d,0x5c3a001d,0x5c3b001d,0x5c3c001d,0x5c3d001d,0x5c3e001d,0x5c3f001d,0x5c40001d,
	0x5c41001d,0x5c42001d,0x5c43001d,0x5c44001d,0x5c45001d,0x5c46001d,0x5c47001d,0x5c48001d,
	0x5c49001d,0x5c4a001d,0x5c4b001d,0x5c4c001d,0x5c4d001d,0x5c4e001d,0x5c4f001d,0x5c50001d,
	0x5c51001d,0x5c52001d,0x5c53001d,0x5c54001d,0x5c55001d,0x5c56001d,0x5c57001d,0x5c58001d,
	0x5c59001d,0x5c5a001d,0x5c5b001d,0x5c5c001d,0x5c5d001d,0x5c5e001d,0x5c5f001d,0x5c60001d,
	0x5c61001d,0x5c62001d,0x5c63001d,0x5c64001d,0x5c65001d,0x5c66001d,0x5c67001d,0x5c68001d,
	0x5c69001d,0x5c6a001d,0x5c6b001d,0x5c6c001d,0x5c6d001d,0x5c6e001d,0x5c6f001d,0x5c70001d,
	0x5c71001d,0x5c72001d,0x5c73001d,0x5c74001d,0x5c75001d,0x5c76001d,0x5c77001d,0x5c78001d,
	0x5c79001d,0x5c7a001d,0x5c7b001d,0x5c7c001d,0x5c7d001d,0x5c7e001d,0x5c7f001d,0x5c80001d,
	0x5c81001d,0x5c82001d,0x5c83001d,0x5c84001d,0x5c85001d,0x5c86001d,0x5c87001d,0x5c88001d,
	0x5c89001d,0x5c8a001d,0x5c8b001d,0x5c8c001d,0x5c8d001d,0x5c8e001d,0x5c8f001d,0x5c90001d,
	0x5c91001d,0x5c92001d,0x5c93001d,0x5c94001d,0x5c95001d,0x5c96001d,0x5c97001d,0x5c98001d,
	0x5c99001d,0x5c9a001d,0x5c9b001d,0x5c9c001d,0x5c9d001d,0x5c9e001d,0x5c9f001d,0x5ca0001d,
	0x5ca1001d,0x5ca2001d,0x5ca3001d,0x5ca4001d,0x5ca5001d,0x5ca6001d,0x5ca7001d,0x5ca8001d,
	0x5ca9001d,0x5caa001d,0x5cab001d,0x5cac001d,0x5cad001d,0x5cae001d,0x5caf001d,0x5cb0001d,
	0x5cb1001d,0x5cb2001d,0x5cb3001d,0x5cb4001d,0x5cb5001d,0x5cb6001d,0x5cb7001d,0x5cb8001d,
	0x5cb9001d,0x5cba001d,0x5cbb001d,0x5cbc001d,0x5cbd001d,0x5cbe001d,0x5cbf001d,0x5cc0001d,
	0x5cc1001d,0x5cc2001d,0x5cc3001d,0x5cc4001d,0x5cc5001d,0x5cc6001d,0x5cc7001d,0x5cc8001d,
	0x5cc9001d,0x5cca001d,0x5ccb001d,0x5ccc001d,0x5ccd001d,0x5cce001d,0x5ccf001d,0x5cd0001d,
	0x5cd1001d,0x5cd2001d,0x5cd3001d,0x5cd4001d,0x5cd5001d,0x5cd6001d,0x5cd7001d,0x5cd8001d,
	0x5cd9001d,0x5cda001d,0x5cdb001d,0x5cdc001d,0x5cdd001d,0x5cde001d,0x5cdf001d,0x5ce0001d,
	0x5ce1001d,0x5ce2001d,0x5ce3001d,0x5ce4001d,0x5ce5001d,0x5ce6001d,0x5ce7001d,0x5ce8001d,
	0x5ce9001d,0x5cea001d,0x5ceb001d,0x5cec001d,0x5ced001d,0x5cee001d,0x5cef001d,0x5cf0001d,
	0x5cf1001d,0x5cf2001d,0x5cf3001d,0x5cf4001d,0x5cf5001d,0x5cf6001d,0x5cf7001d,0x5cf8001d,
	0x5cf9001d,0x5cfa001d,0x5cfb001d,0x5cfc001d,0x5cfd001d,0x5cfe001d,0x5cff001d,0x5d00001d,
	0x5d01001d,0x5d02001d,0x5d03001d,0x5d04001d,0x5d05001d,0x5d06001d,0x5d07001d,0x5d08001d,
	0x5d09001d,0x5d0a001d,0x5d0b001d,0x5d0c001d,0x5d0d001d,0x5d0e001d,0x5d0f001d,0x5d10001d,
	0x5d11001d,0x5d12001d,0x5d13001d,0x5d14001d,0x5d15001d,0x5d16001d,0x5d17001d,0x5d18001d,
	0x5d19001d,0x5d1a001d,0x5d1b001d,0x5d1c001d,0x5d1d001d,0x5d1e001d,0x5d1f001d,0x5d20001d,
	0x5d21001d,0x5d22001d,0x5d23001d,0x5d24001d,0x5d25001d,0x5d26001d,0x5d27001d,0x5d28001d,
	0x5d29001d,0x5d2a001d,0x5d2b001d,0x5d2c001d,0x5d2d001d,0x5d2e001d,0x5d2f001d,0x5d30001d,
	0x5d31001d,0x5d32001d,0x5d33001d,0x5d34001d,0x5d35001d,0x5d36001d,0x5d37001d,0x5d38001d,
	0x5d39001d,0x5d3a001d,0x5d3b001d,0x5d3c001d,0x5d3d001d,0x5d3e001d,0x5d3f001d,0x5d40001d,
	0x5d41001d,0x5d42001d,0x5d43001d,0x5d44001d,0x5d45001d,0x5d46001d,0x5d47001d,0x5d48001d,
	0x5d49001d,0x5d4a001d,0x5d4b001d,0x5d4c001d,0x5d4d001d,0x5d4e001d,0x5d4f001d,0x5d50001d,
	0x5d51001d,0x5d52001d,0x5d53001d,0x5d54001d,0x5d55001d,0x5d56001d,0x5d57001d,0x5d58001d,
	0x5d59001d,0x5d5a001d,0x5d5b001d,0x5d5c001d,0x5d5d001d,0x5d5e001d,0x5d5f001d,0x5d60001d,
	0x5d61001d,0x5d62001d,0x5d63001d,0x5d64001d,0x5d65001d,0x5d66001d,0x5d67001d,0x5d68001d,
	0x5d69001d,0x5d6a001d,0x5d6b001d,0x5d6c001d,0x5d6d001d,0x5d6e001d,0x5d6f001d,0x5d70001d,
	0x5d71001d,0x5d72001d,0x5d73001d,0x5d74001d,0x5d75001d,0x5d76001d,0x5d77001d,0x5d78001d,
	0x5d79001d,0x5d7a001d,0x5d7b001d,0x5d7c001d,0x5d7d001d,0x5d7e001d,0x5d7f001d,0x5d80001d,
	0x5d81001d,0x5d82001d,0x5d83001d,0x5d84001d,0x5d85001d,0x5d86001d,0x5d87001d,0x5d88001d,
	0x5d89001d,0x5d8a001d,0x5d8b001d,0x5d8c001d,0x5d8d001d,0x5d8e001d,0x5d8f001d,0x5d90001d,
	0x5d91001d,0x5d92001d,0x5d93001d,0x5d94001d,0x5d95001d,0x5d96001d,0x5d97001d,0x5d98001d,
	0x5d99001d,0x5d9a001d,0x5d9b001d,0x5d9c001d,0x5d9d001d,0x5d9e001d,0x5d9f001d,0x5da0001d,
	0x5da1001d,0x5da2001d,0x5da3001d,0x5da4001d,0x5da5001d,0x5da6001d,0x5da7001d,0x5da8001d,
	0x5da9001d,0x5daa001d,0x5dab001d,0x5dac001d,0x5dad001d,0x5dae001d,0x5daf001d,0x5db0001d,
	0x5db1001d,0x5db2001d,0x5db3001d,0x5db4001d,0x5db5001d,0x5db6001d,0x5db7001d,0x5db8001d,
	0x5db9001d,0x5dba001d,0x5dbb001d,0x5dbc001d,0x5dbd001d,0x5dbe001d,0x5dbf001d,0x5dc0001d,
	0x5dc1001d,0x5dc2001d,0x5dc3001d,0x5dc4001d,0x5dc5001d,0x5dc6001d,0x5dc7001d,0x5dc8001d,
	0x5dc9001d,0x5dca001d,0x5dcb001d,0x5dcc001d,0x5dcd001d,0x5dce001d,0x5dcf001d,0x5dd0001d,
	0x5dd1001d,0x5dd2001d,0x5dd3001d,0x5dd4001d,0x5dd5001d,0x5dd6001d,0x5dd7001d,0x5dd8001d,
	0x5dd9001d,0x5dda001d,0x5ddb001d,0x5ddc001d,0x5ddd001d,0x5dde001d,0x5ddf001d,0x5de0001d,
	0x5de1001d,0x5de2001d,0x5de3001d,0x5de4001d,0x5de5001d,0x5de6001d,0x5de7001d,0x5de8001d,
	0x5de9001d,0x5dea001d,0x5deb001d,0x5dec001d,0x5ded001d,0x5dee001d,0x5def001d,0x5df0001d,
	0x5df1001d,0x5df2001d,0x5df3001d,0x5df4001d,0x5df5001d,0x5df6001d,0x5df7001d,0x5df8001d,
	0x5df9001d,0x5dfa001d,0x5dfb001d,0x5dfc001d,0x5dfd001d,0x5dfe001d,0x5dff001d,0x5e00001d,
	0x5e01001d,0x5e02001d,0x5e03001d,0x5e04001d,0x5e05001d,0x5e06001d,0x5e07001d,0x5e08001d,
	0x5e09001d,0x5e0a001d,0x5e0b001d,0x5e0c001d,0x5e0d001d,0x5e0e001d,0x5e0f001d,0x5e10001d,
	0x5e11001d,0x5e12001d,0x5e13001d,0x5e14001d,0x5e15001d,0x5e16001d,0x5e17001d,0x5e18001d,
	0x5e19001d,0x5e1a001d,0x5e1b001d,0x5e1c001d,0x5e1d001d,0x5e1e001d,0x5e1f001d,0x5e20001d,
	0x5e21001d,0x5e22001d,0x5e23001d,0x5e24001d,0x5e25001d,0x5e26001d,0x5e27001d,0x5e28001d,
	0x5e29001d,0x5e2a001d,0x5e2b001d,0x5e2c001d,0x5e2d001d,0x5e2e001d,0x5e2f001d,0x5e30001d,
	0x5e31001d,0x5e32001d,0x5e33001d,0x5e34001d,0x5e35001d,0x5e36001d,0x5e37001d,0x5e38001d,
	0x5e39001d,0x5e3a001d,0x5e3b001d,0x5e3c001d,0x5e3d001d,0x5e3e001d,0x5e3f001d,0x5e40001d,
	0x5e41001d,0x5e42001d,0x5e43001d,0x5e44001d,0x5e45001d,0x5e46001d,0x5e47001d,0x5e48001d,
	0x5e49001d,0x5e4a001d,0x5e4b001d,0x5e4c001d,0x5e4d001d,0x5e4e001d,0x5e4f001d,0x5e50001d,
	0x5e51001d,0x5e52001d,0x5e53001d,0x5e54001d,0x5e55001d,0x5e56001d,0x5e57001d,0x5e58001d,
	0x5e59001d,0x5e5a001d,0x5e5b001d,0x5e5c001d,0x5e5d001d,0x5e5e001d,0x5e5f001d,0x5e60001d,
	0x5e61001d,0x5e62001d,0x5e63001d,0x5e64001d,0x5e65001d,0x5e66001d,0x5e67001d,0x5e68001d,
	0x5e69001d,0x5e6a001d,0x5e6b001d,0x5e6c001d,0x5e6d001d,0x5e6e001d,0x5e6f001d,0x5e70001d,
	0x5e71001d,0x5e72001d,0x5e73001d,0x5e74001d,0x5e75001d,0x5e76001d,0x5e77001d,0x5e78001d,
	0x5e79001d,0x5e7a001d,0x5e7b001d,0x5e7c001d,0x5e7d001d,0x5e7e001d,0x5e7f001d,0x5e80001d,
	0x5e81001d,0x5e82001d,0x5e83001d,0x5e84001d,0x5e85001d,0x5e86001d,0x5e87001d,0x5e88001d,
	0x5e89001d,0x5e8a001d,0x5e8b001d,0x5e8c001d,0x5e8d001d,0x5e8e001d,0x5e8f001d,0x5e90001d,
	0x5e91001d,0x5e92001d,0x5e93001d,0x5e94001d,0x5e95001d,0x5e96001d,0x5e97001d,0x5e98001d,
	0x5e99001d,0x5e9a001d,0x5e9b001d,0x5e9c001d,0x5e9d001d,0x5e9e001d,0x5e9f001d,0x5ea0001d,
	0x5ea1001d,0x5ea2001d,0x5ea3001d,0x5ea4001d,0x5ea5001d,0x5ea6001d,0x5ea7001d,0x5ea8001d,
	0x5ea9001d,0x5eaa001d,0x5eab001d,0x5eac001d,0x5ead001d,0x5eae001d,0x5eaf001d,0x5eb0001d,
	0x5eb1001d,0x5eb2001d,0x5eb3001d,0x5eb4001d,0x5eb5001d,0x5eb6001d,0x5eb7001d,0x5eb8001d,
	0x5eb9001d,0x5eba001d,0x5ebb001d,0x5ebc001d,0x5ebd001d,0x5ebe001d,0x5ebf001d,0x5ec0001d,
	0x5ec1001d,0x5ec2001d,0x5ec3001d,0x5ec4001d,0x5ec5001d,0x5ec6001d,0x5ec7001d,0x5ec8001d,
	0x5ec9001d,0x5eca001d,0x5ecb001d,0x5ecc001d,0x5ecd001d,0x5ece001d,0x5ecf001d,0x5ed0001d,
	0x5ed1001d,0x5ed2001d,0x5ed3001d,0x5ed4001d,0x5ed5001d,0x5ed6001d,0x5ed7001d,0x5ed8001d,
	0x5ed9001d,0x5eda001d,0x5edb001d,0x5edc001d,0x5edd001d,0x5ede001d,0x5edf001d,0x5ee0001d,
	0x5ee1001d,0x5ee2001d,0x5ee3001d,0x5ee4001d,0x5ee5001d,0x5ee6001d,0x5ee7001d,0x5ee8001d,
	0x5ee9001d,0x5eea001d,0x5eeb001d,0x5eec001d,0x5eed001d,0x5eee001d,0x5eef001d,0x5ef0001d,
	0x5ef1001d,0x5ef2001d,0x5ef3001d,0x5ef4001d,0x5ef5001d,0x5ef6001d,0x5ef7001d,0x5ef8001d,
	0x5ef9001d,0x5efa001d,0x5efb001d,0x5efc001d,0x5efd001d,0x5efe001d,0x5eff001d,0x5f00001d,
	0x5f01001d,0x5f02001d,0x5f03001d,0x5f04001d,0x5f05001d,0x5f06001d,0x5f07001d,0x5f08001d,
	0x5f09001d,0x5f0a001d,0x5f0b001d,0x5f0c001d,0x5f0d001d,0x5f0e001d,0x5f0f001d,0x5f10001d,
	0x5f11001d,0x5f12001d,0x5f13001d,0x5f14001d,0x5f15001d,0x5f16001d,0x5f17001d,0x5f18001d,
	0x5f19001d,0x5f1a001d,0x5f1b001d,0x5f1c001d,0x5f1d001d,0x5f1e001d,0x5f1f001d,0x5f20001d,
	0x5f21001d,0x5f22001d,0x5f23001d,0x5f24001d,0x5f25001d,0x5f26001d,0x5f27001d,0x5f28001d,
	0x5f29001d,0x5f2a001d,0x5f2b001d,0x5f2c001d,0x5f2d001d,0x5f2e001d,0x5f2f001d,0x5f30001d,
	0x5f31001d,0x5f32001d,0x5f33001d,0x5f34001d,0x5f35001d,0x5f36001d,0x5f37001d,0x5f38001d,
	0x5f39001d,0x5f3a001d,0x5f3b001d,0x5f3c001d,0x5f3d001d,0x5f3e001d,0x5f3f001d,0x5f40001d,
	0x5f41001d,0x5f42001d,0x5f43001d,0x5f44001d,0x5f45001d,0x5f46001d,0x5f47001d,0x5f48001d,
	0x5f49001d,0x5f4a001d,0x5f4b001d,0x5f4c001d,0x5f4d001d,0x5f4e001d,0x5f4f001d,0x5f50001d,
	0x5f51001d,0x5f52001d,0x5f53001d,0x5f54001d,0x5f55001d,0x5f56001d,0x5f57001d,0x5f58001d,
	0x5f59001d,0x5f5a001d,0x5f5b001d,0x5f5c001d,0x5f5d001d,0x5f5e001d,0x5f5f001d,0x5f60001d,
	0x5f61001d,0x5f62001d,0x5f63001d,0x5f64001d,0x5f65001d,0x5f66001d,0x5f67001d,0x5f68001d,
	0x5f69001d,0x5f6a001d,0x5f6b001d,0x5f6c001d,0x5f6d001d,0x5f6e001d,0x5f6f001d,0x5f70001d,
	0x5f71001d,0x5f72001d,0x5f73001d,0x5f74001d,0x5f75001d,0x5f76001d,0x5f77001d,0x5f78001d,
	0x5f79001d,0x5f7a001d,0x5f7b001d,0x5f7c001d,0x5f7d001d,0x5f7e001d,0x5f7f001d,0x5f80001d,
	0x5f81001d,0x5f82001d,0x5f83001d,0x5f84001d,0x5f85001d,0x5f86001d,0x5f87001d,0x5f88001d,
	0x5f89001d,0x5f8a001d,0x5f8b001d,0x5f8c001d,0x5f8d001d,0x5f8e001d,0x5f8f001d,0x5f90001d,
	0x5f91001d,0x5f92001d,0x5f93001d,0x5f94001d,0x5f95001d,0x5f96001d,0x5f97001d,0x5f98001d,
	0x5f99001d,0x5f9a001d,0x5f9b001d,0x5f9c001d,0x5f9d001d,0x5f9e001d,0x5f9f001d,0x5fa0001d,
	0x5fa1001d,0x5fa2001d,0x5fa3001d,0x5fa4001d,0x5fa5001d,0x5fa6001d,0x5fa7001d,0x5fa8001d,
	0x5fa9001d,0x5faa001d,0x5fab001d,0x5fac001d,0x5fad001d,0x5fae001d,0x5faf001d,0x5fb0001d,
	0x5fb1001d,0x5fb2001d,0x5fb3001d,0x5fb4001d,0x5fb5001d,0x5fb6001d,0x5fb7001d,0x5fb8001d,
	0x5fb9001d,0x5fba001d,0x5fbb001d,0x5fbc001d,0x5fbd001d,0x5fbe001d,0x5fbf001d,0x5fc0001d,
	0x5fc1001d,0x5fc2001d,0x5fc3001d,0x5fc4001d,0x5fc5001d,0x5fc6001d,0x5fc7001d,0x5fc8001d,
	0x5fc9001d,0x5fca001d,0x5fcb001d,0x5fcc001d,0x5fcd001d,0x5fce001d,0x5fcf001d,0x5fd0001d,
	0x5fd1001d,0x5fd2001d,0x5fd3001d,0x5fd4001d,0x5fd5001d,0x5fd6001d,0x5fd7001d,0x5fd8001d,
	0x5fd9001d,0x5fda001d,0x5fdb001d,0x5fdc001d,0x5fdd001d,0x5fde001d,0x5fdf001d,0x5fe0001d,
	0x5fe1001d,0x5fe2001d,0x5fe3001d,0x5fe4001d,0x5fe5001d,0x5fe6001d,0x5fe7001d,0x5fe8001d,
	0x5fe9001d,0x5fea001d,0x5feb001d,0x5fec001d,0x5fed001d,0x5fee001d,0x5fef001d,0x5ff0001d,
	0x5ff1001d,0x5ff2001d,0x5ff3001d,0x5ff4001d,0x5ff5001d,0x5ff6001d,0x5ff7001d,0x5ff8001d,
	0x5ff9001d,0x5ffa001d,0x5ffb001d,0x5ffc001d,0x5ffd001d,0x5ffe001d,0x5fff001d,0x6000001d,
	0x6001001d,0x6002001d,0x6003001d,0x6004001d,0x6005001d,0x6006001d,0x6007001d,0x6008001d,
	0x6009001d,0x600a001d,0x600b001d,0x600c001d,0x600d001d,0x600e001d,0x600f001d,0x6010001d,
	0x6011001d,0x6012001d,0x6013001d,0x6014001d,0x6015001d,0x6016001d,0x6017001d,0x6018001d,
	0x6019001d,0x601a001d,0x601b001d,0x601c001d,0x601d001d,0x601e001d,0x601f001d,0x6020001d,
	0x6021001d,0x6022001d,0x6023001d,0x6024001d,0x6025001d,0x6026001d,0x6027001d,0x6028001d,
	0x6029001d,0x602a001d,0x602b001d,0x602c001d,0x602d001d,0x602e001d,0x602f001d,0x6030001d,
	0x6031001d,0x6032001d,0x6033001d,0x6034001d,0x6035001d,0x6036001d,0x6037001d,0x6038001d,
	0x6039001d,0x603a001d,0x603b001d,0x603c001d,0x603d001d,0x603e001d,0x603f001d,0x6040001d,
	0x6041001d,0x6042001d,0x6043001d,0x6044001d,0x6045001d,0x6046001d,0x6047001d,0x6048001d,
	0x6049001d,0x604a001d,0x604b001d,0x604c001d,0x604d001d,0x604e001d,0x604f001d,0x6050001d,
	0x6051001d,0x6052001d,0x6053001d,0x6054001d,0x6055001d,0x6056001d,0x6057001d,0x6058001d,
	0x6059001d,0x605a001d,0x605b001d,0x605c001d,0x605d001d,0x605e001d,0x605f001d,0x6060001d,
	0x6061001d,0x6062001d,0x6063001d,0x6064001d,0x6065001d,0x6066001d,0x6067001d,0x6068001d,
	0x6069001d,0x606a001d,0x606b001d,0x606c001d,0x606d001d,0x606e001d,0x606f001d,0x6070001d,
	0x6071001d,0x6072001d,0x6073001d,0x6074001d,0x6075001d,0x6076001d,0x6077001d,0x6078001d,
	0x6079001d,0x607a001d,0x607b001d,0x607c001d,0x607d001d,0x607e001d,0x607f001d,0x6080001d,
	0x6081001d,0x6082001d,0x6083001d,0x6084001d,0x6085001d,0x6086001d,0x6087001d,0x6088001d,
	0x6089001d,0x608a001d,0x608b001d,0x608c001d,0x608d001d,0x608e001d,0x608f001d,0x6090001d,
	0x6091001d,0x6092001d,0x6093001d,0x6094001d,0x6095001d,0x6096001d,0x6097001d,0x6098001d,
	0x6099001d,0x609a001d,0x609b001d,0x609c001d,0x609d001d,0x609e001d,0x609f001d,0x60a0001d,
	0x60a1001d,0x60a2001d,0x60a3001d,0x60a4001d,0x60a5001d,0x60a6001d,0x60a7001d,0x60a8001d,
	0x60a9001d,0x60aa001d,0x60ab001d,0x60ac001d,0x60ad001d,0x60ae001d,0x60af001d,0x60b0001d,
	0x60b1001d,0x60b2001d,0x60b3001d,0x60b4001d,0x60b5001d,0x60b6001d,0x60b7001d,0x60b8001d,
	0x60b9001d,0x60ba001d,0x60bb001d,0x60bc001d,0x60bd001d,0x60be001d,0x60bf001d,0x60c0001d,
	0x60c1001d,0x60c2001d,0x60c3001d,0x60c4001d,0x60c5001d,0x60c6001d,0x60c7001d,0x60c8001d,
	0x60c9001d,0x60ca001d,0x60cb001d,0x60cc001d,0x60cd001d,0x60ce001d,0x60cf001d,0x60d0001d,
	0x60d1001d,0x60d2001d,0x60d3001d,0x60d4001d,0x60d5001d,0x60d6001d,0x60d7001d,0x60d8001d,
	0x60d9001d,0x60da001d,0x60db001d,0x60dc001d,0x60dd001d,0x60de001d,0x60df001d,0x60e0001d,
	0x60e1001d,0x60e2001d,0x60e3001d,0x60e4001d,0x60e5001d,0x60e6001d,0x60e7001d,0x60e8001d,
	0x60e9001d,0x60ea001d,0x60eb001d,0x60ec001d,0x60ed001d,0x60ee001d,0x60ef001d,0x60f0001d,
	0x60f1001d,0x60f2001d,0x60f3001d,0x60f4001d,0x60f5001d,0x60f6001d,0x60f7001d,0x60f8001d,
	0x60f9001d,0x60fa001d,0x60fb001d,0x60fc001d,0x60fd001d,0x60fe001d,0x60ff001d,0x6100001d,
	0x6101001d,0x6102001d,0x6103001d,0x6104001d,0x6105001d,0x6106001d,0x6107001d,0x6108001d,
	0x6109001d,0x610a001d,0x610b001d,0x610c001d,0x610d001d,0x610e001d,0x610f001d,0x6110001d,
	0x6111001d,0x6112001d,0x6113001d,0x6114001d,0x6115001d,0x6116001d,0x6117001d,0x6118001d,
	0x6119001d,0x611a001d,0x611b001d,0x611c001d,0x611d001d,0x611e001d,0x611f001d,0x6120001d,
	0x6121001d,0x6122001d,0x6123001d,0x6124001d,0x6125001d,0x6126001d,0x6127001d,0x6128001d,
	0x6129001d,0x612a001d,0x612b001d,0x612c001d,0x612d001d,0x612e001d,0x612f001d,0x6130001d,
	0x6131001d,0x6132001d,0x6133001d,0x6134001d,0x6135001d,0x6136001d,0x6137001d,0x6138001d,
	0x6139001d,0x613a001d,0x613b001d,0x613c001d,0x613d001d,0x613e001d,0x613f001d,0x6140001d,
	0x6141001d,0x6142001d,0x6143001d,0x6144001d,0x6145001d,0x6146001d,0x6147001d,0x6148001d,
	0x6149001d,0x614a001d,0x614b001d,0x614c001d,0x614d001d,0x614e001d,0x614f001d,0x6150001d,
	0x6151001d,0x6152001d,0x6153001d,0x6154001d,0x6155001d,0x6156001d,0x6157001d,0x6158001d,
	0x6159001d,0x615a001d,0x615b001d,0x615c001d,0x615d001d,0x615e001d,0x615f001d,0x6160001d,
	0x6161001d,0x6162001d,0x6163001d,0x6164001d,0x6165001d,0x6166001d,0x6167001d,0x6168001d,
	0x6169001d,0x616a001d,0x616b001d,0x616c001d,0x616d001d,0x616e001d,0x616f001d,0x6170001d,
	0x6171001d,0x6172001d,0x6173001d,0x6174001d,0x6175001d,0x6176001d,0x6177001d,0x6178001d,
	0x6179001d,0x617a001d,0x617b001d,0x617c001d,0x617d001d,0x617e001d,0x617f001d,0x6180001d,
	0x6181001d,0x6182001d,0x6183001d,0x6184001d,0x6185001d,0x6186001d,0x6187001d,0x6188001d,
	0x6189001d,0x618a001d,0x618b001d,0x618c001d,0x618d001d,0x618e001d,0x618f001d,0x6190001d,
	0x6191001d,0x6192001d,0x6193001d,0x6194001d,0x6195001d,0x6196001d,0x6197001d,0x6198001d,
	0x6199001d,0x619a001d,0x619b001d,0x619c001d,0x619d001d,0x619e001d,0x619f001d,0x61a0001d,
	0x61a1001d,0x61a2001d,0x61a3001d,0x61a4001d,0x61a5001d,0x61a6001d,0x61a7001d,0x61a8001d,
	0x61a9001d,0x61aa001d,0x61ab001d,0x61ac001d,0x61ad001d,0x61ae001d,0x61af001d,0x61b0001d,
	0x61b1001d,0x61b2001d,0x61b3001d,0x61b4001d,0x61b5001d,0x61b6001d,0x61b7001d,0x61b8001d,
	0x61b9001d,0x61ba001d,0x61bb001d,0x61bc001d,0x61bd001d,0x61be001d,0x61bf001d,0x61c0001d,
	0x61c1001d,0x61c2001d,0x61c3001d,0x61c4001d,0x61c5001d,0x61c6001d,0x61c7001d,0x61c8001d,
	0x61c9001d,0x61ca001d,0x61cb001d,0x61cc001d,0x61cd001d,0x61ce001d,0x61cf001d,0x61d0001d,
	0x61d1001d,0x61d2001d,0x61d3001d,0x61d4001d,0x61d5001d,0x61d6001d,0x61d7001d,0x61d8001d,
	0x61d9001d,0x61da001d,0x61db001d,0x61dc001d,0x61dd001d,0x61de001d,0x61df001d,0x61e0001d,
	0x61e1001d,0x61e2001d,0x61e3001d,0x61e4001d,0x61e5001d,0x61e6001d,0x61e7001d,0x61e8001d,
	0x61e9001d,0x61ea001d,0x61eb001d,0x61ec001d,0x61ed001d,0x61ee001d,0x61ef001d,0x61f0001d,
	0x61f1001d,0x61f2001d,0x61f3001d,0x61f4001d,0x61f5001d,0x61f6001d,0x61f7001d,0x61f8001d,
	0x61f9001d,0x61fa001d,0x61fb001d,0x61fc001d,0x61fd001d,0x61fe001d,0x61ff001d,0x6200001d,
	0x6201001d,0x6202001d,0x6203001d,0x6204001d,0x6205001d,0x6206001d,0x6207001d,0x6208001d,
	0x6209001d,0x620a001d,0x620b001d,0x620c001d,0x620d001d,0x620e001d,0x620f001d,0x6210001d,
	0x6211001d,0x6212001d,0x6213001d,0x6214001d,0x6215001d,0x6216001d,0x6217001d,0x6218001d,
	0x6219001d,0x621a001d,0x621b001d,0x621c001d,0x621d001d,0x621e001d,0x621f001d,0x6220001d,
	0x6221001d,0x6222001d,0x6223001d,0x6224001d,0x6225001d,0x6226001d,0x6227001d,0x6228001d,
	0x6229001d,0x622a001d,0x622b001d,0x622c001d,0x622d001d,0x622e001d,0x622f001d,0x6230001d,
	0x6231001d,0x6232001d,0x6233001d,0x6234001d,0x6235001d,0x6236001d,0x6237001d,0x6238001d,
	0x6239001d,0x623a001d,0x623b001d,0x623c001d,0x623d001d,0x623e001d,0x623f001d,0x6240001d,
	0x6241001d,0x6242001d,0x6243001d,0x6244001d,0x6245001d,0x6246001d,0x6247001d,0x6248001d,
	0x6249001d,0x624a001d,0x624b001d,0x624c001d,0x624d001d,0x624e001d,0x624f001d,0x6250001d,
	0x6251001d,0x6252001d,0x6253001d,0x6254001d,0x6255001d,0x6256001d,0x6257001d,0x6258001d,
	0x6259001d,0x625a001d,0x625b001d,0x625c001d,0x625d001d,0x625e001d,0x625f001d,0x6260001d,
	0x6261001d,0x6262001d,0x6263001d,0x6264001d,0x6265001d,0x6266001d,0x6267001d,0x6268001d,
	0x6269001d,0x626a001d,0x626b001d,0x626c001d,0x626d001d,0x626e001d,0x626f001d,0x6270001d,
	0x6271001d,0x6272001d,0x6273001d,0x6274001d,0x6275001d,0x6276001d,0x6277001d,0x6278001d,
	0x6279001d,0x627a001d,0x627b001d,0x627c001d,0x627d001d,0x627e001d,0x627f001d,0x6280001d,
	0x6281001d,0x6282001d,0x6283001d,0x6284001d,0x6285001d,0x6286001d,0x6287001d,0x6288001d,
	0x6289001d,0x628a001d,0x628b001d,0x628c001d,0x628d001d,0x628e001d,0x628f001d,0x6290001d,
	0x6291001d,0x6292001d,0x6293001d,0x6294001d,0x6295001d,0x6296001d,0x6297001d,0x6298001d,
	0x6299001d,0x629a001d,0x629b001d,0x629c001d,0x629d001d,0x629e001d,0x629f001d,0x62a0001d,
	0x62a1001d,0x62a2001d,0x62a3001d,0x62a4001d,0x62a5001d,0x62a6001d,0x62a7001d,0x62a8001d,
	0x62a9001d,0x62aa001d,0x62ab001d,0x62ac001d,0x62ad001d,0x62ae001d,0x62af001d,0x62b0001d,
	0x62b1001d,0x62b2001d,0x62b3001d,0x62b4001d,0x62b5001d,0x62b6001d,0x62b7001d,0x62b8001d,
	0x62b9001d,0x62ba001d,0x62bb001d,0x62bc001d,0x62bd001d,0x62be001d,0x62bf001d,0x62c0001d,
	0x62c1001d,0x62c2001d,0x62c3001d,0x62c4001d,0x62c5001d,0x62c6001d,0x62c7001d,0x62c8001d,
	0x62c9001d,0x62ca001d,0x62cb001d,0x62cc001d,0x62cd001d,0x62ce001d,0x62cf001d,0x62d0001d,
	0x62d1001d,0x62d2001d,0x62d3001d,0x62d4001d,0x62d5001d,0x62d6001d,0x62d7001d,0x62d8001d,
	0x62d9001d,0x62da001d,0x62db001d,0x62dc001d,0x62dd001d,0x62de001d,0x62df001d,0x62e0001d,
	0x62e1001d,0x62e2001d,0x62e3001d,0x62e4001d,0x62e5001d,0x62e6001d,0x62e7001d,0x62e8001d,
	0x62e9001d,0x62ea001d,0x62eb001d,0x62ec001d,0x62ed001d,0x62ee001d,0x62ef001d,0x62f0001d,
	0x62f1001d,0x62f2001d,0x62f3001d,0x62f4001d,0x62f5001d,0x62f6001d,0x62f7001d,0x62f8001d,
	0x62f9001d,0x62fa001d,0x62fb001d,0x62fc001d,0x62fd001d,0x62fe001d,0x62ff001d,0x6300001d,
	0x6301001d,0x6302001d,0x6303001d,0x6304001d,0x6305001d,0x6306001d,0x6307001d,0x6308001d,
	0x6309001d,0x630a001d,0x630b001d,0x630c001d,0x630d001d,0x630e001d,0x630f001d,0x6310001d,
	0x6311001d,0x6312001d,0x6313001d,0x6314001d,0x6315001d,0x6316001d,0x6317001d,0x6318001d,
	0x6319001d,0x631a001d,0x631b001d,0x631c001d,0x631d001d,0x631e001d,0x631f001d,0x6320001d,
	0x6321001d,0x6322001d,0x6323001d,0x6324001d,0x6325001d,0x6326001d,0x6327001d,0x6328001d,
	0x6329001d,0x632a001d,0x632b001d,0x632c001d,0x632d001d,0x632e001d,0x632f001d,0x6330001d,
	0x6331001d,0x6332001d,0x6333001d,0x6334001d,0x6335001d,0x6336001d,0x6337001d,0x6338001d,
	0x6339001d,0x633a001d,0x633b001d,0x633c001d,0x633d001d,0x633e001d,0x633f001d,0x6340001d,
	0x6341001d,0x6342001d,0x6343001d,0x6344001d,0x6345001d,0x6346001d,0x6347001d,0x6348001d,
	0x6349001d,0x634a001d,0x634b001d,0x634c001d,0x634d001d,0x634e001d,0x634f001d,0x6350001d,
	0x6351001d,0x6352001d,0x6353001d,0x6354001d,0x6355001d,0x6356001d,0x6357001d,0x6358001d,
	0x6359001d,0x635a001d,0x635b001d,0x635c001d,0x635d001d,0x635e001d,0x635f001d,0x6360001d,
	0x6361001d,0x6362001d,0x6363001d,0x6364001d,0x6365001d,0x6366001d,0x6367001d,0x6368001d,
	0x6369001d,0x636a001d,0x636b001d,0x636c001d,0x636d001d,0x636e001d,0x636f001d,0x6370001d,
	0x6371001d,0x6372001d,0x6373001d,0x6374001d,0x6375001d,0x6376001d,0x6377001d,0x6378001d,
	0x6379001d,0x637a001d,0x637b001d,0x637c001d,0x637d001d,0x637e001d,0x637f001d,0x6380001d,
	0x6381001d,0x6382001d,0x6383001d,0x6384001d,0x6385001d,0x6386001d,0x6387001d,0x6388001d,
	0x6389001d,0x638a001d,0x638b001d,0x638c001d,0x638d001d,0x638e001d,0x638f001d,0x6390001d,
	0x6391001d,0x6392001d,0x6393001d,0x6394001d,0x6395001d,0x6396001d,0x6397001d,0x6398001d,
	0x6399001d,0x639a001d,0x639b001d,0x639c001d,0x639d001d,0x639e001d,0x639f001d,0x63a0001d,
	0x63a1001d,0x63a2001d,0x63a3001d,0x63a4001d,0x63a5001d,0x63a6001d,0x63a7001d,0x63a8001d,
	0x63a9001d,0x63aa001d,0x63ab001d,0x63ac001d,0x63ad001d,0x63ae001d,0x63af001d,0x63b0001d,
	0x63b1001d,0x63b2001d,0x63b3001d,0x63b4001d,0x63b5001d,0x63b6001d,0x63b7001d,0x63b8001d,
	0x63b9001d,0x63ba001d,0x63bb001d,0x63bc001d,0x63bd001d,0x63be001d,0x63bf001d,0x63c0001d,
	0x63c1001d,0x63c2001d,0x63c3001d,0x63c4001d,0x63c5001d,0x63c6001d,0x63c7001d,0x63c8001d,
	0x63c9001d,0x63ca001d,0x63cb001d,0x63cc001d,0x63cd001d,0x63ce001d,0x63cf001d,0x63d0001d,
	0x63d1001d,0x63d2001d,0x63d3001d,0x63d4001d,0x63d5001d,0x63d6001d,0x63d7001d,0x63d8001d,
	0x63d9001d,0x63da001d,0x63db001d,0x63dc001d,0x63dd001d,0x63de001d,0x63df001d,0x63e0001d,
	0x63e1001d,0x63e2001d,0x63e3001d,0x63e4001d,0x63e5001d,0x63e6001d,0x63e7001d,0x63e8001d,
	0x63e9001d,0x63ea001d,0x63eb001d,0x63ec001d,0x63ed001d,0x63ee001d,0x63ef001d,0x63f0001d,
	0x63f1001d,0x63f2001d,0x63f3001d,0x63f4001d,0x63f5001d,0x63f6001d,0x63f7001d,0x63f8001d,
	0x63f9001d,0x63fa001d,0x63fb001d,0x63fc001d,0x63fd001d,0x63fe001d,0x63ff001d,0x6400001d,
	0x6401001d,0x6402001d,0x6403001d,0x6404001d,0x6405001d,0x6406001d,0x6407001d,0x6408001d,
	0x6409001d,0x640a001d,0x640b001d,0x640c001d,0x640d001d,0x640e001d,0x640f001d,0x6410001d,
	0x6411001d,0x6412001d,0x6413001d,0x6414001d,0x6415001d,0x6416001d,0x6417001d,0x6418001d,
	0x6419001d,0x641a001d,0x641b001d,0x641c001d,0x641d001d,0x641e001d,0x641f001d,0x6420001d,
	0x6421001d,0x6422001d,0x6423001d,0x6424001d,0x6425001d,0x6426001d,0x6427001d,0x6428001d,
	0x6429001d,0x642a001d,0x642b001d,0x642c001d,0x642d001d,0x642e001d,0x642f001d,0x6430001d,
	0x6431001d,0x6432001d,0x6433001d,0x6434001d,0x6435001d,0x6436001d,0x6437001d,0x6438001d,
	0x6439001d,0x643a001d,0x643b001d,0x643c001d,0x643d001d,0x643e001d,0x643f001d,0x6440001d,
	0x6441001d,0x6442001d,0x6443001d,0x6444001d,0x6445001d,0x6446001d,0x6447001d,0x6448001d,
	0x6449001d,0x644a001d,0x644b001d,0x644c001d,0x644d001d,0x644e001d,0x644f001d,0x6450001d,
	0x6451001d,0x6452001d,0x6453001d,0x6454001d,0x6455001d,0x6456001d,0x6457001d,0x6458001d,
	0x6459001d,0x645a001d,0x645b001d,0x645c001d,0x645d001d,0x645e001d,0x645f001d,0x6460001d,
	0x6461001d,0x6462001d,0x6463001d,0x6464001d,0x6465001d,0x6466001d,0x6467001d,0x6468001d,
	0x6469001d,0x646a001d,0x646b001d,0x646c001d,0x646d001d,0x646e001d,0x646f001d,0x6470001d,
	0x6471001d,0x6472001d,0x6473001d,0x6474001d,0x6475001d,0x6476001d,0x6477001d,0x6478001d,
	0x6479001d,0x647a001d,0x647b001d,0x647c001d,0x647d001d,0x647e001d,0x647f001d,0x6480001d,
	0x6481001d,0x6482001d,0x6483001d,0x6484001d,0x6485001d,0x6486001d,0x6487001d,0x6488001d,
	0x6489001d,0x648a001d,0x648b001d,0x648c001d,0x648d001d,0x648e001d,0x648f001d,0x6490001d,
	0x6491001d,0x6492001d,0x6493001d,0x6494001d,0x6495001d,0x6496001d,0x6497001d,0x6498001d,
	0x6499001d,0x649a001d,0x649b001d,0x649c001d,0x649d001d,0x649e001d,0x649f001d,0x64a0001d,
	0x64a1001d,0x64a2001d,0x64a3001d,0x64a4001d,0x64a5001d,0x64a6001d,0x64a7001d,0x64a8001d,
	0x64a9001d,0x64aa001d,0x64ab001d,0x64ac001d,0x64ad001d,0x64ae001d,0x64af001d,0x64b0001d,
	0x64b1001d,0x64b2001d,0x64b3001d,0x64b4001d,0x64b5001d,0x64b6001d,0x64b7001d,0x64b8001d,
	0x64b9001d,0x64ba001d,0x64bb001d,0x64bc001d,0x64bd001d,0x64be001d,0x64bf001d,0x64c0001d,
	0x64c1001d,0x64c2001d,0x64c3001d,0x64c4001d,0x64c5001d,0x64c6001d,0x64c7001d,0x64c8001d,
	0x64c9001d,0x64ca001d,0x64cb001d,0x64cc001d,0x64cd001d,0x64ce001d,0x64cf001d,0x64d0001d,
	0x64d1001d,0x64d2001d,0x64d3001d,0x64d4001d,0x64d5001d,0x64d6001d,0x64d7001d,0x64d8001d,
	0x64d9001d,0x64da001d,0x64db001d,0x64dc001d,0x64dd001d,0x64de001d,0x64df001d,0x64e0001d,
	0x64e1001d,0x64e2001d,0x64e3001d,0x64e4001d,0x64e5001d,0x64e6001d,0x64e7001d,0x64e8001d,
	0x64e9001d,0x64ea001d,0x64eb001d,0x64ec001d,0x64ed001d,0x64ee001d,0x64ef001d,0x64f0001d,
	0x64f1001d,0x64f2001d,0x64f3001d,0x64f4001d,0x64f5001d,0x64f6001d,0x64f7001d,0x64f8001d,
	0x64f9001d,0x64fa001d,0x64fb001d,0x64fc001d,0x64fd001d,0x64fe001d,0x64ff001d,0x6500001d,
	0x6501001d,0x6502001d,0x6503001d,0x6504001d,0x6505001d,0x6506001d,0x6507001d,0x6508001d,
	0x6509001d,0x650a001d,0x650b001d,0x650c001d,0x650d001d,0x650e001d,0x650f001d,0x6510001d,
	0x6511001d,0x6512001d,0x6513001d,0x6514001d,0x6515001d,0x6516001d,0x6517001d,0x6518001d,
	0x6519001d,0x651a001d,0x651b001d,0x651c001d,0x651d001d,0x651e001d,0x651f001d,0x6520001d,
	0x6521001d,0x6522001d,0x6523001d,0x6524001d,0x6525001d,0x6526001d,0x6527001d,0x6528001d,
	0x6529001d,0x652a001d,0x652b001d,0x652c001d,0x652d001d,0x652e001d,0x652f001d,0x6530001d,
	0x6531001d,0x6532001d,0x6533001d,0x6534001d,0x6535001d,0x6536001d,0x6537001d,0x6538001d,
	0x6539001d,0x653a001d,0x653b001d,0x653c001d,0x653d001d,0x653e001d,0x653f001d,0x6540001d,
	0x6541001d,0x6542001d,0x6543001d,0x6544001d,0x6545001d,0x6546001d,0x6547001d,0x6548001d,
	0x6549001d,0x654a001d,0x654b001d,0x654c001d,0x654d001d,0x654e001d,0x654f001d,0x6550001d,
	0x6551001d,0x6552001d,0x6553001d,0x6554001d,0x6555001d,0x6556001d,0x6557001d,0x6558001d,
	0x6559001d,0x655a001d,0x655b001d,0x655c001d,0x655d001d,0x655e001d,0x655f001d,0x6560001d,
	0x6561001d,0x6562001d,0x6563001d,0x6564001d,0x6565001d,0x6566001d,0x6567001d,0x6568001d,
	0x6569001d,0x656a001d,0x656b001d,0x656c001d,0x656d001d,0x656e001d,0x656f001d,0x6570001d,
	0x6571001d,0x6572001d,0x6573001d,0x6574001d,0x6575001d,0x6576001d,0x6577001d,0x6578001d,
	0x6579001d,0x657a001d,0x657b001d,0x657c001d,0x657d001d,0x657e001d,0x657f001d,0x6580001d,
	0x6581001d,0x6582001d,0x6583001d,0x6584001d,0x6585001d,0x6586001d,0x6587001d,0x6588001d,
	0x6589001d,0x658a001d,0x658b001d,0x658c001d,0x658d001d,0x658e001d,0x658f001d,0x6590001d,
	0x6591001d,0x6592001d,0x6593001d,0x6594001d,0x6595001d,0x6596001d,0x6597001d,0x6598001d,
	0x6599001d,0x659a001d,0x659b001d,0x659c001d,0x659d001d,0x659e001d,0x659f001d,0x65a0001d,
	0x65a1001d,0x65a2001d,0x65a3001d,0x65a4001d,0x65a5001d,0x65a6001d,0x65a7001d,0x65a8001d,
	0x65a9001d,0x65aa001d,0x65ab001d,0x65ac001d,0x65ad001d,0x65ae001d,0x65af001d,0x65b0001d,
	0x65b1001d,0x65b2001d,0x65b3001d,0x65b4001d,0x65b5001d,0x65b6001d,0x65b7001d,0x65b8001d,
	0x65b9001d,0x65ba001d,0x65bb001d,0x65bc001d,0x65bd001d,0x65be001d,0x65bf001d,0x65c0001d,
	0x65c1001d,0x65c2001d,0x65c3001d,0x65c4001d,0x65c5001d,0x65c6001d,0x65c7001d,0x65c8001d,
	0x65c9001d,0x65ca001d,0x65cb001d,0x65cc001d,0x65cd001d,0x65ce001d,0x65cf001d,0x65d0001d,
	0x65d1001d,0x65d2001d,0x65d3001d,0x65d4001d,0x65d5001d,0x65d6001d,0x65d7001d,0x65d8001d,
	0x65d9001d,0x65da001d,0x65db001d,0x65dc001d,0x65dd001d,0x65de001d,0x65df001d,0x65e0001d,
	0x65e1001d,0x65e2001d,0x65e3001d,0x65e4001d,0x65e5001d,0x65e6001d,0x65e7001d,0x65e8001d,
	0x65e9001d,0x65ea001d,0x65eb001d,0x65ec001d,0x65ed001d,0x65ee001d,0x65ef001d,0x65f0001d,
	0x65f1001d,0x65f2001d,0x65f3001d,0x65f4001d,0x65f5001d,0x65f6001d,0x65f7001d,0x65f8001d,
	0x65f9001d,0x65fa001d,0x65fb001d,0x65fc001d,0x65fd001d,0x65fe001d,0x65ff001d,0x6600001d,
	0x6601001d,0x6602001d,0x6603001d,0x6604001d,0x6605001d,0x6606001d,0x6607001d,0x6608001d,
	0x6609001d,0x660a001d,0x660b001d,0x660c001d,0x660d001d,0x660e001d,0x660f001d,0x6610001d,
	0x6611001d,0x6612001d,0x6613001d,0x6614001d,0x6615001d,0x6616001d,0x6617001d,0x6618001d,
	0x6619001d,0x661a001d,0x661b001d,0x661c001d,0x661d001d,0x661e001d,0x661f001d,0x6620001d,
	0x6621001d,0x6622001d,0x6623001d,0x6624001d,0x6625001d,0x6626001d,0x6627001d,0x6628001d,
	0x6629001d,0x662a001d,0x662b001d,0x662c001d,0x662d001d,0x662e001d,0x662f001d,0x6630001d,
	0x6631001d,0x6632001d,0x6633001d,0x6634001d,0x6635001d,0x6636001d,0x6637001d,0x6638001d,
	0x6639001d,0x663a001d,0x663b001d,0x663c001d,0x663d001d,0x663e001d,0x663f001d,0x6640001d,
	0x6641001d,0x6642001d,0x6643001d,0x6644001d,0x6645001d,0x6646001d,0x6647001d,0x6648001d,
	0x6649001d,0x664a001d,0x664b001d,0x664c001d,0x664d001d,0x664e001d,0x664f001d,0x6650001d,
	0x6651001d,0x6652001d,0x6653001d,0x6654001d,0x6655001d,0x6656001d,0x6657001d,0x6658001d,
	0x6659001d,0x665a001d,0x665b001d,0x665c001d,0x665d001d,0x665e001d,0x665f001d,0x6660001d,
	0x6661001d,0x6662001d,0x6663001d,0x6664001d,0x6665001d,0x6666001d,0x6667001d,0x6668001d,
	0x6669001d,0x666a001d,0x666b001d,0x666c001d,0x666d001d,0x666e001d,0x666f001d,0x6670001d,
	0x6671001d,0x6672001d,0x6673001d,0x6674001d,0x6675001d,0x6676001d,0x6677001d,0x6678001d,
	0x6679001d,0x667a001d,0x667b001d,0x667c001d,0x667d001d,0x667e001d,0x667f001d,0x6680001d,
	0x6681001d,0x6682001d,0x6683001d,0x6684001d,0x6685001d,0x6686001d,0x6687001d,0x6688001d,
	0x6689001d,0x668a001d,0x668b001d,0x668c001d,0x668d001d,0x668e001d,0x668f001d,0x6690001d,
	0x6691001d,0x6692001d,0x6693001d,0x6694001d,0x6695001d,0x6696001d,0x6697001d,0x6698001d,
	0x6699001d,0x669a001d,0x669b001d,0x669c001d,0x669d001d,0x669e001d,0x669f001d,0x66a0001d,
	0x66a1001d,0x66a2001d,0x66a3001d,0x66a4001d,0x66a5001d,0x66a6001d,0x66a7001d,0x66a8001d,
	0x66a9001d,0x66aa001d,0x66ab001d,0x66ac001d,0x66ad001d,0x66ae001d,0x66af001d,0x66b0001d,
	0x66b1001d,0x66b2001d,0x66b3001d,0x66b4001d,0x66b5001d,0x66b6001d,0x66b7001d,0x66b8001d,
	0x66b9001d,0x66ba001d,0x66bb001d,0x66bc001d,0x66bd001d,0x66be001d,0x66bf001d,0x66c0001d,
	0x66c1001d,0x66c2001d,0x66c3001d,0x66c4001d,0x66c5001d,0x66c6001d,0x66c7001d,0x66c8001d,
	0x66c9001d,0x66ca001d,0x66cb001d,0x66cc001d,0x66cd001d,0x66ce001d,0x66cf001d,0x66d0001d,
	0x66d1001d,0x66d2001d,0x66d3001d,0x66d4001d,0x66d5001d,0x66d6001d,0x66d7001d,0x66d8001d,
	0x66d9001d,0x66da001d,0x66db001d,0x66dc001d,0x66dd001d,0x66de001d,0x66df001d,0x66e0001d,
	0x66e1001d,0x66e2001d,0x66e3001d,0x66e4001d,0x66e5001d,0x66e6001d,0x66e7001d,0x66e8001d,
	0x66e9001d,0x66ea001d,0x66eb001d,0x66ec001d,0x66ed001d,0x66ee001d,0x66ef001d,0x66f0001d,
	0x66f1001d,0x66f2001d,0x66f3001d,0x66f4001d,0x66f5001d,0x66f6001d,0x66f7001d,0x66f8001d,
	0x66f9001d,0x66fa001d,0x66fb001d,0x66fc001d,0x66fd001d,0x66fe001d,0x66ff001d,0x6700001d,
	0x6701001d,0x6702001d,0x6703001d,0x6704001d,0x6705001d,0x6706001d,0x6707001d,0x6708001d,
	0x6709001d,0x670a001d,0x670b001d,0x670c001d,0x670d001d,0x670e001d,0x670f001d,0x6710001d,
	0x6711001d,0x6712001d,0x6713001d,0x6714001d,0x6715001d,0x6716001d,0x6717001d,0x6718001d,
	0x6719001d,0x671a001d,0x671b001d,0x671c001d,0x671d001d,0x671e001d,0x671f001d,0x6720001d,
	0x6721001d,0x6722001d,0x6723001d,0x6724001d,0x6725001d,0x6726001d,0x6727001d,0x6728001d,
	0x6729001d,0x672a001d,0x672b001d,0x672c001d,0x672d001d,0x672e001d,0x672f001d,0x6730001d,
	0x6731001d,0x6732001d,0x6733001d,0x6734001d,0x6735001d,0x6736001d,0x6737001d,0x6738001d,
	0x6739001d,0x673a001d,0x673b001d,0x673c001d,0x673d001d,0x673e001d,0x673f001d,0x6740001d,
	0x6741001d,0x6742001d,0x6743001d,0x6744001d,0x6745001d,0x6746001d,0x6747001d,0x6748001d,
	0x6749001d,0x674a001d,0x674b001d,0x674c001d,0x674d001d,0x674e001d,0x674f001d,0x6750001d,
	0x6751001d,0x6752001d,0x6753001d,0x6754001d,0x6755001d,0x6756001d,0x6757001d,0x6758001d,
	0x6759001d,0x675a001d,0x675b001d,0x675c001d,0x675d001d,0x675e001d,0x675f001d,0x6760001d,
	0x6761001d,0x6762001d,0x6763001d,0x6764001d,0x6765001d,0x6766001d,0x6767001d,0x6768001d,
	0x6769001d,0x676a001d,0x676b001d,0x676c001d,0x676d001d,0x676e001d,0x676f001d,0x6770001d,
	0x6771001d,0x6772001d,0x6773001d,0x6774001d,0x6775001d,0x6776001d,0x6777001d,0x6778001d,
	0x6779001d,0x677a001d,0x677b001d,0x677c001d,0x677d001d,0x677e001d,0x677f001d,0x6780001d,
	0x6781001d,0x6782001d,0x6783001d,0x6784001d,0x6785001d,0x6786001d,0x6787001d,0x6788001d,
	0x6789001d,0x678a001d,0x678b001d,0x678c001d,0x678d001d,0x678e001d,0x678f001d,0x6790001d,
	0x6791001d,0x6792001d,0x6793001d,0x6794001d,0x6795001d,0x6796001d,0x6797001d,0x6798001d,
	0x6799001d,0x679a001d,0x679b001d,0x679c001d,0x679d001d,0x679e001d,0x679f001d,0x67a0001d,
	0x67a1001d,0x67a2001d,0x67a3001d,0x67a4001d,0x67a5001d,0x67a6001d,0x67a7001d,0x67a8001d,
	0x67a9001d,0x67aa001d,0x67ab001d,0x67ac001d,0x67ad001d,0x67ae001d,0x67af001d,0x67b0001d,
	0x67b1001d,0x67b2001d,0x67b3001d,0x67b4001d,0x67b5001d,0x67b6001d,0x67b7001d,0x67b8001d,
	0x67b9001d,0x67ba001d,0x67bb001d,0x67bc001d,0x67bd001d,0x67be001d,0x67bf001d,0x67c0001d,
	0x67c1001d,0x67c2001d,0x67c3001d,0x67c4001d,0x67c5001d,0x67c6001d,0x67c7001d,0x67c8001d,
	0x67c9001d,0x67ca001d,0x67cb001d,0x67cc001d,0x67cd001d,0x67ce001d,0x67cf001d,0x67d0001d,
	0x67d1001d,0x67d2001d,0x67d3001d,0x67d4001d,0x67d5001d,0x67d6001d,0x67d7001d,0x67d8001d,
	0x67d9001d,0x67da001d,0x67db001d,0x67dc001d,0x67dd001d,0x67de001d,0x67df001d,0x67e0001d,
	0x67e1001d,0x67e2001d,0x67e3001d,0x67e4001d,0x67e5001d,0x67e6001d,0x67e7001d,0x67e8001d,
	0x67e9001d,0x67ea001d,0x67eb001d,0x67ec001d,0x67ed001d,0x67ee001d,0x67ef001d,0x67f0001d,
	0x67f1001d,0x67f2001d,0x67f3001d,0x67f4001d,0x67f5001d,0x67f6001d,0x67f7001d,0x67f8001d,
	0x67f9001d,0x67fa001d,0x67fb001d,0x67fc001d,0x67fd001d,0x67fe001d,0x67ff001d,0x6800001d,
	0x6801001d,0x6802001d,0x6803001d,0x6804001d,0x6805001d,0x6806001d,0x6807001d,0x6808001d,
	0x6809001d,0x680a001d,0x680b001d,0x680c001d,0x680d001d,0x680e001d,0x680f001d,0x6810001d,
	0x6811001d,0x6812001d,0x6813001d,0x6814001d,0x6815001d,0x6816001d,0x6817001d,0x6818001d,
	0x6819001d,0x681a001d,0x681b001d,0x681c001d,0x681d001d,0x681e001d,0x681f001d,0x6820001d,
	0x6821001d,0x6822001d,0x6823001d,0x6824001d,0x6825001d,0x6826001d,0x6827001d,0x6828001d,
	0x6829001d,0x682a001d,0x682b001d,0x682c001d,0x682d001d,0x682e001d,0x682f001d,0x6830001d,
	0x6831001d,0x6832001d,0x6833001d,0x6834001d,0x6835001d,0x6836001d,0x6837001d,0x6838001d,
	0x6839001d,0x683a001d,0x683b001d,0x683c001d,0x683d001d,0x683e001d,0x683f001d,0x6840001d,
	0x6841001d,0x6842001d,0x6843001d,0x6844001d,0x6845001d,0x6846001d,0x6847001d,0x6848001d,
	0x6849001d,0x684a001d,0x684b001d,0x684c001d,0x684d001d,0x684e001d,0x684f001d,0x6850001d,
	0x6851001d,0x6852001d,0x6853001d,0x6854001d,0x6855001d,0x6856001d,0x6857001d,0x6858001d,
	0x6859001d,0x685a001d,0x685b001d,0x685c001d,0x685d001d,0x685e001d,0x685f001d,0x6860001d,
	0x6861001d,0x6862001d,0x6863001d,0x6864001d,0x6865001d,0x6866001d,0x6867001d,0x6868001d,
	0x6869001d,0x686a001d,0x686b001d,0x21e010f,0x234010f,0x269010f,0x2c4010d,0x26a010f,
	0x268010f,0x22d010f,0x23e010f,0x23f010f,0x263010f,0x369010f,0x214010f,0x20c010f,
	0x226010f,0x265010f,0x6c5010d,0x6c6010d,0x6c7010d,0x6c8010d,0x6c9010d,0x6ca010d,
	0x6cb010d,0x6cc010d,0x6cd010d,0x6ce010d,0x21c010f,0x21a010f,0x36d010f,0x36e010f,
	0x36f010f,0x221010f,0x262010f,0x6cf0125,0x6e30125,0x6f70125,0x7030125,0x71f0125,
	0x74b0125,0x7530125,0x7730125,0x78d0125,0x7a10125,0x7b10125,0x7bd0125,0x7e50125,
	0x7ed0125,0x8050125,0x81d0125,0x8290125,0x8350125,0x85d0125,0x8750125,0x88d0125,
	0x8a50125,0x8b10125,0x8b90125,0x8bd0125,0x8c90125,0x240010f,0x241010f,0x2091d0f,
	0x209020f,0x209160f,0x6cf010d,0x6e3010d,0x6f7010d,0x703010d,0x71f010d,0x74b010d,
	0x753010d,0x773010d,0x78d010d,0x7a1010d,0x7b1010d,0x7bd010d,0x7e5010d,0x7ed010d,
	0x805010d,0x81d010d,0x829010d,0x835010d,0x85d010d,0x875010d,0x88d010d,0x8a5010d,
	0x8b1010d,0x8b9010d,0x8bd010d,0x8c9010d,0x242010f,0x371010f,0x243010f,0x374010f,
	0x228010b,0x219010b,0x22e010b,0x22f010b,0x235010b,0x236010b,0xfb400108,0xce020101,
	0xfb400108,0xce040101,0xfb400108,0xce050101,0xfb400108,0xce060101,0xfb400108,0xce0f0101,
	0xfb400108,0xce120101,0xfb400108,0xce170101,0xfb400108,0xce1f0101,0xfb400108,0xce200101,
	0xfb400108,0xce210101,0xfb400108,0xce230101,0xfb400108,0xce260101,0xfb400108,0xce290101,
	0xfb400108,0xce2e0101,0xfb400108,0xce2f0101,0xfb400108,0xce310101,0xfb400108,0xce330101,
	0xfb400108,0xce350101,0xfb400108,0xce370101,0xfb400108,0xce3c0101,0xfb400108,0xce400101,
	0xfb400108,0xce410101,0xfb400108,0xce420101,0xfb400108,0xce440101,0xfb400108,0xce460101,
	0xfb400108,0xce4a0101,0xfb400108,0xce510101,0xfb400108,0xce550101,0xfb400108,0xce570101,
	0xfb400108,0xce5a0101,0xfb400108,0xce5b0101,0xfb400108,0xce620101,0xfb400108,0xce630101,
	0xfb400108,0xce640101,0xfb400108,0xce650101,0xfb400108,0xce670101,0xfb400108,0xce680101,
	0xfb400108,0xce6a0101,0xfb400108,0xce6b0101,0xfb400108,0xce6c0101,0xfb400108,0xce6d0101,
	0xfb400108,0xce6e0101,0xfb400108,0xce6f0101,0xfb400108,0xce720101,0xfb400108,0xce740101,
	0xfb400108,0xce750101,0xfb400108,0xce760101,0xfb400108,0xce770101,0xfb400108,0xce780101,
	0xfb400108,0xce790101,0xfb400108,0xce7a0101,0xfb400108,0xce7b0101,0xfb400108,0xce7c0101,
	0xfb400108,0xce7d0101,0xfb400108,0xce7f0101,0xfb400108,0xce800101,0xfb400108,0xce810101,
	0xfb400108,0xce820101,0xfb400108,0xce830101,0xfb400108,0xce840101,0xfb400108,0xce850101,
	0xfb400108,0xce870101,0xfb400108,0xce8a0101,0xfb400108,0xce900101,0xfb400108,0xce960101,
	0xfb400108,0xce970101,0xfb400108,0xce990101,0xfb400108,0xce9c0101,0xfb400108,0xce9d0101,
	0xfb400108,0xce9e0101,0xfb400108,0xcea30101,0xfb400108,0xceaa0101,0xfb400108,0xceaf0101,
	0xfb400108,0xceb00101,0xfb400108,0xceb10101,0xfb400108,0xceb40101,0xfb400108,0xceb60101,
	0xfb400108,0xceb70101,0xfb400108,0xceb80101,0xfb400108,0xceb90101,0xfb400108,0xcebc0101,
	0xfb400108,0xcebd0101,0xfb400108,0xcebe0101,0xfb400108,0xcec80101,0xfb400108,0xcecc0101,
	0xfb400108,0xcecf0101,0xfb400108,0xced00101,0xfb400108,0xced20101,0xfb400108,0xceda0101,
	0xfb400108,0xcedb0101,0xfb400108,0xcedc0101,0xfb400108,0xcee00101,0xfb400108,0xcee20101,
	0xfb400108,0xcee60101,0xfb400108,0xcee70101,0xfb400108,0xcee90101,0xfb400108,0xceed0101,
	0xfb400108,0xceee0101,0xfb400108,0xceef0101,0xfb400108,0xcef10101,0xfb400108,0xcef40101,
	0xfb400108,0xcef80101,0xfb400108,0xcef90101,0xfb400108,0xcefa0101,0xfb400108,0xcefc0101,
	0xfb400108,0xcefe0101,0xfb400108,0xcf000101,0xfb400108,0xcf020101,0xfb400108,0xcf030101,
	0xfb400108,0xcf040101,0xfb400108,0xcf050101,0xfb400108,0xcf060101,0xfb400108,0xcf070101,
	0xfb400108,0xcf080101,0xfb400108,0xcf0b0101,0xfb400108,0xcf0c0101,0xfb400108,0xcf120101,
	0xfb400108,0xcf130101,0xfb400108,0xcf140101,0xfb400108,0xcf150101,0xfb400108,0xcf160101,
	0xfb400108,0xcf1c0101,0xfb400108,0xcf1d0101,0xfb400108,0xcf210101,0xfb400108,0xcf230101,
	0xfb400108,0xcf280101,0xfb400108,0xcf290101,0xfb400108,0xcf2c0101,0xfb400108,0xcf2d0101,
	0xfb400108,0xcf2e0101,0xfb400108,0xcf310101,0xfb400108,0xcf330101,0xfb400108,0xcf350101,
	0xfb400108,0xcf370101,0xfb400108,0xcf390101,0xfb400108,0xcf3b0101,0xfb400108,0xcf3e0101,
	0xfb400108,0xcf3f0101,0xfb400108,0xcf400101,0xfb400108,0xcf410101,0xfb400108,0xcf420101,
	0xfb400108,0xcf440101,0xfb400108,0xcf450101,0xfb400108,0xcf470101,0xfb400108,0xcf480101,
	0xfb400108,0xcf490101,0xfb400108,0xcf4a0101,0xfb400108,0xcf4b0101,0xfb400108,0xcf4c0101,
	0xfb400108,0xcf520101,0xfb400108,0xcf540101,0xfb400108,0xcf560101,0xfb400108,0xcf610101,
	0xfb400108,0xcf620101,0xfb400108,0xcf660101,0xfb400108,0xcf680101,0xfb400108,0xcf6a0101,
	0xfb400108,0xcf6b0101,0xfb400108,0xcf6d0101,0xfb400108,0xcf6e0101,0xfb400108,0xcf710101,
	0xfb400108,0xcf720101,0xfb400108,0xcf750101,0xfb400108,0xcf770101,0xfb400108,0xcf780101,
	0xfb400108,0xcf790101,0xfb400108,0xcf7a0101,0xfb400108,0xcf7d0101,0xfb400108,0xcf800101,
	0xfb400108,0xcf810101,0xfb400108,0xcf820101,0xfb400108,0xcf850101,0xfb400108,0xcf860101,
	0xfb400108,0xcf870101,0xfb400108,0xcf8a0101,0xfb400108,0xcf8c0101,0xfb400108,0xcf8e0101,
	0xfb400108,0xcf900101,0xfb400108,0xcf920101,0xfb400108,0xcf930101,0xfb400108,0xcf950101,
	0xfb400108,0xcf960101,0xfb400108,0xcf980101,0xfb400108,0xcf990101,0xfb400108,0xcf9a0101,
	0xfb400108,0xcf9c0101,0xfb400108,0xcf9e0101,0xfb400108,0xcf9f0101,0xfb400108,0xcfa10101,
	0xfb400108,0xcfa20101,0xfb400108,0xcfa40101,0xfb400108,0xcfab0101,0xfb400108,0xcfad0101,
	0xfb400108,0xcfb00101,0xfb400108,0xcfb10101,0xfb400108,0xcfb20101,0xfb400108,0xcfb30101,
	0xfb400108,0xcfb40101,0xfb400108,0xcfb60101,0xfb400108,0xcfb70101,0xfb400108,0xcfb80101,
	0xfb400108,0xcfb90101,0xfb400108,0xcfba0101,0xfb400108,0xcfbb0101,0xfb400108,0xcfbc0101,
	0xfb400108,0xcfbd0101,0xfb400108,0xcfbe0101,0xfb400108,0xcfc00101,0xfb400108,0xcfc10101,
	0xfb400108,0xcfc20101,0xfb400108,0xcfc60101,0xfb400108,0xcfc70101,0xfb400108,0xcfc80101,
	0xfb400108,0xcfc90101,0xfb400108,0xcfcb0101,0xfb400108,0xcfcc0101,0xfb400108,0xcfcd0101,
	0xfb400108,0xcfd20101,0xfb400108,0xcfd30101,0xfb400108,0xcfd40101,0xfb400108,0xcfd50101,
	0xfb400108,0xcfd60101,0xfb400108,0xcfd90101,0xfb400108,0xcfdb0101,0xfb400108,0xcfe00101,
	0xfb400108,0xcfe20101,0xfb400108,0xcfe40101,0xfb400108,0xcfe50101,0xfb400108,0xcfe70101,
	0xfb400108,0xcfeb0101,0xfb400108,0xcfec0101,0xfb400108,0xcff00101,0xfb400108,0xcff20101,
	0xfb400108,0xcff40101,0xfb400108,0xcff50101,0xfb400108,0xcff60101,0xfb400108,0xcff70101,
	0xfb400108,0xcff90101,0xfb400108,0xcffb0101,0xfb400108,0xcffc0101,0xfb400108,0xcffd0101,
	0xfb400108,0xcfff0101,0xfb400108,0xd0000101,0xfb400108,0xd0010101,0xfb400108,0xd0020101,
	0xfb400108,0xd0030101,0xfb400108,0xd0040101,0xfb400108,0xd0050101,0xfb400108,0xd0060101,
	0xfb400108,0xd0070101,0xfb400108,0xd0080101,0xfb400108,0xd0090101,0xfb400108,0xd00a0101,
	0xfb400108,0xd00b0101,0xfb400108,0xd00e0101,0xfb400108,0xd0100101,0xfb400108,0xd0110101,
	0xfb400108,0xd0130101,0xfb400108,0xd0150101,0xfb400108,0xd0160101,0xfb400108,0xd0170101,
	0xfb400108,0xd01b0101,0xfb400108,0xd01d0101,0xfb400108,0xd01e0101,0xfb400108,0xd0200101,
	0xfb400108,0xd0220101,0xfb400108,0xd0230101,0xfb400108,0xd0240101,0xfb400108,0xd0270101,
	0xfb400108,0xd02b0101,0xfb400108,0xd02f0101,0xfb400108,0xd0300101,0xfb400108,0xd0310101,
	0xfb400108,0xd0320101,0xfb400108,0xd0330101,0xfb400108,0xd0340101,0xfb400108,0xd0350101,
	0xfb400108,0xd0360101,0xfb400108,0xd0370101,0xfb400108,0xd0380101,0xfb400108,0xd0390101,
	0xfb400108,0xd03b0101,0xfb400108,0xd03d0101,0xfb400108,0xd03f0101,0xfb400108,0xd0400101,
	0xfb400108,0xd0410101,0xfb400108,0xd0420101,0xfb400108,0xd0440101,0xfb400108,0xd0450101,
	0xfb400108,0xd0460101,0xfb400108,0xd0490101,0xfb400108,0xd04a0101,0xfb400108,0xd04b0101,
	0xfb400108,0xd04d0101,0xfb400108,0xd0500101,0xfb400108,0xd0510101,0xfb400108,0xd0520101,
	0xfb400108,0xd0530101,0xfb400108,0xd0540101,0xfb400108,0xd0560101,0xfb400108,0xd0570101,
	0xfb400108,0xd0580101,0xfb400108,0xd0590101,0xfb400108,0xd05b0101,0xfb400108,0xd05d0101,
	0xfb400108,0xd05e0101,0xfb400108,0xd05f0101,0xfb400108,0xd0600101,0xfb400108,0xd0610101,
	0xfb400108,0xd0620101,0xfb400108,0xd0630101,0xfb400108,0xd0640101,0xfb400108,0xd0660101,
	0xfb400108,0xd0670101,0xfb400108,0xd0680101,0xfb400108,0xd0690101,0xfb400108,0xd06a0101,
	0xfb400108,0xd06b0101,0xfb400108,0xd06d0101,0xfb400108,0xd06e0101,0xfb400108,0xd06f0101,
	0xfb400108,0xd0700101,0xfb400108,0xd0710101,0xfb400108,0xd0720101,0xfb400108,0xd0730101,
	0xfb400108,0xd0740101,0xfb400108,0xd0750101,0xfb400108,0xd0780101,0xfb400108,0xd0790101,
	0xfb400108,0xd07a0101,0xfb400108,0xd07c0101,0xfb400108,0xd07d0101,0xfb400108,0xd0810101,
	0xfb400108,0xd0820101,0xfb400108,0xd0830101,0xfb400108,0xd0840101,0xfb400108,0xd0860101,
	0xfb400108,0xd0870101,0xfb400108,0xd0890101,0xfb400108,0xd08a0101,0xfb400108,0xd08b0101,
	0xfb400108,0xd08c0101,0xfb400108,0xd08e0101,0xfb400108,0xd08f0101,0xfb400108,0xd0900101,
	0xfb400108,0xd0910101,0xfb400108,0xd0920101,0xfb400108,0xd0930101,0xfb400108,0xd0940101,
	0xfb400108,0xd0950101,0xfb400108,0xd0960101,0xfb400108,0xd0970101,0xfb400108,0xd0980101,
	0xfb400108,0xd0990101,0xfb400108,0xd09a0101,0xfb400108,0xd09b0101,0xfb400108,0xd09c0101,
	0xfb400108,0xd09d0101,0xfb400108,0xd09e0101,0xfb400108,0xd09f0101,0xfb400108,0xd0a00101,
	0xfb400108,0xd0a10101,0xfb400108,0xd0a20101,0xfb400108,0xd0a40101,0xfb400108,0xd0a60101,
	0xfb400108,0xd0aa0101,0xfb400108,0xd0ab0101,0xfb400108,0xd0ad0101,0xfb400108,0xd0ae0101,
	0xfb400108,0xd0af0101,0xfb400108,0xd0b00101,0xfb400108,0xd0b10101,0xfb400108,0xd0b30101,
	0xfb400108,0xd0b40101,0xfb400108,0xd0b50101,0xfb400108,0xd0b60101,0xfb400108,0xd0b70101,
	0xfb400108,0xd0b80101,0xfb400108,0xd0b90101,0xfb400108,0xd0bc0101,0xfb400108,0xd0bd0101,
	0xfb400108,0xd0be0101,0xfb400108,0xd0bf0101,0xfb400108,0xd0c00101,0xfb400108,0xd0c10101,
	0xfb400108,0xd0c20101,0xfb400108,0xd0c30101,0xfb400108,0xd0c40101,0xfb400108,0xd0c50101,
	0xfb400108,0xd0c60101,0xfb400108,0xd0c70101,0xfb400108,0xd0c80101,0xfb400108,0xd0c90101,
	0xfb400108,0xd0ca0101,0xfb400108,0xd0cb0101,0xfb400108,0xd0cc0101,0xfb400108,0xd0cd0101,
	0xfb400108,0xd0ce0101,0xfb400108,0xd0d00101,0xfb400108,0xd0d10101,0xfb400108,0xd0d20101,
	0xfb400108,0xd0d30101,0xfb400108,0xd0d40101,0xfb400108,0xd0d50101,0xfb400108,0xd0d70101,
	0xfb400108,0xd0d80101,0xfb400108,0xd0d90101,0xfb400108,0xd0db0101,0xfb400108,0xd0dc0101,
	0xfb400108,0xd0dd0101,0xfb400108,0xd0de0101,0xfb400108,0xd0df0101,0xfb400108,0xd0e00101,
	0xfb400108,0xd0e10101,0xfb400108,0xd0e20101,0xfb400108,0xd0e30101,0xfb400108,0xd0e40101,
	0xfb400108,0xd0e50101,0xfb400108,0xd0e80101,0xfb400108,0xd0e90101,0xfb400108,0xd0ea0101,
	0xfb400108,0xd0eb0101,0xfb400108,0xd0ef0101,0xfb400108,0xd0f00101,0xfb400108,0xd0f10101,
	0xfb400108,0xd0f20101,0xfb400108,0xd0f40101,0xfb400108,0xd0f60101,0xfb400108,0xd0f70101,
	0xfb400108,0xd0f80101,0xfb400108,0xd0f90101,0xfb400108,0xd0fa0101,0xfb400108,0xd0fc0101,
	0xfb400108,0xd0fd0101,0xfb400108,0xd0fe0101,0xfb400108,0xd0ff0101,0xfb400108,0xd1000101,
	0xfb400108,0xd1010101,0xfb400108,0xd1020101,0xfb400108,0xd1030101,0xfb400108,0xd1040101,
	0xfb400108,0xd1050101,0xfb400108,0xd1080101,0xfb400108,0xd1090101,0xfb400108,0xd10a0101,
	0xfb400108,0xd10c0101,0xfb400108,0xd10d0101,0xfb400108,0xd10e0101,0xfb400108,0xd10f0101,
	0xfb400108,0xd1100101,0xfb400108,0xd1110101,0xfb400108,0xd1130101,0xfb400108,0xd1140101,
	0xfb400108,0xd1150101,0xfb400108,0xd1160101,0xfb400108,0xd1170101,0xfb400108,0xd1180101,
	0xfb400108,0xd1190101,0xfb400108,0xd11a0101,0xfb400108,0xd11b0101,0xfb400108,0xd11c0101,
	0xfb400108,0xd11d0101,0xfb400108,0xd11e0101,0xfb400108,0xd11f0101,0xfb400108,0xd1200101,
	0xfb400108,0xd1220101,0xfb400108,0xd1230101,0xfb400108,0xd1240101,0xfb400108,0xd1250101,
	0xfb400108,0xd1260101,0xfb400108,0xd1270101,0xfb400108,0xd1280101,0xfb400108,0xd1290101,
	0xfb400108,0xd12a0101,0xfb400108,0xd12b0101,0xfb400108,0xd12c0101,0xfb400108,0xd12d0101,
	0xfb400108,0xd12e0101,0xfb400108,0xd12f0101,0xfb400108,0xd1300101,0xfb400108,0xd1310101,
	0xfb400108,0xd1320101,0xfb400108,0xd1330101,0xfb400108,0xd1340101,0xfb400108,0xd1350101,
	0xfb400108,0xd1360101,0xfb400108,0xd1370101,0xfb400108,0xd1380101,0xfb400108,0xd1390101,
	0xfb400108,0xd13a0101,0xfb400108,0xd13b0101,0xfb400108,0xd13c0101,0xfb400108,0xd13d0101,
	0xfb400108,0xd13e0101,0xfb400108,0xd1420101,0xfb400108,0xd1470101,0xfb400108,0xd14a0101,
	0xfb400108,0xd14c0101,0xfb400108,0xd14e0101,0xfb400108,0xd14f0101,0xfb400108,0xd1500101,
	0xfb400108,0xd1520101,0xfb400108,0xd1530101,0xfb400108,0xd1570101,0xfb400108,0xd1580101,
	0xfb400108,0xd1590101,0xfb400108,0xd15b0101,0xfb400108,0xd15d0101,0xfb400108,0xd15e0101,
	0xfb400108,0xd15f0101,0xfb400108,0xd1600101,0xfb400108,0xd1610101,0xfb400108,0xd1630101,
	0xfb400108,0xd1640101,0xfb400108,0xd1660101,0xfb400108,0xd1670101,0xfb400108,0xd1690101,
	0xfb400108,0xd16a0101,0xfb400108,0xd16f0101,0xfb400108,0xd1720101,0xfb400108,0xd17a0101,
	0xfb400108,0xd17e0101,0xfb400108,0xd17f0101,0xfb400108,0xd1830101,0xfb400108,0xd1840101,
	0xfb400108,0xd1860101,0xfb400108,0xd1870101,0xfb400108,0xd18a0101,0xfb400108,0xd18b0101,
	0xfb400108,0xd18e0101,0xfb400108,0xd18f0101,0xfb400108,0xd1900101,0xfb400108,0xd1910101,
	0xfb400108,0xd1930101,0xfb400108,0xd1940101,0xfb400108,0xd1980101,0xfb400108,0xd19a0101,
	0xfb400108,0xd19d0101,0xfb400108,0xd19e0101,0xfb400108,0xd19f0101,0xfb400108,0xd1a10101,
	0xfb400108,0xd1a30101,0xfb400108,0xd1a60101,0xfb400108,0xd1a70101,0xfb400108,0xd1a80101,
	0xfb400108,0xd1a90101,0xfb400108,0xd1aa0101,0xfb400108,0xd1ad0101,0xfb400108,0xd1ae0101,
	0xfb400108,0xd1b40101,0xfb400108,0xd1b80101,0xfb400108,0xd1b90101,0xfb400108,0xd1ba0101,
	0xfb400108,0xd1be0101,0xfb400108,0xd1bf0101,0xfb400108,0xd1c10101,0xfb400108,0xd1c20101,
	0xfb400108,0xd1c30101,0xfb400108,0xd1c50101,0xfb400108,0xd1c80101,0xfb400108,0xd1ca0101,
	0xfb400108,0xd1cd0101,0xfb400108,0xd1ce0101,0xfb400108,0xd1d00101,0xfb400108,0xd1d20101,
	0xfb400108,0xd1d30101,0xfb400108,0xd1d40101,0xfb400108,0xd1d50101,0xfb400108,0xd1d60101,
	0xfb400108,0xd1d70101,0xfb400108,0xd1d80101,0xfb400108,0xd1d90101,0xfb400108,0xd1da0101,
	0xfb400108,0xd1dc0101,0xfb400108,0xd1de0101,0xfb400108,0xd1df0101,0xfb400108,0xd1e20101,
	0xfb400108,0xd1e30101,0xfb400108,0xd1e50101,0xfb400108,0xd1e60101,0xfb400108,0xd1e70101,
	0xfb400108,0xd1e80101,0xfb400108,0xd1e90101,0xfb400108,0xd1ea0101,0xfb400108,0xd1ec0101,
	0xfb400108,0xd1ee0101,0xfb400108,0xd1f10101,0xfb400108,0xd1f20101,0xfb400108,0xd1f40101,
	0xfb400108,0xd1f70101,0xfb400108,0xd1fe0101,0xfb400108,0xd2040101,0xfb400108,0xd2050101,
	0xfb400108,0xd2090101,0xfb400108,0xd20b0101,0xfb400108,0xd20c0101,0xfb400108,0xd20f0101,
	0xfb400108,0xd2100101,0xfb400108,0xd2130101,0xfb400108,0xd2140101,0xfb400108,0xd2150101,
	0xfb400108,0xd21c0101,0xfb400108,0xd21e0101,0xfb400108,0xd21f0101,0xfb400108,0xd2210101,
	0xfb400108,0xd2220101,0xfb400108,0xd2230101,0xfb400108,0xd2250101,0xfb400108,0xd2260101,
	0xfb400108,0xd2270101,0xfb400108,0xd22a0101,0xfb400108,0xd22c0101,0xfb400108,0xd22f0101,
	0xfb400108,0xd2310101,0xfb400108,0xd2320101,0xfb400108,0xd2340101,0xfb400108,0xd2350101,
	0xfb400108,0xd23c0101,0xfb400108,0xd23e0101,0xfb400108,0xd2440101,0xfb400108,0xd2450101,
	0xfb400108,0xd2460101,0xfb400108,0xd2470101,0xfb400108,0xd2480101,0xfb400108,0xd2490101,
	0xfb400108,0xd24b0101,0xfb400108,0xd24e0101,0xfb400108,0xd24f0101,0xfb400108,0xd2520101,
	0xfb400108,0xd2530101,0xfb400108,0xd2550101,0xfb400108,0xd2570101,0xfb400108,0xd2580101,
	0xfb400108,0xd2590101,0xfb400108,0xd25a0101,0xfb400108,0xd25b0101,0xfb400108,0xd25d0101,
	0xfb400108,0xd25f0101,0xfb400108,0xd2600101,0xfb400108,0xd2620101,0xfb400108,0xd2630101,
	0xfb400108,0xd2640101,0xfb400108,0xd2660101,0xfb400108,0xd2680101,0xfb400108,0xd26b0101,
	0xfb400108,0xd26c0101,0xfb400108,0xd26d0101,0xfb400108,0xd26e0101,0xfb400108,0xd2700101,
	0xfb400108,0xd2710101,0xfb400108,0xd2730101,0xfb400108,0xd2740101,0xfb400108,0xd2750101,
	0xfb400108,0xd2760101,0xfb400108,0xd2770101,0xfb400108,0xd2780101,0xfb400108,0xd2790101,
	0xfb400108,0xd27a0101,0xfb400108,0xd27b0101,0xfb400108,0xd27c0101,0xfb400108,0xd27e0101,
	0xfb400108,0xd2800101,0xfb400108,0xd2830101,0xfb400108,0xd2840101,0xfb400108,0xd2850101,
	0xfb400108,0xd2860101,0xfb400108,0xd2870101,0xfb400108,0xd2890101,0xfb400108,0xd28a0101,
	0xfb400108,0xd28b0101,0xfb400108,0xd28c0101,0xfb400108,0xd28d0101,0xfb400108,0xd28e0101,
	0xfb400108,0xd28f0101,0xfb400108,0xd2910101,0xfb400108,0xd2920101,0xfb400108,0xd2940101,
	0xfb400108,0xd2950101,0xfb400108,0xd2960101,0xfb400108,0xd2970101,0xfb400108,0xd2980101,
	0xfb400108,0xd2990101,0xfb400108,0xd29a0101,0xfb400108,0xd29c0101,0xfb400108,0xd2a40101,
	0xfb400108,0xd2a50101,0xfb400108,0xd2a60101,0xfb400108,0xd2a70101,0xfb400108,0xd2ae0101,
	0xfb400108,0xd2af0101,0xfb400108,0xd2b00101,0xfb400108,0xd2b40101,0xfb400108,0xd2b50101,
	0xfb400108,0xd2b60101,0xfb400108,0xd2b70101,0xfb400108,0xd2b80101,0xfb400108,0xd2b90101,
	0xfb400108,0xd2ba0101,0xfb400108,0xd2bb0101,0xfb400108,0xd2bc0101,0xfb400108,0xd2bd0101,
	0xfb400108,0xd2c00101,0xfb400108,0xd2c10101,0xfb400108,0xd2c20101,0xfb400108,0xd2c40101,
	0xfb400108,0xd2c50101,0xfb400108,0xd2c60101,0xfb400108,0xd2c80101,0xfb400108,0xd2ca0101,
	0xfb400108,0xd2cc0101,0xfb400108,0xd2cd0101,0xfb400108,0xd2ce0101,0xfb400108,0xd2cf0101,
	0xfb400108,0xd2d10101,0xfb400108,0xd2d30101,0xfb400108,0xd2d40101,0xfb400108,0xd2d50101,
	0xfb400108,0xd2d70101,0xfb400108,0xd2d90101,0xfb400108,0xd2da0101,0xfb400108,0xd2db0101,
	0xfb400108,0xd2dc0101,0xfb400108,0xd2dd0101,0xfb400108,0xd2de0101,0xfb400108,0xd2e00101,
	0xfb400108,0xd2e10101,0xfb400108,0xd2e20101,0xfb400108,0xd2e30101,0xfb400108,0xd2e50101,
	0xfb400108,0xd2e60101,0xfb400108,0xd2e70101,0xfb400108,0xd2e80101,0xfb400108,0xd2e90101,
	0xfb400108,0xd2ea0101,0xfb400108,0xd2eb0101,0xfb400108,0xd2ec0101,0xfb400108,0xd2ed0101,
	0xfb400108,0xd2ee0101,0xfb400108,0xd2ef0101,0xfb400108,0xd2f10101,0xfb400108,0xd2f20101,
	0xfb400108,0xd2f30101,0xfb400108,0xd2f40101,0xfb400108,0xd2f50101,0xfb400108,0xd2f60101,
	0xfb400108,0xd2f70101,0xfb400108,0xd2f80101,0xfb400108,0xd2fb0101,0xfb400108,0xd2fc0101,
	0xfb400108,0xd2fd0101,0xfb400108,0xd3010101,0xfb400108,0xd3020101,0xfb400108,0xd3030101,
	0xfb400108,0xd3040101,0xfb400108,0xd3070101,0xfb400108,0xd3090101,0xfb400108,0xd30a0101,
	0xfb400108,0xd30b0101,0xfb400108,0xd30c0101,0xfb400108,0xd30e0101,0xfb400108,0xd3110101,
	0xfb400108,0xd3120101,0xfb400108,0xd3130101,0xfb400108,0xd3140101,0xfb400108,0xd3180101,
	0xfb400108,0xd31b0101,0xfb400108,0xd31c0101,0xfb400108,0xd31e0101,0xfb400108,0xd31f0101,
	0xfb400108,0xd3220101,0xfb400108,0xd3240101,0xfb400108,0xd3250101,0xfb400108,0xd3270101,
	0xfb400108,0xd3280101,0xfb400108,0xd3290101,0xfb400108,0xd32b0101,0xfb400108,0xd32c0101,
	0xfb400108,0xd32d0101,0xfb400108,0xd32f0101,0xfb400108,0xd3300101,0xfb400108,0xd3310101,
	0xfb400108,0xd3320101,0xfb400108,0xd3330101,0xfb400108,0xd3340101,0xfb400108,0xd3350101,
	0xfb400108,0xd3360101,0xfb400108,0xd3370101,0xfb400108,0xd3380101,0xfb400108,0xd33c0101,
	0xfb400108,0xd33d0101,0xfb400108,0xd3400101,0xfb400108,0xd3420101,0xfb400108,0xd3440101,
	0xfb400108,0xd3460101,0xfb400108,0xd34b0101,0xfb400108,0xd34c0101,0xfb400108,0xd34d0101,
	0xfb400108,0xd3500101,0xfb400108,0xd3540101,0xfb400108,0xd3580101,0xfb400108,0xd3590101,
	0xfb400108,0xd35b0101,0xfb400108,0xd35d0101,0xfb400108,0xd3650101,0xfb400108,0xd3680101,
	0xfb400108,0xd36a0101,0xfb400108,0xd36c0101,0xfb400108,0xd36d0101,0xfb400108,0xd3720101,
	0xfb400108,0xd3760101,0xfb400108,0xd3790101,0xfb400108,0xd37b0101,0xfb400108,0xd37c0101,
	0xfb400108,0xd37d0101,0xfb400108,0xd37e0101,0xfb400108,0xd3800101,0xfb400108,0xd3810101,
	0xfb400108,0xd3830101,0xfb400108,0xd3870101,0xfb400108,0xd3880101,0xfb400108,0xd38a0101,
	0xfb400108,0xd38e0101,0xfb400108,0xd38f0101,0xfb400108,0xd3900101,0xfb400108,0xd3910101,
	0xfb400108,0xd3920101,0xfb400108,0xd3930101,0xfb400108,0xd3940101,0xfb400108,0xd3960101,
	0xfb400108,0xd3970101,0xfb400108,0xd3990101,0xfb400108,0xd39b0101,0xfb400108,0xd39c0101,
	0xfb400108,0xd39e0101,0xfb400108,0xd3a00101,0xfb400108,0xd3a10101,0xfb400108,0xd3a40101,
	0xfb400108,0xd3a70101,0xfb400108,0xd3aa0101,0xfb400108,0xd3ab0101,0xfb400108,0xd3ac0101,
	0xfb400108,0xd3ad0101,0xfb400108,0xd3af0101,0xfb400108,0xd3b00101,0xfb400108,0xd3b10101,
	0xfb400108,0xd3b20101,0xfb400108,0xd3b30101,0xfb400108,0xd3b40101,0xfb400108,0xd3b50101,
	0xfb400108,0xd3b70101,0xfb400108,0xd3b80101,0xfb400108,0xd3b90101,0xfb400108,0xd3ba0101,
	0xfb400108,0xd3bc0101,0xfb400108,0xd3bd0101,0xfb400108,0xd3be0101,0xfb400108,0xd3c00101,
	0xfb400108,0xd3c30101,0xfb400108,0xd3c40101,0xfb400108,0xd3c50101,0xfb400108,0xd3c60101,
	0xfb400108,0xd3c70101,0xfb400108,0xd3ce0101,0xfb400108,0xd3cf0101,0xfb400108,0xd3d00101,
	0xfb400108,0xd3d20101,0xfb400108,0xd3d30101,0xfb400108,0xd3d50101,0xfb400108,0xd3da0101,
	0xfb400108,0xd3dc0101,0xfb400108,0xd3dd0101,0xfb400108,0xd3de0101,0xfb400108,0xd3e10101,
	0xfb400108,0xd3e20101,0xfb400108,0xd3e70101,0xfb400108,0xd3f40101,0xfb400108,0xd3fa0101,
	0xfb400108,0xd3fe0101,0xfb400108,0xd3ff0101,0xfb400108,0xd4000101,0xfb400108,0xd4020101,
	0xfb400108,0xd4050101,0xfb400108,0xd4070101,0xfb400108,0xd40b0101,0xfb400108,0xd4140101,
	0xfb400108,0xd4180101,0xfb400108,0xd4190101,0xfb400108,0xd41a0101,0xfb400108,0xd41c0101,
	0xfb400108,0xd4220101,0xfb400108,0xd4240101,0xfb400108,0xd4250101,0xfb400108,0xd42a0101,
	0xfb400108,0xd4300101,0xfb400108,0xd4330101,0xfb400108,0xd4360101,0xfb400108,0xd4370101,
	0xfb400108,0xd43a0101,0xfb400108,0xd43d0101,0xfb400108,0xd43f0101,0xfb400108,0xd4410101,
	0xfb400108,0xd4420101,0xfb400108,0xd4440101,0xfb400108,0xd4450101,0xfb400108,0xd4470101,
	0xfb400108,0xd4490101,0xfb400108,0xd44c0101,0xfb400108,0xd44d0101,0xfb400108,0xd44e0101,
	0xfb400108,0xd44f0101,0xfb400108,0xd4510101,0xfb400108,0xd45a0101,0xfb400108,0xd45d0101,
	0xfb400108,0xd45e0101,0xfb400108,0xd45f0101,0xfb400108,0xd4600101,0xfb400108,0xd4610101,
	0xfb400108,0xd4630101,0xfb400108,0xd4650101,0xfb400108,0xd4670101,0xfb400108,0xd4690101,
	0xfb400108,0xd46a0101,0xfb400108,0xd46b0101,0xfb400108,0xd46c0101,0xfb400108,0xd46d0101,
	0xfb400108,0xd46e0101,0xfb400108,0xd46f0101,0xfb400108,0xd4700101,0xfb400108,0xd4740101,
	0xfb400108,0xd4790101,0xfb400108,0xd47a0101,0xfb400108,0xd47e0101,0xfb400108,0xd47f0101,
	0xfb400108,0xd4810101,0xfb400108,0xd4830101,0xfb400108,0xd4850101,0xfb400108,0xd4870101,
	0xfb400108,0xd4880101,0xfb400108,0xd4890101,0xfb400108,0xd48a0101,0xfb400108,0xd48d0101,
	0xfb400108,0xd4910101,0xfb400108,0xd4930101,0xfb400108,0xd4970101,0xfb400108,0xd4980101,
	0xfb400108,0xd49c0101,0xfb400108,0xd49e0101,0xfb400108,0xd49f0101,0xfb400108,0xd4a00101,
	0xfb400108,0xd4a10101,0xfb400108,0xd4a20101,0xfb400108,0xd4a50101,0xfb400108,0xd4ae0101,
	0xfb400108,0xd4b00101,0xfb400108,0xd4b20101,0xfb400108,0xd4b50101,0xfb400108,0xd4b60101,
	0xfb400108,0xd4b70101,0xfb400108,0xd4b90101,0xfb400108,0xd4ba0101,0xfb400108,0xd4bc0101,
	0xfb400108,0xd4be0101,0xfb400108,0xd4c30101,0xfb400108,0xd4c50101,0xfb400108,0xd4ca0101,
	0xfb400108,0xd4cb0101,0xfb400108,0xd4d60101,0xfb400108,0xd4d80101,0xfb400108,0xd4db0101,
	0xfb400108,0xd4e00101,0xfb400108,0xd4e10101,0xfb400108,0xd4e20101,0xfb400108,0xd4e30101,
	0xfb400108,0xd4e40101,0xfb400108,0xd4eb0101,0xfb400108,0xd4ec0101,0xfb400108,0xd4ef0101,
	0xfb400108,0xd4f00101,0xfb400108,0xd4f10101,0xfb400108,0xd4f40101,0xfb400108,0xd4f50101,
	0xfb400108,0xd4f60101,0xfb400108,0xd4f70101,0xfb400108,0xd4f80101,0xfb400108,0xd4f90101,
	0xfb400108,0xd4fb0101,0xfb400108,0xd4fe0101,0xfb400108,0xd5000101,0xfb400108,0xd5020101,
	0xfb400108,0xd5030101,0xfb400108,0xd5040101,0xfb400108,0xd5050101,0xfb400108,0xd5080101,
	0xfb400108,0xd50a0101,0xfb400108,0xd50b0101,0xfb400108,0xd50c0101,0xfb400108,0xd50d0101,
	0xfb400108,0xd50e0101,0xfb400108,0xd5120101,0xfb400108,0xd5130101,0xfb400108,0xd5150101,
	0xfb400108,0xd5160101,0xfb400108,0xd5170101,0xfb400108,0xd5180101,0xfb400108,0xd5190101,
	0xfb400108,0xd51a0101,0xfb400108,0xd51c0101,0xfb400108,0xd51d0101,0xfb400108,0xd51e0101,
	0xfb400108,0xd51f0101,0xfb400108,0xd5210101,0xfb400108,0xd5250101,0xfb400108,0xd5260101,
	0xfb400108,0xd5280101,0xfb400108,0xd5290101,0xfb400108,0xd52b0101,0xfb400108,0xd52d0101,
	0xfb400108,0xd5320101,0xfb400108,0xd5340101,0xfb400108,0xd5350101,0xfb400108,0xd5360101,
	0xfb400108,0xd5380101,0xfb400108,0xd5390101,0xfb400108,0xd53a0101,0xfb400108,0xd53b0101,
	0xfb400108,0xd53d0101,0xfb400108,0xd5400101,0xfb400108,0xd5420101,0xfb400108,0xd5450101,
	0xfb400108,0xd5470101,0xfb400108,0xd5480101,0xfb400108,0xd54b0101,0xfb400108,0xd54c0101,
	0xfb400108,0xd54d0101,0xfb400108,0xd54e0101,0xfb400108,0xd54f0101,0xfb400108,0xd5510101,
	0xfb400108,0xd5520101,0xfb400108,0xd5530101,0xfb400108,0xd5540101,0xfb400108,0xd5570101,
	0xfb400108,0xd5580101,0xfb400108,0xd5590101,0xfb400108,0xd55a0101,0xfb400108,0xd55b0101,
	0xfb400108,0xd55d0101,0xfb400108,0xd55e0101,0xfb400108,0xd55f0101,0xfb400108,0xd5600101,
	0xfb400108,0xd5620101,0xfb400108,0xd5630101,0xfb400108,0xd5680101,0xfb400108,0xd5690101,
	0xfb400108,0xd56b0101,0xfb400108,0xd56f0101,0xfb400108,0xd5700101,0xfb400108,0xd5710101,
	0xfb400108,0xd5720101,0xfb400108,0xd5730101,0xfb400108,0xd5740101,0xfb400108,0xd5790101,
	0xfb400108,0xd57a0101,0xfb400108,0xd57d0101,0xfb400108,0xd57f0101,0xfb400108,0xd5850101,
	0xfb400108,0xd5860101,0xfb400108,0xd58c0101,0xfb400108,0xd58d0101,0xfb400108,0xd58e0101,
	0xfb400108,0xd5900101,0xfb400108,0xd5920101,0xfb400108,0xd5930101,0xfb400108,0xd5950101,
	0xfb400108,0xd5960101,0xfb400108,0xd5970101,0xfb400108,0xd59a0101,0xfb400108,0xd59b0101,
	0xfb400108,0xd59e0101,0xfb400108,0xd5a00101,0xfb400108,0xd5a10101,0xfb400108,0xd5a20101,
	0xfb400108,0xd5a30101,0xfb400108,0xd5a40101,0xfb400108,0xd5a50101,0xfb400108,0xd5a60101,
	0xfb400108,0xd5a80101,0xfb400108,0xd5a90101,0xfb400108,0xd5aa0101,0xfb400108,0xd5ab0101,
	0xfb400108,0xd5ac0101,0xfb400108,0xd5ad0101,0xfb400108,0xd5ae0101,0xfb400108,0xd5af0101,
	0xfb400108,0xd5b00101,0xfb400108,0xd5b20101,0xfb400108,0xd5b40101,0xfb400108,0xd5b60101,
	0xfb400108,0xd5b80101,0xfb400108,0xd5ba0101,0xfb400108,0xd5bc0101,0xfb400108,0xd5bf0101,
	0xfb400108,0xd5c00101,0xfb400108,0xd5c10101,0xfb400108,0xd5c20101,0xfb400108,0xd5c30101,
	0xfb400108,0xd5c60101,0xfb400108,0xd5c70101,0xfb400108,0xd5c80101,0xfb400108,0xd5ca0101,
	0xfb400108,0xd5cb0101,0xfb400108,0xd5ce0101,0xfb400108,0xd5cf0101,0xfb400108,0xd5d00101,
	0xfb400108,0xd5d50101,0xfb400108,0xd5d70101,0xfb400108,0xd5d80101,0xfb400108,0xd5d90101,
	0xfb400108,0xd5da0101,0xfb400108,0xd5db0101,0xfb400108,0xd5de0101,0xfb400108,0xd5e00101,
	0xfb400108,0xd5e20101,0xfb400108,0xd5e70101,0xfb400108,0xd5e90101,0xfb400108,0xd5ed0101,
	0xfb400108,0xd5ee0101,0xfb400108,0xd5f00101,0xfb400108,0xd5f10101,0xfb400108,0xd5f40101,
	0xfb400108,0xd5f60101,0xfb400108,0xd5f80101,0xfb400108,0xd5f90101,0xfb400108,0xd5fa0101,
	0xfb400108,0xd5fb0101,0xfb400108,0xd5fc0101,0xfb400108,0xd5ff0101,0xfb400108,0xd6020101,
	0xfb400108,0xd6030101,0xfb400108,0xd6040101,0xfb400108,0xd6050101,0xfb400108,0xd6060101,
	0xfb400108,0xd6070101,0xfb400108,0xd60a0101,0xfb400108,0xd60b0101,0xfb400108,0xd60d0101,
	0xfb400108,0xd6100101,0xfb400108,0xd6110101,0xfb400108,0xd6120101,0xfb400108,0xd6130101,
	0xfb400108,0xd6140101,0xfb400108,0xd6150101,0xfb400108,0xd6160101,0xfb400108,0xd6170101,
	0xfb400108,0xd6190101,0xfb400108,0xd61a0101,0xfb400108,0xd61c0101,0xfb400108,0xd61d0101,
	0xfb400108,0xd6200101,0xfb400108,0xd6210101,0xfb400108,0xd6220101,0xfb400108,0xd6250101,
	0xfb400108,0xd6260101,0xfb400108,0xd6280101,0xfb400108,0xd6290101,0xfb400108,0xd62a0101,
	0xfb400108,0xd62b0101,0xfb400108,0xd62e0101,0xfb400108,0xd62f0101,0xfb400108,0xd6300101,
	0xfb400108,0xd6330101,0xfb400108,0xd6350101,0xfb400108,0xd6370101,0xfb400108,0xd6380101,
	0xfb400108,0xd63a0101,0xfb400108,0xd63c0101,0xfb400108,0xd63d0101,0xfb400108,0xd63e0101,
	0xfb400108,0xd6400101,0xfb400108,0xd6410101,0xfb400108,0xd6420101,0xfb400108,0xd6430101,
	0xfb400108,0xd6440101,0xfb400108,0xd6450101,0xfb400108,0xd6460101,0xfb400108,0xd6470101,
	0xfb400108,0xd6480101,0xfb400108,0xd6490101,0xfb400108,0xd64a0101,0xfb400108,0xd64b0101,
	0xfb400108,0xd64f0101,0xfb400108,0xd6500101,0xfb400108,0xd6510101,0xfb400108,0xd6520101,
	0xfb400108,0xd6530101,0xfb400108,0xd6550101,0xfb400108,0xd6560101,0xfb400108,0xd65a0101,
	0xfb400108,0xd65b0101,0xfb400108,0xd65d0101,0xfb400108,0xd65e0101,0xfb400108,0xd65f0101,
	0xfb400108,0xd6600101,0xfb400108,0xd6610101,0xfb400108,0xd6630101,0xfb400108,0xd6650101,
	0xfb400108,0xd6660101,0xfb400108,0xd6670101,0xfb400108,0xd66d0101,0xfb400108,0xd66e0101,
	0xfb400108,0xd66f0101,0xfb400108,0xd6700101,0xfb400108,0xd6720101,0xfb400108,0xd6730101,
	0xfb400108,0xd6740101,0xfb400108,0xd6750101,0xfb400108,0xd6770101,0xfb400108,0xd6780101,
	0xfb400108,0xd6790101,0xfb400108,0xd67a0101,0xfb400108,0xd67d0101,0xfb400108,0xd67e0101,
	0xfb400108,0xd67f0101,0xfb400108,0xd6800101,0xfb400108,0xd6810101,0xfb400108,0xd6820101,
	0xfb400108,0xd6830101,0xfb400108,0xd6840101,0xfb400108,0xd6870101,0xfb400108,0xd6880101,
	0xfb400108,0xd6890101,0xfb400108,0xd68a0101,0xfb400108,0xd68b0101,0xfb400108,0xd68c0101,
	0xfb400108,0xd68d0101,0xfb400108,0xd6900101,0xfb400108,0xd6910101,0xfb400108,0xd6920101,
	0xfb400108,0xd6940101,0xfb400108,0xd6950101,0xfb400108,0xd6960101,0xfb400108,0xd6970101,
	0xfb400108,0xd6980101,0xfb400108,0xd6990101,0xfb400108,0xd69a0101,0xfb400108,0xd69b0101,
	0xfb400108,0xd69c0101,0xfb400108,0xd69d0101,0xfb400108,0xd69e0101,0xfb400108,0xd69f0101,
	0xfb400108,0xd6a00101,0xfb400108,0xd6a10101,0xfb400108,0xd6a20101,0xfb400108,0xd6a40101,
	0xfb400108,0xd6a50101,0xfb400108,0xd6a60101,0xfb400108,0xd6a70101,0xfb400108,0xd6a80101,
	0xfb400108,0xd6a90101,0xfb400108,0xd6aa0101,0xfb400108,0xd6ab0101,0xfb400108,0xd6ac0101,
	0xfb400108,0xd6ad0101,0xfb400108,0xd6ae0101,0xfb400108,0xd6b00101,0xfb400108,0xd6b10101,
	0xfb400108,0xd6b20101,0xfb400108,0xd6b30101,0xfb400108,0xd6b40101,0xfb400108,0xd6b50101,
	0xfb400108,0xd6b60101,0xfb400108,0xd6b80101,0xfb400108,0xd6b90101,0xfb400108,0xd6ba0101,
	0xfb400108,0xd6bb0101,0xfb400108,0xd6bd0101,0xfb400108,0xd6be0101,0xfb400108,0xd6bf0101,
	0xfb400108,0xd6c00101,0xfb400108,0xd6c10101,0xfb400108,0xd6c20101,0xfb400108,0xd6c30101,
	0xfb400108,0xd6c40101,0xfb400108,0xd6c50101,0xfb400108,0xd6c60101,0xfb400108,0xd6c70101,
	0xfb400108,0xd6c80101,0xfb400108,0xd6c90101,0xfb400108,0xd6cb0101,0xfb400108,0xd6cc0101,
	0xfb400108,0xd6cd0101,0xfb400108,0xd6ce0101,0xfb400108,0xd6cf0101,0xfb400108,0xd6d00101,
	0xfb400108,0xd6d10101,0xfb400108,0xd6d20101,0xfb400108,0xd6d30101,0xfb400108,0xd6d50101,
	0xfb400108,0xd6d60101,0xfb400108,0xd6d80101,0xfb400108,0xd6d90101,0xfb400108,0xd6dc0101,
	0xfb400108,0xd6e30101,0xfb400108,0xd6e50101,0xfb400108,0xd6e60101,0xfb400108,0xd6e70101,
	0xfb400108,0xd6e80101,0xfb400108,0xd6e90101,0xfb400108,0xd6ea0101,0xfb400108,0xd6ec0101,
	0xfb400108,0xd6ee0101,0xfb400108,0xd6ef0101,0xfb400108,0xd6f20101,0xfb400108,0xd6f30101,
	0xfb400108,0xd6f60101,0xfb400108,0xd6f70101,0xfb400108,0xd6f80101,0xfb400108,0xd6fb0101,
	0xfb400108,0xd6fc0101,0xfb400108,0xd7000101,0xfb400108,0xd7010101,0xfb400108,0xd7020101,
	0xfb400108,0xd7050101,0xfb400108,0xd7070101,0xfb400108,0xd70b0101,0xfb400108,0xd70c0101,
	0xfb400108,0xd70d0101,0xfb400108,0xd70e0101,0xfb400108,0xd70f0101,0xfb400108,0xd7100101,
	0xfb400108,0xd7110101,0xfb400108,0xd7120101,0xfb400108,0xd7130101,0xfb400108,0xd7140101,
	0xfb400108,0xd7150101,0xfb400108,0xd7160101,0xfb400108,0xd7170101,0xfb400108,0xd7180101,
	0xfb400108,0xd7190101,0xfb400108,0xd71a0101,0xfb400108,0xd71b0101,0xfb400108,0xd71d0101,
	0xfb400108,0xd71e0101,0xfb400108,0xd7200101,0xfb400108,0xd7210101,0xfb400108,0xd7220101,
	0xfb400108,0xd7240101,0xfb400108,0xd7250101,0xfb400108,0xd7260101,0xfb400108,0xd7270101,
	0xfb400108,0xd72b0101,0xfb400108,0xd7310101,0xfb400108,0xd7320101,0xfb400108,0xd7340101,
	0xfb400108,0xd7350101,0xfb400108,0xd7360101,0xfb400108,0xd7370101,0xfb400108,0xd7380101,
	0xfb400108,0xd73c0101,0xfb400108,0xd73d0101,0xfb400108,0xd73f0101,0xfb400108,0xd7410101,
	0xfb400108,0xd7430101,0xfb400108,0xd7440101,0xfb400108,0xd7450101,0xfb400108,0xd7460101,
	0xfb400108,0xd7480101,0xfb400108,0xd7490101,0xfb400108,0xd74b0101,0xfb400108,0xd7520101,
	0xfb400108,0xd7530101,0xfb400108,0xd7540101,0xfb400108,0xd7550101,0xfb400108,0xd7560101,
	0xfb400108,0xd7580101,0xfb400108,0xd7590101,0xfb400108,0xd7620101,0xfb400108,0xd7630101,
	0xfb400108,0xd7650101,0xfb400108,0xd7670101,0xfb400108,0xd76c0101,0xfb400108,0xd76e0101,
	0xfb400108,0xd7700101,0xfb400108,0xd7710101,0xfb400108,0xd7720101,0xfb400108,0xd7740101,
	0xfb400108,0xd7750101,0xfb400108,0xd7780101,0xfb400108,0xd7790101,0xfb400108,0xd77a0101,
	0xfb400108,0xd77d0101,0xfb400108,0xd77e0101,0xfb400108,0xd77f0101,0xfb400108,0xd7800101,
	0xfb400108,0xd7810101,0xfb400108,0xd7870101,0xfb400108,0xd7880101,0xfb400108,0xd7890101,
	0xfb400108,0xd78a0101,0xfb400108,0xd78d0101,0xfb400108,0xd78e0101,0xfb400108,0xd78f0101,
	0xfb400108,0xd7900101,0xfb400108,0xd7910101,0xfb400108,0xd7940101,0xfb400108,0xd7950101,
	0xfb400108,0xd7960101,0xfb400108,0xd7970101,0xfb400108,0xd7980101,0xfb400108,0xd7990101,
	0xfb400108,0xd79a0101,0xfb400108,0xd79c0101,0xfb400108,0xd79d0101,0xfb400108,0xd79e0101,
	0xfb400108,0xd79f0101,0xfb400108,0xd7a50101,0xfb400108,0xd7a80101,0xfb400108,0xd7aa0101,
	0xfb400108,0xd7ac0101,0xfb400108,0xd7af0101,0xfb400108,0xd7b00101,0xfb400108,0xd7b10101,
	0xfb400108,0xd7b30101,0xfb400108,0xd7b50101,0xfb400108,0xd7b60101,0xfb400108,0xd7b70101,
	0xfb400108,0xd7b90101,0xfb400108,0xd7ba0101,0xfb400108,0xd7bb0101,0xfb400108,0xd7bc0101,
	0xfb400108,0xd7bd0101,0xfb400108,0xd7be0101,0xfb400108,0xd7bf0101,0xfb400108,0xd7c00101,
	0xfb400108,0xd7c10101,0xfb400108,0xd7c40101,0xfb400108,0xd7c50101,0xfb400108,0xd7c60101,
	0xfb400108,0xd7c70101,0xfb400108,0xd7c80101,0xfb400108,0xd7c90101,0xfb400108,0xd7ca0101,
	0xfb400108,0xd7cc0101,0xfb400108,0xd7cd0101,0xfb400108,0xd7d00101,0xfb400108,0xd7d10101,
	0xfb400108,0xd7d30101,0xfb400108,0xd7d60101,0xfb400108,0xd7d70101,0xfb400108,0xd7db0101,
	0xfb400108,0xd7dc0101,0xfb400108,0xd7de0101,0xfb400108,0xd7e10101,0xfb400108,0xd7e20101,
	0xfb400108,0xd7e30101,0xfb400108,0xd7e50101,0xfb400108,0xd7e60101,0xfb400108,0xd7e70101,
	0xfb400108,0xd7e80101,0xfb400108,0xd7e90101,0xfb400108,0xd7ea0101,0xfb400108,0xd7eb0101,
	0xfb400108,0xd7ec0101,0xfb400108,0xd7ee0101,0xfb400108,0xd7f00101,0xfb400108,0xd7f10101,
	0xfb400108,0xd7f20101,0xfb400108,0xd7f30101,0xfb400108,0xd7f50101,0xfb400108,0xd7f60101,
	0xfb400108,0xd7f70101,0xfb400108,0xd7fb0101,0xfb400108,0xd7fc0101,0xfb400108,0xd7fe0101,
	0xfb400108,0xd7ff0101,0xfb400108,0xd8010101,0xfb400108,0xd8030101,0xfb400108,0xd8040101,
	0xfb400108,0xd8050101,0xfb400108,0xd8080101,0xfb400108,0xd8090101,0xfb400108,0xd80a0101,
	0xfb400108,0xd80c0101,0xfb400108,0xd80e0101,0xfb400108,0xd80f0101,0xfb400108,0xd8100101,
	0xfb400108,0xd8120101,0xfb400108,0xd8130101,0xfb400108,0xd8140101,0xfb400108,0xd8160101,
	0xfb400108,0xd8170101,0xfb400108,0xd8180101,0xfb400108,0xd81a0101,0xfb400108,0xd81b0101,
	0xfb400108,0xd81c0101,0xfb400108,0xd81d0101,0xfb400108,0xd81f0101,0xfb400108,0xd8220101,
	0xfb400108,0xd8230101,0xfb400108,0xd8250101,0xfb400108,0xd8260101,0xfb400108,0xd8270101,
	0xfb400108,0xd8280101,0xfb400108,0xd8290101,0xfb400108,0xd82b0101,0xfb400108,0xd82c0101,
	0xfb400108,0xd82d0101,0xfb400108,0xd82e0101,0xfb400108,0xd82f0101,0xfb400108,0xd8310101,
	0xfb400108,0xd8320101,0xfb400108,0xd8330101,0xfb400108,0xd8340101,0xfb400108,0xd8360101,
	0xfb400108,0xd8370101,0xfb400108,0xd8380101,0xfb400108,0xd8390101,0xfb400108,0xd83a0101,
	0xfb400108,0xd83b0101,0xfb400108,0xd83c0101,0xfb400108,0xd83d0101,0xfb400108,0xd83e0101,
	0xfb400108,0xd83f0101,0xfb400108,0xd8400101,0xfb400108,0xd8410101,0xfb400108,0xd8420101,
	0xfb400108,0xd8430101,0xfb400108,0xd8450101,0xfb400108,0xd8460101,0xfb400108,0xd8470101,
	0xfb400108,0xd8480101,0xfb400108,0xd8490101,0xfb400108,0xd84a0101,0xfb400108,0xd84b0101,
	0xfb400108,0xd84e0101,0xfb400108,0xd84f0101,0xfb400108,0xd8500101,0xfb400108,0xd8520101,
	0xfb400108,0xd8530101,0xfb400108,0xd8550101,0xfb400108,0xd8560101,0xfb400108,0xd8570101,
	0xfb400108,0xd8590101,0xfb400108,0xd85a0101,0xfb400108,0xd85b0101,0xfb400108,0xd85c0101,
	0xfb400108,0xd85d0101,0xfb400108,0xd85f0101,0xfb400108,0xd8600101,0xfb400108,0xd8610101,
	0xfb400108,0xd8620101,0xfb400108,0xd8630101,0xfb400108,0xd8640101,0xfb400108,0xd8660101,
	0xfb400108,0xd8670101,0xfb400108,0xd8680101,0xfb400108,0xd8690101,0xfb400108,0xd86a0101,
	0xfb400108,0xd86d0101,0xfb400108,0xd86e0101,0xfb400108,0xd86f0101,0xfb400108,0xd8700101,
	0xfb400108,0xd8710101,0xfb400108,0xd8720101,0xfb400108,0xd8730101,0xfb400108,0xd8740101,
	0xfb400108,0xd8750101,0xfb400108,0xd8760101,0xfb400108,0xd8770101,0xfb400108,0xd8780101,
	0xfb400108,0xd8790101,0xfb400108,0xd87a0101,0xfb400108,0xd87b0101,0xfb400108,0xd87c0101,
	0xfb400108,0xd87d0101,0xfb400108,0xd87f0101,0xfb400108,0xd8820101,0xfb400108,0xd8840101,
	0xfb400108,0xd8860101,0xfb400108,0xd8870101,0xfb400108,0xd8880101,0xfb400108,0xd88a0101,
	0xfb400108,0xd88b0101,0xfb400108,0xd88c0101,0xfb400108,0xd88d0101,0xfb400108,0xd88e0101,
	0xfb400108,0xd88f0101,0xfb400108,0xd8900101,0xfb400108,0xd8910101,0xfb400108,0xd8940101,
	0xfb400108,0xd8950101,0xfb400108,0xd8960101,0xfb400108,0xd8970101,0xfb400108,0xd8980101,
	0xfb400108,0xd89b0101,0xfb400108,0xd89c0101,0xfb400108,0xd89d0101,0xfb400108,0xd8a00101,
	0xfb400108,0xd8a10101,0xfb400108,0xd8a20101,0xfb400108,0xd8a30101,0xfb400108,0xd8a40101,
	0xfb400108,0xd8a50101,0xfb400108,0xd8a60101,0xfb400108,0xd8a70101,0xfb400108,0xd8aa0101,
	0xfb400108,0xd8ab0101,0xfb400108,0xd8ac0101,0xfb400108,0xd8ad0101,0xfb400108,0xd8ae0101,
	0xfb400108,0xd8af0101,0xfb400108,0xd8b00101,0xfb400108,0xd8b10101,0xfb400108,0xd8b20101,
	0xfb400108,0xd8b30101,0xfb400108,0xd8b40101,0xfb400108,0xd8b50101,0xfb400108,0xd8b60101,
	0xfb400108,0xd8b70101,0xfb400108,0xd8b80101,0xfb400108,0xd8b90101,0xfb400108,0xd8ba0101,
	0xfb400108,0xd8bb0101,0xfb400108,0xd8bd0101,0xfb400108,0xd8be0101,0xfb400108,0xd8bf0101,
	0xfb400108,0xd8c00101,0xfb400108,0xd8c20101,0xfb400108,0xd8c30101,0xfb400108,0xd8c40101,
	0xfb400108,0xd8c60101,0xfb400108,0xd8c70101,0xfb400108,0xd8c80101,0xfb400108,0xd8c90101,
	0xfb400108,0xd8ca0101,0xfb400108,0xd8cb0101,0xfb400108,0xd8cc0101,0xfb400108,0xd8cd0101,
	0xfb400108,0xd8ce0101,0xfb400108,0xd8cf0101,0xfb400108,0xd8d00101,0xfb400108,0xd8d20101,
	0xfb400108,0xd8d30101,0xfb400108,0xd8d40101,0xfb400108,0xd8d60101,0xfb400108,0xd8d70101,
	0xfb400108,0xd8d80101,0xfb400108,0xd8d90101,0xfb400108,0xd8da0101,0xfb400108,0xd8db0101,
	0xfb400108,0xd8dc0101,0xfb400108,0xd8dd0101,0xfb400108,0xd8de0101,0xfb400108,0xd8df0101,
	0xfb400108,0xd8e00101,0xfb400108,0xd8e10101,0xfb400108,0xd8e20101,0xfb400108,0xd8e30101,
	0xfb400108,0xd8e50101,0xfb400108,0xd8e60101,0xfb400108,0xd8e70101,0xfb400108,0xd8e80101,
	0xfb400108,0xd8e90101,0xfb400108,0xd8ea0101,0xfb400108,0xd8ed0101,0xfb400108,0xd8ef0101,
	0xfb400108,0xd8f10101,0xfb400108,0xd8f20101,0xfb400108,0xd8f40101,0xfb400108,0xd8f50101,
	0xfb400108,0xd8f70101,0xfb400108,0xd8f80101,0xfb400108,0xd8fa0101,0xfb400108,0xd8fb0101,
	0xfb400108,0xd8fc0101,0xfb400108,0xd8fd0101,0xfb400108,0xd8fe0101,0xfb400108,0xd8ff0101,
	0xfb400108,0xd9000101,0xfb400108,0xd9010101,0xfb400108,0xd9030101,0xfb400108,0xd9050101,
	0xfb400108,0xd9060101,0xfb400108,0xd9080101,0xfb400108,0xd9090101,0xfb400108,0xd90a0101,
	0xfb400108,0xd90b0101,0xfb400108,0xd90c0101,0xfb400108,0xd90e0101,0xfb400108,0xd9100101,
	0xfb400108,0xd9110101,0xfb400108,0xd9120101,0xfb400108,0xd9130101,0xfb400108,0xd9170101,
	0xfb400108,0xd9180101,0xfb400108,0xd91b0101,0xfb400108,0xd91d0101,0xfb400108,0xd91e0101,
	0xfb400108,0xd9200101,0xfb400108,0xd9210101,0xfb400108,0xd9220101,0xfb400108,0xd9230101,
	0xfb400108,0xd9260101,0xfb400108,0xd9280101,0xfb400108,0xd92c0101,0xfb400108,0xd9300101,
	0xfb400108,0xd9320101,0xfb400108,0xd9330101,0xfb400108,0xd9350101,0xfb400108,0xd9360101,
	0xfb400108,0xd93b0101,0xfb400108,0xd93d0101,0xfb400108,0xd93e0101,0xfb400108,0xd93f0101,
	0xfb400108,0xd9400101,0xfb400108,0xd9430101,0xfb400108,0xd9450101,0xfb400108,0xd9460101,
	0xfb400108,0xd94a0101,0xfb400108,0xd94c0101,0xfb400108,0xd94d0101,0xfb400108,0xd9500101,
	0xfb400108,0xd9520101,0xfb400108,0xd9530101,0xfb400108,0xd9590101,0xfb400108,0xd95b0101,
	0xfb400108,0xd95c0101,0xfb400108,0xd95d0101,0xfb400108,0xd95e0101,0xfb400108,0xd95f0101,
	0xfb400108,0xd9610101,0xfb400108,0xd9630101,0xfb400108,0xd9640101,0xfb400108,0xd9660101,
	0xfb400108,0xd9670101,0xfb400108,0xd9680101,0xfb400108,0xd9690101,0xfb400108,0xd96a0101,
	0xfb400108,0xd96b0101,0xfb400108,0xd96c0101,0xfb400108,0xd96d0101,0xfb400108,0xd96e0101,
	0xfb400108,0xd96f0101,0xfb400108,0xd9700101,0xfb400108,0xd9710101,0xfb400108,0xd9720101,
	0xfb400108,0xd9750101,0xfb400108,0xd9770101,0xfb400108,0xd97a0101,0xfb400108,0xd97b0101,
	0xfb400108,0xd97c0101,0xfb400108,0xd97e0101,0xfb400108,0xd97f0101,0xfb400108,0xd9800101,
	0xfb400108,0xd9850101,0xfb400108,0xd9890101,0xfb400108,0xd98b0101,0xfb400108,0xd98c0101,
	0xfb400108,0xd98e0101,0xfb400108,0xd98f0101,0xfb400108,0xd9900101,0xfb400108,0xd9910101,
	0xfb400108,0xd9940101,0xfb400108,0xd9950101,0xfb400108,0xd9980101,0xfb400108,0xd99a0101,
	0xfb400108,0xd99b0101,0xfb400108,0xd99c0101,0xfb400108,0xd99d0101,0xfb400108,0xd99f0101,
	0xfb400108,0xd9a00101,0xfb400108,0xd9a10101,0xfb400108,0xd9a20101,0xfb400108,0xd9a60101,
	0xfb400108,0xd9a70101,0xfb400108,0xd9ac0101,0xfb400108,0xd9ad0101,0xfb400108,0xd9b00101,
	0xfb400108,0xd9b10101,0xfb400108,0xd9b30101,0xfb400108,0xd9b40101,0xfb400108,0xd9b50101,
	0xfb400108,0xd9b60101,0xfb400108,0xd9b70101,0xfb400108,0xd9b80101,0xfb400108,0xd9ba0101,
	0xfb400108,0xd9bc0101,0xfb400108,0xd9bd0101,0xfb400108,0xd9bf0101,0xfb400108,0xd9c00101,
	0xfb400108,0xd9c10101,0xfb400108,0xd9c20101,0xfb400108,0xd9c30101,0xfb400108,0xd9c40101,
	0xfb400108,0xd9c50101,0xfb400108,0xd9c70101,0xfb400108,0xd9c80101,0xfb400108,0xd9c90101,
	0xfb400108,0xd9cc0101,0xfb400108,0xd9cd0101,0xfb400108,0xd9ce0101,0xfb400108,0xd9cf0101,
	0xfb400108,0xd9d50101,0xfb400108,0xd9d60101,0xfb400108,0xd9d90101,0xfb400108,0xd9db0101,
	0xfb400108,0xd9de0101,0xfb400108,0xd9df0101,0xfb400108,0xd9e00101,0xfb400108,0xd9e10101,
	0xfb400108,0xd9e20101,0xfb400108,0xd9e40101,0xfb400108,0xd9e60101,0xfb400108,0xd9e70101,
	0xfb400108,0xd9e90101,0xfb400108,0xd9ea0101,0xfb400108,0xd9eb0101,0xfb400108,0xd9ed0101,
	0xfb400108,0xd9ee0101,0xfb400108,0xd9ef0101,0xfb400108,0xd9f00101,0xfb400108,0xd9f10101,
	0xfb400108,0xd9f20101,0xfb400108,0xd9f30101,0xfb400108,0xd9f40101,0xfb400108,0xd9f50101,
	0xfb400108,0xd9f60101,0xfb400108,0xd9f70101,0xfb400108,0xd9f80101,0xfb400108,0xd9fa0101,
	0xfb400108,0xd9fc0101,0xfb400108,0xd9fd0101,0xfb400108,0xd9fe0101,0xfb400108,0xda000101,
	0xfb400108,0xda020101,0xfb400108,0xda0a0101,0xfb400108,0xda0b0101,0xfb400108,0xda0d0101,
	0xfb400108,0xda0e0101,0xfb400108,0xda0f0101,0xfb400108,0xda100101,0xfb400108,0xda120101,
	0xfb400108,0xda140101,0xfb400108,0xda150101,0xfb400108,0xda160101,0xfb400108,0xda170101,
	0xfb400108,0xda190101,0xfb400108,0xda1a0101,0xfb400108,0xda1b0101,0xfb400108,0xda1d0101,
	0xfb400108,0xda1e0101,0xfb400108,0xda210101,0xfb400108,0xda220101,0xfb400108,0xda240101,
	0xfb400108,0xda260101,0xfb400108,0xda270101,0xfb400108,0xda280101,0xfb400108,0xda2a0101,
	0xfb400108,0xda2b0101,0xfb400108,0xda2c0101,0xfb400108,0xda2d0101,0xfb400108,0xda2e0101,
	0xfb400108,0xda2f0101,0xfb400108,0xda300101,0xfb400108,0xda330101,0xfb400108,0xda350101,
	0xfb400108,0xda370101,0xfb400108,0xda380101,0xfb400108,0xda390101,0xfb400108,0xda3a0101,
	0xfb400108,0xda3b0101,0xfb400108,0xda3d0101,0xfb400108,0xda3e0101,0xfb400108,0xda3f0101,
	0xfb400108,0xda410101,0xfb400108,0xda420101,0xfb400108,0xda430101,0xfb400108,0xda440101,
	0xfb400108,0xda450101,0xfb400108,0xda470101,0xfb400108,0xda480101,0xfb400108,0xda4b0101,
	0xfb400108,0xda4c0101,0xfb400108,0xda4d0101,0xfb400108,0xda4e0101,0xfb400108,0xda4f0101,
	0xfb400108,0xda500101,0xfb400108,0xda510101,0xfb400108,0xda520101,0xfb400108,0xda530101,
	0xfb400108,0xda540101,0xfb400108,0xda560101,0xfb400108,0xda570101,0xfb400108,0xda580101,
	0xfb400108,0xda590101,0xfb400108,0xda5b0101,0xfb400108,0xda5c0101,0xfb400108,0xda5d0101,
	0xfb400108,0xda5e0101,0xfb400108,0xda5f0101,0xfb400108,0xda600101,0xfb400108,0xda610101,
	0xfb400108,0xda630101,0xfb400108,0xda640101,0xfb400108,0xda650101,0xfb400108,0xda660101,
	0xfb400108,0xda680101,0xfb400108,0xda690101,0xfb400108,0xda6b0101,0xfb400108,0xda6c0101,
	0xfb400108,0xda6d0101,0xfb400108,0xda6e0101,0xfb400108,0xda6f0101,0xfb400108,0xda700101,
	0xfb400108,0xda710101,0xfb400108,0xda720101,0xfb400108,0xda730101,0xfb400108,0xda780101,
	0xfb400108,0xda790101,0xfb400108,0xda7b0101,0xfb400108,0xda7c0101,0xfb400108,0xda7d0101,
	0xfb400108,0xda7e0101,0xfb400108,0xda800101,0xfb400108,0xda810101,0xfb400108,0xda820101,
	0xfb400108,0xda830101,0xfb400108,0xda840101,0xfb400108,0xda850101,0xfb400108,0xda860101,
	0xfb400108,0xda870101,0xfb400108,0xda880101,0xfb400108,0xda890101,0xfb400108,0xda8a0101,
	0xfb400108,0xda8b0101,0xfb400108,0xda8c0101,0xfb400108,0xda8d0101,0xfb400108,0xda8e0101,
	0xfb400108,0xda8f0101,0xfb400108,0xda900101,0xfb400108,0xda910101,0xfb400108,0xda930101,
	0xfb400108,0xda940101,0xfb400108,0xda950101,0xfb400108,0xda960101,0xfb400108,0xda970101,
	0xfb400108,0xda980101,0xfb400108,0xda990101,0xfb400108,0xda9c0101,0xfb400108,0xda9d0101,
	0xfb400108,0xda9e0101,0xfb400108,0xda9f0101,0xfb400108,0xdaa00101,0xfb400108,0xdaa10101,
	0xfb400108,0xdaa20101,0xfb400108,0xdaa30101,0xfb400108,0xdaa40101,0xfb400108,0xdaa50101,
	0xfb400108,0xdaa60101,0xfb400108,0xdaa70101,0xfb400108,0xdaa80101,0xfb400108,0xdaa90101,
	0xfb400108,0xdaab0101,0xfb400108,0xdaac0101,0xfb400108,0xdaad0101,0xfb400108,0xdaae0101,
	0xfb400108,0xdaaf0101,0xfb400108,0xdab00101,0xfb400108,0xdab10101,0xfb400108,0xdab40101,
	0xfb400108,0xdab60101,0xfb400108,0xdab70101,0xfb400108,0xdab90101,0xfb400108,0xdaba0101,
	0xfb400108,0xdabb0101,0xfb400108,0xdabc0101,0xfb400108,0xdabd0101,0xfb400108,0xdabf0101,
	0xfb400108,0xdac00101,0xfb400108,0xdac30101,0xfb400108,0xdac40101,0xfb400108,0xdac50101,
	0xfb400108,0xdac60101,0xfb400108,0xdac70101,0xfb400108,0xdac80101,0xfb400108,0xdaca0101,
	0xfb400108,0xdacb0101,0xfb400108,0xdacd0101,0xfb400108,0xdace0101,0xfb400108,0xdacf0101,
	0xfb400108,0xdad00101,0xfb400108,0xdad10101,0xfb400108,0xdad30101,0xfb400108,0xdad50101,
	0xfb400108,0xdad70101,0xfb400108,0xdad90101,0xfb400108,0xdada0101,0xfb400108,0xdadb0101,
	0xfb400108,0xdadd0101,0xfb400108,0xdade0101,0xfb400108,0xdadf0101,0xfb400108,0xdae20101,
	0xfb400108,0xdae40101,0xfb400108,0xdae50101,0xfb400108,0xdae70101,0xfb400108,0xdae80101,
	0xfb400108,0xdaea0101,0xfb400108,0xdaec0101,0xfb400108,0xdaed0101,0xfb400108,0xdaee0101,
	0xfb400108,0xdaef0101,0xfb400108,0xdaf00101,0xfb400108,0xdaf20101,0xfb400108,0xdaf30101,
	0xfb400108,0xdaf40101,0xfb400108,0xdaf50101,0xfb400108,0xdaf60101,0xfb400108,0xdaf70101,
	0xfb400108,0xdaf80101,0xfb400108,0xdaf90101,0xfb400108,0xdafa0101,0xfb400108,0xdafb0101,
	0xfb400108,0xdafc0101,0xfb400108,0xdafd0101,0xfb400108,0xdafe0101,0xfb400108,0xdaff0101,
	0xfb400108,0xdb000101,0xfb400108,0xdb010101,0xfb400108,0xdb020101,0xfb400108,0xdb030101,
	0xfb400108,0xdb040101,0xfb400108,0xdb050101,0xfb400108,0xdb060101,0xfb400108,0xdb070101,
	0xfb400108,0xdb080101,0xfb400108,0xdb0a0101,0xfb400108,0xdb0b0101,0xfb400108,0xdb0c0101,
	0xfb400108,0xdb0d0101,0xfb400108,0xdb0e0101,0xfb400108,0xdb0f0101,0xfb400108,0xdb100101,
	0xfb400108,0xdb110101,0xfb400108,0xdb120101,0xfb400108,0xdb130101,0xfb400108,0xdb140101,
	0xfb400108,0xdb150101,0xfb400108,0xdb180101,0xfb400108,0xdb190101,0xfb400108,0xdb1a0101,
	0xfb400108,0xdb1b0101,0xfb400108,0xdb1c0101,0xfb400108,0xdb1d0101,0xfb400108,0xdb1e0101,
	0xfb400108,0xdb1f0101,0xfb400108,0xdb200101,0xfb400108,0xdb210101,0xfb400108,0xdb220101,
	0xfb400108,0xdb230101,0xfb400108,0xdb240101,0xfb400108,0xdb250101,0xfb400108,0xdb260101,
	0xfb400108,0xdb270101,0xfb400108,0xdb280101,0xfb400108,0xdb290101,0xfb400108,0xdb2a0101,
	0xfb400108,0xdb2b0101,0xfb400108,0xdb2c0101,0xfb400108,0xdb2d0101,0xfb400108,0xdb2e0101,
	0xfb400108,0xdb2f0101,0xfb400108,0xdb300101,0xfb400108,0xdb310101,0xfb400108,0xdb330101,
	0xfb400108,0xdb350101,0xfb400108,0xdb360101,0xfb400108,0xdb380101,0xfb400108,0xdb390101,
	0xfb400108,0xdb3a0101,0xfb400108,0xdb3b0101,0xfb400108,0xdb3c0101,0xfb400108,0xdb3d0101,
	0xfb400108,0xdb3e0101,0xfb400108,0xdb3f0101,0xfb400108,0xdb410101,0xfb400108,0xdb420101,
	0xfb400108,0xdb430101,0xfb400108,0xdb440101,0xfb400108,0xdb450101,0xfb400108,0xdb460101,
	0xfb400108,0xdb470101,0xfb400108,0xdb480101,0xfb400108,0xdb490101,0xfb400108,0xdb4a0101,
	0xfb400108,0xdb4b0101,0xfb400108,0xdb4c0101,0xfb400108,0xdb4d0101,0xfb400108,0xdb4e0101,
	0xfb400108,0xdb4f0101,0xfb400108,0xdb520101,0xfb400108,0xdb560101,0xfb400108,0xdb5e0101,
	0xfb400108,0xdb600101,0xfb400108,0xdb610101,0xfb400108,0xdb670101,0xfb400108,0xdb680101,
	0xfb400108,0xdb6b0101,0xfb400108,0xdb6d0101,0xfb400108,0xdb6e0101,0xfb400108,0xdb6f0101,
	0xfb400108,0xdb720101,0xfb400108,0xdb740101,0xfb400108,0xdb760101,0xfb400108,0xdb770101,
	0xfb400108,0xdb780101,0xfb400108,0xdb790101,0xfb400108,0xdb7b0101,0xfb400108,0xdb7c0101,
	0xfb400108,0xdb7e0101,0xfb400108,0xdb7f0101,0xfb400108,0xdb820101,0xfb400108,0xdb860101,
	0xfb400108,0xdb8a0101,0xfb400108,0xdb8d0101,0xfb400108,0xdb8e0101,0xfb400108,0xdb900101,
	0xfb400108,0xdb910101,0xfb400108,0xdb920101,0xfb400108,0xdb940101,0xfb400108,0xdb960101,
	0xfb400108,0xdb9f0101,0xfb400108,0xdba70101,0xfb400108,0xdba80101,0xfb400108,0xdba90101,
	0xfb400108,0xdbac0101,0xfb400108,0xdbad0101,0xfb400108,0xdbae0101,0xfb400108,0xdbaf0101,
	0xfb400108,0xdbb10101,0xfb400108,0xdbb20101,0xfb400108,0xdbb70101,0xfb400108,0xdbba0101,
	0xfb400108,0xdbbb0101,0xfb400108,0xdbbc0101,0xfb400108,0xdbc00101,0xfb400108,0xdbc10101,
	0xfb400108,0xdbc30101,0xfb400108,0xdbc80101,0xfb400108,0xdbc90101,0xfb400108,0xdbca0101,
	0xfb400108,0xdbcb0101,0xfb400108,0xdbcd0101,0xfb400108,0xdbce0101,0xfb400108,0xdbcf0101,
	0xfb400108,0xdbd10101,0xfb400108,0xdbd40101,0xfb400108,0xdbd50101,0xfb400108,0xdbd60101,
	0xfb400108,0xdbd70101,0xfb400108,0xdbd80101,0xfb400108,0xdbd90101,0xfb400108,0xdbda0101,
	0xfb400108,0xdbdb0101,0xfb400108,0xdbdc0101,0xfb400108,0xdbe00101,0xfb400108,0xdbe20101,
	0xfb400108,0xdbe30101,0xfb400108,0xdbe60101,0xfb400108,0xdbe70101,0xfb400108,0xdbe90101,
	0xfb400108,0xdbea0101,0xfb400108,0xdbeb0101,0xfb400108,0xdbec0101,0xfb400108,0xdbed0101,
	0xfb400108,0xdbef0101,0xfb400108,0xdbf10101,0xfb400108,0xdbf20101,0xfb400108,0xdbf30101,
	0xfb400108,0xdbf40101,0xfb400108,0xdbf50101,0xfb400108,0xdbf60101,0xfb400108,0xdbf70101,
	0xfb400108,0xdbfd0101,0xfb400108,0xdbfe0101,0xfb400108,0xdc000101,0xfb400108,0xdc020101,
	0xfb400108,0xdc030101,0xfb400108,0xdc050101,0xfb400108,0xdc070101,0xfb400108,0xdc080101,
	0xfb400108,0xdc0b0101,0xfb400108,0xdc0c0101,0xfb400108,0xdc0d0101,0xfb400108,0xdc0e0101,
	0xfb400108,0xdc100101,0xfb400108,0xdc120101,0xfb400108,0xdc130101,0xfb400108,0xdc170101,
	0xfb400108,0xdc190101,0xfb400108,0xdc1b0101,0xfb400108,0xdc1e0101,0xfb400108,0xdc1f0101,
	0xfb400108,0xdc200101,0xfb400108,0xdc210101,0xfb400108,0xdc230101,0xfb400108,0xdc260101,
	0xfb400108,0xdc280101,0xfb400108,0xdc290101,0xfb400108,0xdc2a0101,0xfb400108,0xdc2b0101,
	0xfb400108,0xdc2d0101,0xfb400108,0xdc2e0101,0xfb400108,0xdc2f0101,0xfb400108,0xdc300101,
	0xfb400108,0xdc320101,0xfb400108,0xdc330101,0xfb400108,0xdc350101,0xfb400108,0xdc360101,
	0xfb400108,0xdc370101,0xfb400108,0xdc430101,0xfb400108,0xdc440101,0xfb400108,0xdc460101,
	0xfb400108,0xdc470101,0xfb400108,0xdc4c0101,0xfb400108,0xdc4d0101,0xfb400108,0xdc520101,
	0xfb400108,0xdc530101,0xfb400108,0xdc540101,0xfb400108,0xdc560101,0xfb400108,0xdc570101,
	0xfb400108,0xdc580101,0xfb400108,0xdc5a0101,0xfb400108,0xdc5b0101,0xfb400108,0xdc5c0101,
	0xfb400108,0xdc5d0101,0xfb400108,0xdc5f0101,0xfb400108,0xdc620101,0xfb400108,0xdc640101,
	0xfb400108,0xdc670101,0xfb400108,0xdc680101,0xfb400108,0xdc690101,0xfb400108,0xdc6a0101,
	0xfb400108,0xdc6b0101,0xfb400108,0xdc6c0101,0xfb400108,0xdc6d0101,0xfb400108,0xdc700101,
	0xfb400108,0xdc720101,0xfb400108,0xdc730101,0xfb400108,0xdc740101,0xfb400108,0xdc750101,
	0xfb400108,0xdc760101,0xfb400108,0xdc770101,0xfb400108,0xdc780101,0xfb400108,0xdc7b0101,
	0xfb400108,0xdc7c0101,0xfb400108,0xdc7d0101,0xfb400108,0xdc7e0101,0xfb400108,0xdc800101,
	0xfb400108,0xdc830101,0xfb400108,0xdc840101,0xfb400108,0xdc850101,0xfb400108,0xdc860101,
	0xfb400108,0xdc870101,0xfb400108,0xdc890101,0xfb400108,0xdc8a0101,0xfb400108,0xdc8b0101,
	0xfb400108,0xdc8e0101,0xfb400108,0xdc8f0101,0xfb400108,0xdc920101,0xfb400108,0xdc930101,
	0xfb400108,0xdc950101,0xfb400108,0xdc9d0101,0xfb400108,0xdc9e0101,0xfb400108,0xdc9f0101,
	0xfb400108,0xdca00101,0xfb400108,0xdca10101,0xfb400108,0xdca40101,0xfb400108,0xdca50101,
	0xfb400108,0xdca60101,0xfb400108,0xdca70101,0xfb400108,0xdca80101,0xfb400108,0xdcaa0101,
	0xfb400108,0xdcae0101,0xfb400108,0xdcaf0101,0xfb400108,0xdcb00101,0xfb400108,0xdcb20101,
	0xfb400108,0xdcb40101,0xfb400108,0xdcb60101,0xfb400108,0xdcb90101,0xfb400108,0xdcba0101,
	0xfb400108,0xdcbb0101,0xfb400108,0xdcbc0101,0xfb400108,0xdcbe0101,0xfb400108,0xdcc00101,
	0xfb400108,0xdcc20101,0xfb400108,0xdcc30101,0xfb400108,0xdcc50101,0xfb400108,0xdcc60101,
	0xfb400108,0xdcc70101,0xfb400108,0xdcc80101,0xfb400108,0xdcc90101,0xfb400108,0xdcca0101,
	0xfb400108,0xdccc0101,0xfb400108,0xdccd0101,0xfb400108,0xdcce0101,0xfb400108,0xdccf0101,
	0xfb400108,0xdcd00101,0xfb400108,0xdcd10101,0xfb400108,0xdcd30101,0xfb400108,0xdcd40101,
	0xfb400108,0xdcd50101,0xfb400108,0xdcd60101,0xfb400108,0xdcd70101,0xfb400108,0xdcd80101,
	0xfb400108,0xdcda0101,0xfb400108,0xdcdb0101,0xfb400108,0xdcdc0101,0xfb400108,0xdcdd0101,
	0xfb400108,0xdcde0101,0xfb400108,0xdcdf0101,0xfb400108,0xdce00101,0xfb400108,0xdce20101,
	0xfb400108,0xdce30101,0xfb400108,0xdce70101,0xfb400108,0xdce90101,0xfb400108,0xdceb0101,
	0xfb400108,0xdcec0101,0xfb400108,0xdcee0101,0xfb400108,0xdcef0101,0xfb400108,0xdcf10101,
	0xfb400108,0xdcf20101,0xfb400108,0xdcf30101,0xfb400108,0xdcf40101,0xfb400108,0xdcf50101,
	0xfb400108,0xdcf60101,0xfb400108,0xdcf70101,0xfb400108,0xdcf80101,0xfb400108,0xdcf90101,
	0xfb400108,0xdcfa0101,0xfb400108,0xdcfc0101,0xfb400108,0xdcfd0101,0xfb400108,0xdcfe0101,
	0xfb400108,0xdcff0101,0xfb400108,0xdd000101,0xfb400108,0xdd010101,0xfb400108,0xdd040101,
	0xfb400108,0xdd050101,0xfb400108,0xdd080101,0xfb400108,0xdd090101,0xfb400108,0xdd0a0101,
	0xfb400108,0xdd0b0101,0xfb400108,0xdd0c0101,0xfb400108,0xdd0d0101,0xfb400108,0xdd0f0101,
	0xfb400108,0xdd100101,0xfb400108,0xdd110101,0xfb400108,0xdd120101,0xfb400108,0xdd130101,
	0xfb400108,0xdd150101,0xfb400108,0xdd170101,0xfb400108,0xdd180101,0xfb400108,0xdd190101,
	0xfb400108,0xdd1a0101,0xfb400108,0xdd1c0101,0xfb400108,0xdd1d0101,0xfb400108,0xdd1f0101,
	0xfb400108,0xdd200101,0xfb400108,0xdd210101,0xfb400108,0xdd220101,0xfb400108,0xdd230101,
	0xfb400108,0xdd250101,0xfb400108,0xdd280101,0xfb400108,0xdd2a0101,0xfb400108,0xdd2b0101,
	0xfb400108,0xdd2c0101,0xfb400108,0xdd2f0101,0xfb400108,0xdd300101,0xfb400108,0xdd310101,
	0xfb400108,0xdd320101,0xfb400108,0xdd330101,0xfb400108,0xdd350101,0xfb400108,0xdd360101,
	0xfb400108,0xdd370101,0xfb400108,0xdd380101,0xfb400108,0xdd390101,0xfb400108,0xdd3a0101,
	0xfb400108,0xdd3b0101,0xfb400108,0xdd3c0101,0xfb400108,0xdd3f0101,0xfb400108,0xdd400101,
	0xfb400108,0xdd410101,0xfb400108,0xdd420101,0xfb400108,0xdd430101,0xfb400108,0xdd440101,
	0xfb400108,0xdd450101,0xfb400108,0xdd460101,0xfb400108,0xdd480101,0xfb400108,0xdd490101,
	0xfb400108,0xdd4d0101,0xfb400108,0xdd4e0101,0xfb400108,0xdd4f0101,0xfb400108,0xdd500101,
	0xfb400108,0xdd510101,0xfb400108,0xdd520101,0xfb400108,0xdd530101,0xfb400108,0xdd540101,
	0xfb400108,0xdd550101,0xfb400108,0xdd560101,0xfb400108,0xdd570101,0xfb400108,0xdd590101,
	0xfb400108,0xdd5a0101,0xfb400108,0xdd5c0101,0xfb400108,0xdd5e0101,0xfb400108,0xdd5f0101,
	0xfb400108,0xdd600101,0xfb400108,0xdd610101,0xfb400108,0xdd620101,0xfb400108,0xdd630101,
	0xfb400108,0xdd640101,0xfb400108,0xdd650101,0xfb400108,0xdd660101,0xfb400108,0xdd670101,
	0xfb400108,0xdd680101,0xfb400108,0xdd6a0101,0xfb400108,0xdd6d0101,0xfb400108,0xdd6e0101,
	0xfb400108,0xdd700101,0xfb400108,0xdd710101,0xfb400108,0xdd720101,0xfb400108,0xdd730101,
	0xfb400108,0xdd750101,0xfb400108,0xdd760101,0xfb400108,0xdd770101,0xfb400108,0xdd780101,
	0xfb400108,0xdd790101,0xfb400108,0xdd7a0101,0xfb400108,0xdd7b0101,0xfb400108,0xdd7c0101,
	0xfb400108,0xdd7d0101,0xfb400108,0xdd7e0101,0xfb400108,0xdd7f0101,0xfb400108,0xdd800101,
	0xfb400108,0xdd810101,0xfb400108,0xdd830101,0xfb400108,0xdd840101,0xfb400108,0xdd850101,
	0xfb400108,0xdd860101,0xfb400108,0xdd870101,0xfb400108,0xdd880101,0xfb400108,0xdd890101,
	0xfb400108,0xdd8a0101,0xfb400108,0xdd8b0101,0xfb400108,0xdd8c0101,0xfb400108,0xdd8d0101,
	0xfb400108,0xdd8e0101,0xfb400108,0xdd8f0101,0xfb400108,0xdd900101,0xfb400108,0xdd910101,
	0xfb400108,0xdd920101,0xfb400108,0xdd930101,0xfb400108,0xdd940101,0xfb400108,0xdd950101,
	0xfb400108,0xdd960101,0xfb400108,0xdd970101,0xfb400108,0xdd980101,0xfb400108,0xdd9a0101,
	0xfb400108,0xdd9b0101,0xfb400108,0xdd9c0101,0xfb400108,0xdd9e0101,0xfb400108,0xdd9f0101,
	0xfb400108,0xdda00101,0xfb400108,0xdda10101,0xfb400108,0xdda20101,0xfb400108,0xdda30101,
	0xfb400108,0xdda40101,0xfb400108,0xdda50101,0xfb400108,0xdda60101,0xfb400108,0xdda70101,
	0xfb400108,0xdda80101,0xfb400108,0xdda90101,0xfb400108,0xddaa0101,0xfb400108,0xddab0101,
	0xfb400108,0xddac0101,0xfb400108,0xddad0101,0xfb400108,0xddae0101,0xfb400108,0xddaf0101,
	0xfb400108,0xddb00101,0xfb400108,0xddb10101,0xfb400108,0xddb20101,0xfb400108,0xddb30101,
	0xfb400108,0xddb40101,0xfb400108,0xddb50101,0xfb400108,0xddb60101,0xfb400108,0xddb80101,
	0xfb400108,0xddb90101,0xfb400108,0xddba0101,0xfb400108,0xddbb0101,0xfb400108,0xddbc0101,
	0xfb400108,0xddbd0101,0xfb400108,0xddbe0101,0xfb400108,0xddbf0101,0xfb400108,0xddc00101,
	0xfb400108,0xddc10101,0xfb400108,0xddc20101,0xfb400108,0xddc30101,0xfb400108,0xddc40101,
	0xfb400108,0xddc60101,0xfb400108,0xddc70101,0xfb400108,0xddc80101,0xfb400108,0xddc90101,
	0xfb400108,0xddca0101,0xfb400108,0xddcb0101,0xfb400108,0xddcc0101,0xfb400108,0xddce0101,
	0xfb400108,0xddcf0101,0xfb400108,0xddd00101,0xfb400108,0xddd10101,0xfb400108,0xddd20101,
	0xfb400108,0xddd30101,0xfb400108,0xddd40101,0xfb400108,0xddd50101,0xfb400108,0xddd60101,
	0xfb400108,0xddd70101,0xfb400108,0xddd80101,0xfb400108,0xddd90101,0xfb400108,0xddda0101,
	0xfb400108,0xdddc0101,0xfb400108,0xdddf0101,0xfb400108,0xdde00101,0xfb400108,0xdde30101,
	0xfb400108,0xdde40101,0xfb400108,0xddea0101,0xfb400108,0xddec0101,0xfb400108,0xdded0101,
	0xfb400108,0xddf00101,0xfb400108,0xddf50101,0xfb400108,0xddf60101,0xfb400108,0xddf80101,
	0xfb400108,0xddf90101,0xfb400108,0xddfa0101,0xfb400108,0xddfb0101,0xfb400108,0xddfc0101,
	0xfb400108,0xddff0101,0xfb400108,0xde000101,0xfb400108,0xde040101,0xfb400108,0xde070101,
	0xfb400108,0xde090101,0xfb400108,0xde0a0101,0xfb400108,0xde0b0101,0xfb400108,0xde0d0101,
	0xfb400108,0xde0e0101,0xfb400108,0xde120101,0xfb400108,0xde130101,0xfb400108,0xde170101,
	0xfb400108,0xde1e0101,0xfb400108,0xde1f0101,0xfb400108,0xde200101,0xfb400108,0xde210101,
	0xfb400108,0xde220101,0xfb400108,0xde230101,0xfb400108,0xde240101,0xfb400108,0xde250101,
	0xfb400108,0xde280101,0xfb400108,0xde290101,0xfb400108,0xde2a0101,0xfb400108,0xde2b0101,
	0xfb400108,0xde2c0101,0xfb400108,0xde2f0101,0xfb400108,0xde300101,0xfb400108,0xde320101,
	0xfb400108,0xde330101,0xfb400108,0xde340101,0xfb400108,0xde350101,0xfb400108,0xde360101,
	0xfb400108,0xde390101,0xfb400108,0xde3a0101,0xfb400108,0xde3e0101,0xfb400108,0xde3f0101,
	0xfb400108,0xde400101,0xfb400108,0xde410101,0xfb400108,0xde430101,0xfb400108,0xde460101,
	0xfb400108,0xde470101,0xfb400108,0xde480101,0xfb400108,0xde490101,0xfb400108,0xde4a0101,
	0xfb400108,0xde4b0101,0xfb400108,0xde4d0101,0xfb400108,0xde4e0101,0xfb400108,0xde4f0101,
	0xfb400108,0xde500101,0xfb400108,0xde510101,0xfb400108,0xde520101,0xfb400108,0xde530101,
	0xfb400108,0xde560101,0xfb400108,0xde570101,0xfb400108,0xde580101,0xfb400108,0xde590101,
	0xfb400108,0xde5a0101,0xfb400108,0xde5c0101,0xfb400108,0xde5d0101,0xfb400108,0xde5f0101,
	0xfb400108,0xde600101,0xfb400108,0xde630101,0xfb400108,0xde640101,0xfb400108,0xde650101,
	0xfb400108,0xde660101,0xfb400108,0xde670101,0xfb400108,0xde680101,0xfb400108,0xde690101,
	0xfb400108,0xde6a0101,0xfb400108,0xde6b0101,0xfb400108,0xde6c0101,0xfb400108,0xde6d0101,
	0xfb400108,0xde6e0101,0xfb400108,0xde6f0101,0xfb400108,0xde700101,0xfb400108,0xde710101,
	0xfb400108,0xde750101,0xfb400108,0xde770101,0xfb400108,0xde790101,0xfb400108,0xde7e0101,
	0xfb400108,0xde810101,0xfb400108,0xde820101,0xfb400108,0xde830101,0xfb400108,0xde850101,
	0xfb400108,0xde880101,0xfb400108,0xde890101,0xfb400108,0xde8c0101,0xfb400108,0xde8d0101,
	0xfb400108,0xde8e0101,0xfb400108,0xde920101,0xfb400108,0xde980101,0xfb400108,0xde9b0101,
	0xfb400108,0xde9d0101,0xfb400108,0xdea10101,0xfb400108,0xdea20101,0xfb400108,0xdea30101,
	0xfb400108,0xdea40101,0xfb400108,0xdea80101,0xfb400108,0xdea90101,0xfb400108,0xdeaa0101,
	0xfb400108,0xdeab0101,0xfb400108,0xdeac0101,0xfb400108,0xdeae0101,0xfb400108,0xdeaf0101,
	0xfb400108,0xdeb00101,0xfb400108,0xdeb10101,0xfb400108,0xdeb20101,0xfb400108,0xdeb40101,
	0xfb400108,0xdeba0101,0xfb400108,0xdebb0101,0xfb400108,0xdebc0101,0xfb400108,0xdebd0101,
	0xfb400108,0xdebf0101,0xfb400108,0xdec00101,0xfb400108,0xdec10101,0xfb400108,0xdec20101,
	0xfb400108,0xdec30101,0xfb400108,0xdec40101,0xfb400108,0xdec50101,0xfb400108,0xdec60101,
	0xfb400108,0xdec70101,0xfb400108,0xdec80101,0xfb400108,0xdecb0101,0xfb400108,0xdecc0101,
	0xfb400108,0xdecd0101,0xfb400108,0xdece0101,0xfb400108,0xdecf0101,0xfb400108,0xded00101,
	0xfb400108,0xded40101,0xfb400108,0xded50101,0xfb400108,0xded70101,0xfb400108,0xded80101,
	0xfb400108,0xded90101,0xfb400108,0xdeda0101,0xfb400108,0xdedc0101,0xfb400108,0xdedd0101,
	0xfb400108,0xdede0101,0xfb400108,0xdedf0101,0xfb400108,0xdee00101,0xfb400108,0xdee10101,
	0xfb400108,0xdee20101,0xfb400108,0xdee30101,0xfb400108,0xdee40101,0xfb400108,0xdee50101,
	0xfb400108,0xdee60101,0xfb400108,0xdee70101,0xfb400108,0xdee90101,0xfb400108,0xdeeb0101,
	0xfb400108,0xdeec0101,0xfb400108,0xdeed0101,0xfb400108,0xdeee0101,0xfb400108,0xdeef0101,
	0xfb400108,0xdef00101,0xfb400108,0xdef10101,0xfb400108,0xdef20101,0xfb400108,0xdef30101,
	0xfb400108,0xdef50101,0xfb400108,0xdef80101,0xfb400108,0xdef90101,0xfb400108,0xdefb0101,
	0xfb400108,0xdefc0101,0xfb400108,0xdefd0101,0xfb400108,0xdf050101,0xfb400108,0xdf060101,
	0xfb400108,0xdf070101,0xfb400108,0xdf090101,0xfb400108,0xdf0c0101,0xfb400108,0xdf0d0101,
	0xfb400108,0xdf0e0101,0xfb400108,0xdf100101,0xfb400108,0xdf120101,0xfb400108,0xdf140101,
	0xfb400108,0xdf160101,0xfb400108,0xdf190101,0xfb400108,0xdf1a0101,0xfb400108,0xdf1c0101,
	0xfb400108,0xdf1d0101,0xfb400108,0xdf1e0101,0xfb400108,0xdf210101,0xfb400108,0xdf220101,
	0xfb400108,0xdf230101,0xfb400108,0xdf240101,0xfb400108,0xdf280101,0xfb400108,0xdf2b0101,
	0xfb400108,0xdf2c0101,0xfb400108,0xdf2e0101,0xfb400108,0xdf300101,0xfb400108,0xdf320101,
	0xfb400108,0xdf330101,0xfb400108,0xdf340101,0xfb400108,0xdf350101,0xfb400108,0xdf360101,
	0xfb400108,0xdf370101,0xfb400108,0xdf380101,0xfb400108,0xdf3b0101,0xfb400108,0xdf3d0101,
	0xfb400108,0xdf3e0101,0xfb400108,0xdf3f0101,0xfb400108,0xdf410101,0xfb400108,0xdf420101,
	0xfb400108,0xdf430101,0xfb400108,0xdf440101,0xfb400108,0xdf450101,0xfb400108,0xdf460101,
	0xfb400108,0xdf470101,0xfb400108,0xdf480101,0xfb400108,0xdf490101,0xfb400108,0xdf4a0101,
	0xfb400108,0xdf4b0101,0xfb400108,0xdf4c0101,0xfb400108,0xdf4d0101,0xfb400108,0xdf4e0101,
	0xfb400108,0xdf4f0101,0xfb400108,0xdf510101,0xfb400108,0xdf540101,0xfb400108,0xdf590101,
	0xfb400108,0xdf5a0101,0xfb400108,0xdf5b0101,0xfb400108,0xdf5c0101,0xfb400108,0xdf5e0101,
	0xfb400108,0xdf5f0101,0xfb400108,0xdf600101,0xfb400108,0xdf630101,0xfb400108,0xdf650101,
	0xfb400108,0xdf670101,0xfb400108,0xdf680101,0xfb400108,0xdf6b0101,0xfb400108,0xdf6e0101,
	0xfb400108,0xdf6f0101,0xfb400108,0xdf720101,0xfb400108,0xdf740101,0xfb400108,0xdf750101,
	0xfb400108,0xdf760101,0xfb400108,0xdf780101,0xfb400108,0xdf7a0101,0xfb400108,0xdf7d0101,
	0xfb400108,0xdf7e0101,0xfb400108,0xdf7f0101,0xfb400108,0xdf830101,0xfb400108,0xdf860101,
	0xfb400108,0xdf8d0101,0xfb400108,0xdf8e0101,0xfb400108,0xdf8f0101,0xfb400108,0xdf910101,
	0xfb400108,0xdf930101,0xfb400108,0xdf940101,0xfb400108,0xdf960101,0xfb400108,0xdf9a0101,
	0xfb400108,0xdf9b0101,0xfb400108,0xdf9d0101,0xfb400108,0xdf9e0101,0xfb400108,0xdf9f0101,
	0xfb400108,0xdfa00101,0xfb400108,0xdfa20101,0xfb400108,0xdfa30101,0xfb400108,0xdfa40101,
	0xfb400108,0xdfa50101,0xfb400108,0xdfa60101,0xfb400108,0xdfa70101,0xfb400108,0xdfa90101,
	0xfb400108,0xdfab0101,0xfb400108,0xdfac0101,0xfb400108,0xdfaf0101,0xfb400108,0xdfb00101,
	0xfb400108,0xdfb10101,0xfb400108,0xdfb20101,0xfb400108,0xdfb30101,0xfb400108,0xdfb40101,
	0xfb400108,0xdfb60101,0xfb400108,0xdfb80101,0xfb400108,0xdfb90101,0xfb400108,0xdfba0101,
	0xfb400108,0xdfbb0101,0xfb400108,0xdfbe0101,0xfb400108,0xdfbf0101,0xfb400108,0xdfc00101,
	0xfb400108,0xdfc10101,0xfb400108,0xdfc20101,0xfb400108,0xdfc70101,0xfb400108,0xdfc80101,
	0xfb400108,0xdfca0101,0xfb400108,0xdfcb0101,0xfb400108,0xdfce0101,0xfb400108,0xdfd30101,
	0xfb400108,0xdfd40101,0xfb400108,0xdfd50101,0xfb400108,0xdfda0101,0xfb400108,0xdfdb0101,
	0xfb400108,0xdfdc0101,0xfb400108,0xdfde0101,0xfb400108,0xdfdf0101,0xfb400108,0xdfe20101,
	0xfb400108,0xdfe30101,0xfb400108,0xdfe50101,0xfb400108,0xdfe60101,0xfb400108,0xdfe80101,
	0xfb400108,0xdfe90101,0xfb400108,0xdfec0101,0xfb400108,0xdfef0101,0xfb400108,0xdff00101,
	0xfb400108,0xdff20101,0xfb400108,0xdff30101,0xfb400108,0xdff40101,0xfb400108,0xdff60101,
	0xfb400108,0xdff70101,0xfb400108,0xdff90101,0xfb400108,0xdffa0101,0xfb400108,0xdffc0101,
	0xfb400108,0xe0070101,0xfb400108,0xe0080101,0xfb400108,0xe0090101,0xfb400108,0xe00b0101,
	0xfb400108,0xe00c0101,0xfb400108,0xe0100101,0xfb400108,0xe0110101,0xfb400108,0xe0130101,
	0xfb400108,0xe0170101,0xfb400108,0xe0180101,0xfb400108,0xe01a0101,0xfb400108,0xe01e0101,
	0xfb400108,0xe01f0101,0xfb400108,0xe0220101,0xfb400108,0xe0230101,0xfb400108,0xe0240101,
	0xfb400108,0xe02c0101,0xfb400108,0xe02d0101,0xfb400108,0xe02e0101,0xfb400108,0xe0300101,
	0xfb400108,0xe0310101,0xfb400108,0xe0320101,0xfb400108,0xe0330101,0xfb400108,0xe0340101,
	0xfb400108,0xe0360101,0xfb400108,0xe0370101,0xfb400108,0xe0380101,0xfb400108,0xe0390101,
	0xfb400108,0xe03a0101,0xfb400108,0xe03d0101,0xfb400108,0xe03e0101,0xfb400108,0xe0400101,
	0xfb400108,0xe0440101,0xfb400108,0xe0450101,0xfb400108,0xe0460101,0xfb400108,0xe0470101,
	0xfb400108,0xe0480101,0xfb400108,0xe0490101,0xfb400108,0xe04a0101,0xfb400108,0xe04c0101,
	0xfb400108,0xe04e0101,0xfb400108,0xe04f0101,0xfb400108,0xe0510101,0xfb400108,0xe0530101,
	0xfb400108,0xe0540101,0xfb400108,0xe0560101,0xfb400108,0xe0570101,0xfb400108,0xe0580101,
	0xfb400108,0xe05b0101,0xfb400108,0xe05c0101,0xfb400108,0xe05e0101,0xfb400108,0xe05f0101,
	0xfb400108,0xe0600101,0xfb400108,0xe0610101,0xfb400108,0xe0650101,0xfb400108,0xe0660101,
	0xfb400108,0xe06e0101,0xfb400108,0xe0710101,0xfb400108,0xe0720101,0xfb400108,0xe0740101,
	0xfb400108,0xe0750101,0xfb400108,0xe0770101,0xfb400108,0xe07e0101,0xfb400108,0xe0800101,
	0xfb400108,0xe0810101,0xfb400108,0xe0820101,0xfb400108,0xe0850101,0xfb400108,0xe0860101,
	0xfb400108,0xe0870101,0xfb400108,0xe0880101,0xfb400108,0xe08a0101,0xfb400108,0xe08b0101,
	0xfb400108,0xe08e0101,0xfb400108,0xe08f0101,0xfb400108,0xe0900101,0xfb400108,0xe0910101,
	0xfb400108,0xe0930101,0xfb400108,0xe0950101,0xfb400108,0xe0970101,0xfb400108,0xe0980101,
	0xfb400108,0xe0990101,0xfb400108,0xe09c0101,0xfb400108,0xe09e0101,0xfb400108,0xe0a10101,
	0xfb400108,0xe0a20101,0xfb400108,0xe0a40101,0xfb400108,0xe0a50101,0xfb400108,0xe0a70101,
	0xfb400108,0xe0a90101,0xfb400108,0xe0aa0101,0xfb400108,0xe0ae0101,0xfb400108,0xe0b00101,
	0xfb400108,0xe0b30101,0xfb400108,0xe0b50101,0xfb400108,0xe0b60101,0xfb400108,0xe0b70101,
	0xfb400108,0xe0b90101,0xfb400108,0xe0ba0101,0xfb400108,0xe0bd0101,0xfb400108,0xe0be0101,
	0xfb400108,0xe0bf0101,0xfb400108,0xe0c00101,0xfb400108,0xe0c10101,0xfb400108,0xe0c20101,
	0xfb400108,0xe0c30101,0xfb400108,0xe0c40101,0xfb400108,0xe0c70101,0xfb400108,0xe0c80101,
	0xfb400108,0xe0c90101,0xfb400108,0xe0cc0101,0xfb400108,0xe0cd0101,0xfb400108,0xe0ce0101,
	0xfb400108,0xe0cf0101,0xfb400108,0xe0d00101,0xfb400108,0xe0d20101,0xfb400108,0xe0d30101,
	0xfb400108,0xe0d40101,0xfb400108,0xe0d60101,0xfb400108,0xe0d70101,0xfb400108,0xe0d90101,
	0xfb400108,0xe0db0101,0xfb400108,0xe0de0101,0xfb400108,0xe0e10101,0xfb400108,0xe0e20101,
	0xfb400108,0xe0e30101,0xfb400108,0xe0e40101,0xfb400108,0xe0e50101,0xfb400108,0xe0ea0101,
	0xfb400108,0xe0f10101,0xfb400108,0xe0f20101,0xfb400108,0xe0f50101,0xfb400108,0xe0f70101,
	0xfb400108,0xe0f80101,0xfb400108,0xe0fb0101,0xfb400108,0xe0fc0101,0xfb400108,0xe0fd0101,
	0xfb400108,0xe0fe0101,0xfb400108,0xe0ff0101,0xfb400108,0xe1020101,0xfb400108,0xe1030101,
	0xfb400108,0xe1040101,0xfb400108,0xe1050101,0xfb400108,0xe1070101,0xfb400108,0xe10a0101,
	0xfb400108,0xe10b0101,0xfb400108,0xe10c0101,0xfb400108,0xe1100101,0xfb400108,0xe1110101,
	0xfb400108,0xe1120101,0xfb400108,0xe1130101,0xfb400108,0xe1140101,0xfb400108,0xe1160101,
	0xfb400108,0xe1170101,0xfb400108,0xe1180101,0xfb400108,0xe1190101,0xfb400108,0xe11b0101,
	0xfb400108,0xe11c0101,0xfb400108,0xe11d0101,0xfb400108,0xe11e0101,0xfb400108,0xe1210101,
	0xfb400108,0xe1220101,0xfb400108,0xe1250101,0xfb400108,0xe1280101,0xfb400108,0xe1290101,
	0xfb400108,0xe12a0101,0xfb400108,0xe12c0101,0xfb400108,0xe12d0101,0xfb400108,0xe12e0101,
	0xfb400108,0xe12f0101,0xfb400108,0xe1300101,0xfb400108,0xe1310101,0xfb400108,0xe1320101,
	0xfb400108,0xe1330101,0xfb400108,0xe1340101,0xfb400108,0xe1350101,0xfb400108,0xe1360101,
	0xfb400108,0xe1370101,0xfb400108,0xe1380101,0xfb400108,0xe1390101,0xfb400108,0xe13a0101,
	0xfb400108,0xe13b0101,0xfb400108,0xe13c0101,0xfb400108,0xe13d0101,0xfb400108,0xe13e0101,
	0xfb400108,0xe1400101,0xfb400108,0xe1410101,0xfb400108,0xe1420101,0xfb400108,0xe1430101,
	0xfb400108,0xe1440101,0xfb400108,0xe1450101,0xfb400108,0xe1460101,0xfb400108,0xe1470101,
	0xfb400108,0xe1490101,0xfb400108,0xe14b0101,0xfb400108,0xe14d0101,0xfb400108,0xe14f0101,
	0xfb400108,0xe1500101,0xfb400108,0xe1520101,0xfb400108,0xe1530101,0xfb400108,0xe1540101,
	0xfb400108,0xe1560101,0xfb400108,0xe1570101,0xfb400108,0xe1580101,0xfb400108,0xe1590101,
	0xfb400108,0xe15a0101,0xfb400108,0xe15b0101,0xfb400108,0xe15c0101,0xfb400108,0xe15e0101,
	0xfb400108,0xe15f0101,0xfb400108,0xe1600101,0xfb400108,0xe1610101,0xfb400108,0xe1630101,
	0xfb400108,0xe1640101,0xfb400108,0xe1650101,0xfb400108,0xe1660101,0xfb400108,0xe1690101,
	0xfb400108,0xe16a0101,0xfb400108,0xe16b0101,0xfb400108,0xe16c0101,0xfb400108,0xe16d0101,
	0xfb400108,0xe16e0101,0xfb400108,0xe16f0101,0xfb400108,0xe1710101,0xfb400108,0xe1720101,
	0xfb400108,0xe1730101,0xfb400108,0xe1740101,0xfb400108,0xe1760101,0xfb400108,0xe1780101,
	0xfb400108,0xe1790101,0xfb400108,0xe17a0101,0xfb400108,0xe17b0101,0xfb400108,0xe17c0101,
	0xfb400108,0xe17d0101,0xfb400108,0xe17e0101,0xfb400108,0xe17f0101,0xfb400108,0xe1800101,
	0xfb400108,0xe1810101,0xfb400108,0xe1820101,0xfb400108,0xe1830101,0xfb400108,0xe1840101,
	0xfb400108,0xe1850101,0xfb400108,0xe1860101,0xfb400108,0xe1870101,0xfb400108,0xe1880101,
	0xfb400108,0xe1890101,0xfb400108,0xe18a0101,0xfb400108,0xe18c0101,0xfb400108,0xe18d0101,
	0xfb400108,0xe18f0101,0xfb400108,0xe1900101,0xfb400108,0xe1910101,0xfb400108,0xe1920101,
	0xfb400108,0xe1930101,0xfb400108,0xe1950101,0xfb400108,0xe1960101,0xfb400108,0xe1970101,
	0xfb400108,0xe1980101,0xfb400108,0xe1990101,0xfb400108,0xe19a0101,0xfb400108,0xe19b0101,
	0xfb400108,0xe19c0101,0xfb400108,0xe19e0101,0xfb400108,0xe19f0101,0xfb400108,0xe1a00101,
	0xfb400108,0xe1a10101,0xfb400108,0xe1a20101,0xfb400108,0xe1a30101,0xfb400108,0xe1a40101,
	0xfb400108,0xe1a50101,0xfb400108,0xe1a60101,0xfb400108,0xe1aa0101,0xfb400108,0xe1ab0101,
	0xfb400108,0xe1ad0101,0xfb400108,0xe1ae0101,0xfb400108,0xe1af0101,0xfb400108,0xe1b00101,
	0xfb400108,0xe1b10101,0xfb400108,0xe1b20101,0xfb400108,0xe1b30101,0xfb400108,0xe1b40101,
	0xfb400108,0xe1b50101,0xfb400108,0xe1b60101,0xfb400108,0xe1b80101,0xfb400108,0xe1b90101,
	0xfb400108,0xe1ba0101,0xfb400108,0xe1bb0101,0xfb400108,0xe1bc0101,0xfb400108,0xe1bd0101,
	0xfb400108,0xe1bf0101,0xfb400108,0xe1c00101,0xfb400108,0xe1c10101,0xfb400108,0xe1c30101,
	0xfb400108,0xe1c40101,0xfb400108,0xe1c50101,0xfb400108,0xe1c60101,0xfb400108,0xe1c70101,
	0xfb400108,0xe1c90101,0xfb400108,0xe1cc0101,0xfb400108,0xe1cd0101,0xfb400108,0xe1ce0101,
	0xfb400108,0xe1cf0101,0xfb400108,0xe1d00101,0xfb400108,0xe1d30101,0xfb400108,0xe1d50101,
	0xfb400108,0xe1d60101,0xfb400108,0xe1d70101,0xfb400108,0xe1d80101,0xfb400108,0xe1d90101,
	0xfb400108,0xe1da0101,0xfb400108,0xe1db0101,0xfb400108,0xe1dc0101,0xfb400108,0xe1dd0101,
	0xfb400108,0xe1de0101,0xfb400108,0xe1df0101,0xfb400108,0xe1e00101,0xfb400108,0xe1e10101,
	0xfb400108,0xe1e20101,0xfb400108,0xe1e30101,0xfb400108,0xe1e40101,0xfb400108,0xe1e50101,
	0xfb400108,0xe1e70101,0xfb400108,0xe1e80101,0xfb400108,0xe1e90101,0xfb400108,0xe1ea0101,
	0xfb400108,0xe1eb0101,0xfb400108,0xe1ec0101,0xfb400108,0xe1ed0101,0xfb400108,0xe1ee0101,
	0xfb400108,0xe1ef0101,0xfb400108,0xe1f00101,0xfb400108,0xe1f10101,0xfb400108,0xe1f20101,
	0xfb400108,0xe1f30101,0xfb400108,0xe1f40101,0xfb400108,0xe1f60101,0xfb400108,0xe1f70101,
	0xfb400108,0xe1f80101,0xfb400108,0xe1f90101,0xfb400108,0xe1fa0101,0xfb400108,0xe1fb0101,
	0xfb400108,0xe1fc0101,0xfb400108,0xe1fd0101,0xfb400108,0xe1fe0101,0xfb400108,0xe2000101,
	0xfb400108,0xe2010101,0xfb400108,0xe2020101,0xfb400108,0xe2030101,0xfb400108,0xe2040101,
	0xfb400108,0xe2050101,0xfb400108,0xe2070101,0xfb400108,0xe2090101,0xfb400108,0xe2130101,
	0xfb400108,0xe2140101,0xfb400108,0xe2190101,0xfb400108,0xe21c0101,0xfb400108,0xe21d0101,
	0xfb400108,0xe21e0101,0xfb400108,0xe2200101,0xfb400108,0xe2230101,0xfb400108,0xe2260101,
	0xfb400108,0xe2270101,0xfb400108,0xe2280101,0xfb400108,0xe2290101,0xfb400108,0xe22b0101,
	0xfb400108,0xe22d0101,0xfb400108,0xe22f0101,0xfb400108,0xe2300101,0xfb400108,0xe2310101,
	0xfb400108,0xe2320101,0xfb400108,0xe2350101,0xfb400108,0xe2360101,0xfb400108,0xe2380101,
	0xfb400108,0xe2390101,0xfb400108,0xe23a0101,0xfb400108,0xe23b0101,0xfb400108,0xe23c0101,
	0xfb400108,0xe2420101,0xfb400108,0xe2440101,0xfb400108,0xe2450101,0xfb400108,0xe2460101,
	0xfb400108,0xe24a0101,0xfb400108,0xe24f0101,0xfb400108,0xe2500101,0xfb400108,0xe2550101,
	0xfb400108,0xe2560101,0xfb400108,0xe2570101,0xfb400108,0xe2590101,0xfb400108,0xe25a0101,
	0xfb400108,0xe25c0101,0xfb400108,0xe25d0101,0xfb400108,0xe25e0101,0xfb400108,0xe25f0101,
	0xfb400108,0xe2600101,0xfb400108,0xe2610101,0xfb400108,0xe2620101,0xfb400108,0xe2640101,
	0xfb400108,0xe2650101,0xfb400108,0xe2680101,0xfb400108,0xe2710101,0xfb400108,0xe2720101,
	0xfb400108,0xe2740101,0xfb400108,0xe2750101,0xfb400108,0xe2770101,0xfb400108,0xe2780101,
	0xfb400108,0xe27a0101,0xfb400108,0xe27b0101,0xfb400108,0xe27d0101,0xfb400108,0xe2810101,
	0xfb400108,0xe2820101,0xfb400108,0xe2830101,0xfb400108,0xe2850101,0xfb400108,0xe2860101,
	0xfb400108,0xe2870101,0xfb400108,0xe2880101,0xfb400108,0xe28b0101,0xfb400108,0xe28c0101,
	0xfb400108,0xe28d0101,0xfb400108,0xe28e0101,0xfb400108,0xe28f0101,0xfb400108,0xe2900101,
	0xfb400108,0xe2940101,0xfb400108,0xe2990101,0xfb400108,0xe29c0101,0xfb400108,0xe29d0101,
	0xfb400108,0xe29e0101,0xfb400108,0xe2a30101,0xfb400108,0xe2a60101,0xfb400108,0xe2a70101,
	0xfb400108,0xe2a90101,0xfb400108,0xe2aa0101,0xfb400108,0xe2ad0101,0xfb400108,0xe2ae0101,
	0xfb400108,0xe2af0101,0xfb400108,0xe2b00101,0xfb400108,0xe2b20101,0xfb400108,0xe2b30101,
	0xfb400108,0xe2b40101,0xfb400108,0xe2b60101,0xfb400108,0xe2b70101,0xfb400108,0xe2b80101,
	0xfb400108,0xe2ba0101,0xfb400108,0xe2be0101,0xfb400108,0xe2c00101,0xfb400108,0xe2c10101,
	0xfb400108,0xe2c30101,0xfb400108,0xe2cb0101,0xfb400108,0xe2cf0101,0xfb400108,0xe2d10101,
	0xfb400108,0xe2d50101,0xfb400108,0xe2dd0101,0xfb400108,0xe2de0101,0xfb400108,0xe2e00101,
	0xfb400108,0xe2e10101,0xfb400108,0xe2e40101,0xfb400108,0xe2ea0101,0xfb400108,0xe2eb0101,
	0xfb400108,0xe2f00101,0xfb400108,0xe2f20101,0xfb400108,0xe2f50101,0xfb400108,0xe2f80101,
	0xfb400108,0xe2f90101,0xfb400108,0xe2fa0101,0xfb400108,0xe2fb0101,0xfb400108,0xe3000101,
	0xfb400108,0xe3030101,0xfb400108,0xe3040101,0xfb400108,0xe3050101,0xfb400108,0xe3060101,
	0xfb400108,0xe30a0101,0xfb400108,0xe30b0101,0xfb400108,0xe30c0101,0xfb400108,0xe30d0101,
	0xfb400108,0xe30f0101,0xfb400108,0xe3100101,0xfb400108,0xe3120101,0xfb400108,0xe3130101,
	0xfb400108,0xe3140101,0xfb400108,0xe3150101,0xfb400108,0xe3170101,0xfb400108,0xe3180101,
	0xfb400108,0xe3190101,0xfb400108,0xe31c0101,0xfb400108,0xe3260101,0xfb400108,0xe3270101,
	0xfb400108,0xe3290101,0xfb400108,0xe32c0101,0xfb400108,0xe32d0101,0xfb400108,0xe32e0101,
	0xfb400108,0xe3300101,0xfb400108,0xe3310101,0xfb400108,0xe3330101,0xfb400108,0xe3340101,
	0xfb400108,0xe3350101,0xfb400108,0xe3360101,0xfb400108,0xe3370101,0xfb400108,0xe3380101,
	0xfb400108,0xe33b0101,0xfb400108,0xe33c0101,0xfb400108,0xe33e0101,0xfb400108,0xe33f0101,
	0xfb400108,0xe3400101,0xfb400108,0xe3410101,0xfb400108,0xe3440101,0xfb400108,0xe3470101,
	0xfb400108,0xe3480101,0xfb400108,0xe34a0101,0xfb400108,0xe3510101,0xfb400108,0xe3520101,
	0xfb400108,0xe3530101,0xfb400108,0xe3540101,0xfb400108,0xe3560101,0xfb400108,0xe3570101,
	0xfb400108,0xe3580101,0xfb400108,0xe3590101,0xfb400108,0xe35a0101,0xfb400108,0xe35b0101,
	0xfb400108,0xe35c0101,0xfb400108,0xe35d0101,0xfb400108,0xe3600101,0xfb400108,0xe3640101,
	0xfb400108,0xe3650101,0xfb400108,0xe3660101,0xfb400108,0xe3680101,0xfb400108,0xe36a0101,
	0xfb400108,0xe36b0101,0xfb400108,0xe36c0101,0xfb400108,0xe36f0101,0xfb400108,0xe3700101,
	0xfb400108,0xe3720101,0xfb400108,0xe3730101,0xfb400108,0xe3740101,0xfb400108,0xe3750101,
	0xfb400108,0xe3780101,0xfb400108,0xe3790101,0xfb400108,0xe37c0101,0xfb400108,0xe37d0101,
	0xfb400108,0xe37e0101,0xfb400108,0xe37f0101,0xfb400108,0xe3810101,0xfb400108,0xe3830101,
	0xfb400108,0xe3840101,0xfb400108,0xe3850101,0xfb400108,0xe3860101,0xfb400108,0xe38b0101,
	0xfb400108,0xe38d0101,0xfb400108,0xe3910101,0xfb400108,0xe3930101,0xfb400108,0xe3940101,
	0xfb400108,0xe3950101,0xfb400108,0xe3970101,0xfb400108,0xe3990101,0xfb400108,0xe39a0101,
	0xfb400108,0xe39b0101,0xfb400108,0xe39c0101,0xfb400108,0xe39d0101,0xfb400108,0xe39e0101,
	0xfb400108,0xe39f0101,0xfb400108,0xe3a10101,0xfb400108,0xe3a40101,0xfb400108,0xe3a60101,
	0xfb400108,0xe3ab0101,0xfb400108,0xe3af0101,0xfb400108,0xe3b10101,0xfb400108,0xe3b20101,
	0xfb400108,0xe3b50101,0xfb400108,0xe3b60101,0xfb400108,0xe3b90101,0xfb400108,0xe3bb0101,
	0xfb400108,0xe3bd0101,0xfb400108,0xe3bf0101,0xfb400108,0xe3c00101,0xfb400108,0xe3c10101,
	0xfb400108,0xe3c20101,0xfb400108,0xe3c30101,0xfb400108,0xe3c50101,0xfb400108,0xe3c70101,
	0xfb400108,0xe3c80101,0xfb400108,0xe3ca0101,0xfb400108,0xe3cb0101,0xfb400108,0xe3cc0101,
	0xfb400108,0xe3d10101,0xfb400108,0xe3d30101,0xfb400108,0xe3d40101,0xfb400108,0xe3d50101,
	0xfb400108,0xe3d70101,0xfb400108,0xe3d80101,0xfb400108,0xe3d90101,0xfb400108,0xe3da0101,
	0xfb400108,0xe3db0101,0xfb400108,0xe3dc0101,0xfb400108,0xe3dd0101,0xfb400108,0xe3df0101,
	0xfb400108,0xe3e20101,0xfb400108,0xe3e40101,0xfb400108,0xe3e50101,0xfb400108,0xe3e60101,
	0xfb400108,0xe3e70101,0xfb400108,0xe3e80101,0xfb400108,0xe3eb0101,0xfb400108,0xe3ec0101,
	0xfb400108,0xe3ee0101,0xfb400108,0xe3ef0101,0xfb400108,0xe3f00101,0xfb400108,0xe3f10101,
	0xfb400108,0xe3f30101,0xfb400108,0xe3f50101,0xfb400108,0xe3f70101,0xfb400108,0xe3f90101,
	0xfb400108,0xe3fa0101,0xfb400108,0xe3fb0101,0xfb400108,0xe3fc0101,0xfb400108,0xe3fe0101,
	0xfb400108,0xe4030101,0xfb400108,0xe4040101,0xfb400108,0xe4060101,0xfb400108,0xe4070101,
	0xfb400108,0xe4080101,0xfb400108,0xe4090101,0xfb400108,0xe40a0101,0xfb400108,0xe40d0101,
	0xfb400108,0xe40e0101,0xfb400108,0xe4110101,0xfb400108,0xe4120101,0xfb400108,0xe4150101,
	0xfb400108,0xe4160101,0xfb400108,0xe4170101,0xfb400108,0xe4180101,0xfb400108,0xe4190101,
	0xfb400108,0xe41a0101,0xfb400108,0xe41d0101,0xfb400108,0xe41f0101,0xfb400108,0xe4220101,
	0xfb400108,0xe4230101,0xfb400108,0xe4240101,0xfb400108,0xe4250101,0xfb400108,0xe4270101,
	0xfb400108,0xe4280101,0xfb400108,0xe4290101,0xfb400108,0xe42b0101,0xfb400108,0xe42e0101,
	0xfb400108,0xe42f0101,0xfb400108,0xe4300101,0xfb400108,0xe4310101,0xfb400108,0xe4320101,
	0xfb400108,0xe4330101,0xfb400108,0xe4350101,0xfb400108,0xe4360101,0xfb400108,0xe4370101,
	0xfb400108,0xe4380101,0xfb400108,0xe4390101,0xfb400108,0xe43b0101,0xfb400108,0xe43c0101,
	0xfb400108,0xe43e0101,0xfb400108,0xe4400101,0xfb400108,0xe4420101,0xfb400108,0xe4430101,
	0xfb400108,0xe4490101,0xfb400108,0xe44b0101,0xfb400108,0xe44c0101,0xfb400108,0xe44d0101,
	0xfb400108,0xe44e0101,0xfb400108,0xe44f0101,0xfb400108,0xe4500101,0xfb400108,0xe4510101,
	0xfb400108,0xe4530101,0xfb400108,0xe4550101,0xfb400108,0xe4560101,0xfb400108,0xe4570101,
	0xfb400108,0xe4590101,0xfb400108,0xe45a0101,0xfb400108,0xe45b0101,0xfb400108,0xe45c0101,
	0xfb400108,0xe45d0101,0xfb400108,0xe45f0101,0xfb400108,0xe4600101,0xfb400108,0xe4610101,
	0xfb400108,0xe4620101,0xfb400108,0xe4630101,0xfb400108,0xe4640101,0xfb400108,0xe4650101,
	0xfb400108,0xe4660101,0xfb400108,0xe4680101,0xfb400108,0xe46a0101,0xfb400108,0xe46b0101,
	0xfb400108,0xe46c0101,0xfb400108,0xe46e0101,0xfb400108,0xe46f0101,0xfb400108,0xe4700101,
	0xfb400108,0xe4710101,0xfb400108,0xe4720101,0xfb400108,0xe4730101,0xfb400108,0xe4740101,
	0xfb400108,0xe4750101,0xfb400108,0xe4760101,0xfb400108,0xe4770101,0xfb400108,0xe47b0101,
	0xfb400108,0xe47c0101,0xfb400108,0xe47d0101,0xfb400108,0xe47e0101,0xfb400108,0xe47f0101,
	0xfb400108,0xe4800101,0xfb400108,0xe4810101,0xfb400108,0xe4830101,0xfb400108,0xe4860101,
	0xfb400108,0xe4880101,0xfb400108,0xe4890101,0xfb400108,0xe48a0101,0xfb400108,0xe48b0101,
	0xfb400108,0xe48c0101,0xfb400108,0xe48d0101,0xfb400108,0xe48e0101,0xfb400108,0xe48f0101,
	0xfb400108,0xe4900101,0xfb400108,0xe4930101,0xfb400108,0xe4940101,0xfb400108,0xe4970101,
	0xfb400108,0xe4980101,0xfb400108,0xe49a0101,0xfb400108,0xe49b0101,0xfb400108,0xe49c0101,
	0xfb400108,0xe49d0101,0xfb400108,0xe49f0101,0xfb400108,0xe4a00101,0xfb400108,0xe4a10101,
	0xfb400108,0xe4a20101,0xfb400108,0xe4a30101,0xfb400108,0xe4a50101,0xfb400108,0xe4a60101,
	0xfb400108,0xe4a70101,0xfb400108,0xe4a80101,0xfb400108,0xe4aa0101,0xfb400108,0xe4ab0101,
	0xfb400108,0xe4af0101,0xfb400108,0xe4b10101,0xfb400108,0xe4b20101,0xfb400108,0xe4b30101,
	0xfb400108,0xe4b40101,0xfb400108,0xe4b60101,0xfb400108,0xe4b90101,0xfb400108,0xe4bb0101,
	0xfb400108,0xe4bd0101,0xfb400108,0xe4be0101,0xfb400108,0xe4bf0101,0xfb400108,0xe4c10101,
	0xfb400108,0xe4c30101,0xfb400108,0xe4c40101,0xfb400108,0xe4c60101,0xfb400108,0xe4c70101,
	0xfb400108,0xe4c80101,0xfb400108,0xe4c90101,0xfb400108,0xe4ca0101,0xfb400108,0xe4cb0101,
	0xfb400108,0xe4cc0101,0xfb400108,0xe4cf0101,0xfb400108,0xe4d10101,0xfb400108,0xe4d30101,
	0xfb400108,0xe4d40101,0xfb400108,0xe4d50101,0xfb400108,0xe4d60101,0xfb400108,0xe4d90101,
	0xfb400108,0xe4da0101,0xfb400108,0xe4db0101,0xfb400108,0xe4dc0101,0xfb400108,0xe4dd0101,
	0xfb400108,0xe4df0101,0xfb400108,0xe4e00101,0xfb400108,0xe4e10101,0xfb400108,0xe4e30101,
	0xfb400108,0xe4e50101,0xfb400108,0xe4e70101,0xfb400108,0xe4e80101,0xfb400108,0xe4e90101,
	0xfb400108,0xe4ea0101,0xfb400108,0xe4eb0101,0xfb400108,0xe4ec0101,0xfb400108,0xe4ed0101,
	0xfb400108,0xe4ee0101,0xfb400108,0xe4ef0101,0xfb400108,0xe4f00101,0xfb400108,0xe4f10101,
	0xfb400108,0xe4f20101,0xfb400108,0xe4f30101,0xfb400108,0xe4f40101,0xfb400108,0xe4f50101,
	0xfb400108,0xe4f60101,0xfb400108,0xe4f70101,0xfb400108,0xe4f80101,0xfb400108,0xe4f90101,
	0xfb400108,0xe4fa0101,0xfb400108,0xe4fb0101,0xfb400108,0xe4fc0101,0xfb400108,0xe4fd0101,
	0xfb400108,0xe4fe0101,0xfb400108,0xe4ff0101,0xfb400108,0xe5010101,0xfb400108,0xe5020101,
	0xfb400108,0xe5030101,0xfb400108,0xe5040101,0xfb400108,0xe5050101,0xfb400108,0xe5060101,
	0xfb400108,0xe5070101,0xfb400108,0xe5080101,0xfb400108,0xe50a0101,0xfb400108,0xe50b0101,
	0xfb400108,0xe50c0101,0xfb400108,0xe50d0101,0xfb400108,0xe50e0101,0xfb400108,0xe50f0101,
	0xfb400108,0xe5100101,0xfb400108,0xe5110101,0xfb400108,0xe5130101,0xfb400108,0xe5140101,
	0xfb400108,0xe5150101,0xfb400108,0xe5160101,0xfb400108,0xe5170101,0xfb400108,0xe5190101,
	0xfb400108,0xe51a0101,0xfb400108,0xe51b0101,0xfb400108,0xe51c0101,0xfb400108,0xe51d0101,
	0xfb400108,0xe51e0101,0xfb400108,0xe51f0101,0xfb400108,0xe5200101,0xfb400108,0xe5210101,
	0xfb400108,0xe5220101,0xfb400108,0xe5230101,0xfb400108,0xe5240101,0xfb400108,0xe5260101,
	0xfb400108,0xe5270101,0xfb400108,0xe5280101,0xfb400108,0xe5290101,0xfb400108,0xe52a0101,
	0xfb400108,0xe52c0101,0xfb400108,0xe52d0101,0xfb400108,0xe5300101,0xfb400108,0xe5310101,
	0xfb400108,0xe5320101,0xfb400108,0xe5330101,0xfb400108,0xe5370101,0xfb400108,0xe53a0101,
	0xfb400108,0xe53c0101,0xfb400108,0xe53d0101,0xfb400108,0xe5400101,0xfb400108,0xe5410101,
	0xfb400108,0xe5420101,0xfb400108,0xe5430101,0xfb400108,0xe5440101,0xfb400108,0xe5460101,
	0xfb400108,0xe5470101,0xfb400108,0xe54a0101,0xfb400108,0xe54b0101,0xfb400108,0xe54d0101,
	0xfb400108,0xe54e0101,0xfb400108,0xe5500101,0xfb400108,0xe5520101,0xfb400108,0xe5530101,
	0xfb400108,0xe5540101,0xfb400108,0xe5570101,0xfb400108,0xe5580101,0xfb400108,0xe55a0101,
	0xfb400108,0xe55c0101,0xfb400108,0xe55f0101,0xfb400108,0xe5600101,0xfb400108,0xe5610101,
	0xfb400108,0xe5640101,0xfb400108,0xe5650101,0xfb400108,0xe5670101,0xfb400108,0xe5680101,
	0xfb400108,0xe5690101,0xfb400108,0xe56a0101,0xfb400108,0xe56d0101,0xfb400108,0xe56e0101,
	0xfb400108,0xe56f0101,0xfb400108,0xe5710101,0xfb400108,0xe5730101,0xfb400108,0xe5750101,
	0xfb400108,0xe5760101,0xfb400108,0xe5780101,0xfb400108,0xe5790101,0xfb400108,0xe57a0101,
	0xfb400108,0xe57b0101,0xfb400108,0xe57c0101,0xfb400108,0xe57d0101,0xfb400108,0xe57e0101,
	0xfb400108,0xe57f0101,0xfb400108,0xe5800101,0xfb400108,0xe5810101,0xfb400108,0xe5820101,
	0xfb400108,0xe5830101,0xfb400108,0xe5840101,0xfb400108,0xe5850101,0xfb400108,0xe5860101,
	0xfb400108,0xe5880101,0xfb400108,0xe5890101,0xfb400108,0xe58a0101,0xfb400108,0xe58d0101,
	0xfb400108,0xe58e0101,0xfb400108,0xe58f0101,0xfb400108,0xe5920101,0xfb400108,0xe5940101,
	0xfb400108,0xe5950101,0xfb400108,0xe5960101,0xfb400108,0xe5980101,0xfb400108,0xe59a0101,
	0xfb400108,0xe59d0101,0xfb400108,0xe59e0101,0xfb400108,0xe5a00101,0xfb400108,0xe5a20101,
	0xfb400108,0xe5a30101,0xfb400108,0xe5a60101,0xfb400108,0xe5a80101,0xfb400108,0xe5aa0101,
	0xfb400108,0xe5ac0101,0xfb400108,0xe5ae0101,0xfb400108,0xe5b10101,0xfb400108,0xe5b20101,
	0xfb400108,0xe5b30101,0xfb400108,0xe5b40101,0xfb400108,0xe5b50101,0xfb400108,0xe5b60101,
	0xfb400108,0xe5b70101,0xfb400108,0xe5b80101,0xfb400108,0xe5ba0101,0xfb400108,0xe5bb0101,
	0xfb400108,0xe5be0101,0xfb400108,0xe5bf0101,0xfb400108,0xe5c00101,0xfb400108,0xe5c20101,
	0xfb400108,0xe5c70101,0xfb400108,0xe5c80101,0xfb400108,0xe5c90101,0xfb400108,0xe5ca0101,
	0xfb400108,0xe5cd0101,0xfb400108,0xe5d00101,0xfb400108,0xe5d10101,0xfb400108,0xe5d30101,
	0xfb400108,0xe5d40101,0xfb400108,0xe5d50101,0xfb400108,0xe5d80101,0xfb400108,0xe5d90101,
	0xfb400108,0xe5da0101,0xfb400108,0xe5db0101,0xfb400108,0xe5dc0101,0xfb400108,0xe5dd0101,
	0xfb400108,0xe5de0101,0xfb400108,0xe5df0101,0xfb400108,0xe5e10101,0xfb400108,0xe5e30101,
	0xfb400108,0xe5e40101,0xfb400108,0xe5ea0101,0xfb400108,0xe5eb0101,0xfb400108,0xe5f20101,
	0xfb400108,0xe5f30101,0xfb400108,0xe5f40101,0xfb400108,0xe5f50101,0xfb400108,0xe5f80101,
	0xfb400108,0xe5f90101,0xfb400108,0xe5fb0101,0xfb400108,0xe5fc0101,0xfb400108,0xe5fd0101,
	0xfb400108,0xe5fe0101,0xfb400108,0xe5ff0101,0xfb400108,0xe6010101,0xfb400108,0xe6040101,
	0xfb400108,0xe6050101,0xfb400108,0xe6070101,0xfb400108,0xe6080101,0xfb400108,0xe6090101,
	0xfb400108,0xe60b0101,0xfb400108,0xe60d0101,0xfb400108,0xe6100101,0xfb400108,0xe6110101,
	0xfb400108,0xe6120101,0xfb400108,0xe6160101,0xfb400108,0xe6170101,0xfb400108,0xe6180101,
	0xfb400108,0xe61a0101,0xfb400108,0xe61b0101,0xfb400108,0xe61c0101,0xfb400108,0xe61e0101,
	0xfb400108,0xe6210101,0xfb400108,0xe6220101,0xfb400108,0xe6230101,0xfb400108,0xe6240101,
	0xfb400108,0xe6260101,0xfb400108,0xe6290101,0xfb400108,0xe62a0101,0xfb400108,0xe62b0101,
	0xfb400108,0xe62c0101,0xfb400108,0xe62e0101,0xfb400108,0xe6300101,0xfb400108,0xe6320101,
	0xfb400108,0xe6330101,0xfb400108,0xe6370101,0xfb400108,0xe6380101,0xfb400108,0xe6390101,
	0xfb400108,0xe63a0101,0xfb400108,0xe63b0101,0xfb400108,0xe63d0101,0xfb400108,0xe63f0101,
	0xfb400108,0xe6400101,0xfb400108,0xe6420101,0xfb400108,0xe6440101,0xfb400108,0xe6450101,
	0xfb400108,0xe6460101,0xfb400108,0xe6470101,0xfb400108,0xe6480101,0xfb400108,0xe6490101,
	0xfb400108,0xe64a0101,0xfb400108,0xe64d0101,0xfb400108,0xe64e0101,0xfb400108,0xe6500101,
	0xfb400108,0xe6510101,0xfb400108,0xe6580101,0xfb400108,0xe6590101,0xfb400108,0xe65b0101,
	0xfb400108,0xe65c0101,0xfb400108,0xe65d0101,0xfb400108,0xe65e0101,0xfb400108,0xe6600101,
	0xfb400108,0xe6620101,0xfb400108,0xe6630101,0xfb400108,0xe6650101,0xfb400108,0xe6670101,
	0xfb400108,0xe6690101,0xfb400108,0xe66a0101,0xfb400108,0xe66b0101,0xfb400108,0xe66c0101,
	0xfb400108,0xe66d0101,0xfb400108,0xe6710101,0xfb400108,0xe6720101,0xfb400108,0xe6730101,
	0xfb400108,0xe6750101,0xfb400108,0xe6780101,0xfb400108,0xe6790101,0xfb400108,0xe67b0101,
	0xfb400108,0xe67c0101,0xfb400108,0xe67d0101,0xfb400108,0xe67f0101,0xfb400108,0xe6800101,
	0xfb400108,0xe6810101,0xfb400108,0xe6830101,0xfb400108,0xe6850101,0xfb400108,0xe6860101,
	0xfb400108,0xe6880101,0xfb400108,0xe6890101,0xfb400108,0xe68a0101,0xfb400108,0xe68b0101,
	0xfb400108,0xe68d0101,0xfb400108,0xe68e0101,0xfb400108,0xe68f0101,0xfb400108,0xe6900101,
	0xfb400108,0xe6920101,0xfb400108,0xe6930101,0xfb400108,0xe6940101,0xfb400108,0xe6950101,
	0xfb400108,0xe6980101,0xfb400108,0xe6990101,0xfb400108,0xe69a0101,0xfb400108,0xe69b0101,
	0xfb400108,0xe69c0101,0xfb400108,0xe69e0101,0xfb400108,0xe69f0101,0xfb400108,0xe6a00101,
	0xfb400108,0xe6a10101,0xfb400108,0xe6a20101,0xfb400108,0xe6a30101,0xfb400108,0xe6a40101,
	0xfb400108,0xe6a50101,0xfb400108,0xe6a60101,0xfb400108,0xe6a90101,0xfb400108,0xe6aa0101,
	0xfb400108,0xe6ab0101,0xfb400108,0xe6ac0101,0xfb400108,0xe6ad0101,0xfb400108,0xe6af0101,
	0xfb400108,0xe6b00101,0xfb400108,0xe6b10101,0xfb400108,0xe6b20101,0xfb400108,0xe6b30101,
	0xfb400108,0xe6b50101,0xfb400108,0xe6b60101,0xfb400108,0xe6b70101,0xfb400108,0xe6b80101,
	0xfb400108,0xe6ba0101,0xfb400108,0xe6bb0101,0xfb400108,0xe6bc0101,0xfb400108,0xe6bd0101,
	0xfb400108,0xe6bf0101,0xfb400108,0xe6c00101,0xfb400108,0xe6c10101,0xfb400108,0xe6c20101,
	0xfb400108,0xe6c30101,0xfb400108,0xe6c40101,0xfb400108,0xe6c50101,0xfb400108,0xe6c60101,
	0xfb400108,0xe6c70101,0xfb400108,0xe6c80101,0xfb400108,0xe6c90101,0xfb400108,0xe6ca0101,
	0xfb400108,0xe6cb0101,0xfb400108,0xe6cc0101,0xfb400108,0xe6cd0101,0xfb400108,0xe6ce0101,
	0xfb400108,0xe6cf0101,0xfb400108,0xe6d00101,0xfb400108,0xe6d10101,0xfb400108,0xe6d20101,
	0xfb400108,0xe6d30101,0xfb400108,0xe6d40101,0xfb400108,0xe6d50101,0xfb400108,0xe6d60101,
	0xfb400108,0xe6d70101,0xfb400108,0xe6d80101,0xfb400108,0xe6da0101,0xfb400108,0xe6de0101,
	0xfb400108,0xe6df0101,0xfb400108,0xe6e00101,0xfb400108,0xe6e10101,0xfb400108,0xe6e20101,
	0xfb400108,0xe6e30101,0xfb400108,0xe6e40101,0xfb400108,0xe6e50101,0xfb400108,0xe6e70101,
	0xfb400108,0xe6e80101,0xfb400108,0xe6ea0101,0xfb400108,0xe6eb0101,0xfb400108,0xe6ec0101,
	0xfb400108,0xe6ed0101,0xfb400108,0xe6ee0101,0xfb400108,0xe6ef0101,0xfb400108,0xe6f10101,
	0xfb400108,0xe6f50101,0xfb400108,0xe6f60101,0xfb400108,0xe6f80101,0xfb400108,0xe6fa0101,
	0xfb400108,0xe6fb0101,0xfb400108,0xe6fd0101,0xfb400108,0xe7010101,0xfb400108,0xe7020101,
	0xfb400108,0xe7030101,0xfb400108,0xe7040101,0xfb400108,0xe7050101,0xfb400108,0xe7060101,
	0xfb400108,0xe7070101,0xfb400108,0xe70c0101,0xfb400108,0xe70e0101,0xfb400108,0xe70f0101,
	0xfb400108,0xe7110101,0xfb400108,0xe7120101,0xfb400108,0xe7130101,0xfb400108,0xe7160101,
	0xfb400108,0xe7180101,0xfb400108,0xe7190101,0xfb400108,0xe71a0101,0xfb400108,0xe71c0101,
	0xfb400108,0xe71e0101,0xfb400108,0xe7200101,0xfb400108,0xe7210101,0xfb400108,0xe7220101,
	0xfb400108,0xe7230101,0xfb400108,0xe7240101,0xfb400108,0xe7250101,0xfb400108,0xe7270101,
	0xfb400108,0xe7290101,0xfb400108,0xe72e0101,0xfb400108,0xe7300101,0xfb400108,0xe7320101,
	0xfb400108,0xe7330101,0xfb400108,0xe7360101,0xfb400108,0xe7370101,0xfb400108,0xe7380101,
	0xfb400108,0xe7390101,0xfb400108,0xe73b0101,0xfb400108,0xe73c0101,0xfb400108,0xe73e0101,
	0xfb400108,0xe73f0101,0xfb400108,0xe7410101,0xfb400108,0xe7440101,0xfb400108,0xe7450101,
	0xfb400108,0xe7470101,0xfb400108,0xe74a0101,0xfb400108,0xe74b0101,0xfb400108,0xe74d0101,
	0xfb400108,0xe7520101,0xfb400108,0xe7540101,0xfb400108,0xe7550101,0xfb400108,0xe7570101,
	0xfb400108,0xe7580101,0xfb400108,0xe7590101,0xfb400108,0xe75a0101,0xfb400108,0xe75b0101,
	0xfb400108,0xe75d0101,0xfb400108,0xe7620101,0xfb400108,0xe7630101,0xfb400108,0xe7640101,
	0xfb400108,0xe7660101,0xfb400108,0xe7670101,0xfb400108,0xe76b0101,0xfb400108,0xe76c0101,
	0xfb400108,0xe76e0101,0xfb400108,0xe7710101,0xfb400108,0xe7740101,0xfb400108,0xe7760101,
	0xfb400108,0xe7780101,0xfb400108,0xe7790101,0xfb400108,0xe77a0101,0xfb400108,0xe77b0101,
	0xfb400108,0xe77d0101,0xfb400108,0xe7800101,0xfb400108,0xe7820101,0xfb400108,0xe7830101,
	0xfb400108,0xe7850101,0xfb400108,0xe7860101,0xfb400108,0xe7880101,0xfb400108,0xe78a0101,
	0xfb400108,0xe78c0101,0xfb400108,0xe78d0101,0xfb400108,0xe78e0101,0xfb400108,0xe78f0101,
	0xfb400108,0xe7910101,0xfb400108,0xe7920101,0xfb400108,0xe7930101,0xfb400108,0xe7940101,
	0xfb400108,0xe7960101,0xfb400108,0xe7990101,0xfb400108,0xe79b0101,0xfb400108,0xe79f0101,
	0xfb400108,0xe7a00101,0xfb400108,0xe7a10101,0xfb400108,0xe7a40101,0xfb400108,0xe7a60101,
	0xfb400108,0xe7a90101,0xfb400108,0xe7ac0101,0xfb400108,0xe7ae0101,0xfb400108,0xe7b10101,
	0xfb400108,0xe7b20101,0xfb400108,0xe7b40101,0xfb400108,0xe7b90101,0xfb400108,0xe7ba0101,
	0xfb400108,0xe7bb0101,0xfb400108,0xe7bc0101,0xfb400108,0xe7bd0101,0xfb400108,0xe7be0101,
	0xfb400108,0xe7bf0101,0xfb400108,0xe7c00101,0xfb400108,0xe7c20101,0xfb400108,0xe7c50101,
	0xfb400108,0xe7c60101,0xfb400108,0xe7c70101,0xfb400108,0xe7c80101,0xfb400108,0xe7c90101,
	0xfb400108,0xe7ca0101,0xfb400108,0xe7cb0101,0xfb400108,0xe7cc0101,0xfb400108,0xe7cd0101,
	0xfb400108,0xe7ce0101,0xfb400108,0xe7d50101,0xfb400108,0xe7d60101,0xfb400108,0xe7d70101,
	0xfb400108,0xe7db0101,0xfb400108,0xe7df0101,0xfb400108,0xe7e10101,0xfb400108,0xe7e30101,
	0xfb400108,0xe7e40101,0xfb400108,0xe7e60101,0xfb400108,0xe7e70101,0xfb400108,0xe7e80101,
	0xfb400108,0xe7ea0101,0xfb400108,0xe7eb0101,0xfb400108,0xe7ed0101,0xfb400108,0xe7ee0101,
	0xfb400108,0xe7f20101,0xfb400108,0xe7f50101,0xfb400108,0xe7f60101,0xfb400108,0xe7f70101,
	0xfb400108,0xe7f80101,0xfb400108,0xe7f90101,0xfb400108,0xe7fa0101,0xfb400108,0xe7fb0101,
	0xfb400108,0xe7fc0101,0xfb400108,0xe7fe0101,0xfb400108,0xe8010101,0xfb400108,0xe8020101,
	0xfb400108,0xe8030101,0xfb400108,0xe8040101,0xfb400108,0xe8060101,0xfb400108,0xe80d0101,
	0xfb400108,0xe8100101,0xfb400108,0xe8120101,0xfb400108,0xe8140101,0xfb400108,0xe8150101,
	0xfb400108,0xe8180101,0xfb400108,0xe8190101,0xfb400108,0xe81a0101,0xfb400108,0xe81b0101,
	0xfb400108,0xe81c0101,0xfb400108,0xe81e0101,0xfb400108,0xe81f0101,0xfb400108,0xe8200101,
	0xfb400108,0xe8220101,0xfb400108,0xe8230101,0xfb400108,0xe8240101,0xfb400108,0xe8250101,
	0xfb400108,0xe8260101,0xfb400108,0xe8270101,0xfb400108,0xe8280101,0xfb400108,0xe82b0101,
	0xfb400108,0xe82c0101,0xfb400108,0xe82d0101,0xfb400108,0xe82e0101,0xfb400108,0xe82f0101,
	0xfb400108,0xe8300101,0xfb400108,0xe8310101,0xfb400108,0xe8340101,0xfb400108,0xe8350101,
	0xfb400108,0xe8360101,0xfb400108,0xe83a0101,0xfb400108,0xe83b0101,0xfb400108,0xe83f0101,
	0xfb400108,0xe8470101,0xfb400108,0xe84b0101,0xfb400108,0xe84d0101,0xfb400108,0xe84f0101,
	0xfb400108,0xe8520101,0xfb400108,0xe8560101,0xfb400108,0xe8570101,0xfb400108,0xe8580101,
	0xfb400108,0xe8590101,0xfb400108,0xe85a0101,0xfb400108,0xe85b0101,0xfb400108,0xe85c0101,
	0xfb400108,0xe85d0101,0xfb400108,0xe85e0101,0xfb400108,0xe85f0101,0xfb400108,0xe86a0101,
	0xfb400108,0xe86c0101,0xfb400108,0xe86d0101,0xfb400108,0xe86e0101,0xfb400108,0xe86f0101,
	0xfb400108,0xe8700101,0xfb400108,0xe8710101,0xfb400108,0xe8720101,0xfb400108,0xe8730101,
	0xfb400108,0xe8750101,0xfb400108,0xe8780101,0xfb400108,0xe8790101,0xfb400108,0xe87a0101,
	0xfb400108,0xe87b0101,0xfb400108,0xe87c0101,0xfb400108,0xe87d0101,0xfb400108,0xe87e0101,
	0xfb400108,0xe87f0101,0xfb400108,0xe8800101,0xfb400108,0xe8820101,0xfb400108,0xe8840101,
	0xfb400108,0xe8870101,0xfb400108,0xe8880101,0xfb400108,0xe8890101,0xfb400108,0xe88a0101,
	0xfb400108,0xe88b0101,0xfb400108,0xe88c0101,0xfb400108,0xe88d0101,0xfb400108,0xe88e0101,
	0xfb400108,0xe8900101,0xfb400108,0xe8910101,0xfb400108,0xe8920101,0xfb400108,0xe8940101,
	0xfb400108,0xe8950101,0xfb400108,0xe8960101,0xfb400108,0xe8980101,0xfb400108,0xe8990101,
	0xfb400108,0xe89a0101,0xfb400108,0xe89b0101,0xfb400108,0xe89c0101,0xfb400108,0xe89d0101,
	0xfb400108,0xe89e0101,0xfb400108,0xe89f0101,0xfb400108,0xe8a00101,0xfb400108,0xe8a10101,
	0xfb400108,0xe8a30101,0xfb400108,0xe8a40101,0xfb400108,0xe8a50101,0xfb400108,0xe8a90101,
	0xfb400108,0xe8aa0101,0xfb400108,0xe8ab0101,0xfb400108,0xe8ac0101,0xfb400108,0xe8ae0101,
	0xfb400108,0xe8b10101,0xfb400108,0xe8b20101,0xfb400108,0xe8b40101,0xfb400108,0xe8b60101,
	0xfb400108,0xe8b70101,0xfb400108,0xe8b80101,0xfb400108,0xe8b90101,0xfb400108,0xe8ba0101,
	0xfb400108,0xe8bb0101,0xfb400108,0xe8bc0101,0xfb400108,0xe8bd0101,0xfb400108,0xe8be0101,
	0xfb400108,0xe8bf0101,0xfb400108,0xe8c10101,0xfb400108,0xe8c30101,0xfb400108,0xe8c40101,
	0xfb400108,0xe8c50101,0xfb400108,0xe8c60101,0xfb400108,0xe8c70101,0xfb400108,0xe8c80101,
	0xfb400108,0xe8ca0101,0xfb400108,0xe8cc0101,0xfb400108,0xe8ce0101,0xfb400108,0xe8cf0101,
	0xfb400108,0xe8d00101,0xfb400108,0xe8d10101,0xfb400108,0xe8d30101,0xfb400108,0xe8d40101,
	0xfb400108,0xe8d60101,0xfb400108,0xe8d70101,0xfb400108,0xe8d90101,0xfb400108,0xe8db0101,
	0xfb400108,0xe8dc0101,0xfb400108,0xe8dd0101,0xfb400108,0xe8de0101,0xfb400108,0xe8df0101,
	0xfb400108,0xe8e10101,0xfb400108,0xe8e20101,0xfb400108,0xe8e40101,0xfb400108,0xe8e50101,
	0xfb400108,0xe8e60101,0xfb400108,0xe8e70101,0xfb400108,0xe8e80101,0xfb400108,0xe8e90101,
	0xfb400108,0xe8ea0101,0xfb400108,0xe8eb0101,0xfb400108,0xe8ec0101,0xfb400108,0xe8ed0101,
	0xfb400108,0xe8ef0101,0xfb400108,0xe8f20101,0xfb400108,0xe8f30101,0xfb400108,0xe8f40101,
	0xfb400108,0xe8f60101,0xfb400108,0xe8f70101,0xfb400108,0xe8f80101,0xfb400108,0xe8fb0101,
	0xfb400108,0xe8fd0101,0xfb400108,0xe8fe0101,0xfb400108,0xe8ff0101,0xfb400108,0xe9000101,
	0xfb400108,0xe9020101,0xfb400108,0xe9030101,0xfb400108,0xe9040101,0xfb400108,0xe9060101,
	0xfb400108,0xe9070101,0xfb400108,0xe9080101,0xfb400108,0xe9090101,0xfb400108,0xe90a0101,
	0xfb400108,0xe90c0101,0xfb400108,0xe90f0101,0xfb400108,0xe9110101,0xfb400108,0xe9130101,
	0xfb400108,0xe9140101,0xfb400108,0xe9150101,0xfb400108,0xe9160101,0xfb400108,0xe9170101,
	0xfb400108,0xe9180101,0xfb400108,0xe9190101,0xfb400108,0xe91a0101,0xfb400108,0xe91b0101,
	0xfb400108,0xe91c0101,0xfb400108,0xe91d0101,0xfb400108,0xe91e0101,0xfb400108,0xe9210101,
	0xfb400108,0xe9220101,0xfb400108,0xe9230101,0xfb400108,0xe9250101,0xfb400108,0xe9260101,
	0xfb400108,0xe9270101,0xfb400108,0xe9280101,0xfb400108,0xe9290101,0xfb400108,0xe92a0101,
	0xfb400108,0xe92b0101,0xfb400108,0xe92c0101,0xfb400108,0xe92e0101,0xfb400108,0xe92f0101,
	0xfb400108,0xe9310101,0xfb400108,0xe9320101,0xfb400108,0xe9330101,0xfb400108,0xe9350101,
	0xfb400108,0xe9360101,0xfb400108,0xe9370101,0xfb400108,0xe9380101,0xfb400108,0xe93a0101,
	0xfb400108,0xe93b0101,0xfb400108,0xe93c0101,0xfb400108,0xe93e0101,0xfb400108,0xe9400101,
	0xfb400108,0xe9410101,0xfb400108,0xe9430101,0xfb400108,0xe9440101,0xfb400108,0xe9450101,
	0xfb400108,0xe9460101,0xfb400108,0xe9470101,0xfb400108,0xe9480101,0xfb400108,0xe9490101,
	0xfb400108,0xe94a0101,0xfb400108,0xe94b0101,0xfb400108,0xe94c0101,0xfb400108,0xe94d0101,
	0xfb400108,0xe94e0101,0xfb400108,0xe94f0101,0xfb400108,0xe9500101,0xfb400108,0xe9510101,
	0xfb400108,0xe9520101,0xfb400108,0xe9530101,0xfb400108,0xe9550101,0xfb400108,0xe9560101,
	0xfb400108,0xe9580101,0xfb400108,0xe9590101,0xfb400108,0xe95b0101,0xfb400108,0xe95c0101,
	0xfb400108,0xe95f0101,0xfb400108,0xe9610101,0xfb400108,0xe9620101,0xfb400108,0xe9640101,
	0xfb400108,0xe9650101,0xfb400108,0xe9670101,0xfb400108,0xe9680101,0xfb400108,0xe9690101,
	0xfb400108,0xe96a0101,0xfb400108,0xe96c0101,0xfb400108,0xe96d0101,0xfb400108,0xe96f0101,
	0xfb400108,0xe9700101,0xfb400108,0xe9720101,0xfb400108,0xe9730101,0xfb400108,0xe9740101,
	0xfb400108,0xe9750101,0xfb400108,0xe9760101,0xfb400108,0xe97a0101,0xfb400108,0xe97b0101,
	0xfb400108,0xe97d0101,0xfb400108,0xe97e0101,0xfb400108,0xe97f0101,0xfb400108,0xe9810101,
	0xfb400108,0xe9830101,0xfb400108,0xe9850101,0xfb400108,0xe98a0101,0xfb400108,0xe98b0101,
	0xfb400108,0xe98c0101,0xfb400108,0xe98e0101,0xfb400108,0xe98f0101,0xfb400108,0xe9900101,
	0xfb400108,0xe9910101,0xfb400108,0xe9920101,0xfb400108,0xe9930101,0xfb400108,0xe9960101,
	0xfb400108,0xe9970101,0xfb400108,0xe9990101,0xfb400108,0xe99a0101,0xfb400108,0xe99d0101,
	0xfb400108,0xe99e0101,0xfb400108,0xe99f0101,0xfb400108,0xe9a00101,0xfb400108,0xe9a10101,
	0xfb400108,0xe9a20101,0xfb400108,0xe9a30101,0xfb400108,0xe9a40101,0xfb400108,0xe9a50101,
	0xfb400108,0xe9a60101,0xfb400108,0xe9a90101,0xfb400108,0xe9aa0101,0xfb400108,0xe9ac0101,
	0xfb400108,0xe9ae0101,0xfb400108,0xe9af0101,0xfb400108,0xe9b00101,0xfb400108,0xe9b20101,
	0xfb400108,0xe9b30101,0xfb400108,0xe9b50101,0xfb400108,0xe9b60101,0xfb400108,0xe9b80101,
	0xfb400108,0xe9b90101,0xfb400108,0xe9ba0101,0xfb400108,0xe9bc0101,0xfb400108,0xe9bd0101,
	0xfb400108,0xe9be0101,0xfb400108,0xe9bf0101,0xfb400108,0xe9c00101,0xfb400108,0xe9c20101,
	0xfb400108,0xe9c30101,0xfb400108,0xe9c40101,0xfb400108,0xe9c50101,0xfb400108,0xe9c60101,
	0xfb400108,0xe9c70101,0xfb400108,0xe9c80101,0xfb400108,0xe9c90101,0xfb400108,0xe9cb0101,
	0xfb400108,0xe9cd0101,0xfb400108,0xe9cf0101,0xfb400108,0xe9d10101,0xfb400108,0xe9d20101,
	0xfb400108,0xe9d30101,0xfb400108,0xe9d50101,0xfb400108,0xe9d60101,0xfb400108,0xe9d70101,
	0xfb400108,0xe9d80101,0xfb400108,0xe9d90101,0xfb400108,0xe9da0101,0xfb400108,0xe9dc0101,
	0xfb400108,0xe9dd0101,0xfb400108,0xe9de0101,0xfb400108,0xe9e10101,0xfb400108,0xe9e20101,
	0xfb400108,0xe9e30101,0xfb400108,0xe9e40101,0xfb400108,0xe9e50101,0xfb400108,0xe9e60101,
	0xfb400108,0xe9e70101,0xfb400108,0xe9e80101,0xfb400108,0xe9e90101,0xfb400108,0xe9ea0101,
	0xfb400108,0xe9eb0101,0xfb400108,0xe9ec0101,0xfb400108,0xe9ee0101,0xfb400108,0xe9ef0101,
	0xfb400108,0xe9f00101,0xfb400108,0xe9f10101,0xfb400108,0xe9f30101,0xfb400108,0xe9f40101,
	0xfb400108,0xe9f50101,0xfb400108,0xe9f60101,0xfb400108,0xe9f70101,0xfb400108,0xe9f80101,
	0xfb400108,0xe9f90101,0xfb400108,0xe9fa0101,0xfb400108,0xe9fb0101,0xfb400108,0xe9fc0101,
	0xfb400108,0xe9fe0101,0xfb400108,0xea000101,0xfb400108,0xea010101,0xfb400108,0xea020101,
	0xfb400108,0xea030101,0xfb400108,0xea040101,0xfb400108,0xea050101,0xfb400108,0xea060101,
	0xfb400108,0xea070101,0xfb400108,0xea080101,0xfb400108,0xea090101,0xfb400108,0xea0b0101,
	0xfb400108,0xea0c0101,0xfb400108,0xea0d0101,0xfb400108,0xea0e0101,0xfb400108,0xea0f0101,
	0xfb400108,0xea100101,0xfb400108,0xea110101,0xfb400108,0xea120101,0xfb400108,0xea130101,
	0xfb400108,0xea140101,0xfb400108,0xea150101,0xfb400108,0xea160101,0xfb400108,0xea190101,
	0xfb400108,0xea1a0101,0xfb400108,0xea1b0101,0xfb400108,0xea1c0101,0xfb400108,0xea1d0101,
	0xfb400108,0xea1e0101,0xfb400108,0xea200101,0xfb400108,0xea220101,0xfb400108,0xea230101,
	0xfb400108,0xea240101,0xfb400108,0xea250101,0xfb400108,0xea260101,0xfb400108,0xea270101,
	0xfb400108,0xea290101,0xfb400108,0xea2b0101,0xfb400108,0xea2c0101,0xfb400108,0xea2d0101,
	0xfb400108,0xea2e0101,0xfb400108,0xea300101,0xfb400108,0xea320101,0xfb400108,0xea330101,
	0xfb400108,0xea340101,0xfb400108,0xea360101,0xfb400108,0xea370101,0xfb400108,0xea380101,
	0xfb400108,0xea390101,0xfb400108,0xea3a0101,0xfb400108,0xea3b0101,0xfb400108,0xea3c0101,
	0xfb400108,0xea3f0101,0xfb400108,0xea400101,0xfb400108,0xea410101,0xfb400108,0xea420101,
	0xfb400108,0xea430101,0xfb400108,0xea450101,0xfb400108,0xea460101,0xfb400108,0xea480101,
	0xfb400108,0xea490101,0xfb400108,0xea4a0101,0xfb400108,0xea4b0101,0xfb400108,0xea4c0101,
	0xfb400108,0xea4d0101,0xfb400108,0xea4e0101,0xfb400108,0xea4f0101,0xfb400108,0xea510101,
	0xfb400108,0xea520101,0xfb400108,0xea530101,0xfb400108,0xea540101,0xfb400108,0xea550101,
	0xfb400108,0xea560101,0xfb400108,0xea570101,0xfb400108,0xea5a0101,0xfb400108,0xea5c0101,
	0xfb400108,0xea5d0101,0xfb400108,0xea5e0101,0xfb400108,0xea5f0101,0xfb400108,0xea600101,
	0xfb400108,0xea620101,0xfb400108,0xea630101,0xfb400108,0xea640101,0xfb400108,0xea660101,
	0xfb400108,0xea670101,0xfb400108,0xea680101,0xfb400108,0xea690101,0xfb400108,0xea6a0101,
	0xfb400108,0xea6b0101,0xfb400108,0xea6c0101,0xfb400108,0xea6d0101,0xfb400108,0xea6e0101,
	0xfb400108,0xea6f0101,0xfb400108,0xea700101,0xfb400108,0xea720101,0xfb400108,0xea730101,
	0xfb400108,0xea740101,0xfb400108,0xea750101,0xfb400108,0xea760101,0xfb400108,0xea770101,
	0xfb400108,0xea780101,0xfb400108,0xea7a0101,0xfb400108,0xea7b0101,0xfb400108,0xea7d0101,
	0xfb400108,0xea7e0101,0xfb400108,0xea7f0101,0xfb400108,0xea810101,0xfb400108,0xea820101,
	0xfb400108,0xea830101,0xfb400108,0xea850101,0xfb400108,0xea860101,0xfb400108,0xea870101,
	0xfb400108,0xea880101,0xfb400108,0xea890101,0xfb400108,0xea8a0101,0xfb400108,0xea8b0101,
	0xfb400108,0xea8c0101,0xfb400108,0xea8d0101,0xfb400108,0xea8f0101,0xfb400108,0xea920101,
	0xfb400108,0xea930101,0xfb400108,0xea940101,0xfb400108,0xea950101,0xfb400108,0xea960101,
	0xfb400108,0xea980101,0xfb400108,0xea990101,0xfb400108,0xea9a0101,0xfb400108,0xea9b0101,
	0xfb400108,0xea9c0101,0xfb400108,0xea9d0101,0xfb400108,0xea9e0101,0xfb400108,0xea9f0101,
	0xfb400108,0xeaa10101,0xfb400108,0xeaa20101,0xfb400108,0xeaa30101,0xfb400108,0xeaa40101,
	0xfb400108,0xeaa50101,0xfb400108,0xeaa60101,0xfb400108,0xeaa70101,0xfb400108,0xeaa80101,
	0xfb400108,0xeaaa0101,0xfb400108,0xeaad0101,0xfb400108,0xeaae0101,0xfb400108,0xeaaf0101,
	0xfb400108,0xeab00101,0xfb400108,0xeab10101,0xfb400108,0xeab20101,0xfb400108,0xeab30101,
	0xfb400108,0xeab40101,0xfb400108,0xeab50101,0xfb400108,0xeab60101,0xfb400108,0xeab70101,
	0xfb400108,0xeab80101,0xfb400108,0xeab90101,0xfb400108,0xeaba0101,0xfb400108,0xeabb0101,
	0xfb400108,0xeabc0101,0xfb400108,0xeabd0101,0xfb400108,0xeabe0101,0xfb400108,0xeabf0101,
	0xfb400108,0xeac00101,0xfb400108,0xeac10101,0xfb400108,0xeac20101,0xfb400108,0xeac30101,
	0xfb400108,0xeac40101,0xfb400108,0xeac50101,0xfb400108,0xeac60101,0xfb400108,0xeac70101,
	0xfb400108,0xeac80101,0xfb400108,0xeac90101,0xfb400108,0xeaca0101,0xfb400108,0xeacb0101,
	0xfb400108,0xeacc0101,0xfb400108,0xeacd0101,0xfb400108,0xeace0101,0xfb400108,0xeacf0101,
	0xfb400108,0xead00101,0xfb400108,0xead10101,0xfb400108,0xead20101,0xfb400108,0xead30101,
	0xfb400108,0xead40101,0xfb400108,0xead50101,0xfb400108,0xead60101,0xfb400108,0xead70101,
	0xfb400108,0xead80101,0xfb400108,0xead90101,0xfb400108,0xeada0101,0xfb400108,0xeadb0101,
	0xfb400108,0xeadc0101,0xfb400108,0xeadd0101,0xfb400108,0xeade0101,0xfb400108,0xeadf0101,
	0xfb400108,0xeae00101,0xfb400108,0xeae10101,0xfb400108,0xeae20101,0xfb400108,0xeae30101,
	0xfb400108,0xeae40101,0xfb400108,0xeae50101,0xfb400108,0xeae60101,0xfb400108,0xeae70101,
	0xfb400108,0xeae80101,0xfb400108,0xeae90101,0xfb400108,0xeaea0101,0xfb400108,0xeaeb0101,
	0xfb400108,0xeaec0101,0xfb400108,0xeaed0101,0xfb400108,0xeaee0101,0xfb400108,0xeaef0101,
	0xfb400108,0xeaf00101,0xfb400108,0xeaf10101,0xfb400108,0xeaf20101,0xfb400108,0xeaf30101,
	0xfb400108,0xeaf40101,0xfb400108,0xeaf50101,0xfb400108,0xeaf60101,0xfb400108,0xeaf70101,
	0xfb400108,0xeaf80101,0xfb400108,0xeaf90101,0xfb400108,0xeafa0101,0xfb400108,0xeafb0101,
	0xfb400108,0xeafc0101,0xfb400108,0xeafd0101,0xfb400108,0xeafe0101,0xfb400108,0xeaff0101,
	0xfb400108,0xeb000101,0xfb400108,0xeb010101,0xfb400108,0xeb020101,0xfb400108,0xeb030101,
	0xfb400108,0xeb040101,0xfb400108,0xeb050101,0xfb400108,0xeb060101,0xfb400108,0xeb070101,
	0xfb400108,0xeb080101,0xfb400108,0xeb090101,0xfb400108,0xeb0a0101,0xfb400108,0xeb0b0101,
	0xfb400108,0xeb0c0101,0xfb400108,0xeb0d0101,0xfb400108,0xeb0e0101,0xfb400108,0xeb0f0101,
	0xfb400108,0xeb100101,0xfb400108,0xeb110101,0xfb400108,0xeb120101,0xfb400108,0xeb130101,
	0xfb400108,0xeb140101,0xfb400108,0xeb150101,0xfb400108,0xeb160101,0xfb400108,0xeb170101,
	0xfb400108,0xeb180101,0xfb400108,0xeb190101,0xfb400108,0xeb1a0101,0xfb400108,0xeb1b0101,
	0xfb400108,0xeb1c0101,0xfb400108,0xeb1d0101,0xfb400108,0xeb1e0101,0xfb400108,0xeb1f0101,
	0xfb400108,0xeb250101,0xfb400108,0xeb260101,0xfb400108,0xeb280101,0xfb400108,0xeb290101,
	0xfb400108,0xeb2a0101,0xfb400108,0xeb2b0101,0xfb400108,0xeb2c0101,0xfb400108,0xeb2d0101,
	0xfb400108,0xeb2e0101,0xfb400108,0xeb2f0101,0xfb400108,0xeb300101,0xfb400108,0xeb310101,
	0xfb400108,0xeb330101,0xfb400108,0xeb340101,0xfb400108,0xeb350101,0xfb400108,0xeb360101,
	0xfb400108,0xeb380101,0xfb400108,0xeb3b0101,0xfb400108,0xeb3c0101,0xfb400108,0xeb3d0101,
	0xfb400108,0xeb3f0101,0xfb400108,0xeb400101,0xfb400108,0xeb410101,0xfb400108,0xeb420101,
	0xfb400108,0xeb440101,0xfb400108,0xeb450101,0xfb400108,0xeb480101,0xfb400108,0xeb4a0101,
	0xfb400108,0xeb4b0101,0xfb400108,0xeb4d0101,0xfb400108,0xeb4e0101,0xfb400108,0xeb4f0101,
	0xfb400108,0xeb500101,0xfb400108,0xeb510101,0xfb400108,0xeb520101,0xfb400108,0xeb530101,
	0xfb400108,0xeb540101,0xfb400108,0xeb550101,0xfb400108,0xeb560101,0xfb400108,0xeb570101,
	0xfb400108,0xeb580101,0xfb400108,0xeb5a0101,0xfb400108,0xeb5b0101,0xfb400108,0xeb5c0101,
	0xfb400108,0xeb5d0101,0xfb400108,0xeb5e0101,0xfb400108,0xeb5f0101,0xfb400108,0xeb600101,
	0xfb400108,0xeb610101,0xfb400108,0xeb680101,0xfb400108,0xeb690101,0xfb400108,0xeb6b0101,
	0xfb400108,0xeb6c0101,0xfb400108,0xeb6d0101,0xfb400108,0xeb6e0101,0xfb400108,0xeb6f0101,
	0xfb400108,0xeb700101,0xfb400108,0xeb710101,0xfb400108,0xeb720101,0xfb400108,0xeb730101,
	0xfb400108,0xeb740101,0xfb400108,0xeb750101,0xfb400108,0xeb760101,0xfb400108,0xeb770101,
	0xfb400108,0xeb780101,0xfb400108,0xeb7a0101,0xfb400108,0xeb7d0101,0xfb400108,0xeb7e0101,
	0xfb400108,0xeb7f0101,0xfb400108,0xeb800101,0xfb400108,0xeb850101,0xfb400108,0xeb880101,
	0xfb400108,0xeb8c0101,0xfb400108,0xeb8e0101,0xfb400108,0xeb8f0101,0xfb400108,0xeb900101,
	0xfb400108,0xeb910101,0xfb400108,0xeb940101,0xfb400108,0xeb950101,0xfb400108,0xeb970101,
	0xfb400108,0xeb980101,0xfb400108,0xeb990101,0xfb400108,0xeb9c0101,0xfb400108,0xeb9d0101,
	0xfb400108,0xeb9e0101,0xfb400108,0xeb9f0101,0xfb400108,0xeba00101,0xfb400108,0xeba20101,
	0xfb400108,0xeba30101,0xfb400108,0xeba40101,0xfb400108,0xeba50101,0xfb400108,0xeba60101,
	0xfb400108,0xeba70101,0xfb400108,0xeba80101,0xfb400108,0xeba90101,0xfb400108,0xebab0101,
	0xfb400108,0xebac0101,0xfb400108,0xebad0101,0xfb400108,0xebae0101,0xfb400108,0xebaf0101,
	0xfb400108,0xebb00101,0xfb400108,0xebb10101,0xfb400108,0xebb20101,0xfb400108,0xebb60101,
	0xfb400108,0xebb80101,0xfb400108,0xebb90101,0xfb400108,0xebba0101,0xfb400108,0xebbb0101,
	0xfb400108,0xebbc0101,0xfb400108,0xebbd0101,0xfb400108,0xebbe0101,0xfb400108,0xebc00101,
	0xfb400108,0xebc30101,0xfb400108,0xebc40101,0xfb400108,0xebc60101,0xfb400108,0xebc70101,
	0xfb400108,0xebc80101,0xfb400108,0xebc90101,0xfb400108,0xebca0101,0xfb400108,0xebcc0101,
	0xfb400108,0xebce0101,0xfb400108,0xebd00101,0xfb400108,0xebd10101,0xfb400108,0xebd80101,
	0xfb400108,0xebda0101,0xfb400108,0xebdc0101,0xfb400108,0xebdd0101,0xfb400108,0xebde0101,
	0xfb400108,0xebdf0101,0xfb400108,0xebe00101,0xfb400108,0xebe20101,0xfb400108,0xebe30101,
	0xfb400108,0xebe40101,0xfb400108,0xebe50101,0xfb400108,0xebe60101,0xfb400108,0xebe70101,
	0xfb400108,0xebe80101,0xfb400108,0xebe90101,0xfb400108,0xebec0101,0xfb400108,0xebed0101,
	0xfb400108,0xebee0101,0xfb400108,0xebf00101,0xfb400108,0xebf10101,0xfb400108,0xebf20101,
	0xfb400108,0xebf40101,0xfb400108,0xebf60101,0xfb400108,0xebf70101,0xfb400108,0xebf80101,
	0xfb400108,0xebfa0101,0xfb400108,0xebfb0101,0xfb400108,0xebfc0101,0xfb400108,0xebfe0101,
	0xfb400108,0xebff0101,0xfb400108,0xec000101,0xfb400108,0xec010101,0xfb400108,0xec020101,
	0xfb400108,0xec030101,0xfb400108,0xec040101,0xfb400108,0xec080101,0xfb400108,0xec090101,
	0xfb400108,0xec0a0101,0xfb400108,0xec0b0101,0xfb400108,0xec0c0101,0xfb400108,0xec0e0101,
	0xfb400108,0xec120101,0xfb400108,0xec170101,0xfb400108,0xec1c0101,0xfb400108,0xec1d0101,
	0xfb400108,0xec1e0101,0xfb400108,0xec200101,0xfb400108,0xec230101,0xfb400108,0xec250101,
	0xfb400108,0xec2b0101,0xfb400108,0xec2c0101,0xfb400108,0xec2d0101,0xfb400108,0xec310101,
	0xfb400108,0xec330101,0xfb400108,0xec360101,0xfb400108,0xec370101,0xfb400108,0xec390101,
	0xfb400108,0xec3a0101,0xfb400108,0xec3b0101,0xfb400108,0xec3c0101,0xfb400108,0xec3e0101,
	0xfb400108,0xec3f0101,0xfb400108,0xec430101,0xfb400108,0xec440101,0xfb400108,0xec450101,
	0xfb400108,0xec480101,0xfb400108,0xec4b0101,0xfb400108,0xec4c0101,0xfb400108,0xec4d0101,
	0xfb400108,0xec4e0101,0xfb400108,0xec4f0101,0xfb400108,0xec510101,0xfb400108,0xec520101,
	0xfb400108,0xec530101,0xfb400108,0xec560101,0xfb400108,0xec580101,0xfb400108,0xec590101,
	0xfb400108,0xec5a0101,0xfb400108,0xec620101,0xfb400108,0xec630101,0xfb400108,0xec650101,
	0xfb400108,0xec660101,0xfb400108,0xec670101,0xfb400108,0xec6b0101,0xfb400108,0xec6c0101,
	0xfb400108,0xec6d0101,0xfb400108,0xec6e0101,0xfb400108,0xec6f0101,0xfb400108,0xec710101,
	0xfb400108,0xec730101,0xfb400108,0xec750101,0xfb400108,0xec770101,0xfb400108,0xec780101,
	0xfb400108,0xec7a0101,0xfb400108,0xec7b0101,0xfb400108,0xec7c0101,0xfb400108,0xec7f0101,
	0xfb400108,0xec800101,0xfb400108,0xec840101,0xfb400108,0xec870101,0xfb400108,0xec8a0101,
	0xfb400108,0xec8b0101,0xfb400108,0xec8d0101,0xfb400108,0xec8e0101,0xfb400108,0xec910101,
	0xfb400108,0xec920101,0xfb400108,0xec950101,0xfb400108,0xec960101,0xfb400108,0xec970101,
	0xfb400108,0xec980101,0xfb400108,0xec9a0101,0xfb400108,0xec9c0101,0xfb400108,0xec9d0101,
	0xfb400108,0xec9e0101,0xfb400108,0xeca00101,0xfb400108,0xeca20101,0xfb400108,0xeca80101,
	0xfb400108,0xecac0101,0xfb400108,0xecaf0101,0xfb400108,0xecb00101,0xfb400108,0xecb40101,
	0xfb400108,0xecb50101,0xfb400108,0xecb60101,0xfb400108,0xecb70101,0xfb400108,0xecba0101,
	0xfb400108,0xecc00101,0xfb400108,0xecc10101,0xfb400108,0xecc20101,0xfb400108,0xecc30101,
	0xfb400108,0xecc60101,0xfb400108,0xecc70101,0xfb400108,0xecc80101,0xfb400108,0xeccb0101,
	0xfb400108,0xeccd0101,0xfb400108,0xecce0101,0xfb400108,0xeccf0101,0xfb400108,0xecd10101,
	0xfb400108,0xecd20101,0xfb400108,0xecd80101,0xfb400108,0xecd90101,0xfb400108,0xecda0101,
	0xfb400108,0xecdc0101,0xfb400108,0xecdd0101,0xfb400108,0xecdf0101,0xfb400108,0xece40101,
	0xfb400108,0xece60101,0xfb400108,0xece70101,0xfb400108,0xece90101,0xfb400108,0xecec0101,
	0xfb400108,0xeced0101,0xfb400108,0xecf20101,0xfb400108,0xecf40101,0xfb400108,0xecf90101,
	0xfb400108,0xecff0101,0xfb400108,0xed000101,0xfb400108,0xed020101,0xfb400108,0xed030101,
	0xfb400108,0xed050101,0xfb400108,0xed060101,0xfb400108,0xed080101,0xfb400108,0xed090101,
	0xfb400108,0xed0a0101,0xfb400108,0xed0d0101,0xfb400108,0xed0f0101,0xfb400108,0xed100101,
	0xfb400108,0xed110101,0xfb400108,0xed130101,0xfb400108,0xed140101,0xfb400108,0xed150101,
	0xfb400108,0xed160101,0xfb400108,0xed180101,0xfb400108,0xed1c0101,0xfb400108,0xed1d0101,
	0xfb400108,0xed1f0101,0xfb400108,0xed200101,0xfb400108,0xed210101,0xfb400108,0xed220101,
	0xfb400108,0xed230101,0xfb400108,0xed240101,0xfb400108,0xed260101,0xfb400108,0xed280101,
	0xfb400108,0xed290101,0xfb400108,0xed2c0101,0xfb400108,0xed2d0101,0xfb400108,0xed2f0101,
	0xfb400108,0xed300101,0xfb400108,0xed340101,0xfb400108,0xed360101,0xfb400108,0xed370101,
	0xfb400108,0xed380101,0xfb400108,0xed3a0101,0xfb400108,0xed3f0101,0xfb400108,0xed400101,
	0xfb400108,0xed420101,0xfb400108,0xed440101,0xfb400108,0xed490101,0xfb400108,0xed4c0101,
	0xfb400108,0xed500101,0xfb400108,0xed550101,0xfb400108,0xed560101,0xfb400108,0xed570101,
	0xfb400108,0xed580101,0xfb400108,0xed5b0101,0xfb400108,0xed5d0101,0xfb400108,0xed5f0101,
	0xfb400108,0xed610101,0xfb400108,0xed620101,0xfb400108,0xed640101,0xfb400108,0xed650101,
	0xfb400108,0xed670101,0xfb400108,0xed680101,0xfb400108,0xed6b0101,0xfb400108,0xed6c0101,
	0xfb400108,0xed6d0101,0xfb400108,0xed700101,0xfb400108,0xed710101,0xfb400108,0xed720101,
	0xfb400108,0xed730101,0xfb400108,0xed750101,0xfb400108,0xed760101,0xfb400108,0xed790101,
	0xfb400108,0xed7a0101,0xfb400108,0xed7b0101,0xfb400108,0xed7d0101,0xfb400108,0xed7e0101,
	0xfb400108,0xed7f0101,0xfb400108,0xed800101,0xfb400108,0xed810101,0xfb400108,0xed830101,
	0xfb400108,0xed840101,0xfb400108,0xed860101,0xfb400108,0xed870101,0xfb400108,0xed8a0101,
	0xfb400108,0xed8b0101,0xfb400108,0xed8d0101,0xfb400108,0xed8f0101,0xfb400108,0xed900101,
	0xfb400108,0xed920101,0xfb400108,0xed960101,0xfb400108,0xed970101,0xfb400108,0xed980101,
	0xfb400108,0xed990101,0xfb400108,0xed9a0101,0xfb400108,0xed9c0101,0xfb400108,0xeda20101,
	0xfb400108,0xeda50101,0xfb400108,0xedac0101,0xfb400108,0xedad0101,0xfb400108,0xedb00101,
	0xfb400108,0xedb10101,0xfb400108,0xedb30101,0xfb400108,0xedb40101,0xfb400108,0xedb60101,
	0xfb400108,0xedb70101,0xfb400108,0xedb90101,0xfb400108,0xedba0101,0xfb400108,0xedbb0101,
	0xfb400108,0xedbc0101,0xfb400108,0xedbd0101,0xfb400108,0xedbe0101,0xfb400108,0xedc10101,
	0xfb400108,0xedc20101,0xfb400108,0xedc30101,0xfb400108,0xedc80101,0xfb400108,0xedc90101,
	0xfb400108,0xedca0101,0xfb400108,0xedcd0101,0xfb400108,0xedce0101,0xfb400108,0xedcf0101,
	0xfb400108,0xedd00101,0xfb400108,0xedd20101,0xfb400108,0xedd30101,0xfb400108,0xedd40101,
	0xfb400108,0xedd50101,0xfb400108,0xedd70101,0xfb400108,0xedda0101,0xfb400108,0xeddb0101,
	0xfb400108,0xeddc0101,0xfb400108,0xeddf0101,0xfb400108,0xede20101,0xfb400108,0xede30101,
	0xfb400108,0xede50101,0xfb400108,0xede70101,0xfb400108,0xede80101,0xfb400108,0xede90101,
	0xfb400108,0xedea0101,0xfb400108,0xeded0101,0xfb400108,0xedef0101,0xfb400108,0xedf00101,
	0xfb400108,0xedf20101,0xfb400108,0xedf40101,0xfb400108,0xedf50101,0xfb400108,0xedf60101,
	0xfb400108,0xedf80101,0xfb400108,0xedfa0101,0xfb400108,0xedfd0101,0xfb400108,0xedfe0101,
	0xfb400108,0xedff0101,0xfb400108,0xee000101,0xfb400108,0xee010101,0xfb400108,0xee020101,
	0xfb400108,0xee030101,0xfb400108,0xee040101,0xfb400108,0xee060101,0xfb400108,0xee070101,
	0xfb400108,0xee080101,0xfb400108,0xee090101,0xfb400108,0xee0b0101,0xfb400108,0xee0f0101,
	0xfb400108,0xee120101,0xfb400108,0xee130101,0xfb400108,0xee150101,0xfb400108,0xee180101,
	0xfb400108,0xee190101,0xfb400108,0xee1b0101,0xfb400108,0xee1c0101,0xfb400108,0xee1e0101,
	0xfb400108,0xee1f0101,0xfb400108,0xee220101,0xfb400108,0xee260101,0xfb400108,0xee270101,
	0xfb400108,0xee280101,0xfb400108,0xee2a0101,0xfb400108,0xee2c0101,0xfb400108,0xee2e0101,
	0xfb400108,0xee300101,0xfb400108,0xee310101,0xfb400108,0xee330101,0xfb400108,0xee350101,
	0xfb400108,0xee360101,0xfb400108,0xee370101,0xfb400108,0xee390101,0xfb400108,0xee3b0101,
	0xfb400108,0xee3c0101,0xfb400108,0xee3d0101,0xfb400108,0xee3e0101,0xfb400108,0xee3f0101,
	0xfb400108,0xee400101,0xfb400108,0xee410101,0xfb400108,0xee420101,0xfb400108,0xee450101,
	0xfb400108,0xee460101,0xfb400108,0xee470101,0xfb400108,0xee480101,0xfb400108,0xee490101,
	0xfb400108,0xee4a0101,0xfb400108,0xee4b0101,0xfb400108,0xee4c0101,0xfb400108,0xee4f0101,
	0xfb400108,0xee500101,0xfb400108,0xee510101,0xfb400108,0xee520101,0xfb400108,0xee550101,
	0xfb400108,0xee570101,0xfb400108,0xee590101,0xfb400108,0xee5a0101,0xfb400108,0xee5c0101,
	0xfb400108,0xee5d0101,0xfb400108,0xee5e0101,0xfb400108,0xee600101,0xfb400108,0xee610101,
	0xfb400108,0xee620101,0xfb400108,0xee630101,0xfb400108,0xee640101,0xfb400108,0xee650101,
	0xfb400108,0xee660101,0xfb400108,0xee670101,0xfb400108,0xee680101,0xfb400108,0xee690101,
	0xfb400108,0xee6a0101,0xfb400108,0xee6c0101,0xfb400108,0xee6d0101,0xfb400108,0xee6f0101,
	0xfb400108,0xee700101,0xfb400108,0xee710101,0xfb400108,0xee720101,0xfb400108,0xee730101,
	0xfb400108,0xee740101,0xfb400108,0xee750101,0xfb400108,0xee760101,0xfb400108,0xee770101,
	0xfb400108,0xee780101,0xfb400108,0xee790101,0xfb400108,0xee7a0101,0xfb400108,0xee7b0101,
	0xfb400108,0xee7c0101,0xfb400108,0xee7d0101,0xfb400108,0xee800101,0xfb400108,0xee810101,
	0xfb400108,0xee820101,0xfb400108,0xee840101,0xfb400108,0xee870101,0xfb400108,0xee880101,
	0xfb400108,0xee8a0101,0xfb400108,0xee8b0101,0xfb400108,0xee8c0101,0xfb400108,0xee8d0101,
	0xfb400108,0xee8e0101,0xfb400108,0xee910101,0xfb400108,0xee920101,0xfb400108,0xee930101,
	0xfb400108,0xee940101,0xfb400108,0xee950101,0xfb400108,0xee960101,0xfb400108,0xee970101,
	0xfb400108,0xee990101,0xfb400108,0xee9a0101,0xfb400108,0xee9b0101,0xfb400108,0xee9d0101,
	0xfb400108,0xee9e0101,0xfb400108,0xeea00101,0xfb400108,0xeea10101,0xfb400108,0xeea30101,
	0xfb400108,0xeea40101,0xfb400108,0xeea60101,0xfb400108,0xeea80101,0xfb400108,0xeea90101,
	0xfb400108,0xeeab0101,0xfb400108,0xeeac0101,0xfb400108,0xeead0101,0xfb400108,0xeeae0101,
	0xfb400108,0xeeb00101,0xfb400108,0xeeb30101,0xfb400108,0xeeb50101,0xfb400108,0xeeb80101,
	0xfb400108,0xeeb90101,0xfb400108,0xeebc0101,0xfb400108,0xeebe0101,0xfb400108,0xeebf0101,
	0xfb400108,0xeec00101,0xfb400108,0xeec30101,0xfb400108,0xeec40101,0xfb400108,0xeec50101,
	0xfb400108,0xeec60101,0xfb400108,0xeec80101,0xfb400108,0xeec90101,0xfb400108,0xeeca0101,
	0xfb400108,0xeecc0101,0xfb400108,0xeecd0101,0xfb400108,0xeece0101,0xfb400108,0xeed00101,
	0xfb400108,0xeed20101,0xfb400108,0xeed60101,0xfb400108,0xeed80101,0xfb400108,0xeed90101,
	0xfb400108,0xeedb0101,0xfb400108,0xeedc0101,0xfb400108,0xeedd0101,0xfb400108,0xeee30101,
	0xfb400108,0xeee70101,0xfb400108,0xeeea0101,0xfb400108,0xeeeb0101,0xfb400108,0xeeec0101,
	0xfb400108,0xeeed0101,0xfb400108,0xeeee0101,0xfb400108,0xeeef0101,0xfb400108,0xeef00101,
	0xfb400108,0xeef10101,0xfb400108,0xeef20101,0xfb400108,0xeef30101,0xfb400108,0xeef50101,
	0xfb400108,0xeef60101,0xfb400108,0xeef70101,0xfb400108,0xeef80101,0xfb400108,0xeefa0101,
	0xfb400108,0xeefb0101,0xfb400108,0xeefc0101,0xfb400108,0xeefd0101,0xfb400108,0xeefe0101,
	0xfb400108,0xeeff0101,0xfb400108,0xef000101,0xfb400108,0xef010101,0xfb400108,0xef030101,
	0xfb400108,0xef040101,0xfb400108,0xef050101,0xfb400108,0xef070101,0xfb400108,0xef080101,
	0xfb400108,0xef0a0101,0xfb400108,0xef0b0101,0xfb400108,0xef0c0101,0xfb400108,0xef0d0101,
	0xfb400108,0xef0e0101,0xfb400108,0xef100101,0xfb400108,0xef110101,0xfb400108,0xef120101,
	0xfb400108,0xef160101,0xfb400108,0xef170101,0xfb400108,0xef180101,0xfb400108,0xef190101,
	0xfb400108,0xef1a0101,0xfb400108,0xef1b0101,0xfb400108,0xef1c0101,0xfb400108,0xef1d0101,
	0xfb400108,0xef1e0101,0xfb400108,0xef1f0101,0xfb400108,0xef210101,0xfb400108,0xef220101,
	0xfb400108,0xef230101,0xfb400108,0xef250101,0xfb400108,0xef260101,0xfb400108,0xef270101,
	0xfb400108,0xef280101,0xfb400108,0xef2c0101,0xfb400108,0xef2e0101,0xfb400108,0xef300101,
	0xfb400108,0xef320101,0xfb400108,0xef340101,0xfb400108,0xef350101,0xfb400108,0xef370101,
	0xfb400108,0xef380101,0xfb400108,0xef390101,0xfb400108,0xef3a0101,0xfb400108,0xef3b0101,
	0xfb400108,0xef3c0101,0xfb400108,0xef3d0101,0xfb400108,0xef3f0101,0xfb400108,0xef400101,
	0xfb400108,0xef410101,0xfb400108,0xef420101,0xfb400108,0xef430101,0xfb400108,0xef440101,
	0xfb400108,0xef450101,0xfb400108,0xef480101,0xfb400108,0xef490101,0xfb400108,0xef4a0101,
	0xfb400108,0xef4c0101,0xfb400108,0xef4e0101,0xfb400108,0xef4f0101,0xfb400108,0xef500101,
	0xfb400108,0xef510101,0xfb400108,0xef520101,0xfb400108,0xef530101,0xfb400108,0xef540101,
	0xfb400108,0xef550101,0xfb400108,0xef560101,0xfb400108,0xef570101,0xfb400108,0xef590101,
	0xfb400108,0xef5a0101,0xfb400108,0xef5b0101,0xfb400108,0xef5d0101,0xfb400108,0xef5f0101,
	0xfb400108,0xef600101,0xfb400108,0xef610101,0xfb400108,0xef630101,0xfb400108,0xef640101,
	0xfb400108,0xef650101,0xfb400108,0xef670101,0xfb400108,0xef680101,0xfb400108,0xef690101,
	0xfb400108,0xef6a0101,0xfb400108,0xef6b0101,0xfb400108,0xef6c0101,0xfb400108,0xef6f0101,
	0xfb400108,0xef700101,0xfb400108,0xef710101,0xfb400108,0xef730101,0xfb400108,0xef750101,
	0xfb400108,0xef760101,0xfb400108,0xef770101,0xfb400108,0xef790101,0xfb400108,0xef7b0101,
	0xfb400108,0xef7d0101,0xfb400108,0xef7e0101,0xfb400108,0xef7f0101,0xfb400108,0xef800101,
	0xfb400108,0xef810101,0xfb400108,0xef820101,0xfb400108,0xef830101,0xfb400108,0xef850101,
	0xfb400108,0xef860101,0xfb400108,0xef870101,0xfb400108,0xef8a0101,0xfb400108,0xef8b0101,
	0xfb400108,0xef8f0101,0xfb400108,0xef900101,0xfb400108,0xef910101,0xfb400108,0xef920101,
	0xfb400108,0xef930101,0xfb400108,0xef940101,0xfb400108,0xef950101,0xfb400108,0xef960101,
	0xfb400108,0xef970101,0xfb400108,0xef980101,0xfb400108,0xef990101,0xfb400108,0xef9a0101,
	0xfb400108,0xef9b0101,0xfb400108,0xef9d0101,0xfb400108,0xef9e0101,0xfb400108,0xef9f0101,
	0xfb400108,0xefa00101,0xfb400108,0xefa20101,0xfb400108,0xefa30101,0xfb400108,0xefa40101,
	0xfb400108,0xefa50101,0xfb400108,0xefa60101,0xfb400108,0xefa80101,0xfb400108,0xefa90101,
	0xfb400108,0xefaa0101,0xfb400108,0xefab0101,0xfb400108,0xefac0101,0xfb400108,0xefad0101,
	0xfb400108,0xefae0101,0xfb400108,0xefaf0101,0xfb400108,0xefb00101,0xfb400108,0xefb10101,
	0xfb400108,0xefb20101,0xfb400108,0xefb40101,0xfb400108,0xefb50101,0xfb400108,0xefb70101,
	0xfb400108,0xefb80101,0xfb400108,0xefba0101,0xfb400108,0xefbb0101,0xfb400108,0xefbc0101,
	0xfb400108,0xefbd0101,0xfb400108,0xefbe0101,0xfb400108,0xefbf0101,0xfb400108,0xefc10101,
	0xfb400108,0xefc30101,0xfb400108,0xefc40101,0xfb400108,0xefc50101,0xfb400108,0xefc60101,
	0xfb400108,0xefc70101,0xfb400108,0xefc80101,0xfb400108,0xefca0101,0xfb400108,0xefcb0101,
	0xfb400108,0xefcc0101,0xfb400108,0xefcd0101,0xfb400108,0xefce0101,0xfb400108,0xefcf0101,
	0xfb400108,0xefd00101,0xfb400108,0xefd30101,0xfb400108,0xefd40101,0xfb400108,0xefd50101,
	0xfb400108,0xefd60101,0xfb400108,0xefd70101,0xfb400108,0xefd80101,0xfb400108,0xefd90101,
	0xfb400108,0xefda0101,0xfb400108,0xefdb0101,0xfb400108,0xefdc0101,0xfb400108,0xefdd0101,
	0xfb400108,0xefdf0101,0xfb400108,0xefe20101,0xfb400108,0xefe30101,0xfb400108,0xefe40101,
	0xfb400108,0xefe50101,0xfb400108,0xefe60101,0xfb400108,0xefe70101,0xfb400108,0xefe80101,
	0xfb400108,0xefe90101,0xfb400108,0xefea0101,0xfb400108,0xefeb0101,0xfb400108,0xefec0101,
	0xfb400108,0xefed0101,0xfb400108,0xeff00101,0xfb400108,0xeff10101,0xfb400108,0xeff20101,
	0xfb400108,0xeff30101,0xfb400108,0xeff40101,0xfb400108,0xeff50101,0xfb400108,0xeff60101,
	0xfb400108,0xeff70101,0xfb400108,0xeff80101,0xfb400108,0xeff90101,0xfb400108,0xeffa0101,
	0xfb400108,0xeffb0101,0xfb400108,0xeffc0101,0xfb400108,0xeffd0101,0xfb400108,0xeffe0101,
	0xfb400108,0xefff0101,0xfb400108,0xf0000101,0xfb400108,0xf0010101,0xfb400108,0xf0020101,
	0xfb400108,0xf0030101,0xfb400108,0xf0040101,0xfb400108,0xf0050101,0xfb400108,0xf0060101,
	0xfb400108,0xf0070101,0xfb400108,0xf0080101,0xfb400108,0xf0090101,0xfb400108,0xf00a0101,
	0xfb400108,0xf00b0101,0xfb400108,0xf00c0101,0xfb400108,0xf00d0101,0xfb400108,0xf00e0101,
	0xfb400108,0xf00f0101,0xfb400108,0xf0100101,0xfb400108,0xf0120101,0xfb400108,0xf0130101,
	0xfb400108,0xf0140101,0xfb400108,0xf0150101,0xfb400108,0xf0160101,0xfb400108,0xf0170101,
	0xfb400108,0xf0180101,0xfb400108,0xf0190101,0xfb400108,0xf01c0101,0xfb400108,0xf01d0101,
	0xfb400108,0xf01e0101,0xfb400108,0xf01f0101,0xfb400108,0xf0200101,0xfb400108,0xf0210101,
	0xfb400108,0xf0220101,0xfb400108,0xf0240101,0xfb400108,0xf0250101,0xfb400108,0xf0260101,
	0xfb400108,0xf0270101,0xfb400108,0xf0280101,0xfb400108,0xf0290101,0xfb400108,0xf02a0101,
	0xfb400108,0xf02b0101,0xfb400108,0xf02c0101,0xfb400108,0xf02d0101,0xfb400108,0xf02e0101,
	0xfb400108,0xf02f0101,0xfb400108,0xf0300101,0xfb400108,0xf0310101,0xfb400108,0xf0320101,
	0xfb400108,0xf0330101,0xfb400108,0xf0340101,0xfb400108,0xf0360101,0xfb400108,0xf0370101,
	0xfb400108,0xf0380101,0xfb400108,0xf03a0101,0xfb400108,0xf03b0101,0xfb400108,0xf03c0101,
	0xfb400108,0xf03d0101,0xfb400108,0xf03e0101,0xfb400108,0xf03f0101,0xfb400108,0xf0400101,
	0xfb400108,0xf0410101,0xfb400108,0xf0420101,0xfb400108,0xf0430101,0xfb400108,0xf0440101,
	0xfb400108,0xf0450101,0xfb400108,0xf0460101,0xfb400108,0xf0470101,0xfb400108,0xf0480101,
	0xfb400108,0xf0490101,0xfb400108,0xf04a0101,0xfb400108,0xf04b0101,0xfb400108,0xf04d0101,
	0xfb400108,0xf04e0101,0xfb400108,0xf0500101,0xfb400108,0xf0510101,0xfb400108,0xf0520101,
	0xfb400108,0xf0530101,0xfb400108,0xf0540101,0xfb400108,0xf0550101,0xfb400108,0xf0560101,
	0xfb400108,0xf0570101,0xfb400108,0xf0580101,0xfb400108,0xf0590101,0xfb400108,0xf05a0101,
	0xfb400108,0xf05b0101,0xfb400108,0xf05c0101,0xfb400108,0xf05d0101,0xfb400108,0xf05f0101,
	0xfb400108,0xf0600101,0xfb400108,0xf0610101,0xfb400108,0xf0620101,0xfb400108,0xf0630101,
	0xfb400108,0xf0640101,0xfb400108,0xf0650101,0xfb400108,0xf0660101,0xfb400108,0xf0670101,
	0xfb400108,0xf0680101,0xfb400108,0xf0690101,0xfb400108,0xf06a0101,0xfb400108,0xf06e0101,
	0xfb400108,0xf0710101,0xfb400108,0xf0720101,0xfb400108,0xf0730101,0xfb400108,0xf0740101,
	0xfb400108,0xf0770101,0xfb400108,0xf0790101,0xfb400108,0xf07a0101,0xfb400108,0xf07b0101,
	0xfb400108,0xf07d0101,0xfb400108,0xf0810101,0xfb400108,0xf0820101,0xfb400108,0xf0830101,
	0xfb400108,0xf0840101,0xfb400108,0xf0860101,0xfb400108,0xf0870101,0xfb400108,0xf0880101,
	0xfb400108,0xf08b0101,0xfb400108,0xf08c0101,0xfb400108,0xf08d0101,0xfb400108,0xf08f0101,
	0xfb400108,0xf0900101,0xfb400108,0xf0910101,0xfb400108,0xf0930101,0xfb400108,0xf0970101,
	0xfb400108,0xf0980101,0xfb400108,0xf09a0101,0xfb400108,0xf09b0101,0xfb400108,0xf09e0101,
	0xfb400108,0xf09f0101,0xfb400108,0xf0a00101,0xfb400108,0xf0a10101,0xfb400108,0xf0a20101,
	0xfb400108,0xf0a30101,0xfb400108,0xf0a40101,0xfb400108,0xf0a50101,0xfb400108,0xf0a60101,
	0xfb400108,0xf0a70101,0xfb400108,0xf0a80101,0xfb400108,0xf0a90101,0xfb400108,0xf0aa0101,
	0xfb400108,0xf0b00101,0xfb400108,0xf0b20101,0xfb400108,0xf0b40101,0xfb400108,0xf0b50101,
	0xfb400108,0xf0b60101,0xfb400108,0xf0ba0101,0xfb400108,0xf0be0101,0xfb400108,0xf0bf0101,
	0xfb400108,0xf0c40101,0xfb400108,0xf0c50101,0xfb400108,0xf0c60101,0xfb400108,0xf0c70101,
	0xfb400108,0xf0c90101,0xfb400108,0xf0cb0101,0xfb400108,0xf0cc0101,0xfb400108,0xf0cd0101,
	0xfb400108,0xf0ce0101,0xfb400108,0xf0cf0101,0xfb400108,0xf0d00101,0xfb400108,0xf0d10101,
	0xfb400108,0xf0d20101,0xfb400108,0xf0d30101,0xfb400108,0xf0d40101,0xfb400108,0xf0d50101,
	0xfb400108,0xf0d60101,0xfb400108,0xf0d70101,0xfb400108,0xf0da0101,0xfb400108,0xf0dc0101,
	0xfb400108,0xf0dd0101,0xfb400108,0xf0de0101,0xfb400108,0xf0e00101,0xfb400108,0xf0e10101,
	0xfb400108,0xf0e20101,0xfb400108,0xf0e30101,0xfb400108,0xf0e50101,0xfb400108,0xf0ea0101,
	0xfb400108,0xf0ee0101,0xfb400108,0xf0f00101,0xfb400108,0xf0f10101,0xfb400108,0xf0f20101,
	0xfb400108,0xf0f30101,0xfb400108,0xf0f40101,0xfb400108,0xf0f50101,0xfb400108,0xf0f60101,
	0xfb400108,0xf0f80101,0xfb400108,0xf0fa0101,0xfb400108,0xf0fb0101,0xfb400108,0xf0fc0101,
	0xfb400108,0xf0fe0101,0xfb400108,0xf0ff0101,0xfb400108,0xf1000101,0xfb400108,0xf1010101,
	0xfb400108,0xf1020101,0xfb400108,0xf1030101,0xfb400108,0xf1040101,0xfb400108,0xf1050101,
	0xfb400108,0xf1060101,0xfb400108,0xf1070101,0xfb400108,0xf1080101,0xfb400108,0xf10b0101,
	0xfb400108,0xf10c0101,0xfb400108,0xf10d0101,0xfb400108,0xf10e0101,0xfb400108,0xf10f0101,
	0xfb400108,0xf1110101,0xfb400108,0xf1120101,0xfb400108,0xf1140101,0xfb400108,0xf1170101,
	0xfb400108,0xf11b0101,0xfb400108,0xf11c0101,0xfb400108,0xf11d0101,0xfb400108,0xf11e0101,
	0xfb400108,0xf11f0101,0xfb400108,0xf1200101,0xfb400108,0xf1210101,0xfb400108,0xf1220101,
	0xfb400108,0xf1230101,0xfb400108,0xf1240101,0xfb400108,0xf1250101,0xfb400108,0xf1270101,
	0xfb400108,0xf1280101,0xfb400108,0xf1290101,0xfb400108,0xf12a0101,0xfb400108,0xf12b0101,
	0xfb400108,0xf12c0101,0xfb400108,0xf12d0101,0xfb400108,0xf12e0101,0xfb400108,0xf1320101,
	0xfb400108,0xf1330101,0xfb400108,0xf1340101,0xfb400108,0xf1350101,0xfb400108,0xf1370101,
	0xfb400108,0xf1380101,0xfb400108,0xf1390101,0xfb400108,0xf13a0101,0xfb400108,0xf13b0101,
	0xfb400108,0xf13c0101,0xfb400108,0xf13d0101,0xfb400108,0xf13e0101,0xfb400108,0xf13f0101,
	0xfb400108,0xf1400101,0xfb400108,0xf1410101,0xfb400108,0xf1420101,0xfb400108,0xf1430101,
	0xfb400108,0xf1440101,0xfb400108,0xf1460101,0xfb400108,0xf1470101,0xfb400108,0xf1480101,
	0xfb400108,0xf1490101,0xfb400108,0xf14b0101,0xfb400108,0xf14d0101,0xfb400108,0xf14f0101,
	0xfb400108,0xf1500101,0xfb400108,0xf1510101,0xfb400108,0xf1520101,0xfb400108,0xf1530101,
	0xfb400108,0xf1540101,0xfb400108,0xf1550101,0xfb400108,0xf1560101,0xfb400108,0xf1570101,
	0xfb400108,0xf1580101,0xfb400108,0xf1590101,0xfb400108,0xf15a0101,0xfb400108,0xf15b0101,
	0xfb400108,0xf15d0101,0xfb400108,0xf15f0101,0xfb400108,0xf1600101,0xfb400108,0xf1610101,
	0xfb400108,0xf1620101,0xfb400108,0xf1630101,0xfb400108,0xf1650101,0xfb400108,0xf1690101,
	0xfb400108,0xf16a0101,0xfb400108,0xf16b0101,0xfb400108,0xf16c0101,0xfb400108,0xf16d0101,
	0xfb400108,0xf16f0101,0xfb400108,0xf1700101,0xfb400108,0xf1710101,0xfb400108,0xf1740101,
	0xfb400108,0xf1750101,0xfb400108,0xf1760101,0xfb400108,0xf1770101,0xfb400108,0xf1790101,
	0xfb400108,0xf17b0101,0xfb400108,0xf17c0101,0xfb400108,0xf17e0101,0xfb400108,0xf17f0101,
	0xfb400108,0xf1800101,0xfb400108,0xf1810101,0xfb400108,0xf1820101,0xfb400108,0xf1830101,
	0xfb400108,0xf1850101,0xfb400108,0xf1860101,0xfb400108,0xf1870101,0xfb400108,0xf1880101,
	0xfb400108,0xf1890101,0xfb400108,0xf18b0101,0xfb400108,0xf18c0101,0xfb400108,0xf18d0101,
	0xfb400108,0xf18e0101,0xfb400108,0xf1900101,0xfb400108,0xf1910101,0xfb400108,0xf1920101,
	0xfb400108,0xf1930101,0xfb400108,0xf1950101,0xfb400108,0xf1960101,0xfb400108,0xf1970101,
	0xfb400108,0xf19a0101,0xfb400108,0xf19b0101,0xfb400108,0xf19c0101,0xfb400108,0xf19d0101,
	0xfb400108,0xf19e0101,0xfb400108,0xf1a10101,0xfb400108,0xf1a20101,0xfb400108,0xf1a30101,
	0xfb400108,0xf1a40101,0xfb400108,0xf1a50101,0xfb400108,0xf1a60101,0xfb400108,0xf1a70101,
	0xfb400108,0xf1a90101,0xfb400108,0xf1aa0101,0xfb400108,0xf1ab0101,0xfb400108,0xf1ad0101,
	0xfb400108,0xf1ae0101,0xfb400108,0xf1af0101,0xfb400108,0xf1b00101,0xfb400108,0xf1b10101,
	0xfb400108,0xf1b20101,0xfb400108,0xf1b40101,0xfb400108,0xf1b60101,0xfb400108,0xf1b70101,
	0xfb400108,0xf1b80101,0xfb400108,0xf1ba0101,0xfb400108,0xf1bb0101,0xfb400108,0xf1bc0101,
	0xfb400108,0xf1bd0101,0xfb400108,0xf1be0101,0xfb400108,0xf1bf0101,0xfb400108,0xf1c00101,
	0xfb400108,0xf1c10101,0xfb400108,0xf1c20101,0xfb400108,0xf1c40101,0xfb400108,0xf1c50101,
	0xfb400108,0xf1c60101,0xfb400108,0xf1c70101,0xfb400108,0xf1c80101,0xfb400108,0xf1c90101,
	0xfb400108,0xf1ca0101,0xfb400108,0xf1cb0101,0xfb400108,0xf1cc0101,0xfb400108,0xf1cd0101,
	0xfb400108,0xf1cf0101,0xfb400108,0xf1d00101,0xfb400108,0xf1d10101,0xfb400108,0xf1d20101,
	0xfb400108,0xf1d30101,0xfb400108,0xf1d60101,0xfb400108,0xf1d70101,0xfb400108,0xf1d80101,
	0xfb400108,0xf1d90101,0xfb400108,0xf1da0101,0xfb400108,0xf1db0101,0xfb400108,0xf1dc0101,
	0xfb400108,0xf1dd0101,0xfb400108,0xf1de0101,0xfb400108,0xf1df0101,0xfb400108,0xf1e10101,
	0xfb400108,0xf1e20101,0xfb400108,0xf1e30101,0xfb400108,0xf1e40101,0xfb400108,0xf1e60101,
	0xfb400108,0xf1e80101,0xfb400108,0xf1e90101,0xfb400108,0xf1ea0101,0xfb400108,0xf1eb0101,
	0xfb400108,0xf1ec0101,0xfb400108,0xf1ed0101,0xfb400108,0xf1ef0101,0xfb400108,0xf1f00101,
	0xfb400108,0xf1f10101,0xfb400108,0xf1f20101,0xfb400108,0xf1f30101,0xfb400108,0xf1f40101,
	0xfb400108,0xf1f50101,0xfb400108,0xf1f60101,0xfb400108,0xf1f70101,0xfb400108,0xf1f80101,
	0xfb400108,0xf1fa0101,0xfb400108,0xf1fb0101,0xfb400108,0xf1fc0101,0xfb400108,0xf1fd0101,
	0xfb400108,0xf1fe0101,0xfb400108,0xf1ff0101,0xfb400108,0xf2000101,0xfb400108,0xf2010101,
	0xfb400108,0xf2020101,0xfb400108,0xf2030101,0xfb400108,0xf2040101,0xfb400108,0xf2050101,
	0xfb400108,0xf2070101,0xfb400108,0xf2080101,0xfb400108,0xf2090101,0xfb400108,0xf20a0101,
	0xfb400108,0xf20b0101,0xfb400108,0xf20c0101,0xfb400108,0xf20d0101,0xfb400108,0xf20e0101,
	0xfb400108,0xf20f0101,0xfb400108,0xf2100101,0xfb400108,0xf2110101,0xfb400108,0xf2120101,
	0xfb400108,0xf2130101,0xfb400108,0xf2140101,0xfb400108,0xf2150101,0xfb400108,0xf2160101,
	0xfb400108,0xf2170101,0xfb400108,0xf2180101,0xfb400108,0xf2190101,0xfb400108,0xf21a0101,
	0xfb400108,0xf21b0101,0xfb400108,0xf21c0101,0xfb400108,0xf21e0101,0xfb400108,0xf21f0101,
	0xfb400108,0xf2200101,0xfb400108,0xf2210101,0xfb400108,0xf2220101,0xfb400108,0xf2230101,
	0xfb400108,0xf2240101,0xfb400108,0xf2250101,0xfb400108,0xf2260101,0xfb400108,0xf2270101,
	0xfb400108,0xf2290101,0xfb400108,0xf22b0101,0xfb400108,0xf22d0101,0xfb400108,0xf22e0101,
	0xfb400108,0xf22f0101,0xfb400108,0xf2320101,0xfb400108,0xf2330101,0xfb400108,0xf2340101,
	0xfb400108,0xf23a0101,0xfb400108,0xf23c0101,0xfb400108,0xf23e0101,0xfb400108,0xf2400101,
	0xfb400108,0xf2410101,0xfb400108,0xf2420101,0xfb400108,0xf2430101,0xfb400108,0xf2440101,
	0xfb400108,0xf2450101,0xfb400108,0xf2460101,0xfb400108,0xf2490101,0xfb400108,0xf24a0101,
	0xfb400108,0xf24b0101,0xfb400108,0xf24e0101,0xfb400108,0xf24f0101,0xfb400108,0xf2500101,
	0xfb400108,0xf2510101,0xfb400108,0xf2530101,0xfb400108,0xf2540101,0xfb400108,0xf2550101,
	0xfb400108,0xf2570101,0xfb400108,0xf2580101,0xfb400108,0xf25a0101,0xfb400108,0xf25c0101,
	0xfb400108,0xf25e0101,0xfb400108,0xf2600101,0xfb400108,0xf2630101,0xfb400108,0xf2640101,
	0xfb400108,0xf2650101,0xfb400108,0xf2680101,0xfb400108,0xf26a0101,0xfb400108,0xf26b0101,
	0xfb400108,0xf26c0101,0xfb400108,0xf26d0101,0xfb400108,0xf2700101,0xfb400108,0xf2710101,
	0xfb400108,0xf2730101,0xfb400108,0xf2740101,0xfb400108,0xf2760101,0xfb400108,0xf2770101,
	0xfb400108,0xf2780101,0xfb400108,0xf27b0101,0xfb400108,0xf27c0101,0xfb400108,0xf27d0101,
	0xfb400108,0xf2820101,0xfb400108,0xf2830101,0xfb400108,0xf2850101,0xfb400108,0xf2860101,
	0xfb400108,0xf2870101,0xfb400108,0xf2880101,0xfb400108,0xf2890101,0xfb400108,0xf28c0101,
	0xfb400108,0xf28e0101,0xfb400108,0xf2900101,0xfb400108,0xf2910101,0xfb400108,0xf2930101,
	0xfb400108,0xf2940101,0xfb400108,0xf2950101,0xfb400108,0xf2960101,0xfb400108,0xf2970101,
	0xfb400108,0xf2980101,0xfb400108,0xf2990101,0xfb400108,0xf29a0101,0xfb400108,0xf29b0101,
	0xfb400108,0xf29c0101,0xfb400108,0xf29d0101,0xfb400108,0xf29e0101,0xfb400108,0xf2a00101,
	0xfb400108,0xf2a10101,0xfb400108,0xf2a20101,0xfb400108,0xf2a30101,0xfb400108,0xf2a40101,
	0xfb400108,0xf2a50101,0xfb400108,0xf2a60101,0xfb400108,0xf2a70101,0xfb400108,0xf2a80101,
	0xfb400108,0xf2a90101,0xfb400108,0xf2aa0101,0xfb400108,0xf2ab0101,0xfb400108,0xf2ae0101,
	0xfb400108,0xf2b10101,0xfb400108,0xf2b20101,0xfb400108,0xf2b30101,0xfb400108,0xf2b50101,
	0xfb400108,0xf2ba0101,0xfb400108,0xf2bb0101,0xfb400108,0xf2bc0101,0xfb400108,0xf2bd0101,
	0xfb400108,0xf2be0101,0xfb400108,0xf2bf0101,0xfb400108,0xf2c00101,0xfb400108,0xf2c50101,
	0xfb400108,0xf2c60101,0xfb400108,0xf2c70101,0xfb400108,0xf2c90101,0xfb400108,0xf2ca0101,
	0xfb400108,0xf2cb0101,0xfb400108,0xf2cc0101,0xfb400108,0xf2cf0101,0xfb400108,0xf2d10101,
	0xfb400108,0xf2d30101,0xfb400108,0xf2d40101,0xfb400108,0xf2d50101,0xfb400108,0xf2d60101,
	0xfb400108,0xf2d80101,0xfb400108,0xf2da0101,0xfb400108,0xf2db0101,0xfb400108,0xf2dc0101,
	0xfb400108,0xf2dd0101,0xfb400108,0xf2df0101,0xfb400108,0xf2e20101,0xfb400108,0xf2e30101,
	0xfb400108,0xf2e40101,0xfb400108,0xf2e50101,0xfb400108,0xf2e60101,0xfb400108,0xf2e70101,
	0xfb400108,0xf2ea0101,0xfb400108,0xf2eb0101,0xfb400108,0xf2f50101,0xfb400108,0xf2f60101,
	0xfb400108,0xf2f90101,0xfb400108,0xf2fd0101,0xfb400108,0xf2fe0101,0xfb400108,0xf2ff0101,
	0xfb400108,0xf3000101,0xfb400108,0xf3020101,0xfb400108,0xf3040101,0xfb400108,0xf3050101,
	0xfb400108,0xf3060101,0xfb400108,0xf3070101,0xfb400108,0xf3080101,0xfb400108,0xf3090101,
	0xfb400108,0xf30b0101,0xfb400108,0xf30c0101,0xfb400108,0xf30d0101,0xfb400108,0xf30f0101,
	0xfb400108,0xf3100101,0xfb400108,0xf3110101,0xfb400108,0xf3120101,0xfb400108,0xf3140101,
	0xfb400108,0xf3180101,0xfb400108,0xf3190101,0xfb400108,0xf31a0101,0xfb400108,0xf31f0101,
	0xfb400108,0xf3200101,0xfb400108,0xf3230101,0xfb400108,0xf3240101,0xfb400108,0xf3260101,
	0xfb400108,0xf3270101,0xfb400108,0xf3280101,0xfb400108,0xf32d0101,0xfb400108,0xf32f0101,
	0xfb400108,0xf3300101,0xfb400108,0xf3320101,0xfb400108,0xf3330101,0xfb400108,0xf3350101,
	0xfb400108,0xf3360101,0xfb400108,0xf33a0101,0xfb400108,0xf33b0101,0xfb400108,0xf33c0101,
	0xfb400108,0xf33d0101,0xfb400108,0xf3400101,0xfb400108,0xf3410101,0xfb400108,0xf3420101,
	0xfb400108,0xf3430101,0xfb400108,0xf3440101,0xfb400108,0xf3450101,0xfb400108,0xf3460101,
	0xfb400108,0xf3470101,0xfb400108,0xf3480101,0xfb400108,0xf3490101,0xfb400108,0xf34a0101,
	0xfb400108,0xf34b0101,0xfb400108,0xf34c0101,0xfb400108,0xf34e0101,0xfb400108,0xf34f0101,
	0xfb400108,0xf3510101,0xfb400108,0xf3530101,0xfb400108,0xf3540101,0xfb400108,0xf3550101,
	0xfb400108,0xf3560101,0xfb400108,0xf3580101,0xfb400108,0xf3590101,0xfb400108,0xf35a0101,
	0xfb400108,0xf35b0101,0xfb400108,0xf35c0101,0xfb400108,0xf35d0101,0xfb400108,0xf35e0101,
	0xfb400108,0xf35f0101,0xfb400108,0xf3610101,0xfb400108,0xf3620101,0xfb400108,0xf3630101,
	0xfb400108,0xf3640101,0xfb400108,0xf3650101,0xfb400108,0xf3660101,0xfb400108,0xf3670101,
	0xfb400108,0xf3680101,0xfb400108,0xf3690101,0xfb400108,0xf36a0101,0xfb400108,0xf36b0101,
	0xfb400108,0xf36e0101,0xfb400108,0xf3700101,0xfb400108,0xf3710101,0xfb400108,0xf3720101,
	0xfb400108,0xf3730101,0xfb400108,0xf3740101,0xfb400108,0xf3750101,0xfb400108,0xf3760101,
	0xfb400108,0xf3770101,0xfb400108,0xf3780101,0xfb400108,0xf3790101,0xfb400108,0xf37a0101,
	0xfb400108,0xf37b0101,0xfb400108,0xf37c0101,0xfb400108,0xf37d0101,0xfb400108,0xf37f0101,
	0xfb400108,0xf3800101,0xfb400108,0xf3810101,0xfb400108,0xf3820101,0xfb400108,0xf3830101,
	0xfb400108,0xf3850101,0xfb400108,0xf3860101,0xfb400108,0xf3880101,0xfb400108,0xf38a0101,
	0xfb400108,0xf38c0101,0xfb400108,0xf38d0101,0xfb400108,0xf38f0101,0xfb400108,0xf3900101,
	0xfb400108,0xf3920101,0xfb400108,0xf3930101,0xfb400108,0xf3940101,0xfb400108,0xf3950101,
	0xfb400108,0xf3970101,0xfb400108,0xf3980101,0xfb400108,0xf3990101,0xfb400108,0xf39a0101,
	0xfb400108,0xf39c0101,0xfb400108,0xf39d0101,0xfb400108,0xf39e0101,0xfb400108,0xf3a00101,
	0xfb400108,0xf3a10101,0xfb400108,0xf3a30101,0xfb400108,0xf3a40101,0xfb400108,0xf3a50101,
	0xfb400108,0xf3a60101,0xfb400108,0xf3a70101,0xfb400108,0xf3a80101,0xfb400108,0xf3aa0101,
	0xfb400108,0xf3ac0101,0xfb400108,0xf3ad0101,0xfb400108,0xf3b10101,0xfb400108,0xf3b40101,
	0xfb400108,0xf3b50101,0xfb400108,0xf3b60101,0xfb400108,0xf3b80101,0xfb400108,0xf3b90101,
	0xfb400108,0xf3bc0101,0xfb400108,0xf3bd0101,0xfb400108,0xf3be0101,0xfb400108,0xf3bf0101,
	0xfb400108,0xf3c10101,0xfb400108,0xf3c30101,0xfb400108,0xf3c40101,0xfb400108,0xf3c50101,
	0xfb400108,0xf3c60101,0xfb400108,0xf3c70101,0xfb400108,0xf3cb0101,0xfb400108,0xf3cc0101,
	0xfb400108,0xf3ce0101,0xfb400108,0xf3d20101,0xfb400108,0xf3d30101,0xfb400108,0xf3d40101,
	0xfb400108,0xf3d50101,0xfb400108,0xf3d60101,0xfb400108,0xf3d70101,0xfb400108,0xf3d80101,
	0xfb400108,0xf3da0101,0xfb400108,0xf3db0101,0xfb400108,0xf3dc0101,0xfb400108,0xf3dd0101,
	0xfb400108,0xf3df0101,0xfb400108,0xf3e10101,0xfb400108,0xf3e20101,0xfb400108,0xf3e30101,
	0xfb400108,0xf3e40101,0xfb400108,0xf3e60101,0xfb400108,0xf3e80101,0xfb400108,0xf3ea0101,
	0xfb400108,0xf3eb0101,0xfb400108,0xf3ec0101,0xfb400108,0xf3ee0101,0xfb400108,0xf3ef0101,
	0xfb400108,0xf3f00101,0xfb400108,0xf3f10101,0xfb400108,0xf3f30101,0xfb400108,0xf3f40101,
	0xfb400108,0xf3f50101,0xfb400108,0xf3f60101,0xfb400108,0xf3f70101,0xfb400108,0xf3f80101,
	0xfb400108,0xf3f90101,0xfb400108,0xf3fa0101,0xfb400108,0xf3fb0101,0xfb400108,0xf3fc0101,
	0xfb400108,0xf3fd0101,0xfb400108,0xf3fe0101,0xfb400108,0xf3ff0101,0xfb400108,0xf4000101,
	0xfb400108,0xf4010101,0xfb400108,0xf4020101,0xfb400108,0xf4040101,0xfb400108,0xf4070101,
	0xfb400108,0xf4080101,0xfb400108,0xf40b0101,0xfb400108,0xf40c0101,0xfb400108,0xf40d0101,
	0xfb400108,0xf40e0101,0xfb400108,0xf4110101,0xfb400108,0xf4120101,0xfb400108,0xf4130101,
	0xfb400108,0xf4140101,0xfb400108,0xf4150101,0xfb400108,0xf4160101,0xfb400108,0xf4170101,
	0xfb400108,0xf4180101,0xfb400108,0xf4190101,0xfb400108,0xf41c0101,0xfb400108,0xf41d0101,
	0xfb400108,0xf41e0101,0xfb400108,0xf41f0101,0xfb400108,0xf4200101,0xfb400108,0xf4210101,
	0xfb400108,0xf4230101,0xfb400108,0xf4240101,0xfb400108,0xf4270101,0xfb400108,0xf4290101,
	0xfb400108,0xf42b0101,0xfb400108,0xf42d0101,0xfb400108,0xf42f0101,0xfb400108,0xf4310101,
	0xfb400108,0xf4320101,0xfb400108,0xf4370101,0xfb400108,0xf4380101,0xfb400108,0xf4390101,
	0xfb400108,0xf43a0101,0xfb400108,0xf43b0101,0xfb400108,0xf43d0101,0xfb400108,0xf43e0101,
	0xfb400108,0xf43f0101,0xfb400108,0xf4400101,0xfb400108,0xf4420101,0xfb400108,0xf4430101,
	0xfb400108,0xf4440101,0xfb400108,0xf4450101,0xfb400108,0xf4460101,0xfb400108,0xf4470101,
	0xfb400108,0xf4480101,0xfb400108,0xf4490101,0xfb400108,0xf44a0101,0xfb400108,0xf44b0101,
	0xfb400108,0xf44c0101,0xfb400108,0xf44d0101,0xfb400108,0xf44e0101,0xfb400108,0xf44f0101,
	0xfb400108,0xf4500101,0xfb400108,0xf4510101,0xfb400108,0xf4520101,0xfb400108,0xf4530101,
	0xfb400108,0xf4540101,0xfb400108,0xf4560101,0xfb400108,0xf4580101,0xfb400108,0xf45d0101,
	0xfb400108,0xf4600101,0xfb400108,0xf4610101,0xfb400108,0xf4620101,0xfb400108,0xf4630101,
	0xfb400108,0xf4640101,0xfb400108,0xf4650101,0xfb400108,0xf4660101,0xfb400108,0xf4670101,
	0xfb400108,0xf4680101,0xfb400108,0xf4690101,0xfb400108,0xf46a0101,0xfb400108,0xf46b0101,
	0xfb400108,0xf46c0101,0xfb400108,0xf46e0101,0xfb400108,0xf46f0101,0xfb400108,0xf4710101,
	0xfb400108,0xf4720101,0xfb400108,0xf4730101,0xfb400108,0xf4740101,0xfb400108,0xf4750101,
	0xfb400108,0xf4780101,0xfb400108,0xf4790101,0xfb400108,0xf47a0101,0xfb400108,0xf47b0101,
	0xfb400108,0xf47c0101,0xfb400108,0xf47d0101,0xfb400108,0xf47f0101,0xfb400108,0xf4820101,
	0xfb400108,0xf4840101,0xfb400108,0xf4850101,0xfb400108,0xf4860101,0xfb400108,0xf4880101,
	0xfb400108,0xf4890101,0xfb400108,0xf48a0101,0xfb400108,0xf48c0101,0xfb400108,0xf48d0101,
	0xfb400108,0xf48f0101,0xfb400108,0xf4910101,0xfb400108,0xf4920101,0xfb400108,0xf4930101,
	0xfb400108,0xf4940101,0xfb400108,0xf4950101,0xfb400108,0xf4960101,0xfb400108,0xf4970101,
	0xfb400108,0xf4980101,0xfb400108,0xf4990101,0xfb400108,0xf49a0101,0xfb400108,0xf49b0101,
	0xfb400108,0xf49d0101,0xfb400108,0xf49f0101,0xfb400108,0xf4a00101,0xfb400108,0xf4a10101,
	0xfb400108,0xf4a20101,0xfb400108,0xf4a30101,0xfb400108,0xf4a40101,0xfb400108,0xf4a50101,
	0xfb400108,0xf4a60101,0xfb400108,0xf4aa0101,0xfb400108,0xf4ab0101,0xfb400108,0xf4ac0101,
	0xfb400108,0xf4ad0101,0xfb400108,0xf4ae0101,0xfb400108,0xf4af0101,0xfb400108,0xf4b00101,
	0xfb400108,0xf4b10101,0xfb400108,0xf4b20101,0xfb400108,0xf4b30101,0xfb400108,0xf4b40101,
	0xfb400108,0xf4b50101,0xfb400108,0xf4b60101,0xfb400108,0xf4b70101,0xfb400108,0xf4b80101,
	0xfb400108,0xf4b90101,0xfb400108,0xf4bb0101,0xfb400108,0xf4bc0101,0xfb400108,0xf4bd0101,
	0xfb400108,0xf4be0101,0xfb400108,0xf4bf0101,0xfb400108,0xf4c00101,0xfb400108,0xf4c10101,
	0xfb400108,0xf4c20101,0xfb400108,0xf4c30101,0xfb400108,0xf4c40101,0xfb400108,0xf4c50101,
	0xfb400108,0xf4c60101,0xfb400108,0xf4c70101,0xfb400108,0xf4c80101,0xfb400108,0xf4c90101,
	0xfb400108,0xf4ca0101,0xfb400108,0xf4cb0101,0xfb400108,0xf4cc0101,0xfb400108,0xf4cd0101,
	0xfb400108,0xf4ce0101,0xfb400108,0xf4cf0101,0xfb400108,0xf4d00101,0xfb400108,0xf4d10101,
	0xfb400108,0xf4d30101,0xfb400108,0xf4d40101,0xfb400108,0xf4d50101,0xfb400108,0xf4d60101,
	0xfb400108,0xf4d70101,0xfb400108,0xf4d80101,0xfb400108,0xf4d90101,0xfb400108,0xf4da0101,
	0xfb400108,0xf4db0101,0xfb400108,0xf4dd0101,0xfb400108,0xf4df0101,0xfb400108,0xf4e10101,
	0xfb400108,0xf4e50101,0xfb400108,0xf4e70101,0xfb400108,0xf4e80101,0xfb400108,0xf4e90101,
	0xfb400108,0xf4ea0101,0xfb400108,0xf4eb0101,0xfb400108,0xf4ec0101,0xfb400108,0xf4ed0101,
	0xfb400108,0xf4f00101,0xfb400108,0xf4f10101,0xfb400108,0xf4f20101,0xfb400108,0xf4f30101,
	0xfb400108,0xf4f50101,0xfb400108,0xf4f80101,0xfb400108,0xf4f90101,0xfb400108,0xf4fa0101,
	0xfb400108,0xf4fb0101,0xfb400108,0xf4fc0101,0xfb400108,0xf4fd0101,0xfb400108,0xf4fe0101,
	0xfb400108,0xf5000101,0xfb400108,0xf5010101,0xfb400108,0xf5020101,0xfb400108,0xf5030101,
	0xfb400108,0xf5050101,0xfb400108,0xf5060101,0xfb400108,0xf5070101,0xfb400108,0xf5080101,
	0xfb400108,0xf5090101,0xfb400108,0xf50a0101,0xfb400108,0xf50b0101,0xfb400108,0xf50c0101,
	0xfb400108,0xf50e0101,0xfb400108,0xf5100101,0xfb400108,0xf5120101,0xfb400108,0xf5140101,
	0xfb400108,0xf5150101,0xfb400108,0xf5160101,0xfb400108,0xf5170101,0xfb400108,0xf51b0101,
	0xfb400108,0xf51d0101,0xfb400108,0xf51e0101,0xfb400108,0xf5200101,0xfb400108,0xf5210101,
	0xfb400108,0xf5220101,0xfb400108,0xf5230101,0xfb400108,0xf5240101,0xfb400108,0xf5260101,
	0xfb400108,0xf5270101,0xfb400108,0xf52a0101,0xfb400108,0xf52e0101,0xfb400108,0xf5340101,
	0xfb400108,0xf5360101,0xfb400108,0xf5390101,0xfb400108,0xf53c0101,0xfb400108,0xf53d0101,
	0xfb400108,0xf53f0101,0xfb400108,0xf5410101,0xfb400108,0xf5420101,0xfb400108,0xf5430101,
	0xfb400108,0xf5440101,0xfb400108,0xf5460101,0xfb400108,0xf5470101,0xfb400108,0xf5490101,
	0xfb400108,0xf54a0101,0xfb400108,0xf54d0101,0xfb400108,0xf5500101,0xfb400108,0xf5510101,
	0xfb400108,0xf5520101,0xfb400108,0xf5530101,0xfb400108,0xf5550101,0xfb400108,0xf5560101,
	0xfb400108,0xf5570101,0xfb400108,0xf5580101,0xfb400108,0xf55d0101,0xfb400108,0xf55e0101,
	0xfb400108,0xf55f0101,0xfb400108,0xf5600101,0xfb400108,0xf5610101,0xfb400108,0xf5620101,
	0xfb400108,0xf5630101,0xfb400108,0xf5640101,0xfb400108,0xf5670101,0xfb400108,0xf5680101,
	0xfb400108,0xf5690101,0xfb400108,0xf56b0101,0xfb400108,0xf56c0101,0xfb400108,0xf56d0101,
	0xfb400108,0xf56e0101,0xfb400108,0xf56f0101,0xfb400108,0xf5700101,0xfb400108,0xf5710101,
	0xfb400108,0xf5730101,0xfb400108,0xf5750101,0xfb400108,0xf5760101,0xfb400108,0xf5770101,
	0xfb400108,0xf57a0101,0xfb400108,0xf57b0101,0xfb400108,0xf57c0101,0xfb400108,0xf57d0101,
	0xfb400108,0xf57e0101,0xfb400108,0xf5800101,0xfb400108,0xf5810101,0xfb400108,0xf5820101,
	0xfb400108,0xf5840101,0xfb400108,0xf5850101,0xfb400108,0xf5870101,0xfb400108,0xf5880101,
	0xfb400108,0xf5890101,0xfb400108,0xf58a0101,0xfb400108,0xf58c0101,0xfb400108,0xf58d0101,
	0xfb400108,0xf58e0101,0xfb400108,0xf5900101,0xfb400108,0xf5930101,0xfb400108,0xf5950101,
	0xfb400108,0xf5980101,0xfb400108,0xf59b0101,0xfb400108,0xf59c0101,0xfb400108,0xf59e0101,
	0xfb400108,0xf5a20101,0xfb400108,0xf5a60101,0xfb400108,0xf5a70101,0xfb400108,0xf5a80101,
	0xfb400108,0xf5a90101,0xfb400108,0xf5aa0101,0xfb400108,0xf5ad0101,0xfb400108,0xf5b60101,
	0xfb400108,0xf5b70101,0xfb400108,0xf5ba0101,0xfb400108,0xf5bb0101,0xfb400108,0xf5bf0101,
	0xfb400108,0xf5c00101,0xfb400108,0xf5c10101,0xfb400108,0xf5c60101,0xfb400108,0xf5cb0101,
	0xfb400108,0xf5cc0101,0xfb400108,0xf5ce0101,0xfb400108,0xf5cf0101,0xfb400108,0xf5d00101,
	0xfb400108,0xf5d10101,0xfb400108,0xf5d30101,0xfb400108,0xf5d70101,0xfb400108,0xf5d90101,
	0xfb400108,0xf5da0101,0xfb400108,0xf5dc0101,0xfb400108,0xf5dd0101,0xfb400108,0xf5df0101,
	0xfb400108,0xf5e00101,0xfb400108,0xf5e10101,0xfb400108,0xf5e50101,0xfb400108,0xf5e90101,
	0xfb400108,0xf5ec0101,0xfb400108,0xf5ed0101,0xfb400108,0xf5ee0101,0xfb400108,0xf5ef0101,
	0xfb400108,0xf5f20101,0xfb400108,0xf5f30101,0xfb400108,0xf5f50101,0xfb400108,0xf5f60101,
	0xfb400108,0xf5f70101,0xfb400108,0xf5f80101,0xfb400108,0xf5fa0101,0xfb400108,0xf5fb0101,
	0xfb400108,0xf5fd0101,0xfb400108,0xf5fe0101,0xfb400108,0xf6020101,0xfb400108,0xf6040101,
	0xfb400108,0xf6060101,0xfb400108,0xf6070101,0xfb400108,0xf6080101,0xfb400108,0xf6090101,
	0xfb400108,0xf60b0101,0xfb400108,0xf60d0101,0xfb400108,0xf60e0101,0xfb400108,0xf60f0101,
	0xfb400108,0xf6110101,0xfb400108,0xf6120101,0xfb400108,0xf6130101,0xfb400108,0xf6140101,
	0xfb400108,0xf6160101,0xfb400108,0xf61a0101,0xfb400108,0xf61c0101,0xfb400108,0xf61d0101,
	0xfb400108,0xf61e0101,0xfb400108,0xf6210101,0xfb400108,0xf6230101,0xfb400108,0xf6270101,
	0xfb400108,0xf6280101,0xfb400108,0xf62c0101,0xfb400108,0xf62e0101,0xfb400108,0xf62f0101,
	0xfb400108,0xf6310101,0xfb400108,0xf6320101,0xfb400108,0xf6360101,0xfb400108,0xf6370101,
	0xfb400108,0xf6390101,0xfb400108,0xf63a0101,0xfb400108,0xf63b0101,0xfb400108,0xf63d0101,
	0xfb400108,0xf6410101,0xfb400108,0xf6420101,0xfb400108,0xf6440101,0xfb400108,0xf6450101,
	0xfb400108,0xf6460101,0xfb400108,0xf6470101,0xfb400108,0xf6480101,0xfb400108,0xf6490101,
	0xfb400108,0xf64a0101,0xfb400108,0xf64b0101,0xfb400108,0xf64e0101,0xfb400108,0xf64f0101,
	0xfb400108,0xf6500101,0xfb400108,0xf6510101,0xfb400108,0xf6520101,0xfb400108,0xf6530101,
	0xfb400108,0xf6550101,0xfb400108,0xf6570101,0xfb400108,0xf6580101,0xfb400108,0xf6590101,
	0xfb400108,0xf65a0101,0xfb400108,0xf65b0101,0xfb400108,0xf65d0101,0xfb400108,0xf65f0101,
	0xfb400108,0xf6600101,0xfb400108,0xf6610101,0xfb400108,0xf6620101,0xfb400108,0xf6640101,
	0xfb400108,0xf6650101,0xfb400108,0xf6660101,0xfb400108,0xf6670101,0xfb400108,0xf6680101,
	0xfb400108,0xf6690101,0xfb400108,0xf66a0101,0xfb400108,0xf66c0101,0xfb400108,0xf66d0101,
	0xfb400108,0xf66e0101,0xfb400108,0xf6700101,0xfb400108,0xf6710101,0xfb400108,0xf6720101,
	0xfb400108,0xf6730101,0xfb400108,0xf6740101,0xfb400108,0xf6750101,0xfb400108,0xf6760101,
	0xfb400108,0xf6770101,0xfb400108,0xf6790101,0xfb400108,0xf67a0101,0xfb400108,0xf67c0101,
	0xfb400108,0xf67f0101,0xfb400108,0xf6800101,0xfb400108,0xf6810101,0xfb400108,0xf6830101,
	0xfb400108,0xf6850101,0xfb400108,0xf6890101,0xfb400108,0xf68a0101,0xfb400108,0xf68c0101,
	0xfb400108,0xf68d0101,0xfb400108,0xf68f0101,0xfb400108,0xf6900101,0xfb400108,0xf6920101,
	0xfb400108,0xf6940101,0xfb400108,0xf6950101,0xfb400108,0xf6970101,0xfb400108,0xf6980101,
	0xfb400108,0xf69a0101,0xfb400108,0xf69b0101,0xfb400108,0xf69c0101,0xfb400108,0xf69d0101,
	0xfb400108,0xf69e0101,0xfb400108,0xf69f0101,0xfb400108,0xf6a00101,0xfb400108,0xf6a10101,
	0xfb400108,0xf6a20101,0xfb400108,0xf6a30101,0xfb400108,0xf6a50101,0xfb400108,0xf6a60101,
	0xfb400108,0xf6a70101,0xfb400108,0xf6a80101,0xfb400108,0xf6a90101,0xfb400108,0xf6aa0101,
	0xfb400108,0xf6ab0101,0xfb400108,0xf6ac0101,0xfb400108,0xf6ad0101,0xfb400108,0xf6af0101,
	0xfb400108,0xf6b00101,0xfb400108,0xf6b30101,0xfb400108,0xf6b50101,0xfb400108,0xf6b60101,
	0xfb400108,0xf6b70101,0xfb400108,0xf6b80101,0xfb400108,0xf6b90101,0xfb400108,0xf6ba0101,
	0xfb400108,0xf6bb0101,0xfb400108,0xf6bc0101,0xfb400108,0xf6bd0101,0xfb400108,0xf6be0101,
	0xfb400108,0xf6c00101,0xfb400108,0xf6c10101,0xfb400108,0xf6c30101,0xfb400108,0xf6c40101,
	0xfb400108,0xf6c70101,0xfb400108,0xf6c90101,0xfb400108,0xf6cb0101,0xfb400108,0xf6cc0101,
	0xfb400108,0xf6d30101,0xfb400108,0xf6d50101,0xfb400108,0xf6d90101,0xfb400108,0xf6da0101,
	0xfb400108,0xf6dc0101,0xfb400108,0xf6dd0101,0xfb400108,0xf6de0101,0xfb400108,0xf6e00101,
	0xfb400108,0xf6e10101,0xfb400108,0xf6e20101,0xfb400108,0xf6e30101,0xfb400108,0xf6e40101,
	0xfb400108,0xf6e60101,0xfb400108,0xf6e70101,0xfb400108,0xf6e80101,0xfb400108,0xf6e90101,
	0xfb400108,0xf6ea0101,0xfb400108,0xf6eb0101,0xfb400108,0xf6ec0101,0xfb400108,0xf6ed0101,
	0xfb400108,0xf6f00101,0xfb400108,0xf6f30101,0xfb400108,0xf6f50101,0xfb400108,0xf6f60101,
	0xfb400108,0xf6f70101,0xfb400108,0xf6fa0101,0xfb400108,0xf6fb0101,0xfb400108,0xf6fd0101,
	0xfb400108,0xf6ff0101,0xfb400108,0xf7000101,0xfb400108,0xf7020101,0xfb400108,0xf7030101,
	0xfb400108,0xf7050101,0xfb400108,0xf7060101,0xfb400108,0xf70a0101,0xfb400108,0xf70c0101,
	0xfb400108,0xf70e0101,0xfb400108,0xf70f0101,0xfb400108,0xf7100101,0xfb400108,0xf7110101,
	0xfb400108,0xf7120101,0xfb400108,0xf7130101,0xfb400108,0xf7140101,0xfb400108,0xf7150101,
	0xfb400108,0xf7160101,0xfb400108,0xf7170101,0xfb400108,0xf7180101,0xfb400108,0xf71b0101,
	0xfb400108,0xf71c0101,0xfb400108,0xf71d0101,0xfb400108,0xf71e0101,0xfb400108,0xf7210101,
	0xfb400108,0xf7230101,0xfb400108,0xf7240101,0xfb400108,0xf7250101,0xfb400108,0xf7270101,
	0xfb400108,0xf72a0101,0xfb400108,0xf72b0101,0xfb400108,0xf72c0101,0xfb400108,0xf72e0101,
	0xfb400108,0xf7300101,0xfb400108,0xf7310101,0xfb400108,0xf7320101,0xfb400108,0xf7330101,
	0xfb400108,0xf7340101,0xfb400108,0xf7390101,0xfb400108,0xf73b0101,0xfb400108,0xf73d0101,
	0xfb400108,0xf73e0101,0xfb400108,0xf73f0101,0xfb400108,0xf7420101,0xfb400108,0xf7440101,
	0xfb400108,0xf7450101,0xfb400108,0xf7460101,0xfb400108,0xf7480101,0xfb400108,0xf7490101,
	0xfb400108,0xf74a0101,0xfb400108,0xf74b0101,0xfb400108,0xf74c0101,0xfb400108,0xf74d0101,
	0xfb400108,0xf74e0101,0xfb400108,0xf74f0101,0xfb400108,0xf7520101,0xfb400108,0xf7530101,
	0xfb400108,0xf7540101,0xfb400108,0xf7550101,0xfb400108,0xf7560101,0xfb400108,0xf7570101,
	0xfb400108,0xf7580101,0xfb400108,0xf7590101,0xfb400108,0xf75c0101,0xfb400108,0xf75d0101,
	0xfb400108,0xf75e0101,0xfb400108,0xf75f0101,0xfb400108,0xf7600101,0xfb400108,0xf7640101,
	0xfb400108,0xf7670101,0xfb400108,0xf7690101,0xfb400108,0xf76a0101,0xfb400108,0xf76d0101,
	0xfb400108,0xf76e0101,0xfb400108,0xf76f0101,0xfb400108,0xf7700101,0xfb400108,0xf7710101,
	0xfb400108,0xf7720101,0xfb400108,0xf7730101,0xfb400108,0xf7740101,0xfb400108,0xf7750101,
	0xfb400108,0xf7760101,0xfb400108,0xf7770101,0xfb400108,0xf7780101,0xfb400108,0xf77a0101,
	0xfb400108,0xf77b0101,0xfb400108,0xf77c0101,0xfb400108,0xf7810101,0xfb400108,0xf7820101,
	0xfb400108,0xf7830101,0xfb400108,0xf7860101,0xfb400108,0xf7870101,0xfb400108,0xf7880101,
	0xfb400108,0xf7890101,0xfb400108,0xf78a0101,0xfb400108,0xf78b0101,0xfb400108,0xf78f0101,
	0xfb400108,0xf7900101,0xfb400108,0xf7930101,0xfb400108,0xf7940101,0xfb400108,0xf7950101,
	0xfb400108,0xf7960101,0xfb400108,0xf7970101,0xfb400108,0xf7980101,0xfb400108,0xf7990101,
	0xfb400108,0xf79a0101,0xfb400108,0xf79b0101,0xfb400108,0xf79c0101,0xfb400108,0xf79d0101,
	0xfb400108,0xf79e0101,0xfb400108,0xf7a10101,0xfb400108,0xf7a30101,0xfb400108,0xf7a40101,
	0xfb400108,0xf7a60101,0xfb400108,0xf7a80101,0xfb400108,0xf7ab0101,0xfb400108,0xf7ad0101,
	0xfb400108,0xf7ae0101,0xfb400108,0xf7af0101,0xfb400108,0xf7b10101,0xfb400108,0xf7b20101,
	0xfb400108,0xf7b40101,0xfb400108,0xf7b60101,0xfb400108,0xf7b70101,0xfb400108,0xf7b80101,
	0xfb400108,0xf7b90101,0xfb400108,0xf7ba0101,0xfb400108,0xf7bc0101,0xfb400108,0xf7be0101,
	0xfb400108,0xf7c00101,0xfb400108,0xf7c10101,0xfb400108,0xf7c20101,0xfb400108,0xf7c30101,
	0xfb400108,0xf7c40101,0xfb400108,0xf7c50101,0xfb400108,0xf7c60101,0xfb400108,0xf7c70101,
	0xfb400108,0xf7c80101,0xfb400108,0xf7c90101,0xfb400108,0xf7ca0101,0xfb400108,0xf7cb0101,
	0xfb400108,0xf7cc0101,0xfb400108,0xf7ce0101,0xfb400108,0xf7cf0101,0xfb400108,0xf7d00101,
	0xfb400108,0xf7d10101,0xfb400108,0xf7d20101,0xfb400108,0xf7d30101,0xfb400108,0xf7d40101,
	0xfb400108,0xf7d50101,0xfb400108,0xf7d60101,0xfb400108,0xf7d80101,0xfb400108,0xf7d90101,
	0xfb400108,0xf7da0101,0xfb400108,0xf7dd0101,0xfb400108,0xf7de0101,0xfb400108,0xf7df0101,
	0xfb400108,0xf7e00101,0xfb400108,0xf7e10101,0xfb400108,0xf7e40101,0xfb400108,0xf7e60101,
	0xfb400108,0xf7e80101,0xfb400108,0xf7ea0101,0xfb400108,0xf7ef0101,0xfb400108,0xf7f00101,
	0xfb400108,0xf7f10101,0xfb400108,0xf7f20101,0xfb400108,0xf7f40101,0xfb400108,0xf7f50101,
	0xfb400108,0xf7f70101,0xfb400108,0xf7f90101,0xfb400108,0xf7fa0101,0xfb400108,0xf7fb0101,
	0xfb400108,0xf7fc0101,0xfb400108,0xf8030101,0xfb400108,0xf8040101,0xfb400108,0xf8050101,
	0xfb400108,0xf8060101,0xfb400108,0xf8070101,0xfb400108,0xf8080101,0xfb400108,0xf80a0101,
	0xfb400108,0xf80b0101,0xfb400108,0xf80e0101,0xfb400108,0xf80f0101,0xfb400108,0xf8100101,
	0xfb400108,0xf8130101,0xfb400108,0xf8150101,0xfb400108,0xf8190101,0xfb400108,0xf81b0101,
	0xfb400108,0xf81e0101,0xfb400108,0xf8200101,0xfb400108,0xf8210101,0xfb400108,0xf8220101,
	0xfb400108,0xf8240101,0xfb400108,0xf8280101,0xfb400108,0xf82a0101,0xfb400108,0xf82b0101,
	0xfb400108,0xf82e0101,0xfb400108,0xf82f0101,0xfb400108,0xf8310101,0xfb400108,0xf8320101,
	0xfb400108,0xf8330101,0xfb400108,0xf8350101,0xfb400108,0xf8360101,0xfb400108,0xf83d0101,
	0xfb400108,0xf83f0101,0xfb400108,0xf8410101,0xfb400108,0xf8420101,0xfb400108,0xf8430101,
	0xfb400108,0xf8440101,0xfb400108,0xf8460101,0xfb400108,0xf8480101,0xfb400108,0xf8490101,
	0xfb400108,0xf84a0101,0xfb400108,0xf84b0101,0xfb400108,0xf84d0101,0xfb400108,0xf84f0101,
	0xfb400108,0xf8510101,0xfb400108,0xf8530101,0xfb400108,0xf8540101,0xfb400108,0xf8580101,
	0xfb400108,0xf8590101,0xfb400108,0xf85a0101,0xfb400108,0xf85b0101,0xfb400108,0xf85c0101,
	0xfb400108,0xf85e0101,0xfb400108,0xf85f0101,0xfb400108,0xf8600101,0xfb400108,0xf8610101,
	0xfb400108,0xf8620101,0xfb400108,0xf8630101,0xfb400108,0xf8640101,0xfb400108,0xf8650101,
	0xfb400108,0xf8660101,0xfb400108,0xf8670101,0xfb400108,0xf8680101,0xfb400108,0xf8690101,
	0xfb400108,0xf86f0101,0xfb400108,0xf8700101,0xfb400108,0xf8710101,0xfb400108,0xf8720101,
	0xfb400108,0xf8730101,0xfb400108,0xf8740101,0xfb400108,0xf8750101,0xfb400108,0xf8760101,
	0xfb400108,0xf8780101,0xfb400108,0xf8790101,0xfb400108,0xf87a0101,0xfb400108,0xf87b0101,
	0xfb400108,0xf87d0101,0xfb400108,0xf87e0101,0xfb400108,0xf87f0101,0xfb400108,0xf8800101,
	0xfb400108,0xf8810101,0xfb400108,0xf8820101,0xfb400108,0xf8830101,0xfb400108,0xf8840101,
	0xfb400108,0xf8850101,0xfb400108,0xf8860101,0xfb400108,0xf8880101,0xfb400108,0xf88a0101,
	0xfb400108,0xf88b0101,0xfb400108,0xf88f0101,0xfb400108,0xf8900101,0xfb400108,0xf8920101,
	0xfb400108,0xf8940101,0xfb400108,0xf8950101,0xfb400108,0xf8960101,0xfb400108,0xf8990101,
	0xfb400108,0xf89d0101,0xfb400108,0xf89e0101,0xfb400108,0xf8a00101,0xfb400108,0xf8a20101,
	0xfb400108,0xf8a40101,0xfb400108,0xf8a60101,0xfb400108,0xf8a80101,0xfb400108,0xf8a90101,
	0xfb400108,0xf8aa0101,0xfb400108,0xf8ab0101,0xfb400108,0xf8ac0101,0xfb400108,0xf8ad0101,
	0xfb400108,0xf8ae0101,0xfb400108,0xf8af0101,0xfb400108,0xf8b50101,0xfb400108,0xf8b60101,
	0xfb400108,0xf8b70101,0xfb400108,0xf8b80101,0xfb400108,0xf8ba0101,0xfb400108,0xf8bb0101,
	0xfb400108,0xf8bc0101,0xfb400108,0xf8bd0101,0xfb400108,0xf8bf0101,0xfb400108,0xf8c00101,
	0xfb400108,0xf8c20101,0xfb400108,0xf8c30101,0xfb400108,0xf8c40101,0xfb400108,0xf8c60101,
	0xfb400108,0xf8c70101,0xfb400108,0xf8c80101,0xfb400108,0xf8cc0101,0xfb400108,0xf8cd0101,
	0xfb400108,0xf8ce0101,0xfb400108,0xf8cf0101,0xfb400108,0xf8d10101,0xfb400108,0xf8d20101,
	0xfb400108,0xf8d30101,0xfb400108,0xf8d60101,0xfb400108,0xf8d70101,0xfb400108,0xf8d80101,
	0xfb400108,0xf8da0101,0xfb400108,0xf8db0101,0xfb400108,0xf8dc0101,0xfb400108,0xf8dd0101,
	0xfb400108,0xf8de0101,0xfb400108,0xf8df0101,0xfb400108,0xf8e00101,0xfb400108,0xf8e10101,
	0xfb400108,0xf8e20101,0xfb400108,0xf8e30101,0xfb400108,0xf8e40101,0xfb400108,0xf8e50101,
	0xfb400108,0xf8e60101,0xfb400108,0xf8e70101,0xfb400108,0xf8e90101,0xfb400108,0xf8ea0101,
	0xfb400108,0xf8eb0101,0xfb400108,0xf8ed0101,0xfb400108,0xf8ee0101,0xfb400108,0xf8ef0101,
	0xfb400108,0xf8f00101,0xfb400108,0xf8f10101,0xfb400108,0xf8f30101,0xfb400108,0xf8f50101,
	0xfb400108,0xf8f60101,0xfb400108,0xf8f80101,0xfb400108,0xf8f90101,0xfb400108,0xf8fb0101,
	0xfb400108,0xf8fc0101,0xfb400108,0xf8fd0101,0xfb400108,0xf8fe0101,0xfb400108,0xf8ff0101,
	0xfb400108,0xf9000101,0xfb400108,0xf9020101,0xfb400108,0xf9030101,0xfb400108,0xf9040101,
	0xfb400108,0xf9060101,0xfb400108,0xf9070101,0xfb400108,0xf9080101,0xfb400108,0xf9090101,
	0xfb400108,0xf90a0101,0xfb400108,0xf90b0101,0xfb400108,0xf90c0101,0xfb400108,0xf90d0101,
	0xfb400108,0xf90e0101,0xfb400108,0xf90f0101,0xfb400108,0xf9100101,0xfb400108,0xf9110101,
	0xfb400108,0xf9120101,0xfb400108,0xf9140101,0xfb400108,0xf9150101,0xfb400108,0xf9160101,
	0xfb400108,0xf9170101,0xfb400108,0xf9180101,0xfb400108,0xf9190101,0xfb400108,0xf91a0101,
	0xfb400108,0xf91b0101,0xfb400108,0xf91c0101,0xfb400108,0xf91d0101,0xfb400108,0xf91f0101,
	0xfb400108,0xf9200101,0xfb400108,0xf9210101,0xfb400108,0xf9220101,0xfb400108,0xf9230101,
	0xfb400108,0xf9250101,0xfb400108,0xf9260101,0xfb400108,0xf9270101,0xfb400108,0xf9280101,
	0xfb400108,0xf9290101,0xfb400108,0xf92a0101,0xfb400108,0xf92b0101,0xfb400108,0xf92c0101,
	0xfb400108,0xf92d0101,0xfb400108,0xf92e0101,0xfb400108,0xf92f0101,0xfb400108,0xf9300101,
	0xfb400108,0xf9310101,0xfb400108,0xf9320101,0xfb400108,0xf9330101,0xfb400108,0xf9350101,
	0xfb400108,0xf9360101,0xfb400108,0xf9370101,0xfb400108,0xf9380101,0xfb400108,0xf9390101,
	0xfb400108,0xf93d0101,0xfb400108,0xf93f0101,0xfb400108,0xf9420101,0xfb400108,0xf9430101,
	0xfb400108,0xf9440101,0xfb400108,0xf9450101,0xfb400108,0xf9470101,0xfb400108,0xf94a0101,
	0xfb400108,0xf94b0101,0xfb400108,0xf94c0101,0xfb400108,0xf94d0101,0xfb400108,0xf94e0101,
	0xfb400108,0xf94f0101,0xfb400108,0xf9500101,0xfb400108,0xf9510101,0xfb400108,0xf9520101,
	0xfb400108,0xf9540101,0xfb400108,0xf9550101,0xfb400108,0xf9580101,0xfb400108,0xf9590101,
	0xfb400108,0xf9610101,0xfb400108,0xf9630101,0xfb400108,0xf9640101,0xfb400108,0xf9660101,
	0xfb400108,0xf9690101,0xfb400108,0xf96a0101,0xfb400108,0xf96b0101,0xfb400108,0xf96c0101,
	0xfb400108,0xf96e0101,0xfb400108,0xf9700101,0xfb400108,0xf9710101,0xfb400108,0xf9720101,
	0xfb400108,0xf9730101,0xfb400108,0xf9740101,0xfb400108,0xf9750101,0xfb400108,0xf9760101,
	0xfb400108,0xf9790101,0xfb400108,0xf97b0101,0xfb400108,0xf97c0101,0xfb400108,0xf97d0101,
	0xfb400108,0xf97e0101,0xfb400108,0xf97f0101,0xfb400108,0xf9820101,0xfb400108,0xf9830101,
	0xfb400108,0xf9860101,0xfb400108,0xf9870101,0xfb400108,0xf9880101,0xfb400108,0xf9890101,
	0xfb400108,0xf98b0101,0xfb400108,0xf98c0101,0xfb400108,0xf98d0101,0xfb400108,0xf98e0101,
	0xfb400108,0xf9900101,0xfb400108,0xf9910101,0xfb400108,0xf9920101,0xfb400108,0xf9930101,
	0xfb400108,0xf9940101,0xfb400108,0xf9950101,0xfb400108,0xf9960101,0xfb400108,0xf9970101,
	0xfb400108,0xf9980101,0xfb400108,0xf9990101,0xfb400108,0xf99b0101,0xfb400108,0xf99c0101,
	0xfb400108,0xf99d0101,0xfb400108,0xf99e0101,0xfb400108,0xf99f0101,0xfb400108,0xf9a00101,
	0xfb400108,0xf9a10101,0xfb400108,0xf9a20101,0xfb400108,0xf9a30101,0xfb400108,0xf9a40101,
	0xfb400108,0xf9a50101,0xfb400108,0xf9a60101,0xfb400108,0xf9a80101,0xfb400108,0xf9a90101,
	0xfb400108,0xf9aa0101,0xfb400108,0xf9ab0101,0xfb400108,0xf9ac0101,0xfb400108,0xf9ad0101,
	0xfb400108,0xf9ae0101,0xfb400108,0xf9af0101,0xfb400108,0xf9b00101,0xfb400108,0xf9b10101,
	0xfb400108,0xf9b20101,0xfb400108,0xf9b40101,0xfb400108,0xf9b50101,0xfb400108,0xf9b60101,
	0xfb400108,0xf9b70101,0xfb400108,0xf9b80101,0xfb400108,0xf9bc0101,0xfb400108,0xf9bf0101,
	0xfb400108,0xf9c20101,0xfb400108,0xf9c40101,0xfb400108,0xf9c50101,0xfb400108,0xf9c70101,
	0xfb400108,0xf9c80101,0xfb400108,0xf9ca0101,0xfb400108,0xf9cc0101,0xfb400108,0xf9ce0101,
	0xfb400108,0xf9cf0101,0xfb400108,0xf9d00101,0xfb400108,0xf9d30101,0xfb400108,0xf9d40101,
	0xfb400108,0xf9d60101,0xfb400108,0xf9d70101,0xfb400108,0xf9d90101,0xfb400108,0xf9da0101,
	0xfb400108,0xf9db0101,0xfb400108,0xf9dc0101,0xfb400108,0xf9dd0101,0xfb400108,0xf9de0101,
	0xfb400108,0xf9e00101,0xfb400108,0xf9e10101,0xfb400108,0xf9e20101,0xfb400108,0xf9e50101,
	0xfb400108,0xf9e80101,0xfb400108,0xf9ea0101,0xfb400108,0xf9ec0101,0xfb400108,0xf9ee0101,
	0xfb400108,0xf9f10101,0xfb400108,0xf9f20101,0xfb400108,0xf9f30101,0xfb400108,0xf9f40101,
	0xfb400108,0xf9f50101,0xfb400108,0xf9f60101,0xfb400108,0xf9f70101,0xfb400108,0xf9f90101,
	0xfb400108,0xf9fa0101,0xfb400108,0xf9fc0101,0xfb400108,0xf9fe0101,0xfb400108,0xf9ff0101,
	0xfb400108,0xfa010101,0xfb400108,0xfa040101,0xfb400108,0xfa050101,0xfb400108,0xfa070101,
	0xfb400108,0xfa080101,0xfb400108,0xfa090101,0xfb400108,0xfa0a0101,0xfb400108,0xfa0c0101,
	0xfb400108,0xfa0f0101,0xfb400108,0xfa100101,0xfb400108,0xfa110101,0xfb400108,0xfa120101,
	0xfb400108,0xfa130101,0xfb400108,0xfa150101,0xfb400108,0xfa160101,0xfb400108,0xfa180101,
	0xfb400108,0xfa190101,0xfb400108,0xfa1b0101,0xfb400108,0xfa1c0101,0xfb400108,0xfa1d0101,
	0xfb400108,0xfa1f0101,0xfb400108,0xfa210101,0xfb400108,0xfa220101,0xfb400108,0xfa240101,
	0xfb400108,0xfa250101,0xfb400108,0xfa260101,0xfb400108,0xfa270101,0xfb400108,0xfa280101,
	0xfb400108,0xfa290101,0xfb400108,0xfa2a0101,0xfb400108,0xfa2b0101,0xfb400108,0xfa2c0101,
	0xfb400108,0xfa2d0101,0xfb400108,0xfa2e0101,0xfb400108,0xfa2f0101,0xfb400108,0xfa300101,
	0xfb400108,0xfa310101,0xfb400108,0xfa320101,0xfb400108,0xfa340101,0xfb400108,0xfa350101,
	0xfb400108,0xfa360101,0xfb400108,0xfa380101,0xfb400108,0xfa3a0101,0xfb400108,0xfa3e0101,
	0xfb400108,0xfa400101,0xfb400108,0xfa410101,0xfb400108,0xfa420101,0xfb400108,0xfa430101,
	0xfb400108,0xfa440101,0xfb400108,0xfa450101,0xfb400108,0xfa470101,0xfb400108,0xfa480101,
	0xfb400108,0xfa490101,0xfb400108,0xfa4a0101,0xfb400108,0xfa4b0101,0xfb400108,0xfa4c0101,
	0xfb400108,0xfa4d0101,0xfb400108,0xfa4e0101,0xfb400108,0xfa4f0101,0xfb400108,0xfa500101,
	0xfb400108,0xfa520101,0xfb400108,0xfa530101,0xfb400108,0xfa540101,0xfb400108,0xfa550101,
	0xfb400108,0xfa560101,0xfb400108,0xfa580101,0xfb400108,0xfa590101,0xfb400108,0xfa5a0101,
	0xfb400108,0xfa5b0101,0xfb400108,0xfa5c0101,0xfb400108,0xfa5d0101,0xfb400108,0xfa5e0101,
	0xfb400108,0xfa5f0101,0xfb400108,0xfa600101,0xfb400108,0xfa610101,0xfb400108,0xfa620101,
	0xfb400108,0xfa630101,0xfb400108,0xfa640101,0xfb400108,0xfa650101,0xfb400108,0xfa660101,
	0xfb400108,0xfa670101,0xfb400108,0xfa680101,0xfb400108,0xfa690101,0xfb400108,0xfa6a0101,
	0xfb400108,0xfa6b0101,0xfb400108,0xfa6c0101,0xfb400108,0xfa6d0101,0xfb400108,0xfa6e0101,
	0xfb400108,0xfa6f0101,0xfb400108,0xfa710101,0xfb400108,0xfa720101,0xfb400108,0xfa730101,
	0xfb400108,0xfa750101,0xfb400108,0xfa7b0101,0xfb400108,0xfa7c0101,0xfb400108,0xfa7d0101,
	0xfb400108,0xfa7e0101,0xfb400108,0xfa820101,0xfb400108,0xfa850101,0xfb400108,0xfa870101,
	0xfb400108,0xfa890101,0xfb400108,0xfa8a0101,0xfb400108,0xfa8b0101,0xfb400108,0xfa8c0101,
	0xfb400108,0xfa8e0101,0xfb400108,0xfa8f0101,0xfb400108,0xfa900101,0xfb400108,0xfa930101,
	0xfb400108,0xfa940101,0xfb400108,0xfa990101,0xfb400108,0xfa9a0101,0xfb400108,0xfa9b0101,
	0xfb400108,0xfa9e0101,0xfb400108,0xfaa10101,0xfb400108,0xfaa20101,0xfb400108,0xfaa30101,
	0xfb400108,0xfaa40101,0xfb400108,0xfaa70101,0xfb400108,0xfaa90101,0xfb400108,0xfaaa0101,
	0xfb400108,0xfaab0101,0xfb400108,0xfaae0101,0xfb400108,0xfaaf0101,0xfb400108,0xfab00101,
	0xfb400108,0xfab10101,0xfb400108,0xfab20101,0xfb400108,0xfab40101,0xfb400108,0xfab50101,
	0xfb400108,0xfab60101,0xfb400108,0xfab70101,0xfb400108,0xfab80101,0xfb400108,0xfab90101,
	0xfb400108,0xfaba0101,0xfb400108,0xfabb0101,0xfb400108,0xfabc0101,0xfb400108,0xfabd0101,
	0xfb400108,0xfabe0101,0xfb400108,0xfac00101,0xfb400108,0xfac10101,0xfb400108,0xfac20101,
	0xfb400108,0xfac30101,0xfb400108,0xfac40101,0xfb400108,0xfac50101,0xfb400108,0xfac60101,
	0xfb400108,0xfac70101,0xfb400108,0xfac80101,0xfb400108,0xfac90101,0xfb400108,0xfaca0101,
	0xfb400108,0xfacc0101,0xfb400108,0xfacd0101,0xfb400108,0xface0101,0xfb400108,0xfacf0101,
	0xfb400108,0xfad00101,0xfb400108,0xfad10101,0xfb400108,0xfad20101,0xfb400108,0xfad30101,
	0xfb400108,0xfad40101,0xfb400108,0xfad50101,0xfb400108,0xfad70101,0xfb400108,0xfad80101,
	0xfb400108,0xfada0101,0xfb400108,0xfadb0101,0xfb400108,0xfadc0101,0xfb400108,0xfadd0101,
	0xfb400108,0xfae10101,0xfb400108,0xfae20101,0xfb400108,0xfae40101,0xfb400108,0xfae70101,
	0xfb400108,0xfae80101,0xfb400108,0xfae90101,0xfb400108,0xfaea0101,0xfb400108,0xfaeb0101,
	0xfb400108,0xfaec0101,0xfb400108,0xfaee0101,0xfb400108,0xfaf00101,0xfb400108,0xfaf10101,
	0xfb400108,0xfaf20101,0xfb400108,0xfaf30101,0xfb400108,0xfaf40101,0xfb400108,0xfaf50101,
	0xfb400108,0xfaf60101,0xfb400108,0xfaf70101,0xfb400108,0xfaf80101,0xfb400108,0xfafb0101,
	0xfb400108,0xfafc0101,0xfb400108,0xfafe0101,0xfb400108,0xfb000101,0xfb400108,0xfb010101,
	0xfb400108,0xfb020101,0xfb400108,0xfb050101,0xfb400108,0xfb070101,0xfb400108,0xfb090101,
	0xfb400108,0xfb0c0101,0xfb400108,0xfb0d0101,0xfb400108,0xfb0e0101,0xfb400108,0xfb100101,
	0xfb400108,0xfb120101,0xfb400108,0xfb130101,0xfb400108,0xfb160101,0xfb400108,0xfb170101,
	0xfb400108,0xfb180101,0xfb400108,0xfb1a0101,0xfb400108,0xfb1c0101,0xfb400108,0xfb1d0101,
	0xfb400108,0xfb1f0101,0xfb400108,0xfb210101,0xfb400108,0xfb220101,0xfb400108,0xfb230101,
	0xfb400108,0xfb270101,0xfb400108,0xfb290101,0xfb400108,0xfb2d0101,0xfb400108,0xfb2f0101,
	0xfb400108,0xfb300101,0xfb400108,0xfb320101,0xfb400108,0xfb340101,0xfb400108,0xfb350101,
	0xfb400108,0xfb360101,0xfb400108,0xfb370101,0xfb400108,0xfb390101,0xfb400108,0xfb3b0101,
	0xfb400108,0xfb3d0101,0xfb400108,0xfb3f0101,0xfb400108,0xfb400101,0xfb400108,0xfb410101,
	0xfb400108,0xfb420101,0xfb400108,0xfb430101,0xfb400108,0xfb440101,0xfb400108,0xfb460101,
	0xfb400108,0xfb480101,0xfb400108,0xfb4a0101,0xfb400108,0xfb4d0101,0xfb400108,0xfb4e0101,
	0xfb400108,0xfb530101,0xfb400108,0xfb550101,0xfb400108,0xfb570101,0xfb400108,0xfb590101,
	0xfb400108,0xfb5c0101,0xfb400108,0xfb5e0101,0xfb400108,0xfb5f0101,0xfb400108,0xfb610101,
	0xfb400108,0xfb630101,0xfb400108,0xfb640101,0xfb400108,0xfb650101,0xfb400108,0xfb660101,
	0xfb400108,0xfb670101,0xfb400108,0xfb680101,0xfb400108,0xfb690101,0xfb400108,0xfb6a0101,
	0xfb400108,0xfb6b0101,0xfb400108,0xfb6c0101,0xfb400108,0xfb6d0101,0xfb400108,0xfb6f0101,
	0xfb400108,0xfb700101,0xfb400108,0xfb730101,0xfb400108,0xfb740101,0xfb400108,0xfb760101,
	0xfb400108,0xfb780101,0xfb400108,0xfb7a0101,0xfb400108,0xfb7c0101,0xfb400108,0xfb7d0101,
	0xfb400108,0xfb7f0101,0xfb400108,0xfb810101,0xfb400108,0xfb820101,0xfb400108,0xfb830101,
	0xfb400108,0xfb840101,0xfb400108,0xfb860101,0xfb400108,0xfb870101,0xfb400108,0xfb880101,
	0xfb400108,0xfb890101,0xfb400108,0xfb8a0101,0xfb400108,0xfb8b0101,0xfb400108,0xfb8c0101,
	0xfb400108,0xfb8e0101,0xfb400108,0xfb8f0101,0xfb400108,0xfb910101,0xfb400108,0xfb920101,
	0xfb400108,0xfb930101,0xfb400108,0xfb960101,0xfb400108,0xfb980101,0xfb400108,0xfb990101,
	0xfb400108,0xfb9a0101,0xfb400108,0xfb9b0101,0xfb400108,0xfb9e0101,0xfb400108,0xfb9f0101,
	0xfb400108,0xfba00101,0xfb400108,0xfba30101,0xfb400108,0xfba40101,0xfb400108,0xfba50101,
	0xfb400108,0xfbae0101,0xfb400108,0xfbaf0101,0xfb400108,0xfbb00101,0xfb400108,0xfbb20101,
	0xfb400108,0xfbb30101,0xfb400108,0xfbb50101,0xfb400108,0xfbb60101,0xfb400108,0xfbb70101,
	0xfb400108,0xfbb90101,0xfb400108,0xfbba0101,0xfb400108,0xfbbb0101,0xfb400108,0xfbbc0101,
	0xfb400108,0xfbbd0101,0xfb400108,0xfbbe0101,0xfb400108,0xfbbf0101,0xfb400108,0xfbc00101,
	0xfb400108,0xfbc20101,0xfb400108,0xfbc30101,0xfb400108,0xfbc40101,0xfb400108,0xfbc50101,
	0xfb400108,0xfbc80101,0xfb400108,0xfbc90101,0xfb400108,0xfbca0101,0xfb400108,0xfbcb0101,
	0xfb400108,0xfbcd0101,0xfb400108,0xfbce0101,0xfb400108,0xfbcf0101,0xfb400108,0xfbd00101,
	0xfb400108,0xfbd20101,0xfb400108,0xfbd40101,0xfb400108,0xfbd50101,0xfb400108,0xfbd60101,
	0xfb400108,0xfbd70101,0xfb400108,0xfbd80101,0xfb400108,0xfbdb0101,0xfb400108,0xfbdc0101,
	0xfb400108,0xfbde0101,0xfb400108,0xfbdf0101,0xfb400108,0xfbe00101,0xfb400108,0xfbe20101,
	0xfb400108,0xfbe30101,0xfb400108,0xfbe40101,0xfb400108,0xfbe70101,0xfb400108,0xfbe80101,
	0xfb400108,0xfbe90101,0xfb400108,0xfbeb0101,0xfb400108,0xfbec0101,0xfb400108,0xfbed0101,
	0xfb400108,0xfbef0101,0xfb400108,0xfbf00101,0xfb400108,0xfbf20101,0xfb400108,0xfbf30101,
	0xfb400108,0xfbf40101,0xfb400108,0xfbf50101,0xfb400108,0xfbf60101,0xfb400108,0xfbf80101,
	0xfb400108,0xfbf90101,0xfb400108,0xfbfa0101,0xfb400108,0xfbfb0101,0xfb400108,0xfbfd0101,
	0xfb400108,0xfbff0101,0xfb400108,0xfc000101,0xfb400108,0xfc010101,0xfb400108,0xfc020101,
	0xfb400108,0xfc030101,0xfb400108,0xfc040101,0xfb400108,0xfc050101,0xfb400108,0xfc060101,
	0xfb400108,0xfc080101,0xfb400108,0xfc090101,0xfb400108,0xfc0a0101,0xfb400108,0xfc0d0101,
	0xfb400108,0xfc0e0101,0xfb400108,0xfc100101,0xfb400108,0xfc110101,0xfb400108,0xfc120101,
	0xfb400108,0xfc130101,0xfb400108,0xfc140101,0xfb400108,0xfc150101,0xfb400108,0xfc170101,
	0xfb400108,0xfc180101,0xfb400108,0xfc190101,0xfb400108,0xfc1a0101,0xfb400108,0xfc1b0101,
	0xfb400108,0xfc1c0101,0xfb400108,0xfc1d0101,0xfb400108,0xfc1e0101,0xfb400108,0xfc200101,
	0xfb400108,0xfc210101,0xfb400108,0xfc220101,0xfb400108,0xfc230101,0xfb400108,0xfc240101,
	0xfb400108,0xfc250101,0xfb400108,0xfc280101,0xfb400108,0xfc290101,0xfb400108,0xfc2b0101,
	0xfb400108,0xfc2c0101,0xfb400108,0xfc2d0101,0xfb400108,0xfc2e0101,0xfb400108,0xfc2f0101,
	0xfb400108,0xfc300101,0xfb400108,0xfc310101,0xfb400108,0xfc320101,0xfb400108,0xfc330101,
	0xfb400108,0xfc340101,0xfb400108,0xfc350101,0xfb400108,0xfc360101,0xfb400108,0xfc370101,
	0xfb400108,0xfc390101,0xfb400108,0xfc3a0101,0xfb400108,0xfc3b0101,0xfb400108,0xfc3c0101,
	0xfb400108,0xfc3d0101,0xfb400108,0xfc3e0101,0xfb400108,0xfc420101,0xfb400108,0xfc430101,
	0xfb400108,0xfc440101,0xfb400108,0xfc450101,0xfb400108,0xfc460101,0xfb400108,0xfc470101,
	0xfb400108,0xfc480101,0xfb400108,0xfc490101,0xfb400108,0xfc4a0101,0xfb400108,0xfc4b0101,
	0xfb400108,0xfc4c0101,0xfb400108,0xfc4e0101,0xfb400108,0xfc4f0101,0xfb400108,0xfc500101,
	0xfb400108,0xfc510101,0xfb400108,0xfc520101,0xfb400108,0xfc530101,0xfb400108,0xfc540101,
	0xfb400108,0xfc550101,0xfb400108,0xfc560101,0xfb400108,0xfc570101,0xfb400108,0xfc580101,
	0xfb400108,0xfc590101,0xfb400108,0xfc5a0101,0xfb400108,0xfc5b0101,0xfb400108,0xfc5c0101,
	0xfb400108,0xfc5d0101,0xfb400108,0xfc5e0101,0xfb400108,0xfc5f0101,0xfb400108,0xfc600101,
	0xfb400108,0xfc610101,0xfb400108,0xfc620101,0xfb400108,0xfc630101,0xfb400108,0xfc640101,
	0xfb400108,0xfc650101,0xfb400108,0xfc660101,0xfb400108,0xfc670101,0xfb400108,0xfc680101,
	0xfb400108,0xfc690101,0xfb400108,0xfc6a0101,0xfb400108,0xfc6b0101,0xfb400108,0xfc6c0101,
	0xfb400108,0xfc6d0101,0xfb400108,0xfc6e0101,0xfb400108,0xfc6f0101,0xfb400108,0xfc700101,
	0xfb400108,0xfc710101,0xfb400108,0xfc720101,0xfb400108,0xfc750101,0xfb400108,0xfc760101,
	0xfb400108,0xfc770101,0xfb400108,0xfc780101,0xfb400108,0xfc790101,0xfb400108,0xfc7a0101,
	0xfb400108,0xfc7e0101,0xfb400108,0xfc7f0101,0xfb400108,0xfc800101,0xfb400108,0xfc810101,
	0xfb400108,0xfc820101,0xfb400108,0xfc830101,0xfb400108,0xfc840101,0xfb400108,0xfc850101,
	0xfb400108,0xfc860101,0xfb400108,0xfc870101,0xfb400108,0xfc880101,0xfb400108,0xfc8a0101,
	0xfb400108,0xfc8b0101,0xfb400108,0xfc8c0101,0xfb400108,0xfc8d0101,0xfb400108,0xfc8e0101,
	0xfb400108,0xfc8f0101,0xfb400108,0xfc900101,0xfb400108,0xfc930101,0xfb400108,0xfc940101,
	0xfb400108,0xfc960101,0xfb400108,0xfc990101,0xfb400108,0xfc9a0101,0xfb400108,0xfc9b0101,
	0xfb400108,0xfca00101,0xfb400108,0xfca10101,0xfb400108,0xfca30101,0xfb400108,0xfca60101,
	0xfb400108,0xfca70101,0xfb400108,0xfca80101,0xfb400108,0xfca90101,0xfb400108,0xfcab0101,
	0xfb400108,0xfcac0101,0xfb400108,0xfcad0101,0xfb400108,0xfcaf0101,0xfb400108,0xfcb00101,
	0xfb400108,0xfcb40101,0xfb400108,0xfcb50101,0xfb400108,0xfcb60101,0xfb400108,0xfcb70101,
	0xfb400108,0xfcb80101,0xfb400108,0xfcba0101,0xfb400108,0xfcbb0101,0xfb400108,0xfcbf0101,
	0xfb400108,0xfcc00101,0xfb400108,0xfcc20101,0xfb400108,0xfcc30101,0xfb400108,0xfcc40101,
	0xfb400108,0xfcc60101,0xfb400108,0xfcc90101,0xfb400108,0xfccb0101,0xfb400108,0xfcce0101,
	0xfb400108,0xfccf0101,0xfb400108,0xfcd00101,0xfb400108,0xfcd10101,0xfb400108,0xfcd20101,
	0xfb400108,0xfcd30101,0xfb400108,0xfcd40101,0xfb400108,0xfcd80101,0xfb400108,0xfcda0101,
	0xfb400108,0xfcdb0101,0xfb400108,0xfcdd0101,0xfb400108,0xfcde0101,0xfb400108,0xfce10101,
	0xfb400108,0xfce20101,0xfb400108,0xfce30101,0xfb400108,0xfce40101,0xfb400108,0xfce50101,
	0xfb400108,0xfce60101,0xfb400108,0xfce70101,0xfb400108,0xfce90101,0xfb400108,0xfcea0101,
	0xfb400108,0xfceb0101,0xfb400108,0xfcec0101,0xfb400108,0xfced0101,0xfb400108,0xfcee0101,
	0xfb400108,0xfcf00101,0xfb400108,0xfcf10101,0xfb400108,0xfcf20101,0xfb400108,0xfcf30101,
	0xfb400108,0xfcf40101,0xfb400108,0xfcf50101,0xfb400108,0xfcf60101,0xfb400108,0xfcf70101,
	0xfb400108,0xfcf90101,0xfb400108,0xfcfa0101,0xfb400108,0xfcfc0101,0xfb400108,0xfcfd0101,
	0xfb400108,0xfcfe0101,0xfb400108,0xfcff0101,0xfb400108,0xfd000101,0xfb400108,0xfd010101,
	0xfb400108,0xfd020101,0xfb400108,0xfd030101,0xfb400108,0xfd040101,0xfb400108,0xfd050101,
	0xfb400108,0xfd060101,0xfb400108,0xfd070101,0xfb400108,0xfd080101,0xfb400108,0xfd090101,
	0xfb400108,0xfd0b0101,0xfb400108,0xfd0c0101,0xfb400108,0xfd0d0101,0xfb400108,0xfd0e0101,
	0xfb400108,0xfd0f0101,0xfb400108,0xfd100101,0xfb400108,0xfd110101,0xfb400108,0xfd120101,
	0xfb400108,0xfd130101,0xfb400108,0xfd140101,0xfb400108,0xfd150101,0xfb400108,0xfd160101,
	0xfb400108,0xfd170101,0xfb400108,0xfd180101,0xfb400108,0xfd190101,0xfb400108,0xfd1a0101,
	0xfb400108,0xfd1b0101,0xfb400108,0xfd1c0101,0xfb400108,0xfd1d0101,0xfb400108,0xfd1e0101,
	0xfb400108,0xfd1f0101,0xfb400108,0xfd210101,0xfb400108,0xfd230101,0xfb400108,0xfd240101,
	0xfb400108,0xfd250101,0xfb400108,0xfd260101,0xfb400108,0xfd280101,0xfb400108,0xfd290101,
	0xfb400108,0xfd2a0101,0xfb400108,0xfd2c0101,0xfb400108,0xfd2d0101,0xfb400108,0xfd2e0101,
	0xfb400108,0xfd300101,0xfb400108,0xfd310101,0xfb400108,0xfd320101,0xfb400108,0xfd330101,
	0xfb400108,0xfd340101,0xfb400108,0xfd350101,0xfb400108,0xfd360101,0xfb400108,0xfd370101,
	0xfb400108,0xfd380101,0xfb400108,0xfd390101,0xfb400108,0xfd3a0101,0xfb400108,0xfd3b0101,
	0xfb400108,0xfd3c0101,0xfb400108,0xfd3d0101,0xfb400108,0xfd3e0101,0xfb400108,0xfd3f0101,
	0xfb400108,0xfd400101,0xfb400108,0xfd410101,0xfb400108,0xfd420101,0xfb400108,0xfd430101,
	0xfb400108,0xfd440101,0xfb400108,0xfd450101,0xfb400108,0xfd460101,0xfb400108,0xfd470101,
	0xfb400108,0xfd480101,0xfb400108,0xfd490101,0xfb400108,0xfd4a0101,0xfb400108,0xfd4b0101,
	0xfb400108,0xfd4c0101,0xfb400108,0xfd4d0101,0xfb400108,0xfd4e0101,0xfb400108,0xfd4f0101,
	0xfb400108,0xfd500101,0xfb400108,0xfd510101,0xfb400108,0xfd520101,0xfb400108,0xfd530101,
	0xfb400108,0xfd540101,0xfb400108,0xfd550101,0xfb400108,0xfd560101,0xfb400108,0xfd570101,
	0xfb400108,0xfd580101,0xfb400108,0xfd590101,0xfb400108,0xfd5a0101,0xfb400108,0xfd5b0101,
	0xfb400108,0xfd5c0101,0xfb400108,0xfd5d0101,0xfb400108,0xfd5e0101,0xfb400108,0xfd5f0101,
	0xfb400108,0xfd600101,0xfb400108,0xfd610101,0xfb400108,0xfd620101,0xfb400108,0xfd630101,
	0xfb400108,0xfd640101,0xfb400108,0xfd650101,0xfb400108,0xfd660101,0xfb400108,0xfd670101,
	0xfb400108,0xfd680101,0xfb400108,0xfd690101,0xfb400108,0xfd6a0101,0xfb400108,0xfd6b0101,
	0xfb400108,0xfd6c0101,0xfb400108,0xfd6d0101,0xfb400108,0xfd6f0101,0xfb400108,0xfd700101,
	0xfb400108,0xfd710101,0xfb400108,0xfd720101,0xfb400108,0xfd730101,0xfb400108,0xfd740101,
	0xfb400108,0xfd750101,0xfb400108,0xfd760101,0xfb400108,0xfd780101,0xfb400108,0xfd790101,
	0xfb400108,0xfd7a0101,0xfb400108,0xfd7b0101,0xfb400108,0xfd7c0101,0xfb400108,0xfd7d0101,
	0xfb400108,0xfd7e0101,0xfb400108,0xfd7f0101,0xfb400108,0xfd800101,0xfb400108,0xfd810101,
	0xfb400108,0xfd820101,0xfb400108,0xfd830101,0xfb400108,0xfd840101,0xfb400108,0xfd850101,
	0xfb400108,0xfd860101,0xfb400108,0xfd870101,0xfb400108,0xfd880101,0xfb400108,0xfd890101,
	0xfb400108,0xfd8a0101,0xfb400108,0xfd8b0101,0xfb400108,0xfd8c0101,0xfb400108,0xfd8d0101,
	0xfb400108,0xfd8e0101,0xfb400108,0xfd8f0101,0xfb400108,0xfd900101,0xfb400108,0xfd910101,
	0xfb400108,0xfd920101,0xfb400108,0xfd930101,0xfb400108,0xfd940101,0xfb400108,0xfd950101,
	0xfb400108,0xfd960101,0xfb400108,0xfd970101,0xfb400108,0xfd980101,0xfb400108,0xfd990101,
	0xfb400108,0xfd9a0101,0xfb400108,0xfd9b0101,0xfb400108,0xfd9c0101,0xfb400108,0xfd9d0101,
	0xfb400108,0xfd9e0101,0xfb400108,0xfd9f0101,0xfb400108,0xfda00101,0xfb400108,0xfda10101,
	0xfb400108,0xfda20101,0xfb400108,0xfda30101,0xfb400108,0xfda40101,0xfb400108,0xfda50101,
	0xfb400108,0xfda70101,0xfb400108,0xfda80101,0xfb400108,0xfda90101,0xfb400108,0xfdaa0101,
	0xfb400108,0xfdab0101,0xfb400108,0xfdac0101,0xfb400108,0xfdad0101,0xfb400108,0xfdaf0101,
	0xfb400108,0xfdb00101,0xfb400108,0xfdb10101,0xfb400108,0xfdb20101,0xfb400108,0xfdb30101,
	0xfb400108,0xfdb40101,0xfb400108,0xfdb50101,0xfb400108,0xfdb60101,0xfb400108,0xfdb70101,
	0xfb400108,0xfdb80101,0xfb400108,0xfdb90101,0xfb400108,0xfdba0101,0xfb400108,0xfdbb0101,
	0xfb400108,0xfdbc0101,0xfb400108,0xfdbd0101,0xfb400108,0xfdbe0101,0xfb400108,0xfdbf0101,
	0xfb400108,0xfdc00101,0xfb400108,0xfdc10101,0xfb400108,0xfdc20101,0xfb400108,0xfdc30101,
	0xfb400108,0xfdc40101,0xfb400108,0xfdc50101,0xfb400108,0xfdc60101,0xfb400108,0xfdc70101,
	0xfb400108,0xfdc80101,0xfb400108,0xfdc90101,0xfb400108,0xfdca0101,0xfb400108,0xfdcb0101,
	0xfb400108,0xfdcc0101,0xfb400108,0xfdcd0101,0xfb400108,0xfdce0101,0xfb400108,0xfdcf0101,
	0xfb400108,0xfdd00101,0xfb400108,0xfdd10101,0xfb400108,0xfdd20101,0xfb400108,0xfdd30101,
	0xfb400108,0xfdd40101,0xfb400108,0xfdd50101,0xfb400108,0xfdd60101,0xfb400108,0xfdd70101,
	0xfb400108,0xfdd80101,0xfb400108,0xfdd90101,0xfb400108,0xfdda0101,0xfb400108,0xfddb0101,
	0xfb400108,0xfddc0101,0xfb400108,0xfddd0101,0xfb400108,0xfdde0101,0xfb400108,0xfddf0101,
	0xfb400108,0xfde00101,0xfb400108,0xfde10101,0xfb400108,0xfde20101,0xfb400108,0xfde30101,
	0xfb400108,0xfde40101,0xfb400108,0xfde50101,0xfb400108,0xfde60101,0xfb400108,0xfde70101,
	0xfb400108,0xfde80101,0xfb400108,0xfde90101,0xfb400108,0xfdea0101,0xfb400108,0xfdeb0101,
	0xfb400108,0xfdec0101,0xfb400108,0xfded0101,0xfb400108,0xfdee0101,0xfb400108,0xfdef0101,
	0xfb400108,0xfdf00101,0xfb400108,0xfdf10101,0xfb400108,0xfdf20101,0xfb400108,0xfdf30101,
	0xfb400108,0xfdf40101,0xfb400108,0xfdf50101,0xfb400108,0xfdf60101,0xfb400108,0xfdf70101,
	0xfb400108,0xfdf80101,0xfb400108,0xfdf90101,0xfb400108,0xfdfa0101,0xfb400108,0xfdfb0101,
	0xfb400108,0xfdfc0101,0xfb400108,0xfdfd0101,0xfb400108,0xfdfe0101,0xfb400108,0xfdff0101,
	0xfb400108,0xfe000101,0xfb400108,0xfe010101,0xfb400108,0xfe020101,0xfb400108,0xfe030101,
	0xfb400108,0xfe040101,0xfb400108,0xfe050101,0xfb400108,0xfe060101,0xfb400108,0xfe070101,
	0xfb400108,0xfe080101,0xfb400108,0xfe090101,0xfb400108,0xfe0a0101,0xfb400108,0xfe0b0101,
	0xfb400108,0xfe0c0101,0xfb400108,0xfe0d0101,0xfb400108,0xfe0e0101,0xfb400108,0xfe0f0101,
	0xfb400108,0xfe100101,0xfb400108,0xfe110101,0xfb400108,0xfe120101,0xfb400108,0xfe130101,
	0xfb400108,0xfe140101,0xfb400108,0xfe150101,0xfb400108,0xfe160101,0xfb400108,0xfe170101,
	0xfb400108,0xfe180101,0xfb400108,0xfe190101,0xfb400108,0xfe1a0101,0xfb400108,0xfe1b0101,
	0xfb400108,0xfe1c0101,0xfb400108,0xfe1d0101,0xfb400108,0xfe1e0101,0xfb400108,0xfe1f0101,
	0xfb400108,0xfe200101,0xfb400108,0xfe210101,0xfb400108,0xfe220101,0xfb400108,0xfe230101,
	0xfb400108,0xfe240101,0xfb400108,0xfe250101,0xfb400108,0xfe260101,0xfb400108,0xfe270101,
	0xfb400108,0xfe280101,0xfb400108,0xfe290101,0xfb400108,0xfe2a0101,0xfb400108,0xfe2b0101,
	0xfb400108,0xfe2c0101,0xfb400108,0xfe2d0101,0xfb400108,0xfe2e0101,0xfb400108,0xfe2f0101,
	0xfb400108,0xfe300101,0xfb400108,0xfe310101,0xfb400108,0xfe320101,0xfb400108,0xfe330101,
	0xfb400108,0xfe340101,0xfb400108,0xfe350101,0xfb400108,0xfe360101,0xfb400108,0xfe370101,
	0xfb400108,0xfe380101,0xfb400108,0xfe390101,0xfb400108,0xfe3a0101,0xfb400108,0xfe3c0101,
	0xfb400108,0xfe3d0101,0xfb400108,0xfe3e0101,0xfb400108,0xfe3f0101,0xfb400108,0xfe400101,
	0xfb400108,0xfe420101,0xfb400108,0xfe430101,0xfb400108,0xfe440101,0xfb400108,0xfe450101,
	0xfb400108,0xfe460101,0xfb400108,0xfe480101,0xfb400108,0xfe490101,0xfb400108,0xfe4a0101,
	0xfb400108,0xfe4b0101,0xfb400108,0xfe4c0101,0xfb400108,0xfe4d0101,0xfb400108,0xfe4e0101,
	0xfb400108,0xfe4f0101,0xfb400108,0xfe500101,0xfb400108,0xfe510101,0xfb400108,0xfe520101,
	0xfb400108,0xfe530101,0xfb400108,0xfe540101,0xfb400108,0xfe550101,0xfb400108,0xfe560101,
	0xfb400108,0xfe570101,0xfb400108,0xfe580101,0xfb400108,0xfe590101,0xfb400108,0xfe5a0101,
	0xfb400108,0xfe5b0101,0xfb400108,0xfe5c0101,0xfb400108,0xfe5d0101,0xfb400108,0xfe5e0101,
	0xfb400108,0xfe5f0101,0xfb400108,0xfe600101,0xfb400108,0xfe610101,0xfb400108,0xfe620101,
	0xfb400108,0xfe630101,0xfb400108,0xfe640101,0xfb400108,0xfe650101,0xfb400108,0xfe660101,
	0xfb400108,0xfe670101,0xfb400108,0xfe680101,0xfb400108,0xfe690101,0xfb400108,0xfe6a0101,
	0xfb400108,0xfe6b0101,0xfb400108,0xfe6c0101,0xfb400108,0xfe6d0101,0xfb400108,0xfe6e0101,
	0xfb400108,0xfe6f0101,0xfb400108,0xfe700101,0xfb400108,0xfe710101,0xfb400108,0xfe720101,
	0xfb400108,0xfe730101,0xfb400108,0xfe740101,0xfb400108,0xfe750101,0xfb400108,0xfe760101,
	0xfb400108,0xfe770101,0xfb400108,0xfe780101,0xfb400108,0xfe790101,0xfb400108,0xfe7a0101,
	0xfb400108,0xfe7b0101,0xfb400108,0xfe7c0101,0xfb400108,0xfe7d0101,0xfb400108,0xfe7e0101,
	0xfb400108,0xfe7f0101,0xfb400108,0xfe800101,0xfb400108,0xfe810101,0xfb400108,0xfe830101,
	0xfb400108,0xfe840101,0xfb400108,0xfe850101,0xfb400108,0xfe860101,0xfb400108,0xfe870101,
	0xfb400108,0xfe880101,0xfb400108,0xfe890101,0xfb400108,0xfe8a0101,0xfb400108,0xfe8b0101,
	0xfb400108,0xfe8c0101,0xfb400108,0xfe8d0101,0xfb400108,0xfe8e0101,0xfb400108,0xfe8f0101,
	0xfb400108,0xfe900101,0xfb400108,0xfe910101,0xfb400108,0xfe920101,0xfb400108,0xfe930101,
	0xfb400108,0xfe940101,0xfb400108,0xfe950101,0xfb400108,0xfe960101,0xfb400108,0xfe970101,
	0xfb400108,0xfe980101,0xfb400108,0xfe990101,0xfb400108,0xfe9a0101,0xfb400108,0xfe9c0101,
	0xfb400108,0xfe9d0101,0xfb400108,0xfe9e0101,0xfb400108,0xfeae0101,0xfb400108,0xfeb40101,
	0xfb400108,0xfebb0101,0xfb400108,0xfebc0101,0xfb400108,0xfed60101,0xfb400108,0xfee40101,
	0xfb400108,0xfeec0101,0xfb400108,0xfef90101,0xfb400108,0xff0a0101,0xfb400108,0xff100101,
	0xfb400108,0xff1e0101,0xfb400108,0xff370101,0xfb400108,0xff390101,0xfb400108,0xff3b0101,
	0xfb400108,0xff3c0101,0xfb400108,0xff3d0101,0xfb400108,0xff3e0101,0xfb400108,0xff3f0101,
	0xfb400108,0xff400101,0xfb400108,0xff410101,0xfb400108,0xff430101,0xfb400108,0xff460101,
	0xfb400108,0xff470101,0xfb400108,0xff480101,0xfb400108,0xff490101,0xfb400108,0xff4a0101,
	0xfb400108,0xff4b0101,0xfb400108,0xff4c0101,0xfb400108,0xff4d0101,0xfb400108,0xff4e0101,
	0xfb400108,0xff4f0101,0xfb400108,0xff520101,0xfb400108,0xff530101,0xfb400108,0xff560101,
	0xfb400108,0xff590101,0xfb400108,0xff5b0101,0xfb400108,0xff5c0101,0xfb400108,0xff5d0101,
	0xfb400108,0xff5e0101,0xfb400108,0xff600101,0xfb400108,0xff630101,0xfb400108,0xff640101,
	0xfb400108,0xff650101,0xfb400108,0xff660101,0xfb400108,0xff670101,0xfb400108,0xff6b0101,
	0xfb400108,0xff6c0101,0xfb400108,0xff6d0101,0xfb400108,0xff6f0101,0xfb400108,0xff700101,
	0xfb400108,0xff730101,0xfb400108,0xff750101,0xfb400108,0xff760101,0xfb400108,0xff770101,
	0xfb400108,0xff780101,0xfb400108,0xff7a0101,0xfb400108,0xff7b0101,0xfb400108,0xff7c0101,
	0xfb400108,0xff7d0101,0xfb400108,0xff7f0101,0xfb400108,0xff800101,0xfb400108,0xff820101,
	0xfb400108,0xff830101,0xfb400108,0xff840101,0xfb400108,0xff850101,0xfb400108,0xff860101,
	0xfb400108,0xff870101,0xfb400108,0xff880101,0xfb400108,0xff890101,0xfb400108,0xff8b0101,
	0xfb400108,0xff8d0101,0xfb400108,0xff8f0101,0xfb400108,0xff900101,0xfb400108,0xff910101,
	0xfb400108,0xff920101,0xfb400108,0xff930101,0xfb400108,0xff950101,0xfb400108,0xff960101,
	0xfb400108,0xff970101,0xfb400108,0xff980101,0xfb400108,0xff990101,0xfb400108,0xff9b0101,
	0xfb400108,0xff9c0101,0xfb400108,0xffa00101,0xfb400108,0xffa20101,0xfb400108,0xffa30101,
	0xfb400108,0xffa50101,0xfb400108,0xffa60101,0xfb400108,0xffa80101,0xfb400108,0xffa90101,
	0xfb400108,0xffaa0101,0xfb400108,0xffab0101,0xfb400108,0xffac0101,0xfb400108,0xffad0101,
	0xfb400108,0xffae0101,0xfb400108,0xffb10101,0xfb400108,0xffb30101,0xfb400108,0xffb40101,
	0xfb400108,0xffb50101,0xfb400108,0xffb60101,0xfb400108,0xffb70101,0xfb400108,0xffba0101,
	0xfb400108,0xffbb0101,0xfb400108,0xffbe0101,0xfb400108,0xffc00101,0xfb400108,0xffc20101,
	0xfb400108,0xffc30101,0xfb400108,0xffc40101,0xfb400108,0xffc60101,0xfb400108,0xffc70101,
	0xfb400108,0xffc80101,0xfb400108,0xffc90101,0xfb400108,0xffcb0101,0xfb400108,0xffcd0101,
	0xfb400108,0xffcf0101,0xfb400108,0xffd00101,0xfb400108,0xffd10101,0xfb400108,0xffd20101,
	0xfb400108,0xffd30101,0xfb400108,0xffd60101,0xfb400108,0xffd70101,0xfb400108,0xffd90101,
	0xfb400108,0xffda0101,0xfb400108,0xffdb0101,0xfb400108,0xffdc0101,0xfb400108,0xffdd0101,
	0xfb400108,0xffde0101,0xfb400108,0xffe20101,0xfb400108,0xffe30101,0xfb400108,0xffe40101,
	0xfb400108,0xffe70101,0xfb400108,0xffe80101,0xfb400108,0xffea0101,0xfb400108,0xffeb0101,
	0xfb400108,0xffec0101,0xfb400108,0xffed0101,0xfb400108,0xffef0101,0xfb400108,0xfff20101,
	0xfb400108,0xfff40101,0xfb400108,0xfff50101,0xfb400108,0xfff60101,0xfb400108,0xfff70101,
	0xfb400108,0xfff80101,0xfb400108,0xfff90101,0xfb400108,0xfffa0101,0xfb400108,0xfffd0101,
	0xfb400108,0xfffe0101,0xfb400108,0xffff0101,0xfb410108,0x80020101,0xfb410108,0x80070101,
	0xfb410108,0x80080101,0xfb410108,0x80090101,0xfb410108,0x800a0101,0xfb410108,0x800e0101,
	0xfb410108,0x800f0101,0xfb410108,0x80110101,0xfb410108,0x80130101,0xfb410108,0x801a0101,
	0xfb410108,0x801b0101,0xfb410108,0x801d0101,0xfb410108,0x801e0101,0xfb410108,0x801f0101,
	0xfb410108,0x80210101,0xfb410108,0x80230101,0xfb410108,0x80240101,0xfb410108,0x802b0101,
	0xfb410108,0x802c0101,0xfb410108,0x802d0101,0xfb410108,0x802e0101,0xfb410108,0x802f0101,
	0xfb410108,0x80300101,0xfb410108,0x80320101,0xfb410108,0x80340101,0xfb410108,0x80390101,
	0xfb410108,0x803a0101,0xfb410108,0x803c0101,0xfb410108,0x803e0101,0xfb410108,0x80400101,
	0xfb410108,0x80410101,0xfb410108,0x80440101,0xfb410108,0x80450101,0xfb410108,0x80470101,
	0xfb410108,0x80480101,0xfb410108,0x80490101,0xfb410108,0x804e0101,0xfb410108,0x804f0101,
	0xfb410108,0x80500101,0xfb410108,0x80510101,0xfb410108,0x80530101,0xfb410108,0x80550101,
	0xfb410108,0x80560101,0xfb410108,0x80570101,0xfb410108,0x80590101,0xfb410108,0x805b0101,
	0xfb410108,0x805c0101,0xfb410108,0x805d0101,0xfb410108,0x805e0101,0xfb410108,0x805f0101,
	0xfb410108,0x80600101,0xfb410108,0x80610101,0xfb410108,0x80620101,0xfb410108,0x80630101,
	0xfb410108,0x80640101,0xfb410108,0x80650101,0xfb410108,0x80660101,0xfb410108,0x80670101,
	0xfb410108,0x80680101,0xfb410108,0x806b0101,0xfb410108,0x806c0101,0xfb410108,0x806d0101,
	0xfb410108,0x806e0101,0xfb410108,0x806f0101,0xfb410108,0x80700101,0xfb410108,0x80720101,
	0xfb410108,0x80730101,0xfb410108,0x80740101,0xfb410108,0x80750101,0xfb410108,0x80760101,
	0xfb410108,0x80770101,0xfb410108,0x80780101,0xfb410108,0x80790101,0xfb410108,0x807a0101,
	0xfb410108,0x807b0101,0xfb410108,0x807c0101,0xfb410108,0x807d0101,0xfb410108,0x807e0101,
	0xfb410108,0x80810101,0xfb410108,0x80820101,0xfb410108,0x80850101,0xfb410108,0x80880101,
	0xfb410108,0x808a0101,0xfb410108,0x808d0101,0xfb410108,0x808e0101,0xfb410108,0x808f0101,
	0xfb410108,0x80900101,0xfb410108,0x80910101,0xfb410108,0x80920101,0xfb410108,0x80940101,
	0xfb410108,0x80950101,0xfb410108,0x80970101,0xfb410108,0x80990101,0xfb410108,0x809e0101,
	0xfb410108,0x80a30101,0xfb410108,0x80a60101,0xfb410108,0x80a70101,0xfb410108,0x80a80101,
	0xfb410108,0x80ac0101,0xfb410108,0x80b00101,0xfb410108,0x80b30101,0xfb410108,0x80b50101,
	0xfb410108,0x80b60101,0xfb410108,0x80b80101,0xfb410108,0x80b90101,0xfb410108,0x80bb0101,
	0xfb410108,0x80c50101,0xfb410108,0x80c70101,0xfb410108,0x80c80101,0xfb410108,0x80c90101,
	0xfb410108,0x80ca0101,0xfb410108,0x80cb0101,0xfb410108,0x80cf0101,0xfb410108,0x80d00101,
	0xfb410108,0x80d10101,0xfb410108,0x80d20101,0xfb410108,0x80d30101,0xfb410108,0x80d40101,
	0xfb410108,0x80d50101,0xfb410108,0x80d80101,0xfb410108,0x80df0101,0xfb410108,0x80e00101,
	0xfb410108,0x80e20101,0xfb410108,0x80e30101,0xfb410108,0x80e60101,0xfb410108,0x80ee0101,
	0xfb410108,0x80f50101,0xfb410108,0x80f70101,0xfb410108,0x80f90101,0xfb410108,0x80fb0101,
	0xfb410108,0x80fe0101,0xfb410108,0x80ff0101,0xfb410108,0x81000101,0xfb410108,0x81010101,
	0xfb410108,0x81030101,0xfb410108,0x81040101,0xfb410108,0x81050101,0xfb410108,0x81070101,
	0xfb410108,0x81080101,0xfb410108,0x810b0101,0xfb410108,0x810c0101,0xfb410108,0x81150101,
	0xfb410108,0x81170101,0xfb410108,0x81190101,0xfb410108,0x811b0101,0xfb410108,0x811c0101,
	0xfb410108,0x811d0101,0xfb410108,0x811f0101,0xfb410108,0x81200101,0xfb410108,0x81210101,
	0xfb410108,0x81220101,0xfb410108,0x81230101,0xfb410108,0x81240101,0xfb410108,0x81250101,
	0xfb410108,0x81260101,0xfb410108,0x81270101,0xfb410108,0x81280101,0xfb410108,0x81290101,
	0xfb410108,0x812a0101,0xfb410108,0x812b0101,0xfb410108,0x812d0101,0xfb410108,0x812e0101,
	0xfb410108,0x81300101,0xfb410108,0x81330101,0xfb410108,0x81340101,0xfb410108,0x81350101,
	0xfb410108,0x81370101,0xfb410108,0x81390101,0xfb410108,0x813a0101,0xfb410108,0x813b0101,
	0xfb410108,0x813c0101,0xfb410108,0x813d0101,0xfb410108,0x813f0101,0xfb410108,0x81400101,
	0xfb410108,0x81410101,0xfb410108,0x81420101,0xfb410108,0x81430101,0xfb410108,0x81440101,
	0xfb410108,0x81450101,0xfb410108,0x81470101,0xfb410108,0x81490101,0xfb410108,0x814d0101,
	0xfb410108,0x814e0101,0xfb410108,0x814f0101,0xfb410108,0x81520101,0xfb410108,0x81560101,
	0xfb410108,0x81570101,0xfb410108,0x81580101,0xfb410108,0x815b0101,0xfb410108,0x815c0101,
	0xfb410108,0x815d0101,0xfb410108,0x815e0101,0xfb410108,0x815f0101,0xfb410108,0x81610101,
	0xfb410108,0x81620101,0xfb410108,0x81630101,0xfb410108,0x81640101,0xfb410108,0x81660101,
	0xfb410108,0x81680101,0xfb410108,0x816a0101,0xfb410108,0x816b0101,0xfb410108,0x816c0101,
	0xfb410108,0x816f0101,0xfb410108,0x81720101,0xfb410108,0x81730101,0xfb410108,0x81750101,
	0xfb410108,0x81760101,0xfb410108,0x81770101,0xfb410108,0x81780101,0xfb410108,0x81810101,
	0xfb410108,0x81830101,0xfb410108,0x81840101,0xfb410108,0x81850101,0xfb410108,0x81860101,
	0xfb410108,0x81870101,0xfb410108,0x81890101,0xfb410108,0x818b0101,0xfb410108,0x818c0101,
	0xfb410108,0x818d0101,0xfb410108,0x818e0101,0xfb410108,0x81900101,0xfb410108,0x81920101,
	0xfb410108,0x81930101,0xfb410108,0x81940101,0xfb410108,0x81950101,0xfb410108,0x81960101,
	0xfb410108,0x81970101,0xfb410108,0x81990101,0xfb410108,0x819a0101,0xfb410108,0x819e0101,
	0xfb410108,0x819f0101,0xfb410108,0x81a00101,0xfb410108,0x81a10101,0xfb410108,0x81a20101,
	0xfb410108,0x81a40101,0xfb410108,0x81a50101,0xfb410108,0x81a70101,0xfb410108,0x81a90101,
	0xfb410108,0x81ab0101,0xfb410108,0x81ac0101,0xfb410108,0x81ad0101,0xfb410108,0x81ae0101,
	0xfb410108,0x81af0101,0xfb410108,0x81b00101,0xfb410108,0x81b10101,0xfb410108,0x81b20101,
	0xfb410108,0x81b40101,0xfb410108,0x81b50101,0xfb410108,0x81b60101,0xfb410108,0x81b70101,
	0xfb410108,0x81b80101,0xfb410108,0x81b90101,0xfb410108,0x81bc0101,0xfb410108,0x81bd0101,
	0xfb410108,0x81be0101,0xfb410108,0x81bf0101,0xfb410108,0x81c40101,0xfb410108,0x81c50101,
	0xfb410108,0x81c70101,0xfb410108,0x81c80101,0xfb410108,0x81c90101,0xfb410108,0x81cb0101,
	0xfb410108,0x81cd0101,0xfb410108,0x81ce0101,0xfb410108,0x81cf0101,0xfb410108,0x81d00101,
	0xfb410108,0x81d10101,0xfb410108,0x81d20101,0xfb410108,0x81d30101,0xfb410108,0x81d40101,
	0xfb410108,0x81d50101,0xfb410108,0x81d60101,0xfb410108,0x81d70101,0xfb410108,0x81d80101,
	0xfb410108,0x81d90101,0xfb410108,0x81da0101,0xfb410108,0x81db0101,0xfb410108,0x81dc0101,
	0xfb410108,0x81dd0101,0xfb410108,0x81de0101,0xfb410108,0x81df0101,0xfb410108,0x81e00101,
	0xfb410108,0x81e10101,0xfb410108,0x81e20101,0xfb410108,0x81e40101,0xfb410108,0x81e50101,
	0xfb410108,0x81e60101,0xfb410108,0x81e80101,0xfb410108,0x81e90101,0xfb410108,0x81eb0101,
	0xfb410108,0x81ee0101,0xfb410108,0x81ef0101,0xfb410108,0x81f00101,0xfb410108,0x81f10101,
	0xfb410108,0x81f20101,0xfb410108,0x81f50101,0xfb410108,0x81f60101,0xfb410108,0x81f70101,
	0xfb410108,0x81f80101,0xfb410108,0x81f90101,0xfb410108,0x81fa0101,0xfb410108,0x81fd0101,
	0xfb410108,0x81ff0101,0xfb410108,0x82030101,0xfb410108,0x82070101,0xfb410108,0x82080101,
	0xfb410108,0x82090101,0xfb410108,0x820a0101,0xfb410108,0x820b0101,0xfb410108,0x820e0101,
	0xfb410108,0x820f0101,0xfb410108,0x82110101,0xfb410108,0x82130101,0xfb410108,0x82150101,
	0xfb410108,0x82160101,0xfb410108,0x82170101,0xfb410108,0x82180101,0xfb410108,0x82190101,
	0xfb410108,0x821a0101,0xfb410108,0x821d0101,0xfb410108,0x82200101,0xfb410108,0x82240101,
	0xfb410108,0x82250101,0xfb410108,0x82260101,0xfb410108,0x82270101,0xfb410108,0x82290101,
	0xfb410108,0x822e0101,0xfb410108,0x82320101,0xfb410108,0x823a0101,0xfb410108,0x823c0101,
	0xfb410108,0x823d0101,0xfb410108,0x823f0101,0xfb410108,0x82400101,0xfb410108,0x82410101,
	0xfb410108,0x82420101,0xfb410108,0x82430101,0xfb410108,0x82450101,0xfb410108,0x82460101,
	0xfb410108,0x82480101,0xfb410108,0x824a0101,0xfb410108,0x824c0101,0xfb410108,0x824d0101,
	0xfb410108,0x824e0101,0xfb410108,0x82500101,0xfb410108,0x82510101,0xfb410108,0x82520101,
	0xfb410108,0x82530101,0xfb410108,0x82540101,0xfb410108,0x82550101,0xfb410108,0x82560101,
	0xfb410108,0x82570101,0xfb410108,0x82590101,0xfb410108,0x825b0101,0xfb410108,0x825c0101,
	0xfb410108,0x825d0101,0xfb410108,0x825e0101,0xfb410108,0x82600101,0xfb410108,0x82610101,
	0xfb410108,0x82620101,0xfb410108,0x82630101,0xfb410108,0x82640101,0xfb410108,0x82650101,
	0xfb410108,0x82660101,0xfb410108,0x82670101,0xfb410108,0x82690101,0xfb410108,0x826a0101,
	0xfb410108,0x826b0101,0xfb410108,0x826c0101,0xfb410108,0x826d0101,0xfb410108,0x82710101,
	0xfb410108,0x82750101,0xfb410108,0x82760101,0xfb410108,0x82770101,0xfb410108,0x82780101,
	0xfb410108,0x827b0101,0xfb410108,0x827c0101,0xfb410108,0x82800101,0xfb410108,0x82810101,
	0xfb410108,0x82830101,0xfb410108,0x82850101,0xfb410108,0x82860101,0xfb410108,0x82870101,
	0xfb410108,0x82890101,0xfb410108,0x828c0101,0xfb410108,0x82900101,0xfb410108,0x82930101,
	0xfb410108,0x82940101,0xfb410108,0x82950101,0xfb410108,0x82960101,0xfb410108,0x829a0101,
	0xfb410108,0x829b0101,0xfb410108,0x829e0101,0xfb410108,0x82a00101,0xfb410108,0x82a20101,
	0xfb410108,0x82a30101,0xfb410108,0x82a70101,0xfb410108,0x82b20101,0xfb410108,0x82b50101,
	0xfb410108,0x82b60101,0xfb410108,0x82ba0101,0xfb410108,0x82bb0101,0xfb410108,0x82bc0101,
	0xfb410108,0x82bf0101,0xfb410108,0x82c00101,0xfb410108,0x82c20101,0xfb410108,0x82c30101,
	0xfb410108,0x82c50101,0xfb410108,0x82c60101,0xfb410108,0x82c90101,0xfb410108,0x82d00101,
	0xfb410108,0x82d60101,0xfb410108,0x82d90101,0xfb410108,0x82da0101,0xfb410108,0x82dd0101,
	0xfb410108,0x82e20101,0xfb410108,0x82e70101,0xfb410108,0x82e80101,0xfb410108,0x82e90101,
	0xfb410108,0x82ea0101,0xfb410108,0x82ec0101,0xfb410108,0x82ed0101,0xfb410108,0x82ee0101,
	0xfb410108,0x82f00101,0xfb410108,0x82f20101,0xfb410108,0x82f30101,0xfb410108,0x82f50101,
	0xfb410108,0x82f60101,0xfb410108,0x82f80101,0xfb410108,0x82fa0101,0xfb410108,0x82fc0101,
	0xfb410108,0x82fd0101,0xfb410108,0x82fe0101,0xfb410108,0x82ff0101,0xfb410108,0x83000101,
	0xfb410108,0x830a0101,0xfb410108,0x830b0101,0xfb410108,0x830d0101,0xfb410108,0x83100101,
	0xfb410108,0x83120101,0xfb410108,0x83130101,0xfb410108,0x83160101,0xfb410108,0x83180101,
	0xfb410108,0x83190101,0xfb410108,0x831d0101,0xfb410108,0x831e0101,0xfb410108,0x831f0101,
	0xfb410108,0x83200101,0xfb410108,0x83210101,0xfb410108,0x83220101,0xfb410108,0x83230101,
	0xfb410108,0x83240101,0xfb410108,0x83250101,0xfb410108,0x83260101,0xfb410108,0x83290101,
	0xfb410108,0x832a0101,0xfb410108,0x832e0101,0xfb410108,0x83300101,0xfb410108,0x83320101,
	0xfb410108,0x83370101,0xfb410108,0x833b0101,0xfb410108,0x833d0101,0xfb410108,0x833e0101,
	0xfb410108,0x833f0101,0xfb410108,0x83410101,0xfb410108,0x83420101,0xfb410108,0x83440101,
	0xfb410108,0x83450101,0xfb410108,0x83480101,0xfb410108,0x834a0101,0xfb410108,0x834b0101,
	0xfb410108,0x834c0101,0xfb410108,0x834d0101,0xfb410108,0x834e0101,0xfb410108,0x83530101,
	0xfb410108,0x83550101,0xfb410108,0x83560101,0xfb410108,0x83570101,0xfb410108,0x83580101,
	0xfb410108,0x83590101,0xfb410108,0x835d0101,0xfb410108,0x83620101,0xfb410108,0x83700101,
	0xfb410108,0x83710101,0xfb410108,0x83720101,0xfb410108,0x83730101,0xfb410108,0x83740101,
	0xfb410108,0x83750101,0xfb410108,0x83760101,0xfb410108,0x83790101,0xfb410108,0x837a0101,
	0xfb410108,0x837e0101,0xfb410108,0x837f0101,0xfb410108,0x83800101,0xfb410108,0x83810101,
	0xfb410108,0x83820101,0xfb410108,0x83830101,0xfb410108,0x83840101,0xfb410108,0x83870101,
	0xfb410108,0x83880101,0xfb410108,0x838a0101,0xfb410108,0x838b0101,0xfb410108,0x838c0101,
	0xfb410108,0x838d0101,0xfb410108,0x838f0101,0xfb410108,0x83900101,0xfb410108,0x83910101,
	0xfb410108,0x83940101,0xfb410108,0x83950101,0xfb410108,0x83960101,0xfb410108,0x83970101,
	0xfb410108,0x83990101,0xfb410108,0x839a0101,0xfb410108,0x839d0101,0xfb410108,0x839f0101,
	0xfb410108,0x83a10101,0xfb410108,0x83a20101,0xfb410108,0x83a30101,0xfb410108,0x83a40101,
	0xfb410108,0x83a50101,0xfb410108,0x83a60101,0xfb410108,0x83a70101,0xfb410108,0x83ac0101,
	0xfb410108,0x83ad0101,0xfb410108,0x83ae0101,0xfb410108,0x83af0101,0xfb410108,0x83b50101,
	0xfb410108,0x83bb0101,0xfb410108,0x83be0101,0xfb410108,0x83bf0101,0xfb410108,0x83c20101,
	0xfb410108,0x83c30101,0xfb410108,0x83c40101,0xfb410108,0x83c60101,0xfb410108,0x83c80101,
	0xfb410108,0x83c90101,0xfb410108,0x83cb0101,0xfb410108,0x83cd0101,0xfb410108,0x83ce0101,
	0xfb410108,0x83d00101,0xfb410108,0x83d10101,0xfb410108,0x83d20101,0xfb410108,0x83d30101,
	0xfb410108,0x83d50101,0xfb410108,0x83d70101,0xfb410108,0x83d90101,0xfb410108,0x83da0101,
	0xfb410108,0x83db0101,0xfb410108,0x83de0101,0xfb410108,0x83e20101,0xfb410108,0x83e30101,
	0xfb410108,0x83e40101,0xfb410108,0x83e60101,0xfb410108,0x83e70101,0xfb410108,0x83e80101,
	0xfb410108,0x83eb0101,0xfb410108,0x83ec0101,0xfb410108,0x83ed0101,0xfb410108,0x83ee0101,
	0xfb410108,0x83ef0101,0xfb410108,0x83f30101,0xfb410108,0x83f40101,0xfb410108,0x83f50101,
	0xfb410108,0x83f60101,0xfb410108,0x83f70101,0xfb410108,0x83fa0101,0xfb410108,0x83fb0101,
	0xfb410108,0x83fc0101,0xfb410108,0x83fe0101,0xfb410108,0x83ff0101,0xfb410108,0x84000101,
	0xfb410108,0x84020101,0xfb410108,0x84050101,0xfb410108,0x84070101,0xfb410108,0x84080101,
	0xfb410108,0x84090101,0xfb410108,0x840a0101,0xfb410108,0x84100101,0xfb410108,0x84120101,
	0xfb410108,0x84130101,0xfb410108,0x84140101,0xfb410108,0x84150101,0xfb410108,0x84160101,
	0xfb410108,0x84170101,0xfb410108,0x84190101,0xfb410108,0x841a0101,0xfb410108,0x841b0101,
	0xfb410108,0x841e0101,0xfb410108,0x841f0101,0xfb410108,0x84200101,0xfb410108,0x84210101,
	0xfb410108,0x84220101,0xfb410108,0x84230101,0xfb410108,0x84290101,0xfb410108,0x842a0101,
	0xfb410108,0x842b0101,0xfb410108,0x842c0101,0xfb410108,0x842d0101,0xfb410108,0x842e0101,
	0xfb410108,0x842f0101,0xfb410108,0x84300101,0xfb410108,0x84320101,0xfb410108,0x84330101,
	0xfb410108,0x84340101,0xfb410108,0x84350101,0xfb410108,0x84360101,0xfb410108,0x84370101,
	0xfb410108,0x84390101,0xfb410108,0x843a0101,0xfb410108,0x843b0101,0xfb410108,0x843e0101,
	0xfb410108,0x843f0101,0xfb410108,0x84400101,0xfb410108,0x84410101,0xfb410108,0x84420101,
	0xfb410108,0x84430101,0xfb410108,0x84440101,0xfb410108,0x84450101,0xfb410108,0x84470101,
	0xfb410108,0x84480101,0xfb410108,0x84490101,0xfb410108,0x844a0101,0xfb410108,0x844b0101,
	0xfb410108,0x844c0101,0xfb410108,0x844d0101,0xfb410108,0x844e0101,0xfb410108,0x844f0101,
	0xfb410108,0x84500101,0xfb410108,0x84520101,0xfb410108,0x84530101,0xfb410108,0x84540101,
	0xfb410108,0x84550101,0xfb410108,0x84560101,0xfb410108,0x84580101,0xfb410108,0x845d0101,
	0xfb410108,0x845e0101,0xfb410108,0x845f0101,0xfb410108,0x84600101,0xfb410108,0x84620101,
	0xfb410108,0x84640101,0xfb410108,0x84650101,0xfb410108,0x84660101,0xfb410108,0x84670101,
	0xfb410108,0x84680101,0xfb410108,0x846a0101,0xfb410108,0x846e0101,0xfb410108,0x846f0101,
	0xfb410108,0x84700101,0xfb410108,0x84720101,0xfb410108,0x84740101,0xfb410108,0x84770101,
	0xfb410108,0x84790101,0xfb410108,0x847b0101,0xfb410108,0x847c0101,0xfb410108,0x847d0101,
	0xfb410108,0x847e0101,0xfb410108,0x847f0101,0xfb410108,0x84800101,0xfb410108,0x84810101,
	0xfb410108,0x84830101,0xfb410108,0x84840101,0xfb410108,0x84850101,0xfb410108,0x84860101,
	0xfb410108,0x848a0101,0xfb410108,0x848d0101,0xfb410108,0x848f0101,0xfb410108,0x84900101,
	0xfb410108,0x84910101,0xfb410108,0x84920101,0xfb410108,0x84930101,0xfb410108,0x84940101,
	0xfb410108,0x84950101,0xfb410108,0x84960101,0xfb410108,0x84980101,0xfb410108,0x849a0101,
	0xfb410108,0x849b0101,0xfb410108,0x849d0101,0xfb410108,0x849e0101,0xfb410108,0x849f0101,
	0xfb410108,0x84a00101,0xfb410108,0x84a20101,0xfb410108,0x84a30101,0xfb410108,0x84a40101,
	0xfb410108,0x84a50101,0xfb410108,0x84a60101,0xfb410108,0x84a70101,0xfb410108,0x84a80101,
	0xfb410108,0x84a90101,0xfb410108,0x84aa0101,0xfb410108,0x84ab0101,0xfb410108,0x84ac0101,
	0xfb410108,0x84ad0101,0xfb410108,0x84ae0101,0xfb410108,0x84b00101,0xfb410108,0x84b10101,
	0xfb410108,0x84b30101,0xfb410108,0x84b50101,0xfb410108,0x84b60101,0xfb410108,0x84b70101,
	0xfb410108,0x84bb0101,0xfb410108,0x84bc0101,0xfb410108,0x84be0101,0xfb410108,0x84c00101,
	0xfb410108,0x84c20101,0xfb410108,0x84c30101,0xfb410108,0x84c50101,0xfb410108,0x84c60101,
	0xfb410108,0x84c70101,0xfb410108,0x84c80101,0xfb410108,0x84cb0101,0xfb410108,0x84cc0101,
	0xfb410108,0x84ce0101,0xfb410108,0x84cf0101,0xfb410108,0x84d20101,0xfb410108,0x84d40101,
	0xfb410108,0x84d50101,0xfb410108,0x84d70101,0xfb410108,0x84d80101,0xfb410108,0x84d90101,
	0xfb410108,0x84da0101,0xfb410108,0x84db0101,0xfb410108,0x84dc0101,0xfb410108,0x84de0101,
	0xfb410108,0x84e10101,0xfb410108,0x84e20101,0xfb410108,0x84e40101,0xfb410108,0x84e70101,
	0xfb410108,0x84e80101,0xfb410108,0x84e90101,0xfb410108,0x84ea0101,0xfb410108,0x84eb0101,
	0xfb410108,0x84ed0101,0xfb410108,0x84ee0101,0xfb410108,0x84ef0101,0xfb410108,0x84f10101,
	0xfb410108,0x84f20101,0xfb410108,0x84f30101,0xfb410108,0x84f40101,0xfb410108,0x84f50101,
	0xfb410108,0x84f60101,0xfb410108,0x84f70101,0xfb410108,0x84f80101,0xfb410108,0x84f90101,
	0xfb410108,0x84fa0101,0xfb410108,0x84fb0101,0xfb410108,0x84fd0101,0xfb410108,0x84fe0101,
	0xfb410108,0x85000101,0xfb410108,0x85010101,0xfb410108,0x85020101,0xfb410108,0x85030101,
	0xfb410108,0x85040101,0xfb410108,0x85050101,0xfb410108,0x85060101,0xfb410108,0x85070101,
	0xfb410108,0x85080101,0xfb410108,0x85090101,0xfb410108,0x850a0101,0xfb410108,0x850b0101,
	0xfb410108,0x850d0101,0xfb410108,0x850e0101,0xfb410108,0x850f0101,0xfb410108,0x85100101,
	0xfb410108,0x85120101,0xfb410108,0x85140101,0xfb410108,0x85150101,0xfb410108,0x85160101,
	0xfb410108,0x85180101,0xfb410108,0x85190101,0xfb410108,0x851b0101,0xfb410108,0x851c0101,
	0xfb410108,0x851d0101,0xfb410108,0x851e0101,0xfb410108,0x85200101,0xfb410108,0x85220101,
	0xfb410108,0x85230101,0xfb410108,0x85240101,0xfb410108,0x85250101,0xfb410108,0x85260101,
	0xfb410108,0x85270101,0xfb410108,0x85280101,0xfb410108,0x85290101,0xfb410108,0x852a0101,
	0xfb410108,0x852d0101,0xfb410108,0x852e0101,0xfb410108,0x852f0101,0xfb410108,0x85300101,
	0xfb410108,0x85310101,0xfb410108,0x85320101,0xfb410108,0x85330101,0xfb410108,0x85340101,
	0xfb410108,0x85350101,0xfb410108,0x85360101,0xfb410108,0x853e0101,0xfb410108,0x853f0101,
	0xfb410108,0x85400101,0xfb410108,0x85410101,0xfb410108,0x85420101,0xfb410108,0x85440101,
	0xfb410108,0x85450101,0xfb410108,0x85460101,0xfb410108,0x85470101,0xfb410108,0x854b0101,
	0xfb410108,0x854c0101,0xfb410108,0x854d0101,0xfb410108,0x854e0101,0xfb410108,0x854f0101,
	0xfb410108,0x85500101,0xfb410108,0x85510101,0xfb410108,0x85520101,0xfb410108,0x85530101,
	0xfb410108,0x85540101,0xfb410108,0x85550101,0xfb410108,0x85570101,0xfb410108,0x85580101,
	0xfb410108,0x855a0101,0xfb410108,0x855b0101,0xfb410108,0x855c0101,0xfb410108,0x855d0101,
	0xfb410108,0x855f0101,0xfb410108,0x85600101,0xfb410108,0x85610101,0xfb410108,0x85620101,
	0xfb410108,0x85630101,0xfb410108,0x85650101,0xfb410108,0x85660101,0xfb410108,0x85670101,
	0xfb410108,0x85690101,0xfb410108,0x856a0101,0xfb410108,0x856b0101,0xfb410108,0x856c0101,
	0xfb410108,0x856d0101,0xfb410108,0x856e0101,0xfb410108,0x856f0101,0xfb410108,0x85700101,
	0xfb410108,0x85710101,0xfb410108,0x85730101,0xfb410108,0x85750101,0xfb410108,0x85760101,
	0xfb410108,0x85770101,0xfb410108,0x85780101,0xfb410108,0x857c0101,0xfb410108,0x857d0101,
	0xfb410108,0x857f0101,0xfb410108,0x85800101,0xfb410108,0x85810101,0xfb410108,0x85820101,
	0xfb410108,0x85830101,0xfb410108,0x85860101,0xfb410108,0x85880101,0xfb410108,0x85890101,
	0xfb410108,0x858a0101,0xfb410108,0x858b0101,0xfb410108,0x858c0101,0xfb410108,0x858d0101,
	0xfb410108,0x858e0101,0xfb410108,0x85900101,0xfb410108,0x85910101,0xfb410108,0x85920101,
	0xfb410108,0x85930101,0xfb410108,0x85940101,0xfb410108,0x85950101,0xfb410108,0x85960101,
	0xfb410108,0x85970101,0xfb410108,0x85980101,0xfb410108,0x85990101,0xfb410108,0x859a0101,
	0xfb410108,0x859d0101,0xfb410108,0x859e0101,0xfb410108,0x859f0101,0xfb410108,0x85a00101,
	0xfb410108,0x85a10101,0xfb410108,0x85a20101,0xfb410108,0x85a30101,0xfb410108,0x85a50101,
	0xfb410108,0x85a60101,0xfb410108,0x85a70101,0xfb410108,0x85a90101,0xfb410108,0x85ab0101,
	0xfb410108,0x85ac0101,0xfb410108,0x85ad0101,0xfb410108,0x85b10101,0xfb410108,0x85b20101,
	0xfb410108,0x85b30101,0xfb410108,0x85b40101,0xfb410108,0x85b50101,0xfb410108,0x85b60101,
	0xfb410108,0x85b80101,0xfb410108,0x85ba0101,0xfb410108,0x85bb0101,0xfb410108,0x85bc0101,
	0xfb410108,0x85bd0101,0xfb410108,0x85be0101,0xfb410108,0x85bf0101,0xfb410108,0x85c00101,
	0xfb410108,0x85c20101,0xfb410108,0x85c30101,0xfb410108,0x85c40101,0xfb410108,0x85c50101,
	0xfb410108,0x85c60101,0xfb410108,0x85c70101,0xfb410108,0x85c80101,0xfb410108,0x85ca0101,
	0xfb410108,0x85cb0101,0xfb410108,0x85cc0101,0xfb410108,0x85cd0101,0xfb410108,0x85ce0101,
	0xfb410108,0x85d10101,0xfb410108,0x85d20101,0xfb410108,0x85d40101,0xfb410108,0x85d60101,
	0xfb410108,0x85d70101,0xfb410108,0x85d80101,0xfb410108,0x85d90101,0xfb410108,0x85da0101,
	0xfb410108,0x85db0101,0xfb410108,0x85dd0101,0xfb410108,0x85de0101,0xfb410108,0x85df0101,
	0xfb410108,0x85e00101,0xfb410108,0x85e10101,0xfb410108,0x85e20101,0xfb410108,0x85e30101,
	0xfb410108,0x85e50101,0xfb410108,0x85e60101,0xfb410108,0x85e70101,0xfb410108,0x85e80101,
	0xfb410108,0x85ea0101,0xfb410108,0x85eb0101,0xfb410108,0x85ec0101,0xfb410108,0x85ed0101,
	0xfb410108,0x85ee0101,0xfb410108,0x85ef0101,0xfb410108,0x85f00101,0xfb410108,0x85f10101,
	0xfb410108,0x85f20101,0xfb410108,0x85f30101,0xfb410108,0x85f40101,0xfb410108,0x85f50101,
	0xfb410108,0x85f60101,0xfb410108,0x85f70101,0xfb410108,0x85f80101,0xfb410108,0x85f90101,
	0xfb410108,0x85fa0101,0xfb410108,0x85fc0101,0xfb410108,0x85fd0101,0xfb410108,0x85fe0101,
	0xfb410108,0x86000101,0xfb410108,0x86010101,0xfb410108,0x86020101,0xfb410108,0x86030101,
	0xfb410108,0x86040101,0xfb410108,0x86060101,0xfb410108,0x86070101,0xfb410108,0x86080101,
	0xfb410108,0x86090101,0xfb410108,0x860a0101,0xfb410108,0x860b0101,0xfb410108,0x860c0101,
	0xfb410108,0x860d0101,0xfb410108,0x860e0101,0xfb410108,0x860f0101,0xfb410108,0x86100101,
	0xfb410108,0x86120101,0xfb410108,0x86130101,0xfb410108,0x86140101,0xfb410108,0x86150101,
	0xfb410108,0x86170101,0xfb410108,0x86180101,0xfb410108,0x86190101,0xfb410108,0x861a0101,
	0xfb410108,0x861b0101,0xfb410108,0x861c0101,0xfb410108,0x861d0101,0xfb410108,0x861e0101,
	0xfb410108,0x861f0101,0xfb410108,0x86200101,0xfb410108,0x86210101,0xfb410108,0x86220101,
	0xfb410108,0x86230101,0xfb410108,0x86240101,0xfb410108,0x86250101,0xfb410108,0x86260101,
	0xfb410108,0x86280101,0xfb410108,0x862a0101,0xfb410108,0x862b0101,0xfb410108,0x862c0101,
	0xfb410108,0x862d0101,0xfb410108,0x862e0101,0xfb410108,0x862f0101,0xfb410108,0x86300101,
	0xfb410108,0x86310101,0xfb410108,0x86320101,0xfb410108,0x86330101,0xfb410108,0x86340101,
	0xfb410108,0x86350101,0xfb410108,0x86360101,0xfb410108,0x86370101,0xfb410108,0x86390101,
	0xfb410108,0x863a0101,0xfb410108,0x863b0101,0xfb410108,0x863d0101,0xfb410108,0x863e0101,
	0xfb410108,0x863f0101,0xfb410108,0x86400101,0xfb410108,0x86410101,0xfb410108,0x86420101,
	0xfb410108,0x86430101,0xfb410108,0x86440101,0xfb410108,0x86450101,0xfb410108,0x86460101,
	0xfb410108,0x86470101,0xfb410108,0x86480101,0xfb410108,0x86490101,0xfb410108,0x864a0101,
	0xfb410108,0x864b0101,0xfb410108,0x864c0101,0xfb410108,0x86520101,0xfb410108,0x86530101,
	0xfb410108,0x86550101,0xfb410108,0x86560101,0xfb410108,0x86570101,0xfb410108,0x86580101,
	0xfb410108,0x86590101,0xfb410108,0x865b0101,0xfb410108,0x865c0101,0xfb410108,0x865d0101,
	0xfb410108,0x865f0101,0xfb410108,0x86600101,0xfb410108,0x86610101,0xfb410108,0x86630101,
	0xfb410108,0x86640101,0xfb410108,0x86650101,0xfb410108,0x86660101,0xfb410108,0x86670101,
	0xfb410108,0x86680101,0xfb410108,0x86690101,0xfb410108,0x866a0101,0xfb410108,0x866d0101,
	0xfb410108,0x866f0101,0xfb410108,0x86700101,0xfb410108,0x86720101,0xfb410108,0x86730101,
	0xfb410108,0x86740101,0xfb410108,0x86750101,0xfb410108,0x86760101,0xfb410108,0x86770101,
	0xfb410108,0x86780101,0xfb410108,0x86830101,0xfb410108,0x86840101,0xfb410108,0x86850101,
	0xfb410108,0x86860101,0xfb410108,0x86870101,0xfb410108,0x86880101,0xfb410108,0x86890101,
	0xfb410108,0x868e0101,0xfb410108,0x868f0101,0xfb410108,0x86900101,0xfb410108,0x86910101,
	0xfb410108,0x86920101,0xfb410108,0x86940101,0xfb410108,0x86960101,0xfb410108,0x86970101,
	0xfb410108,0x86980101,0xfb410108,0x86990101,0xfb410108,0x869a0101,0xfb410108,0x869b0101,
	0xfb410108,0x869e0101,0xfb410108,0x869f0101,0xfb410108,0x86a00101,0xfb410108,0x86a10101,
	0xfb410108,0x86a20101,0xfb410108,0x86a50101,0xfb410108,0x86a60101,0xfb410108,0x86ab0101,
	0xfb410108,0x86ad0101,0xfb410108,0x86ae0101,0xfb410108,0x86b20101,0xfb410108,0x86b30101,
	0xfb410108,0x86b70101,0xfb410108,0x86b80101,0xfb410108,0x86b90101,0xfb410108,0x86bb0101,
	0xfb410108,0x86bc0101,0xfb410108,0x86bd0101,0xfb410108,0x86be0101,0xfb410108,0x86bf0101,
	0xfb410108,0x86c10101,0xfb410108,0x86c20101,0xfb410108,0x86c30101,0xfb410108,0x86c50101,
	0xfb410108,0x86c80101,0xfb410108,0x86cc0101,0xfb410108,0x86cd0101,0xfb410108,0x86d20101,
	0xfb410108,0x86d30101,0xfb410108,0x86d50101,0xfb410108,0x86d60101,0xfb410108,0x86d70101,
	0xfb410108,0x86da0101,0xfb410108,0x86dc0101,0xfb410108,0x86dd0101,0xfb410108,0x86e00101,
	0xfb410108,0x86e10101,0xfb410108,0x86e20101,0xfb410108,0x86e30101,0xfb410108,0x86e50101,
	0xfb410108,0x86e60101,0xfb410108,0x86e70101,0xfb410108,0x86e80101,0xfb410108,0x86ea0101,
	0xfb410108,0x86eb0101,0xfb410108,0x86ec0101,0xfb410108,0x86ef0101,0xfb410108,0x86f50101,
	0xfb410108,0x86f60101,0xfb410108,0x86f70101,0xfb410108,0x86fa0101,0xfb410108,0x86fb0101,
	0xfb410108,0x86fc0101,0xfb410108,0x86fd0101,0xfb410108,0x86ff0101,0xfb410108,0x87010101,
	0xfb410108,0x87040101,0xfb410108,0x87050101,0xfb410108,0x87060101,0xfb410108,0x870b0101,
	0xfb410108,0x870c0101,0xfb410108,0x870e0101,0xfb410108,0x870f0101,0xfb410108,0x87100101,
	0xfb410108,0x87110101,0xfb410108,0x87140101,0xfb410108,0x87160101,0xfb410108,0x87190101,
	0xfb410108,0x871b0101,0xfb410108,0x871d0101,0xfb410108,0x871f0101,0xfb410108,0x87200101,
	0xfb410108,0x87240101,0xfb410108,0x87260101,0xfb410108,0x87270101,0xfb410108,0x87280101,
	0xfb410108,0x872a0101,0xfb410108,0x872b0101,0xfb410108,0x872c0101,0xfb410108,0x872d0101,
	0xfb410108,0x872f0101,0xfb410108,0x87300101,0xfb410108,0x87320101,0xfb410108,0x87330101,
	0xfb410108,0x87350101,0xfb410108,0x87360101,0xfb410108,0x87380101,0xfb410108,0x87390101,
	0xfb410108,0x873a0101,0xfb410108,0x873c0101,0xfb410108,0x873d0101,0xfb410108,0x87400101,
	0xfb410108,0x87410101,0xfb410108,0x87420101,0xfb410108,0x87430101,0xfb410108,0x87440101,
	0xfb410108,0x87450101,0xfb410108,0x87460101,0xfb410108,0x874a0101,0xfb410108,0x874b0101,
	0xfb410108,0x874d0101,0xfb410108,0x874f0101,0xfb410108,0x87500101,0xfb410108,0x87510101,
	0xfb410108,0x87520101,0xfb410108,0x87540101,0xfb410108,0x87550101,0xfb410108,0x87560101,
	0xfb410108,0x87580101,0xfb410108,0x875a0101,0xfb410108,0x875b0101,0xfb410108,0x875c0101,
	0xfb410108,0x875d0101,0xfb410108,0x875e0101,0xfb410108,0x875f0101,0xfb410108,0x87610101,
	0xfb410108,0x87620101,0xfb410108,0x87660101,0xfb410108,0x87670101,0xfb410108,0x87680101,
	0xfb410108,0x87690101,0xfb410108,0x876a0101,0xfb410108,0x876b0101,0xfb410108,0x876c0101,
	0xfb410108,0x876d0101,0xfb410108,0x876f0101,0xfb410108,0x87710101,0xfb410108,0x87720101,
	0xfb410108,0x87730101,0xfb410108,0x87750101,0xfb410108,0x87770101,0xfb410108,0x87780101,
	0xfb410108,0x87790101,0xfb410108,0x877a0101,0xfb410108,0x877f0101,0xfb410108,0x87800101,
	0xfb410108,0x87810101,0xfb410108,0x87840101,0xfb410108,0x87860101,0xfb410108,0x87870101,
	0xfb410108,0x87890101,0xfb410108,0x878a0101,0xfb410108,0x878c0101,0xfb410108,0x878e0101,
	0xfb410108,0x878f0101,0xfb410108,0x87900101,0xfb410108,0x87910101,0xfb410108,0x87920101,
	0xfb410108,0x87940101,0xfb410108,0x87950101,0xfb410108,0x87960101,0xfb410108,0x87980101,
	0xfb410108,0x87990101,0xfb410108,0x879a0101,0xfb410108,0x879b0101,0xfb410108,0x879c0101,
	0xfb410108,0x879d0101,0xfb410108,0x879e0101,0xfb410108,0x87a00101,0xfb410108,0x87a10101,
	0xfb410108,0x87a20101,0xfb410108,0x87a30101,0xfb410108,0x87a40101,0xfb410108,0x87a50101,
	0xfb410108,0x87a60101,0xfb410108,0x87a70101,0xfb410108,0x87a90101,0xfb410108,0x87aa0101,
	0xfb410108,0x87ae0101,0xfb410108,0x87b00101,0xfb410108,0x87b10101,0xfb410108,0x87b20101,
	0xfb410108,0x87b40101,0xfb410108,0x87b60101,0xfb410108,0x87b70101,0xfb410108,0x87b80101,
	0xfb410108,0x87b90101,0xfb410108,0x87bb0101,0xfb410108,0x87bc0101,0xfb410108,0x87be0101,
	0xfb410108,0x87bf0101,0xfb410108,0x87c10101,0xfb410108,0x87c20101,0xfb410108,0x87c30101,
	0xfb410108,0x87c40101,0xfb410108,0x87c50101,0xfb410108,0x87c70101,0xfb410108,0x87c80101,
	0xfb410108,0x87c90101,0xfb410108,0x87cc0101,0xfb410108,0x87cd0101,0xfb410108,0x87ce0101,
	0xfb410108,0x87cf0101,0xfb410108,0x87d00101,0xfb410108,0x87d40101,0xfb410108,0x87d50101,
	0xfb410108,0x87d60101,0xfb410108,0x87d70101,0xfb410108,0x87d80101,0xfb410108,0x87d90101,
	0xfb410108,0x87da0101,0xfb410108,0x87dc0101,0xfb410108,0x87dd0101,0xfb410108,0x87de0101,
	0xfb410108,0x87df0101,0xfb410108,0x87e10101,0xfb410108,0x87e20101,0xfb410108,0x87e30101,
	0xfb410108,0x87e40101,0xfb410108,0x87e60101,0xfb410108,0x87e70101,0xfb410108,0x87e80101,
	0xfb410108,0x87e90101,0xfb410108,0x87eb0101,0xfb410108,0x87ec0101,0xfb410108,0x87ed0101,
	0xfb410108,0x87ef0101,0xfb410108,0x87f00101,0xfb410108,0x87f10101,0xfb410108,0x87f20101,
	0xfb410108,0x87f30101,0xfb410108,0x87f40101,0xfb410108,0x87f50101,0xfb410108,0x87f60101,
	0xfb410108,0x87f70101,0xfb410108,0x87f80101,0xfb410108,0x87fa0101,0xfb410108,0x87fb0101,
	0xfb410108,0x87fc0101,0xfb410108,0x87fd0101,0xfb410108,0x87ff0101,0xfb410108,0x88000101,
	0xfb410108,0x88010101,0xfb410108,0x88020101,0xfb410108,0x88040101,0xfb410108,0x88050101,
	0xfb410108,0x88060101,0xfb410108,0x88070101,0xfb410108,0x88080101,0xfb410108,0x88090101,
	0xfb410108,0x880b0101,0xfb410108,0x880c0101,0xfb410108,0x880d0101,0xfb410108,0x880e0101,
	0xfb410108,0x880f0101,0xfb410108,0x88100101,0xfb410108,0x88110101,0xfb410108,0x88120101,
	0xfb410108,0x88140101,0xfb410108,0x88170101,0xfb410108,0x88180101,0xfb410108,0x88190101,
	0xfb410108,0x881a0101,0xfb410108,0x881c0101,0xfb410108,0x881d0101,0xfb410108,0x881e0101,
	0xfb410108,0x881f0101,0xfb410108,0x88200101,0xfb410108,0x88230101,0xfb410108,0x88240101,
	0xfb410108,0x88250101,0xfb410108,0x88260101,0xfb410108,0x88270101,0xfb410108,0x88280101,
	0xfb410108,0x88290101,0xfb410108,0x882a0101,0xfb410108,0x882b0101,0xfb410108,0x882c0101,
	0xfb410108,0x882d0101,0xfb410108,0x882e0101,0xfb410108,0x882f0101,0xfb410108,0x88300101,
	0xfb410108,0x88310101,0xfb410108,0x88330101,0xfb410108,0x88340101,0xfb410108,0x88350101,
	0xfb410108,0x88360101,0xfb410108,0x88370101,0xfb410108,0x88380101,0xfb410108,0x883a0101,
	0xfb410108,0x883b0101,0xfb410108,0x883d0101,0xfb410108,0x883e0101,0xfb410108,0x883f0101,
	0xfb410108,0x88410101,0xfb410108,0x88420101,0xfb410108,0x88430101,0xfb410108,0x88460101,
	0xfb410108,0x88470101,0xfb410108,0x88480101,0xfb410108,0x88490101,0xfb410108,0x884a0101,
	0xfb410108,0x884b0101,0xfb410108,0x884e0101,0xfb410108,0x884f0101,0xfb410108,0x88500101,
	0xfb410108,0x88510101,0xfb410108,0x88520101,0xfb410108,0x88530101,0xfb410108,0x88550101,
	0xfb410108,0x88560101,0xfb410108,0x88580101,0xfb410108,0x885a0101,0xfb410108,0x885b0101,
	0xfb410108,0x885c0101,0xfb410108,0x885d0101,0xfb410108,0x885e0101,0xfb410108,0x885f0101,
	0xfb410108,0x88600101,0xfb410108,0x88660101,0xfb410108,0x88670101,0xfb410108,0x886a0101,
	0xfb410108,0x886d0101,0xfb410108,0x886f0101,0xfb410108,0x88710101,0xfb410108,0x88730101,
	0xfb410108,0x88740101,0xfb410108,0x88750101,0xfb410108,0x88760101,0xfb410108,0x88780101,
	0xfb410108,0x88790101,0xfb410108,0x887a0101,0xfb410108,0x887b0101,0xfb410108,0x887c0101,
	0xfb410108,0x88800101,0xfb410108,0x88830101,0xfb410108,0x88860101,0xfb410108,0x88870101,
	0xfb410108,0x88890101,0xfb410108,0x888a0101,0xfb410108,0x888c0101,0xfb410108,0x888e0101,
	0xfb410108,0x888f0101,0xfb410108,0x88900101,0xfb410108,0x88910101,0xfb410108,0x88930101,
	0xfb410108,0x88940101,0xfb410108,0x88950101,0xfb410108,0x88970101,0xfb410108,0x88980101,
	0xfb410108,0x88990101,0xfb410108,0x889a0101,0xfb410108,0x889b0101,0xfb410108,0x889d0101,
	0xfb410108,0x889e0101,0xfb410108,0x889f0101,0xfb410108,0x88a00101,0xfb410108,0x88a10101,
	0xfb410108,0x88a30101,0xfb410108,0x88a50101,0xfb410108,0x88a60101,0xfb410108,0x88a70101,
	0xfb410108,0x88a80101,0xfb410108,0x88a90101,0xfb410108,0x88aa0101,0xfb410108,0x88ac0101,
	0xfb410108,0x88ae0101,0xfb410108,0x88af0101,0xfb410108,0x88b00101,0xfb410108,0x88b20101,
	0xfb410108,0x88b30101,0xfb410108,0x88b40101,0xfb410108,0x88b50101,0xfb410108,0x88b60101,
	0xfb410108,0x88b80101,0xfb410108,0x88b90101,0xfb410108,0x88ba0101,0xfb410108,0x88bb0101,
	0xfb410108,0x88bd0101,0xfb410108,0x88be0101,0xfb410108,0x88bf0101,0xfb410108,0x88c00101,
	0xfb410108,0x88c30101,0xfb410108,0x88c40101,0xfb410108,0x88c70101,0xfb410108,0x88c80101,
	0xfb410108,0x88ca0101,0xfb410108,0x88cb0101,0xfb410108,0x88cc0101,0xfb410108,0x88cd0101,
	0xfb410108,0x88cf0101,0xfb410108,0x88d00101,0xfb410108,0x88d10101,0xfb410108,0x88d30101,
	0xfb410108,0x88d60101,0xfb410108,0x88d70101,0xfb410108,0x88da0101,0xfb410108,0x88db0101,
	0xfb410108,0x88dc0101,0xfb410108,0x88dd0101,0xfb410108,0x88de0101,0xfb410108,0x88e00101,
	0xfb410108,0x88e10101,0xfb410108,0x88e60101,0xfb410108,0x88e70101,0xfb410108,0x88e90101,
	0xfb410108,0x88ea0101,0xfb410108,0x88eb0101,0xfb410108,0x88ec0101,0xfb410108,0x88ed0101,
	0xfb410108,0x88ee0101,0xfb410108,0x88ef0101,0xfb410108,0x88f20101,0xfb410108,0x88f50101,
	0xfb410108,0x88f60101,0xfb410108,0x88f70101,0xfb410108,0x88fa0101,0xfb410108,0x88fb0101,
	0xfb410108,0x88fd0101,0xfb410108,0x88ff0101,0xfb410108,0x89000101,0xfb410108,0x89010101,
	0xfb410108,0x89030101,0xfb410108,0x89040101,0xfb410108,0x89050101,0xfb410108,0x89060101,
	0xfb410108,0x89070101,0xfb410108,0x89080101,0xfb410108,0x89090101,0xfb410108,0x890b0101,
	0xfb410108,0x890c0101,0xfb410108,0x890d0101,0xfb410108,0x890e0101,0xfb410108,0x890f0101,
	0xfb410108,0x89110101,0xfb410108,0x89140101,0xfb410108,0x89150101,0xfb410108,0x89160101,
	0xfb410108,0x89170101,0xfb410108,0x89180101,0xfb410108,0x891c0101,0xfb410108,0x891d0101,
	0xfb410108,0x891e0101,0xfb410108,0x891f0101,0xfb410108,0x89200101,0xfb410108,0x89220101,
	0xfb410108,0x89230101,0xfb410108,0x89240101,0xfb410108,0x89260101,0xfb410108,0x89270101,
	0xfb410108,0x89280101,0xfb410108,0x89290101,0xfb410108,0x892c0101,0xfb410108,0x892d0101,
	0xfb410108,0x892e0101,0xfb410108,0x892f0101,0xfb410108,0x89310101,0xfb410108,0x89320101,
	0xfb410108,0x89330101,0xfb410108,0x89350101,0xfb410108,0x89370101,0xfb410108,0x89380101,
	0xfb410108,0x89390101,0xfb410108,0x893a0101,0xfb410108,0x893b0101,0xfb410108,0x893c0101,
	0xfb410108,0x893d0101,0xfb410108,0x893e0101,0xfb410108,0x893f0101,0xfb410108,0x89400101,
	0xfb410108,0x89420101,0xfb410108,0x89430101,0xfb410108,0x89450101,0xfb410108,0x89460101,
	0xfb410108,0x89470101,0xfb410108,0x89480101,0xfb410108,0x89490101,0xfb410108,0x894a0101,
	0xfb410108,0x894b0101,0xfb410108,0x894c0101,0xfb410108,0x894d0101,0xfb410108,0x894e0101,
	0xfb410108,0x894f0101,0xfb410108,0x89500101,0xfb410108,0x89510101,0xfb410108,0x89520101,
	0xfb410108,0x89530101,0xfb410108,0x89540101,0xfb410108,0x89550101,0xfb410108,0x89560101,
	0xfb410108,0x89570101,0xfb410108,0x89580101,0xfb410108,0x89590101,0xfb410108,0x895a0101,
	0xfb410108,0x895b0101,0xfb410108,0x895c0101,0xfb410108,0x895d0101,0xfb410108,0x89600101,
	0xfb410108,0x89610101,0xfb410108,0x89620101,0xfb410108,0x89630101,0xfb410108,0x89640101,
	0xfb410108,0x89650101,0xfb410108,0x89670101,0xfb410108,0x89680101,0xfb410108,0x89690101,
	0xfb410108,0x896a0101,0xfb410108,0x896b0101,0xfb410108,0x896c0101,0xfb410108,0x896d0101,
	0xfb410108,0x896e0101,0xfb410108,0x896f0101,0xfb410108,0x89700101,0xfb410108,0x89710101,
	0xfb410108,0x89720101,0xfb410108,0x89730101,0xfb410108,0x89740101,0xfb410108,0x89750101,
	0xfb410108,0x89760101,0xfb410108,0x89770101,0xfb410108,0x89780101,0xfb410108,0x89790101,
	0xfb410108,0x897a0101,0xfb410108,0x897c0101,0xfb410108,0x897d0101,0xfb410108,0x897e0101,
	0xfb410108,0x89800101,0xfb410108,0x89820101,0xfb410108,0x89840101,0xfb410108,0x89850101,
	0xfb410108,0x89870101,0xfb410108,0x89880101,0xfb410108,0x89890101,0xfb410108,0x898a0101,
	0xfb410108,0x898b0101,0xfb410108,0x898c0101,0xfb410108,0x898d0101,0xfb410108,0x898e0101,
	0xfb410108,0x898f0101,0xfb410108,0x89900101,0xfb410108,0x89910101,0xfb410108,0x89920101,
	0xfb410108,0x89930101,0xfb410108,0x89940101,0xfb410108,0x89950101,0xfb410108,0x89960101,
	0xfb410108,0x89970101,0xfb410108,0x89980101,0xfb410108,0x89990101,0xfb410108,0x899a0101,
	0xfb410108,0x899b0101,0xfb410108,0x899c0101,0xfb410108,0x899d0101,0xfb410108,0x899e0101,
	0xfb410108,0x899f0101,0xfb410108,0x89a00101,0xfb410108,0x89a10101,0xfb410108,0x89a20101,
	0xfb410108,0x89a30101,0xfb410108,0x89a40101,0xfb410108,0x89a50101,0xfb410108,0x89a60101,
	0xfb410108,0x89a70101,0xfb410108,0x89a80101,0xfb410108,0x89a90101,0xfb410108,0x89aa0101,
	0xfb410108,0x89ab0101,0xfb410108,0x89ac0101,0xfb410108,0x89ad0101,0xfb410108,0x89ae0101,
	0xfb410108,0x89af0101,0xfb410108,0x89b00101,0xfb410108,0x89b10101,0xfb410108,0x89b20101,
	0xfb410108,0x89b30101,0xfb410108,0x89b40101,0xfb410108,0x89b50101,0xfb410108,0x89b60101,
	0xfb410108,0x89b70101,0xfb410108,0x89b80101,0xfb410108,0x89b90101,0xfb410108,0x89ba0101,
	0xfb410108,0x89bb0101,0xfb410108,0x89bc0101,0xfb410108,0x89bd0101,0xfb410108,0x89be0101,
	0xfb410108,0x89bf0101,0xfb410108,0x89c00101,0xfb410108,0x89c30101,0xfb410108,0x89cd0101,
	0xfb410108,0x89d30101,0xfb410108,0x89d40101,0xfb410108,0x89d50101,0xfb410108,0x89d70101,
	0xfb410108,0x89d80101,0xfb410108,0x89d90101,0xfb410108,0x89db0101,0xfb410108,0x89dd0101,
	0xfb410108,0x89df0101,0xfb410108,0x89e00101,0xfb410108,0x89e10101,0xfb410108,0x89e20101,
	0xfb410108,0x89e40101,0xfb410108,0x89e70101,0xfb410108,0x89e80101,0xfb410108,0x89e90101,
	0xfb410108,0x89ea0101,0xfb410108,0x89ec0101,0xfb410108,0x89ed0101,0xfb410108,0x89ee0101,
	0xfb410108,0x89f00101,0xfb410108,0x89f10101,0xfb410108,0x89f20101,0xfb410108,0x89f40101,
	0xfb410108,0x89f50101,0xfb410108,0x89f60101,0xfb410108,0x89f70101,0xfb410108,0x89f80101,
	0xfb410108,0x89f90101,0xfb410108,0x89fa0101,0xfb410108,0x89fb0101,0xfb410108,0x89fc0101,
	0xfb410108,0x89fd0101,0xfb410108,0x89fe0101,0xfb410108,0x89ff0101,0xfb410108,0x8a010101,
	0xfb410108,0x8a020101,0xfb410108,0x8a030101,0xfb410108,0x8a040101,0xfb410108,0x8a050101,
	0xfb410108,0x8a060101,0xfb410108,0x8a080101,0xfb410108,0x8a090101,0xfb410108,0x8a0a0101,
	0xfb410108,0x8a0b0101,0xfb410108,0x8a0c0101,0xfb410108,0x8a0d0101,0xfb410108,0x8a0e0101,
	0xfb410108,0x8a0f0101,0xfb410108,0x8a100101,0xfb410108,0x8a110101,0xfb410108,0x8a120101,
	0xfb410108,0x8a130101,0xfb410108,0x8a140101,0xfb410108,0x8a150101,0xfb410108,0x8a160101,
	0xfb410108,0x8a170101,0xfb410108,0x8a180101,0xfb410108,0x8a190101,0xfb410108,0x8a1a0101,
	0xfb410108,0x8a1b0101,0xfb410108,0x8a1c0101,0xfb410108,0x8a1d0101,0xfb410108,0x8a1e0101,
	0xfb410108,0x8a1f0101,0xfb410108,0x8a200101,0xfb410108,0x8a210101,0xfb410108,0x8a220101,
	0xfb410108,0x8a230101,0xfb410108,0x8a240101,0xfb410108,0x8a250101,0xfb410108,0x8a260101,
	0xfb410108,0x8a270101,0xfb410108,0x8a280101,0xfb410108,0x8a290101,0xfb410108,0x8a2a0101,
	0xfb410108,0x8a2b0101,0xfb410108,0x8a2c0101,0xfb410108,0x8a2d0101,0xfb410108,0x8a2e0101,
	0xfb410108,0x8a2f0101,0xfb410108,0x8a300101,0xfb410108,0x8a310101,0xfb410108,0x8a320101,
	0xfb410108,0x8a330101,0xfb410108,0x8a340101,0xfb410108,0x8a350101,0xfb410108,0x8a360101,
	0xfb410108,0x8a370101,0xfb410108,0x8a380101,0xfb410108,0x8a390101,0xfb410108,0x8a3a0101,
	0xfb410108,0x8a3b0101,0xfb410108,0x8a3c0101,0xfb410108,0x8a3d0101,0xfb410108,0x8a3f0101,
	0xfb410108,0x8a400101,0xfb410108,0x8a410101,0xfb410108,0x8a420101,0xfb410108,0x8a430101,
	0xfb410108,0x8a440101,0xfb410108,0x8a450101,0xfb410108,0x8a460101,0xfb410108,0x8a470101,
	0xfb410108,0x8a490101,0xfb410108,0x8a4a0101,0xfb410108,0x8a4b0101,0xfb410108,0x8a4c0101,
	0xfb410108,0x8a4d0101,0xfb410108,0x8a4e0101,0xfb410108,0x8a4f0101,0xfb410108,0x8a500101,
	0xfb410108,0x8a510101,0xfb410108,0x8a520101,0xfb410108,0x8a530101,0xfb410108,0x8a540101,
	0xfb410108,0x8a550101,0xfb410108,0x8a560101,0xfb410108,0x8a570101,0xfb410108,0x8a580101,
	0xfb410108,0x8a590101,0xfb410108,0x8a5a0101,0xfb410108,0x8a5b0101,0xfb410108,0x8a5c0101,
	0xfb410108,0x8a5d0101,0xfb410108,0x8a5e0101,0xfb410108,0x8a5f0101,0xfb410108,0x8a600101,
	0xfb410108,0x8a610101,0xfb410108,0x8a620101,0xfb410108,0x8a630101,0xfb410108,0x8a640101,
	0xfb410108,0x8a650101,0xfb410108,0x8a660101,0xfb410108,0x8a670101,0xfb410108,0x8a680101,
	0xfb410108,0x8a690101,0xfb410108,0x8a6a0101,0xfb410108,0x8a6b0101,0xfb410108,0x8a6c0101,
	0xfb410108,0x8a6d0101,0xfb410108,0x8a6e0101,0xfb410108,0x8a6f0101,0xfb410108,0x8a700101,
	0xfb410108,0x8a710101,0xfb410108,0x8a720101,0xfb410108,0x8a730101,0xfb410108,0x8a740101,
	0xfb410108,0x8a750101,0xfb410108,0x8a760101,0xfb410108,0x8a770101,0xfb410108,0x8a780101,
	0xfb410108,0x8a7a0101,0xfb410108,0x8a7b0101,0xfb410108,0x8a7c0101,0xfb410108,0x8a7d0101,
	0xfb410108,0x8a7e0101,0xfb410108,0x8a7f0101,0xfb410108,0x8a800101,0xfb410108,0x8a810101,
	0xfb410108,0x8a820101,0xfb410108,0x8a830101,0xfb410108,0x8a840101,0xfb410108,0x8a850101,
	0xfb410108,0x8a860101,0xfb410108,0x8a870101,0xfb410108,0x8a880101,0xfb410108,0x8a8b0101,
	0xfb410108,0x8a8c0101,0xfb410108,0x8a8d0101,0xfb410108,0x8a8e0101,0xfb410108,0x8a8f0101,
	0xfb410108,0x8a900101,0xfb410108,0x8a910101,0xfb410108,0x8a920101,0xfb410108,0x8a940101,
	0xfb410108,0x8a950101,0xfb410108,0x8a960101,0xfb410108,0x8a970101,0xfb410108,0x8a980101,
	0xfb410108,0x8a990101,0xfb410108,0x8a9a0101,0xfb410108,0x8a9b0101,0xfb410108,0x8a9c0101,
	0xfb410108,0x8a9d0101,0xfb410108,0x8a9e0101,0xfb410108,0x8a9f0101,0xfb410108,0x8aa00101,
	0xfb410108,0x8aa10101,0xfb410108,0x8aa20101,0xfb410108,0x8aa30101,0xfb410108,0x8aa40101,
	0xfb410108,0x8aa50101,0xfb410108,0x8aa60101,0xfb410108,0x8aa70101,0xfb410108,0x8aa80101,
	0xfb410108,0x8aa90101,0xfb410108,0x8aaa0101,0xfb410108,0x8aab0101,0xfb410108,0x8aac0101,
	0xfb410108,0x8aad0101,0xfb410108,0x8aae0101,0xfb410108,0x8aaf0101,0xfb410108,0x8ab00101,
	0xfb410108,0x8ab10101,0xfb410108,0x8ab20101,0xfb410108,0x8ab30101,0xfb410108,0x8ab40101,
	0xfb410108,0x8ab50101,0xfb410108,0x8ab60101,0xfb410108,0x8ab70101,0xfb410108,0x8ab80101,
	0xfb410108,0x8ab90101,0xfb410108,0x8aba0101,0xfb410108,0x8abb0101,0xfb410108,0x8abc0101,
	0xfb410108,0x8abd0101,0xfb410108,0x8abe0101,0xfb410108,0x8abf0101,0xfb410108,0x8ac00101,
	0xfb410108,0x8ac10101,0xfb410108,0x8ac20101,0xfb410108,0x8ac30101,0xfb410108,0x8ac40101,
	0xfb410108,0x8ac50101,0xfb410108,0x8ac60101,0xfb410108,0x8ac70101,0xfb410108,0x8ac80101,
	0xfb410108,0x8ac90101,0xfb410108,0x8aca0101,0xfb410108,0x8acb0101,0xfb410108,0x8acc0101,
	0xfb410108,0x8acd0101,0xfb410108,0x8ace0101,0xfb410108,0x8acf0101,0xfb410108,0x8ad00101,
	0xfb410108,0x8ad10101,0xfb410108,0x8ad20101,0xfb410108,0x8ad30101,0xfb410108,0x8ad40101,
	0xfb410108,0x8ad50101,0xfb410108,0x8ad60101,0xfb410108,0x8ad70101,0xfb410108,0x8ad80101,
	0xfb410108,0x8ad90101,0xfb410108,0x8ada0101,0xfb410108,0x8adb0101,0xfb410108,0x8adc0101,
	0xfb410108,0x8add0101,0xfb410108,0x8ade0101,0xfb410108,0x8adf0101,0xfb410108,0x8ae00101,
	0xfb410108,0x8ae10101,0xfb410108,0x8ae20101,0xfb410108,0x8ae30101,0xfb410108,0x8ae40101,
	0xfb410108,0x8ae50101,0xfb410108,0x8ae60101,0xfb410108,0x8ae70101,0xfb410108,0x8ae80101,
	0xfb410108,0x8ae90101,0xfb410108,0x8aea0101,0xfb410108,0x8aeb0101,0xfb410108,0x8aec0101,
	0xfb410108,0x8aed0101,0xfb410108,0x8aee0101,0xfb410108,0x8aef0101,0xfb410108,0x8af00101,
	0xfb410108,0x8af10101,0xfb410108,0x8af20101,0xfb410108,0x8af30101,0xfb410108,0x8af40101,
	0xfb410108,0x8af50101,0xfb410108,0x8af60101,0xfb410108,0x8af70101,0xfb410108,0x8af80101,
	0xfb410108,0x8af90101,0xfb410108,0x8afa0101,0xfb410108,0x8afb0101,0xfb410108,0x8afc0101,
	0xfb410108,0x8afd0101,0xfb410108,0x8afe0101,0xfb410108,0x8aff0101,0xfb410108,0x8b000101,
	0xfb410108,0x8b010101,0xfb410108,0x8b020101,0xfb410108,0x8b030101,0xfb410108,0x8b040101,
	0xfb410108,0x8b050101,0xfb410108,0x8b060101,0xfb410108,0x8b080101,0xfb410108,0x8b090101,
	0xfb410108,0x8b0a0101,0xfb410108,0x8b0b0101,0xfb410108,0x8b0c0101,0xfb410108,0x8b0d0101,
	0xfb410108,0x8b0e0101,0xfb410108,0x8b0f0101,0xfb410108,0x8b100101,0xfb410108,0x8b110101,
	0xfb410108,0x8b120101,0xfb410108,0x8b130101,0xfb410108,0x8b140101,0xfb410108,0x8b150101,
	0xfb410108,0x8b160101,0xfb410108,0x8b170101,0xfb410108,0x8b180101,0xfb410108,0x8b190101,
	0xfb410108,0x8b1a0101,0xfb410108,0x8b1b0101,0xfb410108,0x8b1c0101,0xfb410108,0x8b1d0101,
	0xfb410108,0x8b1e0101,0xfb410108,0x8b1f0101,0xfb410108,0x8b200101,0xfb410108,0x8b210101,
	0xfb410108,0x8b220101,0xfb410108,0x8b230101,0xfb410108,0x8b240101,0xfb410108,0x8b250101,
	0xfb410108,0x8b270101,0xfb410108,0x8b280101,0xfb410108,0x8b290101,0xfb410108,0x8b2a0101,
	0xfb410108,0x8b2b0101,0xfb410108,0x8b2c0101,0xfb410108,0x8b2d0101,0xfb410108,0x8b2e0101,
	0xfb410108,0x8b2f0101,0xfb410108,0x8b300101,0xfb410108,0x8b310101,0xfb410108,0x8b320101,
	0xfb410108,0x8b330101,0xfb410108,0x8b340101,0xfb410108,0x8b350101,0xfb410108,0x8b360101,
	0xfb410108,0x8b370101,0xfb410108,0x8b380101,0xfb410108,0x8b390101,0xfb410108,0x8b3a0101,
	0xfb410108,0x8b3b0101,0xfb410108,0x8b3c0101,0xfb410108,0x8b3d0101,0xfb410108,0x8b3e0101,
	0xfb410108,0x8b3f0101,0xfb410108,0x8b400101,0xfb410108,0x8b410101,0xfb410108,0x8b420101,
	0xfb410108,0x8b430101,0xfb410108,0x8b440101,0xfb410108,0x8b450101,0xfb410108,0x8b460101,
	0xfb410108,0x8b470101,0xfb410108,0x8b480101,0xfb410108,0x8b490101,0xfb410108,0x8b4a0101,
	0xfb410108,0x8b4b0101,0xfb410108,0x8b4c0101,0xfb410108,0x8b4d0101,0xfb410108,0x8b4e0101,
	0xfb410108,0x8b4f0101,0xfb410108,0x8b500101,0xfb410108,0x8b510101,0xfb410108,0x8b520101,
	0xfb410108,0x8b530101,0xfb410108,0x8b540101,0xfb410108,0x8b550101,0xfb410108,0x8b560101,
	0xfb410108,0x8b570101,0xfb410108,0x8b580101,0xfb410108,0x8b590101,0xfb410108,0x8b5a0101,
	0xfb410108,0x8b5b0101,0xfb410108,0x8b5c0101,0xfb410108,0x8b5d0101,0xfb410108,0x8b5e0101,
	0xfb410108,0x8b5f0101,0xfb410108,0x8b600101,0xfb410108,0x8b610101,0xfb410108,0x8b620101,
	0xfb410108,0x8b630101,0xfb410108,0x8b640101,0xfb410108,0x8b650101,0xfb410108,0x8b670101,
	0xfb410108,0x8b680101,0xfb410108,0x8b690101,0xfb410108,0x8b6a0101,0xfb410108,0x8b6b0101,
	0xfb410108,0x8b6d0101,0xfb410108,0x8b6e0101,0xfb410108,0x8b6f0101,0xfb410108,0x8b700101,
	0xfb410108,0x8b710101,0xfb410108,0x8b720101,0xfb410108,0x8b730101,0xfb410108,0x8b740101,
	0xfb410108,0x8b750101,0xfb410108,0x8b760101,0xfb410108,0x8b770101,0xfb410108,0x8b780101,
	0xfb410108,0x8b790101,0xfb410108,0x8b7a0101,0xfb410108,0x8b7b0101,0xfb410108,0x8b7c0101,
	0xfb410108,0x8b7d0101,0xfb410108,0x8b7e0101,0xfb410108,0x8b7f0101,0xfb410108,0x8b800101,
	0xfb410108,0x8b810101,0xfb410108,0x8b820101,0xfb410108,0x8b830101,0xfb410108,0x8b840101,
	0xfb410108,0x8b850101,0xfb410108,0x8b860101,0xfb410108,0x8b870101,0xfb410108,0x8b880101,
	0xfb410108,0x8b890101,0xfb410108,0x8b8a0101,0xfb410108,0x8b8b0101,0xfb410108,0x8b8c0101,
	0xfb410108,0x8b8d0101,0xfb410108,0x8b8e0101,0xfb410108,0x8b8f0101,0xfb410108,0x8b900101,
	0xfb410108,0x8b910101,0xfb410108,0x8b920101,0xfb410108,0x8b930101,0xfb410108,0x8b940101,
	0xfb410108,0x8b950101,0xfb410108,0x8b960101,0xfb410108,0x8b970101,0xfb410108,0x8b980101,
	0xfb410108,0x8b990101,0xfb410108,0x8b9a0101,0xfb410108,0x8b9b0101,0xfb410108,0x8b9c0101,
	0xfb410108,0x8b9d0101,0xfb410108,0x8b9e0101,0xfb410108,0x8b9f0101,0xfb410108,0x8bac0101,
	0xfb410108,0x8bb10101,0xfb410108,0x8bbb0101,0xfb410108,0x8bc70101,0xfb410108,0x8bd00101,
	0xfb410108,0x8bea0101,0xfb410108,0x8c090101,0xfb410108,0x8c1e0101,0xfb410108,0x8c380101,
	0xfb410108,0x8c390101,0xfb410108,0x8c3a0101,0xfb410108,0x8c3b0101,0xfb410108,0x8c3c0101,
	0xfb410108,0x8c3d0101,0xfb410108,0x8c3e0101,0xfb410108,0x8c3f0101,0xfb410108,0x8c400101,
	0xfb410108,0x8c420101,0xfb410108,0x8c430101,0xfb410108,0x8c440101,0xfb410108,0x8c450101,
	0xfb410108,0x8c480101,0xfb410108,0x8c4a0101,0xfb410108,0x8c4b0101,0xfb410108,0x8c4d0101,
	0xfb410108,0x8c4e0101,0xfb410108,0x8c4f0101,0xfb410108,0x8c500101,0xfb410108,0x8c510101,
	0xfb410108,0x8c520101,0xfb410108,0x8c530101,0xfb410108,0x8c540101,0xfb410108,0x8c560101,
	0xfb410108,0x8c570101,0xfb410108,0x8c580101,0xfb410108,0x8c590101,0xfb410108,0x8c5b0101,
	0xfb410108,0x8c5c0101,0xfb410108,0x8c5d0101,0xfb410108,0x8c5e0101,0xfb410108,0x8c5f0101,
	0xfb410108,0x8c600101,0xfb410108,0x8c630101,0xfb410108,0x8c640101,0xfb410108,0x8c650101,
	0xfb410108,0x8c660101,0xfb410108,0x8c670101,0xfb410108,0x8c680101,0xfb410108,0x8c690101,
	0xfb410108,0x8c6c0101,0xfb410108,0x8c6d0101,0xfb410108,0x8c6e0101,0xfb410108,0x8c6f0101,
	0xfb410108,0x8c700101,0xfb410108,0x8c710101,0xfb410108,0x8c720101,0xfb410108,0x8c740101,
	0xfb410108,0x8c750101,0xfb410108,0x8c760101,0xfb410108,0x8c770101,0xfb410108,0x8c7b0101,
	0xfb410108,0x8c7c0101,0xfb410108,0x8c7d0101,0xfb410108,0x8c7e0101,0xfb410108,0x8c7f0101,
	0xfb410108,0x8c800101,0xfb410108,0x8c810101,0xfb410108,0x8c830101,0xfb410108,0x8c840101,
	0xfb410108,0x8c860101,0xfb410108,0x8c870101,0xfb410108,0x8c880101,0xfb410108,0x8c8b0101,
	0xfb410108,0x8c8d0101,0xfb410108,0x8c8e0101,0xfb410108,0x8c8f0101,0xfb410108,0x8c900101,
	0xfb410108,0x8c910101,0xfb410108,0x8c920101,0xfb410108,0x8c930101,0xfb410108,0x8c950101,
	0xfb410108,0x8c960101,0xfb410108,0x8c970101,0xfb410108,0x8c990101,0xfb410108,0x8c9a0101,
	0xfb410108,0x8c9b0101,0xfb410108,0x8c9c0101,0xfb410108,0x8c9d0101,0xfb410108,0x8c9e0101,
	0xfb410108,0x8c9f0101,0xfb410108,0x8ca00101,0xfb410108,0x8ca10101,0xfb410108,0x8ca20101,
	0xfb410108,0x8ca30101,0xfb410108,0x8ca40101,0xfb410108,0x8ca50101,0xfb410108,0x8ca60101,
	0xfb410108,0x8ca70101,0xfb410108,0x8ca80101,0xfb410108,0x8ca90101,0xfb410108,0x8caa0101,
	0xfb410108,0x8cab0101,0xfb410108,0x8cac0101,0xfb410108,0x8cad0101,0xfb410108,0x8cae0101,
	0xfb410108,0x8caf0101,0xfb410108,0x8cb00101,0xfb410108,0x8cb10101,0xfb410108,0x8cb20101,
	0xfb410108,0x8cb30101,0xfb410108,0x8cb40101,0xfb410108,0x8cb50101,0xfb410108,0x8cb60101,
	0xfb410108,0x8cb70101,0xfb410108,0x8cb80101,0xfb410108,0x8cb90101,0xfb410108,0x8cba0101,
	0xfb410108,0x8cbb0101,0xfb410108,0x8cbc0101,0xfb410108,0x8cbd0101,0xfb410108,0x8cbe0101,
	0xfb410108,0x8cbf0101,0xfb410108,0x8cc00101,0xfb410108,0x8cc10101,0xfb410108,0x8cc20101,
	0xfb410108,0x8cc30101,0xfb410108,0x8cc40101,0xfb410108,0x8cc50101,0xfb410108,0x8cc60101,
	0xfb410108,0x8cc70101,0xfb410108,0x8cc80101,0xfb410108,0x8cc90101,0xfb410108,0x8cca0101,
	0xfb410108,0x8ccb0101,0xfb410108,0x8ccc0101,0xfb410108,0x8ccd0101,0xfb410108,0x8cce0101,
	0xfb410108,0x8ccf0101,0xfb410108,0x8cd00101,0xfb410108,0x8cd10101,0xfb410108,0x8cd20101,
	0xfb410108,0x8cd30101,0xfb410108,0x8cd40101,0xfb410108,0x8cd50101,0xfb410108,0x8cd60101,
	0xfb410108,0x8cd70101,0xfb410108,0x8cd80101,0xfb410108,0x8cd90101,0xfb410108,0x8cda0101,
	0xfb410108,0x8cdb0101,0xfb410108,0x8cdc0101,0xfb410108,0x8cdd0101,0xfb410108,0x8cde0101,
	0xfb410108,0x8cdf0101,0xfb410108,0x8ce00101,0xfb410108,0x8ce10101,0xfb410108,0x8ce20101,
	0xfb410108,0x8ce30101,0xfb410108,0x8ce40101,0xfb410108,0x8ce50101,0xfb410108,0x8ce60101,
	0xfb410108,0x8ce70101,0xfb410108,0x8ce80101,0xfb410108,0x8ce90101,0xfb410108,0x8cea0101,
	0xfb410108,0x8ceb0101,0xfb410108,0x8cec0101,0xfb410108,0x8ced0101,0xfb410108,0x8cee0101,
	0xfb410108,0x8cef0101,0xfb410108,0x8cf00101,0xfb410108,0x8cf10101,0xfb410108,0x8cf20101,
	0xfb410108,0x8cf30101,0xfb410108,0x8cf40101,0xfb410108,0x8cf50101,0xfb410108,0x8cf60101,
	0xfb410108,0x8cf70101,0xfb410108,0x8cf80101,0xfb410108,0x8cf90101,0xfb410108,0x8cfa0101,
	0xfb410108,0x8cfb0101,0xfb410108,0x8cfc0101,0xfb410108,0x8cfd0101,0xfb410108,0x8cfe0101,
	0xfb410108,0x8cff0101,0xfb410108,0x8d000101,0xfb410108,0x8d010101,0xfb410108,0x8d020101,
	0xfb410108,0x8d030101,0xfb410108,0x8d040101,0xfb410108,0x8d050101,0xfb410108,0x8d060101,
	0xfb410108,0x8d070101,0xfb410108,0x8d080101,0xfb410108,0x8d090101,0xfb410108,0x8d0a0101,
	0xfb410108,0x8d0b0101,0xfb410108,0x8d0c0101,0xfb410108,0x8d0d0101,0xfb410108,0x8d0e0101,
	0xfb410108,0x8d0f0101,0xfb410108,0x8d100101,0xfb410108,0x8d110101,0xfb410108,0x8d120101,
	0xfb410108,0x8d130101,0xfb410108,0x8d140101,0xfb410108,0x8d150101,0xfb410108,0x8d160101,
	0xfb410108,0x8d170101,0xfb410108,0x8d180101,0xfb410108,0x8d190101,0xfb410108,0x8d1a0101,
	0xfb410108,0x8d1b0101,0xfb410108,0x8d1c0101,0xfb410108,0x8d200101,0xfb410108,0x8d510101,
	0xfb410108,0x8d520101,0xfb410108,0x8d570101,0xfb410108,0x8d5f0101,0xfb410108,0x8d650101,
	0xfb410108,0x8d680101,0xfb410108,0x8d690101,0xfb410108,0x8d6a0101,0xfb410108,0x8d6c0101,
	0xfb410108,0x8d6e0101,0xfb410108,0x8d6f0101,0xfb410108,0x8d710101,0xfb410108,0x8d720101,
	0xfb410108,0x8d780101,0xfb410108,0x8d790101,0xfb410108,0x8d7a0101,0xfb410108,0x8d7b0101,
	0xfb410108,0x8d7c0101,0xfb410108,0x8d7d0101,0xfb410108,0x8d7e0101,0xfb410108,0x8d7f0101,
	0xfb410108,0x8d800101,0xfb410108,0x8d820101,0xfb410108,0x8d830101,0xfb410108,0x8d860101,
	0xfb410108,0x8d870101,0xfb410108,0x8d880101,0xfb410108,0x8d890101,0xfb410108,0x8d8c0101,
	0xfb410108,0x8d8d0101,0xfb410108,0x8d8e0101,0xfb410108,0x8d8f0101,0xfb410108,0x8d900101,
	0xfb410108,0x8d920101,0xfb410108,0x8d930101,0xfb410108,0x8d950101,0xfb410108,0x8d960101,
	0xfb410108,0x8d970101,0xfb410108,0x8d980101,0xfb410108,0x8d990101,0xfb410108,0x8d9a0101,
	0xfb410108,0x8d9b0101,0xfb410108,0x8d9c0101,0xfb410108,0x8d9d0101,0xfb410108,0x8d9e0101,
	0xfb410108,0x8da00101,0xfb410108,0x8da10101,0xfb410108,0x8da20101,0xfb410108,0x8da40101,
	0xfb410108,0x8da50101,0xfb410108,0x8da60101,0xfb410108,0x8da70101,0xfb410108,0x8da80101,
	0xfb410108,0x8da90101,0xfb410108,0x8daa0101,0xfb410108,0x8dab0101,0xfb410108,0x8dac0101,
	0xfb410108,0x8dad0101,0xfb410108,0x8dae0101,0xfb410108,0x8daf0101,0xfb410108,0x8db00101,
	0xfb410108,0x8db20101,0xfb410108,0x8db60101,0xfb410108,0x8db70101,0xfb410108,0x8db90101,
	0xfb410108,0x8dbb0101,0xfb410108,0x8dbd0101,0xfb410108,0x8dc00101,0xfb410108,0x8dc10101,
	0xfb410108,0x8dc20101,0xfb410108,0x8dc50101,0xfb410108,0x8dc70101,0xfb410108,0x8dc80101,
	0xfb410108,0x8dc90101,0xfb410108,0x8dca0101,0xfb410108,0x8dcd0101,0xfb410108,0x8dd00101,
	0xfb410108,0x8dd20101,0xfb410108,0x8dd30101,0xfb410108,0x8dd40101,0xfb410108,0x8dd50101,
	0xfb410108,0x8dd80101,0xfb410108,0x8dd90101,0xfb410108,0x8ddc0101,0xfb410108,0x8de00101,
	0xfb410108,0x8de10101,0xfb410108,0x8de20101,0xfb410108,0x8de50101,0xfb410108,0x8de60101,
	0xfb410108,0x8de70101,0xfb410108,0x8de90101,0xfb410108,0x8ded0101,0xfb410108,0x8dee0101,
	0xfb410108,0x8df00101,0xfb410108,0x8df10101,0xfb410108,0x8df20101,0xfb410108,0x8df40101,
	0xfb410108,0x8df60101,0xfb410108,0x8dfc0101,0xfb410108,0x8dfe0101,0xfb410108,0x8dff0101,
	0xfb410108,0x8e000101,0xfb410108,0x8e010101,0xfb410108,0x8e020101,0xfb410108,0x8e030101,
	0xfb410108,0x8e040101,0xfb410108,0x8e060101,0xfb410108,0x8e070101,0xfb410108,0x8e080101,
	0xfb410108,0x8e0b0101,0xfb410108,0x8e0d0101,0xfb410108,0x8e0e0101,0xfb410108,0x8e100101,
	0xfb410108,0x8e110101,0xfb410108,0x8e120101,0xfb410108,0x8e130101,0xfb410108,0x8e150101,
	0xfb410108,0x8e160101,0xfb410108,0x8e170101,0xfb410108,0x8e180101,0xfb410108,0x8e190101,
	0xfb410108,0x8e1a0101,0xfb410108,0x8e1b0101,0xfb410108,0x8e1c0101,0xfb410108,0x8e200101,
	0xfb410108,0x8e210101,0xfb410108,0x8e240101,0xfb410108,0x8e250101,0xfb410108,0x8e260101,
	0xfb410108,0x8e270101,0xfb410108,0x8e280101,0xfb410108,0x8e2b0101,0xfb410108,0x8e2d0101,
	0xfb410108,0x8e300101,0xfb410108,0x8e320101,0xfb410108,0x8e330101,0xfb410108,0x8e340101,
	0xfb410108,0x8e360101,0xfb410108,0x8e370101,0xfb410108,0x8e380101,0xfb410108,0x8e3b0101,
	0xfb410108,0x8e3c0101,0xfb410108,0x8e3e0101,0xfb410108,0x8e3f0101,0xfb410108,0x8e430101,
	0xfb410108,0x8e450101,0xfb410108,0x8e460101,0xfb410108,0x8e4c0101,0xfb410108,0x8e4d0101,
	0xfb410108,0x8e4e0101,0xfb410108,0x8e4f0101,0xfb410108,0x8e500101,0xfb410108,0x8e530101,
	0xfb410108,0x8e540101,0xfb410108,0x8e550101,0xfb410108,0x8e560101,0xfb410108,0x8e570101,
	0xfb410108,0x8e580101,0xfb410108,0x8e5a0101,0xfb410108,0x8e5b0101,0xfb410108,0x8e5c0101,
	0xfb410108,0x8e5d0101,0xfb410108,0x8e5e0101,0xfb410108,0x8e5f0101,0xfb410108,0x8e600101,
	0xfb410108,0x8e610101,0xfb410108,0x8e620101,0xfb410108,0x8e630101,0xfb410108,0x8e640101,
	0xfb410108,0x8e650101,0xfb410108,0x8e670101,0xfb410108,0x8e680101,0xfb410108,0x8e6a0101,
	0xfb410108,0x8e6b0101,0xfb410108,0x8e6e0101,0xfb410108,0x8e710101,0xfb410108,0x8e730101,
	0xfb410108,0x8e750101,0xfb410108,0x8e770101,0xfb410108,0x8e780101,0xfb410108,0x8e790101,
	0xfb410108,0x8e7a0101,0xfb410108,0x8e7b0101,0xfb410108,0x8e7d0101,0xfb410108,0x8e7e0101,
	0xfb410108,0x8e800101,0xfb410108,0x8e820101,0xfb410108,0x8e830101,0xfb410108,0x8e840101,
	0xfb410108,0x8e860101,0xfb410108,0x8e880101,0xfb410108,0x8e890101,0xfb410108,0x8e8a0101,
	0xfb410108,0x8e8b0101,0xfb410108,0x8e8c0101,0xfb410108,0x8e8d0101,0xfb410108,0x8e8e0101,
	0xfb410108,0x8e910101,0xfb410108,0x8e920101,0xfb410108,0x8e930101,0xfb410108,0x8e950101,
	0xfb410108,0x8e960101,0xfb410108,0x8e970101,0xfb410108,0x8e980101,0xfb410108,0x8e990101,
	0xfb410108,0x8e9a0101,0xfb410108,0x8e9b0101,0xfb410108,0x8e9d0101,0xfb410108,0x8e9f0101,
	0xfb410108,0x8ea00101,0xfb410108,0x8ea10101,0xfb410108,0x8ea20101,0xfb410108,0x8ea30101,
	0xfb410108,0x8ea40101,0xfb410108,0x8ea50101,0xfb410108,0x8ea60101,0xfb410108,0x8ea70101,
	0xfb410108,0x8ea80101,0xfb410108,0x8ea90101,0xfb410108,0x8eaa0101,0xfb410108,0x8ead0101,
	0xfb410108,0x8eae0101,0xfb410108,0x8eb00101,0xfb410108,0x8eb10101,0xfb410108,0x8eb30101,
	0xfb410108,0x8eb40101,0xfb410108,0x8eb50101,0xfb410108,0x8eb60101,0xfb410108,0x8eb70101,
	0xfb410108,0x8eb80101,0xfb410108,0x8eb90101,0xfb410108,0x8ebb0101,0xfb410108,0x8ebc0101,
	0xfb410108,0x8ebd0101,0xfb410108,0x8ebe0101,0xfb410108,0x8ebf0101,0xfb410108,0x8ec00101,
	0xfb410108,0x8ec10101,0xfb410108,0x8ec20101,0xfb410108,0x8ec30101,0xfb410108,0x8ec40101,
	0xfb410108,0x8ec50101,0xfb410108,0x8ec60101,0xfb410108,0x8ec70101,0xfb410108,0x8ec80101,
	0xfb410108,0x8ec90101,0xfb410108,0x8eca0101,0xfb410108,0x8ecb0101,0xfb410108,0x8ecc0101,
	0xfb410108,0x8ecd0101,0xfb410108,0x8ecf0101,0xfb410108,0x8ed00101,0xfb410108,0x8ed10101,
	0xfb410108,0x8ed20101,0xfb410108,0x8ed30101,0xfb410108,0x8ed40101,0xfb410108,0x8ed50101,
	0xfb410108,0x8ed60101,0xfb410108,0x8ed70101,0xfb410108,0x8ed80101,0xfb410108,0x8ed90101,
	0xfb410108,0x8eda0101,0xfb410108,0x8edb0101,0xfb410108,0x8edc0101,0xfb410108,0x8edd0101,
	0xfb410108,0x8ede0101,0xfb410108,0x8edf0101,0xfb410108,0x8ee00101,0xfb410108,0x8ee10101,
	0xfb410108,0x8ee20101,0xfb410108,0x8ee30101,0xfb410108,0x8ee40101,0xfb410108,0x8ee50101,
	0xfb410108,0x8ee60101,0xfb410108,0x8ee70101,0xfb410108,0x8ee80101,0xfb410108,0x8ee90101,
	0xfb410108,0x8eea0101,0xfb410108,0x8eeb0101,0xfb410108,0x8eec0101,0xfb410108,0x8eed0101,
	0xfb410108,0x8eee0101,0xfb410108,0x8eef0101,0xfb410108,0x8ef00101,0xfb410108,0x8ef10101,
	0xfb410108,0x8ef20101,0xfb410108,0x8ef30101,0xfb410108,0x8ef40101,0xfb410108,0x8ef50101,
	0xfb410108,0x8ef60101,0xfb410108,0x8ef70101,0xfb410108,0x8ef80101,0xfb410108,0x8ef90101,
	0xfb410108,0x8efa0101,0xfb410108,0x8efb0101,0xfb410108,0x8efc0101,0xfb410108,0x8efd0101,
	0xfb410108,0x8efe0101,0xfb410108,0x8eff0101,0xfb410108,0x8f000101,0xfb410108,0x8f010101,
	0xfb410108,0x8f020101,0xfb410108,0x8f030101,0xfb410108,0x8f040101,0xfb410108,0x8f050101,
	0xfb410108,0x8f060101,0xfb410108,0x8f070101,0xfb410108,0x8f080101,0xfb410108,0x8f090101,
	0xfb410108,0x8f0a0101,0xfb410108,0x8f0b0101,0xfb410108,0x8f0c0101,0xfb410108,0x8f0d0101,
	0xfb410108,0x8f0e0101,0xfb410108,0x8f0f0101,0xfb410108,0x8f100101,0xfb410108,0x8f110101,
	0xfb410108,0x8f120101,0xfb410108,0x8f130101,0xfb410108,0x8f140101,0xfb410108,0x8f150101,
	0xfb410108,0x8f160101,0xfb410108,0x8f170101,0xfb410108,0x8f180101,0xfb410108,0x8f190101,
	0xfb410108,0x8f1a0101,0xfb410108,0x8f1b0101,0xfb410108,0x8f1c0101,0xfb410108,0x8f1d0101,
	0xfb410108,0x8f1e0101,0xfb410108,0x8f1f0101,0xfb410108,0x8f200101,0xfb410108,0x8f210101,
	0xfb410108,0x8f220101,0xfb410108,0x8f230101,0xfb410108,0x8f240101,0xfb410108,0x8f250101,
	0xfb410108,0x8f260101,0xfb410108,0x8f270101,0xfb410108,0x8f280101,0xfb410108,0x8f290101,
	0xfb410108,0x8f2a0101,0xfb410108,0x8f2b0101,0xfb410108,0x8f2c0101,0xfb410108,0x8f2d0101,
	0xfb410108,0x8f2e0101,0xfb410108,0x8f2f0101,0xfb410108,0x8f300101,0xfb410108,0x8f310101,
	0xfb410108,0x8f320101,0xfb410108,0x8f330101,0xfb410108,0x8f340101,0xfb410108,0x8f350101,
	0xfb410108,0x8f360101,0xfb410108,0x8f370101,0xfb410108,0x8f380101,0xfb410108,0x8f390101,
	0xfb410108,0x8f3a0101,0xfb410108,0x8f3b0101,0xfb410108,0x8f3c0101,0xfb410108,0x8f3d0101,
	0xfb410108,0x8f3e0101,0xfb410108,0x8f3f0101,0xfb410108,0x8f400101,0xfb410108,0x8f410101,
	0xfb410108,0x8f420101,0xfb410108,0x8f430101,0xfb410108,0x8f440101,0xfb410108,0x8f450101,
	0xfb410108,0x8f460101,0xfb410108,0x8f470101,0xfb410108,0x8f480101,0xfb410108,0x8f490101,
	0xfb410108,0x8f4a0101,0xfb410108,0x8f4b0101,0xfb410108,0x8f4c0101,0xfb410108,0x8f4d0101,
	0xfb410108,0x8f4e0101,0xfb410108,0x8f4f0101,0xfb410108,0x8f500101,0xfb410108,0x8f510101,
	0xfb410108,0x8f520101,0xfb410108,0x8f530101,0xfb410108,0x8f540101,0xfb410108,0x8f550101,
	0xfb410108,0x8f560101,0xfb410108,0x8f570101,0xfb410108,0x8f580101,0xfb410108,0x8f590101,
	0xfb410108,0x8f5a0101,0xfb410108,0x8f5b0101,0xfb410108,0x8f5c0101,0xfb410108,0x8f5d0101,
	0xfb410108,0x8f5e0101,0xfb410108,0x8f5f0101,0xfb410108,0x8f600101,0xfb410108,0x8f610101,
	0xfb410108,0x8f620101,0xfb410108,0x8f630101,0xfb410108,0x8f640101,0xfb410108,0x8f650101,
	0xfb410108,0x8f6a0101,0xfb410108,0x8f800101,0xfb410108,0x8f8c0101,0xfb410108,0x8f920101,
	0xfb410108,0x8f9d0101,0xfb410108,0x8fa00101,0xfb410108,0x8fa10101,0xfb410108,0x8fa20101,
	0xfb410108,0x8fa40101,0xfb410108,0x8fa50101,0xfb410108,0x8fa60101,0xfb410108,0x8fa70101,
	0xfb410108,0x8faa0101,0xfb410108,0x8fac0101,0xfb410108,0x8fad0101,0xfb410108,0x8fae0101,
	0xfb410108,0x8faf0101,0xfb410108,0x8fb20101,0xfb410108,0x8fb30101,0xfb410108,0x8fb40101,
	0xfb410108,0x8fb50101,0xfb410108,0x8fb70101,0xfb410108,0x8fb80101,0xfb410108,0x8fba0101,
	0xfb410108,0x8fbb0101,0xfb410108,0x8fbc0101,0xfb410108,0x8fbf0101,0xfb410108,0x8fc00101,
	0xfb410108,0x8fc30101,0xfb410108,0x8fc60101,0xfb410108,0x8fc90101,0xfb410108,0x8fca0101,
	0xfb410108,0x8fcb0101,0xfb410108,0x8fcc0101,0xfb410108,0x8fcd0101,0xfb410108,0x8fcf0101,
	0xfb410108,0x8fd20101,0xfb410108,0x8fd60101,0xfb410108,0x8fd70101,0xfb410108,0x8fda0101,
	0xfb410108,0x8fe00101,0xfb410108,0x8fe10101,0xfb410108,0x8fe30101,0xfb410108,0x8fe70101,
	0xfb410108,0x8fec0101,0xfb410108,0x8fef0101,0xfb410108,0x8ff10101,0xfb410108,0x8ff20101,
	0xfb410108,0x8ff40101,0xfb410108,0x8ff50101,0xfb410108,0x8ff60101,0xfb410108,0x8ffa0101,
	0xfb410108,0x8ffb0101,0xfb410108,0x8ffc0101,0xfb410108,0x8ffe0101,0xfb410108,0x8fff0101,
	0xfb410108,0x90070101,0xfb410108,0x90080101,0xfb410108,0x900c0101,0xfb410108,0x900e0101,
	0xfb410108,0x90130101,0xfb410108,0x90150101,0xfb410108,0x90180101,0xfb410108,0x90190101,
	0xfb410108,0x901c0101,0xfb410108,0x90230101,0xfb410108,0x90240101,0xfb410108,0x90250101,
	0xfb410108,0x90270101,0xfb410108,0x90280101,0xfb410108,0x90290101,0xfb410108,0x902a0101,
	0xfb410108,0x902b0101,0xfb410108,0x902c0101,0xfb410108,0x90300101,0xfb410108,0x90310101,
	0xfb410108,0x90320101,0xfb410108,0x90330101,0xfb410108,0x90340101,0xfb410108,0x90370101,
	0xfb410108,0x90390101,0xfb410108,0x903a0101,0xfb410108,0x903d0101,0xfb410108,0x903f0101,
	0xfb410108,0x90400101,0xfb410108,0x90430101,0xfb410108,0x90450101,0xfb410108,0x90460101,
	0xfb410108,0x90480101,0xfb410108,0x90490101,0xfb410108,0x904a0101,0xfb410108,0x904b0101,
	0xfb410108,0x904c0101,0xfb410108,0x904e0101,0xfb410108,0x90540101,0xfb410108,0x90550101,
	0xfb410108,0x90560101,0xfb410108,0x90590101,0xfb410108,0x905a0101,0xfb410108,0x905c0101,
	0xfb410108,0x905d0101,0xfb410108,0x905e0101,0xfb410108,0x905f0101,0xfb410108,0x90600101,
	0xfb410108,0x90610101,0xfb410108,0x90640101,0xfb410108,0x90660101,0xfb410108,0x90670101,
	0xfb410108,0x90690101,0xfb410108,0x906a0101,0xfb410108,0x906b0101,0xfb410108,0x906c0101,
	0xfb410108,0x906f0101,0xfb410108,0x90700101,0xfb410108,0x90710101,0xfb410108,0x90720101,
	0xfb410108,0x90730101,0xfb410108,0x90760101,0xfb410108,0x90770101,0xfb410108,0x90780101,
	0xfb410108,0x90790101,0xfb410108,0x907a0101,0xfb410108,0x907b0101,0xfb410108,0x907c0101,
	0xfb410108,0x907e0101,0xfb410108,0x90810101,0xfb410108,0x90840101,0xfb410108,0x90850101,
	0xfb410108,0x90860101,0xfb410108,0x90870101,0xfb410108,0x90890101,0xfb410108,0x908a0101,
	0xfb410108,0x908c0101,0xfb410108,0x908d0101,0xfb410108,0x908e0101,0xfb410108,0x908f0101,
	0xfb410108,0x90900101,0xfb410108,0x90920101,0xfb410108,0x90940101,0xfb410108,0x90960101,
	0xfb410108,0x90980101,0xfb410108,0x909a0101,0xfb410108,0x909c0101,0xfb410108,0x909e0101,
	0xfb410108,0x909f0101,0xfb410108,0x90a00101,0xfb410108,0x90a40101,0xfb410108,0x90a50101,
	0xfb410108,0x90a70101,0xfb410108,0x90a80101,0xfb410108,0x90a90101,0xfb410108,0x90ab0101,
	0xfb410108,0x90ad0101,0xfb410108,0x90b20101,0xfb410108,0x90b70101,0xfb410108,0x90bc0101,
	0xfb410108,0x90bd0101,0xfb410108,0x90bf0101,0xfb410108,0x90c00101,0xfb410108,0x90c20101,
	0xfb410108,0x90c30101,0xfb410108,0x90c60101,0xfb410108,0x90c80101,0xfb410108,0x90c90101,
	0xfb410108,0x90cb0101,0xfb410108,0x90cc0101,0xfb410108,0x90cd0101,0xfb410108,0x90d20101,
	0xfb410108,0x90d40101,0xfb410108,0x90d50101,0xfb410108,0x90d60101,0xfb410108,0x90d80101,
	0xfb410108,0x90d90101,0xfb410108,0x90da0101,0xfb410108,0x90de0101,0xfb410108,0x90df0101,
	0xfb410108,0x90e00101,0xfb410108,0x90e30101,0xfb410108,0x90e40101,0xfb410108,0x90e50101,
	0xfb410108,0x90e90101,0xfb410108,0x90ea0101,0xfb410108,0x90ec0101,0xfb410108,0x90ee0101,
	0xfb410108,0x90f00101,0xfb410108,0x90f10101,0xfb410108,0x90f20101,0xfb410108,0x90f30101,
	0xfb410108,0x90f50101,0xfb410108,0x90f60101,0xfb410108,0x90f70101,0xfb410108,0x90f90101,
	0xfb410108,0x90fa0101,0xfb410108,0x90fb0101,0xfb410108,0x90fc0101,0xfb410108,0x90ff0101,
	0xfb410108,0x91000101,0xfb410108,0x91010101,0xfb410108,0x91030101,0xfb410108,0x91050101,
	0xfb410108,0x91060101,0xfb410108,0x91070101,0xfb410108,0x91080101,0xfb410108,0x91090101,
	0xfb410108,0x910a0101,0xfb410108,0x910b0101,0xfb410108,0x910c0101,0xfb410108,0x910d0101,
	0xfb410108,0x910e0101,0xfb410108,0x910f0101,0xfb410108,0x91100101,0xfb410108,0x91110101,
	0xfb410108,0x91120101,0xfb410108,0x91130101,0xfb410108,0x91140101,0xfb410108,0x91150101,
	0xfb410108,0x91160101,0xfb410108,0x91170101,0xfb410108,0x91180101,0xfb410108,0x911a0101,
	0xfb410108,0x911b0101,0xfb410108,0x911c0101,0xfb410108,0x911d0101,0xfb410108,0x911f0101,
	0xfb410108,0x91200101,0xfb410108,0x91210101,0xfb410108,0x91240101,0xfb410108,0x91250101,
	0xfb410108,0x91260101,0xfb410108,0x91270101,0xfb410108,0x91280101,0xfb410108,0x91290101,
	0xfb410108,0x912a0101,0xfb410108,0x912b0101,0xfb410108,0x912c0101,0xfb410108,0x912d0101,
	0xfb410108,0x912e0101,0xfb410108,0x91300101,0xfb410108,0x91320101,0xfb410108,0x91330101,
	0xfb410108,0x91340101,0xfb410108,0x91350101,0xfb410108,0x91360101,0xfb410108,0x91370101,
	0xfb410108,0x91380101,0xfb410108,0x913a0101,0xfb410108,0x913b0101,0xfb410108,0x913c0101,
	0xfb410108,0x913d0101,0xfb410108,0x913e0101,0xfb410108,0x913f0101,0xfb410108,0x91400101,
	0xfb410108,0x91410101,0xfb410108,0x91420101,0xfb410108,0x91440101,0xfb410108,0x91450101,
	0xfb410108,0x91470101,0xfb410108,0x91480101,0xfb410108,0x91510101,0xfb410108,0x91530101,
	0xfb410108,0x91540101,0xfb410108,0x91550101,0xfb410108,0x91560101,0xfb410108,0x91580101,
	0xfb410108,0x91590101,0xfb410108,0x915b0101,0xfb410108,0x915c0101,0xfb410108,0x915f0101,
	0xfb410108,0x91600101,0xfb410108,0x91660101,0xfb410108,0x91670101,0xfb410108,0x91680101,
	0xfb410108,0x916b0101,0xfb410108,0x916d0101,0xfb410108,0x91730101,0xfb410108,0x917a0101,
	0xfb410108,0x917b0101,0xfb410108,0x917c0101,0xfb410108,0x91800101,0xfb410108,0x91810101,
	0xfb410108,0x91820101,0xfb410108,0x91830101,0xfb410108,0x91840101,0xfb410108,0x91860101,
	0xfb410108,0x91880101,0xfb410108,0x918a0101,0xfb410108,0x918e0101,0xfb410108,0x918f0101,
	0xfb410108,0x91930101,0xfb410108,0x91940101,0xfb410108,0x91950101,0xfb410108,0x91960101,
	0xfb410108,0x91970101,0xfb410108,0x91980101,0xfb410108,0x91990101,0xfb410108,0x919c0101,
	0xfb410108,0x919d0101,0xfb410108,0x919e0101,0xfb410108,0x919f0101,0xfb410108,0x91a00101,
	0xfb410108,0x91a10101,0xfb410108,0x91a40101,0xfb410108,0x91a50101,0xfb410108,0x91a60101,
	0xfb410108,0x91a70101,0xfb410108,0x91a80101,0xfb410108,0x91a90101,0xfb410108,0x91ab0101,
	0xfb410108,0x91ac0101,0xfb410108,0x91b00101,0xfb410108,0x91b10101,0xfb410108,0x91b20101,
	0xfb410108,0x91b30101,0xfb410108,0x91b60101,0xfb410108,0x91b70101,0xfb410108,0x91b80101,
	0xfb410108,0x91b90101,0xfb410108,0x91bb0101,0xfb410108,0x91bc0101,0xfb410108,0x91bd0101,
	0xfb410108,0x91be0101,0xfb410108,0x91bf0101,0xfb410108,0x91c00101,0xfb410108,0x91c10101,
	0xfb410108,0x91c20101,0xfb410108,0x91c30101,0xfb410108,0x91c40101,0xfb410108,0x91c50101,
	0xfb410108,0x91c60101,0xfb410108,0x91c80101,0xfb410108,0x91cb0101,0xfb410108,0x91d00101,
	0xfb410108,0x91d20101,0xfb410108,0x91d30101,0xfb410108,0x91d40101,0xfb410108,0x91d50101,
	0xfb410108,0x91d60101,0xfb410108,0x91d70101,0xfb410108,0x91d80101,0xfb410108,0x91d90101,
	0xfb410108,0x91da0101,0xfb410108,0x91db0101,0xfb410108,0x91dd0101,0xfb410108,0x91de0101,
	0xfb410108,0x91df0101,0xfb410108,0x91e00101,0xfb410108,0x91e10101,0xfb410108,0x91e20101,
	0xfb410108,0x91e30101,0xfb410108,0x91e40101,0xfb410108,0x91e50101,0xfb410108,0x91e60101,
	0xfb410108,0x91e70101,0xfb410108,0x91e80101,0xfb410108,0x91e90101,0xfb410108,0x91ea0101,
	0xfb410108,0x91eb0101,0xfb410108,0x91ec0101,0xfb410108,0x91ed0101,0xfb410108,0x91ee0101,
	0xfb410108,0x91ef0101,0xfb410108,0x91f00101,0xfb410108,0x91f10101,0xfb410108,0x91f20101,
	0xfb410108,0x91f30101,0xfb410108,0x91f40101,0xfb410108,0x91f50101,0xfb410108,0x91f60101,
	0xfb410108,0x91f70101,0xfb410108,0x91f80101,0xfb410108,0x91f90101,0xfb410108,0x91fa0101,
	0xfb410108,0x91fb0101,0xfb410108,0x91fc0101,0xfb410108,0x91fd0101,0xfb410108,0x91fe0101,
	0xfb410108,0x91ff0101,0xfb410108,0x92000101,0xfb410108,0x92010101,0xfb410108,0x92020101,
	0xfb410108,0x92030101,0xfb410108,0x92040101,0xfb410108,0x92050101,0xfb410108,0x92060101,
	0xfb410108,0x92070101,0xfb410108,0x92080101,0xfb410108,0x92090101,0xfb410108,0x920a0101,
	0xfb410108,0x920b0101,0xfb410108,0x920c0101,0xfb410108,0x920d0101,0xfb410108,0x920e0101,
	0xfb410108,0x920f0101,0xfb410108,0x92100101,0xfb410108,0x92110101,0xfb410108,0x92120101,
	0xfb410108,0x92130101,0xfb410108,0x92140101,0xfb410108,0x92150101,0xfb410108,0x92160101,
	0xfb410108,0x92170101,0xfb410108,0x92180101,0xfb410108,0x92190101,0xfb410108,0x921a0101,
	0xfb410108,0x921b0101,0xfb410108,0x921c0101,0xfb410108,0x921d0101,0xfb410108,0x921e0101,
	0xfb410108,0x921f0101,0xfb410108,0x92200101,0xfb410108,0x92210101,0xfb410108,0x92220101,
	0xfb410108,0x92230101,0xfb410108,0x92240101,0xfb410108,0x92250101,0xfb410108,0x92260101,
	0xfb410108,0x92270101,0xfb410108,0x92280101,0xfb410108,0x92290101,0xfb410108,0x922a0101,
	0xfb410108,0x922b0101,0xfb410108,0x922c0101,0xfb410108,0x922d0101,0xfb410108,0x922e0101,
	0xfb410108,0x922f0101,0xfb410108,0x92300101,0xfb410108,0x92310101,0xfb410108,0x92320101,
	0xfb410108,0x92330101,0xfb410108,0x92340101,0xfb410108,0x92350101,0xfb410108,0x92360101,
	0xfb410108,0x92370101,0xfb410108,0x92380101,0xfb410108,0x92390101,0xfb410108,0x923a0101,
	0xfb410108,0x923b0101,0xfb410108,0x923c0101,0xfb410108,0x923d0101,0xfb410108,0x923e0101,
	0xfb410108,0x923f0101,0xfb410108,0x92400101,0xfb410108,0x92410101,0xfb410108,0x92420101,
	0xfb410108,0x92430101,0xfb410108,0x92440101,0xfb410108,0x92450101,0xfb410108,0x92460101,
	0xfb410108,0x92470101,0xfb410108,0x92480101,0xfb410108,0x92490101,0xfb410108,0x924a0101,
	0xfb410108,0x924b0101,0xfb410108,0x924c0101,0xfb410108,0x924d0101,0xfb410108,0x924e0101,
	0xfb410108,0x924f0101,0xfb410108,0x92500101,0xfb410108,0x92510101,0xfb410108,0x92520101,
	0xfb410108,0x92530101,0xfb410108,0x92540101,0xfb410108,0x92550101,0xfb410108,0x92560101,
	0xfb410108,0x92570101,0xfb410108,0x92580101,0xfb410108,0x92590101,0xfb410108,0x925a0101,
	0xfb410108,0x925b0101,0xfb410108,0x925c0101,0xfb410108,0x925d0101,0xfb410108,0x925e0101,
	0xfb410108,0x925f0101,0xfb410108,0x92600101,0xfb410108,0x92610101,0xfb410108,0x92620101,
	0xfb410108,0x92630101,0xfb410108,0x92640101,0xfb410108,0x92650101,0xfb410108,0x92660101,
	0xfb410108,0x92670101,0xfb410108,0x92680101,0xfb410108,0x92690101,0xfb410108,0x926a0101,
	0xfb410108,0x926b0101,0xfb410108,0x926c0101,0xfb410108,0x926d0101,0xfb410108,0x926e0101,
	0xfb410108,0x926f0101,0xfb410108,0x92700101,0xfb410108,0x92710101,0xfb410108,0x92720101,
	0xfb410108,0x92730101,0xfb410108,0x92750101,0xfb410108,0x92760101,0xfb410108,0x92770101,
	0xfb410108,0x92780101,0xfb410108,0x92790101,0xfb410108,0x927a0101,0xfb410108,0x927b0101,
	0xfb410108,0x927c0101,0xfb410108,0x927d0101,0xfb410108,0x927e0101,0xfb410108,0x927f0101,
	0xfb410108,0x92800101,0xfb410108,0x92810101,0xfb410108,0x92820101,0xfb410108,0x92830101,
	0xfb410108,0x92840101,0xfb410108,0x92850101,0xfb410108,0x92860101,0xfb410108,0x92870101,
	0xfb410108,0x92880101,0xfb410108,0x92890101,0xfb410108,0x928a0101,0xfb410108,0x928b0101,
	0xfb410108,0x928c0101,0xfb410108,0x928d0101,0xfb410108,0x928f0101,0xfb410108,0x92900101,
	0xfb410108,0x92910101,0xfb410108,0x92920101,0xfb410108,0x92930101,0xfb410108,0x92940101,
	0xfb410108,0x92950101,0xfb410108,0x92960101,0xfb410108,0x92970101,0xfb410108,0x92980101,
	0xfb410108,0x92990101,0xfb410108,0x929a0101,0xfb410108,0x929b0101,0xfb410108,0x929c0101,
	0xfb410108,0x929d0101,0xfb410108,0x929e0101,0xfb410108,0x929f0101,0xfb410108,0x92a00101,
	0xfb410108,0x92a10101,0xfb410108,0x92a20101,0xfb410108,0x92a30101,0xfb410108,0x92a40101,
	0xfb410108,0x92a50101,0xfb410108,0x92a60101,0xfb410108,0x92a70101,0xfb410108,0x92a80101,
	0xfb410108,0x92a90101,0xfb410108,0x92aa0101,0xfb410108,0x92ab0101,0xfb410108,0x92ac0101,
	0xfb410108,0x92ad0101,0xfb410108,0x92af0101,0xfb410108,0x92b00101,0xfb410108,0x92b10101,
	0xfb410108,0x92b20101,0xfb410108,0x92b30101,0xfb410108,0x92b40101,0xfb410108,0x92b50101,
	0xfb410108,0x92b60101,0xfb410108,0x92b70101,0xfb410108,0x92b80101,0xfb410108,0x92b90101,
	0xfb410108,0x92ba0101,0xfb410108,0x92bb0101,0xfb410108,0x92bc0101,0xfb410108,0x92bd0101,
	0xfb410108,0x92be0101,0xfb410108,0x92bf0101,0xfb410108,0x92c00101,0xfb410108,0x92c10101,
	0xfb410108,0x92c20101,0xfb410108,0x92c30101,0xfb410108,0x92c40101,0xfb410108,0x92c50101,
	0xfb410108,0x92c60101,0xfb410108,0x92c70101,0xfb410108,0x92c90101,0xfb410108,0x92ca0101,
	0xfb410108,0x92cb0101,0xfb410108,0x92cc0101,0xfb410108,0x92cd0101,0xfb410108,0x92ce0101,
	0xfb410108,0x92cf0101,0xfb410108,0x92d00101,0xfb410108,0x92d10101,0xfb410108,0x92d20101,
	0xfb410108,0x92d30101,0xfb410108,0x92d40101,0xfb410108,0x92d50101,0xfb410108,0x92d60101,
	0xfb410108,0x92d70101,0xfb410108,0x92d80101,0xfb410108,0x92d90101,0xfb410108,0x92da0101,
	0xfb410108,0x92db0101,0xfb410108,0x92dc0101,0xfb410108,0x92dd0101,0xfb410108,0x92de0101,
	0xfb410108,0x92df0101,0xfb410108,0x92e00101,0xfb410108,0x92e10101,0xfb410108,0x92e20101,
	0xfb410108,0x92e30101,0xfb410108,0x92e40101,0xfb410108,0x92e50101,0xfb410108,0x92e60101,
	0xfb410108,0x92e70101,0xfb410108,0x92e80101,0xfb410108,0x92e90101,0xfb410108,0x92ea0101,
	0xfb410108,0x92eb0101,0xfb410108,0x92ec0101,0xfb410108,0x92ed0101,0xfb410108,0x92ee0101,
	0xfb410108,0x92ef0101,0xfb410108,0x92f00101,0xfb410108,0x92f10101,0xfb410108,0x92f20101,
	0xfb410108,0x92f30101,0xfb410108,0x92f40101,0xfb410108,0x92f50101,0xfb410108,0x92f60101,
	0xfb410108,0x92f70101,0xfb410108,0x92f80101,0xfb410108,0x92f90101,0xfb410108,0x92fa0101,
	0xfb410108,0x92fb0101,0xfb410108,0x92fc0101,0xfb410108,0x92fd0101,0xfb410108,0x92fe0101,
	0xfb410108,0x92ff0101,0xfb410108,0x93000101,0xfb410108,0x93010101,0xfb410108,0x93020101,
	0xfb410108,0x93030101,0xfb410108,0x93040101,0xfb410108,0x93050101,0xfb410108,0x93060101,
	0xfb410108,0x93070101,0xfb410108,0x93080101,0xfb410108,0x93090101,0xfb410108,0x930a0101,
	0xfb410108,0x930b0101,0xfb410108,0x930c0101,0xfb410108,0x930d0101,0xfb410108,0x930e0101,
	0xfb410108,0x930f0101,0xfb410108,0x93100101,0xfb410108,0x93110101,0xfb410108,0x93120101,
	0xfb410108,0x93130101,0xfb410108,0x93140101,0xfb410108,0x93150101,0xfb410108,0x93160101,
	0xfb410108,0x93170101,0xfb410108,0x93180101,0xfb410108,0x93190101,0xfb410108,0x931a0101,
	0xfb410108,0x931b0101,0xfb410108,0x931c0101,0xfb410108,0x931d0101,0xfb410108,0x931e0101,
	0xfb410108,0x931f0101,0xfb410108,0x93200101,0xfb410108,0x93210101,0xfb410108,0x93220101,
	0xfb410108,0x93230101,0xfb410108,0x93240101,0xfb410108,0x93250101,0xfb410108,0x93260101,
	0xfb410108,0x93270101,0xfb410108,0x93280101,0xfb410108,0x93290101,0xfb410108,0x932a0101,
	0xfb410108,0x932b0101,0xfb410108,0x932c0101,0xfb410108,0x932d0101,0xfb410108,0x932e0101,
	0xfb410108,0x932f0101,0xfb410108,0x93300101,0xfb410108,0x93310101,0xfb410108,0x93320101,
	0xfb410108,0x93330101,0xfb410108,0x93340101,0xfb410108,0x93350101,0xfb410108,0x93360101,
	0xfb410108,0x93370101,0xfb410108,0x93380101,0xfb410108,0x93390101,0xfb410108,0x933a0101,
	0xfb410108,0x933b0101,0xfb410108,0x933c0101,0xfb410108,0x933d0101,0xfb410108,0x933f0101,
	0xfb410108,0x93400101,0xfb410108,0x93410101,0xfb410108,0x93420101,0xfb410108,0x93430101,
	0xfb410108,0x93440101,0xfb410108,0x93450101,0xfb410108,0x93460101,0xfb410108,0x93470101,
	0xfb410108,0x93480101,0xfb410108,0x93490101,0xfb410108,0x934a0101,0xfb410108,0x934b0101,
	0xfb410108,0x934c0101,0xfb410108,0x934d0101,0xfb410108,0x934e0101,0xfb410108,0x934f0101,
	0xfb410108,0x93500101,0xfb410108,0x93510101,0xfb410108,0x93520101,0xfb410108,0x93530101,
	0xfb410108,0x93540101,0xfb410108,0x93550101,0xfb410108,0x93560101,0xfb410108,0x93570101,
	0xfb410108,0x93580101,0xfb410108,0x93590101,0xfb410108,0x935a0101,0xfb410108,0x935b0101,
	0xfb410108,0x935c0101,0xfb410108,0x935d0101,0xfb410108,0x935e0101,0xfb410108,0x935f0101,
	0xfb410108,0x93600101,0xfb410108,0x93610101,0xfb410108,0x93620101,0xfb410108,0x93630101,
	0xfb410108,0x93640101,0xfb410108,0x93650101,0xfb410108,0x93660101,0xfb410108,0x93670101,
	0xfb410108,0x93680101,0xfb410108,0x93690101,0xfb410108,0x936b0101,0xfb410108,0x936c0101,
	0xfb410108,0x936d0101,0xfb410108,0x936e0101,0xfb410108,0x936f0101,0xfb410108,0x93700101,
	0xfb410108,0x93710101,0xfb410108,0x93720101,0xfb410108,0x93730101,0xfb410108,0x93740101,
	0xfb410108,0x93750101,0xfb410108,0x93760101,0xfb410108,0x93770101,0xfb410108,0x93780101,
	0xfb410108,0x93790101,0xfb410108,0x937a0101,0xfb410108,0x937b0101,0xfb410108,0x937c0101,
	0xfb410108,0x937d0101,0xfb410108,0x937e0101,0xfb410108,0x937f0101,0xfb410108,0x93800101,
	0xfb410108,0x93810101,0xfb410108,0x93820101,0xfb410108,0x93830101,0xfb410108,0x93840101,
	0xfb410108,0x93850101,0xfb410108,0x93860101,0xfb410108,0x93870101,0xfb410108,0x93880101,
	0xfb410108,0x93890101,0xfb410108,0x938a0101,0xfb410108,0x938b0101,0xfb410108,0x938c0101,
	0xfb410108,0x938d0101,0xfb410108,0x938e0101,0xfb410108,0x93900101,0xfb410108,0x93910101,
	0xfb410108,0x93920101,0xfb410108,0x93930101,0xfb410108,0x93940101,0xfb410108,0x93950101,
	0xfb410108,0x93960101,0xfb410108,0x93970101,0xfb410108,0x93980101,0xfb410108,0x93990101,
	0xfb410108,0x939a0101,0xfb410108,0x939b0101,0xfb410108,0x939c0101,0xfb410108,0x939d0101,
	0xfb410108,0x939e0101,0xfb410108,0x939f0101,0xfb410108,0x93a00101,0xfb410108,0x93a10101,
	0xfb410108,0x93a20101,0xfb410108,0x93a30101,0xfb410108,0x93a40101,0xfb410108,0x93a50101,
	0xfb410108,0x93a60101,0xfb410108,0x93a70101,0xfb410108,0x93a80101,0xfb410108,0x93a90101,
	0xfb410108,0x93aa0101,0xfb410108,0x93ab0101,0xfb410108,0x93ac0101,0xfb410108,0x93ad0101,
	0xfb410108,0x93ae0101,0xfb410108,0x93af0101,0xfb410108,0x93b00101,0xfb410108,0x93b10101,
	0xfb410108,0x93b20101,0xfb410108,0x93b30101,0xfb410108,0x93b40101,0xfb410108,0x93b50101,
	0xfb410108,0x93b60101,0xfb410108,0x93b70101,0xfb410108,0x93b80101,0xfb410108,0x93b90101,
	0xfb410108,0x93ba0101,0xfb410108,0x93bb0101,0xfb410108,0x93bc0101,0xfb410108,0x93bd0101,
	0xfb410108,0x93be0101,0xfb410108,0x93bf0101,0xfb410108,0x93c00101,0xfb410108,0x93c10101,
	0xfb410108,0x93c20101,0xfb410108,0x93c30101,0xfb410108,0x93c40101,0xfb410108,0x93c50101,
	0xfb410108,0x93c60101,0xfb410108,0x93c70101,0xfb410108,0x93c80101,0xfb410108,0x93c90101,
	0xfb410108,0x93cb0101,0xfb410108,0x93cc0101,0xfb410108,0x93cd0101,0xfb410108,0x93ce0101,
	0xfb410108,0x93cf0101,0xfb410108,0x93d00101,0xfb410108,0x93d10101,0xfb410108,0x93d20101,
	0xfb410108,0x93d30101,0xfb410108,0x93d40101,0xfb410108,0x93d50101,0xfb410108,0x93d70101,
	0xfb410108,0x93d80101,0xfb410108,0x93d90101,0xfb410108,0x93da0101,0xfb410108,0x93db0101,
	0xfb410108,0x93dc0101,0xfb410108,0x93dd0101,0xfb410108,0x93de0101,0xfb410108,0x93df0101,
	0xfb410108,0x93e00101,0xfb410108,0x93e10101,0xfb410108,0x93e20101,0xfb410108,0x93e30101,
	0xfb410108,0x93e40101,0xfb410108,0x93e50101,0xfb410108,0x93e60101,0xfb410108,0x93e70101,
	0xfb410108,0x93e80101,0xfb410108,0x93e90101,0xfb410108,0x93ea0101,0xfb410108,0x93eb0101,
	0xfb410108,0x93ec0101,0xfb410108,0x93ed0101,0xfb410108,0x93ee0101,0xfb410108,0x93ef0101,
	0xfb410108,0x93f00101,0xfb410108,0x93f10101,0xfb410108,0x93f20101,0xfb410108,0x93f30101,
	0xfb410108,0x93f40101,0xfb410108,0x93f50101,0xfb410108,0x93f60101,0xfb410108,0x93f70101,
	0xfb410108,0x93f80101,0xfb410108,0x93f90101,0xfb410108,0x93fa0101,0xfb410108,0x93fb0101,
	0xfb410108,0x93fc0101,0xfb410108,0x93fd0101,0xfb410108,0x93fe0101,0xfb410108,0x93ff0101,
	0xfb410108,0x94000101,0xfb410108,0x94010101,0xfb410108,0x94020101,0xfb410108,0x94030101,
	0xfb410108,0x94040101,0xfb410108,0x94050101,0xfb410108,0x94060101,0xfb410108,0x94070101,
	0xfb410108,0x94080101,0xfb410108,0x94090101,0xfb410108,0x940a0101,0xfb410108,0x940b0101,
	0xfb410108,0x940c0101,0xfb410108,0x940d0101,0xfb410108,0x940e0101,0xfb410108,0x940f0101,
	0xfb410108,0x94100101,0xfb410108,0x94110101,0xfb410108,0x94120101,0xfb410108,0x94130101,
	0xfb410108,0x94140101,0xfb410108,0x94150101,0xfb410108,0x94160101,0xfb410108,0x94170101,
	0xfb410108,0x94180101,0xfb410108,0x94190101,0xfb410108,0x941a0101,0xfb410108,0x941b0101,
	0xfb410108,0x941c0101,0xfb410108,0x941d0101,0xfb410108,0x941e0101,0xfb410108,0x941f0101,
	0xfb410108,0x94200101,0xfb410108,0x94210101,0xfb410108,0x94220101,0xfb410108,0x94230101,
	0xfb410108,0x94240101,0xfb410108,0x94250101,0xfb410108,0x94260101,0xfb410108,0x94270101,
	0xfb410108,0x94280101,0xfb410108,0x94290101,0xfb410108,0x942a0101,0xfb410108,0x942b0101,
	0xfb410108,0x942c0101,0xfb410108,0x942d0101,0xfb410108,0x942e0101,0xfb410108,0x942f0101,
	0xfb410108,0x94300101,0xfb410108,0x94310101,0xfb410108,0x94320101,0xfb410108,0x94330101,
	0xfb410108,0x94340101,0xfb410108,0x94350101,0xfb410108,0x94360101,0xfb410108,0x94370101,
	0xfb410108,0x94380101,0xfb410108,0x94390101,0xfb410108,0x943a0101,0xfb410108,0x943b0101,
	0xfb410108,0x943c0101,0xfb410108,0x943d0101,0xfb410108,0x943f0101,0xfb410108,0x94400101,
	0xfb410108,0x94410101,0xfb410108,0x94420101,0xfb410108,0x94430101,0xfb410108,0x94440101,
	0xfb410108,0x94450101,0xfb410108,0x94460101,0xfb410108,0x94470101,0xfb410108,0x94480101,
	0xfb410108,0x94490101,0xfb410108,0x944a0101,0xfb410108,0x944b0101,0xfb410108,0x944c0101,
	0xfb410108,0x944d0101,0xfb410108,0x944e0101,0xfb410108,0x944f0101,0xfb410108,0x94500101,
	0xfb410108,0x94510101,0xfb410108,0x94520101,0xfb410108,0x94530101,0xfb410108,0x94540101,
	0xfb410108,0x94550101,0xfb410108,0x94560101,0xfb410108,0x94570101,0xfb410108,0x94580101,
	0xfb410108,0x94590101,0xfb410108,0x945a0101,0xfb410108,0x945b0101,0xfb410108,0x945c0101,
	0xfb410108,0x945d0101,0xfb410108,0x945e0101,0xfb410108,0x945f0101,0xfb410108,0x94600101,
	0xfb410108,0x94610101,0xfb410108,0x94620101,0xfb410108,0x94630101,0xfb410108,0x94640101,
	0xfb410108,0x94650101,0xfb410108,0x94660101,0xfb410108,0x94670101,0xfb410108,0x94680101,
	0xfb410108,0x94690101,0xfb410108,0x946a0101,0xfb410108,0x946c0101,0xfb410108,0x946d0101,
	0xfb410108,0x946e0101,0xfb410108,0x946f0101,0xfb410108,0x94700101,0xfb410108,0x94710101,
	0xfb410108,0x94720101,0xfb410108,0x94730101,0xfb410108,0x94740101,0xfb410108,0x94750101,
	0xfb410108,0x94760101,0xfb410108,0x94770101,0xfb410108,0x94780101,0xfb410108,0x94790101,
	0xfb410108,0x947a0101,0xfb410108,0x947b0101,0xfb410108,0x947c0101,0xfb410108,0x947d0101,
	0xfb410108,0x947e0101,0xfb410108,0x947f0101,0xfb410108,0x94800101,0xfb410108,0x94810101,
	0xfb410108,0x94820101,0xfb410108,0x94830101,0xfb410108,0x94840101,0xfb410108,0x94910101,
	0xfb410108,0x94960101,0xfb410108,0x94980101,0xfb410108,0x94c70101,0xfb410108,0x94cf0101,
	0xfb410108,0x94d30101,0xfb410108,0x94d40101,0xfb410108,0x94da0101,0xfb410108,0x94e60101,
	0xfb410108,0x94fb0101,0xfb410108,0x951c0101,0xfb410108,0x95200101,0xfb410108,0x95270101,
	0xfb410108,0x95330101,0xfb410108,0x953d0101,0xfb410108,0x95430101,0xfb410108,0x95480101,
	0xfb410108,0x954b0101,0xfb410108,0x95550101,0xfb410108,0x955a0101,0xfb410108,0x95600101,
	0xfb410108,0x956e0101,0xfb410108,0x95740101,0xfb410108,0x95750101,0xfb410108,0x95770101,
	0xfb410108,0x95780101,0xfb410108,0x95790101,0xfb410108,0x957a0101,0xfb410108,0x957b0101,
	0xfb410108,0x957c0101,0xfb410108,0x957d0101,0xfb410108,0x957e0101,0xfb410108,0x95800101,
	0xfb410108,0x95810101,0xfb410108,0x95820101,0xfb410108,0x95830101,0xfb410108,0x95840101,
	0xfb410108,0x95850101,0xfb410108,0x95860101,0xfb410108,0x95870101,0xfb410108,0x95880101,
	0xfb410108,0x95890101,0xfb410108,0x958a0101,0xfb410108,0x958b0101,0xfb410108,0x958c0101,
	0xfb410108,0x958d0101,0xfb410108,0x958e0101,0xfb410108,0x958f0101,0xfb410108,0x95900101,
	0xfb410108,0x95910101,0xfb410108,0x95920101,0xfb410108,0x95930101,0xfb410108,0x95940101,
	0xfb410108,0x95950101,0xfb410108,0x95960101,0xfb410108,0x95970101,0xfb410108,0x95980101,
	0xfb410108,0x95990101,0xfb410108,0x959a0101,0xfb410108,0x959b0101,0xfb410108,0x959c0101,
	0xfb410108,0x959d0101,0xfb410108,0x959e0101,0xfb410108,0x959f0101,0xfb410108,0x95a00101,
	0xfb410108,0x95a10101,0xfb410108,0x95a20101,0xfb410108,0x95a30101,0xfb410108,0x95a40101,
	0xfb410108,0x95a50101,0xfb410108,0x95a60101,0xfb410108,0x95a70101,0xfb410108,0x95a80101,
	0xfb410108,0x95a90101,0xfb410108,0x95aa0101,0xfb410108,0x95ab0101,0xfb410108,0x95ac0101,
	0xfb410108,0x95ad0101,0xfb410108,0x95ae0101,0xfb410108,0x95af0101,0xfb410108,0x95b00101,
	0xfb410108,0x95b10101,0xfb410108,0x95b20101,0xfb410108,0x95b30101,0xfb410108,0x95b40101,
	0xfb410108,0x95b50101,0xfb410108,0x95b60101,0xfb410108,0x95b70101,0xfb410108,0x95b80101,
	0xfb410108,0x95b90101,0xfb410108,0x95ba0101,0xfb410108,0x95bb0101,0xfb410108,0x95bc0101,
	0xfb410108,0x95bd0101,0xfb410108,0x95be0101,0xfb410108,0x95bf0101,0xfb410108,0x95c00101,
	0xfb410108,0x95c10101,0xfb410108,0x95c20101,0xfb410108,0x95c30101,0xfb410108,0x95c40101,
	0xfb410108,0x95c50101,0xfb410108,0x95c60101,0xfb410108,0x95c70101,0xfb410108,0x95c80101,
	0xfb410108,0x95c90101,0xfb410108,0x95ca0101,0xfb410108,0x95cb0101,0xfb410108,0x95cc0101,
	0xfb410108,0x95cd0101,0xfb410108,0x95ce0101,0xfb410108,0x95cf0101,0xfb410108,0x95d00101,
	0xfb410108,0x95d10101,0xfb410108,0x95d20101,0xfb410108,0x95d30101,0xfb410108,0x95d40101,
	0xfb410108,0x95d50101,0xfb410108,0x95d60101,0xfb410108,0x95d70101,0xfb410108,0x95d80101,
	0xfb410108,0x95d90101,0xfb410108,0x95da0101,0xfb410108,0x95db0101,0xfb410108,0x95dc0101,
	0xfb410108,0x95dd0101,0xfb410108,0x95de0101,0xfb410108,0x95df0101,0xfb410108,0x95e00101,
	0xfb410108,0x95e10101,0xfb410108,0x95e20101,0xfb410108,0x95e30101,0xfb410108,0x95e40101,
	0xfb410108,0x95e50101,0xfb410108,0x95e60101,0xfb410108,0x95e70101,0xfb410108,0x95ec0101,
	0xfb410108,0x95ff0101,0xfb410108,0x96070101,0xfb410108,0x96130101,0xfb410108,0x96180101,
	0xfb410108,0x961b0101,0xfb410108,0x961e0101,0xfb410108,0x96200101,0xfb410108,0x96230101,
	0xfb410108,0x96240101,0xfb410108,0x96250101,0xfb410108,0x96260101,0xfb410108,0x96270101,
	0xfb410108,0x96280101,0xfb410108,0x96290101,0xfb410108,0x962b0101,0xfb410108,0x962c0101,
	0xfb410108,0x962d0101,0xfb410108,0x962f0101,0xfb410108,0x96300101,0xfb410108,0x96370101,
	0xfb410108,0x96380101,0xfb410108,0x96390101,0xfb410108,0x963a0101,0xfb410108,0x963e0101,
	0xfb410108,0x96410101,0xfb410108,0x96430101,0xfb410108,0x964a0101,0xfb410108,0x964e0101,
	0xfb410108,0x964f0101,0xfb410108,0x96510101,0xfb410108,0x96520101,0xfb410108,0x96530101,
	0xfb410108,0x96560101,0xfb410108,0x96570101,0xfb410108,0x96580101,0xfb410108,0x96590101,
	0xfb410108,0x965a0101,0xfb410108,0x965c0101,0xfb410108,0x965d0101,0xfb410108,0x965e0101,
	0xfb410108,0x96600101,0xfb410108,0x96630101,0xfb410108,0x96650101,0xfb410108,0x96660101,
	0xfb410108,0x966b0101,0xfb410108,0x966d0101,0xfb410108,0x966e0101,0xfb410108,0x966f0101,
	0xfb410108,0x96700101,0xfb410108,0x96710101,0xfb410108,0x96730101,0xfb410108,0x96780101,
	0xfb410108,0x96790101,0xfb410108,0x967a0101,0xfb410108,0x967b0101,0xfb410108,0x967c0101,
	0xfb410108,0x967d0101,0xfb410108,0x967e0101,0xfb410108,0x967f0101,0xfb410108,0x96800101,
	0xfb410108,0x96810101,0xfb410108,0x96820101,0xfb410108,0x96830101,0xfb410108,0x96840101,
	0xfb410108,0x96870101,0xfb410108,0x96890101,0xfb410108,0x968a0101,0xfb410108,0x968c0101,
	0xfb410108,0x968e0101,0xfb410108,0x96910101,0xfb410108,0x96920101,0xfb410108,0x96930101,
	0xfb410108,0x96950101,0xfb410108,0x96960101,0xfb410108,0x969a0101,0xfb410108,0x969b0101,
	0xfb410108,0x969d0101,0xfb410108,0x969e0101,0xfb410108,0x969f0101,0xfb410108,0x96a00101,
	0xfb410108,0x96a10101,0xfb410108,0x96a20101,0xfb410108,0x96a30101,0xfb410108,0x96a40101,
	0xfb410108,0x96a50101,0xfb410108,0x96a60101,0xfb410108,0x96a80101,0xfb410108,0x96a90101,
	0xfb410108,0x96aa0101,0xfb410108,0x96ab0101,0xfb410108,0x96ac0101,0xfb410108,0x96ad0101,
	0xfb410108,0x96ae0101,0xfb410108,0x96af0101,0xfb410108,0x96b10101,0xfb410108,0x96b20101,
	0xfb410108,0x96b40101,0xfb410108,0x96b50101,0xfb410108,0x96b70101,0xfb410108,0x96b80101,
	0xfb410108,0x96ba0101,0xfb410108,0x96bb0101,0xfb410108,0x96bf0101,0xfb410108,0x96c20101,
	0xfb410108,0x96c30101,0xfb410108,0x96c80101,0xfb410108,0x96ca0101,0xfb410108,0x96cb0101,
	0xfb410108,0x96d00101,0xfb410108,0x96d10101,0xfb410108,0x96d30101,0xfb410108,0x96d40101,
	0xfb410108,0x96d60101,0xfb410108,0x96d70101,0xfb410108,0x96d80101,0xfb410108,0x96d90101,
	0xfb410108,0x96da0101,0xfb410108,0x96db0101,0xfb410108,0x96dc0101,0xfb410108,0x96dd0101,
	0xfb410108,0x96de0101,0xfb410108,0x96df0101,0xfb410108,0x96e10101,0xfb410108,0x96e20101,
	0xfb410108,0x96e30101,0xfb410108,0x96e40101,0xfb410108,0x96e50101,0xfb410108,0x96e60101,
	0xfb410108,0x96e70101,0xfb410108,0x96eb0101,0xfb410108,0x96ec0101,0xfb410108,0x96ed0101,
	0xfb410108,0x96ee0101,0xfb410108,0x96f00101,0xfb410108,0x96f10101,0xfb410108,0x96f20101,
	0xfb410108,0x96f40101,0xfb410108,0x96f50101,0xfb410108,0x96f80101,0xfb410108,0x96fa0101,
	0xfb410108,0x96fb0101,0xfb410108,0x96fc0101,0xfb410108,0x96fd0101,0xfb410108,0x96ff0101,
	0xfb410108,0x97020101,0xfb410108,0x97030101,0xfb410108,0x97050101,0xfb410108,0x970a0101,
	0xfb410108,0x970b0101,0xfb410108,0x970c0101,0xfb410108,0x97100101,0xfb410108,0x97110101,
	0xfb410108,0x97120101,0xfb410108,0x97140101,0xfb410108,0x97150101,0xfb410108,0x97170101,
	0xfb410108,0x97180101,0xfb410108,0x97190101,0xfb410108,0x971a0101,0xfb410108,0x971b0101,
	0xfb410108,0x971d0101,0xfb410108,0x971f0101,0xfb410108,0x97200101,0xfb410108,0x97210101,
	0xfb410108,0x97220101,0xfb410108,0x97230101,0xfb410108,0x97240101,0xfb410108,0x97250101,
	0xfb410108,0x97260101,0xfb410108,0x97270101,0xfb410108,0x97280101,0xfb410108,0x97290101,
	0xfb410108,0x972b0101,0xfb410108,0x972c0101,0xfb410108,0x972e0101,0xfb410108,0x972f0101,
	0xfb410108,0x97310101,0xfb410108,0x97330101,0xfb410108,0x97340101,0xfb410108,0x97350101,
	0xfb410108,0x97360101,0xfb410108,0x97370101,0xfb410108,0x973a0101,0xfb410108,0x973b0101,
	0xfb410108,0x973c0101,0xfb410108,0x973d0101,0xfb410108,0x973f0101,0xfb410108,0x97400101,
	0xfb410108,0x97410101,0xfb410108,0x97420101,0xfb410108,0x97430101,0xfb410108,0x97440101,
	0xfb410108,0x97450101,0xfb410108,0x97460101,0xfb410108,0x97470101,0xfb410108,0x97480101,
	0xfb410108,0x97490101,0xfb410108,0x974a0101,0xfb410108,0x974b0101,0xfb410108,0x974c0101,
	0xfb410108,0x974d0101,0xfb410108,0x974e0101,0xfb410108,0x974f0101,0xfb410108,0x97500101,
	0xfb410108,0x97510101,0xfb410108,0x97540101,0xfb410108,0x97550101,0xfb410108,0x97570101,
	0xfb410108,0x97580101,0xfb410108,0x975a0101,0xfb410108,0x975c0101,0xfb410108,0x975d0101,
	0xfb410108,0x975f0101,0xfb410108,0x97630101,0xfb410108,0x97640101,0xfb410108,0x97660101,
	0xfb410108,0x97670101,0xfb410108,0x97680101,0xfb410108,0x976a0101,0xfb410108,0x976b0101,
	0xfb410108,0x976c0101,0xfb410108,0x976d0101,0xfb410108,0x976e0101,0xfb410108,0x976f0101,
	0xfb410108,0x97700101,0xfb410108,0x97710101,0xfb410108,0x97720101,0xfb410108,0x97750101,
	0xfb410108,0x97770101,0xfb410108,0x97780101,0xfb410108,0x97790101,0xfb410108,0x977a0101,
	0xfb410108,0x977b0101,0xfb410108,0x977d0101,0xfb410108,0x977e0101,0xfb410108,0x977f0101,
	0xfb410108,0x97800101,0xfb410108,0x97810101,0xfb410108,0x97820101,0xfb410108,0x97830101,
	0xfb410108,0x97840101,0xfb410108,0x97860101,0xfb410108,0x97870101,0xfb410108,0x97880101,
	0xfb410108,0x97890101,0xfb410108,0x978a0101,0xfb410108,0x978c0101,0xfb410108,0x978e0101,
	0xfb410108,0x978f0101,0xfb410108,0x97900101,0xfb410108,0x97930101,0xfb410108,0x97950101,
	0xfb410108,0x97960101,0xfb410108,0x97970101,0xfb410108,0x97990101,0xfb410108,0x979a0101,
	0xfb410108,0x979b0101,0xfb410108,0x979c0101,0xfb410108,0x979d0101,0xfb410108,0x979e0101,
	0xfb410108,0x979f0101,0xfb410108,0x97a10101,0xfb410108,0x97a20101,0xfb410108,0x97a40101,
	0xfb410108,0x97a50101,0xfb410108,0x97a60101,0xfb410108,0x97a70101,0xfb410108,0x97a80101,
	0xfb410108,0x97a90101,0xfb410108,0x97aa0101,0xfb410108,0x97ac0101,0xfb410108,0x97ae0101,
	0xfb410108,0x97b00101,0xfb410108,0x97b10101,0xfb410108,0x97b30101,0xfb410108,0x97b50101,
	0xfb410108,0x97b60101,0xfb410108,0x97b70101,0xfb410108,0x97b80101,0xfb410108,0x97b90101,
	0xfb410108,0x97ba0101,0xfb410108,0x97bb0101,0xfb410108,0x97bc0101,0xfb410108,0x97bd0101,
	0xfb410108,0x97be0101,0xfb410108,0x97bf0101,0xfb410108,0x97c00101,0xfb410108,0x97c10101,
	0xfb410108,0x97c20101,0xfb410108,0x97c30101,0xfb410108,0x97c40101,0xfb410108,0x97c50101,
	0xfb410108,0x97c60101,0xfb410108,0x97c70101,0xfb410108,0x97c80101,0xfb410108,0x97c90101,
	0xfb410108,0x97ca0101,0xfb410108,0x97cb0101,0xfb410108,0x97cc0101,0xfb410108,0x97cd0101,
	0xfb410108,0x97ce0101,0xfb410108,0x97cf0101,0xfb410108,0x97d00101,0xfb410108,0x97d10101,
	0xfb410108,0x97d20101,0xfb410108,0x97d30101,0xfb410108,0x97d40101,0xfb410108,0x97d50101,
	0xfb410108,0x97d60101,0xfb410108,0x97d70101,0xfb410108,0x97d80101,0xfb410108,0x97d90101,
	0xfb410108,0x97da0101,0xfb410108,0x97db0101,0xfb410108,0x97dc0101,0xfb410108,0x97dd0101,
	0xfb410108,0x97de0101,0xfb410108,0x97df0101,0xfb410108,0x97e00101,0xfb410108,0x97e10101,
	0xfb410108,0x97e20101,0xfb410108,0x97e30101,0xfb410108,0x97e40101,0xfb410108,0x97e50101,
	0xfb410108,0x97e80101,0xfb410108,0x97ee0101,0xfb410108,0x97ef0101,0xfb410108,0x97f00101,
	0xfb410108,0x97f10101,0xfb410108,0x97f20101,0xfb410108,0x97f40101,0xfb410108,0x97f70101,
	0xfb410108,0x97f80101,0xfb410108,0x97f90101,0xfb410108,0x97fa0101,0xfb410108,0x97fb0101,
	0xfb410108,0x97fc0101,0xfb410108,0x97fd0101,0xfb410108,0x97fe0101,0xfb410108,0x97ff0101,
	0xfb410108,0x98000101,0xfb410108,0x98010101,0xfb410108,0x98020101,0xfb410108,0x98030101,
	0xfb410108,0x98040101,0xfb410108,0x98050101,0xfb410108,0x98060101,0xfb410108,0x98070101,
	0xfb410108,0x98080101,0xfb410108,0x98090101,0xfb410108,0x980a0101,0xfb410108,0x980b0101,
	0xfb410108,0x980c0101,0xfb410108,0x980d0101,0xfb410108,0x980e0101,0xfb410108,0x980f0101,
	0xfb410108,0x98100101,0xfb410108,0x98110101,0xfb410108,0x98120101,0xfb410108,0x98130101,
	0xfb410108,0x98140101,0xfb410108,0x98150101,0xfb410108,0x98160101,0xfb410108,0x98170101,
	0xfb410108,0x98180101,0xfb410108,0x98190101,0xfb410108,0x981a0101,0xfb410108,0x981b0101,
	0xfb410108,0x981c0101,0xfb410108,0x981d0101,0xfb410108,0x981e0101,0xfb410108,0x981f0101,
	0xfb410108,0x98200101,0xfb410108,0x98210101,0xfb410108,0x98220101,0xfb410108,0x98230101,
	0xfb410108,0x98240101,0xfb410108,0x98250101,0xfb410108,0x98260101,0xfb410108,0x98270101,
	0xfb410108,0x98280101,0xfb410108,0x98290101,0xfb410108,0x982a0101,0xfb410108,0x982b0101,
	0xfb410108,0x982c0101,0xfb410108,0x982d0101,0xfb410108,0x982e0101,0xfb410108,0x982f0101,
	0xfb410108,0x98300101,0xfb410108,0x98310101,0xfb410108,0x98320101,0xfb410108,0x98330101,
	0xfb410108,0x98340101,0xfb410108,0x98350101,0xfb410108,0x98360101,0xfb410108,0x98370101,
	0xfb410108,0x98380101,0xfb410108,0x98390101,0xfb410108,0x983a0101,0xfb410108,0x983b0101,
	0xfb410108,0x983c0101,0xfb410108,0x983d0101,0xfb410108,0x983e0101,0xfb410108,0x983f0101,
	0xfb410108,0x98400101,0xfb410108,0x98410101,0xfb410108,0x98420101,0xfb410108,0x98430101,
	0xfb410108,0x98440101,0xfb410108,0x98450101,0xfb410108,0x98460101,0xfb410108,0x98470101,
	0xfb410108,0x98480101,0xfb410108,0x98490101,0xfb410108,0x984a0101,0xfb410108,0x984b0101,
	0xfb410108,0x984c0101,0xfb410108,0x984d0101,0xfb410108,0x984e0101,0xfb410108,0x984f0101,
	0xfb410108,0x98500101,0xfb410108,0x98510101,0xfb410108,0x98520101,0xfb410108,0x98530101,
	0xfb410108,0x98540101,0xfb410108,0x98550101,0xfb410108,0x98560101,0xfb410108,0x98570101,
	0xfb410108,0x98580101,0xfb410108,0x98590101,0xfb410108,0x985a0101,0xfb410108,0x985b0101,
	0xfb410108,0x985c0101,0xfb410108,0x985d0101,0xfb410108,0x985e0101,0xfb410108,0x985f0101,
	0xfb410108,0x98600101,0xfb410108,0x98610101,0xfb410108,0x98620101,0xfb410108,0x98630101,
	0xfb410108,0x98640101,0xfb410108,0x98650101,0xfb410108,0x98660101,0xfb410108,0x98670101,
	0xfb410108,0x98680101,0xfb410108,0x98690101,0xfb410108,0x986a0101,0xfb410108,0x986b0101,
	0xfb410108,0x986c0101,0xfb410108,0x986d0101,0xfb410108,0x986e0101,0xfb410108,0x986f0101,
	0xfb410108,0x98700101,0xfb410108,0x98710101,0xfb410108,0x98720101,0xfb410108,0x98730101,
	0xfb410108,0x98740101,0xfb410108,0x988b0101,0xfb410108,0x988e0101,0xfb410108,0x98920101,
	0xfb410108,0x98950101,0xfb410108,0x98990101,0xfb410108,0x98a30101,0xfb410108,0x98a80101,
	0xfb410108,0x98a90101,0xfb410108,0x98aa0101,0xfb410108,0x98ab0101,0xfb410108,0x98ac0101,
	0xfb410108,0x98ad0101,0xfb410108,0x98ae0101,0xfb410108,0x98af0101,0xfb410108,0x98b00101,
	0xfb410108,0x98b10101,0xfb410108,0x98b20101,0xfb410108,0x98b30101,0xfb410108,0x98b40101,
	0xfb410108,0x98b50101,0xfb410108,0x98b60101,0xfb410108,0x98b70101,0xfb410108,0x98b80101,
	0xfb410108,0x98b90101,0xfb410108,0x98ba0101,0xfb410108,0x98bb0101,0xfb410108,0x98bc0101,
	0xfb410108,0x98bd0101,0xfb410108,0x98be0101,0xfb410108,0x98bf0101,0xfb410108,0x98c00101,
	0xfb410108,0x98c10101,0xfb410108,0x98c20101,0xfb410108,0x98c30101,0xfb410108,0x98c40101,
	0xfb410108,0x98c50101,0xfb410108,0x98c60101,0xfb410108,0x98c70101,0xfb410108,0x98c80101,
	0xfb410108,0x98c90101,0xfb410108,0x98ca0101,0xfb410108,0x98cb0101,0xfb410108,0x98cc0101,
	0xfb410108,0x98cd0101,0xfb410108,0x98cf0101,0xfb410108,0x98d00101,0xfb410108,0x98d40101,
	0xfb410108,0x98d60101,0xfb410108,0x98d70101,0xfb410108,0x98db0101,0xfb410108,0x98dc0101,
	0xfb410108,0x98dd0101,0xfb410108,0x98e00101,0xfb410108,0x98e10101,0xfb410108,0x98e20101,
	0xfb410108,0x98e30101,0xfb410108,0x98e40101,0xfb410108,0x98e50101,0xfb410108,0x98e60101,
	0xfb410108,0x98e90101,0xfb410108,0x98ea0101,0xfb410108,0x98eb0101,0xfb410108,0x98ec0101,
	0xfb410108,0x98ed0101,0xfb410108,0x98ee0101,0xfb410108,0x98ef0101,0xfb410108,0x98f00101,
	0xfb410108,0x98f10101,0xfb410108,0x98f20101,0xfb410108,0x98f30101,0xfb410108,0x98f40101,
	0xfb410108,0x98f50101,0xfb410108,0x98f60101,0xfb410108,0x98f70101,0xfb410108,0x98f80101,
	0xfb410108,0x98f90101,0xfb410108,0x98fa0101,0xfb410108,0x98fb0101,0xfb410108,0x98fc0101,
	0xfb410108,0x98fd0101,0xfb410108,0x98fe0101,0xfb410108,0x98ff0101,0xfb410108,0x99000101,
	0xfb410108,0x99010101,0xfb410108,0x99020101,0xfb410108,0x99030101,0xfb410108,0x99040101,
	0xfb410108,0x99050101,0xfb410108,0x99060101,0xfb410108,0x99070101,0xfb410108,0x99080101,
	0xfb410108,0x99090101,0xfb410108,0x990a0101,0xfb410108,0x990b0101,0xfb410108,0x990c0101,
	0xfb410108,0x990e0101,0xfb410108,0x990f0101,0xfb410108,0x99110101,0xfb410108,0x99120101,
	0xfb410108,0x99130101,0xfb410108,0x99140101,0xfb410108,0x99150101,0xfb410108,0x99160101,
	0xfb410108,0x99170101,0xfb410108,0x99180101,0xfb410108,0x99190101,0xfb410108,0x991a0101,
	0xfb410108,0x991b0101,0xfb410108,0x991c0101,0xfb410108,0x991d0101,0xfb410108,0x991e0101,
	0xfb410108,0x991f0101,0xfb410108,0x99200101,0xfb410108,0x99210101,0xfb410108,0x99220101,
	0xfb410108,0x99230101,0xfb410108,0x99240101,0xfb410108,0x99250101,0xfb410108,0x99260101,
	0xfb410108,0x99270101,0xfb410108,0x99280101,0xfb410108,0x99290101,0xfb410108,0x992a0101,
	0xfb410108,0x992b0101,0xfb410108,0x992c0101,0xfb410108,0x992d0101,0xfb410108,0x992f0101,
	0xfb410108,0x99300101,0xfb410108,0x99310101,0xfb410108,0x99320101,0xfb410108,0x99330101,
	0xfb410108,0x99340101,0xfb410108,0x99350101,0xfb410108,0x99360101,0xfb410108,0x99370101,
	0xfb410108,0x99380101,0xfb410108,0x99390101,0xfb410108,0x993a0101,0xfb410108,0x993b0101,
	0xfb410108,0x993c0101,0xfb410108,0x993d0101,0xfb410108,0x993e0101,0xfb410108,0x993f0101,
	0xfb410108,0x99400101,0xfb410108,0x99410101,0xfb410108,0x99420101,0xfb410108,0x99430101,
	0xfb410108,0x99440101,0xfb410108,0x99450101,0xfb410108,0x99460101,0xfb410108,0x99470101,
	0xfb410108,0x99480101,0xfb410108,0x99490101,0xfb410108,0x994a0101,0xfb410108,0x994b0101,
	0xfb410108,0x994c0101,0xfb410108,0x994d0101,0xfb410108,0x994e0101,0xfb410108,0x994f0101,
	0xfb410108,0x99500101,0xfb410108,0x99510101,0xfb410108,0x99520101,0xfb410108,0x99530101,
	0xfb410108,0x99560101,0xfb410108,0x99570101,0xfb410108,0x99580101,0xfb410108,0x99590101,
	0xfb410108,0x995a0101,0xfb410108,0x995b0101,0xfb410108,0x995c0101,0xfb410108,0x995d0101,
	0xfb410108,0x995e0101,0xfb410108,0x995f0101,0xfb410108,0x99600101,0xfb410108,0x99610101,
	0xfb410108,0x99620101,0xfb410108,0x99640101,0xfb410108,0x99660101,0xfb410108,0x99730101,
	0xfb410108,0x99780101,0xfb410108,0x99790101,0xfb410108,0x997b0101,0xfb410108,0x997e0101,
	0xfb410108,0x99820101,0xfb410108,0x99830101,0xfb410108,0x99890101,0xfb410108,0x998c0101,
	0xfb410108,0x998e0101,0xfb410108,0x999a0101,0xfb410108,0x999b0101,0xfb410108,0x999c0101,
	0xfb410108,0x999d0101,0xfb410108,0x999e0101,0xfb410108,0x999f0101,0xfb410108,0x99a00101,
	0xfb410108,0x99a10101,0xfb410108,0x99a20101,0xfb410108,0x99a30101,0xfb410108,0x99a40101,
	0xfb410108,0x99a60101,0xfb410108,0x99a70101,0xfb410108,0x99a90101,0xfb410108,0x99aa0101,
	0xfb410108,0x99ab0101,0xfb410108,0x99ac0101,0xfb410108,0x99ad0101,0xfb410108,0x99ae0101,
	0xfb410108,0x99af0101,0xfb410108,0x99b00101,0xfb410108,0x99b10101,0xfb410108,0x99b20101,
	0xfb410108,0x99b30101,0xfb410108,0x99b40101,0xfb410108,0x99b50101,0xfb410108,0x99b60101,
	0xfb410108,0x99b70101,0xfb410108,0x99b80101,0xfb410108,0x99b90101,0xfb410108,0x99ba0101,
	0xfb410108,0x99bb0101,0xfb410108,0x99bc0101,0xfb410108,0x99bd0101,0xfb410108,0x99be0101,
	0xfb410108,0x99bf0101,0xfb410108,0x99c00101,0xfb410108,0x99c10101,0xfb410108,0x99c20101,
	0xfb410108,0x99c30101,0xfb410108,0x99c40101,0xfb410108,0x99c50101,0xfb410108,0x99c60101,
	0xfb410108,0x99c70101,0xfb410108,0x99c80101,0xfb410108,0x99c90101,0xfb410108,0x99ca0101,
	0xfb410108,0x99cb0101,0xfb410108,0x99cc0101,0xfb410108,0x99cd0101,0xfb410108,0x99ce0101,
	0xfb410108,0x99cf0101,0xfb410108,0x99d00101,0xfb410108,0x99d10101,0xfb410108,0x99d20101,
	0xfb410108,0x99d30101,0xfb410108,0x99d40101,0xfb410108,0x99d50101,0xfb410108,0x99d60101,
	0xfb410108,0x99d70101,0xfb410108,0x99d80101,0xfb410108,0x99d90101,0xfb410108,0x99da0101,
	0xfb410108,0x99db0101,0xfb410108,0x99dc0101,0xfb410108,0x99dd0101,0xfb410108,0x99de0101,
	0xfb410108,0x99df0101,0xfb410108,0x99e00101,0xfb410108,0x99e10101,0xfb410108,0x99e20101,
	0xfb410108,0x99e30101,0xfb410108,0x99e40101,0xfb410108,0x99e50101,0xfb410108,0x99e60101,
	0xfb410108,0x99e70101,0xfb410108,0x99e80101,0xfb410108,0x99e90101,0xfb410108,0x99ea0101,
	0xfb410108,0x99eb0101,0xfb410108,0x99ec0101,0xfb410108,0x99ed0101,0xfb410108,0x99ee0101,
	0xfb410108,0x99ef0101,0xfb410108,0x99f00101,0xfb410108,0x99f10101,0xfb410108,0x99f20101,
	0xfb410108,0x99f30101,0xfb410108,0x99f40101,0xfb410108,0x99f50101,0xfb410108,0x99f60101,
	0xfb410108,0x99f70101,0xfb410108,0x99f80101,0xfb410108,0x99f90101,0xfb410108,0x99fa0101,
	0xfb410108,0x99fb0101,0xfb410108,0x99fc0101,0xfb410108,0x99fd0101,0xfb410108,0x99fe0101,
	0xfb410108,0x99ff0101,0xfb410108,0x9a000101,0xfb410108,0x9a010101,0xfb410108,0x9a020101,
	0xfb410108,0x9a030101,0xfb410108,0x9a040101,0xfb410108,0x9a050101,0xfb410108,0x9a060101,
	0xfb410108,0x9a070101,0xfb410108,0x9a080101,0xfb410108,0x9a090101,0xfb410108,0x9a0a0101,
	0xfb410108,0x9a0b0101,0xfb410108,0x9a0c0101,0xfb410108,0x9a0d0101,0xfb410108,0x9a0e0101,
	0xfb410108,0x9a0f0101,0xfb410108,0x9a100101,0xfb410108,0x9a110101,0xfb410108,0x9a120101,
	0xfb410108,0x9a130101,0xfb410108,0x9a140101,0xfb410108,0x9a150101,0xfb410108,0x9a160101,
	0xfb410108,0x9a170101,0xfb410108,0x9a180101,0xfb410108,0x9a190101,0xfb410108,0x9a1a0101,
	0xfb410108,0x9a1b0101,0xfb410108,0x9a1c0101,0xfb410108,0x9a1d0101,0xfb410108,0x9a1e0101,
	0xfb410108,0x9a1f0101,0xfb410108,0x9a200101,0xfb410108,0x9a210101,0xfb410108,0x9a220101,
	0xfb410108,0x9a230101,0xfb410108,0x9a240101,0xfb410108,0x9a250101,0xfb410108,0x9a260101,
	0xfb410108,0x9a270101,0xfb410108,0x9a280101,0xfb410108,0x9a290101,0xfb410108,0x9a2a0101,
	0xfb410108,0x9a2b0101,0xfb410108,0x9a2c0101,0xfb410108,0x9a2d0101,0xfb410108,0x9a2e0101,
	0xfb410108,0x9a2f0101,0xfb410108,0x9a300101,0xfb410108,0x9a310101,0xfb410108,0x9a320101,
	0xfb410108,0x9a330101,0xfb410108,0x9a340101,0xfb410108,0x9a350101,0xfb410108,0x9a360101,
	0xfb410108,0x9a370101,0xfb410108,0x9a380101,0xfb410108,0x9a390101,0xfb410108,0x9a3a0101,
	0xfb410108,0x9a3b0101,0xfb410108,0x9a3c0101,0xfb410108,0x9a3d0101,0xfb410108,0x9a3e0101,
	0xfb410108,0x9a3f0101,0xfb410108,0x9a400101,0xfb410108,0x9a410101,0xfb410108,0x9a420101,
	0xfb410108,0x9a430101,0xfb410108,0x9a440101,0xfb410108,0x9a450101,0xfb410108,0x9a460101,
	0xfb410108,0x9a470101,0xfb410108,0x9a480101,0xfb410108,0x9a490101,0xfb410108,0x9a4a0101,
	0xfb410108,0x9a4b0101,0xfb410108,0x9a4c0101,0xfb410108,0x9a4d0101,0xfb410108,0x9a4e0101,
	0xfb410108,0x9a4f0101,0xfb410108,0x9a500101,0xfb410108,0x9a510101,0xfb410108,0x9a520101,
	0xfb410108,0x9a530101,0xfb410108,0x9a540101,0xfb410108,0x9a550101,0xfb410108,0x9a560101,
	0xfb410108,0x9a570101,0xfb410108,0x9a580101,0xfb410108,0x9a590101,0xfb410108,0x9a5a0101,
	0xfb410108,0x9a5b0101,0xfb410108,0x9a5c0101,0xfb410108,0x9a5d0101,0xfb410108,0x9a5e0101,
	0xfb410108,0x9a5f0101,0xfb410108,0x9a600101,0xfb410108,0x9a610101,0xfb410108,0x9a620101,
	0xfb410108,0x9a630101,0xfb410108,0x9a640101,0xfb410108,0x9a650101,0xfb410108,0x9a660101,
	0xfb410108,0x9a670101,0xfb410108,0x9a680101,0xfb410108,0x9a690101,0xfb410108,0x9a6a0101,
	0xfb410108,0x9a6b0101,0xfb410108,0x9a720101,0xfb410108,0x9a830101,0xfb410108,0x9a890101,
	0xfb410108,0x9a8d0101,0xfb410108,0x9a8e0101,0xfb410108,0x9a940101,0xfb410108,0x9a950101,
	0xfb410108,0x9a990101,0xfb410108,0x9aa60101,0xfb410108,0x9aa90101,0xfb410108,0x9aaa0101,
	0xfb410108,0x9aab0101,0xfb410108,0x9aac0101,0xfb410108,0x9aad0101,0xfb410108,0x9aae0101,
	0xfb410108,0x9aaf0101,0xfb410108,0x9ab20101,0xfb410108,0x9ab30101,0xfb410108,0x9ab40101,
	0xfb410108,0x9ab50101,0xfb410108,0x9ab90101,0xfb410108,0x9abb0101,0xfb410108,0x9abd0101,
	0xfb410108,0x9abe0101,0xfb410108,0x9abf0101,0xfb410108,0x9ac30101,0xfb410108,0x9ac40101,
	0xfb410108,0x9ac60101,0xfb410108,0x9ac70101,0xfb410108,0x9ac80101,0xfb410108,0x9ac90101,
	0xfb410108,0x9aca0101,0xfb410108,0x9acd0101,0xfb410108,0x9ace0101,0xfb410108,0x9acf0101,
	0xfb410108,0x9ad00101,0xfb410108,0x9ad20101,0xfb410108,0x9ad40101,0xfb410108,0x9ad50101,
	0xfb410108,0x9ad60101,0xfb410108,0x9ad70101,0xfb410108,0x9ad90101,0xfb410108,0x9ada0101,
	0xfb410108,0x9adb0101,0xfb410108,0x9adc0101,0xfb410108,0x9add0101,0xfb410108,0x9ade0101,
	0xfb410108,0x9ae00101,0xfb410108,0x9ae20101,0xfb410108,0x9ae30101,0xfb410108,0x9ae40101,
	0xfb410108,0x9ae50101,0xfb410108,0x9ae70101,0xfb410108,0x9ae80101,0xfb410108,0x9ae90101,
	0xfb410108,0x9aea0101,0xfb410108,0x9aec0101,0xfb410108,0x9aee0101,0xfb410108,0x9af00101,
	0xfb410108,0x9af10101,0xfb410108,0x9af20101,0xfb410108,0x9af30101,0xfb410108,0x9af40101,
	0xfb410108,0x9af50101,0xfb410108,0x9af60101,0xfb410108,0x9af70101,0xfb410108,0x9af80101,
	0xfb410108,0x9afa0101,0xfb410108,0x9afc0101,0xfb410108,0x9afd0101,0xfb410108,0x9afe0101,
	0xfb410108,0x9aff0101,0xfb410108,0x9b000101,0xfb410108,0x9b010101,0xfb410108,0x9b020101,
	0xfb410108,0x9b040101,0xfb410108,0x9b050101,0xfb410108,0x9b060101,0xfb410108,0x9b070101,
	0xfb410108,0x9b090101,0xfb410108,0x9b0a0101,0xfb410108,0x9b0b0101,0xfb410108,0x9b0c0101,
	0xfb410108,0x9b0d0101,0xfb410108,0x9b0e0101,0xfb410108,0x9b100101,0xfb410108,0x9b110101,
	0xfb410108,0x9b120101,0xfb410108,0x9b140101,0xfb410108,0x9b150101,0xfb410108,0x9b160101,
	0xfb410108,0x9b170101,0xfb410108,0x9b180101,0xfb410108,0x9b190101,0xfb410108,0x9b1a0101,
	0xfb410108,0x9b1b0101,0xfb410108,0x9b1c0101,0xfb410108,0x9b1d0101,0xfb410108,0x9b1e0101,
	0xfb410108,0x9b200101,0xfb410108,0x9b210101,0xfb410108,0x9b220101,0xfb410108,0x9b240101,
	0xfb410108,0x9b250101,0xfb410108,0x9b260101,0xfb410108,0x9b270101,0xfb410108,0x9b280101,
	0xfb410108,0x9b290101,0xfb410108,0x9b2a0101,0xfb410108,0x9b2b0101,0xfb410108,0x9b2c0101,
	0xfb410108,0x9b2d0101,0xfb410108,0x9b2e0101,0xfb410108,0x9b300101,0xfb410108,0x9b310101,
	0xfb410108,0x9b330101,0xfb410108,0x9b340101,0xfb410108,0x9b350101,0xfb410108,0x9b360101,
	0xfb410108,0x9b370101,0xfb410108,0x9b380101,0xfb410108,0x9b390101,0xfb410108,0x9b3a0101,
	0xfb410108,0x9b3d0101,0xfb410108,0x9b3e0101,0xfb410108,0x9b3f0101,0xfb410108,0x9b400101,
	0xfb410108,0x9b460101,0xfb410108,0x9b4a0101,0xfb410108,0x9b4b0101,0xfb410108,0x9b4c0101,
	0xfb410108,0x9b4e0101,0xfb410108,0x9b500101,0xfb410108,0x9b520101,0xfb410108,0x9b530101,
	0xfb410108,0x9b550101,0xfb410108,0x9b560101,0xfb410108,0x9b570101,0xfb410108,0x9b580101,
	0xfb410108,0x9b590101,0xfb410108,0x9b5a0101,0xfb410108,0x9b5b0101,0xfb410108,0x9b5c0101,
	0xfb410108,0x9b5d0101,0xfb410108,0x9b5e0101,0xfb410108,0x9b5f0101,0xfb410108,0x9b600101,
	0xfb410108,0x9b610101,0xfb410108,0x9b620101,0xfb410108,0x9b630101,0xfb410108,0x9b640101,
	0xfb410108,0x9b650101,0xfb410108,0x9b660101,0xfb410108,0x9b670101,0xfb410108,0x9b680101,
	0xfb410108,0x9b690101,0xfb410108,0x9b6a0101,0xfb410108,0x9b6b0101,0xfb410108,0x9b6c0101,
	0xfb410108,0x9b6d0101,0xfb410108,0x9b6e0101,0xfb410108,0x9b6f0101,0xfb410108,0x9b700101,
	0xfb410108,0x9b710101,0xfb410108,0x9b720101,0xfb410108,0x9b730101,0xfb410108,0x9b740101,
	0xfb410108,0x9b750101,0xfb410108,0x9b760101,0xfb410108,0x9b770101,0xfb410108,0x9b780101,
	0xfb410108,0x9b790101,0xfb410108,0x9b7a0101,0xfb410108,0x9b7b0101,0xfb410108,0x9b7c0101,
	0xfb410108,0x9b7d0101,0xfb410108,0x9b7e0101,0xfb410108,0x9b7f0101,0xfb410108,0x9b800101,
	0xfb410108,0x9b810101,0xfb410108,0x9b820101,0xfb410108,0x9b830101,0xfb410108,0x9b840101,
	0xfb410108,0x9b850101,0xfb410108,0x9b860101,0xfb410108,0x9b870101,0xfb410108,0x9b880101,
	0xfb410108,0x9b890101,0xfb410108,0x9b8a0101,0xfb410108,0x9b8b0101,0xfb410108,0x9b8c0101,
	0xfb410108,0x9b8d0101,0xfb410108,0x9b8e0101,0xfb410108,0x9b8f0101,0xfb410108,0x9b900101,
	0xfb410108,0x9b910101,0xfb410108,0x9b920101,0xfb410108,0x9b930101,0xfb410108,0x9b940101,
	0xfb410108,0x9b950101,0xfb410108,0x9b960101,0xfb410108,0x9b970101,0xfb410108,0x9b980101,
	0xfb410108,0x9b990101,0xfb410108,0x9b9a0101,0xfb410108,0x9b9b0101,0xfb410108,0x9b9c0101,
	0xfb410108,0x9b9d0101,0xfb410108,0x9b9e0101,0xfb410108,0x9b9f0101,0xfb410108,0x9ba00101,
	0xfb410108,0x9ba10101,0xfb410108,0x9ba20101,0xfb410108,0x9ba30101,0xfb410108,0x9ba40101,
	0xfb410108,0x9ba50101,0xfb410108,0x9ba60101,0xfb410108,0x9ba70101,0xfb410108,0x9ba80101,
	0xfb410108,0x9ba90101,0xfb410108,0x9baa0101,0xfb410108,0x9bab0101,0xfb410108,0x9bac0101,
	0xfb410108,0x9bad0101,0xfb410108,0x9bae0101,0xfb410108,0x9baf0101,0xfb410108,0x9bb00101,
	0xfb410108,0x9bb10101,0xfb410108,0x9bb20101,0xfb410108,0x9bb30101,0xfb410108,0x9bb40101,
	0xfb410108,0x9bb50101,0xfb410108,0x9bb60101,0xfb410108,0x9bb70101,0xfb410108,0x9bb80101,
	0xfb410108,0x9bb90101,0xfb410108,0x9bba0101,0xfb410108,0x9bbb0101,0xfb410108,0x9bbc0101,
	0xfb410108,0x9bbd0101,0xfb410108,0x9bbe0101,0xfb410108,0x9bbf0101,0xfb410108,0x9bc00101,
	0xfb410108,0x9bc10101,0xfb410108,0x9bc20101,0xfb410108,0x9bc30101,0xfb410108,0x9bc40101,
	0xfb410108,0x9bc50101,0xfb410108,0x9bc60101,0xfb410108,0x9bc70101,0xfb410108,0x9bc80101,
	0xfb410108,0x9bc90101,0xfb410108,0x9bca0101,0xfb410108,0x9bcb0101,0xfb410108,0x9bcc0101,
	0xfb410108,0x9bcd0101,0xfb410108,0x9bce0101,0xfb410108,0x9bcf0101,0xfb410108,0x9bd00101,
	0xfb410108,0x9bd10101,0xfb410108,0x9bd20101,0xfb410108,0x9bd30101,0xfb410108,0x9bd40101,
	0xfb410108,0x9bd50101,0xfb410108,0x9bd60101,0xfb410108,0x9bd70101,0xfb410108,0x9bd80101,
	0xfb410108,0x9bd90101,0xfb410108,0x9bda0101,0xfb410108,0x9bdb0101,0xfb410108,0x9bdc0101,
	0xfb410108,0x9bdd0101,0xfb410108,0x9bde0101,0xfb410108,0x9bdf0101,0xfb410108,0x9be00101,
	0xfb410108,0x9be10101,0xfb410108,0x9be20101,0xfb410108,0x9be30101,0xfb410108,0x9be40101,
	0xfb410108,0x9be50101,0xfb410108,0x9be60101,0xfb410108,0x9be70101,0xfb410108,0x9be80101,
	0xfb410108,0x9be90101,0xfb410108,0x9bea0101,0xfb410108,0x9beb0101,0xfb410108,0x9bec0101,
	0xfb410108,0x9bed0101,0xfb410108,0x9bee0101,0xfb410108,0x9bef0101,0xfb410108,0x9bf00101,
	0xfb410108,0x9bf10101,0xfb410108,0x9bf20101,0xfb410108,0x9bf30101,0xfb410108,0x9bf40101,
	0xfb410108,0x9bf50101,0xfb410108,0x9bf60101,0xfb410108,0x9bf70101,0xfb410108,0x9bf80101,
	0xfb410108,0x9bf90101,0xfb410108,0x9bfa0101,0xfb410108,0x9bfb0101,0xfb410108,0x9bfc0101,
	0xfb410108,0x9bfd0101,0xfb410108,0x9bfe0101,0xfb410108,0x9bff0101,0xfb410108,0x9c000101,
	0xfb410108,0x9c010101,0xfb410108,0x9c020101,0xfb410108,0x9c030101,0xfb410108,0x9c040101,
	0xfb410108,0x9c050101,0xfb410108,0x9c060101,0xfb410108,0x9c070101,0xfb410108,0x9c080101,
	0xfb410108,0x9c090101,0xfb410108,0x9c0a0101,0xfb410108,0x9c0b0101,0xfb410108,0x9c0c0101,
	0xfb410108,0x9c0d0101,0xfb410108,0x9c0e0101,0xfb410108,0x9c0f0101,0xfb410108,0x9c100101,
	0xfb410108,0x9c110101,0xfb410108,0x9c120101,0xfb410108,0x9c130101,0xfb410108,0x9c140101,
	0xfb410108,0x9c150101,0xfb410108,0x9c160101,0xfb410108,0x9c170101,0xfb410108,0x9c180101,
	0xfb410108,0x9c190101,0xfb410108,0x9c1a0101,0xfb410108,0x9c1b0101,0xfb410108,0x9c1c0101,
	0xfb410108,0x9c1d0101,0xfb410108,0x9c1e0101,0xfb410108,0x9c1f0101,0xfb410108,0x9c200101,
	0xfb410108,0x9c210101,0xfb410108,0x9c220101,0xfb410108,0x9c230101,0xfb410108,0x9c240101,
	0xfb410108,0x9c250101,0xfb410108,0x9c260101,0xfb410108,0x9c270101,0xfb410108,0x9c280101,
	0xfb410108,0x9c290101,0xfb410108,0x9c2a0101,0xfb410108,0x9c2b0101,0xfb410108,0x9c2c0101,
	0xfb410108,0x9c2d0101,0xfb410108,0x9c2e0101,0xfb410108,0x9c2f0101,0xfb410108,0x9c300101,
	0xfb410108,0x9c310101,0xfb410108,0x9c320101,0xfb410108,0x9c330101,0xfb410108,0x9c340101,
	0xfb410108,0x9c350101,0xfb410108,0x9c360101,0xfb410108,0x9c370101,0xfb410108,0x9c380101,
	0xfb410108,0x9c390101,0xfb410108,0x9c3a0101,0xfb410108,0x9c3b0101,0xfb410108,0x9c3c0101,
	0xfb410108,0x9c3d0101,0xfb410108,0x9c3e0101,0xfb410108,0x9c3f0101,0xfb410108,0x9c400101,
	0xfb410108,0x9c410101,0xfb410108,0x9c420101,0xfb410108,0x9c430101,0xfb410108,0x9c440101,
	0xfb410108,0x9c450101,0xfb410108,0x9c460101,0xfb410108,0x9c470101,0xfb410108,0x9c480101,
	0xfb410108,0x9c490101,0xfb410108,0x9c4a0101,0xfb410108,0x9c4b0101,0xfb410108,0x9c4c0101,
	0xfb410108,0x9c4d0101,0xfb410108,0x9c4e0101,0xfb410108,0x9c4f0101,0xfb410108,0x9c500101,
	0xfb410108,0x9c510101,0xfb410108,0x9c520101,0xfb410108,0x9c530101,0xfb410108,0x9c540101,
	0xfb410108,0x9c550101,0xfb410108,0x9c560101,0xfb410108,0x9c570101,0xfb410108,0x9c580101,
	0xfb410108,0x9c590101,0xfb410108,0x9c5a0101,0xfb410108,0x9c5b0101,0xfb410108,0x9c5c0101,
	0xfb410108,0x9c5d0101,0xfb410108,0x9c5e0101,0xfb410108,0x9c5f0101,0xfb410108,0x9c600101,
	0xfb410108,0x9c610101,0xfb410108,0x9c620101,0xfb410108,0x9c630101,0xfb410108,0x9c640101,
	0xfb410108,0x9c650101,0xfb410108,0x9c660101,0xfb410108,0x9c670101,0xfb410108,0x9c680101,
	0xfb410108,0x9c690101,0xfb410108,0x9c6a0101,0xfb410108,0x9c6b0101,0xfb410108,0x9c6c0101,
	0xfb410108,0x9c6d0101,0xfb410108,0x9c6e0101,0xfb410108,0x9c6f0101,0xfb410108,0x9c700101,
	0xfb410108,0x9c710101,0xfb410108,0x9c720101,0xfb410108,0x9c730101,0xfb410108,0x9c740101,
	0xfb410108,0x9c750101,0xfb410108,0x9c760101,0xfb410108,0x9c770101,0xfb410108,0x9c780101,
	0xfb410108,0x9c790101,0xfb410108,0x9c7a0101,0xfb410108,0x9c7b0101,0xfb410108,0x9c7d0101,
	0xfb410108,0x9c7e0101,0xfb410108,0x9c800101,0xfb410108,0x9c830101,0xfb410108,0x9c840101,
	0xfb410108,0x9c890101,0xfb410108,0x9c8a0101,0xfb410108,0x9c8c0101,0xfb410108,0x9c8f0101,
	0xfb410108,0x9c930101,0xfb410108,0x9c960101,0xfb410108,0x9c970101,0xfb410108,0x9c980101,
	0xfb410108,0x9c990101,0xfb410108,0x9c9d0101,0xfb410108,0x9caa0101,0xfb410108,0x9cac0101,
	0xfb410108,0x9caf0101,0xfb410108,0x9cb90101,0xfb410108,0x9cbe0101,0xfb410108,0x9cbf0101,
	0xfb410108,0x9cc00101,0xfb410108,0x9cc10101,0xfb410108,0x9cc20101,0xfb410108,0x9cc80101,
	0xfb410108,0x9cc90101,0xfb410108,0x9cd10101,0xfb410108,0x9cd20101,0xfb410108,0x9cda0101,
	0xfb410108,0x9cdb0101,0xfb410108,0x9ce00101,0xfb410108,0x9ce10101,0xfb410108,0x9ce30101,
	0xfb410108,0x9ce40101,0xfb410108,0x9ce50101,0xfb410108,0x9ce60101,0xfb410108,0x9ce70101,
	0xfb410108,0x9ce80101,0xfb410108,0x9ce90101,0xfb410108,0x9cea0101,0xfb410108,0x9ceb0101,
	0xfb410108,0x9cec0101,0xfb410108,0x9ced0101,0xfb410108,0x9cee0101,0xfb410108,0x9cef0101,
	0xfb410108,0x9cf00101,0xfb410108,0x9cf10101,0xfb410108,0x9cf20101,0xfb410108,0x9cf30101,
	0xfb410108,0x9cf40101,0xfb410108,0x9cf50101,0xfb410108,0x9cf60101,0xfb410108,0x9cf70101,
	0xfb410108,0x9cf80101,0xfb410108,0x9cf90101,0xfb410108,0x9cfa0101,0xfb410108,0x9cfb0101,
	0xfb410108,0x9cfc0101,0xfb410108,0x9cfd0101,0xfb410108,0x9cfe0101,0xfb410108,0x9cff0101,
	0xfb410108,0x9d000101,0xfb410108,0x9d010101,0xfb410108,0x9d020101,0xfb410108,0x9d030101,
	0xfb410108,0x9d040101,0xfb410108,0x9d050101,0xfb410108,0x9d060101,0xfb410108,0x9d070101,
	0xfb410108,0x9d080101,0xfb410108,0x9d090101,0xfb410108,0x9d0a0101,0xfb410108,0x9d0b0101,
	0xfb410108,0x9d0c0101,0xfb410108,0x9d0d0101,0xfb410108,0x9d0e0101,0xfb410108,0x9d0f0101,
	0xfb410108,0x9d100101,0xfb410108,0x9d110101,0xfb410108,0x9d120101,0xfb410108,0x9d130101,
	0xfb410108,0x9d140101,0xfb410108,0x9d150101,0xfb410108,0x9d160101,0xfb410108,0x9d170101,
	0xfb410108,0x9d180101,0xfb410108,0x9d190101,0xfb410108,0x9d1a0101,0xfb410108,0x9d1b0101,
	0xfb410108,0x9d1c0101,0xfb410108,0x9d1d0101,0xfb410108,0x9d1e0101,0xfb410108,0x9d1f0101,
	0xfb410108,0x9d200101,0xfb410108,0x9d210101,0xfb410108,0x9d220101,0xfb410108,0x9d230101,
	0xfb410108,0x9d240101,0xfb410108,0x9d250101,0xfb410108,0x9d260101,0xfb410108,0x9d270101,
	0xfb410108,0x9d280101,0xfb410108,0x9d290101,0xfb410108,0x9d2a0101,0xfb410108,0x9d2b0101,
	0xfb410108,0x9d2c0101,0xfb410108,0x9d2d0101,0xfb410108,0x9d2e0101,0xfb410108,0x9d2f0101,
	0xfb410108,0x9d300101,0xfb410108,0x9d310101,0xfb410108,0x9d320101,0xfb410108,0x9d330101,
	0xfb410108,0x9d340101,0xfb410108,0x9d350101,0xfb410108,0x9d360101,0xfb410108,0x9d370101,
	0xfb410108,0x9d380101,0xfb410108,0x9d390101,0xfb410108,0x9d3a0101,0xfb410108,0x9d3b0101,
	0xfb410108,0x9d3c0101,0xfb410108,0x9d3d0101,0xfb410108,0x9d3e0101,0xfb410108,0x9d3f0101,
	0xfb410108,0x9d400101,0xfb410108,0x9d410101,0xfb410108,0x9d420101,0xfb410108,0x9d430101,
	0xfb410108,0x9d440101,0xfb410108,0x9d450101,0xfb410108,0x9d460101,0xfb410108,0x9d470101,
	0xfb410108,0x9d480101,0xfb410108,0x9d490101,0xfb410108,0x9d4a0101,0xfb410108,0x9d4b0101,
	0xfb410108,0x9d4c0101,0xfb410108,0x9d4d0101,0xfb410108,0x9d4e0101,0xfb410108,0x9d4f0101,
	0xfb410108,0x9d500101,0xfb410108,0x9d510101,0xfb410108,0x9d520101,0xfb410108,0x9d530101,
	0xfb410108,0x9d540101,0xfb410108,0x9d550101,0xfb410108,0x9d560101,0xfb410108,0x9d570101,
	0xfb410108,0x9d580101,0xfb410108,0x9d590101,0xfb410108,0x9d5a0101,0xfb410108,0x9d5b0101,
	0xfb410108,0x9d5c0101,0xfb410108,0x9d5d0101,0xfb410108,0x9d5e0101,0xfb410108,0x9d5f0101,
	0xfb410108,0x9d600101,0xfb410108,0x9d610101,0xfb410108,0x9d620101,0xfb410108,0x9d630101,
	0xfb410108,0x9d640101,0xfb410108,0x9d650101,0xfb410108,0x9d660101,0xfb410108,0x9d670101,
	0xfb410108,0x9d680101,0xfb410108,0x9d690101,0xfb410108,0x9d6a0101,0xfb410108,0x9d6b0101,
	0xfb410108,0x9d6c0101,0xfb410108,0x9d6d0101,0xfb410108,0x9d6e0101,0xfb410108,0x9d6f0101,
	0xfb410108,0x9d700101,0xfb410108,0x9d710101,0xfb410108,0x9d720101,0xfb410108,0x9d730101,
	0xfb410108,0x9d740101,0xfb410108,0x9d750101,0xfb410108,0x9d760101,0xfb410108,0x9d770101,
	0xfb410108,0x9d780101,0xfb410108,0x9d790101,0xfb410108,0x9d7a0101,0xfb410108,0x9d7b0101,
	0xfb410108,0x9d7c0101,0xfb410108,0x9d7d0101,0xfb410108,0x9d7e0101,0xfb410108,0x9d7f0101,
	0xfb410108,0x9d800101,0xfb410108,0x9d810101,0xfb410108,0x9d820101,0xfb410108,0x9d830101,
	0xfb410108,0x9d840101,0xfb410108,0x9d850101,0xfb410108,0x9d860101,0xfb410108,0x9d870101,
	0xfb410108,0x9d880101,0xfb410108,0x9d890101,0xfb410108,0x9d8a0101,0xfb410108,0x9d8b0101,
	0xfb410108,0x9d8c0101,0xfb410108,0x9d8d0101,0xfb410108,0x9d8e0101,0xfb410108,0x9d8f0101,
	0xfb410108,0x9d900101,0xfb410108,0x9d910101,0xfb410108,0x9d920101,0xfb410108,0x9d930101,
	0xfb410108,0x9d940101,0xfb410108,0x9d950101,0xfb410108,0x9d960101,0xfb410108,0x9d970101,
	0xfb410108,0x9d980101,0xfb410108,0x9d990101,0xfb410108,0x9d9a0101,0xfb410108,0x9d9b0101,
	0xfb410108,0x9d9c0101,0xfb410108,0x9d9d0101,0xfb410108,0x9d9e0101,0xfb410108,0x9d9f0101,
	0xfb410108,0x9da00101,0xfb410108,0x9da10101,0xfb410108,0x9da20101,0xfb410108,0x9da30101,
	0xfb410108,0x9da40101,0xfb410108,0x9da50101,0xfb410108,0x9da60101,0xfb410108,0x9da70101,
	0xfb410108,0x9da80101,0xfb410108,0x9da90101,0xfb410108,0x9daa0101,0xfb410108,0x9dab0101,
	0xfb410108,0x9dac0101,0xfb410108,0x9dad0101,0xfb410108,0x9dae0101,0xfb410108,0x9daf0101,
	0xfb410108,0x9db00101,0xfb410108,0x9db10101,0xfb410108,0x9db20101,0xfb410108,0x9db30101,
	0xfb410108,0x9db40101,0xfb410108,0x9db50101,0xfb410108,0x9db60101,0xfb410108,0x9db70101,
	0xfb410108,0x9db80101,0xfb410108,0x9db90101,0xfb410108,0x9dba0101,0xfb410108,0x9dbb0101,
	0xfb410108,0x9dbc0101,0xfb410108,0x9dbd0101,0xfb410108,0x9dbe0101,0xfb410108,0x9dbf0101,
	0xfb410108,0x9dc00101,0xfb410108,0x9dc10101,0xfb410108,0x9dc20101,0xfb410108,0x9dc30101,
	0xfb410108,0x9dc40101,0xfb410108,0x9dc50101,0xfb410108,0x9dc60101,0xfb410108,0x9dc70101,
	0xfb410108,0x9dc80101,0xfb410108,0x9dc90101,0xfb410108,0x9dca0101,0xfb410108,0x9dcb0101,
	0xfb410108,0x9dcc0101,0xfb410108,0x9dcd0101,0xfb410108,0x9dce0101,0xfb410108,0x9dcf0101,
	0xfb410108,0x9dd00101,0xfb410108,0x9dd10101,0xfb410108,0x9dd20101,0xfb410108,0x9dd30101,
	0xfb410108,0x9dd40101,0xfb410108,0x9dd50101,0xfb410108,0x9dd60101,0xfb410108,0x9dd70101,
	0xfb410108,0x9dd80101,0xfb410108,0x9dd90101,0xfb410108,0x9dda0101,0xfb410108,0x9ddb0101,
	0xfb410108,0x9ddc0101,0xfb410108,0x9ddd0101,0xfb410108,0x9dde0101,0xfb410108,0x9ddf0101,
	0xfb410108,0x9de00101,0xfb410108,0x9de10101,0xfb410108,0x9de20101,0xfb410108,0x9de30101,
	0xfb410108,0x9de40101,0xfb410108,0x9de50101,0xfb410108,0x9de60101,0xfb410108,0x9de70101,
	0xfb410108,0x9de80101,0xfb410108,0x9de90101,0xfb410108,0x9dea0101,0xfb410108,0x9deb0101,
	0xfb410108,0x9dec0101,0xfb410108,0x9ded0101,0xfb410108,0x9dee0101,0xfb410108,0x9def0101,
	0xfb410108,0x9df00101,0xfb410108,0x9df10101,0xfb410108,0x9df20101,0xfb410108,0x9df30101,
	0xfb410108,0x9df40101,0xfb410108,0x9df50101,0xfb410108,0x9df60101,0xfb410108,0x9df70101,
	0xfb410108,0x9df80101,0xfb410108,0x9df90101,0xfb410108,0x9dfa0101,0xfb410108,0x9dfb0101,
	0xfb410108,0x9dfc0101,0xfb410108,0x9dfd0101,0xfb410108,0x9dfe0101,0xfb410108,0x9dff0101,
	0xfb410108,0x9e000101,0xfb410108,0x9e010101,0xfb410108,0x9e020101,0xfb410108,0x9e030101,
	0xfb410108,0x9e040101,0xfb410108,0x9e050101,0xfb410108,0x9e060101,0xfb410108,0x9e070101,
	0xfb410108,0x9e080101,0xfb410108,0x9e090101,0xfb410108,0x9e0a0101,0xfb410108,0x9e0b0101,
	0xfb410108,0x9e0c0101,0xfb410108,0x9e0d0101,0xfb410108,0x9e0e0101,0xfb410108,0x9e0f0101,
	0xfb410108,0x9e100101,0xfb410108,0x9e110101,0xfb410108,0x9e120101,0xfb410108,0x9e130101,
	0xfb410108,0x9e140101,0xfb410108,0x9e150101,0xfb410108,0x9e160101,0xfb410108,0x9e170101,
	0xfb410108,0x9e180101,0xfb410108,0x9e190101,0xfb410108,0x9e1a0101,0xfb410108,0x9e1b0101,
	0xfb410108,0x9e1c0101,0xfb410108,0x9e1d0101,0xfb410108,0x9e1e0101,0xfb410108,0x9e240101,
	0xfb410108,0x9e270101,0xfb410108,0x9e2e0101,0xfb410108,0x9e300101,0xfb410108,0x9e340101,
	0xfb410108,0x9e3b0101,0xfb410108,0x9e3c0101,0xfb410108,0x9e400101,0xfb410108,0x9e4d0101,
	0xfb410108,0x9e500101,0xfb410108,0x9e520101,0xfb410108,0x9e530101,0xfb410108,0x9e540101,
	0xfb410108,0x9e560101,0xfb410108,0x9e590101,0xfb410108,0x9e5d0101,0xfb410108,0x9e5f0101,
	0xfb410108,0x9e600101,0xfb410108,0x9e610101,0xfb410108,0x9e620101,0xfb410108,0x9e650101,
	0xfb410108,0x9e6e0101,0xfb410108,0x9e6f0101,0xfb410108,0x9e720101,0xfb410108,0x9e740101,
	0xfb410108,0x9e750101,0xfb410108,0x9e760101,0xfb410108,0x9e770101,0xfb410108,0x9e780101,
	0xfb410108,0x9e790101,0xfb410108,0x9e7a0101,0xfb410108,0x9e7b0101,0xfb410108,0x9e7c0101,
	0xfb410108,0x9e7d0101,0xfb410108,0x9e800101,0xfb410108,0x9e810101,0xfb410108,0x9e830101,
	0xfb410108,0x9e840101,0xfb410108,0x9e850101,0xfb410108,0x9e860101,0xfb410108,0x9e890101,
	0xfb410108,0x9e8a0101,0xfb410108,0x9e8c0101,0xfb410108,0x9e8d0101,0xfb410108,0x9e8e0101,
	0xfb410108,0x9e8f0101,0xfb410108,0x9e900101,0xfb410108,0x9e910101,0xfb410108,0x9e940101,
	0xfb410108,0x9e950101,0xfb410108,0x9e960101,0xfb410108,0x9e970101,0xfb410108,0x9e980101,
	0xfb410108,0x9e990101,0xfb410108,0x9e9a0101,0xfb410108,0x9e9b0101,0xfb410108,0x9e9c0101,
	0xfb410108,0x9e9e0101,0xfb410108,0x9ea00101,0xfb410108,0x9ea10101,0xfb410108,0x9ea20101,
	0xfb410108,0x9ea30101,0xfb410108,0x9ea40101,0xfb410108,0x9ea50101,0xfb410108,0x9ea70101,
	0xfb410108,0x9ea80101,0xfb410108,0x9ea90101,0xfb410108,0x9eaa0101,0xfb410108,0x9eab0101,
	0xfb410108,0x9eac0101,0xfb410108,0x9ead0101,0xfb410108,0x9eae0101,0xfb410108,0x9eaf0101,
	0xfb410108,0x9eb00101,0xfb410108,0x9eb10101,0xfb410108,0x9eb20101,0xfb410108,0x9eb30101,
	0xfb410108,0x9eb50101,0xfb410108,0x9eb60101,0xfb410108,0x9eb70101,0xfb410108,0x9eb90101,
	0xfb410108,0x9eba0101,0xfb410108,0x9ebc0101,0xfb410108,0x9ebf0101,0xfb410108,0x9ec00101,
	0xfb410108,0x9ec10101,0xfb410108,0x9ec20101,0xfb410108,0x9ec30101,0xfb410108,0x9ec50101,
	0xfb410108,0x9ec60101,0xfb410108,0x9ec70101,0xfb410108,0x9ec80101,0xfb410108,0x9eca0101,
	0xfb410108,0x9ecb0101,0xfb410108,0x9ecc0101,0xfb410108,0x9ed00101,0xfb410108,0x9ed20101,
	0xfb410108,0x9ed30101,0xfb410108,0x9ed50101,0xfb410108,0x9ed60101,0xfb410108,0x9ed70101,
	0xfb410108,0x9ed90101,0xfb410108,0x9eda0101,0xfb410108,0x9ede0101,0xfb410108,0x9ee10101,
	0xfb410108,0x9ee30101,0xfb410108,0x9ee40101,0xfb410108,0x9ee60101,0xfb410108,0x9ee80101,
	0xfb410108,0x9eeb0101,0xfb410108,0x9eec0101,0xfb410108,0x9eed0101,0xfb410108,0x9eee0101,
	0xfb410108,0x9ef00101,0xfb410108,0x9ef10101,0xfb410108,0x9ef20101,0xfb410108,0x9ef30101,
	0xfb410108,0x9ef40101,0xfb410108,0x9ef50101,0xfb410108,0x9ef60101,0xfb410108,0x9ef70101,
	0xfb410108,0x9ef80101,0xfb410108,0x9efa0101,0xfb410108,0x9efd0101,0xfb410108,0x9eff0101,
	0xfb410108,0x9f000101,0xfb410108,0x9f010101,0xfb410108,0x9f020101,0xfb410108,0x9f030101,
	0xfb410108,0x9f040101,0xfb410108,0x9f050101,0xfb410108,0x9f060101,0xfb410108,0x9f070101,
	0xfb410108,0x9f080101,0xfb410108,0x9f090101,0xfb410108,0x9f0a0101,0xfb410108,0x9f0c0101,
	0xfb410108,0x9f0f0101,0xfb410108,0x9f110101,0xfb410108,0x9f120101,0xfb410108,0x9f140101,
	0xfb410108,0x9f150101,0xfb410108,0x9f160101,0xfb410108,0x9f180101,0xfb410108,0x9f1a0101,
	0xfb410108,0x9f1b0101,0xfb410108,0x9f1c0101,0xfb410108,0x9f1d0101,0xfb410108,0x9f1e0101,
	0xfb410108,0x9f1f0101,0xfb410108,0x9f210101,0xfb410108,0x9f230101,0xfb410108,0x9f240101,
	0xfb410108,0x9f250101,0xfb410108,0x9f260101,0xfb410108,0x9f270101,0xfb410108,0x9f280101,
	0xfb410108,0x9f290101,0xfb410108,0x9f2a0101,0xfb410108,0x9f2b0101,0xfb410108,0x9f2d0101,
	0xfb410108,0x9f2e0101,0xfb410108,0x9f300101,0xfb410108,0x9f310101,0xfb410108,0x9f320101,
	0xfb410108,0x9f330101,0xfb410108,0x9f340101,0xfb410108,0x9f350101,0xfb410108,0x9f360101,
	0xfb410108,0x9f380101,0xfb410108,0x9f3a0101,0xfb410108,0x9f3c0101,0xfb410108,0x9f3f0101,
	0xfb410108,0x9f400101,0xfb410108,0x9f410101,0xfb410108,0x9f420101,0xfb410108,0x9f430101,
	0xfb410108,0x9f450101,0xfb410108,0x9f460101,0xfb410108,0x9f470101,0xfb410108,0x9f480101,
	0xfb410108,0x9f490101,0xfb410108,0x9f4a0101,0xfb410108,0x9f4b0101,0xfb410108,0x9f4c0101,
	0xfb410108,0x9f4d0101,0xfb410108,0x9f4e0101,0xfb410108,0x9f4f0101,0xfb410108,0x9f520101,
	0xfb410108,0x9f530101,0xfb410108,0x9f540101,0xfb410108,0x9f550101,0xfb410108,0x9f560101,
	0xfb410108,0x9f570101,0xfb410108,0x9f580101,0xfb410108,0x9f590101,0xfb410108,0x9f5a0101,
	0xfb410108,0x9f5b0101,0xfb410108,0x9f5c0101,0xfb410108,0x9f5d0101,0xfb410108,0x9f5e0101,
	0xfb410108,0x9f5f0101,0xfb410108,0x9f600101,0xfb410108,0x9f610101,0xfb410108,0x9f620101,
	0xfb410108,0x9f630101,0xfb410108,0x9f640101,0xfb410108,0x9f650101,0xfb410108,0x9f660101,
	0xfb410108,0x9f670101,0xfb410108,0x9f680101,0xfb410108,0x9f690101,0xfb410108,0x9f6a0101,
	0xfb410108,0x9f6b0101,0xfb410108,0x9f6c0101,0xfb410108,0x9f6d0101,0xfb410108,0x9f6e0101,
	0xfb410108,0x9f6f0101,0xfb410108,0x9f700101,0xfb410108,0x9f710101,0xfb410108,0x9f720101,
	0xfb410108,0x9f730101,0xfb410108,0x9f740101,0xfb410108,0x9f750101,0xfb410108,0x9f760101,
	0xfb410108,0x9f770101,0xfb410108,0x9f780101,0xfb410108,0x9f790101,0xfb410108,0x9f7a0101,
	0xfb410108,0x9f7b0101,0xfb410108,0x9f7c0101,0xfb410108,0x9f7d0101,0xfb410108,0x9f7e0101,
	0xfb410108,0x9f810101,0xfb410108,0x9f820101,0xfb410108,0x9f8d0101,0xfb410108,0x9f8e0101,
	0xfb410108,0x9f8f0101,0xfb410108,0x9f900101,0xfb410108,0x9f910101,0xfb410108,0x9f920101,
	0xfb410108,0x9f930101,0xfb410108,0x9f940101,0xfb410108,0x9f950101,0xfb410108,0x9f960101,
	0xfb410108,0x9f970101,0xfb410108,0x9f980101,0xfb410108,0x9f9c0101,0xfb410108,0x9f9d0101,
	0xfb410108,0x9f9e0101,0xfb410108,0x9fa10101,0xfb410108,0x9fa20101,0xfb410108,0x9fa30101,
	0xfb410108,0x9fa40101,0xfb410108,0x9fa50101,0xfb410108,0x9fa60101,0xfb410108,0x9fa70101,
	0xfb410108,0x9fa80101,0xfb410108,0x9fa90101,0xfb410108,0x9faa0101,0xfb410108,0x9fab0101,
	0xfb410108,0x9fac0101,0xfb410108,0x9fad0101,0xfb410108,0x9fae0101,0xfb410108,0x9faf0101,
	0xfb410108,0x9fb00101,0xfb410108,0x9fb10101,0xfb410108,0x9fb20101,0xfb410108,0x9fb30101,
	0xfb410108,0x9fb40101,0xfb410108,0x9fb50101,0xfb410108,0x9fb60101,0xfb410108,0x9fb70101,
	0xfb410108,0x9fb80101,0xfb410108,0x9fb90101,0xfb410108,0x9fba0101,0xfb410108,0x9fbb0101,
	0xfb800108,0xb4000101,0xfb800108,0xb4010101,0xfb800108,0xb4020101,0xfb800108,0xb4030101,
	0xfb800108,0xb4040101,0xfb800108,0xb4050101,0xfb800108,0xb4060101,0xfb800108,0xb4070101,
	0xfb800108,0xb4080101,0xfb800108,0xb4090101,0xfb800108,0xb40a0101,0xfb800108,0xb40b0101,
	0xfb800108,0xb40c0101,0xfb800108,0xb40d0101,0xfb800108,0xb40e0101,0xfb800108,0xb40f0101,
	0xfb800108,0xb4100101,0xfb800108,0xb4110101,0xfb800108,0xb4120101,0xfb800108,0xb4130101,
	0xfb800108,0xb4140101,0xfb800108,0xb4150101,0xfb800108,0xb4160101,0xfb800108,0xb4170101,
	0xfb800108,0xb4180101,0xfb800108,0xb4190101,0xfb800108,0xb41a0101,0xfb800108,0xb41b0101,
	0xfb800108,0xb41c0101,0xfb800108,0xb41d0101,0xfb800108,0xb41e0101,0xfb800108,0xb41f0101,
	0xfb800108,0xb4200101,0xfb800108,0xb4210101,0xfb800108,0xb4220101,0xfb800108,0xb4230101,
	0xfb800108,0xb4240101,0xfb800108,0xb4250101,0xfb800108,0xb4260101,0xfb800108,0xb4270101,
	0xfb800108,0xb4280101,0xfb800108,0xb4290101,0xfb800108,0xb42a0101,0xfb800108,0xb42b0101,
	0xfb800108,0xb42c0101,0xfb800108,0xb42d0101,0xfb800108,0xb42e0101,0xfb800108,0xb42f0101,
	0xfb800108,0xb4300101,0xfb800108,0xb4310101,0xfb800108,0xb4320101,0xfb800108,0xb4330101,
	0xfb800108,0xb4340101,0xfb800108,0xb4350101,0xfb800108,0xb4360101,0xfb800108,0xb4370101,
	0xfb800108,0xb4380101,0xfb800108,0xb4390101,0xfb800108,0xb43a0101,0xfb800108,0xb43b0101,
	0xfb800108,0xb43c0101,0xfb800108,0xb43d0101,0xfb800108,0xb43e0101,0xfb800108,0xb43f0101,
	0xfb800108,0xb4400101,0xfb800108,0xb4410101,0xfb800108,0xb4420101,0xfb800108,0xb4430101,
	0xfb800108,0xb4440101,0xfb800108,0xb4450101,0xfb800108,0xb4460101,0xfb800108,0xb4470101,
	0xfb800108,0xb4480101,0xfb800108,0xb4490101,0xfb800108,0xb44a0101,0xfb800108,0xb44b0101,
	0xfb800108,0xb44c0101,0xfb800108,0xb44d0101,0xfb800108,0xb44e0101,0xfb800108,0xb44f0101,
	0xfb800108,0xb4500101,0xfb800108,0xb4510101,0xfb800108,0xb4520101,0xfb800108,0xb4530101,
	0xfb800108,0xb4540101,0xfb800108,0xb4550101,0xfb800108,0xb4560101,0xfb800108,0xb4570101,
	0xfb800108,0xb4580101,0xfb800108,0xb4590101,0xfb800108,0xb45a0101,0xfb800108,0xb45b0101,
	0xfb800108,0xb45c0101,0xfb800108,0xb45d0101,0xfb800108,0xb45e0101,0xfb800108,0xb45f0101,
	0xfb800108,0xb4600101,0xfb800108,0xb4610101,0xfb800108,0xb4620101,0xfb800108,0xb4630101,
	0xfb800108,0xb4640101,0xfb800108,0xb4650101,0xfb800108,0xb4660101,0xfb800108,0xb4670101,
	0xfb800108,0xb4680101,0xfb800108,0xb4690101,0xfb800108,0xb46a0101,0xfb800108,0xb46b0101,
	0xfb800108,0xb46c0101,0xfb800108,0xb46d0101,0xfb800108,0xb46e0101,0xfb800108,0xb46f0101,
	0xfb800108,0xb4700101,0xfb800108,0xb4710101,0xfb800108,0xb4720101,0xfb800108,0xb4730101,
	0xfb800108,0xb4740101,0xfb800108,0xb4750101,0xfb800108,0xb4760101,0xfb800108,0xb4770101,
	0xfb800108,0xb4780101,0xfb800108,0xb4790101,0xfb800108,0xb47a0101,0xfb800108,0xb47b0101,
	0xfb800108,0xb47c0101,0xfb800108,0xb47d0101,0xfb800108,0xb47e0101,0xfb800108,0xb47f0101,
	0xfb800108,0xb4800101,0xfb800108,0xb4810101,0xfb800108,0xb4820101,0xfb800108,0xb4830101,
	0xfb800108,0xb4840101,0xfb800108,0xb4850101,0xfb800108,0xb4860101,0xfb800108,0xb4870101,
	0xfb800108,0xb4880101,0xfb800108,0xb4890101,0xfb800108,0xb48a0101,0xfb800108,0xb48b0101,
	0xfb800108,0xb48c0101,0xfb800108,0xb48d0101,0xfb800108,0xb48e0101,0xfb800108,0xb48f0101,
	0xfb800108,0xb4900101,0xfb800108,0xb4910101,0xfb800108,0xb4920101,0xfb800108,0xb4930101,
	0xfb800108,0xb4940101,0xfb800108,0xb4950101,0xfb800108,0xb4960101,0xfb800108,0xb4970101,
	0xfb800108,0xb4980101,0xfb800108,0xb4990101,0xfb800108,0xb49a0101,0xfb800108,0xb49b0101,
	0xfb800108,0xb49c0101,0xfb800108,0xb49d0101,0xfb800108,0xb49e0101,0xfb800108,0xb49f0101,
	0xfb800108,0xb4a00101,0xfb800108,0xb4a10101,0xfb800108,0xb4a20101,0xfb800108,0xb4a30101,
	0xfb800108,0xb4a40101,0xfb800108,0xb4a50101,0xfb800108,0xb4a60101,0xfb800108,0xb4a70101,
	0xfb800108,0xb4a80101,0xfb800108,0xb4a90101,0xfb800108,0xb4aa0101,0xfb800108,0xb4ab0101,
	0xfb800108,0xb4ac0101,0xfb800108,0xb4ad0101,0xfb800108,0xb4ae0101,0xfb800108,0xb4af0101,
	0xfb800108,0xb4b00101,0xfb800108,0xb4b10101,0xfb800108,0xb4b20101,0xfb800108,0xb4b30101,
	0xfb800108,0xb4b40101,0xfb800108,0xb4b50101,0xfb800108,0xb4b60101,0xfb800108,0xb4b70101,
	0xfb800108,0xb4b80101,0xfb800108,0xb4b90101,0xfb800108,0xb4ba0101,0xfb800108,0xb4bb0101,
	0xfb800108,0xb4bc0101,0xfb800108,0xb4bd0101,0xfb800108,0xb4be0101,0xfb800108,0xb4bf0101,
	0xfb800108,0xb4c00101,0xfb800108,0xb4c10101,0xfb800108,0xb4c20101,0xfb800108,0xb4c30101,
	0xfb800108,0xb4c40101,0xfb800108,0xb4c50101,0xfb800108,0xb4c60101,0xfb800108,0xb4c70101,
	0xfb800108,0xb4c80101,0xfb800108,0xb4c90101,0xfb800108,0xb4ca0101,0xfb800108,0xb4cb0101,
	0xfb800108,0xb4cc0101,0xfb800108,0xb4cd0101,0xfb800108,0xb4ce0101,0xfb800108,0xb4cf0101,
	0xfb800108,0xb4d00101,0xfb800108,0xb4d10101,0xfb800108,0xb4d20101,0xfb800108,0xb4d30101,
	0xfb800108,0xb4d40101,0xfb800108,0xb4d50101,0xfb800108,0xb4d60101,0xfb800108,0xb4d70101,
	0xfb800108,0xb4d80101,0xfb800108,0xb4d90101,0xfb800108,0xb4da0101,0xfb800108,0xb4db0101,
	0xfb800108,0xb4dc0101,0xfb800108,0xb4dd0101,0xfb800108,0xb4de0101,0xfb800108,0xb4df0101,
	0xfb800108,0xb4e00101,0xfb800108,0xb4e10101,0xfb800108,0xb4e20101,0xfb800108,0xb4e30101,
	0xfb800108,0xb4e40101,0xfb800108,0xb4e50101,0xfb800108,0xb4e60101,0xfb800108,0xb4e70101,
	0xfb800108,0xb4e80101,0xfb800108,0xb4e90101,0xfb800108,0xb4ea0101,0xfb800108,0xb4eb0101,
	0xfb800108,0xb4ec0101,0xfb800108,0xb4ed0101,0xfb800108,0xb4ee0101,0xfb800108,0xb4ef0101,
	0xfb800108,0xb4f00101,0xfb800108,0xb4f10101,0xfb800108,0xb4f20101,0xfb800108,0xb4f30101,
	0xfb800108,0xb4f40101,0xfb800108,0xb4f50101,0xfb800108,0xb4f60101,0xfb800108,0xb4f70101,
	0xfb800108,0xb4f80101,0xfb800108,0xb4f90101,0xfb800108,0xb4fa0101,0xfb800108,0xb4fb0101,
	0xfb800108,0xb4fc0101,0xfb800108,0xb4fd0101,0xfb800108,0xb4fe0101,0xfb800108,0xb4ff0101,
	0xfb800108,0xb5000101,0xfb800108,0xb5010101,0xfb800108,0xb5020101,0xfb800108,0xb5030101,
	0xfb800108,0xb5040101,0xfb800108,0xb5050101,0xfb800108,0xb5060101,0xfb800108,0xb5070101,
	0xfb800108,0xb5080101,0xfb800108,0xb5090101,0xfb800108,0xb50a0101,0xfb800108,0xb50b0101,
	0xfb800108,0xb50c0101,0xfb800108,0xb50d0101,0xfb800108,0xb50e0101,0xfb800108,0xb50f0101,
	0xfb800108,0xb5100101,0xfb800108,0xb5110101,0xfb800108,0xb5120101,0xfb800108,0xb5130101,
	0xfb800108,0xb5140101,0xfb800108,0xb5150101,0xfb800108,0xb5160101,0xfb800108,0xb5170101,
	0xfb800108,0xb5180101,0xfb800108,0xb5190101,0xfb800108,0xb51a0101,0xfb800108,0xb51b0101,
	0xfb800108,0xb51c0101,0xfb800108,0xb51d0101,0xfb800108,0xb51e0101,0xfb800108,0xb51f0101,
	0xfb800108,0xb5200101,0xfb800108,0xb5210101,0xfb800108,0xb5220101,0xfb800108,0xb5230101,
	0xfb800108,0xb5240101,0xfb800108,0xb5250101,0xfb800108,0xb5260101,0xfb800108,0xb5270101,
	0xfb800108,0xb5280101,0xfb800108,0xb5290101,0xfb800108,0xb52a0101,0xfb800108,0xb52b0101,
	0xfb800108,0xb52c0101,0xfb800108,0xb52d0101,0xfb800108,0xb52e0101,0xfb800108,0xb52f0101,
	0xfb800108,0xb5300101,0xfb800108,0xb5310101,0xfb800108,0xb5320101,0xfb800108,0xb5330101,
	0xfb800108,0xb5340101,0xfb800108,0xb5350101,0xfb800108,0xb5360101,0xfb800108,0xb5370101,
	0xfb800108,0xb5380101,0xfb800108,0xb5390101,0xfb800108,0xb53a0101,0xfb800108,0xb53b0101,
	0xfb800108,0xb53c0101,0xfb800108,0xb53d0101,0xfb800108,0xb53e0101,0xfb800108,0xb53f0101,
	0xfb800108,0xb5400101,0xfb800108,0xb5410101,0xfb800108,0xb5420101,0xfb800108,0xb5430101,
	0xfb800108,0xb5440101,0xfb800108,0xb5450101,0xfb800108,0xb5460101,0xfb800108,0xb5470101,
	0xfb800108,0xb5480101,0xfb800108,0xb5490101,0xfb800108,0xb54a0101,0xfb800108,0xb54b0101,
	0xfb800108,0xb54c0101,0xfb800108,0xb54d0101,0xfb800108,0xb54e0101,0xfb800108,0xb54f0101,
	0xfb800108,0xb5500101,0xfb800108,0xb5510101,0xfb800108,0xb5520101,0xfb800108,0xb5530101,
	0xfb800108,0xb5540101,0xfb800108,0xb5550101,0xfb800108,0xb5560101,0xfb800108,0xb5570101,
	0xfb800108,0xb5580101,0xfb800108,0xb5590101,0xfb800108,0xb55a0101,0xfb800108,0xb55b0101,
	0xfb800108,0xb55c0101,0xfb800108,0xb55d0101,0xfb800108,0xb55e0101,0xfb800108,0xb55f0101,
	0xfb800108,0xb5600101,0xfb800108,0xb5610101,0xfb800108,0xb5620101,0xfb800108,0xb5630101,
	0xfb800108,0xb5640101,0xfb800108,0xb5650101,0xfb800108,0xb5660101,0xfb800108,0xb5670101,
	0xfb800108,0xb5680101,0xfb800108,0xb5690101,0xfb800108,0xb56a0101,0xfb800108,0xb56b0101,
	0xfb800108,0xb56c0101,0xfb800108,0xb56d0101,0xfb800108,0xb56e0101,0xfb800108,0xb56f0101,
	0xfb800108,0xb5700101,0xfb800108,0xb5710101,0xfb800108,0xb5720101,0xfb800108,0xb5730101,
	0xfb800108,0xb5740101,0xfb800108,0xb5750101,0xfb800108,0xb5760101,0xfb800108,0xb5770101,
	0xfb800108,0xb5780101,0xfb800108,0xb5790101,0xfb800108,0xb57a0101,0xfb800108,0xb57b0101,
	0xfb800108,0xb57c0101,0xfb800108,0xb57d0101,0xfb800108,0xb57e0101,0xfb800108,0xb57f0101,
	0xfb800108,0xb5800101,0xfb800108,0xb5810101,0xfb800108,0xb5820101,0xfb800108,0xb5830101,
	0xfb800108,0xb5840101,0xfb800108,0xb5850101,0xfb800108,0xb5860101,0xfb800108,0xb5870101,
	0xfb800108,0xb5880101,0xfb800108,0xb5890101,0xfb800108,0xb58a0101,0xfb800108,0xb58b0101,
	0xfb800108,0xb58c0101,0xfb800108,0xb58d0101,0xfb800108,0xb58e0101,0xfb800108,0xb58f0101,
	0xfb800108,0xb5900101,0xfb800108,0xb5910101,0xfb800108,0xb5920101,0xfb800108,0xb5930101,
	0xfb800108,0xb5940101,0xfb800108,0xb5950101,0xfb800108,0xb5960101,0xfb800108,0xb5970101,
	0xfb800108,0xb5980101,0xfb800108,0xb5990101,0xfb800108,0xb59a0101,0xfb800108,0xb59b0101,
	0xfb800108,0xb59c0101,0xfb800108,0xb59d0101,0xfb800108,0xb59e0101,0xfb800108,0xb59f0101,
	0xfb800108,0xb5a00101,0xfb800108,0xb5a10101,0xfb800108,0xb5a20101,0xfb800108,0xb5a30101,
	0xfb800108,0xb5a40101,0xfb800108,0xb5a50101,0xfb800108,0xb5a60101,0xfb800108,0xb5a70101,
	0xfb800108,0xb5a80101,0xfb800108,0xb5a90101,0xfb800108,0xb5aa0101,0xfb800108,0xb5ab0101,
	0xfb800108,0xb5ac0101,0xfb800108,0xb5ad0101,0xfb800108,0xb5ae0101,0xfb800108,0xb5af0101,
	0xfb800108,0xb5b00101,0xfb800108,0xb5b10101,0xfb800108,0xb5b20101,0xfb800108,0xb5b30101,
	0xfb800108,0xb5b40101,0xfb800108,0xb5b50101,0xfb800108,0xb5b60101,0xfb800108,0xb5b70101,
	0xfb800108,0xb5b80101,0xfb800108,0xb5b90101,0xfb800108,0xb5ba0101,0xfb800108,0xb5bb0101,
	0xfb800108,0xb5bc0101,0xfb800108,0xb5bd0101,0xfb800108,0xb5be0101,0xfb800108,0xb5bf0101,
	0xfb800108,0xb5c00101,0xfb800108,0xb5c10101,0xfb800108,0xb5c20101,0xfb800108,0xb5c30101,
	0xfb800108,0xb5c40101,0xfb800108,0xb5c50101,0xfb800108,0xb5c60101,0xfb800108,0xb5c70101,
	0xfb800108,0xb5c80101,0xfb800108,0xb5c90101,0xfb800108,0xb5ca0101,0xfb800108,0xb5cb0101,
	0xfb800108,0xb5cc0101,0xfb800108,0xb5cd0101,0xfb800108,0xb5ce0101,0xfb800108,0xb5cf0101,
	0xfb800108,0xb5d00101,0xfb800108,0xb5d10101,0xfb800108,0xb5d20101,0xfb800108,0xb5d30101,
	0xfb800108,0xb5d40101,0xfb800108,0xb5d50101,0xfb800108,0xb5d60101,0xfb800108,0xb5d70101,
	0xfb800108,0xb5d80101,0xfb800108,0xb5d90101,0xfb800108,0xb5da0101,0xfb800108,0xb5db0101,
	0xfb800108,0xb5dc0101,0xfb800108,0xb5dd0101,0xfb800108,0xb5de0101,0xfb800108,0xb5df0101,
	0xfb800108,0xb5e00101,0xfb800108,0xb5e10101,0xfb800108,0xb5e20101,0xfb800108,0xb5e30101,
	0xfb800108,0xb5e40101,0xfb800108,0xb5e50101,0xfb800108,0xb5e60101,0xfb800108,0xb5e70101,
	0xfb800108,0xb5e80101,0xfb800108,0xb5e90101,0xfb800108,0xb5ea0101,0xfb800108,0xb5eb0101,
	0xfb800108,0xb5ec0101,0xfb800108,0xb5ed0101,0xfb800108,0xb5ee0101,0xfb800108,0xb5ef0101,
	0xfb800108,0xb5f00101,0xfb800108,0xb5f10101,0xfb800108,0xb5f20101,0xfb800108,0xb5f30101,
	0xfb800108,0xb5f40101,0xfb800108,0xb5f50101,0xfb800108,0xb5f60101,0xfb800108,0xb5f70101,
	0xfb800108,0xb5f80101,0xfb800108,0xb5f90101,0xfb800108,0xb5fa0101,0xfb800108,0xb5fb0101,
	0xfb800108,0xb5fc0101,0xfb800108,0xb5fd0101,0xfb800108,0xb5fe0101,0xfb800108,0xb5ff0101,
	0xfb800108,0xb6000101,0xfb800108,0xb6010101,0xfb800108,0xb6020101,0xfb800108,0xb6030101,
	0xfb800108,0xb6040101,0xfb800108,0xb6050101,0xfb800108,0xb6060101,0xfb800108,0xb6070101,
	0xfb800108,0xb6080101,0xfb800108,0xb6090101,0xfb800108,0xb60a0101,0xfb800108,0xb60b0101,
	0xfb800108,0xb60c0101,0xfb800108,0xb60d0101,0xfb800108,0xb60e0101,0xfb800108,0xb60f0101,
	0xfb800108,0xb6100101,0xfb800108,0xb6110101,0xfb800108,0xb6120101,0xfb800108,0xb6130101,
	0xfb800108,0xb6140101,0xfb800108,0xb6150101,0xfb800108,0xb6160101,0xfb800108,0xb6170101,
	0xfb800108,0xb6180101,0xfb800108,0xb6190101,0xfb800108,0xb61a0101,0xfb800108,0xb61b0101,
	0xfb800108,0xb61c0101,0xfb800108,0xb61d0101,0xfb800108,0xb61e0101,0xfb800108,0xb61f0101,
	0xfb800108,0xb6200101,0xfb800108,0xb6210101,0xfb800108,0xb6220101,0xfb800108,0xb6230101,
	0xfb800108,0xb6240101,0xfb800108,0xb6250101,0xfb800108,0xb6260101,0xfb800108,0xb6270101,
	0xfb800108,0xb6280101,0xfb800108,0xb6290101,0xfb800108,0xb62a0101,0xfb800108,0xb62b0101,
	0xfb800108,0xb62c0101,0xfb800108,0xb62d0101,0xfb800108,0xb62e0101,0xfb800108,0xb62f0101,
	0xfb800108,0xb6300101,0xfb800108,0xb6310101,0xfb800108,0xb6320101,0xfb800108,0xb6330101,
	0xfb800108,0xb6340101,0xfb800108,0xb6350101,0xfb800108,0xb6360101,0xfb800108,0xb6370101,
	0xfb800108,0xb6380101,0xfb800108,0xb6390101,0xfb800108,0xb63a0101,0xfb800108,0xb63b0101,
	0xfb800108,0xb63c0101,0xfb800108,0xb63d0101,0xfb800108,0xb63e0101,0xfb800108,0xb63f0101,
	0xfb800108,0xb6400101,0xfb800108,0xb6410101,0xfb800108,0xb6420101,0xfb800108,0xb6430101,
	0xfb800108,0xb6440101,0xfb800108,0xb6450101,0xfb800108,0xb6460101,0xfb800108,0xb6470101,
	0xfb800108,0xb6480101,0xfb800108,0xb6490101,0xfb800108,0xb64a0101,0xfb800108,0xb64b0101,
	0xfb800108,0xb64c0101,0xfb800108,0xb64d0101,0xfb800108,0xb64e0101,0xfb800108,0xb64f0101,
	0xfb800108,0xb6500101,0xfb800108,0xb6510101,0xfb800108,0xb6520101,0xfb800108,0xb6530101,
	0xfb800108,0xb6540101,0xfb800108,0xb6550101,0xfb800108,0xb6560101,0xfb800108,0xb6570101,
	0xfb800108,0xb6580101,0xfb800108,0xb6590101,0xfb800108,0xb65a0101,0xfb800108,0xb65b0101,
	0xfb800108,0xb65c0101,0xfb800108,0xb65d0101,0xfb800108,0xb65e0101,0xfb800108,0xb65f0101,
	0xfb800108,0xb6600101,0xfb800108,0xb6610101,0xfb800108,0xb6620101,0xfb800108,0xb6630101,
	0xfb800108,0xb6640101,0xfb800108,0xb6650101,0xfb800108,0xb6660101,0xfb800108,0xb6670101,
	0xfb800108,0xb6680101,0xfb800108,0xb6690101,0xfb800108,0xb66a0101,0xfb800108,0xb66b0101,
	0xfb800108,0xb66c0101,0xfb800108,0xb66d0101,0xfb800108,0xb66e0101,0xfb800108,0xb66f0101,
	0xfb800108,0xb6700101,0xfb800108,0xb6710101,0xfb800108,0xb6720101,0xfb800108,0xb6730101,
	0xfb800108,0xb6740101,0xfb800108,0xb6750101,0xfb800108,0xb6760101,0xfb800108,0xb6770101,
	0xfb800108,0xb6780101,0xfb800108,0xb6790101,0xfb800108,0xb67a0101,0xfb800108,0xb67b0101,
	0xfb800108,0xb67c0101,0xfb800108,0xb67d0101,0xfb800108,0xb67e0101,0xfb800108,0xb67f0101,
	0xfb800108,0xb6800101,0xfb800108,0xb6810101,0xfb800108,0xb6820101,0xfb800108,0xb6830101,
	0xfb800108,0xb6840101,0xfb800108,0xb6850101,0xfb800108,0xb6860101,0xfb800108,0xb6870101,
	0xfb800108,0xb6880101,0xfb800108,0xb6890101,0xfb800108,0xb68a0101,0xfb800108,0xb68b0101,
	0xfb800108,0xb68c0101,0xfb800108,0xb68d0101,0xfb800108,0xb68e0101,0xfb800108,0xb68f0101,
	0xfb800108,0xb6900101,0xfb800108,0xb6910101,0xfb800108,0xb6920101,0xfb800108,0xb6930101,
	0xfb800108,0xb6940101,0xfb800108,0xb6950101,0xfb800108,0xb6960101,0xfb800108,0xb6970101,
	0xfb800108,0xb6980101,0xfb800108,0xb6990101,0xfb800108,0xb69a0101,0xfb800108,0xb69b0101,
	0xfb800108,0xb69c0101,0xfb800108,0xb69d0101,0xfb800108,0xb69e0101,0xfb800108,0xb69f0101,
	0xfb800108,0xb6a00101,0xfb800108,0xb6a10101,0xfb800108,0xb6a20101,0xfb800108,0xb6a30101,
	0xfb800108,0xb6a40101,0xfb800108,0xb6a50101,0xfb800108,0xb6a60101,0xfb800108,0xb6a70101,
	0xfb800108,0xb6a80101,0xfb800108,0xb6a90101,0xfb800108,0xb6aa0101,0xfb800108,0xb6ab0101,
	0xfb800108,0xb6ac0101,0xfb800108,0xb6ad0101,0xfb800108,0xb6ae0101,0xfb800108,0xb6af0101,
	0xfb800108,0xb6b00101,0xfb800108,0xb6b10101,0xfb800108,0xb6b20101,0xfb800108,0xb6b30101,
	0xfb800108,0xb6b40101,0xfb800108,0xb6b50101,0xfb800108,0xb6b60101,0xfb800108,0xb6b70101,
	0xfb800108,0xb6b80101,0xfb800108,0xb6b90101,0xfb800108,0xb6ba0101,0xfb800108,0xb6bb0101,
	0xfb800108,0xb6bc0101,0xfb800108,0xb6bd0101,0xfb800108,0xb6be0101,0xfb800108,0xb6bf0101,
	0xfb800108,0xb6c00101,0xfb800108,0xb6c10101,0xfb800108,0xb6c20101,0xfb800108,0xb6c30101,
	0xfb800108,0xb6c40101,0xfb800108,0xb6c50101,0xfb800108,0xb6c60101,0xfb800108,0xb6c70101,
	0xfb800108,0xb6c80101,0xfb800108,0xb6c90101,0xfb800108,0xb6ca0101,0xfb800108,0xb6cb0101,
	0xfb800108,0xb6cc0101,0xfb800108,0xb6cd0101,0xfb800108,0xb6ce0101,0xfb800108,0xb6cf0101,
	0xfb800108,0xb6d00101,0xfb800108,0xb6d10101,0xfb800108,0xb6d20101,0xfb800108,0xb6d30101,
	0xfb800108,0xb6d40101,0xfb800108,0xb6d50101,0xfb800108,0xb6d60101,0xfb800108,0xb6d70101,
	0xfb800108,0xb6d80101,0xfb800108,0xb6d90101,0xfb800108,0xb6da0101,0xfb800108,0xb6db0101,
	0xfb800108,0xb6dc0101,0xfb800108,0xb6dd0101,0xfb800108,0xb6de0101,0xfb800108,0xb6df0101,
	0xfb800108,0xb6e00101,0xfb800108,0xb6e10101,0xfb800108,0xb6e20101,0xfb800108,0xb6e30101,
	0xfb800108,0xb6e40101,0xfb800108,0xb6e50101,0xfb800108,0xb6e60101,0xfb800108,0xb6e70101,
	0xfb800108,0xb6e80101,0xfb800108,0xb6e90101,0xfb800108,0xb6ea0101,0xfb800108,0xb6eb0101,
	0xfb800108,0xb6ec0101,0xfb800108,0xb6ed0101,0xfb800108,0xb6ee0101,0xfb800108,0xb6ef0101,
	0xfb800108,0xb6f00101,0xfb800108,0xb6f10101,0xfb800108,0xb6f20101,0xfb800108,0xb6f30101,
	0xfb800108,0xb6f40101,0xfb800108,0xb6f50101,0xfb800108,0xb6f60101,0xfb800108,0xb6f70101,
	0xfb800108,0xb6f80101,0xfb800108,0xb6f90101,0xfb800108,0xb6fa0101,0xfb800108,0xb6fb0101,
	0xfb800108,0xb6fc0101,0xfb800108,0xb6fd0101,0xfb800108,0xb6fe0101,0xfb800108,0xb6ff0101,
	0xfb800108,0xb7000101,0xfb800108,0xb7010101,0xfb800108,0xb7020101,0xfb800108,0xb7030101,
	0xfb800108,0xb7040101,0xfb800108,0xb7050101,0xfb800108,0xb7060101,0xfb800108,0xb7070101,
	0xfb800108,0xb7080101,0xfb800108,0xb7090101,0xfb800108,0xb70a0101,0xfb800108,0xb70b0101,
	0xfb800108,0xb70c0101,0xfb800108,0xb70d0101,0xfb800108,0xb70e0101,0xfb800108,0xb70f0101,
	0xfb800108,0xb7100101,0xfb800108,0xb7110101,0xfb800108,0xb7120101,0xfb800108,0xb7130101,
	0xfb800108,0xb7140101,0xfb800108,0xb7150101,0xfb800108,0xb7160101,0xfb800108,0xb7170101,
	0xfb800108,0xb7180101,0xfb800108,0xb7190101,0xfb800108,0xb71a0101,0xfb800108,0xb71b0101,
	0xfb800108,0xb71c0101,0xfb800108,0xb71d0101,0xfb800108,0xb71e0101,0xfb800108,0xb71f0101,
	0xfb800108,0xb7200101,0xfb800108,0xb7210101,0xfb800108,0xb7220101,0xfb800108,0xb7230101,
	0xfb800108,0xb7240101,0xfb800108,0xb7250101,0xfb800108,0xb7260101,0xfb800108,0xb7270101,
	0xfb800108,0xb7280101,0xfb800108,0xb7290101,0xfb800108,0xb72a0101,0xfb800108,0xb72b0101,
	0xfb800108,0xb72c0101,0xfb800108,0xb72d0101,0xfb800108,0xb72e0101,0xfb800108,0xb72f0101,
	0xfb800108,0xb7300101,0xfb800108,0xb7310101,0xfb800108,0xb7320101,0xfb800108,0xb7330101,
	0xfb800108,0xb7340101,0xfb800108,0xb7350101,0xfb800108,0xb7360101,0xfb800108,0xb7370101,
	0xfb800108,0xb7380101,0xfb800108,0xb7390101,0xfb800108,0xb73a0101,0xfb800108,0xb73b0101,
	0xfb800108,0xb73c0101,0xfb800108,0xb73d0101,0xfb800108,0xb73e0101,0xfb800108,0xb73f0101,
	0xfb800108,0xb7400101,0xfb800108,0xb7410101,0xfb800108,0xb7420101,0xfb800108,0xb7430101,
	0xfb800108,0xb7440101,0xfb800108,0xb7450101,0xfb800108,0xb7460101,0xfb800108,0xb7470101,
	0xfb800108,0xb7480101,0xfb800108,0xb7490101,0xfb800108,0xb74a0101,0xfb800108,0xb74b0101,
	0xfb800108,0xb74c0101,0xfb800108,0xb74d0101,0xfb800108,0xb74e0101,0xfb800108,0xb74f0101,
	0xfb800108,0xb7500101,0xfb800108,0xb7510101,0xfb800108,0xb7520101,0xfb800108,0xb7530101,
	0xfb800108,0xb7540101,0xfb800108,0xb7550101,0xfb800108,0xb7560101,0xfb800108,0xb7570101,
	0xfb800108,0xb7580101,0xfb800108,0xb7590101,0xfb800108,0xb75a0101,0xfb800108,0xb75b0101,
	0xfb800108,0xb75c0101,0xfb800108,0xb75d0101,0xfb800108,0xb75e0101,0xfb800108,0xb75f0101,
	0xfb800108,0xb7600101,0xfb800108,0xb7610101,0xfb800108,0xb7620101,0xfb800108,0xb7630101,
	0xfb800108,0xb7640101,0xfb800108,0xb7650101,0xfb800108,0xb7660101,0xfb800108,0xb7670101,
	0xfb800108,0xb7680101,0xfb800108,0xb7690101,0xfb800108,0xb76a0101,0xfb800108,0xb76b0101,
	0xfb800108,0xb76c0101,0xfb800108,0xb76d0101,0xfb800108,0xb76e0101,0xfb800108,0xb76f0101,
	0xfb800108,0xb7700101,0xfb800108,0xb7710101,0xfb800108,0xb7720101,0xfb800108,0xb7730101,
	0xfb800108,0xb7740101,0xfb800108,0xb7750101,0xfb800108,0xb7760101,0xfb800108,0xb7770101,
	0xfb800108,0xb7780101,0xfb800108,0xb7790101,0xfb800108,0xb77a0101,0xfb800108,0xb77b0101,
	0xfb800108,0xb77c0101,0xfb800108,0xb77d0101,0xfb800108,0xb77e0101,0xfb800108,0xb77f0101,
	0xfb800108,0xb7800101,0xfb800108,0xb7810101,0xfb800108,0xb7820101,0xfb800108,0xb7830101,
	0xfb800108,0xb7840101,0xfb800108,0xb7850101,0xfb800108,0xb7860101,0xfb800108,0xb7870101,
	0xfb800108,0xb7880101,0xfb800108,0xb7890101,0xfb800108,0xb78a0101,0xfb800108,0xb78b0101,
	0xfb800108,0xb78c0101,0xfb800108,0xb78d0101,0xfb800108,0xb78e0101,0xfb800108,0xb78f0101,
	0xfb800108,0xb7900101,0xfb800108,0xb7910101,0xfb800108,0xb7920101,0xfb800108,0xb7930101,
	0xfb800108,0xb7940101,0xfb800108,0xb7950101,0xfb800108,0xb7960101,0xfb800108,0xb7970101,
	0xfb800108,0xb7980101,0xfb800108,0xb7990101,0xfb800108,0xb79a0101,0xfb800108,0xb79b0101,
	0xfb800108,0xb79c0101,0xfb800108,0xb79d0101,0xfb800108,0xb79e0101,0xfb800108,0xb79f0101,
	0xfb800108,0xb7a00101,0xfb800108,0xb7a10101,0xfb800108,0xb7a20101,0xfb800108,0xb7a30101,
	0xfb800108,0xb7a40101,0xfb800108,0xb7a50101,0xfb800108,0xb7a60101,0xfb800108,0xb7a70101,
	0xfb800108,0xb7a80101,0xfb800108,0xb7a90101,0xfb800108,0xb7aa0101,0xfb800108,0xb7ab0101,
	0xfb800108,0xb7ac0101,0xfb800108,0xb7ad0101,0xfb800108,0xb7ae0101,0xfb800108,0xb7af0101,
	0xfb800108,0xb7b00101,0xfb800108,0xb7b10101,0xfb800108,0xb7b20101,0xfb800108,0xb7b30101,
	0xfb800108,0xb7b40101,0xfb800108,0xb7b50101,0xfb800108,0xb7b60101,0xfb800108,0xb7b70101,
	0xfb800108,0xb7b80101,0xfb800108,0xb7b90101,0xfb800108,0xb7ba0101,0xfb800108,0xb7bb0101,
	0xfb800108,0xb7bc0101,0xfb800108,0xb7bd0101,0xfb800108,0xb7be0101,0xfb800108,0xb7bf0101,
	0xfb800108,0xb7c00101,0xfb800108,0xb7c10101,0xfb800108,0xb7c20101,0xfb800108,0xb7c30101,
	0xfb800108,0xb7c40101,0xfb800108,0xb7c50101,0xfb800108,0xb7c60101,0xfb800108,0xb7c70101,
	0xfb800108,0xb7c80101,0xfb800108,0xb7c90101,0xfb800108,0xb7ca0101,0xfb800108,0xb7cb0101,
	0xfb800108,0xb7cc0101,0xfb800108,0xb7cd0101,0xfb800108,0xb7ce0101,0xfb800108,0xb7cf0101,
	0xfb800108,0xb7d00101,0xfb800108,0xb7d10101,0xfb800108,0xb7d20101,0xfb800108,0xb7d30101,
	0xfb800108,0xb7d40101,0xfb800108,0xb7d50101,0xfb800108,0xb7d60101,0xfb800108,0xb7d70101,
	0xfb800108,0xb7d80101,0xfb800108,0xb7d90101,0xfb800108,0xb7da0101,0xfb800108,0xb7db0101,
	0xfb800108,0xb7dc0101,0xfb800108,0xb7dd0101,0xfb800108,0xb7de0101,0xfb800108,0xb7df0101,
	0xfb800108,0xb7e00101,0xfb800108,0xb7e10101,0xfb800108,0xb7e20101,0xfb800108,0xb7e30101,
	0xfb800108,0xb7e40101,0xfb800108,0xb7e50101,0xfb800108,0xb7e60101,0xfb800108,0xb7e70101,
	0xfb800108,0xb7e80101,0xfb800108,0xb7e90101,0xfb800108,0xb7ea0101,0xfb800108,0xb7eb0101,
	0xfb800108,0xb7ec0101,0xfb800108,0xb7ed0101,0xfb800108,0xb7ee0101,0xfb800108,0xb7ef0101,
	0xfb800108,0xb7f00101,0xfb800108,0xb7f10101,0xfb800108,0xb7f20101,0xfb800108,0xb7f30101,
	0xfb800108,0xb7f40101,0xfb800108,0xb7f50101,0xfb800108,0xb7f60101,0xfb800108,0xb7f70101,
	0xfb800108,0xb7f80101,0xfb800108,0xb7f90101,0xfb800108,0xb7fa0101,0xfb800108,0xb7fb0101,
	0xfb800108,0xb7fc0101,0xfb800108,0xb7fd0101,0xfb800108,0xb7fe0101,0xfb800108,0xb7ff0101,
	0xfb800108,0xb8000101,0xfb800108,0xb8010101,0xfb800108,0xb8020101,0xfb800108,0xb8030101,
	0xfb800108,0xb8040101,0xfb800108,0xb8050101,0xfb800108,0xb8060101,0xfb800108,0xb8070101,
	0xfb800108,0xb8080101,0xfb800108,0xb8090101,0xfb800108,0xb80a0101,0xfb800108,0xb80b0101,
	0xfb800108,0xb80c0101,0xfb800108,0xb80d0101,0xfb800108,0xb80e0101,0xfb800108,0xb80f0101,
	0xfb800108,0xb8100101,0xfb800108,0xb8110101,0xfb800108,0xb8120101,0xfb800108,0xb8130101,
	0xfb800108,0xb8140101,0xfb800108,0xb8150101,0xfb800108,0xb8160101,0xfb800108,0xb8170101,
	0xfb800108,0xb8180101,0xfb800108,0xb8190101,0xfb800108,0xb81a0101,0xfb800108,0xb81b0101,
	0xfb800108,0xb81c0101,0xfb800108,0xb81d0101,0xfb800108,0xb81e0101,0xfb800108,0xb81f0101,
	0xfb800108,0xb8200101,0xfb800108,0xb8210101,0xfb800108,0xb8220101,0xfb800108,0xb8230101,
	0xfb800108,0xb8240101,0xfb800108,0xb8250101,0xfb800108,0xb8260101,0xfb800108,0xb8270101,
	0xfb800108,0xb8280101,0xfb800108,0xb8290101,0xfb800108,0xb82a0101,0xfb800108,0xb82b0101,
	0xfb800108,0xb82c0101,0xfb800108,0xb82d0101,0xfb800108,0xb82e0101,0xfb800108,0xb82f0101,
	0xfb800108,0xb8300101,0xfb800108,0xb8310101,0xfb800108,0xb8320101,0xfb800108,0xb8330101,
	0xfb800108,0xb8340101,0xfb800108,0xb8350101,0xfb800108,0xb8360101,0xfb800108,0xb8370101,
	0xfb800108,0xb8380101,0xfb800108,0xb8390101,0xfb800108,0xb83a0101,0xfb800108,0xb83b0101,
	0xfb800108,0xb83c0101,0xfb800108,0xb83d0101,0xfb800108,0xb83e0101,0xfb800108,0xb83f0101,
	0xfb800108,0xb8400101,0xfb800108,0xb8410101,0xfb800108,0xb8420101,0xfb800108,0xb8430101,
	0xfb800108,0xb8440101,0xfb800108,0xb8450101,0xfb800108,0xb8460101,0xfb800108,0xb8470101,
	0xfb800108,0xb8480101,0xfb800108,0xb8490101,0xfb800108,0xb84a0101,0xfb800108,0xb84b0101,
	0xfb800108,0xb84c0101,0xfb800108,0xb84d0101,0xfb800108,0xb84e0101,0xfb800108,0xb84f0101,
	0xfb800108,0xb8500101,0xfb800108,0xb8510101,0xfb800108,0xb8520101,0xfb800108,0xb8530101,
	0xfb800108,0xb8540101,0xfb800108,0xb8550101,0xfb800108,0xb8560101,0xfb800108,0xb8570101,
	0xfb800108,0xb8580101,0xfb800108,0xb8590101,0xfb800108,0xb85a0101,0xfb800108,0xb85b0101,
	0xfb800108,0xb85c0101,0xfb800108,0xb85d0101,0xfb800108,0xb85e0101,0xfb800108,0xb85f0101,
	0xfb800108,0xb8600101,0xfb800108,0xb8610101,0xfb800108,0xb8620101,0xfb800108,0xb8630101,
	0xfb800108,0xb8640101,0xfb800108,0xb8650101,0xfb800108,0xb8660101,0xfb800108,0xb8670101,
	0xfb800108,0xb8680101,0xfb800108,0xb8690101,0xfb800108,0xb86a0101,0xfb800108,0xb86b0101,
	0xfb800108,0xb86c0101,0xfb800108,0xb86d0101,0xfb800108,0xb86e0101,0xfb800108,0xb86f0101,
	0xfb800108,0xb8700101,0xfb800108,0xb8710101,0xfb800108,0xb8720101,0xfb800108,0xb8730101,
	0xfb800108,0xb8740101,0xfb800108,0xb8750101,0xfb800108,0xb8760101,0xfb800108,0xb8770101,
	0xfb800108,0xb8780101,0xfb800108,0xb8790101,0xfb800108,0xb87a0101,0xfb800108,0xb87b0101,
	0xfb800108,0xb87c0101,0xfb800108,0xb87d0101,0xfb800108,0xb87e0101,0xfb800108,0xb87f0101,
	0xfb800108,0xb8800101,0xfb800108,0xb8810101,0xfb800108,0xb8820101,0xfb800108,0xb8830101,
	0xfb800108,0xb8840101,0xfb800108,0xb8850101,0xfb800108,0xb8860101,0xfb800108,0xb8870101,
	0xfb800108,0xb8880101,0xfb800108,0xb8890101,0xfb800108,0xb88a0101,0xfb800108,0xb88b0101,
	0xfb800108,0xb88c0101,0xfb800108,0xb88d0101,0xfb800108,0xb88e0101,0xfb800108,0xb88f0101,
	0xfb800108,0xb8900101,0xfb800108,0xb8910101,0xfb800108,0xb8920101,0xfb800108,0xb8930101,
	0xfb800108,0xb8940101,0xfb800108,0xb8950101,0xfb800108,0xb8960101,0xfb800108,0xb8970101,
	0xfb800108,0xb8980101,0xfb800108,0xb8990101,0xfb800108,0xb89a0101,0xfb800108,0xb89b0101,
	0xfb800108,0xb89c0101,0xfb800108,0xb89d0101,0xfb800108,0xb89e0101,0xfb800108,0xb89f0101,
	0xfb800108,0xb8a00101,0xfb800108,0xb8a10101,0xfb800108,0xb8a20101,0xfb800108,0xb8a30101,
	0xfb800108,0xb8a40101,0xfb800108,0xb8a50101,0xfb800108,0xb8a60101,0xfb800108,0xb8a70101,
	0xfb800108,0xb8a80101,0xfb800108,0xb8a90101,0xfb800108,0xb8aa0101,0xfb800108,0xb8ab0101,
	0xfb800108,0xb8ac0101,0xfb800108,0xb8ad0101,0xfb800108,0xb8ae0101,0xfb800108,0xb8af0101,
	0xfb800108,0xb8b00101,0xfb800108,0xb8b10101,0xfb800108,0xb8b20101,0xfb800108,0xb8b30101,
	0xfb800108,0xb8b40101,0xfb800108,0xb8b50101,0xfb800108,0xb8b60101,0xfb800108,0xb8b70101,
	0xfb800108,0xb8b80101,0xfb800108,0xb8b90101,0xfb800108,0xb8ba0101,0xfb800108,0xb8bb0101,
	0xfb800108,0xb8bc0101,0xfb800108,0xb8bd0101,0xfb800108,0xb8be0101,0xfb800108,0xb8bf0101,
	0xfb800108,0xb8c00101,0xfb800108,0xb8c10101,0xfb800108,0xb8c20101,0xfb800108,0xb8c30101,
	0xfb800108,0xb8c40101,0xfb800108,0xb8c50101,0xfb800108,0xb8c60101,0xfb800108,0xb8c70101,
	0xfb800108,0xb8c80101,0xfb800108,0xb8c90101,0xfb800108,0xb8ca0101,0xfb800108,0xb8cb0101,
	0xfb800108,0xb8cc0101,0xfb800108,0xb8cd0101,0xfb800108,0xb8ce0101,0xfb800108,0xb8cf0101,
	0xfb800108,0xb8d00101,0xfb800108,0xb8d10101,0xfb800108,0xb8d20101,0xfb800108,0xb8d30101,
	0xfb800108,0xb8d40101,0xfb800108,0xb8d50101,0xfb800108,0xb8d60101,0xfb800108,0xb8d70101,
	0xfb800108,0xb8d80101,0xfb800108,0xb8d90101,0xfb800108,0xb8da0101,0xfb800108,0xb8db0101,
	0xfb800108,0xb8dc0101,0xfb800108,0xb8dd0101,0xfb800108,0xb8de0101,0xfb800108,0xb8df0101,
	0xfb800108,0xb8e00101,0xfb800108,0xb8e10101,0xfb800108,0xb8e20101,0xfb800108,0xb8e30101,
	0xfb800108,0xb8e40101,0xfb800108,0xb8e50101,0xfb800108,0xb8e60101,0xfb800108,0xb8e70101,
	0xfb800108,0xb8e80101,0xfb800108,0xb8e90101,0xfb800108,0xb8ea0101,0xfb800108,0xb8eb0101,
	0xfb800108,0xb8ec0101,0xfb800108,0xb8ed0101,0xfb800108,0xb8ee0101,0xfb800108,0xb8ef0101,
	0xfb800108,0xb8f00101,0xfb800108,0xb8f10101,0xfb800108,0xb8f20101,0xfb800108,0xb8f30101,
	0xfb800108,0xb8f40101,0xfb800108,0xb8f50101,0xfb800108,0xb8f60101,0xfb800108,0xb8f70101,
	0xfb800108,0xb8f80101,0xfb800108,0xb8f90101,0xfb800108,0xb8fa0101,0xfb800108,0xb8fb0101,
	0xfb800108,0xb8fc0101,0xfb800108,0xb8fd0101,0xfb800108,0xb8fe0101,0xfb800108,0xb8ff0101,
	0xfb800108,0xb9000101,0xfb800108,0xb9010101,0xfb800108,0xb9020101,0xfb800108,0xb9030101,
	0xfb800108,0xb9040101,0xfb800108,0xb9050101,0xfb800108,0xb9060101,0xfb800108,0xb9070101,
	0xfb800108,0xb9080101,0xfb800108,0xb9090101,0xfb800108,0xb90a0101,0xfb800108,0xb90b0101,
	0xfb800108,0xb90c0101,0xfb800108,0xb90d0101,0xfb800108,0xb90e0101,0xfb800108,0xb90f0101,
	0xfb800108,0xb9100101,0xfb800108,0xb9110101,0xfb800108,0xb9120101,0xfb800108,0xb9130101,
	0xfb800108,0xb9140101,0xfb800108,0xb9150101,0xfb800108,0xb9160101,0xfb800108,0xb9170101,
	0xfb800108,0xb9180101,0xfb800108,0xb9190101,0xfb800108,0xb91a0101,0xfb800108,0xb91b0101,
	0xfb800108,0xb91c0101,0xfb800108,0xb91d0101,0xfb800108,0xb91e0101,0xfb800108,0xb91f0101,
	0xfb800108,0xb9200101,0xfb800108,0xb9210101,0xfb800108,0xb9220101,0xfb800108,0xb9230101,
	0xfb800108,0xb9240101,0xfb800108,0xb9250101,0xfb800108,0xb9260101,0xfb800108,0xb9270101,
	0xfb800108,0xb9280101,0xfb800108,0xb9290101,0xfb800108,0xb92a0101,0xfb800108,0xb92b0101,
	0xfb800108,0xb92c0101,0xfb800108,0xb92d0101,0xfb800108,0xb92e0101,0xfb800108,0xb92f0101,
	0xfb800108,0xb9300101,0xfb800108,0xb9310101,0xfb800108,0xb9320101,0xfb800108,0xb9330101,
	0xfb800108,0xb9340101,0xfb800108,0xb9350101,0xfb800108,0xb9360101,0xfb800108,0xb9370101,
	0xfb800108,0xb9380101,0xfb800108,0xb9390101,0xfb800108,0xb93a0101,0xfb800108,0xb93b0101,
	0xfb800108,0xb93c0101,0xfb800108,0xb93d0101,0xfb800108,0xb93e0101,0xfb800108,0xb93f0101,
	0xfb800108,0xb9400101,0xfb800108,0xb9410101,0xfb800108,0xb9420101,0xfb800108,0xb9430101,
	0xfb800108,0xb9440101,0xfb800108,0xb9450101,0xfb800108,0xb9460101,0xfb800108,0xb9470101,
	0xfb800108,0xb9480101,0xfb800108,0xb9490101,0xfb800108,0xb94a0101,0xfb800108,0xb94b0101,
	0xfb800108,0xb94c0101,0xfb800108,0xb94d0101,0xfb800108,0xb94e0101,0xfb800108,0xb94f0101,
	0xfb800108,0xb9500101,0xfb800108,0xb9510101,0xfb800108,0xb9520101,0xfb800108,0xb9530101,
	0xfb800108,0xb9540101,0xfb800108,0xb9550101,0xfb800108,0xb9560101,0xfb800108,0xb9570101,
	0xfb800108,0xb9580101,0xfb800108,0xb9590101,0xfb800108,0xb95a0101,0xfb800108,0xb95b0101,
	0xfb800108,0xb95c0101,0xfb800108,0xb95d0101,0xfb800108,0xb95e0101,0xfb800108,0xb95f0101,
	0xfb800108,0xb9600101,0xfb800108,0xb9610101,0xfb800108,0xb9620101,0xfb800108,0xb9630101,
	0xfb800108,0xb9640101,0xfb800108,0xb9650101,0xfb800108,0xb9660101,0xfb800108,0xb9670101,
	0xfb800108,0xb9680101,0xfb800108,0xb9690101,0xfb800108,0xb96a0101,0xfb800108,0xb96b0101,
	0xfb800108,0xb96c0101,0xfb800108,0xb96d0101,0xfb800108,0xb96e0101,0xfb800108,0xb96f0101,
	0xfb800108,0xb9700101,0xfb800108,0xb9710101,0xfb800108,0xb9720101,0xfb800108,0xb9730101,
	0xfb800108,0xb9740101,0xfb800108,0xb9750101,0xfb800108,0xb9760101,0xfb800108,0xb9770101,
	0xfb800108,0xb9780101,0xfb800108,0xb9790101,0xfb800108,0xb97a0101,0xfb800108,0xb97b0101,
	0xfb800108,0xb97c0101,0xfb800108,0xb97d0101,0xfb800108,0xb97e0101,0xfb800108,0xb97f0101,
	0xfb800108,0xb9800101,0xfb800108,0xb9810101,0xfb800108,0xb9820101,0xfb800108,0xb9830101,
	0xfb800108,0xb9840101,0xfb800108,0xb9850101,0xfb800108,0xb9860101,0xfb800108,0xb9870101,
	0xfb800108,0xb9880101,0xfb800108,0xb9890101,0xfb800108,0xb98a0101,0xfb800108,0xb98b0101,
	0xfb800108,0xb98c0101,0xfb800108,0xb98d0101,0xfb800108,0xb98e0101,0xfb800108,0xb98f0101,
	0xfb800108,0xb9900101,0xfb800108,0xb9910101,0xfb800108,0xb9920101,0xfb800108,0xb9930101,
	0xfb800108,0xb9940101,0xfb800108,0xb9950101,0xfb800108,0xb9960101,0xfb800108,0xb9970101,
	0xfb800108,0xb9980101,0xfb800108,0xb9990101,0xfb800108,0xb99a0101,0xfb800108,0xb99b0101,
	0xfb800108,0xb99c0101,0xfb800108,0xb99d0101,0xfb800108,0xb99e0101,0xfb800108,0xb99f0101,
	0xfb800108,0xb9a00101,0xfb800108,0xb9a10101,0xfb800108,0xb9a20101,0xfb800108,0xb9a30101,
	0xfb800108,0xb9a40101,0xfb800108,0xb9a50101,0xfb800108,0xb9a60101,0xfb800108,0xb9a70101,
	0xfb800108,0xb9a80101,0xfb800108,0xb9a90101,0xfb800108,0xb9aa0101,0xfb800108,0xb9ab0101,
	0xfb800108,0xb9ac0101,0xfb800108,0xb9ad0101,0xfb800108,0xb9ae0101,0xfb800108,0xb9af0101,
	0xfb800108,0xb9b00101,0xfb800108,0xb9b10101,0xfb800108,0xb9b20101,0xfb800108,0xb9b30101,
	0xfb800108,0xb9b40101,0xfb800108,0xb9b50101,0xfb800108,0xb9b60101,0xfb800108,0xb9b70101,
	0xfb800108,0xb9b80101,0xfb800108,0xb9b90101,0xfb800108,0xb9ba0101,0xfb800108,0xb9bb0101,
	0xfb800108,0xb9bc0101,0xfb800108,0xb9bd0101,0xfb800108,0xb9be0101,0xfb800108,0xb9bf0101,
	0xfb800108,0xb9c00101,0xfb800108,0xb9c10101,0xfb800108,0xb9c20101,0xfb800108,0xb9c30101,
	0xfb800108,0xb9c40101,0xfb800108,0xb9c50101,0xfb800108,0xb9c60101,0xfb800108,0xb9c70101,
	0xfb800108,0xb9c80101,0xfb800108,0xb9c90101,0xfb800108,0xb9ca0101,0xfb800108,0xb9cb0101,
	0xfb800108,0xb9cc0101,0xfb800108,0xb9cd0101,0xfb800108,0xb9ce0101,0xfb800108,0xb9cf0101,
	0xfb800108,0xb9d00101,0xfb800108,0xb9d10101,0xfb800108,0xb9d20101,0xfb800108,0xb9d30101,
	0xfb800108,0xb9d40101,0xfb800108,0xb9d50101,0xfb800108,0xb9d60101,0xfb800108,0xb9d70101,
	0xfb800108,0xb9d80101,0xfb800108,0xb9d90101,0xfb800108,0xb9da0101,0xfb800108,0xb9db0101,
	0xfb800108,0xb9dc0101,0xfb800108,0xb9dd0101,0xfb800108,0xb9de0101,0xfb800108,0xb9df0101,
	0xfb800108,0xb9e00101,0xfb800108,0xb9e10101,0xfb800108,0xb9e20101,0xfb800108,0xb9e30101,
	0xfb800108,0xb9e40101,0xfb800108,0xb9e50101,0xfb800108,0xb9e60101,0xfb800108,0xb9e70101,
	0xfb800108,0xb9e80101,0xfb800108,0xb9e90101,0xfb800108,0xb9ea0101,0xfb800108,0xb9eb0101,
	0xfb800108,0xb9ec0101,0xfb800108,0xb9ed0101,0xfb800108,0xb9ee0101,0xfb800108,0xb9ef0101,
	0xfb800108,0xb9f00101,0xfb800108,0xb9f10101,0xfb800108,0xb9f20101,0xfb800108,0xb9f30101,
	0xfb800108,0xb9f40101,0xfb800108,0xb9f50101,0xfb800108,0xb9f60101,0xfb800108,0xb9f70101,
	0xfb800108,0xb9f80101,0xfb800108,0xb9f90101,0xfb800108,0xb9fa0101,0xfb800108,0xb9fb0101,
	0xfb800108,0xb9fc0101,0xfb800108,0xb9fd0101,0xfb800108,0xb9fe0101,0xfb800108,0xb9ff0101,
	0xfb800108,0xba000101,0xfb800108,0xba010101,0xfb800108,0xba020101,0xfb800108,0xba030101,
	0xfb800108,0xba040101,0xfb800108,0xba050101,0xfb800108,0xba060101,0xfb800108,0xba070101,
	0xfb800108,0xba080101,0xfb800108,0xba090101,0xfb800108,0xba0a0101,0xfb800108,0xba0b0101,
	0xfb800108,0xba0c0101,0xfb800108,0xba0d0101,0xfb800108,0xba0e0101,0xfb800108,0xba0f0101,
	0xfb800108,0xba100101,0xfb800108,0xba110101,0xfb800108,0xba120101,0xfb800108,0xba130101,
	0xfb800108,0xba140101,0xfb800108,0xba150101,0xfb800108,0xba160101,0xfb800108,0xba170101,
	0xfb800108,0xba180101,0xfb800108,0xba190101,0xfb800108,0xba1a0101,0xfb800108,0xba1b0101,
	0xfb800108,0xba1c0101,0xfb800108,0xba1d0101,0xfb800108,0xba1e0101,0xfb800108,0xba1f0101,
	0xfb800108,0xba200101,0xfb800108,0xba210101,0xfb800108,0xba220101,0xfb800108,0xba230101,
	0xfb800108,0xba240101,0xfb800108,0xba250101,0xfb800108,0xba260101,0xfb800108,0xba270101,
	0xfb800108,0xba280101,0xfb800108,0xba290101,0xfb800108,0xba2a0101,0xfb800108,0xba2b0101,
	0xfb800108,0xba2c0101,0xfb800108,0xba2d0101,0xfb800108,0xba2e0101,0xfb800108,0xba2f0101,
	0xfb800108,0xba300101,0xfb800108,0xba310101,0xfb800108,0xba320101,0xfb800108,0xba330101,
	0xfb800108,0xba340101,0xfb800108,0xba350101,0xfb800108,0xba360101,0xfb800108,0xba370101,
	0xfb800108,0xba380101,0xfb800108,0xba390101,0xfb800108,0xba3a0101,0xfb800108,0xba3b0101,
	0xfb800108,0xba3c0101,0xfb800108,0xba3d0101,0xfb800108,0xba3e0101,0xfb800108,0xba3f0101,
	0xfb800108,0xba400101,0xfb800108,0xba410101,0xfb800108,0xba420101,0xfb800108,0xba430101,
	0xfb800108,0xba440101,0xfb800108,0xba450101,0xfb800108,0xba460101,0xfb800108,0xba470101,
	0xfb800108,0xba480101,0xfb800108,0xba490101,0xfb800108,0xba4a0101,0xfb800108,0xba4b0101,
	0xfb800108,0xba4c0101,0xfb800108,0xba4d0101,0xfb800108,0xba4e0101,0xfb800108,0xba4f0101,
	0xfb800108,0xba500101,0xfb800108,0xba510101,0xfb800108,0xba520101,0xfb800108,0xba530101,
	0xfb800108,0xba540101,0xfb800108,0xba550101,0xfb800108,0xba560101,0xfb800108,0xba570101,
	0xfb800108,0xba580101,0xfb800108,0xba590101,0xfb800108,0xba5a0101,0xfb800108,0xba5b0101,
	0xfb800108,0xba5c0101,0xfb800108,0xba5d0101,0xfb800108,0xba5e0101,0xfb800108,0xba5f0101,
	0xfb800108,0xba600101,0xfb800108,0xba610101,0xfb800108,0xba620101,0xfb800108,0xba630101,
	0xfb800108,0xba640101,0xfb800108,0xba650101,0xfb800108,0xba660101,0xfb800108,0xba670101,
	0xfb800108,0xba680101,0xfb800108,0xba690101,0xfb800108,0xba6a0101,0xfb800108,0xba6b0101,
	0xfb800108,0xba6c0101,0xfb800108,0xba6d0101,0xfb800108,0xba6e0101,0xfb800108,0xba6f0101,
	0xfb800108,0xba700101,0xfb800108,0xba710101,0xfb800108,0xba720101,0xfb800108,0xba730101,
	0xfb800108,0xba740101,0xfb800108,0xba750101,0xfb800108,0xba760101,0xfb800108,0xba770101,
	0xfb800108,0xba780101,0xfb800108,0xba790101,0xfb800108,0xba7a0101,0xfb800108,0xba7b0101,
	0xfb800108,0xba7c0101,0xfb800108,0xba7d0101,0xfb800108,0xba7e0101,0xfb800108,0xba7f0101,
	0xfb800108,0xba800101,0xfb800108,0xba810101,0xfb800108,0xba820101,0xfb800108,0xba830101,
	0xfb800108,0xba840101,0xfb800108,0xba850101,0xfb800108,0xba860101,0xfb800108,0xba870101,
	0xfb800108,0xba880101,0xfb800108,0xba890101,0xfb800108,0xba8a0101,0xfb800108,0xba8b0101,
	0xfb800108,0xba8c0101,0xfb800108,0xba8d0101,0xfb800108,0xba8e0101,0xfb800108,0xba8f0101,
	0xfb800108,0xba900101,0xfb800108,0xba910101,0xfb800108,0xba920101,0xfb800108,0xba930101,
	0xfb800108,0xba940101,0xfb800108,0xba950101,0xfb800108,0xba960101,0xfb800108,0xba970101,
	0xfb800108,0xba980101,0xfb800108,0xba990101,0xfb800108,0xba9a0101,0xfb800108,0xba9b0101,
	0xfb800108,0xba9c0101,0xfb800108,0xba9d0101,0xfb800108,0xba9e0101,0xfb800108,0xba9f0101,
	0xfb800108,0xbaa00101,0xfb800108,0xbaa10101,0xfb800108,0xbaa20101,0xfb800108,0xbaa30101,
	0xfb800108,0xbaa40101,0xfb800108,0xbaa50101,0xfb800108,0xbaa60101,0xfb800108,0xbaa70101,
	0xfb800108,0xbaa80101,0xfb800108,0xbaa90101,0xfb800108,0xbaaa0101,0xfb800108,0xbaab0101,
	0xfb800108,0xbaac0101,0xfb800108,0xbaad0101,0xfb800108,0xbaae0101,0xfb800108,0xbaaf0101,
	0xfb800108,0xbab00101,0xfb800108,0xbab10101,0xfb800108,0xbab20101,0xfb800108,0xbab30101,
	0xfb800108,0xbab40101,0xfb800108,0xbab50101,0xfb800108,0xbab60101,0xfb800108,0xbab70101,
	0xfb800108,0xbab80101,0xfb800108,0xbab90101,0xfb800108,0xbaba0101,0xfb800108,0xbabb0101,
	0xfb800108,0xbabc0101,0xfb800108,0xbabd0101,0xfb800108,0xbabe0101,0xfb800108,0xbabf0101,
	0xfb800108,0xbac00101,0xfb800108,0xbac10101,0xfb800108,0xbac20101,0xfb800108,0xbac30101,
	0xfb800108,0xbac40101,0xfb800108,0xbac50101,0xfb800108,0xbac60101,0xfb800108,0xbac70101,
	0xfb800108,0xbac80101,0xfb800108,0xbac90101,0xfb800108,0xbaca0101,0xfb800108,0xbacb0101,
	0xfb800108,0xbacc0101,0xfb800108,0xbacd0101,0xfb800108,0xbace0101,0xfb800108,0xbacf0101,
	0xfb800108,0xbad00101,0xfb800108,0xbad10101,0xfb800108,0xbad20101,0xfb800108,0xbad30101,
	0xfb800108,0xbad40101,0xfb800108,0xbad50101,0xfb800108,0xbad60101,0xfb800108,0xbad70101,
	0xfb800108,0xbad80101,0xfb800108,0xbad90101,0xfb800108,0xbada0101,0xfb800108,0xbadb0101,
	0xfb800108,0xbadc0101,0xfb800108,0xbadd0101,0xfb800108,0xbade0101,0xfb800108,0xbadf0101,
	0xfb800108,0xbae00101,0xfb800108,0xbae10101,0xfb800108,0xbae20101,0xfb800108,0xbae30101,
	0xfb800108,0xbae40101,0xfb800108,0xbae50101,0xfb800108,0xbae60101,0xfb800108,0xbae70101,
	0xfb800108,0xbae80101,0xfb800108,0xbae90101,0xfb800108,0xbaea0101,0xfb800108,0xbaeb0101,
	0xfb800108,0xbaec0101,0xfb800108,0xbaed0101,0xfb800108,0xbaee0101,0xfb800108,0xbaef0101,
	0xfb800108,0xbaf00101,0xfb800108,0xbaf10101,0xfb800108,0xbaf20101,0xfb800108,0xbaf30101,
	0xfb800108,0xbaf40101,0xfb800108,0xbaf50101,0xfb800108,0xbaf60101,0xfb800108,0xbaf70101,
	0xfb800108,0xbaf80101,0xfb800108,0xbaf90101,0xfb800108,0xbafa0101,0xfb800108,0xbafb0101,
	0xfb800108,0xbafc0101,0xfb800108,0xbafd0101,0xfb800108,0xbafe0101,0xfb800108,0xbaff0101,
	0xfb800108,0xbb000101,0xfb800108,0xbb010101,0xfb800108,0xbb020101,0xfb800108,0xbb030101,
	0xfb800108,0xbb040101,0xfb800108,0xbb050101,0xfb800108,0xbb060101,0xfb800108,0xbb070101,
	0xfb800108,0xbb080101,0xfb800108,0xbb090101,0xfb800108,0xbb0a0101,0xfb800108,0xbb0b0101,
	0xfb800108,0xbb0c0101,0xfb800108,0xbb0d0101,0xfb800108,0xbb0e0101,0xfb800108,0xbb0f0101,
	0xfb800108,0xbb100101,0xfb800108,0xbb110101,0xfb800108,0xbb120101,0xfb800108,0xbb130101,
	0xfb800108,0xbb140101,0xfb800108,0xbb150101,0xfb800108,0xbb160101,0xfb800108,0xbb170101,
	0xfb800108,0xbb180101,0xfb800108,0xbb190101,0xfb800108,0xbb1a0101,0xfb800108,0xbb1b0101,
	0xfb800108,0xbb1c0101,0xfb800108,0xbb1d0101,0xfb800108,0xbb1e0101,0xfb800108,0xbb1f0101,
	0xfb800108,0xbb200101,0xfb800108,0xbb210101,0xfb800108,0xbb220101,0xfb800108,0xbb230101,
	0xfb800108,0xbb240101,0xfb800108,0xbb250101,0xfb800108,0xbb260101,0xfb800108,0xbb270101,
	0xfb800108,0xbb280101,0xfb800108,0xbb290101,0xfb800108,0xbb2a0101,0xfb800108,0xbb2b0101,
	0xfb800108,0xbb2c0101,0xfb800108,0xbb2d0101,0xfb800108,0xbb2e0101,0xfb800108,0xbb2f0101,
	0xfb800108,0xbb300101,0xfb800108,0xbb310101,0xfb800108,0xbb320101,0xfb800108,0xbb330101,
	0xfb800108,0xbb340101,0xfb800108,0xbb350101,0xfb800108,0xbb360101,0xfb800108,0xbb370101,
	0xfb800108,0xbb380101,0xfb800108,0xbb390101,0xfb800108,0xbb3a0101,0xfb800108,0xbb3b0101,
	0xfb800108,0xbb3c0101,0xfb800108,0xbb3d0101,0xfb800108,0xbb3e0101,0xfb800108,0xbb3f0101,
	0xfb800108,0xbb400101,0xfb800108,0xbb410101,0xfb800108,0xbb420101,0xfb800108,0xbb430101,
	0xfb800108,0xbb440101,0xfb800108,0xbb450101,0xfb800108,0xbb460101,0xfb800108,0xbb470101,
	0xfb800108,0xbb480101,0xfb800108,0xbb490101,0xfb800108,0xbb4a0101,0xfb800108,0xbb4b0101,
	0xfb800108,0xbb4c0101,0xfb800108,0xbb4d0101,0xfb800108,0xbb4e0101,0xfb800108,0xbb4f0101,
	0xfb800108,0xbb500101,0xfb800108,0xbb510101,0xfb800108,0xbb520101,0xfb800108,0xbb530101,
	0xfb800108,0xbb540101,0xfb800108,0xbb550101,0xfb800108,0xbb560101,0xfb800108,0xbb570101,
	0xfb800108,0xbb580101,0xfb800108,0xbb590101,0xfb800108,0xbb5a0101,0xfb800108,0xbb5b0101,
	0xfb800108,0xbb5c0101,0xfb800108,0xbb5d0101,0xfb800108,0xbb5e0101,0xfb800108,0xbb5f0101,
	0xfb800108,0xbb600101,0xfb800108,0xbb610101,0xfb800108,0xbb620101,0xfb800108,0xbb630101,
	0xfb800108,0xbb640101,0xfb800108,0xbb650101,0xfb800108,0xbb660101,0xfb800108,0xbb670101,
	0xfb800108,0xbb680101,0xfb800108,0xbb690101,0xfb800108,0xbb6a0101,0xfb800108,0xbb6b0101,
	0xfb800108,0xbb6c0101,0xfb800108,0xbb6d0101,0xfb800108,0xbb6e0101,0xfb800108,0xbb6f0101,
	0xfb800108,0xbb700101,0xfb800108,0xbb710101,0xfb800108,0xbb720101,0xfb800108,0xbb730101,
	0xfb800108,0xbb740101,0xfb800108,0xbb750101,0xfb800108,0xbb760101,0xfb800108,0xbb770101,
	0xfb800108,0xbb780101,0xfb800108,0xbb790101,0xfb800108,0xbb7a0101,0xfb800108,0xbb7b0101,
	0xfb800108,0xbb7c0101,0xfb800108,0xbb7d0101,0xfb800108,0xbb7e0101,0xfb800108,0xbb7f0101,
	0xfb800108,0xbb800101,0xfb800108,0xbb810101,0xfb800108,0xbb820101,0xfb800108,0xbb830101,
	0xfb800108,0xbb840101,0xfb800108,0xbb850101,0xfb800108,0xbb860101,0xfb800108,0xbb870101,
	0xfb800108,0xbb880101,0xfb800108,0xbb890101,0xfb800108,0xbb8a0101,0xfb800108,0xbb8b0101,
	0xfb800108,0xbb8c0101,0xfb800108,0xbb8d0101,0xfb800108,0xbb8e0101,0xfb800108,0xbb8f0101,
	0xfb800108,0xbb900101,0xfb800108,0xbb910101,0xfb800108,0xbb920101,0xfb800108,0xbb930101,
	0xfb800108,0xbb940101,0xfb800108,0xbb950101,0xfb800108,0xbb960101,0xfb800108,0xbb970101,
	0xfb800108,0xbb980101,0xfb800108,0xbb990101,0xfb800108,0xbb9a0101,0xfb800108,0xbb9b0101,
	0xfb800108,0xbb9c0101,0xfb800108,0xbb9d0101,0xfb800108,0xbb9e0101,0xfb800108,0xbb9f0101,
	0xfb800108,0xbba00101,0xfb800108,0xbba10101,0xfb800108,0xbba20101,0xfb800108,0xbba30101,
	0xfb800108,0xbba40101,0xfb800108,0xbba50101,0xfb800108,0xbba60101,0xfb800108,0xbba70101,
	0xfb800108,0xbba80101,0xfb800108,0xbba90101,0xfb800108,0xbbaa0101,0xfb800108,0xbbab0101,
	0xfb800108,0xbbac0101,0xfb800108,0xbbad0101,0xfb800108,0xbbae0101,0xfb800108,0xbbaf0101,
	0xfb800108,0xbbb00101,0xfb800108,0xbbb10101,0xfb800108,0xbbb20101,0xfb800108,0xbbb30101,
	0xfb800108,0xbbb40101,0xfb800108,0xbbb50101,0xfb800108,0xbbb60101,0xfb800108,0xbbb70101,
	0xfb800108,0xbbb80101,0xfb800108,0xbbb90101,0xfb800108,0xbbba0101,0xfb800108,0xbbbb0101,
	0xfb800108,0xbbbc0101,0xfb800108,0xbbbd0101,0xfb800108,0xbbbe0101,0xfb800108,0xbbbf0101,
	0xfb800108,0xbbc00101,0xfb800108,0xbbc10101,0xfb800108,0xbbc20101,0xfb800108,0xbbc30101,
	0xfb800108,0xbbc40101,0xfb800108,0xbbc50101,0xfb800108,0xbbc60101,0xfb800108,0xbbc70101,
	0xfb800108,0xbbc80101,0xfb800108,0xbbc90101,0xfb800108,0xbbca0101,0xfb800108,0xbbcb0101,
	0xfb800108,0xbbcc0101,0xfb800108,0xbbcd0101,0xfb800108,0xbbce0101,0xfb800108,0xbbcf0101,
	0xfb800108,0xbbd00101,0xfb800108,0xbbd10101,0xfb800108,0xbbd20101,0xfb800108,0xbbd30101,
	0xfb800108,0xbbd40101,0xfb800108,0xbbd50101,0xfb800108,0xbbd60101,0xfb800108,0xbbd70101,
	0xfb800108,0xbbd80101,0xfb800108,0xbbd90101,0xfb800108,0xbbda0101,0xfb800108,0xbbdb0101,
	0xfb800108,0xbbdc0101,0xfb800108,0xbbdd0101,0xfb800108,0xbbde0101,0xfb800108,0xbbdf0101,
	0xfb800108,0xbbe00101,0xfb800108,0xbbe10101,0xfb800108,0xbbe20101,0xfb800108,0xbbe30101,
	0xfb800108,0xbbe40101,0xfb800108,0xbbe50101,0xfb800108,0xbbe60101,0xfb800108,0xbbe70101,
	0xfb800108,0xbbe80101,0xfb800108,0xbbe90101,0xfb800108,0xbbea0101,0xfb800108,0xbbeb0101,
	0xfb800108,0xbbec0101,0xfb800108,0xbbed0101,0xfb800108,0xbbee0101,0xfb800108,0xbbef0101,
	0xfb800108,0xbbf00101,0xfb800108,0xbbf10101,0xfb800108,0xbbf20101,0xfb800108,0xbbf30101,
	0xfb800108,0xbbf40101,0xfb800108,0xbbf50101,0xfb800108,0xbbf60101,0xfb800108,0xbbf70101,
	0xfb800108,0xbbf80101,0xfb800108,0xbbf90101,0xfb800108,0xbbfa0101,0xfb800108,0xbbfb0101,
	0xfb800108,0xbbfc0101,0xfb800108,0xbbfd0101,0xfb800108,0xbbfe0101,0xfb800108,0xbbff0101,
	0xfb800108,0xbc000101,0xfb800108,0xbc010101,0xfb800108,0xbc020101,0xfb800108,0xbc030101,
	0xfb800108,0xbc040101,0xfb800108,0xbc050101,0xfb800108,0xbc060101,0xfb800108,0xbc070101,
	0xfb800108,0xbc080101,0xfb800108,0xbc090101,0xfb800108,0xbc0a0101,0xfb800108,0xbc0b0101,
	0xfb800108,0xbc0c0101,0xfb800108,0xbc0d0101,0xfb800108,0xbc0e0101,0xfb800108,0xbc0f0101,
	0xfb800108,0xbc100101,0xfb800108,0xbc110101,0xfb800108,0xbc120101,0xfb800108,0xbc130101,
	0xfb800108,0xbc140101,0xfb800108,0xbc150101,0xfb800108,0xbc160101,0xfb800108,0xbc170101,
	0xfb800108,0xbc180101,0xfb800108,0xbc190101,0xfb800108,0xbc1a0101,0xfb800108,0xbc1b0101,
	0xfb800108,0xbc1c0101,0xfb800108,0xbc1d0101,0xfb800108,0xbc1e0101,0xfb800108,0xbc1f0101,
	0xfb800108,0xbc200101,0xfb800108,0xbc210101,0xfb800108,0xbc220101,0xfb800108,0xbc230101,
	0xfb800108,0xbc240101,0xfb800108,0xbc250101,0xfb800108,0xbc260101,0xfb800108,0xbc270101,
	0xfb800108,0xbc280101,0xfb800108,0xbc290101,0xfb800108,0xbc2a0101,0xfb800108,0xbc2b0101,
	0xfb800108,0xbc2c0101,0xfb800108,0xbc2d0101,0xfb800108,0xbc2e0101,0xfb800108,0xbc2f0101,
	0xfb800108,0xbc300101,0xfb800108,0xbc310101,0xfb800108,0xbc320101,0xfb800108,0xbc330101,
	0xfb800108,0xbc340101,0xfb800108,0xbc350101,0xfb800108,0xbc360101,0xfb800108,0xbc370101,
	0xfb800108,0xbc380101,0xfb800108,0xbc390101,0xfb800108,0xbc3a0101,0xfb800108,0xbc3b0101,
	0xfb800108,0xbc3c0101,0xfb800108,0xbc3d0101,0xfb800108,0xbc3e0101,0xfb800108,0xbc3f0101,
	0xfb800108,0xbc400101,0xfb800108,0xbc410101,0xfb800108,0xbc420101,0xfb800108,0xbc430101,
	0xfb800108,0xbc440101,0xfb800108,0xbc450101,0xfb800108,0xbc460101,0xfb800108,0xbc470101,
	0xfb800108,0xbc480101,0xfb800108,0xbc490101,0xfb800108,0xbc4a0101,0xfb800108,0xbc4b0101,
	0xfb800108,0xbc4c0101,0xfb800108,0xbc4d0101,0xfb800108,0xbc4e0101,0xfb800108,0xbc4f0101,
	0xfb800108,0xbc500101,0xfb800108,0xbc510101,0xfb800108,0xbc520101,0xfb800108,0xbc530101,
	0xfb800108,0xbc540101,0xfb800108,0xbc550101,0xfb800108,0xbc560101,0xfb800108,0xbc570101,
	0xfb800108,0xbc580101,0xfb800108,0xbc590101,0xfb800108,0xbc5a0101,0xfb800108,0xbc5b0101,
	0xfb800108,0xbc5c0101,0xfb800108,0xbc5d0101,0xfb800108,0xbc5e0101,0xfb800108,0xbc5f0101,
	0xfb800108,0xbc600101,0xfb800108,0xbc610101,0xfb800108,0xbc620101,0xfb800108,0xbc630101,
	0xfb800108,0xbc640101,0xfb800108,0xbc650101,0xfb800108,0xbc660101,0xfb800108,0xbc670101,
	0xfb800108,0xbc680101,0xfb800108,0xbc690101,0xfb800108,0xbc6a0101,0xfb800108,0xbc6b0101,
	0xfb800108,0xbc6c0101,0xfb800108,0xbc6d0101,0xfb800108,0xbc6e0101,0xfb800108,0xbc6f0101,
	0xfb800108,0xbc700101,0xfb800108,0xbc710101,0xfb800108,0xbc720101,0xfb800108,0xbc730101,
	0xfb800108,0xbc740101,0xfb800108,0xbc750101,0xfb800108,0xbc760101,0xfb800108,0xbc770101,
	0xfb800108,0xbc780101,0xfb800108,0xbc790101,0xfb800108,0xbc7a0101,0xfb800108,0xbc7b0101,
	0xfb800108,0xbc7c0101,0xfb800108,0xbc7d0101,0xfb800108,0xbc7e0101,0xfb800108,0xbc7f0101,
	0xfb800108,0xbc800101,0xfb800108,0xbc810101,0xfb800108,0xbc820101,0xfb800108,0xbc830101,
	0xfb800108,0xbc840101,0xfb800108,0xbc850101,0xfb800108,0xbc860101,0xfb800108,0xbc870101,
	0xfb800108,0xbc880101,0xfb800108,0xbc890101,0xfb800108,0xbc8a0101,0xfb800108,0xbc8b0101,
	0xfb800108,0xbc8c0101,0xfb800108,0xbc8d0101,0xfb800108,0xbc8e0101,0xfb800108,0xbc8f0101,
	0xfb800108,0xbc900101,0xfb800108,0xbc910101,0xfb800108,0xbc920101,0xfb800108,0xbc930101,
	0xfb800108,0xbc940101,0xfb800108,0xbc950101,0xfb800108,0xbc960101,0xfb800108,0xbc970101,
	0xfb800108,0xbc980101,0xfb800108,0xbc990101,0xfb800108,0xbc9a0101,0xfb800108,0xbc9b0101,
	0xfb800108,0xbc9c0101,0xfb800108,0xbc9d0101,0xfb800108,0xbc9e0101,0xfb800108,0xbc9f0101,
	0xfb800108,0xbca00101,0xfb800108,0xbca10101,0xfb800108,0xbca20101,0xfb800108,0xbca30101,
	0xfb800108,0xbca40101,0xfb800108,0xbca50101,0xfb800108,0xbca60101,0xfb800108,0xbca70101,
	0xfb800108,0xbca80101,0xfb800108,0xbca90101,0xfb800108,0xbcaa0101,0xfb800108,0xbcab0101,
	0xfb800108,0xbcac0101,0xfb800108,0xbcad0101,0xfb800108,0xbcae0101,0xfb800108,0xbcaf0101,
	0xfb800108,0xbcb00101,0xfb800108,0xbcb10101,0xfb800108,0xbcb20101,0xfb800108,0xbcb30101,
	0xfb800108,0xbcb40101,0xfb800108,0xbcb50101,0xfb800108,0xbcb60101,0xfb800108,0xbcb70101,
	0xfb800108,0xbcb80101,0xfb800108,0xbcb90101,0xfb800108,0xbcba0101,0xfb800108,0xbcbb0101,
	0xfb800108,0xbcbc0101,0xfb800108,0xbcbd0101,0xfb800108,0xbcbe0101,0xfb800108,0xbcbf0101,
	0xfb800108,0xbcc00101,0xfb800108,0xbcc10101,0xfb800108,0xbcc20101,0xfb800108,0xbcc30101,
	0xfb800108,0xbcc40101,0xfb800108,0xbcc50101,0xfb800108,0xbcc60101,0xfb800108,0xbcc70101,
	0xfb800108,0xbcc80101,0xfb800108,0xbcc90101,0xfb800108,0xbcca0101,0xfb800108,0xbccb0101,
	0xfb800108,0xbccc0101,0xfb800108,0xbccd0101,0xfb800108,0xbcce0101,0xfb800108,0xbccf0101,
	0xfb800108,0xbcd00101,0xfb800108,0xbcd10101,0xfb800108,0xbcd20101,0xfb800108,0xbcd30101,
	0xfb800108,0xbcd40101,0xfb800108,0xbcd50101,0xfb800108,0xbcd60101,0xfb800108,0xbcd70101,
	0xfb800108,0xbcd80101,0xfb800108,0xbcd90101,0xfb800108,0xbcda0101,0xfb800108,0xbcdb0101,
	0xfb800108,0xbcdc0101,0xfb800108,0xbcdd0101,0xfb800108,0xbcde0101,0xfb800108,0xbcdf0101,
	0xfb800108,0xbce00101,0xfb800108,0xbce10101,0xfb800108,0xbce20101,0xfb800108,0xbce30101,
	0xfb800108,0xbce40101,0xfb800108,0xbce50101,0xfb800108,0xbce60101,0xfb800108,0xbce70101,
	0xfb800108,0xbce80101,0xfb800108,0xbce90101,0xfb800108,0xbcea0101,0xfb800108,0xbceb0101,
	0xfb800108,0xbcec0101,0xfb800108,0xbced0101,0xfb800108,0xbcee0101,0xfb800108,0xbcef0101,
	0xfb800108,0xbcf00101,0xfb800108,0xbcf10101,0xfb800108,0xbcf20101,0xfb800108,0xbcf30101,
	0xfb800108,0xbcf40101,0xfb800108,0xbcf50101,0xfb800108,0xbcf60101,0xfb800108,0xbcf70101,
	0xfb800108,0xbcf80101,0xfb800108,0xbcf90101,0xfb800108,0xbcfa0101,0xfb800108,0xbcfb0101,
	0xfb800108,0xbcfc0101,0xfb800108,0xbcfd0101,0xfb800108,0xbcfe0101,0xfb800108,0xbcff0101,
	0xfb800108,0xbd000101,0xfb800108,0xbd010101,0xfb800108,0xbd020101,0xfb800108,0xbd030101,
	0xfb800108,0xbd040101,0xfb800108,0xbd050101,0xfb800108,0xbd060101,0xfb800108,0xbd070101,
	0xfb800108,0xbd080101,0xfb800108,0xbd090101,0xfb800108,0xbd0a0101,0xfb800108,0xbd0b0101,
	0xfb800108,0xbd0c0101,0xfb800108,0xbd0d0101,0xfb800108,0xbd0e0101,0xfb800108,0xbd0f0101,
	0xfb800108,0xbd100101,0xfb800108,0xbd110101,0xfb800108,0xbd120101,0xfb800108,0xbd130101,
	0xfb800108,0xbd140101,0xfb800108,0xbd150101,0xfb800108,0xbd160101,0xfb800108,0xbd170101,
	0xfb800108,0xbd180101,0xfb800108,0xbd190101,0xfb800108,0xbd1a0101,0xfb800108,0xbd1b0101,
	0xfb800108,0xbd1c0101,0xfb800108,0xbd1d0101,0xfb800108,0xbd1e0101,0xfb800108,0xbd1f0101,
	0xfb800108,0xbd200101,0xfb800108,0xbd210101,0xfb800108,0xbd220101,0xfb800108,0xbd230101,
	0xfb800108,0xbd240101,0xfb800108,0xbd250101,0xfb800108,0xbd260101,0xfb800108,0xbd270101,
	0xfb800108,0xbd280101,0xfb800108,0xbd290101,0xfb800108,0xbd2a0101,0xfb800108,0xbd2b0101,
	0xfb800108,0xbd2c0101,0xfb800108,0xbd2d0101,0xfb800108,0xbd2e0101,0xfb800108,0xbd2f0101,
	0xfb800108,0xbd300101,0xfb800108,0xbd310101,0xfb800108,0xbd320101,0xfb800108,0xbd330101,
	0xfb800108,0xbd340101,0xfb800108,0xbd350101,0xfb800108,0xbd360101,0xfb800108,0xbd370101,
	0xfb800108,0xbd380101,0xfb800108,0xbd390101,0xfb800108,0xbd3a0101,0xfb800108,0xbd3b0101,
	0xfb800108,0xbd3c0101,0xfb800108,0xbd3d0101,0xfb800108,0xbd3e0101,0xfb800108,0xbd3f0101,
	0xfb800108,0xbd400101,0xfb800108,0xbd410101,0xfb800108,0xbd420101,0xfb800108,0xbd430101,
	0xfb800108,0xbd440101,0xfb800108,0xbd450101,0xfb800108,0xbd460101,0xfb800108,0xbd470101,
	0xfb800108,0xbd480101,0xfb800108,0xbd490101,0xfb800108,0xbd4a0101,0xfb800108,0xbd4b0101,
	0xfb800108,0xbd4c0101,0xfb800108,0xbd4d0101,0xfb800108,0xbd4e0101,0xfb800108,0xbd4f0101,
	0xfb800108,0xbd500101,0xfb800108,0xbd510101,0xfb800108,0xbd520101,0xfb800108,0xbd530101,
	0xfb800108,0xbd540101,0xfb800108,0xbd550101,0xfb800108,0xbd560101,0xfb800108,0xbd570101,
	0xfb800108,0xbd580101,0xfb800108,0xbd590101,0xfb800108,0xbd5a0101,0xfb800108,0xbd5b0101,
	0xfb800108,0xbd5c0101,0xfb800108,0xbd5d0101,0xfb800108,0xbd5e0101,0xfb800108,0xbd5f0101,
	0xfb800108,0xbd600101,0xfb800108,0xbd610101,0xfb800108,0xbd620101,0xfb800108,0xbd630101,
	0xfb800108,0xbd640101,0xfb800108,0xbd650101,0xfb800108,0xbd660101,0xfb800108,0xbd670101,
	0xfb800108,0xbd680101,0xfb800108,0xbd690101,0xfb800108,0xbd6a0101,0xfb800108,0xbd6b0101,
	0xfb800108,0xbd6c0101,0xfb800108,0xbd6d0101,0xfb800108,0xbd6e0101,0xfb800108,0xbd6f0101,
	0xfb800108,0xbd700101,0xfb800108,0xbd710101,0xfb800108,0xbd720101,0xfb800108,0xbd730101,
	0xfb800108,0xbd740101,0xfb800108,0xbd750101,0xfb800108,0xbd760101,0xfb800108,0xbd770101,
	0xfb800108,0xbd780101,0xfb800108,0xbd790101,0xfb800108,0xbd7a0101,0xfb800108,0xbd7b0101,
	0xfb800108,0xbd7c0101,0xfb800108,0xbd7d0101,0xfb800108,0xbd7e0101,0xfb800108,0xbd7f0101,
	0xfb800108,0xbd800101,0xfb800108,0xbd810101,0xfb800108,0xbd820101,0xfb800108,0xbd830101,
	0xfb800108,0xbd840101,0xfb800108,0xbd850101,0xfb800108,0xbd860101,0xfb800108,0xbd870101,
	0xfb800108,0xbd880101,0xfb800108,0xbd890101,0xfb800108,0xbd8a0101,0xfb800108,0xbd8b0101,
	0xfb800108,0xbd8c0101,0xfb800108,0xbd8d0101,0xfb800108,0xbd8e0101,0xfb800108,0xbd8f0101,
	0xfb800108,0xbd900101,0xfb800108,0xbd910101,0xfb800108,0xbd920101,0xfb800108,0xbd930101,
	0xfb800108,0xbd940101,0xfb800108,0xbd950101,0xfb800108,0xbd960101,0xfb800108,0xbd970101,
	0xfb800108,0xbd980101,0xfb800108,0xbd990101,0xfb800108,0xbd9a0101,0xfb800108,0xbd9b0101,
	0xfb800108,0xbd9c0101,0xfb800108,0xbd9d0101,0xfb800108,0xbd9e0101,0xfb800108,0xbd9f0101,
	0xfb800108,0xbda00101,0xfb800108,0xbda10101,0xfb800108,0xbda20101,0xfb800108,0xbda30101,
	0xfb800108,0xbda40101,0xfb800108,0xbda50101,0xfb800108,0xbda60101,0xfb800108,0xbda70101,
	0xfb800108,0xbda80101,0xfb800108,0xbda90101,0xfb800108,0xbdaa0101,0xfb800108,0xbdab0101,
	0xfb800108,0xbdac0101,0xfb800108,0xbdad0101,0xfb800108,0xbdae0101,0xfb800108,0xbdaf0101,
	0xfb800108,0xbdb00101,0xfb800108,0xbdb10101,0xfb800108,0xbdb20101,0xfb800108,0xbdb30101,
	0xfb800108,0xbdb40101,0xfb800108,0xbdb50101,0xfb800108,0xbdb60101,0xfb800108,0xbdb70101,
	0xfb800108,0xbdb80101,0xfb800108,0xbdb90101,0xfb800108,0xbdba0101,0xfb800108,0xbdbb0101,
	0xfb800108,0xbdbc0101,0xfb800108,0xbdbd0101,0xfb800108,0xbdbe0101,0xfb800108,0xbdbf0101,
	0xfb800108,0xbdc00101,0xfb800108,0xbdc10101,0xfb800108,0xbdc20101,0xfb800108,0xbdc30101,
	0xfb800108,0xbdc40101,0xfb800108,0xbdc50101,0xfb800108,0xbdc60101,0xfb800108,0xbdc70101,
	0xfb800108,0xbdc80101,0xfb800108,0xbdc90101,0xfb800108,0xbdca0101,0xfb800108,0xbdcb0101,
	0xfb800108,0xbdcc0101,0xfb800108,0xbdcd0101,0xfb800108,0xbdce0101,0xfb800108,0xbdcf0101,
	0xfb800108,0xbdd00101,0xfb800108,0xbdd10101,0xfb800108,0xbdd20101,0xfb800108,0xbdd30101,
	0xfb800108,0xbdd40101,0xfb800108,0xbdd50101,0xfb800108,0xbdd60101,0xfb800108,0xbdd70101,
	0xfb800108,0xbdd80101,0xfb800108,0xbdd90101,0xfb800108,0xbdda0101,0xfb800108,0xbddb0101,
	0xfb800108,0xbddc0101,0xfb800108,0xbddd0101,0xfb800108,0xbdde0101,0xfb800108,0xbddf0101,
	0xfb800108,0xbde00101,0xfb800108,0xbde10101,0xfb800108,0xbde20101,0xfb800108,0xbde30101,
	0xfb800108,0xbde40101,0xfb800108,0xbde50101,0xfb800108,0xbde60101,0xfb800108,0xbde70101,
	0xfb800108,0xbde80101,0xfb800108,0xbde90101,0xfb800108,0xbdea0101,0xfb800108,0xbdeb0101,
	0xfb800108,0xbdec0101,0xfb800108,0xbded0101,0xfb800108,0xbdee0101,0xfb800108,0xbdef0101,
	0xfb800108,0xbdf00101,0xfb800108,0xbdf10101,0xfb800108,0xbdf20101,0xfb800108,0xbdf30101,
	0xfb800108,0xbdf40101,0xfb800108,0xbdf50101,0xfb800108,0xbdf60101,0xfb800108,0xbdf70101,
	0xfb800108,0xbdf80101,0xfb800108,0xbdf90101,0xfb800108,0xbdfa0101,0xfb800108,0xbdfb0101,
	0xfb800108,0xbdfc0101,0xfb800108,0xbdfd0101,0xfb800108,0xbdfe0101,0xfb800108,0xbdff0101,
	0xfb800108,0xbe000101,0xfb800108,0xbe010101,0xfb800108,0xbe020101,0xfb800108,0xbe030101,
	0xfb800108,0xbe040101,0xfb800108,0xbe050101,0xfb800108,0xbe060101,0xfb800108,0xbe070101,
	0xfb800108,0xbe080101,0xfb800108,0xbe090101,0xfb800108,0xbe0a0101,0xfb800108,0xbe0b0101,
	0xfb800108,0xbe0c0101,0xfb800108,0xbe0d0101,0xfb800108,0xbe0e0101,0xfb800108,0xbe0f0101,
	0xfb800108,0xbe100101,0xfb800108,0xbe110101,0xfb800108,0xbe120101,0xfb800108,0xbe130101,
	0xfb800108,0xbe140101,0xfb800108,0xbe150101,0xfb800108,0xbe160101,0xfb800108,0xbe170101,
	0xfb800108,0xbe180101,0xfb800108,0xbe190101,0xfb800108,0xbe1a0101,0xfb800108,0xbe1b0101,
	0xfb800108,0xbe1c0101,0xfb800108,0xbe1d0101,0xfb800108,0xbe1e0101,0xfb800108,0xbe1f0101,
	0xfb800108,0xbe200101,0xfb800108,0xbe210101,0xfb800108,0xbe220101,0xfb800108,0xbe230101,
	0xfb800108,0xbe240101,0xfb800108,0xbe250101,0xfb800108,0xbe260101,0xfb800108,0xbe270101,
	0xfb800108,0xbe280101,0xfb800108,0xbe290101,0xfb800108,0xbe2a0101,0xfb800108,0xbe2b0101,
	0xfb800108,0xbe2c0101,0xfb800108,0xbe2d0101,0xfb800108,0xbe2e0101,0xfb800108,0xbe2f0101,
	0xfb800108,0xbe300101,0xfb800108,0xbe310101,0xfb800108,0xbe320101,0xfb800108,0xbe330101,
	0xfb800108,0xbe340101,0xfb800108,0xbe350101,0xfb800108,0xbe360101,0xfb800108,0xbe370101,
	0xfb800108,0xbe380101,0xfb800108,0xbe390101,0xfb800108,0xbe3a0101,0xfb800108,0xbe3b0101,
	0xfb800108,0xbe3c0101,0xfb800108,0xbe3d0101,0xfb800108,0xbe3e0101,0xfb800108,0xbe3f0101,
	0xfb800108,0xbe400101,0xfb800108,0xbe410101,0xfb800108,0xbe420101,0xfb800108,0xbe430101,
	0xfb800108,0xbe440101,0xfb800108,0xbe450101,0xfb800108,0xbe460101,0xfb800108,0xbe470101,
	0xfb800108,0xbe480101,0xfb800108,0xbe490101,0xfb800108,0xbe4a0101,0xfb800108,0xbe4b0101,
	0xfb800108,0xbe4c0101,0xfb800108,0xbe4d0101,0xfb800108,0xbe4e0101,0xfb800108,0xbe4f0101,
	0xfb800108,0xbe500101,0xfb800108,0xbe510101,0xfb800108,0xbe520101,0xfb800108,0xbe530101,
	0xfb800108,0xbe540101,0xfb800108,0xbe550101,0xfb800108,0xbe560101,0xfb800108,0xbe570101,
	0xfb800108,0xbe580101,0xfb800108,0xbe590101,0xfb800108,0xbe5a0101,0xfb800108,0xbe5b0101,
	0xfb800108,0xbe5c0101,0xfb800108,0xbe5d0101,0xfb800108,0xbe5e0101,0xfb800108,0xbe5f0101,
	0xfb800108,0xbe600101,0xfb800108,0xbe610101,0xfb800108,0xbe620101,0xfb800108,0xbe630101,
	0xfb800108,0xbe640101,0xfb800108,0xbe650101,0xfb800108,0xbe660101,0xfb800108,0xbe670101,
	0xfb800108,0xbe680101,0xfb800108,0xbe690101,0xfb800108,0xbe6a0101,0xfb800108,0xbe6b0101,
	0xfb800108,0xbe6c0101,0xfb800108,0xbe6d0101,0xfb800108,0xbe6e0101,0xfb800108,0xbe6f0101,
	0xfb800108,0xbe700101,0xfb800108,0xbe710101,0xfb800108,0xbe720101,0xfb800108,0xbe730101,
	0xfb800108,0xbe740101,0xfb800108,0xbe750101,0xfb800108,0xbe760101,0xfb800108,0xbe770101,
	0xfb800108,0xbe780101,0xfb800108,0xbe790101,0xfb800108,0xbe7a0101,0xfb800108,0xbe7b0101,
	0xfb800108,0xbe7c0101,0xfb800108,0xbe7d0101,0xfb800108,0xbe7e0101,0xfb800108,0xbe7f0101,
	0xfb800108,0xbe800101,0xfb800108,0xbe810101,0xfb800108,0xbe820101,0xfb800108,0xbe830101,
	0xfb800108,0xbe840101,0xfb800108,0xbe850101,0xfb800108,0xbe860101,0xfb800108,0xbe870101,
	0xfb800108,0xbe880101,0xfb800108,0xbe890101,0xfb800108,0xbe8a0101,0xfb800108,0xbe8b0101,
	0xfb800108,0xbe8c0101,0xfb800108,0xbe8d0101,0xfb800108,0xbe8e0101,0xfb800108,0xbe8f0101,
	0xfb800108,0xbe900101,0xfb800108,0xbe910101,0xfb800108,0xbe920101,0xfb800108,0xbe930101,
	0xfb800108,0xbe940101,0xfb800108,0xbe950101,0xfb800108,0xbe960101,0xfb800108,0xbe970101,
	0xfb800108,0xbe980101,0xfb800108,0xbe990101,0xfb800108,0xbe9a0101,0xfb800108,0xbe9b0101,
	0xfb800108,0xbe9c0101,0xfb800108,0xbe9d0101,0xfb800108,0xbe9e0101,0xfb800108,0xbe9f0101,
	0xfb800108,0xbea00101,0xfb800108,0xbea10101,0xfb800108,0xbea20101,0xfb800108,0xbea30101,
	0xfb800108,0xbea40101,0xfb800108,0xbea50101,0xfb800108,0xbea60101,0xfb800108,0xbea70101,
	0xfb800108,0xbea80101,0xfb800108,0xbea90101,0xfb800108,0xbeaa0101,0xfb800108,0xbeab0101,
	0xfb800108,0xbeac0101,0xfb800108,0xbead0101,0xfb800108,0xbeae0101,0xfb800108,0xbeaf0101,
	0xfb800108,0xbeb00101,0xfb800108,0xbeb10101,0xfb800108,0xbeb20101,0xfb800108,0xbeb30101,
	0xfb800108,0xbeb40101,0xfb800108,0xbeb50101,0xfb800108,0xbeb60101,0xfb800108,0xbeb70101,
	0xfb800108,0xbeb80101,0xfb800108,0xbeb90101,0xfb800108,0xbeba0101,0xfb800108,0xbebb0101,
	0xfb800108,0xbebc0101,0xfb800108,0xbebd0101,0xfb800108,0xbebe0101,0xfb800108,0xbebf0101,
	0xfb800108,0xbec00101,0xfb800108,0xbec10101,0xfb800108,0xbec20101,0xfb800108,0xbec30101,
	0xfb800108,0xbec40101,0xfb800108,0xbec50101,0xfb800108,0xbec60101,0xfb800108,0xbec70101,
	0xfb800108,0xbec80101,0xfb800108,0xbec90101,0xfb800108,0xbeca0101,0xfb800108,0xbecb0101,
	0xfb800108,0xbecc0101,0xfb800108,0xbecd0101,0xfb800108,0xbece0101,0xfb800108,0xbecf0101,
	0xfb800108,0xbed00101,0xfb800108,0xbed10101,0xfb800108,0xbed20101,0xfb800108,0xbed30101,
	0xfb800108,0xbed40101,0xfb800108,0xbed50101,0xfb800108,0xbed60101,0xfb800108,0xbed70101,
	0xfb800108,0xbed80101,0xfb800108,0xbed90101,0xfb800108,0xbeda0101,0xfb800108,0xbedb0101,
	0xfb800108,0xbedc0101,0xfb800108,0xbedd0101,0xfb800108,0xbede0101,0xfb800108,0xbedf0101,
	0xfb800108,0xbee00101,0xfb800108,0xbee10101,0xfb800108,0xbee20101,0xfb800108,0xbee30101,
	0xfb800108,0xbee40101,0xfb800108,0xbee50101,0xfb800108,0xbee60101,0xfb800108,0xbee70101,
	0xfb800108,0xbee80101,0xfb800108,0xbee90101,0xfb800108,0xbeea0101,0xfb800108,0xbeeb0101,
	0xfb800108,0xbeec0101,0xfb800108,0xbeed0101,0xfb800108,0xbeee0101,0xfb800108,0xbeef0101,
	0xfb800108,0xbef00101,0xfb800108,0xbef10101,0xfb800108,0xbef20101,0xfb800108,0xbef30101,
	0xfb800108,0xbef40101,0xfb800108,0xbef50101,0xfb800108,0xbef60101,0xfb800108,0xbef70101,
	0xfb800108,0xbef80101,0xfb800108,0xbef90101,0xfb800108,0xbefa0101,0xfb800108,0xbefb0101,
	0xfb800108,0xbefc0101,0xfb800108,0xbefd0101,0xfb800108,0xbefe0101,0xfb800108,0xbeff0101,
	0xfb800108,0xbf000101,0xfb800108,0xbf010101,0xfb800108,0xbf020101,0xfb800108,0xbf030101,
	0xfb800108,0xbf040101,0xfb800108,0xbf050101,0xfb800108,0xbf060101,0xfb800108,0xbf070101,
	0xfb800108,0xbf080101,0xfb800108,0xbf090101,0xfb800108,0xbf0a0101,0xfb800108,0xbf0b0101,
	0xfb800108,0xbf0c0101,0xfb800108,0xbf0d0101,0xfb800108,0xbf0e0101,0xfb800108,0xbf0f0101,
	0xfb800108,0xbf100101,0xfb800108,0xbf110101,0xfb800108,0xbf120101,0xfb800108,0xbf130101,
	0xfb800108,0xbf140101,0xfb800108,0xbf150101,0xfb800108,0xbf160101,0xfb800108,0xbf170101,
	0xfb800108,0xbf180101,0xfb800108,0xbf190101,0xfb800108,0xbf1a0101,0xfb800108,0xbf1b0101,
	0xfb800108,0xbf1c0101,0xfb800108,0xbf1d0101,0xfb800108,0xbf1e0101,0xfb800108,0xbf1f0101,
	0xfb800108,0xbf200101,0xfb800108,0xbf210101,0xfb800108,0xbf220101,0xfb800108,0xbf230101,
	0xfb800108,0xbf240101,0xfb800108,0xbf250101,0xfb800108,0xbf260101,0xfb800108,0xbf270101,
	0xfb800108,0xbf280101,0xfb800108,0xbf290101,0xfb800108,0xbf2a0101,0xfb800108,0xbf2b0101,
	0xfb800108,0xbf2c0101,0xfb800108,0xbf2d0101,0xfb800108,0xbf2e0101,0xfb800108,0xbf2f0101,
	0xfb800108,0xbf300101,0xfb800108,0xbf310101,0xfb800108,0xbf320101,0xfb800108,0xbf330101,
	0xfb800108,0xbf340101,0xfb800108,0xbf350101,0xfb800108,0xbf360101,0xfb800108,0xbf370101,
	0xfb800108,0xbf380101,0xfb800108,0xbf390101,0xfb800108,0xbf3a0101,0xfb800108,0xbf3b0101,
	0xfb800108,0xbf3c0101,0xfb800108,0xbf3d0101,0xfb800108,0xbf3e0101,0xfb800108,0xbf3f0101,
	0xfb800108,0xbf400101,0xfb800108,0xbf410101,0xfb800108,0xbf420101,0xfb800108,0xbf430101,
	0xfb800108,0xbf440101,0xfb800108,0xbf450101,0xfb800108,0xbf460101,0xfb800108,0xbf470101,
	0xfb800108,0xbf480101,0xfb800108,0xbf490101,0xfb800108,0xbf4a0101,0xfb800108,0xbf4b0101,
	0xfb800108,0xbf4c0101,0xfb800108,0xbf4d0101,0xfb800108,0xbf4e0101,0xfb800108,0xbf4f0101,
	0xfb800108,0xbf500101,0xfb800108,0xbf510101,0xfb800108,0xbf520101,0xfb800108,0xbf530101,
	0xfb800108,0xbf540101,0xfb800108,0xbf550101,0xfb800108,0xbf560101,0xfb800108,0xbf570101,
	0xfb800108,0xbf580101,0xfb800108,0xbf590101,0xfb800108,0xbf5a0101,0xfb800108,0xbf5b0101,
	0xfb800108,0xbf5c0101,0xfb800108,0xbf5d0101,0xfb800108,0xbf5e0101,0xfb800108,0xbf5f0101,
	0xfb800108,0xbf600101,0xfb800108,0xbf610101,0xfb800108,0xbf620101,0xfb800108,0xbf630101,
	0xfb800108,0xbf640101,0xfb800108,0xbf650101,0xfb800108,0xbf660101,0xfb800108,0xbf670101,
	0xfb800108,0xbf680101,0xfb800108,0xbf690101,0xfb800108,0xbf6a0101,0xfb800108,0xbf6b0101,
	0xfb800108,0xbf6c0101,0xfb800108,0xbf6d0101,0xfb800108,0xbf6e0101,0xfb800108,0xbf6f0101,
	0xfb800108,0xbf700101,0xfb800108,0xbf710101,0xfb800108,0xbf720101,0xfb800108,0xbf730101,
	0xfb800108,0xbf740101,0xfb800108,0xbf750101,0xfb800108,0xbf760101,0xfb800108,0xbf770101,
	0xfb800108,0xbf780101,0xfb800108,0xbf790101,0xfb800108,0xbf7a0101,0xfb800108,0xbf7b0101,
	0xfb800108,0xbf7c0101,0xfb800108,0xbf7d0101,0xfb800108,0xbf7e0101,0xfb800108,0xbf7f0101,
	0xfb800108,0xbf800101,0xfb800108,0xbf810101,0xfb800108,0xbf820101,0xfb800108,0xbf830101,
	0xfb800108,0xbf840101,0xfb800108,0xbf850101,0xfb800108,0xbf860101,0xfb800108,0xbf870101,
	0xfb800108,0xbf880101,0xfb800108,0xbf890101,0xfb800108,0xbf8a0101,0xfb800108,0xbf8b0101,
	0xfb800108,0xbf8c0101,0xfb800108,0xbf8d0101,0xfb800108,0xbf8e0101,0xfb800108,0xbf8f0101,
	0xfb800108,0xbf900101,0xfb800108,0xbf910101,0xfb800108,0xbf920101,0xfb800108,0xbf930101,
	0xfb800108,0xbf940101,0xfb800108,0xbf950101,0xfb800108,0xbf960101,0xfb800108,0xbf970101,
	0xfb800108,0xbf980101,0xfb800108,0xbf990101,0xfb800108,0xbf9a0101,0xfb800108,0xbf9b0101,
	0xfb800108,0xbf9c0101,0xfb800108,0xbf9d0101,0xfb800108,0xbf9e0101,0xfb800108,0xbf9f0101,
	0xfb800108,0xbfa00101,0xfb800108,0xbfa10101,0xfb800108,0xbfa20101,0xfb800108,0xbfa30101,
	0xfb800108,0xbfa40101,0xfb800108,0xbfa50101,0xfb800108,0xbfa60101,0xfb800108,0xbfa70101,
	0xfb800108,0xbfa80101,0xfb800108,0xbfa90101,0xfb800108,0xbfaa0101,0xfb800108,0xbfab0101,
	0xfb800108,0xbfac0101,0xfb800108,0xbfad0101,0xfb800108,0xbfae0101,0xfb800108,0xbfaf0101,
	0xfb800108,0xbfb00101,0xfb800108,0xbfb10101,0xfb800108,0xbfb20101,0xfb800108,0xbfb30101,
	0xfb800108,0xbfb40101,0xfb800108,0xbfb50101,0xfb800108,0xbfb60101,0xfb800108,0xbfb70101,
	0xfb800108,0xbfb80101,0xfb800108,0xbfb90101,0xfb800108,0xbfba0101,0xfb800108,0xbfbb0101,
	0xfb800108,0xbfbc0101,0xfb800108,0xbfbd0101,0xfb800108,0xbfbe0101,0xfb800108,0xbfbf0101,
	0xfb800108,0xbfc00101,0xfb800108,0xbfc10101,0xfb800108,0xbfc20101,0xfb800108,0xbfc30101,
	0xfb800108,0xbfc40101,0xfb800108,0xbfc50101,0xfb800108,0xbfc60101,0xfb800108,0xbfc70101,
	0xfb800108,0xbfc80101,0xfb800108,0xbfc90101,0xfb800108,0xbfca0101,0xfb800108,0xbfcb0101,
	0xfb800108,0xbfcc0101,0xfb800108,0xbfcd0101,0xfb800108,0xbfce0101,0xfb800108,0xbfcf0101,
	0xfb800108,0xbfd00101,0xfb800108,0xbfd10101,0xfb800108,0xbfd20101,0xfb800108,0xbfd30101,
	0xfb800108,0xbfd40101,0xfb800108,0xbfd50101,0xfb800108,0xbfd60101,0xfb800108,0xbfd70101,
	0xfb800108,0xbfd80101,0xfb800108,0xbfd90101,0xfb800108,0xbfda0101,0xfb800108,0xbfdb0101,
	0xfb800108,0xbfdc0101,0xfb800108,0xbfdd0101,0xfb800108,0xbfde0101,0xfb800108,0xbfdf0101,
	0xfb800108,0xbfe00101,0xfb800108,0xbfe10101,0xfb800108,0xbfe20101,0xfb800108,0xbfe30101,
	0xfb800108,0xbfe40101,0xfb800108,0xbfe50101,0xfb800108,0xbfe60101,0xfb800108,0xbfe70101,
	0xfb800108,0xbfe80101,0xfb800108,0xbfe90101,0xfb800108,0xbfea0101,0xfb800108,0xbfeb0101,
	0xfb800108,0xbfec0101,0xfb800108,0xbfed0101,0xfb800108,0xbfee0101,0xfb800108,0xbfef0101,
	0xfb800108,0xbff00101,0xfb800108,0xbff10101,0xfb800108,0xbff20101,0xfb800108,0xbff30101,
	0xfb800108,0xbff40101,0xfb800108,0xbff50101,0xfb800108,0xbff60101,0xfb800108,0xbff70101,
	0xfb800108,0xbff80101,0xfb800108,0xbff90101,0xfb800108,0xbffa0101,0xfb800108,0xbffb0101,
	0xfb800108,0xbffc0101,0xfb800108,0xbffd0101,0xfb800108,0xbffe0101,0xfb800108,0xbfff0101,
	0xfb800108,0xc0000101,0xfb800108,0xc0010101,0xfb800108,0xc0020101,0xfb800108,0xc0030101,
	0xfb800108,0xc0040101,0xfb800108,0xc0050101,0xfb800108,0xc0060101,0xfb800108,0xc0070101,
	0xfb800108,0xc0080101,0xfb800108,0xc0090101,0xfb800108,0xc00a0101,0xfb800108,0xc00b0101,
	0xfb800108,0xc00c0101,0xfb800108,0xc00d0101,0xfb800108,0xc00e0101,0xfb800108,0xc00f0101,
	0xfb800108,0xc0100101,0xfb800108,0xc0110101,0xfb800108,0xc0120101,0xfb800108,0xc0130101,
	0xfb800108,0xc0140101,0xfb800108,0xc0150101,0xfb800108,0xc0160101,0xfb800108,0xc0170101,
	0xfb800108,0xc0180101,0xfb800108,0xc0190101,0xfb800108,0xc01a0101,0xfb800108,0xc01b0101,
	0xfb800108,0xc01c0101,0xfb800108,0xc01d0101,0xfb800108,0xc01e0101,0xfb800108,0xc01f0101,
	0xfb800108,0xc0200101,0xfb800108,0xc0210101,0xfb800108,0xc0220101,0xfb800108,0xc0230101,
	0xfb800108,0xc0240101,0xfb800108,0xc0250101,0xfb800108,0xc0260101,0xfb800108,0xc0270101,
	0xfb800108,0xc0280101,0xfb800108,0xc0290101,0xfb800108,0xc02a0101,0xfb800108,0xc02b0101,
	0xfb800108,0xc02c0101,0xfb800108,0xc02d0101,0xfb800108,0xc02e0101,0xfb800108,0xc02f0101,
	0xfb800108,0xc0300101,0xfb800108,0xc0310101,0xfb800108,0xc0320101,0xfb800108,0xc0330101,
	0xfb800108,0xc0340101,0xfb800108,0xc0350101,0xfb800108,0xc0360101,0xfb800108,0xc0370101,
	0xfb800108,0xc0380101,0xfb800108,0xc0390101,0xfb800108,0xc03a0101,0xfb800108,0xc03b0101,
	0xfb800108,0xc03c0101,0xfb800108,0xc03d0101,0xfb800108,0xc03e0101,0xfb800108,0xc03f0101,
	0xfb800108,0xc0400101,0xfb800108,0xc0410101,0xfb800108,0xc0420101,0xfb800108,0xc0430101,
	0xfb800108,0xc0440101,0xfb800108,0xc0450101,0xfb800108,0xc0460101,0xfb800108,0xc0470101,
	0xfb800108,0xc0480101,0xfb800108,0xc0490101,0xfb800108,0xc04a0101,0xfb800108,0xc04b0101,
	0xfb800108,0xc04c0101,0xfb800108,0xc04d0101,0xfb800108,0xc04e0101,0xfb800108,0xc04f0101,
	0xfb800108,0xc0500101,0xfb800108,0xc0510101,0xfb800108,0xc0520101,0xfb800108,0xc0530101,
	0xfb800108,0xc0540101,0xfb800108,0xc0550101,0xfb800108,0xc0560101,0xfb800108,0xc0570101,
	0xfb800108,0xc0580101,0xfb800108,0xc0590101,0xfb800108,0xc05a0101,0xfb800108,0xc05b0101,
	0xfb800108,0xc05c0101,0xfb800108,0xc05d0101,0xfb800108,0xc05e0101,0xfb800108,0xc05f0101,
	0xfb800108,0xc0600101,0xfb800108,0xc0610101,0xfb800108,0xc0620101,0xfb800108,0xc0630101,
	0xfb800108,0xc0640101,0xfb800108,0xc0650101,0xfb800108,0xc0660101,0xfb800108,0xc0670101,
	0xfb800108,0xc0680101,0xfb800108,0xc0690101,0xfb800108,0xc06a0101,0xfb800108,0xc06b0101,
	0xfb800108,0xc06c0101,0xfb800108,0xc06d0101,0xfb800108,0xc06e0101,0xfb800108,0xc06f0101,
	0xfb800108,0xc0700101,0xfb800108,0xc0710101,0xfb800108,0xc0720101,0xfb800108,0xc0730101,
	0xfb800108,0xc0740101,0xfb800108,0xc0750101,0xfb800108,0xc0760101,0xfb800108,0xc0770101,
	0xfb800108,0xc0780101,0xfb800108,0xc0790101,0xfb800108,0xc07a0101,0xfb800108,0xc07b0101,
	0xfb800108,0xc07c0101,0xfb800108,0xc07d0101,0xfb800108,0xc07e0101,0xfb800108,0xc07f0101,
	0xfb800108,0xc0800101,0xfb800108,0xc0810101,0xfb800108,0xc0820101,0xfb800108,0xc0830101,
	0xfb800108,0xc0840101,0xfb800108,0xc0850101,0xfb800108,0xc0860101,0xfb800108,0xc0870101,
	0xfb800108,0xc0880101,0xfb800108,0xc0890101,0xfb800108,0xc08a0101,0xfb800108,0xc08b0101,
	0xfb800108,0xc08c0101,0xfb800108,0xc08d0101,0xfb800108,0xc08e0101,0xfb800108,0xc08f0101,
	0xfb800108,0xc0900101,0xfb800108,0xc0910101,0xfb800108,0xc0920101,0xfb800108,0xc0930101,
	0xfb800108,0xc0940101,0xfb800108,0xc0950101,0xfb800108,0xc0960101,0xfb800108,0xc0970101,
	0xfb800108,0xc0980101,0xfb800108,0xc0990101,0xfb800108,0xc09a0101,0xfb800108,0xc09b0101,
	0xfb800108,0xc09c0101,0xfb800108,0xc09d0101,0xfb800108,0xc09e0101,0xfb800108,0xc09f0101,
	0xfb800108,0xc0a00101,0xfb800108,0xc0a10101,0xfb800108,0xc0a20101,0xfb800108,0xc0a30101,
	0xfb800108,0xc0a40101,0xfb800108,0xc0a50101,0xfb800108,0xc0a60101,0xfb800108,0xc0a70101,
	0xfb800108,0xc0a80101,0xfb800108,0xc0a90101,0xfb800108,0xc0aa0101,0xfb800108,0xc0ab0101,
	0xfb800108,0xc0ac0101,0xfb800108,0xc0ad0101,0xfb800108,0xc0ae0101,0xfb800108,0xc0af0101,
	0xfb800108,0xc0b00101,0xfb800108,0xc0b10101,0xfb800108,0xc0b20101,0xfb800108,0xc0b30101,
	0xfb800108,0xc0b40101,0xfb800108,0xc0b50101,0xfb800108,0xc0b60101,0xfb800108,0xc0b70101,
	0xfb800108,0xc0b80101,0xfb800108,0xc0b90101,0xfb800108,0xc0ba0101,0xfb800108,0xc0bb0101,
	0xfb800108,0xc0bc0101,0xfb800108,0xc0bd0101,0xfb800108,0xc0be0101,0xfb800108,0xc0bf0101,
	0xfb800108,0xc0c00101,0xfb800108,0xc0c10101,0xfb800108,0xc0c20101,0xfb800108,0xc0c30101,
	0xfb800108,0xc0c40101,0xfb800108,0xc0c50101,0xfb800108,0xc0c60101,0xfb800108,0xc0c70101,
	0xfb800108,0xc0c80101,0xfb800108,0xc0c90101,0xfb800108,0xc0ca0101,0xfb800108,0xc0cb0101,
	0xfb800108,0xc0cc0101,0xfb800108,0xc0cd0101,0xfb800108,0xc0ce0101,0xfb800108,0xc0cf0101,
	0xfb800108,0xc0d00101,0xfb800108,0xc0d10101,0xfb800108,0xc0d20101,0xfb800108,0xc0d30101,
	0xfb800108,0xc0d40101,0xfb800108,0xc0d50101,0xfb800108,0xc0d60101,0xfb800108,0xc0d70101,
	0xfb800108,0xc0d80101,0xfb800108,0xc0d90101,0xfb800108,0xc0da0101,0xfb800108,0xc0db0101,
	0xfb800108,0xc0dc0101,0xfb800108,0xc0dd0101,0xfb800108,0xc0de0101,0xfb800108,0xc0df0101,
	0xfb800108,0xc0e00101,0xfb800108,0xc0e10101,0xfb800108,0xc0e20101,0xfb800108,0xc0e30101,
	0xfb800108,0xc0e40101,0xfb800108,0xc0e50101,0xfb800108,0xc0e60101,0xfb800108,0xc0e70101,
	0xfb800108,0xc0e80101,0xfb800108,0xc0e90101,0xfb800108,0xc0ea0101,0xfb800108,0xc0eb0101,
	0xfb800108,0xc0ec0101,0xfb800108,0xc0ed0101,0xfb800108,0xc0ee0101,0xfb800108,0xc0ef0101,
	0xfb800108,0xc0f00101,0xfb800108,0xc0f10101,0xfb800108,0xc0f20101,0xfb800108,0xc0f30101,
	0xfb800108,0xc0f40101,0xfb800108,0xc0f50101,0xfb800108,0xc0f60101,0xfb800108,0xc0f70101,
	0xfb800108,0xc0f80101,0xfb800108,0xc0f90101,0xfb800108,0xc0fa0101,0xfb800108,0xc0fb0101,
	0xfb800108,0xc0fc0101,0xfb800108,0xc0fd0101,0xfb800108,0xc0fe0101,0xfb800108,0xc0ff0101,
	0xfb800108,0xc1000101,0xfb800108,0xc1010101,0xfb800108,0xc1020101,0xfb800108,0xc1030101,
	0xfb800108,0xc1040101,0xfb800108,0xc1050101,0xfb800108,0xc1060101,0xfb800108,0xc1070101,
	0xfb800108,0xc1080101,0xfb800108,0xc1090101,0xfb800108,0xc10a0101,0xfb800108,0xc10b0101,
	0xfb800108,0xc10c0101,0xfb800108,0xc10d0101,0xfb800108,0xc10e0101,0xfb800108,0xc10f0101,
	0xfb800108,0xc1100101,0xfb800108,0xc1110101,0xfb800108,0xc1120101,0xfb800108,0xc1130101,
	0xfb800108,0xc1140101,0xfb800108,0xc1150101,0xfb800108,0xc1160101,0xfb800108,0xc1170101,
	0xfb800108,0xc1180101,0xfb800108,0xc1190101,0xfb800108,0xc11a0101,0xfb800108,0xc11b0101,
	0xfb800108,0xc11c0101,0xfb800108,0xc11d0101,0xfb800108,0xc11e0101,0xfb800108,0xc11f0101,
	0xfb800108,0xc1200101,0xfb800108,0xc1210101,0xfb800108,0xc1220101,0xfb800108,0xc1230101,
	0xfb800108,0xc1240101,0xfb800108,0xc1250101,0xfb800108,0xc1260101,0xfb800108,0xc1270101,
	0xfb800108,0xc1280101,0xfb800108,0xc1290101,0xfb800108,0xc12a0101,0xfb800108,0xc12b0101,
	0xfb800108,0xc12c0101,0xfb800108,0xc12d0101,0xfb800108,0xc12e0101,0xfb800108,0xc12f0101,
	0xfb800108,0xc1300101,0xfb800108,0xc1310101,0xfb800108,0xc1320101,0xfb800108,0xc1330101,
	0xfb800108,0xc1340101,0xfb800108,0xc1350101,0xfb800108,0xc1360101,0xfb800108,0xc1370101,
	0xfb800108,0xc1380101,0xfb800108,0xc1390101,0xfb800108,0xc13a0101,0xfb800108,0xc13b0101,
	0xfb800108,0xc13c0101,0xfb800108,0xc13d0101,0xfb800108,0xc13e0101,0xfb800108,0xc13f0101,
	0xfb800108,0xc1400101,0xfb800108,0xc1410101,0xfb800108,0xc1420101,0xfb800108,0xc1430101,
	0xfb800108,0xc1440101,0xfb800108,0xc1450101,0xfb800108,0xc1460101,0xfb800108,0xc1470101,
	0xfb800108,0xc1480101,0xfb800108,0xc1490101,0xfb800108,0xc14a0101,0xfb800108,0xc14b0101,
	0xfb800108,0xc14c0101,0xfb800108,0xc14d0101,0xfb800108,0xc14e0101,0xfb800108,0xc14f0101,
	0xfb800108,0xc1500101,0xfb800108,0xc1510101,0xfb800108,0xc1520101,0xfb800108,0xc1530101,
	0xfb800108,0xc1540101,0xfb800108,0xc1550101,0xfb800108,0xc1560101,0xfb800108,0xc1570101,
	0xfb800108,0xc1580101,0xfb800108,0xc1590101,0xfb800108,0xc15a0101,0xfb800108,0xc15b0101,
	0xfb800108,0xc15c0101,0xfb800108,0xc15d0101,0xfb800108,0xc15e0101,0xfb800108,0xc15f0101,
	0xfb800108,0xc1600101,0xfb800108,0xc1610101,0xfb800108,0xc1620101,0xfb800108,0xc1630101,
	0xfb800108,0xc1640101,0xfb800108,0xc1650101,0xfb800108,0xc1660101,0xfb800108,0xc1670101,
	0xfb800108,0xc1680101,0xfb800108,0xc1690101,0xfb800108,0xc16a0101,0xfb800108,0xc16b0101,
	0xfb800108,0xc16c0101,0xfb800108,0xc16d0101,0xfb800108,0xc16e0101,0xfb800108,0xc16f0101,
	0xfb800108,0xc1700101,0xfb800108,0xc1710101,0xfb800108,0xc1720101,0xfb800108,0xc1730101,
	0xfb800108,0xc1740101,0xfb800108,0xc1750101,0xfb800108,0xc1760101,0xfb800108,0xc1770101,
	0xfb800108,0xc1780101,0xfb800108,0xc1790101,0xfb800108,0xc17a0101,0xfb800108,0xc17b0101,
	0xfb800108,0xc17c0101,0xfb800108,0xc17d0101,0xfb800108,0xc17e0101,0xfb800108,0xc17f0101,
	0xfb800108,0xc1800101,0xfb800108,0xc1810101,0xfb800108,0xc1820101,0xfb800108,0xc1830101,
	0xfb800108,0xc1840101,0xfb800108,0xc1850101,0xfb800108,0xc1860101,0xfb800108,0xc1870101,
	0xfb800108,0xc1880101,0xfb800108,0xc1890101,0xfb800108,0xc18a0101,0xfb800108,0xc18b0101,
	0xfb800108,0xc18c0101,0xfb800108,0xc18d0101,0xfb800108,0xc18e0101,0xfb800108,0xc18f0101,
	0xfb800108,0xc1900101,0xfb800108,0xc1910101,0xfb800108,0xc1920101,0xfb800108,0xc1930101,
	0xfb800108,0xc1940101,0xfb800108,0xc1950101,0xfb800108,0xc1960101,0xfb800108,0xc1970101,
	0xfb800108,0xc1980101,0xfb800108,0xc1990101,0xfb800108,0xc19a0101,0xfb800108,0xc19b0101,
	0xfb800108,0xc19c0101,0xfb800108,0xc19d0101,0xfb800108,0xc19e0101,0xfb800108,0xc19f0101,
	0xfb800108,0xc1a00101,0xfb800108,0xc1a10101,0xfb800108,0xc1a20101,0xfb800108,0xc1a30101,
	0xfb800108,0xc1a40101,0xfb800108,0xc1a50101,0xfb800108,0xc1a60101,0xfb800108,0xc1a70101,
	0xfb800108,0xc1a80101,0xfb800108,0xc1a90101,0xfb800108,0xc1aa0101,0xfb800108,0xc1ab0101,
	0xfb800108,0xc1ac0101,0xfb800108,0xc1ad0101,0xfb800108,0xc1ae0101,0xfb800108,0xc1af0101,
	0xfb800108,0xc1b00101,0xfb800108,0xc1b10101,0xfb800108,0xc1b20101,0xfb800108,0xc1b30101,
	0xfb800108,0xc1b40101,0xfb800108,0xc1b50101,0xfb800108,0xc1b60101,0xfb800108,0xc1b70101,
	0xfb800108,0xc1b80101,0xfb800108,0xc1b90101,0xfb800108,0xc1ba0101,0xfb800108,0xc1bb0101,
	0xfb800108,0xc1bc0101,0xfb800108,0xc1bd0101,0xfb800108,0xc1be0101,0xfb800108,0xc1bf0101,
	0xfb800108,0xc1c00101,0xfb800108,0xc1c10101,0xfb800108,0xc1c20101,0xfb800108,0xc1c30101,
	0xfb800108,0xc1c40101,0xfb800108,0xc1c50101,0xfb800108,0xc1c60101,0xfb800108,0xc1c70101,
	0xfb800108,0xc1c80101,0xfb800108,0xc1c90101,0xfb800108,0xc1ca0101,0xfb800108,0xc1cb0101,
	0xfb800108,0xc1cc0101,0xfb800108,0xc1cd0101,0xfb800108,0xc1ce0101,0xfb800108,0xc1cf0101,
	0xfb800108,0xc1d00101,0xfb800108,0xc1d10101,0xfb800108,0xc1d20101,0xfb800108,0xc1d30101,
	0xfb800108,0xc1d40101,0xfb800108,0xc1d50101,0xfb800108,0xc1d60101,0xfb800108,0xc1d70101,
	0xfb800108,0xc1d80101,0xfb800108,0xc1d90101,0xfb800108,0xc1da0101,0xfb800108,0xc1db0101,
	0xfb800108,0xc1dc0101,0xfb800108,0xc1dd0101,0xfb800108,0xc1de0101,0xfb800108,0xc1df0101,
	0xfb800108,0xc1e00101,0xfb800108,0xc1e10101,0xfb800108,0xc1e20101,0xfb800108,0xc1e30101,
	0xfb800108,0xc1e40101,0xfb800108,0xc1e50101,0xfb800108,0xc1e60101,0xfb800108,0xc1e70101,
	0xfb800108,0xc1e80101,0xfb800108,0xc1e90101,0xfb800108,0xc1ea0101,0xfb800108,0xc1eb0101,
	0xfb800108,0xc1ec0101,0xfb800108,0xc1ed0101,0xfb800108,0xc1ee0101,0xfb800108,0xc1ef0101,
	0xfb800108,0xc1f00101,0xfb800108,0xc1f10101,0xfb800108,0xc1f20101,0xfb800108,0xc1f30101,
	0xfb800108,0xc1f40101,0xfb800108,0xc1f50101,0xfb800108,0xc1f60101,0xfb800108,0xc1f70101,
	0xfb800108,0xc1f80101,0xfb800108,0xc1f90101,0xfb800108,0xc1fa0101,0xfb800108,0xc1fb0101,
	0xfb800108,0xc1fc0101,0xfb800108,0xc1fd0101,0xfb800108,0xc1fe0101,0xfb800108,0xc1ff0101,
	0xfb800108,0xc2000101,0xfb800108,0xc2010101,0xfb800108,0xc2020101,0xfb800108,0xc2030101,
	0xfb800108,0xc2040101,0xfb800108,0xc2050101,0xfb800108,0xc2060101,0xfb800108,0xc2070101,
	0xfb800108,0xc2080101,0xfb800108,0xc2090101,0xfb800108,0xc20a0101,0xfb800108,0xc20b0101,
	0xfb800108,0xc20c0101,0xfb800108,0xc20d0101,0xfb800108,0xc20e0101,0xfb800108,0xc20f0101,
	0xfb800108,0xc2100101,0xfb800108,0xc2110101,0xfb800108,0xc2120101,0xfb800108,0xc2130101,
	0xfb800108,0xc2140101,0xfb800108,0xc2150101,0xfb800108,0xc2160101,0xfb800108,0xc2170101,
	0xfb800108,0xc2180101,0xfb800108,0xc2190101,0xfb800108,0xc21a0101,0xfb800108,0xc21b0101,
	0xfb800108,0xc21c0101,0xfb800108,0xc21d0101,0xfb800108,0xc21e0101,0xfb800108,0xc21f0101,
	0xfb800108,0xc2200101,0xfb800108,0xc2210101,0xfb800108,0xc2220101,0xfb800108,0xc2230101,
	0xfb800108,0xc2240101,0xfb800108,0xc2250101,0xfb800108,0xc2260101,0xfb800108,0xc2270101,
	0xfb800108,0xc2280101,0xfb800108,0xc2290101,0xfb800108,0xc22a0101,0xfb800108,0xc22b0101,
	0xfb800108,0xc22c0101,0xfb800108,0xc22d0101,0xfb800108,0xc22e0101,0xfb800108,0xc22f0101,
	0xfb800108,0xc2300101,0xfb800108,0xc2310101,0xfb800108,0xc2320101,0xfb800108,0xc2330101,
	0xfb800108,0xc2340101,0xfb800108,0xc2350101,0xfb800108,0xc2360101,0xfb800108,0xc2370101,
	0xfb800108,0xc2380101,0xfb800108,0xc2390101,0xfb800108,0xc23a0101,0xfb800108,0xc23b0101,
	0xfb800108,0xc23c0101,0xfb800108,0xc23d0101,0xfb800108,0xc23e0101,0xfb800108,0xc23f0101,
	0xfb800108,0xc2400101,0xfb800108,0xc2410101,0xfb800108,0xc2420101,0xfb800108,0xc2430101,
	0xfb800108,0xc2440101,0xfb800108,0xc2450101,0xfb800108,0xc2460101,0xfb800108,0xc2470101,
	0xfb800108,0xc2480101,0xfb800108,0xc2490101,0xfb800108,0xc24a0101,0xfb800108,0xc24b0101,
	0xfb800108,0xc24c0101,0xfb800108,0xc24d0101,0xfb800108,0xc24e0101,0xfb800108,0xc24f0101,
	0xfb800108,0xc2500101,0xfb800108,0xc2510101,0xfb800108,0xc2520101,0xfb800108,0xc2530101,
	0xfb800108,0xc2540101,0xfb800108,0xc2550101,0xfb800108,0xc2560101,0xfb800108,0xc2570101,
	0xfb800108,0xc2580101,0xfb800108,0xc2590101,0xfb800108,0xc25a0101,0xfb800108,0xc25b0101,
	0xfb800108,0xc25c0101,0xfb800108,0xc25d0101,0xfb800108,0xc25e0101,0xfb800108,0xc25f0101,
	0xfb800108,0xc2600101,0xfb800108,0xc2610101,0xfb800108,0xc2620101,0xfb800108,0xc2630101,
	0xfb800108,0xc2640101,0xfb800108,0xc2650101,0xfb800108,0xc2660101,0xfb800108,0xc2670101,
	0xfb800108,0xc2680101,0xfb800108,0xc2690101,0xfb800108,0xc26a0101,0xfb800108,0xc26b0101,
	0xfb800108,0xc26c0101,0xfb800108,0xc26d0101,0xfb800108,0xc26e0101,0xfb800108,0xc26f0101,
	0xfb800108,0xc2700101,0xfb800108,0xc2710101,0xfb800108,0xc2720101,0xfb800108,0xc2730101,
	0xfb800108,0xc2740101,0xfb800108,0xc2750101,0xfb800108,0xc2760101,0xfb800108,0xc2770101,
	0xfb800108,0xc2780101,0xfb800108,0xc2790101,0xfb800108,0xc27a0101,0xfb800108,0xc27b0101,
	0xfb800108,0xc27c0101,0xfb800108,0xc27d0101,0xfb800108,0xc27e0101,0xfb800108,0xc27f0101,
	0xfb800108,0xc2800101,0xfb800108,0xc2810101,0xfb800108,0xc2820101,0xfb800108,0xc2830101,
	0xfb800108,0xc2840101,0xfb800108,0xc2850101,0xfb800108,0xc2860101,0xfb800108,0xc2870101,
	0xfb800108,0xc2880101,0xfb800108,0xc2890101,0xfb800108,0xc28a0101,0xfb800108,0xc28b0101,
	0xfb800108,0xc28c0101,0xfb800108,0xc28d0101,0xfb800108,0xc28e0101,0xfb800108,0xc28f0101,
	0xfb800108,0xc2900101,0xfb800108,0xc2910101,0xfb800108,0xc2920101,0xfb800108,0xc2930101,
	0xfb800108,0xc2940101,0xfb800108,0xc2950101,0xfb800108,0xc2960101,0xfb800108,0xc2970101,
	0xfb800108,0xc2980101,0xfb800108,0xc2990101,0xfb800108,0xc29a0101,0xfb800108,0xc29b0101,
	0xfb800108,0xc29c0101,0xfb800108,0xc29d0101,0xfb800108,0xc29e0101,0xfb800108,0xc29f0101,
	0xfb800108,0xc2a00101,0xfb800108,0xc2a10101,0xfb800108,0xc2a20101,0xfb800108,0xc2a30101,
	0xfb800108,0xc2a40101,0xfb800108,0xc2a50101,0xfb800108,0xc2a60101,0xfb800108,0xc2a70101,
	0xfb800108,0xc2a80101,0xfb800108,0xc2a90101,0xfb800108,0xc2aa0101,0xfb800108,0xc2ab0101,
	0xfb800108,0xc2ac0101,0xfb800108,0xc2ad0101,0xfb800108,0xc2ae0101,0xfb800108,0xc2af0101,
	0xfb800108,0xc2b00101,0xfb800108,0xc2b10101,0xfb800108,0xc2b20101,0xfb800108,0xc2b30101,
	0xfb800108,0xc2b40101,0xfb800108,0xc2b50101,0xfb800108,0xc2b60101,0xfb800108,0xc2b70101,
	0xfb800108,0xc2b80101,0xfb800108,0xc2b90101,0xfb800108,0xc2ba0101,0xfb800108,0xc2bb0101,
	0xfb800108,0xc2bc0101,0xfb800108,0xc2bd0101,0xfb800108,0xc2be0101,0xfb800108,0xc2bf0101,
	0xfb800108,0xc2c00101,0xfb800108,0xc2c10101,0xfb800108,0xc2c20101,0xfb800108,0xc2c30101,
	0xfb800108,0xc2c40101,0xfb800108,0xc2c50101,0xfb800108,0xc2c60101,0xfb800108,0xc2c70101,
	0xfb800108,0xc2c80101,0xfb800108,0xc2c90101,0xfb800108,0xc2ca0101,0xfb800108,0xc2cb0101,
	0xfb800108,0xc2cc0101,0xfb800108,0xc2cd0101,0xfb800108,0xc2ce0101,0xfb800108,0xc2cf0101,
	0xfb800108,0xc2d00101,0xfb800108,0xc2d10101,0xfb800108,0xc2d20101,0xfb800108,0xc2d30101,
	0xfb800108,0xc2d40101,0xfb800108,0xc2d50101,0xfb800108,0xc2d60101,0xfb800108,0xc2d70101,
	0xfb800108,0xc2d80101,0xfb800108,0xc2d90101,0xfb800108,0xc2da0101,0xfb800108,0xc2db0101,
	0xfb800108,0xc2dc0101,0xfb800108,0xc2dd0101,0xfb800108,0xc2de0101,0xfb800108,0xc2df0101,
	0xfb800108,0xc2e00101,0xfb800108,0xc2e10101,0xfb800108,0xc2e20101,0xfb800108,0xc2e30101,
	0xfb800108,0xc2e40101,0xfb800108,0xc2e50101,0xfb800108,0xc2e60101,0xfb800108,0xc2e70101,
	0xfb800108,0xc2e80101,0xfb800108,0xc2e90101,0xfb800108,0xc2ea0101,0xfb800108,0xc2eb0101,
	0xfb800108,0xc2ec0101,0xfb800108,0xc2ed0101,0xfb800108,0xc2ee0101,0xfb800108,0xc2ef0101,
	0xfb800108,0xc2f00101,0xfb800108,0xc2f10101,0xfb800108,0xc2f20101,0xfb800108,0xc2f30101,
	0xfb800108,0xc2f40101,0xfb800108,0xc2f50101,0xfb800108,0xc2f60101,0xfb800108,0xc2f70101,
	0xfb800108,0xc2f80101,0xfb800108,0xc2f90101,0xfb800108,0xc2fa0101,0xfb800108,0xc2fb0101,
	0xfb800108,0xc2fc0101,0xfb800108,0xc2fd0101,0xfb800108,0xc2fe0101,0xfb800108,0xc2ff0101,
	0xfb800108,0xc3000101,0xfb800108,0xc3010101,0xfb800108,0xc3020101,0xfb800108,0xc3030101,
	0xfb800108,0xc3040101,0xfb800108,0xc3050101,0xfb800108,0xc3060101,0xfb800108,0xc3070101,
	0xfb800108,0xc3080101,0xfb800108,0xc3090101,0xfb800108,0xc30a0101,0xfb800108,0xc30b0101,
	0xfb800108,0xc30c0101,0xfb800108,0xc30d0101,0xfb800108,0xc30e0101,0xfb800108,0xc30f0101,
	0xfb800108,0xc3100101,0xfb800108,0xc3110101,0xfb800108,0xc3120101,0xfb800108,0xc3130101,
	0xfb800108,0xc3140101,0xfb800108,0xc3150101,0xfb800108,0xc3160101,0xfb800108,0xc3170101,
	0xfb800108,0xc3180101,0xfb800108,0xc3190101,0xfb800108,0xc31a0101,0xfb800108,0xc31b0101,
	0xfb800108,0xc31c0101,0xfb800108,0xc31d0101,0xfb800108,0xc31e0101,0xfb800108,0xc31f0101,
	0xfb800108,0xc3200101,0xfb800108,0xc3210101,0xfb800108,0xc3220101,0xfb800108,0xc3230101,
	0xfb800108,0xc3240101,0xfb800108,0xc3250101,0xfb800108,0xc3260101,0xfb800108,0xc3270101,
	0xfb800108,0xc3280101,0xfb800108,0xc3290101,0xfb800108,0xc32a0101,0xfb800108,0xc32b0101,
	0xfb800108,0xc32c0101,0xfb800108,0xc32d0101,0xfb800108,0xc32e0101,0xfb800108,0xc32f0101,
	0xfb800108,0xc3300101,0xfb800108,0xc3310101,0xfb800108,0xc3320101,0xfb800108,0xc3330101,
	0xfb800108,0xc3340101,0xfb800108,0xc3350101,0xfb800108,0xc3360101,0xfb800108,0xc3370101,
	0xfb800108,0xc3380101,0xfb800108,0xc3390101,0xfb800108,0xc33a0101,0xfb800108,0xc33b0101,
	0xfb800108,0xc33c0101,0xfb800108,0xc33d0101,0xfb800108,0xc33e0101,0xfb800108,0xc33f0101,
	0xfb800108,0xc3400101,0xfb800108,0xc3410101,0xfb800108,0xc3420101,0xfb800108,0xc3430101,
	0xfb800108,0xc3440101,0xfb800108,0xc3450101,0xfb800108,0xc3460101,0xfb800108,0xc3470101,
	0xfb800108,0xc3480101,0xfb800108,0xc3490101,0xfb800108,0xc34a0101,0xfb800108,0xc34b0101,
	0xfb800108,0xc34c0101,0xfb800108,0xc34d0101,0xfb800108,0xc34e0101,0xfb800108,0xc34f0101,
	0xfb800108,0xc3500101,0xfb800108,0xc3510101,0xfb800108,0xc3520101,0xfb800108,0xc3530101,
	0xfb800108,0xc3540101,0xfb800108,0xc3550101,0xfb800108,0xc3560101,0xfb800108,0xc3570101,
	0xfb800108,0xc3580101,0xfb800108,0xc3590101,0xfb800108,0xc35a0101,0xfb800108,0xc35b0101,
	0xfb800108,0xc35c0101,0xfb800108,0xc35d0101,0xfb800108,0xc35e0101,0xfb800108,0xc35f0101,
	0xfb800108,0xc3600101,0xfb800108,0xc3610101,0xfb800108,0xc3620101,0xfb800108,0xc3630101,
	0xfb800108,0xc3640101,0xfb800108,0xc3650101,0xfb800108,0xc3660101,0xfb800108,0xc3670101,
	0xfb800108,0xc3680101,0xfb800108,0xc3690101,0xfb800108,0xc36a0101,0xfb800108,0xc36b0101,
	0xfb800108,0xc36c0101,0xfb800108,0xc36d0101,0xfb800108,0xc36e0101,0xfb800108,0xc36f0101,
	0xfb800108,0xc3700101,0xfb800108,0xc3710101,0xfb800108,0xc3720101,0xfb800108,0xc3730101,
	0xfb800108,0xc3740101,0xfb800108,0xc3750101,0xfb800108,0xc3760101,0xfb800108,0xc3770101,
	0xfb800108,0xc3780101,0xfb800108,0xc3790101,0xfb800108,0xc37a0101,0xfb800108,0xc37b0101,
	0xfb800108,0xc37c0101,0xfb800108,0xc37d0101,0xfb800108,0xc37e0101,0xfb800108,0xc37f0101,
	0xfb800108,0xc3800101,0xfb800108,0xc3810101,0xfb800108,0xc3820101,0xfb800108,0xc3830101,
	0xfb800108,0xc3840101,0xfb800108,0xc3850101,0xfb800108,0xc3860101,0xfb800108,0xc3870101,
	0xfb800108,0xc3880101,0xfb800108,0xc3890101,0xfb800108,0xc38a0101,0xfb800108,0xc38b0101,
	0xfb800108,0xc38c0101,0xfb800108,0xc38d0101,0xfb800108,0xc38e0101,0xfb800108,0xc38f0101,
	0xfb800108,0xc3900101,0xfb800108,0xc3910101,0xfb800108,0xc3920101,0xfb800108,0xc3930101,
	0xfb800108,0xc3940101,0xfb800108,0xc3950101,0xfb800108,0xc3960101,0xfb800108,0xc3970101,
	0xfb800108,0xc3980101,0xfb800108,0xc3990101,0xfb800108,0xc39a0101,0xfb800108,0xc39b0101,
	0xfb800108,0xc39c0101,0xfb800108,0xc39d0101,0xfb800108,0xc39e0101,0xfb800108,0xc39f0101,
	0xfb800108,0xc3a00101,0xfb800108,0xc3a10101,0xfb800108,0xc3a20101,0xfb800108,0xc3a30101,
	0xfb800108,0xc3a40101,0xfb800108,0xc3a50101,0xfb800108,0xc3a60101,0xfb800108,0xc3a70101,
	0xfb800108,0xc3a80101,0xfb800108,0xc3a90101,0xfb800108,0xc3aa0101,0xfb800108,0xc3ab0101,
	0xfb800108,0xc3ac0101,0xfb800108,0xc3ad0101,0xfb800108,0xc3ae0101,0xfb800108,0xc3af0101,
	0xfb800108,0xc3b00101,0xfb800108,0xc3b10101,0xfb800108,0xc3b20101,0xfb800108,0xc3b30101,
	0xfb800108,0xc3b40101,0xfb800108,0xc3b50101,0xfb800108,0xc3b60101,0xfb800108,0xc3b70101,
	0xfb800108,0xc3b80101,0xfb800108,0xc3b90101,0xfb800108,0xc3ba0101,0xfb800108,0xc3bb0101,
	0xfb800108,0xc3bc0101,0xfb800108,0xc3bd0101,0xfb800108,0xc3be0101,0xfb800108,0xc3bf0101,
	0xfb800108,0xc3c00101,0xfb800108,0xc3c10101,0xfb800108,0xc3c20101,0xfb800108,0xc3c30101,
	0xfb800108,0xc3c40101,0xfb800108,0xc3c50101,0xfb800108,0xc3c60101,0xfb800108,0xc3c70101,
	0xfb800108,0xc3c80101,0xfb800108,0xc3c90101,0xfb800108,0xc3ca0101,0xfb800108,0xc3cb0101,
	0xfb800108,0xc3cc0101,0xfb800108,0xc3cd0101,0xfb800108,0xc3ce0101,0xfb800108,0xc3cf0101,
	0xfb800108,0xc3d00101,0xfb800108,0xc3d10101,0xfb800108,0xc3d20101,0xfb800108,0xc3d30101,
	0xfb800108,0xc3d40101,0xfb800108,0xc3d50101,0xfb800108,0xc3d60101,0xfb800108,0xc3d70101,
	0xfb800108,0xc3d80101,0xfb800108,0xc3d90101,0xfb800108,0xc3da0101,0xfb800108,0xc3db0101,
	0xfb800108,0xc3dc0101,0xfb800108,0xc3dd0101,0xfb800108,0xc3de0101,0xfb800108,0xc3df0101,
	0xfb800108,0xc3e00101,0xfb800108,0xc3e10101,0xfb800108,0xc3e20101,0xfb800108,0xc3e30101,
	0xfb800108,0xc3e40101,0xfb800108,0xc3e50101,0xfb800108,0xc3e60101,0xfb800108,0xc3e70101,
	0xfb800108,0xc3e80101,0xfb800108,0xc3e90101,0xfb800108,0xc3ea0101,0xfb800108,0xc3eb0101,
	0xfb800108,0xc3ec0101,0xfb800108,0xc3ed0101,0xfb800108,0xc3ee0101,0xfb800108,0xc3ef0101,
	0xfb800108,0xc3f00101,0xfb800108,0xc3f10101,0xfb800108,0xc3f20101,0xfb800108,0xc3f30101,
	0xfb800108,0xc3f40101,0xfb800108,0xc3f50101,0xfb800108,0xc3f60101,0xfb800108,0xc3f70101,
	0xfb800108,0xc3f80101,0xfb800108,0xc3f90101,0xfb800108,0xc3fa0101,0xfb800108,0xc3fb0101,
	0xfb800108,0xc3fc0101,0xfb800108,0xc3fd0101,0xfb800108,0xc3fe0101,0xfb800108,0xc3ff0101,
	0xfb800108,0xc4000101,0xfb800108,0xc4010101,0xfb800108,0xc4020101,0xfb800108,0xc4030101,
	0xfb800108,0xc4040101,0xfb800108,0xc4050101,0xfb800108,0xc4060101,0xfb800108,0xc4070101,
	0xfb800108,0xc4080101,0xfb800108,0xc4090101,0xfb800108,0xc40a0101,0xfb800108,0xc40b0101,
	0xfb800108,0xc40c0101,0xfb800108,0xc40d0101,0xfb800108,0xc40e0101,0xfb800108,0xc40f0101,
	0xfb800108,0xc4100101,0xfb800108,0xc4110101,0xfb800108,0xc4120101,0xfb800108,0xc4130101,
	0xfb800108,0xc4140101,0xfb800108,0xc4150101,0xfb800108,0xc4160101,0xfb800108,0xc4170101,
	0xfb800108,0xc4180101,0xfb800108,0xc4190101,0xfb800108,0xc41a0101,0xfb800108,0xc41b0101,
	0xfb800108,0xc41c0101,0xfb800108,0xc41d0101,0xfb800108,0xc41e0101,0xfb800108,0xc41f0101,
	0xfb800108,0xc4200101,0xfb800108,0xc4210101,0xfb800108,0xc4220101,0xfb800108,0xc4230101,
	0xfb800108,0xc4240101,0xfb800108,0xc4250101,0xfb800108,0xc4260101,0xfb800108,0xc4270101,
	0xfb800108,0xc4280101,0xfb800108,0xc4290101,0xfb800108,0xc42a0101,0xfb800108,0xc42b0101,
	0xfb800108,0xc42c0101,0xfb800108,0xc42d0101,0xfb800108,0xc42e0101,0xfb800108,0xc42f0101,
	0xfb800108,0xc4300101,0xfb800108,0xc4310101,0xfb800108,0xc4320101,0xfb800108,0xc4330101,
	0xfb800108,0xc4340101,0xfb800108,0xc4350101,0xfb800108,0xc4360101,0xfb800108,0xc4370101,
	0xfb800108,0xc4380101,0xfb800108,0xc4390101,0xfb800108,0xc43a0101,0xfb800108,0xc43b0101,
	0xfb800108,0xc43c0101,0xfb800108,0xc43d0101,0xfb800108,0xc43e0101,0xfb800108,0xc43f0101,
	0xfb800108,0xc4400101,0xfb800108,0xc4410101,0xfb800108,0xc4420101,0xfb800108,0xc4430101,
	0xfb800108,0xc4440101,0xfb800108,0xc4450101,0xfb800108,0xc4460101,0xfb800108,0xc4470101,
	0xfb800108,0xc4480101,0xfb800108,0xc4490101,0xfb800108,0xc44a0101,0xfb800108,0xc44b0101,
	0xfb800108,0xc44c0101,0xfb800108,0xc44d0101,0xfb800108,0xc44e0101,0xfb800108,0xc44f0101,
	0xfb800108,0xc4500101,0xfb800108,0xc4510101,0xfb800108,0xc4520101,0xfb800108,0xc4530101,
	0xfb800108,0xc4540101,0xfb800108,0xc4550101,0xfb800108,0xc4560101,0xfb800108,0xc4570101,
	0xfb800108,0xc4580101,0xfb800108,0xc4590101,0xfb800108,0xc45a0101,0xfb800108,0xc45b0101,
	0xfb800108,0xc45c0101,0xfb800108,0xc45d0101,0xfb800108,0xc45e0101,0xfb800108,0xc45f0101,
	0xfb800108,0xc4600101,0xfb800108,0xc4610101,0xfb800108,0xc4620101,0xfb800108,0xc4630101,
	0xfb800108,0xc4640101,0xfb800108,0xc4650101,0xfb800108,0xc4660101,0xfb800108,0xc4670101,
	0xfb800108,0xc4680101,0xfb800108,0xc4690101,0xfb800108,0xc46a0101,0xfb800108,0xc46b0101,
	0xfb800108,0xc46c0101,0xfb800108,0xc46d0101,0xfb800108,0xc46e0101,0xfb800108,0xc46f0101,
	0xfb800108,0xc4700101,0xfb800108,0xc4710101,0xfb800108,0xc4720101,0xfb800108,0xc4730101,
	0xfb800108,0xc4740101,0xfb800108,0xc4750101,0xfb800108,0xc4760101,0xfb800108,0xc4770101,
	0xfb800108,0xc4780101,0xfb800108,0xc4790101,0xfb800108,0xc47a0101,0xfb800108,0xc47b0101,
	0xfb800108,0xc47c0101,0xfb800108,0xc47d0101,0xfb800108,0xc47e0101,0xfb800108,0xc47f0101,
	0xfb800108,0xc4800101,0xfb800108,0xc4810101,0xfb800108,0xc4820101,0xfb800108,0xc4830101,
	0xfb800108,0xc4840101,0xfb800108,0xc4850101,0xfb800108,0xc4860101,0xfb800108,0xc4870101,
	0xfb800108,0xc4880101,0xfb800108,0xc4890101,0xfb800108,0xc48a0101,0xfb800108,0xc48b0101,
	0xfb800108,0xc48c0101,0xfb800108,0xc48d0101,0xfb800108,0xc48e0101,0xfb800108,0xc48f0101,
	0xfb800108,0xc4900101,0xfb800108,0xc4910101,0xfb800108,0xc4920101,0xfb800108,0xc4930101,
	0xfb800108,0xc4940101,0xfb800108,0xc4950101,0xfb800108,0xc4960101,0xfb800108,0xc4970101,
	0xfb800108,0xc4980101,0xfb800108,0xc4990101,0xfb800108,0xc49a0101,0xfb800108,0xc49b0101,
	0xfb800108,0xc49c0101,0xfb800108,0xc49d0101,0xfb800108,0xc49e0101,0xfb800108,0xc49f0101,
	0xfb800108,0xc4a00101,0xfb800108,0xc4a10101,0xfb800108,0xc4a20101,0xfb800108,0xc4a30101,
	0xfb800108,0xc4a40101,0xfb800108,0xc4a50101,0xfb800108,0xc4a60101,0xfb800108,0xc4a70101,
	0xfb800108,0xc4a80101,0xfb800108,0xc4a90101,0xfb800108,0xc4aa0101,0xfb800108,0xc4ab0101,
	0xfb800108,0xc4ac0101,0xfb800108,0xc4ad0101,0xfb800108,0xc4ae0101,0xfb800108,0xc4af0101,
	0xfb800108,0xc4b00101,0xfb800108,0xc4b10101,0xfb800108,0xc4b20101,0xfb800108,0xc4b30101,
	0xfb800108,0xc4b40101,0xfb800108,0xc4b50101,0xfb800108,0xc4b60101,0xfb800108,0xc4b70101,
	0xfb800108,0xc4b80101,0xfb800108,0xc4b90101,0xfb800108,0xc4ba0101,0xfb800108,0xc4bb0101,
	0xfb800108,0xc4bc0101,0xfb800108,0xc4bd0101,0xfb800108,0xc4be0101,0xfb800108,0xc4bf0101,
	0xfb800108,0xc4c00101,0xfb800108,0xc4c10101,0xfb800108,0xc4c20101,0xfb800108,0xc4c30101,
	0xfb800108,0xc4c40101,0xfb800108,0xc4c50101,0xfb800108,0xc4c60101,0xfb800108,0xc4c70101,
	0xfb800108,0xc4c80101,0xfb800108,0xc4c90101,0xfb800108,0xc4ca0101,0xfb800108,0xc4cb0101,
	0xfb800108,0xc4cc0101,0xfb800108,0xc4cd0101,0xfb800108,0xc4ce0101,0xfb800108,0xc4cf0101,
	0xfb800108,0xc4d00101,0xfb800108,0xc4d10101,0xfb800108,0xc4d20101,0xfb800108,0xc4d30101,
	0xfb800108,0xc4d40101,0xfb800108,0xc4d50101,0xfb800108,0xc4d60101,0xfb800108,0xc4d70101,
	0xfb800108,0xc4d80101,0xfb800108,0xc4d90101,0xfb800108,0xc4da0101,0xfb800108,0xc4db0101,
	0xfb800108,0xc4dc0101,0xfb800108,0xc4dd0101,0xfb800108,0xc4de0101,0xfb800108,0xc4df0101,
	0xfb800108,0xc4e00101,0xfb800108,0xc4e10101,0xfb800108,0xc4e20101,0xfb800108,0xc4e30101,
	0xfb800108,0xc4e40101,0xfb800108,0xc4e50101,0xfb800108,0xc4e60101,0xfb800108,0xc4e70101,
	0xfb800108,0xc4e80101,0xfb800108,0xc4e90101,0xfb800108,0xc4ea0101,0xfb800108,0xc4eb0101,
	0xfb800108,0xc4ec0101,0xfb800108,0xc4ed0101,0xfb800108,0xc4ee0101,0xfb800108,0xc4ef0101,
	0xfb800108,0xc4f00101,0xfb800108,0xc4f10101,0xfb800108,0xc4f20101,0xfb800108,0xc4f30101,
	0xfb800108,0xc4f40101,0xfb800108,0xc4f50101,0xfb800108,0xc4f60101,0xfb800108,0xc4f70101,
	0xfb800108,0xc4f80101,0xfb800108,0xc4f90101,0xfb800108,0xc4fa0101,0xfb800108,0xc4fb0101,
	0xfb800108,0xc4fc0101,0xfb800108,0xc4fd0101,0xfb800108,0xc4fe0101,0xfb800108,0xc4ff0101,
	0xfb800108,0xc5000101,0xfb800108,0xc5010101,0xfb800108,0xc5020101,0xfb800108,0xc5030101,
	0xfb800108,0xc5040101,0xfb800108,0xc5050101,0xfb800108,0xc5060101,0xfb800108,0xc5070101,
	0xfb800108,0xc5080101,0xfb800108,0xc5090101,0xfb800108,0xc50a0101,0xfb800108,0xc50b0101,
	0xfb800108,0xc50c0101,0xfb800108,0xc50d0101,0xfb800108,0xc50e0101,0xfb800108,0xc50f0101,
	0xfb800108,0xc5100101,0xfb800108,0xc5110101,0xfb800108,0xc5120101,0xfb800108,0xc5130101,
	0xfb800108,0xc5140101,0xfb800108,0xc5150101,0xfb800108,0xc5160101,0xfb800108,0xc5170101,
	0xfb800108,0xc5180101,0xfb800108,0xc5190101,0xfb800108,0xc51a0101,0xfb800108,0xc51b0101,
	0xfb800108,0xc51c0101,0xfb800108,0xc51d0101,0xfb800108,0xc51e0101,0xfb800108,0xc51f0101,
	0xfb800108,0xc5200101,0xfb800108,0xc5210101,0xfb800108,0xc5220101,0xfb800108,0xc5230101,
	0xfb800108,0xc5240101,0xfb800108,0xc5250101,0xfb800108,0xc5260101,0xfb800108,0xc5270101,
	0xfb800108,0xc5280101,0xfb800108,0xc5290101,0xfb800108,0xc52a0101,0xfb800108,0xc52b0101,
	0xfb800108,0xc52c0101,0xfb800108,0xc52d0101,0xfb800108,0xc52e0101,0xfb800108,0xc52f0101,
	0xfb800108,0xc5300101,0xfb800108,0xc5310101,0xfb800108,0xc5320101,0xfb800108,0xc5330101,
	0xfb800108,0xc5340101,0xfb800108,0xc5350101,0xfb800108,0xc5360101,0xfb800108,0xc5370101,
	0xfb800108,0xc5380101,0xfb800108,0xc5390101,0xfb800108,0xc53a0101,0xfb800108,0xc53b0101,
	0xfb800108,0xc53c0101,0xfb800108,0xc53d0101,0xfb800108,0xc53e0101,0xfb800108,0xc53f0101,
	0xfb800108,0xc5400101,0xfb800108,0xc5410101,0xfb800108,0xc5420101,0xfb800108,0xc5430101,
	0xfb800108,0xc5440101,0xfb800108,0xc5450101,0xfb800108,0xc5460101,0xfb800108,0xc5470101,
	0xfb800108,0xc5480101,0xfb800108,0xc5490101,0xfb800108,0xc54a0101,0xfb800108,0xc54b0101,
	0xfb800108,0xc54c0101,0xfb800108,0xc54d0101,0xfb800108,0xc54e0101,0xfb800108,0xc54f0101,
	0xfb800108,0xc5500101,0xfb800108,0xc5510101,0xfb800108,0xc5520101,0xfb800108,0xc5530101,
	0xfb800108,0xc5540101,0xfb800108,0xc5550101,0xfb800108,0xc5560101,0xfb800108,0xc5570101,
	0xfb800108,0xc5580101,0xfb800108,0xc5590101,0xfb800108,0xc55a0101,0xfb800108,0xc55b0101,
	0xfb800108,0xc55c0101,0xfb800108,0xc55d0101,0xfb800108,0xc55e0101,0xfb800108,0xc55f0101,
	0xfb800108,0xc5600101,0xfb800108,0xc5610101,0xfb800108,0xc5620101,0xfb800108,0xc5630101,
	0xfb800108,0xc5640101,0xfb800108,0xc5650101,0xfb800108,0xc5660101,0xfb800108,0xc5670101,
	0xfb800108,0xc5680101,0xfb800108,0xc5690101,0xfb800108,0xc56a0101,0xfb800108,0xc56b0101,
	0xfb800108,0xc56c0101,0xfb800108,0xc56d0101,0xfb800108,0xc56e0101,0xfb800108,0xc56f0101,
	0xfb800108,0xc5700101,0xfb800108,0xc5710101,0xfb800108,0xc5720101,0xfb800108,0xc5730101,
	0xfb800108,0xc5740101,0xfb800108,0xc5750101,0xfb800108,0xc5760101,0xfb800108,0xc5770101,
	0xfb800108,0xc5780101,0xfb800108,0xc5790101,0xfb800108,0xc57a0101,0xfb800108,0xc57b0101,
	0xfb800108,0xc57c0101,0xfb800108,0xc57d0101,0xfb800108,0xc57e0101,0xfb800108,0xc57f0101,
	0xfb800108,0xc5800101,0xfb800108,0xc5810101,0xfb800108,0xc5820101,0xfb800108,0xc5830101,
	0xfb800108,0xc5840101,0xfb800108,0xc5850101,0xfb800108,0xc5860101,0xfb800108,0xc5870101,
	0xfb800108,0xc5880101,0xfb800108,0xc5890101,0xfb800108,0xc58a0101,0xfb800108,0xc58b0101,
	0xfb800108,0xc58c0101,0xfb800108,0xc58d0101,0xfb800108,0xc58e0101,0xfb800108,0xc58f0101,
	0xfb800108,0xc5900101,0xfb800108,0xc5910101,0xfb800108,0xc5920101,0xfb800108,0xc5930101,
	0xfb800108,0xc5940101,0xfb800108,0xc5950101,0xfb800108,0xc5960101,0xfb800108,0xc5970101,
	0xfb800108,0xc5980101,0xfb800108,0xc5990101,0xfb800108,0xc59a0101,0xfb800108,0xc59b0101,
	0xfb800108,0xc59c0101,0xfb800108,0xc59d0101,0xfb800108,0xc59e0101,0xfb800108,0xc59f0101,
	0xfb800108,0xc5a00101,0xfb800108,0xc5a10101,0xfb800108,0xc5a20101,0xfb800108,0xc5a30101,
	0xfb800108,0xc5a40101,0xfb800108,0xc5a50101,0xfb800108,0xc5a60101,0xfb800108,0xc5a70101,
	0xfb800108,0xc5a80101,0xfb800108,0xc5a90101,0xfb800108,0xc5aa0101,0xfb800108,0xc5ab0101,
	0xfb800108,0xc5ac0101,0xfb800108,0xc5ad0101,0xfb800108,0xc5ae0101,0xfb800108,0xc5af0101,
	0xfb800108,0xc5b00101,0xfb800108,0xc5b10101,0xfb800108,0xc5b20101,0xfb800108,0xc5b30101,
	0xfb800108,0xc5b40101,0xfb800108,0xc5b50101,0xfb800108,0xc5b60101,0xfb800108,0xc5b70101,
	0xfb800108,0xc5b80101,0xfb800108,0xc5b90101,0xfb800108,0xc5ba0101,0xfb800108,0xc5bb0101,
	0xfb800108,0xc5bc0101,0xfb800108,0xc5bd0101,0xfb800108,0xc5be0101,0xfb800108,0xc5bf0101,
	0xfb800108,0xc5c00101,0xfb800108,0xc5c10101,0xfb800108,0xc5c20101,0xfb800108,0xc5c30101,
	0xfb800108,0xc5c40101,0xfb800108,0xc5c50101,0xfb800108,0xc5c60101,0xfb800108,0xc5c70101,
	0xfb800108,0xc5c80101,0xfb800108,0xc5c90101,0xfb800108,0xc5ca0101,0xfb800108,0xc5cb0101,
	0xfb800108,0xc5cc0101,0xfb800108,0xc5cd0101,0xfb800108,0xc5ce0101,0xfb800108,0xc5cf0101,
	0xfb800108,0xc5d00101,0xfb800108,0xc5d10101,0xfb800108,0xc5d20101,0xfb800108,0xc5d30101,
	0xfb800108,0xc5d40101,0xfb800108,0xc5d50101,0xfb800108,0xc5d60101,0xfb800108,0xc5d70101,
	0xfb800108,0xc5d80101,0xfb800108,0xc5d90101,0xfb800108,0xc5da0101,0xfb800108,0xc5db0101,
	0xfb800108,0xc5dc0101,0xfb800108,0xc5dd0101,0xfb800108,0xc5de0101,0xfb800108,0xc5df0101,
	0xfb800108,0xc5e00101,0xfb800108,0xc5e10101,0xfb800108,0xc5e20101,0xfb800108,0xc5e30101,
	0xfb800108,0xc5e40101,0xfb800108,0xc5e50101,0xfb800108,0xc5e60101,0xfb800108,0xc5e70101,
	0xfb800108,0xc5e80101,0xfb800108,0xc5e90101,0xfb800108,0xc5ea0101,0xfb800108,0xc5eb0101,
	0xfb800108,0xc5ec0101,0xfb800108,0xc5ed0101,0xfb800108,0xc5ee0101,0xfb800108,0xc5ef0101,
	0xfb800108,0xc5f00101,0xfb800108,0xc5f10101,0xfb800108,0xc5f20101,0xfb800108,0xc5f30101,
	0xfb800108,0xc5f40101,0xfb800108,0xc5f50101,0xfb800108,0xc5f60101,0xfb800108,0xc5f70101,
	0xfb800108,0xc5f80101,0xfb800108,0xc5f90101,0xfb800108,0xc5fa0101,0xfb800108,0xc5fb0101,
	0xfb800108,0xc5fc0101,0xfb800108,0xc5fd0101,0xfb800108,0xc5fe0101,0xfb800108,0xc5ff0101,
	0xfb800108,0xc6000101,0xfb800108,0xc6010101,0xfb800108,0xc6020101,0xfb800108,0xc6030101,
	0xfb800108,0xc6040101,0xfb800108,0xc6050101,0xfb800108,0xc6060101,0xfb800108,0xc6070101,
	0xfb800108,0xc6080101,0xfb800108,0xc6090101,0xfb800108,0xc60a0101,0xfb800108,0xc60b0101,
	0xfb800108,0xc60c0101,0xfb800108,0xc60d0101,0xfb800108,0xc60e0101,0xfb800108,0xc60f0101,
	0xfb800108,0xc6100101,0xfb800108,0xc6110101,0xfb800108,0xc6120101,0xfb800108,0xc6130101,
	0xfb800108,0xc6140101,0xfb800108,0xc6150101,0xfb800108,0xc6160101,0xfb800108,0xc6170101,
	0xfb800108,0xc6180101,0xfb800108,0xc6190101,0xfb800108,0xc61a0101,0xfb800108,0xc61b0101,
	0xfb800108,0xc61c0101,0xfb800108,0xc61d0101,0xfb800108,0xc61e0101,0xfb800108,0xc61f0101,
	0xfb800108,0xc6200101,0xfb800108,0xc6210101,0xfb800108,0xc6220101,0xfb800108,0xc6230101,
	0xfb800108,0xc6240101,0xfb800108,0xc6250101,0xfb800108,0xc6260101,0xfb800108,0xc6270101,
	0xfb800108,0xc6280101,0xfb800108,0xc6290101,0xfb800108,0xc62a0101,0xfb800108,0xc62b0101,
	0xfb800108,0xc62c0101,0xfb800108,0xc62d0101,0xfb800108,0xc62e0101,0xfb800108,0xc62f0101,
	0xfb800108,0xc6300101,0xfb800108,0xc6310101,0xfb800108,0xc6320101,0xfb800108,0xc6330101,
	0xfb800108,0xc6340101,0xfb800108,0xc6350101,0xfb800108,0xc6360101,0xfb800108,0xc6370101,
	0xfb800108,0xc6380101,0xfb800108,0xc6390101,0xfb800108,0xc63a0101,0xfb800108,0xc63b0101,
	0xfb800108,0xc63c0101,0xfb800108,0xc63d0101,0xfb800108,0xc63e0101,0xfb800108,0xc63f0101,
	0xfb800108,0xc6400101,0xfb800108,0xc6410101,0xfb800108,0xc6420101,0xfb800108,0xc6430101,
	0xfb800108,0xc6440101,0xfb800108,0xc6450101,0xfb800108,0xc6460101,0xfb800108,0xc6470101,
	0xfb800108,0xc6480101,0xfb800108,0xc6490101,0xfb800108,0xc64a0101,0xfb800108,0xc64b0101,
	0xfb800108,0xc64c0101,0xfb800108,0xc64d0101,0xfb800108,0xc64e0101,0xfb800108,0xc64f0101,
	0xfb800108,0xc6500101,0xfb800108,0xc6510101,0xfb800108,0xc6520101,0xfb800108,0xc6530101,
	0xfb800108,0xc6540101,0xfb800108,0xc6550101,0xfb800108,0xc6560101,0xfb800108,0xc6570101,
	0xfb800108,0xc6580101,0xfb800108,0xc6590101,0xfb800108,0xc65a0101,0xfb800108,0xc65b0101,
	0xfb800108,0xc65c0101,0xfb800108,0xc65d0101,0xfb800108,0xc65e0101,0xfb800108,0xc65f0101,
	0xfb800108,0xc6600101,0xfb800108,0xc6610101,0xfb800108,0xc6620101,0xfb800108,0xc6630101,
	0xfb800108,0xc6640101,0xfb800108,0xc6650101,0xfb800108,0xc6660101,0xfb800108,0xc6670101,
	0xfb800108,0xc6680101,0xfb800108,0xc6690101,0xfb800108,0xc66a0101,0xfb800108,0xc66b0101,
	0xfb800108,0xc66c0101,0xfb800108,0xc66d0101,0xfb800108,0xc66e0101,0xfb800108,0xc66f0101,
	0xfb800108,0xc6700101,0xfb800108,0xc6710101,0xfb800108,0xc6720101,0xfb800108,0xc6730101,
	0xfb800108,0xc6740101,0xfb800108,0xc6750101,0xfb800108,0xc6760101,0xfb800108,0xc6770101,
	0xfb800108,0xc6780101,0xfb800108,0xc6790101,0xfb800108,0xc67a0101,0xfb800108,0xc67b0101,
	0xfb800108,0xc67c0101,0xfb800108,0xc67d0101,0xfb800108,0xc67e0101,0xfb800108,0xc67f0101,
	0xfb800108,0xc6800101,0xfb800108,0xc6810101,0xfb800108,0xc6820101,0xfb800108,0xc6830101,
	0xfb800108,0xc6840101,0xfb800108,0xc6850101,0xfb800108,0xc6860101,0xfb800108,0xc6870101,
	0xfb800108,0xc6880101,0xfb800108,0xc6890101,0xfb800108,0xc68a0101,0xfb800108,0xc68b0101,
	0xfb800108,0xc68c0101,0xfb800108,0xc68d0101,0xfb800108,0xc68e0101,0xfb800108,0xc68f0101,
	0xfb800108,0xc6900101,0xfb800108,0xc6910101,0xfb800108,0xc6920101,0xfb800108,0xc6930101,
	0xfb800108,0xc6940101,0xfb800108,0xc6950101,0xfb800108,0xc6960101,0xfb800108,0xc6970101,
	0xfb800108,0xc6980101,0xfb800108,0xc6990101,0xfb800108,0xc69a0101,0xfb800108,0xc69b0101,
	0xfb800108,0xc69c0101,0xfb800108,0xc69d0101,0xfb800108,0xc69e0101,0xfb800108,0xc69f0101,
	0xfb800108,0xc6a00101,0xfb800108,0xc6a10101,0xfb800108,0xc6a20101,0xfb800108,0xc6a30101,
	0xfb800108,0xc6a40101,0xfb800108,0xc6a50101,0xfb800108,0xc6a60101,0xfb800108,0xc6a70101,
	0xfb800108,0xc6a80101,0xfb800108,0xc6a90101,0xfb800108,0xc6aa0101,0xfb800108,0xc6ab0101,
	0xfb800108,0xc6ac0101,0xfb800108,0xc6ad0101,0xfb800108,0xc6ae0101,0xfb800108,0xc6af0101,
	0xfb800108,0xc6b00101,0xfb800108,0xc6b10101,0xfb800108,0xc6b20101,0xfb800108,0xc6b30101,
	0xfb800108,0xc6b40101,0xfb800108,0xc6b50101,0xfb800108,0xc6b60101,0xfb800108,0xc6b70101,
	0xfb800108,0xc6b80101,0xfb800108,0xc6b90101,0xfb800108,0xc6ba0101,0xfb800108,0xc6bb0101,
	0xfb800108,0xc6bc0101,0xfb800108,0xc6bd0101,0xfb800108,0xc6be0101,0xfb800108,0xc6bf0101,
	0xfb800108,0xc6c00101,0xfb800108,0xc6c10101,0xfb800108,0xc6c20101,0xfb800108,0xc6c30101,
	0xfb800108,0xc6c40101,0xfb800108,0xc6c50101,0xfb800108,0xc6c60101,0xfb800108,0xc6c70101,
	0xfb800108,0xc6c80101,0xfb800108,0xc6c90101,0xfb800108,0xc6ca0101,0xfb800108,0xc6cb0101,
	0xfb800108,0xc6cc0101,0xfb800108,0xc6cd0101,0xfb800108,0xc6ce0101,0xfb800108,0xc6cf0101,
	0xfb800108,0xc6d00101,0xfb800108,0xc6d10101,0xfb800108,0xc6d20101,0xfb800108,0xc6d30101,
	0xfb800108,0xc6d40101,0xfb800108,0xc6d50101,0xfb800108,0xc6d60101,0xfb800108,0xc6d70101,
	0xfb800108,0xc6d80101,0xfb800108,0xc6d90101,0xfb800108,0xc6da0101,0xfb800108,0xc6db0101,
	0xfb800108,0xc6dc0101,0xfb800108,0xc6dd0101,0xfb800108,0xc6de0101,0xfb800108,0xc6df0101,
	0xfb800108,0xc6e00101,0xfb800108,0xc6e10101,0xfb800108,0xc6e20101,0xfb800108,0xc6e30101,
	0xfb800108,0xc6e40101,0xfb800108,0xc6e50101,0xfb800108,0xc6e60101,0xfb800108,0xc6e70101,
	0xfb800108,0xc6e80101,0xfb800108,0xc6e90101,0xfb800108,0xc6ea0101,0xfb800108,0xc6eb0101,
	0xfb800108,0xc6ec0101,0xfb800108,0xc6ed0101,0xfb800108,0xc6ee0101,0xfb800108,0xc6ef0101,
	0xfb800108,0xc6f00101,0xfb800108,0xc6f10101,0xfb800108,0xc6f20101,0xfb800108,0xc6f30101,
	0xfb800108,0xc6f40101,0xfb800108,0xc6f50101,0xfb800108,0xc6f60101,0xfb800108,0xc6f70101,
	0xfb800108,0xc6f80101,0xfb800108,0xc6f90101,0xfb800108,0xc6fa0101,0xfb800108,0xc6fb0101,
	0xfb800108,0xc6fc0101,0xfb800108,0xc6fd0101,0xfb800108,0xc6fe0101,0xfb800108,0xc6ff0101,
	0xfb800108,0xc7000101,0xfb800108,0xc7010101,0xfb800108,0xc7020101,0xfb800108,0xc7030101,
	0xfb800108,0xc7040101,0xfb800108,0xc7050101,0xfb800108,0xc7060101,0xfb800108,0xc7070101,
	0xfb800108,0xc7080101,0xfb800108,0xc7090101,0xfb800108,0xc70a0101,0xfb800108,0xc70b0101,
	0xfb800108,0xc70c0101,0xfb800108,0xc70d0101,0xfb800108,0xc70e0101,0xfb800108,0xc70f0101,
	0xfb800108,0xc7100101,0xfb800108,0xc7110101,0xfb800108,0xc7120101,0xfb800108,0xc7130101,
	0xfb800108,0xc7140101,0xfb800108,0xc7150101,0xfb800108,0xc7160101,0xfb800108,0xc7170101,
	0xfb800108,0xc7180101,0xfb800108,0xc7190101,0xfb800108,0xc71a0101,0xfb800108,0xc71b0101,
	0xfb800108,0xc71c0101,0xfb800108,0xc71d0101,0xfb800108,0xc71e0101,0xfb800108,0xc71f0101,
	0xfb800108,0xc7200101,0xfb800108,0xc7210101,0xfb800108,0xc7220101,0xfb800108,0xc7230101,
	0xfb800108,0xc7240101,0xfb800108,0xc7250101,0xfb800108,0xc7260101,0xfb800108,0xc7270101,
	0xfb800108,0xc7280101,0xfb800108,0xc7290101,0xfb800108,0xc72a0101,0xfb800108,0xc72b0101,
	0xfb800108,0xc72c0101,0xfb800108,0xc72d0101,0xfb800108,0xc72e0101,0xfb800108,0xc72f0101,
	0xfb800108,0xc7300101,0xfb800108,0xc7310101,0xfb800108,0xc7320101,0xfb800108,0xc7330101,
	0xfb800108,0xc7340101,0xfb800108,0xc7350101,0xfb800108,0xc7360101,0xfb800108,0xc7370101,
	0xfb800108,0xc7380101,0xfb800108,0xc7390101,0xfb800108,0xc73a0101,0xfb800108,0xc73b0101,
	0xfb800108,0xc73c0101,0xfb800108,0xc73d0101,0xfb800108,0xc73e0101,0xfb800108,0xc73f0101,
	0xfb800108,0xc7400101,0xfb800108,0xc7410101,0xfb800108,0xc7420101,0xfb800108,0xc7430101,
	0xfb800108,0xc7440101,0xfb800108,0xc7450101,0xfb800108,0xc7460101,0xfb800108,0xc7470101,
	0xfb800108,0xc7480101,0xfb800108,0xc7490101,0xfb800108,0xc74a0101,0xfb800108,0xc74b0101,
	0xfb800108,0xc74c0101,0xfb800108,0xc74d0101,0xfb800108,0xc74e0101,0xfb800108,0xc74f0101,
	0xfb800108,0xc7500101,0xfb800108,0xc7510101,0xfb800108,0xc7520101,0xfb800108,0xc7530101,
	0xfb800108,0xc7540101,0xfb800108,0xc7550101,0xfb800108,0xc7560101,0xfb800108,0xc7570101,
	0xfb800108,0xc7580101,0xfb800108,0xc7590101,0xfb800108,0xc75a0101,0xfb800108,0xc75b0101,
	0xfb800108,0xc75c0101,0xfb800108,0xc75d0101,0xfb800108,0xc75e0101,0xfb800108,0xc75f0101,
	0xfb800108,0xc7600101,0xfb800108,0xc7610101,0xfb800108,0xc7620101,0xfb800108,0xc7630101,
	0xfb800108,0xc7640101,0xfb800108,0xc7650101,0xfb800108,0xc7660101,0xfb800108,0xc7670101,
	0xfb800108,0xc7680101,0xfb800108,0xc7690101,0xfb800108,0xc76a0101,0xfb800108,0xc76b0101,
	0xfb800108,0xc76c0101,0xfb800108,0xc76d0101,0xfb800108,0xc76e0101,0xfb800108,0xc76f0101,
	0xfb800108,0xc7700101,0xfb800108,0xc7710101,0xfb800108,0xc7720101,0xfb800108,0xc7730101,
	0xfb800108,0xc7740101,0xfb800108,0xc7750101,0xfb800108,0xc7760101,0xfb800108,0xc7770101,
	0xfb800108,0xc7780101,0xfb800108,0xc7790101,0xfb800108,0xc77a0101,0xfb800108,0xc77b0101,
	0xfb800108,0xc77c0101,0xfb800108,0xc77d0101,0xfb800108,0xc77e0101,0xfb800108,0xc77f0101,
	0xfb800108,0xc7800101,0xfb800108,0xc7810101,0xfb800108,0xc7820101,0xfb800108,0xc7830101,
	0xfb800108,0xc7840101,0xfb800108,0xc7850101,0xfb800108,0xc7860101,0xfb800108,0xc7870101,
	0xfb800108,0xc7880101,0xfb800108,0xc7890101,0xfb800108,0xc78a0101,0xfb800108,0xc78b0101,
	0xfb800108,0xc78c0101,0xfb800108,0xc78d0101,0xfb800108,0xc78e0101,0xfb800108,0xc78f0101,
	0xfb800108,0xc7900101,0xfb800108,0xc7910101,0xfb800108,0xc7920101,0xfb800108,0xc7930101,
	0xfb800108,0xc7940101,0xfb800108,0xc7950101,0xfb800108,0xc7960101,0xfb800108,0xc7970101,
	0xfb800108,0xc7980101,0xfb800108,0xc7990101,0xfb800108,0xc79a0101,0xfb800108,0xc79b0101,
	0xfb800108,0xc79c0101,0xfb800108,0xc79d0101,0xfb800108,0xc79e0101,0xfb800108,0xc79f0101,
	0xfb800108,0xc7a00101,0xfb800108,0xc7a10101,0xfb800108,0xc7a20101,0xfb800108,0xc7a30101,
	0xfb800108,0xc7a40101,0xfb800108,0xc7a50101,0xfb800108,0xc7a60101,0xfb800108,0xc7a70101,
	0xfb800108,0xc7a80101,0xfb800108,0xc7a90101,0xfb800108,0xc7aa0101,0xfb800108,0xc7ab0101,
	0xfb800108,0xc7ac0101,0xfb800108,0xc7ad0101,0xfb800108,0xc7ae0101,0xfb800108,0xc7af0101,
	0xfb800108,0xc7b00101,0xfb800108,0xc7b10101,0xfb800108,0xc7b20101,0xfb800108,0xc7b30101,
	0xfb800108,0xc7b40101,0xfb800108,0xc7b50101,0xfb800108,0xc7b60101,0xfb800108,0xc7b70101,
	0xfb800108,0xc7b80101,0xfb800108,0xc7b90101,0xfb800108,0xc7ba0101,0xfb800108,0xc7bb0101,
	0xfb800108,0xc7bc0101,0xfb800108,0xc7bd0101,0xfb800108,0xc7be0101,0xfb800108,0xc7bf0101,
	0xfb800108,0xc7c00101,0xfb800108,0xc7c10101,0xfb800108,0xc7c20101,0xfb800108,0xc7c30101,
	0xfb800108,0xc7c40101,0xfb800108,0xc7c50101,0xfb800108,0xc7c60101,0xfb800108,0xc7c70101,
	0xfb800108,0xc7c80101,0xfb800108,0xc7c90101,0xfb800108,0xc7ca0101,0xfb800108,0xc7cb0101,
	0xfb800108,0xc7cc0101,0xfb800108,0xc7cd0101,0xfb800108,0xc7ce0101,0xfb800108,0xc7cf0101,
	0xfb800108,0xc7d00101,0xfb800108,0xc7d10101,0xfb800108,0xc7d20101,0xfb800108,0xc7d30101,
	0xfb800108,0xc7d40101,0xfb800108,0xc7d50101,0xfb800108,0xc7d60101,0xfb800108,0xc7d70101,
	0xfb800108,0xc7d80101,0xfb800108,0xc7d90101,0xfb800108,0xc7da0101,0xfb800108,0xc7db0101,
	0xfb800108,0xc7dc0101,0xfb800108,0xc7dd0101,0xfb800108,0xc7de0101,0xfb800108,0xc7df0101,
	0xfb800108,0xc7e00101,0xfb800108,0xc7e10101,0xfb800108,0xc7e20101,0xfb800108,0xc7e30101,
	0xfb800108,0xc7e40101,0xfb800108,0xc7e50101,0xfb800108,0xc7e60101,0xfb800108,0xc7e70101,
	0xfb800108,0xc7e80101,0xfb800108,0xc7e90101,0xfb800108,0xc7ea0101,0xfb800108,0xc7eb0101,
	0xfb800108,0xc7ec0101,0xfb800108,0xc7ed0101,0xfb800108,0xc7ee0101,0xfb800108,0xc7ef0101,
	0xfb800108,0xc7f00101,0xfb800108,0xc7f10101,0xfb800108,0xc7f20101,0xfb800108,0xc7f30101,
	0xfb800108,0xc7f40101,0xfb800108,0xc7f50101,0xfb800108,0xc7f60101,0xfb800108,0xc7f70101,
	0xfb800108,0xc7f80101,0xfb800108,0xc7f90101,0xfb800108,0xc7fa0101,0xfb800108,0xc7fb0101,
	0xfb800108,0xc7fc0101,0xfb800108,0xc7fd0101,0xfb800108,0xc7fe0101,0xfb800108,0xc7ff0101,
	0xfb800108,0xc8000101,0xfb800108,0xc8010101,0xfb800108,0xc8020101,0xfb800108,0xc8030101,
	0xfb800108,0xc8040101,0xfb800108,0xc8050101,0xfb800108,0xc8060101,0xfb800108,0xc8070101,
	0xfb800108,0xc8080101,0xfb800108,0xc8090101,0xfb800108,0xc80a0101,0xfb800108,0xc80b0101,
	0xfb800108,0xc80c0101,0xfb800108,0xc80d0101,0xfb800108,0xc80e0101,0xfb800108,0xc80f0101,
	0xfb800108,0xc8100101,0xfb800108,0xc8110101,0xfb800108,0xc8120101,0xfb800108,0xc8130101,
	0xfb800108,0xc8140101,0xfb800108,0xc8150101,0xfb800108,0xc8160101,0xfb800108,0xc8170101,
	0xfb800108,0xc8180101,0xfb800108,0xc8190101,0xfb800108,0xc81a0101,0xfb800108,0xc81b0101,
	0xfb800108,0xc81c0101,0xfb800108,0xc81d0101,0xfb800108,0xc81e0101,0xfb800108,0xc81f0101,
	0xfb800108,0xc8200101,0xfb800108,0xc8210101,0xfb800108,0xc8220101,0xfb800108,0xc8230101,
	0xfb800108,0xc8240101,0xfb800108,0xc8250101,0xfb800108,0xc8260101,0xfb800108,0xc8270101,
	0xfb800108,0xc8280101,0xfb800108,0xc8290101,0xfb800108,0xc82a0101,0xfb800108,0xc82b0101,
	0xfb800108,0xc82c0101,0xfb800108,0xc82d0101,0xfb800108,0xc82e0101,0xfb800108,0xc82f0101,
	0xfb800108,0xc8300101,0xfb800108,0xc8310101,0xfb800108,0xc8320101,0xfb800108,0xc8330101,
	0xfb800108,0xc8340101,0xfb800108,0xc8350101,0xfb800108,0xc8360101,0xfb800108,0xc8370101,
	0xfb800108,0xc8380101,0xfb800108,0xc8390101,0xfb800108,0xc83a0101,0xfb800108,0xc83b0101,
	0xfb800108,0xc83c0101,0xfb800108,0xc83d0101,0xfb800108,0xc83e0101,0xfb800108,0xc83f0101,
	0xfb800108,0xc8400101,0xfb800108,0xc8410101,0xfb800108,0xc8420101,0xfb800108,0xc8430101,
	0xfb800108,0xc8440101,0xfb800108,0xc8450101,0xfb800108,0xc8460101,0xfb800108,0xc8470101,
	0xfb800108,0xc8480101,0xfb800108,0xc8490101,0xfb800108,0xc84a0101,0xfb800108,0xc84b0101,
	0xfb800108,0xc84c0101,0xfb800108,0xc84d0101,0xfb800108,0xc84e0101,0xfb800108,0xc84f0101,
	0xfb800108,0xc8500101,0xfb800108,0xc8510101,0xfb800108,0xc8520101,0xfb800108,0xc8530101,
	0xfb800108,0xc8540101,0xfb800108,0xc8550101,0xfb800108,0xc8560101,0xfb800108,0xc8570101,
	0xfb800108,0xc8580101,0xfb800108,0xc8590101,0xfb800108,0xc85a0101,0xfb800108,0xc85b0101,
	0xfb800108,0xc85c0101,0xfb800108,0xc85d0101,0xfb800108,0xc85e0101,0xfb800108,0xc85f0101,
	0xfb800108,0xc8600101,0xfb800108,0xc8610101,0xfb800108,0xc8620101,0xfb800108,0xc8630101,
	0xfb800108,0xc8640101,0xfb800108,0xc8650101,0xfb800108,0xc8660101,0xfb800108,0xc8670101,
	0xfb800108,0xc8680101,0xfb800108,0xc8690101,0xfb800108,0xc86a0101,0xfb800108,0xc86b0101,
	0xfb800108,0xc86c0101,0xfb800108,0xc86d0101,0xfb800108,0xc86e0101,0xfb800108,0xc86f0101,
	0xfb800108,0xc8700101,0xfb800108,0xc8710101,0xfb800108,0xc8720101,0xfb800108,0xc8730101,
	0xfb800108,0xc8740101,0xfb800108,0xc8750101,0xfb800108,0xc8760101,0xfb800108,0xc8770101,
	0xfb800108,0xc8780101,0xfb800108,0xc8790101,0xfb800108,0xc87a0101,0xfb800108,0xc87b0101,
	0xfb800108,0xc87c0101,0xfb800108,0xc87d0101,0xfb800108,0xc87e0101,0xfb800108,0xc87f0101,
	0xfb800108,0xc8800101,0xfb800108,0xc8810101,0xfb800108,0xc8820101,0xfb800108,0xc8830101,
	0xfb800108,0xc8840101,0xfb800108,0xc8850101,0xfb800108,0xc8860101,0xfb800108,0xc8870101,
	0xfb800108,0xc8880101,0xfb800108,0xc8890101,0xfb800108,0xc88a0101,0xfb800108,0xc88b0101,
	0xfb800108,0xc88c0101,0xfb800108,0xc88d0101,0xfb800108,0xc88e0101,0xfb800108,0xc88f0101,
	0xfb800108,0xc8900101,0xfb800108,0xc8910101,0xfb800108,0xc8920101,0xfb800108,0xc8930101,
	0xfb800108,0xc8940101,0xfb800108,0xc8950101,0xfb800108,0xc8960101,0xfb800108,0xc8970101,
	0xfb800108,0xc8980101,0xfb800108,0xc8990101,0xfb800108,0xc89a0101,0xfb800108,0xc89b0101,
	0xfb800108,0xc89c0101,0xfb800108,0xc89d0101,0xfb800108,0xc89e0101,0xfb800108,0xc89f0101,
	0xfb800108,0xc8a00101,0xfb800108,0xc8a10101,0xfb800108,0xc8a20101,0xfb800108,0xc8a30101,
	0xfb800108,0xc8a40101,0xfb800108,0xc8a50101,0xfb800108,0xc8a60101,0xfb800108,0xc8a70101,
	0xfb800108,0xc8a80101,0xfb800108,0xc8a90101,0xfb800108,0xc8aa0101,0xfb800108,0xc8ab0101,
	0xfb800108,0xc8ac0101,0xfb800108,0xc8ad0101,0xfb800108,0xc8ae0101,0xfb800108,0xc8af0101,
	0xfb800108,0xc8b00101,0xfb800108,0xc8b10101,0xfb800108,0xc8b20101,0xfb800108,0xc8b30101,
	0xfb800108,0xc8b40101,0xfb800108,0xc8b50101,0xfb800108,0xc8b60101,0xfb800108,0xc8b70101,
	0xfb800108,0xc8b80101,0xfb800108,0xc8b90101,0xfb800108,0xc8ba0101,0xfb800108,0xc8bb0101,
	0xfb800108,0xc8bc0101,0xfb800108,0xc8bd0101,0xfb800108,0xc8be0101,0xfb800108,0xc8bf0101,
	0xfb800108,0xc8c00101,0xfb800108,0xc8c10101,0xfb800108,0xc8c20101,0xfb800108,0xc8c30101,
	0xfb800108,0xc8c40101,0xfb800108,0xc8c50101,0xfb800108,0xc8c60101,0xfb800108,0xc8c70101,
	0xfb800108,0xc8c80101,0xfb800108,0xc8c90101,0xfb800108,0xc8ca0101,0xfb800108,0xc8cb0101,
	0xfb800108,0xc8cc0101,0xfb800108,0xc8cd0101,0xfb800108,0xc8ce0101,0xfb800108,0xc8cf0101,
	0xfb800108,0xc8d00101,0xfb800108,0xc8d10101,0xfb800108,0xc8d20101,0xfb800108,0xc8d30101,
	0xfb800108,0xc8d40101,0xfb800108,0xc8d50101,0xfb800108,0xc8d60101,0xfb800108,0xc8d70101,
	0xfb800108,0xc8d80101,0xfb800108,0xc8d90101,0xfb800108,0xc8da0101,0xfb800108,0xc8db0101,
	0xfb800108,0xc8dc0101,0xfb800108,0xc8dd0101,0xfb800108,0xc8de0101,0xfb800108,0xc8df0101,
	0xfb800108,0xc8e00101,0xfb800108,0xc8e10101,0xfb800108,0xc8e20101,0xfb800108,0xc8e30101,
	0xfb800108,0xc8e40101,0xfb800108,0xc8e50101,0xfb800108,0xc8e60101,0xfb800108,0xc8e70101,
	0xfb800108,0xc8e80101,0xfb800108,0xc8e90101,0xfb800108,0xc8ea0101,0xfb800108,0xc8eb0101,
	0xfb800108,0xc8ec0101,0xfb800108,0xc8ed0101,0xfb800108,0xc8ee0101,0xfb800108,0xc8ef0101,
	0xfb800108,0xc8f00101,0xfb800108,0xc8f10101,0xfb800108,0xc8f20101,0xfb800108,0xc8f30101,
	0xfb800108,0xc8f40101,0xfb800108,0xc8f50101,0xfb800108,0xc8f60101,0xfb800108,0xc8f70101,
	0xfb800108,0xc8f80101,0xfb800108,0xc8f90101,0xfb800108,0xc8fa0101,0xfb800108,0xc8fb0101,
	0xfb800108,0xc8fc0101,0xfb800108,0xc8fd0101,0xfb800108,0xc8fe0101,0xfb800108,0xc8ff0101,
	0xfb800108,0xc9000101,0xfb800108,0xc9010101,0xfb800108,0xc9020101,0xfb800108,0xc9030101,
	0xfb800108,0xc9040101,0xfb800108,0xc9050101,0xfb800108,0xc9060101,0xfb800108,0xc9070101,
	0xfb800108,0xc9080101,0xfb800108,0xc9090101,0xfb800108,0xc90a0101,0xfb800108,0xc90b0101,
	0xfb800108,0xc90c0101,0xfb800108,0xc90d0101,0xfb800108,0xc90e0101,0xfb800108,0xc90f0101,
	0xfb800108,0xc9100101,0xfb800108,0xc9110101,0xfb800108,0xc9120101,0xfb800108,0xc9130101,
	0xfb800108,0xc9140101,0xfb800108,0xc9150101,0xfb800108,0xc9160101,0xfb800108,0xc9170101,
	0xfb800108,0xc9180101,0xfb800108,0xc9190101,0xfb800108,0xc91a0101,0xfb800108,0xc91b0101,
	0xfb800108,0xc91c0101,0xfb800108,0xc91d0101,0xfb800108,0xc91e0101,0xfb800108,0xc91f0101,
	0xfb800108,0xc9200101,0xfb800108,0xc9210101,0xfb800108,0xc9220101,0xfb800108,0xc9230101,
	0xfb800108,0xc9240101,0xfb800108,0xc9250101,0xfb800108,0xc9260101,0xfb800108,0xc9270101,
	0xfb800108,0xc9280101,0xfb800108,0xc9290101,0xfb800108,0xc92a0101,0xfb800108,0xc92b0101,
	0xfb800108,0xc92c0101,0xfb800108,0xc92d0101,0xfb800108,0xc92e0101,0xfb800108,0xc92f0101,
	0xfb800108,0xc9300101,0xfb800108,0xc9310101,0xfb800108,0xc9320101,0xfb800108,0xc9330101,
	0xfb800108,0xc9340101,0xfb800108,0xc9350101,0xfb800108,0xc9360101,0xfb800108,0xc9370101,
	0xfb800108,0xc9380101,0xfb800108,0xc9390101,0xfb800108,0xc93a0101,0xfb800108,0xc93b0101,
	0xfb800108,0xc93c0101,0xfb800108,0xc93d0101,0xfb800108,0xc93e0101,0xfb800108,0xc93f0101,
	0xfb800108,0xc9400101,0xfb800108,0xc9410101,0xfb800108,0xc9420101,0xfb800108,0xc9430101,
	0xfb800108,0xc9440101,0xfb800108,0xc9450101,0xfb800108,0xc9460101,0xfb800108,0xc9470101,
	0xfb800108,0xc9480101,0xfb800108,0xc9490101,0xfb800108,0xc94a0101,0xfb800108,0xc94b0101,
	0xfb800108,0xc94c0101,0xfb800108,0xc94d0101,0xfb800108,0xc94e0101,0xfb800108,0xc94f0101,
	0xfb800108,0xc9500101,0xfb800108,0xc9510101,0xfb800108,0xc9520101,0xfb800108,0xc9530101,
	0xfb800108,0xc9540101,0xfb800108,0xc9550101,0xfb800108,0xc9560101,0xfb800108,0xc9570101,
	0xfb800108,0xc9580101,0xfb800108,0xc9590101,0xfb800108,0xc95a0101,0xfb800108,0xc95b0101,
	0xfb800108,0xc95c0101,0xfb800108,0xc95d0101,0xfb800108,0xc95e0101,0xfb800108,0xc95f0101,
	0xfb800108,0xc9600101,0xfb800108,0xc9610101,0xfb800108,0xc9620101,0xfb800108,0xc9630101,
	0xfb800108,0xc9640101,0xfb800108,0xc9650101,0xfb800108,0xc9660101,0xfb800108,0xc9670101,
	0xfb800108,0xc9680101,0xfb800108,0xc9690101,0xfb800108,0xc96a0101,0xfb800108,0xc96b0101,
	0xfb800108,0xc96c0101,0xfb800108,0xc96d0101,0xfb800108,0xc96e0101,0xfb800108,0xc96f0101,
	0xfb800108,0xc9700101,0xfb800108,0xc9710101,0xfb800108,0xc9720101,0xfb800108,0xc9730101,
	0xfb800108,0xc9740101,0xfb800108,0xc9750101,0xfb800108,0xc9760101,0xfb800108,0xc9770101,
	0xfb800108,0xc9780101,0xfb800108,0xc9790101,0xfb800108,0xc97a0101,0xfb800108,0xc97b0101,
	0xfb800108,0xc97c0101,0xfb800108,0xc97d0101,0xfb800108,0xc97e0101,0xfb800108,0xc97f0101,
	0xfb800108,0xc9800101,0xfb800108,0xc9810101,0xfb800108,0xc9820101,0xfb800108,0xc9830101,
	0xfb800108,0xc9840101,0xfb800108,0xc9850101,0xfb800108,0xc9860101,0xfb800108,0xc9870101,
	0xfb800108,0xc9880101,0xfb800108,0xc9890101,0xfb800108,0xc98a0101,0xfb800108,0xc98b0101,
	0xfb800108,0xc98c0101,0xfb800108,0xc98d0101,0xfb800108,0xc98e0101,0xfb800108,0xc98f0101,
	0xfb800108,0xc9900101,0xfb800108,0xc9910101,0xfb800108,0xc9920101,0xfb800108,0xc9930101,
	0xfb800108,0xc9940101,0xfb800108,0xc9950101,0xfb800108,0xc9960101,0xfb800108,0xc9970101,
	0xfb800108,0xc9980101,0xfb800108,0xc9990101,0xfb800108,0xc99a0101,0xfb800108,0xc99b0101,
	0xfb800108,0xc99c0101,0xfb800108,0xc99d0101,0xfb800108,0xc99e0101,0xfb800108,0xc99f0101,
	0xfb800108,0xc9a00101,0xfb800108,0xc9a10101,0xfb800108,0xc9a20101,0xfb800108,0xc9a30101,
	0xfb800108,0xc9a40101,0xfb800108,0xc9a50101,0xfb800108,0xc9a60101,0xfb800108,0xc9a70101,
	0xfb800108,0xc9a80101,0xfb800108,0xc9a90101,0xfb800108,0xc9aa0101,0xfb800108,0xc9ab0101,
	0xfb800108,0xc9ac0101,0xfb800108,0xc9ad0101,0xfb800108,0xc9ae0101,0xfb800108,0xc9af0101,
	0xfb800108,0xc9b00101,0xfb800108,0xc9b10101,0xfb800108,0xc9b20101,0xfb800108,0xc9b30101,
	0xfb800108,0xc9b40101,0xfb800108,0xc9b50101,0xfb800108,0xc9b60101,0xfb800108,0xc9b70101,
	0xfb800108,0xc9b80101,0xfb800108,0xc9b90101,0xfb800108,0xc9ba0101,0xfb800108,0xc9bb0101,
	0xfb800108,0xc9bc0101,0xfb800108,0xc9bd0101,0xfb800108,0xc9be0101,0xfb800108,0xc9bf0101,
	0xfb800108,0xc9c00101,0xfb800108,0xc9c10101,0xfb800108,0xc9c20101,0xfb800108,0xc9c30101,
	0xfb800108,0xc9c40101,0xfb800108,0xc9c50101,0xfb800108,0xc9c60101,0xfb800108,0xc9c70101,
	0xfb800108,0xc9c80101,0xfb800108,0xc9c90101,0xfb800108,0xc9ca0101,0xfb800108,0xc9cb0101,
	0xfb800108,0xc9cc0101,0xfb800108,0xc9cd0101,0xfb800108,0xc9ce0101,0xfb800108,0xc9cf0101,
	0xfb800108,0xc9d00101,0xfb800108,0xc9d10101,0xfb800108,0xc9d20101,0xfb800108,0xc9d30101,
	0xfb800108,0xc9d40101,0xfb800108,0xc9d50101,0xfb800108,0xc9d60101,0xfb800108,0xc9d70101,
	0xfb800108,0xc9d80101,0xfb800108,0xc9d90101,0xfb800108,0xc9da0101,0xfb800108,0xc9db0101,
	0xfb800108,0xc9dc0101,0xfb800108,0xc9dd0101,0xfb800108,0xc9de0101,0xfb800108,0xc9df0101,
	0xfb800108,0xc9e00101,0xfb800108,0xc9e10101,0xfb800108,0xc9e20101,0xfb800108,0xc9e30101,
	0xfb800108,0xc9e40101,0xfb800108,0xc9e50101,0xfb800108,0xc9e60101,0xfb800108,0xc9e70101,
	0xfb800108,0xc9e80101,0xfb800108,0xc9e90101,0xfb800108,0xc9ea0101,0xfb800108,0xc9eb0101,
	0xfb800108,0xc9ec0101,0xfb800108,0xc9ed0101,0xfb800108,0xc9ee0101,0xfb800108,0xc9ef0101,
	0xfb800108,0xc9f00101,0xfb800108,0xc9f10101,0xfb800108,0xc9f20101,0xfb800108,0xc9f30101,
	0xfb800108,0xc9f40101,0xfb800108,0xc9f50101,0xfb800108,0xc9f60101,0xfb800108,0xc9f70101,
	0xfb800108,0xc9f80101,0xfb800108,0xc9f90101,0xfb800108,0xc9fa0101,0xfb800108,0xc9fb0101,
	0xfb800108,0xc9fc0101,0xfb800108,0xc9fd0101,0xfb800108,0xc9fe0101,0xfb800108,0xc9ff0101,
	0xfb800108,0xca000101,0xfb800108,0xca010101,0xfb800108,0xca020101,0xfb800108,0xca030101,
	0xfb800108,0xca040101,0xfb800108,0xca050101,0xfb800108,0xca060101,0xfb800108,0xca070101,
	0xfb800108,0xca080101,0xfb800108,0xca090101,0xfb800108,0xca0a0101,0xfb800108,0xca0b0101,
	0xfb800108,0xca0c0101,0xfb800108,0xca0d0101,0xfb800108,0xca0e0101,0xfb800108,0xca0f0101,
	0xfb800108,0xca100101,0xfb800108,0xca110101,0xfb800108,0xca120101,0xfb800108,0xca130101,
	0xfb800108,0xca140101,0xfb800108,0xca150101,0xfb800108,0xca160101,0xfb800108,0xca170101,
	0xfb800108,0xca180101,0xfb800108,0xca190101,0xfb800108,0xca1a0101,0xfb800108,0xca1b0101,
	0xfb800108,0xca1c0101,0xfb800108,0xca1d0101,0xfb800108,0xca1e0101,0xfb800108,0xca1f0101,
	0xfb800108,0xca200101,0xfb800108,0xca210101,0xfb800108,0xca220101,0xfb800108,0xca230101,
	0xfb800108,0xca240101,0xfb800108,0xca250101,0xfb800108,0xca260101,0xfb800108,0xca270101,
	0xfb800108,0xca280101,0xfb800108,0xca290101,0xfb800108,0xca2a0101,0xfb800108,0xca2b0101,
	0xfb800108,0xca2c0101,0xfb800108,0xca2d0101,0xfb800108,0xca2e0101,0xfb800108,0xca2f0101,
	0xfb800108,0xca300101,0xfb800108,0xca310101,0xfb800108,0xca320101,0xfb800108,0xca330101,
	0xfb800108,0xca340101,0xfb800108,0xca350101,0xfb800108,0xca360101,0xfb800108,0xca370101,
	0xfb800108,0xca380101,0xfb800108,0xca390101,0xfb800108,0xca3a0101,0xfb800108,0xca3b0101,
	0xfb800108,0xca3c0101,0xfb800108,0xca3d0101,0xfb800108,0xca3e0101,0xfb800108,0xca3f0101,
	0xfb800108,0xca400101,0xfb800108,0xca410101,0xfb800108,0xca420101,0xfb800108,0xca430101,
	0xfb800108,0xca440101,0xfb800108,0xca450101,0xfb800108,0xca460101,0xfb800108,0xca470101,
	0xfb800108,0xca480101,0xfb800108,0xca490101,0xfb800108,0xca4a0101,0xfb800108,0xca4b0101,
	0xfb800108,0xca4c0101,0xfb800108,0xca4d0101,0xfb800108,0xca4e0101,0xfb800108,0xca4f0101,
	0xfb800108,0xca500101,0xfb800108,0xca510101,0xfb800108,0xca520101,0xfb800108,0xca530101,
	0xfb800108,0xca540101,0xfb800108,0xca550101,0xfb800108,0xca560101,0xfb800108,0xca570101,
	0xfb800108,0xca580101,0xfb800108,0xca590101,0xfb800108,0xca5a0101,0xfb800108,0xca5b0101,
	0xfb800108,0xca5c0101,0xfb800108,0xca5d0101,0xfb800108,0xca5e0101,0xfb800108,0xca5f0101,
	0xfb800108,0xca600101,0xfb800108,0xca610101,0xfb800108,0xca620101,0xfb800108,0xca630101,
	0xfb800108,0xca640101,0xfb800108,0xca650101,0xfb800108,0xca660101,0xfb800108,0xca670101,
	0xfb800108,0xca680101,0xfb800108,0xca690101,0xfb800108,0xca6a0101,0xfb800108,0xca6b0101,
	0xfb800108,0xca6c0101,0xfb800108,0xca6d0101,0xfb800108,0xca6e0101,0xfb800108,0xca6f0101,
	0xfb800108,0xca700101,0xfb800108,0xca710101,0xfb800108,0xca720101,0xfb800108,0xca730101,
	0xfb800108,0xca740101,0xfb800108,0xca750101,0xfb800108,0xca760101,0xfb800108,0xca770101,
	0xfb800108,0xca780101,0xfb800108,0xca790101,0xfb800108,0xca7a0101,0xfb800108,0xca7b0101,
	0xfb800108,0xca7c0101,0xfb800108,0xca7d0101,0xfb800108,0xca7e0101,0xfb800108,0xca7f0101,
	0xfb800108,0xca800101,0xfb800108,0xca810101,0xfb800108,0xca820101,0xfb800108,0xca830101,
	0xfb800108,0xca840101,0xfb800108,0xca850101,0xfb800108,0xca860101,0xfb800108,0xca870101,
	0xfb800108,0xca880101,0xfb800108,0xca890101,0xfb800108,0xca8a0101,0xfb800108,0xca8b0101,
	0xfb800108,0xca8c0101,0xfb800108,0xca8d0101,0xfb800108,0xca8e0101,0xfb800108,0xca8f0101,
	0xfb800108,0xca900101,0xfb800108,0xca910101,0xfb800108,0xca920101,0xfb800108,0xca930101,
	0xfb800108,0xca940101,0xfb800108,0xca950101,0xfb800108,0xca960101,0xfb800108,0xca970101,
	0xfb800108,0xca980101,0xfb800108,0xca990101,0xfb800108,0xca9a0101,0xfb800108,0xca9b0101,
	0xfb800108,0xca9c0101,0xfb800108,0xca9d0101,0xfb800108,0xca9e0101,0xfb800108,0xca9f0101,
	0xfb800108,0xcaa00101,0xfb800108,0xcaa10101,0xfb800108,0xcaa20101,0xfb800108,0xcaa30101,
	0xfb800108,0xcaa40101,0xfb800108,0xcaa50101,0xfb800108,0xcaa60101,0xfb800108,0xcaa70101,
	0xfb800108,0xcaa80101,0xfb800108,0xcaa90101,0xfb800108,0xcaaa0101,0xfb800108,0xcaab0101,
	0xfb800108,0xcaac0101,0xfb800108,0xcaad0101,0xfb800108,0xcaae0101,0xfb800108,0xcaaf0101,
	0xfb800108,0xcab00101,0xfb800108,0xcab10101,0xfb800108,0xcab20101,0xfb800108,0xcab30101,
	0xfb800108,0xcab40101,0xfb800108,0xcab50101,0xfb800108,0xcab60101,0xfb800108,0xcab70101,
	0xfb800108,0xcab80101,0xfb800108,0xcab90101,0xfb800108,0xcaba0101,0xfb800108,0xcabb0101,
	0xfb800108,0xcabc0101,0xfb800108,0xcabd0101,0xfb800108,0xcabe0101,0xfb800108,0xcabf0101,
	0xfb800108,0xcac00101,0xfb800108,0xcac10101,0xfb800108,0xcac20101,0xfb800108,0xcac30101,
	0xfb800108,0xcac40101,0xfb800108,0xcac50101,0xfb800108,0xcac60101,0xfb800108,0xcac70101,
	0xfb800108,0xcac80101,0xfb800108,0xcac90101,0xfb800108,0xcaca0101,0xfb800108,0xcacb0101,
	0xfb800108,0xcacc0101,0xfb800108,0xcacd0101,0xfb800108,0xcace0101,0xfb800108,0xcacf0101,
	0xfb800108,0xcad00101,0xfb800108,0xcad10101,0xfb800108,0xcad20101,0xfb800108,0xcad30101,
	0xfb800108,0xcad40101,0xfb800108,0xcad50101,0xfb800108,0xcad60101,0xfb800108,0xcad70101,
	0xfb800108,0xcad80101,0xfb800108,0xcad90101,0xfb800108,0xcada0101,0xfb800108,0xcadb0101,
	0xfb800108,0xcadc0101,0xfb800108,0xcadd0101,0xfb800108,0xcade0101,0xfb800108,0xcadf0101,
	0xfb800108,0xcae00101,0xfb800108,0xcae10101,0xfb800108,0xcae20101,0xfb800108,0xcae30101,
	0xfb800108,0xcae40101,0xfb800108,0xcae50101,0xfb800108,0xcae60101,0xfb800108,0xcae70101,
	0xfb800108,0xcae80101,0xfb800108,0xcae90101,0xfb800108,0xcaea0101,0xfb800108,0xcaeb0101,
	0xfb800108,0xcaec0101,0xfb800108,0xcaed0101,0xfb800108,0xcaee0101,0xfb800108,0xcaef0101,
	0xfb800108,0xcaf00101,0xfb800108,0xcaf10101,0xfb800108,0xcaf20101,0xfb800108,0xcaf30101,
	0xfb800108,0xcaf40101,0xfb800108,0xcaf50101,0xfb800108,0xcaf60101,0xfb800108,0xcaf70101,
	0xfb800108,0xcaf80101,0xfb800108,0xcaf90101,0xfb800108,0xcafa0101,0xfb800108,0xcafb0101,
	0xfb800108,0xcafc0101,0xfb800108,0xcafd0101,0xfb800108,0xcafe0101,0xfb800108,0xcaff0101,
	0xfb800108,0xcb000101,0xfb800108,0xcb010101,0xfb800108,0xcb020101,0xfb800108,0xcb030101,
	0xfb800108,0xcb040101,0xfb800108,0xcb050101,0xfb800108,0xcb060101,0xfb800108,0xcb070101,
	0xfb800108,0xcb080101,0xfb800108,0xcb090101,0xfb800108,0xcb0a0101,0xfb800108,0xcb0b0101,
	0xfb800108,0xcb0c0101,0xfb800108,0xcb0d0101,0xfb800108,0xcb0e0101,0xfb800108,0xcb0f0101,
	0xfb800108,0xcb100101,0xfb800108,0xcb110101,0xfb800108,0xcb120101,0xfb800108,0xcb130101,
	0xfb800108,0xcb140101,0xfb800108,0xcb150101,0xfb800108,0xcb160101,0xfb800108,0xcb170101,
	0xfb800108,0xcb180101,0xfb800108,0xcb190101,0xfb800108,0xcb1a0101,0xfb800108,0xcb1b0101,
	0xfb800108,0xcb1c0101,0xfb800108,0xcb1d0101,0xfb800108,0xcb1e0101,0xfb800108,0xcb1f0101,
	0xfb800108,0xcb200101,0xfb800108,0xcb210101,0xfb800108,0xcb220101,0xfb800108,0xcb230101,
	0xfb800108,0xcb240101,0xfb800108,0xcb250101,0xfb800108,0xcb260101,0xfb800108,0xcb270101,
	0xfb800108,0xcb280101,0xfb800108,0xcb290101,0xfb800108,0xcb2a0101,0xfb800108,0xcb2b0101,
	0xfb800108,0xcb2c0101,0xfb800108,0xcb2d0101,0xfb800108,0xcb2e0101,0xfb800108,0xcb2f0101,
	0xfb800108,0xcb300101,0xfb800108,0xcb310101,0xfb800108,0xcb320101,0xfb800108,0xcb330101,
	0xfb800108,0xcb340101,0xfb800108,0xcb350101,0xfb800108,0xcb360101,0xfb800108,0xcb370101,
	0xfb800108,0xcb380101,0xfb800108,0xcb390101,0xfb800108,0xcb3a0101,0xfb800108,0xcb3b0101,
	0xfb800108,0xcb3c0101,0xfb800108,0xcb3d0101,0xfb800108,0xcb3e0101,0xfb800108,0xcb3f0101,
	0xfb800108,0xcb400101,0xfb800108,0xcb410101,0xfb800108,0xcb420101,0xfb800108,0xcb430101,
	0xfb800108,0xcb440101,0xfb800108,0xcb450101,0xfb800108,0xcb460101,0xfb800108,0xcb470101,
	0xfb800108,0xcb480101,0xfb800108,0xcb490101,0xfb800108,0xcb4a0101,0xfb800108,0xcb4b0101,
	0xfb800108,0xcb4c0101,0xfb800108,0xcb4d0101,0xfb800108,0xcb4e0101,0xfb800108,0xcb4f0101,
	0xfb800108,0xcb500101,0xfb800108,0xcb510101,0xfb800108,0xcb520101,0xfb800108,0xcb530101,
	0xfb800108,0xcb540101,0xfb800108,0xcb550101,0xfb800108,0xcb560101,0xfb800108,0xcb570101,
	0xfb800108,0xcb580101,0xfb800108,0xcb590101,0xfb800108,0xcb5a0101,0xfb800108,0xcb5b0101,
	0xfb800108,0xcb5c0101,0xfb800108,0xcb5d0101,0xfb800108,0xcb5e0101,0xfb800108,0xcb5f0101,
	0xfb800108,0xcb600101,0xfb800108,0xcb610101,0xfb800108,0xcb620101,0xfb800108,0xcb630101,
	0xfb800108,0xcb640101,0xfb800108,0xcb650101,0xfb800108,0xcb660101,0xfb800108,0xcb670101,
	0xfb800108,0xcb680101,0xfb800108,0xcb690101,0xfb800108,0xcb6a0101,0xfb800108,0xcb6b0101,
	0xfb800108,0xcb6c0101,0xfb800108,0xcb6d0101,0xfb800108,0xcb6e0101,0xfb800108,0xcb6f0101,
	0xfb800108,0xcb700101,0xfb800108,0xcb710101,0xfb800108,0xcb720101,0xfb800108,0xcb730101,
	0xfb800108,0xcb740101,0xfb800108,0xcb750101,0xfb800108,0xcb760101,0xfb800108,0xcb770101,
	0xfb800108,0xcb780101,0xfb800108,0xcb790101,0xfb800108,0xcb7a0101,0xfb800108,0xcb7b0101,
	0xfb800108,0xcb7c0101,0xfb800108,0xcb7d0101,0xfb800108,0xcb7e0101,0xfb800108,0xcb7f0101,
	0xfb800108,0xcb800101,0xfb800108,0xcb810101,0xfb800108,0xcb820101,0xfb800108,0xcb830101,
	0xfb800108,0xcb840101,0xfb800108,0xcb850101,0xfb800108,0xcb860101,0xfb800108,0xcb870101,
	0xfb800108,0xcb880101,0xfb800108,0xcb890101,0xfb800108,0xcb8a0101,0xfb800108,0xcb8b0101,
	0xfb800108,0xcb8c0101,0xfb800108,0xcb8d0101,0xfb800108,0xcb8e0101,0xfb800108,0xcb8f0101,
	0xfb800108,0xcb900101,0xfb800108,0xcb910101,0xfb800108,0xcb920101,0xfb800108,0xcb930101,
	0xfb800108,0xcb940101,0xfb800108,0xcb950101,0xfb800108,0xcb960101,0xfb800108,0xcb970101,
	0xfb800108,0xcb980101,0xfb800108,0xcb990101,0xfb800108,0xcb9a0101,0xfb800108,0xcb9b0101,
	0xfb800108,0xcb9c0101,0xfb800108,0xcb9d0101,0xfb800108,0xcb9e0101,0xfb800108,0xcb9f0101,
	0xfb800108,0xcba00101,0xfb800108,0xcba10101,0xfb800108,0xcba20101,0xfb800108,0xcba30101,
	0xfb800108,0xcba40101,0xfb800108,0xcba50101,0xfb800108,0xcba60101,0xfb800108,0xcba70101,
	0xfb800108,0xcba80101,0xfb800108,0xcba90101,0xfb800108,0xcbaa0101,0xfb800108,0xcbab0101,
	0xfb800108,0xcbac0101,0xfb800108,0xcbad0101,0xfb800108,0xcbae0101,0xfb800108,0xcbaf0101,
	0xfb800108,0xcbb00101,0xfb800108,0xcbb10101,0xfb800108,0xcbb20101,0xfb800108,0xcbb30101,
	0xfb800108,0xcbb40101,0xfb800108,0xcbb50101,0xfb800108,0xcbb60101,0xfb800108,0xcbb70101,
	0xfb800108,0xcbb80101,0xfb800108,0xcbb90101,0xfb800108,0xcbba0101,0xfb800108,0xcbbb0101,
	0xfb800108,0xcbbc0101,0xfb800108,0xcbbd0101,0xfb800108,0xcbbe0101,0xfb800108,0xcbbf0101,
	0xfb800108,0xcbc00101,0xfb800108,0xcbc10101,0xfb800108,0xcbc20101,0xfb800108,0xcbc30101,
	0xfb800108,0xcbc40101,0xfb800108,0xcbc50101,0xfb800108,0xcbc60101,0xfb800108,0xcbc70101,
	0xfb800108,0xcbc80101,0xfb800108,0xcbc90101,0xfb800108,0xcbca0101,0xfb800108,0xcbcb0101,
	0xfb800108,0xcbcc0101,0xfb800108,0xcbcd0101,0xfb800108,0xcbce0101,0xfb800108,0xcbcf0101,
	0xfb800108,0xcbd00101,0xfb800108,0xcbd10101,0xfb800108,0xcbd20101,0xfb800108,0xcbd30101,
	0xfb800108,0xcbd40101,0xfb800108,0xcbd50101,0xfb800108,0xcbd60101,0xfb800108,0xcbd70101,
	0xfb800108,0xcbd80101,0xfb800108,0xcbd90101,0xfb800108,0xcbda0101,0xfb800108,0xcbdb0101,
	0xfb800108,0xcbdc0101,0xfb800108,0xcbdd0101,0xfb800108,0xcbde0101,0xfb800108,0xcbdf0101,
	0xfb800108,0xcbe00101,0xfb800108,0xcbe10101,0xfb800108,0xcbe20101,0xfb800108,0xcbe30101,
	0xfb800108,0xcbe40101,0xfb800108,0xcbe50101,0xfb800108,0xcbe60101,0xfb800108,0xcbe70101,
	0xfb800108,0xcbe80101,0xfb800108,0xcbe90101,0xfb800108,0xcbea0101,0xfb800108,0xcbeb0101,
	0xfb800108,0xcbec0101,0xfb800108,0xcbed0101,0xfb800108,0xcbee0101,0xfb800108,0xcbef0101,
	0xfb800108,0xcbf00101,0xfb800108,0xcbf10101,0xfb800108,0xcbf20101,0xfb800108,0xcbf30101,
	0xfb800108,0xcbf40101,0xfb800108,0xcbf50101,0xfb800108,0xcbf60101,0xfb800108,0xcbf70101,
	0xfb800108,0xcbf80101,0xfb800108,0xcbf90101,0xfb800108,0xcbfa0101,0xfb800108,0xcbfb0101,
	0xfb800108,0xcbfc0101,0xfb800108,0xcbfd0101,0xfb800108,0xcbfe0101,0xfb800108,0xcbff0101,
	0xfb800108,0xcc000101,0xfb800108,0xcc010101,0xfb800108,0xcc020101,0xfb800108,0xcc030101,
	0xfb800108,0xcc040101,0xfb800108,0xcc050101,0xfb800108,0xcc060101,0xfb800108,0xcc070101,
	0xfb800108,0xcc080101,0xfb800108,0xcc090101,0xfb800108,0xcc0a0101,0xfb800108,0xcc0b0101,
	0xfb800108,0xcc0c0101,0xfb800108,0xcc0d0101,0xfb800108,0xcc0e0101,0xfb800108,0xcc0f0101,
	0xfb800108,0xcc100101,0xfb800108,0xcc110101,0xfb800108,0xcc120101,0xfb800108,0xcc130101,
	0xfb800108,0xcc140101,0xfb800108,0xcc150101,0xfb800108,0xcc160101,0xfb800108,0xcc170101,
	0xfb800108,0xcc180101,0xfb800108,0xcc190101,0xfb800108,0xcc1a0101,0xfb800108,0xcc1b0101,
	0xfb800108,0xcc1c0101,0xfb800108,0xcc1d0101,0xfb800108,0xcc1e0101,0xfb800108,0xcc1f0101,
	0xfb800108,0xcc200101,0xfb800108,0xcc210101,0xfb800108,0xcc220101,0xfb800108,0xcc230101,
	0xfb800108,0xcc240101,0xfb800108,0xcc250101,0xfb800108,0xcc260101,0xfb800108,0xcc270101,
	0xfb800108,0xcc280101,0xfb800108,0xcc290101,0xfb800108,0xcc2a0101,0xfb800108,0xcc2b0101,
	0xfb800108,0xcc2c0101,0xfb800108,0xcc2d0101,0xfb800108,0xcc2e0101,0xfb800108,0xcc2f0101,
	0xfb800108,0xcc300101,0xfb800108,0xcc310101,0xfb800108,0xcc320101,0xfb800108,0xcc330101,
	0xfb800108,0xcc340101,0xfb800108,0xcc350101,0xfb800108,0xcc360101,0xfb800108,0xcc370101,
	0xfb800108,0xcc380101,0xfb800108,0xcc390101,0xfb800108,0xcc3a0101,0xfb800108,0xcc3b0101,
	0xfb800108,0xcc3c0101,0xfb800108,0xcc3d0101,0xfb800108,0xcc3e0101,0xfb800108,0xcc3f0101,
	0xfb800108,0xcc400101,0xfb800108,0xcc410101,0xfb800108,0xcc420101,0xfb800108,0xcc430101,
	0xfb800108,0xcc440101,0xfb800108,0xcc450101,0xfb800108,0xcc460101,0xfb800108,0xcc470101,
	0xfb800108,0xcc480101,0xfb800108,0xcc490101,0xfb800108,0xcc4a0101,0xfb800108,0xcc4b0101,
	0xfb800108,0xcc4c0101,0xfb800108,0xcc4d0101,0xfb800108,0xcc4e0101,0xfb800108,0xcc4f0101,
	0xfb800108,0xcc500101,0xfb800108,0xcc510101,0xfb800108,0xcc520101,0xfb800108,0xcc530101,
	0xfb800108,0xcc540101,0xfb800108,0xcc550101,0xfb800108,0xcc560101,0xfb800108,0xcc570101,
	0xfb800108,0xcc580101,0xfb800108,0xcc590101,0xfb800108,0xcc5a0101,0xfb800108,0xcc5b0101,
	0xfb800108,0xcc5c0101,0xfb800108,0xcc5d0101,0xfb800108,0xcc5e0101,0xfb800108,0xcc5f0101,
	0xfb800108,0xcc600101,0xfb800108,0xcc610101,0xfb800108,0xcc620101,0xfb800108,0xcc630101,
	0xfb800108,0xcc640101,0xfb800108,0xcc650101,0xfb800108,0xcc660101,0xfb800108,0xcc670101,
	0xfb800108,0xcc680101,0xfb800108,0xcc690101,0xfb800108,0xcc6a0101,0xfb800108,0xcc6b0101,
	0xfb800108,0xcc6c0101,0xfb800108,0xcc6d0101,0xfb800108,0xcc6e0101,0xfb800108,0xcc6f0101,
	0xfb800108,0xcc700101,0xfb800108,0xcc710101,0xfb800108,0xcc720101,0xfb800108,0xcc730101,
	0xfb800108,0xcc740101,0xfb800108,0xcc750101,0xfb800108,0xcc760101,0xfb800108,0xcc770101,
	0xfb800108,0xcc780101,0xfb800108,0xcc790101,0xfb800108,0xcc7a0101,0xfb800108,0xcc7b0101,
	0xfb800108,0xcc7c0101,0xfb800108,0xcc7d0101,0xfb800108,0xcc7e0101,0xfb800108,0xcc7f0101,
	0xfb800108,0xcc800101,0xfb800108,0xcc810101,0xfb800108,0xcc820101,0xfb800108,0xcc830101,
	0xfb800108,0xcc840101,0xfb800108,0xcc850101,0xfb800108,0xcc860101,0xfb800108,0xcc870101,
	0xfb800108,0xcc880101,0xfb800108,0xcc890101,0xfb800108,0xcc8a0101,0xfb800108,0xcc8b0101,
	0xfb800108,0xcc8c0101,0xfb800108,0xcc8d0101,0xfb800108,0xcc8e0101,0xfb800108,0xcc8f0101,
	0xfb800108,0xcc900101,0xfb800108,0xcc910101,0xfb800108,0xcc920101,0xfb800108,0xcc930101,
	0xfb800108,0xcc940101,0xfb800108,0xcc950101,0xfb800108,0xcc960101,0xfb800108,0xcc970101,
	0xfb800108,0xcc980101,0xfb800108,0xcc990101,0xfb800108,0xcc9a0101,0xfb800108,0xcc9b0101,
	0xfb800108,0xcc9c0101,0xfb800108,0xcc9d0101,0xfb800108,0xcc9e0101,0xfb800108,0xcc9f0101,
	0xfb800108,0xcca00101,0xfb800108,0xcca10101,0xfb800108,0xcca20101,0xfb800108,0xcca30101,
	0xfb800108,0xcca40101,0xfb800108,0xcca50101,0xfb800108,0xcca60101,0xfb800108,0xcca70101,
	0xfb800108,0xcca80101,0xfb800108,0xcca90101,0xfb800108,0xccaa0101,0xfb800108,0xccab0101,
	0xfb800108,0xccac0101,0xfb800108,0xccad0101,0xfb800108,0xccae0101,0xfb800108,0xccaf0101,
	0xfb800108,0xccb00101,0xfb800108,0xccb10101,0xfb800108,0xccb20101,0xfb800108,0xccb30101,
	0xfb800108,0xccb40101,0xfb800108,0xccb50101,0xfb800108,0xccb60101,0xfb800108,0xccb70101,
	0xfb800108,0xccb80101,0xfb800108,0xccb90101,0xfb800108,0xccba0101,0xfb800108,0xccbb0101,
	0xfb800108,0xccbc0101,0xfb800108,0xccbd0101,0xfb800108,0xccbe0101,0xfb800108,0xccbf0101,
	0xfb800108,0xccc00101,0xfb800108,0xccc10101,0xfb800108,0xccc20101,0xfb800108,0xccc30101,
	0xfb800108,0xccc40101,0xfb800108,0xccc50101,0xfb800108,0xccc60101,0xfb800108,0xccc70101,
	0xfb800108,0xccc80101,0xfb800108,0xccc90101,0xfb800108,0xccca0101,0xfb800108,0xcccb0101,
	0xfb800108,0xcccc0101,0xfb800108,0xcccd0101,0xfb800108,0xccce0101,0xfb800108,0xcccf0101,
	0xfb800108,0xccd00101,0xfb800108,0xccd10101,0xfb800108,0xccd20101,0xfb800108,0xccd30101,
	0xfb800108,0xccd40101,0xfb800108,0xccd50101,0xfb800108,0xccd60101,0xfb800108,0xccd70101,
	0xfb800108,0xccd80101,0xfb800108,0xccd90101,0xfb800108,0xccda0101,0xfb800108,0xccdb0101,
	0xfb800108,0xccdc0101,0xfb800108,0xccdd0101,0xfb800108,0xccde0101,0xfb800108,0xccdf0101,
	0xfb800108,0xcce00101,0xfb800108,0xcce10101,0xfb800108,0xcce20101,0xfb800108,0xcce30101,
	0xfb800108,0xcce40101,0xfb800108,0xcce50101,0xfb800108,0xcce60101,0xfb800108,0xcce70101,
	0xfb800108,0xcce80101,0xfb800108,0xcce90101,0xfb800108,0xccea0101,0xfb800108,0xcceb0101,
	0xfb800108,0xccec0101,0xfb800108,0xcced0101,0xfb800108,0xccee0101,0xfb800108,0xccef0101,
	0xfb800108,0xccf00101,0xfb800108,0xccf10101,0xfb800108,0xccf20101,0xfb800108,0xccf30101,
	0xfb800108,0xccf40101,0xfb800108,0xccf50101,0xfb800108,0xccf60101,0xfb800108,0xccf70101,
	0xfb800108,0xccf80101,0xfb800108,0xccf90101,0xfb800108,0xccfa0101,0xfb800108,0xccfb0101,
	0xfb800108,0xccfc0101,0xfb800108,0xccfd0101,0xfb800108,0xccfe0101,0xfb800108,0xccff0101,
	0xfb800108,0xcd000101,0xfb800108,0xcd010101,0xfb800108,0xcd020101,0xfb800108,0xcd030101,
	0xfb800108,0xcd040101,0xfb800108,0xcd050101,0xfb800108,0xcd060101,0xfb800108,0xcd070101,
	0xfb800108,0xcd080101,0xfb800108,0xcd090101,0xfb800108,0xcd0a0101,0xfb800108,0xcd0b0101,
	0xfb800108,0xcd0c0101,0xfb800108,0xcd0d0101,0xfb800108,0xcd0e0101,0xfb800108,0xcd0f0101,
	0xfb800108,0xcd100101,0xfb800108,0xcd110101,0xfb800108,0xcd120101,0xfb800108,0xcd130101,
	0xfb800108,0xcd140101,0xfb800108,0xcd150101,0xfb800108,0xcd160101,0xfb800108,0xcd170101,
	0xfb800108,0xcd180101,0xfb800108,0xcd190101,0xfb800108,0xcd1a0101,0xfb800108,0xcd1b0101,
	0xfb800108,0xcd1c0101,0xfb800108,0xcd1d0101,0xfb800108,0xcd1e0101,0xfb800108,0xcd1f0101,
	0xfb800108,0xcd200101,0xfb800108,0xcd210101,0xfb800108,0xcd220101,0xfb800108,0xcd230101,
	0xfb800108,0xcd240101,0xfb800108,0xcd250101,0xfb800108,0xcd260101,0xfb800108,0xcd270101,
	0xfb800108,0xcd280101,0xfb800108,0xcd290101,0xfb800108,0xcd2a0101,0xfb800108,0xcd2b0101,
	0xfb800108,0xcd2c0101,0xfb800108,0xcd2d0101,0xfb800108,0xcd2e0101,0xfb800108,0xcd2f0101,
	0xfb800108,0xcd300101,0xfb800108,0xcd310101,0xfb800108,0xcd320101,0xfb800108,0xcd330101,
	0xfb800108,0xcd340101,0xfb800108,0xcd350101,0xfb800108,0xcd360101,0xfb800108,0xcd370101,
	0xfb800108,0xcd380101,0xfb800108,0xcd390101,0xfb800108,0xcd3a0101,0xfb800108,0xcd3b0101,
	0xfb800108,0xcd3c0101,0xfb800108,0xcd3d0101,0xfb800108,0xcd3e0101,0xfb800108,0xcd3f0101,
	0xfb800108,0xcd400101,0xfb800108,0xcd410101,0xfb800108,0xcd420101,0xfb800108,0xcd430101,
	0xfb800108,0xcd440101,0xfb800108,0xcd450101,0xfb800108,0xcd460101,0xfb800108,0xcd470101,
	0xfb800108,0xcd480101,0xfb800108,0xcd490101,0xfb800108,0xcd4a0101,0xfb800108,0xcd4b0101,
	0xfb800108,0xcd4c0101,0xfb800108,0xcd4d0101,0xfb800108,0xcd4e0101,0xfb800108,0xcd4f0101,
	0xfb800108,0xcd500101,0xfb800108,0xcd510101,0xfb800108,0xcd520101,0xfb800108,0xcd530101,
	0xfb800108,0xcd540101,0xfb800108,0xcd550101,0xfb800108,0xcd560101,0xfb800108,0xcd570101,
	0xfb800108,0xcd580101,0xfb800108,0xcd590101,0xfb800108,0xcd5a0101,0xfb800108,0xcd5b0101,
	0xfb800108,0xcd5c0101,0xfb800108,0xcd5d0101,0xfb800108,0xcd5e0101,0xfb800108,0xcd5f0101,
	0xfb800108,0xcd600101,0xfb800108,0xcd610101,0xfb800108,0xcd620101,0xfb800108,0xcd630101,
	0xfb800108,0xcd640101,0xfb800108,0xcd650101,0xfb800108,0xcd660101,0xfb800108,0xcd670101,
	0xfb800108,0xcd680101,0xfb800108,0xcd690101,0xfb800108,0xcd6a0101,0xfb800108,0xcd6b0101,
	0xfb800108,0xcd6c0101,0xfb800108,0xcd6d0101,0xfb800108,0xcd6e0101,0xfb800108,0xcd6f0101,
	0xfb800108,0xcd700101,0xfb800108,0xcd710101,0xfb800108,0xcd720101,0xfb800108,0xcd730101,
	0xfb800108,0xcd740101,0xfb800108,0xcd750101,0xfb800108,0xcd760101,0xfb800108,0xcd770101,
	0xfb800108,0xcd780101,0xfb800108,0xcd790101,0xfb800108,0xcd7a0101,0xfb800108,0xcd7b0101,
	0xfb800108,0xcd7c0101,0xfb800108,0xcd7d0101,0xfb800108,0xcd7e0101,0xfb800108,0xcd7f0101,
	0xfb800108,0xcd800101,0xfb800108,0xcd810101,0xfb800108,0xcd820101,0xfb800108,0xcd830101,
	0xfb800108,0xcd840101,0xfb800108,0xcd850101,0xfb800108,0xcd860101,0xfb800108,0xcd870101,
	0xfb800108,0xcd880101,0xfb800108,0xcd890101,0xfb800108,0xcd8a0101,0xfb800108,0xcd8b0101,
	0xfb800108,0xcd8c0101,0xfb800108,0xcd8d0101,0xfb800108,0xcd8e0101,0xfb800108,0xcd8f0101,
	0xfb800108,0xcd900101,0xfb800108,0xcd910101,0xfb800108,0xcd920101,0xfb800108,0xcd930101,
	0xfb800108,0xcd940101,0xfb800108,0xcd950101,0xfb800108,0xcd960101,0xfb800108,0xcd970101,
	0xfb800108,0xcd980101,0xfb800108,0xcd990101,0xfb800108,0xcd9a0101,0xfb800108,0xcd9b0101,
	0xfb800108,0xcd9c0101,0xfb800108,0xcd9d0101,0xfb800108,0xcd9e0101,0xfb800108,0xcd9f0101,
	0xfb800108,0xcda00101,0xfb800108,0xcda10101,0xfb800108,0xcda20101,0xfb800108,0xcda30101,
	0xfb800108,0xcda40101,0xfb800108,0xcda50101,0xfb800108,0xcda60101,0xfb800108,0xcda70101,
	0xfb800108,0xcda80101,0xfb800108,0xcda90101,0xfb800108,0xcdaa0101,0xfb800108,0xcdab0101,
	0xfb800108,0xcdac0101,0xfb800108,0xcdad0101,0xfb800108,0xcdae0101,0xfb800108,0xcdaf0101,
	0xfb800108,0xcdb00101,0xfb800108,0xcdb10101,0xfb800108,0xcdb20101,0xfb800108,0xcdb30101,
	0xfb800108,0xcdb40101,0xfb800108,0xcdb50101,0xfb840108,0x80000101,0xfb840108,0x80010101,
	0xfb840108,0x80020101,0xfb840108,0x80030101,0xfb840108,0x80040101,0xfb840108,0x80050101,
	0xfb840108,0x80060101,0xfb840108,0x80070101,0xfb840108,0x80080101,0xfb840108,0x80090101,
	0xfb840108,0x800a0101,0xfb840108,0x800b0101,0xfb840108,0x800c0101,0xfb840108,0x800d0101,
	0xfb840108,0x800e0101,0xfb840108,0x800f0101,0xfb840108,0x80100101,0xfb840108,0x80110101,
	0xfb840108,0x80120101,0xfb840108,0x80130101,0xfb840108,0x80140101,0xfb840108,0x80150101,
	0xfb840108,0x80160101,0xfb840108,0x80170101,0xfb840108,0x80180101,0xfb840108,0x80190101,
	0xfb840108,0x801a0101,0xfb840108,0x801b0101,0xfb840108,0x801c0101,0xfb840108,0x801d0101,
	0xfb840108,0x801e0101,0xfb840108,0x801f0101,0xfb840108,0x80200101,0xfb840108,0x80210101,
	0xfb840108,0x80220101,0xfb840108,0x80230101,0xfb840108,0x80240101,0xfb840108,0x80250101,
	0xfb840108,0x80260101,0xfb840108,0x80270101,0xfb840108,0x80280101,0xfb840108,0x80290101,
	0xfb840108,0x802a0101,0xfb840108,0x802b0101,0xfb840108,0x802c0101,0xfb840108,0x802d0101,
	0xfb840108,0x802e0101,0xfb840108,0x802f0101,0xfb840108,0x80300101,0xfb840108,0x80310101,
	0xfb840108,0x80320101,0xfb840108,0x80330101,0xfb840108,0x80340101,0xfb840108,0x80350101,
	0xfb840108,0x80360101,0xfb840108,0x80370101,0xfb840108,0x80380101,0xfb840108,0x80390101,
	0xfb840108,0x803a0101,0xfb840108,0x803b0101,0xfb840108,0x803c0101,0xfb840108,0x803d0101,
	0xfb840108,0x803e0101,0xfb840108,0x803f0101,0xfb840108,0x80400101,0xfb840108,0x80410101,
	0xfb840108,0x80420101,0xfb840108,0x80430101,0xfb840108,0x80440101,0xfb840108,0x80450101,
	0xfb840108,0x80460101,0xfb840108,0x80470101,0xfb840108,0x80480101,0xfb840108,0x80490101,
	0xfb840108,0x804a0101,0xfb840108,0x804b0101,0xfb840108,0x804c0101,0xfb840108,0x804d0101,
	0xfb840108,0x804e0101,0xfb840108,0x804f0101,0xfb840108,0x80500101,0xfb840108,0x80510101,
	0xfb840108,0x80520101,0xfb840108,0x80530101,0xfb840108,0x80540101,0xfb840108,0x80550101,
	0xfb840108,0x80560101,0xfb840108,0x80570101,0xfb840108,0x80580101,0xfb840108,0x80590101,
	0xfb840108,0x805a0101,0xfb840108,0x805b0101,0xfb840108,0x805c0101,0xfb840108,0x805d0101,
	0xfb840108,0x805e0101,0xfb840108,0x805f0101,0xfb840108,0x80600101,0xfb840108,0x80610101,
	0xfb840108,0x80620101,0xfb840108,0x80630101,0xfb840108,0x80640101,0xfb840108,0x80650101,
	0xfb840108,0x80660101,0xfb840108,0x80670101,0xfb840108,0x80680101,0xfb840108,0x80690101,
	0xfb840108,0x806a0101,0xfb840108,0x806b0101,0xfb840108,0x806c0101,0xfb840108,0x806d0101,
	0xfb840108,0x806e0101,0xfb840108,0x806f0101,0xfb840108,0x80700101,0xfb840108,0x80710101,
	0xfb840108,0x80720101,0xfb840108,0x80730101,0xfb840108,0x80740101,0xfb840108,0x80750101,
	0xfb840108,0x80760101,0xfb840108,0x80770101,0xfb840108,0x80780101,0xfb840108,0x80790101,
	0xfb840108,0x807a0101,0xfb840108,0x807b0101,0xfb840108,0x807c0101,0xfb840108,0x807d0101,
	0xfb840108,0x807e0101,0xfb840108,0x807f0101,0xfb840108,0x80800101,0xfb840108,0x80810101,
	0xfb840108,0x80820101,0xfb840108,0x80830101,0xfb840108,0x80840101,0xfb840108,0x80850101,
	0xfb840108,0x80860101,0xfb840108,0x80870101,0xfb840108,0x80880101,0xfb840108,0x80890101,
	0xfb840108,0x808a0101,0xfb840108,0x808b0101,0xfb840108,0x808c0101,0xfb840108,0x808d0101,
	0xfb840108,0x808e0101,0xfb840108,0x808f0101,0xfb840108,0x80900101,0xfb840108,0x80910101,
	0xfb840108,0x80920101,0xfb840108,0x80930101,0xfb840108,0x80940101,0xfb840108,0x80950101,
	0xfb840108,0x80960101,0xfb840108,0x80970101,0xfb840108,0x80980101,0xfb840108,0x80990101,
	0xfb840108,0x809a0101,0xfb840108,0x809b0101,0xfb840108,0x809c0101,0xfb840108,0x809d0101,
	0xfb840108,0x809e0101,0xfb840108,0x809f0101,0xfb840108,0x80a00101,0xfb840108,0x80a10101,
	0xfb840108,0x80a20101,0xfb840108,0x80a30101,0xfb840108,0x80a40101,0xfb840108,0x80a50101,
	0xfb840108,0x80a60101,0xfb840108,0x80a70101,0xfb840108,0x80a80101,0xfb840108,0x80a90101,
	0xfb840108,0x80aa0101,0xfb840108,0x80ab0101,0xfb840108,0x80ac0101,0xfb840108,0x80ad0101,
	0xfb840108,0x80ae0101,0xfb840108,0x80af0101,0xfb840108,0x80b00101,0xfb840108,0x80b10101,
	0xfb840108,0x80b20101,0xfb840108,0x80b30101,0xfb840108,0x80b40101,0xfb840108,0x80b50101,
	0xfb840108,0x80b60101,0xfb840108,0x80b70101,0xfb840108,0x80b80101,0xfb840108,0x80b90101,
	0xfb840108,0x80ba0101,0xfb840108,0x80bb0101,0xfb840108,0x80bc0101,0xfb840108,0x80bd0101,
	0xfb840108,0x80be0101,0xfb840108,0x80bf0101,0xfb840108,0x80c00101,0xfb840108,0x80c10101,
	0xfb840108,0x80c20101,0xfb840108,0x80c30101,0xfb840108,0x80c40101,0xfb840108,0x80c50101,
	0xfb840108,0x80c60101,0xfb840108,0x80c70101,0xfb840108,0x80c80101,0xfb840108,0x80c90101,
	0xfb840108,0x80ca0101,0xfb840108,0x80cb0101,0xfb840108,0x80cc0101,0xfb840108,0x80cd0101,
	0xfb840108,0x80ce0101,0xfb840108,0x80cf0101,0xfb840108,0x80d00101,0xfb840108,0x80d10101,
	0xfb840108,0x80d20101,0xfb840108,0x80d30101,0xfb840108,0x80d40101,0xfb840108,0x80d50101,
	0xfb840108,0x80d60101,0xfb840108,0x80d70101,0xfb840108,0x80d80101,0xfb840108,0x80d90101,
	0xfb840108,0x80da0101,0xfb840108,0x80db0101,0xfb840108,0x80dc0101,0xfb840108,0x80dd0101,
	0xfb840108,0x80de0101,0xfb840108,0x80df0101,0xfb840108,0x80e00101,0xfb840108,0x80e10101,
	0xfb840108,0x80e20101,0xfb840108,0x80e30101,0xfb840108,0x80e40101,0xfb840108,0x80e50101,
	0xfb840108,0x80e60101,0xfb840108,0x80e70101,0xfb840108,0x80e80101,0xfb840108,0x80e90101,
	0xfb840108,0x80ea0101,0xfb840108,0x80eb0101,0xfb840108,0x80ec0101,0xfb840108,0x80ed0101,
	0xfb840108,0x80ee0101,0xfb840108,0x80ef0101,0xfb840108,0x80f00101,0xfb840108,0x80f10101,
	0xfb840108,0x80f20101,0xfb840108,0x80f30101,0xfb840108,0x80f40101,0xfb840108,0x80f50101,
	0xfb840108,0x80f60101,0xfb840108,0x80f70101,0xfb840108,0x80f80101,0xfb840108,0x80f90101,
	0xfb840108,0x80fa0101,0xfb840108,0x80fb0101,0xfb840108,0x80fc0101,0xfb840108,0x80fd0101,
	0xfb840108,0x80fe0101,0xfb840108,0x80ff0101,0xfb840108,0x81000101,0xfb840108,0x81010101,
	0xfb840108,0x81020101,0xfb840108,0x81030101,0xfb840108,0x81040101,0xfb840108,0x81050101,
	0xfb840108,0x81060101,0xfb840108,0x81070101,0xfb840108,0x81080101,0xfb840108,0x81090101,
	0xfb840108,0x810a0101,0xfb840108,0x810b0101,0xfb840108,0x810c0101,0xfb840108,0x810d0101,
	0xfb840108,0x810e0101,0xfb840108,0x810f0101,0xfb840108,0x81100101,0xfb840108,0x81110101,
	0xfb840108,0x81120101,0xfb840108,0x81130101,0xfb840108,0x81140101,0xfb840108,0x81150101,
	0xfb840108,0x81160101,0xfb840108,0x81170101,0xfb840108,0x81180101,0xfb840108,0x81190101,
	0xfb840108,0x811a0101,0xfb840108,0x811b0101,0xfb840108,0x811c0101,0xfb840108,0x811d0101,
	0xfb840108,0x811e0101,0xfb840108,0x811f0101,0xfb840108,0x81200101,0xfb840108,0x81210101,
	0xfb840108,0x81220101,0xfb840108,0x81230101,0xfb840108,0x81240101,0xfb840108,0x81250101,
	0xfb840108,0x81260101,0xfb840108,0x81270101,0xfb840108,0x81280101,0xfb840108,0x81290101,
	0xfb840108,0x812a0101,0xfb840108,0x812b0101,0xfb840108,0x812c0101,0xfb840108,0x812d0101,
	0xfb840108,0x812e0101,0xfb840108,0x812f0101,0xfb840108,0x81300101,0xfb840108,0x81310101,
	0xfb840108,0x81320101,0xfb840108,0x81330101,0xfb840108,0x81340101,0xfb840108,0x81350101,
	0xfb840108,0x81360101,0xfb840108,0x81370101,0xfb840108,0x81380101,0xfb840108,0x81390101,
	0xfb840108,0x813a0101,0xfb840108,0x813b0101,0xfb840108,0x813c0101,0xfb840108,0x813d0101,
	0xfb840108,0x813e0101,0xfb840108,0x813f0101,0xfb840108,0x81400101,0xfb840108,0x81410101,
	0xfb840108,0x81420101,0xfb840108,0x81430101,0xfb840108,0x81440101,0xfb840108,0x81450101,
	0xfb840108,0x81460101,0xfb840108,0x81470101,0xfb840108,0x81480101,0xfb840108,0x81490101,
	0xfb840108,0x814a0101,0xfb840108,0x814b0101,0xfb840108,0x814c0101,0xfb840108,0x814d0101,
	0xfb840108,0x814e0101,0xfb840108,0x814f0101,0xfb840108,0x81500101,0xfb840108,0x81510101,
	0xfb840108,0x81520101,0xfb840108,0x81530101,0xfb840108,0x81540101,0xfb840108,0x81550101,
	0xfb840108,0x81560101,0xfb840108,0x81570101,0xfb840108,0x81580101,0xfb840108,0x81590101,
	0xfb840108,0x815a0101,0xfb840108,0x815b0101,0xfb840108,0x815c0101,0xfb840108,0x815d0101,
	0xfb840108,0x815e0101,0xfb840108,0x815f0101,0xfb840108,0x81600101,0xfb840108,0x81610101,
	0xfb840108,0x81620101,0xfb840108,0x81630101,0xfb840108,0x81640101,0xfb840108,0x81650101,
	0xfb840108,0x81660101,0xfb840108,0x81670101,0xfb840108,0x81680101,0xfb840108,0x81690101,
	0xfb840108,0x816a0101,0xfb840108,0x816b0101,0xfb840108,0x816c0101,0xfb840108,0x816d0101,
	0xfb840108,0x816e0101,0xfb840108,0x816f0101,0xfb840108,0x81700101,0xfb840108,0x81710101,
	0xfb840108,0x81720101,0xfb840108,0x81730101,0xfb840108,0x81740101,0xfb840108,0x81750101,
	0xfb840108,0x81760101,0xfb840108,0x81770101,0xfb840108,0x81780101,0xfb840108,0x81790101,
	0xfb840108,0x817a0101,0xfb840108,0x817b0101,0xfb840108,0x817c0101,0xfb840108,0x817d0101,
	0xfb840108,0x817e0101,0xfb840108,0x817f0101,0xfb840108,0x81800101,0xfb840108,0x81810101,
	0xfb840108,0x81820101,0xfb840108,0x81830101,0xfb840108,0x81840101,0xfb840108,0x81850101,
	0xfb840108,0x81860101,0xfb840108,0x81870101,0xfb840108,0x81880101,0xfb840108,0x81890101,
	0xfb840108,0x818a0101,0xfb840108,0x818b0101,0xfb840108,0x818c0101,0xfb840108,0x818d0101,
	0xfb840108,0x818e0101,0xfb840108,0x818f0101,0xfb840108,0x81900101,0xfb840108,0x81910101,
	0xfb840108,0x81920101,0xfb840108,0x81930101,0xfb840108,0x81940101,0xfb840108,0x81950101,
	0xfb840108,0x81960101,0xfb840108,0x81970101,0xfb840108,0x81980101,0xfb840108,0x81990101,
	0xfb840108,0x819a0101,0xfb840108,0x819b0101,0xfb840108,0x819c0101,0xfb840108,0x819d0101,
	0xfb840108,0x819e0101,0xfb840108,0x819f0101,0xfb840108,0x81a00101,0xfb840108,0x81a10101,
	0xfb840108,0x81a20101,0xfb840108,0x81a30101,0xfb840108,0x81a40101,0xfb840108,0x81a50101,
	0xfb840108,0x81a60101,0xfb840108,0x81a70101,0xfb840108,0x81a80101,0xfb840108,0x81a90101,
	0xfb840108,0x81aa0101,0xfb840108,0x81ab0101,0xfb840108,0x81ac0101,0xfb840108,0x81ad0101,
	0xfb840108,0x81ae0101,0xfb840108,0x81af0101,0xfb840108,0x81b00101,0xfb840108,0x81b10101,
	0xfb840108,0x81b20101,0xfb840108,0x81b30101,0xfb840108,0x81b40101,0xfb840108,0x81b50101,
	0xfb840108,0x81b60101,0xfb840108,0x81b70101,0xfb840108,0x81b80101,0xfb840108,0x81b90101,
	0xfb840108,0x81ba0101,0xfb840108,0x81bb0101,0xfb840108,0x81bc0101,0xfb840108,0x81bd0101,
	0xfb840108,0x81be0101,0xfb840108,0x81bf0101,0xfb840108,0x81c00101,0xfb840108,0x81c10101,
	0xfb840108,0x81c20101,0xfb840108,0x81c30101,0xfb840108,0x81c40101,0xfb840108,0x81c50101,
	0xfb840108,0x81c60101,0xfb840108,0x81c70101,0xfb840108,0x81c80101,0xfb840108,0x81c90101,
	0xfb840108,0x81ca0101,0xfb840108,0x81cb0101,0xfb840108,0x81cc0101,0xfb840108,0x81cd0101,
	0xfb840108,0x81ce0101,0xfb840108,0x81cf0101,0xfb840108,0x81d00101,0xfb840108,0x81d10101,
	0xfb840108,0x81d20101,0xfb840108,0x81d30101,0xfb840108,0x81d40101,0xfb840108,0x81d50101,
	0xfb840108,0x81d60101,0xfb840108,0x81d70101,0xfb840108,0x81d80101,0xfb840108,0x81d90101,
	0xfb840108,0x81da0101,0xfb840108,0x81db0101,0xfb840108,0x81dc0101,0xfb840108,0x81dd0101,
	0xfb840108,0x81de0101,0xfb840108,0x81df0101,0xfb840108,0x81e00101,0xfb840108,0x81e10101,
	0xfb840108,0x81e20101,0xfb840108,0x81e30101,0xfb840108,0x81e40101,0xfb840108,0x81e50101,
	0xfb840108,0x81e60101,0xfb840108,0x81e70101,0xfb840108,0x81e80101,0xfb840108,0x81e90101,
	0xfb840108,0x81ea0101,0xfb840108,0x81eb0101,0xfb840108,0x81ec0101,0xfb840108,0x81ed0101,
	0xfb840108,0x81ee0101,0xfb840108,0x81ef0101,0xfb840108,0x81f00101,0xfb840108,0x81f10101,
	0xfb840108,0x81f20101,0xfb840108,0x81f30101,0xfb840108,0x81f40101,0xfb840108,0x81f50101,
	0xfb840108,0x81f60101,0xfb840108,0x81f70101,0xfb840108,0x81f80101,0xfb840108,0x81f90101,
	0xfb840108,0x81fa0101,0xfb840108,0x81fb0101,0xfb840108,0x81fc0101,0xfb840108,0x81fd0101,
	0xfb840108,0x81fe0101,0xfb840108,0x81ff0101,0xfb840108,0x82000101,0xfb840108,0x82010101,
	0xfb840108,0x82020101,0xfb840108,0x82030101,0xfb840108,0x82040101,0xfb840108,0x82050101,
	0xfb840108,0x82060101,0xfb840108,0x82070101,0xfb840108,0x82080101,0xfb840108,0x82090101,
	0xfb840108,0x820a0101,0xfb840108,0x820b0101,0xfb840108,0x820c0101,0xfb840108,0x820d0101,
	0xfb840108,0x820e0101,0xfb840108,0x820f0101,0xfb840108,0x82100101,0xfb840108,0x82110101,
	0xfb840108,0x82120101,0xfb840108,0x82130101,0xfb840108,0x82140101,0xfb840108,0x82150101,
	0xfb840108,0x82160101,0xfb840108,0x82170101,0xfb840108,0x82180101,0xfb840108,0x82190101,
	0xfb840108,0x821a0101,0xfb840108,0x821b0101,0xfb840108,0x821c0101,0xfb840108,0x821d0101,
	0xfb840108,0x821e0101,0xfb840108,0x821f0101,0xfb840108,0x82200101,0xfb840108,0x82210101,
	0xfb840108,0x82220101,0xfb840108,0x82230101,0xfb840108,0x82240101,0xfb840108,0x82250101,
	0xfb840108,0x82260101,0xfb840108,0x82270101,0xfb840108,0x82280101,0xfb840108,0x82290101,
	0xfb840108,0x822a0101,0xfb840108,0x822b0101,0xfb840108,0x822c0101,0xfb840108,0x822d0101,
	0xfb840108,0x822e0101,0xfb840108,0x822f0101,0xfb840108,0x82300101,0xfb840108,0x82310101,
	0xfb840108,0x82320101,0xfb840108,0x82330101,0xfb840108,0x82340101,0xfb840108,0x82350101,
	0xfb840108,0x82360101,0xfb840108,0x82370101,0xfb840108,0x82380101,0xfb840108,0x82390101,
	0xfb840108,0x823a0101,0xfb840108,0x823b0101,0xfb840108,0x823c0101,0xfb840108,0x823d0101,
	0xfb840108,0x823e0101,0xfb840108,0x823f0101,0xfb840108,0x82400101,0xfb840108,0x82410101,
	0xfb840108,0x82420101,0xfb840108,0x82430101,0xfb840108,0x82440101,0xfb840108,0x82450101,
	0xfb840108,0x82460101,0xfb840108,0x82470101,0xfb840108,0x82480101,0xfb840108,0x82490101,
	0xfb840108,0x824a0101,0xfb840108,0x824b0101,0xfb840108,0x824c0101,0xfb840108,0x824d0101,
	0xfb840108,0x824e0101,0xfb840108,0x824f0101,0xfb840108,0x82500101,0xfb840108,0x82510101,
	0xfb840108,0x82520101,0xfb840108,0x82530101,0xfb840108,0x82540101,0xfb840108,0x82550101,
	0xfb840108,0x82560101,0xfb840108,0x82570101,0xfb840108,0x82580101,0xfb840108,0x82590101,
	0xfb840108,0x825a0101,0xfb840108,0x825b0101,0xfb840108,0x825c0101,0xfb840108,0x825d0101,
	0xfb840108,0x825e0101,0xfb840108,0x825f0101,0xfb840108,0x82600101,0xfb840108,0x82610101,
	0xfb840108,0x82620101,0xfb840108,0x82630101,0xfb840108,0x82640101,0xfb840108,0x82650101,
	0xfb840108,0x82660101,0xfb840108,0x82670101,0xfb840108,0x82680101,0xfb840108,0x82690101,
	0xfb840108,0x826a0101,0xfb840108,0x826b0101,0xfb840108,0x826c0101,0xfb840108,0x826d0101,
	0xfb840108,0x826e0101,0xfb840108,0x826f0101,0xfb840108,0x82700101,0xfb840108,0x82710101,
	0xfb840108,0x82720101,0xfb840108,0x82730101,0xfb840108,0x82740101,0xfb840108,0x82750101,
	0xfb840108,0x82760101,0xfb840108,0x82770101,0xfb840108,0x82780101,0xfb840108,0x82790101,
	0xfb840108,0x827a0101,0xfb840108,0x827b0101,0xfb840108,0x827c0101,0xfb840108,0x827d0101,
	0xfb840108,0x827e0101,0xfb840108,0x827f0101,0xfb840108,0x82800101,0xfb840108,0x82810101,
	0xfb840108,0x82820101,0xfb840108,0x82830101,0xfb840108,0x82840101,0xfb840108,0x82850101,
	0xfb840108,0x82860101,0xfb840108,0x82870101,0xfb840108,0x82880101,0xfb840108,0x82890101,
	0xfb840108,0x828a0101,0xfb840108,0x828b0101,0xfb840108,0x828c0101,0xfb840108,0x828d0101,
	0xfb840108,0x828e0101,0xfb840108,0x828f0101,0xfb840108,0x82900101,0xfb840108,0x82910101,
	0xfb840108,0x82920101,0xfb840108,0x82930101,0xfb840108,0x82940101,0xfb840108,0x82950101,
	0xfb840108,0x82960101,0xfb840108,0x82970101,0xfb840108,0x82980101,0xfb840108,0x82990101,
	0xfb840108,0x829a0101,0xfb840108,0x829b0101,0xfb840108,0x829c0101,0xfb840108,0x829d0101,
	0xfb840108,0x829e0101,0xfb840108,0x829f0101,0xfb840108,0x82a00101,0xfb840108,0x82a10101,
	0xfb840108,0x82a20101,0xfb840108,0x82a30101,0xfb840108,0x82a40101,0xfb840108,0x82a50101,
	0xfb840108,0x82a60101,0xfb840108,0x82a70101,0xfb840108,0x82a80101,0xfb840108,0x82a90101,
	0xfb840108,0x82aa0101,0xfb840108,0x82ab0101,0xfb840108,0x82ac0101,0xfb840108,0x82ad0101,
	0xfb840108,0x82ae0101,0xfb840108,0x82af0101,0xfb840108,0x82b00101,0xfb840108,0x82b10101,
	0xfb840108,0x82b20101,0xfb840108,0x82b30101,0xfb840108,0x82b40101,0xfb840108,0x82b50101,
	0xfb840108,0x82b60101,0xfb840108,0x82b70101,0xfb840108,0x82b80101,0xfb840108,0x82b90101,
	0xfb840108,0x82ba0101,0xfb840108,0x82bb0101,0xfb840108,0x82bc0101,0xfb840108,0x82bd0101,
	0xfb840108,0x82be0101,0xfb840108,0x82bf0101,0xfb840108,0x82c00101,0xfb840108,0x82c10101,
	0xfb840108,0x82c20101,0xfb840108,0x82c30101,0xfb840108,0x82c40101,0xfb840108,0x82c50101,
	0xfb840108,0x82c60101,0xfb840108,0x82c70101,0xfb840108,0x82c80101,0xfb840108,0x82c90101,
	0xfb840108,0x82ca0101,0xfb840108,0x82cb0101,0xfb840108,0x82cc0101,0xfb840108,0x82cd0101,
	0xfb840108,0x82ce0101,0xfb840108,0x82cf0101,0xfb840108,0x82d00101,0xfb840108,0x82d10101,
	0xfb840108,0x82d20101,0xfb840108,0x82d30101,0xfb840108,0x82d40101,0xfb840108,0x82d50101,
	0xfb840108,0x82d60101,0xfb840108,0x82d70101,0xfb840108,0x82d80101,0xfb840108,0x82d90101,
	0xfb840108,0x82da0101,0xfb840108,0x82db0101,0xfb840108,0x82dc0101,0xfb840108,0x82dd0101,
	0xfb840108,0x82de0101,0xfb840108,0x82df0101,0xfb840108,0x82e00101,0xfb840108,0x82e10101,
	0xfb840108,0x82e20101,0xfb840108,0x82e30101,0xfb840108,0x82e40101,0xfb840108,0x82e50101,
	0xfb840108,0x82e60101,0xfb840108,0x82e70101,0xfb840108,0x82e80101,0xfb840108,0x82e90101,
	0xfb840108,0x82ea0101,0xfb840108,0x82eb0101,0xfb840108,0x82ec0101,0xfb840108,0x82ed0101,
	0xfb840108,0x82ee0101,0xfb840108,0x82ef0101,0xfb840108,0x82f00101,0xfb840108,0x82f10101,
	0xfb840108,0x82f20101,0xfb840108,0x82f30101,0xfb840108,0x82f40101,0xfb840108,0x82f50101,
	0xfb840108,0x82f60101,0xfb840108,0x82f70101,0xfb840108,0x82f80101,0xfb840108,0x82f90101,
	0xfb840108,0x82fa0101,0xfb840108,0x82fb0101,0xfb840108,0x82fc0101,0xfb840108,0x82fd0101,
	0xfb840108,0x82fe0101,0xfb840108,0x82ff0101,0xfb840108,0x83000101,0xfb840108,0x83010101,
	0xfb840108,0x83020101,0xfb840108,0x83030101,0xfb840108,0x83040101,0xfb840108,0x83050101,
	0xfb840108,0x83060101,0xfb840108,0x83070101,0xfb840108,0x83080101,0xfb840108,0x83090101,
	0xfb840108,0x830a0101,0xfb840108,0x830b0101,0xfb840108,0x830c0101,0xfb840108,0x830d0101,
	0xfb840108,0x830e0101,0xfb840108,0x830f0101,0xfb840108,0x83100101,0xfb840108,0x83110101,
	0xfb840108,0x83120101,0xfb840108,0x83130101,0xfb840108,0x83140101,0xfb840108,0x83150101,
	0xfb840108,0x83160101,0xfb840108,0x83170101,0xfb840108,0x83180101,0xfb840108,0x83190101,
	0xfb840108,0x831a0101,0xfb840108,0x831b0101,0xfb840108,0x831c0101,0xfb840108,0x831d0101,
	0xfb840108,0x831e0101,0xfb840108,0x831f0101,0xfb840108,0x83200101,0xfb840108,0x83210101,
	0xfb840108,0x83220101,0xfb840108,0x83230101,0xfb840108,0x83240101,0xfb840108,0x83250101,
	0xfb840108,0x83260101,0xfb840108,0x83270101,0xfb840108,0x83280101,0xfb840108,0x83290101,
	0xfb840108,0x832a0101,0xfb840108,0x832b0101,0xfb840108,0x832c0101,0xfb840108,0x832d0101,
	0xfb840108,0x832e0101,0xfb840108,0x832f0101,0xfb840108,0x83300101,0xfb840108,0x83310101,
	0xfb840108,0x83320101,0xfb840108,0x83330101,0xfb840108,0x83340101,0xfb840108,0x83350101,
	0xfb840108,0x83360101,0xfb840108,0x83370101,0xfb840108,0x83380101,0xfb840108,0x83390101,
	0xfb840108,0x833a0101,0xfb840108,0x833b0101,0xfb840108,0x833c0101,0xfb840108,0x833d0101,
	0xfb840108,0x833e0101,0xfb840108,0x833f0101,0xfb840108,0x83400101,0xfb840108,0x83410101,
	0xfb840108,0x83420101,0xfb840108,0x83430101,0xfb840108,0x83440101,0xfb840108,0x83450101,
	0xfb840108,0x83460101,0xfb840108,0x83470101,0xfb840108,0x83480101,0xfb840108,0x83490101,
	0xfb840108,0x834a0101,0xfb840108,0x834b0101,0xfb840108,0x834c0101,0xfb840108,0x834d0101,
	0xfb840108,0x834e0101,0xfb840108,0x834f0101,0xfb840108,0x83500101,0xfb840108,0x83510101,
	0xfb840108,0x83520101,0xfb840108,0x83530101,0xfb840108,0x83540101,0xfb840108,0x83550101,
	0xfb840108,0x83560101,0xfb840108,0x83570101,0xfb840108,0x83580101,0xfb840108,0x83590101,
	0xfb840108,0x835a0101,0xfb840108,0x835b0101,0xfb840108,0x835c0101,0xfb840108,0x835d0101,
	0xfb840108,0x835e0101,0xfb840108,0x835f0101,0xfb840108,0x83600101,0xfb840108,0x83610101,
	0xfb840108,0x83620101,0xfb840108,0x83630101,0xfb840108,0x83640101,0xfb840108,0x83650101,
	0xfb840108,0x83660101,0xfb840108,0x83670101,0xfb840108,0x83680101,0xfb840108,0x83690101,
	0xfb840108,0x836a0101,0xfb840108,0x836b0101,0xfb840108,0x836c0101,0xfb840108,0x836d0101,
	0xfb840108,0x836e0101,0xfb840108,0x836f0101,0xfb840108,0x83700101,0xfb840108,0x83710101,
	0xfb840108,0x83720101,0xfb840108,0x83730101,0xfb840108,0x83740101,0xfb840108,0x83750101,
	0xfb840108,0x83760101,0xfb840108,0x83770101,0xfb840108,0x83780101,0xfb840108,0x83790101,
	0xfb840108,0x837a0101,0xfb840108,0x837b0101,0xfb840108,0x837c0101,0xfb840108,0x837d0101,
	0xfb840108,0x837e0101,0xfb840108,0x837f0101,0xfb840108,0x83800101,0xfb840108,0x83810101,
	0xfb840108,0x83820101,0xfb840108,0x83830101,0xfb840108,0x83840101,0xfb840108,0x83850101,
	0xfb840108,0x83860101,0xfb840108,0x83870101,0xfb840108,0x83880101,0xfb840108,0x83890101,
	0xfb840108,0x838a0101,0xfb840108,0x838b0101,0xfb840108,0x838c0101,0xfb840108,0x838d0101,
	0xfb840108,0x838e0101,0xfb840108,0x838f0101,0xfb840108,0x83900101,0xfb840108,0x83910101,
	0xfb840108,0x83920101,0xfb840108,0x83930101,0xfb840108,0x83940101,0xfb840108,0x83950101,
	0xfb840108,0x83960101,0xfb840108,0x83970101,0xfb840108,0x83980101,0xfb840108,0x83990101,
	0xfb840108,0x839a0101,0xfb840108,0x839b0101,0xfb840108,0x839c0101,0xfb840108,0x839d0101,
	0xfb840108,0x839e0101,0xfb840108,0x839f0101,0xfb840108,0x83a00101,0xfb840108,0x83a10101,
	0xfb840108,0x83a20101,0xfb840108,0x83a30101,0xfb840108,0x83a40101,0xfb840108,0x83a50101,
	0xfb840108,0x83a60101,0xfb840108,0x83a70101,0xfb840108,0x83a80101,0xfb840108,0x83a90101,
	0xfb840108,0x83aa0101,0xfb840108,0x83ab0101,0xfb840108,0x83ac0101,0xfb840108,0x83ad0101,
	0xfb840108,0x83ae0101,0xfb840108,0x83af0101,0xfb840108,0x83b00101,0xfb840108,0x83b10101,
	0xfb840108,0x83b20101,0xfb840108,0x83b30101,0xfb840108,0x83b40101,0xfb840108,0x83b50101,
	0xfb840108,0x83b60101,0xfb840108,0x83b70101,0xfb840108,0x83b80101,0xfb840108,0x83b90101,
	0xfb840108,0x83ba0101,0xfb840108,0x83bb0101,0xfb840108,0x83bc0101,0xfb840108,0x83bd0101,
	0xfb840108,0x83be0101,0xfb840108,0x83bf0101,0xfb840108,0x83c00101,0xfb840108,0x83c10101,
	0xfb840108,0x83c20101,0xfb840108,0x83c30101,0xfb840108,0x83c40101,0xfb840108,0x83c50101,
	0xfb840108,0x83c60101,0xfb840108,0x83c70101,0xfb840108,0x83c80101,0xfb840108,0x83c90101,
	0xfb840108,0x83ca0101,0xfb840108,0x83cb0101,0xfb840108,0x83cc0101,0xfb840108,0x83cd0101,
	0xfb840108,0x83ce0101,0xfb840108,0x83cf0101,0xfb840108,0x83d00101,0xfb840108,0x83d10101,
	0xfb840108,0x83d20101,0xfb840108,0x83d30101,0xfb840108,0x83d40101,0xfb840108,0x83d50101,
	0xfb840108,0x83d60101,0xfb840108,0x83d70101,0xfb840108,0x83d80101,0xfb840108,0x83d90101,
	0xfb840108,0x83da0101,0xfb840108,0x83db0101,0xfb840108,0x83dc0101,0xfb840108,0x83dd0101,
	0xfb840108,0x83de0101,0xfb840108,0x83df0101,0xfb840108,0x83e00101,0xfb840108,0x83e10101,
	0xfb840108,0x83e20101,0xfb840108,0x83e30101,0xfb840108,0x83e40101,0xfb840108,0x83e50101,
	0xfb840108,0x83e60101,0xfb840108,0x83e70101,0xfb840108,0x83e80101,0xfb840108,0x83e90101,
	0xfb840108,0x83ea0101,0xfb840108,0x83eb0101,0xfb840108,0x83ec0101,0xfb840108,0x83ed0101,
	0xfb840108,0x83ee0101,0xfb840108,0x83ef0101,0xfb840108,0x83f00101,0xfb840108,0x83f10101,
	0xfb840108,0x83f20101,0xfb840108,0x83f30101,0xfb840108,0x83f40101,0xfb840108,0x83f50101,
	0xfb840108,0x83f60101,0xfb840108,0x83f70101,0xfb840108,0x83f80101,0xfb840108,0x83f90101,
	0xfb840108,0x83fa0101,0xfb840108,0x83fb0101,0xfb840108,0x83fc0101,0xfb840108,0x83fd0101,
	0xfb840108,0x83fe0101,0xfb840108,0x83ff0101,0xfb840108,0x84000101,0xfb840108,0x84010101,
	0xfb840108,0x84020101,0xfb840108,0x84030101,0xfb840108,0x84040101,0xfb840108,0x84050101,
	0xfb840108,0x84060101,0xfb840108,0x84070101,0xfb840108,0x84080101,0xfb840108,0x84090101,
	0xfb840108,0x840a0101,0xfb840108,0x840b0101,0xfb840108,0x840c0101,0xfb840108,0x840d0101,
	0xfb840108,0x840e0101,0xfb840108,0x840f0101,0xfb840108,0x84100101,0xfb840108,0x84110101,
	0xfb840108,0x84120101,0xfb840108,0x84130101,0xfb840108,0x84140101,0xfb840108,0x84150101,
	0xfb840108,0x84160101,0xfb840108,0x84170101,0xfb840108,0x84180101,0xfb840108,0x84190101,
	0xfb840108,0x841a0101,0xfb840108,0x841b0101,0xfb840108,0x841c0101,0xfb840108,0x841d0101,
	0xfb840108,0x841e0101,0xfb840108,0x841f0101,0xfb840108,0x84200101,0xfb840108,0x84210101,
	0xfb840108,0x84220101,0xfb840108,0x84230101,0xfb840108,0x84240101,0xfb840108,0x84250101,
	0xfb840108,0x84260101,0xfb840108,0x84270101,0xfb840108,0x84280101,0xfb840108,0x84290101,
	0xfb840108,0x842a0101,0xfb840108,0x842b0101,0xfb840108,0x842c0101,0xfb840108,0x842d0101,
	0xfb840108,0x842e0101,0xfb840108,0x842f0101,0xfb840108,0x84300101,0xfb840108,0x84310101,
	0xfb840108,0x84320101,0xfb840108,0x84330101,0xfb840108,0x84340101,0xfb840108,0x84350101,
	0xfb840108,0x84360101,0xfb840108,0x84370101,0xfb840108,0x84380101,0xfb840108,0x84390101,
	0xfb840108,0x843a0101,0xfb840108,0x843b0101,0xfb840108,0x843c0101,0xfb840108,0x843d0101,
	0xfb840108,0x843e0101,0xfb840108,0x843f0101,0xfb840108,0x84400101,0xfb840108,0x84410101,
	0xfb840108,0x84420101,0xfb840108,0x84430101,0xfb840108,0x84440101,0xfb840108,0x84450101,
	0xfb840108,0x84460101,0xfb840108,0x84470101,0xfb840108,0x84480101,0xfb840108,0x84490101,
	0xfb840108,0x844a0101,0xfb840108,0x844b0101,0xfb840108,0x844c0101,0xfb840108,0x844d0101,
	0xfb840108,0x844e0101,0xfb840108,0x844f0101,0xfb840108,0x84500101,0xfb840108,0x84510101,
	0xfb840108,0x84520101,0xfb840108,0x84530101,0xfb840108,0x84540101,0xfb840108,0x84550101,
	0xfb840108,0x84560101,0xfb840108,0x84570101,0xfb840108,0x84580101,0xfb840108,0x84590101,
	0xfb840108,0x845a0101,0xfb840108,0x845b0101,0xfb840108,0x845c0101,0xfb840108,0x845d0101,
	0xfb840108,0x845e0101,0xfb840108,0x845f0101,0xfb840108,0x84600101,0xfb840108,0x84610101,
	0xfb840108,0x84620101,0xfb840108,0x84630101,0xfb840108,0x84640101,0xfb840108,0x84650101,
	0xfb840108,0x84660101,0xfb840108,0x84670101,0xfb840108,0x84680101,0xfb840108,0x84690101,
	0xfb840108,0x846a0101,0xfb840108,0x846b0101,0xfb840108,0x846c0101,0xfb840108,0x846d0101,
	0xfb840108,0x846e0101,0xfb840108,0x846f0101,0xfb840108,0x84700101,0xfb840108,0x84710101,
	0xfb840108,0x84720101,0xfb840108,0x84730101,0xfb840108,0x84740101,0xfb840108,0x84750101,
	0xfb840108,0x84760101,0xfb840108,0x84770101,0xfb840108,0x84780101,0xfb840108,0x84790101,
	0xfb840108,0x847a0101,0xfb840108,0x847b0101,0xfb840108,0x847c0101,0xfb840108,0x847d0101,
	0xfb840108,0x847e0101,0xfb840108,0x847f0101,0xfb840108,0x84800101,0xfb840108,0x84810101,
	0xfb840108,0x84820101,0xfb840108,0x84830101,0xfb840108,0x84840101,0xfb840108,0x84850101,
	0xfb840108,0x84860101,0xfb840108,0x84870101,0xfb840108,0x84880101,0xfb840108,0x84890101,
	0xfb840108,0x848a0101,0xfb840108,0x848b0101,0xfb840108,0x848c0101,0xfb840108,0x848d0101,
	0xfb840108,0x848e0101,0xfb840108,0x848f0101,0xfb840108,0x84900101,0xfb840108,0x84910101,
	0xfb840108,0x84920101,0xfb840108,0x84930101,0xfb840108,0x84940101,0xfb840108,0x84950101,
	0xfb840108,0x84960101,0xfb840108,0x84970101,0xfb840108,0x84980101,0xfb840108,0x84990101,
	0xfb840108,0x849a0101,0xfb840108,0x849b0101,0xfb840108,0x849c0101,0xfb840108,0x849d0101,
	0xfb840108,0x849e0101,0xfb840108,0x849f0101,0xfb840108,0x84a00101,0xfb840108,0x84a10101,
	0xfb840108,0x84a20101,0xfb840108,0x84a30101,0xfb840108,0x84a40101,0xfb840108,0x84a50101,
	0xfb840108,0x84a60101,0xfb840108,0x84a70101,0xfb840108,0x84a80101,0xfb840108,0x84a90101,
	0xfb840108,0x84aa0101,0xfb840108,0x84ab0101,0xfb840108,0x84ac0101,0xfb840108,0x84ad0101,
	0xfb840108,0x84ae0101,0xfb840108,0x84af0101,0xfb840108,0x84b00101,0xfb840108,0x84b10101,
	0xfb840108,0x84b20101,0xfb840108,0x84b30101,0xfb840108,0x84b40101,0xfb840108,0x84b50101,
	0xfb840108,0x84b60101,0xfb840108,0x84b70101,0xfb840108,0x84b80101,0xfb840108,0x84b90101,
	0xfb840108,0x84ba0101,0xfb840108,0x84bb0101,0xfb840108,0x84bc0101,0xfb840108,0x84bd0101,
	0xfb840108,0x84be0101,0xfb840108,0x84bf0101,0xfb840108,0x84c00101,0xfb840108,0x84c10101,
	0xfb840108,0x84c20101,0xfb840108,0x84c30101,0xfb840108,0x84c40101,0xfb840108,0x84c50101,
	0xfb840108,0x84c60101,0xfb840108,0x84c70101,0xfb840108,0x84c80101,0xfb840108,0x84c90101,
	0xfb840108,0x84ca0101,0xfb840108,0x84cb0101,0xfb840108,0x84cc0101,0xfb840108,0x84cd0101,
	0xfb840108,0x84ce0101,0xfb840108,0x84cf0101,0xfb840108,0x84d00101,0xfb840108,0x84d10101,
	0xfb840108,0x84d20101,0xfb840108,0x84d30101,0xfb840108,0x84d40101,0xfb840108,0x84d50101,
	0xfb840108,0x84d60101,0xfb840108,0x84d70101,0xfb840108,0x84d80101,0xfb840108,0x84d90101,
	0xfb840108,0x84da0101,0xfb840108,0x84db0101,0xfb840108,0x84dc0101,0xfb840108,0x84dd0101,
	0xfb840108,0x84de0101,0xfb840108,0x84df0101,0xfb840108,0x84e00101,0xfb840108,0x84e10101,
	0xfb840108,0x84e20101,0xfb840108,0x84e30101,0xfb840108,0x84e40101,0xfb840108,0x84e50101,
	0xfb840108,0x84e60101,0xfb840108,0x84e70101,0xfb840108,0x84e80101,0xfb840108,0x84e90101,
	0xfb840108,0x84ea0101,0xfb840108,0x84eb0101,0xfb840108,0x84ec0101,0xfb840108,0x84ed0101,
	0xfb840108,0x84ee0101,0xfb840108,0x84ef0101,0xfb840108,0x84f00101,0xfb840108,0x84f10101,
	0xfb840108,0x84f20101,0xfb840108,0x84f30101,0xfb840108,0x84f40101,0xfb840108,0x84f50101,
	0xfb840108,0x84f60101,0xfb840108,0x84f70101,0xfb840108,0x84f80101,0xfb840108,0x84f90101,
	0xfb840108,0x84fa0101,0xfb840108,0x84fb0101,0xfb840108,0x84fc0101,0xfb840108,0x84fd0101,
	0xfb840108,0x84fe0101,0xfb840108,0x84ff0101,0xfb840108,0x85000101,0xfb840108,0x85010101,
	0xfb840108,0x85020101,0xfb840108,0x85030101,0xfb840108,0x85040101,0xfb840108,0x85050101,
	0xfb840108,0x85060101,0xfb840108,0x85070101,0xfb840108,0x85080101,0xfb840108,0x85090101,
	0xfb840108,0x850a0101,0xfb840108,0x850b0101,0xfb840108,0x850c0101,0xfb840108,0x850d0101,
	0xfb840108,0x850e0101,0xfb840108,0x850f0101,0xfb840108,0x85100101,0xfb840108,0x85110101,
	0xfb840108,0x85120101,0xfb840108,0x85130101,0xfb840108,0x85140101,0xfb840108,0x85150101,
	0xfb840108,0x85160101,0xfb840108,0x85170101,0xfb840108,0x85180101,0xfb840108,0x85190101,
	0xfb840108,0x851a0101,0xfb840108,0x851b0101,0xfb840108,0x851c0101,0xfb840108,0x851d0101,
	0xfb840108,0x851e0101,0xfb840108,0x851f0101,0xfb840108,0x85200101,0xfb840108,0x85210101,
	0xfb840108,0x85220101,0xfb840108,0x85230101,0xfb840108,0x85240101,0xfb840108,0x85250101,
	0xfb840108,0x85260101,0xfb840108,0x85270101,0xfb840108,0x85280101,0xfb840108,0x85290101,
	0xfb840108,0x852a0101,0xfb840108,0x852b0101,0xfb840108,0x852c0101,0xfb840108,0x852d0101,
	0xfb840108,0x852e0101,0xfb840108,0x852f0101,0xfb840108,0x85300101,0xfb840108,0x85310101,
	0xfb840108,0x85320101,0xfb840108,0x85330101,0xfb840108,0x85340101,0xfb840108,0x85350101,
	0xfb840108,0x85360101,0xfb840108,0x85370101,0xfb840108,0x85380101,0xfb840108,0x85390101,
	0xfb840108,0x853a0101,0xfb840108,0x853b0101,0xfb840108,0x853c0101,0xfb840108,0x853d0101,
	0xfb840108,0x853e0101,0xfb840108,0x853f0101,0xfb840108,0x85400101,0xfb840108,0x85410101,
	0xfb840108,0x85420101,0xfb840108,0x85430101,0xfb840108,0x85440101,0xfb840108,0x85450101,
	0xfb840108,0x85460101,0xfb840108,0x85470101,0xfb840108,0x85480101,0xfb840108,0x85490101,
	0xfb840108,0x854a0101,0xfb840108,0x854b0101,0xfb840108,0x854c0101,0xfb840108,0x854d0101,
	0xfb840108,0x854e0101,0xfb840108,0x854f0101,0xfb840108,0x85500101,0xfb840108,0x85510101,
	0xfb840108,0x85520101,0xfb840108,0x85530101,0xfb840108,0x85540101,0xfb840108,0x85550101,
	0xfb840108,0x85560101,0xfb840108,0x85570101,0xfb840108,0x85580101,0xfb840108,0x85590101,
	0xfb840108,0x855a0101,0xfb840108,0x855b0101,0xfb840108,0x855c0101,0xfb840108,0x855d0101,
	0xfb840108,0x855e0101,0xfb840108,0x855f0101,0xfb840108,0x85600101,0xfb840108,0x85610101,
	0xfb840108,0x85620101,0xfb840108,0x85630101,0xfb840108,0x85640101,0xfb840108,0x85650101,
	0xfb840108,0x85660101,0xfb840108,0x85670101,0xfb840108,0x85680101,0xfb840108,0x85690101,
	0xfb840108,0x856a0101,0xfb840108,0x856b0101,0xfb840108,0x856c0101,0xfb840108,0x856d0101,
	0xfb840108,0x856e0101,0xfb840108,0x856f0101,0xfb840108,0x85700101,0xfb840108,0x85710101,
	0xfb840108,0x85720101,0xfb840108,0x85730101,0xfb840108,0x85740101,0xfb840108,0x85750101,
	0xfb840108,0x85760101,0xfb840108,0x85770101,0xfb840108,0x85780101,0xfb840108,0x85790101,
	0xfb840108,0x857a0101,0xfb840108,0x857b0101,0xfb840108,0x857c0101,0xfb840108,0x857d0101,
	0xfb840108,0x857e0101,0xfb840108,0x857f0101,0xfb840108,0x85800101,0xfb840108,0x85810101,
	0xfb840108,0x85820101,0xfb840108,0x85830101,0xfb840108,0x85840101,0xfb840108,0x85850101,
	0xfb840108,0x85860101,0xfb840108,0x85870101,0xfb840108,0x85880101,0xfb840108,0x85890101,
	0xfb840108,0x858a0101,0xfb840108,0x858b0101,0xfb840108,0x858c0101,0xfb840108,0x858d0101,
	0xfb840108,0x858e0101,0xfb840108,0x858f0101,0xfb840108,0x85900101,0xfb840108,0x85910101,
	0xfb840108,0x85920101,0xfb840108,0x85930101,0xfb840108,0x85940101,0xfb840108,0x85950101,
	0xfb840108,0x85960101,0xfb840108,0x85970101,0xfb840108,0x85980101,0xfb840108,0x85990101,
	0xfb840108,0x859a0101,0xfb840108,0x859b0101,0xfb840108,0x859c0101,0xfb840108,0x859d0101,
	0xfb840108,0x859e0101,0xfb840108,0x859f0101,0xfb840108,0x85a00101,0xfb840108,0x85a10101,
	0xfb840108,0x85a20101,0xfb840108,0x85a30101,0xfb840108,0x85a40101,0xfb840108,0x85a50101,
	0xfb840108,0x85a60101,0xfb840108,0x85a70101,0xfb840108,0x85a80101,0xfb840108,0x85a90101,
	0xfb840108,0x85aa0101,0xfb840108,0x85ab0101,0xfb840108,0x85ac0101,0xfb840108,0x85ad0101,
	0xfb840108,0x85ae0101,0xfb840108,0x85af0101,0xfb840108,0x85b00101,0xfb840108,0x85b10101,
	0xfb840108,0x85b20101,0xfb840108,0x85b30101,0xfb840108,0x85b40101,0xfb840108,0x85b50101,
	0xfb840108,0x85b60101,0xfb840108,0x85b70101,0xfb840108,0x85b80101,0xfb840108,0x85b90101,
	0xfb840108,0x85ba0101,0xfb840108,0x85bb0101,0xfb840108,0x85bc0101,0xfb840108,0x85bd0101,
	0xfb840108,0x85be0101,0xfb840108,0x85bf0101,0xfb840108,0x85c00101,0xfb840108,0x85c10101,
	0xfb840108,0x85c20101,0xfb840108,0x85c30101,0xfb840108,0x85c40101,0xfb840108,0x85c50101,
	0xfb840108,0x85c60101,0xfb840108,0x85c70101,0xfb840108,0x85c80101,0xfb840108,0x85c90101,
	0xfb840108,0x85ca0101,0xfb840108,0x85cb0101,0xfb840108,0x85cc0101,0xfb840108,0x85cd0101,
	0xfb840108,0x85ce0101,0xfb840108,0x85cf0101,0xfb840108,0x85d00101,0xfb840108,0x85d10101,
	0xfb840108,0x85d20101,0xfb840108,0x85d30101,0xfb840108,0x85d40101,0xfb840108,0x85d50101,
	0xfb840108,0x85d60101,0xfb840108,0x85d70101,0xfb840108,0x85d80101,0xfb840108,0x85d90101,
	0xfb840108,0x85da0101,0xfb840108,0x85db0101,0xfb840108,0x85dc0101,0xfb840108,0x85dd0101,
	0xfb840108,0x85de0101,0xfb840108,0x85df0101,0xfb840108,0x85e00101,0xfb840108,0x85e10101,
	0xfb840108,0x85e20101,0xfb840108,0x85e30101,0xfb840108,0x85e40101,0xfb840108,0x85e50101,
	0xfb840108,0x85e60101,0xfb840108,0x85e70101,0xfb840108,0x85e80101,0xfb840108,0x85e90101,
	0xfb840108,0x85ea0101,0xfb840108,0x85eb0101,0xfb840108,0x85ec0101,0xfb840108,0x85ed0101,
	0xfb840108,0x85ee0101,0xfb840108,0x85ef0101,0xfb840108,0x85f00101,0xfb840108,0x85f10101,
	0xfb840108,0x85f20101,0xfb840108,0x85f30101,0xfb840108,0x85f40101,0xfb840108,0x85f50101,
	0xfb840108,0x85f60101,0xfb840108,0x85f70101,0xfb840108,0x85f80101,0xfb840108,0x85f90101,
	0xfb840108,0x85fa0101,0xfb840108,0x85fb0101,0xfb840108,0x85fc0101,0xfb840108,0x85fd0101,
	0xfb840108,0x85fe0101,0xfb840108,0x85ff0101,0xfb840108,0x86000101,0xfb840108,0x86010101,
	0xfb840108,0x86020101,0xfb840108,0x86030101,0xfb840108,0x86040101,0xfb840108,0x86050101,
	0xfb840108,0x86060101,0xfb840108,0x86070101,0xfb840108,0x86080101,0xfb840108,0x86090101,
	0xfb840108,0x860a0101,0xfb840108,0x860b0101,0xfb840108,0x860c0101,0xfb840108,0x860d0101,
	0xfb840108,0x860e0101,0xfb840108,0x860f0101,0xfb840108,0x86100101,0xfb840108,0x86110101,
	0xfb840108,0x86120101,0xfb840108,0x86130101,0xfb840108,0x86140101,0xfb840108,0x86150101,
	0xfb840108,0x86160101,0xfb840108,0x86170101,0xfb840108,0x86180101,0xfb840108,0x86190101,
	0xfb840108,0x861a0101,0xfb840108,0x861b0101,0xfb840108,0x861c0101,0xfb840108,0x861d0101,
	0xfb840108,0x861e0101,0xfb840108,0x861f0101,0xfb840108,0x86200101,0xfb840108,0x86210101,
	0xfb840108,0x86220101,0xfb840108,0x86230101,0xfb840108,0x86240101,0xfb840108,0x86250101,
	0xfb840108,0x86260101,0xfb840108,0x86270101,0xfb840108,0x86280101,0xfb840108,0x86290101,
	0xfb840108,0x862a0101,0xfb840108,0x862b0101,0xfb840108,0x862c0101,0xfb840108,0x862d0101,
	0xfb840108,0x862e0101,0xfb840108,0x862f0101,0xfb840108,0x86300101,0xfb840108,0x86310101,
	0xfb840108,0x86320101,0xfb840108,0x86330101,0xfb840108,0x86340101,0xfb840108,0x86350101,
	0xfb840108,0x86360101,0xfb840108,0x86370101,0xfb840108,0x86380101,0xfb840108,0x86390101,
	0xfb840108,0x863a0101,0xfb840108,0x863b0101,0xfb840108,0x863c0101,0xfb840108,0x863d0101,
	0xfb840108,0x863e0101,0xfb840108,0x863f0101,0xfb840108,0x86400101,0xfb840108,0x86410101,
	0xfb840108,0x86420101,0xfb840108,0x86430101,0xfb840108,0x86440101,0xfb840108,0x86450101,
	0xfb840108,0x86460101,0xfb840108,0x86470101,0xfb840108,0x86480101,0xfb840108,0x86490101,
	0xfb840108,0x864a0101,0xfb840108,0x864b0101,0xfb840108,0x864c0101,0xfb840108,0x864d0101,
	0xfb840108,0x864e0101,0xfb840108,0x864f0101,0xfb840108,0x86500101,0xfb840108,0x86510101,
	0xfb840108,0x86520101,0xfb840108,0x86530101,0xfb840108,0x86540101,0xfb840108,0x86550101,
	0xfb840108,0x86560101,0xfb840108,0x86570101,0xfb840108,0x86580101,0xfb840108,0x86590101,
	0xfb840108,0x865a0101,0xfb840108,0x865b0101,0xfb840108,0x865c0101,0xfb840108,0x865d0101,
	0xfb840108,0x865e0101,0xfb840108,0x865f0101,0xfb840108,0x86600101,0xfb840108,0x86610101,
	0xfb840108,0x86620101,0xfb840108,0x86630101,0xfb840108,0x86640101,0xfb840108,0x86650101,
	0xfb840108,0x86660101,0xfb840108,0x86670101,0xfb840108,0x86680101,0xfb840108,0x86690101,
	0xfb840108,0x866a0101,0xfb840108,0x866b0101,0xfb840108,0x866c0101,0xfb840108,0x866d0101,
	0xfb840108,0x866e0101,0xfb840108,0x866f0101,0xfb840108,0x86700101,0xfb840108,0x86710101,
	0xfb840108,0x86720101,0xfb840108,0x86730101,0xfb840108,0x86740101,0xfb840108,0x86750101,
	0xfb840108,0x86760101,0xfb840108,0x86770101,0xfb840108,0x86780101,0xfb840108,0x86790101,
	0xfb840108,0x867a0101,0xfb840108,0x867b0101,0xfb840108,0x867c0101,0xfb840108,0x867d0101,
	0xfb840108,0x867e0101,0xfb840108,0x867f0101,0xfb840108,0x86800101,0xfb840108,0x86810101,
	0xfb840108,0x86820101,0xfb840108,0x86830101,0xfb840108,0x86840101,0xfb840108,0x86850101,
	0xfb840108,0x86860101,0xfb840108,0x86870101,0xfb840108,0x86880101,0xfb840108,0x86890101,
	0xfb840108,0x868a0101,0xfb840108,0x868b0101,0xfb840108,0x868c0101,0xfb840108,0x868d0101,
	0xfb840108,0x868e0101,0xfb840108,0x868f0101,0xfb840108,0x86900101,0xfb840108,0x86910101,
	0xfb840108,0x86920101,0xfb840108,0x86930101,0xfb840108,0x86940101,0xfb840108,0x86950101,
	0xfb840108,0x86960101,0xfb840108,0x86970101,0xfb840108,0x86980101,0xfb840108,0x86990101,
	0xfb840108,0x869a0101,0xfb840108,0x869b0101,0xfb840108,0x869c0101,0xfb840108,0x869d0101,
	0xfb840108,0x869e0101,0xfb840108,0x869f0101,0xfb840108,0x86a00101,0xfb840108,0x86a10101,
	0xfb840108,0x86a20101,0xfb840108,0x86a30101,0xfb840108,0x86a40101,0xfb840108,0x86a50101,
	0xfb840108,0x86a60101,0xfb840108,0x86a70101,0xfb840108,0x86a80101,0xfb840108,0x86a90101,
	0xfb840108,0x86aa0101,0xfb840108,0x86ab0101,0xfb840108,0x86ac0101,0xfb840108,0x86ad0101,
	0xfb840108,0x86ae0101,0xfb840108,0x86af0101,0xfb840108,0x86b00101,0xfb840108,0x86b10101,
	0xfb840108,0x86b20101,0xfb840108,0x86b30101,0xfb840108,0x86b40101,0xfb840108,0x86b50101,
	0xfb840108,0x86b60101,0xfb840108,0x86b70101,0xfb840108,0x86b80101,0xfb840108,0x86b90101,
	0xfb840108,0x86ba0101,0xfb840108,0x86bb0101,0xfb840108,0x86bc0101,0xfb840108,0x86bd0101,
	0xfb840108,0x86be0101,0xfb840108,0x86bf0101,0xfb840108,0x86c00101,0xfb840108,0x86c10101,
	0xfb840108,0x86c20101,0xfb840108,0x86c30101,0xfb840108,0x86c40101,0xfb840108,0x86c50101,
	0xfb840108,0x86c60101,0xfb840108,0x86c70101,0xfb840108,0x86c80101,0xfb840108,0x86c90101,
	0xfb840108,0x86ca0101,0xfb840108,0x86cb0101,0xfb840108,0x86cc0101,0xfb840108,0x86cd0101,
	0xfb840108,0x86ce0101,0xfb840108,0x86cf0101,0xfb840108,0x86d00101,0xfb840108,0x86d10101,
	0xfb840108,0x86d20101,0xfb840108,0x86d30101,0xfb840108,0x86d40101,0xfb840108,0x86d50101,
	0xfb840108,0x86d60101,0xfb840108,0x86d70101,0xfb840108,0x86d80101,0xfb840108,0x86d90101,
	0xfb840108,0x86da0101,0xfb840108,0x86db0101,0xfb840108,0x86dc0101,0xfb840108,0x86dd0101,
	0xfb840108,0x86de0101,0xfb840108,0x86df0101,0xfb840108,0x86e00101,0xfb840108,0x86e10101,
	0xfb840108,0x86e20101,0xfb840108,0x86e30101,0xfb840108,0x86e40101,0xfb840108,0x86e50101,
	0xfb840108,0x86e60101,0xfb840108,0x86e70101,0xfb840108,0x86e80101,0xfb840108,0x86e90101,
	0xfb840108,0x86ea0101,0xfb840108,0x86eb0101,0xfb840108,0x86ec0101,0xfb840108,0x86ed0101,
	0xfb840108,0x86ee0101,0xfb840108,0x86ef0101,0xfb840108,0x86f00101,0xfb840108,0x86f10101,
	0xfb840108,0x86f20101,0xfb840108,0x86f30101,0xfb840108,0x86f40101,0xfb840108,0x86f50101,
	0xfb840108,0x86f60101,0xfb840108,0x86f70101,0xfb840108,0x86f80101,0xfb840108,0x86f90101,
	0xfb840108,0x86fa0101,0xfb840108,0x86fb0101,0xfb840108,0x86fc0101,0xfb840108,0x86fd0101,
	0xfb840108,0x86fe0101,0xfb840108,0x86ff0101,0xfb840108,0x87000101,0xfb840108,0x87010101,
	0xfb840108,0x87020101,0xfb840108,0x87030101,0xfb840108,0x87040101,0xfb840108,0x87050101,
	0xfb840108,0x87060101,0xfb840108,0x87070101,0xfb840108,0x87080101,0xfb840108,0x87090101,
	0xfb840108,0x870a0101,0xfb840108,0x870b0101,0xfb840108,0x870c0101,0xfb840108,0x870d0101,
	0xfb840108,0x870e0101,0xfb840108,0x870f0101,0xfb840108,0x87100101,0xfb840108,0x87110101,
	0xfb840108,0x87120101,0xfb840108,0x87130101,0xfb840108,0x87140101,0xfb840108,0x87150101,
	0xfb840108,0x87160101,0xfb840108,0x87170101,0xfb840108,0x87180101,0xfb840108,0x87190101,
	0xfb840108,0x871a0101,0xfb840108,0x871b0101,0xfb840108,0x871c0101,0xfb840108,0x871d0101,
	0xfb840108,0x871e0101,0xfb840108,0x871f0101,0xfb840108,0x87200101,0xfb840108,0x87210101,
	0xfb840108,0x87220101,0xfb840108,0x87230101,0xfb840108,0x87240101,0xfb840108,0x87250101,
	0xfb840108,0x87260101,0xfb840108,0x87270101,0xfb840108,0x87280101,0xfb840108,0x87290101,
	0xfb840108,0x872a0101,0xfb840108,0x872b0101,0xfb840108,0x872c0101,0xfb840108,0x872d0101,
	0xfb840108,0x872e0101,0xfb840108,0x872f0101,0xfb840108,0x87300101,0xfb840108,0x87310101,
	0xfb840108,0x87320101,0xfb840108,0x87330101,0xfb840108,0x87340101,0xfb840108,0x87350101,
	0xfb840108,0x87360101,0xfb840108,0x87370101,0xfb840108,0x87380101,0xfb840108,0x87390101,
	0xfb840108,0x873a0101,0xfb840108,0x873b0101,0xfb840108,0x873c0101,0xfb840108,0x873d0101,
	0xfb840108,0x873e0101,0xfb840108,0x873f0101,0xfb840108,0x87400101,0xfb840108,0x87410101,
	0xfb840108,0x87420101,0xfb840108,0x87430101,0xfb840108,0x87440101,0xfb840108,0x87450101,
	0xfb840108,0x87460101,0xfb840108,0x87470101,0xfb840108,0x87480101,0xfb840108,0x87490101,
	0xfb840108,0x874a0101,0xfb840108,0x874b0101,0xfb840108,0x874c0101,0xfb840108,0x874d0101,
	0xfb840108,0x874e0101,0xfb840108,0x874f0101,0xfb840108,0x87500101,0xfb840108,0x87510101,
	0xfb840108,0x87520101,0xfb840108,0x87530101,0xfb840108,0x87540101,0xfb840108,0x87550101,
	0xfb840108,0x87560101,0xfb840108,0x87570101,0xfb840108,0x87580101,0xfb840108,0x87590101,
	0xfb840108,0x875a0101,0xfb840108,0x875b0101,0xfb840108,0x875c0101,0xfb840108,0x875d0101,
	0xfb840108,0x875e0101,0xfb840108,0x875f0101,0xfb840108,0x87600101,0xfb840108,0x87610101,
	0xfb840108,0x87620101,0xfb840108,0x87630101,0xfb840108,0x87640101,0xfb840108,0x87650101,
	0xfb840108,0x87660101,0xfb840108,0x87670101,0xfb840108,0x87680101,0xfb840108,0x87690101,
	0xfb840108,0x876a0101,0xfb840108,0x876b0101,0xfb840108,0x876c0101,0xfb840108,0x876d0101,
	0xfb840108,0x876e0101,0xfb840108,0x876f0101,0xfb840108,0x87700101,0xfb840108,0x87710101,
	0xfb840108,0x87720101,0xfb840108,0x87730101,0xfb840108,0x87740101,0xfb840108,0x87750101,
	0xfb840108,0x87760101,0xfb840108,0x87770101,0xfb840108,0x87780101,0xfb840108,0x87790101,
	0xfb840108,0x877a0101,0xfb840108,0x877b0101,0xfb840108,0x877c0101,0xfb840108,0x877d0101,
	0xfb840108,0x877e0101,0xfb840108,0x877f0101,0xfb840108,0x87800101,0xfb840108,0x87810101,
	0xfb840108,0x87820101,0xfb840108,0x87830101,0xfb840108,0x87840101,0xfb840108,0x87850101,
	0xfb840108,0x87860101,0xfb840108,0x87870101,0xfb840108,0x87880101,0xfb840108,0x87890101,
	0xfb840108,0x878a0101,0xfb840108,0x878b0101,0xfb840108,0x878c0101,0xfb840108,0x878d0101,
	0xfb840108,0x878e0101,0xfb840108,0x878f0101,0xfb840108,0x87900101,0xfb840108,0x87910101,
	0xfb840108,0x87920101,0xfb840108,0x87930101,0xfb840108,0x87940101,0xfb840108,0x87950101,
	0xfb840108,0x87960101,0xfb840108,0x87970101,0xfb840108,0x87980101,0xfb840108,0x87990101,
	0xfb840108,0x879a0101,0xfb840108,0x879b0101,0xfb840108,0x879c0101,0xfb840108,0x879d0101,
	0xfb840108,0x879e0101,0xfb840108,0x879f0101,0xfb840108,0x87a00101,0xfb840108,0x87a10101,
	0xfb840108,0x87a20101,0xfb840108,0x87a30101,0xfb840108,0x87a40101,0xfb840108,0x87a50101,
	0xfb840108,0x87a60101,0xfb840108,0x87a70101,0xfb840108,0x87a80101,0xfb840108,0x87a90101,
	0xfb840108,0x87aa0101,0xfb840108,0x87ab0101,0xfb840108,0x87ac0101,0xfb840108,0x87ad0101,
	0xfb840108,0x87ae0101,0xfb840108,0x87af0101,0xfb840108,0x87b00101,0xfb840108,0x87b10101,
	0xfb840108,0x87b20101,0xfb840108,0x87b30101,0xfb840108,0x87b40101,0xfb840108,0x87b50101,
	0xfb840108,0x87b60101,0xfb840108,0x87b70101,0xfb840108,0x87b80101,0xfb840108,0x87b90101,
	0xfb840108,0x87ba0101,0xfb840108,0x87bb0101,0xfb840108,0x87bc0101,0xfb840108,0x87bd0101,
	0xfb840108,0x87be0101,0xfb840108,0x87bf0101,0xfb840108,0x87c00101,0xfb840108,0x87c10101,
	0xfb840108,0x87c20101,0xfb840108,0x87c30101,0xfb840108,0x87c40101,0xfb840108,0x87c50101,
	0xfb840108,0x87c60101,0xfb840108,0x87c70101,0xfb840108,0x87c80101,0xfb840108,0x87c90101,
	0xfb840108,0x87ca0101,0xfb840108,0x87cb0101,0xfb840108,0x87cc0101,0xfb840108,0x87cd0101,
	0xfb840108,0x87ce0101,0xfb840108,0x87cf0101,0xfb840108,0x87d00101,0xfb840108,0x87d10101,
	0xfb840108,0x87d20101,0xfb840108,0x87d30101,0xfb840108,0x87d40101,0xfb840108,0x87d50101,
	0xfb840108,0x87d60101,0xfb840108,0x87d70101,0xfb840108,0x87d80101,0xfb840108,0x87d90101,
	0xfb840108,0x87da0101,0xfb840108,0x87db0101,0xfb840108,0x87dc0101,0xfb840108,0x87dd0101,
	0xfb840108,0x87de0101,0xfb840108,0x87df0101,0xfb840108,0x87e00101,0xfb840108,0x87e10101,
	0xfb840108,0x87e20101,0xfb840108,0x87e30101,0xfb840108,0x87e40101,0xfb840108,0x87e50101,
	0xfb840108,0x87e60101,0xfb840108,0x87e70101,0xfb840108,0x87e80101,0xfb840108,0x87e90101,
	0xfb840108,0x87ea0101,0xfb840108,0x87eb0101,0xfb840108,0x87ec0101,0xfb840108,0x87ed0101,
	0xfb840108,0x87ee0101,0xfb840108,0x87ef0101,0xfb840108,0x87f00101,0xfb840108,0x87f10101,
	0xfb840108,0x87f20101,0xfb840108,0x87f30101,0xfb840108,0x87f40101,0xfb840108,0x87f50101,
	0xfb840108,0x87f60101,0xfb840108,0x87f70101,0xfb840108,0x87f80101,0xfb840108,0x87f90101,
	0xfb840108,0x87fa0101,0xfb840108,0x87fb0101,0xfb840108,0x87fc0101,0xfb840108,0x87fd0101,
	0xfb840108,0x87fe0101,0xfb840108,0x87ff0101,0xfb840108,0x88000101,0xfb840108,0x88010101,
	0xfb840108,0x88020101,0xfb840108,0x88030101,0xfb840108,0x88040101,0xfb840108,0x88050101,
	0xfb840108,0x88060101,0xfb840108,0x88070101,0xfb840108,0x88080101,0xfb840108,0x88090101,
	0xfb840108,0x880a0101,0xfb840108,0x880b0101,0xfb840108,0x880c0101,0xfb840108,0x880d0101,
	0xfb840108,0x880e0101,0xfb840108,0x880f0101,0xfb840108,0x88100101,0xfb840108,0x88110101,
	0xfb840108,0x88120101,0xfb840108,0x88130101,0xfb840108,0x88140101,0xfb840108,0x88150101,
	0xfb840108,0x88160101,0xfb840108,0x88170101,0xfb840108,0x88180101,0xfb840108,0x88190101,
	0xfb840108,0x881a0101,0xfb840108,0x881b0101,0xfb840108,0x881c0101,0xfb840108,0x881d0101,
	0xfb840108,0x881e0101,0xfb840108,0x881f0101,0xfb840108,0x88200101,0xfb840108,0x88210101,
	0xfb840108,0x88220101,0xfb840108,0x88230101,0xfb840108,0x88240101,0xfb840108,0x88250101,
	0xfb840108,0x88260101,0xfb840108,0x88270101,0xfb840108,0x88280101,0xfb840108,0x88290101,
	0xfb840108,0x882a0101,0xfb840108,0x882b0101,0xfb840108,0x882c0101,0xfb840108,0x882d0101,
	0xfb840108,0x882e0101,0xfb840108,0x882f0101,0xfb840108,0x88300101,0xfb840108,0x88310101,
	0xfb840108,0x88320101,0xfb840108,0x88330101,0xfb840108,0x88340101,0xfb840108,0x88350101,
	0xfb840108,0x88360101,0xfb840108,0x88370101,0xfb840108,0x88380101,0xfb840108,0x88390101,
	0xfb840108,0x883a0101,0xfb840108,0x883b0101,0xfb840108,0x883c0101,0xfb840108,0x883d0101,
	0xfb840108,0x883e0101,0xfb840108,0x883f0101,0xfb840108,0x88400101,0xfb840108,0x88410101,
	0xfb840108,0x88420101,0xfb840108,0x88430101,0xfb840108,0x88440101,0xfb840108,0x88450101,
	0xfb840108,0x88460101,0xfb840108,0x88470101,0xfb840108,0x88480101,0xfb840108,0x88490101,
	0xfb840108,0x884a0101,0xfb840108,0x884b0101,0xfb840108,0x884c0101,0xfb840108,0x884d0101,
	0xfb840108,0x884e0101,0xfb840108,0x884f0101,0xfb840108,0x88500101,0xfb840108,0x88510101,
	0xfb840108,0x88520101,0xfb840108,0x88530101,0xfb840108,0x88540101,0xfb840108,0x88550101,
	0xfb840108,0x88560101,0xfb840108,0x88570101,0xfb840108,0x88580101,0xfb840108,0x88590101,
	0xfb840108,0x885a0101,0xfb840108,0x885b0101,0xfb840108,0x885c0101,0xfb840108,0x885d0101,
	0xfb840108,0x885e0101,0xfb840108,0x885f0101,0xfb840108,0x88600101,0xfb840108,0x88610101,
	0xfb840108,0x88620101,0xfb840108,0x88630101,0xfb840108,0x88640101,0xfb840108,0x88650101,
	0xfb840108,0x88660101,0xfb840108,0x88670101,0xfb840108,0x88680101,0xfb840108,0x88690101,
	0xfb840108,0x886a0101,0xfb840108,0x886b0101,0xfb840108,0x886c0101,0xfb840108,0x886d0101,
	0xfb840108,0x886e0101,0xfb840108,0x886f0101,0xfb840108,0x88700101,0xfb840108,0x88710101,
	0xfb840108,0x88720101,0xfb840108,0x88730101,0xfb840108,0x88740101,0xfb840108,0x88750101,
	0xfb840108,0x88760101,0xfb840108,0x88770101,0xfb840108,0x88780101,0xfb840108,0x88790101,
	0xfb840108,0x887a0101,0xfb840108,0x887b0101,0xfb840108,0x887c0101,0xfb840108,0x887d0101,
	0xfb840108,0x887e0101,0xfb840108,0x887f0101,0xfb840108,0x88800101,0xfb840108,0x88810101,
	0xfb840108,0x88820101,0xfb840108,0x88830101,0xfb840108,0x88840101,0xfb840108,0x88850101,
	0xfb840108,0x88860101,0xfb840108,0x88870101,0xfb840108,0x88880101,0xfb840108,0x88890101,
	0xfb840108,0x888a0101,0xfb840108,0x888b0101,0xfb840108,0x888c0101,0xfb840108,0x888d0101,
	0xfb840108,0x888e0101,0xfb840108,0x888f0101,0xfb840108,0x88900101,0xfb840108,0x88910101,
	0xfb840108,0x88920101,0xfb840108,0x88930101,0xfb840108,0x88940101,0xfb840108,0x88950101,
	0xfb840108,0x88960101,0xfb840108,0x88970101,0xfb840108,0x88980101,0xfb840108,0x88990101,
	0xfb840108,0x889a0101,0xfb840108,0x889b0101,0xfb840108,0x889c0101,0xfb840108,0x889d0101,
	0xfb840108,0x889e0101,0xfb840108,0x889f0101,0xfb840108,0x88a00101,0xfb840108,0x88a10101,
	0xfb840108,0x88a20101,0xfb840108,0x88a30101,0xfb840108,0x88a40101,0xfb840108,0x88a50101,
	0xfb840108,0x88a60101,0xfb840108,0x88a70101,0xfb840108,0x88a80101,0xfb840108,0x88a90101,
	0xfb840108,0x88aa0101,0xfb840108,0x88ab0101,0xfb840108,0x88ac0101,0xfb840108,0x88ad0101,
	0xfb840108,0x88ae0101,0xfb840108,0x88af0101,0xfb840108,0x88b00101,0xfb840108,0x88b10101,
	0xfb840108,0x88b20101,0xfb840108,0x88b30101,0xfb840108,0x88b40101,0xfb840108,0x88b50101,
	0xfb840108,0x88b60101,0xfb840108,0x88b70101,0xfb840108,0x88b80101,0xfb840108,0x88b90101,
	0xfb840108,0x88ba0101,0xfb840108,0x88bb0101,0xfb840108,0x88bc0101,0xfb840108,0x88bd0101,
	0xfb840108,0x88be0101,0xfb840108,0x88bf0101,0xfb840108,0x88c00101,0xfb840108,0x88c10101,
	0xfb840108,0x88c20101,0xfb840108,0x88c30101,0xfb840108,0x88c40101,0xfb840108,0x88c50101,
	0xfb840108,0x88c60101,0xfb840108,0x88c70101,0xfb840108,0x88c80101,0xfb840108,0x88c90101,
	0xfb840108,0x88ca0101,0xfb840108,0x88cb0101,0xfb840108,0x88cc0101,0xfb840108,0x88cd0101,
	0xfb840108,0x88ce0101,0xfb840108,0x88cf0101,0xfb840108,0x88d00101,0xfb840108,0x88d10101,
	0xfb840108,0x88d20101,0xfb840108,0x88d30101,0xfb840108,0x88d40101,0xfb840108,0x88d50101,
	0xfb840108,0x88d60101,0xfb840108,0x88d70101,0xfb840108,0x88d80101,0xfb840108,0x88d90101,
	0xfb840108,0x88da0101,0xfb840108,0x88db0101,0xfb840108,0x88dc0101,0xfb840108,0x88dd0101,
	0xfb840108,0x88de0101,0xfb840108,0x88df0101,0xfb840108,0x88e00101,0xfb840108,0x88e10101,
	0xfb840108,0x88e20101,0xfb840108,0x88e30101,0xfb840108,0x88e40101,0xfb840108,0x88e50101,
	0xfb840108,0x88e60101,0xfb840108,0x88e70101,0xfb840108,0x88e80101,0xfb840108,0x88e90101,
	0xfb840108,0x88ea0101,0xfb840108,0x88eb0101,0xfb840108,0x88ec0101,0xfb840108,0x88ed0101,
	0xfb840108,0x88ee0101,0xfb840108,0x88ef0101,0xfb840108,0x88f00101,0xfb840108,0x88f10101,
	0xfb840108,0x88f20101,0xfb840108,0x88f30101,0xfb840108,0x88f40101,0xfb840108,0x88f50101,
	0xfb840108,0x88f60101,0xfb840108,0x88f70101,0xfb840108,0x88f80101,0xfb840108,0x88f90101,
	0xfb840108,0x88fa0101,0xfb840108,0x88fb0101,0xfb840108,0x88fc0101,0xfb840108,0x88fd0101,
	0xfb840108,0x88fe0101,0xfb840108,0x88ff0101,0xfb840108,0x89000101,0xfb840108,0x89010101,
	0xfb840108,0x89020101,0xfb840108,0x89030101,0xfb840108,0x89040101,0xfb840108,0x89050101,
	0xfb840108,0x89060101,0xfb840108,0x89070101,0xfb840108,0x89080101,0xfb840108,0x89090101,
	0xfb840108,0x890a0101,0xfb840108,0x890b0101,0xfb840108,0x890c0101,0xfb840108,0x890d0101,
	0xfb840108,0x890e0101,0xfb840108,0x890f0101,0xfb840108,0x89100101,0xfb840108,0x89110101,
	0xfb840108,0x89120101,0xfb840108,0x89130101,0xfb840108,0x89140101,0xfb840108,0x89150101,
	0xfb840108,0x89160101,0xfb840108,0x89170101,0xfb840108,0x89180101,0xfb840108,0x89190101,
	0xfb840108,0x891a0101,0xfb840108,0x891b0101,0xfb840108,0x891c0101,0xfb840108,0x891d0101,
	0xfb840108,0x891e0101,0xfb840108,0x891f0101,0xfb840108,0x89200101,0xfb840108,0x89210101,
	0xfb840108,0x89220101,0xfb840108,0x89230101,0xfb840108,0x89240101,0xfb840108,0x89250101,
	0xfb840108,0x89260101,0xfb840108,0x89270101,0xfb840108,0x89280101,0xfb840108,0x89290101,
	0xfb840108,0x892a0101,0xfb840108,0x892b0101,0xfb840108,0x892c0101,0xfb840108,0x892d0101,
	0xfb840108,0x892e0101,0xfb840108,0x892f0101,0xfb840108,0x89300101,0xfb840108,0x89310101,
	0xfb840108,0x89320101,0xfb840108,0x89330101,0xfb840108,0x89340101,0xfb840108,0x89350101,
	0xfb840108,0x89360101,0xfb840108,0x89370101,0xfb840108,0x89380101,0xfb840108,0x89390101,
	0xfb840108,0x893a0101,0xfb840108,0x893b0101,0xfb840108,0x893c0101,0xfb840108,0x893d0101,
	0xfb840108,0x893e0101,0xfb840108,0x893f0101,0xfb840108,0x89400101,0xfb840108,0x89410101,
	0xfb840108,0x89420101,0xfb840108,0x89430101,0xfb840108,0x89440101,0xfb840108,0x89450101,
	0xfb840108,0x89460101,0xfb840108,0x89470101,0xfb840108,0x89480101,0xfb840108,0x89490101,
	0xfb840108,0x894a0101,0xfb840108,0x894b0101,0xfb840108,0x894c0101,0xfb840108,0x894d0101,
	0xfb840108,0x894e0101,0xfb840108,0x894f0101,0xfb840108,0x89500101,0xfb840108,0x89510101,
	0xfb840108,0x89520101,0xfb840108,0x89530101,0xfb840108,0x89540101,0xfb840108,0x89550101,
	0xfb840108,0x89560101,0xfb840108,0x89570101,0xfb840108,0x89580101,0xfb840108,0x89590101,
	0xfb840108,0x895a0101,0xfb840108,0x895b0101,0xfb840108,0x895c0101,0xfb840108,0x895d0101,
	0xfb840108,0x895e0101,0xfb840108,0x895f0101,0xfb840108,0x89600101,0xfb840108,0x89610101,
	0xfb840108,0x89620101,0xfb840108,0x89630101,0xfb840108,0x89640101,0xfb840108,0x89650101,
	0xfb840108,0x89660101,0xfb840108,0x89670101,0xfb840108,0x89680101,0xfb840108,0x89690101,
	0xfb840108,0x896a0101,0xfb840108,0x896b0101,0xfb840108,0x896c0101,0xfb840108,0x896d0101,
	0xfb840108,0x896e0101,0xfb840108,0x896f0101,0xfb840108,0x89700101,0xfb840108,0x89710101,
	0xfb840108,0x89720101,0xfb840108,0x89730101,0xfb840108,0x89740101,0xfb840108,0x89750101,
	0xfb840108,0x89760101,0xfb840108,0x89770101,0xfb840108,0x89780101,0xfb840108,0x89790101,
	0xfb840108,0x897a0101,0xfb840108,0x897b0101,0xfb840108,0x897c0101,0xfb840108,0x897d0101,
	0xfb840108,0x897e0101,0xfb840108,0x897f0101,0xfb840108,0x89800101,0xfb840108,0x89810101,
	0xfb840108,0x89820101,0xfb840108,0x89830101,0xfb840108,0x89840101,0xfb840108,0x89850101,
	0xfb840108,0x89860101,0xfb840108,0x89870101,0xfb840108,0x89880101,0xfb840108,0x89890101,
	0xfb840108,0x898a0101,0xfb840108,0x898b0101,0xfb840108,0x898c0101,0xfb840108,0x898d0101,
	0xfb840108,0x898e0101,0xfb840108,0x898f0101,0xfb840108,0x89900101,0xfb840108,0x89910101,
	0xfb840108,0x89920101,0xfb840108,0x89930101,0xfb840108,0x89940101,0xfb840108,0x89950101,
	0xfb840108,0x89960101,0xfb840108,0x89970101,0xfb840108,0x89980101,0xfb840108,0x89990101,
	0xfb840108,0x899a0101,0xfb840108,0x899b0101,0xfb840108,0x899c0101,0xfb840108,0x899d0101,
	0xfb840108,0x899e0101,0xfb840108,0x899f0101,0xfb840108,0x89a00101,0xfb840108,0x89a10101,
	0xfb840108,0x89a20101,0xfb840108,0x89a30101,0xfb840108,0x89a40101,0xfb840108,0x89a50101,
	0xfb840108,0x89a60101,0xfb840108,0x89a70101,0xfb840108,0x89a80101,0xfb840108,0x89a90101,
	0xfb840108,0x89aa0101,0xfb840108,0x89ab0101,0xfb840108,0x89ac0101,0xfb840108,0x89ad0101,
	0xfb840108,0x89ae0101,0xfb840108,0x89af0101,0xfb840108,0x89b00101,0xfb840108,0x89b10101,
	0xfb840108,0x89b20101,0xfb840108,0x89b30101,0xfb840108,0x89b40101,0xfb840108,0x89b50101,
	0xfb840108,0x89b60101,0xfb840108,0x89b70101,0xfb840108,0x89b80101,0xfb840108,0x89b90101,
	0xfb840108,0x89ba0101,0xfb840108,0x89bb0101,0xfb840108,0x89bc0101,0xfb840108,0x89bd0101,
	0xfb840108,0x89be0101,0xfb840108,0x89bf0101,0xfb840108,0x89c00101,0xfb840108,0x89c10101,
	0xfb840108,0x89c20101,0xfb840108,0x89c30101,0xfb840108,0x89c40101,0xfb840108,0x89c50101,
	0xfb840108,0x89c60101,0xfb840108,0x89c70101,0xfb840108,0x89c80101,0xfb840108,0x89c90101,
	0xfb840108,0x89ca0101,0xfb840108,0x89cb0101,0xfb840108,0x89cc0101,0xfb840108,0x89cd0101,
	0xfb840108,0x89ce0101,0xfb840108,0x89cf0101,0xfb840108,0x89d00101,0xfb840108,0x89d10101,
	0xfb840108,0x89d20101,0xfb840108,0x89d30101,0xfb840108,0x89d40101,0xfb840108,0x89d50101,
	0xfb840108,0x89d60101,0xfb840108,0x89d70101,0xfb840108,0x89d80101,0xfb840108,0x89d90101,
	0xfb840108,0x89da0101,0xfb840108,0x89db0101,0xfb840108,0x89dc0101,0xfb840108,0x89dd0101,
	0xfb840108,0x89de0101,0xfb840108,0x89df0101,0xfb840108,0x89e00101,0xfb840108,0x89e10101,
	0xfb840108,0x89e20101,0xfb840108,0x89e30101,0xfb840108,0x89e40101,0xfb840108,0x89e50101,
	0xfb840108,0x89e60101,0xfb840108,0x89e70101,0xfb840108,0x89e80101,0xfb840108,0x89e90101,
	0xfb840108,0x89ea0101,0xfb840108,0x89eb0101,0xfb840108,0x89ec0101,0xfb840108,0x89ed0101,
	0xfb840108,0x89ee0101,0xfb840108,0x89ef0101,0xfb840108,0x89f00101,0xfb840108,0x89f10101,
	0xfb840108,0x89f20101,0xfb840108,0x89f30101,0xfb840108,0x89f40101,0xfb840108,0x89f50101,
	0xfb840108,0x89f60101,0xfb840108,0x89f70101,0xfb840108,0x89f80101,0xfb840108,0x89f90101,
	0xfb840108,0x89fa0101,0xfb840108,0x89fb0101,0xfb840108,0x89fc0101,0xfb840108,0x89fd0101,
	0xfb840108,0x89fe0101,0xfb840108,0x89ff0101,0xfb840108,0x8a000101,0xfb840108,0x8a010101,
	0xfb840108,0x8a020101,0xfb840108,0x8a030101,0xfb840108,0x8a040101,0xfb840108,0x8a050101,
	0xfb840108,0x8a060101,0xfb840108,0x8a070101,0xfb840108,0x8a080101,0xfb840108,0x8a090101,
	0xfb840108,0x8a0a0101,0xfb840108,0x8a0b0101,0xfb840108,0x8a0c0101,0xfb840108,0x8a0d0101,
	0xfb840108,0x8a0e0101,0xfb840108,0x8a0f0101,0xfb840108,0x8a100101,0xfb840108,0x8a110101,
	0xfb840108,0x8a120101,0xfb840108,0x8a130101,0xfb840108,0x8a140101,0xfb840108,0x8a150101,
	0xfb840108,0x8a160101,0xfb840108,0x8a170101,0xfb840108,0x8a180101,0xfb840108,0x8a190101,
	0xfb840108,0x8a1a0101,0xfb840108,0x8a1b0101,0xfb840108,0x8a1c0101,0xfb840108,0x8a1d0101,
	0xfb840108,0x8a1e0101,0xfb840108,0x8a1f0101,0xfb840108,0x8a200101,0xfb840108,0x8a210101,
	0xfb840108,0x8a220101,0xfb840108,0x8a230101,0xfb840108,0x8a240101,0xfb840108,0x8a250101,
	0xfb840108,0x8a260101,0xfb840108,0x8a270101,0xfb840108,0x8a280101,0xfb840108,0x8a290101,
	0xfb840108,0x8a2a0101,0xfb840108,0x8a2b0101,0xfb840108,0x8a2c0101,0xfb840108,0x8a2d0101,
	0xfb840108,0x8a2e0101,0xfb840108,0x8a2f0101,0xfb840108,0x8a300101,0xfb840108,0x8a310101,
	0xfb840108,0x8a320101,0xfb840108,0x8a330101,0xfb840108,0x8a340101,0xfb840108,0x8a350101,
	0xfb840108,0x8a360101,0xfb840108,0x8a370101,0xfb840108,0x8a380101,0xfb840108,0x8a390101,
	0xfb840108,0x8a3a0101,0xfb840108,0x8a3b0101,0xfb840108,0x8a3c0101,0xfb840108,0x8a3d0101,
	0xfb840108,0x8a3e0101,0xfb840108,0x8a3f0101,0xfb840108,0x8a400101,0xfb840108,0x8a410101,
	0xfb840108,0x8a420101,0xfb840108,0x8a430101,0xfb840108,0x8a440101,0xfb840108,0x8a450101,
	0xfb840108,0x8a460101,0xfb840108,0x8a470101,0xfb840108,0x8a480101,0xfb840108,0x8a490101,
	0xfb840108,0x8a4a0101,0xfb840108,0x8a4b0101,0xfb840108,0x8a4c0101,0xfb840108,0x8a4d0101,
	0xfb840108,0x8a4e0101,0xfb840108,0x8a4f0101,0xfb840108,0x8a500101,0xfb840108,0x8a510101,
	0xfb840108,0x8a520101,0xfb840108,0x8a530101,0xfb840108,0x8a540101,0xfb840108,0x8a550101,
	0xfb840108,0x8a560101,0xfb840108,0x8a570101,0xfb840108,0x8a580101,0xfb840108,0x8a590101,
	0xfb840108,0x8a5a0101,0xfb840108,0x8a5b0101,0xfb840108,0x8a5c0101,0xfb840108,0x8a5d0101,
	0xfb840108,0x8a5e0101,0xfb840108,0x8a5f0101,0xfb840108,0x8a600101,0xfb840108,0x8a610101,
	0xfb840108,0x8a620101,0xfb840108,0x8a630101,0xfb840108,0x8a640101,0xfb840108,0x8a650101,
	0xfb840108,0x8a660101,0xfb840108,0x8a670101,0xfb840108,0x8a680101,0xfb840108,0x8a690101,
	0xfb840108,0x8a6a0101,0xfb840108,0x8a6b0101,0xfb840108,0x8a6c0101,0xfb840108,0x8a6d0101,
	0xfb840108,0x8a6e0101,0xfb840108,0x8a6f0101,0xfb840108,0x8a700101,0xfb840108,0x8a710101,
	0xfb840108,0x8a720101,0xfb840108,0x8a730101,0xfb840108,0x8a740101,0xfb840108,0x8a750101,
	0xfb840108,0x8a760101,0xfb840108,0x8a770101,0xfb840108,0x8a780101,0xfb840108,0x8a790101,
	0xfb840108,0x8a7a0101,0xfb840108,0x8a7b0101,0xfb840108,0x8a7c0101,0xfb840108,0x8a7d0101,
	0xfb840108,0x8a7e0101,0xfb840108,0x8a7f0101,0xfb840108,0x8a800101,0xfb840108,0x8a810101,
	0xfb840108,0x8a820101,0xfb840108,0x8a830101,0xfb840108,0x8a840101,0xfb840108,0x8a850101,
	0xfb840108,0x8a860101,0xfb840108,0x8a870101,0xfb840108,0x8a880101,0xfb840108,0x8a890101,
	0xfb840108,0x8a8a0101,0xfb840108,0x8a8b0101,0xfb840108,0x8a8c0101,0xfb840108,0x8a8d0101,
	0xfb840108,0x8a8e0101,0xfb840108,0x8a8f0101,0xfb840108,0x8a900101,0xfb840108,0x8a910101,
	0xfb840108,0x8a920101,0xfb840108,0x8a930101,0xfb840108,0x8a940101,0xfb840108,0x8a950101,
	0xfb840108,0x8a960101,0xfb840108,0x8a970101,0xfb840108,0x8a980101,0xfb840108,0x8a990101,
	0xfb840108,0x8a9a0101,0xfb840108,0x8a9b0101,0xfb840108,0x8a9c0101,0xfb840108,0x8a9d0101,
	0xfb840108,0x8a9e0101,0xfb840108,0x8a9f0101,0xfb840108,0x8aa00101,0xfb840108,0x8aa10101,
	0xfb840108,0x8aa20101,0xfb840108,0x8aa30101,0xfb840108,0x8aa40101,0xfb840108,0x8aa50101,
	0xfb840108,0x8aa60101,0xfb840108,0x8aa70101,0xfb840108,0x8aa80101,0xfb840108,0x8aa90101,
	0xfb840108,0x8aaa0101,0xfb840108,0x8aab0101,0xfb840108,0x8aac0101,0xfb840108,0x8aad0101,
	0xfb840108,0x8aae0101,0xfb840108,0x8aaf0101,0xfb840108,0x8ab00101,0xfb840108,0x8ab10101,
	0xfb840108,0x8ab20101,0xfb840108,0x8ab30101,0xfb840108,0x8ab40101,0xfb840108,0x8ab50101,
	0xfb840108,0x8ab60101,0xfb840108,0x8ab70101,0xfb840108,0x8ab80101,0xfb840108,0x8ab90101,
	0xfb840108,0x8aba0101,0xfb840108,0x8abb0101,0xfb840108,0x8abc0101,0xfb840108,0x8abd0101,
	0xfb840108,0x8abe0101,0xfb840108,0x8abf0101,0xfb840108,0x8ac00101,0xfb840108,0x8ac10101,
	0xfb840108,0x8ac20101,0xfb840108,0x8ac30101,0xfb840108,0x8ac40101,0xfb840108,0x8ac50101,
	0xfb840108,0x8ac60101,0xfb840108,0x8ac70101,0xfb840108,0x8ac80101,0xfb840108,0x8ac90101,
	0xfb840108,0x8aca0101,0xfb840108,0x8acb0101,0xfb840108,0x8acc0101,0xfb840108,0x8acd0101,
	0xfb840108,0x8ace0101,0xfb840108,0x8acf0101,0xfb840108,0x8ad00101,0xfb840108,0x8ad10101,
	0xfb840108,0x8ad20101,0xfb840108,0x8ad30101,0xfb840108,0x8ad40101,0xfb840108,0x8ad50101,
	0xfb840108,0x8ad60101,0xfb840108,0x8ad70101,0xfb840108,0x8ad80101,0xfb840108,0x8ad90101,
	0xfb840108,0x8ada0101,0xfb840108,0x8adb0101,0xfb840108,0x8adc0101,0xfb840108,0x8add0101,
	0xfb840108,0x8ade0101,0xfb840108,0x8adf0101,0xfb840108,0x8ae00101,0xfb840108,0x8ae10101,
	0xfb840108,0x8ae20101,0xfb840108,0x8ae30101,0xfb840108,0x8ae40101,0xfb840108,0x8ae50101,
	0xfb840108,0x8ae60101,0xfb840108,0x8ae70101,0xfb840108,0x8ae80101,0xfb840108,0x8ae90101,
	0xfb840108,0x8aea0101,0xfb840108,0x8aeb0101,0xfb840108,0x8aec0101,0xfb840108,0x8aed0101,
	0xfb840108,0x8aee0101,0xfb840108,0x8aef0101,0xfb840108,0x8af00101,0xfb840108,0x8af10101,
	0xfb840108,0x8af20101,0xfb840108,0x8af30101,0xfb840108,0x8af40101,0xfb840108,0x8af50101,
	0xfb840108,0x8af60101,0xfb840108,0x8af70101,0xfb840108,0x8af80101,0xfb840108,0x8af90101,
	0xfb840108,0x8afa0101,0xfb840108,0x8afb0101,0xfb840108,0x8afc0101,0xfb840108,0x8afd0101,
	0xfb840108,0x8afe0101,0xfb840108,0x8aff0101,0xfb840108,0x8b000101,0xfb840108,0x8b010101,
	0xfb840108,0x8b020101,0xfb840108,0x8b030101,0xfb840108,0x8b040101,0xfb840108,0x8b050101,
	0xfb840108,0x8b060101,0xfb840108,0x8b070101,0xfb840108,0x8b080101,0xfb840108,0x8b090101,
	0xfb840108,0x8b0a0101,0xfb840108,0x8b0b0101,0xfb840108,0x8b0c0101,0xfb840108,0x8b0d0101,
	0xfb840108,0x8b0e0101,0xfb840108,0x8b0f0101,0xfb840108,0x8b100101,0xfb840108,0x8b110101,
	0xfb840108,0x8b120101,0xfb840108,0x8b130101,0xfb840108,0x8b140101,0xfb840108,0x8b150101,
	0xfb840108,0x8b160101,0xfb840108,0x8b170101,0xfb840108,0x8b180101,0xfb840108,0x8b190101,
	0xfb840108,0x8b1a0101,0xfb840108,0x8b1b0101,0xfb840108,0x8b1c0101,0xfb840108,0x8b1d0101,
	0xfb840108,0x8b1e0101,0xfb840108,0x8b1f0101,0xfb840108,0x8b200101,0xfb840108,0x8b210101,
	0xfb840108,0x8b220101,0xfb840108,0x8b230101,0xfb840108,0x8b240101,0xfb840108,0x8b250101,
	0xfb840108,0x8b260101,0xfb840108,0x8b270101,0xfb840108,0x8b280101,0xfb840108,0x8b290101,
	0xfb840108,0x8b2a0101,0xfb840108,0x8b2b0101,0xfb840108,0x8b2c0101,0xfb840108,0x8b2d0101,
	0xfb840108,0x8b2e0101,0xfb840108,0x8b2f0101,0xfb840108,0x8b300101,0xfb840108,0x8b310101,
	0xfb840108,0x8b320101,0xfb840108,0x8b330101,0xfb840108,0x8b340101,0xfb840108,0x8b350101,
	0xfb840108,0x8b360101,0xfb840108,0x8b370101,0xfb840108,0x8b380101,0xfb840108,0x8b390101,
	0xfb840108,0x8b3a0101,0xfb840108,0x8b3b0101,0xfb840108,0x8b3c0101,0xfb840108,0x8b3d0101,
	0xfb840108,0x8b3e0101,0xfb840108,0x8b3f0101,0xfb840108,0x8b400101,0xfb840108,0x8b410101,
	0xfb840108,0x8b420101,0xfb840108,0x8b430101,0xfb840108,0x8b440101,0xfb840108,0x8b450101,
	0xfb840108,0x8b460101,0xfb840108,0x8b470101,0xfb840108,0x8b480101,0xfb840108,0x8b490101,
	0xfb840108,0x8b4a0101,0xfb840108,0x8b4b0101,0xfb840108,0x8b4c0101,0xfb840108,0x8b4d0101,
	0xfb840108,0x8b4e0101,0xfb840108,0x8b4f0101,0xfb840108,0x8b500101,0xfb840108,0x8b510101,
	0xfb840108,0x8b520101,0xfb840108,0x8b530101,0xfb840108,0x8b540101,0xfb840108,0x8b550101,
	0xfb840108,0x8b560101,0xfb840108,0x8b570101,0xfb840108,0x8b580101,0xfb840108,0x8b590101,
	0xfb840108,0x8b5a0101,0xfb840108,0x8b5b0101,0xfb840108,0x8b5c0101,0xfb840108,0x8b5d0101,
	0xfb840108,0x8b5e0101,0xfb840108,0x8b5f0101,0xfb840108,0x8b600101,0xfb840108,0x8b610101,
	0xfb840108,0x8b620101,0xfb840108,0x8b630101,0xfb840108,0x8b640101,0xfb840108,0x8b650101,
	0xfb840108,0x8b660101,0xfb840108,0x8b670101,0xfb840108,0x8b680101,0xfb840108,0x8b690101,
	0xfb840108,0x8b6a0101,0xfb840108,0x8b6b0101,0xfb840108,0x8b6c0101,0xfb840108,0x8b6d0101,
	0xfb840108,0x8b6e0101,0xfb840108,0x8b6f0101,0xfb840108,0x8b700101,0xfb840108,0x8b710101,
	0xfb840108,0x8b720101,0xfb840108,0x8b730101,0xfb840108,0x8b740101,0xfb840108,0x8b750101,
	0xfb840108,0x8b760101,0xfb840108,0x8b770101,0xfb840108,0x8b780101,0xfb840108,0x8b790101,
	0xfb840108,0x8b7a0101,0xfb840108,0x8b7b0101,0xfb840108,0x8b7c0101,0xfb840108,0x8b7d0101,
	0xfb840108,0x8b7e0101,0xfb840108,0x8b7f0101,0xfb840108,0x8b800101,0xfb840108,0x8b810101,
	0xfb840108,0x8b820101,0xfb840108,0x8b830101,0xfb840108,0x8b840101,0xfb840108,0x8b850101,
	0xfb840108,0x8b860101,0xfb840108,0x8b870101,0xfb840108,0x8b880101,0xfb840108,0x8b890101,
	0xfb840108,0x8b8a0101,0xfb840108,0x8b8b0101,0xfb840108,0x8b8c0101,0xfb840108,0x8b8d0101,
	0xfb840108,0x8b8e0101,0xfb840108,0x8b8f0101,0xfb840108,0x8b900101,0xfb840108,0x8b910101,
	0xfb840108,0x8b920101,0xfb840108,0x8b930101,0xfb840108,0x8b940101,0xfb840108,0x8b950101,
	0xfb840108,0x8b960101,0xfb840108,0x8b970101,0xfb840108,0x8b980101,0xfb840108,0x8b990101,
	0xfb840108,0x8b9a0101,0xfb840108,0x8b9b0101,0xfb840108,0x8b9c0101,0xfb840108,0x8b9d0101,
	0xfb840108,0x8b9e0101,0xfb840108,0x8b9f0101,0xfb840108,0x8ba00101,0xfb840108,0x8ba10101,
	0xfb840108,0x8ba20101,0xfb840108,0x8ba30101,0xfb840108,0x8ba40101,0xfb840108,0x8ba50101,
	0xfb840108,0x8ba60101,0xfb840108,0x8ba70101,0xfb840108,0x8ba80101,0xfb840108,0x8ba90101,
	0xfb840108,0x8baa0101,0xfb840108,0x8bab0101,0xfb840108,0x8bac0101,0xfb840108,0x8bad0101,
	0xfb840108,0x8bae0101,0xfb840108,0x8baf0101,0xfb840108,0x8bb00101,0xfb840108,0x8bb10101,
	0xfb840108,0x8bb20101,0xfb840108,0x8bb30101,0xfb840108,0x8bb40101,0xfb840108,0x8bb50101,
	0xfb840108,0x8bb60101,0xfb840108,0x8bb70101,0xfb840108,0x8bb80101,0xfb840108,0x8bb90101,
	0xfb840108,0x8bba0101,0xfb840108,0x8bbb0101,0xfb840108,0x8bbc0101,0xfb840108,0x8bbd0101,
	0xfb840108,0x8bbe0101,0xfb840108,0x8bbf0101,0xfb840108,0x8bc00101,0xfb840108,0x8bc10101,
	0xfb840108,0x8bc20101,0xfb840108,0x8bc30101,0xfb840108,0x8bc40101,0xfb840108,0x8bc50101,
	0xfb840108,0x8bc60101,0xfb840108,0x8bc70101,0xfb840108,0x8bc80101,0xfb840108,0x8bc90101,
	0xfb840108,0x8bca0101,0xfb840108,0x8bcb0101,0xfb840108,0x8bcc0101,0xfb840108,0x8bcd0101,
	0xfb840108,0x8bce0101,0xfb840108,0x8bcf0101,0xfb840108,0x8bd00101,0xfb840108,0x8bd10101,
	0xfb840108,0x8bd20101,0xfb840108,0x8bd30101,0xfb840108,0x8bd40101,0xfb840108,0x8bd50101,
	0xfb840108,0x8bd60101,0xfb840108,0x8bd70101,0xfb840108,0x8bd80101,0xfb840108,0x8bd90101,
	0xfb840108,0x8bda0101,0xfb840108,0x8bdb0101,0xfb840108,0x8bdc0101,0xfb840108,0x8bdd0101,
	0xfb840108,0x8bde0101,0xfb840108,0x8bdf0101,0xfb840108,0x8be00101,0xfb840108,0x8be10101,
	0xfb840108,0x8be20101,0xfb840108,0x8be30101,0xfb840108,0x8be40101,0xfb840108,0x8be50101,
	0xfb840108,0x8be60101,0xfb840108,0x8be70101,0xfb840108,0x8be80101,0xfb840108,0x8be90101,
	0xfb840108,0x8bea0101,0xfb840108,0x8beb0101,0xfb840108,0x8bec0101,0xfb840108,0x8bed0101,
	0xfb840108,0x8bee0101,0xfb840108,0x8bef0101,0xfb840108,0x8bf00101,0xfb840108,0x8bf10101,
	0xfb840108,0x8bf20101,0xfb840108,0x8bf30101,0xfb840108,0x8bf40101,0xfb840108,0x8bf50101,
	0xfb840108,0x8bf60101,0xfb840108,0x8bf70101,0xfb840108,0x8bf80101,0xfb840108,0x8bf90101,
	0xfb840108,0x8bfa0101,0xfb840108,0x8bfb0101,0xfb840108,0x8bfc0101,0xfb840108,0x8bfd0101,
	0xfb840108,0x8bfe0101,0xfb840108,0x8bff0101,0xfb840108,0x8c000101,0xfb840108,0x8c010101,
	0xfb840108,0x8c020101,0xfb840108,0x8c030101,0xfb840108,0x8c040101,0xfb840108,0x8c050101,
	0xfb840108,0x8c060101,0xfb840108,0x8c070101,0xfb840108,0x8c080101,0xfb840108,0x8c090101,
	0xfb840108,0x8c0a0101,0xfb840108,0x8c0b0101,0xfb840108,0x8c0c0101,0xfb840108,0x8c0d0101,
	0xfb840108,0x8c0e0101,0xfb840108,0x8c0f0101,0xfb840108,0x8c100101,0xfb840108,0x8c110101,
	0xfb840108,0x8c120101,0xfb840108,0x8c130101,0xfb840108,0x8c140101,0xfb840108,0x8c150101,
	0xfb840108,0x8c160101,0xfb840108,0x8c170101,0xfb840108,0x8c180101,0xfb840108,0x8c190101,
	0xfb840108,0x8c1a0101,0xfb840108,0x8c1b0101,0xfb840108,0x8c1c0101,0xfb840108,0x8c1d0101,
	0xfb840108,0x8c1e0101,0xfb840108,0x8c1f0101,0xfb840108,0x8c200101,0xfb840108,0x8c210101,
	0xfb840108,0x8c220101,0xfb840108,0x8c230101,0xfb840108,0x8c240101,0xfb840108,0x8c250101,
	0xfb840108,0x8c260101,0xfb840108,0x8c270101,0xfb840108,0x8c280101,0xfb840108,0x8c290101,
	0xfb840108,0x8c2a0101,0xfb840108,0x8c2b0101,0xfb840108,0x8c2c0101,0xfb840108,0x8c2d0101,
	0xfb840108,0x8c2e0101,0xfb840108,0x8c2f0101,0xfb840108,0x8c300101,0xfb840108,0x8c310101,
	0xfb840108,0x8c320101,0xfb840108,0x8c330101,0xfb840108,0x8c340101,0xfb840108,0x8c350101,
	0xfb840108,0x8c360101,0xfb840108,0x8c370101,0xfb840108,0x8c380101,0xfb840108,0x8c390101,
	0xfb840108,0x8c3a0101,0xfb840108,0x8c3b0101,0xfb840108,0x8c3c0101,0xfb840108,0x8c3d0101,
	0xfb840108,0x8c3e0101,0xfb840108,0x8c3f0101,0xfb840108,0x8c400101,0xfb840108,0x8c410101,
	0xfb840108,0x8c420101,0xfb840108,0x8c430101,0xfb840108,0x8c440101,0xfb840108,0x8c450101,
	0xfb840108,0x8c460101,0xfb840108,0x8c470101,0xfb840108,0x8c480101,0xfb840108,0x8c490101,
	0xfb840108,0x8c4a0101,0xfb840108,0x8c4b0101,0xfb840108,0x8c4c0101,0xfb840108,0x8c4d0101,
	0xfb840108,0x8c4e0101,0xfb840108,0x8c4f0101,0xfb840108,0x8c500101,0xfb840108,0x8c510101,
	0xfb840108,0x8c520101,0xfb840108,0x8c530101,0xfb840108,0x8c540101,0xfb840108,0x8c550101,
	0xfb840108,0x8c560101,0xfb840108,0x8c570101,0xfb840108,0x8c580101,0xfb840108,0x8c590101,
	0xfb840108,0x8c5a0101,0xfb840108,0x8c5b0101,0xfb840108,0x8c5c0101,0xfb840108,0x8c5d0101,
	0xfb840108,0x8c5e0101,0xfb840108,0x8c5f0101,0xfb840108,0x8c600101,0xfb840108,0x8c610101,
	0xfb840108,0x8c620101,0xfb840108,0x8c630101,0xfb840108,0x8c640101,0xfb840108,0x8c650101,
	0xfb840108,0x8c660101,0xfb840108,0x8c670101,0xfb840108,0x8c680101,0xfb840108,0x8c690101,
	0xfb840108,0x8c6a0101,0xfb840108,0x8c6b0101,0xfb840108,0x8c6c0101,0xfb840108,0x8c6d0101,
	0xfb840108,0x8c6e0101,0xfb840108,0x8c6f0101,0xfb840108,0x8c700101,0xfb840108,0x8c710101,
	0xfb840108,0x8c720101,0xfb840108,0x8c730101,0xfb840108,0x8c740101,0xfb840108,0x8c750101,
	0xfb840108,0x8c760101,0xfb840108,0x8c770101,0xfb840108,0x8c780101,0xfb840108,0x8c790101,
	0xfb840108,0x8c7a0101,0xfb840108,0x8c7b0101,0xfb840108,0x8c7c0101,0xfb840108,0x8c7d0101,
	0xfb840108,0x8c7e0101,0xfb840108,0x8c7f0101,0xfb840108,0x8c800101,0xfb840108,0x8c810101,
	0xfb840108,0x8c820101,0xfb840108,0x8c830101,0xfb840108,0x8c840101,0xfb840108,0x8c850101,
	0xfb840108,0x8c860101,0xfb840108,0x8c870101,0xfb840108,0x8c880101,0xfb840108,0x8c890101,
	0xfb840108,0x8c8a0101,0xfb840108,0x8c8b0101,0xfb840108,0x8c8c0101,0xfb840108,0x8c8d0101,
	0xfb840108,0x8c8e0101,0xfb840108,0x8c8f0101,0xfb840108,0x8c900101,0xfb840108,0x8c910101,
	0xfb840108,0x8c920101,0xfb840108,0x8c930101,0xfb840108,0x8c940101,0xfb840108,0x8c950101,
	0xfb840108,0x8c960101,0xfb840108,0x8c970101,0xfb840108,0x8c980101,0xfb840108,0x8c990101,
	0xfb840108,0x8c9a0101,0xfb840108,0x8c9b0101,0xfb840108,0x8c9c0101,0xfb840108,0x8c9d0101,
	0xfb840108,0x8c9e0101,0xfb840108,0x8c9f0101,0xfb840108,0x8ca00101,0xfb840108,0x8ca10101,
	0xfb840108,0x8ca20101,0xfb840108,0x8ca30101,0xfb840108,0x8ca40101,0xfb840108,0x8ca50101,
	0xfb840108,0x8ca60101,0xfb840108,0x8ca70101,0xfb840108,0x8ca80101,0xfb840108,0x8ca90101,
	0xfb840108,0x8caa0101,0xfb840108,0x8cab0101,0xfb840108,0x8cac0101,0xfb840108,0x8cad0101,
	0xfb840108,0x8cae0101,0xfb840108,0x8caf0101,0xfb840108,0x8cb00101,0xfb840108,0x8cb10101,
	0xfb840108,0x8cb20101,0xfb840108,0x8cb30101,0xfb840108,0x8cb40101,0xfb840108,0x8cb50101,
	0xfb840108,0x8cb60101,0xfb840108,0x8cb70101,0xfb840108,0x8cb80101,0xfb840108,0x8cb90101,
	0xfb840108,0x8cba0101,0xfb840108,0x8cbb0101,0xfb840108,0x8cbc0101,0xfb840108,0x8cbd0101,
	0xfb840108,0x8cbe0101,0xfb840108,0x8cbf0101,0xfb840108,0x8cc00101,0xfb840108,0x8cc10101,
	0xfb840108,0x8cc20101,0xfb840108,0x8cc30101,0xfb840108,0x8cc40101,0xfb840108,0x8cc50101,
	0xfb840108,0x8cc60101,0xfb840108,0x8cc70101,0xfb840108,0x8cc80101,0xfb840108,0x8cc90101,
	0xfb840108,0x8cca0101,0xfb840108,0x8ccb0101,0xfb840108,0x8ccc0101,0xfb840108,0x8ccd0101,
	0xfb840108,0x8cce0101,0xfb840108,0x8ccf0101,0xfb840108,0x8cd00101,0xfb840108,0x8cd10101,
	0xfb840108,0x8cd20101,0xfb840108,0x8cd30101,0xfb840108,0x8cd40101,0xfb840108,0x8cd50101,
	0xfb840108,0x8cd60101,0xfb840108,0x8cd70101,0xfb840108,0x8cd80101,0xfb840108,0x8cd90101,
	0xfb840108,0x8cda0101,0xfb840108,0x8cdb0101,0xfb840108,0x8cdc0101,0xfb840108,0x8cdd0101,
	0xfb840108,0x8cde0101,0xfb840108,0x8cdf0101,0xfb840108,0x8ce00101,0xfb840108,0x8ce10101,
	0xfb840108,0x8ce20101,0xfb840108,0x8ce30101,0xfb840108,0x8ce40101,0xfb840108,0x8ce50101,
	0xfb840108,0x8ce60101,0xfb840108,0x8ce70101,0xfb840108,0x8ce80101,0xfb840108,0x8ce90101,
	0xfb840108,0x8cea0101,0xfb840108,0x8ceb0101,0xfb840108,0x8cec0101,0xfb840108,0x8ced0101,
	0xfb840108,0x8cee0101,0xfb840108,0x8cef0101,0xfb840108,0x8cf00101,0xfb840108,0x8cf10101,
	0xfb840108,0x8cf20101,0xfb840108,0x8cf30101,0xfb840108,0x8cf40101,0xfb840108,0x8cf50101,
	0xfb840108,0x8cf60101,0xfb840108,0x8cf70101,0xfb840108,0x8cf80101,0xfb840108,0x8cf90101,
	0xfb840108,0x8cfa0101,0xfb840108,0x8cfb0101,0xfb840108,0x8cfc0101,0xfb840108,0x8cfd0101,
	0xfb840108,0x8cfe0101,0xfb840108,0x8cff0101,0xfb840108,0x8d000101,0xfb840108,0x8d010101,
	0xfb840108,0x8d020101,0xfb840108,0x8d030101,0xfb840108,0x8d040101,0xfb840108,0x8d050101,
	0xfb840108,0x8d060101,0xfb840108,0x8d070101,0xfb840108,0x8d080101,0xfb840108,0x8d090101,
	0xfb840108,0x8d0a0101,0xfb840108,0x8d0b0101,0xfb840108,0x8d0c0101,0xfb840108,0x8d0d0101,
	0xfb840108,0x8d0e0101,0xfb840108,0x8d0f0101,0xfb840108,0x8d100101,0xfb840108,0x8d110101,
	0xfb840108,0x8d120101,0xfb840108,0x8d130101,0xfb840108,0x8d140101,0xfb840108,0x8d150101,
	0xfb840108,0x8d160101,0xfb840108,0x8d170101,0xfb840108,0x8d180101,0xfb840108,0x8d190101,
	0xfb840108,0x8d1a0101,0xfb840108,0x8d1b0101,0xfb840108,0x8d1c0101,0xfb840108,0x8d1d0101,
	0xfb840108,0x8d1e0101,0xfb840108,0x8d1f0101,0xfb840108,0x8d200101,0xfb840108,0x8d210101,
	0xfb840108,0x8d220101,0xfb840108,0x8d230101,0xfb840108,0x8d240101,0xfb840108,0x8d250101,
	0xfb840108,0x8d260101,0xfb840108,0x8d270101,0xfb840108,0x8d280101,0xfb840108,0x8d290101,
	0xfb840108,0x8d2a0101,0xfb840108,0x8d2b0101,0xfb840108,0x8d2c0101,0xfb840108,0x8d2d0101,
	0xfb840108,0x8d2e0101,0xfb840108,0x8d2f0101,0xfb840108,0x8d300101,0xfb840108,0x8d310101,
	0xfb840108,0x8d320101,0xfb840108,0x8d330101,0xfb840108,0x8d340101,0xfb840108,0x8d350101,
	0xfb840108,0x8d360101,0xfb840108,0x8d370101,0xfb840108,0x8d380101,0xfb840108,0x8d390101,
	0xfb840108,0x8d3a0101,0xfb840108,0x8d3b0101,0xfb840108,0x8d3c0101,0xfb840108,0x8d3d0101,
	0xfb840108,0x8d3e0101,0xfb840108,0x8d3f0101,0xfb840108,0x8d400101,0xfb840108,0x8d410101,
	0xfb840108,0x8d420101,0xfb840108,0x8d430101,0xfb840108,0x8d440101,0xfb840108,0x8d450101,
	0xfb840108,0x8d460101,0xfb840108,0x8d470101,0xfb840108,0x8d480101,0xfb840108,0x8d490101,
	0xfb840108,0x8d4a0101,0xfb840108,0x8d4b0101,0xfb840108,0x8d4c0101,0xfb840108,0x8d4d0101,
	0xfb840108,0x8d4e0101,0xfb840108,0x8d4f0101,0xfb840108,0x8d500101,0xfb840108,0x8d510101,
	0xfb840108,0x8d520101,0xfb840108,0x8d530101,0xfb840108,0x8d540101,0xfb840108,0x8d550101,
	0xfb840108,0x8d560101,0xfb840108,0x8d570101,0xfb840108,0x8d580101,0xfb840108,0x8d590101,
	0xfb840108,0x8d5a0101,0xfb840108,0x8d5b0101,0xfb840108,0x8d5c0101,0xfb840108,0x8d5d0101,
	0xfb840108,0x8d5e0101,0xfb840108,0x8d5f0101,0xfb840108,0x8d600101,0xfb840108,0x8d610101,
	0xfb840108,0x8d620101,0xfb840108,0x8d630101,0xfb840108,0x8d640101,0xfb840108,0x8d650101,
	0xfb840108,0x8d660101,0xfb840108,0x8d670101,0xfb840108,0x8d680101,0xfb840108,0x8d690101,
	0xfb840108,0x8d6a0101,0xfb840108,0x8d6b0101,0xfb840108,0x8d6c0101,0xfb840108,0x8d6d0101,
	0xfb840108,0x8d6e0101,0xfb840108,0x8d6f0101,0xfb840108,0x8d700101,0xfb840108,0x8d710101,
	0xfb840108,0x8d720101,0xfb840108,0x8d730101,0xfb840108,0x8d740101,0xfb840108,0x8d750101,
	0xfb840108,0x8d760101,0xfb840108,0x8d770101,0xfb840108,0x8d780101,0xfb840108,0x8d790101,
	0xfb840108,0x8d7a0101,0xfb840108,0x8d7b0101,0xfb840108,0x8d7c0101,0xfb840108,0x8d7d0101,
	0xfb840108,0x8d7e0101,0xfb840108,0x8d7f0101,0xfb840108,0x8d800101,0xfb840108,0x8d810101,
	0xfb840108,0x8d820101,0xfb840108,0x8d830101,0xfb840108,0x8d840101,0xfb840108,0x8d850101,
	0xfb840108,0x8d860101,0xfb840108,0x8d870101,0xfb840108,0x8d880101,0xfb840108,0x8d890101,
	0xfb840108,0x8d8a0101,0xfb840108,0x8d8b0101,0xfb840108,0x8d8c0101,0xfb840108,0x8d8d0101,
	0xfb840108,0x8d8e0101,0xfb840108,0x8d8f0101,0xfb840108,0x8d900101,0xfb840108,0x8d910101,
	0xfb840108,0x8d920101,0xfb840108,0x8d930101,0xfb840108,0x8d940101,0xfb840108,0x8d950101,
	0xfb840108,0x8d960101,0xfb840108,0x8d970101,0xfb840108,0x8d980101,0xfb840108,0x8d990101,
	0xfb840108,0x8d9a0101,0xfb840108,0x8d9b0101,0xfb840108,0x8d9c0101,0xfb840108,0x8d9d0101,
	0xfb840108,0x8d9e0101,0xfb840108,0x8d9f0101,0xfb840108,0x8da00101,0xfb840108,0x8da10101,
	0xfb840108,0x8da20101,0xfb840108,0x8da30101,0xfb840108,0x8da40101,0xfb840108,0x8da50101,
	0xfb840108,0x8da60101,0xfb840108,0x8da70101,0xfb840108,0x8da80101,0xfb840108,0x8da90101,
	0xfb840108,0x8daa0101,0xfb840108,0x8dab0101,0xfb840108,0x8dac0101,0xfb840108,0x8dad0101,
	0xfb840108,0x8dae0101,0xfb840108,0x8daf0101,0xfb840108,0x8db00101,0xfb840108,0x8db10101,
	0xfb840108,0x8db20101,0xfb840108,0x8db30101,0xfb840108,0x8db40101,0xfb840108,0x8db50101,
	0xfb840108,0x8db60101,0xfb840108,0x8db70101,0xfb840108,0x8db80101,0xfb840108,0x8db90101,
	0xfb840108,0x8dba0101,0xfb840108,0x8dbb0101,0xfb840108,0x8dbc0101,0xfb840108,0x8dbd0101,
	0xfb840108,0x8dbe0101,0xfb840108,0x8dbf0101,0xfb840108,0x8dc00101,0xfb840108,0x8dc10101,
	0xfb840108,0x8dc20101,0xfb840108,0x8dc30101,0xfb840108,0x8dc40101,0xfb840108,0x8dc50101,
	0xfb840108,0x8dc60101,0xfb840108,0x8dc70101,0xfb840108,0x8dc80101,0xfb840108,0x8dc90101,
	0xfb840108,0x8dca0101,0xfb840108,0x8dcb0101,0xfb840108,0x8dcc0101,0xfb840108,0x8dcd0101,
	0xfb840108,0x8dce0101,0xfb840108,0x8dcf0101,0xfb840108,0x8dd00101,0xfb840108,0x8dd10101,
	0xfb840108,0x8dd20101,0xfb840108,0x8dd30101,0xfb840108,0x8dd40101,0xfb840108,0x8dd50101,
	0xfb840108,0x8dd60101,0xfb840108,0x8dd70101,0xfb840108,0x8dd80101,0xfb840108,0x8dd90101,
	0xfb840108,0x8dda0101,0xfb840108,0x8ddb0101,0xfb840108,0x8ddc0101,0xfb840108,0x8ddd0101,
	0xfb840108,0x8dde0101,0xfb840108,0x8ddf0101,0xfb840108,0x8de00101,0xfb840108,0x8de10101,
	0xfb840108,0x8de20101,0xfb840108,0x8de30101,0xfb840108,0x8de40101,0xfb840108,0x8de50101,
	0xfb840108,0x8de60101,0xfb840108,0x8de70101,0xfb840108,0x8de80101,0xfb840108,0x8de90101,
	0xfb840108,0x8dea0101,0xfb840108,0x8deb0101,0xfb840108,0x8dec0101,0xfb840108,0x8ded0101,
	0xfb840108,0x8dee0101,0xfb840108,0x8def0101,0xfb840108,0x8df00101,0xfb840108,0x8df10101,
	0xfb840108,0x8df20101,0xfb840108,0x8df30101,0xfb840108,0x8df40101,0xfb840108,0x8df50101,
	0xfb840108,0x8df60101,0xfb840108,0x8df70101,0xfb840108,0x8df80101,0xfb840108,0x8df90101,
	0xfb840108,0x8dfa0101,0xfb840108,0x8dfb0101,0xfb840108,0x8dfc0101,0xfb840108,0x8dfd0101,
	0xfb840108,0x8dfe0101,0xfb840108,0x8dff0101,0xfb840108,0x8e000101,0xfb840108,0x8e010101,
	0xfb840108,0x8e020101,0xfb840108,0x8e030101,0xfb840108,0x8e040101,0xfb840108,0x8e050101,
	0xfb840108,0x8e060101,0xfb840108,0x8e070101,0xfb840108,0x8e080101,0xfb840108,0x8e090101,
	0xfb840108,0x8e0a0101,0xfb840108,0x8e0b0101,0xfb840108,0x8e0c0101,0xfb840108,0x8e0d0101,
	0xfb840108,0x8e0e0101,0xfb840108,0x8e0f0101,0xfb840108,0x8e100101,0xfb840108,0x8e110101,
	0xfb840108,0x8e120101,0xfb840108,0x8e130101,0xfb840108,0x8e140101,0xfb840108,0x8e150101,
	0xfb840108,0x8e160101,0xfb840108,0x8e170101,0xfb840108,0x8e180101,0xfb840108,0x8e190101,
	0xfb840108,0x8e1a0101,0xfb840108,0x8e1b0101,0xfb840108,0x8e1c0101,0xfb840108,0x8e1d0101,
	0xfb840108,0x8e1e0101,0xfb840108,0x8e1f0101,0xfb840108,0x8e200101,0xfb840108,0x8e210101,
	0xfb840108,0x8e220101,0xfb840108,0x8e230101,0xfb840108,0x8e240101,0xfb840108,0x8e250101,
	0xfb840108,0x8e260101,0xfb840108,0x8e270101,0xfb840108,0x8e280101,0xfb840108,0x8e290101,
	0xfb840108,0x8e2a0101,0xfb840108,0x8e2b0101,0xfb840108,0x8e2c0101,0xfb840108,0x8e2d0101,
	0xfb840108,0x8e2e0101,0xfb840108,0x8e2f0101,0xfb840108,0x8e300101,0xfb840108,0x8e310101,
	0xfb840108,0x8e320101,0xfb840108,0x8e330101,0xfb840108,0x8e340101,0xfb840108,0x8e350101,
	0xfb840108,0x8e360101,0xfb840108,0x8e370101,0xfb840108,0x8e380101,0xfb840108,0x8e390101,
	0xfb840108,0x8e3a0101,0xfb840108,0x8e3b0101,0xfb840108,0x8e3c0101,0xfb840108,0x8e3d0101,
	0xfb840108,0x8e3e0101,0xfb840108,0x8e3f0101,0xfb840108,0x8e400101,0xfb840108,0x8e410101,
	0xfb840108,0x8e420101,0xfb840108,0x8e430101,0xfb840108,0x8e440101,0xfb840108,0x8e450101,
	0xfb840108,0x8e460101,0xfb840108,0x8e470101,0xfb840108,0x8e480101,0xfb840108,0x8e490101,
	0xfb840108,0x8e4a0101,0xfb840108,0x8e4b0101,0xfb840108,0x8e4c0101,0xfb840108,0x8e4d0101,
	0xfb840108,0x8e4e0101,0xfb840108,0x8e4f0101,0xfb840108,0x8e500101,0xfb840108,0x8e510101,
	0xfb840108,0x8e520101,0xfb840108,0x8e530101,0xfb840108,0x8e540101,0xfb840108,0x8e550101,
	0xfb840108,0x8e560101,0xfb840108,0x8e570101,0xfb840108,0x8e580101,0xfb840108,0x8e590101,
	0xfb840108,0x8e5a0101,0xfb840108,0x8e5b0101,0xfb840108,0x8e5c0101,0xfb840108,0x8e5d0101,
	0xfb840108,0x8e5e0101,0xfb840108,0x8e5f0101,0xfb840108,0x8e600101,0xfb840108,0x8e610101,
	0xfb840108,0x8e620101,0xfb840108,0x8e630101,0xfb840108,0x8e640101,0xfb840108,0x8e650101,
	0xfb840108,0x8e660101,0xfb840108,0x8e670101,0xfb840108,0x8e680101,0xfb840108,0x8e690101,
	0xfb840108,0x8e6a0101,0xfb840108,0x8e6b0101,0xfb840108,0x8e6c0101,0xfb840108,0x8e6d0101,
	0xfb840108,0x8e6e0101,0xfb840108,0x8e6f0101,0xfb840108,0x8e700101,0xfb840108,0x8e710101,
	0xfb840108,0x8e720101,0xfb840108,0x8e730101,0xfb840108,0x8e740101,0xfb840108,0x8e750101,
	0xfb840108,0x8e760101,0xfb840108,0x8e770101,0xfb840108,0x8e780101,0xfb840108,0x8e790101,
	0xfb840108,0x8e7a0101,0xfb840108,0x8e7b0101,0xfb840108,0x8e7c0101,0xfb840108,0x8e7d0101,
	0xfb840108,0x8e7e0101,0xfb840108,0x8e7f0101,0xfb840108,0x8e800101,0xfb840108,0x8e810101,
	0xfb840108,0x8e820101,0xfb840108,0x8e830101,0xfb840108,0x8e840101,0xfb840108,0x8e850101,
	0xfb840108,0x8e860101,0xfb840108,0x8e870101,0xfb840108,0x8e880101,0xfb840108,0x8e890101,
	0xfb840108,0x8e8a0101,0xfb840108,0x8e8b0101,0xfb840108,0x8e8c0101,0xfb840108,0x8e8d0101,
	0xfb840108,0x8e8e0101,0xfb840108,0x8e8f0101,0xfb840108,0x8e900101,0xfb840108,0x8e910101,
	0xfb840108,0x8e920101,0xfb840108,0x8e930101,0xfb840108,0x8e940101,0xfb840108,0x8e950101,
	0xfb840108,0x8e960101,0xfb840108,0x8e970101,0xfb840108,0x8e980101,0xfb840108,0x8e990101,
	0xfb840108,0x8e9a0101,0xfb840108,0x8e9b0101,0xfb840108,0x8e9c0101,0xfb840108,0x8e9d0101,
	0xfb840108,0x8e9e0101,0xfb840108,0x8e9f0101,0xfb840108,0x8ea00101,0xfb840108,0x8ea10101,
	0xfb840108,0x8ea20101,0xfb840108,0x8ea30101,0xfb840108,0x8ea40101,0xfb840108,0x8ea50101,
	0xfb840108,0x8ea60101,0xfb840108,0x8ea70101,0xfb840108,0x8ea80101,0xfb840108,0x8ea90101,
	0xfb840108,0x8eaa0101,0xfb840108,0x8eab0101,0xfb840108,0x8eac0101,0xfb840108,0x8ead0101,
	0xfb840108,0x8eae0101,0xfb840108,0x8eaf0101,0xfb840108,0x8eb00101,0xfb840108,0x8eb10101,
	0xfb840108,0x8eb20101,0xfb840108,0x8eb30101,0xfb840108,0x8eb40101,0xfb840108,0x8eb50101,
	0xfb840108,0x8eb60101,0xfb840108,0x8eb70101,0xfb840108,0x8eb80101,0xfb840108,0x8eb90101,
	0xfb840108,0x8eba0101,0xfb840108,0x8ebb0101,0xfb840108,0x8ebc0101,0xfb840108,0x8ebd0101,
	0xfb840108,0x8ebe0101,0xfb840108,0x8ebf0101,0xfb840108,0x8ec00101,0xfb840108,0x8ec10101,
	0xfb840108,0x8ec20101,0xfb840108,0x8ec30101,0xfb840108,0x8ec40101,0xfb840108,0x8ec50101,
	0xfb840108,0x8ec60101,0xfb840108,0x8ec70101,0xfb840108,0x8ec80101,0xfb840108,0x8ec90101,
	0xfb840108,0x8eca0101,0xfb840108,0x8ecb0101,0xfb840108,0x8ecc0101,0xfb840108,0x8ecd0101,
	0xfb840108,0x8ece0101,0xfb840108,0x8ecf0101,0xfb840108,0x8ed00101,0xfb840108,0x8ed10101,
	0xfb840108,0x8ed20101,0xfb840108,0x8ed30101,0xfb840108,0x8ed40101,0xfb840108,0x8ed50101,
	0xfb840108,0x8ed60101,0xfb840108,0x8ed70101,0xfb840108,0x8ed80101,0xfb840108,0x8ed90101,
	0xfb840108,0x8eda0101,0xfb840108,0x8edb0101,0xfb840108,0x8edc0101,0xfb840108,0x8edd0101,
	0xfb840108,0x8ede0101,0xfb840108,0x8edf0101,0xfb840108,0x8ee00101,0xfb840108,0x8ee10101,
	0xfb840108,0x8ee20101,0xfb840108,0x8ee30101,0xfb840108,0x8ee40101,0xfb840108,0x8ee50101,
	0xfb840108,0x8ee60101,0xfb840108,0x8ee70101,0xfb840108,0x8ee80101,0xfb840108,0x8ee90101,
	0xfb840108,0x8eea0101,0xfb840108,0x8eeb0101,0xfb840108,0x8eec0101,0xfb840108,0x8eed0101,
	0xfb840108,0x8eee0101,0xfb840108,0x8eef0101,0xfb840108,0x8ef00101,0xfb840108,0x8ef10101,
	0xfb840108,0x8ef20101,0xfb840108,0x8ef30101,0xfb840108,0x8ef40101,0xfb840108,0x8ef50101,
	0xfb840108,0x8ef60101,0xfb840108,0x8ef70101,0xfb840108,0x8ef80101,0xfb840108,0x8ef90101,
	0xfb840108,0x8efa0101,0xfb840108,0x8efb0101,0xfb840108,0x8efc0101,0xfb840108,0x8efd0101,
	0xfb840108,0x8efe0101,0xfb840108,0x8eff0101,0xfb840108,0x8f000101,0xfb840108,0x8f010101,
	0xfb840108,0x8f020101,0xfb840108,0x8f030101,0xfb840108,0x8f040101,0xfb840108,0x8f050101,
	0xfb840108,0x8f060101,0xfb840108,0x8f070101,0xfb840108,0x8f080101,0xfb840108,0x8f090101,
	0xfb840108,0x8f0a0101,0xfb840108,0x8f0b0101,0xfb840108,0x8f0c0101,0xfb840108,0x8f0d0101,
	0xfb840108,0x8f0e0101,0xfb840108,0x8f0f0101,0xfb840108,0x8f100101,0xfb840108,0x8f110101,
	0xfb840108,0x8f120101,0xfb840108,0x8f130101,0xfb840108,0x8f140101,0xfb840108,0x8f150101,
	0xfb840108,0x8f160101,0xfb840108,0x8f170101,0xfb840108,0x8f180101,0xfb840108,0x8f190101,
	0xfb840108,0x8f1a0101,0xfb840108,0x8f1b0101,0xfb840108,0x8f1c0101,0xfb840108,0x8f1d0101,
	0xfb840108,0x8f1e0101,0xfb840108,0x8f1f0101,0xfb840108,0x8f200101,0xfb840108,0x8f210101,
	0xfb840108,0x8f220101,0xfb840108,0x8f230101,0xfb840108,0x8f240101,0xfb840108,0x8f250101,
	0xfb840108,0x8f260101,0xfb840108,0x8f270101,0xfb840108,0x8f280101,0xfb840108,0x8f290101,
	0xfb840108,0x8f2a0101,0xfb840108,0x8f2b0101,0xfb840108,0x8f2c0101,0xfb840108,0x8f2d0101,
	0xfb840108,0x8f2e0101,0xfb840108,0x8f2f0101,0xfb840108,0x8f300101,0xfb840108,0x8f310101,
	0xfb840108,0x8f320101,0xfb840108,0x8f330101,0xfb840108,0x8f340101,0xfb840108,0x8f350101,
	0xfb840108,0x8f360101,0xfb840108,0x8f370101,0xfb840108,0x8f380101,0xfb840108,0x8f390101,
	0xfb840108,0x8f3a0101,0xfb840108,0x8f3b0101,0xfb840108,0x8f3c0101,0xfb840108,0x8f3d0101,
	0xfb840108,0x8f3e0101,0xfb840108,0x8f3f0101,0xfb840108,0x8f400101,0xfb840108,0x8f410101,
	0xfb840108,0x8f420101,0xfb840108,0x8f430101,0xfb840108,0x8f440101,0xfb840108,0x8f450101,
	0xfb840108,0x8f460101,0xfb840108,0x8f470101,0xfb840108,0x8f480101,0xfb840108,0x8f490101,
	0xfb840108,0x8f4a0101,0xfb840108,0x8f4b0101,0xfb840108,0x8f4c0101,0xfb840108,0x8f4d0101,
	0xfb840108,0x8f4e0101,0xfb840108,0x8f4f0101,0xfb840108,0x8f500101,0xfb840108,0x8f510101,
	0xfb840108,0x8f520101,0xfb840108,0x8f530101,0xfb840108,0x8f540101,0xfb840108,0x8f550101,
	0xfb840108,0x8f560101,0xfb840108,0x8f570101,0xfb840108,0x8f580101,0xfb840108,0x8f590101,
	0xfb840108,0x8f5a0101,0xfb840108,0x8f5b0101,0xfb840108,0x8f5c0101,0xfb840108,0x8f5d0101,
	0xfb840108,0x8f5e0101,0xfb840108,0x8f5f0101,0xfb840108,0x8f600101,0xfb840108,0x8f610101,
	0xfb840108,0x8f620101,0xfb840108,0x8f630101,0xfb840108,0x8f640101,0xfb840108,0x8f650101,
	0xfb840108,0x8f660101,0xfb840108,0x8f670101,0xfb840108,0x8f680101,0xfb840108,0x8f690101,
	0xfb840108,0x8f6a0101,0xfb840108,0x8f6b0101,0xfb840108,0x8f6c0101,0xfb840108,0x8f6d0101,
	0xfb840108,0x8f6e0101,0xfb840108,0x8f6f0101,0xfb840108,0x8f700101,0xfb840108,0x8f710101,
	0xfb840108,0x8f720101,0xfb840108,0x8f730101,0xfb840108,0x8f740101,0xfb840108,0x8f750101,
	0xfb840108,0x8f760101,0xfb840108,0x8f770101,0xfb840108,0x8f780101,0xfb840108,0x8f790101,
	0xfb840108,0x8f7a0101,0xfb840108,0x8f7b0101,0xfb840108,0x8f7c0101,0xfb840108,0x8f7d0101,
	0xfb840108,0x8f7e0101,0xfb840108,0x8f7f0101,0xfb840108,0x8f800101,0xfb840108,0x8f810101,
	0xfb840108,0x8f820101,0xfb840108,0x8f830101,0xfb840108,0x8f840101,0xfb840108,0x8f850101,
	0xfb840108,0x8f860101,0xfb840108,0x8f870101,0xfb840108,0x8f880101,0xfb840108,0x8f890101,
	0xfb840108,0x8f8a0101,0xfb840108,0x8f8b0101,0xfb840108,0x8f8c0101,0xfb840108,0x8f8d0101,
	0xfb840108,0x8f8e0101,0xfb840108,0x8f8f0101,0xfb840108,0x8f900101,0xfb840108,0x8f910101,
	0xfb840108,0x8f920101,0xfb840108,0x8f930101,0xfb840108,0x8f940101,0xfb840108,0x8f950101,
	0xfb840108,0x8f960101,0xfb840108,0x8f970101,0xfb840108,0x8f980101,0xfb840108,0x8f990101,
	0xfb840108,0x8f9a0101,0xfb840108,0x8f9b0101,0xfb840108,0x8f9c0101,0xfb840108,0x8f9d0101,
	0xfb840108,0x8f9e0101,0xfb840108,0x8f9f0101,0xfb840108,0x8fa00101,0xfb840108,0x8fa10101,
	0xfb840108,0x8fa20101,0xfb840108,0x8fa30101,0xfb840108,0x8fa40101,0xfb840108,0x8fa50101,
	0xfb840108,0x8fa60101,0xfb840108,0x8fa70101,0xfb840108,0x8fa80101,0xfb840108,0x8fa90101,
	0xfb840108,0x8faa0101,0xfb840108,0x8fab0101,0xfb840108,0x8fac0101,0xfb840108,0x8fad0101,
	0xfb840108,0x8fae0101,0xfb840108,0x8faf0101,0xfb840108,0x8fb00101,0xfb840108,0x8fb10101,
	0xfb840108,0x8fb20101,0xfb840108,0x8fb30101,0xfb840108,0x8fb40101,0xfb840108,0x8fb50101,
	0xfb840108,0x8fb60101,0xfb840108,0x8fb70101,0xfb840108,0x8fb80101,0xfb840108,0x8fb90101,
	0xfb840108,0x8fba0101,0xfb840108,0x8fbb0101,0xfb840108,0x8fbc0101,0xfb840108,0x8fbd0101,
	0xfb840108,0x8fbe0101,0xfb840108,0x8fbf0101,0xfb840108,0x8fc00101,0xfb840108,0x8fc10101,
	0xfb840108,0x8fc20101,0xfb840108,0x8fc30101,0xfb840108,0x8fc40101,0xfb840108,0x8fc50101,
	0xfb840108,0x8fc60101,0xfb840108,0x8fc70101,0xfb840108,0x8fc80101,0xfb840108,0x8fc90101,
	0xfb840108,0x8fca0101,0xfb840108,0x8fcb0101,0xfb840108,0x8fcc0101,0xfb840108,0x8fcd0101,
	0xfb840108,0x8fce0101,0xfb840108,0x8fcf0101,0xfb840108,0x8fd00101,0xfb840108,0x8fd10101,
	0xfb840108,0x8fd20101,0xfb840108,0x8fd30101,0xfb840108,0x8fd40101,0xfb840108,0x8fd50101,
	0xfb840108,0x8fd60101,0xfb840108,0x8fd70101,0xfb840108,0x8fd80101,0xfb840108,0x8fd90101,
	0xfb840108,0x8fda0101,0xfb840108,0x8fdb0101,0xfb840108,0x8fdc0101,0xfb840108,0x8fdd0101,
	0xfb840108,0x8fde0101,0xfb840108,0x8fdf0101,0xfb840108,0x8fe00101,0xfb840108,0x8fe10101,
	0xfb840108,0x8fe20101,0xfb840108,0x8fe30101,0xfb840108,0x8fe40101,0xfb840108,0x8fe50101,
	0xfb840108,0x8fe60101,0xfb840108,0x8fe70101,0xfb840108,0x8fe80101,0xfb840108,0x8fe90101,
	0xfb840108,0x8fea0101,0xfb840108,0x8feb0101,0xfb840108,0x8fec0101,0xfb840108,0x8fed0101,
	0xfb840108,0x8fee0101,0xfb840108,0x8fef0101,0xfb840108,0x8ff00101,0xfb840108,0x8ff10101,
	0xfb840108,0x8ff20101,0xfb840108,0x8ff30101,0xfb840108,0x8ff40101,0xfb840108,0x8ff50101,
	0xfb840108,0x8ff60101,0xfb840108,0x8ff70101,0xfb840108,0x8ff80101,0xfb840108,0x8ff90101,
	0xfb840108,0x8ffa0101,0xfb840108,0x8ffb0101,0xfb840108,0x8ffc0101,0xfb840108,0x8ffd0101,
	0xfb840108,0x8ffe0101,0xfb840108,0x8fff0101,0xfb840108,0x90000101,0xfb840108,0x90010101,
	0xfb840108,0x90020101,0xfb840108,0x90030101,0xfb840108,0x90040101,0xfb840108,0x90050101,
	0xfb840108,0x90060101,0xfb840108,0x90070101,0xfb840108,0x90080101,0xfb840108,0x90090101,
	0xfb840108,0x900a0101,0xfb840108,0x900b0101,0xfb840108,0x900c0101,0xfb840108,0x900d0101,
	0xfb840108,0x900e0101,0xfb840108,0x900f0101,0xfb840108,0x90100101,0xfb840108,0x90110101,
	0xfb840108,0x90120101,0xfb840108,0x90130101,0xfb840108,0x90140101,0xfb840108,0x90150101,
	0xfb840108,0x90160101,0xfb840108,0x90170101,0xfb840108,0x90180101,0xfb840108,0x90190101,
	0xfb840108,0x901a0101,0xfb840108,0x901b0101,0xfb840108,0x901c0101,0xfb840108,0x901d0101,
	0xfb840108,0x901e0101,0xfb840108,0x901f0101,0xfb840108,0x90200101,0xfb840108,0x90210101,
	0xfb840108,0x90220101,0xfb840108,0x90230101,0xfb840108,0x90240101,0xfb840108,0x90250101,
	0xfb840108,0x90260101,0xfb840108,0x90270101,0xfb840108,0x90280101,0xfb840108,0x90290101,
	0xfb840108,0x902a0101,0xfb840108,0x902b0101,0xfb840108,0x902c0101,0xfb840108,0x902d0101,
	0xfb840108,0x902e0101,0xfb840108,0x902f0101,0xfb840108,0x90300101,0xfb840108,0x90310101,
	0xfb840108,0x90320101,0xfb840108,0x90330101,0xfb840108,0x90340101,0xfb840108,0x90350101,
	0xfb840108,0x90360101,0xfb840108,0x90370101,0xfb840108,0x90380101,0xfb840108,0x90390101,
	0xfb840108,0x903a0101,0xfb840108,0x903b0101,0xfb840108,0x903c0101,0xfb840108,0x903d0101,
	0xfb840108,0x903e0101,0xfb840108,0x903f0101,0xfb840108,0x90400101,0xfb840108,0x90410101,
	0xfb840108,0x90420101,0xfb840108,0x90430101,0xfb840108,0x90440101,0xfb840108,0x90450101,
	0xfb840108,0x90460101,0xfb840108,0x90470101,0xfb840108,0x90480101,0xfb840108,0x90490101,
	0xfb840108,0x904a0101,0xfb840108,0x904b0101,0xfb840108,0x904c0101,0xfb840108,0x904d0101,
	0xfb840108,0x904e0101,0xfb840108,0x904f0101,0xfb840108,0x90500101,0xfb840108,0x90510101,
	0xfb840108,0x90520101,0xfb840108,0x90530101,0xfb840108,0x90540101,0xfb840108,0x90550101,
	0xfb840108,0x90560101,0xfb840108,0x90570101,0xfb840108,0x90580101,0xfb840108,0x90590101,
	0xfb840108,0x905a0101,0xfb840108,0x905b0101,0xfb840108,0x905c0101,0xfb840108,0x905d0101,
	0xfb840108,0x905e0101,0xfb840108,0x905f0101,0xfb840108,0x90600101,0xfb840108,0x90610101,
	0xfb840108,0x90620101,0xfb840108,0x90630101,0xfb840108,0x90640101,0xfb840108,0x90650101,
	0xfb840108,0x90660101,0xfb840108,0x90670101,0xfb840108,0x90680101,0xfb840108,0x90690101,
	0xfb840108,0x906a0101,0xfb840108,0x906b0101,0xfb840108,0x906c0101,0xfb840108,0x906d0101,
	0xfb840108,0x906e0101,0xfb840108,0x906f0101,0xfb840108,0x90700101,0xfb840108,0x90710101,
	0xfb840108,0x90720101,0xfb840108,0x90730101,0xfb840108,0x90740101,0xfb840108,0x90750101,
	0xfb840108,0x90760101,0xfb840108,0x90770101,0xfb840108,0x90780101,0xfb840108,0x90790101,
	0xfb840108,0x907a0101,0xfb840108,0x907b0101,0xfb840108,0x907c0101,0xfb840108,0x907d0101,
	0xfb840108,0x907e0101,0xfb840108,0x907f0101,0xfb840108,0x90800101,0xfb840108,0x90810101,
	0xfb840108,0x90820101,0xfb840108,0x90830101,0xfb840108,0x90840101,0xfb840108,0x90850101,
	0xfb840108,0x90860101,0xfb840108,0x90870101,0xfb840108,0x90880101,0xfb840108,0x90890101,
	0xfb840108,0x908a0101,0xfb840108,0x908b0101,0xfb840108,0x908c0101,0xfb840108,0x908d0101,
	0xfb840108,0x908e0101,0xfb840108,0x908f0101,0xfb840108,0x90900101,0xfb840108,0x90910101,
	0xfb840108,0x90920101,0xfb840108,0x90930101,0xfb840108,0x90940101,0xfb840108,0x90950101,
	0xfb840108,0x90960101,0xfb840108,0x90970101,0xfb840108,0x90980101,0xfb840108,0x90990101,
	0xfb840108,0x909a0101,0xfb840108,0x909b0101,0xfb840108,0x909c0101,0xfb840108,0x909d0101,
	0xfb840108,0x909e0101,0xfb840108,0x909f0101,0xfb840108,0x90a00101,0xfb840108,0x90a10101,
	0xfb840108,0x90a20101,0xfb840108,0x90a30101,0xfb840108,0x90a40101,0xfb840108,0x90a50101,
	0xfb840108,0x90a60101,0xfb840108,0x90a70101,0xfb840108,0x90a80101,0xfb840108,0x90a90101,
	0xfb840108,0x90aa0101,0xfb840108,0x90ab0101,0xfb840108,0x90ac0101,0xfb840108,0x90ad0101,
	0xfb840108,0x90ae0101,0xfb840108,0x90af0101,0xfb840108,0x90b00101,0xfb840108,0x90b10101,
	0xfb840108,0x90b20101,0xfb840108,0x90b30101,0xfb840108,0x90b40101,0xfb840108,0x90b50101,
	0xfb840108,0x90b60101,0xfb840108,0x90b70101,0xfb840108,0x90b80101,0xfb840108,0x90b90101,
	0xfb840108,0x90ba0101,0xfb840108,0x90bb0101,0xfb840108,0x90bc0101,0xfb840108,0x90bd0101,
	0xfb840108,0x90be0101,0xfb840108,0x90bf0101,0xfb840108,0x90c00101,0xfb840108,0x90c10101,
	0xfb840108,0x90c20101,0xfb840108,0x90c30101,0xfb840108,0x90c40101,0xfb840108,0x90c50101,
	0xfb840108,0x90c60101,0xfb840108,0x90c70101,0xfb840108,0x90c80101,0xfb840108,0x90c90101,
	0xfb840108,0x90ca0101,0xfb840108,0x90cb0101,0xfb840108,0x90cc0101,0xfb840108,0x90cd0101,
	0xfb840108,0x90ce0101,0xfb840108,0x90cf0101,0xfb840108,0x90d00101,0xfb840108,0x90d10101,
	0xfb840108,0x90d20101,0xfb840108,0x90d30101,0xfb840108,0x90d40101,0xfb840108,0x90d50101,
	0xfb840108,0x90d60101,0xfb840108,0x90d70101,0xfb840108,0x90d80101,0xfb840108,0x90d90101,
	0xfb840108,0x90da0101,0xfb840108,0x90db0101,0xfb840108,0x90dc0101,0xfb840108,0x90dd0101,
	0xfb840108,0x90de0101,0xfb840108,0x90df0101,0xfb840108,0x90e00101,0xfb840108,0x90e10101,
	0xfb840108,0x90e20101,0xfb840108,0x90e30101,0xfb840108,0x90e40101,0xfb840108,0x90e50101,
	0xfb840108,0x90e60101,0xfb840108,0x90e70101,0xfb840108,0x90e80101,0xfb840108,0x90e90101,
	0xfb840108,0x90ea0101,0xfb840108,0x90eb0101,0xfb840108,0x90ec0101,0xfb840108,0x90ed0101,
	0xfb840108,0x90ee0101,0xfb840108,0x90ef0101,0xfb840108,0x90f00101,0xfb840108,0x90f10101,
	0xfb840108,0x90f20101,0xfb840108,0x90f30101,0xfb840108,0x90f40101,0xfb840108,0x90f50101,
	0xfb840108,0x90f60101,0xfb840108,0x90f70101,0xfb840108,0x90f80101,0xfb840108,0x90f90101,
	0xfb840108,0x90fa0101,0xfb840108,0x90fb0101,0xfb840108,0x90fc0101,0xfb840108,0x90fd0101,
	0xfb840108,0x90fe0101,0xfb840108,0x90ff0101,0xfb840108,0x91000101,0xfb840108,0x91010101,
	0xfb840108,0x91020101,0xfb840108,0x91030101,0xfb840108,0x91040101,0xfb840108,0x91050101,
	0xfb840108,0x91060101,0xfb840108,0x91070101,0xfb840108,0x91080101,0xfb840108,0x91090101,
	0xfb840108,0x910a0101,0xfb840108,0x910b0101,0xfb840108,0x910c0101,0xfb840108,0x910d0101,
	0xfb840108,0x910e0101,0xfb840108,0x910f0101,0xfb840108,0x91100101,0xfb840108,0x91110101,
	0xfb840108,0x91120101,0xfb840108,0x91130101,0xfb840108,0x91140101,0xfb840108,0x91150101,
	0xfb840108,0x91160101,0xfb840108,0x91170101,0xfb840108,0x91180101,0xfb840108,0x91190101,
	0xfb840108,0x911a0101,0xfb840108,0x911b0101,0xfb840108,0x911c0101,0xfb840108,0x911d0101,
	0xfb840108,0x911e0101,0xfb840108,0x911f0101,0xfb840108,0x91200101,0xfb840108,0x91210101,
	0xfb840108,0x91220101,0xfb840108,0x91230101,0xfb840108,0x91240101,0xfb840108,0x91250101,
	0xfb840108,0x91260101,0xfb840108,0x91270101,0xfb840108,0x91280101,0xfb840108,0x91290101,
	0xfb840108,0x912a0101,0xfb840108,0x912b0101,0xfb840108,0x912c0101,0xfb840108,0x912d0101,
	0xfb840108,0x912e0101,0xfb840108,0x912f0101,0xfb840108,0x91300101,0xfb840108,0x91310101,
	0xfb840108,0x91320101,0xfb840108,0x91330101,0xfb840108,0x91340101,0xfb840108,0x91350101,
	0xfb840108,0x91360101,0xfb840108,0x91370101,0xfb840108,0x91380101,0xfb840108,0x91390101,
	0xfb840108,0x913a0101,0xfb840108,0x913b0101,0xfb840108,0x913c0101,0xfb840108,0x913d0101,
	0xfb840108,0x913e0101,0xfb840108,0x913f0101,0xfb840108,0x91400101,0xfb840108,0x91410101,
	0xfb840108,0x91420101,0xfb840108,0x91430101,0xfb840108,0x91440101,0xfb840108,0x91450101,
	0xfb840108,0x91460101,0xfb840108,0x91470101,0xfb840108,0x91480101,0xfb840108,0x91490101,
	0xfb840108,0x914a0101,0xfb840108,0x914b0101,0xfb840108,0x914c0101,0xfb840108,0x914d0101,
	0xfb840108,0x914e0101,0xfb840108,0x914f0101,0xfb840108,0x91500101,0xfb840108,0x91510101,
	0xfb840108,0x91520101,0xfb840108,0x91530101,0xfb840108,0x91540101,0xfb840108,0x91550101,
	0xfb840108,0x91560101,0xfb840108,0x91570101,0xfb840108,0x91580101,0xfb840108,0x91590101,
	0xfb840108,0x915a0101,0xfb840108,0x915b0101,0xfb840108,0x915c0101,0xfb840108,0x915d0101,
	0xfb840108,0x915e0101,0xfb840108,0x915f0101,0xfb840108,0x91600101,0xfb840108,0x91610101,
	0xfb840108,0x91620101,0xfb840108,0x91630101,0xfb840108,0x91640101,0xfb840108,0x91650101,
	0xfb840108,0x91660101,0xfb840108,0x91670101,0xfb840108,0x91680101,0xfb840108,0x91690101,
	0xfb840108,0x916a0101,0xfb840108,0x916b0101,0xfb840108,0x916c0101,0xfb840108,0x916d0101,
	0xfb840108,0x916e0101,0xfb840108,0x916f0101,0xfb840108,0x91700101,0xfb840108,0x91710101,
	0xfb840108,0x91720101,0xfb840108,0x91730101,0xfb840108,0x91740101,0xfb840108,0x91750101,
	0xfb840108,0x91760101,0xfb840108,0x91770101,0xfb840108,0x91780101,0xfb840108,0x91790101,
	0xfb840108,0x917a0101,0xfb840108,0x917b0101,0xfb840108,0x917c0101,0xfb840108,0x917d0101,
	0xfb840108,0x917e0101,0xfb840108,0x917f0101,0xfb840108,0x91800101,0xfb840108,0x91810101,
	0xfb840108,0x91820101,0xfb840108,0x91830101,0xfb840108,0x91840101,0xfb840108,0x91850101,
	0xfb840108,0x91860101,0xfb840108,0x91870101,0xfb840108,0x91880101,0xfb840108,0x91890101,
	0xfb840108,0x918a0101,0xfb840108,0x918b0101,0xfb840108,0x918c0101,0xfb840108,0x918d0101,
	0xfb840108,0x918e0101,0xfb840108,0x918f0101,0xfb840108,0x91900101,0xfb840108,0x91910101,
	0xfb840108,0x91920101,0xfb840108,0x91930101,0xfb840108,0x91940101,0xfb840108,0x91950101,
	0xfb840108,0x91960101,0xfb840108,0x91970101,0xfb840108,0x91980101,0xfb840108,0x91990101,
	0xfb840108,0x919a0101,0xfb840108,0x919b0101,0xfb840108,0x919c0101,0xfb840108,0x919d0101,
	0xfb840108,0x919e0101,0xfb840108,0x919f0101,0xfb840108,0x91a00101,0xfb840108,0x91a10101,
	0xfb840108,0x91a20101,0xfb840108,0x91a30101,0xfb840108,0x91a40101,0xfb840108,0x91a50101,
	0xfb840108,0x91a60101,0xfb840108,0x91a70101,0xfb840108,0x91a80101,0xfb840108,0x91a90101,
	0xfb840108,0x91aa0101,0xfb840108,0x91ab0101,0xfb840108,0x91ac0101,0xfb840108,0x91ad0101,
	0xfb840108,0x91ae0101,0xfb840108,0x91af0101,0xfb840108,0x91b00101,0xfb840108,0x91b10101,
	0xfb840108,0x91b20101,0xfb840108,0x91b30101,0xfb840108,0x91b40101,0xfb840108,0x91b50101,
	0xfb840108,0x91b60101,0xfb840108,0x91b70101,0xfb840108,0x91b80101,0xfb840108,0x91b90101,
	0xfb840108,0x91ba0101,0xfb840108,0x91bb0101,0xfb840108,0x91bc0101,0xfb840108,0x91bd0101,
	0xfb840108,0x91be0101,0xfb840108,0x91bf0101,0xfb840108,0x91c00101,0xfb840108,0x91c10101,
	0xfb840108,0x91c20101,0xfb840108,0x91c30101,0xfb840108,0x91c40101,0xfb840108,0x91c50101,
	0xfb840108,0x91c60101,0xfb840108,0x91c70101,0xfb840108,0x91c80101,0xfb840108,0x91c90101,
	0xfb840108,0x91ca0101,0xfb840108,0x91cb0101,0xfb840108,0x91cc0101,0xfb840108,0x91cd0101,
	0xfb840108,0x91ce0101,0xfb840108,0x91cf0101,0xfb840108,0x91d00101,0xfb840108,0x91d10101,
	0xfb840108,0x91d20101,0xfb840108,0x91d30101,0xfb840108,0x91d40101,0xfb840108,0x91d50101,
	0xfb840108,0x91d60101,0xfb840108,0x91d70101,0xfb840108,0x91d80101,0xfb840108,0x91d90101,
	0xfb840108,0x91da0101,0xfb840108,0x91db0101,0xfb840108,0x91dc0101,0xfb840108,0x91dd0101,
	0xfb840108,0x91de0101,0xfb840108,0x91df0101,0xfb840108,0x91e00101,0xfb840108,0x91e10101,
	0xfb840108,0x91e20101,0xfb840108,0x91e30101,0xfb840108,0x91e40101,0xfb840108,0x91e50101,
	0xfb840108,0x91e60101,0xfb840108,0x91e70101,0xfb840108,0x91e80101,0xfb840108,0x91e90101,
	0xfb840108,0x91ea0101,0xfb840108,0x91eb0101,0xfb840108,0x91ec0101,0xfb840108,0x91ed0101,
	0xfb840108,0x91ee0101,0xfb840108,0x91ef0101,0xfb840108,0x91f00101,0xfb840108,0x91f10101,
	0xfb840108,0x91f20101,0xfb840108,0x91f30101,0xfb840108,0x91f40101,0xfb840108,0x91f50101,
	0xfb840108,0x91f60101,0xfb840108,0x91f70101,0xfb840108,0x91f80101,0xfb840108,0x91f90101,
	0xfb840108,0x91fa0101,0xfb840108,0x91fb0101,0xfb840108,0x91fc0101,0xfb840108,0x91fd0101,
	0xfb840108,0x91fe0101,0xfb840108,0x91ff0101,0xfb840108,0x92000101,0xfb840108,0x92010101,
	0xfb840108,0x92020101,0xfb840108,0x92030101,0xfb840108,0x92040101,0xfb840108,0x92050101,
	0xfb840108,0x92060101,0xfb840108,0x92070101,0xfb840108,0x92080101,0xfb840108,0x92090101,
	0xfb840108,0x920a0101,0xfb840108,0x920b0101,0xfb840108,0x920c0101,0xfb840108,0x920d0101,
	0xfb840108,0x920e0101,0xfb840108,0x920f0101,0xfb840108,0x92100101,0xfb840108,0x92110101,
	0xfb840108,0x92120101,0xfb840108,0x92130101,0xfb840108,0x92140101,0xfb840108,0x92150101,
	0xfb840108,0x92160101,0xfb840108,0x92170101,0xfb840108,0x92180101,0xfb840108,0x92190101,
	0xfb840108,0x921a0101,0xfb840108,0x921b0101,0xfb840108,0x921c0101,0xfb840108,0x921d0101,
	0xfb840108,0x921e0101,0xfb840108,0x921f0101,0xfb840108,0x92200101,0xfb840108,0x92210101,
	0xfb840108,0x92220101,0xfb840108,0x92230101,0xfb840108,0x92240101,0xfb840108,0x92250101,
	0xfb840108,0x92260101,0xfb840108,0x92270101,0xfb840108,0x92280101,0xfb840108,0x92290101,
	0xfb840108,0x922a0101,0xfb840108,0x922b0101,0xfb840108,0x922c0101,0xfb840108,0x922d0101,
	0xfb840108,0x922e0101,0xfb840108,0x922f0101,0xfb840108,0x92300101,0xfb840108,0x92310101,
	0xfb840108,0x92320101,0xfb840108,0x92330101,0xfb840108,0x92340101,0xfb840108,0x92350101,
	0xfb840108,0x92360101,0xfb840108,0x92370101,0xfb840108,0x92380101,0xfb840108,0x92390101,
	0xfb840108,0x923a0101,0xfb840108,0x923b0101,0xfb840108,0x923c0101,0xfb840108,0x923d0101,
	0xfb840108,0x923e0101,0xfb840108,0x923f0101,0xfb840108,0x92400101,0xfb840108,0x92410101,
	0xfb840108,0x92420101,0xfb840108,0x92430101,0xfb840108,0x92440101,0xfb840108,0x92450101,
	0xfb840108,0x92460101,0xfb840108,0x92470101,0xfb840108,0x92480101,0xfb840108,0x92490101,
	0xfb840108,0x924a0101,0xfb840108,0x924b0101,0xfb840108,0x924c0101,0xfb840108,0x924d0101,
	0xfb840108,0x924e0101,0xfb840108,0x924f0101,0xfb840108,0x92500101,0xfb840108,0x92510101,
	0xfb840108,0x92520101,0xfb840108,0x92530101,0xfb840108,0x92540101,0xfb840108,0x92550101,
	0xfb840108,0x92560101,0xfb840108,0x92570101,0xfb840108,0x92580101,0xfb840108,0x92590101,
	0xfb840108,0x925a0101,0xfb840108,0x925b0101,0xfb840108,0x925c0101,0xfb840108,0x925d0101,
	0xfb840108,0x925e0101,0xfb840108,0x925f0101,0xfb840108,0x92600101,0xfb840108,0x92610101,
	0xfb840108,0x92620101,0xfb840108,0x92630101,0xfb840108,0x92640101,0xfb840108,0x92650101,
	0xfb840108,0x92660101,0xfb840108,0x92670101,0xfb840108,0x92680101,0xfb840108,0x92690101,
	0xfb840108,0x926a0101,0xfb840108,0x926b0101,0xfb840108,0x926c0101,0xfb840108,0x926d0101,
	0xfb840108,0x926e0101,0xfb840108,0x926f0101,0xfb840108,0x92700101,0xfb840108,0x92710101,
	0xfb840108,0x92720101,0xfb840108,0x92730101,0xfb840108,0x92740101,0xfb840108,0x92750101,
	0xfb840108,0x92760101,0xfb840108,0x92770101,0xfb840108,0x92780101,0xfb840108,0x92790101,
	0xfb840108,0x927a0101,0xfb840108,0x927b0101,0xfb840108,0x927c0101,0xfb840108,0x927d0101,
	0xfb840108,0x927e0101,0xfb840108,0x927f0101,0xfb840108,0x92800101,0xfb840108,0x92810101,
	0xfb840108,0x92820101,0xfb840108,0x92830101,0xfb840108,0x92840101,0xfb840108,0x92850101,
	0xfb840108,0x92860101,0xfb840108,0x92870101,0xfb840108,0x92880101,0xfb840108,0x92890101,
	0xfb840108,0x928a0101,0xfb840108,0x928b0101,0xfb840108,0x928c0101,0xfb840108,0x928d0101,
	0xfb840108,0x928e0101,0xfb840108,0x928f0101,0xfb840108,0x92900101,0xfb840108,0x92910101,
	0xfb840108,0x92920101,0xfb840108,0x92930101,0xfb840108,0x92940101,0xfb840108,0x92950101,
	0xfb840108,0x92960101,0xfb840108,0x92970101,0xfb840108,0x92980101,0xfb840108,0x92990101,
	0xfb840108,0x929a0101,0xfb840108,0x929b0101,0xfb840108,0x929c0101,0xfb840108,0x929d0101,
	0xfb840108,0x929e0101,0xfb840108,0x929f0101,0xfb840108,0x92a00101,0xfb840108,0x92a10101,
	0xfb840108,0x92a20101,0xfb840108,0x92a30101,0xfb840108,0x92a40101,0xfb840108,0x92a50101,
	0xfb840108,0x92a60101,0xfb840108,0x92a70101,0xfb840108,0x92a80101,0xfb840108,0x92a90101,
	0xfb840108,0x92aa0101,0xfb840108,0x92ab0101,0xfb840108,0x92ac0101,0xfb840108,0x92ad0101,
	0xfb840108,0x92ae0101,0xfb840108,0x92af0101,0xfb840108,0x92b00101,0xfb840108,0x92b10101,
	0xfb840108,0x92b20101,0xfb840108,0x92b30101,0xfb840108,0x92b40101,0xfb840108,0x92b50101,
	0xfb840108,0x92b60101,0xfb840108,0x92b70101,0xfb840108,0x92b80101,0xfb840108,0x92b90101,
	0xfb840108,0x92ba0101,0xfb840108,0x92bb0101,0xfb840108,0x92bc0101,0xfb840108,0x92bd0101,
	0xfb840108,0x92be0101,0xfb840108,0x92bf0101,0xfb840108,0x92c00101,0xfb840108,0x92c10101,
	0xfb840108,0x92c20101,0xfb840108,0x92c30101,0xfb840108,0x92c40101,0xfb840108,0x92c50101,
	0xfb840108,0x92c60101,0xfb840108,0x92c70101,0xfb840108,0x92c80101,0xfb840108,0x92c90101,
	0xfb840108,0x92ca0101,0xfb840108,0x92cb0101,0xfb840108,0x92cc0101,0xfb840108,0x92cd0101,
	0xfb840108,0x92ce0101,0xfb840108,0x92cf0101,0xfb840108,0x92d00101,0xfb840108,0x92d10101,
	0xfb840108,0x92d20101,0xfb840108,0x92d30101,0xfb840108,0x92d40101,0xfb840108,0x92d50101,
	0xfb840108,0x92d60101,0xfb840108,0x92d70101,0xfb840108,0x92d80101,0xfb840108,0x92d90101,
	0xfb840108,0x92da0101,0xfb840108,0x92db0101,0xfb840108,0x92dc0101,0xfb840108,0x92dd0101,
	0xfb840108,0x92de0101,0xfb840108,0x92df0101,0xfb840108,0x92e00101,0xfb840108,0x92e10101,
	0xfb840108,0x92e20101,0xfb840108,0x92e30101,0xfb840108,0x92e40101,0xfb840108,0x92e50101,
	0xfb840108,0x92e60101,0xfb840108,0x92e70101,0xfb840108,0x92e80101,0xfb840108,0x92e90101,
	0xfb840108,0x92ea0101,0xfb840108,0x92eb0101,0xfb840108,0x92ec0101,0xfb840108,0x92ed0101,
	0xfb840108,0x92ee0101,0xfb840108,0x92ef0101,0xfb840108,0x92f00101,0xfb840108,0x92f10101,
	0xfb840108,0x92f20101,0xfb840108,0x92f30101,0xfb840108,0x92f40101,0xfb840108,0x92f50101,
	0xfb840108,0x92f60101,0xfb840108,0x92f70101,0xfb840108,0x92f80101,0xfb840108,0x92f90101,
	0xfb840108,0x92fa0101,0xfb840108,0x92fb0101,0xfb840108,0x92fc0101,0xfb840108,0x92fd0101,
	0xfb840108,0x92fe0101,0xfb840108,0x92ff0101,0xfb840108,0x93000101,0xfb840108,0x93010101,
	0xfb840108,0x93020101,0xfb840108,0x93030101,0xfb840108,0x93040101,0xfb840108,0x93050101,
	0xfb840108,0x93060101,0xfb840108,0x93070101,0xfb840108,0x93080101,0xfb840108,0x93090101,
	0xfb840108,0x930a0101,0xfb840108,0x930b0101,0xfb840108,0x930c0101,0xfb840108,0x930d0101,
	0xfb840108,0x930e0101,0xfb840108,0x930f0101,0xfb840108,0x93100101,0xfb840108,0x93110101,
	0xfb840108,0x93120101,0xfb840108,0x93130101,0xfb840108,0x93140101,0xfb840108,0x93150101,
	0xfb840108,0x93160101,0xfb840108,0x93170101,0xfb840108,0x93180101,0xfb840108,0x93190101,
	0xfb840108,0x931a0101,0xfb840108,0x931b0101,0xfb840108,0x931c0101,0xfb840108,0x931d0101,
	0xfb840108,0x931e0101,0xfb840108,0x931f0101,0xfb840108,0x93200101,0xfb840108,0x93210101,
	0xfb840108,0x93220101,0xfb840108,0x93230101,0xfb840108,0x93240101,0xfb840108,0x93250101,
	0xfb840108,0x93260101,0xfb840108,0x93270101,0xfb840108,0x93280101,0xfb840108,0x93290101,
	0xfb840108,0x932a0101,0xfb840108,0x932b0101,0xfb840108,0x932c0101,0xfb840108,0x932d0101,
	0xfb840108,0x932e0101,0xfb840108,0x932f0101,0xfb840108,0x93300101,0xfb840108,0x93310101,
	0xfb840108,0x93320101,0xfb840108,0x93330101,0xfb840108,0x93340101,0xfb840108,0x93350101,
	0xfb840108,0x93360101,0xfb840108,0x93370101,0xfb840108,0x93380101,0xfb840108,0x93390101,
	0xfb840108,0x933a0101,0xfb840108,0x933b0101,0xfb840108,0x933c0101,0xfb840108,0x933d0101,
	0xfb840108,0x933e0101,0xfb840108,0x933f0101,0xfb840108,0x93400101,0xfb840108,0x93410101,
	0xfb840108,0x93420101,0xfb840108,0x93430101,0xfb840108,0x93440101,0xfb840108,0x93450101,
	0xfb840108,0x93460101,0xfb840108,0x93470101,0xfb840108,0x93480101,0xfb840108,0x93490101,
	0xfb840108,0x934a0101,0xfb840108,0x934b0101,0xfb840108,0x934c0101,0xfb840108,0x934d0101,
	0xfb840108,0x934e0101,0xfb840108,0x934f0101,0xfb840108,0x93500101,0xfb840108,0x93510101,
	0xfb840108,0x93520101,0xfb840108,0x93530101,0xfb840108,0x93540101,0xfb840108,0x93550101,
	0xfb840108,0x93560101,0xfb840108,0x93570101,0xfb840108,0x93580101,0xfb840108,0x93590101,
	0xfb840108,0x935a0101,0xfb840108,0x935b0101,0xfb840108,0x935c0101,0xfb840108,0x935d0101,
	0xfb840108,0x935e0101,0xfb840108,0x935f0101,0xfb840108,0x93600101,0xfb840108,0x93610101,
	0xfb840108,0x93620101,0xfb840108,0x93630101,0xfb840108,0x93640101,0xfb840108,0x93650101,
	0xfb840108,0x93660101,0xfb840108,0x93670101,0xfb840108,0x93680101,0xfb840108,0x93690101,
	0xfb840108,0x936a0101,0xfb840108,0x936b0101,0xfb840108,0x936c0101,0xfb840108,0x936d0101,
	0xfb840108,0x936e0101,0xfb840108,0x936f0101,0xfb840108,0x93700101,0xfb840108,0x93710101,
	0xfb840108,0x93720101,0xfb840108,0x93730101,0xfb840108,0x93740101,0xfb840108,0x93750101,
	0xfb840108,0x93760101,0xfb840108,0x93770101,0xfb840108,0x93780101,0xfb840108,0x93790101,
	0xfb840108,0x937a0101,0xfb840108,0x937b0101,0xfb840108,0x937c0101,0xfb840108,0x937d0101,
	0xfb840108,0x937e0101,0xfb840108,0x937f0101,0xfb840108,0x93800101,0xfb840108,0x93810101,
	0xfb840108,0x93820101,0xfb840108,0x93830101,0xfb840108,0x93840101,0xfb840108,0x93850101,
	0xfb840108,0x93860101,0xfb840108,0x93870101,0xfb840108,0x93880101,0xfb840108,0x93890101,
	0xfb840108,0x938a0101,0xfb840108,0x938b0101,0xfb840108,0x938c0101,0xfb840108,0x938d0101,
	0xfb840108,0x938e0101,0xfb840108,0x938f0101,0xfb840108,0x93900101,0xfb840108,0x93910101,
	0xfb840108,0x93920101,0xfb840108,0x93930101,0xfb840108,0x93940101,0xfb840108,0x93950101,
	0xfb840108,0x93960101,0xfb840108,0x93970101,0xfb840108,0x93980101,0xfb840108,0x93990101,
	0xfb840108,0x939a0101,0xfb840108,0x939b0101,0xfb840108,0x939c0101,0xfb840108,0x939d0101,
	0xfb840108,0x939e0101,0xfb840108,0x939f0101,0xfb840108,0x93a00101,0xfb840108,0x93a10101,
	0xfb840108,0x93a20101,0xfb840108,0x93a30101,0xfb840108,0x93a40101,0xfb840108,0x93a50101,
	0xfb840108,0x93a60101,0xfb840108,0x93a70101,0xfb840108,0x93a80101,0xfb840108,0x93a90101,
	0xfb840108,0x93aa0101,0xfb840108,0x93ab0101,0xfb840108,0x93ac0101,0xfb840108,0x93ad0101,
	0xfb840108,0x93ae0101,0xfb840108,0x93af0101,0xfb840108,0x93b00101,0xfb840108,0x93b10101,
	0xfb840108,0x93b20101,0xfb840108,0x93b30101,0xfb840108,0x93b40101,0xfb840108,0x93b50101,
	0xfb840108,0x93b60101,0xfb840108,0x93b70101,0xfb840108,0x93b80101,0xfb840108,0x93b90101,
	0xfb840108,0x93ba0101,0xfb840108,0x93bb0101,0xfb840108,0x93bc0101,0xfb840108,0x93bd0101,
	0xfb840108,0x93be0101,0xfb840108,0x93bf0101,0xfb840108,0x93c00101,0xfb840108,0x93c10101,
	0xfb840108,0x93c20101,0xfb840108,0x93c30101,0xfb840108,0x93c40101,0xfb840108,0x93c50101,
	0xfb840108,0x93c60101,0xfb840108,0x93c70101,0xfb840108,0x93c80101,0xfb840108,0x93c90101,
	0xfb840108,0x93ca0101,0xfb840108,0x93cb0101,0xfb840108,0x93cc0101,0xfb840108,0x93cd0101,
	0xfb840108,0x93ce0101,0xfb840108,0x93cf0101,0xfb840108,0x93d00101,0xfb840108,0x93d10101,
	0xfb840108,0x93d20101,0xfb840108,0x93d30101,0xfb840108,0x93d40101,0xfb840108,0x93d50101,
	0xfb840108,0x93d60101,0xfb840108,0x93d70101,0xfb840108,0x93d80101,0xfb840108,0x93d90101,
	0xfb840108,0x93da0101,0xfb840108,0x93db0101,0xfb840108,0x93dc0101,0xfb840108,0x93dd0101,
	0xfb840108,0x93de0101,0xfb840108,0x93df0101,0xfb840108,0x93e00101,0xfb840108,0x93e10101,
	0xfb840108,0x93e20101,0xfb840108,0x93e30101,0xfb840108,0x93e40101,0xfb840108,0x93e50101,
	0xfb840108,0x93e60101,0xfb840108,0x93e70101,0xfb840108,0x93e80101,0xfb840108,0x93e90101,
	0xfb840108,0x93ea0101,0xfb840108,0x93eb0101,0xfb840108,0x93ec0101,0xfb840108,0x93ed0101,
	0xfb840108,0x93ee0101,0xfb840108,0x93ef0101,0xfb840108,0x93f00101,0xfb840108,0x93f10101,
	0xfb840108,0x93f20101,0xfb840108,0x93f30101,0xfb840108,0x93f40101,0xfb840108,0x93f50101,
	0xfb840108,0x93f60101,0xfb840108,0x93f70101,0xfb840108,0x93f80101,0xfb840108,0x93f90101,
	0xfb840108,0x93fa0101,0xfb840108,0x93fb0101,0xfb840108,0x93fc0101,0xfb840108,0x93fd0101,
	0xfb840108,0x93fe0101,0xfb840108,0x93ff0101,0xfb840108,0x94000101,0xfb840108,0x94010101,
	0xfb840108,0x94020101,0xfb840108,0x94030101,0xfb840108,0x94040101,0xfb840108,0x94050101,
	0xfb840108,0x94060101,0xfb840108,0x94070101,0xfb840108,0x94080101,0xfb840108,0x94090101,
	0xfb840108,0x940a0101,0xfb840108,0x940b0101,0xfb840108,0x940c0101,0xfb840108,0x940d0101,
	0xfb840108,0x940e0101,0xfb840108,0x940f0101,0xfb840108,0x94100101,0xfb840108,0x94110101,
	0xfb840108,0x94120101,0xfb840108,0x94130101,0xfb840108,0x94140101,0xfb840108,0x94150101,
	0xfb840108,0x94160101,0xfb840108,0x94170101,0xfb840108,0x94180101,0xfb840108,0x94190101,
	0xfb840108,0x941a0101,0xfb840108,0x941b0101,0xfb840108,0x941c0101,0xfb840108,0x941d0101,
	0xfb840108,0x941e0101,0xfb840108,0x941f0101,0xfb840108,0x94200101,0xfb840108,0x94210101,
	0xfb840108,0x94220101,0xfb840108,0x94230101,0xfb840108,0x94240101,0xfb840108,0x94250101,
	0xfb840108,0x94260101,0xfb840108,0x94270101,0xfb840108,0x94280101,0xfb840108,0x94290101,
	0xfb840108,0x942a0101,0xfb840108,0x942b0101,0xfb840108,0x942c0101,0xfb840108,0x942d0101,
	0xfb840108,0x942e0101,0xfb840108,0x942f0101,0xfb840108,0x94300101,0xfb840108,0x94310101,
	0xfb840108,0x94320101,0xfb840108,0x94330101,0xfb840108,0x94340101,0xfb840108,0x94350101,
	0xfb840108,0x94360101,0xfb840108,0x94370101,0xfb840108,0x94380101,0xfb840108,0x94390101,
	0xfb840108,0x943a0101,0xfb840108,0x943b0101,0xfb840108,0x943c0101,0xfb840108,0x943d0101,
	0xfb840108,0x943e0101,0xfb840108,0x943f0101,0xfb840108,0x94400101,0xfb840108,0x94410101,
	0xfb840108,0x94420101,0xfb840108,0x94430101,0xfb840108,0x94440101,0xfb840108,0x94450101,
	0xfb840108,0x94460101,0xfb840108,0x94470101,0xfb840108,0x94480101,0xfb840108,0x94490101,
	0xfb840108,0x944a0101,0xfb840108,0x944b0101,0xfb840108,0x944c0101,0xfb840108,0x944d0101,
	0xfb840108,0x944e0101,0xfb840108,0x944f0101,0xfb840108,0x94500101,0xfb840108,0x94510101,
	0xfb840108,0x94520101,0xfb840108,0x94530101,0xfb840108,0x94540101,0xfb840108,0x94550101,
	0xfb840108,0x94560101,0xfb840108,0x94570101,0xfb840108,0x94580101,0xfb840108,0x94590101,
	0xfb840108,0x945a0101,0xfb840108,0x945b0101,0xfb840108,0x945c0101,0xfb840108,0x945d0101,
	0xfb840108,0x945e0101,0xfb840108,0x945f0101,0xfb840108,0x94600101,0xfb840108,0x94610101,
	0xfb840108,0x94620101,0xfb840108,0x94630101,0xfb840108,0x94640101,0xfb840108,0x94650101,
	0xfb840108,0x94660101,0xfb840108,0x94670101,0xfb840108,0x94680101,0xfb840108,0x94690101,
	0xfb840108,0x946a0101,0xfb840108,0x946b0101,0xfb840108,0x946c0101,0xfb840108,0x946d0101,
	0xfb840108,0x946e0101,0xfb840108,0x946f0101,0xfb840108,0x94700101,0xfb840108,0x94710101,
	0xfb840108,0x94720101,0xfb840108,0x94730101,0xfb840108,0x94740101,0xfb840108,0x94750101,
	0xfb840108,0x94760101,0xfb840108,0x94770101,0xfb840108,0x94780101,0xfb840108,0x94790101,
	0xfb840108,0x947a0101,0xfb840108,0x947b0101,0xfb840108,0x947c0101,0xfb840108,0x947d0101,
	0xfb840108,0x947e0101,0xfb840108,0x947f0101,0xfb840108,0x94800101,0xfb840108,0x94810101,
	0xfb840108,0x94820101,0xfb840108,0x94830101,0xfb840108,0x94840101,0xfb840108,0x94850101,
	0xfb840108,0x94860101,0xfb840108,0x94870101,0xfb840108,0x94880101,0xfb840108,0x94890101,
	0xfb840108,0x948a0101,0xfb840108,0x948b0101,0xfb840108,0x948c0101,0xfb840108,0x948d0101,
	0xfb840108,0x948e0101,0xfb840108,0x948f0101,0xfb840108,0x94900101,0xfb840108,0x94910101,
	0xfb840108,0x94920101,0xfb840108,0x94930101,0xfb840108,0x94940101,0xfb840108,0x94950101,
	0xfb840108,0x94960101,0xfb840108,0x94970101,0xfb840108,0x94980101,0xfb840108,0x94990101,
	0xfb840108,0x949a0101,0xfb840108,0x949b0101,0xfb840108,0x949c0101,0xfb840108,0x949d0101,
	0xfb840108,0x949e0101,0xfb840108,0x949f0101,0xfb840108,0x94a00101,0xfb840108,0x94a10101,
	0xfb840108,0x94a20101,0xfb840108,0x94a30101,0xfb840108,0x94a40101,0xfb840108,0x94a50101,
	0xfb840108,0x94a60101,0xfb840108,0x94a70101,0xfb840108,0x94a80101,0xfb840108,0x94a90101,
	0xfb840108,0x94aa0101,0xfb840108,0x94ab0101,0xfb840108,0x94ac0101,0xfb840108,0x94ad0101,
	0xfb840108,0x94ae0101,0xfb840108,0x94af0101,0xfb840108,0x94b00101,0xfb840108,0x94b10101,
	0xfb840108,0x94b20101,0xfb840108,0x94b30101,0xfb840108,0x94b40101,0xfb840108,0x94b50101,
	0xfb840108,0x94b60101,0xfb840108,0x94b70101,0xfb840108,0x94b80101,0xfb840108,0x94b90101,
	0xfb840108,0x94ba0101,0xfb840108,0x94bb0101,0xfb840108,0x94bc0101,0xfb840108,0x94bd0101,
	0xfb840108,0x94be0101,0xfb840108,0x94bf0101,0xfb840108,0x94c00101,0xfb840108,0x94c10101,
	0xfb840108,0x94c20101,0xfb840108,0x94c30101,0xfb840108,0x94c40101,0xfb840108,0x94c50101,
	0xfb840108,0x94c60101,0xfb840108,0x94c70101,0xfb840108,0x94c80101,0xfb840108,0x94c90101,
	0xfb840108,0x94ca0101,0xfb840108,0x94cb0101,0xfb840108,0x94cc0101,0xfb840108,0x94cd0101,
	0xfb840108,0x94ce0101,0xfb840108,0x94cf0101,0xfb840108,0x94d00101,0xfb840108,0x94d10101,
	0xfb840108,0x94d20101,0xfb840108,0x94d30101,0xfb840108,0x94d40101,0xfb840108,0x94d50101,
	0xfb840108,0x94d60101,0xfb840108,0x94d70101,0xfb840108,0x94d80101,0xfb840108,0x94d90101,
	0xfb840108,0x94da0101,0xfb840108,0x94db0101,0xfb840108,0x94dc0101,0xfb840108,0x94dd0101,
	0xfb840108,0x94de0101,0xfb840108,0x94df0101,0xfb840108,0x94e00101,0xfb840108,0x94e10101,
	0xfb840108,0x94e20101,0xfb840108,0x94e30101,0xfb840108,0x94e40101,0xfb840108,0x94e50101,
	0xfb840108,0x94e60101,0xfb840108,0x94e70101,0xfb840108,0x94e80101,0xfb840108,0x94e90101,
	0xfb840108,0x94ea0101,0xfb840108,0x94eb0101,0xfb840108,0x94ec0101,0xfb840108,0x94ed0101,
	0xfb840108,0x94ee0101,0xfb840108,0x94ef0101,0xfb840108,0x94f00101,0xfb840108,0x94f10101,
	0xfb840108,0x94f20101,0xfb840108,0x94f30101,0xfb840108,0x94f40101,0xfb840108,0x94f50101,
	0xfb840108,0x94f60101,0xfb840108,0x94f70101,0xfb840108,0x94f80101,0xfb840108,0x94f90101,
	0xfb840108,0x94fa0101,0xfb840108,0x94fb0101,0xfb840108,0x94fc0101,0xfb840108,0x94fd0101,
	0xfb840108,0x94fe0101,0xfb840108,0x94ff0101,0xfb840108,0x95000101,0xfb840108,0x95010101,
	0xfb840108,0x95020101,0xfb840108,0x95030101,0xfb840108,0x95040101,0xfb840108,0x95050101,
	0xfb840108,0x95060101,0xfb840108,0x95070101,0xfb840108,0x95080101,0xfb840108,0x95090101,
	0xfb840108,0x950a0101,0xfb840108,0x950b0101,0xfb840108,0x950c0101,0xfb840108,0x950d0101,
	0xfb840108,0x950e0101,0xfb840108,0x950f0101,0xfb840108,0x95100101,0xfb840108,0x95110101,
	0xfb840108,0x95120101,0xfb840108,0x95130101,0xfb840108,0x95140101,0xfb840108,0x95150101,
	0xfb840108,0x95160101,0xfb840108,0x95170101,0xfb840108,0x95180101,0xfb840108,0x95190101,
	0xfb840108,0x951a0101,0xfb840108,0x951b0101,0xfb840108,0x951c0101,0xfb840108,0x951d0101,
	0xfb840108,0x951e0101,0xfb840108,0x951f0101,0xfb840108,0x95200101,0xfb840108,0x95210101,
	0xfb840108,0x95220101,0xfb840108,0x95230101,0xfb840108,0x95240101,0xfb840108,0x95250101,
	0xfb840108,0x95260101,0xfb840108,0x95270101,0xfb840108,0x95280101,0xfb840108,0x95290101,
	0xfb840108,0x952a0101,0xfb840108,0x952b0101,0xfb840108,0x952c0101,0xfb840108,0x952d0101,
	0xfb840108,0x952e0101,0xfb840108,0x952f0101,0xfb840108,0x95300101,0xfb840108,0x95310101,
	0xfb840108,0x95320101,0xfb840108,0x95330101,0xfb840108,0x95340101,0xfb840108,0x95350101,
	0xfb840108,0x95360101,0xfb840108,0x95370101,0xfb840108,0x95380101,0xfb840108,0x95390101,
	0xfb840108,0x953a0101,0xfb840108,0x953b0101,0xfb840108,0x953c0101,0xfb840108,0x953d0101,
	0xfb840108,0x953e0101,0xfb840108,0x953f0101,0xfb840108,0x95400101,0xfb840108,0x95410101,
	0xfb840108,0x95420101,0xfb840108,0x95430101,0xfb840108,0x95440101,0xfb840108,0x95450101,
	0xfb840108,0x95460101,0xfb840108,0x95470101,0xfb840108,0x95480101,0xfb840108,0x95490101,
	0xfb840108,0x954a0101,0xfb840108,0x954b0101,0xfb840108,0x954c0101,0xfb840108,0x954d0101,
	0xfb840108,0x954e0101,0xfb840108,0x954f0101,0xfb840108,0x95500101,0xfb840108,0x95510101,
	0xfb840108,0x95520101,0xfb840108,0x95530101,0xfb840108,0x95540101,0xfb840108,0x95550101,
	0xfb840108,0x95560101,0xfb840108,0x95570101,0xfb840108,0x95580101,0xfb840108,0x95590101,
	0xfb840108,0x955a0101,0xfb840108,0x955b0101,0xfb840108,0x955c0101,0xfb840108,0x955d0101,
	0xfb840108,0x955e0101,0xfb840108,0x955f0101,0xfb840108,0x95600101,0xfb840108,0x95610101,
	0xfb840108,0x95620101,0xfb840108,0x95630101,0xfb840108,0x95640101,0xfb840108,0x95650101,
	0xfb840108,0x95660101,0xfb840108,0x95670101,0xfb840108,0x95680101,0xfb840108,0x95690101,
	0xfb840108,0x956a0101,0xfb840108,0x956b0101,0xfb840108,0x956c0101,0xfb840108,0x956d0101,
	0xfb840108,0x956e0101,0xfb840108,0x956f0101,0xfb840108,0x95700101,0xfb840108,0x95710101,
	0xfb840108,0x95720101,0xfb840108,0x95730101,0xfb840108,0x95740101,0xfb840108,0x95750101,
	0xfb840108,0x95760101,0xfb840108,0x95770101,0xfb840108,0x95780101,0xfb840108,0x95790101,
	0xfb840108,0x957a0101,0xfb840108,0x957b0101,0xfb840108,0x957c0101,0xfb840108,0x957d0101,
	0xfb840108,0x957e0101,0xfb840108,0x957f0101,0xfb840108,0x95800101,0xfb840108,0x95810101,
	0xfb840108,0x95820101,0xfb840108,0x95830101,0xfb840108,0x95840101,0xfb840108,0x95850101,
	0xfb840108,0x95860101,0xfb840108,0x95870101,0xfb840108,0x95880101,0xfb840108,0x95890101,
	0xfb840108,0x958a0101,0xfb840108,0x958b0101,0xfb840108,0x958c0101,0xfb840108,0x958d0101,
	0xfb840108,0x958e0101,0xfb840108,0x958f0101,0xfb840108,0x95900101,0xfb840108,0x95910101,
	0xfb840108,0x95920101,0xfb840108,0x95930101,0xfb840108,0x95940101,0xfb840108,0x95950101,
	0xfb840108,0x95960101,0xfb840108,0x95970101,0xfb840108,0x95980101,0xfb840108,0x95990101,
	0xfb840108,0x959a0101,0xfb840108,0x959b0101,0xfb840108,0x959c0101,0xfb840108,0x959d0101,
	0xfb840108,0x959e0101,0xfb840108,0x959f0101,0xfb840108,0x95a00101,0xfb840108,0x95a10101,
	0xfb840108,0x95a20101,0xfb840108,0x95a30101,0xfb840108,0x95a40101,0xfb840108,0x95a50101,
	0xfb840108,0x95a60101,0xfb840108,0x95a70101,0xfb840108,0x95a80101,0xfb840108,0x95a90101,
	0xfb840108,0x95aa0101,0xfb840108,0x95ab0101,0xfb840108,0x95ac0101,0xfb840108,0x95ad0101,
	0xfb840108,0x95ae0101,0xfb840108,0x95af0101,0xfb840108,0x95b00101,0xfb840108,0x95b10101,
	0xfb840108,0x95b20101,0xfb840108,0x95b30101,0xfb840108,0x95b40101,0xfb840108,0x95b50101,
	0xfb840108,0x95b60101,0xfb840108,0x95b70101,0xfb840108,0x95b80101,0xfb840108,0x95b90101,
	0xfb840108,0x95ba0101,0xfb840108,0x95bb0101,0xfb840108,0x95bc0101,0xfb840108,0x95bd0101,
	0xfb840108,0x95be0101,0xfb840108,0x95bf0101,0xfb840108,0x95c00101,0xfb840108,0x95c10101,
	0xfb840108,0x95c20101,0xfb840108,0x95c30101,0xfb840108,0x95c40101,0xfb840108,0x95c50101,
	0xfb840108,0x95c60101,0xfb840108,0x95c70101,0xfb840108,0x95c80101,0xfb840108,0x95c90101,
	0xfb840108,0x95ca0101,0xfb840108,0x95cb0101,0xfb840108,0x95cc0101,0xfb840108,0x95cd0101,
	0xfb840108,0x95ce0101,0xfb840108,0x95cf0101,0xfb840108,0x95d00101,0xfb840108,0x95d10101,
	0xfb840108,0x95d20101,0xfb840108,0x95d30101,0xfb840108,0x95d40101,0xfb840108,0x95d50101,
	0xfb840108,0x95d60101,0xfb840108,0x95d70101,0xfb840108,0x95d80101,0xfb840108,0x95d90101,
	0xfb840108,0x95da0101,0xfb840108,0x95db0101,0xfb840108,0x95dc0101,0xfb840108,0x95dd0101,
	0xfb840108,0x95de0101,0xfb840108,0x95df0101,0xfb840108,0x95e00101,0xfb840108,0x95e10101,
	0xfb840108,0x95e20101,0xfb840108,0x95e30101,0xfb840108,0x95e40101,0xfb840108,0x95e50101,
	0xfb840108,0x95e60101,0xfb840108,0x95e70101,0xfb840108,0x95e80101,0xfb840108,0x95e90101,
	0xfb840108,0x95ea0101,0xfb840108,0x95eb0101,0xfb840108,0x95ec0101,0xfb840108,0x95ed0101,
	0xfb840108,0x95ee0101,0xfb840108,0x95ef0101,0xfb840108,0x95f00101,0xfb840108,0x95f10101,
	0xfb840108,0x95f20101,0xfb840108,0x95f30101,0xfb840108,0x95f40101,0xfb840108,0x95f50101,
	0xfb840108,0x95f60101,0xfb840108,0x95f70101,0xfb840108,0x95f80101,0xfb840108,0x95f90101,
	0xfb840108,0x95fa0101,0xfb840108,0x95fb0101,0xfb840108,0x95fc0101,0xfb840108,0x95fd0101,
	0xfb840108,0x95fe0101,0xfb840108,0x95ff0101,0xfb840108,0x96000101,0xfb840108,0x96010101,
	0xfb840108,0x96020101,0xfb840108,0x96030101,0xfb840108,0x96040101,0xfb840108,0x96050101,
	0xfb840108,0x96060101,0xfb840108,0x96070101,0xfb840108,0x96080101,0xfb840108,0x96090101,
	0xfb840108,0x960a0101,0xfb840108,0x960b0101,0xfb840108,0x960c0101,0xfb840108,0x960d0101,
	0xfb840108,0x960e0101,0xfb840108,0x960f0101,0xfb840108,0x96100101,0xfb840108,0x96110101,
	0xfb840108,0x96120101,0xfb840108,0x96130101,0xfb840108,0x96140101,0xfb840108,0x96150101,
	0xfb840108,0x96160101,0xfb840108,0x96170101,0xfb840108,0x96180101,0xfb840108,0x96190101,
	0xfb840108,0x961a0101,0xfb840108,0x961b0101,0xfb840108,0x961c0101,0xfb840108,0x961d0101,
	0xfb840108,0x961e0101,0xfb840108,0x961f0101,0xfb840108,0x96200101,0xfb840108,0x96210101,
	0xfb840108,0x96220101,0xfb840108,0x96230101,0xfb840108,0x96240101,0xfb840108,0x96250101,
	0xfb840108,0x96260101,0xfb840108,0x96270101,0xfb840108,0x96280101,0xfb840108,0x96290101,
	0xfb840108,0x962a0101,0xfb840108,0x962b0101,0xfb840108,0x962c0101,0xfb840108,0x962d0101,
	0xfb840108,0x962e0101,0xfb840108,0x962f0101,0xfb840108,0x96300101,0xfb840108,0x96310101,
	0xfb840108,0x96320101,0xfb840108,0x96330101,0xfb840108,0x96340101,0xfb840108,0x96350101,
	0xfb840108,0x96360101,0xfb840108,0x96370101,0xfb840108,0x96380101,0xfb840108,0x96390101,
	0xfb840108,0x963a0101,0xfb840108,0x963b0101,0xfb840108,0x963c0101,0xfb840108,0x963d0101,
	0xfb840108,0x963e0101,0xfb840108,0x963f0101,0xfb840108,0x96400101,0xfb840108,0x96410101,
	0xfb840108,0x96420101,0xfb840108,0x96430101,0xfb840108,0x96440101,0xfb840108,0x96450101,
	0xfb840108,0x96460101,0xfb840108,0x96470101,0xfb840108,0x96480101,0xfb840108,0x96490101,
	0xfb840108,0x964a0101,0xfb840108,0x964b0101,0xfb840108,0x964c0101,0xfb840108,0x964d0101,
	0xfb840108,0x964e0101,0xfb840108,0x964f0101,0xfb840108,0x96500101,0xfb840108,0x96510101,
	0xfb840108,0x96520101,0xfb840108,0x96530101,0xfb840108,0x96540101,0xfb840108,0x96550101,
	0xfb840108,0x96560101,0xfb840108,0x96570101,0xfb840108,0x96580101,0xfb840108,0x96590101,
	0xfb840108,0x965a0101,0xfb840108,0x965b0101,0xfb840108,0x965c0101,0xfb840108,0x965d0101,
	0xfb840108,0x965e0101,0xfb840108,0x965f0101,0xfb840108,0x96600101,0xfb840108,0x96610101,
	0xfb840108,0x96620101,0xfb840108,0x96630101,0xfb840108,0x96640101,0xfb840108,0x96650101,
	0xfb840108,0x96660101,0xfb840108,0x96670101,0xfb840108,0x96680101,0xfb840108,0x96690101,
	0xfb840108,0x966a0101,0xfb840108,0x966b0101,0xfb840108,0x966c0101,0xfb840108,0x966d0101,
	0xfb840108,0x966e0101,0xfb840108,0x966f0101,0xfb840108,0x96700101,0xfb840108,0x96710101,
	0xfb840108,0x96720101,0xfb840108,0x96730101,0xfb840108,0x96740101,0xfb840108,0x96750101,
	0xfb840108,0x96760101,0xfb840108,0x96770101,0xfb840108,0x96780101,0xfb840108,0x96790101,
	0xfb840108,0x967a0101,0xfb840108,0x967b0101,0xfb840108,0x967c0101,0xfb840108,0x967d0101,
	0xfb840108,0x967e0101,0xfb840108,0x967f0101,0xfb840108,0x96800101,0xfb840108,0x96810101,
	0xfb840108,0x96820101,0xfb840108,0x96830101,0xfb840108,0x96840101,0xfb840108,0x96850101,
	0xfb840108,0x96860101,0xfb840108,0x96870101,0xfb840108,0x96880101,0xfb840108,0x96890101,
	0xfb840108,0x968a0101,0xfb840108,0x968b0101,0xfb840108,0x968c0101,0xfb840108,0x968d0101,
	0xfb840108,0x968e0101,0xfb840108,0x968f0101,0xfb840108,0x96900101,0xfb840108,0x96910101,
	0xfb840108,0x96920101,0xfb840108,0x96930101,0xfb840108,0x96940101,0xfb840108,0x96950101,
	0xfb840108,0x96960101,0xfb840108,0x96970101,0xfb840108,0x96980101,0xfb840108,0x96990101,
	0xfb840108,0x969a0101,0xfb840108,0x969b0101,0xfb840108,0x969c0101,0xfb840108,0x969d0101,
	0xfb840108,0x969e0101,0xfb840108,0x969f0101,0xfb840108,0x96a00101,0xfb840108,0x96a10101,
	0xfb840108,0x96a20101,0xfb840108,0x96a30101,0xfb840108,0x96a40101,0xfb840108,0x96a50101,
	0xfb840108,0x96a60101,0xfb840108,0x96a70101,0xfb840108,0x96a80101,0xfb840108,0x96a90101,
	0xfb840108,0x96aa0101,0xfb840108,0x96ab0101,0xfb840108,0x96ac0101,0xfb840108,0x96ad0101,
	0xfb840108,0x96ae0101,0xfb840108,0x96af0101,0xfb840108,0x96b00101,0xfb840108,0x96b10101,
	0xfb840108,0x96b20101,0xfb840108,0x96b30101,0xfb840108,0x96b40101,0xfb840108,0x96b50101,
	0xfb840108,0x96b60101,0xfb840108,0x96b70101,0xfb840108,0x96b80101,0xfb840108,0x96b90101,
	0xfb840108,0x96ba0101,0xfb840108,0x96bb0101,0xfb840108,0x96bc0101,0xfb840108,0x96bd0101,
	0xfb840108,0x96be0101,0xfb840108,0x96bf0101,0xfb840108,0x96c00101,0xfb840108,0x96c10101,
	0xfb840108,0x96c20101,0xfb840108,0x96c30101,0xfb840108,0x96c40101,0xfb840108,0x96c50101,
	0xfb840108,0x96c60101,0xfb840108,0x96c70101,0xfb840108,0x96c80101,0xfb840108,0x96c90101,
	0xfb840108,0x96ca0101,0xfb840108,0x96cb0101,0xfb840108,0x96cc0101,0xfb840108,0x96cd0101,
	0xfb840108,0x96ce0101,0xfb840108,0x96cf0101,0xfb840108,0x96d00101,0xfb840108,0x96d10101,
	0xfb840108,0x96d20101,0xfb840108,0x96d30101,0xfb840108,0x96d40101,0xfb840108,0x96d50101,
	0xfb840108,0x96d60101,0xfb840108,0x96d70101,0xfb840108,0x96d80101,0xfb840108,0x96d90101,
	0xfb840108,0x96da0101,0xfb840108,0x96db0101,0xfb840108,0x96dc0101,0xfb840108,0x96dd0101,
	0xfb840108,0x96de0101,0xfb840108,0x96df0101,0xfb840108,0x96e00101,0xfb840108,0x96e10101,
	0xfb840108,0x96e20101,0xfb840108,0x96e30101,0xfb840108,0x96e40101,0xfb840108,0x96e50101,
	0xfb840108,0x96e60101,0xfb840108,0x96e70101,0xfb840108,0x96e80101,0xfb840108,0x96e90101,
	0xfb840108,0x96ea0101,0xfb840108,0x96eb0101,0xfb840108,0x96ec0101,0xfb840108,0x96ed0101,
	0xfb840108,0x96ee0101,0xfb840108,0x96ef0101,0xfb840108,0x96f00101,0xfb840108,0x96f10101,
	0xfb840108,0x96f20101,0xfb840108,0x96f30101,0xfb840108,0x96f40101,0xfb840108,0x96f50101,
	0xfb840108,0x96f60101,0xfb840108,0x96f70101,0xfb840108,0x96f80101,0xfb840108,0x96f90101,
	0xfb840108,0x96fa0101,0xfb840108,0x96fb0101,0xfb840108,0x96fc0101,0xfb840108,0x96fd0101,
	0xfb840108,0x96fe0101,0xfb840108,0x96ff0101,0xfb840108,0x97000101,0xfb840108,0x97010101,
	0xfb840108,0x97020101,0xfb840108,0x97030101,0xfb840108,0x97040101,0xfb840108,0x97050101,
	0xfb840108,0x97060101,0xfb840108,0x97070101,0xfb840108,0x97080101,0xfb840108,0x97090101,
	0xfb840108,0x970a0101,0xfb840108,0x970b0101,0xfb840108,0x970c0101,0xfb840108,0x970d0101,
	0xfb840108,0x970e0101,0xfb840108,0x970f0101,0xfb840108,0x97100101,0xfb840108,0x97110101,
	0xfb840108,0x97120101,0xfb840108,0x97130101,0xfb840108,0x97140101,0xfb840108,0x97150101,
	0xfb840108,0x97160101,0xfb840108,0x97170101,0xfb840108,0x97180101,0xfb840108,0x97190101,
	0xfb840108,0x971a0101,0xfb840108,0x971b0101,0xfb840108,0x971c0101,0xfb840108,0x971d0101,
	0xfb840108,0x971e0101,0xfb840108,0x971f0101,0xfb840108,0x97200101,0xfb840108,0x97210101,
	0xfb840108,0x97220101,0xfb840108,0x97230101,0xfb840108,0x97240101,0xfb840108,0x97250101,
	0xfb840108,0x97260101,0xfb840108,0x97270101,0xfb840108,0x97280101,0xfb840108,0x97290101,
	0xfb840108,0x972a0101,0xfb840108,0x972b0101,0xfb840108,0x972c0101,0xfb840108,0x972d0101,
	0xfb840108,0x972e0101,0xfb840108,0x972f0101,0xfb840108,0x97300101,0xfb840108,0x97310101,
	0xfb840108,0x97320101,0xfb840108,0x97330101,0xfb840108,0x97340101,0xfb840108,0x97350101,
	0xfb840108,0x97360101,0xfb840108,0x97370101,0xfb840108,0x97380101,0xfb840108,0x97390101,
	0xfb840108,0x973a0101,0xfb840108,0x973b0101,0xfb840108,0x973c0101,0xfb840108,0x973d0101,
	0xfb840108,0x973e0101,0xfb840108,0x973f0101,0xfb840108,0x97400101,0xfb840108,0x97410101,
	0xfb840108,0x97420101,0xfb840108,0x97430101,0xfb840108,0x97440101,0xfb840108,0x97450101,
	0xfb840108,0x97460101,0xfb840108,0x97470101,0xfb840108,0x97480101,0xfb840108,0x97490101,
	0xfb840108,0x974a0101,0xfb840108,0x974b0101,0xfb840108,0x974c0101,0xfb840108,0x974d0101,
	0xfb840108,0x974e0101,0xfb840108,0x974f0101,0xfb840108,0x97500101,0xfb840108,0x97510101,
	0xfb840108,0x97520101,0xfb840108,0x97530101,0xfb840108,0x97540101,0xfb840108,0x97550101,
	0xfb840108,0x97560101,0xfb840108,0x97570101,0xfb840108,0x97580101,0xfb840108,0x97590101,
	0xfb840108,0x975a0101,0xfb840108,0x975b0101,0xfb840108,0x975c0101,0xfb840108,0x975d0101,
	0xfb840108,0x975e0101,0xfb840108,0x975f0101,0xfb840108,0x97600101,0xfb840108,0x97610101,
	0xfb840108,0x97620101,0xfb840108,0x97630101,0xfb840108,0x97640101,0xfb840108,0x97650101,
	0xfb840108,0x97660101,0xfb840108,0x97670101,0xfb840108,0x97680101,0xfb840108,0x97690101,
	0xfb840108,0x976a0101,0xfb840108,0x976b0101,0xfb840108,0x976c0101,0xfb840108,0x976d0101,
	0xfb840108,0x976e0101,0xfb840108,0x976f0101,0xfb840108,0x97700101,0xfb840108,0x97710101,
	0xfb840108,0x97720101,0xfb840108,0x97730101,0xfb840108,0x97740101,0xfb840108,0x97750101,
	0xfb840108,0x97760101,0xfb840108,0x97770101,0xfb840108,0x97780101,0xfb840108,0x97790101,
	0xfb840108,0x977a0101,0xfb840108,0x977b0101,0xfb840108,0x977c0101,0xfb840108,0x977d0101,
	0xfb840108,0x977e0101,0xfb840108,0x977f0101,0xfb840108,0x97800101,0xfb840108,0x97810101,
	0xfb840108,0x97820101,0xfb840108,0x97830101,0xfb840108,0x97840101,0xfb840108,0x97850101,
	0xfb840108,0x97860101,0xfb840108,0x97870101,0xfb840108,0x97880101,0xfb840108,0x97890101,
	0xfb840108,0x978a0101,0xfb840108,0x978b0101,0xfb840108,0x978c0101,0xfb840108,0x978d0101,
	0xfb840108,0x978e0101,0xfb840108,0x978f0101,0xfb840108,0x97900101,0xfb840108,0x97910101,
	0xfb840108,0x97920101,0xfb840108,0x97930101,0xfb840108,0x97940101,0xfb840108,0x97950101,
	0xfb840108,0x97960101,0xfb840108,0x97970101,0xfb840108,0x97980101,0xfb840108,0x97990101,
	0xfb840108,0x979a0101,0xfb840108,0x979b0101,0xfb840108,0x979c0101,0xfb840108,0x979d0101,
	0xfb840108,0x979e0101,0xfb840108,0x979f0101,0xfb840108,0x97a00101,0xfb840108,0x97a10101,
	0xfb840108,0x97a20101,0xfb840108,0x97a30101,0xfb840108,0x97a40101,0xfb840108,0x97a50101,
	0xfb840108,0x97a60101,0xfb840108,0x97a70101,0xfb840108,0x97a80101,0xfb840108,0x97a90101,
	0xfb840108,0x97aa0101,0xfb840108,0x97ab0101,0xfb840108,0x97ac0101,0xfb840108,0x97ad0101,
	0xfb840108,0x97ae0101,0xfb840108,0x97af0101,0xfb840108,0x97b00101,0xfb840108,0x97b10101,
	0xfb840108,0x97b20101,0xfb840108,0x97b30101,0xfb840108,0x97b40101,0xfb840108,0x97b50101,
	0xfb840108,0x97b60101,0xfb840108,0x97b70101,0xfb840108,0x97b80101,0xfb840108,0x97b90101,
	0xfb840108,0x97ba0101,0xfb840108,0x97bb0101,0xfb840108,0x97bc0101,0xfb840108,0x97bd0101,
	0xfb840108,0x97be0101,0xfb840108,0x97bf0101,0xfb840108,0x97c00101,0xfb840108,0x97c10101,
	0xfb840108,0x97c20101,0xfb840108,0x97c30101,0xfb840108,0x97c40101,0xfb840108,0x97c50101,
	0xfb840108,0x97c60101,0xfb840108,0x97c70101,0xfb840108,0x97c80101,0xfb840108,0x97c90101,
	0xfb840108,0x97ca0101,0xfb840108,0x97cb0101,0xfb840108,0x97cc0101,0xfb840108,0x97cd0101,
	0xfb840108,0x97ce0101,0xfb840108,0x97cf0101,0xfb840108,0x97d00101,0xfb840108,0x97d10101,
	0xfb840108,0x97d20101,0xfb840108,0x97d30101,0xfb840108,0x97d40101,0xfb840108,0x97d50101,
	0xfb840108,0x97d60101,0xfb840108,0x97d70101,0xfb840108,0x97d80101,0xfb840108,0x97d90101,
	0xfb840108,0x97da0101,0xfb840108,0x97db0101,0xfb840108,0x97dc0101,0xfb840108,0x97dd0101,
	0xfb840108,0x97de0101,0xfb840108,0x97df0101,0xfb840108,0x97e00101,0xfb840108,0x97e10101,
	0xfb840108,0x97e20101,0xfb840108,0x97e30101,0xfb840108,0x97e40101,0xfb840108,0x97e50101,
	0xfb840108,0x97e60101,0xfb840108,0x97e70101,0xfb840108,0x97e80101,0xfb840108,0x97e90101,
	0xfb840108,0x97ea0101,0xfb840108,0x97eb0101,0xfb840108,0x97ec0101,0xfb840108,0x97ed0101,
	0xfb840108,0x97ee0101,0xfb840108,0x97ef0101,0xfb840108,0x97f00101,0xfb840108,0x97f10101,
	0xfb840108,0x97f20101,0xfb840108,0x97f30101,0xfb840108,0x97f40101,0xfb840108,0x97f50101,
	0xfb840108,0x97f60101,0xfb840108,0x97f70101,0xfb840108,0x97f80101,0xfb840108,0x97f90101,
	0xfb840108,0x97fa0101,0xfb840108,0x97fb0101,0xfb840108,0x97fc0101,0xfb840108,0x97fd0101,
	0xfb840108,0x97fe0101,0xfb840108,0x97ff0101,0xfb840108,0x98000101,0xfb840108,0x98010101,
	0xfb840108,0x98020101,0xfb840108,0x98030101,0xfb840108,0x98040101,0xfb840108,0x98050101,
	0xfb840108,0x98060101,0xfb840108,0x98070101,0xfb840108,0x98080101,0xfb840108,0x98090101,
	0xfb840108,0x980a0101,0xfb840108,0x980b0101,0xfb840108,0x980c0101,0xfb840108,0x980d0101,
	0xfb840108,0x980e0101,0xfb840108,0x980f0101,0xfb840108,0x98100101,0xfb840108,0x98110101,
	0xfb840108,0x98120101,0xfb840108,0x98130101,0xfb840108,0x98140101,0xfb840108,0x98150101,
	0xfb840108,0x98160101,0xfb840108,0x98170101,0xfb840108,0x98180101,0xfb840108,0x98190101,
	0xfb840108,0x981a0101,0xfb840108,0x981b0101,0xfb840108,0x981c0101,0xfb840108,0x981d0101,
	0xfb840108,0x981e0101,0xfb840108,0x981f0101,0xfb840108,0x98200101,0xfb840108,0x98210101,
	0xfb840108,0x98220101,0xfb840108,0x98230101,0xfb840108,0x98240101,0xfb840108,0x98250101,
	0xfb840108,0x98260101,0xfb840108,0x98270101,0xfb840108,0x98280101,0xfb840108,0x98290101,
	0xfb840108,0x982a0101,0xfb840108,0x982b0101,0xfb840108,0x982c0101,0xfb840108,0x982d0101,
	0xfb840108,0x982e0101,0xfb840108,0x982f0101,0xfb840108,0x98300101,0xfb840108,0x98310101,
	0xfb840108,0x98320101,0xfb840108,0x98330101,0xfb840108,0x98340101,0xfb840108,0x98350101,
	0xfb840108,0x98360101,0xfb840108,0x98370101,0xfb840108,0x98380101,0xfb840108,0x98390101,
	0xfb840108,0x983a0101,0xfb840108,0x983b0101,0xfb840108,0x983c0101,0xfb840108,0x983d0101,
	0xfb840108,0x983e0101,0xfb840108,0x983f0101,0xfb840108,0x98400101,0xfb840108,0x98410101,
	0xfb840108,0x98420101,0xfb840108,0x98430101,0xfb840108,0x98440101,0xfb840108,0x98450101,
	0xfb840108,0x98460101,0xfb840108,0x98470101,0xfb840108,0x98480101,0xfb840108,0x98490101,
	0xfb840108,0x984a0101,0xfb840108,0x984b0101,0xfb840108,0x984c0101,0xfb840108,0x984d0101,
	0xfb840108,0x984e0101,0xfb840108,0x984f0101,0xfb840108,0x98500101,0xfb840108,0x98510101,
	0xfb840108,0x98520101,0xfb840108,0x98530101,0xfb840108,0x98540101,0xfb840108,0x98550101,
	0xfb840108,0x98560101,0xfb840108,0x98570101,0xfb840108,0x98580101,0xfb840108,0x98590101,
	0xfb840108,0x985a0101,0xfb840108,0x985b0101,0xfb840108,0x985c0101,0xfb840108,0x985d0101,
	0xfb840108,0x985e0101,0xfb840108,0x985f0101,0xfb840108,0x98600101,0xfb840108,0x98610101,
	0xfb840108,0x98620101,0xfb840108,0x98630101,0xfb840108,0x98640101,0xfb840108,0x98650101,
	0xfb840108,0x98660101,0xfb840108,0x98670101,0xfb840108,0x98680101,0xfb840108,0x98690101,
	0xfb840108,0x986a0101,0xfb840108,0x986b0101,0xfb840108,0x986c0101,0xfb840108,0x986d0101,
	0xfb840108,0x986e0101,0xfb840108,0x986f0101,0xfb840108,0x98700101,0xfb840108,0x98710101,
	0xfb840108,0x98720101,0xfb840108,0x98730101,0xfb840108,0x98740101,0xfb840108,0x98750101,
	0xfb840108,0x98760101,0xfb840108,0x98770101,0xfb840108,0x98780101,0xfb840108,0x98790101,
	0xfb840108,0x987a0101,0xfb840108,0x987b0101,0xfb840108,0x987c0101,0xfb840108,0x987d0101,
	0xfb840108,0x987e0101,0xfb840108,0x987f0101,0xfb840108,0x98800101,0xfb840108,0x98810101,
	0xfb840108,0x98820101,0xfb840108,0x98830101,0xfb840108,0x98840101,0xfb840108,0x98850101,
	0xfb840108,0x98860101,0xfb840108,0x98870101,0xfb840108,0x98880101,0xfb840108,0x98890101,
	0xfb840108,0x988a0101,0xfb840108,0x988b0101,0xfb840108,0x988c0101,0xfb840108,0x988d0101,
	0xfb840108,0x988e0101,0xfb840108,0x988f0101,0xfb840108,0x98900101,0xfb840108,0x98910101,
	0xfb840108,0x98920101,0xfb840108,0x98930101,0xfb840108,0x98940101,0xfb840108,0x98950101,
	0xfb840108,0x98960101,0xfb840108,0x98970101,0xfb840108,0x98980101,0xfb840108,0x98990101,
	0xfb840108,0x989a0101,0xfb840108,0x989b0101,0xfb840108,0x989c0101,0xfb840108,0x989d0101,
	0xfb840108,0x989e0101,0xfb840108,0x989f0101,0xfb840108,0x98a00101,0xfb840108,0x98a10101,
	0xfb840108,0x98a20101,0xfb840108,0x98a30101,0xfb840108,0x98a40101,0xfb840108,0x98a50101,
	0xfb840108,0x98a60101,0xfb840108,0x98a70101,0xfb840108,0x98a80101,0xfb840108,0x98a90101,
	0xfb840108,0x98aa0101,0xfb840108,0x98ab0101,0xfb840108,0x98ac0101,0xfb840108,0x98ad0101,
	0xfb840108,0x98ae0101,0xfb840108,0x98af0101,0xfb840108,0x98b00101,0xfb840108,0x98b10101,
	0xfb840108,0x98b20101,0xfb840108,0x98b30101,0xfb840108,0x98b40101,0xfb840108,0x98b50101,
	0xfb840108,0x98b60101,0xfb840108,0x98b70101,0xfb840108,0x98b80101,0xfb840108,0x98b90101,
	0xfb840108,0x98ba0101,0xfb840108,0x98bb0101,0xfb840108,0x98bc0101,0xfb840108,0x98bd0101,
	0xfb840108,0x98be0101,0xfb840108,0x98bf0101,0xfb840108,0x98c00101,0xfb840108,0x98c10101,
	0xfb840108,0x98c20101,0xfb840108,0x98c30101,0xfb840108,0x98c40101,0xfb840108,0x98c50101,
	0xfb840108,0x98c60101,0xfb840108,0x98c70101,0xfb840108,0x98c80101,0xfb840108,0x98c90101,
	0xfb840108,0x98ca0101,0xfb840108,0x98cb0101,0xfb840108,0x98cc0101,0xfb840108,0x98cd0101,
	0xfb840108,0x98ce0101,0xfb840108,0x98cf0101,0xfb840108,0x98d00101,0xfb840108,0x98d10101,
	0xfb840108,0x98d20101,0xfb840108,0x98d30101,0xfb840108,0x98d40101,0xfb840108,0x98d50101,
	0xfb840108,0x98d60101,0xfb840108,0x98d70101,0xfb840108,0x98d80101,0xfb840108,0x98d90101,
	0xfb840108,0x98da0101,0xfb840108,0x98db0101,0xfb840108,0x98dc0101,0xfb840108,0x98dd0101,
	0xfb840108,0x98de0101,0xfb840108,0x98df0101,0xfb840108,0x98e00101,0xfb840108,0x98e10101,
	0xfb840108,0x98e20101,0xfb840108,0x98e30101,0xfb840108,0x98e40101,0xfb840108,0x98e50101,
	0xfb840108,0x98e60101,0xfb840108,0x98e70101,0xfb840108,0x98e80101,0xfb840108,0x98e90101,
	0xfb840108,0x98ea0101,0xfb840108,0x98eb0101,0xfb840108,0x98ec0101,0xfb840108,0x98ed0101,
	0xfb840108,0x98ee0101,0xfb840108,0x98ef0101,0xfb840108,0x98f00101,0xfb840108,0x98f10101,
	0xfb840108,0x98f20101,0xfb840108,0x98f30101,0xfb840108,0x98f40101,0xfb840108,0x98f50101,
	0xfb840108,0x98f60101,0xfb840108,0x98f70101,0xfb840108,0x98f80101,0xfb840108,0x98f90101,
	0xfb840108,0x98fa0101,0xfb840108,0x98fb0101,0xfb840108,0x98fc0101,0xfb840108,0x98fd0101,
	0xfb840108,0x98fe0101,0xfb840108,0x98ff0101,0xfb840108,0x99000101,0xfb840108,0x99010101,
	0xfb840108,0x99020101,0xfb840108,0x99030101,0xfb840108,0x99040101,0xfb840108,0x99050101,
	0xfb840108,0x99060101,0xfb840108,0x99070101,0xfb840108,0x99080101,0xfb840108,0x99090101,
	0xfb840108,0x990a0101,0xfb840108,0x990b0101,0xfb840108,0x990c0101,0xfb840108,0x990d0101,
	0xfb840108,0x990e0101,0xfb840108,0x990f0101,0xfb840108,0x99100101,0xfb840108,0x99110101,
	0xfb840108,0x99120101,0xfb840108,0x99130101,0xfb840108,0x99140101,0xfb840108,0x99150101,
	0xfb840108,0x99160101,0xfb840108,0x99170101,0xfb840108,0x99180101,0xfb840108,0x99190101,
	0xfb840108,0x991a0101,0xfb840108,0x991b0101,0xfb840108,0x991c0101,0xfb840108,0x991d0101,
	0xfb840108,0x991e0101,0xfb840108,0x991f0101,0xfb840108,0x99200101,0xfb840108,0x99210101,
	0xfb840108,0x99220101,0xfb840108,0x99230101,0xfb840108,0x99240101,0xfb840108,0x99250101,
	0xfb840108,0x99260101,0xfb840108,0x99270101,0xfb840108,0x99280101,0xfb840108,0x99290101,
	0xfb840108,0x992a0101,0xfb840108,0x992b0101,0xfb840108,0x992c0101,0xfb840108,0x992d0101,
	0xfb840108,0x992e0101,0xfb840108,0x992f0101,0xfb840108,0x99300101,0xfb840108,0x99310101,
	0xfb840108,0x99320101,0xfb840108,0x99330101,0xfb840108,0x99340101,0xfb840108,0x99350101,
	0xfb840108,0x99360101,0xfb840108,0x99370101,0xfb840108,0x99380101,0xfb840108,0x99390101,
	0xfb840108,0x993a0101,0xfb840108,0x993b0101,0xfb840108,0x993c0101,0xfb840108,0x993d0101,
	0xfb840108,0x993e0101,0xfb840108,0x993f0101,0xfb840108,0x99400101,0xfb840108,0x99410101,
	0xfb840108,0x99420101,0xfb840108,0x99430101,0xfb840108,0x99440101,0xfb840108,0x99450101,
	0xfb840108,0x99460101,0xfb840108,0x99470101,0xfb840108,0x99480101,0xfb840108,0x99490101,
	0xfb840108,0x994a0101,0xfb840108,0x994b0101,0xfb840108,0x994c0101,0xfb840108,0x994d0101,
	0xfb840108,0x994e0101,0xfb840108,0x994f0101,0xfb840108,0x99500101,0xfb840108,0x99510101,
	0xfb840108,0x99520101,0xfb840108,0x99530101,0xfb840108,0x99540101,0xfb840108,0x99550101,
	0xfb840108,0x99560101,0xfb840108,0x99570101,0xfb840108,0x99580101,0xfb840108,0x99590101,
	0xfb840108,0x995a0101,0xfb840108,0x995b0101,0xfb840108,0x995c0101,0xfb840108,0x995d0101,
	0xfb840108,0x995e0101,0xfb840108,0x995f0101,0xfb840108,0x99600101,0xfb840108,0x99610101,
	0xfb840108,0x99620101,0xfb840108,0x99630101,0xfb840108,0x99640101,0xfb840108,0x99650101,
	0xfb840108,0x99660101,0xfb840108,0x99670101,0xfb840108,0x99680101,0xfb840108,0x99690101,
	0xfb840108,0x996a0101,0xfb840108,0x996b0101,0xfb840108,0x996c0101,0xfb840108,0x996d0101,
	0xfb840108,0x996e0101,0xfb840108,0x996f0101,0xfb840108,0x99700101,0xfb840108,0x99710101,
	0xfb840108,0x99720101,0xfb840108,0x99730101,0xfb840108,0x99740101,0xfb840108,0x99750101,
	0xfb840108,0x99760101,0xfb840108,0x99770101,0xfb840108,0x99780101,0xfb840108,0x99790101,
	0xfb840108,0x997a0101,0xfb840108,0x997b0101,0xfb840108,0x997c0101,0xfb840108,0x997d0101,
	0xfb840108,0x997e0101,0xfb840108,0x997f0101,0xfb840108,0x99800101,0xfb840108,0x99810101,
	0xfb840108,0x99820101,0xfb840108,0x99830101,0xfb840108,0x99840101,0xfb840108,0x99850101,
	0xfb840108,0x99860101,0xfb840108,0x99870101,0xfb840108,0x99880101,0xfb840108,0x99890101,
	0xfb840108,0x998a0101,0xfb840108,0x998b0101,0xfb840108,0x998c0101,0xfb840108,0x998d0101,
	0xfb840108,0x998e0101,0xfb840108,0x998f0101,0xfb840108,0x99900101,0xfb840108,0x99910101,
	0xfb840108,0x99920101,0xfb840108,0x99930101,0xfb840108,0x99940101,0xfb840108,0x99950101,
	0xfb840108,0x99960101,0xfb840108,0x99970101,0xfb840108,0x99980101,0xfb840108,0x99990101,
	0xfb840108,0x999a0101,0xfb840108,0x999b0101,0xfb840108,0x999c0101,0xfb840108,0x999d0101,
	0xfb840108,0x999e0101,0xfb840108,0x999f0101,0xfb840108,0x99a00101,0xfb840108,0x99a10101,
	0xfb840108,0x99a20101,0xfb840108,0x99a30101,0xfb840108,0x99a40101,0xfb840108,0x99a50101,
	0xfb840108,0x99a60101,0xfb840108,0x99a70101,0xfb840108,0x99a80101,0xfb840108,0x99a90101,
	0xfb840108,0x99aa0101,0xfb840108,0x99ab0101,0xfb840108,0x99ac0101,0xfb840108,0x99ad0101,
	0xfb840108,0x99ae0101,0xfb840108,0x99af0101,0xfb840108,0x99b00101,0xfb840108,0x99b10101,
	0xfb840108,0x99b20101,0xfb840108,0x99b30101,0xfb840108,0x99b40101,0xfb840108,0x99b50101,
	0xfb840108,0x99b60101,0xfb840108,0x99b70101,0xfb840108,0x99b80101,0xfb840108,0x99b90101,
	0xfb840108,0x99ba0101,0xfb840108,0x99bb0101,0xfb840108,0x99bc0101,0xfb840108,0x99bd0101,
	0xfb840108,0x99be0101,0xfb840108,0x99bf0101,0xfb840108,0x99c00101,0xfb840108,0x99c10101,
	0xfb840108,0x99c20101,0xfb840108,0x99c30101,0xfb840108,0x99c40101,0xfb840108,0x99c50101,
	0xfb840108,0x99c60101,0xfb840108,0x99c70101,0xfb840108,0x99c80101,0xfb840108,0x99c90101,
	0xfb840108,0x99ca0101,0xfb840108,0x99cb0101,0xfb840108,0x99cc0101,0xfb840108,0x99cd0101,
	0xfb840108,0x99ce0101,0xfb840108,0x99cf0101,0xfb840108,0x99d00101,0xfb840108,0x99d10101,
	0xfb840108,0x99d20101,0xfb840108,0x99d30101,0xfb840108,0x99d40101,0xfb840108,0x99d50101,
	0xfb840108,0x99d60101,0xfb840108,0x99d70101,0xfb840108,0x99d80101,0xfb840108,0x99d90101,
	0xfb840108,0x99da0101,0xfb840108,0x99db0101,0xfb840108,0x99dc0101,0xfb840108,0x99dd0101,
	0xfb840108,0x99de0101,0xfb840108,0x99df0101,0xfb840108,0x99e00101,0xfb840108,0x99e10101,
	0xfb840108,0x99e20101,0xfb840108,0x99e30101,0xfb840108,0x99e40101,0xfb840108,0x99e50101,
	0xfb840108,0x99e60101,0xfb840108,0x99e70101,0xfb840108,0x99e80101,0xfb840108,0x99e90101,
	0xfb840108,0x99ea0101,0xfb840108,0x99eb0101,0xfb840108,0x99ec0101,0xfb840108,0x99ed0101,
	0xfb840108,0x99ee0101,0xfb840108,0x99ef0101,0xfb840108,0x99f00101,0xfb840108,0x99f10101,
	0xfb840108,0x99f20101,0xfb840108,0x99f30101,0xfb840108,0x99f40101,0xfb840108,0x99f50101,
	0xfb840108,0x99f60101,0xfb840108,0x99f70101,0xfb840108,0x99f80101,0xfb840108,0x99f90101,
	0xfb840108,0x99fa0101,0xfb840108,0x99fb0101,0xfb840108,0x99fc0101,0xfb840108,0x99fd0101,
	0xfb840108,0x99fe0101,0xfb840108,0x99ff0101,0xfb840108,0x9a000101,0xfb840108,0x9a010101,
	0xfb840108,0x9a020101,0xfb840108,0x9a030101,0xfb840108,0x9a040101,0xfb840108,0x9a050101,
	0xfb840108,0x9a060101,0xfb840108,0x9a070101,0xfb840108,0x9a080101,0xfb840108,0x9a090101,
	0xfb840108,0x9a0a0101,0xfb840108,0x9a0b0101,0xfb840108,0x9a0c0101,0xfb840108,0x9a0d0101,
	0xfb840108,0x9a0e0101,0xfb840108,0x9a0f0101,0xfb840108,0x9a100101,0xfb840108,0x9a110101,
	0xfb840108,0x9a120101,0xfb840108,0x9a130101,0xfb840108,0x9a140101,0xfb840108,0x9a150101,
	0xfb840108,0x9a160101,0xfb840108,0x9a170101,0xfb840108,0x9a180101,0xfb840108,0x9a190101,
	0xfb840108,0x9a1a0101,0xfb840108,0x9a1b0101,0xfb840108,0x9a1c0101,0xfb840108,0x9a1d0101,
	0xfb840108,0x9a1e0101,0xfb840108,0x9a1f0101,0xfb840108,0x9a200101,0xfb840108,0x9a210101,
	0xfb840108,0x9a220101,0xfb840108,0x9a230101,0xfb840108,0x9a240101,0xfb840108,0x9a250101,
	0xfb840108,0x9a260101,0xfb840108,0x9a270101,0xfb840108,0x9a280101,0xfb840108,0x9a290101,
	0xfb840108,0x9a2a0101,0xfb840108,0x9a2b0101,0xfb840108,0x9a2c0101,0xfb840108,0x9a2d0101,
	0xfb840108,0x9a2e0101,0xfb840108,0x9a2f0101,0xfb840108,0x9a300101,0xfb840108,0x9a310101,
	0xfb840108,0x9a320101,0xfb840108,0x9a330101,0xfb840108,0x9a340101,0xfb840108,0x9a350101,
	0xfb840108,0x9a360101,0xfb840108,0x9a370101,0xfb840108,0x9a380101,0xfb840108,0x9a390101,
	0xfb840108,0x9a3a0101,0xfb840108,0x9a3b0101,0xfb840108,0x9a3c0101,0xfb840108,0x9a3d0101,
	0xfb840108,0x9a3e0101,0xfb840108,0x9a3f0101,0xfb840108,0x9a400101,0xfb840108,0x9a410101,
	0xfb840108,0x9a420101,0xfb840108,0x9a430101,0xfb840108,0x9a440101,0xfb840108,0x9a450101,
	0xfb840108,0x9a460101,0xfb840108,0x9a470101,0xfb840108,0x9a480101,0xfb840108,0x9a490101,
	0xfb840108,0x9a4a0101,0xfb840108,0x9a4b0101,0xfb840108,0x9a4c0101,0xfb840108,0x9a4d0101,
	0xfb840108,0x9a4e0101,0xfb840108,0x9a4f0101,0xfb840108,0x9a500101,0xfb840108,0x9a510101,
	0xfb840108,0x9a520101,0xfb840108,0x9a530101,0xfb840108,0x9a540101,0xfb840108,0x9a550101,
	0xfb840108,0x9a560101,0xfb840108,0x9a570101,0xfb840108,0x9a580101,0xfb840108,0x9a590101,
	0xfb840108,0x9a5a0101,0xfb840108,0x9a5b0101,0xfb840108,0x9a5c0101,0xfb840108,0x9a5d0101,
	0xfb840108,0x9a5e0101,0xfb840108,0x9a5f0101,0xfb840108,0x9a600101,0xfb840108,0x9a610101,
	0xfb840108,0x9a620101,0xfb840108,0x9a630101,0xfb840108,0x9a640101,0xfb840108,0x9a650101,
	0xfb840108,0x9a660101,0xfb840108,0x9a670101,0xfb840108,0x9a680101,0xfb840108,0x9a690101,
	0xfb840108,0x9a6a0101,0xfb840108,0x9a6b0101,0xfb840108,0x9a6c0101,0xfb840108,0x9a6d0101,
	0xfb840108,0x9a6e0101,0xfb840108,0x9a6f0101,0xfb840108,0x9a700101,0xfb840108,0x9a710101,
	0xfb840108,0x9a720101,0xfb840108,0x9a730101,0xfb840108,0x9a740101,0xfb840108,0x9a750101,
	0xfb840108,0x9a760101,0xfb840108,0x9a770101,0xfb840108,0x9a780101,0xfb840108,0x9a790101,
	0xfb840108,0x9a7a0101,0xfb840108,0x9a7b0101,0xfb840108,0x9a7c0101,0xfb840108,0x9a7d0101,
	0xfb840108,0x9a7e0101,0xfb840108,0x9a7f0101,0xfb840108,0x9a800101,0xfb840108,0x9a810101,
	0xfb840108,0x9a820101,0xfb840108,0x9a830101,0xfb840108,0x9a840101,0xfb840108,0x9a850101,
	0xfb840108,0x9a860101,0xfb840108,0x9a870101,0xfb840108,0x9a880101,0xfb840108,0x9a890101,
	0xfb840108,0x9a8a0101,0xfb840108,0x9a8b0101,0xfb840108,0x9a8c0101,0xfb840108,0x9a8d0101,
	0xfb840108,0x9a8e0101,0xfb840108,0x9a8f0101,0xfb840108,0x9a900101,0xfb840108,0x9a910101,
	0xfb840108,0x9a920101,0xfb840108,0x9a930101,0xfb840108,0x9a940101,0xfb840108,0x9a950101,
	0xfb840108,0x9a960101,0xfb840108,0x9a970101,0xfb840108,0x9a980101,0xfb840108,0x9a990101,
	0xfb840108,0x9a9a0101,0xfb840108,0x9a9b0101,0xfb840108,0x9a9c0101,0xfb840108,0x9a9d0101,
	0xfb840108,0x9a9e0101,0xfb840108,0x9a9f0101,0xfb840108,0x9aa00101,0xfb840108,0x9aa10101,
	0xfb840108,0x9aa20101,0xfb840108,0x9aa30101,0xfb840108,0x9aa40101,0xfb840108,0x9aa50101,
	0xfb840108,0x9aa60101,0xfb840108,0x9aa70101,0xfb840108,0x9aa80101,0xfb840108,0x9aa90101,
	0xfb840108,0x9aaa0101,0xfb840108,0x9aab0101,0xfb840108,0x9aac0101,0xfb840108,0x9aad0101,
	0xfb840108,0x9aae0101,0xfb840108,0x9aaf0101,0xfb840108,0x9ab00101,0xfb840108,0x9ab10101,
	0xfb840108,0x9ab20101,0xfb840108,0x9ab30101,0xfb840108,0x9ab40101,0xfb840108,0x9ab50101,
	0xfb840108,0x9ab60101,0xfb840108,0x9ab70101,0xfb840108,0x9ab80101,0xfb840108,0x9ab90101,
	0xfb840108,0x9aba0101,0xfb840108,0x9abb0101,0xfb840108,0x9abc0101,0xfb840108,0x9abd0101,
	0xfb840108,0x9abe0101,0xfb840108,0x9abf0101,0xfb840108,0x9ac00101,0xfb840108,0x9ac10101,
	0xfb840108,0x9ac20101,0xfb840108,0x9ac30101,0xfb840108,0x9ac40101,0xfb840108,0x9ac50101,
	0xfb840108,0x9ac60101,0xfb840108,0x9ac70101,0xfb840108,0x9ac80101,0xfb840108,0x9ac90101,
	0xfb840108,0x9aca0101,0xfb840108,0x9acb0101,0xfb840108,0x9acc0101,0xfb840108,0x9acd0101,
	0xfb840108,0x9ace0101,0xfb840108,0x9acf0101,0xfb840108,0x9ad00101,0xfb840108,0x9ad10101,
	0xfb840108,0x9ad20101,0xfb840108,0x9ad30101,0xfb840108,0x9ad40101,0xfb840108,0x9ad50101,
	0xfb840108,0x9ad60101,0xfb840108,0x9ad70101,0xfb840108,0x9ad80101,0xfb840108,0x9ad90101,
	0xfb840108,0x9ada0101,0xfb840108,0x9adb0101,0xfb840108,0x9adc0101,0xfb840108,0x9add0101,
	0xfb840108,0x9ade0101,0xfb840108,0x9adf0101,0xfb840108,0x9ae00101,0xfb840108,0x9ae10101,
	0xfb840108,0x9ae20101,0xfb840108,0x9ae30101,0xfb840108,0x9ae40101,0xfb840108,0x9ae50101,
	0xfb840108,0x9ae60101,0xfb840108,0x9ae70101,0xfb840108,0x9ae80101,0xfb840108,0x9ae90101,
	0xfb840108,0x9aea0101,0xfb840108,0x9aeb0101,0xfb840108,0x9aec0101,0xfb840108,0x9aed0101,
	0xfb840108,0x9aee0101,0xfb840108,0x9aef0101,0xfb840108,0x9af00101,0xfb840108,0x9af10101,
	0xfb840108,0x9af20101,0xfb840108,0x9af30101,0xfb840108,0x9af40101,0xfb840108,0x9af50101,
	0xfb840108,0x9af60101,0xfb840108,0x9af70101,0xfb840108,0x9af80101,0xfb840108,0x9af90101,
	0xfb840108,0x9afa0101,0xfb840108,0x9afb0101,0xfb840108,0x9afc0101,0xfb840108,0x9afd0101,
	0xfb840108,0x9afe0101,0xfb840108,0x9aff0101,0xfb840108,0x9b000101,0xfb840108,0x9b010101,
	0xfb840108,0x9b020101,0xfb840108,0x9b030101,0xfb840108,0x9b040101,0xfb840108,0x9b050101,
	0xfb840108,0x9b060101,0xfb840108,0x9b070101,0xfb840108,0x9b080101,0xfb840108,0x9b090101,
	0xfb840108,0x9b0a0101,0xfb840108,0x9b0b0101,0xfb840108,0x9b0c0101,0xfb840108,0x9b0d0101,
	0xfb840108,0x9b0e0101,0xfb840108,0x9b0f0101,0xfb840108,0x9b100101,0xfb840108,0x9b110101,
	0xfb840108,0x9b120101,0xfb840108,0x9b130101,0xfb840108,0x9b140101,0xfb840108,0x9b150101,
	0xfb840108,0x9b160101,0xfb840108,0x9b170101,0xfb840108,0x9b180101,0xfb840108,0x9b190101,
	0xfb840108,0x9b1a0101,0xfb840108,0x9b1b0101,0xfb840108,0x9b1c0101,0xfb840108,0x9b1d0101,
	0xfb840108,0x9b1e0101,0xfb840108,0x9b1f0101,0xfb840108,0x9b200101,0xfb840108,0x9b210101,
	0xfb840108,0x9b220101,0xfb840108,0x9b230101,0xfb840108,0x9b240101,0xfb840108,0x9b250101,
	0xfb840108,0x9b260101,0xfb840108,0x9b270101,0xfb840108,0x9b280101,0xfb840108,0x9b290101,
	0xfb840108,0x9b2a0101,0xfb840108,0x9b2b0101,0xfb840108,0x9b2c0101,0xfb840108,0x9b2d0101,
	0xfb840108,0x9b2e0101,0xfb840108,0x9b2f0101,0xfb840108,0x9b300101,0xfb840108,0x9b310101,
	0xfb840108,0x9b320101,0xfb840108,0x9b330101,0xfb840108,0x9b340101,0xfb840108,0x9b350101,
	0xfb840108,0x9b360101,0xfb840108,0x9b370101,0xfb840108,0x9b380101,0xfb840108,0x9b390101,
	0xfb840108,0x9b3a0101,0xfb840108,0x9b3b0101,0xfb840108,0x9b3c0101,0xfb840108,0x9b3d0101,
	0xfb840108,0x9b3e0101,0xfb840108,0x9b3f0101,0xfb840108,0x9b400101,0xfb840108,0x9b410101,
	0xfb840108,0x9b420101,0xfb840108,0x9b430101,0xfb840108,0x9b440101,0xfb840108,0x9b450101,
	0xfb840108,0x9b460101,0xfb840108,0x9b470101,0xfb840108,0x9b480101,0xfb840108,0x9b490101,
	0xfb840108,0x9b4a0101,0xfb840108,0x9b4b0101,0xfb840108,0x9b4c0101,0xfb840108,0x9b4d0101,
	0xfb840108,0x9b4e0101,0xfb840108,0x9b4f0101,0xfb840108,0x9b500101,0xfb840108,0x9b510101,
	0xfb840108,0x9b520101,0xfb840108,0x9b530101,0xfb840108,0x9b540101,0xfb840108,0x9b550101,
	0xfb840108,0x9b560101,0xfb840108,0x9b570101,0xfb840108,0x9b580101,0xfb840108,0x9b590101,
	0xfb840108,0x9b5a0101,0xfb840108,0x9b5b0101,0xfb840108,0x9b5c0101,0xfb840108,0x9b5d0101,
	0xfb840108,0x9b5e0101,0xfb840108,0x9b5f0101,0xfb840108,0x9b600101,0xfb840108,0x9b610101,
	0xfb840108,0x9b620101,0xfb840108,0x9b630101,0xfb840108,0x9b640101,0xfb840108,0x9b650101,
	0xfb840108,0x9b660101,0xfb840108,0x9b670101,0xfb840108,0x9b680101,0xfb840108,0x9b690101,
	0xfb840108,0x9b6a0101,0xfb840108,0x9b6b0101,0xfb840108,0x9b6c0101,0xfb840108,0x9b6d0101,
	0xfb840108,0x9b6e0101,0xfb840108,0x9b6f0101,0xfb840108,0x9b700101,0xfb840108,0x9b710101,
	0xfb840108,0x9b720101,0xfb840108,0x9b730101,0xfb840108,0x9b740101,0xfb840108,0x9b750101,
	0xfb840108,0x9b760101,0xfb840108,0x9b770101,0xfb840108,0x9b780101,0xfb840108,0x9b790101,
	0xfb840108,0x9b7a0101,0xfb840108,0x9b7b0101,0xfb840108,0x9b7c0101,0xfb840108,0x9b7d0101,
	0xfb840108,0x9b7e0101,0xfb840108,0x9b7f0101,0xfb840108,0x9b800101,0xfb840108,0x9b810101,
	0xfb840108,0x9b820101,0xfb840108,0x9b830101,0xfb840108,0x9b840101,0xfb840108,0x9b850101,
	0xfb840108,0x9b860101,0xfb840108,0x9b870101,0xfb840108,0x9b880101,0xfb840108,0x9b890101,
	0xfb840108,0x9b8a0101,0xfb840108,0x9b8b0101,0xfb840108,0x9b8c0101,0xfb840108,0x9b8d0101,
	0xfb840108,0x9b8e0101,0xfb840108,0x9b8f0101,0xfb840108,0x9b900101,0xfb840108,0x9b910101,
	0xfb840108,0x9b920101,0xfb840108,0x9b930101,0xfb840108,0x9b940101,0xfb840108,0x9b950101,
	0xfb840108,0x9b960101,0xfb840108,0x9b970101,0xfb840108,0x9b980101,0xfb840108,0x9b990101,
	0xfb840108,0x9b9a0101,0xfb840108,0x9b9b0101,0xfb840108,0x9b9c0101,0xfb840108,0x9b9d0101,
	0xfb840108,0x9b9e0101,0xfb840108,0x9b9f0101,0xfb840108,0x9ba00101,0xfb840108,0x9ba10101,
	0xfb840108,0x9ba20101,0xfb840108,0x9ba30101,0xfb840108,0x9ba40101,0xfb840108,0x9ba50101,
	0xfb840108,0x9ba60101,0xfb840108,0x9ba70101,0xfb840108,0x9ba80101,0xfb840108,0x9ba90101,
	0xfb840108,0x9baa0101,0xfb840108,0x9bab0101,0xfb840108,0x9bac0101,0xfb840108,0x9bad0101,
	0xfb840108,0x9bae0101,0xfb840108,0x9baf0101,0xfb840108,0x9bb00101,0xfb840108,0x9bb10101,
	0xfb840108,0x9bb20101,0xfb840108,0x9bb30101,0xfb840108,0x9bb40101,0xfb840108,0x9bb50101,
	0xfb840108,0x9bb60101,0xfb840108,0x9bb70101,0xfb840108,0x9bb80101,0xfb840108,0x9bb90101,
	0xfb840108,0x9bba0101,0xfb840108,0x9bbb0101,0xfb840108,0x9bbc0101,0xfb840108,0x9bbd0101,
	0xfb840108,0x9bbe0101,0xfb840108,0x9bbf0101,0xfb840108,0x9bc00101,0xfb840108,0x9bc10101,
	0xfb840108,0x9bc20101,0xfb840108,0x9bc30101,0xfb840108,0x9bc40101,0xfb840108,0x9bc50101,
	0xfb840108,0x9bc60101,0xfb840108,0x9bc70101,0xfb840108,0x9bc80101,0xfb840108,0x9bc90101,
	0xfb840108,0x9bca0101,0xfb840108,0x9bcb0101,0xfb840108,0x9bcc0101,0xfb840108,0x9bcd0101,
	0xfb840108,0x9bce0101,0xfb840108,0x9bcf0101,0xfb840108,0x9bd00101,0xfb840108,0x9bd10101,
	0xfb840108,0x9bd20101,0xfb840108,0x9bd30101,0xfb840108,0x9bd40101,0xfb840108,0x9bd50101,
	0xfb840108,0x9bd60101,0xfb840108,0x9bd70101,0xfb840108,0x9bd80101,0xfb840108,0x9bd90101,
	0xfb840108,0x9bda0101,0xfb840108,0x9bdb0101,0xfb840108,0x9bdc0101,0xfb840108,0x9bdd0101,
	0xfb840108,0x9bde0101,0xfb840108,0x9bdf0101,0xfb840108,0x9be00101,0xfb840108,0x9be10101,
	0xfb840108,0x9be20101,0xfb840108,0x9be30101,0xfb840108,0x9be40101,0xfb840108,0x9be50101,
	0xfb840108,0x9be60101,0xfb840108,0x9be70101,0xfb840108,0x9be80101,0xfb840108,0x9be90101,
	0xfb840108,0x9bea0101,0xfb840108,0x9beb0101,0xfb840108,0x9bec0101,0xfb840108,0x9bed0101,
	0xfb840108,0x9bee0101,0xfb840108,0x9bef0101,0xfb840108,0x9bf00101,0xfb840108,0x9bf10101,
	0xfb840108,0x9bf20101,0xfb840108,0x9bf30101,0xfb840108,0x9bf40101,0xfb840108,0x9bf50101,
	0xfb840108,0x9bf60101,0xfb840108,0x9bf70101,0xfb840108,0x9bf80101,0xfb840108,0x9bf90101,
	0xfb840108,0x9bfa0101,0xfb840108,0x9bfb0101,0xfb840108,0x9bfc0101,0xfb840108,0x9bfd0101,
	0xfb840108,0x9bfe0101,0xfb840108,0x9bff0101,0xfb840108,0x9c000101,0xfb840108,0x9c010101,
	0xfb840108,0x9c020101,0xfb840108,0x9c030101,0xfb840108,0x9c040101,0xfb840108,0x9c050101,
	0xfb840108,0x9c060101,0xfb840108,0x9c070101,0xfb840108,0x9c080101,0xfb840108,0x9c090101,
	0xfb840108,0x9c0a0101,0xfb840108,0x9c0b0101,0xfb840108,0x9c0c0101,0xfb840108,0x9c0d0101,
	0xfb840108,0x9c0e0101,0xfb840108,0x9c0f0101,0xfb840108,0x9c100101,0xfb840108,0x9c110101,
	0xfb840108,0x9c120101,0xfb840108,0x9c130101,0xfb840108,0x9c140101,0xfb840108,0x9c150101,
	0xfb840108,0x9c160101,0xfb840108,0x9c170101,0xfb840108,0x9c180101,0xfb840108,0x9c190101,
	0xfb840108,0x9c1a0101,0xfb840108,0x9c1b0101,0xfb840108,0x9c1c0101,0xfb840108,0x9c1d0101,
	0xfb840108,0x9c1e0101,0xfb840108,0x9c1f0101,0xfb840108,0x9c200101,0xfb840108,0x9c210101,
	0xfb840108,0x9c220101,0xfb840108,0x9c230101,0xfb840108,0x9c240101,0xfb840108,0x9c250101,
	0xfb840108,0x9c260101,0xfb840108,0x9c270101,0xfb840108,0x9c280101,0xfb840108,0x9c290101,
	0xfb840108,0x9c2a0101,0xfb840108,0x9c2b0101,0xfb840108,0x9c2c0101,0xfb840108,0x9c2d0101,
	0xfb840108,0x9c2e0101,0xfb840108,0x9c2f0101,0xfb840108,0x9c300101,0xfb840108,0x9c310101,
	0xfb840108,0x9c320101,0xfb840108,0x9c330101,0xfb840108,0x9c340101,0xfb840108,0x9c350101,
	0xfb840108,0x9c360101,0xfb840108,0x9c370101,0xfb840108,0x9c380101,0xfb840108,0x9c390101,
	0xfb840108,0x9c3a0101,0xfb840108,0x9c3b0101,0xfb840108,0x9c3c0101,0xfb840108,0x9c3d0101,
	0xfb840108,0x9c3e0101,0xfb840108,0x9c3f0101,0xfb840108,0x9c400101,0xfb840108,0x9c410101,
	0xfb840108,0x9c420101,0xfb840108,0x9c430101,0xfb840108,0x9c440101,0xfb840108,0x9c450101,
	0xfb840108,0x9c460101,0xfb840108,0x9c470101,0xfb840108,0x9c480101,0xfb840108,0x9c490101,
	0xfb840108,0x9c4a0101,0xfb840108,0x9c4b0101,0xfb840108,0x9c4c0101,0xfb840108,0x9c4d0101,
	0xfb840108,0x9c4e0101,0xfb840108,0x9c4f0101,0xfb840108,0x9c500101,0xfb840108,0x9c510101,
	0xfb840108,0x9c520101,0xfb840108,0x9c530101,0xfb840108,0x9c540101,0xfb840108,0x9c550101,
	0xfb840108,0x9c560101,0xfb840108,0x9c570101,0xfb840108,0x9c580101,0xfb840108,0x9c590101,
	0xfb840108,0x9c5a0101,0xfb840108,0x9c5b0101,0xfb840108,0x9c5c0101,0xfb840108,0x9c5d0101,
	0xfb840108,0x9c5e0101,0xfb840108,0x9c5f0101,0xfb840108,0x9c600101,0xfb840108,0x9c610101,
	0xfb840108,0x9c620101,0xfb840108,0x9c630101,0xfb840108,0x9c640101,0xfb840108,0x9c650101,
	0xfb840108,0x9c660101,0xfb840108,0x9c670101,0xfb840108,0x9c680101,0xfb840108,0x9c690101,
	0xfb840108,0x9c6a0101,0xfb840108,0x9c6b0101,0xfb840108,0x9c6c0101,0xfb840108,0x9c6d0101,
	0xfb840108,0x9c6e0101,0xfb840108,0x9c6f0101,0xfb840108,0x9c700101,0xfb840108,0x9c710101,
	0xfb840108,0x9c720101,0xfb840108,0x9c730101,0xfb840108,0x9c740101,0xfb840108,0x9c750101,
	0xfb840108,0x9c760101,0xfb840108,0x9c770101,0xfb840108,0x9c780101,0xfb840108,0x9c790101,
	0xfb840108,0x9c7a0101,0xfb840108,0x9c7b0101,0xfb840108,0x9c7c0101,0xfb840108,0x9c7d0101,
	0xfb840108,0x9c7e0101,0xfb840108,0x9c7f0101,0xfb840108,0x9c800101,0xfb840108,0x9c810101,
	0xfb840108,0x9c820101,0xfb840108,0x9c830101,0xfb840108,0x9c840101,0xfb840108,0x9c850101,
	0xfb840108,0x9c860101,0xfb840108,0x9c870101,0xfb840108,0x9c880101,0xfb840108,0x9c890101,
	0xfb840108,0x9c8a0101,0xfb840108,0x9c8b0101,0xfb840108,0x9c8c0101,0xfb840108,0x9c8d0101,
	0xfb840108,0x9c8e0101,0xfb840108,0x9c8f0101,0xfb840108,0x9c900101,0xfb840108,0x9c910101,
	0xfb840108,0x9c920101,0xfb840108,0x9c930101,0xfb840108,0x9c940101,0xfb840108,0x9c950101,
	0xfb840108,0x9c960101,0xfb840108,0x9c970101,0xfb840108,0x9c980101,0xfb840108,0x9c990101,
	0xfb840108,0x9c9a0101,0xfb840108,0x9c9b0101,0xfb840108,0x9c9c0101,0xfb840108,0x9c9d0101,
	0xfb840108,0x9c9e0101,0xfb840108,0x9c9f0101,0xfb840108,0x9ca00101,0xfb840108,0x9ca10101,
	0xfb840108,0x9ca20101,0xfb840108,0x9ca30101,0xfb840108,0x9ca40101,0xfb840108,0x9ca50101,
	0xfb840108,0x9ca60101,0xfb840108,0x9ca70101,0xfb840108,0x9ca80101,0xfb840108,0x9ca90101,
	0xfb840108,0x9caa0101,0xfb840108,0x9cab0101,0xfb840108,0x9cac0101,0xfb840108,0x9cad0101,
	0xfb840108,0x9cae0101,0xfb840108,0x9caf0101,0xfb840108,0x9cb00101,0xfb840108,0x9cb10101,
	0xfb840108,0x9cb20101,0xfb840108,0x9cb30101,0xfb840108,0x9cb40101,0xfb840108,0x9cb50101,
	0xfb840108,0x9cb60101,0xfb840108,0x9cb70101,0xfb840108,0x9cb80101,0xfb840108,0x9cb90101,
	0xfb840108,0x9cba0101,0xfb840108,0x9cbb0101,0xfb840108,0x9cbc0101,0xfb840108,0x9cbd0101,
	0xfb840108,0x9cbe0101,0xfb840108,0x9cbf0101,0xfb840108,0x9cc00101,0xfb840108,0x9cc10101,
	0xfb840108,0x9cc20101,0xfb840108,0x9cc30101,0xfb840108,0x9cc40101,0xfb840108,0x9cc50101,
	0xfb840108,0x9cc60101,0xfb840108,0x9cc70101,0xfb840108,0x9cc80101,0xfb840108,0x9cc90101,
	0xfb840108,0x9cca0101,0xfb840108,0x9ccb0101,0xfb840108,0x9ccc0101,0xfb840108,0x9ccd0101,
	0xfb840108,0x9cce0101,0xfb840108,0x9ccf0101,0xfb840108,0x9cd00101,0xfb840108,0x9cd10101,
	0xfb840108,0x9cd20101,0xfb840108,0x9cd30101,0xfb840108,0x9cd40101,0xfb840108,0x9cd50101,
	0xfb840108,0x9cd60101,0xfb840108,0x9cd70101,0xfb840108,0x9cd80101,0xfb840108,0x9cd90101,
	0xfb840108,0x9cda0101,0xfb840108,0x9cdb0101,0xfb840108,0x9cdc0101,0xfb840108,0x9cdd0101,
	0xfb840108,0x9cde0101,0xfb840108,0x9cdf0101,0xfb840108,0x9ce00101,0xfb840108,0x9ce10101,
	0xfb840108,0x9ce20101,0xfb840108,0x9ce30101,0xfb840108,0x9ce40101,0xfb840108,0x9ce50101,
	0xfb840108,0x9ce60101,0xfb840108,0x9ce70101,0xfb840108,0x9ce80101,0xfb840108,0x9ce90101,
	0xfb840108,0x9cea0101,0xfb840108,0x9ceb0101,0xfb840108,0x9cec0101,0xfb840108,0x9ced0101,
	0xfb840108,0x9cee0101,0xfb840108,0x9cef0101,0xfb840108,0x9cf00101,0xfb840108,0x9cf10101,
	0xfb840108,0x9cf20101,0xfb840108,0x9cf30101,0xfb840108,0x9cf40101,0xfb840108,0x9cf50101,
	0xfb840108,0x9cf60101,0xfb840108,0x9cf70101,0xfb840108,0x9cf80101,0xfb840108,0x9cf90101,
	0xfb840108,0x9cfa0101,0xfb840108,0x9cfb0101,0xfb840108,0x9cfc0101,0xfb840108,0x9cfd0101,
	0xfb840108,0x9cfe0101,0xfb840108,0x9cff0101,0xfb840108,0x9d000101,0xfb840108,0x9d010101,
	0xfb840108,0x9d020101,0xfb840108,0x9d030101,0xfb840108,0x9d040101,0xfb840108,0x9d050101,
	0xfb840108,0x9d060101,0xfb840108,0x9d070101,0xfb840108,0x9d080101,0xfb840108,0x9d090101,
	0xfb840108,0x9d0a0101,0xfb840108,0x9d0b0101,0xfb840108,0x9d0c0101,0xfb840108,0x9d0d0101,
	0xfb840108,0x9d0e0101,0xfb840108,0x9d0f0101,0xfb840108,0x9d100101,0xfb840108,0x9d110101,
	0xfb840108,0x9d120101,0xfb840108,0x9d130101,0xfb840108,0x9d140101,0xfb840108,0x9d150101,
	0xfb840108,0x9d160101,0xfb840108,0x9d170101,0xfb840108,0x9d180101,0xfb840108,0x9d190101,
	0xfb840108,0x9d1a0101,0xfb840108,0x9d1b0101,0xfb840108,0x9d1c0101,0xfb840108,0x9d1d0101,
	0xfb840108,0x9d1e0101,0xfb840108,0x9d1f0101,0xfb840108,0x9d200101,0xfb840108,0x9d210101,
	0xfb840108,0x9d220101,0xfb840108,0x9d230101,0xfb840108,0x9d240101,0xfb840108,0x9d250101,
	0xfb840108,0x9d260101,0xfb840108,0x9d270101,0xfb840108,0x9d280101,0xfb840108,0x9d290101,
	0xfb840108,0x9d2a0101,0xfb840108,0x9d2b0101,0xfb840108,0x9d2c0101,0xfb840108,0x9d2d0101,
	0xfb840108,0x9d2e0101,0xfb840108,0x9d2f0101,0xfb840108,0x9d300101,0xfb840108,0x9d310101,
	0xfb840108,0x9d320101,0xfb840108,0x9d330101,0xfb840108,0x9d340101,0xfb840108,0x9d350101,
	0xfb840108,0x9d360101,0xfb840108,0x9d370101,0xfb840108,0x9d380101,0xfb840108,0x9d390101,
	0xfb840108,0x9d3a0101,0xfb840108,0x9d3b0101,0xfb840108,0x9d3c0101,0xfb840108,0x9d3d0101,
	0xfb840108,0x9d3e0101,0xfb840108,0x9d3f0101,0xfb840108,0x9d400101,0xfb840108,0x9d410101,
	0xfb840108,0x9d420101,0xfb840108,0x9d430101,0xfb840108,0x9d440101,0xfb840108,0x9d450101,
	0xfb840108,0x9d460101,0xfb840108,0x9d470101,0xfb840108,0x9d480101,0xfb840108,0x9d490101,
	0xfb840108,0x9d4a0101,0xfb840108,0x9d4b0101,0xfb840108,0x9d4c0101,0xfb840108,0x9d4d0101,
	0xfb840108,0x9d4e0101,0xfb840108,0x9d4f0101,0xfb840108,0x9d500101,0xfb840108,0x9d510101,
	0xfb840108,0x9d520101,0xfb840108,0x9d530101,0xfb840108,0x9d540101,0xfb840108,0x9d550101,
	0xfb840108,0x9d560101,0xfb840108,0x9d570101,0xfb840108,0x9d580101,0xfb840108,0x9d590101,
	0xfb840108,0x9d5a0101,0xfb840108,0x9d5b0101,0xfb840108,0x9d5c0101,0xfb840108,0x9d5d0101,
	0xfb840108,0x9d5e0101,0xfb840108,0x9d5f0101,0xfb840108,0x9d600101,0xfb840108,0x9d610101,
	0xfb840108,0x9d620101,0xfb840108,0x9d630101,0xfb840108,0x9d640101,0xfb840108,0x9d650101,
	0xfb840108,0x9d660101,0xfb840108,0x9d670101,0xfb840108,0x9d680101,0xfb840108,0x9d690101,
	0xfb840108,0x9d6a0101,0xfb840108,0x9d6b0101,0xfb840108,0x9d6c0101,0xfb840108,0x9d6d0101,
	0xfb840108,0x9d6e0101,0xfb840108,0x9d6f0101,0xfb840108,0x9d700101,0xfb840108,0x9d710101,
	0xfb840108,0x9d720101,0xfb840108,0x9d730101,0xfb840108,0x9d740101,0xfb840108,0x9d750101,
	0xfb840108,0x9d760101,0xfb840108,0x9d770101,0xfb840108,0x9d780101,0xfb840108,0x9d790101,
	0xfb840108,0x9d7a0101,0xfb840108,0x9d7b0101,0xfb840108,0x9d7c0101,0xfb840108,0x9d7d0101,
	0xfb840108,0x9d7e0101,0xfb840108,0x9d7f0101,0xfb840108,0x9d800101,0xfb840108,0x9d810101,
	0xfb840108,0x9d820101,0xfb840108,0x9d830101,0xfb840108,0x9d840101,0xfb840108,0x9d850101,
	0xfb840108,0x9d860101,0xfb840108,0x9d870101,0xfb840108,0x9d880101,0xfb840108,0x9d890101,
	0xfb840108,0x9d8a0101,0xfb840108,0x9d8b0101,0xfb840108,0x9d8c0101,0xfb840108,0x9d8d0101,
	0xfb840108,0x9d8e0101,0xfb840108,0x9d8f0101,0xfb840108,0x9d900101,0xfb840108,0x9d910101,
	0xfb840108,0x9d920101,0xfb840108,0x9d930101,0xfb840108,0x9d940101,0xfb840108,0x9d950101,
	0xfb840108,0x9d960101,0xfb840108,0x9d970101,0xfb840108,0x9d980101,0xfb840108,0x9d990101,
	0xfb840108,0x9d9a0101,0xfb840108,0x9d9b0101,0xfb840108,0x9d9c0101,0xfb840108,0x9d9d0101,
	0xfb840108,0x9d9e0101,0xfb840108,0x9d9f0101,0xfb840108,0x9da00101,0xfb840108,0x9da10101,
	0xfb840108,0x9da20101,0xfb840108,0x9da30101,0xfb840108,0x9da40101,0xfb840108,0x9da50101,
	0xfb840108,0x9da60101,0xfb840108,0x9da70101,0xfb840108,0x9da80101,0xfb840108,0x9da90101,
	0xfb840108,0x9daa0101,0xfb840108,0x9dab0101,0xfb840108,0x9dac0101,0xfb840108,0x9dad0101,
	0xfb840108,0x9dae0101,0xfb840108,0x9daf0101,0xfb840108,0x9db00101,0xfb840108,0x9db10101,
	0xfb840108,0x9db20101,0xfb840108,0x9db30101,0xfb840108,0x9db40101,0xfb840108,0x9db50101,
	0xfb840108,0x9db60101,0xfb840108,0x9db70101,0xfb840108,0x9db80101,0xfb840108,0x9db90101,
	0xfb840108,0x9dba0101,0xfb840108,0x9dbb0101,0xfb840108,0x9dbc0101,0xfb840108,0x9dbd0101,
	0xfb840108,0x9dbe0101,0xfb840108,0x9dbf0101,0xfb840108,0x9dc00101,0xfb840108,0x9dc10101,
	0xfb840108,0x9dc20101,0xfb840108,0x9dc30101,0xfb840108,0x9dc40101,0xfb840108,0x9dc50101,
	0xfb840108,0x9dc60101,0xfb840108,0x9dc70101,0xfb840108,0x9dc80101,0xfb840108,0x9dc90101,
	0xfb840108,0x9dca0101,0xfb840108,0x9dcb0101,0xfb840108,0x9dcc0101,0xfb840108,0x9dcd0101,
	0xfb840108,0x9dce0101,0xfb840108,0x9dcf0101,0xfb840108,0x9dd00101,0xfb840108,0x9dd10101,
	0xfb840108,0x9dd20101,0xfb840108,0x9dd30101,0xfb840108,0x9dd40101,0xfb840108,0x9dd50101,
	0xfb840108,0x9dd60101,0xfb840108,0x9dd70101,0xfb840108,0x9dd80101,0xfb840108,0x9dd90101,
	0xfb840108,0x9dda0101,0xfb840108,0x9ddb0101,0xfb840108,0x9ddc0101,0xfb840108,0x9ddd0101,
	0xfb840108,0x9dde0101,0xfb840108,0x9ddf0101,0xfb840108,0x9de00101,0xfb840108,0x9de10101,
	0xfb840108,0x9de20101,0xfb840108,0x9de30101,0xfb840108,0x9de40101,0xfb840108,0x9de50101,
	0xfb840108,0x9de60101,0xfb840108,0x9de70101,0xfb840108,0x9de80101,0xfb840108,0x9de90101,
	0xfb840108,0x9dea0101,0xfb840108,0x9deb0101,0xfb840108,0x9dec0101,0xfb840108,0x9ded0101,
	0xfb840108,0x9dee0101,0xfb840108,0x9def0101,0xfb840108,0x9df00101,0xfb840108,0x9df10101,
	0xfb840108,0x9df20101,0xfb840108,0x9df30101,0xfb840108,0x9df40101,0xfb840108,0x9df50101,
	0xfb840108,0x9df60101,0xfb840108,0x9df70101,0xfb840108,0x9df80101,0xfb840108,0x9df90101,
	0xfb840108,0x9dfa0101,0xfb840108,0x9dfb0101,0xfb840108,0x9dfc0101,0xfb840108,0x9dfd0101,
	0xfb840108,0x9dfe0101,0xfb840108,0x9dff0101,0xfb840108,0x9e000101,0xfb840108,0x9e010101,
	0xfb840108,0x9e020101,0xfb840108,0x9e030101,0xfb840108,0x9e040101,0xfb840108,0x9e050101,
	0xfb840108,0x9e060101,0xfb840108,0x9e070101,0xfb840108,0x9e080101,0xfb840108,0x9e090101,
	0xfb840108,0x9e0a0101,0xfb840108,0x9e0b0101,0xfb840108,0x9e0c0101,0xfb840108,0x9e0d0101,
	0xfb840108,0x9e0e0101,0xfb840108,0x9e0f0101,0xfb840108,0x9e100101,0xfb840108,0x9e110101,
	0xfb840108,0x9e120101,0xfb840108,0x9e130101,0xfb840108,0x9e140101,0xfb840108,0x9e150101,
	0xfb840108,0x9e160101,0xfb840108,0x9e170101,0xfb840108,0x9e180101,0xfb840108,0x9e190101,
	0xfb840108,0x9e1a0101,0xfb840108,0x9e1b0101,0xfb840108,0x9e1c0101,0xfb840108,0x9e1d0101,
	0xfb840108,0x9e1e0101,0xfb840108,0x9e1f0101,0xfb840108,0x9e200101,0xfb840108,0x9e210101,
	0xfb840108,0x9e220101,0xfb840108,0x9e230101,0xfb840108,0x9e240101,0xfb840108,0x9e250101,
	0xfb840108,0x9e260101,0xfb840108,0x9e270101,0xfb840108,0x9e280101,0xfb840108,0x9e290101,
	0xfb840108,0x9e2a0101,0xfb840108,0x9e2b0101,0xfb840108,0x9e2c0101,0xfb840108,0x9e2d0101,
	0xfb840108,0x9e2e0101,0xfb840108,0x9e2f0101,0xfb840108,0x9e300101,0xfb840108,0x9e310101,
	0xfb840108,0x9e320101,0xfb840108,0x9e330101,0xfb840108,0x9e340101,0xfb840108,0x9e350101,
	0xfb840108,0x9e360101,0xfb840108,0x9e370101,0xfb840108,0x9e380101,0xfb840108,0x9e390101,
	0xfb840108,0x9e3a0101,0xfb840108,0x9e3b0101,0xfb840108,0x9e3c0101,0xfb840108,0x9e3d0101,
	0xfb840108,0x9e3e0101,0xfb840108,0x9e3f0101,0xfb840108,0x9e400101,0xfb840108,0x9e410101,
	0xfb840108,0x9e420101,0xfb840108,0x9e430101,0xfb840108,0x9e440101,0xfb840108,0x9e450101,
	0xfb840108,0x9e460101,0xfb840108,0x9e470101,0xfb840108,0x9e480101,0xfb840108,0x9e490101,
	0xfb840108,0x9e4a0101,0xfb840108,0x9e4b0101,0xfb840108,0x9e4c0101,0xfb840108,0x9e4d0101,
	0xfb840108,0x9e4e0101,0xfb840108,0x9e4f0101,0xfb840108,0x9e500101,0xfb840108,0x9e510101,
	0xfb840108,0x9e520101,0xfb840108,0x9e530101,0xfb840108,0x9e540101,0xfb840108,0x9e550101,
	0xfb840108,0x9e560101,0xfb840108,0x9e570101,0xfb840108,0x9e580101,0xfb840108,0x9e590101,
	0xfb840108,0x9e5a0101,0xfb840108,0x9e5b0101,0xfb840108,0x9e5c0101,0xfb840108,0x9e5d0101,
	0xfb840108,0x9e5e0101,0xfb840108,0x9e5f0101,0xfb840108,0x9e600101,0xfb840108,0x9e610101,
	0xfb840108,0x9e620101,0xfb840108,0x9e630101,0xfb840108,0x9e640101,0xfb840108,0x9e650101,
	0xfb840108,0x9e660101,0xfb840108,0x9e670101,0xfb840108,0x9e680101,0xfb840108,0x9e690101,
	0xfb840108,0x9e6a0101,0xfb840108,0x9e6b0101,0xfb840108,0x9e6c0101,0xfb840108,0x9e6d0101,
	0xfb840108,0x9e6e0101,0xfb840108,0x9e6f0101,0xfb840108,0x9e700101,0xfb840108,0x9e710101,
	0xfb840108,0x9e720101,0xfb840108,0x9e730101,0xfb840108,0x9e740101,0xfb840108,0x9e750101,
	0xfb840108,0x9e760101,0xfb840108,0x9e770101,0xfb840108,0x9e780101,0xfb840108,0x9e790101,
	0xfb840108,0x9e7a0101,0xfb840108,0x9e7b0101,0xfb840108,0x9e7c0101,0xfb840108,0x9e7d0101,
	0xfb840108,0x9e7e0101,0xfb840108,0x9e7f0101,0xfb840108,0x9e800101,0xfb840108,0x9e810101,
	0xfb840108,0x9e820101,0xfb840108,0x9e830101,0xfb840108,0x9e840101,0xfb840108,0x9e850101,
	0xfb840108,0x9e860101,0xfb840108,0x9e870101,0xfb840108,0x9e880101,0xfb840108,0x9e890101,
	0xfb840108,0x9e8a0101,0xfb840108,0x9e8b0101,0xfb840108,0x9e8c0101,0xfb840108,0x9e8d0101,
	0xfb840108,0x9e8e0101,0xfb840108,0x9e8f0101,0xfb840108,0x9e900101,0xfb840108,0x9e910101,
	0xfb840108,0x9e920101,0xfb840108,0x9e930101,0xfb840108,0x9e940101,0xfb840108,0x9e950101,
	0xfb840108,0x9e960101,0xfb840108,0x9e970101,0xfb840108,0x9e980101,0xfb840108,0x9e990101,
	0xfb840108,0x9e9a0101,0xfb840108,0x9e9b0101,0xfb840108,0x9e9c0101,0xfb840108,0x9e9d0101,
	0xfb840108,0x9e9e0101,0xfb840108,0x9e9f0101,0xfb840108,0x9ea00101,0xfb840108,0x9ea10101,
	0xfb840108,0x9ea20101,0xfb840108,0x9ea30101,0xfb840108,0x9ea40101,0xfb840108,0x9ea50101,
	0xfb840108,0x9ea60101,0xfb840108,0x9ea70101,0xfb840108,0x9ea80101,0xfb840108,0x9ea90101,
	0xfb840108,0x9eaa0101,0xfb840108,0x9eab0101,0xfb840108,0x9eac0101,0xfb840108,0x9ead0101,
	0xfb840108,0x9eae0101,0xfb840108,0x9eaf0101,0xfb840108,0x9eb00101,0xfb840108,0x9eb10101,
	0xfb840108,0x9eb20101,0xfb840108,0x9eb30101,0xfb840108,0x9eb40101,0xfb840108,0x9eb50101,
	0xfb840108,0x9eb60101,0xfb840108,0x9eb70101,0xfb840108,0x9eb80101,0xfb840108,0x9eb90101,
	0xfb840108,0x9eba0101,0xfb840108,0x9ebb0101,0xfb840108,0x9ebc0101,0xfb840108,0x9ebd0101,
	0xfb840108,0x9ebe0101,0xfb840108,0x9ebf0101,0xfb840108,0x9ec00101,0xfb840108,0x9ec10101,
	0xfb840108,0x9ec20101,0xfb840108,0x9ec30101,0xfb840108,0x9ec40101,0xfb840108,0x9ec50101,
	0xfb840108,0x9ec60101,0xfb840108,0x9ec70101,0xfb840108,0x9ec80101,0xfb840108,0x9ec90101,
	0xfb840108,0x9eca0101,0xfb840108,0x9ecb0101,0xfb840108,0x9ecc0101,0xfb840108,0x9ecd0101,
	0xfb840108,0x9ece0101,0xfb840108,0x9ecf0101,0xfb840108,0x9ed00101,0xfb840108,0x9ed10101,
	0xfb840108,0x9ed20101,0xfb840108,0x9ed30101,0xfb840108,0x9ed40101,0xfb840108,0x9ed50101,
	0xfb840108,0x9ed60101,0xfb840108,0x9ed70101,0xfb840108,0x9ed80101,0xfb840108,0x9ed90101,
	0xfb840108,0x9eda0101,0xfb840108,0x9edb0101,0xfb840108,0x9edc0101,0xfb840108,0x9edd0101,
	0xfb840108,0x9ede0101,0xfb840108,0x9edf0101,0xfb840108,0x9ee00101,0xfb840108,0x9ee10101,
	0xfb840108,0x9ee20101,0xfb840108,0x9ee30101,0xfb840108,0x9ee40101,0xfb840108,0x9ee50101,
	0xfb840108,0x9ee60101,0xfb840108,0x9ee70101,0xfb840108,0x9ee80101,0xfb840108,0x9ee90101,
	0xfb840108,0x9eea0101,0xfb840108,0x9eeb0101,0xfb840108,0x9eec0101,0xfb840108,0x9eed0101,
	0xfb840108,0x9eee0101,0xfb840108,0x9eef0101,0xfb840108,0x9ef00101,0xfb840108,0x9ef10101,
	0xfb840108,0x9ef20101,0xfb840108,0x9ef30101,0xfb840108,0x9ef40101,0xfb840108,0x9ef50101,
	0xfb840108,0x9ef60101,0xfb840108,0x9ef70101,0xfb840108,0x9ef80101,0xfb840108,0x9ef90101,
	0xfb840108,0x9efa0101,0xfb840108,0x9efb0101,0xfb840108,0x9efc0101,0xfb840108,0x9efd0101,
	0xfb840108,0x9efe0101,0xfb840108,0x9eff0101,0xfb840108,0x9f000101,0xfb840108,0x9f010101,
	0xfb840108,0x9f020101,0xfb840108,0x9f030101,0xfb840108,0x9f040101,0xfb840108,0x9f050101,
	0xfb840108,0x9f060101,0xfb840108,0x9f070101,0xfb840108,0x9f080101,0xfb840108,0x9f090101,
	0xfb840108,0x9f0a0101,0xfb840108,0x9f0b0101,0xfb840108,0x9f0c0101,0xfb840108,0x9f0d0101,
	0xfb840108,0x9f0e0101,0xfb840108,0x9f0f0101,0xfb840108,0x9f100101,0xfb840108,0x9f110101,
	0xfb840108,0x9f120101,0xfb840108,0x9f130101,0xfb840108,0x9f140101,0xfb840108,0x9f150101,
	0xfb840108,0x9f160101,0xfb840108,0x9f170101,0xfb840108,0x9f180101,0xfb840108,0x9f190101,
	0xfb840108,0x9f1a0101,0xfb840108,0x9f1b0101,0xfb840108,0x9f1c0101,0xfb840108,0x9f1d0101,
	0xfb840108,0x9f1e0101,0xfb840108,0x9f1f0101,0xfb840108,0x9f200101,0xfb840108,0x9f210101,
	0xfb840108,0x9f220101,0xfb840108,0x9f230101,0xfb840108,0x9f240101,0xfb840108,0x9f250101,
	0xfb840108,0x9f260101,0xfb840108,0x9f270101,0xfb840108,0x9f280101,0xfb840108,0x9f290101,
	0xfb840108,0x9f2a0101,0xfb840108,0x9f2b0101,0xfb840108,0x9f2c0101,0xfb840108,0x9f2d0101,
	0xfb840108,0x9f2e0101,0xfb840108,0x9f2f0101,0xfb840108,0x9f300101,0xfb840108,0x9f310101,
	0xfb840108,0x9f320101,0xfb840108,0x9f330101,0xfb840108,0x9f340101,0xfb840108,0x9f350101,
	0xfb840108,0x9f360101,0xfb840108,0x9f370101,0xfb840108,0x9f380101,0xfb840108,0x9f390101,
	0xfb840108,0x9f3a0101,0xfb840108,0x9f3b0101,0xfb840108,0x9f3c0101,0xfb840108,0x9f3d0101,
	0xfb840108,0x9f3e0101,0xfb840108,0x9f3f0101,0xfb840108,0x9f400101,0xfb840108,0x9f410101,
	0xfb840108,0x9f420101,0xfb840108,0x9f430101,0xfb840108,0x9f440101,0xfb840108,0x9f450101,
	0xfb840108,0x9f460101,0xfb840108,0x9f470101,0xfb840108,0x9f480101,0xfb840108,0x9f490101,
	0xfb840108,0x9f4a0101,0xfb840108,0x9f4b0101,0xfb840108,0x9f4c0101,0xfb840108,0x9f4d0101,
	0xfb840108,0x9f4e0101,0xfb840108,0x9f4f0101,0xfb840108,0x9f500101,0xfb840108,0x9f510101,
	0xfb840108,0x9f520101,0xfb840108,0x9f530101,0xfb840108,0x9f540101,0xfb840108,0x9f550101,
	0xfb840108,0x9f560101,0xfb840108,0x9f570101,0xfb840108,0x9f580101,0xfb840108,0x9f590101,
	0xfb840108,0x9f5a0101,0xfb840108,0x9f5b0101,0xfb840108,0x9f5c0101,0xfb840108,0x9f5d0101,
	0xfb840108,0x9f5e0101,0xfb840108,0x9f5f0101,0xfb840108,0x9f600101,0xfb840108,0x9f610101,
	0xfb840108,0x9f620101,0xfb840108,0x9f630101,0xfb840108,0x9f640101,0xfb840108,0x9f650101,
	0xfb840108,0x9f660101,0xfb840108,0x9f670101,0xfb840108,0x9f680101,0xfb840108,0x9f690101,
	0xfb840108,0x9f6a0101,0xfb840108,0x9f6b0101,0xfb840108,0x9f6c0101,0xfb840108,0x9f6d0101,
	0xfb840108,0x9f6e0101,0xfb840108,0x9f6f0101,0xfb840108,0x9f700101,0xfb840108,0x9f710101,
	0xfb840108,0x9f720101,0xfb840108,0x9f730101,0xfb840108,0x9f740101,0xfb840108,0x9f750101,
	0xfb840108,0x9f760101,0xfb840108,0x9f770101,0xfb840108,0x9f780101,0xfb840108,0x9f790101,
	0xfb840108,0x9f7a0101,0xfb840108,0x9f7b0101,0xfb840108,0x9f7c0101,0xfb840108,0x9f7d0101,
	0xfb840108,0x9f7e0101,0xfb840108,0x9f7f0101,0xfb840108,0x9f800101,0xfb840108,0x9f810101,
	0xfb840108,0x9f820101,0xfb840108,0x9f830101,0xfb840108,0x9f840101,0xfb840108,0x9f850101,
	0xfb840108,0x9f860101,0xfb840108,0x9f870101,0xfb840108,0x9f880101,0xfb840108,0x9f890101,
	0xfb840108,0x9f8a0101,0xfb840108,0x9f8b0101,0xfb840108,0x9f8c0101,0xfb840108,0x9f8d0101,
	0xfb840108,0x9f8e0101,0xfb840108,0x9f8f0101,0xfb840108,0x9f900101,0xfb840108,0x9f910101,
	0xfb840108,0x9f920101,0xfb840108,0x9f930101,0xfb840108,0x9f940101,0xfb840108,0x9f950101,
	0xfb840108,0x9f960101,0xfb840108,0x9f970101,0xfb840108,0x9f980101,0xfb840108,0x9f990101,
	0xfb840108,0x9f9a0101,0xfb840108,0x9f9b0101,0xfb840108,0x9f9c0101,0xfb840108,0x9f9d0101,
	0xfb840108,0x9f9e0101,0xfb840108,0x9f9f0101,0xfb840108,0x9fa00101,0xfb840108,0x9fa10101,
	0xfb840108,0x9fa20101,0xfb840108,0x9fa30101,0xfb840108,0x9fa40101,0xfb840108,0x9fa50101,
	0xfb840108,0x9fa60101,0xfb840108,0x9fa70101,0xfb840108,0x9fa80101,0xfb840108,0x9fa90101,
	0xfb840108,0x9faa0101,0xfb840108,0x9fab0101,0xfb840108,0x9fac0101,0xfb840108,0x9fad0101,
	0xfb840108,0x9fae0101,0xfb840108,0x9faf0101,0xfb840108,0x9fb00101,0xfb840108,0x9fb10101,
	0xfb840108,0x9fb20101,0xfb840108,0x9fb30101,0xfb840108,0x9fb40101,0xfb840108,0x9fb50101,
	0xfb840108,0x9fb60101,0xfb840108,0x9fb70101,0xfb840108,0x9fb80101,0xfb840108,0x9fb90101,
	0xfb840108,0x9fba0101,0xfb840108,0x9fbb0101,0xfb840108,0x9fbc0101,0xfb840108,0x9fbd0101,
	0xfb840108,0x9fbe0101,0xfb840108,0x9fbf0101,0xfb840108,0x9fc00101,0xfb840108,0x9fc10101,
	0xfb840108,0x9fc20101,0xfb840108,0x9fc30101,0xfb840108,0x9fc40101,0xfb840108,0x9fc50101,
	0xfb840108,0x9fc60101,0xfb840108,0x9fc70101,0xfb840108,0x9fc80101,0xfb840108,0x9fc90101,
	0xfb840108,0x9fca0101,0xfb840108,0x9fcb0101,0xfb840108,0x9fcc0101,0xfb840108,0x9fcd0101,
	0xfb840108,0x9fce0101,0xfb840108,0x9fcf0101,0xfb840108,0x9fd00101,0xfb840108,0x9fd10101,
	0xfb840108,0x9fd20101,0xfb840108,0x9fd30101,0xfb840108,0x9fd40101,0xfb840108,0x9fd50101,
	0xfb840108,0x9fd60101,0xfb840108,0x9fd70101,0xfb840108,0x9fd80101,0xfb840108,0x9fd90101,
	0xfb840108,0x9fda0101,0xfb840108,0x9fdb0101,0xfb840108,0x9fdc0101,0xfb840108,0x9fdd0101,
	0xfb840108,0x9fde0101,0xfb840108,0x9fdf0101,0xfb840108,0x9fe00101,0xfb840108,0x9fe10101,
	0xfb840108,0x9fe20101,0xfb840108,0x9fe30101,0xfb840108,0x9fe40101,0xfb840108,0x9fe50101,
	0xfb840108,0x9fe60101,0xfb840108,0x9fe70101,0xfb840108,0x9fe80101,0xfb840108,0x9fe90101,
	0xfb840108,0x9fea0101,0xfb840108,0x9feb0101,0xfb840108,0x9fec0101,0xfb840108,0x9fed0101,
	0xfb840108,0x9fee0101,0xfb840108,0x9fef0101,0xfb840108,0x9ff00101,0xfb840108,0x9ff10101,
	0xfb840108,0x9ff20101,0xfb840108,0x9ff30101,0xfb840108,0x9ff40101,0xfb840108,0x9ff50101,
	0xfb840108,0x9ff60101,0xfb840108,0x9ff70101,0xfb840108,0x9ff80101,0xfb840108,0x9ff90101,
	0xfb840108,0x9ffa0101,0xfb840108,0x9ffb0101,0xfb840108,0x9ffc0101,0xfb840108,0x9ffd0101,
	0xfb840108,0x9ffe0101,0xfb840108,0x9fff0101,0xfb840108,0xa0000101,0xfb840108,0xa0010101,
	0xfb840108,0xa0020101,0xfb840108,0xa0030101,0xfb840108,0xa0040101,0xfb840108,0xa0050101,
	0xfb840108,0xa0060101,0xfb840108,0xa0070101,0xfb840108,0xa0080101,0xfb840108,0xa0090101,
	0xfb840108,0xa00a0101,0xfb840108,0xa00b0101,0xfb840108,0xa00c0101,0xfb840108,0xa00d0101,
	0xfb840108,0xa00e0101,0xfb840108,0xa00f0101,0xfb840108,0xa0100101,0xfb840108,0xa0110101,
	0xfb840108,0xa0120101,0xfb840108,0xa0130101,0xfb840108,0xa0140101,0xfb840108,0xa0150101,
	0xfb840108,0xa0160101,0xfb840108,0xa0170101,0xfb840108,0xa0180101,0xfb840108,0xa0190101,
	0xfb840108,0xa01a0101,0xfb840108,0xa01b0101,0xfb840108,0xa01c0101,0xfb840108,0xa01d0101,
	0xfb840108,0xa01e0101,0xfb840108,0xa01f0101,0xfb840108,0xa0200101,0xfb840108,0xa0210101,
	0xfb840108,0xa0220101,0xfb840108,0xa0230101,0xfb840108,0xa0240101,0xfb840108,0xa0250101,
	0xfb840108,0xa0260101,0xfb840108,0xa0270101,0xfb840108,0xa0280101,0xfb840108,0xa0290101,
	0xfb840108,0xa02a0101,0xfb840108,0xa02b0101,0xfb840108,0xa02c0101,0xfb840108,0xa02d0101,
	0xfb840108,0xa02e0101,0xfb840108,0xa02f0101,0xfb840108,0xa0300101,0xfb840108,0xa0310101,
	0xfb840108,0xa0320101,0xfb840108,0xa0330101,0xfb840108,0xa0340101,0xfb840108,0xa0350101,
	0xfb840108,0xa0360101,0xfb840108,0xa0370101,0xfb840108,0xa0380101,0xfb840108,0xa0390101,
	0xfb840108,0xa03a0101,0xfb840108,0xa03b0101,0xfb840108,0xa03c0101,0xfb840108,0xa03d0101,
	0xfb840108,0xa03e0101,0xfb840108,0xa03f0101,0xfb840108,0xa0400101,0xfb840108,0xa0410101,
	0xfb840108,0xa0420101,0xfb840108,0xa0430101,0xfb840108,0xa0440101,0xfb840108,0xa0450101,
	0xfb840108,0xa0460101,0xfb840108,0xa0470101,0xfb840108,0xa0480101,0xfb840108,0xa0490101,
	0xfb840108,0xa04a0101,0xfb840108,0xa04b0101,0xfb840108,0xa04c0101,0xfb840108,0xa04d0101,
	0xfb840108,0xa04e0101,0xfb840108,0xa04f0101,0xfb840108,0xa0500101,0xfb840108,0xa0510101,
	0xfb840108,0xa0520101,0xfb840108,0xa0530101,0xfb840108,0xa0540101,0xfb840108,0xa0550101,
	0xfb840108,0xa0560101,0xfb840108,0xa0570101,0xfb840108,0xa0580101,0xfb840108,0xa0590101,
	0xfb840108,0xa05a0101,0xfb840108,0xa05b0101,0xfb840108,0xa05c0101,0xfb840108,0xa05d0101,
	0xfb840108,0xa05e0101,0xfb840108,0xa05f0101,0xfb840108,0xa0600101,0xfb840108,0xa0610101,
	0xfb840108,0xa0620101,0xfb840108,0xa0630101,0xfb840108,0xa0640101,0xfb840108,0xa0650101,
	0xfb840108,0xa0660101,0xfb840108,0xa0670101,0xfb840108,0xa0680101,0xfb840108,0xa0690101,
	0xfb840108,0xa06a0101,0xfb840108,0xa06b0101,0xfb840108,0xa06c0101,0xfb840108,0xa06d0101,
	0xfb840108,0xa06e0101,0xfb840108,0xa06f0101,0xfb840108,0xa0700101,0xfb840108,0xa0710101,
	0xfb840108,0xa0720101,0xfb840108,0xa0730101,0xfb840108,0xa0740101,0xfb840108,0xa0750101,
	0xfb840108,0xa0760101,0xfb840108,0xa0770101,0xfb840108,0xa0780101,0xfb840108,0xa0790101,
	0xfb840108,0xa07a0101,0xfb840108,0xa07b0101,0xfb840108,0xa07c0101,0xfb840108,0xa07d0101,
	0xfb840108,0xa07e0101,0xfb840108,0xa07f0101,0xfb840108,0xa0800101,0xfb840108,0xa0810101,
	0xfb840108,0xa0820101,0xfb840108,0xa0830101,0xfb840108,0xa0840101,0xfb840108,0xa0850101,
	0xfb840108,0xa0860101,0xfb840108,0xa0870101,0xfb840108,0xa0880101,0xfb840108,0xa0890101,
	0xfb840108,0xa08a0101,0xfb840108,0xa08b0101,0xfb840108,0xa08c0101,0xfb840108,0xa08d0101,
	0xfb840108,0xa08e0101,0xfb840108,0xa08f0101,0xfb840108,0xa0900101,0xfb840108,0xa0910101,
	0xfb840108,0xa0920101,0xfb840108,0xa0930101,0xfb840108,0xa0940101,0xfb840108,0xa0950101,
	0xfb840108,0xa0960101,0xfb840108,0xa0970101,0xfb840108,0xa0980101,0xfb840108,0xa0990101,
	0xfb840108,0xa09a0101,0xfb840108,0xa09b0101,0xfb840108,0xa09c0101,0xfb840108,0xa09d0101,
	0xfb840108,0xa09e0101,0xfb840108,0xa09f0101,0xfb840108,0xa0a00101,0xfb840108,0xa0a10101,
	0xfb840108,0xa0a20101,0xfb840108,0xa0a30101,0xfb840108,0xa0a40101,0xfb840108,0xa0a50101,
	0xfb840108,0xa0a60101,0xfb840108,0xa0a70101,0xfb840108,0xa0a80101,0xfb840108,0xa0a90101,
	0xfb840108,0xa0aa0101,0xfb840108,0xa0ab0101,0xfb840108,0xa0ac0101,0xfb840108,0xa0ad0101,
	0xfb840108,0xa0ae0101,0xfb840108,0xa0af0101,0xfb840108,0xa0b00101,0xfb840108,0xa0b10101,
	0xfb840108,0xa0b20101,0xfb840108,0xa0b30101,0xfb840108,0xa0b40101,0xfb840108,0xa0b50101,
	0xfb840108,0xa0b60101,0xfb840108,0xa0b70101,0xfb840108,0xa0b80101,0xfb840108,0xa0b90101,
	0xfb840108,0xa0ba0101,0xfb840108,0xa0bb0101,0xfb840108,0xa0bc0101,0xfb840108,0xa0bd0101,
	0xfb840108,0xa0be0101,0xfb840108,0xa0bf0101,0xfb840108,0xa0c00101,0xfb840108,0xa0c10101,
	0xfb840108,0xa0c20101,0xfb840108,0xa0c30101,0xfb840108,0xa0c40101,0xfb840108,0xa0c50101,
	0xfb840108,0xa0c60101,0xfb840108,0xa0c70101,0xfb840108,0xa0c80101,0xfb840108,0xa0c90101,
	0xfb840108,0xa0ca0101,0xfb840108,0xa0cb0101,0xfb840108,0xa0cc0101,0xfb840108,0xa0cd0101,
	0xfb840108,0xa0ce0101,0xfb840108,0xa0cf0101,0xfb840108,0xa0d00101,0xfb840108,0xa0d10101,
	0xfb840108,0xa0d20101,0xfb840108,0xa0d30101,0xfb840108,0xa0d40101,0xfb840108,0xa0d50101,
	0xfb840108,0xa0d60101,0xfb840108,0xa0d70101,0xfb840108,0xa0d80101,0xfb840108,0xa0d90101,
	0xfb840108,0xa0da0101,0xfb840108,0xa0db0101,0xfb840108,0xa0dc0101,0xfb840108,0xa0dd0101,
	0xfb840108,0xa0de0101,0xfb840108,0xa0df0101,0xfb840108,0xa0e00101,0xfb840108,0xa0e10101,
	0xfb840108,0xa0e20101,0xfb840108,0xa0e30101,0xfb840108,0xa0e40101,0xfb840108,0xa0e50101,
	0xfb840108,0xa0e60101,0xfb840108,0xa0e70101,0xfb840108,0xa0e80101,0xfb840108,0xa0e90101,
	0xfb840108,0xa0ea0101,0xfb840108,0xa0eb0101,0xfb840108,0xa0ec0101,0xfb840108,0xa0ed0101,
	0xfb840108,0xa0ee0101,0xfb840108,0xa0ef0101,0xfb840108,0xa0f00101,0xfb840108,0xa0f10101,
	0xfb840108,0xa0f20101,0xfb840108,0xa0f30101,0xfb840108,0xa0f40101,0xfb840108,0xa0f50101,
	0xfb840108,0xa0f60101,0xfb840108,0xa0f70101,0xfb840108,0xa0f80101,0xfb840108,0xa0f90101,
	0xfb840108,0xa0fa0101,0xfb840108,0xa0fb0101,0xfb840108,0xa0fc0101,0xfb840108,0xa0fd0101,
	0xfb840108,0xa0fe0101,0xfb840108,0xa0ff0101,0xfb840108,0xa1000101,0xfb840108,0xa1010101,
	0xfb840108,0xa1020101,0xfb840108,0xa1030101,0xfb840108,0xa1040101,0xfb840108,0xa1050101,
	0xfb840108,0xa1060101,0xfb840108,0xa1070101,0xfb840108,0xa1080101,0xfb840108,0xa1090101,
	0xfb840108,0xa10a0101,0xfb840108,0xa10b0101,0xfb840108,0xa10c0101,0xfb840108,0xa10d0101,
	0xfb840108,0xa10e0101,0xfb840108,0xa10f0101,0xfb840108,0xa1100101,0xfb840108,0xa1110101,
	0xfb840108,0xa1120101,0xfb840108,0xa1130101,0xfb840108,0xa1140101,0xfb840108,0xa1150101,
	0xfb840108,0xa1160101,0xfb840108,0xa1170101,0xfb840108,0xa1180101,0xfb840108,0xa1190101,
	0xfb840108,0xa11a0101,0xfb840108,0xa11b0101,0xfb840108,0xa11c0101,0xfb840108,0xa11d0101,
	0xfb840108,0xa11e0101,0xfb840108,0xa11f0101,0xfb840108,0xa1200101,0xfb840108,0xa1210101,
	0xfb840108,0xa1220101,0xfb840108,0xa1230101,0xfb840108,0xa1240101,0xfb840108,0xa1250101,
	0xfb840108,0xa1260101,0xfb840108,0xa1270101,0xfb840108,0xa1280101,0xfb840108,0xa1290101,
	0xfb840108,0xa12a0101,0xfb840108,0xa12b0101,0xfb840108,0xa12c0101,0xfb840108,0xa12d0101,
	0xfb840108,0xa12e0101,0xfb840108,0xa12f0101,0xfb840108,0xa1300101,0xfb840108,0xa1310101,
	0xfb840108,0xa1320101,0xfb840108,0xa1330101,0xfb840108,0xa1340101,0xfb840108,0xa1350101,
	0xfb840108,0xa1360101,0xfb840108,0xa1370101,0xfb840108,0xa1380101,0xfb840108,0xa1390101,
	0xfb840108,0xa13a0101,0xfb840108,0xa13b0101,0xfb840108,0xa13c0101,0xfb840108,0xa13d0101,
	0xfb840108,0xa13e0101,0xfb840108,0xa13f0101,0xfb840108,0xa1400101,0xfb840108,0xa1410101,
	0xfb840108,0xa1420101,0xfb840108,0xa1430101,0xfb840108,0xa1440101,0xfb840108,0xa1450101,
	0xfb840108,0xa1460101,0xfb840108,0xa1470101,0xfb840108,0xa1480101,0xfb840108,0xa1490101,
	0xfb840108,0xa14a0101,0xfb840108,0xa14b0101,0xfb840108,0xa14c0101,0xfb840108,0xa14d0101,
	0xfb840108,0xa14e0101,0xfb840108,0xa14f0101,0xfb840108,0xa1500101,0xfb840108,0xa1510101,
	0xfb840108,0xa1520101,0xfb840108,0xa1530101,0xfb840108,0xa1540101,0xfb840108,0xa1550101,
	0xfb840108,0xa1560101,0xfb840108,0xa1570101,0xfb840108,0xa1580101,0xfb840108,0xa1590101,
	0xfb840108,0xa15a0101,0xfb840108,0xa15b0101,0xfb840108,0xa15c0101,0xfb840108,0xa15d0101,
	0xfb840108,0xa15e0101,0xfb840108,0xa15f0101,0xfb840108,0xa1600101,0xfb840108,0xa1610101,
	0xfb840108,0xa1620101,0xfb840108,0xa1630101,0xfb840108,0xa1640101,0xfb840108,0xa1650101,
	0xfb840108,0xa1660101,0xfb840108,0xa1670101,0xfb840108,0xa1680101,0xfb840108,0xa1690101,
	0xfb840108,0xa16a0101,0xfb840108,0xa16b0101,0xfb840108,0xa16c0101,0xfb840108,0xa16d0101,
	0xfb840108,0xa16e0101,0xfb840108,0xa16f0101,0xfb840108,0xa1700101,0xfb840108,0xa1710101,
	0xfb840108,0xa1720101,0xfb840108,0xa1730101,0xfb840108,0xa1740101,0xfb840108,0xa1750101,
	0xfb840108,0xa1760101,0xfb840108,0xa1770101,0xfb840108,0xa1780101,0xfb840108,0xa1790101,
	0xfb840108,0xa17a0101,0xfb840108,0xa17b0101,0xfb840108,0xa17c0101,0xfb840108,0xa17d0101,
	0xfb840108,0xa17e0101,0xfb840108,0xa17f0101,0xfb840108,0xa1800101,0xfb840108,0xa1810101,
	0xfb840108,0xa1820101,0xfb840108,0xa1830101,0xfb840108,0xa1840101,0xfb840108,0xa1850101,
	0xfb840108,0xa1860101,0xfb840108,0xa1870101,0xfb840108,0xa1880101,0xfb840108,0xa1890101,
	0xfb840108,0xa18a0101,0xfb840108,0xa18b0101,0xfb840108,0xa18c0101,0xfb840108,0xa18d0101,
	0xfb840108,0xa18e0101,0xfb840108,0xa18f0101,0xfb840108,0xa1900101,0xfb840108,0xa1910101,
	0xfb840108,0xa1920101,0xfb840108,0xa1930101,0xfb840108,0xa1940101,0xfb840108,0xa1950101,
	0xfb840108,0xa1960101,0xfb840108,0xa1970101,0xfb840108,0xa1980101,0xfb840108,0xa1990101,
	0xfb840108,0xa19a0101,0xfb840108,0xa19b0101,0xfb840108,0xa19c0101,0xfb840108,0xa19d0101,
	0xfb840108,0xa19e0101,0xfb840108,0xa19f0101,0xfb840108,0xa1a00101,0xfb840108,0xa1a10101,
	0xfb840108,0xa1a20101,0xfb840108,0xa1a30101,0xfb840108,0xa1a40101,0xfb840108,0xa1a50101,
	0xfb840108,0xa1a60101,0xfb840108,0xa1a70101,0xfb840108,0xa1a80101,0xfb840108,0xa1a90101,
	0xfb840108,0xa1aa0101,0xfb840108,0xa1ab0101,0xfb840108,0xa1ac0101,0xfb840108,0xa1ad0101,
	0xfb840108,0xa1ae0101,0xfb840108,0xa1af0101,0xfb840108,0xa1b00101,0xfb840108,0xa1b10101,
	0xfb840108,0xa1b20101,0xfb840108,0xa1b30101,0xfb840108,0xa1b40101,0xfb840108,0xa1b50101,
	0xfb840108,0xa1b60101,0xfb840108,0xa1b70101,0xfb840108,0xa1b80101,0xfb840108,0xa1b90101,
	0xfb840108,0xa1ba0101,0xfb840108,0xa1bb0101,0xfb840108,0xa1bc0101,0xfb840108,0xa1bd0101,
	0xfb840108,0xa1be0101,0xfb840108,0xa1bf0101,0xfb840108,0xa1c00101,0xfb840108,0xa1c10101,
	0xfb840108,0xa1c20101,0xfb840108,0xa1c30101,0xfb840108,0xa1c40101,0xfb840108,0xa1c50101,
	0xfb840108,0xa1c60101,0xfb840108,0xa1c70101,0xfb840108,0xa1c80101,0xfb840108,0xa1c90101,
	0xfb840108,0xa1ca0101,0xfb840108,0xa1cb0101,0xfb840108,0xa1cc0101,0xfb840108,0xa1cd0101,
	0xfb840108,0xa1ce0101,0xfb840108,0xa1cf0101,0xfb840108,0xa1d00101,0xfb840108,0xa1d10101,
	0xfb840108,0xa1d20101,0xfb840108,0xa1d30101,0xfb840108,0xa1d40101,0xfb840108,0xa1d50101,
	0xfb840108,0xa1d60101,0xfb840108,0xa1d70101,0xfb840108,0xa1d80101,0xfb840108,0xa1d90101,
	0xfb840108,0xa1da0101,0xfb840108,0xa1db0101,0xfb840108,0xa1dc0101,0xfb840108,0xa1dd0101,
	0xfb840108,0xa1de0101,0xfb840108,0xa1df0101,0xfb840108,0xa1e00101,0xfb840108,0xa1e10101,
	0xfb840108,0xa1e20101,0xfb840108,0xa1e30101,0xfb840108,0xa1e40101,0xfb840108,0xa1e50101,
	0xfb840108,0xa1e60101,0xfb840108,0xa1e70101,0xfb840108,0xa1e80101,0xfb840108,0xa1e90101,
	0xfb840108,0xa1ea0101,0xfb840108,0xa1eb0101,0xfb840108,0xa1ec0101,0xfb840108,0xa1ed0101,
	0xfb840108,0xa1ee0101,0xfb840108,0xa1ef0101,0xfb840108,0xa1f00101,0xfb840108,0xa1f10101,
	0xfb840108,0xa1f20101,0xfb840108,0xa1f30101,0xfb840108,0xa1f40101,0xfb840108,0xa1f50101,
	0xfb840108,0xa1f60101,0xfb840108,0xa1f70101,0xfb840108,0xa1f80101,0xfb840108,0xa1f90101,
	0xfb840108,0xa1fa0101,0xfb840108,0xa1fb0101,0xfb840108,0xa1fc0101,0xfb840108,0xa1fd0101,
	0xfb840108,0xa1fe0101,0xfb840108,0xa1ff0101,0xfb840108,0xa2000101,0xfb840108,0xa2010101,
	0xfb840108,0xa2020101,0xfb840108,0xa2030101,0xfb840108,0xa2040101,0xfb840108,0xa2050101,
	0xfb840108,0xa2060101,0xfb840108,0xa2070101,0xfb840108,0xa2080101,0xfb840108,0xa2090101,
	0xfb840108,0xa20a0101,0xfb840108,0xa20b0101,0xfb840108,0xa20c0101,0xfb840108,0xa20d0101,
	0xfb840108,0xa20e0101,0xfb840108,0xa20f0101,0xfb840108,0xa2100101,0xfb840108,0xa2110101,
	0xfb840108,0xa2120101,0xfb840108,0xa2130101,0xfb840108,0xa2140101,0xfb840108,0xa2150101,
	0xfb840108,0xa2160101,0xfb840108,0xa2170101,0xfb840108,0xa2180101,0xfb840108,0xa2190101,
	0xfb840108,0xa21a0101,0xfb840108,0xa21b0101,0xfb840108,0xa21c0101,0xfb840108,0xa21d0101,
	0xfb840108,0xa21e0101,0xfb840108,0xa21f0101,0xfb840108,0xa2200101,0xfb840108,0xa2210101,
	0xfb840108,0xa2220101,0xfb840108,0xa2230101,0xfb840108,0xa2240101,0xfb840108,0xa2250101,
	0xfb840108,0xa2260101,0xfb840108,0xa2270101,0xfb840108,0xa2280101,0xfb840108,0xa2290101,
	0xfb840108,0xa22a0101,0xfb840108,0xa22b0101,0xfb840108,0xa22c0101,0xfb840108,0xa22d0101,
	0xfb840108,0xa22e0101,0xfb840108,0xa22f0101,0xfb840108,0xa2300101,0xfb840108,0xa2310101,
	0xfb840108,0xa2320101,0xfb840108,0xa2330101,0xfb840108,0xa2340101,0xfb840108,0xa2350101,
	0xfb840108,0xa2360101,0xfb840108,0xa2370101,0xfb840108,0xa2380101,0xfb840108,0xa2390101,
	0xfb840108,0xa23a0101,0xfb840108,0xa23b0101,0xfb840108,0xa23c0101,0xfb840108,0xa23d0101,
	0xfb840108,0xa23e0101,0xfb840108,0xa23f0101,0xfb840108,0xa2400101,0xfb840108,0xa2410101,
	0xfb840108,0xa2420101,0xfb840108,0xa2430101,0xfb840108,0xa2440101,0xfb840108,0xa2450101,
	0xfb840108,0xa2460101,0xfb840108,0xa2470101,0xfb840108,0xa2480101,0xfb840108,0xa2490101,
	0xfb840108,0xa24a0101,0xfb840108,0xa24b0101,0xfb840108,0xa24c0101,0xfb840108,0xa24d0101,
	0xfb840108,0xa24e0101,0xfb840108,0xa24f0101,0xfb840108,0xa2500101,0xfb840108,0xa2510101,
	0xfb840108,0xa2520101,0xfb840108,0xa2530101,0xfb840108,0xa2540101,0xfb840108,0xa2550101,
	0xfb840108,0xa2560101,0xfb840108,0xa2570101,0xfb840108,0xa2580101,0xfb840108,0xa2590101,
	0xfb840108,0xa25a0101,0xfb840108,0xa25b0101,0xfb840108,0xa25c0101,0xfb840108,0xa25d0101,
	0xfb840108,0xa25e0101,0xfb840108,0xa25f0101,0xfb840108,0xa2600101,0xfb840108,0xa2610101,
	0xfb840108,0xa2620101,0xfb840108,0xa2630101,0xfb840108,0xa2640101,0xfb840108,0xa2650101,
	0xfb840108,0xa2660101,0xfb840108,0xa2670101,0xfb840108,0xa2680101,0xfb840108,0xa2690101,
	0xfb840108,0xa26a0101,0xfb840108,0xa26b0101,0xfb840108,0xa26c0101,0xfb840108,0xa26d0101,
	0xfb840108,0xa26e0101,0xfb840108,0xa26f0101,0xfb840108,0xa2700101,0xfb840108,0xa2710101,
	0xfb840108,0xa2720101,0xfb840108,0xa2730101,0xfb840108,0xa2740101,0xfb840108,0xa2750101,
	0xfb840108,0xa2760101,0xfb840108,0xa2770101,0xfb840108,0xa2780101,0xfb840108,0xa2790101,
	0xfb840108,0xa27a0101,0xfb840108,0xa27b0101,0xfb840108,0xa27c0101,0xfb840108,0xa27d0101,
	0xfb840108,0xa27e0101,0xfb840108,0xa27f0101,0xfb840108,0xa2800101,0xfb840108,0xa2810101,
	0xfb840108,0xa2820101,0xfb840108,0xa2830101,0xfb840108,0xa2840101,0xfb840108,0xa2850101,
	0xfb840108,0xa2860101,0xfb840108,0xa2870101,0xfb840108,0xa2880101,0xfb840108,0xa2890101,
	0xfb840108,0xa28a0101,0xfb840108,0xa28b0101,0xfb840108,0xa28c0101,0xfb840108,0xa28d0101,
	0xfb840108,0xa28e0101,0xfb840108,0xa28f0101,0xfb840108,0xa2900101,0xfb840108,0xa2910101,
	0xfb840108,0xa2920101,0xfb840108,0xa2930101,0xfb840108,0xa2940101,0xfb840108,0xa2950101,
	0xfb840108,0xa2960101,0xfb840108,0xa2970101,0xfb840108,0xa2980101,0xfb840108,0xa2990101,
	0xfb840108,0xa29a0101,0xfb840108,0xa29b0101,0xfb840108,0xa29c0101,0xfb840108,0xa29d0101,
	0xfb840108,0xa29e0101,0xfb840108,0xa29f0101,0xfb840108,0xa2a00101,0xfb840108,0xa2a10101,
	0xfb840108,0xa2a20101,0xfb840108,0xa2a30101,0xfb840108,0xa2a40101,0xfb840108,0xa2a50101,
	0xfb840108,0xa2a60101,0xfb840108,0xa2a70101,0xfb840108,0xa2a80101,0xfb840108,0xa2a90101,
	0xfb840108,0xa2aa0101,0xfb840108,0xa2ab0101,0xfb840108,0xa2ac0101,0xfb840108,0xa2ad0101,
	0xfb840108,0xa2ae0101,0xfb840108,0xa2af0101,0xfb840108,0xa2b00101,0xfb840108,0xa2b10101,
	0xfb840108,0xa2b20101,0xfb840108,0xa2b30101,0xfb840108,0xa2b40101,0xfb840108,0xa2b50101,
	0xfb840108,0xa2b60101,0xfb840108,0xa2b70101,0xfb840108,0xa2b80101,0xfb840108,0xa2b90101,
	0xfb840108,0xa2ba0101,0xfb840108,0xa2bb0101,0xfb840108,0xa2bc0101,0xfb840108,0xa2bd0101,
	0xfb840108,0xa2be0101,0xfb840108,0xa2bf0101,0xfb840108,0xa2c00101,0xfb840108,0xa2c10101,
	0xfb840108,0xa2c20101,0xfb840108,0xa2c30101,0xfb840108,0xa2c40101,0xfb840108,0xa2c50101,
	0xfb840108,0xa2c60101,0xfb840108,0xa2c70101,0xfb840108,0xa2c80101,0xfb840108,0xa2c90101,
	0xfb840108,0xa2ca0101,0xfb840108,0xa2cb0101,0xfb840108,0xa2cc0101,0xfb840108,0xa2cd0101,
	0xfb840108,0xa2ce0101,0xfb840108,0xa2cf0101,0xfb840108,0xa2d00101,0xfb840108,0xa2d10101,
	0xfb840108,0xa2d20101,0xfb840108,0xa2d30101,0xfb840108,0xa2d40101,0xfb840108,0xa2d50101,
	0xfb840108,0xa2d60101,0xfb840108,0xa2d70101,0xfb840108,0xa2d80101,0xfb840108,0xa2d90101,
	0xfb840108,0xa2da0101,0xfb840108,0xa2db0101,0xfb840108,0xa2dc0101,0xfb840108,0xa2dd0101,
	0xfb840108,0xa2de0101,0xfb840108,0xa2df0101,0xfb840108,0xa2e00101,0xfb840108,0xa2e10101,
	0xfb840108,0xa2e20101,0xfb840108,0xa2e30101,0xfb840108,0xa2e40101,0xfb840108,0xa2e50101,
	0xfb840108,0xa2e60101,0xfb840108,0xa2e70101,0xfb840108,0xa2e80101,0xfb840108,0xa2e90101,
	0xfb840108,0xa2ea0101,0xfb840108,0xa2eb0101,0xfb840108,0xa2ec0101,0xfb840108,0xa2ed0101,
	0xfb840108,0xa2ee0101,0xfb840108,0xa2ef0101,0xfb840108,0xa2f00101,0xfb840108,0xa2f10101,
	0xfb840108,0xa2f20101,0xfb840108,0xa2f30101,0xfb840108,0xa2f40101,0xfb840108,0xa2f50101,
	0xfb840108,0xa2f60101,0xfb840108,0xa2f70101,0xfb840108,0xa2f80101,0xfb840108,0xa2f90101,
	0xfb840108,0xa2fa0101,0xfb840108,0xa2fb0101,0xfb840108,0xa2fc0101,0xfb840108,0xa2fd0101,
	0xfb840108,0xa2fe0101,0xfb840108,0xa2ff0101,0xfb840108,0xa3000101,0xfb840108,0xa3010101,
	0xfb840108,0xa3020101,0xfb840108,0xa3030101,0xfb840108,0xa3040101,0xfb840108,0xa3050101,
	0xfb840108,0xa3060101,0xfb840108,0xa3070101,0xfb840108,0xa3080101,0xfb840108,0xa3090101,
	0xfb840108,0xa30a0101,0xfb840108,0xa30b0101,0xfb840108,0xa30c0101,0xfb840108,0xa30d0101,
	0xfb840108,0xa30e0101,0xfb840108,0xa30f0101,0xfb840108,0xa3100101,0xfb840108,0xa3110101,
	0xfb840108,0xa3120101,0xfb840108,0xa3130101,0xfb840108,0xa3140101,0xfb840108,0xa3150101,
	0xfb840108,0xa3160101,0xfb840108,0xa3170101,0xfb840108,0xa3180101,0xfb840108,0xa3190101,
	0xfb840108,0xa31a0101,0xfb840108,0xa31b0101,0xfb840108,0xa31c0101,0xfb840108,0xa31d0101,
	0xfb840108,0xa31e0101,0xfb840108,0xa31f0101,0xfb840108,0xa3200101,0xfb840108,0xa3210101,
	0xfb840108,0xa3220101,0xfb840108,0xa3230101,0xfb840108,0xa3240101,0xfb840108,0xa3250101,
	0xfb840108,0xa3260101,0xfb840108,0xa3270101,0xfb840108,0xa3280101,0xfb840108,0xa3290101,
	0xfb840108,0xa32a0101,0xfb840108,0xa32b0101,0xfb840108,0xa32c0101,0xfb840108,0xa32d0101,
	0xfb840108,0xa32e0101,0xfb840108,0xa32f0101,0xfb840108,0xa3300101,0xfb840108,0xa3310101,
	0xfb840108,0xa3320101,0xfb840108,0xa3330101,0xfb840108,0xa3340101,0xfb840108,0xa3350101,
	0xfb840108,0xa3360101,0xfb840108,0xa3370101,0xfb840108,0xa3380101,0xfb840108,0xa3390101,
	0xfb840108,0xa33a0101,0xfb840108,0xa33b0101,0xfb840108,0xa33c0101,0xfb840108,0xa33d0101,
	0xfb840108,0xa33e0101,0xfb840108,0xa33f0101,0xfb840108,0xa3400101,0xfb840108,0xa3410101,
	0xfb840108,0xa3420101,0xfb840108,0xa3430101,0xfb840108,0xa3440101,0xfb840108,0xa3450101,
	0xfb840108,0xa3460101,0xfb840108,0xa3470101,0xfb840108,0xa3480101,0xfb840108,0xa3490101,
	0xfb840108,0xa34a0101,0xfb840108,0xa34b0101,0xfb840108,0xa34c0101,0xfb840108,0xa34d0101,
	0xfb840108,0xa34e0101,0xfb840108,0xa34f0101,0xfb840108,0xa3500101,0xfb840108,0xa3510101,
	0xfb840108,0xa3520101,0xfb840108,0xa3530101,0xfb840108,0xa3540101,0xfb840108,0xa3550101,
	0xfb840108,0xa3560101,0xfb840108,0xa3570101,0xfb840108,0xa3580101,0xfb840108,0xa3590101,
	0xfb840108,0xa35a0101,0xfb840108,0xa35b0101,0xfb840108,0xa35c0101,0xfb840108,0xa35d0101,
	0xfb840108,0xa35e0101,0xfb840108,0xa35f0101,0xfb840108,0xa3600101,0xfb840108,0xa3610101,
	0xfb840108,0xa3620101,0xfb840108,0xa3630101,0xfb840108,0xa3640101,0xfb840108,0xa3650101,
	0xfb840108,0xa3660101,0xfb840108,0xa3670101,0xfb840108,0xa3680101,0xfb840108,0xa3690101,
	0xfb840108,0xa36a0101,0xfb840108,0xa36b0101,0xfb840108,0xa36c0101,0xfb840108,0xa36d0101,
	0xfb840108,0xa36e0101,0xfb840108,0xa36f0101,0xfb840108,0xa3700101,0xfb840108,0xa3710101,
	0xfb840108,0xa3720101,0xfb840108,0xa3730101,0xfb840108,0xa3740101,0xfb840108,0xa3750101,
	0xfb840108,0xa3760101,0xfb840108,0xa3770101,0xfb840108,0xa3780101,0xfb840108,0xa3790101,
	0xfb840108,0xa37a0101,0xfb840108,0xa37b0101,0xfb840108,0xa37c0101,0xfb840108,0xa37d0101,
	0xfb840108,0xa37e0101,0xfb840108,0xa37f0101,0xfb840108,0xa3800101,0xfb840108,0xa3810101,
	0xfb840108,0xa3820101,0xfb840108,0xa3830101,0xfb840108,0xa3840101,0xfb840108,0xa3850101,
	0xfb840108,0xa3860101,0xfb840108,0xa3870101,0xfb840108,0xa3880101,0xfb840108,0xa3890101,
	0xfb840108,0xa38a0101,0xfb840108,0xa38b0101,0xfb840108,0xa38c0101,0xfb840108,0xa38d0101,
	0xfb840108,0xa38e0101,0xfb840108,0xa38f0101,0xfb840108,0xa3900101,0xfb840108,0xa3910101,
	0xfb840108,0xa3920101,0xfb840108,0xa3930101,0xfb840108,0xa3940101,0xfb840108,0xa3950101,
	0xfb840108,0xa3960101,0xfb840108,0xa3970101,0xfb840108,0xa3980101,0xfb840108,0xa3990101,
	0xfb840108,0xa39a0101,0xfb840108,0xa39b0101,0xfb840108,0xa39c0101,0xfb840108,0xa39d0101,
	0xfb840108,0xa39e0101,0xfb840108,0xa39f0101,0xfb840108,0xa3a00101,0xfb840108,0xa3a10101,
	0xfb840108,0xa3a20101,0xfb840108,0xa3a30101,0xfb840108,0xa3a40101,0xfb840108,0xa3a50101,
	0xfb840108,0xa3a60101,0xfb840108,0xa3a70101,0xfb840108,0xa3a80101,0xfb840108,0xa3a90101,
	0xfb840108,0xa3aa0101,0xfb840108,0xa3ab0101,0xfb840108,0xa3ac0101,0xfb840108,0xa3ad0101,
	0xfb840108,0xa3ae0101,0xfb840108,0xa3af0101,0xfb840108,0xa3b00101,0xfb840108,0xa3b10101,
	0xfb840108,0xa3b20101,0xfb840108,0xa3b30101,0xfb840108,0xa3b40101,0xfb840108,0xa3b50101,
	0xfb840108,0xa3b60101,0xfb840108,0xa3b70101,0xfb840108,0xa3b80101,0xfb840108,0xa3b90101,
	0xfb840108,0xa3ba0101,0xfb840108,0xa3bb0101,0xfb840108,0xa3bc0101,0xfb840108,0xa3bd0101,
	0xfb840108,0xa3be0101,0xfb840108,0xa3bf0101,0xfb840108,0xa3c00101,0xfb840108,0xa3c10101,
	0xfb840108,0xa3c20101,0xfb840108,0xa3c30101,0xfb840108,0xa3c40101,0xfb840108,0xa3c50101,
	0xfb840108,0xa3c60101,0xfb840108,0xa3c70101,0xfb840108,0xa3c80101,0xfb840108,0xa3c90101,
	0xfb840108,0xa3ca0101,0xfb840108,0xa3cb0101,0xfb840108,0xa3cc0101,0xfb840108,0xa3cd0101,
	0xfb840108,0xa3ce0101,0xfb840108,0xa3cf0101,0xfb840108,0xa3d00101,0xfb840108,0xa3d10101,
	0xfb840108,0xa3d20101,0xfb840108,0xa3d30101,0xfb840108,0xa3d40101,0xfb840108,0xa3d50101,
	0xfb840108,0xa3d60101,0xfb840108,0xa3d70101,0xfb840108,0xa3d80101,0xfb840108,0xa3d90101,
	0xfb840108,0xa3da0101,0xfb840108,0xa3db0101,0xfb840108,0xa3dc0101,0xfb840108,0xa3dd0101,
	0xfb840108,0xa3de0101,0xfb840108,0xa3df0101,0xfb840108,0xa3e00101,0xfb840108,0xa3e10101,
	0xfb840108,0xa3e20101,0xfb840108,0xa3e30101,0xfb840108,0xa3e40101,0xfb840108,0xa3e50101,
	0xfb840108,0xa3e60101,0xfb840108,0xa3e70101,0xfb840108,0xa3e80101,0xfb840108,0xa3e90101,
	0xfb840108,0xa3ea0101,0xfb840108,0xa3eb0101,0xfb840108,0xa3ec0101,0xfb840108,0xa3ed0101,
	0xfb840108,0xa3ee0101,0xfb840108,0xa3ef0101,0xfb840108,0xa3f00101,0xfb840108,0xa3f10101,
	0xfb840108,0xa3f20101,0xfb840108,0xa3f30101,0xfb840108,0xa3f40101,0xfb840108,0xa3f50101,
	0xfb840108,0xa3f60101,0xfb840108,0xa3f70101,0xfb840108,0xa3f80101,0xfb840108,0xa3f90101,
	0xfb840108,0xa3fa0101,0xfb840108,0xa3fb0101,0xfb840108,0xa3fc0101,0xfb840108,0xa3fd0101,
	0xfb840108,0xa3fe0101,0xfb840108,0xa3ff0101,0xfb840108,0xa4000101,0xfb840108,0xa4010101,
	0xfb840108,0xa4020101,0xfb840108,0xa4030101,0xfb840108,0xa4040101,0xfb840108,0xa4050101,
	0xfb840108,0xa4060101,0xfb840108,0xa4070101,0xfb840108,0xa4080101,0xfb840108,0xa4090101,
	0xfb840108,0xa40a0101,0xfb840108,0xa40b0101,0xfb840108,0xa40c0101,0xfb840108,0xa40d0101,
	0xfb840108,0xa40e0101,0xfb840108,0xa40f0101,0xfb840108,0xa4100101,0xfb840108,0xa4110101,
	0xfb840108,0xa4120101,0xfb840108,0xa4130101,0xfb840108,0xa4140101,0xfb840108,0xa4150101,
	0xfb840108,0xa4160101,0xfb840108,0xa4170101,0xfb840108,0xa4180101,0xfb840108,0xa4190101,
	0xfb840108,0xa41a0101,0xfb840108,0xa41b0101,0xfb840108,0xa41c0101,0xfb840108,0xa41d0101,
	0xfb840108,0xa41e0101,0xfb840108,0xa41f0101,0xfb840108,0xa4200101,0xfb840108,0xa4210101,
	0xfb840108,0xa4220101,0xfb840108,0xa4230101,0xfb840108,0xa4240101,0xfb840108,0xa4250101,
	0xfb840108,0xa4260101,0xfb840108,0xa4270101,0xfb840108,0xa4280101,0xfb840108,0xa4290101,
	0xfb840108,0xa42a0101,0xfb840108,0xa42b0101,0xfb840108,0xa42c0101,0xfb840108,0xa42d0101,
	0xfb840108,0xa42e0101,0xfb840108,0xa42f0101,0xfb840108,0xa4300101,0xfb840108,0xa4310101,
	0xfb840108,0xa4320101,0xfb840108,0xa4330101,0xfb840108,0xa4340101,0xfb840108,0xa4350101,
	0xfb840108,0xa4360101,0xfb840108,0xa4370101,0xfb840108,0xa4380101,0xfb840108,0xa4390101,
	0xfb840108,0xa43a0101,0xfb840108,0xa43b0101,0xfb840108,0xa43c0101,0xfb840108,0xa43d0101,
	0xfb840108,0xa43e0101,0xfb840108,0xa43f0101,0xfb840108,0xa4400101,0xfb840108,0xa4410101,
	0xfb840108,0xa4420101,0xfb840108,0xa4430101,0xfb840108,0xa4440101,0xfb840108,0xa4450101,
	0xfb840108,0xa4460101,0xfb840108,0xa4470101,0xfb840108,0xa4480101,0xfb840108,0xa4490101,
	0xfb840108,0xa44a0101,0xfb840108,0xa44b0101,0xfb840108,0xa44c0101,0xfb840108,0xa44d0101,
	0xfb840108,0xa44e0101,0xfb840108,0xa44f0101,0xfb840108,0xa4500101,0xfb840108,0xa4510101,
	0xfb840108,0xa4520101,0xfb840108,0xa4530101,0xfb840108,0xa4540101,0xfb840108,0xa4550101,
	0xfb840108,0xa4560101,0xfb840108,0xa4570101,0xfb840108,0xa4580101,0xfb840108,0xa4590101,
	0xfb840108,0xa45a0101,0xfb840108,0xa45b0101,0xfb840108,0xa45c0101,0xfb840108,0xa45d0101,
	0xfb840108,0xa45e0101,0xfb840108,0xa45f0101,0xfb840108,0xa4600101,0xfb840108,0xa4610101,
	0xfb840108,0xa4620101,0xfb840108,0xa4630101,0xfb840108,0xa4640101,0xfb840108,0xa4650101,
	0xfb840108,0xa4660101,0xfb840108,0xa4670101,0xfb840108,0xa4680101,0xfb840108,0xa4690101,
	0xfb840108,0xa46a0101,0xfb840108,0xa46b0101,0xfb840108,0xa46c0101,0xfb840108,0xa46d0101,
	0xfb840108,0xa46e0101,0xfb840108,0xa46f0101,0xfb840108,0xa4700101,0xfb840108,0xa4710101,
	0xfb840108,0xa4720101,0xfb840108,0xa4730101,0xfb840108,0xa4740101,0xfb840108,0xa4750101,
	0xfb840108,0xa4760101,0xfb840108,0xa4770101,0xfb840108,0xa4780101,0xfb840108,0xa4790101,
	0xfb840108,0xa47a0101,0xfb840108,0xa47b0101,0xfb840108,0xa47c0101,0xfb840108,0xa47d0101,
	0xfb840108,0xa47e0101,0xfb840108,0xa47f0101,0xfb840108,0xa4800101,0xfb840108,0xa4810101,
	0xfb840108,0xa4820101,0xfb840108,0xa4830101,0xfb840108,0xa4840101,0xfb840108,0xa4850101,
	0xfb840108,0xa4860101,0xfb840108,0xa4870101,0xfb840108,0xa4880101,0xfb840108,0xa4890101,
	0xfb840108,0xa48a0101,0xfb840108,0xa48b0101,0xfb840108,0xa48c0101,0xfb840108,0xa48d0101,
	0xfb840108,0xa48e0101,0xfb840108,0xa48f0101,0xfb840108,0xa4900101,0xfb840108,0xa4910101,
	0xfb840108,0xa4920101,0xfb840108,0xa4930101,0xfb840108,0xa4940101,0xfb840108,0xa4950101,
	0xfb840108,0xa4960101,0xfb840108,0xa4970101,0xfb840108,0xa4980101,0xfb840108,0xa4990101,
	0xfb840108,0xa49a0101,0xfb840108,0xa49b0101,0xfb840108,0xa49c0101,0xfb840108,0xa49d0101,
	0xfb840108,0xa49e0101,0xfb840108,0xa49f0101,0xfb840108,0xa4a00101,0xfb840108,0xa4a10101,
	0xfb840108,0xa4a20101,0xfb840108,0xa4a30101,0xfb840108,0xa4a40101,0xfb840108,0xa4a50101,
	0xfb840108,0xa4a60101,0xfb840108,0xa4a70101,0xfb840108,0xa4a80101,0xfb840108,0xa4a90101,
	0xfb840108,0xa4aa0101,0xfb840108,0xa4ab0101,0xfb840108,0xa4ac0101,0xfb840108,0xa4ad0101,
	0xfb840108,0xa4ae0101,0xfb840108,0xa4af0101,0xfb840108,0xa4b00101,0xfb840108,0xa4b10101,
	0xfb840108,0xa4b20101,0xfb840108,0xa4b30101,0xfb840108,0xa4b40101,0xfb840108,0xa4b50101,
	0xfb840108,0xa4b60101,0xfb840108,0xa4b70101,0xfb840108,0xa4b80101,0xfb840108,0xa4b90101,
	0xfb840108,0xa4ba0101,0xfb840108,0xa4bb0101,0xfb840108,0xa4bc0101,0xfb840108,0xa4bd0101,
	0xfb840108,0xa4be0101,0xfb840108,0xa4bf0101,0xfb840108,0xa4c00101,0xfb840108,0xa4c10101,
	0xfb840108,0xa4c20101,0xfb840108,0xa4c30101,0xfb840108,0xa4c40101,0xfb840108,0xa4c50101,
	0xfb840108,0xa4c60101,0xfb840108,0xa4c70101,0xfb840108,0xa4c80101,0xfb840108,0xa4c90101,
	0xfb840108,0xa4ca0101,0xfb840108,0xa4cb0101,0xfb840108,0xa4cc0101,0xfb840108,0xa4cd0101,
	0xfb840108,0xa4ce0101,0xfb840108,0xa4cf0101,0xfb840108,0xa4d00101,0xfb840108,0xa4d10101,
	0xfb840108,0xa4d20101,0xfb840108,0xa4d30101,0xfb840108,0xa4d40101,0xfb840108,0xa4d50101,
	0xfb840108,0xa4d60101,0xfb840108,0xa4d70101,0xfb840108,0xa4d80101,0xfb840108,0xa4d90101,
	0xfb840108,0xa4da0101,0xfb840108,0xa4db0101,0xfb840108,0xa4dc0101,0xfb840108,0xa4dd0101,
	0xfb840108,0xa4de0101,0xfb840108,0xa4df0101,0xfb840108,0xa4e00101,0xfb840108,0xa4e10101,
	0xfb840108,0xa4e20101,0xfb840108,0xa4e30101,0xfb840108,0xa4e40101,0xfb840108,0xa4e50101,
	0xfb840108,0xa4e60101,0xfb840108,0xa4e70101,0xfb840108,0xa4e80101,0xfb840108,0xa4e90101,
	0xfb840108,0xa4ea0101,0xfb840108,0xa4eb0101,0xfb840108,0xa4ec0101,0xfb840108,0xa4ed0101,
	0xfb840108,0xa4ee0101,0xfb840108,0xa4ef0101,0xfb840108,0xa4f00101,0xfb840108,0xa4f10101,
	0xfb840108,0xa4f20101,0xfb840108,0xa4f30101,0xfb840108,0xa4f40101,0xfb840108,0xa4f50101,
	0xfb840108,0xa4f60101,0xfb840108,0xa4f70101,0xfb840108,0xa4f80101,0xfb840108,0xa4f90101,
	0xfb840108,0xa4fa0101,0xfb840108,0xa4fb0101,0xfb840108,0xa4fc0101,0xfb840108,0xa4fd0101,
	0xfb840108,0xa4fe0101,0xfb840108,0xa4ff0101,0xfb840108,0xa5000101,0xfb840108,0xa5010101,
	0xfb840108,0xa5020101,0xfb840108,0xa5030101,0xfb840108,0xa5040101,0xfb840108,0xa5050101,
	0xfb840108,0xa5060101,0xfb840108,0xa5070101,0xfb840108,0xa5080101,0xfb840108,0xa5090101,
	0xfb840108,0xa50a0101,0xfb840108,0xa50b0101,0xfb840108,0xa50c0101,0xfb840108,0xa50d0101,
	0xfb840108,0xa50e0101,0xfb840108,0xa50f0101,0xfb840108,0xa5100101,0xfb840108,0xa5110101,
	0xfb840108,0xa5120101,0xfb840108,0xa5130101,0xfb840108,0xa5140101,0xfb840108,0xa5150101,
	0xfb840108,0xa5160101,0xfb840108,0xa5170101,0xfb840108,0xa5180101,0xfb840108,0xa5190101,
	0xfb840108,0xa51a0101,0xfb840108,0xa51b0101,0xfb840108,0xa51c0101,0xfb840108,0xa51d0101,
	0xfb840108,0xa51e0101,0xfb840108,0xa51f0101,0xfb840108,0xa5200101,0xfb840108,0xa5210101,
	0xfb840108,0xa5220101,0xfb840108,0xa5230101,0xfb840108,0xa5240101,0xfb840108,0xa5250101,
	0xfb840108,0xa5260101,0xfb840108,0xa5270101,0xfb840108,0xa5280101,0xfb840108,0xa5290101,
	0xfb840108,0xa52a0101,0xfb840108,0xa52b0101,0xfb840108,0xa52c0101,0xfb840108,0xa52d0101,
	0xfb840108,0xa52e0101,0xfb840108,0xa52f0101,0xfb840108,0xa5300101,0xfb840108,0xa5310101,
	0xfb840108,0xa5320101,0xfb840108,0xa5330101,0xfb840108,0xa5340101,0xfb840108,0xa5350101,
	0xfb840108,0xa5360101,0xfb840108,0xa5370101,0xfb840108,0xa5380101,0xfb840108,0xa5390101,
	0xfb840108,0xa53a0101,0xfb840108,0xa53b0101,0xfb840108,0xa53c0101,0xfb840108,0xa53d0101,
	0xfb840108,0xa53e0101,0xfb840108,0xa53f0101,0xfb840108,0xa5400101,0xfb840108,0xa5410101,
	0xfb840108,0xa5420101,0xfb840108,0xa5430101,0xfb840108,0xa5440101,0xfb840108,0xa5450101,
	0xfb840108,0xa5460101,0xfb840108,0xa5470101,0xfb840108,0xa5480101,0xfb840108,0xa5490101,
	0xfb840108,0xa54a0101,0xfb840108,0xa54b0101,0xfb840108,0xa54c0101,0xfb840108,0xa54d0101,
	0xfb840108,0xa54e0101,0xfb840108,0xa54f0101,0xfb840108,0xa5500101,0xfb840108,0xa5510101,
	0xfb840108,0xa5520101,0xfb840108,0xa5530101,0xfb840108,0xa5540101,0xfb840108,0xa5550101,
	0xfb840108,0xa5560101,0xfb840108,0xa5570101,0xfb840108,0xa5580101,0xfb840108,0xa5590101,
	0xfb840108,0xa55a0101,0xfb840108,0xa55b0101,0xfb840108,0xa55c0101,0xfb840108,0xa55d0101,
	0xfb840108,0xa55e0101,0xfb840108,0xa55f0101,0xfb840108,0xa5600101,0xfb840108,0xa5610101,
	0xfb840108,0xa5620101,0xfb840108,0xa5630101,0xfb840108,0xa5640101,0xfb840108,0xa5650101,
	0xfb840108,0xa5660101,0xfb840108,0xa5670101,0xfb840108,0xa5680101,0xfb840108,0xa5690101,
	0xfb840108,0xa56a0101,0xfb840108,0xa56b0101,0xfb840108,0xa56c0101,0xfb840108,0xa56d0101,
	0xfb840108,0xa56e0101,0xfb840108,0xa56f0101,0xfb840108,0xa5700101,0xfb840108,0xa5710101,
	0xfb840108,0xa5720101,0xfb840108,0xa5730101,0xfb840108,0xa5740101,0xfb840108,0xa5750101,
	0xfb840108,0xa5760101,0xfb840108,0xa5770101,0xfb840108,0xa5780101,0xfb840108,0xa5790101,
	0xfb840108,0xa57a0101,0xfb840108,0xa57b0101,0xfb840108,0xa57c0101,0xfb840108,0xa57d0101,
	0xfb840108,0xa57e0101,0xfb840108,0xa57f0101,0xfb840108,0xa5800101,0xfb840108,0xa5810101,
	0xfb840108,0xa5820101,0xfb840108,0xa5830101,0xfb840108,0xa5840101,0xfb840108,0xa5850101,
	0xfb840108,0xa5860101,0xfb840108,0xa5870101,0xfb840108,0xa5880101,0xfb840108,0xa5890101,
	0xfb840108,0xa58a0101,0xfb840108,0xa58b0101,0xfb840108,0xa58c0101,0xfb840108,0xa58d0101,
	0xfb840108,0xa58e0101,0xfb840108,0xa58f0101,0xfb840108,0xa5900101,0xfb840108,0xa5910101,
	0xfb840108,0xa5920101,0xfb840108,0xa5930101,0xfb840108,0xa5940101,0xfb840108,0xa5950101,
	0xfb840108,0xa5960101,0xfb840108,0xa5970101,0xfb840108,0xa5980101,0xfb840108,0xa5990101,
	0xfb840108,0xa59a0101,0xfb840108,0xa59b0101,0xfb840108,0xa59c0101,0xfb840108,0xa59d0101,
	0xfb840108,0xa59e0101,0xfb840108,0xa59f0101,0xfb840108,0xa5a00101,0xfb840108,0xa5a10101,
	0xfb840108,0xa5a20101,0xfb840108,0xa5a30101,0xfb840108,0xa5a40101,0xfb840108,0xa5a50101,
	0xfb840108,0xa5a60101,0xfb840108,0xa5a70101,0xfb840108,0xa5a80101,0xfb840108,0xa5a90101,
	0xfb840108,0xa5aa0101,0xfb840108,0xa5ab0101,0xfb840108,0xa5ac0101,0xfb840108,0xa5ad0101,
	0xfb840108,0xa5ae0101,0xfb840108,0xa5af0101,0xfb840108,0xa5b00101,0xfb840108,0xa5b10101,
	0xfb840108,0xa5b20101,0xfb840108,0xa5b30101,0xfb840108,0xa5b40101,0xfb840108,0xa5b50101,
	0xfb840108,0xa5b60101,0xfb840108,0xa5b70101,0xfb840108,0xa5b80101,0xfb840108,0xa5b90101,
	0xfb840108,0xa5ba0101,0xfb840108,0xa5bb0101,0xfb840108,0xa5bc0101,0xfb840108,0xa5bd0101,
	0xfb840108,0xa5be0101,0xfb840108,0xa5bf0101,0xfb840108,0xa5c00101,0xfb840108,0xa5c10101,
	0xfb840108,0xa5c20101,0xfb840108,0xa5c30101,0xfb840108,0xa5c40101,0xfb840108,0xa5c50101,
	0xfb840108,0xa5c60101,0xfb840108,0xa5c70101,0xfb840108,0xa5c80101,0xfb840108,0xa5c90101,
	0xfb840108,0xa5ca0101,0xfb840108,0xa5cb0101,0xfb840108,0xa5cc0101,0xfb840108,0xa5cd0101,
	0xfb840108,0xa5ce0101,0xfb840108,0xa5cf0101,0xfb840108,0xa5d00101,0xfb840108,0xa5d10101,
	0xfb840108,0xa5d20101,0xfb840108,0xa5d30101,0xfb840108,0xa5d40101,0xfb840108,0xa5d50101,
	0xfb840108,0xa5d60101,0xfb840108,0xa5d70101,0xfb840108,0xa5d80101,0xfb840108,0xa5d90101,
	0xfb840108,0xa5da0101,0xfb840108,0xa5db0101,0xfb840108,0xa5dc0101,0xfb840108,0xa5dd0101,
	0xfb840108,0xa5de0101,0xfb840108,0xa5df0101,0xfb840108,0xa5e00101,0xfb840108,0xa5e10101,
	0xfb840108,0xa5e20101,0xfb840108,0xa5e30101,0xfb840108,0xa5e40101,0xfb840108,0xa5e50101,
	0xfb840108,0xa5e60101,0xfb840108,0xa5e70101,0xfb840108,0xa5e80101,0xfb840108,0xa5e90101,
	0xfb840108,0xa5ea0101,0xfb840108,0xa5eb0101,0xfb840108,0xa5ec0101,0xfb840108,0xa5ed0101,
	0xfb840108,0xa5ee0101,0xfb840108,0xa5ef0101,0xfb840108,0xa5f00101,0xfb840108,0xa5f10101,
	0xfb840108,0xa5f20101,0xfb840108,0xa5f30101,0xfb840108,0xa5f40101,0xfb840108,0xa5f50101,
	0xfb840108,0xa5f60101,0xfb840108,0xa5f70101,0xfb840108,0xa5f80101,0xfb840108,0xa5f90101,
	0xfb840108,0xa5fa0101,0xfb840108,0xa5fb0101,0xfb840108,0xa5fc0101,0xfb840108,0xa5fd0101,
	0xfb840108,0xa5fe0101,0xfb840108,0xa5ff0101,0xfb840108,0xa6000101,0xfb840108,0xa6010101,
	0xfb840108,0xa6020101,0xfb840108,0xa6030101,0xfb840108,0xa6040101,0xfb840108,0xa6050101,
	0xfb840108,0xa6060101,0xfb840108,0xa6070101,0xfb840108,0xa6080101,0xfb840108,0xa6090101,
	0xfb840108,0xa60a0101,0xfb840108,0xa60b0101,0xfb840108,0xa60c0101,0xfb840108,0xa60d0101,
	0xfb840108,0xa60e0101,0xfb840108,0xa60f0101,0xfb840108,0xa6100101,0xfb840108,0xa6110101,
	0xfb840108,0xa6120101,0xfb840108,0xa6130101,0xfb840108,0xa6140101,0xfb840108,0xa6150101,
	0xfb840108,0xa6160101,0xfb840108,0xa6170101,0xfb840108,0xa6180101,0xfb840108,0xa6190101,
	0xfb840108,0xa61a0101,0xfb840108,0xa61b0101,0xfb840108,0xa61c0101,0xfb840108,0xa61d0101,
	0xfb840108,0xa61e0101,0xfb840108,0xa61f0101,0xfb840108,0xa6200101,0xfb840108,0xa6210101,
	0xfb840108,0xa6220101,0xfb840108,0xa6230101,0xfb840108,0xa6240101,0xfb840108,0xa6250101,
	0xfb840108,0xa6260101,0xfb840108,0xa6270101,0xfb840108,0xa6280101,0xfb840108,0xa6290101,
	0xfb840108,0xa62a0101,0xfb840108,0xa62b0101,0xfb840108,0xa62c0101,0xfb840108,0xa62d0101,
	0xfb840108,0xa62e0101,0xfb840108,0xa62f0101,0xfb840108,0xa6300101,0xfb840108,0xa6310101,
	0xfb840108,0xa6320101,0xfb840108,0xa6330101,0xfb840108,0xa6340101,0xfb840108,0xa6350101,
	0xfb840108,0xa6360101,0xfb840108,0xa6370101,0xfb840108,0xa6380101,0xfb840108,0xa6390101,
	0xfb840108,0xa63a0101,0xfb840108,0xa63b0101,0xfb840108,0xa63c0101,0xfb840108,0xa63d0101,
	0xfb840108,0xa63e0101,0xfb840108,0xa63f0101,0xfb840108,0xa6400101,0xfb840108,0xa6410101,
	0xfb840108,0xa6420101,0xfb840108,0xa6430101,0xfb840108,0xa6440101,0xfb840108,0xa6450101,
	0xfb840108,0xa6460101,0xfb840108,0xa6470101,0xfb840108,0xa6480101,0xfb840108,0xa6490101,
	0xfb840108,0xa64a0101,0xfb840108,0xa64b0101,0xfb840108,0xa64c0101,0xfb840108,0xa64d0101,
	0xfb840108,0xa64e0101,0xfb840108,0xa64f0101,0xfb840108,0xa6500101,0xfb840108,0xa6510101,
	0xfb840108,0xa6520101,0xfb840108,0xa6530101,0xfb840108,0xa6540101,0xfb840108,0xa6550101,
	0xfb840108,0xa6560101,0xfb840108,0xa6570101,0xfb840108,0xa6580101,0xfb840108,0xa6590101,
	0xfb840108,0xa65a0101,0xfb840108,0xa65b0101,0xfb840108,0xa65c0101,0xfb840108,0xa65d0101,
	0xfb840108,0xa65e0101,0xfb840108,0xa65f0101,0xfb840108,0xa6600101,0xfb840108,0xa6610101,
	0xfb840108,0xa6620101,0xfb840108,0xa6630101,0xfb840108,0xa6640101,0xfb840108,0xa6650101,
	0xfb840108,0xa6660101,0xfb840108,0xa6670101,0xfb840108,0xa6680101,0xfb840108,0xa6690101,
	0xfb840108,0xa66a0101,0xfb840108,0xa66b0101,0xfb840108,0xa66c0101,0xfb840108,0xa66d0101,
	0xfb840108,0xa66e0101,0xfb840108,0xa66f0101,0xfb840108,0xa6700101,0xfb840108,0xa6710101,
	0xfb840108,0xa6720101,0xfb840108,0xa6730101,0xfb840108,0xa6740101,0xfb840108,0xa6750101,
	0xfb840108,0xa6760101,0xfb840108,0xa6770101,0xfb840108,0xa6780101,0xfb840108,0xa6790101,
	0xfb840108,0xa67a0101,0xfb840108,0xa67b0101,0xfb840108,0xa67c0101,0xfb840108,0xa67d0101,
	0xfb840108,0xa67e0101,0xfb840108,0xa67f0101,0xfb840108,0xa6800101,0xfb840108,0xa6810101,
	0xfb840108,0xa6820101,0xfb840108,0xa6830101,0xfb840108,0xa6840101,0xfb840108,0xa6850101,
	0xfb840108,0xa6860101,0xfb840108,0xa6870101,0xfb840108,0xa6880101,0xfb840108,0xa6890101,
	0xfb840108,0xa68a0101,0xfb840108,0xa68b0101,0xfb840108,0xa68c0101,0xfb840108,0xa68d0101,
	0xfb840108,0xa68e0101,0xfb840108,0xa68f0101,0xfb840108,0xa6900101,0xfb840108,0xa6910101,
	0xfb840108,0xa6920101,0xfb840108,0xa6930101,0xfb840108,0xa6940101,0xfb840108,0xa6950101,
	0xfb840108,0xa6960101,0xfb840108,0xa6970101,0xfb840108,0xa6980101,0xfb840108,0xa6990101,
	0xfb840108,0xa69a0101,0xfb840108,0xa69b0101,0xfb840108,0xa69c0101,0xfb840108,0xa69d0101,
	0xfb840108,0xa69e0101,0xfb840108,0xa69f0101,0xfb840108,0xa6a00101,0xfb840108,0xa6a10101,
	0xfb840108,0xa6a20101,0xfb840108,0xa6a30101,0xfb840108,0xa6a40101,0xfb840108,0xa6a50101,
	0xfb840108,0xa6a60101,0xfb840108,0xa6a70101,0xfb840108,0xa6a80101,0xfb840108,0xa6a90101,
	0xfb840108,0xa6aa0101,0xfb840108,0xa6ab0101,0xfb840108,0xa6ac0101,0xfb840108,0xa6ad0101,
	0xfb840108,0xa6ae0101,0xfb840108,0xa6af0101,0xfb840108,0xa6b00101,0xfb840108,0xa6b10101,
	0xfb840108,0xa6b20101,0xfb840108,0xa6b30101,0xfb840108,0xa6b40101,0xfb840108,0xa6b50101,
	0xfb840108,0xa6b60101,0xfb840108,0xa6b70101,0xfb840108,0xa6b80101,0xfb840108,0xa6b90101,
	0xfb840108,0xa6ba0101,0xfb840108,0xa6bb0101,0xfb840108,0xa6bc0101,0xfb840108,0xa6bd0101,
	0xfb840108,0xa6be0101,0xfb840108,0xa6bf0101,0xfb840108,0xa6c00101,0xfb840108,0xa6c10101,
	0xfb840108,0xa6c20101,0xfb840108,0xa6c30101,0xfb840108,0xa6c40101,0xfb840108,0xa6c50101,
	0xfb840108,0xa6c60101,0xfb840108,0xa6c70101,0xfb840108,0xa6c80101,0xfb840108,0xa6c90101,
	0xfb840108,0xa6ca0101,0xfb840108,0xa6cb0101,0xfb840108,0xa6cc0101,0xfb840108,0xa6cd0101,
	0xfb840108,0xa6ce0101,0xfb840108,0xa6cf0101,0xfb840108,0xa6d00101,0xfb840108,0xa6d10101,
	0xfb840108,0xa6d20101,0xfb840108,0xa6d30101,0xfb840108,0xa6d40101,0xfb840108,0xa6d50101,
	0xfb840108,0xa6d60101,0xfb840108,0xa6d70101,0xfb840108,0xa6d80101,0xfb840108,0xa6d90101,
	0xfb840108,0xa6da0101,0xfb840108,0xa6db0101,0xfb840108,0xa6dc0101,0xfb840108,0xa6dd0101,
	0xfb840108,0xa6de0101,0xfb840108,0xa6df0101,0xfb840108,0xa6e00101,0xfb840108,0xa6e10101,
	0xfb840108,0xa6e20101,0xfb840108,0xa6e30101,0xfb840108,0xa6e40101,0xfb840108,0xa6e50101,
	0xfb840108,0xa6e60101,0xfb840108,0xa6e70101,0xfb840108,0xa6e80101,0xfb840108,0xa6e90101,
	0xfb840108,0xa6ea0101,0xfb840108,0xa6eb0101,0xfb840108,0xa6ec0101,0xfb840108,0xa6ed0101,
	0xfb840108,0xa6ee0101,0xfb840108,0xa6ef0101,0xfb840108,0xa6f00101,0xfb840108,0xa6f10101,
	0xfb840108,0xa6f20101,0xfb840108,0xa6f30101,0xfb840108,0xa6f40101,0xfb840108,0xa6f50101,
	0xfb840108,0xa6f60101,0xfb840108,0xa6f70101,0xfb840108,0xa6f80101,0xfb840108,0xa6f90101,
	0xfb840108,0xa6fa0101,0xfb840108,0xa6fb0101,0xfb840108,0xa6fc0101,0xfb840108,0xa6fd0101,
	0xfb840108,0xa6fe0101,0xfb840108,0xa6ff0101,0xfb840108,0xa7000101,0xfb840108,0xa7010101,
	0xfb840108,0xa7020101,0xfb840108,0xa7030101,0xfb840108,0xa7040101,0xfb840108,0xa7050101,
	0xfb840108,0xa7060101,0xfb840108,0xa7070101,0xfb840108,0xa7080101,0xfb840108,0xa7090101,
	0xfb840108,0xa70a0101,0xfb840108,0xa70b0101,0xfb840108,0xa70c0101,0xfb840108,0xa70d0101,
	0xfb840108,0xa70e0101,0xfb840108,0xa70f0101,0xfb840108,0xa7100101,0xfb840108,0xa7110101,
	0xfb840108,0xa7120101,0xfb840108,0xa7130101,0xfb840108,0xa7140101,0xfb840108,0xa7150101,
	0xfb840108,0xa7160101,0xfb840108,0xa7170101,0xfb840108,0xa7180101,0xfb840108,0xa7190101,
	0xfb840108,0xa71a0101,0xfb840108,0xa71b0101,0xfb840108,0xa71c0101,0xfb840108,0xa71d0101,
	0xfb840108,0xa71e0101,0xfb840108,0xa71f0101,0xfb840108,0xa7200101,0xfb840108,0xa7210101,
	0xfb840108,0xa7220101,0xfb840108,0xa7230101,0xfb840108,0xa7240101,0xfb840108,0xa7250101,
	0xfb840108,0xa7260101,0xfb840108,0xa7270101,0xfb840108,0xa7280101,0xfb840108,0xa7290101,
	0xfb840108,0xa72a0101,0xfb840108,0xa72b0101,0xfb840108,0xa72c0101,0xfb840108,0xa72d0101,
	0xfb840108,0xa72e0101,0xfb840108,0xa72f0101,0xfb840108,0xa7300101,0xfb840108,0xa7310101,
	0xfb840108,0xa7320101,0xfb840108,0xa7330101,0xfb840108,0xa7340101,0xfb840108,0xa7350101,
	0xfb840108,0xa7360101,0xfb840108,0xa7370101,0xfb840108,0xa7380101,0xfb840108,0xa7390101,
	0xfb840108,0xa73a0101,0xfb840108,0xa73b0101,0xfb840108,0xa73c0101,0xfb840108,0xa73d0101,
	0xfb840108,0xa73e0101,0xfb840108,0xa73f0101,0xfb840108,0xa7400101,0xfb840108,0xa7410101,
	0xfb840108,0xa7420101,0xfb840108,0xa7430101,0xfb840108,0xa7440101,0xfb840108,0xa7450101,
	0xfb840108,0xa7460101,0xfb840108,0xa7470101,0xfb840108,0xa7480101,0xfb840108,0xa7490101,
	0xfb840108,0xa74a0101,0xfb840108,0xa74b0101,0xfb840108,0xa74c0101,0xfb840108,0xa74d0101,
	0xfb840108,0xa74e0101,0xfb840108,0xa74f0101,0xfb840108,0xa7500101,0xfb840108,0xa7510101,
	0xfb840108,0xa7520101,0xfb840108,0xa7530101,0xfb840108,0xa7540101,0xfb840108,0xa7550101,
	0xfb840108,0xa7560101,0xfb840108,0xa7570101,0xfb840108,0xa7580101,0xfb840108,0xa7590101,
	0xfb840108,0xa75a0101,0xfb840108,0xa75b0101,0xfb840108,0xa75c0101,0xfb840108,0xa75d0101,
	0xfb840108,0xa75e0101,0xfb840108,0xa75f0101,0xfb840108,0xa7600101,0xfb840108,0xa7610101,
	0xfb840108,0xa7620101,0xfb840108,0xa7630101,0xfb840108,0xa7640101,0xfb840108,0xa7650101,
	0xfb840108,0xa7660101,0xfb840108,0xa7670101,0xfb840108,0xa7680101,0xfb840108,0xa7690101,
	0xfb840108,0xa76a0101,0xfb840108,0xa76b0101,0xfb840108,0xa76c0101,0xfb840108,0xa76d0101,
	0xfb840108,0xa76e0101,0xfb840108,0xa76f0101,0xfb840108,0xa7700101,0xfb840108,0xa7710101,
	0xfb840108,0xa7720101,0xfb840108,0xa7730101,0xfb840108,0xa7740101,0xfb840108,0xa7750101,
	0xfb840108,0xa7760101,0xfb840108,0xa7770101,0xfb840108,0xa7780101,0xfb840108,0xa7790101,
	0xfb840108,0xa77a0101,0xfb840108,0xa77b0101,0xfb840108,0xa77c0101,0xfb840108,0xa77d0101,
	0xfb840108,0xa77e0101,0xfb840108,0xa77f0101,0xfb840108,0xa7800101,0xfb840108,0xa7810101,
	0xfb840108,0xa7820101,0xfb840108,0xa7830101,0xfb840108,0xa7840101,0xfb840108,0xa7850101,
	0xfb840108,0xa7860101,0xfb840108,0xa7870101,0xfb840108,0xa7880101,0xfb840108,0xa7890101,
	0xfb840108,0xa78a0101,0xfb840108,0xa78b0101,0xfb840108,0xa78c0101,0xfb840108,0xa78d0101,
	0xfb840108,0xa78e0101,0xfb840108,0xa78f0101,0xfb840108,0xa7900101,0xfb840108,0xa7910101,
	0xfb840108,0xa7920101,0xfb840108,0xa7930101,0xfb840108,0xa7940101,0xfb840108,0xa7950101,
	0xfb840108,0xa7960101,0xfb840108,0xa7970101,0xfb840108,0xa7980101,0xfb840108,0xa7990101,
	0xfb840108,0xa79a0101,0xfb840108,0xa79b0101,0xfb840108,0xa79c0101,0xfb840108,0xa79d0101,
	0xfb840108,0xa79e0101,0xfb840108,0xa79f0101,0xfb840108,0xa7a00101,0xfb840108,0xa7a10101,
	0xfb840108,0xa7a20101,0xfb840108,0xa7a30101,0xfb840108,0xa7a40101,0xfb840108,0xa7a50101,
	0xfb840108,0xa7a60101,0xfb840108,0xa7a70101,0xfb840108,0xa7a80101,0xfb840108,0xa7a90101,
	0xfb840108,0xa7aa0101,0xfb840108,0xa7ab0101,0xfb840108,0xa7ac0101,0xfb840108,0xa7ad0101,
	0xfb840108,0xa7ae0101,0xfb840108,0xa7af0101,0xfb840108,0xa7b00101,0xfb840108,0xa7b10101,
	0xfb840108,0xa7b20101,0xfb840108,0xa7b30101,0xfb840108,0xa7b40101,0xfb840108,0xa7b50101,
	0xfb840108,0xa7b60101,0xfb840108,0xa7b70101,0xfb840108,0xa7b80101,0xfb840108,0xa7b90101,
	0xfb840108,0xa7ba0101,0xfb840108,0xa7bb0101,0xfb840108,0xa7bc0101,0xfb840108,0xa7bd0101,
	0xfb840108,0xa7be0101,0xfb840108,0xa7bf0101,0xfb840108,0xa7c00101,0xfb840108,0xa7c10101,
	0xfb840108,0xa7c20101,0xfb840108,0xa7c30101,0xfb840108,0xa7c40101,0xfb840108,0xa7c50101,
	0xfb840108,0xa7c60101,0xfb840108,0xa7c70101,0xfb840108,0xa7c80101,0xfb840108,0xa7c90101,
	0xfb840108,0xa7ca0101,0xfb840108,0xa7cb0101,0xfb840108,0xa7cc0101,0xfb840108,0xa7cd0101,
	0xfb840108,0xa7ce0101,0xfb840108,0xa7cf0101,0xfb840108,0xa7d00101,0xfb840108,0xa7d10101,
	0xfb840108,0xa7d20101,0xfb840108,0xa7d30101,0xfb840108,0xa7d40101,0xfb840108,0xa7d50101,
	0xfb840108,0xa7d60101,0xfb840108,0xa7d70101,0xfb840108,0xa7d80101,0xfb840108,0xa7d90101,
	0xfb840108,0xa7da0101,0xfb840108,0xa7db0101,0xfb840108,0xa7dc0101,0xfb840108,0xa7dd0101,
	0xfb840108,0xa7de0101,0xfb840108,0xa7df0101,0xfb840108,0xa7e00101,0xfb840108,0xa7e10101,
	0xfb840108,0xa7e20101,0xfb840108,0xa7e30101,0xfb840108,0xa7e40101,0xfb840108,0xa7e50101,
	0xfb840108,0xa7e60101,0xfb840108,0xa7e70101,0xfb840108,0xa7e80101,0xfb840108,0xa7e90101,
	0xfb840108,0xa7ea0101,0xfb840108,0xa7eb0101,0xfb840108,0xa7ec0101,0xfb840108,0xa7ed0101,
	0xfb840108,0xa7ee0101,0xfb840108,0xa7ef0101,0xfb840108,0xa7f00101,0xfb840108,0xa7f10101,
	0xfb840108,0xa7f20101,0xfb840108,0xa7f30101,0xfb840108,0xa7f40101,0xfb840108,0xa7f50101,
	0xfb840108,0xa7f60101,0xfb840108,0xa7f70101,0xfb840108,0xa7f80101,0xfb840108,0xa7f90101,
	0xfb840108,0xa7fa0101,0xfb840108,0xa7fb0101,0xfb840108,0xa7fc0101,0xfb840108,0xa7fd0101,
	0xfb840108,0xa7fe0101,0xfb840108,0xa7ff0101,0xfb840108,0xa8000101,0xfb840108,0xa8010101,
	0xfb840108,0xa8020101,0xfb840108,0xa8030101,0xfb840108,0xa8040101,0xfb840108,0xa8050101,
	0xfb840108,0xa8060101,0xfb840108,0xa8070101,0xfb840108,0xa8080101,0xfb840108,0xa8090101,
	0xfb840108,0xa80a0101,0xfb840108,0xa80b0101,0xfb840108,0xa80c0101,0xfb840108,0xa80d0101,
	0xfb840108,0xa80e0101,0xfb840108,0xa80f0101,0xfb840108,0xa8100101,0xfb840108,0xa8110101,
	0xfb840108,0xa8120101,0xfb840108,0xa8130101,0xfb840108,0xa8140101,0xfb840108,0xa8150101,
	0xfb840108,0xa8160101,0xfb840108,0xa8170101,0xfb840108,0xa8180101,0xfb840108,0xa8190101,
	0xfb840108,0xa81a0101,0xfb840108,0xa81b0101,0xfb840108,0xa81c0101,0xfb840108,0xa81d0101,
	0xfb840108,0xa81e0101,0xfb840108,0xa81f0101,0xfb840108,0xa8200101,0xfb840108,0xa8210101,
	0xfb840108,0xa8220101,0xfb840108,0xa8230101,0xfb840108,0xa8240101,0xfb840108,0xa8250101,
	0xfb840108,0xa8260101,0xfb840108,0xa8270101,0xfb840108,0xa8280101,0xfb840108,0xa8290101,
	0xfb840108,0xa82a0101,0xfb840108,0xa82b0101,0xfb840108,0xa82c0101,0xfb840108,0xa82d0101,
	0xfb840108,0xa82e0101,0xfb840108,0xa82f0101,0xfb840108,0xa8300101,0xfb840108,0xa8310101,
	0xfb840108,0xa8320101,0xfb840108,0xa8330101,0xfb840108,0xa8340101,0xfb840108,0xa8350101,
	0xfb840108,0xa8360101,0xfb840108,0xa8370101,0xfb840108,0xa8380101,0xfb840108,0xa8390101,
	0xfb840108,0xa83a0101,0xfb840108,0xa83b0101,0xfb840108,0xa83c0101,0xfb840108,0xa83d0101,
	0xfb840108,0xa83e0101,0xfb840108,0xa83f0101,0xfb840108,0xa8400101,0xfb840108,0xa8410101,
	0xfb840108,0xa8420101,0xfb840108,0xa8430101,0xfb840108,0xa8440101,0xfb840108,0xa8450101,
	0xfb840108,0xa8460101,0xfb840108,0xa8470101,0xfb840108,0xa8480101,0xfb840108,0xa8490101,
	0xfb840108,0xa84a0101,0xfb840108,0xa84b0101,0xfb840108,0xa84c0101,0xfb840108,0xa84d0101,
	0xfb840108,0xa84e0101,0xfb840108,0xa84f0101,0xfb840108,0xa8500101,0xfb840108,0xa8510101,
	0xfb840108,0xa8520101,0xfb840108,0xa8530101,0xfb840108,0xa8540101,0xfb840108,0xa8550101,
	0xfb840108,0xa8560101,0xfb840108,0xa8570101,0xfb840108,0xa8580101,0xfb840108,0xa8590101,
	0xfb840108,0xa85a0101,0xfb840108,0xa85b0101,0xfb840108,0xa85c0101,0xfb840108,0xa85d0101,
	0xfb840108,0xa85e0101,0xfb840108,0xa85f0101,0xfb840108,0xa8600101,0xfb840108,0xa8610101,
	0xfb840108,0xa8620101,0xfb840108,0xa8630101,0xfb840108,0xa8640101,0xfb840108,0xa8650101,
	0xfb840108,0xa8660101,0xfb840108,0xa8670101,0xfb840108,0xa8680101,0xfb840108,0xa8690101,
	0xfb840108,0xa86a0101,0xfb840108,0xa86b0101,0xfb840108,0xa86c0101,0xfb840108,0xa86d0101,
	0xfb840108,0xa86e0101,0xfb840108,0xa86f0101,0xfb840108,0xa8700101,0xfb840108,0xa8710101,
	0xfb840108,0xa8720101,0xfb840108,0xa8730101,0xfb840108,0xa8740101,0xfb840108,0xa8750101,
	0xfb840108,0xa8760101,0xfb840108,0xa8770101,0xfb840108,0xa8780101,0xfb840108,0xa8790101,
	0xfb840108,0xa87a0101,0xfb840108,0xa87b0101,0xfb840108,0xa87c0101,0xfb840108,0xa87d0101,
	0xfb840108,0xa87e0101,0xfb840108,0xa87f0101,0xfb840108,0xa8800101,0xfb840108,0xa8810101,
	0xfb840108,0xa8820101,0xfb840108,0xa8830101,0xfb840108,0xa8840101,0xfb840108,0xa8850101,
	0xfb840108,0xa8860101,0xfb840108,0xa8870101,0xfb840108,0xa8880101,0xfb840108,0xa8890101,
	0xfb840108,0xa88a0101,0xfb840108,0xa88b0101,0xfb840108,0xa88c0101,0xfb840108,0xa88d0101,
	0xfb840108,0xa88e0101,0xfb840108,0xa88f0101,0xfb840108,0xa8900101,0xfb840108,0xa8910101,
	0xfb840108,0xa8920101,0xfb840108,0xa8930101,0xfb840108,0xa8940101,0xfb840108,0xa8950101,
	0xfb840108,0xa8960101,0xfb840108,0xa8970101,0xfb840108,0xa8980101,0xfb840108,0xa8990101,
	0xfb840108,0xa89a0101,0xfb840108,0xa89b0101,0xfb840108,0xa89c0101,0xfb840108,0xa89d0101,
	0xfb840108,0xa89e0101,0xfb840108,0xa89f0101,0xfb840108,0xa8a00101,0xfb840108,0xa8a10101,
	0xfb840108,0xa8a20101,0xfb840108,0xa8a30101,0xfb840108,0xa8a40101,0xfb840108,0xa8a50101,
	0xfb840108,0xa8a60101,0xfb840108,0xa8a70101,0xfb840108,0xa8a80101,0xfb840108,0xa8a90101,
	0xfb840108,0xa8aa0101,0xfb840108,0xa8ab0101,0xfb840108,0xa8ac0101,0xfb840108,0xa8ad0101,
	0xfb840108,0xa8ae0101,0xfb840108,0xa8af0101,0xfb840108,0xa8b00101,0xfb840108,0xa8b10101,
	0xfb840108,0xa8b20101,0xfb840108,0xa8b30101,0xfb840108,0xa8b40101,0xfb840108,0xa8b50101,
	0xfb840108,0xa8b60101,0xfb840108,0xa8b70101,0xfb840108,0xa8b80101,0xfb840108,0xa8b90101,
	0xfb840108,0xa8ba0101,0xfb840108,0xa8bb0101,0xfb840108,0xa8bc0101,0xfb840108,0xa8bd0101,
	0xfb840108,0xa8be0101,0xfb840108,0xa8bf0101,0xfb840108,0xa8c00101,0xfb840108,0xa8c10101,
	0xfb840108,0xa8c20101,0xfb840108,0xa8c30101,0xfb840108,0xa8c40101,0xfb840108,0xa8c50101,
	0xfb840108,0xa8c60101,0xfb840108,0xa8c70101,0xfb840108,0xa8c80101,0xfb840108,0xa8c90101,
	0xfb840108,0xa8ca0101,0xfb840108,0xa8cb0101,0xfb840108,0xa8cc0101,0xfb840108,0xa8cd0101,
	0xfb840108,0xa8ce0101,0xfb840108,0xa8cf0101,0xfb840108,0xa8d00101,0xfb840108,0xa8d10101,
	0xfb840108,0xa8d20101,0xfb840108,0xa8d30101,0xfb840108,0xa8d40101,0xfb840108,0xa8d50101,
	0xfb840108,0xa8d60101,0xfb840108,0xa8d70101,0xfb840108,0xa8d80101,0xfb840108,0xa8d90101,
	0xfb840108,0xa8da0101,0xfb840108,0xa8db0101,0xfb840108,0xa8dc0101,0xfb840108,0xa8dd0101,
	0xfb840108,0xa8de0101,0xfb840108,0xa8df0101,0xfb840108,0xa8e00101,0xfb840108,0xa8e10101,
	0xfb840108,0xa8e20101,0xfb840108,0xa8e30101,0xfb840108,0xa8e40101,0xfb840108,0xa8e50101,
	0xfb840108,0xa8e60101,0xfb840108,0xa8e70101,0xfb840108,0xa8e80101,0xfb840108,0xa8e90101,
	0xfb840108,0xa8ea0101,0xfb840108,0xa8eb0101,0xfb840108,0xa8ec0101,0xfb840108,0xa8ed0101,
	0xfb840108,0xa8ee0101,0xfb840108,0xa8ef0101,0xfb840108,0xa8f00101,0xfb840108,0xa8f10101,
	0xfb840108,0xa8f20101,0xfb840108,0xa8f30101,0xfb840108,0xa8f40101,0xfb840108,0xa8f50101,
	0xfb840108,0xa8f60101,0xfb840108,0xa8f70101,0xfb840108,0xa8f80101,0xfb840108,0xa8f90101,
	0xfb840108,0xa8fa0101,0xfb840108,0xa8fb0101,0xfb840108,0xa8fc0101,0xfb840108,0xa8fd0101,
	0xfb840108,0xa8fe0101,0xfb840108,0xa8ff0101,0xfb840108,0xa9000101,0xfb840108,0xa9010101,
	0xfb840108,0xa9020101,0xfb840108,0xa9030101,0xfb840108,0xa9040101,0xfb840108,0xa9050101,
	0xfb840108,0xa9060101,0xfb840108,0xa9070101,0xfb840108,0xa9080101,0xfb840108,0xa9090101,
	0xfb840108,0xa90a0101,0xfb840108,0xa90b0101,0xfb840108,0xa90c0101,0xfb840108,0xa90d0101,
	0xfb840108,0xa90e0101,0xfb840108,0xa90f0101,0xfb840108,0xa9100101,0xfb840108,0xa9110101,
	0xfb840108,0xa9120101,0xfb840108,0xa9130101,0xfb840108,0xa9140101,0xfb840108,0xa9150101,
	0xfb840108,0xa9160101,0xfb840108,0xa9170101,0xfb840108,0xa9180101,0xfb840108,0xa9190101,
	0xfb840108,0xa91a0101,0xfb840108,0xa91b0101,0xfb840108,0xa91c0101,0xfb840108,0xa91d0101,
	0xfb840108,0xa91e0101,0xfb840108,0xa91f0101,0xfb840108,0xa9200101,0xfb840108,0xa9210101,
	0xfb840108,0xa9220101,0xfb840108,0xa9230101,0xfb840108,0xa9240101,0xfb840108,0xa9250101,
	0xfb840108,0xa9260101,0xfb840108,0xa9270101,0xfb840108,0xa9280101,0xfb840108,0xa9290101,
	0xfb840108,0xa92a0101,0xfb840108,0xa92b0101,0xfb840108,0xa92c0101,0xfb840108,0xa92d0101,
	0xfb840108,0xa92e0101,0xfb840108,0xa92f0101,0xfb840108,0xa9300101,0xfb840108,0xa9310101,
	0xfb840108,0xa9320101,0xfb840108,0xa9330101,0xfb840108,0xa9340101,0xfb840108,0xa9350101,
	0xfb840108,0xa9360101,0xfb840108,0xa9370101,0xfb840108,0xa9380101,0xfb840108,0xa9390101,
	0xfb840108,0xa93a0101,0xfb840108,0xa93b0101,0xfb840108,0xa93c0101,0xfb840108,0xa93d0101,
	0xfb840108,0xa93e0101,0xfb840108,0xa93f0101,0xfb840108,0xa9400101,0xfb840108,0xa9410101,
	0xfb840108,0xa9420101,0xfb840108,0xa9430101,0xfb840108,0xa9440101,0xfb840108,0xa9450101,
	0xfb840108,0xa9460101,0xfb840108,0xa9470101,0xfb840108,0xa9480101,0xfb840108,0xa9490101,
	0xfb840108,0xa94a0101,0xfb840108,0xa94b0101,0xfb840108,0xa94c0101,0xfb840108,0xa94d0101,
	0xfb840108,0xa94e0101,0xfb840108,0xa94f0101,0xfb840108,0xa9500101,0xfb840108,0xa9510101,
	0xfb840108,0xa9520101,0xfb840108,0xa9530101,0xfb840108,0xa9540101,0xfb840108,0xa9550101,
	0xfb840108,0xa9560101,0xfb840108,0xa9570101,0xfb840108,0xa9580101,0xfb840108,0xa9590101,
	0xfb840108,0xa95a0101,0xfb840108,0xa95b0101,0xfb840108,0xa95c0101,0xfb840108,0xa95d0101,
	0xfb840108,0xa95e0101,0xfb840108,0xa95f0101,0xfb840108,0xa9600101,0xfb840108,0xa9610101,
	0xfb840108,0xa9620101,0xfb840108,0xa9630101,0xfb840108,0xa9640101,0xfb840108,0xa9650101,
	0xfb840108,0xa9660101,0xfb840108,0xa9670101,0xfb840108,0xa9680101,0xfb840108,0xa9690101,
	0xfb840108,0xa96a0101,0xfb840108,0xa96b0101,0xfb840108,0xa96c0101,0xfb840108,0xa96d0101,
	0xfb840108,0xa96e0101,0xfb840108,0xa96f0101,0xfb840108,0xa9700101,0xfb840108,0xa9710101,
	0xfb840108,0xa9720101,0xfb840108,0xa9730101,0xfb840108,0xa9740101,0xfb840108,0xa9750101,
	0xfb840108,0xa9760101,0xfb840108,0xa9770101,0xfb840108,0xa9780101,0xfb840108,0xa9790101,
	0xfb840108,0xa97a0101,0xfb840108,0xa97b0101,0xfb840108,0xa97c0101,0xfb840108,0xa97d0101,
	0xfb840108,0xa97e0101,0xfb840108,0xa97f0101,0xfb840108,0xa9800101,0xfb840108,0xa9810101,
	0xfb840108,0xa9820101,0xfb840108,0xa9830101,0xfb840108,0xa9840101,0xfb840108,0xa9850101,
	0xfb840108,0xa9860101,0xfb840108,0xa9870101,0xfb840108,0xa9880101,0xfb840108,0xa9890101,
	0xfb840108,0xa98a0101,0xfb840108,0xa98b0101,0xfb840108,0xa98c0101,0xfb840108,0xa98d0101,
	0xfb840108,0xa98e0101,0xfb840108,0xa98f0101,0xfb840108,0xa9900101,0xfb840108,0xa9910101,
	0xfb840108,0xa9920101,0xfb840108,0xa9930101,0xfb840108,0xa9940101,0xfb840108,0xa9950101,
	0xfb840108,0xa9960101,0xfb840108,0xa9970101,0xfb840108,0xa9980101,0xfb840108,0xa9990101,
	0xfb840108,0xa99a0101,0xfb840108,0xa99b0101,0xfb840108,0xa99c0101,0xfb840108,0xa99d0101,
	0xfb840108,0xa99e0101,0xfb840108,0xa99f0101,0xfb840108,0xa9a00101,0xfb840108,0xa9a10101,
	0xfb840108,0xa9a20101,0xfb840108,0xa9a30101,0xfb840108,0xa9a40101,0xfb840108,0xa9a50101,
	0xfb840108,0xa9a60101,0xfb840108,0xa9a70101,0xfb840108,0xa9a80101,0xfb840108,0xa9a90101,
	0xfb840108,0xa9aa0101,0xfb840108,0xa9ab0101,0xfb840108,0xa9ac0101,0xfb840108,0xa9ad0101,
	0xfb840108,0xa9ae0101,0xfb840108,0xa9af0101,0xfb840108,0xa9b00101,0xfb840108,0xa9b10101,
	0xfb840108,0xa9b20101,0xfb840108,0xa9b30101,0xfb840108,0xa9b40101,0xfb840108,0xa9b50101,
	0xfb840108,0xa9b60101,0xfb840108,0xa9b70101,0xfb840108,0xa9b80101,0xfb840108,0xa9b90101,
	0xfb840108,0xa9ba0101,0xfb840108,0xa9bb0101,0xfb840108,0xa9bc0101,0xfb840108,0xa9bd0101,
	0xfb840108,0xa9be0101,0xfb840108,0xa9bf0101,0xfb840108,0xa9c00101,0xfb840108,0xa9c10101,
	0xfb840108,0xa9c20101,0xfb840108,0xa9c30101,0xfb840108,0xa9c40101,0xfb840108,0xa9c50101,
	0xfb840108,0xa9c60101,0xfb840108,0xa9c70101,0xfb840108,0xa9c80101,0xfb840108,0xa9c90101,
	0xfb840108,0xa9ca0101,0xfb840108,0xa9cb0101,0xfb840108,0xa9cc0101,0xfb840108,0xa9cd0101,
	0xfb840108,0xa9ce0101,0xfb840108,0xa9cf0101,0xfb840108,0xa9d00101,0xfb840108,0xa9d10101,
	0xfb840108,0xa9d20101,0xfb840108,0xa9d30101,0xfb840108,0xa9d40101,0xfb840108,0xa9d50101,
	0xfb840108,0xa9d60101,0xfb840108,0xa9d70101,0xfb840108,0xa9d80101,0xfb840108,0xa9d90101,
	0xfb840108,0xa9da0101,0xfb840108,0xa9db0101,0xfb840108,0xa9dc0101,0xfb840108,0xa9dd0101,
	0xfb840108,0xa9de0101,0xfb840108,0xa9df0101,0xfb840108,0xa9e00101,0xfb840108,0xa9e10101,
	0xfb840108,0xa9e20101,0xfb840108,0xa9e30101,0xfb840108,0xa9e40101,0xfb840108,0xa9e50101,
	0xfb840108,0xa9e60101,0xfb840108,0xa9e70101,0xfb840108,0xa9e80101,0xfb840108,0xa9e90101,
	0xfb840108,0xa9ea0101,0xfb840108,0xa9eb0101,0xfb840108,0xa9ec0101,0xfb840108,0xa9ed0101,
	0xfb840108,0xa9ee0101,0xfb840108,0xa9ef0101,0xfb840108,0xa9f00101,0xfb840108,0xa9f10101,
	0xfb840108,0xa9f20101,0xfb840108,0xa9f30101,0xfb840108,0xa9f40101,0xfb840108,0xa9f50101,
	0xfb840108,0xa9f60101,0xfb840108,0xa9f70101,0xfb840108,0xa9f80101,0xfb840108,0xa9f90101,
	0xfb840108,0xa9fa0101,0xfb840108,0xa9fb0101,0xfb840108,0xa9fc0101,0xfb840108,0xa9fd0101,
	0xfb840108,0xa9fe0101,0xfb840108,0xa9ff0101,0xfb840108,0xaa000101,0xfb840108,0xaa010101,
	0xfb840108,0xaa020101,0xfb840108,0xaa030101,0xfb840108,0xaa040101,0xfb840108,0xaa050101,
	0xfb840108,0xaa060101,0xfb840108,0xaa070101,0xfb840108,0xaa080101,0xfb840108,0xaa090101,
	0xfb840108,0xaa0a0101,0xfb840108,0xaa0b0101,0xfb840108,0xaa0c0101,0xfb840108,0xaa0d0101,
	0xfb840108,0xaa0e0101,0xfb840108,0xaa0f0101,0xfb840108,0xaa100101,0xfb840108,0xaa110101,
	0xfb840108,0xaa120101,0xfb840108,0xaa130101,0xfb840108,0xaa140101,0xfb840108,0xaa150101,
	0xfb840108,0xaa160101,0xfb840108,0xaa170101,0xfb840108,0xaa180101,0xfb840108,0xaa190101,
	0xfb840108,0xaa1a0101,0xfb840108,0xaa1b0101,0xfb840108,0xaa1c0101,0xfb840108,0xaa1d0101,
	0xfb840108,0xaa1e0101,0xfb840108,0xaa1f0101,0xfb840108,0xaa200101,0xfb840108,0xaa210101,
	0xfb840108,0xaa220101,0xfb840108,0xaa230101,0xfb840108,0xaa240101,0xfb840108,0xaa250101,
	0xfb840108,0xaa260101,0xfb840108,0xaa270101,0xfb840108,0xaa280101,0xfb840108,0xaa290101,
	0xfb840108,0xaa2a0101,0xfb840108,0xaa2b0101,0xfb840108,0xaa2c0101,0xfb840108,0xaa2d0101,
	0xfb840108,0xaa2e0101,0xfb840108,0xaa2f0101,0xfb840108,0xaa300101,0xfb840108,0xaa310101,
	0xfb840108,0xaa320101,0xfb840108,0xaa330101,0xfb840108,0xaa340101,0xfb840108,0xaa350101,
	0xfb840108,0xaa360101,0xfb840108,0xaa370101,0xfb840108,0xaa380101,0xfb840108,0xaa390101,
	0xfb840108,0xaa3a0101,0xfb840108,0xaa3b0101,0xfb840108,0xaa3c0101,0xfb840108,0xaa3d0101,
	0xfb840108,0xaa3e0101,0xfb840108,0xaa3f0101,0xfb840108,0xaa400101,0xfb840108,0xaa410101,
	0xfb840108,0xaa420101,0xfb840108,0xaa430101,0xfb840108,0xaa440101,0xfb840108,0xaa450101,
	0xfb840108,0xaa460101,0xfb840108,0xaa470101,0xfb840108,0xaa480101,0xfb840108,0xaa490101,
	0xfb840108,0xaa4a0101,0xfb840108,0xaa4b0101,0xfb840108,0xaa4c0101,0xfb840108,0xaa4d0101,
	0xfb840108,0xaa4e0101,0xfb840108,0xaa4f0101,0xfb840108,0xaa500101,0xfb840108,0xaa510101,
	0xfb840108,0xaa520101,0xfb840108,0xaa530101,0xfb840108,0xaa540101,0xfb840108,0xaa550101,
	0xfb840108,0xaa560101,0xfb840108,0xaa570101,0xfb840108,0xaa580101,0xfb840108,0xaa590101,
	0xfb840108,0xaa5a0101,0xfb840108,0xaa5b0101,0xfb840108,0xaa5c0101,0xfb840108,0xaa5d0101,
	0xfb840108,0xaa5e0101,0xfb840108,0xaa5f0101,0xfb840108,0xaa600101,0xfb840108,0xaa610101,
	0xfb840108,0xaa620101,0xfb840108,0xaa630101,0xfb840108,0xaa640101,0xfb840108,0xaa650101,
	0xfb840108,0xaa660101,0xfb840108,0xaa670101,0xfb840108,0xaa680101,0xfb840108,0xaa690101,
	0xfb840108,0xaa6a0101,0xfb840108,0xaa6b0101,0xfb840108,0xaa6c0101,0xfb840108,0xaa6d0101,
	0xfb840108,0xaa6e0101,0xfb840108,0xaa6f0101,0xfb840108,0xaa700101,0xfb840108,0xaa710101,
	0xfb840108,0xaa720101,0xfb840108,0xaa730101,0xfb840108,0xaa740101,0xfb840108,0xaa750101,
	0xfb840108,0xaa760101,0xfb840108,0xaa770101,0xfb840108,0xaa780101,0xfb840108,0xaa790101,
	0xfb840108,0xaa7a0101,0xfb840108,0xaa7b0101,0xfb840108,0xaa7c0101,0xfb840108,0xaa7d0101,
	0xfb840108,0xaa7e0101,0xfb840108,0xaa7f0101,0xfb840108,0xaa800101,0xfb840108,0xaa810101,
	0xfb840108,0xaa820101,0xfb840108,0xaa830101,0xfb840108,0xaa840101,0xfb840108,0xaa850101,
	0xfb840108,0xaa860101,0xfb840108,0xaa870101,0xfb840108,0xaa880101,0xfb840108,0xaa890101,
	0xfb840108,0xaa8a0101,0xfb840108,0xaa8b0101,0xfb840108,0xaa8c0101,0xfb840108,0xaa8d0101,
	0xfb840108,0xaa8e0101,0xfb840108,0xaa8f0101,0xfb840108,0xaa900101,0xfb840108,0xaa910101,
	0xfb840108,0xaa920101,0xfb840108,0xaa930101,0xfb840108,0xaa940101,0xfb840108,0xaa950101,
	0xfb840108,0xaa960101,0xfb840108,0xaa970101,0xfb840108,0xaa980101,0xfb840108,0xaa990101,
	0xfb840108,0xaa9a0101,0xfb840108,0xaa9b0101,0xfb840108,0xaa9c0101,0xfb840108,0xaa9d0101,
	0xfb840108,0xaa9e0101,0xfb840108,0xaa9f0101,0xfb840108,0xaaa00101,0xfb840108,0xaaa10101,
	0xfb840108,0xaaa20101,0xfb840108,0xaaa30101,0xfb840108,0xaaa40101,0xfb840108,0xaaa50101,
	0xfb840108,0xaaa60101,0xfb840108,0xaaa70101,0xfb840108,0xaaa80101,0xfb840108,0xaaa90101,
	0xfb840108,0xaaaa0101,0xfb840108,0xaaab0101,0xfb840108,0xaaac0101,0xfb840108,0xaaad0101,
	0xfb840108,0xaaae0101,0xfb840108,0xaaaf0101,0xfb840108,0xaab00101,0xfb840108,0xaab10101,
	0xfb840108,0xaab20101,0xfb840108,0xaab30101,0xfb840108,0xaab40101,0xfb840108,0xaab50101,
	0xfb840108,0xaab60101,0xfb840108,0xaab70101,0xfb840108,0xaab80101,0xfb840108,0xaab90101,
	0xfb840108,0xaaba0101,0xfb840108,0xaabb0101,0xfb840108,0xaabc0101,0xfb840108,0xaabd0101,
	0xfb840108,0xaabe0101,0xfb840108,0xaabf0101,0xfb840108,0xaac00101,0xfb840108,0xaac10101,
	0xfb840108,0xaac20101,0xfb840108,0xaac30101,0xfb840108,0xaac40101,0xfb840108,0xaac50101,
	0xfb840108,0xaac60101,0xfb840108,0xaac70101,0xfb840108,0xaac80101,0xfb840108,0xaac90101,
	0xfb840108,0xaaca0101,0xfb840108,0xaacb0101,0xfb840108,0xaacc0101,0xfb840108,0xaacd0101,
	0xfb840108,0xaace0101,0xfb840108,0xaacf0101,0xfb840108,0xaad00101,0xfb840108,0xaad10101,
	0xfb840108,0xaad20101,0xfb840108,0xaad30101,0xfb840108,0xaad40101,0xfb840108,0xaad50101,
	0xfb840108,0xaad60101,0xfb840108,0xaad70101,0xfb840108,0xaad80101,0xfb840108,0xaad90101,
	0xfb840108,0xaada0101,0xfb840108,0xaadb0101,0xfb840108,0xaadc0101,0xfb840108,0xaadd0101,
	0xfb840108,0xaade0101,0xfb840108,0xaadf0101,0xfb840108,0xaae00101,0xfb840108,0xaae10101,
	0xfb840108,0xaae20101,0xfb840108,0xaae30101,0xfb840108,0xaae40101,0xfb840108,0xaae50101,
	0xfb840108,0xaae60101,0xfb840108,0xaae70101,0xfb840108,0xaae80101,0xfb840108,0xaae90101,
	0xfb840108,0xaaea0101,0xfb840108,0xaaeb0101,0xfb840108,0xaaec0101,0xfb840108,0xaaed0101,
	0xfb840108,0xaaee0101,0xfb840108,0xaaef0101,0xfb840108,0xaaf00101,0xfb840108,0xaaf10101,
	0xfb840108,0xaaf20101,0xfb840108,0xaaf30101,0xfb840108,0xaaf40101,0xfb840108,0xaaf50101,
	0xfb840108,0xaaf60101,0xfb840108,0xaaf70101,0xfb840108,0xaaf80101,0xfb840108,0xaaf90101,
	0xfb840108,0xaafa0101,0xfb840108,0xaafb0101,0xfb840108,0xaafc0101,0xfb840108,0xaafd0101,
	0xfb840108,0xaafe0101,0xfb840108,0xaaff0101,0xfb840108,0xab000101,0xfb840108,0xab010101,
	0xfb840108,0xab020101,0xfb840108,0xab030101,0xfb840108,0xab040101,0xfb840108,0xab050101,
	0xfb840108,0xab060101,0xfb840108,0xab070101,0xfb840108,0xab080101,0xfb840108,0xab090101,
	0xfb840108,0xab0a0101,0xfb840108,0xab0b0101,0xfb840108,0xab0c0101,0xfb840108,0xab0d0101,
	0xfb840108,0xab0e0101,0xfb840108,0xab0f0101,0xfb840108,0xab100101,0xfb840108,0xab110101,
	0xfb840108,0xab120101,0xfb840108,0xab130101,0xfb840108,0xab140101,0xfb840108,0xab150101,
	0xfb840108,0xab160101,0xfb840108,0xab170101,0xfb840108,0xab180101,0xfb840108,0xab190101,
	0xfb840108,0xab1a0101,0xfb840108,0xab1b0101,0xfb840108,0xab1c0101,0xfb840108,0xab1d0101,
	0xfb840108,0xab1e0101,0xfb840108,0xab1f0101,0xfb840108,0xab200101,0xfb840108,0xab210101,
	0xfb840108,0xab220101,0xfb840108,0xab230101,0xfb840108,0xab240101,0xfb840108,0xab250101,
	0xfb840108,0xab260101,0xfb840108,0xab270101,0xfb840108,0xab280101,0xfb840108,0xab290101,
	0xfb840108,0xab2a0101,0xfb840108,0xab2b0101,0xfb840108,0xab2c0101,0xfb840108,0xab2d0101,
	0xfb840108,0xab2e0101,0xfb840108,0xab2f0101,0xfb840108,0xab300101,0xfb840108,0xab310101,
	0xfb840108,0xab320101,0xfb840108,0xab330101,0xfb840108,0xab340101,0xfb840108,0xab350101,
	0xfb840108,0xab360101,0xfb840108,0xab370101,0xfb840108,0xab380101,0xfb840108,0xab390101,
	0xfb840108,0xab3a0101,0xfb840108,0xab3b0101,0xfb840108,0xab3c0101,0xfb840108,0xab3d0101,
	0xfb840108,0xab3e0101,0xfb840108,0xab3f0101,0xfb840108,0xab400101,0xfb840108,0xab410101,
	0xfb840108,0xab420101,0xfb840108,0xab430101,0xfb840108,0xab440101,0xfb840108,0xab450101,
	0xfb840108,0xab460101,0xfb840108,0xab470101,0xfb840108,0xab480101,0xfb840108,0xab490101,
	0xfb840108,0xab4a0101,0xfb840108,0xab4b0101,0xfb840108,0xab4c0101,0xfb840108,0xab4d0101,
	0xfb840108,0xab4e0101,0xfb840108,0xab4f0101,0xfb840108,0xab500101,0xfb840108,0xab510101,
	0xfb840108,0xab520101,0xfb840108,0xab530101,0xfb840108,0xab540101,0xfb840108,0xab550101,
	0xfb840108,0xab560101,0xfb840108,0xab570101,0xfb840108,0xab580101,0xfb840108,0xab590101,
	0xfb840108,0xab5a0101,0xfb840108,0xab5b0101,0xfb840108,0xab5c0101,0xfb840108,0xab5d0101,
	0xfb840108,0xab5e0101,0xfb840108,0xab5f0101,0xfb840108,0xab600101,0xfb840108,0xab610101,
	0xfb840108,0xab620101,0xfb840108,0xab630101,0xfb840108,0xab640101,0xfb840108,0xab650101,
	0xfb840108,0xab660101,0xfb840108,0xab670101,0xfb840108,0xab680101,0xfb840108,0xab690101,
	0xfb840108,0xab6a0101,0xfb840108,0xab6b0101,0xfb840108,0xab6c0101,0xfb840108,0xab6d0101,
	0xfb840108,0xab6e0101,0xfb840108,0xab6f0101,0xfb840108,0xab700101,0xfb840108,0xab710101,
	0xfb840108,0xab720101,0xfb840108,0xab730101,0xfb840108,0xab740101,0xfb840108,0xab750101,
	0xfb840108,0xab760101,0xfb840108,0xab770101,0xfb840108,0xab780101,0xfb840108,0xab790101,
	0xfb840108,0xab7a0101,0xfb840108,0xab7b0101,0xfb840108,0xab7c0101,0xfb840108,0xab7d0101,
	0xfb840108,0xab7e0101,0xfb840108,0xab7f0101,0xfb840108,0xab800101,0xfb840108,0xab810101,
	0xfb840108,0xab820101,0xfb840108,0xab830101,0xfb840108,0xab840101,0xfb840108,0xab850101,
	0xfb840108,0xab860101,0xfb840108,0xab870101,0xfb840108,0xab880101,0xfb840108,0xab890101,
	0xfb840108,0xab8a0101,0xfb840108,0xab8b0101,0xfb840108,0xab8c0101,0xfb840108,0xab8d0101,
	0xfb840108,0xab8e0101,0xfb840108,0xab8f0101,0xfb840108,0xab900101,0xfb840108,0xab910101,
	0xfb840108,0xab920101,0xfb840108,0xab930101,0xfb840108,0xab940101,0xfb840108,0xab950101,
	0xfb840108,0xab960101,0xfb840108,0xab970101,0xfb840108,0xab980101,0xfb840108,0xab990101,
	0xfb840108,0xab9a0101,0xfb840108,0xab9b0101,0xfb840108,0xab9c0101,0xfb840108,0xab9d0101,
	0xfb840108,0xab9e0101,0xfb840108,0xab9f0101,0xfb840108,0xaba00101,0xfb840108,0xaba10101,
	0xfb840108,0xaba20101,0xfb840108,0xaba30101,0xfb840108,0xaba40101,0xfb840108,0xaba50101,
	0xfb840108,0xaba60101,0xfb840108,0xaba70101,0xfb840108,0xaba80101,0xfb840108,0xaba90101,
	0xfb840108,0xabaa0101,0xfb840108,0xabab0101,0xfb840108,0xabac0101,0xfb840108,0xabad0101,
	0xfb840108,0xabae0101,0xfb840108,0xabaf0101,0xfb840108,0xabb00101,0xfb840108,0xabb10101,
	0xfb840108,0xabb20101,0xfb840108,0xabb30101,0xfb840108,0xabb40101,0xfb840108,0xabb50101,
	0xfb840108,0xabb60101,0xfb840108,0xabb70101,0xfb840108,0xabb80101,0xfb840108,0xabb90101,
	0xfb840108,0xabba0101,0xfb840108,0xabbb0101,0xfb840108,0xabbc0101,0xfb840108,0xabbd0101,
	0xfb840108,0xabbe0101,0xfb840108,0xabbf0101,0xfb840108,0xabc00101,0xfb840108,0xabc10101,
	0xfb840108,0xabc20101,0xfb840108,0xabc30101,0xfb840108,0xabc40101,0xfb840108,0xabc50101,
	0xfb840108,0xabc60101,0xfb840108,0xabc70101,0xfb840108,0xabc80101,0xfb840108,0xabc90101,
	0xfb840108,0xabca0101,0xfb840108,0xabcb0101,0xfb840108,0xabcc0101,0xfb840108,0xabcd0101,
	0xfb840108,0xabce0101,0xfb840108,0xabcf0101,0xfb840108,0xabd00101,0xfb840108,0xabd10101,
	0xfb840108,0xabd20101,0xfb840108,0xabd30101,0xfb840108,0xabd40101,0xfb840108,0xabd50101,
	0xfb840108,0xabd60101,0xfb840108,0xabd70101,0xfb840108,0xabd80101,0xfb840108,0xabd90101,
	0xfb840108,0xabda0101,0xfb840108,0xabdb0101,0xfb840108,0xabdc0101,0xfb840108,0xabdd0101,
	0xfb840108,0xabde0101,0xfb840108,0xabdf0101,0xfb840108,0xabe00101,0xfb840108,0xabe10101,
	0xfb840108,0xabe20101,0xfb840108,0xabe30101,0xfb840108,0xabe40101,0xfb840108,0xabe50101,
	0xfb840108,0xabe60101,0xfb840108,0xabe70101,0xfb840108,0xabe80101,0xfb840108,0xabe90101,
	0xfb840108,0xabea0101,0xfb840108,0xabeb0101,0xfb840108,0xabec0101,0xfb840108,0xabed0101,
	0xfb840108,0xabee0101,0xfb840108,0xabef0101,0xfb840108,0xabf00101,0xfb840108,0xabf10101,
	0xfb840108,0xabf20101,0xfb840108,0xabf30101,0xfb840108,0xabf40101,0xfb840108,0xabf50101,
	0xfb840108,0xabf60101,0xfb840108,0xabf70101,0xfb840108,0xabf80101,0xfb840108,0xabf90101,
	0xfb840108,0xabfa0101,0xfb840108,0xabfb0101,0xfb840108,0xabfc0101,0xfb840108,0xabfd0101,
	0xfb840108,0xabfe0101,0xfb840108,0xabff0101,0xfb840108,0xac000101,0xfb840108,0xac010101,
	0xfb840108,0xac020101,0xfb840108,0xac030101,0xfb840108,0xac040101,0xfb840108,0xac050101,
	0xfb840108,0xac060101,0xfb840108,0xac070101,0xfb840108,0xac080101,0xfb840108,0xac090101,
	0xfb840108,0xac0a0101,0xfb840108,0xac0b0101,0xfb840108,0xac0c0101,0xfb840108,0xac0d0101,
	0xfb840108,0xac0e0101,0xfb840108,0xac0f0101,0xfb840108,0xac100101,0xfb840108,0xac110101,
	0xfb840108,0xac120101,0xfb840108,0xac130101,0xfb840108,0xac140101,0xfb840108,0xac150101,
	0xfb840108,0xac160101,0xfb840108,0xac170101,0xfb840108,0xac180101,0xfb840108,0xac190101,
	0xfb840108,0xac1a0101,0xfb840108,0xac1b0101,0xfb840108,0xac1c0101,0xfb840108,0xac1d0101,
	0xfb840108,0xac1e0101,0xfb840108,0xac1f0101,0xfb840108,0xac200101,0xfb840108,0xac210101,
	0xfb840108,0xac220101,0xfb840108,0xac230101,0xfb840108,0xac240101,0xfb840108,0xac250101,
	0xfb840108,0xac260101,0xfb840108,0xac270101,0xfb840108,0xac280101,0xfb840108,0xac290101,
	0xfb840108,0xac2a0101,0xfb840108,0xac2b0101,0xfb840108,0xac2c0101,0xfb840108,0xac2d0101,
	0xfb840108,0xac2e0101,0xfb840108,0xac2f0101,0xfb840108,0xac300101,0xfb840108,0xac310101,
	0xfb840108,0xac320101,0xfb840108,0xac330101,0xfb840108,0xac340101,0xfb840108,0xac350101,
	0xfb840108,0xac360101,0xfb840108,0xac370101,0xfb840108,0xac380101,0xfb840108,0xac390101,
	0xfb840108,0xac3a0101,0xfb840108,0xac3b0101,0xfb840108,0xac3c0101,0xfb840108,0xac3d0101,
	0xfb840108,0xac3e0101,0xfb840108,0xac3f0101,0xfb840108,0xac400101,0xfb840108,0xac410101,
	0xfb840108,0xac420101,0xfb840108,0xac430101,0xfb840108,0xac440101,0xfb840108,0xac450101,
	0xfb840108,0xac460101,0xfb840108,0xac470101,0xfb840108,0xac480101,0xfb840108,0xac490101,
	0xfb840108,0xac4a0101,0xfb840108,0xac4b0101,0xfb840108,0xac4c0101,0xfb840108,0xac4d0101,
	0xfb840108,0xac4e0101,0xfb840108,0xac4f0101,0xfb840108,0xac500101,0xfb840108,0xac510101,
	0xfb840108,0xac520101,0xfb840108,0xac530101,0xfb840108,0xac540101,0xfb840108,0xac550101,
	0xfb840108,0xac560101,0xfb840108,0xac570101,0xfb840108,0xac580101,0xfb840108,0xac590101,
	0xfb840108,0xac5a0101,0xfb840108,0xac5b0101,0xfb840108,0xac5c0101,0xfb840108,0xac5d0101,
	0xfb840108,0xac5e0101,0xfb840108,0xac5f0101,0xfb840108,0xac600101,0xfb840108,0xac610101,
	0xfb840108,0xac620101,0xfb840108,0xac630101,0xfb840108,0xac640101,0xfb840108,0xac650101,
	0xfb840108,0xac660101,0xfb840108,0xac670101,0xfb840108,0xac680101,0xfb840108,0xac690101,
	0xfb840108,0xac6a0101,0xfb840108,0xac6b0101,0xfb840108,0xac6c0101,0xfb840108,0xac6d0101,
	0xfb840108,0xac6e0101,0xfb840108,0xac6f0101,0xfb840108,0xac700101,0xfb840108,0xac710101,
	0xfb840108,0xac720101,0xfb840108,0xac730101,0xfb840108,0xac740101,0xfb840108,0xac750101,
	0xfb840108,0xac760101,0xfb840108,0xac770101,0xfb840108,0xac780101,0xfb840108,0xac790101,
	0xfb840108,0xac7a0101,0xfb840108,0xac7b0101,0xfb840108,0xac7c0101,0xfb840108,0xac7d0101,
	0xfb840108,0xac7e0101,0xfb840108,0xac7f0101,0xfb840108,0xac800101,0xfb840108,0xac810101,
	0xfb840108,0xac820101,0xfb840108,0xac830101,0xfb840108,0xac840101,0xfb840108,0xac850101,
	0xfb840108,0xac860101,0xfb840108,0xac870101,0xfb840108,0xac880101,0xfb840108,0xac890101,
	0xfb840108,0xac8a0101,0xfb840108,0xac8b0101,0xfb840108,0xac8c0101,0xfb840108,0xac8d0101,
	0xfb840108,0xac8e0101,0xfb840108,0xac8f0101,0xfb840108,0xac900101,0xfb840108,0xac910101,
	0xfb840108,0xac920101,0xfb840108,0xac930101,0xfb840108,0xac940101,0xfb840108,0xac950101,
	0xfb840108,0xac960101,0xfb840108,0xac970101,0xfb840108,0xac980101,0xfb840108,0xac990101,
	0xfb840108,0xac9a0101,0xfb840108,0xac9b0101,0xfb840108,0xac9c0101,0xfb840108,0xac9d0101,
	0xfb840108,0xac9e0101,0xfb840108,0xac9f0101,0xfb840108,0xaca00101,0xfb840108,0xaca10101,
	0xfb840108,0xaca20101,0xfb840108,0xaca30101,0xfb840108,0xaca40101,0xfb840108,0xaca50101,
	0xfb840108,0xaca60101,0xfb840108,0xaca70101,0xfb840108,0xaca80101,0xfb840108,0xaca90101,
	0xfb840108,0xacaa0101,0xfb840108,0xacab0101,0xfb840108,0xacac0101,0xfb840108,0xacad0101,
	0xfb840108,0xacae0101,0xfb840108,0xacaf0101,0xfb840108,0xacb00101,0xfb840108,0xacb10101,
	0xfb840108,0xacb20101,0xfb840108,0xacb30101,0xfb840108,0xacb40101,0xfb840108,0xacb50101,
	0xfb840108,0xacb60101,0xfb840108,0xacb70101,0xfb840108,0xacb80101,0xfb840108,0xacb90101,
	0xfb840108,0xacba0101,0xfb840108,0xacbb0101,0xfb840108,0xacbc0101,0xfb840108,0xacbd0101,
	0xfb840108,0xacbe0101,0xfb840108,0xacbf0101,0xfb840108,0xacc00101,0xfb840108,0xacc10101,
	0xfb840108,0xacc20101,0xfb840108,0xacc30101,0xfb840108,0xacc40101,0xfb840108,0xacc50101,
	0xfb840108,0xacc60101,0xfb840108,0xacc70101,0xfb840108,0xacc80101,0xfb840108,0xacc90101,
	0xfb840108,0xacca0101,0xfb840108,0xaccb0101,0xfb840108,0xaccc0101,0xfb840108,0xaccd0101,
	0xfb840108,0xacce0101,0xfb840108,0xaccf0101,0xfb840108,0xacd00101,0xfb840108,0xacd10101,
	0xfb840108,0xacd20101,0xfb840108,0xacd30101,0xfb840108,0xacd40101,0xfb840108,0xacd50101,
	0xfb840108,0xacd60101,0xfb840108,0xacd70101,0xfb840108,0xacd80101,0xfb840108,0xacd90101,
	0xfb840108,0xacda0101,0xfb840108,0xacdb0101,0xfb840108,0xacdc0101,0xfb840108,0xacdd0101,
	0xfb840108,0xacde0101,0xfb840108,0xacdf0101,0xfb840108,0xace00101,0xfb840108,0xace10101,
	0xfb840108,0xace20101,0xfb840108,0xace30101,0xfb840108,0xace40101,0xfb840108,0xace50101,
	0xfb840108,0xace60101,0xfb840108,0xace70101,0xfb840108,0xace80101,0xfb840108,0xace90101,
	0xfb840108,0xacea0101,0xfb840108,0xaceb0101,0xfb840108,0xacec0101,0xfb840108,0xaced0101,
	0xfb840108,0xacee0101,0xfb840108,0xacef0101,0xfb840108,0xacf00101,0xfb840108,0xacf10101,
	0xfb840108,0xacf20101,0xfb840108,0xacf30101,0xfb840108,0xacf40101,0xfb840108,0xacf50101,
	0xfb840108,0xacf60101,0xfb840108,0xacf70101,0xfb840108,0xacf80101,0xfb840108,0xacf90101,
	0xfb840108,0xacfa0101,0xfb840108,0xacfb0101,0xfb840108,0xacfc0101,0xfb840108,0xacfd0101,
	0xfb840108,0xacfe0101,0xfb840108,0xacff0101,0xfb840108,0xad000101,0xfb840108,0xad010101,
	0xfb840108,0xad020101,0xfb840108,0xad030101,0xfb840108,0xad040101,0xfb840108,0xad050101,
	0xfb840108,0xad060101,0xfb840108,0xad070101,0xfb840108,0xad080101,0xfb840108,0xad090101,
	0xfb840108,0xad0a0101,0xfb840108,0xad0b0101,0xfb840108,0xad0c0101,0xfb840108,0xad0d0101,
	0xfb840108,0xad0e0101,0xfb840108,0xad0f0101,0xfb840108,0xad100101,0xfb840108,0xad110101,
	0xfb840108,0xad120101,0xfb840108,0xad130101,0xfb840108,0xad140101,0xfb840108,0xad150101,
	0xfb840108,0xad160101,0xfb840108,0xad170101,0xfb840108,0xad180101,0xfb840108,0xad190101,
	0xfb840108,0xad1a0101,0xfb840108,0xad1b0101,0xfb840108,0xad1c0101,0xfb840108,0xad1d0101,
	0xfb840108,0xad1e0101,0xfb840108,0xad1f0101,0xfb840108,0xad200101,0xfb840108,0xad210101,
	0xfb840108,0xad220101,0xfb840108,0xad230101,0xfb840108,0xad240101,0xfb840108,0xad250101,
	0xfb840108,0xad260101,0xfb840108,0xad270101,0xfb840108,0xad280101,0xfb840108,0xad290101,
	0xfb840108,0xad2a0101,0xfb840108,0xad2b0101,0xfb840108,0xad2c0101,0xfb840108,0xad2d0101,
	0xfb840108,0xad2e0101,0xfb840108,0xad2f0101,0xfb840108,0xad300101,0xfb840108,0xad310101,
	0xfb840108,0xad320101,0xfb840108,0xad330101,0xfb840108,0xad340101,0xfb840108,0xad350101,
	0xfb840108,0xad360101,0xfb840108,0xad370101,0xfb840108,0xad380101,0xfb840108,0xad390101,
	0xfb840108,0xad3a0101,0xfb840108,0xad3b0101,0xfb840108,0xad3c0101,0xfb840108,0xad3d0101,
	0xfb840108,0xad3e0101,0xfb840108,0xad3f0101,0xfb840108,0xad400101,0xfb840108,0xad410101,
	0xfb840108,0xad420101,0xfb840108,0xad430101,0xfb840108,0xad440101,0xfb840108,0xad450101,
	0xfb840108,0xad460101,0xfb840108,0xad470101,0xfb840108,0xad480101,0xfb840108,0xad490101,
	0xfb840108,0xad4a0101,0xfb840108,0xad4b0101,0xfb840108,0xad4c0101,0xfb840108,0xad4d0101,
	0xfb840108,0xad4e0101,0xfb840108,0xad4f0101,0xfb840108,0xad500101,0xfb840108,0xad510101,
	0xfb840108,0xad520101,0xfb840108,0xad530101,0xfb840108,0xad540101,0xfb840108,0xad550101,
	0xfb840108,0xad560101,0xfb840108,0xad570101,0xfb840108,0xad580101,0xfb840108,0xad590101,
	0xfb840108,0xad5a0101,0xfb840108,0xad5b0101,0xfb840108,0xad5c0101,0xfb840108,0xad5d0101,
	0xfb840108,0xad5e0101,0xfb840108,0xad5f0101,0xfb840108,0xad600101,0xfb840108,0xad610101,
	0xfb840108,0xad620101,0xfb840108,0xad630101,0xfb840108,0xad640101,0xfb840108,0xad650101,
	0xfb840108,0xad660101,0xfb840108,0xad670101,0xfb840108,0xad680101,0xfb840108,0xad690101,
	0xfb840108,0xad6a0101,0xfb840108,0xad6b0101,0xfb840108,0xad6c0101,0xfb840108,0xad6d0101,
	0xfb840108,0xad6e0101,0xfb840108,0xad6f0101,0xfb840108,0xad700101,0xfb840108,0xad710101,
	0xfb840108,0xad720101,0xfb840108,0xad730101,0xfb840108,0xad740101,0xfb840108,0xad750101,
	0xfb840108,0xad760101,0xfb840108,0xad770101,0xfb840108,0xad780101,0xfb840108,0xad790101,
	0xfb840108,0xad7a0101,0xfb840108,0xad7b0101,0xfb840108,0xad7c0101,0xfb840108,0xad7d0101,
	0xfb840108,0xad7e0101,0xfb840108,0xad7f0101,0xfb840108,0xad800101,0xfb840108,0xad810101,
	0xfb840108,0xad820101,0xfb840108,0xad830101,0xfb840108,0xad840101,0xfb840108,0xad850101,
	0xfb840108,0xad860101,0xfb840108,0xad870101,0xfb840108,0xad880101,0xfb840108,0xad890101,
	0xfb840108,0xad8a0101,0xfb840108,0xad8b0101,0xfb840108,0xad8c0101,0xfb840108,0xad8d0101,
	0xfb840108,0xad8e0101,0xfb840108,0xad8f0101,0xfb840108,0xad900101,0xfb840108,0xad910101,
	0xfb840108,0xad920101,0xfb840108,0xad930101,0xfb840108,0xad940101,0xfb840108,0xad950101,
	0xfb840108,0xad960101,0xfb840108,0xad970101,0xfb840108,0xad980101,0xfb840108,0xad990101,
	0xfb840108,0xad9a0101,0xfb840108,0xad9b0101,0xfb840108,0xad9c0101,0xfb840108,0xad9d0101,
	0xfb840108,0xad9e0101,0xfb840108,0xad9f0101,0xfb840108,0xada00101,0xfb840108,0xada10101,
	0xfb840108,0xada20101,0xfb840108,0xada30101,0xfb840108,0xada40101,0xfb840108,0xada50101,
	0xfb840108,0xada60101,0xfb840108,0xada70101,0xfb840108,0xada80101,0xfb840108,0xada90101,
	0xfb840108,0xadaa0101,0xfb840108,0xadab0101,0xfb840108,0xadac0101,0xfb840108,0xadad0101,
	0xfb840108,0xadae0101,0xfb840108,0xadaf0101,0xfb840108,0xadb00101,0xfb840108,0xadb10101,
	0xfb840108,0xadb20101,0xfb840108,0xadb30101,0xfb840108,0xadb40101,0xfb840108,0xadb50101,
	0xfb840108,0xadb60101,0xfb840108,0xadb70101,0xfb840108,0xadb80101,0xfb840108,0xadb90101,
	0xfb840108,0xadba0101,0xfb840108,0xadbb0101,0xfb840108,0xadbc0101,0xfb840108,0xadbd0101,
	0xfb840108,0xadbe0101,0xfb840108,0xadbf0101,0xfb840108,0xadc00101,0xfb840108,0xadc10101,
	0xfb840108,0xadc20101,0xfb840108,0xadc30101,0xfb840108,0xadc40101,0xfb840108,0xadc50101,
	0xfb840108,0xadc60101,0xfb840108,0xadc70101,0xfb840108,0xadc80101,0xfb840108,0xadc90101,
	0xfb840108,0xadca0101,0xfb840108,0xadcb0101,0xfb840108,0xadcc0101,0xfb840108,0xadcd0101,
	0xfb840108,0xadce0101,0xfb840108,0xadcf0101,0xfb840108,0xadd00101,0xfb840108,0xadd10101,
	0xfb840108,0xadd20101,0xfb840108,0xadd30101,0xfb840108,0xadd40101,0xfb840108,0xadd50101,
	0xfb840108,0xadd60101,0xfb840108,0xadd70101,0xfb840108,0xadd80101,0xfb840108,0xadd90101,
	0xfb840108,0xadda0101,0xfb840108,0xaddb0101,0xfb840108,0xaddc0101,0xfb840108,0xaddd0101,
	0xfb840108,0xadde0101,0xfb840108,0xaddf0101,0xfb840108,0xade00101,0xfb840108,0xade10101,
	0xfb840108,0xade20101,0xfb840108,0xade30101,0xfb840108,0xade40101,0xfb840108,0xade50101,
	0xfb840108,0xade60101,0xfb840108,0xade70101,0xfb840108,0xade80101,0xfb840108,0xade90101,
	0xfb840108,0xadea0101,0xfb840108,0xadeb0101,0xfb840108,0xadec0101,0xfb840108,0xaded0101,
	0xfb840108,0xadee0101,0xfb840108,0xadef0101,0xfb840108,0xadf00101,0xfb840108,0xadf10101,
	0xfb840108,0xadf20101,0xfb840108,0xadf30101,0xfb840108,0xadf40101,0xfb840108,0xadf50101,
	0xfb840108,0xadf60101,0xfb840108,0xadf70101,0xfb840108,0xadf80101,0xfb840108,0xadf90101,
	0xfb840108,0xadfa0101,0xfb840108,0xadfb0101,0xfb840108,0xadfc0101,0xfb840108,0xadfd0101,
	0xfb840108,0xadfe0101,0xfb840108,0xadff0101,0xfb840108,0xae000101,0xfb840108,0xae010101,
	0xfb840108,0xae020101,0xfb840108,0xae030101,0xfb840108,0xae040101,0xfb840108,0xae050101,
	0xfb840108,0xae060101,0xfb840108,0xae070101,0xfb840108,0xae080101,0xfb840108,0xae090101,
	0xfb840108,0xae0a0101,0xfb840108,0xae0b0101,0xfb840108,0xae0c0101,0xfb840108,0xae0d0101,
	0xfb840108,0xae0e0101,0xfb840108,0xae0f0101,0xfb840108,0xae100101,0xfb840108,0xae110101,
	0xfb840108,0xae120101,0xfb840108,0xae130101,0xfb840108,0xae140101,0xfb840108,0xae150101,
	0xfb840108,0xae160101,0xfb840108,0xae170101,0xfb840108,0xae180101,0xfb840108,0xae190101,
	0xfb840108,0xae1a0101,0xfb840108,0xae1b0101,0xfb840108,0xae1c0101,0xfb840108,0xae1d0101,
	0xfb840108,0xae1e0101,0xfb840108,0xae1f0101,0xfb840108,0xae200101,0xfb840108,0xae210101,
	0xfb840108,0xae220101,0xfb840108,0xae230101,0xfb840108,0xae240101,0xfb840108,0xae250101,
	0xfb840108,0xae260101,0xfb840108,0xae270101,0xfb840108,0xae280101,0xfb840108,0xae290101,
	0xfb840108,0xae2a0101,0xfb840108,0xae2b0101,0xfb840108,0xae2c0101,0xfb840108,0xae2d0101,
	0xfb840108,0xae2e0101,0xfb840108,0xae2f0101,0xfb840108,0xae300101,0xfb840108,0xae310101,
	0xfb840108,0xae320101,0xfb840108,0xae330101,0xfb840108,0xae340101,0xfb840108,0xae350101,
	0xfb840108,0xae360101,0xfb840108,0xae370101,0xfb840108,0xae380101,0xfb840108,0xae390101,
	0xfb840108,0xae3a0101,0xfb840108,0xae3b0101,0xfb840108,0xae3c0101,0xfb840108,0xae3d0101,
	0xfb840108,0xae3e0101,0xfb840108,0xae3f0101,0xfb840108,0xae400101,0xfb840108,0xae410101,
	0xfb840108,0xae420101,0xfb840108,0xae430101,0xfb840108,0xae440101,0xfb840108,0xae450101,
	0xfb840108,0xae460101,0xfb840108,0xae470101,0xfb840108,0xae480101,0xfb840108,0xae490101,
	0xfb840108,0xae4a0101,0xfb840108,0xae4b0101,0xfb840108,0xae4c0101,0xfb840108,0xae4d0101,
	0xfb840108,0xae4e0101,0xfb840108,0xae4f0101,0xfb840108,0xae500101,0xfb840108,0xae510101,
	0xfb840108,0xae520101,0xfb840108,0xae530101,0xfb840108,0xae540101,0xfb840108,0xae550101,
	0xfb840108,0xae560101,0xfb840108,0xae570101,0xfb840108,0xae580101,0xfb840108,0xae590101,
	0xfb840108,0xae5a0101,0xfb840108,0xae5b0101,0xfb840108,0xae5c0101,0xfb840108,0xae5d0101,
	0xfb840108,0xae5e0101,0xfb840108,0xae5f0101,0xfb840108,0xae600101,0xfb840108,0xae610101,
	0xfb840108,0xae620101,0xfb840108,0xae630101,0xfb840108,0xae640101,0xfb840108,0xae650101,
	0xfb840108,0xae660101,0xfb840108,0xae670101,0xfb840108,0xae680101,0xfb840108,0xae690101,
	0xfb840108,0xae6a0101,0xfb840108,0xae6b0101,0xfb840108,0xae6c0101,0xfb840108,0xae6d0101,
	0xfb840108,0xae6e0101,0xfb840108,0xae6f0101,0xfb840108,0xae700101,0xfb840108,0xae710101,
	0xfb840108,0xae720101,0xfb840108,0xae730101,0xfb840108,0xae740101,0xfb840108,0xae750101,
	0xfb840108,0xae760101,0xfb840108,0xae770101,0xfb840108,0xae780101,0xfb840108,0xae790101,
	0xfb840108,0xae7a0101,0xfb840108,0xae7b0101,0xfb840108,0xae7c0101,0xfb840108,0xae7d0101,
	0xfb840108,0xae7e0101,0xfb840108,0xae7f0101,0xfb840108,0xae800101,0xfb840108,0xae810101,
	0xfb840108,0xae820101,0xfb840108,0xae830101,0xfb840108,0xae840101,0xfb840108,0xae850101,
	0xfb840108,0xae860101,0xfb840108,0xae870101,0xfb840108,0xae880101,0xfb840108,0xae890101,
	0xfb840108,0xae8a0101,0xfb840108,0xae8b0101,0xfb840108,0xae8c0101,0xfb840108,0xae8d0101,
	0xfb840108,0xae8e0101,0xfb840108,0xae8f0101,0xfb840108,0xae900101,0xfb840108,0xae910101,
	0xfb840108,0xae920101,0xfb840108,0xae930101,0xfb840108,0xae940101,0xfb840108,0xae950101,
	0xfb840108,0xae960101,0xfb840108,0xae970101,0xfb840108,0xae980101,0xfb840108,0xae990101,
	0xfb840108,0xae9a0101,0xfb840108,0xae9b0101,0xfb840108,0xae9c0101,0xfb840108,0xae9d0101,
	0xfb840108,0xae9e0101,0xfb840108,0xae9f0101,0xfb840108,0xaea00101,0xfb840108,0xaea10101,
	0xfb840108,0xaea20101,0xfb840108,0xaea30101,0xfb840108,0xaea40101,0xfb840108,0xaea50101,
	0xfb840108,0xaea60101,0xfb840108,0xaea70101,0xfb840108,0xaea80101,0xfb840108,0xaea90101,
	0xfb840108,0xaeaa0101,0xfb840108,0xaeab0101,0xfb840108,0xaeac0101,0xfb840108,0xaead0101,
	0xfb840108,0xaeae0101,0xfb840108,0xaeaf0101,0xfb840108,0xaeb00101,0xfb840108,0xaeb10101,
	0xfb840108,0xaeb20101,0xfb840108,0xaeb30101,0xfb840108,0xaeb40101,0xfb840108,0xaeb50101,
	0xfb840108,0xaeb60101,0xfb840108,0xaeb70101,0xfb840108,0xaeb80101,0xfb840108,0xaeb90101,
	0xfb840108,0xaeba0101,0xfb840108,0xaebb0101,0xfb840108,0xaebc0101,0xfb840108,0xaebd0101,
	0xfb840108,0xaebe0101,0xfb840108,0xaebf0101,0xfb840108,0xaec00101,0xfb840108,0xaec10101,
	0xfb840108,0xaec20101,0xfb840108,0xaec30101,0xfb840108,0xaec40101,0xfb840108,0xaec50101,
	0xfb840108,0xaec60101,0xfb840108,0xaec70101,0xfb840108,0xaec80101,0xfb840108,0xaec90101,
	0xfb840108,0xaeca0101,0xfb840108,0xaecb0101,0xfb840108,0xaecc0101,0xfb840108,0xaecd0101,
	0xfb840108,0xaece0101,0xfb840108,0xaecf0101,0xfb840108,0xaed00101,0xfb840108,0xaed10101,
	0xfb840108,0xaed20101,0xfb840108,0xaed30101,0xfb840108,0xaed40101,0xfb840108,0xaed50101,
	0xfb840108,0xaed60101,0xfb840108,0xaed70101,0xfb840108,0xaed80101,0xfb840108,0xaed90101,
	0xfb840108,0xaeda0101,0xfb840108,0xaedb0101,0xfb840108,0xaedc0101,0xfb840108,0xaedd0101,
	0xfb840108,0xaede0101,0xfb840108,0xaedf0101,0xfb840108,0xaee00101,0xfb840108,0xaee10101,
	0xfb840108,0xaee20101,0xfb840108,0xaee30101,0xfb840108,0xaee40101,0xfb840108,0xaee50101,
	0xfb840108,0xaee60101,0xfb840108,0xaee70101,0xfb840108,0xaee80101,0xfb840108,0xaee90101,
	0xfb840108,0xaeea0101,0xfb840108,0xaeeb0101,0xfb840108,0xaeec0101,0xfb840108,0xaeed0101,
	0xfb840108,0xaeee0101,0xfb840108,0xaeef0101,0xfb840108,0xaef00101,0xfb840108,0xaef10101,
	0xfb840108,0xaef20101,0xfb840108,0xaef30101,0xfb840108,0xaef40101,0xfb840108,0xaef50101,
	0xfb840108,0xaef60101,0xfb840108,0xaef70101,0xfb840108,0xaef80101,0xfb840108,0xaef90101,
	0xfb840108,0xaefa0101,0xfb840108,0xaefb0101,0xfb840108,0xaefc0101,0xfb840108,0xaefd0101,
	0xfb840108,0xaefe0101,0xfb840108,0xaeff0101,0xfb840108,0xaf000101,0xfb840108,0xaf010101,
	0xfb840108,0xaf020101,0xfb840108,0xaf030101,0xfb840108,0xaf040101,0xfb840108,0xaf050101,
	0xfb840108,0xaf060101,0xfb840108,0xaf070101,0xfb840108,0xaf080101,0xfb840108,0xaf090101,
	0xfb840108,0xaf0a0101,0xfb840108,0xaf0b0101,0xfb840108,0xaf0c0101,0xfb840108,0xaf0d0101,
	0xfb840108,0xaf0e0101,0xfb840108,0xaf0f0101,0xfb840108,0xaf100101,0xfb840108,0xaf110101,
	0xfb840108,0xaf120101,0xfb840108,0xaf130101,0xfb840108,0xaf140101,0xfb840108,0xaf150101,
	0xfb840108,0xaf160101,0xfb840108,0xaf170101,0xfb840108,0xaf180101,0xfb840108,0xaf190101,
	0xfb840108,0xaf1a0101,0xfb840108,0xaf1b0101,0xfb840108,0xaf1c0101,0xfb840108,0xaf1d0101,
	0xfb840108,0xaf1e0101,0xfb840108,0xaf1f0101,0xfb840108,0xaf200101,0xfb840108,0xaf210101,
	0xfb840108,0xaf220101,0xfb840108,0xaf230101,0xfb840108,0xaf240101,0xfb840108,0xaf250101,
	0xfb840108,0xaf260101,0xfb840108,0xaf270101,0xfb840108,0xaf280101,0xfb840108,0xaf290101,
	0xfb840108,0xaf2a0101,0xfb840108,0xaf2b0101,0xfb840108,0xaf2c0101,0xfb840108,0xaf2d0101,
	0xfb840108,0xaf2e0101,0xfb840108,0xaf2f0101,0xfb840108,0xaf300101,0xfb840108,0xaf310101,
	0xfb840108,0xaf320101,0xfb840108,0xaf330101,0xfb840108,0xaf340101,0xfb840108,0xaf350101,
	0xfb840108,0xaf360101,0xfb840108,0xaf370101,0xfb840108,0xaf380101,0xfb840108,0xaf390101,
	0xfb840108,0xaf3a0101,0xfb840108,0xaf3b0101,0xfb840108,0xaf3c0101,0xfb840108,0xaf3d0101,
	0xfb840108,0xaf3e0101,0xfb840108,0xaf3f0101,0xfb840108,0xaf400101,0xfb840108,0xaf410101,
	0xfb840108,0xaf420101,0xfb840108,0xaf430101,0xfb840108,0xaf440101,0xfb840108,0xaf450101,
	0xfb840108,0xaf460101,0xfb840108,0xaf470101,0xfb840108,0xaf480101,0xfb840108,0xaf490101,
	0xfb840108,0xaf4a0101,0xfb840108,0xaf4b0101,0xfb840108,0xaf4c0101,0xfb840108,0xaf4d0101,
	0xfb840108,0xaf4e0101,0xfb840108,0xaf4f0101,0xfb840108,0xaf500101,0xfb840108,0xaf510101,
	0xfb840108,0xaf520101,0xfb840108,0xaf530101,0xfb840108,0xaf540101,0xfb840108,0xaf550101,
	0xfb840108,0xaf560101,0xfb840108,0xaf570101,0xfb840108,0xaf580101,0xfb840108,0xaf590101,
	0xfb840108,0xaf5a0101,0xfb840108,0xaf5b0101,0xfb840108,0xaf5c0101,0xfb840108,0xaf5d0101,
	0xfb840108,0xaf5e0101,0xfb840108,0xaf5f0101,0xfb840108,0xaf600101,0xfb840108,0xaf610101,
	0xfb840108,0xaf620101,0xfb840108,0xaf630101,0xfb840108,0xaf640101,0xfb840108,0xaf650101,
	0xfb840108,0xaf660101,0xfb840108,0xaf670101,0xfb840108,0xaf680101,0xfb840108,0xaf690101,
	0xfb840108,0xaf6a0101,0xfb840108,0xaf6b0101,0xfb840108,0xaf6c0101,0xfb840108,0xaf6d0101,
	0xfb840108,0xaf6e0101,0xfb840108,0xaf6f0101,0xfb840108,0xaf700101,0xfb840108,0xaf710101,
	0xfb840108,0xaf720101,0xfb840108,0xaf730101,0xfb840108,0xaf740101,0xfb840108,0xaf750101,
	0xfb840108,0xaf760101,0xfb840108,0xaf770101,0xfb840108,0xaf780101,0xfb840108,0xaf790101,
	0xfb840108,0xaf7a0101,0xfb840108,0xaf7b0101,0xfb840108,0xaf7c0101,0xfb840108,0xaf7d0101,
	0xfb840108,0xaf7e0101,0xfb840108,0xaf7f0101,0xfb840108,0xaf800101,0xfb840108,0xaf810101,
	0xfb840108,0xaf820101,0xfb840108,0xaf830101,0xfb840108,0xaf840101,0xfb840108,0xaf850101,
	0xfb840108,0xaf860101,0xfb840108,0xaf870101,0xfb840108,0xaf880101,0xfb840108,0xaf890101,
	0xfb840108,0xaf8a0101,0xfb840108,0xaf8b0101,0xfb840108,0xaf8c0101,0xfb840108,0xaf8d0101,
	0xfb840108,0xaf8e0101,0xfb840108,0xaf8f0101,0xfb840108,0xaf900101,0xfb840108,0xaf910101,
	0xfb840108,0xaf920101,0xfb840108,0xaf930101,0xfb840108,0xaf940101,0xfb840108,0xaf950101,
	0xfb840108,0xaf960101,0xfb840108,0xaf970101,0xfb840108,0xaf980101,0xfb840108,0xaf990101,
	0xfb840108,0xaf9a0101,0xfb840108,0xaf9b0101,0xfb840108,0xaf9c0101,0xfb840108,0xaf9d0101,
	0xfb840108,0xaf9e0101,0xfb840108,0xaf9f0101,0xfb840108,0xafa00101,0xfb840108,0xafa10101,
	0xfb840108,0xafa20101,0xfb840108,0xafa30101,0xfb840108,0xafa40101,0xfb840108,0xafa50101,
	0xfb840108,0xafa60101,0xfb840108,0xafa70101,0xfb840108,0xafa80101,0xfb840108,0xafa90101,
	0xfb840108,0xafaa0101,0xfb840108,0xafab0101,0xfb840108,0xafac0101,0xfb840108,0xafad0101,
	0xfb840108,0xafae0101,0xfb840108,0xafaf0101,0xfb840108,0xafb00101,0xfb840108,0xafb10101,
	0xfb840108,0xafb20101,0xfb840108,0xafb30101,0xfb840108,0xafb40101,0xfb840108,0xafb50101,
	0xfb840108,0xafb60101,0xfb840108,0xafb70101,0xfb840108,0xafb80101,0xfb840108,0xafb90101,
	0xfb840108,0xafba0101,0xfb840108,0xafbb0101,0xfb840108,0xafbc0101,0xfb840108,0xafbd0101,
	0xfb840108,0xafbe0101,0xfb840108,0xafbf0101,0xfb840108,0xafc00101,0xfb840108,0xafc10101,
	0xfb840108,0xafc20101,0xfb840108,0xafc30101,0xfb840108,0xafc40101,0xfb840108,0xafc50101,
	0xfb840108,0xafc60101,0xfb840108,0xafc70101,0xfb840108,0xafc80101,0xfb840108,0xafc90101,
	0xfb840108,0xafca0101,0xfb840108,0xafcb0101,0xfb840108,0xafcc0101,0xfb840108,0xafcd0101,
	0xfb840108,0xafce0101,0xfb840108,0xafcf0101,0xfb840108,0xafd00101,0xfb840108,0xafd10101,
	0xfb840108,0xafd20101,0xfb840108,0xafd30101,0xfb840108,0xafd40101,0xfb840108,0xafd50101,
	0xfb840108,0xafd60101,0xfb840108,0xafd70101,0xfb840108,0xafd80101,0xfb840108,0xafd90101,
	0xfb840108,0xafda0101,0xfb840108,0xafdb0101,0xfb840108,0xafdc0101,0xfb840108,0xafdd0101,
	0xfb840108,0xafde0101,0xfb840108,0xafdf0101,0xfb840108,0xafe00101,0xfb840108,0xafe10101,
	0xfb840108,0xafe20101,0xfb840108,0xafe30101,0xfb840108,0xafe40101,0xfb840108,0xafe50101,
	0xfb840108,0xafe60101,0xfb840108,0xafe70101,0xfb840108,0xafe80101,0xfb840108,0xafe90101,
	0xfb840108,0xafea0101,0xfb840108,0xafeb0101,0xfb840108,0xafec0101,0xfb840108,0xafed0101,
	0xfb840108,0xafee0101,0xfb840108,0xafef0101,0xfb840108,0xaff00101,0xfb840108,0xaff10101,
	0xfb840108,0xaff20101,0xfb840108,0xaff30101,0xfb840108,0xaff40101,0xfb840108,0xaff50101,
	0xfb840108,0xaff60101,0xfb840108,0xaff70101,0xfb840108,0xaff80101,0xfb840108,0xaff90101,
	0xfb840108,0xaffa0101,0xfb840108,0xaffb0101,0xfb840108,0xaffc0101,0xfb840108,0xaffd0101,
	0xfb840108,0xaffe0101,0xfb840108,0xafff0101,0xfb840108,0xb0000101,0xfb840108,0xb0010101,
	0xfb840108,0xb0020101,0xfb840108,0xb0030101,0xfb840108,0xb0040101,0xfb840108,0xb0050101,
	0xfb840108,0xb0060101,0xfb840108,0xb0070101,0xfb840108,0xb0080101,0xfb840108,0xb0090101,
	0xfb840108,0xb00a0101,0xfb840108,0xb00b0101,0xfb840108,0xb00c0101,0xfb840108,0xb00d0101,
	0xfb840108,0xb00e0101,0xfb840108,0xb00f0101,0xfb840108,0xb0100101,0xfb840108,0xb0110101,
	0xfb840108,0xb0120101,0xfb840108,0xb0130101,0xfb840108,0xb0140101,0xfb840108,0xb0150101,
	0xfb840108,0xb0160101,0xfb840108,0xb0170101,0xfb840108,0xb0180101,0xfb840108,0xb0190101,
	0xfb840108,0xb01a0101,0xfb840108,0xb01b0101,0xfb840108,0xb01c0101,0xfb840108,0xb01d0101,
	0xfb840108,0xb01e0101,0xfb840108,0xb01f0101,0xfb840108,0xb0200101,0xfb840108,0xb0210101,
	0xfb840108,0xb0220101,0xfb840108,0xb0230101,0xfb840108,0xb0240101,0xfb840108,0xb0250101,
	0xfb840108,0xb0260101,0xfb840108,0xb0270101,0xfb840108,0xb0280101,0xfb840108,0xb0290101,
	0xfb840108,0xb02a0101,0xfb840108,0xb02b0101,0xfb840108,0xb02c0101,0xfb840108,0xb02d0101,
	0xfb840108,0xb02e0101,0xfb840108,0xb02f0101,0xfb840108,0xb0300101,0xfb840108,0xb0310101,
	0xfb840108,0xb0320101,0xfb840108,0xb0330101,0xfb840108,0xb0340101,0xfb840108,0xb0350101,
	0xfb840108,0xb0360101,0xfb840108,0xb0370101,0xfb840108,0xb0380101,0xfb840108,0xb0390101,
	0xfb840108,0xb03a0101,0xfb840108,0xb03b0101,0xfb840108,0xb03c0101,0xfb840108,0xb03d0101,
	0xfb840108,0xb03e0101,0xfb840108,0xb03f0101,0xfb840108,0xb0400101,0xfb840108,0xb0410101,
	0xfb840108,0xb0420101,0xfb840108,0xb0430101,0xfb840108,0xb0440101,0xfb840108,0xb0450101,
	0xfb840108,0xb0460101,0xfb840108,0xb0470101,0xfb840108,0xb0480101,0xfb840108,0xb0490101,
	0xfb840108,0xb04a0101,0xfb840108,0xb04b0101,0xfb840108,0xb04c0101,0xfb840108,0xb04d0101,
	0xfb840108,0xb04e0101,0xfb840108,0xb04f0101,0xfb840108,0xb0500101,0xfb840108,0xb0510101,
	0xfb840108,0xb0520101,0xfb840108,0xb0530101,0xfb840108,0xb0540101,0xfb840108,0xb0550101,
	0xfb840108,0xb0560101,0xfb840108,0xb0570101,0xfb840108,0xb0580101,0xfb840108,0xb0590101,
	0xfb840108,0xb05a0101,0xfb840108,0xb05b0101,0xfb840108,0xb05c0101,0xfb840108,0xb05d0101,
	0xfb840108,0xb05e0101,0xfb840108,0xb05f0101,0xfb840108,0xb0600101,0xfb840108,0xb0610101,
	0xfb840108,0xb0620101,0xfb840108,0xb0630101,0xfb840108,0xb0640101,0xfb840108,0xb0650101,
	0xfb840108,0xb0660101,0xfb840108,0xb0670101,0xfb840108,0xb0680101,0xfb840108,0xb0690101,
	0xfb840108,0xb06a0101,0xfb840108,0xb06b0101,0xfb840108,0xb06c0101,0xfb840108,0xb06d0101,
	0xfb840108,0xb06e0101,0xfb840108,0xb06f0101,0xfb840108,0xb0700101,0xfb840108,0xb0710101,
	0xfb840108,0xb0720101,0xfb840108,0xb0730101,0xfb840108,0xb0740101,0xfb840108,0xb0750101,
	0xfb840108,0xb0760101,0xfb840108,0xb0770101,0xfb840108,0xb0780101,0xfb840108,0xb0790101,
	0xfb840108,0xb07a0101,0xfb840108,0xb07b0101,0xfb840108,0xb07c0101,0xfb840108,0xb07d0101,
	0xfb840108,0xb07e0101,0xfb840108,0xb07f0101,0xfb840108,0xb0800101,0xfb840108,0xb0810101,
	0xfb840108,0xb0820101,0xfb840108,0xb0830101,0xfb840108,0xb0840101,0xfb840108,0xb0850101,
	0xfb840108,0xb0860101,0xfb840108,0xb0870101,0xfb840108,0xb0880101,0xfb840108,0xb0890101,
	0xfb840108,0xb08a0101,0xfb840108,0xb08b0101,0xfb840108,0xb08c0101,0xfb840108,0xb08d0101,
	0xfb840108,0xb08e0101,0xfb840108,0xb08f0101,0xfb840108,0xb0900101,0xfb840108,0xb0910101,
	0xfb840108,0xb0920101,0xfb840108,0xb0930101,0xfb840108,0xb0940101,0xfb840108,0xb0950101,
	0xfb840108,0xb0960101,0xfb840108,0xb0970101,0xfb840108,0xb0980101,0xfb840108,0xb0990101,
	0xfb840108,0xb09a0101,0xfb840108,0xb09b0101,0xfb840108,0xb09c0101,0xfb840108,0xb09d0101,
	0xfb840108,0xb09e0101,0xfb840108,0xb09f0101,0xfb840108,0xb0a00101,0xfb840108,0xb0a10101,
	0xfb840108,0xb0a20101,0xfb840108,0xb0a30101,0xfb840108,0xb0a40101,0xfb840108,0xb0a50101,
	0xfb840108,0xb0a60101,0xfb840108,0xb0a70101,0xfb840108,0xb0a80101,0xfb840108,0xb0a90101,
	0xfb840108,0xb0aa0101,0xfb840108,0xb0ab0101,0xfb840108,0xb0ac0101,0xfb840108,0xb0ad0101,
	0xfb840108,0xb0ae0101,0xfb840108,0xb0af0101,0xfb840108,0xb0b00101,0xfb840108,0xb0b10101,
	0xfb840108,0xb0b20101,0xfb840108,0xb0b30101,0xfb840108,0xb0b40101,0xfb840108,0xb0b50101,
	0xfb840108,0xb0b60101,0xfb840108,0xb0b70101,0xfb840108,0xb0b80101,0xfb840108,0xb0b90101,
	0xfb840108,0xb0ba0101,0xfb840108,0xb0bb0101,0xfb840108,0xb0bc0101,0xfb840108,0xb0bd0101,
	0xfb840108,0xb0be0101,0xfb840108,0xb0bf0101,0xfb840108,0xb0c00101,0xfb840108,0xb0c10101,
	0xfb840108,0xb0c20101,0xfb840108,0xb0c30101,0xfb840108,0xb0c40101,0xfb840108,0xb0c50101,
	0xfb840108,0xb0c60101,0xfb840108,0xb0c70101,0xfb840108,0xb0c80101,0xfb840108,0xb0c90101,
	0xfb840108,0xb0ca0101,0xfb840108,0xb0cb0101,0xfb840108,0xb0cc0101,0xfb840108,0xb0cd0101,
	0xfb840108,0xb0ce0101,0xfb840108,0xb0cf0101,0xfb840108,0xb0d00101,0xfb840108,0xb0d10101,
	0xfb840108,0xb0d20101,0xfb840108,0xb0d30101,0xfb840108,0xb0d40101,0xfb840108,0xb0d50101,
	0xfb840108,0xb0d60101,0xfb840108,0xb0d70101,0xfb840108,0xb0d80101,0xfb840108,0xb0d90101,
	0xfb840108,0xb0da0101,0xfb840108,0xb0db0101,0xfb840108,0xb0dc0101,0xfb840108,0xb0dd0101,
	0xfb840108,0xb0de0101,0xfb840108,0xb0df0101,0xfb840108,0xb0e00101,0xfb840108,0xb0e10101,
	0xfb840108,0xb0e20101,0xfb840108,0xb0e30101,0xfb840108,0xb0e40101,0xfb840108,0xb0e50101,
	0xfb840108,0xb0e60101,0xfb840108,0xb0e70101,0xfb840108,0xb0e80101,0xfb840108,0xb0e90101,
	0xfb840108,0xb0ea0101,0xfb840108,0xb0eb0101,0xfb840108,0xb0ec0101,0xfb840108,0xb0ed0101,
	0xfb840108,0xb0ee0101,0xfb840108,0xb0ef0101,0xfb840108,0xb0f00101,0xfb840108,0xb0f10101,
	0xfb840108,0xb0f20101,0xfb840108,0xb0f30101,0xfb840108,0xb0f40101,0xfb840108,0xb0f50101,
	0xfb840108,0xb0f60101,0xfb840108,0xb0f70101,0xfb840108,0xb0f80101,0xfb840108,0xb0f90101,
	0xfb840108,0xb0fa0101,0xfb840108,0xb0fb0101,0xfb840108,0xb0fc0101,0xfb840108,0xb0fd0101,
	0xfb840108,0xb0fe0101,0xfb840108,0xb0ff0101,0xfb840108,0xb1000101,0xfb840108,0xb1010101,
	0xfb840108,0xb1020101,0xfb840108,0xb1030101,0xfb840108,0xb1040101,0xfb840108,0xb1050101,
	0xfb840108,0xb1060101,0xfb840108,0xb1070101,0xfb840108,0xb1080101,0xfb840108,0xb1090101,
	0xfb840108,0xb10a0101,0xfb840108,0xb10b0101,0xfb840108,0xb10c0101,0xfb840108,0xb10d0101,
	0xfb840108,0xb10e0101,0xfb840108,0xb10f0101,0xfb840108,0xb1100101,0xfb840108,0xb1110101,
	0xfb840108,0xb1120101,0xfb840108,0xb1130101,0xfb840108,0xb1140101,0xfb840108,0xb1150101,
	0xfb840108,0xb1160101,0xfb840108,0xb1170101,0xfb840108,0xb1180101,0xfb840108,0xb1190101,
	0xfb840108,0xb11a0101,0xfb840108,0xb11b0101,0xfb840108,0xb11c0101,0xfb840108,0xb11d0101,
	0xfb840108,0xb11e0101,0xfb840108,0xb11f0101,0xfb840108,0xb1200101,0xfb840108,0xb1210101,
	0xfb840108,0xb1220101,0xfb840108,0xb1230101,0xfb840108,0xb1240101,0xfb840108,0xb1250101,
	0xfb840108,0xb1260101,0xfb840108,0xb1270101,0xfb840108,0xb1280101,0xfb840108,0xb1290101,
	0xfb840108,0xb12a0101,0xfb840108,0xb12b0101,0xfb840108,0xb12c0101,0xfb840108,0xb12d0101,
	0xfb840108,0xb12e0101,0xfb840108,0xb12f0101,0xfb840108,0xb1300101,0xfb840108,0xb1310101,
	0xfb840108,0xb1320101,0xfb840108,0xb1330101,0xfb840108,0xb1340101,0xfb840108,0xb1350101,
	0xfb840108,0xb1360101,0xfb840108,0xb1370101,0xfb840108,0xb1380101,0xfb840108,0xb1390101,
	0xfb840108,0xb13a0101,0xfb840108,0xb13b0101,0xfb840108,0xb13c0101,0xfb840108,0xb13d0101,
	0xfb840108,0xb13e0101,0xfb840108,0xb13f0101,0xfb840108,0xb1400101,0xfb840108,0xb1410101,
	0xfb840108,0xb1420101,0xfb840108,0xb1430101,0xfb840108,0xb1440101,0xfb840108,0xb1450101,
	0xfb840108,0xb1460101,0xfb840108,0xb1470101,0xfb840108,0xb1480101,0xfb840108,0xb1490101,
	0xfb840108,0xb14a0101,0xfb840108,0xb14b0101,0xfb840108,0xb14c0101,0xfb840108,0xb14d0101,
	0xfb840108,0xb14e0101,0xfb840108,0xb14f0101,0xfb840108,0xb1500101,0xfb840108,0xb1510101,
	0xfb840108,0xb1520101,0xfb840108,0xb1530101,0xfb840108,0xb1540101,0xfb840108,0xb1550101,
	0xfb840108,0xb1560101,0xfb840108,0xb1570101,0xfb840108,0xb1580101,0xfb840108,0xb1590101,
	0xfb840108,0xb15a0101,0xfb840108,0xb15b0101,0xfb840108,0xb15c0101,0xfb840108,0xb15d0101,
	0xfb840108,0xb15e0101,0xfb840108,0xb15f0101,0xfb840108,0xb1600101,0xfb840108,0xb1610101,
	0xfb840108,0xb1620101,0xfb840108,0xb1630101,0xfb840108,0xb1640101,0xfb840108,0xb1650101,
	0xfb840108,0xb1660101,0xfb840108,0xb1670101,0xfb840108,0xb1680101,0xfb840108,0xb1690101,
	0xfb840108,0xb16a0101,0xfb840108,0xb16b0101,0xfb840108,0xb16c0101,0xfb840108,0xb16d0101,
	0xfb840108,0xb16e0101,0xfb840108,0xb16f0101,0xfb840108,0xb1700101,0xfb840108,0xb1710101,
	0xfb840108,0xb1720101,0xfb840108,0xb1730101,0xfb840108,0xb1740101,0xfb840108,0xb1750101,
	0xfb840108,0xb1760101,0xfb840108,0xb1770101,0xfb840108,0xb1780101,0xfb840108,0xb1790101,
	0xfb840108,0xb17a0101,0xfb840108,0xb17b0101,0xfb840108,0xb17c0101,0xfb840108,0xb17d0101,
	0xfb840108,0xb17e0101,0xfb840108,0xb17f0101,0xfb840108,0xb1800101,0xfb840108,0xb1810101,
	0xfb840108,0xb1820101,0xfb840108,0xb1830101,0xfb840108,0xb1840101,0xfb840108,0xb1850101,
	0xfb840108,0xb1860101,0xfb840108,0xb1870101,0xfb840108,0xb1880101,0xfb840108,0xb1890101,
	0xfb840108,0xb18a0101,0xfb840108,0xb18b0101,0xfb840108,0xb18c0101,0xfb840108,0xb18d0101,
	0xfb840108,0xb18e0101,0xfb840108,0xb18f0101,0xfb840108,0xb1900101,0xfb840108,0xb1910101,
	0xfb840108,0xb1920101,0xfb840108,0xb1930101,0xfb840108,0xb1940101,0xfb840108,0xb1950101,
	0xfb840108,0xb1960101,0xfb840108,0xb1970101,0xfb840108,0xb1980101,0xfb840108,0xb1990101,
	0xfb840108,0xb19a0101,0xfb840108,0xb19b0101,0xfb840108,0xb19c0101,0xfb840108,0xb19d0101,
	0xfb840108,0xb19e0101,0xfb840108,0xb19f0101,0xfb840108,0xb1a00101,0xfb840108,0xb1a10101,
	0xfb840108,0xb1a20101,0xfb840108,0xb1a30101,0xfb840108,0xb1a40101,0xfb840108,0xb1a50101,
	0xfb840108,0xb1a60101,0xfb840108,0xb1a70101,0xfb840108,0xb1a80101,0xfb840108,0xb1a90101,
	0xfb840108,0xb1aa0101,0xfb840108,0xb1ab0101,0xfb840108,0xb1ac0101,0xfb840108,0xb1ad0101,
	0xfb840108,0xb1ae0101,0xfb840108,0xb1af0101,0xfb840108,0xb1b00101,0xfb840108,0xb1b10101,
	0xfb840108,0xb1b20101,0xfb840108,0xb1b30101,0xfb840108,0xb1b40101,0xfb840108,0xb1b50101,
	0xfb840108,0xb1b60101,0xfb840108,0xb1b70101,0xfb840108,0xb1b80101,0xfb840108,0xb1b90101,
	0xfb840108,0xb1ba0101,0xfb840108,0xb1bb0101,0xfb840108,0xb1bc0101,0xfb840108,0xb1bd0101,
	0xfb840108,0xb1be0101,0xfb840108,0xb1bf0101,0xfb840108,0xb1c00101,0xfb840108,0xb1c10101,
	0xfb840108,0xb1c20101,0xfb840108,0xb1c30101,0xfb840108,0xb1c40101,0xfb840108,0xb1c50101,
	0xfb840108,0xb1c60101,0xfb840108,0xb1c70101,0xfb840108,0xb1c80101,0xfb840108,0xb1c90101,
	0xfb840108,0xb1ca0101,0xfb840108,0xb1cb0101,0xfb840108,0xb1cc0101,0xfb840108,0xb1cd0101,
	0xfb840108,0xb1ce0101,0xfb840108,0xb1cf0101,0xfb840108,0xb1d00101,0xfb840108,0xb1d10101,
	0xfb840108,0xb1d20101,0xfb840108,0xb1d30101,0xfb840108,0xb1d40101,0xfb840108,0xb1d50101,
	0xfb840108,0xb1d60101,0xfb840108,0xb1d70101,0xfb840108,0xb1d80101,0xfb840108,0xb1d90101,
	0xfb840108,0xb1da0101,0xfb840108,0xb1db0101,0xfb840108,0xb1dc0101,0xfb840108,0xb1dd0101,
	0xfb840108,0xb1de0101,0xfb840108,0xb1df0101,0xfb840108,0xb1e00101,0xfb840108,0xb1e10101,
	0xfb840108,0xb1e20101,0xfb840108,0xb1e30101,0xfb840108,0xb1e40101,0xfb840108,0xb1e50101,
	0xfb840108,0xb1e60101,0xfb840108,0xb1e70101,0xfb840108,0xb1e80101,0xfb840108,0xb1e90101,
	0xfb840108,0xb1ea0101,0xfb840108,0xb1eb0101,0xfb840108,0xb1ec0101,0xfb840108,0xb1ed0101,
	0xfb840108,0xb1ee0101,0xfb840108,0xb1ef0101,0xfb840108,0xb1f00101,0xfb840108,0xb1f10101,
	0xfb840108,0xb1f20101,0xfb840108,0xb1f30101,0xfb840108,0xb1f40101,0xfb840108,0xb1f50101,
	0xfb840108,0xb1f60101,0xfb840108,0xb1f70101,0xfb840108,0xb1f80101,0xfb840108,0xb1f90101,
	0xfb840108,0xb1fa0101,0xfb840108,0xb1fb0101,0xfb840108,0xb1fc0101,0xfb840108,0xb1fd0101,
	0xfb840108,0xb1fe0101,0xfb840108,0xb1ff0101,0xfb840108,0xb2000101,0xfb840108,0xb2010101,
	0xfb840108,0xb2020101,0xfb840108,0xb2030101,0xfb840108,0xb2040101,0xfb840108,0xb2050101,
	0xfb840108,0xb2060101,0xfb840108,0xb2070101,0xfb840108,0xb2080101,0xfb840108,0xb2090101,
	0xfb840108,0xb20a0101,0xfb840108,0xb20b0101,0xfb840108,0xb20c0101,0xfb840108,0xb20d0101,
	0xfb840108,0xb20e0101,0xfb840108,0xb20f0101,0xfb840108,0xb2100101,0xfb840108,0xb2110101,
	0xfb840108,0xb2120101,0xfb840108,0xb2130101,0xfb840108,0xb2140101,0xfb840108,0xb2150101,
	0xfb840108,0xb2160101,0xfb840108,0xb2170101,0xfb840108,0xb2180101,0xfb840108,0xb2190101,
	0xfb840108,0xb21a0101,0xfb840108,0xb21b0101,0xfb840108,0xb21c0101,0xfb840108,0xb21d0101,
	0xfb840108,0xb21e0101,0xfb840108,0xb21f0101,0xfb840108,0xb2200101,0xfb840108,0xb2210101,
	0xfb840108,0xb2220101,0xfb840108,0xb2230101,0xfb840108,0xb2240101,0xfb840108,0xb2250101,
	0xfb840108,0xb2260101,0xfb840108,0xb2270101,0xfb840108,0xb2280101,0xfb840108,0xb2290101,
	0xfb840108,0xb22a0101,0xfb840108,0xb22b0101,0xfb840108,0xb22c0101,0xfb840108,0xb22d0101,
	0xfb840108,0xb22e0101,0xfb840108,0xb22f0101,0xfb840108,0xb2300101,0xfb840108,0xb2310101,
	0xfb840108,0xb2320101,0xfb840108,0xb2330101,0xfb840108,0xb2340101,0xfb840108,0xb2350101,
	0xfb840108,0xb2360101,0xfb840108,0xb2370101,0xfb840108,0xb2380101,0xfb840108,0xb2390101,
	0xfb840108,0xb23a0101,0xfb840108,0xb23b0101,0xfb840108,0xb23c0101,0xfb840108,0xb23d0101,
	0xfb840108,0xb23e0101,0xfb840108,0xb23f0101,0xfb840108,0xb2400101,0xfb840108,0xb2410101,
	0xfb840108,0xb2420101,0xfb840108,0xb2430101,0xfb840108,0xb2440101,0xfb840108,0xb2450101,
	0xfb840108,0xb2460101,0xfb840108,0xb2470101,0xfb840108,0xb2480101,0xfb840108,0xb2490101,
	0xfb840108,0xb24a0101,0xfb840108,0xb24b0101,0xfb840108,0xb24c0101,0xfb840108,0xb24d0101,
	0xfb840108,0xb24e0101,0xfb840108,0xb24f0101,0xfb840108,0xb2500101,0xfb840108,0xb2510101,
	0xfb840108,0xb2520101,0xfb840108,0xb2530101,0xfb840108,0xb2540101,0xfb840108,0xb2550101,
	0xfb840108,0xb2560101,0xfb840108,0xb2570101,0xfb840108,0xb2580101,0xfb840108,0xb2590101,
	0xfb840108,0xb25a0101,0xfb840108,0xb25b0101,0xfb840108,0xb25c0101,0xfb840108,0xb25d0101,
	0xfb840108,0xb25e0101,0xfb840108,0xb25f0101,0xfb840108,0xb2600101,0xfb840108,0xb2610101,
	0xfb840108,0xb2620101,0xfb840108,0xb2630101,0xfb840108,0xb2640101,0xfb840108,0xb2650101,
	0xfb840108,0xb2660101,0xfb840108,0xb2670101,0xfb840108,0xb2680101,0xfb840108,0xb2690101,
	0xfb840108,0xb26a0101,0xfb840108,0xb26b0101,0xfb840108,0xb26c0101,0xfb840108,0xb26d0101,
	0xfb840108,0xb26e0101,0xfb840108,0xb26f0101,0xfb840108,0xb2700101,0xfb840108,0xb2710101,
	0xfb840108,0xb2720101,0xfb840108,0xb2730101,0xfb840108,0xb2740101,0xfb840108,0xb2750101,
	0xfb840108,0xb2760101,0xfb840108,0xb2770101,0xfb840108,0xb2780101,0xfb840108,0xb2790101,
	0xfb840108,0xb27a0101,0xfb840108,0xb27b0101,0xfb840108,0xb27c0101,0xfb840108,0xb27d0101,
	0xfb840108,0xb27e0101,0xfb840108,0xb27f0101,0xfb840108,0xb2800101,0xfb840108,0xb2810101,
	0xfb840108,0xb2820101,0xfb840108,0xb2830101,0xfb840108,0xb2840101,0xfb840108,0xb2850101,
	0xfb840108,0xb2860101,0xfb840108,0xb2870101,0xfb840108,0xb2880101,0xfb840108,0xb2890101,
	0xfb840108,0xb28a0101,0xfb840108,0xb28b0101,0xfb840108,0xb28c0101,0xfb840108,0xb28d0101,
	0xfb840108,0xb28e0101,0xfb840108,0xb28f0101,0xfb840108,0xb2900101,0xfb840108,0xb2910101,
	0xfb840108,0xb2920101,0xfb840108,0xb2930101,0xfb840108,0xb2940101,0xfb840108,0xb2950101,
	0xfb840108,0xb2960101,0xfb840108,0xb2970101,0xfb840108,0xb2980101,0xfb840108,0xb2990101,
	0xfb840108,0xb29a0101,0xfb840108,0xb29b0101,0xfb840108,0xb29c0101,0xfb840108,0xb29d0101,
	0xfb840108,0xb29e0101,0xfb840108,0xb29f0101,0xfb840108,0xb2a00101,0xfb840108,0xb2a10101,
	0xfb840108,0xb2a20101,0xfb840108,0xb2a30101,0xfb840108,0xb2a40101,0xfb840108,0xb2a50101,
	0xfb840108,0xb2a60101,0xfb840108,0xb2a70101,0xfb840108,0xb2a80101,0xfb840108,0xb2a90101,
	0xfb840108,0xb2aa0101,0xfb840108,0xb2ab0101,0xfb840108,0xb2ac0101,0xfb840108,0xb2ad0101,
	0xfb840108,0xb2ae0101,0xfb840108,0xb2af0101,0xfb840108,0xb2b00101,0xfb840108,0xb2b10101,
	0xfb840108,0xb2b20101,0xfb840108,0xb2b30101,0xfb840108,0xb2b40101,0xfb840108,0xb2b50101,
	0xfb840108,0xb2b60101,0xfb840108,0xb2b70101,0xfb840108,0xb2b80101,0xfb840108,0xb2b90101,
	0xfb840108,0xb2ba0101,0xfb840108,0xb2bb0101,0xfb840108,0xb2bc0101,0xfb840108,0xb2bd0101,
	0xfb840108,0xb2be0101,0xfb840108,0xb2bf0101,0xfb840108,0xb2c00101,0xfb840108,0xb2c10101,
	0xfb840108,0xb2c20101,0xfb840108,0xb2c30101,0xfb840108,0xb2c40101,0xfb840108,0xb2c50101,
	0xfb840108,0xb2c60101,0xfb840108,0xb2c70101,0xfb840108,0xb2c80101,0xfb840108,0xb2c90101,
	0xfb840108,0xb2ca0101,0xfb840108,0xb2cb0101,0xfb840108,0xb2cc0101,0xfb840108,0xb2cd0101,
	0xfb840108,0xb2ce0101,0xfb840108,0xb2cf0101,0xfb840108,0xb2d00101,0xfb840108,0xb2d10101,
	0xfb840108,0xb2d20101,0xfb840108,0xb2d30101,0xfb840108,0xb2d40101,0xfb840108,0xb2d50101,
	0xfb840108,0xb2d60101,0xfb840108,0xb2d70101,0xfb840108,0xb2d80101,0xfb840108,0xb2d90101,
	0xfb840108,0xb2da0101,0xfb840108,0xb2db0101,0xfb840108,0xb2dc0101,0xfb840108,0xb2dd0101,
	0xfb840108,0xb2de0101,0xfb840108,0xb2df0101,0xfb840108,0xb2e00101,0xfb840108,0xb2e10101,
	0xfb840108,0xb2e20101,0xfb840108,0xb2e30101,0xfb840108,0xb2e40101,0xfb840108,0xb2e50101,
	0xfb840108,0xb2e60101,0xfb840108,0xb2e70101,0xfb840108,0xb2e80101,0xfb840108,0xb2e90101,
	0xfb840108,0xb2ea0101,0xfb840108,0xb2eb0101,0xfb840108,0xb2ec0101,0xfb840108,0xb2ed0101,
	0xfb840108,0xb2ee0101,0xfb840108,0xb2ef0101,0xfb840108,0xb2f00101,0xfb840108,0xb2f10101,
	0xfb840108,0xb2f20101,0xfb840108,0xb2f30101,0xfb840108,0xb2f40101,0xfb840108,0xb2f50101,
	0xfb840108,0xb2f60101,0xfb840108,0xb2f70101,0xfb840108,0xb2f80101,0xfb840108,0xb2f90101,
	0xfb840108,0xb2fa0101,0xfb840108,0xb2fb0101,0xfb840108,0xb2fc0101,0xfb840108,0xb2fd0101,
	0xfb840108,0xb2fe0101,0xfb840108,0xb2ff0101,0xfb840108,0xb3000101,0xfb840108,0xb3010101,
	0xfb840108,0xb3020101,0xfb840108,0xb3030101,0xfb840108,0xb3040101,0xfb840108,0xb3050101,
	0xfb840108,0xb3060101,0xfb840108,0xb3070101,0xfb840108,0xb3080101,0xfb840108,0xb3090101,
	0xfb840108,0xb30a0101,0xfb840108,0xb30b0101,0xfb840108,0xb30c0101,0xfb840108,0xb30d0101,
	0xfb840108,0xb30e0101,0xfb840108,0xb30f0101,0xfb840108,0xb3100101,0xfb840108,0xb3110101,
	0xfb840108,0xb3120101,0xfb840108,0xb3130101,0xfb840108,0xb3140101,0xfb840108,0xb3150101,
	0xfb840108,0xb3160101,0xfb840108,0xb3170101,0xfb840108,0xb3180101,0xfb840108,0xb3190101,
	0xfb840108,0xb31a0101,0xfb840108,0xb31b0101,0xfb840108,0xb31c0101,0xfb840108,0xb31d0101,
	0xfb840108,0xb31e0101,0xfb840108,0xb31f0101,0xfb840108,0xb3200101,0xfb840108,0xb3210101,
	0xfb840108,0xb3220101,0xfb840108,0xb3230101,0xfb840108,0xb3240101,0xfb840108,0xb3250101,
	0xfb840108,0xb3260101,0xfb840108,0xb3270101,0xfb840108,0xb3280101,0xfb840108,0xb3290101,
	0xfb840108,0xb32a0101,0xfb840108,0xb32b0101,0xfb840108,0xb32c0101,0xfb840108,0xb32d0101,
	0xfb840108,0xb32e0101,0xfb840108,0xb32f0101,0xfb840108,0xb3300101,0xfb840108,0xb3310101,
	0xfb840108,0xb3320101,0xfb840108,0xb3330101,0xfb840108,0xb3340101,0xfb840108,0xb3350101,
	0xfb840108,0xb3360101,0xfb840108,0xb3370101,0xfb840108,0xb3380101,0xfb840108,0xb3390101,
	0xfb840108,0xb33a0101,0xfb840108,0xb33b0101,0xfb840108,0xb33c0101,0xfb840108,0xb33d0101,
	0xfb840108,0xb33e0101,0xfb840108,0xb33f0101,0xfb840108,0xb3400101,0xfb840108,0xb3410101,
	0xfb840108,0xb3420101,0xfb840108,0xb3430101,0xfb840108,0xb3440101,0xfb840108,0xb3450101,
	0xfb840108,0xb3460101,0xfb840108,0xb3470101,0xfb840108,0xb3480101,0xfb840108,0xb3490101,
	0xfb840108,0xb34a0101,0xfb840108,0xb34b0101,0xfb840108,0xb34c0101,0xfb840108,0xb34d0101,
	0xfb840108,0xb34e0101,0xfb840108,0xb34f0101,0xfb840108,0xb3500101,0xfb840108,0xb3510101,
	0xfb840108,0xb3520101,0xfb840108,0xb3530101,0xfb840108,0xb3540101,0xfb840108,0xb3550101,
	0xfb840108,0xb3560101,0xfb840108,0xb3570101,0xfb840108,0xb3580101,0xfb840108,0xb3590101,
	0xfb840108,0xb35a0101,0xfb840108,0xb35b0101,0xfb840108,0xb35c0101,0xfb840108,0xb35d0101,
	0xfb840108,0xb35e0101,0xfb840108,0xb35f0101,0xfb840108,0xb3600101,0xfb840108,0xb3610101,
	0xfb840108,0xb3620101,0xfb840108,0xb3630101,0xfb840108,0xb3640101,0xfb840108,0xb3650101,
	0xfb840108,0xb3660101,0xfb840108,0xb3670101,0xfb840108,0xb3680101,0xfb840108,0xb3690101,
	0xfb840108,0xb36a0101,0xfb840108,0xb36b0101,0xfb840108,0xb36c0101,0xfb840108,0xb36d0101,
	0xfb840108,0xb36e0101,0xfb840108,0xb36f0101,0xfb840108,0xb3700101,0xfb840108,0xb3710101,
	0xfb840108,0xb3720101,0xfb840108,0xb3730101,0xfb840108,0xb3740101,0xfb840108,0xb3750101,
	0xfb840108,0xb3760101,0xfb840108,0xb3770101,0xfb840108,0xb3780101,0xfb840108,0xb3790101,
	0xfb840108,0xb37a0101,0xfb840108,0xb37b0101,0xfb840108,0xb37c0101,0xfb840108,0xb37d0101,
	0xfb840108,0xb37e0101,0xfb840108,0xb37f0101,0xfb840108,0xb3800101,0xfb840108,0xb3810101,
	0xfb840108,0xb3820101,0xfb840108,0xb3830101,0xfb840108,0xb3840101,0xfb840108,0xb3850101,
	0xfb840108,0xb3860101,0xfb840108,0xb3870101,0xfb840108,0xb3880101,0xfb840108,0xb3890101,
	0xfb840108,0xb38a0101,0xfb840108,0xb38b0101,0xfb840108,0xb38c0101,0xfb840108,0xb38d0101,
	0xfb840108,0xb38e0101,0xfb840108,0xb38f0101,0xfb840108,0xb3900101,0xfb840108,0xb3910101,
	0xfb840108,0xb3920101,0xfb840108,0xb3930101,0xfb840108,0xb3940101,0xfb840108,0xb3950101,
	0xfb840108,0xb3960101,0xfb840108,0xb3970101,0xfb840108,0xb3980101,0xfb840108,0xb3990101,
	0xfb840108,0xb39a0101,0xfb840108,0xb39b0101,0xfb840108,0xb39c0101,0xfb840108,0xb39d0101,
	0xfb840108,0xb39e0101,0xfb840108,0xb39f0101,0xfb840108,0xb3a00101,0xfb840108,0xb3a10101,
	0xfb840108,0xb3a20101,0xfb840108,0xb3a30101,0xfb840108,0xb3a40101,0xfb840108,0xb3a50101,
	0xfb840108,0xb3a60101,0xfb840108,0xb3a70101,0xfb840108,0xb3a80101,0xfb840108,0xb3a90101,
	0xfb840108,0xb3aa0101,0xfb840108,0xb3ab0101,0xfb840108,0xb3ac0101,0xfb840108,0xb3ad0101,
	0xfb840108,0xb3ae0101,0xfb840108,0xb3af0101,0xfb840108,0xb3b00101,0xfb840108,0xb3b10101,
	0xfb840108,0xb3b20101,0xfb840108,0xb3b30101,0xfb840108,0xb3b40101,0xfb840108,0xb3b50101,
	0xfb840108,0xb3b60101,0xfb840108,0xb3b70101,0xfb840108,0xb3b80101,0xfb840108,0xb3b90101,
	0xfb840108,0xb3ba0101,0xfb840108,0xb3bb0101,0xfb840108,0xb3bc0101,0xfb840108,0xb3bd0101,
	0xfb840108,0xb3be0101,0xfb840108,0xb3bf0101,0xfb840108,0xb3c00101,0xfb840108,0xb3c10101,
	0xfb840108,0xb3c20101,0xfb840108,0xb3c30101,0xfb840108,0xb3c40101,0xfb840108,0xb3c50101,
	0xfb840108,0xb3c60101,0xfb840108,0xb3c70101,0xfb840108,0xb3c80101,0xfb840108,0xb3c90101,
	0xfb840108,0xb3ca0101,0xfb840108,0xb3cb0101,0xfb840108,0xb3cc0101,0xfb840108,0xb3cd0101,
	0xfb840108,0xb3ce0101,0xfb840108,0xb3cf0101,0xfb840108,0xb3d00101,0xfb840108,0xb3d10101,
	0xfb840108,0xb3d20101,0xfb840108,0xb3d30101,0xfb840108,0xb3d40101,0xfb840108,0xb3d50101,
	0xfb840108,0xb3d60101,0xfb840108,0xb3d70101,0xfb840108,0xb3d80101,0xfb840108,0xb3d90101,
	0xfb840108,0xb3da0101,0xfb840108,0xb3db0101,0xfb840108,0xb3dc0101,0xfb840108,0xb3dd0101,
	0xfb840108,0xb3de0101,0xfb840108,0xb3df0101,0xfb840108,0xb3e00101,0xfb840108,0xb3e10101,
	0xfb840108,0xb3e20101,0xfb840108,0xb3e30101,0xfb840108,0xb3e40101,0xfb840108,0xb3e50101,
	0xfb840108,0xb3e60101,0xfb840108,0xb3e70101,0xfb840108,0xb3e80101,0xfb840108,0xb3e90101,
	0xfb840108,0xb3ea0101,0xfb840108,0xb3eb0101,0xfb840108,0xb3ec0101,0xfb840108,0xb3ed0101,
	0xfb840108,0xb3ee0101,0xfb840108,0xb3ef0101,0xfb840108,0xb3f00101,0xfb840108,0xb3f10101,
	0xfb840108,0xb3f20101,0xfb840108,0xb3f30101,0xfb840108,0xb3f40101,0xfb840108,0xb3f50101,
	0xfb840108,0xb3f60101,0xfb840108,0xb3f70101,0xfb840108,0xb3f80101,0xfb840108,0xb3f90101,
	0xfb840108,0xb3fa0101,0xfb840108,0xb3fb0101,0xfb840108,0xb3fc0101,0xfb840108,0xb3fd0101,
	0xfb840108,0xb3fe0101,0xfb840108,0xb3ff0101,0xfb840108,0xb4000101,0xfb840108,0xb4010101,
	0xfb840108,0xb4020101,0xfb840108,0xb4030101,0xfb840108,0xb4040101,0xfb840108,0xb4050101,
	0xfb840108,0xb4060101,0xfb840108,0xb4070101,0xfb840108,0xb4080101,0xfb840108,0xb4090101,
	0xfb840108,0xb40a0101,0xfb840108,0xb40b0101,0xfb840108,0xb40c0101,0xfb840108,0xb40d0101,
	0xfb840108,0xb40e0101,0xfb840108,0xb40f0101,0xfb840108,0xb4100101,0xfb840108,0xb4110101,
	0xfb840108,0xb4120101,0xfb840108,0xb4130101,0xfb840108,0xb4140101,0xfb840108,0xb4150101,
	0xfb840108,0xb4160101,0xfb840108,0xb4170101,0xfb840108,0xb4180101,0xfb840108,0xb4190101,
	0xfb840108,0xb41a0101,0xfb840108,0xb41b0101,0xfb840108,0xb41c0101,0xfb840108,0xb41d0101,
	0xfb840108,0xb41e0101,0xfb840108,0xb41f0101,0xfb840108,0xb4200101,0xfb840108,0xb4210101,
	0xfb840108,0xb4220101,0xfb840108,0xb4230101,0xfb840108,0xb4240101,0xfb840108,0xb4250101,
	0xfb840108,0xb4260101,0xfb840108,0xb4270101,0xfb840108,0xb4280101,0xfb840108,0xb4290101,
	0xfb840108,0xb42a0101,0xfb840108,0xb42b0101,0xfb840108,0xb42c0101,0xfb840108,0xb42d0101,
	0xfb840108,0xb42e0101,0xfb840108,0xb42f0101,0xfb840108,0xb4300101,0xfb840108,0xb4310101,
	0xfb840108,0xb4320101,0xfb840108,0xb4330101,0xfb840108,0xb4340101,0xfb840108,0xb4350101,
	0xfb840108,0xb4360101,0xfb840108,0xb4370101,0xfb840108,0xb4380101,0xfb840108,0xb4390101,
	0xfb840108,0xb43a0101,0xfb840108,0xb43b0101,0xfb840108,0xb43c0101,0xfb840108,0xb43d0101,
	0xfb840108,0xb43e0101,0xfb840108,0xb43f0101,0xfb840108,0xb4400101,0xfb840108,0xb4410101,
	0xfb840108,0xb4420101,0xfb840108,0xb4430101,0xfb840108,0xb4440101,0xfb840108,0xb4450101,
	0xfb840108,0xb4460101,0xfb840108,0xb4470101,0xfb840108,0xb4480101,0xfb840108,0xb4490101,
	0xfb840108,0xb44a0101,0xfb840108,0xb44b0101,0xfb840108,0xb44c0101,0xfb840108,0xb44d0101,
	0xfb840108,0xb44e0101,0xfb840108,0xb44f0101,0xfb840108,0xb4500101,0xfb840108,0xb4510101,
	0xfb840108,0xb4520101,0xfb840108,0xb4530101,0xfb840108,0xb4540101,0xfb840108,0xb4550101,
	0xfb840108,0xb4560101,0xfb840108,0xb4570101,0xfb840108,0xb4580101,0xfb840108,0xb4590101,
	0xfb840108,0xb45a0101,0xfb840108,0xb45b0101,0xfb840108,0xb45c0101,0xfb840108,0xb45d0101,
	0xfb840108,0xb45e0101,0xfb840108,0xb45f0101,0xfb840108,0xb4600101,0xfb840108,0xb4610101,
	0xfb840108,0xb4620101,0xfb840108,0xb4630101,0xfb840108,0xb4640101,0xfb840108,0xb4650101,
	0xfb840108,0xb4660101,0xfb840108,0xb4670101,0xfb840108,0xb4680101,0xfb840108,0xb4690101,
	0xfb840108,0xb46a0101,0xfb840108,0xb46b0101,0xfb840108,0xb46c0101,0xfb840108,0xb46d0101,
	0xfb840108,0xb46e0101,0xfb840108,0xb46f0101,0xfb840108,0xb4700101,0xfb840108,0xb4710101,
	0xfb840108,0xb4720101,0xfb840108,0xb4730101,0xfb840108,0xb4740101,0xfb840108,0xb4750101,
	0xfb840108,0xb4760101,0xfb840108,0xb4770101,0xfb840108,0xb4780101,0xfb840108,0xb4790101,
	0xfb840108,0xb47a0101,0xfb840108,0xb47b0101,0xfb840108,0xb47c0101,0xfb840108,0xb47d0101,
	0xfb840108,0xb47e0101,0xfb840108,0xb47f0101,0xfb840108,0xb4800101,0xfb840108,0xb4810101,
	0xfb840108,0xb4820101,0xfb840108,0xb4830101,0xfb840108,0xb4840101,0xfb840108,0xb4850101,
	0xfb840108,0xb4860101,0xfb840108,0xb4870101,0xfb840108,0xb4880101,0xfb840108,0xb4890101,
	0xfb840108,0xb48a0101,0xfb840108,0xb48b0101,0xfb840108,0xb48c0101,0xfb840108,0xb48d0101,
	0xfb840108,0xb48e0101,0xfb840108,0xb48f0101,0xfb840108,0xb4900101,0xfb840108,0xb4910101,
	0xfb840108,0xb4920101,0xfb840108,0xb4930101,0xfb840108,0xb4940101,0xfb840108,0xb4950101,
	0xfb840108,0xb4960101,0xfb840108,0xb4970101,0xfb840108,0xb4980101,0xfb840108,0xb4990101,
	0xfb840108,0xb49a0101,0xfb840108,0xb49b0101,0xfb840108,0xb49c0101,0xfb840108,0xb49d0101,
	0xfb840108,0xb49e0101,0xfb840108,0xb49f0101,0xfb840108,0xb4a00101,0xfb840108,0xb4a10101,
	0xfb840108,0xb4a20101,0xfb840108,0xb4a30101,0xfb840108,0xb4a40101,0xfb840108,0xb4a50101,
	0xfb840108,0xb4a60101,0xfb840108,0xb4a70101,0xfb840108,0xb4a80101,0xfb840108,0xb4a90101,
	0xfb840108,0xb4aa0101,0xfb840108,0xb4ab0101,0xfb840108,0xb4ac0101,0xfb840108,0xb4ad0101,
	0xfb840108,0xb4ae0101,0xfb840108,0xb4af0101,0xfb840108,0xb4b00101,0xfb840108,0xb4b10101,
	0xfb840108,0xb4b20101,0xfb840108,0xb4b30101,0xfb840108,0xb4b40101,0xfb840108,0xb4b50101,
	0xfb840108,0xb4b60101,0xfb840108,0xb4b70101,0xfb840108,0xb4b80101,0xfb840108,0xb4b90101,
	0xfb840108,0xb4ba0101,0xfb840108,0xb4bb0101,0xfb840108,0xb4bc0101,0xfb840108,0xb4bd0101,
	0xfb840108,0xb4be0101,0xfb840108,0xb4bf0101,0xfb840108,0xb4c00101,0xfb840108,0xb4c10101,
	0xfb840108,0xb4c20101,0xfb840108,0xb4c30101,0xfb840108,0xb4c40101,0xfb840108,0xb4c50101,
	0xfb840108,0xb4c60101,0xfb840108,0xb4c70101,0xfb840108,0xb4c80101,0xfb840108,0xb4c90101,
	0xfb840108,0xb4ca0101,0xfb840108,0xb4cb0101,0xfb840108,0xb4cc0101,0xfb840108,0xb4cd0101,
	0xfb840108,0xb4ce0101,0xfb840108,0xb4cf0101,0xfb840108,0xb4d00101,0xfb840108,0xb4d10101,
	0xfb840108,0xb4d20101,0xfb840108,0xb4d30101,0xfb840108,0xb4d40101,0xfb840108,0xb4d50101,
	0xfb840108,0xb4d60101,0xfb840108,0xb4d70101,0xfb840108,0xb4d80101,0xfb840108,0xb4d90101,
	0xfb840108,0xb4da0101,0xfb840108,0xb4db0101,0xfb840108,0xb4dc0101,0xfb840108,0xb4dd0101,
	0xfb840108,0xb4de0101,0xfb840108,0xb4df0101,0xfb840108,0xb4e00101,0xfb840108,0xb4e10101,
	0xfb840108,0xb4e20101,0xfb840108,0xb4e30101,0xfb840108,0xb4e40101,0xfb840108,0xb4e50101,
	0xfb840108,0xb4e60101,0xfb840108,0xb4e70101,0xfb840108,0xb4e80101,0xfb840108,0xb4e90101,
	0xfb840108,0xb4ea0101,0xfb840108,0xb4eb0101,0xfb840108,0xb4ec0101,0xfb840108,0xb4ed0101,
	0xfb840108,0xb4ee0101,0xfb840108,0xb4ef0101,0xfb840108,0xb4f00101,0xfb840108,0xb4f10101,
	0xfb840108,0xb4f20101,0xfb840108,0xb4f30101,0xfb840108,0xb4f40101,0xfb840108,0xb4f50101,
	0xfb840108,0xb4f60101,0xfb840108,0xb4f70101,0xfb840108,0xb4f80101,0xfb840108,0xb4f90101,
	0xfb840108,0xb4fa0101,0xfb840108,0xb4fb0101,0xfb840108,0xb4fc0101,0xfb840108,0xb4fd0101,
	0xfb840108,0xb4fe0101,0xfb840108,0xb4ff0101,0xfb840108,0xb5000101,0xfb840108,0xb5010101,
	0xfb840108,0xb5020101,0xfb840108,0xb5030101,0xfb840108,0xb5040101,0xfb840108,0xb5050101,
	0xfb840108,0xb5060101,0xfb840108,0xb5070101,0xfb840108,0xb5080101,0xfb840108,0xb5090101,
	0xfb840108,0xb50a0101,0xfb840108,0xb50b0101,0xfb840108,0xb50c0101,0xfb840108,0xb50d0101,
	0xfb840108,0xb50e0101,0xfb840108,0xb50f0101,0xfb840108,0xb5100101,0xfb840108,0xb5110101,
	0xfb840108,0xb5120101,0xfb840108,0xb5130101,0xfb840108,0xb5140101,0xfb840108,0xb5150101,
	0xfb840108,0xb5160101,0xfb840108,0xb5170101,0xfb840108,0xb5180101,0xfb840108,0xb5190101,
	0xfb840108,0xb51a0101,0xfb840108,0xb51b0101,0xfb840108,0xb51c0101,0xfb840108,0xb51d0101,
	0xfb840108,0xb51e0101,0xfb840108,0xb51f0101,0xfb840108,0xb5200101,0xfb840108,0xb5210101,
	0xfb840108,0xb5220101,0xfb840108,0xb5230101,0xfb840108,0xb5240101,0xfb840108,0xb5250101,
	0xfb840108,0xb5260101,0xfb840108,0xb5270101,0xfb840108,0xb5280101,0xfb840108,0xb5290101,
	0xfb840108,0xb52a0101,0xfb840108,0xb52b0101,0xfb840108,0xb52c0101,0xfb840108,0xb52d0101,
	0xfb840108,0xb52e0101,0xfb840108,0xb52f0101,0xfb840108,0xb5300101,0xfb840108,0xb5310101,
	0xfb840108,0xb5320101,0xfb840108,0xb5330101,0xfb840108,0xb5340101,0xfb840108,0xb5350101,
	0xfb840108,0xb5360101,0xfb840108,0xb5370101,0xfb840108,0xb5380101,0xfb840108,0xb5390101,
	0xfb840108,0xb53a0101,0xfb840108,0xb53b0101,0xfb840108,0xb53c0101,0xfb840108,0xb53d0101,
	0xfb840108,0xb53e0101,0xfb840108,0xb53f0101,0xfb840108,0xb5400101,0xfb840108,0xb5410101,
	0xfb840108,0xb5420101,0xfb840108,0xb5430101,0xfb840108,0xb5440101,0xfb840108,0xb5450101,
	0xfb840108,0xb5460101,0xfb840108,0xb5470101,0xfb840108,0xb5480101,0xfb840108,0xb5490101,
	0xfb840108,0xb54a0101,0xfb840108,0xb54b0101,0xfb840108,0xb54c0101,0xfb840108,0xb54d0101,
	0xfb840108,0xb54e0101,0xfb840108,0xb54f0101,0xfb840108,0xb5500101,0xfb840108,0xb5510101,
	0xfb840108,0xb5520101,0xfb840108,0xb5530101,0xfb840108,0xb5540101,0xfb840108,0xb5550101,
	0xfb840108,0xb5560101,0xfb840108,0xb5570101,0xfb840108,0xb5580101,0xfb840108,0xb5590101,
	0xfb840108,0xb55a0101,0xfb840108,0xb55b0101,0xfb840108,0xb55c0101,0xfb840108,0xb55d0101,
	0xfb840108,0xb55e0101,0xfb840108,0xb55f0101,0xfb840108,0xb5600101,0xfb840108,0xb5610101,
	0xfb840108,0xb5620101,0xfb840108,0xb5630101,0xfb840108,0xb5640101,0xfb840108,0xb5650101,
	0xfb840108,0xb5660101,0xfb840108,0xb5670101,0xfb840108,0xb5680101,0xfb840108,0xb5690101,
	0xfb840108,0xb56a0101,0xfb840108,0xb56b0101,0xfb840108,0xb56c0101,0xfb840108,0xb56d0101,
	0xfb840108,0xb56e0101,0xfb840108,0xb56f0101,0xfb840108,0xb5700101,0xfb840108,0xb5710101,
	0xfb840108,0xb5720101,0xfb840108,0xb5730101,0xfb840108,0xb5740101,0xfb840108,0xb5750101,
	0xfb840108,0xb5760101,0xfb840108,0xb5770101,0xfb840108,0xb5780101,0xfb840108,0xb5790101,
	0xfb840108,0xb57a0101,0xfb840108,0xb57b0101,0xfb840108,0xb57c0101,0xfb840108,0xb57d0101,
	0xfb840108,0xb57e0101,0xfb840108,0xb57f0101,0xfb840108,0xb5800101,0xfb840108,0xb5810101,
	0xfb840108,0xb5820101,0xfb840108,0xb5830101,0xfb840108,0xb5840101,0xfb840108,0xb5850101,
	0xfb840108,0xb5860101,0xfb840108,0xb5870101,0xfb840108,0xb5880101,0xfb840108,0xb5890101,
	0xfb840108,0xb58a0101,0xfb840108,0xb58b0101,0xfb840108,0xb58c0101,0xfb840108,0xb58d0101,
	0xfb840108,0xb58e0101,0xfb840108,0xb58f0101,0xfb840108,0xb5900101,0xfb840108,0xb5910101,
	0xfb840108,0xb5920101,0xfb840108,0xb5930101,0xfb840108,0xb5940101,0xfb840108,0xb5950101,
	0xfb840108,0xb5960101,0xfb840108,0xb5970101,0xfb840108,0xb5980101,0xfb840108,0xb5990101,
	0xfb840108,0xb59a0101,0xfb840108,0xb59b0101,0xfb840108,0xb59c0101,0xfb840108,0xb59d0101,
	0xfb840108,0xb59e0101,0xfb840108,0xb59f0101,0xfb840108,0xb5a00101,0xfb840108,0xb5a10101,
	0xfb840108,0xb5a20101,0xfb840108,0xb5a30101,0xfb840108,0xb5a40101,0xfb840108,0xb5a50101,
	0xfb840108,0xb5a60101,0xfb840108,0xb5a70101,0xfb840108,0xb5a80101,0xfb840108,0xb5a90101,
	0xfb840108,0xb5aa0101,0xfb840108,0xb5ab0101,0xfb840108,0xb5ac0101,0xfb840108,0xb5ad0101,
	0xfb840108,0xb5ae0101,0xfb840108,0xb5af0101,0xfb840108,0xb5b00101,0xfb840108,0xb5b10101,
	0xfb840108,0xb5b20101,0xfb840108,0xb5b30101,0xfb840108,0xb5b40101,0xfb840108,0xb5b50101,
	0xfb840108,0xb5b60101,0xfb840108,0xb5b70101,0xfb840108,0xb5b80101,0xfb840108,0xb5b90101,
	0xfb840108,0xb5ba0101,0xfb840108,0xb5bb0101,0xfb840108,0xb5bc0101,0xfb840108,0xb5bd0101,
	0xfb840108,0xb5be0101,0xfb840108,0xb5bf0101,0xfb840108,0xb5c00101,0xfb840108,0xb5c10101,
	0xfb840108,0xb5c20101,0xfb840108,0xb5c30101,0xfb840108,0xb5c40101,0xfb840108,0xb5c50101,
	0xfb840108,0xb5c60101,0xfb840108,0xb5c70101,0xfb840108,0xb5c80101,0xfb840108,0xb5c90101,
	0xfb840108,0xb5ca0101,0xfb840108,0xb5cb0101,0xfb840108,0xb5cc0101,0xfb840108,0xb5cd0101,
	0xfb840108,0xb5ce0101,0xfb840108,0xb5cf0101,0xfb840108,0xb5d00101,0xfb840108,0xb5d10101,
	0xfb840108,0xb5d20101,0xfb840108,0xb5d30101,0xfb840108,0xb5d40101,0xfb840108,0xb5d50101,
	0xfb840108,0xb5d60101,0xfb840108,0xb5d70101,0xfb840108,0xb5d80101,0xfb840108,0xb5d90101,
	0xfb840108,0xb5da0101,0xfb840108,0xb5db0101,0xfb840108,0xb5dc0101,0xfb840108,0xb5dd0101,
	0xfb840108,0xb5de0101,0xfb840108,0xb5df0101,0xfb840108,0xb5e00101,0xfb840108,0xb5e10101,
	0xfb840108,0xb5e20101,0xfb840108,0xb5e30101,0xfb840108,0xb5e40101,0xfb840108,0xb5e50101,
	0xfb840108,0xb5e60101,0xfb840108,0xb5e70101,0xfb840108,0xb5e80101,0xfb840108,0xb5e90101,
	0xfb840108,0xb5ea0101,0xfb840108,0xb5eb0101,0xfb840108,0xb5ec0101,0xfb840108,0xb5ed0101,
	0xfb840108,0xb5ee0101,0xfb840108,0xb5ef0101,0xfb840108,0xb5f00101,0xfb840108,0xb5f10101,
	0xfb840108,0xb5f20101,0xfb840108,0xb5f30101,0xfb840108,0xb5f40101,0xfb840108,0xb5f50101,
	0xfb840108,0xb5f60101,0xfb840108,0xb5f70101,0xfb840108,0xb5f80101,0xfb840108,0xb5f90101,
	0xfb840108,0xb5fa0101,0xfb840108,0xb5fb0101,0xfb840108,0xb5fc0101,0xfb840108,0xb5fd0101,
	0xfb840108,0xb5fe0101,0xfb840108,0xb5ff0101,0xfb840108,0xb6000101,0xfb840108,0xb6010101,
	0xfb840108,0xb6020101,0xfb840108,0xb6030101,0xfb840108,0xb6040101,0xfb840108,0xb6050101,
	0xfb840108,0xb6060101,0xfb840108,0xb6070101,0xfb840108,0xb6080101,0xfb840108,0xb6090101,
	0xfb840108,0xb60a0101,0xfb840108,0xb60b0101,0xfb840108,0xb60c0101,0xfb840108,0xb60d0101,
	0xfb840108,0xb60e0101,0xfb840108,0xb60f0101,0xfb840108,0xb6100101,0xfb840108,0xb6110101,
	0xfb840108,0xb6120101,0xfb840108,0xb6130101,0xfb840108,0xb6140101,0xfb840108,0xb6150101,
	0xfb840108,0xb6160101,0xfb840108,0xb6170101,0xfb840108,0xb6180101,0xfb840108,0xb6190101,
	0xfb840108,0xb61a0101,0xfb840108,0xb61b0101,0xfb840108,0xb61c0101,0xfb840108,0xb61d0101,
	0xfb840108,0xb61e0101,0xfb840108,0xb61f0101,0xfb840108,0xb6200101,0xfb840108,0xb6210101,
	0xfb840108,0xb6220101,0xfb840108,0xb6230101,0xfb840108,0xb6240101,0xfb840108,0xb6250101,
	0xfb840108,0xb6260101,0xfb840108,0xb6270101,0xfb840108,0xb6280101,0xfb840108,0xb6290101,
	0xfb840108,0xb62a0101,0xfb840108,0xb62b0101,0xfb840108,0xb62c0101,0xfb840108,0xb62d0101,
	0xfb840108,0xb62e0101,0xfb840108,0xb62f0101,0xfb840108,0xb6300101,0xfb840108,0xb6310101,
	0xfb840108,0xb6320101,0xfb840108,0xb6330101,0xfb840108,0xb6340101,0xfb840108,0xb6350101,
	0xfb840108,0xb6360101,0xfb840108,0xb6370101,0xfb840108,0xb6380101,0xfb840108,0xb6390101,
	0xfb840108,0xb63a0101,0xfb840108,0xb63b0101,0xfb840108,0xb63c0101,0xfb840108,0xb63d0101,
	0xfb840108,0xb63e0101,0xfb840108,0xb63f0101,0xfb840108,0xb6400101,0xfb840108,0xb6410101,
	0xfb840108,0xb6420101,0xfb840108,0xb6430101,0xfb840108,0xb6440101,0xfb840108,0xb6450101,
	0xfb840108,0xb6460101,0xfb840108,0xb6470101,0xfb840108,0xb6480101,0xfb840108,0xb6490101,
	0xfb840108,0xb64a0101,0xfb840108,0xb64b0101,0xfb840108,0xb64c0101,0xfb840108,0xb64d0101,
	0xfb840108,0xb64e0101,0xfb840108,0xb64f0101,0xfb840108,0xb6500101,0xfb840108,0xb6510101,
	0xfb840108,0xb6520101,0xfb840108,0xb6530101,0xfb840108,0xb6540101,0xfb840108,0xb6550101,
	0xfb840108,0xb6560101,0xfb840108,0xb6570101,0xfb840108,0xb6580101,0xfb840108,0xb6590101,
	0xfb840108,0xb65a0101,0xfb840108,0xb65b0101,0xfb840108,0xb65c0101,0xfb840108,0xb65d0101,
	0xfb840108,0xb65e0101,0xfb840108,0xb65f0101,0xfb840108,0xb6600101,0xfb840108,0xb6610101,
	0xfb840108,0xb6620101,0xfb840108,0xb6630101,0xfb840108,0xb6640101,0xfb840108,0xb6650101,
	0xfb840108,0xb6660101,0xfb840108,0xb6670101,0xfb840108,0xb6680101,0xfb840108,0xb6690101,
	0xfb840108,0xb66a0101,0xfb840108,0xb66b0101,0xfb840108,0xb66c0101,0xfb840108,0xb66d0101,
	0xfb840108,0xb66e0101,0xfb840108,0xb66f0101,0xfb840108,0xb6700101,0xfb840108,0xb6710101,
	0xfb840108,0xb6720101,0xfb840108,0xb6730101,0xfb840108,0xb6740101,0xfb840108,0xb6750101,
	0xfb840108,0xb6760101,0xfb840108,0xb6770101,0xfb840108,0xb6780101,0xfb840108,0xb6790101,
	0xfb840108,0xb67a0101,0xfb840108,0xb67b0101,0xfb840108,0xb67c0101,0xfb840108,0xb67d0101,
	0xfb840108,0xb67e0101,0xfb840108,0xb67f0101,0xfb840108,0xb6800101,0xfb840108,0xb6810101,
	0xfb840108,0xb6820101,0xfb840108,0xb6830101,0xfb840108,0xb6840101,0xfb840108,0xb6850101,
	0xfb840108,0xb6860101,0xfb840108,0xb6870101,0xfb840108,0xb6880101,0xfb840108,0xb6890101,
	0xfb840108,0xb68a0101,0xfb840108,0xb68b0101,0xfb840108,0xb68c0101,0xfb840108,0xb68d0101,
	0xfb840108,0xb68e0101,0xfb840108,0xb68f0101,0xfb840108,0xb6900101,0xfb840108,0xb6910101,
	0xfb840108,0xb6920101,0xfb840108,0xb6930101,0xfb840108,0xb6940101,0xfb840108,0xb6950101,
	0xfb840108,0xb6960101,0xfb840108,0xb6970101,0xfb840108,0xb6980101,0xfb840108,0xb6990101,
	0xfb840108,0xb69a0101,0xfb840108,0xb69b0101,0xfb840108,0xb69c0101,0xfb840108,0xb69d0101,
	0xfb840108,0xb69e0101,0xfb840108,0xb69f0101,0xfb840108,0xb6a00101,0xfb840108,0xb6a10101,
	0xfb840108,0xb6a20101,0xfb840108,0xb6a30101,0xfb840108,0xb6a40101,0xfb840108,0xb6a50101,
	0xfb840108,0xb6a60101,0xfb840108,0xb6a70101,0xfb840108,0xb6a80101,0xfb840108,0xb6a90101,
	0xfb840108,0xb6aa0101,0xfb840108,0xb6ab0101,0xfb840108,0xb6ac0101,0xfb840108,0xb6ad0101,
	0xfb840108,0xb6ae0101,0xfb840108,0xb6af0101,0xfb840108,0xb6b00101,0xfb840108,0xb6b10101,
	0xfb840108,0xb6b20101,0xfb840108,0xb6b30101,0xfb840108,0xb6b40101,0xfb840108,0xb6b50101,
	0xfb840108,0xb6b60101,0xfb840108,0xb6b70101,0xfb840108,0xb6b80101,0xfb840108,0xb6b90101,
	0xfb840108,0xb6ba0101,0xfb840108,0xb6bb0101,0xfb840108,0xb6bc0101,0xfb840108,0xb6bd0101,
	0xfb840108,0xb6be0101,0xfb840108,0xb6bf0101,0xfb840108,0xb6c00101,0xfb840108,0xb6c10101,
	0xfb840108,0xb6c20101,0xfb840108,0xb6c30101,0xfb840108,0xb6c40101,0xfb840108,0xb6c50101,
	0xfb840108,0xb6c60101,0xfb840108,0xb6c70101,0xfb840108,0xb6c80101,0xfb840108,0xb6c90101,
	0xfb840108,0xb6ca0101,0xfb840108,0xb6cb0101,0xfb840108,0xb6cc0101,0xfb840108,0xb6cd0101,
	0xfb840108,0xb6ce0101,0xfb840108,0xb6cf0101,0xfb840108,0xb6d00101,0xfb840108,0xb6d10101,
	0xfb840108,0xb6d20101,0xfb840108,0xb6d30101,0xfb840108,0xb6d40101,0xfb840108,0xb6d50101,
	0xfb840108,0xb6d60101,0xfb840108,0xb6d70101,0xfb840108,0xb6d80101,0xfb840108,0xb6d90101,
	0xfb840108,0xb6da0101,0xfb840108,0xb6db0101,0xfb840108,0xb6dc0101,0xfb840108,0xb6dd0101,
	0xfb840108,0xb6de0101,0xfb840108,0xb6df0101,0xfb840108,0xb6e00101,0xfb840108,0xb6e10101,
	0xfb840108,0xb6e20101,0xfb840108,0xb6e30101,0xfb840108,0xb6e40101,0xfb840108,0xb6e50101,
	0xfb840108,0xb6e60101,0xfb840108,0xb6e70101,0xfb840108,0xb6e80101,0xfb840108,0xb6e90101,
	0xfb840108,0xb6ea0101,0xfb840108,0xb6eb0101,0xfb840108,0xb6ec0101,0xfb840108,0xb6ed0101,
	0xfb840108,0xb6ee0101,0xfb840108,0xb6ef0101,0xfb840108,0xb6f00101,0xfb840108,0xb6f10101,
	0xfb840108,0xb6f20101,0xfb840108,0xb6f30101,0xfb840108,0xb6f40101,0xfb840108,0xb6f50101,
	0xfb840108,0xb6f60101,0xfb840108,0xb6f70101,0xfb840108,0xb6f80101,0xfb840108,0xb6f90101,
	0xfb840108,0xb6fa0101,0xfb840108,0xb6fb0101,0xfb840108,0xb6fc0101,0xfb840108,0xb6fd0101,
	0xfb840108,0xb6fe0101,0xfb840108,0xb6ff0101,0xfb840108,0xb7000101,0xfb840108,0xb7010101,
	0xfb840108,0xb7020101,0xfb840108,0xb7030101,0xfb840108,0xb7040101,0xfb840108,0xb7050101,
	0xfb840108,0xb7060101,0xfb840108,0xb7070101,0xfb840108,0xb7080101,0xfb840108,0xb7090101,
	0xfb840108,0xb70a0101,0xfb840108,0xb70b0101,0xfb840108,0xb70c0101,0xfb840108,0xb70d0101,
	0xfb840108,0xb70e0101,0xfb840108,0xb70f0101,0xfb840108,0xb7100101,0xfb840108,0xb7110101,
	0xfb840108,0xb7120101,0xfb840108,0xb7130101,0xfb840108,0xb7140101,0xfb840108,0xb7150101,
	0xfb840108,0xb7160101,0xfb840108,0xb7170101,0xfb840108,0xb7180101,0xfb840108,0xb7190101,
	0xfb840108,0xb71a0101,0xfb840108,0xb71b0101,0xfb840108,0xb71c0101,0xfb840108,0xb71d0101,
	0xfb840108,0xb71e0101,0xfb840108,0xb71f0101,0xfb840108,0xb7200101,0xfb840108,0xb7210101,
	0xfb840108,0xb7220101,0xfb840108,0xb7230101,0xfb840108,0xb7240101,0xfb840108,0xb7250101,
	0xfb840108,0xb7260101,0xfb840108,0xb7270101,0xfb840108,0xb7280101,0xfb840108,0xb7290101,
	0xfb840108,0xb72a0101,0xfb840108,0xb72b0101,0xfb840108,0xb72c0101,0xfb840108,0xb72d0101,
	0xfb840108,0xb72e0101,0xfb840108,0xb72f0101,0xfb840108,0xb7300101,0xfb840108,0xb7310101,
	0xfb840108,0xb7320101,0xfb840108,0xb7330101,0xfb840108,0xb7340101,0xfb840108,0xb7350101,
	0xfb840108,0xb7360101,0xfb840108,0xb7370101,0xfb840108,0xb7380101,0xfb840108,0xb7390101,
	0xfb840108,0xb73a0101,0xfb840108,0xb73b0101,0xfb840108,0xb73c0101,0xfb840108,0xb73d0101,
	0xfb840108,0xb73e0101,0xfb840108,0xb73f0101,0xfb840108,0xb7400101,0xfb840108,0xb7410101,
	0xfb840108,0xb7420101,0xfb840108,0xb7430101,0xfb840108,0xb7440101,0xfb840108,0xb7450101,
	0xfb840108,0xb7460101,0xfb840108,0xb7470101,0xfb840108,0xb7480101,0xfb840108,0xb7490101,
	0xfb840108,0xb74a0101,0xfb840108,0xb74b0101,0xfb840108,0xb74c0101,0xfb840108,0xb74d0101,
	0xfb840108,0xb74e0101,0xfb840108,0xb74f0101,0xfb840108,0xb7500101,0xfb840108,0xb7510101,
	0xfb840108,0xb7520101,0xfb840108,0xb7530101,0xfb840108,0xb7540101,0xfb840108,0xb7550101,
	0xfb840108,0xb7560101,0xfb840108,0xb7570101,0xfb840108,0xb7580101,0xfb840108,0xb7590101,
	0xfb840108,0xb75a0101,0xfb840108,0xb75b0101,0xfb840108,0xb75c0101,0xfb840108,0xb75d0101,
	0xfb840108,0xb75e0101,0xfb840108,0xb75f0101,0xfb840108,0xb7600101,0xfb840108,0xb7610101,
	0xfb840108,0xb7620101,0xfb840108,0xb7630101,0xfb840108,0xb7640101,0xfb840108,0xb7650101,
	0xfb840108,0xb7660101,0xfb840108,0xb7670101,0xfb840108,0xb7680101,0xfb840108,0xb7690101,
	0xfb840108,0xb76a0101,0xfb840108,0xb76b0101,0xfb840108,0xb76c0101,0xfb840108,0xb76d0101,
	0xfb840108,0xb76e0101,0xfb840108,0xb76f0101,0xfb840108,0xb7700101,0xfb840108,0xb7710101,
	0xfb840108,0xb7720101,0xfb840108,0xb7730101,0xfb840108,0xb7740101,0xfb840108,0xb7750101,
	0xfb840108,0xb7760101,0xfb840108,0xb7770101,0xfb840108,0xb7780101,0xfb840108,0xb7790101,
	0xfb840108,0xb77a0101,0xfb840108,0xb77b0101,0xfb840108,0xb77c0101,0xfb840108,0xb77d0101,
	0xfb840108,0xb77e0101,0xfb840108,0xb77f0101,0xfb840108,0xb7800101,0xfb840108,0xb7810101,
	0xfb840108,0xb7820101,0xfb840108,0xb7830101,0xfb840108,0xb7840101,0xfb840108,0xb7850101,
	0xfb840108,0xb7860101,0xfb840108,0xb7870101,0xfb840108,0xb7880101,0xfb840108,0xb7890101,
	0xfb840108,0xb78a0101,0xfb840108,0xb78b0101,0xfb840108,0xb78c0101,0xfb840108,0xb78d0101,
	0xfb840108,0xb78e0101,0xfb840108,0xb78f0101,0xfb840108,0xb7900101,0xfb840108,0xb7910101,
	0xfb840108,0xb7920101,0xfb840108,0xb7930101,0xfb840108,0xb7940101,0xfb840108,0xb7950101,
	0xfb840108,0xb7960101,0xfb840108,0xb7970101,0xfb840108,0xb7980101,0xfb840108,0xb7990101,
	0xfb840108,0xb79a0101,0xfb840108,0xb79b0101,0xfb840108,0xb79c0101,0xfb840108,0xb79d0101,
	0xfb840108,0xb79e0101,0xfb840108,0xb79f0101,0xfb840108,0xb7a00101,0xfb840108,0xb7a10101,
	0xfb840108,0xb7a20101,0xfb840108,0xb7a30101,0xfb840108,0xb7a40101,0xfb840108,0xb7a50101,
	0xfb840108,0xb7a60101,0xfb840108,0xb7a70101,0xfb840108,0xb7a80101,0xfb840108,0xb7a90101,
	0xfb840108,0xb7aa0101,0xfb840108,0xb7ab0101,0xfb840108,0xb7ac0101,0xfb840108,0xb7ad0101,
	0xfb840108,0xb7ae0101,0xfb840108,0xb7af0101,0xfb840108,0xb7b00101,0xfb840108,0xb7b10101,
	0xfb840108,0xb7b20101,0xfb840108,0xb7b30101,0xfb840108,0xb7b40101,0xfb840108,0xb7b50101,
	0xfb840108,0xb7b60101,0xfb840108,0xb7b70101,0xfb840108,0xb7b80101,0xfb840108,0xb7b90101,
	0xfb840108,0xb7ba0101,0xfb840108,0xb7bb0101,0xfb840108,0xb7bc0101,0xfb840108,0xb7bd0101,
	0xfb840108,0xb7be0101,0xfb840108,0xb7bf0101,0xfb840108,0xb7c00101,0xfb840108,0xb7c10101,
	0xfb840108,0xb7c20101,0xfb840108,0xb7c30101,0xfb840108,0xb7c40101,0xfb840108,0xb7c50101,
	0xfb840108,0xb7c60101,0xfb840108,0xb7c70101,0xfb840108,0xb7c80101,0xfb840108,0xb7c90101,
	0xfb840108,0xb7ca0101,0xfb840108,0xb7cb0101,0xfb840108,0xb7cc0101,0xfb840108,0xb7cd0101,
	0xfb840108,0xb7ce0101,0xfb840108,0xb7cf0101,0xfb840108,0xb7d00101,0xfb840108,0xb7d10101,
	0xfb840108,0xb7d20101,0xfb840108,0xb7d30101,0xfb840108,0xb7d40101,0xfb840108,0xb7d50101,
	0xfb840108,0xb7d60101,0xfb840108,0xb7d70101,0xfb840108,0xb7d80101,0xfb840108,0xb7d90101,
	0xfb840108,0xb7da0101,0xfb840108,0xb7db0101,0xfb840108,0xb7dc0101,0xfb840108,0xb7dd0101,
	0xfb840108,0xb7de0101,0xfb840108,0xb7df0101,0xfb840108,0xb7e00101,0xfb840108,0xb7e10101,
	0xfb840108,0xb7e20101,0xfb840108,0xb7e30101,0xfb840108,0xb7e40101,0xfb840108,0xb7e50101,
	0xfb840108,0xb7e60101,0xfb840108,0xb7e70101,0xfb840108,0xb7e80101,0xfb840108,0xb7e90101,
	0xfb840108,0xb7ea0101,0xfb840108,0xb7eb0101,0xfb840108,0xb7ec0101,0xfb840108,0xb7ed0101,
	0xfb840108,0xb7ee0101,0xfb840108,0xb7ef0101,0xfb840108,0xb7f00101,0xfb840108,0xb7f10101,
	0xfb840108,0xb7f20101,0xfb840108,0xb7f30101,0xfb840108,0xb7f40101,0xfb840108,0xb7f50101,
	0xfb840108,0xb7f60101,0xfb840108,0xb7f70101,0xfb840108,0xb7f80101,0xfb840108,0xb7f90101,
	0xfb840108,0xb7fa0101,0xfb840108,0xb7fb0101,0xfb840108,0xb7fc0101,0xfb840108,0xb7fd0101,
	0xfb840108,0xb7fe0101,0xfb840108,0xb7ff0101,0xfb840108,0xb8000101,0xfb840108,0xb8010101,
	0xfb840108,0xb8020101,0xfb840108,0xb8030101,0xfb840108,0xb8040101,0xfb840108,0xb8050101,
	0xfb840108,0xb8060101,0xfb840108,0xb8070101,0xfb840108,0xb8080101,0xfb840108,0xb8090101,
	0xfb840108,0xb80a0101,0xfb840108,0xb80b0101,0xfb840108,0xb80c0101,0xfb840108,0xb80d0101,
	0xfb840108,0xb80e0101,0xfb840108,0xb80f0101,0xfb840108,0xb8100101,0xfb840108,0xb8110101,
	0xfb840108,0xb8120101,0xfb840108,0xb8130101,0xfb840108,0xb8140101,0xfb840108,0xb8150101,
	0xfb840108,0xb8160101,0xfb840108,0xb8170101,0xfb840108,0xb8180101,0xfb840108,0xb8190101,
	0xfb840108,0xb81a0101,0xfb840108,0xb81b0101,0xfb840108,0xb81c0101,0xfb840108,0xb81d0101,
	0xfb840108,0xb81e0101,0xfb840108,0xb81f0101,0xfb840108,0xb8200101,0xfb840108,0xb8210101,
	0xfb840108,0xb8220101,0xfb840108,0xb8230101,0xfb840108,0xb8240101,0xfb840108,0xb8250101,
	0xfb840108,0xb8260101,0xfb840108,0xb8270101,0xfb840108,0xb8280101,0xfb840108,0xb8290101,
	0xfb840108,0xb82a0101,0xfb840108,0xb82b0101,0xfb840108,0xb82c0101,0xfb840108,0xb82d0101,
	0xfb840108,0xb82e0101,0xfb840108,0xb82f0101,0xfb840108,0xb8300101,0xfb840108,0xb8310101,
	0xfb840108,0xb8320101,0xfb840108,0xb8330101,0xfb840108,0xb8340101,0xfb840108,0xb8350101,
	0xfb840108,0xb8360101,0xfb840108,0xb8370101,0xfb840108,0xb8380101,0xfb840108,0xb8390101,
	0xfb840108,0xb83a0101,0xfb840108,0xb83b0101,0xfb840108,0xb83c0101,0xfb840108,0xb83d0101,
	0xfb840108,0xb83e0101,0xfb840108,0xb83f0101,0xfb840108,0xb8400101,0xfb840108,0xb8410101,
	0xfb840108,0xb8420101,0xfb840108,0xb8430101,0xfb840108,0xb8440101,0xfb840108,0xb8450101,
	0xfb840108,0xb8460101,0xfb840108,0xb8470101,0xfb840108,0xb8480101,0xfb840108,0xb8490101,
	0xfb840108,0xb84a0101,0xfb840108,0xb84b0101,0xfb840108,0xb84c0101,0xfb840108,0xb84d0101,
	0xfb840108,0xb84e0101,0xfb840108,0xb84f0101,0xfb840108,0xb8500101,0xfb840108,0xb8510101,
	0xfb840108,0xb8520101,0xfb840108,0xb8530101,0xfb840108,0xb8540101,0xfb840108,0xb8550101,
	0xfb840108,0xb8560101,0xfb840108,0xb8570101,0xfb840108,0xb8580101,0xfb840108,0xb8590101,
	0xfb840108,0xb85a0101,0xfb840108,0xb85b0101,0xfb840108,0xb85c0101,0xfb840108,0xb85d0101,
	0xfb840108,0xb85e0101,0xfb840108,0xb85f0101,0xfb840108,0xb8600101,0xfb840108,0xb8610101,
	0xfb840108,0xb8620101,0xfb840108,0xb8630101,0xfb840108,0xb8640101,0xfb840108,0xb8650101,
	0xfb840108,0xb8660101,0xfb840108,0xb8670101,0xfb840108,0xb8680101,0xfb840108,0xb8690101,
	0xfb840108,0xb86a0101,0xfb840108,0xb86b0101,0xfb840108,0xb86c0101,0xfb840108,0xb86d0101,
	0xfb840108,0xb86e0101,0xfb840108,0xb86f0101,0xfb840108,0xb8700101,0xfb840108,0xb8710101,
	0xfb840108,0xb8720101,0xfb840108,0xb8730101,0xfb840108,0xb8740101,0xfb840108,0xb8750101,
	0xfb840108,0xb8760101,0xfb840108,0xb8770101,0xfb840108,0xb8780101,0xfb840108,0xb8790101,
	0xfb840108,0xb87a0101,0xfb840108,0xb87b0101,0xfb840108,0xb87c0101,0xfb840108,0xb87d0101,
	0xfb840108,0xb87e0101,0xfb840108,0xb87f0101,0xfb840108,0xb8800101,0xfb840108,0xb8810101,
	0xfb840108,0xb8820101,0xfb840108,0xb8830101,0xfb840108,0xb8840101,0xfb840108,0xb8850101,
	0xfb840108,0xb8860101,0xfb840108,0xb8870101,0xfb840108,0xb8880101,0xfb840108,0xb8890101,
	0xfb840108,0xb88a0101,0xfb840108,0xb88b0101,0xfb840108,0xb88c0101,0xfb840108,0xb88d0101,
	0xfb840108,0xb88e0101,0xfb840108,0xb88f0101,0xfb840108,0xb8900101,0xfb840108,0xb8910101,
	0xfb840108,0xb8920101,0xfb840108,0xb8930101,0xfb840108,0xb8940101,0xfb840108,0xb8950101,
	0xfb840108,0xb8960101,0xfb840108,0xb8970101,0xfb840108,0xb8980101,0xfb840108,0xb8990101,
	0xfb840108,0xb89a0101,0xfb840108,0xb89b0101,0xfb840108,0xb89c0101,0xfb840108,0xb89d0101,
	0xfb840108,0xb89e0101,0xfb840108,0xb89f0101,0xfb840108,0xb8a00101,0xfb840108,0xb8a10101,
	0xfb840108,0xb8a20101,0xfb840108,0xb8a30101,0xfb840108,0xb8a40101,0xfb840108,0xb8a50101,
	0xfb840108,0xb8a60101,0xfb840108,0xb8a70101,0xfb840108,0xb8a80101,0xfb840108,0xb8a90101,
	0xfb840108,0xb8aa0101,0xfb840108,0xb8ab0101,0xfb840108,0xb8ac0101,0xfb840108,0xb8ad0101,
	0xfb840108,0xb8ae0101,0xfb840108,0xb8af0101,0xfb840108,0xb8b00101,0xfb840108,0xb8b10101,
	0xfb840108,0xb8b20101,0xfb840108,0xb8b30101,0xfb840108,0xb8b40101,0xfb840108,0xb8b50101,
	0xfb840108,0xb8b60101,0xfb840108,0xb8b70101,0xfb840108,0xb8b80101,0xfb840108,0xb8b90101,
	0xfb840108,0xb8ba0101,0xfb840108,0xb8bb0101,0xfb840108,0xb8bc0101,0xfb840108,0xb8bd0101,
	0xfb840108,0xb8be0101,0xfb840108,0xb8bf0101,0xfb840108,0xb8c00101,0xfb840108,0xb8c10101,
	0xfb840108,0xb8c20101,0xfb840108,0xb8c30101,0xfb840108,0xb8c40101,0xfb840108,0xb8c50101,
	0xfb840108,0xb8c60101,0xfb840108,0xb8c70101,0xfb840108,0xb8c80101,0xfb840108,0xb8c90101,
	0xfb840108,0xb8ca0101,0xfb840108,0xb8cb0101,0xfb840108,0xb8cc0101,0xfb840108,0xb8cd0101,
	0xfb840108,0xb8ce0101,0xfb840108,0xb8cf0101,0xfb840108,0xb8d00101,0xfb840108,0xb8d10101,
	0xfb840108,0xb8d20101,0xfb840108,0xb8d30101,0xfb840108,0xb8d40101,0xfb840108,0xb8d50101,
	0xfb840108,0xb8d60101,0xfb840108,0xb8d70101,0xfb840108,0xb8d80101,0xfb840108,0xb8d90101,
	0xfb840108,0xb8da0101,0xfb840108,0xb8db0101,0xfb840108,0xb8dc0101,0xfb840108,0xb8dd0101,
	0xfb840108,0xb8de0101,0xfb840108,0xb8df0101,0xfb840108,0xb8e00101,0xfb840108,0xb8e10101,
	0xfb840108,0xb8e20101,0xfb840108,0xb8e30101,0xfb840108,0xb8e40101,0xfb840108,0xb8e50101,
	0xfb840108,0xb8e60101,0xfb840108,0xb8e70101,0xfb840108,0xb8e80101,0xfb840108,0xb8e90101,
	0xfb840108,0xb8ea0101,0xfb840108,0xb8eb0101,0xfb840108,0xb8ec0101,0xfb840108,0xb8ed0101,
	0xfb840108,0xb8ee0101,0xfb840108,0xb8ef0101,0xfb840108,0xb8f00101,0xfb840108,0xb8f10101,
	0xfb840108,0xb8f20101,0xfb840108,0xb8f30101,0xfb840108,0xb8f40101,0xfb840108,0xb8f50101,
	0xfb840108,0xb8f60101,0xfb840108,0xb8f70101,0xfb840108,0xb8f80101,0xfb840108,0xb8f90101,
	0xfb840108,0xb8fa0101,0xfb840108,0xb8fb0101,0xfb840108,0xb8fc0101,0xfb840108,0xb8fd0101,
	0xfb840108,0xb8fe0101,0xfb840108,0xb8ff0101,0xfb840108,0xb9000101,0xfb840108,0xb9010101,
	0xfb840108,0xb9020101,0xfb840108,0xb9030101,0xfb840108,0xb9040101,0xfb840108,0xb9050101,
	0xfb840108,0xb9060101,0xfb840108,0xb9070101,0xfb840108,0xb9080101,0xfb840108,0xb9090101,
	0xfb840108,0xb90a0101,0xfb840108,0xb90b0101,0xfb840108,0xb90c0101,0xfb840108,0xb90d0101,
	0xfb840108,0xb90e0101,0xfb840108,0xb90f0101,0xfb840108,0xb9100101,0xfb840108,0xb9110101,
	0xfb840108,0xb9120101,0xfb840108,0xb9130101,0xfb840108,0xb9140101,0xfb840108,0xb9150101,
	0xfb840108,0xb9160101,0xfb840108,0xb9170101,0xfb840108,0xb9180101,0xfb840108,0xb9190101,
	0xfb840108,0xb91a0101,0xfb840108,0xb91b0101,0xfb840108,0xb91c0101,0xfb840108,0xb91d0101,
	0xfb840108,0xb91e0101,0xfb840108,0xb91f0101,0xfb840108,0xb9200101,0xfb840108,0xb9210101,
	0xfb840108,0xb9220101,0xfb840108,0xb9230101,0xfb840108,0xb9240101,0xfb840108,0xb9250101,
	0xfb840108,0xb9260101,0xfb840108,0xb9270101,0xfb840108,0xb9280101,0xfb840108,0xb9290101,
	0xfb840108,0xb92a0101,0xfb840108,0xb92b0101,0xfb840108,0xb92c0101,0xfb840108,0xb92d0101,
	0xfb840108,0xb92e0101,0xfb840108,0xb92f0101,0xfb840108,0xb9300101,0xfb840108,0xb9310101,
	0xfb840108,0xb9320101,0xfb840108,0xb9330101,0xfb840108,0xb9340101,0xfb840108,0xb9350101,
	0xfb840108,0xb9360101,0xfb840108,0xb9370101,0xfb840108,0xb9380101,0xfb840108,0xb9390101,
	0xfb840108,0xb93a0101,0xfb840108,0xb93b0101,0xfb840108,0xb93c0101,0xfb840108,0xb93d0101,
	0xfb840108,0xb93e0101,0xfb840108,0xb93f0101,0xfb840108,0xb9400101,0xfb840108,0xb9410101,
	0xfb840108,0xb9420101,0xfb840108,0xb9430101,0xfb840108,0xb9440101,0xfb840108,0xb9450101,
	0xfb840108,0xb9460101,0xfb840108,0xb9470101,0xfb840108,0xb9480101,0xfb840108,0xb9490101,
	0xfb840108,0xb94a0101,0xfb840108,0xb94b0101,0xfb840108,0xb94c0101,0xfb840108,0xb94d0101,
	0xfb840108,0xb94e0101,0xfb840108,0xb94f0101,0xfb840108,0xb9500101,0xfb840108,0xb9510101,
	0xfb840108,0xb9520101,0xfb840108,0xb9530101,0xfb840108,0xb9540101,0xfb840108,0xb9550101,
	0xfb840108,0xb9560101,0xfb840108,0xb9570101,0xfb840108,0xb9580101,0xfb840108,0xb9590101,
	0xfb840108,0xb95a0101,0xfb840108,0xb95b0101,0xfb840108,0xb95c0101,0xfb840108,0xb95d0101,
	0xfb840108,0xb95e0101,0xfb840108,0xb95f0101,0xfb840108,0xb9600101,0xfb840108,0xb9610101,
	0xfb840108,0xb9620101,0xfb840108,0xb9630101,0xfb840108,0xb9640101,0xfb840108,0xb9650101,
	0xfb840108,0xb9660101,0xfb840108,0xb9670101,0xfb840108,0xb9680101,0xfb840108,0xb9690101,
	0xfb840108,0xb96a0101,0xfb840108,0xb96b0101,0xfb840108,0xb96c0101,0xfb840108,0xb96d0101,
	0xfb840108,0xb96e0101,0xfb840108,0xb96f0101,0xfb840108,0xb9700101,0xfb840108,0xb9710101,
	0xfb840108,0xb9720101,0xfb840108,0xb9730101,0xfb840108,0xb9740101,0xfb840108,0xb9750101,
	0xfb840108,0xb9760101,0xfb840108,0xb9770101,0xfb840108,0xb9780101,0xfb840108,0xb9790101,
	0xfb840108,0xb97a0101,0xfb840108,0xb97b0101,0xfb840108,0xb97c0101,0xfb840108,0xb97d0101,
	0xfb840108,0xb97e0101,0xfb840108,0xb97f0101,0xfb840108,0xb9800101,0xfb840108,0xb9810101,
	0xfb840108,0xb9820101,0xfb840108,0xb9830101,0xfb840108,0xb9840101,0xfb840108,0xb9850101,
	0xfb840108,0xb9860101,0xfb840108,0xb9870101,0xfb840108,0xb9880101,0xfb840108,0xb9890101,
	0xfb840108,0xb98a0101,0xfb840108,0xb98b0101,0xfb840108,0xb98c0101,0xfb840108,0xb98d0101,
	0xfb840108,0xb98e0101,0xfb840108,0xb98f0101,0xfb840108,0xb9900101,0xfb840108,0xb9910101,
	0xfb840108,0xb9920101,0xfb840108,0xb9930101,0xfb840108,0xb9940101,0xfb840108,0xb9950101,
	0xfb840108,0xb9960101,0xfb840108,0xb9970101,0xfb840108,0xb9980101,0xfb840108,0xb9990101,
	0xfb840108,0xb99a0101,0xfb840108,0xb99b0101,0xfb840108,0xb99c0101,0xfb840108,0xb99d0101,
	0xfb840108,0xb99e0101,0xfb840108,0xb99f0101,0xfb840108,0xb9a00101,0xfb840108,0xb9a10101,
	0xfb840108,0xb9a20101,0xfb840108,0xb9a30101,0xfb840108,0xb9a40101,0xfb840108,0xb9a50101,
	0xfb840108,0xb9a60101,0xfb840108,0xb9a70101,0xfb840108,0xb9a80101,0xfb840108,0xb9a90101,
	0xfb840108,0xb9aa0101,0xfb840108,0xb9ab0101,0xfb840108,0xb9ac0101,0xfb840108,0xb9ad0101,
	0xfb840108,0xb9ae0101,0xfb840108,0xb9af0101,0xfb840108,0xb9b00101,0xfb840108,0xb9b10101,
	0xfb840108,0xb9b20101,0xfb840108,0xb9b30101,0xfb840108,0xb9b40101,0xfb840108,0xb9b50101,
	0xfb840108,0xb9b60101,0xfb840108,0xb9b70101,0xfb840108,0xb9b80101,0xfb840108,0xb9b90101,
	0xfb840108,0xb9ba0101,0xfb840108,0xb9bb0101,0xfb840108,0xb9bc0101,0xfb840108,0xb9bd0101,
	0xfb840108,0xb9be0101,0xfb840108,0xb9bf0101,0xfb840108,0xb9c00101,0xfb840108,0xb9c10101,
	0xfb840108,0xb9c20101,0xfb840108,0xb9c30101,0xfb840108,0xb9c40101,0xfb840108,0xb9c50101,
	0xfb840108,0xb9c60101,0xfb840108,0xb9c70101,0xfb840108,0xb9c80101,0xfb840108,0xb9c90101,
	0xfb840108,0xb9ca0101,0xfb840108,0xb9cb0101,0xfb840108,0xb9cc0101,0xfb840108,0xb9cd0101,
	0xfb840108,0xb9ce0101,0xfb840108,0xb9cf0101,0xfb840108,0xb9d00101,0xfb840108,0xb9d10101,
	0xfb840108,0xb9d20101,0xfb840108,0xb9d30101,0xfb840108,0xb9d40101,0xfb840108,0xb9d50101,
	0xfb840108,0xb9d60101,0xfb840108,0xb9d70101,0xfb840108,0xb9d80101,0xfb840108,0xb9d90101,
	0xfb840108,0xb9da0101,0xfb840108,0xb9db0101,0xfb840108,0xb9dc0101,0xfb840108,0xb9dd0101,
	0xfb840108,0xb9de0101,0xfb840108,0xb9df0101,0xfb840108,0xb9e00101,0xfb840108,0xb9e10101,
	0xfb840108,0xb9e20101,0xfb840108,0xb9e30101,0xfb840108,0xb9e40101,0xfb840108,0xb9e50101,
	0xfb840108,0xb9e60101,0xfb840108,0xb9e70101,0xfb840108,0xb9e80101,0xfb840108,0xb9e90101,
	0xfb840108,0xb9ea0101,0xfb840108,0xb9eb0101,0xfb840108,0xb9ec0101,0xfb840108,0xb9ed0101,
	0xfb840108,0xb9ee0101,0xfb840108,0xb9ef0101,0xfb840108,0xb9f00101,0xfb840108,0xb9f10101,
	0xfb840108,0xb9f20101,0xfb840108,0xb9f30101,0xfb840108,0xb9f40101,0xfb840108,0xb9f50101,
	0xfb840108,0xb9f60101,0xfb840108,0xb9f70101,0xfb840108,0xb9f80101,0xfb840108,0xb9f90101,
	0xfb840108,0xb9fa0101,0xfb840108,0xb9fb0101,0xfb840108,0xb9fc0101,0xfb840108,0xb9fd0101,
	0xfb840108,0xb9fe0101,0xfb840108,0xb9ff0101,0xfb840108,0xba000101,0xfb840108,0xba010101,
	0xfb840108,0xba020101,0xfb840108,0xba030101,0xfb840108,0xba040101,0xfb840108,0xba050101,
	0xfb840108,0xba060101,0xfb840108,0xba070101,0xfb840108,0xba080101,0xfb840108,0xba090101,
	0xfb840108,0xba0a0101,0xfb840108,0xba0b0101,0xfb840108,0xba0c0101,0xfb840108,0xba0d0101,
	0xfb840108,0xba0e0101,0xfb840108,0xba0f0101,0xfb840108,0xba100101,0xfb840108,0xba110101,
	0xfb840108,0xba120101,0xfb840108,0xba130101,0xfb840108,0xba140101,0xfb840108,0xba150101,
	0xfb840108,0xba160101,0xfb840108,0xba170101,0xfb840108,0xba180101,0xfb840108,0xba190101,
	0xfb840108,0xba1a0101,0xfb840108,0xba1b0101,0xfb840108,0xba1c0101,0xfb840108,0xba1d0101,
	0xfb840108,0xba1e0101,0xfb840108,0xba1f0101,0xfb840108,0xba200101,0xfb840108,0xba210101,
	0xfb840108,0xba220101,0xfb840108,0xba230101,0xfb840108,0xba240101,0xfb840108,0xba250101,
	0xfb840108,0xba260101,0xfb840108,0xba270101,0xfb840108,0xba280101,0xfb840108,0xba290101,
	0xfb840108,0xba2a0101,0xfb840108,0xba2b0101,0xfb840108,0xba2c0101,0xfb840108,0xba2d0101,
	0xfb840108,0xba2e0101,0xfb840108,0xba2f0101,0xfb840108,0xba300101,0xfb840108,0xba310101,
	0xfb840108,0xba320101,0xfb840108,0xba330101,0xfb840108,0xba340101,0xfb840108,0xba350101,
	0xfb840108,0xba360101,0xfb840108,0xba370101,0xfb840108,0xba380101,0xfb840108,0xba390101,
	0xfb840108,0xba3a0101,0xfb840108,0xba3b0101,0xfb840108,0xba3c0101,0xfb840108,0xba3d0101,
	0xfb840108,0xba3e0101,0xfb840108,0xba3f0101,0xfb840108,0xba400101,0xfb840108,0xba410101,
	0xfb840108,0xba420101,0xfb840108,0xba430101,0xfb840108,0xba440101,0xfb840108,0xba450101,
	0xfb840108,0xba460101,0xfb840108,0xba470101,0xfb840108,0xba480101,0xfb840108,0xba490101,
	0xfb840108,0xba4a0101,0xfb840108,0xba4b0101,0xfb840108,0xba4c0101,0xfb840108,0xba4d0101,
	0xfb840108,0xba4e0101,0xfb840108,0xba4f0101,0xfb840108,0xba500101,0xfb840108,0xba510101,
	0xfb840108,0xba520101,0xfb840108,0xba530101,0xfb840108,0xba540101,0xfb840108,0xba550101,
	0xfb840108,0xba560101,0xfb840108,0xba570101,0xfb840108,0xba580101,0xfb840108,0xba590101,
	0xfb840108,0xba5a0101,0xfb840108,0xba5b0101,0xfb840108,0xba5c0101,0xfb840108,0xba5d0101,
	0xfb840108,0xba5e0101,0xfb840108,0xba5f0101,0xfb840108,0xba600101,0xfb840108,0xba610101,
	0xfb840108,0xba620101,0xfb840108,0xba630101,0xfb840108,0xba640101,0xfb840108,0xba650101,
	0xfb840108,0xba660101,0xfb840108,0xba670101,0xfb840108,0xba680101,0xfb840108,0xba690101,
	0xfb840108,0xba6a0101,0xfb840108,0xba6b0101,0xfb840108,0xba6c0101,0xfb840108,0xba6d0101,
	0xfb840108,0xba6e0101,0xfb840108,0xba6f0101,0xfb840108,0xba700101,0xfb840108,0xba710101,
	0xfb840108,0xba720101,0xfb840108,0xba730101,0xfb840108,0xba740101,0xfb840108,0xba750101,
	0xfb840108,0xba760101,0xfb840108,0xba770101,0xfb840108,0xba780101,0xfb840108,0xba790101,
	0xfb840108,0xba7a0101,0xfb840108,0xba7b0101,0xfb840108,0xba7c0101,0xfb840108,0xba7d0101,
	0xfb840108,0xba7e0101,0xfb840108,0xba7f0101,0xfb840108,0xba800101,0xfb840108,0xba810101,
	0xfb840108,0xba820101,0xfb840108,0xba830101,0xfb840108,0xba840101,0xfb840108,0xba850101,
	0xfb840108,0xba860101,0xfb840108,0xba870101,0xfb840108,0xba880101,0xfb840108,0xba890101,
	0xfb840108,0xba8a0101,0xfb840108,0xba8b0101,0xfb840108,0xba8c0101,0xfb840108,0xba8d0101,
	0xfb840108,0xba8e0101,0xfb840108,0xba8f0101,0xfb840108,0xba900101,0xfb840108,0xba910101,
	0xfb840108,0xba920101,0xfb840108,0xba930101,0xfb840108,0xba940101,0xfb840108,0xba950101,
	0xfb840108,0xba960101,0xfb840108,0xba970101,0xfb840108,0xba980101,0xfb840108,0xba990101,
	0xfb840108,0xba9a0101,0xfb840108,0xba9b0101,0xfb840108,0xba9c0101,0xfb840108,0xba9d0101,
	0xfb840108,0xba9e0101,0xfb840108,0xba9f0101,0xfb840108,0xbaa00101,0xfb840108,0xbaa10101,
	0xfb840108,0xbaa20101,0xfb840108,0xbaa30101,0xfb840108,0xbaa40101,0xfb840108,0xbaa50101,
	0xfb840108,0xbaa60101,0xfb840108,0xbaa70101,0xfb840108,0xbaa80101,0xfb840108,0xbaa90101,
	0xfb840108,0xbaaa0101,0xfb840108,0xbaab0101,0xfb840108,0xbaac0101,0xfb840108,0xbaad0101,
	0xfb840108,0xbaae0101,0xfb840108,0xbaaf0101,0xfb840108,0xbab00101,0xfb840108,0xbab10101,
	0xfb840108,0xbab20101,0xfb840108,0xbab30101,0xfb840108,0xbab40101,0xfb840108,0xbab50101,
	0xfb840108,0xbab60101,0xfb840108,0xbab70101,0xfb840108,0xbab80101,0xfb840108,0xbab90101,
	0xfb840108,0xbaba0101,0xfb840108,0xbabb0101,0xfb840108,0xbabc0101,0xfb840108,0xbabd0101,
	0xfb840108,0xbabe0101,0xfb840108,0xbabf0101,0xfb840108,0xbac00101,0xfb840108,0xbac10101,
	0xfb840108,0xbac20101,0xfb840108,0xbac30101,0xfb840108,0xbac40101,0xfb840108,0xbac50101,
	0xfb840108,0xbac60101,0xfb840108,0xbac70101,0xfb840108,0xbac80101,0xfb840108,0xbac90101,
	0xfb840108,0xbaca0101,0xfb840108,0xbacb0101,0xfb840108,0xbacc0101,0xfb840108,0xbacd0101,
	0xfb840108,0xbace0101,0xfb840108,0xbacf0101,0xfb840108,0xbad00101,0xfb840108,0xbad10101,
	0xfb840108,0xbad20101,0xfb840108,0xbad30101,0xfb840108,0xbad40101,0xfb840108,0xbad50101,
	0xfb840108,0xbad60101,0xfb840108,0xbad70101,0xfb840108,0xbad80101,0xfb840108,0xbad90101,
	0xfb840108,0xbada0101,0xfb840108,0xbadb0101,0xfb840108,0xbadc0101,0xfb840108,0xbadd0101,
	0xfb840108,0xbade0101,0xfb840108,0xbadf0101,0xfb840108,0xbae00101,0xfb840108,0xbae10101,
	0xfb840108,0xbae20101,0xfb840108,0xbae30101,0xfb840108,0xbae40101,0xfb840108,0xbae50101,
	0xfb840108,0xbae60101,0xfb840108,0xbae70101,0xfb840108,0xbae80101,0xfb840108,0xbae90101,
	0xfb840108,0xbaea0101,0xfb840108,0xbaeb0101,0xfb840108,0xbaec0101,0xfb840108,0xbaed0101,
	0xfb840108,0xbaee0101,0xfb840108,0xbaef0101,0xfb840108,0xbaf00101,0xfb840108,0xbaf10101,
	0xfb840108,0xbaf20101,0xfb840108,0xbaf30101,0xfb840108,0xbaf40101,0xfb840108,0xbaf50101,
	0xfb840108,0xbaf60101,0xfb840108,0xbaf70101,0xfb840108,0xbaf80101,0xfb840108,0xbaf90101,
	0xfb840108,0xbafa0101,0xfb840108,0xbafb0101,0xfb840108,0xbafc0101,0xfb840108,0xbafd0101,
	0xfb840108,0xbafe0101,0xfb840108,0xbaff0101,0xfb840108,0xbb000101,0xfb840108,0xbb010101,
	0xfb840108,0xbb020101,0xfb840108,0xbb030101,0xfb840108,0xbb040101,0xfb840108,0xbb050101,
	0xfb840108,0xbb060101,0xfb840108,0xbb070101,0xfb840108,0xbb080101,0xfb840108,0xbb090101,
	0xfb840108,0xbb0a0101,0xfb840108,0xbb0b0101,0xfb840108,0xbb0c0101,0xfb840108,0xbb0d0101,
	0xfb840108,0xbb0e0101,0xfb840108,0xbb0f0101,0xfb840108,0xbb100101,0xfb840108,0xbb110101,
	0xfb840108,0xbb120101,0xfb840108,0xbb130101,0xfb840108,0xbb140101,0xfb840108,0xbb150101,
	0xfb840108,0xbb160101,0xfb840108,0xbb170101,0xfb840108,0xbb180101,0xfb840108,0xbb190101,
	0xfb840108,0xbb1a0101,0xfb840108,0xbb1b0101,0xfb840108,0xbb1c0101,0xfb840108,0xbb1d0101,
	0xfb840108,0xbb1e0101,0xfb840108,0xbb1f0101,0xfb840108,0xbb200101,0xfb840108,0xbb210101,
	0xfb840108,0xbb220101,0xfb840108,0xbb230101,0xfb840108,0xbb240101,0xfb840108,0xbb250101,
	0xfb840108,0xbb260101,0xfb840108,0xbb270101,0xfb840108,0xbb280101,0xfb840108,0xbb290101,
	0xfb840108,0xbb2a0101,0xfb840108,0xbb2b0101,0xfb840108,0xbb2c0101,0xfb840108,0xbb2d0101,
	0xfb840108,0xbb2e0101,0xfb840108,0xbb2f0101,0xfb840108,0xbb300101,0xfb840108,0xbb310101,
	0xfb840108,0xbb320101,0xfb840108,0xbb330101,0xfb840108,0xbb340101,0xfb840108,0xbb350101,
	0xfb840108,0xbb360101,0xfb840108,0xbb370101,0xfb840108,0xbb380101,0xfb840108,0xbb390101,
	0xfb840108,0xbb3a0101,0xfb840108,0xbb3b0101,0xfb840108,0xbb3c0101,0xfb840108,0xbb3d0101,
	0xfb840108,0xbb3e0101,0xfb840108,0xbb3f0101,0xfb840108,0xbb400101,0xfb840108,0xbb410101,
	0xfb840108,0xbb420101,0xfb840108,0xbb430101,0xfb840108,0xbb440101,0xfb840108,0xbb450101,
	0xfb840108,0xbb460101,0xfb840108,0xbb470101,0xfb840108,0xbb480101,0xfb840108,0xbb490101,
	0xfb840108,0xbb4a0101,0xfb840108,0xbb4b0101,0xfb840108,0xbb4c0101,0xfb840108,0xbb4d0101,
	0xfb840108,0xbb4e0101,0xfb840108,0xbb4f0101,0xfb840108,0xbb500101,0xfb840108,0xbb510101,
	0xfb840108,0xbb520101,0xfb840108,0xbb530101,0xfb840108,0xbb540101,0xfb840108,0xbb550101,
	0xfb840108,0xbb560101,0xfb840108,0xbb570101,0xfb840108,0xbb580101,0xfb840108,0xbb590101,
	0xfb840108,0xbb5a0101,0xfb840108,0xbb5b0101,0xfb840108,0xbb5c0101,0xfb840108,0xbb5d0101,
	0xfb840108,0xbb5e0101,0xfb840108,0xbb5f0101,0xfb840108,0xbb600101,0xfb840108,0xbb610101,
	0xfb840108,0xbb620101,0xfb840108,0xbb630101,0xfb840108,0xbb640101,0xfb840108,0xbb650101,
	0xfb840108,0xbb660101,0xfb840108,0xbb670101,0xfb840108,0xbb680101,0xfb840108,0xbb690101,
	0xfb840108,0xbb6a0101,0xfb840108,0xbb6b0101,0xfb840108,0xbb6c0101,0xfb840108,0xbb6d0101,
	0xfb840108,0xbb6e0101,0xfb840108,0xbb6f0101,0xfb840108,0xbb700101,0xfb840108,0xbb710101,
	0xfb840108,0xbb720101,0xfb840108,0xbb730101,0xfb840108,0xbb740101,0xfb840108,0xbb750101,
	0xfb840108,0xbb760101,0xfb840108,0xbb770101,0xfb840108,0xbb780101,0xfb840108,0xbb790101,
	0xfb840108,0xbb7a0101,0xfb840108,0xbb7b0101,0xfb840108,0xbb7c0101,0xfb840108,0xbb7d0101,
	0xfb840108,0xbb7e0101,0xfb840108,0xbb7f0101,0xfb840108,0xbb800101,0xfb840108,0xbb810101,
	0xfb840108,0xbb820101,0xfb840108,0xbb830101,0xfb840108,0xbb840101,0xfb840108,0xbb850101,
	0xfb840108,0xbb860101,0xfb840108,0xbb870101,0xfb840108,0xbb880101,0xfb840108,0xbb890101,
	0xfb840108,0xbb8a0101,0xfb840108,0xbb8b0101,0xfb840108,0xbb8c0101,0xfb840108,0xbb8d0101,
	0xfb840108,0xbb8e0101,0xfb840108,0xbb8f0101,0xfb840108,0xbb900101,0xfb840108,0xbb910101,
	0xfb840108,0xbb920101,0xfb840108,0xbb930101,0xfb840108,0xbb940101,0xfb840108,0xbb950101,
	0xfb840108,0xbb960101,0xfb840108,0xbb970101,0xfb840108,0xbb980101,0xfb840108,0xbb990101,
	0xfb840108,0xbb9a0101,0xfb840108,0xbb9b0101,0xfb840108,0xbb9c0101,0xfb840108,0xbb9d0101,
	0xfb840108,0xbb9e0101,0xfb840108,0xbb9f0101,0xfb840108,0xbba00101,0xfb840108,0xbba10101,
	0xfb840108,0xbba20101,0xfb840108,0xbba30101,0xfb840108,0xbba40101,0xfb840108,0xbba50101,
	0xfb840108,0xbba60101,0xfb840108,0xbba70101,0xfb840108,0xbba80101,0xfb840108,0xbba90101,
	0xfb840108,0xbbaa0101,0xfb840108,0xbbab0101,0xfb840108,0xbbac0101,0xfb840108,0xbbad0101,
	0xfb840108,0xbbae0101,0xfb840108,0xbbaf0101,0xfb840108,0xbbb00101,0xfb840108,0xbbb10101,
	0xfb840108,0xbbb20101,0xfb840108,0xbbb30101,0xfb840108,0xbbb40101,0xfb840108,0xbbb50101,
	0xfb840108,0xbbb60101,0xfb840108,0xbbb70101,0xfb840108,0xbbb80101,0xfb840108,0xbbb90101,
	0xfb840108,0xbbba0101,0xfb840108,0xbbbb0101,0xfb840108,0xbbbc0101,0xfb840108,0xbbbd0101,
	0xfb840108,0xbbbe0101,0xfb840108,0xbbbf0101,0xfb840108,0xbbc00101,0xfb840108,0xbbc10101,
	0xfb840108,0xbbc20101,0xfb840108,0xbbc30101,0xfb840108,0xbbc40101,0xfb840108,0xbbc50101,
	0xfb840108,0xbbc60101,0xfb840108,0xbbc70101,0xfb840108,0xbbc80101,0xfb840108,0xbbc90101,
	0xfb840108,0xbbca0101,0xfb840108,0xbbcb0101,0xfb840108,0xbbcc0101,0xfb840108,0xbbcd0101,
	0xfb840108,0xbbce0101,0xfb840108,0xbbcf0101,0xfb840108,0xbbd00101,0xfb840108,0xbbd10101,
	0xfb840108,0xbbd20101,0xfb840108,0xbbd30101,0xfb840108,0xbbd40101,0xfb840108,0xbbd50101,
	0xfb840108,0xbbd60101,0xfb840108,0xbbd70101,0xfb840108,0xbbd80101,0xfb840108,0xbbd90101,
	0xfb840108,0xbbda0101,0xfb840108,0xbbdb0101,0xfb840108,0xbbdc0101,0xfb840108,0xbbdd0101,
	0xfb840108,0xbbde0101,0xfb840108,0xbbdf0101,0xfb840108,0xbbe00101,0xfb840108,0xbbe10101,
	0xfb840108,0xbbe20101,0xfb840108,0xbbe30101,0xfb840108,0xbbe40101,0xfb840108,0xbbe50101,
	0xfb840108,0xbbe60101,0xfb840108,0xbbe70101,0xfb840108,0xbbe80101,0xfb840108,0xbbe90101,
	0xfb840108,0xbbea0101,0xfb840108,0xbbeb0101,0xfb840108,0xbbec0101,0xfb840108,0xbbed0101,
	0xfb840108,0xbbee0101,0xfb840108,0xbbef0101,0xfb840108,0xbbf00101,0xfb840108,0xbbf10101,
	0xfb840108,0xbbf20101,0xfb840108,0xbbf30101,0xfb840108,0xbbf40101,0xfb840108,0xbbf50101,
	0xfb840108,0xbbf60101,0xfb840108,0xbbf70101,0xfb840108,0xbbf80101,0xfb840108,0xbbf90101,
	0xfb840108,0xbbfa0101,0xfb840108,0xbbfb0101,0xfb840108,0xbbfc0101,0xfb840108,0xbbfd0101,
	0xfb840108,0xbbfe0101,0xfb840108,0xbbff0101,0xfb840108,0xbc000101,0xfb840108,0xbc010101,
	0xfb840108,0xbc020101,0xfb840108,0xbc030101,0xfb840108,0xbc040101,0xfb840108,0xbc050101,
	0xfb840108,0xbc060101,0xfb840108,0xbc070101,0xfb840108,0xbc080101,0xfb840108,0xbc090101,
	0xfb840108,0xbc0a0101,0xfb840108,0xbc0b0101,0xfb840108,0xbc0c0101,0xfb840108,0xbc0d0101,
	0xfb840108,0xbc0e0101,0xfb840108,0xbc0f0101,0xfb840108,0xbc100101,0xfb840108,0xbc110101,
	0xfb840108,0xbc120101,0xfb840108,0xbc130101,0xfb840108,0xbc140101,0xfb840108,0xbc150101,
	0xfb840108,0xbc160101,0xfb840108,0xbc170101,0xfb840108,0xbc180101,0xfb840108,0xbc190101,
	0xfb840108,0xbc1a0101,0xfb840108,0xbc1b0101,0xfb840108,0xbc1c0101,0xfb840108,0xbc1d0101,
	0xfb840108,0xbc1e0101,0xfb840108,0xbc1f0101,0xfb840108,0xbc200101,0xfb840108,0xbc210101,
	0xfb840108,0xbc220101,0xfb840108,0xbc230101,0xfb840108,0xbc240101,0xfb840108,0xbc250101,
	0xfb840108,0xbc260101,0xfb840108,0xbc270101,0xfb840108,0xbc280101,0xfb840108,0xbc290101,
	0xfb840108,0xbc2a0101,0xfb840108,0xbc2b0101,0xfb840108,0xbc2c0101,0xfb840108,0xbc2d0101,
	0xfb840108,0xbc2e0101,0xfb840108,0xbc2f0101,0xfb840108,0xbc300101,0xfb840108,0xbc310101,
	0xfb840108,0xbc320101,0xfb840108,0xbc330101,0xfb840108,0xbc340101,0xfb840108,0xbc350101,
	0xfb840108,0xbc360101,0xfb840108,0xbc370101,0xfb840108,0xbc380101,0xfb840108,0xbc390101,
	0xfb840108,0xbc3a0101,0xfb840108,0xbc3b0101,0xfb840108,0xbc3c0101,0xfb840108,0xbc3d0101,
	0xfb840108,0xbc3e0101,0xfb840108,0xbc3f0101,0xfb840108,0xbc400101,0xfb840108,0xbc410101,
	0xfb840108,0xbc420101,0xfb840108,0xbc430101,0xfb840108,0xbc440101,0xfb840108,0xbc450101,
	0xfb840108,0xbc460101,0xfb840108,0xbc470101,0xfb840108,0xbc480101,0xfb840108,0xbc490101,
	0xfb840108,0xbc4a0101,0xfb840108,0xbc4b0101,0xfb840108,0xbc4c0101,0xfb840108,0xbc4d0101,
	0xfb840108,0xbc4e0101,0xfb840108,0xbc4f0101,0xfb840108,0xbc500101,0xfb840108,0xbc510101,
	0xfb840108,0xbc520101,0xfb840108,0xbc530101,0xfb840108,0xbc540101,0xfb840108,0xbc550101,
	0xfb840108,0xbc560101,0xfb840108,0xbc570101,0xfb840108,0xbc580101,0xfb840108,0xbc590101,
	0xfb840108,0xbc5a0101,0xfb840108,0xbc5b0101,0xfb840108,0xbc5c0101,0xfb840108,0xbc5d0101,
	0xfb840108,0xbc5e0101,0xfb840108,0xbc5f0101,0xfb840108,0xbc600101,0xfb840108,0xbc610101,
	0xfb840108,0xbc620101,0xfb840108,0xbc630101,0xfb840108,0xbc640101,0xfb840108,0xbc650101,
	0xfb840108,0xbc660101,0xfb840108,0xbc670101,0xfb840108,0xbc680101,0xfb840108,0xbc690101,
	0xfb840108,0xbc6a0101,0xfb840108,0xbc6b0101,0xfb840108,0xbc6c0101,0xfb840108,0xbc6d0101,
	0xfb840108,0xbc6e0101,0xfb840108,0xbc6f0101,0xfb840108,0xbc700101,0xfb840108,0xbc710101,
	0xfb840108,0xbc720101,0xfb840108,0xbc730101,0xfb840108,0xbc740101,0xfb840108,0xbc750101,
	0xfb840108,0xbc760101,0xfb840108,0xbc770101,0xfb840108,0xbc780101,0xfb840108,0xbc790101,
	0xfb840108,0xbc7a0101,0xfb840108,0xbc7b0101,0xfb840108,0xbc7c0101,0xfb840108,0xbc7d0101,
	0xfb840108,0xbc7e0101,0xfb840108,0xbc7f0101,0xfb840108,0xbc800101,0xfb840108,0xbc810101,
	0xfb840108,0xbc820101,0xfb840108,0xbc830101,0xfb840108,0xbc840101,0xfb840108,0xbc850101,
	0xfb840108,0xbc860101,0xfb840108,0xbc870101,0xfb840108,0xbc880101,0xfb840108,0xbc890101,
	0xfb840108,0xbc8a0101,0xfb840108,0xbc8b0101,0xfb840108,0xbc8c0101,0xfb840108,0xbc8d0101,
	0xfb840108,0xbc8e0101,0xfb840108,0xbc8f0101,0xfb840108,0xbc900101,0xfb840108,0xbc910101,
	0xfb840108,0xbc920101,0xfb840108,0xbc930101,0xfb840108,0xbc940101,0xfb840108,0xbc950101,
	0xfb840108,0xbc960101,0xfb840108,0xbc970101,0xfb840108,0xbc980101,0xfb840108,0xbc990101,
	0xfb840108,0xbc9a0101,0xfb840108,0xbc9b0101,0xfb840108,0xbc9c0101,0xfb840108,0xbc9d0101,
	0xfb840108,0xbc9e0101,0xfb840108,0xbc9f0101,0xfb840108,0xbca00101,0xfb840108,0xbca10101,
	0xfb840108,0xbca20101,0xfb840108,0xbca30101,0xfb840108,0xbca40101,0xfb840108,0xbca50101,
	0xfb840108,0xbca60101,0xfb840108,0xbca70101,0xfb840108,0xbca80101,0xfb840108,0xbca90101,
	0xfb840108,0xbcaa0101,0xfb840108,0xbcab0101,0xfb840108,0xbcac0101,0xfb840108,0xbcad0101,
	0xfb840108,0xbcae0101,0xfb840108,0xbcaf0101,0xfb840108,0xbcb00101,0xfb840108,0xbcb10101,
	0xfb840108,0xbcb20101,0xfb840108,0xbcb30101,0xfb840108,0xbcb40101,0xfb840108,0xbcb50101,
	0xfb840108,0xbcb60101,0xfb840108,0xbcb70101,0xfb840108,0xbcb80101,0xfb840108,0xbcb90101,
	0xfb840108,0xbcba0101,0xfb840108,0xbcbb0101,0xfb840108,0xbcbc0101,0xfb840108,0xbcbd0101,
	0xfb840108,0xbcbe0101,0xfb840108,0xbcbf0101,0xfb840108,0xbcc00101,0xfb840108,0xbcc10101,
	0xfb840108,0xbcc20101,0xfb840108,0xbcc30101,0xfb840108,0xbcc40101,0xfb840108,0xbcc50101,
	0xfb840108,0xbcc60101,0xfb840108,0xbcc70101,0xfb840108,0xbcc80101,0xfb840108,0xbcc90101,
	0xfb840108,0xbcca0101,0xfb840108,0xbccb0101,0xfb840108,0xbccc0101,0xfb840108,0xbccd0101,
	0xfb840108,0xbcce0101,0xfb840108,0xbccf0101,0xfb840108,0xbcd00101,0xfb840108,0xbcd10101,
	0xfb840108,0xbcd20101,0xfb840108,0xbcd30101,0xfb840108,0xbcd40101,0xfb840108,0xbcd50101,
	0xfb840108,0xbcd60101,0xfb840108,0xbcd70101,0xfb840108,0xbcd80101,0xfb840108,0xbcd90101,
	0xfb840108,0xbcda0101,0xfb840108,0xbcdb0101,0xfb840108,0xbcdc0101,0xfb840108,0xbcdd0101,
	0xfb840108,0xbcde0101,0xfb840108,0xbcdf0101,0xfb840108,0xbce00101,0xfb840108,0xbce10101,
	0xfb840108,0xbce20101,0xfb840108,0xbce30101,0xfb840108,0xbce40101,0xfb840108,0xbce50101,
	0xfb840108,0xbce60101,0xfb840108,0xbce70101,0xfb840108,0xbce80101,0xfb840108,0xbce90101,
	0xfb840108,0xbcea0101,0xfb840108,0xbceb0101,0xfb840108,0xbcec0101,0xfb840108,0xbced0101,
	0xfb840108,0xbcee0101,0xfb840108,0xbcef0101,0xfb840108,0xbcf00101,0xfb840108,0xbcf10101,
	0xfb840108,0xbcf20101,0xfb840108,0xbcf30101,0xfb840108,0xbcf40101,0xfb840108,0xbcf50101,
	0xfb840108,0xbcf60101,0xfb840108,0xbcf70101,0xfb840108,0xbcf80101,0xfb840108,0xbcf90101,
	0xfb840108,0xbcfa0101,0xfb840108,0xbcfb0101,0xfb840108,0xbcfc0101,0xfb840108,0xbcfd0101,
	0xfb840108,0xbcfe0101,0xfb840108,0xbcff0101,0xfb840108,0xbd000101,0xfb840108,0xbd010101,
	0xfb840108,0xbd020101,0xfb840108,0xbd030101,0xfb840108,0xbd040101,0xfb840108,0xbd050101,
	0xfb840108,0xbd060101,0xfb840108,0xbd070101,0xfb840108,0xbd080101,0xfb840108,0xbd090101,
	0xfb840108,0xbd0a0101,0xfb840108,0xbd0b0101,0xfb840108,0xbd0c0101,0xfb840108,0xbd0d0101,
	0xfb840108,0xbd0e0101,0xfb840108,0xbd0f0101,0xfb840108,0xbd100101,0xfb840108,0xbd110101,
	0xfb840108,0xbd120101,0xfb840108,0xbd130101,0xfb840108,0xbd140101,0xfb840108,0xbd150101,
	0xfb840108,0xbd160101,0xfb840108,0xbd170101,0xfb840108,0xbd180101,0xfb840108,0xbd190101,
	0xfb840108,0xbd1a0101,0xfb840108,0xbd1b0101,0xfb840108,0xbd1c0101,0xfb840108,0xbd1d0101,
	0xfb840108,0xbd1e0101,0xfb840108,0xbd1f0101,0xfb840108,0xbd200101,0xfb840108,0xbd210101,
	0xfb840108,0xbd220101,0xfb840108,0xbd230101,0xfb840108,0xbd240101,0xfb840108,0xbd250101,
	0xfb840108,0xbd260101,0xfb840108,0xbd270101,0xfb840108,0xbd280101,0xfb840108,0xbd290101,
	0xfb840108,0xbd2a0101,0xfb840108,0xbd2b0101,0xfb840108,0xbd2c0101,0xfb840108,0xbd2d0101,
	0xfb840108,0xbd2e0101,0xfb840108,0xbd2f0101,0xfb840108,0xbd300101,0xfb840108,0xbd310101,
	0xfb840108,0xbd320101,0xfb840108,0xbd330101,0xfb840108,0xbd340101,0xfb840108,0xbd350101,
	0xfb840108,0xbd360101,0xfb840108,0xbd370101,0xfb840108,0xbd380101,0xfb840108,0xbd390101,
	0xfb840108,0xbd3a0101,0xfb840108,0xbd3b0101,0xfb840108,0xbd3c0101,0xfb840108,0xbd3d0101,
	0xfb840108,0xbd3e0101,0xfb840108,0xbd3f0101,0xfb840108,0xbd400101,0xfb840108,0xbd410101,
	0xfb840108,0xbd420101,0xfb840108,0xbd430101,0xfb840108,0xbd440101,0xfb840108,0xbd450101,
	0xfb840108,0xbd460101,0xfb840108,0xbd470101,0xfb840108,0xbd480101,0xfb840108,0xbd490101,
	0xfb840108,0xbd4a0101,0xfb840108,0xbd4b0101,0xfb840108,0xbd4c0101,0xfb840108,0xbd4d0101,
	0xfb840108,0xbd4e0101,0xfb840108,0xbd4f0101,0xfb840108,0xbd500101,0xfb840108,0xbd510101,
	0xfb840108,0xbd520101,0xfb840108,0xbd530101,0xfb840108,0xbd540101,0xfb840108,0xbd550101,
	0xfb840108,0xbd560101,0xfb840108,0xbd570101,0xfb840108,0xbd580101,0xfb840108,0xbd590101,
	0xfb840108,0xbd5a0101,0xfb840108,0xbd5b0101,0xfb840108,0xbd5c0101,0xfb840108,0xbd5d0101,
	0xfb840108,0xbd5e0101,0xfb840108,0xbd5f0101,0xfb840108,0xbd600101,0xfb840108,0xbd610101,
	0xfb840108,0xbd620101,0xfb840108,0xbd630101,0xfb840108,0xbd640101,0xfb840108,0xbd650101,
	0xfb840108,0xbd660101,0xfb840108,0xbd670101,0xfb840108,0xbd680101,0xfb840108,0xbd690101,
	0xfb840108,0xbd6a0101,0xfb840108,0xbd6b0101,0xfb840108,0xbd6c0101,0xfb840108,0xbd6d0101,
	0xfb840108,0xbd6e0101,0xfb840108,0xbd6f0101,0xfb840108,0xbd700101,0xfb840108,0xbd710101,
	0xfb840108,0xbd720101,0xfb840108,0xbd730101,0xfb840108,0xbd740101,0xfb840108,0xbd750101,
	0xfb840108,0xbd760101,0xfb840108,0xbd770101,0xfb840108,0xbd780101,0xfb840108,0xbd790101,
	0xfb840108,0xbd7a0101,0xfb840108,0xbd7b0101,0xfb840108,0xbd7c0101,0xfb840108,0xbd7d0101,
	0xfb840108,0xbd7e0101,0xfb840108,0xbd7f0101,0xfb840108,0xbd800101,0xfb840108,0xbd810101,
	0xfb840108,0xbd820101,0xfb840108,0xbd830101,0xfb840108,0xbd840101,0xfb840108,0xbd850101,
	0xfb840108,0xbd860101,0xfb840108,0xbd870101,0xfb840108,0xbd880101,0xfb840108,0xbd890101,
	0xfb840108,0xbd8a0101,0xfb840108,0xbd8b0101,0xfb840108,0xbd8c0101,0xfb840108,0xbd8d0101,
	0xfb840108,0xbd8e0101,0xfb840108,0xbd8f0101,0xfb840108,0xbd900101,0xfb840108,0xbd910101,
	0xfb840108,0xbd920101,0xfb840108,0xbd930101,0xfb840108,0xbd940101,0xfb840108,0xbd950101,
	0xfb840108,0xbd960101,0xfb840108,0xbd970101,0xfb840108,0xbd980101,0xfb840108,0xbd990101,
	0xfb840108,0xbd9a0101,0xfb840108,0xbd9b0101,0xfb840108,0xbd9c0101,0xfb840108,0xbd9d0101,
	0xfb840108,0xbd9e0101,0xfb840108,0xbd9f0101,0xfb840108,0xbda00101,0xfb840108,0xbda10101,
	0xfb840108,0xbda20101,0xfb840108,0xbda30101,0xfb840108,0xbda40101,0xfb840108,0xbda50101,
	0xfb840108,0xbda60101,0xfb840108,0xbda70101,0xfb840108,0xbda80101,0xfb840108,0xbda90101,
	0xfb840108,0xbdaa0101,0xfb840108,0xbdab0101,0xfb840108,0xbdac0101,0xfb840108,0xbdad0101,
	0xfb840108,0xbdae0101,0xfb840108,0xbdaf0101,0xfb840108,0xbdb00101,0xfb840108,0xbdb10101,
	0xfb840108,0xbdb20101,0xfb840108,0xbdb30101,0xfb840108,0xbdb40101,0xfb840108,0xbdb50101,
	0xfb840108,0xbdb60101,0xfb840108,0xbdb70101,0xfb840108,0xbdb80101,0xfb840108,0xbdb90101,
	0xfb840108,0xbdba0101,0xfb840108,0xbdbb0101,0xfb840108,0xbdbc0101,0xfb840108,0xbdbd0101,
	0xfb840108,0xbdbe0101,0xfb840108,0xbdbf0101,0xfb840108,0xbdc00101,0xfb840108,0xbdc10101,
	0xfb840108,0xbdc20101,0xfb840108,0xbdc30101,0xfb840108,0xbdc40101,0xfb840108,0xbdc50101,
	0xfb840108,0xbdc60101,0xfb840108,0xbdc70101,0xfb840108,0xbdc80101,0xfb840108,0xbdc90101,
	0xfb840108,0xbdca0101,0xfb840108,0xbdcb0101,0xfb840108,0xbdcc0101,0xfb840108,0xbdcd0101,
	0xfb840108,0xbdce0101,0xfb840108,0xbdcf0101,0xfb840108,0xbdd00101,0xfb840108,0xbdd10101,
	0xfb840108,0xbdd20101,0xfb840108,0xbdd30101,0xfb840108,0xbdd40101,0xfb840108,0xbdd50101,
	0xfb840108,0xbdd60101,0xfb840108,0xbdd70101,0xfb840108,0xbdd80101,0xfb840108,0xbdd90101,
	0xfb840108,0xbdda0101,0xfb840108,0xbddb0101,0xfb840108,0xbddc0101,0xfb840108,0xbddd0101,
	0xfb840108,0xbdde0101,0xfb840108,0xbddf0101,0xfb840108,0xbde00101,0xfb840108,0xbde10101,
	0xfb840108,0xbde20101,0xfb840108,0xbde30101,0xfb840108,0xbde40101,0xfb840108,0xbde50101,
	0xfb840108,0xbde60101,0xfb840108,0xbde70101,0xfb840108,0xbde80101,0xfb840108,0xbde90101,
	0xfb840108,0xbdea0101,0xfb840108,0xbdeb0101,0xfb840108,0xbdec0101,0xfb840108,0xbded0101,
	0xfb840108,0xbdee0101,0xfb840108,0xbdef0101,0xfb840108,0xbdf00101,0xfb840108,0xbdf10101,
	0xfb840108,0xbdf20101,0xfb840108,0xbdf30101,0xfb840108,0xbdf40101,0xfb840108,0xbdf50101,
	0xfb840108,0xbdf60101,0xfb840108,0xbdf70101,0xfb840108,0xbdf80101,0xfb840108,0xbdf90101,
	0xfb840108,0xbdfa0101,0xfb840108,0xbdfb0101,0xfb840108,0xbdfc0101,0xfb840108,0xbdfd0101,
	0xfb840108,0xbdfe0101,0xfb840108,0xbdff0101,0xfb840108,0xbe000101,0xfb840108,0xbe010101,
	0xfb840108,0xbe020101,0xfb840108,0xbe030101,0xfb840108,0xbe040101,0xfb840108,0xbe050101,
	0xfb840108,0xbe060101,0xfb840108,0xbe070101,0xfb840108,0xbe080101,0xfb840108,0xbe090101,
	0xfb840108,0xbe0a0101,0xfb840108,0xbe0b0101,0xfb840108,0xbe0c0101,0xfb840108,0xbe0d0101,
	0xfb840108,0xbe0e0101,0xfb840108,0xbe0f0101,0xfb840108,0xbe100101,0xfb840108,0xbe110101,
	0xfb840108,0xbe120101,0xfb840108,0xbe130101,0xfb840108,0xbe140101,0xfb840108,0xbe150101,
	0xfb840108,0xbe160101,0xfb840108,0xbe170101,0xfb840108,0xbe180101,0xfb840108,0xbe190101,
	0xfb840108,0xbe1a0101,0xfb840108,0xbe1b0101,0xfb840108,0xbe1c0101,0xfb840108,0xbe1d0101,
	0xfb840108,0xbe1e0101,0xfb840108,0xbe1f0101,0xfb840108,0xbe200101,0xfb840108,0xbe210101,
	0xfb840108,0xbe220101,0xfb840108,0xbe230101,0xfb840108,0xbe240101,0xfb840108,0xbe250101,
	0xfb840108,0xbe260101,0xfb840108,0xbe270101,0xfb840108,0xbe280101,0xfb840108,0xbe290101,
	0xfb840108,0xbe2a0101,0xfb840108,0xbe2b0101,0xfb840108,0xbe2c0101,0xfb840108,0xbe2d0101,
	0xfb840108,0xbe2e0101,0xfb840108,0xbe2f0101,0xfb840108,0xbe300101,0xfb840108,0xbe310101,
	0xfb840108,0xbe320101,0xfb840108,0xbe330101,0xfb840108,0xbe340101,0xfb840108,0xbe350101,
	0xfb840108,0xbe360101,0xfb840108,0xbe370101,0xfb840108,0xbe380101,0xfb840108,0xbe390101,
	0xfb840108,0xbe3a0101,0xfb840108,0xbe3b0101,0xfb840108,0xbe3c0101,0xfb840108,0xbe3d0101,
	0xfb840108,0xbe3e0101,0xfb840108,0xbe3f0101,0xfb840108,0xbe400101,0xfb840108,0xbe410101,
	0xfb840108,0xbe420101,0xfb840108,0xbe430101,0xfb840108,0xbe440101,0xfb840108,0xbe450101,
	0xfb840108,0xbe460101,0xfb840108,0xbe470101,0xfb840108,0xbe480101,0xfb840108,0xbe490101,
	0xfb840108,0xbe4a0101,0xfb840108,0xbe4b0101,0xfb840108,0xbe4c0101,0xfb840108,0xbe4d0101,
	0xfb840108,0xbe4e0101,0xfb840108,0xbe4f0101,0xfb840108,0xbe500101,0xfb840108,0xbe510101,
	0xfb840108,0xbe520101,0xfb840108,0xbe530101,0xfb840108,0xbe540101,0xfb840108,0xbe550101,
	0xfb840108,0xbe560101,0xfb840108,0xbe570101,0xfb840108,0xbe580101,0xfb840108,0xbe590101,
	0xfb840108,0xbe5a0101,0xfb840108,0xbe5b0101,0xfb840108,0xbe5c0101,0xfb840108,0xbe5d0101,
	0xfb840108,0xbe5e0101,0xfb840108,0xbe5f0101,0xfb840108,0xbe600101,0xfb840108,0xbe610101,
	0xfb840108,0xbe620101,0xfb840108,0xbe630101,0xfb840108,0xbe640101,0xfb840108,0xbe650101,
	0xfb840108,0xbe660101,0xfb840108,0xbe670101,0xfb840108,0xbe680101,0xfb840108,0xbe690101,
	0xfb840108,0xbe6a0101,0xfb840108,0xbe6b0101,0xfb840108,0xbe6c0101,0xfb840108,0xbe6d0101,
	0xfb840108,0xbe6e0101,0xfb840108,0xbe6f0101,0xfb840108,0xbe700101,0xfb840108,0xbe710101,
	0xfb840108,0xbe720101,0xfb840108,0xbe730101,0xfb840108,0xbe740101,0xfb840108,0xbe750101,
	0xfb840108,0xbe760101,0xfb840108,0xbe770101,0xfb840108,0xbe780101,0xfb840108,0xbe790101,
	0xfb840108,0xbe7a0101,0xfb840108,0xbe7b0101,0xfb840108,0xbe7c0101,0xfb840108,0xbe7d0101,
	0xfb840108,0xbe7e0101,0xfb840108,0xbe7f0101,0xfb840108,0xbe800101,0xfb840108,0xbe810101,
	0xfb840108,0xbe820101,0xfb840108,0xbe830101,0xfb840108,0xbe840101,0xfb840108,0xbe850101,
	0xfb840108,0xbe860101,0xfb840108,0xbe870101,0xfb840108,0xbe880101,0xfb840108,0xbe890101,
	0xfb840108,0xbe8a0101,0xfb840108,0xbe8b0101,0xfb840108,0xbe8c0101,0xfb840108,0xbe8d0101,
	0xfb840108,0xbe8e0101,0xfb840108,0xbe8f0101,0xfb840108,0xbe900101,0xfb840108,0xbe910101,
	0xfb840108,0xbe920101,0xfb840108,0xbe930101,0xfb840108,0xbe940101,0xfb840108,0xbe950101,
	0xfb840108,0xbe960101,0xfb840108,0xbe970101,0xfb840108,0xbe980101,0xfb840108,0xbe990101,
	0xfb840108,0xbe9a0101,0xfb840108,0xbe9b0101,0xfb840108,0xbe9c0101,0xfb840108,0xbe9d0101,
	0xfb840108,0xbe9e0101,0xfb840108,0xbe9f0101,0xfb840108,0xbea00101,0xfb840108,0xbea10101,
	0xfb840108,0xbea20101,0xfb840108,0xbea30101,0xfb840108,0xbea40101,0xfb840108,0xbea50101,
	0xfb840108,0xbea60101,0xfb840108,0xbea70101,0xfb840108,0xbea80101,0xfb840108,0xbea90101,
	0xfb840108,0xbeaa0101,0xfb840108,0xbeab0101,0xfb840108,0xbeac0101,0xfb840108,0xbead0101,
	0xfb840108,0xbeae0101,0xfb840108,0xbeaf0101,0xfb840108,0xbeb00101,0xfb840108,0xbeb10101,
	0xfb840108,0xbeb20101,0xfb840108,0xbeb30101,0xfb840108,0xbeb40101,0xfb840108,0xbeb50101,
	0xfb840108,0xbeb60101,0xfb840108,0xbeb70101,0xfb840108,0xbeb80101,0xfb840108,0xbeb90101,
	0xfb840108,0xbeba0101,0xfb840108,0xbebb0101,0xfb840108,0xbebc0101,0xfb840108,0xbebd0101,
	0xfb840108,0xbebe0101,0xfb840108,0xbebf0101,0xfb840108,0xbec00101,0xfb840108,0xbec10101,
	0xfb840108,0xbec20101,0xfb840108,0xbec30101,0xfb840108,0xbec40101,0xfb840108,0xbec50101,
	0xfb840108,0xbec60101,0xfb840108,0xbec70101,0xfb840108,0xbec80101,0xfb840108,0xbec90101,
	0xfb840108,0xbeca0101,0xfb840108,0xbecb0101,0xfb840108,0xbecc0101,0xfb840108,0xbecd0101,
	0xfb840108,0xbece0101,0xfb840108,0xbecf0101,0xfb840108,0xbed00101,0xfb840108,0xbed10101,
	0xfb840108,0xbed20101,0xfb840108,0xbed30101,0xfb840108,0xbed40101,0xfb840108,0xbed50101,
	0xfb840108,0xbed60101,0xfb840108,0xbed70101,0xfb840108,0xbed80101,0xfb840108,0xbed90101,
	0xfb840108,0xbeda0101,0xfb840108,0xbedb0101,0xfb840108,0xbedc0101,0xfb840108,0xbedd0101,
	0xfb840108,0xbede0101,0xfb840108,0xbedf0101,0xfb840108,0xbee00101,0xfb840108,0xbee10101,
	0xfb840108,0xbee20101,0xfb840108,0xbee30101,0xfb840108,0xbee40101,0xfb840108,0xbee50101,
	0xfb840108,0xbee60101,0xfb840108,0xbee70101,0xfb840108,0xbee80101,0xfb840108,0xbee90101,
	0xfb840108,0xbeea0101,0xfb840108,0xbeeb0101,0xfb840108,0xbeec0101,0xfb840108,0xbeed0101,
	0xfb840108,0xbeee0101,0xfb840108,0xbeef0101,0xfb840108,0xbef00101,0xfb840108,0xbef10101,
	0xfb840108,0xbef20101,0xfb840108,0xbef30101,0xfb840108,0xbef40101,0xfb840108,0xbef50101,
	0xfb840108,0xbef60101,0xfb840108,0xbef70101,0xfb840108,0xbef80101,0xfb840108,0xbef90101,
	0xfb840108,0xbefa0101,0xfb840108,0xbefb0101,0xfb840108,0xbefc0101,0xfb840108,0xbefd0101,
	0xfb840108,0xbefe0101,0xfb840108,0xbeff0101,0xfb840108,0xbf000101,0xfb840108,0xbf010101,
	0xfb840108,0xbf020101,0xfb840108,0xbf030101,0xfb840108,0xbf040101,0xfb840108,0xbf050101,
	0xfb840108,0xbf060101,0xfb840108,0xbf070101,0xfb840108,0xbf080101,0xfb840108,0xbf090101,
	0xfb840108,0xbf0a0101,0xfb840108,0xbf0b0101,0xfb840108,0xbf0c0101,0xfb840108,0xbf0d0101,
	0xfb840108,0xbf0e0101,0xfb840108,0xbf0f0101,0xfb840108,0xbf100101,0xfb840108,0xbf110101,
	0xfb840108,0xbf120101,0xfb840108,0xbf130101,0xfb840108,0xbf140101,0xfb840108,0xbf150101,
	0xfb840108,0xbf160101,0xfb840108,0xbf170101,0xfb840108,0xbf180101,0xfb840108,0xbf190101,
	0xfb840108,0xbf1a0101,0xfb840108,0xbf1b0101,0xfb840108,0xbf1c0101,0xfb840108,0xbf1d0101,
	0xfb840108,0xbf1e0101,0xfb840108,0xbf1f0101,0xfb840108,0xbf200101,0xfb840108,0xbf210101,
	0xfb840108,0xbf220101,0xfb840108,0xbf230101,0xfb840108,0xbf240101,0xfb840108,0xbf250101,
	0xfb840108,0xbf260101,0xfb840108,0xbf270101,0xfb840108,0xbf280101,0xfb840108,0xbf290101,
	0xfb840108,0xbf2a0101,0xfb840108,0xbf2b0101,0xfb840108,0xbf2c0101,0xfb840108,0xbf2d0101,
	0xfb840108,0xbf2e0101,0xfb840108,0xbf2f0101,0xfb840108,0xbf300101,0xfb840108,0xbf310101,
	0xfb840108,0xbf320101,0xfb840108,0xbf330101,0xfb840108,0xbf340101,0xfb840108,0xbf350101,
	0xfb840108,0xbf360101,0xfb840108,0xbf370101,0xfb840108,0xbf380101,0xfb840108,0xbf390101,
	0xfb840108,0xbf3a0101,0xfb840108,0xbf3b0101,0xfb840108,0xbf3c0101,0xfb840108,0xbf3d0101,
	0xfb840108,0xbf3e0101,0xfb840108,0xbf3f0101,0xfb840108,0xbf400101,0xfb840108,0xbf410101,
	0xfb840108,0xbf420101,0xfb840108,0xbf430101,0xfb840108,0xbf440101,0xfb840108,0xbf450101,
	0xfb840108,0xbf460101,0xfb840108,0xbf470101,0xfb840108,0xbf480101,0xfb840108,0xbf490101,
	0xfb840108,0xbf4a0101,0xfb840108,0xbf4b0101,0xfb840108,0xbf4c0101,0xfb840108,0xbf4d0101,
	0xfb840108,0xbf4e0101,0xfb840108,0xbf4f0101,0xfb840108,0xbf500101,0xfb840108,0xbf510101,
	0xfb840108,0xbf520101,0xfb840108,0xbf530101,0xfb840108,0xbf540101,0xfb840108,0xbf550101,
	0xfb840108,0xbf560101,0xfb840108,0xbf570101,0xfb840108,0xbf580101,0xfb840108,0xbf590101,
	0xfb840108,0xbf5a0101,0xfb840108,0xbf5b0101,0xfb840108,0xbf5c0101,0xfb840108,0xbf5d0101,
	0xfb840108,0xbf5e0101,0xfb840108,0xbf5f0101,0xfb840108,0xbf600101,0xfb840108,0xbf610101,
	0xfb840108,0xbf620101,0xfb840108,0xbf630101,0xfb840108,0xbf640101,0xfb840108,0xbf650101,
	0xfb840108,0xbf660101,0xfb840108,0xbf670101,0xfb840108,0xbf680101,0xfb840108,0xbf690101,
	0xfb840108,0xbf6a0101,0xfb840108,0xbf6b0101,0xfb840108,0xbf6c0101,0xfb840108,0xbf6d0101,
	0xfb840108,0xbf6e0101,0xfb840108,0xbf6f0101,0xfb840108,0xbf700101,0xfb840108,0xbf710101,
	0xfb840108,0xbf720101,0xfb840108,0xbf730101,0xfb840108,0xbf740101,0xfb840108,0xbf750101,
	0xfb840108,0xbf760101,0xfb840108,0xbf770101,0xfb840108,0xbf780101,0xfb840108,0xbf790101,
	0xfb840108,0xbf7a0101,0xfb840108,0xbf7b0101,0xfb840108,0xbf7c0101,0xfb840108,0xbf7d0101,
	0xfb840108,0xbf7e0101,0xfb840108,0xbf7f0101,0xfb840108,0xbf800101,0xfb840108,0xbf810101,
	0xfb840108,0xbf820101,0xfb840108,0xbf830101,0xfb840108,0xbf840101,0xfb840108,0xbf850101,
	0xfb840108,0xbf860101,0xfb840108,0xbf870101,0xfb840108,0xbf880101,0xfb840108,0xbf890101,
	0xfb840108,0xbf8a0101,0xfb840108,0xbf8b0101,0xfb840108,0xbf8c0101,0xfb840108,0xbf8d0101,
	0xfb840108,0xbf8e0101,0xfb840108,0xbf8f0101,0xfb840108,0xbf900101,0xfb840108,0xbf910101,
	0xfb840108,0xbf920101,0xfb840108,0xbf930101,0xfb840108,0xbf940101,0xfb840108,0xbf950101,
	0xfb840108,0xbf960101,0xfb840108,0xbf970101,0xfb840108,0xbf980101,0xfb840108,0xbf990101,
	0xfb840108,0xbf9a0101,0xfb840108,0xbf9b0101,0xfb840108,0xbf9c0101,0xfb840108,0xbf9d0101,
	0xfb840108,0xbf9e0101,0xfb840108,0xbf9f0101,0xfb840108,0xbfa00101,0xfb840108,0xbfa10101,
	0xfb840108,0xbfa20101,0xfb840108,0xbfa30101,0xfb840108,0xbfa40101,0xfb840108,0xbfa50101,
	0xfb840108,0xbfa60101,0xfb840108,0xbfa70101,0xfb840108,0xbfa80101,0xfb840108,0xbfa90101,
	0xfb840108,0xbfaa0101,0xfb840108,0xbfab0101,0xfb840108,0xbfac0101,0xfb840108,0xbfad0101,
	0xfb840108,0xbfae0101,0xfb840108,0xbfaf0101,0xfb840108,0xbfb00101,0xfb840108,0xbfb10101,
	0xfb840108,0xbfb20101,0xfb840108,0xbfb30101,0xfb840108,0xbfb40101,0xfb840108,0xbfb50101,
	0xfb840108,0xbfb60101,0xfb840108,0xbfb70101,0xfb840108,0xbfb80101,0xfb840108,0xbfb90101,
	0xfb840108,0xbfba0101,0xfb840108,0xbfbb0101,0xfb840108,0xbfbc0101,0xfb840108,0xbfbd0101,
	0xfb840108,0xbfbe0101,0xfb840108,0xbfbf0101,0xfb840108,0xbfc00101,0xfb840108,0xbfc10101,
	0xfb840108,0xbfc20101,0xfb840108,0xbfc30101,0xfb840108,0xbfc40101,0xfb840108,0xbfc50101,
	0xfb840108,0xbfc60101,0xfb840108,0xbfc70101,0xfb840108,0xbfc80101,0xfb840108,0xbfc90101,
	0xfb840108,0xbfca0101,0xfb840108,0xbfcb0101,0xfb840108,0xbfcc0101,0xfb840108,0xbfcd0101,
	0xfb840108,0xbfce0101,0xfb840108,0xbfcf0101,0xfb840108,0xbfd00101,0xfb840108,0xbfd10101,
	0xfb840108,0xbfd20101,0xfb840108,0xbfd30101,0xfb840108,0xbfd40101,0xfb840108,0xbfd50101,
	0xfb840108,0xbfd60101,0xfb840108,0xbfd70101,0xfb840108,0xbfd80101,0xfb840108,0xbfd90101,
	0xfb840108,0xbfda0101,0xfb840108,0xbfdb0101,0xfb840108,0xbfdc0101,0xfb840108,0xbfdd0101,
	0xfb840108,0xbfde0101,0xfb840108,0xbfdf0101,0xfb840108,0xbfe00101,0xfb840108,0xbfe10101,
	0xfb840108,0xbfe20101,0xfb840108,0xbfe30101,0xfb840108,0xbfe40101,0xfb840108,0xbfe50101,
	0xfb840108,0xbfe60101,0xfb840108,0xbfe70101,0xfb840108,0xbfe80101,0xfb840108,0xbfe90101,
	0xfb840108,0xbfea0101,0xfb840108,0xbfeb0101,0xfb840108,0xbfec0101,0xfb840108,0xbfed0101,
	0xfb840108,0xbfee0101,0xfb840108,0xbfef0101,0xfb840108,0xbff00101,0xfb840108,0xbff10101,
	0xfb840108,0xbff20101,0xfb840108,0xbff30101,0xfb840108,0xbff40101,0xfb840108,0xbff50101,
	0xfb840108,0xbff60101,0xfb840108,0xbff70101,0xfb840108,0xbff80101,0xfb840108,0xbff90101,
	0xfb840108,0xbffa0101,0xfb840108,0xbffb0101,0xfb840108,0xbffc0101,0xfb840108,0xbffd0101,
	0xfb840108,0xbffe0101,0xfb840108,0xbfff0101,0xfb840108,0xc0000101,0xfb840108,0xc0010101,
	0xfb840108,0xc0020101,0xfb840108,0xc0030101,0xfb840108,0xc0040101,0xfb840108,0xc0050101,
	0xfb840108,0xc0060101,0xfb840108,0xc0070101,0xfb840108,0xc0080101,0xfb840108,0xc0090101,
	0xfb840108,0xc00a0101,0xfb840108,0xc00b0101,0xfb840108,0xc00c0101,0xfb840108,0xc00d0101,
	0xfb840108,0xc00e0101,0xfb840108,0xc00f0101,0xfb840108,0xc0100101,0xfb840108,0xc0110101,
	0xfb840108,0xc0120101,0xfb840108,0xc0130101,0xfb840108,0xc0140101,0xfb840108,0xc0150101,
	0xfb840108,0xc0160101,0xfb840108,0xc0170101,0xfb840108,0xc0180101,0xfb840108,0xc0190101,
	0xfb840108,0xc01a0101,0xfb840108,0xc01b0101,0xfb840108,0xc01c0101,0xfb840108,0xc01d0101,
	0xfb840108,0xc01e0101,0xfb840108,0xc01f0101,0xfb840108,0xc0200101,0xfb840108,0xc0210101,
	0xfb840108,0xc0220101,0xfb840108,0xc0230101,0xfb840108,0xc0240101,0xfb840108,0xc0250101,
	0xfb840108,0xc0260101,0xfb840108,0xc0270101,0xfb840108,0xc0280101,0xfb840108,0xc0290101,
	0xfb840108,0xc02a0101,0xfb840108,0xc02b0101,0xfb840108,0xc02c0101,0xfb840108,0xc02d0101,
	0xfb840108,0xc02e0101,0xfb840108,0xc02f0101,0xfb840108,0xc0300101,0xfb840108,0xc0310101,
	0xfb840108,0xc0320101,0xfb840108,0xc0330101,0xfb840108,0xc0340101,0xfb840108,0xc0350101,
	0xfb840108,0xc0360101,0xfb840108,0xc0370101,0xfb840108,0xc0380101,0xfb840108,0xc0390101,
	0xfb840108,0xc03a0101,0xfb840108,0xc03b0101,0xfb840108,0xc03c0101,0xfb840108,0xc03d0101,
	0xfb840108,0xc03e0101,0xfb840108,0xc03f0101,0xfb840108,0xc0400101,0xfb840108,0xc0410101,
	0xfb840108,0xc0420101,0xfb840108,0xc0430101,0xfb840108,0xc0440101,0xfb840108,0xc0450101,
	0xfb840108,0xc0460101,0xfb840108,0xc0470101,0xfb840108,0xc0480101,0xfb840108,0xc0490101,
	0xfb840108,0xc04a0101,0xfb840108,0xc04b0101,0xfb840108,0xc04c0101,0xfb840108,0xc04d0101,
	0xfb840108,0xc04e0101,0xfb840108,0xc04f0101,0xfb840108,0xc0500101,0xfb840108,0xc0510101,
	0xfb840108,0xc0520101,0xfb840108,0xc0530101,0xfb840108,0xc0540101,0xfb840108,0xc0550101,
	0xfb840108,0xc0560101,0xfb840108,0xc0570101,0xfb840108,0xc0580101,0xfb840108,0xc0590101,
	0xfb840108,0xc05a0101,0xfb840108,0xc05b0101,0xfb840108,0xc05c0101,0xfb840108,0xc05d0101,
	0xfb840108,0xc05e0101,0xfb840108,0xc05f0101,0xfb840108,0xc0600101,0xfb840108,0xc0610101,
	0xfb840108,0xc0620101,0xfb840108,0xc0630101,0xfb840108,0xc0640101,0xfb840108,0xc0650101,
	0xfb840108,0xc0660101,0xfb840108,0xc0670101,0xfb840108,0xc0680101,0xfb840108,0xc0690101,
	0xfb840108,0xc06a0101,0xfb840108,0xc06b0101,0xfb840108,0xc06c0101,0xfb840108,0xc06d0101,
	0xfb840108,0xc06e0101,0xfb840108,0xc06f0101,0xfb840108,0xc0700101,0xfb840108,0xc0710101,
	0xfb840108,0xc0720101,0xfb840108,0xc0730101,0xfb840108,0xc0740101,0xfb840108,0xc0750101,
	0xfb840108,0xc0760101,0xfb840108,0xc0770101,0xfb840108,0xc0780101,0xfb840108,0xc0790101,
	0xfb840108,0xc07a0101,0xfb840108,0xc07b0101,0xfb840108,0xc07c0101,0xfb840108,0xc07d0101,
	0xfb840108,0xc07e0101,0xfb840108,0xc07f0101,0xfb840108,0xc0800101,0xfb840108,0xc0810101,
	0xfb840108,0xc0820101,0xfb840108,0xc0830101,0xfb840108,0xc0840101,0xfb840108,0xc0850101,
	0xfb840108,0xc0860101,0xfb840108,0xc0870101,0xfb840108,0xc0880101,0xfb840108,0xc0890101,
	0xfb840108,0xc08a0101,0xfb840108,0xc08b0101,0xfb840108,0xc08c0101,0xfb840108,0xc08d0101,
	0xfb840108,0xc08e0101,0xfb840108,0xc08f0101,0xfb840108,0xc0900101,0xfb840108,0xc0910101,
	0xfb840108,0xc0920101,0xfb840108,0xc0930101,0xfb840108,0xc0940101,0xfb840108,0xc0950101,
	0xfb840108,0xc0960101,0xfb840108,0xc0970101,0xfb840108,0xc0980101,0xfb840108,0xc0990101,
	0xfb840108,0xc09a0101,0xfb840108,0xc09b0101,0xfb840108,0xc09c0101,0xfb840108,0xc09d0101,
	0xfb840108,0xc09e0101,0xfb840108,0xc09f0101,0xfb840108,0xc0a00101,0xfb840108,0xc0a10101,
	0xfb840108,0xc0a20101,0xfb840108,0xc0a30101,0xfb840108,0xc0a40101,0xfb840108,0xc0a50101,
	0xfb840108,0xc0a60101,0xfb840108,0xc0a70101,0xfb840108,0xc0a80101,0xfb840108,0xc0a90101,
	0xfb840108,0xc0aa0101,0xfb840108,0xc0ab0101,0xfb840108,0xc0ac0101,0xfb840108,0xc0ad0101,
	0xfb840108,0xc0ae0101,0xfb840108,0xc0af0101,0xfb840108,0xc0b00101,0xfb840108,0xc0b10101,
	0xfb840108,0xc0b20101,0xfb840108,0xc0b30101,0xfb840108,0xc0b40101,0xfb840108,0xc0b50101,
	0xfb840108,0xc0b60101,0xfb840108,0xc0b70101,0xfb840108,0xc0b80101,0xfb840108,0xc0b90101,
	0xfb840108,0xc0ba0101,0xfb840108,0xc0bb0101,0xfb840108,0xc0bc0101,0xfb840108,0xc0bd0101,
	0xfb840108,0xc0be0101,0xfb840108,0xc0bf0101,0xfb840108,0xc0c00101,0xfb840108,0xc0c10101,
	0xfb840108,0xc0c20101,0xfb840108,0xc0c30101,0xfb840108,0xc0c40101,0xfb840108,0xc0c50101,
	0xfb840108,0xc0c60101,0xfb840108,0xc0c70101,0xfb840108,0xc0c80101,0xfb840108,0xc0c90101,
	0xfb840108,0xc0ca0101,0xfb840108,0xc0cb0101,0xfb840108,0xc0cc0101,0xfb840108,0xc0cd0101,
	0xfb840108,0xc0ce0101,0xfb840108,0xc0cf0101,0xfb840108,0xc0d00101,0xfb840108,0xc0d10101,
	0xfb840108,0xc0d20101,0xfb840108,0xc0d30101,0xfb840108,0xc0d40101,0xfb840108,0xc0d50101,
	0xfb840108,0xc0d60101,0xfb840108,0xc0d70101,0xfb840108,0xc0d80101,0xfb840108,0xc0d90101,
	0xfb840108,0xc0da0101,0xfb840108,0xc0db0101,0xfb840108,0xc0dc0101,0xfb840108,0xc0dd0101,
	0xfb840108,0xc0de0101,0xfb840108,0xc0df0101,0xfb840108,0xc0e00101,0xfb840108,0xc0e10101,
	0xfb840108,0xc0e20101,0xfb840108,0xc0e30101,0xfb840108,0xc0e40101,0xfb840108,0xc0e50101,
	0xfb840108,0xc0e60101,0xfb840108,0xc0e70101,0xfb840108,0xc0e80101,0xfb840108,0xc0e90101,
	0xfb840108,0xc0ea0101,0xfb840108,0xc0eb0101,0xfb840108,0xc0ec0101,0xfb840108,0xc0ed0101,
	0xfb840108,0xc0ee0101,0xfb840108,0xc0ef0101,0xfb840108,0xc0f00101,0xfb840108,0xc0f10101,
	0xfb840108,0xc0f20101,0xfb840108,0xc0f30101,0xfb840108,0xc0f40101,0xfb840108,0xc0f50101,
	0xfb840108,0xc0f60101,0xfb840108,0xc0f70101,0xfb840108,0xc0f80101,0xfb840108,0xc0f90101,
	0xfb840108,0xc0fa0101,0xfb840108,0xc0fb0101,0xfb840108,0xc0fc0101,0xfb840108,0xc0fd0101,
	0xfb840108,0xc0fe0101,0xfb840108,0xc0ff0101,0xfb840108,0xc1000101,0xfb840108,0xc1010101,
	0xfb840108,0xc1020101,0xfb840108,0xc1030101,0xfb840108,0xc1040101,0xfb840108,0xc1050101,
	0xfb840108,0xc1060101,0xfb840108,0xc1070101,0xfb840108,0xc1080101,0xfb840108,0xc1090101,
	0xfb840108,0xc10a0101,0xfb840108,0xc10b0101,0xfb840108,0xc10c0101,0xfb840108,0xc10d0101,
	0xfb840108,0xc10e0101,0xfb840108,0xc10f0101,0xfb840108,0xc1100101,0xfb840108,0xc1110101,
	0xfb840108,0xc1120101,0xfb840108,0xc1130101,0xfb840108,0xc1140101,0xfb840108,0xc1150101,
	0xfb840108,0xc1160101,0xfb840108,0xc1170101,0xfb840108,0xc1180101,0xfb840108,0xc1190101,
	0xfb840108,0xc11a0101,0xfb840108,0xc11b0101,0xfb840108,0xc11c0101,0xfb840108,0xc11d0101,
	0xfb840108,0xc11e0101,0xfb840108,0xc11f0101,0xfb840108,0xc1200101,0xfb840108,0xc1210101,
	0xfb840108,0xc1220101,0xfb840108,0xc1230101,0xfb840108,0xc1240101,0xfb840108,0xc1250101,
	0xfb840108,0xc1260101,0xfb840108,0xc1270101,0xfb840108,0xc1280101,0xfb840108,0xc1290101,
	0xfb840108,0xc12a0101,0xfb840108,0xc12b0101,0xfb840108,0xc12c0101,0xfb840108,0xc12d0101,
	0xfb840108,0xc12e0101,0xfb840108,0xc12f0101,0xfb840108,0xc1300101,0xfb840108,0xc1310101,
	0xfb840108,0xc1320101,0xfb840108,0xc1330101,0xfb840108,0xc1340101,0xfb840108,0xc1350101,
	0xfb840108,0xc1360101,0xfb840108,0xc1370101,0xfb840108,0xc1380101,0xfb840108,0xc1390101,
	0xfb840108,0xc13a0101,0xfb840108,0xc13b0101,0xfb840108,0xc13c0101,0xfb840108,0xc13d0101,
	0xfb840108,0xc13e0101,0xfb840108,0xc13f0101,0xfb840108,0xc1400101,0xfb840108,0xc1410101,
	0xfb840108,0xc1420101,0xfb840108,0xc1430101,0xfb840108,0xc1440101,0xfb840108,0xc1450101,
	0xfb840108,0xc1460101,0xfb840108,0xc1470101,0xfb840108,0xc1480101,0xfb840108,0xc1490101,
	0xfb840108,0xc14a0101,0xfb840108,0xc14b0101,0xfb840108,0xc14c0101,0xfb840108,0xc14d0101,
	0xfb840108,0xc14e0101,0xfb840108,0xc14f0101,0xfb840108,0xc1500101,0xfb840108,0xc1510101,
	0xfb840108,0xc1520101,0xfb840108,0xc1530101,0xfb840108,0xc1540101,0xfb840108,0xc1550101,
	0xfb840108,0xc1560101,0xfb840108,0xc1570101,0xfb840108,0xc1580101,0xfb840108,0xc1590101,
	0xfb840108,0xc15a0101,0xfb840108,0xc15b0101,0xfb840108,0xc15c0101,0xfb840108,0xc15d0101,
	0xfb840108,0xc15e0101,0xfb840108,0xc15f0101,0xfb840108,0xc1600101,0xfb840108,0xc1610101,
	0xfb840108,0xc1620101,0xfb840108,0xc1630101,0xfb840108,0xc1640101,0xfb840108,0xc1650101,
	0xfb840108,0xc1660101,0xfb840108,0xc1670101,0xfb840108,0xc1680101,0xfb840108,0xc1690101,
	0xfb840108,0xc16a0101,0xfb840108,0xc16b0101,0xfb840108,0xc16c0101,0xfb840108,0xc16d0101,
	0xfb840108,0xc16e0101,0xfb840108,0xc16f0101,0xfb840108,0xc1700101,0xfb840108,0xc1710101,
	0xfb840108,0xc1720101,0xfb840108,0xc1730101,0xfb840108,0xc1740101,0xfb840108,0xc1750101,
	0xfb840108,0xc1760101,0xfb840108,0xc1770101,0xfb840108,0xc1780101,0xfb840108,0xc1790101,
	0xfb840108,0xc17a0101,0xfb840108,0xc17b0101,0xfb840108,0xc17c0101,0xfb840108,0xc17d0101,
	0xfb840108,0xc17e0101,0xfb840108,0xc17f0101,0xfb840108,0xc1800101,0xfb840108,0xc1810101,
	0xfb840108,0xc1820101,0xfb840108,0xc1830101,0xfb840108,0xc1840101,0xfb840108,0xc1850101,
	0xfb840108,0xc1860101,0xfb840108,0xc1870101,0xfb840108,0xc1880101,0xfb840108,0xc1890101,
	0xfb840108,0xc18a0101,0xfb840108,0xc18b0101,0xfb840108,0xc18c0101,0xfb840108,0xc18d0101,
	0xfb840108,0xc18e0101,0xfb840108,0xc18f0101,0xfb840108,0xc1900101,0xfb840108,0xc1910101,
	0xfb840108,0xc1920101,0xfb840108,0xc1930101,0xfb840108,0xc1940101,0xfb840108,0xc1950101,
	0xfb840108,0xc1960101,0xfb840108,0xc1970101,0xfb840108,0xc1980101,0xfb840108,0xc1990101,
	0xfb840108,0xc19a0101,0xfb840108,0xc19b0101,0xfb840108,0xc19c0101,0xfb840108,0xc19d0101,
	0xfb840108,0xc19e0101,0xfb840108,0xc19f0101,0xfb840108,0xc1a00101,0xfb840108,0xc1a10101,
	0xfb840108,0xc1a20101,0xfb840108,0xc1a30101,0xfb840108,0xc1a40101,0xfb840108,0xc1a50101,
	0xfb840108,0xc1a60101,0xfb840108,0xc1a70101,0xfb840108,0xc1a80101,0xfb840108,0xc1a90101,
	0xfb840108,0xc1aa0101,0xfb840108,0xc1ab0101,0xfb840108,0xc1ac0101,0xfb840108,0xc1ad0101,
	0xfb840108,0xc1ae0101,0xfb840108,0xc1af0101,0xfb840108,0xc1b00101,0xfb840108,0xc1b10101,
	0xfb840108,0xc1b20101,0xfb840108,0xc1b30101,0xfb840108,0xc1b40101,0xfb840108,0xc1b50101,
	0xfb840108,0xc1b60101,0xfb840108,0xc1b70101,0xfb840108,0xc1b80101,0xfb840108,0xc1b90101,
	0xfb840108,0xc1ba0101,0xfb840108,0xc1bb0101,0xfb840108,0xc1bc0101,0xfb840108,0xc1bd0101,
	0xfb840108,0xc1be0101,0xfb840108,0xc1bf0101,0xfb840108,0xc1c00101,0xfb840108,0xc1c10101,
	0xfb840108,0xc1c20101,0xfb840108,0xc1c30101,0xfb840108,0xc1c40101,0xfb840108,0xc1c50101,
	0xfb840108,0xc1c60101,0xfb840108,0xc1c70101,0xfb840108,0xc1c80101,0xfb840108,0xc1c90101,
	0xfb840108,0xc1ca0101,0xfb840108,0xc1cb0101,0xfb840108,0xc1cc0101,0xfb840108,0xc1cd0101,
	0xfb840108,0xc1ce0101,0xfb840108,0xc1cf0101,0xfb840108,0xc1d00101,0xfb840108,0xc1d10101,
	0xfb840108,0xc1d20101,0xfb840108,0xc1d30101,0xfb840108,0xc1d40101,0xfb840108,0xc1d50101,
	0xfb840108,0xc1d60101,0xfb840108,0xc1d70101,0xfb840108,0xc1d80101,0xfb840108,0xc1d90101,
	0xfb840108,0xc1da0101,0xfb840108,0xc1db0101,0xfb840108,0xc1dc0101,0xfb840108,0xc1dd0101,
	0xfb840108,0xc1de0101,0xfb840108,0xc1df0101,0xfb840108,0xc1e00101,0xfb840108,0xc1e10101,
	0xfb840108,0xc1e20101,0xfb840108,0xc1e30101,0xfb840108,0xc1e40101,0xfb840108,0xc1e50101,
	0xfb840108,0xc1e60101,0xfb840108,0xc1e70101,0xfb840108,0xc1e80101,0xfb840108,0xc1e90101,
	0xfb840108,0xc1ea0101,0xfb840108,0xc1eb0101,0xfb840108,0xc1ec0101,0xfb840108,0xc1ed0101,
	0xfb840108,0xc1ee0101,0xfb840108,0xc1ef0101,0xfb840108,0xc1f00101,0xfb840108,0xc1f10101,
	0xfb840108,0xc1f20101,0xfb840108,0xc1f30101,0xfb840108,0xc1f40101,0xfb840108,0xc1f50101,
	0xfb840108,0xc1f60101,0xfb840108,0xc1f70101,0xfb840108,0xc1f80101,0xfb840108,0xc1f90101,
	0xfb840108,0xc1fa0101,0xfb840108,0xc1fb0101,0xfb840108,0xc1fc0101,0xfb840108,0xc1fd0101,
	0xfb840108,0xc1fe0101,0xfb840108,0xc1ff0101,0xfb840108,0xc2000101,0xfb840108,0xc2010101,
	0xfb840108,0xc2020101,0xfb840108,0xc2030101,0xfb840108,0xc2040101,0xfb840108,0xc2050101,
	0xfb840108,0xc2060101,0xfb840108,0xc2070101,0xfb840108,0xc2080101,0xfb840108,0xc2090101,
	0xfb840108,0xc20a0101,0xfb840108,0xc20b0101,0xfb840108,0xc20c0101,0xfb840108,0xc20d0101,
	0xfb840108,0xc20e0101,0xfb840108,0xc20f0101,0xfb840108,0xc2100101,0xfb840108,0xc2110101,
	0xfb840108,0xc2120101,0xfb840108,0xc2130101,0xfb840108,0xc2140101,0xfb840108,0xc2150101,
	0xfb840108,0xc2160101,0xfb840108,0xc2170101,0xfb840108,0xc2180101,0xfb840108,0xc2190101,
	0xfb840108,0xc21a0101,0xfb840108,0xc21b0101,0xfb840108,0xc21c0101,0xfb840108,0xc21d0101,
	0xfb840108,0xc21e0101,0xfb840108,0xc21f0101,0xfb840108,0xc2200101,0xfb840108,0xc2210101,
	0xfb840108,0xc2220101,0xfb840108,0xc2230101,0xfb840108,0xc2240101,0xfb840108,0xc2250101,
	0xfb840108,0xc2260101,0xfb840108,0xc2270101,0xfb840108,0xc2280101,0xfb840108,0xc2290101,
	0xfb840108,0xc22a0101,0xfb840108,0xc22b0101,0xfb840108,0xc22c0101,0xfb840108,0xc22d0101,
	0xfb840108,0xc22e0101,0xfb840108,0xc22f0101,0xfb840108,0xc2300101,0xfb840108,0xc2310101,
	0xfb840108,0xc2320101,0xfb840108,0xc2330101,0xfb840108,0xc2340101,0xfb840108,0xc2350101,
	0xfb840108,0xc2360101,0xfb840108,0xc2370101,0xfb840108,0xc2380101,0xfb840108,0xc2390101,
	0xfb840108,0xc23a0101,0xfb840108,0xc23b0101,0xfb840108,0xc23c0101,0xfb840108,0xc23d0101,
	0xfb840108,0xc23e0101,0xfb840108,0xc23f0101,0xfb840108,0xc2400101,0xfb840108,0xc2410101,
	0xfb840108,0xc2420101,0xfb840108,0xc2430101,0xfb840108,0xc2440101,0xfb840108,0xc2450101,
	0xfb840108,0xc2460101,0xfb840108,0xc2470101,0xfb840108,0xc2480101,0xfb840108,0xc2490101,
	0xfb840108,0xc24a0101,0xfb840108,0xc24b0101,0xfb840108,0xc24c0101,0xfb840108,0xc24d0101,
	0xfb840108,0xc24e0101,0xfb840108,0xc24f0101,0xfb840108,0xc2500101,0xfb840108,0xc2510101,
	0xfb840108,0xc2520101,0xfb840108,0xc2530101,0xfb840108,0xc2540101,0xfb840108,0xc2550101,
	0xfb840108,0xc2560101,0xfb840108,0xc2570101,0xfb840108,0xc2580101,0xfb840108,0xc2590101,
	0xfb840108,0xc25a0101,0xfb840108,0xc25b0101,0xfb840108,0xc25c0101,0xfb840108,0xc25d0101,
	0xfb840108,0xc25e0101,0xfb840108,0xc25f0101,0xfb840108,0xc2600101,0xfb840108,0xc2610101,
	0xfb840108,0xc2620101,0xfb840108,0xc2630101,0xfb840108,0xc2640101,0xfb840108,0xc2650101,
	0xfb840108,0xc2660101,0xfb840108,0xc2670101,0xfb840108,0xc2680101,0xfb840108,0xc2690101,
	0xfb840108,0xc26a0101,0xfb840108,0xc26b0101,0xfb840108,0xc26c0101,0xfb840108,0xc26d0101,
	0xfb840108,0xc26e0101,0xfb840108,0xc26f0101,0xfb840108,0xc2700101,0xfb840108,0xc2710101,
	0xfb840108,0xc2720101,0xfb840108,0xc2730101,0xfb840108,0xc2740101,0xfb840108,0xc2750101,
	0xfb840108,0xc2760101,0xfb840108,0xc2770101,0xfb840108,0xc2780101,0xfb840108,0xc2790101,
	0xfb840108,0xc27a0101,0xfb840108,0xc27b0101,0xfb840108,0xc27c0101,0xfb840108,0xc27d0101,
	0xfb840108,0xc27e0101,0xfb840108,0xc27f0101,0xfb840108,0xc2800101,0xfb840108,0xc2810101,
	0xfb840108,0xc2820101,0xfb840108,0xc2830101,0xfb840108,0xc2840101,0xfb840108,0xc2850101,
	0xfb840108,0xc2860101,0xfb840108,0xc2870101,0xfb840108,0xc2880101,0xfb840108,0xc2890101,
	0xfb840108,0xc28a0101,0xfb840108,0xc28b0101,0xfb840108,0xc28c0101,0xfb840108,0xc28d0101,
	0xfb840108,0xc28e0101,0xfb840108,0xc28f0101,0xfb840108,0xc2900101,0xfb840108,0xc2910101,
	0xfb840108,0xc2920101,0xfb840108,0xc2930101,0xfb840108,0xc2940101,0xfb840108,0xc2950101,
	0xfb840108,0xc2960101,0xfb840108,0xc2970101,0xfb840108,0xc2980101,0xfb840108,0xc2990101,
	0xfb840108,0xc29a0101,0xfb840108,0xc29b0101,0xfb840108,0xc29c0101,0xfb840108,0xc29d0101,
	0xfb840108,0xc29e0101,0xfb840108,0xc29f0101,0xfb840108,0xc2a00101,0xfb840108,0xc2a10101,
	0xfb840108,0xc2a20101,0xfb840108,0xc2a30101,0xfb840108,0xc2a40101,0xfb840108,0xc2a50101,
	0xfb840108,0xc2a60101,0xfb840108,0xc2a70101,0xfb840108,0xc2a80101,0xfb840108,0xc2a90101,
	0xfb840108,0xc2aa0101,0xfb840108,0xc2ab0101,0xfb840108,0xc2ac0101,0xfb840108,0xc2ad0101,
	0xfb840108,0xc2ae0101,0xfb840108,0xc2af0101,0xfb840108,0xc2b00101,0xfb840108,0xc2b10101,
	0xfb840108,0xc2b20101,0xfb840108,0xc2b30101,0xfb840108,0xc2b40101,0xfb840108,0xc2b50101,
	0xfb840108,0xc2b60101,0xfb840108,0xc2b70101,0xfb840108,0xc2b80101,0xfb840108,0xc2b90101,
	0xfb840108,0xc2ba0101,0xfb840108,0xc2bb0101,0xfb840108,0xc2bc0101,0xfb840108,0xc2bd0101,
	0xfb840108,0xc2be0101,0xfb840108,0xc2bf0101,0xfb840108,0xc2c00101,0xfb840108,0xc2c10101,
	0xfb840108,0xc2c20101,0xfb840108,0xc2c30101,0xfb840108,0xc2c40101,0xfb840108,0xc2c50101,
	0xfb840108,0xc2c60101,0xfb840108,0xc2c70101,0xfb840108,0xc2c80101,0xfb840108,0xc2c90101,
	0xfb840108,0xc2ca0101,0xfb840108,0xc2cb0101,0xfb840108,0xc2cc0101,0xfb840108,0xc2cd0101,
	0xfb840108,0xc2ce0101,0xfb840108,0xc2cf0101,0xfb840108,0xc2d00101,0xfb840108,0xc2d10101,
	0xfb840108,0xc2d20101,0xfb840108,0xc2d30101,0xfb840108,0xc2d40101,0xfb840108,0xc2d50101,
	0xfb840108,0xc2d60101,0xfb840108,0xc2d70101,0xfb840108,0xc2d80101,0xfb840108,0xc2d90101,
	0xfb840108,0xc2da0101,0xfb840108,0xc2db0101,0xfb840108,0xc2dc0101,0xfb840108,0xc2dd0101,
	0xfb840108,0xc2de0101,0xfb840108,0xc2df0101,0xfb840108,0xc2e00101,0xfb840108,0xc2e10101,
	0xfb840108,0xc2e20101,0xfb840108,0xc2e30101,0xfb840108,0xc2e40101,0xfb840108,0xc2e50101,
	0xfb840108,0xc2e60101,0xfb840108,0xc2e70101,0xfb840108,0xc2e80101,0xfb840108,0xc2e90101,
	0xfb840108,0xc2ea0101,0xfb840108,0xc2eb0101,0xfb840108,0xc2ec0101,0xfb840108,0xc2ed0101,
	0xfb840108,0xc2ee0101,0xfb840108,0xc2ef0101,0xfb840108,0xc2f00101,0xfb840108,0xc2f10101,
	0xfb840108,0xc2f20101,0xfb840108,0xc2f30101,0xfb840108,0xc2f40101,0xfb840108,0xc2f50101,
	0xfb840108,0xc2f60101,0xfb840108,0xc2f70101,0xfb840108,0xc2f80101,0xfb840108,0xc2f90101,
	0xfb840108,0xc2fa0101,0xfb840108,0xc2fb0101,0xfb840108,0xc2fc0101,0xfb840108,0xc2fd0101,
	0xfb840108,0xc2fe0101,0xfb840108,0xc2ff0101,0xfb840108,0xc3000101,0xfb840108,0xc3010101,
	0xfb840108,0xc3020101,0xfb840108,0xc3030101,0xfb840108,0xc3040101,0xfb840108,0xc3050101,
	0xfb840108,0xc3060101,0xfb840108,0xc3070101,0xfb840108,0xc3080101,0xfb840108,0xc3090101,
	0xfb840108,0xc30a0101,0xfb840108,0xc30b0101,0xfb840108,0xc30c0101,0xfb840108,0xc30d0101,
	0xfb840108,0xc30e0101,0xfb840108,0xc30f0101,0xfb840108,0xc3100101,0xfb840108,0xc3110101,
	0xfb840108,0xc3120101,0xfb840108,0xc3130101,0xfb840108,0xc3140101,0xfb840108,0xc3150101,
	0xfb840108,0xc3160101,0xfb840108,0xc3170101,0xfb840108,0xc3180101,0xfb840108,0xc3190101,
	0xfb840108,0xc31a0101,0xfb840108,0xc31b0101,0xfb840108,0xc31c0101,0xfb840108,0xc31d0101,
	0xfb840108,0xc31e0101,0xfb840108,0xc31f0101,0xfb840108,0xc3200101,0xfb840108,0xc3210101,
	0xfb840108,0xc3220101,0xfb840108,0xc3230101,0xfb840108,0xc3240101,0xfb840108,0xc3250101,
	0xfb840108,0xc3260101,0xfb840108,0xc3270101,0xfb840108,0xc3280101,0xfb840108,0xc3290101,
	0xfb840108,0xc32a0101,0xfb840108,0xc32b0101,0xfb840108,0xc32c0101,0xfb840108,0xc32d0101,
	0xfb840108,0xc32e0101,0xfb840108,0xc32f0101,0xfb840108,0xc3300101,0xfb840108,0xc3310101,
	0xfb840108,0xc3320101,0xfb840108,0xc3330101,0xfb840108,0xc3340101,0xfb840108,0xc3350101,
	0xfb840108,0xc3360101,0xfb840108,0xc3370101,0xfb840108,0xc3380101,0xfb840108,0xc3390101,
	0xfb840108,0xc33a0101,0xfb840108,0xc33b0101,0xfb840108,0xc33c0101,0xfb840108,0xc33d0101,
	0xfb840108,0xc33e0101,0xfb840108,0xc33f0101,0xfb840108,0xc3400101,0xfb840108,0xc3410101,
	0xfb840108,0xc3420101,0xfb840108,0xc3430101,0xfb840108,0xc3440101,0xfb840108,0xc3450101,
	0xfb840108,0xc3460101,0xfb840108,0xc3470101,0xfb840108,0xc3480101,0xfb840108,0xc3490101,
	0xfb840108,0xc34a0101,0xfb840108,0xc34b0101,0xfb840108,0xc34c0101,0xfb840108,0xc34d0101,
	0xfb840108,0xc34e0101,0xfb840108,0xc34f0101,0xfb840108,0xc3500101,0xfb840108,0xc3510101,
	0xfb840108,0xc3520101,0xfb840108,0xc3530101,0xfb840108,0xc3540101,0xfb840108,0xc3550101,
	0xfb840108,0xc3560101,0xfb840108,0xc3570101,0xfb840108,0xc3580101,0xfb840108,0xc3590101,
	0xfb840108,0xc35a0101,0xfb840108,0xc35b0101,0xfb840108,0xc35c0101,0xfb840108,0xc35d0101,
	0xfb840108,0xc35e0101,0xfb840108,0xc35f0101,0xfb840108,0xc3600101,0xfb840108,0xc3610101,
	0xfb840108,0xc3620101,0xfb840108,0xc3630101,0xfb840108,0xc3640101,0xfb840108,0xc3650101,
	0xfb840108,0xc3660101,0xfb840108,0xc3670101,0xfb840108,0xc3680101,0xfb840108,0xc3690101,
	0xfb840108,0xc36a0101,0xfb840108,0xc36b0101,0xfb840108,0xc36c0101,0xfb840108,0xc36d0101,
	0xfb840108,0xc36e0101,0xfb840108,0xc36f0101,0xfb840108,0xc3700101,0xfb840108,0xc3710101,
	0xfb840108,0xc3720101,0xfb840108,0xc3730101,0xfb840108,0xc3740101,0xfb840108,0xc3750101,
	0xfb840108,0xc3760101,0xfb840108,0xc3770101,0xfb840108,0xc3780101,0xfb840108,0xc3790101,
	0xfb840108,0xc37a0101,0xfb840108,0xc37b0101,0xfb840108,0xc37c0101,0xfb840108,0xc37d0101,
	0xfb840108,0xc37e0101,0xfb840108,0xc37f0101,0xfb840108,0xc3800101,0xfb840108,0xc3810101,
	0xfb840108,0xc3820101,0xfb840108,0xc3830101,0xfb840108,0xc3840101,0xfb840108,0xc3850101,
	0xfb840108,0xc3860101,0xfb840108,0xc3870101,0xfb840108,0xc3880101,0xfb840108,0xc3890101,
	0xfb840108,0xc38a0101,0xfb840108,0xc38b0101,0xfb840108,0xc38c0101,0xfb840108,0xc38d0101,
	0xfb840108,0xc38e0101,0xfb840108,0xc38f0101,0xfb840108,0xc3900101,0xfb840108,0xc3910101,
	0xfb840108,0xc3920101,0xfb840108,0xc3930101,0xfb840108,0xc3940101,0xfb840108,0xc3950101,
	0xfb840108,0xc3960101,0xfb840108,0xc3970101,0xfb840108,0xc3980101,0xfb840108,0xc3990101,
	0xfb840108,0xc39a0101,0xfb840108,0xc39b0101,0xfb840108,0xc39c0101,0xfb840108,0xc39d0101,
	0xfb840108,0xc39e0101,0xfb840108,0xc39f0101,0xfb840108,0xc3a00101,0xfb840108,0xc3a10101,
	0xfb840108,0xc3a20101,0xfb840108,0xc3a30101,0xfb840108,0xc3a40101,0xfb840108,0xc3a50101,
	0xfb840108,0xc3a60101,0xfb840108,0xc3a70101,0xfb840108,0xc3a80101,0xfb840108,0xc3a90101,
	0xfb840108,0xc3aa0101,0xfb840108,0xc3ab0101,0xfb840108,0xc3ac0101,0xfb840108,0xc3ad0101,
	0xfb840108,0xc3ae0101,0xfb840108,0xc3af0101,0xfb840108,0xc3b00101,0xfb840108,0xc3b10101,
	0xfb840108,0xc3b20101,0xfb840108,0xc3b30101,0xfb840108,0xc3b40101,0xfb840108,0xc3b50101,
	0xfb840108,0xc3b60101,0xfb840108,0xc3b70101,0xfb840108,0xc3b80101,0xfb840108,0xc3b90101,
	0xfb840108,0xc3ba0101,0xfb840108,0xc3bb0101,0xfb840108,0xc3bc0101,0xfb840108,0xc3bd0101,
	0xfb840108,0xc3be0101,0xfb840108,0xc3bf0101,0xfb840108,0xc3c00101,0xfb840108,0xc3c10101,
	0xfb840108,0xc3c20101,0xfb840108,0xc3c30101,0xfb840108,0xc3c40101,0xfb840108,0xc3c50101,
	0xfb840108,0xc3c60101,0xfb840108,0xc3c70101,0xfb840108,0xc3c80101,0xfb840108,0xc3c90101,
	0xfb840108,0xc3ca0101,0xfb840108,0xc3cb0101,0xfb840108,0xc3cc0101,0xfb840108,0xc3cd0101,
	0xfb840108,0xc3ce0101,0xfb840108,0xc3cf0101,0xfb840108,0xc3d00101,0xfb840108,0xc3d10101,
	0xfb840108,0xc3d20101,0xfb840108,0xc3d30101,0xfb840108,0xc3d40101,0xfb840108,0xc3d50101,
	0xfb840108,0xc3d60101,0xfb840108,0xc3d70101,0xfb840108,0xc3d80101,0xfb840108,0xc3d90101,
	0xfb840108,0xc3da0101,0xfb840108,0xc3db0101,0xfb840108,0xc3dc0101,0xfb840108,0xc3dd0101,
	0xfb840108,0xc3de0101,0xfb840108,0xc3df0101,0xfb840108,0xc3e00101,0xfb840108,0xc3e10101,
	0xfb840108,0xc3e20101,0xfb840108,0xc3e30101,0xfb840108,0xc3e40101,0xfb840108,0xc3e50101,
	0xfb840108,0xc3e60101,0xfb840108,0xc3e70101,0xfb840108,0xc3e80101,0xfb840108,0xc3e90101,
	0xfb840108,0xc3ea0101,0xfb840108,0xc3eb0101,0xfb840108,0xc3ec0101,0xfb840108,0xc3ed0101,
	0xfb840108,0xc3ee0101,0xfb840108,0xc3ef0101,0xfb840108,0xc3f00101,0xfb840108,0xc3f10101,
	0xfb840108,0xc3f20101,0xfb840108,0xc3f30101,0xfb840108,0xc3f40101,0xfb840108,0xc3f50101,
	0xfb840108,0xc3f60101,0xfb840108,0xc3f70101,0xfb840108,0xc3f80101,0xfb840108,0xc3f90101,
	0xfb840108,0xc3fa0101,0xfb840108,0xc3fb0101,0xfb840108,0xc3fc0101,0xfb840108,0xc3fd0101,
	0xfb840108,0xc3fe0101,0xfb840108,0xc3ff0101,0xfb840108,0xc4000101,0xfb840108,0xc4010101,
	0xfb840108,0xc4020101,0xfb840108,0xc4030101,0xfb840108,0xc4040101,0xfb840108,0xc4050101,
	0xfb840108,0xc4060101,0xfb840108,0xc4070101,0xfb840108,0xc4080101,0xfb840108,0xc4090101,
	0xfb840108,0xc40a0101,0xfb840108,0xc40b0101,0xfb840108,0xc40c0101,0xfb840108,0xc40d0101,
	0xfb840108,0xc40e0101,0xfb840108,0xc40f0101,0xfb840108,0xc4100101,0xfb840108,0xc4110101,
	0xfb840108,0xc4120101,0xfb840108,0xc4130101,0xfb840108,0xc4140101,0xfb840108,0xc4150101,
	0xfb840108,0xc4160101,0xfb840108,0xc4170101,0xfb840108,0xc4180101,0xfb840108,0xc4190101,
	0xfb840108,0xc41a0101,0xfb840108,0xc41b0101,0xfb840108,0xc41c0101,0xfb840108,0xc41d0101,
	0xfb840108,0xc41e0101,0xfb840108,0xc41f0101,0xfb840108,0xc4200101,0xfb840108,0xc4210101,
	0xfb840108,0xc4220101,0xfb840108,0xc4230101,0xfb840108,0xc4240101,0xfb840108,0xc4250101,
	0xfb840108,0xc4260101,0xfb840108,0xc4270101,0xfb840108,0xc4280101,0xfb840108,0xc4290101,
	0xfb840108,0xc42a0101,0xfb840108,0xc42b0101,0xfb840108,0xc42c0101,0xfb840108,0xc42d0101,
	0xfb840108,0xc42e0101,0xfb840108,0xc42f0101,0xfb840108,0xc4300101,0xfb840108,0xc4310101,
	0xfb840108,0xc4320101,0xfb840108,0xc4330101,0xfb840108,0xc4340101,0xfb840108,0xc4350101,
	0xfb840108,0xc4360101,0xfb840108,0xc4370101,0xfb840108,0xc4380101,0xfb840108,0xc4390101,
	0xfb840108,0xc43a0101,0xfb840108,0xc43b0101,0xfb840108,0xc43c0101,0xfb840108,0xc43d0101,
	0xfb840108,0xc43e0101,0xfb840108,0xc43f0101,0xfb840108,0xc4400101,0xfb840108,0xc4410101,
	0xfb840108,0xc4420101,0xfb840108,0xc4430101,0xfb840108,0xc4440101,0xfb840108,0xc4450101,
	0xfb840108,0xc4460101,0xfb840108,0xc4470101,0xfb840108,0xc4480101,0xfb840108,0xc4490101,
	0xfb840108,0xc44a0101,0xfb840108,0xc44b0101,0xfb840108,0xc44c0101,0xfb840108,0xc44d0101,
	0xfb840108,0xc44e0101,0xfb840108,0xc44f0101,0xfb840108,0xc4500101,0xfb840108,0xc4510101,
	0xfb840108,0xc4520101,0xfb840108,0xc4530101,0xfb840108,0xc4540101,0xfb840108,0xc4550101,
	0xfb840108,0xc4560101,0xfb840108,0xc4570101,0xfb840108,0xc4580101,0xfb840108,0xc4590101,
	0xfb840108,0xc45a0101,0xfb840108,0xc45b0101,0xfb840108,0xc45c0101,0xfb840108,0xc45d0101,
	0xfb840108,0xc45e0101,0xfb840108,0xc45f0101,0xfb840108,0xc4600101,0xfb840108,0xc4610101,
	0xfb840108,0xc4620101,0xfb840108,0xc4630101,0xfb840108,0xc4640101,0xfb840108,0xc4650101,
	0xfb840108,0xc4660101,0xfb840108,0xc4670101,0xfb840108,0xc4680101,0xfb840108,0xc4690101,
	0xfb840108,0xc46a0101,0xfb840108,0xc46b0101,0xfb840108,0xc46c0101,0xfb840108,0xc46d0101,
	0xfb840108,0xc46e0101,0xfb840108,0xc46f0101,0xfb840108,0xc4700101,0xfb840108,0xc4710101,
	0xfb840108,0xc4720101,0xfb840108,0xc4730101,0xfb840108,0xc4740101,0xfb840108,0xc4750101,
	0xfb840108,0xc4760101,0xfb840108,0xc4770101,0xfb840108,0xc4780101,0xfb840108,0xc4790101,
	0xfb840108,0xc47a0101,0xfb840108,0xc47b0101,0xfb840108,0xc47c0101,0xfb840108,0xc47d0101,
	0xfb840108,0xc47e0101,0xfb840108,0xc47f0101,0xfb840108,0xc4800101,0xfb840108,0xc4810101,
	0xfb840108,0xc4820101,0xfb840108,0xc4830101,0xfb840108,0xc4840101,0xfb840108,0xc4850101,
	0xfb840108,0xc4860101,0xfb840108,0xc4870101,0xfb840108,0xc4880101,0xfb840108,0xc4890101,
	0xfb840108,0xc48a0101,0xfb840108,0xc48b0101,0xfb840108,0xc48c0101,0xfb840108,0xc48d0101,
	0xfb840108,0xc48e0101,0xfb840108,0xc48f0101,0xfb840108,0xc4900101,0xfb840108,0xc4910101,
	0xfb840108,0xc4920101,0xfb840108,0xc4930101,0xfb840108,0xc4940101,0xfb840108,0xc4950101,
	0xfb840108,0xc4960101,0xfb840108,0xc4970101,0xfb840108,0xc4980101,0xfb840108,0xc4990101,
	0xfb840108,0xc49a0101,0xfb840108,0xc49b0101,0xfb840108,0xc49c0101,0xfb840108,0xc49d0101,
	0xfb840108,0xc49e0101,0xfb840108,0xc49f0101,0xfb840108,0xc4a00101,0xfb840108,0xc4a10101,
	0xfb840108,0xc4a20101,0xfb840108,0xc4a30101,0xfb840108,0xc4a40101,0xfb840108,0xc4a50101,
	0xfb840108,0xc4a60101,0xfb840108,0xc4a70101,0xfb840108,0xc4a80101,0xfb840108,0xc4a90101,
	0xfb840108,0xc4aa0101,0xfb840108,0xc4ab0101,0xfb840108,0xc4ac0101,0xfb840108,0xc4ad0101,
	0xfb840108,0xc4ae0101,0xfb840108,0xc4af0101,0xfb840108,0xc4b00101,0xfb840108,0xc4b10101,
	0xfb840108,0xc4b20101,0xfb840108,0xc4b30101,0xfb840108,0xc4b40101,0xfb840108,0xc4b50101,
	0xfb840108,0xc4b60101,0xfb840108,0xc4b70101,0xfb840108,0xc4b80101,0xfb840108,0xc4b90101,
	0xfb840108,0xc4ba0101,0xfb840108,0xc4bb0101,0xfb840108,0xc4bc0101,0xfb840108,0xc4bd0101,
	0xfb840108,0xc4be0101,0xfb840108,0xc4bf0101,0xfb840108,0xc4c00101,0xfb840108,0xc4c10101,
	0xfb840108,0xc4c20101,0xfb840108,0xc4c30101,0xfb840108,0xc4c40101,0xfb840108,0xc4c50101,
	0xfb840108,0xc4c60101,0xfb840108,0xc4c70101,0xfb840108,0xc4c80101,0xfb840108,0xc4c90101,
	0xfb840108,0xc4ca0101,0xfb840108,0xc4cb0101,0xfb840108,0xc4cc0101,0xfb840108,0xc4cd0101,
	0xfb840108,0xc4ce0101,0xfb840108,0xc4cf0101,0xfb840108,0xc4d00101,0xfb840108,0xc4d10101,
	0xfb840108,0xc4d20101,0xfb840108,0xc4d30101,0xfb840108,0xc4d40101,0xfb840108,0xc4d50101,
	0xfb840108,0xc4d60101,0xfb840108,0xc4d70101,0xfb840108,0xc4d80101,0xfb840108,0xc4d90101,
	0xfb840108,0xc4da0101,0xfb840108,0xc4db0101,0xfb840108,0xc4dc0101,0xfb840108,0xc4dd0101,
	0xfb840108,0xc4de0101,0xfb840108,0xc4df0101,0xfb840108,0xc4e00101,0xfb840108,0xc4e10101,
	0xfb840108,0xc4e20101,0xfb840108,0xc4e30101,0xfb840108,0xc4e40101,0xfb840108,0xc4e50101,
	0xfb840108,0xc4e60101,0xfb840108,0xc4e70101,0xfb840108,0xc4e80101,0xfb840108,0xc4e90101,
	0xfb840108,0xc4ea0101,0xfb840108,0xc4eb0101,0xfb840108,0xc4ec0101,0xfb840108,0xc4ed0101,
	0xfb840108,0xc4ee0101,0xfb840108,0xc4ef0101,0xfb840108,0xc4f00101,0xfb840108,0xc4f10101,
	0xfb840108,0xc4f20101,0xfb840108,0xc4f30101,0xfb840108,0xc4f40101,0xfb840108,0xc4f50101,
	0xfb840108,0xc4f60101,0xfb840108,0xc4f70101,0xfb840108,0xc4f80101,0xfb840108,0xc4f90101,
	0xfb840108,0xc4fa0101,0xfb840108,0xc4fb0101,0xfb840108,0xc4fc0101,0xfb840108,0xc4fd0101,
	0xfb840108,0xc4fe0101,0xfb840108,0xc4ff0101,0xfb840108,0xc5000101,0xfb840108,0xc5010101,
	0xfb840108,0xc5020101,0xfb840108,0xc5030101,0xfb840108,0xc5040101,0xfb840108,0xc5050101,
	0xfb840108,0xc5060101,0xfb840108,0xc5070101,0xfb840108,0xc5080101,0xfb840108,0xc5090101,
	0xfb840108,0xc50a0101,0xfb840108,0xc50b0101,0xfb840108,0xc50c0101,0xfb840108,0xc50d0101,
	0xfb840108,0xc50e0101,0xfb840108,0xc50f0101,0xfb840108,0xc5100101,0xfb840108,0xc5110101,
	0xfb840108,0xc5120101,0xfb840108,0xc5130101,0xfb840108,0xc5140101,0xfb840108,0xc5150101,
	0xfb840108,0xc5160101,0xfb840108,0xc5170101,0xfb840108,0xc5180101,0xfb840108,0xc5190101,
	0xfb840108,0xc51a0101,0xfb840108,0xc51b0101,0xfb840108,0xc51c0101,0xfb840108,0xc51d0101,
	0xfb840108,0xc51e0101,0xfb840108,0xc51f0101,0xfb840108,0xc5200101,0xfb840108,0xc5210101,
	0xfb840108,0xc5220101,0xfb840108,0xc5230101,0xfb840108,0xc5240101,0xfb840108,0xc5250101,
	0xfb840108,0xc5260101,0xfb840108,0xc5270101,0xfb840108,0xc5280101,0xfb840108,0xc5290101,
	0xfb840108,0xc52a0101,0xfb840108,0xc52b0101,0xfb840108,0xc52c0101,0xfb840108,0xc52d0101,
	0xfb840108,0xc52e0101,0xfb840108,0xc52f0101,0xfb840108,0xc5300101,0xfb840108,0xc5310101,
	0xfb840108,0xc5320101,0xfb840108,0xc5330101,0xfb840108,0xc5340101,0xfb840108,0xc5350101,
	0xfb840108,0xc5360101,0xfb840108,0xc5370101,0xfb840108,0xc5380101,0xfb840108,0xc5390101,
	0xfb840108,0xc53a0101,0xfb840108,0xc53b0101,0xfb840108,0xc53c0101,0xfb840108,0xc53d0101,
	0xfb840108,0xc53e0101,0xfb840108,0xc53f0101,0xfb840108,0xc5400101,0xfb840108,0xc5410101,
	0xfb840108,0xc5420101,0xfb840108,0xc5430101,0xfb840108,0xc5440101,0xfb840108,0xc5450101,
	0xfb840108,0xc5460101,0xfb840108,0xc5470101,0xfb840108,0xc5480101,0xfb840108,0xc5490101,
	0xfb840108,0xc54a0101,0xfb840108,0xc54b0101,0xfb840108,0xc54c0101,0xfb840108,0xc54d0101,
	0xfb840108,0xc54e0101,0xfb840108,0xc54f0101,0xfb840108,0xc5500101,0xfb840108,0xc5510101,
	0xfb840108,0xc5520101,0xfb840108,0xc5530101,0xfb840108,0xc5540101,0xfb840108,0xc5550101,
	0xfb840108,0xc5560101,0xfb840108,0xc5570101,0xfb840108,0xc5580101,0xfb840108,0xc5590101,
	0xfb840108,0xc55a0101,0xfb840108,0xc55b0101,0xfb840108,0xc55c0101,0xfb840108,0xc55d0101,
	0xfb840108,0xc55e0101,0xfb840108,0xc55f0101,0xfb840108,0xc5600101,0xfb840108,0xc5610101,
	0xfb840108,0xc5620101,0xfb840108,0xc5630101,0xfb840108,0xc5640101,0xfb840108,0xc5650101,
	0xfb840108,0xc5660101,0xfb840108,0xc5670101,0xfb840108,0xc5680101,0xfb840108,0xc5690101,
	0xfb840108,0xc56a0101,0xfb840108,0xc56b0101,0xfb840108,0xc56c0101,0xfb840108,0xc56d0101,
	0xfb840108,0xc56e0101,0xfb840108,0xc56f0101,0xfb840108,0xc5700101,0xfb840108,0xc5710101,
	0xfb840108,0xc5720101,0xfb840108,0xc5730101,0xfb840108,0xc5740101,0xfb840108,0xc5750101,
	0xfb840108,0xc5760101,0xfb840108,0xc5770101,0xfb840108,0xc5780101,0xfb840108,0xc5790101,
	0xfb840108,0xc57a0101,0xfb840108,0xc57b0101,0xfb840108,0xc57c0101,0xfb840108,0xc57d0101,
	0xfb840108,0xc57e0101,0xfb840108,0xc57f0101,0xfb840108,0xc5800101,0xfb840108,0xc5810101,
	0xfb840108,0xc5820101,0xfb840108,0xc5830101,0xfb840108,0xc5840101,0xfb840108,0xc5850101,
	0xfb840108,0xc5860101,0xfb840108,0xc5870101,0xfb840108,0xc5880101,0xfb840108,0xc5890101,
	0xfb840108,0xc58a0101,0xfb840108,0xc58b0101,0xfb840108,0xc58c0101,0xfb840108,0xc58d0101,
	0xfb840108,0xc58e0101,0xfb840108,0xc58f0101,0xfb840108,0xc5900101,0xfb840108,0xc5910101,
	0xfb840108,0xc5920101,0xfb840108,0xc5930101,0xfb840108,0xc5940101,0xfb840108,0xc5950101,
	0xfb840108,0xc5960101,0xfb840108,0xc5970101,0xfb840108,0xc5980101,0xfb840108,0xc5990101,
	0xfb840108,0xc59a0101,0xfb840108,0xc59b0101,0xfb840108,0xc59c0101,0xfb840108,0xc59d0101,
	0xfb840108,0xc59e0101,0xfb840108,0xc59f0101,0xfb840108,0xc5a00101,0xfb840108,0xc5a10101,
	0xfb840108,0xc5a20101,0xfb840108,0xc5a30101,0xfb840108,0xc5a40101,0xfb840108,0xc5a50101,
	0xfb840108,0xc5a60101,0xfb840108,0xc5a70101,0xfb840108,0xc5a80101,0xfb840108,0xc5a90101,
	0xfb840108,0xc5aa0101,0xfb840108,0xc5ab0101,0xfb840108,0xc5ac0101,0xfb840108,0xc5ad0101,
	0xfb840108,0xc5ae0101,0xfb840108,0xc5af0101,0xfb840108,0xc5b00101,0xfb840108,0xc5b10101,
	0xfb840108,0xc5b20101,0xfb840108,0xc5b30101,0xfb840108,0xc5b40101,0xfb840108,0xc5b50101,
	0xfb840108,0xc5b60101,0xfb840108,0xc5b70101,0xfb840108,0xc5b80101,0xfb840108,0xc5b90101,
	0xfb840108,0xc5ba0101,0xfb840108,0xc5bb0101,0xfb840108,0xc5bc0101,0xfb840108,0xc5bd0101,
	0xfb840108,0xc5be0101,0xfb840108,0xc5bf0101,0xfb840108,0xc5c00101,0xfb840108,0xc5c10101,
	0xfb840108,0xc5c20101,0xfb840108,0xc5c30101,0xfb840108,0xc5c40101,0xfb840108,0xc5c50101,
	0xfb840108,0xc5c60101,0xfb840108,0xc5c70101,0xfb840108,0xc5c80101,0xfb840108,0xc5c90101,
	0xfb840108,0xc5ca0101,0xfb840108,0xc5cb0101,0xfb840108,0xc5cc0101,0xfb840108,0xc5cd0101,
	0xfb840108,0xc5ce0101,0xfb840108,0xc5cf0101,0xfb840108,0xc5d00101,0xfb840108,0xc5d10101,
	0xfb840108,0xc5d20101,0xfb840108,0xc5d30101,0xfb840108,0xc5d40101,0xfb840108,0xc5d50101,
	0xfb840108,0xc5d60101,0xfb840108,0xc5d70101,0xfb840108,0xc5d80101,0xfb840108,0xc5d90101,
	0xfb840108,0xc5da0101,0xfb840108,0xc5db0101,0xfb840108,0xc5dc0101,0xfb840108,0xc5dd0101,
	0xfb840108,0xc5de0101,0xfb840108,0xc5df0101,0xfb840108,0xc5e00101,0xfb840108,0xc5e10101,
	0xfb840108,0xc5e20101,0xfb840108,0xc5e30101,0xfb840108,0xc5e40101,0xfb840108,0xc5e50101,
	0xfb840108,0xc5e60101,0xfb840108,0xc5e70101,0xfb840108,0xc5e80101,0xfb840108,0xc5e90101,
	0xfb840108,0xc5ea0101,0xfb840108,0xc5eb0101,0xfb840108,0xc5ec0101,0xfb840108,0xc5ed0101,
	0xfb840108,0xc5ee0101,0xfb840108,0xc5ef0101,0xfb840108,0xc5f00101,0xfb840108,0xc5f10101,
	0xfb840108,0xc5f20101,0xfb840108,0xc5f30101,0xfb840108,0xc5f40101,0xfb840108,0xc5f50101,
	0xfb840108,0xc5f60101,0xfb840108,0xc5f70101,0xfb840108,0xc5f80101,0xfb840108,0xc5f90101,
	0xfb840108,0xc5fa0101,0xfb840108,0xc5fb0101,0xfb840108,0xc5fc0101,0xfb840108,0xc5fd0101,
	0xfb840108,0xc5fe0101,0xfb840108,0xc5ff0101,0xfb840108,0xc6000101,0xfb840108,0xc6010101,
	0xfb840108,0xc6020101,0xfb840108,0xc6030101,0xfb840108,0xc6040101,0xfb840108,0xc6050101,
	0xfb840108,0xc6060101,0xfb840108,0xc6070101,0xfb840108,0xc6080101,0xfb840108,0xc6090101,
	0xfb840108,0xc60a0101,0xfb840108,0xc60b0101,0xfb840108,0xc60c0101,0xfb840108,0xc60d0101,
	0xfb840108,0xc60e0101,0xfb840108,0xc60f0101,0xfb840108,0xc6100101,0xfb840108,0xc6110101,
	0xfb840108,0xc6120101,0xfb840108,0xc6130101,0xfb840108,0xc6140101,0xfb840108,0xc6150101,
	0xfb840108,0xc6160101,0xfb840108,0xc6170101,0xfb840108,0xc6180101,0xfb840108,0xc6190101,
	0xfb840108,0xc61a0101,0xfb840108,0xc61b0101,0xfb840108,0xc61c0101,0xfb840108,0xc61d0101,
	0xfb840108,0xc61e0101,0xfb840108,0xc61f0101,0xfb840108,0xc6200101,0xfb840108,0xc6210101,
	0xfb840108,0xc6220101,0xfb840108,0xc6230101,0xfb840108,0xc6240101,0xfb840108,0xc6250101,
	0xfb840108,0xc6260101,0xfb840108,0xc6270101,0xfb840108,0xc6280101,0xfb840108,0xc6290101,
	0xfb840108,0xc62a0101,0xfb840108,0xc62b0101,0xfb840108,0xc62c0101,0xfb840108,0xc62d0101,
	0xfb840108,0xc62e0101,0xfb840108,0xc62f0101,0xfb840108,0xc6300101,0xfb840108,0xc6310101,
	0xfb840108,0xc6320101,0xfb840108,0xc6330101,0xfb840108,0xc6340101,0xfb840108,0xc6350101,
	0xfb840108,0xc6360101,0xfb840108,0xc6370101,0xfb840108,0xc6380101,0xfb840108,0xc6390101,
	0xfb840108,0xc63a0101,0xfb840108,0xc63b0101,0xfb840108,0xc63c0101,0xfb840108,0xc63d0101,
	0xfb840108,0xc63e0101,0xfb840108,0xc63f0101,0xfb840108,0xc6400101,0xfb840108,0xc6410101,
	0xfb840108,0xc6420101,0xfb840108,0xc6430101,0xfb840108,0xc6440101,0xfb840108,0xc6450101,
	0xfb840108,0xc6460101,0xfb840108,0xc6470101,0xfb840108,0xc6480101,0xfb840108,0xc6490101,
	0xfb840108,0xc64a0101,0xfb840108,0xc64b0101,0xfb840108,0xc64c0101,0xfb840108,0xc64d0101,
	0xfb840108,0xc64e0101,0xfb840108,0xc64f0101,0xfb840108,0xc6500101,0xfb840108,0xc6510101,
	0xfb840108,0xc6520101,0xfb840108,0xc6530101,0xfb840108,0xc6540101,0xfb840108,0xc6550101,
	0xfb840108,0xc6560101,0xfb840108,0xc6570101,0xfb840108,0xc6580101,0xfb840108,0xc6590101,
	0xfb840108,0xc65a0101,0xfb840108,0xc65b0101,0xfb840108,0xc65c0101,0xfb840108,0xc65d0101,
	0xfb840108,0xc65e0101,0xfb840108,0xc65f0101,0xfb840108,0xc6600101,0xfb840108,0xc6610101,
	0xfb840108,0xc6620101,0xfb840108,0xc6630101,0xfb840108,0xc6640101,0xfb840108,0xc6650101,
	0xfb840108,0xc6660101,0xfb840108,0xc6670101,0xfb840108,0xc6680101,0xfb840108,0xc6690101,
	0xfb840108,0xc66a0101,0xfb840108,0xc66b0101,0xfb840108,0xc66c0101,0xfb840108,0xc66d0101,
	0xfb840108,0xc66e0101,0xfb840108,0xc66f0101,0xfb840108,0xc6700101,0xfb840108,0xc6710101,
	0xfb840108,0xc6720101,0xfb840108,0xc6730101,0xfb840108,0xc6740101,0xfb840108,0xc6750101,
	0xfb840108,0xc6760101,0xfb840108,0xc6770101,0xfb840108,0xc6780101,0xfb840108,0xc6790101,
	0xfb840108,0xc67a0101,0xfb840108,0xc67b0101,0xfb840108,0xc67c0101,0xfb840108,0xc67d0101,
	0xfb840108,0xc67e0101,0xfb840108,0xc67f0101,0xfb840108,0xc6800101,0xfb840108,0xc6810101,
	0xfb840108,0xc6820101,0xfb840108,0xc6830101,0xfb840108,0xc6840101,0xfb840108,0xc6850101,
	0xfb840108,0xc6860101,0xfb840108,0xc6870101,0xfb840108,0xc6880101,0xfb840108,0xc6890101,
	0xfb840108,0xc68a0101,0xfb840108,0xc68b0101,0xfb840108,0xc68c0101,0xfb840108,0xc68d0101,
	0xfb840108,0xc68e0101,0xfb840108,0xc68f0101,0xfb840108,0xc6900101,0xfb840108,0xc6910101,
	0xfb840108,0xc6920101,0xfb840108,0xc6930101,0xfb840108,0xc6940101,0xfb840108,0xc6950101,
	0xfb840108,0xc6960101,0xfb840108,0xc6970101,0xfb840108,0xc6980101,0xfb840108,0xc6990101,
	0xfb840108,0xc69a0101,0xfb840108,0xc69b0101,0xfb840108,0xc69c0101,0xfb840108,0xc69d0101,
	0xfb840108,0xc69e0101,0xfb840108,0xc69f0101,0xfb840108,0xc6a00101,0xfb840108,0xc6a10101,
	0xfb840108,0xc6a20101,0xfb840108,0xc6a30101,0xfb840108,0xc6a40101,0xfb840108,0xc6a50101,
	0xfb840108,0xc6a60101,0xfb840108,0xc6a70101,0xfb840108,0xc6a80101,0xfb840108,0xc6a90101,
	0xfb840108,0xc6aa0101,0xfb840108,0xc6ab0101,0xfb840108,0xc6ac0101,0xfb840108,0xc6ad0101,
	0xfb840108,0xc6ae0101,0xfb840108,0xc6af0101,0xfb840108,0xc6b00101,0xfb840108,0xc6b10101,
	0xfb840108,0xc6b20101,0xfb840108,0xc6b30101,0xfb840108,0xc6b40101,0xfb840108,0xc6b50101,
	0xfb840108,0xc6b60101,0xfb840108,0xc6b70101,0xfb840108,0xc6b80101,0xfb840108,0xc6b90101,
	0xfb840108,0xc6ba0101,0xfb840108,0xc6bb0101,0xfb840108,0xc6bc0101,0xfb840108,0xc6bd0101,
	0xfb840108,0xc6be0101,0xfb840108,0xc6bf0101,0xfb840108,0xc6c00101,0xfb840108,0xc6c10101,
	0xfb840108,0xc6c20101,0xfb840108,0xc6c30101,0xfb840108,0xc6c40101,0xfb840108,0xc6c50101,
	0xfb840108,0xc6c60101,0xfb840108,0xc6c70101,0xfb840108,0xc6c80101,0xfb840108,0xc6c90101,
	0xfb840108,0xc6ca0101,0xfb840108,0xc6cb0101,0xfb840108,0xc6cc0101,0xfb840108,0xc6cd0101,
	0xfb840108,0xc6ce0101,0xfb840108,0xc6cf0101,0xfb840108,0xc6d00101,0xfb840108,0xc6d10101,
	0xfb840108,0xc6d20101,0xfb840108,0xc6d30101,0xfb840108,0xc6d40101,0xfb840108,0xc6d50101,
	0xfb840108,0xc6d60101,0xfb840108,0xc6d70101,0xfb840108,0xc6d80101,0xfb840108,0xc6d90101,
	0xfb840108,0xc6da0101,0xfb840108,0xc6db0101,0xfb840108,0xc6dc0101,0xfb840108,0xc6dd0101,
	0xfb840108,0xc6de0101,0xfb840108,0xc6df0101,0xfb840108,0xc6e00101,0xfb840108,0xc6e10101,
	0xfb840108,0xc6e20101,0xfb840108,0xc6e30101,0xfb840108,0xc6e40101,0xfb840108,0xc6e50101,
	0xfb840108,0xc6e60101,0xfb840108,0xc6e70101,0xfb840108,0xc6e80101,0xfb840108,0xc6e90101,
	0xfb840108,0xc6ea0101,0xfb840108,0xc6eb0101,0xfb840108,0xc6ec0101,0xfb840108,0xc6ed0101,
	0xfb840108,0xc6ee0101,0xfb840108,0xc6ef0101,0xfb840108,0xc6f00101,0xfb840108,0xc6f10101,
	0xfb840108,0xc6f20101,0xfb840108,0xc6f30101,0xfb840108,0xc6f40101,0xfb840108,0xc6f50101,
	0xfb840108,0xc6f60101,0xfb840108,0xc6f70101,0xfb840108,0xc6f80101,0xfb840108,0xc6f90101,
	0xfb840108,0xc6fa0101,0xfb840108,0xc6fb0101,0xfb840108,0xc6fc0101,0xfb840108,0xc6fd0101,
	0xfb840108,0xc6fe0101,0xfb840108,0xc6ff0101,0xfb840108,0xc7000101,0xfb840108,0xc7010101,
	0xfb840108,0xc7020101,0xfb840108,0xc7030101,0xfb840108,0xc7040101,0xfb840108,0xc7050101,
	0xfb840108,0xc7060101,0xfb840108,0xc7070101,0xfb840108,0xc7080101,0xfb840108,0xc7090101,
	0xfb840108,0xc70a0101,0xfb840108,0xc70b0101,0xfb840108,0xc70c0101,0xfb840108,0xc70d0101,
	0xfb840108,0xc70e0101,0xfb840108,0xc70f0101,0xfb840108,0xc7100101,0xfb840108,0xc7110101,
	0xfb840108,0xc7120101,0xfb840108,0xc7130101,0xfb840108,0xc7140101,0xfb840108,0xc7150101,
	0xfb840108,0xc7160101,0xfb840108,0xc7170101,0xfb840108,0xc7180101,0xfb840108,0xc7190101,
	0xfb840108,0xc71a0101,0xfb840108,0xc71b0101,0xfb840108,0xc71c0101,0xfb840108,0xc71d0101,
	0xfb840108,0xc71e0101,0xfb840108,0xc71f0101,0xfb840108,0xc7200101,0xfb840108,0xc7210101,
	0xfb840108,0xc7220101,0xfb840108,0xc7230101,0xfb840108,0xc7240101,0xfb840108,0xc7250101,
	0xfb840108,0xc7260101,0xfb840108,0xc7270101,0xfb840108,0xc7280101,0xfb840108,0xc7290101,
	0xfb840108,0xc72a0101,0xfb840108,0xc72b0101,0xfb840108,0xc72c0101,0xfb840108,0xc72d0101,
	0xfb840108,0xc72e0101,0xfb840108,0xc72f0101,0xfb840108,0xc7300101,0xfb840108,0xc7310101,
	0xfb840108,0xc7320101,0xfb840108,0xc7330101,0xfb840108,0xc7340101,0xfb840108,0xc7350101,
	0xfb840108,0xc7360101,0xfb840108,0xc7370101,0xfb840108,0xc7380101,0xfb840108,0xc7390101,
	0xfb840108,0xc73a0101,0xfb840108,0xc73b0101,0xfb840108,0xc73c0101,0xfb840108,0xc73d0101,
	0xfb840108,0xc73e0101,0xfb840108,0xc73f0101,0xfb840108,0xc7400101,0xfb840108,0xc7410101,
	0xfb840108,0xc7420101,0xfb840108,0xc7430101,0xfb840108,0xc7440101,0xfb840108,0xc7450101,
	0xfb840108,0xc7460101,0xfb840108,0xc7470101,0xfb840108,0xc7480101,0xfb840108,0xc7490101,
	0xfb840108,0xc74a0101,0xfb840108,0xc74b0101,0xfb840108,0xc74c0101,0xfb840108,0xc74d0101,
	0xfb840108,0xc74e0101,0xfb840108,0xc74f0101,0xfb840108,0xc7500101,0xfb840108,0xc7510101,
	0xfb840108,0xc7520101,0xfb840108,0xc7530101,0xfb840108,0xc7540101,0xfb840108,0xc7550101,
	0xfb840108,0xc7560101,0xfb840108,0xc7570101,0xfb840108,0xc7580101,0xfb840108,0xc7590101,
	0xfb840108,0xc75a0101,0xfb840108,0xc75b0101,0xfb840108,0xc75c0101,0xfb840108,0xc75d0101,
	0xfb840108,0xc75e0101,0xfb840108,0xc75f0101,0xfb840108,0xc7600101,0xfb840108,0xc7610101,
	0xfb840108,0xc7620101,0xfb840108,0xc7630101,0xfb840108,0xc7640101,0xfb840108,0xc7650101,
	0xfb840108,0xc7660101,0xfb840108,0xc7670101,0xfb840108,0xc7680101,0xfb840108,0xc7690101,
	0xfb840108,0xc76a0101,0xfb840108,0xc76b0101,0xfb840108,0xc76c0101,0xfb840108,0xc76d0101,
	0xfb840108,0xc76e0101,0xfb840108,0xc76f0101,0xfb840108,0xc7700101,0xfb840108,0xc7710101,
	0xfb840108,0xc7720101,0xfb840108,0xc7730101,0xfb840108,0xc7740101,0xfb840108,0xc7750101,
	0xfb840108,0xc7760101,0xfb840108,0xc7770101,0xfb840108,0xc7780101,0xfb840108,0xc7790101,
	0xfb840108,0xc77a0101,0xfb840108,0xc77b0101,0xfb840108,0xc77c0101,0xfb840108,0xc77d0101,
	0xfb840108,0xc77e0101,0xfb840108,0xc77f0101,0xfb840108,0xc7800101,0xfb840108,0xc7810101,
	0xfb840108,0xc7820101,0xfb840108,0xc7830101,0xfb840108,0xc7840101,0xfb840108,0xc7850101,
	0xfb840108,0xc7860101,0xfb840108,0xc7870101,0xfb840108,0xc7880101,0xfb840108,0xc7890101,
	0xfb840108,0xc78a0101,0xfb840108,0xc78b0101,0xfb840108,0xc78c0101,0xfb840108,0xc78d0101,
	0xfb840108,0xc78e0101,0xfb840108,0xc78f0101,0xfb840108,0xc7900101,0xfb840108,0xc7910101,
	0xfb840108,0xc7920101,0xfb840108,0xc7930101,0xfb840108,0xc7940101,0xfb840108,0xc7950101,
	0xfb840108,0xc7960101,0xfb840108,0xc7970101,0xfb840108,0xc7980101,0xfb840108,0xc7990101,
	0xfb840108,0xc79a0101,0xfb840108,0xc79b0101,0xfb840108,0xc79c0101,0xfb840108,0xc79d0101,
	0xfb840108,0xc79e0101,0xfb840108,0xc79f0101,0xfb840108,0xc7a00101,0xfb840108,0xc7a10101,
	0xfb840108,0xc7a20101,0xfb840108,0xc7a30101,0xfb840108,0xc7a40101,0xfb840108,0xc7a50101,
	0xfb840108,0xc7a60101,0xfb840108,0xc7a70101,0xfb840108,0xc7a80101,0xfb840108,0xc7a90101,
	0xfb840108,0xc7aa0101,0xfb840108,0xc7ab0101,0xfb840108,0xc7ac0101,0xfb840108,0xc7ad0101,
	0xfb840108,0xc7ae0101,0xfb840108,0xc7af0101,0xfb840108,0xc7b00101,0xfb840108,0xc7b10101,
	0xfb840108,0xc7b20101,0xfb840108,0xc7b30101,0xfb840108,0xc7b40101,0xfb840108,0xc7b50101,
	0xfb840108,0xc7b60101,0xfb840108,0xc7b70101,0xfb840108,0xc7b80101,0xfb840108,0xc7b90101,
	0xfb840108,0xc7ba0101,0xfb840108,0xc7bb0101,0xfb840108,0xc7bc0101,0xfb840108,0xc7bd0101,
	0xfb840108,0xc7be0101,0xfb840108,0xc7bf0101,0xfb840108,0xc7c00101,0xfb840108,0xc7c10101,
	0xfb840108,0xc7c20101,0xfb840108,0xc7c30101,0xfb840108,0xc7c40101,0xfb840108,0xc7c50101,
	0xfb840108,0xc7c60101,0xfb840108,0xc7c70101,0xfb840108,0xc7c80101,0xfb840108,0xc7c90101,
	0xfb840108,0xc7ca0101,0xfb840108,0xc7cb0101,0xfb840108,0xc7cc0101,0xfb840108,0xc7cd0101,
	0xfb840108,0xc7ce0101,0xfb840108,0xc7cf0101,0xfb840108,0xc7d00101,0xfb840108,0xc7d10101,
	0xfb840108,0xc7d20101,0xfb840108,0xc7d30101,0xfb840108,0xc7d40101,0xfb840108,0xc7d50101,
	0xfb840108,0xc7d60101,0xfb840108,0xc7d70101,0xfb840108,0xc7d80101,0xfb840108,0xc7d90101,
	0xfb840108,0xc7da0101,0xfb840108,0xc7db0101,0xfb840108,0xc7dc0101,0xfb840108,0xc7dd0101,
	0xfb840108,0xc7de0101,0xfb840108,0xc7df0101,0xfb840108,0xc7e00101,0xfb840108,0xc7e10101,
	0xfb840108,0xc7e20101,0xfb840108,0xc7e30101,0xfb840108,0xc7e40101,0xfb840108,0xc7e50101,
	0xfb840108,0xc7e60101,0xfb840108,0xc7e70101,0xfb840108,0xc7e80101,0xfb840108,0xc7e90101,
	0xfb840108,0xc7ea0101,0xfb840108,0xc7eb0101,0xfb840108,0xc7ec0101,0xfb840108,0xc7ed0101,
	0xfb840108,0xc7ee0101,0xfb840108,0xc7ef0101,0xfb840108,0xc7f00101,0xfb840108,0xc7f10101,
	0xfb840108,0xc7f20101,0xfb840108,0xc7f30101,0xfb840108,0xc7f40101,0xfb840108,0xc7f50101,
	0xfb840108,0xc7f60101,0xfb840108,0xc7f70101,0xfb840108,0xc7f80101,0xfb840108,0xc7f90101,
	0xfb840108,0xc7fa0101,0xfb840108,0xc7fb0101,0xfb840108,0xc7fc0101,0xfb840108,0xc7fd0101,
	0xfb840108,0xc7fe0101,0xfb840108,0xc7ff0101,0xfb840108,0xc8000101,0xfb840108,0xc8010101,
	0xfb840108,0xc8020101,0xfb840108,0xc8030101,0xfb840108,0xc8040101,0xfb840108,0xc8050101,
	0xfb840108,0xc8060101,0xfb840108,0xc8070101,0xfb840108,0xc8080101,0xfb840108,0xc8090101,
	0xfb840108,0xc80a0101,0xfb840108,0xc80b0101,0xfb840108,0xc80c0101,0xfb840108,0xc80d0101,
	0xfb840108,0xc80e0101,0xfb840108,0xc80f0101,0xfb840108,0xc8100101,0xfb840108,0xc8110101,
	0xfb840108,0xc8120101,0xfb840108,0xc8130101,0xfb840108,0xc8140101,0xfb840108,0xc8150101,
	0xfb840108,0xc8160101,0xfb840108,0xc8170101,0xfb840108,0xc8180101,0xfb840108,0xc8190101,
	0xfb840108,0xc81a0101,0xfb840108,0xc81b0101,0xfb840108,0xc81c0101,0xfb840108,0xc81d0101,
	0xfb840108,0xc81e0101,0xfb840108,0xc81f0101,0xfb840108,0xc8200101,0xfb840108,0xc8210101,
	0xfb840108,0xc8220101,0xfb840108,0xc8230101,0xfb840108,0xc8240101,0xfb840108,0xc8250101,
	0xfb840108,0xc8260101,0xfb840108,0xc8270101,0xfb840108,0xc8280101,0xfb840108,0xc8290101,
	0xfb840108,0xc82a0101,0xfb840108,0xc82b0101,0xfb840108,0xc82c0101,0xfb840108,0xc82d0101,
	0xfb840108,0xc82e0101,0xfb840108,0xc82f0101,0xfb840108,0xc8300101,0xfb840108,0xc8310101,
	0xfb840108,0xc8320101,0xfb840108,0xc8330101,0xfb840108,0xc8340101,0xfb840108,0xc8350101,
	0xfb840108,0xc8360101,0xfb840108,0xc8370101,0xfb840108,0xc8380101,0xfb840108,0xc8390101,
	0xfb840108,0xc83a0101,0xfb840108,0xc83b0101,0xfb840108,0xc83c0101,0xfb840108,0xc83d0101,
	0xfb840108,0xc83e0101,0xfb840108,0xc83f0101,0xfb840108,0xc8400101,0xfb840108,0xc8410101,
	0xfb840108,0xc8420101,0xfb840108,0xc8430101,0xfb840108,0xc8440101,0xfb840108,0xc8450101,
	0xfb840108,0xc8460101,0xfb840108,0xc8470101,0xfb840108,0xc8480101,0xfb840108,0xc8490101,
	0xfb840108,0xc84a0101,0xfb840108,0xc84b0101,0xfb840108,0xc84c0101,0xfb840108,0xc84d0101,
	0xfb840108,0xc84e0101,0xfb840108,0xc84f0101,0xfb840108,0xc8500101,0xfb840108,0xc8510101,
	0xfb840108,0xc8520101,0xfb840108,0xc8530101,0xfb840108,0xc8540101,0xfb840108,0xc8550101,
	0xfb840108,0xc8560101,0xfb840108,0xc8570101,0xfb840108,0xc8580101,0xfb840108,0xc8590101,
	0xfb840108,0xc85a0101,0xfb840108,0xc85b0101,0xfb840108,0xc85c0101,0xfb840108,0xc85d0101,
	0xfb840108,0xc85e0101,0xfb840108,0xc85f0101,0xfb840108,0xc8600101,0xfb840108,0xc8610101,
	0xfb840108,0xc8620101,0xfb840108,0xc8630101,0xfb840108,0xc8640101,0xfb840108,0xc8650101,
	0xfb840108,0xc8660101,0xfb840108,0xc8670101,0xfb840108,0xc8680101,0xfb840108,0xc8690101,
	0xfb840108,0xc86a0101,0xfb840108,0xc86b0101,0xfb840108,0xc86c0101,0xfb840108,0xc86d0101,
	0xfb840108,0xc86e0101,0xfb840108,0xc86f0101,0xfb840108,0xc8700101,0xfb840108,0xc8710101,
	0xfb840108,0xc8720101,0xfb840108,0xc8730101,0xfb840108,0xc8740101,0xfb840108,0xc8750101,
	0xfb840108,0xc8760101,0xfb840108,0xc8770101,0xfb840108,0xc8780101,0xfb840108,0xc8790101,
	0xfb840108,0xc87a0101,0xfb840108,0xc87b0101,0xfb840108,0xc87c0101,0xfb840108,0xc87d0101,
	0xfb840108,0xc87e0101,0xfb840108,0xc87f0101,0xfb840108,0xc8800101,0xfb840108,0xc8810101,
	0xfb840108,0xc8820101,0xfb840108,0xc8830101,0xfb840108,0xc8840101,0xfb840108,0xc8850101,
	0xfb840108,0xc8860101,0xfb840108,0xc8870101,0xfb840108,0xc8880101,0xfb840108,0xc8890101,
	0xfb840108,0xc88a0101,0xfb840108,0xc88b0101,0xfb840108,0xc88c0101,0xfb840108,0xc88d0101,
	0xfb840108,0xc88e0101,0xfb840108,0xc88f0101,0xfb840108,0xc8900101,0xfb840108,0xc8910101,
	0xfb840108,0xc8920101,0xfb840108,0xc8930101,0xfb840108,0xc8940101,0xfb840108,0xc8950101,
	0xfb840108,0xc8960101,0xfb840108,0xc8970101,0xfb840108,0xc8980101,0xfb840108,0xc8990101,
	0xfb840108,0xc89a0101,0xfb840108,0xc89b0101,0xfb840108,0xc89c0101,0xfb840108,0xc89d0101,
	0xfb840108,0xc89e0101,0xfb840108,0xc89f0101,0xfb840108,0xc8a00101,0xfb840108,0xc8a10101,
	0xfb840108,0xc8a20101,0xfb840108,0xc8a30101,0xfb840108,0xc8a40101,0xfb840108,0xc8a50101,
	0xfb840108,0xc8a60101,0xfb840108,0xc8a70101,0xfb840108,0xc8a80101,0xfb840108,0xc8a90101,
	0xfb840108,0xc8aa0101,0xfb840108,0xc8ab0101,0xfb840108,0xc8ac0101,0xfb840108,0xc8ad0101,
	0xfb840108,0xc8ae0101,0xfb840108,0xc8af0101,0xfb840108,0xc8b00101,0xfb840108,0xc8b10101,
	0xfb840108,0xc8b20101,0xfb840108,0xc8b30101,0xfb840108,0xc8b40101,0xfb840108,0xc8b50101,
	0xfb840108,0xc8b60101,0xfb840108,0xc8b70101,0xfb840108,0xc8b80101,0xfb840108,0xc8b90101,
	0xfb840108,0xc8ba0101,0xfb840108,0xc8bb0101,0xfb840108,0xc8bc0101,0xfb840108,0xc8bd0101,
	0xfb840108,0xc8be0101,0xfb840108,0xc8bf0101,0xfb840108,0xc8c00101,0xfb840108,0xc8c10101,
	0xfb840108,0xc8c20101,0xfb840108,0xc8c30101,0xfb840108,0xc8c40101,0xfb840108,0xc8c50101,
	0xfb840108,0xc8c60101,0xfb840108,0xc8c70101,0xfb840108,0xc8c80101,0xfb840108,0xc8c90101,
	0xfb840108,0xc8ca0101,0xfb840108,0xc8cb0101,0xfb840108,0xc8cc0101,0xfb840108,0xc8cd0101,
	0xfb840108,0xc8ce0101,0xfb840108,0xc8cf0101,0xfb840108,0xc8d00101,0xfb840108,0xc8d10101,
	0xfb840108,0xc8d20101,0xfb840108,0xc8d30101,0xfb840108,0xc8d40101,0xfb840108,0xc8d50101,
	0xfb840108,0xc8d60101,0xfb840108,0xc8d70101,0xfb840108,0xc8d80101,0xfb840108,0xc8d90101,
	0xfb840108,0xc8da0101,0xfb840108,0xc8db0101,0xfb840108,0xc8dc0101,0xfb840108,0xc8dd0101,
	0xfb840108,0xc8de0101,0xfb840108,0xc8df0101,0xfb840108,0xc8e00101,0xfb840108,0xc8e10101,
	0xfb840108,0xc8e20101,0xfb840108,0xc8e30101,0xfb840108,0xc8e40101,0xfb840108,0xc8e50101,
	0xfb840108,0xc8e60101,0xfb840108,0xc8e70101,0xfb840108,0xc8e80101,0xfb840108,0xc8e90101,
	0xfb840108,0xc8ea0101,0xfb840108,0xc8eb0101,0xfb840108,0xc8ec0101,0xfb840108,0xc8ed0101,
	0xfb840108,0xc8ee0101,0xfb840108,0xc8ef0101,0xfb840108,0xc8f00101,0xfb840108,0xc8f10101,
	0xfb840108,0xc8f20101,0xfb840108,0xc8f30101,0xfb840108,0xc8f40101,0xfb840108,0xc8f50101,
	0xfb840108,0xc8f60101,0xfb840108,0xc8f70101,0xfb840108,0xc8f80101,0xfb840108,0xc8f90101,
	0xfb840108,0xc8fa0101,0xfb840108,0xc8fb0101,0xfb840108,0xc8fc0101,0xfb840108,0xc8fd0101,
	0xfb840108,0xc8fe0101,0xfb840108,0xc8ff0101,0xfb840108,0xc9000101,0xfb840108,0xc9010101,
	0xfb840108,0xc9020101,0xfb840108,0xc9030101,0xfb840108,0xc9040101,0xfb840108,0xc9050101,
	0xfb840108,0xc9060101,0xfb840108,0xc9070101,0xfb840108,0xc9080101,0xfb840108,0xc9090101,
	0xfb840108,0xc90a0101,0xfb840108,0xc90b0101,0xfb840108,0xc90c0101,0xfb840108,0xc90d0101,
	0xfb840108,0xc90e0101,0xfb840108,0xc90f0101,0xfb840108,0xc9100101,0xfb840108,0xc9110101,
	0xfb840108,0xc9120101,0xfb840108,0xc9130101,0xfb840108,0xc9140101,0xfb840108,0xc9150101,
	0xfb840108,0xc9160101,0xfb840108,0xc9170101,0xfb840108,0xc9180101,0xfb840108,0xc9190101,
	0xfb840108,0xc91a0101,0xfb840108,0xc91b0101,0xfb840108,0xc91c0101,0xfb840108,0xc91d0101,
	0xfb840108,0xc91e0101,0xfb840108,0xc91f0101,0xfb840108,0xc9200101,0xfb840108,0xc9210101,
	0xfb840108,0xc9220101,0xfb840108,0xc9230101,0xfb840108,0xc9240101,0xfb840108,0xc9250101,
	0xfb840108,0xc9260101,0xfb840108,0xc9270101,0xfb840108,0xc9280101,0xfb840108,0xc9290101,
	0xfb840108,0xc92a0101,0xfb840108,0xc92b0101,0xfb840108,0xc92c0101,0xfb840108,0xc92d0101,
	0xfb840108,0xc92e0101,0xfb840108,0xc92f0101,0xfb840108,0xc9300101,0xfb840108,0xc9310101,
	0xfb840108,0xc9320101,0xfb840108,0xc9330101,0xfb840108,0xc9340101,0xfb840108,0xc9350101,
	0xfb840108,0xc9360101,0xfb840108,0xc9370101,0xfb840108,0xc9380101,0xfb840108,0xc9390101,
	0xfb840108,0xc93a0101,0xfb840108,0xc93b0101,0xfb840108,0xc93c0101,0xfb840108,0xc93d0101,
	0xfb840108,0xc93e0101,0xfb840108,0xc93f0101,0xfb840108,0xc9400101,0xfb840108,0xc9410101,
	0xfb840108,0xc9420101,0xfb840108,0xc9430101,0xfb840108,0xc9440101,0xfb840108,0xc9450101,
	0xfb840108,0xc9460101,0xfb840108,0xc9470101,0xfb840108,0xc9480101,0xfb840108,0xc9490101,
	0xfb840108,0xc94a0101,0xfb840108,0xc94b0101,0xfb840108,0xc94c0101,0xfb840108,0xc94d0101,
	0xfb840108,0xc94e0101,0xfb840108,0xc94f0101,0xfb840108,0xc9500101,0xfb840108,0xc9510101,
	0xfb840108,0xc9520101,0xfb840108,0xc9530101,0xfb840108,0xc9540101,0xfb840108,0xc9550101,
	0xfb840108,0xc9560101,0xfb840108,0xc9570101,0xfb840108,0xc9580101,0xfb840108,0xc9590101,
	0xfb840108,0xc95a0101,0xfb840108,0xc95b0101,0xfb840108,0xc95c0101,0xfb840108,0xc95d0101,
	0xfb840108,0xc95e0101,0xfb840108,0xc95f0101,0xfb840108,0xc9600101,0xfb840108,0xc9610101,
	0xfb840108,0xc9620101,0xfb840108,0xc9630101,0xfb840108,0xc9640101,0xfb840108,0xc9650101,
	0xfb840108,0xc9660101,0xfb840108,0xc9670101,0xfb840108,0xc9680101,0xfb840108,0xc9690101,
	0xfb840108,0xc96a0101,0xfb840108,0xc96b0101,0xfb840108,0xc96c0101,0xfb840108,0xc96d0101,
	0xfb840108,0xc96e0101,0xfb840108,0xc96f0101,0xfb840108,0xc9700101,0xfb840108,0xc9710101,
	0xfb840108,0xc9720101,0xfb840108,0xc9730101,0xfb840108,0xc9740101,0xfb840108,0xc9750101,
	0xfb840108,0xc9760101,0xfb840108,0xc9770101,0xfb840108,0xc9780101,0xfb840108,0xc9790101,
	0xfb840108,0xc97a0101,0xfb840108,0xc97b0101,0xfb840108,0xc97c0101,0xfb840108,0xc97d0101,
	0xfb840108,0xc97e0101,0xfb840108,0xc97f0101,0xfb840108,0xc9800101,0xfb840108,0xc9810101,
	0xfb840108,0xc9820101,0xfb840108,0xc9830101,0xfb840108,0xc9840101,0xfb840108,0xc9850101,
	0xfb840108,0xc9860101,0xfb840108,0xc9870101,0xfb840108,0xc9880101,0xfb840108,0xc9890101,
	0xfb840108,0xc98a0101,0xfb840108,0xc98b0101,0xfb840108,0xc98c0101,0xfb840108,0xc98d0101,
	0xfb840108,0xc98e0101,0xfb840108,0xc98f0101,0xfb840108,0xc9900101,0xfb840108,0xc9910101,
	0xfb840108,0xc9920101,0xfb840108,0xc9930101,0xfb840108,0xc9940101,0xfb840108,0xc9950101,
	0xfb840108,0xc9960101,0xfb840108,0xc9970101,0xfb840108,0xc9980101,0xfb840108,0xc9990101,
	0xfb840108,0xc99a0101,0xfb840108,0xc99b0101,0xfb840108,0xc99c0101,0xfb840108,0xc99d0101,
	0xfb840108,0xc99e0101,0xfb840108,0xc99f0101,0xfb840108,0xc9a00101,0xfb840108,0xc9a10101,
	0xfb840108,0xc9a20101,0xfb840108,0xc9a30101,0xfb840108,0xc9a40101,0xfb840108,0xc9a50101,
	0xfb840108,0xc9a60101,0xfb840108,0xc9a70101,0xfb840108,0xc9a80101,0xfb840108,0xc9a90101,
	0xfb840108,0xc9aa0101,0xfb840108,0xc9ab0101,0xfb840108,0xc9ac0101,0xfb840108,0xc9ad0101,
	0xfb840108,0xc9ae0101,0xfb840108,0xc9af0101,0xfb840108,0xc9b00101,0xfb840108,0xc9b10101,
	0xfb840108,0xc9b20101,0xfb840108,0xc9b30101,0xfb840108,0xc9b40101,0xfb840108,0xc9b50101,
	0xfb840108,0xc9b60101,0xfb840108,0xc9b70101,0xfb840108,0xc9b80101,0xfb840108,0xc9b90101,
	0xfb840108,0xc9ba0101,0xfb840108,0xc9bb0101,0xfb840108,0xc9bc0101,0xfb840108,0xc9bd0101,
	0xfb840108,0xc9be0101,0xfb840108,0xc9bf0101,0xfb840108,0xc9c00101,0xfb840108,0xc9c10101,
	0xfb840108,0xc9c20101,0xfb840108,0xc9c30101,0xfb840108,0xc9c40101,0xfb840108,0xc9c50101,
	0xfb840108,0xc9c60101,0xfb840108,0xc9c70101,0xfb840108,0xc9c80101,0xfb840108,0xc9c90101,
	0xfb840108,0xc9ca0101,0xfb840108,0xc9cb0101,0xfb840108,0xc9cc0101,0xfb840108,0xc9cd0101,
	0xfb840108,0xc9ce0101,0xfb840108,0xc9cf0101,0xfb840108,0xc9d00101,0xfb840108,0xc9d10101,
	0xfb840108,0xc9d20101,0xfb840108,0xc9d30101,0xfb840108,0xc9d40101,0xfb840108,0xc9d50101,
	0xfb840108,0xc9d60101,0xfb840108,0xc9d70101,0xfb840108,0xc9d80101,0xfb840108,0xc9d90101,
	0xfb840108,0xc9da0101,0xfb840108,0xc9db0101,0xfb840108,0xc9dc0101,0xfb840108,0xc9dd0101,
	0xfb840108,0xc9de0101,0xfb840108,0xc9df0101,0xfb840108,0xc9e00101,0xfb840108,0xc9e10101,
	0xfb840108,0xc9e20101,0xfb840108,0xc9e30101,0xfb840108,0xc9e40101,0xfb840108,0xc9e50101,
	0xfb840108,0xc9e60101,0xfb840108,0xc9e70101,0xfb840108,0xc9e80101,0xfb840108,0xc9e90101,
	0xfb840108,0xc9ea0101,0xfb840108,0xc9eb0101,0xfb840108,0xc9ec0101,0xfb840108,0xc9ed0101,
	0xfb840108,0xc9ee0101,0xfb840108,0xc9ef0101,0xfb840108,0xc9f00101,0xfb840108,0xc9f10101,
	0xfb840108,0xc9f20101,0xfb840108,0xc9f30101,0xfb840108,0xc9f40101,0xfb840108,0xc9f50101,
	0xfb840108,0xc9f60101,0xfb840108,0xc9f70101,0xfb840108,0xc9f80101,0xfb840108,0xc9f90101,
	0xfb840108,0xc9fa0101,0xfb840108,0xc9fb0101,0xfb840108,0xc9fc0101,0xfb840108,0xc9fd0101,
	0xfb840108,0xc9fe0101,0xfb840108,0xc9ff0101,0xfb840108,0xca000101,0xfb840108,0xca010101,
	0xfb840108,0xca020101,0xfb840108,0xca030101,0xfb840108,0xca040101,0xfb840108,0xca050101,
	0xfb840108,0xca060101,0xfb840108,0xca070101,0xfb840108,0xca080101,0xfb840108,0xca090101,
	0xfb840108,0xca0a0101,0xfb840108,0xca0b0101,0xfb840108,0xca0c0101,0xfb840108,0xca0d0101,
	0xfb840108,0xca0e0101,0xfb840108,0xca0f0101,0xfb840108,0xca100101,0xfb840108,0xca110101,
	0xfb840108,0xca120101,0xfb840108,0xca130101,0xfb840108,0xca140101,0xfb840108,0xca150101,
	0xfb840108,0xca160101,0xfb840108,0xca170101,0xfb840108,0xca180101,0xfb840108,0xca190101,
	0xfb840108,0xca1a0101,0xfb840108,0xca1b0101,0xfb840108,0xca1c0101,0xfb840108,0xca1d0101,
	0xfb840108,0xca1e0101,0xfb840108,0xca1f0101,0xfb840108,0xca200101,0xfb840108,0xca210101,
	0xfb840108,0xca220101,0xfb840108,0xca230101,0xfb840108,0xca240101,0xfb840108,0xca250101,
	0xfb840108,0xca260101,0xfb840108,0xca270101,0xfb840108,0xca280101,0xfb840108,0xca290101,
	0xfb840108,0xca2a0101,0xfb840108,0xca2b0101,0xfb840108,0xca2c0101,0xfb840108,0xca2d0101,
	0xfb840108,0xca2e0101,0xfb840108,0xca2f0101,0xfb840108,0xca300101,0xfb840108,0xca310101,
	0xfb840108,0xca320101,0xfb840108,0xca330101,0xfb840108,0xca340101,0xfb840108,0xca350101,
	0xfb840108,0xca360101,0xfb840108,0xca370101,0xfb840108,0xca380101,0xfb840108,0xca390101,
	0xfb840108,0xca3a0101,0xfb840108,0xca3b0101,0xfb840108,0xca3c0101,0xfb840108,0xca3d0101,
	0xfb840108,0xca3e0101,0xfb840108,0xca3f0101,0xfb840108,0xca400101,0xfb840108,0xca410101,
	0xfb840108,0xca420101,0xfb840108,0xca430101,0xfb840108,0xca440101,0xfb840108,0xca450101,
	0xfb840108,0xca460101,0xfb840108,0xca470101,0xfb840108,0xca480101,0xfb840108,0xca490101,
	0xfb840108,0xca4a0101,0xfb840108,0xca4b0101,0xfb840108,0xca4c0101,0xfb840108,0xca4d0101,
	0xfb840108,0xca4e0101,0xfb840108,0xca4f0101,0xfb840108,0xca500101,0xfb840108,0xca510101,
	0xfb840108,0xca520101,0xfb840108,0xca530101,0xfb840108,0xca540101,0xfb840108,0xca550101,
	0xfb840108,0xca560101,0xfb840108,0xca570101,0xfb840108,0xca580101,0xfb840108,0xca590101,
	0xfb840108,0xca5a0101,0xfb840108,0xca5b0101,0xfb840108,0xca5c0101,0xfb840108,0xca5d0101,
	0xfb840108,0xca5e0101,0xfb840108,0xca5f0101,0xfb840108,0xca600101,0xfb840108,0xca610101,
	0xfb840108,0xca620101,0xfb840108,0xca630101,0xfb840108,0xca640101,0xfb840108,0xca650101,
	0xfb840108,0xca660101,0xfb840108,0xca670101,0xfb840108,0xca680101,0xfb840108,0xca690101,
	0xfb840108,0xca6a0101,0xfb840108,0xca6b0101,0xfb840108,0xca6c0101,0xfb840108,0xca6d0101,
	0xfb840108,0xca6e0101,0xfb840108,0xca6f0101,0xfb840108,0xca700101,0xfb840108,0xca710101,
	0xfb840108,0xca720101,0xfb840108,0xca730101,0xfb840108,0xca740101,0xfb840108,0xca750101,
	0xfb840108,0xca760101,0xfb840108,0xca770101,0xfb840108,0xca780101,0xfb840108,0xca790101,
	0xfb840108,0xca7a0101,0xfb840108,0xca7b0101,0xfb840108,0xca7c0101,0xfb840108,0xca7d0101,
	0xfb840108,0xca7e0101,0xfb840108,0xca7f0101,0xfb840108,0xca800101,0xfb840108,0xca810101,
	0xfb840108,0xca820101,0xfb840108,0xca830101,0xfb840108,0xca840101,0xfb840108,0xca850101,
	0xfb840108,0xca860101,0xfb840108,0xca870101,0xfb840108,0xca880101,0xfb840108,0xca890101,
	0xfb840108,0xca8a0101,0xfb840108,0xca8b0101,0xfb840108,0xca8c0101,0xfb840108,0xca8d0101,
	0xfb840108,0xca8e0101,0xfb840108,0xca8f0101,0xfb840108,0xca900101,0xfb840108,0xca910101,
	0xfb840108,0xca920101,0xfb840108,0xca930101,0xfb840108,0xca940101,0xfb840108,0xca950101,
	0xfb840108,0xca960101,0xfb840108,0xca970101,0xfb840108,0xca980101,0xfb840108,0xca990101,
	0xfb840108,0xca9a0101,0xfb840108,0xca9b0101,0xfb840108,0xca9c0101,0xfb840108,0xca9d0101,
	0xfb840108,0xca9e0101,0xfb840108,0xca9f0101,0xfb840108,0xcaa00101,0xfb840108,0xcaa10101,
	0xfb840108,0xcaa20101,0xfb840108,0xcaa30101,0xfb840108,0xcaa40101,0xfb840108,0xcaa50101,
	0xfb840108,0xcaa60101,0xfb840108,0xcaa70101,0xfb840108,0xcaa80101,0xfb840108,0xcaa90101,
	0xfb840108,0xcaaa0101,0xfb840108,0xcaab0101,0xfb840108,0xcaac0101,0xfb840108,0xcaad0101,
	0xfb840108,0xcaae0101,0xfb840108,0xcaaf0101,0xfb840108,0xcab00101,0xfb840108,0xcab10101,
	0xfb840108,0xcab20101,0xfb840108,0xcab30101,0xfb840108,0xcab40101,0xfb840108,0xcab50101,
	0xfb840108,0xcab60101,0xfb840108,0xcab70101,0xfb840108,0xcab80101,0xfb840108,0xcab90101,
	0xfb840108,0xcaba0101,0xfb840108,0xcabb0101,0xfb840108,0xcabc0101,0xfb840108,0xcabd0101,
	0xfb840108,0xcabe0101,0xfb840108,0xcabf0101,0xfb840108,0xcac00101,0xfb840108,0xcac10101,
	0xfb840108,0xcac20101,0xfb840108,0xcac30101,0xfb840108,0xcac40101,0xfb840108,0xcac50101,
	0xfb840108,0xcac60101,0xfb840108,0xcac70101,0xfb840108,0xcac80101,0xfb840108,0xcac90101,
	0xfb840108,0xcaca0101,0xfb840108,0xcacb0101,0xfb840108,0xcacc0101,0xfb840108,0xcacd0101,
	0xfb840108,0xcace0101,0xfb840108,0xcacf0101,0xfb840108,0xcad00101,0xfb840108,0xcad10101,
	0xfb840108,0xcad20101,0xfb840108,0xcad30101,0xfb840108,0xcad40101,0xfb840108,0xcad50101,
	0xfb840108,0xcad60101,0xfb840108,0xcad70101,0xfb840108,0xcad80101,0xfb840108,0xcad90101,
	0xfb840108,0xcada0101,0xfb840108,0xcadb0101,0xfb840108,0xcadc0101,0xfb840108,0xcadd0101,
	0xfb840108,0xcade0101,0xfb840108,0xcadf0101,0xfb840108,0xcae00101,0xfb840108,0xcae10101,
	0xfb840108,0xcae20101,0xfb840108,0xcae30101,0xfb840108,0xcae40101,0xfb840108,0xcae50101,
	0xfb840108,0xcae60101,0xfb840108,0xcae70101,0xfb840108,0xcae80101,0xfb840108,0xcae90101,
	0xfb840108,0xcaea0101,0xfb840108,0xcaeb0101,0xfb840108,0xcaec0101,0xfb840108,0xcaed0101,
	0xfb840108,0xcaee0101,0xfb840108,0xcaef0101,0xfb840108,0xcaf00101,0xfb840108,0xcaf10101,
	0xfb840108,0xcaf20101,0xfb840108,0xcaf30101,0xfb840108,0xcaf40101,0xfb840108,0xcaf50101,
	0xfb840108,0xcaf60101,0xfb840108,0xcaf70101,0xfb840108,0xcaf80101,0xfb840108,0xcaf90101,
	0xfb840108,0xcafa0101,0xfb840108,0xcafb0101,0xfb840108,0xcafc0101,0xfb840108,0xcafd0101,
	0xfb840108,0xcafe0101,0xfb840108,0xcaff0101,0xfb840108,0xcb000101,0xfb840108,0xcb010101,
	0xfb840108,0xcb020101,0xfb840108,0xcb030101,0xfb840108,0xcb040101,0xfb840108,0xcb050101,
	0xfb840108,0xcb060101,0xfb840108,0xcb070101,0xfb840108,0xcb080101,0xfb840108,0xcb090101,
	0xfb840108,0xcb0a0101,0xfb840108,0xcb0b0101,0xfb840108,0xcb0c0101,0xfb840108,0xcb0d0101,
	0xfb840108,0xcb0e0101,0xfb840108,0xcb0f0101,0xfb840108,0xcb100101,0xfb840108,0xcb110101,
	0xfb840108,0xcb120101,0xfb840108,0xcb130101,0xfb840108,0xcb140101,0xfb840108,0xcb150101,
	0xfb840108,0xcb160101,0xfb840108,0xcb170101,0xfb840108,0xcb180101,0xfb840108,0xcb190101,
	0xfb840108,0xcb1a0101,0xfb840108,0xcb1b0101,0xfb840108,0xcb1c0101,0xfb840108,0xcb1d0101,
	0xfb840108,0xcb1e0101,0xfb840108,0xcb1f0101,0xfb840108,0xcb200101,0xfb840108,0xcb210101,
	0xfb840108,0xcb220101,0xfb840108,0xcb230101,0xfb840108,0xcb240101,0xfb840108,0xcb250101,
	0xfb840108,0xcb260101,0xfb840108,0xcb270101,0xfb840108,0xcb280101,0xfb840108,0xcb290101,
	0xfb840108,0xcb2a0101,0xfb840108,0xcb2b0101,0xfb840108,0xcb2c0101,0xfb840108,0xcb2d0101,
	0xfb840108,0xcb2e0101,0xfb840108,0xcb2f0101,0xfb840108,0xcb300101,0xfb840108,0xcb310101,
	0xfb840108,0xcb320101,0xfb840108,0xcb330101,0xfb840108,0xcb340101,0xfb840108,0xcb350101,
	0xfb840108,0xcb360101,0xfb840108,0xcb370101,0xfb840108,0xcb380101,0xfb840108,0xcb390101,
	0xfb840108,0xcb3a0101,0xfb840108,0xcb3b0101,0xfb840108,0xcb3c0101,0xfb840108,0xcb3d0101,
	0xfb840108,0xcb3e0101,0xfb840108,0xcb3f0101,0xfb840108,0xcb400101,0xfb840108,0xcb410101,
	0xfb840108,0xcb420101,0xfb840108,0xcb430101,0xfb840108,0xcb440101,0xfb840108,0xcb450101,
	0xfb840108,0xcb460101,0xfb840108,0xcb470101,0xfb840108,0xcb480101,0xfb840108,0xcb490101,
	0xfb840108,0xcb4a0101,0xfb840108,0xcb4b0101,0xfb840108,0xcb4c0101,0xfb840108,0xcb4d0101,
	0xfb840108,0xcb4e0101,0xfb840108,0xcb4f0101,0xfb840108,0xcb500101,0xfb840108,0xcb510101,
	0xfb840108,0xcb520101,0xfb840108,0xcb530101,0xfb840108,0xcb540101,0xfb840108,0xcb550101,
	0xfb840108,0xcb560101,0xfb840108,0xcb570101,0xfb840108,0xcb580101,0xfb840108,0xcb590101,
	0xfb840108,0xcb5a0101,0xfb840108,0xcb5b0101,0xfb840108,0xcb5c0101,0xfb840108,0xcb5d0101,
	0xfb840108,0xcb5e0101,0xfb840108,0xcb5f0101,0xfb840108,0xcb600101,0xfb840108,0xcb610101,
	0xfb840108,0xcb620101,0xfb840108,0xcb630101,0xfb840108,0xcb640101,0xfb840108,0xcb650101,
	0xfb840108,0xcb660101,0xfb840108,0xcb670101,0xfb840108,0xcb680101,0xfb840108,0xcb690101,
	0xfb840108,0xcb6a0101,0xfb840108,0xcb6b0101,0xfb840108,0xcb6c0101,0xfb840108,0xcb6d0101,
	0xfb840108,0xcb6e0101,0xfb840108,0xcb6f0101,0xfb840108,0xcb700101,0xfb840108,0xcb710101,
	0xfb840108,0xcb720101,0xfb840108,0xcb730101,0xfb840108,0xcb740101,0xfb840108,0xcb750101,
	0xfb840108,0xcb760101,0xfb840108,0xcb770101,0xfb840108,0xcb780101,0xfb840108,0xcb790101,
	0xfb840108,0xcb7a0101,0xfb840108,0xcb7b0101,0xfb840108,0xcb7c0101,0xfb840108,0xcb7d0101,
	0xfb840108,0xcb7e0101,0xfb840108,0xcb7f0101,0xfb840108,0xcb800101,0xfb840108,0xcb810101,
	0xfb840108,0xcb820101,0xfb840108,0xcb830101,0xfb840108,0xcb840101,0xfb840108,0xcb850101,
	0xfb840108,0xcb860101,0xfb840108,0xcb870101,0xfb840108,0xcb880101,0xfb840108,0xcb890101,
	0xfb840108,0xcb8a0101,0xfb840108,0xcb8b0101,0xfb840108,0xcb8c0101,0xfb840108,0xcb8d0101,
	0xfb840108,0xcb8e0101,0xfb840108,0xcb8f0101,0xfb840108,0xcb900101,0xfb840108,0xcb910101,
	0xfb840108,0xcb920101,0xfb840108,0xcb930101,0xfb840108,0xcb940101,0xfb840108,0xcb950101,
	0xfb840108,0xcb960101,0xfb840108,0xcb970101,0xfb840108,0xcb980101,0xfb840108,0xcb990101,
	0xfb840108,0xcb9a0101,0xfb840108,0xcb9b0101,0xfb840108,0xcb9c0101,0xfb840108,0xcb9d0101,
	0xfb840108,0xcb9e0101,0xfb840108,0xcb9f0101,0xfb840108,0xcba00101,0xfb840108,0xcba10101,
	0xfb840108,0xcba20101,0xfb840108,0xcba30101,0xfb840108,0xcba40101,0xfb840108,0xcba50101,
	0xfb840108,0xcba60101,0xfb840108,0xcba70101,0xfb840108,0xcba80101,0xfb840108,0xcba90101,
	0xfb840108,0xcbaa0101,0xfb840108,0xcbab0101,0xfb840108,0xcbac0101,0xfb840108,0xcbad0101,
	0xfb840108,0xcbae0101,0xfb840108,0xcbaf0101,0xfb840108,0xcbb00101,0xfb840108,0xcbb10101,
	0xfb840108,0xcbb20101,0xfb840108,0xcbb30101,0xfb840108,0xcbb40101,0xfb840108,0xcbb50101,
	0xfb840108,0xcbb60101,0xfb840108,0xcbb70101,0xfb840108,0xcbb80101,0xfb840108,0xcbb90101,
	0xfb840108,0xcbba0101,0xfb840108,0xcbbb0101,0xfb840108,0xcbbc0101,0xfb840108,0xcbbd0101,
	0xfb840108,0xcbbe0101,0xfb840108,0xcbbf0101,0xfb840108,0xcbc00101,0xfb840108,0xcbc10101,
	0xfb840108,0xcbc20101,0xfb840108,0xcbc30101,0xfb840108,0xcbc40101,0xfb840108,0xcbc50101,
	0xfb840108,0xcbc60101,0xfb840108,0xcbc70101,0xfb840108,0xcbc80101,0xfb840108,0xcbc90101,
	0xfb840108,0xcbca0101,0xfb840108,0xcbcb0101,0xfb840108,0xcbcc0101,0xfb840108,0xcbcd0101,
	0xfb840108,0xcbce0101,0xfb840108,0xcbcf0101,0xfb840108,0xcbd00101,0xfb840108,0xcbd10101,
	0xfb840108,0xcbd20101,0xfb840108,0xcbd30101,0xfb840108,0xcbd40101,0xfb840108,0xcbd50101,
	0xfb840108,0xcbd60101,0xfb840108,0xcbd70101,0xfb840108,0xcbd80101,0xfb840108,0xcbd90101,
	0xfb840108,0xcbda0101,0xfb840108,0xcbdb0101,0xfb840108,0xcbdc0101,0xfb840108,0xcbdd0101,
	0xfb840108,0xcbde0101,0xfb840108,0xcbdf0101,0xfb840108,0xcbe00101,0xfb840108,0xcbe10101,
	0xfb840108,0xcbe20101,0xfb840108,0xcbe30101,0xfb840108,0xcbe40101,0xfb840108,0xcbe50101,
	0xfb840108,0xcbe60101,0xfb840108,0xcbe70101,0xfb840108,0xcbe80101,0xfb840108,0xcbe90101,
	0xfb840108,0xcbea0101,0xfb840108,0xcbeb0101,0xfb840108,0xcbec0101,0xfb840108,0xcbed0101,
	0xfb840108,0xcbee0101,0xfb840108,0xcbef0101,0xfb840108,0xcbf00101,0xfb840108,0xcbf10101,
	0xfb840108,0xcbf20101,0xfb840108,0xcbf30101,0xfb840108,0xcbf40101,0xfb840108,0xcbf50101,
	0xfb840108,0xcbf60101,0xfb840108,0xcbf70101,0xfb840108,0xcbf80101,0xfb840108,0xcbf90101,
	0xfb840108,0xcbfa0101,0xfb840108,0xcbfb0101,0xfb840108,0xcbfc0101,0xfb840108,0xcbfd0101,
	0xfb840108,0xcbfe0101,0xfb840108,0xcbff0101,0xfb840108,0xcc000101,0xfb840108,0xcc010101,
	0xfb840108,0xcc020101,0xfb840108,0xcc030101,0xfb840108,0xcc040101,0xfb840108,0xcc050101,
	0xfb840108,0xcc060101,0xfb840108,0xcc070101,0xfb840108,0xcc080101,0xfb840108,0xcc090101,
	0xfb840108,0xcc0a0101,0xfb840108,0xcc0b0101,0xfb840108,0xcc0c0101,0xfb840108,0xcc0d0101,
	0xfb840108,0xcc0e0101,0xfb840108,0xcc0f0101,0xfb840108,0xcc100101,0xfb840108,0xcc110101,
	0xfb840108,0xcc120101,0xfb840108,0xcc130101,0xfb840108,0xcc140101,0xfb840108,0xcc150101,
	0xfb840108,0xcc160101,0xfb840108,0xcc170101,0xfb840108,0xcc180101,0xfb840108,0xcc190101,
	0xfb840108,0xcc1a0101,0xfb840108,0xcc1b0101,0xfb840108,0xcc1c0101,0xfb840108,0xcc1d0101,
	0xfb840108,0xcc1e0101,0xfb840108,0xcc1f0101,0xfb840108,0xcc200101,0xfb840108,0xcc210101,
	0xfb840108,0xcc220101,0xfb840108,0xcc230101,0xfb840108,0xcc240101,0xfb840108,0xcc250101,
	0xfb840108,0xcc260101,0xfb840108,0xcc270101,0xfb840108,0xcc280101,0xfb840108,0xcc290101,
	0xfb840108,0xcc2a0101,0xfb840108,0xcc2b0101,0xfb840108,0xcc2c0101,0xfb840108,0xcc2d0101,
	0xfb840108,0xcc2e0101,0xfb840108,0xcc2f0101,0xfb840108,0xcc300101,0xfb840108,0xcc310101,
	0xfb840108,0xcc320101,0xfb840108,0xcc330101,0xfb840108,0xcc340101,0xfb840108,0xcc350101,
	0xfb840108,0xcc360101,0xfb840108,0xcc370101,0xfb840108,0xcc380101,0xfb840108,0xcc390101,
	0xfb840108,0xcc3a0101,0xfb840108,0xcc3b0101,0xfb840108,0xcc3c0101,0xfb840108,0xcc3d0101,
	0xfb840108,0xcc3e0101,0xfb840108,0xcc3f0101,0xfb840108,0xcc400101,0xfb840108,0xcc410101,
	0xfb840108,0xcc420101,0xfb840108,0xcc430101,0xfb840108,0xcc440101,0xfb840108,0xcc450101,
	0xfb840108,0xcc460101,0xfb840108,0xcc470101,0xfb840108,0xcc480101,0xfb840108,0xcc490101,
	0xfb840108,0xcc4a0101,0xfb840108,0xcc4b0101,0xfb840108,0xcc4c0101,0xfb840108,0xcc4d0101,
	0xfb840108,0xcc4e0101,0xfb840108,0xcc4f0101,0xfb840108,0xcc500101,0xfb840108,0xcc510101,
	0xfb840108,0xcc520101,0xfb840108,0xcc530101,0xfb840108,0xcc540101,0xfb840108,0xcc550101,
	0xfb840108,0xcc560101,0xfb840108,0xcc570101,0xfb840108,0xcc580101,0xfb840108,0xcc590101,
	0xfb840108,0xcc5a0101,0xfb840108,0xcc5b0101,0xfb840108,0xcc5c0101,0xfb840108,0xcc5d0101,
	0xfb840108,0xcc5e0101,0xfb840108,0xcc5f0101,0xfb840108,0xcc600101,0xfb840108,0xcc610101,
	0xfb840108,0xcc620101,0xfb840108,0xcc630101,0xfb840108,0xcc640101,0xfb840108,0xcc650101,
	0xfb840108,0xcc660101,0xfb840108,0xcc670101,0xfb840108,0xcc680101,0xfb840108,0xcc690101,
	0xfb840108,0xcc6a0101,0xfb840108,0xcc6b0101,0xfb840108,0xcc6c0101,0xfb840108,0xcc6d0101,
	0xfb840108,0xcc6e0101,0xfb840108,0xcc6f0101,0xfb840108,0xcc700101,0xfb840108,0xcc710101,
	0xfb840108,0xcc720101,0xfb840108,0xcc730101,0xfb840108,0xcc740101,0xfb840108,0xcc750101,
	0xfb840108,0xcc760101,0xfb840108,0xcc770101,0xfb840108,0xcc780101,0xfb840108,0xcc790101,
	0xfb840108,0xcc7a0101,0xfb840108,0xcc7b0101,0xfb840108,0xcc7c0101,0xfb840108,0xcc7d0101,
	0xfb840108,0xcc7e0101,0xfb840108,0xcc7f0101,0xfb840108,0xcc800101,0xfb840108,0xcc810101,
	0xfb840108,0xcc820101,0xfb840108,0xcc830101,0xfb840108,0xcc840101,0xfb840108,0xcc850101,
	0xfb840108,0xcc860101,0xfb840108,0xcc870101,0xfb840108,0xcc880101,0xfb840108,0xcc890101,
	0xfb840108,0xcc8a0101,0xfb840108,0xcc8b0101,0xfb840108,0xcc8c0101,0xfb840108,0xcc8d0101,
	0xfb840108,0xcc8e0101,0xfb840108,0xcc8f0101,0xfb840108,0xcc900101,0xfb840108,0xcc910101,
	0xfb840108,0xcc920101,0xfb840108,0xcc930101,0xfb840108,0xcc940101,0xfb840108,0xcc950101,
	0xfb840108,0xcc960101,0xfb840108,0xcc970101,0xfb840108,0xcc980101,0xfb840108,0xcc990101,
	0xfb840108,0xcc9a0101,0xfb840108,0xcc9b0101,0xfb840108,0xcc9c0101,0xfb840108,0xcc9d0101,
	0xfb840108,0xcc9e0101,0xfb840108,0xcc9f0101,0xfb840108,0xcca00101,0xfb840108,0xcca10101,
	0xfb840108,0xcca20101,0xfb840108,0xcca30101,0xfb840108,0xcca40101,0xfb840108,0xcca50101,
	0xfb840108,0xcca60101,0xfb840108,0xcca70101,0xfb840108,0xcca80101,0xfb840108,0xcca90101,
	0xfb840108,0xccaa0101,0xfb840108,0xccab0101,0xfb840108,0xccac0101,0xfb840108,0xccad0101,
	0xfb840108,0xccae0101,0xfb840108,0xccaf0101,0xfb840108,0xccb00101,0xfb840108,0xccb10101,
	0xfb840108,0xccb20101,0xfb840108,0xccb30101,0xfb840108,0xccb40101,0xfb840108,0xccb50101,
	0xfb840108,0xccb60101,0xfb840108,0xccb70101,0xfb840108,0xccb80101,0xfb840108,0xccb90101,
	0xfb840108,0xccba0101,0xfb840108,0xccbb0101,0xfb840108,0xccbc0101,0xfb840108,0xccbd0101,
	0xfb840108,0xccbe0101,0xfb840108,0xccbf0101,0xfb840108,0xccc00101,0xfb840108,0xccc10101,
	0xfb840108,0xccc20101,0xfb840108,0xccc30101,0xfb840108,0xccc40101,0xfb840108,0xccc50101,
	0xfb840108,0xccc60101,0xfb840108,0xccc70101,0xfb840108,0xccc80101,0xfb840108,0xccc90101,
	0xfb840108,0xccca0101,0xfb840108,0xcccb0101,0xfb840108,0xcccc0101,0xfb840108,0xcccd0101,
	0xfb840108,0xccce0101,0xfb840108,0xcccf0101,0xfb840108,0xccd00101,0xfb840108,0xccd10101,
	0xfb840108,0xccd20101,0xfb840108,0xccd30101,0xfb840108,0xccd40101,0xfb840108,0xccd50101,
	0xfb840108,0xccd60101,0xfb840108,0xccd70101,0xfb840108,0xccd80101,0xfb840108,0xccd90101,
	0xfb840108,0xccda0101,0xfb840108,0xccdb0101,0xfb840108,0xccdc0101,0xfb840108,0xccdd0101,
	0xfb840108,0xccde0101,0xfb840108,0xccdf0101,0xfb840108,0xcce00101,0xfb840108,0xcce10101,
	0xfb840108,0xcce20101,0xfb840108,0xcce30101,0xfb840108,0xcce40101,0xfb840108,0xcce50101,
	0xfb840108,0xcce60101,0xfb840108,0xcce70101,0xfb840108,0xcce80101,0xfb840108,0xcce90101,
	0xfb840108,0xccea0101,0xfb840108,0xcceb0101,0xfb840108,0xccec0101,0xfb840108,0xcced0101,
	0xfb840108,0xccee0101,0xfb840108,0xccef0101,0xfb840108,0xccf00101,0xfb840108,0xccf10101,
	0xfb840108,0xccf20101,0xfb840108,0xccf30101,0xfb840108,0xccf40101,0xfb840108,0xccf50101,
	0xfb840108,0xccf60101,0xfb840108,0xccf70101,0xfb840108,0xccf80101,0xfb840108,0xccf90101,
	0xfb840108,0xccfa0101,0xfb840108,0xccfb0101,0xfb840108,0xccfc0101,0xfb840108,0xccfd0101,
	0xfb840108,0xccfe0101,0xfb840108,0xccff0101,0xfb840108,0xcd000101,0xfb840108,0xcd010101,
	0xfb840108,0xcd020101,0xfb840108,0xcd030101,0xfb840108,0xcd040101,0xfb840108,0xcd050101,
	0xfb840108,0xcd060101,0xfb840108,0xcd070101,0xfb840108,0xcd080101,0xfb840108,0xcd090101,
	0xfb840108,0xcd0a0101,0xfb840108,0xcd0b0101,0xfb840108,0xcd0c0101,0xfb840108,0xcd0d0101,
	0xfb840108,0xcd0e0101,0xfb840108,0xcd0f0101,0xfb840108,0xcd100101,0xfb840108,0xcd110101,
	0xfb840108,0xcd120101,0xfb840108,0xcd130101,0xfb840108,0xcd140101,0xfb840108,0xcd150101,
	0xfb840108,0xcd160101,0xfb840108,0xcd170101,0xfb840108,0xcd180101,0xfb840108,0xcd190101,
	0xfb840108,0xcd1a0101,0xfb840108,0xcd1b0101,0xfb840108,0xcd1c0101,0xfb840108,0xcd1d0101,
	0xfb840108,0xcd1e0101,0xfb840108,0xcd1f0101,0xfb840108,0xcd200101,0xfb840108,0xcd210101,
	0xfb840108,0xcd220101,0xfb840108,0xcd230101,0xfb840108,0xcd240101,0xfb840108,0xcd250101,
	0xfb840108,0xcd260101,0xfb840108,0xcd270101,0xfb840108,0xcd280101,0xfb840108,0xcd290101,
	0xfb840108,0xcd2a0101,0xfb840108,0xcd2b0101,0xfb840108,0xcd2c0101,0xfb840108,0xcd2d0101,
	0xfb840108,0xcd2e0101,0xfb840108,0xcd2f0101,0xfb840108,0xcd300101,0xfb840108,0xcd310101,
	0xfb840108,0xcd320101,0xfb840108,0xcd330101,0xfb840108,0xcd340101,0xfb840108,0xcd350101,
	0xfb840108,0xcd360101,0xfb840108,0xcd370101,0xfb840108,0xcd380101,0xfb840108,0xcd390101,
	0xfb840108,0xcd3a0101,0xfb840108,0xcd3b0101,0xfb840108,0xcd3c0101,0xfb840108,0xcd3d0101,
	0xfb840108,0xcd3e0101,0xfb840108,0xcd3f0101,0xfb840108,0xcd400101,0xfb840108,0xcd410101,
	0xfb840108,0xcd420101,0xfb840108,0xcd430101,0xfb840108,0xcd440101,0xfb840108,0xcd450101,
	0xfb840108,0xcd460101,0xfb840108,0xcd470101,0xfb840108,0xcd480101,0xfb840108,0xcd490101,
	0xfb840108,0xcd4a0101,0xfb840108,0xcd4b0101,0xfb840108,0xcd4c0101,0xfb840108,0xcd4d0101,
	0xfb840108,0xcd4e0101,0xfb840108,0xcd4f0101,0xfb840108,0xcd500101,0xfb840108,0xcd510101,
	0xfb840108,0xcd520101,0xfb840108,0xcd530101,0xfb840108,0xcd540101,0xfb840108,0xcd550101,
	0xfb840108,0xcd560101,0xfb840108,0xcd570101,0xfb840108,0xcd580101,0xfb840108,0xcd590101,
	0xfb840108,0xcd5a0101,0xfb840108,0xcd5b0101,0xfb840108,0xcd5c0101,0xfb840108,0xcd5d0101,
	0xfb840108,0xcd5e0101,0xfb840108,0xcd5f0101,0xfb840108,0xcd600101,0xfb840108,0xcd610101,
	0xfb840108,0xcd620101,0xfb840108,0xcd630101,0xfb840108,0xcd640101,0xfb840108,0xcd650101,
	0xfb840108,0xcd660101,0xfb840108,0xcd670101,0xfb840108,0xcd680101,0xfb840108,0xcd690101,
	0xfb840108,0xcd6a0101,0xfb840108,0xcd6b0101,0xfb840108,0xcd6c0101,0xfb840108,0xcd6d0101,
	0xfb840108,0xcd6e0101,0xfb840108,0xcd6f0101,0xfb840108,0xcd700101,0xfb840108,0xcd710101,
	0xfb840108,0xcd720101,0xfb840108,0xcd730101,0xfb840108,0xcd740101,0xfb840108,0xcd750101,
	0xfb840108,0xcd760101,0xfb840108,0xcd770101,0xfb840108,0xcd780101,0xfb840108,0xcd790101,
	0xfb840108,0xcd7a0101,0xfb840108,0xcd7b0101,0xfb840108,0xcd7c0101,0xfb840108,0xcd7d0101,
	0xfb840108,0xcd7e0101,0xfb840108,0xcd7f0101,0xfb840108,0xcd800101,0xfb840108,0xcd810101,
	0xfb840108,0xcd820101,0xfb840108,0xcd830101,0xfb840108,0xcd840101,0xfb840108,0xcd850101,
	0xfb840108,0xcd860101,0xfb840108,0xcd870101,0xfb840108,0xcd880101,0xfb840108,0xcd890101,
	0xfb840108,0xcd8a0101,0xfb840108,0xcd8b0101,0xfb840108,0xcd8c0101,0xfb840108,0xcd8d0101,
	0xfb840108,0xcd8e0101,0xfb840108,0xcd8f0101,0xfb840108,0xcd900101,0xfb840108,0xcd910101,
	0xfb840108,0xcd920101,0xfb840108,0xcd930101,0xfb840108,0xcd940101,0xfb840108,0xcd950101,
	0xfb840108,0xcd960101,0xfb840108,0xcd970101,0xfb840108,0xcd980101,0xfb840108,0xcd990101,
	0xfb840108,0xcd9a0101,0xfb840108,0xcd9b0101,0xfb840108,0xcd9c0101,0xfb840108,0xcd9d0101,
	0xfb840108,0xcd9e0101,0xfb840108,0xcd9f0101,0xfb840108,0xcda00101,0xfb840108,0xcda10101,
	0xfb840108,0xcda20101,0xfb840108,0xcda30101,0xfb840108,0xcda40101,0xfb840108,0xcda50101,
	0xfb840108,0xcda60101,0xfb840108,0xcda70101,0xfb840108,0xcda80101,0xfb840108,0xcda90101,
	0xfb840108,0xcdaa0101,0xfb840108,0xcdab0101,0xfb840108,0xcdac0101,0xfb840108,0xcdad0101,
	0xfb840108,0xcdae0101,0xfb840108,0xcdaf0101,0xfb840108,0xcdb00101,0xfb840108,0xcdb10101,
	0xfb840108,0xcdb20101,0xfb840108,0xcdb30101,0xfb840108,0xcdb40101,0xfb840108,0xcdb50101,
	0xfb840108,0xcdb60101,0xfb840108,0xcdb70101,0xfb840108,0xcdb80101,0xfb840108,0xcdb90101,
	0xfb840108,0xcdba0101,0xfb840108,0xcdbb0101,0xfb840108,0xcdbc0101,0xfb840108,0xcdbd0101,
	0xfb840108,0xcdbe0101,0xfb840108,0xcdbf0101,0xfb840108,0xcdc00101,0xfb840108,0xcdc10101,
	0xfb840108,0xcdc20101,0xfb840108,0xcdc30101,0xfb840108,0xcdc40101,0xfb840108,0xcdc50101,
	0xfb840108,0xcdc60101,0xfb840108,0xcdc70101,0xfb840108,0xcdc80101,0xfb840108,0xcdc90101,
	0xfb840108,0xcdca0101,0xfb840108,0xcdcb0101,0xfb840108,0xcdcc0101,0xfb840108,0xcdcd0101,
	0xfb840108,0xcdce0101,0xfb840108,0xcdcf0101,0xfb840108,0xcdd00101,0xfb840108,0xcdd10101,
	0xfb840108,0xcdd20101,0xfb840108,0xcdd30101,0xfb840108,0xcdd40101,0xfb840108,0xcdd50101,
	0xfb840108,0xcdd60101,0xfb840108,0xcdd70101,0xfb840108,0xcdd80101,0xfb840108,0xcdd90101,
	0xfb840108,0xcdda0101,0xfb840108,0xcddb0101,0xfb840108,0xcddc0101,0xfb840108,0xcddd0101,
	0xfb840108,0xcdde0101,0xfb840108,0xcddf0101,0xfb840108,0xcde00101,0xfb840108,0xcde10101,
	0xfb840108,0xcde20101,0xfb840108,0xcde30101,0xfb840108,0xcde40101,0xfb840108,0xcde50101,
	0xfb840108,0xcde60101,0xfb840108,0xcde70101,0xfb840108,0xcde80101,0xfb840108,0xcde90101,
	0xfb840108,0xcdea0101,0xfb840108,0xcdeb0101,0xfb840108,0xcdec0101,0xfb840108,0xcded0101,
	0xfb840108,0xcdee0101,0xfb840108,0xcdef0101,0xfb840108,0xcdf00101,0xfb840108,0xcdf10101,
	0xfb840108,0xcdf20101,0xfb840108,0xcdf30101,0xfb840108,0xcdf40101,0xfb840108,0xcdf50101,
	0xfb840108,0xcdf60101,0xfb840108,0xcdf70101,0xfb840108,0xcdf80101,0xfb840108,0xcdf90101,
	0xfb840108,0xcdfa0101,0xfb840108,0xcdfb0101,0xfb840108,0xcdfc0101,0xfb840108,0xcdfd0101,
	0xfb840108,0xcdfe0101,0xfb840108,0xcdff0101,0xfb840108,0xce000101,0xfb840108,0xce010101,
	0xfb840108,0xce020101,0xfb840108,0xce030101,0xfb840108,0xce040101,0xfb840108,0xce050101,
	0xfb840108,0xce060101,0xfb840108,0xce070101,0xfb840108,0xce080101,0xfb840108,0xce090101,
	0xfb840108,0xce0a0101,0xfb840108,0xce0b0101,0xfb840108,0xce0c0101,0xfb840108,0xce0d0101,
	0xfb840108,0xce0e0101,0xfb840108,0xce0f0101,0xfb840108,0xce100101,0xfb840108,0xce110101,
	0xfb840108,0xce120101,0xfb840108,0xce130101,0xfb840108,0xce140101,0xfb840108,0xce150101,
	0xfb840108,0xce160101,0xfb840108,0xce170101,0xfb840108,0xce180101,0xfb840108,0xce190101,
	0xfb840108,0xce1a0101,0xfb840108,0xce1b0101,0xfb840108,0xce1c0101,0xfb840108,0xce1d0101,
	0xfb840108,0xce1e0101,0xfb840108,0xce1f0101,0xfb840108,0xce200101,0xfb840108,0xce210101,
	0xfb840108,0xce220101,0xfb840108,0xce230101,0xfb840108,0xce240101,0xfb840108,0xce250101,
	0xfb840108,0xce260101,0xfb840108,0xce270101,0xfb840108,0xce280101,0xfb840108,0xce290101,
	0xfb840108,0xce2a0101,0xfb840108,0xce2b0101,0xfb840108,0xce2c0101,0xfb840108,0xce2d0101,
	0xfb840108,0xce2e0101,0xfb840108,0xce2f0101,0xfb840108,0xce300101,0xfb840108,0xce310101,
	0xfb840108,0xce320101,0xfb840108,0xce330101,0xfb840108,0xce340101,0xfb840108,0xce350101,
	0xfb840108,0xce360101,0xfb840108,0xce370101,0xfb840108,0xce380101,0xfb840108,0xce390101,
	0xfb840108,0xce3a0101,0xfb840108,0xce3b0101,0xfb840108,0xce3c0101,0xfb840108,0xce3d0101,
	0xfb840108,0xce3e0101,0xfb840108,0xce3f0101,0xfb840108,0xce400101,0xfb840108,0xce410101,
	0xfb840108,0xce420101,0xfb840108,0xce430101,0xfb840108,0xce440101,0xfb840108,0xce450101,
	0xfb840108,0xce460101,0xfb840108,0xce470101,0xfb840108,0xce480101,0xfb840108,0xce490101,
	0xfb840108,0xce4a0101,0xfb840108,0xce4b0101,0xfb840108,0xce4c0101,0xfb840108,0xce4d0101,
	0xfb840108,0xce4e0101,0xfb840108,0xce4f0101,0xfb840108,0xce500101,0xfb840108,0xce510101,
	0xfb840108,0xce520101,0xfb840108,0xce530101,0xfb840108,0xce540101,0xfb840108,0xce550101,
	0xfb840108,0xce560101,0xfb840108,0xce570101,0xfb840108,0xce580101,0xfb840108,0xce590101,
	0xfb840108,0xce5a0101,0xfb840108,0xce5b0101,0xfb840108,0xce5c0101,0xfb840108,0xce5d0101,
	0xfb840108,0xce5e0101,0xfb840108,0xce5f0101,0xfb840108,0xce600101,0xfb840108,0xce610101,
	0xfb840108,0xce620101,0xfb840108,0xce630101,0xfb840108,0xce640101,0xfb840108,0xce650101,
	0xfb840108,0xce660101,0xfb840108,0xce670101,0xfb840108,0xce680101,0xfb840108,0xce690101,
	0xfb840108,0xce6a0101,0xfb840108,0xce6b0101,0xfb840108,0xce6c0101,0xfb840108,0xce6d0101,
	0xfb840108,0xce6e0101,0xfb840108,0xce6f0101,0xfb840108,0xce700101,0xfb840108,0xce710101,
	0xfb840108,0xce720101,0xfb840108,0xce730101,0xfb840108,0xce740101,0xfb840108,0xce750101,
	0xfb840108,0xce760101,0xfb840108,0xce770101,0xfb840108,0xce780101,0xfb840108,0xce790101,
	0xfb840108,0xce7a0101,0xfb840108,0xce7b0101,0xfb840108,0xce7c0101,0xfb840108,0xce7d0101,
	0xfb840108,0xce7e0101,0xfb840108,0xce7f0101,0xfb840108,0xce800101,0xfb840108,0xce810101,
	0xfb840108,0xce820101,0xfb840108,0xce830101,0xfb840108,0xce840101,0xfb840108,0xce850101,
	0xfb840108,0xce860101,0xfb840108,0xce870101,0xfb840108,0xce880101,0xfb840108,0xce890101,
	0xfb840108,0xce8a0101,0xfb840108,0xce8b0101,0xfb840108,0xce8c0101,0xfb840108,0xce8d0101,
	0xfb840108,0xce8e0101,0xfb840108,0xce8f0101,0xfb840108,0xce900101,0xfb840108,0xce910101,
	0xfb840108,0xce920101,0xfb840108,0xce930101,0xfb840108,0xce940101,0xfb840108,0xce950101,
	0xfb840108,0xce960101,0xfb840108,0xce970101,0xfb840108,0xce980101,0xfb840108,0xce990101,
	0xfb840108,0xce9a0101,0xfb840108,0xce9b0101,0xfb840108,0xce9c0101,0xfb840108,0xce9d0101,
	0xfb840108,0xce9e0101,0xfb840108,0xce9f0101,0xfb840108,0xcea00101,0xfb840108,0xcea10101,
	0xfb840108,0xcea20101,0xfb840108,0xcea30101,0xfb840108,0xcea40101,0xfb840108,0xcea50101,
	0xfb840108,0xcea60101,0xfb840108,0xcea70101,0xfb840108,0xcea80101,0xfb840108,0xcea90101,
	0xfb840108,0xceaa0101,0xfb840108,0xceab0101,0xfb840108,0xceac0101,0xfb840108,0xcead0101,
	0xfb840108,0xceae0101,0xfb840108,0xceaf0101,0xfb840108,0xceb00101,0xfb840108,0xceb10101,
	0xfb840108,0xceb20101,0xfb840108,0xceb30101,0xfb840108,0xceb40101,0xfb840108,0xceb50101,
	0xfb840108,0xceb60101,0xfb840108,0xceb70101,0xfb840108,0xceb80101,0xfb840108,0xceb90101,
	0xfb840108,0xceba0101,0xfb840108,0xcebb0101,0xfb840108,0xcebc0101,0xfb840108,0xcebd0101,
	0xfb840108,0xcebe0101,0xfb840108,0xcebf0101,0xfb840108,0xcec00101,0xfb840108,0xcec10101,
	0xfb840108,0xcec20101,0xfb840108,0xcec30101,0xfb840108,0xcec40101,0xfb840108,0xcec50101,
	0xfb840108,0xcec60101,0xfb840108,0xcec70101,0xfb840108,0xcec80101,0xfb840108,0xcec90101,
	0xfb840108,0xceca0101,0xfb840108,0xcecb0101,0xfb840108,0xcecc0101,0xfb840108,0xcecd0101,
	0xfb840108,0xcece0101,0xfb840108,0xcecf0101,0xfb840108,0xced00101,0xfb840108,0xced10101,
	0xfb840108,0xced20101,0xfb840108,0xced30101,0xfb840108,0xced40101,0xfb840108,0xced50101,
	0xfb840108,0xced60101,0xfb840108,0xced70101,0xfb840108,0xced80101,0xfb840108,0xced90101,
	0xfb840108,0xceda0101,0xfb840108,0xcedb0101,0xfb840108,0xcedc0101,0xfb840108,0xcedd0101,
	0xfb840108,0xcede0101,0xfb840108,0xcedf0101,0xfb840108,0xcee00101,0xfb840108,0xcee10101,
	0xfb840108,0xcee20101,0xfb840108,0xcee30101,0xfb840108,0xcee40101,0xfb840108,0xcee50101,
	0xfb840108,0xcee60101,0xfb840108,0xcee70101,0xfb840108,0xcee80101,0xfb840108,0xcee90101,
	0xfb840108,0xceea0101,0xfb840108,0xceeb0101,0xfb840108,0xceec0101,0xfb840108,0xceed0101,
	0xfb840108,0xceee0101,0xfb840108,0xceef0101,0xfb840108,0xcef00101,0xfb840108,0xcef10101,
	0xfb840108,0xcef20101,0xfb840108,0xcef30101,0xfb840108,0xcef40101,0xfb840108,0xcef50101,
	0xfb840108,0xcef60101,0xfb840108,0xcef70101,0xfb840108,0xcef80101,0xfb840108,0xcef90101,
	0xfb840108,0xcefa0101,0xfb840108,0xcefb0101,0xfb840108,0xcefc0101,0xfb840108,0xcefd0101,
	0xfb840108,0xcefe0101,0xfb840108,0xceff0101,0xfb840108,0xcf000101,0xfb840108,0xcf010101,
	0xfb840108,0xcf020101,0xfb840108,0xcf030101,0xfb840108,0xcf040101,0xfb840108,0xcf050101,
	0xfb840108,0xcf060101,0xfb840108,0xcf070101,0xfb840108,0xcf080101,0xfb840108,0xcf090101,
	0xfb840108,0xcf0a0101,0xfb840108,0xcf0b0101,0xfb840108,0xcf0c0101,0xfb840108,0xcf0d0101,
	0xfb840108,0xcf0e0101,0xfb840108,0xcf0f0101,0xfb840108,0xcf100101,0xfb840108,0xcf110101,
	0xfb840108,0xcf120101,0xfb840108,0xcf130101,0xfb840108,0xcf140101,0xfb840108,0xcf150101,
	0xfb840108,0xcf160101,0xfb840108,0xcf170101,0xfb840108,0xcf180101,0xfb840108,0xcf190101,
	0xfb840108,0xcf1a0101,0xfb840108,0xcf1b0101,0xfb840108,0xcf1c0101,0xfb840108,0xcf1d0101,
	0xfb840108,0xcf1e0101,0xfb840108,0xcf1f0101,0xfb840108,0xcf200101,0xfb840108,0xcf210101,
	0xfb840108,0xcf220101,0xfb840108,0xcf230101,0xfb840108,0xcf240101,0xfb840108,0xcf250101,
	0xfb840108,0xcf260101,0xfb840108,0xcf270101,0xfb840108,0xcf280101,0xfb840108,0xcf290101,
	0xfb840108,0xcf2a0101,0xfb840108,0xcf2b0101,0xfb840108,0xcf2c0101,0xfb840108,0xcf2d0101,
	0xfb840108,0xcf2e0101,0xfb840108,0xcf2f0101,0xfb840108,0xcf300101,0xfb840108,0xcf310101,
	0xfb840108,0xcf320101,0xfb840108,0xcf330101,0xfb840108,0xcf340101,0xfb840108,0xcf350101,
	0xfb840108,0xcf360101,0xfb840108,0xcf370101,0xfb840108,0xcf380101,0xfb840108,0xcf390101,
	0xfb840108,0xcf3a0101,0xfb840108,0xcf3b0101,0xfb840108,0xcf3c0101,0xfb840108,0xcf3d0101,
	0xfb840108,0xcf3e0101,0xfb840108,0xcf3f0101,0xfb840108,0xcf400101,0xfb840108,0xcf410101,
	0xfb840108,0xcf420101,0xfb840108,0xcf430101,0xfb840108,0xcf440101,0xfb840108,0xcf450101,
	0xfb840108,0xcf460101,0xfb840108,0xcf470101,0xfb840108,0xcf480101,0xfb840108,0xcf490101,
	0xfb840108,0xcf4a0101,0xfb840108,0xcf4b0101,0xfb840108,0xcf4c0101,0xfb840108,0xcf4d0101,
	0xfb840108,0xcf4e0101,0xfb840108,0xcf4f0101,0xfb840108,0xcf500101,0xfb840108,0xcf510101,
	0xfb840108,0xcf520101,0xfb840108,0xcf530101,0xfb840108,0xcf540101,0xfb840108,0xcf550101,
	0xfb840108,0xcf560101,0xfb840108,0xcf570101,0xfb840108,0xcf580101,0xfb840108,0xcf590101,
	0xfb840108,0xcf5a0101,0xfb840108,0xcf5b0101,0xfb840108,0xcf5c0101,0xfb840108,0xcf5d0101,
	0xfb840108,0xcf5e0101,0xfb840108,0xcf5f0101,0xfb840108,0xcf600101,0xfb840108,0xcf610101,
	0xfb840108,0xcf620101,0xfb840108,0xcf630101,0xfb840108,0xcf640101,0xfb840108,0xcf650101,
	0xfb840108,0xcf660101,0xfb840108,0xcf670101,0xfb840108,0xcf680101,0xfb840108,0xcf690101,
	0xfb840108,0xcf6a0101,0xfb840108,0xcf6b0101,0xfb840108,0xcf6c0101,0xfb840108,0xcf6d0101,
	0xfb840108,0xcf6e0101,0xfb840108,0xcf6f0101,0xfb840108,0xcf700101,0xfb840108,0xcf710101,
	0xfb840108,0xcf720101,0xfb840108,0xcf730101,0xfb840108,0xcf740101,0xfb840108,0xcf750101,
	0xfb840108,0xcf760101,0xfb840108,0xcf770101,0xfb840108,0xcf780101,0xfb840108,0xcf790101,
	0xfb840108,0xcf7a0101,0xfb840108,0xcf7b0101,0xfb840108,0xcf7c0101,0xfb840108,0xcf7d0101,
	0xfb840108,0xcf7e0101,0xfb840108,0xcf7f0101,0xfb840108,0xcf800101,0xfb840108,0xcf810101,
	0xfb840108,0xcf820101,0xfb840108,0xcf830101,0xfb840108,0xcf840101,0xfb840108,0xcf850101,
	0xfb840108,0xcf860101,0xfb840108,0xcf870101,0xfb840108,0xcf880101,0xfb840108,0xcf890101,
	0xfb840108,0xcf8a0101,0xfb840108,0xcf8b0101,0xfb840108,0xcf8c0101,0xfb840108,0xcf8d0101,
	0xfb840108,0xcf8e0101,0xfb840108,0xcf8f0101,0xfb840108,0xcf900101,0xfb840108,0xcf910101,
	0xfb840108,0xcf920101,0xfb840108,0xcf930101,0xfb840108,0xcf940101,0xfb840108,0xcf950101,
	0xfb840108,0xcf960101,0xfb840108,0xcf970101,0xfb840108,0xcf980101,0xfb840108,0xcf990101,
	0xfb840108,0xcf9a0101,0xfb840108,0xcf9b0101,0xfb840108,0xcf9c0101,0xfb840108,0xcf9d0101,
	0xfb840108,0xcf9e0101,0xfb840108,0xcf9f0101,0xfb840108,0xcfa00101,0xfb840108,0xcfa10101,
	0xfb840108,0xcfa20101,0xfb840108,0xcfa30101,0xfb840108,0xcfa40101,0xfb840108,0xcfa50101,
	0xfb840108,0xcfa60101,0xfb840108,0xcfa70101,0xfb840108,0xcfa80101,0xfb840108,0xcfa90101,
	0xfb840108,0xcfaa0101,0xfb840108,0xcfab0101,0xfb840108,0xcfac0101,0xfb840108,0xcfad0101,
	0xfb840108,0xcfae0101,0xfb840108,0xcfaf0101,0xfb840108,0xcfb00101,0xfb840108,0xcfb10101,
	0xfb840108,0xcfb20101,0xfb840108,0xcfb30101,0xfb840108,0xcfb40101,0xfb840108,0xcfb50101,
	0xfb840108,0xcfb60101,0xfb840108,0xcfb70101,0xfb840108,0xcfb80101,0xfb840108,0xcfb90101,
	0xfb840108,0xcfba0101,0xfb840108,0xcfbb0101,0xfb840108,0xcfbc0101,0xfb840108,0xcfbd0101,
	0xfb840108,0xcfbe0101,0xfb840108,0xcfbf0101,0xfb840108,0xcfc00101,0xfb840108,0xcfc10101,
	0xfb840108,0xcfc20101,0xfb840108,0xcfc30101,0xfb840108,0xcfc40101,0xfb840108,0xcfc50101,
	0xfb840108,0xcfc60101,0xfb840108,0xcfc70101,0xfb840108,0xcfc80101,0xfb840108,0xcfc90101,
	0xfb840108,0xcfca0101,0xfb840108,0xcfcb0101,0xfb840108,0xcfcc0101,0xfb840108,0xcfcd0101,
	0xfb840108,0xcfce0101,0xfb840108,0xcfcf0101,0xfb840108,0xcfd00101,0xfb840108,0xcfd10101,
	0xfb840108,0xcfd20101,0xfb840108,0xcfd30101,0xfb840108,0xcfd40101,0xfb840108,0xcfd50101,
	0xfb840108,0xcfd60101,0xfb840108,0xcfd70101,0xfb840108,0xcfd80101,0xfb840108,0xcfd90101,
	0xfb840108,0xcfda0101,0xfb840108,0xcfdb0101,0xfb840108,0xcfdc0101,0xfb840108,0xcfdd0101,
	0xfb840108,0xcfde0101,0xfb840108,0xcfdf0101,0xfb840108,0xcfe00101,0xfb840108,0xcfe10101,
	0xfb840108,0xcfe20101,0xfb840108,0xcfe30101,0xfb840108,0xcfe40101,0xfb840108,0xcfe50101,
	0xfb840108,0xcfe60101,0xfb840108,0xcfe70101,0xfb840108,0xcfe80101,0xfb840108,0xcfe90101,
	0xfb840108,0xcfea0101,0xfb840108,0xcfeb0101,0xfb840108,0xcfec0101,0xfb840108,0xcfed0101,
	0xfb840108,0xcfee0101,0xfb840108,0xcfef0101,0xfb840108,0xcff00101,0xfb840108,0xcff10101,
	0xfb840108,0xcff20101,0xfb840108,0xcff30101,0xfb840108,0xcff40101,0xfb840108,0xcff50101,
	0xfb840108,0xcff60101,0xfb840108,0xcff70101,0xfb840108,0xcff80101,0xfb840108,0xcff90101,
	0xfb840108,0xcffa0101,0xfb840108,0xcffb0101,0xfb840108,0xcffc0101,0xfb840108,0xcffd0101,
	0xfb840108,0xcffe0101,0xfb840108,0xcfff0101,0xfb840108,0xd0000101,0xfb840108,0xd0010101,
	0xfb840108,0xd0020101,0xfb840108,0xd0030101,0xfb840108,0xd0040101,0xfb840108,0xd0050101,
	0xfb840108,0xd0060101,0xfb840108,0xd0070101,0xfb840108,0xd0080101,0xfb840108,0xd0090101,
	0xfb840108,0xd00a0101,0xfb840108,0xd00b0101,0xfb840108,0xd00c0101,0xfb840108,0xd00d0101,
	0xfb840108,0xd00e0101,0xfb840108,0xd00f0101,0xfb840108,0xd0100101,0xfb840108,0xd0110101,
	0xfb840108,0xd0120101,0xfb840108,0xd0130101,0xfb840108,0xd0140101,0xfb840108,0xd0150101,
	0xfb840108,0xd0160101,0xfb840108,0xd0170101,0xfb840108,0xd0180101,0xfb840108,0xd0190101,
	0xfb840108,0xd01a0101,0xfb840108,0xd01b0101,0xfb840108,0xd01c0101,0xfb840108,0xd01d0101,
	0xfb840108,0xd01e0101,0xfb840108,0xd01f0101,0xfb840108,0xd0200101,0xfb840108,0xd0210101,
	0xfb840108,0xd0220101,0xfb840108,0xd0230101,0xfb840108,0xd0240101,0xfb840108,0xd0250101,
	0xfb840108,0xd0260101,0xfb840108,0xd0270101,0xfb840108,0xd0280101,0xfb840108,0xd0290101,
	0xfb840108,0xd02a0101,0xfb840108,0xd02b0101,0xfb840108,0xd02c0101,0xfb840108,0xd02d0101,
	0xfb840108,0xd02e0101,0xfb840108,0xd02f0101,0xfb840108,0xd0300101,0xfb840108,0xd0310101,
	0xfb840108,0xd0320101,0xfb840108,0xd0330101,0xfb840108,0xd0340101,0xfb840108,0xd0350101,
	0xfb840108,0xd0360101,0xfb840108,0xd0370101,0xfb840108,0xd0380101,0xfb840108,0xd0390101,
	0xfb840108,0xd03a0101,0xfb840108,0xd03b0101,0xfb840108,0xd03c0101,0xfb840108,0xd03d0101,
	0xfb840108,0xd03e0101,0xfb840108,0xd03f0101,0xfb840108,0xd0400101,0xfb840108,0xd0410101,
	0xfb840108,0xd0420101,0xfb840108,0xd0430101,0xfb840108,0xd0440101,0xfb840108,0xd0450101,
	0xfb840108,0xd0460101,0xfb840108,0xd0470101,0xfb840108,0xd0480101,0xfb840108,0xd0490101,
	0xfb840108,0xd04a0101,0xfb840108,0xd04b0101,0xfb840108,0xd04c0101,0xfb840108,0xd04d0101,
	0xfb840108,0xd04e0101,0xfb840108,0xd04f0101,0xfb840108,0xd0500101,0xfb840108,0xd0510101,
	0xfb840108,0xd0520101,0xfb840108,0xd0530101,0xfb840108,0xd0540101,0xfb840108,0xd0550101,
	0xfb840108,0xd0560101,0xfb840108,0xd0570101,0xfb840108,0xd0580101,0xfb840108,0xd0590101,
	0xfb840108,0xd05a0101,0xfb840108,0xd05b0101,0xfb840108,0xd05c0101,0xfb840108,0xd05d0101,
	0xfb840108,0xd05e0101,0xfb840108,0xd05f0101,0xfb840108,0xd0600101,0xfb840108,0xd0610101,
	0xfb840108,0xd0620101,0xfb840108,0xd0630101,0xfb840108,0xd0640101,0xfb840108,0xd0650101,
	0xfb840108,0xd0660101,0xfb840108,0xd0670101,0xfb840108,0xd0680101,0xfb840108,0xd0690101,
	0xfb840108,0xd06a0101,0xfb840108,0xd06b0101,0xfb840108,0xd06c0101,0xfb840108,0xd06d0101,
	0xfb840108,0xd06e0101,0xfb840108,0xd06f0101,0xfb840108,0xd0700101,0xfb840108,0xd0710101,
	0xfb840108,0xd0720101,0xfb840108,0xd0730101,0xfb840108,0xd0740101,0xfb840108,0xd0750101,
	0xfb840108,0xd0760101,0xfb840108,0xd0770101,0xfb840108,0xd0780101,0xfb840108,0xd0790101,
	0xfb840108,0xd07a0101,0xfb840108,0xd07b0101,0xfb840108,0xd07c0101,0xfb840108,0xd07d0101,
	0xfb840108,0xd07e0101,0xfb840108,0xd07f0101,0xfb840108,0xd0800101,0xfb840108,0xd0810101,
	0xfb840108,0xd0820101,0xfb840108,0xd0830101,0xfb840108,0xd0840101,0xfb840108,0xd0850101,
	0xfb840108,0xd0860101,0xfb840108,0xd0870101,0xfb840108,0xd0880101,0xfb840108,0xd0890101,
	0xfb840108,0xd08a0101,0xfb840108,0xd08b0101,0xfb840108,0xd08c0101,0xfb840108,0xd08d0101,
	0xfb840108,0xd08e0101,0xfb840108,0xd08f0101,0xfb840108,0xd0900101,0xfb840108,0xd0910101,
	0xfb840108,0xd0920101,0xfb840108,0xd0930101,0xfb840108,0xd0940101,0xfb840108,0xd0950101,
	0xfb840108,0xd0960101,0xfb840108,0xd0970101,0xfb840108,0xd0980101,0xfb840108,0xd0990101,
	0xfb840108,0xd09a0101,0xfb840108,0xd09b0101,0xfb840108,0xd09c0101,0xfb840108,0xd09d0101,
	0xfb840108,0xd09e0101,0xfb840108,0xd09f0101,0xfb840108,0xd0a00101,0xfb840108,0xd0a10101,
	0xfb840108,0xd0a20101,0xfb840108,0xd0a30101,0xfb840108,0xd0a40101,0xfb840108,0xd0a50101,
	0xfb840108,0xd0a60101,0xfb840108,0xd0a70101,0xfb840108,0xd0a80101,0xfb840108,0xd0a90101,
	0xfb840108,0xd0aa0101,0xfb840108,0xd0ab0101,0xfb840108,0xd0ac0101,0xfb840108,0xd0ad0101,
	0xfb840108,0xd0ae0101,0xfb840108,0xd0af0101,0xfb840108,0xd0b00101,0xfb840108,0xd0b10101,
	0xfb840108,0xd0b20101,0xfb840108,0xd0b30101,0xfb840108,0xd0b40101,0xfb840108,0xd0b50101,
	0xfb840108,0xd0b60101,0xfb840108,0xd0b70101,0xfb840108,0xd0b80101,0xfb840108,0xd0b90101,
	0xfb840108,0xd0ba0101,0xfb840108,0xd0bb0101,0xfb840108,0xd0bc0101,0xfb840108,0xd0bd0101,
	0xfb840108,0xd0be0101,0xfb840108,0xd0bf0101,0xfb840108,0xd0c00101,0xfb840108,0xd0c10101,
	0xfb840108,0xd0c20101,0xfb840108,0xd0c30101,0xfb840108,0xd0c40101,0xfb840108,0xd0c50101,
	0xfb840108,0xd0c60101,0xfb840108,0xd0c70101,0xfb840108,0xd0c80101,0xfb840108,0xd0c90101,
	0xfb840108,0xd0ca0101,0xfb840108,0xd0cb0101,0xfb840108,0xd0cc0101,0xfb840108,0xd0cd0101,
	0xfb840108,0xd0ce0101,0xfb840108,0xd0cf0101,0xfb840108,0xd0d00101,0xfb840108,0xd0d10101,
	0xfb840108,0xd0d20101,0xfb840108,0xd0d30101,0xfb840108,0xd0d40101,0xfb840108,0xd0d50101,
	0xfb840108,0xd0d60101,0xfb840108,0xd0d70101,0xfb840108,0xd0d80101,0xfb840108,0xd0d90101,
	0xfb840108,0xd0da0101,0xfb840108,0xd0db0101,0xfb840108,0xd0dc0101,0xfb840108,0xd0dd0101,
	0xfb840108,0xd0de0101,0xfb840108,0xd0df0101,0xfb840108,0xd0e00101,0xfb840108,0xd0e10101,
	0xfb840108,0xd0e20101,0xfb840108,0xd0e30101,0xfb840108,0xd0e40101,0xfb840108,0xd0e50101,
	0xfb840108,0xd0e60101,0xfb840108,0xd0e70101,0xfb840108,0xd0e80101,0xfb840108,0xd0e90101,
	0xfb840108,0xd0ea0101,0xfb840108,0xd0eb0101,0xfb840108,0xd0ec0101,0xfb840108,0xd0ed0101,
	0xfb840108,0xd0ee0101,0xfb840108,0xd0ef0101,0xfb840108,0xd0f00101,0xfb840108,0xd0f10101,
	0xfb840108,0xd0f20101,0xfb840108,0xd0f30101,0xfb840108,0xd0f40101,0xfb840108,0xd0f50101,
	0xfb840108,0xd0f60101,0xfb840108,0xd0f70101,0xfb840108,0xd0f80101,0xfb840108,0xd0f90101,
	0xfb840108,0xd0fa0101,0xfb840108,0xd0fb0101,0xfb840108,0xd0fc0101,0xfb840108,0xd0fd0101,
	0xfb840108,0xd0fe0101,0xfb840108,0xd0ff0101,0xfb840108,0xd1000101,0xfb840108,0xd1010101,
	0xfb840108,0xd1020101,0xfb840108,0xd1030101,0xfb840108,0xd1040101,0xfb840108,0xd1050101,
	0xfb840108,0xd1060101,0xfb840108,0xd1070101,0xfb840108,0xd1080101,0xfb840108,0xd1090101,
	0xfb840108,0xd10a0101,0xfb840108,0xd10b0101,0xfb840108,0xd10c0101,0xfb840108,0xd10d0101,
	0xfb840108,0xd10e0101,0xfb840108,0xd10f0101,0xfb840108,0xd1100101,0xfb840108,0xd1110101,
	0xfb840108,0xd1120101,0xfb840108,0xd1130101,0xfb840108,0xd1140101,0xfb840108,0xd1150101,
	0xfb840108,0xd1160101,0xfb840108,0xd1170101,0xfb840108,0xd1180101,0xfb840108,0xd1190101,
	0xfb840108,0xd11a0101,0xfb840108,0xd11b0101,0xfb840108,0xd11c0101,0xfb840108,0xd11d0101,
	0xfb840108,0xd11e0101,0xfb840108,0xd11f0101,0xfb840108,0xd1200101,0xfb840108,0xd1210101,
	0xfb840108,0xd1220101,0xfb840108,0xd1230101,0xfb840108,0xd1240101,0xfb840108,0xd1250101,
	0xfb840108,0xd1260101,0xfb840108,0xd1270101,0xfb840108,0xd1280101,0xfb840108,0xd1290101,
	0xfb840108,0xd12a0101,0xfb840108,0xd12b0101,0xfb840108,0xd12c0101,0xfb840108,0xd12d0101,
	0xfb840108,0xd12e0101,0xfb840108,0xd12f0101,0xfb840108,0xd1300101,0xfb840108,0xd1310101,
	0xfb840108,0xd1320101,0xfb840108,0xd1330101,0xfb840108,0xd1340101,0xfb840108,0xd1350101,
	0xfb840108,0xd1360101,0xfb840108,0xd1370101,0xfb840108,0xd1380101,0xfb840108,0xd1390101,
	0xfb840108,0xd13a0101,0xfb840108,0xd13b0101,0xfb840108,0xd13c0101,0xfb840108,0xd13d0101,
	0xfb840108,0xd13e0101,0xfb840108,0xd13f0101,0xfb840108,0xd1400101,0xfb840108,0xd1410101,
	0xfb840108,0xd1420101,0xfb840108,0xd1430101,0xfb840108,0xd1440101,0xfb840108,0xd1450101,
	0xfb840108,0xd1460101,0xfb840108,0xd1470101,0xfb840108,0xd1480101,0xfb840108,0xd1490101,
	0xfb840108,0xd14a0101,0xfb840108,0xd14b0101,0xfb840108,0xd14c0101,0xfb840108,0xd14d0101,
	0xfb840108,0xd14e0101,0xfb840108,0xd14f0101,0xfb840108,0xd1500101,0xfb840108,0xd1510101,
	0xfb840108,0xd1520101,0xfb840108,0xd1530101,0xfb840108,0xd1540101,0xfb840108,0xd1550101,
	0xfb840108,0xd1560101,0xfb840108,0xd1570101,0xfb840108,0xd1580101,0xfb840108,0xd1590101,
	0xfb840108,0xd15a0101,0xfb840108,0xd15b0101,0xfb840108,0xd15c0101,0xfb840108,0xd15d0101,
	0xfb840108,0xd15e0101,0xfb840108,0xd15f0101,0xfb840108,0xd1600101,0xfb840108,0xd1610101,
	0xfb840108,0xd1620101,0xfb840108,0xd1630101,0xfb840108,0xd1640101,0xfb840108,0xd1650101,
	0xfb840108,0xd1660101,0xfb840108,0xd1670101,0xfb840108,0xd1680101,0xfb840108,0xd1690101,
	0xfb840108,0xd16a0101,0xfb840108,0xd16b0101,0xfb840108,0xd16c0101,0xfb840108,0xd16d0101,
	0xfb840108,0xd16e0101,0xfb840108,0xd16f0101,0xfb840108,0xd1700101,0xfb840108,0xd1710101,
	0xfb840108,0xd1720101,0xfb840108,0xd1730101,0xfb840108,0xd1740101,0xfb840108,0xd1750101,
	0xfb840108,0xd1760101,0xfb840108,0xd1770101,0xfb840108,0xd1780101,0xfb840108,0xd1790101,
	0xfb840108,0xd17a0101,0xfb840108,0xd17b0101,0xfb840108,0xd17c0101,0xfb840108,0xd17d0101,
	0xfb840108,0xd17e0101,0xfb840108,0xd17f0101,0xfb840108,0xd1800101,0xfb840108,0xd1810101,
	0xfb840108,0xd1820101,0xfb840108,0xd1830101,0xfb840108,0xd1840101,0xfb840108,0xd1850101,
	0xfb840108,0xd1860101,0xfb840108,0xd1870101,0xfb840108,0xd1880101,0xfb840108,0xd1890101,
	0xfb840108,0xd18a0101,0xfb840108,0xd18b0101,0xfb840108,0xd18c0101,0xfb840108,0xd18d0101,
	0xfb840108,0xd18e0101,0xfb840108,0xd18f0101,0xfb840108,0xd1900101,0xfb840108,0xd1910101,
	0xfb840108,0xd1920101,0xfb840108,0xd1930101,0xfb840108,0xd1940101,0xfb840108,0xd1950101,
	0xfb840108,0xd1960101,0xfb840108,0xd1970101,0xfb840108,0xd1980101,0xfb840108,0xd1990101,
	0xfb840108,0xd19a0101,0xfb840108,0xd19b0101,0xfb840108,0xd19c0101,0xfb840108,0xd19d0101,
	0xfb840108,0xd19e0101,0xfb840108,0xd19f0101,0xfb840108,0xd1a00101,0xfb840108,0xd1a10101,
	0xfb840108,0xd1a20101,0xfb840108,0xd1a30101,0xfb840108,0xd1a40101,0xfb840108,0xd1a50101,
	0xfb840108,0xd1a60101,0xfb840108,0xd1a70101,0xfb840108,0xd1a80101,0xfb840108,0xd1a90101,
	0xfb840108,0xd1aa0101,0xfb840108,0xd1ab0101,0xfb840108,0xd1ac0101,0xfb840108,0xd1ad0101,
	0xfb840108,0xd1ae0101,0xfb840108,0xd1af0101,0xfb840108,0xd1b00101,0xfb840108,0xd1b10101,
	0xfb840108,0xd1b20101,0xfb840108,0xd1b30101,0xfb840108,0xd1b40101,0xfb840108,0xd1b50101,
	0xfb840108,0xd1b60101,0xfb840108,0xd1b70101,0xfb840108,0xd1b80101,0xfb840108,0xd1b90101,
	0xfb840108,0xd1ba0101,0xfb840108,0xd1bb0101,0xfb840108,0xd1bc0101,0xfb840108,0xd1bd0101,
	0xfb840108,0xd1be0101,0xfb840108,0xd1bf0101,0xfb840108,0xd1c00101,0xfb840108,0xd1c10101,
	0xfb840108,0xd1c20101,0xfb840108,0xd1c30101,0xfb840108,0xd1c40101,0xfb840108,0xd1c50101,
	0xfb840108,0xd1c60101,0xfb840108,0xd1c70101,0xfb840108,0xd1c80101,0xfb840108,0xd1c90101,
	0xfb840108,0xd1ca0101,0xfb840108,0xd1cb0101,0xfb840108,0xd1cc0101,0xfb840108,0xd1cd0101,
	0xfb840108,0xd1ce0101,0xfb840108,0xd1cf0101,0xfb840108,0xd1d00101,0xfb840108,0xd1d10101,
	0xfb840108,0xd1d20101,0xfb840108,0xd1d30101,0xfb840108,0xd1d40101,0xfb840108,0xd1d50101,
	0xfb840108,0xd1d60101,0xfb840108,0xd1d70101,0xfb840108,0xd1d80101,0xfb840108,0xd1d90101,
	0xfb840108,0xd1da0101,0xfb840108,0xd1db0101,0xfb840108,0xd1dc0101,0xfb840108,0xd1dd0101,
	0xfb840108,0xd1de0101,0xfb840108,0xd1df0101,0xfb840108,0xd1e00101,0xfb840108,0xd1e10101,
	0xfb840108,0xd1e20101,0xfb840108,0xd1e30101,0xfb840108,0xd1e40101,0xfb840108,0xd1e50101,
	0xfb840108,0xd1e60101,0xfb840108,0xd1e70101,0xfb840108,0xd1e80101,0xfb840108,0xd1e90101,
	0xfb840108,0xd1ea0101,0xfb840108,0xd1eb0101,0xfb840108,0xd1ec0101,0xfb840108,0xd1ed0101,
	0xfb840108,0xd1ee0101,0xfb840108,0xd1ef0101,0xfb840108,0xd1f00101,0xfb840108,0xd1f10101,
	0xfb840108,0xd1f20101,0xfb840108,0xd1f30101,0xfb840108,0xd1f40101,0xfb840108,0xd1f50101,
	0xfb840108,0xd1f60101,0xfb840108,0xd1f70101,0xfb840108,0xd1f80101,0xfb840108,0xd1f90101,
	0xfb840108,0xd1fa0101,0xfb840108,0xd1fb0101,0xfb840108,0xd1fc0101,0xfb840108,0xd1fd0101,
	0xfb840108,0xd1fe0101,0xfb840108,0xd1ff0101,0xfb840108,0xd2000101,0xfb840108,0xd2010101,
	0xfb840108,0xd2020101,0xfb840108,0xd2030101,0xfb840108,0xd2040101,0xfb840108,0xd2050101,
	0xfb840108,0xd2060101,0xfb840108,0xd2070101,0xfb840108,0xd2080101,0xfb840108,0xd2090101,
	0xfb840108,0xd20a0101,0xfb840108,0xd20b0101,0xfb840108,0xd20c0101,0xfb840108,0xd20d0101,
	0xfb840108,0xd20e0101,0xfb840108,0xd20f0101,0xfb840108,0xd2100101,0xfb840108,0xd2110101,
	0xfb840108,0xd2120101,0xfb840108,0xd2130101,0xfb840108,0xd2140101,0xfb840108,0xd2150101,
	0xfb840108,0xd2160101,0xfb840108,0xd2170101,0xfb840108,0xd2180101,0xfb840108,0xd2190101,
	0xfb840108,0xd21a0101,0xfb840108,0xd21b0101,0xfb840108,0xd21c0101,0xfb840108,0xd21d0101,
	0xfb840108,0xd21e0101,0xfb840108,0xd21f0101,0xfb840108,0xd2200101,0xfb840108,0xd2210101,
	0xfb840108,0xd2220101,0xfb840108,0xd2230101,0xfb840108,0xd2240101,0xfb840108,0xd2250101,
	0xfb840108,0xd2260101,0xfb840108,0xd2270101,0xfb840108,0xd2280101,0xfb840108,0xd2290101,
	0xfb840108,0xd22a0101,0xfb840108,0xd22b0101,0xfb840108,0xd22c0101,0xfb840108,0xd22d0101,
	0xfb840108,0xd22e0101,0xfb840108,0xd22f0101,0xfb840108,0xd2300101,0xfb840108,0xd2310101,
	0xfb840108,0xd2320101,0xfb840108,0xd2330101,0xfb840108,0xd2340101,0xfb840108,0xd2350101,
	0xfb840108,0xd2360101,0xfb840108,0xd2370101,0xfb840108,0xd2380101,0xfb840108,0xd2390101,
	0xfb840108,0xd23a0101,0xfb840108,0xd23b0101,0xfb840108,0xd23c0101,0xfb840108,0xd23d0101,
	0xfb840108,0xd23e0101,0xfb840108,0xd23f0101,0xfb840108,0xd2400101,0xfb840108,0xd2410101,
	0xfb840108,0xd2420101,0xfb840108,0xd2430101,0xfb840108,0xd2440101,0xfb840108,0xd2450101,
	0xfb840108,0xd2460101,0xfb840108,0xd2470101,0xfb840108,0xd2480101,0xfb840108,0xd2490101,
	0xfb840108,0xd24a0101,0xfb840108,0xd24b0101,0xfb840108,0xd24c0101,0xfb840108,0xd24d0101,
	0xfb840108,0xd24e0101,0xfb840108,0xd24f0101,0xfb840108,0xd2500101,0xfb840108,0xd2510101,
	0xfb840108,0xd2520101,0xfb840108,0xd2530101,0xfb840108,0xd2540101,0xfb840108,0xd2550101,
	0xfb840108,0xd2560101,0xfb840108,0xd2570101,0xfb840108,0xd2580101,0xfb840108,0xd2590101,
	0xfb840108,0xd25a0101,0xfb840108,0xd25b0101,0xfb840108,0xd25c0101,0xfb840108,0xd25d0101,
	0xfb840108,0xd25e0101,0xfb840108,0xd25f0101,0xfb840108,0xd2600101,0xfb840108,0xd2610101,
	0xfb840108,0xd2620101,0xfb840108,0xd2630101,0xfb840108,0xd2640101,0xfb840108,0xd2650101,
	0xfb840108,0xd2660101,0xfb840108,0xd2670101,0xfb840108,0xd2680101,0xfb840108,0xd2690101,
	0xfb840108,0xd26a0101,0xfb840108,0xd26b0101,0xfb840108,0xd26c0101,0xfb840108,0xd26d0101,
	0xfb840108,0xd26e0101,0xfb840108,0xd26f0101,0xfb840108,0xd2700101,0xfb840108,0xd2710101,
	0xfb840108,0xd2720101,0xfb840108,0xd2730101,0xfb840108,0xd2740101,0xfb840108,0xd2750101,
	0xfb840108,0xd2760101,0xfb840108,0xd2770101,0xfb840108,0xd2780101,0xfb840108,0xd2790101,
	0xfb840108,0xd27a0101,0xfb840108,0xd27b0101,0xfb840108,0xd27c0101,0xfb840108,0xd27d0101,
	0xfb840108,0xd27e0101,0xfb840108,0xd27f0101,0xfb840108,0xd2800101,0xfb840108,0xd2810101,
	0xfb840108,0xd2820101,0xfb840108,0xd2830101,0xfb840108,0xd2840101,0xfb840108,0xd2850101,
	0xfb840108,0xd2860101,0xfb840108,0xd2870101,0xfb840108,0xd2880101,0xfb840108,0xd2890101,
	0xfb840108,0xd28a0101,0xfb840108,0xd28b0101,0xfb840108,0xd28c0101,0xfb840108,0xd28d0101,
	0xfb840108,0xd28e0101,0xfb840108,0xd28f0101,0xfb840108,0xd2900101,0xfb840108,0xd2910101,
	0xfb840108,0xd2920101,0xfb840108,0xd2930101,0xfb840108,0xd2940101,0xfb840108,0xd2950101,
	0xfb840108,0xd2960101,0xfb840108,0xd2970101,0xfb840108,0xd2980101,0xfb840108,0xd2990101,
	0xfb840108,0xd29a0101,0xfb840108,0xd29b0101,0xfb840108,0xd29c0101,0xfb840108,0xd29d0101,
	0xfb840108,0xd29e0101,0xfb840108,0xd29f0101,0xfb840108,0xd2a00101,0xfb840108,0xd2a10101,
	0xfb840108,0xd2a20101,0xfb840108,0xd2a30101,0xfb840108,0xd2a40101,0xfb840108,0xd2a50101,
	0xfb840108,0xd2a60101,0xfb840108,0xd2a70101,0xfb840108,0xd2a80101,0xfb840108,0xd2a90101,
	0xfb840108,0xd2aa0101,0xfb840108,0xd2ab0101,0xfb840108,0xd2ac0101,0xfb840108,0xd2ad0101,
	0xfb840108,0xd2ae0101,0xfb840108,0xd2af0101,0xfb840108,0xd2b00101,0xfb840108,0xd2b10101,
	0xfb840108,0xd2b20101,0xfb840108,0xd2b30101,0xfb840108,0xd2b40101,0xfb840108,0xd2b50101,
	0xfb840108,0xd2b60101,0xfb840108,0xd2b70101,0xfb840108,0xd2b80101,0xfb840108,0xd2b90101,
	0xfb840108,0xd2ba0101,0xfb840108,0xd2bb0101,0xfb840108,0xd2bc0101,0xfb840108,0xd2bd0101,
	0xfb840108,0xd2be0101,0xfb840108,0xd2bf0101,0xfb840108,0xd2c00101,0xfb840108,0xd2c10101,
	0xfb840108,0xd2c20101,0xfb840108,0xd2c30101,0xfb840108,0xd2c40101,0xfb840108,0xd2c50101,
	0xfb840108,0xd2c60101,0xfb840108,0xd2c70101,0xfb840108,0xd2c80101,0xfb840108,0xd2c90101,
	0xfb840108,0xd2ca0101,0xfb840108,0xd2cb0101,0xfb840108,0xd2cc0101,0xfb840108,0xd2cd0101,
	0xfb840108,0xd2ce0101,0xfb840108,0xd2cf0101,0xfb840108,0xd2d00101,0xfb840108,0xd2d10101,
	0xfb840108,0xd2d20101,0xfb840108,0xd2d30101,0xfb840108,0xd2d40101,0xfb840108,0xd2d50101,
	0xfb840108,0xd2d60101,0xfb840108,0xd2d70101,0xfb840108,0xd2d80101,0xfb840108,0xd2d90101,
	0xfb840108,0xd2da0101,0xfb840108,0xd2db0101,0xfb840108,0xd2dc0101,0xfb840108,0xd2dd0101,
	0xfb840108,0xd2de0101,0xfb840108,0xd2df0101,0xfb840108,0xd2e00101,0xfb840108,0xd2e10101,
	0xfb840108,0xd2e20101,0xfb840108,0xd2e30101,0xfb840108,0xd2e40101,0xfb840108,0xd2e50101,
	0xfb840108,0xd2e60101,0xfb840108,0xd2e70101,0xfb840108,0xd2e80101,0xfb840108,0xd2e90101,
	0xfb840108,0xd2ea0101,0xfb840108,0xd2eb0101,0xfb840108,0xd2ec0101,0xfb840108,0xd2ed0101,
	0xfb840108,0xd2ee0101,0xfb840108,0xd2ef0101,0xfb840108,0xd2f00101,0xfb840108,0xd2f10101,
	0xfb840108,0xd2f20101,0xfb840108,0xd2f30101,0xfb840108,0xd2f40101,0xfb840108,0xd2f50101,
	0xfb840108,0xd2f60101,0xfb840108,0xd2f70101,0xfb840108,0xd2f80101,0xfb840108,0xd2f90101,
	0xfb840108,0xd2fa0101,0xfb840108,0xd2fb0101,0xfb840108,0xd2fc0101,0xfb840108,0xd2fd0101,
	0xfb840108,0xd2fe0101,0xfb840108,0xd2ff0101,0xfb840108,0xd3000101,0xfb840108,0xd3010101,
	0xfb840108,0xd3020101,0xfb840108,0xd3030101,0xfb840108,0xd3040101,0xfb840108,0xd3050101,
	0xfb840108,0xd3060101,0xfb840108,0xd3070101,0xfb840108,0xd3080101,0xfb840108,0xd3090101,
	0xfb840108,0xd30a0101,0xfb840108,0xd30b0101,0xfb840108,0xd30c0101,0xfb840108,0xd30d0101,
	0xfb840108,0xd30e0101,0xfb840108,0xd30f0101,0xfb840108,0xd3100101,0xfb840108,0xd3110101,
	0xfb840108,0xd3120101,0xfb840108,0xd3130101,0xfb840108,0xd3140101,0xfb840108,0xd3150101,
	0xfb840108,0xd3160101,0xfb840108,0xd3170101,0xfb840108,0xd3180101,0xfb840108,0xd3190101,
	0xfb840108,0xd31a0101,0xfb840108,0xd31b0101,0xfb840108,0xd31c0101,0xfb840108,0xd31d0101,
	0xfb840108,0xd31e0101,0xfb840108,0xd31f0101,0xfb840108,0xd3200101,0xfb840108,0xd3210101,
	0xfb840108,0xd3220101,0xfb840108,0xd3230101,0xfb840108,0xd3240101,0xfb840108,0xd3250101,
	0xfb840108,0xd3260101,0xfb840108,0xd3270101,0xfb840108,0xd3280101,0xfb840108,0xd3290101,
	0xfb840108,0xd32a0101,0xfb840108,0xd32b0101,0xfb840108,0xd32c0101,0xfb840108,0xd32d0101,
	0xfb840108,0xd32e0101,0xfb840108,0xd32f0101,0xfb840108,0xd3300101,0xfb840108,0xd3310101,
	0xfb840108,0xd3320101,0xfb840108,0xd3330101,0xfb840108,0xd3340101,0xfb840108,0xd3350101,
	0xfb840108,0xd3360101,0xfb840108,0xd3370101,0xfb840108,0xd3380101,0xfb840108,0xd3390101,
	0xfb840108,0xd33a0101,0xfb840108,0xd33b0101,0xfb840108,0xd33c0101,0xfb840108,0xd33d0101,
	0xfb840108,0xd33e0101,0xfb840108,0xd33f0101,0xfb840108,0xd3400101,0xfb840108,0xd3410101,
	0xfb840108,0xd3420101,0xfb840108,0xd3430101,0xfb840108,0xd3440101,0xfb840108,0xd3450101,
	0xfb840108,0xd3460101,0xfb840108,0xd3470101,0xfb840108,0xd3480101,0xfb840108,0xd3490101,
	0xfb840108,0xd34a0101,0xfb840108,0xd34b0101,0xfb840108,0xd34c0101,0xfb840108,0xd34d0101,
	0xfb840108,0xd34e0101,0xfb840108,0xd34f0101,0xfb840108,0xd3500101,0xfb840108,0xd3510101,
	0xfb840108,0xd3520101,0xfb840108,0xd3530101,0xfb840108,0xd3540101,0xfb840108,0xd3550101,
	0xfb840108,0xd3560101,0xfb840108,0xd3570101,0xfb840108,0xd3580101,0xfb840108,0xd3590101,
	0xfb840108,0xd35a0101,0xfb840108,0xd35b0101,0xfb840108,0xd35c0101,0xfb840108,0xd35d0101,
	0xfb840108,0xd35e0101,0xfb840108,0xd35f0101,0xfb840108,0xd3600101,0xfb840108,0xd3610101,
	0xfb840108,0xd3620101,0xfb840108,0xd3630101,0xfb840108,0xd3640101,0xfb840108,0xd3650101,
	0xfb840108,0xd3660101,0xfb840108,0xd3670101,0xfb840108,0xd3680101,0xfb840108,0xd3690101,
	0xfb840108,0xd36a0101,0xfb840108,0xd36b0101,0xfb840108,0xd36c0101,0xfb840108,0xd36d0101,
	0xfb840108,0xd36e0101,0xfb840108,0xd36f0101,0xfb840108,0xd3700101,0xfb840108,0xd3710101,
	0xfb840108,0xd3720101,0xfb840108,0xd3730101,0xfb840108,0xd3740101,0xfb840108,0xd3750101,
	0xfb840108,0xd3760101,0xfb840108,0xd3770101,0xfb840108,0xd3780101,0xfb840108,0xd3790101,
	0xfb840108,0xd37a0101,0xfb840108,0xd37b0101,0xfb840108,0xd37c0101,0xfb840108,0xd37d0101,
	0xfb840108,0xd37e0101,0xfb840108,0xd37f0101,0xfb840108,0xd3800101,0xfb840108,0xd3810101,
	0xfb840108,0xd3820101,0xfb840108,0xd3830101,0xfb840108,0xd3840101,0xfb840108,0xd3850101,
	0xfb840108,0xd3860101,0xfb840108,0xd3870101,0xfb840108,0xd3880101,0xfb840108,0xd3890101,
	0xfb840108,0xd38a0101,0xfb840108,0xd38b0101,0xfb840108,0xd38c0101,0xfb840108,0xd38d0101,
	0xfb840108,0xd38e0101,0xfb840108,0xd38f0101,0xfb840108,0xd3900101,0xfb840108,0xd3910101,
	0xfb840108,0xd3920101,0xfb840108,0xd3930101,0xfb840108,0xd3940101,0xfb840108,0xd3950101,
	0xfb840108,0xd3960101,0xfb840108,0xd3970101,0xfb840108,0xd3980101,0xfb840108,0xd3990101,
	0xfb840108,0xd39a0101,0xfb840108,0xd39b0101,0xfb840108,0xd39c0101,0xfb840108,0xd39d0101,
	0xfb840108,0xd39e0101,0xfb840108,0xd39f0101,0xfb840108,0xd3a00101,0xfb840108,0xd3a10101,
	0xfb840108,0xd3a20101,0xfb840108,0xd3a30101,0xfb840108,0xd3a40101,0xfb840108,0xd3a50101,
	0xfb840108,0xd3a60101,0xfb840108,0xd3a70101,0xfb840108,0xd3a80101,0xfb840108,0xd3a90101,
	0xfb840108,0xd3aa0101,0xfb840108,0xd3ab0101,0xfb840108,0xd3ac0101,0xfb840108,0xd3ad0101,
	0xfb840108,0xd3ae0101,0xfb840108,0xd3af0101,0xfb840108,0xd3b00101,0xfb840108,0xd3b10101,
	0xfb840108,0xd3b20101,0xfb840108,0xd3b30101,0xfb840108,0xd3b40101,0xfb840108,0xd3b50101,
	0xfb840108,0xd3b60101,0xfb840108,0xd3b70101,0xfb840108,0xd3b80101,0xfb840108,0xd3b90101,
	0xfb840108,0xd3ba0101,0xfb840108,0xd3bb0101,0xfb840108,0xd3bc0101,0xfb840108,0xd3bd0101,
	0xfb840108,0xd3be0101,0xfb840108,0xd3bf0101,0xfb840108,0xd3c00101,0xfb840108,0xd3c10101,
	0xfb840108,0xd3c20101,0xfb840108,0xd3c30101,0xfb840108,0xd3c40101,0xfb840108,0xd3c50101,
	0xfb840108,0xd3c60101,0xfb840108,0xd3c70101,0xfb840108,0xd3c80101,0xfb840108,0xd3c90101,
	0xfb840108,0xd3ca0101,0xfb840108,0xd3cb0101,0xfb840108,0xd3cc0101,0xfb840108,0xd3cd0101,
	0xfb840108,0xd3ce0101,0xfb840108,0xd3cf0101,0xfb840108,0xd3d00101,0xfb840108,0xd3d10101,
	0xfb840108,0xd3d20101,0xfb840108,0xd3d30101,0xfb840108,0xd3d40101,0xfb840108,0xd3d50101,
	0xfb840108,0xd3d60101,0xfb840108,0xd3d70101,0xfb840108,0xd3d80101,0xfb840108,0xd3d90101,
	0xfb840108,0xd3da0101,0xfb840108,0xd3db0101,0xfb840108,0xd3dc0101,0xfb840108,0xd3dd0101,
	0xfb840108,0xd3de0101,0xfb840108,0xd3df0101,0xfb840108,0xd3e00101,0xfb840108,0xd3e10101,
	0xfb840108,0xd3e20101,0xfb840108,0xd3e30101,0xfb840108,0xd3e40101,0xfb840108,0xd3e50101,
	0xfb840108,0xd3e60101,0xfb840108,0xd3e70101,0xfb840108,0xd3e80101,0xfb840108,0xd3e90101,
	0xfb840108,0xd3ea0101,0xfb840108,0xd3eb0101,0xfb840108,0xd3ec0101,0xfb840108,0xd3ed0101,
	0xfb840108,0xd3ee0101,0xfb840108,0xd3ef0101,0xfb840108,0xd3f00101,0xfb840108,0xd3f10101,
	0xfb840108,0xd3f20101,0xfb840108,0xd3f30101,0xfb840108,0xd3f40101,0xfb840108,0xd3f50101,
	0xfb840108,0xd3f60101,0xfb840108,0xd3f70101,0xfb840108,0xd3f80101,0xfb840108,0xd3f90101,
	0xfb840108,0xd3fa0101,0xfb840108,0xd3fb0101,0xfb840108,0xd3fc0101,0xfb840108,0xd3fd0101,
	0xfb840108,0xd3fe0101,0xfb840108,0xd3ff0101,0xfb840108,0xd4000101,0xfb840108,0xd4010101,
	0xfb840108,0xd4020101,0xfb840108,0xd4030101,0xfb840108,0xd4040101,0xfb840108,0xd4050101,
	0xfb840108,0xd4060101,0xfb840108,0xd4070101,0xfb840108,0xd4080101,0xfb840108,0xd4090101,
	0xfb840108,0xd40a0101,0xfb840108,0xd40b0101,0xfb840108,0xd40c0101,0xfb840108,0xd40d0101,
	0xfb840108,0xd40e0101,0xfb840108,0xd40f0101,0xfb840108,0xd4100101,0xfb840108,0xd4110101,
	0xfb840108,0xd4120101,0xfb840108,0xd4130101,0xfb840108,0xd4140101,0xfb840108,0xd4150101,
	0xfb840108,0xd4160101,0xfb840108,0xd4170101,0xfb840108,0xd4180101,0xfb840108,0xd4190101,
	0xfb840108,0xd41a0101,0xfb840108,0xd41b0101,0xfb840108,0xd41c0101,0xfb840108,0xd41d0101,
	0xfb840108,0xd41e0101,0xfb840108,0xd41f0101,0xfb840108,0xd4200101,0xfb840108,0xd4210101,
	0xfb840108,0xd4220101,0xfb840108,0xd4230101,0xfb840108,0xd4240101,0xfb840108,0xd4250101,
	0xfb840108,0xd4260101,0xfb840108,0xd4270101,0xfb840108,0xd4280101,0xfb840108,0xd4290101,
	0xfb840108,0xd42a0101,0xfb840108,0xd42b0101,0xfb840108,0xd42c0101,0xfb840108,0xd42d0101,
	0xfb840108,0xd42e0101,0xfb840108,0xd42f0101,0xfb840108,0xd4300101,0xfb840108,0xd4310101,
	0xfb840108,0xd4320101,0xfb840108,0xd4330101,0xfb840108,0xd4340101,0xfb840108,0xd4350101,
	0xfb840108,0xd4360101,0xfb840108,0xd4370101,0xfb840108,0xd4380101,0xfb840108,0xd4390101,
	0xfb840108,0xd43a0101,0xfb840108,0xd43b0101,0xfb840108,0xd43c0101,0xfb840108,0xd43d0101,
	0xfb840108,0xd43e0101,0xfb840108,0xd43f0101,0xfb840108,0xd4400101,0xfb840108,0xd4410101,
	0xfb840108,0xd4420101,0xfb840108,0xd4430101,0xfb840108,0xd4440101,0xfb840108,0xd4450101,
	0xfb840108,0xd4460101,0xfb840108,0xd4470101,0xfb840108,0xd4480101,0xfb840108,0xd4490101,
	0xfb840108,0xd44a0101,0xfb840108,0xd44b0101,0xfb840108,0xd44c0101,0xfb840108,0xd44d0101,
	0xfb840108,0xd44e0101,0xfb840108,0xd44f0101,0xfb840108,0xd4500101,0xfb840108,0xd4510101,
	0xfb840108,0xd4520101,0xfb840108,0xd4530101,0xfb840108,0xd4540101,0xfb840108,0xd4550101,
	0xfb840108,0xd4560101,0xfb840108,0xd4570101,0xfb840108,0xd4580101,0xfb840108,0xd4590101,
	0xfb840108,0xd45a0101,0xfb840108,0xd45b0101,0xfb840108,0xd45c0101,0xfb840108,0xd45d0101,
	0xfb840108,0xd45e0101,0xfb840108,0xd45f0101,0xfb840108,0xd4600101,0xfb840108,0xd4610101,
	0xfb840108,0xd4620101,0xfb840108,0xd4630101,0xfb840108,0xd4640101,0xfb840108,0xd4650101,
	0xfb840108,0xd4660101,0xfb840108,0xd4670101,0xfb840108,0xd4680101,0xfb840108,0xd4690101,
	0xfb840108,0xd46a0101,0xfb840108,0xd46b0101,0xfb840108,0xd46c0101,0xfb840108,0xd46d0101,
	0xfb840108,0xd46e0101,0xfb840108,0xd46f0101,0xfb840108,0xd4700101,0xfb840108,0xd4710101,
	0xfb840108,0xd4720101,0xfb840108,0xd4730101,0xfb840108,0xd4740101,0xfb840108,0xd4750101,
	0xfb840108,0xd4760101,0xfb840108,0xd4770101,0xfb840108,0xd4780101,0xfb840108,0xd4790101,
	0xfb840108,0xd47a0101,0xfb840108,0xd47b0101,0xfb840108,0xd47c0101,0xfb840108,0xd47d0101,
	0xfb840108,0xd47e0101,0xfb840108,0xd47f0101,0xfb840108,0xd4800101,0xfb840108,0xd4810101,
	0xfb840108,0xd4820101,0xfb840108,0xd4830101,0xfb840108,0xd4840101,0xfb840108,0xd4850101,
	0xfb840108,0xd4860101,0xfb840108,0xd4870101,0xfb840108,0xd4880101,0xfb840108,0xd4890101,
	0xfb840108,0xd48a0101,0xfb840108,0xd48b0101,0xfb840108,0xd48c0101,0xfb840108,0xd48d0101,
	0xfb840108,0xd48e0101,0xfb840108,0xd48f0101,0xfb840108,0xd4900101,0xfb840108,0xd4910101,
	0xfb840108,0xd4920101,0xfb840108,0xd4930101,0xfb840108,0xd4940101,0xfb840108,0xd4950101,
	0xfb840108,0xd4960101,0xfb840108,0xd4970101,0xfb840108,0xd4980101,0xfb840108,0xd4990101,
	0xfb840108,0xd49a0101,0xfb840108,0xd49b0101,0xfb840108,0xd49c0101,0xfb840108,0xd49d0101,
	0xfb840108,0xd49e0101,0xfb840108,0xd49f0101,0xfb840108,0xd4a00101,0xfb840108,0xd4a10101,
	0xfb840108,0xd4a20101,0xfb840108,0xd4a30101,0xfb840108,0xd4a40101,0xfb840108,0xd4a50101,
	0xfb840108,0xd4a60101,0xfb840108,0xd4a70101,0xfb840108,0xd4a80101,0xfb840108,0xd4a90101,
	0xfb840108,0xd4aa0101,0xfb840108,0xd4ab0101,0xfb840108,0xd4ac0101,0xfb840108,0xd4ad0101,
	0xfb840108,0xd4ae0101,0xfb840108,0xd4af0101,0xfb840108,0xd4b00101,0xfb840108,0xd4b10101,
	0xfb840108,0xd4b20101,0xfb840108,0xd4b30101,0xfb840108,0xd4b40101,0xfb840108,0xd4b50101,
	0xfb840108,0xd4b60101,0xfb840108,0xd4b70101,0xfb840108,0xd4b80101,0xfb840108,0xd4b90101,
	0xfb840108,0xd4ba0101,0xfb840108,0xd4bb0101,0xfb840108,0xd4bc0101,0xfb840108,0xd4bd0101,
	0xfb840108,0xd4be0101,0xfb840108,0xd4bf0101,0xfb840108,0xd4c00101,0xfb840108,0xd4c10101,
	0xfb840108,0xd4c20101,0xfb840108,0xd4c30101,0xfb840108,0xd4c40101,0xfb840108,0xd4c50101,
	0xfb840108,0xd4c60101,0xfb840108,0xd4c70101,0xfb840108,0xd4c80101,0xfb840108,0xd4c90101,
	0xfb840108,0xd4ca0101,0xfb840108,0xd4cb0101,0xfb840108,0xd4cc0101,0xfb840108,0xd4cd0101,
	0xfb840108,0xd4ce0101,0xfb840108,0xd4cf0101,0xfb840108,0xd4d00101,0xfb840108,0xd4d10101,
	0xfb840108,0xd4d20101,0xfb840108,0xd4d30101,0xfb840108,0xd4d40101,0xfb840108,0xd4d50101,
	0xfb840108,0xd4d60101,0xfb840108,0xd4d70101,0xfb840108,0xd4d80101,0xfb840108,0xd4d90101,
	0xfb840108,0xd4da0101,0xfb840108,0xd4db0101,0xfb840108,0xd4dc0101,0xfb840108,0xd4dd0101,
	0xfb840108,0xd4de0101,0xfb840108,0xd4df0101,0xfb840108,0xd4e00101,0xfb840108,0xd4e10101,
	0xfb840108,0xd4e20101,0xfb840108,0xd4e30101,0xfb840108,0xd4e40101,0xfb840108,0xd4e50101,
	0xfb840108,0xd4e60101,0xfb840108,0xd4e70101,0xfb840108,0xd4e80101,0xfb840108,0xd4e90101,
	0xfb840108,0xd4ea0101,0xfb840108,0xd4eb0101,0xfb840108,0xd4ec0101,0xfb840108,0xd4ed0101,
	0xfb840108,0xd4ee0101,0xfb840108,0xd4ef0101,0xfb840108,0xd4f00101,0xfb840108,0xd4f10101,
	0xfb840108,0xd4f20101,0xfb840108,0xd4f30101,0xfb840108,0xd4f40101,0xfb840108,0xd4f50101,
	0xfb840108,0xd4f60101,0xfb840108,0xd4f70101,0xfb840108,0xd4f80101,0xfb840108,0xd4f90101,
	0xfb840108,0xd4fa0101,0xfb840108,0xd4fb0101,0xfb840108,0xd4fc0101,0xfb840108,0xd4fd0101,
	0xfb840108,0xd4fe0101,0xfb840108,0xd4ff0101,0xfb840108,0xd5000101,0xfb840108,0xd5010101,
	0xfb840108,0xd5020101,0xfb840108,0xd5030101,0xfb840108,0xd5040101,0xfb840108,0xd5050101,
	0xfb840108,0xd5060101,0xfb840108,0xd5070101,0xfb840108,0xd5080101,0xfb840108,0xd5090101,
	0xfb840108,0xd50a0101,0xfb840108,0xd50b0101,0xfb840108,0xd50c0101,0xfb840108,0xd50d0101,
	0xfb840108,0xd50e0101,0xfb840108,0xd50f0101,0xfb840108,0xd5100101,0xfb840108,0xd5110101,
	0xfb840108,0xd5120101,0xfb840108,0xd5130101,0xfb840108,0xd5140101,0xfb840108,0xd5150101,
	0xfb840108,0xd5160101,0xfb840108,0xd5170101,0xfb840108,0xd5180101,0xfb840108,0xd5190101,
	0xfb840108,0xd51a0101,0xfb840108,0xd51b0101,0xfb840108,0xd51c0101,0xfb840108,0xd51d0101,
	0xfb840108,0xd51e0101,0xfb840108,0xd51f0101,0xfb840108,0xd5200101,0xfb840108,0xd5210101,
	0xfb840108,0xd5220101,0xfb840108,0xd5230101,0xfb840108,0xd5240101,0xfb840108,0xd5250101,
	0xfb840108,0xd5260101,0xfb840108,0xd5270101,0xfb840108,0xd5280101,0xfb840108,0xd5290101,
	0xfb840108,0xd52a0101,0xfb840108,0xd52b0101,0xfb840108,0xd52c0101,0xfb840108,0xd52d0101,
	0xfb840108,0xd52e0101,0xfb840108,0xd52f0101,0xfb840108,0xd5300101,0xfb840108,0xd5310101,
	0xfb840108,0xd5320101,0xfb840108,0xd5330101,0xfb840108,0xd5340101,0xfb840108,0xd5350101,
	0xfb840108,0xd5360101,0xfb840108,0xd5370101,0xfb840108,0xd5380101,0xfb840108,0xd5390101,
	0xfb840108,0xd53a0101,0xfb840108,0xd53b0101,0xfb840108,0xd53c0101,0xfb840108,0xd53d0101,
	0xfb840108,0xd53e0101,0xfb840108,0xd53f0101,0xfb840108,0xd5400101,0xfb840108,0xd5410101,
	0xfb840108,0xd5420101,0xfb840108,0xd5430101,0xfb840108,0xd5440101,0xfb840108,0xd5450101,
	0xfb840108,0xd5460101,0xfb840108,0xd5470101,0xfb840108,0xd5480101,0xfb840108,0xd5490101,
	0xfb840108,0xd54a0101,0xfb840108,0xd54b0101,0xfb840108,0xd54c0101,0xfb840108,0xd54d0101,
	0xfb840108,0xd54e0101,0xfb840108,0xd54f0101,0xfb840108,0xd5500101,0xfb840108,0xd5510101,
	0xfb840108,0xd5520101,0xfb840108,0xd5530101,0xfb840108,0xd5540101,0xfb840108,0xd5550101,
	0xfb840108,0xd5560101,0xfb840108,0xd5570101,0xfb840108,0xd5580101,0xfb840108,0xd5590101,
	0xfb840108,0xd55a0101,0xfb840108,0xd55b0101,0xfb840108,0xd55c0101,0xfb840108,0xd55d0101,
	0xfb840108,0xd55e0101,0xfb840108,0xd55f0101,0xfb840108,0xd5600101,0xfb840108,0xd5610101,
	0xfb840108,0xd5620101,0xfb840108,0xd5630101,0xfb840108,0xd5640101,0xfb840108,0xd5650101,
	0xfb840108,0xd5660101,0xfb840108,0xd5670101,0xfb840108,0xd5680101,0xfb840108,0xd5690101,
	0xfb840108,0xd56a0101,0xfb840108,0xd56b0101,0xfb840108,0xd56c0101,0xfb840108,0xd56d0101,
	0xfb840108,0xd56e0101,0xfb840108,0xd56f0101,0xfb840108,0xd5700101,0xfb840108,0xd5710101,
	0xfb840108,0xd5720101,0xfb840108,0xd5730101,0xfb840108,0xd5740101,0xfb840108,0xd5750101,
	0xfb840108,0xd5760101,0xfb840108,0xd5770101,0xfb840108,0xd5780101,0xfb840108,0xd5790101,
	0xfb840108,0xd57a0101,0xfb840108,0xd57b0101,0xfb840108,0xd57c0101,0xfb840108,0xd57d0101,
	0xfb840108,0xd57e0101,0xfb840108,0xd57f0101,0xfb840108,0xd5800101,0xfb840108,0xd5810101,
	0xfb840108,0xd5820101,0xfb840108,0xd5830101,0xfb840108,0xd5840101,0xfb840108,0xd5850101,
	0xfb840108,0xd5860101,0xfb840108,0xd5870101,0xfb840108,0xd5880101,0xfb840108,0xd5890101,
	0xfb840108,0xd58a0101,0xfb840108,0xd58b0101,0xfb840108,0xd58c0101,0xfb840108,0xd58d0101,
	0xfb840108,0xd58e0101,0xfb840108,0xd58f0101,0xfb840108,0xd5900101,0xfb840108,0xd5910101,
	0xfb840108,0xd5920101,0xfb840108,0xd5930101,0xfb840108,0xd5940101,0xfb840108,0xd5950101,
	0xfb840108,0xd5960101,0xfb840108,0xd5970101,0xfb840108,0xd5980101,0xfb840108,0xd5990101,
	0xfb840108,0xd59a0101,0xfb840108,0xd59b0101,0xfb840108,0xd59c0101,0xfb840108,0xd59d0101,
	0xfb840108,0xd59e0101,0xfb840108,0xd59f0101,0xfb840108,0xd5a00101,0xfb840108,0xd5a10101,
	0xfb840108,0xd5a20101,0xfb840108,0xd5a30101,0xfb840108,0xd5a40101,0xfb840108,0xd5a50101,
	0xfb840108,0xd5a60101,0xfb840108,0xd5a70101,0xfb840108,0xd5a80101,0xfb840108,0xd5a90101,
	0xfb840108,0xd5aa0101,0xfb840108,0xd5ab0101,0xfb840108,0xd5ac0101,0xfb840108,0xd5ad0101,
	0xfb840108,0xd5ae0101,0xfb840108,0xd5af0101,0xfb840108,0xd5b00101,0xfb840108,0xd5b10101,
	0xfb840108,0xd5b20101,0xfb840108,0xd5b30101,0xfb840108,0xd5b40101,0xfb840108,0xd5b50101,
	0xfb840108,0xd5b60101,0xfb840108,0xd5b70101,0xfb840108,0xd5b80101,0xfb840108,0xd5b90101,
	0xfb840108,0xd5ba0101,0xfb840108,0xd5bb0101,0xfb840108,0xd5bc0101,0xfb840108,0xd5bd0101,
	0xfb840108,0xd5be0101,0xfb840108,0xd5bf0101,0xfb840108,0xd5c00101,0xfb840108,0xd5c10101,
	0xfb840108,0xd5c20101,0xfb840108,0xd5c30101,0xfb840108,0xd5c40101,0xfb840108,0xd5c50101,
	0xfb840108,0xd5c60101,0xfb840108,0xd5c70101,0xfb840108,0xd5c80101,0xfb840108,0xd5c90101,
	0xfb840108,0xd5ca0101,0xfb840108,0xd5cb0101,0xfb840108,0xd5cc0101,0xfb840108,0xd5cd0101,
	0xfb840108,0xd5ce0101,0xfb840108,0xd5cf0101,0xfb840108,0xd5d00101,0xfb840108,0xd5d10101,
	0xfb840108,0xd5d20101,0xfb840108,0xd5d30101,0xfb840108,0xd5d40101,0xfb840108,0xd5d50101,
	0xfb840108,0xd5d60101,0xfb840108,0xd5d70101,0xfb840108,0xd5d80101,0xfb840108,0xd5d90101,
	0xfb840108,0xd5da0101,0xfb840108,0xd5db0101,0xfb840108,0xd5dc0101,0xfb840108,0xd5dd0101,
	0xfb840108,0xd5de0101,0xfb840108,0xd5df0101,0xfb840108,0xd5e00101,0xfb840108,0xd5e10101,
	0xfb840108,0xd5e20101,0xfb840108,0xd5e30101,0xfb840108,0xd5e40101,0xfb840108,0xd5e50101,
	0xfb840108,0xd5e60101,0xfb840108,0xd5e70101,0xfb840108,0xd5e80101,0xfb840108,0xd5e90101,
	0xfb840108,0xd5ea0101,0xfb840108,0xd5eb0101,0xfb840108,0xd5ec0101,0xfb840108,0xd5ed0101,
	0xfb840108,0xd5ee0101,0xfb840108,0xd5ef0101,0xfb840108,0xd5f00101,0xfb840108,0xd5f10101,
	0xfb840108,0xd5f20101,0xfb840108,0xd5f30101,0xfb840108,0xd5f40101,0xfb840108,0xd5f50101,
	0xfb840108,0xd5f60101,0xfb840108,0xd5f70101,0xfb840108,0xd5f80101,0xfb840108,0xd5f90101,
	0xfb840108,0xd5fa0101,0xfb840108,0xd5fb0101,0xfb840108,0xd5fc0101,0xfb840108,0xd5fd0101,
	0xfb840108,0xd5fe0101,0xfb840108,0xd5ff0101,0xfb840108,0xd6000101,0xfb840108,0xd6010101,
	0xfb840108,0xd6020101,0xfb840108,0xd6030101,0xfb840108,0xd6040101,0xfb840108,0xd6050101,
	0xfb840108,0xd6060101,0xfb840108,0xd6070101,0xfb840108,0xd6080101,0xfb840108,0xd6090101,
	0xfb840108,0xd60a0101,0xfb840108,0xd60b0101,0xfb840108,0xd60c0101,0xfb840108,0xd60d0101,
	0xfb840108,0xd60e0101,0xfb840108,0xd60f0101,0xfb840108,0xd6100101,0xfb840108,0xd6110101,
	0xfb840108,0xd6120101,0xfb840108,0xd6130101,0xfb840108,0xd6140101,0xfb840108,0xd6150101,
	0xfb840108,0xd6160101,0xfb840108,0xd6170101,0xfb840108,0xd6180101,0xfb840108,0xd6190101,
	0xfb840108,0xd61a0101,0xfb840108,0xd61b0101,0xfb840108,0xd61c0101,0xfb840108,0xd61d0101,
	0xfb840108,0xd61e0101,0xfb840108,0xd61f0101,0xfb840108,0xd6200101,0xfb840108,0xd6210101,
	0xfb840108,0xd6220101,0xfb840108,0xd6230101,0xfb840108,0xd6240101,0xfb840108,0xd6250101,
	0xfb840108,0xd6260101,0xfb840108,0xd6270101,0xfb840108,0xd6280101,0xfb840108,0xd6290101,
	0xfb840108,0xd62a0101,0xfb840108,0xd62b0101,0xfb840108,0xd62c0101,0xfb840108,0xd62d0101,
	0xfb840108,0xd62e0101,0xfb840108,0xd62f0101,0xfb840108,0xd6300101,0xfb840108,0xd6310101,
	0xfb840108,0xd6320101,0xfb840108,0xd6330101,0xfb840108,0xd6340101,0xfb840108,0xd6350101,
	0xfb840108,0xd6360101,0xfb840108,0xd6370101,0xfb840108,0xd6380101,0xfb840108,0xd6390101,
	0xfb840108,0xd63a0101,0xfb840108,0xd63b0101,0xfb840108,0xd63c0101,0xfb840108,0xd63d0101,
	0xfb840108,0xd63e0101,0xfb840108,0xd63f0101,0xfb840108,0xd6400101,0xfb840108,0xd6410101,
	0xfb840108,0xd6420101,0xfb840108,0xd6430101,0xfb840108,0xd6440101,0xfb840108,0xd6450101,
	0xfb840108,0xd6460101,0xfb840108,0xd6470101,0xfb840108,0xd6480101,0xfb840108,0xd6490101,
	0xfb840108,0xd64a0101,0xfb840108,0xd64b0101,0xfb840108,0xd64c0101,0xfb840108,0xd64d0101,
	0xfb840108,0xd64e0101,0xfb840108,0xd64f0101,0xfb840108,0xd6500101,0xfb840108,0xd6510101,
	0xfb840108,0xd6520101,0xfb840108,0xd6530101,0xfb840108,0xd6540101,0xfb840108,0xd6550101,
	0xfb840108,0xd6560101,0xfb840108,0xd6570101,0xfb840108,0xd6580101,0xfb840108,0xd6590101,
	0xfb840108,0xd65a0101,0xfb840108,0xd65b0101,0xfb840108,0xd65c0101,0xfb840108,0xd65d0101,
	0xfb840108,0xd65e0101,0xfb840108,0xd65f0101,0xfb840108,0xd6600101,0xfb840108,0xd6610101,
	0xfb840108,0xd6620101,0xfb840108,0xd6630101,0xfb840108,0xd6640101,0xfb840108,0xd6650101,
	0xfb840108,0xd6660101,0xfb840108,0xd6670101,0xfb840108,0xd6680101,0xfb840108,0xd6690101,
	0xfb840108,0xd66a0101,0xfb840108,0xd66b0101,0xfb840108,0xd66c0101,0xfb840108,0xd66d0101,
	0xfb840108,0xd66e0101,0xfb840108,0xd66f0101,0xfb840108,0xd6700101,0xfb840108,0xd6710101,
	0xfb840108,0xd6720101,0xfb840108,0xd6730101,0xfb840108,0xd6740101,0xfb840108,0xd6750101,
	0xfb840108,0xd6760101,0xfb840108,0xd6770101,0xfb840108,0xd6780101,0xfb840108,0xd6790101,
	0xfb840108,0xd67a0101,0xfb840108,0xd67b0101,0xfb840108,0xd67c0101,0xfb840108,0xd67d0101,
	0xfb840108,0xd67e0101,0xfb840108,0xd67f0101,0xfb840108,0xd6800101,0xfb840108,0xd6810101,
	0xfb840108,0xd6820101,0xfb840108,0xd6830101,0xfb840108,0xd6840101,0xfb840108,0xd6850101,
	0xfb840108,0xd6860101,0xfb840108,0xd6870101,0xfb840108,0xd6880101,0xfb840108,0xd6890101,
	0xfb840108,0xd68a0101,0xfb840108,0xd68b0101,0xfb840108,0xd68c0101,0xfb840108,0xd68d0101,
	0xfb840108,0xd68e0101,0xfb840108,0xd68f0101,0xfb840108,0xd6900101,0xfb840108,0xd6910101,
	0xfb840108,0xd6920101,0xfb840108,0xd6930101,0xfb840108,0xd6940101,0xfb840108,0xd6950101,
	0xfb840108,0xd6960101,0xfb840108,0xd6970101,0xfb840108,0xd6980101,0xfb840108,0xd6990101,
	0xfb840108,0xd69a0101,0xfb840108,0xd69b0101,0xfb840108,0xd69c0101,0xfb840108,0xd69d0101,
	0xfb840108,0xd69e0101,0xfb840108,0xd69f0101,0xfb840108,0xd6a00101,0xfb840108,0xd6a10101,
	0xfb840108,0xd6a20101,0xfb840108,0xd6a30101,0xfb840108,0xd6a40101,0xfb840108,0xd6a50101,
	0xfb840108,0xd6a60101,0xfb840108,0xd6a70101,0xfb840108,0xd6a80101,0xfb840108,0xd6a90101,
	0xfb840108,0xd6aa0101,0xfb840108,0xd6ab0101,0xfb840108,0xd6ac0101,0xfb840108,0xd6ad0101,
	0xfb840108,0xd6ae0101,0xfb840108,0xd6af0101,0xfb840108,0xd6b00101,0xfb840108,0xd6b10101,
	0xfb840108,0xd6b20101,0xfb840108,0xd6b30101,0xfb840108,0xd6b40101,0xfb840108,0xd6b50101,
	0xfb840108,0xd6b60101,0xfb840108,0xd6b70101,0xfb840108,0xd6b80101,0xfb840108,0xd6b90101,
	0xfb840108,0xd6ba0101,0xfb840108,0xd6bb0101,0xfb840108,0xd6bc0101,0xfb840108,0xd6bd0101,
	0xfb840108,0xd6be0101,0xfb840108,0xd6bf0101,0xfb840108,0xd6c00101,0xfb840108,0xd6c10101,
	0xfb840108,0xd6c20101,0xfb840108,0xd6c30101,0xfb840108,0xd6c40101,0xfb840108,0xd6c50101,
	0xfb840108,0xd6c60101,0xfb840108,0xd6c70101,0xfb840108,0xd6c80101,0xfb840108,0xd6c90101,
	0xfb840108,0xd6ca0101,0xfb840108,0xd6cb0101,0xfb840108,0xd6cc0101,0xfb840108,0xd6cd0101,
	0xfb840108,0xd6ce0101,0xfb840108,0xd6cf0101,0xfb840108,0xd6d00101,0xfb840108,0xd6d10101,
	0xfb840108,0xd6d20101,0xfb840108,0xd6d30101,0xfb840108,0xd6d40101,0xfb840108,0xd6d50101,
	0xfb840108,0xd6d60101,0xfb840108,0xd6d70101,0xfb840108,0xd6d80101,0xfb840108,0xd6d90101,
	0xfb840108,0xd6da0101,0xfb840108,0xd6db0101,0xfb840108,0xd6dc0101,0xfb840108,0xd6dd0101,
	0xfb840108,0xd6de0101,0xfb840108,0xd6df0101,0xfb840108,0xd6e00101,0xfb840108,0xd6e10101,
	0xfb840108,0xd6e20101,0xfb840108,0xd6e30101,0xfb840108,0xd6e40101,0xfb840108,0xd6e50101,
	0xfb840108,0xd6e60101,0xfb840108,0xd6e70101,0xfb840108,0xd6e80101,0xfb840108,0xd6e90101,
	0xfb840108,0xd6ea0101,0xfb840108,0xd6eb0101,0xfb840108,0xd6ec0101,0xfb840108,0xd6ed0101,
	0xfb840108,0xd6ee0101,0xfb840108,0xd6ef0101,0xfb840108,0xd6f00101,0xfb840108,0xd6f10101,
	0xfb840108,0xd6f20101,0xfb840108,0xd6f30101,0xfb840108,0xd6f40101,0xfb840108,0xd6f50101,
	0xfb840108,0xd6f60101,0xfb840108,0xd6f70101,0xfb840108,0xd6f80101,0xfb840108,0xd6f90101,
	0xfb840108,0xd6fa0101,0xfb840108,0xd6fb0101,0xfb840108,0xd6fc0101,0xfb840108,0xd6fd0101,
	0xfb840108,0xd6fe0101,0xfb840108,0xd6ff0101,0xfb840108,0xd7000101,0xfb840108,0xd7010101,
	0xfb840108,0xd7020101,0xfb840108,0xd7030101,0xfb840108,0xd7040101,0xfb840108,0xd7050101,
	0xfb840108,0xd7060101,0xfb840108,0xd7070101,0xfb840108,0xd7080101,0xfb840108,0xd7090101,
	0xfb840108,0xd70a0101,0xfb840108,0xd70b0101,0xfb840108,0xd70c0101,0xfb840108,0xd70d0101,
	0xfb840108,0xd70e0101,0xfb840108,0xd70f0101,0xfb840108,0xd7100101,0xfb840108,0xd7110101,
	0xfb840108,0xd7120101,0xfb840108,0xd7130101,0xfb840108,0xd7140101,0xfb840108,0xd7150101,
	0xfb840108,0xd7160101,0xfb840108,0xd7170101,0xfb840108,0xd7180101,0xfb840108,0xd7190101,
	0xfb840108,0xd71a0101,0xfb840108,0xd71b0101,0xfb840108,0xd71c0101,0xfb840108,0xd71d0101,
	0xfb840108,0xd71e0101,0xfb840108,0xd71f0101,0xfb840108,0xd7200101,0xfb840108,0xd7210101,
	0xfb840108,0xd7220101,0xfb840108,0xd7230101,0xfb840108,0xd7240101,0xfb840108,0xd7250101,
	0xfb840108,0xd7260101,0xfb840108,0xd7270101,0xfb840108,0xd7280101,0xfb840108,0xd7290101,
	0xfb840108,0xd72a0101,0xfb840108,0xd72b0101,0xfb840108,0xd72c0101,0xfb840108,0xd72d0101,
	0xfb840108,0xd72e0101,0xfb840108,0xd72f0101,0xfb840108,0xd7300101,0xfb840108,0xd7310101,
	0xfb840108,0xd7320101,0xfb840108,0xd7330101,0xfb840108,0xd7340101,0xfb840108,0xd7350101,
	0xfb840108,0xd7360101,0xfb840108,0xd7370101,0xfb840108,0xd7380101,0xfb840108,0xd7390101,
	0xfb840108,0xd73a0101,0xfb840108,0xd73b0101,0xfb840108,0xd73c0101,0xfb840108,0xd73d0101,
	0xfb840108,0xd73e0101,0xfb840108,0xd73f0101,0xfb840108,0xd7400101,0xfb840108,0xd7410101,
	0xfb840108,0xd7420101,0xfb840108,0xd7430101,0xfb840108,0xd7440101,0xfb840108,0xd7450101,
	0xfb840108,0xd7460101,0xfb840108,0xd7470101,0xfb840108,0xd7480101,0xfb840108,0xd7490101,
	0xfb840108,0xd74a0101,0xfb840108,0xd74b0101,0xfb840108,0xd74c0101,0xfb840108,0xd74d0101,
	0xfb840108,0xd74e0101,0xfb840108,0xd74f0101,0xfb840108,0xd7500101,0xfb840108,0xd7510101,
	0xfb840108,0xd7520101,0xfb840108,0xd7530101,0xfb840108,0xd7540101,0xfb840108,0xd7550101,
	0xfb840108,0xd7560101,0xfb840108,0xd7570101,0xfb840108,0xd7580101,0xfb840108,0xd7590101,
	0xfb840108,0xd75a0101,0xfb840108,0xd75b0101,0xfb840108,0xd75c0101,0xfb840108,0xd75d0101,
	0xfb840108,0xd75e0101,0xfb840108,0xd75f0101,0xfb840108,0xd7600101,0xfb840108,0xd7610101,
	0xfb840108,0xd7620101,0xfb840108,0xd7630101,0xfb840108,0xd7640101,0xfb840108,0xd7650101,
	0xfb840108,0xd7660101,0xfb840108,0xd7670101,0xfb840108,0xd7680101,0xfb840108,0xd7690101,
	0xfb840108,0xd76a0101,0xfb840108,0xd76b0101,0xfb840108,0xd76c0101,0xfb840108,0xd76d0101,
	0xfb840108,0xd76e0101,0xfb840108,0xd76f0101,0xfb840108,0xd7700101,0xfb840108,0xd7710101,
	0xfb840108,0xd7720101,0xfb840108,0xd7730101,0xfb840108,0xd7740101,0xfb840108,0xd7750101,
	0xfb840108,0xd7760101,0xfb840108,0xd7770101,0xfb840108,0xd7780101,0xfb840108,0xd7790101,
	0xfb840108,0xd77a0101,0xfb840108,0xd77b0101,0xfb840108,0xd77c0101,0xfb840108,0xd77d0101,
	0xfb840108,0xd77e0101,0xfb840108,0xd77f0101,0xfb840108,0xd7800101,0xfb840108,0xd7810101,
	0xfb840108,0xd7820101,0xfb840108,0xd7830101,0xfb840108,0xd7840101,0xfb840108,0xd7850101,
	0xfb840108,0xd7860101,0xfb840108,0xd7870101,0xfb840108,0xd7880101,0xfb840108,0xd7890101,
	0xfb840108,0xd78a0101,0xfb840108,0xd78b0101,0xfb840108,0xd78c0101,0xfb840108,0xd78d0101,
	0xfb840108,0xd78e0101,0xfb840108,0xd78f0101,0xfb840108,0xd7900101,0xfb840108,0xd7910101,
	0xfb840108,0xd7920101,0xfb840108,0xd7930101,0xfb840108,0xd7940101,0xfb840108,0xd7950101,
	0xfb840108,0xd7960101,0xfb840108,0xd7970101,0xfb840108,0xd7980101,0xfb840108,0xd7990101,
	0xfb840108,0xd79a0101,0xfb840108,0xd79b0101,0xfb840108,0xd79c0101,0xfb840108,0xd79d0101,
	0xfb840108,0xd79e0101,0xfb840108,0xd79f0101,0xfb840108,0xd7a00101,0xfb840108,0xd7a10101,
	0xfb840108,0xd7a20101,0xfb840108,0xd7a30101,0xfb840108,0xd7a40101,0xfb840108,0xd7a50101,
	0xfb840108,0xd7a60101,0xfb840108,0xd7a70101,0xfb840108,0xd7a80101,0xfb840108,0xd7a90101,
	0xfb840108,0xd7aa0101,0xfb840108,0xd7ab0101,0xfb840108,0xd7ac0101,0xfb840108,0xd7ad0101,
	0xfb840108,0xd7ae0101,0xfb840108,0xd7af0101,0xfb840108,0xd7b00101,0xfb840108,0xd7b10101,
	0xfb840108,0xd7b20101,0xfb840108,0xd7b30101,0xfb840108,0xd7b40101,0xfb840108,0xd7b50101,
	0xfb840108,0xd7b60101,0xfb840108,0xd7b70101,0xfb840108,0xd7b80101,0xfb840108,0xd7b90101,
	0xfb840108,0xd7ba0101,0xfb840108,0xd7bb0101,0xfb840108,0xd7bc0101,0xfb840108,0xd7bd0101,
	0xfb840108,0xd7be0101,0xfb840108,0xd7bf0101,0xfb840108,0xd7c00101,0xfb840108,0xd7c10101,
	0xfb840108,0xd7c20101,0xfb840108,0xd7c30101,0xfb840108,0xd7c40101,0xfb840108,0xd7c50101,
	0xfb840108,0xd7c60101,0xfb840108,0xd7c70101,0xfb840108,0xd7c80101,0xfb840108,0xd7c90101,
	0xfb840108,0xd7ca0101,0xfb840108,0xd7cb0101,0xfb840108,0xd7cc0101,0xfb840108,0xd7cd0101,
	0xfb840108,0xd7ce0101,0xfb840108,0xd7cf0101,0xfb840108,0xd7d00101,0xfb840108,0xd7d10101,
	0xfb840108,0xd7d20101,0xfb840108,0xd7d30101,0xfb840108,0xd7d40101,0xfb840108,0xd7d50101,
	0xfb840108,0xd7d60101,0xfb840108,0xd7d70101,0xfb840108,0xd7d80101,0xfb840108,0xd7d90101,
	0xfb840108,0xd7da0101,0xfb840108,0xd7db0101,0xfb840108,0xd7dc0101,0xfb840108,0xd7dd0101,
	0xfb840108,0xd7de0101,0xfb840108,0xd7df0101,0xfb840108,0xd7e00101,0xfb840108,0xd7e10101,
	0xfb840108,0xd7e20101,0xfb840108,0xd7e30101,0xfb840108,0xd7e40101,0xfb840108,0xd7e50101,
	0xfb840108,0xd7e60101,0xfb840108,0xd7e70101,0xfb840108,0xd7e80101,0xfb840108,0xd7e90101,
	0xfb840108,0xd7ea0101,0xfb840108,0xd7eb0101,0xfb840108,0xd7ec0101,0xfb840108,0xd7ed0101,
	0xfb840108,0xd7ee0101,0xfb840108,0xd7ef0101,0xfb840108,0xd7f00101,0xfb840108,0xd7f10101,
	0xfb840108,0xd7f20101,0xfb840108,0xd7f30101,0xfb840108,0xd7f40101,0xfb840108,0xd7f50101,
	0xfb840108,0xd7f60101,0xfb840108,0xd7f70101,0xfb840108,0xd7f80101,0xfb840108,0xd7f90101,
	0xfb840108,0xd7fa0101,0xfb840108,0xd7fb0101,0xfb840108,0xd7fc0101,0xfb840108,0xd7fd0101,
	0xfb840108,0xd7fe0101,0xfb840108,0xd7ff0101,0xfb840108,0xd8000101,0xfb840108,0xd8010101,
	0xfb840108,0xd8020101,0xfb840108,0xd8030101,0xfb840108,0xd8040101,0xfb840108,0xd8050101,
	0xfb840108,0xd8060101,0xfb840108,0xd8070101,0xfb840108,0xd8080101,0xfb840108,0xd8090101,
	0xfb840108,0xd80a0101,0xfb840108,0xd80b0101,0xfb840108,0xd80c0101,0xfb840108,0xd80d0101,
	0xfb840108,0xd80e0101,0xfb840108,0xd80f0101,0xfb840108,0xd8100101,0xfb840108,0xd8110101,
	0xfb840108,0xd8120101,0xfb840108,0xd8130101,0xfb840108,0xd8140101,0xfb840108,0xd8150101,
	0xfb840108,0xd8160101,0xfb840108,0xd8170101,0xfb840108,0xd8180101,0xfb840108,0xd8190101,
	0xfb840108,0xd81a0101,0xfb840108,0xd81b0101,0xfb840108,0xd81c0101,0xfb840108,0xd81d0101,
	0xfb840108,0xd81e0101,0xfb840108,0xd81f0101,0xfb840108,0xd8200101,0xfb840108,0xd8210101,
	0xfb840108,0xd8220101,0xfb840108,0xd8230101,0xfb840108,0xd8240101,0xfb840108,0xd8250101,
	0xfb840108,0xd8260101,0xfb840108,0xd8270101,0xfb840108,0xd8280101,0xfb840108,0xd8290101,
	0xfb840108,0xd82a0101,0xfb840108,0xd82b0101,0xfb840108,0xd82c0101,0xfb840108,0xd82d0101,
	0xfb840108,0xd82e0101,0xfb840108,0xd82f0101,0xfb840108,0xd8300101,0xfb840108,0xd8310101,
	0xfb840108,0xd8320101,0xfb840108,0xd8330101,0xfb840108,0xd8340101,0xfb840108,0xd8350101,
	0xfb840108,0xd8360101,0xfb840108,0xd8370101,0xfb840108,0xd8380101,0xfb840108,0xd8390101,
	0xfb840108,0xd83a0101,0xfb840108,0xd83b0101,0xfb840108,0xd83c0101,0xfb840108,0xd83d0101,
	0xfb840108,0xd83e0101,0xfb840108,0xd83f0101,0xfb840108,0xd8400101,0xfb840108,0xd8410101,
	0xfb840108,0xd8420101,0xfb840108,0xd8430101,0xfb840108,0xd8440101,0xfb840108,0xd8450101,
	0xfb840108,0xd8460101,0xfb840108,0xd8470101,0xfb840108,0xd8480101,0xfb840108,0xd8490101,
	0xfb840108,0xd84a0101,0xfb840108,0xd84b0101,0xfb840108,0xd84c0101,0xfb840108,0xd84d0101,
	0xfb840108,0xd84e0101,0xfb840108,0xd84f0101,0xfb840108,0xd8500101,0xfb840108,0xd8510101,
	0xfb840108,0xd8520101,0xfb840108,0xd8530101,0xfb840108,0xd8540101,0xfb840108,0xd8550101,
	0xfb840108,0xd8560101,0xfb840108,0xd8570101,0xfb840108,0xd8580101,0xfb840108,0xd8590101,
	0xfb840108,0xd85a0101,0xfb840108,0xd85b0101,0xfb840108,0xd85c0101,0xfb840108,0xd85d0101,
	0xfb840108,0xd85e0101,0xfb840108,0xd85f0101,0xfb840108,0xd8600101,0xfb840108,0xd8610101,
	0xfb840108,0xd8620101,0xfb840108,0xd8630101,0xfb840108,0xd8640101,0xfb840108,0xd8650101,
	0xfb840108,0xd8660101,0xfb840108,0xd8670101,0xfb840108,0xd8680101,0xfb840108,0xd8690101,
	0xfb840108,0xd86a0101,0xfb840108,0xd86b0101,0xfb840108,0xd86c0101,0xfb840108,0xd86d0101,
	0xfb840108,0xd86e0101,0xfb840108,0xd86f0101,0xfb840108,0xd8700101,0xfb840108,0xd8710101,
	0xfb840108,0xd8720101,0xfb840108,0xd8730101,0xfb840108,0xd8740101,0xfb840108,0xd8750101,
	0xfb840108,0xd8760101,0xfb840108,0xd8770101,0xfb840108,0xd8780101,0xfb840108,0xd8790101,
	0xfb840108,0xd87a0101,0xfb840108,0xd87b0101,0xfb840108,0xd87c0101,0xfb840108,0xd87d0101,
	0xfb840108,0xd87e0101,0xfb840108,0xd87f0101,0xfb840108,0xd8800101,0xfb840108,0xd8810101,
	0xfb840108,0xd8820101,0xfb840108,0xd8830101,0xfb840108,0xd8840101,0xfb840108,0xd8850101,
	0xfb840108,0xd8860101,0xfb840108,0xd8870101,0xfb840108,0xd8880101,0xfb840108,0xd8890101,
	0xfb840108,0xd88a0101,0xfb840108,0xd88b0101,0xfb840108,0xd88c0101,0xfb840108,0xd88d0101,
	0xfb840108,0xd88e0101,0xfb840108,0xd88f0101,0xfb840108,0xd8900101,0xfb840108,0xd8910101,
	0xfb840108,0xd8920101,0xfb840108,0xd8930101,0xfb840108,0xd8940101,0xfb840108,0xd8950101,
	0xfb840108,0xd8960101,0xfb840108,0xd8970101,0xfb840108,0xd8980101,0xfb840108,0xd8990101,
	0xfb840108,0xd89a0101,0xfb840108,0xd89b0101,0xfb840108,0xd89c0101,0xfb840108,0xd89d0101,
	0xfb840108,0xd89e0101,0xfb840108,0xd89f0101,0xfb840108,0xd8a00101,0xfb840108,0xd8a10101,
	0xfb840108,0xd8a20101,0xfb840108,0xd8a30101,0xfb840108,0xd8a40101,0xfb840108,0xd8a50101,
	0xfb840108,0xd8a60101,0xfb840108,0xd8a70101,0xfb840108,0xd8a80101,0xfb840108,0xd8a90101,
	0xfb840108,0xd8aa0101,0xfb840108,0xd8ab0101,0xfb840108,0xd8ac0101,0xfb840108,0xd8ad0101,
	0xfb840108,0xd8ae0101,0xfb840108,0xd8af0101,0xfb840108,0xd8b00101,0xfb840108,0xd8b10101,
	0xfb840108,0xd8b20101,0xfb840108,0xd8b30101,0xfb840108,0xd8b40101,0xfb840108,0xd8b50101,
	0xfb840108,0xd8b60101,0xfb840108,0xd8b70101,0xfb840108,0xd8b80101,0xfb840108,0xd8b90101,
	0xfb840108,0xd8ba0101,0xfb840108,0xd8bb0101,0xfb840108,0xd8bc0101,0xfb840108,0xd8bd0101,
	0xfb840108,0xd8be0101,0xfb840108,0xd8bf0101,0xfb840108,0xd8c00101,0xfb840108,0xd8c10101,
	0xfb840108,0xd8c20101,0xfb840108,0xd8c30101,0xfb840108,0xd8c40101,0xfb840108,0xd8c50101,
	0xfb840108,0xd8c60101,0xfb840108,0xd8c70101,0xfb840108,0xd8c80101,0xfb840108,0xd8c90101,
	0xfb840108,0xd8ca0101,0xfb840108,0xd8cb0101,0xfb840108,0xd8cc0101,0xfb840108,0xd8cd0101,
	0xfb840108,0xd8ce0101,0xfb840108,0xd8cf0101,0xfb840108,0xd8d00101,0xfb840108,0xd8d10101,
	0xfb840108,0xd8d20101,0xfb840108,0xd8d30101,0xfb840108,0xd8d40101,0xfb840108,0xd8d50101,
	0xfb840108,0xd8d60101,0xfb840108,0xd8d70101,0xfb840108,0xd8d80101,0xfb840108,0xd8d90101,
	0xfb840108,0xd8da0101,0xfb840108,0xd8db0101,0xfb840108,0xd8dc0101,0xfb840108,0xd8dd0101,
	0xfb840108,0xd8de0101,0xfb840108,0xd8df0101,0xfb840108,0xd8e00101,0xfb840108,0xd8e10101,
	0xfb840108,0xd8e20101,0xfb840108,0xd8e30101,0xfb840108,0xd8e40101,0xfb840108,0xd8e50101,
	0xfb840108,0xd8e60101,0xfb840108,0xd8e70101,0xfb840108,0xd8e80101,0xfb840108,0xd8e90101,
	0xfb840108,0xd8ea0101,0xfb840108,0xd8eb0101,0xfb840108,0xd8ec0101,0xfb840108,0xd8ed0101,
	0xfb840108,0xd8ee0101,0xfb840108,0xd8ef0101,0xfb840108,0xd8f00101,0xfb840108,0xd8f10101,
	0xfb840108,0xd8f20101,0xfb840108,0xd8f30101,0xfb840108,0xd8f40101,0xfb840108,0xd8f50101,
	0xfb840108,0xd8f60101,0xfb840108,0xd8f70101,0xfb840108,0xd8f80101,0xfb840108,0xd8f90101,
	0xfb840108,0xd8fa0101,0xfb840108,0xd8fb0101,0xfb840108,0xd8fc0101,0xfb840108,0xd8fd0101,
	0xfb840108,0xd8fe0101,0xfb840108,0xd8ff0101,0xfb840108,0xd9000101,0xfb840108,0xd9010101,
	0xfb840108,0xd9020101,0xfb840108,0xd9030101,0xfb840108,0xd9040101,0xfb840108,0xd9050101,
	0xfb840108,0xd9060101,0xfb840108,0xd9070101,0xfb840108,0xd9080101,0xfb840108,0xd9090101,
	0xfb840108,0xd90a0101,0xfb840108,0xd90b0101,0xfb840108,0xd90c0101,0xfb840108,0xd90d0101,
	0xfb840108,0xd90e0101,0xfb840108,0xd90f0101,0xfb840108,0xd9100101,0xfb840108,0xd9110101,
	0xfb840108,0xd9120101,0xfb840108,0xd9130101,0xfb840108,0xd9140101,0xfb840108,0xd9150101,
	0xfb840108,0xd9160101,0xfb840108,0xd9170101,0xfb840108,0xd9180101,0xfb840108,0xd9190101,
	0xfb840108,0xd91a0101,0xfb840108,0xd91b0101,0xfb840108,0xd91c0101,0xfb840108,0xd91d0101,
	0xfb840108,0xd91e0101,0xfb840108,0xd91f0101,0xfb840108,0xd9200101,0xfb840108,0xd9210101,
	0xfb840108,0xd9220101,0xfb840108,0xd9230101,0xfb840108,0xd9240101,0xfb840108,0xd9250101,
	0xfb840108,0xd9260101,0xfb840108,0xd9270101,0xfb840108,0xd9280101,0xfb840108,0xd9290101,
	0xfb840108,0xd92a0101,0xfb840108,0xd92b0101,0xfb840108,0xd92c0101,0xfb840108,0xd92d0101,
	0xfb840108,0xd92e0101,0xfb840108,0xd92f0101,0xfb840108,0xd9300101,0xfb840108,0xd9310101,
	0xfb840108,0xd9320101,0xfb840108,0xd9330101,0xfb840108,0xd9340101,0xfb840108,0xd9350101,
	0xfb840108,0xd9360101,0xfb840108,0xd9370101,0xfb840108,0xd9380101,0xfb840108,0xd9390101,
	0xfb840108,0xd93a0101,0xfb840108,0xd93b0101,0xfb840108,0xd93c0101,0xfb840108,0xd93d0101,
	0xfb840108,0xd93e0101,0xfb840108,0xd93f0101,0xfb840108,0xd9400101,0xfb840108,0xd9410101,
	0xfb840108,0xd9420101,0xfb840108,0xd9430101,0xfb840108,0xd9440101,0xfb840108,0xd9450101,
	0xfb840108,0xd9460101,0xfb840108,0xd9470101,0xfb840108,0xd9480101,0xfb840108,0xd9490101,
	0xfb840108,0xd94a0101,0xfb840108,0xd94b0101,0xfb840108,0xd94c0101,0xfb840108,0xd94d0101,
	0xfb840108,0xd94e0101,0xfb840108,0xd94f0101,0xfb840108,0xd9500101,0xfb840108,0xd9510101,
	0xfb840108,0xd9520101,0xfb840108,0xd9530101,0xfb840108,0xd9540101,0xfb840108,0xd9550101,
	0xfb840108,0xd9560101,0xfb840108,0xd9570101,0xfb840108,0xd9580101,0xfb840108,0xd9590101,
	0xfb840108,0xd95a0101,0xfb840108,0xd95b0101,0xfb840108,0xd95c0101,0xfb840108,0xd95d0101,
	0xfb840108,0xd95e0101,0xfb840108,0xd95f0101,0xfb840108,0xd9600101,0xfb840108,0xd9610101,
	0xfb840108,0xd9620101,0xfb840108,0xd9630101,0xfb840108,0xd9640101,0xfb840108,0xd9650101,
	0xfb840108,0xd9660101,0xfb840108,0xd9670101,0xfb840108,0xd9680101,0xfb840108,0xd9690101,
	0xfb840108,0xd96a0101,0xfb840108,0xd96b0101,0xfb840108,0xd96c0101,0xfb840108,0xd96d0101,
	0xfb840108,0xd96e0101,0xfb840108,0xd96f0101,0xfb840108,0xd9700101,0xfb840108,0xd9710101,
	0xfb840108,0xd9720101,0xfb840108,0xd9730101,0xfb840108,0xd9740101,0xfb840108,0xd9750101,
	0xfb840108,0xd9760101,0xfb840108,0xd9770101,0xfb840108,0xd9780101,0xfb840108,0xd9790101,
	0xfb840108,0xd97a0101,0xfb840108,0xd97b0101,0xfb840108,0xd97c0101,0xfb840108,0xd97d0101,
	0xfb840108,0xd97e0101,0xfb840108,0xd97f0101,0xfb840108,0xd9800101,0xfb840108,0xd9810101,
	0xfb840108,0xd9820101,0xfb840108,0xd9830101,0xfb840108,0xd9840101,0xfb840108,0xd9850101,
	0xfb840108,0xd9860101,0xfb840108,0xd9870101,0xfb840108,0xd9880101,0xfb840108,0xd9890101,
	0xfb840108,0xd98a0101,0xfb840108,0xd98b0101,0xfb840108,0xd98c0101,0xfb840108,0xd98d0101,
	0xfb840108,0xd98e0101,0xfb840108,0xd98f0101,0xfb840108,0xd9900101,0xfb840108,0xd9910101,
	0xfb840108,0xd9920101,0xfb840108,0xd9930101,0xfb840108,0xd9940101,0xfb840108,0xd9950101,
	0xfb840108,0xd9960101,0xfb840108,0xd9970101,0xfb840108,0xd9980101,0xfb840108,0xd9990101,
	0xfb840108,0xd99a0101,0xfb840108,0xd99b0101,0xfb840108,0xd99c0101,0xfb840108,0xd99d0101,
	0xfb840108,0xd99e0101,0xfb840108,0xd99f0101,0xfb840108,0xd9a00101,0xfb840108,0xd9a10101,
	0xfb840108,0xd9a20101,0xfb840108,0xd9a30101,0xfb840108,0xd9a40101,0xfb840108,0xd9a50101,
	0xfb840108,0xd9a60101,0xfb840108,0xd9a70101,0xfb840108,0xd9a80101,0xfb840108,0xd9a90101,
	0xfb840108,0xd9aa0101,0xfb840108,0xd9ab0101,0xfb840108,0xd9ac0101,0xfb840108,0xd9ad0101,
	0xfb840108,0xd9ae0101,0xfb840108,0xd9af0101,0xfb840108,0xd9b00101,0xfb840108,0xd9b10101,
	0xfb840108,0xd9b20101,0xfb840108,0xd9b30101,0xfb840108,0xd9b40101,0xfb840108,0xd9b50101,
	0xfb840108,0xd9b60101,0xfb840108,0xd9b70101,0xfb840108,0xd9b80101,0xfb840108,0xd9b90101,
	0xfb840108,0xd9ba0101,0xfb840108,0xd9bb0101,0xfb840108,0xd9bc0101,0xfb840108,0xd9bd0101,
	0xfb840108,0xd9be0101,0xfb840108,0xd9bf0101,0xfb840108,0xd9c00101,0xfb840108,0xd9c10101,
	0xfb840108,0xd9c20101,0xfb840108,0xd9c30101,0xfb840108,0xd9c40101,0xfb840108,0xd9c50101,
	0xfb840108,0xd9c60101,0xfb840108,0xd9c70101,0xfb840108,0xd9c80101,0xfb840108,0xd9c90101,
	0xfb840108,0xd9ca0101,0xfb840108,0xd9cb0101,0xfb840108,0xd9cc0101,0xfb840108,0xd9cd0101,
	0xfb840108,0xd9ce0101,0xfb840108,0xd9cf0101,0xfb840108,0xd9d00101,0xfb840108,0xd9d10101,
	0xfb840108,0xd9d20101,0xfb840108,0xd9d30101,0xfb840108,0xd9d40101,0xfb840108,0xd9d50101,
	0xfb840108,0xd9d60101,0xfb840108,0xd9d70101,0xfb840108,0xd9d80101,0xfb840108,0xd9d90101,
	0xfb840108,0xd9da0101,0xfb840108,0xd9db0101,0xfb840108,0xd9dc0101,0xfb840108,0xd9dd0101,
	0xfb840108,0xd9de0101,0xfb840108,0xd9df0101,0xfb840108,0xd9e00101,0xfb840108,0xd9e10101,
	0xfb840108,0xd9e20101,0xfb840108,0xd9e30101,0xfb840108,0xd9e40101,0xfb840108,0xd9e50101,
	0xfb840108,0xd9e60101,0xfb840108,0xd9e70101,0xfb840108,0xd9e80101,0xfb840108,0xd9e90101,
	0xfb840108,0xd9ea0101,0xfb840108,0xd9eb0101,0xfb840108,0xd9ec0101,0xfb840108,0xd9ed0101,
	0xfb840108,0xd9ee0101,0xfb840108,0xd9ef0101,0xfb840108,0xd9f00101,0xfb840108,0xd9f10101,
	0xfb840108,0xd9f20101,0xfb840108,0xd9f30101,0xfb840108,0xd9f40101,0xfb840108,0xd9f50101,
	0xfb840108,0xd9f60101,0xfb840108,0xd9f70101,0xfb840108,0xd9f80101,0xfb840108,0xd9f90101,
	0xfb840108,0xd9fa0101,0xfb840108,0xd9fb0101,0xfb840108,0xd9fc0101,0xfb840108,0xd9fd0101,
	0xfb840108,0xd9fe0101,0xfb840108,0xd9ff0101,0xfb840108,0xda000101,0xfb840108,0xda010101,
	0xfb840108,0xda020101,0xfb840108,0xda030101,0xfb840108,0xda040101,0xfb840108,0xda050101,
	0xfb840108,0xda060101,0xfb840108,0xda070101,0xfb840108,0xda080101,0xfb840108,0xda090101,
	0xfb840108,0xda0a0101,0xfb840108,0xda0b0101,0xfb840108,0xda0c0101,0xfb840108,0xda0d0101,
	0xfb840108,0xda0e0101,0xfb840108,0xda0f0101,0xfb840108,0xda100101,0xfb840108,0xda110101,
	0xfb840108,0xda120101,0xfb840108,0xda130101,0xfb840108,0xda140101,0xfb840108,0xda150101,
	0xfb840108,0xda160101,0xfb840108,0xda170101,0xfb840108,0xda180101,0xfb840108,0xda190101,
	0xfb840108,0xda1a0101,0xfb840108,0xda1b0101,0xfb840108,0xda1c0101,0xfb840108,0xda1d0101,
	0xfb840108,0xda1e0101,0xfb840108,0xda1f0101,0xfb840108,0xda200101,0xfb840108,0xda210101,
	0xfb840108,0xda220101,0xfb840108,0xda230101,0xfb840108,0xda240101,0xfb840108,0xda250101,
	0xfb840108,0xda260101,0xfb840108,0xda270101,0xfb840108,0xda280101,0xfb840108,0xda290101,
	0xfb840108,0xda2a0101,0xfb840108,0xda2b0101,0xfb840108,0xda2c0101,0xfb840108,0xda2d0101,
	0xfb840108,0xda2e0101,0xfb840108,0xda2f0101,0xfb840108,0xda300101,0xfb840108,0xda310101,
	0xfb840108,0xda320101,0xfb840108,0xda330101,0xfb840108,0xda340101,0xfb840108,0xda350101,
	0xfb840108,0xda360101,0xfb840108,0xda370101,0xfb840108,0xda380101,0xfb840108,0xda390101,
	0xfb840108,0xda3a0101,0xfb840108,0xda3b0101,0xfb840108,0xda3c0101,0xfb840108,0xda3d0101,
	0xfb840108,0xda3e0101,0xfb840108,0xda3f0101,0xfb840108,0xda400101,0xfb840108,0xda410101,
	0xfb840108,0xda420101,0xfb840108,0xda430101,0xfb840108,0xda440101,0xfb840108,0xda450101,
	0xfb840108,0xda460101,0xfb840108,0xda470101,0xfb840108,0xda480101,0xfb840108,0xda490101,
	0xfb840108,0xda4a0101,0xfb840108,0xda4b0101,0xfb840108,0xda4c0101,0xfb840108,0xda4d0101,
	0xfb840108,0xda4e0101,0xfb840108,0xda4f0101,0xfb840108,0xda500101,0xfb840108,0xda510101,
	0xfb840108,0xda520101,0xfb840108,0xda530101,0xfb840108,0xda540101,0xfb840108,0xda550101,
	0xfb840108,0xda560101,0xfb840108,0xda570101,0xfb840108,0xda580101,0xfb840108,0xda590101,
	0xfb840108,0xda5a0101,0xfb840108,0xda5b0101,0xfb840108,0xda5c0101,0xfb840108,0xda5d0101,
	0xfb840108,0xda5e0101,0xfb840108,0xda5f0101,0xfb840108,0xda600101,0xfb840108,0xda610101,
	0xfb840108,0xda620101,0xfb840108,0xda630101,0xfb840108,0xda640101,0xfb840108,0xda650101,
	0xfb840108,0xda660101,0xfb840108,0xda670101,0xfb840108,0xda680101,0xfb840108,0xda690101,
	0xfb840108,0xda6a0101,0xfb840108,0xda6b0101,0xfb840108,0xda6c0101,0xfb840108,0xda6d0101,
	0xfb840108,0xda6e0101,0xfb840108,0xda6f0101,0xfb840108,0xda700101,0xfb840108,0xda710101,
	0xfb840108,0xda720101,0xfb840108,0xda730101,0xfb840108,0xda740101,0xfb840108,0xda750101,
	0xfb840108,0xda760101,0xfb840108,0xda770101,0xfb840108,0xda780101,0xfb840108,0xda790101,
	0xfb840108,0xda7a0101,0xfb840108,0xda7b0101,0xfb840108,0xda7c0101,0xfb840108,0xda7d0101,
	0xfb840108,0xda7e0101,0xfb840108,0xda7f0101,0xfb840108,0xda800101,0xfb840108,0xda810101,
	0xfb840108,0xda820101,0xfb840108,0xda830101,0xfb840108,0xda840101,0xfb840108,0xda850101,
	0xfb840108,0xda860101,0xfb840108,0xda870101,0xfb840108,0xda880101,0xfb840108,0xda890101,
	0xfb840108,0xda8a0101,0xfb840108,0xda8b0101,0xfb840108,0xda8c0101,0xfb840108,0xda8d0101,
	0xfb840108,0xda8e0101,0xfb840108,0xda8f0101,0xfb840108,0xda900101,0xfb840108,0xda910101,
	0xfb840108,0xda920101,0xfb840108,0xda930101,0xfb840108,0xda940101,0xfb840108,0xda950101,
	0xfb840108,0xda960101,0xfb840108,0xda970101,0xfb840108,0xda980101,0xfb840108,0xda990101,
	0xfb840108,0xda9a0101,0xfb840108,0xda9b0101,0xfb840108,0xda9c0101,0xfb840108,0xda9d0101,
	0xfb840108,0xda9e0101,0xfb840108,0xda9f0101,0xfb840108,0xdaa00101,0xfb840108,0xdaa10101,
	0xfb840108,0xdaa20101,0xfb840108,0xdaa30101,0xfb840108,0xdaa40101,0xfb840108,0xdaa50101,
	0xfb840108,0xdaa60101,0xfb840108,0xdaa70101,0xfb840108,0xdaa80101,0xfb840108,0xdaa90101,
	0xfb840108,0xdaaa0101,0xfb840108,0xdaab0101,0xfb840108,0xdaac0101,0xfb840108,0xdaad0101,
	0xfb840108,0xdaae0101,0xfb840108,0xdaaf0101,0xfb840108,0xdab00101,0xfb840108,0xdab10101,
	0xfb840108,0xdab20101,0xfb840108,0xdab30101,0xfb840108,0xdab40101,0xfb840108,0xdab50101,
	0xfb840108,0xdab60101,0xfb840108,0xdab70101,0xfb840108,0xdab80101,0xfb840108,0xdab90101,
	0xfb840108,0xdaba0101,0xfb840108,0xdabb0101,0xfb840108,0xdabc0101,0xfb840108,0xdabd0101,
	0xfb840108,0xdabe0101,0xfb840108,0xdabf0101,0xfb840108,0xdac00101,0xfb840108,0xdac10101,
	0xfb840108,0xdac20101,0xfb840108,0xdac30101,0xfb840108,0xdac40101,0xfb840108,0xdac50101,
	0xfb840108,0xdac60101,0xfb840108,0xdac70101,0xfb840108,0xdac80101,0xfb840108,0xdac90101,
	0xfb840108,0xdaca0101,0xfb840108,0xdacb0101,0xfb840108,0xdacc0101,0xfb840108,0xdacd0101,
	0xfb840108,0xdace0101,0xfb840108,0xdacf0101,0xfb840108,0xdad00101,0xfb840108,0xdad10101,
	0xfb840108,0xdad20101,0xfb840108,0xdad30101,0xfb840108,0xdad40101,0xfb840108,0xdad50101,
	0xfb840108,0xdad60101,0xfb840108,0xdad70101,0xfb840108,0xdad80101,0xfb840108,0xdad90101,
	0xfb840108,0xdada0101,0xfb840108,0xdadb0101,0xfb840108,0xdadc0101,0xfb840108,0xdadd0101,
	0xfb840108,0xdade0101,0xfb840108,0xdadf0101,0xfb840108,0xdae00101,0xfb840108,0xdae10101,
	0xfb840108,0xdae20101,0xfb840108,0xdae30101,0xfb840108,0xdae40101,0xfb840108,0xdae50101,
	0xfb840108,0xdae60101,0xfb840108,0xdae70101,0xfb840108,0xdae80101,0xfb840108,0xdae90101,
	0xfb840108,0xdaea0101,0xfb840108,0xdaeb0101,0xfb840108,0xdaec0101,0xfb840108,0xdaed0101,
	0xfb840108,0xdaee0101,0xfb840108,0xdaef0101,0xfb840108,0xdaf00101,0xfb840108,0xdaf10101,
	0xfb840108,0xdaf20101,0xfb840108,0xdaf30101,0xfb840108,0xdaf40101,0xfb840108,0xdaf50101,
	0xfb840108,0xdaf60101,0xfb840108,0xdaf70101,0xfb840108,0xdaf80101,0xfb840108,0xdaf90101,
	0xfb840108,0xdafa0101,0xfb840108,0xdafb0101,0xfb840108,0xdafc0101,0xfb840108,0xdafd0101,
	0xfb840108,0xdafe0101,0xfb840108,0xdaff0101,0xfb840108,0xdb000101,0xfb840108,0xdb010101,
	0xfb840108,0xdb020101,0xfb840108,0xdb030101,0xfb840108,0xdb040101,0xfb840108,0xdb050101,
	0xfb840108,0xdb060101,0xfb840108,0xdb070101,0xfb840108,0xdb080101,0xfb840108,0xdb090101,
	0xfb840108,0xdb0a0101,0xfb840108,0xdb0b0101,0xfb840108,0xdb0c0101,0xfb840108,0xdb0d0101,
	0xfb840108,0xdb0e0101,0xfb840108,0xdb0f0101,0xfb840108,0xdb100101,0xfb840108,0xdb110101,
	0xfb840108,0xdb120101,0xfb840108,0xdb130101,0xfb840108,0xdb140101,0xfb840108,0xdb150101,
	0xfb840108,0xdb160101,0xfb840108,0xdb170101,0xfb840108,0xdb180101,0xfb840108,0xdb190101,
	0xfb840108,0xdb1a0101,0xfb840108,0xdb1b0101,0xfb840108,0xdb1c0101,0xfb840108,0xdb1d0101,
	0xfb840108,0xdb1e0101,0xfb840108,0xdb1f0101,0xfb840108,0xdb200101,0xfb840108,0xdb210101,
	0xfb840108,0xdb220101,0xfb840108,0xdb230101,0xfb840108,0xdb240101,0xfb840108,0xdb250101,
	0xfb840108,0xdb260101,0xfb840108,0xdb270101,0xfb840108,0xdb280101,0xfb840108,0xdb290101,
	0xfb840108,0xdb2a0101,0xfb840108,0xdb2b0101,0xfb840108,0xdb2c0101,0xfb840108,0xdb2d0101,
	0xfb840108,0xdb2e0101,0xfb840108,0xdb2f0101,0xfb840108,0xdb300101,0xfb840108,0xdb310101,
	0xfb840108,0xdb320101,0xfb840108,0xdb330101,0xfb840108,0xdb340101,0xfb840108,0xdb350101,
	0xfb840108,0xdb360101,0xfb840108,0xdb370101,0xfb840108,0xdb380101,0xfb840108,0xdb390101,
	0xfb840108,0xdb3a0101,0xfb840108,0xdb3b0101,0xfb840108,0xdb3c0101,0xfb840108,0xdb3d0101,
	0xfb840108,0xdb3e0101,0xfb840108,0xdb3f0101,0xfb840108,0xdb400101,0xfb840108,0xdb410101,
	0xfb840108,0xdb420101,0xfb840108,0xdb430101,0xfb840108,0xdb440101,0xfb840108,0xdb450101,
	0xfb840108,0xdb460101,0xfb840108,0xdb470101,0xfb840108,0xdb480101,0xfb840108,0xdb490101,
	0xfb840108,0xdb4a0101,0xfb840108,0xdb4b0101,0xfb840108,0xdb4c0101,0xfb840108,0xdb4d0101,
	0xfb840108,0xdb4e0101,0xfb840108,0xdb4f0101,0xfb840108,0xdb500101,0xfb840108,0xdb510101,
	0xfb840108,0xdb520101,0xfb840108,0xdb530101,0xfb840108,0xdb540101,0xfb840108,0xdb550101,
	0xfb840108,0xdb560101,0xfb840108,0xdb570101,0xfb840108,0xdb580101,0xfb840108,0xdb590101,
	0xfb840108,0xdb5a0101,0xfb840108,0xdb5b0101,0xfb840108,0xdb5c0101,0xfb840108,0xdb5d0101,
	0xfb840108,0xdb5e0101,0xfb840108,0xdb5f0101,0xfb840108,0xdb600101,0xfb840108,0xdb610101,
	0xfb840108,0xdb620101,0xfb840108,0xdb630101,0xfb840108,0xdb640101,0xfb840108,0xdb650101,
	0xfb840108,0xdb660101,0xfb840108,0xdb670101,0xfb840108,0xdb680101,0xfb840108,0xdb690101,
	0xfb840108,0xdb6a0101,0xfb840108,0xdb6b0101,0xfb840108,0xdb6c0101,0xfb840108,0xdb6d0101,
	0xfb840108,0xdb6e0101,0xfb840108,0xdb6f0101,0xfb840108,0xdb700101,0xfb840108,0xdb710101,
	0xfb840108,0xdb720101,0xfb840108,0xdb730101,0xfb840108,0xdb740101,0xfb840108,0xdb750101,
	0xfb840108,0xdb760101,0xfb840108,0xdb770101,0xfb840108,0xdb780101,0xfb840108,0xdb790101,
	0xfb840108,0xdb7a0101,0xfb840108,0xdb7b0101,0xfb840108,0xdb7c0101,0xfb840108,0xdb7d0101,
	0xfb840108,0xdb7e0101,0xfb840108,0xdb7f0101,0xfb840108,0xdb800101,0xfb840108,0xdb810101,
	0xfb840108,0xdb820101,0xfb840108,0xdb830101,0xfb840108,0xdb840101,0xfb840108,0xdb850101,
	0xfb840108,0xdb860101,0xfb840108,0xdb870101,0xfb840108,0xdb880101,0xfb840108,0xdb890101,
	0xfb840108,0xdb8a0101,0xfb840108,0xdb8b0101,0xfb840108,0xdb8c0101,0xfb840108,0xdb8d0101,
	0xfb840108,0xdb8e0101,0xfb840108,0xdb8f0101,0xfb840108,0xdb900101,0xfb840108,0xdb910101,
	0xfb840108,0xdb920101,0xfb840108,0xdb930101,0xfb840108,0xdb940101,0xfb840108,0xdb950101,
	0xfb840108,0xdb960101,0xfb840108,0xdb970101,0xfb840108,0xdb980101,0xfb840108,0xdb990101,
	0xfb840108,0xdb9a0101,0xfb840108,0xdb9b0101,0xfb840108,0xdb9c0101,0xfb840108,0xdb9d0101,
	0xfb840108,0xdb9e0101,0xfb840108,0xdb9f0101,0xfb840108,0xdba00101,0xfb840108,0xdba10101,
	0xfb840108,0xdba20101,0xfb840108,0xdba30101,0xfb840108,0xdba40101,0xfb840108,0xdba50101,
	0xfb840108,0xdba60101,0xfb840108,0xdba70101,0xfb840108,0xdba80101,0xfb840108,0xdba90101,
	0xfb840108,0xdbaa0101,0xfb840108,0xdbab0101,0xfb840108,0xdbac0101,0xfb840108,0xdbad0101,
	0xfb840108,0xdbae0101,0xfb840108,0xdbaf0101,0xfb840108,0xdbb00101,0xfb840108,0xdbb10101,
	0xfb840108,0xdbb20101,0xfb840108,0xdbb30101,0xfb840108,0xdbb40101,0xfb840108,0xdbb50101,
	0xfb840108,0xdbb60101,0xfb840108,0xdbb70101,0xfb840108,0xdbb80101,0xfb840108,0xdbb90101,
	0xfb840108,0xdbba0101,0xfb840108,0xdbbb0101,0xfb840108,0xdbbc0101,0xfb840108,0xdbbd0101,
	0xfb840108,0xdbbe0101,0xfb840108,0xdbbf0101,0xfb840108,0xdbc00101,0xfb840108,0xdbc10101,
	0xfb840108,0xdbc20101,0xfb840108,0xdbc30101,0xfb840108,0xdbc40101,0xfb840108,0xdbc50101,
	0xfb840108,0xdbc60101,0xfb840108,0xdbc70101,0xfb840108,0xdbc80101,0xfb840108,0xdbc90101,
	0xfb840108,0xdbca0101,0xfb840108,0xdbcb0101,0xfb840108,0xdbcc0101,0xfb840108,0xdbcd0101,
	0xfb840108,0xdbce0101,0xfb840108,0xdbcf0101,0xfb840108,0xdbd00101,0xfb840108,0xdbd10101,
	0xfb840108,0xdbd20101,0xfb840108,0xdbd30101,0xfb840108,0xdbd40101,0xfb840108,0xdbd50101,
	0xfb840108,0xdbd60101,0xfb840108,0xdbd70101,0xfb840108,0xdbd80101,0xfb840108,0xdbd90101,
	0xfb840108,0xdbda0101,0xfb840108,0xdbdb0101,0xfb840108,0xdbdc0101,0xfb840108,0xdbdd0101,
	0xfb840108,0xdbde0101,0xfb840108,0xdbdf0101,0xfb840108,0xdbe00101,0xfb840108,0xdbe10101,
	0xfb840108,0xdbe20101,0xfb840108,0xdbe30101,0xfb840108,0xdbe40101,0xfb840108,0xdbe50101,
	0xfb840108,0xdbe60101,0xfb840108,0xdbe70101,0xfb840108,0xdbe80101,0xfb840108,0xdbe90101,
	0xfb840108,0xdbea0101,0xfb840108,0xdbeb0101,0xfb840108,0xdbec0101,0xfb840108,0xdbed0101,
	0xfb840108,0xdbee0101,0xfb840108,0xdbef0101,0xfb840108,0xdbf00101,0xfb840108,0xdbf10101,
	0xfb840108,0xdbf20101,0xfb840108,0xdbf30101,0xfb840108,0xdbf40101,0xfb840108,0xdbf50101,
	0xfb840108,0xdbf60101,0xfb840108,0xdbf70101,0xfb840108,0xdbf80101,0xfb840108,0xdbf90101,
	0xfb840108,0xdbfa0101,0xfb840108,0xdbfb0101,0xfb840108,0xdbfc0101,0xfb840108,0xdbfd0101,
	0xfb840108,0xdbfe0101,0xfb840108,0xdbff0101,0xfb840108,0xdc000101,0xfb840108,0xdc010101,
	0xfb840108,0xdc020101,0xfb840108,0xdc030101,0xfb840108,0xdc040101,0xfb840108,0xdc050101,
	0xfb840108,0xdc060101,0xfb840108,0xdc070101,0xfb840108,0xdc080101,0xfb840108,0xdc090101,
	0xfb840108,0xdc0a0101,0xfb840108,0xdc0b0101,0xfb840108,0xdc0c0101,0xfb840108,0xdc0d0101,
	0xfb840108,0xdc0e0101,0xfb840108,0xdc0f0101,0xfb840108,0xdc100101,0xfb840108,0xdc110101,
	0xfb840108,0xdc120101,0xfb840108,0xdc130101,0xfb840108,0xdc140101,0xfb840108,0xdc150101,
	0xfb840108,0xdc160101,0xfb840108,0xdc170101,0xfb840108,0xdc180101,0xfb840108,0xdc190101,
	0xfb840108,0xdc1a0101,0xfb840108,0xdc1b0101,0xfb840108,0xdc1c0101,0xfb840108,0xdc1d0101,
	0xfb840108,0xdc1e0101,0xfb840108,0xdc1f0101,0xfb840108,0xdc200101,0xfb840108,0xdc210101,
	0xfb840108,0xdc220101,0xfb840108,0xdc230101,0xfb840108,0xdc240101,0xfb840108,0xdc250101,
	0xfb840108,0xdc260101,0xfb840108,0xdc270101,0xfb840108,0xdc280101,0xfb840108,0xdc290101,
	0xfb840108,0xdc2a0101,0xfb840108,0xdc2b0101,0xfb840108,0xdc2c0101,0xfb840108,0xdc2d0101,
	0xfb840108,0xdc2e0101,0xfb840108,0xdc2f0101,0xfb840108,0xdc300101,0xfb840108,0xdc310101,
	0xfb840108,0xdc320101,0xfb840108,0xdc330101,0xfb840108,0xdc340101,0xfb840108,0xdc350101,
	0xfb840108,0xdc360101,0xfb840108,0xdc370101,0xfb840108,0xdc380101,0xfb840108,0xdc390101,
	0xfb840108,0xdc3a0101,0xfb840108,0xdc3b0101,0xfb840108,0xdc3c0101,0xfb840108,0xdc3d0101,
	0xfb840108,0xdc3e0101,0xfb840108,0xdc3f0101,0xfb840108,0xdc400101,0xfb840108,0xdc410101,
	0xfb840108,0xdc420101,0xfb840108,0xdc430101,0xfb840108,0xdc440101,0xfb840108,0xdc450101,
	0xfb840108,0xdc460101,0xfb840108,0xdc470101,0xfb840108,0xdc480101,0xfb840108,0xdc490101,
	0xfb840108,0xdc4a0101,0xfb840108,0xdc4b0101,0xfb840108,0xdc4c0101,0xfb840108,0xdc4d0101,
	0xfb840108,0xdc4e0101,0xfb840108,0xdc4f0101,0xfb840108,0xdc500101,0xfb840108,0xdc510101,
	0xfb840108,0xdc520101,0xfb840108,0xdc530101,0xfb840108,0xdc540101,0xfb840108,0xdc550101,
	0xfb840108,0xdc560101,0xfb840108,0xdc570101,0xfb840108,0xdc580101,0xfb840108,0xdc590101,
	0xfb840108,0xdc5a0101,0xfb840108,0xdc5b0101,0xfb840108,0xdc5c0101,0xfb840108,0xdc5d0101,
	0xfb840108,0xdc5e0101,0xfb840108,0xdc5f0101,0xfb840108,0xdc600101,0xfb840108,0xdc610101,
	0xfb840108,0xdc620101,0xfb840108,0xdc630101,0xfb840108,0xdc640101,0xfb840108,0xdc650101,
	0xfb840108,0xdc660101,0xfb840108,0xdc670101,0xfb840108,0xdc680101,0xfb840108,0xdc690101,
	0xfb840108,0xdc6a0101,0xfb840108,0xdc6b0101,0xfb840108,0xdc6c0101,0xfb840108,0xdc6d0101,
	0xfb840108,0xdc6e0101,0xfb840108,0xdc6f0101,0xfb840108,0xdc700101,0xfb840108,0xdc710101,
	0xfb840108,0xdc720101,0xfb840108,0xdc730101,0xfb840108,0xdc740101,0xfb840108,0xdc750101,
	0xfb840108,0xdc760101,0xfb840108,0xdc770101,0xfb840108,0xdc780101,0xfb840108,0xdc790101,
	0xfb840108,0xdc7a0101,0xfb840108,0xdc7b0101,0xfb840108,0xdc7c0101,0xfb840108,0xdc7d0101,
	0xfb840108,0xdc7e0101,0xfb840108,0xdc7f0101,0xfb840108,0xdc800101,0xfb840108,0xdc810101,
	0xfb840108,0xdc820101,0xfb840108,0xdc830101,0xfb840108,0xdc840101,0xfb840108,0xdc850101,
	0xfb840108,0xdc860101,0xfb840108,0xdc870101,0xfb840108,0xdc880101,0xfb840108,0xdc890101,
	0xfb840108,0xdc8a0101,0xfb840108,0xdc8b0101,0xfb840108,0xdc8c0101,0xfb840108,0xdc8d0101,
	0xfb840108,0xdc8e0101,0xfb840108,0xdc8f0101,0xfb840108,0xdc900101,0xfb840108,0xdc910101,
	0xfb840108,0xdc920101,0xfb840108,0xdc930101,0xfb840108,0xdc940101,0xfb840108,0xdc950101,
	0xfb840108,0xdc960101,0xfb840108,0xdc970101,0xfb840108,0xdc980101,0xfb840108,0xdc990101,
	0xfb840108,0xdc9a0101,0xfb840108,0xdc9b0101,0xfb840108,0xdc9c0101,0xfb840108,0xdc9d0101,
	0xfb840108,0xdc9e0101,0xfb840108,0xdc9f0101,0xfb840108,0xdca00101,0xfb840108,0xdca10101,
	0xfb840108,0xdca20101,0xfb840108,0xdca30101,0xfb840108,0xdca40101,0xfb840108,0xdca50101,
	0xfb840108,0xdca60101,0xfb840108,0xdca70101,0xfb840108,0xdca80101,0xfb840108,0xdca90101,
	0xfb840108,0xdcaa0101,0xfb840108,0xdcab0101,0xfb840108,0xdcac0101,0xfb840108,0xdcad0101,
	0xfb840108,0xdcae0101,0xfb840108,0xdcaf0101,0xfb840108,0xdcb00101,0xfb840108,0xdcb10101,
	0xfb840108,0xdcb20101,0xfb840108,0xdcb30101,0xfb840108,0xdcb40101,0xfb840108,0xdcb50101,
	0xfb840108,0xdcb60101,0xfb840108,0xdcb70101,0xfb840108,0xdcb80101,0xfb840108,0xdcb90101,
	0xfb840108,0xdcba0101,0xfb840108,0xdcbb0101,0xfb840108,0xdcbc0101,0xfb840108,0xdcbd0101,
	0xfb840108,0xdcbe0101,0xfb840108,0xdcbf0101,0xfb840108,0xdcc00101,0xfb840108,0xdcc10101,
	0xfb840108,0xdcc20101,0xfb840108,0xdcc30101,0xfb840108,0xdcc40101,0xfb840108,0xdcc50101,
	0xfb840108,0xdcc60101,0xfb840108,0xdcc70101,0xfb840108,0xdcc80101,0xfb840108,0xdcc90101,
	0xfb840108,0xdcca0101,0xfb840108,0xdccb0101,0xfb840108,0xdccc0101,0xfb840108,0xdccd0101,
	0xfb840108,0xdcce0101,0xfb840108,0xdccf0101,0xfb840108,0xdcd00101,0xfb840108,0xdcd10101,
	0xfb840108,0xdcd20101,0xfb840108,0xdcd30101,0xfb840108,0xdcd40101,0xfb840108,0xdcd50101,
	0xfb840108,0xdcd60101,0xfb840108,0xdcd70101,0xfb840108,0xdcd80101,0xfb840108,0xdcd90101,
	0xfb840108,0xdcda0101,0xfb840108,0xdcdb0101,0xfb840108,0xdcdc0101,0xfb840108,0xdcdd0101,
	0xfb840108,0xdcde0101,0xfb840108,0xdcdf0101,0xfb840108,0xdce00101,0xfb840108,0xdce10101,
	0xfb840108,0xdce20101,0xfb840108,0xdce30101,0xfb840108,0xdce40101,0xfb840108,0xdce50101,
	0xfb840108,0xdce60101,0xfb840108,0xdce70101,0xfb840108,0xdce80101,0xfb840108,0xdce90101,
	0xfb840108,0xdcea0101,0xfb840108,0xdceb0101,0xfb840108,0xdcec0101,0xfb840108,0xdced0101,
	0xfb840108,0xdcee0101,0xfb840108,0xdcef0101,0xfb840108,0xdcf00101,0xfb840108,0xdcf10101,
	0xfb840108,0xdcf20101,0xfb840108,0xdcf30101,0xfb840108,0xdcf40101,0xfb840108,0xdcf50101,
	0xfb840108,0xdcf60101,0xfb840108,0xdcf70101,0xfb840108,0xdcf80101,0xfb840108,0xdcf90101,
	0xfb840108,0xdcfa0101,0xfb840108,0xdcfb0101,0xfb840108,0xdcfc0101,0xfb840108,0xdcfd0101,
	0xfb840108,0xdcfe0101,0xfb840108,0xdcff0101,0xfb840108,0xdd000101,0xfb840108,0xdd010101,
	0xfb840108,0xdd020101,0xfb840108,0xdd030101,0xfb840108,0xdd040101,0xfb840108,0xdd050101,
	0xfb840108,0xdd060101,0xfb840108,0xdd070101,0xfb840108,0xdd080101,0xfb840108,0xdd090101,
	0xfb840108,0xdd0a0101,0xfb840108,0xdd0b0101,0xfb840108,0xdd0c0101,0xfb840108,0xdd0d0101,
	0xfb840108,0xdd0e0101,0xfb840108,0xdd0f0101,0xfb840108,0xdd100101,0xfb840108,0xdd110101,
	0xfb840108,0xdd120101,0xfb840108,0xdd130101,0xfb840108,0xdd140101,0xfb840108,0xdd150101,
	0xfb840108,0xdd160101,0xfb840108,0xdd170101,0xfb840108,0xdd180101,0xfb840108,0xdd190101,
	0xfb840108,0xdd1a0101,0xfb840108,0xdd1b0101,0xfb840108,0xdd1c0101,0xfb840108,0xdd1d0101,
	0xfb840108,0xdd1e0101,0xfb840108,0xdd1f0101,0xfb840108,0xdd200101,0xfb840108,0xdd210101,
	0xfb840108,0xdd220101,0xfb840108,0xdd230101,0xfb840108,0xdd240101,0xfb840108,0xdd250101,
	0xfb840108,0xdd260101,0xfb840108,0xdd270101,0xfb840108,0xdd280101,0xfb840108,0xdd290101,
	0xfb840108,0xdd2a0101,0xfb840108,0xdd2b0101,0xfb840108,0xdd2c0101,0xfb840108,0xdd2d0101,
	0xfb840108,0xdd2e0101,0xfb840108,0xdd2f0101,0xfb840108,0xdd300101,0xfb840108,0xdd310101,
	0xfb840108,0xdd320101,0xfb840108,0xdd330101,0xfb840108,0xdd340101,0xfb840108,0xdd350101,
	0xfb840108,0xdd360101,0xfb840108,0xdd370101,0xfb840108,0xdd380101,0xfb840108,0xdd390101,
	0xfb840108,0xdd3a0101,0xfb840108,0xdd3b0101,0xfb840108,0xdd3c0101,0xfb840108,0xdd3d0101,
	0xfb840108,0xdd3e0101,0xfb840108,0xdd3f0101,0xfb840108,0xdd400101,0xfb840108,0xdd410101,
	0xfb840108,0xdd420101,0xfb840108,0xdd430101,0xfb840108,0xdd440101,0xfb840108,0xdd450101,
	0xfb840108,0xdd460101,0xfb840108,0xdd470101,0xfb840108,0xdd480101,0xfb840108,0xdd490101,
	0xfb840108,0xdd4a0101,0xfb840108,0xdd4b0101,0xfb840108,0xdd4c0101,0xfb840108,0xdd4d0101,
	0xfb840108,0xdd4e0101,0xfb840108,0xdd4f0101,0xfb840108,0xdd500101,0xfb840108,0xdd510101,
	0xfb840108,0xdd520101,0xfb840108,0xdd530101,0xfb840108,0xdd540101,0xfb840108,0xdd550101,
	0xfb840108,0xdd560101,0xfb840108,0xdd570101,0xfb840108,0xdd580101,0xfb840108,0xdd590101,
	0xfb840108,0xdd5a0101,0xfb840108,0xdd5b0101,0xfb840108,0xdd5c0101,0xfb840108,0xdd5d0101,
	0xfb840108,0xdd5e0101,0xfb840108,0xdd5f0101,0xfb840108,0xdd600101,0xfb840108,0xdd610101,
	0xfb840108,0xdd620101,0xfb840108,0xdd630101,0xfb840108,0xdd640101,0xfb840108,0xdd650101,
	0xfb840108,0xdd660101,0xfb840108,0xdd670101,0xfb840108,0xdd680101,0xfb840108,0xdd690101,
	0xfb840108,0xdd6a0101,0xfb840108,0xdd6b0101,0xfb840108,0xdd6c0101,0xfb840108,0xdd6d0101,
	0xfb840108,0xdd6e0101,0xfb840108,0xdd6f0101,0xfb840108,0xdd700101,0xfb840108,0xdd710101,
	0xfb840108,0xdd720101,0xfb840108,0xdd730101,0xfb840108,0xdd740101,0xfb840108,0xdd750101,
	0xfb840108,0xdd760101,0xfb840108,0xdd770101,0xfb840108,0xdd780101,0xfb840108,0xdd790101,
	0xfb840108,0xdd7a0101,0xfb840108,0xdd7b0101,0xfb840108,0xdd7c0101,0xfb840108,0xdd7d0101,
	0xfb840108,0xdd7e0101,0xfb840108,0xdd7f0101,0xfb840108,0xdd800101,0xfb840108,0xdd810101,
	0xfb840108,0xdd820101,0xfb840108,0xdd830101,0xfb840108,0xdd840101,0xfb840108,0xdd850101,
	0xfb840108,0xdd860101,0xfb840108,0xdd870101,0xfb840108,0xdd880101,0xfb840108,0xdd890101,
	0xfb840108,0xdd8a0101,0xfb840108,0xdd8b0101,0xfb840108,0xdd8c0101,0xfb840108,0xdd8d0101,
	0xfb840108,0xdd8e0101,0xfb840108,0xdd8f0101,0xfb840108,0xdd900101,0xfb840108,0xdd910101,
	0xfb840108,0xdd920101,0xfb840108,0xdd930101,0xfb840108,0xdd940101,0xfb840108,0xdd950101,
	0xfb840108,0xdd960101,0xfb840108,0xdd970101,0xfb840108,0xdd980101,0xfb840108,0xdd990101,
	0xfb840108,0xdd9a0101,0xfb840108,0xdd9b0101,0xfb840108,0xdd9c0101,0xfb840108,0xdd9d0101,
	0xfb840108,0xdd9e0101,0xfb840108,0xdd9f0101,0xfb840108,0xdda00101,0xfb840108,0xdda10101,
	0xfb840108,0xdda20101,0xfb840108,0xdda30101,0xfb840108,0xdda40101,0xfb840108,0xdda50101,
	0xfb840108,0xdda60101,0xfb840108,0xdda70101,0xfb840108,0xdda80101,0xfb840108,0xdda90101,
	0xfb840108,0xddaa0101,0xfb840108,0xddab0101,0xfb840108,0xddac0101,0xfb840108,0xddad0101,
	0xfb840108,0xddae0101,0xfb840108,0xddaf0101,0xfb840108,0xddb00101,0xfb840108,0xddb10101,
	0xfb840108,0xddb20101,0xfb840108,0xddb30101,0xfb840108,0xddb40101,0xfb840108,0xddb50101,
	0xfb840108,0xddb60101,0xfb840108,0xddb70101,0xfb840108,0xddb80101,0xfb840108,0xddb90101,
	0xfb840108,0xddba0101,0xfb840108,0xddbb0101,0xfb840108,0xddbc0101,0xfb840108,0xddbd0101,
	0xfb840108,0xddbe0101,0xfb840108,0xddbf0101,0xfb840108,0xddc00101,0xfb840108,0xddc10101,
	0xfb840108,0xddc20101,0xfb840108,0xddc30101,0xfb840108,0xddc40101,0xfb840108,0xddc50101,
	0xfb840108,0xddc60101,0xfb840108,0xddc70101,0xfb840108,0xddc80101,0xfb840108,0xddc90101,
	0xfb840108,0xddca0101,0xfb840108,0xddcb0101,0xfb840108,0xddcc0101,0xfb840108,0xddcd0101,
	0xfb840108,0xddce0101,0xfb840108,0xddcf0101,0xfb840108,0xddd00101,0xfb840108,0xddd10101,
	0xfb840108,0xddd20101,0xfb840108,0xddd30101,0xfb840108,0xddd40101,0xfb840108,0xddd50101,
	0xfb840108,0xddd60101,0xfb840108,0xddd70101,0xfb840108,0xddd80101,0xfb840108,0xddd90101,
	0xfb840108,0xddda0101,0xfb840108,0xdddb0101,0xfb840108,0xdddc0101,0xfb840108,0xdddd0101,
	0xfb840108,0xddde0101,0xfb840108,0xdddf0101,0xfb840108,0xdde00101,0xfb840108,0xdde10101,
	0xfb840108,0xdde20101,0xfb840108,0xdde30101,0xfb840108,0xdde40101,0xfb840108,0xdde50101,
	0xfb840108,0xdde60101,0xfb840108,0xdde70101,0xfb840108,0xdde80101,0xfb840108,0xdde90101,
	0xfb840108,0xddea0101,0xfb840108,0xddeb0101,0xfb840108,0xddec0101,0xfb840108,0xdded0101,
	0xfb840108,0xddee0101,0xfb840108,0xddef0101,0xfb840108,0xddf00101,0xfb840108,0xddf10101,
	0xfb840108,0xddf20101,0xfb840108,0xddf30101,0xfb840108,0xddf40101,0xfb840108,0xddf50101,
	0xfb840108,0xddf60101,0xfb840108,0xddf70101,0xfb840108,0xddf80101,0xfb840108,0xddf90101,
	0xfb840108,0xddfa0101,0xfb840108,0xddfb0101,0xfb840108,0xddfc0101,0xfb840108,0xddfd0101,
	0xfb840108,0xddfe0101,0xfb840108,0xddff0101,0xfb840108,0xde000101,0xfb840108,0xde010101,
	0xfb840108,0xde020101,0xfb840108,0xde030101,0xfb840108,0xde040101,0xfb840108,0xde050101,
	0xfb840108,0xde060101,0xfb840108,0xde070101,0xfb840108,0xde080101,0xfb840108,0xde090101,
	0xfb840108,0xde0a0101,0xfb840108,0xde0b0101,0xfb840108,0xde0c0101,0xfb840108,0xde0d0101,
	0xfb840108,0xde0e0101,0xfb840108,0xde0f0101,0xfb840108,0xde100101,0xfb840108,0xde110101,
	0xfb840108,0xde120101,0xfb840108,0xde130101,0xfb840108,0xde140101,0xfb840108,0xde150101,
	0xfb840108,0xde160101,0xfb840108,0xde170101,0xfb840108,0xde180101,0xfb840108,0xde190101,
	0xfb840108,0xde1a0101,0xfb840108,0xde1b0101,0xfb840108,0xde1c0101,0xfb840108,0xde1d0101,
	0xfb840108,0xde1e0101,0xfb840108,0xde1f0101,0xfb840108,0xde200101,0xfb840108,0xde210101,
	0xfb840108,0xde220101,0xfb840108,0xde230101,0xfb840108,0xde240101,0xfb840108,0xde250101,
	0xfb840108,0xde260101,0xfb840108,0xde270101,0xfb840108,0xde280101,0xfb840108,0xde290101,
	0xfb840108,0xde2a0101,0xfb840108,0xde2b0101,0xfb840108,0xde2c0101,0xfb840108,0xde2d0101,
	0xfb840108,0xde2e0101,0xfb840108,0xde2f0101,0xfb840108,0xde300101,0xfb840108,0xde310101,
	0xfb840108,0xde320101,0xfb840108,0xde330101,0xfb840108,0xde340101,0xfb840108,0xde350101,
	0xfb840108,0xde360101,0xfb840108,0xde370101,0xfb840108,0xde380101,0xfb840108,0xde390101,
	0xfb840108,0xde3a0101,0xfb840108,0xde3b0101,0xfb840108,0xde3c0101,0xfb840108,0xde3d0101,
	0xfb840108,0xde3e0101,0xfb840108,0xde3f0101,0xfb840108,0xde400101,0xfb840108,0xde410101,
	0xfb840108,0xde420101,0xfb840108,0xde430101,0xfb840108,0xde440101,0xfb840108,0xde450101,
	0xfb840108,0xde460101,0xfb840108,0xde470101,0xfb840108,0xde480101,0xfb840108,0xde490101,
	0xfb840108,0xde4a0101,0xfb840108,0xde4b0101,0xfb840108,0xde4c0101,0xfb840108,0xde4d0101,
	0xfb840108,0xde4e0101,0xfb840108,0xde4f0101,0xfb840108,0xde500101,0xfb840108,0xde510101,
	0xfb840108,0xde520101,0xfb840108,0xde530101,0xfb840108,0xde540101,0xfb840108,0xde550101,
	0xfb840108,0xde560101,0xfb840108,0xde570101,0xfb840108,0xde580101,0xfb840108,0xde590101,
	0xfb840108,0xde5a0101,0xfb840108,0xde5b0101,0xfb840108,0xde5c0101,0xfb840108,0xde5d0101,
	0xfb840108,0xde5e0101,0xfb840108,0xde5f0101,0xfb840108,0xde600101,0xfb840108,0xde610101,
	0xfb840108,0xde620101,0xfb840108,0xde630101,0xfb840108,0xde640101,0xfb840108,0xde650101,
	0xfb840108,0xde660101,0xfb840108,0xde670101,0xfb840108,0xde680101,0xfb840108,0xde690101,
	0xfb840108,0xde6a0101,0xfb840108,0xde6b0101,0xfb840108,0xde6c0101,0xfb840108,0xde6d0101,
	0xfb840108,0xde6e0101,0xfb840108,0xde6f0101,0xfb840108,0xde700101,0xfb840108,0xde710101,
	0xfb840108,0xde720101,0xfb840108,0xde730101,0xfb840108,0xde740101,0xfb840108,0xde750101,
	0xfb840108,0xde760101,0xfb840108,0xde770101,0xfb840108,0xde780101,0xfb840108,0xde790101,
	0xfb840108,0xde7a0101,0xfb840108,0xde7b0101,0xfb840108,0xde7c0101,0xfb840108,0xde7d0101,
	0xfb840108,0xde7e0101,0xfb840108,0xde7f0101,0xfb840108,0xde800101,0xfb840108,0xde810101,
	0xfb840108,0xde820101,0xfb840108,0xde830101,0xfb840108,0xde840101,0xfb840108,0xde850101,
	0xfb840108,0xde860101,0xfb840108,0xde870101,0xfb840108,0xde880101,0xfb840108,0xde890101,
	0xfb840108,0xde8a0101,0xfb840108,0xde8b0101,0xfb840108,0xde8c0101,0xfb840108,0xde8d0101,
	0xfb840108,0xde8e0101,0xfb840108,0xde8f0101,0xfb840108,0xde900101,0xfb840108,0xde910101,
	0xfb840108,0xde920101,0xfb840108,0xde930101,0xfb840108,0xde940101,0xfb840108,0xde950101,
	0xfb840108,0xde960101,0xfb840108,0xde970101,0xfb840108,0xde980101,0xfb840108,0xde990101,
	0xfb840108,0xde9a0101,0xfb840108,0xde9b0101,0xfb840108,0xde9c0101,0xfb840108,0xde9d0101,
	0xfb840108,0xde9e0101,0xfb840108,0xde9f0101,0xfb840108,0xdea00101,0xfb840108,0xdea10101,
	0xfb840108,0xdea20101,0xfb840108,0xdea30101,0xfb840108,0xdea40101,0xfb840108,0xdea50101,
	0xfb840108,0xdea60101,0xfb840108,0xdea70101,0xfb840108,0xdea80101,0xfb840108,0xdea90101,
	0xfb840108,0xdeaa0101,0xfb840108,0xdeab0101,0xfb840108,0xdeac0101,0xfb840108,0xdead0101,
	0xfb840108,0xdeae0101,0xfb840108,0xdeaf0101,0xfb840108,0xdeb00101,0xfb840108,0xdeb10101,
	0xfb840108,0xdeb20101,0xfb840108,0xdeb30101,0xfb840108,0xdeb40101,0xfb840108,0xdeb50101,
	0xfb840108,0xdeb60101,0xfb840108,0xdeb70101,0xfb840108,0xdeb80101,0xfb840108,0xdeb90101,
	0xfb840108,0xdeba0101,0xfb840108,0xdebb0101,0xfb840108,0xdebc0101,0xfb840108,0xdebd0101,
	0xfb840108,0xdebe0101,0xfb840108,0xdebf0101,0xfb840108,0xdec00101,0xfb840108,0xdec10101,
	0xfb840108,0xdec20101,0xfb840108,0xdec30101,0xfb840108,0xdec40101,0xfb840108,0xdec50101,
	0xfb840108,0xdec60101,0xfb840108,0xdec70101,0xfb840108,0xdec80101,0xfb840108,0xdec90101,
	0xfb840108,0xdeca0101,0xfb840108,0xdecb0101,0xfb840108,0xdecc0101,0xfb840108,0xdecd0101,
	0xfb840108,0xdece0101,0xfb840108,0xdecf0101,0xfb840108,0xded00101,0xfb840108,0xded10101,
	0xfb840108,0xded20101,0xfb840108,0xded30101,0xfb840108,0xded40101,0xfb840108,0xded50101,
	0xfb840108,0xded60101,0xfb840108,0xded70101,0xfb840108,0xded80101,0xfb840108,0xded90101,
	0xfb840108,0xdeda0101,0xfb840108,0xdedb0101,0xfb840108,0xdedc0101,0xfb840108,0xdedd0101,
	0xfb840108,0xdede0101,0xfb840108,0xdedf0101,0xfb840108,0xdee00101,0xfb840108,0xdee10101,
	0xfb840108,0xdee20101,0xfb840108,0xdee30101,0xfb840108,0xdee40101,0xfb840108,0xdee50101,
	0xfb840108,0xdee60101,0xfb840108,0xdee70101,0xfb840108,0xdee80101,0xfb840108,0xdee90101,
	0xfb840108,0xdeea0101,0xfb840108,0xdeeb0101,0xfb840108,0xdeec0101,0xfb840108,0xdeed0101,
	0xfb840108,0xdeee0101,0xfb840108,0xdeef0101,0xfb840108,0xdef00101,0xfb840108,0xdef10101,
	0xfb840108,0xdef20101,0xfb840108,0xdef30101,0xfb840108,0xdef40101,0xfb840108,0xdef50101,
	0xfb840108,0xdef60101,0xfb840108,0xdef70101,0xfb840108,0xdef80101,0xfb840108,0xdef90101,
	0xfb840108,0xdefa0101,0xfb840108,0xdefb0101,0xfb840108,0xdefc0101,0xfb840108,0xdefd0101,
	0xfb840108,0xdefe0101,0xfb840108,0xdeff0101,0xfb840108,0xdf000101,0xfb840108,0xdf010101,
	0xfb840108,0xdf020101,0xfb840108,0xdf030101,0xfb840108,0xdf040101,0xfb840108,0xdf050101,
	0xfb840108,0xdf060101,0xfb840108,0xdf070101,0xfb840108,0xdf080101,0xfb840108,0xdf090101,
	0xfb840108,0xdf0a0101,0xfb840108,0xdf0b0101,0xfb840108,0xdf0c0101,0xfb840108,0xdf0d0101,
	0xfb840108,0xdf0e0101,0xfb840108,0xdf0f0101,0xfb840108,0xdf100101,0xfb840108,0xdf110101,
	0xfb840108,0xdf120101,0xfb840108,0xdf130101,0xfb840108,0xdf140101,0xfb840108,0xdf150101,
	0xfb840108,0xdf160101,0xfb840108,0xdf170101,0xfb840108,0xdf180101,0xfb840108,0xdf190101,
	0xfb840108,0xdf1a0101,0xfb840108,0xdf1b0101,0xfb840108,0xdf1c0101,0xfb840108,0xdf1d0101,
	0xfb840108,0xdf1e0101,0xfb840108,0xdf1f0101,0xfb840108,0xdf200101,0xfb840108,0xdf210101,
	0xfb840108,0xdf220101,0xfb840108,0xdf230101,0xfb840108,0xdf240101,0xfb840108,0xdf250101,
	0xfb840108,0xdf260101,0xfb840108,0xdf270101,0xfb840108,0xdf280101,0xfb840108,0xdf290101,
	0xfb840108,0xdf2a0101,0xfb840108,0xdf2b0101,0xfb840108,0xdf2c0101,0xfb840108,0xdf2d0101,
	0xfb840108,0xdf2e0101,0xfb840108,0xdf2f0101,0xfb840108,0xdf300101,0xfb840108,0xdf310101,
	0xfb840108,0xdf320101,0xfb840108,0xdf330101,0xfb840108,0xdf340101,0xfb840108,0xdf350101,
	0xfb840108,0xdf360101,0xfb840108,0xdf370101,0xfb840108,0xdf380101,0xfb840108,0xdf390101,
	0xfb840108,0xdf3a0101,0xfb840108,0xdf3b0101,0xfb840108,0xdf3c0101,0xfb840108,0xdf3d0101,
	0xfb840108,0xdf3e0101,0xfb840108,0xdf3f0101,0xfb840108,0xdf400101,0xfb840108,0xdf410101,
	0xfb840108,0xdf420101,0xfb840108,0xdf430101,0xfb840108,0xdf440101,0xfb840108,0xdf450101,
	0xfb840108,0xdf460101,0xfb840108,0xdf470101,0xfb840108,0xdf480101,0xfb840108,0xdf490101,
	0xfb840108,0xdf4a0101,0xfb840108,0xdf4b0101,0xfb840108,0xdf4c0101,0xfb840108,0xdf4d0101,
	0xfb840108,0xdf4e0101,0xfb840108,0xdf4f0101,0xfb840108,0xdf500101,0xfb840108,0xdf510101,
	0xfb840108,0xdf520101,0xfb840108,0xdf530101,0xfb840108,0xdf540101,0xfb840108,0xdf550101,
	0xfb840108,0xdf560101,0xfb840108,0xdf570101,0xfb840108,0xdf580101,0xfb840108,0xdf590101,
	0xfb840108,0xdf5a0101,0xfb840108,0xdf5b0101,0xfb840108,0xdf5c0101,0xfb840108,0xdf5d0101,
	0xfb840108,0xdf5e0101,0xfb840108,0xdf5f0101,0xfb840108,0xdf600101,0xfb840108,0xdf610101,
	0xfb840108,0xdf620101,0xfb840108,0xdf630101,0xfb840108,0xdf640101,0xfb840108,0xdf650101,
	0xfb840108,0xdf660101,0xfb840108,0xdf670101,0xfb840108,0xdf680101,0xfb840108,0xdf690101,
	0xfb840108,0xdf6a0101,0xfb840108,0xdf6b0101,0xfb840108,0xdf6c0101,0xfb840108,0xdf6d0101,
	0xfb840108,0xdf6e0101,0xfb840108,0xdf6f0101,0xfb840108,0xdf700101,0xfb840108,0xdf710101,
	0xfb840108,0xdf720101,0xfb840108,0xdf730101,0xfb840108,0xdf740101,0xfb840108,0xdf750101,
	0xfb840108,0xdf760101,0xfb840108,0xdf770101,0xfb840108,0xdf780101,0xfb840108,0xdf790101,
	0xfb840108,0xdf7a0101,0xfb840108,0xdf7b0101,0xfb840108,0xdf7c0101,0xfb840108,0xdf7d0101,
	0xfb840108,0xdf7e0101,0xfb840108,0xdf7f0101,0xfb840108,0xdf800101,0xfb840108,0xdf810101,
	0xfb840108,0xdf820101,0xfb840108,0xdf830101,0xfb840108,0xdf840101,0xfb840108,0xdf850101,
	0xfb840108,0xdf860101,0xfb840108,0xdf870101,0xfb840108,0xdf880101,0xfb840108,0xdf890101,
	0xfb840108,0xdf8a0101,0xfb840108,0xdf8b0101,0xfb840108,0xdf8c0101,0xfb840108,0xdf8d0101,
	0xfb840108,0xdf8e0101,0xfb840108,0xdf8f0101,0xfb840108,0xdf900101,0xfb840108,0xdf910101,
	0xfb840108,0xdf920101,0xfb840108,0xdf930101,0xfb840108,0xdf940101,0xfb840108,0xdf950101,
	0xfb840108,0xdf960101,0xfb840108,0xdf970101,0xfb840108,0xdf980101,0xfb840108,0xdf990101,
	0xfb840108,0xdf9a0101,0xfb840108,0xdf9b0101,0xfb840108,0xdf9c0101,0xfb840108,0xdf9d0101,
	0xfb840108,0xdf9e0101,0xfb840108,0xdf9f0101,0xfb840108,0xdfa00101,0xfb840108,0xdfa10101,
	0xfb840108,0xdfa20101,0xfb840108,0xdfa30101,0xfb840108,0xdfa40101,0xfb840108,0xdfa50101,
	0xfb840108,0xdfa60101,0xfb840108,0xdfa70101,0xfb840108,0xdfa80101,0xfb840108,0xdfa90101,
	0xfb840108,0xdfaa0101,0xfb840108,0xdfab0101,0xfb840108,0xdfac0101,0xfb840108,0xdfad0101,
	0xfb840108,0xdfae0101,0xfb840108,0xdfaf0101,0xfb840108,0xdfb00101,0xfb840108,0xdfb10101,
	0xfb840108,0xdfb20101,0xfb840108,0xdfb30101,0xfb840108,0xdfb40101,0xfb840108,0xdfb50101,
	0xfb840108,0xdfb60101,0xfb840108,0xdfb70101,0xfb840108,0xdfb80101,0xfb840108,0xdfb90101,
	0xfb840108,0xdfba0101,0xfb840108,0xdfbb0101,0xfb840108,0xdfbc0101,0xfb840108,0xdfbd0101,
	0xfb840108,0xdfbe0101,0xfb840108,0xdfbf0101,0xfb840108,0xdfc00101,0xfb840108,0xdfc10101,
	0xfb840108,0xdfc20101,0xfb840108,0xdfc30101,0xfb840108,0xdfc40101,0xfb840108,0xdfc50101,
	0xfb840108,0xdfc60101,0xfb840108,0xdfc70101,0xfb840108,0xdfc80101,0xfb840108,0xdfc90101,
	0xfb840108,0xdfca0101,0xfb840108,0xdfcb0101,0xfb840108,0xdfcc0101,0xfb840108,0xdfcd0101,
	0xfb840108,0xdfce0101,0xfb840108,0xdfcf0101,0xfb840108,0xdfd00101,0xfb840108,0xdfd10101,
	0xfb840108,0xdfd20101,0xfb840108,0xdfd30101,0xfb840108,0xdfd40101,0xfb840108,0xdfd50101,
	0xfb840108,0xdfd60101,0xfb840108,0xdfd70101,0xfb840108,0xdfd80101,0xfb840108,0xdfd90101,
	0xfb840108,0xdfda0101,0xfb840108,0xdfdb0101,0xfb840108,0xdfdc0101,0xfb840108,0xdfdd0101,
	0xfb840108,0xdfde0101,0xfb840108,0xdfdf0101,0xfb840108,0xdfe00101,0xfb840108,0xdfe10101,
	0xfb840108,0xdfe20101,0xfb840108,0xdfe30101,0xfb840108,0xdfe40101,0xfb840108,0xdfe50101,
	0xfb840108,0xdfe60101,0xfb840108,0xdfe70101,0xfb840108,0xdfe80101,0xfb840108,0xdfe90101,
	0xfb840108,0xdfea0101,0xfb840108,0xdfeb0101,0xfb840108,0xdfec0101,0xfb840108,0xdfed0101,
	0xfb840108,0xdfee0101,0xfb840108,0xdfef0101,0xfb840108,0xdff00101,0xfb840108,0xdff10101,
	0xfb840108,0xdff20101,0xfb840108,0xdff30101,0xfb840108,0xdff40101,0xfb840108,0xdff50101,
	0xfb840108,0xdff60101,0xfb840108,0xdff70101,0xfb840108,0xdff80101,0xfb840108,0xdff90101,
	0xfb840108,0xdffa0101,0xfb840108,0xdffb0101,0xfb840108,0xdffc0101,0xfb840108,0xdffd0101,
	0xfb840108,0xdffe0101,0xfb840108,0xdfff0101,0xfb840108,0xe0000101,0xfb840108,0xe0010101,
	0xfb840108,0xe0020101,0xfb840108,0xe0030101,0xfb840108,0xe0040101,0xfb840108,0xe0050101,
	0xfb840108,0xe0060101,0xfb840108,0xe0070101,0xfb840108,0xe0080101,0xfb840108,0xe0090101,
	0xfb840108,0xe00a0101,0xfb840108,0xe00b0101,0xfb840108,0xe00c0101,0xfb840108,0xe00d0101,
	0xfb840108,0xe00e0101,0xfb840108,0xe00f0101,0xfb840108,0xe0100101,0xfb840108,0xe0110101,
	0xfb840108,0xe0120101,0xfb840108,0xe0130101,0xfb840108,0xe0140101,0xfb840108,0xe0150101,
	0xfb840108,0xe0160101,0xfb840108,0xe0170101,0xfb840108,0xe0180101,0xfb840108,0xe0190101,
	0xfb840108,0xe01a0101,0xfb840108,0xe01b0101,0xfb840108,0xe01c0101,0xfb840108,0xe01d0101,
	0xfb840108,0xe01e0101,0xfb840108,0xe01f0101,0xfb840108,0xe0200101,0xfb840108,0xe0210101,
	0xfb840108,0xe0220101,0xfb840108,0xe0230101,0xfb840108,0xe0240101,0xfb840108,0xe0250101,
	0xfb840108,0xe0260101,0xfb840108,0xe0270101,0xfb840108,0xe0280101,0xfb840108,0xe0290101,
	0xfb840108,0xe02a0101,0xfb840108,0xe02b0101,0xfb840108,0xe02c0101,0xfb840108,0xe02d0101,
	0xfb840108,0xe02e0101,0xfb840108,0xe02f0101,0xfb840108,0xe0300101,0xfb840108,0xe0310101,
	0xfb840108,0xe0320101,0xfb840108,0xe0330101,0xfb840108,0xe0340101,0xfb840108,0xe0350101,
	0xfb840108,0xe0360101,0xfb840108,0xe0370101,0xfb840108,0xe0380101,0xfb840108,0xe0390101,
	0xfb840108,0xe03a0101,0xfb840108,0xe03b0101,0xfb840108,0xe03c0101,0xfb840108,0xe03d0101,
	0xfb840108,0xe03e0101,0xfb840108,0xe03f0101,0xfb840108,0xe0400101,0xfb840108,0xe0410101,
	0xfb840108,0xe0420101,0xfb840108,0xe0430101,0xfb840108,0xe0440101,0xfb840108,0xe0450101,
	0xfb840108,0xe0460101,0xfb840108,0xe0470101,0xfb840108,0xe0480101,0xfb840108,0xe0490101,
	0xfb840108,0xe04a0101,0xfb840108,0xe04b0101,0xfb840108,0xe04c0101,0xfb840108,0xe04d0101,
	0xfb840108,0xe04e0101,0xfb840108,0xe04f0101,0xfb840108,0xe0500101,0xfb840108,0xe0510101,
	0xfb840108,0xe0520101,0xfb840108,0xe0530101,0xfb840108,0xe0540101,0xfb840108,0xe0550101,
	0xfb840108,0xe0560101,0xfb840108,0xe0570101,0xfb840108,0xe0580101,0xfb840108,0xe0590101,
	0xfb840108,0xe05a0101,0xfb840108,0xe05b0101,0xfb840108,0xe05c0101,0xfb840108,0xe05d0101,
	0xfb840108,0xe05e0101,0xfb840108,0xe05f0101,0xfb840108,0xe0600101,0xfb840108,0xe0610101,
	0xfb840108,0xe0620101,0xfb840108,0xe0630101,0xfb840108,0xe0640101,0xfb840108,0xe0650101,
	0xfb840108,0xe0660101,0xfb840108,0xe0670101,0xfb840108,0xe0680101,0xfb840108,0xe0690101,
	0xfb840108,0xe06a0101,0xfb840108,0xe06b0101,0xfb840108,0xe06c0101,0xfb840108,0xe06d0101,
	0xfb840108,0xe06e0101,0xfb840108,0xe06f0101,0xfb840108,0xe0700101,0xfb840108,0xe0710101,
	0xfb840108,0xe0720101,0xfb840108,0xe0730101,0xfb840108,0xe0740101,0xfb840108,0xe0750101,
	0xfb840108,0xe0760101,0xfb840108,0xe0770101,0xfb840108,0xe0780101,0xfb840108,0xe0790101,
	0xfb840108,0xe07a0101,0xfb840108,0xe07b0101,0xfb840108,0xe07c0101,0xfb840108,0xe07d0101,
	0xfb840108,0xe07e0101,0xfb840108,0xe07f0101,0xfb840108,0xe0800101,0xfb840108,0xe0810101,
	0xfb840108,0xe0820101,0xfb840108,0xe0830101,0xfb840108,0xe0840101,0xfb840108,0xe0850101,
	0xfb840108,0xe0860101,0xfb840108,0xe0870101,0xfb840108,0xe0880101,0xfb840108,0xe0890101,
	0xfb840108,0xe08a0101,0xfb840108,0xe08b0101,0xfb840108,0xe08c0101,0xfb840108,0xe08d0101,
	0xfb840108,0xe08e0101,0xfb840108,0xe08f0101,0xfb840108,0xe0900101,0xfb840108,0xe0910101,
	0xfb840108,0xe0920101,0xfb840108,0xe0930101,0xfb840108,0xe0940101,0xfb840108,0xe0950101,
	0xfb840108,0xe0960101,0xfb840108,0xe0970101,0xfb840108,0xe0980101,0xfb840108,0xe0990101,
	0xfb840108,0xe09a0101,0xfb840108,0xe09b0101,0xfb840108,0xe09c0101,0xfb840108,0xe09d0101,
	0xfb840108,0xe09e0101,0xfb840108,0xe09f0101,0xfb840108,0xe0a00101,0xfb840108,0xe0a10101,
	0xfb840108,0xe0a20101,0xfb840108,0xe0a30101,0xfb840108,0xe0a40101,0xfb840108,0xe0a50101,
	0xfb840108,0xe0a60101,0xfb840108,0xe0a70101,0xfb840108,0xe0a80101,0xfb840108,0xe0a90101,
	0xfb840108,0xe0aa0101,0xfb840108,0xe0ab0101,0xfb840108,0xe0ac0101,0xfb840108,0xe0ad0101,
	0xfb840108,0xe0ae0101,0xfb840108,0xe0af0101,0xfb840108,0xe0b00101,0xfb840108,0xe0b10101,
	0xfb840108,0xe0b20101,0xfb840108,0xe0b30101,0xfb840108,0xe0b40101,0xfb840108,0xe0b50101,
	0xfb840108,0xe0b60101,0xfb840108,0xe0b70101,0xfb840108,0xe0b80101,0xfb840108,0xe0b90101,
	0xfb840108,0xe0ba0101,0xfb840108,0xe0bb0101,0xfb840108,0xe0bc0101,0xfb840108,0xe0bd0101,
	0xfb840108,0xe0be0101,0xfb840108,0xe0bf0101,0xfb840108,0xe0c00101,0xfb840108,0xe0c10101,
	0xfb840108,0xe0c20101,0xfb840108,0xe0c30101,0xfb840108,0xe0c40101,0xfb840108,0xe0c50101,
	0xfb840108,0xe0c60101,0xfb840108,0xe0c70101,0xfb840108,0xe0c80101,0xfb840108,0xe0c90101,
	0xfb840108,0xe0ca0101,0xfb840108,0xe0cb0101,0xfb840108,0xe0cc0101,0xfb840108,0xe0cd0101,
	0xfb840108,0xe0ce0101,0xfb840108,0xe0cf0101,0xfb840108,0xe0d00101,0xfb840108,0xe0d10101,
	0xfb840108,0xe0d20101,0xfb840108,0xe0d30101,0xfb840108,0xe0d40101,0xfb840108,0xe0d50101,
	0xfb840108,0xe0d60101,0xfb840108,0xe0d70101,0xfb840108,0xe0d80101,0xfb840108,0xe0d90101,
	0xfb840108,0xe0da0101,0xfb840108,0xe0db0101,0xfb840108,0xe0dc0101,0xfb840108,0xe0dd0101,
	0xfb840108,0xe0de0101,0xfb840108,0xe0df0101,0xfb840108,0xe0e00101,0xfb840108,0xe0e10101,
	0xfb840108,0xe0e20101,0xfb840108,0xe0e30101,0xfb840108,0xe0e40101,0xfb840108,0xe0e50101,
	0xfb840108,0xe0e60101,0xfb840108,0xe0e70101,0xfb840108,0xe0e80101,0xfb840108,0xe0e90101,
	0xfb840108,0xe0ea0101,0xfb840108,0xe0eb0101,0xfb840108,0xe0ec0101,0xfb840108,0xe0ed0101,
	0xfb840108,0xe0ee0101,0xfb840108,0xe0ef0101,0xfb840108,0xe0f00101,0xfb840108,0xe0f10101,
	0xfb840108,0xe0f20101,0xfb840108,0xe0f30101,0xfb840108,0xe0f40101,0xfb840108,0xe0f50101,
	0xfb840108,0xe0f60101,0xfb840108,0xe0f70101,0xfb840108,0xe0f80101,0xfb840108,0xe0f90101,
	0xfb840108,0xe0fa0101,0xfb840108,0xe0fb0101,0xfb840108,0xe0fc0101,0xfb840108,0xe0fd0101,
	0xfb840108,0xe0fe0101,0xfb840108,0xe0ff0101,0xfb840108,0xe1000101,0xfb840108,0xe1010101,
	0xfb840108,0xe1020101,0xfb840108,0xe1030101,0xfb840108,0xe1040101,0xfb840108,0xe1050101,
	0xfb840108,0xe1060101,0xfb840108,0xe1070101,0xfb840108,0xe1080101,0xfb840108,0xe1090101,
	0xfb840108,0xe10a0101,0xfb840108,0xe10b0101,0xfb840108,0xe10c0101,0xfb840108,0xe10d0101,
	0xfb840108,0xe10e0101,0xfb840108,0xe10f0101,0xfb840108,0xe1100101,0xfb840108,0xe1110101,
	0xfb840108,0xe1120101,0xfb840108,0xe1130101,0xfb840108,0xe1140101,0xfb840108,0xe1150101,
	0xfb840108,0xe1160101,0xfb840108,0xe1170101,0xfb840108,0xe1180101,0xfb840108,0xe1190101,
	0xfb840108,0xe11a0101,0xfb840108,0xe11b0101,0xfb840108,0xe11c0101,0xfb840108,0xe11d0101,
	0xfb840108,0xe11e0101,0xfb840108,0xe11f0101,0xfb840108,0xe1200101,0xfb840108,0xe1210101,
	0xfb840108,0xe1220101,0xfb840108,0xe1230101,0xfb840108,0xe1240101,0xfb840108,0xe1250101,
	0xfb840108,0xe1260101,0xfb840108,0xe1270101,0xfb840108,0xe1280101,0xfb840108,0xe1290101,
	0xfb840108,0xe12a0101,0xfb840108,0xe12b0101,0xfb840108,0xe12c0101,0xfb840108,0xe12d0101,
	0xfb840108,0xe12e0101,0xfb840108,0xe12f0101,0xfb840108,0xe1300101,0xfb840108,0xe1310101,
	0xfb840108,0xe1320101,0xfb840108,0xe1330101,0xfb840108,0xe1340101,0xfb840108,0xe1350101,
	0xfb840108,0xe1360101,0xfb840108,0xe1370101,0xfb840108,0xe1380101,0xfb840108,0xe1390101,
	0xfb840108,0xe13a0101,0xfb840108,0xe13b0101,0xfb840108,0xe13c0101,0xfb840108,0xe13d0101,
	0xfb840108,0xe13e0101,0xfb840108,0xe13f0101,0xfb840108,0xe1400101,0xfb840108,0xe1410101,
	0xfb840108,0xe1420101,0xfb840108,0xe1430101,0xfb840108,0xe1440101,0xfb840108,0xe1450101,
	0xfb840108,0xe1460101,0xfb840108,0xe1470101,0xfb840108,0xe1480101,0xfb840108,0xe1490101,
	0xfb840108,0xe14a0101,0xfb840108,0xe14b0101,0xfb840108,0xe14c0101,0xfb840108,0xe14d0101,
	0xfb840108,0xe14e0101,0xfb840108,0xe14f0101,0xfb840108,0xe1500101,0xfb840108,0xe1510101,
	0xfb840108,0xe1520101,0xfb840108,0xe1530101,0xfb840108,0xe1540101,0xfb840108,0xe1550101,
	0xfb840108,0xe1560101,0xfb840108,0xe1570101,0xfb840108,0xe1580101,0xfb840108,0xe1590101,
	0xfb840108,0xe15a0101,0xfb840108,0xe15b0101,0xfb840108,0xe15c0101,0xfb840108,0xe15d0101,
	0xfb840108,0xe15e0101,0xfb840108,0xe15f0101,0xfb840108,0xe1600101,0xfb840108,0xe1610101,
	0xfb840108,0xe1620101,0xfb840108,0xe1630101,0xfb840108,0xe1640101,0xfb840108,0xe1650101,
	0xfb840108,0xe1660101,0xfb840108,0xe1670101,0xfb840108,0xe1680101,0xfb840108,0xe1690101,
	0xfb840108,0xe16a0101,0xfb840108,0xe16b0101,0xfb840108,0xe16c0101,0xfb840108,0xe16d0101,
	0xfb840108,0xe16e0101,0xfb840108,0xe16f0101,0xfb840108,0xe1700101,0xfb840108,0xe1710101,
	0xfb840108,0xe1720101,0xfb840108,0xe1730101,0xfb840108,0xe1740101,0xfb840108,0xe1750101,
	0xfb840108,0xe1760101,0xfb840108,0xe1770101,0xfb840108,0xe1780101,0xfb840108,0xe1790101,
	0xfb840108,0xe17a0101,0xfb840108,0xe17b0101,0xfb840108,0xe17c0101,0xfb840108,0xe17d0101,
	0xfb840108,0xe17e0101,0xfb840108,0xe17f0101,0xfb840108,0xe1800101,0xfb840108,0xe1810101,
	0xfb840108,0xe1820101,0xfb840108,0xe1830101,0xfb840108,0xe1840101,0xfb840108,0xe1850101,
	0xfb840108,0xe1860101,0xfb840108,0xe1870101,0xfb840108,0xe1880101,0xfb840108,0xe1890101,
	0xfb840108,0xe18a0101,0xfb840108,0xe18b0101,0xfb840108,0xe18c0101,0xfb840108,0xe18d0101,
	0xfb840108,0xe18e0101,0xfb840108,0xe18f0101,0xfb840108,0xe1900101,0xfb840108,0xe1910101,
	0xfb840108,0xe1920101,0xfb840108,0xe1930101,0xfb840108,0xe1940101,0xfb840108,0xe1950101,
	0xfb840108,0xe1960101,0xfb840108,0xe1970101,0xfb840108,0xe1980101,0xfb840108,0xe1990101,
	0xfb840108,0xe19a0101,0xfb840108,0xe19b0101,0xfb840108,0xe19c0101,0xfb840108,0xe19d0101,
	0xfb840108,0xe19e0101,0xfb840108,0xe19f0101,0xfb840108,0xe1a00101,0xfb840108,0xe1a10101,
	0xfb840108,0xe1a20101,0xfb840108,0xe1a30101,0xfb840108,0xe1a40101,0xfb840108,0xe1a50101,
	0xfb840108,0xe1a60101,0xfb840108,0xe1a70101,0xfb840108,0xe1a80101,0xfb840108,0xe1a90101,
	0xfb840108,0xe1aa0101,0xfb840108,0xe1ab0101,0xfb840108,0xe1ac0101,0xfb840108,0xe1ad0101,
	0xfb840108,0xe1ae0101,0xfb840108,0xe1af0101,0xfb840108,0xe1b00101,0xfb840108,0xe1b10101,
	0xfb840108,0xe1b20101,0xfb840108,0xe1b30101,0xfb840108,0xe1b40101,0xfb840108,0xe1b50101,
	0xfb840108,0xe1b60101,0xfb840108,0xe1b70101,0xfb840108,0xe1b80101,0xfb840108,0xe1b90101,
	0xfb840108,0xe1ba0101,0xfb840108,0xe1bb0101,0xfb840108,0xe1bc0101,0xfb840108,0xe1bd0101,
	0xfb840108,0xe1be0101,0xfb840108,0xe1bf0101,0xfb840108,0xe1c00101,0xfb840108,0xe1c10101,
	0xfb840108,0xe1c20101,0xfb840108,0xe1c30101,0xfb840108,0xe1c40101,0xfb840108,0xe1c50101,
	0xfb840108,0xe1c60101,0xfb840108,0xe1c70101,0xfb840108,0xe1c80101,0xfb840108,0xe1c90101,
	0xfb840108,0xe1ca0101,0xfb840108,0xe1cb0101,0xfb840108,0xe1cc0101,0xfb840108,0xe1cd0101,
	0xfb840108,0xe1ce0101,0xfb840108,0xe1cf0101,0xfb840108,0xe1d00101,0xfb840108,0xe1d10101,
	0xfb840108,0xe1d20101,0xfb840108,0xe1d30101,0xfb840108,0xe1d40101,0xfb840108,0xe1d50101,
	0xfb840108,0xe1d60101,0xfb840108,0xe1d70101,0xfb840108,0xe1d80101,0xfb840108,0xe1d90101,
	0xfb840108,0xe1da0101,0xfb840108,0xe1db0101,0xfb840108,0xe1dc0101,0xfb840108,0xe1dd0101,
	0xfb840108,0xe1de0101,0xfb840108,0xe1df0101,0xfb840108,0xe1e00101,0xfb840108,0xe1e10101,
	0xfb840108,0xe1e20101,0xfb840108,0xe1e30101,0xfb840108,0xe1e40101,0xfb840108,0xe1e50101,
	0xfb840108,0xe1e60101,0xfb840108,0xe1e70101,0xfb840108,0xe1e80101,0xfb840108,0xe1e90101,
	0xfb840108,0xe1ea0101,0xfb840108,0xe1eb0101,0xfb840108,0xe1ec0101,0xfb840108,0xe1ed0101,
	0xfb840108,0xe1ee0101,0xfb840108,0xe1ef0101,0xfb840108,0xe1f00101,0xfb840108,0xe1f10101,
	0xfb840108,0xe1f20101,0xfb840108,0xe1f30101,0xfb840108,0xe1f40101,0xfb840108,0xe1f50101,
	0xfb840108,0xe1f60101,0xfb840108,0xe1f70101,0xfb840108,0xe1f80101,0xfb840108,0xe1f90101,
	0xfb840108,0xe1fa0101,0xfb840108,0xe1fb0101,0xfb840108,0xe1fc0101,0xfb840108,0xe1fd0101,
	0xfb840108,0xe1fe0101,0xfb840108,0xe1ff0101,0xfb840108,0xe2000101,0xfb840108,0xe2010101,
	0xfb840108,0xe2020101,0xfb840108,0xe2030101,0xfb840108,0xe2040101,0xfb840108,0xe2050101,
	0xfb840108,0xe2060101,0xfb840108,0xe2070101,0xfb840108,0xe2080101,0xfb840108,0xe2090101,
	0xfb840108,0xe20a0101,0xfb840108,0xe20b0101,0xfb840108,0xe20c0101,0xfb840108,0xe20d0101,
	0xfb840108,0xe20e0101,0xfb840108,0xe20f0101,0xfb840108,0xe2100101,0xfb840108,0xe2110101,
	0xfb840108,0xe2120101,0xfb840108,0xe2130101,0xfb840108,0xe2140101,0xfb840108,0xe2150101,
	0xfb840108,0xe2160101,0xfb840108,0xe2170101,0xfb840108,0xe2180101,0xfb840108,0xe2190101,
	0xfb840108,0xe21a0101,0xfb840108,0xe21b0101,0xfb840108,0xe21c0101,0xfb840108,0xe21d0101,
	0xfb840108,0xe21e0101,0xfb840108,0xe21f0101,0xfb840108,0xe2200101,0xfb840108,0xe2210101,
	0xfb840108,0xe2220101,0xfb840108,0xe2230101,0xfb840108,0xe2240101,0xfb840108,0xe2250101,
	0xfb840108,0xe2260101,0xfb840108,0xe2270101,0xfb840108,0xe2280101,0xfb840108,0xe2290101,
	0xfb840108,0xe22a0101,0xfb840108,0xe22b0101,0xfb840108,0xe22c0101,0xfb840108,0xe22d0101,
	0xfb840108,0xe22e0101,0xfb840108,0xe22f0101,0xfb840108,0xe2300101,0xfb840108,0xe2310101,
	0xfb840108,0xe2320101,0xfb840108,0xe2330101,0xfb840108,0xe2340101,0xfb840108,0xe2350101,
	0xfb840108,0xe2360101,0xfb840108,0xe2370101,0xfb840108,0xe2380101,0xfb840108,0xe2390101,
	0xfb840108,0xe23a0101,0xfb840108,0xe23b0101,0xfb840108,0xe23c0101,0xfb840108,0xe23d0101,
	0xfb840108,0xe23e0101,0xfb840108,0xe23f0101,0xfb840108,0xe2400101,0xfb840108,0xe2410101,
	0xfb840108,0xe2420101,0xfb840108,0xe2430101,0xfb840108,0xe2440101,0xfb840108,0xe2450101,
	0xfb840108,0xe2460101,0xfb840108,0xe2470101,0xfb840108,0xe2480101,0xfb840108,0xe2490101,
	0xfb840108,0xe24a0101,0xfb840108,0xe24b0101,0xfb840108,0xe24c0101,0xfb840108,0xe24d0101,
	0xfb840108,0xe24e0101,0xfb840108,0xe24f0101,0xfb840108,0xe2500101,0xfb840108,0xe2510101,
	0xfb840108,0xe2520101,0xfb840108,0xe2530101,0xfb840108,0xe2540101,0xfb840108,0xe2550101,
	0xfb840108,0xe2560101,0xfb840108,0xe2570101,0xfb840108,0xe2580101,0xfb840108,0xe2590101,
	0xfb840108,0xe25a0101,0xfb840108,0xe25b0101,0xfb840108,0xe25c0101,0xfb840108,0xe25d0101,
	0xfb840108,0xe25e0101,0xfb840108,0xe25f0101,0xfb840108,0xe2600101,0xfb840108,0xe2610101,
	0xfb840108,0xe2620101,0xfb840108,0xe2630101,0xfb840108,0xe2640101,0xfb840108,0xe2650101,
	0xfb840108,0xe2660101,0xfb840108,0xe2670101,0xfb840108,0xe2680101,0xfb840108,0xe2690101,
	0xfb840108,0xe26a0101,0xfb840108,0xe26b0101,0xfb840108,0xe26c0101,0xfb840108,0xe26d0101,
	0xfb840108,0xe26e0101,0xfb840108,0xe26f0101,0xfb840108,0xe2700101,0xfb840108,0xe2710101,
	0xfb840108,0xe2720101,0xfb840108,0xe2730101,0xfb840108,0xe2740101,0xfb840108,0xe2750101,
	0xfb840108,0xe2760101,0xfb840108,0xe2770101,0xfb840108,0xe2780101,0xfb840108,0xe2790101,
	0xfb840108,0xe27a0101,0xfb840108,0xe27b0101,0xfb840108,0xe27c0101,0xfb840108,0xe27d0101,
	0xfb840108,0xe27e0101,0xfb840108,0xe27f0101,0xfb840108,0xe2800101,0xfb840108,0xe2810101,
	0xfb840108,0xe2820101,0xfb840108,0xe2830101,0xfb840108,0xe2840101,0xfb840108,0xe2850101,
	0xfb840108,0xe2860101,0xfb840108,0xe2870101,0xfb840108,0xe2880101,0xfb840108,0xe2890101,
	0xfb840108,0xe28a0101,0xfb840108,0xe28b0101,0xfb840108,0xe28c0101,0xfb840108,0xe28d0101,
	0xfb840108,0xe28e0101,0xfb840108,0xe28f0101,0xfb840108,0xe2900101,0xfb840108,0xe2910101,
	0xfb840108,0xe2920101,0xfb840108,0xe2930101,0xfb840108,0xe2940101,0xfb840108,0xe2950101,
	0xfb840108,0xe2960101,0xfb840108,0xe2970101,0xfb840108,0xe2980101,0xfb840108,0xe2990101,
	0xfb840108,0xe29a0101,0xfb840108,0xe29b0101,0xfb840108,0xe29c0101,0xfb840108,0xe29d0101,
	0xfb840108,0xe29e0101,0xfb840108,0xe29f0101,0xfb840108,0xe2a00101,0xfb840108,0xe2a10101,
	0xfb840108,0xe2a20101,0xfb840108,0xe2a30101,0xfb840108,0xe2a40101,0xfb840108,0xe2a50101,
	0xfb840108,0xe2a60101,0xfb840108,0xe2a70101,0xfb840108,0xe2a80101,0xfb840108,0xe2a90101,
	0xfb840108,0xe2aa0101,0xfb840108,0xe2ab0101,0xfb840108,0xe2ac0101,0xfb840108,0xe2ad0101,
	0xfb840108,0xe2ae0101,0xfb840108,0xe2af0101,0xfb840108,0xe2b00101,0xfb840108,0xe2b10101,
	0xfb840108,0xe2b20101,0xfb840108,0xe2b30101,0xfb840108,0xe2b40101,0xfb840108,0xe2b50101,
	0xfb840108,0xe2b60101,0xfb840108,0xe2b70101,0xfb840108,0xe2b80101,0xfb840108,0xe2b90101,
	0xfb840108,0xe2ba0101,0xfb840108,0xe2bb0101,0xfb840108,0xe2bc0101,0xfb840108,0xe2bd0101,
	0xfb840108,0xe2be0101,0xfb840108,0xe2bf0101,0xfb840108,0xe2c00101,0xfb840108,0xe2c10101,
	0xfb840108,0xe2c20101,0xfb840108,0xe2c30101,0xfb840108,0xe2c40101,0xfb840108,0xe2c50101,
	0xfb840108,0xe2c60101,0xfb840108,0xe2c70101,0xfb840108,0xe2c80101,0xfb840108,0xe2c90101,
	0xfb840108,0xe2ca0101,0xfb840108,0xe2cb0101,0xfb840108,0xe2cc0101,0xfb840108,0xe2cd0101,
	0xfb840108,0xe2ce0101,0xfb840108,0xe2cf0101,0xfb840108,0xe2d00101,0xfb840108,0xe2d10101,
	0xfb840108,0xe2d20101,0xfb840108,0xe2d30101,0xfb840108,0xe2d40101,0xfb840108,0xe2d50101,
	0xfb840108,0xe2d60101,0xfb840108,0xe2d70101,0xfb840108,0xe2d80101,0xfb840108,0xe2d90101,
	0xfb840108,0xe2da0101,0xfb840108,0xe2db0101,0xfb840108,0xe2dc0101,0xfb840108,0xe2dd0101,
	0xfb840108,0xe2de0101,0xfb840108,0xe2df0101,0xfb840108,0xe2e00101,0xfb840108,0xe2e10101,
	0xfb840108,0xe2e20101,0xfb840108,0xe2e30101,0xfb840108,0xe2e40101,0xfb840108,0xe2e50101,
	0xfb840108,0xe2e60101,0xfb840108,0xe2e70101,0xfb840108,0xe2e80101,0xfb840108,0xe2e90101,
	0xfb840108,0xe2ea0101,0xfb840108,0xe2eb0101,0xfb840108,0xe2ec0101,0xfb840108,0xe2ed0101,
	0xfb840108,0xe2ee0101,0xfb840108,0xe2ef0101,0xfb840108,0xe2f00101,0xfb840108,0xe2f10101,
	0xfb840108,0xe2f20101,0xfb840108,0xe2f30101,0xfb840108,0xe2f40101,0xfb840108,0xe2f50101,
	0xfb840108,0xe2f60101,0xfb840108,0xe2f70101,0xfb840108,0xe2f80101,0xfb840108,0xe2f90101,
	0xfb840108,0xe2fa0101,0xfb840108,0xe2fb0101,0xfb840108,0xe2fc0101,0xfb840108,0xe2fd0101,
	0xfb840108,0xe2fe0101,0xfb840108,0xe2ff0101,0xfb840108,0xe3000101,0xfb840108,0xe3010101,
	0xfb840108,0xe3020101,0xfb840108,0xe3030101,0xfb840108,0xe3040101,0xfb840108,0xe3050101,
	0xfb840108,0xe3060101,0xfb840108,0xe3070101,0xfb840108,0xe3080101,0xfb840108,0xe3090101,
	0xfb840108,0xe30a0101,0xfb840108,0xe30b0101,0xfb840108,0xe30c0101,0xfb840108,0xe30d0101,
	0xfb840108,0xe30e0101,0xfb840108,0xe30f0101,0xfb840108,0xe3100101,0xfb840108,0xe3110101,
	0xfb840108,0xe3120101,0xfb840108,0xe3130101,0xfb840108,0xe3140101,0xfb840108,0xe3150101,
	0xfb840108,0xe3160101,0xfb840108,0xe3170101,0xfb840108,0xe3180101,0xfb840108,0xe3190101,
	0xfb840108,0xe31a0101,0xfb840108,0xe31b0101,0xfb840108,0xe31c0101,0xfb840108,0xe31d0101,
	0xfb840108,0xe31e0101,0xfb840108,0xe31f0101,0xfb840108,0xe3200101,0xfb840108,0xe3210101,
	0xfb840108,0xe3220101,0xfb840108,0xe3230101,0xfb840108,0xe3240101,0xfb840108,0xe3250101,
	0xfb840108,0xe3260101,0xfb840108,0xe3270101,0xfb840108,0xe3280101,0xfb840108,0xe3290101,
	0xfb840108,0xe32a0101,0xfb840108,0xe32b0101,0xfb840108,0xe32c0101,0xfb840108,0xe32d0101,
	0xfb840108,0xe32e0101,0xfb840108,0xe32f0101,0xfb840108,0xe3300101,0xfb840108,0xe3310101,
	0xfb840108,0xe3320101,0xfb840108,0xe3330101,0xfb840108,0xe3340101,0xfb840108,0xe3350101,
	0xfb840108,0xe3360101,0xfb840108,0xe3370101,0xfb840108,0xe3380101,0xfb840108,0xe3390101,
	0xfb840108,0xe33a0101,0xfb840108,0xe33b0101,0xfb840108,0xe33c0101,0xfb840108,0xe33d0101,
	0xfb840108,0xe33e0101,0xfb840108,0xe33f0101,0xfb840108,0xe3400101,0xfb840108,0xe3410101,
	0xfb840108,0xe3420101,0xfb840108,0xe3430101,0xfb840108,0xe3440101,0xfb840108,0xe3450101,
	0xfb840108,0xe3460101,0xfb840108,0xe3470101,0xfb840108,0xe3480101,0xfb840108,0xe3490101,
	0xfb840108,0xe34a0101,0xfb840108,0xe34b0101,0xfb840108,0xe34c0101,0xfb840108,0xe34d0101,
	0xfb840108,0xe34e0101,0xfb840108,0xe34f0101,0xfb840108,0xe3500101,0xfb840108,0xe3510101,
	0xfb840108,0xe3520101,0xfb840108,0xe3530101,0xfb840108,0xe3540101,0xfb840108,0xe3550101,
	0xfb840108,0xe3560101,0xfb840108,0xe3570101,0xfb840108,0xe3580101,0xfb840108,0xe3590101,
	0xfb840108,0xe35a0101,0xfb840108,0xe35b0101,0xfb840108,0xe35c0101,0xfb840108,0xe35d0101,
	0xfb840108,0xe35e0101,0xfb840108,0xe35f0101,0xfb840108,0xe3600101,0xfb840108,0xe3610101,
	0xfb840108,0xe3620101,0xfb840108,0xe3630101,0xfb840108,0xe3640101,0xfb840108,0xe3650101,
	0xfb840108,0xe3660101,0xfb840108,0xe3670101,0xfb840108,0xe3680101,0xfb840108,0xe3690101,
	0xfb840108,0xe36a0101,0xfb840108,0xe36b0101,0xfb840108,0xe36c0101,0xfb840108,0xe36d0101,
	0xfb840108,0xe36e0101,0xfb840108,0xe36f0101,0xfb840108,0xe3700101,0xfb840108,0xe3710101,
	0xfb840108,0xe3720101,0xfb840108,0xe3730101,0xfb840108,0xe3740101,0xfb840108,0xe3750101,
	0xfb840108,0xe3760101,0xfb840108,0xe3770101,0xfb840108,0xe3780101,0xfb840108,0xe3790101,
	0xfb840108,0xe37a0101,0xfb840108,0xe37b0101,0xfb840108,0xe37c0101,0xfb840108,0xe37d0101,
	0xfb840108,0xe37e0101,0xfb840108,0xe37f0101,0xfb840108,0xe3800101,0xfb840108,0xe3810101,
	0xfb840108,0xe3820101,0xfb840108,0xe3830101,0xfb840108,0xe3840101,0xfb840108,0xe3850101,
	0xfb840108,0xe3860101,0xfb840108,0xe3870101,0xfb840108,0xe3880101,0xfb840108,0xe3890101,
	0xfb840108,0xe38a0101,0xfb840108,0xe38b0101,0xfb840108,0xe38c0101,0xfb840108,0xe38d0101,
	0xfb840108,0xe38e0101,0xfb840108,0xe38f0101,0xfb840108,0xe3900101,0xfb840108,0xe3910101,
	0xfb840108,0xe3920101,0xfb840108,0xe3930101,0xfb840108,0xe3940101,0xfb840108,0xe3950101,
	0xfb840108,0xe3960101,0xfb840108,0xe3970101,0xfb840108,0xe3980101,0xfb840108,0xe3990101,
	0xfb840108,0xe39a0101,0xfb840108,0xe39b0101,0xfb840108,0xe39c0101,0xfb840108,0xe39d0101,
	0xfb840108,0xe39e0101,0xfb840108,0xe39f0101,0xfb840108,0xe3a00101,0xfb840108,0xe3a10101,
	0xfb840108,0xe3a20101,0xfb840108,0xe3a30101,0xfb840108,0xe3a40101,0xfb840108,0xe3a50101,
	0xfb840108,0xe3a60101,0xfb840108,0xe3a70101,0xfb840108,0xe3a80101,0xfb840108,0xe3a90101,
	0xfb840108,0xe3aa0101,0xfb840108,0xe3ab0101,0xfb840108,0xe3ac0101,0xfb840108,0xe3ad0101,
	0xfb840108,0xe3ae0101,0xfb840108,0xe3af0101,0xfb840108,0xe3b00101,0xfb840108,0xe3b10101,
	0xfb840108,0xe3b20101,0xfb840108,0xe3b30101,0xfb840108,0xe3b40101,0xfb840108,0xe3b50101,
	0xfb840108,0xe3b60101,0xfb840108,0xe3b70101,0xfb840108,0xe3b80101,0xfb840108,0xe3b90101,
	0xfb840108,0xe3ba0101,0xfb840108,0xe3bb0101,0xfb840108,0xe3bc0101,0xfb840108,0xe3bd0101,
	0xfb840108,0xe3be0101,0xfb840108,0xe3bf0101,0xfb840108,0xe3c00101,0xfb840108,0xe3c10101,
	0xfb840108,0xe3c20101,0xfb840108,0xe3c30101,0xfb840108,0xe3c40101,0xfb840108,0xe3c50101,
	0xfb840108,0xe3c60101,0xfb840108,0xe3c70101,0xfb840108,0xe3c80101,0xfb840108,0xe3c90101,
	0xfb840108,0xe3ca0101,0xfb840108,0xe3cb0101,0xfb840108,0xe3cc0101,0xfb840108,0xe3cd0101,
	0xfb840108,0xe3ce0101,0xfb840108,0xe3cf0101,0xfb840108,0xe3d00101,0xfb840108,0xe3d10101,
	0xfb840108,0xe3d20101,0xfb840108,0xe3d30101,0xfb840108,0xe3d40101,0xfb840108,0xe3d50101,
	0xfb840108,0xe3d60101,0xfb840108,0xe3d70101,0xfb840108,0xe3d80101,0xfb840108,0xe3d90101,
	0xfb840108,0xe3da0101,0xfb840108,0xe3db0101,0xfb840108,0xe3dc0101,0xfb840108,0xe3dd0101,
	0xfb840108,0xe3de0101,0xfb840108,0xe3df0101,0xfb840108,0xe3e00101,0xfb840108,0xe3e10101,
	0xfb840108,0xe3e20101,0xfb840108,0xe3e30101,0xfb840108,0xe3e40101,0xfb840108,0xe3e50101,
	0xfb840108,0xe3e60101,0xfb840108,0xe3e70101,0xfb840108,0xe3e80101,0xfb840108,0xe3e90101,
	0xfb840108,0xe3ea0101,0xfb840108,0xe3eb0101,0xfb840108,0xe3ec0101,0xfb840108,0xe3ed0101,
	0xfb840108,0xe3ee0101,0xfb840108,0xe3ef0101,0xfb840108,0xe3f00101,0xfb840108,0xe3f10101,
	0xfb840108,0xe3f20101,0xfb840108,0xe3f30101,0xfb840108,0xe3f40101,0xfb840108,0xe3f50101,
	0xfb840108,0xe3f60101,0xfb840108,0xe3f70101,0xfb840108,0xe3f80101,0xfb840108,0xe3f90101,
	0xfb840108,0xe3fa0101,0xfb840108,0xe3fb0101,0xfb840108,0xe3fc0101,0xfb840108,0xe3fd0101,
	0xfb840108,0xe3fe0101,0xfb840108,0xe3ff0101,0xfb840108,0xe4000101,0xfb840108,0xe4010101,
	0xfb840108,0xe4020101,0xfb840108,0xe4030101,0xfb840108,0xe4040101,0xfb840108,0xe4050101,
	0xfb840108,0xe4060101,0xfb840108,0xe4070101,0xfb840108,0xe4080101,0xfb840108,0xe4090101,
	0xfb840108,0xe40a0101,0xfb840108,0xe40b0101,0xfb840108,0xe40c0101,0xfb840108,0xe40d0101,
	0xfb840108,0xe40e0101,0xfb840108,0xe40f0101,0xfb840108,0xe4100101,0xfb840108,0xe4110101,
	0xfb840108,0xe4120101,0xfb840108,0xe4130101,0xfb840108,0xe4140101,0xfb840108,0xe4150101,
	0xfb840108,0xe4160101,0xfb840108,0xe4170101,0xfb840108,0xe4180101,0xfb840108,0xe4190101,
	0xfb840108,0xe41a0101,0xfb840108,0xe41b0101,0xfb840108,0xe41c0101,0xfb840108,0xe41d0101,
	0xfb840108,0xe41e0101,0xfb840108,0xe41f0101,0xfb840108,0xe4200101,0xfb840108,0xe4210101,
	0xfb840108,0xe4220101,0xfb840108,0xe4230101,0xfb840108,0xe4240101,0xfb840108,0xe4250101,
	0xfb840108,0xe4260101,0xfb840108,0xe4270101,0xfb840108,0xe4280101,0xfb840108,0xe4290101,
	0xfb840108,0xe42a0101,0xfb840108,0xe42b0101,0xfb840108,0xe42c0101,0xfb840108,0xe42d0101,
	0xfb840108,0xe42e0101,0xfb840108,0xe42f0101,0xfb840108,0xe4300101,0xfb840108,0xe4310101,
	0xfb840108,0xe4320101,0xfb840108,0xe4330101,0xfb840108,0xe4340101,0xfb840108,0xe4350101,
	0xfb840108,0xe4360101,0xfb840108,0xe4370101,0xfb840108,0xe4380101,0xfb840108,0xe4390101,
	0xfb840108,0xe43a0101,0xfb840108,0xe43b0101,0xfb840108,0xe43c0101,0xfb840108,0xe43d0101,
	0xfb840108,0xe43e0101,0xfb840108,0xe43f0101,0xfb840108,0xe4400101,0xfb840108,0xe4410101,
	0xfb840108,0xe4420101,0xfb840108,0xe4430101,0xfb840108,0xe4440101,0xfb840108,0xe4450101,
	0xfb840108,0xe4460101,0xfb840108,0xe4470101,0xfb840108,0xe4480101,0xfb840108,0xe4490101,
	0xfb840108,0xe44a0101,0xfb840108,0xe44b0101,0xfb840108,0xe44c0101,0xfb840108,0xe44d0101,
	0xfb840108,0xe44e0101,0xfb840108,0xe44f0101,0xfb840108,0xe4500101,0xfb840108,0xe4510101,
	0xfb840108,0xe4520101,0xfb840108,0xe4530101,0xfb840108,0xe4540101,0xfb840108,0xe4550101,
	0xfb840108,0xe4560101,0xfb840108,0xe4570101,0xfb840108,0xe4580101,0xfb840108,0xe4590101,
	0xfb840108,0xe45a0101,0xfb840108,0xe45b0101,0xfb840108,0xe45c0101,0xfb840108,0xe45d0101,
	0xfb840108,0xe45e0101,0xfb840108,0xe45f0101,0xfb840108,0xe4600101,0xfb840108,0xe4610101,
	0xfb840108,0xe4620101,0xfb840108,0xe4630101,0xfb840108,0xe4640101,0xfb840108,0xe4650101,
	0xfb840108,0xe4660101,0xfb840108,0xe4670101,0xfb840108,0xe4680101,0xfb840108,0xe4690101,
	0xfb840108,0xe46a0101,0xfb840108,0xe46b0101,0xfb840108,0xe46c0101,0xfb840108,0xe46d0101,
	0xfb840108,0xe46e0101,0xfb840108,0xe46f0101,0xfb840108,0xe4700101,0xfb840108,0xe4710101,
	0xfb840108,0xe4720101,0xfb840108,0xe4730101,0xfb840108,0xe4740101,0xfb840108,0xe4750101,
	0xfb840108,0xe4760101,0xfb840108,0xe4770101,0xfb840108,0xe4780101,0xfb840108,0xe4790101,
	0xfb840108,0xe47a0101,0xfb840108,0xe47b0101,0xfb840108,0xe47c0101,0xfb840108,0xe47d0101,
	0xfb840108,0xe47e0101,0xfb840108,0xe47f0101,0xfb840108,0xe4800101,0xfb840108,0xe4810101,
	0xfb840108,0xe4820101,0xfb840108,0xe4830101,0xfb840108,0xe4840101,0xfb840108,0xe4850101,
	0xfb840108,0xe4860101,0xfb840108,0xe4870101,0xfb840108,0xe4880101,0xfb840108,0xe4890101,
	0xfb840108,0xe48a0101,0xfb840108,0xe48b0101,0xfb840108,0xe48c0101,0xfb840108,0xe48d0101,
	0xfb840108,0xe48e0101,0xfb840108,0xe48f0101,0xfb840108,0xe4900101,0xfb840108,0xe4910101,
	0xfb840108,0xe4920101,0xfb840108,0xe4930101,0xfb840108,0xe4940101,0xfb840108,0xe4950101,
	0xfb840108,0xe4960101,0xfb840108,0xe4970101,0xfb840108,0xe4980101,0xfb840108,0xe4990101,
	0xfb840108,0xe49a0101,0xfb840108,0xe49b0101,0xfb840108,0xe49c0101,0xfb840108,0xe49d0101,
	0xfb840108,0xe49e0101,0xfb840108,0xe49f0101,0xfb840108,0xe4a00101,0xfb840108,0xe4a10101,
	0xfb840108,0xe4a20101,0xfb840108,0xe4a30101,0xfb840108,0xe4a40101,0xfb840108,0xe4a50101,
	0xfb840108,0xe4a60101,0xfb840108,0xe4a70101,0xfb840108,0xe4a80101,0xfb840108,0xe4a90101,
	0xfb840108,0xe4aa0101,0xfb840108,0xe4ab0101,0xfb840108,0xe4ac0101,0xfb840108,0xe4ad0101,
	0xfb840108,0xe4ae0101,0xfb840108,0xe4af0101,0xfb840108,0xe4b00101,0xfb840108,0xe4b10101,
	0xfb840108,0xe4b20101,0xfb840108,0xe4b30101,0xfb840108,0xe4b40101,0xfb840108,0xe4b50101,
	0xfb840108,0xe4b60101,0xfb840108,0xe4b70101,0xfb840108,0xe4b80101,0xfb840108,0xe4b90101,
	0xfb840108,0xe4ba0101,0xfb840108,0xe4bb0101,0xfb840108,0xe4bc0101,0xfb840108,0xe4bd0101,
	0xfb840108,0xe4be0101,0xfb840108,0xe4bf0101,0xfb840108,0xe4c00101,0xfb840108,0xe4c10101,
	0xfb840108,0xe4c20101,0xfb840108,0xe4c30101,0xfb840108,0xe4c40101,0xfb840108,0xe4c50101,
	0xfb840108,0xe4c60101,0xfb840108,0xe4c70101,0xfb840108,0xe4c80101,0xfb840108,0xe4c90101,
	0xfb840108,0xe4ca0101,0xfb840108,0xe4cb0101,0xfb840108,0xe4cc0101,0xfb840108,0xe4cd0101,
	0xfb840108,0xe4ce0101,0xfb840108,0xe4cf0101,0xfb840108,0xe4d00101,0xfb840108,0xe4d10101,
	0xfb840108,0xe4d20101,0xfb840108,0xe4d30101,0xfb840108,0xe4d40101,0xfb840108,0xe4d50101,
	0xfb840108,0xe4d60101,0xfb840108,0xe4d70101,0xfb840108,0xe4d80101,0xfb840108,0xe4d90101,
	0xfb840108,0xe4da0101,0xfb840108,0xe4db0101,0xfb840108,0xe4dc0101,0xfb840108,0xe4dd0101,
	0xfb840108,0xe4de0101,0xfb840108,0xe4df0101,0xfb840108,0xe4e00101,0xfb840108,0xe4e10101,
	0xfb840108,0xe4e20101,0xfb840108,0xe4e30101,0xfb840108,0xe4e40101,0xfb840108,0xe4e50101,
	0xfb840108,0xe4e60101,0xfb840108,0xe4e70101,0xfb840108,0xe4e80101,0xfb840108,0xe4e90101,
	0xfb840108,0xe4ea0101,0xfb840108,0xe4eb0101,0xfb840108,0xe4ec0101,0xfb840108,0xe4ed0101,
	0xfb840108,0xe4ee0101,0xfb840108,0xe4ef0101,0xfb840108,0xe4f00101,0xfb840108,0xe4f10101,
	0xfb840108,0xe4f20101,0xfb840108,0xe4f30101,0xfb840108,0xe4f40101,0xfb840108,0xe4f50101,
	0xfb840108,0xe4f60101,0xfb840108,0xe4f70101,0xfb840108,0xe4f80101,0xfb840108,0xe4f90101,
	0xfb840108,0xe4fa0101,0xfb840108,0xe4fb0101,0xfb840108,0xe4fc0101,0xfb840108,0xe4fd0101,
	0xfb840108,0xe4fe0101,0xfb840108,0xe4ff0101,0xfb840108,0xe5000101,0xfb840108,0xe5010101,
	0xfb840108,0xe5020101,0xfb840108,0xe5030101,0xfb840108,0xe5040101,0xfb840108,0xe5050101,
	0xfb840108,0xe5060101,0xfb840108,0xe5070101,0xfb840108,0xe5080101,0xfb840108,0xe5090101,
	0xfb840108,0xe50a0101,0xfb840108,0xe50b0101,0xfb840108,0xe50c0101,0xfb840108,0xe50d0101,
	0xfb840108,0xe50e0101,0xfb840108,0xe50f0101,0xfb840108,0xe5100101,0xfb840108,0xe5110101,
	0xfb840108,0xe5120101,0xfb840108,0xe5130101,0xfb840108,0xe5140101,0xfb840108,0xe5150101,
	0xfb840108,0xe5160101,0xfb840108,0xe5170101,0xfb840108,0xe5180101,0xfb840108,0xe5190101,
	0xfb840108,0xe51a0101,0xfb840108,0xe51b0101,0xfb840108,0xe51c0101,0xfb840108,0xe51d0101,
	0xfb840108,0xe51e0101,0xfb840108,0xe51f0101,0xfb840108,0xe5200101,0xfb840108,0xe5210101,
	0xfb840108,0xe5220101,0xfb840108,0xe5230101,0xfb840108,0xe5240101,0xfb840108,0xe5250101,
	0xfb840108,0xe5260101,0xfb840108,0xe5270101,0xfb840108,0xe5280101,0xfb840108,0xe5290101,
	0xfb840108,0xe52a0101,0xfb840108,0xe52b0101,0xfb840108,0xe52c0101,0xfb840108,0xe52d0101,
	0xfb840108,0xe52e0101,0xfb840108,0xe52f0101,0xfb840108,0xe5300101,0xfb840108,0xe5310101,
	0xfb840108,0xe5320101,0xfb840108,0xe5330101,0xfb840108,0xe5340101,0xfb840108,0xe5350101,
	0xfb840108,0xe5360101,0xfb840108,0xe5370101,0xfb840108,0xe5380101,0xfb840108,0xe5390101,
	0xfb840108,0xe53a0101,0xfb840108,0xe53b0101,0xfb840108,0xe53c0101,0xfb840108,0xe53d0101,
	0xfb840108,0xe53e0101,0xfb840108,0xe53f0101,0xfb840108,0xe5400101,0xfb840108,0xe5410101,
	0xfb840108,0xe5420101,0xfb840108,0xe5430101,0xfb840108,0xe5440101,0xfb840108,0xe5450101,
	0xfb840108,0xe5460101,0xfb840108,0xe5470101,0xfb840108,0xe5480101,0xfb840108,0xe5490101,
	0xfb840108,0xe54a0101,0xfb840108,0xe54b0101,0xfb840108,0xe54c0101,0xfb840108,0xe54d0101,
	0xfb840108,0xe54e0101,0xfb840108,0xe54f0101,0xfb840108,0xe5500101,0xfb840108,0xe5510101,
	0xfb840108,0xe5520101,0xfb840108,0xe5530101,0xfb840108,0xe5540101,0xfb840108,0xe5550101,
	0xfb840108,0xe5560101,0xfb840108,0xe5570101,0xfb840108,0xe5580101,0xfb840108,0xe5590101,
	0xfb840108,0xe55a0101,0xfb840108,0xe55b0101,0xfb840108,0xe55c0101,0xfb840108,0xe55d0101,
	0xfb840108,0xe55e0101,0xfb840108,0xe55f0101,0xfb840108,0xe5600101,0xfb840108,0xe5610101,
	0xfb840108,0xe5620101,0xfb840108,0xe5630101,0xfb840108,0xe5640101,0xfb840108,0xe5650101,
	0xfb840108,0xe5660101,0xfb840108,0xe5670101,0xfb840108,0xe5680101,0xfb840108,0xe5690101,
	0xfb840108,0xe56a0101,0xfb840108,0xe56b0101,0xfb840108,0xe56c0101,0xfb840108,0xe56d0101,
	0xfb840108,0xe56e0101,0xfb840108,0xe56f0101,0xfb840108,0xe5700101,0xfb840108,0xe5710101,
	0xfb840108,0xe5720101,0xfb840108,0xe5730101,0xfb840108,0xe5740101,0xfb840108,0xe5750101,
	0xfb840108,0xe5760101,0xfb840108,0xe5770101,0xfb840108,0xe5780101,0xfb840108,0xe5790101,
	0xfb840108,0xe57a0101,0xfb840108,0xe57b0101,0xfb840108,0xe57c0101,0xfb840108,0xe57d0101,
	0xfb840108,0xe57e0101,0xfb840108,0xe57f0101,0xfb840108,0xe5800101,0xfb840108,0xe5810101,
	0xfb840108,0xe5820101,0xfb840108,0xe5830101,0xfb840108,0xe5840101,0xfb840108,0xe5850101,
	0xfb840108,0xe5860101,0xfb840108,0xe5870101,0xfb840108,0xe5880101,0xfb840108,0xe5890101,
	0xfb840108,0xe58a0101,0xfb840108,0xe58b0101,0xfb840108,0xe58c0101,0xfb840108,0xe58d0101,
	0xfb840108,0xe58e0101,0xfb840108,0xe58f0101,0xfb840108,0xe5900101,0xfb840108,0xe5910101,
	0xfb840108,0xe5920101,0xfb840108,0xe5930101,0xfb840108,0xe5940101,0xfb840108,0xe5950101,
	0xfb840108,0xe5960101,0xfb840108,0xe5970101,0xfb840108,0xe5980101,0xfb840108,0xe5990101,
	0xfb840108,0xe59a0101,0xfb840108,0xe59b0101,0xfb840108,0xe59c0101,0xfb840108,0xe59d0101,
	0xfb840108,0xe59e0101,0xfb840108,0xe59f0101,0xfb840108,0xe5a00101,0xfb840108,0xe5a10101,
	0xfb840108,0xe5a20101,0xfb840108,0xe5a30101,0xfb840108,0xe5a40101,0xfb840108,0xe5a50101,
	0xfb840108,0xe5a60101,0xfb840108,0xe5a70101,0xfb840108,0xe5a80101,0xfb840108,0xe5a90101,
	0xfb840108,0xe5aa0101,0xfb840108,0xe5ab0101,0xfb840108,0xe5ac0101,0xfb840108,0xe5ad0101,
	0xfb840108,0xe5ae0101,0xfb840108,0xe5af0101,0xfb840108,0xe5b00101,0xfb840108,0xe5b10101,
	0xfb840108,0xe5b20101,0xfb840108,0xe5b30101,0xfb840108,0xe5b40101,0xfb840108,0xe5b50101,
	0xfb840108,0xe5b60101,0xfb840108,0xe5b70101,0xfb840108,0xe5b80101,0xfb840108,0xe5b90101,
	0xfb840108,0xe5ba0101,0xfb840108,0xe5bb0101,0xfb840108,0xe5bc0101,0xfb840108,0xe5bd0101,
	0xfb840108,0xe5be0101,0xfb840108,0xe5bf0101,0xfb840108,0xe5c00101,0xfb840108,0xe5c10101,
	0xfb840108,0xe5c20101,0xfb840108,0xe5c30101,0xfb840108,0xe5c40101,0xfb840108,0xe5c50101,
	0xfb840108,0xe5c60101,0xfb840108,0xe5c70101,0xfb840108,0xe5c80101,0xfb840108,0xe5c90101,
	0xfb840108,0xe5ca0101,0xfb840108,0xe5cb0101,0xfb840108,0xe5cc0101,0xfb840108,0xe5cd0101,
	0xfb840108,0xe5ce0101,0xfb840108,0xe5cf0101,0xfb840108,0xe5d00101,0xfb840108,0xe5d10101,
	0xfb840108,0xe5d20101,0xfb840108,0xe5d30101,0xfb840108,0xe5d40101,0xfb840108,0xe5d50101,
	0xfb840108,0xe5d60101,0xfb840108,0xe5d70101,0xfb840108,0xe5d80101,0xfb840108,0xe5d90101,
	0xfb840108,0xe5da0101,0xfb840108,0xe5db0101,0xfb840108,0xe5dc0101,0xfb840108,0xe5dd0101,
	0xfb840108,0xe5de0101,0xfb840108,0xe5df0101,0xfb840108,0xe5e00101,0xfb840108,0xe5e10101,
	0xfb840108,0xe5e20101,0xfb840108,0xe5e30101,0xfb840108,0xe5e40101,0xfb840108,0xe5e50101,
	0xfb840108,0xe5e60101,0xfb840108,0xe5e70101,0xfb840108,0xe5e80101,0xfb840108,0xe5e90101,
	0xfb840108,0xe5ea0101,0xfb840108,0xe5eb0101,0xfb840108,0xe5ec0101,0xfb840108,0xe5ed0101,
	0xfb840108,0xe5ee0101,0xfb840108,0xe5ef0101,0xfb840108,0xe5f00101,0xfb840108,0xe5f10101,
	0xfb840108,0xe5f20101,0xfb840108,0xe5f30101,0xfb840108,0xe5f40101,0xfb840108,0xe5f50101,
	0xfb840108,0xe5f60101,0xfb840108,0xe5f70101,0xfb840108,0xe5f80101,0xfb840108,0xe5f90101,
	0xfb840108,0xe5fa0101,0xfb840108,0xe5fb0101,0xfb840108,0xe5fc0101,0xfb840108,0xe5fd0101,
	0xfb840108,0xe5fe0101,0xfb840108,0xe5ff0101,0xfb840108,0xe6000101,0xfb840108,0xe6010101,
	0xfb840108,0xe6020101,0xfb840108,0xe6030101,0xfb840108,0xe6040101,0xfb840108,0xe6050101,
	0xfb840108,0xe6060101,0xfb840108,0xe6070101,0xfb840108,0xe6080101,0xfb840108,0xe6090101,
	0xfb840108,0xe60a0101,0xfb840108,0xe60b0101,0xfb840108,0xe60c0101,0xfb840108,0xe60d0101,
	0xfb840108,0xe60e0101,0xfb840108,0xe60f0101,0xfb840108,0xe6100101,0xfb840108,0xe6110101,
	0xfb840108,0xe6120101,0xfb840108,0xe6130101,0xfb840108,0xe6140101,0xfb840108,0xe6150101,
	0xfb840108,0xe6160101,0xfb840108,0xe6170101,0xfb840108,0xe6180101,0xfb840108,0xe6190101,
	0xfb840108,0xe61a0101,0xfb840108,0xe61b0101,0xfb840108,0xe61c0101,0xfb840108,0xe61d0101,
	0xfb840108,0xe61e0101,0xfb840108,0xe61f0101,0xfb840108,0xe6200101,0xfb840108,0xe6210101,
	0xfb840108,0xe6220101,0xfb840108,0xe6230101,0xfb840108,0xe6240101,0xfb840108,0xe6250101,
	0xfb840108,0xe6260101,0xfb840108,0xe6270101,0xfb840108,0xe6280101,0xfb840108,0xe6290101,
	0xfb840108,0xe62a0101,0xfb840108,0xe62b0101,0xfb840108,0xe62c0101,0xfb840108,0xe62d0101,
	0xfb840108,0xe62e0101,0xfb840108,0xe62f0101,0xfb840108,0xe6300101,0xfb840108,0xe6310101,
	0xfb840108,0xe6320101,0xfb840108,0xe6330101,0xfb840108,0xe6340101,0xfb840108,0xe6350101,
	0xfb840108,0xe6360101,0xfb840108,0xe6370101,0xfb840108,0xe6380101,0xfb840108,0xe6390101,
	0xfb840108,0xe63a0101,0xfb840108,0xe63b0101,0xfb840108,0xe63c0101,0xfb840108,0xe63d0101,
	0xfb840108,0xe63e0101,0xfb840108,0xe63f0101,0xfb840108,0xe6400101,0xfb840108,0xe6410101,
	0xfb840108,0xe6420101,0xfb840108,0xe6430101,0xfb840108,0xe6440101,0xfb840108,0xe6450101,
	0xfb840108,0xe6460101,0xfb840108,0xe6470101,0xfb840108,0xe6480101,0xfb840108,0xe6490101,
	0xfb840108,0xe64a0101,0xfb840108,0xe64b0101,0xfb840108,0xe64c0101,0xfb840108,0xe64d0101,
	0xfb840108,0xe64e0101,0xfb840108,0xe64f0101,0xfb840108,0xe6500101,0xfb840108,0xe6510101,
	0xfb840108,0xe6520101,0xfb840108,0xe6530101,0xfb840108,0xe6540101,0xfb840108,0xe6550101,
	0xfb840108,0xe6560101,0xfb840108,0xe6570101,0xfb840108,0xe6580101,0xfb840108,0xe6590101,
	0xfb840108,0xe65a0101,0xfb840108,0xe65b0101,0xfb840108,0xe65c0101,0xfb840108,0xe65d0101,
	0xfb840108,0xe65e0101,0xfb840108,0xe65f0101,0xfb840108,0xe6600101,0xfb840108,0xe6610101,
	0xfb840108,0xe6620101,0xfb840108,0xe6630101,0xfb840108,0xe6640101,0xfb840108,0xe6650101,
	0xfb840108,0xe6660101,0xfb840108,0xe6670101,0xfb840108,0xe6680101,0xfb840108,0xe6690101,
	0xfb840108,0xe66a0101,0xfb840108,0xe66b0101,0xfb840108,0xe66c0101,0xfb840108,0xe66d0101,
	0xfb840108,0xe66e0101,0xfb840108,0xe66f0101,0xfb840108,0xe6700101,0xfb840108,0xe6710101,
	0xfb840108,0xe6720101,0xfb840108,0xe6730101,0xfb840108,0xe6740101,0xfb840108,0xe6750101,
	0xfb840108,0xe6760101,0xfb840108,0xe6770101,0xfb840108,0xe6780101,0xfb840108,0xe6790101,
	0xfb840108,0xe67a0101,0xfb840108,0xe67b0101,0xfb840108,0xe67c0101,0xfb840108,0xe67d0101,
	0xfb840108,0xe67e0101,0xfb840108,0xe67f0101,0xfb840108,0xe6800101,0xfb840108,0xe6810101,
	0xfb840108,0xe6820101,0xfb840108,0xe6830101,0xfb840108,0xe6840101,0xfb840108,0xe6850101,
	0xfb840108,0xe6860101,0xfb840108,0xe6870101,0xfb840108,0xe6880101,0xfb840108,0xe6890101,
	0xfb840108,0xe68a0101,0xfb840108,0xe68b0101,0xfb840108,0xe68c0101,0xfb840108,0xe68d0101,
	0xfb840108,0xe68e0101,0xfb840108,0xe68f0101,0xfb840108,0xe6900101,0xfb840108,0xe6910101,
	0xfb840108,0xe6920101,0xfb840108,0xe6930101,0xfb840108,0xe6940101,0xfb840108,0xe6950101,
	0xfb840108,0xe6960101,0xfb840108,0xe6970101,0xfb840108,0xe6980101,0xfb840108,0xe6990101,
	0xfb840108,0xe69a0101,0xfb840108,0xe69b0101,0xfb840108,0xe69c0101,0xfb840108,0xe69d0101,
	0xfb840108,0xe69e0101,0xfb840108,0xe69f0101,0xfb840108,0xe6a00101,0xfb840108,0xe6a10101,
	0xfb840108,0xe6a20101,0xfb840108,0xe6a30101,0xfb840108,0xe6a40101,0xfb840108,0xe6a50101,
	0xfb840108,0xe6a60101,0xfb840108,0xe6a70101,0xfb840108,0xe6a80101,0xfb840108,0xe6a90101,
	0xfb840108,0xe6aa0101,0xfb840108,0xe6ab0101,0xfb840108,0xe6ac0101,0xfb840108,0xe6ad0101,
	0xfb840108,0xe6ae0101,0xfb840108,0xe6af0101,0xfb840108,0xe6b00101,0xfb840108,0xe6b10101,
	0xfb840108,0xe6b20101,0xfb840108,0xe6b30101,0xfb840108,0xe6b40101,0xfb840108,0xe6b50101,
	0xfb840108,0xe6b60101,0xfb840108,0xe6b70101,0xfb840108,0xe6b80101,0xfb840108,0xe6b90101,
	0xfb840108,0xe6ba0101,0xfb840108,0xe6bb0101,0xfb840108,0xe6bc0101,0xfb840108,0xe6bd0101,
	0xfb840108,0xe6be0101,0xfb840108,0xe6bf0101,0xfb840108,0xe6c00101,0xfb840108,0xe6c10101,
	0xfb840108,0xe6c20101,0xfb840108,0xe6c30101,0xfb840108,0xe6c40101,0xfb840108,0xe6c50101,
	0xfb840108,0xe6c60101,0xfb840108,0xe6c70101,0xfb840108,0xe6c80101,0xfb840108,0xe6c90101,
	0xfb840108,0xe6ca0101,0xfb840108,0xe6cb0101,0xfb840108,0xe6cc0101,0xfb840108,0xe6cd0101,
	0xfb840108,0xe6ce0101,0xfb840108,0xe6cf0101,0xfb840108,0xe6d00101,0xfb840108,0xe6d10101,
	0xfb840108,0xe6d20101,0xfb840108,0xe6d30101,0xfb840108,0xe6d40101,0xfb840108,0xe6d50101,
	0xfb840108,0xe6d60101,0xfb840108,0xe6d70101,0xfb840108,0xe6d80101,0xfb840108,0xe6d90101,
	0xfb840108,0xe6da0101,0xfb840108,0xe6db0101,0xfb840108,0xe6dc0101,0xfb840108,0xe6dd0101,
	0xfb840108,0xe6de0101,0xfb840108,0xe6df0101,0xfb840108,0xe6e00101,0xfb840108,0xe6e10101,
	0xfb840108,0xe6e20101,0xfb840108,0xe6e30101,0xfb840108,0xe6e40101,0xfb840108,0xe6e50101,
	0xfb840108,0xe6e60101,0xfb840108,0xe6e70101,0xfb840108,0xe6e80101,0xfb840108,0xe6e90101,
	0xfb840108,0xe6ea0101,0xfb840108,0xe6eb0101,0xfb840108,0xe6ec0101,0xfb840108,0xe6ed0101,
	0xfb840108,0xe6ee0101,0xfb840108,0xe6ef0101,0xfb840108,0xe6f00101,0xfb840108,0xe6f10101,
	0xfb840108,0xe6f20101,0xfb840108,0xe6f30101,0xfb840108,0xe6f40101,0xfb840108,0xe6f50101,
	0xfb840108,0xe6f60101,0xfb840108,0xe6f70101,0xfb840108,0xe6f80101,0xfb840108,0xe6f90101,
	0xfb840108,0xe6fa0101,0xfb840108,0xe6fb0101,0xfb840108,0xe6fc0101,0xfb840108,0xe6fd0101,
	0xfb840108,0xe6fe0101,0xfb840108,0xe6ff0101,0xfb840108,0xe7000101,0xfb840108,0xe7010101,
	0xfb840108,0xe7020101,0xfb840108,0xe7030101,0xfb840108,0xe7040101,0xfb840108,0xe7050101,
	0xfb840108,0xe7060101,0xfb840108,0xe7070101,0xfb840108,0xe7080101,0xfb840108,0xe7090101,
	0xfb840108,0xe70a0101,0xfb840108,0xe70b0101,0xfb840108,0xe70c0101,0xfb840108,0xe70d0101,
	0xfb840108,0xe70e0101,0xfb840108,0xe70f0101,0xfb840108,0xe7100101,0xfb840108,0xe7110101,
	0xfb840108,0xe7120101,0xfb840108,0xe7130101,0xfb840108,0xe7140101,0xfb840108,0xe7150101,
	0xfb840108,0xe7160101,0xfb840108,0xe7170101,0xfb840108,0xe7180101,0xfb840108,0xe7190101,
	0xfb840108,0xe71a0101,0xfb840108,0xe71b0101,0xfb840108,0xe71c0101,0xfb840108,0xe71d0101,
	0xfb840108,0xe71e0101,0xfb840108,0xe71f0101,0xfb840108,0xe7200101,0xfb840108,0xe7210101,
	0xfb840108,0xe7220101,0xfb840108,0xe7230101,0xfb840108,0xe7240101,0xfb840108,0xe7250101,
	0xfb840108,0xe7260101,0xfb840108,0xe7270101,0xfb840108,0xe7280101,0xfb840108,0xe7290101,
	0xfb840108,0xe72a0101,0xfb840108,0xe72b0101,0xfb840108,0xe72c0101,0xfb840108,0xe72d0101,
	0xfb840108,0xe72e0101,0xfb840108,0xe72f0101,0xfb840108,0xe7300101,0xfb840108,0xe7310101,
	0xfb840108,0xe7320101,0xfb840108,0xe7330101,0xfb840108,0xe7340101,0xfb840108,0xe7350101,
	0xfb840108,0xe7360101,0xfb840108,0xe7370101,0xfb840108,0xe7380101,0xfb840108,0xe7390101,
	0xfb840108,0xe73a0101,0xfb840108,0xe73b0101,0xfb840108,0xe73c0101,0xfb840108,0xe73d0101,
	0xfb840108,0xe73e0101,0xfb840108,0xe73f0101,0xfb840108,0xe7400101,0xfb840108,0xe7410101,
	0xfb840108,0xe7420101,0xfb840108,0xe7430101,0xfb840108,0xe7440101,0xfb840108,0xe7450101,
	0xfb840108,0xe7460101,0xfb840108,0xe7470101,0xfb840108,0xe7480101,0xfb840108,0xe7490101,
	0xfb840108,0xe74a0101,0xfb840108,0xe74b0101,0xfb840108,0xe74c0101,0xfb840108,0xe74d0101,
	0xfb840108,0xe74e0101,0xfb840108,0xe74f0101,0xfb840108,0xe7500101,0xfb840108,0xe7510101,
	0xfb840108,0xe7520101,0xfb840108,0xe7530101,0xfb840108,0xe7540101,0xfb840108,0xe7550101,
	0xfb840108,0xe7560101,0xfb840108,0xe7570101,0xfb840108,0xe7580101,0xfb840108,0xe7590101,
	0xfb840108,0xe75a0101,0xfb840108,0xe75b0101,0xfb840108,0xe75c0101,0xfb840108,0xe75d0101,
	0xfb840108,0xe75e0101,0xfb840108,0xe75f0101,0xfb840108,0xe7600101,0xfb840108,0xe7610101,
	0xfb840108,0xe7620101,0xfb840108,0xe7630101,0xfb840108,0xe7640101,0xfb840108,0xe7650101,
	0xfb840108,0xe7660101,0xfb840108,0xe7670101,0xfb840108,0xe7680101,0xfb840108,0xe7690101,
	0xfb840108,0xe76a0101,0xfb840108,0xe76b0101,0xfb840108,0xe76c0101,0xfb840108,0xe76d0101,
	0xfb840108,0xe76e0101,0xfb840108,0xe76f0101,0xfb840108,0xe7700101,0xfb840108,0xe7710101,
	0xfb840108,0xe7720101,0xfb840108,0xe7730101,0xfb840108,0xe7740101,0xfb840108,0xe7750101,
	0xfb840108,0xe7760101,0xfb840108,0xe7770101,0xfb840108,0xe7780101,0xfb840108,0xe7790101,
	0xfb840108,0xe77a0101,0xfb840108,0xe77b0101,0xfb840108,0xe77c0101,0xfb840108,0xe77d0101,
	0xfb840108,0xe77e0101,0xfb840108,0xe77f0101,0xfb840108,0xe7800101,0xfb840108,0xe7810101,
	0xfb840108,0xe7820101,0xfb840108,0xe7830101,0xfb840108,0xe7840101,0xfb840108,0xe7850101,
	0xfb840108,0xe7860101,0xfb840108,0xe7870101,0xfb840108,0xe7880101,0xfb840108,0xe7890101,
	0xfb840108,0xe78a0101,0xfb840108,0xe78b0101,0xfb840108,0xe78c0101,0xfb840108,0xe78d0101,
	0xfb840108,0xe78e0101,0xfb840108,0xe78f0101,0xfb840108,0xe7900101,0xfb840108,0xe7910101,
	0xfb840108,0xe7920101,0xfb840108,0xe7930101,0xfb840108,0xe7940101,0xfb840108,0xe7950101,
	0xfb840108,0xe7960101,0xfb840108,0xe7970101,0xfb840108,0xe7980101,0xfb840108,0xe7990101,
	0xfb840108,0xe79a0101,0xfb840108,0xe79b0101,0xfb840108,0xe79c0101,0xfb840108,0xe79d0101,
	0xfb840108,0xe79e0101,0xfb840108,0xe79f0101,0xfb840108,0xe7a00101,0xfb840108,0xe7a10101,
	0xfb840108,0xe7a20101,0xfb840108,0xe7a30101,0xfb840108,0xe7a40101,0xfb840108,0xe7a50101,
	0xfb840108,0xe7a60101,0xfb840108,0xe7a70101,0xfb840108,0xe7a80101,0xfb840108,0xe7a90101,
	0xfb840108,0xe7aa0101,0xfb840108,0xe7ab0101,0xfb840108,0xe7ac0101,0xfb840108,0xe7ad0101,
	0xfb840108,0xe7ae0101,0xfb840108,0xe7af0101,0xfb840108,0xe7b00101,0xfb840108,0xe7b10101,
	0xfb840108,0xe7b20101,0xfb840108,0xe7b30101,0xfb840108,0xe7b40101,0xfb840108,0xe7b50101,
	0xfb840108,0xe7b60101,0xfb840108,0xe7b70101,0xfb840108,0xe7b80101,0xfb840108,0xe7b90101,
	0xfb840108,0xe7ba0101,0xfb840108,0xe7bb0101,0xfb840108,0xe7bc0101,0xfb840108,0xe7bd0101,
	0xfb840108,0xe7be0101,0xfb840108,0xe7bf0101,0xfb840108,0xe7c00101,0xfb840108,0xe7c10101,
	0xfb840108,0xe7c20101,0xfb840108,0xe7c30101,0xfb840108,0xe7c40101,0xfb840108,0xe7c50101,
	0xfb840108,0xe7c60101,0xfb840108,0xe7c70101,0xfb840108,0xe7c80101,0xfb840108,0xe7c90101,
	0xfb840108,0xe7ca0101,0xfb840108,0xe7cb0101,0xfb840108,0xe7cc0101,0xfb840108,0xe7cd0101,
	0xfb840108,0xe7ce0101,0xfb840108,0xe7cf0101,0xfb840108,0xe7d00101,0xfb840108,0xe7d10101,
	0xfb840108,0xe7d20101,0xfb840108,0xe7d30101,0xfb840108,0xe7d40101,0xfb840108,0xe7d50101,
	0xfb840108,0xe7d60101,0xfb840108,0xe7d70101,0xfb840108,0xe7d80101,0xfb840108,0xe7d90101,
	0xfb840108,0xe7da0101,0xfb840108,0xe7db0101,0xfb840108,0xe7dc0101,0xfb840108,0xe7dd0101,
	0xfb840108,0xe7de0101,0xfb840108,0xe7df0101,0xfb840108,0xe7e00101,0xfb840108,0xe7e10101,
	0xfb840108,0xe7e20101,0xfb840108,0xe7e30101,0xfb840108,0xe7e40101,0xfb840108,0xe7e50101,
	0xfb840108,0xe7e60101,0xfb840108,0xe7e70101,0xfb840108,0xe7e80101,0xfb840108,0xe7e90101,
	0xfb840108,0xe7ea0101,0xfb840108,0xe7eb0101,0xfb840108,0xe7ec0101,0xfb840108,0xe7ed0101,
	0xfb840108,0xe7ee0101,0xfb840108,0xe7ef0101,0xfb840108,0xe7f00101,0xfb840108,0xe7f10101,
	0xfb840108,0xe7f20101,0xfb840108,0xe7f30101,0xfb840108,0xe7f40101,0xfb840108,0xe7f50101,
	0xfb840108,0xe7f60101,0xfb840108,0xe7f70101,0xfb840108,0xe7f80101,0xfb840108,0xe7f90101,
	0xfb840108,0xe7fa0101,0xfb840108,0xe7fb0101,0xfb840108,0xe7fc0101,0xfb840108,0xe7fd0101,
	0xfb840108,0xe7fe0101,0xfb840108,0xe7ff0101,0xfb840108,0xe8000101,0xfb840108,0xe8010101,
	0xfb840108,0xe8020101,0xfb840108,0xe8030101,0xfb840108,0xe8040101,0xfb840108,0xe8050101,
	0xfb840108,0xe8060101,0xfb840108,0xe8070101,0xfb840108,0xe8080101,0xfb840108,0xe8090101,
	0xfb840108,0xe80a0101,0xfb840108,0xe80b0101,0xfb840108,0xe80c0101,0xfb840108,0xe80d0101,
	0xfb840108,0xe80e0101,0xfb840108,0xe80f0101,0xfb840108,0xe8100101,0xfb840108,0xe8110101,
	0xfb840108,0xe8120101,0xfb840108,0xe8130101,0xfb840108,0xe8140101,0xfb840108,0xe8150101,
	0xfb840108,0xe8160101,0xfb840108,0xe8170101,0xfb840108,0xe8180101,0xfb840108,0xe8190101,
	0xfb840108,0xe81a0101,0xfb840108,0xe81b0101,0xfb840108,0xe81c0101,0xfb840108,0xe81d0101,
	0xfb840108,0xe81e0101,0xfb840108,0xe81f0101,0xfb840108,0xe8200101,0xfb840108,0xe8210101,
	0xfb840108,0xe8220101,0xfb840108,0xe8230101,0xfb840108,0xe8240101,0xfb840108,0xe8250101,
	0xfb840108,0xe8260101,0xfb840108,0xe8270101,0xfb840108,0xe8280101,0xfb840108,0xe8290101,
	0xfb840108,0xe82a0101,0xfb840108,0xe82b0101,0xfb840108,0xe82c0101,0xfb840108,0xe82d0101,
	0xfb840108,0xe82e0101,0xfb840108,0xe82f0101,0xfb840108,0xe8300101,0xfb840108,0xe8310101,
	0xfb840108,0xe8320101,0xfb840108,0xe8330101,0xfb840108,0xe8340101,0xfb840108,0xe8350101,
	0xfb840108,0xe8360101,0xfb840108,0xe8370101,0xfb840108,0xe8380101,0xfb840108,0xe8390101,
	0xfb840108,0xe83a0101,0xfb840108,0xe83b0101,0xfb840108,0xe83c0101,0xfb840108,0xe83d0101,
	0xfb840108,0xe83e0101,0xfb840108,0xe83f0101,0xfb840108,0xe8400101,0xfb840108,0xe8410101,
	0xfb840108,0xe8420101,0xfb840108,0xe8430101,0xfb840108,0xe8440101,0xfb840108,0xe8450101,
	0xfb840108,0xe8460101,0xfb840108,0xe8470101,0xfb840108,0xe8480101,0xfb840108,0xe8490101,
	0xfb840108,0xe84a0101,0xfb840108,0xe84b0101,0xfb840108,0xe84c0101,0xfb840108,0xe84d0101,
	0xfb840108,0xe84e0101,0xfb840108,0xe84f0101,0xfb840108,0xe8500101,0xfb840108,0xe8510101,
	0xfb840108,0xe8520101,0xfb840108,0xe8530101,0xfb840108,0xe8540101,0xfb840108,0xe8550101,
	0xfb840108,0xe8560101,0xfb840108,0xe8570101,0xfb840108,0xe8580101,0xfb840108,0xe8590101,
	0xfb840108,0xe85a0101,0xfb840108,0xe85b0101,0xfb840108,0xe85c0101,0xfb840108,0xe85d0101,
	0xfb840108,0xe85e0101,0xfb840108,0xe85f0101,0xfb840108,0xe8600101,0xfb840108,0xe8610101,
	0xfb840108,0xe8620101,0xfb840108,0xe8630101,0xfb840108,0xe8640101,0xfb840108,0xe8650101,
	0xfb840108,0xe8660101,0xfb840108,0xe8670101,0xfb840108,0xe8680101,0xfb840108,0xe8690101,
	0xfb840108,0xe86a0101,0xfb840108,0xe86b0101,0xfb840108,0xe86c0101,0xfb840108,0xe86d0101,
	0xfb840108,0xe86e0101,0xfb840108,0xe86f0101,0xfb840108,0xe8700101,0xfb840108,0xe8710101,
	0xfb840108,0xe8720101,0xfb840108,0xe8730101,0xfb840108,0xe8740101,0xfb840108,0xe8750101,
	0xfb840108,0xe8760101,0xfb840108,0xe8770101,0xfb840108,0xe8780101,0xfb840108,0xe8790101,
	0xfb840108,0xe87a0101,0xfb840108,0xe87b0101,0xfb840108,0xe87c0101,0xfb840108,0xe87d0101,
	0xfb840108,0xe87e0101,0xfb840108,0xe87f0101,0xfb840108,0xe8800101,0xfb840108,0xe8810101,
	0xfb840108,0xe8820101,0xfb840108,0xe8830101,0xfb840108,0xe8840101,0xfb840108,0xe8850101,
	0xfb840108,0xe8860101,0xfb840108,0xe8870101,0xfb840108,0xe8880101,0xfb840108,0xe8890101,
	0xfb840108,0xe88a0101,0xfb840108,0xe88b0101,0xfb840108,0xe88c0101,0xfb840108,0xe88d0101,
	0xfb840108,0xe88e0101,0xfb840108,0xe88f0101,0xfb840108,0xe8900101,0xfb840108,0xe8910101,
	0xfb840108,0xe8920101,0xfb840108,0xe8930101,0xfb840108,0xe8940101,0xfb840108,0xe8950101,
	0xfb840108,0xe8960101,0xfb840108,0xe8970101,0xfb840108,0xe8980101,0xfb840108,0xe8990101,
	0xfb840108,0xe89a0101,0xfb840108,0xe89b0101,0xfb840108,0xe89c0101,0xfb840108,0xe89d0101,
	0xfb840108,0xe89e0101,0xfb840108,0xe89f0101,0xfb840108,0xe8a00101,0xfb840108,0xe8a10101,
	0xfb840108,0xe8a20101,0xfb840108,0xe8a30101,0xfb840108,0xe8a40101,0xfb840108,0xe8a50101,
	0xfb840108,0xe8a60101,0xfb840108,0xe8a70101,0xfb840108,0xe8a80101,0xfb840108,0xe8a90101,
	0xfb840108,0xe8aa0101,0xfb840108,0xe8ab0101,0xfb840108,0xe8ac0101,0xfb840108,0xe8ad0101,
	0xfb840108,0xe8ae0101,0xfb840108,0xe8af0101,0xfb840108,0xe8b00101,0xfb840108,0xe8b10101,
	0xfb840108,0xe8b20101,0xfb840108,0xe8b30101,0xfb840108,0xe8b40101,0xfb840108,0xe8b50101,
	0xfb840108,0xe8b60101,0xfb840108,0xe8b70101,0xfb840108,0xe8b80101,0xfb840108,0xe8b90101,
	0xfb840108,0xe8ba0101,0xfb840108,0xe8bb0101,0xfb840108,0xe8bc0101,0xfb840108,0xe8bd0101,
	0xfb840108,0xe8be0101,0xfb840108,0xe8bf0101,0xfb840108,0xe8c00101,0xfb840108,0xe8c10101,
	0xfb840108,0xe8c20101,0xfb840108,0xe8c30101,0xfb840108,0xe8c40101,0xfb840108,0xe8c50101,
	0xfb840108,0xe8c60101,0xfb840108,0xe8c70101,0xfb840108,0xe8c80101,0xfb840108,0xe8c90101,
	0xfb840108,0xe8ca0101,0xfb840108,0xe8cb0101,0xfb840108,0xe8cc0101,0xfb840108,0xe8cd0101,
	0xfb840108,0xe8ce0101,0xfb840108,0xe8cf0101,0xfb840108,0xe8d00101,0xfb840108,0xe8d10101,
	0xfb840108,0xe8d20101,0xfb840108,0xe8d30101,0xfb840108,0xe8d40101,0xfb840108,0xe8d50101,
	0xfb840108,0xe8d60101,0xfb840108,0xe8d70101,0xfb840108,0xe8d80101,0xfb840108,0xe8d90101,
	0xfb840108,0xe8da0101,0xfb840108,0xe8db0101,0xfb840108,0xe8dc0101,0xfb840108,0xe8dd0101,
	0xfb840108,0xe8de0101,0xfb840108,0xe8df0101,0xfb840108,0xe8e00101,0xfb840108,0xe8e10101,
	0xfb840108,0xe8e20101,0xfb840108,0xe8e30101,0xfb840108,0xe8e40101,0xfb840108,0xe8e50101,
	0xfb840108,0xe8e60101,0xfb840108,0xe8e70101,0xfb840108,0xe8e80101,0xfb840108,0xe8e90101,
	0xfb840108,0xe8ea0101,0xfb840108,0xe8eb0101,0xfb840108,0xe8ec0101,0xfb840108,0xe8ed0101,
	0xfb840108,0xe8ee0101,0xfb840108,0xe8ef0101,0xfb840108,0xe8f00101,0xfb840108,0xe8f10101,
	0xfb840108,0xe8f20101,0xfb840108,0xe8f30101,0xfb840108,0xe8f40101,0xfb840108,0xe8f50101,
	0xfb840108,0xe8f60101,0xfb840108,0xe8f70101,0xfb840108,0xe8f80101,0xfb840108,0xe8f90101,
	0xfb840108,0xe8fa0101,0xfb840108,0xe8fb0101,0xfb840108,0xe8fc0101,0xfb840108,0xe8fd0101,
	0xfb840108,0xe8fe0101,0xfb840108,0xe8ff0101,0xfb840108,0xe9000101,0xfb840108,0xe9010101,
	0xfb840108,0xe9020101,0xfb840108,0xe9030101,0xfb840108,0xe9040101,0xfb840108,0xe9050101,
	0xfb840108,0xe9060101,0xfb840108,0xe9070101,0xfb840108,0xe9080101,0xfb840108,0xe9090101,
	0xfb840108,0xe90a0101,0xfb840108,0xe90b0101,0xfb840108,0xe90c0101,0xfb840108,0xe90d0101,
	0xfb840108,0xe90e0101,0xfb840108,0xe90f0101,0xfb840108,0xe9100101,0xfb840108,0xe9110101,
	0xfb840108,0xe9120101,0xfb840108,0xe9130101,0xfb840108,0xe9140101,0xfb840108,0xe9150101,
	0xfb840108,0xe9160101,0xfb840108,0xe9170101,0xfb840108,0xe9180101,0xfb840108,0xe9190101,
	0xfb840108,0xe91a0101,0xfb840108,0xe91b0101,0xfb840108,0xe91c0101,0xfb840108,0xe91d0101,
	0xfb840108,0xe91e0101,0xfb840108,0xe91f0101,0xfb840108,0xe9200101,0xfb840108,0xe9210101,
	0xfb840108,0xe9220101,0xfb840108,0xe9230101,0xfb840108,0xe9240101,0xfb840108,0xe9250101,
	0xfb840108,0xe9260101,0xfb840108,0xe9270101,0xfb840108,0xe9280101,0xfb840108,0xe9290101,
	0xfb840108,0xe92a0101,0xfb840108,0xe92b0101,0xfb840108,0xe92c0101,0xfb840108,0xe92d0101,
	0xfb840108,0xe92e0101,0xfb840108,0xe92f0101,0xfb840108,0xe9300101,0xfb840108,0xe9310101,
	0xfb840108,0xe9320101,0xfb840108,0xe9330101,0xfb840108,0xe9340101,0xfb840108,0xe9350101,
	0xfb840108,0xe9360101,0xfb840108,0xe9370101,0xfb840108,0xe9380101,0xfb840108,0xe9390101,
	0xfb840108,0xe93a0101,0xfb840108,0xe93b0101,0xfb840108,0xe93c0101,0xfb840108,0xe93d0101,
	0xfb840108,0xe93e0101,0xfb840108,0xe93f0101,0xfb840108,0xe9400101,0xfb840108,0xe9410101,
	0xfb840108,0xe9420101,0xfb840108,0xe9430101,0xfb840108,0xe9440101,0xfb840108,0xe9450101,
	0xfb840108,0xe9460101,0xfb840108,0xe9470101,0xfb840108,0xe9480101,0xfb840108,0xe9490101,
	0xfb840108,0xe94a0101,0xfb840108,0xe94b0101,0xfb840108,0xe94c0101,0xfb840108,0xe94d0101,
	0xfb840108,0xe94e0101,0xfb840108,0xe94f0101,0xfb840108,0xe9500101,0xfb840108,0xe9510101,
	0xfb840108,0xe9520101,0xfb840108,0xe9530101,0xfb840108,0xe9540101,0xfb840108,0xe9550101,
	0xfb840108,0xe9560101,0xfb840108,0xe9570101,0xfb840108,0xe9580101,0xfb840108,0xe9590101,
	0xfb840108,0xe95a0101,0xfb840108,0xe95b0101,0xfb840108,0xe95c0101,0xfb840108,0xe95d0101,
	0xfb840108,0xe95e0101,0xfb840108,0xe95f0101,0xfb840108,0xe9600101,0xfb840108,0xe9610101,
	0xfb840108,0xe9620101,0xfb840108,0xe9630101,0xfb840108,0xe9640101,0xfb840108,0xe9650101,
	0xfb840108,0xe9660101,0xfb840108,0xe9670101,0xfb840108,0xe9680101,0xfb840108,0xe9690101,
	0xfb840108,0xe96a0101,0xfb840108,0xe96b0101,0xfb840108,0xe96c0101,0xfb840108,0xe96d0101,
	0xfb840108,0xe96e0101,0xfb840108,0xe96f0101,0xfb840108,0xe9700101,0xfb840108,0xe9710101,
	0xfb840108,0xe9720101,0xfb840108,0xe9730101,0xfb840108,0xe9740101,0xfb840108,0xe9750101,
	0xfb840108,0xe9760101,0xfb840108,0xe9770101,0xfb840108,0xe9780101,0xfb840108,0xe9790101,
	0xfb840108,0xe97a0101,0xfb840108,0xe97b0101,0xfb840108,0xe97c0101,0xfb840108,0xe97d0101,
	0xfb840108,0xe97e0101,0xfb840108,0xe97f0101,0xfb840108,0xe9800101,0xfb840108,0xe9810101,
	0xfb840108,0xe9820101,0xfb840108,0xe9830101,0xfb840108,0xe9840101,0xfb840108,0xe9850101,
	0xfb840108,0xe9860101,0xfb840108,0xe9870101,0xfb840108,0xe9880101,0xfb840108,0xe9890101,
	0xfb840108,0xe98a0101,0xfb840108,0xe98b0101,0xfb840108,0xe98c0101,0xfb840108,0xe98d0101,
	0xfb840108,0xe98e0101,0xfb840108,0xe98f0101,0xfb840108,0xe9900101,0xfb840108,0xe9910101,
	0xfb840108,0xe9920101,0xfb840108,0xe9930101,0xfb840108,0xe9940101,0xfb840108,0xe9950101,
	0xfb840108,0xe9960101,0xfb840108,0xe9970101,0xfb840108,0xe9980101,0xfb840108,0xe9990101,
	0xfb840108,0xe99a0101,0xfb840108,0xe99b0101,0xfb840108,0xe99c0101,0xfb840108,0xe99d0101,
	0xfb840108,0xe99e0101,0xfb840108,0xe99f0101,0xfb840108,0xe9a00101,0xfb840108,0xe9a10101,
	0xfb840108,0xe9a20101,0xfb840108,0xe9a30101,0xfb840108,0xe9a40101,0xfb840108,0xe9a50101,
	0xfb840108,0xe9a60101,0xfb840108,0xe9a70101,0xfb840108,0xe9a80101,0xfb840108,0xe9a90101,
	0xfb840108,0xe9aa0101,0xfb840108,0xe9ab0101,0xfb840108,0xe9ac0101,0xfb840108,0xe9ad0101,
	0xfb840108,0xe9ae0101,0xfb840108,0xe9af0101,0xfb840108,0xe9b00101,0xfb840108,0xe9b10101,
	0xfb840108,0xe9b20101,0xfb840108,0xe9b30101,0xfb840108,0xe9b40101,0xfb840108,0xe9b50101,
	0xfb840108,0xe9b60101,0xfb840108,0xe9b70101,0xfb840108,0xe9b80101,0xfb840108,0xe9b90101,
	0xfb840108,0xe9ba0101,0xfb840108,0xe9bb0101,0xfb840108,0xe9bc0101,0xfb840108,0xe9bd0101,
	0xfb840108,0xe9be0101,0xfb840108,0xe9bf0101,0xfb840108,0xe9c00101,0xfb840108,0xe9c10101,
	0xfb840108,0xe9c20101,0xfb840108,0xe9c30101,0xfb840108,0xe9c40101,0xfb840108,0xe9c50101,
	0xfb840108,0xe9c60101,0xfb840108,0xe9c70101,0xfb840108,0xe9c80101,0xfb840108,0xe9c90101,
	0xfb840108,0xe9ca0101,0xfb840108,0xe9cb0101,0xfb840108,0xe9cc0101,0xfb840108,0xe9cd0101,
	0xfb840108,0xe9ce0101,0xfb840108,0xe9cf0101,0xfb840108,0xe9d00101,0xfb840108,0xe9d10101,
	0xfb840108,0xe9d20101,0xfb840108,0xe9d30101,0xfb840108,0xe9d40101,0xfb840108,0xe9d50101,
	0xfb840108,0xe9d60101,0xfb840108,0xe9d70101,0xfb840108,0xe9d80101,0xfb840108,0xe9d90101,
	0xfb840108,0xe9da0101,0xfb840108,0xe9db0101,0xfb840108,0xe9dc0101,0xfb840108,0xe9dd0101,
	0xfb840108,0xe9de0101,0xfb840108,0xe9df0101,0xfb840108,0xe9e00101,0xfb840108,0xe9e10101,
	0xfb840108,0xe9e20101,0xfb840108,0xe9e30101,0xfb840108,0xe9e40101,0xfb840108,0xe9e50101,
	0xfb840108,0xe9e60101,0xfb840108,0xe9e70101,0xfb840108,0xe9e80101,0xfb840108,0xe9e90101,
	0xfb840108,0xe9ea0101,0xfb840108,0xe9eb0101,0xfb840108,0xe9ec0101,0xfb840108,0xe9ed0101,
	0xfb840108,0xe9ee0101,0xfb840108,0xe9ef0101,0xfb840108,0xe9f00101,0xfb840108,0xe9f10101,
	0xfb840108,0xe9f20101,0xfb840108,0xe9f30101,0xfb840108,0xe9f40101,0xfb840108,0xe9f50101,
	0xfb840108,0xe9f60101,0xfb840108,0xe9f70101,0xfb840108,0xe9f80101,0xfb840108,0xe9f90101,
	0xfb840108,0xe9fa0101,0xfb840108,0xe9fb0101,0xfb840108,0xe9fc0101,0xfb840108,0xe9fd0101,
	0xfb840108,0xe9fe0101,0xfb840108,0xe9ff0101,0xfb840108,0xea000101,0xfb840108,0xea010101,
	0xfb840108,0xea020101,0xfb840108,0xea030101,0xfb840108,0xea040101,0xfb840108,0xea050101,
	0xfb840108,0xea060101,0xfb840108,0xea070101,0xfb840108,0xea080101,0xfb840108,0xea090101,
	0xfb840108,0xea0a0101,0xfb840108,0xea0b0101,0xfb840108,0xea0c0101,0xfb840108,0xea0d0101,
	0xfb840108,0xea0e0101,0xfb840108,0xea0f0101,0xfb840108,0xea100101,0xfb840108,0xea110101,
	0xfb840108,0xea120101,0xfb840108,0xea130101,0xfb840108,0xea140101,0xfb840108,0xea150101,
	0xfb840108,0xea160101,0xfb840108,0xea170101,0xfb840108,0xea180101,0xfb840108,0xea190101,
	0xfb840108,0xea1a0101,0xfb840108,0xea1b0101,0xfb840108,0xea1c0101,0xfb840108,0xea1d0101,
	0xfb840108,0xea1e0101,0xfb840108,0xea1f0101,0xfb840108,0xea200101,0xfb840108,0xea210101,
	0xfb840108,0xea220101,0xfb840108,0xea230101,0xfb840108,0xea240101,0xfb840108,0xea250101,
	0xfb840108,0xea260101,0xfb840108,0xea270101,0xfb840108,0xea280101,0xfb840108,0xea290101,
	0xfb840108,0xea2a0101,0xfb840108,0xea2b0101,0xfb840108,0xea2c0101,0xfb840108,0xea2d0101,
	0xfb840108,0xea2e0101,0xfb840108,0xea2f0101,0xfb840108,0xea300101,0xfb840108,0xea310101,
	0xfb840108,0xea320101,0xfb840108,0xea330101,0xfb840108,0xea340101,0xfb840108,0xea350101,
	0xfb840108,0xea360101,0xfb840108,0xea370101,0xfb840108,0xea380101,0xfb840108,0xea390101,
	0xfb840108,0xea3a0101,0xfb840108,0xea3b0101,0xfb840108,0xea3c0101,0xfb840108,0xea3d0101,
	0xfb840108,0xea3e0101,0xfb840108,0xea3f0101,0xfb840108,0xea400101,0xfb840108,0xea410101,
	0xfb840108,0xea420101,0xfb840108,0xea430101,0xfb840108,0xea440101,0xfb840108,0xea450101,
	0xfb840108,0xea460101,0xfb840108,0xea470101,0xfb840108,0xea480101,0xfb840108,0xea490101,
	0xfb840108,0xea4a0101,0xfb840108,0xea4b0101,0xfb840108,0xea4c0101,0xfb840108,0xea4d0101,
	0xfb840108,0xea4e0101,0xfb840108,0xea4f0101,0xfb840108,0xea500101,0xfb840108,0xea510101,
	0xfb840108,0xea520101,0xfb840108,0xea530101,0xfb840108,0xea540101,0xfb840108,0xea550101,
	0xfb840108,0xea560101,0xfb840108,0xea570101,0xfb840108,0xea580101,0xfb840108,0xea590101,
	0xfb840108,0xea5a0101,0xfb840108,0xea5b0101,0xfb840108,0xea5c0101,0xfb840108,0xea5d0101,
	0xfb840108,0xea5e0101,0xfb840108,0xea5f0101,0xfb840108,0xea600101,0xfb840108,0xea610101,
	0xfb840108,0xea620101,0xfb840108,0xea630101,0xfb840108,0xea640101,0xfb840108,0xea650101,
	0xfb840108,0xea660101,0xfb840108,0xea670101,0xfb840108,0xea680101,0xfb840108,0xea690101,
	0xfb840108,0xea6a0101,0xfb840108,0xea6b0101,0xfb840108,0xea6c0101,0xfb840108,0xea6d0101,
	0xfb840108,0xea6e0101,0xfb840108,0xea6f0101,0xfb840108,0xea700101,0xfb840108,0xea710101,
	0xfb840108,0xea720101,0xfb840108,0xea730101,0xfb840108,0xea740101,0xfb840108,0xea750101,
	0xfb840108,0xea760101,0xfb840108,0xea770101,0xfb840108,0xea780101,0xfb840108,0xea790101,
	0xfb840108,0xea7a0101,0xfb840108,0xea7b0101,0xfb840108,0xea7c0101,0xfb840108,0xea7d0101,
	0xfb840108,0xea7e0101,0xfb840108,0xea7f0101,0xfb840108,0xea800101,0xfb840108,0xea810101,
	0xfb840108,0xea820101,0xfb840108,0xea830101,0xfb840108,0xea840101,0xfb840108,0xea850101,
	0xfb840108,0xea860101,0xfb840108,0xea870101,0xfb840108,0xea880101,0xfb840108,0xea890101,
	0xfb840108,0xea8a0101,0xfb840108,0xea8b0101,0xfb840108,0xea8c0101,0xfb840108,0xea8d0101,
	0xfb840108,0xea8e0101,0xfb840108,0xea8f0101,0xfb840108,0xea900101,0xfb840108,0xea910101,
	0xfb840108,0xea920101,0xfb840108,0xea930101,0xfb840108,0xea940101,0xfb840108,0xea950101,
	0xfb840108,0xea960101,0xfb840108,0xea970101,0xfb840108,0xea980101,0xfb840108,0xea990101,
	0xfb840108,0xea9a0101,0xfb840108,0xea9b0101,0xfb840108,0xea9c0101,0xfb840108,0xea9d0101,
	0xfb840108,0xea9e0101,0xfb840108,0xea9f0101,0xfb840108,0xeaa00101,0xfb840108,0xeaa10101,
	0xfb840108,0xeaa20101,0xfb840108,0xeaa30101,0xfb840108,0xeaa40101,0xfb840108,0xeaa50101,
	0xfb840108,0xeaa60101,0xfb840108,0xeaa70101,0xfb840108,0xeaa80101,0xfb840108,0xeaa90101,
	0xfb840108,0xeaaa0101,0xfb840108,0xeaab0101,0xfb840108,0xeaac0101,0xfb840108,0xeaad0101,
	0xfb840108,0xeaae0101,0xfb840108,0xeaaf0101,0xfb840108,0xeab00101,0xfb840108,0xeab10101,
	0xfb840108,0xeab20101,0xfb840108,0xeab30101,0xfb840108,0xeab40101,0xfb840108,0xeab50101,
	0xfb840108,0xeab60101,0xfb840108,0xeab70101,0xfb840108,0xeab80101,0xfb840108,0xeab90101,
	0xfb840108,0xeaba0101,0xfb840108,0xeabb0101,0xfb840108,0xeabc0101,0xfb840108,0xeabd0101,
	0xfb840108,0xeabe0101,0xfb840108,0xeabf0101,0xfb840108,0xeac00101,0xfb840108,0xeac10101,
	0xfb840108,0xeac20101,0xfb840108,0xeac30101,0xfb840108,0xeac40101,0xfb840108,0xeac50101,
	0xfb840108,0xeac60101,0xfb840108,0xeac70101,0xfb840108,0xeac80101,0xfb840108,0xeac90101,
	0xfb840108,0xeaca0101,0xfb840108,0xeacb0101,0xfb840108,0xeacc0101,0xfb840108,0xeacd0101,
	0xfb840108,0xeace0101,0xfb840108,0xeacf0101,0xfb840108,0xead00101,0xfb840108,0xead10101,
	0xfb840108,0xead20101,0xfb840108,0xead30101,0xfb840108,0xead40101,0xfb840108,0xead50101,
	0xfb840108,0xead60101,0xfb840108,0xead70101,0xfb840108,0xead80101,0xfb840108,0xead90101,
	0xfb840108,0xeada0101,0xfb840108,0xeadb0101,0xfb840108,0xeadc0101,0xfb840108,0xeadd0101,
	0xfb840108,0xeade0101,0xfb840108,0xeadf0101,0xfb840108,0xeae00101,0xfb840108,0xeae10101,
	0xfb840108,0xeae20101,0xfb840108,0xeae30101,0xfb840108,0xeae40101,0xfb840108,0xeae50101,
	0xfb840108,0xeae60101,0xfb840108,0xeae70101,0xfb840108,0xeae80101,0xfb840108,0xeae90101,
	0xfb840108,0xeaea0101,0xfb840108,0xeaeb0101,0xfb840108,0xeaec0101,0xfb840108,0xeaed0101,
	0xfb840108,0xeaee0101,0xfb840108,0xeaef0101,0xfb840108,0xeaf00101,0xfb840108,0xeaf10101,
	0xfb840108,0xeaf20101,0xfb840108,0xeaf30101,0xfb840108,0xeaf40101,0xfb840108,0xeaf50101,
	0xfb840108,0xeaf60101,0xfb840108,0xeaf70101,0xfb840108,0xeaf80101,0xfb840108,0xeaf90101,
	0xfb840108,0xeafa0101,0xfb840108,0xeafb0101,0xfb840108,0xeafc0101,0xfb840108,0xeafd0101,
	0xfb840108,0xeafe0101,0xfb840108,0xeaff0101,0xfb840108,0xeb000101,0xfb840108,0xeb010101,
	0xfb840108,0xeb020101,0xfb840108,0xeb030101,0xfb840108,0xeb040101,0xfb840108,0xeb050101,
	0xfb840108,0xeb060101,0xfb840108,0xeb070101,0xfb840108,0xeb080101,0xfb840108,0xeb090101,
	0xfb840108,0xeb0a0101,0xfb840108,0xeb0b0101,0xfb840108,0xeb0c0101,0xfb840108,0xeb0d0101,
	0xfb840108,0xeb0e0101,0xfb840108,0xeb0f0101,0xfb840108,0xeb100101,0xfb840108,0xeb110101,
	0xfb840108,0xeb120101,0xfb840108,0xeb130101,0xfb840108,0xeb140101,0xfb840108,0xeb150101,
	0xfb840108,0xeb160101,0xfb840108,0xeb170101,0xfb840108,0xeb180101,0xfb840108,0xeb190101,
	0xfb840108,0xeb1a0101,0xfb840108,0xeb1b0101,0xfb840108,0xeb1c0101,0xfb840108,0xeb1d0101,
	0xfb840108,0xeb1e0101,0xfb840108,0xeb1f0101,0xfb840108,0xeb200101,0xfb840108,0xeb210101,
	0xfb840108,0xeb220101,0xfb840108,0xeb230101,0xfb840108,0xeb240101,0xfb840108,0xeb250101,
	0xfb840108,0xeb260101,0xfb840108,0xeb270101,0xfb840108,0xeb280101,0xfb840108,0xeb290101,
	0xfb840108,0xeb2a0101,0xfb840108,0xeb2b0101,0xfb840108,0xeb2c0101,0xfb840108,0xeb2d0101,
	0xfb840108,0xeb2e0101,0xfb840108,0xeb2f0101,0xfb840108,0xeb300101,0xfb840108,0xeb310101,
	0xfb840108,0xeb320101,0xfb840108,0xeb330101,0xfb840108,0xeb340101,0xfb840108,0xeb350101,
	0xfb840108,0xeb360101,0xfb840108,0xeb370101,0xfb840108,0xeb380101,0xfb840108,0xeb390101,
	0xfb840108,0xeb3a0101,0xfb840108,0xeb3b0101,0xfb840108,0xeb3c0101,0xfb840108,0xeb3d0101,
	0xfb840108,0xeb3e0101,0xfb840108,0xeb3f0101,0xfb840108,0xeb400101,0xfb840108,0xeb410101,
	0xfb840108,0xeb420101,0xfb840108,0xeb430101,0xfb840108,0xeb440101,0xfb840108,0xeb450101,
	0xfb840108,0xeb460101,0xfb840108,0xeb470101,0xfb840108,0xeb480101,0xfb840108,0xeb490101,
	0xfb840108,0xeb4a0101,0xfb840108,0xeb4b0101,0xfb840108,0xeb4c0101,0xfb840108,0xeb4d0101,
	0xfb840108,0xeb4e0101,0xfb840108,0xeb4f0101,0xfb840108,0xeb500101,0xfb840108,0xeb510101,
	0xfb840108,0xeb520101,0xfb840108,0xeb530101,0xfb840108,0xeb540101,0xfb840108,0xeb550101,
	0xfb840108,0xeb560101,0xfb840108,0xeb570101,0xfb840108,0xeb580101,0xfb840108,0xeb590101,
	0xfb840108,0xeb5a0101,0xfb840108,0xeb5b0101,0xfb840108,0xeb5c0101,0xfb840108,0xeb5d0101,
	0xfb840108,0xeb5e0101,0xfb840108,0xeb5f0101,0xfb840108,0xeb600101,0xfb840108,0xeb610101,
	0xfb840108,0xeb620101,0xfb840108,0xeb630101,0xfb840108,0xeb640101,0xfb840108,0xeb650101,
	0xfb840108,0xeb660101,0xfb840108,0xeb670101,0xfb840108,0xeb680101,0xfb840108,0xeb690101,
	0xfb840108,0xeb6a0101,0xfb840108,0xeb6b0101,0xfb840108,0xeb6c0101,0xfb840108,0xeb6d0101,
	0xfb840108,0xeb6e0101,0xfb840108,0xeb6f0101,0xfb840108,0xeb700101,0xfb840108,0xeb710101,
	0xfb840108,0xeb720101,0xfb840108,0xeb730101,0xfb840108,0xeb740101,0xfb840108,0xeb750101,
	0xfb840108,0xeb760101,0xfb840108,0xeb770101,0xfb840108,0xeb780101,0xfb840108,0xeb790101,
	0xfb840108,0xeb7a0101,0xfb840108,0xeb7b0101,0xfb840108,0xeb7c0101,0xfb840108,0xeb7d0101,
	0xfb840108,0xeb7e0101,0xfb840108,0xeb7f0101,0xfb840108,0xeb800101,0xfb840108,0xeb810101,
	0xfb840108,0xeb820101,0xfb840108,0xeb830101,0xfb840108,0xeb840101,0xfb840108,0xeb850101,
	0xfb840108,0xeb860101,0xfb840108,0xeb870101,0xfb840108,0xeb880101,0xfb840108,0xeb890101,
	0xfb840108,0xeb8a0101,0xfb840108,0xeb8b0101,0xfb840108,0xeb8c0101,0xfb840108,0xeb8d0101,
	0xfb840108,0xeb8e0101,0xfb840108,0xeb8f0101,0xfb840108,0xeb900101,0xfb840108,0xeb910101,
	0xfb840108,0xeb920101,0xfb840108,0xeb930101,0xfb840108,0xeb940101,0xfb840108,0xeb950101,
	0xfb840108,0xeb960101,0xfb840108,0xeb970101,0xfb840108,0xeb980101,0xfb840108,0xeb990101,
	0xfb840108,0xeb9a0101,0xfb840108,0xeb9b0101,0xfb840108,0xeb9c0101,0xfb840108,0xeb9d0101,
	0xfb840108,0xeb9e0101,0xfb840108,0xeb9f0101,0xfb840108,0xeba00101,0xfb840108,0xeba10101,
	0xfb840108,0xeba20101,0xfb840108,0xeba30101,0xfb840108,0xeba40101,0xfb840108,0xeba50101,
	0xfb840108,0xeba60101,0xfb840108,0xeba70101,0xfb840108,0xeba80101,0xfb840108,0xeba90101,
	0xfb840108,0xebaa0101,0xfb840108,0xebab0101,0xfb840108,0xebac0101,0xfb840108,0xebad0101,
	0xfb840108,0xebae0101,0xfb840108,0xebaf0101,0xfb840108,0xebb00101,0xfb840108,0xebb10101,
	0xfb840108,0xebb20101,0xfb840108,0xebb30101,0xfb840108,0xebb40101,0xfb840108,0xebb50101,
	0xfb840108,0xebb60101,0xfb840108,0xebb70101,0xfb840108,0xebb80101,0xfb840108,0xebb90101,
	0xfb840108,0xebba0101,0xfb840108,0xebbb0101,0xfb840108,0xebbc0101,0xfb840108,0xebbd0101,
	0xfb840108,0xebbe0101,0xfb840108,0xebbf0101,0xfb840108,0xebc00101,0xfb840108,0xebc10101,
	0xfb840108,0xebc20101,0xfb840108,0xebc30101,0xfb840108,0xebc40101,0xfb840108,0xebc50101,
	0xfb840108,0xebc60101,0xfb840108,0xebc70101,0xfb840108,0xebc80101,0xfb840108,0xebc90101,
	0xfb840108,0xebca0101,0xfb840108,0xebcb0101,0xfb840108,0xebcc0101,0xfb840108,0xebcd0101,
	0xfb840108,0xebce0101,0xfb840108,0xebcf0101,0xfb840108,0xebd00101,0xfb840108,0xebd10101,
	0xfb840108,0xebd20101,0xfb840108,0xebd30101,0xfb840108,0xebd40101,0xfb840108,0xebd50101,
	0xfb840108,0xebd60101,0xfb840108,0xebd70101,0xfb840108,0xebd80101,0xfb840108,0xebd90101,
	0xfb840108,0xebda0101,0xfb840108,0xebdb0101,0xfb840108,0xebdc0101,0xfb840108,0xebdd0101,
	0xfb840108,0xebde0101,0xfb840108,0xebdf0101,0xfb840108,0xebe00101,0xfb840108,0xebe10101,
	0xfb840108,0xebe20101,0xfb840108,0xebe30101,0xfb840108,0xebe40101,0xfb840108,0xebe50101,
	0xfb840108,0xebe60101,0xfb840108,0xebe70101,0xfb840108,0xebe80101,0xfb840108,0xebe90101,
	0xfb840108,0xebea0101,0xfb840108,0xebeb0101,0xfb840108,0xebec0101,0xfb840108,0xebed0101,
	0xfb840108,0xebee0101,0xfb840108,0xebef0101,0xfb840108,0xebf00101,0xfb840108,0xebf10101,
	0xfb840108,0xebf20101,0xfb840108,0xebf30101,0xfb840108,0xebf40101,0xfb840108,0xebf50101,
	0xfb840108,0xebf60101,0xfb840108,0xebf70101,0xfb840108,0xebf80101,0xfb840108,0xebf90101,
	0xfb840108,0xebfa0101,0xfb840108,0xebfb0101,0xfb840108,0xebfc0101,0xfb840108,0xebfd0101,
	0xfb840108,0xebfe0101,0xfb840108,0xebff0101,0xfb840108,0xec000101,0xfb840108,0xec010101,
	0xfb840108,0xec020101,0xfb840108,0xec030101,0xfb840108,0xec040101,0xfb840108,0xec050101,
	0xfb840108,0xec060101,0xfb840108,0xec070101,0xfb840108,0xec080101,0xfb840108,0xec090101,
	0xfb840108,0xec0a0101,0xfb840108,0xec0b0101,0xfb840108,0xec0c0101,0xfb840108,0xec0d0101,
	0xfb840108,0xec0e0101,0xfb840108,0xec0f0101,0xfb840108,0xec100101,0xfb840108,0xec110101,
	0xfb840108,0xec120101,0xfb840108,0xec130101,0xfb840108,0xec140101,0xfb840108,0xec150101,
	0xfb840108,0xec160101,0xfb840108,0xec170101,0xfb840108,0xec180101,0xfb840108,0xec190101,
	0xfb840108,0xec1a0101,0xfb840108,0xec1b0101,0xfb840108,0xec1c0101,0xfb840108,0xec1d0101,
	0xfb840108,0xec1e0101,0xfb840108,0xec1f0101,0xfb840108,0xec200101,0xfb840108,0xec210101,
	0xfb840108,0xec220101,0xfb840108,0xec230101,0xfb840108,0xec240101,0xfb840108,0xec250101,
	0xfb840108,0xec260101,0xfb840108,0xec270101,0xfb840108,0xec280101,0xfb840108,0xec290101,
	0xfb840108,0xec2a0101,0xfb840108,0xec2b0101,0xfb840108,0xec2c0101,0xfb840108,0xec2d0101,
	0xfb840108,0xec2e0101,0xfb840108,0xec2f0101,0xfb840108,0xec300101,0xfb840108,0xec310101,
	0xfb840108,0xec320101,0xfb840108,0xec330101,0xfb840108,0xec340101,0xfb840108,0xec350101,
	0xfb840108,0xec360101,0xfb840108,0xec370101,0xfb840108,0xec380101,0xfb840108,0xec390101,
	0xfb840108,0xec3a0101,0xfb840108,0xec3b0101,0xfb840108,0xec3c0101,0xfb840108,0xec3d0101,
	0xfb840108,0xec3e0101,0xfb840108,0xec3f0101,0xfb840108,0xec400101,0xfb840108,0xec410101,
	0xfb840108,0xec420101,0xfb840108,0xec430101,0xfb840108,0xec440101,0xfb840108,0xec450101,
	0xfb840108,0xec460101,0xfb840108,0xec470101,0xfb840108,0xec480101,0xfb840108,0xec490101,
	0xfb840108,0xec4a0101,0xfb840108,0xec4b0101,0xfb840108,0xec4c0101,0xfb840108,0xec4d0101,
	0xfb840108,0xec4e0101,0xfb840108,0xec4f0101,0xfb840108,0xec500101,0xfb840108,0xec510101,
	0xfb840108,0xec520101,0xfb840108,0xec530101,0xfb840108,0xec540101,0xfb840108,0xec550101,
	0xfb840108,0xec560101,0xfb840108,0xec570101,0xfb840108,0xec580101,0xfb840108,0xec590101,
	0xfb840108,0xec5a0101,0xfb840108,0xec5b0101,0xfb840108,0xec5c0101,0xfb840108,0xec5d0101,
	0xfb840108,0xec5e0101,0xfb840108,0xec5f0101,0xfb840108,0xec600101,0xfb840108,0xec610101,
	0xfb840108,0xec620101,0xfb840108,0xec630101,0xfb840108,0xec640101,0xfb840108,0xec650101,
	0xfb840108,0xec660101,0xfb840108,0xec670101,0xfb840108,0xec680101,0xfb840108,0xec690101,
	0xfb840108,0xec6a0101,0xfb840108,0xec6b0101,0xfb840108,0xec6c0101,0xfb840108,0xec6d0101,
	0xfb840108,0xec6e0101,0xfb840108,0xec6f0101,0xfb840108,0xec700101,0xfb840108,0xec710101,
	0xfb840108,0xec720101,0xfb840108,0xec730101,0xfb840108,0xec740101,0xfb840108,0xec750101,
	0xfb840108,0xec760101,0xfb840108,0xec770101,0xfb840108,0xec780101,0xfb840108,0xec790101,
	0xfb840108,0xec7a0101,0xfb840108,0xec7b0101,0xfb840108,0xec7c0101,0xfb840108,0xec7d0101,
	0xfb840108,0xec7e0101,0xfb840108,0xec7f0101,0xfb840108,0xec800101,0xfb840108,0xec810101,
	0xfb840108,0xec820101,0xfb840108,0xec830101,0xfb840108,0xec840101,0xfb840108,0xec850101,
	0xfb840108,0xec860101,0xfb840108,0xec870101,0xfb840108,0xec880101,0xfb840108,0xec890101,
	0xfb840108,0xec8a0101,0xfb840108,0xec8b0101,0xfb840108,0xec8c0101,0xfb840108,0xec8d0101,
	0xfb840108,0xec8e0101,0xfb840108,0xec8f0101,0xfb840108,0xec900101,0xfb840108,0xec910101,
	0xfb840108,0xec920101,0xfb840108,0xec930101,0xfb840108,0xec940101,0xfb840108,0xec950101,
	0xfb840108,0xec960101,0xfb840108,0xec970101,0xfb840108,0xec980101,0xfb840108,0xec990101,
	0xfb840108,0xec9a0101,0xfb840108,0xec9b0101,0xfb840108,0xec9c0101,0xfb840108,0xec9d0101,
	0xfb840108,0xec9e0101,0xfb840108,0xec9f0101,0xfb840108,0xeca00101,0xfb840108,0xeca10101,
	0xfb840108,0xeca20101,0xfb840108,0xeca30101,0xfb840108,0xeca40101,0xfb840108,0xeca50101,
	0xfb840108,0xeca60101,0xfb840108,0xeca70101,0xfb840108,0xeca80101,0xfb840108,0xeca90101,
	0xfb840108,0xecaa0101,0xfb840108,0xecab0101,0xfb840108,0xecac0101,0xfb840108,0xecad0101,
	0xfb840108,0xecae0101,0xfb840108,0xecaf0101,0xfb840108,0xecb00101,0xfb840108,0xecb10101,
	0xfb840108,0xecb20101,0xfb840108,0xecb30101,0xfb840108,0xecb40101,0xfb840108,0xecb50101,
	0xfb840108,0xecb60101,0xfb840108,0xecb70101,0xfb840108,0xecb80101,0xfb840108,0xecb90101,
	0xfb840108,0xecba0101,0xfb840108,0xecbb0101,0xfb840108,0xecbc0101,0xfb840108,0xecbd0101,
	0xfb840108,0xecbe0101,0xfb840108,0xecbf0101,0xfb840108,0xecc00101,0xfb840108,0xecc10101,
	0xfb840108,0xecc20101,0xfb840108,0xecc30101,0xfb840108,0xecc40101,0xfb840108,0xecc50101,
	0xfb840108,0xecc60101,0xfb840108,0xecc70101,0xfb840108,0xecc80101,0xfb840108,0xecc90101,
	0xfb840108,0xecca0101,0xfb840108,0xeccb0101,0xfb840108,0xeccc0101,0xfb840108,0xeccd0101,
	0xfb840108,0xecce0101,0xfb840108,0xeccf0101,0xfb840108,0xecd00101,0xfb840108,0xecd10101,
	0xfb840108,0xecd20101,0xfb840108,0xecd30101,0xfb840108,0xecd40101,0xfb840108,0xecd50101,
	0xfb840108,0xecd60101,0xfb840108,0xecd70101,0xfb840108,0xecd80101,0xfb840108,0xecd90101,
	0xfb840108,0xecda0101,0xfb840108,0xecdb0101,0xfb840108,0xecdc0101,0xfb840108,0xecdd0101,
	0xfb840108,0xecde0101,0xfb840108,0xecdf0101,0xfb840108,0xece00101,0xfb840108,0xece10101,
	0xfb840108,0xece20101,0xfb840108,0xece30101,0xfb840108,0xece40101,0xfb840108,0xece50101,
	0xfb840108,0xece60101,0xfb840108,0xece70101,0xfb840108,0xece80101,0xfb840108,0xece90101,
	0xfb840108,0xecea0101,0xfb840108,0xeceb0101,0xfb840108,0xecec0101,0xfb840108,0xeced0101,
	0xfb840108,0xecee0101,0xfb840108,0xecef0101,0xfb840108,0xecf00101,0xfb840108,0xecf10101,
	0xfb840108,0xecf20101,0xfb840108,0xecf30101,0xfb840108,0xecf40101,0xfb840108,0xecf50101,
	0xfb840108,0xecf60101,0xfb840108,0xecf70101,0xfb840108,0xecf80101,0xfb840108,0xecf90101,
	0xfb840108,0xecfa0101,0xfb840108,0xecfb0101,0xfb840108,0xecfc0101,0xfb840108,0xecfd0101,
	0xfb840108,0xecfe0101,0xfb840108,0xecff0101,0xfb840108,0xed000101,0xfb840108,0xed010101,
	0xfb840108,0xed020101,0xfb840108,0xed030101,0xfb840108,0xed040101,0xfb840108,0xed050101,
	0xfb840108,0xed060101,0xfb840108,0xed070101,0xfb840108,0xed080101,0xfb840108,0xed090101,
	0xfb840108,0xed0a0101,0xfb840108,0xed0b0101,0xfb840108,0xed0c0101,0xfb840108,0xed0d0101,
	0xfb840108,0xed0e0101,0xfb840108,0xed0f0101,0xfb840108,0xed100101,0xfb840108,0xed110101,
	0xfb840108,0xed120101,0xfb840108,0xed130101,0xfb840108,0xed140101,0xfb840108,0xed150101,
	0xfb840108,0xed160101,0xfb840108,0xed170101,0xfb840108,0xed180101,0xfb840108,0xed190101,
	0xfb840108,0xed1a0101,0xfb840108,0xed1b0101,0xfb840108,0xed1c0101,0xfb840108,0xed1d0101,
	0xfb840108,0xed1e0101,0xfb840108,0xed1f0101,0xfb840108,0xed200101,0xfb840108,0xed210101,
	0xfb840108,0xed220101,0xfb840108,0xed230101,0xfb840108,0xed240101,0xfb840108,0xed250101,
	0xfb840108,0xed260101,0xfb840108,0xed270101,0xfb840108,0xed280101,0xfb840108,0xed290101,
	0xfb840108,0xed2a0101,0xfb840108,0xed2b0101,0xfb840108,0xed2c0101,0xfb840108,0xed2d0101,
	0xfb840108,0xed2e0101,0xfb840108,0xed2f0101,0xfb840108,0xed300101,0xfb840108,0xed310101,
	0xfb840108,0xed320101,0xfb840108,0xed330101,0xfb840108,0xed340101,0xfb840108,0xed350101,
	0xfb840108,0xed360101,0xfb840108,0xed370101,0xfb840108,0xed380101,0xfb840108,0xed390101,
	0xfb840108,0xed3a0101,0xfb840108,0xed3b0101,0xfb840108,0xed3c0101,0xfb840108,0xed3d0101,
	0xfb840108,0xed3e0101,0xfb840108,0xed3f0101,0xfb840108,0xed400101,0xfb840108,0xed410101,
	0xfb840108,0xed420101,0xfb840108,0xed430101,0xfb840108,0xed440101,0xfb840108,0xed450101,
	0xfb840108,0xed460101,0xfb840108,0xed470101,0xfb840108,0xed480101,0xfb840108,0xed490101,
	0xfb840108,0xed4a0101,0xfb840108,0xed4b0101,0xfb840108,0xed4c0101,0xfb840108,0xed4d0101,
	0xfb840108,0xed4e0101,0xfb840108,0xed4f0101,0xfb840108,0xed500101,0xfb840108,0xed510101,
	0xfb840108,0xed520101,0xfb840108,0xed530101,0xfb840108,0xed540101,0xfb840108,0xed550101,
	0xfb840108,0xed560101,0xfb840108,0xed570101,0xfb840108,0xed580101,0xfb840108,0xed590101,
	0xfb840108,0xed5a0101,0xfb840108,0xed5b0101,0xfb840108,0xed5c0101,0xfb840108,0xed5d0101,
	0xfb840108,0xed5e0101,0xfb840108,0xed5f0101,0xfb840108,0xed600101,0xfb840108,0xed610101,
	0xfb840108,0xed620101,0xfb840108,0xed630101,0xfb840108,0xed640101,0xfb840108,0xed650101,
	0xfb840108,0xed660101,0xfb840108,0xed670101,0xfb840108,0xed680101,0xfb840108,0xed690101,
	0xfb840108,0xed6a0101,0xfb840108,0xed6b0101,0xfb840108,0xed6c0101,0xfb840108,0xed6d0101,
	0xfb840108,0xed6e0101,0xfb840108,0xed6f0101,0xfb840108,0xed700101,0xfb840108,0xed710101,
	0xfb840108,0xed720101,0xfb840108,0xed730101,0xfb840108,0xed740101,0xfb840108,0xed750101,
	0xfb840108,0xed760101,0xfb840108,0xed770101,0xfb840108,0xed780101,0xfb840108,0xed790101,
	0xfb840108,0xed7a0101,0xfb840108,0xed7b0101,0xfb840108,0xed7c0101,0xfb840108,0xed7d0101,
	0xfb840108,0xed7e0101,0xfb840108,0xed7f0101,0xfb840108,0xed800101,0xfb840108,0xed810101,
	0xfb840108,0xed820101,0xfb840108,0xed830101,0xfb840108,0xed840101,0xfb840108,0xed850101,
	0xfb840108,0xed860101,0xfb840108,0xed870101,0xfb840108,0xed880101,0xfb840108,0xed890101,
	0xfb840108,0xed8a0101,0xfb840108,0xed8b0101,0xfb840108,0xed8c0101,0xfb840108,0xed8d0101,
	0xfb840108,0xed8e0101,0xfb840108,0xed8f0101,0xfb840108,0xed900101,0xfb840108,0xed910101,
	0xfb840108,0xed920101,0xfb840108,0xed930101,0xfb840108,0xed940101,0xfb840108,0xed950101,
	0xfb840108,0xed960101,0xfb840108,0xed970101,0xfb840108,0xed980101,0xfb840108,0xed990101,
	0xfb840108,0xed9a0101,0xfb840108,0xed9b0101,0xfb840108,0xed9c0101,0xfb840108,0xed9d0101,
	0xfb840108,0xed9e0101,0xfb840108,0xed9f0101,0xfb840108,0xeda00101,0xfb840108,0xeda10101,
	0xfb840108,0xeda20101,0xfb840108,0xeda30101,0xfb840108,0xeda40101,0xfb840108,0xeda50101,
	0xfb840108,0xeda60101,0xfb840108,0xeda70101,0xfb840108,0xeda80101,0xfb840108,0xeda90101,
	0xfb840108,0xedaa0101,0xfb840108,0xedab0101,0xfb840108,0xedac0101,0xfb840108,0xedad0101,
	0xfb840108,0xedae0101,0xfb840108,0xedaf0101,0xfb840108,0xedb00101,0xfb840108,0xedb10101,
	0xfb840108,0xedb20101,0xfb840108,0xedb30101,0xfb840108,0xedb40101,0xfb840108,0xedb50101,
	0xfb840108,0xedb60101,0xfb840108,0xedb70101,0xfb840108,0xedb80101,0xfb840108,0xedb90101,
	0xfb840108,0xedba0101,0xfb840108,0xedbb0101,0xfb840108,0xedbc0101,0xfb840108,0xedbd0101,
	0xfb840108,0xedbe0101,0xfb840108,0xedbf0101,0xfb840108,0xedc00101,0xfb840108,0xedc10101,
	0xfb840108,0xedc20101,0xfb840108,0xedc30101,0xfb840108,0xedc40101,0xfb840108,0xedc50101,
	0xfb840108,0xedc60101,0xfb840108,0xedc70101,0xfb840108,0xedc80101,0xfb840108,0xedc90101,
	0xfb840108,0xedca0101,0xfb840108,0xedcb0101,0xfb840108,0xedcc0101,0xfb840108,0xedcd0101,
	0xfb840108,0xedce0101,0xfb840108,0xedcf0101,0xfb840108,0xedd00101,0xfb840108,0xedd10101,
	0xfb840108,0xedd20101,0xfb840108,0xedd30101,0xfb840108,0xedd40101,0xfb840108,0xedd50101,
	0xfb840108,0xedd60101,0xfb840108,0xedd70101,0xfb840108,0xedd80101,0xfb840108,0xedd90101,
	0xfb840108,0xedda0101,0xfb840108,0xeddb0101,0xfb840108,0xeddc0101,0xfb840108,0xeddd0101,
	0xfb840108,0xedde0101,0xfb840108,0xeddf0101,0xfb840108,0xede00101,0xfb840108,0xede10101,
	0xfb840108,0xede20101,0xfb840108,0xede30101,0xfb840108,0xede40101,0xfb840108,0xede50101,
	0xfb840108,0xede60101,0xfb840108,0xede70101,0xfb840108,0xede80101,0xfb840108,0xede90101,
	0xfb840108,0xedea0101,0xfb840108,0xedeb0101,0xfb840108,0xedec0101,0xfb840108,0xeded0101,
	0xfb840108,0xedee0101,0xfb840108,0xedef0101,0xfb840108,0xedf00101,0xfb840108,0xedf10101,
	0xfb840108,0xedf20101,0xfb840108,0xedf30101,0xfb840108,0xedf40101,0xfb840108,0xedf50101,
	0xfb840108,0xedf60101,0xfb840108,0xedf70101,0xfb840108,0xedf80101,0xfb840108,0xedf90101,
	0xfb840108,0xedfa0101,0xfb840108,0xedfb0101,0xfb840108,0xedfc0101,0xfb840108,0xedfd0101,
	0xfb840108,0xedfe0101,0xfb840108,0xedff0101,0xfb840108,0xee000101,0xfb840108,0xee010101,
	0xfb840108,0xee020101,0xfb840108,0xee030101,0xfb840108,0xee040101,0xfb840108,0xee050101,
	0xfb840108,0xee060101,0xfb840108,0xee070101,0xfb840108,0xee080101,0xfb840108,0xee090101,
	0xfb840108,0xee0a0101,0xfb840108,0xee0b0101,0xfb840108,0xee0c0101,0xfb840108,0xee0d0101,
	0xfb840108,0xee0e0101,0xfb840108,0xee0f0101,0xfb840108,0xee100101,0xfb840108,0xee110101,
	0xfb840108,0xee120101,0xfb840108,0xee130101,0xfb840108,0xee140101,0xfb840108,0xee150101,
	0xfb840108,0xee160101,0xfb840108,0xee170101,0xfb840108,0xee180101,0xfb840108,0xee190101,
	0xfb840108,0xee1a0101,0xfb840108,0xee1b0101,0xfb840108,0xee1c0101,0xfb840108,0xee1d0101,
	0xfb840108,0xee1e0101,0xfb840108,0xee1f0101,0xfb840108,0xee200101,0xfb840108,0xee210101,
	0xfb840108,0xee220101,0xfb840108,0xee230101,0xfb840108,0xee240101,0xfb840108,0xee250101,
	0xfb840108,0xee260101,0xfb840108,0xee270101,0xfb840108,0xee280101,0xfb840108,0xee290101,
	0xfb840108,0xee2a0101,0xfb840108,0xee2b0101,0xfb840108,0xee2c0101,0xfb840108,0xee2d0101,
	0xfb840108,0xee2e0101,0xfb840108,0xee2f0101,0xfb840108,0xee300101,0xfb840108,0xee310101,
	0xfb840108,0xee320101,0xfb840108,0xee330101,0xfb840108,0xee340101,0xfb840108,0xee350101,
	0xfb840108,0xee360101,0xfb840108,0xee370101,0xfb840108,0xee380101,0xfb840108,0xee390101,
	0xfb840108,0xee3a0101,0xfb840108,0xee3b0101,0xfb840108,0xee3c0101,0xfb840108,0xee3d0101,
	0xfb840108,0xee3e0101,0xfb840108,0xee3f0101,0xfb840108,0xee400101,0xfb840108,0xee410101,
	0xfb840108,0xee420101,0xfb840108,0xee430101,0xfb840108,0xee440101,0xfb840108,0xee450101,
	0xfb840108,0xee460101,0xfb840108,0xee470101,0xfb840108,0xee480101,0xfb840108,0xee490101,
	0xfb840108,0xee4a0101,0xfb840108,0xee4b0101,0xfb840108,0xee4c0101,0xfb840108,0xee4d0101,
	0xfb840108,0xee4e0101,0xfb840108,0xee4f0101,0xfb840108,0xee500101,0xfb840108,0xee510101,
	0xfb840108,0xee520101,0xfb840108,0xee530101,0xfb840108,0xee540101,0xfb840108,0xee550101,
	0xfb840108,0xee560101,0xfb840108,0xee570101,0xfb840108,0xee580101,0xfb840108,0xee590101,
	0xfb840108,0xee5a0101,0xfb840108,0xee5b0101,0xfb840108,0xee5c0101,0xfb840108,0xee5d0101,
	0xfb840108,0xee5e0101,0xfb840108,0xee5f0101,0xfb840108,0xee600101,0xfb840108,0xee610101,
	0xfb840108,0xee620101,0xfb840108,0xee630101,0xfb840108,0xee640101,0xfb840108,0xee650101,
	0xfb840108,0xee660101,0xfb840108,0xee670101,0xfb840108,0xee680101,0xfb840108,0xee690101,
	0xfb840108,0xee6a0101,0xfb840108,0xee6b0101,0xfb840108,0xee6c0101,0xfb840108,0xee6d0101,
	0xfb840108,0xee6e0101,0xfb840108,0xee6f0101,0xfb840108,0xee700101,0xfb840108,0xee710101,
	0xfb840108,0xee720101,0xfb840108,0xee730101,0xfb840108,0xee740101,0xfb840108,0xee750101,
	0xfb840108,0xee760101,0xfb840108,0xee770101,0xfb840108,0xee780101,0xfb840108,0xee790101,
	0xfb840108,0xee7a0101,0xfb840108,0xee7b0101,0xfb840108,0xee7c0101,0xfb840108,0xee7d0101,
	0xfb840108,0xee7e0101,0xfb840108,0xee7f0101,0xfb840108,0xee800101,0xfb840108,0xee810101,
	0xfb840108,0xee820101,0xfb840108,0xee830101,0xfb840108,0xee840101,0xfb840108,0xee850101,
	0xfb840108,0xee860101,0xfb840108,0xee870101,0xfb840108,0xee880101,0xfb840108,0xee890101,
	0xfb840108,0xee8a0101,0xfb840108,0xee8b0101,0xfb840108,0xee8c0101,0xfb840108,0xee8d0101,
	0xfb840108,0xee8e0101,0xfb840108,0xee8f0101,0xfb840108,0xee900101,0xfb840108,0xee910101,
	0xfb840108,0xee920101,0xfb840108,0xee930101,0xfb840108,0xee940101,0xfb840108,0xee950101,
	0xfb840108,0xee960101,0xfb840108,0xee970101,0xfb840108,0xee980101,0xfb840108,0xee990101,
	0xfb840108,0xee9a0101,0xfb840108,0xee9b0101,0xfb840108,0xee9c0101,0xfb840108,0xee9d0101,
	0xfb840108,0xee9e0101,0xfb840108,0xee9f0101,0xfb840108,0xeea00101,0xfb840108,0xeea10101,
	0xfb840108,0xeea20101,0xfb840108,0xeea30101,0xfb840108,0xeea40101,0xfb840108,0xeea50101,
	0xfb840108,0xeea60101,0xfb840108,0xeea70101,0xfb840108,0xeea80101,0xfb840108,0xeea90101,
	0xfb840108,0xeeaa0101,0xfb840108,0xeeab0101,0xfb840108,0xeeac0101,0xfb840108,0xeead0101,
	0xfb840108,0xeeae0101,0xfb840108,0xeeaf0101,0xfb840108,0xeeb00101,0xfb840108,0xeeb10101,
	0xfb840108,0xeeb20101,0xfb840108,0xeeb30101,0xfb840108,0xeeb40101,0xfb840108,0xeeb50101,
	0xfb840108,0xeeb60101,0xfb840108,0xeeb70101,0xfb840108,0xeeb80101,0xfb840108,0xeeb90101,
	0xfb840108,0xeeba0101,0xfb840108,0xeebb0101,0xfb840108,0xeebc0101,0xfb840108,0xeebd0101,
	0xfb840108,0xeebe0101,0xfb840108,0xeebf0101,0xfb840108,0xeec00101,0xfb840108,0xeec10101,
	0xfb840108,0xeec20101,0xfb840108,0xeec30101,0xfb840108,0xeec40101,0xfb840108,0xeec50101,
	0xfb840108,0xeec60101,0xfb840108,0xeec70101,0xfb840108,0xeec80101,0xfb840108,0xeec90101,
	0xfb840108,0xeeca0101,0xfb840108,0xeecb0101,0xfb840108,0xeecc0101,0xfb840108,0xeecd0101,
	0xfb840108,0xeece0101,0xfb840108,0xeecf0101,0xfb840108,0xeed00101,0xfb840108,0xeed10101,
	0xfb840108,0xeed20101,0xfb840108,0xeed30101,0xfb840108,0xeed40101,0xfb840108,0xeed50101,
	0xfb840108,0xeed60101,0xfb840108,0xeed70101,0xfb840108,0xeed80101,0xfb840108,0xeed90101,
	0xfb840108,0xeeda0101,0xfb840108,0xeedb0101,0xfb840108,0xeedc0101,0xfb840108,0xeedd0101,
	0xfb840108,0xeede0101,0xfb840108,0xeedf0101,0xfb840108,0xeee00101,0xfb840108,0xeee10101,
	0xfb840108,0xeee20101,0xfb840108,0xeee30101,0xfb840108,0xeee40101,0xfb840108,0xeee50101,
	0xfb840108,0xeee60101,0xfb840108,0xeee70101,0xfb840108,0xeee80101,0xfb840108,0xeee90101,
	0xfb840108,0xeeea0101,0xfb840108,0xeeeb0101,0xfb840108,0xeeec0101,0xfb840108,0xeeed0101,
	0xfb840108,0xeeee0101,0xfb840108,0xeeef0101,0xfb840108,0xeef00101,0xfb840108,0xeef10101,
	0xfb840108,0xeef20101,0xfb840108,0xeef30101,0xfb840108,0xeef40101,0xfb840108,0xeef50101,
	0xfb840108,0xeef60101,0xfb840108,0xeef70101,0xfb840108,0xeef80101,0xfb840108,0xeef90101,
	0xfb840108,0xeefa0101,0xfb840108,0xeefb0101,0xfb840108,0xeefc0101,0xfb840108,0xeefd0101,
	0xfb840108,0xeefe0101,0xfb840108,0xeeff0101,0xfb840108,0xef000101,0xfb840108,0xef010101,
	0xfb840108,0xef020101,0xfb840108,0xef030101,0xfb840108,0xef040101,0xfb840108,0xef050101,
	0xfb840108,0xef060101,0xfb840108,0xef070101,0xfb840108,0xef080101,0xfb840108,0xef090101,
	0xfb840108,0xef0a0101,0xfb840108,0xef0b0101,0xfb840108,0xef0c0101,0xfb840108,0xef0d0101,
	0xfb840108,0xef0e0101,0xfb840108,0xef0f0101,0xfb840108,0xef100101,0xfb840108,0xef110101,
	0xfb840108,0xef120101,0xfb840108,0xef130101,0xfb840108,0xef140101,0xfb840108,0xef150101,
	0xfb840108,0xef160101,0xfb840108,0xef170101,0xfb840108,0xef180101,0xfb840108,0xef190101,
	0xfb840108,0xef1a0101,0xfb840108,0xef1b0101,0xfb840108,0xef1c0101,0xfb840108,0xef1d0101,
	0xfb840108,0xef1e0101,0xfb840108,0xef1f0101,0xfb840108,0xef200101,0xfb840108,0xef210101,
	0xfb840108,0xef220101,0xfb840108,0xef230101,0xfb840108,0xef240101,0xfb840108,0xef250101,
	0xfb840108,0xef260101,0xfb840108,0xef270101,0xfb840108,0xef280101,0xfb840108,0xef290101,
	0xfb840108,0xef2a0101,0xfb840108,0xef2b0101,0xfb840108,0xef2c0101,0xfb840108,0xef2d0101,
	0xfb840108,0xef2e0101,0xfb840108,0xef2f0101,0xfb840108,0xef300101,0xfb840108,0xef310101,
	0xfb840108,0xef320101,0xfb840108,0xef330101,0xfb840108,0xef340101,0xfb840108,0xef350101,
	0xfb840108,0xef360101,0xfb840108,0xef370101,0xfb840108,0xef380101,0xfb840108,0xef390101,
	0xfb840108,0xef3a0101,0xfb840108,0xef3b0101,0xfb840108,0xef3c0101,0xfb840108,0xef3d0101,
	0xfb840108,0xef3e0101,0xfb840108,0xef3f0101,0xfb840108,0xef400101,0xfb840108,0xef410101,
	0xfb840108,0xef420101,0xfb840108,0xef430101,0xfb840108,0xef440101,0xfb840108,0xef450101,
	0xfb840108,0xef460101,0xfb840108,0xef470101,0xfb840108,0xef480101,0xfb840108,0xef490101,
	0xfb840108,0xef4a0101,0xfb840108,0xef4b0101,0xfb840108,0xef4c0101,0xfb840108,0xef4d0101,
	0xfb840108,0xef4e0101,0xfb840108,0xef4f0101,0xfb840108,0xef500101,0xfb840108,0xef510101,
	0xfb840108,0xef520101,0xfb840108,0xef530101,0xfb840108,0xef540101,0xfb840108,0xef550101,
	0xfb840108,0xef560101,0xfb840108,0xef570101,0xfb840108,0xef580101,0xfb840108,0xef590101,
	0xfb840108,0xef5a0101,0xfb840108,0xef5b0101,0xfb840108,0xef5c0101,0xfb840108,0xef5d0101,
	0xfb840108,0xef5e0101,0xfb840108,0xef5f0101,0xfb840108,0xef600101,0xfb840108,0xef610101,
	0xfb840108,0xef620101,0xfb840108,0xef630101,0xfb840108,0xef640101,0xfb840108,0xef650101,
	0xfb840108,0xef660101,0xfb840108,0xef670101,0xfb840108,0xef680101,0xfb840108,0xef690101,
	0xfb840108,0xef6a0101,0xfb840108,0xef6b0101,0xfb840108,0xef6c0101,0xfb840108,0xef6d0101,
	0xfb840108,0xef6e0101,0xfb840108,0xef6f0101,0xfb840108,0xef700101,0xfb840108,0xef710101,
	0xfb840108,0xef720101,0xfb840108,0xef730101,0xfb840108,0xef740101,0xfb840108,0xef750101,
	0xfb840108,0xef760101,0xfb840108,0xef770101,0xfb840108,0xef780101,0xfb840108,0xef790101,
	0xfb840108,0xef7a0101,0xfb840108,0xef7b0101,0xfb840108,0xef7c0101,0xfb840108,0xef7d0101,
	0xfb840108,0xef7e0101,0xfb840108,0xef7f0101,0xfb840108,0xef800101,0xfb840108,0xef810101,
	0xfb840108,0xef820101,0xfb840108,0xef830101,0xfb840108,0xef840101,0xfb840108,0xef850101,
	0xfb840108,0xef860101,0xfb840108,0xef870101,0xfb840108,0xef880101,0xfb840108,0xef890101,
	0xfb840108,0xef8a0101,0xfb840108,0xef8b0101,0xfb840108,0xef8c0101,0xfb840108,0xef8d0101,
	0xfb840108,0xef8e0101,0xfb840108,0xef8f0101,0xfb840108,0xef900101,0xfb840108,0xef910101,
	0xfb840108,0xef920101,0xfb840108,0xef930101,0xfb840108,0xef940101,0xfb840108,0xef950101,
	0xfb840108,0xef960101,0xfb840108,0xef970101,0xfb840108,0xef980101,0xfb840108,0xef990101,
	0xfb840108,0xef9a0101,0xfb840108,0xef9b0101,0xfb840108,0xef9c0101,0xfb840108,0xef9d0101,
	0xfb840108,0xef9e0101,0xfb840108,0xef9f0101,0xfb840108,0xefa00101,0xfb840108,0xefa10101,
	0xfb840108,0xefa20101,0xfb840108,0xefa30101,0xfb840108,0xefa40101,0xfb840108,0xefa50101,
	0xfb840108,0xefa60101,0xfb840108,0xefa70101,0xfb840108,0xefa80101,0xfb840108,0xefa90101,
	0xfb840108,0xefaa0101,0xfb840108,0xefab0101,0xfb840108,0xefac0101,0xfb840108,0xefad0101,
	0xfb840108,0xefae0101,0xfb840108,0xefaf0101,0xfb840108,0xefb00101,0xfb840108,0xefb10101,
	0xfb840108,0xefb20101,0xfb840108,0xefb30101,0xfb840108,0xefb40101,0xfb840108,0xefb50101,
	0xfb840108,0xefb60101,0xfb840108,0xefb70101,0xfb840108,0xefb80101,0xfb840108,0xefb90101,
	0xfb840108,0xefba0101,0xfb840108,0xefbb0101,0xfb840108,0xefbc0101,0xfb840108,0xefbd0101,
	0xfb840108,0xefbe0101,0xfb840108,0xefbf0101,0xfb840108,0xefc00101,0xfb840108,0xefc10101,
	0xfb840108,0xefc20101,0xfb840108,0xefc30101,0xfb840108,0xefc40101,0xfb840108,0xefc50101,
	0xfb840108,0xefc60101,0xfb840108,0xefc70101,0xfb840108,0xefc80101,0xfb840108,0xefc90101,
	0xfb840108,0xefca0101,0xfb840108,0xefcb0101,0xfb840108,0xefcc0101,0xfb840108,0xefcd0101,
	0xfb840108,0xefce0101,0xfb840108,0xefcf0101,0xfb840108,0xefd00101,0xfb840108,0xefd10101,
	0xfb840108,0xefd20101,0xfb840108,0xefd30101,0xfb840108,0xefd40101,0xfb840108,0xefd50101,
	0xfb840108,0xefd60101,0xfb840108,0xefd70101,0xfb840108,0xefd80101,0xfb840108,0xefd90101,
	0xfb840108,0xefda0101,0xfb840108,0xefdb0101,0xfb840108,0xefdc0101,0xfb840108,0xefdd0101,
	0xfb840108,0xefde0101,0xfb840108,0xefdf0101,0xfb840108,0xefe00101,0xfb840108,0xefe10101,
	0xfb840108,0xefe20101,0xfb840108,0xefe30101,0xfb840108,0xefe40101,0xfb840108,0xefe50101,
	0xfb840108,0xefe60101,0xfb840108,0xefe70101,0xfb840108,0xefe80101,0xfb840108,0xefe90101,
	0xfb840108,0xefea0101,0xfb840108,0xefeb0101,0xfb840108,0xefec0101,0xfb840108,0xefed0101,
	0xfb840108,0xefee0101,0xfb840108,0xefef0101,0xfb840108,0xeff00101,0xfb840108,0xeff10101,
	0xfb840108,0xeff20101,0xfb840108,0xeff30101,0xfb840108,0xeff40101,0xfb840108,0xeff50101,
	0xfb840108,0xeff60101,0xfb840108,0xeff70101,0xfb840108,0xeff80101,0xfb840108,0xeff90101,
	0xfb840108,0xeffa0101,0xfb840108,0xeffb0101,0xfb840108,0xeffc0101,0xfb840108,0xeffd0101,
	0xfb840108,0xeffe0101,0xfb840108,0xefff0101,0xfb840108,0xf0000101,0xfb840108,0xf0010101,
	0xfb840108,0xf0020101,0xfb840108,0xf0030101,0xfb840108,0xf0040101,0xfb840108,0xf0050101,
	0xfb840108,0xf0060101,0xfb840108,0xf0070101,0xfb840108,0xf0080101,0xfb840108,0xf0090101,
	0xfb840108,0xf00a0101,0xfb840108,0xf00b0101,0xfb840108,0xf00c0101,0xfb840108,0xf00d0101,
	0xfb840108,0xf00e0101,0xfb840108,0xf00f0101,0xfb840108,0xf0100101,0xfb840108,0xf0110101,
	0xfb840108,0xf0120101,0xfb840108,0xf0130101,0xfb840108,0xf0140101,0xfb840108,0xf0150101,
	0xfb840108,0xf0160101,0xfb840108,0xf0170101,0xfb840108,0xf0180101,0xfb840108,0xf0190101,
	0xfb840108,0xf01a0101,0xfb840108,0xf01b0101,0xfb840108,0xf01c0101,0xfb840108,0xf01d0101,
	0xfb840108,0xf01e0101,0xfb840108,0xf01f0101,0xfb840108,0xf0200101,0xfb840108,0xf0210101,
	0xfb840108,0xf0220101,0xfb840108,0xf0230101,0xfb840108,0xf0240101,0xfb840108,0xf0250101,
	0xfb840108,0xf0260101,0xfb840108,0xf0270101,0xfb840108,0xf0280101,0xfb840108,0xf0290101,
	0xfb840108,0xf02a0101,0xfb840108,0xf02b0101,0xfb840108,0xf02c0101,0xfb840108,0xf02d0101,
	0xfb840108,0xf02e0101,0xfb840108,0xf02f0101,0xfb840108,0xf0300101,0xfb840108,0xf0310101,
	0xfb840108,0xf0320101,0xfb840108,0xf0330101,0xfb840108,0xf0340101,0xfb840108,0xf0350101,
	0xfb840108,0xf0360101,0xfb840108,0xf0370101,0xfb840108,0xf0380101,0xfb840108,0xf0390101,
	0xfb840108,0xf03a0101,0xfb840108,0xf03b0101,0xfb840108,0xf03c0101,0xfb840108,0xf03d0101,
	0xfb840108,0xf03e0101,0xfb840108,0xf03f0101,0xfb840108,0xf0400101,0xfb840108,0xf0410101,
	0xfb840108,0xf0420101,0xfb840108,0xf0430101,0xfb840108,0xf0440101,0xfb840108,0xf0450101,
	0xfb840108,0xf0460101,0xfb840108,0xf0470101,0xfb840108,0xf0480101,0xfb840108,0xf0490101,
	0xfb840108,0xf04a0101,0xfb840108,0xf04b0101,0xfb840108,0xf04c0101,0xfb840108,0xf04d0101,
	0xfb840108,0xf04e0101,0xfb840108,0xf04f0101,0xfb840108,0xf0500101,0xfb840108,0xf0510101,
	0xfb840108,0xf0520101,0xfb840108,0xf0530101,0xfb840108,0xf0540101,0xfb840108,0xf0550101,
	0xfb840108,0xf0560101,0xfb840108,0xf0570101,0xfb840108,0xf0580101,0xfb840108,0xf0590101,
	0xfb840108,0xf05a0101,0xfb840108,0xf05b0101,0xfb840108,0xf05c0101,0xfb840108,0xf05d0101,
	0xfb840108,0xf05e0101,0xfb840108,0xf05f0101,0xfb840108,0xf0600101,0xfb840108,0xf0610101,
	0xfb840108,0xf0620101,0xfb840108,0xf0630101,0xfb840108,0xf0640101,0xfb840108,0xf0650101,
	0xfb840108,0xf0660101,0xfb840108,0xf0670101,0xfb840108,0xf0680101,0xfb840108,0xf0690101,
	0xfb840108,0xf06a0101,0xfb840108,0xf06b0101,0xfb840108,0xf06c0101,0xfb840108,0xf06d0101,
	0xfb840108,0xf06e0101,0xfb840108,0xf06f0101,0xfb840108,0xf0700101,0xfb840108,0xf0710101,
	0xfb840108,0xf0720101,0xfb840108,0xf0730101,0xfb840108,0xf0740101,0xfb840108,0xf0750101,
	0xfb840108,0xf0760101,0xfb840108,0xf0770101,0xfb840108,0xf0780101,0xfb840108,0xf0790101,
	0xfb840108,0xf07a0101,0xfb840108,0xf07b0101,0xfb840108,0xf07c0101,0xfb840108,0xf07d0101,
	0xfb840108,0xf07e0101,0xfb840108,0xf07f0101,0xfb840108,0xf0800101,0xfb840108,0xf0810101,
	0xfb840108,0xf0820101,0xfb840108,0xf0830101,0xfb840108,0xf0840101,0xfb840108,0xf0850101,
	0xfb840108,0xf0860101,0xfb840108,0xf0870101,0xfb840108,0xf0880101,0xfb840108,0xf0890101,
	0xfb840108,0xf08a0101,0xfb840108,0xf08b0101,0xfb840108,0xf08c0101,0xfb840108,0xf08d0101,
	0xfb840108,0xf08e0101,0xfb840108,0xf08f0101,0xfb840108,0xf0900101,0xfb840108,0xf0910101,
	0xfb840108,0xf0920101,0xfb840108,0xf0930101,0xfb840108,0xf0940101,0xfb840108,0xf0950101,
	0xfb840108,0xf0960101,0xfb840108,0xf0970101,0xfb840108,0xf0980101,0xfb840108,0xf0990101,
	0xfb840108,0xf09a0101,0xfb840108,0xf09b0101,0xfb840108,0xf09c0101,0xfb840108,0xf09d0101,
	0xfb840108,0xf09e0101,0xfb840108,0xf09f0101,0xfb840108,0xf0a00101,0xfb840108,0xf0a10101,
	0xfb840108,0xf0a20101,0xfb840108,0xf0a30101,0xfb840108,0xf0a40101,0xfb840108,0xf0a50101,
	0xfb840108,0xf0a60101,0xfb840108,0xf0a70101,0xfb840108,0xf0a80101,0xfb840108,0xf0a90101,
	0xfb840108,0xf0aa0101,0xfb840108,0xf0ab0101,0xfb840108,0xf0ac0101,0xfb840108,0xf0ad0101,
	0xfb840108,0xf0ae0101,0xfb840108,0xf0af0101,0xfb840108,0xf0b00101,0xfb840108,0xf0b10101,
	0xfb840108,0xf0b20101,0xfb840108,0xf0b30101,0xfb840108,0xf0b40101,0xfb840108,0xf0b50101,
	0xfb840108,0xf0b60101,0xfb840108,0xf0b70101,0xfb840108,0xf0b80101,0xfb840108,0xf0b90101,
	0xfb840108,0xf0ba0101,0xfb840108,0xf0bb0101,0xfb840108,0xf0bc0101,0xfb840108,0xf0bd0101,
	0xfb840108,0xf0be0101,0xfb840108,0xf0bf0101,0xfb840108,0xf0c00101,0xfb840108,0xf0c10101,
	0xfb840108,0xf0c20101,0xfb840108,0xf0c30101,0xfb840108,0xf0c40101,0xfb840108,0xf0c50101,
	0xfb840108,0xf0c60101,0xfb840108,0xf0c70101,0xfb840108,0xf0c80101,0xfb840108,0xf0c90101,
	0xfb840108,0xf0ca0101,0xfb840108,0xf0cb0101,0xfb840108,0xf0cc0101,0xfb840108,0xf0cd0101,
	0xfb840108,0xf0ce0101,0xfb840108,0xf0cf0101,0xfb840108,0xf0d00101,0xfb840108,0xf0d10101,
	0xfb840108,0xf0d20101,0xfb840108,0xf0d30101,0xfb840108,0xf0d40101,0xfb840108,0xf0d50101,
	0xfb840108,0xf0d60101,0xfb840108,0xf0d70101,0xfb840108,0xf0d80101,0xfb840108,0xf0d90101,
	0xfb840108,0xf0da0101,0xfb840108,0xf0db0101,0xfb840108,0xf0dc0101,0xfb840108,0xf0dd0101,
	0xfb840108,0xf0de0101,0xfb840108,0xf0df0101,0xfb840108,0xf0e00101,0xfb840108,0xf0e10101,
	0xfb840108,0xf0e20101,0xfb840108,0xf0e30101,0xfb840108,0xf0e40101,0xfb840108,0xf0e50101,
	0xfb840108,0xf0e60101,0xfb840108,0xf0e70101,0xfb840108,0xf0e80101,0xfb840108,0xf0e90101,
	0xfb840108,0xf0ea0101,0xfb840108,0xf0eb0101,0xfb840108,0xf0ec0101,0xfb840108,0xf0ed0101,
	0xfb840108,0xf0ee0101,0xfb840108,0xf0ef0101,0xfb840108,0xf0f00101,0xfb840108,0xf0f10101,
	0xfb840108,0xf0f20101,0xfb840108,0xf0f30101,0xfb840108,0xf0f40101,0xfb840108,0xf0f50101,
	0xfb840108,0xf0f60101,0xfb840108,0xf0f70101,0xfb840108,0xf0f80101,0xfb840108,0xf0f90101,
	0xfb840108,0xf0fa0101,0xfb840108,0xf0fb0101,0xfb840108,0xf0fc0101,0xfb840108,0xf0fd0101,
	0xfb840108,0xf0fe0101,0xfb840108,0xf0ff0101,0xfb840108,0xf1000101,0xfb840108,0xf1010101,
	0xfb840108,0xf1020101,0xfb840108,0xf1030101,0xfb840108,0xf1040101,0xfb840108,0xf1050101,
	0xfb840108,0xf1060101,0xfb840108,0xf1070101,0xfb840108,0xf1080101,0xfb840108,0xf1090101,
	0xfb840108,0xf10a0101,0xfb840108,0xf10b0101,0xfb840108,0xf10c0101,0xfb840108,0xf10d0101,
	0xfb840108,0xf10e0101,0xfb840108,0xf10f0101,0xfb840108,0xf1100101,0xfb840108,0xf1110101,
	0xfb840108,0xf1120101,0xfb840108,0xf1130101,0xfb840108,0xf1140101,0xfb840108,0xf1150101,
	0xfb840108,0xf1160101,0xfb840108,0xf1170101,0xfb840108,0xf1180101,0xfb840108,0xf1190101,
	0xfb840108,0xf11a0101,0xfb840108,0xf11b0101,0xfb840108,0xf11c0101,0xfb840108,0xf11d0101,
	0xfb840108,0xf11e0101,0xfb840108,0xf11f0101,0xfb840108,0xf1200101,0xfb840108,0xf1210101,
	0xfb840108,0xf1220101,0xfb840108,0xf1230101,0xfb840108,0xf1240101,0xfb840108,0xf1250101,
	0xfb840108,0xf1260101,0xfb840108,0xf1270101,0xfb840108,0xf1280101,0xfb840108,0xf1290101,
	0xfb840108,0xf12a0101,0xfb840108,0xf12b0101,0xfb840108,0xf12c0101,0xfb840108,0xf12d0101,
	0xfb840108,0xf12e0101,0xfb840108,0xf12f0101,0xfb840108,0xf1300101,0xfb840108,0xf1310101,
	0xfb840108,0xf1320101,0xfb840108,0xf1330101,0xfb840108,0xf1340101,0xfb840108,0xf1350101,
	0xfb840108,0xf1360101,0xfb840108,0xf1370101,0xfb840108,0xf1380101,0xfb840108,0xf1390101,
	0xfb840108,0xf13a0101,0xfb840108,0xf13b0101,0xfb840108,0xf13c0101,0xfb840108,0xf13d0101,
	0xfb840108,0xf13e0101,0xfb840108,0xf13f0101,0xfb840108,0xf1400101,0xfb840108,0xf1410101,
	0xfb840108,0xf1420101,0xfb840108,0xf1430101,0xfb840108,0xf1440101,0xfb840108,0xf1450101,
	0xfb840108,0xf1460101,0xfb840108,0xf1470101,0xfb840108,0xf1480101,0xfb840108,0xf1490101,
	0xfb840108,0xf14a0101,0xfb840108,0xf14b0101,0xfb840108,0xf14c0101,0xfb840108,0xf14d0101,
	0xfb840108,0xf14e0101,0xfb840108,0xf14f0101,0xfb840108,0xf1500101,0xfb840108,0xf1510101,
	0xfb840108,0xf1520101,0xfb840108,0xf1530101,0xfb840108,0xf1540101,0xfb840108,0xf1550101,
	0xfb840108,0xf1560101,0xfb840108,0xf1570101,0xfb840108,0xf1580101,0xfb840108,0xf1590101,
	0xfb840108,0xf15a0101,0xfb840108,0xf15b0101,0xfb840108,0xf15c0101,0xfb840108,0xf15d0101,
	0xfb840108,0xf15e0101,0xfb840108,0xf15f0101,0xfb840108,0xf1600101,0xfb840108,0xf1610101,
	0xfb840108,0xf1620101,0xfb840108,0xf1630101,0xfb840108,0xf1640101,0xfb840108,0xf1650101,
	0xfb840108,0xf1660101,0xfb840108,0xf1670101,0xfb840108,0xf1680101,0xfb840108,0xf1690101,
	0xfb840108,0xf16a0101,0xfb840108,0xf16b0101,0xfb840108,0xf16c0101,0xfb840108,0xf16d0101,
	0xfb840108,0xf16e0101,0xfb840108,0xf16f0101,0xfb840108,0xf1700101,0xfb840108,0xf1710101,
	0xfb840108,0xf1720101,0xfb840108,0xf1730101,0xfb840108,0xf1740101,0xfb840108,0xf1750101,
	0xfb840108,0xf1760101,0xfb840108,0xf1770101,0xfb840108,0xf1780101,0xfb840108,0xf1790101,
	0xfb840108,0xf17a0101,0xfb840108,0xf17b0101,0xfb840108,0xf17c0101,0xfb840108,0xf17d0101,
	0xfb840108,0xf17e0101,0xfb840108,0xf17f0101,0xfb840108,0xf1800101,0xfb840108,0xf1810101,
	0xfb840108,0xf1820101,0xfb840108,0xf1830101,0xfb840108,0xf1840101,0xfb840108,0xf1850101,
	0xfb840108,0xf1860101,0xfb840108,0xf1870101,0xfb840108,0xf1880101,0xfb840108,0xf1890101,
	0xfb840108,0xf18a0101,0xfb840108,0xf18b0101,0xfb840108,0xf18c0101,0xfb840108,0xf18d0101,
	0xfb840108,0xf18e0101,0xfb840108,0xf18f0101,0xfb840108,0xf1900101,0xfb840108,0xf1910101,
	0xfb840108,0xf1920101,0xfb840108,0xf1930101,0xfb840108,0xf1940101,0xfb840108,0xf1950101,
	0xfb840108,0xf1960101,0xfb840108,0xf1970101,0xfb840108,0xf1980101,0xfb840108,0xf1990101,
	0xfb840108,0xf19a0101,0xfb840108,0xf19b0101,0xfb840108,0xf19c0101,0xfb840108,0xf19d0101,
	0xfb840108,0xf19e0101,0xfb840108,0xf19f0101,0xfb840108,0xf1a00101,0xfb840108,0xf1a10101,
	0xfb840108,0xf1a20101,0xfb840108,0xf1a30101,0xfb840108,0xf1a40101,0xfb840108,0xf1a50101,
	0xfb840108,0xf1a60101,0xfb840108,0xf1a70101,0xfb840108,0xf1a80101,0xfb840108,0xf1a90101,
	0xfb840108,0xf1aa0101,0xfb840108,0xf1ab0101,0xfb840108,0xf1ac0101,0xfb840108,0xf1ad0101,
	0xfb840108,0xf1ae0101,0xfb840108,0xf1af0101,0xfb840108,0xf1b00101,0xfb840108,0xf1b10101,
	0xfb840108,0xf1b20101,0xfb840108,0xf1b30101,0xfb840108,0xf1b40101,0xfb840108,0xf1b50101,
	0xfb840108,0xf1b60101,0xfb840108,0xf1b70101,0xfb840108,0xf1b80101,0xfb840108,0xf1b90101,
	0xfb840108,0xf1ba0101,0xfb840108,0xf1bb0101,0xfb840108,0xf1bc0101,0xfb840108,0xf1bd0101,
	0xfb840108,0xf1be0101,0xfb840108,0xf1bf0101,0xfb840108,0xf1c00101,0xfb840108,0xf1c10101,
	0xfb840108,0xf1c20101,0xfb840108,0xf1c30101,0xfb840108,0xf1c40101,0xfb840108,0xf1c50101,
	0xfb840108,0xf1c60101,0xfb840108,0xf1c70101,0xfb840108,0xf1c80101,0xfb840108,0xf1c90101,
	0xfb840108,0xf1ca0101,0xfb840108,0xf1cb0101,0xfb840108,0xf1cc0101,0xfb840108,0xf1cd0101,
	0xfb840108,0xf1ce0101,0xfb840108,0xf1cf0101,0xfb840108,0xf1d00101,0xfb840108,0xf1d10101,
	0xfb840108,0xf1d20101,0xfb840108,0xf1d30101,0xfb840108,0xf1d40101,0xfb840108,0xf1d50101,
	0xfb840108,0xf1d60101,0xfb840108,0xf1d70101,0xfb840108,0xf1d80101,0xfb840108,0xf1d90101,
	0xfb840108,0xf1da0101,0xfb840108,0xf1db0101,0xfb840108,0xf1dc0101,0xfb840108,0xf1dd0101,
	0xfb840108,0xf1de0101,0xfb840108,0xf1df0101,0xfb840108,0xf1e00101,0xfb840108,0xf1e10101,
	0xfb840108,0xf1e20101,0xfb840108,0xf1e30101,0xfb840108,0xf1e40101,0xfb840108,0xf1e50101,
	0xfb840108,0xf1e60101,0xfb840108,0xf1e70101,0xfb840108,0xf1e80101,0xfb840108,0xf1e90101,
	0xfb840108,0xf1ea0101,0xfb840108,0xf1eb0101,0xfb840108,0xf1ec0101,0xfb840108,0xf1ed0101,
	0xfb840108,0xf1ee0101,0xfb840108,0xf1ef0101,0xfb840108,0xf1f00101,0xfb840108,0xf1f10101,
	0xfb840108,0xf1f20101,0xfb840108,0xf1f30101,0xfb840108,0xf1f40101,0xfb840108,0xf1f50101,
	0xfb840108,0xf1f60101,0xfb840108,0xf1f70101,0xfb840108,0xf1f80101,0xfb840108,0xf1f90101,
	0xfb840108,0xf1fa0101,0xfb840108,0xf1fb0101,0xfb840108,0xf1fc0101,0xfb840108,0xf1fd0101,
	0xfb840108,0xf1fe0101,0xfb840108,0xf1ff0101,0xfb840108,0xf2000101,0xfb840108,0xf2010101,
	0xfb840108,0xf2020101,0xfb840108,0xf2030101,0xfb840108,0xf2040101,0xfb840108,0xf2050101,
	0xfb840108,0xf2060101,0xfb840108,0xf2070101,0xfb840108,0xf2080101,0xfb840108,0xf2090101,
	0xfb840108,0xf20a0101,0xfb840108,0xf20b0101,0xfb840108,0xf20c0101,0xfb840108,0xf20d0101,
	0xfb840108,0xf20e0101,0xfb840108,0xf20f0101,0xfb840108,0xf2100101,0xfb840108,0xf2110101,
	0xfb840108,0xf2120101,0xfb840108,0xf2130101,0xfb840108,0xf2140101,0xfb840108,0xf2150101,
	0xfb840108,0xf2160101,0xfb840108,0xf2170101,0xfb840108,0xf2180101,0xfb840108,0xf2190101,
	0xfb840108,0xf21a0101,0xfb840108,0xf21b0101,0xfb840108,0xf21c0101,0xfb840108,0xf21d0101,
	0xfb840108,0xf21e0101,0xfb840108,0xf21f0101,0xfb840108,0xf2200101,0xfb840108,0xf2210101,
	0xfb840108,0xf2220101,0xfb840108,0xf2230101,0xfb840108,0xf2240101,0xfb840108,0xf2250101,
	0xfb840108,0xf2260101,0xfb840108,0xf2270101,0xfb840108,0xf2280101,0xfb840108,0xf2290101,
	0xfb840108,0xf22a0101,0xfb840108,0xf22b0101,0xfb840108,0xf22c0101,0xfb840108,0xf22d0101,
	0xfb840108,0xf22e0101,0xfb840108,0xf22f0101,0xfb840108,0xf2300101,0xfb840108,0xf2310101,
	0xfb840108,0xf2320101,0xfb840108,0xf2330101,0xfb840108,0xf2340101,0xfb840108,0xf2350101,
	0xfb840108,0xf2360101,0xfb840108,0xf2370101,0xfb840108,0xf2380101,0xfb840108,0xf2390101,
	0xfb840108,0xf23a0101,0xfb840108,0xf23b0101,0xfb840108,0xf23c0101,0xfb840108,0xf23d0101,
	0xfb840108,0xf23e0101,0xfb840108,0xf23f0101,0xfb840108,0xf2400101,0xfb840108,0xf2410101,
	0xfb840108,0xf2420101,0xfb840108,0xf2430101,0xfb840108,0xf2440101,0xfb840108,0xf2450101,
	0xfb840108,0xf2460101,0xfb840108,0xf2470101,0xfb840108,0xf2480101,0xfb840108,0xf2490101,
	0xfb840108,0xf24a0101,0xfb840108,0xf24b0101,0xfb840108,0xf24c0101,0xfb840108,0xf24d0101,
	0xfb840108,0xf24e0101,0xfb840108,0xf24f0101,0xfb840108,0xf2500101,0xfb840108,0xf2510101,
	0xfb840108,0xf2520101,0xfb840108,0xf2530101,0xfb840108,0xf2540101,0xfb840108,0xf2550101,
	0xfb840108,0xf2560101,0xfb840108,0xf2570101,0xfb840108,0xf2580101,0xfb840108,0xf2590101,
	0xfb840108,0xf25a0101,0xfb840108,0xf25b0101,0xfb840108,0xf25c0101,0xfb840108,0xf25d0101,
	0xfb840108,0xf25e0101,0xfb840108,0xf25f0101,0xfb840108,0xf2600101,0xfb840108,0xf2610101,
	0xfb840108,0xf2620101,0xfb840108,0xf2630101,0xfb840108,0xf2640101,0xfb840108,0xf2650101,
	0xfb840108,0xf2660101,0xfb840108,0xf2670101,0xfb840108,0xf2680101,0xfb840108,0xf2690101,
	0xfb840108,0xf26a0101,0xfb840108,0xf26b0101,0xfb840108,0xf26c0101,0xfb840108,0xf26d0101,
	0xfb840108,0xf26e0101,0xfb840108,0xf26f0101,0xfb840108,0xf2700101,0xfb840108,0xf2710101,
	0xfb840108,0xf2720101,0xfb840108,0xf2730101,0xfb840108,0xf2740101,0xfb840108,0xf2750101,
	0xfb840108,0xf2760101,0xfb840108,0xf2770101,0xfb840108,0xf2780101,0xfb840108,0xf2790101,
	0xfb840108,0xf27a0101,0xfb840108,0xf27b0101,0xfb840108,0xf27c0101,0xfb840108,0xf27d0101,
	0xfb840108,0xf27e0101,0xfb840108,0xf27f0101,0xfb840108,0xf2800101,0xfb840108,0xf2810101,
	0xfb840108,0xf2820101,0xfb840108,0xf2830101,0xfb840108,0xf2840101,0xfb840108,0xf2850101,
	0xfb840108,0xf2860101,0xfb840108,0xf2870101,0xfb840108,0xf2880101,0xfb840108,0xf2890101,
	0xfb840108,0xf28a0101,0xfb840108,0xf28b0101,0xfb840108,0xf28c0101,0xfb840108,0xf28d0101,
	0xfb840108,0xf28e0101,0xfb840108,0xf28f0101,0xfb840108,0xf2900101,0xfb840108,0xf2910101,
	0xfb840108,0xf2920101,0xfb840108,0xf2930101,0xfb840108,0xf2940101,0xfb840108,0xf2950101,
	0xfb840108,0xf2960101,0xfb840108,0xf2970101,0xfb840108,0xf2980101,0xfb840108,0xf2990101,
	0xfb840108,0xf29a0101,0xfb840108,0xf29b0101,0xfb840108,0xf29c0101,0xfb840108,0xf29d0101,
	0xfb840108,0xf29e0101,0xfb840108,0xf29f0101,0xfb840108,0xf2a00101,0xfb840108,0xf2a10101,
	0xfb840108,0xf2a20101,0xfb840108,0xf2a30101,0xfb840108,0xf2a40101,0xfb840108,0xf2a50101,
	0xfb840108,0xf2a60101,0xfb840108,0xf2a70101,0xfb840108,0xf2a80101,0xfb840108,0xf2a90101,
	0xfb840108,0xf2aa0101,0xfb840108,0xf2ab0101,0xfb840108,0xf2ac0101,0xfb840108,0xf2ad0101,
	0xfb840108,0xf2ae0101,0xfb840108,0xf2af0101,0xfb840108,0xf2b00101,0xfb840108,0xf2b10101,
	0xfb840108,0xf2b20101,0xfb840108,0xf2b30101,0xfb840108,0xf2b40101,0xfb840108,0xf2b50101,
	0xfb840108,0xf2b60101,0xfb840108,0xf2b70101,0xfb840108,0xf2b80101,0xfb840108,0xf2b90101,
	0xfb840108,0xf2ba0101,0xfb840108,0xf2bb0101,0xfb840108,0xf2bc0101,0xfb840108,0xf2bd0101,
	0xfb840108,0xf2be0101,0xfb840108,0xf2bf0101,0xfb840108,0xf2c00101,0xfb840108,0xf2c10101,
	0xfb840108,0xf2c20101,0xfb840108,0xf2c30101,0xfb840108,0xf2c40101,0xfb840108,0xf2c50101,
	0xfb840108,0xf2c60101,0xfb840108,0xf2c70101,0xfb840108,0xf2c80101,0xfb840108,0xf2c90101,
	0xfb840108,0xf2ca0101,0xfb840108,0xf2cb0101,0xfb840108,0xf2cc0101,0xfb840108,0xf2cd0101,
	0xfb840108,0xf2ce0101,0xfb840108,0xf2cf0101,0xfb840108,0xf2d00101,0xfb840108,0xf2d10101,
	0xfb840108,0xf2d20101,0xfb840108,0xf2d30101,0xfb840108,0xf2d40101,0xfb840108,0xf2d50101,
	0xfb840108,0xf2d60101,0xfb840108,0xf2d70101,0xfb840108,0xf2d80101,0xfb840108,0xf2d90101,
	0xfb840108,0xf2da0101,0xfb840108,0xf2db0101,0xfb840108,0xf2dc0101,0xfb840108,0xf2dd0101,
	0xfb840108,0xf2de0101,0xfb840108,0xf2df0101,0xfb840108,0xf2e00101,0xfb840108,0xf2e10101,
	0xfb840108,0xf2e20101,0xfb840108,0xf2e30101,0xfb840108,0xf2e40101,0xfb840108,0xf2e50101,
	0xfb840108,0xf2e60101,0xfb840108,0xf2e70101,0xfb840108,0xf2e80101,0xfb840108,0xf2e90101,
	0xfb840108,0xf2ea0101,0xfb840108,0xf2eb0101,0xfb840108,0xf2ec0101,0xfb840108,0xf2ed0101,
	0xfb840108,0xf2ee0101,0xfb840108,0xf2ef0101,0xfb840108,0xf2f00101,0xfb840108,0xf2f10101,
	0xfb840108,0xf2f20101,0xfb840108,0xf2f30101,0xfb840108,0xf2f40101,0xfb840108,0xf2f50101,
	0xfb840108,0xf2f60101,0xfb840108,0xf2f70101,0xfb840108,0xf2f80101,0xfb840108,0xf2f90101,
	0xfb840108,0xf2fa0101,0xfb840108,0xf2fb0101,0xfb840108,0xf2fc0101,0xfb840108,0xf2fd0101,
	0xfb840108,0xf2fe0101,0xfb840108,0xf2ff0101,0xfb840108,0xf3000101,0xfb840108,0xf3010101,
	0xfb840108,0xf3020101,0xfb840108,0xf3030101,0xfb840108,0xf3040101,0xfb840108,0xf3050101,
	0xfb840108,0xf3060101,0xfb840108,0xf3070101,0xfb840108,0xf3080101,0xfb840108,0xf3090101,
	0xfb840108,0xf30a0101,0xfb840108,0xf30b0101,0xfb840108,0xf30c0101,0xfb840108,0xf30d0101,
	0xfb840108,0xf30e0101,0xfb840108,0xf30f0101,0xfb840108,0xf3100101,0xfb840108,0xf3110101,
	0xfb840108,0xf3120101,0xfb840108,0xf3130101,0xfb840108,0xf3140101,0xfb840108,0xf3150101,
	0xfb840108,0xf3160101,0xfb840108,0xf3170101,0xfb840108,0xf3180101,0xfb840108,0xf3190101,
	0xfb840108,0xf31a0101,0xfb840108,0xf31b0101,0xfb840108,0xf31c0101,0xfb840108,0xf31d0101,
	0xfb840108,0xf31e0101,0xfb840108,0xf31f0101,0xfb840108,0xf3200101,0xfb840108,0xf3210101,
	0xfb840108,0xf3220101,0xfb840108,0xf3230101,0xfb840108,0xf3240101,0xfb840108,0xf3250101,
	0xfb840108,0xf3260101,0xfb840108,0xf3270101,0xfb840108,0xf3280101,0xfb840108,0xf3290101,
	0xfb840108,0xf32a0101,0xfb840108,0xf32b0101,0xfb840108,0xf32c0101,0xfb840108,0xf32d0101,
	0xfb840108,0xf32e0101,0xfb840108,0xf32f0101,0xfb840108,0xf3300101,0xfb840108,0xf3310101,
	0xfb840108,0xf3320101,0xfb840108,0xf3330101,0xfb840108,0xf3340101,0xfb840108,0xf3350101,
	0xfb840108,0xf3360101,0xfb840108,0xf3370101,0xfb840108,0xf3380101,0xfb840108,0xf3390101,
	0xfb840108,0xf33a0101,0xfb840108,0xf33b0101,0xfb840108,0xf33c0101,0xfb840108,0xf33d0101,
	0xfb840108,0xf33e0101,0xfb840108,0xf33f0101,0xfb840108,0xf3400101,0xfb840108,0xf3410101,
	0xfb840108,0xf3420101,0xfb840108,0xf3430101,0xfb840108,0xf3440101,0xfb840108,0xf3450101,
	0xfb840108,0xf3460101,0xfb840108,0xf3470101,0xfb840108,0xf3480101,0xfb840108,0xf3490101,
	0xfb840108,0xf34a0101,0xfb840108,0xf34b0101,0xfb840108,0xf34c0101,0xfb840108,0xf34d0101,
	0xfb840108,0xf34e0101,0xfb840108,0xf34f0101,0xfb840108,0xf3500101,0xfb840108,0xf3510101,
	0xfb840108,0xf3520101,0xfb840108,0xf3530101,0xfb840108,0xf3540101,0xfb840108,0xf3550101,
	0xfb840108,0xf3560101,0xfb840108,0xf3570101,0xfb840108,0xf3580101,0xfb840108,0xf3590101,
	0xfb840108,0xf35a0101,0xfb840108,0xf35b0101,0xfb840108,0xf35c0101,0xfb840108,0xf35d0101,
	0xfb840108,0xf35e0101,0xfb840108,0xf35f0101,0xfb840108,0xf3600101,0xfb840108,0xf3610101,
	0xfb840108,0xf3620101,0xfb840108,0xf3630101,0xfb840108,0xf3640101,0xfb840108,0xf3650101,
	0xfb840108,0xf3660101,0xfb840108,0xf3670101,0xfb840108,0xf3680101,0xfb840108,0xf3690101,
	0xfb840108,0xf36a0101,0xfb840108,0xf36b0101,0xfb840108,0xf36c0101,0xfb840108,0xf36d0101,
	0xfb840108,0xf36e0101,0xfb840108,0xf36f0101,0xfb840108,0xf3700101,0xfb840108,0xf3710101,
	0xfb840108,0xf3720101,0xfb840108,0xf3730101,0xfb840108,0xf3740101,0xfb840108,0xf3750101,
	0xfb840108,0xf3760101,0xfb840108,0xf3770101,0xfb840108,0xf3780101,0xfb840108,0xf3790101,
	0xfb840108,0xf37a0101,0xfb840108,0xf37b0101,0xfb840108,0xf37c0101,0xfb840108,0xf37d0101,
	0xfb840108,0xf37e0101,0xfb840108,0xf37f0101,0xfb840108,0xf3800101,0xfb840108,0xf3810101,
	0xfb840108,0xf3820101,0xfb840108,0xf3830101,0xfb840108,0xf3840101,0xfb840108,0xf3850101,
	0xfb840108,0xf3860101,0xfb840108,0xf3870101,0xfb840108,0xf3880101,0xfb840108,0xf3890101,
	0xfb840108,0xf38a0101,0xfb840108,0xf38b0101,0xfb840108,0xf38c0101,0xfb840108,0xf38d0101,
	0xfb840108,0xf38e0101,0xfb840108,0xf38f0101,0xfb840108,0xf3900101,0xfb840108,0xf3910101,
	0xfb840108,0xf3920101,0xfb840108,0xf3930101,0xfb840108,0xf3940101,0xfb840108,0xf3950101,
	0xfb840108,0xf3960101,0xfb840108,0xf3970101,0xfb840108,0xf3980101,0xfb840108,0xf3990101,
	0xfb840108,0xf39a0101,0xfb840108,0xf39b0101,0xfb840108,0xf39c0101,0xfb840108,0xf39d0101,
	0xfb840108,0xf39e0101,0xfb840108,0xf39f0101,0xfb840108,0xf3a00101,0xfb840108,0xf3a10101,
	0xfb840108,0xf3a20101,0xfb840108,0xf3a30101,0xfb840108,0xf3a40101,0xfb840108,0xf3a50101,
	0xfb840108,0xf3a60101,0xfb840108,0xf3a70101,0xfb840108,0xf3a80101,0xfb840108,0xf3a90101,
	0xfb840108,0xf3aa0101,0xfb840108,0xf3ab0101,0xfb840108,0xf3ac0101,0xfb840108,0xf3ad0101,
	0xfb840108,0xf3ae0101,0xfb840108,0xf3af0101,0xfb840108,0xf3b00101,0xfb840108,0xf3b10101,
	0xfb840108,0xf3b20101,0xfb840108,0xf3b30101,0xfb840108,0xf3b40101,0xfb840108,0xf3b50101,
	0xfb840108,0xf3b60101,0xfb840108,0xf3b70101,0xfb840108,0xf3b80101,0xfb840108,0xf3b90101,
	0xfb840108,0xf3ba0101,0xfb840108,0xf3bb0101,0xfb840108,0xf3bc0101,0xfb840108,0xf3bd0101,
	0xfb840108,0xf3be0101,0xfb840108,0xf3bf0101,0xfb840108,0xf3c00101,0xfb840108,0xf3c10101,
	0xfb840108,0xf3c20101,0xfb840108,0xf3c30101,0xfb840108,0xf3c40101,0xfb840108,0xf3c50101,
	0xfb840108,0xf3c60101,0xfb840108,0xf3c70101,0xfb840108,0xf3c80101,0xfb840108,0xf3c90101,
	0xfb840108,0xf3ca0101,0xfb840108,0xf3cb0101,0xfb840108,0xf3cc0101,0xfb840108,0xf3cd0101,
	0xfb840108,0xf3ce0101,0xfb840108,0xf3cf0101,0xfb840108,0xf3d00101,0xfb840108,0xf3d10101,
	0xfb840108,0xf3d20101,0xfb840108,0xf3d30101,0xfb840108,0xf3d40101,0xfb840108,0xf3d50101,
	0xfb840108,0xf3d60101,0xfb840108,0xf3d70101,0xfb840108,0xf3d80101,0xfb840108,0xf3d90101,
	0xfb840108,0xf3da0101,0xfb840108,0xf3db0101,0xfb840108,0xf3dc0101,0xfb840108,0xf3dd0101,
	0xfb840108,0xf3de0101,0xfb840108,0xf3df0101,0xfb840108,0xf3e00101,0xfb840108,0xf3e10101,
	0xfb840108,0xf3e20101,0xfb840108,0xf3e30101,0xfb840108,0xf3e40101,0xfb840108,0xf3e50101,
	0xfb840108,0xf3e60101,0xfb840108,0xf3e70101,0xfb840108,0xf3e80101,0xfb840108,0xf3e90101,
	0xfb840108,0xf3ea0101,0xfb840108,0xf3eb0101,0xfb840108,0xf3ec0101,0xfb840108,0xf3ed0101,
	0xfb840108,0xf3ee0101,0xfb840108,0xf3ef0101,0xfb840108,0xf3f00101,0xfb840108,0xf3f10101,
	0xfb840108,0xf3f20101,0xfb840108,0xf3f30101,0xfb840108,0xf3f40101,0xfb840108,0xf3f50101,
	0xfb840108,0xf3f60101,0xfb840108,0xf3f70101,0xfb840108,0xf3f80101,0xfb840108,0xf3f90101,
	0xfb840108,0xf3fa0101,0xfb840108,0xf3fb0101,0xfb840108,0xf3fc0101,0xfb840108,0xf3fd0101,
	0xfb840108,0xf3fe0101,0xfb840108,0xf3ff0101,0xfb840108,0xf4000101,0xfb840108,0xf4010101,
	0xfb840108,0xf4020101,0xfb840108,0xf4030101,0xfb840108,0xf4040101,0xfb840108,0xf4050101,
	0xfb840108,0xf4060101,0xfb840108,0xf4070101,0xfb840108,0xf4080101,0xfb840108,0xf4090101,
	0xfb840108,0xf40a0101,0xfb840108,0xf40b0101,0xfb840108,0xf40c0101,0xfb840108,0xf40d0101,
	0xfb840108,0xf40e0101,0xfb840108,0xf40f0101,0xfb840108,0xf4100101,0xfb840108,0xf4110101,
	0xfb840108,0xf4120101,0xfb840108,0xf4130101,0xfb840108,0xf4140101,0xfb840108,0xf4150101,
	0xfb840108,0xf4160101,0xfb840108,0xf4170101,0xfb840108,0xf4180101,0xfb840108,0xf4190101,
	0xfb840108,0xf41a0101,0xfb840108,0xf41b0101,0xfb840108,0xf41c0101,0xfb840108,0xf41d0101,
	0xfb840108,0xf41e0101,0xfb840108,0xf41f0101,0xfb840108,0xf4200101,0xfb840108,0xf4210101,
	0xfb840108,0xf4220101,0xfb840108,0xf4230101,0xfb840108,0xf4240101,0xfb840108,0xf4250101,
	0xfb840108,0xf4260101,0xfb840108,0xf4270101,0xfb840108,0xf4280101,0xfb840108,0xf4290101,
	0xfb840108,0xf42a0101,0xfb840108,0xf42b0101,0xfb840108,0xf42c0101,0xfb840108,0xf42d0101,
	0xfb840108,0xf42e0101,0xfb840108,0xf42f0101,0xfb840108,0xf4300101,0xfb840108,0xf4310101,
	0xfb840108,0xf4320101,0xfb840108,0xf4330101,0xfb840108,0xf4340101,0xfb840108,0xf4350101,
	0xfb840108,0xf4360101,0xfb840108,0xf4370101,0xfb840108,0xf4380101,0xfb840108,0xf4390101,
	0xfb840108,0xf43a0101,0xfb840108,0xf43b0101,0xfb840108,0xf43c0101,0xfb840108,0xf43d0101,
	0xfb840108,0xf43e0101,0xfb840108,0xf43f0101,0xfb840108,0xf4400101,0xfb840108,0xf4410101,
	0xfb840108,0xf4420101,0xfb840108,0xf4430101,0xfb840108,0xf4440101,0xfb840108,0xf4450101,
	0xfb840108,0xf4460101,0xfb840108,0xf4470101,0xfb840108,0xf4480101,0xfb840108,0xf4490101,
	0xfb840108,0xf44a0101,0xfb840108,0xf44b0101,0xfb840108,0xf44c0101,0xfb840108,0xf44d0101,
	0xfb840108,0xf44e0101,0xfb840108,0xf44f0101,0xfb840108,0xf4500101,0xfb840108,0xf4510101,
	0xfb840108,0xf4520101,0xfb840108,0xf4530101,0xfb840108,0xf4540101,0xfb840108,0xf4550101,
	0xfb840108,0xf4560101,0xfb840108,0xf4570101,0xfb840108,0xf4580101,0xfb840108,0xf4590101,
	0xfb840108,0xf45a0101,0xfb840108,0xf45b0101,0xfb840108,0xf45c0101,0xfb840108,0xf45d0101,
	0xfb840108,0xf45e0101,0xfb840108,0xf45f0101,0xfb840108,0xf4600101,0xfb840108,0xf4610101,
	0xfb840108,0xf4620101,0xfb840108,0xf4630101,0xfb840108,0xf4640101,0xfb840108,0xf4650101,
	0xfb840108,0xf4660101,0xfb840108,0xf4670101,0xfb840108,0xf4680101,0xfb840108,0xf4690101,
	0xfb840108,0xf46a0101,0xfb840108,0xf46b0101,0xfb840108,0xf46c0101,0xfb840108,0xf46d0101,
	0xfb840108,0xf46e0101,0xfb840108,0xf46f0101,0xfb840108,0xf4700101,0xfb840108,0xf4710101,
	0xfb840108,0xf4720101,0xfb840108,0xf4730101,0xfb840108,0xf4740101,0xfb840108,0xf4750101,
	0xfb840108,0xf4760101,0xfb840108,0xf4770101,0xfb840108,0xf4780101,0xfb840108,0xf4790101,
	0xfb840108,0xf47a0101,0xfb840108,0xf47b0101,0xfb840108,0xf47c0101,0xfb840108,0xf47d0101,
	0xfb840108,0xf47e0101,0xfb840108,0xf47f0101,0xfb840108,0xf4800101,0xfb840108,0xf4810101,
	0xfb840108,0xf4820101,0xfb840108,0xf4830101,0xfb840108,0xf4840101,0xfb840108,0xf4850101,
	0xfb840108,0xf4860101,0xfb840108,0xf4870101,0xfb840108,0xf4880101,0xfb840108,0xf4890101,
	0xfb840108,0xf48a0101,0xfb840108,0xf48b0101,0xfb840108,0xf48c0101,0xfb840108,0xf48d0101,
	0xfb840108,0xf48e0101,0xfb840108,0xf48f0101,0xfb840108,0xf4900101,0xfb840108,0xf4910101,
	0xfb840108,0xf4920101,0xfb840108,0xf4930101,0xfb840108,0xf4940101,0xfb840108,0xf4950101,
	0xfb840108,0xf4960101,0xfb840108,0xf4970101,0xfb840108,0xf4980101,0xfb840108,0xf4990101,
	0xfb840108,0xf49a0101,0xfb840108,0xf49b0101,0xfb840108,0xf49c0101,0xfb840108,0xf49d0101,
	0xfb840108,0xf49e0101,0xfb840108,0xf49f0101,0xfb840108,0xf4a00101,0xfb840108,0xf4a10101,
	0xfb840108,0xf4a20101,0xfb840108,0xf4a30101,0xfb840108,0xf4a40101,0xfb840108,0xf4a50101,
	0xfb840108,0xf4a60101,0xfb840108,0xf4a70101,0xfb840108,0xf4a80101,0xfb840108,0xf4a90101,
	0xfb840108,0xf4aa0101,0xfb840108,0xf4ab0101,0xfb840108,0xf4ac0101,0xfb840108,0xf4ad0101,
	0xfb840108,0xf4ae0101,0xfb840108,0xf4af0101,0xfb840108,0xf4b00101,0xfb840108,0xf4b10101,
	0xfb840108,0xf4b20101,0xfb840108,0xf4b30101,0xfb840108,0xf4b40101,0xfb840108,0xf4b50101,
	0xfb840108,0xf4b60101,0xfb840108,0xf4b70101,0xfb840108,0xf4b80101,0xfb840108,0xf4b90101,
	0xfb840108,0xf4ba0101,0xfb840108,0xf4bb0101,0xfb840108,0xf4bc0101,0xfb840108,0xf4bd0101,
	0xfb840108,0xf4be0101,0xfb840108,0xf4bf0101,0xfb840108,0xf4c00101,0xfb840108,0xf4c10101,
	0xfb840108,0xf4c20101,0xfb840108,0xf4c30101,0xfb840108,0xf4c40101,0xfb840108,0xf4c50101,
	0xfb840108,0xf4c60101,0xfb840108,0xf4c70101,0xfb840108,0xf4c80101,0xfb840108,0xf4c90101,
	0xfb840108,0xf4ca0101,0xfb840108,0xf4cb0101,0xfb840108,0xf4cc0101,0xfb840108,0xf4cd0101,
	0xfb840108,0xf4ce0101,0xfb840108,0xf4cf0101,0xfb840108,0xf4d00101,0xfb840108,0xf4d10101,
	0xfb840108,0xf4d20101,0xfb840108,0xf4d30101,0xfb840108,0xf4d40101,0xfb840108,0xf4d50101,
	0xfb840108,0xf4d60101,0xfb840108,0xf4d70101,0xfb840108,0xf4d80101,0xfb840108,0xf4d90101,
	0xfb840108,0xf4da0101,0xfb840108,0xf4db0101,0xfb840108,0xf4dc0101,0xfb840108,0xf4dd0101,
	0xfb840108,0xf4de0101,0xfb840108,0xf4df0101,0xfb840108,0xf4e00101,0xfb840108,0xf4e10101,
	0xfb840108,0xf4e20101,0xfb840108,0xf4e30101,0xfb840108,0xf4e40101,0xfb840108,0xf4e50101,
	0xfb840108,0xf4e60101,0xfb840108,0xf4e70101,0xfb840108,0xf4e80101,0xfb840108,0xf4e90101,
	0xfb840108,0xf4ea0101,0xfb840108,0xf4eb0101,0xfb840108,0xf4ec0101,0xfb840108,0xf4ed0101,
	0xfb840108,0xf4ee0101,0xfb840108,0xf4ef0101,0xfb840108,0xf4f00101,0xfb840108,0xf4f10101,
	0xfb840108,0xf4f20101,0xfb840108,0xf4f30101,0xfb840108,0xf4f40101,0xfb840108,0xf4f50101,
	0xfb840108,0xf4f60101,0xfb840108,0xf4f70101,0xfb840108,0xf4f80101,0xfb840108,0xf4f90101,
	0xfb840108,0xf4fa0101,0xfb840108,0xf4fb0101,0xfb840108,0xf4fc0101,0xfb840108,0xf4fd0101,
	0xfb840108,0xf4fe0101,0xfb840108,0xf4ff0101,0xfb840108,0xf5000101,0xfb840108,0xf5010101,
	0xfb840108,0xf5020101,0xfb840108,0xf5030101,0xfb840108,0xf5040101,0xfb840108,0xf5050101,
	0xfb840108,0xf5060101,0xfb840108,0xf5070101,0xfb840108,0xf5080101,0xfb840108,0xf5090101,
	0xfb840108,0xf50a0101,0xfb840108,0xf50b0101,0xfb840108,0xf50c0101,0xfb840108,0xf50d0101,
	0xfb840108,0xf50e0101,0xfb840108,0xf50f0101,0xfb840108,0xf5100101,0xfb840108,0xf5110101,
	0xfb840108,0xf5120101,0xfb840108,0xf5130101,0xfb840108,0xf5140101,0xfb840108,0xf5150101,
	0xfb840108,0xf5160101,0xfb840108,0xf5170101,0xfb840108,0xf5180101,0xfb840108,0xf5190101,
	0xfb840108,0xf51a0101,0xfb840108,0xf51b0101,0xfb840108,0xf51c0101,0xfb840108,0xf51d0101,
	0xfb840108,0xf51e0101,0xfb840108,0xf51f0101,0xfb840108,0xf5200101,0xfb840108,0xf5210101,
	0xfb840108,0xf5220101,0xfb840108,0xf5230101,0xfb840108,0xf5240101,0xfb840108,0xf5250101,
	0xfb840108,0xf5260101,0xfb840108,0xf5270101,0xfb840108,0xf5280101,0xfb840108,0xf5290101,
	0xfb840108,0xf52a0101,0xfb840108,0xf52b0101,0xfb840108,0xf52c0101,0xfb840108,0xf52d0101,
	0xfb840108,0xf52e0101,0xfb840108,0xf52f0101,0xfb840108,0xf5300101,0xfb840108,0xf5310101,
	0xfb840108,0xf5320101,0xfb840108,0xf5330101,0xfb840108,0xf5340101,0xfb840108,0xf5350101,
	0xfb840108,0xf5360101,0xfb840108,0xf5370101,0xfb840108,0xf5380101,0xfb840108,0xf5390101,
	0xfb840108,0xf53a0101,0xfb840108,0xf53b0101,0xfb840108,0xf53c0101,0xfb840108,0xf53d0101,
	0xfb840108,0xf53e0101,0xfb840108,0xf53f0101,0xfb840108,0xf5400101,0xfb840108,0xf5410101,
	0xfb840108,0xf5420101,0xfb840108,0xf5430101,0xfb840108,0xf5440101,0xfb840108,0xf5450101,
	0xfb840108,0xf5460101,0xfb840108,0xf5470101,0xfb840108,0xf5480101,0xfb840108,0xf5490101,
	0xfb840108,0xf54a0101,0xfb840108,0xf54b0101,0xfb840108,0xf54c0101,0xfb840108,0xf54d0101,
	0xfb840108,0xf54e0101,0xfb840108,0xf54f0101,0xfb840108,0xf5500101,0xfb840108,0xf5510101,
	0xfb840108,0xf5520101,0xfb840108,0xf5530101,0xfb840108,0xf5540101,0xfb840108,0xf5550101,
	0xfb840108,0xf5560101,0xfb840108,0xf5570101,0xfb840108,0xf5580101,0xfb840108,0xf5590101,
	0xfb840108,0xf55a0101,0xfb840108,0xf55b0101,0xfb840108,0xf55c0101,0xfb840108,0xf55d0101,
	0xfb840108,0xf55e0101,0xfb840108,0xf55f0101,0xfb840108,0xf5600101,0xfb840108,0xf5610101,
	0xfb840108,0xf5620101,0xfb840108,0xf5630101,0xfb840108,0xf5640101,0xfb840108,0xf5650101,
	0xfb840108,0xf5660101,0xfb840108,0xf5670101,0xfb840108,0xf5680101,0xfb840108,0xf5690101,
	0xfb840108,0xf56a0101,0xfb840108,0xf56b0101,0xfb840108,0xf56c0101,0xfb840108,0xf56d0101,
	0xfb840108,0xf56e0101,0xfb840108,0xf56f0101,0xfb840108,0xf5700101,0xfb840108,0xf5710101,
	0xfb840108,0xf5720101,0xfb840108,0xf5730101,0xfb840108,0xf5740101,0xfb840108,0xf5750101,
	0xfb840108,0xf5760101,0xfb840108,0xf5770101,0xfb840108,0xf5780101,0xfb840108,0xf5790101,
	0xfb840108,0xf57a0101,0xfb840108,0xf57b0101,0xfb840108,0xf57c0101,0xfb840108,0xf57d0101,
	0xfb840108,0xf57e0101,0xfb840108,0xf57f0101,0xfb840108,0xf5800101,0xfb840108,0xf5810101,
	0xfb840108,0xf5820101,0xfb840108,0xf5830101,0xfb840108,0xf5840101,0xfb840108,0xf5850101,
	0xfb840108,0xf5860101,0xfb840108,0xf5870101,0xfb840108,0xf5880101,0xfb840108,0xf5890101,
	0xfb840108,0xf58a0101,0xfb840108,0xf58b0101,0xfb840108,0xf58c0101,0xfb840108,0xf58d0101,
	0xfb840108,0xf58e0101,0xfb840108,0xf58f0101,0xfb840108,0xf5900101,0xfb840108,0xf5910101,
	0xfb840108,0xf5920101,0xfb840108,0xf5930101,0xfb840108,0xf5940101,0xfb840108,0xf5950101,
	0xfb840108,0xf5960101,0xfb840108,0xf5970101,0xfb840108,0xf5980101,0xfb840108,0xf5990101,
	0xfb840108,0xf59a0101,0xfb840108,0xf59b0101,0xfb840108,0xf59c0101,0xfb840108,0xf59d0101,
	0xfb840108,0xf59e0101,0xfb840108,0xf59f0101,0xfb840108,0xf5a00101,0xfb840108,0xf5a10101,
	0xfb840108,0xf5a20101,0xfb840108,0xf5a30101,0xfb840108,0xf5a40101,0xfb840108,0xf5a50101,
	0xfb840108,0xf5a60101,0xfb840108,0xf5a70101,0xfb840108,0xf5a80101,0xfb840108,0xf5a90101,
	0xfb840108,0xf5aa0101,0xfb840108,0xf5ab0101,0xfb840108,0xf5ac0101,0xfb840108,0xf5ad0101,
	0xfb840108,0xf5ae0101,0xfb840108,0xf5af0101,0xfb840108,0xf5b00101,0xfb840108,0xf5b10101,
	0xfb840108,0xf5b20101,0xfb840108,0xf5b30101,0xfb840108,0xf5b40101,0xfb840108,0xf5b50101,
	0xfb840108,0xf5b60101,0xfb840108,0xf5b70101,0xfb840108,0xf5b80101,0xfb840108,0xf5b90101,
	0xfb840108,0xf5ba0101,0xfb840108,0xf5bb0101,0xfb840108,0xf5bc0101,0xfb840108,0xf5bd0101,
	0xfb840108,0xf5be0101,0xfb840108,0xf5bf0101,0xfb840108,0xf5c00101,0xfb840108,0xf5c10101,
	0xfb840108,0xf5c20101,0xfb840108,0xf5c30101,0xfb840108,0xf5c40101,0xfb840108,0xf5c50101,
	0xfb840108,0xf5c60101,0xfb840108,0xf5c70101,0xfb840108,0xf5c80101,0xfb840108,0xf5c90101,
	0xfb840108,0xf5ca0101,0xfb840108,0xf5cb0101,0xfb840108,0xf5cc0101,0xfb840108,0xf5cd0101,
	0xfb840108,0xf5ce0101,0xfb840108,0xf5cf0101,0xfb840108,0xf5d00101,0xfb840108,0xf5d10101,
	0xfb840108,0xf5d20101,0xfb840108,0xf5d30101,0xfb840108,0xf5d40101,0xfb840108,0xf5d50101,
	0xfb840108,0xf5d60101,0xfb840108,0xf5d70101,0xfb840108,0xf5d80101,0xfb840108,0xf5d90101,
	0xfb840108,0xf5da0101,0xfb840108,0xf5db0101,0xfb840108,0xf5dc0101,0xfb840108,0xf5dd0101,
	0xfb840108,0xf5de0101,0xfb840108,0xf5df0101,0xfb840108,0xf5e00101,0xfb840108,0xf5e10101,
	0xfb840108,0xf5e20101,0xfb840108,0xf5e30101,0xfb840108,0xf5e40101,0xfb840108,0xf5e50101,
	0xfb840108,0xf5e60101,0xfb840108,0xf5e70101,0xfb840108,0xf5e80101,0xfb840108,0xf5e90101,
	0xfb840108,0xf5ea0101,0xfb840108,0xf5eb0101,0xfb840108,0xf5ec0101,0xfb840108,0xf5ed0101,
	0xfb840108,0xf5ee0101,0xfb840108,0xf5ef0101,0xfb840108,0xf5f00101,0xfb840108,0xf5f10101,
	0xfb840108,0xf5f20101,0xfb840108,0xf5f30101,0xfb840108,0xf5f40101,0xfb840108,0xf5f50101,
	0xfb840108,0xf5f60101,0xfb840108,0xf5f70101,0xfb840108,0xf5f80101,0xfb840108,0xf5f90101,
	0xfb840108,0xf5fa0101,0xfb840108,0xf5fb0101,0xfb840108,0xf5fc0101,0xfb840108,0xf5fd0101,
	0xfb840108,0xf5fe0101,0xfb840108,0xf5ff0101,0xfb840108,0xf6000101,0xfb840108,0xf6010101,
	0xfb840108,0xf6020101,0xfb840108,0xf6030101,0xfb840108,0xf6040101,0xfb840108,0xf6050101,
	0xfb840108,0xf6060101,0xfb840108,0xf6070101,0xfb840108,0xf6080101,0xfb840108,0xf6090101,
	0xfb840108,0xf60a0101,0xfb840108,0xf60b0101,0xfb840108,0xf60c0101,0xfb840108,0xf60d0101,
	0xfb840108,0xf60e0101,0xfb840108,0xf60f0101,0xfb840108,0xf6100101,0xfb840108,0xf6110101,
	0xfb840108,0xf6120101,0xfb840108,0xf6130101,0xfb840108,0xf6140101,0xfb840108,0xf6150101,
	0xfb840108,0xf6160101,0xfb840108,0xf6170101,0xfb840108,0xf6180101,0xfb840108,0xf6190101,
	0xfb840108,0xf61a0101,0xfb840108,0xf61b0101,0xfb840108,0xf61c0101,0xfb840108,0xf61d0101,
	0xfb840108,0xf61e0101,0xfb840108,0xf61f0101,0xfb840108,0xf6200101,0xfb840108,0xf6210101,
	0xfb840108,0xf6220101,0xfb840108,0xf6230101,0xfb840108,0xf6240101,0xfb840108,0xf6250101,
	0xfb840108,0xf6260101,0xfb840108,0xf6270101,0xfb840108,0xf6280101,0xfb840108,0xf6290101,
	0xfb840108,0xf62a0101,0xfb840108,0xf62b0101,0xfb840108,0xf62c0101,0xfb840108,0xf62d0101,
	0xfb840108,0xf62e0101,0xfb840108,0xf62f0101,0xfb840108,0xf6300101,0xfb840108,0xf6310101,
	0xfb840108,0xf6320101,0xfb840108,0xf6330101,0xfb840108,0xf6340101,0xfb840108,0xf6350101,
	0xfb840108,0xf6360101,0xfb840108,0xf6370101,0xfb840108,0xf6380101,0xfb840108,0xf6390101,
	0xfb840108,0xf63a0101,0xfb840108,0xf63b0101,0xfb840108,0xf63c0101,0xfb840108,0xf63d0101,
	0xfb840108,0xf63e0101,0xfb840108,0xf63f0101,0xfb840108,0xf6400101,0xfb840108,0xf6410101,
	0xfb840108,0xf6420101,0xfb840108,0xf6430101,0xfb840108,0xf6440101,0xfb840108,0xf6450101,
	0xfb840108,0xf6460101,0xfb840108,0xf6470101,0xfb840108,0xf6480101,0xfb840108,0xf6490101,
	0xfb840108,0xf64a0101,0xfb840108,0xf64b0101,0xfb840108,0xf64c0101,0xfb840108,0xf64d0101,
	0xfb840108,0xf64e0101,0xfb840108,0xf64f0101,0xfb840108,0xf6500101,0xfb840108,0xf6510101,
	0xfb840108,0xf6520101,0xfb840108,0xf6530101,0xfb840108,0xf6540101,0xfb840108,0xf6550101,
	0xfb840108,0xf6560101,0xfb840108,0xf6570101,0xfb840108,0xf6580101,0xfb840108,0xf6590101,
	0xfb840108,0xf65a0101,0xfb840108,0xf65b0101,0xfb840108,0xf65c0101,0xfb840108,0xf65d0101,
	0xfb840108,0xf65e0101,0xfb840108,0xf65f0101,0xfb840108,0xf6600101,0xfb840108,0xf6610101,
	0xfb840108,0xf6620101,0xfb840108,0xf6630101,0xfb840108,0xf6640101,0xfb840108,0xf6650101,
	0xfb840108,0xf6660101,0xfb840108,0xf6670101,0xfb840108,0xf6680101,0xfb840108,0xf6690101,
	0xfb840108,0xf66a0101,0xfb840108,0xf66b0101,0xfb840108,0xf66c0101,0xfb840108,0xf66d0101,
	0xfb840108,0xf66e0101,0xfb840108,0xf66f0101,0xfb840108,0xf6700101,0xfb840108,0xf6710101,
	0xfb840108,0xf6720101,0xfb840108,0xf6730101,0xfb840108,0xf6740101,0xfb840108,0xf6750101,
	0xfb840108,0xf6760101,0xfb840108,0xf6770101,0xfb840108,0xf6780101,0xfb840108,0xf6790101,
	0xfb840108,0xf67a0101,0xfb840108,0xf67b0101,0xfb840108,0xf67c0101,0xfb840108,0xf67d0101,
	0xfb840108,0xf67e0101,0xfb840108,0xf67f0101,0xfb840108,0xf6800101,0xfb840108,0xf6810101,
	0xfb840108,0xf6820101,0xfb840108,0xf6830101,0xfb840108,0xf6840101,0xfb840108,0xf6850101,
	0xfb840108,0xf6860101,0xfb840108,0xf6870101,0xfb840108,0xf6880101,0xfb840108,0xf6890101,
	0xfb840108,0xf68a0101,0xfb840108,0xf68b0101,0xfb840108,0xf68c0101,0xfb840108,0xf68d0101,
	0xfb840108,0xf68e0101,0xfb840108,0xf68f0101,0xfb840108,0xf6900101,0xfb840108,0xf6910101,
	0xfb840108,0xf6920101,0xfb840108,0xf6930101,0xfb840108,0xf6940101,0xfb840108,0xf6950101,
	0xfb840108,0xf6960101,0xfb840108,0xf6970101,0xfb840108,0xf6980101,0xfb840108,0xf6990101,
	0xfb840108,0xf69a0101,0xfb840108,0xf69b0101,0xfb840108,0xf69c0101,0xfb840108,0xf69d0101,
	0xfb840108,0xf69e0101,0xfb840108,0xf69f0101,0xfb840108,0xf6a00101,0xfb840108,0xf6a10101,
	0xfb840108,0xf6a20101,0xfb840108,0xf6a30101,0xfb840108,0xf6a40101,0xfb840108,0xf6a50101,
	0xfb840108,0xf6a60101,0xfb840108,0xf6a70101,0xfb840108,0xf6a80101,0xfb840108,0xf6a90101,
	0xfb840108,0xf6aa0101,0xfb840108,0xf6ab0101,0xfb840108,0xf6ac0101,0xfb840108,0xf6ad0101,
	0xfb840108,0xf6ae0101,0xfb840108,0xf6af0101,0xfb840108,0xf6b00101,0xfb840108,0xf6b10101,
	0xfb840108,0xf6b20101,0xfb840108,0xf6b30101,0xfb840108,0xf6b40101,0xfb840108,0xf6b50101,
	0xfb840108,0xf6b60101,0xfb840108,0xf6b70101,0xfb840108,0xf6b80101,0xfb840108,0xf6b90101,
	0xfb840108,0xf6ba0101,0xfb840108,0xf6bb0101,0xfb840108,0xf6bc0101,0xfb840108,0xf6bd0101,
	0xfb840108,0xf6be0101,0xfb840108,0xf6bf0101,0xfb840108,0xf6c00101,0xfb840108,0xf6c10101,
	0xfb840108,0xf6c20101,0xfb840108,0xf6c30101,0xfb840108,0xf6c40101,0xfb840108,0xf6c50101,
	0xfb840108,0xf6c60101,0xfb840108,0xf6c70101,0xfb840108,0xf6c80101,0xfb840108,0xf6c90101,
	0xfb840108,0xf6ca0101,0xfb840108,0xf6cb0101,0xfb840108,0xf6cc0101,0xfb840108,0xf6cd0101,
	0xfb840108,0xf6ce0101,0xfb840108,0xf6cf0101,0xfb840108,0xf6d00101,0xfb840108,0xf6d10101,
	0xfb840108,0xf6d20101,0xfb840108,0xf6d30101,0xfb840108,0xf6d40101,0xfb840108,0xf6d50101,
	0xfb840108,0xf6d60101,0xfb840108,0xf6d70101,0xfb840108,0xf6d80101,0xfb840108,0xf6d90101,
	0xfb840108,0xf6da0101,0xfb840108,0xf6db0101,0xfb840108,0xf6dc0101,0xfb840108,0xf6dd0101,
	0xfb840108,0xf6de0101,0xfb840108,0xf6df0101,0xfb840108,0xf6e00101,0xfb840108,0xf6e10101,
	0xfb840108,0xf6e20101,0xfb840108,0xf6e30101,0xfb840108,0xf6e40101,0xfb840108,0xf6e50101,
	0xfb840108,0xf6e60101,0xfb840108,0xf6e70101,0xfb840108,0xf6e80101,0xfb840108,0xf6e90101,
	0xfb840108,0xf6ea0101,0xfb840108,0xf6eb0101,0xfb840108,0xf6ec0101,0xfb840108,0xf6ed0101,
	0xfb840108,0xf6ee0101,0xfb840108,0xf6ef0101,0xfb840108,0xf6f00101,0xfb840108,0xf6f10101,
	0xfb840108,0xf6f20101,0xfb840108,0xf6f30101,0xfb840108,0xf6f40101,0xfb840108,0xf6f50101,
	0xfb840108,0xf6f60101,0xfb840108,0xf6f70101,0xfb840108,0xf6f80101,0xfb840108,0xf6f90101,
	0xfb840108,0xf6fa0101,0xfb840108,0xf6fb0101,0xfb840108,0xf6fc0101,0xfb840108,0xf6fd0101,
	0xfb840108,0xf6fe0101,0xfb840108,0xf6ff0101,0xfb840108,0xf7000101,0xfb840108,0xf7010101,
	0xfb840108,0xf7020101,0xfb840108,0xf7030101,0xfb840108,0xf7040101,0xfb840108,0xf7050101,
	0xfb840108,0xf7060101,0xfb840108,0xf7070101,0xfb840108,0xf7080101,0xfb840108,0xf7090101,
	0xfb840108,0xf70a0101,0xfb840108,0xf70b0101,0xfb840108,0xf70c0101,0xfb840108,0xf70d0101,
	0xfb840108,0xf70e0101,0xfb840108,0xf70f0101,0xfb840108,0xf7100101,0xfb840108,0xf7110101,
	0xfb840108,0xf7120101,0xfb840108,0xf7130101,0xfb840108,0xf7140101,0xfb840108,0xf7150101,
	0xfb840108,0xf7160101,0xfb840108,0xf7170101,0xfb840108,0xf7180101,0xfb840108,0xf7190101,
	0xfb840108,0xf71a0101,0xfb840108,0xf71b0101,0xfb840108,0xf71c0101,0xfb840108,0xf71d0101,
	0xfb840108,0xf71e0101,0xfb840108,0xf71f0101,0xfb840108,0xf7200101,0xfb840108,0xf7210101,
	0xfb840108,0xf7220101,0xfb840108,0xf7230101,0xfb840108,0xf7240101,0xfb840108,0xf7250101,
	0xfb840108,0xf7260101,0xfb840108,0xf7270101,0xfb840108,0xf7280101,0xfb840108,0xf7290101,
	0xfb840108,0xf72a0101,0xfb840108,0xf72b0101,0xfb840108,0xf72c0101,0xfb840108,0xf72d0101,
	0xfb840108,0xf72e0101,0xfb840108,0xf72f0101,0xfb840108,0xf7300101,0xfb840108,0xf7310101,
	0xfb840108,0xf7320101,0xfb840108,0xf7330101,0xfb840108,0xf7340101,0xfb840108,0xf7350101,
	0xfb840108,0xf7360101,0xfb840108,0xf7370101,0xfb840108,0xf7380101,0xfb840108,0xf7390101,
	0xfb840108,0xf73a0101,0xfb840108,0xf73b0101,0xfb840108,0xf73c0101,0xfb840108,0xf73d0101,
	0xfb840108,0xf73e0101,0xfb840108,0xf73f0101,0xfb840108,0xf7400101,0xfb840108,0xf7410101,
	0xfb840108,0xf7420101,0xfb840108,0xf7430101,0xfb840108,0xf7440101,0xfb840108,0xf7450101,
	0xfb840108,0xf7460101,0xfb840108,0xf7470101,0xfb840108,0xf7480101,0xfb840108,0xf7490101,
	0xfb840108,0xf74a0101,0xfb840108,0xf74b0101,0xfb840108,0xf74c0101,0xfb840108,0xf74d0101,
	0xfb840108,0xf74e0101,0xfb840108,0xf74f0101,0xfb840108,0xf7500101,0xfb840108,0xf7510101,
	0xfb840108,0xf7520101,0xfb840108,0xf7530101,0xfb840108,0xf7540101,0xfb840108,0xf7550101,
	0xfb840108,0xf7560101,0xfb840108,0xf7570101,0xfb840108,0xf7580101,0xfb840108,0xf7590101,
	0xfb840108,0xf75a0101,0xfb840108,0xf75b0101,0xfb840108,0xf75c0101,0xfb840108,0xf75d0101,
	0xfb840108,0xf75e0101,0xfb840108,0xf75f0101,0xfb840108,0xf7600101,0xfb840108,0xf7610101,
	0xfb840108,0xf7620101,0xfb840108,0xf7630101,0xfb840108,0xf7640101,0xfb840108,0xf7650101,
	0xfb840108,0xf7660101,0xfb840108,0xf7670101,0xfb840108,0xf7680101,0xfb840108,0xf7690101,
	0xfb840108,0xf76a0101,0xfb840108,0xf76b0101,0xfb840108,0xf76c0101,0xfb840108,0xf76d0101,
	0xfb840108,0xf76e0101,0xfb840108,0xf76f0101,0xfb840108,0xf7700101,0xfb840108,0xf7710101,
	0xfb840108,0xf7720101,0xfb840108,0xf7730101,0xfb840108,0xf7740101,0xfb840108,0xf7750101,
	0xfb840108,0xf7760101,0xfb840108,0xf7770101,0xfb840108,0xf7780101,0xfb840108,0xf7790101,
	0xfb840108,0xf77a0101,0xfb840108,0xf77b0101,0xfb840108,0xf77c0101,0xfb840108,0xf77d0101,
	0xfb840108,0xf77e0101,0xfb840108,0xf77f0101,0xfb840108,0xf7800101,0xfb840108,0xf7810101,
	0xfb840108,0xf7820101,0xfb840108,0xf7830101,0xfb840108,0xf7840101,0xfb840108,0xf7850101,
	0xfb840108,0xf7860101,0xfb840108,0xf7870101,0xfb840108,0xf7880101,0xfb840108,0xf7890101,
	0xfb840108,0xf78a0101,0xfb840108,0xf78b0101,0xfb840108,0xf78c0101,0xfb840108,0xf78d0101,
	0xfb840108,0xf78e0101,0xfb840108,0xf78f0101,0xfb840108,0xf7900101,0xfb840108,0xf7910101,
	0xfb840108,0xf7920101,0xfb840108,0xf7930101,0xfb840108,0xf7940101,0xfb840108,0xf7950101,
	0xfb840108,0xf7960101,0xfb840108,0xf7970101,0xfb840108,0xf7980101,0xfb840108,0xf7990101,
	0xfb840108,0xf79a0101,0xfb840108,0xf79b0101,0xfb840108,0xf79c0101,0xfb840108,0xf79d0101,
	0xfb840108,0xf79e0101,0xfb840108,0xf79f0101,0xfb840108,0xf7a00101,0xfb840108,0xf7a10101,
	0xfb840108,0xf7a20101,0xfb840108,0xf7a30101,0xfb840108,0xf7a40101,0xfb840108,0xf7a50101,
	0xfb840108,0xf7a60101,0xfb840108,0xf7a70101,0xfb840108,0xf7a80101,0xfb840108,0xf7a90101,
	0xfb840108,0xf7aa0101,0xfb840108,0xf7ab0101,0xfb840108,0xf7ac0101,0xfb840108,0xf7ad0101,
	0xfb840108,0xf7ae0101,0xfb840108,0xf7af0101,0xfb840108,0xf7b00101,0xfb840108,0xf7b10101,
	0xfb840108,0xf7b20101,0xfb840108,0xf7b30101,0xfb840108,0xf7b40101,0xfb840108,0xf7b50101,
	0xfb840108,0xf7b60101,0xfb840108,0xf7b70101,0xfb840108,0xf7b80101,0xfb840108,0xf7b90101,
	0xfb840108,0xf7ba0101,0xfb840108,0xf7bb0101,0xfb840108,0xf7bc0101,0xfb840108,0xf7bd0101,
	0xfb840108,0xf7be0101,0xfb840108,0xf7bf0101,0xfb840108,0xf7c00101,0xfb840108,0xf7c10101,
	0xfb840108,0xf7c20101,0xfb840108,0xf7c30101,0xfb840108,0xf7c40101,0xfb840108,0xf7c50101,
	0xfb840108,0xf7c60101,0xfb840108,0xf7c70101,0xfb840108,0xf7c80101,0xfb840108,0xf7c90101,
	0xfb840108,0xf7ca0101,0xfb840108,0xf7cb0101,0xfb840108,0xf7cc0101,0xfb840108,0xf7cd0101,
	0xfb840108,0xf7ce0101,0xfb840108,0xf7cf0101,0xfb840108,0xf7d00101,0xfb840108,0xf7d10101,
	0xfb840108,0xf7d20101,0xfb840108,0xf7d30101,0xfb840108,0xf7d40101,0xfb840108,0xf7d50101,
	0xfb840108,0xf7d60101,0xfb840108,0xf7d70101,0xfb840108,0xf7d80101,0xfb840108,0xf7d90101,
	0xfb840108,0xf7da0101,0xfb840108,0xf7db0101,0xfb840108,0xf7dc0101,0xfb840108,0xf7dd0101,
	0xfb840108,0xf7de0101,0xfb840108,0xf7df0101,0xfb840108,0xf7e00101,0xfb840108,0xf7e10101,
	0xfb840108,0xf7e20101,0xfb840108,0xf7e30101,0xfb840108,0xf7e40101,0xfb840108,0xf7e50101,
	0xfb840108,0xf7e60101,0xfb840108,0xf7e70101,0xfb840108,0xf7e80101,0xfb840108,0xf7e90101,
	0xfb840108,0xf7ea0101,0xfb840108,0xf7eb0101,0xfb840108,0xf7ec0101,0xfb840108,0xf7ed0101,
	0xfb840108,0xf7ee0101,0xfb840108,0xf7ef0101,0xfb840108,0xf7f00101,0xfb840108,0xf7f10101,
	0xfb840108,0xf7f20101,0xfb840108,0xf7f30101,0xfb840108,0xf7f40101,0xfb840108,0xf7f50101,
	0xfb840108,0xf7f60101,0xfb840108,0xf7f70101,0xfb840108,0xf7f80101,0xfb840108,0xf7f90101,
	0xfb840108,0xf7fa0101,0xfb840108,0xf7fb0101,0xfb840108,0xf7fc0101,0xfb840108,0xf7fd0101,
	0xfb840108,0xf7fe0101,0xfb840108,0xf7ff0101,0xfb840108,0xf8000101,0xfb840108,0xf8010101,
	0xfb840108,0xf8020101,0xfb840108,0xf8030101,0xfb840108,0xf8040101,0xfb840108,0xf8050101,
	0xfb840108,0xf8060101,0xfb840108,0xf8070101,0xfb840108,0xf8080101,0xfb840108,0xf8090101,
	0xfb840108,0xf80a0101,0xfb840108,0xf80b0101,0xfb840108,0xf80c0101,0xfb840108,0xf80d0101,
	0xfb840108,0xf80e0101,0xfb840108,0xf80f0101,0xfb840108,0xf8100101,0xfb840108,0xf8110101,
	0xfb840108,0xf8120101,0xfb840108,0xf8130101,0xfb840108,0xf8140101,0xfb840108,0xf8150101,
	0xfb840108,0xf8160101,0xfb840108,0xf8170101,0xfb840108,0xf8180101,0xfb840108,0xf8190101,
	0xfb840108,0xf81a0101,0xfb840108,0xf81b0101,0xfb840108,0xf81c0101,0xfb840108,0xf81d0101,
	0xfb840108,0xf81e0101,0xfb840108,0xf81f0101,0xfb840108,0xf8200101,0xfb840108,0xf8210101,
	0xfb840108,0xf8220101,0xfb840108,0xf8230101,0xfb840108,0xf8240101,0xfb840108,0xf8250101,
	0xfb840108,0xf8260101,0xfb840108,0xf8270101,0xfb840108,0xf8280101,0xfb840108,0xf8290101,
	0xfb840108,0xf82a0101,0xfb840108,0xf82b0101,0xfb840108,0xf82c0101,0xfb840108,0xf82d0101,
	0xfb840108,0xf82e0101,0xfb840108,0xf82f0101,0xfb840108,0xf8300101,0xfb840108,0xf8310101,
	0xfb840108,0xf8320101,0xfb840108,0xf8330101,0xfb840108,0xf8340101,0xfb840108,0xf8350101,
	0xfb840108,0xf8360101,0xfb840108,0xf8370101,0xfb840108,0xf8380101,0xfb840108,0xf8390101,
	0xfb840108,0xf83a0101,0xfb840108,0xf83b0101,0xfb840108,0xf83c0101,0xfb840108,0xf83d0101,
	0xfb840108,0xf83e0101,0xfb840108,0xf83f0101,0xfb840108,0xf8400101,0xfb840108,0xf8410101,
	0xfb840108,0xf8420101,0xfb840108,0xf8430101,0xfb840108,0xf8440101,0xfb840108,0xf8450101,
	0xfb840108,0xf8460101,0xfb840108,0xf8470101,0xfb840108,0xf8480101,0xfb840108,0xf8490101,
	0xfb840108,0xf84a0101,0xfb840108,0xf84b0101,0xfb840108,0xf84c0101,0xfb840108,0xf84d0101,
	0xfb840108,0xf84e0101,0xfb840108,0xf84f0101,0xfb840108,0xf8500101,0xfb840108,0xf8510101,
	0xfb840108,0xf8520101,0xfb840108,0xf8530101,0xfb840108,0xf8540101,0xfb840108,0xf8550101,
	0xfb840108,0xf8560101,0xfb840108,0xf8570101,0xfb840108,0xf8580101,0xfb840108,0xf8590101,
	0xfb840108,0xf85a0101,0xfb840108,0xf85b0101,0xfb840108,0xf85c0101,0xfb840108,0xf85d0101,
	0xfb840108,0xf85e0101,0xfb840108,0xf85f0101,0xfb840108,0xf8600101,0xfb840108,0xf8610101,
	0xfb840108,0xf8620101,0xfb840108,0xf8630101,0xfb840108,0xf8640101,0xfb840108,0xf8650101,
	0xfb840108,0xf8660101,0xfb840108,0xf8670101,0xfb840108,0xf8680101,0xfb840108,0xf8690101,
	0xfb840108,0xf86a0101,0xfb840108,0xf86b0101,0xfb840108,0xf86c0101,0xfb840108,0xf86d0101,
	0xfb840108,0xf86e0101,0xfb840108,0xf86f0101,0xfb840108,0xf8700101,0xfb840108,0xf8710101,
	0xfb840108,0xf8720101,0xfb840108,0xf8730101,0xfb840108,0xf8740101,0xfb840108,0xf8750101,
	0xfb840108,0xf8760101,0xfb840108,0xf8770101,0xfb840108,0xf8780101,0xfb840108,0xf8790101,
	0xfb840108,0xf87a0101,0xfb840108,0xf87b0101,0xfb840108,0xf87c0101,0xfb840108,0xf87d0101,
	0xfb840108,0xf87e0101,0xfb840108,0xf87f0101,0xfb840108,0xf8800101,0xfb840108,0xf8810101,
	0xfb840108,0xf8820101,0xfb840108,0xf8830101,0xfb840108,0xf8840101,0xfb840108,0xf8850101,
	0xfb840108,0xf8860101,0xfb840108,0xf8870101,0xfb840108,0xf8880101,0xfb840108,0xf8890101,
	0xfb840108,0xf88a0101,0xfb840108,0xf88b0101,0xfb840108,0xf88c0101,0xfb840108,0xf88d0101,
	0xfb840108,0xf88e0101,0xfb840108,0xf88f0101,0xfb840108,0xf8900101,0xfb840108,0xf8910101,
	0xfb840108,0xf8920101,0xfb840108,0xf8930101,0xfb840108,0xf8940101,0xfb840108,0xf8950101,
	0xfb840108,0xf8960101,0xfb840108,0xf8970101,0xfb840108,0xf8980101,0xfb840108,0xf8990101,
	0xfb840108,0xf89a0101,0xfb840108,0xf89b0101,0xfb840108,0xf89c0101,0xfb840108,0xf89d0101,
	0xfb840108,0xf89e0101,0xfb840108,0xf89f0101,0xfb840108,0xf8a00101,0xfb840108,0xf8a10101,
	0xfb840108,0xf8a20101,0xfb840108,0xf8a30101,0xfb840108,0xf8a40101,0xfb840108,0xf8a50101,
	0xfb840108,0xf8a60101,0xfb840108,0xf8a70101,0xfb840108,0xf8a80101,0xfb840108,0xf8a90101,
	0xfb840108,0xf8aa0101,0xfb840108,0xf8ab0101,0xfb840108,0xf8ac0101,0xfb840108,0xf8ad0101,
	0xfb840108,0xf8ae0101,0xfb840108,0xf8af0101,0xfb840108,0xf8b00101,0xfb840108,0xf8b10101,
	0xfb840108,0xf8b20101,0xfb840108,0xf8b30101,0xfb840108,0xf8b40101,0xfb840108,0xf8b50101,
	0xfb840108,0xf8b60101,0xfb840108,0xf8b70101,0xfb840108,0xf8b80101,0xfb840108,0xf8b90101,
	0xfb840108,0xf8ba0101,0xfb840108,0xf8bb0101,0xfb840108,0xf8bc0101,0xfb840108,0xf8bd0101,
	0xfb840108,0xf8be0101,0xfb840108,0xf8bf0101,0xfb840108,0xf8c00101,0xfb840108,0xf8c10101,
	0xfb840108,0xf8c20101,0xfb840108,0xf8c30101,0xfb840108,0xf8c40101,0xfb840108,0xf8c50101,
	0xfb840108,0xf8c60101,0xfb840108,0xf8c70101,0xfb840108,0xf8c80101,0xfb840108,0xf8c90101,
	0xfb840108,0xf8ca0101,0xfb840108,0xf8cb0101,0xfb840108,0xf8cc0101,0xfb840108,0xf8cd0101,
	0xfb840108,0xf8ce0101,0xfb840108,0xf8cf0101,0xfb840108,0xf8d00101,0xfb840108,0xf8d10101,
	0xfb840108,0xf8d20101,0xfb840108,0xf8d30101,0xfb840108,0xf8d40101,0xfb840108,0xf8d50101,
	0xfb840108,0xf8d60101,0xfb840108,0xf8d70101,0xfb840108,0xf8d80101,0xfb840108,0xf8d90101,
	0xfb840108,0xf8da0101,0xfb840108,0xf8db0101,0xfb840108,0xf8dc0101,0xfb840108,0xf8dd0101,
	0xfb840108,0xf8de0101,0xfb840108,0xf8df0101,0xfb840108,0xf8e00101,0xfb840108,0xf8e10101,
	0xfb840108,0xf8e20101,0xfb840108,0xf8e30101,0xfb840108,0xf8e40101,0xfb840108,0xf8e50101,
	0xfb840108,0xf8e60101,0xfb840108,0xf8e70101,0xfb840108,0xf8e80101,0xfb840108,0xf8e90101,
	0xfb840108,0xf8ea0101,0xfb840108,0xf8eb0101,0xfb840108,0xf8ec0101,0xfb840108,0xf8ed0101,
	0xfb840108,0xf8ee0101,0xfb840108,0xf8ef0101,0xfb840108,0xf8f00101,0xfb840108,0xf8f10101,
	0xfb840108,0xf8f20101,0xfb840108,0xf8f30101,0xfb840108,0xf8f40101,0xfb840108,0xf8f50101,
	0xfb840108,0xf8f60101,0xfb840108,0xf8f70101,0xfb840108,0xf8f80101,0xfb840108,0xf8f90101,
	0xfb840108,0xf8fa0101,0xfb840108,0xf8fb0101,0xfb840108,0xf8fc0101,0xfb840108,0xf8fd0101,
	0xfb840108,0xf8fe0101,0xfb840108,0xf8ff0101,0xfb840108,0xf9000101,0xfb840108,0xf9010101,
	0xfb840108,0xf9020101,0xfb840108,0xf9030101,0xfb840108,0xf9040101,0xfb840108,0xf9050101,
	0xfb840108,0xf9060101,0xfb840108,0xf9070101,0xfb840108,0xf9080101,0xfb840108,0xf9090101,
	0xfb840108,0xf90a0101,0xfb840108,0xf90b0101,0xfb840108,0xf90c0101,0xfb840108,0xf90d0101,
	0xfb840108,0xf90e0101,0xfb840108,0xf90f0101,0xfb840108,0xf9100101,0xfb840108,0xf9110101,
	0xfb840108,0xf9120101,0xfb840108,0xf9130101,0xfb840108,0xf9140101,0xfb840108,0xf9150101,
	0xfb840108,0xf9160101,0xfb840108,0xf9170101,0xfb840108,0xf9180101,0xfb840108,0xf9190101,
	0xfb840108,0xf91a0101,0xfb840108,0xf91b0101,0xfb840108,0xf91c0101,0xfb840108,0xf91d0101,
	0xfb840108,0xf91e0101,0xfb840108,0xf91f0101,0xfb840108,0xf9200101,0xfb840108,0xf9210101,
	0xfb840108,0xf9220101,0xfb840108,0xf9230101,0xfb840108,0xf9240101,0xfb840108,0xf9250101,
	0xfb840108,0xf9260101,0xfb840108,0xf9270101,0xfb840108,0xf9280101,0xfb840108,0xf9290101,
	0xfb840108,0xf92a0101,0xfb840108,0xf92b0101,0xfb840108,0xf92c0101,0xfb840108,0xf92d0101,
	0xfb840108,0xf92e0101,0xfb840108,0xf92f0101,0xfb840108,0xf9300101,0xfb840108,0xf9310101,
	0xfb840108,0xf9320101,0xfb840108,0xf9330101,0xfb840108,0xf9340101,0xfb840108,0xf9350101,
	0xfb840108,0xf9360101,0xfb840108,0xf9370101,0xfb840108,0xf9380101,0xfb840108,0xf9390101,
	0xfb840108,0xf93a0101,0xfb840108,0xf93b0101,0xfb840108,0xf93c0101,0xfb840108,0xf93d0101,
	0xfb840108,0xf93e0101,0xfb840108,0xf93f0101,0xfb840108,0xf9400101,0xfb840108,0xf9410101,
	0xfb840108,0xf9420101,0xfb840108,0xf9430101,0xfb840108,0xf9440101,0xfb840108,0xf9450101,
	0xfb840108,0xf9460101,0xfb840108,0xf9470101,0xfb840108,0xf9480101,0xfb840108,0xf9490101,
	0xfb840108,0xf94a0101,0xfb840108,0xf94b0101,0xfb840108,0xf94c0101,0xfb840108,0xf94d0101,
	0xfb840108,0xf94e0101,0xfb840108,0xf94f0101,0xfb840108,0xf9500101,0xfb840108,0xf9510101,
	0xfb840108,0xf9520101,0xfb840108,0xf9530101,0xfb840108,0xf9540101,0xfb840108,0xf9550101,
	0xfb840108,0xf9560101,0xfb840108,0xf9570101,0xfb840108,0xf9580101,0xfb840108,0xf9590101,
	0xfb840108,0xf95a0101,0xfb840108,0xf95b0101,0xfb840108,0xf95c0101,0xfb840108,0xf95d0101,
	0xfb840108,0xf95e0101,0xfb840108,0xf95f0101,0xfb840108,0xf9600101,0xfb840108,0xf9610101,
	0xfb840108,0xf9620101,0xfb840108,0xf9630101,0xfb840108,0xf9640101,0xfb840108,0xf9650101,
	0xfb840108,0xf9660101,0xfb840108,0xf9670101,0xfb840108,0xf9680101,0xfb840108,0xf9690101,
	0xfb840108,0xf96a0101,0xfb840108,0xf96b0101,0xfb840108,0xf96c0101,0xfb840108,0xf96d0101,
	0xfb840108,0xf96e0101,0xfb840108,0xf96f0101,0xfb840108,0xf9700101,0xfb840108,0xf9710101,
	0xfb840108,0xf9720101,0xfb840108,0xf9730101,0xfb840108,0xf9740101,0xfb840108,0xf9750101,
	0xfb840108,0xf9760101,0xfb840108,0xf9770101,0xfb840108,0xf9780101,0xfb840108,0xf9790101,
	0xfb840108,0xf97a0101,0xfb840108,0xf97b0101,0xfb840108,0xf97c0101,0xfb840108,0xf97d0101,
	0xfb840108,0xf97e0101,0xfb840108,0xf97f0101,0xfb840108,0xf9800101,0xfb840108,0xf9810101,
	0xfb840108,0xf9820101,0xfb840108,0xf9830101,0xfb840108,0xf9840101,0xfb840108,0xf9850101,
	0xfb840108,0xf9860101,0xfb840108,0xf9870101,0xfb840108,0xf9880101,0xfb840108,0xf9890101,
	0xfb840108,0xf98a0101,0xfb840108,0xf98b0101,0xfb840108,0xf98c0101,0xfb840108,0xf98d0101,
	0xfb840108,0xf98e0101,0xfb840108,0xf98f0101,0xfb840108,0xf9900101,0xfb840108,0xf9910101,
	0xfb840108,0xf9920101,0xfb840108,0xf9930101,0xfb840108,0xf9940101,0xfb840108,0xf9950101,
	0xfb840108,0xf9960101,0xfb840108,0xf9970101,0xfb840108,0xf9980101,0xfb840108,0xf9990101,
	0xfb840108,0xf99a0101,0xfb840108,0xf99b0101,0xfb840108,0xf99c0101,0xfb840108,0xf99d0101,
	0xfb840108,0xf99e0101,0xfb840108,0xf99f0101,0xfb840108,0xf9a00101,0xfb840108,0xf9a10101,
	0xfb840108,0xf9a20101,0xfb840108,0xf9a30101,0xfb840108,0xf9a40101,0xfb840108,0xf9a50101,
	0xfb840108,0xf9a60101,0xfb840108,0xf9a70101,0xfb840108,0xf9a80101,0xfb840108,0xf9a90101,
	0xfb840108,0xf9aa0101,0xfb840108,0xf9ab0101,0xfb840108,0xf9ac0101,0xfb840108,0xf9ad0101,
	0xfb840108,0xf9ae0101,0xfb840108,0xf9af0101,0xfb840108,0xf9b00101,0xfb840108,0xf9b10101,
	0xfb840108,0xf9b20101,0xfb840108,0xf9b30101,0xfb840108,0xf9b40101,0xfb840108,0xf9b50101,
	0xfb840108,0xf9b60101,0xfb840108,0xf9b70101,0xfb840108,0xf9b80101,0xfb840108,0xf9b90101,
	0xfb840108,0xf9ba0101,0xfb840108,0xf9bb0101,0xfb840108,0xf9bc0101,0xfb840108,0xf9bd0101,
	0xfb840108,0xf9be0101,0xfb840108,0xf9bf0101,0xfb840108,0xf9c00101,0xfb840108,0xf9c10101,
	0xfb840108,0xf9c20101,0xfb840108,0xf9c30101,0xfb840108,0xf9c40101,0xfb840108,0xf9c50101,
	0xfb840108,0xf9c60101,0xfb840108,0xf9c70101,0xfb840108,0xf9c80101,0xfb840108,0xf9c90101,
	0xfb840108,0xf9ca0101,0xfb840108,0xf9cb0101,0xfb840108,0xf9cc0101,0xfb840108,0xf9cd0101,
	0xfb840108,0xf9ce0101,0xfb840108,0xf9cf0101,0xfb840108,0xf9d00101,0xfb840108,0xf9d10101,
	0xfb840108,0xf9d20101,0xfb840108,0xf9d30101,0xfb840108,0xf9d40101,0xfb840108,0xf9d50101,
	0xfb840108,0xf9d60101,0xfb840108,0xf9d70101,0xfb840108,0xf9d80101,0xfb840108,0xf9d90101,
	0xfb840108,0xf9da0101,0xfb840108,0xf9db0101,0xfb840108,0xf9dc0101,0xfb840108,0xf9dd0101,
	0xfb840108,0xf9de0101,0xfb840108,0xf9df0101,0xfb840108,0xf9e00101,0xfb840108,0xf9e10101,
	0xfb840108,0xf9e20101,0xfb840108,0xf9e30101,0xfb840108,0xf9e40101,0xfb840108,0xf9e50101,
	0xfb840108,0xf9e60101,0xfb840108,0xf9e70101,0xfb840108,0xf9e80101,0xfb840108,0xf9e90101,
	0xfb840108,0xf9ea0101,0xfb840108,0xf9eb0101,0xfb840108,0xf9ec0101,0xfb840108,0xf9ed0101,
	0xfb840108,0xf9ee0101,0xfb840108,0xf9ef0101,0xfb840108,0xf9f00101,0xfb840108,0xf9f10101,
	0xfb840108,0xf9f20101,0xfb840108,0xf9f30101,0xfb840108,0xf9f40101,0xfb840108,0xf9f50101,
	0xfb840108,0xf9f60101,0xfb840108,0xf9f70101,0xfb840108,0xf9f80101,0xfb840108,0xf9f90101,
	0xfb840108,0xf9fa0101,0xfb840108,0xf9fb0101,0xfb840108,0xf9fc0101,0xfb840108,0xf9fd0101,
	0xfb840108,0xf9fe0101,0xfb840108,0xf9ff0101,0xfb840108,0xfa000101,0xfb840108,0xfa010101,
	0xfb840108,0xfa020101,0xfb840108,0xfa030101,0xfb840108,0xfa040101,0xfb840108,0xfa050101,
	0xfb840108,0xfa060101,0xfb840108,0xfa070101,0xfb840108,0xfa080101,0xfb840108,0xfa090101,
	0xfb840108,0xfa0a0101,0xfb840108,0xfa0b0101,0xfb840108,0xfa0c0101,0xfb840108,0xfa0d0101,
	0xfb840108,0xfa0e0101,0xfb840108,0xfa0f0101,0xfb840108,0xfa100101,0xfb840108,0xfa110101,
	0xfb840108,0xfa120101,0xfb840108,0xfa130101,0xfb840108,0xfa140101,0xfb840108,0xfa150101,
	0xfb840108,0xfa160101,0xfb840108,0xfa170101,0xfb840108,0xfa180101,0xfb840108,0xfa190101,
	0xfb840108,0xfa1a0101,0xfb840108,0xfa1b0101,0xfb840108,0xfa1c0101,0xfb840108,0xfa1d0101,
	0xfb840108,0xfa1e0101,0xfb840108,0xfa1f0101,0xfb840108,0xfa200101,0xfb840108,0xfa210101,
	0xfb840108,0xfa220101,0xfb840108,0xfa230101,0xfb840108,0xfa240101,0xfb840108,0xfa250101,
	0xfb840108,0xfa260101,0xfb840108,0xfa270101,0xfb840108,0xfa280101,0xfb840108,0xfa290101,
	0xfb840108,0xfa2a0101,0xfb840108,0xfa2b0101,0xfb840108,0xfa2c0101,0xfb840108,0xfa2d0101,
	0xfb840108,0xfa2e0101,0xfb840108,0xfa2f0101,0xfb840108,0xfa300101,0xfb840108,0xfa310101,
	0xfb840108,0xfa320101,0xfb840108,0xfa330101,0xfb840108,0xfa340101,0xfb840108,0xfa350101,
	0xfb840108,0xfa360101,0xfb840108,0xfa370101,0xfb840108,0xfa380101,0xfb840108,0xfa390101,
	0xfb840108,0xfa3a0101,0xfb840108,0xfa3b0101,0xfb840108,0xfa3c0101,0xfb840108,0xfa3d0101,
	0xfb840108,0xfa3e0101,0xfb840108,0xfa3f0101,0xfb840108,0xfa400101,0xfb840108,0xfa410101,
	0xfb840108,0xfa420101,0xfb840108,0xfa430101,0xfb840108,0xfa440101,0xfb840108,0xfa450101,
	0xfb840108,0xfa460101,0xfb840108,0xfa470101,0xfb840108,0xfa480101,0xfb840108,0xfa490101,
	0xfb840108,0xfa4a0101,0xfb840108,0xfa4b0101,0xfb840108,0xfa4c0101,0xfb840108,0xfa4d0101,
	0xfb840108,0xfa4e0101,0xfb840108,0xfa4f0101,0xfb840108,0xfa500101,0xfb840108,0xfa510101,
	0xfb840108,0xfa520101,0xfb840108,0xfa530101,0xfb840108,0xfa540101,0xfb840108,0xfa550101,
	0xfb840108,0xfa560101,0xfb840108,0xfa570101,0xfb840108,0xfa580101,0xfb840108,0xfa590101,
	0xfb840108,0xfa5a0101,0xfb840108,0xfa5b0101,0xfb840108,0xfa5c0101,0xfb840108,0xfa5d0101,
	0xfb840108,0xfa5e0101,0xfb840108,0xfa5f0101,0xfb840108,0xfa600101,0xfb840108,0xfa610101,
	0xfb840108,0xfa620101,0xfb840108,0xfa630101,0xfb840108,0xfa640101,0xfb840108,0xfa650101,
	0xfb840108,0xfa660101,0xfb840108,0xfa670101,0xfb840108,0xfa680101,0xfb840108,0xfa690101,
	0xfb840108,0xfa6a0101,0xfb840108,0xfa6b0101,0xfb840108,0xfa6c0101,0xfb840108,0xfa6d0101,
	0xfb840108,0xfa6e0101,0xfb840108,0xfa6f0101,0xfb840108,0xfa700101,0xfb840108,0xfa710101,
	0xfb840108,0xfa720101,0xfb840108,0xfa730101,0xfb840108,0xfa740101,0xfb840108,0xfa750101,
	0xfb840108,0xfa760101,0xfb840108,0xfa770101,0xfb840108,0xfa780101,0xfb840108,0xfa790101,
	0xfb840108,0xfa7a0101,0xfb840108,0xfa7b0101,0xfb840108,0xfa7c0101,0xfb840108,0xfa7d0101,
	0xfb840108,0xfa7e0101,0xfb840108,0xfa7f0101,0xfb840108,0xfa800101,0xfb840108,0xfa810101,
	0xfb840108,0xfa820101,0xfb840108,0xfa830101,0xfb840108,0xfa840101,0xfb840108,0xfa850101,
	0xfb840108,0xfa860101,0xfb840108,0xfa870101,0xfb840108,0xfa880101,0xfb840108,0xfa890101,
	0xfb840108,0xfa8a0101,0xfb840108,0xfa8b0101,0xfb840108,0xfa8c0101,0xfb840108,0xfa8d0101,
	0xfb840108,0xfa8e0101,0xfb840108,0xfa8f0101,0xfb840108,0xfa900101,0xfb840108,0xfa910101,
	0xfb840108,0xfa920101,0xfb840108,0xfa930101,0xfb840108,0xfa940101,0xfb840108,0xfa950101,
	0xfb840108,0xfa960101,0xfb840108,0xfa970101,0xfb840108,0xfa980101,0xfb840108,0xfa990101,
	0xfb840108,0xfa9a0101,0xfb840108,0xfa9b0101,0xfb840108,0xfa9c0101,0xfb840108,0xfa9d0101,
	0xfb840108,0xfa9e0101,0xfb840108,0xfa9f0101,0xfb840108,0xfaa00101,0xfb840108,0xfaa10101,
	0xfb840108,0xfaa20101,0xfb840108,0xfaa30101,0xfb840108,0xfaa40101,0xfb840108,0xfaa50101,
	0xfb840108,0xfaa60101,0xfb840108,0xfaa70101,0xfb840108,0xfaa80101,0xfb840108,0xfaa90101,
	0xfb840108,0xfaaa0101,0xfb840108,0xfaab0101,0xfb840108,0xfaac0101,0xfb840108,0xfaad0101,
	0xfb840108,0xfaae0101,0xfb840108,0xfaaf0101,0xfb840108,0xfab00101,0xfb840108,0xfab10101,
	0xfb840108,0xfab20101,0xfb840108,0xfab30101,0xfb840108,0xfab40101,0xfb840108,0xfab50101,
	0xfb840108,0xfab60101,0xfb840108,0xfab70101,0xfb840108,0xfab80101,0xfb840108,0xfab90101,
	0xfb840108,0xfaba0101,0xfb840108,0xfabb0101,0xfb840108,0xfabc0101,0xfb840108,0xfabd0101,
	0xfb840108,0xfabe0101,0xfb840108,0xfabf0101,0xfb840108,0xfac00101,0xfb840108,0xfac10101,
	0xfb840108,0xfac20101,0xfb840108,0xfac30101,0xfb840108,0xfac40101,0xfb840108,0xfac50101,
	0xfb840108,0xfac60101,0xfb840108,0xfac70101,0xfb840108,0xfac80101,0xfb840108,0xfac90101,
	0xfb840108,0xfaca0101,0xfb840108,0xfacb0101,0xfb840108,0xfacc0101,0xfb840108,0xfacd0101,
	0xfb840108,0xface0101,0xfb840108,0xfacf0101,0xfb840108,0xfad00101,0xfb840108,0xfad10101,
	0xfb840108,0xfad20101,0xfb840108,0xfad30101,0xfb840108,0xfad40101,0xfb840108,0xfad50101,
	0xfb840108,0xfad60101,0xfb840108,0xfad70101,0xfb840108,0xfad80101,0xfb840108,0xfad90101,
	0xfb840108,0xfada0101,0xfb840108,0xfadb0101,0xfb840108,0xfadc0101,0xfb840108,0xfadd0101,
	0xfb840108,0xfade0101,0xfb840108,0xfadf0101,0xfb840108,0xfae00101,0xfb840108,0xfae10101,
	0xfb840108,0xfae20101,0xfb840108,0xfae30101,0xfb840108,0xfae40101,0xfb840108,0xfae50101,
	0xfb840108,0xfae60101,0xfb840108,0xfae70101,0xfb840108,0xfae80101,0xfb840108,0xfae90101,
	0xfb840108,0xfaea0101,0xfb840108,0xfaeb0101,0xfb840108,0xfaec0101,0xfb840108,0xfaed0101,
	0xfb840108,0xfaee0101,0xfb840108,0xfaef0101,0xfb840108,0xfaf00101,0xfb840108,0xfaf10101,
	0xfb840108,0xfaf20101,0xfb840108,0xfaf30101,0xfb840108,0xfaf40101,0xfb840108,0xfaf50101,
	0xfb840108,0xfaf60101,0xfb840108,0xfaf70101,0xfb840108,0xfaf80101,0xfb840108,0xfaf90101,
	0xfb840108,0xfafa0101,0xfb840108,0xfafb0101,0xfb840108,0xfafc0101,0xfb840108,0xfafd0101,
	0xfb840108,0xfafe0101,0xfb840108,0xfaff0101,0xfb840108,0xfb000101,0xfb840108,0xfb010101,
	0xfb840108,0xfb020101,0xfb840108,0xfb030101,0xfb840108,0xfb040101,0xfb840108,0xfb050101,
	0xfb840108,0xfb060101,0xfb840108,0xfb070101,0xfb840108,0xfb080101,0xfb840108,0xfb090101,
	0xfb840108,0xfb0a0101,0xfb840108,0xfb0b0101,0xfb840108,0xfb0c0101,0xfb840108,0xfb0d0101,
	0xfb840108,0xfb0e0101,0xfb840108,0xfb0f0101,0xfb840108,0xfb100101,0xfb840108,0xfb110101,
	0xfb840108,0xfb120101,0xfb840108,0xfb130101,0xfb840108,0xfb140101,0xfb840108,0xfb150101,
	0xfb840108,0xfb160101,0xfb840108,0xfb170101,0xfb840108,0xfb180101,0xfb840108,0xfb190101,
	0xfb840108,0xfb1a0101,0xfb840108,0xfb1b0101,0xfb840108,0xfb1c0101,0xfb840108,0xfb1d0101,
	0xfb840108,0xfb1e0101,0xfb840108,0xfb1f0101,0xfb840108,0xfb200101,0xfb840108,0xfb210101,
	0xfb840108,0xfb220101,0xfb840108,0xfb230101,0xfb840108,0xfb240101,0xfb840108,0xfb250101,
	0xfb840108,0xfb260101,0xfb840108,0xfb270101,0xfb840108,0xfb280101,0xfb840108,0xfb290101,
	0xfb840108,0xfb2a0101,0xfb840108,0xfb2b0101,0xfb840108,0xfb2c0101,0xfb840108,0xfb2d0101,
	0xfb840108,0xfb2e0101,0xfb840108,0xfb2f0101,0xfb840108,0xfb300101,0xfb840108,0xfb310101,
	0xfb840108,0xfb320101,0xfb840108,0xfb330101,0xfb840108,0xfb340101,0xfb840108,0xfb350101,
	0xfb840108,0xfb360101,0xfb840108,0xfb370101,0xfb840108,0xfb380101,0xfb840108,0xfb390101,
	0xfb840108,0xfb3a0101,0xfb840108,0xfb3b0101,0xfb840108,0xfb3c0101,0xfb840108,0xfb3d0101,
	0xfb840108,0xfb3e0101,0xfb840108,0xfb3f0101,0xfb840108,0xfb400101,0xfb840108,0xfb410101,
	0xfb840108,0xfb420101,0xfb840108,0xfb430101,0xfb840108,0xfb440101,0xfb840108,0xfb450101,
	0xfb840108,0xfb460101,0xfb840108,0xfb470101,0xfb840108,0xfb480101,0xfb840108,0xfb490101,
	0xfb840108,0xfb4a0101,0xfb840108,0xfb4b0101,0xfb840108,0xfb4c0101,0xfb840108,0xfb4d0101,
	0xfb840108,0xfb4e0101,0xfb840108,0xfb4f0101,0xfb840108,0xfb500101,0xfb840108,0xfb510101,
	0xfb840108,0xfb520101,0xfb840108,0xfb530101,0xfb840108,0xfb540101,0xfb840108,0xfb550101,
	0xfb840108,0xfb560101,0xfb840108,0xfb570101,0xfb840108,0xfb580101,0xfb840108,0xfb590101,
	0xfb840108,0xfb5a0101,0xfb840108,0xfb5b0101,0xfb840108,0xfb5c0101,0xfb840108,0xfb5d0101,
	0xfb840108,0xfb5e0101,0xfb840108,0xfb5f0101,0xfb840108,0xfb600101,0xfb840108,0xfb610101,
	0xfb840108,0xfb620101,0xfb840108,0xfb630101,0xfb840108,0xfb640101,0xfb840108,0xfb650101,
	0xfb840108,0xfb660101,0xfb840108,0xfb670101,0xfb840108,0xfb680101,0xfb840108,0xfb690101,
	0xfb840108,0xfb6a0101,0xfb840108,0xfb6b0101,0xfb840108,0xfb6c0101,0xfb840108,0xfb6d0101,
	0xfb840108,0xfb6e0101,0xfb840108,0xfb6f0101,0xfb840108,0xfb700101,0xfb840108,0xfb710101,
	0xfb840108,0xfb720101,0xfb840108,0xfb730101,0xfb840108,0xfb740101,0xfb840108,0xfb750101,
	0xfb840108,0xfb760101,0xfb840108,0xfb770101,0xfb840108,0xfb780101,0xfb840108,0xfb790101,
	0xfb840108,0xfb7a0101,0xfb840108,0xfb7b0101,0xfb840108,0xfb7c0101,0xfb840108,0xfb7d0101,
	0xfb840108,0xfb7e0101,0xfb840108,0xfb7f0101,0xfb840108,0xfb800101,0xfb840108,0xfb810101,
	0xfb840108,0xfb820101,0xfb840108,0xfb830101,0xfb840108,0xfb840101,0xfb840108,0xfb850101,
	0xfb840108,0xfb860101,0xfb840108,0xfb870101,0xfb840108,0xfb880101,0xfb840108,0xfb890101,
	0xfb840108,0xfb8a0101,0xfb840108,0xfb8b0101,0xfb840108,0xfb8c0101,0xfb840108,0xfb8d0101,
	0xfb840108,0xfb8e0101,0xfb840108,0xfb8f0101,0xfb840108,0xfb900101,0xfb840108,0xfb910101,
	0xfb840108,0xfb920101,0xfb840108,0xfb930101,0xfb840108,0xfb940101,0xfb840108,0xfb950101,
	0xfb840108,0xfb960101,0xfb840108,0xfb970101,0xfb840108,0xfb980101,0xfb840108,0xfb990101,
	0xfb840108,0xfb9a0101,0xfb840108,0xfb9b0101,0xfb840108,0xfb9c0101,0xfb840108,0xfb9d0101,
	0xfb840108,0xfb9e0101,0xfb840108,0xfb9f0101,0xfb840108,0xfba00101,0xfb840108,0xfba10101,
	0xfb840108,0xfba20101,0xfb840108,0xfba30101,0xfb840108,0xfba40101,0xfb840108,0xfba50101,
	0xfb840108,0xfba60101,0xfb840108,0xfba70101,0xfb840108,0xfba80101,0xfb840108,0xfba90101,
	0xfb840108,0xfbaa0101,0xfb840108,0xfbab0101,0xfb840108,0xfbac0101,0xfb840108,0xfbad0101,
	0xfb840108,0xfbae0101,0xfb840108,0xfbaf0101,0xfb840108,0xfbb00101,0xfb840108,0xfbb10101,
	0xfb840108,0xfbb20101,0xfb840108,0xfbb30101,0xfb840108,0xfbb40101,0xfb840108,0xfbb50101,
	0xfb840108,0xfbb60101,0xfb840108,0xfbb70101,0xfb840108,0xfbb80101,0xfb840108,0xfbb90101,
	0xfb840108,0xfbba0101,0xfb840108,0xfbbb0101,0xfb840108,0xfbbc0101,0xfb840108,0xfbbd0101,
	0xfb840108,0xfbbe0101,0xfb840108,0xfbbf0101,0xfb840108,0xfbc00101,0xfb840108,0xfbc10101,
	0xfb840108,0xfbc20101,0xfb840108,0xfbc30101,0xfb840108,0xfbc40101,0xfb840108,0xfbc50101,
	0xfb840108,0xfbc60101,0xfb840108,0xfbc70101,0xfb840108,0xfbc80101,0xfb840108,0xfbc90101,
	0xfb840108,0xfbca0101,0xfb840108,0xfbcb0101,0xfb840108,0xfbcc0101,0xfb840108,0xfbcd0101,
	0xfb840108,0xfbce0101,0xfb840108,0xfbcf0101,0xfb840108,0xfbd00101,0xfb840108,0xfbd10101,
	0xfb840108,0xfbd20101,0xfb840108,0xfbd30101,0xfb840108,0xfbd40101,0xfb840108,0xfbd50101,
	0xfb840108,0xfbd60101,0xfb840108,0xfbd70101,0xfb840108,0xfbd80101,0xfb840108,0xfbd90101,
	0xfb840108,0xfbda0101,0xfb840108,0xfbdb0101,0xfb840108,0xfbdc0101,0xfb840108,0xfbdd0101,
	0xfb840108,0xfbde0101,0xfb840108,0xfbdf0101,0xfb840108,0xfbe00101,0xfb840108,0xfbe10101,
	0xfb840108,0xfbe20101,0xfb840108,0xfbe30101,0xfb840108,0xfbe40101,0xfb840108,0xfbe50101,
	0xfb840108,0xfbe60101,0xfb840108,0xfbe70101,0xfb840108,0xfbe80101,0xfb840108,0xfbe90101,
	0xfb840108,0xfbea0101,0xfb840108,0xfbeb0101,0xfb840108,0xfbec0101,0xfb840108,0xfbed0101,
	0xfb840108,0xfbee0101,0xfb840108,0xfbef0101,0xfb840108,0xfbf00101,0xfb840108,0xfbf10101,
	0xfb840108,0xfbf20101,0xfb840108,0xfbf30101,0xfb840108,0xfbf40101,0xfb840108,0xfbf50101,
	0xfb840108,0xfbf60101,0xfb840108,0xfbf70101,0xfb840108,0xfbf80101,0xfb840108,0xfbf90101,
	0xfb840108,0xfbfa0101,0xfb840108,0xfbfb0101,0xfb840108,0xfbfc0101,0xfb840108,0xfbfd0101,
	0xfb840108,0xfbfe0101,0xfb840108,0xfbff0101,0xfb840108,0xfc000101,0xfb840108,0xfc010101,
	0xfb840108,0xfc020101,0xfb840108,0xfc030101,0xfb840108,0xfc040101,0xfb840108,0xfc050101,
	0xfb840108,0xfc060101,0xfb840108,0xfc070101,0xfb840108,0xfc080101,0xfb840108,0xfc090101,
	0xfb840108,0xfc0a0101,0xfb840108,0xfc0b0101,0xfb840108,0xfc0c0101,0xfb840108,0xfc0d0101,
	0xfb840108,0xfc0e0101,0xfb840108,0xfc0f0101,0xfb840108,0xfc100101,0xfb840108,0xfc110101,
	0xfb840108,0xfc120101,0xfb840108,0xfc130101,0xfb840108,0xfc140101,0xfb840108,0xfc150101,
	0xfb840108,0xfc160101,0xfb840108,0xfc170101,0xfb840108,0xfc180101,0xfb840108,0xfc190101,
	0xfb840108,0xfc1a0101,0xfb840108,0xfc1b0101,0xfb840108,0xfc1c0101,0xfb840108,0xfc1d0101,
	0xfb840108,0xfc1e0101,0xfb840108,0xfc1f0101,0xfb840108,0xfc200101,0xfb840108,0xfc210101,
	0xfb840108,0xfc220101,0xfb840108,0xfc230101,0xfb840108,0xfc240101,0xfb840108,0xfc250101,
	0xfb840108,0xfc260101,0xfb840108,0xfc270101,0xfb840108,0xfc280101,0xfb840108,0xfc290101,
	0xfb840108,0xfc2a0101,0xfb840108,0xfc2b0101,0xfb840108,0xfc2c0101,0xfb840108,0xfc2d0101,
	0xfb840108,0xfc2e0101,0xfb840108,0xfc2f0101,0xfb840108,0xfc300101,0xfb840108,0xfc310101,
	0xfb840108,0xfc320101,0xfb840108,0xfc330101,0xfb840108,0xfc340101,0xfb840108,0xfc350101,
	0xfb840108,0xfc360101,0xfb840108,0xfc370101,0xfb840108,0xfc380101,0xfb840108,0xfc390101,
	0xfb840108,0xfc3a0101,0xfb840108,0xfc3b0101,0xfb840108,0xfc3c0101,0xfb840108,0xfc3d0101,
	0xfb840108,0xfc3e0101,0xfb840108,0xfc3f0101,0xfb840108,0xfc400101,0xfb840108,0xfc410101,
	0xfb840108,0xfc420101,0xfb840108,0xfc430101,0xfb840108,0xfc440101,0xfb840108,0xfc450101,
	0xfb840108,0xfc460101,0xfb840108,0xfc470101,0xfb840108,0xfc480101,0xfb840108,0xfc490101,
	0xfb840108,0xfc4a0101,0xfb840108,0xfc4b0101,0xfb840108,0xfc4c0101,0xfb840108,0xfc4d0101,
	0xfb840108,0xfc4e0101,0xfb840108,0xfc4f0101,0xfb840108,0xfc500101,0xfb840108,0xfc510101,
	0xfb840108,0xfc520101,0xfb840108,0xfc530101,0xfb840108,0xfc540101,0xfb840108,0xfc550101,
	0xfb840108,0xfc560101,0xfb840108,0xfc570101,0xfb840108,0xfc580101,0xfb840108,0xfc590101,
	0xfb840108,0xfc5a0101,0xfb840108,0xfc5b0101,0xfb840108,0xfc5c0101,0xfb840108,0xfc5d0101,
	0xfb840108,0xfc5e0101,0xfb840108,0xfc5f0101,0xfb840108,0xfc600101,0xfb840108,0xfc610101,
	0xfb840108,0xfc620101,0xfb840108,0xfc630101,0xfb840108,0xfc640101,0xfb840108,0xfc650101,
	0xfb840108,0xfc660101,0xfb840108,0xfc670101,0xfb840108,0xfc680101,0xfb840108,0xfc690101,
	0xfb840108,0xfc6a0101,0xfb840108,0xfc6b0101,0xfb840108,0xfc6c0101,0xfb840108,0xfc6d0101,
	0xfb840108,0xfc6e0101,0xfb840108,0xfc6f0101,0xfb840108,0xfc700101,0xfb840108,0xfc710101,
	0xfb840108,0xfc720101,0xfb840108,0xfc730101,0xfb840108,0xfc740101,0xfb840108,0xfc750101,
	0xfb840108,0xfc760101,0xfb840108,0xfc770101,0xfb840108,0xfc780101,0xfb840108,0xfc790101,
	0xfb840108,0xfc7a0101,0xfb840108,0xfc7b0101,0xfb840108,0xfc7c0101,0xfb840108,0xfc7d0101,
	0xfb840108,0xfc7e0101,0xfb840108,0xfc7f0101,0xfb840108,0xfc800101,0xfb840108,0xfc810101,
	0xfb840108,0xfc820101,0xfb840108,0xfc830101,0xfb840108,0xfc840101,0xfb840108,0xfc850101,
	0xfb840108,0xfc860101,0xfb840108,0xfc870101,0xfb840108,0xfc880101,0xfb840108,0xfc890101,
	0xfb840108,0xfc8a0101,0xfb840108,0xfc8b0101,0xfb840108,0xfc8c0101,0xfb840108,0xfc8d0101,
	0xfb840108,0xfc8e0101,0xfb840108,0xfc8f0101,0xfb840108,0xfc900101,0xfb840108,0xfc910101,
	0xfb840108,0xfc920101,0xfb840108,0xfc930101,0xfb840108,0xfc940101,0xfb840108,0xfc950101,
	0xfb840108,0xfc960101,0xfb840108,0xfc970101,0xfb840108,0xfc980101,0xfb840108,0xfc990101,
	0xfb840108,0xfc9a0101,0xfb840108,0xfc9b0101,0xfb840108,0xfc9c0101,0xfb840108,0xfc9d0101,
	0xfb840108,0xfc9e0101,0xfb840108,0xfc9f0101,0xfb840108,0xfca00101,0xfb840108,0xfca10101,
	0xfb840108,0xfca20101,0xfb840108,0xfca30101,0xfb840108,0xfca40101,0xfb840108,0xfca50101,
	0xfb840108,0xfca60101,0xfb840108,0xfca70101,0xfb840108,0xfca80101,0xfb840108,0xfca90101,
	0xfb840108,0xfcaa0101,0xfb840108,0xfcab0101,0xfb840108,0xfcac0101,0xfb840108,0xfcad0101,
	0xfb840108,0xfcae0101,0xfb840108,0xfcaf0101,0xfb840108,0xfcb00101,0xfb840108,0xfcb10101,
	0xfb840108,0xfcb20101,0xfb840108,0xfcb30101,0xfb840108,0xfcb40101,0xfb840108,0xfcb50101,
	0xfb840108,0xfcb60101,0xfb840108,0xfcb70101,0xfb840108,0xfcb80101,0xfb840108,0xfcb90101,
	0xfb840108,0xfcba0101,0xfb840108,0xfcbb0101,0xfb840108,0xfcbc0101,0xfb840108,0xfcbd0101,
	0xfb840108,0xfcbe0101,0xfb840108,0xfcbf0101,0xfb840108,0xfcc00101,0xfb840108,0xfcc10101,
	0xfb840108,0xfcc20101,0xfb840108,0xfcc30101,0xfb840108,0xfcc40101,0xfb840108,0xfcc50101,
	0xfb840108,0xfcc60101,0xfb840108,0xfcc70101,0xfb840108,0xfcc80101,0xfb840108,0xfcc90101,
	0xfb840108,0xfcca0101,0xfb840108,0xfccb0101,0xfb840108,0xfccc0101,0xfb840108,0xfccd0101,
	0xfb840108,0xfcce0101,0xfb840108,0xfccf0101,0xfb840108,0xfcd00101,0xfb840108,0xfcd10101,
	0xfb840108,0xfcd20101,0xfb840108,0xfcd30101,0xfb840108,0xfcd40101,0xfb840108,0xfcd50101,
	0xfb840108,0xfcd60101,0xfb840108,0xfcd70101,0xfb840108,0xfcd80101,0xfb840108,0xfcd90101,
	0xfb840108,0xfcda0101,0xfb840108,0xfcdb0101,0xfb840108,0xfcdc0101,0xfb840108,0xfcdd0101,
	0xfb840108,0xfcde0101,0xfb840108,0xfcdf0101,0xfb840108,0xfce00101,0xfb840108,0xfce10101,
	0xfb840108,0xfce20101,0xfb840108,0xfce30101,0xfb840108,0xfce40101,0xfb840108,0xfce50101,
	0xfb840108,0xfce60101,0xfb840108,0xfce70101,0xfb840108,0xfce80101,0xfb840108,0xfce90101,
	0xfb840108,0xfcea0101,0xfb840108,0xfceb0101,0xfb840108,0xfcec0101,0xfb840108,0xfced0101,
	0xfb840108,0xfcee0101,0xfb840108,0xfcef0101,0xfb840108,0xfcf00101,0xfb840108,0xfcf10101,
	0xfb840108,0xfcf20101,0xfb840108,0xfcf30101,0xfb840108,0xfcf40101,0xfb840108,0xfcf50101,
	0xfb840108,0xfcf60101,0xfb840108,0xfcf70101,0xfb840108,0xfcf80101,0xfb840108,0xfcf90101,
	0xfb840108,0xfcfa0101,0xfb840108,0xfcfb0101,0xfb840108,0xfcfc0101,0xfb840108,0xfcfd0101,
	0xfb840108,0xfcfe0101,0xfb840108,0xfcff0101,0xfb840108,0xfd000101,0xfb840108,0xfd010101,
	0xfb840108,0xfd020101,0xfb840108,0xfd030101,0xfb840108,0xfd040101,0xfb840108,0xfd050101,
	0xfb840108,0xfd060101,0xfb840108,0xfd070101,0xfb840108,0xfd080101,0xfb840108,0xfd090101,
	0xfb840108,0xfd0a0101,0xfb840108,0xfd0b0101,0xfb840108,0xfd0c0101,0xfb840108,0xfd0d0101,
	0xfb840108,0xfd0e0101,0xfb840108,0xfd0f0101,0xfb840108,0xfd100101,0xfb840108,0xfd110101,
	0xfb840108,0xfd120101,0xfb840108,0xfd130101,0xfb840108,0xfd140101,0xfb840108,0xfd150101,
	0xfb840108,0xfd160101,0xfb840108,0xfd170101,0xfb840108,0xfd180101,0xfb840108,0xfd190101,
	0xfb840108,0xfd1a0101,0xfb840108,0xfd1b0101,0xfb840108,0xfd1c0101,0xfb840108,0xfd1d0101,
	0xfb840108,0xfd1e0101,0xfb840108,0xfd1f0101,0xfb840108,0xfd200101,0xfb840108,0xfd210101,
	0xfb840108,0xfd220101,0xfb840108,0xfd230101,0xfb840108,0xfd240101,0xfb840108,0xfd250101,
	0xfb840108,0xfd260101,0xfb840108,0xfd270101,0xfb840108,0xfd280101,0xfb840108,0xfd290101,
	0xfb840108,0xfd2a0101,0xfb840108,0xfd2b0101,0xfb840108,0xfd2c0101,0xfb840108,0xfd2d0101,
	0xfb840108,0xfd2e0101,0xfb840108,0xfd2f0101,0xfb840108,0xfd300101,0xfb840108,0xfd310101,
	0xfb840108,0xfd320101,0xfb840108,0xfd330101,0xfb840108,0xfd340101,0xfb840108,0xfd350101,
	0xfb840108,0xfd360101,0xfb840108,0xfd370101,0xfb840108,0xfd380101,0xfb840108,0xfd390101,
	0xfb840108,0xfd3a0101,0xfb840108,0xfd3b0101,0xfb840108,0xfd3c0101,0xfb840108,0xfd3d0101,
	0xfb840108,0xfd3e0101,0xfb840108,0xfd3f0101,0xfb840108,0xfd400101,0xfb840108,0xfd410101,
	0xfb840108,0xfd420101,0xfb840108,0xfd430101,0xfb840108,0xfd440101,0xfb840108,0xfd450101,
	0xfb840108,0xfd460101,0xfb840108,0xfd470101,0xfb840108,0xfd480101,0xfb840108,0xfd490101,
	0xfb840108,0xfd4a0101,0xfb840108,0xfd4b0101,0xfb840108,0xfd4c0101,0xfb840108,0xfd4d0101,
	0xfb840108,0xfd4e0101,0xfb840108,0xfd4f0101,0xfb840108,0xfd500101,0xfb840108,0xfd510101,
	0xfb840108,0xfd520101,0xfb840108,0xfd530101,0xfb840108,0xfd540101,0xfb840108,0xfd550101,
	0xfb840108,0xfd560101,0xfb840108,0xfd570101,0xfb840108,0xfd580101,0xfb840108,0xfd590101,
	0xfb840108,0xfd5a0101,0xfb840108,0xfd5b0101,0xfb840108,0xfd5c0101,0xfb840108,0xfd5d0101,
	0xfb840108,0xfd5e0101,0xfb840108,0xfd5f0101,0xfb840108,0xfd600101,0xfb840108,0xfd610101,
	0xfb840108,0xfd620101,0xfb840108,0xfd630101,0xfb840108,0xfd640101,0xfb840108,0xfd650101,
	0xfb840108,0xfd660101,0xfb840108,0xfd670101,0xfb840108,0xfd680101,0xfb840108,0xfd690101,
	0xfb840108,0xfd6a0101,0xfb840108,0xfd6b0101,0xfb840108,0xfd6c0101,0xfb840108,0xfd6d0101,
	0xfb840108,0xfd6e0101,0xfb840108,0xfd6f0101,0xfb840108,0xfd700101,0xfb840108,0xfd710101,
	0xfb840108,0xfd720101,0xfb840108,0xfd730101,0xfb840108,0xfd740101,0xfb840108,0xfd750101,
	0xfb840108,0xfd760101,0xfb840108,0xfd770101,0xfb840108,0xfd780101,0xfb840108,0xfd790101,
	0xfb840108,0xfd7a0101,0xfb840108,0xfd7b0101,0xfb840108,0xfd7c0101,0xfb840108,0xfd7d0101,
	0xfb840108,0xfd7e0101,0xfb840108,0xfd7f0101,0xfb840108,0xfd800101,0xfb840108,0xfd810101,
	0xfb840108,0xfd820101,0xfb840108,0xfd830101,0xfb840108,0xfd840101,0xfb840108,0xfd850101,
	0xfb840108,0xfd860101,0xfb840108,0xfd870101,0xfb840108,0xfd880101,0xfb840108,0xfd890101,
	0xfb840108,0xfd8a0101,0xfb840108,0xfd8b0101,0xfb840108,0xfd8c0101,0xfb840108,0xfd8d0101,
	0xfb840108,0xfd8e0101,0xfb840108,0xfd8f0101,0xfb840108,0xfd900101,0xfb840108,0xfd910101,
	0xfb840108,0xfd920101,0xfb840108,0xfd930101,0xfb840108,0xfd940101,0xfb840108,0xfd950101,
	0xfb840108,0xfd960101,0xfb840108,0xfd970101,0xfb840108,0xfd980101,0xfb840108,0xfd990101,
	0xfb840108,0xfd9a0101,0xfb840108,0xfd9b0101,0xfb840108,0xfd9c0101,0xfb840108,0xfd9d0101,
	0xfb840108,0xfd9e0101,0xfb840108,0xfd9f0101,0xfb840108,0xfda00101,0xfb840108,0xfda10101,
	0xfb840108,0xfda20101,0xfb840108,0xfda30101,0xfb840108,0xfda40101,0xfb840108,0xfda50101,
	0xfb840108,0xfda60101,0xfb840108,0xfda70101,0xfb840108,0xfda80101,0xfb840108,0xfda90101,
	0xfb840108,0xfdaa0101,0xfb840108,0xfdab0101,0xfb840108,0xfdac0101,0xfb840108,0xfdad0101,
	0xfb840108,0xfdae0101,0xfb840108,0xfdaf0101,0xfb840108,0xfdb00101,0xfb840108,0xfdb10101,
	0xfb840108,0xfdb20101,0xfb840108,0xfdb30101,0xfb840108,0xfdb40101,0xfb840108,0xfdb50101,
	0xfb840108,0xfdb60101,0xfb840108,0xfdb70101,0xfb840108,0xfdb80101,0xfb840108,0xfdb90101,
	0xfb840108,0xfdba0101,0xfb840108,0xfdbb0101,0xfb840108,0xfdbc0101,0xfb840108,0xfdbd0101,
	0xfb840108,0xfdbe0101,0xfb840108,0xfdbf0101,0xfb840108,0xfdc00101,0xfb840108,0xfdc10101,
	0xfb840108,0xfdc20101,0xfb840108,0xfdc30101,0xfb840108,0xfdc40101,0xfb840108,0xfdc50101,
	0xfb840108,0xfdc60101,0xfb840108,0xfdc70101,0xfb840108,0xfdc80101,0xfb840108,0xfdc90101,
	0xfb840108,0xfdca0101,0xfb840108,0xfdcb0101,0xfb840108,0xfdcc0101,0xfb840108,0xfdcd0101,
	0xfb840108,0xfdce0101,0xfb840108,0xfdcf0101,0xfb840108,0xfdd00101,0xfb840108,0xfdd10101,
	0xfb840108,0xfdd20101,0xfb840108,0xfdd30101,0xfb840108,0xfdd40101,0xfb840108,0xfdd50101,
	0xfb840108,0xfdd60101,0xfb840108,0xfdd70101,0xfb840108,0xfdd80101,0xfb840108,0xfdd90101,
	0xfb840108,0xfdda0101,0xfb840108,0xfddb0101,0xfb840108,0xfddc0101,0xfb840108,0xfddd0101,
	0xfb840108,0xfdde0101,0xfb840108,0xfddf0101,0xfb840108,0xfde00101,0xfb840108,0xfde10101,
	0xfb840108,0xfde20101,0xfb840108,0xfde30101,0xfb840108,0xfde40101,0xfb840108,0xfde50101,
	0xfb840108,0xfde60101,0xfb840108,0xfde70101,0xfb840108,0xfde80101,0xfb840108,0xfde90101,
	0xfb840108,0xfdea0101,0xfb840108,0xfdeb0101,0xfb840108,0xfdec0101,0xfb840108,0xfded0101,
	0xfb840108,0xfdee0101,0xfb840108,0xfdef0101,0xfb840108,0xfdf00101,0xfb840108,0xfdf10101,
	0xfb840108,0xfdf20101,0xfb840108,0xfdf30101,0xfb840108,0xfdf40101,0xfb840108,0xfdf50101,
	0xfb840108,0xfdf60101,0xfb840108,0xfdf70101,0xfb840108,0xfdf80101,0xfb840108,0xfdf90101,
	0xfb840108,0xfdfa0101,0xfb840108,0xfdfb0101,0xfb840108,0xfdfc0101,0xfb840108,0xfdfd0101,
	0xfb840108,0xfdfe0101,0xfb840108,0xfdff0101,0xfb840108,0xfe000101,0xfb840108,0xfe010101,
	0xfb840108,0xfe020101,0xfb840108,0xfe030101,0xfb840108,0xfe040101,0xfb840108,0xfe050101,
	0xfb840108,0xfe060101,0xfb840108,0xfe070101,0xfb840108,0xfe080101,0xfb840108,0xfe090101,
	0xfb840108,0xfe0a0101,0xfb840108,0xfe0b0101,0xfb840108,0xfe0c0101,0xfb840108,0xfe0d0101,
	0xfb840108,0xfe0e0101,0xfb840108,0xfe0f0101,0xfb840108,0xfe100101,0xfb840108,0xfe110101,
	0xfb840108,0xfe120101,0xfb840108,0xfe130101,0xfb840108,0xfe140101,0xfb840108,0xfe150101,
	0xfb840108,0xfe160101,0xfb840108,0xfe170101,0xfb840108,0xfe180101,0xfb840108,0xfe190101,
	0xfb840108,0xfe1a0101,0xfb840108,0xfe1b0101,0xfb840108,0xfe1c0101,0xfb840108,0xfe1d0101,
	0xfb840108,0xfe1e0101,0xfb840108,0xfe1f0101,0xfb840108,0xfe200101,0xfb840108,0xfe210101,
	0xfb840108,0xfe220101,0xfb840108,0xfe230101,0xfb840108,0xfe240101,0xfb840108,0xfe250101,
	0xfb840108,0xfe260101,0xfb840108,0xfe270101,0xfb840108,0xfe280101,0xfb840108,0xfe290101,
	0xfb840108,0xfe2a0101,0xfb840108,0xfe2b0101,0xfb840108,0xfe2c0101,0xfb840108,0xfe2d0101,
	0xfb840108,0xfe2e0101,0xfb840108,0xfe2f0101,0xfb840108,0xfe300101,0xfb840108,0xfe310101,
	0xfb840108,0xfe320101,0xfb840108,0xfe330101,0xfb840108,0xfe340101,0xfb840108,0xfe350101,
	0xfb840108,0xfe360101,0xfb840108,0xfe370101,0xfb840108,0xfe380101,0xfb840108,0xfe390101,
	0xfb840108,0xfe3a0101,0xfb840108,0xfe3b0101,0xfb840108,0xfe3c0101,0xfb840108,0xfe3d0101,
	0xfb840108,0xfe3e0101,0xfb840108,0xfe3f0101,0xfb840108,0xfe400101,0xfb840108,0xfe410101,
	0xfb840108,0xfe420101,0xfb840108,0xfe430101,0xfb840108,0xfe440101,0xfb840108,0xfe450101,
	0xfb840108,0xfe460101,0xfb840108,0xfe470101,0xfb840108,0xfe480101,0xfb840108,0xfe490101,
	0xfb840108,0xfe4a0101,0xfb840108,0xfe4b0101,0xfb840108,0xfe4c0101,0xfb840108,0xfe4d0101,
	0xfb840108,0xfe4e0101,0xfb840108,0xfe4f0101,0xfb840108,0xfe500101,0xfb840108,0xfe510101,
	0xfb840108,0xfe520101,0xfb840108,0xfe530101,0xfb840108,0xfe540101,0xfb840108,0xfe550101,
	0xfb840108,0xfe560101,0xfb840108,0xfe570101,0xfb840108,0xfe580101,0xfb840108,0xfe590101,
	0xfb840108,0xfe5a0101,0xfb840108,0xfe5b0101,0xfb840108,0xfe5c0101,0xfb840108,0xfe5d0101,
	0xfb840108,0xfe5e0101,0xfb840108,0xfe5f0101,0xfb840108,0xfe600101,0xfb840108,0xfe610101,
	0xfb840108,0xfe620101,0xfb840108,0xfe630101,0xfb840108,0xfe640101,0xfb840108,0xfe650101,
	0xfb840108,0xfe660101,0xfb840108,0xfe670101,0xfb840108,0xfe680101,0xfb840108,0xfe690101,
	0xfb840108,0xfe6a0101,0xfb840108,0xfe6b0101,0xfb840108,0xfe6c0101,0xfb840108,0xfe6d0101,
	0xfb840108,0xfe6e0101,0xfb840108,0xfe6f0101,0xfb840108,0xfe700101,0xfb840108,0xfe710101,
	0xfb840108,0xfe720101,0xfb840108,0xfe730101,0xfb840108,0xfe740101,0xfb840108,0xfe750101,
	0xfb840108,0xfe760101,0xfb840108,0xfe770101,0xfb840108,0xfe780101,0xfb840108,0xfe790101,
	0xfb840108,0xfe7a0101,0xfb840108,0xfe7b0101,0xfb840108,0xfe7c0101,0xfb840108,0xfe7d0101,
	0xfb840108,0xfe7e0101,0xfb840108,0xfe7f0101,0xfb840108,0xfe800101,0xfb840108,0xfe810101,
	0xfb840108,0xfe820101,0xfb840108,0xfe830101,0xfb840108,0xfe840101,0xfb840108,0xfe850101,
	0xfb840108,0xfe860101,0xfb840108,0xfe870101,0xfb840108,0xfe880101,0xfb840108,0xfe890101,
	0xfb840108,0xfe8a0101,0xfb840108,0xfe8b0101,0xfb840108,0xfe8c0101,0xfb840108,0xfe8d0101,
	0xfb840108,0xfe8e0101,0xfb840108,0xfe8f0101,0xfb840108,0xfe900101,0xfb840108,0xfe910101,
	0xfb840108,0xfe920101,0xfb840108,0xfe930101,0xfb840108,0xfe940101,0xfb840108,0xfe950101,
	0xfb840108,0xfe960101,0xfb840108,0xfe970101,0xfb840108,0xfe980101,0xfb840108,0xfe990101,
	0xfb840108,0xfe9a0101,0xfb840108,0xfe9b0101,0xfb840108,0xfe9c0101,0xfb840108,0xfe9d0101,
	0xfb840108,0xfe9e0101,0xfb840108,0xfe9f0101,0xfb840108,0xfea00101,0xfb840108,0xfea10101,
	0xfb840108,0xfea20101,0xfb840108,0xfea30101,0xfb840108,0xfea40101,0xfb840108,0xfea50101,
	0xfb840108,0xfea60101,0xfb840108,0xfea70101,0xfb840108,0xfea80101,0xfb840108,0xfea90101,
	0xfb840108,0xfeaa0101,0xfb840108,0xfeab0101,0xfb840108,0xfeac0101,0xfb840108,0xfead0101,
	0xfb840108,0xfeae0101,0xfb840108,0xfeaf0101,0xfb840108,0xfeb00101,0xfb840108,0xfeb10101,
	0xfb840108,0xfeb20101,0xfb840108,0xfeb30101,0xfb840108,0xfeb40101,0xfb840108,0xfeb50101,
	0xfb840108,0xfeb60101,0xfb840108,0xfeb70101,0xfb840108,0xfeb80101,0xfb840108,0xfeb90101,
	0xfb840108,0xfeba0101,0xfb840108,0xfebb0101,0xfb840108,0xfebc0101,0xfb840108,0xfebd0101,
	0xfb840108,0xfebe0101,0xfb840108,0xfebf0101,0xfb840108,0xfec00101,0xfb840108,0xfec10101,
	0xfb840108,0xfec20101,0xfb840108,0xfec30101,0xfb840108,0xfec40101,0xfb840108,0xfec50101,
	0xfb840108,0xfec60101,0xfb840108,0xfec70101,0xfb840108,0xfec80101,0xfb840108,0xfec90101,
	0xfb840108,0xfeca0101,0xfb840108,0xfecb0101,0xfb840108,0xfecc0101,0xfb840108,0xfecd0101,
	0xfb840108,0xfece0101,0xfb840108,0xfecf0101,0xfb840108,0xfed00101,0xfb840108,0xfed10101,
	0xfb840108,0xfed20101,0xfb840108,0xfed30101,0xfb840108,0xfed40101,0xfb840108,0xfed50101,
	0xfb840108,0xfed60101,0xfb840108,0xfed70101,0xfb840108,0xfed80101,0xfb840108,0xfed90101,
	0xfb840108,0xfeda0101,0xfb840108,0xfedb0101,0xfb840108,0xfedc0101,0xfb840108,0xfedd0101,
	0xfb840108,0xfede0101,0xfb840108,0xfedf0101,0xfb840108,0xfee00101,0xfb840108,0xfee10101,
	0xfb840108,0xfee20101,0xfb840108,0xfee30101,0xfb840108,0xfee40101,0xfb840108,0xfee50101,
	0xfb840108,0xfee60101,0xfb840108,0xfee70101,0xfb840108,0xfee80101,0xfb840108,0xfee90101,
	0xfb840108,0xfeea0101,0xfb840108,0xfeeb0101,0xfb840108,0xfeec0101,0xfb840108,0xfeed0101,
	0xfb840108,0xfeee0101,0xfb840108,0xfeef0101,0xfb840108,0xfef00101,0xfb840108,0xfef10101,
	0xfb840108,0xfef20101,0xfb840108,0xfef30101,0xfb840108,0xfef40101,0xfb840108,0xfef50101,
	0xfb840108,0xfef60101,0xfb840108,0xfef70101,0xfb840108,0xfef80101,0xfb840108,0xfef90101,
	0xfb840108,0xfefa0101,0xfb840108,0xfefb0101,0xfb840108,0xfefc0101,0xfb840108,0xfefd0101,
	0xfb840108,0xfefe0101,0xfb840108,0xfeff0101,0xfb840108,0xff000101,0xfb840108,0xff010101,
	0xfb840108,0xff020101,0xfb840108,0xff030101,0xfb840108,0xff040101,0xfb840108,0xff050101,
	0xfb840108,0xff060101,0xfb840108,0xff070101,0xfb840108,0xff080101,0xfb840108,0xff090101,
	0xfb840108,0xff0a0101,0xfb840108,0xff0b0101,0xfb840108,0xff0c0101,0xfb840108,0xff0d0101,
	0xfb840108,0xff0e0101,0xfb840108,0xff0f0101,0xfb840108,0xff100101,0xfb840108,0xff110101,
	0xfb840108,0xff120101,0xfb840108,0xff130101,0xfb840108,0xff140101,0xfb840108,0xff150101,
	0xfb840108,0xff160101,0xfb840108,0xff170101,0xfb840108,0xff180101,0xfb840108,0xff190101,
	0xfb840108,0xff1a0101,0xfb840108,0xff1b0101,0xfb840108,0xff1c0101,0xfb840108,0xff1d0101,
	0xfb840108,0xff1e0101,0xfb840108,0xff1f0101,0xfb840108,0xff200101,0xfb840108,0xff210101,
	0xfb840108,0xff220101,0xfb840108,0xff230101,0xfb840108,0xff240101,0xfb840108,0xff250101,
	0xfb840108,0xff260101,0xfb840108,0xff270101,0xfb840108,0xff280101,0xfb840108,0xff290101,
	0xfb840108,0xff2a0101,0xfb840108,0xff2b0101,0xfb840108,0xff2c0101,0xfb840108,0xff2d0101,
	0xfb840108,0xff2e0101,0xfb840108,0xff2f0101,0xfb840108,0xff300101,0xfb840108,0xff310101,
	0xfb840108,0xff320101,0xfb840108,0xff330101,0xfb840108,0xff340101,0xfb840108,0xff350101,
	0xfb840108,0xff360101,0xfb840108,0xff370101,0xfb840108,0xff380101,0xfb840108,0xff390101,
	0xfb840108,0xff3a0101,0xfb840108,0xff3b0101,0xfb840108,0xff3c0101,0xfb840108,0xff3d0101,
	0xfb840108,0xff3e0101,0xfb840108,0xff3f0101,0xfb840108,0xff400101,0xfb840108,0xff410101,
	0xfb840108,0xff420101,0xfb840108,0xff430101,0xfb840108,0xff440101,0xfb840108,0xff450101,
	0xfb840108,0xff460101,0xfb840108,0xff470101,0xfb840108,0xff480101,0xfb840108,0xff490101,
	0xfb840108,0xff4a0101,0xfb840108,0xff4b0101,0xfb840108,0xff4c0101,0xfb840108,0xff4d0101,
	0xfb840108,0xff4e0101,0xfb840108,0xff4f0101,0xfb840108,0xff500101,0xfb840108,0xff510101,
	0xfb840108,0xff520101,0xfb840108,0xff530101,0xfb840108,0xff540101,0xfb840108,0xff550101,
	0xfb840108,0xff560101,0xfb840108,0xff570101,0xfb840108,0xff580101,0xfb840108,0xff590101,
	0xfb840108,0xff5a0101,0xfb840108,0xff5b0101,0xfb840108,0xff5c0101,0xfb840108,0xff5d0101,
	0xfb840108,0xff5e0101,0xfb840108,0xff5f0101,0xfb840108,0xff600101,0xfb840108,0xff610101,
	0xfb840108,0xff620101,0xfb840108,0xff630101,0xfb840108,0xff640101,0xfb840108,0xff650101,
	0xfb840108,0xff660101,0xfb840108,0xff670101,0xfb840108,0xff680101,0xfb840108,0xff690101,
	0xfb840108,0xff6a0101,0xfb840108,0xff6b0101,0xfb840108,0xff6c0101,0xfb840108,0xff6d0101,
	0xfb840108,0xff6e0101,0xfb840108,0xff6f0101,0xfb840108,0xff700101,0xfb840108,0xff710101,
	0xfb840108,0xff720101,0xfb840108,0xff730101,0xfb840108,0xff740101,0xfb840108,0xff750101,
	0xfb840108,0xff760101,0xfb840108,0xff770101,0xfb840108,0xff780101,0xfb840108,0xff790101,
	0xfb840108,0xff7a0101,0xfb840108,0xff7b0101,0xfb840108,0xff7c0101,0xfb840108,0xff7d0101,
	0xfb840108,0xff7e0101,0xfb840108,0xff7f0101,0xfb840108,0xff800101,0xfb840108,0xff810101,
	0xfb840108,0xff820101,0xfb840108,0xff830101,0xfb840108,0xff840101,0xfb840108,0xff850101,
	0xfb840108,0xff860101,0xfb840108,0xff870101,0xfb840108,0xff880101,0xfb840108,0xff890101,
	0xfb840108,0xff8a0101,0xfb840108,0xff8b0101,0xfb840108,0xff8c0101,0xfb840108,0xff8d0101,
	0xfb840108,0xff8e0101,0xfb840108,0xff8f0101,0xfb840108,0xff900101,0xfb840108,0xff910101,
	0xfb840108,0xff920101,0xfb840108,0xff930101,0xfb840108,0xff940101,0xfb840108,0xff950101,
	0xfb840108,0xff960101,0xfb840108,0xff970101,0xfb840108,0xff980101,0xfb840108,0xff990101,
	0xfb840108,0xff9a0101,0xfb840108,0xff9b0101,0xfb840108,0xff9c0101,0xfb840108,0xff9d0101,
	0xfb840108,0xff9e0101,0xfb840108,0xff9f0101,0xfb840108,0xffa00101,0xfb840108,0xffa10101,
	0xfb840108,0xffa20101,0xfb840108,0xffa30101,0xfb840108,0xffa40101,0xfb840108,0xffa50101,
	0xfb840108,0xffa60101,0xfb840108,0xffa70101,0xfb840108,0xffa80101,0xfb840108,0xffa90101,
	0xfb840108,0xffaa0101,0xfb840108,0xffab0101,0xfb840108,0xffac0101,0xfb840108,0xffad0101,
	0xfb840108,0xffae0101,0xfb840108,0xffaf0101,0xfb840108,0xffb00101,0xfb840108,0xffb10101,
	0xfb840108,0xffb20101,0xfb840108,0xffb30101,0xfb840108,0xffb40101,0xfb840108,0xffb50101,
	0xfb840108,0xffb60101,0xfb840108,0xffb70101,0xfb840108,0xffb80101,0xfb840108,0xffb90101,
	0xfb840108,0xffba0101,0xfb840108,0xffbb0101,0xfb840108,0xffbc0101,0xfb840108,0xffbd0101,
	0xfb840108,0xffbe0101,0xfb840108,0xffbf0101,0xfb840108,0xffc00101,0xfb840108,0xffc10101,
	0xfb840108,0xffc20101,0xfb840108,0xffc30101,0xfb840108,0xffc40101,0xfb840108,0xffc50101,
	0xfb840108,0xffc60101,0xfb840108,0xffc70101,0xfb840108,0xffc80101,0xfb840108,0xffc90101,
	0xfb840108,0xffca0101,0xfb840108,0xffcb0101,0xfb840108,0xffcc0101,0xfb840108,0xffcd0101,
	0xfb840108,0xffce0101,0xfb840108,0xffcf0101,0xfb840108,0xffd00101,0xfb840108,0xffd10101,
	0xfb840108,0xffd20101,0xfb840108,0xffd30101,0xfb840108,0xffd40101,0xfb840108,0xffd50101,
	0xfb840108,0xffd60101,0xfb840108,0xffd70101,0xfb840108,0xffd80101,0xfb840108,0xffd90101,
	0xfb840108,0xffda0101,0xfb840108,0xffdb0101,0xfb840108,0xffdc0101,0xfb840108,0xffdd0101,
	0xfb840108,0xffde0101,0xfb840108,0xffdf0101,0xfb840108,0xffe00101,0xfb840108,0xffe10101,
	0xfb840108,0xffe20101,0xfb840108,0xffe30101,0xfb840108,0xffe40101,0xfb840108,0xffe50101,
	0xfb840108,0xffe60101,0xfb840108,0xffe70101,0xfb840108,0xffe80101,0xfb840108,0xffe90101,
	0xfb840108,0xffea0101,0xfb840108,0xffeb0101,0xfb840108,0xffec0101,0xfb840108,0xffed0101,
	0xfb840108,0xffee0101,0xfb840108,0xffef0101,0xfb840108,0xfff00101,0xfb840108,0xfff10101,
	0xfb840108,0xfff20101,0xfb840108,0xfff30101,0xfb840108,0xfff40101,0xfb840108,0xfff50101,
	0xfb840108,0xfff60101,0xfb840108,0xfff70101,0xfb840108,0xfff80101,0xfb840108,0xfff90101,
	0xfb840108,0xfffa0101,0xfb840108,0xfffb0101,0xfb840108,0xfffc0101,0xfb840108,0xfffd0101,
	0xfb840108,0xfffe0101,0xfb840108,0xffff0101,0xfb850108,0x80000101,0xfb850108,0x80010101,
	0xfb850108,0x80020101,0xfb850108,0x80030101,0xfb850108,0x80040101,0xfb850108,0x80050101,
	0xfb850108,0x80060101,0xfb850108,0x80070101,0xfb850108,0x80080101,0xfb850108,0x80090101,
	0xfb850108,0x800a0101,0xfb850108,0x800b0101,0xfb850108,0x800c0101,0xfb850108,0x800d0101,
	0xfb850108,0x800e0101,0xfb850108,0x800f0101,0xfb850108,0x80100101,0xfb850108,0x80110101,
	0xfb850108,0x80120101,0xfb850108,0x80130101,0xfb850108,0x80140101,0xfb850108,0x80150101,
	0xfb850108,0x80160101,0xfb850108,0x80170101,0xfb850108,0x80180101,0xfb850108,0x80190101,
	0xfb850108,0x801a0101,0xfb850108,0x801b0101,0xfb850108,0x801c0101,0xfb850108,0x801d0101,
	0xfb850108,0x801e0101,0xfb850108,0x801f0101,0xfb850108,0x80200101,0xfb850108,0x80210101,
	0xfb850108,0x80220101,0xfb850108,0x80230101,0xfb850108,0x80240101,0xfb850108,0x80250101,
	0xfb850108,0x80260101,0xfb850108,0x80270101,0xfb850108,0x80280101,0xfb850108,0x80290101,
	0xfb850108,0x802a0101,0xfb850108,0x802b0101,0xfb850108,0x802c0101,0xfb850108,0x802d0101,
	0xfb850108,0x802e0101,0xfb850108,0x802f0101,0xfb850108,0x80300101,0xfb850108,0x80310101,
	0xfb850108,0x80320101,0xfb850108,0x80330101,0xfb850108,0x80340101,0xfb850108,0x80350101,
	0xfb850108,0x80360101,0xfb850108,0x80370101,0xfb850108,0x80380101,0xfb850108,0x80390101,
	0xfb850108,0x803a0101,0xfb850108,0x803b0101,0xfb850108,0x803c0101,0xfb850108,0x803d0101,
	0xfb850108,0x803e0101,0xfb850108,0x803f0101,0xfb850108,0x80400101,0xfb850108,0x80410101,
	0xfb850108,0x80420101,0xfb850108,0x80430101,0xfb850108,0x80440101,0xfb850108,0x80450101,
	0xfb850108,0x80460101,0xfb850108,0x80470101,0xfb850108,0x80480101,0xfb850108,0x80490101,
	0xfb850108,0x804a0101,0xfb850108,0x804b0101,0xfb850108,0x804c0101,0xfb850108,0x804d0101,
	0xfb850108,0x804e0101,0xfb850108,0x804f0101,0xfb850108,0x80500101,0xfb850108,0x80510101,
	0xfb850108,0x80520101,0xfb850108,0x80530101,0xfb850108,0x80540101,0xfb850108,0x80550101,
	0xfb850108,0x80560101,0xfb850108,0x80570101,0xfb850108,0x80580101,0xfb850108,0x80590101,
	0xfb850108,0x805a0101,0xfb850108,0x805b0101,0xfb850108,0x805c0101,0xfb850108,0x805d0101,
	0xfb850108,0x805e0101,0xfb850108,0x805f0101,0xfb850108,0x80600101,0xfb850108,0x80610101,
	0xfb850108,0x80620101,0xfb850108,0x80630101,0xfb850108,0x80640101,0xfb850108,0x80650101,
	0xfb850108,0x80660101,0xfb850108,0x80670101,0xfb850108,0x80680101,0xfb850108,0x80690101,
	0xfb850108,0x806a0101,0xfb850108,0x806b0101,0xfb850108,0x806c0101,0xfb850108,0x806d0101,
	0xfb850108,0x806e0101,0xfb850108,0x806f0101,0xfb850108,0x80700101,0xfb850108,0x80710101,
	0xfb850108,0x80720101,0xfb850108,0x80730101,0xfb850108,0x80740101,0xfb850108,0x80750101,
	0xfb850108,0x80760101,0xfb850108,0x80770101,0xfb850108,0x80780101,0xfb850108,0x80790101,
	0xfb850108,0x807a0101,0xfb850108,0x807b0101,0xfb850108,0x807c0101,0xfb850108,0x807d0101,
	0xfb850108,0x807e0101,0xfb850108,0x807f0101,0xfb850108,0x80800101,0xfb850108,0x80810101,
	0xfb850108,0x80820101,0xfb850108,0x80830101,0xfb850108,0x80840101,0xfb850108,0x80850101,
	0xfb850108,0x80860101,0xfb850108,0x80870101,0xfb850108,0x80880101,0xfb850108,0x80890101,
	0xfb850108,0x808a0101,0xfb850108,0x808b0101,0xfb850108,0x808c0101,0xfb850108,0x808d0101,
	0xfb850108,0x808e0101,0xfb850108,0x808f0101,0xfb850108,0x80900101,0xfb850108,0x80910101,
	0xfb850108,0x80920101,0xfb850108,0x80930101,0xfb850108,0x80940101,0xfb850108,0x80950101,
	0xfb850108,0x80960101,0xfb850108,0x80970101,0xfb850108,0x80980101,0xfb850108,0x80990101,
	0xfb850108,0x809a0101,0xfb850108,0x809b0101,0xfb850108,0x809c0101,0xfb850108,0x809d0101,
	0xfb850108,0x809e0101,0xfb850108,0x809f0101,0xfb850108,0x80a00101,0xfb850108,0x80a10101,
	0xfb850108,0x80a20101,0xfb850108,0x80a30101,0xfb850108,0x80a40101,0xfb850108,0x80a50101,
	0xfb850108,0x80a60101,0xfb850108,0x80a70101,0xfb850108,0x80a80101,0xfb850108,0x80a90101,
	0xfb850108,0x80aa0101,0xfb850108,0x80ab0101,0xfb850108,0x80ac0101,0xfb850108,0x80ad0101,
	0xfb850108,0x80ae0101,0xfb850108,0x80af0101,0xfb850108,0x80b00101,0xfb850108,0x80b10101,
	0xfb850108,0x80b20101,0xfb850108,0x80b30101,0xfb850108,0x80b40101,0xfb850108,0x80b50101,
	0xfb850108,0x80b60101,0xfb850108,0x80b70101,0xfb850108,0x80b80101,0xfb850108,0x80b90101,
	0xfb850108,0x80ba0101,0xfb850108,0x80bb0101,0xfb850108,0x80bc0101,0xfb850108,0x80bd0101,
	0xfb850108,0x80be0101,0xfb850108,0x80bf0101,0xfb850108,0x80c00101,0xfb850108,0x80c10101,
	0xfb850108,0x80c20101,0xfb850108,0x80c30101,0xfb850108,0x80c40101,0xfb850108,0x80c50101,
	0xfb850108,0x80c60101,0xfb850108,0x80c70101,0xfb850108,0x80c80101,0xfb850108,0x80c90101,
	0xfb850108,0x80ca0101,0xfb850108,0x80cb0101,0xfb850108,0x80cc0101,0xfb850108,0x80cd0101,
	0xfb850108,0x80ce0101,0xfb850108,0x80cf0101,0xfb850108,0x80d00101,0xfb850108,0x80d10101,
	0xfb850108,0x80d20101,0xfb850108,0x80d30101,0xfb850108,0x80d40101,0xfb850108,0x80d50101,
	0xfb850108,0x80d60101,0xfb850108,0x80d70101,0xfb850108,0x80d80101,0xfb850108,0x80d90101,
	0xfb850108,0x80da0101,0xfb850108,0x80db0101,0xfb850108,0x80dc0101,0xfb850108,0x80dd0101,
	0xfb850108,0x80de0101,0xfb850108,0x80df0101,0xfb850108,0x80e00101,0xfb850108,0x80e10101,
	0xfb850108,0x80e20101,0xfb850108,0x80e30101,0xfb850108,0x80e40101,0xfb850108,0x80e50101,
	0xfb850108,0x80e60101,0xfb850108,0x80e70101,0xfb850108,0x80e80101,0xfb850108,0x80e90101,
	0xfb850108,0x80ea0101,0xfb850108,0x80eb0101,0xfb850108,0x80ec0101,0xfb850108,0x80ed0101,
	0xfb850108,0x80ee0101,0xfb850108,0x80ef0101,0xfb850108,0x80f00101,0xfb850108,0x80f10101,
	0xfb850108,0x80f20101,0xfb850108,0x80f30101,0xfb850108,0x80f40101,0xfb850108,0x80f50101,
	0xfb850108,0x80f60101,0xfb850108,0x80f70101,0xfb850108,0x80f80101,0xfb850108,0x80f90101,
	0xfb850108,0x80fa0101,0xfb850108,0x80fb0101,0xfb850108,0x80fc0101,0xfb850108,0x80fd0101,
	0xfb850108,0x80fe0101,0xfb850108,0x80ff0101,0xfb850108,0x81000101,0xfb850108,0x81010101,
	0xfb850108,0x81020101,0xfb850108,0x81030101,0xfb850108,0x81040101,0xfb850108,0x81050101,
	0xfb850108,0x81060101,0xfb850108,0x81070101,0xfb850108,0x81080101,0xfb850108,0x81090101,
	0xfb850108,0x810a0101,0xfb850108,0x810b0101,0xfb850108,0x810c0101,0xfb850108,0x810d0101,
	0xfb850108,0x810e0101,0xfb850108,0x810f0101,0xfb850108,0x81100101,0xfb850108,0x81110101,
	0xfb850108,0x81120101,0xfb850108,0x81130101,0xfb850108,0x81140101,0xfb850108,0x81150101,
	0xfb850108,0x81160101,0xfb850108,0x81170101,0xfb850108,0x81180101,0xfb850108,0x81190101,
	0xfb850108,0x811a0101,0xfb850108,0x811b0101,0xfb850108,0x811c0101,0xfb850108,0x811d0101,
	0xfb850108,0x811e0101,0xfb850108,0x811f0101,0xfb850108,0x81200101,0xfb850108,0x81210101,
	0xfb850108,0x81220101,0xfb850108,0x81230101,0xfb850108,0x81240101,0xfb850108,0x81250101,
	0xfb850108,0x81260101,0xfb850108,0x81270101,0xfb850108,0x81280101,0xfb850108,0x81290101,
	0xfb850108,0x812a0101,0xfb850108,0x812b0101,0xfb850108,0x812c0101,0xfb850108,0x812d0101,
	0xfb850108,0x812e0101,0xfb850108,0x812f0101,0xfb850108,0x81300101,0xfb850108,0x81310101,
	0xfb850108,0x81320101,0xfb850108,0x81330101,0xfb850108,0x81340101,0xfb850108,0x81350101,
	0xfb850108,0x81360101,0xfb850108,0x81370101,0xfb850108,0x81380101,0xfb850108,0x81390101,
	0xfb850108,0x813a0101,0xfb850108,0x813b0101,0xfb850108,0x813c0101,0xfb850108,0x813d0101,
	0xfb850108,0x813e0101,0xfb850108,0x813f0101,0xfb850108,0x81400101,0xfb850108,0x81410101,
	0xfb850108,0x81420101,0xfb850108,0x81430101,0xfb850108,0x81440101,0xfb850108,0x81450101,
	0xfb850108,0x81460101,0xfb850108,0x81470101,0xfb850108,0x81480101,0xfb850108,0x81490101,
	0xfb850108,0x814a0101,0xfb850108,0x814b0101,0xfb850108,0x814c0101,0xfb850108,0x814d0101,
	0xfb850108,0x814e0101,0xfb850108,0x814f0101,0xfb850108,0x81500101,0xfb850108,0x81510101,
	0xfb850108,0x81520101,0xfb850108,0x81530101,0xfb850108,0x81540101,0xfb850108,0x81550101,
	0xfb850108,0x81560101,0xfb850108,0x81570101,0xfb850108,0x81580101,0xfb850108,0x81590101,
	0xfb850108,0x815a0101,0xfb850108,0x815b0101,0xfb850108,0x815c0101,0xfb850108,0x815d0101,
	0xfb850108,0x815e0101,0xfb850108,0x815f0101,0xfb850108,0x81600101,0xfb850108,0x81610101,
	0xfb850108,0x81620101,0xfb850108,0x81630101,0xfb850108,0x81640101,0xfb850108,0x81650101,
	0xfb850108,0x81660101,0xfb850108,0x81670101,0xfb850108,0x81680101,0xfb850108,0x81690101,
	0xfb850108,0x816a0101,0xfb850108,0x816b0101,0xfb850108,0x816c0101,0xfb850108,0x816d0101,
	0xfb850108,0x816e0101,0xfb850108,0x816f0101,0xfb850108,0x81700101,0xfb850108,0x81710101,
	0xfb850108,0x81720101,0xfb850108,0x81730101,0xfb850108,0x81740101,0xfb850108,0x81750101,
	0xfb850108,0x81760101,0xfb850108,0x81770101,0xfb850108,0x81780101,0xfb850108,0x81790101,
	0xfb850108,0x817a0101,0xfb850108,0x817b0101,0xfb850108,0x817c0101,0xfb850108,0x817d0101,
	0xfb850108,0x817e0101,0xfb850108,0x817f0101,0xfb850108,0x81800101,0xfb850108,0x81810101,
	0xfb850108,0x81820101,0xfb850108,0x81830101,0xfb850108,0x81840101,0xfb850108,0x81850101,
	0xfb850108,0x81860101,0xfb850108,0x81870101,0xfb850108,0x81880101,0xfb850108,0x81890101,
	0xfb850108,0x818a0101,0xfb850108,0x818b0101,0xfb850108,0x818c0101,0xfb850108,0x818d0101,
	0xfb850108,0x818e0101,0xfb850108,0x818f0101,0xfb850108,0x81900101,0xfb850108,0x81910101,
	0xfb850108,0x81920101,0xfb850108,0x81930101,0xfb850108,0x81940101,0xfb850108,0x81950101,
	0xfb850108,0x81960101,0xfb850108,0x81970101,0xfb850108,0x81980101,0xfb850108,0x81990101,
	0xfb850108,0x819a0101,0xfb850108,0x819b0101,0xfb850108,0x819c0101,0xfb850108,0x819d0101,
	0xfb850108,0x819e0101,0xfb850108,0x819f0101,0xfb850108,0x81a00101,0xfb850108,0x81a10101,
	0xfb850108,0x81a20101,0xfb850108,0x81a30101,0xfb850108,0x81a40101,0xfb850108,0x81a50101,
	0xfb850108,0x81a60101,0xfb850108,0x81a70101,0xfb850108,0x81a80101,0xfb850108,0x81a90101,
	0xfb850108,0x81aa0101,0xfb850108,0x81ab0101,0xfb850108,0x81ac0101,0xfb850108,0x81ad0101,
	0xfb850108,0x81ae0101,0xfb850108,0x81af0101,0xfb850108,0x81b00101,0xfb850108,0x81b10101,
	0xfb850108,0x81b20101,0xfb850108,0x81b30101,0xfb850108,0x81b40101,0xfb850108,0x81b50101,
	0xfb850108,0x81b60101,0xfb850108,0x81b70101,0xfb850108,0x81b80101,0xfb850108,0x81b90101,
	0xfb850108,0x81ba0101,0xfb850108,0x81bb0101,0xfb850108,0x81bc0101,0xfb850108,0x81bd0101,
	0xfb850108,0x81be0101,0xfb850108,0x81bf0101,0xfb850108,0x81c00101,0xfb850108,0x81c10101,
	0xfb850108,0x81c20101,0xfb850108,0x81c30101,0xfb850108,0x81c40101,0xfb850108,0x81c50101,
	0xfb850108,0x81c60101,0xfb850108,0x81c70101,0xfb850108,0x81c80101,0xfb850108,0x81c90101,
	0xfb850108,0x81ca0101,0xfb850108,0x81cb0101,0xfb850108,0x81cc0101,0xfb850108,0x81cd0101,
	0xfb850108,0x81ce0101,0xfb850108,0x81cf0101,0xfb850108,0x81d00101,0xfb850108,0x81d10101,
	0xfb850108,0x81d20101,0xfb850108,0x81d30101,0xfb850108,0x81d40101,0xfb850108,0x81d50101,
	0xfb850108,0x81d60101,0xfb850108,0x81d70101,0xfb850108,0x81d80101,0xfb850108,0x81d90101,
	0xfb850108,0x81da0101,0xfb850108,0x81db0101,0xfb850108,0x81dc0101,0xfb850108,0x81dd0101,
	0xfb850108,0x81de0101,0xfb850108,0x81df0101,0xfb850108,0x81e00101,0xfb850108,0x81e10101,
	0xfb850108,0x81e20101,0xfb850108,0x81e30101,0xfb850108,0x81e40101,0xfb850108,0x81e50101,
	0xfb850108,0x81e60101,0xfb850108,0x81e70101,0xfb850108,0x81e80101,0xfb850108,0x81e90101,
	0xfb850108,0x81ea0101,0xfb850108,0x81eb0101,0xfb850108,0x81ec0101,0xfb850108,0x81ed0101,
	0xfb850108,0x81ee0101,0xfb850108,0x81ef0101,0xfb850108,0x81f00101,0xfb850108,0x81f10101,
	0xfb850108,0x81f20101,0xfb850108,0x81f30101,0xfb850108,0x81f40101,0xfb850108,0x81f50101,
	0xfb850108,0x81f60101,0xfb850108,0x81f70101,0xfb850108,0x81f80101,0xfb850108,0x81f90101,
	0xfb850108,0x81fa0101,0xfb850108,0x81fb0101,0xfb850108,0x81fc0101,0xfb850108,0x81fd0101,
	0xfb850108,0x81fe0101,0xfb850108,0x81ff0101,0xfb850108,0x82000101,0xfb850108,0x82010101,
	0xfb850108,0x82020101,0xfb850108,0x82030101,0xfb850108,0x82040101,0xfb850108,0x82050101,
	0xfb850108,0x82060101,0xfb850108,0x82070101,0xfb850108,0x82080101,0xfb850108,0x82090101,
	0xfb850108,0x820a0101,0xfb850108,0x820b0101,0xfb850108,0x820c0101,0xfb850108,0x820d0101,
	0xfb850108,0x820e0101,0xfb850108,0x820f0101,0xfb850108,0x82100101,0xfb850108,0x82110101,
	0xfb850108,0x82120101,0xfb850108,0x82130101,0xfb850108,0x82140101,0xfb850108,0x82150101,
	0xfb850108,0x82160101,0xfb850108,0x82170101,0xfb850108,0x82180101,0xfb850108,0x82190101,
	0xfb850108,0x821a0101,0xfb850108,0x821b0101,0xfb850108,0x821c0101,0xfb850108,0x821d0101,
	0xfb850108,0x821e0101,0xfb850108,0x821f0101,0xfb850108,0x82200101,0xfb850108,0x82210101,
	0xfb850108,0x82220101,0xfb850108,0x82230101,0xfb850108,0x82240101,0xfb850108,0x82250101,
	0xfb850108,0x82260101,0xfb850108,0x82270101,0xfb850108,0x82280101,0xfb850108,0x82290101,
	0xfb850108,0x822a0101,0xfb850108,0x822b0101,0xfb850108,0x822c0101,0xfb850108,0x822d0101,
	0xfb850108,0x822e0101,0xfb850108,0x822f0101,0xfb850108,0x82300101,0xfb850108,0x82310101,
	0xfb850108,0x82320101,0xfb850108,0x82330101,0xfb850108,0x82340101,0xfb850108,0x82350101,
	0xfb850108,0x82360101,0xfb850108,0x82370101,0xfb850108,0x82380101,0xfb850108,0x82390101,
	0xfb850108,0x823a0101,0xfb850108,0x823b0101,0xfb850108,0x823c0101,0xfb850108,0x823d0101,
	0xfb850108,0x823e0101,0xfb850108,0x823f0101,0xfb850108,0x82400101,0xfb850108,0x82410101,
	0xfb850108,0x82420101,0xfb850108,0x82430101,0xfb850108,0x82440101,0xfb850108,0x82450101,
	0xfb850108,0x82460101,0xfb850108,0x82470101,0xfb850108,0x82480101,0xfb850108,0x82490101,
	0xfb850108,0x824a0101,0xfb850108,0x824b0101,0xfb850108,0x824c0101,0xfb850108,0x824d0101,
	0xfb850108,0x824e0101,0xfb850108,0x824f0101,0xfb850108,0x82500101,0xfb850108,0x82510101,
	0xfb850108,0x82520101,0xfb850108,0x82530101,0xfb850108,0x82540101,0xfb850108,0x82550101,
	0xfb850108,0x82560101,0xfb850108,0x82570101,0xfb850108,0x82580101,0xfb850108,0x82590101,
	0xfb850108,0x825a0101,0xfb850108,0x825b0101,0xfb850108,0x825c0101,0xfb850108,0x825d0101,
	0xfb850108,0x825e0101,0xfb850108,0x825f0101,0xfb850108,0x82600101,0xfb850108,0x82610101,
	0xfb850108,0x82620101,0xfb850108,0x82630101,0xfb850108,0x82640101,0xfb850108,0x82650101,
	0xfb850108,0x82660101,0xfb850108,0x82670101,0xfb850108,0x82680101,0xfb850108,0x82690101,
	0xfb850108,0x826a0101,0xfb850108,0x826b0101,0xfb850108,0x826c0101,0xfb850108,0x826d0101,
	0xfb850108,0x826e0101,0xfb850108,0x826f0101,0xfb850108,0x82700101,0xfb850108,0x82710101,
	0xfb850108,0x82720101,0xfb850108,0x82730101,0xfb850108,0x82740101,0xfb850108,0x82750101,
	0xfb850108,0x82760101,0xfb850108,0x82770101,0xfb850108,0x82780101,0xfb850108,0x82790101,
	0xfb850108,0x827a0101,0xfb850108,0x827b0101,0xfb850108,0x827c0101,0xfb850108,0x827d0101,
	0xfb850108,0x827e0101,0xfb850108,0x827f0101,0xfb850108,0x82800101,0xfb850108,0x82810101,
	0xfb850108,0x82820101,0xfb850108,0x82830101,0xfb850108,0x82840101,0xfb850108,0x82850101,
	0xfb850108,0x82860101,0xfb850108,0x82870101,0xfb850108,0x82880101,0xfb850108,0x82890101,
	0xfb850108,0x828a0101,0xfb850108,0x828b0101,0xfb850108,0x828c0101,0xfb850108,0x828d0101,
	0xfb850108,0x828e0101,0xfb850108,0x828f0101,0xfb850108,0x82900101,0xfb850108,0x82910101,
	0xfb850108,0x82920101,0xfb850108,0x82930101,0xfb850108,0x82940101,0xfb850108,0x82950101,
	0xfb850108,0x82960101,0xfb850108,0x82970101,0xfb850108,0x82980101,0xfb850108,0x82990101,
	0xfb850108,0x829a0101,0xfb850108,0x829b0101,0xfb850108,0x829c0101,0xfb850108,0x829d0101,
	0xfb850108,0x829e0101,0xfb850108,0x829f0101,0xfb850108,0x82a00101,0xfb850108,0x82a10101,
	0xfb850108,0x82a20101,0xfb850108,0x82a30101,0xfb850108,0x82a40101,0xfb850108,0x82a50101,
	0xfb850108,0x82a60101,0xfb850108,0x82a70101,0xfb850108,0x82a80101,0xfb850108,0x82a90101,
	0xfb850108,0x82aa0101,0xfb850108,0x82ab0101,0xfb850108,0x82ac0101,0xfb850108,0x82ad0101,
	0xfb850108,0x82ae0101,0xfb850108,0x82af0101,0xfb850108,0x82b00101,0xfb850108,0x82b10101,
	0xfb850108,0x82b20101,0xfb850108,0x82b30101,0xfb850108,0x82b40101,0xfb850108,0x82b50101,
	0xfb850108,0x82b60101,0xfb850108,0x82b70101,0xfb850108,0x82b80101,0xfb850108,0x82b90101,
	0xfb850108,0x82ba0101,0xfb850108,0x82bb0101,0xfb850108,0x82bc0101,0xfb850108,0x82bd0101,
	0xfb850108,0x82be0101,0xfb850108,0x82bf0101,0xfb850108,0x82c00101,0xfb850108,0x82c10101,
	0xfb850108,0x82c20101,0xfb850108,0x82c30101,0xfb850108,0x82c40101,0xfb850108,0x82c50101,
	0xfb850108,0x82c60101,0xfb850108,0x82c70101,0xfb850108,0x82c80101,0xfb850108,0x82c90101,
	0xfb850108,0x82ca0101,0xfb850108,0x82cb0101,0xfb850108,0x82cc0101,0xfb850108,0x82cd0101,
	0xfb850108,0x82ce0101,0xfb850108,0x82cf0101,0xfb850108,0x82d00101,0xfb850108,0x82d10101,
	0xfb850108,0x82d20101,0xfb850108,0x82d30101,0xfb850108,0x82d40101,0xfb850108,0x82d50101,
	0xfb850108,0x82d60101,0xfb850108,0x82d70101,0xfb850108,0x82d80101,0xfb850108,0x82d90101,
	0xfb850108,0x82da0101,0xfb850108,0x82db0101,0xfb850108,0x82dc0101,0xfb850108,0x82dd0101,
	0xfb850108,0x82de0101,0xfb850108,0x82df0101,0xfb850108,0x82e00101,0xfb850108,0x82e10101,
	0xfb850108,0x82e20101,0xfb850108,0x82e30101,0xfb850108,0x82e40101,0xfb850108,0x82e50101,
	0xfb850108,0x82e60101,0xfb850108,0x82e70101,0xfb850108,0x82e80101,0xfb850108,0x82e90101,
	0xfb850108,0x82ea0101,0xfb850108,0x82eb0101,0xfb850108,0x82ec0101,0xfb850108,0x82ed0101,
	0xfb850108,0x82ee0101,0xfb850108,0x82ef0101,0xfb850108,0x82f00101,0xfb850108,0x82f10101,
	0xfb850108,0x82f20101,0xfb850108,0x82f30101,0xfb850108,0x82f40101,0xfb850108,0x82f50101,
	0xfb850108,0x82f60101,0xfb850108,0x82f70101,0xfb850108,0x82f80101,0xfb850108,0x82f90101,
	0xfb850108,0x82fa0101,0xfb850108,0x82fb0101,0xfb850108,0x82fc0101,0xfb850108,0x82fd0101,
	0xfb850108,0x82fe0101,0xfb850108,0x82ff0101,0xfb850108,0x83000101,0xfb850108,0x83010101,
	0xfb850108,0x83020101,0xfb850108,0x83030101,0xfb850108,0x83040101,0xfb850108,0x83050101,
	0xfb850108,0x83060101,0xfb850108,0x83070101,0xfb850108,0x83080101,0xfb850108,0x83090101,
	0xfb850108,0x830a0101,0xfb850108,0x830b0101,0xfb850108,0x830c0101,0xfb850108,0x830d0101,
	0xfb850108,0x830e0101,0xfb850108,0x830f0101,0xfb850108,0x83100101,0xfb850108,0x83110101,
	0xfb850108,0x83120101,0xfb850108,0x83130101,0xfb850108,0x83140101,0xfb850108,0x83150101,
	0xfb850108,0x83160101,0xfb850108,0x83170101,0xfb850108,0x83180101,0xfb850108,0x83190101,
	0xfb850108,0x831a0101,0xfb850108,0x831b0101,0xfb850108,0x831c0101,0xfb850108,0x831d0101,
	0xfb850108,0x831e0101,0xfb850108,0x831f0101,0xfb850108,0x83200101,0xfb850108,0x83210101,
	0xfb850108,0x83220101,0xfb850108,0x83230101,0xfb850108,0x83240101,0xfb850108,0x83250101,
	0xfb850108,0x83260101,0xfb850108,0x83270101,0xfb850108,0x83280101,0xfb850108,0x83290101,
	0xfb850108,0x832a0101,0xfb850108,0x832b0101,0xfb850108,0x832c0101,0xfb850108,0x832d0101,
	0xfb850108,0x832e0101,0xfb850108,0x832f0101,0xfb850108,0x83300101,0xfb850108,0x83310101,
	0xfb850108,0x83320101,0xfb850108,0x83330101,0xfb850108,0x83340101,0xfb850108,0x83350101,
	0xfb850108,0x83360101,0xfb850108,0x83370101,0xfb850108,0x83380101,0xfb850108,0x83390101,
	0xfb850108,0x833a0101,0xfb850108,0x833b0101,0xfb850108,0x833c0101,0xfb850108,0x833d0101,
	0xfb850108,0x833e0101,0xfb850108,0x833f0101,0xfb850108,0x83400101,0xfb850108,0x83410101,
	0xfb850108,0x83420101,0xfb850108,0x83430101,0xfb850108,0x83440101,0xfb850108,0x83450101,
	0xfb850108,0x83460101,0xfb850108,0x83470101,0xfb850108,0x83480101,0xfb850108,0x83490101,
	0xfb850108,0x834a0101,0xfb850108,0x834b0101,0xfb850108,0x834c0101,0xfb850108,0x834d0101,
	0xfb850108,0x834e0101,0xfb850108,0x834f0101,0xfb850108,0x83500101,0xfb850108,0x83510101,
	0xfb850108,0x83520101,0xfb850108,0x83530101,0xfb850108,0x83540101,0xfb850108,0x83550101,
	0xfb850108,0x83560101,0xfb850108,0x83570101,0xfb850108,0x83580101,0xfb850108,0x83590101,
	0xfb850108,0x835a0101,0xfb850108,0x835b0101,0xfb850108,0x835c0101,0xfb850108,0x835d0101,
	0xfb850108,0x835e0101,0xfb850108,0x835f0101,0xfb850108,0x83600101,0xfb850108,0x83610101,
	0xfb850108,0x83620101,0xfb850108,0x83630101,0xfb850108,0x83640101,0xfb850108,0x83650101,
	0xfb850108,0x83660101,0xfb850108,0x83670101,0xfb850108,0x83680101,0xfb850108,0x83690101,
	0xfb850108,0x836a0101,0xfb850108,0x836b0101,0xfb850108,0x836c0101,0xfb850108,0x836d0101,
	0xfb850108,0x836e0101,0xfb850108,0x836f0101,0xfb850108,0x83700101,0xfb850108,0x83710101,
	0xfb850108,0x83720101,0xfb850108,0x83730101,0xfb850108,0x83740101,0xfb850108,0x83750101,
	0xfb850108,0x83760101,0xfb850108,0x83770101,0xfb850108,0x83780101,0xfb850108,0x83790101,
	0xfb850108,0x837a0101,0xfb850108,0x837b0101,0xfb850108,0x837c0101,0xfb850108,0x837d0101,
	0xfb850108,0x837e0101,0xfb850108,0x837f0101,0xfb850108,0x83800101,0xfb850108,0x83810101,
	0xfb850108,0x83820101,0xfb850108,0x83830101,0xfb850108,0x83840101,0xfb850108,0x83850101,
	0xfb850108,0x83860101,0xfb850108,0x83870101,0xfb850108,0x83880101,0xfb850108,0x83890101,
	0xfb850108,0x838a0101,0xfb850108,0x838b0101,0xfb850108,0x838c0101,0xfb850108,0x838d0101,
	0xfb850108,0x838e0101,0xfb850108,0x838f0101,0xfb850108,0x83900101,0xfb850108,0x83910101,
	0xfb850108,0x83920101,0xfb850108,0x83930101,0xfb850108,0x83940101,0xfb850108,0x83950101,
	0xfb850108,0x83960101,0xfb850108,0x83970101,0xfb850108,0x83980101,0xfb850108,0x83990101,
	0xfb850108,0x839a0101,0xfb850108,0x839b0101,0xfb850108,0x839c0101,0xfb850108,0x839d0101,
	0xfb850108,0x839e0101,0xfb850108,0x839f0101,0xfb850108,0x83a00101,0xfb850108,0x83a10101,
	0xfb850108,0x83a20101,0xfb850108,0x83a30101,0xfb850108,0x83a40101,0xfb850108,0x83a50101,
	0xfb850108,0x83a60101,0xfb850108,0x83a70101,0xfb850108,0x83a80101,0xfb850108,0x83a90101,
	0xfb850108,0x83aa0101,0xfb850108,0x83ab0101,0xfb850108,0x83ac0101,0xfb850108,0x83ad0101,
	0xfb850108,0x83ae0101,0xfb850108,0x83af0101,0xfb850108,0x83b00101,0xfb850108,0x83b10101,
	0xfb850108,0x83b20101,0xfb850108,0x83b30101,0xfb850108,0x83b40101,0xfb850108,0x83b50101,
	0xfb850108,0x83b60101,0xfb850108,0x83b70101,0xfb850108,0x83b80101,0xfb850108,0x83b90101,
	0xfb850108,0x83ba0101,0xfb850108,0x83bb0101,0xfb850108,0x83bc0101,0xfb850108,0x83bd0101,
	0xfb850108,0x83be0101,0xfb850108,0x83bf0101,0xfb850108,0x83c00101,0xfb850108,0x83c10101,
	0xfb850108,0x83c20101,0xfb850108,0x83c30101,0xfb850108,0x83c40101,0xfb850108,0x83c50101,
	0xfb850108,0x83c60101,0xfb850108,0x83c70101,0xfb850108,0x83c80101,0xfb850108,0x83c90101,
	0xfb850108,0x83ca0101,0xfb850108,0x83cb0101,0xfb850108,0x83cc0101,0xfb850108,0x83cd0101,
	0xfb850108,0x83ce0101,0xfb850108,0x83cf0101,0xfb850108,0x83d00101,0xfb850108,0x83d10101,
	0xfb850108,0x83d20101,0xfb850108,0x83d30101,0xfb850108,0x83d40101,0xfb850108,0x83d50101,
	0xfb850108,0x83d60101,0xfb850108,0x83d70101,0xfb850108,0x83d80101,0xfb850108,0x83d90101,
	0xfb850108,0x83da0101,0xfb850108,0x83db0101,0xfb850108,0x83dc0101,0xfb850108,0x83dd0101,
	0xfb850108,0x83de0101,0xfb850108,0x83df0101,0xfb850108,0x83e00101,0xfb850108,0x83e10101,
	0xfb850108,0x83e20101,0xfb850108,0x83e30101,0xfb850108,0x83e40101,0xfb850108,0x83e50101,
	0xfb850108,0x83e60101,0xfb850108,0x83e70101,0xfb850108,0x83e80101,0xfb850108,0x83e90101,
	0xfb850108,0x83ea0101,0xfb850108,0x83eb0101,0xfb850108,0x83ec0101,0xfb850108,0x83ed0101,
	0xfb850108,0x83ee0101,0xfb850108,0x83ef0101,0xfb850108,0x83f00101,0xfb850108,0x83f10101,
	0xfb850108,0x83f20101,0xfb850108,0x83f30101,0xfb850108,0x83f40101,0xfb850108,0x83f50101,
	0xfb850108,0x83f60101,0xfb850108,0x83f70101,0xfb850108,0x83f80101,0xfb850108,0x83f90101,
	0xfb850108,0x83fa0101,0xfb850108,0x83fb0101,0xfb850108,0x83fc0101,0xfb850108,0x83fd0101,
	0xfb850108,0x83fe0101,0xfb850108,0x83ff0101,0xfb850108,0x84000101,0xfb850108,0x84010101,
	0xfb850108,0x84020101,0xfb850108,0x84030101,0xfb850108,0x84040101,0xfb850108,0x84050101,
	0xfb850108,0x84060101,0xfb850108,0x84070101,0xfb850108,0x84080101,0xfb850108,0x84090101,
	0xfb850108,0x840a0101,0xfb850108,0x840b0101,0xfb850108,0x840c0101,0xfb850108,0x840d0101,
	0xfb850108,0x840e0101,0xfb850108,0x840f0101,0xfb850108,0x84100101,0xfb850108,0x84110101,
	0xfb850108,0x84120101,0xfb850108,0x84130101,0xfb850108,0x84140101,0xfb850108,0x84150101,
	0xfb850108,0x84160101,0xfb850108,0x84170101,0xfb850108,0x84180101,0xfb850108,0x84190101,
	0xfb850108,0x841a0101,0xfb850108,0x841b0101,0xfb850108,0x841c0101,0xfb850108,0x841d0101,
	0xfb850108,0x841e0101,0xfb850108,0x841f0101,0xfb850108,0x84200101,0xfb850108,0x84210101,
	0xfb850108,0x84220101,0xfb850108,0x84230101,0xfb850108,0x84240101,0xfb850108,0x84250101,
	0xfb850108,0x84260101,0xfb850108,0x84270101,0xfb850108,0x84280101,0xfb850108,0x84290101,
	0xfb850108,0x842a0101,0xfb850108,0x842b0101,0xfb850108,0x842c0101,0xfb850108,0x842d0101,
	0xfb850108,0x842e0101,0xfb850108,0x842f0101,0xfb850108,0x84300101,0xfb850108,0x84310101,
	0xfb850108,0x84320101,0xfb850108,0x84330101,0xfb850108,0x84340101,0xfb850108,0x84350101,
	0xfb850108,0x84360101,0xfb850108,0x84370101,0xfb850108,0x84380101,0xfb850108,0x84390101,
	0xfb850108,0x843a0101,0xfb850108,0x843b0101,0xfb850108,0x843c0101,0xfb850108,0x843d0101,
	0xfb850108,0x843e0101,0xfb850108,0x843f0101,0xfb850108,0x84400101,0xfb850108,0x84410101,
	0xfb850108,0x84420101,0xfb850108,0x84430101,0xfb850108,0x84440101,0xfb850108,0x84450101,
	0xfb850108,0x84460101,0xfb850108,0x84470101,0xfb850108,0x84480101,0xfb850108,0x84490101,
	0xfb850108,0x844a0101,0xfb850108,0x844b0101,0xfb850108,0x844c0101,0xfb850108,0x844d0101,
	0xfb850108,0x844e0101,0xfb850108,0x844f0101,0xfb850108,0x84500101,0xfb850108,0x84510101,
	0xfb850108,0x84520101,0xfb850108,0x84530101,0xfb850108,0x84540101,0xfb850108,0x84550101,
	0xfb850108,0x84560101,0xfb850108,0x84570101,0xfb850108,0x84580101,0xfb850108,0x84590101,
	0xfb850108,0x845a0101,0xfb850108,0x845b0101,0xfb850108,0x845c0101,0xfb850108,0x845d0101,
	0xfb850108,0x845e0101,0xfb850108,0x845f0101,0xfb850108,0x84600101,0xfb850108,0x84610101,
	0xfb850108,0x84620101,0xfb850108,0x84630101,0xfb850108,0x84640101,0xfb850108,0x84650101,
	0xfb850108,0x84660101,0xfb850108,0x84670101,0xfb850108,0x84680101,0xfb850108,0x84690101,
	0xfb850108,0x846a0101,0xfb850108,0x846b0101,0xfb850108,0x846c0101,0xfb850108,0x846d0101,
	0xfb850108,0x846e0101,0xfb850108,0x846f0101,0xfb850108,0x84700101,0xfb850108,0x84710101,
	0xfb850108,0x84720101,0xfb850108,0x84730101,0xfb850108,0x84740101,0xfb850108,0x84750101,
	0xfb850108,0x84760101,0xfb850108,0x84770101,0xfb850108,0x84780101,0xfb850108,0x84790101,
	0xfb850108,0x847a0101,0xfb850108,0x847b0101,0xfb850108,0x847c0101,0xfb850108,0x847d0101,
	0xfb850108,0x847e0101,0xfb850108,0x847f0101,0xfb850108,0x84800101,0xfb850108,0x84810101,
	0xfb850108,0x84820101,0xfb850108,0x84830101,0xfb850108,0x84840101,0xfb850108,0x84850101,
	0xfb850108,0x84860101,0xfb850108,0x84870101,0xfb850108,0x84880101,0xfb850108,0x84890101,
	0xfb850108,0x848a0101,0xfb850108,0x848b0101,0xfb850108,0x848c0101,0xfb850108,0x848d0101,
	0xfb850108,0x848e0101,0xfb850108,0x848f0101,0xfb850108,0x84900101,0xfb850108,0x84910101,
	0xfb850108,0x84920101,0xfb850108,0x84930101,0xfb850108,0x84940101,0xfb850108,0x84950101,
	0xfb850108,0x84960101,0xfb850108,0x84970101,0xfb850108,0x84980101,0xfb850108,0x84990101,
	0xfb850108,0x849a0101,0xfb850108,0x849b0101,0xfb850108,0x849c0101,0xfb850108,0x849d0101,
	0xfb850108,0x849e0101,0xfb850108,0x849f0101,0xfb850108,0x84a00101,0xfb850108,0x84a10101,
	0xfb850108,0x84a20101,0xfb850108,0x84a30101,0xfb850108,0x84a40101,0xfb850108,0x84a50101,
	0xfb850108,0x84a60101,0xfb850108,0x84a70101,0xfb850108,0x84a80101,0xfb850108,0x84a90101,
	0xfb850108,0x84aa0101,0xfb850108,0x84ab0101,0xfb850108,0x84ac0101,0xfb850108,0x84ad0101,
	0xfb850108,0x84ae0101,0xfb850108,0x84af0101,0xfb850108,0x84b00101,0xfb850108,0x84b10101,
	0xfb850108,0x84b20101,0xfb850108,0x84b30101,0xfb850108,0x84b40101,0xfb850108,0x84b50101,
	0xfb850108,0x84b60101,0xfb850108,0x84b70101,0xfb850108,0x84b80101,0xfb850108,0x84b90101,
	0xfb850108,0x84ba0101,0xfb850108,0x84bb0101,0xfb850108,0x84bc0101,0xfb850108,0x84bd0101,
	0xfb850108,0x84be0101,0xfb850108,0x84bf0101,0xfb850108,0x84c00101,0xfb850108,0x84c10101,
	0xfb850108,0x84c20101,0xfb850108,0x84c30101,0xfb850108,0x84c40101,0xfb850108,0x84c50101,
	0xfb850108,0x84c60101,0xfb850108,0x84c70101,0xfb850108,0x84c80101,0xfb850108,0x84c90101,
	0xfb850108,0x84ca0101,0xfb850108,0x84cb0101,0xfb850108,0x84cc0101,0xfb850108,0x84cd0101,
	0xfb850108,0x84ce0101,0xfb850108,0x84cf0101,0xfb850108,0x84d00101,0xfb850108,0x84d10101,
	0xfb850108,0x84d20101,0xfb850108,0x84d30101,0xfb850108,0x84d40101,0xfb850108,0x84d50101,
	0xfb850108,0x84d60101,0xfb850108,0x84d70101,0xfb850108,0x84d80101,0xfb850108,0x84d90101,
	0xfb850108,0x84da0101,0xfb850108,0x84db0101,0xfb850108,0x84dc0101,0xfb850108,0x84dd0101,
	0xfb850108,0x84de0101,0xfb850108,0x84df0101,0xfb850108,0x84e00101,0xfb850108,0x84e10101,
	0xfb850108,0x84e20101,0xfb850108,0x84e30101,0xfb850108,0x84e40101,0xfb850108,0x84e50101,
	0xfb850108,0x84e60101,0xfb850108,0x84e70101,0xfb850108,0x84e80101,0xfb850108,0x84e90101,
	0xfb850108,0x84ea0101,0xfb850108,0x84eb0101,0xfb850108,0x84ec0101,0xfb850108,0x84ed0101,
	0xfb850108,0x84ee0101,0xfb850108,0x84ef0101,0xfb850108,0x84f00101,0xfb850108,0x84f10101,
	0xfb850108,0x84f20101,0xfb850108,0x84f30101,0xfb850108,0x84f40101,0xfb850108,0x84f50101,
	0xfb850108,0x84f60101,0xfb850108,0x84f70101,0xfb850108,0x84f80101,0xfb850108,0x84f90101,
	0xfb850108,0x84fa0101,0xfb850108,0x84fb0101,0xfb850108,0x84fc0101,0xfb850108,0x84fd0101,
	0xfb850108,0x84fe0101,0xfb850108,0x84ff0101,0xfb850108,0x85000101,0xfb850108,0x85010101,
	0xfb850108,0x85020101,0xfb850108,0x85030101,0xfb850108,0x85040101,0xfb850108,0x85050101,
	0xfb850108,0x85060101,0xfb850108,0x85070101,0xfb850108,0x85080101,0xfb850108,0x85090101,
	0xfb850108,0x850a0101,0xfb850108,0x850b0101,0xfb850108,0x850c0101,0xfb850108,0x850d0101,
	0xfb850108,0x850e0101,0xfb850108,0x850f0101,0xfb850108,0x85100101,0xfb850108,0x85110101,
	0xfb850108,0x85120101,0xfb850108,0x85130101,0xfb850108,0x85140101,0xfb850108,0x85150101,
	0xfb850108,0x85160101,0xfb850108,0x85170101,0xfb850108,0x85180101,0xfb850108,0x85190101,
	0xfb850108,0x851a0101,0xfb850108,0x851b0101,0xfb850108,0x851c0101,0xfb850108,0x851d0101,
	0xfb850108,0x851e0101,0xfb850108,0x851f0101,0xfb850108,0x85200101,0xfb850108,0x85210101,
	0xfb850108,0x85220101,0xfb850108,0x85230101,0xfb850108,0x85240101,0xfb850108,0x85250101,
	0xfb850108,0x85260101,0xfb850108,0x85270101,0xfb850108,0x85280101,0xfb850108,0x85290101,
	0xfb850108,0x852a0101,0xfb850108,0x852b0101,0xfb850108,0x852c0101,0xfb850108,0x852d0101,
	0xfb850108,0x852e0101,0xfb850108,0x852f0101,0xfb850108,0x85300101,0xfb850108,0x85310101,
	0xfb850108,0x85320101,0xfb850108,0x85330101,0xfb850108,0x85340101,0xfb850108,0x85350101,
	0xfb850108,0x85360101,0xfb850108,0x85370101,0xfb850108,0x85380101,0xfb850108,0x85390101,
	0xfb850108,0x853a0101,0xfb850108,0x853b0101,0xfb850108,0x853c0101,0xfb850108,0x853d0101,
	0xfb850108,0x853e0101,0xfb850108,0x853f0101,0xfb850108,0x85400101,0xfb850108,0x85410101,
	0xfb850108,0x85420101,0xfb850108,0x85430101,0xfb850108,0x85440101,0xfb850108,0x85450101,
	0xfb850108,0x85460101,0xfb850108,0x85470101,0xfb850108,0x85480101,0xfb850108,0x85490101,
	0xfb850108,0x854a0101,0xfb850108,0x854b0101,0xfb850108,0x854c0101,0xfb850108,0x854d0101,
	0xfb850108,0x854e0101,0xfb850108,0x854f0101,0xfb850108,0x85500101,0xfb850108,0x85510101,
	0xfb850108,0x85520101,0xfb850108,0x85530101,0xfb850108,0x85540101,0xfb850108,0x85550101,
	0xfb850108,0x85560101,0xfb850108,0x85570101,0xfb850108,0x85580101,0xfb850108,0x85590101,
	0xfb850108,0x855a0101,0xfb850108,0x855b0101,0xfb850108,0x855c0101,0xfb850108,0x855d0101,
	0xfb850108,0x855e0101,0xfb850108,0x855f0101,0xfb850108,0x85600101,0xfb850108,0x85610101,
	0xfb850108,0x85620101,0xfb850108,0x85630101,0xfb850108,0x85640101,0xfb850108,0x85650101,
	0xfb850108,0x85660101,0xfb850108,0x85670101,0xfb850108,0x85680101,0xfb850108,0x85690101,
	0xfb850108,0x856a0101,0xfb850108,0x856b0101,0xfb850108,0x856c0101,0xfb850108,0x856d0101,
	0xfb850108,0x856e0101,0xfb850108,0x856f0101,0xfb850108,0x85700101,0xfb850108,0x85710101,
	0xfb850108,0x85720101,0xfb850108,0x85730101,0xfb850108,0x85740101,0xfb850108,0x85750101,
	0xfb850108,0x85760101,0xfb850108,0x85770101,0xfb850108,0x85780101,0xfb850108,0x85790101,
	0xfb850108,0x857a0101,0xfb850108,0x857b0101,0xfb850108,0x857c0101,0xfb850108,0x857d0101,
	0xfb850108,0x857e0101,0xfb850108,0x857f0101,0xfb850108,0x85800101,0xfb850108,0x85810101,
	0xfb850108,0x85820101,0xfb850108,0x85830101,0xfb850108,0x85840101,0xfb850108,0x85850101,
	0xfb850108,0x85860101,0xfb850108,0x85870101,0xfb850108,0x85880101,0xfb850108,0x85890101,
	0xfb850108,0x858a0101,0xfb850108,0x858b0101,0xfb850108,0x858c0101,0xfb850108,0x858d0101,
	0xfb850108,0x858e0101,0xfb850108,0x858f0101,0xfb850108,0x85900101,0xfb850108,0x85910101,
	0xfb850108,0x85920101,0xfb850108,0x85930101,0xfb850108,0x85940101,0xfb850108,0x85950101,
	0xfb850108,0x85960101,0xfb850108,0x85970101,0xfb850108,0x85980101,0xfb850108,0x85990101,
	0xfb850108,0x859a0101,0xfb850108,0x859b0101,0xfb850108,0x859c0101,0xfb850108,0x859d0101,
	0xfb850108,0x859e0101,0xfb850108,0x859f0101,0xfb850108,0x85a00101,0xfb850108,0x85a10101,
	0xfb850108,0x85a20101,0xfb850108,0x85a30101,0xfb850108,0x85a40101,0xfb850108,0x85a50101,
	0xfb850108,0x85a60101,0xfb850108,0x85a70101,0xfb850108,0x85a80101,0xfb850108,0x85a90101,
	0xfb850108,0x85aa0101,0xfb850108,0x85ab0101,0xfb850108,0x85ac0101,0xfb850108,0x85ad0101,
	0xfb850108,0x85ae0101,0xfb850108,0x85af0101,0xfb850108,0x85b00101,0xfb850108,0x85b10101,
	0xfb850108,0x85b20101,0xfb850108,0x85b30101,0xfb850108,0x85b40101,0xfb850108,0x85b50101,
	0xfb850108,0x85b60101,0xfb850108,0x85b70101,0xfb850108,0x85b80101,0xfb850108,0x85b90101,
	0xfb850108,0x85ba0101,0xfb850108,0x85bb0101,0xfb850108,0x85bc0101,0xfb850108,0x85bd0101,
	0xfb850108,0x85be0101,0xfb850108,0x85bf0101,0xfb850108,0x85c00101,0xfb850108,0x85c10101,
	0xfb850108,0x85c20101,0xfb850108,0x85c30101,0xfb850108,0x85c40101,0xfb850108,0x85c50101,
	0xfb850108,0x85c60101,0xfb850108,0x85c70101,0xfb850108,0x85c80101,0xfb850108,0x85c90101,
	0xfb850108,0x85ca0101,0xfb850108,0x85cb0101,0xfb850108,0x85cc0101,0xfb850108,0x85cd0101,
	0xfb850108,0x85ce0101,0xfb850108,0x85cf0101,0xfb850108,0x85d00101,0xfb850108,0x85d10101,
	0xfb850108,0x85d20101,0xfb850108,0x85d30101,0xfb850108,0x85d40101,0xfb850108,0x85d50101,
	0xfb850108,0x85d60101,0xfb850108,0x85d70101,0xfb850108,0x85d80101,0xfb850108,0x85d90101,
	0xfb850108,0x85da0101,0xfb850108,0x85db0101,0xfb850108,0x85dc0101,0xfb850108,0x85dd0101,
	0xfb850108,0x85de0101,0xfb850108,0x85df0101,0xfb850108,0x85e00101,0xfb850108,0x85e10101,
	0xfb850108,0x85e20101,0xfb850108,0x85e30101,0xfb850108,0x85e40101,0xfb850108,0x85e50101,
	0xfb850108,0x85e60101,0xfb850108,0x85e70101,0xfb850108,0x85e80101,0xfb850108,0x85e90101,
	0xfb850108,0x85ea0101,0xfb850108,0x85eb0101,0xfb850108,0x85ec0101,0xfb850108,0x85ed0101,
	0xfb850108,0x85ee0101,0xfb850108,0x85ef0101,0xfb850108,0x85f00101,0xfb850108,0x85f10101,
	0xfb850108,0x85f20101,0xfb850108,0x85f30101,0xfb850108,0x85f40101,0xfb850108,0x85f50101,
	0xfb850108,0x85f60101,0xfb850108,0x85f70101,0xfb850108,0x85f80101,0xfb850108,0x85f90101,
	0xfb850108,0x85fa0101,0xfb850108,0x85fb0101,0xfb850108,0x85fc0101,0xfb850108,0x85fd0101,
	0xfb850108,0x85fe0101,0xfb850108,0x85ff0101,0xfb850108,0x86000101,0xfb850108,0x86010101,
	0xfb850108,0x86020101,0xfb850108,0x86030101,0xfb850108,0x86040101,0xfb850108,0x86050101,
	0xfb850108,0x86060101,0xfb850108,0x86070101,0xfb850108,0x86080101,0xfb850108,0x86090101,
	0xfb850108,0x860a0101,0xfb850108,0x860b0101,0xfb850108,0x860c0101,0xfb850108,0x860d0101,
	0xfb850108,0x860e0101,0xfb850108,0x860f0101,0xfb850108,0x86100101,0xfb850108,0x86110101,
	0xfb850108,0x86120101,0xfb850108,0x86130101,0xfb850108,0x86140101,0xfb850108,0x86150101,
	0xfb850108,0x86160101,0xfb850108,0x86170101,0xfb850108,0x86180101,0xfb850108,0x86190101,
	0xfb850108,0x861a0101,0xfb850108,0x861b0101,0xfb850108,0x861c0101,0xfb850108,0x861d0101,
	0xfb850108,0x861e0101,0xfb850108,0x861f0101,0xfb850108,0x86200101,0xfb850108,0x86210101,
	0xfb850108,0x86220101,0xfb850108,0x86230101,0xfb850108,0x86240101,0xfb850108,0x86250101,
	0xfb850108,0x86260101,0xfb850108,0x86270101,0xfb850108,0x86280101,0xfb850108,0x86290101,
	0xfb850108,0x862a0101,0xfb850108,0x862b0101,0xfb850108,0x862c0101,0xfb850108,0x862d0101,
	0xfb850108,0x862e0101,0xfb850108,0x862f0101,0xfb850108,0x86300101,0xfb850108,0x86310101,
	0xfb850108,0x86320101,0xfb850108,0x86330101,0xfb850108,0x86340101,0xfb850108,0x86350101,
	0xfb850108,0x86360101,0xfb850108,0x86370101,0xfb850108,0x86380101,0xfb850108,0x86390101,
	0xfb850108,0x863a0101,0xfb850108,0x863b0101,0xfb850108,0x863c0101,0xfb850108,0x863d0101,
	0xfb850108,0x863e0101,0xfb850108,0x863f0101,0xfb850108,0x86400101,0xfb850108,0x86410101,
	0xfb850108,0x86420101,0xfb850108,0x86430101,0xfb850108,0x86440101,0xfb850108,0x86450101,
	0xfb850108,0x86460101,0xfb850108,0x86470101,0xfb850108,0x86480101,0xfb850108,0x86490101,
	0xfb850108,0x864a0101,0xfb850108,0x864b0101,0xfb850108,0x864c0101,0xfb850108,0x864d0101,
	0xfb850108,0x864e0101,0xfb850108,0x864f0101,0xfb850108,0x86500101,0xfb850108,0x86510101,
	0xfb850108,0x86520101,0xfb850108,0x86530101,0xfb850108,0x86540101,0xfb850108,0x86550101,
	0xfb850108,0x86560101,0xfb850108,0x86570101,0xfb850108,0x86580101,0xfb850108,0x86590101,
	0xfb850108,0x865a0101,0xfb850108,0x865b0101,0xfb850108,0x865c0101,0xfb850108,0x865d0101,
	0xfb850108,0x865e0101,0xfb850108,0x865f0101,0xfb850108,0x86600101,0xfb850108,0x86610101,
	0xfb850108,0x86620101,0xfb850108,0x86630101,0xfb850108,0x86640101,0xfb850108,0x86650101,
	0xfb850108,0x86660101,0xfb850108,0x86670101,0xfb850108,0x86680101,0xfb850108,0x86690101,
	0xfb850108,0x866a0101,0xfb850108,0x866b0101,0xfb850108,0x866c0101,0xfb850108,0x866d0101,
	0xfb850108,0x866e0101,0xfb850108,0x866f0101,0xfb850108,0x86700101,0xfb850108,0x86710101,
	0xfb850108,0x86720101,0xfb850108,0x86730101,0xfb850108,0x86740101,0xfb850108,0x86750101,
	0xfb850108,0x86760101,0xfb850108,0x86770101,0xfb850108,0x86780101,0xfb850108,0x86790101,
	0xfb850108,0x867a0101,0xfb850108,0x867b0101,0xfb850108,0x867c0101,0xfb850108,0x867d0101,
	0xfb850108,0x867e0101,0xfb850108,0x867f0101,0xfb850108,0x86800101,0xfb850108,0x86810101,
	0xfb850108,0x86820101,0xfb850108,0x86830101,0xfb850108,0x86840101,0xfb850108,0x86850101,
	0xfb850108,0x86860101,0xfb850108,0x86870101,0xfb850108,0x86880101,0xfb850108,0x86890101,
	0xfb850108,0x868a0101,0xfb850108,0x868b0101,0xfb850108,0x868c0101,0xfb850108,0x868d0101,
	0xfb850108,0x868e0101,0xfb850108,0x868f0101,0xfb850108,0x86900101,0xfb850108,0x86910101,
	0xfb850108,0x86920101,0xfb850108,0x86930101,0xfb850108,0x86940101,0xfb850108,0x86950101,
	0xfb850108,0x86960101,0xfb850108,0x86970101,0xfb850108,0x86980101,0xfb850108,0x86990101,
	0xfb850108,0x869a0101,0xfb850108,0x869b0101,0xfb850108,0x869c0101,0xfb850108,0x869d0101,
	0xfb850108,0x869e0101,0xfb850108,0x869f0101,0xfb850108,0x86a00101,0xfb850108,0x86a10101,
	0xfb850108,0x86a20101,0xfb850108,0x86a30101,0xfb850108,0x86a40101,0xfb850108,0x86a50101,
	0xfb850108,0x86a60101,0xfb850108,0x86a70101,0xfb850108,0x86a80101,0xfb850108,0x86a90101,
	0xfb850108,0x86aa0101,0xfb850108,0x86ab0101,0xfb850108,0x86ac0101,0xfb850108,0x86ad0101,
	0xfb850108,0x86ae0101,0xfb850108,0x86af0101,0xfb850108,0x86b00101,0xfb850108,0x86b10101,
	0xfb850108,0x86b20101,0xfb850108,0x86b30101,0xfb850108,0x86b40101,0xfb850108,0x86b50101,
	0xfb850108,0x86b60101,0xfb850108,0x86b70101,0xfb850108,0x86b80101,0xfb850108,0x86b90101,
	0xfb850108,0x86ba0101,0xfb850108,0x86bb0101,0xfb850108,0x86bc0101,0xfb850108,0x86bd0101,
	0xfb850108,0x86be0101,0xfb850108,0x86bf0101,0xfb850108,0x86c00101,0xfb850108,0x86c10101,
	0xfb850108,0x86c20101,0xfb850108,0x86c30101,0xfb850108,0x86c40101,0xfb850108,0x86c50101,
	0xfb850108,0x86c60101,0xfb850108,0x86c70101,0xfb850108,0x86c80101,0xfb850108,0x86c90101,
	0xfb850108,0x86ca0101,0xfb850108,0x86cb0101,0xfb850108,0x86cc0101,0xfb850108,0x86cd0101,
	0xfb850108,0x86ce0101,0xfb850108,0x86cf0101,0xfb850108,0x86d00101,0xfb850108,0x86d10101,
	0xfb850108,0x86d20101,0xfb850108,0x86d30101,0xfb850108,0x86d40101,0xfb850108,0x86d50101,
	0xfb850108,0x86d60101,0xfb850108,0x86d70101,0xfb850108,0x86d80101,0xfb850108,0x86d90101,
	0xfb850108,0x86da0101,0xfb850108,0x86db0101,0xfb850108,0x86dc0101,0xfb850108,0x86dd0101,
	0xfb850108,0x86de0101,0xfb850108,0x86df0101,0xfb850108,0x86e00101,0xfb850108,0x86e10101,
	0xfb850108,0x86e20101,0xfb850108,0x86e30101,0xfb850108,0x86e40101,0xfb850108,0x86e50101,
	0xfb850108,0x86e60101,0xfb850108,0x86e70101,0xfb850108,0x86e80101,0xfb850108,0x86e90101,
	0xfb850108,0x86ea0101,0xfb850108,0x86eb0101,0xfb850108,0x86ec0101,0xfb850108,0x86ed0101,
	0xfb850108,0x86ee0101,0xfb850108,0x86ef0101,0xfb850108,0x86f00101,0xfb850108,0x86f10101,
	0xfb850108,0x86f20101,0xfb850108,0x86f30101,0xfb850108,0x86f40101,0xfb850108,0x86f50101,
	0xfb850108,0x86f60101,0xfb850108,0x86f70101,0xfb850108,0x86f80101,0xfb850108,0x86f90101,
	0xfb850108,0x86fa0101,0xfb850108,0x86fb0101,0xfb850108,0x86fc0101,0xfb850108,0x86fd0101,
	0xfb850108,0x86fe0101,0xfb850108,0x86ff0101,0xfb850108,0x87000101,0xfb850108,0x87010101,
	0xfb850108,0x87020101,0xfb850108,0x87030101,0xfb850108,0x87040101,0xfb850108,0x87050101,
	0xfb850108,0x87060101,0xfb850108,0x87070101,0xfb850108,0x87080101,0xfb850108,0x87090101,
	0xfb850108,0x870a0101,0xfb850108,0x870b0101,0xfb850108,0x870c0101,0xfb850108,0x870d0101,
	0xfb850108,0x870e0101,0xfb850108,0x870f0101,0xfb850108,0x87100101,0xfb850108,0x87110101,
	0xfb850108,0x87120101,0xfb850108,0x87130101,0xfb850108,0x87140101,0xfb850108,0x87150101,
	0xfb850108,0x87160101,0xfb850108,0x87170101,0xfb850108,0x87180101,0xfb850108,0x87190101,
	0xfb850108,0x871a0101,0xfb850108,0x871b0101,0xfb850108,0x871c0101,0xfb850108,0x871d0101,
	0xfb850108,0x871e0101,0xfb850108,0x871f0101,0xfb850108,0x87200101,0xfb850108,0x87210101,
	0xfb850108,0x87220101,0xfb850108,0x87230101,0xfb850108,0x87240101,0xfb850108,0x87250101,
	0xfb850108,0x87260101,0xfb850108,0x87270101,0xfb850108,0x87280101,0xfb850108,0x87290101,
	0xfb850108,0x872a0101,0xfb850108,0x872b0101,0xfb850108,0x872c0101,0xfb850108,0x872d0101,
	0xfb850108,0x872e0101,0xfb850108,0x872f0101,0xfb850108,0x87300101,0xfb850108,0x87310101,
	0xfb850108,0x87320101,0xfb850108,0x87330101,0xfb850108,0x87340101,0xfb850108,0x87350101,
	0xfb850108,0x87360101,0xfb850108,0x87370101,0xfb850108,0x87380101,0xfb850108,0x87390101,
	0xfb850108,0x873a0101,0xfb850108,0x873b0101,0xfb850108,0x873c0101,0xfb850108,0x873d0101,
	0xfb850108,0x873e0101,0xfb850108,0x873f0101,0xfb850108,0x87400101,0xfb850108,0x87410101,
	0xfb850108,0x87420101,0xfb850108,0x87430101,0xfb850108,0x87440101,0xfb850108,0x87450101,
	0xfb850108,0x87460101,0xfb850108,0x87470101,0xfb850108,0x87480101,0xfb850108,0x87490101,
	0xfb850108,0x874a0101,0xfb850108,0x874b0101,0xfb850108,0x874c0101,0xfb850108,0x874d0101,
	0xfb850108,0x874e0101,0xfb850108,0x874f0101,0xfb850108,0x87500101,0xfb850108,0x87510101,
	0xfb850108,0x87520101,0xfb850108,0x87530101,0xfb850108,0x87540101,0xfb850108,0x87550101,
	0xfb850108,0x87560101,0xfb850108,0x87570101,0xfb850108,0x87580101,0xfb850108,0x87590101,
	0xfb850108,0x875a0101,0xfb850108,0x875b0101,0xfb850108,0x875c0101,0xfb850108,0x875d0101,
	0xfb850108,0x875e0101,0xfb850108,0x875f0101,0xfb850108,0x87600101,0xfb850108,0x87610101,
	0xfb850108,0x87620101,0xfb850108,0x87630101,0xfb850108,0x87640101,0xfb850108,0x87650101,
	0xfb850108,0x87660101,0xfb850108,0x87670101,0xfb850108,0x87680101,0xfb850108,0x87690101,
	0xfb850108,0x876a0101,0xfb850108,0x876b0101,0xfb850108,0x876c0101,0xfb850108,0x876d0101,
	0xfb850108,0x876e0101,0xfb850108,0x876f0101,0xfb850108,0x87700101,0xfb850108,0x87710101,
	0xfb850108,0x87720101,0xfb850108,0x87730101,0xfb850108,0x87740101,0xfb850108,0x87750101,
	0xfb850108,0x87760101,0xfb850108,0x87770101,0xfb850108,0x87780101,0xfb850108,0x87790101,
	0xfb850108,0x877a0101,0xfb850108,0x877b0101,0xfb850108,0x877c0101,0xfb850108,0x877d0101,
	0xfb850108,0x877e0101,0xfb850108,0x877f0101,0xfb850108,0x87800101,0xfb850108,0x87810101,
	0xfb850108,0x87820101,0xfb850108,0x87830101,0xfb850108,0x87840101,0xfb850108,0x87850101,
	0xfb850108,0x87860101,0xfb850108,0x87870101,0xfb850108,0x87880101,0xfb850108,0x87890101,
	0xfb850108,0x878a0101,0xfb850108,0x878b0101,0xfb850108,0x878c0101,0xfb850108,0x878d0101,
	0xfb850108,0x878e0101,0xfb850108,0x878f0101,0xfb850108,0x87900101,0xfb850108,0x87910101,
	0xfb850108,0x87920101,0xfb850108,0x87930101,0xfb850108,0x87940101,0xfb850108,0x87950101,
	0xfb850108,0x87960101,0xfb850108,0x87970101,0xfb850108,0x87980101,0xfb850108,0x87990101,
	0xfb850108,0x879a0101,0xfb850108,0x879b0101,0xfb850108,0x879c0101,0xfb850108,0x879d0101,
	0xfb850108,0x879e0101,0xfb850108,0x879f0101,0xfb850108,0x87a00101,0xfb850108,0x87a10101,
	0xfb850108,0x87a20101,0xfb850108,0x87a30101,0xfb850108,0x87a40101,0xfb850108,0x87a50101,
	0xfb850108,0x87a60101,0xfb850108,0x87a70101,0xfb850108,0x87a80101,0xfb850108,0x87a90101,
	0xfb850108,0x87aa0101,0xfb850108,0x87ab0101,0xfb850108,0x87ac0101,0xfb850108,0x87ad0101,
	0xfb850108,0x87ae0101,0xfb850108,0x87af0101,0xfb850108,0x87b00101,0xfb850108,0x87b10101,
	0xfb850108,0x87b20101,0xfb850108,0x87b30101,0xfb850108,0x87b40101,0xfb850108,0x87b50101,
	0xfb850108,0x87b60101,0xfb850108,0x87b70101,0xfb850108,0x87b80101,0xfb850108,0x87b90101,
	0xfb850108,0x87ba0101,0xfb850108,0x87bb0101,0xfb850108,0x87bc0101,0xfb850108,0x87bd0101,
	0xfb850108,0x87be0101,0xfb850108,0x87bf0101,0xfb850108,0x87c00101,0xfb850108,0x87c10101,
	0xfb850108,0x87c20101,0xfb850108,0x87c30101,0xfb850108,0x87c40101,0xfb850108,0x87c50101,
	0xfb850108,0x87c60101,0xfb850108,0x87c70101,0xfb850108,0x87c80101,0xfb850108,0x87c90101,
	0xfb850108,0x87ca0101,0xfb850108,0x87cb0101,0xfb850108,0x87cc0101,0xfb850108,0x87cd0101,
	0xfb850108,0x87ce0101,0xfb850108,0x87cf0101,0xfb850108,0x87d00101,0xfb850108,0x87d10101,
	0xfb850108,0x87d20101,0xfb850108,0x87d30101,0xfb850108,0x87d40101,0xfb850108,0x87d50101,
	0xfb850108,0x87d60101,0xfb850108,0x87d70101,0xfb850108,0x87d80101,0xfb850108,0x87d90101,
	0xfb850108,0x87da0101,0xfb850108,0x87db0101,0xfb850108,0x87dc0101,0xfb850108,0x87dd0101,
	0xfb850108,0x87de0101,0xfb850108,0x87df0101,0xfb850108,0x87e00101,0xfb850108,0x87e10101,
	0xfb850108,0x87e20101,0xfb850108,0x87e30101,0xfb850108,0x87e40101,0xfb850108,0x87e50101,
	0xfb850108,0x87e60101,0xfb850108,0x87e70101,0xfb850108,0x87e80101,0xfb850108,0x87e90101,
	0xfb850108,0x87ea0101,0xfb850108,0x87eb0101,0xfb850108,0x87ec0101,0xfb850108,0x87ed0101,
	0xfb850108,0x87ee0101,0xfb850108,0x87ef0101,0xfb850108,0x87f00101,0xfb850108,0x87f10101,
	0xfb850108,0x87f20101,0xfb850108,0x87f30101,0xfb850108,0x87f40101,0xfb850108,0x87f50101,
	0xfb850108,0x87f60101,0xfb850108,0x87f70101,0xfb850108,0x87f80101,0xfb850108,0x87f90101,
	0xfb850108,0x87fa0101,0xfb850108,0x87fb0101,0xfb850108,0x87fc0101,0xfb850108,0x87fd0101,
	0xfb850108,0x87fe0101,0xfb850108,0x87ff0101,0xfb850108,0x88000101,0xfb850108,0x88010101,
	0xfb850108,0x88020101,0xfb850108,0x88030101,0xfb850108,0x88040101,0xfb850108,0x88050101,
	0xfb850108,0x88060101,0xfb850108,0x88070101,0xfb850108,0x88080101,0xfb850108,0x88090101,
	0xfb850108,0x880a0101,0xfb850108,0x880b0101,0xfb850108,0x880c0101,0xfb850108,0x880d0101,
	0xfb850108,0x880e0101,0xfb850108,0x880f0101,0xfb850108,0x88100101,0xfb850108,0x88110101,
	0xfb850108,0x88120101,0xfb850108,0x88130101,0xfb850108,0x88140101,0xfb850108,0x88150101,
	0xfb850108,0x88160101,0xfb850108,0x88170101,0xfb850108,0x88180101,0xfb850108,0x88190101,
	0xfb850108,0x881a0101,0xfb850108,0x881b0101,0xfb850108,0x881c0101,0xfb850108,0x881d0101,
	0xfb850108,0x881e0101,0xfb850108,0x881f0101,0xfb850108,0x88200101,0xfb850108,0x88210101,
	0xfb850108,0x88220101,0xfb850108,0x88230101,0xfb850108,0x88240101,0xfb850108,0x88250101,
	0xfb850108,0x88260101,0xfb850108,0x88270101,0xfb850108,0x88280101,0xfb850108,0x88290101,
	0xfb850108,0x882a0101,0xfb850108,0x882b0101,0xfb850108,0x882c0101,0xfb850108,0x882d0101,
	0xfb850108,0x882e0101,0xfb850108,0x882f0101,0xfb850108,0x88300101,0xfb850108,0x88310101,
	0xfb850108,0x88320101,0xfb850108,0x88330101,0xfb850108,0x88340101,0xfb850108,0x88350101,
	0xfb850108,0x88360101,0xfb850108,0x88370101,0xfb850108,0x88380101,0xfb850108,0x88390101,
	0xfb850108,0x883a0101,0xfb850108,0x883b0101,0xfb850108,0x883c0101,0xfb850108,0x883d0101,
	0xfb850108,0x883e0101,0xfb850108,0x883f0101,0xfb850108,0x88400101,0xfb850108,0x88410101,
	0xfb850108,0x88420101,0xfb850108,0x88430101,0xfb850108,0x88440101,0xfb850108,0x88450101,
	0xfb850108,0x88460101,0xfb850108,0x88470101,0xfb850108,0x88480101,0xfb850108,0x88490101,
	0xfb850108,0x884a0101,0xfb850108,0x884b0101,0xfb850108,0x884c0101,0xfb850108,0x884d0101,
	0xfb850108,0x884e0101,0xfb850108,0x884f0101,0xfb850108,0x88500101,0xfb850108,0x88510101,
	0xfb850108,0x88520101,0xfb850108,0x88530101,0xfb850108,0x88540101,0xfb850108,0x88550101,
	0xfb850108,0x88560101,0xfb850108,0x88570101,0xfb850108,0x88580101,0xfb850108,0x88590101,
	0xfb850108,0x885a0101,0xfb850108,0x885b0101,0xfb850108,0x885c0101,0xfb850108,0x885d0101,
	0xfb850108,0x885e0101,0xfb850108,0x885f0101,0xfb850108,0x88600101,0xfb850108,0x88610101,
	0xfb850108,0x88620101,0xfb850108,0x88630101,0xfb850108,0x88640101,0xfb850108,0x88650101,
	0xfb850108,0x88660101,0xfb850108,0x88670101,0xfb850108,0x88680101,0xfb850108,0x88690101,
	0xfb850108,0x886a0101,0xfb850108,0x886b0101,0xfb850108,0x886c0101,0xfb850108,0x886d0101,
	0xfb850108,0x886e0101,0xfb850108,0x886f0101,0xfb850108,0x88700101,0xfb850108,0x88710101,
	0xfb850108,0x88720101,0xfb850108,0x88730101,0xfb850108,0x88740101,0xfb850108,0x88750101,
	0xfb850108,0x88760101,0xfb850108,0x88770101,0xfb850108,0x88780101,0xfb850108,0x88790101,
	0xfb850108,0x887a0101,0xfb850108,0x887b0101,0xfb850108,0x887c0101,0xfb850108,0x887d0101,
	0xfb850108,0x887e0101,0xfb850108,0x887f0101,0xfb850108,0x88800101,0xfb850108,0x88810101,
	0xfb850108,0x88820101,0xfb850108,0x88830101,0xfb850108,0x88840101,0xfb850108,0x88850101,
	0xfb850108,0x88860101,0xfb850108,0x88870101,0xfb850108,0x88880101,0xfb850108,0x88890101,
	0xfb850108,0x888a0101,0xfb850108,0x888b0101,0xfb850108,0x888c0101,0xfb850108,0x888d0101,
	0xfb850108,0x888e0101,0xfb850108,0x888f0101,0xfb850108,0x88900101,0xfb850108,0x88910101,
	0xfb850108,0x88920101,0xfb850108,0x88930101,0xfb850108,0x88940101,0xfb850108,0x88950101,
	0xfb850108,0x88960101,0xfb850108,0x88970101,0xfb850108,0x88980101,0xfb850108,0x88990101,
	0xfb850108,0x889a0101,0xfb850108,0x889b0101,0xfb850108,0x889c0101,0xfb850108,0x889d0101,
	0xfb850108,0x889e0101,0xfb850108,0x889f0101,0xfb850108,0x88a00101,0xfb850108,0x88a10101,
	0xfb850108,0x88a20101,0xfb850108,0x88a30101,0xfb850108,0x88a40101,0xfb850108,0x88a50101,
	0xfb850108,0x88a60101,0xfb850108,0x88a70101,0xfb850108,0x88a80101,0xfb850108,0x88a90101,
	0xfb850108,0x88aa0101,0xfb850108,0x88ab0101,0xfb850108,0x88ac0101,0xfb850108,0x88ad0101,
	0xfb850108,0x88ae0101,0xfb850108,0x88af0101,0xfb850108,0x88b00101,0xfb850108,0x88b10101,
	0xfb850108,0x88b20101,0xfb850108,0x88b30101,0xfb850108,0x88b40101,0xfb850108,0x88b50101,
	0xfb850108,0x88b60101,0xfb850108,0x88b70101,0xfb850108,0x88b80101,0xfb850108,0x88b90101,
	0xfb850108,0x88ba0101,0xfb850108,0x88bb0101,0xfb850108,0x88bc0101,0xfb850108,0x88bd0101,
	0xfb850108,0x88be0101,0xfb850108,0x88bf0101,0xfb850108,0x88c00101,0xfb850108,0x88c10101,
	0xfb850108,0x88c20101,0xfb850108,0x88c30101,0xfb850108,0x88c40101,0xfb850108,0x88c50101,
	0xfb850108,0x88c60101,0xfb850108,0x88c70101,0xfb850108,0x88c80101,0xfb850108,0x88c90101,
	0xfb850108,0x88ca0101,0xfb850108,0x88cb0101,0xfb850108,0x88cc0101,0xfb850108,0x88cd0101,
	0xfb850108,0x88ce0101,0xfb850108,0x88cf0101,0xfb850108,0x88d00101,0xfb850108,0x88d10101,
	0xfb850108,0x88d20101,0xfb850108,0x88d30101,0xfb850108,0x88d40101,0xfb850108,0x88d50101,
	0xfb850108,0x88d60101,0xfb850108,0x88d70101,0xfb850108,0x88d80101,0xfb850108,0x88d90101,
	0xfb850108,0x88da0101,0xfb850108,0x88db0101,0xfb850108,0x88dc0101,0xfb850108,0x88dd0101,
	0xfb850108,0x88de0101,0xfb850108,0x88df0101,0xfb850108,0x88e00101,0xfb850108,0x88e10101,
	0xfb850108,0x88e20101,0xfb850108,0x88e30101,0xfb850108,0x88e40101,0xfb850108,0x88e50101,
	0xfb850108,0x88e60101,0xfb850108,0x88e70101,0xfb850108,0x88e80101,0xfb850108,0x88e90101,
	0xfb850108,0x88ea0101,0xfb850108,0x88eb0101,0xfb850108,0x88ec0101,0xfb850108,0x88ed0101,
	0xfb850108,0x88ee0101,0xfb850108,0x88ef0101,0xfb850108,0x88f00101,0xfb850108,0x88f10101,
	0xfb850108,0x88f20101,0xfb850108,0x88f30101,0xfb850108,0x88f40101,0xfb850108,0x88f50101,
	0xfb850108,0x88f60101,0xfb850108,0x88f70101,0xfb850108,0x88f80101,0xfb850108,0x88f90101,
	0xfb850108,0x88fa0101,0xfb850108,0x88fb0101,0xfb850108,0x88fc0101,0xfb850108,0x88fd0101,
	0xfb850108,0x88fe0101,0xfb850108,0x88ff0101,0xfb850108,0x89000101,0xfb850108,0x89010101,
	0xfb850108,0x89020101,0xfb850108,0x89030101,0xfb850108,0x89040101,0xfb850108,0x89050101,
	0xfb850108,0x89060101,0xfb850108,0x89070101,0xfb850108,0x89080101,0xfb850108,0x89090101,
	0xfb850108,0x890a0101,0xfb850108,0x890b0101,0xfb850108,0x890c0101,0xfb850108,0x890d0101,
	0xfb850108,0x890e0101,0xfb850108,0x890f0101,0xfb850108,0x89100101,0xfb850108,0x89110101,
	0xfb850108,0x89120101,0xfb850108,0x89130101,0xfb850108,0x89140101,0xfb850108,0x89150101,
	0xfb850108,0x89160101,0xfb850108,0x89170101,0xfb850108,0x89180101,0xfb850108,0x89190101,
	0xfb850108,0x891a0101,0xfb850108,0x891b0101,0xfb850108,0x891c0101,0xfb850108,0x891d0101,
	0xfb850108,0x891e0101,0xfb850108,0x891f0101,0xfb850108,0x89200101,0xfb850108,0x89210101,
	0xfb850108,0x89220101,0xfb850108,0x89230101,0xfb850108,0x89240101,0xfb850108,0x89250101,
	0xfb850108,0x89260101,0xfb850108,0x89270101,0xfb850108,0x89280101,0xfb850108,0x89290101,
	0xfb850108,0x892a0101,0xfb850108,0x892b0101,0xfb850108,0x892c0101,0xfb850108,0x892d0101,
	0xfb850108,0x892e0101,0xfb850108,0x892f0101,0xfb850108,0x89300101,0xfb850108,0x89310101,
	0xfb850108,0x89320101,0xfb850108,0x89330101,0xfb850108,0x89340101,0xfb850108,0x89350101,
	0xfb850108,0x89360101,0xfb850108,0x89370101,0xfb850108,0x89380101,0xfb850108,0x89390101,
	0xfb850108,0x893a0101,0xfb850108,0x893b0101,0xfb850108,0x893c0101,0xfb850108,0x893d0101,
	0xfb850108,0x893e0101,0xfb850108,0x893f0101,0xfb850108,0x89400101,0xfb850108,0x89410101,
	0xfb850108,0x89420101,0xfb850108,0x89430101,0xfb850108,0x89440101,0xfb850108,0x89450101,
	0xfb850108,0x89460101,0xfb850108,0x89470101,0xfb850108,0x89480101,0xfb850108,0x89490101,
	0xfb850108,0x894a0101,0xfb850108,0x894b0101,0xfb850108,0x894c0101,0xfb850108,0x894d0101,
	0xfb850108,0x894e0101,0xfb850108,0x894f0101,0xfb850108,0x89500101,0xfb850108,0x89510101,
	0xfb850108,0x89520101,0xfb850108,0x89530101,0xfb850108,0x89540101,0xfb850108,0x89550101,
	0xfb850108,0x89560101,0xfb850108,0x89570101,0xfb850108,0x89580101,0xfb850108,0x89590101,
	0xfb850108,0x895a0101,0xfb850108,0x895b0101,0xfb850108,0x895c0101,0xfb850108,0x895d0101,
	0xfb850108,0x895e0101,0xfb850108,0x895f0101,0xfb850108,0x89600101,0xfb850108,0x89610101,
	0xfb850108,0x89620101,0xfb850108,0x89630101,0xfb850108,0x89640101,0xfb850108,0x89650101,
	0xfb850108,0x89660101,0xfb850108,0x89670101,0xfb850108,0x89680101,0xfb850108,0x89690101,
	0xfb850108,0x896a0101,0xfb850108,0x896b0101,0xfb850108,0x896c0101,0xfb850108,0x896d0101,
	0xfb850108,0x896e0101,0xfb850108,0x896f0101,0xfb850108,0x89700101,0xfb850108,0x89710101,
	0xfb850108,0x89720101,0xfb850108,0x89730101,0xfb850108,0x89740101,0xfb850108,0x89750101,
	0xfb850108,0x89760101,0xfb850108,0x89770101,0xfb850108,0x89780101,0xfb850108,0x89790101,
	0xfb850108,0x897a0101,0xfb850108,0x897b0101,0xfb850108,0x897c0101,0xfb850108,0x897d0101,
	0xfb850108,0x897e0101,0xfb850108,0x897f0101,0xfb850108,0x89800101,0xfb850108,0x89810101,
	0xfb850108,0x89820101,0xfb850108,0x89830101,0xfb850108,0x89840101,0xfb850108,0x89850101,
	0xfb850108,0x89860101,0xfb850108,0x89870101,0xfb850108,0x89880101,0xfb850108,0x89890101,
	0xfb850108,0x898a0101,0xfb850108,0x898b0101,0xfb850108,0x898c0101,0xfb850108,0x898d0101,
	0xfb850108,0x898e0101,0xfb850108,0x898f0101,0xfb850108,0x89900101,0xfb850108,0x89910101,
	0xfb850108,0x89920101,0xfb850108,0x89930101,0xfb850108,0x89940101,0xfb850108,0x89950101,
	0xfb850108,0x89960101,0xfb850108,0x89970101,0xfb850108,0x89980101,0xfb850108,0x89990101,
	0xfb850108,0x899a0101,0xfb850108,0x899b0101,0xfb850108,0x899c0101,0xfb850108,0x899d0101,
	0xfb850108,0x899e0101,0xfb850108,0x899f0101,0xfb850108,0x89a00101,0xfb850108,0x89a10101,
	0xfb850108,0x89a20101,0xfb850108,0x89a30101,0xfb850108,0x89a40101,0xfb850108,0x89a50101,
	0xfb850108,0x89a60101,0xfb850108,0x89a70101,0xfb850108,0x89a80101,0xfb850108,0x89a90101,
	0xfb850108,0x89aa0101,0xfb850108,0x89ab0101,0xfb850108,0x89ac0101,0xfb850108,0x89ad0101,
	0xfb850108,0x89ae0101,0xfb850108,0x89af0101,0xfb850108,0x89b00101,0xfb850108,0x89b10101,
	0xfb850108,0x89b20101,0xfb850108,0x89b30101,0xfb850108,0x89b40101,0xfb850108,0x89b50101,
	0xfb850108,0x89b60101,0xfb850108,0x89b70101,0xfb850108,0x89b80101,0xfb850108,0x89b90101,
	0xfb850108,0x89ba0101,0xfb850108,0x89bb0101,0xfb850108,0x89bc0101,0xfb850108,0x89bd0101,
	0xfb850108,0x89be0101,0xfb850108,0x89bf0101,0xfb850108,0x89c00101,0xfb850108,0x89c10101,
	0xfb850108,0x89c20101,0xfb850108,0x89c30101,0xfb850108,0x89c40101,0xfb850108,0x89c50101,
	0xfb850108,0x89c60101,0xfb850108,0x89c70101,0xfb850108,0x89c80101,0xfb850108,0x89c90101,
	0xfb850108,0x89ca0101,0xfb850108,0x89cb0101,0xfb850108,0x89cc0101,0xfb850108,0x89cd0101,
	0xfb850108,0x89ce0101,0xfb850108,0x89cf0101,0xfb850108,0x89d00101,0xfb850108,0x89d10101,
	0xfb850108,0x89d20101,0xfb850108,0x89d30101,0xfb850108,0x89d40101,0xfb850108,0x89d50101,
	0xfb850108,0x89d60101,0xfb850108,0x89d70101,0xfb850108,0x89d80101,0xfb850108,0x89d90101,
	0xfb850108,0x89da0101,0xfb850108,0x89db0101,0xfb850108,0x89dc0101,0xfb850108,0x89dd0101,
	0xfb850108,0x89de0101,0xfb850108,0x89df0101,0xfb850108,0x89e00101,0xfb850108,0x89e10101,
	0xfb850108,0x89e20101,0xfb850108,0x89e30101,0xfb850108,0x89e40101,0xfb850108,0x89e50101,
	0xfb850108,0x89e60101,0xfb850108,0x89e70101,0xfb850108,0x89e80101,0xfb850108,0x89e90101,
	0xfb850108,0x89ea0101,0xfb850108,0x89eb0101,0xfb850108,0x89ec0101,0xfb850108,0x89ed0101,
	0xfb850108,0x89ee0101,0xfb850108,0x89ef0101,0xfb850108,0x89f00101,0xfb850108,0x89f10101,
	0xfb850108,0x89f20101,0xfb850108,0x89f30101,0xfb850108,0x89f40101,0xfb850108,0x89f50101,
	0xfb850108,0x89f60101,0xfb850108,0x89f70101,0xfb850108,0x89f80101,0xfb850108,0x89f90101,
	0xfb850108,0x89fa0101,0xfb850108,0x89fb0101,0xfb850108,0x89fc0101,0xfb850108,0x89fd0101,
	0xfb850108,0x89fe0101,0xfb850108,0x89ff0101,0xfb850108,0x8a000101,0xfb850108,0x8a010101,
	0xfb850108,0x8a020101,0xfb850108,0x8a030101,0xfb850108,0x8a040101,0xfb850108,0x8a050101,
	0xfb850108,0x8a060101,0xfb850108,0x8a070101,0xfb850108,0x8a080101,0xfb850108,0x8a090101,
	0xfb850108,0x8a0a0101,0xfb850108,0x8a0b0101,0xfb850108,0x8a0c0101,0xfb850108,0x8a0d0101,
	0xfb850108,0x8a0e0101,0xfb850108,0x8a0f0101,0xfb850108,0x8a100101,0xfb850108,0x8a110101,
	0xfb850108,0x8a120101,0xfb850108,0x8a130101,0xfb850108,0x8a140101,0xfb850108,0x8a150101,
	0xfb850108,0x8a160101,0xfb850108,0x8a170101,0xfb850108,0x8a180101,0xfb850108,0x8a190101,
	0xfb850108,0x8a1a0101,0xfb850108,0x8a1b0101,0xfb850108,0x8a1c0101,0xfb850108,0x8a1d0101,
	0xfb850108,0x8a1e0101,0xfb850108,0x8a1f0101,0xfb850108,0x8a200101,0xfb850108,0x8a210101,
	0xfb850108,0x8a220101,0xfb850108,0x8a230101,0xfb850108,0x8a240101,0xfb850108,0x8a250101,
	0xfb850108,0x8a260101,0xfb850108,0x8a270101,0xfb850108,0x8a280101,0xfb850108,0x8a290101,
	0xfb850108,0x8a2a0101,0xfb850108,0x8a2b0101,0xfb850108,0x8a2c0101,0xfb850108,0x8a2d0101,
	0xfb850108,0x8a2e0101,0xfb850108,0x8a2f0101,0xfb850108,0x8a300101,0xfb850108,0x8a310101,
	0xfb850108,0x8a320101,0xfb850108,0x8a330101,0xfb850108,0x8a340101,0xfb850108,0x8a350101,
	0xfb850108,0x8a360101,0xfb850108,0x8a370101,0xfb850108,0x8a380101,0xfb850108,0x8a390101,
	0xfb850108,0x8a3a0101,0xfb850108,0x8a3b0101,0xfb850108,0x8a3c0101,0xfb850108,0x8a3d0101,
	0xfb850108,0x8a3e0101,0xfb850108,0x8a3f0101,0xfb850108,0x8a400101,0xfb850108,0x8a410101,
	0xfb850108,0x8a420101,0xfb850108,0x8a430101,0xfb850108,0x8a440101,0xfb850108,0x8a450101,
	0xfb850108,0x8a460101,0xfb850108,0x8a470101,0xfb850108,0x8a480101,0xfb850108,0x8a490101,
	0xfb850108,0x8a4a0101,0xfb850108,0x8a4b0101,0xfb850108,0x8a4c0101,0xfb850108,0x8a4d0101,
	0xfb850108,0x8a4e0101,0xfb850108,0x8a4f0101,0xfb850108,0x8a500101,0xfb850108,0x8a510101,
	0xfb850108,0x8a520101,0xfb850108,0x8a530101,0xfb850108,0x8a540101,0xfb850108,0x8a550101,
	0xfb850108,0x8a560101,0xfb850108,0x8a570101,0xfb850108,0x8a580101,0xfb850108,0x8a590101,
	0xfb850108,0x8a5a0101,0xfb850108,0x8a5b0101,0xfb850108,0x8a5c0101,0xfb850108,0x8a5d0101,
	0xfb850108,0x8a5e0101,0xfb850108,0x8a5f0101,0xfb850108,0x8a600101,0xfb850108,0x8a610101,
	0xfb850108,0x8a620101,0xfb850108,0x8a630101,0xfb850108,0x8a640101,0xfb850108,0x8a650101,
	0xfb850108,0x8a660101,0xfb850108,0x8a670101,0xfb850108,0x8a680101,0xfb850108,0x8a690101,
	0xfb850108,0x8a6a0101,0xfb850108,0x8a6b0101,0xfb850108,0x8a6c0101,0xfb850108,0x8a6d0101,
	0xfb850108,0x8a6e0101,0xfb850108,0x8a6f0101,0xfb850108,0x8a700101,0xfb850108,0x8a710101,
	0xfb850108,0x8a720101,0xfb850108,0x8a730101,0xfb850108,0x8a740101,0xfb850108,0x8a750101,
	0xfb850108,0x8a760101,0xfb850108,0x8a770101,0xfb850108,0x8a780101,0xfb850108,0x8a790101,
	0xfb850108,0x8a7a0101,0xfb850108,0x8a7b0101,0xfb850108,0x8a7c0101,0xfb850108,0x8a7d0101,
	0xfb850108,0x8a7e0101,0xfb850108,0x8a7f0101,0xfb850108,0x8a800101,0xfb850108,0x8a810101,
	0xfb850108,0x8a820101,0xfb850108,0x8a830101,0xfb850108,0x8a840101,0xfb850108,0x8a850101,
	0xfb850108,0x8a860101,0xfb850108,0x8a870101,0xfb850108,0x8a880101,0xfb850108,0x8a890101,
	0xfb850108,0x8a8a0101,0xfb850108,0x8a8b0101,0xfb850108,0x8a8c0101,0xfb850108,0x8a8d0101,
	0xfb850108,0x8a8e0101,0xfb850108,0x8a8f0101,0xfb850108,0x8a900101,0xfb850108,0x8a910101,
	0xfb850108,0x8a920101,0xfb850108,0x8a930101,0xfb850108,0x8a940101,0xfb850108,0x8a950101,
	0xfb850108,0x8a960101,0xfb850108,0x8a970101,0xfb850108,0x8a980101,0xfb850108,0x8a990101,
	0xfb850108,0x8a9a0101,0xfb850108,0x8a9b0101,0xfb850108,0x8a9c0101,0xfb850108,0x8a9d0101,
	0xfb850108,0x8a9e0101,0xfb850108,0x8a9f0101,0xfb850108,0x8aa00101,0xfb850108,0x8aa10101,
	0xfb850108,0x8aa20101,0xfb850108,0x8aa30101,0xfb850108,0x8aa40101,0xfb850108,0x8aa50101,
	0xfb850108,0x8aa60101,0xfb850108,0x8aa70101,0xfb850108,0x8aa80101,0xfb850108,0x8aa90101,
	0xfb850108,0x8aaa0101,0xfb850108,0x8aab0101,0xfb850108,0x8aac0101,0xfb850108,0x8aad0101,
	0xfb850108,0x8aae0101,0xfb850108,0x8aaf0101,0xfb850108,0x8ab00101,0xfb850108,0x8ab10101,
	0xfb850108,0x8ab20101,0xfb850108,0x8ab30101,0xfb850108,0x8ab40101,0xfb850108,0x8ab50101,
	0xfb850108,0x8ab60101,0xfb850108,0x8ab70101,0xfb850108,0x8ab80101,0xfb850108,0x8ab90101,
	0xfb850108,0x8aba0101,0xfb850108,0x8abb0101,0xfb850108,0x8abc0101,0xfb850108,0x8abd0101,
	0xfb850108,0x8abe0101,0xfb850108,0x8abf0101,0xfb850108,0x8ac00101,0xfb850108,0x8ac10101,
	0xfb850108,0x8ac20101,0xfb850108,0x8ac30101,0xfb850108,0x8ac40101,0xfb850108,0x8ac50101,
	0xfb850108,0x8ac60101,0xfb850108,0x8ac70101,0xfb850108,0x8ac80101,0xfb850108,0x8ac90101,
	0xfb850108,0x8aca0101,0xfb850108,0x8acb0101,0xfb850108,0x8acc0101,0xfb850108,0x8acd0101,
	0xfb850108,0x8ace0101,0xfb850108,0x8acf0101,0xfb850108,0x8ad00101,0xfb850108,0x8ad10101,
	0xfb850108,0x8ad20101,0xfb850108,0x8ad30101,0xfb850108,0x8ad40101,0xfb850108,0x8ad50101,
	0xfb850108,0x8ad60101,0xfb850108,0x8ad70101,0xfb850108,0x8ad80101,0xfb850108,0x8ad90101,
	0xfb850108,0x8ada0101,0xfb850108,0x8adb0101,0xfb850108,0x8adc0101,0xfb850108,0x8add0101,
	0xfb850108,0x8ade0101,0xfb850108,0x8adf0101,0xfb850108,0x8ae00101,0xfb850108,0x8ae10101,
	0xfb850108,0x8ae20101,0xfb850108,0x8ae30101,0xfb850108,0x8ae40101,0xfb850108,0x8ae50101,
	0xfb850108,0x8ae60101,0xfb850108,0x8ae70101,0xfb850108,0x8ae80101,0xfb850108,0x8ae90101,
	0xfb850108,0x8aea0101,0xfb850108,0x8aeb0101,0xfb850108,0x8aec0101,0xfb850108,0x8aed0101,
	0xfb850108,0x8aee0101,0xfb850108,0x8aef0101,0xfb850108,0x8af00101,0xfb850108,0x8af10101,
	0xfb850108,0x8af20101,0xfb850108,0x8af30101,0xfb850108,0x8af40101,0xfb850108,0x8af50101,
	0xfb850108,0x8af60101,0xfb850108,0x8af70101,0xfb850108,0x8af80101,0xfb850108,0x8af90101,
	0xfb850108,0x8afa0101,0xfb850108,0x8afb0101,0xfb850108,0x8afc0101,0xfb850108,0x8afd0101,
	0xfb850108,0x8afe0101,0xfb850108,0x8aff0101,0xfb850108,0x8b000101,0xfb850108,0x8b010101,
	0xfb850108,0x8b020101,0xfb850108,0x8b030101,0xfb850108,0x8b040101,0xfb850108,0x8b050101,
	0xfb850108,0x8b060101,0xfb850108,0x8b070101,0xfb850108,0x8b080101,0xfb850108,0x8b090101,
	0xfb850108,0x8b0a0101,0xfb850108,0x8b0b0101,0xfb850108,0x8b0c0101,0xfb850108,0x8b0d0101,
	0xfb850108,0x8b0e0101,0xfb850108,0x8b0f0101,0xfb850108,0x8b100101,0xfb850108,0x8b110101,
	0xfb850108,0x8b120101,0xfb850108,0x8b130101,0xfb850108,0x8b140101,0xfb850108,0x8b150101,
	0xfb850108,0x8b160101,0xfb850108,0x8b170101,0xfb850108,0x8b180101,0xfb850108,0x8b190101,
	0xfb850108,0x8b1a0101,0xfb850108,0x8b1b0101,0xfb850108,0x8b1c0101,0xfb850108,0x8b1d0101,
	0xfb850108,0x8b1e0101,0xfb850108,0x8b1f0101,0xfb850108,0x8b200101,0xfb850108,0x8b210101,
	0xfb850108,0x8b220101,0xfb850108,0x8b230101,0xfb850108,0x8b240101,0xfb850108,0x8b250101,
	0xfb850108,0x8b260101,0xfb850108,0x8b270101,0xfb850108,0x8b280101,0xfb850108,0x8b290101,
	0xfb850108,0x8b2a0101,0xfb850108,0x8b2b0101,0xfb850108,0x8b2c0101,0xfb850108,0x8b2d0101,
	0xfb850108,0x8b2e0101,0xfb850108,0x8b2f0101,0xfb850108,0x8b300101,0xfb850108,0x8b310101,
	0xfb850108,0x8b320101,0xfb850108,0x8b330101,0xfb850108,0x8b340101,0xfb850108,0x8b350101,
	0xfb850108,0x8b360101,0xfb850108,0x8b370101,0xfb850108,0x8b380101,0xfb850108,0x8b390101,
	0xfb850108,0x8b3a0101,0xfb850108,0x8b3b0101,0xfb850108,0x8b3c0101,0xfb850108,0x8b3d0101,
	0xfb850108,0x8b3e0101,0xfb850108,0x8b3f0101,0xfb850108,0x8b400101,0xfb850108,0x8b410101,
	0xfb850108,0x8b420101,0xfb850108,0x8b430101,0xfb850108,0x8b440101,0xfb850108,0x8b450101,
	0xfb850108,0x8b460101,0xfb850108,0x8b470101,0xfb850108,0x8b480101,0xfb850108,0x8b490101,
	0xfb850108,0x8b4a0101,0xfb850108,0x8b4b0101,0xfb850108,0x8b4c0101,0xfb850108,0x8b4d0101,
	0xfb850108,0x8b4e0101,0xfb850108,0x8b4f0101,0xfb850108,0x8b500101,0xfb850108,0x8b510101,
	0xfb850108,0x8b520101,0xfb850108,0x8b530101,0xfb850108,0x8b540101,0xfb850108,0x8b550101,
	0xfb850108,0x8b560101,0xfb850108,0x8b570101,0xfb850108,0x8b580101,0xfb850108,0x8b590101,
	0xfb850108,0x8b5a0101,0xfb850108,0x8b5b0101,0xfb850108,0x8b5c0101,0xfb850108,0x8b5d0101,
	0xfb850108,0x8b5e0101,0xfb850108,0x8b5f0101,0xfb850108,0x8b600101,0xfb850108,0x8b610101,
	0xfb850108,0x8b620101,0xfb850108,0x8b630101,0xfb850108,0x8b640101,0xfb850108,0x8b650101,
	0xfb850108,0x8b660101,0xfb850108,0x8b670101,0xfb850108,0x8b680101,0xfb850108,0x8b690101,
	0xfb850108,0x8b6a0101,0xfb850108,0x8b6b0101,0xfb850108,0x8b6c0101,0xfb850108,0x8b6d0101,
	0xfb850108,0x8b6e0101,0xfb850108,0x8b6f0101,0xfb850108,0x8b700101,0xfb850108,0x8b710101,
	0xfb850108,0x8b720101,0xfb850108,0x8b730101,0xfb850108,0x8b740101,0xfb850108,0x8b750101,
	0xfb850108,0x8b760101,0xfb850108,0x8b770101,0xfb850108,0x8b780101,0xfb850108,0x8b790101,
	0xfb850108,0x8b7a0101,0xfb850108,0x8b7b0101,0xfb850108,0x8b7c0101,0xfb850108,0x8b7d0101,
	0xfb850108,0x8b7e0101,0xfb850108,0x8b7f0101,0xfb850108,0x8b800101,0xfb850108,0x8b810101,
	0xfb850108,0x8b820101,0xfb850108,0x8b830101,0xfb850108,0x8b840101,0xfb850108,0x8b850101,
	0xfb850108,0x8b860101,0xfb850108,0x8b870101,0xfb850108,0x8b880101,0xfb850108,0x8b890101,
	0xfb850108,0x8b8a0101,0xfb850108,0x8b8b0101,0xfb850108,0x8b8c0101,0xfb850108,0x8b8d0101,
	0xfb850108,0x8b8e0101,0xfb850108,0x8b8f0101,0xfb850108,0x8b900101,0xfb850108,0x8b910101,
	0xfb850108,0x8b920101,0xfb850108,0x8b930101,0xfb850108,0x8b940101,0xfb850108,0x8b950101,
	0xfb850108,0x8b960101,0xfb850108,0x8b970101,0xfb850108,0x8b980101,0xfb850108,0x8b990101,
	0xfb850108,0x8b9a0101,0xfb850108,0x8b9b0101,0xfb850108,0x8b9c0101,0xfb850108,0x8b9d0101,
	0xfb850108,0x8b9e0101,0xfb850108,0x8b9f0101,0xfb850108,0x8ba00101,0xfb850108,0x8ba10101,
	0xfb850108,0x8ba20101,0xfb850108,0x8ba30101,0xfb850108,0x8ba40101,0xfb850108,0x8ba50101,
	0xfb850108,0x8ba60101,0xfb850108,0x8ba70101,0xfb850108,0x8ba80101,0xfb850108,0x8ba90101,
	0xfb850108,0x8baa0101,0xfb850108,0x8bab0101,0xfb850108,0x8bac0101,0xfb850108,0x8bad0101,
	0xfb850108,0x8bae0101,0xfb850108,0x8baf0101,0xfb850108,0x8bb00101,0xfb850108,0x8bb10101,
	0xfb850108,0x8bb20101,0xfb850108,0x8bb30101,0xfb850108,0x8bb40101,0xfb850108,0x8bb50101,
	0xfb850108,0x8bb60101,0xfb850108,0x8bb70101,0xfb850108,0x8bb80101,0xfb850108,0x8bb90101,
	0xfb850108,0x8bba0101,0xfb850108,0x8bbb0101,0xfb850108,0x8bbc0101,0xfb850108,0x8bbd0101,
	0xfb850108,0x8bbe0101,0xfb850108,0x8bbf0101,0xfb850108,0x8bc00101,0xfb850108,0x8bc10101,
	0xfb850108,0x8bc20101,0xfb850108,0x8bc30101,0xfb850108,0x8bc40101,0xfb850108,0x8bc50101,
	0xfb850108,0x8bc60101,0xfb850108,0x8bc70101,0xfb850108,0x8bc80101,0xfb850108,0x8bc90101,
	0xfb850108,0x8bca0101,0xfb850108,0x8bcb0101,0xfb850108,0x8bcc0101,0xfb850108,0x8bcd0101,
	0xfb850108,0x8bce0101,0xfb850108,0x8bcf0101,0xfb850108,0x8bd00101,0xfb850108,0x8bd10101,
	0xfb850108,0x8bd20101,0xfb850108,0x8bd30101,0xfb850108,0x8bd40101,0xfb850108,0x8bd50101,
	0xfb850108,0x8bd60101,0xfb850108,0x8bd70101,0xfb850108,0x8bd80101,0xfb850108,0x8bd90101,
	0xfb850108,0x8bda0101,0xfb850108,0x8bdb0101,0xfb850108,0x8bdc0101,0xfb850108,0x8bdd0101,
	0xfb850108,0x8bde0101,0xfb850108,0x8bdf0101,0xfb850108,0x8be00101,0xfb850108,0x8be10101,
	0xfb850108,0x8be20101,0xfb850108,0x8be30101,0xfb850108,0x8be40101,0xfb850108,0x8be50101,
	0xfb850108,0x8be60101,0xfb850108,0x8be70101,0xfb850108,0x8be80101,0xfb850108,0x8be90101,
	0xfb850108,0x8bea0101,0xfb850108,0x8beb0101,0xfb850108,0x8bec0101,0xfb850108,0x8bed0101,
	0xfb850108,0x8bee0101,0xfb850108,0x8bef0101,0xfb850108,0x8bf00101,0xfb850108,0x8bf10101,
	0xfb850108,0x8bf20101,0xfb850108,0x8bf30101,0xfb850108,0x8bf40101,0xfb850108,0x8bf50101,
	0xfb850108,0x8bf60101,0xfb850108,0x8bf70101,0xfb850108,0x8bf80101,0xfb850108,0x8bf90101,
	0xfb850108,0x8bfa0101,0xfb850108,0x8bfb0101,0xfb850108,0x8bfc0101,0xfb850108,0x8bfd0101,
	0xfb850108,0x8bfe0101,0xfb850108,0x8bff0101,0xfb850108,0x8c000101,0xfb850108,0x8c010101,
	0xfb850108,0x8c020101,0xfb850108,0x8c030101,0xfb850108,0x8c040101,0xfb850108,0x8c050101,
	0xfb850108,0x8c060101,0xfb850108,0x8c070101,0xfb850108,0x8c080101,0xfb850108,0x8c090101,
	0xfb850108,0x8c0a0101,0xfb850108,0x8c0b0101,0xfb850108,0x8c0c0101,0xfb850108,0x8c0d0101,
	0xfb850108,0x8c0e0101,0xfb850108,0x8c0f0101,0xfb850108,0x8c100101,0xfb850108,0x8c110101,
	0xfb850108,0x8c120101,0xfb850108,0x8c130101,0xfb850108,0x8c140101,0xfb850108,0x8c150101,
	0xfb850108,0x8c160101,0xfb850108,0x8c170101,0xfb850108,0x8c180101,0xfb850108,0x8c190101,
	0xfb850108,0x8c1a0101,0xfb850108,0x8c1b0101,0xfb850108,0x8c1c0101,0xfb850108,0x8c1d0101,
	0xfb850108,0x8c1e0101,0xfb850108,0x8c1f0101,0xfb850108,0x8c200101,0xfb850108,0x8c210101,
	0xfb850108,0x8c220101,0xfb850108,0x8c230101,0xfb850108,0x8c240101,0xfb850108,0x8c250101,
	0xfb850108,0x8c260101,0xfb850108,0x8c270101,0xfb850108,0x8c280101,0xfb850108,0x8c290101,
	0xfb850108,0x8c2a0101,0xfb850108,0x8c2b0101,0xfb850108,0x8c2c0101,0xfb850108,0x8c2d0101,
	0xfb850108,0x8c2e0101,0xfb850108,0x8c2f0101,0xfb850108,0x8c300101,0xfb850108,0x8c310101,
	0xfb850108,0x8c320101,0xfb850108,0x8c330101,0xfb850108,0x8c340101,0xfb850108,0x8c350101,
	0xfb850108,0x8c360101,0xfb850108,0x8c370101,0xfb850108,0x8c380101,0xfb850108,0x8c390101,
	0xfb850108,0x8c3a0101,0xfb850108,0x8c3b0101,0xfb850108,0x8c3c0101,0xfb850108,0x8c3d0101,
	0xfb850108,0x8c3e0101,0xfb850108,0x8c3f0101,0xfb850108,0x8c400101,0xfb850108,0x8c410101,
	0xfb850108,0x8c420101,0xfb850108,0x8c430101,0xfb850108,0x8c440101,0xfb850108,0x8c450101,
	0xfb850108,0x8c460101,0xfb850108,0x8c470101,0xfb850108,0x8c480101,0xfb850108,0x8c490101,
	0xfb850108,0x8c4a0101,0xfb850108,0x8c4b0101,0xfb850108,0x8c4c0101,0xfb850108,0x8c4d0101,
	0xfb850108,0x8c4e0101,0xfb850108,0x8c4f0101,0xfb850108,0x8c500101,0xfb850108,0x8c510101,
	0xfb850108,0x8c520101,0xfb850108,0x8c530101,0xfb850108,0x8c540101,0xfb850108,0x8c550101,
	0xfb850108,0x8c560101,0xfb850108,0x8c570101,0xfb850108,0x8c580101,0xfb850108,0x8c590101,
	0xfb850108,0x8c5a0101,0xfb850108,0x8c5b0101,0xfb850108,0x8c5c0101,0xfb850108,0x8c5d0101,
	0xfb850108,0x8c5e0101,0xfb850108,0x8c5f0101,0xfb850108,0x8c600101,0xfb850108,0x8c610101,
	0xfb850108,0x8c620101,0xfb850108,0x8c630101,0xfb850108,0x8c640101,0xfb850108,0x8c650101,
	0xfb850108,0x8c660101,0xfb850108,0x8c670101,0xfb850108,0x8c680101,0xfb850108,0x8c690101,
	0xfb850108,0x8c6a0101,0xfb850108,0x8c6b0101,0xfb850108,0x8c6c0101,0xfb850108,0x8c6d0101,
	0xfb850108,0x8c6e0101,0xfb850108,0x8c6f0101,0xfb850108,0x8c700101,0xfb850108,0x8c710101,
	0xfb850108,0x8c720101,0xfb850108,0x8c730101,0xfb850108,0x8c740101,0xfb850108,0x8c750101,
	0xfb850108,0x8c760101,0xfb850108,0x8c770101,0xfb850108,0x8c780101,0xfb850108,0x8c790101,
	0xfb850108,0x8c7a0101,0xfb850108,0x8c7b0101,0xfb850108,0x8c7c0101,0xfb850108,0x8c7d0101,
	0xfb850108,0x8c7e0101,0xfb850108,0x8c7f0101,0xfb850108,0x8c800101,0xfb850108,0x8c810101,
	0xfb850108,0x8c820101,0xfb850108,0x8c830101,0xfb850108,0x8c840101,0xfb850108,0x8c850101,
	0xfb850108,0x8c860101,0xfb850108,0x8c870101,0xfb850108,0x8c880101,0xfb850108,0x8c890101,
	0xfb850108,0x8c8a0101,0xfb850108,0x8c8b0101,0xfb850108,0x8c8c0101,0xfb850108,0x8c8d0101,
	0xfb850108,0x8c8e0101,0xfb850108,0x8c8f0101,0xfb850108,0x8c900101,0xfb850108,0x8c910101,
	0xfb850108,0x8c920101,0xfb850108,0x8c930101,0xfb850108,0x8c940101,0xfb850108,0x8c950101,
	0xfb850108,0x8c960101,0xfb850108,0x8c970101,0xfb850108,0x8c980101,0xfb850108,0x8c990101,
	0xfb850108,0x8c9a0101,0xfb850108,0x8c9b0101,0xfb850108,0x8c9c0101,0xfb850108,0x8c9d0101,
	0xfb850108,0x8c9e0101,0xfb850108,0x8c9f0101,0xfb850108,0x8ca00101,0xfb850108,0x8ca10101,
	0xfb850108,0x8ca20101,0xfb850108,0x8ca30101,0xfb850108,0x8ca40101,0xfb850108,0x8ca50101,
	0xfb850108,0x8ca60101,0xfb850108,0x8ca70101,0xfb850108,0x8ca80101,0xfb850108,0x8ca90101,
	0xfb850108,0x8caa0101,0xfb850108,0x8cab0101,0xfb850108,0x8cac0101,0xfb850108,0x8cad0101,
	0xfb850108,0x8cae0101,0xfb850108,0x8caf0101,0xfb850108,0x8cb00101,0xfb850108,0x8cb10101,
	0xfb850108,0x8cb20101,0xfb850108,0x8cb30101,0xfb850108,0x8cb40101,0xfb850108,0x8cb50101,
	0xfb850108,0x8cb60101,0xfb850108,0x8cb70101,0xfb850108,0x8cb80101,0xfb850108,0x8cb90101,
	0xfb850108,0x8cba0101,0xfb850108,0x8cbb0101,0xfb850108,0x8cbc0101,0xfb850108,0x8cbd0101,
	0xfb850108,0x8cbe0101,0xfb850108,0x8cbf0101,0xfb850108,0x8cc00101,0xfb850108,0x8cc10101,
	0xfb850108,0x8cc20101,0xfb850108,0x8cc30101,0xfb850108,0x8cc40101,0xfb850108,0x8cc50101,
	0xfb850108,0x8cc60101,0xfb850108,0x8cc70101,0xfb850108,0x8cc80101,0xfb850108,0x8cc90101,
	0xfb850108,0x8cca0101,0xfb850108,0x8ccb0101,0xfb850108,0x8ccc0101,0xfb850108,0x8ccd0101,
	0xfb850108,0x8cce0101,0xfb850108,0x8ccf0101,0xfb850108,0x8cd00101,0xfb850108,0x8cd10101,
	0xfb850108,0x8cd20101,0xfb850108,0x8cd30101,0xfb850108,0x8cd40101,0xfb850108,0x8cd50101,
	0xfb850108,0x8cd60101,0xfb850108,0x8cd70101,0xfb850108,0x8cd80101,0xfb850108,0x8cd90101,
	0xfb850108,0x8cda0101,0xfb850108,0x8cdb0101,0xfb850108,0x8cdc0101,0xfb850108,0x8cdd0101,
	0xfb850108,0x8cde0101,0xfb850108,0x8cdf0101,0xfb850108,0x8ce00101,0xfb850108,0x8ce10101,
	0xfb850108,0x8ce20101,0xfb850108,0x8ce30101,0xfb850108,0x8ce40101,0xfb850108,0x8ce50101,
	0xfb850108,0x8ce60101,0xfb850108,0x8ce70101,0xfb850108,0x8ce80101,0xfb850108,0x8ce90101,
	0xfb850108,0x8cea0101,0xfb850108,0x8ceb0101,0xfb850108,0x8cec0101,0xfb850108,0x8ced0101,
	0xfb850108,0x8cee0101,0xfb850108,0x8cef0101,0xfb850108,0x8cf00101,0xfb850108,0x8cf10101,
	0xfb850108,0x8cf20101,0xfb850108,0x8cf30101,0xfb850108,0x8cf40101,0xfb850108,0x8cf50101,
	0xfb850108,0x8cf60101,0xfb850108,0x8cf70101,0xfb850108,0x8cf80101,0xfb850108,0x8cf90101,
	0xfb850108,0x8cfa0101,0xfb850108,0x8cfb0101,0xfb850108,0x8cfc0101,0xfb850108,0x8cfd0101,
	0xfb850108,0x8cfe0101,0xfb850108,0x8cff0101,0xfb850108,0x8d000101,0xfb850108,0x8d010101,
	0xfb850108,0x8d020101,0xfb850108,0x8d030101,0xfb850108,0x8d040101,0xfb850108,0x8d050101,
	0xfb850108,0x8d060101,0xfb850108,0x8d070101,0xfb850108,0x8d080101,0xfb850108,0x8d090101,
	0xfb850108,0x8d0a0101,0xfb850108,0x8d0b0101,0xfb850108,0x8d0c0101,0xfb850108,0x8d0d0101,
	0xfb850108,0x8d0e0101,0xfb850108,0x8d0f0101,0xfb850108,0x8d100101,0xfb850108,0x8d110101,
	0xfb850108,0x8d120101,0xfb850108,0x8d130101,0xfb850108,0x8d140101,0xfb850108,0x8d150101,
	0xfb850108,0x8d160101,0xfb850108,0x8d170101,0xfb850108,0x8d180101,0xfb850108,0x8d190101,
	0xfb850108,0x8d1a0101,0xfb850108,0x8d1b0101,0xfb850108,0x8d1c0101,0xfb850108,0x8d1d0101,
	0xfb850108,0x8d1e0101,0xfb850108,0x8d1f0101,0xfb850108,0x8d200101,0xfb850108,0x8d210101,
	0xfb850108,0x8d220101,0xfb850108,0x8d230101,0xfb850108,0x8d240101,0xfb850108,0x8d250101,
	0xfb850108,0x8d260101,0xfb850108,0x8d270101,0xfb850108,0x8d280101,0xfb850108,0x8d290101,
	0xfb850108,0x8d2a0101,0xfb850108,0x8d2b0101,0xfb850108,0x8d2c0101,0xfb850108,0x8d2d0101,
	0xfb850108,0x8d2e0101,0xfb850108,0x8d2f0101,0xfb850108,0x8d300101,0xfb850108,0x8d310101,
	0xfb850108,0x8d320101,0xfb850108,0x8d330101,0xfb850108,0x8d340101,0xfb850108,0x8d350101,
	0xfb850108,0x8d360101,0xfb850108,0x8d370101,0xfb850108,0x8d380101,0xfb850108,0x8d390101,
	0xfb850108,0x8d3a0101,0xfb850108,0x8d3b0101,0xfb850108,0x8d3c0101,0xfb850108,0x8d3d0101,
	0xfb850108,0x8d3e0101,0xfb850108,0x8d3f0101,0xfb850108,0x8d400101,0xfb850108,0x8d410101,
	0xfb850108,0x8d420101,0xfb850108,0x8d430101,0xfb850108,0x8d440101,0xfb850108,0x8d450101,
	0xfb850108,0x8d460101,0xfb850108,0x8d470101,0xfb850108,0x8d480101,0xfb850108,0x8d490101,
	0xfb850108,0x8d4a0101,0xfb850108,0x8d4b0101,0xfb850108,0x8d4c0101,0xfb850108,0x8d4d0101,
	0xfb850108,0x8d4e0101,0xfb850108,0x8d4f0101,0xfb850108,0x8d500101,0xfb850108,0x8d510101,
	0xfb850108,0x8d520101,0xfb850108,0x8d530101,0xfb850108,0x8d540101,0xfb850108,0x8d550101,
	0xfb850108,0x8d560101,0xfb850108,0x8d570101,0xfb850108,0x8d580101,0xfb850108,0x8d590101,
	0xfb850108,0x8d5a0101,0xfb850108,0x8d5b0101,0xfb850108,0x8d5c0101,0xfb850108,0x8d5d0101,
	0xfb850108,0x8d5e0101,0xfb850108,0x8d5f0101,0xfb850108,0x8d600101,0xfb850108,0x8d610101,
	0xfb850108,0x8d620101,0xfb850108,0x8d630101,0xfb850108,0x8d640101,0xfb850108,0x8d650101,
	0xfb850108,0x8d660101,0xfb850108,0x8d670101,0xfb850108,0x8d680101,0xfb850108,0x8d690101,
	0xfb850108,0x8d6a0101,0xfb850108,0x8d6b0101,0xfb850108,0x8d6c0101,0xfb850108,0x8d6d0101,
	0xfb850108,0x8d6e0101,0xfb850108,0x8d6f0101,0xfb850108,0x8d700101,0xfb850108,0x8d710101,
	0xfb850108,0x8d720101,0xfb850108,0x8d730101,0xfb850108,0x8d740101,0xfb850108,0x8d750101,
	0xfb850108,0x8d760101,0xfb850108,0x8d770101,0xfb850108,0x8d780101,0xfb850108,0x8d790101,
	0xfb850108,0x8d7a0101,0xfb850108,0x8d7b0101,0xfb850108,0x8d7c0101,0xfb850108,0x8d7d0101,
	0xfb850108,0x8d7e0101,0xfb850108,0x8d7f0101,0xfb850108,0x8d800101,0xfb850108,0x8d810101,
	0xfb850108,0x8d820101,0xfb850108,0x8d830101,0xfb850108,0x8d840101,0xfb850108,0x8d850101,
	0xfb850108,0x8d860101,0xfb850108,0x8d870101,0xfb850108,0x8d880101,0xfb850108,0x8d890101,
	0xfb850108,0x8d8a0101,0xfb850108,0x8d8b0101,0xfb850108,0x8d8c0101,0xfb850108,0x8d8d0101,
	0xfb850108,0x8d8e0101,0xfb850108,0x8d8f0101,0xfb850108,0x8d900101,0xfb850108,0x8d910101,
	0xfb850108,0x8d920101,0xfb850108,0x8d930101,0xfb850108,0x8d940101,0xfb850108,0x8d950101,
	0xfb850108,0x8d960101,0xfb850108,0x8d970101,0xfb850108,0x8d980101,0xfb850108,0x8d990101,
	0xfb850108,0x8d9a0101,0xfb850108,0x8d9b0101,0xfb850108,0x8d9c0101,0xfb850108,0x8d9d0101,
	0xfb850108,0x8d9e0101,0xfb850108,0x8d9f0101,0xfb850108,0x8da00101,0xfb850108,0x8da10101,
	0xfb850108,0x8da20101,0xfb850108,0x8da30101,0xfb850108,0x8da40101,0xfb850108,0x8da50101,
	0xfb850108,0x8da60101,0xfb850108,0x8da70101,0xfb850108,0x8da80101,0xfb850108,0x8da90101,
	0xfb850108,0x8daa0101,0xfb850108,0x8dab0101,0xfb850108,0x8dac0101,0xfb850108,0x8dad0101,
	0xfb850108,0x8dae0101,0xfb850108,0x8daf0101,0xfb850108,0x8db00101,0xfb850108,0x8db10101,
	0xfb850108,0x8db20101,0xfb850108,0x8db30101,0xfb850108,0x8db40101,0xfb850108,0x8db50101,
	0xfb850108,0x8db60101,0xfb850108,0x8db70101,0xfb850108,0x8db80101,0xfb850108,0x8db90101,
	0xfb850108,0x8dba0101,0xfb850108,0x8dbb0101,0xfb850108,0x8dbc0101,0xfb850108,0x8dbd0101,
	0xfb850108,0x8dbe0101,0xfb850108,0x8dbf0101,0xfb850108,0x8dc00101,0xfb850108,0x8dc10101,
	0xfb850108,0x8dc20101,0xfb850108,0x8dc30101,0xfb850108,0x8dc40101,0xfb850108,0x8dc50101,
	0xfb850108,0x8dc60101,0xfb850108,0x8dc70101,0xfb850108,0x8dc80101,0xfb850108,0x8dc90101,
	0xfb850108,0x8dca0101,0xfb850108,0x8dcb0101,0xfb850108,0x8dcc0101,0xfb850108,0x8dcd0101,
	0xfb850108,0x8dce0101,0xfb850108,0x8dcf0101,0xfb850108,0x8dd00101,0xfb850108,0x8dd10101,
	0xfb850108,0x8dd20101,0xfb850108,0x8dd30101,0xfb850108,0x8dd40101,0xfb850108,0x8dd50101,
	0xfb850108,0x8dd60101,0xfb850108,0x8dd70101,0xfb850108,0x8dd80101,0xfb850108,0x8dd90101,
	0xfb850108,0x8dda0101,0xfb850108,0x8ddb0101,0xfb850108,0x8ddc0101,0xfb850108,0x8ddd0101,
	0xfb850108,0x8dde0101,0xfb850108,0x8ddf0101,0xfb850108,0x8de00101,0xfb850108,0x8de10101,
	0xfb850108,0x8de20101,0xfb850108,0x8de30101,0xfb850108,0x8de40101,0xfb850108,0x8de50101,
	0xfb850108,0x8de60101,0xfb850108,0x8de70101,0xfb850108,0x8de80101,0xfb850108,0x8de90101,
	0xfb850108,0x8dea0101,0xfb850108,0x8deb0101,0xfb850108,0x8dec0101,0xfb850108,0x8ded0101,
	0xfb850108,0x8dee0101,0xfb850108,0x8def0101,0xfb850108,0x8df00101,0xfb850108,0x8df10101,
	0xfb850108,0x8df20101,0xfb850108,0x8df30101,0xfb850108,0x8df40101,0xfb850108,0x8df50101,
	0xfb850108,0x8df60101,0xfb850108,0x8df70101,0xfb850108,0x8df80101,0xfb850108,0x8df90101,
	0xfb850108,0x8dfa0101,0xfb850108,0x8dfb0101,0xfb850108,0x8dfc0101,0xfb850108,0x8dfd0101,
	0xfb850108,0x8dfe0101,0xfb850108,0x8dff0101,0xfb850108,0x8e000101,0xfb850108,0x8e010101,
	0xfb850108,0x8e020101,0xfb850108,0x8e030101,0xfb850108,0x8e040101,0xfb850108,0x8e050101,
	0xfb850108,0x8e060101,0xfb850108,0x8e070101,0xfb850108,0x8e080101,0xfb850108,0x8e090101,
	0xfb850108,0x8e0a0101,0xfb850108,0x8e0b0101,0xfb850108,0x8e0c0101,0xfb850108,0x8e0d0101,
	0xfb850108,0x8e0e0101,0xfb850108,0x8e0f0101,0xfb850108,0x8e100101,0xfb850108,0x8e110101,
	0xfb850108,0x8e120101,0xfb850108,0x8e130101,0xfb850108,0x8e140101,0xfb850108,0x8e150101,
	0xfb850108,0x8e160101,0xfb850108,0x8e170101,0xfb850108,0x8e180101,0xfb850108,0x8e190101,
	0xfb850108,0x8e1a0101,0xfb850108,0x8e1b0101,0xfb850108,0x8e1c0101,0xfb850108,0x8e1d0101,
	0xfb850108,0x8e1e0101,0xfb850108,0x8e1f0101,0xfb850108,0x8e200101,0xfb850108,0x8e210101,
	0xfb850108,0x8e220101,0xfb850108,0x8e230101,0xfb850108,0x8e240101,0xfb850108,0x8e250101,
	0xfb850108,0x8e260101,0xfb850108,0x8e270101,0xfb850108,0x8e280101,0xfb850108,0x8e290101,
	0xfb850108,0x8e2a0101,0xfb850108,0x8e2b0101,0xfb850108,0x8e2c0101,0xfb850108,0x8e2d0101,
	0xfb850108,0x8e2e0101,0xfb850108,0x8e2f0101,0xfb850108,0x8e300101,0xfb850108,0x8e310101,
	0xfb850108,0x8e320101,0xfb850108,0x8e330101,0xfb850108,0x8e340101,0xfb850108,0x8e350101,
	0xfb850108,0x8e360101,0xfb850108,0x8e370101,0xfb850108,0x8e380101,0xfb850108,0x8e390101,
	0xfb850108,0x8e3a0101,0xfb850108,0x8e3b0101,0xfb850108,0x8e3c0101,0xfb850108,0x8e3d0101,
	0xfb850108,0x8e3e0101,0xfb850108,0x8e3f0101,0xfb850108,0x8e400101,0xfb850108,0x8e410101,
	0xfb850108,0x8e420101,0xfb850108,0x8e430101,0xfb850108,0x8e440101,0xfb850108,0x8e450101,
	0xfb850108,0x8e460101,0xfb850108,0x8e470101,0xfb850108,0x8e480101,0xfb850108,0x8e490101,
	0xfb850108,0x8e4a0101,0xfb850108,0x8e4b0101,0xfb850108,0x8e4c0101,0xfb850108,0x8e4d0101,
	0xfb850108,0x8e4e0101,0xfb850108,0x8e4f0101,0xfb850108,0x8e500101,0xfb850108,0x8e510101,
	0xfb850108,0x8e520101,0xfb850108,0x8e530101,0xfb850108,0x8e540101,0xfb850108,0x8e550101,
	0xfb850108,0x8e560101,0xfb850108,0x8e570101,0xfb850108,0x8e580101,0xfb850108,0x8e590101,
	0xfb850108,0x8e5a0101,0xfb850108,0x8e5b0101,0xfb850108,0x8e5c0101,0xfb850108,0x8e5d0101,
	0xfb850108,0x8e5e0101,0xfb850108,0x8e5f0101,0xfb850108,0x8e600101,0xfb850108,0x8e610101,
	0xfb850108,0x8e620101,0xfb850108,0x8e630101,0xfb850108,0x8e640101,0xfb850108,0x8e650101,
	0xfb850108,0x8e660101,0xfb850108,0x8e670101,0xfb850108,0x8e680101,0xfb850108,0x8e690101,
	0xfb850108,0x8e6a0101,0xfb850108,0x8e6b0101,0xfb850108,0x8e6c0101,0xfb850108,0x8e6d0101,
	0xfb850108,0x8e6e0101,0xfb850108,0x8e6f0101,0xfb850108,0x8e700101,0xfb850108,0x8e710101,
	0xfb850108,0x8e720101,0xfb850108,0x8e730101,0xfb850108,0x8e740101,0xfb850108,0x8e750101,
	0xfb850108,0x8e760101,0xfb850108,0x8e770101,0xfb850108,0x8e780101,0xfb850108,0x8e790101,
	0xfb850108,0x8e7a0101,0xfb850108,0x8e7b0101,0xfb850108,0x8e7c0101,0xfb850108,0x8e7d0101,
	0xfb850108,0x8e7e0101,0xfb850108,0x8e7f0101,0xfb850108,0x8e800101,0xfb850108,0x8e810101,
	0xfb850108,0x8e820101,0xfb850108,0x8e830101,0xfb850108,0x8e840101,0xfb850108,0x8e850101,
	0xfb850108,0x8e860101,0xfb850108,0x8e870101,0xfb850108,0x8e880101,0xfb850108,0x8e890101,
	0xfb850108,0x8e8a0101,0xfb850108,0x8e8b0101,0xfb850108,0x8e8c0101,0xfb850108,0x8e8d0101,
	0xfb850108,0x8e8e0101,0xfb850108,0x8e8f0101,0xfb850108,0x8e900101,0xfb850108,0x8e910101,
	0xfb850108,0x8e920101,0xfb850108,0x8e930101,0xfb850108,0x8e940101,0xfb850108,0x8e950101,
	0xfb850108,0x8e960101,0xfb850108,0x8e970101,0xfb850108,0x8e980101,0xfb850108,0x8e990101,
	0xfb850108,0x8e9a0101,0xfb850108,0x8e9b0101,0xfb850108,0x8e9c0101,0xfb850108,0x8e9d0101,
	0xfb850108,0x8e9e0101,0xfb850108,0x8e9f0101,0xfb850108,0x8ea00101,0xfb850108,0x8ea10101,
	0xfb850108,0x8ea20101,0xfb850108,0x8ea30101,0xfb850108,0x8ea40101,0xfb850108,0x8ea50101,
	0xfb850108,0x8ea60101,0xfb850108,0x8ea70101,0xfb850108,0x8ea80101,0xfb850108,0x8ea90101,
	0xfb850108,0x8eaa0101,0xfb850108,0x8eab0101,0xfb850108,0x8eac0101,0xfb850108,0x8ead0101,
	0xfb850108,0x8eae0101,0xfb850108,0x8eaf0101,0xfb850108,0x8eb00101,0xfb850108,0x8eb10101,
	0xfb850108,0x8eb20101,0xfb850108,0x8eb30101,0xfb850108,0x8eb40101,0xfb850108,0x8eb50101,
	0xfb850108,0x8eb60101,0xfb850108,0x8eb70101,0xfb850108,0x8eb80101,0xfb850108,0x8eb90101,
	0xfb850108,0x8eba0101,0xfb850108,0x8ebb0101,0xfb850108,0x8ebc0101,0xfb850108,0x8ebd0101,
	0xfb850108,0x8ebe0101,0xfb850108,0x8ebf0101,0xfb850108,0x8ec00101,0xfb850108,0x8ec10101,
	0xfb850108,0x8ec20101,0xfb850108,0x8ec30101,0xfb850108,0x8ec40101,0xfb850108,0x8ec50101,
	0xfb850108,0x8ec60101,0xfb850108,0x8ec70101,0xfb850108,0x8ec80101,0xfb850108,0x8ec90101,
	0xfb850108,0x8eca0101,0xfb850108,0x8ecb0101,0xfb850108,0x8ecc0101,0xfb850108,0x8ecd0101,
	0xfb850108,0x8ece0101,0xfb850108,0x8ecf0101,0xfb850108,0x8ed00101,0xfb850108,0x8ed10101,
	0xfb850108,0x8ed20101,0xfb850108,0x8ed30101,0xfb850108,0x8ed40101,0xfb850108,0x8ed50101,
	0xfb850108,0x8ed60101,0xfb850108,0x8ed70101,0xfb850108,0x8ed80101,0xfb850108,0x8ed90101,
	0xfb850108,0x8eda0101,0xfb850108,0x8edb0101,0xfb850108,0x8edc0101,0xfb850108,0x8edd0101,
	0xfb850108,0x8ede0101,0xfb850108,0x8edf0101,0xfb850108,0x8ee00101,0xfb850108,0x8ee10101,
	0xfb850108,0x8ee20101,0xfb850108,0x8ee30101,0xfb850108,0x8ee40101,0xfb850108,0x8ee50101,
	0xfb850108,0x8ee60101,0xfb850108,0x8ee70101,0xfb850108,0x8ee80101,0xfb850108,0x8ee90101,
	0xfb850108,0x8eea0101,0xfb850108,0x8eeb0101,0xfb850108,0x8eec0101,0xfb850108,0x8eed0101,
	0xfb850108,0x8eee0101,0xfb850108,0x8eef0101,0xfb850108,0x8ef00101,0xfb850108,0x8ef10101,
	0xfb850108,0x8ef20101,0xfb850108,0x8ef30101,0xfb850108,0x8ef40101,0xfb850108,0x8ef50101,
	0xfb850108,0x8ef60101,0xfb850108,0x8ef70101,0xfb850108,0x8ef80101,0xfb850108,0x8ef90101,
	0xfb850108,0x8efa0101,0xfb850108,0x8efb0101,0xfb850108,0x8efc0101,0xfb850108,0x8efd0101,
	0xfb850108,0x8efe0101,0xfb850108,0x8eff0101,0xfb850108,0x8f000101,0xfb850108,0x8f010101,
	0xfb850108,0x8f020101,0xfb850108,0x8f030101,0xfb850108,0x8f040101,0xfb850108,0x8f050101,
	0xfb850108,0x8f060101,0xfb850108,0x8f070101,0xfb850108,0x8f080101,0xfb850108,0x8f090101,
	0xfb850108,0x8f0a0101,0xfb850108,0x8f0b0101,0xfb850108,0x8f0c0101,0xfb850108,0x8f0d0101,
	0xfb850108,0x8f0e0101,0xfb850108,0x8f0f0101,0xfb850108,0x8f100101,0xfb850108,0x8f110101,
	0xfb850108,0x8f120101,0xfb850108,0x8f130101,0xfb850108,0x8f140101,0xfb850108,0x8f150101,
	0xfb850108,0x8f160101,0xfb850108,0x8f170101,0xfb850108,0x8f180101,0xfb850108,0x8f190101,
	0xfb850108,0x8f1a0101,0xfb850108,0x8f1b0101,0xfb850108,0x8f1c0101,0xfb850108,0x8f1d0101,
	0xfb850108,0x8f1e0101,0xfb850108,0x8f1f0101,0xfb850108,0x8f200101,0xfb850108,0x8f210101,
	0xfb850108,0x8f220101,0xfb850108,0x8f230101,0xfb850108,0x8f240101,0xfb850108,0x8f250101,
	0xfb850108,0x8f260101,0xfb850108,0x8f270101,0xfb850108,0x8f280101,0xfb850108,0x8f290101,
	0xfb850108,0x8f2a0101,0xfb850108,0x8f2b0101,0xfb850108,0x8f2c0101,0xfb850108,0x8f2d0101,
	0xfb850108,0x8f2e0101,0xfb850108,0x8f2f0101,0xfb850108,0x8f300101,0xfb850108,0x8f310101,
	0xfb850108,0x8f320101,0xfb850108,0x8f330101,0xfb850108,0x8f340101,0xfb850108,0x8f350101,
	0xfb850108,0x8f360101,0xfb850108,0x8f370101,0xfb850108,0x8f380101,0xfb850108,0x8f390101,
	0xfb850108,0x8f3a0101,0xfb850108,0x8f3b0101,0xfb850108,0x8f3c0101,0xfb850108,0x8f3d0101,
	0xfb850108,0x8f3e0101,0xfb850108,0x8f3f0101,0xfb850108,0x8f400101,0xfb850108,0x8f410101,
	0xfb850108,0x8f420101,0xfb850108,0x8f430101,0xfb850108,0x8f440101,0xfb850108,0x8f450101,
	0xfb850108,0x8f460101,0xfb850108,0x8f470101,0xfb850108,0x8f480101,0xfb850108,0x8f490101,
	0xfb850108,0x8f4a0101,0xfb850108,0x8f4b0101,0xfb850108,0x8f4c0101,0xfb850108,0x8f4d0101,
	0xfb850108,0x8f4e0101,0xfb850108,0x8f4f0101,0xfb850108,0x8f500101,0xfb850108,0x8f510101,
	0xfb850108,0x8f520101,0xfb850108,0x8f530101,0xfb850108,0x8f540101,0xfb850108,0x8f550101,
	0xfb850108,0x8f560101,0xfb850108,0x8f570101,0xfb850108,0x8f580101,0xfb850108,0x8f590101,
	0xfb850108,0x8f5a0101,0xfb850108,0x8f5b0101,0xfb850108,0x8f5c0101,0xfb850108,0x8f5d0101,
	0xfb850108,0x8f5e0101,0xfb850108,0x8f5f0101,0xfb850108,0x8f600101,0xfb850108,0x8f610101,
	0xfb850108,0x8f620101,0xfb850108,0x8f630101,0xfb850108,0x8f640101,0xfb850108,0x8f650101,
	0xfb850108,0x8f660101,0xfb850108,0x8f670101,0xfb850108,0x8f680101,0xfb850108,0x8f690101,
	0xfb850108,0x8f6a0101,0xfb850108,0x8f6b0101,0xfb850108,0x8f6c0101,0xfb850108,0x8f6d0101,
	0xfb850108,0x8f6e0101,0xfb850108,0x8f6f0101,0xfb850108,0x8f700101,0xfb850108,0x8f710101,
	0xfb850108,0x8f720101,0xfb850108,0x8f730101,0xfb850108,0x8f740101,0xfb850108,0x8f750101,
	0xfb850108,0x8f760101,0xfb850108,0x8f770101,0xfb850108,0x8f780101,0xfb850108,0x8f790101,
	0xfb850108,0x8f7a0101,0xfb850108,0x8f7b0101,0xfb850108,0x8f7c0101,0xfb850108,0x8f7d0101,
	0xfb850108,0x8f7e0101,0xfb850108,0x8f7f0101,0xfb850108,0x8f800101,0xfb850108,0x8f810101,
	0xfb850108,0x8f820101,0xfb850108,0x8f830101,0xfb850108,0x8f840101,0xfb850108,0x8f850101,
	0xfb850108,0x8f860101,0xfb850108,0x8f870101,0xfb850108,0x8f880101,0xfb850108,0x8f890101,
	0xfb850108,0x8f8a0101,0xfb850108,0x8f8b0101,0xfb850108,0x8f8c0101,0xfb850108,0x8f8d0101,
	0xfb850108,0x8f8e0101,0xfb850108,0x8f8f0101,0xfb850108,0x8f900101,0xfb850108,0x8f910101,
	0xfb850108,0x8f920101,0xfb850108,0x8f930101,0xfb850108,0x8f940101,0xfb850108,0x8f950101,
	0xfb850108,0x8f960101,0xfb850108,0x8f970101,0xfb850108,0x8f980101,0xfb850108,0x8f990101,
	0xfb850108,0x8f9a0101,0xfb850108,0x8f9b0101,0xfb850108,0x8f9c0101,0xfb850108,0x8f9d0101,
	0xfb850108,0x8f9e0101,0xfb850108,0x8f9f0101,0xfb850108,0x8fa00101,0xfb850108,0x8fa10101,
	0xfb850108,0x8fa20101,0xfb850108,0x8fa30101,0xfb850108,0x8fa40101,0xfb850108,0x8fa50101,
	0xfb850108,0x8fa60101,0xfb850108,0x8fa70101,0xfb850108,0x8fa80101,0xfb850108,0x8fa90101,
	0xfb850108,0x8faa0101,0xfb850108,0x8fab0101,0xfb850108,0x8fac0101,0xfb850108,0x8fad0101,
	0xfb850108,0x8fae0101,0xfb850108,0x8faf0101,0xfb850108,0x8fb00101,0xfb850108,0x8fb10101,
	0xfb850108,0x8fb20101,0xfb850108,0x8fb30101,0xfb850108,0x8fb40101,0xfb850108,0x8fb50101,
	0xfb850108,0x8fb60101,0xfb850108,0x8fb70101,0xfb850108,0x8fb80101,0xfb850108,0x8fb90101,
	0xfb850108,0x8fba0101,0xfb850108,0x8fbb0101,0xfb850108,0x8fbc0101,0xfb850108,0x8fbd0101,
	0xfb850108,0x8fbe0101,0xfb850108,0x8fbf0101,0xfb850108,0x8fc00101,0xfb850108,0x8fc10101,
	0xfb850108,0x8fc20101,0xfb850108,0x8fc30101,0xfb850108,0x8fc40101,0xfb850108,0x8fc50101,
	0xfb850108,0x8fc60101,0xfb850108,0x8fc70101,0xfb850108,0x8fc80101,0xfb850108,0x8fc90101,
	0xfb850108,0x8fca0101,0xfb850108,0x8fcb0101,0xfb850108,0x8fcc0101,0xfb850108,0x8fcd0101,
	0xfb850108,0x8fce0101,0xfb850108,0x8fcf0101,0xfb850108,0x8fd00101,0xfb850108,0x8fd10101,
	0xfb850108,0x8fd20101,0xfb850108,0x8fd30101,0xfb850108,0x8fd40101,0xfb850108,0x8fd50101,
	0xfb850108,0x8fd60101,0xfb850108,0x8fd70101,0xfb850108,0x8fd80101,0xfb850108,0x8fd90101,
	0xfb850108,0x8fda0101,0xfb850108,0x8fdb0101,0xfb850108,0x8fdc0101,0xfb850108,0x8fdd0101,
	0xfb850108,0x8fde0101,0xfb850108,0x8fdf0101,0xfb850108,0x8fe00101,0xfb850108,0x8fe10101,
	0xfb850108,0x8fe20101,0xfb850108,0x8fe30101,0xfb850108,0x8fe40101,0xfb850108,0x8fe50101,
	0xfb850108,0x8fe60101,0xfb850108,0x8fe70101,0xfb850108,0x8fe80101,0xfb850108,0x8fe90101,
	0xfb850108,0x8fea0101,0xfb850108,0x8feb0101,0xfb850108,0x8fec0101,0xfb850108,0x8fed0101,
	0xfb850108,0x8fee0101,0xfb850108,0x8fef0101,0xfb850108,0x8ff00101,0xfb850108,0x8ff10101,
	0xfb850108,0x8ff20101,0xfb850108,0x8ff30101,0xfb850108,0x8ff40101,0xfb850108,0x8ff50101,
	0xfb850108,0x8ff60101,0xfb850108,0x8ff70101,0xfb850108,0x8ff80101,0xfb850108,0x8ff90101,
	0xfb850108,0x8ffa0101,0xfb850108,0x8ffb0101,0xfb850108,0x8ffc0101,0xfb850108,0x8ffd0101,
	0xfb850108,0x8ffe0101,0xfb850108,0x8fff0101,0xfb850108,0x90000101,0xfb850108,0x90010101,
	0xfb850108,0x90020101,0xfb850108,0x90030101,0xfb850108,0x90040101,0xfb850108,0x90050101,
	0xfb850108,0x90060101,0xfb850108,0x90070101,0xfb850108,0x90080101,0xfb850108,0x90090101,
	0xfb850108,0x900a0101,0xfb850108,0x900b0101,0xfb850108,0x900c0101,0xfb850108,0x900d0101,
	0xfb850108,0x900e0101,0xfb850108,0x900f0101,0xfb850108,0x90100101,0xfb850108,0x90110101,
	0xfb850108,0x90120101,0xfb850108,0x90130101,0xfb850108,0x90140101,0xfb850108,0x90150101,
	0xfb850108,0x90160101,0xfb850108,0x90170101,0xfb850108,0x90180101,0xfb850108,0x90190101,
	0xfb850108,0x901a0101,0xfb850108,0x901b0101,0xfb850108,0x901c0101,0xfb850108,0x901d0101,
	0xfb850108,0x901e0101,0xfb850108,0x901f0101,0xfb850108,0x90200101,0xfb850108,0x90210101,
	0xfb850108,0x90220101,0xfb850108,0x90230101,0xfb850108,0x90240101,0xfb850108,0x90250101,
	0xfb850108,0x90260101,0xfb850108,0x90270101,0xfb850108,0x90280101,0xfb850108,0x90290101,
	0xfb850108,0x902a0101,0xfb850108,0x902b0101,0xfb850108,0x902c0101,0xfb850108,0x902d0101,
	0xfb850108,0x902e0101,0xfb850108,0x902f0101,0xfb850108,0x90300101,0xfb850108,0x90310101,
	0xfb850108,0x90320101,0xfb850108,0x90330101,0xfb850108,0x90340101,0xfb850108,0x90350101,
	0xfb850108,0x90360101,0xfb850108,0x90370101,0xfb850108,0x90380101,0xfb850108,0x90390101,
	0xfb850108,0x903a0101,0xfb850108,0x903b0101,0xfb850108,0x903c0101,0xfb850108,0x903d0101,
	0xfb850108,0x903e0101,0xfb850108,0x903f0101,0xfb850108,0x90400101,0xfb850108,0x90410101,
	0xfb850108,0x90420101,0xfb850108,0x90430101,0xfb850108,0x90440101,0xfb850108,0x90450101,
	0xfb850108,0x90460101,0xfb850108,0x90470101,0xfb850108,0x90480101,0xfb850108,0x90490101,
	0xfb850108,0x904a0101,0xfb850108,0x904b0101,0xfb850108,0x904c0101,0xfb850108,0x904d0101,
	0xfb850108,0x904e0101,0xfb850108,0x904f0101,0xfb850108,0x90500101,0xfb850108,0x90510101,
	0xfb850108,0x90520101,0xfb850108,0x90530101,0xfb850108,0x90540101,0xfb850108,0x90550101,
	0xfb850108,0x90560101,0xfb850108,0x90570101,0xfb850108,0x90580101,0xfb850108,0x90590101,
	0xfb850108,0x905a0101,0xfb850108,0x905b0101,0xfb850108,0x905c0101,0xfb850108,0x905d0101,
	0xfb850108,0x905e0101,0xfb850108,0x905f0101,0xfb850108,0x90600101,0xfb850108,0x90610101,
	0xfb850108,0x90620101,0xfb850108,0x90630101,0xfb850108,0x90640101,0xfb850108,0x90650101,
	0xfb850108,0x90660101,0xfb850108,0x90670101,0xfb850108,0x90680101,0xfb850108,0x90690101,
	0xfb850108,0x906a0101,0xfb850108,0x906b0101,0xfb850108,0x906c0101,0xfb850108,0x906d0101,
	0xfb850108,0x906e0101,0xfb850108,0x906f0101,0xfb850108,0x90700101,0xfb850108,0x90710101,
	0xfb850108,0x90720101,0xfb850108,0x90730101,0xfb850108,0x90740101,0xfb850108,0x90750101,
	0xfb850108,0x90760101,0xfb850108,0x90770101,0xfb850108,0x90780101,0xfb850108,0x90790101,
	0xfb850108,0x907a0101,0xfb850108,0x907b0101,0xfb850108,0x907c0101,0xfb850108,0x907d0101,
	0xfb850108,0x907e0101,0xfb850108,0x907f0101,0xfb850108,0x90800101,0xfb850108,0x90810101,
	0xfb850108,0x90820101,0xfb850108,0x90830101,0xfb850108,0x90840101,0xfb850108,0x90850101,
	0xfb850108,0x90860101,0xfb850108,0x90870101,0xfb850108,0x90880101,0xfb850108,0x90890101,
	0xfb850108,0x908a0101,0xfb850108,0x908b0101,0xfb850108,0x908c0101,0xfb850108,0x908d0101,
	0xfb850108,0x908e0101,0xfb850108,0x908f0101,0xfb850108,0x90900101,0xfb850108,0x90910101,
	0xfb850108,0x90920101,0xfb850108,0x90930101,0xfb850108,0x90940101,0xfb850108,0x90950101,
	0xfb850108,0x90960101,0xfb850108,0x90970101,0xfb850108,0x90980101,0xfb850108,0x90990101,
	0xfb850108,0x909a0101,0xfb850108,0x909b0101,0xfb850108,0x909c0101,0xfb850108,0x909d0101,
	0xfb850108,0x909e0101,0xfb850108,0x909f0101,0xfb850108,0x90a00101,0xfb850108,0x90a10101,
	0xfb850108,0x90a20101,0xfb850108,0x90a30101,0xfb850108,0x90a40101,0xfb850108,0x90a50101,
	0xfb850108,0x90a60101,0xfb850108,0x90a70101,0xfb850108,0x90a80101,0xfb850108,0x90a90101,
	0xfb850108,0x90aa0101,0xfb850108,0x90ab0101,0xfb850108,0x90ac0101,0xfb850108,0x90ad0101,
	0xfb850108,0x90ae0101,0xfb850108,0x90af0101,0xfb850108,0x90b00101,0xfb850108,0x90b10101,
	0xfb850108,0x90b20101,0xfb850108,0x90b30101,0xfb850108,0x90b40101,0xfb850108,0x90b50101,
	0xfb850108,0x90b60101,0xfb850108,0x90b70101,0xfb850108,0x90b80101,0xfb850108,0x90b90101,
	0xfb850108,0x90ba0101,0xfb850108,0x90bb0101,0xfb850108,0x90bc0101,0xfb850108,0x90bd0101,
	0xfb850108,0x90be0101,0xfb850108,0x90bf0101,0xfb850108,0x90c00101,0xfb850108,0x90c10101,
	0xfb850108,0x90c20101,0xfb850108,0x90c30101,0xfb850108,0x90c40101,0xfb850108,0x90c50101,
	0xfb850108,0x90c60101,0xfb850108,0x90c70101,0xfb850108,0x90c80101,0xfb850108,0x90c90101,
	0xfb850108,0x90ca0101,0xfb850108,0x90cb0101,0xfb850108,0x90cc0101,0xfb850108,0x90cd0101,
	0xfb850108,0x90ce0101,0xfb850108,0x90cf0101,0xfb850108,0x90d00101,0xfb850108,0x90d10101,
	0xfb850108,0x90d20101,0xfb850108,0x90d30101,0xfb850108,0x90d40101,0xfb850108,0x90d50101,
	0xfb850108,0x90d60101,0xfb850108,0x90d70101,0xfb850108,0x90d80101,0xfb850108,0x90d90101,
	0xfb850108,0x90da0101,0xfb850108,0x90db0101,0xfb850108,0x90dc0101,0xfb850108,0x90dd0101,
	0xfb850108,0x90de0101,0xfb850108,0x90df0101,0xfb850108,0x90e00101,0xfb850108,0x90e10101,
	0xfb850108,0x90e20101,0xfb850108,0x90e30101,0xfb850108,0x90e40101,0xfb850108,0x90e50101,
	0xfb850108,0x90e60101,0xfb850108,0x90e70101,0xfb850108,0x90e80101,0xfb850108,0x90e90101,
	0xfb850108,0x90ea0101,0xfb850108,0x90eb0101,0xfb850108,0x90ec0101,0xfb850108,0x90ed0101,
	0xfb850108,0x90ee0101,0xfb850108,0x90ef0101,0xfb850108,0x90f00101,0xfb850108,0x90f10101,
	0xfb850108,0x90f20101,0xfb850108,0x90f30101,0xfb850108,0x90f40101,0xfb850108,0x90f50101,
	0xfb850108,0x90f60101,0xfb850108,0x90f70101,0xfb850108,0x90f80101,0xfb850108,0x90f90101,
	0xfb850108,0x90fa0101,0xfb850108,0x90fb0101,0xfb850108,0x90fc0101,0xfb850108,0x90fd0101,
	0xfb850108,0x90fe0101,0xfb850108,0x90ff0101,0xfb850108,0x91000101,0xfb850108,0x91010101,
	0xfb850108,0x91020101,0xfb850108,0x91030101,0xfb850108,0x91040101,0xfb850108,0x91050101,
	0xfb850108,0x91060101,0xfb850108,0x91070101,0xfb850108,0x91080101,0xfb850108,0x91090101,
	0xfb850108,0x910a0101,0xfb850108,0x910b0101,0xfb850108,0x910c0101,0xfb850108,0x910d0101,
	0xfb850108,0x910e0101,0xfb850108,0x910f0101,0xfb850108,0x91100101,0xfb850108,0x91110101,
	0xfb850108,0x91120101,0xfb850108,0x91130101,0xfb850108,0x91140101,0xfb850108,0x91150101,
	0xfb850108,0x91160101,0xfb850108,0x91170101,0xfb850108,0x91180101,0xfb850108,0x91190101,
	0xfb850108,0x911a0101,0xfb850108,0x911b0101,0xfb850108,0x911c0101,0xfb850108,0x911d0101,
	0xfb850108,0x911e0101,0xfb850108,0x911f0101,0xfb850108,0x91200101,0xfb850108,0x91210101,
	0xfb850108,0x91220101,0xfb850108,0x91230101,0xfb850108,0x91240101,0xfb850108,0x91250101,
	0xfb850108,0x91260101,0xfb850108,0x91270101,0xfb850108,0x91280101,0xfb850108,0x91290101,
	0xfb850108,0x912a0101,0xfb850108,0x912b0101,0xfb850108,0x912c0101,0xfb850108,0x912d0101,
	0xfb850108,0x912e0101,0xfb850108,0x912f0101,0xfb850108,0x91300101,0xfb850108,0x91310101,
	0xfb850108,0x91320101,0xfb850108,0x91330101,0xfb850108,0x91340101,0xfb850108,0x91350101,
	0xfb850108,0x91360101,0xfb850108,0x91370101,0xfb850108,0x91380101,0xfb850108,0x91390101,
	0xfb850108,0x913a0101,0xfb850108,0x913b0101,0xfb850108,0x913c0101,0xfb850108,0x913d0101,
	0xfb850108,0x913e0101,0xfb850108,0x913f0101,0xfb850108,0x91400101,0xfb850108,0x91410101,
	0xfb850108,0x91420101,0xfb850108,0x91430101,0xfb850108,0x91440101,0xfb850108,0x91450101,
	0xfb850108,0x91460101,0xfb850108,0x91470101,0xfb850108,0x91480101,0xfb850108,0x91490101,
	0xfb850108,0x914a0101,0xfb850108,0x914b0101,0xfb850108,0x914c0101,0xfb850108,0x914d0101,
	0xfb850108,0x914e0101,0xfb850108,0x914f0101,0xfb850108,0x91500101,0xfb850108,0x91510101,
	0xfb850108,0x91520101,0xfb850108,0x91530101,0xfb850108,0x91540101,0xfb850108,0x91550101,
	0xfb850108,0x91560101,0xfb850108,0x91570101,0xfb850108,0x91580101,0xfb850108,0x91590101,
	0xfb850108,0x915a0101,0xfb850108,0x915b0101,0xfb850108,0x915c0101,0xfb850108,0x915d0101,
	0xfb850108,0x915e0101,0xfb850108,0x915f0101,0xfb850108,0x91600101,0xfb850108,0x91610101,
	0xfb850108,0x91620101,0xfb850108,0x91630101,0xfb850108,0x91640101,0xfb850108,0x91650101,
	0xfb850108,0x91660101,0xfb850108,0x91670101,0xfb850108,0x91680101,0xfb850108,0x91690101,
	0xfb850108,0x916a0101,0xfb850108,0x916b0101,0xfb850108,0x916c0101,0xfb850108,0x916d0101,
	0xfb850108,0x916e0101,0xfb850108,0x916f0101,0xfb850108,0x91700101,0xfb850108,0x91710101,
	0xfb850108,0x91720101,0xfb850108,0x91730101,0xfb850108,0x91740101,0xfb850108,0x91750101,
	0xfb850108,0x91760101,0xfb850108,0x91770101,0xfb850108,0x91780101,0xfb850108,0x91790101,
	0xfb850108,0x917a0101,0xfb850108,0x917b0101,0xfb850108,0x917c0101,0xfb850108,0x917d0101,
	0xfb850108,0x917e0101,0xfb850108,0x917f0101,0xfb850108,0x91800101,0xfb850108,0x91810101,
	0xfb850108,0x91820101,0xfb850108,0x91830101,0xfb850108,0x91840101,0xfb850108,0x91850101,
	0xfb850108,0x91860101,0xfb850108,0x91870101,0xfb850108,0x91880101,0xfb850108,0x91890101,
	0xfb850108,0x918a0101,0xfb850108,0x918b0101,0xfb850108,0x918c0101,0xfb850108,0x918d0101,
	0xfb850108,0x918e0101,0xfb850108,0x918f0101,0xfb850108,0x91900101,0xfb850108,0x91910101,
	0xfb850108,0x91920101,0xfb850108,0x91930101,0xfb850108,0x91940101,0xfb850108,0x91950101,
	0xfb850108,0x91960101,0xfb850108,0x91970101,0xfb850108,0x91980101,0xfb850108,0x91990101,
	0xfb850108,0x919a0101,0xfb850108,0x919b0101,0xfb850108,0x919c0101,0xfb850108,0x919d0101,
	0xfb850108,0x919e0101,0xfb850108,0x919f0101,0xfb850108,0x91a00101,0xfb850108,0x91a10101,
	0xfb850108,0x91a20101,0xfb850108,0x91a30101,0xfb850108,0x91a40101,0xfb850108,0x91a50101,
	0xfb850108,0x91a60101,0xfb850108,0x91a70101,0xfb850108,0x91a80101,0xfb850108,0x91a90101,
	0xfb850108,0x91aa0101,0xfb850108,0x91ab0101,0xfb850108,0x91ac0101,0xfb850108,0x91ad0101,
	0xfb850108,0x91ae0101,0xfb850108,0x91af0101,0xfb850108,0x91b00101,0xfb850108,0x91b10101,
	0xfb850108,0x91b20101,0xfb850108,0x91b30101,0xfb850108,0x91b40101,0xfb850108,0x91b50101,
	0xfb850108,0x91b60101,0xfb850108,0x91b70101,0xfb850108,0x91b80101,0xfb850108,0x91b90101,
	0xfb850108,0x91ba0101,0xfb850108,0x91bb0101,0xfb850108,0x91bc0101,0xfb850108,0x91bd0101,
	0xfb850108,0x91be0101,0xfb850108,0x91bf0101,0xfb850108,0x91c00101,0xfb850108,0x91c10101,
	0xfb850108,0x91c20101,0xfb850108,0x91c30101,0xfb850108,0x91c40101,0xfb850108,0x91c50101,
	0xfb850108,0x91c60101,0xfb850108,0x91c70101,0xfb850108,0x91c80101,0xfb850108,0x91c90101,
	0xfb850108,0x91ca0101,0xfb850108,0x91cb0101,0xfb850108,0x91cc0101,0xfb850108,0x91cd0101,
	0xfb850108,0x91ce0101,0xfb850108,0x91cf0101,0xfb850108,0x91d00101,0xfb850108,0x91d10101,
	0xfb850108,0x91d20101,0xfb850108,0x91d30101,0xfb850108,0x91d40101,0xfb850108,0x91d50101,
	0xfb850108,0x91d60101,0xfb850108,0x91d70101,0xfb850108,0x91d80101,0xfb850108,0x91d90101,
	0xfb850108,0x91da0101,0xfb850108,0x91db0101,0xfb850108,0x91dc0101,0xfb850108,0x91dd0101,
	0xfb850108,0x91de0101,0xfb850108,0x91df0101,0xfb850108,0x91e00101,0xfb850108,0x91e10101,
	0xfb850108,0x91e20101,0xfb850108,0x91e30101,0xfb850108,0x91e40101,0xfb850108,0x91e50101,
	0xfb850108,0x91e60101,0xfb850108,0x91e70101,0xfb850108,0x91e80101,0xfb850108,0x91e90101,
	0xfb850108,0x91ea0101,0xfb850108,0x91eb0101,0xfb850108,0x91ec0101,0xfb850108,0x91ed0101,
	0xfb850108,0x91ee0101,0xfb850108,0x91ef0101,0xfb850108,0x91f00101,0xfb850108,0x91f10101,
	0xfb850108,0x91f20101,0xfb850108,0x91f30101,0xfb850108,0x91f40101,0xfb850108,0x91f50101,
	0xfb850108,0x91f60101,0xfb850108,0x91f70101,0xfb850108,0x91f80101,0xfb850108,0x91f90101,
	0xfb850108,0x91fa0101,0xfb850108,0x91fb0101,0xfb850108,0x91fc0101,0xfb850108,0x91fd0101,
	0xfb850108,0x91fe0101,0xfb850108,0x91ff0101,0xfb850108,0x92000101,0xfb850108,0x92010101,
	0xfb850108,0x92020101,0xfb850108,0x92030101,0xfb850108,0x92040101,0xfb850108,0x92050101,
	0xfb850108,0x92060101,0xfb850108,0x92070101,0xfb850108,0x92080101,0xfb850108,0x92090101,
	0xfb850108,0x920a0101,0xfb850108,0x920b0101,0xfb850108,0x920c0101,0xfb850108,0x920d0101,
	0xfb850108,0x920e0101,0xfb850108,0x920f0101,0xfb850108,0x92100101,0xfb850108,0x92110101,
	0xfb850108,0x92120101,0xfb850108,0x92130101,0xfb850108,0x92140101,0xfb850108,0x92150101,
	0xfb850108,0x92160101,0xfb850108,0x92170101,0xfb850108,0x92180101,0xfb850108,0x92190101,
	0xfb850108,0x921a0101,0xfb850108,0x921b0101,0xfb850108,0x921c0101,0xfb850108,0x921d0101,
	0xfb850108,0x921e0101,0xfb850108,0x921f0101,0xfb850108,0x92200101,0xfb850108,0x92210101,
	0xfb850108,0x92220101,0xfb850108,0x92230101,0xfb850108,0x92240101,0xfb850108,0x92250101,
	0xfb850108,0x92260101,0xfb850108,0x92270101,0xfb850108,0x92280101,0xfb850108,0x92290101,
	0xfb850108,0x922a0101,0xfb850108,0x922b0101,0xfb850108,0x922c0101,0xfb850108,0x922d0101,
	0xfb850108,0x922e0101,0xfb850108,0x922f0101,0xfb850108,0x92300101,0xfb850108,0x92310101,
	0xfb850108,0x92320101,0xfb850108,0x92330101,0xfb850108,0x92340101,0xfb850108,0x92350101,
	0xfb850108,0x92360101,0xfb850108,0x92370101,0xfb850108,0x92380101,0xfb850108,0x92390101,
	0xfb850108,0x923a0101,0xfb850108,0x923b0101,0xfb850108,0x923c0101,0xfb850108,0x923d0101,
	0xfb850108,0x923e0101,0xfb850108,0x923f0101,0xfb850108,0x92400101,0xfb850108,0x92410101,
	0xfb850108,0x92420101,0xfb850108,0x92430101,0xfb850108,0x92440101,0xfb850108,0x92450101,
	0xfb850108,0x92460101,0xfb850108,0x92470101,0xfb850108,0x92480101,0xfb850108,0x92490101,
	0xfb850108,0x924a0101,0xfb850108,0x924b0101,0xfb850108,0x924c0101,0xfb850108,0x924d0101,
	0xfb850108,0x924e0101,0xfb850108,0x924f0101,0xfb850108,0x92500101,0xfb850108,0x92510101,
	0xfb850108,0x92520101,0xfb850108,0x92530101,0xfb850108,0x92540101,0xfb850108,0x92550101,
	0xfb850108,0x92560101,0xfb850108,0x92570101,0xfb850108,0x92580101,0xfb850108,0x92590101,
	0xfb850108,0x925a0101,0xfb850108,0x925b0101,0xfb850108,0x925c0101,0xfb850108,0x925d0101,
	0xfb850108,0x925e0101,0xfb850108,0x925f0101,0xfb850108,0x92600101,0xfb850108,0x92610101,
	0xfb850108,0x92620101,0xfb850108,0x92630101,0xfb850108,0x92640101,0xfb850108,0x92650101,
	0xfb850108,0x92660101,0xfb850108,0x92670101,0xfb850108,0x92680101,0xfb850108,0x92690101,
	0xfb850108,0x926a0101,0xfb850108,0x926b0101,0xfb850108,0x926c0101,0xfb850108,0x926d0101,
	0xfb850108,0x926e0101,0xfb850108,0x926f0101,0xfb850108,0x92700101,0xfb850108,0x92710101,
	0xfb850108,0x92720101,0xfb850108,0x92730101,0xfb850108,0x92740101,0xfb850108,0x92750101,
	0xfb850108,0x92760101,0xfb850108,0x92770101,0xfb850108,0x92780101,0xfb850108,0x92790101,
	0xfb850108,0x927a0101,0xfb850108,0x927b0101,0xfb850108,0x927c0101,0xfb850108,0x927d0101,
	0xfb850108,0x927e0101,0xfb850108,0x927f0101,0xfb850108,0x92800101,0xfb850108,0x92810101,
	0xfb850108,0x92820101,0xfb850108,0x92830101,0xfb850108,0x92840101,0xfb850108,0x92850101,
	0xfb850108,0x92860101,0xfb850108,0x92870101,0xfb850108,0x92880101,0xfb850108,0x92890101,
	0xfb850108,0x928a0101,0xfb850108,0x928b0101,0xfb850108,0x928c0101,0xfb850108,0x928d0101,
	0xfb850108,0x928e0101,0xfb850108,0x928f0101,0xfb850108,0x92900101,0xfb850108,0x92910101,
	0xfb850108,0x92920101,0xfb850108,0x92930101,0xfb850108,0x92940101,0xfb850108,0x92950101,
	0xfb850108,0x92960101,0xfb850108,0x92970101,0xfb850108,0x92980101,0xfb850108,0x92990101,
	0xfb850108,0x929a0101,0xfb850108,0x929b0101,0xfb850108,0x929c0101,0xfb850108,0x929d0101,
	0xfb850108,0x929e0101,0xfb850108,0x929f0101,0xfb850108,0x92a00101,0xfb850108,0x92a10101,
	0xfb850108,0x92a20101,0xfb850108,0x92a30101,0xfb850108,0x92a40101,0xfb850108,0x92a50101,
	0xfb850108,0x92a60101,0xfb850108,0x92a70101,0xfb850108,0x92a80101,0xfb850108,0x92a90101,
	0xfb850108,0x92aa0101,0xfb850108,0x92ab0101,0xfb850108,0x92ac0101,0xfb850108,0x92ad0101,
	0xfb850108,0x92ae0101,0xfb850108,0x92af0101,0xfb850108,0x92b00101,0xfb850108,0x92b10101,
	0xfb850108,0x92b20101,0xfb850108,0x92b30101,0xfb850108,0x92b40101,0xfb850108,0x92b50101,
	0xfb850108,0x92b60101,0xfb850108,0x92b70101,0xfb850108,0x92b80101,0xfb850108,0x92b90101,
	0xfb850108,0x92ba0101,0xfb850108,0x92bb0101,0xfb850108,0x92bc0101,0xfb850108,0x92bd0101,
	0xfb850108,0x92be0101,0xfb850108,0x92bf0101,0xfb850108,0x92c00101,0xfb850108,0x92c10101,
	0xfb850108,0x92c20101,0xfb850108,0x92c30101,0xfb850108,0x92c40101,0xfb850108,0x92c50101,
	0xfb850108,0x92c60101,0xfb850108,0x92c70101,0xfb850108,0x92c80101,0xfb850108,0x92c90101,
	0xfb850108,0x92ca0101,0xfb850108,0x92cb0101,0xfb850108,0x92cc0101,0xfb850108,0x92cd0101,
	0xfb850108,0x92ce0101,0xfb850108,0x92cf0101,0xfb850108,0x92d00101,0xfb850108,0x92d10101,
	0xfb850108,0x92d20101,0xfb850108,0x92d30101,0xfb850108,0x92d40101,0xfb850108,0x92d50101,
	0xfb850108,0x92d60101,0xfb850108,0x92d70101,0xfb850108,0x92d80101,0xfb850108,0x92d90101,
	0xfb850108,0x92da0101,0xfb850108,0x92db0101,0xfb850108,0x92dc0101,0xfb850108,0x92dd0101,
	0xfb850108,0x92de0101,0xfb850108,0x92df0101,0xfb850108,0x92e00101,0xfb850108,0x92e10101,
	0xfb850108,0x92e20101,0xfb850108,0x92e30101,0xfb850108,0x92e40101,0xfb850108,0x92e50101,
	0xfb850108,0x92e60101,0xfb850108,0x92e70101,0xfb850108,0x92e80101,0xfb850108,0x92e90101,
	0xfb850108,0x92ea0101,0xfb850108,0x92eb0101,0xfb850108,0x92ec0101,0xfb850108,0x92ed0101,
	0xfb850108,0x92ee0101,0xfb850108,0x92ef0101,0xfb850108,0x92f00101,0xfb850108,0x92f10101,
	0xfb850108,0x92f20101,0xfb850108,0x92f30101,0xfb850108,0x92f40101,0xfb850108,0x92f50101,
	0xfb850108,0x92f60101,0xfb850108,0x92f70101,0xfb850108,0x92f80101,0xfb850108,0x92f90101,
	0xfb850108,0x92fa0101,0xfb850108,0x92fb0101,0xfb850108,0x92fc0101,0xfb850108,0x92fd0101,
	0xfb850108,0x92fe0101,0xfb850108,0x92ff0101,0xfb850108,0x93000101,0xfb850108,0x93010101,
	0xfb850108,0x93020101,0xfb850108,0x93030101,0xfb850108,0x93040101,0xfb850108,0x93050101,
	0xfb850108,0x93060101,0xfb850108,0x93070101,0xfb850108,0x93080101,0xfb850108,0x93090101,
	0xfb850108,0x930a0101,0xfb850108,0x930b0101,0xfb850108,0x930c0101,0xfb850108,0x930d0101,
	0xfb850108,0x930e0101,0xfb850108,0x930f0101,0xfb850108,0x93100101,0xfb850108,0x93110101,
	0xfb850108,0x93120101,0xfb850108,0x93130101,0xfb850108,0x93140101,0xfb850108,0x93150101,
	0xfb850108,0x93160101,0xfb850108,0x93170101,0xfb850108,0x93180101,0xfb850108,0x93190101,
	0xfb850108,0x931a0101,0xfb850108,0x931b0101,0xfb850108,0x931c0101,0xfb850108,0x931d0101,
	0xfb850108,0x931e0101,0xfb850108,0x931f0101,0xfb850108,0x93200101,0xfb850108,0x93210101,
	0xfb850108,0x93220101,0xfb850108,0x93230101,0xfb850108,0x93240101,0xfb850108,0x93250101,
	0xfb850108,0x93260101,0xfb850108,0x93270101,0xfb850108,0x93280101,0xfb850108,0x93290101,
	0xfb850108,0x932a0101,0xfb850108,0x932b0101,0xfb850108,0x932c0101,0xfb850108,0x932d0101,
	0xfb850108,0x932e0101,0xfb850108,0x932f0101,0xfb850108,0x93300101,0xfb850108,0x93310101,
	0xfb850108,0x93320101,0xfb850108,0x93330101,0xfb850108,0x93340101,0xfb850108,0x93350101,
	0xfb850108,0x93360101,0xfb850108,0x93370101,0xfb850108,0x93380101,0xfb850108,0x93390101,
	0xfb850108,0x933a0101,0xfb850108,0x933b0101,0xfb850108,0x933c0101,0xfb850108,0x933d0101,
	0xfb850108,0x933e0101,0xfb850108,0x933f0101,0xfb850108,0x93400101,0xfb850108,0x93410101,
	0xfb850108,0x93420101,0xfb850108,0x93430101,0xfb850108,0x93440101,0xfb850108,0x93450101,
	0xfb850108,0x93460101,0xfb850108,0x93470101,0xfb850108,0x93480101,0xfb850108,0x93490101,
	0xfb850108,0x934a0101,0xfb850108,0x934b0101,0xfb850108,0x934c0101,0xfb850108,0x934d0101,
	0xfb850108,0x934e0101,0xfb850108,0x934f0101,0xfb850108,0x93500101,0xfb850108,0x93510101,
	0xfb850108,0x93520101,0xfb850108,0x93530101,0xfb850108,0x93540101,0xfb850108,0x93550101,
	0xfb850108,0x93560101,0xfb850108,0x93570101,0xfb850108,0x93580101,0xfb850108,0x93590101,
	0xfb850108,0x935a0101,0xfb850108,0x935b0101,0xfb850108,0x935c0101,0xfb850108,0x935d0101,
	0xfb850108,0x935e0101,0xfb850108,0x935f0101,0xfb850108,0x93600101,0xfb850108,0x93610101,
	0xfb850108,0x93620101,0xfb850108,0x93630101,0xfb850108,0x93640101,0xfb850108,0x93650101,
	0xfb850108,0x93660101,0xfb850108,0x93670101,0xfb850108,0x93680101,0xfb850108,0x93690101,
	0xfb850108,0x936a0101,0xfb850108,0x936b0101,0xfb850108,0x936c0101,0xfb850108,0x936d0101,
	0xfb850108,0x936e0101,0xfb850108,0x936f0101,0xfb850108,0x93700101,0xfb850108,0x93710101,
	0xfb850108,0x93720101,0xfb850108,0x93730101,0xfb850108,0x93740101,0xfb850108,0x93750101,
	0xfb850108,0x93760101,0xfb850108,0x93770101,0xfb850108,0x93780101,0xfb850108,0x93790101,
	0xfb850108,0x937a0101,0xfb850108,0x937b0101,0xfb850108,0x937c0101,0xfb850108,0x937d0101,
	0xfb850108,0x937e0101,0xfb850108,0x937f0101,0xfb850108,0x93800101,0xfb850108,0x93810101,
	0xfb850108,0x93820101,0xfb850108,0x93830101,0xfb850108,0x93840101,0xfb850108,0x93850101,
	0xfb850108,0x93860101,0xfb850108,0x93870101,0xfb850108,0x93880101,0xfb850108,0x93890101,
	0xfb850108,0x938a0101,0xfb850108,0x938b0101,0xfb850108,0x938c0101,0xfb850108,0x938d0101,
	0xfb850108,0x938e0101,0xfb850108,0x938f0101,0xfb850108,0x93900101,0xfb850108,0x93910101,
	0xfb850108,0x93920101,0xfb850108,0x93930101,0xfb850108,0x93940101,0xfb850108,0x93950101,
	0xfb850108,0x93960101,0xfb850108,0x93970101,0xfb850108,0x93980101,0xfb850108,0x93990101,
	0xfb850108,0x939a0101,0xfb850108,0x939b0101,0xfb850108,0x939c0101,0xfb850108,0x939d0101,
	0xfb850108,0x939e0101,0xfb850108,0x939f0101,0xfb850108,0x93a00101,0xfb850108,0x93a10101,
	0xfb850108,0x93a20101,0xfb850108,0x93a30101,0xfb850108,0x93a40101,0xfb850108,0x93a50101,
	0xfb850108,0x93a60101,0xfb850108,0x93a70101,0xfb850108,0x93a80101,0xfb850108,0x93a90101,
	0xfb850108,0x93aa0101,0xfb850108,0x93ab0101,0xfb850108,0x93ac0101,0xfb850108,0x93ad0101,
	0xfb850108,0x93ae0101,0xfb850108,0x93af0101,0xfb850108,0x93b00101,0xfb850108,0x93b10101,
	0xfb850108,0x93b20101,0xfb850108,0x93b30101,0xfb850108,0x93b40101,0xfb850108,0x93b50101,
	0xfb850108,0x93b60101,0xfb850108,0x93b70101,0xfb850108,0x93b80101,0xfb850108,0x93b90101,
	0xfb850108,0x93ba0101,0xfb850108,0x93bb0101,0xfb850108,0x93bc0101,0xfb850108,0x93bd0101,
	0xfb850108,0x93be0101,0xfb850108,0x93bf0101,0xfb850108,0x93c00101,0xfb850108,0x93c10101,
	0xfb850108,0x93c20101,0xfb850108,0x93c30101,0xfb850108,0x93c40101,0xfb850108,0x93c50101,
	0xfb850108,0x93c60101,0xfb850108,0x93c70101,0xfb850108,0x93c80101,0xfb850108,0x93c90101,
	0xfb850108,0x93ca0101,0xfb850108,0x93cb0101,0xfb850108,0x93cc0101,0xfb850108,0x93cd0101,
	0xfb850108,0x93ce0101,0xfb850108,0x93cf0101,0xfb850108,0x93d00101,0xfb850108,0x93d10101,
	0xfb850108,0x93d20101,0xfb850108,0x93d30101,0xfb850108,0x93d40101,0xfb850108,0x93d50101,
	0xfb850108,0x93d60101,0xfb850108,0x93d70101,0xfb850108,0x93d80101,0xfb850108,0x93d90101,
	0xfb850108,0x93da0101,0xfb850108,0x93db0101,0xfb850108,0x93dc0101,0xfb850108,0x93dd0101,
	0xfb850108,0x93de0101,0xfb850108,0x93df0101,0xfb850108,0x93e00101,0xfb850108,0x93e10101,
	0xfb850108,0x93e20101,0xfb850108,0x93e30101,0xfb850108,0x93e40101,0xfb850108,0x93e50101,
	0xfb850108,0x93e60101,0xfb850108,0x93e70101,0xfb850108,0x93e80101,0xfb850108,0x93e90101,
	0xfb850108,0x93ea0101,0xfb850108,0x93eb0101,0xfb850108,0x93ec0101,0xfb850108,0x93ed0101,
	0xfb850108,0x93ee0101,0xfb850108,0x93ef0101,0xfb850108,0x93f00101,0xfb850108,0x93f10101,
	0xfb850108,0x93f20101,0xfb850108,0x93f30101,0xfb850108,0x93f40101,0xfb850108,0x93f50101,
	0xfb850108,0x93f60101,0xfb850108,0x93f70101,0xfb850108,0x93f80101,0xfb850108,0x93f90101,
	0xfb850108,0x93fa0101,0xfb850108,0x93fb0101,0xfb850108,0x93fc0101,0xfb850108,0x93fd0101,
	0xfb850108,0x93fe0101,0xfb850108,0x93ff0101,0xfb850108,0x94000101,0xfb850108,0x94010101,
	0xfb850108,0x94020101,0xfb850108,0x94030101,0xfb850108,0x94040101,0xfb850108,0x94050101,
	0xfb850108,0x94060101,0xfb850108,0x94070101,0xfb850108,0x94080101,0xfb850108,0x94090101,
	0xfb850108,0x940a0101,0xfb850108,0x940b0101,0xfb850108,0x940c0101,0xfb850108,0x940d0101,
	0xfb850108,0x940e0101,0xfb850108,0x940f0101,0xfb850108,0x94100101,0xfb850108,0x94110101,
	0xfb850108,0x94120101,0xfb850108,0x94130101,0xfb850108,0x94140101,0xfb850108,0x94150101,
	0xfb850108,0x94160101,0xfb850108,0x94170101,0xfb850108,0x94180101,0xfb850108,0x94190101,
	0xfb850108,0x941a0101,0xfb850108,0x941b0101,0xfb850108,0x941c0101,0xfb850108,0x941d0101,
	0xfb850108,0x941e0101,0xfb850108,0x941f0101,0xfb850108,0x94200101,0xfb850108,0x94210101,
	0xfb850108,0x94220101,0xfb850108,0x94230101,0xfb850108,0x94240101,0xfb850108,0x94250101,
	0xfb850108,0x94260101,0xfb850108,0x94270101,0xfb850108,0x94280101,0xfb850108,0x94290101,
	0xfb850108,0x942a0101,0xfb850108,0x942b0101,0xfb850108,0x942c0101,0xfb850108,0x942d0101,
	0xfb850108,0x942e0101,0xfb850108,0x942f0101,0xfb850108,0x94300101,0xfb850108,0x94310101,
	0xfb850108,0x94320101,0xfb850108,0x94330101,0xfb850108,0x94340101,0xfb850108,0x94350101,
	0xfb850108,0x94360101,0xfb850108,0x94370101,0xfb850108,0x94380101,0xfb850108,0x94390101,
	0xfb850108,0x943a0101,0xfb850108,0x943b0101,0xfb850108,0x943c0101,0xfb850108,0x943d0101,
	0xfb850108,0x943e0101,0xfb850108,0x943f0101,0xfb850108,0x94400101,0xfb850108,0x94410101,
	0xfb850108,0x94420101,0xfb850108,0x94430101,0xfb850108,0x94440101,0xfb850108,0x94450101,
	0xfb850108,0x94460101,0xfb850108,0x94470101,0xfb850108,0x94480101,0xfb850108,0x94490101,
	0xfb850108,0x944a0101,0xfb850108,0x944b0101,0xfb850108,0x944c0101,0xfb850108,0x944d0101,
	0xfb850108,0x944e0101,0xfb850108,0x944f0101,0xfb850108,0x94500101,0xfb850108,0x94510101,
	0xfb850108,0x94520101,0xfb850108,0x94530101,0xfb850108,0x94540101,0xfb850108,0x94550101,
	0xfb850108,0x94560101,0xfb850108,0x94570101,0xfb850108,0x94580101,0xfb850108,0x94590101,
	0xfb850108,0x945a0101,0xfb850108,0x945b0101,0xfb850108,0x945c0101,0xfb850108,0x945d0101,
	0xfb850108,0x945e0101,0xfb850108,0x945f0101,0xfb850108,0x94600101,0xfb850108,0x94610101,
	0xfb850108,0x94620101,0xfb850108,0x94630101,0xfb850108,0x94640101,0xfb850108,0x94650101,
	0xfb850108,0x94660101,0xfb850108,0x94670101,0xfb850108,0x94680101,0xfb850108,0x94690101,
	0xfb850108,0x946a0101,0xfb850108,0x946b0101,0xfb850108,0x946c0101,0xfb850108,0x946d0101,
	0xfb850108,0x946e0101,0xfb850108,0x946f0101,0xfb850108,0x94700101,0xfb850108,0x94710101,
	0xfb850108,0x94720101,0xfb850108,0x94730101,0xfb850108,0x94740101,0xfb850108,0x94750101,
	0xfb850108,0x94760101,0xfb850108,0x94770101,0xfb850108,0x94780101,0xfb850108,0x94790101,
	0xfb850108,0x947a0101,0xfb850108,0x947b0101,0xfb850108,0x947c0101,0xfb850108,0x947d0101,
	0xfb850108,0x947e0101,0xfb850108,0x947f0101,0xfb850108,0x94800101,0xfb850108,0x94810101,
	0xfb850108,0x94820101,0xfb850108,0x94830101,0xfb850108,0x94840101,0xfb850108,0x94850101,
	0xfb850108,0x94860101,0xfb850108,0x94870101,0xfb850108,0x94880101,0xfb850108,0x94890101,
	0xfb850108,0x948a0101,0xfb850108,0x948b0101,0xfb850108,0x948c0101,0xfb850108,0x948d0101,
	0xfb850108,0x948e0101,0xfb850108,0x948f0101,0xfb850108,0x94900101,0xfb850108,0x94910101,
	0xfb850108,0x94920101,0xfb850108,0x94930101,0xfb850108,0x94940101,0xfb850108,0x94950101,
	0xfb850108,0x94960101,0xfb850108,0x94970101,0xfb850108,0x94980101,0xfb850108,0x94990101,
	0xfb850108,0x949a0101,0xfb850108,0x949b0101,0xfb850108,0x949c0101,0xfb850108,0x949d0101,
	0xfb850108,0x949e0101,0xfb850108,0x949f0101,0xfb850108,0x94a00101,0xfb850108,0x94a10101,
	0xfb850108,0x94a20101,0xfb850108,0x94a30101,0xfb850108,0x94a40101,0xfb850108,0x94a50101,
	0xfb850108,0x94a60101,0xfb850108,0x94a70101,0xfb850108,0x94a80101,0xfb850108,0x94a90101,
	0xfb850108,0x94aa0101,0xfb850108,0x94ab0101,0xfb850108,0x94ac0101,0xfb850108,0x94ad0101,
	0xfb850108,0x94ae0101,0xfb850108,0x94af0101,0xfb850108,0x94b00101,0xfb850108,0x94b10101,
	0xfb850108,0x94b20101,0xfb850108,0x94b30101,0xfb850108,0x94b40101,0xfb850108,0x94b50101,
	0xfb850108,0x94b60101,0xfb850108,0x94b70101,0xfb850108,0x94b80101,0xfb850108,0x94b90101,
	0xfb850108,0x94ba0101,0xfb850108,0x94bb0101,0xfb850108,0x94bc0101,0xfb850108,0x94bd0101,
	0xfb850108,0x94be0101,0xfb850108,0x94bf0101,0xfb850108,0x94c00101,0xfb850108,0x94c10101,
	0xfb850108,0x94c20101,0xfb850108,0x94c30101,0xfb850108,0x94c40101,0xfb850108,0x94c50101,
	0xfb850108,0x94c60101,0xfb850108,0x94c70101,0xfb850108,0x94c80101,0xfb850108,0x94c90101,
	0xfb850108,0x94ca0101,0xfb850108,0x94cb0101,0xfb850108,0x94cc0101,0xfb850108,0x94cd0101,
	0xfb850108,0x94ce0101,0xfb850108,0x94cf0101,0xfb850108,0x94d00101,0xfb850108,0x94d10101,
	0xfb850108,0x94d20101,0xfb850108,0x94d30101,0xfb850108,0x94d40101,0xfb850108,0x94d50101,
	0xfb850108,0x94d60101,0xfb850108,0x94d70101,0xfb850108,0x94d80101,0xfb850108,0x94d90101,
	0xfb850108,0x94da0101,0xfb850108,0x94db0101,0xfb850108,0x94dc0101,0xfb850108,0x94dd0101,
	0xfb850108,0x94de0101,0xfb850108,0x94df0101,0xfb850108,0x94e00101,0xfb850108,0x94e10101,
	0xfb850108,0x94e20101,0xfb850108,0x94e30101,0xfb850108,0x94e40101,0xfb850108,0x94e50101,
	0xfb850108,0x94e60101,0xfb850108,0x94e70101,0xfb850108,0x94e80101,0xfb850108,0x94e90101,
	0xfb850108,0x94ea0101,0xfb850108,0x94eb0101,0xfb850108,0x94ec0101,0xfb850108,0x94ed0101,
	0xfb850108,0x94ee0101,0xfb850108,0x94ef0101,0xfb850108,0x94f00101,0xfb850108,0x94f10101,
	0xfb850108,0x94f20101,0xfb850108,0x94f30101,0xfb850108,0x94f40101,0xfb850108,0x94f50101,
	0xfb850108,0x94f60101,0xfb850108,0x94f70101,0xfb850108,0x94f80101,0xfb850108,0x94f90101,
	0xfb850108,0x94fa0101,0xfb850108,0x94fb0101,0xfb850108,0x94fc0101,0xfb850108,0x94fd0101,
	0xfb850108,0x94fe0101,0xfb850108,0x94ff0101,0xfb850108,0x95000101,0xfb850108,0x95010101,
	0xfb850108,0x95020101,0xfb850108,0x95030101,0xfb850108,0x95040101,0xfb850108,0x95050101,
	0xfb850108,0x95060101,0xfb850108,0x95070101,0xfb850108,0x95080101,0xfb850108,0x95090101,
	0xfb850108,0x950a0101,0xfb850108,0x950b0101,0xfb850108,0x950c0101,0xfb850108,0x950d0101,
	0xfb850108,0x950e0101,0xfb850108,0x950f0101,0xfb850108,0x95100101,0xfb850108,0x95110101,
	0xfb850108,0x95120101,0xfb850108,0x95130101,0xfb850108,0x95140101,0xfb850108,0x95150101,
	0xfb850108,0x95160101,0xfb850108,0x95170101,0xfb850108,0x95180101,0xfb850108,0x95190101,
	0xfb850108,0x951a0101,0xfb850108,0x951b0101,0xfb850108,0x951c0101,0xfb850108,0x951d0101,
	0xfb850108,0x951e0101,0xfb850108,0x951f0101,0xfb850108,0x95200101,0xfb850108,0x95210101,
	0xfb850108,0x95220101,0xfb850108,0x95230101,0xfb850108,0x95240101,0xfb850108,0x95250101,
	0xfb850108,0x95260101,0xfb850108,0x95270101,0xfb850108,0x95280101,0xfb850108,0x95290101,
	0xfb850108,0x952a0101,0xfb850108,0x952b0101,0xfb850108,0x952c0101,0xfb850108,0x952d0101,
	0xfb850108,0x952e0101,0xfb850108,0x952f0101,0xfb850108,0x95300101,0xfb850108,0x95310101,
	0xfb850108,0x95320101,0xfb850108,0x95330101,0xfb850108,0x95340101,0xfb850108,0x95350101,
	0xfb850108,0x95360101,0xfb850108,0x95370101,0xfb850108,0x95380101,0xfb850108,0x95390101,
	0xfb850108,0x953a0101,0xfb850108,0x953b0101,0xfb850108,0x953c0101,0xfb850108,0x953d0101,
	0xfb850108,0x953e0101,0xfb850108,0x953f0101,0xfb850108,0x95400101,0xfb850108,0x95410101,
	0xfb850108,0x95420101,0xfb850108,0x95430101,0xfb850108,0x95440101,0xfb850108,0x95450101,
	0xfb850108,0x95460101,0xfb850108,0x95470101,0xfb850108,0x95480101,0xfb850108,0x95490101,
	0xfb850108,0x954a0101,0xfb850108,0x954b0101,0xfb850108,0x954c0101,0xfb850108,0x954d0101,
	0xfb850108,0x954e0101,0xfb850108,0x954f0101,0xfb850108,0x95500101,0xfb850108,0x95510101,
	0xfb850108,0x95520101,0xfb850108,0x95530101,0xfb850108,0x95540101,0xfb850108,0x95550101,
	0xfb850108,0x95560101,0xfb850108,0x95570101,0xfb850108,0x95580101,0xfb850108,0x95590101,
	0xfb850108,0x955a0101,0xfb850108,0x955b0101,0xfb850108,0x955c0101,0xfb850108,0x955d0101,
	0xfb850108,0x955e0101,0xfb850108,0x955f0101,0xfb850108,0x95600101,0xfb850108,0x95610101,
	0xfb850108,0x95620101,0xfb850108,0x95630101,0xfb850108,0x95640101,0xfb850108,0x95650101,
	0xfb850108,0x95660101,0xfb850108,0x95670101,0xfb850108,0x95680101,0xfb850108,0x95690101,
	0xfb850108,0x956a0101,0xfb850108,0x956b0101,0xfb850108,0x956c0101,0xfb850108,0x956d0101,
	0xfb850108,0x956e0101,0xfb850108,0x956f0101,0xfb850108,0x95700101,0xfb850108,0x95710101,
	0xfb850108,0x95720101,0xfb850108,0x95730101,0xfb850108,0x95740101,0xfb850108,0x95750101,
	0xfb850108,0x95760101,0xfb850108,0x95770101,0xfb850108,0x95780101,0xfb850108,0x95790101,
	0xfb850108,0x957a0101,0xfb850108,0x957b0101,0xfb850108,0x957c0101,0xfb850108,0x957d0101,
	0xfb850108,0x957e0101,0xfb850108,0x957f0101,0xfb850108,0x95800101,0xfb850108,0x95810101,
	0xfb850108,0x95820101,0xfb850108,0x95830101,0xfb850108,0x95840101,0xfb850108,0x95850101,
	0xfb850108,0x95860101,0xfb850108,0x95870101,0xfb850108,0x95880101,0xfb850108,0x95890101,
	0xfb850108,0x958a0101,0xfb850108,0x958b0101,0xfb850108,0x958c0101,0xfb850108,0x958d0101,
	0xfb850108,0x958e0101,0xfb850108,0x958f0101,0xfb850108,0x95900101,0xfb850108,0x95910101,
	0xfb850108,0x95920101,0xfb850108,0x95930101,0xfb850108,0x95940101,0xfb850108,0x95950101,
	0xfb850108,0x95960101,0xfb850108,0x95970101,0xfb850108,0x95980101,0xfb850108,0x95990101,
	0xfb850108,0x959a0101,0xfb850108,0x959b0101,0xfb850108,0x959c0101,0xfb850108,0x959d0101,
	0xfb850108,0x959e0101,0xfb850108,0x959f0101,0xfb850108,0x95a00101,0xfb850108,0x95a10101,
	0xfb850108,0x95a20101,0xfb850108,0x95a30101,0xfb850108,0x95a40101,0xfb850108,0x95a50101,
	0xfb850108,0x95a60101,0xfb850108,0x95a70101,0xfb850108,0x95a80101,0xfb850108,0x95a90101,
	0xfb850108,0x95aa0101,0xfb850108,0x95ab0101,0xfb850108,0x95ac0101,0xfb850108,0x95ad0101,
	0xfb850108,0x95ae0101,0xfb850108,0x95af0101,0xfb850108,0x95b00101,0xfb850108,0x95b10101,
	0xfb850108,0x95b20101,0xfb850108,0x95b30101,0xfb850108,0x95b40101,0xfb850108,0x95b50101,
	0xfb850108,0x95b60101,0xfb850108,0x95b70101,0xfb850108,0x95b80101,0xfb850108,0x95b90101,
	0xfb850108,0x95ba0101,0xfb850108,0x95bb0101,0xfb850108,0x95bc0101,0xfb850108,0x95bd0101,
	0xfb850108,0x95be0101,0xfb850108,0x95bf0101,0xfb850108,0x95c00101,0xfb850108,0x95c10101,
	0xfb850108,0x95c20101,0xfb850108,0x95c30101,0xfb850108,0x95c40101,0xfb850108,0x95c50101,
	0xfb850108,0x95c60101,0xfb850108,0x95c70101,0xfb850108,0x95c80101,0xfb850108,0x95c90101,
	0xfb850108,0x95ca0101,0xfb850108,0x95cb0101,0xfb850108,0x95cc0101,0xfb850108,0x95cd0101,
	0xfb850108,0x95ce0101,0xfb850108,0x95cf0101,0xfb850108,0x95d00101,0xfb850108,0x95d10101,
	0xfb850108,0x95d20101,0xfb850108,0x95d30101,0xfb850108,0x95d40101,0xfb850108,0x95d50101,
	0xfb850108,0x95d60101,0xfb850108,0x95d70101,0xfb850108,0x95d80101,0xfb850108,0x95d90101,
	0xfb850108,0x95da0101,0xfb850108,0x95db0101,0xfb850108,0x95dc0101,0xfb850108,0x95dd0101,
	0xfb850108,0x95de0101,0xfb850108,0x95df0101,0xfb850108,0x95e00101,0xfb850108,0x95e10101,
	0xfb850108,0x95e20101,0xfb850108,0x95e30101,0xfb850108,0x95e40101,0xfb850108,0x95e50101,
	0xfb850108,0x95e60101,0xfb850108,0x95e70101,0xfb850108,0x95e80101,0xfb850108,0x95e90101,
	0xfb850108,0x95ea0101,0xfb850108,0x95eb0101,0xfb850108,0x95ec0101,0xfb850108,0x95ed0101,
	0xfb850108,0x95ee0101,0xfb850108,0x95ef0101,0xfb850108,0x95f00101,0xfb850108,0x95f10101,
	0xfb850108,0x95f20101,0xfb850108,0x95f30101,0xfb850108,0x95f40101,0xfb850108,0x95f50101,
	0xfb850108,0x95f60101,0xfb850108,0x95f70101,0xfb850108,0x95f80101,0xfb850108,0x95f90101,
	0xfb850108,0x95fa0101,0xfb850108,0x95fb0101,0xfb850108,0x95fc0101,0xfb850108,0x95fd0101,
	0xfb850108,0x95fe0101,0xfb850108,0x95ff0101,0xfb850108,0x96000101,0xfb850108,0x96010101,
	0xfb850108,0x96020101,0xfb850108,0x96030101,0xfb850108,0x96040101,0xfb850108,0x96050101,
	0xfb850108,0x96060101,0xfb850108,0x96070101,0xfb850108,0x96080101,0xfb850108,0x96090101,
	0xfb850108,0x960a0101,0xfb850108,0x960b0101,0xfb850108,0x960c0101,0xfb850108,0x960d0101,
	0xfb850108,0x960e0101,0xfb850108,0x960f0101,0xfb850108,0x96100101,0xfb850108,0x96110101,
	0xfb850108,0x96120101,0xfb850108,0x96130101,0xfb850108,0x96140101,0xfb850108,0x96150101,
	0xfb850108,0x96160101,0xfb850108,0x96170101,0xfb850108,0x96180101,0xfb850108,0x96190101,
	0xfb850108,0x961a0101,0xfb850108,0x961b0101,0xfb850108,0x961c0101,0xfb850108,0x961d0101,
	0xfb850108,0x961e0101,0xfb850108,0x961f0101,0xfb850108,0x96200101,0xfb850108,0x96210101,
	0xfb850108,0x96220101,0xfb850108,0x96230101,0xfb850108,0x96240101,0xfb850108,0x96250101,
	0xfb850108,0x96260101,0xfb850108,0x96270101,0xfb850108,0x96280101,0xfb850108,0x96290101,
	0xfb850108,0x962a0101,0xfb850108,0x962b0101,0xfb850108,0x962c0101,0xfb850108,0x962d0101,
	0xfb850108,0x962e0101,0xfb850108,0x962f0101,0xfb850108,0x96300101,0xfb850108,0x96310101,
	0xfb850108,0x96320101,0xfb850108,0x96330101,0xfb850108,0x96340101,0xfb850108,0x96350101,
	0xfb850108,0x96360101,0xfb850108,0x96370101,0xfb850108,0x96380101,0xfb850108,0x96390101,
	0xfb850108,0x963a0101,0xfb850108,0x963b0101,0xfb850108,0x963c0101,0xfb850108,0x963d0101,
	0xfb850108,0x963e0101,0xfb850108,0x963f0101,0xfb850108,0x96400101,0xfb850108,0x96410101,
	0xfb850108,0x96420101,0xfb850108,0x96430101,0xfb850108,0x96440101,0xfb850108,0x96450101,
	0xfb850108,0x96460101,0xfb850108,0x96470101,0xfb850108,0x96480101,0xfb850108,0x96490101,
	0xfb850108,0x964a0101,0xfb850108,0x964b0101,0xfb850108,0x964c0101,0xfb850108,0x964d0101,
	0xfb850108,0x964e0101,0xfb850108,0x964f0101,0xfb850108,0x96500101,0xfb850108,0x96510101,
	0xfb850108,0x96520101,0xfb850108,0x96530101,0xfb850108,0x96540101,0xfb850108,0x96550101,
	0xfb850108,0x96560101,0xfb850108,0x96570101,0xfb850108,0x96580101,0xfb850108,0x96590101,
	0xfb850108,0x965a0101,0xfb850108,0x965b0101,0xfb850108,0x965c0101,0xfb850108,0x965d0101,
	0xfb850108,0x965e0101,0xfb850108,0x965f0101,0xfb850108,0x96600101,0xfb850108,0x96610101,
	0xfb850108,0x96620101,0xfb850108,0x96630101,0xfb850108,0x96640101,0xfb850108,0x96650101,
	0xfb850108,0x96660101,0xfb850108,0x96670101,0xfb850108,0x96680101,0xfb850108,0x96690101,
	0xfb850108,0x966a0101,0xfb850108,0x966b0101,0xfb850108,0x966c0101,0xfb850108,0x966d0101,
	0xfb850108,0x966e0101,0xfb850108,0x966f0101,0xfb850108,0x96700101,0xfb850108,0x96710101,
	0xfb850108,0x96720101,0xfb850108,0x96730101,0xfb850108,0x96740101,0xfb850108,0x96750101,
	0xfb850108,0x96760101,0xfb850108,0x96770101,0xfb850108,0x96780101,0xfb850108,0x96790101,
	0xfb850108,0x967a0101,0xfb850108,0x967b0101,0xfb850108,0x967c0101,0xfb850108,0x967d0101,
	0xfb850108,0x967e0101,0xfb850108,0x967f0101,0xfb850108,0x96800101,0xfb850108,0x96810101,
	0xfb850108,0x96820101,0xfb850108,0x96830101,0xfb850108,0x96840101,0xfb850108,0x96850101,
	0xfb850108,0x96860101,0xfb850108,0x96870101,0xfb850108,0x96880101,0xfb850108,0x96890101,
	0xfb850108,0x968a0101,0xfb850108,0x968b0101,0xfb850108,0x968c0101,0xfb850108,0x968d0101,
	0xfb850108,0x968e0101,0xfb850108,0x968f0101,0xfb850108,0x96900101,0xfb850108,0x96910101,
	0xfb850108,0x96920101,0xfb850108,0x96930101,0xfb850108,0x96940101,0xfb850108,0x96950101,
	0xfb850108,0x96960101,0xfb850108,0x96970101,0xfb850108,0x96980101,0xfb850108,0x96990101,
	0xfb850108,0x969a0101,0xfb850108,0x969b0101,0xfb850108,0x969c0101,0xfb850108,0x969d0101,
	0xfb850108,0x969e0101,0xfb850108,0x969f0101,0xfb850108,0x96a00101,0xfb850108,0x96a10101,
	0xfb850108,0x96a20101,0xfb850108,0x96a30101,0xfb850108,0x96a40101,0xfb850108,0x96a50101,
	0xfb850108,0x96a60101,0xfb850108,0x96a70101,0xfb850108,0x96a80101,0xfb850108,0x96a90101,
	0xfb850108,0x96aa0101,0xfb850108,0x96ab0101,0xfb850108,0x96ac0101,0xfb850108,0x96ad0101,
	0xfb850108,0x96ae0101,0xfb850108,0x96af0101,0xfb850108,0x96b00101,0xfb850108,0x96b10101,
	0xfb850108,0x96b20101,0xfb850108,0x96b30101,0xfb850108,0x96b40101,0xfb850108,0x96b50101,
	0xfb850108,0x96b60101,0xfb850108,0x96b70101,0xfb850108,0x96b80101,0xfb850108,0x96b90101,
	0xfb850108,0x96ba0101,0xfb850108,0x96bb0101,0xfb850108,0x96bc0101,0xfb850108,0x96bd0101,
	0xfb850108,0x96be0101,0xfb850108,0x96bf0101,0xfb850108,0x96c00101,0xfb850108,0x96c10101,
	0xfb850108,0x96c20101,0xfb850108,0x96c30101,0xfb850108,0x96c40101,0xfb850108,0x96c50101,
	0xfb850108,0x96c60101,0xfb850108,0x96c70101,0xfb850108,0x96c80101,0xfb850108,0x96c90101,
	0xfb850108,0x96ca0101,0xfb850108,0x96cb0101,0xfb850108,0x96cc0101,0xfb850108,0x96cd0101,
	0xfb850108,0x96ce0101,0xfb850108,0x96cf0101,0xfb850108,0x96d00101,0xfb850108,0x96d10101,
	0xfb850108,0x96d20101,0xfb850108,0x96d30101,0xfb850108,0x96d40101,0xfb850108,0x96d50101,
	0xfb850108,0x96d60101,0xfb850108,0x96d70101,0xfb850108,0x96d80101,0xfb850108,0x96d90101,
	0xfb850108,0x96da0101,0xfb850108,0x96db0101,0xfb850108,0x96dc0101,0xfb850108,0x96dd0101,
	0xfb850108,0x96de0101,0xfb850108,0x96df0101,0xfb850108,0x96e00101,0xfb850108,0x96e10101,
	0xfb850108,0x96e20101,0xfb850108,0x96e30101,0xfb850108,0x96e40101,0xfb850108,0x96e50101,
	0xfb850108,0x96e60101,0xfb850108,0x96e70101,0xfb850108,0x96e80101,0xfb850108,0x96e90101,
	0xfb850108,0x96ea0101,0xfb850108,0x96eb0101,0xfb850108,0x96ec0101,0xfb850108,0x96ed0101,
	0xfb850108,0x96ee0101,0xfb850108,0x96ef0101,0xfb850108,0x96f00101,0xfb850108,0x96f10101,
	0xfb850108,0x96f20101,0xfb850108,0x96f30101,0xfb850108,0x96f40101,0xfb850108,0x96f50101,
	0xfb850108,0x96f60101,0xfb850108,0x96f70101,0xfb850108,0x96f80101,0xfb850108,0x96f90101,
	0xfb850108,0x96fa0101,0xfb850108,0x96fb0101,0xfb850108,0x96fc0101,0xfb850108,0x96fd0101,
	0xfb850108,0x96fe0101,0xfb850108,0x96ff0101,0xfb850108,0x97000101,0xfb850108,0x97010101,
	0xfb850108,0x97020101,0xfb850108,0x97030101,0xfb850108,0x97040101,0xfb850108,0x97050101,
	0xfb850108,0x97060101,0xfb850108,0x97070101,0xfb850108,0x97080101,0xfb850108,0x97090101,
	0xfb850108,0x970a0101,0xfb850108,0x970b0101,0xfb850108,0x970c0101,0xfb850108,0x970d0101,
	0xfb850108,0x970e0101,0xfb850108,0x970f0101,0xfb850108,0x97100101,0xfb850108,0x97110101,
	0xfb850108,0x97120101,0xfb850108,0x97130101,0xfb850108,0x97140101,0xfb850108,0x97150101,
	0xfb850108,0x97160101,0xfb850108,0x97170101,0xfb850108,0x97180101,0xfb850108,0x97190101,
	0xfb850108,0x971a0101,0xfb850108,0x971b0101,0xfb850108,0x971c0101,0xfb850108,0x971d0101,
	0xfb850108,0x971e0101,0xfb850108,0x971f0101,0xfb850108,0x97200101,0xfb850108,0x97210101,
	0xfb850108,0x97220101,0xfb850108,0x97230101,0xfb850108,0x97240101,0xfb850108,0x97250101,
	0xfb850108,0x97260101,0xfb850108,0x97270101,0xfb850108,0x97280101,0xfb850108,0x97290101,
	0xfb850108,0x972a0101,0xfb850108,0x972b0101,0xfb850108,0x972c0101,0xfb850108,0x972d0101,
	0xfb850108,0x972e0101,0xfb850108,0x972f0101,0xfb850108,0x97300101,0xfb850108,0x97310101,
	0xfb850108,0x97320101,0xfb850108,0x97330101,0xfb850108,0x97340101,0xfb850108,0x97350101,
	0xfb850108,0x97360101,0xfb850108,0x97370101,0xfb850108,0x97380101,0xfb850108,0x97390101,
	0xfb850108,0x973a0101,0xfb850108,0x973b0101,0xfb850108,0x973c0101,0xfb850108,0x973d0101,
	0xfb850108,0x973e0101,0xfb850108,0x973f0101,0xfb850108,0x97400101,0xfb850108,0x97410101,
	0xfb850108,0x97420101,0xfb850108,0x97430101,0xfb850108,0x97440101,0xfb850108,0x97450101,
	0xfb850108,0x97460101,0xfb850108,0x97470101,0xfb850108,0x97480101,0xfb850108,0x97490101,
	0xfb850108,0x974a0101,0xfb850108,0x974b0101,0xfb850108,0x974c0101,0xfb850108,0x974d0101,
	0xfb850108,0x974e0101,0xfb850108,0x974f0101,0xfb850108,0x97500101,0xfb850108,0x97510101,
	0xfb850108,0x97520101,0xfb850108,0x97530101,0xfb850108,0x97540101,0xfb850108,0x97550101,
	0xfb850108,0x97560101,0xfb850108,0x97570101,0xfb850108,0x97580101,0xfb850108,0x97590101,
	0xfb850108,0x975a0101,0xfb850108,0x975b0101,0xfb850108,0x975c0101,0xfb850108,0x975d0101,
	0xfb850108,0x975e0101,0xfb850108,0x975f0101,0xfb850108,0x97600101,0xfb850108,0x97610101,
	0xfb850108,0x97620101,0xfb850108,0x97630101,0xfb850108,0x97640101,0xfb850108,0x97650101,
	0xfb850108,0x97660101,0xfb850108,0x97670101,0xfb850108,0x97680101,0xfb850108,0x97690101,
	0xfb850108,0x976a0101,0xfb850108,0x976b0101,0xfb850108,0x976c0101,0xfb850108,0x976d0101,
	0xfb850108,0x976e0101,0xfb850108,0x976f0101,0xfb850108,0x97700101,0xfb850108,0x97710101,
	0xfb850108,0x97720101,0xfb850108,0x97730101,0xfb850108,0x97740101,0xfb850108,0x97750101,
	0xfb850108,0x97760101,0xfb850108,0x97770101,0xfb850108,0x97780101,0xfb850108,0x97790101,
	0xfb850108,0x977a0101,0xfb850108,0x977b0101,0xfb850108,0x977c0101,0xfb850108,0x977d0101,
	0xfb850108,0x977e0101,0xfb850108,0x977f0101,0xfb850108,0x97800101,0xfb850108,0x97810101,
	0xfb850108,0x97820101,0xfb850108,0x97830101,0xfb850108,0x97840101,0xfb850108,0x97850101,
	0xfb850108,0x97860101,0xfb850108,0x97870101,0xfb850108,0x97880101,0xfb850108,0x97890101,
	0xfb850108,0x978a0101,0xfb850108,0x978b0101,0xfb850108,0x978c0101,0xfb850108,0x978d0101,
	0xfb850108,0x978e0101,0xfb850108,0x978f0101,0xfb850108,0x97900101,0xfb850108,0x97910101,
	0xfb850108,0x97920101,0xfb850108,0x97930101,0xfb850108,0x97940101,0xfb850108,0x97950101,
	0xfb850108,0x97960101,0xfb850108,0x97970101,0xfb850108,0x97980101,0xfb850108,0x97990101,
	0xfb850108,0x979a0101,0xfb850108,0x979b0101,0xfb850108,0x979c0101,0xfb850108,0x979d0101,
	0xfb850108,0x979e0101,0xfb850108,0x979f0101,0xfb850108,0x97a00101,0xfb850108,0x97a10101,
	0xfb850108,0x97a20101,0xfb850108,0x97a30101,0xfb850108,0x97a40101,0xfb850108,0x97a50101,
	0xfb850108,0x97a60101,0xfb850108,0x97a70101,0xfb850108,0x97a80101,0xfb850108,0x97a90101,
	0xfb850108,0x97aa0101,0xfb850108,0x97ab0101,0xfb850108,0x97ac0101,0xfb850108,0x97ad0101,
	0xfb850108,0x97ae0101,0xfb850108,0x97af0101,0xfb850108,0x97b00101,0xfb850108,0x97b10101,
	0xfb850108,0x97b20101,0xfb850108,0x97b30101,0xfb850108,0x97b40101,0xfb850108,0x97b50101,
	0xfb850108,0x97b60101,0xfb850108,0x97b70101,0xfb850108,0x97b80101,0xfb850108,0x97b90101,
	0xfb850108,0x97ba0101,0xfb850108,0x97bb0101,0xfb850108,0x97bc0101,0xfb850108,0x97bd0101,
	0xfb850108,0x97be0101,0xfb850108,0x97bf0101,0xfb850108,0x97c00101,0xfb850108,0x97c10101,
	0xfb850108,0x97c20101,0xfb850108,0x97c30101,0xfb850108,0x97c40101,0xfb850108,0x97c50101,
	0xfb850108,0x97c60101,0xfb850108,0x97c70101,0xfb850108,0x97c80101,0xfb850108,0x97c90101,
	0xfb850108,0x97ca0101,0xfb850108,0x97cb0101,0xfb850108,0x97cc0101,0xfb850108,0x97cd0101,
	0xfb850108,0x97ce0101,0xfb850108,0x97cf0101,0xfb850108,0x97d00101,0xfb850108,0x97d10101,
	0xfb850108,0x97d20101,0xfb850108,0x97d30101,0xfb850108,0x97d40101,0xfb850108,0x97d50101,
	0xfb850108,0x97d60101,0xfb850108,0x97d70101,0xfb850108,0x97d80101,0xfb850108,0x97d90101,
	0xfb850108,0x97da0101,0xfb850108,0x97db0101,0xfb850108,0x97dc0101,0xfb850108,0x97dd0101,
	0xfb850108,0x97de0101,0xfb850108,0x97df0101,0xfb850108,0x97e00101,0xfb850108,0x97e10101,
	0xfb850108,0x97e20101,0xfb850108,0x97e30101,0xfb850108,0x97e40101,0xfb850108,0x97e50101,
	0xfb850108,0x97e60101,0xfb850108,0x97e70101,0xfb850108,0x97e80101,0xfb850108,0x97e90101,
	0xfb850108,0x97ea0101,0xfb850108,0x97eb0101,0xfb850108,0x97ec0101,0xfb850108,0x97ed0101,
	0xfb850108,0x97ee0101,0xfb850108,0x97ef0101,0xfb850108,0x97f00101,0xfb850108,0x97f10101,
	0xfb850108,0x97f20101,0xfb850108,0x97f30101,0xfb850108,0x97f40101,0xfb850108,0x97f50101,
	0xfb850108,0x97f60101,0xfb850108,0x97f70101,0xfb850108,0x97f80101,0xfb850108,0x97f90101,
	0xfb850108,0x97fa0101,0xfb850108,0x97fb0101,0xfb850108,0x97fc0101,0xfb850108,0x97fd0101,
	0xfb850108,0x97fe0101,0xfb850108,0x97ff0101,0xfb850108,0x98000101,0xfb850108,0x98010101,
	0xfb850108,0x98020101,0xfb850108,0x98030101,0xfb850108,0x98040101,0xfb850108,0x98050101,
	0xfb850108,0x98060101,0xfb850108,0x98070101,0xfb850108,0x98080101,0xfb850108,0x98090101,
	0xfb850108,0x980a0101,0xfb850108,0x980b0101,0xfb850108,0x980c0101,0xfb850108,0x980d0101,
	0xfb850108,0x980e0101,0xfb850108,0x980f0101,0xfb850108,0x98100101,0xfb850108,0x98110101,
	0xfb850108,0x98120101,0xfb850108,0x98130101,0xfb850108,0x98140101,0xfb850108,0x98150101,
	0xfb850108,0x98160101,0xfb850108,0x98170101,0xfb850108,0x98180101,0xfb850108,0x98190101,
	0xfb850108,0x981a0101,0xfb850108,0x981b0101,0xfb850108,0x981c0101,0xfb850108,0x981d0101,
	0xfb850108,0x981e0101,0xfb850108,0x981f0101,0xfb850108,0x98200101,0xfb850108,0x98210101,
	0xfb850108,0x98220101,0xfb850108,0x98230101,0xfb850108,0x98240101,0xfb850108,0x98250101,
	0xfb850108,0x98260101,0xfb850108,0x98270101,0xfb850108,0x98280101,0xfb850108,0x98290101,
	0xfb850108,0x982a0101,0xfb850108,0x982b0101,0xfb850108,0x982c0101,0xfb850108,0x982d0101,
	0xfb850108,0x982e0101,0xfb850108,0x982f0101,0xfb850108,0x98300101,0xfb850108,0x98310101,
	0xfb850108,0x98320101,0xfb850108,0x98330101,0xfb850108,0x98340101,0xfb850108,0x98350101,
	0xfb850108,0x98360101,0xfb850108,0x98370101,0xfb850108,0x98380101,0xfb850108,0x98390101,
	0xfb850108,0x983a0101,0xfb850108,0x983b0101,0xfb850108,0x983c0101,0xfb850108,0x983d0101,
	0xfb850108,0x983e0101,0xfb850108,0x983f0101,0xfb850108,0x98400101,0xfb850108,0x98410101,
	0xfb850108,0x98420101,0xfb850108,0x98430101,0xfb850108,0x98440101,0xfb850108,0x98450101,
	0xfb850108,0x98460101,0xfb850108,0x98470101,0xfb850108,0x98480101,0xfb850108,0x98490101,
	0xfb850108,0x984a0101,0xfb850108,0x984b0101,0xfb850108,0x984c0101,0xfb850108,0x984d0101,
	0xfb850108,0x984e0101,0xfb850108,0x984f0101,0xfb850108,0x98500101,0xfb850108,0x98510101,
	0xfb850108,0x98520101,0xfb850108,0x98530101,0xfb850108,0x98540101,0xfb850108,0x98550101,
	0xfb850108,0x98560101,0xfb850108,0x98570101,0xfb850108,0x98580101,0xfb850108,0x98590101,
	0xfb850108,0x985a0101,0xfb850108,0x985b0101,0xfb850108,0x985c0101,0xfb850108,0x985d0101,
	0xfb850108,0x985e0101,0xfb850108,0x985f0101,0xfb850108,0x98600101,0xfb850108,0x98610101,
	0xfb850108,0x98620101,0xfb850108,0x98630101,0xfb850108,0x98640101,0xfb850108,0x98650101,
	0xfb850108,0x98660101,0xfb850108,0x98670101,0xfb850108,0x98680101,0xfb850108,0x98690101,
	0xfb850108,0x986a0101,0xfb850108,0x986b0101,0xfb850108,0x986c0101,0xfb850108,0x986d0101,
	0xfb850108,0x986e0101,0xfb850108,0x986f0101,0xfb850108,0x98700101,0xfb850108,0x98710101,
	0xfb850108,0x98720101,0xfb850108,0x98730101,0xfb850108,0x98740101,0xfb850108,0x98750101,
	0xfb850108,0x98760101,0xfb850108,0x98770101,0xfb850108,0x98780101,0xfb850108,0x98790101,
	0xfb850108,0x987a0101,0xfb850108,0x987b0101,0xfb850108,0x987c0101,0xfb850108,0x987d0101,
	0xfb850108,0x987e0101,0xfb850108,0x987f0101,0xfb850108,0x98800101,0xfb850108,0x98810101,
	0xfb850108,0x98820101,0xfb850108,0x98830101,0xfb850108,0x98840101,0xfb850108,0x98850101,
	0xfb850108,0x98860101,0xfb850108,0x98870101,0xfb850108,0x98880101,0xfb850108,0x98890101,
	0xfb850108,0x988a0101,0xfb850108,0x988b0101,0xfb850108,0x988c0101,0xfb850108,0x988d0101,
	0xfb850108,0x988e0101,0xfb850108,0x988f0101,0xfb850108,0x98900101,0xfb850108,0x98910101,
	0xfb850108,0x98920101,0xfb850108,0x98930101,0xfb850108,0x98940101,0xfb850108,0x98950101,
	0xfb850108,0x98960101,0xfb850108,0x98970101,0xfb850108,0x98980101,0xfb850108,0x98990101,
	0xfb850108,0x989a0101,0xfb850108,0x989b0101,0xfb850108,0x989c0101,0xfb850108,0x989d0101,
	0xfb850108,0x989e0101,0xfb850108,0x989f0101,0xfb850108,0x98a00101,0xfb850108,0x98a10101,
	0xfb850108,0x98a20101,0xfb850108,0x98a30101,0xfb850108,0x98a40101,0xfb850108,0x98a50101,
	0xfb850108,0x98a60101,0xfb850108,0x98a70101,0xfb850108,0x98a80101,0xfb850108,0x98a90101,
	0xfb850108,0x98aa0101,0xfb850108,0x98ab0101,0xfb850108,0x98ac0101,0xfb850108,0x98ad0101,
	0xfb850108,0x98ae0101,0xfb850108,0x98af0101,0xfb850108,0x98b00101,0xfb850108,0x98b10101,
	0xfb850108,0x98b20101,0xfb850108,0x98b30101,0xfb850108,0x98b40101,0xfb850108,0x98b50101,
	0xfb850108,0x98b60101,0xfb850108,0x98b70101,0xfb850108,0x98b80101,0xfb850108,0x98b90101,
	0xfb850108,0x98ba0101,0xfb850108,0x98bb0101,0xfb850108,0x98bc0101,0xfb850108,0x98bd0101,
	0xfb850108,0x98be0101,0xfb850108,0x98bf0101,0xfb850108,0x98c00101,0xfb850108,0x98c10101,
	0xfb850108,0x98c20101,0xfb850108,0x98c30101,0xfb850108,0x98c40101,0xfb850108,0x98c50101,
	0xfb850108,0x98c60101,0xfb850108,0x98c70101,0xfb850108,0x98c80101,0xfb850108,0x98c90101,
	0xfb850108,0x98ca0101,0xfb850108,0x98cb0101,0xfb850108,0x98cc0101,0xfb850108,0x98cd0101,
	0xfb850108,0x98ce0101,0xfb850108,0x98cf0101,0xfb850108,0x98d00101,0xfb850108,0x98d10101,
	0xfb850108,0x98d20101,0xfb850108,0x98d30101,0xfb850108,0x98d40101,0xfb850108,0x98d50101,
	0xfb850108,0x98d60101,0xfb850108,0x98d70101,0xfb850108,0x98d80101,0xfb850108,0x98d90101,
	0xfb850108,0x98da0101,0xfb850108,0x98db0101,0xfb850108,0x98dc0101,0xfb850108,0x98dd0101,
	0xfb850108,0x98de0101,0xfb850108,0x98df0101,0xfb850108,0x98e00101,0xfb850108,0x98e10101,
	0xfb850108,0x98e20101,0xfb850108,0x98e30101,0xfb850108,0x98e40101,0xfb850108,0x98e50101,
	0xfb850108,0x98e60101,0xfb850108,0x98e70101,0xfb850108,0x98e80101,0xfb850108,0x98e90101,
	0xfb850108,0x98ea0101,0xfb850108,0x98eb0101,0xfb850108,0x98ec0101,0xfb850108,0x98ed0101,
	0xfb850108,0x98ee0101,0xfb850108,0x98ef0101,0xfb850108,0x98f00101,0xfb850108,0x98f10101,
	0xfb850108,0x98f20101,0xfb850108,0x98f30101,0xfb850108,0x98f40101,0xfb850108,0x98f50101,
	0xfb850108,0x98f60101,0xfb850108,0x98f70101,0xfb850108,0x98f80101,0xfb850108,0x98f90101,
	0xfb850108,0x98fa0101,0xfb850108,0x98fb0101,0xfb850108,0x98fc0101,0xfb850108,0x98fd0101,
	0xfb850108,0x98fe0101,0xfb850108,0x98ff0101,0xfb850108,0x99000101,0xfb850108,0x99010101,
	0xfb850108,0x99020101,0xfb850108,0x99030101,0xfb850108,0x99040101,0xfb850108,0x99050101,
	0xfb850108,0x99060101,0xfb850108,0x99070101,0xfb850108,0x99080101,0xfb850108,0x99090101,
	0xfb850108,0x990a0101,0xfb850108,0x990b0101,0xfb850108,0x990c0101,0xfb850108,0x990d0101,
	0xfb850108,0x990e0101,0xfb850108,0x990f0101,0xfb850108,0x99100101,0xfb850108,0x99110101,
	0xfb850108,0x99120101,0xfb850108,0x99130101,0xfb850108,0x99140101,0xfb850108,0x99150101,
	0xfb850108,0x99160101,0xfb850108,0x99170101,0xfb850108,0x99180101,0xfb850108,0x99190101,
	0xfb850108,0x991a0101,0xfb850108,0x991b0101,0xfb850108,0x991c0101,0xfb850108,0x991d0101,
	0xfb850108,0x991e0101,0xfb850108,0x991f0101,0xfb850108,0x99200101,0xfb850108,0x99210101,
	0xfb850108,0x99220101,0xfb850108,0x99230101,0xfb850108,0x99240101,0xfb850108,0x99250101,
	0xfb850108,0x99260101,0xfb850108,0x99270101,0xfb850108,0x99280101,0xfb850108,0x99290101,
	0xfb850108,0x992a0101,0xfb850108,0x992b0101,0xfb850108,0x992c0101,0xfb850108,0x992d0101,
	0xfb850108,0x992e0101,0xfb850108,0x992f0101,0xfb850108,0x99300101,0xfb850108,0x99310101,
	0xfb850108,0x99320101,0xfb850108,0x99330101,0xfb850108,0x99340101,0xfb850108,0x99350101,
	0xfb850108,0x99360101,0xfb850108,0x99370101,0xfb850108,0x99380101,0xfb850108,0x99390101,
	0xfb850108,0x993a0101,0xfb850108,0x993b0101,0xfb850108,0x993c0101,0xfb850108,0x993d0101,
	0xfb850108,0x993e0101,0xfb850108,0x993f0101,0xfb850108,0x99400101,0xfb850108,0x99410101,
	0xfb850108,0x99420101,0xfb850108,0x99430101,0xfb850108,0x99440101,0xfb850108,0x99450101,
	0xfb850108,0x99460101,0xfb850108,0x99470101,0xfb850108,0x99480101,0xfb850108,0x99490101,
	0xfb850108,0x994a0101,0xfb850108,0x994b0101,0xfb850108,0x994c0101,0xfb850108,0x994d0101,
	0xfb850108,0x994e0101,0xfb850108,0x994f0101,0xfb850108,0x99500101,0xfb850108,0x99510101,
	0xfb850108,0x99520101,0xfb850108,0x99530101,0xfb850108,0x99540101,0xfb850108,0x99550101,
	0xfb850108,0x99560101,0xfb850108,0x99570101,0xfb850108,0x99580101,0xfb850108,0x99590101,
	0xfb850108,0x995a0101,0xfb850108,0x995b0101,0xfb850108,0x995c0101,0xfb850108,0x995d0101,
	0xfb850108,0x995e0101,0xfb850108,0x995f0101,0xfb850108,0x99600101,0xfb850108,0x99610101,
	0xfb850108,0x99620101,0xfb850108,0x99630101,0xfb850108,0x99640101,0xfb850108,0x99650101,
	0xfb850108,0x99660101,0xfb850108,0x99670101,0xfb850108,0x99680101,0xfb850108,0x99690101,
	0xfb850108,0x996a0101,0xfb850108,0x996b0101,0xfb850108,0x996c0101,0xfb850108,0x996d0101,
	0xfb850108,0x996e0101,0xfb850108,0x996f0101,0xfb850108,0x99700101,0xfb850108,0x99710101,
	0xfb850108,0x99720101,0xfb850108,0x99730101,0xfb850108,0x99740101,0xfb850108,0x99750101,
	0xfb850108,0x99760101,0xfb850108,0x99770101,0xfb850108,0x99780101,0xfb850108,0x99790101,
	0xfb850108,0x997a0101,0xfb850108,0x997b0101,0xfb850108,0x997c0101,0xfb850108,0x997d0101,
	0xfb850108,0x997e0101,0xfb850108,0x997f0101,0xfb850108,0x99800101,0xfb850108,0x99810101,
	0xfb850108,0x99820101,0xfb850108,0x99830101,0xfb850108,0x99840101,0xfb850108,0x99850101,
	0xfb850108,0x99860101,0xfb850108,0x99870101,0xfb850108,0x99880101,0xfb850108,0x99890101,
	0xfb850108,0x998a0101,0xfb850108,0x998b0101,0xfb850108,0x998c0101,0xfb850108,0x998d0101,
	0xfb850108,0x998e0101,0xfb850108,0x998f0101,0xfb850108,0x99900101,0xfb850108,0x99910101,
	0xfb850108,0x99920101,0xfb850108,0x99930101,0xfb850108,0x99940101,0xfb850108,0x99950101,
	0xfb850108,0x99960101,0xfb850108,0x99970101,0xfb850108,0x99980101,0xfb850108,0x99990101,
	0xfb850108,0x999a0101,0xfb850108,0x999b0101,0xfb850108,0x999c0101,0xfb850108,0x999d0101,
	0xfb850108,0x999e0101,0xfb850108,0x999f0101,0xfb850108,0x99a00101,0xfb850108,0x99a10101,
	0xfb850108,0x99a20101,0xfb850108,0x99a30101,0xfb850108,0x99a40101,0xfb850108,0x99a50101,
	0xfb850108,0x99a60101,0xfb850108,0x99a70101,0xfb850108,0x99a80101,0xfb850108,0x99a90101,
	0xfb850108,0x99aa0101,0xfb850108,0x99ab0101,0xfb850108,0x99ac0101,0xfb850108,0x99ad0101,
	0xfb850108,0x99ae0101,0xfb850108,0x99af0101,0xfb850108,0x99b00101,0xfb850108,0x99b10101,
	0xfb850108,0x99b20101,0xfb850108,0x99b30101,0xfb850108,0x99b40101,0xfb850108,0x99b50101,
	0xfb850108,0x99b60101,0xfb850108,0x99b70101,0xfb850108,0x99b80101,0xfb850108,0x99b90101,
	0xfb850108,0x99ba0101,0xfb850108,0x99bb0101,0xfb850108,0x99bc0101,0xfb850108,0x99bd0101,
	0xfb850108,0x99be0101,0xfb850108,0x99bf0101,0xfb850108,0x99c00101,0xfb850108,0x99c10101,
	0xfb850108,0x99c20101,0xfb850108,0x99c30101,0xfb850108,0x99c40101,0xfb850108,0x99c50101,
	0xfb850108,0x99c60101,0xfb850108,0x99c70101,0xfb850108,0x99c80101,0xfb850108,0x99c90101,
	0xfb850108,0x99ca0101,0xfb850108,0x99cb0101,0xfb850108,0x99cc0101,0xfb850108,0x99cd0101,
	0xfb850108,0x99ce0101,0xfb850108,0x99cf0101,0xfb850108,0x99d00101,0xfb850108,0x99d10101,
	0xfb850108,0x99d20101,0xfb850108,0x99d30101,0xfb850108,0x99d40101,0xfb850108,0x99d50101,
	0xfb850108,0x99d60101,0xfb850108,0x99d70101,0xfb850108,0x99d80101,0xfb850108,0x99d90101,
	0xfb850108,0x99da0101,0xfb850108,0x99db0101,0xfb850108,0x99dc0101,0xfb850108,0x99dd0101,
	0xfb850108,0x99de0101,0xfb850108,0x99df0101,0xfb850108,0x99e00101,0xfb850108,0x99e10101,
	0xfb850108,0x99e20101,0xfb850108,0x99e30101,0xfb850108,0x99e40101,0xfb850108,0x99e50101,
	0xfb850108,0x99e60101,0xfb850108,0x99e70101,0xfb850108,0x99e80101,0xfb850108,0x99e90101,
	0xfb850108,0x99ea0101,0xfb850108,0x99eb0101,0xfb850108,0x99ec0101,0xfb850108,0x99ed0101,
	0xfb850108,0x99ee0101,0xfb850108,0x99ef0101,0xfb850108,0x99f00101,0xfb850108,0x99f10101,
	0xfb850108,0x99f20101,0xfb850108,0x99f30101,0xfb850108,0x99f40101,0xfb850108,0x99f50101,
	0xfb850108,0x99f60101,0xfb850108,0x99f70101,0xfb850108,0x99f80101,0xfb850108,0x99f90101,
	0xfb850108,0x99fa0101,0xfb850108,0x99fb0101,0xfb850108,0x99fc0101,0xfb850108,0x99fd0101,
	0xfb850108,0x99fe0101,0xfb850108,0x99ff0101,0xfb850108,0x9a000101,0xfb850108,0x9a010101,
	0xfb850108,0x9a020101,0xfb850108,0x9a030101,0xfb850108,0x9a040101,0xfb850108,0x9a050101,
	0xfb850108,0x9a060101,0xfb850108,0x9a070101,0xfb850108,0x9a080101,0xfb850108,0x9a090101,
	0xfb850108,0x9a0a0101,0xfb850108,0x9a0b0101,0xfb850108,0x9a0c0101,0xfb850108,0x9a0d0101,
	0xfb850108,0x9a0e0101,0xfb850108,0x9a0f0101,0xfb850108,0x9a100101,0xfb850108,0x9a110101,
	0xfb850108,0x9a120101,0xfb850108,0x9a130101,0xfb850108,0x9a140101,0xfb850108,0x9a150101,
	0xfb850108,0x9a160101,0xfb850108,0x9a170101,0xfb850108,0x9a180101,0xfb850108,0x9a190101,
	0xfb850108,0x9a1a0101,0xfb850108,0x9a1b0101,0xfb850108,0x9a1c0101,0xfb850108,0x9a1d0101,
	0xfb850108,0x9a1e0101,0xfb850108,0x9a1f0101,0xfb850108,0x9a200101,0xfb850108,0x9a210101,
	0xfb850108,0x9a220101,0xfb850108,0x9a230101,0xfb850108,0x9a240101,0xfb850108,0x9a250101,
	0xfb850108,0x9a260101,0xfb850108,0x9a270101,0xfb850108,0x9a280101,0xfb850108,0x9a290101,
	0xfb850108,0x9a2a0101,0xfb850108,0x9a2b0101,0xfb850108,0x9a2c0101,0xfb850108,0x9a2d0101,
	0xfb850108,0x9a2e0101,0xfb850108,0x9a2f0101,0xfb850108,0x9a300101,0xfb850108,0x9a310101,
	0xfb850108,0x9a320101,0xfb850108,0x9a330101,0xfb850108,0x9a340101,0xfb850108,0x9a350101,
	0xfb850108,0x9a360101,0xfb850108,0x9a370101,0xfb850108,0x9a380101,0xfb850108,0x9a390101,
	0xfb850108,0x9a3a0101,0xfb850108,0x9a3b0101,0xfb850108,0x9a3c0101,0xfb850108,0x9a3d0101,
	0xfb850108,0x9a3e0101,0xfb850108,0x9a3f0101,0xfb850108,0x9a400101,0xfb850108,0x9a410101,
	0xfb850108,0x9a420101,0xfb850108,0x9a430101,0xfb850108,0x9a440101,0xfb850108,0x9a450101,
	0xfb850108,0x9a460101,0xfb850108,0x9a470101,0xfb850108,0x9a480101,0xfb850108,0x9a490101,
	0xfb850108,0x9a4a0101,0xfb850108,0x9a4b0101,0xfb850108,0x9a4c0101,0xfb850108,0x9a4d0101,
	0xfb850108,0x9a4e0101,0xfb850108,0x9a4f0101,0xfb850108,0x9a500101,0xfb850108,0x9a510101,
	0xfb850108,0x9a520101,0xfb850108,0x9a530101,0xfb850108,0x9a540101,0xfb850108,0x9a550101,
	0xfb850108,0x9a560101,0xfb850108,0x9a570101,0xfb850108,0x9a580101,0xfb850108,0x9a590101,
	0xfb850108,0x9a5a0101,0xfb850108,0x9a5b0101,0xfb850108,0x9a5c0101,0xfb850108,0x9a5d0101,
	0xfb850108,0x9a5e0101,0xfb850108,0x9a5f0101,0xfb850108,0x9a600101,0xfb850108,0x9a610101,
	0xfb850108,0x9a620101,0xfb850108,0x9a630101,0xfb850108,0x9a640101,0xfb850108,0x9a650101,
	0xfb850108,0x9a660101,0xfb850108,0x9a670101,0xfb850108,0x9a680101,0xfb850108,0x9a690101,
	0xfb850108,0x9a6a0101,0xfb850108,0x9a6b0101,0xfb850108,0x9a6c0101,0xfb850108,0x9a6d0101,
	0xfb850108,0x9a6e0101,0xfb850108,0x9a6f0101,0xfb850108,0x9a700101,0xfb850108,0x9a710101,
	0xfb850108,0x9a720101,0xfb850108,0x9a730101,0xfb850108,0x9a740101,0xfb850108,0x9a750101,
	0xfb850108,0x9a760101,0xfb850108,0x9a770101,0xfb850108,0x9a780101,0xfb850108,0x9a790101,
	0xfb850108,0x9a7a0101,0xfb850108,0x9a7b0101,0xfb850108,0x9a7c0101,0xfb850108,0x9a7d0101,
	0xfb850108,0x9a7e0101,0xfb850108,0x9a7f0101,0xfb850108,0x9a800101,0xfb850108,0x9a810101,
	0xfb850108,0x9a820101,0xfb850108,0x9a830101,0xfb850108,0x9a840101,0xfb850108,0x9a850101,
	0xfb850108,0x9a860101,0xfb850108,0x9a870101,0xfb850108,0x9a880101,0xfb850108,0x9a890101,
	0xfb850108,0x9a8a0101,0xfb850108,0x9a8b0101,0xfb850108,0x9a8c0101,0xfb850108,0x9a8d0101,
	0xfb850108,0x9a8e0101,0xfb850108,0x9a8f0101,0xfb850108,0x9a900101,0xfb850108,0x9a910101,
	0xfb850108,0x9a920101,0xfb850108,0x9a930101,0xfb850108,0x9a940101,0xfb850108,0x9a950101,
	0xfb850108,0x9a960101,0xfb850108,0x9a970101,0xfb850108,0x9a980101,0xfb850108,0x9a990101,
	0xfb850108,0x9a9a0101,0xfb850108,0x9a9b0101,0xfb850108,0x9a9c0101,0xfb850108,0x9a9d0101,
	0xfb850108,0x9a9e0101,0xfb850108,0x9a9f0101,0xfb850108,0x9aa00101,0xfb850108,0x9aa10101,
	0xfb850108,0x9aa20101,0xfb850108,0x9aa30101,0xfb850108,0x9aa40101,0xfb850108,0x9aa50101,
	0xfb850108,0x9aa60101,0xfb850108,0x9aa70101,0xfb850108,0x9aa80101,0xfb850108,0x9aa90101,
	0xfb850108,0x9aaa0101,0xfb850108,0x9aab0101,0xfb850108,0x9aac0101,0xfb850108,0x9aad0101,
	0xfb850108,0x9aae0101,0xfb850108,0x9aaf0101,0xfb850108,0x9ab00101,0xfb850108,0x9ab10101,
	0xfb850108,0x9ab20101,0xfb850108,0x9ab30101,0xfb850108,0x9ab40101,0xfb850108,0x9ab50101,
	0xfb850108,0x9ab60101,0xfb850108,0x9ab70101,0xfb850108,0x9ab80101,0xfb850108,0x9ab90101,
	0xfb850108,0x9aba0101,0xfb850108,0x9abb0101,0xfb850108,0x9abc0101,0xfb850108,0x9abd0101,
	0xfb850108,0x9abe0101,0xfb850108,0x9abf0101,0xfb850108,0x9ac00101,0xfb850108,0x9ac10101,
	0xfb850108,0x9ac20101,0xfb850108,0x9ac30101,0xfb850108,0x9ac40101,0xfb850108,0x9ac50101,
	0xfb850108,0x9ac60101,0xfb850108,0x9ac70101,0xfb850108,0x9ac80101,0xfb850108,0x9ac90101,
	0xfb850108,0x9aca0101,0xfb850108,0x9acb0101,0xfb850108,0x9acc0101,0xfb850108,0x9acd0101,
	0xfb850108,0x9ace0101,0xfb850108,0x9acf0101,0xfb850108,0x9ad00101,0xfb850108,0x9ad10101,
	0xfb850108,0x9ad20101,0xfb850108,0x9ad30101,0xfb850108,0x9ad40101,0xfb850108,0x9ad50101,
	0xfb850108,0x9ad60101,0xfb850108,0x9ad70101,0xfb850108,0x9ad80101,0xfb850108,0x9ad90101,
	0xfb850108,0x9ada0101,0xfb850108,0x9adb0101,0xfb850108,0x9adc0101,0xfb850108,0x9add0101,
	0xfb850108,0x9ade0101,0xfb850108,0x9adf0101,0xfb850108,0x9ae00101,0xfb850108,0x9ae10101,
	0xfb850108,0x9ae20101,0xfb850108,0x9ae30101,0xfb850108,0x9ae40101,0xfb850108,0x9ae50101,
	0xfb850108,0x9ae60101,0xfb850108,0x9ae70101,0xfb850108,0x9ae80101,0xfb850108,0x9ae90101,
	0xfb850108,0x9aea0101,0xfb850108,0x9aeb0101,0xfb850108,0x9aec0101,0xfb850108,0x9aed0101,
	0xfb850108,0x9aee0101,0xfb850108,0x9aef0101,0xfb850108,0x9af00101,0xfb850108,0x9af10101,
	0xfb850108,0x9af20101,0xfb850108,0x9af30101,0xfb850108,0x9af40101,0xfb850108,0x9af50101,
	0xfb850108,0x9af60101,0xfb850108,0x9af70101,0xfb850108,0x9af80101,0xfb850108,0x9af90101,
	0xfb850108,0x9afa0101,0xfb850108,0x9afb0101,0xfb850108,0x9afc0101,0xfb850108,0x9afd0101,
	0xfb850108,0x9afe0101,0xfb850108,0x9aff0101,0xfb850108,0x9b000101,0xfb850108,0x9b010101,
	0xfb850108,0x9b020101,0xfb850108,0x9b030101,0xfb850108,0x9b040101,0xfb850108,0x9b050101,
	0xfb850108,0x9b060101,0xfb850108,0x9b070101,0xfb850108,0x9b080101,0xfb850108,0x9b090101,
	0xfb850108,0x9b0a0101,0xfb850108,0x9b0b0101,0xfb850108,0x9b0c0101,0xfb850108,0x9b0d0101,
	0xfb850108,0x9b0e0101,0xfb850108,0x9b0f0101,0xfb850108,0x9b100101,0xfb850108,0x9b110101,
	0xfb850108,0x9b120101,0xfb850108,0x9b130101,0xfb850108,0x9b140101,0xfb850108,0x9b150101,
	0xfb850108,0x9b160101,0xfb850108,0x9b170101,0xfb850108,0x9b180101,0xfb850108,0x9b190101,
	0xfb850108,0x9b1a0101,0xfb850108,0x9b1b0101,0xfb850108,0x9b1c0101,0xfb850108,0x9b1d0101,
	0xfb850108,0x9b1e0101,0xfb850108,0x9b1f0101,0xfb850108,0x9b200101,0xfb850108,0x9b210101,
	0xfb850108,0x9b220101,0xfb850108,0x9b230101,0xfb850108,0x9b240101,0xfb850108,0x9b250101,
	0xfb850108,0x9b260101,0xfb850108,0x9b270101,0xfb850108,0x9b280101,0xfb850108,0x9b290101,
	0xfb850108,0x9b2a0101,0xfb850108,0x9b2b0101,0xfb850108,0x9b2c0101,0xfb850108,0x9b2d0101,
	0xfb850108,0x9b2e0101,0xfb850108,0x9b2f0101,0xfb850108,0x9b300101,0xfb850108,0x9b310101,
	0xfb850108,0x9b320101,0xfb850108,0x9b330101,0xfb850108,0x9b340101,0xfb850108,0x9b350101,
	0xfb850108,0x9b360101,0xfb850108,0x9b370101,0xfb850108,0x9b380101,0xfb850108,0x9b390101,
	0xfb850108,0x9b3a0101,0xfb850108,0x9b3b0101,0xfb850108,0x9b3c0101,0xfb850108,0x9b3d0101,
	0xfb850108,0x9b3e0101,0xfb850108,0x9b3f0101,0xfb850108,0x9b400101,0xfb850108,0x9b410101,
	0xfb850108,0x9b420101,0xfb850108,0x9b430101,0xfb850108,0x9b440101,0xfb850108,0x9b450101,
	0xfb850108,0x9b460101,0xfb850108,0x9b470101,0xfb850108,0x9b480101,0xfb850108,0x9b490101,
	0xfb850108,0x9b4a0101,0xfb850108,0x9b4b0101,0xfb850108,0x9b4c0101,0xfb850108,0x9b4d0101,
	0xfb850108,0x9b4e0101,0xfb850108,0x9b4f0101,0xfb850108,0x9b500101,0xfb850108,0x9b510101,
	0xfb850108,0x9b520101,0xfb850108,0x9b530101,0xfb850108,0x9b540101,0xfb850108,0x9b550101,
	0xfb850108,0x9b560101,0xfb850108,0x9b570101,0xfb850108,0x9b580101,0xfb850108,0x9b590101,
	0xfb850108,0x9b5a0101,0xfb850108,0x9b5b0101,0xfb850108,0x9b5c0101,0xfb850108,0x9b5d0101,
	0xfb850108,0x9b5e0101,0xfb850108,0x9b5f0101,0xfb850108,0x9b600101,0xfb850108,0x9b610101,
	0xfb850108,0x9b620101,0xfb850108,0x9b630101,0xfb850108,0x9b640101,0xfb850108,0x9b650101,
	0xfb850108,0x9b660101,0xfb850108,0x9b670101,0xfb850108,0x9b680101,0xfb850108,0x9b690101,
	0xfb850108,0x9b6a0101,0xfb850108,0x9b6b0101,0xfb850108,0x9b6c0101,0xfb850108,0x9b6d0101,
	0xfb850108,0x9b6e0101,0xfb850108,0x9b6f0101,0xfb850108,0x9b700101,0xfb850108,0x9b710101,
	0xfb850108,0x9b720101,0xfb850108,0x9b730101,0xfb850108,0x9b740101,0xfb850108,0x9b750101,
	0xfb850108,0x9b760101,0xfb850108,0x9b770101,0xfb850108,0x9b780101,0xfb850108,0x9b790101,
	0xfb850108,0x9b7a0101,0xfb850108,0x9b7b0101,0xfb850108,0x9b7c0101,0xfb850108,0x9b7d0101,
	0xfb850108,0x9b7e0101,0xfb850108,0x9b7f0101,0xfb850108,0x9b800101,0xfb850108,0x9b810101,
	0xfb850108,0x9b820101,0xfb850108,0x9b830101,0xfb850108,0x9b840101,0xfb850108,0x9b850101,
	0xfb850108,0x9b860101,0xfb850108,0x9b870101,0xfb850108,0x9b880101,0xfb850108,0x9b890101,
	0xfb850108,0x9b8a0101,0xfb850108,0x9b8b0101,0xfb850108,0x9b8c0101,0xfb850108,0x9b8d0101,
	0xfb850108,0x9b8e0101,0xfb850108,0x9b8f0101,0xfb850108,0x9b900101,0xfb850108,0x9b910101,
	0xfb850108,0x9b920101,0xfb850108,0x9b930101,0xfb850108,0x9b940101,0xfb850108,0x9b950101,
	0xfb850108,0x9b960101,0xfb850108,0x9b970101,0xfb850108,0x9b980101,0xfb850108,0x9b990101,
	0xfb850108,0x9b9a0101,0xfb850108,0x9b9b0101,0xfb850108,0x9b9c0101,0xfb850108,0x9b9d0101,
	0xfb850108,0x9b9e0101,0xfb850108,0x9b9f0101,0xfb850108,0x9ba00101,0xfb850108,0x9ba10101,
	0xfb850108,0x9ba20101,0xfb850108,0x9ba30101,0xfb850108,0x9ba40101,0xfb850108,0x9ba50101,
	0xfb850108,0x9ba60101,0xfb850108,0x9ba70101,0xfb850108,0x9ba80101,0xfb850108,0x9ba90101,
	0xfb850108,0x9baa0101,0xfb850108,0x9bab0101,0xfb850108,0x9bac0101,0xfb850108,0x9bad0101,
	0xfb850108,0x9bae0101,0xfb850108,0x9baf0101,0xfb850108,0x9bb00101,0xfb850108,0x9bb10101,
	0xfb850108,0x9bb20101,0xfb850108,0x9bb30101,0xfb850108,0x9bb40101,0xfb850108,0x9bb50101,
	0xfb850108,0x9bb60101,0xfb850108,0x9bb70101,0xfb850108,0x9bb80101,0xfb850108,0x9bb90101,
	0xfb850108,0x9bba0101,0xfb850108,0x9bbb0101,0xfb850108,0x9bbc0101,0xfb850108,0x9bbd0101,
	0xfb850108,0x9bbe0101,0xfb850108,0x9bbf0101,0xfb850108,0x9bc00101,0xfb850108,0x9bc10101,
	0xfb850108,0x9bc20101,0xfb850108,0x9bc30101,0xfb850108,0x9bc40101,0xfb850108,0x9bc50101,
	0xfb850108,0x9bc60101,0xfb850108,0x9bc70101,0xfb850108,0x9bc80101,0xfb850108,0x9bc90101,
	0xfb850108,0x9bca0101,0xfb850108,0x9bcb0101,0xfb850108,0x9bcc0101,0xfb850108,0x9bcd0101,
	0xfb850108,0x9bce0101,0xfb850108,0x9bcf0101,0xfb850108,0x9bd00101,0xfb850108,0x9bd10101,
	0xfb850108,0x9bd20101,0xfb850108,0x9bd30101,0xfb850108,0x9bd40101,0xfb850108,0x9bd50101,
	0xfb850108,0x9bd60101,0xfb850108,0x9bd70101,0xfb850108,0x9bd80101,0xfb850108,0x9bd90101,
	0xfb850108,0x9bda0101,0xfb850108,0x9bdb0101,0xfb850108,0x9bdc0101,0xfb850108,0x9bdd0101,
	0xfb850108,0x9bde0101,0xfb850108,0x9bdf0101,0xfb850108,0x9be00101,0xfb850108,0x9be10101,
	0xfb850108,0x9be20101,0xfb850108,0x9be30101,0xfb850108,0x9be40101,0xfb850108,0x9be50101,
	0xfb850108,0x9be60101,0xfb850108,0x9be70101,0xfb850108,0x9be80101,0xfb850108,0x9be90101,
	0xfb850108,0x9bea0101,0xfb850108,0x9beb0101,0xfb850108,0x9bec0101,0xfb850108,0x9bed0101,
	0xfb850108,0x9bee0101,0xfb850108,0x9bef0101,0xfb850108,0x9bf00101,0xfb850108,0x9bf10101,
	0xfb850108,0x9bf20101,0xfb850108,0x9bf30101,0xfb850108,0x9bf40101,0xfb850108,0x9bf50101,
	0xfb850108,0x9bf60101,0xfb850108,0x9bf70101,0xfb850108,0x9bf80101,0xfb850108,0x9bf90101,
	0xfb850108,0x9bfa0101,0xfb850108,0x9bfb0101,0xfb850108,0x9bfc0101,0xfb850108,0x9bfd0101,
	0xfb850108,0x9bfe0101,0xfb850108,0x9bff0101,0xfb850108,0x9c000101,0xfb850108,0x9c010101,
	0xfb850108,0x9c020101,0xfb850108,0x9c030101,0xfb850108,0x9c040101,0xfb850108,0x9c050101,
	0xfb850108,0x9c060101,0xfb850108,0x9c070101,0xfb850108,0x9c080101,0xfb850108,0x9c090101,
	0xfb850108,0x9c0a0101,0xfb850108,0x9c0b0101,0xfb850108,0x9c0c0101,0xfb850108,0x9c0d0101,
	0xfb850108,0x9c0e0101,0xfb850108,0x9c0f0101,0xfb850108,0x9c100101,0xfb850108,0x9c110101,
	0xfb850108,0x9c120101,0xfb850108,0x9c130101,0xfb850108,0x9c140101,0xfb850108,0x9c150101,
	0xfb850108,0x9c160101,0xfb850108,0x9c170101,0xfb850108,0x9c180101,0xfb850108,0x9c190101,
	0xfb850108,0x9c1a0101,0xfb850108,0x9c1b0101,0xfb850108,0x9c1c0101,0xfb850108,0x9c1d0101,
	0xfb850108,0x9c1e0101,0xfb850108,0x9c1f0101,0xfb850108,0x9c200101,0xfb850108,0x9c210101,
	0xfb850108,0x9c220101,0xfb850108,0x9c230101,0xfb850108,0x9c240101,0xfb850108,0x9c250101,
	0xfb850108,0x9c260101,0xfb850108,0x9c270101,0xfb850108,0x9c280101,0xfb850108,0x9c290101,
	0xfb850108,0x9c2a0101,0xfb850108,0x9c2b0101,0xfb850108,0x9c2c0101,0xfb850108,0x9c2d0101,
	0xfb850108,0x9c2e0101,0xfb850108,0x9c2f0101,0xfb850108,0x9c300101,0xfb850108,0x9c310101,
	0xfb850108,0x9c320101,0xfb850108,0x9c330101,0xfb850108,0x9c340101,0xfb850108,0x9c350101,
	0xfb850108,0x9c360101,0xfb850108,0x9c370101,0xfb850108,0x9c380101,0xfb850108,0x9c390101,
	0xfb850108,0x9c3a0101,0xfb850108,0x9c3b0101,0xfb850108,0x9c3c0101,0xfb850108,0x9c3d0101,
	0xfb850108,0x9c3e0101,0xfb850108,0x9c3f0101,0xfb850108,0x9c400101,0xfb850108,0x9c410101,
	0xfb850108,0x9c420101,0xfb850108,0x9c430101,0xfb850108,0x9c440101,0xfb850108,0x9c450101,
	0xfb850108,0x9c460101,0xfb850108,0x9c470101,0xfb850108,0x9c480101,0xfb850108,0x9c490101,
	0xfb850108,0x9c4a0101,0xfb850108,0x9c4b0101,0xfb850108,0x9c4c0101,0xfb850108,0x9c4d0101,
	0xfb850108,0x9c4e0101,0xfb850108,0x9c4f0101,0xfb850108,0x9c500101,0xfb850108,0x9c510101,
	0xfb850108,0x9c520101,0xfb850108,0x9c530101,0xfb850108,0x9c540101,0xfb850108,0x9c550101,
	0xfb850108,0x9c560101,0xfb850108,0x9c570101,0xfb850108,0x9c580101,0xfb850108,0x9c590101,
	0xfb850108,0x9c5a0101,0xfb850108,0x9c5b0101,0xfb850108,0x9c5c0101,0xfb850108,0x9c5d0101,
	0xfb850108,0x9c5e0101,0xfb850108,0x9c5f0101,0xfb850108,0x9c600101,0xfb850108,0x9c610101,
	0xfb850108,0x9c620101,0xfb850108,0x9c630101,0xfb850108,0x9c640101,0xfb850108,0x9c650101,
	0xfb850108,0x9c660101,0xfb850108,0x9c670101,0xfb850108,0x9c680101,0xfb850108,0x9c690101,
	0xfb850108,0x9c6a0101,0xfb850108,0x9c6b0101,0xfb850108,0x9c6c0101,0xfb850108,0x9c6d0101,
	0xfb850108,0x9c6e0101,0xfb850108,0x9c6f0101,0xfb850108,0x9c700101,0xfb850108,0x9c710101,
	0xfb850108,0x9c720101,0xfb850108,0x9c730101,0xfb850108,0x9c740101,0xfb850108,0x9c750101,
	0xfb850108,0x9c760101,0xfb850108,0x9c770101,0xfb850108,0x9c780101,0xfb850108,0x9c790101,
	0xfb850108,0x9c7a0101,0xfb850108,0x9c7b0101,0xfb850108,0x9c7c0101,0xfb850108,0x9c7d0101,
	0xfb850108,0x9c7e0101,0xfb850108,0x9c7f0101,0xfb850108,0x9c800101,0xfb850108,0x9c810101,
	0xfb850108,0x9c820101,0xfb850108,0x9c830101,0xfb850108,0x9c840101,0xfb850108,0x9c850101,
	0xfb850108,0x9c860101,0xfb850108,0x9c870101,0xfb850108,0x9c880101,0xfb850108,0x9c890101,
	0xfb850108,0x9c8a0101,0xfb850108,0x9c8b0101,0xfb850108,0x9c8c0101,0xfb850108,0x9c8d0101,
	0xfb850108,0x9c8e0101,0xfb850108,0x9c8f0101,0xfb850108,0x9c900101,0xfb850108,0x9c910101,
	0xfb850108,0x9c920101,0xfb850108,0x9c930101,0xfb850108,0x9c940101,0xfb850108,0x9c950101,
	0xfb850108,0x9c960101,0xfb850108,0x9c970101,0xfb850108,0x9c980101,0xfb850108,0x9c990101,
	0xfb850108,0x9c9a0101,0xfb850108,0x9c9b0101,0xfb850108,0x9c9c0101,0xfb850108,0x9c9d0101,
	0xfb850108,0x9c9e0101,0xfb850108,0x9c9f0101,0xfb850108,0x9ca00101,0xfb850108,0x9ca10101,
	0xfb850108,0x9ca20101,0xfb850108,0x9ca30101,0xfb850108,0x9ca40101,0xfb850108,0x9ca50101,
	0xfb850108,0x9ca60101,0xfb850108,0x9ca70101,0xfb850108,0x9ca80101,0xfb850108,0x9ca90101,
	0xfb850108,0x9caa0101,0xfb850108,0x9cab0101,0xfb850108,0x9cac0101,0xfb850108,0x9cad0101,
	0xfb850108,0x9cae0101,0xfb850108,0x9caf0101,0xfb850108,0x9cb00101,0xfb850108,0x9cb10101,
	0xfb850108,0x9cb20101,0xfb850108,0x9cb30101,0xfb850108,0x9cb40101,0xfb850108,0x9cb50101,
	0xfb850108,0x9cb60101,0xfb850108,0x9cb70101,0xfb850108,0x9cb80101,0xfb850108,0x9cb90101,
	0xfb850108,0x9cba0101,0xfb850108,0x9cbb0101,0xfb850108,0x9cbc0101,0xfb850108,0x9cbd0101,
	0xfb850108,0x9cbe0101,0xfb850108,0x9cbf0101,0xfb850108,0x9cc00101,0xfb850108,0x9cc10101,
	0xfb850108,0x9cc20101,0xfb850108,0x9cc30101,0xfb850108,0x9cc40101,0xfb850108,0x9cc50101,
	0xfb850108,0x9cc60101,0xfb850108,0x9cc70101,0xfb850108,0x9cc80101,0xfb850108,0x9cc90101,
	0xfb850108,0x9cca0101,0xfb850108,0x9ccb0101,0xfb850108,0x9ccc0101,0xfb850108,0x9ccd0101,
	0xfb850108,0x9cce0101,0xfb850108,0x9ccf0101,0xfb850108,0x9cd00101,0xfb850108,0x9cd10101,
	0xfb850108,0x9cd20101,0xfb850108,0x9cd30101,0xfb850108,0x9cd40101,0xfb850108,0x9cd50101,
	0xfb850108,0x9cd60101,0xfb850108,0x9cd70101,0xfb850108,0x9cd80101,0xfb850108,0x9cd90101,
	0xfb850108,0x9cda0101,0xfb850108,0x9cdb0101,0xfb850108,0x9cdc0101,0xfb850108,0x9cdd0101,
	0xfb850108,0x9cde0101,0xfb850108,0x9cdf0101,0xfb850108,0x9ce00101,0xfb850108,0x9ce10101,
	0xfb850108,0x9ce20101,0xfb850108,0x9ce30101,0xfb850108,0x9ce40101,0xfb850108,0x9ce50101,
	0xfb850108,0x9ce60101,0xfb850108,0x9ce70101,0xfb850108,0x9ce80101,0xfb850108,0x9ce90101,
	0xfb850108,0x9cea0101,0xfb850108,0x9ceb0101,0xfb850108,0x9cec0101,0xfb850108,0x9ced0101,
	0xfb850108,0x9cee0101,0xfb850108,0x9cef0101,0xfb850108,0x9cf00101,0xfb850108,0x9cf10101,
	0xfb850108,0x9cf20101,0xfb850108,0x9cf30101,0xfb850108,0x9cf40101,0xfb850108,0x9cf50101,
	0xfb850108,0x9cf60101,0xfb850108,0x9cf70101,0xfb850108,0x9cf80101,0xfb850108,0x9cf90101,
	0xfb850108,0x9cfa0101,0xfb850108,0x9cfb0101,0xfb850108,0x9cfc0101,0xfb850108,0x9cfd0101,
	0xfb850108,0x9cfe0101,0xfb850108,0x9cff0101,0xfb850108,0x9d000101,0xfb850108,0x9d010101,
	0xfb850108,0x9d020101,0xfb850108,0x9d030101,0xfb850108,0x9d040101,0xfb850108,0x9d050101,
	0xfb850108,0x9d060101,0xfb850108,0x9d070101,0xfb850108,0x9d080101,0xfb850108,0x9d090101,
	0xfb850108,0x9d0a0101,0xfb850108,0x9d0b0101,0xfb850108,0x9d0c0101,0xfb850108,0x9d0d0101,
	0xfb850108,0x9d0e0101,0xfb850108,0x9d0f0101,0xfb850108,0x9d100101,0xfb850108,0x9d110101,
	0xfb850108,0x9d120101,0xfb850108,0x9d130101,0xfb850108,0x9d140101,0xfb850108,0x9d150101,
	0xfb850108,0x9d160101,0xfb850108,0x9d170101,0xfb850108,0x9d180101,0xfb850108,0x9d190101,
	0xfb850108,0x9d1a0101,0xfb850108,0x9d1b0101,0xfb850108,0x9d1c0101,0xfb850108,0x9d1d0101,
	0xfb850108,0x9d1e0101,0xfb850108,0x9d1f0101,0xfb850108,0x9d200101,0xfb850108,0x9d210101,
	0xfb850108,0x9d220101,0xfb850108,0x9d230101,0xfb850108,0x9d240101,0xfb850108,0x9d250101,
	0xfb850108,0x9d260101,0xfb850108,0x9d270101,0xfb850108,0x9d280101,0xfb850108,0x9d290101,
	0xfb850108,0x9d2a0101,0xfb850108,0x9d2b0101,0xfb850108,0x9d2c0101,0xfb850108,0x9d2d0101,
	0xfb850108,0x9d2e0101,0xfb850108,0x9d2f0101,0xfb850108,0x9d300101,0xfb850108,0x9d310101,
	0xfb850108,0x9d320101,0xfb850108,0x9d330101,0xfb850108,0x9d340101,0xfb850108,0x9d350101,
	0xfb850108,0x9d360101,0xfb850108,0x9d370101,0xfb850108,0x9d380101,0xfb850108,0x9d390101,
	0xfb850108,0x9d3a0101,0xfb850108,0x9d3b0101,0xfb850108,0x9d3c0101,0xfb850108,0x9d3d0101,
	0xfb850108,0x9d3e0101,0xfb850108,0x9d3f0101,0xfb850108,0x9d400101,0xfb850108,0x9d410101,
	0xfb850108,0x9d420101,0xfb850108,0x9d430101,0xfb850108,0x9d440101,0xfb850108,0x9d450101,
	0xfb850108,0x9d460101,0xfb850108,0x9d470101,0xfb850108,0x9d480101,0xfb850108,0x9d490101,
	0xfb850108,0x9d4a0101,0xfb850108,0x9d4b0101,0xfb850108,0x9d4c0101,0xfb850108,0x9d4d0101,
	0xfb850108,0x9d4e0101,0xfb850108,0x9d4f0101,0xfb850108,0x9d500101,0xfb850108,0x9d510101,
	0xfb850108,0x9d520101,0xfb850108,0x9d530101,0xfb850108,0x9d540101,0xfb850108,0x9d550101,
	0xfb850108,0x9d560101,0xfb850108,0x9d570101,0xfb850108,0x9d580101,0xfb850108,0x9d590101,
	0xfb850108,0x9d5a0101,0xfb850108,0x9d5b0101,0xfb850108,0x9d5c0101,0xfb850108,0x9d5d0101,
	0xfb850108,0x9d5e0101,0xfb850108,0x9d5f0101,0xfb850108,0x9d600101,0xfb850108,0x9d610101,
	0xfb850108,0x9d620101,0xfb850108,0x9d630101,0xfb850108,0x9d640101,0xfb850108,0x9d650101,
	0xfb850108,0x9d660101,0xfb850108,0x9d670101,0xfb850108,0x9d680101,0xfb850108,0x9d690101,
	0xfb850108,0x9d6a0101,0xfb850108,0x9d6b0101,0xfb850108,0x9d6c0101,0xfb850108,0x9d6d0101,
	0xfb850108,0x9d6e0101,0xfb850108,0x9d6f0101,0xfb850108,0x9d700101,0xfb850108,0x9d710101,
	0xfb850108,0x9d720101,0xfb850108,0x9d730101,0xfb850108,0x9d740101,0xfb850108,0x9d750101,
	0xfb850108,0x9d760101,0xfb850108,0x9d770101,0xfb850108,0x9d780101,0xfb850108,0x9d790101,
	0xfb850108,0x9d7a0101,0xfb850108,0x9d7b0101,0xfb850108,0x9d7c0101,0xfb850108,0x9d7d0101,
	0xfb850108,0x9d7e0101,0xfb850108,0x9d7f0101,0xfb850108,0x9d800101,0xfb850108,0x9d810101,
	0xfb850108,0x9d820101,0xfb850108,0x9d830101,0xfb850108,0x9d840101,0xfb850108,0x9d850101,
	0xfb850108,0x9d860101,0xfb850108,0x9d870101,0xfb850108,0x9d880101,0xfb850108,0x9d890101,
	0xfb850108,0x9d8a0101,0xfb850108,0x9d8b0101,0xfb850108,0x9d8c0101,0xfb850108,0x9d8d0101,
	0xfb850108,0x9d8e0101,0xfb850108,0x9d8f0101,0xfb850108,0x9d900101,0xfb850108,0x9d910101,
	0xfb850108,0x9d920101,0xfb850108,0x9d930101,0xfb850108,0x9d940101,0xfb850108,0x9d950101,
	0xfb850108,0x9d960101,0xfb850108,0x9d970101,0xfb850108,0x9d980101,0xfb850108,0x9d990101,
	0xfb850108,0x9d9a0101,0xfb850108,0x9d9b0101,0xfb850108,0x9d9c0101,0xfb850108,0x9d9d0101,
	0xfb850108,0x9d9e0101,0xfb850108,0x9d9f0101,0xfb850108,0x9da00101,0xfb850108,0x9da10101,
	0xfb850108,0x9da20101,0xfb850108,0x9da30101,0xfb850108,0x9da40101,0xfb850108,0x9da50101,
	0xfb850108,0x9da60101,0xfb850108,0x9da70101,0xfb850108,0x9da80101,0xfb850108,0x9da90101,
	0xfb850108,0x9daa0101,0xfb850108,0x9dab0101,0xfb850108,0x9dac0101,0xfb850108,0x9dad0101,
	0xfb850108,0x9dae0101,0xfb850108,0x9daf0101,0xfb850108,0x9db00101,0xfb850108,0x9db10101,
	0xfb850108,0x9db20101,0xfb850108,0x9db30101,0xfb850108,0x9db40101,0xfb850108,0x9db50101,
	0xfb850108,0x9db60101,0xfb850108,0x9db70101,0xfb850108,0x9db80101,0xfb850108,0x9db90101,
	0xfb850108,0x9dba0101,0xfb850108,0x9dbb0101,0xfb850108,0x9dbc0101,0xfb850108,0x9dbd0101,
	0xfb850108,0x9dbe0101,0xfb850108,0x9dbf0101,0xfb850108,0x9dc00101,0xfb850108,0x9dc10101,
	0xfb850108,0x9dc20101,0xfb850108,0x9dc30101,0xfb850108,0x9dc40101,0xfb850108,0x9dc50101,
	0xfb850108,0x9dc60101,0xfb850108,0x9dc70101,0xfb850108,0x9dc80101,0xfb850108,0x9dc90101,
	0xfb850108,0x9dca0101,0xfb850108,0x9dcb0101,0xfb850108,0x9dcc0101,0xfb850108,0x9dcd0101,
	0xfb850108,0x9dce0101,0xfb850108,0x9dcf0101,0xfb850108,0x9dd00101,0xfb850108,0x9dd10101,
	0xfb850108,0x9dd20101,0xfb850108,0x9dd30101,0xfb850108,0x9dd40101,0xfb850108,0x9dd50101,
	0xfb850108,0x9dd60101,0xfb850108,0x9dd70101,0xfb850108,0x9dd80101,0xfb850108,0x9dd90101,
	0xfb850108,0x9dda0101,0xfb850108,0x9ddb0101,0xfb850108,0x9ddc0101,0xfb850108,0x9ddd0101,
	0xfb850108,0x9dde0101,0xfb850108,0x9ddf0101,0xfb850108,0x9de00101,0xfb850108,0x9de10101,
	0xfb850108,0x9de20101,0xfb850108,0x9de30101,0xfb850108,0x9de40101,0xfb850108,0x9de50101,
	0xfb850108,0x9de60101,0xfb850108,0x9de70101,0xfb850108,0x9de80101,0xfb850108,0x9de90101,
	0xfb850108,0x9dea0101,0xfb850108,0x9deb0101,0xfb850108,0x9dec0101,0xfb850108,0x9ded0101,
	0xfb850108,0x9dee0101,0xfb850108,0x9def0101,0xfb850108,0x9df00101,0xfb850108,0x9df10101,
	0xfb850108,0x9df20101,0xfb850108,0x9df30101,0xfb850108,0x9df40101,0xfb850108,0x9df50101,
	0xfb850108,0x9df60101,0xfb850108,0x9df70101,0xfb850108,0x9df80101,0xfb850108,0x9df90101,
	0xfb850108,0x9dfa0101,0xfb850108,0x9dfb0101,0xfb850108,0x9dfc0101,0xfb850108,0x9dfd0101,
	0xfb850108,0x9dfe0101,0xfb850108,0x9dff0101,0xfb850108,0x9e000101,0xfb850108,0x9e010101,
	0xfb850108,0x9e020101,0xfb850108,0x9e030101,0xfb850108,0x9e040101,0xfb850108,0x9e050101,
	0xfb850108,0x9e060101,0xfb850108,0x9e070101,0xfb850108,0x9e080101,0xfb850108,0x9e090101,
	0xfb850108,0x9e0a0101,0xfb850108,0x9e0b0101,0xfb850108,0x9e0c0101,0xfb850108,0x9e0d0101,
	0xfb850108,0x9e0e0101,0xfb850108,0x9e0f0101,0xfb850108,0x9e100101,0xfb850108,0x9e110101,
	0xfb850108,0x9e120101,0xfb850108,0x9e130101,0xfb850108,0x9e140101,0xfb850108,0x9e150101,
	0xfb850108,0x9e160101,0xfb850108,0x9e170101,0xfb850108,0x9e180101,0xfb850108,0x9e190101,
	0xfb850108,0x9e1a0101,0xfb850108,0x9e1b0101,0xfb850108,0x9e1c0101,0xfb850108,0x9e1d0101,
	0xfb850108,0x9e1e0101,0xfb850108,0x9e1f0101,0xfb850108,0x9e200101,0xfb850108,0x9e210101,
	0xfb850108,0x9e220101,0xfb850108,0x9e230101,0xfb850108,0x9e240101,0xfb850108,0x9e250101,
	0xfb850108,0x9e260101,0xfb850108,0x9e270101,0xfb850108,0x9e280101,0xfb850108,0x9e290101,
	0xfb850108,0x9e2a0101,0xfb850108,0x9e2b0101,0xfb850108,0x9e2c0101,0xfb850108,0x9e2d0101,
	0xfb850108,0x9e2e0101,0xfb850108,0x9e2f0101,0xfb850108,0x9e300101,0xfb850108,0x9e310101,
	0xfb850108,0x9e320101,0xfb850108,0x9e330101,0xfb850108,0x9e340101,0xfb850108,0x9e350101,
	0xfb850108,0x9e360101,0xfb850108,0x9e370101,0xfb850108,0x9e380101,0xfb850108,0x9e390101,
	0xfb850108,0x9e3a0101,0xfb850108,0x9e3b0101,0xfb850108,0x9e3c0101,0xfb850108,0x9e3d0101,
	0xfb850108,0x9e3e0101,0xfb850108,0x9e3f0101,0xfb850108,0x9e400101,0xfb850108,0x9e410101,
	0xfb850108,0x9e420101,0xfb850108,0x9e430101,0xfb850108,0x9e440101,0xfb850108,0x9e450101,
	0xfb850108,0x9e460101,0xfb850108,0x9e470101,0xfb850108,0x9e480101,0xfb850108,0x9e490101,
	0xfb850108,0x9e4a0101,0xfb850108,0x9e4b0101,0xfb850108,0x9e4c0101,0xfb850108,0x9e4d0101,
	0xfb850108,0x9e4e0101,0xfb850108,0x9e4f0101,0xfb850108,0x9e500101,0xfb850108,0x9e510101,
	0xfb850108,0x9e520101,0xfb850108,0x9e530101,0xfb850108,0x9e540101,0xfb850108,0x9e550101,
	0xfb850108,0x9e560101,0xfb850108,0x9e570101,0xfb850108,0x9e580101,0xfb850108,0x9e590101,
	0xfb850108,0x9e5a0101,0xfb850108,0x9e5b0101,0xfb850108,0x9e5c0101,0xfb850108,0x9e5d0101,
	0xfb850108,0x9e5e0101,0xfb850108,0x9e5f0101,0xfb850108,0x9e600101,0xfb850108,0x9e610101,
	0xfb850108,0x9e620101,0xfb850108,0x9e630101,0xfb850108,0x9e640101,0xfb850108,0x9e650101,
	0xfb850108,0x9e660101,0xfb850108,0x9e670101,0xfb850108,0x9e680101,0xfb850108,0x9e690101,
	0xfb850108,0x9e6a0101,0xfb850108,0x9e6b0101,0xfb850108,0x9e6c0101,0xfb850108,0x9e6d0101,
	0xfb850108,0x9e6e0101,0xfb850108,0x9e6f0101,0xfb850108,0x9e700101,0xfb850108,0x9e710101,
	0xfb850108,0x9e720101,0xfb850108,0x9e730101,0xfb850108,0x9e740101,0xfb850108,0x9e750101,
	0xfb850108,0x9e760101,0xfb850108,0x9e770101,0xfb850108,0x9e780101,0xfb850108,0x9e790101,
	0xfb850108,0x9e7a0101,0xfb850108,0x9e7b0101,0xfb850108,0x9e7c0101,0xfb850108,0x9e7d0101,
	0xfb850108,0x9e7e0101,0xfb850108,0x9e7f0101,0xfb850108,0x9e800101,0xfb850108,0x9e810101,
	0xfb850108,0x9e820101,0xfb850108,0x9e830101,0xfb850108,0x9e840101,0xfb850108,0x9e850101,
	0xfb850108,0x9e860101,0xfb850108,0x9e870101,0xfb850108,0x9e880101,0xfb850108,0x9e890101,
	0xfb850108,0x9e8a0101,0xfb850108,0x9e8b0101,0xfb850108,0x9e8c0101,0xfb850108,0x9e8d0101,
	0xfb850108,0x9e8e0101,0xfb850108,0x9e8f0101,0xfb850108,0x9e900101,0xfb850108,0x9e910101,
	0xfb850108,0x9e920101,0xfb850108,0x9e930101,0xfb850108,0x9e940101,0xfb850108,0x9e950101,
	0xfb850108,0x9e960101,0xfb850108,0x9e970101,0xfb850108,0x9e980101,0xfb850108,0x9e990101,
	0xfb850108,0x9e9a0101,0xfb850108,0x9e9b0101,0xfb850108,0x9e9c0101,0xfb850108,0x9e9d0101,
	0xfb850108,0x9e9e0101,0xfb850108,0x9e9f0101,0xfb850108,0x9ea00101,0xfb850108,0x9ea10101,
	0xfb850108,0x9ea20101,0xfb850108,0x9ea30101,0xfb850108,0x9ea40101,0xfb850108,0x9ea50101,
	0xfb850108,0x9ea60101,0xfb850108,0x9ea70101,0xfb850108,0x9ea80101,0xfb850108,0x9ea90101,
	0xfb850108,0x9eaa0101,0xfb850108,0x9eab0101,0xfb850108,0x9eac0101,0xfb850108,0x9ead0101,
	0xfb850108,0x9eae0101,0xfb850108,0x9eaf0101,0xfb850108,0x9eb00101,0xfb850108,0x9eb10101,
	0xfb850108,0x9eb20101,0xfb850108,0x9eb30101,0xfb850108,0x9eb40101,0xfb850108,0x9eb50101,
	0xfb850108,0x9eb60101,0xfb850108,0x9eb70101,0xfb850108,0x9eb80101,0xfb850108,0x9eb90101,
	0xfb850108,0x9eba0101,0xfb850108,0x9ebb0101,0xfb850108,0x9ebc0101,0xfb850108,0x9ebd0101,
	0xfb850108,0x9ebe0101,0xfb850108,0x9ebf0101,0xfb850108,0x9ec00101,0xfb850108,0x9ec10101,
	0xfb850108,0x9ec20101,0xfb850108,0x9ec30101,0xfb850108,0x9ec40101,0xfb850108,0x9ec50101,
	0xfb850108,0x9ec60101,0xfb850108,0x9ec70101,0xfb850108,0x9ec80101,0xfb850108,0x9ec90101,
	0xfb850108,0x9eca0101,0xfb850108,0x9ecb0101,0xfb850108,0x9ecc0101,0xfb850108,0x9ecd0101,
	0xfb850108,0x9ece0101,0xfb850108,0x9ecf0101,0xfb850108,0x9ed00101,0xfb850108,0x9ed10101,
	0xfb850108,0x9ed20101,0xfb850108,0x9ed30101,0xfb850108,0x9ed40101,0xfb850108,0x9ed50101,
	0xfb850108,0x9ed60101,0xfb850108,0x9ed70101,0xfb850108,0x9ed80101,0xfb850108,0x9ed90101,
	0xfb850108,0x9eda0101,0xfb850108,0x9edb0101,0xfb850108,0x9edc0101,0xfb850108,0x9edd0101,
	0xfb850108,0x9ede0101,0xfb850108,0x9edf0101,0xfb850108,0x9ee00101,0xfb850108,0x9ee10101,
	0xfb850108,0x9ee20101,0xfb850108,0x9ee30101,0xfb850108,0x9ee40101,0xfb850108,0x9ee50101,
	0xfb850108,0x9ee60101,0xfb850108,0x9ee70101,0xfb850108,0x9ee80101,0xfb850108,0x9ee90101,
	0xfb850108,0x9eea0101,0xfb850108,0x9eeb0101,0xfb850108,0x9eec0101,0xfb850108,0x9eed0101,
	0xfb850108,0x9eee0101,0xfb850108,0x9eef0101,0xfb850108,0x9ef00101,0xfb850108,0x9ef10101,
	0xfb850108,0x9ef20101,0xfb850108,0x9ef30101,0xfb850108,0x9ef40101,0xfb850108,0x9ef50101,
	0xfb850108,0x9ef60101,0xfb850108,0x9ef70101,0xfb850108,0x9ef80101,0xfb850108,0x9ef90101,
	0xfb850108,0x9efa0101,0xfb850108,0x9efb0101,0xfb850108,0x9efc0101,0xfb850108,0x9efd0101,
	0xfb850108,0x9efe0101,0xfb850108,0x9eff0101,0xfb850108,0x9f000101,0xfb850108,0x9f010101,
	0xfb850108,0x9f020101,0xfb850108,0x9f030101,0xfb850108,0x9f040101,0xfb850108,0x9f050101,
	0xfb850108,0x9f060101,0xfb850108,0x9f070101,0xfb850108,0x9f080101,0xfb850108,0x9f090101,
	0xfb850108,0x9f0a0101,0xfb850108,0x9f0b0101,0xfb850108,0x9f0c0101,0xfb850108,0x9f0d0101,
	0xfb850108,0x9f0e0101,0xfb850108,0x9f0f0101,0xfb850108,0x9f100101,0xfb850108,0x9f110101,
	0xfb850108,0x9f120101,0xfb850108,0x9f130101,0xfb850108,0x9f140101,0xfb850108,0x9f150101,
	0xfb850108,0x9f160101,0xfb850108,0x9f170101,0xfb850108,0x9f180101,0xfb850108,0x9f190101,
	0xfb850108,0x9f1a0101,0xfb850108,0x9f1b0101,0xfb850108,0x9f1c0101,0xfb850108,0x9f1d0101,
	0xfb850108,0x9f1e0101,0xfb850108,0x9f1f0101,0xfb850108,0x9f200101,0xfb850108,0x9f210101,
	0xfb850108,0x9f220101,0xfb850108,0x9f230101,0xfb850108,0x9f240101,0xfb850108,0x9f250101,
	0xfb850108,0x9f260101,0xfb850108,0x9f270101,0xfb850108,0x9f280101,0xfb850108,0x9f290101,
	0xfb850108,0x9f2a0101,0xfb850108,0x9f2b0101,0xfb850108,0x9f2c0101,0xfb850108,0x9f2d0101,
	0xfb850108,0x9f2e0101,0xfb850108,0x9f2f0101,0xfb850108,0x9f300101,0xfb850108,0x9f310101,
	0xfb850108,0x9f320101,0xfb850108,0x9f330101,0xfb850108,0x9f340101,0xfb850108,0x9f350101,
	0xfb850108,0x9f360101,0xfb850108,0x9f370101,0xfb850108,0x9f380101,0xfb850108,0x9f390101,
	0xfb850108,0x9f3a0101,0xfb850108,0x9f3b0101,0xfb850108,0x9f3c0101,0xfb850108,0x9f3d0101,
	0xfb850108,0x9f3e0101,0xfb850108,0x9f3f0101,0xfb850108,0x9f400101,0xfb850108,0x9f410101,
	0xfb850108,0x9f420101,0xfb850108,0x9f430101,0xfb850108,0x9f440101,0xfb850108,0x9f450101,
	0xfb850108,0x9f460101,0xfb850108,0x9f470101,0xfb850108,0x9f480101,0xfb850108,0x9f490101,
	0xfb850108,0x9f4a0101,0xfb850108,0x9f4b0101,0xfb850108,0x9f4c0101,0xfb850108,0x9f4d0101,
	0xfb850108,0x9f4e0101,0xfb850108,0x9f4f0101,0xfb850108,0x9f500101,0xfb850108,0x9f510101,
	0xfb850108,0x9f520101,0xfb850108,0x9f530101,0xfb850108,0x9f540101,0xfb850108,0x9f550101,
	0xfb850108,0x9f560101,0xfb850108,0x9f570101,0xfb850108,0x9f580101,0xfb850108,0x9f590101,
	0xfb850108,0x9f5a0101,0xfb850108,0x9f5b0101,0xfb850108,0x9f5c0101,0xfb850108,0x9f5d0101,
	0xfb850108,0x9f5e0101,0xfb850108,0x9f5f0101,0xfb850108,0x9f600101,0xfb850108,0x9f610101,
	0xfb850108,0x9f620101,0xfb850108,0x9f630101,0xfb850108,0x9f640101,0xfb850108,0x9f650101,
	0xfb850108,0x9f660101,0xfb850108,0x9f670101,0xfb850108,0x9f680101,0xfb850108,0x9f690101,
	0xfb850108,0x9f6a0101,0xfb850108,0x9f6b0101,0xfb850108,0x9f6c0101,0xfb850108,0x9f6d0101,
	0xfb850108,0x9f6e0101,0xfb850108,0x9f6f0101,0xfb850108,0x9f700101,0xfb850108,0x9f710101,
	0xfb850108,0x9f720101,0xfb850108,0x9f730101,0xfb850108,0x9f740101,0xfb850108,0x9f750101,
	0xfb850108,0x9f760101,0xfb850108,0x9f770101,0xfb850108,0x9f780101,0xfb850108,0x9f790101,
	0xfb850108,0x9f7a0101,0xfb850108,0x9f7b0101,0xfb850108,0x9f7c0101,0xfb850108,0x9f7d0101,
	0xfb850108,0x9f7e0101,0xfb850108,0x9f7f0101,0xfb850108,0x9f800101,0xfb850108,0x9f810101,
	0xfb850108,0x9f820101,0xfb850108,0x9f830101,0xfb850108,0x9f840101,0xfb850108,0x9f850101,
	0xfb850108,0x9f860101,0xfb850108,0x9f870101,0xfb850108,0x9f880101,0xfb850108,0x9f890101,
	0xfb850108,0x9f8a0101,0xfb850108,0x9f8b0101,0xfb850108,0x9f8c0101,0xfb850108,0x9f8d0101,
	0xfb850108,0x9f8e0101,0xfb850108,0x9f8f0101,0xfb850108,0x9f900101,0xfb850108,0x9f910101,
	0xfb850108,0x9f920101,0xfb850108,0x9f930101,0xfb850108,0x9f940101,0xfb850108,0x9f950101,
	0xfb850108,0x9f960101,0xfb850108,0x9f970101,0xfb850108,0x9f980101,0xfb850108,0x9f990101,
	0xfb850108,0x9f9a0101,0xfb850108,0x9f9b0101,0xfb850108,0x9f9c0101,0xfb850108,0x9f9d0101,
	0xfb850108,0x9f9e0101,0xfb850108,0x9f9f0101,0xfb850108,0x9fa00101,0xfb850108,0x9fa10101,
	0xfb850108,0x9fa20101,0xfb850108,0x9fa30101,0xfb850108,0x9fa40101,0xfb850108,0x9fa50101,
	0xfb850108,0x9fa60101,0xfb850108,0x9fa70101,0xfb850108,0x9fa80101,0xfb850108,0x9fa90101,
	0xfb850108,0x9faa0101,0xfb850108,0x9fab0101,0xfb850108,0x9fac0101,0xfb850108,0x9fad0101,
	0xfb850108,0x9fae0101,0xfb850108,0x9faf0101,0xfb850108,0x9fb00101,0xfb850108,0x9fb10101,
	0xfb850108,0x9fb20101,0xfb850108,0x9fb30101,0xfb850108,0x9fb40101,0xfb850108,0x9fb50101,
	0xfb850108,0x9fb60101,0xfb850108,0x9fb70101,0xfb850108,0x9fb80101,0xfb850108,0x9fb90101,
	0xfb850108,0x9fba0101,0xfb850108,0x9fbb0101,0xfb850108,0x9fbc0101,0xfb850108,0x9fbd0101,
	0xfb850108,0x9fbe0101,0xfb850108,0x9fbf0101,0xfb850108,0x9fc00101,0xfb850108,0x9fc10101,
	0xfb850108,0x9fc20101,0xfb850108,0x9fc30101,0xfb850108,0x9fc40101,0xfb850108,0x9fc50101,
	0xfb850108,0x9fc60101,0xfb850108,0x9fc70101,0xfb850108,0x9fc80101,0xfb850108,0x9fc90101,
	0xfb850108,0x9fca0101,0xfb850108,0x9fcb0101,0xfb850108,0x9fcc0101,0xfb850108,0x9fcd0101,
	0xfb850108,0x9fce0101,0xfb850108,0x9fcf0101,0xfb850108,0x9fd00101,0xfb850108,0x9fd10101,
	0xfb850108,0x9fd20101,0xfb850108,0x9fd30101,0xfb850108,0x9fd40101,0xfb850108,0x9fd50101,
	0xfb850108,0x9fd60101,0xfb850108,0x9fd70101,0xfb850108,0x9fd80101,0xfb850108,0x9fd90101,
	0xfb850108,0x9fda0101,0xfb850108,0x9fdb0101,0xfb850108,0x9fdc0101,0xfb850108,0x9fdd0101,
	0xfb850108,0x9fde0101,0xfb850108,0x9fdf0101,0xfb850108,0x9fe00101,0xfb850108,0x9fe10101,
	0xfb850108,0x9fe20101,0xfb850108,0x9fe30101,0xfb850108,0x9fe40101,0xfb850108,0x9fe50101,
	0xfb850108,0x9fe60101,0xfb850108,0x9fe70101,0xfb850108,0x9fe80101,0xfb850108,0x9fe90101,
	0xfb850108,0x9fea0101,0xfb850108,0x9feb0101,0xfb850108,0x9fec0101,0xfb850108,0x9fed0101,
	0xfb850108,0x9fee0101,0xfb850108,0x9fef0101,0xfb850108,0x9ff00101,0xfb850108,0x9ff10101,
	0xfb850108,0x9ff20101,0xfb850108,0x9ff30101,0xfb850108,0x9ff40101,0xfb850108,0x9ff50101,
	0xfb850108,0x9ff60101,0xfb850108,0x9ff70101,0xfb850108,0x9ff80101,0xfb850108,0x9ff90101,
	0xfb850108,0x9ffa0101,0xfb850108,0x9ffb0101,0xfb850108,0x9ffc0101,0xfb850108,0x9ffd0101,
	0xfb850108,0x9ffe0101,0xfb850108,0x9fff0101,0xfb850108,0xa0000101,0xfb850108,0xa0010101,
	0xfb850108,0xa0020101,0xfb850108,0xa0030101,0xfb850108,0xa0040101,0xfb850108,0xa0050101,
	0xfb850108,0xa0060101,0xfb850108,0xa0070101,0xfb850108,0xa0080101,0xfb850108,0xa0090101,
	0xfb850108,0xa00a0101,0xfb850108,0xa00b0101,0xfb850108,0xa00c0101,0xfb850108,0xa00d0101,
	0xfb850108,0xa00e0101,0xfb850108,0xa00f0101,0xfb850108,0xa0100101,0xfb850108,0xa0110101,
	0xfb850108,0xa0120101,0xfb850108,0xa0130101,0xfb850108,0xa0140101,0xfb850108,0xa0150101,
	0xfb850108,0xa0160101,0xfb850108,0xa0170101,0xfb850108,0xa0180101,0xfb850108,0xa0190101,
	0xfb850108,0xa01a0101,0xfb850108,0xa01b0101,0xfb850108,0xa01c0101,0xfb850108,0xa01d0101,
	0xfb850108,0xa01e0101,0xfb850108,0xa01f0101,0xfb850108,0xa0200101,0xfb850108,0xa0210101,
	0xfb850108,0xa0220101,0xfb850108,0xa0230101,0xfb850108,0xa0240101,0xfb850108,0xa0250101,
	0xfb850108,0xa0260101,0xfb850108,0xa0270101,0xfb850108,0xa0280101,0xfb850108,0xa0290101,
	0xfb850108,0xa02a0101,0xfb850108,0xa02b0101,0xfb850108,0xa02c0101,0xfb850108,0xa02d0101,
	0xfb850108,0xa02e0101,0xfb850108,0xa02f0101,0xfb850108,0xa0300101,0xfb850108,0xa0310101,
	0xfb850108,0xa0320101,0xfb850108,0xa0330101,0xfb850108,0xa0340101,0xfb850108,0xa0350101,
	0xfb850108,0xa0360101,0xfb850108,0xa0370101,0xfb850108,0xa0380101,0xfb850108,0xa0390101,
	0xfb850108,0xa03a0101,0xfb850108,0xa03b0101,0xfb850108,0xa03c0101,0xfb850108,0xa03d0101,
	0xfb850108,0xa03e0101,0xfb850108,0xa03f0101,0xfb850108,0xa0400101,0xfb850108,0xa0410101,
	0xfb850108,0xa0420101,0xfb850108,0xa0430101,0xfb850108,0xa0440101,0xfb850108,0xa0450101,
	0xfb850108,0xa0460101,0xfb850108,0xa0470101,0xfb850108,0xa0480101,0xfb850108,0xa0490101,
	0xfb850108,0xa04a0101,0xfb850108,0xa04b0101,0xfb850108,0xa04c0101,0xfb850108,0xa04d0101,
	0xfb850108,0xa04e0101,0xfb850108,0xa04f0101,0xfb850108,0xa0500101,0xfb850108,0xa0510101,
	0xfb850108,0xa0520101,0xfb850108,0xa0530101,0xfb850108,0xa0540101,0xfb850108,0xa0550101,
	0xfb850108,0xa0560101,0xfb850108,0xa0570101,0xfb850108,0xa0580101,0xfb850108,0xa0590101,
	0xfb850108,0xa05a0101,0xfb850108,0xa05b0101,0xfb850108,0xa05c0101,0xfb850108,0xa05d0101,
	0xfb850108,0xa05e0101,0xfb850108,0xa05f0101,0xfb850108,0xa0600101,0xfb850108,0xa0610101,
	0xfb850108,0xa0620101,0xfb850108,0xa0630101,0xfb850108,0xa0640101,0xfb850108,0xa0650101,
	0xfb850108,0xa0660101,0xfb850108,0xa0670101,0xfb850108,0xa0680101,0xfb850108,0xa0690101,
	0xfb850108,0xa06a0101,0xfb850108,0xa06b0101,0xfb850108,0xa06c0101,0xfb850108,0xa06d0101,
	0xfb850108,0xa06e0101,0xfb850108,0xa06f0101,0xfb850108,0xa0700101,0xfb850108,0xa0710101,
	0xfb850108,0xa0720101,0xfb850108,0xa0730101,0xfb850108,0xa0740101,0xfb850108,0xa0750101,
	0xfb850108,0xa0760101,0xfb850108,0xa0770101,0xfb850108,0xa0780101,0xfb850108,0xa0790101,
	0xfb850108,0xa07a0101,0xfb850108,0xa07b0101,0xfb850108,0xa07c0101,0xfb850108,0xa07d0101,
	0xfb850108,0xa07e0101,0xfb850108,0xa07f0101,0xfb850108,0xa0800101,0xfb850108,0xa0810101,
	0xfb850108,0xa0820101,0xfb850108,0xa0830101,0xfb850108,0xa0840101,0xfb850108,0xa0850101,
	0xfb850108,0xa0860101,0xfb850108,0xa0870101,0xfb850108,0xa0880101,0xfb850108,0xa0890101,
	0xfb850108,0xa08a0101,0xfb850108,0xa08b0101,0xfb850108,0xa08c0101,0xfb850108,0xa08d0101,
	0xfb850108,0xa08e0101,0xfb850108,0xa08f0101,0xfb850108,0xa0900101,0xfb850108,0xa0910101,
	0xfb850108,0xa0920101,0xfb850108,0xa0930101,0xfb850108,0xa0940101,0xfb850108,0xa0950101,
	0xfb850108,0xa0960101,0xfb850108,0xa0970101,0xfb850108,0xa0980101,0xfb850108,0xa0990101,
	0xfb850108,0xa09a0101,0xfb850108,0xa09b0101,0xfb850108,0xa09c0101,0xfb850108,0xa09d0101,
	0xfb850108,0xa09e0101,0xfb850108,0xa09f0101,0xfb850108,0xa0a00101,0xfb850108,0xa0a10101,
	0xfb850108,0xa0a20101,0xfb850108,0xa0a30101,0xfb850108,0xa0a40101,0xfb850108,0xa0a50101,
	0xfb850108,0xa0a60101,0xfb850108,0xa0a70101,0xfb850108,0xa0a80101,0xfb850108,0xa0a90101,
	0xfb850108,0xa0aa0101,0xfb850108,0xa0ab0101,0xfb850108,0xa0ac0101,0xfb850108,0xa0ad0101,
	0xfb850108,0xa0ae0101,0xfb850108,0xa0af0101,0xfb850108,0xa0b00101,0xfb850108,0xa0b10101,
	0xfb850108,0xa0b20101,0xfb850108,0xa0b30101,0xfb850108,0xa0b40101,0xfb850108,0xa0b50101,
	0xfb850108,0xa0b60101,0xfb850108,0xa0b70101,0xfb850108,0xa0b80101,0xfb850108,0xa0b90101,
	0xfb850108,0xa0ba0101,0xfb850108,0xa0bb0101,0xfb850108,0xa0bc0101,0xfb850108,0xa0bd0101,
	0xfb850108,0xa0be0101,0xfb850108,0xa0bf0101,0xfb850108,0xa0c00101,0xfb850108,0xa0c10101,
	0xfb850108,0xa0c20101,0xfb850108,0xa0c30101,0xfb850108,0xa0c40101,0xfb850108,0xa0c50101,
	0xfb850108,0xa0c60101,0xfb850108,0xa0c70101,0xfb850108,0xa0c80101,0xfb850108,0xa0c90101,
	0xfb850108,0xa0ca0101,0xfb850108,0xa0cb0101,0xfb850108,0xa0cc0101,0xfb850108,0xa0cd0101,
	0xfb850108,0xa0ce0101,0xfb850108,0xa0cf0101,0xfb850108,0xa0d00101,0xfb850108,0xa0d10101,
	0xfb850108,0xa0d20101,0xfb850108,0xa0d30101,0xfb850108,0xa0d40101,0xfb850108,0xa0d50101,
	0xfb850108,0xa0d60101,0xfb850108,0xa0d70101,0xfb850108,0xa0d80101,0xfb850108,0xa0d90101,
	0xfb850108,0xa0da0101,0xfb850108,0xa0db0101,0xfb850108,0xa0dc0101,0xfb850108,0xa0dd0101,
	0xfb850108,0xa0de0101,0xfb850108,0xa0df0101,0xfb850108,0xa0e00101,0xfb850108,0xa0e10101,
	0xfb850108,0xa0e20101,0xfb850108,0xa0e30101,0xfb850108,0xa0e40101,0xfb850108,0xa0e50101,
	0xfb850108,0xa0e60101,0xfb850108,0xa0e70101,0xfb850108,0xa0e80101,0xfb850108,0xa0e90101,
	0xfb850108,0xa0ea0101,0xfb850108,0xa0eb0101,0xfb850108,0xa0ec0101,0xfb850108,0xa0ed0101,
	0xfb850108,0xa0ee0101,0xfb850108,0xa0ef0101,0xfb850108,0xa0f00101,0xfb850108,0xa0f10101,
	0xfb850108,0xa0f20101,0xfb850108,0xa0f30101,0xfb850108,0xa0f40101,0xfb850108,0xa0f50101,
	0xfb850108,0xa0f60101,0xfb850108,0xa0f70101,0xfb850108,0xa0f80101,0xfb850108,0xa0f90101,
	0xfb850108,0xa0fa0101,0xfb850108,0xa0fb0101,0xfb850108,0xa0fc0101,0xfb850108,0xa0fd0101,
	0xfb850108,0xa0fe0101,0xfb850108,0xa0ff0101,0xfb850108,0xa1000101,0xfb850108,0xa1010101,
	0xfb850108,0xa1020101,0xfb850108,0xa1030101,0xfb850108,0xa1040101,0xfb850108,0xa1050101,
	0xfb850108,0xa1060101,0xfb850108,0xa1070101,0xfb850108,0xa1080101,0xfb850108,0xa1090101,
	0xfb850108,0xa10a0101,0xfb850108,0xa10b0101,0xfb850108,0xa10c0101,0xfb850108,0xa10d0101,
	0xfb850108,0xa10e0101,0xfb850108,0xa10f0101,0xfb850108,0xa1100101,0xfb850108,0xa1110101,
	0xfb850108,0xa1120101,0xfb850108,0xa1130101,0xfb850108,0xa1140101,0xfb850108,0xa1150101,
	0xfb850108,0xa1160101,0xfb850108,0xa1170101,0xfb850108,0xa1180101,0xfb850108,0xa1190101,
	0xfb850108,0xa11a0101,0xfb850108,0xa11b0101,0xfb850108,0xa11c0101,0xfb850108,0xa11d0101,
	0xfb850108,0xa11e0101,0xfb850108,0xa11f0101,0xfb850108,0xa1200101,0xfb850108,0xa1210101,
	0xfb850108,0xa1220101,0xfb850108,0xa1230101,0xfb850108,0xa1240101,0xfb850108,0xa1250101,
	0xfb850108,0xa1260101,0xfb850108,0xa1270101,0xfb850108,0xa1280101,0xfb850108,0xa1290101,
	0xfb850108,0xa12a0101,0xfb850108,0xa12b0101,0xfb850108,0xa12c0101,0xfb850108,0xa12d0101,
	0xfb850108,0xa12e0101,0xfb850108,0xa12f0101,0xfb850108,0xa1300101,0xfb850108,0xa1310101,
	0xfb850108,0xa1320101,0xfb850108,0xa1330101,0xfb850108,0xa1340101,0xfb850108,0xa1350101,
	0xfb850108,0xa1360101,0xfb850108,0xa1370101,0xfb850108,0xa1380101,0xfb850108,0xa1390101,
	0xfb850108,0xa13a0101,0xfb850108,0xa13b0101,0xfb850108,0xa13c0101,0xfb850108,0xa13d0101,
	0xfb850108,0xa13e0101,0xfb850108,0xa13f0101,0xfb850108,0xa1400101,0xfb850108,0xa1410101,
	0xfb850108,0xa1420101,0xfb850108,0xa1430101,0xfb850108,0xa1440101,0xfb850108,0xa1450101,
	0xfb850108,0xa1460101,0xfb850108,0xa1470101,0xfb850108,0xa1480101,0xfb850108,0xa1490101,
	0xfb850108,0xa14a0101,0xfb850108,0xa14b0101,0xfb850108,0xa14c0101,0xfb850108,0xa14d0101,
	0xfb850108,0xa14e0101,0xfb850108,0xa14f0101,0xfb850108,0xa1500101,0xfb850108,0xa1510101,
	0xfb850108,0xa1520101,0xfb850108,0xa1530101,0xfb850108,0xa1540101,0xfb850108,0xa1550101,
	0xfb850108,0xa1560101,0xfb850108,0xa1570101,0xfb850108,0xa1580101,0xfb850108,0xa1590101,
	0xfb850108,0xa15a0101,0xfb850108,0xa15b0101,0xfb850108,0xa15c0101,0xfb850108,0xa15d0101,
	0xfb850108,0xa15e0101,0xfb850108,0xa15f0101,0xfb850108,0xa1600101,0xfb850108,0xa1610101,
	0xfb850108,0xa1620101,0xfb850108,0xa1630101,0xfb850108,0xa1640101,0xfb850108,0xa1650101,
	0xfb850108,0xa1660101,0xfb850108,0xa1670101,0xfb850108,0xa1680101,0xfb850108,0xa1690101,
	0xfb850108,0xa16a0101,0xfb850108,0xa16b0101,0xfb850108,0xa16c0101,0xfb850108,0xa16d0101,
	0xfb850108,0xa16e0101,0xfb850108,0xa16f0101,0xfb850108,0xa1700101,0xfb850108,0xa1710101,
	0xfb850108,0xa1720101,0xfb850108,0xa1730101,0xfb850108,0xa1740101,0xfb850108,0xa1750101,
	0xfb850108,0xa1760101,0xfb850108,0xa1770101,0xfb850108,0xa1780101,0xfb850108,0xa1790101,
	0xfb850108,0xa17a0101,0xfb850108,0xa17b0101,0xfb850108,0xa17c0101,0xfb850108,0xa17d0101,
	0xfb850108,0xa17e0101,0xfb850108,0xa17f0101,0xfb850108,0xa1800101,0xfb850108,0xa1810101,
	0xfb850108,0xa1820101,0xfb850108,0xa1830101,0xfb850108,0xa1840101,0xfb850108,0xa1850101,
	0xfb850108,0xa1860101,0xfb850108,0xa1870101,0xfb850108,0xa1880101,0xfb850108,0xa1890101,
	0xfb850108,0xa18a0101,0xfb850108,0xa18b0101,0xfb850108,0xa18c0101,0xfb850108,0xa18d0101,
	0xfb850108,0xa18e0101,0xfb850108,0xa18f0101,0xfb850108,0xa1900101,0xfb850108,0xa1910101,
	0xfb850108,0xa1920101,0xfb850108,0xa1930101,0xfb850108,0xa1940101,0xfb850108,0xa1950101,
	0xfb850108,0xa1960101,0xfb850108,0xa1970101,0xfb850108,0xa1980101,0xfb850108,0xa1990101,
	0xfb850108,0xa19a0101,0xfb850108,0xa19b0101,0xfb850108,0xa19c0101,0xfb850108,0xa19d0101,
	0xfb850108,0xa19e0101,0xfb850108,0xa19f0101,0xfb850108,0xa1a00101,0xfb850108,0xa1a10101,
	0xfb850108,0xa1a20101,0xfb850108,0xa1a30101,0xfb850108,0xa1a40101,0xfb850108,0xa1a50101,
	0xfb850108,0xa1a60101,0xfb850108,0xa1a70101,0xfb850108,0xa1a80101,0xfb850108,0xa1a90101,
	0xfb850108,0xa1aa0101,0xfb850108,0xa1ab0101,0xfb850108,0xa1ac0101,0xfb850108,0xa1ad0101,
	0xfb850108,0xa1ae0101,0xfb850108,0xa1af0101,0xfb850108,0xa1b00101,0xfb850108,0xa1b10101,
	0xfb850108,0xa1b20101,0xfb850108,0xa1b30101,0xfb850108,0xa1b40101,0xfb850108,0xa1b50101,
	0xfb850108,0xa1b60101,0xfb850108,0xa1b70101,0xfb850108,0xa1b80101,0xfb850108,0xa1b90101,
	0xfb850108,0xa1ba0101,0xfb850108,0xa1bb0101,0xfb850108,0xa1bc0101,0xfb850108,0xa1bd0101,
	0xfb850108,0xa1be0101,0xfb850108,0xa1bf0101,0xfb850108,0xa1c00101,0xfb850108,0xa1c10101,
	0xfb850108,0xa1c20101,0xfb850108,0xa1c30101,0xfb850108,0xa1c40101,0xfb850108,0xa1c50101,
	0xfb850108,0xa1c60101,0xfb850108,0xa1c70101,0xfb850108,0xa1c80101,0xfb850108,0xa1c90101,
	0xfb850108,0xa1ca0101,0xfb850108,0xa1cb0101,0xfb850108,0xa1cc0101,0xfb850108,0xa1cd0101,
	0xfb850108,0xa1ce0101,0xfb850108,0xa1cf0101,0xfb850108,0xa1d00101,0xfb850108,0xa1d10101,
	0xfb850108,0xa1d20101,0xfb850108,0xa1d30101,0xfb850108,0xa1d40101,0xfb850108,0xa1d50101,
	0xfb850108,0xa1d60101,0xfb850108,0xa1d70101,0xfb850108,0xa1d80101,0xfb850108,0xa1d90101,
	0xfb850108,0xa1da0101,0xfb850108,0xa1db0101,0xfb850108,0xa1dc0101,0xfb850108,0xa1dd0101,
	0xfb850108,0xa1de0101,0xfb850108,0xa1df0101,0xfb850108,0xa1e00101,0xfb850108,0xa1e10101,
	0xfb850108,0xa1e20101,0xfb850108,0xa1e30101,0xfb850108,0xa1e40101,0xfb850108,0xa1e50101,
	0xfb850108,0xa1e60101,0xfb850108,0xa1e70101,0xfb850108,0xa1e80101,0xfb850108,0xa1e90101,
	0xfb850108,0xa1ea0101,0xfb850108,0xa1eb0101,0xfb850108,0xa1ec0101,0xfb850108,0xa1ed0101,
	0xfb850108,0xa1ee0101,0xfb850108,0xa1ef0101,0xfb850108,0xa1f00101,0xfb850108,0xa1f10101,
	0xfb850108,0xa1f20101,0xfb850108,0xa1f30101,0xfb850108,0xa1f40101,0xfb850108,0xa1f50101,
	0xfb850108,0xa1f60101,0xfb850108,0xa1f70101,0xfb850108,0xa1f80101,0xfb850108,0xa1f90101,
	0xfb850108,0xa1fa0101,0xfb850108,0xa1fb0101,0xfb850108,0xa1fc0101,0xfb850108,0xa1fd0101,
	0xfb850108,0xa1fe0101,0xfb850108,0xa1ff0101,0xfb850108,0xa2000101,0xfb850108,0xa2010101,
	0xfb850108,0xa2020101,0xfb850108,0xa2030101,0xfb850108,0xa2040101,0xfb850108,0xa2050101,
	0xfb850108,0xa2060101,0xfb850108,0xa2070101,0xfb850108,0xa2080101,0xfb850108,0xa2090101,
	0xfb850108,0xa20a0101,0xfb850108,0xa20b0101,0xfb850108,0xa20c0101,0xfb850108,0xa20d0101,
	0xfb850108,0xa20e0101,0xfb850108,0xa20f0101,0xfb850108,0xa2100101,0xfb850108,0xa2110101,
	0xfb850108,0xa2120101,0xfb850108,0xa2130101,0xfb850108,0xa2140101,0xfb850108,0xa2150101,
	0xfb850108,0xa2160101,0xfb850108,0xa2170101,0xfb850108,0xa2180101,0xfb850108,0xa2190101,
	0xfb850108,0xa21a0101,0xfb850108,0xa21b0101,0xfb850108,0xa21c0101,0xfb850108,0xa21d0101,
	0xfb850108,0xa21e0101,0xfb850108,0xa21f0101,0xfb850108,0xa2200101,0xfb850108,0xa2210101,
	0xfb850108,0xa2220101,0xfb850108,0xa2230101,0xfb850108,0xa2240101,0xfb850108,0xa2250101,
	0xfb850108,0xa2260101,0xfb850108,0xa2270101,0xfb850108,0xa2280101,0xfb850108,0xa2290101,
	0xfb850108,0xa22a0101,0xfb850108,0xa22b0101,0xfb850108,0xa22c0101,0xfb850108,0xa22d0101,
	0xfb850108,0xa22e0101,0xfb850108,0xa22f0101,0xfb850108,0xa2300101,0xfb850108,0xa2310101,
	0xfb850108,0xa2320101,0xfb850108,0xa2330101,0xfb850108,0xa2340101,0xfb850108,0xa2350101,
	0xfb850108,0xa2360101,0xfb850108,0xa2370101,0xfb850108,0xa2380101,0xfb850108,0xa2390101,
	0xfb850108,0xa23a0101,0xfb850108,0xa23b0101,0xfb850108,0xa23c0101,0xfb850108,0xa23d0101,
	0xfb850108,0xa23e0101,0xfb850108,0xa23f0101,0xfb850108,0xa2400101,0xfb850108,0xa2410101,
	0xfb850108,0xa2420101,0xfb850108,0xa2430101,0xfb850108,0xa2440101,0xfb850108,0xa2450101,
	0xfb850108,0xa2460101,0xfb850108,0xa2470101,0xfb850108,0xa2480101,0xfb850108,0xa2490101,
	0xfb850108,0xa24a0101,0xfb850108,0xa24b0101,0xfb850108,0xa24c0101,0xfb850108,0xa24d0101,
	0xfb850108,0xa24e0101,0xfb850108,0xa24f0101,0xfb850108,0xa2500101,0xfb850108,0xa2510101,
	0xfb850108,0xa2520101,0xfb850108,0xa2530101,0xfb850108,0xa2540101,0xfb850108,0xa2550101,
	0xfb850108,0xa2560101,0xfb850108,0xa2570101,0xfb850108,0xa2580101,0xfb850108,0xa2590101,
	0xfb850108,0xa25a0101,0xfb850108,0xa25b0101,0xfb850108,0xa25c0101,0xfb850108,0xa25d0101,
	0xfb850108,0xa25e0101,0xfb850108,0xa25f0101,0xfb850108,0xa2600101,0xfb850108,0xa2610101,
	0xfb850108,0xa2620101,0xfb850108,0xa2630101,0xfb850108,0xa2640101,0xfb850108,0xa2650101,
	0xfb850108,0xa2660101,0xfb850108,0xa2670101,0xfb850108,0xa2680101,0xfb850108,0xa2690101,
	0xfb850108,0xa26a0101,0xfb850108,0xa26b0101,0xfb850108,0xa26c0101,0xfb850108,0xa26d0101,
	0xfb850108,0xa26e0101,0xfb850108,0xa26f0101,0xfb850108,0xa2700101,0xfb850108,0xa2710101,
	0xfb850108,0xa2720101,0xfb850108,0xa2730101,0xfb850108,0xa2740101,0xfb850108,0xa2750101,
	0xfb850108,0xa2760101,0xfb850108,0xa2770101,0xfb850108,0xa2780101,0xfb850108,0xa2790101,
	0xfb850108,0xa27a0101,0xfb850108,0xa27b0101,0xfb850108,0xa27c0101,0xfb850108,0xa27d0101,
	0xfb850108,0xa27e0101,0xfb850108,0xa27f0101,0xfb850108,0xa2800101,0xfb850108,0xa2810101,
	0xfb850108,0xa2820101,0xfb850108,0xa2830101,0xfb850108,0xa2840101,0xfb850108,0xa2850101,
	0xfb850108,0xa2860101,0xfb850108,0xa2870101,0xfb850108,0xa2880101,0xfb850108,0xa2890101,
	0xfb850108,0xa28a0101,0xfb850108,0xa28b0101,0xfb850108,0xa28c0101,0xfb850108,0xa28d0101,
	0xfb850108,0xa28e0101,0xfb850108,0xa28f0101,0xfb850108,0xa2900101,0xfb850108,0xa2910101,
	0xfb850108,0xa2920101,0xfb850108,0xa2930101,0xfb850108,0xa2940101,0xfb850108,0xa2950101,
	0xfb850108,0xa2960101,0xfb850108,0xa2970101,0xfb850108,0xa2980101,0xfb850108,0xa2990101,
	0xfb850108,0xa29a0101,0xfb850108,0xa29b0101,0xfb850108,0xa29c0101,0xfb850108,0xa29d0101,
	0xfb850108,0xa29e0101,0xfb850108,0xa29f0101,0xfb850108,0xa2a00101,0xfb850108,0xa2a10101,
	0xfb850108,0xa2a20101,0xfb850108,0xa2a30101,0xfb850108,0xa2a40101,0xfb850108,0xa2a50101,
	0xfb850108,0xa2a60101,0xfb850108,0xa2a70101,0xfb850108,0xa2a80101,0xfb850108,0xa2a90101,
	0xfb850108,0xa2aa0101,0xfb850108,0xa2ab0101,0xfb850108,0xa2ac0101,0xfb850108,0xa2ad0101,
	0xfb850108,0xa2ae0101,0xfb850108,0xa2af0101,0xfb850108,0xa2b00101,0xfb850108,0xa2b10101,
	0xfb850108,0xa2b20101,0xfb850108,0xa2b30101,0xfb850108,0xa2b40101,0xfb850108,0xa2b50101,
	0xfb850108,0xa2b60101,0xfb850108,0xa2b70101,0xfb850108,0xa2b80101,0xfb850108,0xa2b90101,
	0xfb850108,0xa2ba0101,0xfb850108,0xa2bb0101,0xfb850108,0xa2bc0101,0xfb850108,0xa2bd0101,
	0xfb850108,0xa2be0101,0xfb850108,0xa2bf0101,0xfb850108,0xa2c00101,0xfb850108,0xa2c10101,
	0xfb850108,0xa2c20101,0xfb850108,0xa2c30101,0xfb850108,0xa2c40101,0xfb850108,0xa2c50101,
	0xfb850108,0xa2c60101,0xfb850108,0xa2c70101,0xfb850108,0xa2c80101,0xfb850108,0xa2c90101,
	0xfb850108,0xa2ca0101,0xfb850108,0xa2cb0101,0xfb850108,0xa2cc0101,0xfb850108,0xa2cd0101,
	0xfb850108,0xa2ce0101,0xfb850108,0xa2cf0101,0xfb850108,0xa2d00101,0xfb850108,0xa2d10101,
	0xfb850108,0xa2d20101,0xfb850108,0xa2d30101,0xfb850108,0xa2d40101,0xfb850108,0xa2d50101,
	0xfb850108,0xa2d60101,0xfb850108,0xa2d70101,0xfb850108,0xa2d80101,0xfb850108,0xa2d90101,
	0xfb850108,0xa2da0101,0xfb850108,0xa2db0101,0xfb850108,0xa2dc0101,0xfb850108,0xa2dd0101,
	0xfb850108,0xa2de0101,0xfb850108,0xa2df0101,0xfb850108,0xa2e00101,0xfb850108,0xa2e10101,
	0xfb850108,0xa2e20101,0xfb850108,0xa2e30101,0xfb850108,0xa2e40101,0xfb850108,0xa2e50101,
	0xfb850108,0xa2e60101,0xfb850108,0xa2e70101,0xfb850108,0xa2e80101,0xfb850108,0xa2e90101,
	0xfb850108,0xa2ea0101,0xfb850108,0xa2eb0101,0xfb850108,0xa2ec0101,0xfb850108,0xa2ed0101,
	0xfb850108,0xa2ee0101,0xfb850108,0xa2ef0101,0xfb850108,0xa2f00101,0xfb850108,0xa2f10101,
	0xfb850108,0xa2f20101,0xfb850108,0xa2f30101,0xfb850108,0xa2f40101,0xfb850108,0xa2f50101,
	0xfb850108,0xa2f60101,0xfb850108,0xa2f70101,0xfb850108,0xa2f80101,0xfb850108,0xa2f90101,
	0xfb850108,0xa2fa0101,0xfb850108,0xa2fb0101,0xfb850108,0xa2fc0101,0xfb850108,0xa2fd0101,
	0xfb850108,0xa2fe0101,0xfb850108,0xa2ff0101,0xfb850108,0xa3000101,0xfb850108,0xa3010101,
	0xfb850108,0xa3020101,0xfb850108,0xa3030101,0xfb850108,0xa3040101,0xfb850108,0xa3050101,
	0xfb850108,0xa3060101,0xfb850108,0xa3070101,0xfb850108,0xa3080101,0xfb850108,0xa3090101,
	0xfb850108,0xa30a0101,0xfb850108,0xa30b0101,0xfb850108,0xa30c0101,0xfb850108,0xa30d0101,
	0xfb850108,0xa30e0101,0xfb850108,0xa30f0101,0xfb850108,0xa3100101,0xfb850108,0xa3110101,
	0xfb850108,0xa3120101,0xfb850108,0xa3130101,0xfb850108,0xa3140101,0xfb850108,0xa3150101,
	0xfb850108,0xa3160101,0xfb850108,0xa3170101,0xfb850108,0xa3180101,0xfb850108,0xa3190101,
	0xfb850108,0xa31a0101,0xfb850108,0xa31b0101,0xfb850108,0xa31c0101,0xfb850108,0xa31d0101,
	0xfb850108,0xa31e0101,0xfb850108,0xa31f0101,0xfb850108,0xa3200101,0xfb850108,0xa3210101,
	0xfb850108,0xa3220101,0xfb850108,0xa3230101,0xfb850108,0xa3240101,0xfb850108,0xa3250101,
	0xfb850108,0xa3260101,0xfb850108,0xa3270101,0xfb850108,0xa3280101,0xfb850108,0xa3290101,
	0xfb850108,0xa32a0101,0xfb850108,0xa32b0101,0xfb850108,0xa32c0101,0xfb850108,0xa32d0101,
	0xfb850108,0xa32e0101,0xfb850108,0xa32f0101,0xfb850108,0xa3300101,0xfb850108,0xa3310101,
	0xfb850108,0xa3320101,0xfb850108,0xa3330101,0xfb850108,0xa3340101,0xfb850108,0xa3350101,
	0xfb850108,0xa3360101,0xfb850108,0xa3370101,0xfb850108,0xa3380101,0xfb850108,0xa3390101,
	0xfb850108,0xa33a0101,0xfb850108,0xa33b0101,0xfb850108,0xa33c0101,0xfb850108,0xa33d0101,
	0xfb850108,0xa33e0101,0xfb850108,0xa33f0101,0xfb850108,0xa3400101,0xfb850108,0xa3410101,
	0xfb850108,0xa3420101,0xfb850108,0xa3430101,0xfb850108,0xa3440101,0xfb850108,0xa3450101,
	0xfb850108,0xa3460101,0xfb850108,0xa3470101,0xfb850108,0xa3480101,0xfb850108,0xa3490101,
	0xfb850108,0xa34a0101,0xfb850108,0xa34b0101,0xfb850108,0xa34c0101,0xfb850108,0xa34d0101,
	0xfb850108,0xa34e0101,0xfb850108,0xa34f0101,0xfb850108,0xa3500101,0xfb850108,0xa3510101,
	0xfb850108,0xa3520101,0xfb850108,0xa3530101,0xfb850108,0xa3540101,0xfb850108,0xa3550101,
	0xfb850108,0xa3560101,0xfb850108,0xa3570101,0xfb850108,0xa3580101,0xfb850108,0xa3590101,
	0xfb850108,0xa35a0101,0xfb850108,0xa35b0101,0xfb850108,0xa35c0101,0xfb850108,0xa35d0101,
	0xfb850108,0xa35e0101,0xfb850108,0xa35f0101,0xfb850108,0xa3600101,0xfb850108,0xa3610101,
	0xfb850108,0xa3620101,0xfb850108,0xa3630101,0xfb850108,0xa3640101,0xfb850108,0xa3650101,
	0xfb850108,0xa3660101,0xfb850108,0xa3670101,0xfb850108,0xa3680101,0xfb850108,0xa3690101,
	0xfb850108,0xa36a0101,0xfb850108,0xa36b0101,0xfb850108,0xa36c0101,0xfb850108,0xa36d0101,
	0xfb850108,0xa36e0101,0xfb850108,0xa36f0101,0xfb850108,0xa3700101,0xfb850108,0xa3710101,
	0xfb850108,0xa3720101,0xfb850108,0xa3730101,0xfb850108,0xa3740101,0xfb850108,0xa3750101,
	0xfb850108,0xa3760101,0xfb850108,0xa3770101,0xfb850108,0xa3780101,0xfb850108,0xa3790101,
	0xfb850108,0xa37a0101,0xfb850108,0xa37b0101,0xfb850108,0xa37c0101,0xfb850108,0xa37d0101,
	0xfb850108,0xa37e0101,0xfb850108,0xa37f0101,0xfb850108,0xa3800101,0xfb850108,0xa3810101,
	0xfb850108,0xa3820101,0xfb850108,0xa3830101,0xfb850108,0xa3840101,0xfb850108,0xa3850101,
	0xfb850108,0xa3860101,0xfb850108,0xa3870101,0xfb850108,0xa3880101,0xfb850108,0xa3890101,
	0xfb850108,0xa38a0101,0xfb850108,0xa38b0101,0xfb850108,0xa38c0101,0xfb850108,0xa38d0101,
	0xfb850108,0xa38e0101,0xfb850108,0xa38f0101,0xfb850108,0xa3900101,0xfb850108,0xa3910101,
	0xfb850108,0xa3920101,0xfb850108,0xa3930101,0xfb850108,0xa3940101,0xfb850108,0xa3950101,
	0xfb850108,0xa3960101,0xfb850108,0xa3970101,0xfb850108,0xa3980101,0xfb850108,0xa3990101,
	0xfb850108,0xa39a0101,0xfb850108,0xa39b0101,0xfb850108,0xa39c0101,0xfb850108,0xa39d0101,
	0xfb850108,0xa39e0101,0xfb850108,0xa39f0101,0xfb850108,0xa3a00101,0xfb850108,0xa3a10101,
	0xfb850108,0xa3a20101,0xfb850108,0xa3a30101,0xfb850108,0xa3a40101,0xfb850108,0xa3a50101,
	0xfb850108,0xa3a60101,0xfb850108,0xa3a70101,0xfb850108,0xa3a80101,0xfb850108,0xa3a90101,
	0xfb850108,0xa3aa0101,0xfb850108,0xa3ab0101,0xfb850108,0xa3ac0101,0xfb850108,0xa3ad0101,
	0xfb850108,0xa3ae0101,0xfb850108,0xa3af0101,0xfb850108,0xa3b00101,0xfb850108,0xa3b10101,
	0xfb850108,0xa3b20101,0xfb850108,0xa3b30101,0xfb850108,0xa3b40101,0xfb850108,0xa3b50101,
	0xfb850108,0xa3b60101,0xfb850108,0xa3b70101,0xfb850108,0xa3b80101,0xfb850108,0xa3b90101,
	0xfb850108,0xa3ba0101,0xfb850108,0xa3bb0101,0xfb850108,0xa3bc0101,0xfb850108,0xa3bd0101,
	0xfb850108,0xa3be0101,0xfb850108,0xa3bf0101,0xfb850108,0xa3c00101,0xfb850108,0xa3c10101,
	0xfb850108,0xa3c20101,0xfb850108,0xa3c30101,0xfb850108,0xa3c40101,0xfb850108,0xa3c50101,
	0xfb850108,0xa3c60101,0xfb850108,0xa3c70101,0xfb850108,0xa3c80101,0xfb850108,0xa3c90101,
	0xfb850108,0xa3ca0101,0xfb850108,0xa3cb0101,0xfb850108,0xa3cc0101,0xfb850108,0xa3cd0101,
	0xfb850108,0xa3ce0101,0xfb850108,0xa3cf0101,0xfb850108,0xa3d00101,0xfb850108,0xa3d10101,
	0xfb850108,0xa3d20101,0xfb850108,0xa3d30101,0xfb850108,0xa3d40101,0xfb850108,0xa3d50101,
	0xfb850108,0xa3d60101,0xfb850108,0xa3d70101,0xfb850108,0xa3d80101,0xfb850108,0xa3d90101,
	0xfb850108,0xa3da0101,0xfb850108,0xa3db0101,0xfb850108,0xa3dc0101,0xfb850108,0xa3dd0101,
	0xfb850108,0xa3de0101,0xfb850108,0xa3df0101,0xfb850108,0xa3e00101,0xfb850108,0xa3e10101,
	0xfb850108,0xa3e20101,0xfb850108,0xa3e30101,0xfb850108,0xa3e40101,0xfb850108,0xa3e50101,
	0xfb850108,0xa3e60101,0xfb850108,0xa3e70101,0xfb850108,0xa3e80101,0xfb850108,0xa3e90101,
	0xfb850108,0xa3ea0101,0xfb850108,0xa3eb0101,0xfb850108,0xa3ec0101,0xfb850108,0xa3ed0101,
	0xfb850108,0xa3ee0101,0xfb850108,0xa3ef0101,0xfb850108,0xa3f00101,0xfb850108,0xa3f10101,
	0xfb850108,0xa3f20101,0xfb850108,0xa3f30101,0xfb850108,0xa3f40101,0xfb850108,0xa3f50101,
	0xfb850108,0xa3f60101,0xfb850108,0xa3f70101,0xfb850108,0xa3f80101,0xfb850108,0xa3f90101,
	0xfb850108,0xa3fa0101,0xfb850108,0xa3fb0101,0xfb850108,0xa3fc0101,0xfb850108,0xa3fd0101,
	0xfb850108,0xa3fe0101,0xfb850108,0xa3ff0101,0xfb850108,0xa4000101,0xfb850108,0xa4010101,
	0xfb850108,0xa4020101,0xfb850108,0xa4030101,0xfb850108,0xa4040101,0xfb850108,0xa4050101,
	0xfb850108,0xa4060101,0xfb850108,0xa4070101,0xfb850108,0xa4080101,0xfb850108,0xa4090101,
	0xfb850108,0xa40a0101,0xfb850108,0xa40b0101,0xfb850108,0xa40c0101,0xfb850108,0xa40d0101,
	0xfb850108,0xa40e0101,0xfb850108,0xa40f0101,0xfb850108,0xa4100101,0xfb850108,0xa4110101,
	0xfb850108,0xa4120101,0xfb850108,0xa4130101,0xfb850108,0xa4140101,0xfb850108,0xa4150101,
	0xfb850108,0xa4160101,0xfb850108,0xa4170101,0xfb850108,0xa4180101,0xfb850108,0xa4190101,
	0xfb850108,0xa41a0101,0xfb850108,0xa41b0101,0xfb850108,0xa41c0101,0xfb850108,0xa41d0101,
	0xfb850108,0xa41e0101,0xfb850108,0xa41f0101,0xfb850108,0xa4200101,0xfb850108,0xa4210101,
	0xfb850108,0xa4220101,0xfb850108,0xa4230101,0xfb850108,0xa4240101,0xfb850108,0xa4250101,
	0xfb850108,0xa4260101,0xfb850108,0xa4270101,0xfb850108,0xa4280101,0xfb850108,0xa4290101,
	0xfb850108,0xa42a0101,0xfb850108,0xa42b0101,0xfb850108,0xa42c0101,0xfb850108,0xa42d0101,
	0xfb850108,0xa42e0101,0xfb850108,0xa42f0101,0xfb850108,0xa4300101,0xfb850108,0xa4310101,
	0xfb850108,0xa4320101,0xfb850108,0xa4330101,0xfb850108,0xa4340101,0xfb850108,0xa4350101,
	0xfb850108,0xa4360101,0xfb850108,0xa4370101,0xfb850108,0xa4380101,0xfb850108,0xa4390101,
	0xfb850108,0xa43a0101,0xfb850108,0xa43b0101,0xfb850108,0xa43c0101,0xfb850108,0xa43d0101,
	0xfb850108,0xa43e0101,0xfb850108,0xa43f0101,0xfb850108,0xa4400101,0xfb850108,0xa4410101,
	0xfb850108,0xa4420101,0xfb850108,0xa4430101,0xfb850108,0xa4440101,0xfb850108,0xa4450101,
	0xfb850108,0xa4460101,0xfb850108,0xa4470101,0xfb850108,0xa4480101,0xfb850108,0xa4490101,
	0xfb850108,0xa44a0101,0xfb850108,0xa44b0101,0xfb850108,0xa44c0101,0xfb850108,0xa44d0101,
	0xfb850108,0xa44e0101,0xfb850108,0xa44f0101,0xfb850108,0xa4500101,0xfb850108,0xa4510101,
	0xfb850108,0xa4520101,0xfb850108,0xa4530101,0xfb850108,0xa4540101,0xfb850108,0xa4550101,
	0xfb850108,0xa4560101,0xfb850108,0xa4570101,0xfb850108,0xa4580101,0xfb850108,0xa4590101,
	0xfb850108,0xa45a0101,0xfb850108,0xa45b0101,0xfb850108,0xa45c0101,0xfb850108,0xa45d0101,
	0xfb850108,0xa45e0101,0xfb850108,0xa45f0101,0xfb850108,0xa4600101,0xfb850108,0xa4610101,
	0xfb850108,0xa4620101,0xfb850108,0xa4630101,0xfb850108,0xa4640101,0xfb850108,0xa4650101,
	0xfb850108,0xa4660101,0xfb850108,0xa4670101,0xfb850108,0xa4680101,0xfb850108,0xa4690101,
	0xfb850108,0xa46a0101,0xfb850108,0xa46b0101,0xfb850108,0xa46c0101,0xfb850108,0xa46d0101,
	0xfb850108,0xa46e0101,0xfb850108,0xa46f0101,0xfb850108,0xa4700101,0xfb850108,0xa4710101,
	0xfb850108,0xa4720101,0xfb850108,0xa4730101,0xfb850108,0xa4740101,0xfb850108,0xa4750101,
	0xfb850108,0xa4760101,0xfb850108,0xa4770101,0xfb850108,0xa4780101,0xfb850108,0xa4790101,
	0xfb850108,0xa47a0101,0xfb850108,0xa47b0101,0xfb850108,0xa47c0101,0xfb850108,0xa47d0101,
	0xfb850108,0xa47e0101,0xfb850108,0xa47f0101,0xfb850108,0xa4800101,0xfb850108,0xa4810101,
	0xfb850108,0xa4820101,0xfb850108,0xa4830101,0xfb850108,0xa4840101,0xfb850108,0xa4850101,
	0xfb850108,0xa4860101,0xfb850108,0xa4870101,0xfb850108,0xa4880101,0xfb850108,0xa4890101,
	0xfb850108,0xa48a0101,0xfb850108,0xa48b0101,0xfb850108,0xa48c0101,0xfb850108,0xa48d0101,
	0xfb850108,0xa48e0101,0xfb850108,0xa48f0101,0xfb850108,0xa4900101,0xfb850108,0xa4910101,
	0xfb850108,0xa4920101,0xfb850108,0xa4930101,0xfb850108,0xa4940101,0xfb850108,0xa4950101,
	0xfb850108,0xa4960101,0xfb850108,0xa4970101,0xfb850108,0xa4980101,0xfb850108,0xa4990101,
	0xfb850108,0xa49a0101,0xfb850108,0xa49b0101,0xfb850108,0xa49c0101,0xfb850108,0xa49d0101,
	0xfb850108,0xa49e0101,0xfb850108,0xa49f0101,0xfb850108,0xa4a00101,0xfb850108,0xa4a10101,
	0xfb850108,0xa4a20101,0xfb850108,0xa4a30101,0xfb850108,0xa4a40101,0xfb850108,0xa4a50101,
	0xfb850108,0xa4a60101,0xfb850108,0xa4a70101,0xfb850108,0xa4a80101,0xfb850108,0xa4a90101,
	0xfb850108,0xa4aa0101,0xfb850108,0xa4ab0101,0xfb850108,0xa4ac0101,0xfb850108,0xa4ad0101,
	0xfb850108,0xa4ae0101,0xfb850108,0xa4af0101,0xfb850108,0xa4b00101,0xfb850108,0xa4b10101,
	0xfb850108,0xa4b20101,0xfb850108,0xa4b30101,0xfb850108,0xa4b40101,0xfb850108,0xa4b50101,
	0xfb850108,0xa4b60101,0xfb850108,0xa4b70101,0xfb850108,0xa4b80101,0xfb850108,0xa4b90101,
	0xfb850108,0xa4ba0101,0xfb850108,0xa4bb0101,0xfb850108,0xa4bc0101,0xfb850108,0xa4bd0101,
	0xfb850108,0xa4be0101,0xfb850108,0xa4bf0101,0xfb850108,0xa4c00101,0xfb850108,0xa4c10101,
	0xfb850108,0xa4c20101,0xfb850108,0xa4c30101,0xfb850108,0xa4c40101,0xfb850108,0xa4c50101,
	0xfb850108,0xa4c60101,0xfb850108,0xa4c70101,0xfb850108,0xa4c80101,0xfb850108,0xa4c90101,
	0xfb850108,0xa4ca0101,0xfb850108,0xa4cb0101,0xfb850108,0xa4cc0101,0xfb850108,0xa4cd0101,
	0xfb850108,0xa4ce0101,0xfb850108,0xa4cf0101,0xfb850108,0xa4d00101,0xfb850108,0xa4d10101,
	0xfb850108,0xa4d20101,0xfb850108,0xa4d30101,0xfb850108,0xa4d40101,0xfb850108,0xa4d50101,
	0xfb850108,0xa4d60101,0xfb850108,0xa4d70101,0xfb850108,0xa4d80101,0xfb850108,0xa4d90101,
	0xfb850108,0xa4da0101,0xfb850108,0xa4db0101,0xfb850108,0xa4dc0101,0xfb850108,0xa4dd0101,
	0xfb850108,0xa4de0101,0xfb850108,0xa4df0101,0xfb850108,0xa4e00101,0xfb850108,0xa4e10101,
	0xfb850108,0xa4e20101,0xfb850108,0xa4e30101,0xfb850108,0xa4e40101,0xfb850108,0xa4e50101,
	0xfb850108,0xa4e60101,0xfb850108,0xa4e70101,0xfb850108,0xa4e80101,0xfb850108,0xa4e90101,
	0xfb850108,0xa4ea0101,0xfb850108,0xa4eb0101,0xfb850108,0xa4ec0101,0xfb850108,0xa4ed0101,
	0xfb850108,0xa4ee0101,0xfb850108,0xa4ef0101,0xfb850108,0xa4f00101,0xfb850108,0xa4f10101,
	0xfb850108,0xa4f20101,0xfb850108,0xa4f30101,0xfb850108,0xa4f40101,0xfb850108,0xa4f50101,
	0xfb850108,0xa4f60101,0xfb850108,0xa4f70101,0xfb850108,0xa4f80101,0xfb850108,0xa4f90101,
	0xfb850108,0xa4fa0101,0xfb850108,0xa4fb0101,0xfb850108,0xa4fc0101,0xfb850108,0xa4fd0101,
	0xfb850108,0xa4fe0101,0xfb850108,0xa4ff0101,0xfb850108,0xa5000101,0xfb850108,0xa5010101,
	0xfb850108,0xa5020101,0xfb850108,0xa5030101,0xfb850108,0xa5040101,0xfb850108,0xa5050101,
	0xfb850108,0xa5060101,0xfb850108,0xa5070101,0xfb850108,0xa5080101,0xfb850108,0xa5090101,
	0xfb850108,0xa50a0101,0xfb850108,0xa50b0101,0xfb850108,0xa50c0101,0xfb850108,0xa50d0101,
	0xfb850108,0xa50e0101,0xfb850108,0xa50f0101,0xfb850108,0xa5100101,0xfb850108,0xa5110101,
	0xfb850108,0xa5120101,0xfb850108,0xa5130101,0xfb850108,0xa5140101,0xfb850108,0xa5150101,
	0xfb850108,0xa5160101,0xfb850108,0xa5170101,0xfb850108,0xa5180101,0xfb850108,0xa5190101,
	0xfb850108,0xa51a0101,0xfb850108,0xa51b0101,0xfb850108,0xa51c0101,0xfb850108,0xa51d0101,
	0xfb850108,0xa51e0101,0xfb850108,0xa51f0101,0xfb850108,0xa5200101,0xfb850108,0xa5210101,
	0xfb850108,0xa5220101,0xfb850108,0xa5230101,0xfb850108,0xa5240101,0xfb850108,0xa5250101,
	0xfb850108,0xa5260101,0xfb850108,0xa5270101,0xfb850108,0xa5280101,0xfb850108,0xa5290101,
	0xfb850108,0xa52a0101,0xfb850108,0xa52b0101,0xfb850108,0xa52c0101,0xfb850108,0xa52d0101,
	0xfb850108,0xa52e0101,0xfb850108,0xa52f0101,0xfb850108,0xa5300101,0xfb850108,0xa5310101,
	0xfb850108,0xa5320101,0xfb850108,0xa5330101,0xfb850108,0xa5340101,0xfb850108,0xa5350101,
	0xfb850108,0xa5360101,0xfb850108,0xa5370101,0xfb850108,0xa5380101,0xfb850108,0xa5390101,
	0xfb850108,0xa53a0101,0xfb850108,0xa53b0101,0xfb850108,0xa53c0101,0xfb850108,0xa53d0101,
	0xfb850108,0xa53e0101,0xfb850108,0xa53f0101,0xfb850108,0xa5400101,0xfb850108,0xa5410101,
	0xfb850108,0xa5420101,0xfb850108,0xa5430101,0xfb850108,0xa5440101,0xfb850108,0xa5450101,
	0xfb850108,0xa5460101,0xfb850108,0xa5470101,0xfb850108,0xa5480101,0xfb850108,0xa5490101,
	0xfb850108,0xa54a0101,0xfb850108,0xa54b0101,0xfb850108,0xa54c0101,0xfb850108,0xa54d0101,
	0xfb850108,0xa54e0101,0xfb850108,0xa54f0101,0xfb850108,0xa5500101,0xfb850108,0xa5510101,
	0xfb850108,0xa5520101,0xfb850108,0xa5530101,0xfb850108,0xa5540101,0xfb850108,0xa5550101,
	0xfb850108,0xa5560101,0xfb850108,0xa5570101,0xfb850108,0xa5580101,0xfb850108,0xa5590101,
	0xfb850108,0xa55a0101,0xfb850108,0xa55b0101,0xfb850108,0xa55c0101,0xfb850108,0xa55d0101,
	0xfb850108,0xa55e0101,0xfb850108,0xa55f0101,0xfb850108,0xa5600101,0xfb850108,0xa5610101,
	0xfb850108,0xa5620101,0xfb850108,0xa5630101,0xfb850108,0xa5640101,0xfb850108,0xa5650101,
	0xfb850108,0xa5660101,0xfb850108,0xa5670101,0xfb850108,0xa5680101,0xfb850108,0xa5690101,
	0xfb850108,0xa56a0101,0xfb850108,0xa56b0101,0xfb850108,0xa56c0101,0xfb850108,0xa56d0101,
	0xfb850108,0xa56e0101,0xfb850108,0xa56f0101,0xfb850108,0xa5700101,0xfb850108,0xa5710101,
	0xfb850108,0xa5720101,0xfb850108,0xa5730101,0xfb850108,0xa5740101,0xfb850108,0xa5750101,
	0xfb850108,0xa5760101,0xfb850108,0xa5770101,0xfb850108,0xa5780101,0xfb850108,0xa5790101,
	0xfb850108,0xa57a0101,0xfb850108,0xa57b0101,0xfb850108,0xa57c0101,0xfb850108,0xa57d0101,
	0xfb850108,0xa57e0101,0xfb850108,0xa57f0101,0xfb850108,0xa5800101,0xfb850108,0xa5810101,
	0xfb850108,0xa5820101,0xfb850108,0xa5830101,0xfb850108,0xa5840101,0xfb850108,0xa5850101,
	0xfb850108,0xa5860101,0xfb850108,0xa5870101,0xfb850108,0xa5880101,0xfb850108,0xa5890101,
	0xfb850108,0xa58a0101,0xfb850108,0xa58b0101,0xfb850108,0xa58c0101,0xfb850108,0xa58d0101,
	0xfb850108,0xa58e0101,0xfb850108,0xa58f0101,0xfb850108,0xa5900101,0xfb850108,0xa5910101,
	0xfb850108,0xa5920101,0xfb850108,0xa5930101,0xfb850108,0xa5940101,0xfb850108,0xa5950101,
	0xfb850108,0xa5960101,0xfb850108,0xa5970101,0xfb850108,0xa5980101,0xfb850108,0xa5990101,
	0xfb850108,0xa59a0101,0xfb850108,0xa59b0101,0xfb850108,0xa59c0101,0xfb850108,0xa59d0101,
	0xfb850108,0xa59e0101,0xfb850108,0xa59f0101,0xfb850108,0xa5a00101,0xfb850108,0xa5a10101,
	0xfb850108,0xa5a20101,0xfb850108,0xa5a30101,0xfb850108,0xa5a40101,0xfb850108,0xa5a50101,
	0xfb850108,0xa5a60101,0xfb850108,0xa5a70101,0xfb850108,0xa5a80101,0xfb850108,0xa5a90101,
	0xfb850108,0xa5aa0101,0xfb850108,0xa5ab0101,0xfb850108,0xa5ac0101,0xfb850108,0xa5ad0101,
	0xfb850108,0xa5ae0101,0xfb850108,0xa5af0101,0xfb850108,0xa5b00101,0xfb850108,0xa5b10101,
	0xfb850108,0xa5b20101,0xfb850108,0xa5b30101,0xfb850108,0xa5b40101,0xfb850108,0xa5b50101,
	0xfb850108,0xa5b60101,0xfb850108,0xa5b70101,0xfb850108,0xa5b80101,0xfb850108,0xa5b90101,
	0xfb850108,0xa5ba0101,0xfb850108,0xa5bb0101,0xfb850108,0xa5bc0101,0xfb850108,0xa5bd0101,
	0xfb850108,0xa5be0101,0xfb850108,0xa5bf0101,0xfb850108,0xa5c00101,0xfb850108,0xa5c10101,
	0xfb850108,0xa5c20101,0xfb850108,0xa5c30101,0xfb850108,0xa5c40101,0xfb850108,0xa5c50101,
	0xfb850108,0xa5c60101,0xfb850108,0xa5c70101,0xfb850108,0xa5c80101,0xfb850108,0xa5c90101,
	0xfb850108,0xa5ca0101,0xfb850108,0xa5cb0101,0xfb850108,0xa5cc0101,0xfb850108,0xa5cd0101,
	0xfb850108,0xa5ce0101,0xfb850108,0xa5cf0101,0xfb850108,0xa5d00101,0xfb850108,0xa5d10101,
	0xfb850108,0xa5d20101,0xfb850108,0xa5d30101,0xfb850108,0xa5d40101,0xfb850108,0xa5d50101,
	0xfb850108,0xa5d60101,0xfb850108,0xa5d70101,0xfb850108,0xa5d80101,0xfb850108,0xa5d90101,
	0xfb850108,0xa5da0101,0xfb850108,0xa5db0101,0xfb850108,0xa5dc0101,0xfb850108,0xa5dd0101,
	0xfb850108,0xa5de0101,0xfb850108,0xa5df0101,0xfb850108,0xa5e00101,0xfb850108,0xa5e10101,
	0xfb850108,0xa5e20101,0xfb850108,0xa5e30101,0xfb850108,0xa5e40101,0xfb850108,0xa5e50101,
	0xfb850108,0xa5e60101,0xfb850108,0xa5e70101,0xfb850108,0xa5e80101,0xfb850108,0xa5e90101,
	0xfb850108,0xa5ea0101,0xfb850108,0xa5eb0101,0xfb850108,0xa5ec0101,0xfb850108,0xa5ed0101,
	0xfb850108,0xa5ee0101,0xfb850108,0xa5ef0101,0xfb850108,0xa5f00101,0xfb850108,0xa5f10101,
	0xfb850108,0xa5f20101,0xfb850108,0xa5f30101,0xfb850108,0xa5f40101,0xfb850108,0xa5f50101,
	0xfb850108,0xa5f60101,0xfb850108,0xa5f70101,0xfb850108,0xa5f80101,0xfb850108,0xa5f90101,
	0xfb850108,0xa5fa0101,0xfb850108,0xa5fb0101,0xfb850108,0xa5fc0101,0xfb850108,0xa5fd0101,
	0xfb850108,0xa5fe0101,0xfb850108,0xa5ff0101,0xfb850108,0xa6000101,0xfb850108,0xa6010101,
	0xfb850108,0xa6020101,0xfb850108,0xa6030101,0xfb850108,0xa6040101,0xfb850108,0xa6050101,
	0xfb850108,0xa6060101,0xfb850108,0xa6070101,0xfb850108,0xa6080101,0xfb850108,0xa6090101,
	0xfb850108,0xa60a0101,0xfb850108,0xa60b0101,0xfb850108,0xa60c0101,0xfb850108,0xa60d0101,
	0xfb850108,0xa60e0101,0xfb850108,0xa60f0101,0xfb850108,0xa6100101,0xfb850108,0xa6110101,
	0xfb850108,0xa6120101,0xfb850108,0xa6130101,0xfb850108,0xa6140101,0xfb850108,0xa6150101,
	0xfb850108,0xa6160101,0xfb850108,0xa6170101,0xfb850108,0xa6180101,0xfb850108,0xa6190101,
	0xfb850108,0xa61a0101,0xfb850108,0xa61b0101,0xfb850108,0xa61c0101,0xfb850108,0xa61d0101,
	0xfb850108,0xa61e0101,0xfb850108,0xa61f0101,0xfb850108,0xa6200101,0xfb850108,0xa6210101,
	0xfb850108,0xa6220101,0xfb850108,0xa6230101,0xfb850108,0xa6240101,0xfb850108,0xa6250101,
	0xfb850108,0xa6260101,0xfb850108,0xa6270101,0xfb850108,0xa6280101,0xfb850108,0xa6290101,
	0xfb850108,0xa62a0101,0xfb850108,0xa62b0101,0xfb850108,0xa62c0101,0xfb850108,0xa62d0101,
	0xfb850108,0xa62e0101,0xfb850108,0xa62f0101,0xfb850108,0xa6300101,0xfb850108,0xa6310101,
	0xfb850108,0xa6320101,0xfb850108,0xa6330101,0xfb850108,0xa6340101,0xfb850108,0xa6350101,
	0xfb850108,0xa6360101,0xfb850108,0xa6370101,0xfb850108,0xa6380101,0xfb850108,0xa6390101,
	0xfb850108,0xa63a0101,0xfb850108,0xa63b0101,0xfb850108,0xa63c0101,0xfb850108,0xa63d0101,
	0xfb850108,0xa63e0101,0xfb850108,0xa63f0101,0xfb850108,0xa6400101,0xfb850108,0xa6410101,
	0xfb850108,0xa6420101,0xfb850108,0xa6430101,0xfb850108,0xa6440101,0xfb850108,0xa6450101,
	0xfb850108,0xa6460101,0xfb850108,0xa6470101,0xfb850108,0xa6480101,0xfb850108,0xa6490101,
	0xfb850108,0xa64a0101,0xfb850108,0xa64b0101,0xfb850108,0xa64c0101,0xfb850108,0xa64d0101,
	0xfb850108,0xa64e0101,0xfb850108,0xa64f0101,0xfb850108,0xa6500101,0xfb850108,0xa6510101,
	0xfb850108,0xa6520101,0xfb850108,0xa6530101,0xfb850108,0xa6540101,0xfb850108,0xa6550101,
	0xfb850108,0xa6560101,0xfb850108,0xa6570101,0xfb850108,0xa6580101,0xfb850108,0xa6590101,
	0xfb850108,0xa65a0101,0xfb850108,0xa65b0101,0xfb850108,0xa65c0101,0xfb850108,0xa65d0101,
	0xfb850108,0xa65e0101,0xfb850108,0xa65f0101,0xfb850108,0xa6600101,0xfb850108,0xa6610101,
	0xfb850108,0xa6620101,0xfb850108,0xa6630101,0xfb850108,0xa6640101,0xfb850108,0xa6650101,
	0xfb850108,0xa6660101,0xfb850108,0xa6670101,0xfb850108,0xa6680101,0xfb850108,0xa6690101,
	0xfb850108,0xa66a0101,0xfb850108,0xa66b0101,0xfb850108,0xa66c0101,0xfb850108,0xa66d0101,
	0xfb850108,0xa66e0101,0xfb850108,0xa66f0101,0xfb850108,0xa6700101,0xfb850108,0xa6710101,
	0xfb850108,0xa6720101,0xfb850108,0xa6730101,0xfb850108,0xa6740101,0xfb850108,0xa6750101,
	0xfb850108,0xa6760101,0xfb850108,0xa6770101,0xfb850108,0xa6780101,0xfb850108,0xa6790101,
	0xfb850108,0xa67a0101,0xfb850108,0xa67b0101,0xfb850108,0xa67c0101,0xfb850108,0xa67d0101,
	0xfb850108,0xa67e0101,0xfb850108,0xa67f0101,0xfb850108,0xa6800101,0xfb850108,0xa6810101,
	0xfb850108,0xa6820101,0xfb850108,0xa6830101,0xfb850108,0xa6840101,0xfb850108,0xa6850101,
	0xfb850108,0xa6860101,0xfb850108,0xa6870101,0xfb850108,0xa6880101,0xfb850108,0xa6890101,
	0xfb850108,0xa68a0101,0xfb850108,0xa68b0101,0xfb850108,0xa68c0101,0xfb850108,0xa68d0101,
	0xfb850108,0xa68e0101,0xfb850108,0xa68f0101,0xfb850108,0xa6900101,0xfb850108,0xa6910101,
	0xfb850108,0xa6920101,0xfb850108,0xa6930101,0xfb850108,0xa6940101,0xfb850108,0xa6950101,
	0xfb850108,0xa6960101,0xfb850108,0xa6970101,0xfb850108,0xa6980101,0xfb850108,0xa6990101,
	0xfb850108,0xa69a0101,0xfb850108,0xa69b0101,0xfb850108,0xa69c0101,0xfb850108,0xa69d0101,
	0xfb850108,0xa69e0101,0xfb850108,0xa69f0101,0xfb850108,0xa6a00101,0xfb850108,0xa6a10101,
	0xfb850108,0xa6a20101,0xfb850108,0xa6a30101,0xfb850108,0xa6a40101,0xfb850108,0xa6a50101,
	0xfb850108,0xa6a60101,0xfb850108,0xa6a70101,0xfb850108,0xa6a80101,0xfb850108,0xa6a90101,
	0xfb850108,0xa6aa0101,0xfb850108,0xa6ab0101,0xfb850108,0xa6ac0101,0xfb850108,0xa6ad0101,
	0xfb850108,0xa6ae0101,0xfb850108,0xa6af0101,0xfb850108,0xa6b00101,0xfb850108,0xa6b10101,
	0xfb850108,0xa6b20101,0xfb850108,0xa6b30101,0xfb850108,0xa6b40101,0xfb850108,0xa6b50101,
	0xfb850108,0xa6b60101,0xfb850108,0xa6b70101,0xfb850108,0xa6b80101,0xfb850108,0xa6b90101,
	0xfb850108,0xa6ba0101,0xfb850108,0xa6bb0101,0xfb850108,0xa6bc0101,0xfb850108,0xa6bd0101,
	0xfb850108,0xa6be0101,0xfb850108,0xa6bf0101,0xfb850108,0xa6c00101,0xfb850108,0xa6c10101,
	0xfb850108,0xa6c20101,0xfb850108,0xa6c30101,0xfb850108,0xa6c40101,0xfb850108,0xa6c50101,
	0xfb850108,0xa6c60101,0xfb850108,0xa6c70101,0xfb850108,0xa6c80101,0xfb850108,0xa6c90101,
	0xfb850108,0xa6ca0101,0xfb850108,0xa6cb0101,0xfb850108,0xa6cc0101,0xfb850108,0xa6cd0101,
	0xfb850108,0xa6ce0101,0xfb850108,0xa6cf0101,0xfb850108,0xa6d00101,0xfb850108,0xa6d10101,
	0xfb850108,0xa6d20101,0xfb850108,0xa6d30101,0xfb850108,0xa6d40101,0xfb850108,0xa6d50101,
	0xfb850108,0xa6d60101,
	};

static const TUint32 TheGB18030Index[] = 
	{	 // 70316 indices
	0x20181aca,0x20191acb,0x201c1acc,0x201d1acd,0x30011ac9,0x30021ac8,0x34008970,0x34018972,
	0x34028974,0x34038976,0x34048978,0x3405897a,0x3406897c,0x3407897e,0x34088980,0x34098982,
	0x340a8984,0x340b8986,0x340c8988,0x340d898a,0x340e898c,0x340f898e,0x34108990,0x34118992,
	0x34128994,0x34138996,0x34148998,0x3415899a,0x3416899c,0x3417899e,0x341889a0,0x341989a2,
	0x341a89a4,0x341b89a6,0x341c89a8,0x341d89aa,0x341e89ac,0x341f89ae,0x342089b0,0x342189b2,
	0x342289b4,0x342389b6,0x342489b8,0x342589ba,0x342689bc,0x342789be,0x342889c0,0x342989c2,
	0x342a89c4,0x342b89c6,0x342c89c8,0x342d89ca,0x342e89cc,0x342f89ce,0x343089d0,0x343189d2,
	0x343289d4,0x343389d6,0x343489d8,0x343589da,0x343689dc,0x343789de,0x343889e0,0x343989e2,
	0x343a89e4,0x343b89e6,0x343c89e8,0x343d89ea,0x343e89ec,0x343f89ee,0x344089f0,0x344189f2,
	0x344289f4,0x344389f6,0x344489f8,0x344589fa,0x344689fc,0x344789fe,0x34488a00,0x34498a02,
	0x344a8a04,0x344b8a06,0x344c8a08,0x344d8a0a,0x344e8a0c,0x344f8a0e,0x34508a10,0x34518a12,
	0x34528a14,0x34538a16,0x34548a18,0x34558a1a,0x34568a1c,0x34578a1e,0x34588a20,0x34598a22,
	0x345a8a24,0x345b8a26,0x345c8a28,0x345d8a2a,0x345e8a2c,0x345f8a2e,0x34608a30,0x34618a32,
	0x34628a34,0x34638a36,0x34648a38,0x34658a3a,0x34668a3c,0x34678a3e,0x34688a40,0x34698a42,
	0x346a8a44,0x346b8a46,0x346c8a48,0x346d8a4a,0x346e8a4c,0x346f8a4e,0x34708a50,0x34718a52,
	0x34728a54,0x34738a56,0x34748a58,0x34758a5a,0x34768a5c,0x34778a5e,0x34788a60,0x34798a62,
	0x347a8a64,0x347b8a66,0x347c8a68,0x347d8a6a,0x347e8a6c,0x347f8a6e,0x34808a70,0x34818a72,
	0x34828a74,0x34838a76,0x34848a78,0x34858a7a,0x34868a7c,0x34878a7e,0x34888a80,0x34898a82,
	0x348a8a84,0x348b8a86,0x348c8a88,0x348d8a8a,0x348e8a8c,0x348f8a8e,0x34908a90,0x34918a92,
	0x34928a94,0x34938a96,0x34948a98,0x34958a9a,0x34968a9c,0x34978a9e,0x34988aa0,0x34998aa2,
	0x349a8aa4,0x349b8aa6,0x349c8aa8,0x349d8aaa,0x349e8aac,0x349f8aae,0x34a08ab0,0x34a18ab2,
	0x34a28ab4,0x34a38ab6,0x34a48ab8,0x34a58aba,0x34a68abc,0x34a78abe,0x34a88ac0,0x34a98ac2,
	0x34aa8ac4,0x34ab8ac6,0x34ac8ac8,0x34ad8aca,0x34ae8acc,0x34af8ace,0x34b08ad0,0x34b18ad2,
	0x34b28ad4,0x34b38ad6,0x34b48ad8,0x34b58ada,0x34b68adc,0x34b78ade,0x34b88ae0,0x34b98ae2,
	0x34ba8ae4,0x34bb8ae6,0x34bc8ae8,0x34bd8aea,0x34be8aec,0x34bf8aee,0x34c08af0,0x34c18af2,
	0x34c28af4,0x34c38af6,0x34c48af8,0x34c58afa,0x34c68afc,0x34c78afe,0x34c88b00,0x34c98b02,
	0x34ca8b04,0x34cb8b06,0x34cc8b08,0x34cd8b0a,0x34ce8b0c,0x34cf8b0e,0x34d08b10,0x34d18b12,
	0x34d28b14,0x34d38b16,0x34d48b18,0x34d58b1a,0x34d68b1c,0x34d78b1e,0x34d88b20,0x34d98b22,
	0x34da8b24,0x34db8b26,0x34dc8b28,0x34dd8b2a,0x34de8b2c,0x34df8b2e,0x34e08b30,0x34e18b32,
	0x34e28b34,0x34e38b36,0x34e48b38,0x34e58b3a,0x34e68b3c,0x34e78b3e,0x34e88b40,0x34e98b42,
	0x34ea8b44,0x34eb8b46,0x34ec8b48,0x34ed8b4a,0x34ee8b4c,0x34ef8b4e,0x34f08b50,0x34f18b52,
	0x34f28b54,0x34f38b56,0x34f48b58,0x34f58b5a,0x34f68b5c,0x34f78b5e,0x34f88b60,0x34f98b62,
	0x34fa8b64,0x34fb8b66,0x34fc8b68,0x34fd8b6a,0x34fe8b6c,0x34ff8b6e,0x35008b70,0x35018b72,
	0x35028b74,0x35038b76,0x35048b78,0x35058b7a,0x35068b7c,0x35078b7e,0x35088b80,0x35098b82,
	0x350a8b84,0x350b8b86,0x350c8b88,0x350d8b8a,0x350e8b8c,0x350f8b8e,0x35108b90,0x35118b92,
	0x35128b94,0x35138b96,0x35148b98,0x35158b9a,0x35168b9c,0x35178b9e,0x35188ba0,0x35198ba2,
	0x351a8ba4,0x351b8ba6,0x351c8ba8,0x351d8baa,0x351e8bac,0x351f8bae,0x35208bb0,0x35218bb2,
	0x35228bb4,0x35238bb6,0x35248bb8,0x35258bba,0x35268bbc,0x35278bbe,0x35288bc0,0x35298bc2,
	0x352a8bc4,0x352b8bc6,0x352c8bc8,0x352d8bca,0x352e8bcc,0x352f8bce,0x35308bd0,0x35318bd2,
	0x35328bd4,0x35338bd6,0x35348bd8,0x35358bda,0x35368bdc,0x35378bde,0x35388be0,0x35398be2,
	0x353a8be4,0x353b8be6,0x353c8be8,0x353d8bea,0x353e8bec,0x353f8bee,0x35408bf0,0x35418bf2,
	0x35428bf4,0x35438bf6,0x35448bf8,0x35458bfa,0x35468bfc,0x35478bfe,0x35488c00,0x35498c02,
	0x354a8c04,0x354b8c06,0x354c8c08,0x354d8c0a,0x354e8c0c,0x354f8c0e,0x35508c10,0x35518c12,
	0x35528c14,0x35538c16,0x35548c18,0x35558c1a,0x35568c1c,0x35578c1e,0x35588c20,0x35598c22,
	0x355a8c24,0x355b8c26,0x355c8c28,0x355d8c2a,0x355e8c2c,0x355f8c2e,0x35608c30,0x35618c32,
	0x35628c34,0x35638c36,0x35648c38,0x35658c3a,0x35668c3c,0x35678c3e,0x35688c40,0x35698c42,
	0x356a8c44,0x356b8c46,0x356c8c48,0x356d8c4a,0x356e8c4c,0x356f8c4e,0x35708c50,0x35718c52,
	0x35728c54,0x35738c56,0x35748c58,0x35758c5a,0x35768c5c,0x35778c5e,0x35788c60,0x35798c62,
	0x357a8c64,0x357b8c66,0x357c8c68,0x357d8c6a,0x357e8c6c,0x357f8c6e,0x35808c70,0x35818c72,
	0x35828c74,0x35838c76,0x35848c78,0x35858c7a,0x35868c7c,0x35878c7e,0x35888c80,0x35898c82,
	0x358a8c84,0x358b8c86,0x358c8c88,0x358d8c8a,0x358e8c8c,0x358f8c8e,0x35908c90,0x35918c92,
	0x35928c94,0x35938c96,0x35948c98,0x35958c9a,0x35968c9c,0x35978c9e,0x35988ca0,0x35998ca2,
	0x359a8ca4,0x359b8ca6,0x359c8ca8,0x359d8caa,0x359e8cac,0x359f8cae,0x35a08cb0,0x35a18cb2,
	0x35a28cb4,0x35a38cb6,0x35a48cb8,0x35a58cba,0x35a68cbc,0x35a78cbe,0x35a88cc0,0x35a98cc2,
	0x35aa8cc4,0x35ab8cc6,0x35ac8cc8,0x35ad8cca,0x35ae8ccc,0x35af8cce,0x35b08cd0,0x35b18cd2,
	0x35b28cd4,0x35b38cd6,0x35b48cd8,0x35b58cda,0x35b68cdc,0x35b78cde,0x35b88ce0,0x35b98ce2,
	0x35ba8ce4,0x35bb8ce6,0x35bc8ce8,0x35bd8cea,0x35be8cec,0x35bf8cee,0x35c08cf0,0x35c18cf2,
	0x35c28cf4,0x35c38cf6,0x35c48cf8,0x35c58cfa,0x35c68cfc,0x35c78cfe,0x35c88d00,0x35c98d02,
	0x35ca8d04,0x35cb8d06,0x35cc8d08,0x35cd8d0a,0x35ce8d0c,0x35cf8d0e,0x35d08d10,0x35d18d12,
	0x35d28d14,0x35d38d16,0x35d48d18,0x35d58d1a,0x35d68d1c,0x35d78d1e,0x35d88d20,0x35d98d22,
	0x35da8d24,0x35db8d26,0x35dc8d28,0x35dd8d2a,0x35de8d2c,0x35df8d2e,0x35e08d30,0x35e18d32,
	0x35e28d34,0x35e38d36,0x35e48d38,0x35e58d3a,0x35e68d3c,0x35e78d3e,0x35e88d40,0x35e98d42,
	0x35ea8d44,0x35eb8d46,0x35ec8d48,0x35ed8d4a,0x35ee8d4c,0x35ef8d4e,0x35f08d50,0x35f18d52,
	0x35f28d54,0x35f38d56,0x35f48d58,0x35f58d5a,0x35f68d5c,0x35f78d5e,0x35f88d60,0x35f98d62,
	0x35fa8d64,0x35fb8d66,0x35fc8d68,0x35fd8d6a,0x35fe8d6c,0x35ff8d6e,0x36008d70,0x36018d72,
	0x36028d74,0x36038d76,0x36048d78,0x36058d7a,0x36068d7c,0x36078d7e,0x36088d80,0x36098d82,
	0x360a8d84,0x360b8d86,0x360c8d88,0x360d8d8a,0x360e8d8c,0x360f8d8e,0x36108d90,0x36118d92,
	0x36128d94,0x36138d96,0x36148d98,0x36158d9a,0x36168d9c,0x36178d9e,0x36188da0,0x36198da2,
	0x361a8da4,0x361b8da6,0x361c8da8,0x361d8daa,0x361e8dac,0x361f8dae,0x36208db0,0x36218db2,
	0x36228db4,0x36238db6,0x36248db8,0x36258dba,0x36268dbc,0x36278dbe,0x36288dc0,0x36298dc2,
	0x362a8dc4,0x362b8dc6,0x362c8dc8,0x362d8dca,0x362e8dcc,0x362f8dce,0x36308dd0,0x36318dd2,
	0x36328dd4,0x36338dd6,0x36348dd8,0x36358dda,0x36368ddc,0x36378dde,0x36388de0,0x36398de2,
	0x363a8de4,0x363b8de6,0x363c8de8,0x363d8dea,0x363e8dec,0x363f8dee,0x36408df0,0x36418df2,
	0x36428df4,0x36438df6,0x36448df8,0x36458dfa,0x36468dfc,0x36478dfe,0x36488e00,0x36498e02,
	0x364a8e04,0x364b8e06,0x364c8e08,0x364d8e0a,0x364e8e0c,0x364f8e0e,0x36508e10,0x36518e12,
	0x36528e14,0x36538e16,0x36548e18,0x36558e1a,0x36568e1c,0x36578e1e,0x36588e20,0x36598e22,
	0x365a8e24,0x365b8e26,0x365c8e28,0x365d8e2a,0x365e8e2c,0x365f8e2e,0x36608e30,0x36618e32,
	0x36628e34,0x36638e36,0x36648e38,0x36658e3a,0x36668e3c,0x36678e3e,0x36688e40,0x36698e42,
	0x366a8e44,0x366b8e46,0x366c8e48,0x366d8e4a,0x366e8e4c,0x366f8e4e,0x36708e50,0x36718e52,
	0x36728e54,0x36738e56,0x36748e58,0x36758e5a,0x36768e5c,0x36778e5e,0x36788e60,0x36798e62,
	0x367a8e64,0x367b8e66,0x367c8e68,0x367d8e6a,0x367e8e6c,0x367f8e6e,0x36808e70,0x36818e72,
	0x36828e74,0x36838e76,0x36848e78,0x36858e7a,0x36868e7c,0x36878e7e,0x36888e80,0x36898e82,
	0x368a8e84,0x368b8e86,0x368c8e88,0x368d8e8a,0x368e8e8c,0x368f8e8e,0x36908e90,0x36918e92,
	0x36928e94,0x36938e96,0x36948e98,0x36958e9a,0x36968e9c,0x36978e9e,0x36988ea0,0x36998ea2,
	0x369a8ea4,0x369b8ea6,0x369c8ea8,0x369d8eaa,0x369e8eac,0x369f8eae,0x36a08eb0,0x36a18eb2,
	0x36a28eb4,0x36a38eb6,0x36a48eb8,0x36a58eba,0x36a68ebc,0x36a78ebe,0x36a88ec0,0x36a98ec2,
	0x36aa8ec4,0x36ab8ec6,0x36ac8ec8,0x36ad8eca,0x36ae8ecc,0x36af8ece,0x36b08ed0,0x36b18ed2,
	0x36b28ed4,0x36b38ed6,0x36b48ed8,0x36b58eda,0x36b68edc,0x36b78ede,0x36b88ee0,0x36b98ee2,
	0x36ba8ee4,0x36bb8ee6,0x36bc8ee8,0x36bd8eea,0x36be8eec,0x36bf8eee,0x36c08ef0,0x36c18ef2,
	0x36c28ef4,0x36c38ef6,0x36c48ef8,0x36c58efa,0x36c68efc,0x36c78efe,0x36c88f00,0x36c98f02,
	0x36ca8f04,0x36cb8f06,0x36cc8f08,0x36cd8f0a,0x36ce8f0c,0x36cf8f0e,0x36d08f10,0x36d18f12,
	0x36d28f14,0x36d38f16,0x36d48f18,0x36d58f1a,0x36d68f1c,0x36d78f1e,0x36d88f20,0x36d98f22,
	0x36da8f24,0x36db8f26,0x36dc8f28,0x36dd8f2a,0x36de8f2c,0x36df8f2e,0x36e08f30,0x36e18f32,
	0x36e28f34,0x36e38f36,0x36e48f38,0x36e58f3a,0x36e68f3c,0x36e78f3e,0x36e88f40,0x36e98f42,
	0x36ea8f44,0x36eb8f46,0x36ec8f48,0x36ed8f4a,0x36ee8f4c,0x36ef8f4e,0x36f08f50,0x36f18f52,
	0x36f28f54,0x36f38f56,0x36f48f58,0x36f58f5a,0x36f68f5c,0x36f78f5e,0x36f88f60,0x36f98f62,
	0x36fa8f64,0x36fb8f66,0x36fc8f68,0x36fd8f6a,0x36fe8f6c,0x36ff8f6e,0x37008f70,0x37018f72,
	0x37028f74,0x37038f76,0x37048f78,0x37058f7a,0x37068f7c,0x37078f7e,0x37088f80,0x37098f82,
	0x370a8f84,0x370b8f86,0x370c8f88,0x370d8f8a,0x370e8f8c,0x370f8f8e,0x37108f90,0x37118f92,
	0x37128f94,0x37138f96,0x37148f98,0x37158f9a,0x37168f9c,0x37178f9e,0x37188fa0,0x37198fa2,
	0x371a8fa4,0x371b8fa6,0x371c8fa8,0x371d8faa,0x371e8fac,0x371f8fae,0x37208fb0,0x37218fb2,
	0x37228fb4,0x37238fb6,0x37248fb8,0x37258fba,0x37268fbc,0x37278fbe,0x37288fc0,0x37298fc2,
	0x372a8fc4,0x372b8fc6,0x372c8fc8,0x372d8fca,0x372e8fcc,0x372f8fce,0x37308fd0,0x37318fd2,
	0x37328fd4,0x37338fd6,0x37348fd8,0x37358fda,0x37368fdc,0x37378fde,0x37388fe0,0x37398fe2,
	0x373a8fe4,0x373b8fe6,0x373c8fe8,0x373d8fea,0x373e8fec,0x373f8fee,0x37408ff0,0x37418ff2,
	0x37428ff4,0x37438ff6,0x37448ff8,0x37458ffa,0x37468ffc,0x37478ffe,0x37489000,0x37499002,
	0x374a9004,0x374b9006,0x374c9008,0x374d900a,0x374e900c,0x374f900e,0x37509010,0x37519012,
	0x37529014,0x37539016,0x37549018,0x3755901a,0x3756901c,0x3757901e,0x37589020,0x37599022,
	0x375a9024,0x375b9026,0x375c9028,0x375d902a,0x375e902c,0x375f902e,0x37609030,0x37619032,
	0x37629034,0x37639036,0x37649038,0x3765903a,0x3766903c,0x3767903e,0x37689040,0x37699042,
	0x376a9044,0x376b9046,0x376c9048,0x376d904a,0x376e904c,0x376f904e,0x37709050,0x37719052,
	0x37729054,0x37739056,0x37749058,0x3775905a,0x3776905c,0x3777905e,0x37789060,0x37799062,
	0x377a9064,0x377b9066,0x377c9068,0x377d906a,0x377e906c,0x377f906e,0x37809070,0x37819072,
	0x37829074,0x37839076,0x37849078,0x3785907a,0x3786907c,0x3787907e,0x37889080,0x37899082,
	0x378a9084,0x378b9086,0x378c9088,0x378d908a,0x378e908c,0x378f908e,0x37909090,0x37919092,
	0x37929094,0x37939096,0x37949098,0x3795909a,0x3796909c,0x3797909e,0x379890a0,0x379990a2,
	0x379a90a4,0x379b90a6,0x379c90a8,0x379d90aa,0x379e90ac,0x379f90ae,0x37a090b0,0x37a190b2,
	0x37a290b4,0x37a390b6,0x37a490b8,0x37a590ba,0x37a690bc,0x37a790be,0x37a890c0,0x37a990c2,
	0x37aa90c4,0x37ab90c6,0x37ac90c8,0x37ad90ca,0x37ae90cc,0x37af90ce,0x37b090d0,0x37b190d2,
	0x37b290d4,0x37b390d6,0x37b490d8,0x37b590da,0x37b690dc,0x37b790de,0x37b890e0,0x37b990e2,
	0x37ba90e4,0x37bb90e6,0x37bc90e8,0x37bd90ea,0x37be90ec,0x37bf90ee,0x37c090f0,0x37c190f2,
	0x37c290f4,0x37c390f6,0x37c490f8,0x37c590fa,0x37c690fc,0x37c790fe,0x37c89100,0x37c99102,
	0x37ca9104,0x37cb9106,0x37cc9108,0x37cd910a,0x37ce910c,0x37cf910e,0x37d09110,0x37d19112,
	0x37d29114,0x37d39116,0x37d49118,0x37d5911a,0x37d6911c,0x37d7911e,0x37d89120,0x37d99122,
	0x37da9124,0x37db9126,0x37dc9128,0x37dd912a,0x37de912c,0x37df912e,0x37e09130,0x37e19132,
	0x37e29134,0x37e39136,0x37e49138,0x37e5913a,0x37e6913c,0x37e7913e,0x37e89140,0x37e99142,
	0x37ea9144,0x37eb9146,0x37ec9148,0x37ed914a,0x37ee914c,0x37ef914e,0x37f09150,0x37f19152,
	0x37f29154,0x37f39156,0x37f49158,0x37f5915a,0x37f6915c,0x37f7915e,0x37f89160,0x37f99162,
	0x37fa9164,0x37fb9166,0x37fc9168,0x37fd916a,0x37fe916c,0x37ff916e,0x38009170,0x38019172,
	0x38029174,0x38039176,0x38049178,0x3805917a,0x3806917c,0x3807917e,0x38089180,0x38099182,
	0x380a9184,0x380b9186,0x380c9188,0x380d918a,0x380e918c,0x380f918e,0x38109190,0x38119192,
	0x38129194,0x38139196,0x38149198,0x3815919a,0x3816919c,0x3817919e,0x381891a0,0x381991a2,
	0x381a91a4,0x381b91a6,0x381c91a8,0x381d91aa,0x381e91ac,0x381f91ae,0x382091b0,0x382191b2,
	0x382291b4,0x382391b6,0x382491b8,0x382591ba,0x382691bc,0x382791be,0x382891c0,0x382991c2,
	0x382a91c4,0x382b91c6,0x382c91c8,0x382d91ca,0x382e91cc,0x382f91ce,0x383091d0,0x383191d2,
	0x383291d4,0x383391d6,0x383491d8,0x383591da,0x383691dc,0x383791de,0x383891e0,0x383991e2,
	0x383a91e4,0x383b91e6,0x383c91e8,0x383d91ea,0x383e91ec,0x383f91ee,0x384091f0,0x384191f2,
	0x384291f4,0x384391f6,0x384491f8,0x384591fa,0x384691fc,0x384791fe,0x38489200,0x38499202,
	0x384a9204,0x384b9206,0x384c9208,0x384d920a,0x384e920c,0x384f920e,0x38509210,0x38519212,
	0x38529214,0x38539216,0x38549218,0x3855921a,0x3856921c,0x3857921e,0x38589220,0x38599222,
	0x385a9224,0x385b9226,0x385c9228,0x385d922a,0x385e922c,0x385f922e,0x38609230,0x38619232,
	0x38629234,0x38639236,0x38649238,0x3865923a,0x3866923c,0x3867923e,0x38689240,0x38699242,
	0x386a9244,0x386b9246,0x386c9248,0x386d924a,0x386e924c,0x386f924e,0x38709250,0x38719252,
	0x38729254,0x38739256,0x38749258,0x3875925a,0x3876925c,0x3877925e,0x38789260,0x38799262,
	0x387a9264,0x387b9266,0x387c9268,0x387d926a,0x387e926c,0x387f926e,0x38809270,0x38819272,
	0x38829274,0x38839276,0x38849278,0x3885927a,0x3886927c,0x3887927e,0x38889280,0x38899282,
	0x388a9284,0x388b9286,0x388c9288,0x388d928a,0x388e928c,0x388f928e,0x38909290,0x38919292,
	0x38929294,0x38939296,0x38949298,0x3895929a,0x3896929c,0x3897929e,0x389892a0,0x389992a2,
	0x389a92a4,0x389b92a6,0x389c92a8,0x389d92aa,0x389e92ac,0x389f92ae,0x38a092b0,0x38a192b2,
	0x38a292b4,0x38a392b6,0x38a492b8,0x38a592ba,0x38a692bc,0x38a792be,0x38a892c0,0x38a992c2,
	0x38aa92c4,0x38ab92c6,0x38ac92c8,0x38ad92ca,0x38ae92cc,0x38af92ce,0x38b092d0,0x38b192d2,
	0x38b292d4,0x38b392d6,0x38b492d8,0x38b592da,0x38b692dc,0x38b792de,0x38b892e0,0x38b992e2,
	0x38ba92e4,0x38bb92e6,0x38bc92e8,0x38bd92ea,0x38be92ec,0x38bf92ee,0x38c092f0,0x38c192f2,
	0x38c292f4,0x38c392f6,0x38c492f8,0x38c592fa,0x38c692fc,0x38c792fe,0x38c89300,0x38c99302,
	0x38ca9304,0x38cb9306,0x38cc9308,0x38cd930a,0x38ce930c,0x38cf930e,0x38d09310,0x38d19312,
	0x38d29314,0x38d39316,0x38d49318,0x38d5931a,0x38d6931c,0x38d7931e,0x38d89320,0x38d99322,
	0x38da9324,0x38db9326,0x38dc9328,0x38dd932a,0x38de932c,0x38df932e,0x38e09330,0x38e19332,
	0x38e29334,0x38e39336,0x38e49338,0x38e5933a,0x38e6933c,0x38e7933e,0x38e89340,0x38e99342,
	0x38ea9344,0x38eb9346,0x38ec9348,0x38ed934a,0x38ee934c,0x38ef934e,0x38f09350,0x38f19352,
	0x38f29354,0x38f39356,0x38f49358,0x38f5935a,0x38f6935c,0x38f7935e,0x38f89360,0x38f99362,
	0x38fa9364,0x38fb9366,0x38fc9368,0x38fd936a,0x38fe936c,0x38ff936e,0x39009370,0x39019372,
	0x39029374,0x39039376,0x39049378,0x3905937a,0x3906937c,0x3907937e,0x39089380,0x39099382,
	0x390a9384,0x390b9386,0x390c9388,0x390d938a,0x390e938c,0x390f938e,0x39109390,0x39119392,
	0x39129394,0x39139396,0x39149398,0x3915939a,0x3916939c,0x3917939e,0x391893a0,0x391993a2,
	0x391a93a4,0x391b93a6,0x391c93a8,0x391d93aa,0x391e93ac,0x391f93ae,0x392093b0,0x392193b2,
	0x392293b4,0x392393b6,0x392493b8,0x392593ba,0x392693bc,0x392793be,0x392893c0,0x392993c2,
	0x392a93c4,0x392b93c6,0x392c93c8,0x392d93ca,0x392e93cc,0x392f93ce,0x393093d0,0x393193d2,
	0x393293d4,0x393393d6,0x393493d8,0x393593da,0x393693dc,0x393793de,0x393893e0,0x393993e2,
	0x393a93e4,0x393b93e6,0x393c93e8,0x393d93ea,0x393e93ec,0x393f93ee,0x394093f0,0x394193f2,
	0x394293f4,0x394393f6,0x394493f8,0x394593fa,0x394693fc,0x394793fe,0x39489400,0x39499402,
	0x394a9404,0x394b9406,0x394c9408,0x394d940a,0x394e940c,0x394f940e,0x39509410,0x39519412,
	0x39529414,0x39539416,0x39549418,0x3955941a,0x3956941c,0x3957941e,0x39589420,0x39599422,
	0x395a9424,0x395b9426,0x395c9428,0x395d942a,0x395e942c,0x395f942e,0x39609430,0x39619432,
	0x39629434,0x39639436,0x39649438,0x3965943a,0x3966943c,0x3967943e,0x39689440,0x39699442,
	0x396a9444,0x396b9446,0x396c9448,0x396d944a,0x396e944c,0x396f944e,0x39709450,0x39719452,
	0x39729454,0x39739456,0x39749458,0x3975945a,0x3976945c,0x3977945e,0x39789460,0x39799462,
	0x397a9464,0x397b9466,0x397c9468,0x397d946a,0x397e946c,0x397f946e,0x39809470,0x39819472,
	0x39829474,0x39839476,0x39849478,0x3985947a,0x3986947c,0x3987947e,0x39889480,0x39899482,
	0x398a9484,0x398b9486,0x398c9488,0x398d948a,0x398e948c,0x398f948e,0x39909490,0x39919492,
	0x39929494,0x39939496,0x39949498,0x3995949a,0x3996949c,0x3997949e,0x399894a0,0x399994a2,
	0x399a94a4,0x399b94a6,0x399c94a8,0x399d94aa,0x399e94ac,0x399f94ae,0x39a094b0,0x39a194b2,
	0x39a294b4,0x39a394b6,0x39a494b8,0x39a594ba,0x39a694bc,0x39a794be,0x39a894c0,0x39a994c2,
	0x39aa94c4,0x39ab94c6,0x39ac94c8,0x39ad94ca,0x39ae94cc,0x39af94ce,0x39b094d0,0x39b194d2,
	0x39b294d4,0x39b394d6,0x39b494d8,0x39b594da,0x39b694dc,0x39b794de,0x39b894e0,0x39b994e2,
	0x39ba94e4,0x39bb94e6,0x39bc94e8,0x39bd94ea,0x39be94ec,0x39bf94ee,0x39c094f0,0x39c194f2,
	0x39c294f4,0x39c394f6,0x39c494f8,0x39c594fa,0x39c694fc,0x39c794fe,0x39c89500,0x39c99502,
	0x39ca9504,0x39cb9506,0x39cc9508,0x39cd950a,0x39ce950c,0x39cf950e,0x39d09510,0x39d19512,
	0x39d29514,0x39d39516,0x39d49518,0x39d5951a,0x39d6951c,0x39d7951e,0x39d89520,0x39d99522,
	0x39da9524,0x39db9526,0x39dc9528,0x39dd952a,0x39de952c,0x39df952e,0x39e09530,0x39e19532,
	0x39e29534,0x39e39536,0x39e49538,0x39e5953a,0x39e6953c,0x39e7953e,0x39e89540,0x39e99542,
	0x39ea9544,0x39eb9546,0x39ec9548,0x39ed954a,0x39ee954c,0x39ef954e,0x39f09550,0x39f19552,
	0x39f29554,0x39f39556,0x39f49558,0x39f5955a,0x39f6955c,0x39f7955e,0x39f89560,0x39f99562,
	0x39fa9564,0x39fb9566,0x39fc9568,0x39fd956a,0x39fe956c,0x39ff956e,0x3a009570,0x3a019572,
	0x3a029574,0x3a039576,0x3a049578,0x3a05957a,0x3a06957c,0x3a07957e,0x3a089580,0x3a099582,
	0x3a0a9584,0x3a0b9586,0x3a0c9588,0x3a0d958a,0x3a0e958c,0x3a0f958e,0x3a109590,0x3a119592,
	0x3a129594,0x3a139596,0x3a149598,0x3a15959a,0x3a16959c,0x3a17959e,0x3a1895a0,0x3a1995a2,
	0x3a1a95a4,0x3a1b95a6,0x3a1c95a8,0x3a1d95aa,0x3a1e95ac,0x3a1f95ae,0x3a2095b0,0x3a2195b2,
	0x3a2295b4,0x3a2395b6,0x3a2495b8,0x3a2595ba,0x3a2695bc,0x3a2795be,0x3a2895c0,0x3a2995c2,
	0x3a2a95c4,0x3a2b95c6,0x3a2c95c8,0x3a2d95ca,0x3a2e95cc,0x3a2f95ce,0x3a3095d0,0x3a3195d2,
	0x3a3295d4,0x3a3395d6,0x3a3495d8,0x3a3595da,0x3a3695dc,0x3a3795de,0x3a3895e0,0x3a3995e2,
	0x3a3a95e4,0x3a3b95e6,0x3a3c95e8,0x3a3d95ea,0x3a3e95ec,0x3a3f95ee,0x3a4095f0,0x3a4195f2,
	0x3a4295f4,0x3a4395f6,0x3a4495f8,0x3a4595fa,0x3a4695fc,0x3a4795fe,0x3a489600,0x3a499602,
	0x3a4a9604,0x3a4b9606,0x3a4c9608,0x3a4d960a,0x3a4e960c,0x3a4f960e,0x3a509610,0x3a519612,
	0x3a529614,0x3a539616,0x3a549618,0x3a55961a,0x3a56961c,0x3a57961e,0x3a589620,0x3a599622,
	0x3a5a9624,0x3a5b9626,0x3a5c9628,0x3a5d962a,0x3a5e962c,0x3a5f962e,0x3a609630,0x3a619632,
	0x3a629634,0x3a639636,0x3a649638,0x3a65963a,0x3a66963c,0x3a67963e,0x3a689640,0x3a699642,
	0x3a6a9644,0x3a6b9646,0x3a6c9648,0x3a6d964a,0x3a6e964c,0x3a6f964e,0x3a709650,0x3a719652,
	0x3a729654,0x3a739656,0x3a749658,0x3a75965a,0x3a76965c,0x3a77965e,0x3a789660,0x3a799662,
	0x3a7a9664,0x3a7b9666,0x3a7c9668,0x3a7d966a,0x3a7e966c,0x3a7f966e,0x3a809670,0x3a819672,
	0x3a829674,0x3a839676,0x3a849678,0x3a85967a,0x3a86967c,0x3a87967e,0x3a889680,0x3a899682,
	0x3a8a9684,0x3a8b9686,0x3a8c9688,0x3a8d968a,0x3a8e968c,0x3a8f968e,0x3a909690,0x3a919692,
	0x3a929694,0x3a939696,0x3a949698,0x3a95969a,0x3a96969c,0x3a97969e,0x3a9896a0,0x3a9996a2,
	0x3a9a96a4,0x3a9b96a6,0x3a9c96a8,0x3a9d96aa,0x3a9e96ac,0x3a9f96ae,0x3aa096b0,0x3aa196b2,
	0x3aa296b4,0x3aa396b6,0x3aa496b8,0x3aa596ba,0x3aa696bc,0x3aa796be,0x3aa896c0,0x3aa996c2,
	0x3aaa96c4,0x3aab96c6,0x3aac96c8,0x3aad96ca,0x3aae96cc,0x3aaf96ce,0x3ab096d0,0x3ab196d2,
	0x3ab296d4,0x3ab396d6,0x3ab496d8,0x3ab596da,0x3ab696dc,0x3ab796de,0x3ab896e0,0x3ab996e2,
	0x3aba96e4,0x3abb96e6,0x3abc96e8,0x3abd96ea,0x3abe96ec,0x3abf96ee,0x3ac096f0,0x3ac196f2,
	0x3ac296f4,0x3ac396f6,0x3ac496f8,0x3ac596fa,0x3ac696fc,0x3ac796fe,0x3ac89700,0x3ac99702,
	0x3aca9704,0x3acb9706,0x3acc9708,0x3acd970a,0x3ace970c,0x3acf970e,0x3ad09710,0x3ad19712,
	0x3ad29714,0x3ad39716,0x3ad49718,0x3ad5971a,0x3ad6971c,0x3ad7971e,0x3ad89720,0x3ad99722,
	0x3ada9724,0x3adb9726,0x3adc9728,0x3add972a,0x3ade972c,0x3adf972e,0x3ae09730,0x3ae19732,
	0x3ae29734,0x3ae39736,0x3ae49738,0x3ae5973a,0x3ae6973c,0x3ae7973e,0x3ae89740,0x3ae99742,
	0x3aea9744,0x3aeb9746,0x3aec9748,0x3aed974a,0x3aee974c,0x3aef974e,0x3af09750,0x3af19752,
	0x3af29754,0x3af39756,0x3af49758,0x3af5975a,0x3af6975c,0x3af7975e,0x3af89760,0x3af99762,
	0x3afa9764,0x3afb9766,0x3afc9768,0x3afd976a,0x3afe976c,0x3aff976e,0x3b009770,0x3b019772,
	0x3b029774,0x3b039776,0x3b049778,0x3b05977a,0x3b06977c,0x3b07977e,0x3b089780,0x3b099782,
	0x3b0a9784,0x3b0b9786,0x3b0c9788,0x3b0d978a,0x3b0e978c,0x3b0f978e,0x3b109790,0x3b119792,
	0x3b129794,0x3b139796,0x3b149798,0x3b15979a,0x3b16979c,0x3b17979e,0x3b1897a0,0x3b1997a2,
	0x3b1a97a4,0x3b1b97a6,0x3b1c97a8,0x3b1d97aa,0x3b1e97ac,0x3b1f97ae,0x3b2097b0,0x3b2197b2,
	0x3b2297b4,0x3b2397b6,0x3b2497b8,0x3b2597ba,0x3b2697bc,0x3b2797be,0x3b2897c0,0x3b2997c2,
	0x3b2a97c4,0x3b2b97c6,0x3b2c97c8,0x3b2d97ca,0x3b2e97cc,0x3b2f97ce,0x3b3097d0,0x3b3197d2,
	0x3b3297d4,0x3b3397d6,0x3b3497d8,0x3b3597da,0x3b3697dc,0x3b3797de,0x3b3897e0,0x3b3997e2,
	0x3b3a97e4,0x3b3b97e6,0x3b3c97e8,0x3b3d97ea,0x3b3e97ec,0x3b3f97ee,0x3b4097f0,0x3b4197f2,
	0x3b4297f4,0x3b4397f6,0x3b4497f8,0x3b4597fa,0x3b4697fc,0x3b4797fe,0x3b489800,0x3b499802,
	0x3b4a9804,0x3b4b9806,0x3b4c9808,0x3b4d980a,0x3b4e980c,0x3b4f980e,0x3b509810,0x3b519812,
	0x3b529814,0x3b539816,0x3b549818,0x3b55981a,0x3b56981c,0x3b57981e,0x3b589820,0x3b599822,
	0x3b5a9824,0x3b5b9826,0x3b5c9828,0x3b5d982a,0x3b5e982c,0x3b5f982e,0x3b609830,0x3b619832,
	0x3b629834,0x3b639836,0x3b649838,0x3b65983a,0x3b66983c,0x3b67983e,0x3b689840,0x3b699842,
	0x3b6a9844,0x3b6b9846,0x3b6c9848,0x3b6d984a,0x3b6e984c,0x3b6f984e,0x3b709850,0x3b719852,
	0x3b729854,0x3b739856,0x3b749858,0x3b75985a,0x3b76985c,0x3b77985e,0x3b789860,0x3b799862,
	0x3b7a9864,0x3b7b9866,0x3b7c9868,0x3b7d986a,0x3b7e986c,0x3b7f986e,0x3b809870,0x3b819872,
	0x3b829874,0x3b839876,0x3b849878,0x3b85987a,0x3b86987c,0x3b87987e,0x3b889880,0x3b899882,
	0x3b8a9884,0x3b8b9886,0x3b8c9888,0x3b8d988a,0x3b8e988c,0x3b8f988e,0x3b909890,0x3b919892,
	0x3b929894,0x3b939896,0x3b949898,0x3b95989a,0x3b96989c,0x3b97989e,0x3b9898a0,0x3b9998a2,
	0x3b9a98a4,0x3b9b98a6,0x3b9c98a8,0x3b9d98aa,0x3b9e98ac,0x3b9f98ae,0x3ba098b0,0x3ba198b2,
	0x3ba298b4,0x3ba398b6,0x3ba498b8,0x3ba598ba,0x3ba698bc,0x3ba798be,0x3ba898c0,0x3ba998c2,
	0x3baa98c4,0x3bab98c6,0x3bac98c8,0x3bad98ca,0x3bae98cc,0x3baf98ce,0x3bb098d0,0x3bb198d2,
	0x3bb298d4,0x3bb398d6,0x3bb498d8,0x3bb598da,0x3bb698dc,0x3bb798de,0x3bb898e0,0x3bb998e2,
	0x3bba98e4,0x3bbb98e6,0x3bbc98e8,0x3bbd98ea,0x3bbe98ec,0x3bbf98ee,0x3bc098f0,0x3bc198f2,
	0x3bc298f4,0x3bc398f6,0x3bc498f8,0x3bc598fa,0x3bc698fc,0x3bc798fe,0x3bc89900,0x3bc99902,
	0x3bca9904,0x3bcb9906,0x3bcc9908,0x3bcd990a,0x3bce990c,0x3bcf990e,0x3bd09910,0x3bd19912,
	0x3bd29914,0x3bd39916,0x3bd49918,0x3bd5991a,0x3bd6991c,0x3bd7991e,0x3bd89920,0x3bd99922,
	0x3bda9924,0x3bdb9926,0x3bdc9928,0x3bdd992a,0x3bde992c,0x3bdf992e,0x3be09930,0x3be19932,
	0x3be29934,0x3be39936,0x3be49938,0x3be5993a,0x3be6993c,0x3be7993e,0x3be89940,0x3be99942,
	0x3bea9944,0x3beb9946,0x3bec9948,0x3bed994a,0x3bee994c,0x3bef994e,0x3bf09950,0x3bf19952,
	0x3bf29954,0x3bf39956,0x3bf49958,0x3bf5995a,0x3bf6995c,0x3bf7995e,0x3bf89960,0x3bf99962,
	0x3bfa9964,0x3bfb9966,0x3bfc9968,0x3bfd996a,0x3bfe996c,0x3bff996e,0x3c009970,0x3c019972,
	0x3c029974,0x3c039976,0x3c049978,0x3c05997a,0x3c06997c,0x3c07997e,0x3c089980,0x3c099982,
	0x3c0a9984,0x3c0b9986,0x3c0c9988,0x3c0d998a,0x3c0e998c,0x3c0f998e,0x3c109990,0x3c119992,
	0x3c129994,0x3c139996,0x3c149998,0x3c15999a,0x3c16999c,0x3c17999e,0x3c1899a0,0x3c1999a2,
	0x3c1a99a4,0x3c1b99a6,0x3c1c99a8,0x3c1d99aa,0x3c1e99ac,0x3c1f99ae,0x3c2099b0,0x3c2199b2,
	0x3c2299b4,0x3c2399b6,0x3c2499b8,0x3c2599ba,0x3c2699bc,0x3c2799be,0x3c2899c0,0x3c2999c2,
	0x3c2a99c4,0x3c2b99c6,0x3c2c99c8,0x3c2d99ca,0x3c2e99cc,0x3c2f99ce,0x3c3099d0,0x3c3199d2,
	0x3c3299d4,0x3c3399d6,0x3c3499d8,0x3c3599da,0x3c3699dc,0x3c3799de,0x3c3899e0,0x3c3999e2,
	0x3c3a99e4,0x3c3b99e6,0x3c3c99e8,0x3c3d99ea,0x3c3e99ec,0x3c3f99ee,0x3c4099f0,0x3c4199f2,
	0x3c4299f4,0x3c4399f6,0x3c4499f8,0x3c4599fa,0x3c4699fc,0x3c4799fe,0x3c489a00,0x3c499a02,
	0x3c4a9a04,0x3c4b9a06,0x3c4c9a08,0x3c4d9a0a,0x3c4e9a0c,0x3c4f9a0e,0x3c509a10,0x3c519a12,
	0x3c529a14,0x3c539a16,0x3c549a18,0x3c559a1a,0x3c569a1c,0x3c579a1e,0x3c589a20,0x3c599a22,
	0x3c5a9a24,0x3c5b9a26,0x3c5c9a28,0x3c5d9a2a,0x3c5e9a2c,0x3c5f9a2e,0x3c609a30,0x3c619a32,
	0x3c629a34,0x3c639a36,0x3c649a38,0x3c659a3a,0x3c669a3c,0x3c679a3e,0x3c689a40,0x3c699a42,
	0x3c6a9a44,0x3c6b9a46,0x3c6c9a48,0x3c6d9a4a,0x3c6e9a4c,0x3c6f9a4e,0x3c709a50,0x3c719a52,
	0x3c729a54,0x3c739a56,0x3c749a58,0x3c759a5a,0x3c769a5c,0x3c779a5e,0x3c789a60,0x3c799a62,
	0x3c7a9a64,0x3c7b9a66,0x3c7c9a68,0x3c7d9a6a,0x3c7e9a6c,0x3c7f9a6e,0x3c809a70,0x3c819a72,
	0x3c829a74,0x3c839a76,0x3c849a78,0x3c859a7a,0x3c869a7c,0x3c879a7e,0x3c889a80,0x3c899a82,
	0x3c8a9a84,0x3c8b9a86,0x3c8c9a88,0x3c8d9a8a,0x3c8e9a8c,0x3c8f9a8e,0x3c909a90,0x3c919a92,
	0x3c929a94,0x3c939a96,0x3c949a98,0x3c959a9a,0x3c969a9c,0x3c979a9e,0x3c989aa0,0x3c999aa2,
	0x3c9a9aa4,0x3c9b9aa6,0x3c9c9aa8,0x3c9d9aaa,0x3c9e9aac,0x3c9f9aae,0x3ca09ab0,0x3ca19ab2,
	0x3ca29ab4,0x3ca39ab6,0x3ca49ab8,0x3ca59aba,0x3ca69abc,0x3ca79abe,0x3ca89ac0,0x3ca99ac2,
	0x3caa9ac4,0x3cab9ac6,0x3cac9ac8,0x3cad9aca,0x3cae9acc,0x3caf9ace,0x3cb09ad0,0x3cb19ad2,
	0x3cb29ad4,0x3cb39ad6,0x3cb49ad8,0x3cb59ada,0x3cb69adc,0x3cb79ade,0x3cb89ae0,0x3cb99ae2,
	0x3cba9ae4,0x3cbb9ae6,0x3cbc9ae8,0x3cbd9aea,0x3cbe9aec,0x3cbf9aee,0x3cc09af0,0x3cc19af2,
	0x3cc29af4,0x3cc39af6,0x3cc49af8,0x3cc59afa,0x3cc69afc,0x3cc79afe,0x3cc89b00,0x3cc99b02,
	0x3cca9b04,0x3ccb9b06,0x3ccc9b08,0x3ccd9b0a,0x3cce9b0c,0x3ccf9b0e,0x3cd09b10,0x3cd19b12,
	0x3cd29b14,0x3cd39b16,0x3cd49b18,0x3cd59b1a,0x3cd69b1c,0x3cd79b1e,0x3cd89b20,0x3cd99b22,
	0x3cda9b24,0x3cdb9b26,0x3cdc9b28,0x3cdd9b2a,0x3cde9b2c,0x3cdf9b2e,0x3ce09b30,0x3ce19b32,
	0x3ce29b34,0x3ce39b36,0x3ce49b38,0x3ce59b3a,0x3ce69b3c,0x3ce79b3e,0x3ce89b40,0x3ce99b42,
	0x3cea9b44,0x3ceb9b46,0x3cec9b48,0x3ced9b4a,0x3cee9b4c,0x3cef9b4e,0x3cf09b50,0x3cf19b52,
	0x3cf29b54,0x3cf39b56,0x3cf49b58,0x3cf59b5a,0x3cf69b5c,0x3cf79b5e,0x3cf89b60,0x3cf99b62,
	0x3cfa9b64,0x3cfb9b66,0x3cfc9b68,0x3cfd9b6a,0x3cfe9b6c,0x3cff9b6e,0x3d009b70,0x3d019b72,
	0x3d029b74,0x3d039b76,0x3d049b78,0x3d059b7a,0x3d069b7c,0x3d079b7e,0x3d089b80,0x3d099b82,
	0x3d0a9b84,0x3d0b9b86,0x3d0c9b88,0x3d0d9b8a,0x3d0e9b8c,0x3d0f9b8e,0x3d109b90,0x3d119b92,
	0x3d129b94,0x3d139b96,0x3d149b98,0x3d159b9a,0x3d169b9c,0x3d179b9e,0x3d189ba0,0x3d199ba2,
	0x3d1a9ba4,0x3d1b9ba6,0x3d1c9ba8,0x3d1d9baa,0x3d1e9bac,0x3d1f9bae,0x3d209bb0,0x3d219bb2,
	0x3d229bb4,0x3d239bb6,0x3d249bb8,0x3d259bba,0x3d269bbc,0x3d279bbe,0x3d289bc0,0x3d299bc2,
	0x3d2a9bc4,0x3d2b9bc6,0x3d2c9bc8,0x3d2d9bca,0x3d2e9bcc,0x3d2f9bce,0x3d309bd0,0x3d319bd2,
	0x3d329bd4,0x3d339bd6,0x3d349bd8,0x3d359bda,0x3d369bdc,0x3d379bde,0x3d389be0,0x3d399be2,
	0x3d3a9be4,0x3d3b9be6,0x3d3c9be8,0x3d3d9bea,0x3d3e9bec,0x3d3f9bee,0x3d409bf0,0x3d419bf2,
	0x3d429bf4,0x3d439bf6,0x3d449bf8,0x3d459bfa,0x3d469bfc,0x3d479bfe,0x3d489c00,0x3d499c02,
	0x3d4a9c04,0x3d4b9c06,0x3d4c9c08,0x3d4d9c0a,0x3d4e9c0c,0x3d4f9c0e,0x3d509c10,0x3d519c12,
	0x3d529c14,0x3d539c16,0x3d549c18,0x3d559c1a,0x3d569c1c,0x3d579c1e,0x3d589c20,0x3d599c22,
	0x3d5a9c24,0x3d5b9c26,0x3d5c9c28,0x3d5d9c2a,0x3d5e9c2c,0x3d5f9c2e,0x3d609c30,0x3d619c32,
	0x3d629c34,0x3d639c36,0x3d649c38,0x3d659c3a,0x3d669c3c,0x3d679c3e,0x3d689c40,0x3d699c42,
	0x3d6a9c44,0x3d6b9c46,0x3d6c9c48,0x3d6d9c4a,0x3d6e9c4c,0x3d6f9c4e,0x3d709c50,0x3d719c52,
	0x3d729c54,0x3d739c56,0x3d749c58,0x3d759c5a,0x3d769c5c,0x3d779c5e,0x3d789c60,0x3d799c62,
	0x3d7a9c64,0x3d7b9c66,0x3d7c9c68,0x3d7d9c6a,0x3d7e9c6c,0x3d7f9c6e,0x3d809c70,0x3d819c72,
	0x3d829c74,0x3d839c76,0x3d849c78,0x3d859c7a,0x3d869c7c,0x3d879c7e,0x3d889c80,0x3d899c82,
	0x3d8a9c84,0x3d8b9c86,0x3d8c9c88,0x3d8d9c8a,0x3d8e9c8c,0x3d8f9c8e,0x3d909c90,0x3d919c92,
	0x3d929c94,0x3d939c96,0x3d949c98,0x3d959c9a,0x3d969c9c,0x3d979c9e,0x3d989ca0,0x3d999ca2,
	0x3d9a9ca4,0x3d9b9ca6,0x3d9c9ca8,0x3d9d9caa,0x3d9e9cac,0x3d9f9cae,0x3da09cb0,0x3da19cb2,
	0x3da29cb4,0x3da39cb6,0x3da49cb8,0x3da59cba,0x3da69cbc,0x3da79cbe,0x3da89cc0,0x3da99cc2,
	0x3daa9cc4,0x3dab9cc6,0x3dac9cc8,0x3dad9cca,0x3dae9ccc,0x3daf9cce,0x3db09cd0,0x3db19cd2,
	0x3db29cd4,0x3db39cd6,0x3db49cd8,0x3db59cda,0x3db69cdc,0x3db79cde,0x3db89ce0,0x3db99ce2,
	0x3dba9ce4,0x3dbb9ce6,0x3dbc9ce8,0x3dbd9cea,0x3dbe9cec,0x3dbf9cee,0x3dc09cf0,0x3dc19cf2,
	0x3dc29cf4,0x3dc39cf6,0x3dc49cf8,0x3dc59cfa,0x3dc69cfc,0x3dc79cfe,0x3dc89d00,0x3dc99d02,
	0x3dca9d04,0x3dcb9d06,0x3dcc9d08,0x3dcd9d0a,0x3dce9d0c,0x3dcf9d0e,0x3dd09d10,0x3dd19d12,
	0x3dd29d14,0x3dd39d16,0x3dd49d18,0x3dd59d1a,0x3dd69d1c,0x3dd79d1e,0x3dd89d20,0x3dd99d22,
	0x3dda9d24,0x3ddb9d26,0x3ddc9d28,0x3ddd9d2a,0x3dde9d2c,0x3ddf9d2e,0x3de09d30,0x3de19d32,
	0x3de29d34,0x3de39d36,0x3de49d38,0x3de59d3a,0x3de69d3c,0x3de79d3e,0x3de89d40,0x3de99d42,
	0x3dea9d44,0x3deb9d46,0x3dec9d48,0x3ded9d4a,0x3dee9d4c,0x3def9d4e,0x3df09d50,0x3df19d52,
	0x3df29d54,0x3df39d56,0x3df49d58,0x3df59d5a,0x3df69d5c,0x3df79d5e,0x3df89d60,0x3df99d62,
	0x3dfa9d64,0x3dfb9d66,0x3dfc9d68,0x3dfd9d6a,0x3dfe9d6c,0x3dff9d6e,0x3e009d70,0x3e019d72,
	0x3e029d74,0x3e039d76,0x3e049d78,0x3e059d7a,0x3e069d7c,0x3e079d7e,0x3e089d80,0x3e099d82,
	0x3e0a9d84,0x3e0b9d86,0x3e0c9d88,0x3e0d9d8a,0x3e0e9d8c,0x3e0f9d8e,0x3e109d90,0x3e119d92,
	0x3e129d94,0x3e139d96,0x3e149d98,0x3e159d9a,0x3e169d9c,0x3e179d9e,0x3e189da0,0x3e199da2,
	0x3e1a9da4,0x3e1b9da6,0x3e1c9da8,0x3e1d9daa,0x3e1e9dac,0x3e1f9dae,0x3e209db0,0x3e219db2,
	0x3e229db4,0x3e239db6,0x3e249db8,0x3e259dba,0x3e269dbc,0x3e279dbe,0x3e289dc0,0x3e299dc2,
	0x3e2a9dc4,0x3e2b9dc6,0x3e2c9dc8,0x3e2d9dca,0x3e2e9dcc,0x3e2f9dce,0x3e309dd0,0x3e319dd2,
	0x3e329dd4,0x3e339dd6,0x3e349dd8,0x3e359dda,0x3e369ddc,0x3e379dde,0x3e389de0,0x3e399de2,
	0x3e3a9de4,0x3e3b9de6,0x3e3c9de8,0x3e3d9dea,0x3e3e9dec,0x3e3f9dee,0x3e409df0,0x3e419df2,
	0x3e429df4,0x3e439df6,0x3e449df8,0x3e459dfa,0x3e469dfc,0x3e479dfe,0x3e489e00,0x3e499e02,
	0x3e4a9e04,0x3e4b9e06,0x3e4c9e08,0x3e4d9e0a,0x3e4e9e0c,0x3e4f9e0e,0x3e509e10,0x3e519e12,
	0x3e529e14,0x3e539e16,0x3e549e18,0x3e559e1a,0x3e569e1c,0x3e579e1e,0x3e589e20,0x3e599e22,
	0x3e5a9e24,0x3e5b9e26,0x3e5c9e28,0x3e5d9e2a,0x3e5e9e2c,0x3e5f9e2e,0x3e609e30,0x3e619e32,
	0x3e629e34,0x3e639e36,0x3e649e38,0x3e659e3a,0x3e669e3c,0x3e679e3e,0x3e689e40,0x3e699e42,
	0x3e6a9e44,0x3e6b9e46,0x3e6c9e48,0x3e6d9e4a,0x3e6e9e4c,0x3e6f9e4e,0x3e709e50,0x3e719e52,
	0x3e729e54,0x3e739e56,0x3e749e58,0x3e759e5a,0x3e769e5c,0x3e779e5e,0x3e789e60,0x3e799e62,
	0x3e7a9e64,0x3e7b9e66,0x3e7c9e68,0x3e7d9e6a,0x3e7e9e6c,0x3e7f9e6e,0x3e809e70,0x3e819e72,
	0x3e829e74,0x3e839e76,0x3e849e78,0x3e859e7a,0x3e869e7c,0x3e879e7e,0x3e889e80,0x3e899e82,
	0x3e8a9e84,0x3e8b9e86,0x3e8c9e88,0x3e8d9e8a,0x3e8e9e8c,0x3e8f9e8e,0x3e909e90,0x3e919e92,
	0x3e929e94,0x3e939e96,0x3e949e98,0x3e959e9a,0x3e969e9c,0x3e979e9e,0x3e989ea0,0x3e999ea2,
	0x3e9a9ea4,0x3e9b9ea6,0x3e9c9ea8,0x3e9d9eaa,0x3e9e9eac,0x3e9f9eae,0x3ea09eb0,0x3ea19eb2,
	0x3ea29eb4,0x3ea39eb6,0x3ea49eb8,0x3ea59eba,0x3ea69ebc,0x3ea79ebe,0x3ea89ec0,0x3ea99ec2,
	0x3eaa9ec4,0x3eab9ec6,0x3eac9ec8,0x3ead9eca,0x3eae9ecc,0x3eaf9ece,0x3eb09ed0,0x3eb19ed2,
	0x3eb29ed4,0x3eb39ed6,0x3eb49ed8,0x3eb59eda,0x3eb69edc,0x3eb79ede,0x3eb89ee0,0x3eb99ee2,
	0x3eba9ee4,0x3ebb9ee6,0x3ebc9ee8,0x3ebd9eea,0x3ebe9eec,0x3ebf9eee,0x3ec09ef0,0x3ec19ef2,
	0x3ec29ef4,0x3ec39ef6,0x3ec49ef8,0x3ec59efa,0x3ec69efc,0x3ec79efe,0x3ec89f00,0x3ec99f02,
	0x3eca9f04,0x3ecb9f06,0x3ecc9f08,0x3ecd9f0a,0x3ece9f0c,0x3ecf9f0e,0x3ed09f10,0x3ed19f12,
	0x3ed29f14,0x3ed39f16,0x3ed49f18,0x3ed59f1a,0x3ed69f1c,0x3ed79f1e,0x3ed89f20,0x3ed99f22,
	0x3eda9f24,0x3edb9f26,0x3edc9f28,0x3edd9f2a,0x3ede9f2c,0x3edf9f2e,0x3ee09f30,0x3ee19f32,
	0x3ee29f34,0x3ee39f36,0x3ee49f38,0x3ee59f3a,0x3ee69f3c,0x3ee79f3e,0x3ee89f40,0x3ee99f42,
	0x3eea9f44,0x3eeb9f46,0x3eec9f48,0x3eed9f4a,0x3eee9f4c,0x3eef9f4e,0x3ef09f50,0x3ef19f52,
	0x3ef29f54,0x3ef39f56,0x3ef49f58,0x3ef59f5a,0x3ef69f5c,0x3ef79f5e,0x3ef89f60,0x3ef99f62,
	0x3efa9f64,0x3efb9f66,0x3efc9f68,0x3efd9f6a,0x3efe9f6c,0x3eff9f6e,0x3f009f70,0x3f019f72,
	0x3f029f74,0x3f039f76,0x3f049f78,0x3f059f7a,0x3f069f7c,0x3f079f7e,0x3f089f80,0x3f099f82,
	0x3f0a9f84,0x3f0b9f86,0x3f0c9f88,0x3f0d9f8a,0x3f0e9f8c,0x3f0f9f8e,0x3f109f90,0x3f119f92,
	0x3f129f94,0x3f139f96,0x3f149f98,0x3f159f9a,0x3f169f9c,0x3f179f9e,0x3f189fa0,0x3f199fa2,
	0x3f1a9fa4,0x3f1b9fa6,0x3f1c9fa8,0x3f1d9faa,0x3f1e9fac,0x3f1f9fae,0x3f209fb0,0x3f219fb2,
	0x3f229fb4,0x3f239fb6,0x3f249fb8,0x3f259fba,0x3f269fbc,0x3f279fbe,0x3f289fc0,0x3f299fc2,
	0x3f2a9fc4,0x3f2b9fc6,0x3f2c9fc8,0x3f2d9fca,0x3f2e9fcc,0x3f2f9fce,0x3f309fd0,0x3f319fd2,
	0x3f329fd4,0x3f339fd6,0x3f349fd8,0x3f359fda,0x3f369fdc,0x3f379fde,0x3f389fe0,0x3f399fe2,
	0x3f3a9fe4,0x3f3b9fe6,0x3f3c9fe8,0x3f3d9fea,0x3f3e9fec,0x3f3f9fee,0x3f409ff0,0x3f419ff2,
	0x3f429ff4,0x3f439ff6,0x3f449ff8,0x3f459ffa,0x3f469ffc,0x3f479ffe,0x3f48a000,0x3f49a002,
	0x3f4aa004,0x3f4ba006,0x3f4ca008,0x3f4da00a,0x3f4ea00c,0x3f4fa00e,0x3f50a010,0x3f51a012,
	0x3f52a014,0x3f53a016,0x3f54a018,0x3f55a01a,0x3f56a01c,0x3f57a01e,0x3f58a020,0x3f59a022,
	0x3f5aa024,0x3f5ba026,0x3f5ca028,0x3f5da02a,0x3f5ea02c,0x3f5fa02e,0x3f60a030,0x3f61a032,
	0x3f62a034,0x3f63a036,0x3f64a038,0x3f65a03a,0x3f66a03c,0x3f67a03e,0x3f68a040,0x3f69a042,
	0x3f6aa044,0x3f6ba046,0x3f6ca048,0x3f6da04a,0x3f6ea04c,0x3f6fa04e,0x3f70a050,0x3f71a052,
	0x3f72a054,0x3f73a056,0x3f74a058,0x3f75a05a,0x3f76a05c,0x3f77a05e,0x3f78a060,0x3f79a062,
	0x3f7aa064,0x3f7ba066,0x3f7ca068,0x3f7da06a,0x3f7ea06c,0x3f7fa06e,0x3f80a070,0x3f81a072,
	0x3f82a074,0x3f83a076,0x3f84a078,0x3f85a07a,0x3f86a07c,0x3f87a07e,0x3f88a080,0x3f89a082,
	0x3f8aa084,0x3f8ba086,0x3f8ca088,0x3f8da08a,0x3f8ea08c,0x3f8fa08e,0x3f90a090,0x3f91a092,
	0x3f92a094,0x3f93a096,0x3f94a098,0x3f95a09a,0x3f96a09c,0x3f97a09e,0x3f98a0a0,0x3f99a0a2,
	0x3f9aa0a4,0x3f9ba0a6,0x3f9ca0a8,0x3f9da0aa,0x3f9ea0ac,0x3f9fa0ae,0x3fa0a0b0,0x3fa1a0b2,
	0x3fa2a0b4,0x3fa3a0b6,0x3fa4a0b8,0x3fa5a0ba,0x3fa6a0bc,0x3fa7a0be,0x3fa8a0c0,0x3fa9a0c2,
	0x3faaa0c4,0x3faba0c6,0x3faca0c8,0x3fada0ca,0x3faea0cc,0x3fafa0ce,0x3fb0a0d0,0x3fb1a0d2,
	0x3fb2a0d4,0x3fb3a0d6,0x3fb4a0d8,0x3fb5a0da,0x3fb6a0dc,0x3fb7a0de,0x3fb8a0e0,0x3fb9a0e2,
	0x3fbaa0e4,0x3fbba0e6,0x3fbca0e8,0x3fbda0ea,0x3fbea0ec,0x3fbfa0ee,0x3fc0a0f0,0x3fc1a0f2,
	0x3fc2a0f4,0x3fc3a0f6,0x3fc4a0f8,0x3fc5a0fa,0x3fc6a0fc,0x3fc7a0fe,0x3fc8a100,0x3fc9a102,
	0x3fcaa104,0x3fcba106,0x3fcca108,0x3fcda10a,0x3fcea10c,0x3fcfa10e,0x3fd0a110,0x3fd1a112,
	0x3fd2a114,0x3fd3a116,0x3fd4a118,0x3fd5a11a,0x3fd6a11c,0x3fd7a11e,0x3fd8a120,0x3fd9a122,
	0x3fdaa124,0x3fdba126,0x3fdca128,0x3fdda12a,0x3fdea12c,0x3fdfa12e,0x3fe0a130,0x3fe1a132,
	0x3fe2a134,0x3fe3a136,0x3fe4a138,0x3fe5a13a,0x3fe6a13c,0x3fe7a13e,0x3fe8a140,0x3fe9a142,
	0x3feaa144,0x3feba146,0x3feca148,0x3feda14a,0x3feea14c,0x3fefa14e,0x3ff0a150,0x3ff1a152,
	0x3ff2a154,0x3ff3a156,0x3ff4a158,0x3ff5a15a,0x3ff6a15c,0x3ff7a15e,0x3ff8a160,0x3ff9a162,
	0x3ffaa164,0x3ffba166,0x3ffca168,0x3ffda16a,0x3ffea16c,0x3fffa16e,0x4000a170,0x4001a172,
	0x4002a174,0x4003a176,0x4004a178,0x4005a17a,0x4006a17c,0x4007a17e,0x4008a180,0x4009a182,
	0x400aa184,0x400ba186,0x400ca188,0x400da18a,0x400ea18c,0x400fa18e,0x4010a190,0x4011a192,
	0x4012a194,0x4013a196,0x4014a198,0x4015a19a,0x4016a19c,0x4017a19e,0x4018a1a0,0x4019a1a2,
	0x401aa1a4,0x401ba1a6,0x401ca1a8,0x401da1aa,0x401ea1ac,0x401fa1ae,0x4020a1b0,0x4021a1b2,
	0x4022a1b4,0x4023a1b6,0x4024a1b8,0x4025a1ba,0x4026a1bc,0x4027a1be,0x4028a1c0,0x4029a1c2,
	0x402aa1c4,0x402ba1c6,0x402ca1c8,0x402da1ca,0x402ea1cc,0x402fa1ce,0x4030a1d0,0x4031a1d2,
	0x4032a1d4,0x4033a1d6,0x4034a1d8,0x4035a1da,0x4036a1dc,0x4037a1de,0x4038a1e0,0x4039a1e2,
	0x403aa1e4,0x403ba1e6,0x403ca1e8,0x403da1ea,0x403ea1ec,0x403fa1ee,0x4040a1f0,0x4041a1f2,
	0x4042a1f4,0x4043a1f6,0x4044a1f8,0x4045a1fa,0x4046a1fc,0x4047a1fe,0x4048a200,0x4049a202,
	0x404aa204,0x404ba206,0x404ca208,0x404da20a,0x404ea20c,0x404fa20e,0x4050a210,0x4051a212,
	0x4052a214,0x4053a216,0x4054a218,0x4055a21a,0x4056a21c,0x4057a21e,0x4058a220,0x4059a222,
	0x405aa224,0x405ba226,0x405ca228,0x405da22a,0x405ea22c,0x405fa22e,0x4060a230,0x4061a232,
	0x4062a234,0x4063a236,0x4064a238,0x4065a23a,0x4066a23c,0x4067a23e,0x4068a240,0x4069a242,
	0x406aa244,0x406ba246,0x406ca248,0x406da24a,0x406ea24c,0x406fa24e,0x4070a250,0x4071a252,
	0x4072a254,0x4073a256,0x4074a258,0x4075a25a,0x4076a25c,0x4077a25e,0x4078a260,0x4079a262,
	0x407aa264,0x407ba266,0x407ca268,0x407da26a,0x407ea26c,0x407fa26e,0x4080a270,0x4081a272,
	0x4082a274,0x4083a276,0x4084a278,0x4085a27a,0x4086a27c,0x4087a27e,0x4088a280,0x4089a282,
	0x408aa284,0x408ba286,0x408ca288,0x408da28a,0x408ea28c,0x408fa28e,0x4090a290,0x4091a292,
	0x4092a294,0x4093a296,0x4094a298,0x4095a29a,0x4096a29c,0x4097a29e,0x4098a2a0,0x4099a2a2,
	0x409aa2a4,0x409ba2a6,0x409ca2a8,0x409da2aa,0x409ea2ac,0x409fa2ae,0x40a0a2b0,0x40a1a2b2,
	0x40a2a2b4,0x40a3a2b6,0x40a4a2b8,0x40a5a2ba,0x40a6a2bc,0x40a7a2be,0x40a8a2c0,0x40a9a2c2,
	0x40aaa2c4,0x40aba2c6,0x40aca2c8,0x40ada2ca,0x40aea2cc,0x40afa2ce,0x40b0a2d0,0x40b1a2d2,
	0x40b2a2d4,0x40b3a2d6,0x40b4a2d8,0x40b5a2da,0x40b6a2dc,0x40b7a2de,0x40b8a2e0,0x40b9a2e2,
	0x40baa2e4,0x40bba2e6,0x40bca2e8,0x40bda2ea,0x40bea2ec,0x40bfa2ee,0x40c0a2f0,0x40c1a2f2,
	0x40c2a2f4,0x40c3a2f6,0x40c4a2f8,0x40c5a2fa,0x40c6a2fc,0x40c7a2fe,0x40c8a300,0x40c9a302,
	0x40caa304,0x40cba306,0x40cca308,0x40cda30a,0x40cea30c,0x40cfa30e,0x40d0a310,0x40d1a312,
	0x40d2a314,0x40d3a316,0x40d4a318,0x40d5a31a,0x40d6a31c,0x40d7a31e,0x40d8a320,0x40d9a322,
	0x40daa324,0x40dba326,0x40dca328,0x40dda32a,0x40dea32c,0x40dfa32e,0x40e0a330,0x40e1a332,
	0x40e2a334,0x40e3a336,0x40e4a338,0x40e5a33a,0x40e6a33c,0x40e7a33e,0x40e8a340,0x40e9a342,
	0x40eaa344,0x40eba346,0x40eca348,0x40eda34a,0x40eea34c,0x40efa34e,0x40f0a350,0x40f1a352,
	0x40f2a354,0x40f3a356,0x40f4a358,0x40f5a35a,0x40f6a35c,0x40f7a35e,0x40f8a360,0x40f9a362,
	0x40faa364,0x40fba366,0x40fca368,0x40fda36a,0x40fea36c,0x40ffa36e,0x4100a370,0x4101a372,
	0x4102a374,0x4103a376,0x4104a378,0x4105a37a,0x4106a37c,0x4107a37e,0x4108a380,0x4109a382,
	0x410aa384,0x410ba386,0x410ca388,0x410da38a,0x410ea38c,0x410fa38e,0x4110a390,0x4111a392,
	0x4112a394,0x4113a396,0x4114a398,0x4115a39a,0x4116a39c,0x4117a39e,0x4118a3a0,0x4119a3a2,
	0x411aa3a4,0x411ba3a6,0x411ca3a8,0x411da3aa,0x411ea3ac,0x411fa3ae,0x4120a3b0,0x4121a3b2,
	0x4122a3b4,0x4123a3b6,0x4124a3b8,0x4125a3ba,0x4126a3bc,0x4127a3be,0x4128a3c0,0x4129a3c2,
	0x412aa3c4,0x412ba3c6,0x412ca3c8,0x412da3ca,0x412ea3cc,0x412fa3ce,0x4130a3d0,0x4131a3d2,
	0x4132a3d4,0x4133a3d6,0x4134a3d8,0x4135a3da,0x4136a3dc,0x4137a3de,0x4138a3e0,0x4139a3e2,
	0x413aa3e4,0x413ba3e6,0x413ca3e8,0x413da3ea,0x413ea3ec,0x413fa3ee,0x4140a3f0,0x4141a3f2,
	0x4142a3f4,0x4143a3f6,0x4144a3f8,0x4145a3fa,0x4146a3fc,0x4147a3fe,0x4148a400,0x4149a402,
	0x414aa404,0x414ba406,0x414ca408,0x414da40a,0x414ea40c,0x414fa40e,0x4150a410,0x4151a412,
	0x4152a414,0x4153a416,0x4154a418,0x4155a41a,0x4156a41c,0x4157a41e,0x4158a420,0x4159a422,
	0x415aa424,0x415ba426,0x415ca428,0x415da42a,0x415ea42c,0x415fa42e,0x4160a430,0x4161a432,
	0x4162a434,0x4163a436,0x4164a438,0x4165a43a,0x4166a43c,0x4167a43e,0x4168a440,0x4169a442,
	0x416aa444,0x416ba446,0x416ca448,0x416da44a,0x416ea44c,0x416fa44e,0x4170a450,0x4171a452,
	0x4172a454,0x4173a456,0x4174a458,0x4175a45a,0x4176a45c,0x4177a45e,0x4178a460,0x4179a462,
	0x417aa464,0x417ba466,0x417ca468,0x417da46a,0x417ea46c,0x417fa46e,0x4180a470,0x4181a472,
	0x4182a474,0x4183a476,0x4184a478,0x4185a47a,0x4186a47c,0x4187a47e,0x4188a480,0x4189a482,
	0x418aa484,0x418ba486,0x418ca488,0x418da48a,0x418ea48c,0x418fa48e,0x4190a490,0x4191a492,
	0x4192a494,0x4193a496,0x4194a498,0x4195a49a,0x4196a49c,0x4197a49e,0x4198a4a0,0x4199a4a2,
	0x419aa4a4,0x419ba4a6,0x419ca4a8,0x419da4aa,0x419ea4ac,0x419fa4ae,0x41a0a4b0,0x41a1a4b2,
	0x41a2a4b4,0x41a3a4b6,0x41a4a4b8,0x41a5a4ba,0x41a6a4bc,0x41a7a4be,0x41a8a4c0,0x41a9a4c2,
	0x41aaa4c4,0x41aba4c6,0x41aca4c8,0x41ada4ca,0x41aea4cc,0x41afa4ce,0x41b0a4d0,0x41b1a4d2,
	0x41b2a4d4,0x41b3a4d6,0x41b4a4d8,0x41b5a4da,0x41b6a4dc,0x41b7a4de,0x41b8a4e0,0x41b9a4e2,
	0x41baa4e4,0x41bba4e6,0x41bca4e8,0x41bda4ea,0x41bea4ec,0x41bfa4ee,0x41c0a4f0,0x41c1a4f2,
	0x41c2a4f4,0x41c3a4f6,0x41c4a4f8,0x41c5a4fa,0x41c6a4fc,0x41c7a4fe,0x41c8a500,0x41c9a502,
	0x41caa504,0x41cba506,0x41cca508,0x41cda50a,0x41cea50c,0x41cfa50e,0x41d0a510,0x41d1a512,
	0x41d2a514,0x41d3a516,0x41d4a518,0x41d5a51a,0x41d6a51c,0x41d7a51e,0x41d8a520,0x41d9a522,
	0x41daa524,0x41dba526,0x41dca528,0x41dda52a,0x41dea52c,0x41dfa52e,0x41e0a530,0x41e1a532,
	0x41e2a534,0x41e3a536,0x41e4a538,0x41e5a53a,0x41e6a53c,0x41e7a53e,0x41e8a540,0x41e9a542,
	0x41eaa544,0x41eba546,0x41eca548,0x41eda54a,0x41eea54c,0x41efa54e,0x41f0a550,0x41f1a552,
	0x41f2a554,0x41f3a556,0x41f4a558,0x41f5a55a,0x41f6a55c,0x41f7a55e,0x41f8a560,0x41f9a562,
	0x41faa564,0x41fba566,0x41fca568,0x41fda56a,0x41fea56c,0x41ffa56e,0x4200a570,0x4201a572,
	0x4202a574,0x4203a576,0x4204a578,0x4205a57a,0x4206a57c,0x4207a57e,0x4208a580,0x4209a582,
	0x420aa584,0x420ba586,0x420ca588,0x420da58a,0x420ea58c,0x420fa58e,0x4210a590,0x4211a592,
	0x4212a594,0x4213a596,0x4214a598,0x4215a59a,0x4216a59c,0x4217a59e,0x4218a5a0,0x4219a5a2,
	0x421aa5a4,0x421ba5a6,0x421ca5a8,0x421da5aa,0x421ea5ac,0x421fa5ae,0x4220a5b0,0x4221a5b2,
	0x4222a5b4,0x4223a5b6,0x4224a5b8,0x4225a5ba,0x4226a5bc,0x4227a5be,0x4228a5c0,0x4229a5c2,
	0x422aa5c4,0x422ba5c6,0x422ca5c8,0x422da5ca,0x422ea5cc,0x422fa5ce,0x4230a5d0,0x4231a5d2,
	0x4232a5d4,0x4233a5d6,0x4234a5d8,0x4235a5da,0x4236a5dc,0x4237a5de,0x4238a5e0,0x4239a5e2,
	0x423aa5e4,0x423ba5e6,0x423ca5e8,0x423da5ea,0x423ea5ec,0x423fa5ee,0x4240a5f0,0x4241a5f2,
	0x4242a5f4,0x4243a5f6,0x4244a5f8,0x4245a5fa,0x4246a5fc,0x4247a5fe,0x4248a600,0x4249a602,
	0x424aa604,0x424ba606,0x424ca608,0x424da60a,0x424ea60c,0x424fa60e,0x4250a610,0x4251a612,
	0x4252a614,0x4253a616,0x4254a618,0x4255a61a,0x4256a61c,0x4257a61e,0x4258a620,0x4259a622,
	0x425aa624,0x425ba626,0x425ca628,0x425da62a,0x425ea62c,0x425fa62e,0x4260a630,0x4261a632,
	0x4262a634,0x4263a636,0x4264a638,0x4265a63a,0x4266a63c,0x4267a63e,0x4268a640,0x4269a642,
	0x426aa644,0x426ba646,0x426ca648,0x426da64a,0x426ea64c,0x426fa64e,0x4270a650,0x4271a652,
	0x4272a654,0x4273a656,0x4274a658,0x4275a65a,0x4276a65c,0x4277a65e,0x4278a660,0x4279a662,
	0x427aa664,0x427ba666,0x427ca668,0x427da66a,0x427ea66c,0x427fa66e,0x4280a670,0x4281a672,
	0x4282a674,0x4283a676,0x4284a678,0x4285a67a,0x4286a67c,0x4287a67e,0x4288a680,0x4289a682,
	0x428aa684,0x428ba686,0x428ca688,0x428da68a,0x428ea68c,0x428fa68e,0x4290a690,0x4291a692,
	0x4292a694,0x4293a696,0x4294a698,0x4295a69a,0x4296a69c,0x4297a69e,0x4298a6a0,0x4299a6a2,
	0x429aa6a4,0x429ba6a6,0x429ca6a8,0x429da6aa,0x429ea6ac,0x429fa6ae,0x42a0a6b0,0x42a1a6b2,
	0x42a2a6b4,0x42a3a6b6,0x42a4a6b8,0x42a5a6ba,0x42a6a6bc,0x42a7a6be,0x42a8a6c0,0x42a9a6c2,
	0x42aaa6c4,0x42aba6c6,0x42aca6c8,0x42ada6ca,0x42aea6cc,0x42afa6ce,0x42b0a6d0,0x42b1a6d2,
	0x42b2a6d4,0x42b3a6d6,0x42b4a6d8,0x42b5a6da,0x42b6a6dc,0x42b7a6de,0x42b8a6e0,0x42b9a6e2,
	0x42baa6e4,0x42bba6e6,0x42bca6e8,0x42bda6ea,0x42bea6ec,0x42bfa6ee,0x42c0a6f0,0x42c1a6f2,
	0x42c2a6f4,0x42c3a6f6,0x42c4a6f8,0x42c5a6fa,0x42c6a6fc,0x42c7a6fe,0x42c8a700,0x42c9a702,
	0x42caa704,0x42cba706,0x42cca708,0x42cda70a,0x42cea70c,0x42cfa70e,0x42d0a710,0x42d1a712,
	0x42d2a714,0x42d3a716,0x42d4a718,0x42d5a71a,0x42d6a71c,0x42d7a71e,0x42d8a720,0x42d9a722,
	0x42daa724,0x42dba726,0x42dca728,0x42dda72a,0x42dea72c,0x42dfa72e,0x42e0a730,0x42e1a732,
	0x42e2a734,0x42e3a736,0x42e4a738,0x42e5a73a,0x42e6a73c,0x42e7a73e,0x42e8a740,0x42e9a742,
	0x42eaa744,0x42eba746,0x42eca748,0x42eda74a,0x42eea74c,0x42efa74e,0x42f0a750,0x42f1a752,
	0x42f2a754,0x42f3a756,0x42f4a758,0x42f5a75a,0x42f6a75c,0x42f7a75e,0x42f8a760,0x42f9a762,
	0x42faa764,0x42fba766,0x42fca768,0x42fda76a,0x42fea76c,0x42ffa76e,0x4300a770,0x4301a772,
	0x4302a774,0x4303a776,0x4304a778,0x4305a77a,0x4306a77c,0x4307a77e,0x4308a780,0x4309a782,
	0x430aa784,0x430ba786,0x430ca788,0x430da78a,0x430ea78c,0x430fa78e,0x4310a790,0x4311a792,
	0x4312a794,0x4313a796,0x4314a798,0x4315a79a,0x4316a79c,0x4317a79e,0x4318a7a0,0x4319a7a2,
	0x431aa7a4,0x431ba7a6,0x431ca7a8,0x431da7aa,0x431ea7ac,0x431fa7ae,0x4320a7b0,0x4321a7b2,
	0x4322a7b4,0x4323a7b6,0x4324a7b8,0x4325a7ba,0x4326a7bc,0x4327a7be,0x4328a7c0,0x4329a7c2,
	0x432aa7c4,0x432ba7c6,0x432ca7c8,0x432da7ca,0x432ea7cc,0x432fa7ce,0x4330a7d0,0x4331a7d2,
	0x4332a7d4,0x4333a7d6,0x4334a7d8,0x4335a7da,0x4336a7dc,0x4337a7de,0x4338a7e0,0x4339a7e2,
	0x433aa7e4,0x433ba7e6,0x433ca7e8,0x433da7ea,0x433ea7ec,0x433fa7ee,0x4340a7f0,0x4341a7f2,
	0x4342a7f4,0x4343a7f6,0x4344a7f8,0x4345a7fa,0x4346a7fc,0x4347a7fe,0x4348a800,0x4349a802,
	0x434aa804,0x434ba806,0x434ca808,0x434da80a,0x434ea80c,0x434fa80e,0x4350a810,0x4351a812,
	0x4352a814,0x4353a816,0x4354a818,0x4355a81a,0x4356a81c,0x4357a81e,0x4358a820,0x4359a822,
	0x435aa824,0x435ba826,0x435ca828,0x435da82a,0x435ea82c,0x435fa82e,0x4360a830,0x4361a832,
	0x4362a834,0x4363a836,0x4364a838,0x4365a83a,0x4366a83c,0x4367a83e,0x4368a840,0x4369a842,
	0x436aa844,0x436ba846,0x436ca848,0x436da84a,0x436ea84c,0x436fa84e,0x4370a850,0x4371a852,
	0x4372a854,0x4373a856,0x4374a858,0x4375a85a,0x4376a85c,0x4377a85e,0x4378a860,0x4379a862,
	0x437aa864,0x437ba866,0x437ca868,0x437da86a,0x437ea86c,0x437fa86e,0x4380a870,0x4381a872,
	0x4382a874,0x4383a876,0x4384a878,0x4385a87a,0x4386a87c,0x4387a87e,0x4388a880,0x4389a882,
	0x438aa884,0x438ba886,0x438ca888,0x438da88a,0x438ea88c,0x438fa88e,0x4390a890,0x4391a892,
	0x4392a894,0x4393a896,0x4394a898,0x4395a89a,0x4396a89c,0x4397a89e,0x4398a8a0,0x4399a8a2,
	0x439aa8a4,0x439ba8a6,0x439ca8a8,0x439da8aa,0x439ea8ac,0x439fa8ae,0x43a0a8b0,0x43a1a8b2,
	0x43a2a8b4,0x43a3a8b6,0x43a4a8b8,0x43a5a8ba,0x43a6a8bc,0x43a7a8be,0x43a8a8c0,0x43a9a8c2,
	0x43aaa8c4,0x43aba8c6,0x43aca8c8,0x43ada8ca,0x43aea8cc,0x43afa8ce,0x43b0a8d0,0x43b1a8d2,
	0x43b2a8d4,0x43b3a8d6,0x43b4a8d8,0x43b5a8da,0x43b6a8dc,0x43b7a8de,0x43b8a8e0,0x43b9a8e2,
	0x43baa8e4,0x43bba8e6,0x43bca8e8,0x43bda8ea,0x43bea8ec,0x43bfa8ee,0x43c0a8f0,0x43c1a8f2,
	0x43c2a8f4,0x43c3a8f6,0x43c4a8f8,0x43c5a8fa,0x43c6a8fc,0x43c7a8fe,0x43c8a900,0x43c9a902,
	0x43caa904,0x43cba906,0x43cca908,0x43cda90a,0x43cea90c,0x43cfa90e,0x43d0a910,0x43d1a912,
	0x43d2a914,0x43d3a916,0x43d4a918,0x43d5a91a,0x43d6a91c,0x43d7a91e,0x43d8a920,0x43d9a922,
	0x43daa924,0x43dba926,0x43dca928,0x43dda92a,0x43dea92c,0x43dfa92e,0x43e0a930,0x43e1a932,
	0x43e2a934,0x43e3a936,0x43e4a938,0x43e5a93a,0x43e6a93c,0x43e7a93e,0x43e8a940,0x43e9a942,
	0x43eaa944,0x43eba946,0x43eca948,0x43eda94a,0x43eea94c,0x43efa94e,0x43f0a950,0x43f1a952,
	0x43f2a954,0x43f3a956,0x43f4a958,0x43f5a95a,0x43f6a95c,0x43f7a95e,0x43f8a960,0x43f9a962,
	0x43faa964,0x43fba966,0x43fca968,0x43fda96a,0x43fea96c,0x43ffa96e,0x4400a970,0x4401a972,
	0x4402a974,0x4403a976,0x4404a978,0x4405a97a,0x4406a97c,0x4407a97e,0x4408a980,0x4409a982,
	0x440aa984,0x440ba986,0x440ca988,0x440da98a,0x440ea98c,0x440fa98e,0x4410a990,0x4411a992,
	0x4412a994,0x4413a996,0x4414a998,0x4415a99a,0x4416a99c,0x4417a99e,0x4418a9a0,0x4419a9a2,
	0x441aa9a4,0x441ba9a6,0x441ca9a8,0x441da9aa,0x441ea9ac,0x441fa9ae,0x4420a9b0,0x4421a9b2,
	0x4422a9b4,0x4423a9b6,0x4424a9b8,0x4425a9ba,0x4426a9bc,0x4427a9be,0x4428a9c0,0x4429a9c2,
	0x442aa9c4,0x442ba9c6,0x442ca9c8,0x442da9ca,0x442ea9cc,0x442fa9ce,0x4430a9d0,0x4431a9d2,
	0x4432a9d4,0x4433a9d6,0x4434a9d8,0x4435a9da,0x4436a9dc,0x4437a9de,0x4438a9e0,0x4439a9e2,
	0x443aa9e4,0x443ba9e6,0x443ca9e8,0x443da9ea,0x443ea9ec,0x443fa9ee,0x4440a9f0,0x4441a9f2,
	0x4442a9f4,0x4443a9f6,0x4444a9f8,0x4445a9fa,0x4446a9fc,0x4447a9fe,0x4448aa00,0x4449aa02,
	0x444aaa04,0x444baa06,0x444caa08,0x444daa0a,0x444eaa0c,0x444faa0e,0x4450aa10,0x4451aa12,
	0x4452aa14,0x4453aa16,0x4454aa18,0x4455aa1a,0x4456aa1c,0x4457aa1e,0x4458aa20,0x4459aa22,
	0x445aaa24,0x445baa26,0x445caa28,0x445daa2a,0x445eaa2c,0x445faa2e,0x4460aa30,0x4461aa32,
	0x4462aa34,0x4463aa36,0x4464aa38,0x4465aa3a,0x4466aa3c,0x4467aa3e,0x4468aa40,0x4469aa42,
	0x446aaa44,0x446baa46,0x446caa48,0x446daa4a,0x446eaa4c,0x446faa4e,0x4470aa50,0x4471aa52,
	0x4472aa54,0x4473aa56,0x4474aa58,0x4475aa5a,0x4476aa5c,0x4477aa5e,0x4478aa60,0x4479aa62,
	0x447aaa64,0x447baa66,0x447caa68,0x447daa6a,0x447eaa6c,0x447faa6e,0x4480aa70,0x4481aa72,
	0x4482aa74,0x4483aa76,0x4484aa78,0x4485aa7a,0x4486aa7c,0x4487aa7e,0x4488aa80,0x4489aa82,
	0x448aaa84,0x448baa86,0x448caa88,0x448daa8a,0x448eaa8c,0x448faa8e,0x4490aa90,0x4491aa92,
	0x4492aa94,0x4493aa96,0x4494aa98,0x4495aa9a,0x4496aa9c,0x4497aa9e,0x4498aaa0,0x4499aaa2,
	0x449aaaa4,0x449baaa6,0x449caaa8,0x449daaaa,0x449eaaac,0x449faaae,0x44a0aab0,0x44a1aab2,
	0x44a2aab4,0x44a3aab6,0x44a4aab8,0x44a5aaba,0x44a6aabc,0x44a7aabe,0x44a8aac0,0x44a9aac2,
	0x44aaaac4,0x44abaac6,0x44acaac8,0x44adaaca,0x44aeaacc,0x44afaace,0x44b0aad0,0x44b1aad2,
	0x44b2aad4,0x44b3aad6,0x44b4aad8,0x44b5aada,0x44b6aadc,0x44b7aade,0x44b8aae0,0x44b9aae2,
	0x44baaae4,0x44bbaae6,0x44bcaae8,0x44bdaaea,0x44beaaec,0x44bfaaee,0x44c0aaf0,0x44c1aaf2,
	0x44c2aaf4,0x44c3aaf6,0x44c4aaf8,0x44c5aafa,0x44c6aafc,0x44c7aafe,0x44c8ab00,0x44c9ab02,
	0x44caab04,0x44cbab06,0x44ccab08,0x44cdab0a,0x44ceab0c,0x44cfab0e,0x44d0ab10,0x44d1ab12,
	0x44d2ab14,0x44d3ab16,0x44d4ab18,0x44d5ab1a,0x44d6ab1c,0x44d7ab1e,0x44d8ab20,0x44d9ab22,
	0x44daab24,0x44dbab26,0x44dcab28,0x44ddab2a,0x44deab2c,0x44dfab2e,0x44e0ab30,0x44e1ab32,
	0x44e2ab34,0x44e3ab36,0x44e4ab38,0x44e5ab3a,0x44e6ab3c,0x44e7ab3e,0x44e8ab40,0x44e9ab42,
	0x44eaab44,0x44ebab46,0x44ecab48,0x44edab4a,0x44eeab4c,0x44efab4e,0x44f0ab50,0x44f1ab52,
	0x44f2ab54,0x44f3ab56,0x44f4ab58,0x44f5ab5a,0x44f6ab5c,0x44f7ab5e,0x44f8ab60,0x44f9ab62,
	0x44faab64,0x44fbab66,0x44fcab68,0x44fdab6a,0x44feab6c,0x44ffab6e,0x4500ab70,0x4501ab72,
	0x4502ab74,0x4503ab76,0x4504ab78,0x4505ab7a,0x4506ab7c,0x4507ab7e,0x4508ab80,0x4509ab82,
	0x450aab84,0x450bab86,0x450cab88,0x450dab8a,0x450eab8c,0x450fab8e,0x4510ab90,0x4511ab92,
	0x4512ab94,0x4513ab96,0x4514ab98,0x4515ab9a,0x4516ab9c,0x4517ab9e,0x4518aba0,0x4519aba2,
	0x451aaba4,0x451baba6,0x451caba8,0x451dabaa,0x451eabac,0x451fabae,0x4520abb0,0x4521abb2,
	0x4522abb4,0x4523abb6,0x4524abb8,0x4525abba,0x4526abbc,0x4527abbe,0x4528abc0,0x4529abc2,
	0x452aabc4,0x452babc6,0x452cabc8,0x452dabca,0x452eabcc,0x452fabce,0x4530abd0,0x4531abd2,
	0x4532abd4,0x4533abd6,0x4534abd8,0x4535abda,0x4536abdc,0x4537abde,0x4538abe0,0x4539abe2,
	0x453aabe4,0x453babe6,0x453cabe8,0x453dabea,0x453eabec,0x453fabee,0x4540abf0,0x4541abf2,
	0x4542abf4,0x4543abf6,0x4544abf8,0x4545abfa,0x4546abfc,0x4547abfe,0x4548ac00,0x4549ac02,
	0x454aac04,0x454bac06,0x454cac08,0x454dac0a,0x454eac0c,0x454fac0e,0x4550ac10,0x4551ac12,
	0x4552ac14,0x4553ac16,0x4554ac18,0x4555ac1a,0x4556ac1c,0x4557ac1e,0x4558ac20,0x4559ac22,
	0x455aac24,0x455bac26,0x455cac28,0x455dac2a,0x455eac2c,0x455fac2e,0x4560ac30,0x4561ac32,
	0x4562ac34,0x4563ac36,0x4564ac38,0x4565ac3a,0x4566ac3c,0x4567ac3e,0x4568ac40,0x4569ac42,
	0x456aac44,0x456bac46,0x456cac48,0x456dac4a,0x456eac4c,0x456fac4e,0x4570ac50,0x4571ac52,
	0x4572ac54,0x4573ac56,0x4574ac58,0x4575ac5a,0x4576ac5c,0x4577ac5e,0x4578ac60,0x4579ac62,
	0x457aac64,0x457bac66,0x457cac68,0x457dac6a,0x457eac6c,0x457fac6e,0x4580ac70,0x4581ac72,
	0x4582ac74,0x4583ac76,0x4584ac78,0x4585ac7a,0x4586ac7c,0x4587ac7e,0x4588ac80,0x4589ac82,
	0x458aac84,0x458bac86,0x458cac88,0x458dac8a,0x458eac8c,0x458fac8e,0x4590ac90,0x4591ac92,
	0x4592ac94,0x4593ac96,0x4594ac98,0x4595ac9a,0x4596ac9c,0x4597ac9e,0x4598aca0,0x4599aca2,
	0x459aaca4,0x459baca6,0x459caca8,0x459dacaa,0x459eacac,0x459facae,0x45a0acb0,0x45a1acb2,
	0x45a2acb4,0x45a3acb6,0x45a4acb8,0x45a5acba,0x45a6acbc,0x45a7acbe,0x45a8acc0,0x45a9acc2,
	0x45aaacc4,0x45abacc6,0x45acacc8,0x45adacca,0x45aeaccc,0x45afacce,0x45b0acd0,0x45b1acd2,
	0x45b2acd4,0x45b3acd6,0x45b4acd8,0x45b5acda,0x45b6acdc,0x45b7acde,0x45b8ace0,0x45b9ace2,
	0x45baace4,0x45bbace6,0x45bcace8,0x45bdacea,0x45beacec,0x45bfacee,0x45c0acf0,0x45c1acf2,
	0x45c2acf4,0x45c3acf6,0x45c4acf8,0x45c5acfa,0x45c6acfc,0x45c7acfe,0x45c8ad00,0x45c9ad02,
	0x45caad04,0x45cbad06,0x45ccad08,0x45cdad0a,0x45cead0c,0x45cfad0e,0x45d0ad10,0x45d1ad12,
	0x45d2ad14,0x45d3ad16,0x45d4ad18,0x45d5ad1a,0x45d6ad1c,0x45d7ad1e,0x45d8ad20,0x45d9ad22,
	0x45daad24,0x45dbad26,0x45dcad28,0x45ddad2a,0x45dead2c,0x45dfad2e,0x45e0ad30,0x45e1ad32,
	0x45e2ad34,0x45e3ad36,0x45e4ad38,0x45e5ad3a,0x45e6ad3c,0x45e7ad3e,0x45e8ad40,0x45e9ad42,
	0x45eaad44,0x45ebad46,0x45ecad48,0x45edad4a,0x45eead4c,0x45efad4e,0x45f0ad50,0x45f1ad52,
	0x45f2ad54,0x45f3ad56,0x45f4ad58,0x45f5ad5a,0x45f6ad5c,0x45f7ad5e,0x45f8ad60,0x45f9ad62,
	0x45faad64,0x45fbad66,0x45fcad68,0x45fdad6a,0x45fead6c,0x45ffad6e,0x4600ad70,0x4601ad72,
	0x4602ad74,0x4603ad76,0x4604ad78,0x4605ad7a,0x4606ad7c,0x4607ad7e,0x4608ad80,0x4609ad82,
	0x460aad84,0x460bad86,0x460cad88,0x460dad8a,0x460ead8c,0x460fad8e,0x4610ad90,0x4611ad92,
	0x4612ad94,0x4613ad96,0x4614ad98,0x4615ad9a,0x4616ad9c,0x4617ad9e,0x4618ada0,0x4619ada2,
	0x461aada4,0x461bada6,0x461cada8,0x461dadaa,0x461eadac,0x461fadae,0x4620adb0,0x4621adb2,
	0x4622adb4,0x4623adb6,0x4624adb8,0x4625adba,0x4626adbc,0x4627adbe,0x4628adc0,0x4629adc2,
	0x462aadc4,0x462badc6,0x462cadc8,0x462dadca,0x462eadcc,0x462fadce,0x4630add0,0x4631add2,
	0x4632add4,0x4633add6,0x4634add8,0x4635adda,0x4636addc,0x4637adde,0x4638ade0,0x4639ade2,
	0x463aade4,0x463bade6,0x463cade8,0x463dadea,0x463eadec,0x463fadee,0x4640adf0,0x4641adf2,
	0x4642adf4,0x4643adf6,0x4644adf8,0x4645adfa,0x4646adfc,0x4647adfe,0x4648ae00,0x4649ae02,
	0x464aae04,0x464bae06,0x464cae08,0x464dae0a,0x464eae0c,0x464fae0e,0x4650ae10,0x4651ae12,
	0x4652ae14,0x4653ae16,0x4654ae18,0x4655ae1a,0x4656ae1c,0x4657ae1e,0x4658ae20,0x4659ae22,
	0x465aae24,0x465bae26,0x465cae28,0x465dae2a,0x465eae2c,0x465fae2e,0x4660ae30,0x4661ae32,
	0x4662ae34,0x4663ae36,0x4664ae38,0x4665ae3a,0x4666ae3c,0x4667ae3e,0x4668ae40,0x4669ae42,
	0x466aae44,0x466bae46,0x466cae48,0x466dae4a,0x466eae4c,0x466fae4e,0x4670ae50,0x4671ae52,
	0x4672ae54,0x4673ae56,0x4674ae58,0x4675ae5a,0x4676ae5c,0x4677ae5e,0x4678ae60,0x4679ae62,
	0x467aae64,0x467bae66,0x467cae68,0x467dae6a,0x467eae6c,0x467fae6e,0x4680ae70,0x4681ae72,
	0x4682ae74,0x4683ae76,0x4684ae78,0x4685ae7a,0x4686ae7c,0x4687ae7e,0x4688ae80,0x4689ae82,
	0x468aae84,0x468bae86,0x468cae88,0x468dae8a,0x468eae8c,0x468fae8e,0x4690ae90,0x4691ae92,
	0x4692ae94,0x4693ae96,0x4694ae98,0x4695ae9a,0x4696ae9c,0x4697ae9e,0x4698aea0,0x4699aea2,
	0x469aaea4,0x469baea6,0x469caea8,0x469daeaa,0x469eaeac,0x469faeae,0x46a0aeb0,0x46a1aeb2,
	0x46a2aeb4,0x46a3aeb6,0x46a4aeb8,0x46a5aeba,0x46a6aebc,0x46a7aebe,0x46a8aec0,0x46a9aec2,
	0x46aaaec4,0x46abaec6,0x46acaec8,0x46adaeca,0x46aeaecc,0x46afaece,0x46b0aed0,0x46b1aed2,
	0x46b2aed4,0x46b3aed6,0x46b4aed8,0x46b5aeda,0x46b6aedc,0x46b7aede,0x46b8aee0,0x46b9aee2,
	0x46baaee4,0x46bbaee6,0x46bcaee8,0x46bdaeea,0x46beaeec,0x46bfaeee,0x46c0aef0,0x46c1aef2,
	0x46c2aef4,0x46c3aef6,0x46c4aef8,0x46c5aefa,0x46c6aefc,0x46c7aefe,0x46c8af00,0x46c9af02,
	0x46caaf04,0x46cbaf06,0x46ccaf08,0x46cdaf0a,0x46ceaf0c,0x46cfaf0e,0x46d0af10,0x46d1af12,
	0x46d2af14,0x46d3af16,0x46d4af18,0x46d5af1a,0x46d6af1c,0x46d7af1e,0x46d8af20,0x46d9af22,
	0x46daaf24,0x46dbaf26,0x46dcaf28,0x46ddaf2a,0x46deaf2c,0x46dfaf2e,0x46e0af30,0x46e1af32,
	0x46e2af34,0x46e3af36,0x46e4af38,0x46e5af3a,0x46e6af3c,0x46e7af3e,0x46e8af40,0x46e9af42,
	0x46eaaf44,0x46ebaf46,0x46ecaf48,0x46edaf4a,0x46eeaf4c,0x46efaf4e,0x46f0af50,0x46f1af52,
	0x46f2af54,0x46f3af56,0x46f4af58,0x46f5af5a,0x46f6af5c,0x46f7af5e,0x46f8af60,0x46f9af62,
	0x46faaf64,0x46fbaf66,0x46fcaf68,0x46fdaf6a,0x46feaf6c,0x46ffaf6e,0x4700af70,0x4701af72,
	0x4702af74,0x4703af76,0x4704af78,0x4705af7a,0x4706af7c,0x4707af7e,0x4708af80,0x4709af82,
	0x470aaf84,0x470baf86,0x470caf88,0x470daf8a,0x470eaf8c,0x470faf8e,0x4710af90,0x4711af92,
	0x4712af94,0x4713af96,0x4714af98,0x4715af9a,0x4716af9c,0x4717af9e,0x4718afa0,0x4719afa2,
	0x471aafa4,0x471bafa6,0x471cafa8,0x471dafaa,0x471eafac,0x471fafae,0x4720afb0,0x4721afb2,
	0x4722afb4,0x4723afb6,0x4724afb8,0x4725afba,0x4726afbc,0x4727afbe,0x4728afc0,0x4729afc2,
	0x472aafc4,0x472bafc6,0x472cafc8,0x472dafca,0x472eafcc,0x472fafce,0x4730afd0,0x4731afd2,
	0x4732afd4,0x4733afd6,0x4734afd8,0x4735afda,0x4736afdc,0x4737afde,0x4738afe0,0x4739afe2,
	0x473aafe4,0x473bafe6,0x473cafe8,0x473dafea,0x473eafec,0x473fafee,0x4740aff0,0x4741aff2,
	0x4742aff4,0x4743aff6,0x4744aff8,0x4745affa,0x4746affc,0x4747affe,0x4748b000,0x4749b002,
	0x474ab004,0x474bb006,0x474cb008,0x474db00a,0x474eb00c,0x474fb00e,0x4750b010,0x4751b012,
	0x4752b014,0x4753b016,0x4754b018,0x4755b01a,0x4756b01c,0x4757b01e,0x4758b020,0x4759b022,
	0x475ab024,0x475bb026,0x475cb028,0x475db02a,0x475eb02c,0x475fb02e,0x4760b030,0x4761b032,
	0x4762b034,0x4763b036,0x4764b038,0x4765b03a,0x4766b03c,0x4767b03e,0x4768b040,0x4769b042,
	0x476ab044,0x476bb046,0x476cb048,0x476db04a,0x476eb04c,0x476fb04e,0x4770b050,0x4771b052,
	0x4772b054,0x4773b056,0x4774b058,0x4775b05a,0x4776b05c,0x4777b05e,0x4778b060,0x4779b062,
	0x477ab064,0x477bb066,0x477cb068,0x477db06a,0x477eb06c,0x477fb06e,0x4780b070,0x4781b072,
	0x4782b074,0x4783b076,0x4784b078,0x4785b07a,0x4786b07c,0x4787b07e,0x4788b080,0x4789b082,
	0x478ab084,0x478bb086,0x478cb088,0x478db08a,0x478eb08c,0x478fb08e,0x4790b090,0x4791b092,
	0x4792b094,0x4793b096,0x4794b098,0x4795b09a,0x4796b09c,0x4797b09e,0x4798b0a0,0x4799b0a2,
	0x479ab0a4,0x479bb0a6,0x479cb0a8,0x479db0aa,0x479eb0ac,0x479fb0ae,0x47a0b0b0,0x47a1b0b2,
	0x47a2b0b4,0x47a3b0b6,0x47a4b0b8,0x47a5b0ba,0x47a6b0bc,0x47a7b0be,0x47a8b0c0,0x47a9b0c2,
	0x47aab0c4,0x47abb0c6,0x47acb0c8,0x47adb0ca,0x47aeb0cc,0x47afb0ce,0x47b0b0d0,0x47b1b0d2,
	0x47b2b0d4,0x47b3b0d6,0x47b4b0d8,0x47b5b0da,0x47b6b0dc,0x47b7b0de,0x47b8b0e0,0x47b9b0e2,
	0x47bab0e4,0x47bbb0e6,0x47bcb0e8,0x47bdb0ea,0x47beb0ec,0x47bfb0ee,0x47c0b0f0,0x47c1b0f2,
	0x47c2b0f4,0x47c3b0f6,0x47c4b0f8,0x47c5b0fa,0x47c6b0fc,0x47c7b0fe,0x47c8b100,0x47c9b102,
	0x47cab104,0x47cbb106,0x47ccb108,0x47cdb10a,0x47ceb10c,0x47cfb10e,0x47d0b110,0x47d1b112,
	0x47d2b114,0x47d3b116,0x47d4b118,0x47d5b11a,0x47d6b11c,0x47d7b11e,0x47d8b120,0x47d9b122,
	0x47dab124,0x47dbb126,0x47dcb128,0x47ddb12a,0x47deb12c,0x47dfb12e,0x47e0b130,0x47e1b132,
	0x47e2b134,0x47e3b136,0x47e4b138,0x47e5b13a,0x47e6b13c,0x47e7b13e,0x47e8b140,0x47e9b142,
	0x47eab144,0x47ebb146,0x47ecb148,0x47edb14a,0x47eeb14c,0x47efb14e,0x47f0b150,0x47f1b152,
	0x47f2b154,0x47f3b156,0x47f4b158,0x47f5b15a,0x47f6b15c,0x47f7b15e,0x47f8b160,0x47f9b162,
	0x47fab164,0x47fbb166,0x47fcb168,0x47fdb16a,0x47feb16c,0x47ffb16e,0x4800b170,0x4801b172,
	0x4802b174,0x4803b176,0x4804b178,0x4805b17a,0x4806b17c,0x4807b17e,0x4808b180,0x4809b182,
	0x480ab184,0x480bb186,0x480cb188,0x480db18a,0x480eb18c,0x480fb18e,0x4810b190,0x4811b192,
	0x4812b194,0x4813b196,0x4814b198,0x4815b19a,0x4816b19c,0x4817b19e,0x4818b1a0,0x4819b1a2,
	0x481ab1a4,0x481bb1a6,0x481cb1a8,0x481db1aa,0x481eb1ac,0x481fb1ae,0x4820b1b0,0x4821b1b2,
	0x4822b1b4,0x4823b1b6,0x4824b1b8,0x4825b1ba,0x4826b1bc,0x4827b1be,0x4828b1c0,0x4829b1c2,
	0x482ab1c4,0x482bb1c6,0x482cb1c8,0x482db1ca,0x482eb1cc,0x482fb1ce,0x4830b1d0,0x4831b1d2,
	0x4832b1d4,0x4833b1d6,0x4834b1d8,0x4835b1da,0x4836b1dc,0x4837b1de,0x4838b1e0,0x4839b1e2,
	0x483ab1e4,0x483bb1e6,0x483cb1e8,0x483db1ea,0x483eb1ec,0x483fb1ee,0x4840b1f0,0x4841b1f2,
	0x4842b1f4,0x4843b1f6,0x4844b1f8,0x4845b1fa,0x4846b1fc,0x4847b1fe,0x4848b200,0x4849b202,
	0x484ab204,0x484bb206,0x484cb208,0x484db20a,0x484eb20c,0x484fb20e,0x4850b210,0x4851b212,
	0x4852b214,0x4853b216,0x4854b218,0x4855b21a,0x4856b21c,0x4857b21e,0x4858b220,0x4859b222,
	0x485ab224,0x485bb226,0x485cb228,0x485db22a,0x485eb22c,0x485fb22e,0x4860b230,0x4861b232,
	0x4862b234,0x4863b236,0x4864b238,0x4865b23a,0x4866b23c,0x4867b23e,0x4868b240,0x4869b242,
	0x486ab244,0x486bb246,0x486cb248,0x486db24a,0x486eb24c,0x486fb24e,0x4870b250,0x4871b252,
	0x4872b254,0x4873b256,0x4874b258,0x4875b25a,0x4876b25c,0x4877b25e,0x4878b260,0x4879b262,
	0x487ab264,0x487bb266,0x487cb268,0x487db26a,0x487eb26c,0x487fb26e,0x4880b270,0x4881b272,
	0x4882b274,0x4883b276,0x4884b278,0x4885b27a,0x4886b27c,0x4887b27e,0x4888b280,0x4889b282,
	0x488ab284,0x488bb286,0x488cb288,0x488db28a,0x488eb28c,0x488fb28e,0x4890b290,0x4891b292,
	0x4892b294,0x4893b296,0x4894b298,0x4895b29a,0x4896b29c,0x4897b29e,0x4898b2a0,0x4899b2a2,
	0x489ab2a4,0x489bb2a6,0x489cb2a8,0x489db2aa,0x489eb2ac,0x489fb2ae,0x48a0b2b0,0x48a1b2b2,
	0x48a2b2b4,0x48a3b2b6,0x48a4b2b8,0x48a5b2ba,0x48a6b2bc,0x48a7b2be,0x48a8b2c0,0x48a9b2c2,
	0x48aab2c4,0x48abb2c6,0x48acb2c8,0x48adb2ca,0x48aeb2cc,0x48afb2ce,0x48b0b2d0,0x48b1b2d2,
	0x48b2b2d4,0x48b3b2d6,0x48b4b2d8,0x48b5b2da,0x48b6b2dc,0x48b7b2de,0x48b8b2e0,0x48b9b2e2,
	0x48bab2e4,0x48bbb2e6,0x48bcb2e8,0x48bdb2ea,0x48beb2ec,0x48bfb2ee,0x48c0b2f0,0x48c1b2f2,
	0x48c2b2f4,0x48c3b2f6,0x48c4b2f8,0x48c5b2fa,0x48c6b2fc,0x48c7b2fe,0x48c8b300,0x48c9b302,
	0x48cab304,0x48cbb306,0x48ccb308,0x48cdb30a,0x48ceb30c,0x48cfb30e,0x48d0b310,0x48d1b312,
	0x48d2b314,0x48d3b316,0x48d4b318,0x48d5b31a,0x48d6b31c,0x48d7b31e,0x48d8b320,0x48d9b322,
	0x48dab324,0x48dbb326,0x48dcb328,0x48ddb32a,0x48deb32c,0x48dfb32e,0x48e0b330,0x48e1b332,
	0x48e2b334,0x48e3b336,0x48e4b338,0x48e5b33a,0x48e6b33c,0x48e7b33e,0x48e8b340,0x48e9b342,
	0x48eab344,0x48ebb346,0x48ecb348,0x48edb34a,0x48eeb34c,0x48efb34e,0x48f0b350,0x48f1b352,
	0x48f2b354,0x48f3b356,0x48f4b358,0x48f5b35a,0x48f6b35c,0x48f7b35e,0x48f8b360,0x48f9b362,
	0x48fab364,0x48fbb366,0x48fcb368,0x48fdb36a,0x48feb36c,0x48ffb36e,0x4900b370,0x4901b372,
	0x4902b374,0x4903b376,0x4904b378,0x4905b37a,0x4906b37c,0x4907b37e,0x4908b380,0x4909b382,
	0x490ab384,0x490bb386,0x490cb388,0x490db38a,0x490eb38c,0x490fb38e,0x4910b390,0x4911b392,
	0x4912b394,0x4913b396,0x4914b398,0x4915b39a,0x4916b39c,0x4917b39e,0x4918b3a0,0x4919b3a2,
	0x491ab3a4,0x491bb3a6,0x491cb3a8,0x491db3aa,0x491eb3ac,0x491fb3ae,0x4920b3b0,0x4921b3b2,
	0x4922b3b4,0x4923b3b6,0x4924b3b8,0x4925b3ba,0x4926b3bc,0x4927b3be,0x4928b3c0,0x4929b3c2,
	0x492ab3c4,0x492bb3c6,0x492cb3c8,0x492db3ca,0x492eb3cc,0x492fb3ce,0x4930b3d0,0x4931b3d2,
	0x4932b3d4,0x4933b3d6,0x4934b3d8,0x4935b3da,0x4936b3dc,0x4937b3de,0x4938b3e0,0x4939b3e2,
	0x493ab3e4,0x493bb3e6,0x493cb3e8,0x493db3ea,0x493eb3ec,0x493fb3ee,0x4940b3f0,0x4941b3f2,
	0x4942b3f4,0x4943b3f6,0x4944b3f8,0x4945b3fa,0x4946b3fc,0x4947b3fe,0x4948b400,0x4949b402,
	0x494ab404,0x494bb406,0x494cb408,0x494db40a,0x494eb40c,0x494fb40e,0x4950b410,0x4951b412,
	0x4952b414,0x4953b416,0x4954b418,0x4955b41a,0x4956b41c,0x4957b41e,0x4958b420,0x4959b422,
	0x495ab424,0x495bb426,0x495cb428,0x495db42a,0x495eb42c,0x495fb42e,0x4960b430,0x4961b432,
	0x4962b434,0x4963b436,0x4964b438,0x4965b43a,0x4966b43c,0x4967b43e,0x4968b440,0x4969b442,
	0x496ab444,0x496bb446,0x496cb448,0x496db44a,0x496eb44c,0x496fb44e,0x4970b450,0x4971b452,
	0x4972b454,0x4973b456,0x4974b458,0x4975b45a,0x4976b45c,0x4977b45e,0x4978b460,0x4979b462,
	0x497ab464,0x497bb466,0x497cb468,0x497db46a,0x497eb46c,0x497fb46e,0x4980b470,0x4981b472,
	0x4982b474,0x4983b476,0x4984b478,0x4985b47a,0x4986b47c,0x4987b47e,0x4988b480,0x4989b482,
	0x498ab484,0x498bb486,0x498cb488,0x498db48a,0x498eb48c,0x498fb48e,0x4990b490,0x4991b492,
	0x4992b494,0x4993b496,0x4994b498,0x4995b49a,0x4996b49c,0x4997b49e,0x4998b4a0,0x4999b4a2,
	0x499ab4a4,0x499bb4a6,0x499cb4a8,0x499db4aa,0x499eb4ac,0x499fb4ae,0x49a0b4b0,0x49a1b4b2,
	0x49a2b4b4,0x49a3b4b6,0x49a4b4b8,0x49a5b4ba,0x49a6b4bc,0x49a7b4be,0x49a8b4c0,0x49a9b4c2,
	0x49aab4c4,0x49abb4c6,0x49acb4c8,0x49adb4ca,0x49aeb4cc,0x49afb4ce,0x49b0b4d0,0x49b1b4d2,
	0x49b2b4d4,0x49b3b4d6,0x49b4b4d8,0x49b5b4da,0x49b6b4dc,0x49b7b4de,0x49b8b4e0,0x49b9b4e2,
	0x49bab4e4,0x49bbb4e6,0x49bcb4e8,0x49bdb4ea,0x49beb4ec,0x49bfb4ee,0x49c0b4f0,0x49c1b4f2,
	0x49c2b4f4,0x49c3b4f6,0x49c4b4f8,0x49c5b4fa,0x49c6b4fc,0x49c7b4fe,0x49c8b500,0x49c9b502,
	0x49cab504,0x49cbb506,0x49ccb508,0x49cdb50a,0x49ceb50c,0x49cfb50e,0x49d0b510,0x49d1b512,
	0x49d2b514,0x49d3b516,0x49d4b518,0x49d5b51a,0x49d6b51c,0x49d7b51e,0x49d8b520,0x49d9b522,
	0x49dab524,0x49dbb526,0x49dcb528,0x49ddb52a,0x49deb52c,0x49dfb52e,0x49e0b530,0x49e1b532,
	0x49e2b534,0x49e3b536,0x49e4b538,0x49e5b53a,0x49e6b53c,0x49e7b53e,0x49e8b540,0x49e9b542,
	0x49eab544,0x49ebb546,0x49ecb548,0x49edb54a,0x49eeb54c,0x49efb54e,0x49f0b550,0x49f1b552,
	0x49f2b554,0x49f3b556,0x49f4b558,0x49f5b55a,0x49f6b55c,0x49f7b55e,0x49f8b560,0x49f9b562,
	0x49fab564,0x49fbb566,0x49fcb568,0x49fdb56a,0x49feb56c,0x49ffb56e,0x4a00b570,0x4a01b572,
	0x4a02b574,0x4a03b576,0x4a04b578,0x4a05b57a,0x4a06b57c,0x4a07b57e,0x4a08b580,0x4a09b582,
	0x4a0ab584,0x4a0bb586,0x4a0cb588,0x4a0db58a,0x4a0eb58c,0x4a0fb58e,0x4a10b590,0x4a11b592,
	0x4a12b594,0x4a13b596,0x4a14b598,0x4a15b59a,0x4a16b59c,0x4a17b59e,0x4a18b5a0,0x4a19b5a2,
	0x4a1ab5a4,0x4a1bb5a6,0x4a1cb5a8,0x4a1db5aa,0x4a1eb5ac,0x4a1fb5ae,0x4a20b5b0,0x4a21b5b2,
	0x4a22b5b4,0x4a23b5b6,0x4a24b5b8,0x4a25b5ba,0x4a26b5bc,0x4a27b5be,0x4a28b5c0,0x4a29b5c2,
	0x4a2ab5c4,0x4a2bb5c6,0x4a2cb5c8,0x4a2db5ca,0x4a2eb5cc,0x4a2fb5ce,0x4a30b5d0,0x4a31b5d2,
	0x4a32b5d4,0x4a33b5d6,0x4a34b5d8,0x4a35b5da,0x4a36b5dc,0x4a37b5de,0x4a38b5e0,0x4a39b5e2,
	0x4a3ab5e4,0x4a3bb5e6,0x4a3cb5e8,0x4a3db5ea,0x4a3eb5ec,0x4a3fb5ee,0x4a40b5f0,0x4a41b5f2,
	0x4a42b5f4,0x4a43b5f6,0x4a44b5f8,0x4a45b5fa,0x4a46b5fc,0x4a47b5fe,0x4a48b600,0x4a49b602,
	0x4a4ab604,0x4a4bb606,0x4a4cb608,0x4a4db60a,0x4a4eb60c,0x4a4fb60e,0x4a50b610,0x4a51b612,
	0x4a52b614,0x4a53b616,0x4a54b618,0x4a55b61a,0x4a56b61c,0x4a57b61e,0x4a58b620,0x4a59b622,
	0x4a5ab624,0x4a5bb626,0x4a5cb628,0x4a5db62a,0x4a5eb62c,0x4a5fb62e,0x4a60b630,0x4a61b632,
	0x4a62b634,0x4a63b636,0x4a64b638,0x4a65b63a,0x4a66b63c,0x4a67b63e,0x4a68b640,0x4a69b642,
	0x4a6ab644,0x4a6bb646,0x4a6cb648,0x4a6db64a,0x4a6eb64c,0x4a6fb64e,0x4a70b650,0x4a71b652,
	0x4a72b654,0x4a73b656,0x4a74b658,0x4a75b65a,0x4a76b65c,0x4a77b65e,0x4a78b660,0x4a79b662,
	0x4a7ab664,0x4a7bb666,0x4a7cb668,0x4a7db66a,0x4a7eb66c,0x4a7fb66e,0x4a80b670,0x4a81b672,
	0x4a82b674,0x4a83b676,0x4a84b678,0x4a85b67a,0x4a86b67c,0x4a87b67e,0x4a88b680,0x4a89b682,
	0x4a8ab684,0x4a8bb686,0x4a8cb688,0x4a8db68a,0x4a8eb68c,0x4a8fb68e,0x4a90b690,0x4a91b692,
	0x4a92b694,0x4a93b696,0x4a94b698,0x4a95b69a,0x4a96b69c,0x4a97b69e,0x4a98b6a0,0x4a99b6a2,
	0x4a9ab6a4,0x4a9bb6a6,0x4a9cb6a8,0x4a9db6aa,0x4a9eb6ac,0x4a9fb6ae,0x4aa0b6b0,0x4aa1b6b2,
	0x4aa2b6b4,0x4aa3b6b6,0x4aa4b6b8,0x4aa5b6ba,0x4aa6b6bc,0x4aa7b6be,0x4aa8b6c0,0x4aa9b6c2,
	0x4aaab6c4,0x4aabb6c6,0x4aacb6c8,0x4aadb6ca,0x4aaeb6cc,0x4aafb6ce,0x4ab0b6d0,0x4ab1b6d2,
	0x4ab2b6d4,0x4ab3b6d6,0x4ab4b6d8,0x4ab5b6da,0x4ab6b6dc,0x4ab7b6de,0x4ab8b6e0,0x4ab9b6e2,
	0x4abab6e4,0x4abbb6e6,0x4abcb6e8,0x4abdb6ea,0x4abeb6ec,0x4abfb6ee,0x4ac0b6f0,0x4ac1b6f2,
	0x4ac2b6f4,0x4ac3b6f6,0x4ac4b6f8,0x4ac5b6fa,0x4ac6b6fc,0x4ac7b6fe,0x4ac8b700,0x4ac9b702,
	0x4acab704,0x4acbb706,0x4accb708,0x4acdb70a,0x4aceb70c,0x4acfb70e,0x4ad0b710,0x4ad1b712,
	0x4ad2b714,0x4ad3b716,0x4ad4b718,0x4ad5b71a,0x4ad6b71c,0x4ad7b71e,0x4ad8b720,0x4ad9b722,
	0x4adab724,0x4adbb726,0x4adcb728,0x4addb72a,0x4adeb72c,0x4adfb72e,0x4ae0b730,0x4ae1b732,
	0x4ae2b734,0x4ae3b736,0x4ae4b738,0x4ae5b73a,0x4ae6b73c,0x4ae7b73e,0x4ae8b740,0x4ae9b742,
	0x4aeab744,0x4aebb746,0x4aecb748,0x4aedb74a,0x4aeeb74c,0x4aefb74e,0x4af0b750,0x4af1b752,
	0x4af2b754,0x4af3b756,0x4af4b758,0x4af5b75a,0x4af6b75c,0x4af7b75e,0x4af8b760,0x4af9b762,
	0x4afab764,0x4afbb766,0x4afcb768,0x4afdb76a,0x4afeb76c,0x4affb76e,0x4b00b770,0x4b01b772,
	0x4b02b774,0x4b03b776,0x4b04b778,0x4b05b77a,0x4b06b77c,0x4b07b77e,0x4b08b780,0x4b09b782,
	0x4b0ab784,0x4b0bb786,0x4b0cb788,0x4b0db78a,0x4b0eb78c,0x4b0fb78e,0x4b10b790,0x4b11b792,
	0x4b12b794,0x4b13b796,0x4b14b798,0x4b15b79a,0x4b16b79c,0x4b17b79e,0x4b18b7a0,0x4b19b7a2,
	0x4b1ab7a4,0x4b1bb7a6,0x4b1cb7a8,0x4b1db7aa,0x4b1eb7ac,0x4b1fb7ae,0x4b20b7b0,0x4b21b7b2,
	0x4b22b7b4,0x4b23b7b6,0x4b24b7b8,0x4b25b7ba,0x4b26b7bc,0x4b27b7be,0x4b28b7c0,0x4b29b7c2,
	0x4b2ab7c4,0x4b2bb7c6,0x4b2cb7c8,0x4b2db7ca,0x4b2eb7cc,0x4b2fb7ce,0x4b30b7d0,0x4b31b7d2,
	0x4b32b7d4,0x4b33b7d6,0x4b34b7d8,0x4b35b7da,0x4b36b7dc,0x4b37b7de,0x4b38b7e0,0x4b39b7e2,
	0x4b3ab7e4,0x4b3bb7e6,0x4b3cb7e8,0x4b3db7ea,0x4b3eb7ec,0x4b3fb7ee,0x4b40b7f0,0x4b41b7f2,
	0x4b42b7f4,0x4b43b7f6,0x4b44b7f8,0x4b45b7fa,0x4b46b7fc,0x4b47b7fe,0x4b48b800,0x4b49b802,
	0x4b4ab804,0x4b4bb806,0x4b4cb808,0x4b4db80a,0x4b4eb80c,0x4b4fb80e,0x4b50b810,0x4b51b812,
	0x4b52b814,0x4b53b816,0x4b54b818,0x4b55b81a,0x4b56b81c,0x4b57b81e,0x4b58b820,0x4b59b822,
	0x4b5ab824,0x4b5bb826,0x4b5cb828,0x4b5db82a,0x4b5eb82c,0x4b5fb82e,0x4b60b830,0x4b61b832,
	0x4b62b834,0x4b63b836,0x4b64b838,0x4b65b83a,0x4b66b83c,0x4b67b83e,0x4b68b840,0x4b69b842,
	0x4b6ab844,0x4b6bb846,0x4b6cb848,0x4b6db84a,0x4b6eb84c,0x4b6fb84e,0x4b70b850,0x4b71b852,
	0x4b72b854,0x4b73b856,0x4b74b858,0x4b75b85a,0x4b76b85c,0x4b77b85e,0x4b78b860,0x4b79b862,
	0x4b7ab864,0x4b7bb866,0x4b7cb868,0x4b7db86a,0x4b7eb86c,0x4b7fb86e,0x4b80b870,0x4b81b872,
	0x4b82b874,0x4b83b876,0x4b84b878,0x4b85b87a,0x4b86b87c,0x4b87b87e,0x4b88b880,0x4b89b882,
	0x4b8ab884,0x4b8bb886,0x4b8cb888,0x4b8db88a,0x4b8eb88c,0x4b8fb88e,0x4b90b890,0x4b91b892,
	0x4b92b894,0x4b93b896,0x4b94b898,0x4b95b89a,0x4b96b89c,0x4b97b89e,0x4b98b8a0,0x4b99b8a2,
	0x4b9ab8a4,0x4b9bb8a6,0x4b9cb8a8,0x4b9db8aa,0x4b9eb8ac,0x4b9fb8ae,0x4ba0b8b0,0x4ba1b8b2,
	0x4ba2b8b4,0x4ba3b8b6,0x4ba4b8b8,0x4ba5b8ba,0x4ba6b8bc,0x4ba7b8be,0x4ba8b8c0,0x4ba9b8c2,
	0x4baab8c4,0x4babb8c6,0x4bacb8c8,0x4badb8ca,0x4baeb8cc,0x4bafb8ce,0x4bb0b8d0,0x4bb1b8d2,
	0x4bb2b8d4,0x4bb3b8d6,0x4bb4b8d8,0x4bb5b8da,0x4bb6b8dc,0x4bb7b8de,0x4bb8b8e0,0x4bb9b8e2,
	0x4bbab8e4,0x4bbbb8e6,0x4bbcb8e8,0x4bbdb8ea,0x4bbeb8ec,0x4bbfb8ee,0x4bc0b8f0,0x4bc1b8f2,
	0x4bc2b8f4,0x4bc3b8f6,0x4bc4b8f8,0x4bc5b8fa,0x4bc6b8fc,0x4bc7b8fe,0x4bc8b900,0x4bc9b902,
	0x4bcab904,0x4bcbb906,0x4bccb908,0x4bcdb90a,0x4bceb90c,0x4bcfb90e,0x4bd0b910,0x4bd1b912,
	0x4bd2b914,0x4bd3b916,0x4bd4b918,0x4bd5b91a,0x4bd6b91c,0x4bd7b91e,0x4bd8b920,0x4bd9b922,
	0x4bdab924,0x4bdbb926,0x4bdcb928,0x4bddb92a,0x4bdeb92c,0x4bdfb92e,0x4be0b930,0x4be1b932,
	0x4be2b934,0x4be3b936,0x4be4b938,0x4be5b93a,0x4be6b93c,0x4be7b93e,0x4be8b940,0x4be9b942,
	0x4beab944,0x4bebb946,0x4becb948,0x4bedb94a,0x4beeb94c,0x4befb94e,0x4bf0b950,0x4bf1b952,
	0x4bf2b954,0x4bf3b956,0x4bf4b958,0x4bf5b95a,0x4bf6b95c,0x4bf7b95e,0x4bf8b960,0x4bf9b962,
	0x4bfab964,0x4bfbb966,0x4bfcb968,0x4bfdb96a,0x4bfeb96c,0x4bffb96e,0x4c00b970,0x4c01b972,
	0x4c02b974,0x4c03b976,0x4c04b978,0x4c05b97a,0x4c06b97c,0x4c07b97e,0x4c08b980,0x4c09b982,
	0x4c0ab984,0x4c0bb986,0x4c0cb988,0x4c0db98a,0x4c0eb98c,0x4c0fb98e,0x4c10b990,0x4c11b992,
	0x4c12b994,0x4c13b996,0x4c14b998,0x4c15b99a,0x4c16b99c,0x4c17b99e,0x4c18b9a0,0x4c19b9a2,
	0x4c1ab9a4,0x4c1bb9a6,0x4c1cb9a8,0x4c1db9aa,0x4c1eb9ac,0x4c1fb9ae,0x4c20b9b0,0x4c21b9b2,
	0x4c22b9b4,0x4c23b9b6,0x4c24b9b8,0x4c25b9ba,0x4c26b9bc,0x4c27b9be,0x4c28b9c0,0x4c29b9c2,
	0x4c2ab9c4,0x4c2bb9c6,0x4c2cb9c8,0x4c2db9ca,0x4c2eb9cc,0x4c2fb9ce,0x4c30b9d0,0x4c31b9d2,
	0x4c32b9d4,0x4c33b9d6,0x4c34b9d8,0x4c35b9da,0x4c36b9dc,0x4c37b9de,0x4c38b9e0,0x4c39b9e2,
	0x4c3ab9e4,0x4c3bb9e6,0x4c3cb9e8,0x4c3db9ea,0x4c3eb9ec,0x4c3fb9ee,0x4c40b9f0,0x4c41b9f2,
	0x4c42b9f4,0x4c43b9f6,0x4c44b9f8,0x4c45b9fa,0x4c46b9fc,0x4c47b9fe,0x4c48ba00,0x4c49ba02,
	0x4c4aba04,0x4c4bba06,0x4c4cba08,0x4c4dba0a,0x4c4eba0c,0x4c4fba0e,0x4c50ba10,0x4c51ba12,
	0x4c52ba14,0x4c53ba16,0x4c54ba18,0x4c55ba1a,0x4c56ba1c,0x4c57ba1e,0x4c58ba20,0x4c59ba22,
	0x4c5aba24,0x4c5bba26,0x4c5cba28,0x4c5dba2a,0x4c5eba2c,0x4c5fba2e,0x4c60ba30,0x4c61ba32,
	0x4c62ba34,0x4c63ba36,0x4c64ba38,0x4c65ba3a,0x4c66ba3c,0x4c67ba3e,0x4c68ba40,0x4c69ba42,
	0x4c6aba44,0x4c6bba46,0x4c6cba48,0x4c6dba4a,0x4c6eba4c,0x4c6fba4e,0x4c70ba50,0x4c71ba52,
	0x4c72ba54,0x4c73ba56,0x4c74ba58,0x4c75ba5a,0x4c76ba5c,0x4c77ba5e,0x4c78ba60,0x4c79ba62,
	0x4c7aba64,0x4c7bba66,0x4c7cba68,0x4c7dba6a,0x4c7eba6c,0x4c7fba6e,0x4c80ba70,0x4c81ba72,
	0x4c82ba74,0x4c83ba76,0x4c84ba78,0x4c85ba7a,0x4c86ba7c,0x4c87ba7e,0x4c88ba80,0x4c89ba82,
	0x4c8aba84,0x4c8bba86,0x4c8cba88,0x4c8dba8a,0x4c8eba8c,0x4c8fba8e,0x4c90ba90,0x4c91ba92,
	0x4c92ba94,0x4c93ba96,0x4c94ba98,0x4c95ba9a,0x4c96ba9c,0x4c97ba9e,0x4c98baa0,0x4c99baa2,
	0x4c9abaa4,0x4c9bbaa6,0x4c9cbaa8,0x4c9dbaaa,0x4c9ebaac,0x4c9fbaae,0x4ca0bab0,0x4ca1bab2,
	0x4ca2bab4,0x4ca3bab6,0x4ca4bab8,0x4ca5baba,0x4ca6babc,0x4ca7babe,0x4ca8bac0,0x4ca9bac2,
	0x4caabac4,0x4cabbac6,0x4cacbac8,0x4cadbaca,0x4caebacc,0x4cafbace,0x4cb0bad0,0x4cb1bad2,
	0x4cb2bad4,0x4cb3bad6,0x4cb4bad8,0x4cb5bada,0x4cb6badc,0x4cb7bade,0x4cb8bae0,0x4cb9bae2,
	0x4cbabae4,0x4cbbbae6,0x4cbcbae8,0x4cbdbaea,0x4cbebaec,0x4cbfbaee,0x4cc0baf0,0x4cc1baf2,
	0x4cc2baf4,0x4cc3baf6,0x4cc4baf8,0x4cc5bafa,0x4cc6bafc,0x4cc7bafe,0x4cc8bb00,0x4cc9bb02,
	0x4ccabb04,0x4ccbbb06,0x4cccbb08,0x4ccdbb0a,0x4ccebb0c,0x4ccfbb0e,0x4cd0bb10,0x4cd1bb12,
	0x4cd2bb14,0x4cd3bb16,0x4cd4bb18,0x4cd5bb1a,0x4cd6bb1c,0x4cd7bb1e,0x4cd8bb20,0x4cd9bb22,
	0x4cdabb24,0x4cdbbb26,0x4cdcbb28,0x4cddbb2a,0x4cdebb2c,0x4cdfbb2e,0x4ce0bb30,0x4ce1bb32,
	0x4ce2bb34,0x4ce3bb36,0x4ce4bb38,0x4ce5bb3a,0x4ce6bb3c,0x4ce7bb3e,0x4ce8bb40,0x4ce9bb42,
	0x4ceabb44,0x4cebbb46,0x4cecbb48,0x4cedbb4a,0x4ceebb4c,0x4cefbb4e,0x4cf0bb50,0x4cf1bb52,
	0x4cf2bb54,0x4cf3bb56,0x4cf4bb58,0x4cf5bb5a,0x4cf6bb5c,0x4cf7bb5e,0x4cf8bb60,0x4cf9bb62,
	0x4cfabb64,0x4cfbbb66,0x4cfcbb68,0x4cfdbb6a,0x4cfebb6c,0x4cffbb6e,0x4d00bb70,0x4d01bb72,
	0x4d02bb74,0x4d03bb76,0x4d04bb78,0x4d05bb7a,0x4d06bb7c,0x4d07bb7e,0x4d08bb80,0x4d09bb82,
	0x4d0abb84,0x4d0bbb86,0x4d0cbb88,0x4d0dbb8a,0x4d0ebb8c,0x4d0fbb8e,0x4d10bb90,0x4d11bb92,
	0x4d12bb94,0x4d13bb96,0x4d14bb98,0x4d15bb9a,0x4d16bb9c,0x4d17bb9e,0x4d18bba0,0x4d19bba2,
	0x4d1abba4,0x4d1bbba6,0x4d1cbba8,0x4d1dbbaa,0x4d1ebbac,0x4d1fbbae,0x4d20bbb0,0x4d21bbb2,
	0x4d22bbb4,0x4d23bbb6,0x4d24bbb8,0x4d25bbba,0x4d26bbbc,0x4d27bbbe,0x4d28bbc0,0x4d29bbc2,
	0x4d2abbc4,0x4d2bbbc6,0x4d2cbbc8,0x4d2dbbca,0x4d2ebbcc,0x4d2fbbce,0x4d30bbd0,0x4d31bbd2,
	0x4d32bbd4,0x4d33bbd6,0x4d34bbd8,0x4d35bbda,0x4d36bbdc,0x4d37bbde,0x4d38bbe0,0x4d39bbe2,
	0x4d3abbe4,0x4d3bbbe6,0x4d3cbbe8,0x4d3dbbea,0x4d3ebbec,0x4d3fbbee,0x4d40bbf0,0x4d41bbf2,
	0x4d42bbf4,0x4d43bbf6,0x4d44bbf8,0x4d45bbfa,0x4d46bbfc,0x4d47bbfe,0x4d48bc00,0x4d49bc02,
	0x4d4abc04,0x4d4bbc06,0x4d4cbc08,0x4d4dbc0a,0x4d4ebc0c,0x4d4fbc0e,0x4d50bc10,0x4d51bc12,
	0x4d52bc14,0x4d53bc16,0x4d54bc18,0x4d55bc1a,0x4d56bc1c,0x4d57bc1e,0x4d58bc20,0x4d59bc22,
	0x4d5abc24,0x4d5bbc26,0x4d5cbc28,0x4d5dbc2a,0x4d5ebc2c,0x4d5fbc2e,0x4d60bc30,0x4d61bc32,
	0x4d62bc34,0x4d63bc36,0x4d64bc38,0x4d65bc3a,0x4d66bc3c,0x4d67bc3e,0x4d68bc40,0x4d69bc42,
	0x4d6abc44,0x4d6bbc46,0x4d6cbc48,0x4d6dbc4a,0x4d6ebc4c,0x4d6fbc4e,0x4d70bc50,0x4d71bc52,
	0x4d72bc54,0x4d73bc56,0x4d74bc58,0x4d75bc5a,0x4d76bc5c,0x4d77bc5e,0x4d78bc60,0x4d79bc62,
	0x4d7abc64,0x4d7bbc66,0x4d7cbc68,0x4d7dbc6a,0x4d7ebc6c,0x4d7fbc6e,0x4d80bc70,0x4d81bc72,
	0x4d82bc74,0x4d83bc76,0x4d84bc78,0x4d85bc7a,0x4d86bc7c,0x4d87bc7e,0x4d88bc80,0x4d89bc82,
	0x4d8abc84,0x4d8bbc86,0x4d8cbc88,0x4d8dbc8a,0x4d8ebc8c,0x4d8fbc8e,0x4d90bc90,0x4d91bc92,
	0x4d92bc94,0x4d93bc96,0x4d94bc98,0x4d95bc9a,0x4d96bc9c,0x4d97bc9e,0x4d98bca0,0x4d99bca2,
	0x4d9abca4,0x4d9bbca6,0x4d9cbca8,0x4d9dbcaa,0x4d9ebcac,0x4d9fbcae,0x4da0bcb0,0x4da1bcb2,
	0x4da2bcb4,0x4da3bcb6,0x4da4bcb8,0x4da5bcba,0x4da6bcbc,0x4da7bcbe,0x4da8bcc0,0x4da9bcc2,
	0x4daabcc4,0x4dabbcc6,0x4dacbcc8,0x4dadbcca,0x4daebccc,0x4dafbcce,0x4db0bcd0,0x4db1bcd2,
	0x4db2bcd4,0x4db3bcd6,0x4db4bcd8,0x4db5bcda,0x4e0016ba,0x4e0103eb,0x4e021ace,0x4e030f2d,
	0x4e041ad0,0x4e051ad2,0x4e061ad4,0x4e0713de,0x4e0818d0,0x4e0910e0,0x4e0a113a,0x4e0b14db,
	0x4e0c0f3c,0x4e0d0176,0x4e0e17d4,0x4e0f1ad6,0x4e100575,0x4e110284,0x4e121ad8,0x4e1319d8,
	0x4e140fd3,0x4e150ea7,0x4e1611b0,0x4e171ada,0x4e181014,0x4e190143,0x4e1a16ad,0x4e1b02ea,
	0x4e1c03ff,0x4e1d122e,0x4e1e0232,0x4e1f1adc,0x4e201ade,0x4e211ae0,0x4e2203fd,0x4e231ae2,
	0x4e240b87,0x4e25163c,0x4e261ae4,0x4e2710e8,0x4e28067b,0x4e291ae6,0x4e2a05cd,0x4e2b160f,
	0x4e2c0e65,0x4e2d197e,0x4e2e1ae8,0x4e2f1aea,0x4e3004fe,0x4e311aec,0x4e3202af,0x4e331aee,
	0x4e340bb1,0x4e351af0,0x4e3619bb,0x4e371af2,0x4e3813c9,0x4e390348,0x4e3a13fb,0x4e3b19bc,
	0x4e3c1af4,0x4e3d0b3f,0x4e3e098c,0x4e3f0ee9,0x4e401af6,0x4e411af8,0x4e421afa,0x4e430dab,
	0x4e441afc,0x4e450968,0x4e461afe,0x4e47139f,0x4e480ccf,0x4e4916ea,0x4e4a1b00,0x4e4b1930,
	0x4e4c144e,0x4e4d189f,0x4e4e071f,0x4e4f049b,0x4e500af8,0x4e511b02,0x4e520ef6,0x4e530e6e,
	0x4e540fbe,0x4e551b04,0x4e560642,0x4e571b06,0x4e580239,0x4e5916df,0x4e5a1b08,0x4e5b1b0a,
	0x4e5c0d41,0x4e5d0967,0x4e5e0f5b,0x4e5f16aa,0x4e6014b3,0x4e61150d,0x4e621b0c,0x4e631b0e,
	0x4e641b10,0x4e651b12,0x4e6611dd,0x4e671b14,0x4e681b16,0x4e6907db,0x4e6a1b18,0x4e6b1b1a,
	0x4e6c1b1c,0x4e6d1b1e,0x4e6e1b20,0x4e6f1b22,0x4e700c92,0x4e710c49,0x4e721b24,0x4e7310bb,
	0x4e741b26,0x4e751b28,0x4e761b2a,0x4e771b2c,0x4e781b2e,0x4e791b30,0x4e7a1b32,0x4e7b1b34,
	0x4e7c1b36,0x4e7d1b38,0x4e7e0f8f,0x4e7f1b3a,0x4e801b3c,0x4e811b3e,0x4e821b40,0x4e831b42,
	0x4e841b44,0x4e851b46,0x4e860afd,0x4e871b48,0x4e8817b4,0x4e89191c,0x4e8a1b4a,0x4e8b11b5,
	0x4e8c0498,0x4e8d029b,0x4e8e17b3,0x4e8f0a7e,0x4e901b4c,0x4e911836,0x4e92073d,0x4e930f3d,
	0x4e941460,0x4e950941,0x4e961b4e,0x4e971b50,0x4e9805d6,0x4e991b52,0x4e9a1623,0x4e9b1547,
	0x4e9c1b54,0x4e9d1b56,0x4e9e1b58,0x4e9f0801,0x4ea01377,0x4ea113e1,0x4ea20a08,0x4ea31b5a,
	0x4ea408ba,0x4ea5069c,0x4ea616f1,0x4ea701e1,0x4ea806f7,0x4ea90d8c,0x4eaa1b5c,0x4eab151d,
	0x4eac0932,0x4ead1355,0x4eae0b89,0x4eaf1b5e,0x4eb01b60,0x4eb11b62,0x4eb20fdd,0x4eb3015b,
	0x4eb41b64,0x4eb5155e,0x4eb61b66,0x4eb71b68,0x4eb81b6a,0x4eb91b6c,0x4eba1086,0x4ebb1087,
	0x4ebc1b6e,0x4ebd1b70,0x4ebe1b72,0x4ebf16eb,0x4ec0116f,0x4ec11088,0x4ec20af7,0x4ec303ec,
	0x4ec41883,0x4ec5091b,0x4ec60f14,0x4ec70278,0x4ec81b74,0x4ec918cd,0x4eca0910,0x4ecb0904,
	0x4ecc1b76,0x4ecd1098,0x4ece02e9,0x4ecf1b78,0x4ed01b7a,0x4ed10c4e,0x4ed21b7c,0x4ed30198,
	0x4ed41a0c,0x4ed511b1,0x4ed612ae,0x4ed718d1,0x4ed80554,0x4ed914df,0x4eda1b7e,0x4edb1b80,
	0x4edc1b82,0x4edd1363,0x4ede108f,0x4edf0f78,0x4ee01b84,0x4ee105bd,0x4ee21b86,0x4ee30337,
	0x4ee40bdc,0x4ee516e1,0x4ee61b88,0x4ee71b8a,0x4ee810d4,0x4ee91b8c,0x4eea16c7,0x4eeb0d91,
	0x4eec0cee,0x4eed1b8e,0x4eee1b90,0x4eef1b92,0x4ef0167d,0x4ef11b94,0x4ef2198b,0x4ef30ec3,
	0x4ef41b96,0x4ef51462,0x4ef6088e,0x4ef7085b,0x4ef81b98,0x4ef91b9a,0x4efa1b9c,0x4efb1090,
	0x4efc1b9e,0x4efd04f6,0x4efe1ba0,0x4eff04c8,0x4f001ba2,0x4f010f5c,0x4f021ba4,0x4f031ba6,
	0x4f041ba8,0x4f051baa,0x4f061bac,0x4f071bae,0x4f081bb0,0x4f090a09,0x4f0a16bc,0x4f0b1bb2,
	0x4f0c1bb4,0x4f0d1463,0x4f0e081d,0x4f0f0520,0x4f10049c,0x4f11159b,0x4f121bb6,0x4f131bb8,
	0x4f141bba,0x4f151bbc,0x4f161bbe,0x4f17198c,0x4f181784,0x4f1907c9,0x4f1a07a6,0x4f1b17d5,
	0x4f1c1bc0,0x4f1d1bc2,0x4f1e10e3,0x4f1f140b,0x4f2002a8,0x4f211bc4,0x4f221617,0x4f231bc6,
	0x4f241131,0x4f2501eb,0x4f260c4f,0x4f270199,0x4f281bc8,0x4f291bca,0x4f2a140c,0x4f2b19c3,
	0x4f2c1bcc,0x4f2d1bce,0x4f2e1bd0,0x4f2f0156,0x4f30060d,0x4f311bd2,0x4f320de1,0x4f331bd4,
	0x4f340078,0x4f351bd6,0x4f360bc6,0x4f371bd8,0x4f381164,0x4f391bda,0x4f3a1241,0x4f3b1bdc,
	0x4f3c1242,0x4f3d083c,0x4f3e1bde,0x4f3f1be0,0x4f401be2,0x4f411be4,0x4f421be6,0x4f4303c0,
	0x4f441be8,0x4f451bea,0x4f460358,0x4f471bec,0x4f481bee,0x4f491bf0,0x4f4a1bf2,0x4f4b1bf4,
	0x4f4c1bf6,0x4f4d141d,0x4f4e0390,0x4f4f19c4,0x4f501a60,0x4f5117a5,0x4f521bf8,0x4f531325,
	0x4f541bfa,0x4f5506dc,0x4f561bfc,0x4f5713a5,0x4f581155,0x4f5917b5,0x4f5a16f4,0x4f5b0513,
	0x4f5c1a61,0x4f5d05f9,0x4f5e0e16,0x4f5f1365,0x4f600de2,0x4f611bfe,0x4f621c00,0x4f63176b,
	0x4f6413be,0x4f650f7d,0x4f661c02,0x4f6709e6,0x4f681c04,0x4f690e8c,0x4f6a1c06,0x4f6b1c08,
	0x4f6c0aee,0x4f6d1c0a,0x4f6e1c0c,0x4f6f1677,0x4f700062,0x4f711c0e,0x4f721c10,0x4f73083f,
	0x4f740497,0x4f751c12,0x4f760802,0x4f771c14,0x4f781c16,0x4f791c18,0x4f7a1c1a,0x4f7b133d,
	0x4f7c08cd,0x4f7d1c1c,0x4f7e16fa,0x4f7f11a8,0x4f801c1e,0x4f811c20,0x4f821c22,0x4f8309fd,
	0x4f841942,0x4f851c24,0x4f861c26,0x4f871c28,0x4f88025b,0x4f890a5e,0x4f8a1c2a,0x4f8b0b46,
	0x4f8c1c2c,0x4f8d11b6,0x4f8e1c2e,0x4f8f19a6,0x4f901c30,0x4f9117a6,0x4f921c32,0x4f931c34,
	0x4f940d85,0x4f951c36,0x4f961c38,0x4f970409,0x4f981c3a,0x4f991c3c,0x4f9a1c3e,0x4f9b05e9,
	0x4f9c1c40,0x4f9d16bf,0x4f9e1c42,0x4f9f1c44,0x4fa014cf,0x4fa11c46,0x4fa21c48,0x4fa30c72,
	0x4fa41c4a,0x4fa508ce,0x4fa618fe,0x4fa701a8,0x4fa80fbf,0x4fa90a65,0x4faa01ca,0x4fab1c4c,
	0x4fac0e21,0x4fad1c4e,0x4fae1469,0x4faf0710,0x4fb01c50,0x4fb11c52,0x4fb21c54,0x4fb31c56,
	0x4fb41c58,0x4fb50fde,0x4fb61c5a,0x4fb71c5c,0x4fb81c5e,0x4fb91c60,0x4fba1c62,0x4fbb1c64,
	0x4fbc1c66,0x4fbd1c68,0x4fbe1c6a,0x4fbf0115,0x4fc01c6c,0x4fc11c6e,0x4fc21c70,0x4fc302f4,
	0x4fc4046c,0x4fc51020,0x4fc61c72,0x4fc71c74,0x4fc81c76,0x4fc91c78,0x4fca09db,0x4fcb1c7a,
	0x4fcc1c7c,0x4fcd1c7e,0x4fce1a4c,0x4fcf0fcb,0x4fd00b4b,0x4fd1177d,0x4fd21c80,0x4fd31c82,
	0x4fd41c84,0x4fd51c86,0x4fd61c88,0x4fd7126e,0x4fd8052b,0x4fd91c8a,0x4fda0b2f,0x4fdb1c8c,
	0x4fdc0ef7,0x4fdd0096,0x4fde17bb,0x4fdf0f5a,0x4fe01c8e,0x4fe1157b,0x4fe21c90,0x4fe317da,
	0x4fe41c92,0x4fe51c94,0x4fe60279,0x4fe71c96,0x4fe8164d,0x4fe90b63,0x4fea0b4c,0x4feb1c98,
	0x4fec1c9a,0x4fed0879,0x4fee159c,0x4fef0549,0x4ff01c9c,0x4ff1099b,0x4ff21c9e,0x4ff30e56,
	0x4ff41ca0,0x4ff51ca2,0x4ff61ca4,0x4ff71ca6,0x4ff80512,0x4ff91ca8,0x4ffa0020,0x4ffb1caa,
	0x4ffc1cac,0x4ffd1cae,0x4ffe00d7,0x4fff1cb0,0x50001cb2,0x50011cb4,0x50021cb6,0x50031cb8,
	0x50041cba,0x50051cbc,0x50061cbe,0x50071cc0,0x50081cc2,0x50091cc4,0x500a1cc6,0x500b1cc8,
	0x500c0649,0x500d00b1,0x500e1cca,0x500f11e4,0x50101ccc,0x50111cce,0x50120374,0x50131cd0,
	0x501409c2,0x50151cd2,0x50161cd4,0x50171cd6,0x501812f7,0x5019071c,0x501a16e7,0x501b1cd8,
	0x501c1328,0x501d1cda,0x501e1cdc,0x501f090b,0x50201cde,0x50210202,0x50221ce0,0x50231ce2,
	0x50241ce4,0x50250a41,0x502609b2,0x50271ce6,0x5028099c,0x50290f9d,0x502a0ddc,0x502b1ce8,
	0x502c19f9,0x502d143f,0x502e0c60,0x502f1cea,0x50301cec,0x50311cee,0x50321cf0,0x50331cf2,
	0x50341cf4,0x50351cf6,0x50361cf8,0x50371cfa,0x50381cfc,0x50391cfe,0x503a18ab,0x503b1d00,
	0x503c1944,0x503d1d02,0x503e0ff6,0x503f1d04,0x50401d06,0x50411d08,0x50421d0a,0x5043164f,
	0x50441d0c,0x50451d0e,0x50461d10,0x5047085a,0x5048082c,0x50491d12,0x504a1d14,0x504b1d16,
	0x504c10d1,0x504d1d18,0x504e13f1,0x504f0ed1,0x50501d1a,0x50511d1c,0x50521d1e,0x50531d20,
	0x50541d22,0x5055154f,0x50561d24,0x50571d26,0x50581d28,0x50591d2a,0x505a1a6a,0x505b1d2c,
	0x505c1358,0x505d1d2e,0x505e1d30,0x505f1d32,0x50601d34,0x50611d36,0x50621d38,0x50631d3a,
	0x50641d3c,0x50650895,0x50661d3e,0x50671d40,0x50681d42,0x50691d44,0x506a1d46,0x506b1d48,
	0x506c1a37,0x506d1d4a,0x506e1d4c,0x506f1d4e,0x50701d50,0x50711d52,0x50721d54,0x50731d56,
	0x50741d58,0x50751d5a,0x50760e46,0x50771376,0x50781d5c,0x50791d5e,0x507a1d60,0x507b0c08,
	0x507c1d62,0x507d1d64,0x507e04f9,0x507f01f6,0x50800a8f,0x50811d66,0x50821d68,0x50831d6a,
	0x50841d6c,0x5085055e,0x50861d6e,0x50871d70,0x50880b5c,0x50891d72,0x508a1d74,0x508b1d76,
	0x508c1d78,0x508d0085,0x508e1d7a,0x508f1d7c,0x50901d7e,0x50911d80,0x50921d82,0x50931d84,
	0x50941d86,0x50951d88,0x50961d8a,0x50971d8c,0x50981d8e,0x50991d90,0x509a1d92,0x509b1d94,
	0x509c1d96,0x509d1d98,0x509e1d9a,0x509f1d9c,0x50a01d9e,0x50a11da0,0x50a21da2,0x50a30336,
	0x50a41da4,0x50a512f9,0x50a61da6,0x50a70132,0x50a80296,0x50a90e32,0x50aa1da8,0x50ab1daa,
	0x50ac0304,0x50ad1dac,0x50ae1dae,0x50af1db0,0x50b01db2,0x50b11db4,0x50b20040,0x50b31db6,
	0x50b41db8,0x50b51dba,0x50b61dbc,0x50b71dbe,0x50b81dc0,0x50b91dc2,0x50ba0269,0x50bb110a,
	0x50bc1dc4,0x50bd1dc6,0x50be1dc8,0x50bf1dca,0x50c01dcc,0x50c11dce,0x50c21dd0,0x50c31dd2,
	0x50c41dd4,0x50c51dd6,0x50c61dd8,0x50c71dda,0x50c81ddc,0x50c91dde,0x50ca1de0,0x50cb1de2,
	0x50cc1de4,0x50cd1de6,0x50ce1de8,0x50cf1527,0x50d01dea,0x50d11dec,0x50d21dee,0x50d31df0,
	0x50d41df2,0x50d51df4,0x50d614a4,0x50d71df6,0x50d81df8,0x50d91dfa,0x50da0b92,0x50db1dfc,
	0x50dc1dfe,0x50dd1e00,0x50de1e02,0x50df1e04,0x50e01e06,0x50e11e08,0x50e21e0a,0x50e31e0c,
	0x50e41e0e,0x50e51e10,0x50e60977,0x50e710fd,0x50e81e12,0x50e91e14,0x50ea1e16,0x50eb1e18,
	0x50ec08c6,0x50ed08a1,0x50ee136d,0x50ef1e1a,0x50f01e1c,0x50f11e1e,0x50f21e20,0x50f3127c,
	0x50f41e22,0x50f508aa,0x50f61e24,0x50f71e26,0x50f81e28,0x50f91e2a,0x50fa1e2c,0x50fb0ece,
	0x50fc1e2e,0x50fd1e30,0x50fe1e32,0x50ff1e34,0x51001e36,0x51011e38,0x51021e3a,0x51031e3c,
	0x51041e3e,0x51051e40,0x51060947,0x510715d3,0x51081e42,0x51091e44,0x510a1e46,0x510b0351,
	0x510c1e48,0x510d1e4a,0x510e1e4c,0x510f1e4e,0x51101e50,0x51111e52,0x511210b2,0x51131e54,
	0x51141e56,0x51151e58,0x51161e5a,0x51171e5c,0x51181e5e,0x51191e60,0x511a1e62,0x511b1e64,
	0x511c1e66,0x511d1e68,0x511e1e6a,0x511f1e6c,0x51201e6e,0x51210b0b,0x51221e70,0x51231e72,
	0x51241e74,0x51251e76,0x51261e78,0x51271e7a,0x51281e7c,0x51291e7e,0x512a1e80,0x512b1e82,
	0x512c1e84,0x512d1e86,0x512e1e88,0x512f1e8a,0x51301e8c,0x51311e8e,0x51321e90,0x51331e92,
	0x51341e94,0x51351e96,0x51361e98,0x51371e9a,0x51381e9c,0x51391e9e,0x513a1ea0,0x513b1ea2,
	0x513c1ea4,0x513d1ea6,0x513e1ea8,0x513f048c,0x51401470,0x5141183d,0x51421eaa,0x5143180b,
	0x51441594,0x5145026b,0x514618e0,0x51471eac,0x514814e0,0x51490657,0x514a1eae,0x514b0a2e,
	0x514c1eb0,0x514d0d26,0x514e1eb2,0x514f1eb4,0x51501eb6,0x51510442,0x51521eb8,0x51531eba,
	0x51541389,0x51551243,0x5156164b,0x51571ebc,0x51581ebe,0x51591ec0,0x515a0365,0x515b1ec2,
	0x515c0413,0x515d1ec4,0x515e1ec6,0x515f1ec8,0x51601eca,0x51611ecc,0x5162093e,0x51631ece,
	0x51641ed0,0x516510bd,0x51661ed2,0x51671ed4,0x5168104e,0x51691ed6,0x516a1ed8,0x516b0045,
	0x516c05e6,0x516d0bf4,0x516e1485,0x516f1eda,0x51700ac1,0x517105f6,0x51721edc,0x51730646,
	0x5174157d,0x51750142,0x51760f43,0x51770998,0x517803bb,0x51791a0e,0x517a1ede,0x517b167e,
	0x517c0868,0x517d11d8,0x517e1ee0,0x517f1ee2,0x51800838,0x518101e6,0x5182095a,0x51831ee4,
	0x51841ee6,0x51850dd3,0x51861ee8,0x51871eea,0x51880593,0x51891074,0x518a1eec,0x518b1eee,
	0x518c01a7,0x518d1858,0x518e1ef0,0x518f1ef2,0x51901ef4,0x51911ef6,0x51920cc6,0x51931ef8,
	0x51941efa,0x51950d2a,0x51960d16,0x519710a8,0x51981efc,0x51991557,0x519a1efe,0x519b09d3,
	0x519c0e20,0x519d1f00,0x519e1f02,0x519f1f04,0x51a00648,0x51a11f06,0x51a21989,0x51a31f08,
	0x51a41807,0x51a50d5b,0x51a61f0a,0x51a71f0c,0x51a81f0e,0x51a91f10,0x51aa1f12,0x51ab0140,
	0x51ac0400,0x51ad1f14,0x51ae1f16,0x51af050b,0x51b00141,0x51b1073f,0x51b2026c,0x51b309bc,
	0x51b41f18,0x51b50a78,0x51b616ab,0x51b70b14,0x51b81f1a,0x51b91f1c,0x51ba1f1e,0x51bb0408,
	0x51bc14f7,0x51bd0ba4,0x51be1f20,0x51bf1f22,0x51c0094a,0x51c11f24,0x51c21f26,0x51c31f28,
	0x51c40f31,0x51c51f2a,0x51c619f6,0x51c7124f,0x51c81f2c,0x51c90b7f,0x51ca1f2e,0x51cb03d1,
	0x51cc0bce,0x51cd1f30,0x51ce1f32,0x51cf087e,0x51d01f34,0x51d102ed,0x51d21f36,0x51d31f38,
	0x51d41f3a,0x51d51f3c,0x51d61f3e,0x51d71f40,0x51d81f42,0x51d91f44,0x51da1f46,0x51db0bbd,
	0x51dc1f48,0x51dd0e14,0x51de1f4a,0x51df1f4c,0x51e007f8,0x51e104a9,0x51e21f4e,0x51e31f50,
	0x51e40510,0x51e51f52,0x51e61f54,0x51e71f56,0x51e81f58,0x51e91f5a,0x51ea1f5c,0x51eb0521,
	0x51ec1f5e,0x51ed0efa,0x51ee1f60,0x51ef09ec,0x51f0077e,0x51f11f62,0x51f21f64,0x51f3038b,
	0x51f41f66,0x51f50a00,0x51f61593,0x51f71f68,0x51f8137c,0x51f9002e,0x51fa0287,0x51fb07d8,
	0x51fc0367,0x51fd06a8,0x51fe1f6a,0x51ff186b,0x5200036d,0x520103cf,0x5202036e,0x5203108d,
	0x52041f6c,0x52051f6e,0x520604ea,0x52070fd1,0x520816ed,0x52091f70,0x520a09f6,0x520b1f72,
	0x520c1f74,0x520d028a,0x520e1432,0x520f1f76,0x52101f78,0x52111583,0x5212074b,0x52131f7a,
	0x52141f7c,0x52151f7e,0x5216182a,0x52170ba2,0x52180be3,0x52191878,0x521a0594,0x521b02b6,
	0x521c1f80,0x521d0288,0x521e1f82,0x521f1f84,0x52201114,0x52211f86,0x52221f88,0x52231f8a,
	0x52240e67,0x52251f8c,0x52261f8e,0x52271f90,0x52280e79,0x52290b40,0x522a1f92,0x522b012e,
	0x522c1f94,0x522d0943,0x522e0638,0x522f1f96,0x52300378,0x52311f98,0x52321f9a,0x52330a50,
	0x52341f9c,0x52351f9e,0x5236195e,0x5237120a,0x52381061,0x52391101,0x523a02df,0x523b0a2f,
	0x523c1fa0,0x523d0671,0x523e1fa2,0x523f0672,0x524009ed,0x52410463,0x52420824,0x52431327,
	0x52441fa4,0x52451fa6,0x52461fa8,0x52471faa,0x52481fac,0x52491fae,0x524a15e5,0x524b1fb0,
	0x524c0aaf,0x524d0f89,0x524e1fb2,0x524f1fb4,0x5250063c,0x52510891,0x52521fb6,0x52531fb8,
	0x52541319,0x52551fba,0x52560f11,0x52571fbc,0x52581fbe,0x52591fc0,0x525a1fc2,0x525b1fc4,
	0x525c13c5,0x525d1fc6,0x525e07e4,0x525f1fc8,0x52601fca,0x5261164a,0x52621fcc,0x52631fce,
	0x52641fd0,0x52650150,0x52661fd2,0x5267099d,0x52681fd4,0x5269118c,0x526a087f,0x526b1fd6,
	0x526c1fd8,0x526d1fda,0x526e1fdc,0x526f055d,0x52701fde,0x52711fe0,0x527205ba,0x52731fe2,
	0x52741fe4,0x52751fe6,0x52761fe8,0x52771fea,0x52781fec,0x52791fee,0x527a1ff0,0x527b1ff2,
	0x527c1ff4,0x527d0edb,0x527e1ff6,0x527f08d9,0x52801ff8,0x52810fb8,0x528209c8,0x52831ffa,
	0x52841ffc,0x52851ffe,0x52862000,0x52872002,0x52880eae,0x52892004,0x528a2006,0x528b2008,
	0x528c200a,0x528d200c,0x528e200e,0x528f2010,0x529007c3,0x52912012,0x52922014,0x52931718,
	0x52942016,0x52952018,0x5296201a,0x5297201c,0x5298201e,0x52992020,0x529a2022,0x529b0b2a,
	0x529c2024,0x529d1060,0x529e0076,0x529f05e7,0x52a0083d,0x52a11472,0x52a20c94,0x52a30ba3,
	0x52a42026,0x52a52028,0x52a6202a,0x52a7202c,0x52a80407,0x52a919c5,0x52aa0e2a,0x52ab08f2,
	0x52ac1039,0x52ad114a,0x52ae202e,0x52af2030,0x52b02032,0x52b10b41,0x52b20925,0x52b30ae7,
	0x52b42034,0x52b52036,0x52b62038,0x52b7203a,0x52b8203c,0x52b9203e,0x52ba2040,0x52bb2042,
	0x52bc2044,0x52bd2046,0x52be06dd,0x52bf11b7,0x52c02048,0x52c1204a,0x52c2204c,0x52c3015a,
	0x52c4204e,0x52c52050,0x52c62052,0x52c7177e,0x52c82054,0x52c90d28,0x52ca2056,0x52cb15f2,
	0x52cc2058,0x52cd205a,0x52ce205c,0x52cf205e,0x52d00cfc,0x52d12060,0x52d20afb,0x52d32062,
	0x52d42064,0x52d52066,0x52d615c1,0x52d72068,0x52d809f7,0x52d9206a,0x52da206c,0x52db206e,
	0x52dc2070,0x52dd2072,0x52de2074,0x52df0d98,0x52e02076,0x52e12078,0x52e2207a,0x52e3207c,
	0x52e40fe6,0x52e5207e,0x52e62080,0x52e72082,0x52e82084,0x52e92086,0x52ea2088,0x52eb208a,
	0x52ec208c,0x52ed208e,0x52ee2090,0x52ef2092,0x52f01553,0x52f12094,0x52f22096,0x52f32098,
	0x52f4209a,0x52f5209c,0x52f6209e,0x52f720a0,0x52f820a2,0x52f9008b,0x52fa1145,0x52fb20a4,
	0x52fc20a6,0x52fd20a8,0x52fe05f8,0x52ff1471,0x53001837,0x530120aa,0x530220ac,0x530320ae,
	0x530420b0,0x5305008c,0x530602e2,0x530720b2,0x53081595,0x530920b4,0x530a20b6,0x530b20b8,
	0x530c20ba,0x530d0f1a,0x530e20bc,0x530f0e7f,0x53100535,0x531120be,0x531220c0,0x531320c2,
	0x531420c4,0x531500d1,0x53160753,0x531700aa,0x531820c6,0x531911ce,0x531a04bc,0x531b20c8,
	0x531c20ca,0x531d184c,0x531e20cc,0x531f20ce,0x532008b3,0x53210a6e,0x532220d0,0x532314ce,
	0x532420d2,0x532520d4,0x5326066b,0x532720d6,0x532820d8,0x532920da,0x532a04db,0x532b20dc,
	0x532c20de,0x532d20e0,0x532e0a91,0x532f20e2,0x533020e4,0x533120e6,0x533220e8,0x533320ea,
	0x533420ec,0x533520ee,0x533620f0,0x533720f2,0x533820f4,0x53390ec1,0x533a102b,0x533b16be,
	0x533c20f6,0x533d20f8,0x533e010d,0x533f0de7,0x534020fa,0x53411199,0x534220fc,0x53430f77,
	0x534420fe,0x534510d7,0x53462100,0x5347117e,0x53481461,0x534907a4,0x534a0077,0x534b2102,
	0x534c2104,0x534d2106,0x534e074d,0x534f154b,0x53502108,0x535100a4,0x53521a46,0x535319fd,
	0x5354210a,0x53550349,0x53560c97,0x53570db6,0x5358210c,0x5359210e,0x535a015f,0x535b2110,
	0x535c0171,0x535d2112,0x535e0110,0x535f0172,0x536018b6,0x536109e5,0x53620c18,0x5363179b,
	0x53640c24,0x53652114,0x5366063e,0x53671447,0x53682116,0x536908ee,0x536a2118,0x536b141b,
	0x536c211a,0x536d211c,0x536e1932,0x536f0cc0,0x53701740,0x537113ef,0x5372211e,0x537307ff,
	0x53741065,0x53750c48,0x53762120,0x537709b1,0x5378155b,0x53792122,0x537a091c,0x537b2124,
	0x537c2126,0x537d2128,0x537e212a,0x537f0ff7,0x5380212c,0x5381212e,0x538201fb,0x53832130,
	0x53840475,0x53851350,0x53860b3b,0x53872132,0x53882134,0x53890b3c,0x538a2136,0x538b1610,
	0x538c1659,0x538d1158,0x538e2138,0x538f213a,0x5390213c,0x5391213e,0x53922140,0x53932142,
	0x53942144,0x539501a9,0x53962146,0x53972148,0x53980b17,0x5399214a,0x539a0719,0x539b214c,
	0x539c214e,0x539d031f,0x539e2150,0x539f1811,0x53a02152,0x53a12154,0x53a21511,0x53a31650,
	0x53a42156,0x53a509c7,0x53a6110f,0x53a72158,0x53a8028c,0x53a90973,0x53aa215a,0x53ab215c,
	0x53ac215e,0x53ad2160,0x53ae1237,0x53af2162,0x53b02164,0x53b12166,0x53b22168,0x53b3216a,
	0x53b4216c,0x53b5216e,0x53b6122c,0x53b72170,0x53b82172,0x53b92174,0x53ba2176,0x53bb1048,
	0x53bc2178,0x53bd217a,0x53be217c,0x53bf1500,0x53c0217e,0x53c110e1,0x53c2018d,0x53c32180,
	0x53c42182,0x53c52184,0x53c62186,0x53c72188,0x53c817a2,0x53c901b2,0x53ca07fa,0x53cb1799,
	0x53cc1216,0x53cd04b3,0x53ce218a,0x53cf218c,0x53d0218e,0x53d1049a,0x53d22190,0x53d32192,
	0x53d411e1,0x53d52194,0x53d61044,0x53d711d6,0x53d80116,0x53d915be,0x53da2196,0x53db0e69,
	0x53dc2198,0x53dd219a,0x53de219c,0x53df1265,0x53e003e5,0x53e1219e,0x53e221a0,0x53e30a4a,
	0x53e4061e,0x53e50993,0x53e60be0,0x53e721a2,0x53e8036f,0x53e90a4b,0x53ea193f,0x53eb08dd,
	0x53ec18df,0x53ed0047,0x53ee03ed,0x53ef0a2b,0x53f012bf,0x53f10261,0x53f211a5,0x53f317a3,
	0x53f421a4,0x53f50f09,0x53f616ae,0x53f706cf,0x53f8122f,0x53f912e2,0x53fa21a6,0x53fb0af9,
	0x53fc03d0,0x53fd07d9,0x53fe21a8,0x53ff21aa,0x540021ac,0x540115b4,0x540221ae,0x54030246,
	0x540405ce,0x540521b0,0x54061687,0x540721b2,0x540806db,0x540907fb,0x540a03d6,0x540b21b4,
	0x540c1364,0x540d0d57,0x540e0718,0x540f0b3e,0x54101387,0x54111523,0x5412189d,0x541314dc,
	0x541421b6,0x54150c71,0x5416160e,0x54170c85,0x541821b8,0x541921ba,0x541a21bc,0x541b09d4,
	0x541c21be,0x541d0bc1,0x541e1398,0x541f172d,0x542004e1,0x542100d3,0x542221c0,0x54230ff0,
	0x542421c2,0x542521c4,0x54260515,0x54270049,0x54280447,0x542904eb,0x542a21c6,0x542b06a6,
	0x542c1352,0x542d0a3d,0x542e121f,0x542f0f60,0x543021c8,0x54311935,0x5432173a,0x543321ca,
	0x54341459,0x5435020e,0x543621cc,0x543721ce,0x54381488,0x543902b8,0x543a21d0,0x543b1433,
	0x543c0717,0x543d21d2,0x543e145a,0x543f21d4,0x54401611,0x544121d6,0x544221d8,0x54430476,
	0x544421da,0x544521dc,0x54460333,0x544721de,0x54480234,0x544921e0,0x544a05ae,0x544b0517,
	0x544c21e2,0x544d21e4,0x544e21e6,0x544f21e8,0x54500daa,0x545121ea,0x54520543,0x545316f5,
	0x54540334,0x54550e45,0x54560b42,0x545700bd,0x5458180c,0x545913c1,0x545a21ec,0x545b0fa2,
	0x545c1452,0x545d21ee,0x545e21f0,0x545f21f2,0x546021f4,0x546121f6,0x54620dd1,0x546321f8,
	0x54640bdd,0x546521fa,0x54661787,0x546721fc,0x54681992,0x546921fe,0x546a2200,0x546b2202,
	0x546c2204,0x546d2206,0x546e2208,0x546f220a,0x5470220c,0x5471061b,0x54721a0b,0x5473141e,
	0x5474220e,0x547506d7,0x54760dc3,0x547714cb,0x54780e83,0x54792210,0x547a2212,0x547b1166,
	0x547c0721,0x547d0d62,0x547e2214,0x547f2216,0x5480098a,0x54812218,0x5482184d,0x5483221a,
	0x54840457,0x5485221c,0x54860e7a,0x5487221e,0x54882220,0x54892222,0x548a2224,0x548b189e,
	0x548c06de,0x548d2226,0x548e096f,0x548f177b,0x54900558,0x54912228,0x5492199a,0x5493222a,
	0x549409e4,0x5495060e,0x549609e2,0x5497222c,0x5498222e,0x54990bf7,0x549a0401,0x549b0e0f,
	0x549c2230,0x549d1231,0x549e2232,0x549f2234,0x54a02236,0x54a12238,0x54a2223a,0x54a30658,
	0x54a418a1,0x54a5223c,0x54a616cf,0x54a70ba1,0x54a81a0f,0x54a90d42,0x54aa0d05,0x54ab1954,
	0x54ac169a,0x54ad07e3,0x54ae223e,0x54af09e8,0x54b02240,0x54b1185c,0x54b22242,0x54b30a2a,
	0x54b40793,0x54b52244,0x54b62246,0x54b72248,0x54b814ef,0x54b9224a,0x54ba224c,0x54bb159d,
	0x54bc224e,0x54bd162b,0x54be2250,0x54bf16c0,0x54c00004,0x54c10ef1,0x54c2116c,0x54c32252,
	0x54c40700,0x54c52254,0x54c60458,0x54c713b7,0x54c80694,0x54c91853,0x54ca2256,0x54cb2258,
	0x54cc0e5a,0x54cd151e,0x54ce0003,0x54cf05d5,0x54d00a70,0x54d11620,0x54d20324,0x54d3152c,
	0x54d400e1,0x54d51828,0x54d6225a,0x54d7074e,0x54d8225c,0x54d90a67,0x54da045f,0x54db225e,
	0x54dc0826,0x54dd0e22,0x54de0d82,0x54df1769,0x54e02260,0x54e12262,0x54e22264,0x54e32266,
	0x54e42268,0x54e505b6,0x54e60e3f,0x54e70247,0x54e8114d,0x54e90b62,0x54ea0da2,0x54eb226a,
	0x54ec226c,0x54ed0a52,0x54ee1542,0x54ef226e,0x54f02270,0x54f12272,0x54f218eb,0x54f31890,
	0x54f42274,0x54f52276,0x54f62278,0x54f7227a,0x54f8227c,0x54f9227e,0x54fa0174,0x54fb2280,
	0x54fc06f8,0x54fd05de,0x54fe2282,0x54ff05cb,0x55002284,0x5501165c,0x55022286,0x55032288,
	0x5504228a,0x5505228c,0x5506129e,0x550702c4,0x5508228e,0x55090005,0x550a2290,0x550b2292,
	0x550c2294,0x550d2296,0x550e2298,0x550f148f,0x551012e9,0x55111a68,0x5512229a,0x5513229c,
	0x55140c82,0x5515229e,0x551622a0,0x551722a2,0x551822a4,0x551922a6,0x551a22a8,0x551b0c8d,
	0x551c22aa,0x551d22ac,0x551e22ae,0x551f22b0,0x55200af2,0x552122b2,0x552212a8,0x55231873,
	0x5524076d,0x552522b4,0x552622b6,0x552707e5,0x552822b8,0x552922ba,0x552a050f,0x552b22bc,
	0x552c073b,0x552d22be,0x552e11d9,0x552f1405,0x5530120b,0x55310204,0x553222c0,0x55330b57,
	0x553422c2,0x553522c4,0x553622c6,0x5537176a,0x553822c8,0x553922ca,0x553a22cc,0x553b22ce,
	0x553c110c,0x553d22d0,0x553e13b5,0x553f0725,0x554022d2,0x5541198e,0x554222d4,0x55430a3b,
	0x55441a05,0x554522d6,0x55461133,0x554722d8,0x554822da,0x55490ace,0x554a0001,0x554b22dc,
	0x554c22de,0x554d22e0,0x554e22e2,0x554f22e4,0x55500309,0x555122e6,0x555222e8,0x555322ea,
	0x555422ec,0x55551309,0x5556035a,0x555722ee,0x555822f0,0x555922f2,0x555a22f4,0x555b22f6,
	0x555c02cc,0x555d22f8,0x555e22fa,0x555f22fc,0x556022fe,0x556104d0,0x55622300,0x55632302,
	0x55640eba,0x55651109,0x55660ab4,0x5567187d,0x55682304,0x55692306,0x556a0e4c,0x556b2308,
	0x556c10f8,0x556d19dc,0x556e0e05,0x556f230a,0x5570230c,0x5571230e,0x55722310,0x55732312,
	0x55742314,0x5575016c,0x557603f9,0x55770ad8,0x55781546,0x55792316,0x557a2318,0x557b0268,
	0x557c131e,0x557d231a,0x557e0964,0x557f231c,0x558009e3,0x55811778,0x55821424,0x55830db8,
	0x55841127,0x5585231e,0x55862320,0x55870aae,0x558808ea,0x55890711,0x558a06af,0x558b03e2,
	0x558c2322,0x558d2324,0x558e2326,0x558f0e34,0x55902328,0x5591172b,0x5592232a,0x5593232c,
	0x55940e3e,0x5595232e,0x55962330,0x55972332,0x559802ae,0x559907b2,0x559a2334,0x559b2336,
	0x559c14be,0x559d06d8,0x559e2338,0x559f0a92,0x55a0233a,0x55a1233c,0x55a2233e,0x55a32340,
	0x55a42342,0x55a52344,0x55a62346,0x55a715ce,0x55a82348,0x55a9234a,0x55aa234c,0x55ab234e,
	0x55ac2350,0x55ad2352,0x55ae2354,0x55af2356,0x55b02358,0x55b10b1d,0x55b2235a,0x55b31891,
	0x55b4235c,0x55b50d30,0x55b6235e,0x55b70e92,0x55b82360,0x55b90a86,0x55ba2362,0x55bb17f6,
	0x55bc2364,0x55bd0c09,0x55be0a59,0x55bf2366,0x55c02368,0x55c1236a,0x55c2236c,0x55c3236e,
	0x55c4110b,0x55c515ab,0x55c62370,0x55c72372,0x55c82374,0x55c91278,0x55ca2376,0x55cb2378,
	0x55cc0014,0x55cd129d,0x55ce237a,0x55cf237c,0x55d0237e,0x55d10a28,0x55d20327,0x55d310e9,
	0x55d4021d,0x55d52380,0x55d6125d,0x55d72382,0x55d82384,0x55d92386,0x55da2388,0x55db238a,
	0x55dc11c9,0x55dd05c5,0x55de238c,0x55df08eb,0x55e0238e,0x55e1143a,0x55e22390,0x55e3124b,
	0x55e4024c,0x55e506c8,0x55e612ac,0x55e72392,0x55e80695,0x55e92394,0x55ea0fe7,0x55eb0e06,
	0x55ec06d9,0x55ed2396,0x55ee2398,0x55ef0d9f,0x55f0239a,0x55f1239c,0x55f203b5,0x55f3000c,
	0x55f4239e,0x55f51362,0x55f623a0,0x55f70031,0x55f823a2,0x55f923a4,0x55fa23a6,0x55fb23a8,
	0x55fc23aa,0x55fd1269,0x55fe1266,0x55ff23ac,0x56000395,0x56010f38,0x560223ae,0x560323b0,
	0x560423b2,0x560523b4,0x560623b6,0x560723b8,0x560801a1,0x5609084a,0x560a23ba,0x560b23bc,
	0x560c0edf,0x560d23be,0x560e056a,0x560f062a,0x561023c0,0x561123c2,0x561223c4,0x561323c6,
	0x561423c8,0x561523ca,0x561623cc,0x561723ce,0x561815b1,0x561923d0,0x561a23d2,0x561b0c8f,
	0x561c23d4,0x561d23d6,0x561e0b10,0x561f041f,0x562023d8,0x562123da,0x562223dc,0x562300c8,
	0x56241747,0x562523de,0x562623e0,0x56270d20,0x562823e2,0x562923e4,0x562a23e6,0x562b23e8,
	0x562c1a5d,0x562d0e99,0x562e23ea,0x562f23ec,0x563023ee,0x563119c0,0x5632020c,0x563323f0,
	0x56341a53,0x563523f2,0x56361239,0x563723f4,0x563823f6,0x56390b94,0x563a23f8,0x563b14a8,
	0x563c23fa,0x563d23fc,0x563e23fe,0x563f06f2,0x56402400,0x56412402,0x56422404,0x56432406,
	0x56442408,0x5645240a,0x5646240c,0x5647240e,0x56482410,0x56492412,0x564a2414,0x564b2416,
	0x564c01af,0x564d08e3,0x564e16a5,0x564f2418,0x5650241a,0x5651241c,0x5652241e,0x56532420,
	0x56540385,0x56552422,0x56562424,0x56570f19,0x565809b9,0x56590fe9,0x565a2426,0x565b2428,
	0x565c0c16,0x565d242a,0x565e242c,0x565f242e,0x56602430,0x56612432,0x56620e3d,0x56632434,
	0x56640931,0x56652436,0x56662438,0x5667243a,0x56680f70,0x56690486,0x566a1875,0x566b16c5,
	0x566c11cc,0x566d243c,0x566e243e,0x566f2440,0x56702442,0x567115e8,0x56722444,0x56732446,
	0x56742448,0x5675244a,0x5676056d,0x5677244c,0x5678244e,0x56792450,0x567a2452,0x567b10dd,
	0x567c0eaf,0x567d2454,0x567e2456,0x567f2458,0x5680245a,0x5681245c,0x5682245e,0x56832460,
	0x56842462,0x568510b3,0x568606c4,0x56872464,0x56882466,0x56892468,0x568a246a,0x568b246c,
	0x568c246e,0x568d2470,0x568e06ca,0x568f132f,0x56902472,0x56912474,0x56922476,0x5693017f,
	0x56942478,0x5695247a,0x5696247c,0x5697247e,0x56982480,0x56992482,0x569a2484,0x569b2486,
	0x569c2488,0x569d248a,0x569e248c,0x569f248e,0x56a02490,0x56a12492,0x56a22494,0x56a3153a,
	0x56a42496,0x56a52498,0x56a6249a,0x56a7249c,0x56a8249e,0x56a924a0,0x56aa24a2,0x56ab24a4,
	0x56ac24a6,0x56ad24a8,0x56ae24aa,0x56af07d4,0x56b024ac,0x56b124ae,0x56b224b0,0x56b324b2,
	0x56b424b4,0x56b524b6,0x56b624b8,0x56b7107a,0x56b824ba,0x56b924bc,0x56ba24be,0x56bb24c0,
	0x56bc08cb,0x56bd24c2,0x56be24c4,0x56bf24c6,0x56c024c8,0x56c124ca,0x56c224cc,0x56c324ce,
	0x56c424d0,0x56c524d2,0x56c624d4,0x56c724d6,0x56c824d8,0x56c924da,0x56ca0dbe,0x56cb24dc,
	0x56cc24de,0x56cd24e0,0x56ce24e2,0x56cf24e4,0x56d024e6,0x56d124e8,0x56d224ea,0x56d324ec,
	0x56d40dbd,0x56d524ee,0x56d624f0,0x56d713fa,0x56d824f2,0x56d924f4,0x56da101b,0x56db123e,
	0x56dc24f6,0x56dd0876,0x56de079c,0x56df157a,0x56e01721,0x56e10db4,0x56e2138d,0x56e324f8,
	0x56e4044e,0x56e524fa,0x56e624fc,0x56e724fe,0x56e82500,0x56e92502,0x56ea2504,0x56eb0729,
	0x56ec2506,0x56ed180d,0x56ee2508,0x56ef250a,0x56f00aa4,0x56f102e1,0x56f2250c,0x56f3250e,
	0x56f413fe,0x56f50c50,0x56f62510,0x56f72512,0x56f82514,0x56f90bc8,0x56fa062d,0x56fb2516,
	0x56fc2518,0x56fd068a,0x56fe137f,0x56ff17a7,0x5700251a,0x5701251c,0x5702251e,0x57030f23,
	0x570417dd,0x57052520,0x57061812,0x57072522,0x5708104d,0x570917de,0x570a0ff8,0x570b2524,
	0x570c2526,0x570d2528,0x570e252a,0x570f252c,0x5710252e,0x57112530,0x57122532,0x57132534,
	0x57142536,0x57152538,0x5716253a,0x5717253c,0x5718253e,0x57192540,0x571a2542,0x571b2544,
	0x571c0768,0x571d2546,0x571e2548,0x571f1386,0x5720254a,0x5721254c,0x5722254e,0x57231189,
	0x57242550,0x57252552,0x57262554,0x57272556,0x57281859,0x572913fd,0x572a05b3,0x572b2558,
	0x572c144f,0x572d065f,0x572e0ec4,0x572f16c8,0x573003a9,0x5731255a,0x5732255c,0x57331914,
	0x5734255e,0x57352560,0x57362562,0x57372564,0x57382566,0x57390a76,0x573a01fa,0x573b0f40,
	0x573c2568,0x573d256a,0x573e07dc,0x573f256c,0x57401950,0x5741256e,0x57420071,0x57432570,
	0x57442572,0x57452574,0x57462576,0x574709d5,0x57482578,0x5749257a,0x574a04bf,0x574b257c,
	0x574c00c4,0x574d12ce,0x574e09fc,0x574f075c,0x57501a62,0x57510a3e,0x5752257e,0x57532580,
	0x57542582,0x57552584,0x57562586,0x57570a63,0x57582588,0x5759258a,0x575a0863,0x575b12d4,
	0x575c0b43,0x575d0058,0x575e146e,0x575f04f0,0x576019ee,0x57610f03,0x5762258c,0x5763258e,
	0x57640a9a,0x57652590,0x576612de,0x57672592,0x576813a7,0x5769057f,0x576a0efb,0x576b03c3,
	0x576c2594,0x576d0dd9,0x576e2596,0x576f0eab,0x57702598,0x5771259a,0x5772259c,0x5773003e,
	0x5774259e,0x577525a0,0x57760d94,0x57770a17,0x577825a2,0x577925a4,0x577a25a6,0x577b03a3,
	0x577c0216,0x577d25a8,0x577e25aa,0x577f25ac,0x578025ae,0x578125b0,0x578202ba,0x57830aaa,
	0x57840c04,0x57850c05,0x57860c1b,0x578725b2,0x578825b4,0x578925b6,0x578a25b8,0x578b1588,
	0x578c040a,0x578d25ba,0x578e25bc,0x578f25be,0x579025c0,0x579125c2,0x57920b08,0x57930572,
	0x579425c4,0x579525c6,0x579625c8,0x579725ca,0x579825cc,0x579925ce,0x579a25d0,0x579b0460,
	0x579c25d2,0x579d25d4,0x579e25d6,0x579f25d8,0x57a0172e,0x57a1049d,0x57a20607,0x57a3180f,
	0x57a403df,0x57a525da,0x57a60a39,0x57a71137,0x57a825dc,0x57a9047a,0x57aa25de,0x57ab03c5,
	0x57ac25e0,0x57ad1626,0x57ae0a5d,0x57af25e2,0x57b025e4,0x57b125e6,0x57b209ee,0x57b325e8,
	0x57b40dc9,0x57b525ea,0x57b625ec,0x57b725ee,0x57b81821,0x57b925f0,0x57ba25f2,0x57bb25f4,
	0x57bc25f6,0x57bd25f8,0x57be25fa,0x57bf25fc,0x57c025fe,0x57c12600,0x57c205df,0x57c30006,
	0x57c42602,0x57c52604,0x57c62606,0x57c72608,0x57c8260a,0x57c9260c,0x57ca260e,0x57cb0c90,
	0x57cc2610,0x57cd2612,0x57ce0238,0x57cf1638,0x57d02614,0x57d12616,0x57d20ba6,0x57d32618,
	0x57d40f24,0x57d5023a,0x57d6261a,0x57d7261c,0x57d811a3,0x57d915f3,0x57da0683,0x57db261e,
	0x57dc2620,0x57dd0df7,0x57de2622,0x57df17f2,0x57e0017c,0x57e12624,0x57e22626,0x57e32628,
	0x57e40ebb,0x57e5262a,0x57e6262c,0x57e7262e,0x57e82630,0x57e92632,0x57ea2634,0x57eb2636,
	0x57ec2638,0x57ed0342,0x57ee263a,0x57ef0021,0x57f0263c,0x57f1263e,0x57f22640,0x57f32642,
	0x57f41945,0x57f52644,0x57f62646,0x57f72648,0x57f8170a,0x57f90e87,0x57fa07ea,0x57fb264a,
	0x57fc264c,0x57fd10f4,0x57fe264e,0x57ff2650,0x58000a53,0x58012652,0x580212ea,0x58032654,
	0x58042656,0x58052658,0x5806043f,0x58070919,0x5808265a,0x5809265c,0x580a265e,0x580b0e9b,
	0x580c2660,0x580d138a,0x580e2662,0x580f2664,0x58102666,0x58110f9e,0x58122668,0x5813266a,
	0x5814266c,0x58150464,0x5816266e,0x58172670,0x58182672,0x5819172c,0x581a2674,0x581b2676,
	0x581c2678,0x581d267a,0x581e03e3,0x581f267c,0x5820071d,0x58210098,0x5822267e,0x58232680,
	0x58240392,0x58252682,0x58262684,0x58272686,0x58282688,0x5829268a,0x582a09f9,0x582b268c,
	0x582c268e,0x582d2690,0x582e2692,0x582f2694,0x58301662,0x58312696,0x58322698,0x5833269a,
	0x5834269c,0x5835042b,0x5836269e,0x583726a0,0x583826a2,0x583926a4,0x583a26a6,0x583b26a8,
	0x583c26aa,0x583d26ac,0x583e26ae,0x583f26b0,0x584026b2,0x584126b4,0x584226b6,0x584326b8,
	0x58440b11,0x584526ba,0x584626bc,0x584726be,0x584826c0,0x584926c2,0x584a26c4,0x584b26c6,
	0x584c12b2,0x584d023e,0x584e26c8,0x584f26ca,0x585026cc,0x58511279,0x585226ce,0x585326d0,
	0x585412b4,0x585526d2,0x585626d4,0x585726d6,0x585812ec,0x585926d8,0x585a26da,0x585b26dc,
	0x585c26de,0x585d26e0,0x585e10db,0x585f26e2,0x586026e4,0x586126e6,0x586226e8,0x586326ea,
	0x586426ec,0x586505c6,0x586626ee,0x586726f0,0x586826f2,0x586926f4,0x586a26f6,0x586b1336,
	0x586c1817,0x586d26f8,0x586e26fa,0x586f26fc,0x587026fe,0x58712700,0x58722702,0x58732704,
	0x58742706,0x58752708,0x5876270a,0x5877270c,0x5878270e,0x58792710,0x587a2712,0x587b2714,
	0x587c2716,0x587d2718,0x587e11f2,0x587f271a,0x58800257,0x58810ca3,0x5882271c,0x58830956,
	0x5884271e,0x58851207,0x58862720,0x58872722,0x58882724,0x58891771,0x588a2726,0x588b2728,
	0x588c272a,0x588d272c,0x588e272e,0x588f2730,0x58902732,0x58912734,0x58921135,0x58930d99,
	0x58942736,0x58952738,0x5896273a,0x5897273c,0x5898273e,0x58990fad,0x589a0b84,0x589b2740,
	0x589c2742,0x589d2744,0x589e1889,0x589f15b3,0x58a02746,0x58a12748,0x58a2274a,0x58a3274c,
	0x58a4274e,0x58a52750,0x58a62752,0x58a72754,0x58a80d7c,0x58a90449,0x58aa2756,0x58ab2758,
	0x58ac275a,0x58ad275c,0x58ae275e,0x58af2760,0x58b02762,0x58b12764,0x58b22766,0x58b32768,
	0x58b4276a,0x58b5276c,0x58b6276e,0x58b72770,0x58b82772,0x58b92774,0x58ba2776,0x58bb2778,
	0x58bc07f4,0x58bd277a,0x58be277c,0x58bf277e,0x58c02780,0x58c100f7,0x58c22782,0x58c32784,
	0x58c42786,0x58c51773,0x58c62788,0x58c7278a,0x58c8278c,0x58c9278e,0x58ca2790,0x58cb2792,
	0x58cc2794,0x58cd2796,0x58ce2798,0x58cf279a,0x58d0279c,0x58d106f0,0x58d2279e,0x58d327a0,
	0x58d427a2,0x58d506cb,0x58d627a4,0x58d727a6,0x58d827a8,0x58d927aa,0x58da27ac,0x58db27ae,
	0x58dc27b0,0x58dd27b2,0x58de27b4,0x58df27b6,0x58e027b8,0x58e127ba,0x58e227bc,0x58e327be,
	0x58e4107b,0x58e527c0,0x58e627c2,0x58e727c4,0x58e827c6,0x58e927c8,0x58ea27ca,0x58eb11ad,
	0x58ec1089,0x58ed27cc,0x58ee19e6,0x58ef27ce,0x58f01180,0x58f127d0,0x58f227d2,0x58f30a29,
	0x58f427d4,0x58f527d6,0x58f6072d,0x58f727d8,0x58f827da,0x58f916c2,0x58fa27dc,0x58fb27de,
	0x58fc27e0,0x58fd27e2,0x58fe27e4,0x58ff27e6,0x590027e8,0x590127ea,0x5902194d,0x590327ec,
	0x5904029a,0x590527ee,0x590627f0,0x590700ae,0x590827f2,0x590927f4,0x590a27f6,0x590b27f8,
	0x590c27fa,0x590d055b,0x590e27fc,0x590f14dd,0x591027fe,0x59112800,0x59122802,0x59132804,
	0x59140a8d,0x59151484,0x591613c3,0x59172806,0x59182808,0x5919126f,0x591a0456,0x591b280a,
	0x591c16b2,0x591d280c,0x591e280e,0x591f0608,0x59202810,0x59212812,0x59222814,0x59232816,
	0x59241734,0x592507cb,0x59262818,0x59270332,0x5928281a,0x59291330,0x592a12c7,0x592b0516,
	0x592c281c,0x592d1686,0x592e166c,0x592f06bd,0x5930281e,0x5931118f,0x59322820,0x59332822,
	0x59341378,0x59352824,0x59362826,0x593716c9,0x59380a5c,0x5939083e,0x593a045b,0x593b2828,
	0x593c0a74,0x593d282a,0x593e282c,0x593f282e,0x59402830,0x59410b64,0x5942076b,0x59432832,
	0x5944164c,0x59452834,0x59462836,0x59470f44,0x59480daf,0x59490511,0x594a2838,0x594b04f7,
	0x594c283a,0x594d283c,0x594e0a83,0x594f1a41,0x5950283e,0x59510f6c,0x59522840,0x59532842,
	0x595400be,0x59551701,0x595608af,0x5957130e,0x595819e5,0x59592844,0x595a1490,0x595b2846,
	0x595c2848,0x595d284a,0x595e284c,0x595f284e,0x596003ca,0x59612850,0x5962114f,0x59632852,
	0x59642854,0x59650041,0x59662856,0x59672858,0x5968285a,0x5969285c,0x596a285e,0x596b2860,
	0x596c2862,0x596d2864,0x596e2866,0x596f2868,0x5970286a,0x5971286c,0x5972286e,0x59730e39,
	0x59740e27,0x59752870,0x59760dac,0x59772872,0x59780861,0x597912ad,0x597a2874,0x597b2876,
	0x597c2878,0x597d06cd,0x597e287a,0x597f287c,0x5980287e,0x59811224,0x598210af,0x598304ce,
	0x598413ea,0x59852880,0x598619e1,0x59870555,0x59880c83,0x59892882,0x598a1092,0x598b2884,
	0x598c2886,0x598d163d,0x598e2888,0x598f288a,0x5990288c,0x5991288e,0x5992042f,0x5993081f,
	0x59942890,0x59952892,0x59961688,0x59970926,0x59982894,0x59990d3d,0x599a2896,0x599b2898,
	0x599c289a,0x599d289c,0x599e0e18,0x599f289e,0x59a028a0,0x59a128a2,0x59a228a4,0x59a300d4,
	0x59a417b6,0x59a513b0,0x59a628a6,0x59a728a8,0x59a804c4,0x59a91464,0x59aa17e8,0x59ab0660,
	0x59ac28aa,0x59ad28ac,0x59ae0dd7,0x59af1995,0x59b028ae,0x59b128b0,0x59b2032a,0x59b328b2,
	0x59b428b4,0x59b528b6,0x59b628b8,0x59b728ba,0x59b828bc,0x59b90ce5,0x59ba28be,0x59bb0f2f,
	0x59bc28c0,0x59bd28c2,0x59be0fd5,0x59bf28c4,0x59c028c6,0x59c128c8,0x59c228ca,0x59c328cc,
	0x59c428ce,0x59c528d0,0x59c60d8e,0x59c728d2,0x59c828d4,0x59c928d6,0x59ca1a25,0x59cb11a9,
	0x59cc28d8,0x59cd28da,0x59ce28dc,0x59cf28de,0x59d00902,0x59d1060f,0x59d21244,0x59d3158e,
	0x59d4140f,0x59d528e0,0x59d628e2,0x59d71117,0x59d80eeb,0x59d928e4,0x59da168e,0x59db28e6,
	0x59dc08a5,0x59dd11e3,0x59de28e8,0x59df28ea,0x59e028ec,0x59e128ee,0x59e228f0,0x59e308bc,
	0x59e428f2,0x59e50aef,0x59e628f4,0x59e728f6,0x59e816d0,0x59e928f8,0x59ea28fa,0x59eb28fc,
	0x59ec07e6,0x59ed28fe,0x59ee2900,0x59ef2902,0x59f02904,0x59f12906,0x59f22908,0x59f3290a,
	0x59f4290c,0x59f5290e,0x59f62910,0x59f72912,0x59f82914,0x59f901c5,0x59fa2916,0x59fb1723,
	0x59fc2918,0x59fd291a,0x59fe291c,0x59ff1a10,0x5a00291e,0x5a0113f0,0x5a022920,0x5a0313bc,
	0x5a040c07,0x5a051627,0x5a061081,0x5a0708bd,0x5a080c3f,0x5a090ef5,0x5a0a2922,0x5a0b2924,
	0x5a0c0b30,0x5a0d2926,0x5a0e2928,0x5a0f292a,0x5a10292c,0x5a11129f,0x5a12292e,0x5a131413,
	0x5a142930,0x5a152932,0x5a162934,0x5a172936,0x5a180df8,0x5a192938,0x5a1a293a,0x5a1b293c,
	0x5a1c0da6,0x5a1d293e,0x5a1e2940,0x5a1f09aa,0x5a201169,0x5a212942,0x5a222944,0x5a2303ac,
	0x5a242946,0x5a25046d,0x5a262948,0x5a27294a,0x5a28294c,0x5a290d29,0x5a2a294e,0x5a2b2950,
	0x5a2c2952,0x5a2d2954,0x5a2e2956,0x5a2f2958,0x5a30295a,0x5a3117bf,0x5a3213ba,0x5a33295c,
	0x5a3414f1,0x5a35295e,0x5a361045,0x5a372960,0x5a382962,0x5a392964,0x5a3a2966,0x5a3b2968,
	0x5a3c01ed,0x5a3d296a,0x5a3e296c,0x5a3f296e,0x5a40046a,0x5a412970,0x5a422972,0x5a432974,
	0x5a442976,0x5a452978,0x5a460f06,0x5a47297a,0x5a48297c,0x5a4913d4,0x5a4a0128,0x5a4b297e,
	0x5a4c2980,0x5a4d2982,0x5a4e2984,0x5a4f2986,0x5a502988,0x5a51298a,0x5a52298c,0x5a53298e,
	0x5a542990,0x5a5508f8,0x5a562992,0x5a572994,0x5a582996,0x5a592998,0x5a5a07ba,0x5a5b299a,
	0x5a5c299c,0x5a5d299e,0x5a5e29a0,0x5a5f29a2,0x5a6029a4,0x5a6129a6,0x5a6200e8,0x5a6329a8,
	0x5a6429aa,0x5a6529ac,0x5a6629ae,0x5a670951,0x5a6829b0,0x5a6929b2,0x5a6a0ac5,0x5a6b29b4,
	0x5a6c29b6,0x5a6d29b8,0x5a6e29ba,0x5a6f29bc,0x5a7029be,0x5a7129c0,0x5a7229c2,0x5a7329c4,
	0x5a741745,0x5a7501d5,0x5a761174,0x5a771359,0x5a7829c6,0x5a7929c8,0x5a7a147c,0x5a7b29ca,
	0x5a7c29cc,0x5a7d29ce,0x5a7e29d0,0x5a7f15c5,0x5a8029d2,0x5a8129d4,0x5a8229d6,0x5a8329d8,
	0x5a8429da,0x5a8529dc,0x5a8629de,0x5a8729e0,0x5a8829e2,0x5a8929e4,0x5a8a29e6,0x5a8b29e8,
	0x5a8c29ea,0x5a8d29ec,0x5a8e29ee,0x5a8f29f0,0x5a9029f2,0x5a9129f4,0x5a920cd7,0x5a9329f6,
	0x5a9429f8,0x5a9529fa,0x5a9629fc,0x5a9729fe,0x5a982a00,0x5a992a02,0x5a9a0ce8,0x5a9b1822,
	0x5a9c2a04,0x5a9d2a06,0x5a9e2a08,0x5a9f2a0a,0x5aa02a0c,0x5aa12a0e,0x5aa22a10,0x5aa32a12,
	0x5aa42a14,0x5aa52a16,0x5aa62a18,0x5aa72a1a,0x5aa82a1c,0x5aa92a1e,0x5aaa003c,0x5aab2a20,
	0x5aac2a22,0x5aad2a24,0x5aae2a26,0x5aaf2a28,0x5ab02a2a,0x5ab12a2c,0x5ab20ecd,0x5ab314b7,
	0x5ab42a2e,0x5ab51768,0x5ab62a30,0x5ab72a32,0x5ab8024d,0x5ab92a34,0x5aba2a36,0x5abb2a38,
	0x5abc2a3a,0x5abd2a3c,0x5abe0609,0x5abf2a3e,0x5ac02a40,0x5ac1085e,0x5ac210f3,0x5ac32a42,
	0x5ac42a44,0x5ac52a46,0x5ac62a48,0x5ac72a4a,0x5ac82a4c,0x5ac907f7,0x5aca2a4e,0x5acb2a50,
	0x5acc14f6,0x5acd2a52,0x5ace2a54,0x5acf2a56,0x5ad02a58,0x5ad12a5a,0x5ad20015,0x5ad32a5c,
	0x5ad40eee,0x5ad52a5e,0x5ad60ee1,0x5ad72a60,0x5ad80aff,0x5ad92a62,0x5ada2a64,0x5adb2a66,
	0x5adc18c6,0x5add2a68,0x5ade2a6a,0x5adf2a6c,0x5ae00b22,0x5ae1039f,0x5ae22a6e,0x5ae31637,
	0x5ae42a70,0x5ae52a72,0x5ae601f9,0x5ae72a74,0x5ae82a76,0x5ae90dd4,0x5aea2a78,0x5aeb0d67,
	0x5aec2a7a,0x5aed2a7c,0x5aee2a7e,0x5aef2a80,0x5af02a82,0x5af10fae,0x5af22a84,0x5af32a86,
	0x5af42a88,0x5af52a8a,0x5af62a8c,0x5af72a8e,0x5af82a90,0x5af92a92,0x5afa2a94,0x5afb2a96,
	0x5afc2a98,0x5afd2a9a,0x5afe2a9c,0x5aff2a9e,0x5b002aa0,0x5b012aa2,0x5b022aa4,0x5b032aa6,
	0x5b042aa8,0x5b052aaa,0x5b062aac,0x5b072aae,0x5b082ab0,0x5b0914a9,0x5b0a2ab2,0x5b0b2ab4,
	0x5b0c2ab6,0x5b0d2ab8,0x5b0e2aba,0x5b0f2abc,0x5b102abe,0x5b112ac0,0x5b122ac2,0x5b132ac4,
	0x5b142ac6,0x5b152ac8,0x5b1600f8,0x5b17112b,0x5b182aca,0x5b192acc,0x5b1a2ace,0x5b1b2ad0,
	0x5b1c2ad2,0x5b1d2ad4,0x5b1e2ad6,0x5b1f2ad8,0x5b202ada,0x5b212adc,0x5b222ade,0x5b232ae0,
	0x5b242ae2,0x5b252ae4,0x5b262ae6,0x5b272ae8,0x5b282aea,0x5b292aec,0x5b2a2aee,0x5b2b2af0,
	0x5b2c2af2,0x5b2d2af4,0x5b2e2af6,0x5b2f2af8,0x5b302afa,0x5b312afc,0x5b320dfd,0x5b332afe,
	0x5b34175d,0x5b352b00,0x5b362b02,0x5b370d65,0x5b382b04,0x5b392b06,0x5b3a2b08,0x5b3b2b0a,
	0x5b3c2b0c,0x5b3d2b0e,0x5b3e2b10,0x5b3f2b12,0x5b401218,0x5b412b14,0x5b422b16,0x5b432b18,
	0x5b442b1a,0x5b452b1c,0x5b462b1e,0x5b472b20,0x5b482b22,0x5b492b24,0x5b4a2b26,0x5b4b2b28,
	0x5b4c2b2a,0x5b4d2b2c,0x5b4e2b2e,0x5b4f2b30,0x5b501a24,0x5b5108ef,0x5b522b32,0x5b5309bb,
	0x5b540a44,0x5b551841,0x5b562b34,0x5b571a2c,0x5b580313,0x5b591295,0x5b5a0522,0x5b5b00bc,
	0x5b5c1a0d,0x5b5d1541,0x5b5e2b36,0x5b5f0d03,0x5b602b38,0x5b612b3a,0x5b62008d,0x5b630825,
	0x5b640610,0x5b650e28,0x5b6615ea,0x5b672b3c,0x5b682b3e,0x5b690696,0x5b6a0c40,0x5b6b2b40,
	0x5b6c0dc2,0x5b6d2b42,0x5b6e2b44,0x5b6f2b46,0x5b7011f0,0x5b7101d7,0x5b722b48,0x5b731a16,
	0x5b742b4a,0x5b75051d,0x5b762b4c,0x5b772b4e,0x5b782b50,0x5b792b52,0x5b7a10b4,0x5b7b2b54,
	0x5b7c2b56,0x5b7d0e0b,0x5b7e2b58,0x5b7f2b5a,0x5b800d22,0x5b810e0e,0x5b822b5c,0x5b8312af,
	0x5b840668,0x5b8518a9,0x5b862b5e,0x5b8717d6,0x5b8811d2,0x5b890019,0x5b8a2b60,0x5b8b1259,
	0x5b8c13cc,0x5b8d2b62,0x5b8e2b64,0x5b8f0706,0x5b902b66,0x5b912b68,0x5b922b6a,0x5b930d19,
	0x5b942b6c,0x5b950368,0x5b962b6e,0x5b971a30,0x5b980656,0x5b99199b,0x5b9a03f8,0x5b9b13d0,
	0x5b9c16cc,0x5b9d0094,0x5b9e119e,0x5b9f2b70,0x5ba00274,0x5ba11171,0x5ba20a30,0x5ba315cc,
	0x5ba411bb,0x5ba517a8,0x5ba6076c,0x5ba72b72,0x5ba82b74,0x5ba92b76,0x5baa1506,0x5bab05eb,
	0x5bac2b78,0x5bad2b7a,0x5bae2b7c,0x5baf2b7e,0x5bb01855,0x5bb12b80,0x5bb22b82,0x5bb3069e,
	0x5bb4165d,0x5bb5152f,0x5bb60844,0x5bb72b84,0x5bb80224,0x5bb910a0,0x5bba2b86,0x5bbb2b88,
	0x5bbc2b8a,0x5bbd0a6b,0x5bbe0130,0x5bbf1275,0x5bc02b8c,0x5bc12b8e,0x5bc2082d,0x5bc32b90,
	0x5bc4082e,0x5bc51730,0x5bc60d1d,0x5bc70a4d,0x5bc82b92,0x5bc92b94,0x5bca2b96,0x5bcb2b98,
	0x5bcc055f,0x5bcd2b9a,0x5bce2b9c,0x5bcf2b9e,0x5bd00ce9,0x5bd12ba0,0x5bd206ac,0x5bd317f7,
	0x5bd42ba2,0x5bd52ba4,0x5bd62ba6,0x5bd72ba8,0x5bd82baa,0x5bd92bac,0x5bda2bae,0x5bdb2bb0,
	0x5bdc2bb2,0x5bdd0fef,0x5bde0d78,0x5bdf01bd,0x5be02bb4,0x5be1063d,0x5be22bb6,0x5be32bb8,
	0x5be41480,0x5be50b93,0x5be62bba,0x5be72bbc,0x5be818ad,0x5be92bbe,0x5bea2bc0,0x5beb2bc2,
	0x5bec2bc4,0x5bed2bc6,0x5bee0b95,0x5bef2bc8,0x5bf00765,0x5bf12bca,0x5bf22bcc,0x5bf32bce,
	0x5bf42bd0,0x5bf52bd2,0x5bf62bd4,0x5bf72bd6,0x5bf80315,0x5bf90441,0x5bfa123f,0x5bfb15f9,
	0x5bfc0372,0x5bfd2bd8,0x5bfe2bda,0x5bff11d5,0x5c002bdc,0x5c010502,0x5c022bde,0x5c032be0,
	0x5c04115b,0x5c052be2,0x5c0608a6,0x5c072be4,0x5c082be6,0x5c091422,0x5c0a1a58,0x5c0b2be8,
	0x5c0c2bea,0x5c0d2bec,0x5c0e2bee,0x5c0f153e,0x5c102bf0,0x5c111149,0x5c122bf2,0x5c132bf4,
	0x5c140490,0x5c15056e,0x5c160862,0x5c172bf6,0x5c18021e,0x5c192bf8,0x5c1a113b,0x5c1b2bfa,
	0x5c1c056c,0x5c1d01f5,0x5c1e2bfc,0x5c1f2bfe,0x5c202c00,0x5c212c02,0x5c22178a,0x5c232c04,
	0x5c24178b,0x5c250b9e,0x5c262c06,0x5c27168c,0x5c282c08,0x5c292c0a,0x5c2a2c0c,0x5c2b2c0e,
	0x5c2c056f,0x5c2d2c10,0x5c2e2c12,0x5c2f2c14,0x5c302c16,0x5c310975,0x5c322c18,0x5c332c1a,
	0x5c340586,0x5c352c1c,0x5c362c1e,0x5c372c20,0x5c38118e,0x5c391738,0x5c3a025a,0x5c3b0a0e,
	0x5c3c0dd8,0x5c3d0924,0x5c3e13f9,0x5c3f0dfe,0x5c400987,0x5c410ecb,0x5c4201b0,0x5c432c22,
	0x5c442c24,0x5c450979,0x5c462c26,0x5c472c28,0x5c481030,0x5c491326,0x5c4a0907,0x5c4b1454,
	0x5c4c2c2a,0x5c4d2c2c,0x5c4e11ab,0x5c4f0efd,0x5c5007e7,0x5c51155c,0x5c522c2e,0x5c532c30,
	0x5c542c32,0x5c5518b8,0x5c562c34,0x5c572c36,0x5c582c38,0x5c590469,0x5c5a2c3a,0x5c5b2c3c,
	0x5c5c2c3e,0x5c5d2c40,0x5c5e11fb,0x5c5f2c42,0x5c601384,0x5c610c76,0x5c622c44,0x5c6314c0,
	0x5c642c46,0x5c650c7a,0x5c6609a6,0x5c672c48,0x5c682c4a,0x5c692c4c,0x5c6a2c4e,0x5c6b2c50,
	0x5c6c2c52,0x5c6d2c54,0x5c6e0214,0x5c6f139a,0x5c702c56,0x5c711112,0x5c722c58,0x5c732c5a,
	0x5c742c5c,0x5c752c5e,0x5c762c60,0x5c772c62,0x5c782c64,0x5c7916f2,0x5c7a0f5d,0x5c7b2c66,
	0x5c7c2c68,0x5c7d2c6a,0x5c7e2c6c,0x5c7f17d7,0x5c802c6e,0x5c81128c,0x5c820f5e,0x5c832c70,
	0x5c842c72,0x5c852c74,0x5c862c76,0x5c872c78,0x5c881618,0x5c892c7a,0x5c8a2c7c,0x5c8b2c7e,
	0x5c8c07fc,0x5c8d0f7e,0x5c8e2c80,0x5c8f2c82,0x5c900f41,0x5c9101ad,0x5c922c84,0x5c932c86,
	0x5c9401c3,0x5c952c88,0x5c96102d,0x5c97059a,0x5c981501,0x5c99003f,0x5c9a0ac2,0x5c9b0373,
	0x5c9c004a,0x5c9d2c8a,0x5c9e2c8c,0x5c9f2c8e,0x5ca02c90,0x5ca12c92,0x5ca20a2c,0x5ca305ff,
	0x5ca42c94,0x5ca52c96,0x5ca62c98,0x5ca72c9a,0x5ca82c9c,0x5ca91640,0x5caa2c9e,0x5cab15a6,
	0x5cac0854,0x5cad0bde,0x5cae2ca0,0x5caf2ca2,0x5cb02ca4,0x5cb10338,0x5cb22ca6,0x5cb3182b,
	0x5cb42ca8,0x5cb50742,0x5cb62caa,0x5cb70d48,0x5cb80025,0x5cb92cac,0x5cba2cae,0x5cbb2cb0,
	0x5cbc2cb2,0x5cbd0402,0x5cbe2cb4,0x5cbf0a7f,0x5cc02cb6,0x5cc10cc1,0x5cc22cb8,0x5cc32cba,
	0x5cc416fb,0x5cc52cbc,0x5cc62cbe,0x5cc72cc0,0x5cc82cc2,0x5cc92cc4,0x5cca2cc6,0x5ccb15fd,
	0x5ccc2cc8,0x5ccd2cca,0x5cce2ccc,0x5ccf2cce,0x5cd02cd0,0x5cd12cd2,0x5cd2040b,0x5cd32cd4,
	0x5cd42cd6,0x5cd52cd8,0x5cd62cda,0x5cd72cdc,0x5cd82cde,0x5cd91965,0x5cda2ce0,0x5cdb2ce2,
	0x5cdc2ce4,0x5cdd2ce6,0x5cde2ce8,0x5cdf2cea,0x5ce02cec,0x5ce114d1,0x5ce22cee,0x5ce32cf0,
	0x5ce40fbd,0x5ce5191f,0x5ce60c41,0x5ce72cf2,0x5ce8046e,0x5ce92cf4,0x5cea17ee,0x5ceb2cf6,
	0x5cec2cf8,0x5ced0fcd,0x5cee2cfa,0x5cef2cfc,0x5cf00505,0x5cf12cfe,0x5cf22d00,0x5cf32d02,
	0x5cf42d04,0x5cf52d06,0x5cf62d08,0x5cf72d0a,0x5cf82d0c,0x5cf92d0e,0x5cfa2d10,0x5cfb09dd,
	0x5cfc2d12,0x5cfd2d14,0x5cfe2d16,0x5cff2d18,0x5d002d1a,0x5d012d1c,0x5d020ae9,0x5d030ab6,
	0x5d042d1e,0x5d052d20,0x5d060a42,0x5d070273,0x5d082d22,0x5d092d24,0x5d0a2d26,0x5d0b2d28,
	0x5d0c2d2a,0x5d0d2d2c,0x5d0e0f4a,0x5d0f2d2e,0x5d102d30,0x5d112d32,0x5d122d34,0x5d132d36,
	0x5d140303,0x5d152d38,0x5d16161c,0x5d172d3a,0x5d182d3c,0x5d192d3e,0x5d1a2d40,0x5d1b09c3,
	0x5d1c2d42,0x5d1d2d44,0x5d1e0685,0x5d1f2d46,0x5d202d48,0x5d212d4a,0x5d222d4c,0x5d232d4e,
	0x5d24153c,0x5d252d50,0x5d26162f,0x5d271250,0x5d282d52,0x5d2900c6,0x5d2a2d54,0x5d2b2d56,
	0x5d2c2d58,0x5d2d18ba,0x5d2e0630,0x5d2f2d5a,0x5d302d5c,0x5d312d5e,0x5d322d60,0x5d332d62,
	0x5d3413ee,0x5d352d64,0x5d362d66,0x5d372d68,0x5d382d6a,0x5d392d6c,0x5d3a2d6e,0x5d3b2d70,
	0x5d3c2d72,0x5d3d1856,0x5d3e169d,0x5d3f2d74,0x5d402d76,0x5d412d78,0x5d422d7a,0x5d432d7c,
	0x5d442d7e,0x5d452d80,0x5d462d82,0x5d4707f9,0x5d482d84,0x5d492d86,0x5d4a118d,0x5d4b0cd8,
	0x5d4c0f9f,0x5d4d2d88,0x5d4e2d8a,0x5d4f2d8c,0x5d502d8e,0x5d512d90,0x5d522d92,0x5d532d94,
	0x5d542d96,0x5d552d98,0x5d562d9a,0x5d572d9c,0x5d5810a1,0x5d592d9e,0x5d5a2da0,0x5d5b17c7,
	0x5d5c2da2,0x5d5d0c0f,0x5d5e2da4,0x5d5f2da6,0x5d602da8,0x5d612daa,0x5d622dac,0x5d632dae,
	0x5d642db0,0x5d652db2,0x5d662db4,0x5d672db6,0x5d682db8,0x5d691253,0x5d6a2dba,0x5d6b1a17,
	0x5d6c1409,0x5d6d2dbc,0x5d6e2dbe,0x5d6f031a,0x5d702dc0,0x5d712dc2,0x5d722dc4,0x5d732dc6,
	0x5d740817,0x5d752dc8,0x5d762dca,0x5d772dcc,0x5d782dce,0x5d792dd0,0x5d7a2dd2,0x5d7b2dd4,
	0x5d7c2dd6,0x5d7d2dd8,0x5d7e2dda,0x5d7f2ddc,0x5d802dde,0x5d812de0,0x5d8218d7,0x5d832de2,
	0x5d842de4,0x5d852de6,0x5d862de8,0x5d872dea,0x5d882dec,0x5d892dee,0x5d8a2df0,0x5d8b2df2,
	0x5d8c2df4,0x5d8d2df6,0x5d8e2df8,0x5d8f2dfa,0x5d902dfc,0x5d912dfe,0x5d922e00,0x5d932e02,
	0x5d942e04,0x5d952e06,0x5d962e08,0x5d972e0a,0x5d982e0c,0x5d990bb5,0x5d9a2e0e,0x5d9b2e10,
	0x5d9c2e12,0x5d9d038c,0x5d9e2e14,0x5d9f2e16,0x5da02e18,0x5da12e1a,0x5da22e1c,0x5da32e1e,
	0x5da42e20,0x5da52e22,0x5da62e24,0x5da72e26,0x5da82e28,0x5da92e2a,0x5daa2e2c,0x5dab2e2e,
	0x5dac2e30,0x5dad2e32,0x5dae2e34,0x5daf2e36,0x5db02e38,0x5db12e3a,0x5db22e3c,0x5db32e3e,
	0x5db42e40,0x5db52e42,0x5db62e44,0x5db716da,0x5db82e46,0x5db92e48,0x5dba2e4a,0x5dbb2e4c,
	0x5dbc2e4e,0x5dbd2e50,0x5dbe2e52,0x5dbf2e54,0x5dc02e56,0x5dc12e58,0x5dc22e5a,0x5dc32e5c,
	0x5dc42e5e,0x5dc503b9,0x5dc62e60,0x5dc72e62,0x5dc82e64,0x5dc92e66,0x5dca2e68,0x5dcb2e6a,
	0x5dcc2e6c,0x5dcd13f8,0x5dce2e6e,0x5dcf2e70,0x5dd02e72,0x5dd12e74,0x5dd22e76,0x5dd32e78,
	0x5dd42e7a,0x5dd52e7c,0x5dd62e7e,0x5dd72e80,0x5dd82e82,0x5dd92e84,0x5dda2e86,0x5ddb02a4,
	0x5ddc2e88,0x5ddd02a5,0x5dde198f,0x5ddf2e8a,0x5de02e8c,0x5de115fa,0x5de2020a,0x5de32e8e,
	0x5de42e90,0x5de505e4,0x5de61a5f,0x5de70fc9,0x5de80994,0x5de905f2,0x5dea2e92,0x5deb1453,
	0x5dec2e94,0x5ded2e96,0x5dee01c6,0x5def1025,0x5df02e98,0x5df10811,0x5df216e0,0x5df3123d,
	0x5df40046,0x5df52e9a,0x5df62e9c,0x5df71524,0x5df82e9e,0x5df92ea0,0x5dfa2ea2,0x5dfb2ea4,
	0x5dfc2ea6,0x5dfd160c,0x5dfe090f,0x5dff2ea8,0x5e002eaa,0x5e0100db,0x5e0211b2,0x5e030177,
	0x5e042eac,0x5e051210,0x5e0604a4,0x5e072eae,0x5e081190,0x5e092eb0,0x5e0a2eb2,0x5e0b2eb4,
	0x5e0c1489,0x5e0d2eb6,0x5e0e2eb8,0x5e0f13ff,0x5e1018d2,0x5e1112f6,0x5e122eba,0x5e132ebc,
	0x5e140e8d,0x5e150e53,0x5e16134e,0x5e172ebe,0x5e180b66,0x5e19195f,0x5e1a1998,0x5e1b0158,
	0x5e1c1960,0x5e1d03ab,0x5e1e2ec0,0x5e1f2ec2,0x5e202ec4,0x5e212ec6,0x5e222ec8,0x5e232eca,
	0x5e242ecc,0x5e252ece,0x5e26033c,0x5e27190c,0x5e282ed0,0x5e292ed2,0x5e2a2ed4,0x5e2b2ed6,
	0x5e2c2ed8,0x5e2d14b4,0x5e2e007e,0x5e2f2eda,0x5e302edc,0x5e31027a,0x5e322ede,0x5e332ee0,
	0x5e342ee2,0x5e352ee4,0x5e362ee6,0x5e371406,0x5e3801f7,0x5e392ee8,0x5e3a2eea,0x5e3b187e,
	0x5e3c068b,0x5e3d0cca,0x5e3e2eec,0x5e3f2eee,0x5e402ef0,0x5e412ef2,0x5e420d1e,0x5e432ef4,
	0x5e441448,0x5e45053c,0x5e462ef6,0x5e472ef8,0x5e482efa,0x5e492efc,0x5e4a2efe,0x5e4b2f00,
	0x5e4c0791,0x5e4d2f02,0x5e4e2f04,0x5e4f2f06,0x5e502f08,0x5e512f0a,0x5e522f0c,0x5e532f0e,
	0x5e540ca4,0x5e550d9a,0x5e562f10,0x5e572f12,0x5e582f14,0x5e592f16,0x5e5a2f18,0x5e5b18d8,
	0x5e5c2f1a,0x5e5d2f1c,0x5e5e0540,0x5e5f2f1e,0x5e602f20,0x5e6104a6,0x5e6202b4,0x5e632f22,
	0x5e642f24,0x5e652f26,0x5e662f28,0x5e672f2a,0x5e682f2c,0x5e692f2e,0x5e6a2f30,0x5e6b2f32,
	0x5e6c2f34,0x5e6d2f36,0x5e6e2f38,0x5e6f2f3a,0x5e702f3c,0x5e712f3e,0x5e72057b,0x5e730ef8,
	0x5e740ded,0x5e752f40,0x5e76014a,0x5e772f42,0x5e78158f,0x5e792f44,0x5e7a1685,0x5e7b076a,
	0x5e7c17a4,0x5e7d1788,0x5e7e2f46,0x5e7f065b,0x5e800ec2,0x5e812f48,0x5e822f4a,0x5e832f4c,
	0x5e8419e2,0x5e852f4e,0x5e861007,0x5e8700df,0x5e882f50,0x5e892f52,0x5e8a02b3,0x5e8b066a,
	0x5e8c2f54,0x5e8d2f56,0x5e8e2f58,0x5e8f15bd,0x5e900c19,0x5e911465,0x5e922f5a,0x5e930a57,
	0x5e941743,0x5e9503a4,0x5e960e7b,0x5e9703c4,0x5e982f5c,0x5e990d3e,0x5e9a05da,0x5e9b2f5e,
	0x5e9c0546,0x5e9d2f60,0x5e9e0e71,0x5e9f04e2,0x5ea0151a,0x5ea12f62,0x5ea22f64,0x5ea32f66,
	0x5ea42f68,0x5ea5159e,0x5ea60432,0x5ea71a69,0x5ea82f6a,0x5ea92f6c,0x5eaa2f6e,0x5eab2f70,
	0x5eac2f72,0x5ead1356,0x5eae2f74,0x5eaf2f76,0x5eb02f78,0x5eb12f7a,0x5eb22f7c,0x5eb300a9,
	0x5eb42f7e,0x5eb5001c,0x5eb61204,0x5eb70a04,0x5eb8176f,0x5eb913b1,0x5eba2f80,0x5ebb2f82,
	0x5ebc2f84,0x5ebd2f86,0x5ebe17df,0x5ebf2f88,0x5ec02f8a,0x5ec12f8c,0x5ec22f8e,0x5ec32f90,
	0x5ec42f92,0x5ec52f94,0x5ec62f96,0x5ec72f98,0x5ec82f9a,0x5ec90b6c,0x5eca0adb,0x5ecb2f9c,
	0x5ecc2f9e,0x5ecd2fa0,0x5ece2fa2,0x5ecf2fa4,0x5ed02fa6,0x5ed10920,0x5ed20032,0x5ed30aa9,
	0x5ed42fa8,0x5ed52faa,0x5ed60b9c,0x5ed72fac,0x5ed82fae,0x5ed92fb0,0x5eda2fb2,0x5edb01dc,
	0x5edc2fb4,0x5edd2fb6,0x5ede2fb8,0x5edf2fba,0x5ee02fbc,0x5ee12fbe,0x5ee22fc0,0x5ee32fc2,
	0x5ee42fc4,0x5ee52fc6,0x5ee62fc8,0x5ee72fca,0x5ee81563,0x5ee92fcc,0x5eea0bbe,0x5eeb2fce,
	0x5eec2fd0,0x5eed2fd2,0x5eee2fd4,0x5eef2fd6,0x5ef02fd8,0x5ef12fda,0x5ef22fdc,0x5ef32fde,
	0x5ef41737,0x5ef52fe0,0x5ef6163a,0x5ef71354,0x5ef82fe2,0x5ef92fe4,0x5efa088f,0x5efb2fe6,
	0x5efc2fe8,0x5efd2fea,0x5efe05f1,0x5eff0df5,0x5f0009e9,0x5f010111,0x5f0216f3,0x5f030f69,
	0x5f040e25,0x5f052fec,0x5f062fee,0x5f072ff0,0x5f081702,0x5f092ff2,0x5f0a00f3,0x5f0b16ec,
	0x5f0c2ff4,0x5f0d2ff6,0x5f0e2ff8,0x5f0f11b4,0x5f102ffa,0x5f1111c6,0x5f122ffc,0x5f1305e5,
	0x5f142ffe,0x5f151739,0x5f163000,0x5f17051f,0x5f180704,0x5f193002,0x5f1a3004,0x5f1b0251,
	0x5f1c3006,0x5f1d3008,0x5f1e300a,0x5f1f03aa,0x5f2018c3,0x5f21300c,0x5f22300e,0x5f233010,
	0x5f243012,0x5f250d07,0x5f2614ed,0x5f27072a,0x5f283014,0x5f290e2b,0x5f2a094b,0x5f2b3016,
	0x5f2c3018,0x5f2d0d13,0x5f2e301a,0x5f2f13c4,0x5f30301c,0x5f3110d2,0x5f32301e,0x5f333020,
	0x5f343022,0x5f353024,0x5f363026,0x5f373028,0x5f38302a,0x5f39035b,0x5f3a0fac,0x5f3b302c,
	0x5f3c00eb,0x5f3d302e,0x5f3e3030,0x5f3f3032,0x5f40060a,0x5f413034,0x5f423036,0x5f433038,
	0x5f44303a,0x5f45303c,0x5f46303e,0x5f473040,0x5f483042,0x5f493044,0x5f4a3046,0x5f4b3048,
	0x5f4c304a,0x5f4d304c,0x5f4e304e,0x5f4f3050,0x5f5015ed,0x5f513052,0x5f52065e,0x5f530362,
	0x5f543054,0x5f550c2d,0x5f561390,0x5f5707af,0x5f581972,0x5f593056,0x5f5a3058,0x5f5b305a,
	0x5f5c305c,0x5f5d16db,0x5f5e305e,0x5f5f3060,0x5f603062,0x5f611113,0x5f621586,0x5f633064,
	0x5f641366,0x5f653066,0x5f66165a,0x5f673068,0x5f68306a,0x5f690188,0x5f6a011c,0x5f6b306c,
	0x5f6c0131,0x5f6d0e9c,0x5f6e306e,0x5f6f3070,0x5f7018c7,0x5f711764,0x5f723072,0x5f730260,
	0x5f743074,0x5f753076,0x5f763078,0x5f770e70,0x5f78307a,0x5f7916f6,0x5f7a307c,0x5f7b0215,
	0x5f7c00d5,0x5f7d307e,0x5f7e3080,0x5f7f3082,0x5f8013e4,0x5f81191d,0x5f8202f2,0x5f833084,
	0x5f84094c,0x5f85033d,0x5f863086,0x5f871609,0x5f8806f4,0x5f891679,0x5f8a0758,0x5f8b0c7d,
	0x5f8c071a,0x5f8d3088,0x5f8e308a,0x5f8f308c,0x5f9015b5,0x5f91308e,0x5f921380,0x5f933090,
	0x5f943092,0x5f950aba,0x5f963094,0x5f97037f,0x5f980e57,0x5f9914bc,0x5f9a3096,0x5f9b3098,
	0x5f9c01f8,0x5f9d309a,0x5f9e309c,0x5f9f309e,0x5fa030a0,0x5fa117f8,0x5fa230a2,0x5fa330a4,
	0x5fa430a6,0x5fa530a8,0x5fa630aa,0x5fa730ac,0x5fa80781,0x5fa930ae,0x5faa1602,0x5fab30b0,
	0x5fac30b2,0x5fad1693,0x5fae13f5,0x5faf30b4,0x5fb030b6,0x5fb130b8,0x5fb230ba,0x5fb330bc,
	0x5fb430be,0x5fb51926,0x5fb630c0,0x5fb70381,0x5fb830c2,0x5fb930c4,0x5fba30c6,0x5fbb30c8,
	0x5fbc08db,0x5fbd079a,0x5fbe30ca,0x5fbf30cc,0x5fc030ce,0x5fc130d0,0x5fc230d2,0x5fc3156d,
	0x5fc4156c,0x5fc500dc,0x5fc616ee,0x5fc730d4,0x5fc830d6,0x5fc90370,0x5fca30d8,0x5fcb30da,
	0x5fcc0820,0x5fcd108a,0x5fce30dc,0x5fcf01e8,0x5fd012dd,0x5fd1130f,0x5fd21310,0x5fd330de,
	0x5fd430e0,0x5fd530e2,0x5fd60314,0x5fd7195b,0x5fd813eb,0x5fd90cac,0x5fda30e4,0x5fdb30e6,
	0x5fdc30e8,0x5fdd1338,0x5fde30ea,0x5fdf30ec,0x5fe0197f,0x5fe1026d,0x5fe230ee,0x5fe330f0,
	0x5fe41466,0x5fe530f2,0x5fe630f4,0x5fe71785,0x5fe830f6,0x5fe930f8,0x5fea124d,0x5feb0a64,
	0x5fec30fa,0x5fed0112,0x5fee195c,0x5fef30fc,0x5ff030fe,0x5ff10220,0x5ff23100,0x5ff33102,
	0x5ff43104,0x5ff50df6,0x5ff63106,0x5ff73108,0x5ff80e1a,0x5ff9310a,0x5ffa310c,0x5ffb156e,
	0x5ffc310e,0x5ffd0722,0x5ffe09f5,0x5fff04f8,0x60000757,0x600112c9,0x60021254,0x60031467,
	0x60040e49,0x60050200,0x600602b7,0x60073110,0x60083112,0x60093114,0x600a0206,0x600b3116,
	0x600c3118,0x600d1a64,0x600e1886,0x600f1681,0x6010311a,0x6011311c,0x60120e2d,0x6013311e,
	0x6014191e,0x60150e54,0x60160179,0x60173120,0x60183122,0x60190743,0x601a3124,0x601b032b,
	0x601c0b67,0x601d1232,0x601e3126,0x601f3128,0x6020033e,0x602116cd,0x6022312a,0x6023312c,
	0x6024312e,0x60250803,0x60260e95,0x60271590,0x6028181f,0x60290dda,0x602a0645,0x602b0525,
	0x602c3130,0x602d3132,0x602e3134,0x602f0fd6,0x60303136,0x60313138,0x6032313a,0x6033313c,
	0x6034313e,0x6035029c,0x60363140,0x60373142,0x60383144,0x60393146,0x603a3148,0x603b1a36,
	0x603c0443,0x603d314a,0x603e314c,0x603f16fc,0x6040314e,0x60410dd5,0x604215fe,0x604311bc,
	0x60443150,0x60453152,0x60463154,0x60473156,0x60483158,0x6049315a,0x604a315c,0x604b0b79,
	0x604c315e,0x604d078e,0x604e3160,0x604f3162,0x60500a45,0x60513164,0x605206f9,0x60533166,
	0x60543168,0x60551203,0x6056316a,0x6057316c,0x6058316e,0x60591682,0x605a07ac,0x605b3170,
	0x605c3172,0x605d084f,0x605e3174,0x605f3176,0x60603178,0x6061317a,0x60620794,0x60631a22,
	0x606415bf,0x6065317c,0x6066317e,0x60670e3b,0x606806f6,0x60690489,0x606a0a31,0x606b040c,
	0x606c1333,0x606d05ec,0x606e3180,0x606f1491,0x60700f74,0x60713182,0x60723184,0x60730a3a,
	0x60743186,0x60753188,0x6076047b,0x6077318a,0x60781374,0x6079162c,0x607a09ef,0x607b01aa,
	0x607c0dca,0x607d1844,0x607e318c,0x607f1780,0x6080318e,0x60813190,0x60823192,0x60830aa1,
	0x60840fc8,0x60853194,0x60863196,0x60873198,0x6088319a,0x60891494,0x608a319c,0x608b319e,
	0x608c1329,0x608d06b3,0x608e31a0,0x608f31a2,0x609031a4,0x609131a6,0x60921706,0x609331a8,
	0x609407a1,0x609531aa,0x609600b2,0x609731ac,0x609831ae,0x609931b0,0x609a1255,0x609b104c,
	0x609c31b2,0x609d0a80,0x609e31b4,0x609f1479,0x60a01789,0x60a131b6,0x60a231b8,0x60a30771,
	0x60a431ba,0x60a531bc,0x60a6182c,0x60a731be,0x60a80e0d,0x60a931c0,0x60aa31c2,0x60ab1066,
	0x60ac15d6,0x60ad0f81,0x60ae31c4,0x60af0d53,0x60b031c6,0x60b104dd,0x60b200a6,0x60b331c8,
	0x60b4030a,0x60b531ca,0x60b631cc,0x60b731ce,0x60b8082f,0x60b931d0,0x60ba31d2,0x60bb1592,
	0x60bc0379,0x60bd31d4,0x60be31d6,0x60bf31d8,0x60c031da,0x60c131dc,0x60c231de,0x60c331e0,
	0x60c431e2,0x60c50ffc,0x60c6027b,0x60c731e4,0x60c831e6,0x60c931e8,0x60ca0937,0x60cb13d5,
	0x60cc31ea,0x60cd31ec,0x60ce31ee,0x60cf31f0,0x60d031f2,0x60d107d1,0x60d231f4,0x60d331f6,
	0x60d431f8,0x60d5132c,0x60d631fa,0x60d731fc,0x60d813e7,0x60d931fe,0x60da0726,0x60db3200,
	0x60dc1495,0x60dd01fd,0x60de3202,0x60df1407,0x60e007b3,0x60e13204,0x60e23206,0x60e33208,
	0x60e4320a,0x60e5320c,0x60e603c8,0x60e7099e,0x60e80193,0x60e90242,0x60ea320e,0x60eb00b4,
	0x60ec0fd9,0x60ed0192,0x60ee035c,0x60ef0650,0x60f00466,0x60f13210,0x60f23212,0x60f31521,
	0x60f419f0,0x60f53214,0x60f60782,0x60f73216,0x60f83218,0x60f91084,0x60fa157f,0x60fb321a,
	0x60fc321c,0x60fd321e,0x60fe3220,0x60ff3222,0x61000fca,0x6101027e,0x61023224,0x61033226,
	0x61043228,0x6105322a,0x61060f84,0x6107322c,0x610817fc,0x610917c8,0x610a322e,0x610b3230,
	0x610c3232,0x610d0d55,0x610e00ec,0x610f170e,0x61103234,0x61113236,0x61123238,0x6113323a,
	0x6114323c,0x61150480,0x6116323e,0x61173240,0x61183242,0x61193244,0x611a17cc,0x611b3246,
	0x611c3248,0x611d324a,0x611e324c,0x611f058b,0x61201847,0x6121324e,0x61223250,0x61230b15,
	0x612404fa,0x61253252,0x61260a93,0x61270a94,0x61283254,0x61293256,0x612a3258,0x612b127a,
	0x612c325a,0x612d325c,0x612e325e,0x612f3260,0x61303262,0x61313264,0x61323266,0x61333268,
	0x6134326a,0x6135326c,0x6136326e,0x61373270,0x61383272,0x61393274,0x613a3276,0x613b3278,
	0x613c327a,0x613d327c,0x613e327e,0x613f1825,0x61403280,0x61413282,0x61423284,0x61433286,
	0x61443288,0x6145328a,0x6146328c,0x6147328e,0x614802d7,0x61493290,0x614a0f99,0x614b3292,
	0x614c077b,0x614d3294,0x614e117b,0x614f3296,0x61503298,0x6151115e,0x6152329a,0x6153329c,
	0x6154329e,0x61550d9c,0x615632a0,0x615732a2,0x615832a4,0x615932a6,0x615a32a8,0x615b32aa,
	0x615c32ac,0x615d1313,0x615e32ae,0x615f32b0,0x616032b2,0x616132b4,0x61620ca5,0x616332b6,
	0x616432b8,0x616532ba,0x616632bc,0x616707b5,0x616809f1,0x616932be,0x616a32c0,0x616b32c2,
	0x616c32c4,0x616d32c6,0x616e32c8,0x616f32ca,0x61701428,0x617132cc,0x617232ce,0x617332d0,
	0x617432d2,0x61751772,0x617632d4,0x61770a05,0x617832d6,0x617932d8,0x617a32da,0x617b32dc,
	0x617c32de,0x617d32e0,0x617e32e2,0x617f32e4,0x618032e6,0x618132e8,0x618232ea,0x618332ec,
	0x618432ee,0x618532f0,0x618632f2,0x618732f4,0x618832f6,0x618932f8,0x618a32fa,0x618b012b,
	0x618c32fc,0x618d32fe,0x618e188a,0x618f3300,0x61903302,0x61913304,0x61923306,0x61933308,
	0x61940fc3,0x6195330a,0x6196330c,0x6197330e,0x61983310,0x61993312,0x619a3314,0x619b3316,
	0x619c3318,0x619d0445,0x619e331a,0x619f331c,0x61a0331e,0x61a13320,0x61a23322,0x61a33324,
	0x61a43326,0x61a53328,0x61a6332a,0x61a70270,0x61a806a3,0x61a90f71,0x61aa332c,0x61ab332e,
	0x61ac0948,0x61ad3330,0x61ae3332,0x61af3334,0x61b03336,0x61b13338,0x61b2333a,0x61b3333c,
	0x61b4333e,0x61b53340,0x61b63342,0x61b702a0,0x61b83344,0x61b93346,0x61ba3348,0x61bb334a,
	0x61bc334c,0x61bd334e,0x61be06b9,0x61bf3350,0x61c03352,0x61c13354,0x61c20406,0x61c33356,
	0x61c43358,0x61c5335a,0x61c6335c,0x61c7335e,0x61c81564,0x61c93360,0x61ca0043,0x61cb0cce,
	0x61cc3362,0x61cd3364,0x61ce3366,0x61cf3368,0x61d0336a,0x61d10cf1,0x61d20ad5,0x61d3336c,
	0x61d40bbf,0x61d5336e,0x61d63370,0x61d73372,0x61d83374,0x61d93376,0x61da3378,0x61db337a,
	0x61dc337c,0x61dd337e,0x61de3380,0x61df3382,0x61e03384,0x61e13386,0x61e23388,0x61e3338a,
	0x61e4338c,0x61e5338e,0x61e60e37,0x61e73390,0x61e83392,0x61e93394,0x61ea3396,0x61eb3398,
	0x61ec339a,0x61ed339c,0x61ee339e,0x61ef33a0,0x61f033a2,0x61f133a4,0x61f233a6,0x61f333a8,
	0x61f433aa,0x61f50d01,0x61f633ac,0x61f733ae,0x61f833b0,0x61f933b2,0x61fa33b4,0x61fb33b6,
	0x61fc33b8,0x61fd33ba,0x61fe33bc,0x61ff1720,0x620033be,0x620133c0,0x620233c2,0x620333c4,
	0x620433c6,0x620533c8,0x6206059e,0x620733ca,0x620805b2,0x620933cc,0x620a1473,0x620b0860,
	0x620c15ac,0x620d11fd,0x620e109a,0x620f14c3,0x62100233,0x62111444,0x62120905,0x621333ce,
	0x621433d0,0x62150fa4,0x621607cd,0x62170fa5,0x621818bd,0x621933d2,0x621a0f34,0x621b0850,
	0x621c33d4,0x621d33d6,0x621e33d8,0x621f0816,0x622033da,0x622109fa,0x62220806,0x622333dc,
	0x62240579,0x62250389,0x622633de,0x622733e0,0x622833e2,0x622933e4,0x622a08fc,0x622b33e6,
	0x622c0888,0x622d33e8,0x622e0c37,0x622f33ea,0x623033ec,0x623133ee,0x623233f0,0x623302cb,
	0x62340345,0x623533f2,0x623633f4,0x6237073e,0x623833f6,0x623933f8,0x623a33fa,0x623b33fc,
	0x623c33fe,0x623d0744,0x623e0b47,0x623f04c5,0x624012a7,0x6241010b,0x62423400,0x6243095b,
	0x62443402,0x62453404,0x62463406,0x62471126,0x62480747,0x624904d3,0x624a3408,0x624b11d0,
	0x624c11d1,0x624d0183,0x624e1896,0x624f340a,0x6250340c,0x62510f17,0x62520048,0x62530330,
	0x62541097,0x6255340e,0x62563410,0x62573412,0x625813a0,0x62593414,0x625a3416,0x625b0a07,
	0x625c3418,0x625d341a,0x625e341c,0x625f341e,0x62603420,0x62613422,0x62623424,0x62630a4c,
	0x62643426,0x62653428,0x62660f7a,0x62671941,0x6268342a,0x62690aa5,0x626a0cec,0x626b10f2,
	0x626c1672,0x626d0e1b,0x626e0079,0x626f0213,0x62701082,0x6271342c,0x6272342e,0x62730068,
	0x62743430,0x62753432,0x62760523,0x62773434,0x62783436,0x62790ea8,0x627a3438,0x627b343a,
	0x627c0477,0x627d343c,0x627e18dd,0x627f0235,0x62800821,0x6281343e,0x62823440,0x62833442,
	0x62840205,0x62853444,0x62863446,0x62873448,0x6288344a,0x628909be,0x628a0055,0x628b344c,
	0x628c344e,0x628d3450,0x628e3452,0x628f3454,0x62903456,0x629116f7,0x629211df,0x629319d5,
	0x62943458,0x62951379,0x62960417,0x62970a0a,0x629818e9,0x6299345a,0x629a0544,0x629b0e77,
	0x629c345c,0x629d345e,0x629e3460,0x629f138e,0x62a00a47,0x62a10c4d,0x62a20fb2,0x62a33462,
	0x62a40740,0x62a5009b,0x62a63464,0x62a73466,0x62a80e96,0x62a93468,0x62aa346a,0x62ab0eac,
	0x62ac12c1,0x62ad346c,0x62ae346e,0x62af3470,0x62b03472,0x62b1009c,0x62b23474,0x62b33476,
	0x62b43478,0x62b503a5,0x62b6347a,0x62b7347c,0x62b8347e,0x62b90d70,0x62ba3480,0x62bb021a,
	0x62bc1612,0x62bd0276,0x62be3482,0x62bf0d50,0x62c03484,0x62c13486,0x62c20526,0x62c33488,
	0x62c419bd,0x62c5034a,0x62c601c7,0x62c70d8f,0x62c80deb,0x62c90aab,0x62ca0547,0x62cb348a,
	0x62cc007a,0x62cd0e55,0x62ce0bae,0x62cf348c,0x62d00644,0x62d1348e,0x62d20996,0x62d313b3,
	0x62d40050,0x62d53490,0x62d613a1,0x62d70e1f,0x62d8097a,0x62d919f8,0x62da0e66,0x62db18db,
	0x62dc005f,0x62dd3492,0x62de3494,0x62df0de3,0x62e03496,0x62e13498,0x62e20c06,0x62e30877,
	0x62e4349a,0x62e5176c,0x62e60ac3,0x62e70e10,0x62e8014d,0x62e91879,0x62ea349c,0x62eb349e,
	0x62ec0aa6,0x62ed11bd,0x62ee08f4,0x62ef1927,0x62f034a0,0x62f105f4,0x62f234a2,0x62f31053,
	0x62f41213,0x62f534a4,0x62f6184e,0x62f70a10,0x62f834a6,0x62f934a8,0x62fa34aa,0x62fb34ac,
	0x62fc0eec,0x62fd19d7,0x62fe119f,0x62ff0da0,0x630034ae,0x63010256,0x63020640,0x630334b0,
	0x630434b2,0x630534b4,0x630634b6,0x63071955,0x63080fd8,0x63090026,0x630a34b8,0x630b34ba,
	0x630c34bc,0x630d34be,0x630e0a5f,0x630f34c0,0x631034c2,0x6311133e,0x631234c4,0x631334c6,
	0x631434c8,0x631534ca,0x631613b8,0x631734cc,0x631834ce,0x631934d0,0x631a1968,0x631b0c42,
	0x631c34d2,0x631d143e,0x631e12b8,0x631f154e,0x63200dc4,0x63210364,0x632208cf,0x63231929,
	0x63240813,0x63250795,0x632634d4,0x632734d6,0x63280007,0x632934d8,0x632a0e31,0x632b0320,
	0x632c34da,0x632d34dc,0x632e34de,0x632f1917,0x633034e0,0x633134e2,0x633212a0,0x633334e4,
	0x633434e6,0x633534e8,0x633634ea,0x633734ec,0x633834ee,0x63391707,0x633a135d,0x633b34f0,
	0x633c34f2,0x633d13d1,0x633e34f4,0x633f34f6,0x634034f8,0x634134fa,0x6342146a,0x634309de,
	0x634434fc,0x63451371,0x63460aa2,0x634734fe,0x63483500,0x634919fa,0x634a3502,0x634b0c7b,
	0x634c004d,0x634d06b4,0x634e113e,0x634f0e00,0x635009ab,0x63513504,0x63523506,0x63533508,
	0x6354350a,0x63550175,0x6356350c,0x6357350e,0x63583510,0x63593512,0x635a3514,0x635b3516,
	0x635c3518,0x635d351a,0x635e0ae6,0x635f1299,0x6360351c,0x6361087c,0x6362076e,0x63630375,
	0x6364351e,0x63653520,0x63663522,0x63670ea5,0x63683524,0x63690ba8,0x636a3526,0x636b3528,
	0x636c352a,0x636d0064,0x636e099f,0x636f352c,0x6370352e,0x63710009,0x63723530,0x63733532,
	0x63743534,0x63753536,0x637602bc,0x637708f9,0x63783538,0x6379353a,0x637a0da9,0x637b0df1,
	0x637c353c,0x637d353e,0x637e3540,0x637f3542,0x638014e6,0x63813544,0x638203b6,0x63833546,
	0x63843548,0x6385354a,0x6386354c,0x6387045a,0x638811da,0x638903d9,0x638a0f13,0x638b354e,
	0x638c18cf,0x638d3550,0x638e0815,0x638f1301,0x63900f72,0x63913552,0x63920e58,0x63933554,
	0x63943556,0x63953558,0x639616b5,0x6397355a,0x639809c4,0x6399355c,0x639a355e,0x639b3560,
	0x639c3562,0x639d3564,0x639e3566,0x639f3568,0x63a00c4a,0x63a1356a,0x63a212e4,0x63a30217,
	0x63a4356c,0x63a508e8,0x63a6356e,0x63a70a46,0x63a81391,0x63a91651,0x63aa0321,0x63ab3570,
	0x63ac097f,0x63ad133c,0x63ae0f90,0x63af3572,0x63b0005e,0x63b13574,0x63b23576,0x63b30c26,
	0x63b40643,0x63b53578,0x63b6357a,0x63b7196e,0x63b80354,0x63b9357c,0x63ba01d3,0x63bb357e,
	0x63bc0651,0x63bd3580,0x63be1823,0x63bf3582,0x63c03584,0x63c13586,0x63c23588,0x63c3358a,
	0x63c417c9,0x63c5358c,0x63c60a87,0x63c7358e,0x63c83590,0x63c910aa,0x63ca3592,0x63cb3594,
	0x63cc3596,0x63cd1a42,0x63ce15cf,0x63cf0d32,0x63d0131f,0x63d13598,0x63d201b4,0x63d3359a,
	0x63d4359c,0x63d5359e,0x63d616c3,0x63d735a0,0x63d835a2,0x63d935a4,0x63da35a6,0x63db35a8,
	0x63dc35aa,0x63dd35ac,0x63de0023,0x63df35ae,0x63e0162a,0x63e11449,0x63e235b0,0x63e301cd,
	0x63e435b2,0x63e535b4,0x63e635b6,0x63e735b8,0x63e835ba,0x63e909ea,0x63ea0965,0x63eb35bc,
	0x63ec35be,0x63ed08ec,0x63ee35c0,0x63ef35c2,0x63f035c4,0x63f135c6,0x63f21153,0x63f335c8,
	0x63f41814,0x63f535ca,0x63f616a8,0x63f735cc,0x63f81892,0x63f935ce,0x63fa35d0,0x63fb35d2,
	0x63fc35d4,0x63fd0ad0,0x63fe35d6,0x63ff0ff2,0x640001d4,0x640105bb,0x64020c10,0x640335d8,
	0x640435da,0x640508d7,0x640635dc,0x640735de,0x640835e0,0x640935e2,0x640a35e4,0x640b01c9,
	0x640c18bb,0x640d35e6,0x640e35e8,0x640f0162,0x6410029e,0x641135ea,0x641235ec,0x64130316,
	0x641410ed,0x641535ee,0x641635f0,0x641735f2,0x641835f4,0x641935f6,0x641a35f8,0x641b086f,
	0x641c125e,0x641d35fa,0x641e05a8,0x641f35fc,0x64201229,0x642110ea,0x642235fe,0x64233600,
	0x64243602,0x64253604,0x64260e35,0x64273606,0x64283608,0x6429360a,0x642a12ed,0x642b360c,
	0x642c006d,0x642d0326,0x642e360e,0x642f3610,0x64303612,0x64313614,0x64323616,0x64333618,
	0x64340f87,0x6435361a,0x6436361c,0x6437361e,0x64383620,0x64393622,0x643a1552,0x643b3624,
	0x643c3626,0x643d01ba,0x643e3628,0x643f05c7,0x6440362a,0x6441048b,0x6442362c,0x6443362e,
	0x6444115f,0x644511eb,0x64460065,0x64471694,0x6448013b,0x64493630,0x644a12d0,0x644b3632,
	0x644c3634,0x644d3636,0x644e3638,0x644f363a,0x6450363c,0x6451363e,0x6452014c,0x64533640,
	0x6454120e,0x64553642,0x64563644,0x64573646,0x645818a8,0x64593648,0x645a364a,0x645b364c,
	0x645c364e,0x645d3650,0x645e0c6b,0x645f3652,0x64603654,0x64613656,0x64623658,0x6463365a,
	0x6464365c,0x6465365e,0x64663660,0x64670305,0x64683662,0x64690d6c,0x646a3664,0x646b3666,
	0x646c3668,0x646d194b,0x646e366a,0x646f366c,0x6470366e,0x64713670,0x64723672,0x64733674,
	0x64743676,0x64753678,0x6476367a,0x6477367c,0x64780d64,0x64790d69,0x647a18ef,0x647b367e,
	0x647c3680,0x647d3682,0x647e3684,0x647f3686,0x64803688,0x6481368a,0x64820ba0,0x6483368c,
	0x64841748,0x648509ba,0x6486368e,0x64870ee7,0x64883690,0x64893692,0x648a3694,0x648b3696,
	0x648c3698,0x648d369a,0x648e369c,0x648f369e,0x649036a0,0x64910230,0x649210d5,0x649336a2,
	0x649436a4,0x6495123a,0x649606b8,0x649736a6,0x649836a8,0x64991a5c,0x649a36aa,0x649b36ac,
	0x649c36ae,0x649d36b0,0x649e19e8,0x649f36b2,0x64a036b4,0x64a136b6,0x64a236b8,0x64a336ba,
	0x64a40218,0x64a536bc,0x64a636be,0x64a736c0,0x64a836c2,0x64a90b8d,0x64aa36c4,0x64ab36c6,
	0x64ac0fcf,0x64ad0154,0x64ae0318,0x64af36c8,0x64b019de,0x64b136ca,0x64b236cc,0x64b336ce,
	0x64b436d0,0x64b50df3,0x64b636d2,0x64b71554,0x64b80c17,0x64b936d4,0x64ba02fd,0x64bb36d6,
	0x64bc06ba,0x64bd36d8,0x64be36da,0x64bf36dc,0x64c0058e,0x64c136de,0x64c20b05,0x64c336e0,
	0x64c436e2,0x64c5112c,0x64c636e4,0x64c736e6,0x64c836e8,0x64c936ea,0x64ca36ec,0x64cb36ee,
	0x64cc36f0,0x64cd019e,0x64ce0fff,0x64cf36f2,0x64d00778,0x64d136f4,0x64d20fea,0x64d336f6,
	0x64d436f8,0x64d536fa,0x64d636fc,0x64d70ec6,0x64d8016a,0x64d936fe,0x64da3700,0x64db3702,
	0x64dc3704,0x64dd3706,0x64de126a,0x64df3708,0x64e0370a,0x64e1370c,0x64e21a08,0x64e3370e,
	0x64e4158c,0x64e53710,0x64e60180,0x64e73712,0x64e83714,0x64e93716,0x64ea3718,0x64eb371a,
	0x64ec371c,0x64ed371e,0x64ee3720,0x64ef3722,0x64f03724,0x64f13726,0x64f23728,0x64f3372a,
	0x64f4372c,0x64f5372e,0x64f63730,0x64f73732,0x64f83734,0x64f93736,0x64fa3738,0x64fb373a,
	0x64fc373c,0x64fd373e,0x64fe3740,0x64ff3742,0x65000e5f,0x65013744,0x65023746,0x65033748,
	0x6504374a,0x6505374c,0x6506374e,0x65073750,0x65083752,0x650907c5,0x650a3754,0x650b3756,
	0x650c3758,0x650d375a,0x650e375c,0x650f375e,0x65103760,0x65113762,0x6512185e,0x65133764,
	0x65143766,0x65153768,0x6516376a,0x6517376c,0x6518107c,0x6519376e,0x651a3770,0x651b3772,
	0x651c3774,0x651d3776,0x651e3778,0x651f377a,0x6520377c,0x6521377e,0x65223780,0x65233782,
	0x65243784,0x65251a52,0x65263786,0x65273788,0x6528378a,0x6529378c,0x652a378e,0x652b09d2,
	0x652c3790,0x652d3792,0x652e0dc1,0x652f1931,0x65303794,0x65313796,0x65323798,0x6533379a,
	0x65340f15,0x65350f16,0x653611cf,0x6537379c,0x65381786,0x65390574,0x653a379e,0x653b05e8,
	0x653c37a0,0x653d37a2,0x653e04cc,0x653f192e,0x654037a4,0x654137a6,0x654237a8,0x654337aa,
	0x654437ac,0x6545062e,0x654637ae,0x654737b0,0x65481543,0x65490d14,0x654a37b2,0x654b37b4,
	0x654c039a,0x654d37b6,0x654e37b8,0x654f0d54,0x655037ba,0x65510974,0x655237bc,0x655337be,
	0x655437c0,0x65550267,0x65560030,0x655737c2,0x655837c4,0x655908e0,0x655a37c6,0x655b0b72,
	0x655c37c8,0x655d00e9,0x655e01fe,0x655f37ca,0x656037cc,0x656137ce,0x6562058a,0x656310e6,
	0x656437d0,0x656537d2,0x65660448,0x656737d4,0x656837d6,0x656937d8,0x656a37da,0x656b08da,
	0x656c0953,0x656d37dc,0x656e37de,0x656f37e0,0x65701205,0x657137e2,0x65720fb9,0x657337e4,
	0x65741928,0x657537e6,0x657637e8,0x6577051e,0x657837ea,0x657937ec,0x657a37ee,0x657b37f0,
	0x657c37f2,0x657d37f4,0x657e37f6,0x657f37f8,0x658037fa,0x658137fc,0x658237fe,0x65833800,
	0x65843802,0x65853804,0x65863806,0x6587142c,0x65883808,0x6589380a,0x658a380c,0x658b18a7,
	0x658c0133,0x658d380e,0x658e3810,0x658f3812,0x659004d7,0x6591006c,0x65923814,0x65930acb,
	0x65943816,0x65953818,0x6596381a,0x65970416,0x6598381c,0x65990b9f,0x659a381e,0x659b072e,
	0x659c1550,0x659d3820,0x659e3822,0x659f1905,0x65a03824,0x65a1144c,0x65a23826,0x65a33828,
	0x65a40911,0x65a50262,0x65a6382a,0x65a70548,0x65a8382c,0x65a918b7,0x65aa382e,0x65ab1a00,
	0x65ac3830,0x65ad0439,0x65ae3832,0x65af1234,0x65b01574,0x65b13834,0x65b23836,0x65b33838,
	0x65b4383a,0x65b5383c,0x65b6383e,0x65b73840,0x65b83842,0x65b904bd,0x65ba3844,0x65bb3846,
	0x65bc17b1,0x65bd1193,0x65be3848,0x65bf384a,0x65c0384c,0x65c10e73,0x65c2384e,0x65c318b1,
	0x65c40cba,0x65c50c73,0x65c60e8e,0x65c73850,0x65c83852,0x65c93854,0x65ca3856,0x65cb15d7,
	0x65cc0938,0x65cd3858,0x65ce0de4,0x65cf1a47,0x65d0385a,0x65d1385c,0x65d20be9,0x65d3385e,
	0x65d43860,0x65d53862,0x65d616e9,0x65d70f54,0x65d83864,0x65d93866,0x65da3868,0x65db386a,
	0x65dc386c,0x65dd386e,0x65de3870,0x65df3872,0x65e01457,0x65e13874,0x65e20827,0x65e33876,
	0x65e43878,0x65e51099,0x65e60357,0x65e7096d,0x65e8194f,0x65e9186c,0x65ea387a,0x65eb387c,
	0x65ec15fb,0x65ed15bc,0x65ee0569,0x65ef0aad,0x65f0058f,0x65f106b2,0x65f2387e,0x65f33880,
	0x65f43882,0x65f53884,0x65f6119c,0x65f70a79,0x65f83886,0x65f93888,0x65fa13ec,0x65fb388a,
	0x65fc388c,0x65fd388e,0x65fe3890,0x65ff3892,0x6600183a,0x66013894,0x6602002c,0x66031884,
	0x66043896,0x66053898,0x66060a9b,0x6607389a,0x6608389c,0x6609389e,0x660a06d0,0x660b38a0,
	0x660c01ec,0x660d38a2,0x660e0d58,0x660f07b8,0x661038a4,0x661138a6,0x661238a8,0x661316fd,
	0x6614148a,0x66151571,0x661638aa,0x661738ac,0x661838ae,0x661912d5,0x661a38b0,0x661b38b2,
	0x661c38b4,0x661d185d,0x661e38b6,0x661f157e,0x66201766,0x662138b8,0x662238ba,0x662338bc,
	0x662438be,0x662502c0,0x662638c0,0x66270ce6,0x66281a5e,0x662938c2,0x662a38c4,0x662b38c6,
	0x662c38c8,0x662d18dc,0x662e38ca,0x662f11be,0x663038cc,0x663117ec,0x663238ce,0x663338d0,
	0x66340cc3,0x66350de5,0x663601fc,0x663738d2,0x663838d4,0x663938d6,0x663a38d8,0x663b38da,
	0x663c199d,0x663d38dc,0x663e14f8,0x663f38de,0x664038e0,0x66410209,0x664238e2,0x6643078f,
	0x664438e4,0x664538e6,0x664638e8,0x664738ea,0x664838ec,0x664938ee,0x664a38f0,0x664b092a,
	0x664c1138,0x664d38f2,0x664e38f4,0x664f165e,0x665038f6,0x665138f8,0x66521111,0x6653153f,
	0x665416b3,0x66551845,0x66560796,0x665706a9,0x665838fa,0x665938fc,0x665a13d2,0x665b38fe,
	0x665c3900,0x665d3902,0x665e3904,0x665f118a,0x66603906,0x6661016f,0x66623908,0x6663390a,
	0x6664147a,0x6665390c,0x666607b0,0x6667390e,0x66680225,0x66693910,0x666a3912,0x666b3914,
	0x666c3916,0x666d3918,0x666e0f26,0x666f0946,0x6670149b,0x6671391a,0x6672391c,0x6673391e,
	0x66740ffd,0x66753920,0x6676093a,0x6677066f,0x66783922,0x66793924,0x667a1973,0x667b3926,
	0x667c3928,0x667d392a,0x667e0b8c,0x667f392c,0x6680392e,0x66813930,0x66821860,0x66833932,
	0x668415d1,0x66853934,0x66863936,0x668714d6,0x66883938,0x6689393a,0x668a393c,0x668b393e,
	0x668c0a89,0x668d3940,0x668e3942,0x668f3944,0x66903946,0x669111f4,0x66923948,0x6693394a,
	0x6694394c,0x6695394e,0x66960e2e,0x66970029,0x66983950,0x66993952,0x669a3954,0x669b3956,
	0x669c3958,0x669d0d5e,0x669e395a,0x669f395c,0x66a0395e,0x66a13960,0x66a23962,0x66a33964,
	0x66a43966,0x66a53968,0x66a6396a,0x66a70017,0x66a80832,0x66a9396c,0x66aa396e,0x66ab3970,
	0x66ac3972,0x66ad3974,0x66ae0d9d,0x66af3976,0x66b03978,0x66b1397a,0x66b2397c,0x66b3397e,
	0x66b400a0,0x66b53980,0x66b63982,0x66b73984,0x66b83986,0x66b914eb,0x66ba3988,0x66bb398a,
	0x66bc398c,0x66bd398e,0x66be1399,0x66bf3990,0x66c03992,0x66c13994,0x66c23996,0x66c33998,
	0x66c4399a,0x66c5399c,0x66c6399e,0x66c739a0,0x66c839a2,0x66c939a4,0x66ca39a6,0x66cb39a8,
	0x66cc39aa,0x66cd39ac,0x66ce39ae,0x66cf39b0,0x66d039b2,0x66d139b4,0x66d239b6,0x66d339b8,
	0x66d439ba,0x66d539bc,0x66d639be,0x66d739c0,0x66d839c2,0x66d911fa,0x66da39c4,0x66db15f7,
	0x66dc16a1,0x66dd00a2,0x66de39c6,0x66df39c8,0x66e039ca,0x66e139cc,0x66e239ce,0x66e339d0,
	0x66e439d2,0x66e539d4,0x66e614b1,0x66e739d6,0x66e839d8,0x66e90dc0,0x66ea39da,0x66eb39dc,
	0x66ec39de,0x66ed39e0,0x66ee39e2,0x66ef39e4,0x66f01826,0x66f139e6,0x66f2102c,0x66f316af,
	0x66f405d9,0x66f539e8,0x66f639ea,0x66f706e0,0x66f839ec,0x66f901a0,0x66fa39ee,0x66fb39f0,
	0x66fc0ca2,0x66fd39f2,0x66fe1888,0x66ff132d,0x67001a54,0x670139f4,0x670239f6,0x670339f8,
	0x670439fa,0x670539fc,0x670639fe,0x67073a00,0x67081829,0x6709179a,0x670a10c4,0x670b0e9a,
	0x670c3a02,0x670d0527,0x670e3a04,0x670f3a06,0x6710103a,0x67113a08,0x67123a0a,0x67133a0c,
	0x67141226,0x67151918,0x67163a0e,0x67170ae1,0x67183a10,0x67193a12,0x671a3a14,0x671b13ed,
	0x671c3a16,0x671d020b,0x671e3a18,0x671f0f36,0x67203a1a,0x67213a1c,0x67223a1e,0x67233a20,
	0x67243a22,0x67253a24,0x67260cf8,0x67273a26,0x67280d90,0x67293a28,0x672a141c,0x672b0d71,
	0x672c00c1,0x672d1897,0x672e3a2a,0x672f11fc,0x67303a2c,0x673119a5,0x67323a2e,0x67333a30,
	0x67340f22,0x6735045e,0x67363a32,0x67373a34,0x67383a36,0x67393a38,0x673a07dd,0x673b3a3a,
	0x673c3a3c,0x673d15a4,0x673e3a3e,0x673f3a40,0x674010fe,0x67413a42,0x6742184f,0x6743104f,
	0x67443a44,0x67453a46,0x6746057d,0x67473a48,0x674801b3,0x67491115,0x674a3a4a,0x674b3a4c,
	0x674c1475,0x674d3a4e,0x674e0b2d,0x674f158d,0x67500184,0x67510311,0x67523a50,0x67531144,
	0x67543a52,0x67553a54,0x675618d3,0x67573a56,0x67583a58,0x67593a5a,0x675a3a5c,0x675b3a5e,
	0x675c0430,0x675d3a60,0x675e0f61,0x675f11fe,0x6760059c,0x67611340,0x67623a62,0x67633a64,
	0x67643a66,0x67650ab5,0x67663a68,0x67673a6a,0x67681675,0x67690c8b,0x676a0d35,0x676b3a6c,
	0x676c3a6e,0x676d06be,0x676e3a70,0x676f00a5,0x677008f3,0x67713a72,0x677205a7,0x67731699,
	0x67743a74,0x67750294,0x67763a76,0x67770e4e,0x67783a78,0x67793a7a,0x677a3a7c,0x677b3a7e,
	0x677c19c7,0x677d3a80,0x677e124e,0x677f0072,0x67803a82,0x67810800,0x67823a84,0x67833a86,
	0x67840604,0x67853a88,0x67863a8a,0x67870eb5,0x67883a8c,0x678913e5,0x678a3a8e,0x678b04c1,
	0x678c3a90,0x678d3a92,0x678e3a94,0x678f3a96,0x6790148b,0x67913a98,0x67923a9a,0x67933a9c,
	0x67943a9e,0x6795190e,0x67963aa0,0x67970bb0,0x679810c9,0x67993aa2,0x679a0cd2,0x679b3aa4,
	0x679c068e,0x679d1936,0x679e02e4,0x679f3aa6,0x67a03aa8,0x67a13aaa,0x67a211e2,0x67a3186d,
	0x67a43aac,0x67a50b48,0x67a63aae,0x67a70878,0x67a80236,0x67a93ab0,0x67aa0fa6,0x67ab0501,
	0x67ac3ab2,0x67ad152b,0x67ae3ab4,0x67af0a51,0x67b00efe,0x67b13ab6,0x67b23ab8,0x67b31940,
	0x67b43aba,0x67b5152d,0x67b6085d,0x67b70841,0x67b80602,0x67b93abc,0x67ba3abe,0x67bb3ac0,
	0x67bc3ac2,0x67bd3ac4,0x67be3ac6,0x67bf3ac8,0x67c03aca,0x67c10468,0x67c23acc,0x67c30bcb,
	0x67c40146,0x67c53ace,0x67c63ad0,0x67c73ad2,0x67c83ad4,0x67c93ad6,0x67ca3ad8,0x67cb3ada,
	0x67cc3adc,0x67cd3ade,0x67ce3ae0,0x67cf0063,0x67d00d89,0x67d10582,0x67d20f30,0x67d31076,
	0x67d410a9,0x67d53ae2,0x67d63ae4,0x67d73ae6,0x67d818f7,0x67d914d2,0x67da17a1,0x67db3ae8,
	0x67dc0673,0x67dd13b4,0x67de1a65,0x67df3aea,0x67e00e12,0x67e13aec,0x67e203a6,0x67e33aee,
	0x67e43af0,0x67e501b7,0x67e63af2,0x67e73af4,0x67e83af6,0x67e90971,0x67ea3af8,0x67eb3afa,
	0x67ec087a,0x67ed3afc,0x67ee3afe,0x67ef0a19,0x67f00db0,0x67f119cb,0x67f23b00,0x67f30bef,
	0x67f401cb,0x67f53b02,0x67f63b04,0x67f73b06,0x67f83b08,0x67f93b0a,0x67fa3b0c,0x67fb3b0e,
	0x67fc3b10,0x67fd022e,0x67fe3b12,0x67ff11bf,0x6800193a,0x68013b14,0x68023b16,0x68033b18,
	0x68043b1a,0x680518a2,0x68063b1c,0x6807011d,0x680818be,0x68091966,0x680a0bfa,0x680b040d,
	0x680c0c1e,0x680d3b1e,0x680e0b4d,0x680f0ac4,0x68103b20,0x68111201,0x68123b22,0x68131214,
	0x68143b24,0x68153b26,0x68160f32,0x68170b52,0x68183b28,0x68193b2a,0x681a3b2c,0x681b3b2e,
	0x681c3b30,0x681d063b,0x681e3b32,0x681f3b34,0x68203b36,0x68211544,0x68223b38,0x68233b3a,
	0x68243b3c,0x68253b3e,0x68263b40,0x68273b42,0x68283b44,0x682915b8,0x682a19ab,0x682b3b46,
	0x682c3b48,0x682d3b4a,0x682e3b4c,0x682f3b4e,0x68303b50,0x68313b52,0x68320a11,0x68330af0,
	0x68343b54,0x68353b56,0x68363b58,0x68371683,0x683806e2,0x683905d3,0x683a3b5a,0x683b3b5c,
	0x683c05c0,0x683d1854,0x683e0c43,0x683f3b5e,0x684008f7,0x684106fa,0x68420676,0x68431307,
	0x68440659,0x68451403,0x68460a7c,0x68473b60,0x68480027,0x6849001a,0x684a09b3,0x684b3b62,
	0x684c19fb,0x684d3b64,0x684e1969,0x684f3b66,0x68501368,0x685110e7,0x68523b68,0x68530762,
	0x68540901,0x68550972,0x68563b6a,0x68573b6c,0x68583b6e,0x68593b70,0x685a3b72,0x685b3b74,
	0x685c3b76,0x685d3b78,0x685e3b7a,0x685f3b7c,0x68601614,0x68611080,0x68621901,0x6863036b,
	0x68640f33,0x68650fc1,0x68660756,0x68670679,0x686808b0,0x686919e3,0x686a3b7e,0x686b12a1,
	0x686c3b80,0x686d3b82,0x686e3b84,0x686f3b86,0x68703b88,0x68713b8a,0x68723b8c,0x68733b8e,
	0x68740536,0x68753b90,0x68761372,0x687709c5,0x68783b92,0x68793b94,0x687a3b96,0x687b3b98,
	0x687c3b9a,0x687d3b9c,0x687e3b9e,0x687f3ba0,0x68803ba2,0x68810b80,0x68823ba4,0x6883135e,
	0x68843ba6,0x68850cd6,0x6886007f,0x68873ba8,0x68883baa,0x68893bac,0x688a3bae,0x688b3bb0,
	0x688c3bb2,0x688d3bb4,0x688e3bb6,0x688f0631,0x68903bb8,0x68913bba,0x68923bbc,0x68931a29,
	0x68943bbe,0x68953bc0,0x68963bc2,0x689705e2,0x68983bc4,0x68993bc6,0x689a3bc8,0x689b3bca,
	0x689c3bcc,0x689d3bce,0x689e3bd0,0x689f3bd2,0x68a03bd4,0x68a13bd6,0x68a2113f,0x68a33bd8,
	0x68a43bda,0x68a53bdc,0x68a60d04,0x68a7145c,0x68a80b18,0x68a93bde,0x68aa3be0,0x68ab3be2,
	0x68ac3be4,0x68ad12a2,0x68ae3be6,0x68af131a,0x68b0155d,0x68b13be8,0x68b23bea,0x68b311e6,
	0x68b43bec,0x68b504bb,0x68b63bee,0x68b73bf0,0x68b83bf2,0x68b93bf4,0x68ba3bf6,0x68bb3bf8,
	0x68bc3bfa,0x68bd3bfc,0x68be3bfe,0x68bf3c00,0x68c00880,0x68c13c02,0x68c20bd1,0x68c33c04,
	0x68c43c06,0x68c53c08,0x68c63c0a,0x68c73c0c,0x68c83c0e,0x68c90d25,0x68ca3c10,0x68cb0f4f,
	0x68cc3c12,0x68cd0682,0x68ce3c14,0x68cf3c16,0x68d03c18,0x68d13c1a,0x68d20086,0x68d33c1c,
	0x68d43c1e,0x68d51a32,0x68d63c20,0x68d73c22,0x68d80807,0x68d93c24,0x68da0e9d,0x68db3c26,
	0x68dc3c28,0x68dd3c2a,0x68de3c2c,0x68df3c2e,0x68e012eb,0x68e13c30,0x68e23c32,0x68e303b0,
	0x68e43c34,0x68e53c36,0x68e63c38,0x68e73c3a,0x68e83c3c,0x68e93c3e,0x68ea3c40,0x68eb3c42,
	0x68ec3c44,0x68ed3c46,0x68ee10fc,0x68ef3c48,0x68f002bd,0x68f10b12,0x68f23c4a,0x68f33c4c,
	0x68f43c4e,0x68f50a1f,0x68f63c50,0x68f73c52,0x68f83c54,0x68f918e4,0x68fa064a,0x68fb3c56,
	0x68fc04f2,0x68fd3c58,0x68fe3c5a,0x68ff3c5c,0x69003c5e,0x6901068f,0x69023c60,0x69033c62,
	0x69043c64,0x690516e8,0x69063c66,0x69073c68,0x69083c6a,0x69093c6c,0x690a3c6e,0x690b0b81,
	0x690c3c70,0x690d1947,0x690e19ec,0x690f3c72,0x69100980,0x69113c74,0x691208c2,0x69133c76,
	0x69143c78,0x69153c7a,0x69163c7c,0x69173c7e,0x69183c80,0x69193c82,0x691a3c84,0x691b3c86,
	0x691c3c88,0x691d3c8a,0x691e3c8c,0x691f0425,0x69200fa0,0x69213c8e,0x69223c90,0x69233c92,
	0x69240c5a,0x69253c94,0x69263c96,0x69273c98,0x69283c9a,0x69293c9c,0x692a3c9e,0x692b3ca0,
	0x692c3ca2,0x692d13b2,0x692e3ca4,0x692f3ca6,0x693016a4,0x69313ca8,0x69323caa,0x69333cac,
	0x6934043b,0x69353cae,0x69363cb0,0x69373cb2,0x69383cb4,0x6939117c,0x693a3cb6,0x693b3cb8,
	0x693c3cba,0x693d02ac,0x693e3cbc,0x693f02c1,0x69403cbe,0x69413cc0,0x69421894,0x69433cc2,
	0x69443cc4,0x69453cc6,0x69463cc8,0x69473cca,0x69483ccc,0x69493cce,0x694a3cd0,0x694b3cd2,
	0x694c3cd4,0x694d3cd6,0x694e3cd8,0x694f3cda,0x69503cdc,0x69513cde,0x69523ce0,0x69533ce2,
	0x69541548,0x69553ce4,0x69563ce6,0x6957089a,0x69583ce8,0x69593cea,0x695a0298,0x695b3cec,
	0x695c3cee,0x695d0b7c,0x695e0b13,0x695f3cf0,0x69600db9,0x69613cf2,0x69623cf4,0x69630cdb,
	0x69643cf6,0x69653cf8,0x696615e2,0x69673cfa,0x69683cfc,0x69693cfe,0x696a3d00,0x696b080a,
	0x696c3d02,0x696d3d04,0x696e0297,0x696f3d06,0x69703d08,0x697102ee,0x69723d0a,0x69733d0c,
	0x69743d0e,0x69753d10,0x69763d12,0x697709f3,0x69781018,0x69791758,0x697a3d14,0x697b3d16,
	0x697c0c0b,0x697d3d18,0x697e3d1a,0x697f3d1c,0x69800ef2,0x69813d1e,0x6982057a,0x69833d20,
	0x69840ad2,0x69853d22,0x698617cd,0x6987022b,0x69880c70,0x6989098f,0x698a3d24,0x698b3d26,
	0x698c3d28,0x698d1561,0x698e3d2a,0x698f3d2c,0x69903d2e,0x69913d30,0x69923d32,0x69933d34,
	0x69940add,0x699510a4,0x69963d36,0x69973d38,0x69980990,0x69993d3a,0x699a3d3c,0x699b1908,
	0x699c0083,0x699d3d3e,0x699e3d40,0x699f3d42,0x69a03d44,0x69a13d46,0x69a23d48,0x69a33d4a,
	0x69a43d4c,0x69a53d4e,0x69a63d50,0x69a704de,0x69a818a6,0x69a93d52,0x69aa3d54,0x69ab129c,
	0x69ac3d56,0x69ad1562,0x69ae3d58,0x69af3d5a,0x69b03d5c,0x69b10306,0x69b23d5e,0x69b33d60,
	0x69b40beb,0x69b53d62,0x69b63d64,0x69b7106c,0x69b83d66,0x69b93d68,0x69ba3d6a,0x69bb12bb,
	0x69bc3d6c,0x69bd3d6e,0x69be3d70,0x69bf3d72,0x69c03d74,0x69c105aa,0x69c23d76,0x69c33d78,
	0x69c43d7a,0x69c53d7c,0x69c63d7e,0x69c73d80,0x69c83d82,0x69c93d84,0x69ca122b,0x69cb3d86,
	0x69cc02be,0x69cd3d88,0x69ce01bc,0x69cf3d8a,0x69d0075a,0x69d13d8c,0x69d23d8e,0x69d33d90,
	0x69d405a2,0x69d53d92,0x69d63d94,0x69d73d96,0x69d83d98,0x69d93d9a,0x69da3d9c,0x69db09fb,
	0x69dc3d9e,0x69dd3da0,0x69de3da2,0x69df0136,0x69e019b1,0x69e13da4,0x69e23da6,0x69e33da8,
	0x69e43daa,0x69e53dac,0x69e63dae,0x69e73db0,0x69e83db2,0x69e93db4,0x69ea3db6,0x69eb3db8,
	0x69ec3dba,0x69ed0f64,0x69ee3dbc,0x69ef3dbe,0x69f03dc0,0x69f13dc2,0x69f20735,0x69f33dc4,
	0x69f43dc6,0x69f53dc8,0x69f63dca,0x69f73dcc,0x69f83dce,0x69f93dd0,0x69fa3dd2,0x69fb3dd4,
	0x69fc3dd6,0x69fd01a3,0x69fe3dd8,0x69ff0922,0x6a003dda,0x6a013ddc,0x6a023dde,0x6a033de0,
	0x6a043de2,0x6a053de4,0x6a063de6,0x6a073de8,0x6a083dea,0x6a093dec,0x6a0a04ad,0x6a0b3dee,
	0x6a0c3df0,0x6a0d3df2,0x6a0e3df4,0x6a0f3df6,0x6a103df8,0x6a113dfa,0x6a123dfc,0x6a133dfe,
	0x6a143e00,0x6a153e02,0x6a163e04,0x6a170289,0x6a1812f0,0x6a193e06,0x6a1a3e08,0x6a1b3e0a,
	0x6a1c3e0c,0x6a1d3e0e,0x6a1e3e10,0x6a1f18ca,0x6a203e12,0x6a210d6a,0x6a223e14,0x6a233e16,
	0x6a243e18,0x6a253e1a,0x6a263e1c,0x6a273e1e,0x6a2814ab,0x6a293e20,0x6a2a06fc,0x6a2b3e22,
	0x6a2c3e24,0x6a2d3e26,0x6a2e3e28,0x6a2f0fb0,0x6a303e2a,0x6a31174b,0x6a323e2c,0x6a333e2e,
	0x6a343e30,0x6a350fc5,0x6a363e32,0x6a373e34,0x6a383e36,0x6a393e38,0x6a3a3e3a,0x6a3b3e3c,
	0x6a3c3e3e,0x6a3d1a5a,0x6a3e1832,0x6a3f3e40,0x6a403e42,0x6a413e44,0x6a423e46,0x6a433e48,
	0x6a44058d,0x6a453e4a,0x6a463e4c,0x6a470fbb,0x6a483e4e,0x6a493e50,0x6a4a3e52,0x6a4b3e54,
	0x6a4c3e56,0x6a4d3e58,0x6a4e3e5a,0x6a4f3e5c,0x6a5013ae,0x6a513e5e,0x6a523e60,0x6a533e62,
	0x6a543e64,0x6a553e66,0x6a563e68,0x6a573e6a,0x6a580989,0x6a590241,0x6a5a3e6c,0x6a5b09cc,
	0x6a5c3e6e,0x6a5d3e70,0x6a5e3e72,0x6a5f3e74,0x6a603e76,0x6a611528,0x6a623e78,0x6a633e7a,
	0x6a643e7c,0x6a6519b3,0x6a663e7e,0x6a673e80,0x6a683e82,0x6a693e84,0x6a6a3e86,0x6a6b3e88,
	0x6a6c3e8a,0x6a6d3e8c,0x6a6e3e8e,0x6a6f3e90,0x6a703e92,0x6a710291,0x6a723e94,0x6a733e96,
	0x6a743e98,0x6a753e9a,0x6a763e9c,0x6a773e9e,0x6a783ea0,0x6a790c28,0x6a7a3ea2,0x6a7b3ea4,
	0x6a7c181b,0x6a7d3ea6,0x6a7e3ea8,0x6a7f3eaa,0x6a8012dc,0x6a813eac,0x6a823eae,0x6a833eb0,
	0x6a8414b9,0x6a853eb2,0x6a863eb4,0x6a873eb6,0x6a883eb8,0x6a893eba,0x6a8a3ebc,0x6a8b3ebe,
	0x6a8c3ec0,0x6a8d3ec2,0x6a8e0feb,0x6a8f3ec4,0x6a901649,0x6a910b01,0x6a923ec6,0x6a933ec8,
	0x6a943eca,0x6a953ecc,0x6a963ece,0x6a97016b,0x6a983ed0,0x6a993ed2,0x6a9a3ed4,0x6a9b3ed6,
	0x6a9c3ed8,0x6a9d3eda,0x6a9e3edc,0x6a9f3ede,0x6aa01000,0x6aa13ee0,0x6aa23ee2,0x6aa33ee4,
	0x6aa43ee6,0x6aa53ee8,0x6aa63eea,0x6aa73eec,0x6aa83eee,0x6aa90bc0,0x6aaa3ef0,0x6aab01bf,
	0x6aac0cf9,0x6aad3ef2,0x6aae3ef4,0x6aaf3ef6,0x6ab03ef8,0x6ab13efa,0x6ab23efc,0x6ab33efe,
	0x6ab43f00,0x6ab53f02,0x6ab63f04,0x6ab73f06,0x6ab83f08,0x6ab93f0a,0x6aba3f0c,0x6abb3f0e,
	0x6abc3f10,0x6abd3f12,0x6abe3f14,0x6abf3f16,0x6ac03f18,0x6ac13f1a,0x6ac23f1c,0x6ac33f1e,
	0x6ac43f20,0x6ac53f22,0x6ac63f24,0x6ac73f26,0x6ac83f28,0x6ac93f2a,0x6aca3f2c,0x6acb3f2e,
	0x6acc3f30,0x6acd3f32,0x6ace3f34,0x6acf3f36,0x6ad03f38,0x6ad13f3a,0x6ad23f3c,0x6ad33f3e,
	0x6ad43f40,0x6ad53f42,0x6ad63f44,0x6ad73f46,0x6ad83f48,0x6ad93f4a,0x6ada3f4c,0x6adb3f4e,
	0x6adc3f50,0x6add3f52,0x6ade3f54,0x6adf3f56,0x6ae03f58,0x6ae13f5a,0x6ae23f5c,0x6ae33f5e,
	0x6ae43f60,0x6ae53f62,0x6ae63f64,0x6ae73f66,0x6ae83f68,0x6ae93f6a,0x6aea3f6c,0x6aeb3f6e,
	0x6aec3f70,0x6aed3f72,0x6aee3f74,0x6aef3f76,0x6af03f78,0x6af13f7a,0x6af23f7c,0x6af33f7e,
	0x6af43f80,0x6af53f82,0x6af63f84,0x6af73f86,0x6af83f88,0x6af93f8a,0x6afa3f8c,0x6afb3f8e,
	0x6afc3f90,0x6afd3f92,0x6afe3f94,0x6aff3f96,0x6b003f98,0x6b013f9a,0x6b023f9c,0x6b033f9e,
	0x6b043fa0,0x6b053fa2,0x6b063fa4,0x6b073fa6,0x6b083fa8,0x6b093faa,0x6b0a3fac,0x6b0b3fae,
	0x6b0c3fb0,0x6b0d3fb2,0x6b0e3fb4,0x6b0f3fb6,0x6b103fb8,0x6b113fba,0x6b123fbc,0x6b133fbe,
	0x6b143fc0,0x6b153fc2,0x6b163fc4,0x6b173fc6,0x6b183fc8,0x6b193fca,0x6b1a3fcc,0x6b1b3fce,
	0x6b1c3fd0,0x6b1d3fd2,0x6b1e3fd4,0x6b1f3fd6,0x6b200f9a,0x6b2102de,0x6b22075d,0x6b231572,
	0x6b2417b7,0x6b253fd8,0x6b263fda,0x6b270e41,0x6b283fdc,0x6b293fde,0x6b2a3fe0,0x6b2b3fe2,
	0x6b2c3fe4,0x6b2d3fe6,0x6b2e3fe8,0x6b2f3fea,0x6b303fec,0x6b313fee,0x6b3217f3,0x6b333ff0,
	0x6b343ff2,0x6b353ff4,0x6b363ff6,0x6b371496,0x6b383ff8,0x6b390f3b,0x6b3a0f37,0x6b3b3ffa,
	0x6b3c3ffc,0x6b3d3ffe,0x6b3e0a6d,0x6b3f4000,0x6b404002,0x6b414004,0x6b424006,0x6b43110d,
	0x6b444008,0x6b45400a,0x6b461575,0x6b471549,0x6b48400c,0x6b490fa1,0x6b4a400e,0x6b4b4010,
	0x6b4c05bc,0x6b4d4012,0x6b4e4014,0x6b4f4016,0x6b504018,0x6b51401a,0x6b52401c,0x6b53401e,
	0x6b544020,0x6b554022,0x6b564024,0x6b574026,0x6b584028,0x6b591162,0x6b5a402a,0x6b5b402c,
	0x6b5c402e,0x6b5d4030,0x6b5e4032,0x6b5f4034,0x6b604036,0x6b614038,0x6b62194e,0x6b63192a,
	0x6b6402dd,0x6b650178,0x6b661468,0x6b670f45,0x6b68403a,0x6b69403c,0x6b6a13c2,0x6b6b403e,
	0x6b6c4040,0x6b6d4042,0x6b6e4044,0x6b6f4046,0x6b704048,0x6b71404a,0x6b72404c,0x6b73404e,
	0x6b744050,0x6b754052,0x6b764054,0x6b774056,0x6b784058,0x6b790335,0x6b7a405a,0x6b7b123c,
	0x6b7c0864,0x6b7d405c,0x6b7e405e,0x6b7f4060,0x6b804062,0x6b810d72,0x6b8202f3,0x6b83166e,
	0x6b841339,0x6b854064,0x6b86033f,0x6b871132,0x6b884066,0x6b89160b,0x6b8a11e5,0x6b8b0190,
	0x6b8c4068,0x6b8d0ee3,0x6b8e406a,0x6b8f406c,0x6b90406e,0x6b914070,0x6b921840,0x6b930b7a,
	0x6b944072,0x6b954074,0x6b961948,0x6b974076,0x6b984078,0x6b99407a,0x6b9a034f,0x6b9b0808,
	0x6b9c407c,0x6b9d407e,0x6b9e4080,0x6b9f4082,0x6ba04084,0x6ba1013c,0x6ba24086,0x6ba34088,
	0x6ba4408a,0x6ba5408c,0x6ba6408e,0x6ba74090,0x6ba84092,0x6ba94094,0x6baa1719,0x6bab4096,
	0x6bac4098,0x6bad409a,0x6bae409c,0x6baf409e,0x6bb040a0,0x6bb140a2,0x6bb240a4,0x6bb311de,
	0x6bb40e42,0x6bb50438,0x6bb640a6,0x6bb71728,0x6bb840a8,0x6bb940aa,0x6bba40ac,0x6bbb40ae,
	0x6bbc40b0,0x6bbd40b2,0x6bbe40b4,0x6bbf03cb,0x6bc040b6,0x6bc107a2,0x6bc2061c,0x6bc340b8,
	0x6bc440ba,0x6bc51715,0x6bc640bc,0x6bc740be,0x6bc840c0,0x6bc940c2,0x6bca40c4,0x6bcb1458,
	0x6bcc40c6,0x6bcd0d8b,0x6bce40c8,0x6bcf0ce1,0x6bd040ca,0x6bd140cc,0x6bd20422,0x6bd31800,
	0x6bd400d2,0x6bd500dd,0x6bd600e2,0x6bd70eb6,0x6bd840ce,0x6bd900e5,0x6bda40d0,0x6bdb0cb6,
	0x6bdc40d2,0x6bdd40d4,0x6bde40d6,0x6bdf40d8,0x6be040da,0x6be118b0,0x6be240dc,0x6be340de,
	0x6be440e0,0x6be540e2,0x6be640e4,0x6be740e6,0x6be840e8,0x6be940ea,0x6bea0d8a,0x6beb06c7,
	0x6bec40ec,0x6bed40ee,0x6bee40f0,0x6bef12e1,0x6bf040f2,0x6bf140f4,0x6bf240f6,0x6bf3030d,
	0x6bf440f8,0x6bf510e2,0x6bf640fa,0x6bf740fc,0x6bf840fe,0x6bf911ec,0x6bfa4100,0x6bfb4102,
	0x6bfc4104,0x6bfd089b,0x6bfe4106,0x6bff4108,0x6c00410a,0x6c01410c,0x6c02410e,0x6c034110,
	0x6c044112,0x6c0501ff,0x6c060f29,0x6c070c2a,0x6c084114,0x6c094116,0x6c0a4118,0x6c0b411a,
	0x6c0c411c,0x6c0d1041,0x6c0e411e,0x6c0f11ae,0x6c100394,0x6c110d47,0x6c124120,0x6c130cb1,
	0x6c140f65,0x6c150ee6,0x6c160dae,0x6c174122,0x6c180371,0x6c1914e2,0x6c1a02a6,0x6c1b04ee,
	0x6c1c4124,0x6c1d4126,0x6c1e4128,0x6c1f052c,0x6c20412a,0x6c210403,0x6c220ff4,0x6c23412c,
	0x6c241729,0x6c25412e,0x6c26069f,0x6c27167f,0x6c28001b,0x6c291629,0x6c2a0a33,0x6c2b4130,
	0x6c2c4132,0x6c2d4134,0x6c2e0360,0x6c2f0c81,0x6c300ffe,0x6c314136,0x6c321835,0x6c334138,
	0x6c34121c,0x6c35121b,0x6c36413a,0x6c37413c,0x6c381779,0x6c39413e,0x6c3a4140,0x6c3b4142,
	0x6c3c4144,0x6c3d139e,0x6c3e4146,0x6c3f4148,0x6c401351,0x6c411933,0x6c42101d,0x6c43414a,
	0x6c44414c,0x6c45414e,0x6c4602fc,0x6c4707a5,0x6c484150,0x6c4906b0,0x6c4a01c2,0x6c4b4152,
	0x6c4c4154,0x6c4d4156,0x6c4e4158,0x6c4f415a,0x6c501486,0x6c51415c,0x6c52415e,0x6c534160,
	0x6c540f67,0x6c551124,0x6c564162,0x6c5706b1,0x6c584164,0x6c594166,0x6c5a4168,0x6c5b1607,
	0x6c5c1240,0x6c5d10ba,0x6c5e05f3,0x6c5f08a4,0x6c600252,0x6c611450,0x6c62416a,0x6c63416c,
	0x6c6412e6,0x6c65416e,0x6c664170,0x6c674172,0x6c680d18,0x6c69061f,0x6c6a13e0,0x6c6b4174,
	0x6c6c4176,0x6c6d4178,0x6c6e417a,0x6c6f417c,0x6c7012c8,0x6c71417e,0x6c7207fd,0x6c734180,
	0x6c740113,0x6c754182,0x6c761437,0x6c774184,0x6c784186,0x6c791597,0x6c7a4188,0x6c7b418a,
	0x6c7c418c,0x6c7d0f6a,0x6c7e04f1,0x6c7f418e,0x6c804190,0x6c810ff1,0x6c8216ca,0x6c831445,
	0x6c844192,0x6c85180e,0x6c8606c2,0x6c874194,0x6c881170,0x6c890221,0x6c8a4196,0x6c8b4198,
	0x6c8c044f,0x6c8d419a,0x6c8e419c,0x6c8f0f2e,0x6c900d93,0x6c91419e,0x6c9241a0,0x6c9312b7,
	0x6c940d27,0x6c9541a2,0x6c9641a4,0x6c9741a6,0x6c9841a8,0x6c9910ff,0x6c9a41aa,0x6c9b0e8b,
	0x6c9c41ac,0x6c9d41ae,0x6c9e41b0,0x6c9f05fa,0x6ca041b2,0x6ca10cd1,0x6ca241b4,0x6ca30500,
	0x6ca40e4a,0x6ca50b44,0x6ca60c51,0x6ca7019a,0x6ca841b6,0x6ca91400,0x6caa0741,0x6cab0d73,
	0x6cac41b8,0x6cad11ff,0x6cae098b,0x6caf41ba,0x6cb041bc,0x6cb113a8,0x6cb213a3,0x6cb306df,
	0x6cb441be,0x6cb541c0,0x6cb641c2,0x6cb741c4,0x6cb804e3,0x6cb9178f,0x6cba41c6,0x6cbb1961,
	0x6cbc18de,0x6cbd0611,0x6cbe18af,0x6cbf1641,0x6cc041c8,0x6cc141ca,0x6cc241cc,0x6cc341ce,
	0x6cc41558,0x6cc5101f,0x6cc641d0,0x6cc741d2,0x6cc841d4,0x6cc91051,0x6cca0159,0x6ccb41d6,
	0x6ccc0d1a,0x6ccd41d8,0x6cce41da,0x6ccf41dc,0x6cd00afa,0x6cd141de,0x6cd241e0,0x6cd30708,
	0x6cd40580,0x6cd504a1,0x6cd60cc2,0x6cd71246,0x6cd841e2,0x6cd941e4,0x6cda41e6,0x6cdb04b6,
	0x6cdc41e8,0x6cdd41ea,0x6cde0e17,0x6cdf41ec,0x6ce00bc9,0x6ce10e81,0x6ce2014e,0x6ce30f6b,
	0x6ce441ee,0x6ce50ddb,0x6ce641f0,0x6ce741f2,0x6ce819c8,0x6ce941f4,0x6cea0b0d,0x6ceb15dc,
	0x6cec41f6,0x6ced41f8,0x6cee0e68,0x6cef0d51,0x6cf012cc,0x6cf1166d,0x6cf241fa,0x6cf3177c,
	0x6cf441fc,0x6cf500ca,0x6cf615eb,0x6cf70bf8,0x6cf80c1c,0x6cf941fe,0x6cfa0c64,0x6cfb1559,
	0x6cfc0f04,0x6cfd187a,0x6cfe0933,0x6cff4200,0x6d004202,0x6d0108f5,0x6d024204,0x6d034206,
	0x6d04079d,0x6d054208,0x6d06420a,0x6d071724,0x6d08420c,0x6d09420e,0x6d0a4210,0x6d0b167a,
	0x6d0c0ba5,0x6d0d4212,0x6d0e0828,0x6d0f4214,0x6d104216,0x6d114218,0x6d1210d6,0x6d13421a,
	0x6d14421c,0x6d15421e,0x6d164220,0x6d1714ba,0x6d184222,0x6d1919a9,0x6d1a08b5,0x6d1b0c65,
	0x6d1c4224,0x6d1d4226,0x6d1e040e,0x6d1f4228,0x6d20422a,0x6d21422c,0x6d22422e,0x6d234230,
	0x6d244232,0x6d250914,0x6d264234,0x6d271412,0x6d284236,0x6d294238,0x6d2a0709,0x6d2b15c0,
	0x6d2c423a,0x6d2d423c,0x6d2e1305,0x6d2f423e,0x6d304240,0x6d310493,0x6d321993,0x6d3310be,
	0x6d344242,0x6d3515ff,0x6d364244,0x6d374246,0x6d384248,0x6d390761,0x6d3a424a,0x6d3b07c6,
	0x6d3c13b9,0x6d3d0f75,0x6d3e0e5b,0x6d3f424c,0x6d40424e,0x6d410be5,0x6d424250,0x6d430842,
	0x6d444252,0x6d450f94,0x6d4608a8,0x6d4708be,0x6d4818ff,0x6d494254,0x6d4a1a01,0x6d4b01ab,
	0x6d4c4256,0x6d4d07a8,0x6d4e0829,0x6d4f0be4,0x6d504258,0x6d5107bc,0x6d52073a,0x6d530e23,
	0x6d541600,0x6d55425a,0x6d56425c,0x6d57425e,0x6d584260,0x6d5918f8,0x6d5a09df,0x6d5b4262,
	0x6d5c0080,0x6d5d4264,0x6d5e1a02,0x6d5f4266,0x6d601492,0x6d614268,0x6d62426a,0x6d63076f,
	0x6d64426c,0x6d65426e,0x6d660f25,0x6d674270,0x6d684272,0x6d6906d1,0x6d6a0ae4,0x6d6b4274,
	0x6d6c4276,0x6d6d4278,0x6d6e0531,0x6d6f145d,0x6d70427a,0x6d71427c,0x6d72427e,0x6d734280,
	0x6d7417ef,0x6d754282,0x6d764284,0x6d770699,0x6d78092b,0x6d794286,0x6d7a4288,0x6d7b428a,
	0x6d7c0ce3,0x6d7d428c,0x6d7e428e,0x6d7f4290,0x6d804292,0x6d814294,0x6d821381,0x6d834296,
	0x6d844298,0x6d850e02,0x6d86429a,0x6d87429c,0x6d881530,0x6d89115c,0x6d8a429e,0x6d8b42a0,
	0x6d8c177f,0x6d8d42a2,0x6d8e14f0,0x6d8f42a4,0x6d9042a6,0x6d911272,0x6d9242a8,0x6d9309ac,
	0x6d9401ae,0x6d95132a,0x6d9642aa,0x6d9742ac,0x6d9842ae,0x6d9942b0,0x6d9a42b2,0x6d9b12ff,
	0x6d9c42b4,0x6d9d0af3,0x6d9e0ab7,0x6d9f0b68,0x6da01404,0x6da11440,0x6da242b6,0x6da30770,
	0x6da4039b,0x6da542b8,0x6da610cf,0x6da70896,0x6da818ce,0x6da910f7,0x6daa0537,0x6dab064c,
	0x6dac42ba,0x6dad42bc,0x6dae1215,0x6daf161d,0x6db042be,0x6db142c0,0x6db216b6,0x6db342c2,
	0x6db442c4,0x6db506aa,0x6db642c6,0x6db742c8,0x6db806e5,0x6db942ca,0x6dba42cc,0x6dbb42ce,
	0x6dbc42d0,0x6dbd42d2,0x6dbe42d4,0x6dbf19fc,0x6dc003c9,0x6dc142d6,0x6dc242d8,0x6dc342da,
	0x6dc41a13,0x6dc51497,0x6dc6153d,0x6dc70f4b,0x6dc842dc,0x6dc942de,0x6dca42e0,0x6dcb0bb2,
	0x6dcc12f8,0x6dcd42e2,0x6dce42e4,0x6dcf42e6,0x6dd042e8,0x6dd111e7,0x6dd242ea,0x6dd342ec,
	0x6dd442ee,0x6dd542f0,0x6dd60dce,0x6dd742f2,0x6dd8130a,0x6dd902eb,0x6dda42f4,0x6ddb42f6,
	0x6ddc42f8,0x6ddd04d9,0x6dde1251,0x6ddf42fa,0x6de00ecc,0x6de1035d,0x6de242fc,0x6de342fe,
	0x6de417af,0x6de54300,0x6de60591,0x6de74302,0x6de84304,0x6de94306,0x6dea4308,0x6deb1731,
	0x6dec030b,0x6ded430a,0x6dee0759,0x6def430c,0x6df0430e,0x6df1116b,0x6df24310,0x6df302c6,
	0x6df44312,0x6df54314,0x6df64316,0x6df707c0,0x6df84318,0x6df91630,0x6dfa431a,0x6dfb1331,
	0x6dfc0d38,0x6dfd431c,0x6dfe431e,0x6dff4320,0x6e004322,0x6e014324,0x6e024326,0x6e034328,
	0x6e04432a,0x6e050ff9,0x6e06432c,0x6e07432e,0x6e084330,0x6e094332,0x6e0a180a,0x6e0b4334,
	0x6e0c0c30,0x6e0d1a2e,0x6e0e0424,0x6e0f4336,0x6e100898,0x6e111184,0x6e124338,0x6e13433a,
	0x6e1417c3,0x6e15433c,0x6e161175,0x6e17117a,0x6e18433e,0x6e194340,0x6e1a19be,0x6e1b4342,
	0x6e1c4344,0x6e1d17b2,0x6e1e4346,0x6e1f4348,0x6e20103c,0x6e210433,0x6e22434a,0x6e231893,
	0x6e240160,0x6e25144a,0x6e26434c,0x6e27434e,0x6e284350,0x6e29142a,0x6e2a4352,0x6e2b155f,
	0x6e2c4354,0x6e2d1425,0x6e2e4356,0x6e2f059b,0x6e304358,0x6e31435a,0x6e3215e1,0x6e33435c,
	0x6e340a2d,0x6e35435e,0x6e364360,0x6e374362,0x6e381795,0x6e394364,0x6e3a0d39,0x6e3b4366,
	0x6e3c4368,0x6e3d436a,0x6e3e436c,0x6e3f436e,0x6e404370,0x6e414372,0x6e424374,0x6e430e5c,
	0x6e440cd9,0x6e454376,0x6e464378,0x6e47437a,0x6e48437c,0x6e49437e,0x6e4a4380,0x6e4b4382,
	0x6e4c4384,0x6e4d138c,0x6e4e0d2b,0x6e4f4386,0x6e504388,0x6e51438a,0x6e52438c,0x6e530e94,
	0x6e54086c,0x6e55438e,0x6e56072f,0x6e574390,0x6e581512,0x6e594392,0x6e5a4394,0x6e5b18c1,
	0x6e5c4396,0x6e5d4398,0x6e5e439a,0x6e5f0783,0x6e60439c,0x6e61439e,0x6e6243a0,0x6e6343a2,
	0x6e6443a4,0x6e6543a6,0x6e6643a8,0x6e6743aa,0x6e6843ac,0x6e6943ae,0x6e6a43b0,0x6e6b08d8,
	0x6e6c43b2,0x6e6d43b4,0x6e6e1634,0x6e6f43b6,0x6e7043b8,0x6e7143ba,0x6e7243bc,0x6e7343be,
	0x6e7443c0,0x6e7543c2,0x6e7643c4,0x6e7743c6,0x6e7843c8,0x6e7943ca,0x6e7a43cc,0x6e7b43ce,
	0x6e7c43d0,0x6e7d43d2,0x6e7e13c6,0x6e7f1195,0x6e8043d4,0x6e8143d6,0x6e8243d8,0x6e830a95,
	0x6e8443da,0x6e85089c,0x6e8615c6,0x6e8743dc,0x6e8843de,0x6e890578,0x6e8a43e0,0x6e8b43e2,
	0x6e8c43e4,0x6e8d43e6,0x6e8e43e8,0x6e8f12ee,0x6e901818,0x6e9143ea,0x6e9243ec,0x6e9343ee,
	0x6e9443f0,0x6e9543f2,0x6e9643f4,0x6e9743f6,0x6e980a36,0x6e9943f8,0x6e9a43fa,0x6e9b43fc,
	0x6e9c0be1,0x6e9d43fe,0x6e9e4400,0x6e9f0d5d,0x6ea04402,0x6ea14404,0x6ea2170f,0x6ea34406,
	0x6ea44408,0x6ea50f27,0x6ea6440a,0x6ea70b16,0x6ea8440c,0x6ea9440e,0x6eaa14a1,0x6eab4410,
	0x6eac4412,0x6ead4414,0x6eae4416,0x6eaf127b,0x6eb04418,0x6eb10fe8,0x6eb2125f,0x6eb3441a,
	0x6eb415aa,0x6eb5441c,0x6eb610a2,0x6eb707c1,0x6eb8441e,0x6eb94420,0x6eba0de8,0x6ebb12b3,
	0x6ebc4422,0x6ebd10bf,0x6ebe4424,0x6ebf4426,0x6ec04428,0x6ec1028d,0x6ec20e6f,0x6ec3442a,
	0x6ec4442c,0x6ec5442e,0x6ec64430,0x6ec703b7,0x6ec84432,0x6ec94434,0x6eca4436,0x6ecb1a18,
	0x6ecc4438,0x6ecd443a,0x6ece443c,0x6ecf054e,0x6ed0443e,0x6ed10751,0x6ed24440,0x6ed31a2b,
	0x6ed41302,0x6ed51317,0x6ed64442,0x6ed700ee,0x6ed84444,0x6ed94446,0x6eda067f,0x6edb4448,
	0x6edc444a,0x6edd444c,0x6ede1974,0x6edf1666,0x6ee01160,0x6ee10ca0,0x6ee21759,0x6ee3444e,
	0x6ee40c7c,0x6ee50ad7,0x6ee60c46,0x6ee74450,0x6ee80134,0x6ee912d1,0x6eea4452,0x6eeb4454,
	0x6eec4456,0x6eed4458,0x6eee445a,0x6eef445c,0x6ef0445e,0x6ef14460,0x6ef24462,0x6ef34464,
	0x6ef40393,0x6ef54466,0x6ef64468,0x6ef7446a,0x6ef8446c,0x6ef90727,0x6efa446e,0x6efb4470,
	0x6efc4472,0x6efd4474,0x6efe4476,0x6eff4478,0x6f00447a,0x6f01447c,0x6f020edc,0x6f03447e,
	0x6f044480,0x6f054482,0x6f060f39,0x6f074484,0x6f084486,0x6f090c36,0x6f0a4488,0x6f0b448a,
	0x6f0c448c,0x6f0d448e,0x6f0e4490,0x6f0f0c13,0x6f104492,0x6f114494,0x6f124496,0x6f130b1f,
	0x6f141656,0x6f1501a2,0x6f164498,0x6f17449a,0x6f18449c,0x6f19449e,0x6f1a44a0,0x6f1b44a2,
	0x6f1c44a4,0x6f1d44a6,0x6f1e44a8,0x6f1f44aa,0x6f200d79,0x6f2144ac,0x6f2244ae,0x6f2344b0,
	0x6f240ad3,0x6f2544b2,0x6f2644b4,0x6f2744b6,0x6f2844b8,0x6f2915d8,0x6f2a16c4,0x6f2b0ca6,
	0x6f2c44ba,0x6f2d0cb3,0x6f2e44bc,0x6f2f0c6c,0x6f3044be,0x6f311208,0x6f3244c0,0x6f3318c8,
	0x6f3444c2,0x6f3544c4,0x6f360776,0x6f3744c6,0x6f3844c8,0x6f3944ca,0x6f3a44cc,0x6f3b44ce,
	0x6f3c44d0,0x6f3d44d2,0x6f3e1684,0x6f3f44d4,0x6f4044d6,0x6f4144d8,0x6f4244da,0x6f4344dc,
	0x6f4444de,0x6f4544e0,0x6f46175b,0x6f471536,0x6f4844e2,0x6f4944e4,0x6f4a44e6,0x6f4b0b7d,
	0x6f4c44e8,0x6f4d140a,0x6f4e44ea,0x6f4f44ec,0x6f5044ee,0x6f5144f0,0x6f5244f2,0x6f5344f4,
	0x6f5444f6,0x6f5544f8,0x6f5644fa,0x6f5744fc,0x6f580e5e,0x6f5944fe,0x6f5a4500,0x6f5b4502,
	0x6f5c0f92,0x6f5d4504,0x6f5e0c39,0x6f5f4506,0x6f604508,0x6f61450a,0x6f620786,0x6f63450c,
	0x6f64450e,0x6f654510,0x6f660b8e,0x6f674512,0x6f684514,0x6f694516,0x6f6a4518,0x6f6b451a,
	0x6f6c451c,0x6f6d12db,0x6f6e020d,0x6f6f451e,0x6f704520,0x6f714522,0x6f72114e,0x6f734524,
	0x6f7419b2,0x6f754526,0x6f764528,0x6f77452a,0x6f78111e,0x6f79452c,0x6f7a01dd,0x6f7b452e,
	0x6f7c136e,0x6f7d4530,0x6f7e4532,0x6f7f4534,0x6f804536,0x6f814538,0x6f82453a,0x6f83453c,
	0x6f840240,0x6f85453e,0x6f864540,0x6f874542,0x6f880219,0x6f89058c,0x6f8a4544,0x6f8b4546,
	0x6f8c123b,0x6f8d1209,0x6f8e0ea1,0x6f8f4548,0x6f90454a,0x6f91454c,0x6f92454e,0x6f934550,
	0x6f944552,0x6f954554,0x6f964556,0x6f974558,0x6f98455a,0x6f99455c,0x6f9a455e,0x6f9b4560,
	0x6f9c0ac9,0x6f9d4562,0x6f9e4564,0x6f9f4566,0x6fa04568,0x6fa1186f,0x6fa2456a,0x6fa3456c,
	0x6fa4456e,0x6fa54570,0x6fa64572,0x6fa70b35,0x6fa84574,0x6fa94576,0x6faa4578,0x6fab457a,
	0x6fac457c,0x6fad457e,0x6fae4580,0x6faf4582,0x6fb04584,0x6fb14586,0x6fb24588,0x6fb30042,
	0x6fb4458a,0x6fb5458c,0x6fb601e0,0x6fb7458e,0x6fb84590,0x6fb912d3,0x6fba4592,0x6fbb4594,
	0x6fbc4596,0x6fbd4598,0x6fbe459a,0x6fbf459c,0x6fc007f5,0x6fc1459e,0x6fc20b6e,0x6fc345a0,
	0x6fc445a2,0x6fc545a4,0x6fc645a6,0x6fc745a8,0x6fc845aa,0x6fc91287,0x6fca45ac,0x6fcb45ae,
	0x6fcc45b0,0x6fcd45b2,0x6fce45b4,0x6fcf45b6,0x6fd045b8,0x6fd10abe,0x6fd20138,0x6fd345ba,
	0x6fd445bc,0x6fd545be,0x6fd645c0,0x6fd745c2,0x6fd845c4,0x6fd945c6,0x6fda45c8,0x6fdb45ca,
	0x6fdc45cc,0x6fdd45ce,0x6fde00fc,0x6fdf45d0,0x6fe006cc,0x6fe110b5,0x6fe245d2,0x6fe345d4,
	0x6fe445d6,0x6fe545d8,0x6fe645da,0x6fe745dc,0x6fe845de,0x6fe945e0,0x6fea45e2,0x6feb45e4,
	0x6fec45e6,0x6fed45e8,0x6fee0f20,0x6fef1a09,0x6ff045ea,0x6ff145ec,0x6ff245ee,0x6ff345f0,
	0x6ff445f2,0x6ff545f4,0x6ff645f6,0x6ff745f8,0x6ff845fa,0x6ff945fc,0x6ffa45fe,0x6ffb4600,
	0x6ffc4602,0x6ffd4604,0x6ffe4606,0x6fff4608,0x7000460a,0x7001460c,0x7002460e,0x70034610,
	0x70044612,0x70054614,0x70064616,0x70074618,0x7008461a,0x7009461c,0x700a461e,0x700b4620,
	0x700c4622,0x700d4624,0x700e4626,0x700f4628,0x7010462a,0x70110f2c,0x7012462c,0x7013462e,
	0x70144630,0x70154632,0x70164634,0x70174636,0x70184638,0x7019463a,0x701a06bc,0x701b175f,
	0x701c463c,0x701d463e,0x701e4640,0x701f4642,0x70204644,0x70214646,0x70224648,0x70231569,
	0x7024464a,0x7025464c,0x7026464e,0x70274650,0x70284652,0x70294654,0x702a4656,0x702b4658,
	0x702c465a,0x702d465c,0x702e465e,0x702f4660,0x70304662,0x70314664,0x70324666,0x70334668,
	0x7034466a,0x703504fd,0x7036466c,0x7037466e,0x70384670,0x70391834,0x703a4672,0x703b4674,
	0x703c4676,0x703d4678,0x703e467a,0x703f467c,0x7040467e,0x70414680,0x70424682,0x70434684,
	0x70444686,0x70454688,0x7046468a,0x7047468c,0x7048468e,0x70494690,0x704a4692,0x704b4694,
	0x704c0653,0x704d4696,0x704e4698,0x704f06d5,0x7050469a,0x7051469c,0x7052469e,0x705346a0,
	0x705446a2,0x705546a4,0x705646a6,0x705746a8,0x705846aa,0x705946ac,0x705a46ae,0x705b46b0,
	0x705c46b2,0x705d46b4,0x705e005d,0x705f46b6,0x706046b8,0x706146ba,0x706246bc,0x706346be,
	0x706446c0,0x706546c2,0x706646c4,0x706746c6,0x706846c8,0x706946ca,0x706a46cc,0x706b07c8,
	0x706c07c7,0x706d0d43,0x706e46ce,0x706f0383,0x70700792,0x707146d0,0x707246d2,0x707346d4,
	0x707446d6,0x70750bc7,0x70761871,0x707746d8,0x70780969,0x707946da,0x707a46dc,0x707b46de,
	0x707c19fe,0x707d46e0,0x707e1851,0x707f0195,0x70801676,0x708146e2,0x708246e4,0x708346e6,
	0x708446e8,0x70850674,0x708646ea,0x708746ec,0x708846ee,0x70890c1d,0x708a02b9,0x708b46f0,
	0x708c46f2,0x708d46f4,0x708e1642,0x708f46f6,0x709046f8,0x709146fa,0x7092020f,0x709346fc,
	0x70941062,0x70950a0c,0x70960450,0x709746fe,0x70984700,0x70991962,0x709a4702,0x709b4704,
	0x709c1410,0x709d0fb5,0x709e4706,0x709f4708,0x70a0470a,0x70a1470c,0x70a2470e,0x70a34710,
	0x70a44712,0x70a54714,0x70a64716,0x70a74718,0x70a8471a,0x70a9471c,0x70aa471e,0x70ab15dd,
	0x70ac0999,0x70ad12e3,0x70ae0e7d,0x70af095d,0x70b04720,0x70b112c3,0x70b24722,0x70b30147,
	0x70b44724,0x70b54726,0x70b64728,0x70b719cc,0x70b818a3,0x70b903bc,0x70ba472a,0x70bb11a0,
	0x70bc0b78,0x70bd0265,0x70be472c,0x70bf472e,0x70c00723,0x70c11225,0x70c20ad6,0x70c31353,
	0x70c44730,0x70c54732,0x70c64734,0x70c74736,0x70c80ba7,0x70c94738,0x70ca167b,0x70cb473a,
	0x70cc473c,0x70cd473e,0x70ce4740,0x70cf4742,0x70d04744,0x70d14746,0x70d24748,0x70d3474a,
	0x70d4474c,0x70d5474e,0x70d64750,0x70d74752,0x70d80702,0x70d90af4,0x70da4754,0x70db19b6,
	0x70dc4756,0x70dd4758,0x70de475a,0x70df162d,0x70e0475c,0x70e1475e,0x70e24760,0x70e34762,
	0x70e40a12,0x70e54764,0x70e604ac,0x70e71140,0x70e816b4,0x70e907ad,0x70ea4766,0x70eb12fd,
	0x70ec092c,0x70ed1085,0x70ee4768,0x70ef1498,0x70f0476a,0x70f1476c,0x70f2476e,0x70f34770,
	0x70f44772,0x70f54774,0x70f64776,0x70f713cf,0x70f84778,0x70f90e98,0x70fa477a,0x70fb477c,
	0x70fc477e,0x70fd0506,0x70fe4780,0x70ff4782,0x71004784,0x71014786,0x71024788,0x7103478a,
	0x7104478c,0x7105478e,0x71064790,0x71074792,0x71084794,0x71091631,0x710a06b5,0x710b4796,
	0x710c4798,0x710d479a,0x710e479c,0x710f479e,0x7110147b,0x711147a0,0x711247a2,0x711306ab,
	0x711447a4,0x71150772,0x71160cf0,0x711747a6,0x7118037a,0x711900b5,0x711a04f3,0x711b47a8,
	0x711c47aa,0x711d47ac,0x711e47ae,0x711f47b0,0x712047b2,0x712147b4,0x712247b6,0x712347b8,
	0x712447ba,0x712547bc,0x712608c3,0x712747be,0x712847c0,0x712947c2,0x712a47c4,0x712b47c6,
	0x712c47c8,0x712d47ca,0x712e47cc,0x712f19f7,0x71301663,0x71311664,0x713247ce,0x713347d0,
	0x713447d2,0x713547d4,0x71361071,0x713747d6,0x713847d8,0x713947da,0x713a47dc,0x713b47de,
	0x713c47e0,0x713d47e2,0x713e47e4,0x713f47e6,0x714047e8,0x714147ea,0x714247ec,0x714347ee,
	0x714447f0,0x7145043c,0x714647f2,0x714747f4,0x714847f6,0x714947f8,0x714a15d2,0x714b47fa,
	0x714c0785,0x714d47fc,0x714e0870,0x714f47fe,0x71504800,0x71514802,0x71524804,0x71534806,
	0x71544808,0x7155480a,0x7156480c,0x7157480e,0x71584810,0x71594812,0x715a4814,0x715b4816,
	0x715c17fd,0x715d4818,0x715e1108,0x715f481a,0x7160481c,0x7161481e,0x71624820,0x71634822,
	0x71640cdc,0x71654824,0x716615c8,0x716718e5,0x716813f6,0x71694826,0x716a4828,0x716b482a,
	0x716c482c,0x716d482e,0x716e19bf,0x716f4830,0x71704832,0x71714834,0x71720090,0x71730732,
	0x71744836,0x71754838,0x7176483a,0x7177483c,0x71780106,0x7179483e,0x717a1395,0x717b4840,
	0x717c4842,0x717d111d,0x717e4844,0x717f4846,0x71804848,0x7181484a,0x7182484c,0x7183484e,
	0x718414a5,0x71854850,0x71864852,0x71874854,0x71884856,0x71894858,0x718a159a,0x718b485a,
	0x718c485c,0x718d485e,0x718e4860,0x718f15f4,0x71904862,0x71914864,0x71924866,0x71934868,
	0x719410a5,0x7195486a,0x7196486c,0x7197486e,0x71980be2,0x719914a6,0x719a4870,0x719b4872,
	0x719c4874,0x719d4876,0x719e4878,0x719f11f3,0x71a01716,0x71a1487a,0x71a2487c,0x71a3487e,
	0x71a44880,0x71a54882,0x71a64884,0x71a74886,0x71a8184a,0x71a94888,0x71aa488a,0x71ab488c,
	0x71ac002f,0x71ad488e,0x71ae4890,0x71af4892,0x71b04894,0x71b14896,0x71b24898,0x71b30ca9,
	0x71b4489a,0x71b51136,0x71b6489c,0x71b7489e,0x71b848a0,0x71b914ac,0x71ba48a2,0x71bb48a4,
	0x71bc48a6,0x71bd48a8,0x71be48aa,0x71bf48ac,0x71c048ae,0x71c148b0,0x71c248b2,0x71c31073,
	0x71c448b4,0x71c548b6,0x71c648b8,0x71c748ba,0x71c848bc,0x71c948be,0x71ca48c0,0x71cb48c2,
	0x71cc48c4,0x71cd48c6,0x71ce0b98,0x71cf48c8,0x71d048ca,0x71d148cc,0x71d248ce,0x71d348d0,
	0x71d404af,0x71d5166a,0x71d648d2,0x71d748d4,0x71d848d6,0x71d948d8,0x71da48da,0x71db48dc,
	0x71dc48de,0x71dd48e0,0x71de48e2,0x71df48e4,0x71e01803,0x71e148e6,0x71e248e8,0x71e348ea,
	0x71e448ec,0x71e51876,0x71e648ee,0x71e71292,0x71e848f0,0x71e948f2,0x71ea48f4,0x71eb48f6,
	0x71ec48f8,0x71ed48fa,0x71ee1568,0x71ef48fc,0x71f048fe,0x71f14900,0x71f24902,0x71f34904,
	0x71f44906,0x71f54908,0x71f6490a,0x71f7490c,0x71f8490e,0x71f914ff,0x71fa4910,0x71fb4912,
	0x71fc4914,0x71fd4916,0x71fe4918,0x71ff491a,0x7200491c,0x7201491e,0x72024920,0x72034922,
	0x72044924,0x72054926,0x720600a1,0x72074928,0x7208492a,0x7209492c,0x720a492e,0x720b4930,
	0x720c4932,0x720d4934,0x720e4936,0x720f4938,0x7210493a,0x7211493c,0x7212493e,0x72134940,
	0x72144942,0x72154944,0x72164946,0x72174948,0x7218494a,0x7219494c,0x721a494e,0x721b4950,
	0x721c4952,0x721d09d1,0x721e4954,0x721f4956,0x72204958,0x7221495a,0x7222495c,0x7223495e,
	0x72244960,0x72254962,0x72264964,0x72274966,0x72280302,0x72294968,0x722a19d6,0x722b496a,
	0x722c0e4f,0x722d496c,0x722e496e,0x722f4970,0x72301810,0x72310011,0x72324972,0x72334974,
	0x72344976,0x723509cd,0x72360552,0x723716a6,0x72380059,0x723903dc,0x723a4978,0x723b168b,
	0x723c497a,0x723d1219,0x723e497c,0x723f0e60,0x7240497e,0x72414980,0x72424982,0x72434984,
	0x72444986,0x72454988,0x7246498a,0x72470ed9,0x72480073,0x7249498c,0x724a498e,0x724b4990,
	0x724c0e59,0x724d0426,0x724e4992,0x724f4994,0x72504996,0x72514998,0x725203e6,0x7253499a,
	0x7254499c,0x7255499e,0x7256179f,0x725749a0,0x725849a2,0x72591616,0x725a49a4,0x725b0e19,
	0x725c49a6,0x725d0ef3,0x725e49a8,0x725f0d84,0x726049aa,0x72610d8d,0x72620ae8,0x726349ac,
	0x726449ae,0x726549b0,0x72660cb8,0x72670d95,0x726849b2,0x72691477,0x726a49b4,0x726b49b6,
	0x726c49b8,0x726d49ba,0x726e0892,0x726f0623,0x727049bc,0x727149be,0x72721181,0x727349c0,
	0x727449c2,0x72750f80,0x727649c4,0x727749c6,0x727849c8,0x72791311,0x727a1493,0x727b49ca,
	0x727c49cc,0x727d49ce,0x727e146b,0x727f0632,0x7280149c,0x72810b1c,0x728249d0,0x728349d2,
	0x728407eb,0x728549d4,0x728649d6,0x728749d8,0x728849da,0x728949dc,0x728a0427,0x728b09a1,
	0x728c49de,0x728d086d,0x728e49e0,0x728f0ed2,0x729049e2,0x729149e4,0x72920a14,0x729349e6,
	0x729449e8,0x729549ea,0x729649ec,0x729749ee,0x729849f0,0x729949f2,0x729a49f4,0x729b49f6,
	0x729c49f8,0x729d49fa,0x729e49fc,0x729f08b8,0x72a049fe,0x72a14a00,0x72a24a02,0x72a34a04,
	0x72a44a06,0x72a54a08,0x72a64a0a,0x72a74a0c,0x72a84a0e,0x72a94a10,0x72aa4a12,0x72ab4a14,
	0x72ac105d,0x72ad105c,0x72ae4a16,0x72af04b5,0x72b0101c,0x72b14a18,0x72b24a1a,0x72b34a1c,
	0x72b40024,0x72b54a1e,0x72b619e7,0x72b7065c,0x72b80c87,0x72b9178d,0x72ba4a20,0x72bb4a22,
	0x72bc4a24,0x72bd4a26,0x72be4a28,0x72bf4a2a,0x72c04a2c,0x72c1183e,0x72c20a72,0x72c30e1c,
	0x72c40397,0x72c54a2e,0x72c64a30,0x72c74a32,0x72c800ac,0x72c94a34,0x72ca4a36,0x72cb4a38,
	0x72cc4a3a,0x72cd0e7c,0x72ce14d0,0x72cf4a3c,0x72d0072b,0x72d14a3e,0x72d204e4,0x72d34a40,
	0x72d44a42,0x72d54a44,0x72d64a46,0x72d70600,0x72d84a48,0x72d9097b,0x72da4a4a,0x72db4a4c,
	0x72dc4a4e,0x72dd4a50,0x72de0e11,0x72df4a52,0x72e006f5,0x72e108d0,0x72e24a54,0x72e34a56,
	0x72e44a58,0x72e54a5a,0x72e64a5c,0x72e74a5e,0x72e8109c,0x72e911d7,0x72ea4a60,0x72eb4a62,
	0x72ec0429,0x72ed14d3,0x72ee1194,0x72ef0a68,0x72f01920,0x72f117ed,0x72f21296,0x72f317c0,
	0x72f400e6,0x72f54a64,0x72f64a66,0x72f709b4,0x72f80b19,0x72f94a68,0x72fa172f,0x72fb1280,
	0x72fc0ada,0x72fd4a6a,0x72fe4a6c,0x72ff4a6e,0x73004a70,0x73010b53,0x73024a72,0x730314fa,
	0x73044a74,0x73054a76,0x73064a78,0x73074a7a,0x73084a7c,0x73094a7e,0x730a0dde,0x730b4a80,
	0x730c4a82,0x730d4a84,0x730e0ba9,0x730f4a86,0x73104a88,0x73114a8a,0x73124a8c,0x73130689,
	0x73144a8e,0x73150d0a,0x731601ee,0x731716dc,0x73184a90,0x73194a92,0x731a4a94,0x731b0cfd,
	0x731c0182,0x731d02f5,0x731e1150,0x731f4a96,0x73204a98,0x73210c56,0x73220730,0x73234a9a,
	0x73244a9c,0x73251416,0x73264a9e,0x73274aa0,0x73284aa2,0x73291580,0x732a19ae,0x732b0cb5,
	0x732c1426,0x732d4aa4,0x732e150a,0x732f4aa6,0x73304aa8,0x73310dc7,0x73324aaa,0x73334aac,
	0x73340712,0x73354aae,0x73364ab0,0x73371797,0x73380cda,0x733901bb,0x733a4ab2,0x733b4ab4,
	0x733c4ab6,0x733d4ab8,0x733e0752,0x733f1819,0x73404aba,0x73414abc,0x73424abe,0x73434ac0,
	0x73444ac2,0x73454ac4,0x73464ac6,0x73474ac8,0x73484aca,0x73494acc,0x734a4ace,0x734b4ad0,
	0x734c4ad2,0x734d0957,0x734e4ad4,0x734f4ad6,0x735018c9,0x73514ad8,0x73520033,0x73534ada,
	0x73544adc,0x73554ade,0x73564ae0,0x735709ca,0x73584ae2,0x73594ae4,0x735a4ae6,0x735b4ae8,
	0x735c4aea,0x735d4aec,0x735e4aee,0x735f4af0,0x73600b96,0x73614af2,0x73624af4,0x73634af6,
	0x73644af8,0x73654afa,0x73664afc,0x73674afe,0x73684b00,0x73694b02,0x736a4b04,0x736b4b06,
	0x736c1565,0x736d12b5,0x736e4b08,0x736f15f5,0x73704b0a,0x73714b0c,0x73724b0e,0x73734b10,
	0x73744b12,0x73754b14,0x73764b16,0x73774b18,0x73784b1a,0x73794b1c,0x737a4b1e,0x737b4b20,
	0x737c4b22,0x737d4b24,0x737e075e,0x737f4b26,0x73804b28,0x73814b2a,0x73824b2c,0x73834b2e,
	0x738415d4,0x73854b30,0x73864b32,0x73870c7f,0x73884b34,0x738917e4,0x738a4b36,0x738b13e2,
	0x738c4b38,0x738d4b3a,0x738e03ee,0x738f4b3c,0x73904b3e,0x739107de,0x73924b40,0x73934b42,
	0x73944b44,0x73954b46,0x7396096a,0x73974b48,0x73984b4a,0x73994b4c,0x739a4b4e,0x739b0c88,
	0x739c4b50,0x739d4b52,0x739e4b54,0x739f0d49,0x73a04b56,0x73a14b58,0x73a2013a,0x73a34b5a,
	0x73a44b5c,0x73a54b5e,0x73a64b60,0x73a74b62,0x73a84b64,0x73a913cd,0x73aa4b66,0x73ab0cd3,
	0x73ac4b68,0x73ad4b6a,0x73ae1411,0x73af075f,0x73b01503,0x73b14b6c,0x73b20bcc,0x73b30340,
	0x73b44b6e,0x73b54b70,0x73b64b72,0x73b703c6,0x73b84b74,0x73b94b76,0x73ba14bb,0x73bb014f,
	0x73bc4b78,0x73bd4b7a,0x73be4b7c,0x73bf4b7e,0x73c00f0d,0x73c14b80,0x73c20a1a,0x73c34b82,
	0x73c44b84,0x73c54b86,0x73c64b88,0x73c74b8a,0x73c80843,0x73c90d4b,0x73ca111a,0x73cb4b8c,
	0x73cc4b8e,0x73cd1900,0x73ce4b90,0x73cf09bf,0x73d004a3,0x73d10bfb,0x73d24b92,0x73d34b94,
	0x73d44b96,0x73d54b98,0x73d64b9a,0x73d74b9c,0x73d84b9e,0x73d905f5,0x73da4ba0,0x73db4ba2,
	0x73dc4ba4,0x73dd4ba6,0x73de0c69,0x73df4ba8,0x73e019ac,0x73e14baa,0x73e24bac,0x73e34bae,
	0x73e44bb0,0x73e50495,0x73e64bb2,0x73e71690,0x73e84bb4,0x73e906fb,0x73ea4bb6,0x73eb4bb8,
	0x73ec4bba,0x73ed0069,0x73ee4bbc,0x73ef4bbe,0x73f04bc0,0x73f14bc2,0x73f20797,0x73f34bc4,
	0x73f44bc6,0x73f54bc8,0x73f64bca,0x73f74bcc,0x73f84bce,0x73f94bd0,0x73fa4bd2,0x73fb4bd4,
	0x73fc4bd6,0x73fd4bd8,0x73fe4bda,0x73ff4bdc,0x74004bde,0x74014be0,0x74024be2,0x74031023,
	0x74044be4,0x74050adc,0x74060b32,0x74074be6,0x74084be8,0x74090be7,0x740a161a,0x740b4bea,
	0x740c4bec,0x740d4bee,0x740e4bf0,0x740f0b73,0x741012aa,0x74114bf2,0x74124bf4,0x74134bf6,
	0x74144bf8,0x74154bfa,0x74164bfc,0x74174bfe,0x74184c00,0x74194c02,0x741a0981,0x741b021c,
	0x741c4c04,0x741d4c06,0x741e4c08,0x741f4c0a,0x74204c0c,0x74214c0e,0x74221a06,0x74234c10,
	0x74244c12,0x7425073c,0x74260f50,0x74274c14,0x74280a9c,0x74294c16,0x742a0f51,0x742b4c18,
	0x742c13d9,0x742d4c1a,0x742e02ec,0x742f4c1c,0x74301654,0x74314c1e,0x74324c20,0x74330bb3,
	0x74340fe4,0x74350ebc,0x74360e51,0x74374c22,0x74384c24,0x74394c26,0x743a4c28,0x743b4c2a,
	0x743c1012,0x743d4c2c,0x743e4c2e,0x743f4c30,0x74404c32,0x74410ccb,0x74424c34,0x74434c36,
	0x74444c38,0x74454c3a,0x74464c3c,0x74474c3e,0x74484c40,0x74494c42,0x744a4c44,0x744b4c46,
	0x744c4c48,0x744d4c4a,0x744e4c4c,0x744f4c4e,0x74504c50,0x74514c52,0x74524c54,0x74534c56,
	0x74544c58,0x745514d7,0x74564c5a,0x74571824,0x74584c5c,0x74590dcc,0x745a0733,0x745b1746,
	0x745c17ce,0x745d4c5e,0x745e10cc,0x745f10fa,0x74604c60,0x74614c62,0x74624c64,0x74634c66,
	0x74644c68,0x74654c6a,0x74664c6c,0x74674c6e,0x74684c70,0x74694c72,0x746a4c74,0x746b4c76,
	0x746c4c78,0x746d12ef,0x746e4c7a,0x746f4c7c,0x74700666,0x74714c7e,0x74724c80,0x74734c82,
	0x74744c84,0x74754c86,0x74761696,0x74770018,0x74784c88,0x74794c8a,0x747a4c8c,0x747b4c8e,
	0x747c4c90,0x747d4c92,0x747e0923,0x747f4c94,0x74800307,0x748102e7,0x74824c96,0x74830b23,
	0x74844c98,0x74854c9a,0x74864c9c,0x748715d9,0x74884c9e,0x74894ca0,0x748a4ca2,0x748b18cb,
	0x748c4ca4,0x748d4ca6,0x748e174c,0x748f4ca8,0x74900c3a,0x74914caa,0x74924cac,0x74934cae,
	0x74944cb0,0x74954cb2,0x74964cb4,0x74974cb6,0x74984cb8,0x74994cba,0x749a4cbc,0x749b4cbe,
	0x749c0787,0x749d4cc0,0x749e0f1f,0x749f4cc2,0x74a04cc4,0x74a14cc6,0x74a24cc8,0x74a34cca,
	0x74a44ccc,0x74a54cce,0x74a64cd0,0x74a700ff,0x74a80197,0x74a9103f,0x74aa4cd2,0x74ab4cd4,
	0x74ac4cd6,0x74ad4cd8,0x74ae4cda,0x74af4cdc,0x74b04cde,0x74b14ce0,0x74b24ce2,0x74b34ce4,
	0x74b44ce6,0x74b54ce8,0x74b64cea,0x74b74cec,0x74b84cee,0x74b94cf0,0x74ba1438,0x74bb4cf2,
	0x74bc4cf4,0x74bd4cf6,0x74be4cf8,0x74bf4cfa,0x74c04cfc,0x74c14cfe,0x74c24d00,0x74c34d02,
	0x74c44d04,0x74c54d06,0x74c64d08,0x74c74d0a,0x74c84d0c,0x74c94d0e,0x74ca4d10,0x74cb4d12,
	0x74cc4d14,0x74cd4d16,0x74ce4d18,0x74cf4d1a,0x74d04d1c,0x74d14d1e,0x74d21863,0x74d34d20,
	0x74d44d22,0x74d54d24,0x74d64d26,0x74d74d28,0x74d84d2a,0x74d94d2c,0x74da4d2e,0x74db4d30,
	0x74dc0637,0x74dd4d32,0x74de03e0,0x74df4d34,0x74e00748,0x74e14d36,0x74e20ee0,0x74e3007c,
	0x74e41078,0x74e54d38,0x74e613bd,0x74e74d3a,0x74e84d3c,0x74e94d3e,0x74ea4d40,0x74eb4d42,
	0x74ec4d44,0x74ed4d46,0x74ee143c,0x74ef0e43,0x74f04d48,0x74f14d4a,0x74f24d4c,0x74f34d4e,
	0x74f40bcd,0x74f54d50,0x74f60eff,0x74f702d6,0x74f84d52,0x74f94d54,0x74fa4d56,0x74fb4d58,
	0x74fc4d5a,0x74fd4d5c,0x74fe4d5e,0x74ff017d,0x75004d60,0x75014d62,0x75024d64,0x75034d66,
	0x75041906,0x75054d68,0x75064d6a,0x75074d6c,0x75084d6e,0x75094d70,0x750a4d72,0x750b4d74,
	0x750c4d76,0x750d0cf6,0x750e4d78,0x750f00cc,0x75104d7a,0x7511188e,0x75124d7c,0x75130ecf,
	0x75144d7e,0x75154d80,0x75164d82,0x75174d84,0x7518057c,0x75190339,0x751a1178,0x751b4d86,
	0x751c1335,0x751d4d88,0x751e4d8a,0x751f117f,0x75204d8c,0x75214d8e,0x75224d90,0x75234d92,
	0x75244d94,0x75251183,0x75264d96,0x75274d98,0x75281783,0x7529120f,0x752a4d9a,0x752b0545,
	0x752c177a,0x752d00c9,0x752e4d9c,0x752f0e15,0x75301332,0x7531178c,0x75320855,0x75331163,
	0x75344d9e,0x753503bf,0x75364da0,0x75370db5,0x753803c1,0x75394da2,0x753a03f4,0x753b0754,
	0x753c4da4,0x753d4da6,0x753e1852,0x753f4da8,0x754000e0,0x75414daa,0x75424dac,0x75434dae,
	0x75444db0,0x75450201,0x75464db2,0x75474db4,0x754804ba,0x75494db6,0x754a4db8,0x754b1334,
	0x754c0908,0x754d4dba,0x754e105e,0x754f141f,0x75504dbc,0x75514dbe,0x75524dc0,0x75534dc2,
	0x75540e6b,0x75554dc4,0x75564dc6,0x75574dc8,0x75584dca,0x75590be6,0x755a00c3,0x755b1910,
	0x755c02a3,0x755d4dcc,0x755e4dce,0x755f4dd0,0x75604dd2,0x75614dd4,0x75624dd6,0x75634dd8,
	0x75644dda,0x75650c4b,0x75660f4c,0x75674ddc,0x75684dde,0x75694de0,0x756a04a5,0x756b4de2,
	0x756c4de4,0x756d4de6,0x756e4de8,0x756f4dea,0x75704dec,0x75714dee,0x75721152,0x75734df0,
	0x7574027d,0x75754df2,0x75764df4,0x75774df6,0x757807ee,0x757913db,0x757a4df8,0x757b4dfa,
	0x757c4dfc,0x757d4dfe,0x757e4e00,0x757f07f1,0x75804e02,0x75814e04,0x75824e06,0x7583138f,
	0x75844e08,0x75854e0a,0x758608ad,0x75874e0c,0x75884e0e,0x75894e10,0x758a4e12,0x758b0ec8,
	0x758c4e14,0x758d4e16,0x758e4e18,0x758f11e9,0x75904e1a,0x759116d9,0x75920dcf,0x75934e1c,
	0x759403ef,0x75954e1e,0x759608e6,0x75970b90,0x75984e20,0x759905b5,0x759a0970,0x759b4e22,
	0x759c4e24,0x759d1125,0x759e4e26,0x759f0e2f,0x75a00b49,0x75a11678,0x75a24e28,0x75a31790,
	0x75a4004c,0x75a50909,0x75a64e2a,0x75a74e2c,0x75a84e2e,0x75a94e30,0x75aa4e32,0x75ab1703,
	0x75ac0b4e,0x75ad4e34,0x75ae02b1,0x75af0503,0x75b019ce,0x75b10e82,0x75b20eb7,0x75b30584,
	0x75b40a1d,0x75b502d1,0x75b64e36,0x75b74e38,0x75b80353,0x75b91911,0x75ba4e3a,0x75bb4e3c,
	0x75bc1314,0x75bd097e,0x75be0805,0x75bf4e3e,0x75c04e40,0x75c14e42,0x75c20845,0x75c315d5,
	0x75c418a4,0x75c5014b,0x75c64e44,0x75c7192f,0x75c8176d,0x75c9094f,0x75ca1055,0x75cb4e46,
	0x75cc4e48,0x75cd16d5,0x75ce4e4a,0x75cf4e4c,0x75d04e4e,0x75d14e50,0x75d21680,0x75d34e52,
	0x75d4196f,0x75d506f3,0x75d61621,0x75d74e54,0x75d8041d,0x75d94e56,0x75da4e58,0x75db1375,
	0x75dc4e5a,0x75dd4e5c,0x75de0ec5,0x75df4e5e,0x75e04e60,0x75e14e62,0x75e20b5d,0x75e31975,
	0x75e4031b,0x75e54e64,0x75e6147d,0x75e71105,0x75e80aea,0x75e94e66,0x75ea0774,0x75eb14f4,
	0x75ec4e68,0x75ed4e6a,0x75ee4e6c,0x75ef4e6e,0x75f012d8,0x75f104e7,0x75f24e70,0x75f34e72,
	0x75f4024e,0x75f54e74,0x75f64e76,0x75f74e78,0x75f84e7a,0x75f900ef,0x75fa4e7c,0x75fb4e7e,
	0x75fc0634,0x75fd4e80,0x75fe4e82,0x75ff1417,0x760017b0,0x7601030e,0x76024e84,0x760319b9,
	0x76044e86,0x76050356,0x76064e88,0x76074e8a,0x76084e8c,0x76094e8e,0x760a0713,0x760b4e90,
	0x760c0ab1,0x760d4e92,0x760e4e94,0x760f4e96,0x761017e0,0x76114e98,0x76124e9a,0x76134e9c,
	0x76144e9e,0x76150859,0x76164ea0,0x76171713,0x76180c14,0x761910f5,0x761a4ea2,0x761b026a,
	0x761c4ea4,0x761d4ea6,0x761e4ea8,0x761f142b,0x7620080d,0x76214eaa,0x7622006e,0x76234eac,
	0x76240bec,0x762501cf,0x762611dc,0x76274eae,0x76284eb0,0x7629032d,0x762a012d,0x762b12d2,
	0x762c4eb2,0x762d0122,0x762e4eb4,0x762f4eb6,0x76300c62,0x76314eb8,0x76324eba,0x76330277,
	0x763418d9,0x763518ae,0x76364ebc,0x76374ebe,0x76381064,0x76394ec0,0x763a4ec2,0x763b4ec4,
	0x763c0d7d,0x763d4ec6,0x763e173e,0x763f1765,0x7640078a,0x76414ec8,0x76424eca,0x76430c01,
	0x76444ecc,0x76454ece,0x76464ed0,0x76474ed2,0x76484ed4,0x76494ed6,0x764a4ed8,0x764b4eda,
	0x764c000b,0x764d006f,0x764e4edc,0x764f4ede,0x76504ee0,0x76514ee2,0x76524ee4,0x76534ee6,
	0x7654171e,0x76554ee8,0x76560ec7,0x76574eea,0x76584eec,0x76594eee,0x765a4ef0,0x765b4ef2,
	0x765c03cd,0x765d4ef4,0x765e0abf,0x765f4ef6,0x76604ef8,0x76614efa,0x76624efc,0x766315db,
	0x76644efe,0x76654f00,0x76664f02,0x76674f04,0x76684f06,0x76694f08,0x766a4f0a,0x766b03ba,
	0x766c4f0c,0x766d4f0e,0x766e4f10,0x766f1042,0x76704f12,0x76714f14,0x76724f16,0x76734f18,
	0x76744f1a,0x76754f1c,0x76764f1e,0x76774f20,0x7678066d,0x76794f22,0x767a4f24,0x767b0384,
	0x767c4f26,0x767d0060,0x767e0061,0x767f4f28,0x76804f2a,0x76814f2c,0x76821872,0x76834f2e,
	0x76840382,0x76854f30,0x768608e7,0x7687077d,0x76880663,0x76894f32,0x768a4f34,0x768b059f,
	0x768c4f36,0x768d4f38,0x768e08d3,0x768f4f3a,0x76904f3c,0x7691000a,0x76924f3e,0x769306d3,
	0x76944f40,0x76954f42,0x769613da,0x76974f44,0x76984f46,0x769914a2,0x769a4f48,0x769b4f4a,
	0x769c4f4c,0x769d4f4e,0x769e4f50,0x769f4f52,0x76a04f54,0x76a14f56,0x76a24f58,0x76a34f5a,
	0x76a40f08,0x76a54f5c,0x76a64f5e,0x76a74f60,0x76a84f62,0x76a94f64,0x76aa4f66,0x76ab4f68,
	0x76ac4f6a,0x76ad4f6c,0x76ae0eb3,0x76af4f6e,0x76b04f70,0x76b119a0,0x76b209d7,0x76b34f72,
	0x76b40312,0x76b54f74,0x76b64f76,0x76b74f78,0x76b84f7a,0x76b94f7c,0x76ba4f7e,0x76bb4f80,
	0x76bc4f82,0x76bd4f84,0x76be4f86,0x76bf0d4e,0x76c04f88,0x76c14f8a,0x76c217b8,0x76c34f8c,
	0x76c44f8e,0x76c51981,0x76c60e93,0x76c74f90,0x76c81752,0x76c94f92,0x76ca1708,0x76cb4f94,
	0x76cc4f96,0x76cd06e3,0x76ce002d,0x76cf18b9,0x76d01644,0x76d10869,0x76d206e6,0x76d34f98,
	0x76d40a81,0x76d54f9a,0x76d60577,0x76d7037b,0x76d80e61,0x76d94f9c,0x76da4f9e,0x76db118b,
	0x76dc4fa0,0x76dd4fa2,0x76de4fa4,0x76df0cf5,0x76e04fa6,0x76e14fa8,0x76e24faa,0x76e34fac,
	0x76e44fae,0x76e50652,0x76e64fb0,0x76e74fb2,0x76e84fb4,0x76e94fb6,0x76ea4fb8,0x76eb4fba,
	0x76ec4fbc,0x76ed4fbe,0x76ee0d92,0x76ef03f0,0x76f04fc0,0x76f115ad,0x76f20cae,0x76f34fc2,
	0x76f41943,0x76f54fc4,0x76f64fc6,0x76f74fc8,0x76f8150f,0x76f9044c,0x76fa4fca,0x76fb4fcc,
	0x76fc0e6a,0x76fd4fce,0x76fe0451,0x76ff4fd0,0x77004fd2,0x77011186,0x77024fd4,0x77034fd6,
	0x77040d2e,0x77054fd8,0x77064fda,0x77070d36,0x7708034b,0x77090cd4,0x770a4fdc,0x770b0a01,
	0x770c4fde,0x770d0a49,0x770e4fe0,0x770f4fe2,0x77104fe4,0x77114fe6,0x77124fe8,0x77134fea,
	0x77144fec,0x77154fee,0x77164ff0,0x77174ff2,0x77184ff4,0x771916d3,0x771a1187,0x771b4ff6,
	0x771c4ff8,0x771d4ffa,0x771e4ffc,0x771f1902,0x77200d23,0x77214ffe,0x77221808,0x77235000,
	0x77245002,0x77255004,0x77261a2f,0x77275006,0x7728189b,0x772915df,0x772a5008,0x772b500a,
	0x772c500c,0x772d067a,0x772e500e,0x772f0d06,0x77305010,0x77315012,0x77325014,0x77335016,
	0x77345018,0x7735024a,0x77360a7d,0x773709b7,0x77380d86,0x7739501a,0x773a1348,0x773b501c,
	0x773c1652,0x773d501e,0x773e5020,0x773f5022,0x774018fb,0x77411922,0x77425024,0x774312a3,
	0x77445026,0x77455028,0x7746502a,0x774703b1,0x7748502c,0x7749502e,0x774a5030,0x774b5032,
	0x774c5034,0x774d5036,0x774e5038,0x774f503a,0x77500abc,0x77510882,0x7752503c,0x7753503e,
	0x77545040,0x77555042,0x77565044,0x77575046,0x77585048,0x7759504a,0x775a161e,0x775b093c,
	0x775c504c,0x775d504e,0x775e5050,0x775f5052,0x77605054,0x7761121e,0x77621286,0x77630420,
	0x77645056,0x77650101,0x77660d9b,0x77675058,0x77680de9,0x7769505a,0x776a505c,0x776b08fb,
	0x776c0189,0x776d505e,0x776e5060,0x776f5062,0x77705064,0x77715066,0x77725068,0x7773506a,
	0x7774506c,0x7775506e,0x77765070,0x77775072,0x77785074,0x7779042d,0x777a5076,0x777b5078,
	0x777c507a,0x777d0a8b,0x777e05a3,0x777f10cd,0x77800ccc,0x7781507c,0x7782507e,0x77835080,
	0x77840d33,0x77850285,0x77865082,0x77875084,0x77885086,0x77895088,0x778a508a,0x778b508c,
	0x778c0a24,0x778d1267,0x778e14cd,0x778f508e,0x77905090,0x77910d5f,0x77920c9c,0x77935092,
	0x77945094,0x77955096,0x77965098,0x7797509a,0x7798509c,0x7799509e,0x779a50a0,0x779b50a2,
	0x779c50a4,0x779d50a6,0x779e50a8,0x779f0ee4,0x77a00231,0x77a150aa,0x77a20cf7,0x77a350ac,
	0x77a450ae,0x77a50ee8,0x77a650b0,0x77a70fc6,0x77a850b2,0x77a919c2,0x77aa038d,0x77ab50b4,
	0x77ac1222,0x77ad50b6,0x77ae50b8,0x77af50ba,0x77b00a03,0x77b150bc,0x77b250be,0x77b3136f,
	0x77b450c0,0x77b50bb9,0x77b650c2,0x77b750c4,0x77b850c6,0x77b950c8,0x77ba50ca,0x77bb18b5,
	0x77bc50cc,0x77bd062c,0x77be50ce,0x77bf1038,0x77c050d0,0x77c150d2,0x77c250d4,0x77c350d6,
	0x77c450d8,0x77c550da,0x77c650dc,0x77c750de,0x77c850e0,0x77c950e2,0x77ca50e4,0x77cb50e6,
	0x77cc50e8,0x77cd09d0,0x77ce50ea,0x77cf50ec,0x77d050ee,0x77d150f0,0x77d250f2,0x77d350f4,
	0x77d450f6,0x77d550f8,0x77d650fa,0x77d702a2,0x77d850fc,0x77d950fe,0x77da5100,0x77db0cb7,
	0x77dc0915,0x77dd5102,0x77de5104,0x77df5106,0x77e05108,0x77e1510a,0x77e211a6,0x77e316e3,
	0x77e4510c,0x77e51937,0x77e6510e,0x77e71172,0x77e85110,0x77e9098d,0x77ea5112,0x77eb08d4,
	0x77ec031c,0x77ed0437,0x77ee000d,0x77ef5114,0x77f05116,0x77f15118,0x77f2511a,0x77f3119b,
	0x77f4511c,0x77f5511e,0x77f607e1,0x77f75120,0x77f80587,0x77f95122,0x77fa5124,0x77fb5126,
	0x77fc5128,0x77fd148c,0x77fe04aa,0x77ff0a7a,0x78000369,0x78010c89,0x78021102,0x7803512a,
	0x7804512c,0x7805512e,0x78065130,0x78075132,0x78085134,0x7809074c,0x780a5136,0x780b5138,
	0x780c0f6d,0x780d09fe,0x780e513a,0x780f513c,0x7810513e,0x78111628,0x78120ead,0x78135140,
	0x78141643,0x78155142,0x781619d9,0x78170212,0x78180452,0x78195144,0x781a165b,0x781b5146,
	0x781c0504,0x781d04a2,0x781e5148,0x781f189c,0x7820514a,0x7821514c,0x7822514e,0x782313aa,
	0x78245150,0x782503a7,0x782618ac,0x78271903,0x78285152,0x78290532,0x782a5154,0x782b5156,
	0x782c0b38,0x782d0103,0x782e5158,0x782f515a,0x78300e97,0x7831515c,0x7832515e,0x78335160,
	0x78340f0e,0x78355162,0x78365164,0x7837116a,0x78381850,0x78390012,0x783a0b54,0x783b0bfd,
	0x783c1369,0x783d5166,0x783e0b55,0x783f5168,0x78400295,0x7841516a,0x7842516c,0x7843516e,
	0x78445170,0x78450665,0x78465172,0x78470dc5,0x78485174,0x78495176,0x784a5178,0x784b517a,
	0x784c05d0,0x784d517c,0x784e1589,0x784f517e,0x78500411,0x78515180,0x78521499,0x78535182,
	0x78545184,0x78551228,0x785614d4,0x78570fb6,0x78585186,0x78595188,0x785a518a,0x785b518c,
	0x785c518e,0x785d1534,0x785e5190,0x785f5192,0x78605194,0x78615196,0x78625198,0x7863519a,
	0x7864519c,0x7865519e,0x786651a0,0x786751a2,0x786851a4,0x786951a6,0x786a144b,0x786b0be8,
	0x786c1767,0x786d0cb0,0x786e1068,0x786f51a8,0x787051aa,0x787151ac,0x787251ae,0x787351b0,
	0x787451b2,0x787551b4,0x787651b6,0x78770883,0x787851b8,0x787951ba,0x787a51bc,0x787b51be,
	0x787c0e9e,0x787d51c0,0x787e51c2,0x787f51c4,0x788051c6,0x788151c8,0x788251ca,0x788351cc,
	0x788451ce,0x788551d0,0x788651d2,0x788703fb,0x788851d4,0x788903d3,0x788a51d6,0x788b51d8,
	0x788c0c34,0x788d0016,0x788e1290,0x788f51da,0x789051dc,0x789100a7,0x789251de,0x78930444,
	0x789451e0,0x789551e2,0x789651e4,0x789713dc,0x789803bd,0x789951e6,0x789a00b7,0x789b0f6f,
	0x789c0227,0x789d51e8,0x789e51ea,0x789f03e7,0x78a051ec,0x78a119a4,0x78a251ee,0x78a308fd,
	0x78a451f0,0x78a5010e,0x78a651f2,0x78a700f4,0x78a851f4,0x78a951f6,0x78aa51f8,0x78ab51fa,
	0x78ac51fc,0x78ad51fe,0x78ae5200,0x78af5202,0x78b00ea6,0x78b10889,0x78b203b4,0x78b312e5,
	0x78b401be,0x78b55204,0x78b65206,0x78b75208,0x78b8520a,0x78b915e3,0x78ba520c,0x78bb520e,
	0x78bc5210,0x78bd5212,0x78be0df4,0x78bf5214,0x78c05216,0x78c102d9,0x78c25218,0x78c3521a,
	0x78c4521c,0x78c50089,0x78c6521e,0x78c75220,0x78c85222,0x78c910eb,0x78ca0b09,0x78cb0317,
	0x78cc5224,0x78cd5226,0x78ce5228,0x78cf522a,0x78d00e62,0x78d1522c,0x78d2522e,0x78d35230,
	0x78d418f0,0x78d50a25,0x78d65232,0x78d75234,0x78d85236,0x78d90680,0x78da5238,0x78db523a,
	0x78dc523c,0x78dd523e,0x78de5240,0x78df5242,0x78e05244,0x78e15246,0x78e25248,0x78e3524a,
	0x78e4524c,0x78e5524e,0x78e65250,0x78e75252,0x78e80d6d,0x78e95254,0x78ea5256,0x78eb5258,
	0x78ec1009,0x78ed525a,0x78ee525c,0x78ef525e,0x78f05260,0x78f15262,0x78f2103e,0x78f35264,
	0x78f4038e,0x78f55266,0x78f65268,0x78f70bba,0x78f8526a,0x78f9526c,0x78fa078b,0x78fb526e,
	0x78fc5270,0x78fd5272,0x78fe5274,0x78ff5276,0x79005278,0x790108c9,0x7902527a,0x7903527c,
	0x7904527e,0x7905044a,0x79065280,0x79075282,0x79085284,0x79095286,0x790a5288,0x790b528a,
	0x790c528c,0x790d528e,0x790e5290,0x790f5292,0x79105294,0x79115296,0x79125298,0x791308ac,
	0x7914529a,0x7915529c,0x7916529e,0x791752a0,0x791852a2,0x791952a4,0x791a52a6,0x791b52a8,
	0x791c52aa,0x791d52ac,0x791e0cfa,0x791f52ae,0x792052b0,0x792152b2,0x792252b4,0x792352b6,
	0x79240181,0x792552b8,0x792652ba,0x792752bc,0x792852be,0x792952c0,0x792a52c2,0x792b52c4,
	0x792c52c6,0x792d52c8,0x792e52ca,0x792f52cc,0x793052ce,0x793152d0,0x793252d2,0x793352d4,
	0x79340167,0x793552d6,0x793652d8,0x793752da,0x793852dc,0x793952de,0x793a11b3,0x793b11af,
	0x793c0b2c,0x793d52e0,0x793e115a,0x793f52e2,0x79401245,0x79410f3e,0x794252e4,0x794352e6,
	0x794452e8,0x794552ea,0x794614e3,0x794752ec,0x79480f46,0x79491953,0x794a52ee,0x794b52f0,
	0x794c52f2,0x794d52f4,0x794e52f6,0x794f52f8,0x795052fa,0x795152fc,0x795252fe,0x7953052d,
	0x79545300,0x79555302,0x79561a4d,0x7957193b,0x79585304,0x79595306,0x795a1a66,0x795b1031,
	0x795c0745,0x795d19cd,0x795e116e,0x795f128d,0x796002d3,0x79615308,0x79620d08,0x7963530a,
	0x7964530c,0x7965151b,0x7966530e,0x7967133f,0x79680ee5,0x79695310,0x796a5312,0x796b5314,
	0x796c5316,0x796d0830,0x796e5318,0x796f1904,0x7970531a,0x7971531c,0x7972531e,0x79735320,
	0x79745322,0x79755324,0x79765326,0x79770376,0x797807d0,0x79795328,0x797a0f52,0x797b532a,
	0x797c532c,0x797d532e,0x797e5330,0x797f5332,0x79800149,0x7981091a,0x79825334,0x79835336,
	0x79840c33,0x798501d8,0x79865338,0x7987533a,0x7988533c,0x7989533e,0x798a14ca,0x798b5340,
	0x798c5342,0x798d5344,0x798e5346,0x798f053d,0x79905348,0x7991534a,0x7992534c,0x7993534e,
	0x79945350,0x79955352,0x79965354,0x79975356,0x79985358,0x7999535a,0x799a1a07,0x799b535c,
	0x799c535e,0x799d5360,0x799e5362,0x799f5364,0x79a05366,0x79a15368,0x79a2536a,0x79a3536c,
	0x79a4536e,0x79a55370,0x79a65372,0x79a714c2,0x79a85374,0x79a95376,0x79aa5378,0x79ab537a,
	0x79ac537c,0x79ad537e,0x79ae5380,0x79af5382,0x79b05384,0x79b15386,0x79b25388,0x79b31077,
	0x79b4538a,0x79b5538c,0x79b6538e,0x79b75390,0x79b85392,0x79b917db,0x79ba17bc,0x79bb0b1a,
	0x79bc5394,0x79bd0fe5,0x79be06da,0x79bf5396,0x79c015a5,0x79c11230,0x79c25398,0x79c3137d,
	0x79c4539a,0x79c5539c,0x79c60588,0x79c7539e,0x79c853a0,0x79c90145,0x79ca53a2,0x79cb1016,
	0x79cc53a4,0x79cd1988,0x79ce53a6,0x79cf53a8,0x79d053aa,0x79d10a1b,0x79d20d37,0x79d353ac,
	0x79d453ae,0x79d500d6,0x79d653b0,0x79d753b2,0x79d80d1c,0x79d953b4,0x79da53b6,0x79db53b8,
	0x79dc53ba,0x79dd53bc,0x79de53be,0x79df1a43,0x79e053c0,0x79e153c2,0x79e253c4,0x79e30d76,
	0x79e40245,0x79e553c6,0x79e60fe3,0x79e7166f,0x79e853c8,0x79e9196a,0x79ea53ca,0x79eb11ef,
	0x79ec53cc,0x79ed1a26,0x79ee53ce,0x79ef07e8,0x79f0022d,0x79f153d0,0x79f253d2,0x79f353d4,
	0x79f453d6,0x79f553d8,0x79f653da,0x79f753dc,0x79f808e9,0x79f953de,0x79fa53e0,0x79fb16d6,
	0x79fc53e2,0x79fd07b1,0x79fe53e4,0x79ff53e6,0x7a00149d,0x7a0153e8,0x7a020ade,0x7a03051b,
	0x7a0453ea,0x7a0553ec,0x7a060c74,0x7a0753ee,0x7a0853f0,0x7a0953f2,0x7a0a53f4,0x7a0b023c,
	0x7a0c53f6,0x7a0d1141,0x7a0e121d,0x7a0f53f8,0x7a1053fa,0x7a1153fc,0x7a1253fe,0x7a135400,
	0x7a14108c,0x7a155402,0x7a165404,0x7a170067,0x7a185406,0x7a195408,0x7a1a1978,0x7a1b540a,
	0x7a1c540c,0x7a1d540e,0x7a1e0a21,0x7a1f5410,0x7a20027f,0x7a215412,0x7a225414,0x7a23126d,
	0x7a245416,0x7a255418,0x7a26541a,0x7a27541c,0x7a28541e,0x7a295420,0x7a2a5422,0x7a2b5424,
	0x7a2c5426,0x7a2d5428,0x7a2e542a,0x7a2f542c,0x7a30542e,0x7a315430,0x7a325432,0x7a331435,
	0x7a345434,0x7a355436,0x7a365438,0x7a370836,0x7a38543a,0x7a391913,0x7a3a543c,0x7a3b037d,
	0x7a3c085f,0x7a3d07f2,0x7a3e543e,0x7a3f05ab,0x7a405440,0x7a415442,0x7a425444,0x7a435446,
	0x7a445448,0x7a45544a,0x7a460d9e,0x7a47544c,0x7a48544e,0x7a495450,0x7a4a5452,0x7a4b5454,
	0x7a4c5456,0x7a4d5458,0x7a4e545a,0x7a4f545c,0x7a50545e,0x7a5110fb,0x7a525460,0x7a535462,
	0x7a545464,0x7a555466,0x7a565468,0x7a571293,0x7a58546a,0x7a59546c,0x7a5a546e,0x7a5b5470,
	0x7a5c5472,0x7a5d5474,0x7a5e5476,0x7a5f5478,0x7a60547a,0x7a61547c,0x7a62547e,0x7a635480,
	0x7a645482,0x7a655484,0x7a665486,0x7a675488,0x7a68548a,0x7a69548c,0x7a6a548e,0x7a6b5490,
	0x7a6c5492,0x7a6d5494,0x7a6e5496,0x7a6f5498,0x7a701079,0x7a71549a,0x7a72549c,0x7a73549e,
	0x7a7415e9,0x7a7554a0,0x7a760960,0x7a77100e,0x7a78148d,0x7a79100f,0x7a7a0a40,0x7a7b54a2,
	0x7a7c54a4,0x7a7d54a6,0x7a7e54a8,0x7a7f02a7,0x7a8019f4,0x7a81137e,0x7a8254aa,0x7a830fd7,
	0x7a8418aa,0x7a8554ac,0x7a86010c,0x7a8754ae,0x7a88169b,0x7a8954b0,0x7a8a54b2,0x7a8b54b4,
	0x7a8c54b6,0x7a8d0fce,0x7a8e54b8,0x7a8f54ba,0x7a9054bc,0x7a911691,0x7a921970,0x7a9354be,
	0x7a9454c0,0x7a951347,0x7a9608e1,0x7a9702b2,0x7a98095e,0x7a9954c2,0x7a9a54c4,0x7a9b54c6,
	0x7a9c0300,0x7a9d1442,0x7a9e54c8,0x7a9f0a54,0x7aa00a22,0x7aa154ca,0x7aa254cc,0x7aa354ce,
	0x7aa454d0,0x7aa50a82,0x7aa6041e,0x7aa754d2,0x7aa8173f,0x7aa954d4,0x7aaa54d6,0x7aab54d8,
	0x7aac17d1,0x7aad09a4,0x7aae54da,0x7aaf54dc,0x7ab054de,0x7ab154e0,0x7ab254e2,0x7ab317e1,
	0x7ab454e4,0x7ab554e6,0x7ab654e8,0x7ab754ea,0x7ab854ec,0x7ab954ee,0x7aba54f0,0x7abb54f2,
	0x7abc54f4,0x7abd54f6,0x7abe54f8,0x7abf0c02,0x7ac054fa,0x7ac154fc,0x7ac254fe,0x7ac35500,
	0x7ac45502,0x7ac55504,0x7ac65506,0x7ac75508,0x7ac8550a,0x7ac9550c,0x7aca550e,0x7acb0b3d,
	0x7acc5510,0x7acd5512,0x7ace5514,0x7acf5516,0x7ad05518,0x7ad1551a,0x7ad2551c,0x7ad3551e,
	0x7ad45520,0x7ad55522,0x7ad61202,0x7ad75524,0x7ad85526,0x7ad918bf,0x7ada5528,0x7adb552a,
	0x7adc552c,0x7add552e,0x7ade0950,0x7adf0952,0x7ae018c4,0x7ae15530,0x7ae25532,0x7ae309e1,
	0x7ae45534,0x7ae5136b,0x7ae61257,0x7ae75536,0x7ae85538,0x7ae9553a,0x7aea553c,0x7aeb553e,
	0x7aec5540,0x7aed08fe,0x7aee5542,0x7aef0436,0x7af05544,0x7af15546,0x7af25548,0x7af3554a,
	0x7af4554c,0x7af5554e,0x7af65550,0x7af75552,0x7af85554,0x7af919b4,0x7afa19b5,0x7afb5556,
	0x7afc5558,0x7afd17bd,0x7afe555a,0x7aff0583,0x7b00555c,0x7b01555e,0x7b025560,0x7b03042a,
	0x7b0407e9,0x7b055562,0x7b06004e,0x7b075564,0x7b080804,0x7b095566,0x7b0a18e3,0x7b0b129a,
	0x7b0c5568,0x7b0d556a,0x7b0e556c,0x7b0f0746,0x7b10556e,0x7b111545,0x7b125570,0x7b135572,
	0x7b1400d8,0x7b15087d,0x7b165574,0x7b175576,0x7b185578,0x7b191182,0x7b1a557a,0x7b1b039d,
	0x7b1c557c,0x7b1d557e,0x7b1e024b,0x7b1f5580,0x7b200b58,0x7b215582,0x7b225584,0x7b235586,
	0x7b241342,0x7b251249,0x7b260538,0x7b275588,0x7b2800c5,0x7b29558a,0x7b2a0331,0x7b2b1a28,
	0x7b2c03ae,0x7b2d558c,0x7b2e187f,0x7b2f558e,0x7b305590,0x7b310603,0x7b325592,0x7b330846,
	0x7b345594,0x7b355596,0x7b365598,0x7b37559a,0x7b380f0b,0x7b39559c,0x7b3a086a,0x7b3b559e,
	0x7b3c0bfe,0x7b3d55a0,0x7b3e0104,0x7b3f55a2,0x7b4055a4,0x7b4155a6,0x7b4255a8,0x7b4355aa,
	0x7b4455ac,0x7b4514fc,0x7b4655ae,0x7b471011,0x7b4855b0,0x7b490388,0x7b4a55b2,0x7b4b0917,
	0x7b4c1057,0x7b4d55b4,0x7b4e55b6,0x7b4f04a0,0x7b500a71,0x7b5119d1,0x7b521373,0x7b5355b8,
	0x7b54032c,0x7b5555ba,0x7b5601ac,0x7b5755bc,0x7b580a4e,0x7b5955be,0x7b5a00ed,0x7b5b1110,
	0x7b5c55c0,0x7b5d1924,0x7b5e55c2,0x7b5f55c4,0x7b6009d9,0x7b6155c6,0x7b620e52,0x7b6355c8,
	0x7b6455ca,0x7b6555cc,0x7b6655ce,0x7b6755d0,0x7b6855d2,0x7b6955d4,0x7b6a55d6,0x7b6b55d8,
	0x7b6c55da,0x7b6d55dc,0x7b6e11ca,0x7b6f55de,0x7b7055e0,0x7b711540,0x7b721142,0x7b7355e2,
	0x7b7455e4,0x7b751646,0x7b7655e6,0x7b770a6a,0x7b7855e8,0x7b790280,0x7b7a55ea,0x7b7b059d,
	0x7b7c55ec,0x7b7d55ee,0x7b7e0f85,0x7b7f55f0,0x7b800886,0x7b8155f2,0x7b8255f4,0x7b8355f6,
	0x7b8455f8,0x7b8500f5,0x7b8655fa,0x7b8755fc,0x7b8855fe,0x7b895600,0x7b8a5602,0x7b8b5604,
	0x7b8c5606,0x7b8d061a,0x7b8e5608,0x7b8f560a,0x7b901008,0x7b91560c,0x7b92560e,0x7b935610,
	0x7b940163,0x7b9507f0,0x7b965612,0x7b971283,0x7b985614,0x7b995616,0x7b9a5618,0x7b9b561a,
	0x7b9c0a43,0x7b9d0f91,0x7b9e561c,0x7b9f561e,0x7ba05620,0x7ba1064e,0x7ba213dd,0x7ba35622,
	0x7ba45624,0x7ba55626,0x7ba61881,0x7ba70fda,0x7ba813b6,0x7ba90c5c,0x7baa0350,0x7bab1537,
	0x7bac10d3,0x7bad08a2,0x7bae5628,0x7baf562a,0x7bb0562c,0x7bb11515,0x7bb2562e,0x7bb35630,
	0x7bb41909,0x7bb55632,0x7bb65634,0x7bb75636,0x7bb819d3,0x7bb95638,0x7bba563a,0x7bbb563c,
	0x7bbc563e,0x7bbd5640,0x7bbe5642,0x7bbf5644,0x7bc05646,0x7bc10788,0x7bc25648,0x7bc3564a,
	0x7bc4564c,0x7bc5564e,0x7bc619df,0x7bc70ed3,0x7bc85650,0x7bc95652,0x7bca5654,0x7bcb5656,
	0x7bcc0714,0x7bcd5658,0x7bce565a,0x7bcf565c,0x7bd0565e,0x7bd10a98,0x7bd25660,0x7bd30c11,
	0x7bd45662,0x7bd55664,0x7bd65666,0x7bd75668,0x7bd8566a,0x7bd905a5,0x7bda04e0,0x7bdb566c,
	0x7bdc566e,0x7bdd05fd,0x7bde5670,0x7bdf5672,0x7be05674,0x7be10301,0x7be25676,0x7be35678,
	0x7be4567a,0x7be50b61,0x7be600f9,0x7be7567c,0x7be8567e,0x7be95680,0x7bea0259,0x7beb5682,
	0x7bec5684,0x7bed5686,0x7bee0acc,0x7bef5688,0x7bf0568a,0x7bf10b26,0x7bf2568c,0x7bf3568e,
	0x7bf45690,0x7bf55692,0x7bf65694,0x7bf70ea2,0x7bf85696,0x7bf95698,0x7bfa569a,0x7bfb569c,
	0x7bfc0415,0x7bfd569e,0x7bfe0d45,0x7bff56a0,0x7c0056a2,0x7c0156a4,0x7c0256a6,0x7c0356a8,
	0x7c0456aa,0x7c0556ac,0x7c0656ae,0x7c0702f9,0x7c0856b0,0x7c0956b2,0x7c0a56b4,0x7c0b0670,
	0x7c0c127f,0x7c0d56b6,0x7c0e56b8,0x7c0f0c3b,0x7c1056ba,0x7c1156bc,0x7c1256be,0x7c1356c0,
	0x7c1456c2,0x7c1556c4,0x7c16043e,0x7c1756c6,0x7c1856c8,0x7c1956ca,0x7c1a56cc,0x7c1b56ce,
	0x7c1c56d0,0x7c1d56d2,0x7c1e56d4,0x7c1f03ce,0x7c2056d6,0x7c2156d8,0x7c2256da,0x7c2356dc,
	0x7c2456de,0x7c2556e0,0x7c260386,0x7c27078c,0x7c2856e2,0x7c2956e4,0x7c2a185b,0x7c2b56e6,
	0x7c2c56e8,0x7c2d56ea,0x7c2e56ec,0x7c2f56ee,0x7c3056f0,0x7c3156f2,0x7c3256f4,0x7c3356f6,
	0x7c3456f8,0x7c3556fa,0x7c3656fc,0x7c3756fe,0x7c380169,0x7c395700,0x7c3a5702,0x7c3b5704,
	0x7c3c5706,0x7c3d5708,0x7c3e570a,0x7c3f017e,0x7c4019a3,0x7c410ac0,0x7c42570c,0x7c43570e,
	0x7c445710,0x7c455712,0x7c465714,0x7c475716,0x7c485718,0x7c49571a,0x7c4a571c,0x7c4b571e,
	0x7c4c5720,0x7c4d080f,0x7c4e5722,0x7c4f5724,0x7c505726,0x7c515728,0x7c52572a,0x7c53572c,
	0x7c54572e,0x7c555730,0x7c565732,0x7c575734,0x7c585736,0x7c595738,0x7c5a573a,0x7c5b573c,
	0x7c5c573e,0x7c5d5740,0x7c5e5742,0x7c5f5744,0x7c605746,0x7c615748,0x7c62574a,0x7c63574c,
	0x7c64574e,0x7c655750,0x7c665752,0x7c675754,0x7c685756,0x7c695758,0x7c6a575a,0x7c6b575c,
	0x7c6c575e,0x7c6d5760,0x7c6e5762,0x7c6f5764,0x7c705766,0x7c715768,0x7c72576a,0x7c730d12,
	0x7c740398,0x7c75576c,0x7c76576e,0x7c775770,0x7c785772,0x7c795774,0x7c7a5776,0x7c7b0b0e,
	0x7c7c14e4,0x7c7d1a23,0x7c7e5778,0x7c7f577a,0x7c80577c,0x7c81577e,0x7c825780,0x7c835782,
	0x7c845784,0x7c855786,0x7c865788,0x7c87578a,0x7c88578c,0x7c8904f5,0x7c8a578e,0x7c8b5790,
	0x7c8c5792,0x7c8d5794,0x7c8e5796,0x7c8f5798,0x7c90579a,0x7c91004f,0x7c920b59,0x7c93579c,
	0x7c94579e,0x7c950f0f,0x7c9657a0,0x7c9702f1,0x7c9818b2,0x7c9957a2,0x7c9a57a4,0x7c9b57a6,
	0x7c9c1349,0x7c9d0b5a,0x7c9e149e,0x7c9f1276,0x7ca057a8,0x7ca157aa,0x7ca21a19,0x7ca357ac,
	0x7ca41830,0x7ca51994,0x7ca657ae,0x7ca757b0,0x7ca857b2,0x7ca957b4,0x7caa04fb,0x7cab57b6,
	0x7cac57b8,0x7cad57ba,0x7cae0b82,0x7caf57bc,0x7cb057be,0x7cb10b83,0x7cb20196,0x7cb3093d,
	0x7cb457c0,0x7cb557c2,0x7cb657c4,0x7cb757c6,0x7cb857c8,0x7cb9030f,0x7cba57ca,0x7cbb57cc,
	0x7cbc0bb4,0x7cbd1a39,0x7cbe093f,0x7cbf57ce,0x7cc057d0,0x7cc110e4,0x7cc257d2,0x7cc357d4,
	0x7cc457d6,0x7cc510ab,0x7cc657d8,0x7cc70715,0x7cc815b9,0x7cc957da,0x7cca0728,0x7ccb57dc,
	0x7ccc185a,0x7ccd02dc,0x7cce57de,0x7ccf57e0,0x7cd057e2,0x7cd157e4,0x7cd257e6,0x7cd357e8,
	0x7cd457ea,0x7cd505a6,0x7cd612f2,0x7cd7102a,0x7cd857ec,0x7cd9019f,0x7cda57ee,0x7cdb57f0,
	0x7cdc0d0d,0x7cdd57f2,0x7cde57f4,0x7cdf186a,0x7ce00a06,0x7ce157f6,0x7ce257f8,0x7ce357fa,
	0x7ce457fc,0x7ce557fe,0x7ce65800,0x7ce75802,0x7ce808b9,0x7ce95804,0x7cea5806,0x7ceb5808,
	0x7cec580a,0x7ced580c,0x7cee580e,0x7cef0e38,0x7cf05810,0x7cf15812,0x7cf25814,0x7cf35816,
	0x7cf45818,0x7cf5581a,0x7cf6581c,0x7cf7581e,0x7cf80d17,0x7cf95820,0x7cfa5822,0x7cfb14c4,
	0x7cfc5824,0x7cfd5826,0x7cfe5828,0x7cff582a,0x7d00582c,0x7d01582e,0x7d025830,0x7d035832,
	0x7d045834,0x7d055836,0x7d065838,0x7d07583a,0x7d08583c,0x7d09583e,0x7d0a1434,0x7d0b5840,
	0x7d0c5842,0x7d0d5844,0x7d0e5846,0x7d0f5848,0x7d10584a,0x7d11584c,0x7d12584e,0x7d135850,
	0x7d145852,0x7d155854,0x7d165856,0x7d175858,0x7d18585a,0x7d19585c,0x7d1a585e,0x7d1b5860,
	0x7d1c5862,0x7d1d5864,0x7d1e5866,0x7d1f5868,0x7d201273,0x7d21586a,0x7d2212a9,0x7d23586c,
	0x7d24586e,0x7d255870,0x7d265872,0x7d27091d,0x7d285874,0x7d295876,0x7d2a5878,0x7d2b1a2a,
	0x7d2c587a,0x7d2d587c,0x7d2e587e,0x7d2f0b04,0x7d305880,0x7d315882,0x7d325884,0x7d335886,
	0x7d345888,0x7d35588a,0x7d36588c,0x7d37588e,0x7d385890,0x7d395892,0x7d3a5894,0x7d3b5896,
	0x7d3c5898,0x7d3d589a,0x7d3e589c,0x7d3f589e,0x7d4058a0,0x7d4158a2,0x7d4258a4,0x7d4358a6,
	0x7d4458a8,0x7d4558aa,0x7d4658ac,0x7d4758ae,0x7d4858b0,0x7d4958b2,0x7d4a58b4,0x7d4b58b6,
	0x7d4c58b8,0x7d4d58ba,0x7d4e58bc,0x7d4f58be,0x7d5058c0,0x7d5158c2,0x7d5258c4,0x7d5358c6,
	0x7d5458c8,0x7d5558ca,0x7d5658cc,0x7d5758ce,0x7d5858d0,0x7d5958d2,0x7d5a58d4,0x7d5b58d6,
	0x7d5c58d8,0x7d5d58da,0x7d5e58dc,0x7d5f58de,0x7d6058e0,0x7d6158e2,0x7d6258e4,0x7d6358e6,
	0x7d6458e8,0x7d6558ea,0x7d6658ec,0x7d6758ee,0x7d6858f0,0x7d6958f2,0x7d6a58f4,0x7d6b58f6,
	0x7d6c58f8,0x7d6d58fa,0x7d6e15c7,0x7d6f58fc,0x7d7058fe,0x7d715900,0x7d725902,0x7d735904,
	0x7d745906,0x7d755908,0x7d76590a,0x7d771949,0x7d78590c,0x7d79590e,0x7d7a5910,0x7d7b5912,
	0x7d7c5914,0x7d7d5916,0x7d7e5918,0x7d7f591a,0x7d80591c,0x7d81591e,0x7d825920,0x7d835922,
	0x7d845924,0x7d855926,0x7d865928,0x7d87592a,0x7d88592c,0x7d89592e,0x7d8a5930,0x7d8b5932,
	0x7d8c5934,0x7d8d5936,0x7d8e5938,0x7d8f593a,0x7d90593c,0x7d91593e,0x7d925940,0x7d935942,
	0x7d945944,0x7d955946,0x7d965948,0x7d97594a,0x7d98594c,0x7d99594e,0x7d9a5950,0x7d9b5952,
	0x7d9c5954,0x7d9d5956,0x7d9e5958,0x7d9f595a,0x7da0595c,0x7da1595e,0x7da25960,0x7da35962,
	0x7da45964,0x7da55966,0x7da60f55,0x7da75968,0x7da8596a,0x7da9596c,0x7daa596e,0x7dab5970,
	0x7dac5972,0x7dad5974,0x7dae1006,0x7daf5976,0x7db05978,0x7db1597a,0x7db2597c,0x7db3597e,
	0x7db45980,0x7db55982,0x7db65984,0x7db75986,0x7db85988,0x7db9598a,0x7dba598c,0x7dbb598e,
	0x7dbc5990,0x7dbd5992,0x7dbe5994,0x7dbf5996,0x7dc05998,0x7dc1599a,0x7dc2599c,0x7dc3599e,
	0x7dc459a0,0x7dc559a2,0x7dc659a4,0x7dc759a6,0x7dc859a8,0x7dc959aa,0x7dca59ac,0x7dcb59ae,
	0x7dcc59b0,0x7dcd59b2,0x7dce59b4,0x7dcf59b6,0x7dd059b8,0x7dd159ba,0x7dd259bc,0x7dd359be,
	0x7dd459c0,0x7dd559c2,0x7dd659c4,0x7dd759c6,0x7dd859c8,0x7dd959ca,0x7dda59cc,0x7ddb59ce,
	0x7ddc59d0,0x7ddd59d2,0x7dde59d4,0x7ddf59d6,0x7de059d8,0x7de159da,0x7de259dc,0x7de359de,
	0x7de459e0,0x7de559e2,0x7de659e4,0x7de759e6,0x7de859e8,0x7de959ea,0x7dea59ec,0x7deb59ee,
	0x7dec59f0,0x7ded59f2,0x7dee59f4,0x7def59f6,0x7df059f8,0x7df159fa,0x7df259fc,0x7df359fe,
	0x7df45a00,0x7df55a02,0x7df65a04,0x7df75a06,0x7df85a08,0x7df95a0a,0x7dfa5a0c,0x7dfb5a0e,
	0x7dfc5a10,0x7dfd5a12,0x7dfe5a14,0x7dff5a16,0x7e005a18,0x7e015a1a,0x7e025a1c,0x7e035a1e,
	0x7e045a20,0x7e055a22,0x7e065a24,0x7e075a26,0x7e085a28,0x7e095a2a,0x7e0a5a2c,0x7e0b5a2e,
	0x7e0c5a30,0x7e0d5a32,0x7e0e5a34,0x7e0f5a36,0x7e105a38,0x7e115a3a,0x7e125a3c,0x7e135a3e,
	0x7e145a40,0x7e155a42,0x7e165a44,0x7e175a46,0x7e185a48,0x7e195a4a,0x7e1a5a4c,0x7e1b5a4e,
	0x7e1c5a50,0x7e1d5a52,0x7e1e5a54,0x7e1f5a56,0x7e205a58,0x7e215a5a,0x7e225a5c,0x7e235a5e,
	0x7e245a60,0x7e255a62,0x7e265a64,0x7e275a66,0x7e285a68,0x7e295a6a,0x7e2a5a6c,0x7e2b5a6e,
	0x7e2c5a70,0x7e2d5a72,0x7e2e5a74,0x7e2f5a76,0x7e305a78,0x7e315a7a,0x7e325a7c,0x7e335a7e,
	0x7e345a80,0x7e355a82,0x7e365a84,0x7e375a86,0x7e385a88,0x7e395a8a,0x7e3a5a8c,0x7e3b0d0e,
	0x7e3c5a8e,0x7e3d5a90,0x7e3e5a92,0x7e3f5a94,0x7e405a96,0x7e4104b0,0x7e425a98,0x7e435a9a,
	0x7e445a9c,0x7e455a9e,0x7e465aa0,0x7e471697,0x7e485aa2,0x7e495aa4,0x7e4a5aa6,0x7e4b5aa8,
	0x7e4c5aaa,0x7e4d5aac,0x7e4e5aae,0x7e4f5ab0,0x7e505ab2,0x7e515ab4,0x7e525ab6,0x7e535ab8,
	0x7e545aba,0x7e555abc,0x7e565abe,0x7e575ac0,0x7e585ac2,0x7e595ac4,0x7e5a5ac6,0x7e5b5ac8,
	0x7e5c5aca,0x7e5d5acc,0x7e5e5ace,0x7e5f5ad0,0x7e605ad2,0x7e615ad4,0x7e625ad6,0x7e635ad8,
	0x7e645ada,0x7e655adc,0x7e665ade,0x7e675ae0,0x7e685ae2,0x7e695ae4,0x7e6a5ae6,0x7e6b5ae8,
	0x7e6c5aea,0x7e6d5aec,0x7e6e5aee,0x7e6f5af0,0x7e705af2,0x7e715af4,0x7e725af6,0x7e735af8,
	0x7e745afa,0x7e755afc,0x7e765afe,0x7e775b00,0x7e785b02,0x7e795b04,0x7e7a5b06,0x7e7b5b08,
	0x7e7c5b0a,0x7e7d5b0c,0x7e7e5b0e,0x7e7f5b10,0x7e805b12,0x7e815b14,0x7e821a51,0x7e835b16,
	0x7e845b18,0x7e855b1a,0x7e865b1c,0x7e875b1e,0x7e885b20,0x7e895b22,0x7e8a5b24,0x7e8b5b26,
	0x7e8c5b28,0x7e8d5b2a,0x7e8e5b2c,0x7e8f5b2e,0x7e905b30,0x7e915b32,0x7e925b34,0x7e935b36,
	0x7e945b38,0x7e955b3a,0x7e965b3c,0x7e975b3e,0x7e985b40,0x7e995b42,0x7e9a5b44,0x7e9b037e,
	0x7e9c5b46,0x7e9d5b48,0x7e9e5b4a,0x7e9f122d,0x7ea0095f,0x7ea117ad,0x7ea20705,0x7ea31999,
	0x7ea414e1,0x7ea505b4,0x7ea61827,0x7ea707fe,0x7ea813ca,0x7ea90a77,0x7eaa081e,0x7eab1091,
	0x7eac140d,0x7ead1838,0x7eae5b4c,0x7eaf02c3,0x7eb00ea9,0x7eb11100,0x7eb20595,0x7eb30da4,
	0x7eb45b4e,0x7eb51a38,0x7eb60c52,0x7eb704ec,0x7eb81951,0x7eb9142d,0x7eba04ca,0x7ebb5b50,
	0x7ebc5b52,0x7ebd0e1d,0x7ebe11e0,0x7ebf1504,0x7ec00590,0x7ec1155a,0x7ec20528,0x7ec30b77,
	0x7ec41a4b,0x7ec51167,0x7ec614c6,0x7ec71938,0x7ec81980,0x7ec9199c,0x7eca007b,0x7ecb0529,
	0x7ecc029d,0x7ecd114c,0x7ece16fe,0x7ecf0934,0x7ed0033a,0x7ed10082,0x7ed2109d,0x7ed308f6,
	0x7ed40a58,0x7ed51083,0x7ed65b54,0x7ed706bf,0x7ed807a9,0x7ed905d2,0x7eda15de,0x7edb08b6,
	0x7edc0c66,0x7edd09c0,0x7ede08d1,0x7edf1370,0x7ee005e0,0x7ee11531,0x7ee209b5,0x7ee315a7,
	0x7ee45b56,0x7ee51288,0x7ee61300,0x7ee7082a,0x7ee8131d,0x7ee9081a,0x7eea15c2,0x7eeb0bd2,
	0x7eec5b58,0x7eed15c3,0x7eee0f63,0x7eef04d1,0x7ef002ce,0x7ef1113c,0x7ef2067d,0x7ef31185,
	0x7ef41408,0x7ef50d24,0x7ef611db,0x7ef700c7,0x7ef8027c,0x7ef95b5a,0x7efa0bf0,0x7efb105f,
	0x7efc1a31,0x7efd18c0,0x7efe13d6,0x7eff0c80,0x7f0019ef,0x7f011a14,0x7f020a35,0x7f031513,
	0x7f04086e,0x7f050d2c,0x7f060ad1,0x7f071320,0x7f080d3a,0x7f0907ec,0x7f0a5b5c,0x7f0b07b4,
	0x7f0c1235,0x7f0d0461,0x7f0e043a,0x7f0f0117,0x7f105b5e,0x7f1105fc,0x7f1219f1,0x7f130769,
	0x7f1403b2,0x7f150c77,0x7f160105,0x7f170d4c,0x7f181815,0x7f19092e,0x7f1a0561,0x7f1b10c0,
	0x7f1c1912,0x7f1d050d,0x7f1e5b60,0x7f1f05a9,0x7f2001da,0x7f210b20,0x7f221710,0x7f230871,
	0x7f240135,0x7f250ee2,0x7f260ca7,0x7f270b00,0x7f281749,0x7f2912a6,0x7f2a0d3f,0x7f2b10ef,
	0x7f2c1555,0x7f2d0b97,0x7f2e112a,0x7f2f188b,0x7f3008ab,0x7f310f98,0x7f320fbc,0x7f330766,
	0x7f3408dc,0x7f351a50,0x7f360514,0x7f375b62,0x7f380597,0x7f395b64,0x7f3a1063,0x7f3b5b66,
	0x7f3c5b68,0x7f3d5b6a,0x7f3e5b6c,0x7f3f5b6e,0x7f405b70,0x7f415b72,0x7f42174a,0x7f435b74,
	0x7f44100a,0x7f4514de,0x7f465b76,0x7f475b78,0x7f485b7a,0x7f495b7c,0x7f4a5b7e,0x7f4b5b80,
	0x7f4c5b82,0x7f4d5b84,0x7f4e5b86,0x7f4f5b88,0x7f500655,0x7f5113e3,0x7f525b8a,0x7f535b8c,
	0x7f5413e6,0x7f5506ae,0x7f565b8e,0x7f570c55,0x7f58052e,0x7f595b90,0x7f5a049e,0x7f5b5b92,
	0x7f5c5b94,0x7f5d5b96,0x7f5e5b98,0x7f5f0625,0x7f605b9a,0x7f610599,0x7f62005a,0x7f635b9c,
	0x7f645b9e,0x7f655ba0,0x7f665ba2,0x7f675ba4,0x7f681655,0x7f6918e6,0x7f6a1a55,0x7f6b5ba6,
	0x7f6c5ba8,0x7f6d5baa,0x7f6e1979,0x7f6f5bac,0x7f705bae,0x7f710ad4,0x7f7211f6,0x7f735bb0,
	0x7f740ebe,0x7f755bb2,0x7f765bb4,0x7f775bb6,0x7f785bb8,0x7f790b27,0x7f7a5bba,0x7f7b5bbc,
	0x7f7c5bbe,0x7f7d5bc0,0x7f7e188c,0x7f7f5bc2,0x7f805bc4,0x7f8107f6,0x7f825bc6,0x7f835bc8,
	0x7f845bca,0x7f855bcc,0x7f865bce,0x7f875bd0,0x7f885bd2,0x7f895bd4,0x7f8a1673,0x7f8b5bd6,
	0x7f8c0fa3,0x7f8d5bd8,0x7f8e0ce2,0x7f8f5bda,0x7f905bdc,0x7f915bde,0x7f925be0,0x7f935be2,
	0x7f9405a0,0x7f955be4,0x7f965be6,0x7f975be8,0x7f985bea,0x7f995bec,0x7f9a0bd3,0x7f9b5bee,
	0x7f9c5bf0,0x7f9d0391,0x7f9e159f,0x7f9f0fb3,0x7fa05bf2,0x7fa11509,0x7fa25bf4,0x7fa35bf6,
	0x7fa4106f,0x7fa55bf8,0x7fa65bfa,0x7fa712a4,0x7fa85bfc,0x7fa95bfe,0x7faa5c00,0x7fab5c02,
	0x7fac5c04,0x7fad5c06,0x7fae5c08,0x7faf0900,0x7fb012e7,0x7fb15c0a,0x7fb214ad,0x7fb35c0c,
	0x7fb45c0e,0x7fb55c10,0x7fb65c12,0x7fb75c14,0x7fb80b03,0x7fb905dd,0x7fba5c16,0x7fbb5c18,
	0x7fbc01ea,0x7fbd17d8,0x7fbe5c1a,0x7fbf1704,0x7fc05c1c,0x7fc11439,0x7fc25c1e,0x7fc35c20,
	0x7fc45c22,0x7fc50266,0x7fc65c24,0x7fc75c26,0x7fc85c28,0x7fc95c2a,0x7fca170b,0x7fcb5c2c,
	0x7fcc170c,0x7fcd5c2e,0x7fce0bd4,0x7fcf5c30,0x7fd05c32,0x7fd15c34,0x7fd25c36,0x7fd35c38,
	0x7fd4151c,0x7fd5149f,0x7fd65c3a,0x7fd75c3c,0x7fd80fc7,0x7fd95c3e,0x7fda5c40,0x7fdb5c42,
	0x7fdc5c44,0x7fdd5c46,0x7fde5c48,0x7fdf03a0,0x7fe00310,0x7fe104df,0x7fe25c4a,0x7fe35c4c,
	0x7fe45c4e,0x7fe519d4,0x7fe6088a,0x7fe75c50,0x7fe85c52,0x7fe90ed4,0x7fea5c54,0x7feb5c56,
	0x7fec5c58,0x7fed5c5a,0x7fee06eb,0x7fef5c5c,0x7ff006bb,0x7ff10035,0x7ff25c5e,0x7ff3171b,
	0x7ff45c60,0x7ff55c62,0x7ff65c64,0x7ff75c66,0x7ff85c68,0x7ff95c6a,0x7ffa5c6c,0x7ffb04a7,
	0x7ffc171c,0x7ffd5c6e,0x7ffe5c70,0x7fff5c72,0x800016a2,0x80010aed,0x80025c74,0x80030a0f,
	0x80040cc8,0x800518f2,0x80060f47,0x80075c76,0x80085c78,0x80095c7a,0x800a5c7c,0x800b03e4,
	0x800c048d,0x800d120c,0x800e5c7e,0x800f5c80,0x80100db1,0x80115c82,0x80120b06,0x80135c84,
	0x80141a27,0x801505db,0x80160210,0x801706d2,0x8018183c,0x80190e50,0x801a5c86,0x801b5c88,
	0x801c124a,0x801d5c8a,0x801e5c8c,0x801f5c8e,0x802007c2,0x80215c90,0x80220af5,0x80235c92,
	0x80245c94,0x802512fa,0x80260e47,0x80270c0c,0x80280e26,0x802908b2,0x802a0e75,0x802b5c96,
	0x802c5c98,0x802d5c9a,0x802e5c9c,0x802f5c9e,0x80305ca0,0x80310d81,0x80325ca2,0x80330491,
	0x80345ca4,0x803503f2,0x803616a7,0x80370325,0x80381256,0x80395ca6,0x803a5ca8,0x803b025d,
	0x803c5caa,0x803d034c,0x803e5cac,0x803f05e1,0x80405cae,0x80415cb0,0x80420e03,0x8043034e,
	0x80445cb2,0x80455cb4,0x80460bd5,0x80475cb6,0x80485cb8,0x80495cba,0x804a0b91,0x804b0bff,
	0x804c1946,0x804d0e13,0x804e5cbc,0x804f5cbe,0x80505cc0,0x80515cc2,0x80520686,0x80535cc4,
	0x80540b6a,0x80555cc6,0x80565cc8,0x80575cca,0x80580ef4,0x80595ccc,0x805a09a5,0x805b5cce,
	0x805c5cd0,0x805d5cd2,0x805e5cd4,0x805f5cd6,0x80605cd8,0x80615cda,0x80625cdc,0x80635cde,
	0x80645ce0,0x80655ce2,0x80665ce4,0x80675ce6,0x80685ce8,0x80690a99,0x806a02e8,0x806b5cea,
	0x806c5cec,0x806d5cee,0x806e5cf0,0x806f5cf2,0x80705cf4,0x80710036,0x80725cf6,0x80735cf8,
	0x80745cfa,0x80755cfc,0x80765cfe,0x80775d00,0x80785d02,0x80795d04,0x807a5d06,0x807b5d08,
	0x807c5d0a,0x807d5d0c,0x807e5d0e,0x807f17e6,0x808017e3,0x80815d10,0x80825d12,0x80831271,
	0x80841711,0x80855d14,0x8086124c,0x808718e7,0x80885d16,0x808910ae,0x808a5d18,0x808b0b0c,
	0x808c07df,0x808d5d1a,0x808e5d1c,0x808f5d1e,0x80905d20,0x80915d22,0x80925d24,0x80930779,
	0x80945d26,0x80955d28,0x8096153b,0x80975d2a,0x80981997,0x80995d2c,0x809a0431,0x809b0596,
	0x809c109b,0x809d057e,0x809e5d2e,0x809f1446,0x80a001f3,0x80a10622,0x80a21939,0x80a35d30,
	0x80a40518,0x80a504d8,0x80a65d32,0x80a75d34,0x80a85d36,0x80a90866,0x80aa04c6,0x80ab19f3,
	0x80ac5d38,0x80ad0da5,0x80ae002b,0x80af0a38,0x80b05d3a,0x80b105ea,0x80b217ea,0x80b35d3c,
	0x80b4168d,0x80b55d3e,0x80b65d40,0x80b70f95,0x80b85d42,0x80b95d44,0x80ba04e5,0x80bb5d46,
	0x80bc0944,0x80bd12ca,0x80be1177,0x80bf1987,0x80c018d4,0x80c1154d,0x80c21179,0x80c31420,
	0x80c4199e,0x80c55d48,0x80c60352,0x80c75d4a,0x80c85d4c,0x80c95d4e,0x80ca5d50,0x80cb5d52,
	0x80cc00af,0x80cd0639,0x80ce12be,0x80cf5d54,0x80d05d56,0x80d15d58,0x80d25d5a,0x80d35d5c,
	0x80d45d5e,0x80d55d60,0x80d60e76,0x80d7190b,0x80d85d62,0x80d91a67,0x80da0e84,0x80db0856,
	0x80dc1188,0x80dd193c,0x80de008f,0x80df5d64,0x80e05d66,0x80e1072c,0x80e25d68,0x80e35d6a,
	0x80e41742,0x80e515ae,0x80e65d6c,0x80e70bfc,0x80e8040f,0x80e909e7,0x80ea0c1f,0x80eb094e,
	0x80ec0e2c,0x80ed162e,0x80ee5d6e,0x80ef0a60,0x80f016d4,0x80f1065a,0x80f2069a,0x80f305b7,
	0x80f40410,0x80f55d70,0x80f608c1,0x80f75d72,0x80f81598,0x80f95d74,0x80fa0028,0x80fb5d76,
	0x80fc0ed6,0x80fd0dd6,0x80fe5d78,0x80ff5d7a,0x81005d7c,0x81015d7e,0x8102193d,0x81035d80,
	0x81045d82,0x81055d84,0x81060308,0x81075d86,0x81085d88,0x81090c98,0x810a0814,0x810b5d8a,
	0x810c5d8c,0x810d0a69,0x810e10d9,0x810f1867,0x81100f48,0x81110dcb,0x81120d15,0x81130e24,
	0x81140c45,0x81155d8e,0x81160155,0x81175d90,0x811813d7,0x81195d92,0x811a08d5,0x811b5d94,
	0x811c5d96,0x811d5d98,0x811e031e,0x811f5d9a,0x81205d9c,0x81215d9e,0x81225da0,0x81235da2,
	0x81245da4,0x81255da6,0x81265da8,0x81275daa,0x81285dac,0x81295dae,0x812a5db0,0x812b5db2,
	0x812c0e78,0x812d5db4,0x812e5db6,0x812f054b,0x81305db8,0x813113a2,0x81320dff,0x81335dba,
	0x81345dbc,0x81355dbe,0x81360c57,0x81375dc0,0x81380b74,0x81395dc2,0x813a5dc4,0x813b5dc6,
	0x813c5dc8,0x813d5dca,0x813e0ebd,0x813f5dcc,0x81405dce,0x81415dd0,0x81425dd2,0x81435dd4,
	0x81445dd6,0x81455dd8,0x8146133a,0x81475dda,0x8148093b,0x81495ddc,0x814a0ab0,0x814b16b8,
	0x814c1635,0x814d5dde,0x814e5de0,0x814f5de2,0x8150054f,0x8151054d,0x81525de4,0x815304da,
	0x81540fa8,0x815513df,0x81565de6,0x81575de8,0x81585dea,0x81591a33,0x815a03fa,0x815b5dec,
	0x815c5dee,0x815d5df0,0x815e5df2,0x815f5df4,0x816002ef,0x81615df6,0x81625df8,0x81635dfa,
	0x81645dfc,0x81651581,0x81665dfe,0x81671206,0x81685e00,0x81690dbb,0x816a5e02,0x816b5e04,
	0x816c5e06,0x816d0483,0x816e10dc,0x816f5e08,0x81701689,0x8171089d,0x81725e0a,0x81735e0c,
	0x817417ca,0x81755e0e,0x81765e10,0x81775e12,0x81785e14,0x81790562,0x817a150b,0x817b0dea,
	0x817c0d2d,0x817d13c0,0x817e1315,0x817f1393,0x81800084,0x81815e16,0x81820c78,0x81835e18,
	0x81845e1a,0x81855e1c,0x81865e1e,0x81875e20,0x818805c8,0x81895e22,0x818a0164,0x818b5e24,
	0x818c5e26,0x818d5e28,0x818e5e2a,0x818f05a4,0x81905e2c,0x8191013d,0x81925e2e,0x81935e30,
	0x81945e32,0x81955e34,0x81965e36,0x81975e38,0x81980121,0x81995e3a,0x819a5e3c,0x819b12f1,
	0x819c0d6b,0x819d14aa,0x819e5e3e,0x819f5e40,0x81a05e42,0x81a15e44,0x81a25e46,0x81a3197b,
	0x81a45e48,0x81a55e4a,0x81a60bc4,0x81a75e4c,0x81a80ea3,0x81a95e4e,0x81aa01d1,0x81ab5e50,
	0x81ac5e52,0x81ad5e54,0x81ae5e56,0x81af5e58,0x81b05e5a,0x81b15e5c,0x81b25e5e,0x81b3112d,
	0x81b45e60,0x81b55e62,0x81b65e64,0x81b75e66,0x81b85e68,0x81b95e6a,0x81ba174e,0x81bb111f,
	0x81bc5e6c,0x81bd5e6e,0x81be5e70,0x81bf5e72,0x81c0139d,0x81c10b6f,0x81c200fd,0x81c31775,
	0x81c45e74,0x81c55e76,0x81c6171d,0x81c75e78,0x81c85e7a,0x81c95e7c,0x81ca10f0,0x81cb5e7e,
	0x81cc062b,0x81cd5e80,0x81ce5e82,0x81cf5e84,0x81d05e86,0x81d15e88,0x81d25e8a,0x81d35e8c,
	0x81d45e8e,0x81d55e90,0x81d65e92,0x81d75e94,0x81d85e96,0x81d95e98,0x81da5e9a,0x81db5e9c,
	0x81dc5e9e,0x81dd5ea0,0x81de5ea2,0x81df5ea4,0x81e05ea6,0x81e15ea8,0x81e25eaa,0x81e3021f,
	0x81e45eac,0x81e55eae,0x81e65eb0,0x81e71865,0x81e85eb2,0x81e95eb4,0x81ea1a2d,0x81eb5eb6,
	0x81ec0e04,0x81ed0286,0x81ee5eb8,0x81ef5eba,0x81f05ebc,0x81f15ebe,0x81f25ec0,0x81f3195a,
	0x81f4196b,0x81f55ec2,0x81f65ec4,0x81f75ec6,0x81f85ec8,0x81f95eca,0x81fa5ecc,0x81fb190a,
	0x81fc096e,0x81fd5ece,0x81fe17b9,0x81ff5ed0,0x8200169c,0x820117be,0x8202026f,0x82035ed2,
	0x820414c8,0x82050976,0x820617d2,0x82075ed4,0x82085ed6,0x82095ed8,0x820a5eda,0x820b5edc,
	0x820c1154,0x820d1157,0x820e5ede,0x820f5ee0,0x821011c2,0x82115ee2,0x821211ea,0x82135ee4,
	0x8214133b,0x82155ee6,0x82165ee8,0x82175eea,0x82185eec,0x82195eee,0x821a5ef0,0x821b02ad,
	0x821c1221,0x821d5ef2,0x821e146d,0x821f1990,0x82205ef4,0x822102a9,0x8222111b,0x822316e5,
	0x82245ef6,0x82255ef8,0x82265efa,0x82275efc,0x82280075,0x82295efe,0x822a06c0,0x822b04cb,
	0x822c006a,0x822d00d9,0x822e5f00,0x822f1983,0x82300897,0x8231019c,0x82325f02,0x823319b8,
	0x82341880,0x82350465,0x8236015e,0x823714f2,0x823805cc,0x823902aa,0x823a5f04,0x823b0c21,
	0x823c5f06,0x823d5f08,0x823e14a0,0x823f5f0a,0x82405f0c,0x82415f0e,0x82425f10,0x82435f12,
	0x82441143,0x82455f14,0x82465f16,0x82471360,0x82485f18,0x82491418,0x824a5f1a,0x824b0cff,
	0x824c5f1c,0x824d5f1e,0x824e5f20,0x824f11d4,0x82505f22,0x82515f24,0x82525f26,0x82535f28,
	0x82545f2a,0x82555f2c,0x82565f2e,0x82575f30,0x82581263,0x82595f32,0x825a01a4,0x825b5f34,
	0x825c5f36,0x825d5f38,0x825e5f3a,0x825f0271,0x82605f3c,0x82615f3e,0x82625f40,0x82635f42,
	0x82645f44,0x82655f46,0x82665f48,0x82675f4a,0x82680cfb,0x82695f4c,0x826a5f4e,0x826b5f50,
	0x826c5f52,0x826d5f54,0x826e05d7,0x826f0b7e,0x82700867,0x82715f56,0x827210f6,0x8273165f,
	0x82740539,0x82755f58,0x82765f5a,0x82775f5c,0x82785f5e,0x8279003d,0x827a16ef,0x827b5f60,
	0x827c5f62,0x827d101a,0x827e0010,0x827f0dad,0x82805f64,0x82815f66,0x828208f0,0x82835f68,
	0x828413cb,0x82855f6a,0x82865f6c,0x82875f6e,0x82880d40,0x82895f70,0x828a0f7b,0x828b17e7,
	0x828c5f72,0x828d1146,0x828e1596,0x828f042e,0x82905f74,0x82910f5f,0x82920cad,0x82935f76,
	0x82945f78,0x82955f7a,0x82965f7c,0x8297150e,0x82980eb4,0x82990524,0x829a5f7e,0x829b5f80,
	0x829c145b,0x829d1934,0x829e5f82,0x829f1116,0x82a05f84,0x82a10f9b,0x82a25f86,0x82a35f88,
	0x82a40a48,0x82a50906,0x82a60c1a,0x82a75f8a,0x82a807e0,0x82a90fe1,0x82aa0f42,0x82ab163e,
	0x82ac04ed,0x82ad004b,0x82ae10c8,0x82af156f,0x82b00822,0x82b1074a,0x82b25f8c,0x82b304c0,
	0x82b41476,0x82b55f8e,0x82b65f90,0x82b71952,0x82b81839,0x82b90fe2,0x82ba5f92,0x82bb5f94,
	0x82bc5f96,0x82bd1619,0x82be04e9,0x82bf5f98,0x82c05f9a,0x82c102e3,0x82c25f9c,0x82c35f9e,
	0x82c40114,0x82c55fa0,0x82c65fa2,0x82c7140e,0x82c80b45,0x82c95fa4,0x82ca0478,0x82cb1502,
	0x82cc01f4,0x82cd019b,0x82ce19c6,0x82cf126b,0x82d05fa6,0x82d1181e,0x82d21075,0x82d30bca,
	0x82d412c2,0x82d51147,0x82d65fa8,0x82d70d31,0x82d81002,0x82d95faa,0x82da5fac,0x82db0a18,
	0x82dc0d96,0x82dd5fae,0x82de008e,0x82df0601,0x82e00d4a,0x82e116e4,0x82e25fb0,0x82e30997,
	0x82e40eea,0x82e510d0,0x82e60a56,0x82e75fb2,0x82e85fb4,0x82e95fb6,0x82ea5fb8,0x82eb1120,
	0x82ec5fba,0x82ed5fbc,0x82ee5fbe,0x82ef00c2,0x82f05fc0,0x82f11760,0x82f25fc2,0x82f35fc4,
	0x82f4097c,0x82f55fc6,0x82f65fc8,0x82f70581,0x82f85fca,0x82f90efc,0x82fa5fcc,0x82fb052a,
	0x82fc5fce,0x82fd5fd0,0x82fe5fd2,0x82ff5fd4,0x83005fd6,0x830119ff,0x83020cc5,0x830304b8,
	0x83040fd2,0x83050cb9,0x83060cbf,0x83070051,0x830802d4,0x83090d74,0x830a5fd8,0x830b5fda,
	0x830c0255,0x830d5fdc,0x830e0935,0x830f0bf9,0x83105fde,0x83110dfb,0x83125fe0,0x83135fe2,
	0x83141751,0x83151010,0x83165fe4,0x83170d5a,0x83185fe6,0x83195fe8,0x831a1741,0x831b05d8,
	0x831c0f9c,0x831d5fea,0x831e5fec,0x831f5fee,0x83205ff0,0x83215ff2,0x83225ff4,0x83235ff6,
	0x83245ff8,0x83255ffa,0x83265ffc,0x8327087b,0x832802d5,0x83295ffe,0x832a6000,0x832b0caf,
	0x832c01b8,0x832d08bf,0x832e6002,0x832f052f,0x83306004,0x833119aa,0x83326006,0x833308a7,
	0x8334079e,0x83351725,0x833601b9,0x83376008,0x8338109e,0x833910b0,0x833a026e,0x833b600a,
	0x833c1367,0x833d600c,0x833e600e,0x833f6010,0x83401601,0x83416012,0x83426014,0x83431052,
	0x83446016,0x83456018,0x83460936,0x83471591,0x8348601a,0x834901a6,0x834a601c,0x834b601e,
	0x834c6020,0x834d6022,0x834e6024,0x834f108b,0x83500893,0x835116d1,0x8352077a,0x83536026,
	0x83540b4f,0x83556028,0x8356602a,0x8357602c,0x8358602e,0x83596030,0x835a084e,0x835b107e,
	0x835c00e3,0x835d6032,0x835e0fc0,0x835f07aa,0x8360083b,0x8361036a,0x83626034,0x8363109f,
	0x836407b9,0x8365158a,0x83660c67,0x83671753,0x83680f8a,0x83690929,0x836a1297,0x836b1726,
	0x836c0c93,0x836d070a,0x836e199f,0x836f169e,0x83706036,0x83716038,0x8372603a,0x8373603c,
	0x8374603e,0x83756040,0x83766042,0x837706e4,0x837800cf,0x83796044,0x837a6046,0x837b039c,
	0x837c1382,0x837d1285,0x837e6048,0x837f604a,0x8380604c,0x8381604e,0x83826050,0x83836052,
	0x83846054,0x83850b56,0x83860f1b,0x83876056,0x83886058,0x83890b39,0x838a605a,0x838b605c,
	0x838c605e,0x838d6060,0x838e1103,0x838f6062,0x83906064,0x83916066,0x8392098e,0x83930cd5,
	0x83946068,0x8395606a,0x8396606c,0x8397606e,0x8398116d,0x83996070,0x839a6072,0x839b1357,
	0x839c1791,0x839d6074,0x839e13d3,0x839f6076,0x83a0179d,0x83a16078,0x83a2607a,0x83a3607c,
	0x83a4607e,0x83a56080,0x83a66082,0x83a76084,0x83a80ae2,0x83a90533,0x83aa046f,0x83ab0d77,
	0x83ac6086,0x83ad6088,0x83ae608a,0x83af608c,0x83b009ff,0x83b10ab8,0x83b20b69,0x83b311ac,
	0x83b41441,0x83b5608e,0x83b614e5,0x83b707cf,0x83b81792,0x83b91754,0x83ba1744,0x83bb6090,
	0x83bc02c5,0x83bd0cb2,0x83be6092,0x83bf6094,0x83c013d8,0x83c10939,0x83c26096,0x83c36098,
	0x83c4609a,0x83c5086b,0x83c6609c,0x83c70614,0x83c8609e,0x83c960a0,0x83ca0988,0x83cb60a2,
	0x83cc09d8,0x83cd60a4,0x83ce60a6,0x83cf06e7,0x83d060a8,0x83d160aa,0x83d260ac,0x83d360ae,
	0x83d4053a,0x83d560b0,0x83d601ef,0x83d760b2,0x83d81252,0x83d960b4,0x83da60b6,0x83db60b8,
	0x83dc018b,0x83dd0052,0x83de60ba,0x83df138b,0x83e00153,0x83e106b6,0x83e260bc,0x83e360be,
	0x83e460c0,0x83e5149a,0x83e660c2,0x83e760c4,0x83e860c6,0x83e90f1c,0x83ea036c,0x83eb60c8,
	0x83ec60ca,0x83ed60cc,0x83ee60ce,0x83ef60d0,0x83f00615,0x83f10bd6,0x83f204d2,0x83f360d2,
	0x83f460d4,0x83f560d6,0x83f660d8,0x83f760da,0x83f81632,0x83f91a44,0x83fa60dc,0x83fb60de,
	0x83fc60e0,0x83fd11e8,0x83fe60e2,0x83ff60e4,0x840060e6,0x84010f4d,0x840260e8,0x8403030c,
	0x8404130b,0x840560ea,0x840600ea,0x840760ec,0x840860ee,0x840960f0,0x840a60f2,0x840b0f35,
	0x840c0cf4,0x840d0f00,0x840e1415,0x840f035e,0x841060f4,0x84110763,0x841260f6,0x841360f8,
	0x841460fa,0x841560fc,0x841660fe,0x84176100,0x84180db2,0x84196102,0x841a6104,0x841b6106,
	0x841c134c,0x841d0c58,0x841e6108,0x841f610a,0x8420610c,0x8421610e,0x84226110,0x84236112,
	0x84241755,0x84251756,0x84261757,0x84271533,0x842810da,0x84296114,0x842a6116,0x842b6118,
	0x842c611a,0x842d611c,0x842e611e,0x842f6120,0x84306122,0x843115d0,0x84326124,0x84336126,
	0x84346128,0x8435612a,0x8436612c,0x8437612e,0x843817c4,0x84396130,0x843a6132,0x843b6134,
	0x843c0481,0x843d0c6a,0x843e6136,0x843f6138,0x8440613a,0x8441613c,0x8442613e,0x84436140,
	0x84446142,0x84456144,0x84460099,0x84476146,0x84486148,0x8449614a,0x844a614c,0x844b614e,
	0x844c6150,0x844d6152,0x844e6154,0x844f6156,0x84506158,0x84510507,0x8452615a,0x8453615c,
	0x8454615e,0x84556160,0x84566162,0x845719cf,0x84586164,0x84591514,0x845a1176,0x845b05c2,
	0x845c0f73,0x845d6166,0x845e6168,0x845f616a,0x8460616c,0x84610f1d,0x8462616e,0x84630405,
	0x84646170,0x84656172,0x84666174,0x84676176,0x84686178,0x84690e4d,0x846a617a,0x846b0731,
	0x846c1868,0x846d0848,0x846e617c,0x846f617e,0x84706180,0x847102e5,0x84726182,0x847313f4,
	0x84746184,0x84750a88,0x8476135a,0x84776186,0x847814bf,0x84796188,0x847a0f6e,0x847b618a,
	0x847c618c,0x847d618e,0x847e6190,0x847f6192,0x84806194,0x84816196,0x848203b3,0x84836198,
	0x8484619a,0x8485619c,0x8486619e,0x848701e5,0x848809f2,0x84890a96,0x848a61a0,0x848b08b1,
	0x848c0c0a,0x848d61a2,0x848e0e5d,0x848f61a4,0x849061a6,0x849161a8,0x849261aa,0x849361ac,
	0x849461ae,0x849561b0,0x849661b2,0x84970ae5,0x849861b4,0x84990cf2,0x849a61b6,0x849b61b8,
	0x849c1282,0x849d61ba,0x849e61bc,0x849f61be,0x84a061c0,0x84a10088,0x84a261c2,0x84a361c4,
	0x84a461c6,0x84a561c8,0x84a661ca,0x84a761cc,0x84a861ce,0x84a961d0,0x84aa61d2,0x84ab61d4,
	0x84ac61d6,0x84ad61d8,0x84ae61da,0x84af0a62,0x84b061dc,0x84b161de,0x84b20f1e,0x84b361e0,
	0x84b4122a,0x84b561e2,0x84b661e4,0x84b761e6,0x84b81925,0x84b90872,0x84ba080b,0x84bb61e8,
	0x84bc61ea,0x84bd048a,0x84be61ec,0x84bf06c3,0x84c061ee,0x84c11907,0x84c261f0,0x84c361f2,
	0x84c415c9,0x84c561f4,0x84c661f6,0x84c761f8,0x84c861fa,0x84c910a3,0x84ca143b,0x84cb61fc,
	0x84cc61fe,0x84cd1196,0x84ce6200,0x84cf6202,0x84d010c1,0x84d112a5,0x84d26204,0x84d300b8,
	0x84d46206,0x84d56208,0x84d600f0,0x84d7620a,0x84d8620c,0x84d9620e,0x84da6210,0x84db6212,
	0x84dc6214,0x84dd0ac7,0x84de6216,0x84df0831,0x84e00b21,0x84e16218,0x84e2621a,0x84e317fe,
	0x84e4621c,0x84e5175a,0x84e60d7a,0x84e7621e,0x84e86220,0x84e96222,0x84ea6224,0x84eb6226,
	0x84ec0e9f,0x84ed6228,0x84ee622a,0x84ef622c,0x84f014c1,0x84f1622e,0x84f26230,0x84f36232,
	0x84f46234,0x84f56236,0x84f66238,0x84f7623a,0x84f8623c,0x84f9623e,0x84fa6240,0x84fb6242,
	0x84fc0b9b,0x84fd6244,0x84fe6246,0x84ff15ca,0x85006248,0x8501624a,0x8502624c,0x8503624e,
	0x85046250,0x85056252,0x85066254,0x85076256,0x85086258,0x8509625a,0x850a625c,0x850b625e,
	0x850c127d,0x850d6260,0x850e6262,0x850f6264,0x85106266,0x85110d44,0x85126268,0x85130ca8,
	0x8514626a,0x8515626c,0x8516626e,0x851718f9,0x85186270,0x85196272,0x851a1427,0x851b6274,
	0x851c6276,0x851d6278,0x851e627a,0x851f02f7,0x8520627c,0x8521018c,0x8522627e,0x85236280,
	0x85246282,0x85256284,0x85266286,0x85276288,0x8528628a,0x8529628c,0x852a628e,0x852b0dec,
	0x852c11ee,0x852d6290,0x852e6292,0x852f6294,0x85306296,0x85316298,0x8532629a,0x8533629c,
	0x8534629e,0x853562a0,0x853662a2,0x85370faf,0x85380414,0x85390b76,0x853a0bc3,0x853b0a4f,
	0x853c000e,0x853d00f6,0x853e62a4,0x853f62a6,0x854062a8,0x854162aa,0x854262ac,0x854304ae,
	0x854462ae,0x854562b0,0x854662b2,0x854762b4,0x8548160d,0x854908c8,0x854a10c7,0x854b62b6,
	0x854c62b8,0x854d62ba,0x854e62bc,0x854f62be,0x855062c0,0x855162c2,0x855262c4,0x855362c6,
	0x855462c8,0x855562ca,0x8556103d,0x855762cc,0x855862ce,0x855907b6,0x855a62d0,0x855b62d2,
	0x855c62d4,0x855d62d6,0x855e1a56,0x855f62d8,0x856062da,0x856162dc,0x856262de,0x856362e0,
	0x856410c6,0x856562e2,0x856662e4,0x856762e6,0x856809cb,0x856962e8,0x856a62ea,0x856b62ec,
	0x856c62ee,0x856d62f0,0x856e62f2,0x856f62f4,0x857062f6,0x857162f8,0x85720f57,0x857362fa,
	0x8574184b,0x857562fc,0x857662fe,0x85776300,0x85786302,0x8579143d,0x857a080e,0x857b070e,
	0x857c6304,0x857d6306,0x857e0b0a,0x857f6308,0x8580630a,0x8581630c,0x8582630e,0x85836310,
	0x85840166,0x858506c5,0x85866312,0x858713f7,0x85886314,0x85896316,0x858a6318,0x858b631a,
	0x858c631c,0x858d631e,0x858e6320,0x858f171a,0x85906322,0x85916324,0x85926326,0x85936328,
	0x8594632a,0x8595632c,0x8596632e,0x85976330,0x85986332,0x85996334,0x859a6336,0x859b15e7,
	0x859c00fa,0x859d6338,0x859e633a,0x859f633c,0x85a0633e,0x85a16340,0x85a26342,0x85a36344,
	0x85a41566,0x85a56346,0x85a66348,0x85a7634a,0x85a80703,0x85a9634c,0x85aa1576,0x85ab634e,
	0x85ac6350,0x85ad6352,0x85ae1268,0x85af11f9,0x85b015f6,0x85b16354,0x85b26356,0x85b36358,
	0x85b4635a,0x85b5635c,0x85b6635e,0x85b710b6,0x85b86360,0x85b912c6,0x85ba6362,0x85bb6364,
	0x85bc6366,0x85bd6368,0x85be636a,0x85bf636c,0x85c0636e,0x85c105ad,0x85c26370,0x85c36372,
	0x85c46374,0x85c56376,0x85c66378,0x85c7637a,0x85c8637c,0x85c9090e,0x85ca637e,0x85cb6380,
	0x85cc6382,0x85cd6384,0x85ce6386,0x85cf019d,0x85d00d3b,0x85d16388,0x85d2638a,0x85d314fe,
	0x85d4638c,0x85d50e48,0x85d6638e,0x85d76390,0x85d86392,0x85d96394,0x85da6396,0x85db6398,
	0x85dc0b28,0x85dd639a,0x85de639c,0x85df639e,0x85e063a0,0x85e163a2,0x85e263a4,0x85e363a6,
	0x85e41318,0x85e563a8,0x85e663aa,0x85e763ac,0x85e863ae,0x85e904a8,0x85ea63b0,0x85eb63b2,
	0x85ec63b4,0x85ed63b6,0x85ee63b8,0x85ef63ba,0x85f063bc,0x85f163be,0x85f263c0,0x85f363c2,
	0x85f463c4,0x85f563c6,0x85f663c8,0x85f763ca,0x85f863cc,0x85f963ce,0x85fa63d0,0x85fb1870,
	0x85fc63d2,0x85fd63d4,0x85fe63d6,0x85ff07d5,0x860063d8,0x860163da,0x860263dc,0x860363de,
	0x860463e0,0x860506fe,0x860663e2,0x860763e4,0x860863e6,0x860963e8,0x860a63ea,0x860b63ec,
	0x860c63ee,0x860d63f0,0x860e63f2,0x860f63f4,0x861063f6,0x86110d6e,0x861263f8,0x861363fa,
	0x861463fc,0x861563fe,0x86160e0c,0x86176400,0x86186402,0x86196404,0x861a6406,0x861b6408,
	0x861c640a,0x861d640c,0x861e640e,0x861f6410,0x86206412,0x86216414,0x86226416,0x86236418,
	0x8624641a,0x8625641c,0x8626641e,0x86271040,0x86286420,0x862904b2,0x862a6422,0x862b6424,
	0x862c6426,0x862d6428,0x862e642a,0x862f642c,0x8630642e,0x86316430,0x86326432,0x86336434,
	0x86346436,0x86356438,0x8636643a,0x8637643c,0x863818c2,0x8639643e,0x863a6440,0x863b6442,
	0x863c0d11,0x863d6444,0x863e6446,0x863f6448,0x8640644a,0x8641644c,0x8642644e,0x86436450,
	0x86446452,0x86456454,0x86466456,0x86476458,0x8648645a,0x8649645c,0x864a645e,0x864b6460,
	0x864c6462,0x864d0720,0x864e0739,0x864f0c25,0x86500e30,0x86510c7e,0x86526464,0x86536466,
	0x86540f8c,0x86556468,0x8656646a,0x8657646c,0x8658646e,0x86596470,0x865a15b0,0x865b6472,
	0x865c6474,0x865d6476,0x865e17cf,0x865f6478,0x8660647a,0x8661647c,0x8662068c,0x8663647e,
	0x86646480,0x86656482,0x86666484,0x86676486,0x86686488,0x8669648a,0x866a648c,0x866b0272,
	0x866c101e,0x866d648e,0x866e0812,0x866f6490,0x86706492,0x86711191,0x86726494,0x86736496,
	0x86746498,0x8675649a,0x8676649c,0x8677649e,0x867864a0,0x8679070b,0x867a07a0,0x867b0cf3,
	0x867c05cf,0x867d1284,0x867e14cc,0x867f01ce,0x868011a1,0x868116e6,0x86820c8a,0x868364a2,
	0x868464a4,0x868564a6,0x868664a8,0x868764aa,0x868864ac,0x868964ae,0x868a142f,0x868b10ca,
	0x868c0081,0x868d0eb8,0x868e64b0,0x868f64b2,0x869064b4,0x869164b6,0x869264b8,0x8693173c,
	0x869464ba,0x86950191,0x869664bc,0x869764be,0x869864c0,0x869964c2,0x869a64c4,0x869b64c6,
	0x869c161b,0x869d06c6,0x869e64c8,0x869f64ca,0x86a064cc,0x86a164ce,0x86a264d0,0x86a305ed,
	0x86a4186e,0x86a564d2,0x86a664d4,0x86a7090c,0x86a80534,0x86a90248,0x86aa041a,0x86ab64d6,
	0x86ac14fb,0x86ad64d8,0x86ae64da,0x86af1017,0x86b01794,0x86b118a5,0x86b264dc,0x86b364de,
	0x86b417aa,0x86b50a16,0x86b606a1,0x86b764e0,0x86b864e2,0x86b964e4,0x86ba1070,0x86bb64e6,
	0x86bc64e8,0x86bd64ea,0x86be64ec,0x86bf64ee,0x86c019d0,0x86c164f0,0x86c264f2,0x86c364f4,
	0x86c40616,0x86c564f6,0x86c61032,0x86c71156,0x86c864f8,0x86c90bd7,0x86ca0627,0x86cb035f,
	0x86cc64fa,0x86cd64fc,0x86ce0b5b,0x86cf022f,0x86d01034,0x86d10d83,0x86d264fe,0x86d36500,
	0x86d4079f,0x86d56502,0x86d66504,0x86d76506,0x86d8167c,0x86d913bb,0x86da6508,0x86db19b0,
	0x86dc650a,0x86dd650c,0x86de0aa7,0x86df08c4,0x86e0650e,0x86e16510,0x86e26512,0x86e36514,
	0x86e405c3,0x86e56516,0x86e66518,0x86e7651a,0x86e8651c,0x86e91013,0x86ea651e,0x86eb6520,
	0x86ec6522,0x86ed1976,0x86ee0c9a,0x86ef6524,0x86f018ed,0x86f10852,0x86f20dc8,0x86f31236,
	0x86f40f53,0x86f56526,0x86f66528,0x86f7652a,0x86f8152a,0x86f91781,0x86fa652c,0x86fb652e,
	0x86fc6530,0x86fd6532,0x86fe0472,0x86ff6534,0x870011f8,0x87016536,0x87020509,0x8703117d,
	0x87046538,0x8705653a,0x8706653c,0x870718ea,0x8708145e,0x8709053e,0x870a0b3a,0x870b653e,
	0x870c6540,0x870d028f,0x870e6542,0x870f6544,0x87106546,0x87116548,0x87121647,0x8713135b,
	0x8714654a,0x87151396,0x8716654c,0x87171443,0x8718193e,0x8719654e,0x871a04d4,0x871b6550,
	0x871c0d21,0x871d6552,0x871e0f56,0x871f6554,0x87206556,0x87210ab2,0x87220d00,0x87230fa9,
	0x87246558,0x872514a7,0x8726655a,0x8727655c,0x8728655e,0x87291344,0x872a6560,0x872b6562,
	0x872c6564,0x872d6566,0x872e1801,0x872f6568,0x8730656a,0x87310ebf,0x8732656c,0x8733656e,
	0x87341714,0x87356570,0x87366572,0x87371058,0x87386574,0x87396576,0x873a6578,0x873b0ffa,
	0x873c657a,0x873d657c,0x873e0690,0x873f13c7,0x8740657e,0x87416580,0x87426582,0x87436584,
	0x87446586,0x87456588,0x8746658a,0x8747175c,0x87480688,0x874901db,0x874a658c,0x874b658e,
	0x874c0a26,0x874d6590,0x874e154a,0x874f6592,0x87506594,0x87516596,0x87526598,0x875317d3,
	0x8754659a,0x8755659c,0x8756659e,0x87570789,0x875865a0,0x87590107,0x875a65a2,0x875b65a4,
	0x875c65a6,0x875d65a8,0x875e65aa,0x875f65ac,0x87600541,0x876165ae,0x876265b0,0x87631798,
	0x87641028,0x87650cbd,0x876665b2,0x876765b4,0x876865b6,0x876965b8,0x876a65ba,0x876b65bc,
	0x876c65be,0x876d65c0,0x876e0565,0x876f65c2,0x87700a8c,0x877165c4,0x877265c6,0x877365c8,
	0x87740736,0x877565ca,0x877603e8,0x877765cc,0x877865ce,0x877965d0,0x877a65d2,0x877b0dbc,
	0x877c0c0d,0x877d02c2,0x877e10a6,0x877f65d4,0x878065d6,0x878165d8,0x87820ae0,0x87830e74,
	0x878465da,0x878514ae,0x878665dc,0x878765de,0x8788181c,0x878965e0,0x878a65e2,0x878b1264,
	0x878c65e4,0x878d10a7,0x878e65e6,0x878f65e8,0x879065ea,0x879165ec,0x879265ee,0x87930fec,
	0x879465f0,0x879565f2,0x879665f4,0x879712f3,0x879865f6,0x879965f8,0x879a65fa,0x879b65fc,
	0x879c65fe,0x879d6600,0x879e6602,0x879f0d60,0x87a06604,0x87a16606,0x87a26608,0x87a3660a,
	0x87a4660c,0x87a5660e,0x87a66610,0x87a76612,0x87a80ca1,0x87a96614,0x87aa6616,0x87ab11cd,
	0x87ac01a5,0x87ad024f,0x87ae6618,0x87af0037,0x87b0661a,0x87b1661c,0x87b2661e,0x87b312f4,
	0x87b46620,0x87b50ede,0x87b66622,0x87b76624,0x87b86626,0x87b96628,0x87ba0c5f,0x87bb662a,
	0x87bc662c,0x87bd1986,0x87be662e,0x87bf6630,0x87c01211,0x87c16632,0x87c26634,0x87c36636,
	0x87c46638,0x87c5663a,0x87c60c8e,0x87c7663c,0x87c8663e,0x87c96640,0x87ca0cbe,0x87cb14af,
	0x87cc6642,0x87cd6644,0x87ce6646,0x87cf6648,0x87d0664a,0x87d118cc,0x87d20cb4,0x87d31529,
	0x87d4664c,0x87d5664e,0x87d66650,0x87d76652,0x87d86654,0x87d96656,0x87da6658,0x87db0ea4,
	0x87dc665a,0x87dd665c,0x87de665e,0x87df6660,0x87e00e64,0x87e16662,0x87e26664,0x87e36666,
	0x87e46668,0x87e5077c,0x87e6666a,0x87e7666c,0x87e8666e,0x87e96670,0x87ea07b7,0x87eb6672,
	0x87ec6674,0x87ed6676,0x87ee112f,0x87ef6678,0x87f0667a,0x87f1667c,0x87f2667e,0x87f36680,
	0x87f46682,0x87f56684,0x87f66686,0x87f76688,0x87f8668a,0x87f9156a,0x87fa668c,0x87fb668e,
	0x87fc6690,0x87fd6692,0x87fe01de,0x87ff6694,0x88006696,0x88016698,0x8802669a,0x88030c63,
	0x8804669c,0x8805669e,0x880666a0,0x880766a2,0x880866a4,0x880966a6,0x880a0b71,0x880b66a8,
	0x880c66aa,0x880d66ac,0x880e66ae,0x880f66b0,0x881066b2,0x881166b4,0x881266b6,0x88130d02,
	0x881466b8,0x881510b8,0x881607d6,0x881766ba,0x881866bc,0x881966be,0x881a66c0,0x881b0d46,
	0x881c66c2,0x881d66c4,0x881e66c6,0x881f66c8,0x882066ca,0x88210b2b,0x882202c9,0x882366cc,
	0x882466ce,0x882566d0,0x882666d2,0x882766d4,0x882866d6,0x882966d8,0x882a66da,0x882b66dc,
	0x882c66de,0x882d66e0,0x882e66e2,0x882f66e4,0x883066e6,0x883166e8,0x883209af,0x883366ea,
	0x883466ec,0x883566ee,0x883666f0,0x883766f2,0x883866f4,0x88390435,0x883a66f6,0x883b66f8,
	0x883c1043,0x883d66fa,0x883e66fc,0x883f66fe,0x884015f0,0x88416700,0x88426702,0x88436704,
	0x88440e3c,0x8845157c,0x88466706,0x88476708,0x8848670a,0x8849670c,0x884a670e,0x884b6710,
	0x884c1584,0x884d164e,0x884e6712,0x884f6714,0x88506716,0x88516718,0x8852671a,0x8853671c,
	0x885414f3,0x8855671e,0x88566720,0x885708ed,0x88586722,0x8859161f,0x885a6724,0x885b6726,
	0x885c6728,0x885d672a,0x885e672c,0x885f672e,0x88606730,0x886106fd,0x88621047,0x886316bd,
	0x886416bb,0x88650173,0x88666732,0x88676734,0x88680127,0x886901c1,0x886a6736,0x886b1118,
	0x886c0228,0x886d6738,0x886e067c,0x886f673a,0x8870120d,0x8871673c,0x88720da7,0x8873673e,
	0x88746740,0x88756742,0x88766744,0x88771984,0x88786746,0x88796748,0x887a674a,0x887b674c,
	0x887c674e,0x887d1096,0x887e0fe0,0x887f0916,0x88806750,0x88811813,0x88820ce7,0x88836752,
	0x8884003b,0x88850dfc,0x88866754,0x88876756,0x88880847,0x88896758,0x888a675a,0x888b0343,
	0x888c675c,0x888d0e7e,0x888e675e,0x888f6760,0x88906762,0x88916764,0x889212df,0x88936766,
	0x88946768,0x8895676a,0x889615a8,0x8897676c,0x8898676e,0x88996770,0x889a6772,0x889b6774,
	0x889c13bf,0x889d6776,0x889e6778,0x889f677a,0x88a0677c,0x88a1677e,0x88a20e6c,0x88a36780,
	0x88a40cc9,0x88a56782,0x88a66784,0x88a76786,0x88a86788,0x88a9678a,0x88aa678c,0x88ab00b3,
	0x88ac678e,0x88ad14b5,0x88ae6790,0x88af6792,0x88b06794,0x88b1053b,0x88b26796,0x88b36798,
	0x88b4679a,0x88b5679c,0x88b6679e,0x88b7084c,0x88b867a0,0x88b967a2,0x88ba67a4,0x88bb67a6,
	0x88bc05b8,0x88bd67a8,0x88be67aa,0x88bf67ac,0x88c067ae,0x88c10186,0x88c20baa,0x88c367b0,
	0x88c467b2,0x88c519e4,0x88c60363,0x88c767b4,0x88c867b6,0x88c90a3c,0x88ca67b8,0x88cb67ba,
	0x88cc67bc,0x88cd67be,0x88ce023d,0x88cf67c0,0x88d067c2,0x88d167c4,0x88d20f12,0x88d367c6,
	0x88d41712,0x88d517f9,0x88d667c8,0x88d767ca,0x88d81027,0x88d9106e,0x88da67cc,0x88db67ce,
	0x88dc67d0,0x88dd67d2,0x88de67d4,0x88df1106,0x88e067d6,0x88e167d8,0x88e20b6b,0x88e30b75,
	0x88e40a5a,0x88e50884,0x88e667da,0x88e767dc,0x88e800f1,0x88e967de,0x88ea67e0,0x88eb67e2,
	0x88ec67e4,0x88ed67e6,0x88ee67e8,0x88ef67ea,0x88f00459,0x88f10129,0x88f267ec,0x88f3113d,
	0x88f40e8a,0x88f567ee,0x88f667f0,0x88f767f2,0x88f80c61,0x88f90691,0x88fa67f4,0x88fb67f6,
	0x88fc132e,0x88fd67f8,0x88fe0983,0x88ff67fa,0x890067fc,0x890167fe,0x89020641,0x89036800,
	0x89046802,0x89056804,0x89066806,0x89076808,0x8908680a,0x8909680c,0x890a010f,0x890b680e,
	0x890c6810,0x890d6812,0x890e6814,0x890f6816,0x891006ed,0x89116818,0x89120093,0x8913009a,
	0x8914681a,0x8915681c,0x8916681e,0x89176820,0x89186822,0x891900b9,0x891a0299,0x891b0c79,
	0x891c6824,0x891d6826,0x891e6828,0x891f682a,0x8920682c,0x89210328,0x8922682e,0x89236830,
	0x89246832,0x892510c2,0x89266834,0x89276836,0x89286838,0x8929683a,0x892a1397,0x892b025f,
	0x892c683c,0x892d683e,0x892e6840,0x892f6842,0x89300f88,0x89316844,0x89326846,0x89336848,
	0x89340aca,0x8935684a,0x893618f5,0x8937684c,0x8938684e,0x89396850,0x893a6852,0x893b6854,
	0x893c6856,0x893d6858,0x893e685a,0x893f685c,0x8940685e,0x89410fb4,0x89426860,0x89436862,
	0x89441516,0x89456864,0x89466866,0x89476868,0x8948686a,0x8949686c,0x894a686e,0x894b6870,
	0x894c6872,0x894d6874,0x894e6876,0x894f6878,0x8950687a,0x8951687c,0x8952687e,0x89536880,
	0x89546882,0x89556884,0x89566886,0x89576888,0x8958688a,0x8959688c,0x895a688e,0x895b6890,
	0x895c6892,0x895d6894,0x895e0100,0x895f0918,0x89606896,0x89616898,0x8962689a,0x8963689c,
	0x8964689e,0x896568a0,0x896610b7,0x896768a2,0x896868a4,0x896968a6,0x896a68a8,0x896b68aa,
	0x896c68ac,0x896d68ae,0x896e68b0,0x896f68b2,0x897068b4,0x897168b6,0x897268b8,0x897368ba,
	0x897468bc,0x897568be,0x897668c0,0x897768c2,0x897868c4,0x897968c6,0x897a68c8,0x897b0e6d,
	0x897c68ca,0x897d68cc,0x897e68ce,0x897f1487,0x898068d0,0x8981169f,0x898268d2,0x89830fed,
	0x898468d4,0x898568d6,0x89860567,0x898768d8,0x898868da,0x898968dc,0x898a68de,0x898b68e0,
	0x898c68e2,0x898d68e4,0x898e68e6,0x898f68e8,0x899068ea,0x899168ec,0x899268ee,0x899368f0,
	0x899468f2,0x899568f4,0x899668f6,0x899768f8,0x899868fa,0x899968fc,0x899a68fe,0x899b6900,
	0x899c6902,0x899d6904,0x899e6906,0x899f6908,0x89a0690a,0x89a1690c,0x89a2690e,0x89a36910,
	0x89a46912,0x89a56914,0x89a66916,0x89a76918,0x89a8691a,0x89a9691c,0x89aa691e,0x89ab6920,
	0x89ac6922,0x89ad6924,0x89ae6926,0x89af6928,0x89b0692a,0x89b1692c,0x89b2692e,0x89b36930,
	0x89b46932,0x89b56934,0x89b66936,0x89b76938,0x89b8693a,0x89b9693c,0x89ba693e,0x89bb6940,
	0x89bc6942,0x89bd6944,0x89be6946,0x89bf6948,0x89c0694a,0x89c1088d,0x89c20647,0x89c3694c,
	0x89c40662,0x89c50d1b,0x89c611b8,0x89c701d2,0x89c80acf,0x89c909c1,0x89ca082b,0x89cb14b6,
	0x89cc039e,0x89cd694e,0x89ce17d0,0x89cf060c,0x89d00930,0x89d1104a,0x89d208cc,0x89d36950,
	0x89d46952,0x89d56954,0x89d609c6,0x89d76956,0x89d86958,0x89d9695a,0x89da0617,0x89db695c,
	0x89dc1a1b,0x89dd695e,0x89de1134,0x89df6960,0x89e06962,0x89e16964,0x89e26966,0x89e30903,
	0x89e46968,0x89e505f0,0x89e6029f,0x89e7696a,0x89e8696c,0x89e9696e,0x89ea6970,0x89eb127e,
	0x89ec6972,0x89ed6974,0x89ee6976,0x89ef197c,0x89f06978,0x89f1697a,0x89f2697c,0x89f30738,
	0x89f4697e,0x89f56980,0x89f66982,0x89f76984,0x89f86986,0x89f96988,0x89fa698a,0x89fb698c,
	0x89fc698e,0x89fd6990,0x89fe6992,0x89ff6994,0x8a00163f,0x8a016996,0x8a026998,0x8a03699a,
	0x8a04699c,0x8a05699e,0x8a0669a0,0x8a070701,0x8a0869a2,0x8a0969a4,0x8a0a69a6,0x8a0b69a8,
	0x8a0c69aa,0x8a0d69ac,0x8a0e69ae,0x8a0f69b0,0x8a1069b2,0x8a1169b4,0x8a1269b6,0x8a1369b8,
	0x8a1469ba,0x8a1569bc,0x8a1669be,0x8a1769c0,0x8a1869c2,0x8a1969c4,0x8a1a69c6,0x8a1b69c8,
	0x8a1c69ca,0x8a1d69cc,0x8a1e69ce,0x8a1f69d0,0x8a2069d2,0x8a2169d4,0x8a2269d6,0x8a2369d8,
	0x8a2469da,0x8a2569dc,0x8a2669de,0x8a2769e0,0x8a2869e2,0x8a2969e4,0x8a2a69e6,0x8a2b69e8,
	0x8a2c69ea,0x8a2d69ec,0x8a2e69ee,0x8a2f69f0,0x8a3069f2,0x8a3169f4,0x8a3269f6,0x8a3369f8,
	0x8a3469fa,0x8a3569fc,0x8a3669fe,0x8a376a00,0x8a386a02,0x8a396a04,0x8a3a6a06,0x8a3b6a08,
	0x8a3c6a0a,0x8a3d6a0c,0x8a3e1a21,0x8a3f6a0e,0x8a406a10,0x8a416a12,0x8a426a14,0x8a436a16,
	0x8a446a18,0x8a456a1a,0x8a466a1c,0x8a476a1e,0x8a480b5e,0x8a496a20,0x8a4a6a22,0x8a4b6a24,
	0x8a4c6a26,0x8a4d6a28,0x8a4e6a2a,0x8a4f6a2c,0x8a506a2e,0x8a516a30,0x8a526a32,0x8a536a34,
	0x8a546a36,0x8a556a38,0x8a566a3a,0x8a576a3c,0x8a586a3e,0x8a596a40,0x8a5a6a42,0x8a5b6a44,
	0x8a5c6a46,0x8a5d6a48,0x8a5e6a4a,0x8a5f6a4c,0x8a606a4e,0x8a616a50,0x8a626a52,0x8a636a54,
	0x8a646a56,0x8a656a58,0x8a666a5a,0x8a676a5c,0x8a686a5e,0x8a696a60,0x8a6a6a62,0x8a6b6a64,
	0x8a6c6a66,0x8a6d6a68,0x8a6e6a6a,0x8a6f6a6c,0x8a706a6e,0x8a716a70,0x8a726a72,0x8a736a74,
	0x8a746a76,0x8a756a78,0x8a766a7a,0x8a776a7c,0x8a786a7e,0x8a7918b3,0x8a7a6a80,0x8a7b6a82,
	0x8a7c6a84,0x8a7d6a86,0x8a7e6a88,0x8a7f6a8a,0x8a806a8c,0x8a816a8e,0x8a826a90,0x8a836a92,
	0x8a846a94,0x8a856a96,0x8a866a98,0x8a876a9a,0x8a886a9c,0x8a8917ff,0x8a8a1316,0x8a8b6a9e,
	0x8a8c6aa0,0x8a8d6aa2,0x8a8e6aa4,0x8a8f6aa6,0x8a906aa8,0x8a916aaa,0x8a926aac,0x8a9311cb,
	0x8a946aae,0x8a956ab0,0x8a966ab2,0x8a976ab4,0x8a986ab6,0x8a996ab8,0x8a9a6aba,0x8a9b6abc,
	0x8a9c6abe,0x8a9d6ac0,0x8a9e6ac2,0x8a9f6ac4,0x8aa06ac6,0x8aa16ac8,0x8aa26aca,0x8aa36acc,
	0x8aa46ace,0x8aa56ad0,0x8aa66ad2,0x8aa76ad4,0x8aa86ad6,0x8aa96ad8,0x8aaa6ada,0x8aab6adc,
	0x8aac6ade,0x8aad6ae0,0x8aae6ae2,0x8aaf6ae4,0x8ab06ae6,0x8ab16ae8,0x8ab26aea,0x8ab36aec,
	0x8ab46aee,0x8ab56af0,0x8ab66af2,0x8ab76af4,0x8ab86af6,0x8ab96af8,0x8aba6afa,0x8abb6afc,
	0x8abc6afe,0x8abd6b00,0x8abe6b02,0x8abf6b04,0x8ac06b06,0x8ac16b08,0x8ac26b0a,0x8ac36b0c,
	0x8ac46b0e,0x8ac56b10,0x8ac66b12,0x8ac76b14,0x8ac86b16,0x8ac96b18,0x8aca6b1a,0x8acb6b1c,
	0x8acc6b1e,0x8acd6b20,0x8ace6b22,0x8acf6b24,0x8ad06b26,0x8ad16b28,0x8ad26b2a,0x8ad36b2c,
	0x8ad46b2e,0x8ad56b30,0x8ad66b32,0x8ad76b34,0x8ad86b36,0x8ad96b38,0x8ada6b3a,0x8adb6b3c,
	0x8adc6b3e,0x8add6b40,0x8ade6b42,0x8adf6b44,0x8ae06b46,0x8ae16b48,0x8ae26b4a,0x8ae36b4c,
	0x8ae46b4e,0x8ae56b50,0x8ae66b52,0x8ae76b54,0x8ae86b56,0x8ae96b58,0x8aea6b5a,0x8aeb6b5c,
	0x8aec6b5e,0x8aed6b60,0x8aee6b62,0x8aef6b64,0x8af06b66,0x8af16b68,0x8af26b6a,0x8af36b6c,
	0x8af46b6e,0x8af56b70,0x8af66b72,0x8af76b74,0x8af86b76,0x8af96b78,0x8afa6b7a,0x8afb6b7c,
	0x8afc6b7e,0x8afd6b80,0x8afe6b82,0x8aff6b84,0x8b006b86,0x8b016b88,0x8b026b8a,0x8b036b8c,
	0x8b046b8e,0x8b056b90,0x8b066b92,0x8b07088b,0x8b086b94,0x8b096b96,0x8b0a6b98,0x8b0b6b9a,
	0x8b0c6b9c,0x8b0d6b9e,0x8b0e6ba0,0x8b0f6ba2,0x8b106ba4,0x8b116ba6,0x8b126ba8,0x8b136baa,
	0x8b146bac,0x8b156bae,0x8b166bb0,0x8b176bb2,0x8b186bb4,0x8b196bb6,0x8b1a6bb8,0x8b1b6bba,
	0x8b1c6bbc,0x8b1d6bbe,0x8b1e6bc0,0x8b1f6bc2,0x8b206bc4,0x8b216bc6,0x8b226bc8,0x8b236bca,
	0x8b246bcc,0x8b256bce,0x8b261005,0x8b276bd0,0x8b286bd2,0x8b296bd4,0x8b2a6bd6,0x8b2b6bd8,
	0x8b2c6bda,0x8b2d6bdc,0x8b2e6bde,0x8b2f6be0,0x8b306be2,0x8b316be4,0x8b326be6,0x8b336be8,
	0x8b346bea,0x8b356bec,0x8b366bee,0x8b376bf0,0x8b386bf2,0x8b396bf4,0x8b3a6bf6,0x8b3b6bf8,
	0x8b3c6bfa,0x8b3d6bfc,0x8b3e6bfe,0x8b3f6c00,0x8b406c02,0x8b416c04,0x8b426c06,0x8b436c08,
	0x8b446c0a,0x8b456c0c,0x8b466c0e,0x8b476c10,0x8b486c12,0x8b496c14,0x8b4a6c16,0x8b4b6c18,
	0x8b4c6c1a,0x8b4d6c1c,0x8b4e6c1e,0x8b4f6c20,0x8b506c22,0x8b516c24,0x8b526c26,0x8b536c28,
	0x8b546c2a,0x8b556c2c,0x8b566c2e,0x8b576c30,0x8b586c32,0x8b596c34,0x8b5a6c36,0x8b5b6c38,
	0x8b5c6c3a,0x8b5d6c3c,0x8b5e6c3e,0x8b5f6c40,0x8b606c42,0x8b616c44,0x8b626c46,0x8b636c48,
	0x8b646c4a,0x8b656c4c,0x8b660949,0x8b676c4e,0x8b686c50,0x8b696c52,0x8b6a6c54,0x8b6b6c56,
	0x8b6c0ed0,0x8b6d6c58,0x8b6e6c5a,0x8b6f6c5c,0x8b706c5e,0x8b716c60,0x8b726c62,0x8b736c64,
	0x8b746c66,0x8b756c68,0x8b766c6a,0x8b776c6c,0x8b786c6e,0x8b796c70,0x8b7a6c72,0x8b7b6c74,
	0x8b7c6c76,0x8b7d6c78,0x8b7e6c7a,0x8b7f6c7c,0x8b806c7e,0x8b816c80,0x8b826c82,0x8b836c84,
	0x8b846c86,0x8b856c88,0x8b866c8a,0x8b876c8c,0x8b886c8e,0x8b896c90,0x8b8a6c92,0x8b8b6c94,
	0x8b8c6c96,0x8b8d6c98,0x8b8e6c9a,0x8b8f6c9c,0x8b906c9e,0x8b916ca0,0x8b926ca2,0x8b936ca4,
	0x8b946ca6,0x8b956ca8,0x8b966caa,0x8b976cac,0x8b986cae,0x8b996cb0,0x8b9a6cb2,0x8b9b6cb4,
	0x8b9c6cb6,0x8b9d6cb8,0x8b9e6cba,0x8b9f6cbc,0x8ba01639,0x8ba1081b,0x8ba203f7,0x8ba30553,
	0x8ba4108e,0x8ba507d7,0x8ba608f1,0x8ba7070f,0x8ba8130d,0x8ba9107d,0x8baa1123,0x8bab0f66,
	0x8bac6cbe,0x8bad1605,0x8bae16f0,0x8baf1606,0x8bb0081c,0x8bb16cc0,0x8bb208ae,0x8bb307a7,
	0x8bb40e40,0x8bb50995,0x8bb61624,0x8bb70dd0,0x8bb815b6,0x8bb9046b,0x8bba0c54,0x8bbb6cc2,
	0x8bbc1258,0x8bbd050e,0x8bbe1159,0x8bbf04c9,0x8bc009bd,0x8bc1192b,0x8bc20620,0x8bc306d6,
	0x8bc40ef9,0x8bc51a49,0x8bc6119d,0x8bc76cc4,0x8bc818a0,0x8bc91270,0x8bca190d,0x8bcb03a1,
	0x8bcc1991,0x8bcd02d2,0x8bce102e,0x8bcf18e1,0x8bd06cc6,0x8bd116f8,0x8bd216cb,0x8bd30a6f,
	0x8bd40b07,0x8bd511b9,0x8bd6063f,0x8bd71192,0x8bd80810,0x8bd907a3,0x8bda0237,0x8bdb19a7,
	0x8bdc1168,0x8bdd0755,0x8bde0359,0x8bdf0605,0x8be01050,0x8be1066c,0x8be215fc,0x8be316ff,
	0x8be4192c,0x8be50570,0x8be61519,0x8be701c4,0x8be807bf,0x8be915b7,0x8bea6cc8,0x8beb090a,
	0x8bec1455,0x8bed17dc,0x8bee0fcc,0x8bef1478,0x8bf005af,0x8bf117a9,0x8bf207ab,0x8bf30a73,
	0x8bf41223,0x8bf5125a,0x8bf60488,0x8bf71004,0x8bf819ad,0x8bf91a3c,0x8bfa0e33,0x8bfb0423,
	0x8bfc1a03,0x8bfd04dc,0x8bfe0a32,0x8bff1414,0x8c0017c1,0x8c01121a,0x8c021173,0x8c0303d8,
	0x8c0401e2,0x8c050b8a,0x8c0619f5,0x8c07128e,0x8c0812d6,0x8c096cca,0x8c0a1709,0x8c0b0d87,
	0x8c0c0226,0x8c0d03e1,0x8c0e0790,0x8c0f0899,0x8c101551,0x8c1115f1,0x8c1216b7,0x8c131423,
	0x8c14047d,0x8c1517f4,0x8c1615cd,0x8c1701d6,0x8c181a15,0x8c19001d,0x8c1a1661,0x8c1b03af,
	0x8c1c0d0b,0x8c1d0ed8,0x8c1e6ccc,0x8c1f0d66,0x8c200366,0x8c211277,0x8c221560,0x8c231692,
	0x8c240087,0x8c2511c7,0x8c260f83,0x8c270d1f,0x8c28091e,0x8c290c9f,0x8c2a18ee,0x8c2b0887,
	0x8c2c0d63,0x8c2d12da,0x8c2e1887,0x8c2f0fc2,0x8c300ac8,0x8c310f28,0x8c3209c9,0x8c331668,
	0x8c340f97,0x8c3518b4,0x8c36022c,0x8c370621,0x8c386cce,0x8c396cd0,0x8c3a6cd2,0x8c3b6cd4,
	0x8c3c6cd6,0x8c3d6cd8,0x8c3e6cda,0x8c3f6cdc,0x8c406cde,0x8c4107c4,0x8c426ce0,0x8c436ce2,
	0x8c446ce4,0x8c456ce6,0x8c46041b,0x8c4708a9,0x8c486ce8,0x8c49025e,0x8c4a6cea,0x8c4b6cec,
	0x8c4c13c8,0x8c4d6cee,0x8c4e6cf0,0x8c4f6cf2,0x8c506cf4,0x8c516cf6,0x8c526cf8,0x8c536cfa,
	0x8c546cfc,0x8c5511a7,0x8c566cfe,0x8c576d00,0x8c586d02,0x8c596d04,0x8c5a139c,0x8c5b6d06,
	0x8c5c6d08,0x8c5d6d0a,0x8c5e6d0c,0x8c5f6d0e,0x8c606d10,0x8c611526,0x8c620775,0x8c636d12,
	0x8c646d14,0x8c656d16,0x8c666d18,0x8c676d1a,0x8c686d1c,0x8c696d1e,0x8c6a06c9,0x8c6b1802,
	0x8c6c6d20,0x8c6d6d22,0x8c6e6d24,0x8c6f6d26,0x8c706d28,0x8c716d2a,0x8c726d2c,0x8c730139,
	0x8c746d2e,0x8c756d30,0x8c766d32,0x8c776d34,0x8c78195d,0x8c79009d,0x8c7a01cc,0x8c7b6d36,
	0x8c7c6d38,0x8c7d6d3a,0x8c7e6d3c,0x8c7f6d3e,0x8c806d40,0x8c816d42,0x8c8203d2,0x8c836d44,
	0x8c846d46,0x8c8515a1,0x8c866d48,0x8c876d4a,0x8c886d4c,0x8c8906e9,0x8c8a0d7b,0x8c8b6d4e,
	0x8c8c0ccd,0x8c8d6d50,0x8c8e6d52,0x8c8f6d54,0x8c906d56,0x8c916d58,0x8c926d5a,0x8c936d5c,
	0x8c940eca,0x8c956d5e,0x8c966d60,0x8c976d62,0x8c980d80,0x8c996d64,0x8c9a6d66,0x8c9b6d68,
	0x8c9c6d6a,0x8c9d6d6c,0x8c9e6d6e,0x8c9f6d70,0x8ca06d72,0x8ca16d74,0x8ca26d76,0x8ca36d78,
	0x8ca46d7a,0x8ca56d7c,0x8ca66d7e,0x8ca76d80,0x8ca86d82,0x8ca96d84,0x8caa6d86,0x8cab6d88,
	0x8cac6d8a,0x8cad6d8c,0x8cae6d8e,0x8caf6d90,0x8cb06d92,0x8cb16d94,0x8cb26d96,0x8cb36d98,
	0x8cb46d9a,0x8cb56d9c,0x8cb66d9e,0x8cb76da0,0x8cb86da2,0x8cb96da4,0x8cba6da6,0x8cbb6da8,
	0x8cbc6daa,0x8cbd6dac,0x8cbe6dae,0x8cbf6db0,0x8cc06db2,0x8cc16db4,0x8cc26db6,0x8cc36db8,
	0x8cc46dba,0x8cc56dbc,0x8cc66dbe,0x8cc76dc0,0x8cc86dc2,0x8cc96dc4,0x8cca6dc6,0x8ccb6dc8,
	0x8ccc6dca,0x8ccd6dcc,0x8cce6dce,0x8ccf6dd0,0x8cd06dd2,0x8cd16dd4,0x8cd26dd6,0x8cd36dd8,
	0x8cd46dda,0x8cd56ddc,0x8cd66dde,0x8cd76de0,0x8cd86de2,0x8cd96de4,0x8cda6de6,0x8cdb6de8,
	0x8cdc6dea,0x8cdd6dec,0x8cde6dee,0x8cdf6df0,0x8ce06df2,0x8ce16df4,0x8ce26df6,0x8ce36df8,
	0x8ce46dfa,0x8ce56dfc,0x8ce66dfe,0x8ce76e00,0x8ce86e02,0x8ce96e04,0x8cea6e06,0x8ceb6e08,
	0x8cec6e0a,0x8ced6e0c,0x8cee6e0e,0x8cef6e10,0x8cf06e12,0x8cf16e14,0x8cf26e16,0x8cf36e18,
	0x8cf46e1a,0x8cf56e1c,0x8cf66e1e,0x8cf76e20,0x8cf86e22,0x8cf96e24,0x8cfa6e26,0x8cfb6e28,
	0x8cfc6e2a,0x8cfd6e2c,0x8cfe6e2e,0x8cff6e30,0x8d006e32,0x8d016e34,0x8d026e36,0x8d036e38,
	0x8d046e3a,0x8d056e3c,0x8d066e3e,0x8d076e40,0x8d086e42,0x8d096e44,0x8d0a6e46,0x8d0b6e48,
	0x8d0c6e4a,0x8d0d6e4c,0x8d0e6e4e,0x8d0f6e50,0x8d106e52,0x8d116e54,0x8d126e56,0x8d136e58,
	0x8d146e5a,0x8d156e5c,0x8d166e5e,0x8d176e60,0x8d186e62,0x8d196e64,0x8d1a6e66,0x8d1b6e68,
	0x8d1c6e6a,0x8d1d00ab,0x8d1e18fc,0x8d1f0556,0x8d206e6c,0x8d2105f7,0x8d220185,0x8d23187b,
	0x8d2414ee,0x8d250066,0x8d2618d5,0x8d2707ce,0x8d281963,0x8d2904b9,0x8d2a12cf,0x8d2b0eed,
	0x8d2c010a,0x8d2d0606,0x8d2e19c9,0x8d2f064f,0x8d300499,0x8d310894,0x8d3200bf,0x8d3311c0,
	0x8d34134b,0x8d350675,0x8d360a7b,0x8d370341,0x8d380cc7,0x8d3904e6,0x8d3a06ec,0x8d3b16d2,
	0x8d3c1885,0x8d3d196c,0x8d3e0857,0x8d3f07ae,0x8d401a11,0x8d410bc2,0x8d420c2e,0x8d431864,
	0x8d441a12,0x8d450573,0x8d46092d,0x8d471024,0x8d481919,0x8d490abb,0x8d4a1151,0x8d4b0560,
	0x8d4c042c,0x8d4d07ed,0x8d4e11f1,0x8d4f1139,0x8d5002e0,0x8d516e6e,0x8d526e70,0x8d5305dc,
	0x8d540e88,0x8d550355,0x8d560abd,0x8d576e72,0x8d5819f2,0x8d590564,0x8d5a19dd,0x8d5b10df,
	0x8d5c1882,0x8d5d166b,0x8d5e1861,0x8d5f6e74,0x8d60188f,0x8d61112e,0x8d62175e,0x8d630592,
	0x8d640263,0x8d656e76,0x8d66115d,0x8d670dba,0x8d686e78,0x8d696e7a,0x8d6a6e7c,0x8d6b06ee,
	0x8d6c6e7e,0x8d6d18f4,0x8d6e6e80,0x8d6f6e82,0x8d701a40,0x8d716e84,0x8d726e86,0x8d730962,
	0x8d74055c,0x8d7518e2,0x8d760589,0x8d770f62,0x8d786e88,0x8d796e8a,0x8d7a6e8c,0x8d7b6e8e,
	0x8d7c6e90,0x8d7d6e92,0x8d7e6e94,0x8d7f6e96,0x8d806e98,0x8d81022a,0x8d826e9a,0x8d836e9c,
	0x8d840fd4,0x8d850208,0x8d866e9e,0x8d876ea0,0x8d886ea2,0x8d896ea4,0x8d8a1831,0x8d8b1035,
	0x8d8c6ea6,0x8d8d6ea8,0x8d8e6eaa,0x8d8f6eac,0x8d906eae,0x8d911a1c,0x8d926eb0,0x8d936eb2,
	0x8d940bab,0x8d956eb4,0x8d966eb6,0x8d976eb8,0x8d986eba,0x8d996ebc,0x8d9a6ebe,0x8d9b6ec0,
	0x8d9c6ec2,0x8d9d6ec4,0x8d9e6ec6,0x8d9f12fe,0x8da06ec8,0x8da16eca,0x8da26ecc,0x8da3104b,
	0x8da46ece,0x8da56ed0,0x8da66ed2,0x8da76ed4,0x8da86ed6,0x8da96ed8,0x8daa6eda,0x8dab6edc,
	0x8dac6ede,0x8dad6ee0,0x8dae6ee2,0x8daf6ee4,0x8db06ee6,0x8db1185f,0x8db26ee8,0x8db31a45,
	0x8db40e4b,0x8db5009e,0x8db66eea,0x8db76eec,0x8db8044d,0x8db96eee,0x8dba0519,0x8dbb6ef0,
	0x8dbc0881,0x8dbd6ef2,0x8dbe1957,0x8dbf12b0,0x8dc06ef4,0x8dc16ef6,0x8dc26ef8,0x8dc3182f,
	0x8dc40fa7,0x8dc56efa,0x8dc612c4,0x8dc76efc,0x8dc86efe,0x8dc96f00,0x8dca6f02,0x8dcb0053,
	0x8dcc03dd,0x8dcd6f04,0x8dce13ac,0x8dcf0849,0x8dd06f06,0x8dd10e80,0x8dd26f08,0x8dd36f0a,
	0x8dd46f0c,0x8dd56f0e,0x8dd6194a,0x8dd7051c,0x8dd86f10,0x8dd96f12,0x8dda111c,0x8ddb0168,
	0x8ddc6f14,0x8ddd09a0,0x8dde0b5f,0x8ddf05d4,0x8de06f16,0x8de16f18,0x8de26f1a,0x8de314fd,
	0x8de408c5,0x8de56f1c,0x8de66f1e,0x8de76f20,0x8de80a61,0x8de96f22,0x8dea0677,0x8deb100b,
	0x8dec0a90,0x8ded6f24,0x8dee6f26,0x8def0c35,0x8df06f28,0x8df16f2a,0x8df26f2c,0x8df3134a,
	0x8df46f2e,0x8df5089e,0x8df66f30,0x8df70fb7,0x8df800f2,0x8df914e7,0x8dfa0467,0x8dfb07ef,
	0x8dfc6f32,0x8dfd0833,0x8dfe6f34,0x8dff6f36,0x8e006f38,0x8e016f3a,0x8e026f3c,0x8e036f3e,
	0x8e046f40,0x8e0515ec,0x8e066f42,0x8e076f44,0x8e086f46,0x8e090b85,0x8e0a1782,0x8e0b6f48,
	0x8e0c0282,0x8e0d6f4a,0x8e0e6f4c,0x8e0f12bc,0x8e106f4e,0x8e116f50,0x8e126f52,0x8e136f54,
	0x8e1402ca,0x8e156f56,0x8e166f58,0x8e176f5a,0x8e186f5c,0x8e196f5e,0x8e1a6f60,0x8e1b6f62,
	0x8e1c6f64,0x8e1d075b,0x8e1e09a7,0x8e1f0258,0x8e206f66,0x8e216f68,0x8e22131c,0x8e230165,
	0x8e246f6a,0x8e256f6c,0x8e266f6e,0x8e276f70,0x8e286f72,0x8e29018a,0x8e2a1a34,0x8e2b6f74,
	0x8e2c197d,0x8e2d6f76,0x8e2e03be,0x8e2f194c,0x8e306f78,0x8e31045d,0x8e326f7a,0x8e336f7c,
	0x8e346f7e,0x8e35198a,0x8e366f80,0x8e376f82,0x8e386f84,0x8e3901d0,0x8e3a08a3,0x8e3b6f86,
	0x8e3c6f88,0x8e3d0992,0x8e3e6f8a,0x8e3f6f8c,0x8e4003e9,0x8e410ed7,0x8e4210ac,0x8e436f8e,
	0x8e441323,0x8e456f90,0x8e466f92,0x8e47088c,0x8e480377,0x8e490319,0x8e4a1483,0x8e4b12bd,
	0x8e4c6f94,0x8e4d6f96,0x8e4e6f98,0x8e4f6f9a,0x8e506f9c,0x8e510e0a,0x8e520e63,0x8e536f9e,
	0x8e546fa0,0x8e556fa2,0x8e566fa4,0x8e576fa6,0x8e586fa8,0x8e5902fa,0x8e5a6faa,0x8e5b6fac,
	0x8e5c6fae,0x8e5d6fb0,0x8e5e6fb2,0x8e5f6fb4,0x8e606fb6,0x8e616fb8,0x8e626fba,0x8e636fbc,
	0x8e646fbe,0x8e656fc0,0x8e6600cd,0x8e676fc2,0x8e686fc4,0x8e69012f,0x8e6a6fc6,0x8e6b6fc8,
	0x8e6c0387,0x8e6d01b1,0x8e6e6fca,0x8e6f04b1,0x8e700293,0x8e716fcc,0x8e72044b,0x8e736fce,
	0x8e7402fb,0x8e756fd0,0x8e7609cf,0x8e776fd2,0x8e786fd4,0x8e796fd6,0x8e7a6fd8,0x8e7b6fda,
	0x8e7c0f2b,0x8e7d6fdc,0x8e7e6fde,0x8e7f02ff,0x8e806fe0,0x8e811877,0x8e826fe2,0x8e836fe4,
	0x8e846fe6,0x8e8519ba,0x8e866fe8,0x8e870292,0x8e886fea,0x8e896fec,0x8e8a6fee,0x8e8b6ff0,
	0x8e8c6ff2,0x8e8d6ff4,0x8e8e6ff6,0x8e8f0bc5,0x8e900bac,0x8e916ff8,0x8e926ffa,0x8e936ffc,
	0x8e9401df,0x8e956ffe,0x8e967000,0x8e977002,0x8e987004,0x8e997006,0x8e9a7008,0x8e9b700a,
	0x8e9c1a4e,0x8e9d700c,0x8e9e156b,0x8e9f700e,0x8ea07010,0x8ea17012,0x8ea27014,0x8ea37016,
	0x8ea47018,0x8ea5701a,0x8ea6701c,0x8ea7701e,0x8ea87020,0x8ea97022,0x8eaa7024,0x8eab1165,
	0x8eac05ee,0x8ead7026,0x8eae7028,0x8eaf1033,0x8eb0702a,0x8eb1702c,0x8eb20462,0x8eb3702e,
	0x8eb47030,0x8eb57032,0x8eb67034,0x8eb77036,0x8eb87038,0x8eb9703a,0x8eba12fb,0x8ebb703c,
	0x8ebc703e,0x8ebd7040,0x8ebe7042,0x8ebf7044,0x8ec07046,0x8ec17048,0x8ec2704a,0x8ec3704c,
	0x8ec4704e,0x8ec57050,0x8ec67052,0x8ec77054,0x8ec87056,0x8ec97058,0x8eca705a,0x8ecb705c,
	0x8ecc705e,0x8ecd7060,0x8ece1421,0x8ecf7062,0x8ed07064,0x8ed17066,0x8ed27068,0x8ed3706a,
	0x8ed4706c,0x8ed5706e,0x8ed67070,0x8ed77072,0x8ed87074,0x8ed97076,0x8eda7078,0x8edb707a,
	0x8edc707c,0x8edd707e,0x8ede7080,0x8edf7082,0x8ee07084,0x8ee17086,0x8ee27088,0x8ee3708a,
	0x8ee4708c,0x8ee5708e,0x8ee67090,0x8ee77092,0x8ee87094,0x8ee97096,0x8eea7098,0x8eeb709a,
	0x8eec709c,0x8eed709e,0x8eee70a0,0x8eef70a2,0x8ef070a4,0x8ef170a6,0x8ef270a8,0x8ef370aa,
	0x8ef470ac,0x8ef570ae,0x8ef670b0,0x8ef770b2,0x8ef870b4,0x8ef970b6,0x8efa70b8,0x8efb70ba,
	0x8efc70bc,0x8efd70be,0x8efe70c0,0x8eff70c2,0x8f0070c4,0x8f0170c6,0x8f0270c8,0x8f0370ca,
	0x8f0470cc,0x8f0570ce,0x8f0670d0,0x8f0770d2,0x8f0870d4,0x8f0970d6,0x8f0a70d8,0x8f0b70da,
	0x8f0c70dc,0x8f0d70de,0x8f0e70e0,0x8f0f70e2,0x8f1070e4,0x8f1170e6,0x8f1270e8,0x8f1370ea,
	0x8f1470ec,0x8f1570ee,0x8f1670f0,0x8f1770f2,0x8f1870f4,0x8f1970f6,0x8f1a70f8,0x8f1b70fa,
	0x8f1c70fc,0x8f1d70fe,0x8f1e7100,0x8f1f7102,0x8f207104,0x8f217106,0x8f227108,0x8f23710a,
	0x8f24710c,0x8f25710e,0x8f267110,0x8f277112,0x8f287114,0x8f297116,0x8f2a7118,0x8f2b711a,
	0x8f2c711c,0x8f2d711e,0x8f2e7120,0x8f2f7122,0x8f307124,0x8f317126,0x8f327128,0x8f33712a,
	0x8f34712c,0x8f35712e,0x8f367130,0x8f377132,0x8f387134,0x8f397136,0x8f3a7138,0x8f3b713a,
	0x8f3c713c,0x8f3d713e,0x8f3e7140,0x8f3f7142,0x8f407144,0x8f417146,0x8f427148,0x8f43714a,
	0x8f44714c,0x8f45714e,0x8f467150,0x8f477152,0x8f487154,0x8f497156,0x8f4a7158,0x8f4b715a,
	0x8f4c715c,0x8f4d715e,0x8f4e7160,0x8f4f7162,0x8f507164,0x8f517166,0x8f527168,0x8f53716a,
	0x8f54716c,0x8f55716e,0x8f567170,0x8f577172,0x8f587174,0x8f597176,0x8f5a7178,0x8f5b717a,
	0x8f5c717c,0x8f5d717e,0x8f5e7180,0x8f5f7182,0x8f607184,0x8f617186,0x8f627188,0x8f63718a,
	0x8f64718c,0x8f65718e,0x8f660211,0x8f671898,0x8f680669,0x8f6915cb,0x8f6a7190,0x8f6b1093,
	0x8f6c19db,0x8f6d0479,0x8f6e0c53,0x8f6f10c5,0x8f7006ff,0x8f710612,0x8f720a1c,0x8f730c3e,
	0x8f741996,0x8f751956,0x8f761705,0x8f770724,0x8f78190f,0x8f790b50,0x8f7a168f,0x8f7b0ff5,
	0x8f7c11c3,0x8f7d1857,0x8f7e196d,0x8f7f08de,0x8f807192,0x8f811054,0x8f820c2f,0x8f8308df,
	0x8f8418ec,0x8f85054c,0x8f860b8b,0x8f870df2,0x8f8800b6,0x8f890798,0x8f8a067e,0x8f8b13e8,
	0x8f8c7194,0x8f8d02cf,0x8f8e1a1a,0x8f8f02f0,0x8f90053f,0x8f91080c,0x8f927196,0x8f9311ed,
	0x8f940e90,0x8f95181a,0x8f9614d8,0x8f9718bc,0x8f980c38,0x8f9918f1,0x8f9a0bb7,0x8f9b1570,
	0x8f9c0618,0x8f9d7198,0x8f9e02d8,0x8f9f0ec9,0x8fa0719a,0x8fa1719c,0x8fa2719e,0x8fa30ab3,
	0x8fa471a0,0x8fa571a2,0x8fa671a4,0x8fa771a6,0x8fa80119,0x8fa9011a,0x8faa71a8,0x8fab011b,
	0x8fac71aa,0x8fad71ac,0x8fae71ae,0x8faf71b0,0x8fb00222,0x8fb110bc,0x8fb271b2,0x8fb371b4,
	0x8fb471b6,0x8fb571b8,0x8fb602cd,0x8fb771ba,0x8fb871bc,0x8fb90102,0x8fba71be,0x8fbb71c0,
	0x8fbc71c2,0x8fbd0b8f,0x8fbe0329,0x8fbf71c4,0x8fc071c6,0x8fc10f7c,0x8fc217ae,0x8fc371c8,
	0x8fc40f68,0x8fc51608,0x8fc671ca,0x8fc70692,0x8fc80c95,0x8fc971cc,0x8fca71ce,0x8fcb71d0,
	0x8fcc71d2,0x8fcd71d4,0x8fce1750,0x8fcf71d6,0x8fd01842,0x8fd10927,0x8fd271d8,0x8fd31625,
	0x8fd404b4,0x8fd5146f,0x8fd671da,0x8fd771dc,0x8fd80698,0x8fd918f6,0x8fda71de,0x8fdb0928,
	0x8fdc181d,0x8fdd1401,0x8fde0b65,0x8fdf0254,0x8fe071e0,0x8fe171e2,0x8fe21341,0x8fe371e4,
	0x8fe416dd,0x8fe5095c,0x8fe60840,0x8fe771e6,0x8fe8033b,0x8fe90492,0x8fea0399,0x8feb0f0c,
	0x8fec71e8,0x8fed03de,0x8fee187c,0x8fef71ea,0x8ff01200,0x8ff171ec,0x8ff271ee,0x8ff3094d,
	0x8ff471f0,0x8ff571f2,0x8ff671f4,0x8ff70d09,0x8ff800cb,0x8ff90819,0x8ffa71f6,0x8ffb71f8,
	0x8ffc71fa,0x8ffd19ea,0x8ffe71fc,0x8fff71fe,0x90001394,0x9001125b,0x900211c1,0x90031306,
	0x90040e72,0x9005071b,0x90060de6,0x90077200,0x90087202,0x900915da,0x900a160a,0x900b016d,
	0x900c7204,0x900d1532,0x900e7206,0x900f137b,0x901019b7,0x90111022,0x901203ad,0x90137208,
	0x90141383,0x9015720a,0x9016132b,0x9017041c,0x9018720c,0x9019720e,0x901a1361,0x901b065d,
	0x901c7210,0x901d11c4,0x901e0243,0x901f1274,0x90201874,0x9021106d,0x9022050c,0x90237212,
	0x90247214,0x90257216,0x90260b31,0x90277218,0x9028721a,0x9029721c,0x902a721e,0x902b7220,
	0x902c7222,0x902d0773,0x902e0344,0x902f0c31,0x90307224,0x90317226,0x90327228,0x9033722a,
	0x9034722c,0x90350a84,0x903613f2,0x9037722e,0x9038170d,0x90397230,0x903a7232,0x903b0c59,
	0x903c00ce,0x903d7234,0x903e17cb,0x903f7236,0x90407238,0x90410455,0x9042128f,0x9043723a,
	0x904402ab,0x9045723c,0x9046723e,0x904717fa,0x90487240,0x90497242,0x904a7244,0x904b7246,
	0x904c7248,0x904d0118,0x904e724a,0x904f0482,0x905014d5,0x90510784,0x90521026,0x9053037c,
	0x9054724c,0x9055724e,0x90567250,0x905716d7,0x9058060b,0x90597252,0x905a7254,0x905b0bf2,
	0x905c7256,0x905d7258,0x905e725a,0x905f725c,0x9060725e,0x90617260,0x906212ba,0x90630f96,
	0x90647262,0x90651695,0x90667264,0x90677266,0x90680034,0x90697268,0x906a726a,0x906b726c,
	0x906c726e,0x906d1869,0x906e18e8,0x906f7270,0x90707272,0x90717274,0x90727276,0x90737278,
	0x90740bb6,0x90751a59,0x9076727a,0x9077727c,0x9078727e,0x90797280,0x907a7282,0x907b7284,
	0x907c7286,0x907d09a8,0x907e7288,0x907f00fb,0x9080168a,0x9081728a,0x90821567,0x90831294,
	0x9084728c,0x9085728e,0x90867290,0x90877292,0x90880d3c,0x90897294,0x908a7296,0x908b0aac,
	0x908c7298,0x908d729a,0x908e729c,0x908f729e,0x909072a0,0x909116f9,0x909272a2,0x9093038a,
	0x909472a4,0x9095176e,0x909672a6,0x909706a5,0x909872a8,0x90990cab,0x909a72aa,0x909b100d,
	0x909c72ac,0x909d0a75,0x909e72ae,0x909f72b0,0x90a072b2,0x90a104be,0x90a21585,0x90a30da3,
	0x90a472b4,0x90a572b6,0x90a6007d,0x90a772b8,0x90a872ba,0x90a972bc,0x90aa154c,0x90ab72be,
	0x90ac1451,0x90ad72c0,0x90ae178e,0x90af06a7,0x90b012c0,0x90b11015,0x90b272c2,0x90b30eaa,
	0x90b40144,0x90b5114b,0x90b600ad,0x90b772c4,0x90b803a2,0x90b91a3a,0x90ba16b1,0x90bb0baf,
	0x90bc72c6,0x90bd72c8,0x90be19a8,0x90bf72ca,0x90c072cc,0x90c117eb,0x90c272ce,0x90c372d0,
	0x90c40fdc,0x90c51964,0x90c672d2,0x90c70760,0x90c872d4,0x90c972d6,0x90ca08bb,0x90cb72d8,
	0x90cc72da,0x90cd72dc,0x90ce0ad9,0x90cf084d,0x90d00a66,0x90d1192d,0x90d272de,0x90d31843,
	0x90d472e0,0x90d572e2,0x90d672e4,0x90d7148e,0x90d872e6,0x90d972e8,0x90da72ea,0x90db0530,
	0x90dc05b0,0x90dd06ce,0x90de72ec,0x90df72ee,0x90e072f0,0x90e109dc,0x90e21761,0x90e372f2,
	0x90e472f4,0x90e572f6,0x90e60b51,0x90e7183b,0x90e8017b,0x90e972f8,0x90ea72fa,0x90eb0eb9,
	0x90ec72fc,0x90ed0684,0x90ee72fe,0x90ef12d7,0x90f07300,0x90f17302,0x90f27304,0x90f37306,
	0x90f4021b,0x90f57308,0x90f6730a,0x90f7730c,0x90f8034d,0x90f9730e,0x90fa7310,0x90fb7312,
	0x90fc7314,0x90fd0412,0x90fe1653,0x90ff7316,0x91007318,0x9101731a,0x9102047e,0x9103731c,
	0x910409b8,0x9105731e,0x91067320,0x91077322,0x91087324,0x91097326,0x910a7328,0x910b732a,
	0x910c732c,0x910d732e,0x910e7330,0x910f7332,0x91107334,0x91117336,0x91127338,0x9113733a,
	0x9114733c,0x9115733e,0x91167340,0x91177342,0x91187344,0x911900da,0x911a7346,0x911b7348,
	0x911c734a,0x911d734c,0x911e1733,0x911f734e,0x91207350,0x91217352,0x91221636,0x912318c5,
	0x91247354,0x91257356,0x91267358,0x9127735a,0x9128735c,0x9129735e,0x912a7360,0x912b7362,
	0x912c7364,0x912d7366,0x912e7368,0x912f1129,0x9130736a,0x91310f07,0x9132736c,0x9133736e,
	0x91347370,0x91357372,0x91367374,0x91377376,0x91387378,0x91391a3e,0x913a737a,0x913b737c,
	0x913c737e,0x913d7380,0x913e7382,0x913f7384,0x91407386,0x91417388,0x9142738a,0x91430bdb,
	0x9144738c,0x9145738e,0x9146050a,0x91477390,0x91487392,0x9149179c,0x914a03f3,0x914b1021,
	0x914c1a04,0x914d0e8f,0x914e19a1,0x914f16de,0x91500585,0x91517394,0x9152096c,0x91537396,
	0x91547398,0x9155739a,0x9156739c,0x915715c4,0x9158739e,0x915973a0,0x915a04ef,0x915b73a2,
	0x915c73a4,0x915d1846,0x915e12cd,0x915f73a6,0x916073a8,0x916113ad,0x916202f6,0x916306a2,
	0x91640619,0x9165126c,0x916673aa,0x916773ac,0x916873ae,0x91690d61,0x916a0af6,0x916b73b0,
	0x916c0281,0x916d73b2,0x916e136c,0x916f1959,0x917014e8,0x917108b7,0x9172023f,0x917373b4,
	0x91741385,0x917508e2,0x91760cdd,0x91770a5b,0x91781281,0x91790b0f,0x917a73b6,0x917b73b8,
	0x917c73ba,0x917d1667,0x917e1197,0x917f0df9,0x918073bc,0x918173be,0x918273c0,0x918373c2,
	0x918473c4,0x91850e85,0x918673c6,0x918702c8,0x918873c8,0x91891a57,0x918a73ca,0x918b02f8,
	0x918c0a9f,0x918d1324,0x918e73cc,0x918f73ce,0x91900737,0x919115ba,0x9192158b,0x919373d0,
	0x919473d2,0x919573d4,0x919673d6,0x919773d8,0x919873da,0x919973dc,0x919a0d0c,0x919b1059,
	0x919c73de,0x919d73e0,0x919e73e2,0x919f73e4,0x91a073e6,0x91a173e8,0x91a2069b,0x91a312f5,
	0x91a473ea,0x91a573ec,0x91a673ee,0x91a773f0,0x91a873f2,0x91a973f4,0x91aa0aec,0x91ab73f6,
	0x91ac73f8,0x91ad0170,0x91ae08e4,0x91af14b0,0x91b073fa,0x91b173fc,0x91b273fe,0x91b37400,
	0x91b40b36,0x91b509a9,0x91b67402,0x91b77404,0x91b87406,0x91b97408,0x91ba15f8,0x91bb740a,
	0x91bc740c,0x91bd740e,0x91be7410,0x91bf7412,0x91c07414,0x91c17416,0x91c27418,0x91c3741a,
	0x91c4741c,0x91c5741e,0x91c67420,0x91c70187,0x91c87422,0x91c917ab,0x91ca11c8,0x91cb7424,
	0x91cc0b2e,0x91cd198d,0x91ce16ac,0x91cf0b86,0x91d07426,0x91d10913,0x91d27428,0x91d3742a,
	0x91d4742c,0x91d5742e,0x91d67430,0x91d77432,0x91d87434,0x91d97436,0x91da7438,0x91db743a,
	0x91dc054a,0x91dd743c,0x91de743e,0x91df7440,0x91e07442,0x91e17444,0x91e27446,0x91e37448,
	0x91e4744a,0x91e5744c,0x91e6744e,0x91e77450,0x91e87452,0x91e97454,0x91ea7456,0x91eb7458,
	0x91ec745a,0x91ed745c,0x91ee745e,0x91ef7460,0x91f07462,0x91f17464,0x91f27466,0x91f37468,
	0x91f4746a,0x91f5746c,0x91f6746e,0x91f77470,0x91f87472,0x91f97474,0x91fa7476,0x91fb7478,
	0x91fc747a,0x91fd747c,0x91fe747e,0x91ff7480,0x92007482,0x92017484,0x92027486,0x92037488,
	0x9204748a,0x9205748c,0x9206748e,0x92077490,0x92087492,0x92097494,0x920a7496,0x920b7498,
	0x920c749a,0x920d749c,0x920e749e,0x920f74a0,0x921074a2,0x921174a4,0x921274a6,0x921374a8,
	0x921474aa,0x921574ac,0x921674ae,0x921774b0,0x921874b2,0x921974b4,0x921a74b6,0x921b74b8,
	0x921c74ba,0x921d74bc,0x921e74be,0x921f74c0,0x922074c2,0x922174c4,0x922274c6,0x922374c8,
	0x922474ca,0x922574cc,0x922674ce,0x922774d0,0x922874d2,0x922974d4,0x922a74d6,0x922b74d8,
	0x922c74da,0x922d74dc,0x922e74de,0x922f74e0,0x923074e2,0x923174e4,0x923274e6,0x923374e8,
	0x923474ea,0x923574ec,0x923674ee,0x923774f0,0x923874f2,0x923974f4,0x923a74f6,0x923b74f8,
	0x923c74fa,0x923d74fc,0x923e74fe,0x923f7500,0x92407502,0x92417504,0x92427506,0x92437508,
	0x9244750a,0x9245750c,0x9246750e,0x92477510,0x92487512,0x92497514,0x924a7516,0x924b7518,
	0x924c751a,0x924d751c,0x924e751e,0x924f7520,0x92507522,0x92517524,0x92527526,0x92537528,
	0x9254752a,0x9255752c,0x9256752e,0x92577530,0x92587532,0x92597534,0x925a7536,0x925b7538,
	0x925c753a,0x925d753c,0x925e753e,0x925f7540,0x92607542,0x92617544,0x92627546,0x92637548,
	0x9264754a,0x9265754c,0x9266754e,0x92677550,0x92687552,0x92697554,0x926a7556,0x926b7558,
	0x926c755a,0x926d755c,0x926e755e,0x926f7560,0x92707562,0x92717564,0x92727566,0x92737568,
	0x9274089f,0x9275756a,0x9276756c,0x9277756e,0x92787570,0x92797572,0x927a7574,0x927b7576,
	0x927c7578,0x927d757a,0x927e757c,0x927f757e,0x92807580,0x92817582,0x92827584,0x92837586,
	0x92847588,0x9285758a,0x9286758c,0x9287758e,0x92887590,0x92897592,0x928a7594,0x928b7596,
	0x928c7598,0x928d759a,0x928e100c,0x928f759c,0x9290759e,0x929175a0,0x929275a2,0x929375a4,
	0x929475a6,0x929575a8,0x929675aa,0x929775ac,0x929875ae,0x929975b0,0x929a75b2,0x929b75b4,
	0x929c75b6,0x929d75b8,0x929e75ba,0x929f75bc,0x92a075be,0x92a175c0,0x92a275c2,0x92a375c4,
	0x92a475c6,0x92a575c8,0x92a675ca,0x92a775cc,0x92a875ce,0x92a975d0,0x92aa75d2,0x92ab75d4,
	0x92ac75d6,0x92ad75d8,0x92ae0c47,0x92af75da,0x92b075dc,0x92b175de,0x92b275e0,0x92b375e2,
	0x92b475e4,0x92b575e6,0x92b675e8,0x92b775ea,0x92b875ec,0x92b975ee,0x92ba75f0,0x92bb75f2,
	0x92bc75f4,0x92bd75f6,0x92be75f8,0x92bf75fa,0x92c075fc,0x92c175fe,0x92c27600,0x92c37602,
	0x92c47604,0x92c57606,0x92c67608,0x92c7760a,0x92c81482,0x92c9760c,0x92ca760e,0x92cb7610,
	0x92cc7612,0x92cd7614,0x92ce7616,0x92cf7618,0x92d0761a,0x92d1761c,0x92d2761e,0x92d37620,
	0x92d47622,0x92d57624,0x92d67626,0x92d77628,0x92d8762a,0x92d9762c,0x92da762e,0x92db7630,
	0x92dc7632,0x92dd7634,0x92de7636,0x92df7638,0x92e0763a,0x92e1763c,0x92e2763e,0x92e37640,
	0x92e47642,0x92e57644,0x92e67646,0x92e77648,0x92e8764a,0x92e9764c,0x92ea764e,0x92eb7650,
	0x92ec7652,0x92ed7654,0x92ee7656,0x92ef7658,0x92f0765a,0x92f1765c,0x92f2765e,0x92f37660,
	0x92f47662,0x92f57664,0x92f67666,0x92f77668,0x92f8766a,0x92f9766c,0x92fa766e,0x92fb7670,
	0x92fc7672,0x92fd7674,0x92fe7676,0x92ff7678,0x9300767a,0x9301767c,0x9302767e,0x93037680,
	0x93047682,0x93057684,0x93067686,0x93077688,0x9308768a,0x9309768c,0x930a768e,0x930b7690,
	0x930c7692,0x930d7694,0x930e7696,0x930f7698,0x9310769a,0x9311769c,0x9312769e,0x931376a0,
	0x931476a2,0x931576a4,0x931676a6,0x931776a8,0x931876aa,0x931976ac,0x931a76ae,0x931b76b0,
	0x931c76b2,0x931d76b4,0x931e76b6,0x931f76b8,0x932076ba,0x932176bc,0x932276be,0x932376c0,
	0x932476c2,0x932576c4,0x932676c6,0x932776c8,0x932876ca,0x932976cc,0x932a76ce,0x932b76d0,
	0x932c76d2,0x932d76d4,0x932e76d6,0x932f76d8,0x933076da,0x933176dc,0x933276de,0x933376e0,
	0x933476e2,0x933576e4,0x933676e6,0x933776e8,0x933876ea,0x933976ec,0x933a76ee,0x933b76f0,
	0x933c76f2,0x933d76f4,0x933e1862,0x933f76f6,0x934076f8,0x934176fa,0x934276fc,0x934376fe,
	0x93447700,0x93457702,0x93467704,0x93477706,0x93487708,0x9349770a,0x934a770c,0x934b770e,
	0x934c7710,0x934d7712,0x934e7714,0x934f7716,0x93507718,0x9351771a,0x9352771c,0x9353771e,
	0x93547720,0x93557722,0x93567724,0x93577726,0x93587728,0x9359772a,0x935a772c,0x935b772e,
	0x935c7730,0x935d7732,0x935e7734,0x935f7736,0x93607738,0x9361773a,0x9362773c,0x9363773e,
	0x93647740,0x93657742,0x93667744,0x93677746,0x93687748,0x9369774a,0x936a0d88,0x936b774c,
	0x936c774e,0x936d7750,0x936e7752,0x936f7754,0x93707756,0x93717758,0x9372775a,0x9373775c,
	0x9374775e,0x93757760,0x93767762,0x93777764,0x93787766,0x93797768,0x937a776a,0x937b776c,
	0x937c776e,0x937d7770,0x937e7772,0x937f7774,0x93807776,0x93817778,0x9382777a,0x9383777c,
	0x9384777e,0x93857780,0x93867782,0x93877784,0x93887786,0x93897788,0x938a778a,0x938b778c,
	0x938c778e,0x938d7790,0x938e7792,0x938f0bee,0x93907794,0x93917796,0x93927798,0x9393779a,
	0x9394779c,0x9395779e,0x939677a0,0x939777a2,0x939877a4,0x939977a6,0x939a77a8,0x939b77aa,
	0x939c77ac,0x939d77ae,0x939e77b0,0x939f77b2,0x93a077b4,0x93a177b6,0x93a277b8,0x93a377ba,
	0x93a477bc,0x93a577be,0x93a677c0,0x93a777c2,0x93a877c4,0x93a977c6,0x93aa77c8,0x93ab77ca,
	0x93ac77cc,0x93ad77ce,0x93ae77d0,0x93af77d2,0x93b077d4,0x93b177d6,0x93b277d8,0x93b377da,
	0x93b477dc,0x93b577de,0x93b677e0,0x93b777e2,0x93b877e4,0x93b977e6,0x93ba77e8,0x93bb77ea,
	0x93bc77ec,0x93bd77ee,0x93be77f0,0x93bf77f2,0x93c077f4,0x93c177f6,0x93c277f8,0x93c377fa,
	0x93c477fc,0x93c577fe,0x93c67800,0x93c77802,0x93c87804,0x93c97806,0x93ca0044,0x93cb7808,
	0x93cc780a,0x93cd780c,0x93ce780e,0x93cf7810,0x93d07812,0x93d17814,0x93d27816,0x93d37818,
	0x93d4781a,0x93d5781c,0x93d60039,0x93d7781e,0x93d87820,0x93d97822,0x93da7824,0x93db7826,
	0x93dc7828,0x93dd782a,0x93de782c,0x93df782e,0x93e07830,0x93e17832,0x93e27834,0x93e37836,
	0x93e47838,0x93e5783a,0x93e6783c,0x93e7783e,0x93e87840,0x93e97842,0x93ea7844,0x93eb7846,
	0x93ec7848,0x93ed784a,0x93ee784c,0x93ef784e,0x93f07850,0x93f17852,0x93f27854,0x93f37856,
	0x93f47858,0x93f5785a,0x93f6785c,0x93f7785e,0x93f87860,0x93f97862,0x93fa7864,0x93fb7866,
	0x93fc7868,0x93fd786a,0x93fe786c,0x93ff786e,0x94007870,0x94017872,0x94027874,0x94037876,
	0x94047878,0x9405787a,0x9406787c,0x9407787e,0x94087880,0x94097882,0x940a7884,0x940b7886,
	0x940c7888,0x940d788a,0x940e788c,0x940f788e,0x94107890,0x94117892,0x94127894,0x94137896,
	0x94147898,0x9415789a,0x9416789c,0x9417789e,0x941878a0,0x941978a2,0x941a78a4,0x941b78a6,
	0x941c78a8,0x941d78aa,0x941e78ac,0x941f78ae,0x942078b0,0x942178b2,0x942278b4,0x942378b6,
	0x942478b8,0x942578ba,0x942678bc,0x942778be,0x942878c0,0x942978c2,0x942a78c4,0x942b78c6,
	0x942c78c8,0x942d78ca,0x942e78cc,0x942f78ce,0x943078d0,0x943178d2,0x943278d4,0x943378d6,
	0x943478d8,0x943578da,0x943678dc,0x943778de,0x943878e0,0x943978e2,0x943a78e4,0x943b78e6,
	0x943c78e8,0x943d78ea,0x943e00bb,0x943f78ec,0x944078ee,0x944178f0,0x944278f2,0x944378f4,
	0x944478f6,0x944578f8,0x944678fa,0x944778fc,0x944878fe,0x94497900,0x944a7902,0x944b7904,
	0x944c7906,0x944d7908,0x944e790a,0x944f790c,0x9450790e,0x94517910,0x94527912,0x94537914,
	0x94547916,0x94557918,0x9456791a,0x9457791c,0x9458791e,0x94597920,0x945a7922,0x945b7924,
	0x945c7926,0x945d7928,0x945e792a,0x945f792c,0x9460792e,0x94617930,0x94627932,0x94637934,
	0x94647936,0x94657938,0x9466793a,0x9467793c,0x9468793e,0x94697940,0x946a7942,0x946b1578,
	0x946c7944,0x946d7946,0x946e7948,0x946f794a,0x9470794c,0x9471794e,0x94727950,0x94737952,
	0x94747954,0x94757956,0x94767958,0x9477795a,0x9478795c,0x9479795e,0x947a7960,0x947b7962,
	0x947c7964,0x947d7966,0x947e7968,0x947f796a,0x9480796c,0x9481796e,0x94827970,0x94837972,
	0x94847974,0x94850912,0x9486056b,0x948716e2,0x948818fd,0x948903f1,0x948a18da,0x948b0f02,
	0x948c0b9a,0x948d1388,0x948e0f7f,0x948f02b0,0x94901119,0x94917976,0x949204ab,0x949303d7,
	0x94940ced,0x94950e3a,0x94967978,0x949701c8,0x9498797a,0x94990576,0x949a017a,0x949b12cb,
	0x949c099a,0x949d0453,0x949e0207,0x949f1982,0x94a00da8,0x94a100b0,0x94a20598,0x94a30074,
	0x94a40f8b,0x94a516a3,0x94a60fdf,0x94a709d6,0x94a81456,0x94a905fb,0x94aa0a0d,0x94ab04c2,
	0x94ac07ca,0x94ad0418,0x94ae0e1e,0x94af0056,0x94b017f0,0x94b10f8d,0x94b21921,0x94b30f8e,
	0x94b40626,0x94b50151,0x94b60a1e,0x94b70f0a,0x94b8016e,0x94b9015c,0x94ba182d,0x94bb1a4f,
	0x94bc0d97,0x94bd12e0,0x94be0858,0x94bf03c7,0x94c01793,0x94c1134d,0x94c2015d,0x94c30bcf,
	0x94c41227,0x94c50f82,0x94c60cc4,0x94c7797c,0x94c811c5,0x94c915e0,0x94ca12b1,0x94cb00e7,
	0x94cc0ddd,0x94cd0eb2,0x94ce045c,0x94cf797e,0x94d00a13,0x94d10af1,0x94d20496,0x94d37980,
	0x94d47982,0x94d5179e,0x94d6023b,0x94d70851,0x94d816a9,0x94d90dc6,0x94da7984,0x94db0361,
	0x94dc136a,0x94dd0c75,0x94de03da,0x94df172a,0x94e009f0,0x94e1189a,0x94e219af,0x94e314bd,
	0x94e4135f,0x94e503fe,0x94e67986,0x94e70750,0x94e81056,0x94e91104,0x94ea0693,0x94eb03db,
	0x94ec05d1,0x94ed0d5c,0x94ee1923,0x94ef10f9,0x94f008d6,0x94f116c1,0x94f201e3,0x94f30275,
	0x94f412fc,0x94f50022,0x94f61732,0x94f710b1,0x94f819d2,0x94f90aeb,0x94fa0f18,0x94fb7988,
	0x94fc0ab9,0x94fd1312,0x94fe0b7b,0x94ff0a3f,0x95001535,0x950112ab,0x95020b33,0x9503188d,
	0x9504028e,0x95050687,0x950605b1,0x95070470,0x950815a9,0x95090322,0x950a0c4c,0x950b0508,
	0x950c1573,0x950d0bf1,0x950e09eb,0x950f0885,0x951010cb,0x9511131b,0x95120adf,0x95130fee,
	0x95140982,0x95150002,0x95160faa,0x951718f3,0x95180e36,0x95190323,0x951a0cbb,0x951b00c0,
	0x951c798a,0x951d0380,0x951e0a37,0x951f0a9d,0x9520798c,0x952114a3,0x95220635,0x95230c5b,
	0x952402bf,0x952519ed,0x9526091f,0x9527798e,0x952814e9,0x952909b0,0x952a07cc,0x952b0e89,
	0x952c12d9,0x952d03fc,0x952e08a0,0x952f09a3,0x95300cfe,0x95311a1d,0x95320fdb,0x95337990,
	0x953409f4,0x95350fab,0x95361238,0x95370484,0x953801b6,0x95390fba,0x953a1985,0x953b043d,
	0x953c1262,0x953d7992,0x953e0764,0x953f0008,0x95400434,0x95410ce4,0x95420c15,0x95437994,
	0x954404e8,0x95450cde,0x95460d7e,0x9547191a,0x95487996,0x954905c9,0x954a0e07,0x954b7998,
	0x954c09ae,0x954d0e08,0x954e0da1,0x954f0bed,0x955005ac,0x9551008a,0x95521717,0x9553084b,
	0x95540137,0x9555799a,0x95560123,0x955712e8,0x95580caa,0x95590c5e,0x955a799c,0x955b1774,
	0x955c0959,0x955d0396,0x955e1a48,0x955f15e4,0x9560799e,0x95611579,0x956209ce,0x95630b9d,
	0x95640f21,0x95650c29,0x95660446,0x95670acd,0x95680f2a,0x956902fe,0x956a0fb1,0x956b038f,
	0x956c07d3,0x956d0b02,0x956e79a0,0x956f1a0a,0x95700b70,0x9571171f,0x957201c0,0x95730126,
	0x957479a2,0x957579a4,0x95761518,0x957779a6,0x957879a8,0x957979aa,0x957a79ac,0x957b79ae,
	0x957c79b0,0x957d79b2,0x957e79b4,0x957f01f2,0x958079b6,0x958179b8,0x958279ba,0x958379bc,
	0x958479be,0x958579c0,0x958679c2,0x958779c4,0x958879c6,0x958979c8,0x958a79ca,0x958b79cc,
	0x958c79ce,0x958d79d0,0x958e79d2,0x958f79d4,0x959079d6,0x959179d8,0x959279da,0x959379dc,
	0x959479de,0x959579e0,0x959679e2,0x959779e4,0x959879e6,0x959979e8,0x959a79ea,0x959b79ec,
	0x959c79ee,0x959d79f0,0x959e79f2,0x959f79f4,0x95a079f6,0x95a179f8,0x95a279fa,0x95a379fc,
	0x95a479fe,0x95a57a00,0x95a67a02,0x95a77a04,0x95a87a06,0x95a97a08,0x95aa7a0a,0x95ab7a0c,
	0x95ac7a0e,0x95ad7a10,0x95ae7a12,0x95af7a14,0x95b07a16,0x95b17a18,0x95b27a1a,0x95b37a1c,
	0x95b47a1e,0x95b57a20,0x95b67a22,0x95b77a24,0x95b87a26,0x95b97a28,0x95ba7a2a,0x95bb7a2c,
	0x95bc7a2e,0x95bd7a30,0x95be7a32,0x95bf7a34,0x95c07a36,0x95c17a38,0x95c27a3a,0x95c37a3c,
	0x95c47a3e,0x95c57a40,0x95c67a42,0x95c77a44,0x95c87a46,0x95c97a48,0x95ca7a4a,0x95cb7a4c,
	0x95cc7a4e,0x95cd7a50,0x95ce7a52,0x95cf7a54,0x95d07a56,0x95d17a58,0x95d27a5a,0x95d37a5c,
	0x95d47a5e,0x95d57a60,0x95d67a62,0x95d77a64,0x95d87a66,0x95d97a68,0x95da7a6a,0x95db7a6c,
	0x95dc7a6e,0x95dd7a70,0x95de7a72,0x95df7a74,0x95e07a76,0x95e17a78,0x95e27a7a,0x95e37a7c,
	0x95e47a7e,0x95e57a80,0x95e67a82,0x95e77a84,0x95e80ceb,0x95e91212,0x95ea1121,0x95eb163b,
	0x95ec7a86,0x95ed00de,0x95ee1436,0x95ef02b5,0x95f010ce,0x95f11402,0x95f214ec,0x95f30707,
	0x95f40865,0x95f50d4f,0x95f60a0b,0x95f70cef,0x95f81899,0x95f90dcd,0x95fa0664,0x95fb142e,
	0x95fc12b9,0x95fd0d52,0x95fe0c6f,0x95ff7a88,0x9600049f,0x960105be,0x960206e1,0x96030aa3,
	0x96040963,0x9605182e,0x96060ae3,0x96077a8a,0x960817f5,0x96091633,0x960a01f0,0x960b14c7,
	0x960c1430,0x960d07bb,0x960e1645,0x960f047f,0x961001e4,0x96110ac6,0x96121049,0x96137a8c,
	0x96140aa8,0x96151069,0x961606ea,0x96171337,0x96187a8e,0x9619106a,0x961a0a02,0x961b7a90,
	0x961c0559,0x961d0551,0x961e7a92,0x961f0440,0x96207a94,0x96210f79,0x96221474,0x96237a96,
	0x96247a98,0x96257a9a,0x96267a9c,0x96277a9e,0x96287aa0,0x96297aa2,0x962a0070,0x962b7aa4,
	0x962c7aa6,0x962d7aa8,0x962e10c3,0x962f7aaa,0x96307aac,0x96310942,0x963204c3,0x96331674,
	0x96341722,0x96351915,0x963608e5,0x96377aae,0x96387ab0,0x96397ab2,0x963a7ab4,0x963b1a4a,
	0x963c1a63,0x963d03c2,0x963e7ab6,0x963f0000,0x964013a6,0x96417ab8,0x964200a3,0x96437aba,
	0x96440557,0x96450823,0x96460c2c,0x96470c03,0x96480223,0x96491587,0x964a7abc,0x964b0c12,
	0x964c0d75,0x964d08b4,0x964e7abe,0x964f7ac0,0x96501505,0x96517ac2,0x96527ac4,0x96537ac6,
	0x96540571,0x96551122,0x96567ac8,0x96577aca,0x96587acc,0x96597ace,0x965a7ad0,0x965b00e4,
	0x965c7ad2,0x965d7ad4,0x965e7ad6,0x965f1967,0x96607ad8,0x96610419,0x96621820,0x96637ada,
	0x9664028b,0x96657adc,0x96667ade,0x96670e01,0x9668183f,0x966914f9,0x966a0e86,0x966b7ae0,
	0x966c1a3d,0x966d7ae2,0x966e7ae4,0x966f7ae6,0x96707ae8,0x96717aea,0x967202bb,0x96737aec,
	0x96740eb1,0x96750bd0,0x96761308,0x96771507,0x96787aee,0x96797af0,0x967a7af2,0x967b7af4,
	0x967c7af6,0x967d7af8,0x967e7afa,0x967f7afc,0x96807afe,0x96817b00,0x96827b02,0x96837b04,
	0x96847b06,0x968517c5,0x96860c00,0x96877b08,0x968813f3,0x96897b0a,0x968a7b0c,0x968b1289,
	0x968c7b0e,0x968d077f,0x968e7b10,0x968f128a,0x9690173d,0x96917b12,0x96927b14,0x96937b16,
	0x969405c4,0x96957b18,0x96967b1a,0x96970a8e,0x96980013,0x969914c9,0x969a7b1c,0x969b7b1e,
	0x969c18d6,0x969d7b20,0x969e7b22,0x969f7b24,0x96a07b26,0x96a17b28,0x96a27b2a,0x96a37b2c,
	0x96a47b2e,0x96a57b30,0x96a67b32,0x96a71291,0x96a87b34,0x96a97b36,0x96aa7b38,0x96ab7b3a,
	0x96ac7b3c,0x96ad7b3e,0x96ae7b40,0x96af7b42,0x96b014b8,0x96b17b44,0x96b27b46,0x96b3079b,
	0x96b47b48,0x96b57b4a,0x96b60b4a,0x96b77b4c,0x96b87b4e,0x96b919e9,0x96ba7b50,0x96bb7b52,
	0x96bc129b,0x96bd09b6,0x96be0db7,0x96bf7b54,0x96c01067,0x96c11665,0x96c27b56,0x96c37b58,
	0x96c41599,0x96c51622,0x96c60809,0x96c70633,0x96c87b5a,0x96c9197a,0x96ca7b5c,0x96cb7b5e,
	0x96cc02da,0x96cd1770,0x96ce0984,0x96cf0290,0x96d07b60,0x96d17b62,0x96d20c6d,0x96d37b64,
	0x96d47b66,0x96d503d4,0x96d67b68,0x96d77b6a,0x96d87b6c,0x96d97b6e,0x96da7b70,0x96db7b72,
	0x96dc7b74,0x96dd7b76,0x96de7b78,0x96df7b7a,0x96e00283,0x96e17b7c,0x96e27b7e,0x96e37b80,
	0x96e47b82,0x96e57b84,0x96e67b86,0x96e77b88,0x96e817d9,0x96e917c6,0x96ea15ee,0x96eb7b8a,
	0x96ec7b8c,0x96ed7b8e,0x96ee7b90,0x96ef1431,0x96f07b92,0x96f17b94,0x96f27b96,0x96f30b60,
	0x96f47b98,0x96f57b9a,0x96f60bd8,0x96f70afe,0x96f87b9c,0x96f90092,0x96fa7b9e,0x96fb7ba0,
	0x96fc7ba2,0x96fd7ba4,0x96fe147f,0x96ff7ba6,0x970015b2,0x97010834,0x97027ba8,0x97037baa,
	0x97041538,0x97057bac,0x9706135c,0x9707191b,0x97080e91,0x97090ce0,0x970a7bae,0x970b7bb0,
	0x970c7bb2,0x970d07d2,0x970e110e,0x970f04d5,0x97107bb4,0x97117bb6,0x97127bb8,0x97130ddf,
	0x97147bba,0x97157bbc,0x97160bb8,0x97177bbe,0x97187bc0,0x97197bc2,0x971a7bc4,0x971b7bc6,
	0x971c1217,0x971d7bc8,0x971e14d9,0x971f7bca,0x97207bcc,0x97217bce,0x97227bd0,0x97237bd2,
	0x97247bd4,0x97257bd6,0x97267bd8,0x97277bda,0x97287bdc,0x97297bde,0x972a1736,0x972b7be0,
	0x972c7be2,0x972d000f,0x972e7be4,0x972f7be6,0x9730150c,0x97317be8,0x97320c2b,0x97337bea,
	0x97347bec,0x97357bee,0x97367bf0,0x97377bf2,0x9738005c,0x97390eb0,0x973a7bf4,0x973b7bf6,
	0x973c7bf8,0x973d7bfa,0x973e0c91,0x973f7bfc,0x97407bfe,0x97417c00,0x97427c02,0x97437c04,
	0x97447c06,0x97457c08,0x97467c0a,0x97477c0c,0x97487c0e,0x97497c10,0x974a7c12,0x974b7c14,
	0x974c7c16,0x974d7c18,0x974e7c1a,0x974f7c1c,0x97507c1e,0x97517c20,0x97520ff3,0x97530954,
	0x97547c22,0x97557c24,0x97560955,0x97577c26,0x97587c28,0x97590958,0x975a7c2a,0x975b03cc,
	0x975c7c2c,0x975d7c2e,0x975e04cf,0x975f7c30,0x97600a15,0x97610d10,0x97620d2f,0x97637c32,
	0x97647c34,0x976516b9,0x97667c36,0x97677c38,0x97687c3a,0x976905bf,0x976a7c3c,0x976b7c3e,
	0x976c7c40,0x976d7c42,0x976e7c44,0x976f7c46,0x97707c48,0x97717c4a,0x97727c4c,0x9773092f,
	0x977415e6,0x97757c4e,0x97760057,0x97777c50,0x97787c52,0x97797c54,0x977a7c56,0x977b7c58,
	0x977c032e,0x977d7c5a,0x977e7c5c,0x977f7c5e,0x97807c60,0x97817c62,0x97827c64,0x97837c66,
	0x97847c68,0x97851671,0x97867c6a,0x97877c6c,0x97887c6e,0x97897c70,0x978a7c72,0x978b1556,
	0x978c7c74,0x978d001f,0x978e7c76,0x978f7c78,0x97907c7a,0x9791032f,0x97920fc4,0x97937c7c,
	0x97940c9d,0x97957c7e,0x97967c80,0x97977c82,0x97980fd0,0x97997c84,0x979a7c86,0x979b7c88,
	0x979c7c8a,0x979d7c8c,0x979e7c8e,0x979f7c90,0x97a00985,0x97a17c92,0x97a27c94,0x97a310ad,
	0x97a47c96,0x97a57c98,0x97a67c9a,0x97a77c9c,0x97a87c9e,0x97a97ca0,0x97aa7ca2,0x97ab0986,
	0x97ac7ca4,0x97ad0109,0x97ae7ca6,0x97af0875,0x97b07ca8,0x97b17caa,0x97b205fe,0x97b37cac,
	0x97b400ba,0x97b57cae,0x97b67cb0,0x97b77cb2,0x97b87cb4,0x97b97cb6,0x97ba7cb8,0x97bb7cba,
	0x97bc7cbc,0x97bd7cbe,0x97be7cc0,0x97bf7cc2,0x97c07cc4,0x97c17cc6,0x97c27cc8,0x97c37cca,
	0x97c47ccc,0x97c57cce,0x97c67cd0,0x97c77cd2,0x97c87cd4,0x97c97cd6,0x97ca7cd8,0x97cb7cda,
	0x97cc7cdc,0x97cd7cde,0x97ce7ce0,0x97cf7ce2,0x97d07ce4,0x97d17ce6,0x97d27ce8,0x97d37cea,
	0x97d47cec,0x97d57cee,0x97d67cf0,0x97d77cf2,0x97d87cf4,0x97d97cf6,0x97da7cf8,0x97db7cfa,
	0x97dc7cfc,0x97dd7cfe,0x97de7d00,0x97df7d02,0x97e07d04,0x97e17d06,0x97e27d08,0x97e37d0a,
	0x97e47d0c,0x97e57d0e,0x97e613fc,0x97e71094,0x97e87d10,0x97e906ad,0x97ea1419,0x97eb1848,
	0x97ec1303,0x97ed096b,0x97ee7d12,0x97ef7d14,0x97f07d16,0x97f17d18,0x97f27d1a,0x97f31727,
	0x97f47d1c,0x97f51849,0x97f61148,0x97f77d1e,0x97f87d20,0x97f97d22,0x97fa7d24,0x97fb7d26,
	0x97fc7d28,0x97fd7d2a,0x97fe7d2c,0x97ff7d2e,0x98007d30,0x98017d32,0x98027d34,0x98037d36,
	0x98047d38,0x98057d3a,0x98067d3c,0x98077d3e,0x98087d40,0x98097d42,0x980a7d44,0x980b7d46,
	0x980c7d48,0x980d7d4a,0x980e7d4c,0x980f7d4e,0x98107d50,0x98117d52,0x98127d54,0x98137d56,
	0x98147d58,0x98157d5a,0x98167d5c,0x98177d5e,0x98187d60,0x98197d62,0x981a7d64,0x981b7d66,
	0x981c7d68,0x981d7d6a,0x981e7d6c,0x981f7d6e,0x98207d70,0x98217d72,0x98227d74,0x98237d76,
	0x98247d78,0x98257d7a,0x98267d7c,0x98277d7e,0x98287d80,0x98297d82,0x982a7d84,0x982b7d86,
	0x982c7d88,0x982d7d8a,0x982e7d8c,0x982f7d8e,0x98307d90,0x98317d92,0x98327d94,0x98337d96,
	0x98347d98,0x98357d9a,0x98367d9c,0x98377d9e,0x98387da0,0x98397da2,0x983a7da4,0x983b7da6,
	0x983c7da8,0x983d7daa,0x983e7dac,0x983f7dae,0x98407db0,0x98417db2,0x98427db4,0x98437db6,
	0x98447db8,0x98457dba,0x98467dbc,0x98477dbe,0x98487dc0,0x98497dc2,0x984a7dc4,0x984b7dc6,
	0x984c7dc8,0x984d7dca,0x984e7dcc,0x984f7dce,0x98507dd0,0x98517dd2,0x98527dd4,0x98537dd6,
	0x98547dd8,0x98557dda,0x98567ddc,0x98577dde,0x98587de0,0x98597de2,0x985a7de4,0x985b7de6,
	0x985c7de8,0x985d7dea,0x985e7dec,0x985f7dee,0x98607df0,0x98617df2,0x98627df4,0x98637df6,
	0x98647df8,0x98657dfa,0x98667dfc,0x98677dfe,0x98687e00,0x98697e02,0x986a7e04,0x986b7e06,
	0x986c7e08,0x986d7e0a,0x986e7e0c,0x986f7e0e,0x98707e10,0x98717e12,0x98727e14,0x98737e16,
	0x98747e18,0x987516b0,0x987603f5,0x98771003,0x987806a0,0x98791525,0x987a1220,0x987b15af,
	0x987c15bb,0x987d13ce,0x987e062f,0x987f0454,0x98800f49,0x9881006b,0x9882125c,0x988306c1,
	0x988417f1,0x98850c22,0x98860bdf,0x98870f05,0x98880945,0x988908fa,0x988a0853,0x988b7e1a,
	0x988c06e8,0x988d1762,0x988e7e1c,0x988f0a20,0x989016d8,0x98910eef,0x98927e1e,0x98931392,
	0x989406b7,0x98957e20,0x98961763,0x98970a23,0x98981322,0x98997e22,0x989a0474,0x989b19da,
	0x989c1648,0x989d0473,0x989e0e09,0x989f0c99,0x98a003b8,0x98a110ec,0x98a206d4,0x98a37e24,
	0x98a401e9,0x98a510b9,0x98a60ef0,0x98a7105b,0x98a87e26,0x98a97e28,0x98aa7e2a,0x98ab7e2c,
	0x98ac7e2e,0x98ad7e30,0x98ae7e32,0x98af7e34,0x98b07e36,0x98b17e38,0x98b27e3a,0x98b37e3c,
	0x98b47e3e,0x98b57e40,0x98b67e42,0x98b77e44,0x98b87e46,0x98b97e48,0x98ba7e4a,0x98bb7e4c,
	0x98bc7e4e,0x98bd7e50,0x98be7e52,0x98bf7e54,0x98c07e56,0x98c17e58,0x98c27e5a,0x98c37e5c,
	0x98c47e5e,0x98c57e60,0x98c67e62,0x98c77e64,0x98c87e66,0x98c97e68,0x98ca7e6a,0x98cb7e6c,
	0x98cc7e6e,0x98cd7e70,0x98ce04ff,0x98cf7e72,0x98d07e74,0x98d1011e,0x98d210d8,0x98d309a2,
	0x98d47e76,0x98d51261,0x98d67e78,0x98d77e7a,0x98d80edd,0x98d90124,0x98da0125,0x98db7e7c,
	0x98dc7e7e,0x98dd7e80,0x98de04cd,0x98df11a2,0x98e07e82,0x98e17e84,0x98e27e86,0x98e37e88,
	0x98e47e8a,0x98e57e8c,0x98e67e8e,0x98e71298,0x98e81520,0x98e97e90,0x98ea7e92,0x98eb7e94,
	0x98ec7e96,0x98ed7e98,0x98ee7e9a,0x98ef7e9c,0x98f07e9e,0x98f17ea0,0x98f27ea2,0x98f37ea4,
	0x98f47ea6,0x98f57ea8,0x98f67eaa,0x98f77eac,0x98f87eae,0x98f97eb0,0x98fa7eb2,0x98fb7eb4,
	0x98fc7eb6,0x98fd7eb8,0x98fe7eba,0x98ff7ebc,0x99007ebe,0x99017ec0,0x99027ec2,0x99037ec4,
	0x99047ec6,0x99057ec8,0x99067eca,0x99077ecc,0x99087ece,0x99097ed0,0x990a7ed2,0x990b7ed4,
	0x990c7ed6,0x990d1669,0x990e7ed8,0x990f7eda,0x9910018f,0x99117edc,0x99127ede,0x99137ee0,
	0x99147ee2,0x99157ee4,0x99167ee6,0x99177ee8,0x99187eea,0x99197eec,0x991a7eee,0x991b7ef0,
	0x991c7ef2,0x991d7ef4,0x991e7ef6,0x991f7ef8,0x99207efa,0x99217efc,0x99227efe,0x99237f00,
	0x99247f02,0x99257f04,0x99267f06,0x99277f08,0x99287f0a,0x99297f0c,0x992a7f0e,0x992b7f10,
	0x992c7f12,0x992d7f14,0x992e134f,0x992f7f16,0x99307f18,0x99317f1a,0x99327f1c,0x99337f1e,
	0x99347f20,0x99357f22,0x99367f24,0x99377f26,0x99387f28,0x99397f2a,0x993a7f2c,0x993b7f2e,
	0x993c7f30,0x993d7f32,0x993e7f34,0x993f7f36,0x99407f38,0x99417f3a,0x99427f3c,0x99437f3e,
	0x99447f40,0x99457f42,0x99467f44,0x99477f46,0x99487f48,0x99497f4a,0x994a7f4c,0x994b7f4e,
	0x994c7f50,0x994d7f52,0x994e7f54,0x994f7f56,0x99507f58,0x99517f5a,0x99527f5c,0x99537f5e,
	0x99541777,0x99551304,0x99567f60,0x99577f62,0x99587f64,0x99597f66,0x995a7f68,0x995b7f6a,
	0x995c7f6c,0x995d7f6e,0x995e7f70,0x995f7f72,0x99607f74,0x99617f76,0x99627f78,0x9963119a,
	0x99647f7a,0x996507da,0x99667f7c,0x99671582,0x9968139b,0x996914c5,0x996a1095,0x996b17e9,
	0x996c0264,0x996d04b7,0x996e173b,0x996f0890,0x997011ba,0x99710095,0x99721247,0x99737f7e,
	0x997416ce,0x99750494,0x9976107f,0x9977151f,0x99787f80,0x99797f82,0x997a08d2,0x997b7f84,
	0x997c0148,0x997d0152,0x997e7f86,0x997f047c,0x998017c2,0x99810dd2,0x99827f88,0x99837f8a,
	0x998407bd,0x99851508,0x9986064d,0x998701b5,0x99880a97,0x99897f8c,0x998a1260,0x998b01d9,
	0x998c7f8e,0x998d0d68,0x998e7f90,0x998f0bf3,0x999015a2,0x99910921,0x99920c9b,0x999310e5,
	0x999419e0,0x99950dbf,0x999611d3,0x99970a85,0x9998068d,0x99991510,0x999a7f92,0x999b7f94,
	0x999c7f96,0x999d7f98,0x999e7f9a,0x999f7f9c,0x99a07f9e,0x99a17fa0,0x99a27fa2,0x99a37fa4,
	0x99a47fa6,0x99a50568,0x99a67fa8,0x99a77faa,0x99a81577,0x99a97fac,0x99aa7fae,0x99ab7fb0,
	0x99ac7fb2,0x99ad7fb4,0x99ae7fb6,0x99af7fb8,0x99b07fba,0x99b17fbc,0x99b27fbe,0x99b37fc0,
	0x99b47fc2,0x99b57fc4,0x99b67fc6,0x99b77fc8,0x99b87fca,0x99b97fcc,0x99ba7fce,0x99bb7fd0,
	0x99bc7fd2,0x99bd7fd4,0x99be7fd6,0x99bf7fd8,0x99c07fda,0x99c17fdc,0x99c27fde,0x99c37fe0,
	0x99c47fe2,0x99c57fe4,0x99c67fe6,0x99c77fe8,0x99c87fea,0x99c97fec,0x99ca7fee,0x99cb7ff0,
	0x99cc7ff2,0x99cd7ff4,0x99ce7ff6,0x99cf7ff8,0x99d07ffa,0x99d17ffc,0x99d27ffe,0x99d38000,
	0x99d48002,0x99d58004,0x99d68006,0x99d78008,0x99d8800a,0x99d9800c,0x99da800e,0x99db8010,
	0x99dc8012,0x99dd8014,0x99de8016,0x99df8018,0x99e0801a,0x99e1801c,0x99e2801e,0x99e38020,
	0x99e48022,0x99e58024,0x99e68026,0x99e78028,0x99e8802a,0x99e9802c,0x99ea802e,0x99eb8030,
	0x99ec8032,0x99ed8034,0x99ee8036,0x99ef8038,0x99f0803a,0x99f1803c,0x99f2803e,0x99f38040,
	0x99f48042,0x99f58044,0x99f68046,0x99f78048,0x99f8804a,0x99f9804c,0x99fa804e,0x99fb8050,
	0x99fc8052,0x99fd8054,0x99fe8056,0x99ff8058,0x9a00805a,0x9a01805c,0x9a02805e,0x9a038060,
	0x9a048062,0x9a058064,0x9a068066,0x9a078068,0x9a08806a,0x9a09806c,0x9a0a806e,0x9a0b8070,
	0x9a0c8072,0x9a0d8074,0x9a0e8076,0x9a0f8078,0x9a10807a,0x9a11807c,0x9a12807e,0x9a138080,
	0x9a148082,0x9a158084,0x9a168086,0x9a178088,0x9a18808a,0x9a19808c,0x9a1a808e,0x9a1b8090,
	0x9a1c8092,0x9a1d8094,0x9a1e8096,0x9a1f8098,0x9a20809a,0x9a21809c,0x9a22809e,0x9a2380a0,
	0x9a2480a2,0x9a2580a4,0x9a2680a6,0x9a2780a8,0x9a2880aa,0x9a2980ac,0x9a2a80ae,0x9a2b80b0,
	0x9a2c80b2,0x9a2d80b4,0x9a2e80b6,0x9a2f80b8,0x9a3080ba,0x9a3180bc,0x9a3280be,0x9a3380c0,
	0x9a3480c2,0x9a3580c4,0x9a3680c6,0x9a3780c8,0x9a3880ca,0x9a3980cc,0x9a3a80ce,0x9a3b80d0,
	0x9a3c80d2,0x9a3d80d4,0x9a3e80d6,0x9a3f80d8,0x9a4080da,0x9a4180dc,0x9a4280de,0x9a4380e0,
	0x9a4480e2,0x9a4580e4,0x9a4680e6,0x9a4780e8,0x9a4880ea,0x9a4980ec,0x9a4a80ee,0x9a4b80f0,
	0x9a4c80f2,0x9a4d80f4,0x9a4e80f6,0x9a4f80f8,0x9a5080fa,0x9a5180fc,0x9a5280fe,0x9a538100,
	0x9a548102,0x9a558104,0x9a568106,0x9a578108,0x9a58810a,0x9a59810c,0x9a5a810e,0x9a5b8110,
	0x9a5c8112,0x9a5d8114,0x9a5e8116,0x9a5f8118,0x9a60811a,0x9a61811c,0x9a62811e,0x9a638120,
	0x9a648122,0x9a658124,0x9a668126,0x9a678128,0x9a68812a,0x9a69812c,0x9a6a812e,0x9a6b8130,
	0x9a6c0c86,0x9a6d17e5,0x9a6e13a4,0x9a6f1604,0x9a700253,0x9a71102f,0x9a728132,0x9a730157,
	0x9a740c6e,0x9a751866,0x9a7611aa,0x9a771248,0x9a78055a,0x9a79097d,0x9a7a1a3b,0x9a7b19ca,
	0x9a7c13a9,0x9a7d0e29,0x9a7e085c,0x9a7f1700,0x9a800347,0x9a81152e,0x9a820c8c,0x9a838134,
	0x9a8408c0,0x9a85074f,0x9a860c68,0x9a87069d,0x9a880ed5,0x9a898136,0x9a8a0b1b,0x9a8b0244,
	0x9a8c1660,0x9a8d8138,0x9a8e813a,0x9a8f09e0,0x9a900f4e,0x9a910f3a,0x9a920a34,0x9a9319eb,
	0x9a94813c,0x9a95813e,0x9a96018e,0x9a970eda,0x9a981977,0x9a998140,0x9a9a10ee,0x9a9b147e,
	0x9a9c003a,0x9a9d0bea,0x9a9e0f86,0x9a9f1128,0x9aa00120,0x9aa10c5d,0x9aa202e6,0x9aa301e7,
	0x9aa419a2,0x9aa5083a,0x9aa68142,0x9aa71517,0x9aa80624,0x9aa98144,0x9aaa8146,0x9aab8148,
	0x9aac814a,0x9aad814c,0x9aae814e,0x9aaf8150,0x9ab0137a,0x9ab1090d,0x9ab28152,0x9ab38154,
	0x9ab48156,0x9ab58158,0x9ab603a8,0x9ab70a55,0x9ab80697,0x9ab9815a,0x9aba0716,0x9abb815c,
	0x9abc05ca,0x9abd815e,0x9abe8160,0x9abf8162,0x9ac000fe,0x9ac10a27,0x9ac20f76,0x9ac38164,
	0x9ac48166,0x9ac50c0e,0x9ac68168,0x9ac7816a,0x9ac8816c,0x9ac9816e,0x9aca8170,0x9acb0a6c,
	0x9acc013e,0x9acd8172,0x9ace8174,0x9acf8176,0x9ad08178,0x9ad10421,0x9ad2817a,0x9ad3128b,
	0x9ad4817c,0x9ad5817e,0x9ad68180,0x9ad78182,0x9ad805a1,0x9ad98184,0x9ada8186,0x9adb8188,
	0x9adc818a,0x9add818c,0x9ade818e,0x9adf011f,0x9ae08190,0x9ae10a9e,0x9ae28192,0x9ae38194,
	0x9ae48196,0x9ae58198,0x9ae60cbc,0x9ae7819a,0x9ae8819c,0x9ae9819e,0x9aea81a0,0x9aeb1345,
	0x9aec81a2,0x9aed1a1f,0x9aee81a4,0x9aef1072,0x9af081a6,0x9af181a8,0x9af281aa,0x9af381ac,
	0x9af481ae,0x9af581b0,0x9af681b2,0x9af781b4,0x9af881b6,0x9af915a3,0x9afa81b8,0x9afb0839,
	0x9afc81ba,0x9afd81bc,0x9afe81be,0x9aff81c0,0x9b0081c2,0x9b0181c4,0x9b0281c6,0x9b031a35,
	0x9b0481c8,0x9b0581ca,0x9b0681cc,0x9b0781ce,0x9b08105a,0x9b0981d0,0x9b0a81d2,0x9b0b81d4,
	0x9b0c81d6,0x9b0d81d8,0x9b0e81da,0x9b0f0966,0x9b1081dc,0x9b1181de,0x9b1281e0,0x9b13013f,
	0x9b1481e2,0x9b1581e4,0x9b1681e6,0x9b1781e8,0x9b1881ea,0x9b1981ec,0x9b1a81ee,0x9b1b81f0,
	0x9b1c81f2,0x9b1d81f4,0x9b1e81f6,0x9b1f0767,0x9b2081f8,0x9b2181fa,0x9b2281fc,0x9b230bad,
	0x9b2481fe,0x9b258200,0x9b268202,0x9b278204,0x9b288206,0x9b298208,0x9b2a820a,0x9b2b820c,
	0x9b2c820e,0x9b2d8210,0x9b2e8212,0x9b2f0203,0x9b308214,0x9b318216,0x9b3205c1,0x9b338218,
	0x9b34821a,0x9b35821c,0x9b36821e,0x9b378220,0x9b388222,0x9b398224,0x9b3a8226,0x9b3b1805,
	0x9b3c066e,0x9b3d8228,0x9b3e822a,0x9b3f822c,0x9b40822e,0x9b410a8a,0x9b4207be,0x9b430054,
	0x9b440f10,0x9b450cea,0x9b468230,0x9b471657,0x9b481539,0x9b490b88,0x9b4a8232,0x9b4b8234,
	0x9b4c8236,0x9b4d13e9,0x9b4e8238,0x9b4f1429,0x9b50823a,0x9b510250,0x9b52823c,0x9b53823e,
	0x9b540d6f,0x9b558240,0x9b568242,0x9b578244,0x9b588246,0x9b598248,0x9b5a824a,0x9b5b824c,
	0x9b5c824e,0x9b5d8250,0x9b5e8252,0x9b5f8254,0x9b608256,0x9b618258,0x9b62825a,0x9b63825c,
	0x9b64825e,0x9b658260,0x9b668262,0x9b678264,0x9b688266,0x9b698268,0x9b6a826a,0x9b6b826c,
	0x9b6c826e,0x9b6d8270,0x9b6e8272,0x9b6f8274,0x9b708276,0x9b718278,0x9b72827a,0x9b73827c,
	0x9b74827e,0x9b758280,0x9b768282,0x9b778284,0x9b788286,0x9b798288,0x9b7a828a,0x9b7b828c,
	0x9b7c828e,0x9b7d8290,0x9b7e8292,0x9b7f8294,0x9b808296,0x9b818298,0x9b82829a,0x9b83829c,
	0x9b84829e,0x9b8582a0,0x9b8682a2,0x9b8782a4,0x9b8882a6,0x9b8982a8,0x9b8a82aa,0x9b8b82ac,
	0x9b8c82ae,0x9b8d82b0,0x9b8e82b2,0x9b8f82b4,0x9b9082b6,0x9b9182b8,0x9b9282ba,0x9b9382bc,
	0x9b9482be,0x9b9582c0,0x9b9682c2,0x9b9782c4,0x9b9882c6,0x9b9982c8,0x9b9a82ca,0x9b9b82cc,
	0x9b9c82ce,0x9b9d82d0,0x9b9e82d2,0x9b9f82d4,0x9ba082d6,0x9ba182d8,0x9ba282da,0x9ba382dc,
	0x9ba482de,0x9ba582e0,0x9ba682e2,0x9ba782e4,0x9ba882e6,0x9ba982e8,0x9baa82ea,0x9bab82ec,
	0x9bac82ee,0x9bad82f0,0x9bae82f2,0x9baf82f4,0x9bb082f6,0x9bb182f8,0x9bb282fa,0x9bb382fc,
	0x9bb482fe,0x9bb58300,0x9bb68302,0x9bb78304,0x9bb88306,0x9bb98308,0x9bba830a,0x9bbb830c,
	0x9bbc830e,0x9bbd8310,0x9bbe8312,0x9bbf8314,0x9bc08316,0x9bc18318,0x9bc2831a,0x9bc3831c,
	0x9bc4831e,0x9bc58320,0x9bc68322,0x9bc78324,0x9bc88326,0x9bc98328,0x9bca832a,0x9bcb832c,
	0x9bcc832e,0x9bcd8330,0x9bce8332,0x9bcf8334,0x9bd08336,0x9bd18338,0x9bd2833a,0x9bd3833c,
	0x9bd4833e,0x9bd58340,0x9bd68342,0x9bd78344,0x9bd88346,0x9bd98348,0x9bda834a,0x9bdb834c,
	0x9bdc834e,0x9bdd8350,0x9bde8352,0x9bdf8354,0x9be08356,0x9be18358,0x9be2835a,0x9be3835c,
	0x9be4835e,0x9be58360,0x9be68362,0x9be78364,0x9be88366,0x9be98368,0x9bea836a,0x9beb836c,
	0x9bec836e,0x9bed8370,0x9bee8372,0x9bef8374,0x9bf08376,0x9bf18378,0x9bf2837a,0x9bf3837c,
	0x9bf4837e,0x9bf58380,0x9bf68382,0x9bf78384,0x9bf88386,0x9bf98388,0x9bfa838a,0x9bfb838c,
	0x9bfc838e,0x9bfd8390,0x9bfe8392,0x9bff8394,0x9c008396,0x9c018398,0x9c02839a,0x9c03839c,
	0x9c04839e,0x9c0583a0,0x9c0683a2,0x9c0783a4,0x9c0883a6,0x9c0983a8,0x9c0a83aa,0x9c0b83ac,
	0x9c0c83ae,0x9c0d83b0,0x9c0e83b2,0x9c0f83b4,0x9c1083b6,0x9c1183b8,0x9c1283ba,0x9c1383bc,
	0x9c1483be,0x9c1583c0,0x9c1683c2,0x9c1783c4,0x9c1883c6,0x9c1983c8,0x9c1a83ca,0x9c1b83cc,
	0x9c1c83ce,0x9c1d83d0,0x9c1e83d2,0x9c1f83d4,0x9c2083d6,0x9c2183d8,0x9c2283da,0x9c2383dc,
	0x9c2483de,0x9c2583e0,0x9c2683e2,0x9c2783e4,0x9c2883e6,0x9c2983e8,0x9c2a83ea,0x9c2b83ec,
	0x9c2c83ee,0x9c2d83f0,0x9c2e83f2,0x9c2f83f4,0x9c3083f6,0x9c3183f8,0x9c3283fa,0x9c3383fc,
	0x9c3483fe,0x9c358400,0x9c368402,0x9c378404,0x9c388406,0x9c398408,0x9c3a840a,0x9c3b840c,
	0x9c3c840e,0x9c3d8410,0x9c3e8412,0x9c3f8414,0x9c408416,0x9c418418,0x9c42841a,0x9c43841c,
	0x9c44841e,0x9c458420,0x9c468422,0x9c478424,0x9c488426,0x9c498428,0x9c4a842a,0x9c4b842c,
	0x9c4c842e,0x9c4d8430,0x9c4e8432,0x9c4f8434,0x9c508436,0x9c518438,0x9c52843a,0x9c53843c,
	0x9c54843e,0x9c558440,0x9c568442,0x9c578444,0x9c588446,0x9c598448,0x9c5a844a,0x9c5b844c,
	0x9c5c844e,0x9c5d8450,0x9c5e8452,0x9c5f8454,0x9c608456,0x9c618458,0x9c62845a,0x9c63845c,
	0x9c64845e,0x9c658460,0x9c668462,0x9c678464,0x9c688466,0x9c698468,0x9c6a846a,0x9c6b846c,
	0x9c6c846e,0x9c6d8470,0x9c6e8472,0x9c6f8474,0x9c708476,0x9c718478,0x9c72847a,0x9c73847c,
	0x9c74847e,0x9c758480,0x9c768482,0x9c778484,0x9c788486,0x9c798488,0x9c7a848a,0x9c7b848c,
	0x9c7c17ba,0x9c7d848e,0x9c7e8490,0x9c7f1796,0x9c808492,0x9c810c27,0x9c8204c7,0x9c838494,
	0x9c848496,0x9c85005b,0x9c860f01,0x9c870dee,0x9c880c23,0x9c898498,0x9c8a849a,0x9c8b0563,
	0x9c8c849c,0x9c8d009f,0x9c8e071e,0x9c8f849e,0x9c9012c5,0x9c910667,0x9c9208ff,0x9c9384a0,
	0x9c94141a,0x9c95048f,0x9c9684a2,0x9c9784a4,0x9c9884a6,0x9c9984a8,0x9c9a0835,0x9c9b08c7,
	0x9c9c14ea,0x9c9d84aa,0x9c9e1522,0x9c9f1603,0x9ca005e3,0x9ca10b24,0x9ca20b6d,0x9ca30873,
	0x9ca40b34,0x9ca511a4,0x9ca61346,0x9ca70681,0x9ca81107,0x9ca90777,0x9caa84ac,0x9cab0837,
	0x9cac84ae,0x9cad0ffb,0x9cae0bda,0x9caf84b0,0x9cb01a3f,0x9cb104d6,0x9cb20aa0,0x9cb301f1,
	0x9cb40636,0x9cb50de0,0x9cb60def,0x9cb703d5,0x9cb80940,0x9cb984b2,0x9cba1198,0x9cbb1a20,
	0x9cbc04fc,0x9cbd03ea,0x9cbe84b4,0x9cbf84b6,0x9cc084b8,0x9cc184ba,0x9cc284bc,0x9cc310de,
	0x9cc40487,0x9cc51019,0x9cc60566,0x9cc7078d,0x9cc884be,0x9cc984c0,0x9cca0108,0x9ccb10f1,
	0x9ccc0038,0x9ccd0f58,0x9cce12b6,0x9ccf064b,0x9cd01698,0x9cd184c2,0x9cd284c4,0x9cd30afc,
	0x9cd4012a,0x9cd515ef,0x9cd6012c,0x9cd70c9e,0x9cd80d56,0x9cd91776,0x9cda84c6,0x9cdb84c8,
	0x9cdc0678,0x9cdd1130,0x9cde0bbb,0x9cdf1a5b,0x9ce084ca,0x9ce184cc,0x9ce20b37,0x9ce384ce,
	0x9ce484d0,0x9ce584d2,0x9ce684d4,0x9ce784d6,0x9ce884d8,0x9ce984da,0x9cea84dc,0x9ceb84de,
	0x9cec84e0,0x9ced84e2,0x9cee84e4,0x9cef84e6,0x9cf084e8,0x9cf184ea,0x9cf284ec,0x9cf384ee,
	0x9cf484f0,0x9cf584f2,0x9cf684f4,0x9cf784f6,0x9cf884f8,0x9cf984fa,0x9cfa84fc,0x9cfb84fe,
	0x9cfc8500,0x9cfd8502,0x9cfe8504,0x9cff8506,0x9d008508,0x9d01850a,0x9d02850c,0x9d03850e,
	0x9d048510,0x9d058512,0x9d068514,0x9d078516,0x9d088518,0x9d09851a,0x9d0a851c,0x9d0b851e,
	0x9d0c8520,0x9d0d8522,0x9d0e8524,0x9d0f8526,0x9d108528,0x9d11852a,0x9d12852c,0x9d13852e,
	0x9d148530,0x9d158532,0x9d168534,0x9d178536,0x9d188538,0x9d19853a,0x9d1a853c,0x9d1b853e,
	0x9d1c8540,0x9d1d8542,0x9d1e8544,0x9d1f8546,0x9d208548,0x9d21854a,0x9d22854c,0x9d23854e,
	0x9d248550,0x9d258552,0x9d268554,0x9d278556,0x9d288558,0x9d29855a,0x9d2a855c,0x9d2b855e,
	0x9d2c8560,0x9d2d8562,0x9d2e8564,0x9d2f8566,0x9d308568,0x9d31856a,0x9d32856c,0x9d33856e,
	0x9d348570,0x9d358572,0x9d368574,0x9d378576,0x9d388578,0x9d39857a,0x9d3a857c,0x9d3b857e,
	0x9d3c8580,0x9d3d8582,0x9d3e8584,0x9d3f8586,0x9d408588,0x9d41858a,0x9d42858c,0x9d43858e,
	0x9d448590,0x9d458592,0x9d468594,0x9d478596,0x9d488598,0x9d49859a,0x9d4a859c,0x9d4b859e,
	0x9d4c85a0,0x9d4d85a2,0x9d4e85a4,0x9d4f85a6,0x9d5085a8,0x9d5185aa,0x9d5285ac,0x9d5385ae,
	0x9d5485b0,0x9d5585b2,0x9d5685b4,0x9d5785b6,0x9d5885b8,0x9d5985ba,0x9d5a85bc,0x9d5b85be,
	0x9d5c85c0,0x9d5d85c2,0x9d5e85c4,0x9d5f85c6,0x9d6085c8,0x9d6185ca,0x9d6285cc,0x9d6385ce,
	0x9d6485d0,0x9d6585d2,0x9d6685d4,0x9d6785d6,0x9d6885d8,0x9d6985da,0x9d6a85dc,0x9d6b85de,
	0x9d6c85e0,0x9d6d85e2,0x9d6e85e4,0x9d6f85e6,0x9d7085e8,0x9d7185ea,0x9d7285ec,0x9d7385ee,
	0x9d7485f0,0x9d7585f2,0x9d7685f4,0x9d7785f6,0x9d7885f8,0x9d7985fa,0x9d7a85fc,0x9d7b85fe,
	0x9d7c8600,0x9d7d8602,0x9d7e8604,0x9d7f8606,0x9d808608,0x9d81860a,0x9d82860c,0x9d83860e,
	0x9d848610,0x9d858612,0x9d868614,0x9d878616,0x9d888618,0x9d89861a,0x9d8a861c,0x9d8b861e,
	0x9d8c8620,0x9d8d8622,0x9d8e8624,0x9d8f8626,0x9d908628,0x9d91862a,0x9d92862c,0x9d93862e,
	0x9d948630,0x9d958632,0x9d968634,0x9d978636,0x9d988638,0x9d99863a,0x9d9a863c,0x9d9b863e,
	0x9d9c8640,0x9d9d8642,0x9d9e8644,0x9d9f8646,0x9da08648,0x9da1864a,0x9da2864c,0x9da3864e,
	0x9da48650,0x9da58652,0x9da68654,0x9da78656,0x9da88658,0x9da9865a,0x9daa865c,0x9dab865e,
	0x9dac8660,0x9dad8662,0x9dae8664,0x9daf8666,0x9db08668,0x9db1866a,0x9db2866c,0x9db3866e,
	0x9db48670,0x9db58672,0x9db68674,0x9db78676,0x9db88678,0x9db9867a,0x9dba867c,0x9dbb867e,
	0x9dbc8680,0x9dbd8682,0x9dbe8684,0x9dbf8686,0x9dc08688,0x9dc1868a,0x9dc2868c,0x9dc3868e,
	0x9dc48690,0x9dc58692,0x9dc68694,0x9dc78696,0x9dc88698,0x9dc9869a,0x9dca869c,0x9dcb869e,
	0x9dcc86a0,0x9dcd86a2,0x9dce86a4,0x9dcf86a6,0x9dd086a8,0x9dd186aa,0x9dd286ac,0x9dd386ae,
	0x9dd486b0,0x9dd586b2,0x9dd686b4,0x9dd786b6,0x9dd886b8,0x9dd986ba,0x9dda86bc,0x9ddb86be,
	0x9ddc86c0,0x9ddd86c2,0x9dde86c4,0x9ddf86c6,0x9de086c8,0x9de186ca,0x9de286cc,0x9de386ce,
	0x9de486d0,0x9de586d2,0x9de686d4,0x9de786d6,0x9de886d8,0x9de986da,0x9dea86dc,0x9deb86de,
	0x9dec86e0,0x9ded86e2,0x9dee86e4,0x9def86e6,0x9df086e8,0x9df186ea,0x9df286ec,0x9df386ee,
	0x9df486f0,0x9df586f2,0x9df686f4,0x9df786f6,0x9df886f8,0x9df986fa,0x9dfa86fc,0x9dfb86fe,
	0x9dfc8700,0x9dfd8702,0x9dfe8704,0x9dff8706,0x9e008708,0x9e01870a,0x9e02870c,0x9e03870e,
	0x9e048710,0x9e058712,0x9e068714,0x9e078716,0x9e088718,0x9e09871a,0x9e0a871c,0x9e0b871e,
	0x9e0c8720,0x9e0d8722,0x9e0e8724,0x9e0f8726,0x9e108728,0x9e11872a,0x9e12872c,0x9e13872e,
	0x9e148730,0x9e158732,0x9e168734,0x9e178736,0x9e188738,0x9e19873a,0x9e1a873c,0x9e1b873e,
	0x9e1c8740,0x9e1d8742,0x9e1e8744,0x9e1f0dfa,0x9e200961,0x9e2107e2,0x9e221806,0x9e230d59,
	0x9e248746,0x9e250e44,0x9e261613,0x9e278748,0x9e280097,0x9e291916,0x9e2a0613,0x9e2b0404,
	0x9e2c0c20,0x9e2d1615,0x9e2e874a,0x9e2f1670,0x9e30874c,0x9e310249,0x9e32103b,0x9e331809,
	0x9e34874e,0x9e3513ab,0x9e361233,0x9e371971,0x9e38048e,0x9e39063a,0x9e3a15a0,0x9e3b8750,
	0x9e3c8752,0x9e3d05b9,0x9e3e0c44,0x9e3f070c,0x9e408754,0x9e410161,0x9e420b1e,0x9e4309ad,
	0x9e440628,0x9e450471,0x9e4617fb,0x9e4714f5,0x9e481321,0x9e49146c,0x9e4a106b,0x9e4b0d34,
	0x9e4c001e,0x9e4d8756,0x9e4e00a8,0x9e4f0ea0,0x9e508758,0x9e5102c7,0x9e52875a,0x9e53875c,
	0x9e54875e,0x9e550734,0x9e568760,0x9e570485,0x9e58061d,0x9e598762,0x9e5a02db,0x9e5b0cdf,
	0x9e5c1481,0x9e5d8764,0x9e5e16a0,0x9e5f8766,0x9e608768,0x9e61876a,0x9e62876c,0x9e630874,
	0x9e6406ef,0x9e65876e,0x9e66174d,0x9e6718fa,0x9e680bf5,0x9e690b99,0x9e6a08ca,0x9e6b0978,
	0x9e6c1804,0x9e6d0c3c,0x9e6e8770,0x9e6f8772,0x9e70174f,0x9e710749,0x9e728774,0x9e730654,
	0x9e748776,0x9e758778,0x9e76877a,0x9e77877c,0x9e78877e,0x9e798780,0x9e7a8782,0x9e7b8784,
	0x9e7c8786,0x9e7d8788,0x9e7e031d,0x9e7f0c32,0x9e80878a,0x9e81878c,0x9e820818,0x9e83878e,
	0x9e848790,0x9e858792,0x9e868794,0x9e8709da,0x9e8819c1,0x9e898796,0x9e8a8798,0x9e8b0d0f,
	0x9e8c879a,0x9e8d879c,0x9e8e879e,0x9e8f87a0,0x9e9087a2,0x9e9187a4,0x9e920f59,0x9e930c3d,
	0x9e9487a6,0x9e9587a8,0x9e9687aa,0x9e9787ac,0x9e9887ae,0x9e9987b0,0x9e9a87b2,0x9e9b87b4,
	0x9e9c87b6,0x9e9d1161,0x9e9e87b8,0x9e9f0bbc,0x9ea087ba,0x9ea187bc,0x9ea287be,0x9ea387c0,
	0x9ea487c2,0x9ea587c4,0x9ea60c96,0x9ea787c6,0x9ea887c8,0x9ea987ca,0x9eaa87cc,0x9eab87ce,
	0x9eac87d0,0x9ead87d2,0x9eae87d4,0x9eaf87d6,0x9eb087d8,0x9eb187da,0x9eb287dc,0x9eb387de,
	0x9eb41036,0x9eb587e0,0x9eb687e2,0x9eb787e4,0x9eb8051a,0x9eb987e6,0x9eba87e8,0x9ebb0c84,
	0x9ebc87ea,0x9ebd0cd0,0x9ebe0799,0x9ebf87ec,0x9ec087ee,0x9ec187f0,0x9ec287f2,0x9ec387f4,
	0x9ec40780,0x9ec587f6,0x9ec687f8,0x9ec787fa,0x9ec887fc,0x9ec9070d,0x9eca87fe,0x9ecb8800,
	0x9ecc8802,0x9ecd11f5,0x9ece0b25,0x9ecf0df0,0x9ed08804,0x9ed106f1,0x9ed28806,0x9ed38808,
	0x9ed40f93,0x9ed5880a,0x9ed6880c,0x9ed7880e,0x9ed80d7f,0x9ed98810,0x9eda8812,0x9edb0346,
	0x9edc02a1,0x9edd17a0,0x9ede8814,0x9edf16c6,0x9ee014da,0x9ee18816,0x9ee21037,0x9ee38818,
	0x9ee4881a,0x9ee51001,0x9ee6881c,0x9ee70b29,0x9ee8881e,0x9ee90428,0x9eea0194,0x9eeb8820,
	0x9eec8822,0x9eed8824,0x9eee8826,0x9eef002a,0x9ef08828,0x9ef1882a,0x9ef2882c,0x9ef3882e,
	0x9ef48830,0x9ef58832,0x9ef68834,0x9ef78836,0x9ef88838,0x9ef91958,0x9efa883a,0x9efb0542,
	0x9efc0550,0x9efd883c,0x9efe0d4d,0x9eff883e,0x9f008840,0x9f018842,0x9f028844,0x9f038846,
	0x9f048848,0x9f05884a,0x9f06884c,0x9f07884e,0x9f088850,0x9f098852,0x9f0a8854,0x9f0b1816,
	0x9f0c8856,0x9f0d13af,0x9f0e03f6,0x9f0f8858,0x9f100db3,0x9f11885a,0x9f12885c,0x9f130629,
	0x9f14885e,0x9f158860,0x9f168862,0x9f17130c,0x9f188864,0x9f190ec0,0x9f1a8866,0x9f1b8868,
	0x9f1c886a,0x9f1d886c,0x9f1e886e,0x9f1f8870,0x9f2011f7,0x9f218872,0x9f2204f4,0x9f238874,
	0x9f248876,0x9f258878,0x9f26887a,0x9f27887c,0x9f28887e,0x9f298880,0x9f2a8882,0x9f2b8884,
	0x9f2c17ac,0x9f2d8886,0x9f2e8888,0x9f2f145f,0x9f30888a,0x9f31888c,0x9f32888e,0x9f338890,
	0x9f348892,0x9f358894,0x9f368896,0x9f3714b2,0x9f388898,0x9f391658,0x9f3a889a,0x9f3b00d0,
	0x9f3c889c,0x9f3d1029,0x9f3e06a4,0x9f3f889e,0x9f4088a0,0x9f4188a2,0x9f4288a4,0x9f4388a6,
	0x9f441895,0x9f4588a8,0x9f4688aa,0x9f4788ac,0x9f4888ae,0x9f4988b0,0x9f4a88b2,0x9f4b88b4,
	0x9f4c88b6,0x9f4d88b8,0x9f4e88ba,0x9f4f88bc,0x9f500f3f,0x9f5107f3,0x9f5288be,0x9f5388c0,
	0x9f5488c2,0x9f5588c4,0x9f5688c6,0x9f5788c8,0x9f5888ca,0x9f5988cc,0x9f5a88ce,0x9f5b88d0,
	0x9f5c88d2,0x9f5d88d4,0x9f5e88d6,0x9f5f88d8,0x9f6088da,0x9f6188dc,0x9f6288de,0x9f6388e0,
	0x9f6488e2,0x9f6588e4,0x9f6688e6,0x9f6788e8,0x9f6888ea,0x9f6988ec,0x9f6a88ee,0x9f6b88f0,
	0x9f6c88f2,0x9f6d88f4,0x9f6e88f6,0x9f6f88f8,0x9f7088fa,0x9f7188fc,0x9f7288fe,0x9f738900,
	0x9f748902,0x9f758904,0x9f768906,0x9f778908,0x9f78890a,0x9f79890c,0x9f7a890e,0x9f7b8910,
	0x9f7c8912,0x9f7d8914,0x9f7e8916,0x9f7f025c,0x9f800229,0x9f818918,0x9f82891a,0x9f830991,
	0x9f840bd9,0x9f850091,0x9f861343,0x9f871a1e,0x9f881735,0x9f8917e2,0x9f8a02d0,0x9f8b1046,
	0x9f8c144d,0x9f8d891c,0x9f8e891e,0x9f8f8920,0x9f908922,0x9f918924,0x9f928926,0x9f938928,
	0x9f94892a,0x9f95892c,0x9f96892e,0x9f978930,0x9f988932,0x9f990bf6,0x9f9a05ef,0x9f9b09f8,
	0x9f9c8934,0x9f9d8936,0x9f9e8938,0x9f9f0661,0x9fa01833,0x9fa1893a,0x9fa2893c,0x9fa3893e,
	0x9fa48940,0x9fa58942,0x9fa68944,0x9fa78946,0x9fa88948,0x9fa9894a,0x9faa894c,0x9fab894e,
	0x9fac8950,0x9fad8952,0x9fae8954,0x9faf8956,0x9fb08958,0x9fb1895a,0x9fb2895c,0x9fb3895e,
	0x9fb48960,0x9fb58962,0x9fb68964,0x9fb78966,0x9fb88968,0x9fb9896a,0x9fba896c,0x9fbb896e,
	0xff011a6b,0xff021a6c,0xff031a6d,0xff041a6e,0xff051a6f,0xff061a70,0xff071a71,0xff081a72,
	0xff091a73,0xff0a1a74,0xff0b1a75,0xff0c1a76,0xff0d1a77,0xff0e1a78,0xff0f1a79,0xff101a7a,
	0xff111a7b,0xff121a7c,0xff131a7d,0xff141a7e,0xff151a7f,0xff161a80,0xff171a81,0xff181a82,
	0xff191a83,0xff1a1a84,0xff1b1a85,0xff1c1a86,0xff1d1a87,0xff1e1a88,0xff1f1a89,0xff201a8a,
	0xff211a8b,0xff221a8c,0xff231a8d,0xff241a8e,0xff251a8f,0xff261a90,0xff271a91,0xff281a92,
	0xff291a93,0xff2a1a94,0xff2b1a95,0xff2c1a96,0xff2d1a97,0xff2e1a98,0xff2f1a99,0xff301a9a,
	0xff311a9b,0xff321a9c,0xff331a9d,0xff341a9e,0xff351a9f,0xff361aa0,0xff371aa1,0xff381aa2,
	0xff391aa3,0xff3a1aa4,0xff3b1aa5,0xff3d1aa6,0xff3e1aa7,0xff3f1aa8,0xff401aa9,0xff411aaa,
	0xff421aab,0xff431aac,0xff441aad,0xff451aae,0xff461aaf,0xff471ab0,0xff481ab1,0xff491ab2,
	0xff4a1ab3,0xff4b1ab4,0xff4c1ab5,0xff4d1ab6,0xff4e1ab7,0xff4f1ab8,0xff501ab9,0xff511aba,
	0xff521abb,0xff531abc,0xff541abd,0xff551abe,0xff561abf,0xff571ac0,0xff581ac1,0xff591ac2,
	0xff5a1ac3,0xff5b1ac4,0xff5c1ac5,0xff5d1ac6,0xff5e1ac7,0xd840bcdc,0xdc00bcdc,0xd840bcde,
	0xdc01bcde,0xd840bce0,0xdc02bce0,0xd840bce2,0xdc03bce2,0xd840bce4,0xdc04bce4,0xd840bce6,
	0xdc05bce6,0xd840bce8,0xdc06bce8,0xd840bcea,0xdc07bcea,0xd840bcec,0xdc08bcec,0xd840bcee,
	0xdc09bcee,0xd840bcf0,0xdc0abcf0,0xd840bcf2,0xdc0bbcf2,0xd840bcf4,0xdc0cbcf4,0xd840bcf6,
	0xdc0dbcf6,0xd840bcf8,0xdc0ebcf8,0xd840bcfa,0xdc0fbcfa,0xd840bcfc,0xdc10bcfc,0xd840bcfe,
	0xdc11bcfe,0xd840bd00,0xdc12bd00,0xd840bd02,0xdc13bd02,0xd840bd04,0xdc14bd04,0xd840bd06,
	0xdc15bd06,0xd840bd08,0xdc16bd08,0xd840bd0a,0xdc17bd0a,0xd840bd0c,0xdc18bd0c,0xd840bd0e,
	0xdc19bd0e,0xd840bd10,0xdc1abd10,0xd840bd12,0xdc1bbd12,0xd840bd14,0xdc1cbd14,0xd840bd16,
	0xdc1dbd16,0xd840bd18,0xdc1ebd18,0xd840bd1a,0xdc1fbd1a,0xd840bd1c,0xdc20bd1c,0xd840bd1e,
	0xdc21bd1e,0xd840bd20,0xdc22bd20,0xd840bd22,0xdc23bd22,0xd840bd24,0xdc24bd24,0xd840bd26,
	0xdc25bd26,0xd840bd28,0xdc26bd28,0xd840bd2a,0xdc27bd2a,0xd840bd2c,0xdc28bd2c,0xd840bd2e,
	0xdc29bd2e,0xd840bd30,0xdc2abd30,0xd840bd32,0xdc2bbd32,0xd840bd34,0xdc2cbd34,0xd840bd36,
	0xdc2dbd36,0xd840bd38,0xdc2ebd38,0xd840bd3a,0xdc2fbd3a,0xd840bd3c,0xdc30bd3c,0xd840bd3e,
	0xdc31bd3e,0xd840bd40,0xdc32bd40,0xd840bd42,0xdc33bd42,0xd840bd44,0xdc34bd44,0xd840bd46,
	0xdc35bd46,0xd840bd48,0xdc36bd48,0xd840bd4a,0xdc37bd4a,0xd840bd4c,0xdc38bd4c,0xd840bd4e,
	0xdc39bd4e,0xd840bd50,0xdc3abd50,0xd840bd52,0xdc3bbd52,0xd840bd54,0xdc3cbd54,0xd840bd56,
	0xdc3dbd56,0xd840bd58,0xdc3ebd58,0xd840bd5a,0xdc3fbd5a,0xd840bd5c,0xdc40bd5c,0xd840bd5e,
	0xdc41bd5e,0xd840bd60,0xdc42bd60,0xd840bd62,0xdc43bd62,0xd840bd64,0xdc44bd64,0xd840bd66,
	0xdc45bd66,0xd840bd68,0xdc46bd68,0xd840bd6a,0xdc47bd6a,0xd840bd6c,0xdc48bd6c,0xd840bd6e,
	0xdc49bd6e,0xd840bd70,0xdc4abd70,0xd840bd72,0xdc4bbd72,0xd840bd74,0xdc4cbd74,0xd840bd76,
	0xdc4dbd76,0xd840bd78,0xdc4ebd78,0xd840bd7a,0xdc4fbd7a,0xd840bd7c,0xdc50bd7c,0xd840bd7e,
	0xdc51bd7e,0xd840bd80,0xdc52bd80,0xd840bd82,0xdc53bd82,0xd840bd84,0xdc54bd84,0xd840bd86,
	0xdc55bd86,0xd840bd88,0xdc56bd88,0xd840bd8a,0xdc57bd8a,0xd840bd8c,0xdc58bd8c,0xd840bd8e,
	0xdc59bd8e,0xd840bd90,0xdc5abd90,0xd840bd92,0xdc5bbd92,0xd840bd94,0xdc5cbd94,0xd840bd96,
	0xdc5dbd96,0xd840bd98,0xdc5ebd98,0xd840bd9a,0xdc5fbd9a,0xd840bd9c,0xdc60bd9c,0xd840bd9e,
	0xdc61bd9e,0xd840bda0,0xdc62bda0,0xd840bda2,0xdc63bda2,0xd840bda4,0xdc64bda4,0xd840bda6,
	0xdc65bda6,0xd840bda8,0xdc66bda8,0xd840bdaa,0xdc67bdaa,0xd840bdac,0xdc68bdac,0xd840bdae,
	0xdc69bdae,0xd840bdb0,0xdc6abdb0,0xd840bdb2,0xdc6bbdb2,0xd840bdb4,0xdc6cbdb4,0xd840bdb6,
	0xdc6dbdb6,0xd840bdb8,0xdc6ebdb8,0xd840bdba,0xdc6fbdba,0xd840bdbc,0xdc70bdbc,0xd840bdbe,
	0xdc71bdbe,0xd840bdc0,0xdc72bdc0,0xd840bdc2,0xdc73bdc2,0xd840bdc4,0xdc74bdc4,0xd840bdc6,
	0xdc75bdc6,0xd840bdc8,0xdc76bdc8,0xd840bdca,0xdc77bdca,0xd840bdcc,0xdc78bdcc,0xd840bdce,
	0xdc79bdce,0xd840bdd0,0xdc7abdd0,0xd840bdd2,0xdc7bbdd2,0xd840bdd4,0xdc7cbdd4,0xd840bdd6,
	0xdc7dbdd6,0xd840bdd8,0xdc7ebdd8,0xd840bdda,0xdc7fbdda,0xd840bddc,0xdc80bddc,0xd840bdde,
	0xdc81bdde,0xd840bde0,0xdc82bde0,0xd840bde2,0xdc83bde2,0xd840bde4,0xdc84bde4,0xd840bde6,
	0xdc85bde6,0xd840bde8,0xdc86bde8,0xd840bdea,0xdc87bdea,0xd840bdec,0xdc88bdec,0xd840bdee,
	0xdc89bdee,0xd840bdf0,0xdc8abdf0,0xd840bdf2,0xdc8bbdf2,0xd840bdf4,0xdc8cbdf4,0xd840bdf6,
	0xdc8dbdf6,0xd840bdf8,0xdc8ebdf8,0xd840bdfa,0xdc8fbdfa,0xd840bdfc,0xdc90bdfc,0xd840bdfe,
	0xdc91bdfe,0xd840be00,0xdc92be00,0xd840be02,0xdc93be02,0xd840be04,0xdc94be04,0xd840be06,
	0xdc95be06,0xd840be08,0xdc96be08,0xd840be0a,0xdc97be0a,0xd840be0c,0xdc98be0c,0xd840be0e,
	0xdc99be0e,0xd840be10,0xdc9abe10,0xd840be12,0xdc9bbe12,0xd840be14,0xdc9cbe14,0xd840be16,
	0xdc9dbe16,0xd840be18,0xdc9ebe18,0xd840be1a,0xdc9fbe1a,0xd840be1c,0xdca0be1c,0xd840be1e,
	0xdca1be1e,0xd840be20,0xdca2be20,0xd840be22,0xdca3be22,0xd840be24,0xdca4be24,0xd840be26,
	0xdca5be26,0xd840be28,0xdca6be28,0xd840be2a,0xdca7be2a,0xd840be2c,0xdca8be2c,0xd840be2e,
	0xdca9be2e,0xd840be30,0xdcaabe30,0xd840be32,0xdcabbe32,0xd840be34,0xdcacbe34,0xd840be36,
	0xdcadbe36,0xd840be38,0xdcaebe38,0xd840be3a,0xdcafbe3a,0xd840be3c,0xdcb0be3c,0xd840be3e,
	0xdcb1be3e,0xd840be40,0xdcb2be40,0xd840be42,0xdcb3be42,0xd840be44,0xdcb4be44,0xd840be46,
	0xdcb5be46,0xd840be48,0xdcb6be48,0xd840be4a,0xdcb7be4a,0xd840be4c,0xdcb8be4c,0xd840be4e,
	0xdcb9be4e,0xd840be50,0xdcbabe50,0xd840be52,0xdcbbbe52,0xd840be54,0xdcbcbe54,0xd840be56,
	0xdcbdbe56,0xd840be58,0xdcbebe58,0xd840be5a,0xdcbfbe5a,0xd840be5c,0xdcc0be5c,0xd840be5e,
	0xdcc1be5e,0xd840be60,0xdcc2be60,0xd840be62,0xdcc3be62,0xd840be64,0xdcc4be64,0xd840be66,
	0xdcc5be66,0xd840be68,0xdcc6be68,0xd840be6a,0xdcc7be6a,0xd840be6c,0xdcc8be6c,0xd840be6e,
	0xdcc9be6e,0xd840be70,0xdccabe70,0xd840be72,0xdccbbe72,0xd840be74,0xdcccbe74,0xd840be76,
	0xdccdbe76,0xd840be78,0xdccebe78,0xd840be7a,0xdccfbe7a,0xd840be7c,0xdcd0be7c,0xd840be7e,
	0xdcd1be7e,0xd840be80,0xdcd2be80,0xd840be82,0xdcd3be82,0xd840be84,0xdcd4be84,0xd840be86,
	0xdcd5be86,0xd840be88,0xdcd6be88,0xd840be8a,0xdcd7be8a,0xd840be8c,0xdcd8be8c,0xd840be8e,
	0xdcd9be8e,0xd840be90,0xdcdabe90,0xd840be92,0xdcdbbe92,0xd840be94,0xdcdcbe94,0xd840be96,
	0xdcddbe96,0xd840be98,0xdcdebe98,0xd840be9a,0xdcdfbe9a,0xd840be9c,0xdce0be9c,0xd840be9e,
	0xdce1be9e,0xd840bea0,0xdce2bea0,0xd840bea2,0xdce3bea2,0xd840bea4,0xdce4bea4,0xd840bea6,
	0xdce5bea6,0xd840bea8,0xdce6bea8,0xd840beaa,0xdce7beaa,0xd840beac,0xdce8beac,0xd840beae,
	0xdce9beae,0xd840beb0,0xdceabeb0,0xd840beb2,0xdcebbeb2,0xd840beb4,0xdcecbeb4,0xd840beb6,
	0xdcedbeb6,0xd840beb8,0xdceebeb8,0xd840beba,0xdcefbeba,0xd840bebc,0xdcf0bebc,0xd840bebe,
	0xdcf1bebe,0xd840bec0,0xdcf2bec0,0xd840bec2,0xdcf3bec2,0xd840bec4,0xdcf4bec4,0xd840bec6,
	0xdcf5bec6,0xd840bec8,0xdcf6bec8,0xd840beca,0xdcf7beca,0xd840becc,0xdcf8becc,0xd840bece,
	0xdcf9bece,0xd840bed0,0xdcfabed0,0xd840bed2,0xdcfbbed2,0xd840bed4,0xdcfcbed4,0xd840bed6,
	0xdcfdbed6,0xd840bed8,0xdcfebed8,0xd840beda,0xdcffbeda,0xd840bedc,0xdd00bedc,0xd840bede,
	0xdd01bede,0xd840bee0,0xdd02bee0,0xd840bee2,0xdd03bee2,0xd840bee4,0xdd04bee4,0xd840bee6,
	0xdd05bee6,0xd840bee8,0xdd06bee8,0xd840beea,0xdd07beea,0xd840beec,0xdd08beec,0xd840beee,
	0xdd09beee,0xd840bef0,0xdd0abef0,0xd840bef2,0xdd0bbef2,0xd840bef4,0xdd0cbef4,0xd840bef6,
	0xdd0dbef6,0xd840bef8,0xdd0ebef8,0xd840befa,0xdd0fbefa,0xd840befc,0xdd10befc,0xd840befe,
	0xdd11befe,0xd840bf00,0xdd12bf00,0xd840bf02,0xdd13bf02,0xd840bf04,0xdd14bf04,0xd840bf06,
	0xdd15bf06,0xd840bf08,0xdd16bf08,0xd840bf0a,0xdd17bf0a,0xd840bf0c,0xdd18bf0c,0xd840bf0e,
	0xdd19bf0e,0xd840bf10,0xdd1abf10,0xd840bf12,0xdd1bbf12,0xd840bf14,0xdd1cbf14,0xd840bf16,
	0xdd1dbf16,0xd840bf18,0xdd1ebf18,0xd840bf1a,0xdd1fbf1a,0xd840bf1c,0xdd20bf1c,0xd840bf1e,
	0xdd21bf1e,0xd840bf20,0xdd22bf20,0xd840bf22,0xdd23bf22,0xd840bf24,0xdd24bf24,0xd840bf26,
	0xdd25bf26,0xd840bf28,0xdd26bf28,0xd840bf2a,0xdd27bf2a,0xd840bf2c,0xdd28bf2c,0xd840bf2e,
	0xdd29bf2e,0xd840bf30,0xdd2abf30,0xd840bf32,0xdd2bbf32,0xd840bf34,0xdd2cbf34,0xd840bf36,
	0xdd2dbf36,0xd840bf38,0xdd2ebf38,0xd840bf3a,0xdd2fbf3a,0xd840bf3c,0xdd30bf3c,0xd840bf3e,
	0xdd31bf3e,0xd840bf40,0xdd32bf40,0xd840bf42,0xdd33bf42,0xd840bf44,0xdd34bf44,0xd840bf46,
	0xdd35bf46,0xd840bf48,0xdd36bf48,0xd840bf4a,0xdd37bf4a,0xd840bf4c,0xdd38bf4c,0xd840bf4e,
	0xdd39bf4e,0xd840bf50,0xdd3abf50,0xd840bf52,0xdd3bbf52,0xd840bf54,0xdd3cbf54,0xd840bf56,
	0xdd3dbf56,0xd840bf58,0xdd3ebf58,0xd840bf5a,0xdd3fbf5a,0xd840bf5c,0xdd40bf5c,0xd840bf5e,
	0xdd41bf5e,0xd840bf60,0xdd42bf60,0xd840bf62,0xdd43bf62,0xd840bf64,0xdd44bf64,0xd840bf66,
	0xdd45bf66,0xd840bf68,0xdd46bf68,0xd840bf6a,0xdd47bf6a,0xd840bf6c,0xdd48bf6c,0xd840bf6e,
	0xdd49bf6e,0xd840bf70,0xdd4abf70,0xd840bf72,0xdd4bbf72,0xd840bf74,0xdd4cbf74,0xd840bf76,
	0xdd4dbf76,0xd840bf78,0xdd4ebf78,0xd840bf7a,0xdd4fbf7a,0xd840bf7c,0xdd50bf7c,0xd840bf7e,
	0xdd51bf7e,0xd840bf80,0xdd52bf80,0xd840bf82,0xdd53bf82,0xd840bf84,0xdd54bf84,0xd840bf86,
	0xdd55bf86,0xd840bf88,0xdd56bf88,0xd840bf8a,0xdd57bf8a,0xd840bf8c,0xdd58bf8c,0xd840bf8e,
	0xdd59bf8e,0xd840bf90,0xdd5abf90,0xd840bf92,0xdd5bbf92,0xd840bf94,0xdd5cbf94,0xd840bf96,
	0xdd5dbf96,0xd840bf98,0xdd5ebf98,0xd840bf9a,0xdd5fbf9a,0xd840bf9c,0xdd60bf9c,0xd840bf9e,
	0xdd61bf9e,0xd840bfa0,0xdd62bfa0,0xd840bfa2,0xdd63bfa2,0xd840bfa4,0xdd64bfa4,0xd840bfa6,
	0xdd65bfa6,0xd840bfa8,0xdd66bfa8,0xd840bfaa,0xdd67bfaa,0xd840bfac,0xdd68bfac,0xd840bfae,
	0xdd69bfae,0xd840bfb0,0xdd6abfb0,0xd840bfb2,0xdd6bbfb2,0xd840bfb4,0xdd6cbfb4,0xd840bfb6,
	0xdd6dbfb6,0xd840bfb8,0xdd6ebfb8,0xd840bfba,0xdd6fbfba,0xd840bfbc,0xdd70bfbc,0xd840bfbe,
	0xdd71bfbe,0xd840bfc0,0xdd72bfc0,0xd840bfc2,0xdd73bfc2,0xd840bfc4,0xdd74bfc4,0xd840bfc6,
	0xdd75bfc6,0xd840bfc8,0xdd76bfc8,0xd840bfca,0xdd77bfca,0xd840bfcc,0xdd78bfcc,0xd840bfce,
	0xdd79bfce,0xd840bfd0,0xdd7abfd0,0xd840bfd2,0xdd7bbfd2,0xd840bfd4,0xdd7cbfd4,0xd840bfd6,
	0xdd7dbfd6,0xd840bfd8,0xdd7ebfd8,0xd840bfda,0xdd7fbfda,0xd840bfdc,0xdd80bfdc,0xd840bfde,
	0xdd81bfde,0xd840bfe0,0xdd82bfe0,0xd840bfe2,0xdd83bfe2,0xd840bfe4,0xdd84bfe4,0xd840bfe6,
	0xdd85bfe6,0xd840bfe8,0xdd86bfe8,0xd840bfea,0xdd87bfea,0xd840bfec,0xdd88bfec,0xd840bfee,
	0xdd89bfee,0xd840bff0,0xdd8abff0,0xd840bff2,0xdd8bbff2,0xd840bff4,0xdd8cbff4,0xd840bff6,
	0xdd8dbff6,0xd840bff8,0xdd8ebff8,0xd840bffa,0xdd8fbffa,0xd840bffc,0xdd90bffc,0xd840bffe,
	0xdd91bffe,0xd840c000,0xdd92c000,0xd840c002,0xdd93c002,0xd840c004,0xdd94c004,0xd840c006,
	0xdd95c006,0xd840c008,0xdd96c008,0xd840c00a,0xdd97c00a,0xd840c00c,0xdd98c00c,0xd840c00e,
	0xdd99c00e,0xd840c010,0xdd9ac010,0xd840c012,0xdd9bc012,0xd840c014,0xdd9cc014,0xd840c016,
	0xdd9dc016,0xd840c018,0xdd9ec018,0xd840c01a,0xdd9fc01a,0xd840c01c,0xdda0c01c,0xd840c01e,
	0xdda1c01e,0xd840c020,0xdda2c020,0xd840c022,0xdda3c022,0xd840c024,0xdda4c024,0xd840c026,
	0xdda5c026,0xd840c028,0xdda6c028,0xd840c02a,0xdda7c02a,0xd840c02c,0xdda8c02c,0xd840c02e,
	0xdda9c02e,0xd840c030,0xddaac030,0xd840c032,0xddabc032,0xd840c034,0xddacc034,0xd840c036,
	0xddadc036,0xd840c038,0xddaec038,0xd840c03a,0xddafc03a,0xd840c03c,0xddb0c03c,0xd840c03e,
	0xddb1c03e,0xd840c040,0xddb2c040,0xd840c042,0xddb3c042,0xd840c044,0xddb4c044,0xd840c046,
	0xddb5c046,0xd840c048,0xddb6c048,0xd840c04a,0xddb7c04a,0xd840c04c,0xddb8c04c,0xd840c04e,
	0xddb9c04e,0xd840c050,0xddbac050,0xd840c052,0xddbbc052,0xd840c054,0xddbcc054,0xd840c056,
	0xddbdc056,0xd840c058,0xddbec058,0xd840c05a,0xddbfc05a,0xd840c05c,0xddc0c05c,0xd840c05e,
	0xddc1c05e,0xd840c060,0xddc2c060,0xd840c062,0xddc3c062,0xd840c064,0xddc4c064,0xd840c066,
	0xddc5c066,0xd840c068,0xddc6c068,0xd840c06a,0xddc7c06a,0xd840c06c,0xddc8c06c,0xd840c06e,
	0xddc9c06e,0xd840c070,0xddcac070,0xd840c072,0xddcbc072,0xd840c074,0xddccc074,0xd840c076,
	0xddcdc076,0xd840c078,0xddcec078,0xd840c07a,0xddcfc07a,0xd840c07c,0xddd0c07c,0xd840c07e,
	0xddd1c07e,0xd840c080,0xddd2c080,0xd840c082,0xddd3c082,0xd840c084,0xddd4c084,0xd840c086,
	0xddd5c086,0xd840c088,0xddd6c088,0xd840c08a,0xddd7c08a,0xd840c08c,0xddd8c08c,0xd840c08e,
	0xddd9c08e,0xd840c090,0xdddac090,0xd840c092,0xdddbc092,0xd840c094,0xdddcc094,0xd840c096,
	0xddddc096,0xd840c098,0xdddec098,0xd840c09a,0xdddfc09a,0xd840c09c,0xdde0c09c,0xd840c09e,
	0xdde1c09e,0xd840c0a0,0xdde2c0a0,0xd840c0a2,0xdde3c0a2,0xd840c0a4,0xdde4c0a4,0xd840c0a6,
	0xdde5c0a6,0xd840c0a8,0xdde6c0a8,0xd840c0aa,0xdde7c0aa,0xd840c0ac,0xdde8c0ac,0xd840c0ae,
	0xdde9c0ae,0xd840c0b0,0xddeac0b0,0xd840c0b2,0xddebc0b2,0xd840c0b4,0xddecc0b4,0xd840c0b6,
	0xddedc0b6,0xd840c0b8,0xddeec0b8,0xd840c0ba,0xddefc0ba,0xd840c0bc,0xddf0c0bc,0xd840c0be,
	0xddf1c0be,0xd840c0c0,0xddf2c0c0,0xd840c0c2,0xddf3c0c2,0xd840c0c4,0xddf4c0c4,0xd840c0c6,
	0xddf5c0c6,0xd840c0c8,0xddf6c0c8,0xd840c0ca,0xddf7c0ca,0xd840c0cc,0xddf8c0cc,0xd840c0ce,
	0xddf9c0ce,0xd840c0d0,0xddfac0d0,0xd840c0d2,0xddfbc0d2,0xd840c0d4,0xddfcc0d4,0xd840c0d6,
	0xddfdc0d6,0xd840c0d8,0xddfec0d8,0xd840c0da,0xddffc0da,0xd840c0dc,0xde00c0dc,0xd840c0de,
	0xde01c0de,0xd840c0e0,0xde02c0e0,0xd840c0e2,0xde03c0e2,0xd840c0e4,0xde04c0e4,0xd840c0e6,
	0xde05c0e6,0xd840c0e8,0xde06c0e8,0xd840c0ea,0xde07c0ea,0xd840c0ec,0xde08c0ec,0xd840c0ee,
	0xde09c0ee,0xd840c0f0,0xde0ac0f0,0xd840c0f2,0xde0bc0f2,0xd840c0f4,0xde0cc0f4,0xd840c0f6,
	0xde0dc0f6,0xd840c0f8,0xde0ec0f8,0xd840c0fa,0xde0fc0fa,0xd840c0fc,0xde10c0fc,0xd840c0fe,
	0xde11c0fe,0xd840c100,0xde12c100,0xd840c102,0xde13c102,0xd840c104,0xde14c104,0xd840c106,
	0xde15c106,0xd840c108,0xde16c108,0xd840c10a,0xde17c10a,0xd840c10c,0xde18c10c,0xd840c10e,
	0xde19c10e,0xd840c110,0xde1ac110,0xd840c112,0xde1bc112,0xd840c114,0xde1cc114,0xd840c116,
	0xde1dc116,0xd840c118,0xde1ec118,0xd840c11a,0xde1fc11a,0xd840c11c,0xde20c11c,0xd840c11e,
	0xde21c11e,0xd840c120,0xde22c120,0xd840c122,0xde23c122,0xd840c124,0xde24c124,0xd840c126,
	0xde25c126,0xd840c128,0xde26c128,0xd840c12a,0xde27c12a,0xd840c12c,0xde28c12c,0xd840c12e,
	0xde29c12e,0xd840c130,0xde2ac130,0xd840c132,0xde2bc132,0xd840c134,0xde2cc134,0xd840c136,
	0xde2dc136,0xd840c138,0xde2ec138,0xd840c13a,0xde2fc13a,0xd840c13c,0xde30c13c,0xd840c13e,
	0xde31c13e,0xd840c140,0xde32c140,0xd840c142,0xde33c142,0xd840c144,0xde34c144,0xd840c146,
	0xde35c146,0xd840c148,0xde36c148,0xd840c14a,0xde37c14a,0xd840c14c,0xde38c14c,0xd840c14e,
	0xde39c14e,0xd840c150,0xde3ac150,0xd840c152,0xde3bc152,0xd840c154,0xde3cc154,0xd840c156,
	0xde3dc156,0xd840c158,0xde3ec158,0xd840c15a,0xde3fc15a,0xd840c15c,0xde40c15c,0xd840c15e,
	0xde41c15e,0xd840c160,0xde42c160,0xd840c162,0xde43c162,0xd840c164,0xde44c164,0xd840c166,
	0xde45c166,0xd840c168,0xde46c168,0xd840c16a,0xde47c16a,0xd840c16c,0xde48c16c,0xd840c16e,
	0xde49c16e,0xd840c170,0xde4ac170,0xd840c172,0xde4bc172,0xd840c174,0xde4cc174,0xd840c176,
	0xde4dc176,0xd840c178,0xde4ec178,0xd840c17a,0xde4fc17a,0xd840c17c,0xde50c17c,0xd840c17e,
	0xde51c17e,0xd840c180,0xde52c180,0xd840c182,0xde53c182,0xd840c184,0xde54c184,0xd840c186,
	0xde55c186,0xd840c188,0xde56c188,0xd840c18a,0xde57c18a,0xd840c18c,0xde58c18c,0xd840c18e,
	0xde59c18e,0xd840c190,0xde5ac190,0xd840c192,0xde5bc192,0xd840c194,0xde5cc194,0xd840c196,
	0xde5dc196,0xd840c198,0xde5ec198,0xd840c19a,0xde5fc19a,0xd840c19c,0xde60c19c,0xd840c19e,
	0xde61c19e,0xd840c1a0,0xde62c1a0,0xd840c1a2,0xde63c1a2,0xd840c1a4,0xde64c1a4,0xd840c1a6,
	0xde65c1a6,0xd840c1a8,0xde66c1a8,0xd840c1aa,0xde67c1aa,0xd840c1ac,0xde68c1ac,0xd840c1ae,
	0xde69c1ae,0xd840c1b0,0xde6ac1b0,0xd840c1b2,0xde6bc1b2,0xd840c1b4,0xde6cc1b4,0xd840c1b6,
	0xde6dc1b6,0xd840c1b8,0xde6ec1b8,0xd840c1ba,0xde6fc1ba,0xd840c1bc,0xde70c1bc,0xd840c1be,
	0xde71c1be,0xd840c1c0,0xde72c1c0,0xd840c1c2,0xde73c1c2,0xd840c1c4,0xde74c1c4,0xd840c1c6,
	0xde75c1c6,0xd840c1c8,0xde76c1c8,0xd840c1ca,0xde77c1ca,0xd840c1cc,0xde78c1cc,0xd840c1ce,
	0xde79c1ce,0xd840c1d0,0xde7ac1d0,0xd840c1d2,0xde7bc1d2,0xd840c1d4,0xde7cc1d4,0xd840c1d6,
	0xde7dc1d6,0xd840c1d8,0xde7ec1d8,0xd840c1da,0xde7fc1da,0xd840c1dc,0xde80c1dc,0xd840c1de,
	0xde81c1de,0xd840c1e0,0xde82c1e0,0xd840c1e2,0xde83c1e2,0xd840c1e4,0xde84c1e4,0xd840c1e6,
	0xde85c1e6,0xd840c1e8,0xde86c1e8,0xd840c1ea,0xde87c1ea,0xd840c1ec,0xde88c1ec,0xd840c1ee,
	0xde89c1ee,0xd840c1f0,0xde8ac1f0,0xd840c1f2,0xde8bc1f2,0xd840c1f4,0xde8cc1f4,0xd840c1f6,
	0xde8dc1f6,0xd840c1f8,0xde8ec1f8,0xd840c1fa,0xde8fc1fa,0xd840c1fc,0xde90c1fc,0xd840c1fe,
	0xde91c1fe,0xd840c200,0xde92c200,0xd840c202,0xde93c202,0xd840c204,0xde94c204,0xd840c206,
	0xde95c206,0xd840c208,0xde96c208,0xd840c20a,0xde97c20a,0xd840c20c,0xde98c20c,0xd840c20e,
	0xde99c20e,0xd840c210,0xde9ac210,0xd840c212,0xde9bc212,0xd840c214,0xde9cc214,0xd840c216,
	0xde9dc216,0xd840c218,0xde9ec218,0xd840c21a,0xde9fc21a,0xd840c21c,0xdea0c21c,0xd840c21e,
	0xdea1c21e,0xd840c220,0xdea2c220,0xd840c222,0xdea3c222,0xd840c224,0xdea4c224,0xd840c226,
	0xdea5c226,0xd840c228,0xdea6c228,0xd840c22a,0xdea7c22a,0xd840c22c,0xdea8c22c,0xd840c22e,
	0xdea9c22e,0xd840c230,0xdeaac230,0xd840c232,0xdeabc232,0xd840c234,0xdeacc234,0xd840c236,
	0xdeadc236,0xd840c238,0xdeaec238,0xd840c23a,0xdeafc23a,0xd840c23c,0xdeb0c23c,0xd840c23e,
	0xdeb1c23e,0xd840c240,0xdeb2c240,0xd840c242,0xdeb3c242,0xd840c244,0xdeb4c244,0xd840c246,
	0xdeb5c246,0xd840c248,0xdeb6c248,0xd840c24a,0xdeb7c24a,0xd840c24c,0xdeb8c24c,0xd840c24e,
	0xdeb9c24e,0xd840c250,0xdebac250,0xd840c252,0xdebbc252,0xd840c254,0xdebcc254,0xd840c256,
	0xdebdc256,0xd840c258,0xdebec258,0xd840c25a,0xdebfc25a,0xd840c25c,0xdec0c25c,0xd840c25e,
	0xdec1c25e,0xd840c260,0xdec2c260,0xd840c262,0xdec3c262,0xd840c264,0xdec4c264,0xd840c266,
	0xdec5c266,0xd840c268,0xdec6c268,0xd840c26a,0xdec7c26a,0xd840c26c,0xdec8c26c,0xd840c26e,
	0xdec9c26e,0xd840c270,0xdecac270,0xd840c272,0xdecbc272,0xd840c274,0xdeccc274,0xd840c276,
	0xdecdc276,0xd840c278,0xdecec278,0xd840c27a,0xdecfc27a,0xd840c27c,0xded0c27c,0xd840c27e,
	0xded1c27e,0xd840c280,0xded2c280,0xd840c282,0xded3c282,0xd840c284,0xded4c284,0xd840c286,
	0xded5c286,0xd840c288,0xded6c288,0xd840c28a,0xded7c28a,0xd840c28c,0xded8c28c,0xd840c28e,
	0xded9c28e,0xd840c290,0xdedac290,0xd840c292,0xdedbc292,0xd840c294,0xdedcc294,0xd840c296,
	0xdeddc296,0xd840c298,0xdedec298,0xd840c29a,0xdedfc29a,0xd840c29c,0xdee0c29c,0xd840c29e,
	0xdee1c29e,0xd840c2a0,0xdee2c2a0,0xd840c2a2,0xdee3c2a2,0xd840c2a4,0xdee4c2a4,0xd840c2a6,
	0xdee5c2a6,0xd840c2a8,0xdee6c2a8,0xd840c2aa,0xdee7c2aa,0xd840c2ac,0xdee8c2ac,0xd840c2ae,
	0xdee9c2ae,0xd840c2b0,0xdeeac2b0,0xd840c2b2,0xdeebc2b2,0xd840c2b4,0xdeecc2b4,0xd840c2b6,
	0xdeedc2b6,0xd840c2b8,0xdeeec2b8,0xd840c2ba,0xdeefc2ba,0xd840c2bc,0xdef0c2bc,0xd840c2be,
	0xdef1c2be,0xd840c2c0,0xdef2c2c0,0xd840c2c2,0xdef3c2c2,0xd840c2c4,0xdef4c2c4,0xd840c2c6,
	0xdef5c2c6,0xd840c2c8,0xdef6c2c8,0xd840c2ca,0xdef7c2ca,0xd840c2cc,0xdef8c2cc,0xd840c2ce,
	0xdef9c2ce,0xd840c2d0,0xdefac2d0,0xd840c2d2,0xdefbc2d2,0xd840c2d4,0xdefcc2d4,0xd840c2d6,
	0xdefdc2d6,0xd840c2d8,0xdefec2d8,0xd840c2da,0xdeffc2da,0xd840c2dc,0xdf00c2dc,0xd840c2de,
	0xdf01c2de,0xd840c2e0,0xdf02c2e0,0xd840c2e2,0xdf03c2e2,0xd840c2e4,0xdf04c2e4,0xd840c2e6,
	0xdf05c2e6,0xd840c2e8,0xdf06c2e8,0xd840c2ea,0xdf07c2ea,0xd840c2ec,0xdf08c2ec,0xd840c2ee,
	0xdf09c2ee,0xd840c2f0,0xdf0ac2f0,0xd840c2f2,0xdf0bc2f2,0xd840c2f4,0xdf0cc2f4,0xd840c2f6,
	0xdf0dc2f6,0xd840c2f8,0xdf0ec2f8,0xd840c2fa,0xdf0fc2fa,0xd840c2fc,0xdf10c2fc,0xd840c2fe,
	0xdf11c2fe,0xd840c300,0xdf12c300,0xd840c302,0xdf13c302,0xd840c304,0xdf14c304,0xd840c306,
	0xdf15c306,0xd840c308,0xdf16c308,0xd840c30a,0xdf17c30a,0xd840c30c,0xdf18c30c,0xd840c30e,
	0xdf19c30e,0xd840c310,0xdf1ac310,0xd840c312,0xdf1bc312,0xd840c314,0xdf1cc314,0xd840c316,
	0xdf1dc316,0xd840c318,0xdf1ec318,0xd840c31a,0xdf1fc31a,0xd840c31c,0xdf20c31c,0xd840c31e,
	0xdf21c31e,0xd840c320,0xdf22c320,0xd840c322,0xdf23c322,0xd840c324,0xdf24c324,0xd840c326,
	0xdf25c326,0xd840c328,0xdf26c328,0xd840c32a,0xdf27c32a,0xd840c32c,0xdf28c32c,0xd840c32e,
	0xdf29c32e,0xd840c330,0xdf2ac330,0xd840c332,0xdf2bc332,0xd840c334,0xdf2cc334,0xd840c336,
	0xdf2dc336,0xd840c338,0xdf2ec338,0xd840c33a,0xdf2fc33a,0xd840c33c,0xdf30c33c,0xd840c33e,
	0xdf31c33e,0xd840c340,0xdf32c340,0xd840c342,0xdf33c342,0xd840c344,0xdf34c344,0xd840c346,
	0xdf35c346,0xd840c348,0xdf36c348,0xd840c34a,0xdf37c34a,0xd840c34c,0xdf38c34c,0xd840c34e,
	0xdf39c34e,0xd840c350,0xdf3ac350,0xd840c352,0xdf3bc352,0xd840c354,0xdf3cc354,0xd840c356,
	0xdf3dc356,0xd840c358,0xdf3ec358,0xd840c35a,0xdf3fc35a,0xd840c35c,0xdf40c35c,0xd840c35e,
	0xdf41c35e,0xd840c360,0xdf42c360,0xd840c362,0xdf43c362,0xd840c364,0xdf44c364,0xd840c366,
	0xdf45c366,0xd840c368,0xdf46c368,0xd840c36a,0xdf47c36a,0xd840c36c,0xdf48c36c,0xd840c36e,
	0xdf49c36e,0xd840c370,0xdf4ac370,0xd840c372,0xdf4bc372,0xd840c374,0xdf4cc374,0xd840c376,
	0xdf4dc376,0xd840c378,0xdf4ec378,0xd840c37a,0xdf4fc37a,0xd840c37c,0xdf50c37c,0xd840c37e,
	0xdf51c37e,0xd840c380,0xdf52c380,0xd840c382,0xdf53c382,0xd840c384,0xdf54c384,0xd840c386,
	0xdf55c386,0xd840c388,0xdf56c388,0xd840c38a,0xdf57c38a,0xd840c38c,0xdf58c38c,0xd840c38e,
	0xdf59c38e,0xd840c390,0xdf5ac390,0xd840c392,0xdf5bc392,0xd840c394,0xdf5cc394,0xd840c396,
	0xdf5dc396,0xd840c398,0xdf5ec398,0xd840c39a,0xdf5fc39a,0xd840c39c,0xdf60c39c,0xd840c39e,
	0xdf61c39e,0xd840c3a0,0xdf62c3a0,0xd840c3a2,0xdf63c3a2,0xd840c3a4,0xdf64c3a4,0xd840c3a6,
	0xdf65c3a6,0xd840c3a8,0xdf66c3a8,0xd840c3aa,0xdf67c3aa,0xd840c3ac,0xdf68c3ac,0xd840c3ae,
	0xdf69c3ae,0xd840c3b0,0xdf6ac3b0,0xd840c3b2,0xdf6bc3b2,0xd840c3b4,0xdf6cc3b4,0xd840c3b6,
	0xdf6dc3b6,0xd840c3b8,0xdf6ec3b8,0xd840c3ba,0xdf6fc3ba,0xd840c3bc,0xdf70c3bc,0xd840c3be,
	0xdf71c3be,0xd840c3c0,0xdf72c3c0,0xd840c3c2,0xdf73c3c2,0xd840c3c4,0xdf74c3c4,0xd840c3c6,
	0xdf75c3c6,0xd840c3c8,0xdf76c3c8,0xd840c3ca,0xdf77c3ca,0xd840c3cc,0xdf78c3cc,0xd840c3ce,
	0xdf79c3ce,0xd840c3d0,0xdf7ac3d0,0xd840c3d2,0xdf7bc3d2,0xd840c3d4,0xdf7cc3d4,0xd840c3d6,
	0xdf7dc3d6,0xd840c3d8,0xdf7ec3d8,0xd840c3da,0xdf7fc3da,0xd840c3dc,0xdf80c3dc,0xd840c3de,
	0xdf81c3de,0xd840c3e0,0xdf82c3e0,0xd840c3e2,0xdf83c3e2,0xd840c3e4,0xdf84c3e4,0xd840c3e6,
	0xdf85c3e6,0xd840c3e8,0xdf86c3e8,0xd840c3ea,0xdf87c3ea,0xd840c3ec,0xdf88c3ec,0xd840c3ee,
	0xdf89c3ee,0xd840c3f0,0xdf8ac3f0,0xd840c3f2,0xdf8bc3f2,0xd840c3f4,0xdf8cc3f4,0xd840c3f6,
	0xdf8dc3f6,0xd840c3f8,0xdf8ec3f8,0xd840c3fa,0xdf8fc3fa,0xd840c3fc,0xdf90c3fc,0xd840c3fe,
	0xdf91c3fe,0xd840c400,0xdf92c400,0xd840c402,0xdf93c402,0xd840c404,0xdf94c404,0xd840c406,
	0xdf95c406,0xd840c408,0xdf96c408,0xd840c40a,0xdf97c40a,0xd840c40c,0xdf98c40c,0xd840c40e,
	0xdf99c40e,0xd840c410,0xdf9ac410,0xd840c412,0xdf9bc412,0xd840c414,0xdf9cc414,0xd840c416,
	0xdf9dc416,0xd840c418,0xdf9ec418,0xd840c41a,0xdf9fc41a,0xd840c41c,0xdfa0c41c,0xd840c41e,
	0xdfa1c41e,0xd840c420,0xdfa2c420,0xd840c422,0xdfa3c422,0xd840c424,0xdfa4c424,0xd840c426,
	0xdfa5c426,0xd840c428,0xdfa6c428,0xd840c42a,0xdfa7c42a,0xd840c42c,0xdfa8c42c,0xd840c42e,
	0xdfa9c42e,0xd840c430,0xdfaac430,0xd840c432,0xdfabc432,0xd840c434,0xdfacc434,0xd840c436,
	0xdfadc436,0xd840c438,0xdfaec438,0xd840c43a,0xdfafc43a,0xd840c43c,0xdfb0c43c,0xd840c43e,
	0xdfb1c43e,0xd840c440,0xdfb2c440,0xd840c442,0xdfb3c442,0xd840c444,0xdfb4c444,0xd840c446,
	0xdfb5c446,0xd840c448,0xdfb6c448,0xd840c44a,0xdfb7c44a,0xd840c44c,0xdfb8c44c,0xd840c44e,
	0xdfb9c44e,0xd840c450,0xdfbac450,0xd840c452,0xdfbbc452,0xd840c454,0xdfbcc454,0xd840c456,
	0xdfbdc456,0xd840c458,0xdfbec458,0xd840c45a,0xdfbfc45a,0xd840c45c,0xdfc0c45c,0xd840c45e,
	0xdfc1c45e,0xd840c460,0xdfc2c460,0xd840c462,0xdfc3c462,0xd840c464,0xdfc4c464,0xd840c466,
	0xdfc5c466,0xd840c468,0xdfc6c468,0xd840c46a,0xdfc7c46a,0xd840c46c,0xdfc8c46c,0xd840c46e,
	0xdfc9c46e,0xd840c470,0xdfcac470,0xd840c472,0xdfcbc472,0xd840c474,0xdfccc474,0xd840c476,
	0xdfcdc476,0xd840c478,0xdfcec478,0xd840c47a,0xdfcfc47a,0xd840c47c,0xdfd0c47c,0xd840c47e,
	0xdfd1c47e,0xd840c480,0xdfd2c480,0xd840c482,0xdfd3c482,0xd840c484,0xdfd4c484,0xd840c486,
	0xdfd5c486,0xd840c488,0xdfd6c488,0xd840c48a,0xdfd7c48a,0xd840c48c,0xdfd8c48c,0xd840c48e,
	0xdfd9c48e,0xd840c490,0xdfdac490,0xd840c492,0xdfdbc492,0xd840c494,0xdfdcc494,0xd840c496,
	0xdfddc496,0xd840c498,0xdfdec498,0xd840c49a,0xdfdfc49a,0xd840c49c,0xdfe0c49c,0xd840c49e,
	0xdfe1c49e,0xd840c4a0,0xdfe2c4a0,0xd840c4a2,0xdfe3c4a2,0xd840c4a4,0xdfe4c4a4,0xd840c4a6,
	0xdfe5c4a6,0xd840c4a8,0xdfe6c4a8,0xd840c4aa,0xdfe7c4aa,0xd840c4ac,0xdfe8c4ac,0xd840c4ae,
	0xdfe9c4ae,0xd840c4b0,0xdfeac4b0,0xd840c4b2,0xdfebc4b2,0xd840c4b4,0xdfecc4b4,0xd840c4b6,
	0xdfedc4b6,0xd840c4b8,0xdfeec4b8,0xd840c4ba,0xdfefc4ba,0xd840c4bc,0xdff0c4bc,0xd840c4be,
	0xdff1c4be,0xd840c4c0,0xdff2c4c0,0xd840c4c2,0xdff3c4c2,0xd840c4c4,0xdff4c4c4,0xd840c4c6,
	0xdff5c4c6,0xd840c4c8,0xdff6c4c8,0xd840c4ca,0xdff7c4ca,0xd840c4cc,0xdff8c4cc,0xd840c4ce,
	0xdff9c4ce,0xd840c4d0,0xdffac4d0,0xd840c4d2,0xdffbc4d2,0xd840c4d4,0xdffcc4d4,0xd840c4d6,
	0xdffdc4d6,0xd840c4d8,0xdffec4d8,0xd840c4da,0xdfffc4da,0xd841c4dc,0xdc00c4dc,0xd841c4de,
	0xdc01c4de,0xd841c4e0,0xdc02c4e0,0xd841c4e2,0xdc03c4e2,0xd841c4e4,0xdc04c4e4,0xd841c4e6,
	0xdc05c4e6,0xd841c4e8,0xdc06c4e8,0xd841c4ea,0xdc07c4ea,0xd841c4ec,0xdc08c4ec,0xd841c4ee,
	0xdc09c4ee,0xd841c4f0,0xdc0ac4f0,0xd841c4f2,0xdc0bc4f2,0xd841c4f4,0xdc0cc4f4,0xd841c4f6,
	0xdc0dc4f6,0xd841c4f8,0xdc0ec4f8,0xd841c4fa,0xdc0fc4fa,0xd841c4fc,0xdc10c4fc,0xd841c4fe,
	0xdc11c4fe,0xd841c500,0xdc12c500,0xd841c502,0xdc13c502,0xd841c504,0xdc14c504,0xd841c506,
	0xdc15c506,0xd841c508,0xdc16c508,0xd841c50a,0xdc17c50a,0xd841c50c,0xdc18c50c,0xd841c50e,
	0xdc19c50e,0xd841c510,0xdc1ac510,0xd841c512,0xdc1bc512,0xd841c514,0xdc1cc514,0xd841c516,
	0xdc1dc516,0xd841c518,0xdc1ec518,0xd841c51a,0xdc1fc51a,0xd841c51c,0xdc20c51c,0xd841c51e,
	0xdc21c51e,0xd841c520,0xdc22c520,0xd841c522,0xdc23c522,0xd841c524,0xdc24c524,0xd841c526,
	0xdc25c526,0xd841c528,0xdc26c528,0xd841c52a,0xdc27c52a,0xd841c52c,0xdc28c52c,0xd841c52e,
	0xdc29c52e,0xd841c530,0xdc2ac530,0xd841c532,0xdc2bc532,0xd841c534,0xdc2cc534,0xd841c536,
	0xdc2dc536,0xd841c538,0xdc2ec538,0xd841c53a,0xdc2fc53a,0xd841c53c,0xdc30c53c,0xd841c53e,
	0xdc31c53e,0xd841c540,0xdc32c540,0xd841c542,0xdc33c542,0xd841c544,0xdc34c544,0xd841c546,
	0xdc35c546,0xd841c548,0xdc36c548,0xd841c54a,0xdc37c54a,0xd841c54c,0xdc38c54c,0xd841c54e,
	0xdc39c54e,0xd841c550,0xdc3ac550,0xd841c552,0xdc3bc552,0xd841c554,0xdc3cc554,0xd841c556,
	0xdc3dc556,0xd841c558,0xdc3ec558,0xd841c55a,0xdc3fc55a,0xd841c55c,0xdc40c55c,0xd841c55e,
	0xdc41c55e,0xd841c560,0xdc42c560,0xd841c562,0xdc43c562,0xd841c564,0xdc44c564,0xd841c566,
	0xdc45c566,0xd841c568,0xdc46c568,0xd841c56a,0xdc47c56a,0xd841c56c,0xdc48c56c,0xd841c56e,
	0xdc49c56e,0xd841c570,0xdc4ac570,0xd841c572,0xdc4bc572,0xd841c574,0xdc4cc574,0xd841c576,
	0xdc4dc576,0xd841c578,0xdc4ec578,0xd841c57a,0xdc4fc57a,0xd841c57c,0xdc50c57c,0xd841c57e,
	0xdc51c57e,0xd841c580,0xdc52c580,0xd841c582,0xdc53c582,0xd841c584,0xdc54c584,0xd841c586,
	0xdc55c586,0xd841c588,0xdc56c588,0xd841c58a,0xdc57c58a,0xd841c58c,0xdc58c58c,0xd841c58e,
	0xdc59c58e,0xd841c590,0xdc5ac590,0xd841c592,0xdc5bc592,0xd841c594,0xdc5cc594,0xd841c596,
	0xdc5dc596,0xd841c598,0xdc5ec598,0xd841c59a,0xdc5fc59a,0xd841c59c,0xdc60c59c,0xd841c59e,
	0xdc61c59e,0xd841c5a0,0xdc62c5a0,0xd841c5a2,0xdc63c5a2,0xd841c5a4,0xdc64c5a4,0xd841c5a6,
	0xdc65c5a6,0xd841c5a8,0xdc66c5a8,0xd841c5aa,0xdc67c5aa,0xd841c5ac,0xdc68c5ac,0xd841c5ae,
	0xdc69c5ae,0xd841c5b0,0xdc6ac5b0,0xd841c5b2,0xdc6bc5b2,0xd841c5b4,0xdc6cc5b4,0xd841c5b6,
	0xdc6dc5b6,0xd841c5b8,0xdc6ec5b8,0xd841c5ba,0xdc6fc5ba,0xd841c5bc,0xdc70c5bc,0xd841c5be,
	0xdc71c5be,0xd841c5c0,0xdc72c5c0,0xd841c5c2,0xdc73c5c2,0xd841c5c4,0xdc74c5c4,0xd841c5c6,
	0xdc75c5c6,0xd841c5c8,0xdc76c5c8,0xd841c5ca,0xdc77c5ca,0xd841c5cc,0xdc78c5cc,0xd841c5ce,
	0xdc79c5ce,0xd841c5d0,0xdc7ac5d0,0xd841c5d2,0xdc7bc5d2,0xd841c5d4,0xdc7cc5d4,0xd841c5d6,
	0xdc7dc5d6,0xd841c5d8,0xdc7ec5d8,0xd841c5da,0xdc7fc5da,0xd841c5dc,0xdc80c5dc,0xd841c5de,
	0xdc81c5de,0xd841c5e0,0xdc82c5e0,0xd841c5e2,0xdc83c5e2,0xd841c5e4,0xdc84c5e4,0xd841c5e6,
	0xdc85c5e6,0xd841c5e8,0xdc86c5e8,0xd841c5ea,0xdc87c5ea,0xd841c5ec,0xdc88c5ec,0xd841c5ee,
	0xdc89c5ee,0xd841c5f0,0xdc8ac5f0,0xd841c5f2,0xdc8bc5f2,0xd841c5f4,0xdc8cc5f4,0xd841c5f6,
	0xdc8dc5f6,0xd841c5f8,0xdc8ec5f8,0xd841c5fa,0xdc8fc5fa,0xd841c5fc,0xdc90c5fc,0xd841c5fe,
	0xdc91c5fe,0xd841c600,0xdc92c600,0xd841c602,0xdc93c602,0xd841c604,0xdc94c604,0xd841c606,
	0xdc95c606,0xd841c608,0xdc96c608,0xd841c60a,0xdc97c60a,0xd841c60c,0xdc98c60c,0xd841c60e,
	0xdc99c60e,0xd841c610,0xdc9ac610,0xd841c612,0xdc9bc612,0xd841c614,0xdc9cc614,0xd841c616,
	0xdc9dc616,0xd841c618,0xdc9ec618,0xd841c61a,0xdc9fc61a,0xd841c61c,0xdca0c61c,0xd841c61e,
	0xdca1c61e,0xd841c620,0xdca2c620,0xd841c622,0xdca3c622,0xd841c624,0xdca4c624,0xd841c626,
	0xdca5c626,0xd841c628,0xdca6c628,0xd841c62a,0xdca7c62a,0xd841c62c,0xdca8c62c,0xd841c62e,
	0xdca9c62e,0xd841c630,0xdcaac630,0xd841c632,0xdcabc632,0xd841c634,0xdcacc634,0xd841c636,
	0xdcadc636,0xd841c638,0xdcaec638,0xd841c63a,0xdcafc63a,0xd841c63c,0xdcb0c63c,0xd841c63e,
	0xdcb1c63e,0xd841c640,0xdcb2c640,0xd841c642,0xdcb3c642,0xd841c644,0xdcb4c644,0xd841c646,
	0xdcb5c646,0xd841c648,0xdcb6c648,0xd841c64a,0xdcb7c64a,0xd841c64c,0xdcb8c64c,0xd841c64e,
	0xdcb9c64e,0xd841c650,0xdcbac650,0xd841c652,0xdcbbc652,0xd841c654,0xdcbcc654,0xd841c656,
	0xdcbdc656,0xd841c658,0xdcbec658,0xd841c65a,0xdcbfc65a,0xd841c65c,0xdcc0c65c,0xd841c65e,
	0xdcc1c65e,0xd841c660,0xdcc2c660,0xd841c662,0xdcc3c662,0xd841c664,0xdcc4c664,0xd841c666,
	0xdcc5c666,0xd841c668,0xdcc6c668,0xd841c66a,0xdcc7c66a,0xd841c66c,0xdcc8c66c,0xd841c66e,
	0xdcc9c66e,0xd841c670,0xdccac670,0xd841c672,0xdccbc672,0xd841c674,0xdcccc674,0xd841c676,
	0xdccdc676,0xd841c678,0xdccec678,0xd841c67a,0xdccfc67a,0xd841c67c,0xdcd0c67c,0xd841c67e,
	0xdcd1c67e,0xd841c680,0xdcd2c680,0xd841c682,0xdcd3c682,0xd841c684,0xdcd4c684,0xd841c686,
	0xdcd5c686,0xd841c688,0xdcd6c688,0xd841c68a,0xdcd7c68a,0xd841c68c,0xdcd8c68c,0xd841c68e,
	0xdcd9c68e,0xd841c690,0xdcdac690,0xd841c692,0xdcdbc692,0xd841c694,0xdcdcc694,0xd841c696,
	0xdcddc696,0xd841c698,0xdcdec698,0xd841c69a,0xdcdfc69a,0xd841c69c,0xdce0c69c,0xd841c69e,
	0xdce1c69e,0xd841c6a0,0xdce2c6a0,0xd841c6a2,0xdce3c6a2,0xd841c6a4,0xdce4c6a4,0xd841c6a6,
	0xdce5c6a6,0xd841c6a8,0xdce6c6a8,0xd841c6aa,0xdce7c6aa,0xd841c6ac,0xdce8c6ac,0xd841c6ae,
	0xdce9c6ae,0xd841c6b0,0xdceac6b0,0xd841c6b2,0xdcebc6b2,0xd841c6b4,0xdcecc6b4,0xd841c6b6,
	0xdcedc6b6,0xd841c6b8,0xdceec6b8,0xd841c6ba,0xdcefc6ba,0xd841c6bc,0xdcf0c6bc,0xd841c6be,
	0xdcf1c6be,0xd841c6c0,0xdcf2c6c0,0xd841c6c2,0xdcf3c6c2,0xd841c6c4,0xdcf4c6c4,0xd841c6c6,
	0xdcf5c6c6,0xd841c6c8,0xdcf6c6c8,0xd841c6ca,0xdcf7c6ca,0xd841c6cc,0xdcf8c6cc,0xd841c6ce,
	0xdcf9c6ce,0xd841c6d0,0xdcfac6d0,0xd841c6d2,0xdcfbc6d2,0xd841c6d4,0xdcfcc6d4,0xd841c6d6,
	0xdcfdc6d6,0xd841c6d8,0xdcfec6d8,0xd841c6da,0xdcffc6da,0xd841c6dc,0xdd00c6dc,0xd841c6de,
	0xdd01c6de,0xd841c6e0,0xdd02c6e0,0xd841c6e2,0xdd03c6e2,0xd841c6e4,0xdd04c6e4,0xd841c6e6,
	0xdd05c6e6,0xd841c6e8,0xdd06c6e8,0xd841c6ea,0xdd07c6ea,0xd841c6ec,0xdd08c6ec,0xd841c6ee,
	0xdd09c6ee,0xd841c6f0,0xdd0ac6f0,0xd841c6f2,0xdd0bc6f2,0xd841c6f4,0xdd0cc6f4,0xd841c6f6,
	0xdd0dc6f6,0xd841c6f8,0xdd0ec6f8,0xd841c6fa,0xdd0fc6fa,0xd841c6fc,0xdd10c6fc,0xd841c6fe,
	0xdd11c6fe,0xd841c700,0xdd12c700,0xd841c702,0xdd13c702,0xd841c704,0xdd14c704,0xd841c706,
	0xdd15c706,0xd841c708,0xdd16c708,0xd841c70a,0xdd17c70a,0xd841c70c,0xdd18c70c,0xd841c70e,
	0xdd19c70e,0xd841c710,0xdd1ac710,0xd841c712,0xdd1bc712,0xd841c714,0xdd1cc714,0xd841c716,
	0xdd1dc716,0xd841c718,0xdd1ec718,0xd841c71a,0xdd1fc71a,0xd841c71c,0xdd20c71c,0xd841c71e,
	0xdd21c71e,0xd841c720,0xdd22c720,0xd841c722,0xdd23c722,0xd841c724,0xdd24c724,0xd841c726,
	0xdd25c726,0xd841c728,0xdd26c728,0xd841c72a,0xdd27c72a,0xd841c72c,0xdd28c72c,0xd841c72e,
	0xdd29c72e,0xd841c730,0xdd2ac730,0xd841c732,0xdd2bc732,0xd841c734,0xdd2cc734,0xd841c736,
	0xdd2dc736,0xd841c738,0xdd2ec738,0xd841c73a,0xdd2fc73a,0xd841c73c,0xdd30c73c,0xd841c73e,
	0xdd31c73e,0xd841c740,0xdd32c740,0xd841c742,0xdd33c742,0xd841c744,0xdd34c744,0xd841c746,
	0xdd35c746,0xd841c748,0xdd36c748,0xd841c74a,0xdd37c74a,0xd841c74c,0xdd38c74c,0xd841c74e,
	0xdd39c74e,0xd841c750,0xdd3ac750,0xd841c752,0xdd3bc752,0xd841c754,0xdd3cc754,0xd841c756,
	0xdd3dc756,0xd841c758,0xdd3ec758,0xd841c75a,0xdd3fc75a,0xd841c75c,0xdd40c75c,0xd841c75e,
	0xdd41c75e,0xd841c760,0xdd42c760,0xd841c762,0xdd43c762,0xd841c764,0xdd44c764,0xd841c766,
	0xdd45c766,0xd841c768,0xdd46c768,0xd841c76a,0xdd47c76a,0xd841c76c,0xdd48c76c,0xd841c76e,
	0xdd49c76e,0xd841c770,0xdd4ac770,0xd841c772,0xdd4bc772,0xd841c774,0xdd4cc774,0xd841c776,
	0xdd4dc776,0xd841c778,0xdd4ec778,0xd841c77a,0xdd4fc77a,0xd841c77c,0xdd50c77c,0xd841c77e,
	0xdd51c77e,0xd841c780,0xdd52c780,0xd841c782,0xdd53c782,0xd841c784,0xdd54c784,0xd841c786,
	0xdd55c786,0xd841c788,0xdd56c788,0xd841c78a,0xdd57c78a,0xd841c78c,0xdd58c78c,0xd841c78e,
	0xdd59c78e,0xd841c790,0xdd5ac790,0xd841c792,0xdd5bc792,0xd841c794,0xdd5cc794,0xd841c796,
	0xdd5dc796,0xd841c798,0xdd5ec798,0xd841c79a,0xdd5fc79a,0xd841c79c,0xdd60c79c,0xd841c79e,
	0xdd61c79e,0xd841c7a0,0xdd62c7a0,0xd841c7a2,0xdd63c7a2,0xd841c7a4,0xdd64c7a4,0xd841c7a6,
	0xdd65c7a6,0xd841c7a8,0xdd66c7a8,0xd841c7aa,0xdd67c7aa,0xd841c7ac,0xdd68c7ac,0xd841c7ae,
	0xdd69c7ae,0xd841c7b0,0xdd6ac7b0,0xd841c7b2,0xdd6bc7b2,0xd841c7b4,0xdd6cc7b4,0xd841c7b6,
	0xdd6dc7b6,0xd841c7b8,0xdd6ec7b8,0xd841c7ba,0xdd6fc7ba,0xd841c7bc,0xdd70c7bc,0xd841c7be,
	0xdd71c7be,0xd841c7c0,0xdd72c7c0,0xd841c7c2,0xdd73c7c2,0xd841c7c4,0xdd74c7c4,0xd841c7c6,
	0xdd75c7c6,0xd841c7c8,0xdd76c7c8,0xd841c7ca,0xdd77c7ca,0xd841c7cc,0xdd78c7cc,0xd841c7ce,
	0xdd79c7ce,0xd841c7d0,0xdd7ac7d0,0xd841c7d2,0xdd7bc7d2,0xd841c7d4,0xdd7cc7d4,0xd841c7d6,
	0xdd7dc7d6,0xd841c7d8,0xdd7ec7d8,0xd841c7da,0xdd7fc7da,0xd841c7dc,0xdd80c7dc,0xd841c7de,
	0xdd81c7de,0xd841c7e0,0xdd82c7e0,0xd841c7e2,0xdd83c7e2,0xd841c7e4,0xdd84c7e4,0xd841c7e6,
	0xdd85c7e6,0xd841c7e8,0xdd86c7e8,0xd841c7ea,0xdd87c7ea,0xd841c7ec,0xdd88c7ec,0xd841c7ee,
	0xdd89c7ee,0xd841c7f0,0xdd8ac7f0,0xd841c7f2,0xdd8bc7f2,0xd841c7f4,0xdd8cc7f4,0xd841c7f6,
	0xdd8dc7f6,0xd841c7f8,0xdd8ec7f8,0xd841c7fa,0xdd8fc7fa,0xd841c7fc,0xdd90c7fc,0xd841c7fe,
	0xdd91c7fe,0xd841c800,0xdd92c800,0xd841c802,0xdd93c802,0xd841c804,0xdd94c804,0xd841c806,
	0xdd95c806,0xd841c808,0xdd96c808,0xd841c80a,0xdd97c80a,0xd841c80c,0xdd98c80c,0xd841c80e,
	0xdd99c80e,0xd841c810,0xdd9ac810,0xd841c812,0xdd9bc812,0xd841c814,0xdd9cc814,0xd841c816,
	0xdd9dc816,0xd841c818,0xdd9ec818,0xd841c81a,0xdd9fc81a,0xd841c81c,0xdda0c81c,0xd841c81e,
	0xdda1c81e,0xd841c820,0xdda2c820,0xd841c822,0xdda3c822,0xd841c824,0xdda4c824,0xd841c826,
	0xdda5c826,0xd841c828,0xdda6c828,0xd841c82a,0xdda7c82a,0xd841c82c,0xdda8c82c,0xd841c82e,
	0xdda9c82e,0xd841c830,0xddaac830,0xd841c832,0xddabc832,0xd841c834,0xddacc834,0xd841c836,
	0xddadc836,0xd841c838,0xddaec838,0xd841c83a,0xddafc83a,0xd841c83c,0xddb0c83c,0xd841c83e,
	0xddb1c83e,0xd841c840,0xddb2c840,0xd841c842,0xddb3c842,0xd841c844,0xddb4c844,0xd841c846,
	0xddb5c846,0xd841c848,0xddb6c848,0xd841c84a,0xddb7c84a,0xd841c84c,0xddb8c84c,0xd841c84e,
	0xddb9c84e,0xd841c850,0xddbac850,0xd841c852,0xddbbc852,0xd841c854,0xddbcc854,0xd841c856,
	0xddbdc856,0xd841c858,0xddbec858,0xd841c85a,0xddbfc85a,0xd841c85c,0xddc0c85c,0xd841c85e,
	0xddc1c85e,0xd841c860,0xddc2c860,0xd841c862,0xddc3c862,0xd841c864,0xddc4c864,0xd841c866,
	0xddc5c866,0xd841c868,0xddc6c868,0xd841c86a,0xddc7c86a,0xd841c86c,0xddc8c86c,0xd841c86e,
	0xddc9c86e,0xd841c870,0xddcac870,0xd841c872,0xddcbc872,0xd841c874,0xddccc874,0xd841c876,
	0xddcdc876,0xd841c878,0xddcec878,0xd841c87a,0xddcfc87a,0xd841c87c,0xddd0c87c,0xd841c87e,
	0xddd1c87e,0xd841c880,0xddd2c880,0xd841c882,0xddd3c882,0xd841c884,0xddd4c884,0xd841c886,
	0xddd5c886,0xd841c888,0xddd6c888,0xd841c88a,0xddd7c88a,0xd841c88c,0xddd8c88c,0xd841c88e,
	0xddd9c88e,0xd841c890,0xdddac890,0xd841c892,0xdddbc892,0xd841c894,0xdddcc894,0xd841c896,
	0xddddc896,0xd841c898,0xdddec898,0xd841c89a,0xdddfc89a,0xd841c89c,0xdde0c89c,0xd841c89e,
	0xdde1c89e,0xd841c8a0,0xdde2c8a0,0xd841c8a2,0xdde3c8a2,0xd841c8a4,0xdde4c8a4,0xd841c8a6,
	0xdde5c8a6,0xd841c8a8,0xdde6c8a8,0xd841c8aa,0xdde7c8aa,0xd841c8ac,0xdde8c8ac,0xd841c8ae,
	0xdde9c8ae,0xd841c8b0,0xddeac8b0,0xd841c8b2,0xddebc8b2,0xd841c8b4,0xddecc8b4,0xd841c8b6,
	0xddedc8b6,0xd841c8b8,0xddeec8b8,0xd841c8ba,0xddefc8ba,0xd841c8bc,0xddf0c8bc,0xd841c8be,
	0xddf1c8be,0xd841c8c0,0xddf2c8c0,0xd841c8c2,0xddf3c8c2,0xd841c8c4,0xddf4c8c4,0xd841c8c6,
	0xddf5c8c6,0xd841c8c8,0xddf6c8c8,0xd841c8ca,0xddf7c8ca,0xd841c8cc,0xddf8c8cc,0xd841c8ce,
	0xddf9c8ce,0xd841c8d0,0xddfac8d0,0xd841c8d2,0xddfbc8d2,0xd841c8d4,0xddfcc8d4,0xd841c8d6,
	0xddfdc8d6,0xd841c8d8,0xddfec8d8,0xd841c8da,0xddffc8da,0xd841c8dc,0xde00c8dc,0xd841c8de,
	0xde01c8de,0xd841c8e0,0xde02c8e0,0xd841c8e2,0xde03c8e2,0xd841c8e4,0xde04c8e4,0xd841c8e6,
	0xde05c8e6,0xd841c8e8,0xde06c8e8,0xd841c8ea,0xde07c8ea,0xd841c8ec,0xde08c8ec,0xd841c8ee,
	0xde09c8ee,0xd841c8f0,0xde0ac8f0,0xd841c8f2,0xde0bc8f2,0xd841c8f4,0xde0cc8f4,0xd841c8f6,
	0xde0dc8f6,0xd841c8f8,0xde0ec8f8,0xd841c8fa,0xde0fc8fa,0xd841c8fc,0xde10c8fc,0xd841c8fe,
	0xde11c8fe,0xd841c900,0xde12c900,0xd841c902,0xde13c902,0xd841c904,0xde14c904,0xd841c906,
	0xde15c906,0xd841c908,0xde16c908,0xd841c90a,0xde17c90a,0xd841c90c,0xde18c90c,0xd841c90e,
	0xde19c90e,0xd841c910,0xde1ac910,0xd841c912,0xde1bc912,0xd841c914,0xde1cc914,0xd841c916,
	0xde1dc916,0xd841c918,0xde1ec918,0xd841c91a,0xde1fc91a,0xd841c91c,0xde20c91c,0xd841c91e,
	0xde21c91e,0xd841c920,0xde22c920,0xd841c922,0xde23c922,0xd841c924,0xde24c924,0xd841c926,
	0xde25c926,0xd841c928,0xde26c928,0xd841c92a,0xde27c92a,0xd841c92c,0xde28c92c,0xd841c92e,
	0xde29c92e,0xd841c930,0xde2ac930,0xd841c932,0xde2bc932,0xd841c934,0xde2cc934,0xd841c936,
	0xde2dc936,0xd841c938,0xde2ec938,0xd841c93a,0xde2fc93a,0xd841c93c,0xde30c93c,0xd841c93e,
	0xde31c93e,0xd841c940,0xde32c940,0xd841c942,0xde33c942,0xd841c944,0xde34c944,0xd841c946,
	0xde35c946,0xd841c948,0xde36c948,0xd841c94a,0xde37c94a,0xd841c94c,0xde38c94c,0xd841c94e,
	0xde39c94e,0xd841c950,0xde3ac950,0xd841c952,0xde3bc952,0xd841c954,0xde3cc954,0xd841c956,
	0xde3dc956,0xd841c958,0xde3ec958,0xd841c95a,0xde3fc95a,0xd841c95c,0xde40c95c,0xd841c95e,
	0xde41c95e,0xd841c960,0xde42c960,0xd841c962,0xde43c962,0xd841c964,0xde44c964,0xd841c966,
	0xde45c966,0xd841c968,0xde46c968,0xd841c96a,0xde47c96a,0xd841c96c,0xde48c96c,0xd841c96e,
	0xde49c96e,0xd841c970,0xde4ac970,0xd841c972,0xde4bc972,0xd841c974,0xde4cc974,0xd841c976,
	0xde4dc976,0xd841c978,0xde4ec978,0xd841c97a,0xde4fc97a,0xd841c97c,0xde50c97c,0xd841c97e,
	0xde51c97e,0xd841c980,0xde52c980,0xd841c982,0xde53c982,0xd841c984,0xde54c984,0xd841c986,
	0xde55c986,0xd841c988,0xde56c988,0xd841c98a,0xde57c98a,0xd841c98c,0xde58c98c,0xd841c98e,
	0xde59c98e,0xd841c990,0xde5ac990,0xd841c992,0xde5bc992,0xd841c994,0xde5cc994,0xd841c996,
	0xde5dc996,0xd841c998,0xde5ec998,0xd841c99a,0xde5fc99a,0xd841c99c,0xde60c99c,0xd841c99e,
	0xde61c99e,0xd841c9a0,0xde62c9a0,0xd841c9a2,0xde63c9a2,0xd841c9a4,0xde64c9a4,0xd841c9a6,
	0xde65c9a6,0xd841c9a8,0xde66c9a8,0xd841c9aa,0xde67c9aa,0xd841c9ac,0xde68c9ac,0xd841c9ae,
	0xde69c9ae,0xd841c9b0,0xde6ac9b0,0xd841c9b2,0xde6bc9b2,0xd841c9b4,0xde6cc9b4,0xd841c9b6,
	0xde6dc9b6,0xd841c9b8,0xde6ec9b8,0xd841c9ba,0xde6fc9ba,0xd841c9bc,0xde70c9bc,0xd841c9be,
	0xde71c9be,0xd841c9c0,0xde72c9c0,0xd841c9c2,0xde73c9c2,0xd841c9c4,0xde74c9c4,0xd841c9c6,
	0xde75c9c6,0xd841c9c8,0xde76c9c8,0xd841c9ca,0xde77c9ca,0xd841c9cc,0xde78c9cc,0xd841c9ce,
	0xde79c9ce,0xd841c9d0,0xde7ac9d0,0xd841c9d2,0xde7bc9d2,0xd841c9d4,0xde7cc9d4,0xd841c9d6,
	0xde7dc9d6,0xd841c9d8,0xde7ec9d8,0xd841c9da,0xde7fc9da,0xd841c9dc,0xde80c9dc,0xd841c9de,
	0xde81c9de,0xd841c9e0,0xde82c9e0,0xd841c9e2,0xde83c9e2,0xd841c9e4,0xde84c9e4,0xd841c9e6,
	0xde85c9e6,0xd841c9e8,0xde86c9e8,0xd841c9ea,0xde87c9ea,0xd841c9ec,0xde88c9ec,0xd841c9ee,
	0xde89c9ee,0xd841c9f0,0xde8ac9f0,0xd841c9f2,0xde8bc9f2,0xd841c9f4,0xde8cc9f4,0xd841c9f6,
	0xde8dc9f6,0xd841c9f8,0xde8ec9f8,0xd841c9fa,0xde8fc9fa,0xd841c9fc,0xde90c9fc,0xd841c9fe,
	0xde91c9fe,0xd841ca00,0xde92ca00,0xd841ca02,0xde93ca02,0xd841ca04,0xde94ca04,0xd841ca06,
	0xde95ca06,0xd841ca08,0xde96ca08,0xd841ca0a,0xde97ca0a,0xd841ca0c,0xde98ca0c,0xd841ca0e,
	0xde99ca0e,0xd841ca10,0xde9aca10,0xd841ca12,0xde9bca12,0xd841ca14,0xde9cca14,0xd841ca16,
	0xde9dca16,0xd841ca18,0xde9eca18,0xd841ca1a,0xde9fca1a,0xd841ca1c,0xdea0ca1c,0xd841ca1e,
	0xdea1ca1e,0xd841ca20,0xdea2ca20,0xd841ca22,0xdea3ca22,0xd841ca24,0xdea4ca24,0xd841ca26,
	0xdea5ca26,0xd841ca28,0xdea6ca28,0xd841ca2a,0xdea7ca2a,0xd841ca2c,0xdea8ca2c,0xd841ca2e,
	0xdea9ca2e,0xd841ca30,0xdeaaca30,0xd841ca32,0xdeabca32,0xd841ca34,0xdeacca34,0xd841ca36,
	0xdeadca36,0xd841ca38,0xdeaeca38,0xd841ca3a,0xdeafca3a,0xd841ca3c,0xdeb0ca3c,0xd841ca3e,
	0xdeb1ca3e,0xd841ca40,0xdeb2ca40,0xd841ca42,0xdeb3ca42,0xd841ca44,0xdeb4ca44,0xd841ca46,
	0xdeb5ca46,0xd841ca48,0xdeb6ca48,0xd841ca4a,0xdeb7ca4a,0xd841ca4c,0xdeb8ca4c,0xd841ca4e,
	0xdeb9ca4e,0xd841ca50,0xdebaca50,0xd841ca52,0xdebbca52,0xd841ca54,0xdebcca54,0xd841ca56,
	0xdebdca56,0xd841ca58,0xdebeca58,0xd841ca5a,0xdebfca5a,0xd841ca5c,0xdec0ca5c,0xd841ca5e,
	0xdec1ca5e,0xd841ca60,0xdec2ca60,0xd841ca62,0xdec3ca62,0xd841ca64,0xdec4ca64,0xd841ca66,
	0xdec5ca66,0xd841ca68,0xdec6ca68,0xd841ca6a,0xdec7ca6a,0xd841ca6c,0xdec8ca6c,0xd841ca6e,
	0xdec9ca6e,0xd841ca70,0xdecaca70,0xd841ca72,0xdecbca72,0xd841ca74,0xdeccca74,0xd841ca76,
	0xdecdca76,0xd841ca78,0xdececa78,0xd841ca7a,0xdecfca7a,0xd841ca7c,0xded0ca7c,0xd841ca7e,
	0xded1ca7e,0xd841ca80,0xded2ca80,0xd841ca82,0xded3ca82,0xd841ca84,0xded4ca84,0xd841ca86,
	0xded5ca86,0xd841ca88,0xded6ca88,0xd841ca8a,0xded7ca8a,0xd841ca8c,0xded8ca8c,0xd841ca8e,
	0xded9ca8e,0xd841ca90,0xdedaca90,0xd841ca92,0xdedbca92,0xd841ca94,0xdedcca94,0xd841ca96,
	0xdeddca96,0xd841ca98,0xdedeca98,0xd841ca9a,0xdedfca9a,0xd841ca9c,0xdee0ca9c,0xd841ca9e,
	0xdee1ca9e,0xd841caa0,0xdee2caa0,0xd841caa2,0xdee3caa2,0xd841caa4,0xdee4caa4,0xd841caa6,
	0xdee5caa6,0xd841caa8,0xdee6caa8,0xd841caaa,0xdee7caaa,0xd841caac,0xdee8caac,0xd841caae,
	0xdee9caae,0xd841cab0,0xdeeacab0,0xd841cab2,0xdeebcab2,0xd841cab4,0xdeeccab4,0xd841cab6,
	0xdeedcab6,0xd841cab8,0xdeeecab8,0xd841caba,0xdeefcaba,0xd841cabc,0xdef0cabc,0xd841cabe,
	0xdef1cabe,0xd841cac0,0xdef2cac0,0xd841cac2,0xdef3cac2,0xd841cac4,0xdef4cac4,0xd841cac6,
	0xdef5cac6,0xd841cac8,0xdef6cac8,0xd841caca,0xdef7caca,0xd841cacc,0xdef8cacc,0xd841cace,
	0xdef9cace,0xd841cad0,0xdefacad0,0xd841cad2,0xdefbcad2,0xd841cad4,0xdefccad4,0xd841cad6,
	0xdefdcad6,0xd841cad8,0xdefecad8,0xd841cada,0xdeffcada,0xd841cadc,0xdf00cadc,0xd841cade,
	0xdf01cade,0xd841cae0,0xdf02cae0,0xd841cae2,0xdf03cae2,0xd841cae4,0xdf04cae4,0xd841cae6,
	0xdf05cae6,0xd841cae8,0xdf06cae8,0xd841caea,0xdf07caea,0xd841caec,0xdf08caec,0xd841caee,
	0xdf09caee,0xd841caf0,0xdf0acaf0,0xd841caf2,0xdf0bcaf2,0xd841caf4,0xdf0ccaf4,0xd841caf6,
	0xdf0dcaf6,0xd841caf8,0xdf0ecaf8,0xd841cafa,0xdf0fcafa,0xd841cafc,0xdf10cafc,0xd841cafe,
	0xdf11cafe,0xd841cb00,0xdf12cb00,0xd841cb02,0xdf13cb02,0xd841cb04,0xdf14cb04,0xd841cb06,
	0xdf15cb06,0xd841cb08,0xdf16cb08,0xd841cb0a,0xdf17cb0a,0xd841cb0c,0xdf18cb0c,0xd841cb0e,
	0xdf19cb0e,0xd841cb10,0xdf1acb10,0xd841cb12,0xdf1bcb12,0xd841cb14,0xdf1ccb14,0xd841cb16,
	0xdf1dcb16,0xd841cb18,0xdf1ecb18,0xd841cb1a,0xdf1fcb1a,0xd841cb1c,0xdf20cb1c,0xd841cb1e,
	0xdf21cb1e,0xd841cb20,0xdf22cb20,0xd841cb22,0xdf23cb22,0xd841cb24,0xdf24cb24,0xd841cb26,
	0xdf25cb26,0xd841cb28,0xdf26cb28,0xd841cb2a,0xdf27cb2a,0xd841cb2c,0xdf28cb2c,0xd841cb2e,
	0xdf29cb2e,0xd841cb30,0xdf2acb30,0xd841cb32,0xdf2bcb32,0xd841cb34,0xdf2ccb34,0xd841cb36,
	0xdf2dcb36,0xd841cb38,0xdf2ecb38,0xd841cb3a,0xdf2fcb3a,0xd841cb3c,0xdf30cb3c,0xd841cb3e,
	0xdf31cb3e,0xd841cb40,0xdf32cb40,0xd841cb42,0xdf33cb42,0xd841cb44,0xdf34cb44,0xd841cb46,
	0xdf35cb46,0xd841cb48,0xdf36cb48,0xd841cb4a,0xdf37cb4a,0xd841cb4c,0xdf38cb4c,0xd841cb4e,
	0xdf39cb4e,0xd841cb50,0xdf3acb50,0xd841cb52,0xdf3bcb52,0xd841cb54,0xdf3ccb54,0xd841cb56,
	0xdf3dcb56,0xd841cb58,0xdf3ecb58,0xd841cb5a,0xdf3fcb5a,0xd841cb5c,0xdf40cb5c,0xd841cb5e,
	0xdf41cb5e,0xd841cb60,0xdf42cb60,0xd841cb62,0xdf43cb62,0xd841cb64,0xdf44cb64,0xd841cb66,
	0xdf45cb66,0xd841cb68,0xdf46cb68,0xd841cb6a,0xdf47cb6a,0xd841cb6c,0xdf48cb6c,0xd841cb6e,
	0xdf49cb6e,0xd841cb70,0xdf4acb70,0xd841cb72,0xdf4bcb72,0xd841cb74,0xdf4ccb74,0xd841cb76,
	0xdf4dcb76,0xd841cb78,0xdf4ecb78,0xd841cb7a,0xdf4fcb7a,0xd841cb7c,0xdf50cb7c,0xd841cb7e,
	0xdf51cb7e,0xd841cb80,0xdf52cb80,0xd841cb82,0xdf53cb82,0xd841cb84,0xdf54cb84,0xd841cb86,
	0xdf55cb86,0xd841cb88,0xdf56cb88,0xd841cb8a,0xdf57cb8a,0xd841cb8c,0xdf58cb8c,0xd841cb8e,
	0xdf59cb8e,0xd841cb90,0xdf5acb90,0xd841cb92,0xdf5bcb92,0xd841cb94,0xdf5ccb94,0xd841cb96,
	0xdf5dcb96,0xd841cb98,0xdf5ecb98,0xd841cb9a,0xdf5fcb9a,0xd841cb9c,0xdf60cb9c,0xd841cb9e,
	0xdf61cb9e,0xd841cba0,0xdf62cba0,0xd841cba2,0xdf63cba2,0xd841cba4,0xdf64cba4,0xd841cba6,
	0xdf65cba6,0xd841cba8,0xdf66cba8,0xd841cbaa,0xdf67cbaa,0xd841cbac,0xdf68cbac,0xd841cbae,
	0xdf69cbae,0xd841cbb0,0xdf6acbb0,0xd841cbb2,0xdf6bcbb2,0xd841cbb4,0xdf6ccbb4,0xd841cbb6,
	0xdf6dcbb6,0xd841cbb8,0xdf6ecbb8,0xd841cbba,0xdf6fcbba,0xd841cbbc,0xdf70cbbc,0xd841cbbe,
	0xdf71cbbe,0xd841cbc0,0xdf72cbc0,0xd841cbc2,0xdf73cbc2,0xd841cbc4,0xdf74cbc4,0xd841cbc6,
	0xdf75cbc6,0xd841cbc8,0xdf76cbc8,0xd841cbca,0xdf77cbca,0xd841cbcc,0xdf78cbcc,0xd841cbce,
	0xdf79cbce,0xd841cbd0,0xdf7acbd0,0xd841cbd2,0xdf7bcbd2,0xd841cbd4,0xdf7ccbd4,0xd841cbd6,
	0xdf7dcbd6,0xd841cbd8,0xdf7ecbd8,0xd841cbda,0xdf7fcbda,0xd841cbdc,0xdf80cbdc,0xd841cbde,
	0xdf81cbde,0xd841cbe0,0xdf82cbe0,0xd841cbe2,0xdf83cbe2,0xd841cbe4,0xdf84cbe4,0xd841cbe6,
	0xdf85cbe6,0xd841cbe8,0xdf86cbe8,0xd841cbea,0xdf87cbea,0xd841cbec,0xdf88cbec,0xd841cbee,
	0xdf89cbee,0xd841cbf0,0xdf8acbf0,0xd841cbf2,0xdf8bcbf2,0xd841cbf4,0xdf8ccbf4,0xd841cbf6,
	0xdf8dcbf6,0xd841cbf8,0xdf8ecbf8,0xd841cbfa,0xdf8fcbfa,0xd841cbfc,0xdf90cbfc,0xd841cbfe,
	0xdf91cbfe,0xd841cc00,0xdf92cc00,0xd841cc02,0xdf93cc02,0xd841cc04,0xdf94cc04,0xd841cc06,
	0xdf95cc06,0xd841cc08,0xdf96cc08,0xd841cc0a,0xdf97cc0a,0xd841cc0c,0xdf98cc0c,0xd841cc0e,
	0xdf99cc0e,0xd841cc10,0xdf9acc10,0xd841cc12,0xdf9bcc12,0xd841cc14,0xdf9ccc14,0xd841cc16,
	0xdf9dcc16,0xd841cc18,0xdf9ecc18,0xd841cc1a,0xdf9fcc1a,0xd841cc1c,0xdfa0cc1c,0xd841cc1e,
	0xdfa1cc1e,0xd841cc20,0xdfa2cc20,0xd841cc22,0xdfa3cc22,0xd841cc24,0xdfa4cc24,0xd841cc26,
	0xdfa5cc26,0xd841cc28,0xdfa6cc28,0xd841cc2a,0xdfa7cc2a,0xd841cc2c,0xdfa8cc2c,0xd841cc2e,
	0xdfa9cc2e,0xd841cc30,0xdfaacc30,0xd841cc32,0xdfabcc32,0xd841cc34,0xdfaccc34,0xd841cc36,
	0xdfadcc36,0xd841cc38,0xdfaecc38,0xd841cc3a,0xdfafcc3a,0xd841cc3c,0xdfb0cc3c,0xd841cc3e,
	0xdfb1cc3e,0xd841cc40,0xdfb2cc40,0xd841cc42,0xdfb3cc42,0xd841cc44,0xdfb4cc44,0xd841cc46,
	0xdfb5cc46,0xd841cc48,0xdfb6cc48,0xd841cc4a,0xdfb7cc4a,0xd841cc4c,0xdfb8cc4c,0xd841cc4e,
	0xdfb9cc4e,0xd841cc50,0xdfbacc50,0xd841cc52,0xdfbbcc52,0xd841cc54,0xdfbccc54,0xd841cc56,
	0xdfbdcc56,0xd841cc58,0xdfbecc58,0xd841cc5a,0xdfbfcc5a,0xd841cc5c,0xdfc0cc5c,0xd841cc5e,
	0xdfc1cc5e,0xd841cc60,0xdfc2cc60,0xd841cc62,0xdfc3cc62,0xd841cc64,0xdfc4cc64,0xd841cc66,
	0xdfc5cc66,0xd841cc68,0xdfc6cc68,0xd841cc6a,0xdfc7cc6a,0xd841cc6c,0xdfc8cc6c,0xd841cc6e,
	0xdfc9cc6e,0xd841cc70,0xdfcacc70,0xd841cc72,0xdfcbcc72,0xd841cc74,0xdfcccc74,0xd841cc76,
	0xdfcdcc76,0xd841cc78,0xdfcecc78,0xd841cc7a,0xdfcfcc7a,0xd841cc7c,0xdfd0cc7c,0xd841cc7e,
	0xdfd1cc7e,0xd841cc80,0xdfd2cc80,0xd841cc82,0xdfd3cc82,0xd841cc84,0xdfd4cc84,0xd841cc86,
	0xdfd5cc86,0xd841cc88,0xdfd6cc88,0xd841cc8a,0xdfd7cc8a,0xd841cc8c,0xdfd8cc8c,0xd841cc8e,
	0xdfd9cc8e,0xd841cc90,0xdfdacc90,0xd841cc92,0xdfdbcc92,0xd841cc94,0xdfdccc94,0xd841cc96,
	0xdfddcc96,0xd841cc98,0xdfdecc98,0xd841cc9a,0xdfdfcc9a,0xd841cc9c,0xdfe0cc9c,0xd841cc9e,
	0xdfe1cc9e,0xd841cca0,0xdfe2cca0,0xd841cca2,0xdfe3cca2,0xd841cca4,0xdfe4cca4,0xd841cca6,
	0xdfe5cca6,0xd841cca8,0xdfe6cca8,0xd841ccaa,0xdfe7ccaa,0xd841ccac,0xdfe8ccac,0xd841ccae,
	0xdfe9ccae,0xd841ccb0,0xdfeaccb0,0xd841ccb2,0xdfebccb2,0xd841ccb4,0xdfecccb4,0xd841ccb6,
	0xdfedccb6,0xd841ccb8,0xdfeeccb8,0xd841ccba,0xdfefccba,0xd841ccbc,0xdff0ccbc,0xd841ccbe,
	0xdff1ccbe,0xd841ccc0,0xdff2ccc0,0xd841ccc2,0xdff3ccc2,0xd841ccc4,0xdff4ccc4,0xd841ccc6,
	0xdff5ccc6,0xd841ccc8,0xdff6ccc8,0xd841ccca,0xdff7ccca,0xd841cccc,0xdff8cccc,0xd841ccce,
	0xdff9ccce,0xd841ccd0,0xdffaccd0,0xd841ccd2,0xdffbccd2,0xd841ccd4,0xdffcccd4,0xd841ccd6,
	0xdffdccd6,0xd841ccd8,0xdffeccd8,0xd841ccda,0xdfffccda,0xd842ccdc,0xdc00ccdc,0xd842ccde,
	0xdc01ccde,0xd842cce0,0xdc02cce0,0xd842cce2,0xdc03cce2,0xd842cce4,0xdc04cce4,0xd842cce6,
	0xdc05cce6,0xd842cce8,0xdc06cce8,0xd842ccea,0xdc07ccea,0xd842ccec,0xdc08ccec,0xd842ccee,
	0xdc09ccee,0xd842ccf0,0xdc0accf0,0xd842ccf2,0xdc0bccf2,0xd842ccf4,0xdc0cccf4,0xd842ccf6,
	0xdc0dccf6,0xd842ccf8,0xdc0eccf8,0xd842ccfa,0xdc0fccfa,0xd842ccfc,0xdc10ccfc,0xd842ccfe,
	0xdc11ccfe,0xd842cd00,0xdc12cd00,0xd842cd02,0xdc13cd02,0xd842cd04,0xdc14cd04,0xd842cd06,
	0xdc15cd06,0xd842cd08,0xdc16cd08,0xd842cd0a,0xdc17cd0a,0xd842cd0c,0xdc18cd0c,0xd842cd0e,
	0xdc19cd0e,0xd842cd10,0xdc1acd10,0xd842cd12,0xdc1bcd12,0xd842cd14,0xdc1ccd14,0xd842cd16,
	0xdc1dcd16,0xd842cd18,0xdc1ecd18,0xd842cd1a,0xdc1fcd1a,0xd842cd1c,0xdc20cd1c,0xd842cd1e,
	0xdc21cd1e,0xd842cd20,0xdc22cd20,0xd842cd22,0xdc23cd22,0xd842cd24,0xdc24cd24,0xd842cd26,
	0xdc25cd26,0xd842cd28,0xdc26cd28,0xd842cd2a,0xdc27cd2a,0xd842cd2c,0xdc28cd2c,0xd842cd2e,
	0xdc29cd2e,0xd842cd30,0xdc2acd30,0xd842cd32,0xdc2bcd32,0xd842cd34,0xdc2ccd34,0xd842cd36,
	0xdc2dcd36,0xd842cd38,0xdc2ecd38,0xd842cd3a,0xdc2fcd3a,0xd842cd3c,0xdc30cd3c,0xd842cd3e,
	0xdc31cd3e,0xd842cd40,0xdc32cd40,0xd842cd42,0xdc33cd42,0xd842cd44,0xdc34cd44,0xd842cd46,
	0xdc35cd46,0xd842cd48,0xdc36cd48,0xd842cd4a,0xdc37cd4a,0xd842cd4c,0xdc38cd4c,0xd842cd4e,
	0xdc39cd4e,0xd842cd50,0xdc3acd50,0xd842cd52,0xdc3bcd52,0xd842cd54,0xdc3ccd54,0xd842cd56,
	0xdc3dcd56,0xd842cd58,0xdc3ecd58,0xd842cd5a,0xdc3fcd5a,0xd842cd5c,0xdc40cd5c,0xd842cd5e,
	0xdc41cd5e,0xd842cd60,0xdc42cd60,0xd842cd62,0xdc43cd62,0xd842cd64,0xdc44cd64,0xd842cd66,
	0xdc45cd66,0xd842cd68,0xdc46cd68,0xd842cd6a,0xdc47cd6a,0xd842cd6c,0xdc48cd6c,0xd842cd6e,
	0xdc49cd6e,0xd842cd70,0xdc4acd70,0xd842cd72,0xdc4bcd72,0xd842cd74,0xdc4ccd74,0xd842cd76,
	0xdc4dcd76,0xd842cd78,0xdc4ecd78,0xd842cd7a,0xdc4fcd7a,0xd842cd7c,0xdc50cd7c,0xd842cd7e,
	0xdc51cd7e,0xd842cd80,0xdc52cd80,0xd842cd82,0xdc53cd82,0xd842cd84,0xdc54cd84,0xd842cd86,
	0xdc55cd86,0xd842cd88,0xdc56cd88,0xd842cd8a,0xdc57cd8a,0xd842cd8c,0xdc58cd8c,0xd842cd8e,
	0xdc59cd8e,0xd842cd90,0xdc5acd90,0xd842cd92,0xdc5bcd92,0xd842cd94,0xdc5ccd94,0xd842cd96,
	0xdc5dcd96,0xd842cd98,0xdc5ecd98,0xd842cd9a,0xdc5fcd9a,0xd842cd9c,0xdc60cd9c,0xd842cd9e,
	0xdc61cd9e,0xd842cda0,0xdc62cda0,0xd842cda2,0xdc63cda2,0xd842cda4,0xdc64cda4,0xd842cda6,
	0xdc65cda6,0xd842cda8,0xdc66cda8,0xd842cdaa,0xdc67cdaa,0xd842cdac,0xdc68cdac,0xd842cdae,
	0xdc69cdae,0xd842cdb0,0xdc6acdb0,0xd842cdb2,0xdc6bcdb2,0xd842cdb4,0xdc6ccdb4,0xd842cdb6,
	0xdc6dcdb6,0xd842cdb8,0xdc6ecdb8,0xd842cdba,0xdc6fcdba,0xd842cdbc,0xdc70cdbc,0xd842cdbe,
	0xdc71cdbe,0xd842cdc0,0xdc72cdc0,0xd842cdc2,0xdc73cdc2,0xd842cdc4,0xdc74cdc4,0xd842cdc6,
	0xdc75cdc6,0xd842cdc8,0xdc76cdc8,0xd842cdca,0xdc77cdca,0xd842cdcc,0xdc78cdcc,0xd842cdce,
	0xdc79cdce,0xd842cdd0,0xdc7acdd0,0xd842cdd2,0xdc7bcdd2,0xd842cdd4,0xdc7ccdd4,0xd842cdd6,
	0xdc7dcdd6,0xd842cdd8,0xdc7ecdd8,0xd842cdda,0xdc7fcdda,0xd842cddc,0xdc80cddc,0xd842cdde,
	0xdc81cdde,0xd842cde0,0xdc82cde0,0xd842cde2,0xdc83cde2,0xd842cde4,0xdc84cde4,0xd842cde6,
	0xdc85cde6,0xd842cde8,0xdc86cde8,0xd842cdea,0xdc87cdea,0xd842cdec,0xdc88cdec,0xd842cdee,
	0xdc89cdee,0xd842cdf0,0xdc8acdf0,0xd842cdf2,0xdc8bcdf2,0xd842cdf4,0xdc8ccdf4,0xd842cdf6,
	0xdc8dcdf6,0xd842cdf8,0xdc8ecdf8,0xd842cdfa,0xdc8fcdfa,0xd842cdfc,0xdc90cdfc,0xd842cdfe,
	0xdc91cdfe,0xd842ce00,0xdc92ce00,0xd842ce02,0xdc93ce02,0xd842ce04,0xdc94ce04,0xd842ce06,
	0xdc95ce06,0xd842ce08,0xdc96ce08,0xd842ce0a,0xdc97ce0a,0xd842ce0c,0xdc98ce0c,0xd842ce0e,
	0xdc99ce0e,0xd842ce10,0xdc9ace10,0xd842ce12,0xdc9bce12,0xd842ce14,0xdc9cce14,0xd842ce16,
	0xdc9dce16,0xd842ce18,0xdc9ece18,0xd842ce1a,0xdc9fce1a,0xd842ce1c,0xdca0ce1c,0xd842ce1e,
	0xdca1ce1e,0xd842ce20,0xdca2ce20,0xd842ce22,0xdca3ce22,0xd842ce24,0xdca4ce24,0xd842ce26,
	0xdca5ce26,0xd842ce28,0xdca6ce28,0xd842ce2a,0xdca7ce2a,0xd842ce2c,0xdca8ce2c,0xd842ce2e,
	0xdca9ce2e,0xd842ce30,0xdcaace30,0xd842ce32,0xdcabce32,0xd842ce34,0xdcacce34,0xd842ce36,
	0xdcadce36,0xd842ce38,0xdcaece38,0xd842ce3a,0xdcafce3a,0xd842ce3c,0xdcb0ce3c,0xd842ce3e,
	0xdcb1ce3e,0xd842ce40,0xdcb2ce40,0xd842ce42,0xdcb3ce42,0xd842ce44,0xdcb4ce44,0xd842ce46,
	0xdcb5ce46,0xd842ce48,0xdcb6ce48,0xd842ce4a,0xdcb7ce4a,0xd842ce4c,0xdcb8ce4c,0xd842ce4e,
	0xdcb9ce4e,0xd842ce50,0xdcbace50,0xd842ce52,0xdcbbce52,0xd842ce54,0xdcbcce54,0xd842ce56,
	0xdcbdce56,0xd842ce58,0xdcbece58,0xd842ce5a,0xdcbfce5a,0xd842ce5c,0xdcc0ce5c,0xd842ce5e,
	0xdcc1ce5e,0xd842ce60,0xdcc2ce60,0xd842ce62,0xdcc3ce62,0xd842ce64,0xdcc4ce64,0xd842ce66,
	0xdcc5ce66,0xd842ce68,0xdcc6ce68,0xd842ce6a,0xdcc7ce6a,0xd842ce6c,0xdcc8ce6c,0xd842ce6e,
	0xdcc9ce6e,0xd842ce70,0xdccace70,0xd842ce72,0xdccbce72,0xd842ce74,0xdcccce74,0xd842ce76,
	0xdccdce76,0xd842ce78,0xdccece78,0xd842ce7a,0xdccfce7a,0xd842ce7c,0xdcd0ce7c,0xd842ce7e,
	0xdcd1ce7e,0xd842ce80,0xdcd2ce80,0xd842ce82,0xdcd3ce82,0xd842ce84,0xdcd4ce84,0xd842ce86,
	0xdcd5ce86,0xd842ce88,0xdcd6ce88,0xd842ce8a,0xdcd7ce8a,0xd842ce8c,0xdcd8ce8c,0xd842ce8e,
	0xdcd9ce8e,0xd842ce90,0xdcdace90,0xd842ce92,0xdcdbce92,0xd842ce94,0xdcdcce94,0xd842ce96,
	0xdcddce96,0xd842ce98,0xdcdece98,0xd842ce9a,0xdcdfce9a,0xd842ce9c,0xdce0ce9c,0xd842ce9e,
	0xdce1ce9e,0xd842cea0,0xdce2cea0,0xd842cea2,0xdce3cea2,0xd842cea4,0xdce4cea4,0xd842cea6,
	0xdce5cea6,0xd842cea8,0xdce6cea8,0xd842ceaa,0xdce7ceaa,0xd842ceac,0xdce8ceac,0xd842ceae,
	0xdce9ceae,0xd842ceb0,0xdceaceb0,0xd842ceb2,0xdcebceb2,0xd842ceb4,0xdcecceb4,0xd842ceb6,
	0xdcedceb6,0xd842ceb8,0xdceeceb8,0xd842ceba,0xdcefceba,0xd842cebc,0xdcf0cebc,0xd842cebe,
	0xdcf1cebe,0xd842cec0,0xdcf2cec0,0xd842cec2,0xdcf3cec2,0xd842cec4,0xdcf4cec4,0xd842cec6,
	0xdcf5cec6,0xd842cec8,0xdcf6cec8,0xd842ceca,0xdcf7ceca,0xd842cecc,0xdcf8cecc,0xd842cece,
	0xdcf9cece,0xd842ced0,0xdcfaced0,0xd842ced2,0xdcfbced2,0xd842ced4,0xdcfcced4,0xd842ced6,
	0xdcfdced6,0xd842ced8,0xdcfeced8,0xd842ceda,0xdcffceda,0xd842cedc,0xdd00cedc,0xd842cede,
	0xdd01cede,0xd842cee0,0xdd02cee0,0xd842cee2,0xdd03cee2,0xd842cee4,0xdd04cee4,0xd842cee6,
	0xdd05cee6,0xd842cee8,0xdd06cee8,0xd842ceea,0xdd07ceea,0xd842ceec,0xdd08ceec,0xd842ceee,
	0xdd09ceee,0xd842cef0,0xdd0acef0,0xd842cef2,0xdd0bcef2,0xd842cef4,0xdd0ccef4,0xd842cef6,
	0xdd0dcef6,0xd842cef8,0xdd0ecef8,0xd842cefa,0xdd0fcefa,0xd842cefc,0xdd10cefc,0xd842cefe,
	0xdd11cefe,0xd842cf00,0xdd12cf00,0xd842cf02,0xdd13cf02,0xd842cf04,0xdd14cf04,0xd842cf06,
	0xdd15cf06,0xd842cf08,0xdd16cf08,0xd842cf0a,0xdd17cf0a,0xd842cf0c,0xdd18cf0c,0xd842cf0e,
	0xdd19cf0e,0xd842cf10,0xdd1acf10,0xd842cf12,0xdd1bcf12,0xd842cf14,0xdd1ccf14,0xd842cf16,
	0xdd1dcf16,0xd842cf18,0xdd1ecf18,0xd842cf1a,0xdd1fcf1a,0xd842cf1c,0xdd20cf1c,0xd842cf1e,
	0xdd21cf1e,0xd842cf20,0xdd22cf20,0xd842cf22,0xdd23cf22,0xd842cf24,0xdd24cf24,0xd842cf26,
	0xdd25cf26,0xd842cf28,0xdd26cf28,0xd842cf2a,0xdd27cf2a,0xd842cf2c,0xdd28cf2c,0xd842cf2e,
	0xdd29cf2e,0xd842cf30,0xdd2acf30,0xd842cf32,0xdd2bcf32,0xd842cf34,0xdd2ccf34,0xd842cf36,
	0xdd2dcf36,0xd842cf38,0xdd2ecf38,0xd842cf3a,0xdd2fcf3a,0xd842cf3c,0xdd30cf3c,0xd842cf3e,
	0xdd31cf3e,0xd842cf40,0xdd32cf40,0xd842cf42,0xdd33cf42,0xd842cf44,0xdd34cf44,0xd842cf46,
	0xdd35cf46,0xd842cf48,0xdd36cf48,0xd842cf4a,0xdd37cf4a,0xd842cf4c,0xdd38cf4c,0xd842cf4e,
	0xdd39cf4e,0xd842cf50,0xdd3acf50,0xd842cf52,0xdd3bcf52,0xd842cf54,0xdd3ccf54,0xd842cf56,
	0xdd3dcf56,0xd842cf58,0xdd3ecf58,0xd842cf5a,0xdd3fcf5a,0xd842cf5c,0xdd40cf5c,0xd842cf5e,
	0xdd41cf5e,0xd842cf60,0xdd42cf60,0xd842cf62,0xdd43cf62,0xd842cf64,0xdd44cf64,0xd842cf66,
	0xdd45cf66,0xd842cf68,0xdd46cf68,0xd842cf6a,0xdd47cf6a,0xd842cf6c,0xdd48cf6c,0xd842cf6e,
	0xdd49cf6e,0xd842cf70,0xdd4acf70,0xd842cf72,0xdd4bcf72,0xd842cf74,0xdd4ccf74,0xd842cf76,
	0xdd4dcf76,0xd842cf78,0xdd4ecf78,0xd842cf7a,0xdd4fcf7a,0xd842cf7c,0xdd50cf7c,0xd842cf7e,
	0xdd51cf7e,0xd842cf80,0xdd52cf80,0xd842cf82,0xdd53cf82,0xd842cf84,0xdd54cf84,0xd842cf86,
	0xdd55cf86,0xd842cf88,0xdd56cf88,0xd842cf8a,0xdd57cf8a,0xd842cf8c,0xdd58cf8c,0xd842cf8e,
	0xdd59cf8e,0xd842cf90,0xdd5acf90,0xd842cf92,0xdd5bcf92,0xd842cf94,0xdd5ccf94,0xd842cf96,
	0xdd5dcf96,0xd842cf98,0xdd5ecf98,0xd842cf9a,0xdd5fcf9a,0xd842cf9c,0xdd60cf9c,0xd842cf9e,
	0xdd61cf9e,0xd842cfa0,0xdd62cfa0,0xd842cfa2,0xdd63cfa2,0xd842cfa4,0xdd64cfa4,0xd842cfa6,
	0xdd65cfa6,0xd842cfa8,0xdd66cfa8,0xd842cfaa,0xdd67cfaa,0xd842cfac,0xdd68cfac,0xd842cfae,
	0xdd69cfae,0xd842cfb0,0xdd6acfb0,0xd842cfb2,0xdd6bcfb2,0xd842cfb4,0xdd6ccfb4,0xd842cfb6,
	0xdd6dcfb6,0xd842cfb8,0xdd6ecfb8,0xd842cfba,0xdd6fcfba,0xd842cfbc,0xdd70cfbc,0xd842cfbe,
	0xdd71cfbe,0xd842cfc0,0xdd72cfc0,0xd842cfc2,0xdd73cfc2,0xd842cfc4,0xdd74cfc4,0xd842cfc6,
	0xdd75cfc6,0xd842cfc8,0xdd76cfc8,0xd842cfca,0xdd77cfca,0xd842cfcc,0xdd78cfcc,0xd842cfce,
	0xdd79cfce,0xd842cfd0,0xdd7acfd0,0xd842cfd2,0xdd7bcfd2,0xd842cfd4,0xdd7ccfd4,0xd842cfd6,
	0xdd7dcfd6,0xd842cfd8,0xdd7ecfd8,0xd842cfda,0xdd7fcfda,0xd842cfdc,0xdd80cfdc,0xd842cfde,
	0xdd81cfde,0xd842cfe0,0xdd82cfe0,0xd842cfe2,0xdd83cfe2,0xd842cfe4,0xdd84cfe4,0xd842cfe6,
	0xdd85cfe6,0xd842cfe8,0xdd86cfe8,0xd842cfea,0xdd87cfea,0xd842cfec,0xdd88cfec,0xd842cfee,
	0xdd89cfee,0xd842cff0,0xdd8acff0,0xd842cff2,0xdd8bcff2,0xd842cff4,0xdd8ccff4,0xd842cff6,
	0xdd8dcff6,0xd842cff8,0xdd8ecff8,0xd842cffa,0xdd8fcffa,0xd842cffc,0xdd90cffc,0xd842cffe,
	0xdd91cffe,0xd842d000,0xdd92d000,0xd842d002,0xdd93d002,0xd842d004,0xdd94d004,0xd842d006,
	0xdd95d006,0xd842d008,0xdd96d008,0xd842d00a,0xdd97d00a,0xd842d00c,0xdd98d00c,0xd842d00e,
	0xdd99d00e,0xd842d010,0xdd9ad010,0xd842d012,0xdd9bd012,0xd842d014,0xdd9cd014,0xd842d016,
	0xdd9dd016,0xd842d018,0xdd9ed018,0xd842d01a,0xdd9fd01a,0xd842d01c,0xdda0d01c,0xd842d01e,
	0xdda1d01e,0xd842d020,0xdda2d020,0xd842d022,0xdda3d022,0xd842d024,0xdda4d024,0xd842d026,
	0xdda5d026,0xd842d028,0xdda6d028,0xd842d02a,0xdda7d02a,0xd842d02c,0xdda8d02c,0xd842d02e,
	0xdda9d02e,0xd842d030,0xddaad030,0xd842d032,0xddabd032,0xd842d034,0xddacd034,0xd842d036,
	0xddadd036,0xd842d038,0xddaed038,0xd842d03a,0xddafd03a,0xd842d03c,0xddb0d03c,0xd842d03e,
	0xddb1d03e,0xd842d040,0xddb2d040,0xd842d042,0xddb3d042,0xd842d044,0xddb4d044,0xd842d046,
	0xddb5d046,0xd842d048,0xddb6d048,0xd842d04a,0xddb7d04a,0xd842d04c,0xddb8d04c,0xd842d04e,
	0xddb9d04e,0xd842d050,0xddbad050,0xd842d052,0xddbbd052,0xd842d054,0xddbcd054,0xd842d056,
	0xddbdd056,0xd842d058,0xddbed058,0xd842d05a,0xddbfd05a,0xd842d05c,0xddc0d05c,0xd842d05e,
	0xddc1d05e,0xd842d060,0xddc2d060,0xd842d062,0xddc3d062,0xd842d064,0xddc4d064,0xd842d066,
	0xddc5d066,0xd842d068,0xddc6d068,0xd842d06a,0xddc7d06a,0xd842d06c,0xddc8d06c,0xd842d06e,
	0xddc9d06e,0xd842d070,0xddcad070,0xd842d072,0xddcbd072,0xd842d074,0xddccd074,0xd842d076,
	0xddcdd076,0xd842d078,0xddced078,0xd842d07a,0xddcfd07a,0xd842d07c,0xddd0d07c,0xd842d07e,
	0xddd1d07e,0xd842d080,0xddd2d080,0xd842d082,0xddd3d082,0xd842d084,0xddd4d084,0xd842d086,
	0xddd5d086,0xd842d088,0xddd6d088,0xd842d08a,0xddd7d08a,0xd842d08c,0xddd8d08c,0xd842d08e,
	0xddd9d08e,0xd842d090,0xdddad090,0xd842d092,0xdddbd092,0xd842d094,0xdddcd094,0xd842d096,
	0xddddd096,0xd842d098,0xddded098,0xd842d09a,0xdddfd09a,0xd842d09c,0xdde0d09c,0xd842d09e,
	0xdde1d09e,0xd842d0a0,0xdde2d0a0,0xd842d0a2,0xdde3d0a2,0xd842d0a4,0xdde4d0a4,0xd842d0a6,
	0xdde5d0a6,0xd842d0a8,0xdde6d0a8,0xd842d0aa,0xdde7d0aa,0xd842d0ac,0xdde8d0ac,0xd842d0ae,
	0xdde9d0ae,0xd842d0b0,0xddead0b0,0xd842d0b2,0xddebd0b2,0xd842d0b4,0xddecd0b4,0xd842d0b6,
	0xddedd0b6,0xd842d0b8,0xddeed0b8,0xd842d0ba,0xddefd0ba,0xd842d0bc,0xddf0d0bc,0xd842d0be,
	0xddf1d0be,0xd842d0c0,0xddf2d0c0,0xd842d0c2,0xddf3d0c2,0xd842d0c4,0xddf4d0c4,0xd842d0c6,
	0xddf5d0c6,0xd842d0c8,0xddf6d0c8,0xd842d0ca,0xddf7d0ca,0xd842d0cc,0xddf8d0cc,0xd842d0ce,
	0xddf9d0ce,0xd842d0d0,0xddfad0d0,0xd842d0d2,0xddfbd0d2,0xd842d0d4,0xddfcd0d4,0xd842d0d6,
	0xddfdd0d6,0xd842d0d8,0xddfed0d8,0xd842d0da,0xddffd0da,0xd842d0dc,0xde00d0dc,0xd842d0de,
	0xde01d0de,0xd842d0e0,0xde02d0e0,0xd842d0e2,0xde03d0e2,0xd842d0e4,0xde04d0e4,0xd842d0e6,
	0xde05d0e6,0xd842d0e8,0xde06d0e8,0xd842d0ea,0xde07d0ea,0xd842d0ec,0xde08d0ec,0xd842d0ee,
	0xde09d0ee,0xd842d0f0,0xde0ad0f0,0xd842d0f2,0xde0bd0f2,0xd842d0f4,0xde0cd0f4,0xd842d0f6,
	0xde0dd0f6,0xd842d0f8,0xde0ed0f8,0xd842d0fa,0xde0fd0fa,0xd842d0fc,0xde10d0fc,0xd842d0fe,
	0xde11d0fe,0xd842d100,0xde12d100,0xd842d102,0xde13d102,0xd842d104,0xde14d104,0xd842d106,
	0xde15d106,0xd842d108,0xde16d108,0xd842d10a,0xde17d10a,0xd842d10c,0xde18d10c,0xd842d10e,
	0xde19d10e,0xd842d110,0xde1ad110,0xd842d112,0xde1bd112,0xd842d114,0xde1cd114,0xd842d116,
	0xde1dd116,0xd842d118,0xde1ed118,0xd842d11a,0xde1fd11a,0xd842d11c,0xde20d11c,0xd842d11e,
	0xde21d11e,0xd842d120,0xde22d120,0xd842d122,0xde23d122,0xd842d124,0xde24d124,0xd842d126,
	0xde25d126,0xd842d128,0xde26d128,0xd842d12a,0xde27d12a,0xd842d12c,0xde28d12c,0xd842d12e,
	0xde29d12e,0xd842d130,0xde2ad130,0xd842d132,0xde2bd132,0xd842d134,0xde2cd134,0xd842d136,
	0xde2dd136,0xd842d138,0xde2ed138,0xd842d13a,0xde2fd13a,0xd842d13c,0xde30d13c,0xd842d13e,
	0xde31d13e,0xd842d140,0xde32d140,0xd842d142,0xde33d142,0xd842d144,0xde34d144,0xd842d146,
	0xde35d146,0xd842d148,0xde36d148,0xd842d14a,0xde37d14a,0xd842d14c,0xde38d14c,0xd842d14e,
	0xde39d14e,0xd842d150,0xde3ad150,0xd842d152,0xde3bd152,0xd842d154,0xde3cd154,0xd842d156,
	0xde3dd156,0xd842d158,0xde3ed158,0xd842d15a,0xde3fd15a,0xd842d15c,0xde40d15c,0xd842d15e,
	0xde41d15e,0xd842d160,0xde42d160,0xd842d162,0xde43d162,0xd842d164,0xde44d164,0xd842d166,
	0xde45d166,0xd842d168,0xde46d168,0xd842d16a,0xde47d16a,0xd842d16c,0xde48d16c,0xd842d16e,
	0xde49d16e,0xd842d170,0xde4ad170,0xd842d172,0xde4bd172,0xd842d174,0xde4cd174,0xd842d176,
	0xde4dd176,0xd842d178,0xde4ed178,0xd842d17a,0xde4fd17a,0xd842d17c,0xde50d17c,0xd842d17e,
	0xde51d17e,0xd842d180,0xde52d180,0xd842d182,0xde53d182,0xd842d184,0xde54d184,0xd842d186,
	0xde55d186,0xd842d188,0xde56d188,0xd842d18a,0xde57d18a,0xd842d18c,0xde58d18c,0xd842d18e,
	0xde59d18e,0xd842d190,0xde5ad190,0xd842d192,0xde5bd192,0xd842d194,0xde5cd194,0xd842d196,
	0xde5dd196,0xd842d198,0xde5ed198,0xd842d19a,0xde5fd19a,0xd842d19c,0xde60d19c,0xd842d19e,
	0xde61d19e,0xd842d1a0,0xde62d1a0,0xd842d1a2,0xde63d1a2,0xd842d1a4,0xde64d1a4,0xd842d1a6,
	0xde65d1a6,0xd842d1a8,0xde66d1a8,0xd842d1aa,0xde67d1aa,0xd842d1ac,0xde68d1ac,0xd842d1ae,
	0xde69d1ae,0xd842d1b0,0xde6ad1b0,0xd842d1b2,0xde6bd1b2,0xd842d1b4,0xde6cd1b4,0xd842d1b6,
	0xde6dd1b6,0xd842d1b8,0xde6ed1b8,0xd842d1ba,0xde6fd1ba,0xd842d1bc,0xde70d1bc,0xd842d1be,
	0xde71d1be,0xd842d1c0,0xde72d1c0,0xd842d1c2,0xde73d1c2,0xd842d1c4,0xde74d1c4,0xd842d1c6,
	0xde75d1c6,0xd842d1c8,0xde76d1c8,0xd842d1ca,0xde77d1ca,0xd842d1cc,0xde78d1cc,0xd842d1ce,
	0xde79d1ce,0xd842d1d0,0xde7ad1d0,0xd842d1d2,0xde7bd1d2,0xd842d1d4,0xde7cd1d4,0xd842d1d6,
	0xde7dd1d6,0xd842d1d8,0xde7ed1d8,0xd842d1da,0xde7fd1da,0xd842d1dc,0xde80d1dc,0xd842d1de,
	0xde81d1de,0xd842d1e0,0xde82d1e0,0xd842d1e2,0xde83d1e2,0xd842d1e4,0xde84d1e4,0xd842d1e6,
	0xde85d1e6,0xd842d1e8,0xde86d1e8,0xd842d1ea,0xde87d1ea,0xd842d1ec,0xde88d1ec,0xd842d1ee,
	0xde89d1ee,0xd842d1f0,0xde8ad1f0,0xd842d1f2,0xde8bd1f2,0xd842d1f4,0xde8cd1f4,0xd842d1f6,
	0xde8dd1f6,0xd842d1f8,0xde8ed1f8,0xd842d1fa,0xde8fd1fa,0xd842d1fc,0xde90d1fc,0xd842d1fe,
	0xde91d1fe,0xd842d200,0xde92d200,0xd842d202,0xde93d202,0xd842d204,0xde94d204,0xd842d206,
	0xde95d206,0xd842d208,0xde96d208,0xd842d20a,0xde97d20a,0xd842d20c,0xde98d20c,0xd842d20e,
	0xde99d20e,0xd842d210,0xde9ad210,0xd842d212,0xde9bd212,0xd842d214,0xde9cd214,0xd842d216,
	0xde9dd216,0xd842d218,0xde9ed218,0xd842d21a,0xde9fd21a,0xd842d21c,0xdea0d21c,0xd842d21e,
	0xdea1d21e,0xd842d220,0xdea2d220,0xd842d222,0xdea3d222,0xd842d224,0xdea4d224,0xd842d226,
	0xdea5d226,0xd842d228,0xdea6d228,0xd842d22a,0xdea7d22a,0xd842d22c,0xdea8d22c,0xd842d22e,
	0xdea9d22e,0xd842d230,0xdeaad230,0xd842d232,0xdeabd232,0xd842d234,0xdeacd234,0xd842d236,
	0xdeadd236,0xd842d238,0xdeaed238,0xd842d23a,0xdeafd23a,0xd842d23c,0xdeb0d23c,0xd842d23e,
	0xdeb1d23e,0xd842d240,0xdeb2d240,0xd842d242,0xdeb3d242,0xd842d244,0xdeb4d244,0xd842d246,
	0xdeb5d246,0xd842d248,0xdeb6d248,0xd842d24a,0xdeb7d24a,0xd842d24c,0xdeb8d24c,0xd842d24e,
	0xdeb9d24e,0xd842d250,0xdebad250,0xd842d252,0xdebbd252,0xd842d254,0xdebcd254,0xd842d256,
	0xdebdd256,0xd842d258,0xdebed258,0xd842d25a,0xdebfd25a,0xd842d25c,0xdec0d25c,0xd842d25e,
	0xdec1d25e,0xd842d260,0xdec2d260,0xd842d262,0xdec3d262,0xd842d264,0xdec4d264,0xd842d266,
	0xdec5d266,0xd842d268,0xdec6d268,0xd842d26a,0xdec7d26a,0xd842d26c,0xdec8d26c,0xd842d26e,
	0xdec9d26e,0xd842d270,0xdecad270,0xd842d272,0xdecbd272,0xd842d274,0xdeccd274,0xd842d276,
	0xdecdd276,0xd842d278,0xdeced278,0xd842d27a,0xdecfd27a,0xd842d27c,0xded0d27c,0xd842d27e,
	0xded1d27e,0xd842d280,0xded2d280,0xd842d282,0xded3d282,0xd842d284,0xded4d284,0xd842d286,
	0xded5d286,0xd842d288,0xded6d288,0xd842d28a,0xded7d28a,0xd842d28c,0xded8d28c,0xd842d28e,
	0xded9d28e,0xd842d290,0xdedad290,0xd842d292,0xdedbd292,0xd842d294,0xdedcd294,0xd842d296,
	0xdeddd296,0xd842d298,0xdeded298,0xd842d29a,0xdedfd29a,0xd842d29c,0xdee0d29c,0xd842d29e,
	0xdee1d29e,0xd842d2a0,0xdee2d2a0,0xd842d2a2,0xdee3d2a2,0xd842d2a4,0xdee4d2a4,0xd842d2a6,
	0xdee5d2a6,0xd842d2a8,0xdee6d2a8,0xd842d2aa,0xdee7d2aa,0xd842d2ac,0xdee8d2ac,0xd842d2ae,
	0xdee9d2ae,0xd842d2b0,0xdeead2b0,0xd842d2b2,0xdeebd2b2,0xd842d2b4,0xdeecd2b4,0xd842d2b6,
	0xdeedd2b6,0xd842d2b8,0xdeeed2b8,0xd842d2ba,0xdeefd2ba,0xd842d2bc,0xdef0d2bc,0xd842d2be,
	0xdef1d2be,0xd842d2c0,0xdef2d2c0,0xd842d2c2,0xdef3d2c2,0xd842d2c4,0xdef4d2c4,0xd842d2c6,
	0xdef5d2c6,0xd842d2c8,0xdef6d2c8,0xd842d2ca,0xdef7d2ca,0xd842d2cc,0xdef8d2cc,0xd842d2ce,
	0xdef9d2ce,0xd842d2d0,0xdefad2d0,0xd842d2d2,0xdefbd2d2,0xd842d2d4,0xdefcd2d4,0xd842d2d6,
	0xdefdd2d6,0xd842d2d8,0xdefed2d8,0xd842d2da,0xdeffd2da,0xd842d2dc,0xdf00d2dc,0xd842d2de,
	0xdf01d2de,0xd842d2e0,0xdf02d2e0,0xd842d2e2,0xdf03d2e2,0xd842d2e4,0xdf04d2e4,0xd842d2e6,
	0xdf05d2e6,0xd842d2e8,0xdf06d2e8,0xd842d2ea,0xdf07d2ea,0xd842d2ec,0xdf08d2ec,0xd842d2ee,
	0xdf09d2ee,0xd842d2f0,0xdf0ad2f0,0xd842d2f2,0xdf0bd2f2,0xd842d2f4,0xdf0cd2f4,0xd842d2f6,
	0xdf0dd2f6,0xd842d2f8,0xdf0ed2f8,0xd842d2fa,0xdf0fd2fa,0xd842d2fc,0xdf10d2fc,0xd842d2fe,
	0xdf11d2fe,0xd842d300,0xdf12d300,0xd842d302,0xdf13d302,0xd842d304,0xdf14d304,0xd842d306,
	0xdf15d306,0xd842d308,0xdf16d308,0xd842d30a,0xdf17d30a,0xd842d30c,0xdf18d30c,0xd842d30e,
	0xdf19d30e,0xd842d310,0xdf1ad310,0xd842d312,0xdf1bd312,0xd842d314,0xdf1cd314,0xd842d316,
	0xdf1dd316,0xd842d318,0xdf1ed318,0xd842d31a,0xdf1fd31a,0xd842d31c,0xdf20d31c,0xd842d31e,
	0xdf21d31e,0xd842d320,0xdf22d320,0xd842d322,0xdf23d322,0xd842d324,0xdf24d324,0xd842d326,
	0xdf25d326,0xd842d328,0xdf26d328,0xd842d32a,0xdf27d32a,0xd842d32c,0xdf28d32c,0xd842d32e,
	0xdf29d32e,0xd842d330,0xdf2ad330,0xd842d332,0xdf2bd332,0xd842d334,0xdf2cd334,0xd842d336,
	0xdf2dd336,0xd842d338,0xdf2ed338,0xd842d33a,0xdf2fd33a,0xd842d33c,0xdf30d33c,0xd842d33e,
	0xdf31d33e,0xd842d340,0xdf32d340,0xd842d342,0xdf33d342,0xd842d344,0xdf34d344,0xd842d346,
	0xdf35d346,0xd842d348,0xdf36d348,0xd842d34a,0xdf37d34a,0xd842d34c,0xdf38d34c,0xd842d34e,
	0xdf39d34e,0xd842d350,0xdf3ad350,0xd842d352,0xdf3bd352,0xd842d354,0xdf3cd354,0xd842d356,
	0xdf3dd356,0xd842d358,0xdf3ed358,0xd842d35a,0xdf3fd35a,0xd842d35c,0xdf40d35c,0xd842d35e,
	0xdf41d35e,0xd842d360,0xdf42d360,0xd842d362,0xdf43d362,0xd842d364,0xdf44d364,0xd842d366,
	0xdf45d366,0xd842d368,0xdf46d368,0xd842d36a,0xdf47d36a,0xd842d36c,0xdf48d36c,0xd842d36e,
	0xdf49d36e,0xd842d370,0xdf4ad370,0xd842d372,0xdf4bd372,0xd842d374,0xdf4cd374,0xd842d376,
	0xdf4dd376,0xd842d378,0xdf4ed378,0xd842d37a,0xdf4fd37a,0xd842d37c,0xdf50d37c,0xd842d37e,
	0xdf51d37e,0xd842d380,0xdf52d380,0xd842d382,0xdf53d382,0xd842d384,0xdf54d384,0xd842d386,
	0xdf55d386,0xd842d388,0xdf56d388,0xd842d38a,0xdf57d38a,0xd842d38c,0xdf58d38c,0xd842d38e,
	0xdf59d38e,0xd842d390,0xdf5ad390,0xd842d392,0xdf5bd392,0xd842d394,0xdf5cd394,0xd842d396,
	0xdf5dd396,0xd842d398,0xdf5ed398,0xd842d39a,0xdf5fd39a,0xd842d39c,0xdf60d39c,0xd842d39e,
	0xdf61d39e,0xd842d3a0,0xdf62d3a0,0xd842d3a2,0xdf63d3a2,0xd842d3a4,0xdf64d3a4,0xd842d3a6,
	0xdf65d3a6,0xd842d3a8,0xdf66d3a8,0xd842d3aa,0xdf67d3aa,0xd842d3ac,0xdf68d3ac,0xd842d3ae,
	0xdf69d3ae,0xd842d3b0,0xdf6ad3b0,0xd842d3b2,0xdf6bd3b2,0xd842d3b4,0xdf6cd3b4,0xd842d3b6,
	0xdf6dd3b6,0xd842d3b8,0xdf6ed3b8,0xd842d3ba,0xdf6fd3ba,0xd842d3bc,0xdf70d3bc,0xd842d3be,
	0xdf71d3be,0xd842d3c0,0xdf72d3c0,0xd842d3c2,0xdf73d3c2,0xd842d3c4,0xdf74d3c4,0xd842d3c6,
	0xdf75d3c6,0xd842d3c8,0xdf76d3c8,0xd842d3ca,0xdf77d3ca,0xd842d3cc,0xdf78d3cc,0xd842d3ce,
	0xdf79d3ce,0xd842d3d0,0xdf7ad3d0,0xd842d3d2,0xdf7bd3d2,0xd842d3d4,0xdf7cd3d4,0xd842d3d6,
	0xdf7dd3d6,0xd842d3d8,0xdf7ed3d8,0xd842d3da,0xdf7fd3da,0xd842d3dc,0xdf80d3dc,0xd842d3de,
	0xdf81d3de,0xd842d3e0,0xdf82d3e0,0xd842d3e2,0xdf83d3e2,0xd842d3e4,0xdf84d3e4,0xd842d3e6,
	0xdf85d3e6,0xd842d3e8,0xdf86d3e8,0xd842d3ea,0xdf87d3ea,0xd842d3ec,0xdf88d3ec,0xd842d3ee,
	0xdf89d3ee,0xd842d3f0,0xdf8ad3f0,0xd842d3f2,0xdf8bd3f2,0xd842d3f4,0xdf8cd3f4,0xd842d3f6,
	0xdf8dd3f6,0xd842d3f8,0xdf8ed3f8,0xd842d3fa,0xdf8fd3fa,0xd842d3fc,0xdf90d3fc,0xd842d3fe,
	0xdf91d3fe,0xd842d400,0xdf92d400,0xd842d402,0xdf93d402,0xd842d404,0xdf94d404,0xd842d406,
	0xdf95d406,0xd842d408,0xdf96d408,0xd842d40a,0xdf97d40a,0xd842d40c,0xdf98d40c,0xd842d40e,
	0xdf99d40e,0xd842d410,0xdf9ad410,0xd842d412,0xdf9bd412,0xd842d414,0xdf9cd414,0xd842d416,
	0xdf9dd416,0xd842d418,0xdf9ed418,0xd842d41a,0xdf9fd41a,0xd842d41c,0xdfa0d41c,0xd842d41e,
	0xdfa1d41e,0xd842d420,0xdfa2d420,0xd842d422,0xdfa3d422,0xd842d424,0xdfa4d424,0xd842d426,
	0xdfa5d426,0xd842d428,0xdfa6d428,0xd842d42a,0xdfa7d42a,0xd842d42c,0xdfa8d42c,0xd842d42e,
	0xdfa9d42e,0xd842d430,0xdfaad430,0xd842d432,0xdfabd432,0xd842d434,0xdfacd434,0xd842d436,
	0xdfadd436,0xd842d438,0xdfaed438,0xd842d43a,0xdfafd43a,0xd842d43c,0xdfb0d43c,0xd842d43e,
	0xdfb1d43e,0xd842d440,0xdfb2d440,0xd842d442,0xdfb3d442,0xd842d444,0xdfb4d444,0xd842d446,
	0xdfb5d446,0xd842d448,0xdfb6d448,0xd842d44a,0xdfb7d44a,0xd842d44c,0xdfb8d44c,0xd842d44e,
	0xdfb9d44e,0xd842d450,0xdfbad450,0xd842d452,0xdfbbd452,0xd842d454,0xdfbcd454,0xd842d456,
	0xdfbdd456,0xd842d458,0xdfbed458,0xd842d45a,0xdfbfd45a,0xd842d45c,0xdfc0d45c,0xd842d45e,
	0xdfc1d45e,0xd842d460,0xdfc2d460,0xd842d462,0xdfc3d462,0xd842d464,0xdfc4d464,0xd842d466,
	0xdfc5d466,0xd842d468,0xdfc6d468,0xd842d46a,0xdfc7d46a,0xd842d46c,0xdfc8d46c,0xd842d46e,
	0xdfc9d46e,0xd842d470,0xdfcad470,0xd842d472,0xdfcbd472,0xd842d474,0xdfccd474,0xd842d476,
	0xdfcdd476,0xd842d478,0xdfced478,0xd842d47a,0xdfcfd47a,0xd842d47c,0xdfd0d47c,0xd842d47e,
	0xdfd1d47e,0xd842d480,0xdfd2d480,0xd842d482,0xdfd3d482,0xd842d484,0xdfd4d484,0xd842d486,
	0xdfd5d486,0xd842d488,0xdfd6d488,0xd842d48a,0xdfd7d48a,0xd842d48c,0xdfd8d48c,0xd842d48e,
	0xdfd9d48e,0xd842d490,0xdfdad490,0xd842d492,0xdfdbd492,0xd842d494,0xdfdcd494,0xd842d496,
	0xdfddd496,0xd842d498,0xdfded498,0xd842d49a,0xdfdfd49a,0xd842d49c,0xdfe0d49c,0xd842d49e,
	0xdfe1d49e,0xd842d4a0,0xdfe2d4a0,0xd842d4a2,0xdfe3d4a2,0xd842d4a4,0xdfe4d4a4,0xd842d4a6,
	0xdfe5d4a6,0xd842d4a8,0xdfe6d4a8,0xd842d4aa,0xdfe7d4aa,0xd842d4ac,0xdfe8d4ac,0xd842d4ae,
	0xdfe9d4ae,0xd842d4b0,0xdfead4b0,0xd842d4b2,0xdfebd4b2,0xd842d4b4,0xdfecd4b4,0xd842d4b6,
	0xdfedd4b6,0xd842d4b8,0xdfeed4b8,0xd842d4ba,0xdfefd4ba,0xd842d4bc,0xdff0d4bc,0xd842d4be,
	0xdff1d4be,0xd842d4c0,0xdff2d4c0,0xd842d4c2,0xdff3d4c2,0xd842d4c4,0xdff4d4c4,0xd842d4c6,
	0xdff5d4c6,0xd842d4c8,0xdff6d4c8,0xd842d4ca,0xdff7d4ca,0xd842d4cc,0xdff8d4cc,0xd842d4ce,
	0xdff9d4ce,0xd842d4d0,0xdffad4d0,0xd842d4d2,0xdffbd4d2,0xd842d4d4,0xdffcd4d4,0xd842d4d6,
	0xdffdd4d6,0xd842d4d8,0xdffed4d8,0xd842d4da,0xdfffd4da,0xd843d4dc,0xdc00d4dc,0xd843d4de,
	0xdc01d4de,0xd843d4e0,0xdc02d4e0,0xd843d4e2,0xdc03d4e2,0xd843d4e4,0xdc04d4e4,0xd843d4e6,
	0xdc05d4e6,0xd843d4e8,0xdc06d4e8,0xd843d4ea,0xdc07d4ea,0xd843d4ec,0xdc08d4ec,0xd843d4ee,
	0xdc09d4ee,0xd843d4f0,0xdc0ad4f0,0xd843d4f2,0xdc0bd4f2,0xd843d4f4,0xdc0cd4f4,0xd843d4f6,
	0xdc0dd4f6,0xd843d4f8,0xdc0ed4f8,0xd843d4fa,0xdc0fd4fa,0xd843d4fc,0xdc10d4fc,0xd843d4fe,
	0xdc11d4fe,0xd843d500,0xdc12d500,0xd843d502,0xdc13d502,0xd843d504,0xdc14d504,0xd843d506,
	0xdc15d506,0xd843d508,0xdc16d508,0xd843d50a,0xdc17d50a,0xd843d50c,0xdc18d50c,0xd843d50e,
	0xdc19d50e,0xd843d510,0xdc1ad510,0xd843d512,0xdc1bd512,0xd843d514,0xdc1cd514,0xd843d516,
	0xdc1dd516,0xd843d518,0xdc1ed518,0xd843d51a,0xdc1fd51a,0xd843d51c,0xdc20d51c,0xd843d51e,
	0xdc21d51e,0xd843d520,0xdc22d520,0xd843d522,0xdc23d522,0xd843d524,0xdc24d524,0xd843d526,
	0xdc25d526,0xd843d528,0xdc26d528,0xd843d52a,0xdc27d52a,0xd843d52c,0xdc28d52c,0xd843d52e,
	0xdc29d52e,0xd843d530,0xdc2ad530,0xd843d532,0xdc2bd532,0xd843d534,0xdc2cd534,0xd843d536,
	0xdc2dd536,0xd843d538,0xdc2ed538,0xd843d53a,0xdc2fd53a,0xd843d53c,0xdc30d53c,0xd843d53e,
	0xdc31d53e,0xd843d540,0xdc32d540,0xd843d542,0xdc33d542,0xd843d544,0xdc34d544,0xd843d546,
	0xdc35d546,0xd843d548,0xdc36d548,0xd843d54a,0xdc37d54a,0xd843d54c,0xdc38d54c,0xd843d54e,
	0xdc39d54e,0xd843d550,0xdc3ad550,0xd843d552,0xdc3bd552,0xd843d554,0xdc3cd554,0xd843d556,
	0xdc3dd556,0xd843d558,0xdc3ed558,0xd843d55a,0xdc3fd55a,0xd843d55c,0xdc40d55c,0xd843d55e,
	0xdc41d55e,0xd843d560,0xdc42d560,0xd843d562,0xdc43d562,0xd843d564,0xdc44d564,0xd843d566,
	0xdc45d566,0xd843d568,0xdc46d568,0xd843d56a,0xdc47d56a,0xd843d56c,0xdc48d56c,0xd843d56e,
	0xdc49d56e,0xd843d570,0xdc4ad570,0xd843d572,0xdc4bd572,0xd843d574,0xdc4cd574,0xd843d576,
	0xdc4dd576,0xd843d578,0xdc4ed578,0xd843d57a,0xdc4fd57a,0xd843d57c,0xdc50d57c,0xd843d57e,
	0xdc51d57e,0xd843d580,0xdc52d580,0xd843d582,0xdc53d582,0xd843d584,0xdc54d584,0xd843d586,
	0xdc55d586,0xd843d588,0xdc56d588,0xd843d58a,0xdc57d58a,0xd843d58c,0xdc58d58c,0xd843d58e,
	0xdc59d58e,0xd843d590,0xdc5ad590,0xd843d592,0xdc5bd592,0xd843d594,0xdc5cd594,0xd843d596,
	0xdc5dd596,0xd843d598,0xdc5ed598,0xd843d59a,0xdc5fd59a,0xd843d59c,0xdc60d59c,0xd843d59e,
	0xdc61d59e,0xd843d5a0,0xdc62d5a0,0xd843d5a2,0xdc63d5a2,0xd843d5a4,0xdc64d5a4,0xd843d5a6,
	0xdc65d5a6,0xd843d5a8,0xdc66d5a8,0xd843d5aa,0xdc67d5aa,0xd843d5ac,0xdc68d5ac,0xd843d5ae,
	0xdc69d5ae,0xd843d5b0,0xdc6ad5b0,0xd843d5b2,0xdc6bd5b2,0xd843d5b4,0xdc6cd5b4,0xd843d5b6,
	0xdc6dd5b6,0xd843d5b8,0xdc6ed5b8,0xd843d5ba,0xdc6fd5ba,0xd843d5bc,0xdc70d5bc,0xd843d5be,
	0xdc71d5be,0xd843d5c0,0xdc72d5c0,0xd843d5c2,0xdc73d5c2,0xd843d5c4,0xdc74d5c4,0xd843d5c6,
	0xdc75d5c6,0xd843d5c8,0xdc76d5c8,0xd843d5ca,0xdc77d5ca,0xd843d5cc,0xdc78d5cc,0xd843d5ce,
	0xdc79d5ce,0xd843d5d0,0xdc7ad5d0,0xd843d5d2,0xdc7bd5d2,0xd843d5d4,0xdc7cd5d4,0xd843d5d6,
	0xdc7dd5d6,0xd843d5d8,0xdc7ed5d8,0xd843d5da,0xdc7fd5da,0xd843d5dc,0xdc80d5dc,0xd843d5de,
	0xdc81d5de,0xd843d5e0,0xdc82d5e0,0xd843d5e2,0xdc83d5e2,0xd843d5e4,0xdc84d5e4,0xd843d5e6,
	0xdc85d5e6,0xd843d5e8,0xdc86d5e8,0xd843d5ea,0xdc87d5ea,0xd843d5ec,0xdc88d5ec,0xd843d5ee,
	0xdc89d5ee,0xd843d5f0,0xdc8ad5f0,0xd843d5f2,0xdc8bd5f2,0xd843d5f4,0xdc8cd5f4,0xd843d5f6,
	0xdc8dd5f6,0xd843d5f8,0xdc8ed5f8,0xd843d5fa,0xdc8fd5fa,0xd843d5fc,0xdc90d5fc,0xd843d5fe,
	0xdc91d5fe,0xd843d600,0xdc92d600,0xd843d602,0xdc93d602,0xd843d604,0xdc94d604,0xd843d606,
	0xdc95d606,0xd843d608,0xdc96d608,0xd843d60a,0xdc97d60a,0xd843d60c,0xdc98d60c,0xd843d60e,
	0xdc99d60e,0xd843d610,0xdc9ad610,0xd843d612,0xdc9bd612,0xd843d614,0xdc9cd614,0xd843d616,
	0xdc9dd616,0xd843d618,0xdc9ed618,0xd843d61a,0xdc9fd61a,0xd843d61c,0xdca0d61c,0xd843d61e,
	0xdca1d61e,0xd843d620,0xdca2d620,0xd843d622,0xdca3d622,0xd843d624,0xdca4d624,0xd843d626,
	0xdca5d626,0xd843d628,0xdca6d628,0xd843d62a,0xdca7d62a,0xd843d62c,0xdca8d62c,0xd843d62e,
	0xdca9d62e,0xd843d630,0xdcaad630,0xd843d632,0xdcabd632,0xd843d634,0xdcacd634,0xd843d636,
	0xdcadd636,0xd843d638,0xdcaed638,0xd843d63a,0xdcafd63a,0xd843d63c,0xdcb0d63c,0xd843d63e,
	0xdcb1d63e,0xd843d640,0xdcb2d640,0xd843d642,0xdcb3d642,0xd843d644,0xdcb4d644,0xd843d646,
	0xdcb5d646,0xd843d648,0xdcb6d648,0xd843d64a,0xdcb7d64a,0xd843d64c,0xdcb8d64c,0xd843d64e,
	0xdcb9d64e,0xd843d650,0xdcbad650,0xd843d652,0xdcbbd652,0xd843d654,0xdcbcd654,0xd843d656,
	0xdcbdd656,0xd843d658,0xdcbed658,0xd843d65a,0xdcbfd65a,0xd843d65c,0xdcc0d65c,0xd843d65e,
	0xdcc1d65e,0xd843d660,0xdcc2d660,0xd843d662,0xdcc3d662,0xd843d664,0xdcc4d664,0xd843d666,
	0xdcc5d666,0xd843d668,0xdcc6d668,0xd843d66a,0xdcc7d66a,0xd843d66c,0xdcc8d66c,0xd843d66e,
	0xdcc9d66e,0xd843d670,0xdccad670,0xd843d672,0xdccbd672,0xd843d674,0xdcccd674,0xd843d676,
	0xdccdd676,0xd843d678,0xdcced678,0xd843d67a,0xdccfd67a,0xd843d67c,0xdcd0d67c,0xd843d67e,
	0xdcd1d67e,0xd843d680,0xdcd2d680,0xd843d682,0xdcd3d682,0xd843d684,0xdcd4d684,0xd843d686,
	0xdcd5d686,0xd843d688,0xdcd6d688,0xd843d68a,0xdcd7d68a,0xd843d68c,0xdcd8d68c,0xd843d68e,
	0xdcd9d68e,0xd843d690,0xdcdad690,0xd843d692,0xdcdbd692,0xd843d694,0xdcdcd694,0xd843d696,
	0xdcddd696,0xd843d698,0xdcded698,0xd843d69a,0xdcdfd69a,0xd843d69c,0xdce0d69c,0xd843d69e,
	0xdce1d69e,0xd843d6a0,0xdce2d6a0,0xd843d6a2,0xdce3d6a2,0xd843d6a4,0xdce4d6a4,0xd843d6a6,
	0xdce5d6a6,0xd843d6a8,0xdce6d6a8,0xd843d6aa,0xdce7d6aa,0xd843d6ac,0xdce8d6ac,0xd843d6ae,
	0xdce9d6ae,0xd843d6b0,0xdcead6b0,0xd843d6b2,0xdcebd6b2,0xd843d6b4,0xdcecd6b4,0xd843d6b6,
	0xdcedd6b6,0xd843d6b8,0xdceed6b8,0xd843d6ba,0xdcefd6ba,0xd843d6bc,0xdcf0d6bc,0xd843d6be,
	0xdcf1d6be,0xd843d6c0,0xdcf2d6c0,0xd843d6c2,0xdcf3d6c2,0xd843d6c4,0xdcf4d6c4,0xd843d6c6,
	0xdcf5d6c6,0xd843d6c8,0xdcf6d6c8,0xd843d6ca,0xdcf7d6ca,0xd843d6cc,0xdcf8d6cc,0xd843d6ce,
	0xdcf9d6ce,0xd843d6d0,0xdcfad6d0,0xd843d6d2,0xdcfbd6d2,0xd843d6d4,0xdcfcd6d4,0xd843d6d6,
	0xdcfdd6d6,0xd843d6d8,0xdcfed6d8,0xd843d6da,0xdcffd6da,0xd843d6dc,0xdd00d6dc,0xd843d6de,
	0xdd01d6de,0xd843d6e0,0xdd02d6e0,0xd843d6e2,0xdd03d6e2,0xd843d6e4,0xdd04d6e4,0xd843d6e6,
	0xdd05d6e6,0xd843d6e8,0xdd06d6e8,0xd843d6ea,0xdd07d6ea,0xd843d6ec,0xdd08d6ec,0xd843d6ee,
	0xdd09d6ee,0xd843d6f0,0xdd0ad6f0,0xd843d6f2,0xdd0bd6f2,0xd843d6f4,0xdd0cd6f4,0xd843d6f6,
	0xdd0dd6f6,0xd843d6f8,0xdd0ed6f8,0xd843d6fa,0xdd0fd6fa,0xd843d6fc,0xdd10d6fc,0xd843d6fe,
	0xdd11d6fe,0xd843d700,0xdd12d700,0xd843d702,0xdd13d702,0xd843d704,0xdd14d704,0xd843d706,
	0xdd15d706,0xd843d708,0xdd16d708,0xd843d70a,0xdd17d70a,0xd843d70c,0xdd18d70c,0xd843d70e,
	0xdd19d70e,0xd843d710,0xdd1ad710,0xd843d712,0xdd1bd712,0xd843d714,0xdd1cd714,0xd843d716,
	0xdd1dd716,0xd843d718,0xdd1ed718,0xd843d71a,0xdd1fd71a,0xd843d71c,0xdd20d71c,0xd843d71e,
	0xdd21d71e,0xd843d720,0xdd22d720,0xd843d722,0xdd23d722,0xd843d724,0xdd24d724,0xd843d726,
	0xdd25d726,0xd843d728,0xdd26d728,0xd843d72a,0xdd27d72a,0xd843d72c,0xdd28d72c,0xd843d72e,
	0xdd29d72e,0xd843d730,0xdd2ad730,0xd843d732,0xdd2bd732,0xd843d734,0xdd2cd734,0xd843d736,
	0xdd2dd736,0xd843d738,0xdd2ed738,0xd843d73a,0xdd2fd73a,0xd843d73c,0xdd30d73c,0xd843d73e,
	0xdd31d73e,0xd843d740,0xdd32d740,0xd843d742,0xdd33d742,0xd843d744,0xdd34d744,0xd843d746,
	0xdd35d746,0xd843d748,0xdd36d748,0xd843d74a,0xdd37d74a,0xd843d74c,0xdd38d74c,0xd843d74e,
	0xdd39d74e,0xd843d750,0xdd3ad750,0xd843d752,0xdd3bd752,0xd843d754,0xdd3cd754,0xd843d756,
	0xdd3dd756,0xd843d758,0xdd3ed758,0xd843d75a,0xdd3fd75a,0xd843d75c,0xdd40d75c,0xd843d75e,
	0xdd41d75e,0xd843d760,0xdd42d760,0xd843d762,0xdd43d762,0xd843d764,0xdd44d764,0xd843d766,
	0xdd45d766,0xd843d768,0xdd46d768,0xd843d76a,0xdd47d76a,0xd843d76c,0xdd48d76c,0xd843d76e,
	0xdd49d76e,0xd843d770,0xdd4ad770,0xd843d772,0xdd4bd772,0xd843d774,0xdd4cd774,0xd843d776,
	0xdd4dd776,0xd843d778,0xdd4ed778,0xd843d77a,0xdd4fd77a,0xd843d77c,0xdd50d77c,0xd843d77e,
	0xdd51d77e,0xd843d780,0xdd52d780,0xd843d782,0xdd53d782,0xd843d784,0xdd54d784,0xd843d786,
	0xdd55d786,0xd843d788,0xdd56d788,0xd843d78a,0xdd57d78a,0xd843d78c,0xdd58d78c,0xd843d78e,
	0xdd59d78e,0xd843d790,0xdd5ad790,0xd843d792,0xdd5bd792,0xd843d794,0xdd5cd794,0xd843d796,
	0xdd5dd796,0xd843d798,0xdd5ed798,0xd843d79a,0xdd5fd79a,0xd843d79c,0xdd60d79c,0xd843d79e,
	0xdd61d79e,0xd843d7a0,0xdd62d7a0,0xd843d7a2,0xdd63d7a2,0xd843d7a4,0xdd64d7a4,0xd843d7a6,
	0xdd65d7a6,0xd843d7a8,0xdd66d7a8,0xd843d7aa,0xdd67d7aa,0xd843d7ac,0xdd68d7ac,0xd843d7ae,
	0xdd69d7ae,0xd843d7b0,0xdd6ad7b0,0xd843d7b2,0xdd6bd7b2,0xd843d7b4,0xdd6cd7b4,0xd843d7b6,
	0xdd6dd7b6,0xd843d7b8,0xdd6ed7b8,0xd843d7ba,0xdd6fd7ba,0xd843d7bc,0xdd70d7bc,0xd843d7be,
	0xdd71d7be,0xd843d7c0,0xdd72d7c0,0xd843d7c2,0xdd73d7c2,0xd843d7c4,0xdd74d7c4,0xd843d7c6,
	0xdd75d7c6,0xd843d7c8,0xdd76d7c8,0xd843d7ca,0xdd77d7ca,0xd843d7cc,0xdd78d7cc,0xd843d7ce,
	0xdd79d7ce,0xd843d7d0,0xdd7ad7d0,0xd843d7d2,0xdd7bd7d2,0xd843d7d4,0xdd7cd7d4,0xd843d7d6,
	0xdd7dd7d6,0xd843d7d8,0xdd7ed7d8,0xd843d7da,0xdd7fd7da,0xd843d7dc,0xdd80d7dc,0xd843d7de,
	0xdd81d7de,0xd843d7e0,0xdd82d7e0,0xd843d7e2,0xdd83d7e2,0xd843d7e4,0xdd84d7e4,0xd843d7e6,
	0xdd85d7e6,0xd843d7e8,0xdd86d7e8,0xd843d7ea,0xdd87d7ea,0xd843d7ec,0xdd88d7ec,0xd843d7ee,
	0xdd89d7ee,0xd843d7f0,0xdd8ad7f0,0xd843d7f2,0xdd8bd7f2,0xd843d7f4,0xdd8cd7f4,0xd843d7f6,
	0xdd8dd7f6,0xd843d7f8,0xdd8ed7f8,0xd843d7fa,0xdd8fd7fa,0xd843d7fc,0xdd90d7fc,0xd843d7fe,
	0xdd91d7fe,0xd843d800,0xdd92d800,0xd843d802,0xdd93d802,0xd843d804,0xdd94d804,0xd843d806,
	0xdd95d806,0xd843d808,0xdd96d808,0xd843d80a,0xdd97d80a,0xd843d80c,0xdd98d80c,0xd843d80e,
	0xdd99d80e,0xd843d810,0xdd9ad810,0xd843d812,0xdd9bd812,0xd843d814,0xdd9cd814,0xd843d816,
	0xdd9dd816,0xd843d818,0xdd9ed818,0xd843d81a,0xdd9fd81a,0xd843d81c,0xdda0d81c,0xd843d81e,
	0xdda1d81e,0xd843d820,0xdda2d820,0xd843d822,0xdda3d822,0xd843d824,0xdda4d824,0xd843d826,
	0xdda5d826,0xd843d828,0xdda6d828,0xd843d82a,0xdda7d82a,0xd843d82c,0xdda8d82c,0xd843d82e,
	0xdda9d82e,0xd843d830,0xddaad830,0xd843d832,0xddabd832,0xd843d834,0xddacd834,0xd843d836,
	0xddadd836,0xd843d838,0xddaed838,0xd843d83a,0xddafd83a,0xd843d83c,0xddb0d83c,0xd843d83e,
	0xddb1d83e,0xd843d840,0xddb2d840,0xd843d842,0xddb3d842,0xd843d844,0xddb4d844,0xd843d846,
	0xddb5d846,0xd843d848,0xddb6d848,0xd843d84a,0xddb7d84a,0xd843d84c,0xddb8d84c,0xd843d84e,
	0xddb9d84e,0xd843d850,0xddbad850,0xd843d852,0xddbbd852,0xd843d854,0xddbcd854,0xd843d856,
	0xddbdd856,0xd843d858,0xddbed858,0xd843d85a,0xddbfd85a,0xd843d85c,0xddc0d85c,0xd843d85e,
	0xddc1d85e,0xd843d860,0xddc2d860,0xd843d862,0xddc3d862,0xd843d864,0xddc4d864,0xd843d866,
	0xddc5d866,0xd843d868,0xddc6d868,0xd843d86a,0xddc7d86a,0xd843d86c,0xddc8d86c,0xd843d86e,
	0xddc9d86e,0xd843d870,0xddcad870,0xd843d872,0xddcbd872,0xd843d874,0xddccd874,0xd843d876,
	0xddcdd876,0xd843d878,0xddced878,0xd843d87a,0xddcfd87a,0xd843d87c,0xddd0d87c,0xd843d87e,
	0xddd1d87e,0xd843d880,0xddd2d880,0xd843d882,0xddd3d882,0xd843d884,0xddd4d884,0xd843d886,
	0xddd5d886,0xd843d888,0xddd6d888,0xd843d88a,0xddd7d88a,0xd843d88c,0xddd8d88c,0xd843d88e,
	0xddd9d88e,0xd843d890,0xdddad890,0xd843d892,0xdddbd892,0xd843d894,0xdddcd894,0xd843d896,
	0xddddd896,0xd843d898,0xddded898,0xd843d89a,0xdddfd89a,0xd843d89c,0xdde0d89c,0xd843d89e,
	0xdde1d89e,0xd843d8a0,0xdde2d8a0,0xd843d8a2,0xdde3d8a2,0xd843d8a4,0xdde4d8a4,0xd843d8a6,
	0xdde5d8a6,0xd843d8a8,0xdde6d8a8,0xd843d8aa,0xdde7d8aa,0xd843d8ac,0xdde8d8ac,0xd843d8ae,
	0xdde9d8ae,0xd843d8b0,0xddead8b0,0xd843d8b2,0xddebd8b2,0xd843d8b4,0xddecd8b4,0xd843d8b6,
	0xddedd8b6,0xd843d8b8,0xddeed8b8,0xd843d8ba,0xddefd8ba,0xd843d8bc,0xddf0d8bc,0xd843d8be,
	0xddf1d8be,0xd843d8c0,0xddf2d8c0,0xd843d8c2,0xddf3d8c2,0xd843d8c4,0xddf4d8c4,0xd843d8c6,
	0xddf5d8c6,0xd843d8c8,0xddf6d8c8,0xd843d8ca,0xddf7d8ca,0xd843d8cc,0xddf8d8cc,0xd843d8ce,
	0xddf9d8ce,0xd843d8d0,0xddfad8d0,0xd843d8d2,0xddfbd8d2,0xd843d8d4,0xddfcd8d4,0xd843d8d6,
	0xddfdd8d6,0xd843d8d8,0xddfed8d8,0xd843d8da,0xddffd8da,0xd843d8dc,0xde00d8dc,0xd843d8de,
	0xde01d8de,0xd843d8e0,0xde02d8e0,0xd843d8e2,0xde03d8e2,0xd843d8e4,0xde04d8e4,0xd843d8e6,
	0xde05d8e6,0xd843d8e8,0xde06d8e8,0xd843d8ea,0xde07d8ea,0xd843d8ec,0xde08d8ec,0xd843d8ee,
	0xde09d8ee,0xd843d8f0,0xde0ad8f0,0xd843d8f2,0xde0bd8f2,0xd843d8f4,0xde0cd8f4,0xd843d8f6,
	0xde0dd8f6,0xd843d8f8,0xde0ed8f8,0xd843d8fa,0xde0fd8fa,0xd843d8fc,0xde10d8fc,0xd843d8fe,
	0xde11d8fe,0xd843d900,0xde12d900,0xd843d902,0xde13d902,0xd843d904,0xde14d904,0xd843d906,
	0xde15d906,0xd843d908,0xde16d908,0xd843d90a,0xde17d90a,0xd843d90c,0xde18d90c,0xd843d90e,
	0xde19d90e,0xd843d910,0xde1ad910,0xd843d912,0xde1bd912,0xd843d914,0xde1cd914,0xd843d916,
	0xde1dd916,0xd843d918,0xde1ed918,0xd843d91a,0xde1fd91a,0xd843d91c,0xde20d91c,0xd843d91e,
	0xde21d91e,0xd843d920,0xde22d920,0xd843d922,0xde23d922,0xd843d924,0xde24d924,0xd843d926,
	0xde25d926,0xd843d928,0xde26d928,0xd843d92a,0xde27d92a,0xd843d92c,0xde28d92c,0xd843d92e,
	0xde29d92e,0xd843d930,0xde2ad930,0xd843d932,0xde2bd932,0xd843d934,0xde2cd934,0xd843d936,
	0xde2dd936,0xd843d938,0xde2ed938,0xd843d93a,0xde2fd93a,0xd843d93c,0xde30d93c,0xd843d93e,
	0xde31d93e,0xd843d940,0xde32d940,0xd843d942,0xde33d942,0xd843d944,0xde34d944,0xd843d946,
	0xde35d946,0xd843d948,0xde36d948,0xd843d94a,0xde37d94a,0xd843d94c,0xde38d94c,0xd843d94e,
	0xde39d94e,0xd843d950,0xde3ad950,0xd843d952,0xde3bd952,0xd843d954,0xde3cd954,0xd843d956,
	0xde3dd956,0xd843d958,0xde3ed958,0xd843d95a,0xde3fd95a,0xd843d95c,0xde40d95c,0xd843d95e,
	0xde41d95e,0xd843d960,0xde42d960,0xd843d962,0xde43d962,0xd843d964,0xde44d964,0xd843d966,
	0xde45d966,0xd843d968,0xde46d968,0xd843d96a,0xde47d96a,0xd843d96c,0xde48d96c,0xd843d96e,
	0xde49d96e,0xd843d970,0xde4ad970,0xd843d972,0xde4bd972,0xd843d974,0xde4cd974,0xd843d976,
	0xde4dd976,0xd843d978,0xde4ed978,0xd843d97a,0xde4fd97a,0xd843d97c,0xde50d97c,0xd843d97e,
	0xde51d97e,0xd843d980,0xde52d980,0xd843d982,0xde53d982,0xd843d984,0xde54d984,0xd843d986,
	0xde55d986,0xd843d988,0xde56d988,0xd843d98a,0xde57d98a,0xd843d98c,0xde58d98c,0xd843d98e,
	0xde59d98e,0xd843d990,0xde5ad990,0xd843d992,0xde5bd992,0xd843d994,0xde5cd994,0xd843d996,
	0xde5dd996,0xd843d998,0xde5ed998,0xd843d99a,0xde5fd99a,0xd843d99c,0xde60d99c,0xd843d99e,
	0xde61d99e,0xd843d9a0,0xde62d9a0,0xd843d9a2,0xde63d9a2,0xd843d9a4,0xde64d9a4,0xd843d9a6,
	0xde65d9a6,0xd843d9a8,0xde66d9a8,0xd843d9aa,0xde67d9aa,0xd843d9ac,0xde68d9ac,0xd843d9ae,
	0xde69d9ae,0xd843d9b0,0xde6ad9b0,0xd843d9b2,0xde6bd9b2,0xd843d9b4,0xde6cd9b4,0xd843d9b6,
	0xde6dd9b6,0xd843d9b8,0xde6ed9b8,0xd843d9ba,0xde6fd9ba,0xd843d9bc,0xde70d9bc,0xd843d9be,
	0xde71d9be,0xd843d9c0,0xde72d9c0,0xd843d9c2,0xde73d9c2,0xd843d9c4,0xde74d9c4,0xd843d9c6,
	0xde75d9c6,0xd843d9c8,0xde76d9c8,0xd843d9ca,0xde77d9ca,0xd843d9cc,0xde78d9cc,0xd843d9ce,
	0xde79d9ce,0xd843d9d0,0xde7ad9d0,0xd843d9d2,0xde7bd9d2,0xd843d9d4,0xde7cd9d4,0xd843d9d6,
	0xde7dd9d6,0xd843d9d8,0xde7ed9d8,0xd843d9da,0xde7fd9da,0xd843d9dc,0xde80d9dc,0xd843d9de,
	0xde81d9de,0xd843d9e0,0xde82d9e0,0xd843d9e2,0xde83d9e2,0xd843d9e4,0xde84d9e4,0xd843d9e6,
	0xde85d9e6,0xd843d9e8,0xde86d9e8,0xd843d9ea,0xde87d9ea,0xd843d9ec,0xde88d9ec,0xd843d9ee,
	0xde89d9ee,0xd843d9f0,0xde8ad9f0,0xd843d9f2,0xde8bd9f2,0xd843d9f4,0xde8cd9f4,0xd843d9f6,
	0xde8dd9f6,0xd843d9f8,0xde8ed9f8,0xd843d9fa,0xde8fd9fa,0xd843d9fc,0xde90d9fc,0xd843d9fe,
	0xde91d9fe,0xd843da00,0xde92da00,0xd843da02,0xde93da02,0xd843da04,0xde94da04,0xd843da06,
	0xde95da06,0xd843da08,0xde96da08,0xd843da0a,0xde97da0a,0xd843da0c,0xde98da0c,0xd843da0e,
	0xde99da0e,0xd843da10,0xde9ada10,0xd843da12,0xde9bda12,0xd843da14,0xde9cda14,0xd843da16,
	0xde9dda16,0xd843da18,0xde9eda18,0xd843da1a,0xde9fda1a,0xd843da1c,0xdea0da1c,0xd843da1e,
	0xdea1da1e,0xd843da20,0xdea2da20,0xd843da22,0xdea3da22,0xd843da24,0xdea4da24,0xd843da26,
	0xdea5da26,0xd843da28,0xdea6da28,0xd843da2a,0xdea7da2a,0xd843da2c,0xdea8da2c,0xd843da2e,
	0xdea9da2e,0xd843da30,0xdeaada30,0xd843da32,0xdeabda32,0xd843da34,0xdeacda34,0xd843da36,
	0xdeadda36,0xd843da38,0xdeaeda38,0xd843da3a,0xdeafda3a,0xd843da3c,0xdeb0da3c,0xd843da3e,
	0xdeb1da3e,0xd843da40,0xdeb2da40,0xd843da42,0xdeb3da42,0xd843da44,0xdeb4da44,0xd843da46,
	0xdeb5da46,0xd843da48,0xdeb6da48,0xd843da4a,0xdeb7da4a,0xd843da4c,0xdeb8da4c,0xd843da4e,
	0xdeb9da4e,0xd843da50,0xdebada50,0xd843da52,0xdebbda52,0xd843da54,0xdebcda54,0xd843da56,
	0xdebdda56,0xd843da58,0xdebeda58,0xd843da5a,0xdebfda5a,0xd843da5c,0xdec0da5c,0xd843da5e,
	0xdec1da5e,0xd843da60,0xdec2da60,0xd843da62,0xdec3da62,0xd843da64,0xdec4da64,0xd843da66,
	0xdec5da66,0xd843da68,0xdec6da68,0xd843da6a,0xdec7da6a,0xd843da6c,0xdec8da6c,0xd843da6e,
	0xdec9da6e,0xd843da70,0xdecada70,0xd843da72,0xdecbda72,0xd843da74,0xdeccda74,0xd843da76,
	0xdecdda76,0xd843da78,0xdeceda78,0xd843da7a,0xdecfda7a,0xd843da7c,0xded0da7c,0xd843da7e,
	0xded1da7e,0xd843da80,0xded2da80,0xd843da82,0xded3da82,0xd843da84,0xded4da84,0xd843da86,
	0xded5da86,0xd843da88,0xded6da88,0xd843da8a,0xded7da8a,0xd843da8c,0xded8da8c,0xd843da8e,
	0xded9da8e,0xd843da90,0xdedada90,0xd843da92,0xdedbda92,0xd843da94,0xdedcda94,0xd843da96,
	0xdeddda96,0xd843da98,0xdededa98,0xd843da9a,0xdedfda9a,0xd843da9c,0xdee0da9c,0xd843da9e,
	0xdee1da9e,0xd843daa0,0xdee2daa0,0xd843daa2,0xdee3daa2,0xd843daa4,0xdee4daa4,0xd843daa6,
	0xdee5daa6,0xd843daa8,0xdee6daa8,0xd843daaa,0xdee7daaa,0xd843daac,0xdee8daac,0xd843daae,
	0xdee9daae,0xd843dab0,0xdeeadab0,0xd843dab2,0xdeebdab2,0xd843dab4,0xdeecdab4,0xd843dab6,
	0xdeeddab6,0xd843dab8,0xdeeedab8,0xd843daba,0xdeefdaba,0xd843dabc,0xdef0dabc,0xd843dabe,
	0xdef1dabe,0xd843dac0,0xdef2dac0,0xd843dac2,0xdef3dac2,0xd843dac4,0xdef4dac4,0xd843dac6,
	0xdef5dac6,0xd843dac8,0xdef6dac8,0xd843daca,0xdef7daca,0xd843dacc,0xdef8dacc,0xd843dace,
	0xdef9dace,0xd843dad0,0xdefadad0,0xd843dad2,0xdefbdad2,0xd843dad4,0xdefcdad4,0xd843dad6,
	0xdefddad6,0xd843dad8,0xdefedad8,0xd843dada,0xdeffdada,0xd843dadc,0xdf00dadc,0xd843dade,
	0xdf01dade,0xd843dae0,0xdf02dae0,0xd843dae2,0xdf03dae2,0xd843dae4,0xdf04dae4,0xd843dae6,
	0xdf05dae6,0xd843dae8,0xdf06dae8,0xd843daea,0xdf07daea,0xd843daec,0xdf08daec,0xd843daee,
	0xdf09daee,0xd843daf0,0xdf0adaf0,0xd843daf2,0xdf0bdaf2,0xd843daf4,0xdf0cdaf4,0xd843daf6,
	0xdf0ddaf6,0xd843daf8,0xdf0edaf8,0xd843dafa,0xdf0fdafa,0xd843dafc,0xdf10dafc,0xd843dafe,
	0xdf11dafe,0xd843db00,0xdf12db00,0xd843db02,0xdf13db02,0xd843db04,0xdf14db04,0xd843db06,
	0xdf15db06,0xd843db08,0xdf16db08,0xd843db0a,0xdf17db0a,0xd843db0c,0xdf18db0c,0xd843db0e,
	0xdf19db0e,0xd843db10,0xdf1adb10,0xd843db12,0xdf1bdb12,0xd843db14,0xdf1cdb14,0xd843db16,
	0xdf1ddb16,0xd843db18,0xdf1edb18,0xd843db1a,0xdf1fdb1a,0xd843db1c,0xdf20db1c,0xd843db1e,
	0xdf21db1e,0xd843db20,0xdf22db20,0xd843db22,0xdf23db22,0xd843db24,0xdf24db24,0xd843db26,
	0xdf25db26,0xd843db28,0xdf26db28,0xd843db2a,0xdf27db2a,0xd843db2c,0xdf28db2c,0xd843db2e,
	0xdf29db2e,0xd843db30,0xdf2adb30,0xd843db32,0xdf2bdb32,0xd843db34,0xdf2cdb34,0xd843db36,
	0xdf2ddb36,0xd843db38,0xdf2edb38,0xd843db3a,0xdf2fdb3a,0xd843db3c,0xdf30db3c,0xd843db3e,
	0xdf31db3e,0xd843db40,0xdf32db40,0xd843db42,0xdf33db42,0xd843db44,0xdf34db44,0xd843db46,
	0xdf35db46,0xd843db48,0xdf36db48,0xd843db4a,0xdf37db4a,0xd843db4c,0xdf38db4c,0xd843db4e,
	0xdf39db4e,0xd843db50,0xdf3adb50,0xd843db52,0xdf3bdb52,0xd843db54,0xdf3cdb54,0xd843db56,
	0xdf3ddb56,0xd843db58,0xdf3edb58,0xd843db5a,0xdf3fdb5a,0xd843db5c,0xdf40db5c,0xd843db5e,
	0xdf41db5e,0xd843db60,0xdf42db60,0xd843db62,0xdf43db62,0xd843db64,0xdf44db64,0xd843db66,
	0xdf45db66,0xd843db68,0xdf46db68,0xd843db6a,0xdf47db6a,0xd843db6c,0xdf48db6c,0xd843db6e,
	0xdf49db6e,0xd843db70,0xdf4adb70,0xd843db72,0xdf4bdb72,0xd843db74,0xdf4cdb74,0xd843db76,
	0xdf4ddb76,0xd843db78,0xdf4edb78,0xd843db7a,0xdf4fdb7a,0xd843db7c,0xdf50db7c,0xd843db7e,
	0xdf51db7e,0xd843db80,0xdf52db80,0xd843db82,0xdf53db82,0xd843db84,0xdf54db84,0xd843db86,
	0xdf55db86,0xd843db88,0xdf56db88,0xd843db8a,0xdf57db8a,0xd843db8c,0xdf58db8c,0xd843db8e,
	0xdf59db8e,0xd843db90,0xdf5adb90,0xd843db92,0xdf5bdb92,0xd843db94,0xdf5cdb94,0xd843db96,
	0xdf5ddb96,0xd843db98,0xdf5edb98,0xd843db9a,0xdf5fdb9a,0xd843db9c,0xdf60db9c,0xd843db9e,
	0xdf61db9e,0xd843dba0,0xdf62dba0,0xd843dba2,0xdf63dba2,0xd843dba4,0xdf64dba4,0xd843dba6,
	0xdf65dba6,0xd843dba8,0xdf66dba8,0xd843dbaa,0xdf67dbaa,0xd843dbac,0xdf68dbac,0xd843dbae,
	0xdf69dbae,0xd843dbb0,0xdf6adbb0,0xd843dbb2,0xdf6bdbb2,0xd843dbb4,0xdf6cdbb4,0xd843dbb6,
	0xdf6ddbb6,0xd843dbb8,0xdf6edbb8,0xd843dbba,0xdf6fdbba,0xd843dbbc,0xdf70dbbc,0xd843dbbe,
	0xdf71dbbe,0xd843dbc0,0xdf72dbc0,0xd843dbc2,0xdf73dbc2,0xd843dbc4,0xdf74dbc4,0xd843dbc6,
	0xdf75dbc6,0xd843dbc8,0xdf76dbc8,0xd843dbca,0xdf77dbca,0xd843dbcc,0xdf78dbcc,0xd843dbce,
	0xdf79dbce,0xd843dbd0,0xdf7adbd0,0xd843dbd2,0xdf7bdbd2,0xd843dbd4,0xdf7cdbd4,0xd843dbd6,
	0xdf7ddbd6,0xd843dbd8,0xdf7edbd8,0xd843dbda,0xdf7fdbda,0xd843dbdc,0xdf80dbdc,0xd843dbde,
	0xdf81dbde,0xd843dbe0,0xdf82dbe0,0xd843dbe2,0xdf83dbe2,0xd843dbe4,0xdf84dbe4,0xd843dbe6,
	0xdf85dbe6,0xd843dbe8,0xdf86dbe8,0xd843dbea,0xdf87dbea,0xd843dbec,0xdf88dbec,0xd843dbee,
	0xdf89dbee,0xd843dbf0,0xdf8adbf0,0xd843dbf2,0xdf8bdbf2,0xd843dbf4,0xdf8cdbf4,0xd843dbf6,
	0xdf8ddbf6,0xd843dbf8,0xdf8edbf8,0xd843dbfa,0xdf8fdbfa,0xd843dbfc,0xdf90dbfc,0xd843dbfe,
	0xdf91dbfe,0xd843dc00,0xdf92dc00,0xd843dc02,0xdf93dc02,0xd843dc04,0xdf94dc04,0xd843dc06,
	0xdf95dc06,0xd843dc08,0xdf96dc08,0xd843dc0a,0xdf97dc0a,0xd843dc0c,0xdf98dc0c,0xd843dc0e,
	0xdf99dc0e,0xd843dc10,0xdf9adc10,0xd843dc12,0xdf9bdc12,0xd843dc14,0xdf9cdc14,0xd843dc16,
	0xdf9ddc16,0xd843dc18,0xdf9edc18,0xd843dc1a,0xdf9fdc1a,0xd843dc1c,0xdfa0dc1c,0xd843dc1e,
	0xdfa1dc1e,0xd843dc20,0xdfa2dc20,0xd843dc22,0xdfa3dc22,0xd843dc24,0xdfa4dc24,0xd843dc26,
	0xdfa5dc26,0xd843dc28,0xdfa6dc28,0xd843dc2a,0xdfa7dc2a,0xd843dc2c,0xdfa8dc2c,0xd843dc2e,
	0xdfa9dc2e,0xd843dc30,0xdfaadc30,0xd843dc32,0xdfabdc32,0xd843dc34,0xdfacdc34,0xd843dc36,
	0xdfaddc36,0xd843dc38,0xdfaedc38,0xd843dc3a,0xdfafdc3a,0xd843dc3c,0xdfb0dc3c,0xd843dc3e,
	0xdfb1dc3e,0xd843dc40,0xdfb2dc40,0xd843dc42,0xdfb3dc42,0xd843dc44,0xdfb4dc44,0xd843dc46,
	0xdfb5dc46,0xd843dc48,0xdfb6dc48,0xd843dc4a,0xdfb7dc4a,0xd843dc4c,0xdfb8dc4c,0xd843dc4e,
	0xdfb9dc4e,0xd843dc50,0xdfbadc50,0xd843dc52,0xdfbbdc52,0xd843dc54,0xdfbcdc54,0xd843dc56,
	0xdfbddc56,0xd843dc58,0xdfbedc58,0xd843dc5a,0xdfbfdc5a,0xd843dc5c,0xdfc0dc5c,0xd843dc5e,
	0xdfc1dc5e,0xd843dc60,0xdfc2dc60,0xd843dc62,0xdfc3dc62,0xd843dc64,0xdfc4dc64,0xd843dc66,
	0xdfc5dc66,0xd843dc68,0xdfc6dc68,0xd843dc6a,0xdfc7dc6a,0xd843dc6c,0xdfc8dc6c,0xd843dc6e,
	0xdfc9dc6e,0xd843dc70,0xdfcadc70,0xd843dc72,0xdfcbdc72,0xd843dc74,0xdfccdc74,0xd843dc76,
	0xdfcddc76,0xd843dc78,0xdfcedc78,0xd843dc7a,0xdfcfdc7a,0xd843dc7c,0xdfd0dc7c,0xd843dc7e,
	0xdfd1dc7e,0xd843dc80,0xdfd2dc80,0xd843dc82,0xdfd3dc82,0xd843dc84,0xdfd4dc84,0xd843dc86,
	0xdfd5dc86,0xd843dc88,0xdfd6dc88,0xd843dc8a,0xdfd7dc8a,0xd843dc8c,0xdfd8dc8c,0xd843dc8e,
	0xdfd9dc8e,0xd843dc90,0xdfdadc90,0xd843dc92,0xdfdbdc92,0xd843dc94,0xdfdcdc94,0xd843dc96,
	0xdfdddc96,0xd843dc98,0xdfdedc98,0xd843dc9a,0xdfdfdc9a,0xd843dc9c,0xdfe0dc9c,0xd843dc9e,
	0xdfe1dc9e,0xd843dca0,0xdfe2dca0,0xd843dca2,0xdfe3dca2,0xd843dca4,0xdfe4dca4,0xd843dca6,
	0xdfe5dca6,0xd843dca8,0xdfe6dca8,0xd843dcaa,0xdfe7dcaa,0xd843dcac,0xdfe8dcac,0xd843dcae,
	0xdfe9dcae,0xd843dcb0,0xdfeadcb0,0xd843dcb2,0xdfebdcb2,0xd843dcb4,0xdfecdcb4,0xd843dcb6,
	0xdfeddcb6,0xd843dcb8,0xdfeedcb8,0xd843dcba,0xdfefdcba,0xd843dcbc,0xdff0dcbc,0xd843dcbe,
	0xdff1dcbe,0xd843dcc0,0xdff2dcc0,0xd843dcc2,0xdff3dcc2,0xd843dcc4,0xdff4dcc4,0xd843dcc6,
	0xdff5dcc6,0xd843dcc8,0xdff6dcc8,0xd843dcca,0xdff7dcca,0xd843dccc,0xdff8dccc,0xd843dcce,
	0xdff9dcce,0xd843dcd0,0xdffadcd0,0xd843dcd2,0xdffbdcd2,0xd843dcd4,0xdffcdcd4,0xd843dcd6,
	0xdffddcd6,0xd843dcd8,0xdffedcd8,0xd843dcda,0xdfffdcda,0xd844dcdc,0xdc00dcdc,0xd844dcde,
	0xdc01dcde,0xd844dce0,0xdc02dce0,0xd844dce2,0xdc03dce2,0xd844dce4,0xdc04dce4,0xd844dce6,
	0xdc05dce6,0xd844dce8,0xdc06dce8,0xd844dcea,0xdc07dcea,0xd844dcec,0xdc08dcec,0xd844dcee,
	0xdc09dcee,0xd844dcf0,0xdc0adcf0,0xd844dcf2,0xdc0bdcf2,0xd844dcf4,0xdc0cdcf4,0xd844dcf6,
	0xdc0ddcf6,0xd844dcf8,0xdc0edcf8,0xd844dcfa,0xdc0fdcfa,0xd844dcfc,0xdc10dcfc,0xd844dcfe,
	0xdc11dcfe,0xd844dd00,0xdc12dd00,0xd844dd02,0xdc13dd02,0xd844dd04,0xdc14dd04,0xd844dd06,
	0xdc15dd06,0xd844dd08,0xdc16dd08,0xd844dd0a,0xdc17dd0a,0xd844dd0c,0xdc18dd0c,0xd844dd0e,
	0xdc19dd0e,0xd844dd10,0xdc1add10,0xd844dd12,0xdc1bdd12,0xd844dd14,0xdc1cdd14,0xd844dd16,
	0xdc1ddd16,0xd844dd18,0xdc1edd18,0xd844dd1a,0xdc1fdd1a,0xd844dd1c,0xdc20dd1c,0xd844dd1e,
	0xdc21dd1e,0xd844dd20,0xdc22dd20,0xd844dd22,0xdc23dd22,0xd844dd24,0xdc24dd24,0xd844dd26,
	0xdc25dd26,0xd844dd28,0xdc26dd28,0xd844dd2a,0xdc27dd2a,0xd844dd2c,0xdc28dd2c,0xd844dd2e,
	0xdc29dd2e,0xd844dd30,0xdc2add30,0xd844dd32,0xdc2bdd32,0xd844dd34,0xdc2cdd34,0xd844dd36,
	0xdc2ddd36,0xd844dd38,0xdc2edd38,0xd844dd3a,0xdc2fdd3a,0xd844dd3c,0xdc30dd3c,0xd844dd3e,
	0xdc31dd3e,0xd844dd40,0xdc32dd40,0xd844dd42,0xdc33dd42,0xd844dd44,0xdc34dd44,0xd844dd46,
	0xdc35dd46,0xd844dd48,0xdc36dd48,0xd844dd4a,0xdc37dd4a,0xd844dd4c,0xdc38dd4c,0xd844dd4e,
	0xdc39dd4e,0xd844dd50,0xdc3add50,0xd844dd52,0xdc3bdd52,0xd844dd54,0xdc3cdd54,0xd844dd56,
	0xdc3ddd56,0xd844dd58,0xdc3edd58,0xd844dd5a,0xdc3fdd5a,0xd844dd5c,0xdc40dd5c,0xd844dd5e,
	0xdc41dd5e,0xd844dd60,0xdc42dd60,0xd844dd62,0xdc43dd62,0xd844dd64,0xdc44dd64,0xd844dd66,
	0xdc45dd66,0xd844dd68,0xdc46dd68,0xd844dd6a,0xdc47dd6a,0xd844dd6c,0xdc48dd6c,0xd844dd6e,
	0xdc49dd6e,0xd844dd70,0xdc4add70,0xd844dd72,0xdc4bdd72,0xd844dd74,0xdc4cdd74,0xd844dd76,
	0xdc4ddd76,0xd844dd78,0xdc4edd78,0xd844dd7a,0xdc4fdd7a,0xd844dd7c,0xdc50dd7c,0xd844dd7e,
	0xdc51dd7e,0xd844dd80,0xdc52dd80,0xd844dd82,0xdc53dd82,0xd844dd84,0xdc54dd84,0xd844dd86,
	0xdc55dd86,0xd844dd88,0xdc56dd88,0xd844dd8a,0xdc57dd8a,0xd844dd8c,0xdc58dd8c,0xd844dd8e,
	0xdc59dd8e,0xd844dd90,0xdc5add90,0xd844dd92,0xdc5bdd92,0xd844dd94,0xdc5cdd94,0xd844dd96,
	0xdc5ddd96,0xd844dd98,0xdc5edd98,0xd844dd9a,0xdc5fdd9a,0xd844dd9c,0xdc60dd9c,0xd844dd9e,
	0xdc61dd9e,0xd844dda0,0xdc62dda0,0xd844dda2,0xdc63dda2,0xd844dda4,0xdc64dda4,0xd844dda6,
	0xdc65dda6,0xd844dda8,0xdc66dda8,0xd844ddaa,0xdc67ddaa,0xd844ddac,0xdc68ddac,0xd844ddae,
	0xdc69ddae,0xd844ddb0,0xdc6addb0,0xd844ddb2,0xdc6bddb2,0xd844ddb4,0xdc6cddb4,0xd844ddb6,
	0xdc6dddb6,0xd844ddb8,0xdc6eddb8,0xd844ddba,0xdc6fddba,0xd844ddbc,0xdc70ddbc,0xd844ddbe,
	0xdc71ddbe,0xd844ddc0,0xdc72ddc0,0xd844ddc2,0xdc73ddc2,0xd844ddc4,0xdc74ddc4,0xd844ddc6,
	0xdc75ddc6,0xd844ddc8,0xdc76ddc8,0xd844ddca,0xdc77ddca,0xd844ddcc,0xdc78ddcc,0xd844ddce,
	0xdc79ddce,0xd844ddd0,0xdc7addd0,0xd844ddd2,0xdc7bddd2,0xd844ddd4,0xdc7cddd4,0xd844ddd6,
	0xdc7dddd6,0xd844ddd8,0xdc7eddd8,0xd844ddda,0xdc7fddda,0xd844dddc,0xdc80dddc,0xd844ddde,
	0xdc81ddde,0xd844dde0,0xdc82dde0,0xd844dde2,0xdc83dde2,0xd844dde4,0xdc84dde4,0xd844dde6,
	0xdc85dde6,0xd844dde8,0xdc86dde8,0xd844ddea,0xdc87ddea,0xd844ddec,0xdc88ddec,0xd844ddee,
	0xdc89ddee,0xd844ddf0,0xdc8addf0,0xd844ddf2,0xdc8bddf2,0xd844ddf4,0xdc8cddf4,0xd844ddf6,
	0xdc8dddf6,0xd844ddf8,0xdc8eddf8,0xd844ddfa,0xdc8fddfa,0xd844ddfc,0xdc90ddfc,0xd844ddfe,
	0xdc91ddfe,0xd844de00,0xdc92de00,0xd844de02,0xdc93de02,0xd844de04,0xdc94de04,0xd844de06,
	0xdc95de06,0xd844de08,0xdc96de08,0xd844de0a,0xdc97de0a,0xd844de0c,0xdc98de0c,0xd844de0e,
	0xdc99de0e,0xd844de10,0xdc9ade10,0xd844de12,0xdc9bde12,0xd844de14,0xdc9cde14,0xd844de16,
	0xdc9dde16,0xd844de18,0xdc9ede18,0xd844de1a,0xdc9fde1a,0xd844de1c,0xdca0de1c,0xd844de1e,
	0xdca1de1e,0xd844de20,0xdca2de20,0xd844de22,0xdca3de22,0xd844de24,0xdca4de24,0xd844de26,
	0xdca5de26,0xd844de28,0xdca6de28,0xd844de2a,0xdca7de2a,0xd844de2c,0xdca8de2c,0xd844de2e,
	0xdca9de2e,0xd844de30,0xdcaade30,0xd844de32,0xdcabde32,0xd844de34,0xdcacde34,0xd844de36,
	0xdcadde36,0xd844de38,0xdcaede38,0xd844de3a,0xdcafde3a,0xd844de3c,0xdcb0de3c,0xd844de3e,
	0xdcb1de3e,0xd844de40,0xdcb2de40,0xd844de42,0xdcb3de42,0xd844de44,0xdcb4de44,0xd844de46,
	0xdcb5de46,0xd844de48,0xdcb6de48,0xd844de4a,0xdcb7de4a,0xd844de4c,0xdcb8de4c,0xd844de4e,
	0xdcb9de4e,0xd844de50,0xdcbade50,0xd844de52,0xdcbbde52,0xd844de54,0xdcbcde54,0xd844de56,
	0xdcbdde56,0xd844de58,0xdcbede58,0xd844de5a,0xdcbfde5a,0xd844de5c,0xdcc0de5c,0xd844de5e,
	0xdcc1de5e,0xd844de60,0xdcc2de60,0xd844de62,0xdcc3de62,0xd844de64,0xdcc4de64,0xd844de66,
	0xdcc5de66,0xd844de68,0xdcc6de68,0xd844de6a,0xdcc7de6a,0xd844de6c,0xdcc8de6c,0xd844de6e,
	0xdcc9de6e,0xd844de70,0xdccade70,0xd844de72,0xdccbde72,0xd844de74,0xdcccde74,0xd844de76,
	0xdccdde76,0xd844de78,0xdccede78,0xd844de7a,0xdccfde7a,0xd844de7c,0xdcd0de7c,0xd844de7e,
	0xdcd1de7e,0xd844de80,0xdcd2de80,0xd844de82,0xdcd3de82,0xd844de84,0xdcd4de84,0xd844de86,
	0xdcd5de86,0xd844de88,0xdcd6de88,0xd844de8a,0xdcd7de8a,0xd844de8c,0xdcd8de8c,0xd844de8e,
	0xdcd9de8e,0xd844de90,0xdcdade90,0xd844de92,0xdcdbde92,0xd844de94,0xdcdcde94,0xd844de96,
	0xdcddde96,0xd844de98,0xdcdede98,0xd844de9a,0xdcdfde9a,0xd844de9c,0xdce0de9c,0xd844de9e,
	0xdce1de9e,0xd844dea0,0xdce2dea0,0xd844dea2,0xdce3dea2,0xd844dea4,0xdce4dea4,0xd844dea6,
	0xdce5dea6,0xd844dea8,0xdce6dea8,0xd844deaa,0xdce7deaa,0xd844deac,0xdce8deac,0xd844deae,
	0xdce9deae,0xd844deb0,0xdceadeb0,0xd844deb2,0xdcebdeb2,0xd844deb4,0xdcecdeb4,0xd844deb6,
	0xdceddeb6,0xd844deb8,0xdceedeb8,0xd844deba,0xdcefdeba,0xd844debc,0xdcf0debc,0xd844debe,
	0xdcf1debe,0xd844dec0,0xdcf2dec0,0xd844dec2,0xdcf3dec2,0xd844dec4,0xdcf4dec4,0xd844dec6,
	0xdcf5dec6,0xd844dec8,0xdcf6dec8,0xd844deca,0xdcf7deca,0xd844decc,0xdcf8decc,0xd844dece,
	0xdcf9dece,0xd844ded0,0xdcfaded0,0xd844ded2,0xdcfbded2,0xd844ded4,0xdcfcded4,0xd844ded6,
	0xdcfdded6,0xd844ded8,0xdcfeded8,0xd844deda,0xdcffdeda,0xd844dedc,0xdd00dedc,0xd844dede,
	0xdd01dede,0xd844dee0,0xdd02dee0,0xd844dee2,0xdd03dee2,0xd844dee4,0xdd04dee4,0xd844dee6,
	0xdd05dee6,0xd844dee8,0xdd06dee8,0xd844deea,0xdd07deea,0xd844deec,0xdd08deec,0xd844deee,
	0xdd09deee,0xd844def0,0xdd0adef0,0xd844def2,0xdd0bdef2,0xd844def4,0xdd0cdef4,0xd844def6,
	0xdd0ddef6,0xd844def8,0xdd0edef8,0xd844defa,0xdd0fdefa,0xd844defc,0xdd10defc,0xd844defe,
	0xdd11defe,0xd844df00,0xdd12df00,0xd844df02,0xdd13df02,0xd844df04,0xdd14df04,0xd844df06,
	0xdd15df06,0xd844df08,0xdd16df08,0xd844df0a,0xdd17df0a,0xd844df0c,0xdd18df0c,0xd844df0e,
	0xdd19df0e,0xd844df10,0xdd1adf10,0xd844df12,0xdd1bdf12,0xd844df14,0xdd1cdf14,0xd844df16,
	0xdd1ddf16,0xd844df18,0xdd1edf18,0xd844df1a,0xdd1fdf1a,0xd844df1c,0xdd20df1c,0xd844df1e,
	0xdd21df1e,0xd844df20,0xdd22df20,0xd844df22,0xdd23df22,0xd844df24,0xdd24df24,0xd844df26,
	0xdd25df26,0xd844df28,0xdd26df28,0xd844df2a,0xdd27df2a,0xd844df2c,0xdd28df2c,0xd844df2e,
	0xdd29df2e,0xd844df30,0xdd2adf30,0xd844df32,0xdd2bdf32,0xd844df34,0xdd2cdf34,0xd844df36,
	0xdd2ddf36,0xd844df38,0xdd2edf38,0xd844df3a,0xdd2fdf3a,0xd844df3c,0xdd30df3c,0xd844df3e,
	0xdd31df3e,0xd844df40,0xdd32df40,0xd844df42,0xdd33df42,0xd844df44,0xdd34df44,0xd844df46,
	0xdd35df46,0xd844df48,0xdd36df48,0xd844df4a,0xdd37df4a,0xd844df4c,0xdd38df4c,0xd844df4e,
	0xdd39df4e,0xd844df50,0xdd3adf50,0xd844df52,0xdd3bdf52,0xd844df54,0xdd3cdf54,0xd844df56,
	0xdd3ddf56,0xd844df58,0xdd3edf58,0xd844df5a,0xdd3fdf5a,0xd844df5c,0xdd40df5c,0xd844df5e,
	0xdd41df5e,0xd844df60,0xdd42df60,0xd844df62,0xdd43df62,0xd844df64,0xdd44df64,0xd844df66,
	0xdd45df66,0xd844df68,0xdd46df68,0xd844df6a,0xdd47df6a,0xd844df6c,0xdd48df6c,0xd844df6e,
	0xdd49df6e,0xd844df70,0xdd4adf70,0xd844df72,0xdd4bdf72,0xd844df74,0xdd4cdf74,0xd844df76,
	0xdd4ddf76,0xd844df78,0xdd4edf78,0xd844df7a,0xdd4fdf7a,0xd844df7c,0xdd50df7c,0xd844df7e,
	0xdd51df7e,0xd844df80,0xdd52df80,0xd844df82,0xdd53df82,0xd844df84,0xdd54df84,0xd844df86,
	0xdd55df86,0xd844df88,0xdd56df88,0xd844df8a,0xdd57df8a,0xd844df8c,0xdd58df8c,0xd844df8e,
	0xdd59df8e,0xd844df90,0xdd5adf90,0xd844df92,0xdd5bdf92,0xd844df94,0xdd5cdf94,0xd844df96,
	0xdd5ddf96,0xd844df98,0xdd5edf98,0xd844df9a,0xdd5fdf9a,0xd844df9c,0xdd60df9c,0xd844df9e,
	0xdd61df9e,0xd844dfa0,0xdd62dfa0,0xd844dfa2,0xdd63dfa2,0xd844dfa4,0xdd64dfa4,0xd844dfa6,
	0xdd65dfa6,0xd844dfa8,0xdd66dfa8,0xd844dfaa,0xdd67dfaa,0xd844dfac,0xdd68dfac,0xd844dfae,
	0xdd69dfae,0xd844dfb0,0xdd6adfb0,0xd844dfb2,0xdd6bdfb2,0xd844dfb4,0xdd6cdfb4,0xd844dfb6,
	0xdd6ddfb6,0xd844dfb8,0xdd6edfb8,0xd844dfba,0xdd6fdfba,0xd844dfbc,0xdd70dfbc,0xd844dfbe,
	0xdd71dfbe,0xd844dfc0,0xdd72dfc0,0xd844dfc2,0xdd73dfc2,0xd844dfc4,0xdd74dfc4,0xd844dfc6,
	0xdd75dfc6,0xd844dfc8,0xdd76dfc8,0xd844dfca,0xdd77dfca,0xd844dfcc,0xdd78dfcc,0xd844dfce,
	0xdd79dfce,0xd844dfd0,0xdd7adfd0,0xd844dfd2,0xdd7bdfd2,0xd844dfd4,0xdd7cdfd4,0xd844dfd6,
	0xdd7ddfd6,0xd844dfd8,0xdd7edfd8,0xd844dfda,0xdd7fdfda,0xd844dfdc,0xdd80dfdc,0xd844dfde,
	0xdd81dfde,0xd844dfe0,0xdd82dfe0,0xd844dfe2,0xdd83dfe2,0xd844dfe4,0xdd84dfe4,0xd844dfe6,
	0xdd85dfe6,0xd844dfe8,0xdd86dfe8,0xd844dfea,0xdd87dfea,0xd844dfec,0xdd88dfec,0xd844dfee,
	0xdd89dfee,0xd844dff0,0xdd8adff0,0xd844dff2,0xdd8bdff2,0xd844dff4,0xdd8cdff4,0xd844dff6,
	0xdd8ddff6,0xd844dff8,0xdd8edff8,0xd844dffa,0xdd8fdffa,0xd844dffc,0xdd90dffc,0xd844dffe,
	0xdd91dffe,0xd844e000,0xdd92e000,0xd844e002,0xdd93e002,0xd844e004,0xdd94e004,0xd844e006,
	0xdd95e006,0xd844e008,0xdd96e008,0xd844e00a,0xdd97e00a,0xd844e00c,0xdd98e00c,0xd844e00e,
	0xdd99e00e,0xd844e010,0xdd9ae010,0xd844e012,0xdd9be012,0xd844e014,0xdd9ce014,0xd844e016,
	0xdd9de016,0xd844e018,0xdd9ee018,0xd844e01a,0xdd9fe01a,0xd844e01c,0xdda0e01c,0xd844e01e,
	0xdda1e01e,0xd844e020,0xdda2e020,0xd844e022,0xdda3e022,0xd844e024,0xdda4e024,0xd844e026,
	0xdda5e026,0xd844e028,0xdda6e028,0xd844e02a,0xdda7e02a,0xd844e02c,0xdda8e02c,0xd844e02e,
	0xdda9e02e,0xd844e030,0xddaae030,0xd844e032,0xddabe032,0xd844e034,0xddace034,0xd844e036,
	0xddade036,0xd844e038,0xddaee038,0xd844e03a,0xddafe03a,0xd844e03c,0xddb0e03c,0xd844e03e,
	0xddb1e03e,0xd844e040,0xddb2e040,0xd844e042,0xddb3e042,0xd844e044,0xddb4e044,0xd844e046,
	0xddb5e046,0xd844e048,0xddb6e048,0xd844e04a,0xddb7e04a,0xd844e04c,0xddb8e04c,0xd844e04e,
	0xddb9e04e,0xd844e050,0xddbae050,0xd844e052,0xddbbe052,0xd844e054,0xddbce054,0xd844e056,
	0xddbde056,0xd844e058,0xddbee058,0xd844e05a,0xddbfe05a,0xd844e05c,0xddc0e05c,0xd844e05e,
	0xddc1e05e,0xd844e060,0xddc2e060,0xd844e062,0xddc3e062,0xd844e064,0xddc4e064,0xd844e066,
	0xddc5e066,0xd844e068,0xddc6e068,0xd844e06a,0xddc7e06a,0xd844e06c,0xddc8e06c,0xd844e06e,
	0xddc9e06e,0xd844e070,0xddcae070,0xd844e072,0xddcbe072,0xd844e074,0xddcce074,0xd844e076,
	0xddcde076,0xd844e078,0xddcee078,0xd844e07a,0xddcfe07a,0xd844e07c,0xddd0e07c,0xd844e07e,
	0xddd1e07e,0xd844e080,0xddd2e080,0xd844e082,0xddd3e082,0xd844e084,0xddd4e084,0xd844e086,
	0xddd5e086,0xd844e088,0xddd6e088,0xd844e08a,0xddd7e08a,0xd844e08c,0xddd8e08c,0xd844e08e,
	0xddd9e08e,0xd844e090,0xdddae090,0xd844e092,0xdddbe092,0xd844e094,0xdddce094,0xd844e096,
	0xdddde096,0xd844e098,0xdddee098,0xd844e09a,0xdddfe09a,0xd844e09c,0xdde0e09c,0xd844e09e,
	0xdde1e09e,0xd844e0a0,0xdde2e0a0,0xd844e0a2,0xdde3e0a2,0xd844e0a4,0xdde4e0a4,0xd844e0a6,
	0xdde5e0a6,0xd844e0a8,0xdde6e0a8,0xd844e0aa,0xdde7e0aa,0xd844e0ac,0xdde8e0ac,0xd844e0ae,
	0xdde9e0ae,0xd844e0b0,0xddeae0b0,0xd844e0b2,0xddebe0b2,0xd844e0b4,0xddece0b4,0xd844e0b6,
	0xddede0b6,0xd844e0b8,0xddeee0b8,0xd844e0ba,0xddefe0ba,0xd844e0bc,0xddf0e0bc,0xd844e0be,
	0xddf1e0be,0xd844e0c0,0xddf2e0c0,0xd844e0c2,0xddf3e0c2,0xd844e0c4,0xddf4e0c4,0xd844e0c6,
	0xddf5e0c6,0xd844e0c8,0xddf6e0c8,0xd844e0ca,0xddf7e0ca,0xd844e0cc,0xddf8e0cc,0xd844e0ce,
	0xddf9e0ce,0xd844e0d0,0xddfae0d0,0xd844e0d2,0xddfbe0d2,0xd844e0d4,0xddfce0d4,0xd844e0d6,
	0xddfde0d6,0xd844e0d8,0xddfee0d8,0xd844e0da,0xddffe0da,0xd844e0dc,0xde00e0dc,0xd844e0de,
	0xde01e0de,0xd844e0e0,0xde02e0e0,0xd844e0e2,0xde03e0e2,0xd844e0e4,0xde04e0e4,0xd844e0e6,
	0xde05e0e6,0xd844e0e8,0xde06e0e8,0xd844e0ea,0xde07e0ea,0xd844e0ec,0xde08e0ec,0xd844e0ee,
	0xde09e0ee,0xd844e0f0,0xde0ae0f0,0xd844e0f2,0xde0be0f2,0xd844e0f4,0xde0ce0f4,0xd844e0f6,
	0xde0de0f6,0xd844e0f8,0xde0ee0f8,0xd844e0fa,0xde0fe0fa,0xd844e0fc,0xde10e0fc,0xd844e0fe,
	0xde11e0fe,0xd844e100,0xde12e100,0xd844e102,0xde13e102,0xd844e104,0xde14e104,0xd844e106,
	0xde15e106,0xd844e108,0xde16e108,0xd844e10a,0xde17e10a,0xd844e10c,0xde18e10c,0xd844e10e,
	0xde19e10e,0xd844e110,0xde1ae110,0xd844e112,0xde1be112,0xd844e114,0xde1ce114,0xd844e116,
	0xde1de116,0xd844e118,0xde1ee118,0xd844e11a,0xde1fe11a,0xd844e11c,0xde20e11c,0xd844e11e,
	0xde21e11e,0xd844e120,0xde22e120,0xd844e122,0xde23e122,0xd844e124,0xde24e124,0xd844e126,
	0xde25e126,0xd844e128,0xde26e128,0xd844e12a,0xde27e12a,0xd844e12c,0xde28e12c,0xd844e12e,
	0xde29e12e,0xd844e130,0xde2ae130,0xd844e132,0xde2be132,0xd844e134,0xde2ce134,0xd844e136,
	0xde2de136,0xd844e138,0xde2ee138,0xd844e13a,0xde2fe13a,0xd844e13c,0xde30e13c,0xd844e13e,
	0xde31e13e,0xd844e140,0xde32e140,0xd844e142,0xde33e142,0xd844e144,0xde34e144,0xd844e146,
	0xde35e146,0xd844e148,0xde36e148,0xd844e14a,0xde37e14a,0xd844e14c,0xde38e14c,0xd844e14e,
	0xde39e14e,0xd844e150,0xde3ae150,0xd844e152,0xde3be152,0xd844e154,0xde3ce154,0xd844e156,
	0xde3de156,0xd844e158,0xde3ee158,0xd844e15a,0xde3fe15a,0xd844e15c,0xde40e15c,0xd844e15e,
	0xde41e15e,0xd844e160,0xde42e160,0xd844e162,0xde43e162,0xd844e164,0xde44e164,0xd844e166,
	0xde45e166,0xd844e168,0xde46e168,0xd844e16a,0xde47e16a,0xd844e16c,0xde48e16c,0xd844e16e,
	0xde49e16e,0xd844e170,0xde4ae170,0xd844e172,0xde4be172,0xd844e174,0xde4ce174,0xd844e176,
	0xde4de176,0xd844e178,0xde4ee178,0xd844e17a,0xde4fe17a,0xd844e17c,0xde50e17c,0xd844e17e,
	0xde51e17e,0xd844e180,0xde52e180,0xd844e182,0xde53e182,0xd844e184,0xde54e184,0xd844e186,
	0xde55e186,0xd844e188,0xde56e188,0xd844e18a,0xde57e18a,0xd844e18c,0xde58e18c,0xd844e18e,
	0xde59e18e,0xd844e190,0xde5ae190,0xd844e192,0xde5be192,0xd844e194,0xde5ce194,0xd844e196,
	0xde5de196,0xd844e198,0xde5ee198,0xd844e19a,0xde5fe19a,0xd844e19c,0xde60e19c,0xd844e19e,
	0xde61e19e,0xd844e1a0,0xde62e1a0,0xd844e1a2,0xde63e1a2,0xd844e1a4,0xde64e1a4,0xd844e1a6,
	0xde65e1a6,0xd844e1a8,0xde66e1a8,0xd844e1aa,0xde67e1aa,0xd844e1ac,0xde68e1ac,0xd844e1ae,
	0xde69e1ae,0xd844e1b0,0xde6ae1b0,0xd844e1b2,0xde6be1b2,0xd844e1b4,0xde6ce1b4,0xd844e1b6,
	0xde6de1b6,0xd844e1b8,0xde6ee1b8,0xd844e1ba,0xde6fe1ba,0xd844e1bc,0xde70e1bc,0xd844e1be,
	0xde71e1be,0xd844e1c0,0xde72e1c0,0xd844e1c2,0xde73e1c2,0xd844e1c4,0xde74e1c4,0xd844e1c6,
	0xde75e1c6,0xd844e1c8,0xde76e1c8,0xd844e1ca,0xde77e1ca,0xd844e1cc,0xde78e1cc,0xd844e1ce,
	0xde79e1ce,0xd844e1d0,0xde7ae1d0,0xd844e1d2,0xde7be1d2,0xd844e1d4,0xde7ce1d4,0xd844e1d6,
	0xde7de1d6,0xd844e1d8,0xde7ee1d8,0xd844e1da,0xde7fe1da,0xd844e1dc,0xde80e1dc,0xd844e1de,
	0xde81e1de,0xd844e1e0,0xde82e1e0,0xd844e1e2,0xde83e1e2,0xd844e1e4,0xde84e1e4,0xd844e1e6,
	0xde85e1e6,0xd844e1e8,0xde86e1e8,0xd844e1ea,0xde87e1ea,0xd844e1ec,0xde88e1ec,0xd844e1ee,
	0xde89e1ee,0xd844e1f0,0xde8ae1f0,0xd844e1f2,0xde8be1f2,0xd844e1f4,0xde8ce1f4,0xd844e1f6,
	0xde8de1f6,0xd844e1f8,0xde8ee1f8,0xd844e1fa,0xde8fe1fa,0xd844e1fc,0xde90e1fc,0xd844e1fe,
	0xde91e1fe,0xd844e200,0xde92e200,0xd844e202,0xde93e202,0xd844e204,0xde94e204,0xd844e206,
	0xde95e206,0xd844e208,0xde96e208,0xd844e20a,0xde97e20a,0xd844e20c,0xde98e20c,0xd844e20e,
	0xde99e20e,0xd844e210,0xde9ae210,0xd844e212,0xde9be212,0xd844e214,0xde9ce214,0xd844e216,
	0xde9de216,0xd844e218,0xde9ee218,0xd844e21a,0xde9fe21a,0xd844e21c,0xdea0e21c,0xd844e21e,
	0xdea1e21e,0xd844e220,0xdea2e220,0xd844e222,0xdea3e222,0xd844e224,0xdea4e224,0xd844e226,
	0xdea5e226,0xd844e228,0xdea6e228,0xd844e22a,0xdea7e22a,0xd844e22c,0xdea8e22c,0xd844e22e,
	0xdea9e22e,0xd844e230,0xdeaae230,0xd844e232,0xdeabe232,0xd844e234,0xdeace234,0xd844e236,
	0xdeade236,0xd844e238,0xdeaee238,0xd844e23a,0xdeafe23a,0xd844e23c,0xdeb0e23c,0xd844e23e,
	0xdeb1e23e,0xd844e240,0xdeb2e240,0xd844e242,0xdeb3e242,0xd844e244,0xdeb4e244,0xd844e246,
	0xdeb5e246,0xd844e248,0xdeb6e248,0xd844e24a,0xdeb7e24a,0xd844e24c,0xdeb8e24c,0xd844e24e,
	0xdeb9e24e,0xd844e250,0xdebae250,0xd844e252,0xdebbe252,0xd844e254,0xdebce254,0xd844e256,
	0xdebde256,0xd844e258,0xdebee258,0xd844e25a,0xdebfe25a,0xd844e25c,0xdec0e25c,0xd844e25e,
	0xdec1e25e,0xd844e260,0xdec2e260,0xd844e262,0xdec3e262,0xd844e264,0xdec4e264,0xd844e266,
	0xdec5e266,0xd844e268,0xdec6e268,0xd844e26a,0xdec7e26a,0xd844e26c,0xdec8e26c,0xd844e26e,
	0xdec9e26e,0xd844e270,0xdecae270,0xd844e272,0xdecbe272,0xd844e274,0xdecce274,0xd844e276,
	0xdecde276,0xd844e278,0xdecee278,0xd844e27a,0xdecfe27a,0xd844e27c,0xded0e27c,0xd844e27e,
	0xded1e27e,0xd844e280,0xded2e280,0xd844e282,0xded3e282,0xd844e284,0xded4e284,0xd844e286,
	0xded5e286,0xd844e288,0xded6e288,0xd844e28a,0xded7e28a,0xd844e28c,0xded8e28c,0xd844e28e,
	0xded9e28e,0xd844e290,0xdedae290,0xd844e292,0xdedbe292,0xd844e294,0xdedce294,0xd844e296,
	0xdedde296,0xd844e298,0xdedee298,0xd844e29a,0xdedfe29a,0xd844e29c,0xdee0e29c,0xd844e29e,
	0xdee1e29e,0xd844e2a0,0xdee2e2a0,0xd844e2a2,0xdee3e2a2,0xd844e2a4,0xdee4e2a4,0xd844e2a6,
	0xdee5e2a6,0xd844e2a8,0xdee6e2a8,0xd844e2aa,0xdee7e2aa,0xd844e2ac,0xdee8e2ac,0xd844e2ae,
	0xdee9e2ae,0xd844e2b0,0xdeeae2b0,0xd844e2b2,0xdeebe2b2,0xd844e2b4,0xdeece2b4,0xd844e2b6,
	0xdeede2b6,0xd844e2b8,0xdeeee2b8,0xd844e2ba,0xdeefe2ba,0xd844e2bc,0xdef0e2bc,0xd844e2be,
	0xdef1e2be,0xd844e2c0,0xdef2e2c0,0xd844e2c2,0xdef3e2c2,0xd844e2c4,0xdef4e2c4,0xd844e2c6,
	0xdef5e2c6,0xd844e2c8,0xdef6e2c8,0xd844e2ca,0xdef7e2ca,0xd844e2cc,0xdef8e2cc,0xd844e2ce,
	0xdef9e2ce,0xd844e2d0,0xdefae2d0,0xd844e2d2,0xdefbe2d2,0xd844e2d4,0xdefce2d4,0xd844e2d6,
	0xdefde2d6,0xd844e2d8,0xdefee2d8,0xd844e2da,0xdeffe2da,0xd844e2dc,0xdf00e2dc,0xd844e2de,
	0xdf01e2de,0xd844e2e0,0xdf02e2e0,0xd844e2e2,0xdf03e2e2,0xd844e2e4,0xdf04e2e4,0xd844e2e6,
	0xdf05e2e6,0xd844e2e8,0xdf06e2e8,0xd844e2ea,0xdf07e2ea,0xd844e2ec,0xdf08e2ec,0xd844e2ee,
	0xdf09e2ee,0xd844e2f0,0xdf0ae2f0,0xd844e2f2,0xdf0be2f2,0xd844e2f4,0xdf0ce2f4,0xd844e2f6,
	0xdf0de2f6,0xd844e2f8,0xdf0ee2f8,0xd844e2fa,0xdf0fe2fa,0xd844e2fc,0xdf10e2fc,0xd844e2fe,
	0xdf11e2fe,0xd844e300,0xdf12e300,0xd844e302,0xdf13e302,0xd844e304,0xdf14e304,0xd844e306,
	0xdf15e306,0xd844e308,0xdf16e308,0xd844e30a,0xdf17e30a,0xd844e30c,0xdf18e30c,0xd844e30e,
	0xdf19e30e,0xd844e310,0xdf1ae310,0xd844e312,0xdf1be312,0xd844e314,0xdf1ce314,0xd844e316,
	0xdf1de316,0xd844e318,0xdf1ee318,0xd844e31a,0xdf1fe31a,0xd844e31c,0xdf20e31c,0xd844e31e,
	0xdf21e31e,0xd844e320,0xdf22e320,0xd844e322,0xdf23e322,0xd844e324,0xdf24e324,0xd844e326,
	0xdf25e326,0xd844e328,0xdf26e328,0xd844e32a,0xdf27e32a,0xd844e32c,0xdf28e32c,0xd844e32e,
	0xdf29e32e,0xd844e330,0xdf2ae330,0xd844e332,0xdf2be332,0xd844e334,0xdf2ce334,0xd844e336,
	0xdf2de336,0xd844e338,0xdf2ee338,0xd844e33a,0xdf2fe33a,0xd844e33c,0xdf30e33c,0xd844e33e,
	0xdf31e33e,0xd844e340,0xdf32e340,0xd844e342,0xdf33e342,0xd844e344,0xdf34e344,0xd844e346,
	0xdf35e346,0xd844e348,0xdf36e348,0xd844e34a,0xdf37e34a,0xd844e34c,0xdf38e34c,0xd844e34e,
	0xdf39e34e,0xd844e350,0xdf3ae350,0xd844e352,0xdf3be352,0xd844e354,0xdf3ce354,0xd844e356,
	0xdf3de356,0xd844e358,0xdf3ee358,0xd844e35a,0xdf3fe35a,0xd844e35c,0xdf40e35c,0xd844e35e,
	0xdf41e35e,0xd844e360,0xdf42e360,0xd844e362,0xdf43e362,0xd844e364,0xdf44e364,0xd844e366,
	0xdf45e366,0xd844e368,0xdf46e368,0xd844e36a,0xdf47e36a,0xd844e36c,0xdf48e36c,0xd844e36e,
	0xdf49e36e,0xd844e370,0xdf4ae370,0xd844e372,0xdf4be372,0xd844e374,0xdf4ce374,0xd844e376,
	0xdf4de376,0xd844e378,0xdf4ee378,0xd844e37a,0xdf4fe37a,0xd844e37c,0xdf50e37c,0xd844e37e,
	0xdf51e37e,0xd844e380,0xdf52e380,0xd844e382,0xdf53e382,0xd844e384,0xdf54e384,0xd844e386,
	0xdf55e386,0xd844e388,0xdf56e388,0xd844e38a,0xdf57e38a,0xd844e38c,0xdf58e38c,0xd844e38e,
	0xdf59e38e,0xd844e390,0xdf5ae390,0xd844e392,0xdf5be392,0xd844e394,0xdf5ce394,0xd844e396,
	0xdf5de396,0xd844e398,0xdf5ee398,0xd844e39a,0xdf5fe39a,0xd844e39c,0xdf60e39c,0xd844e39e,
	0xdf61e39e,0xd844e3a0,0xdf62e3a0,0xd844e3a2,0xdf63e3a2,0xd844e3a4,0xdf64e3a4,0xd844e3a6,
	0xdf65e3a6,0xd844e3a8,0xdf66e3a8,0xd844e3aa,0xdf67e3aa,0xd844e3ac,0xdf68e3ac,0xd844e3ae,
	0xdf69e3ae,0xd844e3b0,0xdf6ae3b0,0xd844e3b2,0xdf6be3b2,0xd844e3b4,0xdf6ce3b4,0xd844e3b6,
	0xdf6de3b6,0xd844e3b8,0xdf6ee3b8,0xd844e3ba,0xdf6fe3ba,0xd844e3bc,0xdf70e3bc,0xd844e3be,
	0xdf71e3be,0xd844e3c0,0xdf72e3c0,0xd844e3c2,0xdf73e3c2,0xd844e3c4,0xdf74e3c4,0xd844e3c6,
	0xdf75e3c6,0xd844e3c8,0xdf76e3c8,0xd844e3ca,0xdf77e3ca,0xd844e3cc,0xdf78e3cc,0xd844e3ce,
	0xdf79e3ce,0xd844e3d0,0xdf7ae3d0,0xd844e3d2,0xdf7be3d2,0xd844e3d4,0xdf7ce3d4,0xd844e3d6,
	0xdf7de3d6,0xd844e3d8,0xdf7ee3d8,0xd844e3da,0xdf7fe3da,0xd844e3dc,0xdf80e3dc,0xd844e3de,
	0xdf81e3de,0xd844e3e0,0xdf82e3e0,0xd844e3e2,0xdf83e3e2,0xd844e3e4,0xdf84e3e4,0xd844e3e6,
	0xdf85e3e6,0xd844e3e8,0xdf86e3e8,0xd844e3ea,0xdf87e3ea,0xd844e3ec,0xdf88e3ec,0xd844e3ee,
	0xdf89e3ee,0xd844e3f0,0xdf8ae3f0,0xd844e3f2,0xdf8be3f2,0xd844e3f4,0xdf8ce3f4,0xd844e3f6,
	0xdf8de3f6,0xd844e3f8,0xdf8ee3f8,0xd844e3fa,0xdf8fe3fa,0xd844e3fc,0xdf90e3fc,0xd844e3fe,
	0xdf91e3fe,0xd844e400,0xdf92e400,0xd844e402,0xdf93e402,0xd844e404,0xdf94e404,0xd844e406,
	0xdf95e406,0xd844e408,0xdf96e408,0xd844e40a,0xdf97e40a,0xd844e40c,0xdf98e40c,0xd844e40e,
	0xdf99e40e,0xd844e410,0xdf9ae410,0xd844e412,0xdf9be412,0xd844e414,0xdf9ce414,0xd844e416,
	0xdf9de416,0xd844e418,0xdf9ee418,0xd844e41a,0xdf9fe41a,0xd844e41c,0xdfa0e41c,0xd844e41e,
	0xdfa1e41e,0xd844e420,0xdfa2e420,0xd844e422,0xdfa3e422,0xd844e424,0xdfa4e424,0xd844e426,
	0xdfa5e426,0xd844e428,0xdfa6e428,0xd844e42a,0xdfa7e42a,0xd844e42c,0xdfa8e42c,0xd844e42e,
	0xdfa9e42e,0xd844e430,0xdfaae430,0xd844e432,0xdfabe432,0xd844e434,0xdface434,0xd844e436,
	0xdfade436,0xd844e438,0xdfaee438,0xd844e43a,0xdfafe43a,0xd844e43c,0xdfb0e43c,0xd844e43e,
	0xdfb1e43e,0xd844e440,0xdfb2e440,0xd844e442,0xdfb3e442,0xd844e444,0xdfb4e444,0xd844e446,
	0xdfb5e446,0xd844e448,0xdfb6e448,0xd844e44a,0xdfb7e44a,0xd844e44c,0xdfb8e44c,0xd844e44e,
	0xdfb9e44e,0xd844e450,0xdfbae450,0xd844e452,0xdfbbe452,0xd844e454,0xdfbce454,0xd844e456,
	0xdfbde456,0xd844e458,0xdfbee458,0xd844e45a,0xdfbfe45a,0xd844e45c,0xdfc0e45c,0xd844e45e,
	0xdfc1e45e,0xd844e460,0xdfc2e460,0xd844e462,0xdfc3e462,0xd844e464,0xdfc4e464,0xd844e466,
	0xdfc5e466,0xd844e468,0xdfc6e468,0xd844e46a,0xdfc7e46a,0xd844e46c,0xdfc8e46c,0xd844e46e,
	0xdfc9e46e,0xd844e470,0xdfcae470,0xd844e472,0xdfcbe472,0xd844e474,0xdfcce474,0xd844e476,
	0xdfcde476,0xd844e478,0xdfcee478,0xd844e47a,0xdfcfe47a,0xd844e47c,0xdfd0e47c,0xd844e47e,
	0xdfd1e47e,0xd844e480,0xdfd2e480,0xd844e482,0xdfd3e482,0xd844e484,0xdfd4e484,0xd844e486,
	0xdfd5e486,0xd844e488,0xdfd6e488,0xd844e48a,0xdfd7e48a,0xd844e48c,0xdfd8e48c,0xd844e48e,
	0xdfd9e48e,0xd844e490,0xdfdae490,0xd844e492,0xdfdbe492,0xd844e494,0xdfdce494,0xd844e496,
	0xdfdde496,0xd844e498,0xdfdee498,0xd844e49a,0xdfdfe49a,0xd844e49c,0xdfe0e49c,0xd844e49e,
	0xdfe1e49e,0xd844e4a0,0xdfe2e4a0,0xd844e4a2,0xdfe3e4a2,0xd844e4a4,0xdfe4e4a4,0xd844e4a6,
	0xdfe5e4a6,0xd844e4a8,0xdfe6e4a8,0xd844e4aa,0xdfe7e4aa,0xd844e4ac,0xdfe8e4ac,0xd844e4ae,
	0xdfe9e4ae,0xd844e4b0,0xdfeae4b0,0xd844e4b2,0xdfebe4b2,0xd844e4b4,0xdfece4b4,0xd844e4b6,
	0xdfede4b6,0xd844e4b8,0xdfeee4b8,0xd844e4ba,0xdfefe4ba,0xd844e4bc,0xdff0e4bc,0xd844e4be,
	0xdff1e4be,0xd844e4c0,0xdff2e4c0,0xd844e4c2,0xdff3e4c2,0xd844e4c4,0xdff4e4c4,0xd844e4c6,
	0xdff5e4c6,0xd844e4c8,0xdff6e4c8,0xd844e4ca,0xdff7e4ca,0xd844e4cc,0xdff8e4cc,0xd844e4ce,
	0xdff9e4ce,0xd844e4d0,0xdffae4d0,0xd844e4d2,0xdffbe4d2,0xd844e4d4,0xdffce4d4,0xd844e4d6,
	0xdffde4d6,0xd844e4d8,0xdffee4d8,0xd844e4da,0xdfffe4da,0xd845e4dc,0xdc00e4dc,0xd845e4de,
	0xdc01e4de,0xd845e4e0,0xdc02e4e0,0xd845e4e2,0xdc03e4e2,0xd845e4e4,0xdc04e4e4,0xd845e4e6,
	0xdc05e4e6,0xd845e4e8,0xdc06e4e8,0xd845e4ea,0xdc07e4ea,0xd845e4ec,0xdc08e4ec,0xd845e4ee,
	0xdc09e4ee,0xd845e4f0,0xdc0ae4f0,0xd845e4f2,0xdc0be4f2,0xd845e4f4,0xdc0ce4f4,0xd845e4f6,
	0xdc0de4f6,0xd845e4f8,0xdc0ee4f8,0xd845e4fa,0xdc0fe4fa,0xd845e4fc,0xdc10e4fc,0xd845e4fe,
	0xdc11e4fe,0xd845e500,0xdc12e500,0xd845e502,0xdc13e502,0xd845e504,0xdc14e504,0xd845e506,
	0xdc15e506,0xd845e508,0xdc16e508,0xd845e50a,0xdc17e50a,0xd845e50c,0xdc18e50c,0xd845e50e,
	0xdc19e50e,0xd845e510,0xdc1ae510,0xd845e512,0xdc1be512,0xd845e514,0xdc1ce514,0xd845e516,
	0xdc1de516,0xd845e518,0xdc1ee518,0xd845e51a,0xdc1fe51a,0xd845e51c,0xdc20e51c,0xd845e51e,
	0xdc21e51e,0xd845e520,0xdc22e520,0xd845e522,0xdc23e522,0xd845e524,0xdc24e524,0xd845e526,
	0xdc25e526,0xd845e528,0xdc26e528,0xd845e52a,0xdc27e52a,0xd845e52c,0xdc28e52c,0xd845e52e,
	0xdc29e52e,0xd845e530,0xdc2ae530,0xd845e532,0xdc2be532,0xd845e534,0xdc2ce534,0xd845e536,
	0xdc2de536,0xd845e538,0xdc2ee538,0xd845e53a,0xdc2fe53a,0xd845e53c,0xdc30e53c,0xd845e53e,
	0xdc31e53e,0xd845e540,0xdc32e540,0xd845e542,0xdc33e542,0xd845e544,0xdc34e544,0xd845e546,
	0xdc35e546,0xd845e548,0xdc36e548,0xd845e54a,0xdc37e54a,0xd845e54c,0xdc38e54c,0xd845e54e,
	0xdc39e54e,0xd845e550,0xdc3ae550,0xd845e552,0xdc3be552,0xd845e554,0xdc3ce554,0xd845e556,
	0xdc3de556,0xd845e558,0xdc3ee558,0xd845e55a,0xdc3fe55a,0xd845e55c,0xdc40e55c,0xd845e55e,
	0xdc41e55e,0xd845e560,0xdc42e560,0xd845e562,0xdc43e562,0xd845e564,0xdc44e564,0xd845e566,
	0xdc45e566,0xd845e568,0xdc46e568,0xd845e56a,0xdc47e56a,0xd845e56c,0xdc48e56c,0xd845e56e,
	0xdc49e56e,0xd845e570,0xdc4ae570,0xd845e572,0xdc4be572,0xd845e574,0xdc4ce574,0xd845e576,
	0xdc4de576,0xd845e578,0xdc4ee578,0xd845e57a,0xdc4fe57a,0xd845e57c,0xdc50e57c,0xd845e57e,
	0xdc51e57e,0xd845e580,0xdc52e580,0xd845e582,0xdc53e582,0xd845e584,0xdc54e584,0xd845e586,
	0xdc55e586,0xd845e588,0xdc56e588,0xd845e58a,0xdc57e58a,0xd845e58c,0xdc58e58c,0xd845e58e,
	0xdc59e58e,0xd845e590,0xdc5ae590,0xd845e592,0xdc5be592,0xd845e594,0xdc5ce594,0xd845e596,
	0xdc5de596,0xd845e598,0xdc5ee598,0xd845e59a,0xdc5fe59a,0xd845e59c,0xdc60e59c,0xd845e59e,
	0xdc61e59e,0xd845e5a0,0xdc62e5a0,0xd845e5a2,0xdc63e5a2,0xd845e5a4,0xdc64e5a4,0xd845e5a6,
	0xdc65e5a6,0xd845e5a8,0xdc66e5a8,0xd845e5aa,0xdc67e5aa,0xd845e5ac,0xdc68e5ac,0xd845e5ae,
	0xdc69e5ae,0xd845e5b0,0xdc6ae5b0,0xd845e5b2,0xdc6be5b2,0xd845e5b4,0xdc6ce5b4,0xd845e5b6,
	0xdc6de5b6,0xd845e5b8,0xdc6ee5b8,0xd845e5ba,0xdc6fe5ba,0xd845e5bc,0xdc70e5bc,0xd845e5be,
	0xdc71e5be,0xd845e5c0,0xdc72e5c0,0xd845e5c2,0xdc73e5c2,0xd845e5c4,0xdc74e5c4,0xd845e5c6,
	0xdc75e5c6,0xd845e5c8,0xdc76e5c8,0xd845e5ca,0xdc77e5ca,0xd845e5cc,0xdc78e5cc,0xd845e5ce,
	0xdc79e5ce,0xd845e5d0,0xdc7ae5d0,0xd845e5d2,0xdc7be5d2,0xd845e5d4,0xdc7ce5d4,0xd845e5d6,
	0xdc7de5d6,0xd845e5d8,0xdc7ee5d8,0xd845e5da,0xdc7fe5da,0xd845e5dc,0xdc80e5dc,0xd845e5de,
	0xdc81e5de,0xd845e5e0,0xdc82e5e0,0xd845e5e2,0xdc83e5e2,0xd845e5e4,0xdc84e5e4,0xd845e5e6,
	0xdc85e5e6,0xd845e5e8,0xdc86e5e8,0xd845e5ea,0xdc87e5ea,0xd845e5ec,0xdc88e5ec,0xd845e5ee,
	0xdc89e5ee,0xd845e5f0,0xdc8ae5f0,0xd845e5f2,0xdc8be5f2,0xd845e5f4,0xdc8ce5f4,0xd845e5f6,
	0xdc8de5f6,0xd845e5f8,0xdc8ee5f8,0xd845e5fa,0xdc8fe5fa,0xd845e5fc,0xdc90e5fc,0xd845e5fe,
	0xdc91e5fe,0xd845e600,0xdc92e600,0xd845e602,0xdc93e602,0xd845e604,0xdc94e604,0xd845e606,
	0xdc95e606,0xd845e608,0xdc96e608,0xd845e60a,0xdc97e60a,0xd845e60c,0xdc98e60c,0xd845e60e,
	0xdc99e60e,0xd845e610,0xdc9ae610,0xd845e612,0xdc9be612,0xd845e614,0xdc9ce614,0xd845e616,
	0xdc9de616,0xd845e618,0xdc9ee618,0xd845e61a,0xdc9fe61a,0xd845e61c,0xdca0e61c,0xd845e61e,
	0xdca1e61e,0xd845e620,0xdca2e620,0xd845e622,0xdca3e622,0xd845e624,0xdca4e624,0xd845e626,
	0xdca5e626,0xd845e628,0xdca6e628,0xd845e62a,0xdca7e62a,0xd845e62c,0xdca8e62c,0xd845e62e,
	0xdca9e62e,0xd845e630,0xdcaae630,0xd845e632,0xdcabe632,0xd845e634,0xdcace634,0xd845e636,
	0xdcade636,0xd845e638,0xdcaee638,0xd845e63a,0xdcafe63a,0xd845e63c,0xdcb0e63c,0xd845e63e,
	0xdcb1e63e,0xd845e640,0xdcb2e640,0xd845e642,0xdcb3e642,0xd845e644,0xdcb4e644,0xd845e646,
	0xdcb5e646,0xd845e648,0xdcb6e648,0xd845e64a,0xdcb7e64a,0xd845e64c,0xdcb8e64c,0xd845e64e,
	0xdcb9e64e,0xd845e650,0xdcbae650,0xd845e652,0xdcbbe652,0xd845e654,0xdcbce654,0xd845e656,
	0xdcbde656,0xd845e658,0xdcbee658,0xd845e65a,0xdcbfe65a,0xd845e65c,0xdcc0e65c,0xd845e65e,
	0xdcc1e65e,0xd845e660,0xdcc2e660,0xd845e662,0xdcc3e662,0xd845e664,0xdcc4e664,0xd845e666,
	0xdcc5e666,0xd845e668,0xdcc6e668,0xd845e66a,0xdcc7e66a,0xd845e66c,0xdcc8e66c,0xd845e66e,
	0xdcc9e66e,0xd845e670,0xdccae670,0xd845e672,0xdccbe672,0xd845e674,0xdccce674,0xd845e676,
	0xdccde676,0xd845e678,0xdccee678,0xd845e67a,0xdccfe67a,0xd845e67c,0xdcd0e67c,0xd845e67e,
	0xdcd1e67e,0xd845e680,0xdcd2e680,0xd845e682,0xdcd3e682,0xd845e684,0xdcd4e684,0xd845e686,
	0xdcd5e686,0xd845e688,0xdcd6e688,0xd845e68a,0xdcd7e68a,0xd845e68c,0xdcd8e68c,0xd845e68e,
	0xdcd9e68e,0xd845e690,0xdcdae690,0xd845e692,0xdcdbe692,0xd845e694,0xdcdce694,0xd845e696,
	0xdcdde696,0xd845e698,0xdcdee698,0xd845e69a,0xdcdfe69a,0xd845e69c,0xdce0e69c,0xd845e69e,
	0xdce1e69e,0xd845e6a0,0xdce2e6a0,0xd845e6a2,0xdce3e6a2,0xd845e6a4,0xdce4e6a4,0xd845e6a6,
	0xdce5e6a6,0xd845e6a8,0xdce6e6a8,0xd845e6aa,0xdce7e6aa,0xd845e6ac,0xdce8e6ac,0xd845e6ae,
	0xdce9e6ae,0xd845e6b0,0xdceae6b0,0xd845e6b2,0xdcebe6b2,0xd845e6b4,0xdcece6b4,0xd845e6b6,
	0xdcede6b6,0xd845e6b8,0xdceee6b8,0xd845e6ba,0xdcefe6ba,0xd845e6bc,0xdcf0e6bc,0xd845e6be,
	0xdcf1e6be,0xd845e6c0,0xdcf2e6c0,0xd845e6c2,0xdcf3e6c2,0xd845e6c4,0xdcf4e6c4,0xd845e6c6,
	0xdcf5e6c6,0xd845e6c8,0xdcf6e6c8,0xd845e6ca,0xdcf7e6ca,0xd845e6cc,0xdcf8e6cc,0xd845e6ce,
	0xdcf9e6ce,0xd845e6d0,0xdcfae6d0,0xd845e6d2,0xdcfbe6d2,0xd845e6d4,0xdcfce6d4,0xd845e6d6,
	0xdcfde6d6,0xd845e6d8,0xdcfee6d8,0xd845e6da,0xdcffe6da,0xd845e6dc,0xdd00e6dc,0xd845e6de,
	0xdd01e6de,0xd845e6e0,0xdd02e6e0,0xd845e6e2,0xdd03e6e2,0xd845e6e4,0xdd04e6e4,0xd845e6e6,
	0xdd05e6e6,0xd845e6e8,0xdd06e6e8,0xd845e6ea,0xdd07e6ea,0xd845e6ec,0xdd08e6ec,0xd845e6ee,
	0xdd09e6ee,0xd845e6f0,0xdd0ae6f0,0xd845e6f2,0xdd0be6f2,0xd845e6f4,0xdd0ce6f4,0xd845e6f6,
	0xdd0de6f6,0xd845e6f8,0xdd0ee6f8,0xd845e6fa,0xdd0fe6fa,0xd845e6fc,0xdd10e6fc,0xd845e6fe,
	0xdd11e6fe,0xd845e700,0xdd12e700,0xd845e702,0xdd13e702,0xd845e704,0xdd14e704,0xd845e706,
	0xdd15e706,0xd845e708,0xdd16e708,0xd845e70a,0xdd17e70a,0xd845e70c,0xdd18e70c,0xd845e70e,
	0xdd19e70e,0xd845e710,0xdd1ae710,0xd845e712,0xdd1be712,0xd845e714,0xdd1ce714,0xd845e716,
	0xdd1de716,0xd845e718,0xdd1ee718,0xd845e71a,0xdd1fe71a,0xd845e71c,0xdd20e71c,0xd845e71e,
	0xdd21e71e,0xd845e720,0xdd22e720,0xd845e722,0xdd23e722,0xd845e724,0xdd24e724,0xd845e726,
	0xdd25e726,0xd845e728,0xdd26e728,0xd845e72a,0xdd27e72a,0xd845e72c,0xdd28e72c,0xd845e72e,
	0xdd29e72e,0xd845e730,0xdd2ae730,0xd845e732,0xdd2be732,0xd845e734,0xdd2ce734,0xd845e736,
	0xdd2de736,0xd845e738,0xdd2ee738,0xd845e73a,0xdd2fe73a,0xd845e73c,0xdd30e73c,0xd845e73e,
	0xdd31e73e,0xd845e740,0xdd32e740,0xd845e742,0xdd33e742,0xd845e744,0xdd34e744,0xd845e746,
	0xdd35e746,0xd845e748,0xdd36e748,0xd845e74a,0xdd37e74a,0xd845e74c,0xdd38e74c,0xd845e74e,
	0xdd39e74e,0xd845e750,0xdd3ae750,0xd845e752,0xdd3be752,0xd845e754,0xdd3ce754,0xd845e756,
	0xdd3de756,0xd845e758,0xdd3ee758,0xd845e75a,0xdd3fe75a,0xd845e75c,0xdd40e75c,0xd845e75e,
	0xdd41e75e,0xd845e760,0xdd42e760,0xd845e762,0xdd43e762,0xd845e764,0xdd44e764,0xd845e766,
	0xdd45e766,0xd845e768,0xdd46e768,0xd845e76a,0xdd47e76a,0xd845e76c,0xdd48e76c,0xd845e76e,
	0xdd49e76e,0xd845e770,0xdd4ae770,0xd845e772,0xdd4be772,0xd845e774,0xdd4ce774,0xd845e776,
	0xdd4de776,0xd845e778,0xdd4ee778,0xd845e77a,0xdd4fe77a,0xd845e77c,0xdd50e77c,0xd845e77e,
	0xdd51e77e,0xd845e780,0xdd52e780,0xd845e782,0xdd53e782,0xd845e784,0xdd54e784,0xd845e786,
	0xdd55e786,0xd845e788,0xdd56e788,0xd845e78a,0xdd57e78a,0xd845e78c,0xdd58e78c,0xd845e78e,
	0xdd59e78e,0xd845e790,0xdd5ae790,0xd845e792,0xdd5be792,0xd845e794,0xdd5ce794,0xd845e796,
	0xdd5de796,0xd845e798,0xdd5ee798,0xd845e79a,0xdd5fe79a,0xd845e79c,0xdd60e79c,0xd845e79e,
	0xdd61e79e,0xd845e7a0,0xdd62e7a0,0xd845e7a2,0xdd63e7a2,0xd845e7a4,0xdd64e7a4,0xd845e7a6,
	0xdd65e7a6,0xd845e7a8,0xdd66e7a8,0xd845e7aa,0xdd67e7aa,0xd845e7ac,0xdd68e7ac,0xd845e7ae,
	0xdd69e7ae,0xd845e7b0,0xdd6ae7b0,0xd845e7b2,0xdd6be7b2,0xd845e7b4,0xdd6ce7b4,0xd845e7b6,
	0xdd6de7b6,0xd845e7b8,0xdd6ee7b8,0xd845e7ba,0xdd6fe7ba,0xd845e7bc,0xdd70e7bc,0xd845e7be,
	0xdd71e7be,0xd845e7c0,0xdd72e7c0,0xd845e7c2,0xdd73e7c2,0xd845e7c4,0xdd74e7c4,0xd845e7c6,
	0xdd75e7c6,0xd845e7c8,0xdd76e7c8,0xd845e7ca,0xdd77e7ca,0xd845e7cc,0xdd78e7cc,0xd845e7ce,
	0xdd79e7ce,0xd845e7d0,0xdd7ae7d0,0xd845e7d2,0xdd7be7d2,0xd845e7d4,0xdd7ce7d4,0xd845e7d6,
	0xdd7de7d6,0xd845e7d8,0xdd7ee7d8,0xd845e7da,0xdd7fe7da,0xd845e7dc,0xdd80e7dc,0xd845e7de,
	0xdd81e7de,0xd845e7e0,0xdd82e7e0,0xd845e7e2,0xdd83e7e2,0xd845e7e4,0xdd84e7e4,0xd845e7e6,
	0xdd85e7e6,0xd845e7e8,0xdd86e7e8,0xd845e7ea,0xdd87e7ea,0xd845e7ec,0xdd88e7ec,0xd845e7ee,
	0xdd89e7ee,0xd845e7f0,0xdd8ae7f0,0xd845e7f2,0xdd8be7f2,0xd845e7f4,0xdd8ce7f4,0xd845e7f6,
	0xdd8de7f6,0xd845e7f8,0xdd8ee7f8,0xd845e7fa,0xdd8fe7fa,0xd845e7fc,0xdd90e7fc,0xd845e7fe,
	0xdd91e7fe,0xd845e800,0xdd92e800,0xd845e802,0xdd93e802,0xd845e804,0xdd94e804,0xd845e806,
	0xdd95e806,0xd845e808,0xdd96e808,0xd845e80a,0xdd97e80a,0xd845e80c,0xdd98e80c,0xd845e80e,
	0xdd99e80e,0xd845e810,0xdd9ae810,0xd845e812,0xdd9be812,0xd845e814,0xdd9ce814,0xd845e816,
	0xdd9de816,0xd845e818,0xdd9ee818,0xd845e81a,0xdd9fe81a,0xd845e81c,0xdda0e81c,0xd845e81e,
	0xdda1e81e,0xd845e820,0xdda2e820,0xd845e822,0xdda3e822,0xd845e824,0xdda4e824,0xd845e826,
	0xdda5e826,0xd845e828,0xdda6e828,0xd845e82a,0xdda7e82a,0xd845e82c,0xdda8e82c,0xd845e82e,
	0xdda9e82e,0xd845e830,0xddaae830,0xd845e832,0xddabe832,0xd845e834,0xddace834,0xd845e836,
	0xddade836,0xd845e838,0xddaee838,0xd845e83a,0xddafe83a,0xd845e83c,0xddb0e83c,0xd845e83e,
	0xddb1e83e,0xd845e840,0xddb2e840,0xd845e842,0xddb3e842,0xd845e844,0xddb4e844,0xd845e846,
	0xddb5e846,0xd845e848,0xddb6e848,0xd845e84a,0xddb7e84a,0xd845e84c,0xddb8e84c,0xd845e84e,
	0xddb9e84e,0xd845e850,0xddbae850,0xd845e852,0xddbbe852,0xd845e854,0xddbce854,0xd845e856,
	0xddbde856,0xd845e858,0xddbee858,0xd845e85a,0xddbfe85a,0xd845e85c,0xddc0e85c,0xd845e85e,
	0xddc1e85e,0xd845e860,0xddc2e860,0xd845e862,0xddc3e862,0xd845e864,0xddc4e864,0xd845e866,
	0xddc5e866,0xd845e868,0xddc6e868,0xd845e86a,0xddc7e86a,0xd845e86c,0xddc8e86c,0xd845e86e,
	0xddc9e86e,0xd845e870,0xddcae870,0xd845e872,0xddcbe872,0xd845e874,0xddcce874,0xd845e876,
	0xddcde876,0xd845e878,0xddcee878,0xd845e87a,0xddcfe87a,0xd845e87c,0xddd0e87c,0xd845e87e,
	0xddd1e87e,0xd845e880,0xddd2e880,0xd845e882,0xddd3e882,0xd845e884,0xddd4e884,0xd845e886,
	0xddd5e886,0xd845e888,0xddd6e888,0xd845e88a,0xddd7e88a,0xd845e88c,0xddd8e88c,0xd845e88e,
	0xddd9e88e,0xd845e890,0xdddae890,0xd845e892,0xdddbe892,0xd845e894,0xdddce894,0xd845e896,
	0xdddde896,0xd845e898,0xdddee898,0xd845e89a,0xdddfe89a,0xd845e89c,0xdde0e89c,0xd845e89e,
	0xdde1e89e,0xd845e8a0,0xdde2e8a0,0xd845e8a2,0xdde3e8a2,0xd845e8a4,0xdde4e8a4,0xd845e8a6,
	0xdde5e8a6,0xd845e8a8,0xdde6e8a8,0xd845e8aa,0xdde7e8aa,0xd845e8ac,0xdde8e8ac,0xd845e8ae,
	0xdde9e8ae,0xd845e8b0,0xddeae8b0,0xd845e8b2,0xddebe8b2,0xd845e8b4,0xddece8b4,0xd845e8b6,
	0xddede8b6,0xd845e8b8,0xddeee8b8,0xd845e8ba,0xddefe8ba,0xd845e8bc,0xddf0e8bc,0xd845e8be,
	0xddf1e8be,0xd845e8c0,0xddf2e8c0,0xd845e8c2,0xddf3e8c2,0xd845e8c4,0xddf4e8c4,0xd845e8c6,
	0xddf5e8c6,0xd845e8c8,0xddf6e8c8,0xd845e8ca,0xddf7e8ca,0xd845e8cc,0xddf8e8cc,0xd845e8ce,
	0xddf9e8ce,0xd845e8d0,0xddfae8d0,0xd845e8d2,0xddfbe8d2,0xd845e8d4,0xddfce8d4,0xd845e8d6,
	0xddfde8d6,0xd845e8d8,0xddfee8d8,0xd845e8da,0xddffe8da,0xd845e8dc,0xde00e8dc,0xd845e8de,
	0xde01e8de,0xd845e8e0,0xde02e8e0,0xd845e8e2,0xde03e8e2,0xd845e8e4,0xde04e8e4,0xd845e8e6,
	0xde05e8e6,0xd845e8e8,0xde06e8e8,0xd845e8ea,0xde07e8ea,0xd845e8ec,0xde08e8ec,0xd845e8ee,
	0xde09e8ee,0xd845e8f0,0xde0ae8f0,0xd845e8f2,0xde0be8f2,0xd845e8f4,0xde0ce8f4,0xd845e8f6,
	0xde0de8f6,0xd845e8f8,0xde0ee8f8,0xd845e8fa,0xde0fe8fa,0xd845e8fc,0xde10e8fc,0xd845e8fe,
	0xde11e8fe,0xd845e900,0xde12e900,0xd845e902,0xde13e902,0xd845e904,0xde14e904,0xd845e906,
	0xde15e906,0xd845e908,0xde16e908,0xd845e90a,0xde17e90a,0xd845e90c,0xde18e90c,0xd845e90e,
	0xde19e90e,0xd845e910,0xde1ae910,0xd845e912,0xde1be912,0xd845e914,0xde1ce914,0xd845e916,
	0xde1de916,0xd845e918,0xde1ee918,0xd845e91a,0xde1fe91a,0xd845e91c,0xde20e91c,0xd845e91e,
	0xde21e91e,0xd845e920,0xde22e920,0xd845e922,0xde23e922,0xd845e924,0xde24e924,0xd845e926,
	0xde25e926,0xd845e928,0xde26e928,0xd845e92a,0xde27e92a,0xd845e92c,0xde28e92c,0xd845e92e,
	0xde29e92e,0xd845e930,0xde2ae930,0xd845e932,0xde2be932,0xd845e934,0xde2ce934,0xd845e936,
	0xde2de936,0xd845e938,0xde2ee938,0xd845e93a,0xde2fe93a,0xd845e93c,0xde30e93c,0xd845e93e,
	0xde31e93e,0xd845e940,0xde32e940,0xd845e942,0xde33e942,0xd845e944,0xde34e944,0xd845e946,
	0xde35e946,0xd845e948,0xde36e948,0xd845e94a,0xde37e94a,0xd845e94c,0xde38e94c,0xd845e94e,
	0xde39e94e,0xd845e950,0xde3ae950,0xd845e952,0xde3be952,0xd845e954,0xde3ce954,0xd845e956,
	0xde3de956,0xd845e958,0xde3ee958,0xd845e95a,0xde3fe95a,0xd845e95c,0xde40e95c,0xd845e95e,
	0xde41e95e,0xd845e960,0xde42e960,0xd845e962,0xde43e962,0xd845e964,0xde44e964,0xd845e966,
	0xde45e966,0xd845e968,0xde46e968,0xd845e96a,0xde47e96a,0xd845e96c,0xde48e96c,0xd845e96e,
	0xde49e96e,0xd845e970,0xde4ae970,0xd845e972,0xde4be972,0xd845e974,0xde4ce974,0xd845e976,
	0xde4de976,0xd845e978,0xde4ee978,0xd845e97a,0xde4fe97a,0xd845e97c,0xde50e97c,0xd845e97e,
	0xde51e97e,0xd845e980,0xde52e980,0xd845e982,0xde53e982,0xd845e984,0xde54e984,0xd845e986,
	0xde55e986,0xd845e988,0xde56e988,0xd845e98a,0xde57e98a,0xd845e98c,0xde58e98c,0xd845e98e,
	0xde59e98e,0xd845e990,0xde5ae990,0xd845e992,0xde5be992,0xd845e994,0xde5ce994,0xd845e996,
	0xde5de996,0xd845e998,0xde5ee998,0xd845e99a,0xde5fe99a,0xd845e99c,0xde60e99c,0xd845e99e,
	0xde61e99e,0xd845e9a0,0xde62e9a0,0xd845e9a2,0xde63e9a2,0xd845e9a4,0xde64e9a4,0xd845e9a6,
	0xde65e9a6,0xd845e9a8,0xde66e9a8,0xd845e9aa,0xde67e9aa,0xd845e9ac,0xde68e9ac,0xd845e9ae,
	0xde69e9ae,0xd845e9b0,0xde6ae9b0,0xd845e9b2,0xde6be9b2,0xd845e9b4,0xde6ce9b4,0xd845e9b6,
	0xde6de9b6,0xd845e9b8,0xde6ee9b8,0xd845e9ba,0xde6fe9ba,0xd845e9bc,0xde70e9bc,0xd845e9be,
	0xde71e9be,0xd845e9c0,0xde72e9c0,0xd845e9c2,0xde73e9c2,0xd845e9c4,0xde74e9c4,0xd845e9c6,
	0xde75e9c6,0xd845e9c8,0xde76e9c8,0xd845e9ca,0xde77e9ca,0xd845e9cc,0xde78e9cc,0xd845e9ce,
	0xde79e9ce,0xd845e9d0,0xde7ae9d0,0xd845e9d2,0xde7be9d2,0xd845e9d4,0xde7ce9d4,0xd845e9d6,
	0xde7de9d6,0xd845e9d8,0xde7ee9d8,0xd845e9da,0xde7fe9da,0xd845e9dc,0xde80e9dc,0xd845e9de,
	0xde81e9de,0xd845e9e0,0xde82e9e0,0xd845e9e2,0xde83e9e2,0xd845e9e4,0xde84e9e4,0xd845e9e6,
	0xde85e9e6,0xd845e9e8,0xde86e9e8,0xd845e9ea,0xde87e9ea,0xd845e9ec,0xde88e9ec,0xd845e9ee,
	0xde89e9ee,0xd845e9f0,0xde8ae9f0,0xd845e9f2,0xde8be9f2,0xd845e9f4,0xde8ce9f4,0xd845e9f6,
	0xde8de9f6,0xd845e9f8,0xde8ee9f8,0xd845e9fa,0xde8fe9fa,0xd845e9fc,0xde90e9fc,0xd845e9fe,
	0xde91e9fe,0xd845ea00,0xde92ea00,0xd845ea02,0xde93ea02,0xd845ea04,0xde94ea04,0xd845ea06,
	0xde95ea06,0xd845ea08,0xde96ea08,0xd845ea0a,0xde97ea0a,0xd845ea0c,0xde98ea0c,0xd845ea0e,
	0xde99ea0e,0xd845ea10,0xde9aea10,0xd845ea12,0xde9bea12,0xd845ea14,0xde9cea14,0xd845ea16,
	0xde9dea16,0xd845ea18,0xde9eea18,0xd845ea1a,0xde9fea1a,0xd845ea1c,0xdea0ea1c,0xd845ea1e,
	0xdea1ea1e,0xd845ea20,0xdea2ea20,0xd845ea22,0xdea3ea22,0xd845ea24,0xdea4ea24,0xd845ea26,
	0xdea5ea26,0xd845ea28,0xdea6ea28,0xd845ea2a,0xdea7ea2a,0xd845ea2c,0xdea8ea2c,0xd845ea2e,
	0xdea9ea2e,0xd845ea30,0xdeaaea30,0xd845ea32,0xdeabea32,0xd845ea34,0xdeacea34,0xd845ea36,
	0xdeadea36,0xd845ea38,0xdeaeea38,0xd845ea3a,0xdeafea3a,0xd845ea3c,0xdeb0ea3c,0xd845ea3e,
	0xdeb1ea3e,0xd845ea40,0xdeb2ea40,0xd845ea42,0xdeb3ea42,0xd845ea44,0xdeb4ea44,0xd845ea46,
	0xdeb5ea46,0xd845ea48,0xdeb6ea48,0xd845ea4a,0xdeb7ea4a,0xd845ea4c,0xdeb8ea4c,0xd845ea4e,
	0xdeb9ea4e,0xd845ea50,0xdebaea50,0xd845ea52,0xdebbea52,0xd845ea54,0xdebcea54,0xd845ea56,
	0xdebdea56,0xd845ea58,0xdebeea58,0xd845ea5a,0xdebfea5a,0xd845ea5c,0xdec0ea5c,0xd845ea5e,
	0xdec1ea5e,0xd845ea60,0xdec2ea60,0xd845ea62,0xdec3ea62,0xd845ea64,0xdec4ea64,0xd845ea66,
	0xdec5ea66,0xd845ea68,0xdec6ea68,0xd845ea6a,0xdec7ea6a,0xd845ea6c,0xdec8ea6c,0xd845ea6e,
	0xdec9ea6e,0xd845ea70,0xdecaea70,0xd845ea72,0xdecbea72,0xd845ea74,0xdeccea74,0xd845ea76,
	0xdecdea76,0xd845ea78,0xdeceea78,0xd845ea7a,0xdecfea7a,0xd845ea7c,0xded0ea7c,0xd845ea7e,
	0xded1ea7e,0xd845ea80,0xded2ea80,0xd845ea82,0xded3ea82,0xd845ea84,0xded4ea84,0xd845ea86,
	0xded5ea86,0xd845ea88,0xded6ea88,0xd845ea8a,0xded7ea8a,0xd845ea8c,0xded8ea8c,0xd845ea8e,
	0xded9ea8e,0xd845ea90,0xdedaea90,0xd845ea92,0xdedbea92,0xd845ea94,0xdedcea94,0xd845ea96,
	0xdeddea96,0xd845ea98,0xdedeea98,0xd845ea9a,0xdedfea9a,0xd845ea9c,0xdee0ea9c,0xd845ea9e,
	0xdee1ea9e,0xd845eaa0,0xdee2eaa0,0xd845eaa2,0xdee3eaa2,0xd845eaa4,0xdee4eaa4,0xd845eaa6,
	0xdee5eaa6,0xd845eaa8,0xdee6eaa8,0xd845eaaa,0xdee7eaaa,0xd845eaac,0xdee8eaac,0xd845eaae,
	0xdee9eaae,0xd845eab0,0xdeeaeab0,0xd845eab2,0xdeebeab2,0xd845eab4,0xdeeceab4,0xd845eab6,
	0xdeedeab6,0xd845eab8,0xdeeeeab8,0xd845eaba,0xdeefeaba,0xd845eabc,0xdef0eabc,0xd845eabe,
	0xdef1eabe,0xd845eac0,0xdef2eac0,0xd845eac2,0xdef3eac2,0xd845eac4,0xdef4eac4,0xd845eac6,
	0xdef5eac6,0xd845eac8,0xdef6eac8,0xd845eaca,0xdef7eaca,0xd845eacc,0xdef8eacc,0xd845eace,
	0xdef9eace,0xd845ead0,0xdefaead0,0xd845ead2,0xdefbead2,0xd845ead4,0xdefcead4,0xd845ead6,
	0xdefdead6,0xd845ead8,0xdefeead8,0xd845eada,0xdeffeada,0xd845eadc,0xdf00eadc,0xd845eade,
	0xdf01eade,0xd845eae0,0xdf02eae0,0xd845eae2,0xdf03eae2,0xd845eae4,0xdf04eae4,0xd845eae6,
	0xdf05eae6,0xd845eae8,0xdf06eae8,0xd845eaea,0xdf07eaea,0xd845eaec,0xdf08eaec,0xd845eaee,
	0xdf09eaee,0xd845eaf0,0xdf0aeaf0,0xd845eaf2,0xdf0beaf2,0xd845eaf4,0xdf0ceaf4,0xd845eaf6,
	0xdf0deaf6,0xd845eaf8,0xdf0eeaf8,0xd845eafa,0xdf0feafa,0xd845eafc,0xdf10eafc,0xd845eafe,
	0xdf11eafe,0xd845eb00,0xdf12eb00,0xd845eb02,0xdf13eb02,0xd845eb04,0xdf14eb04,0xd845eb06,
	0xdf15eb06,0xd845eb08,0xdf16eb08,0xd845eb0a,0xdf17eb0a,0xd845eb0c,0xdf18eb0c,0xd845eb0e,
	0xdf19eb0e,0xd845eb10,0xdf1aeb10,0xd845eb12,0xdf1beb12,0xd845eb14,0xdf1ceb14,0xd845eb16,
	0xdf1deb16,0xd845eb18,0xdf1eeb18,0xd845eb1a,0xdf1feb1a,0xd845eb1c,0xdf20eb1c,0xd845eb1e,
	0xdf21eb1e,0xd845eb20,0xdf22eb20,0xd845eb22,0xdf23eb22,0xd845eb24,0xdf24eb24,0xd845eb26,
	0xdf25eb26,0xd845eb28,0xdf26eb28,0xd845eb2a,0xdf27eb2a,0xd845eb2c,0xdf28eb2c,0xd845eb2e,
	0xdf29eb2e,0xd845eb30,0xdf2aeb30,0xd845eb32,0xdf2beb32,0xd845eb34,0xdf2ceb34,0xd845eb36,
	0xdf2deb36,0xd845eb38,0xdf2eeb38,0xd845eb3a,0xdf2feb3a,0xd845eb3c,0xdf30eb3c,0xd845eb3e,
	0xdf31eb3e,0xd845eb40,0xdf32eb40,0xd845eb42,0xdf33eb42,0xd845eb44,0xdf34eb44,0xd845eb46,
	0xdf35eb46,0xd845eb48,0xdf36eb48,0xd845eb4a,0xdf37eb4a,0xd845eb4c,0xdf38eb4c,0xd845eb4e,
	0xdf39eb4e,0xd845eb50,0xdf3aeb50,0xd845eb52,0xdf3beb52,0xd845eb54,0xdf3ceb54,0xd845eb56,
	0xdf3deb56,0xd845eb58,0xdf3eeb58,0xd845eb5a,0xdf3feb5a,0xd845eb5c,0xdf40eb5c,0xd845eb5e,
	0xdf41eb5e,0xd845eb60,0xdf42eb60,0xd845eb62,0xdf43eb62,0xd845eb64,0xdf44eb64,0xd845eb66,
	0xdf45eb66,0xd845eb68,0xdf46eb68,0xd845eb6a,0xdf47eb6a,0xd845eb6c,0xdf48eb6c,0xd845eb6e,
	0xdf49eb6e,0xd845eb70,0xdf4aeb70,0xd845eb72,0xdf4beb72,0xd845eb74,0xdf4ceb74,0xd845eb76,
	0xdf4deb76,0xd845eb78,0xdf4eeb78,0xd845eb7a,0xdf4feb7a,0xd845eb7c,0xdf50eb7c,0xd845eb7e,
	0xdf51eb7e,0xd845eb80,0xdf52eb80,0xd845eb82,0xdf53eb82,0xd845eb84,0xdf54eb84,0xd845eb86,
	0xdf55eb86,0xd845eb88,0xdf56eb88,0xd845eb8a,0xdf57eb8a,0xd845eb8c,0xdf58eb8c,0xd845eb8e,
	0xdf59eb8e,0xd845eb90,0xdf5aeb90,0xd845eb92,0xdf5beb92,0xd845eb94,0xdf5ceb94,0xd845eb96,
	0xdf5deb96,0xd845eb98,0xdf5eeb98,0xd845eb9a,0xdf5feb9a,0xd845eb9c,0xdf60eb9c,0xd845eb9e,
	0xdf61eb9e,0xd845eba0,0xdf62eba0,0xd845eba2,0xdf63eba2,0xd845eba4,0xdf64eba4,0xd845eba6,
	0xdf65eba6,0xd845eba8,0xdf66eba8,0xd845ebaa,0xdf67ebaa,0xd845ebac,0xdf68ebac,0xd845ebae,
	0xdf69ebae,0xd845ebb0,0xdf6aebb0,0xd845ebb2,0xdf6bebb2,0xd845ebb4,0xdf6cebb4,0xd845ebb6,
	0xdf6debb6,0xd845ebb8,0xdf6eebb8,0xd845ebba,0xdf6febba,0xd845ebbc,0xdf70ebbc,0xd845ebbe,
	0xdf71ebbe,0xd845ebc0,0xdf72ebc0,0xd845ebc2,0xdf73ebc2,0xd845ebc4,0xdf74ebc4,0xd845ebc6,
	0xdf75ebc6,0xd845ebc8,0xdf76ebc8,0xd845ebca,0xdf77ebca,0xd845ebcc,0xdf78ebcc,0xd845ebce,
	0xdf79ebce,0xd845ebd0,0xdf7aebd0,0xd845ebd2,0xdf7bebd2,0xd845ebd4,0xdf7cebd4,0xd845ebd6,
	0xdf7debd6,0xd845ebd8,0xdf7eebd8,0xd845ebda,0xdf7febda,0xd845ebdc,0xdf80ebdc,0xd845ebde,
	0xdf81ebde,0xd845ebe0,0xdf82ebe0,0xd845ebe2,0xdf83ebe2,0xd845ebe4,0xdf84ebe4,0xd845ebe6,
	0xdf85ebe6,0xd845ebe8,0xdf86ebe8,0xd845ebea,0xdf87ebea,0xd845ebec,0xdf88ebec,0xd845ebee,
	0xdf89ebee,0xd845ebf0,0xdf8aebf0,0xd845ebf2,0xdf8bebf2,0xd845ebf4,0xdf8cebf4,0xd845ebf6,
	0xdf8debf6,0xd845ebf8,0xdf8eebf8,0xd845ebfa,0xdf8febfa,0xd845ebfc,0xdf90ebfc,0xd845ebfe,
	0xdf91ebfe,0xd845ec00,0xdf92ec00,0xd845ec02,0xdf93ec02,0xd845ec04,0xdf94ec04,0xd845ec06,
	0xdf95ec06,0xd845ec08,0xdf96ec08,0xd845ec0a,0xdf97ec0a,0xd845ec0c,0xdf98ec0c,0xd845ec0e,
	0xdf99ec0e,0xd845ec10,0xdf9aec10,0xd845ec12,0xdf9bec12,0xd845ec14,0xdf9cec14,0xd845ec16,
	0xdf9dec16,0xd845ec18,0xdf9eec18,0xd845ec1a,0xdf9fec1a,0xd845ec1c,0xdfa0ec1c,0xd845ec1e,
	0xdfa1ec1e,0xd845ec20,0xdfa2ec20,0xd845ec22,0xdfa3ec22,0xd845ec24,0xdfa4ec24,0xd845ec26,
	0xdfa5ec26,0xd845ec28,0xdfa6ec28,0xd845ec2a,0xdfa7ec2a,0xd845ec2c,0xdfa8ec2c,0xd845ec2e,
	0xdfa9ec2e,0xd845ec30,0xdfaaec30,0xd845ec32,0xdfabec32,0xd845ec34,0xdfacec34,0xd845ec36,
	0xdfadec36,0xd845ec38,0xdfaeec38,0xd845ec3a,0xdfafec3a,0xd845ec3c,0xdfb0ec3c,0xd845ec3e,
	0xdfb1ec3e,0xd845ec40,0xdfb2ec40,0xd845ec42,0xdfb3ec42,0xd845ec44,0xdfb4ec44,0xd845ec46,
	0xdfb5ec46,0xd845ec48,0xdfb6ec48,0xd845ec4a,0xdfb7ec4a,0xd845ec4c,0xdfb8ec4c,0xd845ec4e,
	0xdfb9ec4e,0xd845ec50,0xdfbaec50,0xd845ec52,0xdfbbec52,0xd845ec54,0xdfbcec54,0xd845ec56,
	0xdfbdec56,0xd845ec58,0xdfbeec58,0xd845ec5a,0xdfbfec5a,0xd845ec5c,0xdfc0ec5c,0xd845ec5e,
	0xdfc1ec5e,0xd845ec60,0xdfc2ec60,0xd845ec62,0xdfc3ec62,0xd845ec64,0xdfc4ec64,0xd845ec66,
	0xdfc5ec66,0xd845ec68,0xdfc6ec68,0xd845ec6a,0xdfc7ec6a,0xd845ec6c,0xdfc8ec6c,0xd845ec6e,
	0xdfc9ec6e,0xd845ec70,0xdfcaec70,0xd845ec72,0xdfcbec72,0xd845ec74,0xdfccec74,0xd845ec76,
	0xdfcdec76,0xd845ec78,0xdfceec78,0xd845ec7a,0xdfcfec7a,0xd845ec7c,0xdfd0ec7c,0xd845ec7e,
	0xdfd1ec7e,0xd845ec80,0xdfd2ec80,0xd845ec82,0xdfd3ec82,0xd845ec84,0xdfd4ec84,0xd845ec86,
	0xdfd5ec86,0xd845ec88,0xdfd6ec88,0xd845ec8a,0xdfd7ec8a,0xd845ec8c,0xdfd8ec8c,0xd845ec8e,
	0xdfd9ec8e,0xd845ec90,0xdfdaec90,0xd845ec92,0xdfdbec92,0xd845ec94,0xdfdcec94,0xd845ec96,
	0xdfddec96,0xd845ec98,0xdfdeec98,0xd845ec9a,0xdfdfec9a,0xd845ec9c,0xdfe0ec9c,0xd845ec9e,
	0xdfe1ec9e,0xd845eca0,0xdfe2eca0,0xd845eca2,0xdfe3eca2,0xd845eca4,0xdfe4eca4,0xd845eca6,
	0xdfe5eca6,0xd845eca8,0xdfe6eca8,0xd845ecaa,0xdfe7ecaa,0xd845ecac,0xdfe8ecac,0xd845ecae,
	0xdfe9ecae,0xd845ecb0,0xdfeaecb0,0xd845ecb2,0xdfebecb2,0xd845ecb4,0xdfececb4,0xd845ecb6,
	0xdfedecb6,0xd845ecb8,0xdfeeecb8,0xd845ecba,0xdfefecba,0xd845ecbc,0xdff0ecbc,0xd845ecbe,
	0xdff1ecbe,0xd845ecc0,0xdff2ecc0,0xd845ecc2,0xdff3ecc2,0xd845ecc4,0xdff4ecc4,0xd845ecc6,
	0xdff5ecc6,0xd845ecc8,0xdff6ecc8,0xd845ecca,0xdff7ecca,0xd845eccc,0xdff8eccc,0xd845ecce,
	0xdff9ecce,0xd845ecd0,0xdffaecd0,0xd845ecd2,0xdffbecd2,0xd845ecd4,0xdffcecd4,0xd845ecd6,
	0xdffdecd6,0xd845ecd8,0xdffeecd8,0xd845ecda,0xdfffecda,0xd846ecdc,0xdc00ecdc,0xd846ecde,
	0xdc01ecde,0xd846ece0,0xdc02ece0,0xd846ece2,0xdc03ece2,0xd846ece4,0xdc04ece4,0xd846ece6,
	0xdc05ece6,0xd846ece8,0xdc06ece8,0xd846ecea,0xdc07ecea,0xd846ecec,0xdc08ecec,0xd846ecee,
	0xdc09ecee,0xd846ecf0,0xdc0aecf0,0xd846ecf2,0xdc0becf2,0xd846ecf4,0xdc0cecf4,0xd846ecf6,
	0xdc0decf6,0xd846ecf8,0xdc0eecf8,0xd846ecfa,0xdc0fecfa,0xd846ecfc,0xdc10ecfc,0xd846ecfe,
	0xdc11ecfe,0xd846ed00,0xdc12ed00,0xd846ed02,0xdc13ed02,0xd846ed04,0xdc14ed04,0xd846ed06,
	0xdc15ed06,0xd846ed08,0xdc16ed08,0xd846ed0a,0xdc17ed0a,0xd846ed0c,0xdc18ed0c,0xd846ed0e,
	0xdc19ed0e,0xd846ed10,0xdc1aed10,0xd846ed12,0xdc1bed12,0xd846ed14,0xdc1ced14,0xd846ed16,
	0xdc1ded16,0xd846ed18,0xdc1eed18,0xd846ed1a,0xdc1fed1a,0xd846ed1c,0xdc20ed1c,0xd846ed1e,
	0xdc21ed1e,0xd846ed20,0xdc22ed20,0xd846ed22,0xdc23ed22,0xd846ed24,0xdc24ed24,0xd846ed26,
	0xdc25ed26,0xd846ed28,0xdc26ed28,0xd846ed2a,0xdc27ed2a,0xd846ed2c,0xdc28ed2c,0xd846ed2e,
	0xdc29ed2e,0xd846ed30,0xdc2aed30,0xd846ed32,0xdc2bed32,0xd846ed34,0xdc2ced34,0xd846ed36,
	0xdc2ded36,0xd846ed38,0xdc2eed38,0xd846ed3a,0xdc2fed3a,0xd846ed3c,0xdc30ed3c,0xd846ed3e,
	0xdc31ed3e,0xd846ed40,0xdc32ed40,0xd846ed42,0xdc33ed42,0xd846ed44,0xdc34ed44,0xd846ed46,
	0xdc35ed46,0xd846ed48,0xdc36ed48,0xd846ed4a,0xdc37ed4a,0xd846ed4c,0xdc38ed4c,0xd846ed4e,
	0xdc39ed4e,0xd846ed50,0xdc3aed50,0xd846ed52,0xdc3bed52,0xd846ed54,0xdc3ced54,0xd846ed56,
	0xdc3ded56,0xd846ed58,0xdc3eed58,0xd846ed5a,0xdc3fed5a,0xd846ed5c,0xdc40ed5c,0xd846ed5e,
	0xdc41ed5e,0xd846ed60,0xdc42ed60,0xd846ed62,0xdc43ed62,0xd846ed64,0xdc44ed64,0xd846ed66,
	0xdc45ed66,0xd846ed68,0xdc46ed68,0xd846ed6a,0xdc47ed6a,0xd846ed6c,0xdc48ed6c,0xd846ed6e,
	0xdc49ed6e,0xd846ed70,0xdc4aed70,0xd846ed72,0xdc4bed72,0xd846ed74,0xdc4ced74,0xd846ed76,
	0xdc4ded76,0xd846ed78,0xdc4eed78,0xd846ed7a,0xdc4fed7a,0xd846ed7c,0xdc50ed7c,0xd846ed7e,
	0xdc51ed7e,0xd846ed80,0xdc52ed80,0xd846ed82,0xdc53ed82,0xd846ed84,0xdc54ed84,0xd846ed86,
	0xdc55ed86,0xd846ed88,0xdc56ed88,0xd846ed8a,0xdc57ed8a,0xd846ed8c,0xdc58ed8c,0xd846ed8e,
	0xdc59ed8e,0xd846ed90,0xdc5aed90,0xd846ed92,0xdc5bed92,0xd846ed94,0xdc5ced94,0xd846ed96,
	0xdc5ded96,0xd846ed98,0xdc5eed98,0xd846ed9a,0xdc5fed9a,0xd846ed9c,0xdc60ed9c,0xd846ed9e,
	0xdc61ed9e,0xd846eda0,0xdc62eda0,0xd846eda2,0xdc63eda2,0xd846eda4,0xdc64eda4,0xd846eda6,
	0xdc65eda6,0xd846eda8,0xdc66eda8,0xd846edaa,0xdc67edaa,0xd846edac,0xdc68edac,0xd846edae,
	0xdc69edae,0xd846edb0,0xdc6aedb0,0xd846edb2,0xdc6bedb2,0xd846edb4,0xdc6cedb4,0xd846edb6,
	0xdc6dedb6,0xd846edb8,0xdc6eedb8,0xd846edba,0xdc6fedba,0xd846edbc,0xdc70edbc,0xd846edbe,
	0xdc71edbe,0xd846edc0,0xdc72edc0,0xd846edc2,0xdc73edc2,0xd846edc4,0xdc74edc4,0xd846edc6,
	0xdc75edc6,0xd846edc8,0xdc76edc8,0xd846edca,0xdc77edca,0xd846edcc,0xdc78edcc,0xd846edce,
	0xdc79edce,0xd846edd0,0xdc7aedd0,0xd846edd2,0xdc7bedd2,0xd846edd4,0xdc7cedd4,0xd846edd6,
	0xdc7dedd6,0xd846edd8,0xdc7eedd8,0xd846edda,0xdc7fedda,0xd846eddc,0xdc80eddc,0xd846edde,
	0xdc81edde,0xd846ede0,0xdc82ede0,0xd846ede2,0xdc83ede2,0xd846ede4,0xdc84ede4,0xd846ede6,
	0xdc85ede6,0xd846ede8,0xdc86ede8,0xd846edea,0xdc87edea,0xd846edec,0xdc88edec,0xd846edee,
	0xdc89edee,0xd846edf0,0xdc8aedf0,0xd846edf2,0xdc8bedf2,0xd846edf4,0xdc8cedf4,0xd846edf6,
	0xdc8dedf6,0xd846edf8,0xdc8eedf8,0xd846edfa,0xdc8fedfa,0xd846edfc,0xdc90edfc,0xd846edfe,
	0xdc91edfe,0xd846ee00,0xdc92ee00,0xd846ee02,0xdc93ee02,0xd846ee04,0xdc94ee04,0xd846ee06,
	0xdc95ee06,0xd846ee08,0xdc96ee08,0xd846ee0a,0xdc97ee0a,0xd846ee0c,0xdc98ee0c,0xd846ee0e,
	0xdc99ee0e,0xd846ee10,0xdc9aee10,0xd846ee12,0xdc9bee12,0xd846ee14,0xdc9cee14,0xd846ee16,
	0xdc9dee16,0xd846ee18,0xdc9eee18,0xd846ee1a,0xdc9fee1a,0xd846ee1c,0xdca0ee1c,0xd846ee1e,
	0xdca1ee1e,0xd846ee20,0xdca2ee20,0xd846ee22,0xdca3ee22,0xd846ee24,0xdca4ee24,0xd846ee26,
	0xdca5ee26,0xd846ee28,0xdca6ee28,0xd846ee2a,0xdca7ee2a,0xd846ee2c,0xdca8ee2c,0xd846ee2e,
	0xdca9ee2e,0xd846ee30,0xdcaaee30,0xd846ee32,0xdcabee32,0xd846ee34,0xdcacee34,0xd846ee36,
	0xdcadee36,0xd846ee38,0xdcaeee38,0xd846ee3a,0xdcafee3a,0xd846ee3c,0xdcb0ee3c,0xd846ee3e,
	0xdcb1ee3e,0xd846ee40,0xdcb2ee40,0xd846ee42,0xdcb3ee42,0xd846ee44,0xdcb4ee44,0xd846ee46,
	0xdcb5ee46,0xd846ee48,0xdcb6ee48,0xd846ee4a,0xdcb7ee4a,0xd846ee4c,0xdcb8ee4c,0xd846ee4e,
	0xdcb9ee4e,0xd846ee50,0xdcbaee50,0xd846ee52,0xdcbbee52,0xd846ee54,0xdcbcee54,0xd846ee56,
	0xdcbdee56,0xd846ee58,0xdcbeee58,0xd846ee5a,0xdcbfee5a,0xd846ee5c,0xdcc0ee5c,0xd846ee5e,
	0xdcc1ee5e,0xd846ee60,0xdcc2ee60,0xd846ee62,0xdcc3ee62,0xd846ee64,0xdcc4ee64,0xd846ee66,
	0xdcc5ee66,0xd846ee68,0xdcc6ee68,0xd846ee6a,0xdcc7ee6a,0xd846ee6c,0xdcc8ee6c,0xd846ee6e,
	0xdcc9ee6e,0xd846ee70,0xdccaee70,0xd846ee72,0xdccbee72,0xd846ee74,0xdcccee74,0xd846ee76,
	0xdccdee76,0xd846ee78,0xdcceee78,0xd846ee7a,0xdccfee7a,0xd846ee7c,0xdcd0ee7c,0xd846ee7e,
	0xdcd1ee7e,0xd846ee80,0xdcd2ee80,0xd846ee82,0xdcd3ee82,0xd846ee84,0xdcd4ee84,0xd846ee86,
	0xdcd5ee86,0xd846ee88,0xdcd6ee88,0xd846ee8a,0xdcd7ee8a,0xd846ee8c,0xdcd8ee8c,0xd846ee8e,
	0xdcd9ee8e,0xd846ee90,0xdcdaee90,0xd846ee92,0xdcdbee92,0xd846ee94,0xdcdcee94,0xd846ee96,
	0xdcddee96,0xd846ee98,0xdcdeee98,0xd846ee9a,0xdcdfee9a,0xd846ee9c,0xdce0ee9c,0xd846ee9e,
	0xdce1ee9e,0xd846eea0,0xdce2eea0,0xd846eea2,0xdce3eea2,0xd846eea4,0xdce4eea4,0xd846eea6,
	0xdce5eea6,0xd846eea8,0xdce6eea8,0xd846eeaa,0xdce7eeaa,0xd846eeac,0xdce8eeac,0xd846eeae,
	0xdce9eeae,0xd846eeb0,0xdceaeeb0,0xd846eeb2,0xdcebeeb2,0xd846eeb4,0xdceceeb4,0xd846eeb6,
	0xdcedeeb6,0xd846eeb8,0xdceeeeb8,0xd846eeba,0xdcefeeba,0xd846eebc,0xdcf0eebc,0xd846eebe,
	0xdcf1eebe,0xd846eec0,0xdcf2eec0,0xd846eec2,0xdcf3eec2,0xd846eec4,0xdcf4eec4,0xd846eec6,
	0xdcf5eec6,0xd846eec8,0xdcf6eec8,0xd846eeca,0xdcf7eeca,0xd846eecc,0xdcf8eecc,0xd846eece,
	0xdcf9eece,0xd846eed0,0xdcfaeed0,0xd846eed2,0xdcfbeed2,0xd846eed4,0xdcfceed4,0xd846eed6,
	0xdcfdeed6,0xd846eed8,0xdcfeeed8,0xd846eeda,0xdcffeeda,0xd846eedc,0xdd00eedc,0xd846eede,
	0xdd01eede,0xd846eee0,0xdd02eee0,0xd846eee2,0xdd03eee2,0xd846eee4,0xdd04eee4,0xd846eee6,
	0xdd05eee6,0xd846eee8,0xdd06eee8,0xd846eeea,0xdd07eeea,0xd846eeec,0xdd08eeec,0xd846eeee,
	0xdd09eeee,0xd846eef0,0xdd0aeef0,0xd846eef2,0xdd0beef2,0xd846eef4,0xdd0ceef4,0xd846eef6,
	0xdd0deef6,0xd846eef8,0xdd0eeef8,0xd846eefa,0xdd0feefa,0xd846eefc,0xdd10eefc,0xd846eefe,
	0xdd11eefe,0xd846ef00,0xdd12ef00,0xd846ef02,0xdd13ef02,0xd846ef04,0xdd14ef04,0xd846ef06,
	0xdd15ef06,0xd846ef08,0xdd16ef08,0xd846ef0a,0xdd17ef0a,0xd846ef0c,0xdd18ef0c,0xd846ef0e,
	0xdd19ef0e,0xd846ef10,0xdd1aef10,0xd846ef12,0xdd1bef12,0xd846ef14,0xdd1cef14,0xd846ef16,
	0xdd1def16,0xd846ef18,0xdd1eef18,0xd846ef1a,0xdd1fef1a,0xd846ef1c,0xdd20ef1c,0xd846ef1e,
	0xdd21ef1e,0xd846ef20,0xdd22ef20,0xd846ef22,0xdd23ef22,0xd846ef24,0xdd24ef24,0xd846ef26,
	0xdd25ef26,0xd846ef28,0xdd26ef28,0xd846ef2a,0xdd27ef2a,0xd846ef2c,0xdd28ef2c,0xd846ef2e,
	0xdd29ef2e,0xd846ef30,0xdd2aef30,0xd846ef32,0xdd2bef32,0xd846ef34,0xdd2cef34,0xd846ef36,
	0xdd2def36,0xd846ef38,0xdd2eef38,0xd846ef3a,0xdd2fef3a,0xd846ef3c,0xdd30ef3c,0xd846ef3e,
	0xdd31ef3e,0xd846ef40,0xdd32ef40,0xd846ef42,0xdd33ef42,0xd846ef44,0xdd34ef44,0xd846ef46,
	0xdd35ef46,0xd846ef48,0xdd36ef48,0xd846ef4a,0xdd37ef4a,0xd846ef4c,0xdd38ef4c,0xd846ef4e,
	0xdd39ef4e,0xd846ef50,0xdd3aef50,0xd846ef52,0xdd3bef52,0xd846ef54,0xdd3cef54,0xd846ef56,
	0xdd3def56,0xd846ef58,0xdd3eef58,0xd846ef5a,0xdd3fef5a,0xd846ef5c,0xdd40ef5c,0xd846ef5e,
	0xdd41ef5e,0xd846ef60,0xdd42ef60,0xd846ef62,0xdd43ef62,0xd846ef64,0xdd44ef64,0xd846ef66,
	0xdd45ef66,0xd846ef68,0xdd46ef68,0xd846ef6a,0xdd47ef6a,0xd846ef6c,0xdd48ef6c,0xd846ef6e,
	0xdd49ef6e,0xd846ef70,0xdd4aef70,0xd846ef72,0xdd4bef72,0xd846ef74,0xdd4cef74,0xd846ef76,
	0xdd4def76,0xd846ef78,0xdd4eef78,0xd846ef7a,0xdd4fef7a,0xd846ef7c,0xdd50ef7c,0xd846ef7e,
	0xdd51ef7e,0xd846ef80,0xdd52ef80,0xd846ef82,0xdd53ef82,0xd846ef84,0xdd54ef84,0xd846ef86,
	0xdd55ef86,0xd846ef88,0xdd56ef88,0xd846ef8a,0xdd57ef8a,0xd846ef8c,0xdd58ef8c,0xd846ef8e,
	0xdd59ef8e,0xd846ef90,0xdd5aef90,0xd846ef92,0xdd5bef92,0xd846ef94,0xdd5cef94,0xd846ef96,
	0xdd5def96,0xd846ef98,0xdd5eef98,0xd846ef9a,0xdd5fef9a,0xd846ef9c,0xdd60ef9c,0xd846ef9e,
	0xdd61ef9e,0xd846efa0,0xdd62efa0,0xd846efa2,0xdd63efa2,0xd846efa4,0xdd64efa4,0xd846efa6,
	0xdd65efa6,0xd846efa8,0xdd66efa8,0xd846efaa,0xdd67efaa,0xd846efac,0xdd68efac,0xd846efae,
	0xdd69efae,0xd846efb0,0xdd6aefb0,0xd846efb2,0xdd6befb2,0xd846efb4,0xdd6cefb4,0xd846efb6,
	0xdd6defb6,0xd846efb8,0xdd6eefb8,0xd846efba,0xdd6fefba,0xd846efbc,0xdd70efbc,0xd846efbe,
	0xdd71efbe,0xd846efc0,0xdd72efc0,0xd846efc2,0xdd73efc2,0xd846efc4,0xdd74efc4,0xd846efc6,
	0xdd75efc6,0xd846efc8,0xdd76efc8,0xd846efca,0xdd77efca,0xd846efcc,0xdd78efcc,0xd846efce,
	0xdd79efce,0xd846efd0,0xdd7aefd0,0xd846efd2,0xdd7befd2,0xd846efd4,0xdd7cefd4,0xd846efd6,
	0xdd7defd6,0xd846efd8,0xdd7eefd8,0xd846efda,0xdd7fefda,0xd846efdc,0xdd80efdc,0xd846efde,
	0xdd81efde,0xd846efe0,0xdd82efe0,0xd846efe2,0xdd83efe2,0xd846efe4,0xdd84efe4,0xd846efe6,
	0xdd85efe6,0xd846efe8,0xdd86efe8,0xd846efea,0xdd87efea,0xd846efec,0xdd88efec,0xd846efee,
	0xdd89efee,0xd846eff0,0xdd8aeff0,0xd846eff2,0xdd8beff2,0xd846eff4,0xdd8ceff4,0xd846eff6,
	0xdd8deff6,0xd846eff8,0xdd8eeff8,0xd846effa,0xdd8feffa,0xd846effc,0xdd90effc,0xd846effe,
	0xdd91effe,0xd846f000,0xdd92f000,0xd846f002,0xdd93f002,0xd846f004,0xdd94f004,0xd846f006,
	0xdd95f006,0xd846f008,0xdd96f008,0xd846f00a,0xdd97f00a,0xd846f00c,0xdd98f00c,0xd846f00e,
	0xdd99f00e,0xd846f010,0xdd9af010,0xd846f012,0xdd9bf012,0xd846f014,0xdd9cf014,0xd846f016,
	0xdd9df016,0xd846f018,0xdd9ef018,0xd846f01a,0xdd9ff01a,0xd846f01c,0xdda0f01c,0xd846f01e,
	0xdda1f01e,0xd846f020,0xdda2f020,0xd846f022,0xdda3f022,0xd846f024,0xdda4f024,0xd846f026,
	0xdda5f026,0xd846f028,0xdda6f028,0xd846f02a,0xdda7f02a,0xd846f02c,0xdda8f02c,0xd846f02e,
	0xdda9f02e,0xd846f030,0xddaaf030,0xd846f032,0xddabf032,0xd846f034,0xddacf034,0xd846f036,
	0xddadf036,0xd846f038,0xddaef038,0xd846f03a,0xddaff03a,0xd846f03c,0xddb0f03c,0xd846f03e,
	0xddb1f03e,0xd846f040,0xddb2f040,0xd846f042,0xddb3f042,0xd846f044,0xddb4f044,0xd846f046,
	0xddb5f046,0xd846f048,0xddb6f048,0xd846f04a,0xddb7f04a,0xd846f04c,0xddb8f04c,0xd846f04e,
	0xddb9f04e,0xd846f050,0xddbaf050,0xd846f052,0xddbbf052,0xd846f054,0xddbcf054,0xd846f056,
	0xddbdf056,0xd846f058,0xddbef058,0xd846f05a,0xddbff05a,0xd846f05c,0xddc0f05c,0xd846f05e,
	0xddc1f05e,0xd846f060,0xddc2f060,0xd846f062,0xddc3f062,0xd846f064,0xddc4f064,0xd846f066,
	0xddc5f066,0xd846f068,0xddc6f068,0xd846f06a,0xddc7f06a,0xd846f06c,0xddc8f06c,0xd846f06e,
	0xddc9f06e,0xd846f070,0xddcaf070,0xd846f072,0xddcbf072,0xd846f074,0xddccf074,0xd846f076,
	0xddcdf076,0xd846f078,0xddcef078,0xd846f07a,0xddcff07a,0xd846f07c,0xddd0f07c,0xd846f07e,
	0xddd1f07e,0xd846f080,0xddd2f080,0xd846f082,0xddd3f082,0xd846f084,0xddd4f084,0xd846f086,
	0xddd5f086,0xd846f088,0xddd6f088,0xd846f08a,0xddd7f08a,0xd846f08c,0xddd8f08c,0xd846f08e,
	0xddd9f08e,0xd846f090,0xdddaf090,0xd846f092,0xdddbf092,0xd846f094,0xdddcf094,0xd846f096,
	0xddddf096,0xd846f098,0xdddef098,0xd846f09a,0xdddff09a,0xd846f09c,0xdde0f09c,0xd846f09e,
	0xdde1f09e,0xd846f0a0,0xdde2f0a0,0xd846f0a2,0xdde3f0a2,0xd846f0a4,0xdde4f0a4,0xd846f0a6,
	0xdde5f0a6,0xd846f0a8,0xdde6f0a8,0xd846f0aa,0xdde7f0aa,0xd846f0ac,0xdde8f0ac,0xd846f0ae,
	0xdde9f0ae,0xd846f0b0,0xddeaf0b0,0xd846f0b2,0xddebf0b2,0xd846f0b4,0xddecf0b4,0xd846f0b6,
	0xddedf0b6,0xd846f0b8,0xddeef0b8,0xd846f0ba,0xddeff0ba,0xd846f0bc,0xddf0f0bc,0xd846f0be,
	0xddf1f0be,0xd846f0c0,0xddf2f0c0,0xd846f0c2,0xddf3f0c2,0xd846f0c4,0xddf4f0c4,0xd846f0c6,
	0xddf5f0c6,0xd846f0c8,0xddf6f0c8,0xd846f0ca,0xddf7f0ca,0xd846f0cc,0xddf8f0cc,0xd846f0ce,
	0xddf9f0ce,0xd846f0d0,0xddfaf0d0,0xd846f0d2,0xddfbf0d2,0xd846f0d4,0xddfcf0d4,0xd846f0d6,
	0xddfdf0d6,0xd846f0d8,0xddfef0d8,0xd846f0da,0xddfff0da,0xd846f0dc,0xde00f0dc,0xd846f0de,
	0xde01f0de,0xd846f0e0,0xde02f0e0,0xd846f0e2,0xde03f0e2,0xd846f0e4,0xde04f0e4,0xd846f0e6,
	0xde05f0e6,0xd846f0e8,0xde06f0e8,0xd846f0ea,0xde07f0ea,0xd846f0ec,0xde08f0ec,0xd846f0ee,
	0xde09f0ee,0xd846f0f0,0xde0af0f0,0xd846f0f2,0xde0bf0f2,0xd846f0f4,0xde0cf0f4,0xd846f0f6,
	0xde0df0f6,0xd846f0f8,0xde0ef0f8,0xd846f0fa,0xde0ff0fa,0xd846f0fc,0xde10f0fc,0xd846f0fe,
	0xde11f0fe,0xd846f100,0xde12f100,0xd846f102,0xde13f102,0xd846f104,0xde14f104,0xd846f106,
	0xde15f106,0xd846f108,0xde16f108,0xd846f10a,0xde17f10a,0xd846f10c,0xde18f10c,0xd846f10e,
	0xde19f10e,0xd846f110,0xde1af110,0xd846f112,0xde1bf112,0xd846f114,0xde1cf114,0xd846f116,
	0xde1df116,0xd846f118,0xde1ef118,0xd846f11a,0xde1ff11a,0xd846f11c,0xde20f11c,0xd846f11e,
	0xde21f11e,0xd846f120,0xde22f120,0xd846f122,0xde23f122,0xd846f124,0xde24f124,0xd846f126,
	0xde25f126,0xd846f128,0xde26f128,0xd846f12a,0xde27f12a,0xd846f12c,0xde28f12c,0xd846f12e,
	0xde29f12e,0xd846f130,0xde2af130,0xd846f132,0xde2bf132,0xd846f134,0xde2cf134,0xd846f136,
	0xde2df136,0xd846f138,0xde2ef138,0xd846f13a,0xde2ff13a,0xd846f13c,0xde30f13c,0xd846f13e,
	0xde31f13e,0xd846f140,0xde32f140,0xd846f142,0xde33f142,0xd846f144,0xde34f144,0xd846f146,
	0xde35f146,0xd846f148,0xde36f148,0xd846f14a,0xde37f14a,0xd846f14c,0xde38f14c,0xd846f14e,
	0xde39f14e,0xd846f150,0xde3af150,0xd846f152,0xde3bf152,0xd846f154,0xde3cf154,0xd846f156,
	0xde3df156,0xd846f158,0xde3ef158,0xd846f15a,0xde3ff15a,0xd846f15c,0xde40f15c,0xd846f15e,
	0xde41f15e,0xd846f160,0xde42f160,0xd846f162,0xde43f162,0xd846f164,0xde44f164,0xd846f166,
	0xde45f166,0xd846f168,0xde46f168,0xd846f16a,0xde47f16a,0xd846f16c,0xde48f16c,0xd846f16e,
	0xde49f16e,0xd846f170,0xde4af170,0xd846f172,0xde4bf172,0xd846f174,0xde4cf174,0xd846f176,
	0xde4df176,0xd846f178,0xde4ef178,0xd846f17a,0xde4ff17a,0xd846f17c,0xde50f17c,0xd846f17e,
	0xde51f17e,0xd846f180,0xde52f180,0xd846f182,0xde53f182,0xd846f184,0xde54f184,0xd846f186,
	0xde55f186,0xd846f188,0xde56f188,0xd846f18a,0xde57f18a,0xd846f18c,0xde58f18c,0xd846f18e,
	0xde59f18e,0xd846f190,0xde5af190,0xd846f192,0xde5bf192,0xd846f194,0xde5cf194,0xd846f196,
	0xde5df196,0xd846f198,0xde5ef198,0xd846f19a,0xde5ff19a,0xd846f19c,0xde60f19c,0xd846f19e,
	0xde61f19e,0xd846f1a0,0xde62f1a0,0xd846f1a2,0xde63f1a2,0xd846f1a4,0xde64f1a4,0xd846f1a6,
	0xde65f1a6,0xd846f1a8,0xde66f1a8,0xd846f1aa,0xde67f1aa,0xd846f1ac,0xde68f1ac,0xd846f1ae,
	0xde69f1ae,0xd846f1b0,0xde6af1b0,0xd846f1b2,0xde6bf1b2,0xd846f1b4,0xde6cf1b4,0xd846f1b6,
	0xde6df1b6,0xd846f1b8,0xde6ef1b8,0xd846f1ba,0xde6ff1ba,0xd846f1bc,0xde70f1bc,0xd846f1be,
	0xde71f1be,0xd846f1c0,0xde72f1c0,0xd846f1c2,0xde73f1c2,0xd846f1c4,0xde74f1c4,0xd846f1c6,
	0xde75f1c6,0xd846f1c8,0xde76f1c8,0xd846f1ca,0xde77f1ca,0xd846f1cc,0xde78f1cc,0xd846f1ce,
	0xde79f1ce,0xd846f1d0,0xde7af1d0,0xd846f1d2,0xde7bf1d2,0xd846f1d4,0xde7cf1d4,0xd846f1d6,
	0xde7df1d6,0xd846f1d8,0xde7ef1d8,0xd846f1da,0xde7ff1da,0xd846f1dc,0xde80f1dc,0xd846f1de,
	0xde81f1de,0xd846f1e0,0xde82f1e0,0xd846f1e2,0xde83f1e2,0xd846f1e4,0xde84f1e4,0xd846f1e6,
	0xde85f1e6,0xd846f1e8,0xde86f1e8,0xd846f1ea,0xde87f1ea,0xd846f1ec,0xde88f1ec,0xd846f1ee,
	0xde89f1ee,0xd846f1f0,0xde8af1f0,0xd846f1f2,0xde8bf1f2,0xd846f1f4,0xde8cf1f4,0xd846f1f6,
	0xde8df1f6,0xd846f1f8,0xde8ef1f8,0xd846f1fa,0xde8ff1fa,0xd846f1fc,0xde90f1fc,0xd846f1fe,
	0xde91f1fe,0xd846f200,0xde92f200,0xd846f202,0xde93f202,0xd846f204,0xde94f204,0xd846f206,
	0xde95f206,0xd846f208,0xde96f208,0xd846f20a,0xde97f20a,0xd846f20c,0xde98f20c,0xd846f20e,
	0xde99f20e,0xd846f210,0xde9af210,0xd846f212,0xde9bf212,0xd846f214,0xde9cf214,0xd846f216,
	0xde9df216,0xd846f218,0xde9ef218,0xd846f21a,0xde9ff21a,0xd846f21c,0xdea0f21c,0xd846f21e,
	0xdea1f21e,0xd846f220,0xdea2f220,0xd846f222,0xdea3f222,0xd846f224,0xdea4f224,0xd846f226,
	0xdea5f226,0xd846f228,0xdea6f228,0xd846f22a,0xdea7f22a,0xd846f22c,0xdea8f22c,0xd846f22e,
	0xdea9f22e,0xd846f230,0xdeaaf230,0xd846f232,0xdeabf232,0xd846f234,0xdeacf234,0xd846f236,
	0xdeadf236,0xd846f238,0xdeaef238,0xd846f23a,0xdeaff23a,0xd846f23c,0xdeb0f23c,0xd846f23e,
	0xdeb1f23e,0xd846f240,0xdeb2f240,0xd846f242,0xdeb3f242,0xd846f244,0xdeb4f244,0xd846f246,
	0xdeb5f246,0xd846f248,0xdeb6f248,0xd846f24a,0xdeb7f24a,0xd846f24c,0xdeb8f24c,0xd846f24e,
	0xdeb9f24e,0xd846f250,0xdebaf250,0xd846f252,0xdebbf252,0xd846f254,0xdebcf254,0xd846f256,
	0xdebdf256,0xd846f258,0xdebef258,0xd846f25a,0xdebff25a,0xd846f25c,0xdec0f25c,0xd846f25e,
	0xdec1f25e,0xd846f260,0xdec2f260,0xd846f262,0xdec3f262,0xd846f264,0xdec4f264,0xd846f266,
	0xdec5f266,0xd846f268,0xdec6f268,0xd846f26a,0xdec7f26a,0xd846f26c,0xdec8f26c,0xd846f26e,
	0xdec9f26e,0xd846f270,0xdecaf270,0xd846f272,0xdecbf272,0xd846f274,0xdeccf274,0xd846f276,
	0xdecdf276,0xd846f278,0xdecef278,0xd846f27a,0xdecff27a,0xd846f27c,0xded0f27c,0xd846f27e,
	0xded1f27e,0xd846f280,0xded2f280,0xd846f282,0xded3f282,0xd846f284,0xded4f284,0xd846f286,
	0xded5f286,0xd846f288,0xded6f288,0xd846f28a,0xded7f28a,0xd846f28c,0xded8f28c,0xd846f28e,
	0xded9f28e,0xd846f290,0xdedaf290,0xd846f292,0xdedbf292,0xd846f294,0xdedcf294,0xd846f296,
	0xdeddf296,0xd846f298,0xdedef298,0xd846f29a,0xdedff29a,0xd846f29c,0xdee0f29c,0xd846f29e,
	0xdee1f29e,0xd846f2a0,0xdee2f2a0,0xd846f2a2,0xdee3f2a2,0xd846f2a4,0xdee4f2a4,0xd846f2a6,
	0xdee5f2a6,0xd846f2a8,0xdee6f2a8,0xd846f2aa,0xdee7f2aa,0xd846f2ac,0xdee8f2ac,0xd846f2ae,
	0xdee9f2ae,0xd846f2b0,0xdeeaf2b0,0xd846f2b2,0xdeebf2b2,0xd846f2b4,0xdeecf2b4,0xd846f2b6,
	0xdeedf2b6,0xd846f2b8,0xdeeef2b8,0xd846f2ba,0xdeeff2ba,0xd846f2bc,0xdef0f2bc,0xd846f2be,
	0xdef1f2be,0xd846f2c0,0xdef2f2c0,0xd846f2c2,0xdef3f2c2,0xd846f2c4,0xdef4f2c4,0xd846f2c6,
	0xdef5f2c6,0xd846f2c8,0xdef6f2c8,0xd846f2ca,0xdef7f2ca,0xd846f2cc,0xdef8f2cc,0xd846f2ce,
	0xdef9f2ce,0xd846f2d0,0xdefaf2d0,0xd846f2d2,0xdefbf2d2,0xd846f2d4,0xdefcf2d4,0xd846f2d6,
	0xdefdf2d6,0xd846f2d8,0xdefef2d8,0xd846f2da,0xdefff2da,0xd846f2dc,0xdf00f2dc,0xd846f2de,
	0xdf01f2de,0xd846f2e0,0xdf02f2e0,0xd846f2e2,0xdf03f2e2,0xd846f2e4,0xdf04f2e4,0xd846f2e6,
	0xdf05f2e6,0xd846f2e8,0xdf06f2e8,0xd846f2ea,0xdf07f2ea,0xd846f2ec,0xdf08f2ec,0xd846f2ee,
	0xdf09f2ee,0xd846f2f0,0xdf0af2f0,0xd846f2f2,0xdf0bf2f2,0xd846f2f4,0xdf0cf2f4,0xd846f2f6,
	0xdf0df2f6,0xd846f2f8,0xdf0ef2f8,0xd846f2fa,0xdf0ff2fa,0xd846f2fc,0xdf10f2fc,0xd846f2fe,
	0xdf11f2fe,0xd846f300,0xdf12f300,0xd846f302,0xdf13f302,0xd846f304,0xdf14f304,0xd846f306,
	0xdf15f306,0xd846f308,0xdf16f308,0xd846f30a,0xdf17f30a,0xd846f30c,0xdf18f30c,0xd846f30e,
	0xdf19f30e,0xd846f310,0xdf1af310,0xd846f312,0xdf1bf312,0xd846f314,0xdf1cf314,0xd846f316,
	0xdf1df316,0xd846f318,0xdf1ef318,0xd846f31a,0xdf1ff31a,0xd846f31c,0xdf20f31c,0xd846f31e,
	0xdf21f31e,0xd846f320,0xdf22f320,0xd846f322,0xdf23f322,0xd846f324,0xdf24f324,0xd846f326,
	0xdf25f326,0xd846f328,0xdf26f328,0xd846f32a,0xdf27f32a,0xd846f32c,0xdf28f32c,0xd846f32e,
	0xdf29f32e,0xd846f330,0xdf2af330,0xd846f332,0xdf2bf332,0xd846f334,0xdf2cf334,0xd846f336,
	0xdf2df336,0xd846f338,0xdf2ef338,0xd846f33a,0xdf2ff33a,0xd846f33c,0xdf30f33c,0xd846f33e,
	0xdf31f33e,0xd846f340,0xdf32f340,0xd846f342,0xdf33f342,0xd846f344,0xdf34f344,0xd846f346,
	0xdf35f346,0xd846f348,0xdf36f348,0xd846f34a,0xdf37f34a,0xd846f34c,0xdf38f34c,0xd846f34e,
	0xdf39f34e,0xd846f350,0xdf3af350,0xd846f352,0xdf3bf352,0xd846f354,0xdf3cf354,0xd846f356,
	0xdf3df356,0xd846f358,0xdf3ef358,0xd846f35a,0xdf3ff35a,0xd846f35c,0xdf40f35c,0xd846f35e,
	0xdf41f35e,0xd846f360,0xdf42f360,0xd846f362,0xdf43f362,0xd846f364,0xdf44f364,0xd846f366,
	0xdf45f366,0xd846f368,0xdf46f368,0xd846f36a,0xdf47f36a,0xd846f36c,0xdf48f36c,0xd846f36e,
	0xdf49f36e,0xd846f370,0xdf4af370,0xd846f372,0xdf4bf372,0xd846f374,0xdf4cf374,0xd846f376,
	0xdf4df376,0xd846f378,0xdf4ef378,0xd846f37a,0xdf4ff37a,0xd846f37c,0xdf50f37c,0xd846f37e,
	0xdf51f37e,0xd846f380,0xdf52f380,0xd846f382,0xdf53f382,0xd846f384,0xdf54f384,0xd846f386,
	0xdf55f386,0xd846f388,0xdf56f388,0xd846f38a,0xdf57f38a,0xd846f38c,0xdf58f38c,0xd846f38e,
	0xdf59f38e,0xd846f390,0xdf5af390,0xd846f392,0xdf5bf392,0xd846f394,0xdf5cf394,0xd846f396,
	0xdf5df396,0xd846f398,0xdf5ef398,0xd846f39a,0xdf5ff39a,0xd846f39c,0xdf60f39c,0xd846f39e,
	0xdf61f39e,0xd846f3a0,0xdf62f3a0,0xd846f3a2,0xdf63f3a2,0xd846f3a4,0xdf64f3a4,0xd846f3a6,
	0xdf65f3a6,0xd846f3a8,0xdf66f3a8,0xd846f3aa,0xdf67f3aa,0xd846f3ac,0xdf68f3ac,0xd846f3ae,
	0xdf69f3ae,0xd846f3b0,0xdf6af3b0,0xd846f3b2,0xdf6bf3b2,0xd846f3b4,0xdf6cf3b4,0xd846f3b6,
	0xdf6df3b6,0xd846f3b8,0xdf6ef3b8,0xd846f3ba,0xdf6ff3ba,0xd846f3bc,0xdf70f3bc,0xd846f3be,
	0xdf71f3be,0xd846f3c0,0xdf72f3c0,0xd846f3c2,0xdf73f3c2,0xd846f3c4,0xdf74f3c4,0xd846f3c6,
	0xdf75f3c6,0xd846f3c8,0xdf76f3c8,0xd846f3ca,0xdf77f3ca,0xd846f3cc,0xdf78f3cc,0xd846f3ce,
	0xdf79f3ce,0xd846f3d0,0xdf7af3d0,0xd846f3d2,0xdf7bf3d2,0xd846f3d4,0xdf7cf3d4,0xd846f3d6,
	0xdf7df3d6,0xd846f3d8,0xdf7ef3d8,0xd846f3da,0xdf7ff3da,0xd846f3dc,0xdf80f3dc,0xd846f3de,
	0xdf81f3de,0xd846f3e0,0xdf82f3e0,0xd846f3e2,0xdf83f3e2,0xd846f3e4,0xdf84f3e4,0xd846f3e6,
	0xdf85f3e6,0xd846f3e8,0xdf86f3e8,0xd846f3ea,0xdf87f3ea,0xd846f3ec,0xdf88f3ec,0xd846f3ee,
	0xdf89f3ee,0xd846f3f0,0xdf8af3f0,0xd846f3f2,0xdf8bf3f2,0xd846f3f4,0xdf8cf3f4,0xd846f3f6,
	0xdf8df3f6,0xd846f3f8,0xdf8ef3f8,0xd846f3fa,0xdf8ff3fa,0xd846f3fc,0xdf90f3fc,0xd846f3fe,
	0xdf91f3fe,0xd846f400,0xdf92f400,0xd846f402,0xdf93f402,0xd846f404,0xdf94f404,0xd846f406,
	0xdf95f406,0xd846f408,0xdf96f408,0xd846f40a,0xdf97f40a,0xd846f40c,0xdf98f40c,0xd846f40e,
	0xdf99f40e,0xd846f410,0xdf9af410,0xd846f412,0xdf9bf412,0xd846f414,0xdf9cf414,0xd846f416,
	0xdf9df416,0xd846f418,0xdf9ef418,0xd846f41a,0xdf9ff41a,0xd846f41c,0xdfa0f41c,0xd846f41e,
	0xdfa1f41e,0xd846f420,0xdfa2f420,0xd846f422,0xdfa3f422,0xd846f424,0xdfa4f424,0xd846f426,
	0xdfa5f426,0xd846f428,0xdfa6f428,0xd846f42a,0xdfa7f42a,0xd846f42c,0xdfa8f42c,0xd846f42e,
	0xdfa9f42e,0xd846f430,0xdfaaf430,0xd846f432,0xdfabf432,0xd846f434,0xdfacf434,0xd846f436,
	0xdfadf436,0xd846f438,0xdfaef438,0xd846f43a,0xdfaff43a,0xd846f43c,0xdfb0f43c,0xd846f43e,
	0xdfb1f43e,0xd846f440,0xdfb2f440,0xd846f442,0xdfb3f442,0xd846f444,0xdfb4f444,0xd846f446,
	0xdfb5f446,0xd846f448,0xdfb6f448,0xd846f44a,0xdfb7f44a,0xd846f44c,0xdfb8f44c,0xd846f44e,
	0xdfb9f44e,0xd846f450,0xdfbaf450,0xd846f452,0xdfbbf452,0xd846f454,0xdfbcf454,0xd846f456,
	0xdfbdf456,0xd846f458,0xdfbef458,0xd846f45a,0xdfbff45a,0xd846f45c,0xdfc0f45c,0xd846f45e,
	0xdfc1f45e,0xd846f460,0xdfc2f460,0xd846f462,0xdfc3f462,0xd846f464,0xdfc4f464,0xd846f466,
	0xdfc5f466,0xd846f468,0xdfc6f468,0xd846f46a,0xdfc7f46a,0xd846f46c,0xdfc8f46c,0xd846f46e,
	0xdfc9f46e,0xd846f470,0xdfcaf470,0xd846f472,0xdfcbf472,0xd846f474,0xdfccf474,0xd846f476,
	0xdfcdf476,0xd846f478,0xdfcef478,0xd846f47a,0xdfcff47a,0xd846f47c,0xdfd0f47c,0xd846f47e,
	0xdfd1f47e,0xd846f480,0xdfd2f480,0xd846f482,0xdfd3f482,0xd846f484,0xdfd4f484,0xd846f486,
	0xdfd5f486,0xd846f488,0xdfd6f488,0xd846f48a,0xdfd7f48a,0xd846f48c,0xdfd8f48c,0xd846f48e,
	0xdfd9f48e,0xd846f490,0xdfdaf490,0xd846f492,0xdfdbf492,0xd846f494,0xdfdcf494,0xd846f496,
	0xdfddf496,0xd846f498,0xdfdef498,0xd846f49a,0xdfdff49a,0xd846f49c,0xdfe0f49c,0xd846f49e,
	0xdfe1f49e,0xd846f4a0,0xdfe2f4a0,0xd846f4a2,0xdfe3f4a2,0xd846f4a4,0xdfe4f4a4,0xd846f4a6,
	0xdfe5f4a6,0xd846f4a8,0xdfe6f4a8,0xd846f4aa,0xdfe7f4aa,0xd846f4ac,0xdfe8f4ac,0xd846f4ae,
	0xdfe9f4ae,0xd846f4b0,0xdfeaf4b0,0xd846f4b2,0xdfebf4b2,0xd846f4b4,0xdfecf4b4,0xd846f4b6,
	0xdfedf4b6,0xd846f4b8,0xdfeef4b8,0xd846f4ba,0xdfeff4ba,0xd846f4bc,0xdff0f4bc,0xd846f4be,
	0xdff1f4be,0xd846f4c0,0xdff2f4c0,0xd846f4c2,0xdff3f4c2,0xd846f4c4,0xdff4f4c4,0xd846f4c6,
	0xdff5f4c6,0xd846f4c8,0xdff6f4c8,0xd846f4ca,0xdff7f4ca,0xd846f4cc,0xdff8f4cc,0xd846f4ce,
	0xdff9f4ce,0xd846f4d0,0xdffaf4d0,0xd846f4d2,0xdffbf4d2,0xd846f4d4,0xdffcf4d4,0xd846f4d6,
	0xdffdf4d6,0xd846f4d8,0xdffef4d8,0xd846f4da,0xdffff4da,0xd847f4dc,0xdc00f4dc,0xd847f4de,
	0xdc01f4de,0xd847f4e0,0xdc02f4e0,0xd847f4e2,0xdc03f4e2,0xd847f4e4,0xdc04f4e4,0xd847f4e6,
	0xdc05f4e6,0xd847f4e8,0xdc06f4e8,0xd847f4ea,0xdc07f4ea,0xd847f4ec,0xdc08f4ec,0xd847f4ee,
	0xdc09f4ee,0xd847f4f0,0xdc0af4f0,0xd847f4f2,0xdc0bf4f2,0xd847f4f4,0xdc0cf4f4,0xd847f4f6,
	0xdc0df4f6,0xd847f4f8,0xdc0ef4f8,0xd847f4fa,0xdc0ff4fa,0xd847f4fc,0xdc10f4fc,0xd847f4fe,
	0xdc11f4fe,0xd847f500,0xdc12f500,0xd847f502,0xdc13f502,0xd847f504,0xdc14f504,0xd847f506,
	0xdc15f506,0xd847f508,0xdc16f508,0xd847f50a,0xdc17f50a,0xd847f50c,0xdc18f50c,0xd847f50e,
	0xdc19f50e,0xd847f510,0xdc1af510,0xd847f512,0xdc1bf512,0xd847f514,0xdc1cf514,0xd847f516,
	0xdc1df516,0xd847f518,0xdc1ef518,0xd847f51a,0xdc1ff51a,0xd847f51c,0xdc20f51c,0xd847f51e,
	0xdc21f51e,0xd847f520,0xdc22f520,0xd847f522,0xdc23f522,0xd847f524,0xdc24f524,0xd847f526,
	0xdc25f526,0xd847f528,0xdc26f528,0xd847f52a,0xdc27f52a,0xd847f52c,0xdc28f52c,0xd847f52e,
	0xdc29f52e,0xd847f530,0xdc2af530,0xd847f532,0xdc2bf532,0xd847f534,0xdc2cf534,0xd847f536,
	0xdc2df536,0xd847f538,0xdc2ef538,0xd847f53a,0xdc2ff53a,0xd847f53c,0xdc30f53c,0xd847f53e,
	0xdc31f53e,0xd847f540,0xdc32f540,0xd847f542,0xdc33f542,0xd847f544,0xdc34f544,0xd847f546,
	0xdc35f546,0xd847f548,0xdc36f548,0xd847f54a,0xdc37f54a,0xd847f54c,0xdc38f54c,0xd847f54e,
	0xdc39f54e,0xd847f550,0xdc3af550,0xd847f552,0xdc3bf552,0xd847f554,0xdc3cf554,0xd847f556,
	0xdc3df556,0xd847f558,0xdc3ef558,0xd847f55a,0xdc3ff55a,0xd847f55c,0xdc40f55c,0xd847f55e,
	0xdc41f55e,0xd847f560,0xdc42f560,0xd847f562,0xdc43f562,0xd847f564,0xdc44f564,0xd847f566,
	0xdc45f566,0xd847f568,0xdc46f568,0xd847f56a,0xdc47f56a,0xd847f56c,0xdc48f56c,0xd847f56e,
	0xdc49f56e,0xd847f570,0xdc4af570,0xd847f572,0xdc4bf572,0xd847f574,0xdc4cf574,0xd847f576,
	0xdc4df576,0xd847f578,0xdc4ef578,0xd847f57a,0xdc4ff57a,0xd847f57c,0xdc50f57c,0xd847f57e,
	0xdc51f57e,0xd847f580,0xdc52f580,0xd847f582,0xdc53f582,0xd847f584,0xdc54f584,0xd847f586,
	0xdc55f586,0xd847f588,0xdc56f588,0xd847f58a,0xdc57f58a,0xd847f58c,0xdc58f58c,0xd847f58e,
	0xdc59f58e,0xd847f590,0xdc5af590,0xd847f592,0xdc5bf592,0xd847f594,0xdc5cf594,0xd847f596,
	0xdc5df596,0xd847f598,0xdc5ef598,0xd847f59a,0xdc5ff59a,0xd847f59c,0xdc60f59c,0xd847f59e,
	0xdc61f59e,0xd847f5a0,0xdc62f5a0,0xd847f5a2,0xdc63f5a2,0xd847f5a4,0xdc64f5a4,0xd847f5a6,
	0xdc65f5a6,0xd847f5a8,0xdc66f5a8,0xd847f5aa,0xdc67f5aa,0xd847f5ac,0xdc68f5ac,0xd847f5ae,
	0xdc69f5ae,0xd847f5b0,0xdc6af5b0,0xd847f5b2,0xdc6bf5b2,0xd847f5b4,0xdc6cf5b4,0xd847f5b6,
	0xdc6df5b6,0xd847f5b8,0xdc6ef5b8,0xd847f5ba,0xdc6ff5ba,0xd847f5bc,0xdc70f5bc,0xd847f5be,
	0xdc71f5be,0xd847f5c0,0xdc72f5c0,0xd847f5c2,0xdc73f5c2,0xd847f5c4,0xdc74f5c4,0xd847f5c6,
	0xdc75f5c6,0xd847f5c8,0xdc76f5c8,0xd847f5ca,0xdc77f5ca,0xd847f5cc,0xdc78f5cc,0xd847f5ce,
	0xdc79f5ce,0xd847f5d0,0xdc7af5d0,0xd847f5d2,0xdc7bf5d2,0xd847f5d4,0xdc7cf5d4,0xd847f5d6,
	0xdc7df5d6,0xd847f5d8,0xdc7ef5d8,0xd847f5da,0xdc7ff5da,0xd847f5dc,0xdc80f5dc,0xd847f5de,
	0xdc81f5de,0xd847f5e0,0xdc82f5e0,0xd847f5e2,0xdc83f5e2,0xd847f5e4,0xdc84f5e4,0xd847f5e6,
	0xdc85f5e6,0xd847f5e8,0xdc86f5e8,0xd847f5ea,0xdc87f5ea,0xd847f5ec,0xdc88f5ec,0xd847f5ee,
	0xdc89f5ee,0xd847f5f0,0xdc8af5f0,0xd847f5f2,0xdc8bf5f2,0xd847f5f4,0xdc8cf5f4,0xd847f5f6,
	0xdc8df5f6,0xd847f5f8,0xdc8ef5f8,0xd847f5fa,0xdc8ff5fa,0xd847f5fc,0xdc90f5fc,0xd847f5fe,
	0xdc91f5fe,0xd847f600,0xdc92f600,0xd847f602,0xdc93f602,0xd847f604,0xdc94f604,0xd847f606,
	0xdc95f606,0xd847f608,0xdc96f608,0xd847f60a,0xdc97f60a,0xd847f60c,0xdc98f60c,0xd847f60e,
	0xdc99f60e,0xd847f610,0xdc9af610,0xd847f612,0xdc9bf612,0xd847f614,0xdc9cf614,0xd847f616,
	0xdc9df616,0xd847f618,0xdc9ef618,0xd847f61a,0xdc9ff61a,0xd847f61c,0xdca0f61c,0xd847f61e,
	0xdca1f61e,0xd847f620,0xdca2f620,0xd847f622,0xdca3f622,0xd847f624,0xdca4f624,0xd847f626,
	0xdca5f626,0xd847f628,0xdca6f628,0xd847f62a,0xdca7f62a,0xd847f62c,0xdca8f62c,0xd847f62e,
	0xdca9f62e,0xd847f630,0xdcaaf630,0xd847f632,0xdcabf632,0xd847f634,0xdcacf634,0xd847f636,
	0xdcadf636,0xd847f638,0xdcaef638,0xd847f63a,0xdcaff63a,0xd847f63c,0xdcb0f63c,0xd847f63e,
	0xdcb1f63e,0xd847f640,0xdcb2f640,0xd847f642,0xdcb3f642,0xd847f644,0xdcb4f644,0xd847f646,
	0xdcb5f646,0xd847f648,0xdcb6f648,0xd847f64a,0xdcb7f64a,0xd847f64c,0xdcb8f64c,0xd847f64e,
	0xdcb9f64e,0xd847f650,0xdcbaf650,0xd847f652,0xdcbbf652,0xd847f654,0xdcbcf654,0xd847f656,
	0xdcbdf656,0xd847f658,0xdcbef658,0xd847f65a,0xdcbff65a,0xd847f65c,0xdcc0f65c,0xd847f65e,
	0xdcc1f65e,0xd847f660,0xdcc2f660,0xd847f662,0xdcc3f662,0xd847f664,0xdcc4f664,0xd847f666,
	0xdcc5f666,0xd847f668,0xdcc6f668,0xd847f66a,0xdcc7f66a,0xd847f66c,0xdcc8f66c,0xd847f66e,
	0xdcc9f66e,0xd847f670,0xdccaf670,0xd847f672,0xdccbf672,0xd847f674,0xdcccf674,0xd847f676,
	0xdccdf676,0xd847f678,0xdccef678,0xd847f67a,0xdccff67a,0xd847f67c,0xdcd0f67c,0xd847f67e,
	0xdcd1f67e,0xd847f680,0xdcd2f680,0xd847f682,0xdcd3f682,0xd847f684,0xdcd4f684,0xd847f686,
	0xdcd5f686,0xd847f688,0xdcd6f688,0xd847f68a,0xdcd7f68a,0xd847f68c,0xdcd8f68c,0xd847f68e,
	0xdcd9f68e,0xd847f690,0xdcdaf690,0xd847f692,0xdcdbf692,0xd847f694,0xdcdcf694,0xd847f696,
	0xdcddf696,0xd847f698,0xdcdef698,0xd847f69a,0xdcdff69a,0xd847f69c,0xdce0f69c,0xd847f69e,
	0xdce1f69e,0xd847f6a0,0xdce2f6a0,0xd847f6a2,0xdce3f6a2,0xd847f6a4,0xdce4f6a4,0xd847f6a6,
	0xdce5f6a6,0xd847f6a8,0xdce6f6a8,0xd847f6aa,0xdce7f6aa,0xd847f6ac,0xdce8f6ac,0xd847f6ae,
	0xdce9f6ae,0xd847f6b0,0xdceaf6b0,0xd847f6b2,0xdcebf6b2,0xd847f6b4,0xdcecf6b4,0xd847f6b6,
	0xdcedf6b6,0xd847f6b8,0xdceef6b8,0xd847f6ba,0xdceff6ba,0xd847f6bc,0xdcf0f6bc,0xd847f6be,
	0xdcf1f6be,0xd847f6c0,0xdcf2f6c0,0xd847f6c2,0xdcf3f6c2,0xd847f6c4,0xdcf4f6c4,0xd847f6c6,
	0xdcf5f6c6,0xd847f6c8,0xdcf6f6c8,0xd847f6ca,0xdcf7f6ca,0xd847f6cc,0xdcf8f6cc,0xd847f6ce,
	0xdcf9f6ce,0xd847f6d0,0xdcfaf6d0,0xd847f6d2,0xdcfbf6d2,0xd847f6d4,0xdcfcf6d4,0xd847f6d6,
	0xdcfdf6d6,0xd847f6d8,0xdcfef6d8,0xd847f6da,0xdcfff6da,0xd847f6dc,0xdd00f6dc,0xd847f6de,
	0xdd01f6de,0xd847f6e0,0xdd02f6e0,0xd847f6e2,0xdd03f6e2,0xd847f6e4,0xdd04f6e4,0xd847f6e6,
	0xdd05f6e6,0xd847f6e8,0xdd06f6e8,0xd847f6ea,0xdd07f6ea,0xd847f6ec,0xdd08f6ec,0xd847f6ee,
	0xdd09f6ee,0xd847f6f0,0xdd0af6f0,0xd847f6f2,0xdd0bf6f2,0xd847f6f4,0xdd0cf6f4,0xd847f6f6,
	0xdd0df6f6,0xd847f6f8,0xdd0ef6f8,0xd847f6fa,0xdd0ff6fa,0xd847f6fc,0xdd10f6fc,0xd847f6fe,
	0xdd11f6fe,0xd847f700,0xdd12f700,0xd847f702,0xdd13f702,0xd847f704,0xdd14f704,0xd847f706,
	0xdd15f706,0xd847f708,0xdd16f708,0xd847f70a,0xdd17f70a,0xd847f70c,0xdd18f70c,0xd847f70e,
	0xdd19f70e,0xd847f710,0xdd1af710,0xd847f712,0xdd1bf712,0xd847f714,0xdd1cf714,0xd847f716,
	0xdd1df716,0xd847f718,0xdd1ef718,0xd847f71a,0xdd1ff71a,0xd847f71c,0xdd20f71c,0xd847f71e,
	0xdd21f71e,0xd847f720,0xdd22f720,0xd847f722,0xdd23f722,0xd847f724,0xdd24f724,0xd847f726,
	0xdd25f726,0xd847f728,0xdd26f728,0xd847f72a,0xdd27f72a,0xd847f72c,0xdd28f72c,0xd847f72e,
	0xdd29f72e,0xd847f730,0xdd2af730,0xd847f732,0xdd2bf732,0xd847f734,0xdd2cf734,0xd847f736,
	0xdd2df736,0xd847f738,0xdd2ef738,0xd847f73a,0xdd2ff73a,0xd847f73c,0xdd30f73c,0xd847f73e,
	0xdd31f73e,0xd847f740,0xdd32f740,0xd847f742,0xdd33f742,0xd847f744,0xdd34f744,0xd847f746,
	0xdd35f746,0xd847f748,0xdd36f748,0xd847f74a,0xdd37f74a,0xd847f74c,0xdd38f74c,0xd847f74e,
	0xdd39f74e,0xd847f750,0xdd3af750,0xd847f752,0xdd3bf752,0xd847f754,0xdd3cf754,0xd847f756,
	0xdd3df756,0xd847f758,0xdd3ef758,0xd847f75a,0xdd3ff75a,0xd847f75c,0xdd40f75c,0xd847f75e,
	0xdd41f75e,0xd847f760,0xdd42f760,0xd847f762,0xdd43f762,0xd847f764,0xdd44f764,0xd847f766,
	0xdd45f766,0xd847f768,0xdd46f768,0xd847f76a,0xdd47f76a,0xd847f76c,0xdd48f76c,0xd847f76e,
	0xdd49f76e,0xd847f770,0xdd4af770,0xd847f772,0xdd4bf772,0xd847f774,0xdd4cf774,0xd847f776,
	0xdd4df776,0xd847f778,0xdd4ef778,0xd847f77a,0xdd4ff77a,0xd847f77c,0xdd50f77c,0xd847f77e,
	0xdd51f77e,0xd847f780,0xdd52f780,0xd847f782,0xdd53f782,0xd847f784,0xdd54f784,0xd847f786,
	0xdd55f786,0xd847f788,0xdd56f788,0xd847f78a,0xdd57f78a,0xd847f78c,0xdd58f78c,0xd847f78e,
	0xdd59f78e,0xd847f790,0xdd5af790,0xd847f792,0xdd5bf792,0xd847f794,0xdd5cf794,0xd847f796,
	0xdd5df796,0xd847f798,0xdd5ef798,0xd847f79a,0xdd5ff79a,0xd847f79c,0xdd60f79c,0xd847f79e,
	0xdd61f79e,0xd847f7a0,0xdd62f7a0,0xd847f7a2,0xdd63f7a2,0xd847f7a4,0xdd64f7a4,0xd847f7a6,
	0xdd65f7a6,0xd847f7a8,0xdd66f7a8,0xd847f7aa,0xdd67f7aa,0xd847f7ac,0xdd68f7ac,0xd847f7ae,
	0xdd69f7ae,0xd847f7b0,0xdd6af7b0,0xd847f7b2,0xdd6bf7b2,0xd847f7b4,0xdd6cf7b4,0xd847f7b6,
	0xdd6df7b6,0xd847f7b8,0xdd6ef7b8,0xd847f7ba,0xdd6ff7ba,0xd847f7bc,0xdd70f7bc,0xd847f7be,
	0xdd71f7be,0xd847f7c0,0xdd72f7c0,0xd847f7c2,0xdd73f7c2,0xd847f7c4,0xdd74f7c4,0xd847f7c6,
	0xdd75f7c6,0xd847f7c8,0xdd76f7c8,0xd847f7ca,0xdd77f7ca,0xd847f7cc,0xdd78f7cc,0xd847f7ce,
	0xdd79f7ce,0xd847f7d0,0xdd7af7d0,0xd847f7d2,0xdd7bf7d2,0xd847f7d4,0xdd7cf7d4,0xd847f7d6,
	0xdd7df7d6,0xd847f7d8,0xdd7ef7d8,0xd847f7da,0xdd7ff7da,0xd847f7dc,0xdd80f7dc,0xd847f7de,
	0xdd81f7de,0xd847f7e0,0xdd82f7e0,0xd847f7e2,0xdd83f7e2,0xd847f7e4,0xdd84f7e4,0xd847f7e6,
	0xdd85f7e6,0xd847f7e8,0xdd86f7e8,0xd847f7ea,0xdd87f7ea,0xd847f7ec,0xdd88f7ec,0xd847f7ee,
	0xdd89f7ee,0xd847f7f0,0xdd8af7f0,0xd847f7f2,0xdd8bf7f2,0xd847f7f4,0xdd8cf7f4,0xd847f7f6,
	0xdd8df7f6,0xd847f7f8,0xdd8ef7f8,0xd847f7fa,0xdd8ff7fa,0xd847f7fc,0xdd90f7fc,0xd847f7fe,
	0xdd91f7fe,0xd847f800,0xdd92f800,0xd847f802,0xdd93f802,0xd847f804,0xdd94f804,0xd847f806,
	0xdd95f806,0xd847f808,0xdd96f808,0xd847f80a,0xdd97f80a,0xd847f80c,0xdd98f80c,0xd847f80e,
	0xdd99f80e,0xd847f810,0xdd9af810,0xd847f812,0xdd9bf812,0xd847f814,0xdd9cf814,0xd847f816,
	0xdd9df816,0xd847f818,0xdd9ef818,0xd847f81a,0xdd9ff81a,0xd847f81c,0xdda0f81c,0xd847f81e,
	0xdda1f81e,0xd847f820,0xdda2f820,0xd847f822,0xdda3f822,0xd847f824,0xdda4f824,0xd847f826,
	0xdda5f826,0xd847f828,0xdda6f828,0xd847f82a,0xdda7f82a,0xd847f82c,0xdda8f82c,0xd847f82e,
	0xdda9f82e,0xd847f830,0xddaaf830,0xd847f832,0xddabf832,0xd847f834,0xddacf834,0xd847f836,
	0xddadf836,0xd847f838,0xddaef838,0xd847f83a,0xddaff83a,0xd847f83c,0xddb0f83c,0xd847f83e,
	0xddb1f83e,0xd847f840,0xddb2f840,0xd847f842,0xddb3f842,0xd847f844,0xddb4f844,0xd847f846,
	0xddb5f846,0xd847f848,0xddb6f848,0xd847f84a,0xddb7f84a,0xd847f84c,0xddb8f84c,0xd847f84e,
	0xddb9f84e,0xd847f850,0xddbaf850,0xd847f852,0xddbbf852,0xd847f854,0xddbcf854,0xd847f856,
	0xddbdf856,0xd847f858,0xddbef858,0xd847f85a,0xddbff85a,0xd847f85c,0xddc0f85c,0xd847f85e,
	0xddc1f85e,0xd847f860,0xddc2f860,0xd847f862,0xddc3f862,0xd847f864,0xddc4f864,0xd847f866,
	0xddc5f866,0xd847f868,0xddc6f868,0xd847f86a,0xddc7f86a,0xd847f86c,0xddc8f86c,0xd847f86e,
	0xddc9f86e,0xd847f870,0xddcaf870,0xd847f872,0xddcbf872,0xd847f874,0xddccf874,0xd847f876,
	0xddcdf876,0xd847f878,0xddcef878,0xd847f87a,0xddcff87a,0xd847f87c,0xddd0f87c,0xd847f87e,
	0xddd1f87e,0xd847f880,0xddd2f880,0xd847f882,0xddd3f882,0xd847f884,0xddd4f884,0xd847f886,
	0xddd5f886,0xd847f888,0xddd6f888,0xd847f88a,0xddd7f88a,0xd847f88c,0xddd8f88c,0xd847f88e,
	0xddd9f88e,0xd847f890,0xdddaf890,0xd847f892,0xdddbf892,0xd847f894,0xdddcf894,0xd847f896,
	0xddddf896,0xd847f898,0xdddef898,0xd847f89a,0xdddff89a,0xd847f89c,0xdde0f89c,0xd847f89e,
	0xdde1f89e,0xd847f8a0,0xdde2f8a0,0xd847f8a2,0xdde3f8a2,0xd847f8a4,0xdde4f8a4,0xd847f8a6,
	0xdde5f8a6,0xd847f8a8,0xdde6f8a8,0xd847f8aa,0xdde7f8aa,0xd847f8ac,0xdde8f8ac,0xd847f8ae,
	0xdde9f8ae,0xd847f8b0,0xddeaf8b0,0xd847f8b2,0xddebf8b2,0xd847f8b4,0xddecf8b4,0xd847f8b6,
	0xddedf8b6,0xd847f8b8,0xddeef8b8,0xd847f8ba,0xddeff8ba,0xd847f8bc,0xddf0f8bc,0xd847f8be,
	0xddf1f8be,0xd847f8c0,0xddf2f8c0,0xd847f8c2,0xddf3f8c2,0xd847f8c4,0xddf4f8c4,0xd847f8c6,
	0xddf5f8c6,0xd847f8c8,0xddf6f8c8,0xd847f8ca,0xddf7f8ca,0xd847f8cc,0xddf8f8cc,0xd847f8ce,
	0xddf9f8ce,0xd847f8d0,0xddfaf8d0,0xd847f8d2,0xddfbf8d2,0xd847f8d4,0xddfcf8d4,0xd847f8d6,
	0xddfdf8d6,0xd847f8d8,0xddfef8d8,0xd847f8da,0xddfff8da,0xd847f8dc,0xde00f8dc,0xd847f8de,
	0xde01f8de,0xd847f8e0,0xde02f8e0,0xd847f8e2,0xde03f8e2,0xd847f8e4,0xde04f8e4,0xd847f8e6,
	0xde05f8e6,0xd847f8e8,0xde06f8e8,0xd847f8ea,0xde07f8ea,0xd847f8ec,0xde08f8ec,0xd847f8ee,
	0xde09f8ee,0xd847f8f0,0xde0af8f0,0xd847f8f2,0xde0bf8f2,0xd847f8f4,0xde0cf8f4,0xd847f8f6,
	0xde0df8f6,0xd847f8f8,0xde0ef8f8,0xd847f8fa,0xde0ff8fa,0xd847f8fc,0xde10f8fc,0xd847f8fe,
	0xde11f8fe,0xd847f900,0xde12f900,0xd847f902,0xde13f902,0xd847f904,0xde14f904,0xd847f906,
	0xde15f906,0xd847f908,0xde16f908,0xd847f90a,0xde17f90a,0xd847f90c,0xde18f90c,0xd847f90e,
	0xde19f90e,0xd847f910,0xde1af910,0xd847f912,0xde1bf912,0xd847f914,0xde1cf914,0xd847f916,
	0xde1df916,0xd847f918,0xde1ef918,0xd847f91a,0xde1ff91a,0xd847f91c,0xde20f91c,0xd847f91e,
	0xde21f91e,0xd847f920,0xde22f920,0xd847f922,0xde23f922,0xd847f924,0xde24f924,0xd847f926,
	0xde25f926,0xd847f928,0xde26f928,0xd847f92a,0xde27f92a,0xd847f92c,0xde28f92c,0xd847f92e,
	0xde29f92e,0xd847f930,0xde2af930,0xd847f932,0xde2bf932,0xd847f934,0xde2cf934,0xd847f936,
	0xde2df936,0xd847f938,0xde2ef938,0xd847f93a,0xde2ff93a,0xd847f93c,0xde30f93c,0xd847f93e,
	0xde31f93e,0xd847f940,0xde32f940,0xd847f942,0xde33f942,0xd847f944,0xde34f944,0xd847f946,
	0xde35f946,0xd847f948,0xde36f948,0xd847f94a,0xde37f94a,0xd847f94c,0xde38f94c,0xd847f94e,
	0xde39f94e,0xd847f950,0xde3af950,0xd847f952,0xde3bf952,0xd847f954,0xde3cf954,0xd847f956,
	0xde3df956,0xd847f958,0xde3ef958,0xd847f95a,0xde3ff95a,0xd847f95c,0xde40f95c,0xd847f95e,
	0xde41f95e,0xd847f960,0xde42f960,0xd847f962,0xde43f962,0xd847f964,0xde44f964,0xd847f966,
	0xde45f966,0xd847f968,0xde46f968,0xd847f96a,0xde47f96a,0xd847f96c,0xde48f96c,0xd847f96e,
	0xde49f96e,0xd847f970,0xde4af970,0xd847f972,0xde4bf972,0xd847f974,0xde4cf974,0xd847f976,
	0xde4df976,0xd847f978,0xde4ef978,0xd847f97a,0xde4ff97a,0xd847f97c,0xde50f97c,0xd847f97e,
	0xde51f97e,0xd847f980,0xde52f980,0xd847f982,0xde53f982,0xd847f984,0xde54f984,0xd847f986,
	0xde55f986,0xd847f988,0xde56f988,0xd847f98a,0xde57f98a,0xd847f98c,0xde58f98c,0xd847f98e,
	0xde59f98e,0xd847f990,0xde5af990,0xd847f992,0xde5bf992,0xd847f994,0xde5cf994,0xd847f996,
	0xde5df996,0xd847f998,0xde5ef998,0xd847f99a,0xde5ff99a,0xd847f99c,0xde60f99c,0xd847f99e,
	0xde61f99e,0xd847f9a0,0xde62f9a0,0xd847f9a2,0xde63f9a2,0xd847f9a4,0xde64f9a4,0xd847f9a6,
	0xde65f9a6,0xd847f9a8,0xde66f9a8,0xd847f9aa,0xde67f9aa,0xd847f9ac,0xde68f9ac,0xd847f9ae,
	0xde69f9ae,0xd847f9b0,0xde6af9b0,0xd847f9b2,0xde6bf9b2,0xd847f9b4,0xde6cf9b4,0xd847f9b6,
	0xde6df9b6,0xd847f9b8,0xde6ef9b8,0xd847f9ba,0xde6ff9ba,0xd847f9bc,0xde70f9bc,0xd847f9be,
	0xde71f9be,0xd847f9c0,0xde72f9c0,0xd847f9c2,0xde73f9c2,0xd847f9c4,0xde74f9c4,0xd847f9c6,
	0xde75f9c6,0xd847f9c8,0xde76f9c8,0xd847f9ca,0xde77f9ca,0xd847f9cc,0xde78f9cc,0xd847f9ce,
	0xde79f9ce,0xd847f9d0,0xde7af9d0,0xd847f9d2,0xde7bf9d2,0xd847f9d4,0xde7cf9d4,0xd847f9d6,
	0xde7df9d6,0xd847f9d8,0xde7ef9d8,0xd847f9da,0xde7ff9da,0xd847f9dc,0xde80f9dc,0xd847f9de,
	0xde81f9de,0xd847f9e0,0xde82f9e0,0xd847f9e2,0xde83f9e2,0xd847f9e4,0xde84f9e4,0xd847f9e6,
	0xde85f9e6,0xd847f9e8,0xde86f9e8,0xd847f9ea,0xde87f9ea,0xd847f9ec,0xde88f9ec,0xd847f9ee,
	0xde89f9ee,0xd847f9f0,0xde8af9f0,0xd847f9f2,0xde8bf9f2,0xd847f9f4,0xde8cf9f4,0xd847f9f6,
	0xde8df9f6,0xd847f9f8,0xde8ef9f8,0xd847f9fa,0xde8ff9fa,0xd847f9fc,0xde90f9fc,0xd847f9fe,
	0xde91f9fe,0xd847fa00,0xde92fa00,0xd847fa02,0xde93fa02,0xd847fa04,0xde94fa04,0xd847fa06,
	0xde95fa06,0xd847fa08,0xde96fa08,0xd847fa0a,0xde97fa0a,0xd847fa0c,0xde98fa0c,0xd847fa0e,
	0xde99fa0e,0xd847fa10,0xde9afa10,0xd847fa12,0xde9bfa12,0xd847fa14,0xde9cfa14,0xd847fa16,
	0xde9dfa16,0xd847fa18,0xde9efa18,0xd847fa1a,0xde9ffa1a,0xd847fa1c,0xdea0fa1c,0xd847fa1e,
	0xdea1fa1e,0xd847fa20,0xdea2fa20,0xd847fa22,0xdea3fa22,0xd847fa24,0xdea4fa24,0xd847fa26,
	0xdea5fa26,0xd847fa28,0xdea6fa28,0xd847fa2a,0xdea7fa2a,0xd847fa2c,0xdea8fa2c,0xd847fa2e,
	0xdea9fa2e,0xd847fa30,0xdeaafa30,0xd847fa32,0xdeabfa32,0xd847fa34,0xdeacfa34,0xd847fa36,
	0xdeadfa36,0xd847fa38,0xdeaefa38,0xd847fa3a,0xdeaffa3a,0xd847fa3c,0xdeb0fa3c,0xd847fa3e,
	0xdeb1fa3e,0xd847fa40,0xdeb2fa40,0xd847fa42,0xdeb3fa42,0xd847fa44,0xdeb4fa44,0xd847fa46,
	0xdeb5fa46,0xd847fa48,0xdeb6fa48,0xd847fa4a,0xdeb7fa4a,0xd847fa4c,0xdeb8fa4c,0xd847fa4e,
	0xdeb9fa4e,0xd847fa50,0xdebafa50,0xd847fa52,0xdebbfa52,0xd847fa54,0xdebcfa54,0xd847fa56,
	0xdebdfa56,0xd847fa58,0xdebefa58,0xd847fa5a,0xdebffa5a,0xd847fa5c,0xdec0fa5c,0xd847fa5e,
	0xdec1fa5e,0xd847fa60,0xdec2fa60,0xd847fa62,0xdec3fa62,0xd847fa64,0xdec4fa64,0xd847fa66,
	0xdec5fa66,0xd847fa68,0xdec6fa68,0xd847fa6a,0xdec7fa6a,0xd847fa6c,0xdec8fa6c,0xd847fa6e,
	0xdec9fa6e,0xd847fa70,0xdecafa70,0xd847fa72,0xdecbfa72,0xd847fa74,0xdeccfa74,0xd847fa76,
	0xdecdfa76,0xd847fa78,0xdecefa78,0xd847fa7a,0xdecffa7a,0xd847fa7c,0xded0fa7c,0xd847fa7e,
	0xded1fa7e,0xd847fa80,0xded2fa80,0xd847fa82,0xded3fa82,0xd847fa84,0xded4fa84,0xd847fa86,
	0xded5fa86,0xd847fa88,0xded6fa88,0xd847fa8a,0xded7fa8a,0xd847fa8c,0xded8fa8c,0xd847fa8e,
	0xded9fa8e,0xd847fa90,0xdedafa90,0xd847fa92,0xdedbfa92,0xd847fa94,0xdedcfa94,0xd847fa96,
	0xdeddfa96,0xd847fa98,0xdedefa98,0xd847fa9a,0xdedffa9a,0xd847fa9c,0xdee0fa9c,0xd847fa9e,
	0xdee1fa9e,0xd847faa0,0xdee2faa0,0xd847faa2,0xdee3faa2,0xd847faa4,0xdee4faa4,0xd847faa6,
	0xdee5faa6,0xd847faa8,0xdee6faa8,0xd847faaa,0xdee7faaa,0xd847faac,0xdee8faac,0xd847faae,
	0xdee9faae,0xd847fab0,0xdeeafab0,0xd847fab2,0xdeebfab2,0xd847fab4,0xdeecfab4,0xd847fab6,
	0xdeedfab6,0xd847fab8,0xdeeefab8,0xd847faba,0xdeeffaba,0xd847fabc,0xdef0fabc,0xd847fabe,
	0xdef1fabe,0xd847fac0,0xdef2fac0,0xd847fac2,0xdef3fac2,0xd847fac4,0xdef4fac4,0xd847fac6,
	0xdef5fac6,0xd847fac8,0xdef6fac8,0xd847faca,0xdef7faca,0xd847facc,0xdef8facc,0xd847face,
	0xdef9face,0xd847fad0,0xdefafad0,0xd847fad2,0xdefbfad2,0xd847fad4,0xdefcfad4,0xd847fad6,
	0xdefdfad6,0xd847fad8,0xdefefad8,0xd847fada,0xdefffada,0xd847fadc,0xdf00fadc,0xd847fade,
	0xdf01fade,0xd847fae0,0xdf02fae0,0xd847fae2,0xdf03fae2,0xd847fae4,0xdf04fae4,0xd847fae6,
	0xdf05fae6,0xd847fae8,0xdf06fae8,0xd847faea,0xdf07faea,0xd847faec,0xdf08faec,0xd847faee,
	0xdf09faee,0xd847faf0,0xdf0afaf0,0xd847faf2,0xdf0bfaf2,0xd847faf4,0xdf0cfaf4,0xd847faf6,
	0xdf0dfaf6,0xd847faf8,0xdf0efaf8,0xd847fafa,0xdf0ffafa,0xd847fafc,0xdf10fafc,0xd847fafe,
	0xdf11fafe,0xd847fb00,0xdf12fb00,0xd847fb02,0xdf13fb02,0xd847fb04,0xdf14fb04,0xd847fb06,
	0xdf15fb06,0xd847fb08,0xdf16fb08,0xd847fb0a,0xdf17fb0a,0xd847fb0c,0xdf18fb0c,0xd847fb0e,
	0xdf19fb0e,0xd847fb10,0xdf1afb10,0xd847fb12,0xdf1bfb12,0xd847fb14,0xdf1cfb14,0xd847fb16,
	0xdf1dfb16,0xd847fb18,0xdf1efb18,0xd847fb1a,0xdf1ffb1a,0xd847fb1c,0xdf20fb1c,0xd847fb1e,
	0xdf21fb1e,0xd847fb20,0xdf22fb20,0xd847fb22,0xdf23fb22,0xd847fb24,0xdf24fb24,0xd847fb26,
	0xdf25fb26,0xd847fb28,0xdf26fb28,0xd847fb2a,0xdf27fb2a,0xd847fb2c,0xdf28fb2c,0xd847fb2e,
	0xdf29fb2e,0xd847fb30,0xdf2afb30,0xd847fb32,0xdf2bfb32,0xd847fb34,0xdf2cfb34,0xd847fb36,
	0xdf2dfb36,0xd847fb38,0xdf2efb38,0xd847fb3a,0xdf2ffb3a,0xd847fb3c,0xdf30fb3c,0xd847fb3e,
	0xdf31fb3e,0xd847fb40,0xdf32fb40,0xd847fb42,0xdf33fb42,0xd847fb44,0xdf34fb44,0xd847fb46,
	0xdf35fb46,0xd847fb48,0xdf36fb48,0xd847fb4a,0xdf37fb4a,0xd847fb4c,0xdf38fb4c,0xd847fb4e,
	0xdf39fb4e,0xd847fb50,0xdf3afb50,0xd847fb52,0xdf3bfb52,0xd847fb54,0xdf3cfb54,0xd847fb56,
	0xdf3dfb56,0xd847fb58,0xdf3efb58,0xd847fb5a,0xdf3ffb5a,0xd847fb5c,0xdf40fb5c,0xd847fb5e,
	0xdf41fb5e,0xd847fb60,0xdf42fb60,0xd847fb62,0xdf43fb62,0xd847fb64,0xdf44fb64,0xd847fb66,
	0xdf45fb66,0xd847fb68,0xdf46fb68,0xd847fb6a,0xdf47fb6a,0xd847fb6c,0xdf48fb6c,0xd847fb6e,
	0xdf49fb6e,0xd847fb70,0xdf4afb70,0xd847fb72,0xdf4bfb72,0xd847fb74,0xdf4cfb74,0xd847fb76,
	0xdf4dfb76,0xd847fb78,0xdf4efb78,0xd847fb7a,0xdf4ffb7a,0xd847fb7c,0xdf50fb7c,0xd847fb7e,
	0xdf51fb7e,0xd847fb80,0xdf52fb80,0xd847fb82,0xdf53fb82,0xd847fb84,0xdf54fb84,0xd847fb86,
	0xdf55fb86,0xd847fb88,0xdf56fb88,0xd847fb8a,0xdf57fb8a,0xd847fb8c,0xdf58fb8c,0xd847fb8e,
	0xdf59fb8e,0xd847fb90,0xdf5afb90,0xd847fb92,0xdf5bfb92,0xd847fb94,0xdf5cfb94,0xd847fb96,
	0xdf5dfb96,0xd847fb98,0xdf5efb98,0xd847fb9a,0xdf5ffb9a,0xd847fb9c,0xdf60fb9c,0xd847fb9e,
	0xdf61fb9e,0xd847fba0,0xdf62fba0,0xd847fba2,0xdf63fba2,0xd847fba4,0xdf64fba4,0xd847fba6,
	0xdf65fba6,0xd847fba8,0xdf66fba8,0xd847fbaa,0xdf67fbaa,0xd847fbac,0xdf68fbac,0xd847fbae,
	0xdf69fbae,0xd847fbb0,0xdf6afbb0,0xd847fbb2,0xdf6bfbb2,0xd847fbb4,0xdf6cfbb4,0xd847fbb6,
	0xdf6dfbb6,0xd847fbb8,0xdf6efbb8,0xd847fbba,0xdf6ffbba,0xd847fbbc,0xdf70fbbc,0xd847fbbe,
	0xdf71fbbe,0xd847fbc0,0xdf72fbc0,0xd847fbc2,0xdf73fbc2,0xd847fbc4,0xdf74fbc4,0xd847fbc6,
	0xdf75fbc6,0xd847fbc8,0xdf76fbc8,0xd847fbca,0xdf77fbca,0xd847fbcc,0xdf78fbcc,0xd847fbce,
	0xdf79fbce,0xd847fbd0,0xdf7afbd0,0xd847fbd2,0xdf7bfbd2,0xd847fbd4,0xdf7cfbd4,0xd847fbd6,
	0xdf7dfbd6,0xd847fbd8,0xdf7efbd8,0xd847fbda,0xdf7ffbda,0xd847fbdc,0xdf80fbdc,0xd847fbde,
	0xdf81fbde,0xd847fbe0,0xdf82fbe0,0xd847fbe2,0xdf83fbe2,0xd847fbe4,0xdf84fbe4,0xd847fbe6,
	0xdf85fbe6,0xd847fbe8,0xdf86fbe8,0xd847fbea,0xdf87fbea,0xd847fbec,0xdf88fbec,0xd847fbee,
	0xdf89fbee,0xd847fbf0,0xdf8afbf0,0xd847fbf2,0xdf8bfbf2,0xd847fbf4,0xdf8cfbf4,0xd847fbf6,
	0xdf8dfbf6,0xd847fbf8,0xdf8efbf8,0xd847fbfa,0xdf8ffbfa,0xd847fbfc,0xdf90fbfc,0xd847fbfe,
	0xdf91fbfe,0xd847fc00,0xdf92fc00,0xd847fc02,0xdf93fc02,0xd847fc04,0xdf94fc04,0xd847fc06,
	0xdf95fc06,0xd847fc08,0xdf96fc08,0xd847fc0a,0xdf97fc0a,0xd847fc0c,0xdf98fc0c,0xd847fc0e,
	0xdf99fc0e,0xd847fc10,0xdf9afc10,0xd847fc12,0xdf9bfc12,0xd847fc14,0xdf9cfc14,0xd847fc16,
	0xdf9dfc16,0xd847fc18,0xdf9efc18,0xd847fc1a,0xdf9ffc1a,0xd847fc1c,0xdfa0fc1c,0xd847fc1e,
	0xdfa1fc1e,0xd847fc20,0xdfa2fc20,0xd847fc22,0xdfa3fc22,0xd847fc24,0xdfa4fc24,0xd847fc26,
	0xdfa5fc26,0xd847fc28,0xdfa6fc28,0xd847fc2a,0xdfa7fc2a,0xd847fc2c,0xdfa8fc2c,0xd847fc2e,
	0xdfa9fc2e,0xd847fc30,0xdfaafc30,0xd847fc32,0xdfabfc32,0xd847fc34,0xdfacfc34,0xd847fc36,
	0xdfadfc36,0xd847fc38,0xdfaefc38,0xd847fc3a,0xdfaffc3a,0xd847fc3c,0xdfb0fc3c,0xd847fc3e,
	0xdfb1fc3e,0xd847fc40,0xdfb2fc40,0xd847fc42,0xdfb3fc42,0xd847fc44,0xdfb4fc44,0xd847fc46,
	0xdfb5fc46,0xd847fc48,0xdfb6fc48,0xd847fc4a,0xdfb7fc4a,0xd847fc4c,0xdfb8fc4c,0xd847fc4e,
	0xdfb9fc4e,0xd847fc50,0xdfbafc50,0xd847fc52,0xdfbbfc52,0xd847fc54,0xdfbcfc54,0xd847fc56,
	0xdfbdfc56,0xd847fc58,0xdfbefc58,0xd847fc5a,0xdfbffc5a,0xd847fc5c,0xdfc0fc5c,0xd847fc5e,
	0xdfc1fc5e,0xd847fc60,0xdfc2fc60,0xd847fc62,0xdfc3fc62,0xd847fc64,0xdfc4fc64,0xd847fc66,
	0xdfc5fc66,0xd847fc68,0xdfc6fc68,0xd847fc6a,0xdfc7fc6a,0xd847fc6c,0xdfc8fc6c,0xd847fc6e,
	0xdfc9fc6e,0xd847fc70,0xdfcafc70,0xd847fc72,0xdfcbfc72,0xd847fc74,0xdfccfc74,0xd847fc76,
	0xdfcdfc76,0xd847fc78,0xdfcefc78,0xd847fc7a,0xdfcffc7a,0xd847fc7c,0xdfd0fc7c,0xd847fc7e,
	0xdfd1fc7e,0xd847fc80,0xdfd2fc80,0xd847fc82,0xdfd3fc82,0xd847fc84,0xdfd4fc84,0xd847fc86,
	0xdfd5fc86,0xd847fc88,0xdfd6fc88,0xd847fc8a,0xdfd7fc8a,0xd847fc8c,0xdfd8fc8c,0xd847fc8e,
	0xdfd9fc8e,0xd847fc90,0xdfdafc90,0xd847fc92,0xdfdbfc92,0xd847fc94,0xdfdcfc94,0xd847fc96,
	0xdfddfc96,0xd847fc98,0xdfdefc98,0xd847fc9a,0xdfdffc9a,0xd847fc9c,0xdfe0fc9c,0xd847fc9e,
	0xdfe1fc9e,0xd847fca0,0xdfe2fca0,0xd847fca2,0xdfe3fca2,0xd847fca4,0xdfe4fca4,0xd847fca6,
	0xdfe5fca6,0xd847fca8,0xdfe6fca8,0xd847fcaa,0xdfe7fcaa,0xd847fcac,0xdfe8fcac,0xd847fcae,
	0xdfe9fcae,0xd847fcb0,0xdfeafcb0,0xd847fcb2,0xdfebfcb2,0xd847fcb4,0xdfecfcb4,0xd847fcb6,
	0xdfedfcb6,0xd847fcb8,0xdfeefcb8,0xd847fcba,0xdfeffcba,0xd847fcbc,0xdff0fcbc,0xd847fcbe,
	0xdff1fcbe,0xd847fcc0,0xdff2fcc0,0xd847fcc2,0xdff3fcc2,0xd847fcc4,0xdff4fcc4,0xd847fcc6,
	0xdff5fcc6,0xd847fcc8,0xdff6fcc8,0xd847fcca,0xdff7fcca,0xd847fccc,0xdff8fccc,0xd847fcce,
	0xdff9fcce,0xd847fcd0,0xdffafcd0,0xd847fcd2,0xdffbfcd2,0xd847fcd4,0xdffcfcd4,0xd847fcd6,
	0xdffdfcd6,0xd847fcd8,0xdffefcd8,0xd847fcda,0xdffffcda,0xd848fcdc,0xdc00fcdc,0xd848fcde,
	0xdc01fcde,0xd848fce0,0xdc02fce0,0xd848fce2,0xdc03fce2,0xd848fce4,0xdc04fce4,0xd848fce6,
	0xdc05fce6,0xd848fce8,0xdc06fce8,0xd848fcea,0xdc07fcea,0xd848fcec,0xdc08fcec,0xd848fcee,
	0xdc09fcee,0xd848fcf0,0xdc0afcf0,0xd848fcf2,0xdc0bfcf2,0xd848fcf4,0xdc0cfcf4,0xd848fcf6,
	0xdc0dfcf6,0xd848fcf8,0xdc0efcf8,0xd848fcfa,0xdc0ffcfa,0xd848fcfc,0xdc10fcfc,0xd848fcfe,
	0xdc11fcfe,0xd848fd00,0xdc12fd00,0xd848fd02,0xdc13fd02,0xd848fd04,0xdc14fd04,0xd848fd06,
	0xdc15fd06,0xd848fd08,0xdc16fd08,0xd848fd0a,0xdc17fd0a,0xd848fd0c,0xdc18fd0c,0xd848fd0e,
	0xdc19fd0e,0xd848fd10,0xdc1afd10,0xd848fd12,0xdc1bfd12,0xd848fd14,0xdc1cfd14,0xd848fd16,
	0xdc1dfd16,0xd848fd18,0xdc1efd18,0xd848fd1a,0xdc1ffd1a,0xd848fd1c,0xdc20fd1c,0xd848fd1e,
	0xdc21fd1e,0xd848fd20,0xdc22fd20,0xd848fd22,0xdc23fd22,0xd848fd24,0xdc24fd24,0xd848fd26,
	0xdc25fd26,0xd848fd28,0xdc26fd28,0xd848fd2a,0xdc27fd2a,0xd848fd2c,0xdc28fd2c,0xd848fd2e,
	0xdc29fd2e,0xd848fd30,0xdc2afd30,0xd848fd32,0xdc2bfd32,0xd848fd34,0xdc2cfd34,0xd848fd36,
	0xdc2dfd36,0xd848fd38,0xdc2efd38,0xd848fd3a,0xdc2ffd3a,0xd848fd3c,0xdc30fd3c,0xd848fd3e,
	0xdc31fd3e,0xd848fd40,0xdc32fd40,0xd848fd42,0xdc33fd42,0xd848fd44,0xdc34fd44,0xd848fd46,
	0xdc35fd46,0xd848fd48,0xdc36fd48,0xd848fd4a,0xdc37fd4a,0xd848fd4c,0xdc38fd4c,0xd848fd4e,
	0xdc39fd4e,0xd848fd50,0xdc3afd50,0xd848fd52,0xdc3bfd52,0xd848fd54,0xdc3cfd54,0xd848fd56,
	0xdc3dfd56,0xd848fd58,0xdc3efd58,0xd848fd5a,0xdc3ffd5a,0xd848fd5c,0xdc40fd5c,0xd848fd5e,
	0xdc41fd5e,0xd848fd60,0xdc42fd60,0xd848fd62,0xdc43fd62,0xd848fd64,0xdc44fd64,0xd848fd66,
	0xdc45fd66,0xd848fd68,0xdc46fd68,0xd848fd6a,0xdc47fd6a,0xd848fd6c,0xdc48fd6c,0xd848fd6e,
	0xdc49fd6e,0xd848fd70,0xdc4afd70,0xd848fd72,0xdc4bfd72,0xd848fd74,0xdc4cfd74,0xd848fd76,
	0xdc4dfd76,0xd848fd78,0xdc4efd78,0xd848fd7a,0xdc4ffd7a,0xd848fd7c,0xdc50fd7c,0xd848fd7e,
	0xdc51fd7e,0xd848fd80,0xdc52fd80,0xd848fd82,0xdc53fd82,0xd848fd84,0xdc54fd84,0xd848fd86,
	0xdc55fd86,0xd848fd88,0xdc56fd88,0xd848fd8a,0xdc57fd8a,0xd848fd8c,0xdc58fd8c,0xd848fd8e,
	0xdc59fd8e,0xd848fd90,0xdc5afd90,0xd848fd92,0xdc5bfd92,0xd848fd94,0xdc5cfd94,0xd848fd96,
	0xdc5dfd96,0xd848fd98,0xdc5efd98,0xd848fd9a,0xdc5ffd9a,0xd848fd9c,0xdc60fd9c,0xd848fd9e,
	0xdc61fd9e,0xd848fda0,0xdc62fda0,0xd848fda2,0xdc63fda2,0xd848fda4,0xdc64fda4,0xd848fda6,
	0xdc65fda6,0xd848fda8,0xdc66fda8,0xd848fdaa,0xdc67fdaa,0xd848fdac,0xdc68fdac,0xd848fdae,
	0xdc69fdae,0xd848fdb0,0xdc6afdb0,0xd848fdb2,0xdc6bfdb2,0xd848fdb4,0xdc6cfdb4,0xd848fdb6,
	0xdc6dfdb6,0xd848fdb8,0xdc6efdb8,0xd848fdba,0xdc6ffdba,0xd848fdbc,0xdc70fdbc,0xd848fdbe,
	0xdc71fdbe,0xd848fdc0,0xdc72fdc0,0xd848fdc2,0xdc73fdc2,0xd848fdc4,0xdc74fdc4,0xd848fdc6,
	0xdc75fdc6,0xd848fdc8,0xdc76fdc8,0xd848fdca,0xdc77fdca,0xd848fdcc,0xdc78fdcc,0xd848fdce,
	0xdc79fdce,0xd848fdd0,0xdc7afdd0,0xd848fdd2,0xdc7bfdd2,0xd848fdd4,0xdc7cfdd4,0xd848fdd6,
	0xdc7dfdd6,0xd848fdd8,0xdc7efdd8,0xd848fdda,0xdc7ffdda,0xd848fddc,0xdc80fddc,0xd848fdde,
	0xdc81fdde,0xd848fde0,0xdc82fde0,0xd848fde2,0xdc83fde2,0xd848fde4,0xdc84fde4,0xd848fde6,
	0xdc85fde6,0xd848fde8,0xdc86fde8,0xd848fdea,0xdc87fdea,0xd848fdec,0xdc88fdec,0xd848fdee,
	0xdc89fdee,0xd848fdf0,0xdc8afdf0,0xd848fdf2,0xdc8bfdf2,0xd848fdf4,0xdc8cfdf4,0xd848fdf6,
	0xdc8dfdf6,0xd848fdf8,0xdc8efdf8,0xd848fdfa,0xdc8ffdfa,0xd848fdfc,0xdc90fdfc,0xd848fdfe,
	0xdc91fdfe,0xd848fe00,0xdc92fe00,0xd848fe02,0xdc93fe02,0xd848fe04,0xdc94fe04,0xd848fe06,
	0xdc95fe06,0xd848fe08,0xdc96fe08,0xd848fe0a,0xdc97fe0a,0xd848fe0c,0xdc98fe0c,0xd848fe0e,
	0xdc99fe0e,0xd848fe10,0xdc9afe10,0xd848fe12,0xdc9bfe12,0xd848fe14,0xdc9cfe14,0xd848fe16,
	0xdc9dfe16,0xd848fe18,0xdc9efe18,0xd848fe1a,0xdc9ffe1a,0xd848fe1c,0xdca0fe1c,0xd848fe1e,
	0xdca1fe1e,0xd848fe20,0xdca2fe20,0xd848fe22,0xdca3fe22,0xd848fe24,0xdca4fe24,0xd848fe26,
	0xdca5fe26,0xd848fe28,0xdca6fe28,0xd848fe2a,0xdca7fe2a,0xd848fe2c,0xdca8fe2c,0xd848fe2e,
	0xdca9fe2e,0xd848fe30,0xdcaafe30,0xd848fe32,0xdcabfe32,0xd848fe34,0xdcacfe34,0xd848fe36,
	0xdcadfe36,0xd848fe38,0xdcaefe38,0xd848fe3a,0xdcaffe3a,0xd848fe3c,0xdcb0fe3c,0xd848fe3e,
	0xdcb1fe3e,0xd848fe40,0xdcb2fe40,0xd848fe42,0xdcb3fe42,0xd848fe44,0xdcb4fe44,0xd848fe46,
	0xdcb5fe46,0xd848fe48,0xdcb6fe48,0xd848fe4a,0xdcb7fe4a,0xd848fe4c,0xdcb8fe4c,0xd848fe4e,
	0xdcb9fe4e,0xd848fe50,0xdcbafe50,0xd848fe52,0xdcbbfe52,0xd848fe54,0xdcbcfe54,0xd848fe56,
	0xdcbdfe56,0xd848fe58,0xdcbefe58,0xd848fe5a,0xdcbffe5a,0xd848fe5c,0xdcc0fe5c,0xd848fe5e,
	0xdcc1fe5e,0xd848fe60,0xdcc2fe60,0xd848fe62,0xdcc3fe62,0xd848fe64,0xdcc4fe64,0xd848fe66,
	0xdcc5fe66,0xd848fe68,0xdcc6fe68,0xd848fe6a,0xdcc7fe6a,0xd848fe6c,0xdcc8fe6c,0xd848fe6e,
	0xdcc9fe6e,0xd848fe70,0xdccafe70,0xd848fe72,0xdccbfe72,0xd848fe74,0xdcccfe74,0xd848fe76,
	0xdccdfe76,0xd848fe78,0xdccefe78,0xd848fe7a,0xdccffe7a,0xd848fe7c,0xdcd0fe7c,0xd848fe7e,
	0xdcd1fe7e,0xd848fe80,0xdcd2fe80,0xd848fe82,0xdcd3fe82,0xd848fe84,0xdcd4fe84,0xd848fe86,
	0xdcd5fe86,0xd848fe88,0xdcd6fe88,0xd848fe8a,0xdcd7fe8a,0xd848fe8c,0xdcd8fe8c,0xd848fe8e,
	0xdcd9fe8e,0xd848fe90,0xdcdafe90,0xd848fe92,0xdcdbfe92,0xd848fe94,0xdcdcfe94,0xd848fe96,
	0xdcddfe96,0xd848fe98,0xdcdefe98,0xd848fe9a,0xdcdffe9a,0xd848fe9c,0xdce0fe9c,0xd848fe9e,
	0xdce1fe9e,0xd848fea0,0xdce2fea0,0xd848fea2,0xdce3fea2,0xd848fea4,0xdce4fea4,0xd848fea6,
	0xdce5fea6,0xd848fea8,0xdce6fea8,0xd848feaa,0xdce7feaa,0xd848feac,0xdce8feac,0xd848feae,
	0xdce9feae,0xd848feb0,0xdceafeb0,0xd848feb2,0xdcebfeb2,0xd848feb4,0xdcecfeb4,0xd848feb6,
	0xdcedfeb6,0xd848feb8,0xdceefeb8,0xd848feba,0xdceffeba,0xd848febc,0xdcf0febc,0xd848febe,
	0xdcf1febe,0xd848fec0,0xdcf2fec0,0xd848fec2,0xdcf3fec2,0xd848fec4,0xdcf4fec4,0xd848fec6,
	0xdcf5fec6,0xd848fec8,0xdcf6fec8,0xd848feca,0xdcf7feca,0xd848fecc,0xdcf8fecc,0xd848fece,
	0xdcf9fece,0xd848fed0,0xdcfafed0,0xd848fed2,0xdcfbfed2,0xd848fed4,0xdcfcfed4,0xd848fed6,
	0xdcfdfed6,0xd848fed8,0xdcfefed8,0xd848feda,0xdcfffeda,0xd848fedc,0xdd00fedc,0xd848fede,
	0xdd01fede,0xd848fee0,0xdd02fee0,0xd848fee2,0xdd03fee2,0xd848fee4,0xdd04fee4,0xd848fee6,
	0xdd05fee6,0xd848fee8,0xdd06fee8,0xd848feea,0xdd07feea,0xd848feec,0xdd08feec,0xd848feee,
	0xdd09feee,0xd848fef0,0xdd0afef0,0xd848fef2,0xdd0bfef2,0xd848fef4,0xdd0cfef4,0xd848fef6,
	0xdd0dfef6,0xd848fef8,0xdd0efef8,0xd848fefa,0xdd0ffefa,0xd848fefc,0xdd10fefc,0xd848fefe,
	0xdd11fefe,0xd848ff00,0xdd12ff00,0xd848ff02,0xdd13ff02,0xd848ff04,0xdd14ff04,0xd848ff06,
	0xdd15ff06,0xd848ff08,0xdd16ff08,0xd848ff0a,0xdd17ff0a,0xd848ff0c,0xdd18ff0c,0xd848ff0e,
	0xdd19ff0e,0xd848ff10,0xdd1aff10,0xd848ff12,0xdd1bff12,0xd848ff14,0xdd1cff14,0xd848ff16,
	0xdd1dff16,0xd848ff18,0xdd1eff18,0xd848ff1a,0xdd1fff1a,0xd848ff1c,0xdd20ff1c,0xd848ff1e,
	0xdd21ff1e,0xd848ff20,0xdd22ff20,0xd848ff22,0xdd23ff22,0xd848ff24,0xdd24ff24,0xd848ff26,
	0xdd25ff26,0xd848ff28,0xdd26ff28,0xd848ff2a,0xdd27ff2a,0xd848ff2c,0xdd28ff2c,0xd848ff2e,
	0xdd29ff2e,0xd848ff30,0xdd2aff30,0xd848ff32,0xdd2bff32,0xd848ff34,0xdd2cff34,0xd848ff36,
	0xdd2dff36,0xd848ff38,0xdd2eff38,0xd848ff3a,0xdd2fff3a,0xd848ff3c,0xdd30ff3c,0xd848ff3e,
	0xdd31ff3e,0xd848ff40,0xdd32ff40,0xd848ff42,0xdd33ff42,0xd848ff44,0xdd34ff44,0xd848ff46,
	0xdd35ff46,0xd848ff48,0xdd36ff48,0xd848ff4a,0xdd37ff4a,0xd848ff4c,0xdd38ff4c,0xd848ff4e,
	0xdd39ff4e,0xd848ff50,0xdd3aff50,0xd848ff52,0xdd3bff52,0xd848ff54,0xdd3cff54,0xd848ff56,
	0xdd3dff56,0xd848ff58,0xdd3eff58,0xd848ff5a,0xdd3fff5a,0xd848ff5c,0xdd40ff5c,0xd848ff5e,
	0xdd41ff5e,0xd848ff60,0xdd42ff60,0xd848ff62,0xdd43ff62,0xd848ff64,0xdd44ff64,0xd848ff66,
	0xdd45ff66,0xd848ff68,0xdd46ff68,0xd848ff6a,0xdd47ff6a,0xd848ff6c,0xdd48ff6c,0xd848ff6e,
	0xdd49ff6e,0xd848ff70,0xdd4aff70,0xd848ff72,0xdd4bff72,0xd848ff74,0xdd4cff74,0xd848ff76,
	0xdd4dff76,0xd848ff78,0xdd4eff78,0xd848ff7a,0xdd4fff7a,0xd848ff7c,0xdd50ff7c,0xd848ff7e,
	0xdd51ff7e,0xd848ff80,0xdd52ff80,0xd848ff82,0xdd53ff82,0xd848ff84,0xdd54ff84,0xd848ff86,
	0xdd55ff86,0xd848ff88,0xdd56ff88,0xd848ff8a,0xdd57ff8a,0xd848ff8c,0xdd58ff8c,0xd848ff8e,
	0xdd59ff8e,0xd848ff90,0xdd5aff90,0xd848ff92,0xdd5bff92,0xd848ff94,0xdd5cff94,0xd848ff96,
	0xdd5dff96,0xd848ff98,0xdd5eff98,0xd848ff9a,0xdd5fff9a,0xd848ff9c,0xdd60ff9c,0xd848ff9e,
	0xdd61ff9e,0xd848ffa0,0xdd62ffa0,0xd848ffa2,0xdd63ffa2,0xd848ffa4,0xdd64ffa4,0xd848ffa6,
	0xdd65ffa6,0xd848ffa8,0xdd66ffa8,0xd848ffaa,0xdd67ffaa,0xd848ffac,0xdd68ffac,0xd848ffae,
	0xdd69ffae,0xd848ffb0,0xdd6affb0,0xd848ffb2,0xdd6bffb2,0xd848ffb4,0xdd6cffb4,0xd848ffb6,
	0xdd6dffb6,0xd848ffb8,0xdd6effb8,0xd848ffba,0xdd6fffba,0xd848ffbc,0xdd70ffbc,0xd848ffbe,
	0xdd71ffbe,0xd848ffc0,0xdd72ffc0,0xd848ffc2,0xdd73ffc2,0xd848ffc4,0xdd74ffc4,0xd848ffc6,
	0xdd75ffc6,0xd848ffc8,0xdd76ffc8,0xd848ffca,0xdd77ffca,0xd848ffcc,0xdd78ffcc,0xd848ffce,
	0xdd79ffce,0xd848ffd0,0xdd7affd0,0xd848ffd2,0xdd7bffd2,0xd848ffd4,0xdd7cffd4,0xd848ffd6,
	0xdd7dffd6,0xd848ffd8,0xdd7effd8,0xd848ffda,0xdd7fffda,0xd848ffdc,0xdd80ffdc,0xd848ffde,
	0xdd81ffde,0xd848ffe0,0xdd82ffe0,0xd848ffe2,0xdd83ffe2,0xd848ffe4,0xdd84ffe4,0xd848ffe6,
	0xdd85ffe6,0xd848ffe8,0xdd86ffe8,0xd848ffea,0xdd87ffea,0xd848ffec,0xdd88ffec,0xd848ffee,
	0xdd89ffee,0xd848fff0,0xdd8afff0,0xd848fff2,0xdd8bfff2,0xd848fff4,0xdd8cfff4,0xd848fff6,
	0xdd8dfff6,0xd848fff8,0xdd8efff8,0xd848fffa,0xdd8ffffa,0xd848fffc,0xdd90fffc,0xd848fffe,
	0xdd91fffe,0xd8490000,0xdd930000,0xd8490002,0xdd930002,0xd8490004,0xdd950004,0xd8490006,
	0xdd950006,0xd8490008,0xdd970008,0xd849000a,0xdd97000a,0xd849000c,0xdd99000c,0xd849000e,
	0xdd99000e,0xd8490010,0xdd9b0010,0xd8490012,0xdd9b0012,0xd8490014,0xdd9d0014,0xd8490016,
	0xdd9d0016,0xd8490018,0xdd9f0018,0xd849001a,0xdd9f001a,0xd849001c,0xdda1001c,0xd849001e,
	0xdda1001e,0xd8490020,0xdda30020,0xd8490022,0xdda30022,0xd8490024,0xdda50024,0xd8490026,
	0xdda50026,0xd8490028,0xdda70028,0xd849002a,0xdda7002a,0xd849002c,0xdda9002c,0xd849002e,
	0xdda9002e,0xd8490030,0xddab0030,0xd8490032,0xddab0032,0xd8490034,0xddad0034,0xd8490036,
	0xddad0036,0xd8490038,0xddaf0038,0xd849003a,0xddaf003a,0xd849003c,0xddb1003c,0xd849003e,
	0xddb1003e,0xd8490040,0xddb30040,0xd8490042,0xddb30042,0xd8490044,0xddb50044,0xd8490046,
	0xddb50046,0xd8490048,0xddb70048,0xd849004a,0xddb7004a,0xd849004c,0xddb9004c,0xd849004e,
	0xddb9004e,0xd8490050,0xddbb0050,0xd8490052,0xddbb0052,0xd8490054,0xddbd0054,0xd8490056,
	0xddbd0056,0xd8490058,0xddbf0058,0xd849005a,0xddbf005a,0xd849005c,0xddc1005c,0xd849005e,
	0xddc1005e,0xd8490060,0xddc30060,0xd8490062,0xddc30062,0xd8490064,0xddc50064,0xd8490066,
	0xddc50066,0xd8490068,0xddc70068,0xd849006a,0xddc7006a,0xd849006c,0xddc9006c,0xd849006e,
	0xddc9006e,0xd8490070,0xddcb0070,0xd8490072,0xddcb0072,0xd8490074,0xddcd0074,0xd8490076,
	0xddcd0076,0xd8490078,0xddcf0078,0xd849007a,0xddcf007a,0xd849007c,0xddd1007c,0xd849007e,
	0xddd1007e,0xd8490080,0xddd30080,0xd8490082,0xddd30082,0xd8490084,0xddd50084,0xd8490086,
	0xddd50086,0xd8490088,0xddd70088,0xd849008a,0xddd7008a,0xd849008c,0xddd9008c,0xd849008e,
	0xddd9008e,0xd8490090,0xdddb0090,0xd8490092,0xdddb0092,0xd8490094,0xdddd0094,0xd8490096,
	0xdddd0096,0xd8490098,0xdddf0098,0xd849009a,0xdddf009a,0xd849009c,0xdde1009c,0xd849009e,
	0xdde1009e,0xd84900a0,0xdde300a0,0xd84900a2,0xdde300a2,0xd84900a4,0xdde500a4,0xd84900a6,
	0xdde500a6,0xd84900a8,0xdde700a8,0xd84900aa,0xdde700aa,0xd84900ac,0xdde900ac,0xd84900ae,
	0xdde900ae,0xd84900b0,0xddeb00b0,0xd84900b2,0xddeb00b2,0xd84900b4,0xdded00b4,0xd84900b6,
	0xdded00b6,0xd84900b8,0xddef00b8,0xd84900ba,0xddef00ba,0xd84900bc,0xddf100bc,0xd84900be,
	0xddf100be,0xd84900c0,0xddf300c0,0xd84900c2,0xddf300c2,0xd84900c4,0xddf500c4,0xd84900c6,
	0xddf500c6,0xd84900c8,0xddf700c8,0xd84900ca,0xddf700ca,0xd84900cc,0xddf900cc,0xd84900ce,
	0xddf900ce,0xd84900d0,0xddfb00d0,0xd84900d2,0xddfb00d2,0xd84900d4,0xddfd00d4,0xd84900d6,
	0xddfd00d6,0xd84900d8,0xddff00d8,0xd84900da,0xddff00da,0xd84900dc,0xde0100dc,0xd84900de,
	0xde0100de,0xd84900e0,0xde0300e0,0xd84900e2,0xde0300e2,0xd84900e4,0xde0500e4,0xd84900e6,
	0xde0500e6,0xd84900e8,0xde0700e8,0xd84900ea,0xde0700ea,0xd84900ec,0xde0900ec,0xd84900ee,
	0xde0900ee,0xd84900f0,0xde0b00f0,0xd84900f2,0xde0b00f2,0xd84900f4,0xde0d00f4,0xd84900f6,
	0xde0d00f6,0xd84900f8,0xde0f00f8,0xd84900fa,0xde0f00fa,0xd84900fc,0xde1100fc,0xd84900fe,
	0xde1100fe,0xd8490100,0xde130100,0xd8490102,0xde130102,0xd8490104,0xde150104,0xd8490106,
	0xde150106,0xd8490108,0xde170108,0xd849010a,0xde17010a,0xd849010c,0xde19010c,0xd849010e,
	0xde19010e,0xd8490110,0xde1b0110,0xd8490112,0xde1b0112,0xd8490114,0xde1d0114,0xd8490116,
	0xde1d0116,0xd8490118,0xde1f0118,0xd849011a,0xde1f011a,0xd849011c,0xde21011c,0xd849011e,
	0xde21011e,0xd8490120,0xde230120,0xd8490122,0xde230122,0xd8490124,0xde250124,0xd8490126,
	0xde250126,0xd8490128,0xde270128,0xd849012a,0xde27012a,0xd849012c,0xde29012c,0xd849012e,
	0xde29012e,0xd8490130,0xde2b0130,0xd8490132,0xde2b0132,0xd8490134,0xde2d0134,0xd8490136,
	0xde2d0136,0xd8490138,0xde2f0138,0xd849013a,0xde2f013a,0xd849013c,0xde31013c,0xd849013e,
	0xde31013e,0xd8490140,0xde330140,0xd8490142,0xde330142,0xd8490144,0xde350144,0xd8490146,
	0xde350146,0xd8490148,0xde370148,0xd849014a,0xde37014a,0xd849014c,0xde39014c,0xd849014e,
	0xde39014e,0xd8490150,0xde3b0150,0xd8490152,0xde3b0152,0xd8490154,0xde3d0154,0xd8490156,
	0xde3d0156,0xd8490158,0xde3f0158,0xd849015a,0xde3f015a,0xd849015c,0xde41015c,0xd849015e,
	0xde41015e,0xd8490160,0xde430160,0xd8490162,0xde430162,0xd8490164,0xde450164,0xd8490166,
	0xde450166,0xd8490168,0xde470168,0xd849016a,0xde47016a,0xd849016c,0xde49016c,0xd849016e,
	0xde49016e,0xd8490170,0xde4b0170,0xd8490172,0xde4b0172,0xd8490174,0xde4d0174,0xd8490176,
	0xde4d0176,0xd8490178,0xde4f0178,0xd849017a,0xde4f017a,0xd849017c,0xde51017c,0xd849017e,
	0xde51017e,0xd8490180,0xde530180,0xd8490182,0xde530182,0xd8490184,0xde550184,0xd8490186,
	0xde550186,0xd8490188,0xde570188,0xd849018a,0xde57018a,0xd849018c,0xde59018c,0xd849018e,
	0xde59018e,0xd8490190,0xde5b0190,0xd8490192,0xde5b0192,0xd8490194,0xde5d0194,0xd8490196,
	0xde5d0196,0xd8490198,0xde5f0198,0xd849019a,0xde5f019a,0xd849019c,0xde61019c,0xd849019e,
	0xde61019e,0xd84901a0,0xde6301a0,0xd84901a2,0xde6301a2,0xd84901a4,0xde6501a4,0xd84901a6,
	0xde6501a6,0xd84901a8,0xde6701a8,0xd84901aa,0xde6701aa,0xd84901ac,0xde6901ac,0xd84901ae,
	0xde6901ae,0xd84901b0,0xde6b01b0,0xd84901b2,0xde6b01b2,0xd84901b4,0xde6d01b4,0xd84901b6,
	0xde6d01b6,0xd84901b8,0xde6f01b8,0xd84901ba,0xde6f01ba,0xd84901bc,0xde7101bc,0xd84901be,
	0xde7101be,0xd84901c0,0xde7301c0,0xd84901c2,0xde7301c2,0xd84901c4,0xde7501c4,0xd84901c6,
	0xde7501c6,0xd84901c8,0xde7701c8,0xd84901ca,0xde7701ca,0xd84901cc,0xde7901cc,0xd84901ce,
	0xde7901ce,0xd84901d0,0xde7b01d0,0xd84901d2,0xde7b01d2,0xd84901d4,0xde7d01d4,0xd84901d6,
	0xde7d01d6,0xd84901d8,0xde7f01d8,0xd84901da,0xde7f01da,0xd84901dc,0xde8101dc,0xd84901de,
	0xde8101de,0xd84901e0,0xde8301e0,0xd84901e2,0xde8301e2,0xd84901e4,0xde8501e4,0xd84901e6,
	0xde8501e6,0xd84901e8,0xde8701e8,0xd84901ea,0xde8701ea,0xd84901ec,0xde8901ec,0xd84901ee,
	0xde8901ee,0xd84901f0,0xde8b01f0,0xd84901f2,0xde8b01f2,0xd84901f4,0xde8d01f4,0xd84901f6,
	0xde8d01f6,0xd84901f8,0xde8f01f8,0xd84901fa,0xde8f01fa,0xd84901fc,0xde9101fc,0xd84901fe,
	0xde9101fe,0xd8490200,0xde930200,0xd8490202,0xde930202,0xd8490204,0xde950204,0xd8490206,
	0xde950206,0xd8490208,0xde970208,0xd849020a,0xde97020a,0xd849020c,0xde99020c,0xd849020e,
	0xde99020e,0xd8490210,0xde9b0210,0xd8490212,0xde9b0212,0xd8490214,0xde9d0214,0xd8490216,
	0xde9d0216,0xd8490218,0xde9f0218,0xd849021a,0xde9f021a,0xd849021c,0xdea1021c,0xd849021e,
	0xdea1021e,0xd8490220,0xdea30220,0xd8490222,0xdea30222,0xd8490224,0xdea50224,0xd8490226,
	0xdea50226,0xd8490228,0xdea70228,0xd849022a,0xdea7022a,0xd849022c,0xdea9022c,0xd849022e,
	0xdea9022e,0xd8490230,0xdeab0230,0xd8490232,0xdeab0232,0xd8490234,0xdead0234,0xd8490236,
	0xdead0236,0xd8490238,0xdeaf0238,0xd849023a,0xdeaf023a,0xd849023c,0xdeb1023c,0xd849023e,
	0xdeb1023e,0xd8490240,0xdeb30240,0xd8490242,0xdeb30242,0xd8490244,0xdeb50244,0xd8490246,
	0xdeb50246,0xd8490248,0xdeb70248,0xd849024a,0xdeb7024a,0xd849024c,0xdeb9024c,0xd849024e,
	0xdeb9024e,0xd8490250,0xdebb0250,0xd8490252,0xdebb0252,0xd8490254,0xdebd0254,0xd8490256,
	0xdebd0256,0xd8490258,0xdebf0258,0xd849025a,0xdebf025a,0xd849025c,0xdec1025c,0xd849025e,
	0xdec1025e,0xd8490260,0xdec30260,0xd8490262,0xdec30262,0xd8490264,0xdec50264,0xd8490266,
	0xdec50266,0xd8490268,0xdec70268,0xd849026a,0xdec7026a,0xd849026c,0xdec9026c,0xd849026e,
	0xdec9026e,0xd8490270,0xdecb0270,0xd8490272,0xdecb0272,0xd8490274,0xdecd0274,0xd8490276,
	0xdecd0276,0xd8490278,0xdecf0278,0xd849027a,0xdecf027a,0xd849027c,0xded1027c,0xd849027e,
	0xded1027e,0xd8490280,0xded30280,0xd8490282,0xded30282,0xd8490284,0xded50284,0xd8490286,
	0xded50286,0xd8490288,0xded70288,0xd849028a,0xded7028a,0xd849028c,0xded9028c,0xd849028e,
	0xded9028e,0xd8490290,0xdedb0290,0xd8490292,0xdedb0292,0xd8490294,0xdedd0294,0xd8490296,
	0xdedd0296,0xd8490298,0xdedf0298,0xd849029a,0xdedf029a,0xd849029c,0xdee1029c,0xd849029e,
	0xdee1029e,0xd84902a0,0xdee302a0,0xd84902a2,0xdee302a2,0xd84902a4,0xdee502a4,0xd84902a6,
	0xdee502a6,0xd84902a8,0xdee702a8,0xd84902aa,0xdee702aa,0xd84902ac,0xdee902ac,0xd84902ae,
	0xdee902ae,0xd84902b0,0xdeeb02b0,0xd84902b2,0xdeeb02b2,0xd84902b4,0xdeed02b4,0xd84902b6,
	0xdeed02b6,0xd84902b8,0xdeef02b8,0xd84902ba,0xdeef02ba,0xd84902bc,0xdef102bc,0xd84902be,
	0xdef102be,0xd84902c0,0xdef302c0,0xd84902c2,0xdef302c2,0xd84902c4,0xdef502c4,0xd84902c6,
	0xdef502c6,0xd84902c8,0xdef702c8,0xd84902ca,0xdef702ca,0xd84902cc,0xdef902cc,0xd84902ce,
	0xdef902ce,0xd84902d0,0xdefb02d0,0xd84902d2,0xdefb02d2,0xd84902d4,0xdefd02d4,0xd84902d6,
	0xdefd02d6,0xd84902d8,0xdeff02d8,0xd84902da,0xdeff02da,0xd84902dc,0xdf0102dc,0xd84902de,
	0xdf0102de,0xd84902e0,0xdf0302e0,0xd84902e2,0xdf0302e2,0xd84902e4,0xdf0502e4,0xd84902e6,
	0xdf0502e6,0xd84902e8,0xdf0702e8,0xd84902ea,0xdf0702ea,0xd84902ec,0xdf0902ec,0xd84902ee,
	0xdf0902ee,0xd84902f0,0xdf0b02f0,0xd84902f2,0xdf0b02f2,0xd84902f4,0xdf0d02f4,0xd84902f6,
	0xdf0d02f6,0xd84902f8,0xdf0f02f8,0xd84902fa,0xdf0f02fa,0xd84902fc,0xdf1102fc,0xd84902fe,
	0xdf1102fe,0xd8490300,0xdf130300,0xd8490302,0xdf130302,0xd8490304,0xdf150304,0xd8490306,
	0xdf150306,0xd8490308,0xdf170308,0xd849030a,0xdf17030a,0xd849030c,0xdf19030c,0xd849030e,
	0xdf19030e,0xd8490310,0xdf1b0310,0xd8490312,0xdf1b0312,0xd8490314,0xdf1d0314,0xd8490316,
	0xdf1d0316,0xd8490318,0xdf1f0318,0xd849031a,0xdf1f031a,0xd849031c,0xdf21031c,0xd849031e,
	0xdf21031e,0xd8490320,0xdf230320,0xd8490322,0xdf230322,0xd8490324,0xdf250324,0xd8490326,
	0xdf250326,0xd8490328,0xdf270328,0xd849032a,0xdf27032a,0xd849032c,0xdf29032c,0xd849032e,
	0xdf29032e,0xd8490330,0xdf2b0330,0xd8490332,0xdf2b0332,0xd8490334,0xdf2d0334,0xd8490336,
	0xdf2d0336,0xd8490338,0xdf2f0338,0xd849033a,0xdf2f033a,0xd849033c,0xdf31033c,0xd849033e,
	0xdf31033e,0xd8490340,0xdf330340,0xd8490342,0xdf330342,0xd8490344,0xdf350344,0xd8490346,
	0xdf350346,0xd8490348,0xdf370348,0xd849034a,0xdf37034a,0xd849034c,0xdf39034c,0xd849034e,
	0xdf39034e,0xd8490350,0xdf3b0350,0xd8490352,0xdf3b0352,0xd8490354,0xdf3d0354,0xd8490356,
	0xdf3d0356,0xd8490358,0xdf3f0358,0xd849035a,0xdf3f035a,0xd849035c,0xdf41035c,0xd849035e,
	0xdf41035e,0xd8490360,0xdf430360,0xd8490362,0xdf430362,0xd8490364,0xdf450364,0xd8490366,
	0xdf450366,0xd8490368,0xdf470368,0xd849036a,0xdf47036a,0xd849036c,0xdf49036c,0xd849036e,
	0xdf49036e,0xd8490370,0xdf4b0370,0xd8490372,0xdf4b0372,0xd8490374,0xdf4d0374,0xd8490376,
	0xdf4d0376,0xd8490378,0xdf4f0378,0xd849037a,0xdf4f037a,0xd849037c,0xdf51037c,0xd849037e,
	0xdf51037e,0xd8490380,0xdf530380,0xd8490382,0xdf530382,0xd8490384,0xdf550384,0xd8490386,
	0xdf550386,0xd8490388,0xdf570388,0xd849038a,0xdf57038a,0xd849038c,0xdf59038c,0xd849038e,
	0xdf59038e,0xd8490390,0xdf5b0390,0xd8490392,0xdf5b0392,0xd8490394,0xdf5d0394,0xd8490396,
	0xdf5d0396,0xd8490398,0xdf5f0398,0xd849039a,0xdf5f039a,0xd849039c,0xdf61039c,0xd849039e,
	0xdf61039e,0xd84903a0,0xdf6303a0,0xd84903a2,0xdf6303a2,0xd84903a4,0xdf6503a4,0xd84903a6,
	0xdf6503a6,0xd84903a8,0xdf6703a8,0xd84903aa,0xdf6703aa,0xd84903ac,0xdf6903ac,0xd84903ae,
	0xdf6903ae,0xd84903b0,0xdf6b03b0,0xd84903b2,0xdf6b03b2,0xd84903b4,0xdf6d03b4,0xd84903b6,
	0xdf6d03b6,0xd84903b8,0xdf6f03b8,0xd84903ba,0xdf6f03ba,0xd84903bc,0xdf7103bc,0xd84903be,
	0xdf7103be,0xd84903c0,0xdf7303c0,0xd84903c2,0xdf7303c2,0xd84903c4,0xdf7503c4,0xd84903c6,
	0xdf7503c6,0xd84903c8,0xdf7703c8,0xd84903ca,0xdf7703ca,0xd84903cc,0xdf7903cc,0xd84903ce,
	0xdf7903ce,0xd84903d0,0xdf7b03d0,0xd84903d2,0xdf7b03d2,0xd84903d4,0xdf7d03d4,0xd84903d6,
	0xdf7d03d6,0xd84903d8,0xdf7f03d8,0xd84903da,0xdf7f03da,0xd84903dc,0xdf8103dc,0xd84903de,
	0xdf8103de,0xd84903e0,0xdf8303e0,0xd84903e2,0xdf8303e2,0xd84903e4,0xdf8503e4,0xd84903e6,
	0xdf8503e6,0xd84903e8,0xdf8703e8,0xd84903ea,0xdf8703ea,0xd84903ec,0xdf8903ec,0xd84903ee,
	0xdf8903ee,0xd84903f0,0xdf8b03f0,0xd84903f2,0xdf8b03f2,0xd84903f4,0xdf8d03f4,0xd84903f6,
	0xdf8d03f6,0xd84903f8,0xdf8f03f8,0xd84903fa,0xdf8f03fa,0xd84903fc,0xdf9103fc,0xd84903fe,
	0xdf9103fe,0xd8490400,0xdf930400,0xd8490402,0xdf930402,0xd8490404,0xdf950404,0xd8490406,
	0xdf950406,0xd8490408,0xdf970408,0xd849040a,0xdf97040a,0xd849040c,0xdf99040c,0xd849040e,
	0xdf99040e,0xd8490410,0xdf9b0410,0xd8490412,0xdf9b0412,0xd8490414,0xdf9d0414,0xd8490416,
	0xdf9d0416,0xd8490418,0xdf9f0418,0xd849041a,0xdf9f041a,0xd849041c,0xdfa1041c,0xd849041e,
	0xdfa1041e,0xd8490420,0xdfa30420,0xd8490422,0xdfa30422,0xd8490424,0xdfa50424,0xd8490426,
	0xdfa50426,0xd8490428,0xdfa70428,0xd849042a,0xdfa7042a,0xd849042c,0xdfa9042c,0xd849042e,
	0xdfa9042e,0xd8490430,0xdfab0430,0xd8490432,0xdfab0432,0xd8490434,0xdfad0434,0xd8490436,
	0xdfad0436,0xd8490438,0xdfaf0438,0xd849043a,0xdfaf043a,0xd849043c,0xdfb1043c,0xd849043e,
	0xdfb1043e,0xd8490440,0xdfb30440,0xd8490442,0xdfb30442,0xd8490444,0xdfb50444,0xd8490446,
	0xdfb50446,0xd8490448,0xdfb70448,0xd849044a,0xdfb7044a,0xd849044c,0xdfb9044c,0xd849044e,
	0xdfb9044e,0xd8490450,0xdfbb0450,0xd8490452,0xdfbb0452,0xd8490454,0xdfbd0454,0xd8490456,
	0xdfbd0456,0xd8490458,0xdfbf0458,0xd849045a,0xdfbf045a,0xd849045c,0xdfc1045c,0xd849045e,
	0xdfc1045e,0xd8490460,0xdfc30460,0xd8490462,0xdfc30462,0xd8490464,0xdfc50464,0xd8490466,
	0xdfc50466,0xd8490468,0xdfc70468,0xd849046a,0xdfc7046a,0xd849046c,0xdfc9046c,0xd849046e,
	0xdfc9046e,0xd8490470,0xdfcb0470,0xd8490472,0xdfcb0472,0xd8490474,0xdfcd0474,0xd8490476,
	0xdfcd0476,0xd8490478,0xdfcf0478,0xd849047a,0xdfcf047a,0xd849047c,0xdfd1047c,0xd849047e,
	0xdfd1047e,0xd8490480,0xdfd30480,0xd8490482,0xdfd30482,0xd8490484,0xdfd50484,0xd8490486,
	0xdfd50486,0xd8490488,0xdfd70488,0xd849048a,0xdfd7048a,0xd849048c,0xdfd9048c,0xd849048e,
	0xdfd9048e,0xd8490490,0xdfdb0490,0xd8490492,0xdfdb0492,0xd8490494,0xdfdd0494,0xd8490496,
	0xdfdd0496,0xd8490498,0xdfdf0498,0xd849049a,0xdfdf049a,0xd849049c,0xdfe1049c,0xd849049e,
	0xdfe1049e,0xd84904a0,0xdfe304a0,0xd84904a2,0xdfe304a2,0xd84904a4,0xdfe504a4,0xd84904a6,
	0xdfe504a6,0xd84904a8,0xdfe704a8,0xd84904aa,0xdfe704aa,0xd84904ac,0xdfe904ac,0xd84904ae,
	0xdfe904ae,0xd84904b0,0xdfeb04b0,0xd84904b2,0xdfeb04b2,0xd84904b4,0xdfed04b4,0xd84904b6,
	0xdfed04b6,0xd84904b8,0xdfef04b8,0xd84904ba,0xdfef04ba,0xd84904bc,0xdff104bc,0xd84904be,
	0xdff104be,0xd84904c0,0xdff304c0,0xd84904c2,0xdff304c2,0xd84904c4,0xdff504c4,0xd84904c6,
	0xdff504c6,0xd84904c8,0xdff704c8,0xd84904ca,0xdff704ca,0xd84904cc,0xdff904cc,0xd84904ce,
	0xdff904ce,0xd84904d0,0xdffb04d0,0xd84904d2,0xdffb04d2,0xd84904d4,0xdffd04d4,0xd84904d6,
	0xdffd04d6,0xd84904d8,0xdfff04d8,0xd84904da,0xdfff04da,0xd84904dc,0xdc0104dc,0xd84904de,
	0xdc0104de,0xd84904e0,0xdc0304e0,0xd84904e2,0xdc0304e2,0xd84904e4,0xdc0504e4,0xd84904e6,
	0xdc0504e6,0xd84904e8,0xdc0704e8,0xd84904ea,0xdc0704ea,0xd84904ec,0xdc0904ec,0xd84904ee,
	0xdc0904ee,0xd84904f0,0xdc0b04f0,0xd84904f2,0xdc0b04f2,0xd84904f4,0xdc0d04f4,0xd84904f6,
	0xdc0d04f6,0xd84904f8,0xdc0f04f8,0xd84904fa,0xdc0f04fa,0xd84904fc,0xdc1104fc,0xd84904fe,
	0xdc1104fe,0xd8490500,0xdc130500,0xd8490502,0xdc130502,0xd8490504,0xdc150504,0xd8490506,
	0xdc150506,0xd8490508,0xdc170508,0xd849050a,0xdc17050a,0xd849050c,0xdc19050c,0xd849050e,
	0xdc19050e,0xd8490510,0xdc1b0510,0xd8490512,0xdc1b0512,0xd8490514,0xdc1d0514,0xd8490516,
	0xdc1d0516,0xd8490518,0xdc1f0518,0xd849051a,0xdc1f051a,0xd849051c,0xdc21051c,0xd849051e,
	0xdc21051e,0xd8490520,0xdc230520,0xd8490522,0xdc230522,0xd8490524,0xdc250524,0xd8490526,
	0xdc250526,0xd8490528,0xdc270528,0xd849052a,0xdc27052a,0xd849052c,0xdc29052c,0xd849052e,
	0xdc29052e,0xd8490530,0xdc2b0530,0xd8490532,0xdc2b0532,0xd8490534,0xdc2d0534,0xd8490536,
	0xdc2d0536,0xd8490538,0xdc2f0538,0xd849053a,0xdc2f053a,0xd849053c,0xdc31053c,0xd849053e,
	0xdc31053e,0xd8490540,0xdc330540,0xd8490542,0xdc330542,0xd8490544,0xdc350544,0xd8490546,
	0xdc350546,0xd8490548,0xdc370548,0xd849054a,0xdc37054a,0xd849054c,0xdc39054c,0xd849054e,
	0xdc39054e,0xd8490550,0xdc3b0550,0xd8490552,0xdc3b0552,0xd8490554,0xdc3d0554,0xd8490556,
	0xdc3d0556,0xd8490558,0xdc3f0558,0xd849055a,0xdc3f055a,0xd849055c,0xdc41055c,0xd849055e,
	0xdc41055e,0xd8490560,0xdc430560,0xd8490562,0xdc430562,0xd8490564,0xdc450564,0xd8490566,
	0xdc450566,0xd8490568,0xdc470568,0xd849056a,0xdc47056a,0xd849056c,0xdc49056c,0xd849056e,
	0xdc49056e,0xd8490570,0xdc4b0570,0xd8490572,0xdc4b0572,0xd8490574,0xdc4d0574,0xd8490576,
	0xdc4d0576,0xd8490578,0xdc4f0578,0xd849057a,0xdc4f057a,0xd849057c,0xdc51057c,0xd849057e,
	0xdc51057e,0xd8490580,0xdc530580,0xd8490582,0xdc530582,0xd8490584,0xdc550584,0xd8490586,
	0xdc550586,0xd8490588,0xdc570588,0xd849058a,0xdc57058a,0xd849058c,0xdc59058c,0xd849058e,
	0xdc59058e,0xd8490590,0xdc5b0590,0xd8490592,0xdc5b0592,0xd8490594,0xdc5d0594,0xd8490596,
	0xdc5d0596,0xd8490598,0xdc5f0598,0xd849059a,0xdc5f059a,0xd849059c,0xdc61059c,0xd849059e,
	0xdc61059e,0xd84905a0,0xdc6305a0,0xd84905a2,0xdc6305a2,0xd84905a4,0xdc6505a4,0xd84905a6,
	0xdc6505a6,0xd84905a8,0xdc6705a8,0xd84905aa,0xdc6705aa,0xd84905ac,0xdc6905ac,0xd84905ae,
	0xdc6905ae,0xd84905b0,0xdc6b05b0,0xd84905b2,0xdc6b05b2,0xd84905b4,0xdc6d05b4,0xd84905b6,
	0xdc6d05b6,0xd84905b8,0xdc6f05b8,0xd84905ba,0xdc6f05ba,0xd84905bc,0xdc7105bc,0xd84905be,
	0xdc7105be,0xd84905c0,0xdc7305c0,0xd84905c2,0xdc7305c2,0xd84905c4,0xdc7505c4,0xd84905c6,
	0xdc7505c6,0xd84905c8,0xdc7705c8,0xd84905ca,0xdc7705ca,0xd84905cc,0xdc7905cc,0xd84905ce,
	0xdc7905ce,0xd84905d0,0xdc7b05d0,0xd84905d2,0xdc7b05d2,0xd84905d4,0xdc7d05d4,0xd84905d6,
	0xdc7d05d6,0xd84905d8,0xdc7f05d8,0xd84905da,0xdc7f05da,0xd84905dc,0xdc8105dc,0xd84905de,
	0xdc8105de,0xd84905e0,0xdc8305e0,0xd84905e2,0xdc8305e2,0xd84905e4,0xdc8505e4,0xd84905e6,
	0xdc8505e6,0xd84905e8,0xdc8705e8,0xd84905ea,0xdc8705ea,0xd84905ec,0xdc8905ec,0xd84905ee,
	0xdc8905ee,0xd84905f0,0xdc8b05f0,0xd84905f2,0xdc8b05f2,0xd84905f4,0xdc8d05f4,0xd84905f6,
	0xdc8d05f6,0xd84905f8,0xdc8f05f8,0xd84905fa,0xdc8f05fa,0xd84905fc,0xdc9105fc,0xd84905fe,
	0xdc9105fe,0xd8490600,0xdc930600,0xd8490602,0xdc930602,0xd8490604,0xdc950604,0xd8490606,
	0xdc950606,0xd8490608,0xdc970608,0xd849060a,0xdc97060a,0xd849060c,0xdc99060c,0xd849060e,
	0xdc99060e,0xd8490610,0xdc9b0610,0xd8490612,0xdc9b0612,0xd8490614,0xdc9d0614,0xd8490616,
	0xdc9d0616,0xd8490618,0xdc9f0618,0xd849061a,0xdc9f061a,0xd849061c,0xdca1061c,0xd849061e,
	0xdca1061e,0xd8490620,0xdca30620,0xd8490622,0xdca30622,0xd8490624,0xdca50624,0xd8490626,
	0xdca50626,0xd8490628,0xdca70628,0xd849062a,0xdca7062a,0xd849062c,0xdca9062c,0xd849062e,
	0xdca9062e,0xd8490630,0xdcab0630,0xd8490632,0xdcab0632,0xd8490634,0xdcad0634,0xd8490636,
	0xdcad0636,0xd8490638,0xdcaf0638,0xd849063a,0xdcaf063a,0xd849063c,0xdcb1063c,0xd849063e,
	0xdcb1063e,0xd8490640,0xdcb30640,0xd8490642,0xdcb30642,0xd8490644,0xdcb50644,0xd8490646,
	0xdcb50646,0xd8490648,0xdcb70648,0xd849064a,0xdcb7064a,0xd849064c,0xdcb9064c,0xd849064e,
	0xdcb9064e,0xd8490650,0xdcbb0650,0xd8490652,0xdcbb0652,0xd8490654,0xdcbd0654,0xd8490656,
	0xdcbd0656,0xd8490658,0xdcbf0658,0xd849065a,0xdcbf065a,0xd849065c,0xdcc1065c,0xd849065e,
	0xdcc1065e,0xd8490660,0xdcc30660,0xd8490662,0xdcc30662,0xd8490664,0xdcc50664,0xd8490666,
	0xdcc50666,0xd8490668,0xdcc70668,0xd849066a,0xdcc7066a,0xd849066c,0xdcc9066c,0xd849066e,
	0xdcc9066e,0xd8490670,0xdccb0670,0xd8490672,0xdccb0672,0xd8490674,0xdccd0674,0xd8490676,
	0xdccd0676,0xd8490678,0xdccf0678,0xd849067a,0xdccf067a,0xd849067c,0xdcd1067c,0xd849067e,
	0xdcd1067e,0xd8490680,0xdcd30680,0xd8490682,0xdcd30682,0xd8490684,0xdcd50684,0xd8490686,
	0xdcd50686,0xd8490688,0xdcd70688,0xd849068a,0xdcd7068a,0xd849068c,0xdcd9068c,0xd849068e,
	0xdcd9068e,0xd8490690,0xdcdb0690,0xd8490692,0xdcdb0692,0xd8490694,0xdcdd0694,0xd8490696,
	0xdcdd0696,0xd8490698,0xdcdf0698,0xd849069a,0xdcdf069a,0xd849069c,0xdce1069c,0xd849069e,
	0xdce1069e,0xd84906a0,0xdce306a0,0xd84906a2,0xdce306a2,0xd84906a4,0xdce506a4,0xd84906a6,
	0xdce506a6,0xd84906a8,0xdce706a8,0xd84906aa,0xdce706aa,0xd84906ac,0xdce906ac,0xd84906ae,
	0xdce906ae,0xd84906b0,0xdceb06b0,0xd84906b2,0xdceb06b2,0xd84906b4,0xdced06b4,0xd84906b6,
	0xdced06b6,0xd84906b8,0xdcef06b8,0xd84906ba,0xdcef06ba,0xd84906bc,0xdcf106bc,0xd84906be,
	0xdcf106be,0xd84906c0,0xdcf306c0,0xd84906c2,0xdcf306c2,0xd84906c4,0xdcf506c4,0xd84906c6,
	0xdcf506c6,0xd84906c8,0xdcf706c8,0xd84906ca,0xdcf706ca,0xd84906cc,0xdcf906cc,0xd84906ce,
	0xdcf906ce,0xd84906d0,0xdcfb06d0,0xd84906d2,0xdcfb06d2,0xd84906d4,0xdcfd06d4,0xd84906d6,
	0xdcfd06d6,0xd84906d8,0xdcff06d8,0xd84906da,0xdcff06da,0xd84906dc,0xdd0106dc,0xd84906de,
	0xdd0106de,0xd84906e0,0xdd0306e0,0xd84906e2,0xdd0306e2,0xd84906e4,0xdd0506e4,0xd84906e6,
	0xdd0506e6,0xd84906e8,0xdd0706e8,0xd84906ea,0xdd0706ea,0xd84906ec,0xdd0906ec,0xd84906ee,
	0xdd0906ee,0xd84906f0,0xdd0b06f0,0xd84906f2,0xdd0b06f2,0xd84906f4,0xdd0d06f4,0xd84906f6,
	0xdd0d06f6,0xd84906f8,0xdd0f06f8,0xd84906fa,0xdd0f06fa,0xd84906fc,0xdd1106fc,0xd84906fe,
	0xdd1106fe,0xd8490700,0xdd130700,0xd8490702,0xdd130702,0xd8490704,0xdd150704,0xd8490706,
	0xdd150706,0xd8490708,0xdd170708,0xd849070a,0xdd17070a,0xd849070c,0xdd19070c,0xd849070e,
	0xdd19070e,0xd8490710,0xdd1b0710,0xd8490712,0xdd1b0712,0xd8490714,0xdd1d0714,0xd8490716,
	0xdd1d0716,0xd8490718,0xdd1f0718,0xd849071a,0xdd1f071a,0xd849071c,0xdd21071c,0xd849071e,
	0xdd21071e,0xd8490720,0xdd230720,0xd8490722,0xdd230722,0xd8490724,0xdd250724,0xd8490726,
	0xdd250726,0xd8490728,0xdd270728,0xd849072a,0xdd27072a,0xd849072c,0xdd29072c,0xd849072e,
	0xdd29072e,0xd8490730,0xdd2b0730,0xd8490732,0xdd2b0732,0xd8490734,0xdd2d0734,0xd8490736,
	0xdd2d0736,0xd8490738,0xdd2f0738,0xd849073a,0xdd2f073a,0xd849073c,0xdd31073c,0xd849073e,
	0xdd31073e,0xd8490740,0xdd330740,0xd8490742,0xdd330742,0xd8490744,0xdd350744,0xd8490746,
	0xdd350746,0xd8490748,0xdd370748,0xd849074a,0xdd37074a,0xd849074c,0xdd39074c,0xd849074e,
	0xdd39074e,0xd8490750,0xdd3b0750,0xd8490752,0xdd3b0752,0xd8490754,0xdd3d0754,0xd8490756,
	0xdd3d0756,0xd8490758,0xdd3f0758,0xd849075a,0xdd3f075a,0xd849075c,0xdd41075c,0xd849075e,
	0xdd41075e,0xd8490760,0xdd430760,0xd8490762,0xdd430762,0xd8490764,0xdd450764,0xd8490766,
	0xdd450766,0xd8490768,0xdd470768,0xd849076a,0xdd47076a,0xd849076c,0xdd49076c,0xd849076e,
	0xdd49076e,0xd8490770,0xdd4b0770,0xd8490772,0xdd4b0772,0xd8490774,0xdd4d0774,0xd8490776,
	0xdd4d0776,0xd8490778,0xdd4f0778,0xd849077a,0xdd4f077a,0xd849077c,0xdd51077c,0xd849077e,
	0xdd51077e,0xd8490780,0xdd530780,0xd8490782,0xdd530782,0xd8490784,0xdd550784,0xd8490786,
	0xdd550786,0xd8490788,0xdd570788,0xd849078a,0xdd57078a,0xd849078c,0xdd59078c,0xd849078e,
	0xdd59078e,0xd8490790,0xdd5b0790,0xd8490792,0xdd5b0792,0xd8490794,0xdd5d0794,0xd8490796,
	0xdd5d0796,0xd8490798,0xdd5f0798,0xd849079a,0xdd5f079a,0xd849079c,0xdd61079c,0xd849079e,
	0xdd61079e,0xd84907a0,0xdd6307a0,0xd84907a2,0xdd6307a2,0xd84907a4,0xdd6507a4,0xd84907a6,
	0xdd6507a6,0xd84907a8,0xdd6707a8,0xd84907aa,0xdd6707aa,0xd84907ac,0xdd6907ac,0xd84907ae,
	0xdd6907ae,0xd84907b0,0xdd6b07b0,0xd84907b2,0xdd6b07b2,0xd84907b4,0xdd6d07b4,0xd84907b6,
	0xdd6d07b6,0xd84907b8,0xdd6f07b8,0xd84907ba,0xdd6f07ba,0xd84907bc,0xdd7107bc,0xd84907be,
	0xdd7107be,0xd84907c0,0xdd7307c0,0xd84907c2,0xdd7307c2,0xd84907c4,0xdd7507c4,0xd84907c6,
	0xdd7507c6,0xd84907c8,0xdd7707c8,0xd84907ca,0xdd7707ca,0xd84907cc,0xdd7907cc,0xd84907ce,
	0xdd7907ce,0xd84907d0,0xdd7b07d0,0xd84907d2,0xdd7b07d2,0xd84907d4,0xdd7d07d4,0xd84907d6,
	0xdd7d07d6,0xd84907d8,0xdd7f07d8,0xd84907da,0xdd7f07da,0xd84907dc,0xdd8107dc,0xd84907de,
	0xdd8107de,0xd84907e0,0xdd8307e0,0xd84907e2,0xdd8307e2,0xd84907e4,0xdd8507e4,0xd84907e6,
	0xdd8507e6,0xd84907e8,0xdd8707e8,0xd84907ea,0xdd8707ea,0xd84907ec,0xdd8907ec,0xd84907ee,
	0xdd8907ee,0xd84907f0,0xdd8b07f0,0xd84907f2,0xdd8b07f2,0xd84907f4,0xdd8d07f4,0xd84907f6,
	0xdd8d07f6,0xd84907f8,0xdd8f07f8,0xd84907fa,0xdd8f07fa,0xd84907fc,0xdd9107fc,0xd84907fe,
	0xdd9107fe,0xd8490800,0xdd930800,0xd8490802,0xdd930802,0xd8490804,0xdd950804,0xd8490806,
	0xdd950806,0xd8490808,0xdd970808,0xd849080a,0xdd97080a,0xd849080c,0xdd99080c,0xd849080e,
	0xdd99080e,0xd8490810,0xdd9b0810,0xd8490812,0xdd9b0812,0xd8490814,0xdd9d0814,0xd8490816,
	0xdd9d0816,0xd8490818,0xdd9f0818,0xd849081a,0xdd9f081a,0xd849081c,0xdda1081c,0xd849081e,
	0xdda1081e,0xd8490820,0xdda30820,0xd8490822,0xdda30822,0xd8490824,0xdda50824,0xd8490826,
	0xdda50826,0xd8490828,0xdda70828,0xd849082a,0xdda7082a,0xd849082c,0xdda9082c,0xd849082e,
	0xdda9082e,0xd8490830,0xddab0830,0xd8490832,0xddab0832,0xd8490834,0xddad0834,0xd8490836,
	0xddad0836,0xd8490838,0xddaf0838,0xd849083a,0xddaf083a,0xd849083c,0xddb1083c,0xd849083e,
	0xddb1083e,0xd8490840,0xddb30840,0xd8490842,0xddb30842,0xd8490844,0xddb50844,0xd8490846,
	0xddb50846,0xd8490848,0xddb70848,0xd849084a,0xddb7084a,0xd849084c,0xddb9084c,0xd849084e,
	0xddb9084e,0xd8490850,0xddbb0850,0xd8490852,0xddbb0852,0xd8490854,0xddbd0854,0xd8490856,
	0xddbd0856,0xd8490858,0xddbf0858,0xd849085a,0xddbf085a,0xd849085c,0xddc1085c,0xd849085e,
	0xddc1085e,0xd8490860,0xddc30860,0xd8490862,0xddc30862,0xd8490864,0xddc50864,0xd8490866,
	0xddc50866,0xd8490868,0xddc70868,0xd849086a,0xddc7086a,0xd849086c,0xddc9086c,0xd849086e,
	0xddc9086e,0xd8490870,0xddcb0870,0xd8490872,0xddcb0872,0xd8490874,0xddcd0874,0xd8490876,
	0xddcd0876,0xd8490878,0xddcf0878,0xd849087a,0xddcf087a,0xd849087c,0xddd1087c,0xd849087e,
	0xddd1087e,0xd8490880,0xddd30880,0xd8490882,0xddd30882,0xd8490884,0xddd50884,0xd8490886,
	0xddd50886,0xd8490888,0xddd70888,0xd849088a,0xddd7088a,0xd849088c,0xddd9088c,0xd849088e,
	0xddd9088e,0xd8490890,0xdddb0890,0xd8490892,0xdddb0892,0xd8490894,0xdddd0894,0xd8490896,
	0xdddd0896,0xd8490898,0xdddf0898,0xd849089a,0xdddf089a,0xd849089c,0xdde1089c,0xd849089e,
	0xdde1089e,0xd84908a0,0xdde308a0,0xd84908a2,0xdde308a2,0xd84908a4,0xdde508a4,0xd84908a6,
	0xdde508a6,0xd84908a8,0xdde708a8,0xd84908aa,0xdde708aa,0xd84908ac,0xdde908ac,0xd84908ae,
	0xdde908ae,0xd84908b0,0xddeb08b0,0xd84908b2,0xddeb08b2,0xd84908b4,0xdded08b4,0xd84908b6,
	0xdded08b6,0xd84908b8,0xddef08b8,0xd84908ba,0xddef08ba,0xd84908bc,0xddf108bc,0xd84908be,
	0xddf108be,0xd84908c0,0xddf308c0,0xd84908c2,0xddf308c2,0xd84908c4,0xddf508c4,0xd84908c6,
	0xddf508c6,0xd84908c8,0xddf708c8,0xd84908ca,0xddf708ca,0xd84908cc,0xddf908cc,0xd84908ce,
	0xddf908ce,0xd84908d0,0xddfb08d0,0xd84908d2,0xddfb08d2,0xd84908d4,0xddfd08d4,0xd84908d6,
	0xddfd08d6,0xd84908d8,0xddff08d8,0xd84908da,0xddff08da,0xd84908dc,0xde0108dc,0xd84908de,
	0xde0108de,0xd84908e0,0xde0308e0,0xd84908e2,0xde0308e2,0xd84908e4,0xde0508e4,0xd84908e6,
	0xde0508e6,0xd84908e8,0xde0708e8,0xd84908ea,0xde0708ea,0xd84908ec,0xde0908ec,0xd84908ee,
	0xde0908ee,0xd84908f0,0xde0b08f0,0xd84908f2,0xde0b08f2,0xd84908f4,0xde0d08f4,0xd84908f6,
	0xde0d08f6,0xd84908f8,0xde0f08f8,0xd84908fa,0xde0f08fa,0xd84908fc,0xde1108fc,0xd84908fe,
	0xde1108fe,0xd8490900,0xde130900,0xd8490902,0xde130902,0xd8490904,0xde150904,0xd8490906,
	0xde150906,0xd8490908,0xde170908,0xd849090a,0xde17090a,0xd849090c,0xde19090c,0xd849090e,
	0xde19090e,0xd8490910,0xde1b0910,0xd8490912,0xde1b0912,0xd8490914,0xde1d0914,0xd8490916,
	0xde1d0916,0xd8490918,0xde1f0918,0xd849091a,0xde1f091a,0xd849091c,0xde21091c,0xd849091e,
	0xde21091e,0xd8490920,0xde230920,0xd8490922,0xde230922,0xd8490924,0xde250924,0xd8490926,
	0xde250926,0xd8490928,0xde270928,0xd849092a,0xde27092a,0xd849092c,0xde29092c,0xd849092e,
	0xde29092e,0xd8490930,0xde2b0930,0xd8490932,0xde2b0932,0xd8490934,0xde2d0934,0xd8490936,
	0xde2d0936,0xd8490938,0xde2f0938,0xd849093a,0xde2f093a,0xd849093c,0xde31093c,0xd849093e,
	0xde31093e,0xd8490940,0xde330940,0xd8490942,0xde330942,0xd8490944,0xde350944,0xd8490946,
	0xde350946,0xd8490948,0xde370948,0xd849094a,0xde37094a,0xd849094c,0xde39094c,0xd849094e,
	0xde39094e,0xd8490950,0xde3b0950,0xd8490952,0xde3b0952,0xd8490954,0xde3d0954,0xd8490956,
	0xde3d0956,0xd8490958,0xde3f0958,0xd849095a,0xde3f095a,0xd849095c,0xde41095c,0xd849095e,
	0xde41095e,0xd8490960,0xde430960,0xd8490962,0xde430962,0xd8490964,0xde450964,0xd8490966,
	0xde450966,0xd8490968,0xde470968,0xd849096a,0xde47096a,0xd849096c,0xde49096c,0xd849096e,
	0xde49096e,0xd8490970,0xde4b0970,0xd8490972,0xde4b0972,0xd8490974,0xde4d0974,0xd8490976,
	0xde4d0976,0xd8490978,0xde4f0978,0xd849097a,0xde4f097a,0xd849097c,0xde51097c,0xd849097e,
	0xde51097e,0xd8490980,0xde530980,0xd8490982,0xde530982,0xd8490984,0xde550984,0xd8490986,
	0xde550986,0xd8490988,0xde570988,0xd849098a,0xde57098a,0xd849098c,0xde59098c,0xd849098e,
	0xde59098e,0xd8490990,0xde5b0990,0xd8490992,0xde5b0992,0xd8490994,0xde5d0994,0xd8490996,
	0xde5d0996,0xd8490998,0xde5f0998,0xd849099a,0xde5f099a,0xd849099c,0xde61099c,0xd849099e,
	0xde61099e,0xd84909a0,0xde6309a0,0xd84909a2,0xde6309a2,0xd84909a4,0xde6509a4,0xd84909a6,
	0xde6509a6,0xd84909a8,0xde6709a8,0xd84909aa,0xde6709aa,0xd84909ac,0xde6909ac,0xd84909ae,
	0xde6909ae,0xd84909b0,0xde6b09b0,0xd84909b2,0xde6b09b2,0xd84909b4,0xde6d09b4,0xd84909b6,
	0xde6d09b6,0xd84909b8,0xde6f09b8,0xd84909ba,0xde6f09ba,0xd84909bc,0xde7109bc,0xd84909be,
	0xde7109be,0xd84909c0,0xde7309c0,0xd84909c2,0xde7309c2,0xd84909c4,0xde7509c4,0xd84909c6,
	0xde7509c6,0xd84909c8,0xde7709c8,0xd84909ca,0xde7709ca,0xd84909cc,0xde7909cc,0xd84909ce,
	0xde7909ce,0xd84909d0,0xde7b09d0,0xd84909d2,0xde7b09d2,0xd84909d4,0xde7d09d4,0xd84909d6,
	0xde7d09d6,0xd84909d8,0xde7f09d8,0xd84909da,0xde7f09da,0xd84909dc,0xde8109dc,0xd84909de,
	0xde8109de,0xd84909e0,0xde8309e0,0xd84909e2,0xde8309e2,0xd84909e4,0xde8509e4,0xd84909e6,
	0xde8509e6,0xd84909e8,0xde8709e8,0xd84909ea,0xde8709ea,0xd84909ec,0xde8909ec,0xd84909ee,
	0xde8909ee,0xd84909f0,0xde8b09f0,0xd84909f2,0xde8b09f2,0xd84909f4,0xde8d09f4,0xd84909f6,
	0xde8d09f6,0xd84909f8,0xde8f09f8,0xd84909fa,0xde8f09fa,0xd84909fc,0xde9109fc,0xd84909fe,
	0xde9109fe,0xd8490a00,0xde930a00,0xd8490a02,0xde930a02,0xd8490a04,0xde950a04,0xd8490a06,
	0xde950a06,0xd8490a08,0xde970a08,0xd8490a0a,0xde970a0a,0xd8490a0c,0xde990a0c,0xd8490a0e,
	0xde990a0e,0xd8490a10,0xde9b0a10,0xd8490a12,0xde9b0a12,0xd8490a14,0xde9d0a14,0xd8490a16,
	0xde9d0a16,0xd8490a18,0xde9f0a18,0xd8490a1a,0xde9f0a1a,0xd8490a1c,0xdea10a1c,0xd8490a1e,
	0xdea10a1e,0xd8490a20,0xdea30a20,0xd8490a22,0xdea30a22,0xd8490a24,0xdea50a24,0xd8490a26,
	0xdea50a26,0xd8490a28,0xdea70a28,0xd8490a2a,0xdea70a2a,0xd8490a2c,0xdea90a2c,0xd8490a2e,
	0xdea90a2e,0xd8490a30,0xdeab0a30,0xd8490a32,0xdeab0a32,0xd8490a34,0xdead0a34,0xd8490a36,
	0xdead0a36,0xd8490a38,0xdeaf0a38,0xd8490a3a,0xdeaf0a3a,0xd8490a3c,0xdeb10a3c,0xd8490a3e,
	0xdeb10a3e,0xd8490a40,0xdeb30a40,0xd8490a42,0xdeb30a42,0xd8490a44,0xdeb50a44,0xd8490a46,
	0xdeb50a46,0xd8490a48,0xdeb70a48,0xd8490a4a,0xdeb70a4a,0xd8490a4c,0xdeb90a4c,0xd8490a4e,
	0xdeb90a4e,0xd8490a50,0xdebb0a50,0xd8490a52,0xdebb0a52,0xd8490a54,0xdebd0a54,0xd8490a56,
	0xdebd0a56,0xd8490a58,0xdebf0a58,0xd8490a5a,0xdebf0a5a,0xd8490a5c,0xdec10a5c,0xd8490a5e,
	0xdec10a5e,0xd8490a60,0xdec30a60,0xd8490a62,0xdec30a62,0xd8490a64,0xdec50a64,0xd8490a66,
	0xdec50a66,0xd8490a68,0xdec70a68,0xd8490a6a,0xdec70a6a,0xd8490a6c,0xdec90a6c,0xd8490a6e,
	0xdec90a6e,0xd8490a70,0xdecb0a70,0xd8490a72,0xdecb0a72,0xd8490a74,0xdecd0a74,0xd8490a76,
	0xdecd0a76,0xd8490a78,0xdecf0a78,0xd8490a7a,0xdecf0a7a,0xd8490a7c,0xded10a7c,0xd8490a7e,
	0xded10a7e,0xd8490a80,0xded30a80,0xd8490a82,0xded30a82,0xd8490a84,0xded50a84,0xd8490a86,
	0xded50a86,0xd8490a88,0xded70a88,0xd8490a8a,0xded70a8a,0xd8490a8c,0xded90a8c,0xd8490a8e,
	0xded90a8e,0xd8490a90,0xdedb0a90,0xd8490a92,0xdedb0a92,0xd8490a94,0xdedd0a94,0xd8490a96,
	0xdedd0a96,0xd8490a98,0xdedf0a98,0xd8490a9a,0xdedf0a9a,0xd8490a9c,0xdee10a9c,0xd8490a9e,
	0xdee10a9e,0xd8490aa0,0xdee30aa0,0xd8490aa2,0xdee30aa2,0xd8490aa4,0xdee50aa4,0xd8490aa6,
	0xdee50aa6,0xd8490aa8,0xdee70aa8,0xd8490aaa,0xdee70aaa,0xd8490aac,0xdee90aac,0xd8490aae,
	0xdee90aae,0xd8490ab0,0xdeeb0ab0,0xd8490ab2,0xdeeb0ab2,0xd8490ab4,0xdeed0ab4,0xd8490ab6,
	0xdeed0ab6,0xd8490ab8,0xdeef0ab8,0xd8490aba,0xdeef0aba,0xd8490abc,0xdef10abc,0xd8490abe,
	0xdef10abe,0xd8490ac0,0xdef30ac0,0xd8490ac2,0xdef30ac2,0xd8490ac4,0xdef50ac4,0xd8490ac6,
	0xdef50ac6,0xd8490ac8,0xdef70ac8,0xd8490aca,0xdef70aca,0xd8490acc,0xdef90acc,0xd8490ace,
	0xdef90ace,0xd8490ad0,0xdefb0ad0,0xd8490ad2,0xdefb0ad2,0xd8490ad4,0xdefd0ad4,0xd8490ad6,
	0xdefd0ad6,0xd8490ad8,0xdeff0ad8,0xd8490ada,0xdeff0ada,0xd8490adc,0xdf010adc,0xd8490ade,
	0xdf010ade,0xd8490ae0,0xdf030ae0,0xd8490ae2,0xdf030ae2,0xd8490ae4,0xdf050ae4,0xd8490ae6,
	0xdf050ae6,0xd8490ae8,0xdf070ae8,0xd8490aea,0xdf070aea,0xd8490aec,0xdf090aec,0xd8490aee,
	0xdf090aee,0xd8490af0,0xdf0b0af0,0xd8490af2,0xdf0b0af2,0xd8490af4,0xdf0d0af4,0xd8490af6,
	0xdf0d0af6,0xd8490af8,0xdf0f0af8,0xd8490afa,0xdf0f0afa,0xd8490afc,0xdf110afc,0xd8490afe,
	0xdf110afe,0xd8490b00,0xdf130b00,0xd8490b02,0xdf130b02,0xd8490b04,0xdf150b04,0xd8490b06,
	0xdf150b06,0xd8490b08,0xdf170b08,0xd8490b0a,0xdf170b0a,0xd8490b0c,0xdf190b0c,0xd8490b0e,
	0xdf190b0e,0xd8490b10,0xdf1b0b10,0xd8490b12,0xdf1b0b12,0xd8490b14,0xdf1d0b14,0xd8490b16,
	0xdf1d0b16,0xd8490b18,0xdf1f0b18,0xd8490b1a,0xdf1f0b1a,0xd8490b1c,0xdf210b1c,0xd8490b1e,
	0xdf210b1e,0xd8490b20,0xdf230b20,0xd8490b22,0xdf230b22,0xd8490b24,0xdf250b24,0xd8490b26,
	0xdf250b26,0xd8490b28,0xdf270b28,0xd8490b2a,0xdf270b2a,0xd8490b2c,0xdf290b2c,0xd8490b2e,
	0xdf290b2e,0xd8490b30,0xdf2b0b30,0xd8490b32,0xdf2b0b32,0xd8490b34,0xdf2d0b34,0xd8490b36,
	0xdf2d0b36,0xd8490b38,0xdf2f0b38,0xd8490b3a,0xdf2f0b3a,0xd8490b3c,0xdf310b3c,0xd8490b3e,
	0xdf310b3e,0xd8490b40,0xdf330b40,0xd8490b42,0xdf330b42,0xd8490b44,0xdf350b44,0xd8490b46,
	0xdf350b46,0xd8490b48,0xdf370b48,0xd8490b4a,0xdf370b4a,0xd8490b4c,0xdf390b4c,0xd8490b4e,
	0xdf390b4e,0xd8490b50,0xdf3b0b50,0xd8490b52,0xdf3b0b52,0xd8490b54,0xdf3d0b54,0xd8490b56,
	0xdf3d0b56,0xd8490b58,0xdf3f0b58,0xd8490b5a,0xdf3f0b5a,0xd8490b5c,0xdf410b5c,0xd8490b5e,
	0xdf410b5e,0xd8490b60,0xdf430b60,0xd8490b62,0xdf430b62,0xd8490b64,0xdf450b64,0xd8490b66,
	0xdf450b66,0xd8490b68,0xdf470b68,0xd8490b6a,0xdf470b6a,0xd8490b6c,0xdf490b6c,0xd8490b6e,
	0xdf490b6e,0xd8490b70,0xdf4b0b70,0xd8490b72,0xdf4b0b72,0xd8490b74,0xdf4d0b74,0xd8490b76,
	0xdf4d0b76,0xd8490b78,0xdf4f0b78,0xd8490b7a,0xdf4f0b7a,0xd8490b7c,0xdf510b7c,0xd8490b7e,
	0xdf510b7e,0xd8490b80,0xdf530b80,0xd8490b82,0xdf530b82,0xd8490b84,0xdf550b84,0xd8490b86,
	0xdf550b86,0xd8490b88,0xdf570b88,0xd8490b8a,0xdf570b8a,0xd8490b8c,0xdf590b8c,0xd8490b8e,
	0xdf590b8e,0xd8490b90,0xdf5b0b90,0xd8490b92,0xdf5b0b92,0xd8490b94,0xdf5d0b94,0xd8490b96,
	0xdf5d0b96,0xd8490b98,0xdf5f0b98,0xd8490b9a,0xdf5f0b9a,0xd8490b9c,0xdf610b9c,0xd8490b9e,
	0xdf610b9e,0xd8490ba0,0xdf630ba0,0xd8490ba2,0xdf630ba2,0xd8490ba4,0xdf650ba4,0xd8490ba6,
	0xdf650ba6,0xd8490ba8,0xdf670ba8,0xd8490baa,0xdf670baa,0xd8490bac,0xdf690bac,0xd8490bae,
	0xdf690bae,0xd8490bb0,0xdf6b0bb0,0xd8490bb2,0xdf6b0bb2,0xd8490bb4,0xdf6d0bb4,0xd8490bb6,
	0xdf6d0bb6,0xd8490bb8,0xdf6f0bb8,0xd8490bba,0xdf6f0bba,0xd8490bbc,0xdf710bbc,0xd8490bbe,
	0xdf710bbe,0xd8490bc0,0xdf730bc0,0xd8490bc2,0xdf730bc2,0xd8490bc4,0xdf750bc4,0xd8490bc6,
	0xdf750bc6,0xd8490bc8,0xdf770bc8,0xd8490bca,0xdf770bca,0xd8490bcc,0xdf790bcc,0xd8490bce,
	0xdf790bce,0xd8490bd0,0xdf7b0bd0,0xd8490bd2,0xdf7b0bd2,0xd8490bd4,0xdf7d0bd4,0xd8490bd6,
	0xdf7d0bd6,0xd8490bd8,0xdf7f0bd8,0xd8490bda,0xdf7f0bda,0xd8490bdc,0xdf810bdc,0xd8490bde,
	0xdf810bde,0xd8490be0,0xdf830be0,0xd8490be2,0xdf830be2,0xd8490be4,0xdf850be4,0xd8490be6,
	0xdf850be6,0xd8490be8,0xdf870be8,0xd8490bea,0xdf870bea,0xd8490bec,0xdf890bec,0xd8490bee,
	0xdf890bee,0xd8490bf0,0xdf8b0bf0,0xd8490bf2,0xdf8b0bf2,0xd8490bf4,0xdf8d0bf4,0xd8490bf6,
	0xdf8d0bf6,0xd8490bf8,0xdf8f0bf8,0xd8490bfa,0xdf8f0bfa,0xd8490bfc,0xdf910bfc,0xd8490bfe,
	0xdf910bfe,0xd8490c00,0xdf930c00,0xd8490c02,0xdf930c02,0xd8490c04,0xdf950c04,0xd8490c06,
	0xdf950c06,0xd8490c08,0xdf970c08,0xd8490c0a,0xdf970c0a,0xd8490c0c,0xdf990c0c,0xd8490c0e,
	0xdf990c0e,0xd8490c10,0xdf9b0c10,0xd8490c12,0xdf9b0c12,0xd8490c14,0xdf9d0c14,0xd8490c16,
	0xdf9d0c16,0xd8490c18,0xdf9f0c18,0xd8490c1a,0xdf9f0c1a,0xd8490c1c,0xdfa10c1c,0xd8490c1e,
	0xdfa10c1e,0xd8490c20,0xdfa30c20,0xd8490c22,0xdfa30c22,0xd8490c24,0xdfa50c24,0xd8490c26,
	0xdfa50c26,0xd8490c28,0xdfa70c28,0xd8490c2a,0xdfa70c2a,0xd8490c2c,0xdfa90c2c,0xd8490c2e,
	0xdfa90c2e,0xd8490c30,0xdfab0c30,0xd8490c32,0xdfab0c32,0xd8490c34,0xdfad0c34,0xd8490c36,
	0xdfad0c36,0xd8490c38,0xdfaf0c38,0xd8490c3a,0xdfaf0c3a,0xd8490c3c,0xdfb10c3c,0xd8490c3e,
	0xdfb10c3e,0xd8490c40,0xdfb30c40,0xd8490c42,0xdfb30c42,0xd8490c44,0xdfb50c44,0xd8490c46,
	0xdfb50c46,0xd8490c48,0xdfb70c48,0xd8490c4a,0xdfb70c4a,0xd8490c4c,0xdfb90c4c,0xd8490c4e,
	0xdfb90c4e,0xd8490c50,0xdfbb0c50,0xd8490c52,0xdfbb0c52,0xd8490c54,0xdfbd0c54,0xd8490c56,
	0xdfbd0c56,0xd8490c58,0xdfbf0c58,0xd8490c5a,0xdfbf0c5a,0xd8490c5c,0xdfc10c5c,0xd8490c5e,
	0xdfc10c5e,0xd8490c60,0xdfc30c60,0xd8490c62,0xdfc30c62,0xd8490c64,0xdfc50c64,0xd8490c66,
	0xdfc50c66,0xd8490c68,0xdfc70c68,0xd8490c6a,0xdfc70c6a,0xd8490c6c,0xdfc90c6c,0xd8490c6e,
	0xdfc90c6e,0xd8490c70,0xdfcb0c70,0xd8490c72,0xdfcb0c72,0xd8490c74,0xdfcd0c74,0xd8490c76,
	0xdfcd0c76,0xd8490c78,0xdfcf0c78,0xd8490c7a,0xdfcf0c7a,0xd8490c7c,0xdfd10c7c,0xd8490c7e,
	0xdfd10c7e,0xd8490c80,0xdfd30c80,0xd8490c82,0xdfd30c82,0xd8490c84,0xdfd50c84,0xd8490c86,
	0xdfd50c86,0xd8490c88,0xdfd70c88,0xd8490c8a,0xdfd70c8a,0xd8490c8c,0xdfd90c8c,0xd8490c8e,
	0xdfd90c8e,0xd8490c90,0xdfdb0c90,0xd8490c92,0xdfdb0c92,0xd8490c94,0xdfdd0c94,0xd8490c96,
	0xdfdd0c96,0xd8490c98,0xdfdf0c98,0xd8490c9a,0xdfdf0c9a,0xd8490c9c,0xdfe10c9c,0xd8490c9e,
	0xdfe10c9e,0xd8490ca0,0xdfe30ca0,0xd8490ca2,0xdfe30ca2,0xd8490ca4,0xdfe50ca4,0xd8490ca6,
	0xdfe50ca6,0xd8490ca8,0xdfe70ca8,0xd8490caa,0xdfe70caa,0xd8490cac,0xdfe90cac,0xd8490cae,
	0xdfe90cae,0xd8490cb0,0xdfeb0cb0,0xd8490cb2,0xdfeb0cb2,0xd8490cb4,0xdfed0cb4,0xd8490cb6,
	0xdfed0cb6,0xd8490cb8,0xdfef0cb8,0xd8490cba,0xdfef0cba,0xd8490cbc,0xdff10cbc,0xd8490cbe,
	0xdff10cbe,0xd8490cc0,0xdff30cc0,0xd8490cc2,0xdff30cc2,0xd8490cc4,0xdff50cc4,0xd8490cc6,
	0xdff50cc6,0xd8490cc8,0xdff70cc8,0xd8490cca,0xdff70cca,0xd8490ccc,0xdff90ccc,0xd8490cce,
	0xdff90cce,0xd8490cd0,0xdffb0cd0,0xd8490cd2,0xdffb0cd2,0xd8490cd4,0xdffd0cd4,0xd8490cd6,
	0xdffd0cd6,0xd8490cd8,0xdfff0cd8,0xd8490cda,0xdfff0cda,0xd84b0cdc,0xdc010cdc,0xd84b0cde,
	0xdc010cde,0xd84b0ce0,0xdc030ce0,0xd84b0ce2,0xdc030ce2,0xd84b0ce4,0xdc050ce4,0xd84b0ce6,
	0xdc050ce6,0xd84b0ce8,0xdc070ce8,0xd84b0cea,0xdc070cea,0xd84b0cec,0xdc090cec,0xd84b0cee,
	0xdc090cee,0xd84b0cf0,0xdc0b0cf0,0xd84b0cf2,0xdc0b0cf2,0xd84b0cf4,0xdc0d0cf4,0xd84b0cf6,
	0xdc0d0cf6,0xd84b0cf8,0xdc0f0cf8,0xd84b0cfa,0xdc0f0cfa,0xd84b0cfc,0xdc110cfc,0xd84b0cfe,
	0xdc110cfe,0xd84b0d00,0xdc130d00,0xd84b0d02,0xdc130d02,0xd84b0d04,0xdc150d04,0xd84b0d06,
	0xdc150d06,0xd84b0d08,0xdc170d08,0xd84b0d0a,0xdc170d0a,0xd84b0d0c,0xdc190d0c,0xd84b0d0e,
	0xdc190d0e,0xd84b0d10,0xdc1b0d10,0xd84b0d12,0xdc1b0d12,0xd84b0d14,0xdc1d0d14,0xd84b0d16,
	0xdc1d0d16,0xd84b0d18,0xdc1f0d18,0xd84b0d1a,0xdc1f0d1a,0xd84b0d1c,0xdc210d1c,0xd84b0d1e,
	0xdc210d1e,0xd84b0d20,0xdc230d20,0xd84b0d22,0xdc230d22,0xd84b0d24,0xdc250d24,0xd84b0d26,
	0xdc250d26,0xd84b0d28,0xdc270d28,0xd84b0d2a,0xdc270d2a,0xd84b0d2c,0xdc290d2c,0xd84b0d2e,
	0xdc290d2e,0xd84b0d30,0xdc2b0d30,0xd84b0d32,0xdc2b0d32,0xd84b0d34,0xdc2d0d34,0xd84b0d36,
	0xdc2d0d36,0xd84b0d38,0xdc2f0d38,0xd84b0d3a,0xdc2f0d3a,0xd84b0d3c,0xdc310d3c,0xd84b0d3e,
	0xdc310d3e,0xd84b0d40,0xdc330d40,0xd84b0d42,0xdc330d42,0xd84b0d44,0xdc350d44,0xd84b0d46,
	0xdc350d46,0xd84b0d48,0xdc370d48,0xd84b0d4a,0xdc370d4a,0xd84b0d4c,0xdc390d4c,0xd84b0d4e,
	0xdc390d4e,0xd84b0d50,0xdc3b0d50,0xd84b0d52,0xdc3b0d52,0xd84b0d54,0xdc3d0d54,0xd84b0d56,
	0xdc3d0d56,0xd84b0d58,0xdc3f0d58,0xd84b0d5a,0xdc3f0d5a,0xd84b0d5c,0xdc410d5c,0xd84b0d5e,
	0xdc410d5e,0xd84b0d60,0xdc430d60,0xd84b0d62,0xdc430d62,0xd84b0d64,0xdc450d64,0xd84b0d66,
	0xdc450d66,0xd84b0d68,0xdc470d68,0xd84b0d6a,0xdc470d6a,0xd84b0d6c,0xdc490d6c,0xd84b0d6e,
	0xdc490d6e,0xd84b0d70,0xdc4b0d70,0xd84b0d72,0xdc4b0d72,0xd84b0d74,0xdc4d0d74,0xd84b0d76,
	0xdc4d0d76,0xd84b0d78,0xdc4f0d78,0xd84b0d7a,0xdc4f0d7a,0xd84b0d7c,0xdc510d7c,0xd84b0d7e,
	0xdc510d7e,0xd84b0d80,0xdc530d80,0xd84b0d82,0xdc530d82,0xd84b0d84,0xdc550d84,0xd84b0d86,
	0xdc550d86,0xd84b0d88,0xdc570d88,0xd84b0d8a,0xdc570d8a,0xd84b0d8c,0xdc590d8c,0xd84b0d8e,
	0xdc590d8e,0xd84b0d90,0xdc5b0d90,0xd84b0d92,0xdc5b0d92,0xd84b0d94,0xdc5d0d94,0xd84b0d96,
	0xdc5d0d96,0xd84b0d98,0xdc5f0d98,0xd84b0d9a,0xdc5f0d9a,0xd84b0d9c,0xdc610d9c,0xd84b0d9e,
	0xdc610d9e,0xd84b0da0,0xdc630da0,0xd84b0da2,0xdc630da2,0xd84b0da4,0xdc650da4,0xd84b0da6,
	0xdc650da6,0xd84b0da8,0xdc670da8,0xd84b0daa,0xdc670daa,0xd84b0dac,0xdc690dac,0xd84b0dae,
	0xdc690dae,0xd84b0db0,0xdc6b0db0,0xd84b0db2,0xdc6b0db2,0xd84b0db4,0xdc6d0db4,0xd84b0db6,
	0xdc6d0db6,0xd84b0db8,0xdc6f0db8,0xd84b0dba,0xdc6f0dba,0xd84b0dbc,0xdc710dbc,0xd84b0dbe,
	0xdc710dbe,0xd84b0dc0,0xdc730dc0,0xd84b0dc2,0xdc730dc2,0xd84b0dc4,0xdc750dc4,0xd84b0dc6,
	0xdc750dc6,0xd84b0dc8,0xdc770dc8,0xd84b0dca,0xdc770dca,0xd84b0dcc,0xdc790dcc,0xd84b0dce,
	0xdc790dce,0xd84b0dd0,0xdc7b0dd0,0xd84b0dd2,0xdc7b0dd2,0xd84b0dd4,0xdc7d0dd4,0xd84b0dd6,
	0xdc7d0dd6,0xd84b0dd8,0xdc7f0dd8,0xd84b0dda,0xdc7f0dda,0xd84b0ddc,0xdc810ddc,0xd84b0dde,
	0xdc810dde,0xd84b0de0,0xdc830de0,0xd84b0de2,0xdc830de2,0xd84b0de4,0xdc850de4,0xd84b0de6,
	0xdc850de6,0xd84b0de8,0xdc870de8,0xd84b0dea,0xdc870dea,0xd84b0dec,0xdc890dec,0xd84b0dee,
	0xdc890dee,0xd84b0df0,0xdc8b0df0,0xd84b0df2,0xdc8b0df2,0xd84b0df4,0xdc8d0df4,0xd84b0df6,
	0xdc8d0df6,0xd84b0df8,0xdc8f0df8,0xd84b0dfa,0xdc8f0dfa,0xd84b0dfc,0xdc910dfc,0xd84b0dfe,
	0xdc910dfe,0xd84b0e00,0xdc930e00,0xd84b0e02,0xdc930e02,0xd84b0e04,0xdc950e04,0xd84b0e06,
	0xdc950e06,0xd84b0e08,0xdc970e08,0xd84b0e0a,0xdc970e0a,0xd84b0e0c,0xdc990e0c,0xd84b0e0e,
	0xdc990e0e,0xd84b0e10,0xdc9b0e10,0xd84b0e12,0xdc9b0e12,0xd84b0e14,0xdc9d0e14,0xd84b0e16,
	0xdc9d0e16,0xd84b0e18,0xdc9f0e18,0xd84b0e1a,0xdc9f0e1a,0xd84b0e1c,0xdca10e1c,0xd84b0e1e,
	0xdca10e1e,0xd84b0e20,0xdca30e20,0xd84b0e22,0xdca30e22,0xd84b0e24,0xdca50e24,0xd84b0e26,
	0xdca50e26,0xd84b0e28,0xdca70e28,0xd84b0e2a,0xdca70e2a,0xd84b0e2c,0xdca90e2c,0xd84b0e2e,
	0xdca90e2e,0xd84b0e30,0xdcab0e30,0xd84b0e32,0xdcab0e32,0xd84b0e34,0xdcad0e34,0xd84b0e36,
	0xdcad0e36,0xd84b0e38,0xdcaf0e38,0xd84b0e3a,0xdcaf0e3a,0xd84b0e3c,0xdcb10e3c,0xd84b0e3e,
	0xdcb10e3e,0xd84b0e40,0xdcb30e40,0xd84b0e42,0xdcb30e42,0xd84b0e44,0xdcb50e44,0xd84b0e46,
	0xdcb50e46,0xd84b0e48,0xdcb70e48,0xd84b0e4a,0xdcb70e4a,0xd84b0e4c,0xdcb90e4c,0xd84b0e4e,
	0xdcb90e4e,0xd84b0e50,0xdcbb0e50,0xd84b0e52,0xdcbb0e52,0xd84b0e54,0xdcbd0e54,0xd84b0e56,
	0xdcbd0e56,0xd84b0e58,0xdcbf0e58,0xd84b0e5a,0xdcbf0e5a,0xd84b0e5c,0xdcc10e5c,0xd84b0e5e,
	0xdcc10e5e,0xd84b0e60,0xdcc30e60,0xd84b0e62,0xdcc30e62,0xd84b0e64,0xdcc50e64,0xd84b0e66,
	0xdcc50e66,0xd84b0e68,0xdcc70e68,0xd84b0e6a,0xdcc70e6a,0xd84b0e6c,0xdcc90e6c,0xd84b0e6e,
	0xdcc90e6e,0xd84b0e70,0xdccb0e70,0xd84b0e72,0xdccb0e72,0xd84b0e74,0xdccd0e74,0xd84b0e76,
	0xdccd0e76,0xd84b0e78,0xdccf0e78,0xd84b0e7a,0xdccf0e7a,0xd84b0e7c,0xdcd10e7c,0xd84b0e7e,
	0xdcd10e7e,0xd84b0e80,0xdcd30e80,0xd84b0e82,0xdcd30e82,0xd84b0e84,0xdcd50e84,0xd84b0e86,
	0xdcd50e86,0xd84b0e88,0xdcd70e88,0xd84b0e8a,0xdcd70e8a,0xd84b0e8c,0xdcd90e8c,0xd84b0e8e,
	0xdcd90e8e,0xd84b0e90,0xdcdb0e90,0xd84b0e92,0xdcdb0e92,0xd84b0e94,0xdcdd0e94,0xd84b0e96,
	0xdcdd0e96,0xd84b0e98,0xdcdf0e98,0xd84b0e9a,0xdcdf0e9a,0xd84b0e9c,0xdce10e9c,0xd84b0e9e,
	0xdce10e9e,0xd84b0ea0,0xdce30ea0,0xd84b0ea2,0xdce30ea2,0xd84b0ea4,0xdce50ea4,0xd84b0ea6,
	0xdce50ea6,0xd84b0ea8,0xdce70ea8,0xd84b0eaa,0xdce70eaa,0xd84b0eac,0xdce90eac,0xd84b0eae,
	0xdce90eae,0xd84b0eb0,0xdceb0eb0,0xd84b0eb2,0xdceb0eb2,0xd84b0eb4,0xdced0eb4,0xd84b0eb6,
	0xdced0eb6,0xd84b0eb8,0xdcef0eb8,0xd84b0eba,0xdcef0eba,0xd84b0ebc,0xdcf10ebc,0xd84b0ebe,
	0xdcf10ebe,0xd84b0ec0,0xdcf30ec0,0xd84b0ec2,0xdcf30ec2,0xd84b0ec4,0xdcf50ec4,0xd84b0ec6,
	0xdcf50ec6,0xd84b0ec8,0xdcf70ec8,0xd84b0eca,0xdcf70eca,0xd84b0ecc,0xdcf90ecc,0xd84b0ece,
	0xdcf90ece,0xd84b0ed0,0xdcfb0ed0,0xd84b0ed2,0xdcfb0ed2,0xd84b0ed4,0xdcfd0ed4,0xd84b0ed6,
	0xdcfd0ed6,0xd84b0ed8,0xdcff0ed8,0xd84b0eda,0xdcff0eda,0xd84b0edc,0xdd010edc,0xd84b0ede,
	0xdd010ede,0xd84b0ee0,0xdd030ee0,0xd84b0ee2,0xdd030ee2,0xd84b0ee4,0xdd050ee4,0xd84b0ee6,
	0xdd050ee6,0xd84b0ee8,0xdd070ee8,0xd84b0eea,0xdd070eea,0xd84b0eec,0xdd090eec,0xd84b0eee,
	0xdd090eee,0xd84b0ef0,0xdd0b0ef0,0xd84b0ef2,0xdd0b0ef2,0xd84b0ef4,0xdd0d0ef4,0xd84b0ef6,
	0xdd0d0ef6,0xd84b0ef8,0xdd0f0ef8,0xd84b0efa,0xdd0f0efa,0xd84b0efc,0xdd110efc,0xd84b0efe,
	0xdd110efe,0xd84b0f00,0xdd130f00,0xd84b0f02,0xdd130f02,0xd84b0f04,0xdd150f04,0xd84b0f06,
	0xdd150f06,0xd84b0f08,0xdd170f08,0xd84b0f0a,0xdd170f0a,0xd84b0f0c,0xdd190f0c,0xd84b0f0e,
	0xdd190f0e,0xd84b0f10,0xdd1b0f10,0xd84b0f12,0xdd1b0f12,0xd84b0f14,0xdd1d0f14,0xd84b0f16,
	0xdd1d0f16,0xd84b0f18,0xdd1f0f18,0xd84b0f1a,0xdd1f0f1a,0xd84b0f1c,0xdd210f1c,0xd84b0f1e,
	0xdd210f1e,0xd84b0f20,0xdd230f20,0xd84b0f22,0xdd230f22,0xd84b0f24,0xdd250f24,0xd84b0f26,
	0xdd250f26,0xd84b0f28,0xdd270f28,0xd84b0f2a,0xdd270f2a,0xd84b0f2c,0xdd290f2c,0xd84b0f2e,
	0xdd290f2e,0xd84b0f30,0xdd2b0f30,0xd84b0f32,0xdd2b0f32,0xd84b0f34,0xdd2d0f34,0xd84b0f36,
	0xdd2d0f36,0xd84b0f38,0xdd2f0f38,0xd84b0f3a,0xdd2f0f3a,0xd84b0f3c,0xdd310f3c,0xd84b0f3e,
	0xdd310f3e,0xd84b0f40,0xdd330f40,0xd84b0f42,0xdd330f42,0xd84b0f44,0xdd350f44,0xd84b0f46,
	0xdd350f46,0xd84b0f48,0xdd370f48,0xd84b0f4a,0xdd370f4a,0xd84b0f4c,0xdd390f4c,0xd84b0f4e,
	0xdd390f4e,0xd84b0f50,0xdd3b0f50,0xd84b0f52,0xdd3b0f52,0xd84b0f54,0xdd3d0f54,0xd84b0f56,
	0xdd3d0f56,0xd84b0f58,0xdd3f0f58,0xd84b0f5a,0xdd3f0f5a,0xd84b0f5c,0xdd410f5c,0xd84b0f5e,
	0xdd410f5e,0xd84b0f60,0xdd430f60,0xd84b0f62,0xdd430f62,0xd84b0f64,0xdd450f64,0xd84b0f66,
	0xdd450f66,0xd84b0f68,0xdd470f68,0xd84b0f6a,0xdd470f6a,0xd84b0f6c,0xdd490f6c,0xd84b0f6e,
	0xdd490f6e,0xd84b0f70,0xdd4b0f70,0xd84b0f72,0xdd4b0f72,0xd84b0f74,0xdd4d0f74,0xd84b0f76,
	0xdd4d0f76,0xd84b0f78,0xdd4f0f78,0xd84b0f7a,0xdd4f0f7a,0xd84b0f7c,0xdd510f7c,0xd84b0f7e,
	0xdd510f7e,0xd84b0f80,0xdd530f80,0xd84b0f82,0xdd530f82,0xd84b0f84,0xdd550f84,0xd84b0f86,
	0xdd550f86,0xd84b0f88,0xdd570f88,0xd84b0f8a,0xdd570f8a,0xd84b0f8c,0xdd590f8c,0xd84b0f8e,
	0xdd590f8e,0xd84b0f90,0xdd5b0f90,0xd84b0f92,0xdd5b0f92,0xd84b0f94,0xdd5d0f94,0xd84b0f96,
	0xdd5d0f96,0xd84b0f98,0xdd5f0f98,0xd84b0f9a,0xdd5f0f9a,0xd84b0f9c,0xdd610f9c,0xd84b0f9e,
	0xdd610f9e,0xd84b0fa0,0xdd630fa0,0xd84b0fa2,0xdd630fa2,0xd84b0fa4,0xdd650fa4,0xd84b0fa6,
	0xdd650fa6,0xd84b0fa8,0xdd670fa8,0xd84b0faa,0xdd670faa,0xd84b0fac,0xdd690fac,0xd84b0fae,
	0xdd690fae,0xd84b0fb0,0xdd6b0fb0,0xd84b0fb2,0xdd6b0fb2,0xd84b0fb4,0xdd6d0fb4,0xd84b0fb6,
	0xdd6d0fb6,0xd84b0fb8,0xdd6f0fb8,0xd84b0fba,0xdd6f0fba,0xd84b0fbc,0xdd710fbc,0xd84b0fbe,
	0xdd710fbe,0xd84b0fc0,0xdd730fc0,0xd84b0fc2,0xdd730fc2,0xd84b0fc4,0xdd750fc4,0xd84b0fc6,
	0xdd750fc6,0xd84b0fc8,0xdd770fc8,0xd84b0fca,0xdd770fca,0xd84b0fcc,0xdd790fcc,0xd84b0fce,
	0xdd790fce,0xd84b0fd0,0xdd7b0fd0,0xd84b0fd2,0xdd7b0fd2,0xd84b0fd4,0xdd7d0fd4,0xd84b0fd6,
	0xdd7d0fd6,0xd84b0fd8,0xdd7f0fd8,0xd84b0fda,0xdd7f0fda,0xd84b0fdc,0xdd810fdc,0xd84b0fde,
	0xdd810fde,0xd84b0fe0,0xdd830fe0,0xd84b0fe2,0xdd830fe2,0xd84b0fe4,0xdd850fe4,0xd84b0fe6,
	0xdd850fe6,0xd84b0fe8,0xdd870fe8,0xd84b0fea,0xdd870fea,0xd84b0fec,0xdd890fec,0xd84b0fee,
	0xdd890fee,0xd84b0ff0,0xdd8b0ff0,0xd84b0ff2,0xdd8b0ff2,0xd84b0ff4,0xdd8d0ff4,0xd84b0ff6,
	0xdd8d0ff6,0xd84b0ff8,0xdd8f0ff8,0xd84b0ffa,0xdd8f0ffa,0xd84b0ffc,0xdd910ffc,0xd84b0ffe,
	0xdd910ffe,0xd84b1000,0xdd931000,0xd84b1002,0xdd931002,0xd84b1004,0xdd951004,0xd84b1006,
	0xdd951006,0xd84b1008,0xdd971008,0xd84b100a,0xdd97100a,0xd84b100c,0xdd99100c,0xd84b100e,
	0xdd99100e,0xd84b1010,0xdd9b1010,0xd84b1012,0xdd9b1012,0xd84b1014,0xdd9d1014,0xd84b1016,
	0xdd9d1016,0xd84b1018,0xdd9f1018,0xd84b101a,0xdd9f101a,0xd84b101c,0xdda1101c,0xd84b101e,
	0xdda1101e,0xd84b1020,0xdda31020,0xd84b1022,0xdda31022,0xd84b1024,0xdda51024,0xd84b1026,
	0xdda51026,0xd84b1028,0xdda71028,0xd84b102a,0xdda7102a,0xd84b102c,0xdda9102c,0xd84b102e,
	0xdda9102e,0xd84b1030,0xddab1030,0xd84b1032,0xddab1032,0xd84b1034,0xddad1034,0xd84b1036,
	0xddad1036,0xd84b1038,0xddaf1038,0xd84b103a,0xddaf103a,0xd84b103c,0xddb1103c,0xd84b103e,
	0xddb1103e,0xd84b1040,0xddb31040,0xd84b1042,0xddb31042,0xd84b1044,0xddb51044,0xd84b1046,
	0xddb51046,0xd84b1048,0xddb71048,0xd84b104a,0xddb7104a,0xd84b104c,0xddb9104c,0xd84b104e,
	0xddb9104e,0xd84b1050,0xddbb1050,0xd84b1052,0xddbb1052,0xd84b1054,0xddbd1054,0xd84b1056,
	0xddbd1056,0xd84b1058,0xddbf1058,0xd84b105a,0xddbf105a,0xd84b105c,0xddc1105c,0xd84b105e,
	0xddc1105e,0xd84b1060,0xddc31060,0xd84b1062,0xddc31062,0xd84b1064,0xddc51064,0xd84b1066,
	0xddc51066,0xd84b1068,0xddc71068,0xd84b106a,0xddc7106a,0xd84b106c,0xddc9106c,0xd84b106e,
	0xddc9106e,0xd84b1070,0xddcb1070,0xd84b1072,0xddcb1072,0xd84b1074,0xddcd1074,0xd84b1076,
	0xddcd1076,0xd84b1078,0xddcf1078,0xd84b107a,0xddcf107a,0xd84b107c,0xddd1107c,0xd84b107e,
	0xddd1107e,0xd84b1080,0xddd31080,0xd84b1082,0xddd31082,0xd84b1084,0xddd51084,0xd84b1086,
	0xddd51086,0xd84b1088,0xddd71088,0xd84b108a,0xddd7108a,0xd84b108c,0xddd9108c,0xd84b108e,
	0xddd9108e,0xd84b1090,0xdddb1090,0xd84b1092,0xdddb1092,0xd84b1094,0xdddd1094,0xd84b1096,
	0xdddd1096,0xd84b1098,0xdddf1098,0xd84b109a,0xdddf109a,0xd84b109c,0xdde1109c,0xd84b109e,
	0xdde1109e,0xd84b10a0,0xdde310a0,0xd84b10a2,0xdde310a2,0xd84b10a4,0xdde510a4,0xd84b10a6,
	0xdde510a6,0xd84b10a8,0xdde710a8,0xd84b10aa,0xdde710aa,0xd84b10ac,0xdde910ac,0xd84b10ae,
	0xdde910ae,0xd84b10b0,0xddeb10b0,0xd84b10b2,0xddeb10b2,0xd84b10b4,0xdded10b4,0xd84b10b6,
	0xdded10b6,0xd84b10b8,0xddef10b8,0xd84b10ba,0xddef10ba,0xd84b10bc,0xddf110bc,0xd84b10be,
	0xddf110be,0xd84b10c0,0xddf310c0,0xd84b10c2,0xddf310c2,0xd84b10c4,0xddf510c4,0xd84b10c6,
	0xddf510c6,0xd84b10c8,0xddf710c8,0xd84b10ca,0xddf710ca,0xd84b10cc,0xddf910cc,0xd84b10ce,
	0xddf910ce,0xd84b10d0,0xddfb10d0,0xd84b10d2,0xddfb10d2,0xd84b10d4,0xddfd10d4,0xd84b10d6,
	0xddfd10d6,0xd84b10d8,0xddff10d8,0xd84b10da,0xddff10da,0xd84b10dc,0xde0110dc,0xd84b10de,
	0xde0110de,0xd84b10e0,0xde0310e0,0xd84b10e2,0xde0310e2,0xd84b10e4,0xde0510e4,0xd84b10e6,
	0xde0510e6,0xd84b10e8,0xde0710e8,0xd84b10ea,0xde0710ea,0xd84b10ec,0xde0910ec,0xd84b10ee,
	0xde0910ee,0xd84b10f0,0xde0b10f0,0xd84b10f2,0xde0b10f2,0xd84b10f4,0xde0d10f4,0xd84b10f6,
	0xde0d10f6,0xd84b10f8,0xde0f10f8,0xd84b10fa,0xde0f10fa,0xd84b10fc,0xde1110fc,0xd84b10fe,
	0xde1110fe,0xd84b1100,0xde131100,0xd84b1102,0xde131102,0xd84b1104,0xde151104,0xd84b1106,
	0xde151106,0xd84b1108,0xde171108,0xd84b110a,0xde17110a,0xd84b110c,0xde19110c,0xd84b110e,
	0xde19110e,0xd84b1110,0xde1b1110,0xd84b1112,0xde1b1112,0xd84b1114,0xde1d1114,0xd84b1116,
	0xde1d1116,0xd84b1118,0xde1f1118,0xd84b111a,0xde1f111a,0xd84b111c,0xde21111c,0xd84b111e,
	0xde21111e,0xd84b1120,0xde231120,0xd84b1122,0xde231122,0xd84b1124,0xde251124,0xd84b1126,
	0xde251126,0xd84b1128,0xde271128,0xd84b112a,0xde27112a,0xd84b112c,0xde29112c,0xd84b112e,
	0xde29112e,0xd84b1130,0xde2b1130,0xd84b1132,0xde2b1132,0xd84b1134,0xde2d1134,0xd84b1136,
	0xde2d1136,0xd84b1138,0xde2f1138,0xd84b113a,0xde2f113a,0xd84b113c,0xde31113c,0xd84b113e,
	0xde31113e,0xd84b1140,0xde331140,0xd84b1142,0xde331142,0xd84b1144,0xde351144,0xd84b1146,
	0xde351146,0xd84b1148,0xde371148,0xd84b114a,0xde37114a,0xd84b114c,0xde39114c,0xd84b114e,
	0xde39114e,0xd84b1150,0xde3b1150,0xd84b1152,0xde3b1152,0xd84b1154,0xde3d1154,0xd84b1156,
	0xde3d1156,0xd84b1158,0xde3f1158,0xd84b115a,0xde3f115a,0xd84b115c,0xde41115c,0xd84b115e,
	0xde41115e,0xd84b1160,0xde431160,0xd84b1162,0xde431162,0xd84b1164,0xde451164,0xd84b1166,
	0xde451166,0xd84b1168,0xde471168,0xd84b116a,0xde47116a,0xd84b116c,0xde49116c,0xd84b116e,
	0xde49116e,0xd84b1170,0xde4b1170,0xd84b1172,0xde4b1172,0xd84b1174,0xde4d1174,0xd84b1176,
	0xde4d1176,0xd84b1178,0xde4f1178,0xd84b117a,0xde4f117a,0xd84b117c,0xde51117c,0xd84b117e,
	0xde51117e,0xd84b1180,0xde531180,0xd84b1182,0xde531182,0xd84b1184,0xde551184,0xd84b1186,
	0xde551186,0xd84b1188,0xde571188,0xd84b118a,0xde57118a,0xd84b118c,0xde59118c,0xd84b118e,
	0xde59118e,0xd84b1190,0xde5b1190,0xd84b1192,0xde5b1192,0xd84b1194,0xde5d1194,0xd84b1196,
	0xde5d1196,0xd84b1198,0xde5f1198,0xd84b119a,0xde5f119a,0xd84b119c,0xde61119c,0xd84b119e,
	0xde61119e,0xd84b11a0,0xde6311a0,0xd84b11a2,0xde6311a2,0xd84b11a4,0xde6511a4,0xd84b11a6,
	0xde6511a6,0xd84b11a8,0xde6711a8,0xd84b11aa,0xde6711aa,0xd84b11ac,0xde6911ac,0xd84b11ae,
	0xde6911ae,0xd84b11b0,0xde6b11b0,0xd84b11b2,0xde6b11b2,0xd84b11b4,0xde6d11b4,0xd84b11b6,
	0xde6d11b6,0xd84b11b8,0xde6f11b8,0xd84b11ba,0xde6f11ba,0xd84b11bc,0xde7111bc,0xd84b11be,
	0xde7111be,0xd84b11c0,0xde7311c0,0xd84b11c2,0xde7311c2,0xd84b11c4,0xde7511c4,0xd84b11c6,
	0xde7511c6,0xd84b11c8,0xde7711c8,0xd84b11ca,0xde7711ca,0xd84b11cc,0xde7911cc,0xd84b11ce,
	0xde7911ce,0xd84b11d0,0xde7b11d0,0xd84b11d2,0xde7b11d2,0xd84b11d4,0xde7d11d4,0xd84b11d6,
	0xde7d11d6,0xd84b11d8,0xde7f11d8,0xd84b11da,0xde7f11da,0xd84b11dc,0xde8111dc,0xd84b11de,
	0xde8111de,0xd84b11e0,0xde8311e0,0xd84b11e2,0xde8311e2,0xd84b11e4,0xde8511e4,0xd84b11e6,
	0xde8511e6,0xd84b11e8,0xde8711e8,0xd84b11ea,0xde8711ea,0xd84b11ec,0xde8911ec,0xd84b11ee,
	0xde8911ee,0xd84b11f0,0xde8b11f0,0xd84b11f2,0xde8b11f2,0xd84b11f4,0xde8d11f4,0xd84b11f6,
	0xde8d11f6,0xd84b11f8,0xde8f11f8,0xd84b11fa,0xde8f11fa,0xd84b11fc,0xde9111fc,0xd84b11fe,
	0xde9111fe,0xd84b1200,0xde931200,0xd84b1202,0xde931202,0xd84b1204,0xde951204,0xd84b1206,
	0xde951206,0xd84b1208,0xde971208,0xd84b120a,0xde97120a,0xd84b120c,0xde99120c,0xd84b120e,
	0xde99120e,0xd84b1210,0xde9b1210,0xd84b1212,0xde9b1212,0xd84b1214,0xde9d1214,0xd84b1216,
	0xde9d1216,0xd84b1218,0xde9f1218,0xd84b121a,0xde9f121a,0xd84b121c,0xdea1121c,0xd84b121e,
	0xdea1121e,0xd84b1220,0xdea31220,0xd84b1222,0xdea31222,0xd84b1224,0xdea51224,0xd84b1226,
	0xdea51226,0xd84b1228,0xdea71228,0xd84b122a,0xdea7122a,0xd84b122c,0xdea9122c,0xd84b122e,
	0xdea9122e,0xd84b1230,0xdeab1230,0xd84b1232,0xdeab1232,0xd84b1234,0xdead1234,0xd84b1236,
	0xdead1236,0xd84b1238,0xdeaf1238,0xd84b123a,0xdeaf123a,0xd84b123c,0xdeb1123c,0xd84b123e,
	0xdeb1123e,0xd84b1240,0xdeb31240,0xd84b1242,0xdeb31242,0xd84b1244,0xdeb51244,0xd84b1246,
	0xdeb51246,0xd84b1248,0xdeb71248,0xd84b124a,0xdeb7124a,0xd84b124c,0xdeb9124c,0xd84b124e,
	0xdeb9124e,0xd84b1250,0xdebb1250,0xd84b1252,0xdebb1252,0xd84b1254,0xdebd1254,0xd84b1256,
	0xdebd1256,0xd84b1258,0xdebf1258,0xd84b125a,0xdebf125a,0xd84b125c,0xdec1125c,0xd84b125e,
	0xdec1125e,0xd84b1260,0xdec31260,0xd84b1262,0xdec31262,0xd84b1264,0xdec51264,0xd84b1266,
	0xdec51266,0xd84b1268,0xdec71268,0xd84b126a,0xdec7126a,0xd84b126c,0xdec9126c,0xd84b126e,
	0xdec9126e,0xd84b1270,0xdecb1270,0xd84b1272,0xdecb1272,0xd84b1274,0xdecd1274,0xd84b1276,
	0xdecd1276,0xd84b1278,0xdecf1278,0xd84b127a,0xdecf127a,0xd84b127c,0xded1127c,0xd84b127e,
	0xded1127e,0xd84b1280,0xded31280,0xd84b1282,0xded31282,0xd84b1284,0xded51284,0xd84b1286,
	0xded51286,0xd84b1288,0xded71288,0xd84b128a,0xded7128a,0xd84b128c,0xded9128c,0xd84b128e,
	0xded9128e,0xd84b1290,0xdedb1290,0xd84b1292,0xdedb1292,0xd84b1294,0xdedd1294,0xd84b1296,
	0xdedd1296,0xd84b1298,0xdedf1298,0xd84b129a,0xdedf129a,0xd84b129c,0xdee1129c,0xd84b129e,
	0xdee1129e,0xd84b12a0,0xdee312a0,0xd84b12a2,0xdee312a2,0xd84b12a4,0xdee512a4,0xd84b12a6,
	0xdee512a6,0xd84b12a8,0xdee712a8,0xd84b12aa,0xdee712aa,0xd84b12ac,0xdee912ac,0xd84b12ae,
	0xdee912ae,0xd84b12b0,0xdeeb12b0,0xd84b12b2,0xdeeb12b2,0xd84b12b4,0xdeed12b4,0xd84b12b6,
	0xdeed12b6,0xd84b12b8,0xdeef12b8,0xd84b12ba,0xdeef12ba,0xd84b12bc,0xdef112bc,0xd84b12be,
	0xdef112be,0xd84b12c0,0xdef312c0,0xd84b12c2,0xdef312c2,0xd84b12c4,0xdef512c4,0xd84b12c6,
	0xdef512c6,0xd84b12c8,0xdef712c8,0xd84b12ca,0xdef712ca,0xd84b12cc,0xdef912cc,0xd84b12ce,
	0xdef912ce,0xd84b12d0,0xdefb12d0,0xd84b12d2,0xdefb12d2,0xd84b12d4,0xdefd12d4,0xd84b12d6,
	0xdefd12d6,0xd84b12d8,0xdeff12d8,0xd84b12da,0xdeff12da,0xd84b12dc,0xdf0112dc,0xd84b12de,
	0xdf0112de,0xd84b12e0,0xdf0312e0,0xd84b12e2,0xdf0312e2,0xd84b12e4,0xdf0512e4,0xd84b12e6,
	0xdf0512e6,0xd84b12e8,0xdf0712e8,0xd84b12ea,0xdf0712ea,0xd84b12ec,0xdf0912ec,0xd84b12ee,
	0xdf0912ee,0xd84b12f0,0xdf0b12f0,0xd84b12f2,0xdf0b12f2,0xd84b12f4,0xdf0d12f4,0xd84b12f6,
	0xdf0d12f6,0xd84b12f8,0xdf0f12f8,0xd84b12fa,0xdf0f12fa,0xd84b12fc,0xdf1112fc,0xd84b12fe,
	0xdf1112fe,0xd84b1300,0xdf131300,0xd84b1302,0xdf131302,0xd84b1304,0xdf151304,0xd84b1306,
	0xdf151306,0xd84b1308,0xdf171308,0xd84b130a,0xdf17130a,0xd84b130c,0xdf19130c,0xd84b130e,
	0xdf19130e,0xd84b1310,0xdf1b1310,0xd84b1312,0xdf1b1312,0xd84b1314,0xdf1d1314,0xd84b1316,
	0xdf1d1316,0xd84b1318,0xdf1f1318,0xd84b131a,0xdf1f131a,0xd84b131c,0xdf21131c,0xd84b131e,
	0xdf21131e,0xd84b1320,0xdf231320,0xd84b1322,0xdf231322,0xd84b1324,0xdf251324,0xd84b1326,
	0xdf251326,0xd84b1328,0xdf271328,0xd84b132a,0xdf27132a,0xd84b132c,0xdf29132c,0xd84b132e,
	0xdf29132e,0xd84b1330,0xdf2b1330,0xd84b1332,0xdf2b1332,0xd84b1334,0xdf2d1334,0xd84b1336,
	0xdf2d1336,0xd84b1338,0xdf2f1338,0xd84b133a,0xdf2f133a,0xd84b133c,0xdf31133c,0xd84b133e,
	0xdf31133e,0xd84b1340,0xdf331340,0xd84b1342,0xdf331342,0xd84b1344,0xdf351344,0xd84b1346,
	0xdf351346,0xd84b1348,0xdf371348,0xd84b134a,0xdf37134a,0xd84b134c,0xdf39134c,0xd84b134e,
	0xdf39134e,0xd84b1350,0xdf3b1350,0xd84b1352,0xdf3b1352,0xd84b1354,0xdf3d1354,0xd84b1356,
	0xdf3d1356,0xd84b1358,0xdf3f1358,0xd84b135a,0xdf3f135a,0xd84b135c,0xdf41135c,0xd84b135e,
	0xdf41135e,0xd84b1360,0xdf431360,0xd84b1362,0xdf431362,0xd84b1364,0xdf451364,0xd84b1366,
	0xdf451366,0xd84b1368,0xdf471368,0xd84b136a,0xdf47136a,0xd84b136c,0xdf49136c,0xd84b136e,
	0xdf49136e,0xd84b1370,0xdf4b1370,0xd84b1372,0xdf4b1372,0xd84b1374,0xdf4d1374,0xd84b1376,
	0xdf4d1376,0xd84b1378,0xdf4f1378,0xd84b137a,0xdf4f137a,0xd84b137c,0xdf51137c,0xd84b137e,
	0xdf51137e,0xd84b1380,0xdf531380,0xd84b1382,0xdf531382,0xd84b1384,0xdf551384,0xd84b1386,
	0xdf551386,0xd84b1388,0xdf571388,0xd84b138a,0xdf57138a,0xd84b138c,0xdf59138c,0xd84b138e,
	0xdf59138e,0xd84b1390,0xdf5b1390,0xd84b1392,0xdf5b1392,0xd84b1394,0xdf5d1394,0xd84b1396,
	0xdf5d1396,0xd84b1398,0xdf5f1398,0xd84b139a,0xdf5f139a,0xd84b139c,0xdf61139c,0xd84b139e,
	0xdf61139e,0xd84b13a0,0xdf6313a0,0xd84b13a2,0xdf6313a2,0xd84b13a4,0xdf6513a4,0xd84b13a6,
	0xdf6513a6,0xd84b13a8,0xdf6713a8,0xd84b13aa,0xdf6713aa,0xd84b13ac,0xdf6913ac,0xd84b13ae,
	0xdf6913ae,0xd84b13b0,0xdf6b13b0,0xd84b13b2,0xdf6b13b2,0xd84b13b4,0xdf6d13b4,0xd84b13b6,
	0xdf6d13b6,0xd84b13b8,0xdf6f13b8,0xd84b13ba,0xdf6f13ba,0xd84b13bc,0xdf7113bc,0xd84b13be,
	0xdf7113be,0xd84b13c0,0xdf7313c0,0xd84b13c2,0xdf7313c2,0xd84b13c4,0xdf7513c4,0xd84b13c6,
	0xdf7513c6,0xd84b13c8,0xdf7713c8,0xd84b13ca,0xdf7713ca,0xd84b13cc,0xdf7913cc,0xd84b13ce,
	0xdf7913ce,0xd84b13d0,0xdf7b13d0,0xd84b13d2,0xdf7b13d2,0xd84b13d4,0xdf7d13d4,0xd84b13d6,
	0xdf7d13d6,0xd84b13d8,0xdf7f13d8,0xd84b13da,0xdf7f13da,0xd84b13dc,0xdf8113dc,0xd84b13de,
	0xdf8113de,0xd84b13e0,0xdf8313e0,0xd84b13e2,0xdf8313e2,0xd84b13e4,0xdf8513e4,0xd84b13e6,
	0xdf8513e6,0xd84b13e8,0xdf8713e8,0xd84b13ea,0xdf8713ea,0xd84b13ec,0xdf8913ec,0xd84b13ee,
	0xdf8913ee,0xd84b13f0,0xdf8b13f0,0xd84b13f2,0xdf8b13f2,0xd84b13f4,0xdf8d13f4,0xd84b13f6,
	0xdf8d13f6,0xd84b13f8,0xdf8f13f8,0xd84b13fa,0xdf8f13fa,0xd84b13fc,0xdf9113fc,0xd84b13fe,
	0xdf9113fe,0xd84b1400,0xdf931400,0xd84b1402,0xdf931402,0xd84b1404,0xdf951404,0xd84b1406,
	0xdf951406,0xd84b1408,0xdf971408,0xd84b140a,0xdf97140a,0xd84b140c,0xdf99140c,0xd84b140e,
	0xdf99140e,0xd84b1410,0xdf9b1410,0xd84b1412,0xdf9b1412,0xd84b1414,0xdf9d1414,0xd84b1416,
	0xdf9d1416,0xd84b1418,0xdf9f1418,0xd84b141a,0xdf9f141a,0xd84b141c,0xdfa1141c,0xd84b141e,
	0xdfa1141e,0xd84b1420,0xdfa31420,0xd84b1422,0xdfa31422,0xd84b1424,0xdfa51424,0xd84b1426,
	0xdfa51426,0xd84b1428,0xdfa71428,0xd84b142a,0xdfa7142a,0xd84b142c,0xdfa9142c,0xd84b142e,
	0xdfa9142e,0xd84b1430,0xdfab1430,0xd84b1432,0xdfab1432,0xd84b1434,0xdfad1434,0xd84b1436,
	0xdfad1436,0xd84b1438,0xdfaf1438,0xd84b143a,0xdfaf143a,0xd84b143c,0xdfb1143c,0xd84b143e,
	0xdfb1143e,0xd84b1440,0xdfb31440,0xd84b1442,0xdfb31442,0xd84b1444,0xdfb51444,0xd84b1446,
	0xdfb51446,0xd84b1448,0xdfb71448,0xd84b144a,0xdfb7144a,0xd84b144c,0xdfb9144c,0xd84b144e,
	0xdfb9144e,0xd84b1450,0xdfbb1450,0xd84b1452,0xdfbb1452,0xd84b1454,0xdfbd1454,0xd84b1456,
	0xdfbd1456,0xd84b1458,0xdfbf1458,0xd84b145a,0xdfbf145a,0xd84b145c,0xdfc1145c,0xd84b145e,
	0xdfc1145e,0xd84b1460,0xdfc31460,0xd84b1462,0xdfc31462,0xd84b1464,0xdfc51464,0xd84b1466,
	0xdfc51466,0xd84b1468,0xdfc71468,0xd84b146a,0xdfc7146a,0xd84b146c,0xdfc9146c,0xd84b146e,
	0xdfc9146e,0xd84b1470,0xdfcb1470,0xd84b1472,0xdfcb1472,0xd84b1474,0xdfcd1474,0xd84b1476,
	0xdfcd1476,0xd84b1478,0xdfcf1478,0xd84b147a,0xdfcf147a,0xd84b147c,0xdfd1147c,0xd84b147e,
	0xdfd1147e,0xd84b1480,0xdfd31480,0xd84b1482,0xdfd31482,0xd84b1484,0xdfd51484,0xd84b1486,
	0xdfd51486,0xd84b1488,0xdfd71488,0xd84b148a,0xdfd7148a,0xd84b148c,0xdfd9148c,0xd84b148e,
	0xdfd9148e,0xd84b1490,0xdfdb1490,0xd84b1492,0xdfdb1492,0xd84b1494,0xdfdd1494,0xd84b1496,
	0xdfdd1496,0xd84b1498,0xdfdf1498,0xd84b149a,0xdfdf149a,0xd84b149c,0xdfe1149c,0xd84b149e,
	0xdfe1149e,0xd84b14a0,0xdfe314a0,0xd84b14a2,0xdfe314a2,0xd84b14a4,0xdfe514a4,0xd84b14a6,
	0xdfe514a6,0xd84b14a8,0xdfe714a8,0xd84b14aa,0xdfe714aa,0xd84b14ac,0xdfe914ac,0xd84b14ae,
	0xdfe914ae,0xd84b14b0,0xdfeb14b0,0xd84b14b2,0xdfeb14b2,0xd84b14b4,0xdfed14b4,0xd84b14b6,
	0xdfed14b6,0xd84b14b8,0xdfef14b8,0xd84b14ba,0xdfef14ba,0xd84b14bc,0xdff114bc,0xd84b14be,
	0xdff114be,0xd84b14c0,0xdff314c0,0xd84b14c2,0xdff314c2,0xd84b14c4,0xdff514c4,0xd84b14c6,
	0xdff514c6,0xd84b14c8,0xdff714c8,0xd84b14ca,0xdff714ca,0xd84b14cc,0xdff914cc,0xd84b14ce,
	0xdff914ce,0xd84b14d0,0xdffb14d0,0xd84b14d2,0xdffb14d2,0xd84b14d4,0xdffd14d4,0xd84b14d6,
	0xdffd14d6,0xd84b14d8,0xdfff14d8,0xd84b14da,0xdfff14da,0xd84b14dc,0xdc0114dc,0xd84b14de,
	0xdc0114de,0xd84b14e0,0xdc0314e0,0xd84b14e2,0xdc0314e2,0xd84b14e4,0xdc0514e4,0xd84b14e6,
	0xdc0514e6,0xd84b14e8,0xdc0714e8,0xd84b14ea,0xdc0714ea,0xd84b14ec,0xdc0914ec,0xd84b14ee,
	0xdc0914ee,0xd84b14f0,0xdc0b14f0,0xd84b14f2,0xdc0b14f2,0xd84b14f4,0xdc0d14f4,0xd84b14f6,
	0xdc0d14f6,0xd84b14f8,0xdc0f14f8,0xd84b14fa,0xdc0f14fa,0xd84b14fc,0xdc1114fc,0xd84b14fe,
	0xdc1114fe,0xd84b1500,0xdc131500,0xd84b1502,0xdc131502,0xd84b1504,0xdc151504,0xd84b1506,
	0xdc151506,0xd84b1508,0xdc171508,0xd84b150a,0xdc17150a,0xd84b150c,0xdc19150c,0xd84b150e,
	0xdc19150e,0xd84b1510,0xdc1b1510,0xd84b1512,0xdc1b1512,0xd84b1514,0xdc1d1514,0xd84b1516,
	0xdc1d1516,0xd84b1518,0xdc1f1518,0xd84b151a,0xdc1f151a,0xd84b151c,0xdc21151c,0xd84b151e,
	0xdc21151e,0xd84b1520,0xdc231520,0xd84b1522,0xdc231522,0xd84b1524,0xdc251524,0xd84b1526,
	0xdc251526,0xd84b1528,0xdc271528,0xd84b152a,0xdc27152a,0xd84b152c,0xdc29152c,0xd84b152e,
	0xdc29152e,0xd84b1530,0xdc2b1530,0xd84b1532,0xdc2b1532,0xd84b1534,0xdc2d1534,0xd84b1536,
	0xdc2d1536,0xd84b1538,0xdc2f1538,0xd84b153a,0xdc2f153a,0xd84b153c,0xdc31153c,0xd84b153e,
	0xdc31153e,0xd84b1540,0xdc331540,0xd84b1542,0xdc331542,0xd84b1544,0xdc351544,0xd84b1546,
	0xdc351546,0xd84b1548,0xdc371548,0xd84b154a,0xdc37154a,0xd84b154c,0xdc39154c,0xd84b154e,
	0xdc39154e,0xd84b1550,0xdc3b1550,0xd84b1552,0xdc3b1552,0xd84b1554,0xdc3d1554,0xd84b1556,
	0xdc3d1556,0xd84b1558,0xdc3f1558,0xd84b155a,0xdc3f155a,0xd84b155c,0xdc41155c,0xd84b155e,
	0xdc41155e,0xd84b1560,0xdc431560,0xd84b1562,0xdc431562,0xd84b1564,0xdc451564,0xd84b1566,
	0xdc451566,0xd84b1568,0xdc471568,0xd84b156a,0xdc47156a,0xd84b156c,0xdc49156c,0xd84b156e,
	0xdc49156e,0xd84b1570,0xdc4b1570,0xd84b1572,0xdc4b1572,0xd84b1574,0xdc4d1574,0xd84b1576,
	0xdc4d1576,0xd84b1578,0xdc4f1578,0xd84b157a,0xdc4f157a,0xd84b157c,0xdc51157c,0xd84b157e,
	0xdc51157e,0xd84b1580,0xdc531580,0xd84b1582,0xdc531582,0xd84b1584,0xdc551584,0xd84b1586,
	0xdc551586,0xd84b1588,0xdc571588,0xd84b158a,0xdc57158a,0xd84b158c,0xdc59158c,0xd84b158e,
	0xdc59158e,0xd84b1590,0xdc5b1590,0xd84b1592,0xdc5b1592,0xd84b1594,0xdc5d1594,0xd84b1596,
	0xdc5d1596,0xd84b1598,0xdc5f1598,0xd84b159a,0xdc5f159a,0xd84b159c,0xdc61159c,0xd84b159e,
	0xdc61159e,0xd84b15a0,0xdc6315a0,0xd84b15a2,0xdc6315a2,0xd84b15a4,0xdc6515a4,0xd84b15a6,
	0xdc6515a6,0xd84b15a8,0xdc6715a8,0xd84b15aa,0xdc6715aa,0xd84b15ac,0xdc6915ac,0xd84b15ae,
	0xdc6915ae,0xd84b15b0,0xdc6b15b0,0xd84b15b2,0xdc6b15b2,0xd84b15b4,0xdc6d15b4,0xd84b15b6,
	0xdc6d15b6,0xd84b15b8,0xdc6f15b8,0xd84b15ba,0xdc6f15ba,0xd84b15bc,0xdc7115bc,0xd84b15be,
	0xdc7115be,0xd84b15c0,0xdc7315c0,0xd84b15c2,0xdc7315c2,0xd84b15c4,0xdc7515c4,0xd84b15c6,
	0xdc7515c6,0xd84b15c8,0xdc7715c8,0xd84b15ca,0xdc7715ca,0xd84b15cc,0xdc7915cc,0xd84b15ce,
	0xdc7915ce,0xd84b15d0,0xdc7b15d0,0xd84b15d2,0xdc7b15d2,0xd84b15d4,0xdc7d15d4,0xd84b15d6,
	0xdc7d15d6,0xd84b15d8,0xdc7f15d8,0xd84b15da,0xdc7f15da,0xd84b15dc,0xdc8115dc,0xd84b15de,
	0xdc8115de,0xd84b15e0,0xdc8315e0,0xd84b15e2,0xdc8315e2,0xd84b15e4,0xdc8515e4,0xd84b15e6,
	0xdc8515e6,0xd84b15e8,0xdc8715e8,0xd84b15ea,0xdc8715ea,0xd84b15ec,0xdc8915ec,0xd84b15ee,
	0xdc8915ee,0xd84b15f0,0xdc8b15f0,0xd84b15f2,0xdc8b15f2,0xd84b15f4,0xdc8d15f4,0xd84b15f6,
	0xdc8d15f6,0xd84b15f8,0xdc8f15f8,0xd84b15fa,0xdc8f15fa,0xd84b15fc,0xdc9115fc,0xd84b15fe,
	0xdc9115fe,0xd84b1600,0xdc931600,0xd84b1602,0xdc931602,0xd84b1604,0xdc951604,0xd84b1606,
	0xdc951606,0xd84b1608,0xdc971608,0xd84b160a,0xdc97160a,0xd84b160c,0xdc99160c,0xd84b160e,
	0xdc99160e,0xd84b1610,0xdc9b1610,0xd84b1612,0xdc9b1612,0xd84b1614,0xdc9d1614,0xd84b1616,
	0xdc9d1616,0xd84b1618,0xdc9f1618,0xd84b161a,0xdc9f161a,0xd84b161c,0xdca1161c,0xd84b161e,
	0xdca1161e,0xd84b1620,0xdca31620,0xd84b1622,0xdca31622,0xd84b1624,0xdca51624,0xd84b1626,
	0xdca51626,0xd84b1628,0xdca71628,0xd84b162a,0xdca7162a,0xd84b162c,0xdca9162c,0xd84b162e,
	0xdca9162e,0xd84b1630,0xdcab1630,0xd84b1632,0xdcab1632,0xd84b1634,0xdcad1634,0xd84b1636,
	0xdcad1636,0xd84b1638,0xdcaf1638,0xd84b163a,0xdcaf163a,0xd84b163c,0xdcb1163c,0xd84b163e,
	0xdcb1163e,0xd84b1640,0xdcb31640,0xd84b1642,0xdcb31642,0xd84b1644,0xdcb51644,0xd84b1646,
	0xdcb51646,0xd84b1648,0xdcb71648,0xd84b164a,0xdcb7164a,0xd84b164c,0xdcb9164c,0xd84b164e,
	0xdcb9164e,0xd84b1650,0xdcbb1650,0xd84b1652,0xdcbb1652,0xd84b1654,0xdcbd1654,0xd84b1656,
	0xdcbd1656,0xd84b1658,0xdcbf1658,0xd84b165a,0xdcbf165a,0xd84b165c,0xdcc1165c,0xd84b165e,
	0xdcc1165e,0xd84b1660,0xdcc31660,0xd84b1662,0xdcc31662,0xd84b1664,0xdcc51664,0xd84b1666,
	0xdcc51666,0xd84b1668,0xdcc71668,0xd84b166a,0xdcc7166a,0xd84b166c,0xdcc9166c,0xd84b166e,
	0xdcc9166e,0xd84b1670,0xdccb1670,0xd84b1672,0xdccb1672,0xd84b1674,0xdccd1674,0xd84b1676,
	0xdccd1676,0xd84b1678,0xdccf1678,0xd84b167a,0xdccf167a,0xd84b167c,0xdcd1167c,0xd84b167e,
	0xdcd1167e,0xd84b1680,0xdcd31680,0xd84b1682,0xdcd31682,0xd84b1684,0xdcd51684,0xd84b1686,
	0xdcd51686,0xd84b1688,0xdcd71688,0xd84b168a,0xdcd7168a,0xd84b168c,0xdcd9168c,0xd84b168e,
	0xdcd9168e,0xd84b1690,0xdcdb1690,0xd84b1692,0xdcdb1692,0xd84b1694,0xdcdd1694,0xd84b1696,
	0xdcdd1696,0xd84b1698,0xdcdf1698,0xd84b169a,0xdcdf169a,0xd84b169c,0xdce1169c,0xd84b169e,
	0xdce1169e,0xd84b16a0,0xdce316a0,0xd84b16a2,0xdce316a2,0xd84b16a4,0xdce516a4,0xd84b16a6,
	0xdce516a6,0xd84b16a8,0xdce716a8,0xd84b16aa,0xdce716aa,0xd84b16ac,0xdce916ac,0xd84b16ae,
	0xdce916ae,0xd84b16b0,0xdceb16b0,0xd84b16b2,0xdceb16b2,0xd84b16b4,0xdced16b4,0xd84b16b6,
	0xdced16b6,0xd84b16b8,0xdcef16b8,0xd84b16ba,0xdcef16ba,0xd84b16bc,0xdcf116bc,0xd84b16be,
	0xdcf116be,0xd84b16c0,0xdcf316c0,0xd84b16c2,0xdcf316c2,0xd84b16c4,0xdcf516c4,0xd84b16c6,
	0xdcf516c6,0xd84b16c8,0xdcf716c8,0xd84b16ca,0xdcf716ca,0xd84b16cc,0xdcf916cc,0xd84b16ce,
	0xdcf916ce,0xd84b16d0,0xdcfb16d0,0xd84b16d2,0xdcfb16d2,0xd84b16d4,0xdcfd16d4,0xd84b16d6,
	0xdcfd16d6,0xd84b16d8,0xdcff16d8,0xd84b16da,0xdcff16da,0xd84b16dc,0xdd0116dc,0xd84b16de,
	0xdd0116de,0xd84b16e0,0xdd0316e0,0xd84b16e2,0xdd0316e2,0xd84b16e4,0xdd0516e4,0xd84b16e6,
	0xdd0516e6,0xd84b16e8,0xdd0716e8,0xd84b16ea,0xdd0716ea,0xd84b16ec,0xdd0916ec,0xd84b16ee,
	0xdd0916ee,0xd84b16f0,0xdd0b16f0,0xd84b16f2,0xdd0b16f2,0xd84b16f4,0xdd0d16f4,0xd84b16f6,
	0xdd0d16f6,0xd84b16f8,0xdd0f16f8,0xd84b16fa,0xdd0f16fa,0xd84b16fc,0xdd1116fc,0xd84b16fe,
	0xdd1116fe,0xd84b1700,0xdd131700,0xd84b1702,0xdd131702,0xd84b1704,0xdd151704,0xd84b1706,
	0xdd151706,0xd84b1708,0xdd171708,0xd84b170a,0xdd17170a,0xd84b170c,0xdd19170c,0xd84b170e,
	0xdd19170e,0xd84b1710,0xdd1b1710,0xd84b1712,0xdd1b1712,0xd84b1714,0xdd1d1714,0xd84b1716,
	0xdd1d1716,0xd84b1718,0xdd1f1718,0xd84b171a,0xdd1f171a,0xd84b171c,0xdd21171c,0xd84b171e,
	0xdd21171e,0xd84b1720,0xdd231720,0xd84b1722,0xdd231722,0xd84b1724,0xdd251724,0xd84b1726,
	0xdd251726,0xd84b1728,0xdd271728,0xd84b172a,0xdd27172a,0xd84b172c,0xdd29172c,0xd84b172e,
	0xdd29172e,0xd84b1730,0xdd2b1730,0xd84b1732,0xdd2b1732,0xd84b1734,0xdd2d1734,0xd84b1736,
	0xdd2d1736,0xd84b1738,0xdd2f1738,0xd84b173a,0xdd2f173a,0xd84b173c,0xdd31173c,0xd84b173e,
	0xdd31173e,0xd84b1740,0xdd331740,0xd84b1742,0xdd331742,0xd84b1744,0xdd351744,0xd84b1746,
	0xdd351746,0xd84b1748,0xdd371748,0xd84b174a,0xdd37174a,0xd84b174c,0xdd39174c,0xd84b174e,
	0xdd39174e,0xd84b1750,0xdd3b1750,0xd84b1752,0xdd3b1752,0xd84b1754,0xdd3d1754,0xd84b1756,
	0xdd3d1756,0xd84b1758,0xdd3f1758,0xd84b175a,0xdd3f175a,0xd84b175c,0xdd41175c,0xd84b175e,
	0xdd41175e,0xd84b1760,0xdd431760,0xd84b1762,0xdd431762,0xd84b1764,0xdd451764,0xd84b1766,
	0xdd451766,0xd84b1768,0xdd471768,0xd84b176a,0xdd47176a,0xd84b176c,0xdd49176c,0xd84b176e,
	0xdd49176e,0xd84b1770,0xdd4b1770,0xd84b1772,0xdd4b1772,0xd84b1774,0xdd4d1774,0xd84b1776,
	0xdd4d1776,0xd84b1778,0xdd4f1778,0xd84b177a,0xdd4f177a,0xd84b177c,0xdd51177c,0xd84b177e,
	0xdd51177e,0xd84b1780,0xdd531780,0xd84b1782,0xdd531782,0xd84b1784,0xdd551784,0xd84b1786,
	0xdd551786,0xd84b1788,0xdd571788,0xd84b178a,0xdd57178a,0xd84b178c,0xdd59178c,0xd84b178e,
	0xdd59178e,0xd84b1790,0xdd5b1790,0xd84b1792,0xdd5b1792,0xd84b1794,0xdd5d1794,0xd84b1796,
	0xdd5d1796,0xd84b1798,0xdd5f1798,0xd84b179a,0xdd5f179a,0xd84b179c,0xdd61179c,0xd84b179e,
	0xdd61179e,0xd84b17a0,0xdd6317a0,0xd84b17a2,0xdd6317a2,0xd84b17a4,0xdd6517a4,0xd84b17a6,
	0xdd6517a6,0xd84b17a8,0xdd6717a8,0xd84b17aa,0xdd6717aa,0xd84b17ac,0xdd6917ac,0xd84b17ae,
	0xdd6917ae,0xd84b17b0,0xdd6b17b0,0xd84b17b2,0xdd6b17b2,0xd84b17b4,0xdd6d17b4,0xd84b17b6,
	0xdd6d17b6,0xd84b17b8,0xdd6f17b8,0xd84b17ba,0xdd6f17ba,0xd84b17bc,0xdd7117bc,0xd84b17be,
	0xdd7117be,0xd84b17c0,0xdd7317c0,0xd84b17c2,0xdd7317c2,0xd84b17c4,0xdd7517c4,0xd84b17c6,
	0xdd7517c6,0xd84b17c8,0xdd7717c8,0xd84b17ca,0xdd7717ca,0xd84b17cc,0xdd7917cc,0xd84b17ce,
	0xdd7917ce,0xd84b17d0,0xdd7b17d0,0xd84b17d2,0xdd7b17d2,0xd84b17d4,0xdd7d17d4,0xd84b17d6,
	0xdd7d17d6,0xd84b17d8,0xdd7f17d8,0xd84b17da,0xdd7f17da,0xd84b17dc,0xdd8117dc,0xd84b17de,
	0xdd8117de,0xd84b17e0,0xdd8317e0,0xd84b17e2,0xdd8317e2,0xd84b17e4,0xdd8517e4,0xd84b17e6,
	0xdd8517e6,0xd84b17e8,0xdd8717e8,0xd84b17ea,0xdd8717ea,0xd84b17ec,0xdd8917ec,0xd84b17ee,
	0xdd8917ee,0xd84b17f0,0xdd8b17f0,0xd84b17f2,0xdd8b17f2,0xd84b17f4,0xdd8d17f4,0xd84b17f6,
	0xdd8d17f6,0xd84b17f8,0xdd8f17f8,0xd84b17fa,0xdd8f17fa,0xd84b17fc,0xdd9117fc,0xd84b17fe,
	0xdd9117fe,0xd84b1800,0xdd931800,0xd84b1802,0xdd931802,0xd84b1804,0xdd951804,0xd84b1806,
	0xdd951806,0xd84b1808,0xdd971808,0xd84b180a,0xdd97180a,0xd84b180c,0xdd99180c,0xd84b180e,
	0xdd99180e,0xd84b1810,0xdd9b1810,0xd84b1812,0xdd9b1812,0xd84b1814,0xdd9d1814,0xd84b1816,
	0xdd9d1816,0xd84b1818,0xdd9f1818,0xd84b181a,0xdd9f181a,0xd84b181c,0xdda1181c,0xd84b181e,
	0xdda1181e,0xd84b1820,0xdda31820,0xd84b1822,0xdda31822,0xd84b1824,0xdda51824,0xd84b1826,
	0xdda51826,0xd84b1828,0xdda71828,0xd84b182a,0xdda7182a,0xd84b182c,0xdda9182c,0xd84b182e,
	0xdda9182e,0xd84b1830,0xddab1830,0xd84b1832,0xddab1832,0xd84b1834,0xddad1834,0xd84b1836,
	0xddad1836,0xd84b1838,0xddaf1838,0xd84b183a,0xddaf183a,0xd84b183c,0xddb1183c,0xd84b183e,
	0xddb1183e,0xd84b1840,0xddb31840,0xd84b1842,0xddb31842,0xd84b1844,0xddb51844,0xd84b1846,
	0xddb51846,0xd84b1848,0xddb71848,0xd84b184a,0xddb7184a,0xd84b184c,0xddb9184c,0xd84b184e,
	0xddb9184e,0xd84b1850,0xddbb1850,0xd84b1852,0xddbb1852,0xd84b1854,0xddbd1854,0xd84b1856,
	0xddbd1856,0xd84b1858,0xddbf1858,0xd84b185a,0xddbf185a,0xd84b185c,0xddc1185c,0xd84b185e,
	0xddc1185e,0xd84b1860,0xddc31860,0xd84b1862,0xddc31862,0xd84b1864,0xddc51864,0xd84b1866,
	0xddc51866,0xd84b1868,0xddc71868,0xd84b186a,0xddc7186a,0xd84b186c,0xddc9186c,0xd84b186e,
	0xddc9186e,0xd84b1870,0xddcb1870,0xd84b1872,0xddcb1872,0xd84b1874,0xddcd1874,0xd84b1876,
	0xddcd1876,0xd84b1878,0xddcf1878,0xd84b187a,0xddcf187a,0xd84b187c,0xddd1187c,0xd84b187e,
	0xddd1187e,0xd84b1880,0xddd31880,0xd84b1882,0xddd31882,0xd84b1884,0xddd51884,0xd84b1886,
	0xddd51886,0xd84b1888,0xddd71888,0xd84b188a,0xddd7188a,0xd84b188c,0xddd9188c,0xd84b188e,
	0xddd9188e,0xd84b1890,0xdddb1890,0xd84b1892,0xdddb1892,0xd84b1894,0xdddd1894,0xd84b1896,
	0xdddd1896,0xd84b1898,0xdddf1898,0xd84b189a,0xdddf189a,0xd84b189c,0xdde1189c,0xd84b189e,
	0xdde1189e,0xd84b18a0,0xdde318a0,0xd84b18a2,0xdde318a2,0xd84b18a4,0xdde518a4,0xd84b18a6,
	0xdde518a6,0xd84b18a8,0xdde718a8,0xd84b18aa,0xdde718aa,0xd84b18ac,0xdde918ac,0xd84b18ae,
	0xdde918ae,0xd84b18b0,0xddeb18b0,0xd84b18b2,0xddeb18b2,0xd84b18b4,0xdded18b4,0xd84b18b6,
	0xdded18b6,0xd84b18b8,0xddef18b8,0xd84b18ba,0xddef18ba,0xd84b18bc,0xddf118bc,0xd84b18be,
	0xddf118be,0xd84b18c0,0xddf318c0,0xd84b18c2,0xddf318c2,0xd84b18c4,0xddf518c4,0xd84b18c6,
	0xddf518c6,0xd84b18c8,0xddf718c8,0xd84b18ca,0xddf718ca,0xd84b18cc,0xddf918cc,0xd84b18ce,
	0xddf918ce,0xd84b18d0,0xddfb18d0,0xd84b18d2,0xddfb18d2,0xd84b18d4,0xddfd18d4,0xd84b18d6,
	0xddfd18d6,0xd84b18d8,0xddff18d8,0xd84b18da,0xddff18da,0xd84b18dc,0xde0118dc,0xd84b18de,
	0xde0118de,0xd84b18e0,0xde0318e0,0xd84b18e2,0xde0318e2,0xd84b18e4,0xde0518e4,0xd84b18e6,
	0xde0518e6,0xd84b18e8,0xde0718e8,0xd84b18ea,0xde0718ea,0xd84b18ec,0xde0918ec,0xd84b18ee,
	0xde0918ee,0xd84b18f0,0xde0b18f0,0xd84b18f2,0xde0b18f2,0xd84b18f4,0xde0d18f4,0xd84b18f6,
	0xde0d18f6,0xd84b18f8,0xde0f18f8,0xd84b18fa,0xde0f18fa,0xd84b18fc,0xde1118fc,0xd84b18fe,
	0xde1118fe,0xd84b1900,0xde131900,0xd84b1902,0xde131902,0xd84b1904,0xde151904,0xd84b1906,
	0xde151906,0xd84b1908,0xde171908,0xd84b190a,0xde17190a,0xd84b190c,0xde19190c,0xd84b190e,
	0xde19190e,0xd84b1910,0xde1b1910,0xd84b1912,0xde1b1912,0xd84b1914,0xde1d1914,0xd84b1916,
	0xde1d1916,0xd84b1918,0xde1f1918,0xd84b191a,0xde1f191a,0xd84b191c,0xde21191c,0xd84b191e,
	0xde21191e,0xd84b1920,0xde231920,0xd84b1922,0xde231922,0xd84b1924,0xde251924,0xd84b1926,
	0xde251926,0xd84b1928,0xde271928,0xd84b192a,0xde27192a,0xd84b192c,0xde29192c,0xd84b192e,
	0xde29192e,0xd84b1930,0xde2b1930,0xd84b1932,0xde2b1932,0xd84b1934,0xde2d1934,0xd84b1936,
	0xde2d1936,0xd84b1938,0xde2f1938,0xd84b193a,0xde2f193a,0xd84b193c,0xde31193c,0xd84b193e,
	0xde31193e,0xd84b1940,0xde331940,0xd84b1942,0xde331942,0xd84b1944,0xde351944,0xd84b1946,
	0xde351946,0xd84b1948,0xde371948,0xd84b194a,0xde37194a,0xd84b194c,0xde39194c,0xd84b194e,
	0xde39194e,0xd84b1950,0xde3b1950,0xd84b1952,0xde3b1952,0xd84b1954,0xde3d1954,0xd84b1956,
	0xde3d1956,0xd84b1958,0xde3f1958,0xd84b195a,0xde3f195a,0xd84b195c,0xde41195c,0xd84b195e,
	0xde41195e,0xd84b1960,0xde431960,0xd84b1962,0xde431962,0xd84b1964,0xde451964,0xd84b1966,
	0xde451966,0xd84b1968,0xde471968,0xd84b196a,0xde47196a,0xd84b196c,0xde49196c,0xd84b196e,
	0xde49196e,0xd84b1970,0xde4b1970,0xd84b1972,0xde4b1972,0xd84b1974,0xde4d1974,0xd84b1976,
	0xde4d1976,0xd84b1978,0xde4f1978,0xd84b197a,0xde4f197a,0xd84b197c,0xde51197c,0xd84b197e,
	0xde51197e,0xd84b1980,0xde531980,0xd84b1982,0xde531982,0xd84b1984,0xde551984,0xd84b1986,
	0xde551986,0xd84b1988,0xde571988,0xd84b198a,0xde57198a,0xd84b198c,0xde59198c,0xd84b198e,
	0xde59198e,0xd84b1990,0xde5b1990,0xd84b1992,0xde5b1992,0xd84b1994,0xde5d1994,0xd84b1996,
	0xde5d1996,0xd84b1998,0xde5f1998,0xd84b199a,0xde5f199a,0xd84b199c,0xde61199c,0xd84b199e,
	0xde61199e,0xd84b19a0,0xde6319a0,0xd84b19a2,0xde6319a2,0xd84b19a4,0xde6519a4,0xd84b19a6,
	0xde6519a6,0xd84b19a8,0xde6719a8,0xd84b19aa,0xde6719aa,0xd84b19ac,0xde6919ac,0xd84b19ae,
	0xde6919ae,0xd84b19b0,0xde6b19b0,0xd84b19b2,0xde6b19b2,0xd84b19b4,0xde6d19b4,0xd84b19b6,
	0xde6d19b6,0xd84b19b8,0xde6f19b8,0xd84b19ba,0xde6f19ba,0xd84b19bc,0xde7119bc,0xd84b19be,
	0xde7119be,0xd84b19c0,0xde7319c0,0xd84b19c2,0xde7319c2,0xd84b19c4,0xde7519c4,0xd84b19c6,
	0xde7519c6,0xd84b19c8,0xde7719c8,0xd84b19ca,0xde7719ca,0xd84b19cc,0xde7919cc,0xd84b19ce,
	0xde7919ce,0xd84b19d0,0xde7b19d0,0xd84b19d2,0xde7b19d2,0xd84b19d4,0xde7d19d4,0xd84b19d6,
	0xde7d19d6,0xd84b19d8,0xde7f19d8,0xd84b19da,0xde7f19da,0xd84b19dc,0xde8119dc,0xd84b19de,
	0xde8119de,0xd84b19e0,0xde8319e0,0xd84b19e2,0xde8319e2,0xd84b19e4,0xde8519e4,0xd84b19e6,
	0xde8519e6,0xd84b19e8,0xde8719e8,0xd84b19ea,0xde8719ea,0xd84b19ec,0xde8919ec,0xd84b19ee,
	0xde8919ee,0xd84b19f0,0xde8b19f0,0xd84b19f2,0xde8b19f2,0xd84b19f4,0xde8d19f4,0xd84b19f6,
	0xde8d19f6,0xd84b19f8,0xde8f19f8,0xd84b19fa,0xde8f19fa,0xd84b19fc,0xde9119fc,0xd84b19fe,
	0xde9119fe,0xd84b1a00,0xde931a00,0xd84b1a02,0xde931a02,0xd84b1a04,0xde951a04,0xd84b1a06,
	0xde951a06,0xd84b1a08,0xde971a08,0xd84b1a0a,0xde971a0a,0xd84b1a0c,0xde991a0c,0xd84b1a0e,
	0xde991a0e,0xd84b1a10,0xde9b1a10,0xd84b1a12,0xde9b1a12,0xd84b1a14,0xde9d1a14,0xd84b1a16,
	0xde9d1a16,0xd84b1a18,0xde9f1a18,0xd84b1a1a,0xde9f1a1a,0xd84b1a1c,0xdea11a1c,0xd84b1a1e,
	0xdea11a1e,0xd84b1a20,0xdea31a20,0xd84b1a22,0xdea31a22,0xd84b1a24,0xdea51a24,0xd84b1a26,
	0xdea51a26,0xd84b1a28,0xdea71a28,0xd84b1a2a,0xdea71a2a,0xd84b1a2c,0xdea91a2c,0xd84b1a2e,
	0xdea91a2e,0xd84b1a30,0xdeab1a30,0xd84b1a32,0xdeab1a32,0xd84b1a34,0xdead1a34,0xd84b1a36,
	0xdead1a36,0xd84b1a38,0xdeaf1a38,0xd84b1a3a,0xdeaf1a3a,0xd84b1a3c,0xdeb11a3c,0xd84b1a3e,
	0xdeb11a3e,0xd84b1a40,0xdeb31a40,0xd84b1a42,0xdeb31a42,0xd84b1a44,0xdeb51a44,0xd84b1a46,
	0xdeb51a46,0xd84b1a48,0xdeb71a48,0xd84b1a4a,0xdeb71a4a,0xd84b1a4c,0xdeb91a4c,0xd84b1a4e,
	0xdeb91a4e,0xd84b1a50,0xdebb1a50,0xd84b1a52,0xdebb1a52,0xd84b1a54,0xdebd1a54,0xd84b1a56,
	0xdebd1a56,0xd84b1a58,0xdebf1a58,0xd84b1a5a,0xdebf1a5a,0xd84b1a5c,0xdec11a5c,0xd84b1a5e,
	0xdec11a5e,0xd84b1a60,0xdec31a60,0xd84b1a62,0xdec31a62,0xd84b1a64,0xdec51a64,0xd84b1a66,
	0xdec51a66,0xd84b1a68,0xdec71a68,0xd84b1a6a,0xdec71a6a,0xd84b1a6c,0xdec91a6c,0xd84b1a6e,
	0xdec91a6e,0xd84b1a70,0xdecb1a70,0xd84b1a72,0xdecb1a72,0xd84b1a74,0xdecd1a74,0xd84b1a76,
	0xdecd1a76,0xd84b1a78,0xdecf1a78,0xd84b1a7a,0xdecf1a7a,0xd84b1a7c,0xded11a7c,0xd84b1a7e,
	0xded11a7e,0xd84b1a80,0xded31a80,0xd84b1a82,0xded31a82,0xd84b1a84,0xded51a84,0xd84b1a86,
	0xded51a86,0xd84b1a88,0xded71a88,0xd84b1a8a,0xded71a8a,0xd84b1a8c,0xded91a8c,0xd84b1a8e,
	0xded91a8e,0xd84b1a90,0xdedb1a90,0xd84b1a92,0xdedb1a92,0xd84b1a94,0xdedd1a94,0xd84b1a96,
	0xdedd1a96,0xd84b1a98,0xdedf1a98,0xd84b1a9a,0xdedf1a9a,0xd84b1a9c,0xdee11a9c,0xd84b1a9e,
	0xdee11a9e,0xd84b1aa0,0xdee31aa0,0xd84b1aa2,0xdee31aa2,0xd84b1aa4,0xdee51aa4,0xd84b1aa6,
	0xdee51aa6,0xd84b1aa8,0xdee71aa8,0xd84b1aaa,0xdee71aaa,0xd84b1aac,0xdee91aac,0xd84b1aae,
	0xdee91aae,0xd84b1ab0,0xdeeb1ab0,0xd84b1ab2,0xdeeb1ab2,0xd84b1ab4,0xdeed1ab4,0xd84b1ab6,
	0xdeed1ab6,0xd84b1ab8,0xdeef1ab8,0xd84b1aba,0xdeef1aba,0xd84b1abc,0xdef11abc,0xd84b1abe,
	0xdef11abe,0xd84b1ac0,0xdef31ac0,0xd84b1ac2,0xdef31ac2,0xd84b1ac4,0xdef51ac4,0xd84b1ac6,
	0xdef51ac6,0xd84b1ac8,0xdef71ac8,0xd84b1aca,0xdef71aca,0xd84b1acc,0xdef91acc,0xd84b1ace,
	0xdef91ace,0xd84b1ad0,0xdefb1ad0,0xd84b1ad2,0xdefb1ad2,0xd84b1ad4,0xdefd1ad4,0xd84b1ad6,
	0xdefd1ad6,0xd84b1ad8,0xdeff1ad8,0xd84b1ada,0xdeff1ada,0xd84b1adc,0xdf011adc,0xd84b1ade,
	0xdf011ade,0xd84b1ae0,0xdf031ae0,0xd84b1ae2,0xdf031ae2,0xd84b1ae4,0xdf051ae4,0xd84b1ae6,
	0xdf051ae6,0xd84b1ae8,0xdf071ae8,0xd84b1aea,0xdf071aea,0xd84b1aec,0xdf091aec,0xd84b1aee,
	0xdf091aee,0xd84b1af0,0xdf0b1af0,0xd84b1af2,0xdf0b1af2,0xd84b1af4,0xdf0d1af4,0xd84b1af6,
	0xdf0d1af6,0xd84b1af8,0xdf0f1af8,0xd84b1afa,0xdf0f1afa,0xd84b1afc,0xdf111afc,0xd84b1afe,
	0xdf111afe,0xd84b1b00,0xdf131b00,0xd84b1b02,0xdf131b02,0xd84b1b04,0xdf151b04,0xd84b1b06,
	0xdf151b06,0xd84b1b08,0xdf171b08,0xd84b1b0a,0xdf171b0a,0xd84b1b0c,0xdf191b0c,0xd84b1b0e,
	0xdf191b0e,0xd84b1b10,0xdf1b1b10,0xd84b1b12,0xdf1b1b12,0xd84b1b14,0xdf1d1b14,0xd84b1b16,
	0xdf1d1b16,0xd84b1b18,0xdf1f1b18,0xd84b1b1a,0xdf1f1b1a,0xd84b1b1c,0xdf211b1c,0xd84b1b1e,
	0xdf211b1e,0xd84b1b20,0xdf231b20,0xd84b1b22,0xdf231b22,0xd84b1b24,0xdf251b24,0xd84b1b26,
	0xdf251b26,0xd84b1b28,0xdf271b28,0xd84b1b2a,0xdf271b2a,0xd84b1b2c,0xdf291b2c,0xd84b1b2e,
	0xdf291b2e,0xd84b1b30,0xdf2b1b30,0xd84b1b32,0xdf2b1b32,0xd84b1b34,0xdf2d1b34,0xd84b1b36,
	0xdf2d1b36,0xd84b1b38,0xdf2f1b38,0xd84b1b3a,0xdf2f1b3a,0xd84b1b3c,0xdf311b3c,0xd84b1b3e,
	0xdf311b3e,0xd84b1b40,0xdf331b40,0xd84b1b42,0xdf331b42,0xd84b1b44,0xdf351b44,0xd84b1b46,
	0xdf351b46,0xd84b1b48,0xdf371b48,0xd84b1b4a,0xdf371b4a,0xd84b1b4c,0xdf391b4c,0xd84b1b4e,
	0xdf391b4e,0xd84b1b50,0xdf3b1b50,0xd84b1b52,0xdf3b1b52,0xd84b1b54,0xdf3d1b54,0xd84b1b56,
	0xdf3d1b56,0xd84b1b58,0xdf3f1b58,0xd84b1b5a,0xdf3f1b5a,0xd84b1b5c,0xdf411b5c,0xd84b1b5e,
	0xdf411b5e,0xd84b1b60,0xdf431b60,0xd84b1b62,0xdf431b62,0xd84b1b64,0xdf451b64,0xd84b1b66,
	0xdf451b66,0xd84b1b68,0xdf471b68,0xd84b1b6a,0xdf471b6a,0xd84b1b6c,0xdf491b6c,0xd84b1b6e,
	0xdf491b6e,0xd84b1b70,0xdf4b1b70,0xd84b1b72,0xdf4b1b72,0xd84b1b74,0xdf4d1b74,0xd84b1b76,
	0xdf4d1b76,0xd84b1b78,0xdf4f1b78,0xd84b1b7a,0xdf4f1b7a,0xd84b1b7c,0xdf511b7c,0xd84b1b7e,
	0xdf511b7e,0xd84b1b80,0xdf531b80,0xd84b1b82,0xdf531b82,0xd84b1b84,0xdf551b84,0xd84b1b86,
	0xdf551b86,0xd84b1b88,0xdf571b88,0xd84b1b8a,0xdf571b8a,0xd84b1b8c,0xdf591b8c,0xd84b1b8e,
	0xdf591b8e,0xd84b1b90,0xdf5b1b90,0xd84b1b92,0xdf5b1b92,0xd84b1b94,0xdf5d1b94,0xd84b1b96,
	0xdf5d1b96,0xd84b1b98,0xdf5f1b98,0xd84b1b9a,0xdf5f1b9a,0xd84b1b9c,0xdf611b9c,0xd84b1b9e,
	0xdf611b9e,0xd84b1ba0,0xdf631ba0,0xd84b1ba2,0xdf631ba2,0xd84b1ba4,0xdf651ba4,0xd84b1ba6,
	0xdf651ba6,0xd84b1ba8,0xdf671ba8,0xd84b1baa,0xdf671baa,0xd84b1bac,0xdf691bac,0xd84b1bae,
	0xdf691bae,0xd84b1bb0,0xdf6b1bb0,0xd84b1bb2,0xdf6b1bb2,0xd84b1bb4,0xdf6d1bb4,0xd84b1bb6,
	0xdf6d1bb6,0xd84b1bb8,0xdf6f1bb8,0xd84b1bba,0xdf6f1bba,0xd84b1bbc,0xdf711bbc,0xd84b1bbe,
	0xdf711bbe,0xd84b1bc0,0xdf731bc0,0xd84b1bc2,0xdf731bc2,0xd84b1bc4,0xdf751bc4,0xd84b1bc6,
	0xdf751bc6,0xd84b1bc8,0xdf771bc8,0xd84b1bca,0xdf771bca,0xd84b1bcc,0xdf791bcc,0xd84b1bce,
	0xdf791bce,0xd84b1bd0,0xdf7b1bd0,0xd84b1bd2,0xdf7b1bd2,0xd84b1bd4,0xdf7d1bd4,0xd84b1bd6,
	0xdf7d1bd6,0xd84b1bd8,0xdf7f1bd8,0xd84b1bda,0xdf7f1bda,0xd84b1bdc,0xdf811bdc,0xd84b1bde,
	0xdf811bde,0xd84b1be0,0xdf831be0,0xd84b1be2,0xdf831be2,0xd84b1be4,0xdf851be4,0xd84b1be6,
	0xdf851be6,0xd84b1be8,0xdf871be8,0xd84b1bea,0xdf871bea,0xd84b1bec,0xdf891bec,0xd84b1bee,
	0xdf891bee,0xd84b1bf0,0xdf8b1bf0,0xd84b1bf2,0xdf8b1bf2,0xd84b1bf4,0xdf8d1bf4,0xd84b1bf6,
	0xdf8d1bf6,0xd84b1bf8,0xdf8f1bf8,0xd84b1bfa,0xdf8f1bfa,0xd84b1bfc,0xdf911bfc,0xd84b1bfe,
	0xdf911bfe,0xd84b1c00,0xdf931c00,0xd84b1c02,0xdf931c02,0xd84b1c04,0xdf951c04,0xd84b1c06,
	0xdf951c06,0xd84b1c08,0xdf971c08,0xd84b1c0a,0xdf971c0a,0xd84b1c0c,0xdf991c0c,0xd84b1c0e,
	0xdf991c0e,0xd84b1c10,0xdf9b1c10,0xd84b1c12,0xdf9b1c12,0xd84b1c14,0xdf9d1c14,0xd84b1c16,
	0xdf9d1c16,0xd84b1c18,0xdf9f1c18,0xd84b1c1a,0xdf9f1c1a,0xd84b1c1c,0xdfa11c1c,0xd84b1c1e,
	0xdfa11c1e,0xd84b1c20,0xdfa31c20,0xd84b1c22,0xdfa31c22,0xd84b1c24,0xdfa51c24,0xd84b1c26,
	0xdfa51c26,0xd84b1c28,0xdfa71c28,0xd84b1c2a,0xdfa71c2a,0xd84b1c2c,0xdfa91c2c,0xd84b1c2e,
	0xdfa91c2e,0xd84b1c30,0xdfab1c30,0xd84b1c32,0xdfab1c32,0xd84b1c34,0xdfad1c34,0xd84b1c36,
	0xdfad1c36,0xd84b1c38,0xdfaf1c38,0xd84b1c3a,0xdfaf1c3a,0xd84b1c3c,0xdfb11c3c,0xd84b1c3e,
	0xdfb11c3e,0xd84b1c40,0xdfb31c40,0xd84b1c42,0xdfb31c42,0xd84b1c44,0xdfb51c44,0xd84b1c46,
	0xdfb51c46,0xd84b1c48,0xdfb71c48,0xd84b1c4a,0xdfb71c4a,0xd84b1c4c,0xdfb91c4c,0xd84b1c4e,
	0xdfb91c4e,0xd84b1c50,0xdfbb1c50,0xd84b1c52,0xdfbb1c52,0xd84b1c54,0xdfbd1c54,0xd84b1c56,
	0xdfbd1c56,0xd84b1c58,0xdfbf1c58,0xd84b1c5a,0xdfbf1c5a,0xd84b1c5c,0xdfc11c5c,0xd84b1c5e,
	0xdfc11c5e,0xd84b1c60,0xdfc31c60,0xd84b1c62,0xdfc31c62,0xd84b1c64,0xdfc51c64,0xd84b1c66,
	0xdfc51c66,0xd84b1c68,0xdfc71c68,0xd84b1c6a,0xdfc71c6a,0xd84b1c6c,0xdfc91c6c,0xd84b1c6e,
	0xdfc91c6e,0xd84b1c70,0xdfcb1c70,0xd84b1c72,0xdfcb1c72,0xd84b1c74,0xdfcd1c74,0xd84b1c76,
	0xdfcd1c76,0xd84b1c78,0xdfcf1c78,0xd84b1c7a,0xdfcf1c7a,0xd84b1c7c,0xdfd11c7c,0xd84b1c7e,
	0xdfd11c7e,0xd84b1c80,0xdfd31c80,0xd84b1c82,0xdfd31c82,0xd84b1c84,0xdfd51c84,0xd84b1c86,
	0xdfd51c86,0xd84b1c88,0xdfd71c88,0xd84b1c8a,0xdfd71c8a,0xd84b1c8c,0xdfd91c8c,0xd84b1c8e,
	0xdfd91c8e,0xd84b1c90,0xdfdb1c90,0xd84b1c92,0xdfdb1c92,0xd84b1c94,0xdfdd1c94,0xd84b1c96,
	0xdfdd1c96,0xd84b1c98,0xdfdf1c98,0xd84b1c9a,0xdfdf1c9a,0xd84b1c9c,0xdfe11c9c,0xd84b1c9e,
	0xdfe11c9e,0xd84b1ca0,0xdfe31ca0,0xd84b1ca2,0xdfe31ca2,0xd84b1ca4,0xdfe51ca4,0xd84b1ca6,
	0xdfe51ca6,0xd84b1ca8,0xdfe71ca8,0xd84b1caa,0xdfe71caa,0xd84b1cac,0xdfe91cac,0xd84b1cae,
	0xdfe91cae,0xd84b1cb0,0xdfeb1cb0,0xd84b1cb2,0xdfeb1cb2,0xd84b1cb4,0xdfed1cb4,0xd84b1cb6,
	0xdfed1cb6,0xd84b1cb8,0xdfef1cb8,0xd84b1cba,0xdfef1cba,0xd84b1cbc,0xdff11cbc,0xd84b1cbe,
	0xdff11cbe,0xd84b1cc0,0xdff31cc0,0xd84b1cc2,0xdff31cc2,0xd84b1cc4,0xdff51cc4,0xd84b1cc6,
	0xdff51cc6,0xd84b1cc8,0xdff71cc8,0xd84b1cca,0xdff71cca,0xd84b1ccc,0xdff91ccc,0xd84b1cce,
	0xdff91cce,0xd84b1cd0,0xdffb1cd0,0xd84b1cd2,0xdffb1cd2,0xd84b1cd4,0xdffd1cd4,0xd84b1cd6,
	0xdffd1cd6,0xd84b1cd8,0xdfff1cd8,0xd84b1cda,0xdfff1cda,0xd84d1cdc,0xdc011cdc,0xd84d1cde,
	0xdc011cde,0xd84d1ce0,0xdc031ce0,0xd84d1ce2,0xdc031ce2,0xd84d1ce4,0xdc051ce4,0xd84d1ce6,
	0xdc051ce6,0xd84d1ce8,0xdc071ce8,0xd84d1cea,0xdc071cea,0xd84d1cec,0xdc091cec,0xd84d1cee,
	0xdc091cee,0xd84d1cf0,0xdc0b1cf0,0xd84d1cf2,0xdc0b1cf2,0xd84d1cf4,0xdc0d1cf4,0xd84d1cf6,
	0xdc0d1cf6,0xd84d1cf8,0xdc0f1cf8,0xd84d1cfa,0xdc0f1cfa,0xd84d1cfc,0xdc111cfc,0xd84d1cfe,
	0xdc111cfe,0xd84d1d00,0xdc131d00,0xd84d1d02,0xdc131d02,0xd84d1d04,0xdc151d04,0xd84d1d06,
	0xdc151d06,0xd84d1d08,0xdc171d08,0xd84d1d0a,0xdc171d0a,0xd84d1d0c,0xdc191d0c,0xd84d1d0e,
	0xdc191d0e,0xd84d1d10,0xdc1b1d10,0xd84d1d12,0xdc1b1d12,0xd84d1d14,0xdc1d1d14,0xd84d1d16,
	0xdc1d1d16,0xd84d1d18,0xdc1f1d18,0xd84d1d1a,0xdc1f1d1a,0xd84d1d1c,0xdc211d1c,0xd84d1d1e,
	0xdc211d1e,0xd84d1d20,0xdc231d20,0xd84d1d22,0xdc231d22,0xd84d1d24,0xdc251d24,0xd84d1d26,
	0xdc251d26,0xd84d1d28,0xdc271d28,0xd84d1d2a,0xdc271d2a,0xd84d1d2c,0xdc291d2c,0xd84d1d2e,
	0xdc291d2e,0xd84d1d30,0xdc2b1d30,0xd84d1d32,0xdc2b1d32,0xd84d1d34,0xdc2d1d34,0xd84d1d36,
	0xdc2d1d36,0xd84d1d38,0xdc2f1d38,0xd84d1d3a,0xdc2f1d3a,0xd84d1d3c,0xdc311d3c,0xd84d1d3e,
	0xdc311d3e,0xd84d1d40,0xdc331d40,0xd84d1d42,0xdc331d42,0xd84d1d44,0xdc351d44,0xd84d1d46,
	0xdc351d46,0xd84d1d48,0xdc371d48,0xd84d1d4a,0xdc371d4a,0xd84d1d4c,0xdc391d4c,0xd84d1d4e,
	0xdc391d4e,0xd84d1d50,0xdc3b1d50,0xd84d1d52,0xdc3b1d52,0xd84d1d54,0xdc3d1d54,0xd84d1d56,
	0xdc3d1d56,0xd84d1d58,0xdc3f1d58,0xd84d1d5a,0xdc3f1d5a,0xd84d1d5c,0xdc411d5c,0xd84d1d5e,
	0xdc411d5e,0xd84d1d60,0xdc431d60,0xd84d1d62,0xdc431d62,0xd84d1d64,0xdc451d64,0xd84d1d66,
	0xdc451d66,0xd84d1d68,0xdc471d68,0xd84d1d6a,0xdc471d6a,0xd84d1d6c,0xdc491d6c,0xd84d1d6e,
	0xdc491d6e,0xd84d1d70,0xdc4b1d70,0xd84d1d72,0xdc4b1d72,0xd84d1d74,0xdc4d1d74,0xd84d1d76,
	0xdc4d1d76,0xd84d1d78,0xdc4f1d78,0xd84d1d7a,0xdc4f1d7a,0xd84d1d7c,0xdc511d7c,0xd84d1d7e,
	0xdc511d7e,0xd84d1d80,0xdc531d80,0xd84d1d82,0xdc531d82,0xd84d1d84,0xdc551d84,0xd84d1d86,
	0xdc551d86,0xd84d1d88,0xdc571d88,0xd84d1d8a,0xdc571d8a,0xd84d1d8c,0xdc591d8c,0xd84d1d8e,
	0xdc591d8e,0xd84d1d90,0xdc5b1d90,0xd84d1d92,0xdc5b1d92,0xd84d1d94,0xdc5d1d94,0xd84d1d96,
	0xdc5d1d96,0xd84d1d98,0xdc5f1d98,0xd84d1d9a,0xdc5f1d9a,0xd84d1d9c,0xdc611d9c,0xd84d1d9e,
	0xdc611d9e,0xd84d1da0,0xdc631da0,0xd84d1da2,0xdc631da2,0xd84d1da4,0xdc651da4,0xd84d1da6,
	0xdc651da6,0xd84d1da8,0xdc671da8,0xd84d1daa,0xdc671daa,0xd84d1dac,0xdc691dac,0xd84d1dae,
	0xdc691dae,0xd84d1db0,0xdc6b1db0,0xd84d1db2,0xdc6b1db2,0xd84d1db4,0xdc6d1db4,0xd84d1db6,
	0xdc6d1db6,0xd84d1db8,0xdc6f1db8,0xd84d1dba,0xdc6f1dba,0xd84d1dbc,0xdc711dbc,0xd84d1dbe,
	0xdc711dbe,0xd84d1dc0,0xdc731dc0,0xd84d1dc2,0xdc731dc2,0xd84d1dc4,0xdc751dc4,0xd84d1dc6,
	0xdc751dc6,0xd84d1dc8,0xdc771dc8,0xd84d1dca,0xdc771dca,0xd84d1dcc,0xdc791dcc,0xd84d1dce,
	0xdc791dce,0xd84d1dd0,0xdc7b1dd0,0xd84d1dd2,0xdc7b1dd2,0xd84d1dd4,0xdc7d1dd4,0xd84d1dd6,
	0xdc7d1dd6,0xd84d1dd8,0xdc7f1dd8,0xd84d1dda,0xdc7f1dda,0xd84d1ddc,0xdc811ddc,0xd84d1dde,
	0xdc811dde,0xd84d1de0,0xdc831de0,0xd84d1de2,0xdc831de2,0xd84d1de4,0xdc851de4,0xd84d1de6,
	0xdc851de6,0xd84d1de8,0xdc871de8,0xd84d1dea,0xdc871dea,0xd84d1dec,0xdc891dec,0xd84d1dee,
	0xdc891dee,0xd84d1df0,0xdc8b1df0,0xd84d1df2,0xdc8b1df2,0xd84d1df4,0xdc8d1df4,0xd84d1df6,
	0xdc8d1df6,0xd84d1df8,0xdc8f1df8,0xd84d1dfa,0xdc8f1dfa,0xd84d1dfc,0xdc911dfc,0xd84d1dfe,
	0xdc911dfe,0xd84d1e00,0xdc931e00,0xd84d1e02,0xdc931e02,0xd84d1e04,0xdc951e04,0xd84d1e06,
	0xdc951e06,0xd84d1e08,0xdc971e08,0xd84d1e0a,0xdc971e0a,0xd84d1e0c,0xdc991e0c,0xd84d1e0e,
	0xdc991e0e,0xd84d1e10,0xdc9b1e10,0xd84d1e12,0xdc9b1e12,0xd84d1e14,0xdc9d1e14,0xd84d1e16,
	0xdc9d1e16,0xd84d1e18,0xdc9f1e18,0xd84d1e1a,0xdc9f1e1a,0xd84d1e1c,0xdca11e1c,0xd84d1e1e,
	0xdca11e1e,0xd84d1e20,0xdca31e20,0xd84d1e22,0xdca31e22,0xd84d1e24,0xdca51e24,0xd84d1e26,
	0xdca51e26,0xd84d1e28,0xdca71e28,0xd84d1e2a,0xdca71e2a,0xd84d1e2c,0xdca91e2c,0xd84d1e2e,
	0xdca91e2e,0xd84d1e30,0xdcab1e30,0xd84d1e32,0xdcab1e32,0xd84d1e34,0xdcad1e34,0xd84d1e36,
	0xdcad1e36,0xd84d1e38,0xdcaf1e38,0xd84d1e3a,0xdcaf1e3a,0xd84d1e3c,0xdcb11e3c,0xd84d1e3e,
	0xdcb11e3e,0xd84d1e40,0xdcb31e40,0xd84d1e42,0xdcb31e42,0xd84d1e44,0xdcb51e44,0xd84d1e46,
	0xdcb51e46,0xd84d1e48,0xdcb71e48,0xd84d1e4a,0xdcb71e4a,0xd84d1e4c,0xdcb91e4c,0xd84d1e4e,
	0xdcb91e4e,0xd84d1e50,0xdcbb1e50,0xd84d1e52,0xdcbb1e52,0xd84d1e54,0xdcbd1e54,0xd84d1e56,
	0xdcbd1e56,0xd84d1e58,0xdcbf1e58,0xd84d1e5a,0xdcbf1e5a,0xd84d1e5c,0xdcc11e5c,0xd84d1e5e,
	0xdcc11e5e,0xd84d1e60,0xdcc31e60,0xd84d1e62,0xdcc31e62,0xd84d1e64,0xdcc51e64,0xd84d1e66,
	0xdcc51e66,0xd84d1e68,0xdcc71e68,0xd84d1e6a,0xdcc71e6a,0xd84d1e6c,0xdcc91e6c,0xd84d1e6e,
	0xdcc91e6e,0xd84d1e70,0xdccb1e70,0xd84d1e72,0xdccb1e72,0xd84d1e74,0xdccd1e74,0xd84d1e76,
	0xdccd1e76,0xd84d1e78,0xdccf1e78,0xd84d1e7a,0xdccf1e7a,0xd84d1e7c,0xdcd11e7c,0xd84d1e7e,
	0xdcd11e7e,0xd84d1e80,0xdcd31e80,0xd84d1e82,0xdcd31e82,0xd84d1e84,0xdcd51e84,0xd84d1e86,
	0xdcd51e86,0xd84d1e88,0xdcd71e88,0xd84d1e8a,0xdcd71e8a,0xd84d1e8c,0xdcd91e8c,0xd84d1e8e,
	0xdcd91e8e,0xd84d1e90,0xdcdb1e90,0xd84d1e92,0xdcdb1e92,0xd84d1e94,0xdcdd1e94,0xd84d1e96,
	0xdcdd1e96,0xd84d1e98,0xdcdf1e98,0xd84d1e9a,0xdcdf1e9a,0xd84d1e9c,0xdce11e9c,0xd84d1e9e,
	0xdce11e9e,0xd84d1ea0,0xdce31ea0,0xd84d1ea2,0xdce31ea2,0xd84d1ea4,0xdce51ea4,0xd84d1ea6,
	0xdce51ea6,0xd84d1ea8,0xdce71ea8,0xd84d1eaa,0xdce71eaa,0xd84d1eac,0xdce91eac,0xd84d1eae,
	0xdce91eae,0xd84d1eb0,0xdceb1eb0,0xd84d1eb2,0xdceb1eb2,0xd84d1eb4,0xdced1eb4,0xd84d1eb6,
	0xdced1eb6,0xd84d1eb8,0xdcef1eb8,0xd84d1eba,0xdcef1eba,0xd84d1ebc,0xdcf11ebc,0xd84d1ebe,
	0xdcf11ebe,0xd84d1ec0,0xdcf31ec0,0xd84d1ec2,0xdcf31ec2,0xd84d1ec4,0xdcf51ec4,0xd84d1ec6,
	0xdcf51ec6,0xd84d1ec8,0xdcf71ec8,0xd84d1eca,0xdcf71eca,0xd84d1ecc,0xdcf91ecc,0xd84d1ece,
	0xdcf91ece,0xd84d1ed0,0xdcfb1ed0,0xd84d1ed2,0xdcfb1ed2,0xd84d1ed4,0xdcfd1ed4,0xd84d1ed6,
	0xdcfd1ed6,0xd84d1ed8,0xdcff1ed8,0xd84d1eda,0xdcff1eda,0xd84d1edc,0xdd011edc,0xd84d1ede,
	0xdd011ede,0xd84d1ee0,0xdd031ee0,0xd84d1ee2,0xdd031ee2,0xd84d1ee4,0xdd051ee4,0xd84d1ee6,
	0xdd051ee6,0xd84d1ee8,0xdd071ee8,0xd84d1eea,0xdd071eea,0xd84d1eec,0xdd091eec,0xd84d1eee,
	0xdd091eee,0xd84d1ef0,0xdd0b1ef0,0xd84d1ef2,0xdd0b1ef2,0xd84d1ef4,0xdd0d1ef4,0xd84d1ef6,
	0xdd0d1ef6,0xd84d1ef8,0xdd0f1ef8,0xd84d1efa,0xdd0f1efa,0xd84d1efc,0xdd111efc,0xd84d1efe,
	0xdd111efe,0xd84d1f00,0xdd131f00,0xd84d1f02,0xdd131f02,0xd84d1f04,0xdd151f04,0xd84d1f06,
	0xdd151f06,0xd84d1f08,0xdd171f08,0xd84d1f0a,0xdd171f0a,0xd84d1f0c,0xdd191f0c,0xd84d1f0e,
	0xdd191f0e,0xd84d1f10,0xdd1b1f10,0xd84d1f12,0xdd1b1f12,0xd84d1f14,0xdd1d1f14,0xd84d1f16,
	0xdd1d1f16,0xd84d1f18,0xdd1f1f18,0xd84d1f1a,0xdd1f1f1a,0xd84d1f1c,0xdd211f1c,0xd84d1f1e,
	0xdd211f1e,0xd84d1f20,0xdd231f20,0xd84d1f22,0xdd231f22,0xd84d1f24,0xdd251f24,0xd84d1f26,
	0xdd251f26,0xd84d1f28,0xdd271f28,0xd84d1f2a,0xdd271f2a,0xd84d1f2c,0xdd291f2c,0xd84d1f2e,
	0xdd291f2e,0xd84d1f30,0xdd2b1f30,0xd84d1f32,0xdd2b1f32,0xd84d1f34,0xdd2d1f34,0xd84d1f36,
	0xdd2d1f36,0xd84d1f38,0xdd2f1f38,0xd84d1f3a,0xdd2f1f3a,0xd84d1f3c,0xdd311f3c,0xd84d1f3e,
	0xdd311f3e,0xd84d1f40,0xdd331f40,0xd84d1f42,0xdd331f42,0xd84d1f44,0xdd351f44,0xd84d1f46,
	0xdd351f46,0xd84d1f48,0xdd371f48,0xd84d1f4a,0xdd371f4a,0xd84d1f4c,0xdd391f4c,0xd84d1f4e,
	0xdd391f4e,0xd84d1f50,0xdd3b1f50,0xd84d1f52,0xdd3b1f52,0xd84d1f54,0xdd3d1f54,0xd84d1f56,
	0xdd3d1f56,0xd84d1f58,0xdd3f1f58,0xd84d1f5a,0xdd3f1f5a,0xd84d1f5c,0xdd411f5c,0xd84d1f5e,
	0xdd411f5e,0xd84d1f60,0xdd431f60,0xd84d1f62,0xdd431f62,0xd84d1f64,0xdd451f64,0xd84d1f66,
	0xdd451f66,0xd84d1f68,0xdd471f68,0xd84d1f6a,0xdd471f6a,0xd84d1f6c,0xdd491f6c,0xd84d1f6e,
	0xdd491f6e,0xd84d1f70,0xdd4b1f70,0xd84d1f72,0xdd4b1f72,0xd84d1f74,0xdd4d1f74,0xd84d1f76,
	0xdd4d1f76,0xd84d1f78,0xdd4f1f78,0xd84d1f7a,0xdd4f1f7a,0xd84d1f7c,0xdd511f7c,0xd84d1f7e,
	0xdd511f7e,0xd84d1f80,0xdd531f80,0xd84d1f82,0xdd531f82,0xd84d1f84,0xdd551f84,0xd84d1f86,
	0xdd551f86,0xd84d1f88,0xdd571f88,0xd84d1f8a,0xdd571f8a,0xd84d1f8c,0xdd591f8c,0xd84d1f8e,
	0xdd591f8e,0xd84d1f90,0xdd5b1f90,0xd84d1f92,0xdd5b1f92,0xd84d1f94,0xdd5d1f94,0xd84d1f96,
	0xdd5d1f96,0xd84d1f98,0xdd5f1f98,0xd84d1f9a,0xdd5f1f9a,0xd84d1f9c,0xdd611f9c,0xd84d1f9e,
	0xdd611f9e,0xd84d1fa0,0xdd631fa0,0xd84d1fa2,0xdd631fa2,0xd84d1fa4,0xdd651fa4,0xd84d1fa6,
	0xdd651fa6,0xd84d1fa8,0xdd671fa8,0xd84d1faa,0xdd671faa,0xd84d1fac,0xdd691fac,0xd84d1fae,
	0xdd691fae,0xd84d1fb0,0xdd6b1fb0,0xd84d1fb2,0xdd6b1fb2,0xd84d1fb4,0xdd6d1fb4,0xd84d1fb6,
	0xdd6d1fb6,0xd84d1fb8,0xdd6f1fb8,0xd84d1fba,0xdd6f1fba,0xd84d1fbc,0xdd711fbc,0xd84d1fbe,
	0xdd711fbe,0xd84d1fc0,0xdd731fc0,0xd84d1fc2,0xdd731fc2,0xd84d1fc4,0xdd751fc4,0xd84d1fc6,
	0xdd751fc6,0xd84d1fc8,0xdd771fc8,0xd84d1fca,0xdd771fca,0xd84d1fcc,0xdd791fcc,0xd84d1fce,
	0xdd791fce,0xd84d1fd0,0xdd7b1fd0,0xd84d1fd2,0xdd7b1fd2,0xd84d1fd4,0xdd7d1fd4,0xd84d1fd6,
	0xdd7d1fd6,0xd84d1fd8,0xdd7f1fd8,0xd84d1fda,0xdd7f1fda,0xd84d1fdc,0xdd811fdc,0xd84d1fde,
	0xdd811fde,0xd84d1fe0,0xdd831fe0,0xd84d1fe2,0xdd831fe2,0xd84d1fe4,0xdd851fe4,0xd84d1fe6,
	0xdd851fe6,0xd84d1fe8,0xdd871fe8,0xd84d1fea,0xdd871fea,0xd84d1fec,0xdd891fec,0xd84d1fee,
	0xdd891fee,0xd84d1ff0,0xdd8b1ff0,0xd84d1ff2,0xdd8b1ff2,0xd84d1ff4,0xdd8d1ff4,0xd84d1ff6,
	0xdd8d1ff6,0xd84d1ff8,0xdd8f1ff8,0xd84d1ffa,0xdd8f1ffa,0xd84d1ffc,0xdd911ffc,0xd84d1ffe,
	0xdd911ffe,0xd84d2000,0xdd932000,0xd84d2002,0xdd932002,0xd84d2004,0xdd952004,0xd84d2006,
	0xdd952006,0xd84d2008,0xdd972008,0xd84d200a,0xdd97200a,0xd84d200c,0xdd99200c,0xd84d200e,
	0xdd99200e,0xd84d2010,0xdd9b2010,0xd84d2012,0xdd9b2012,0xd84d2014,0xdd9d2014,0xd84d2016,
	0xdd9d2016,0xd84d2018,0xdd9f2018,0xd84d201a,0xdd9f201a,0xd84d201c,0xdda1201c,0xd84d201e,
	0xdda1201e,0xd84d2020,0xdda32020,0xd84d2022,0xdda32022,0xd84d2024,0xdda52024,0xd84d2026,
	0xdda52026,0xd84d2028,0xdda72028,0xd84d202a,0xdda7202a,0xd84d202c,0xdda9202c,0xd84d202e,
	0xdda9202e,0xd84d2030,0xddab2030,0xd84d2032,0xddab2032,0xd84d2034,0xddad2034,0xd84d2036,
	0xddad2036,0xd84d2038,0xddaf2038,0xd84d203a,0xddaf203a,0xd84d203c,0xddb1203c,0xd84d203e,
	0xddb1203e,0xd84d2040,0xddb32040,0xd84d2042,0xddb32042,0xd84d2044,0xddb52044,0xd84d2046,
	0xddb52046,0xd84d2048,0xddb72048,0xd84d204a,0xddb7204a,0xd84d204c,0xddb9204c,0xd84d204e,
	0xddb9204e,0xd84d2050,0xddbb2050,0xd84d2052,0xddbb2052,0xd84d2054,0xddbd2054,0xd84d2056,
	0xddbd2056,0xd84d2058,0xddbf2058,0xd84d205a,0xddbf205a,0xd84d205c,0xddc1205c,0xd84d205e,
	0xddc1205e,0xd84d2060,0xddc32060,0xd84d2062,0xddc32062,0xd84d2064,0xddc52064,0xd84d2066,
	0xddc52066,0xd84d2068,0xddc72068,0xd84d206a,0xddc7206a,0xd84d206c,0xddc9206c,0xd84d206e,
	0xddc9206e,0xd84d2070,0xddcb2070,0xd84d2072,0xddcb2072,0xd84d2074,0xddcd2074,0xd84d2076,
	0xddcd2076,0xd84d2078,0xddcf2078,0xd84d207a,0xddcf207a,0xd84d207c,0xddd1207c,0xd84d207e,
	0xddd1207e,0xd84d2080,0xddd32080,0xd84d2082,0xddd32082,0xd84d2084,0xddd52084,0xd84d2086,
	0xddd52086,0xd84d2088,0xddd72088,0xd84d208a,0xddd7208a,0xd84d208c,0xddd9208c,0xd84d208e,
	0xddd9208e,0xd84d2090,0xdddb2090,0xd84d2092,0xdddb2092,0xd84d2094,0xdddd2094,0xd84d2096,
	0xdddd2096,0xd84d2098,0xdddf2098,0xd84d209a,0xdddf209a,0xd84d209c,0xdde1209c,0xd84d209e,
	0xdde1209e,0xd84d20a0,0xdde320a0,0xd84d20a2,0xdde320a2,0xd84d20a4,0xdde520a4,0xd84d20a6,
	0xdde520a6,0xd84d20a8,0xdde720a8,0xd84d20aa,0xdde720aa,0xd84d20ac,0xdde920ac,0xd84d20ae,
	0xdde920ae,0xd84d20b0,0xddeb20b0,0xd84d20b2,0xddeb20b2,0xd84d20b4,0xdded20b4,0xd84d20b6,
	0xdded20b6,0xd84d20b8,0xddef20b8,0xd84d20ba,0xddef20ba,0xd84d20bc,0xddf120bc,0xd84d20be,
	0xddf120be,0xd84d20c0,0xddf320c0,0xd84d20c2,0xddf320c2,0xd84d20c4,0xddf520c4,0xd84d20c6,
	0xddf520c6,0xd84d20c8,0xddf720c8,0xd84d20ca,0xddf720ca,0xd84d20cc,0xddf920cc,0xd84d20ce,
	0xddf920ce,0xd84d20d0,0xddfb20d0,0xd84d20d2,0xddfb20d2,0xd84d20d4,0xddfd20d4,0xd84d20d6,
	0xddfd20d6,0xd84d20d8,0xddff20d8,0xd84d20da,0xddff20da,0xd84d20dc,0xde0120dc,0xd84d20de,
	0xde0120de,0xd84d20e0,0xde0320e0,0xd84d20e2,0xde0320e2,0xd84d20e4,0xde0520e4,0xd84d20e6,
	0xde0520e6,0xd84d20e8,0xde0720e8,0xd84d20ea,0xde0720ea,0xd84d20ec,0xde0920ec,0xd84d20ee,
	0xde0920ee,0xd84d20f0,0xde0b20f0,0xd84d20f2,0xde0b20f2,0xd84d20f4,0xde0d20f4,0xd84d20f6,
	0xde0d20f6,0xd84d20f8,0xde0f20f8,0xd84d20fa,0xde0f20fa,0xd84d20fc,0xde1120fc,0xd84d20fe,
	0xde1120fe,0xd84d2100,0xde132100,0xd84d2102,0xde132102,0xd84d2104,0xde152104,0xd84d2106,
	0xde152106,0xd84d2108,0xde172108,0xd84d210a,0xde17210a,0xd84d210c,0xde19210c,0xd84d210e,
	0xde19210e,0xd84d2110,0xde1b2110,0xd84d2112,0xde1b2112,0xd84d2114,0xde1d2114,0xd84d2116,
	0xde1d2116,0xd84d2118,0xde1f2118,0xd84d211a,0xde1f211a,0xd84d211c,0xde21211c,0xd84d211e,
	0xde21211e,0xd84d2120,0xde232120,0xd84d2122,0xde232122,0xd84d2124,0xde252124,0xd84d2126,
	0xde252126,0xd84d2128,0xde272128,0xd84d212a,0xde27212a,0xd84d212c,0xde29212c,0xd84d212e,
	0xde29212e,0xd84d2130,0xde2b2130,0xd84d2132,0xde2b2132,0xd84d2134,0xde2d2134,0xd84d2136,
	0xde2d2136,0xd84d2138,0xde2f2138,0xd84d213a,0xde2f213a,0xd84d213c,0xde31213c,0xd84d213e,
	0xde31213e,0xd84d2140,0xde332140,0xd84d2142,0xde332142,0xd84d2144,0xde352144,0xd84d2146,
	0xde352146,0xd84d2148,0xde372148,0xd84d214a,0xde37214a,0xd84d214c,0xde39214c,0xd84d214e,
	0xde39214e,0xd84d2150,0xde3b2150,0xd84d2152,0xde3b2152,0xd84d2154,0xde3d2154,0xd84d2156,
	0xde3d2156,0xd84d2158,0xde3f2158,0xd84d215a,0xde3f215a,0xd84d215c,0xde41215c,0xd84d215e,
	0xde41215e,0xd84d2160,0xde432160,0xd84d2162,0xde432162,0xd84d2164,0xde452164,0xd84d2166,
	0xde452166,0xd84d2168,0xde472168,0xd84d216a,0xde47216a,0xd84d216c,0xde49216c,0xd84d216e,
	0xde49216e,0xd84d2170,0xde4b2170,0xd84d2172,0xde4b2172,0xd84d2174,0xde4d2174,0xd84d2176,
	0xde4d2176,0xd84d2178,0xde4f2178,0xd84d217a,0xde4f217a,0xd84d217c,0xde51217c,0xd84d217e,
	0xde51217e,0xd84d2180,0xde532180,0xd84d2182,0xde532182,0xd84d2184,0xde552184,0xd84d2186,
	0xde552186,0xd84d2188,0xde572188,0xd84d218a,0xde57218a,0xd84d218c,0xde59218c,0xd84d218e,
	0xde59218e,0xd84d2190,0xde5b2190,0xd84d2192,0xde5b2192,0xd84d2194,0xde5d2194,0xd84d2196,
	0xde5d2196,0xd84d2198,0xde5f2198,0xd84d219a,0xde5f219a,0xd84d219c,0xde61219c,0xd84d219e,
	0xde61219e,0xd84d21a0,0xde6321a0,0xd84d21a2,0xde6321a2,0xd84d21a4,0xde6521a4,0xd84d21a6,
	0xde6521a6,0xd84d21a8,0xde6721a8,0xd84d21aa,0xde6721aa,0xd84d21ac,0xde6921ac,0xd84d21ae,
	0xde6921ae,0xd84d21b0,0xde6b21b0,0xd84d21b2,0xde6b21b2,0xd84d21b4,0xde6d21b4,0xd84d21b6,
	0xde6d21b6,0xd84d21b8,0xde6f21b8,0xd84d21ba,0xde6f21ba,0xd84d21bc,0xde7121bc,0xd84d21be,
	0xde7121be,0xd84d21c0,0xde7321c0,0xd84d21c2,0xde7321c2,0xd84d21c4,0xde7521c4,0xd84d21c6,
	0xde7521c6,0xd84d21c8,0xde7721c8,0xd84d21ca,0xde7721ca,0xd84d21cc,0xde7921cc,0xd84d21ce,
	0xde7921ce,0xd84d21d0,0xde7b21d0,0xd84d21d2,0xde7b21d2,0xd84d21d4,0xde7d21d4,0xd84d21d6,
	0xde7d21d6,0xd84d21d8,0xde7f21d8,0xd84d21da,0xde7f21da,0xd84d21dc,0xde8121dc,0xd84d21de,
	0xde8121de,0xd84d21e0,0xde8321e0,0xd84d21e2,0xde8321e2,0xd84d21e4,0xde8521e4,0xd84d21e6,
	0xde8521e6,0xd84d21e8,0xde8721e8,0xd84d21ea,0xde8721ea,0xd84d21ec,0xde8921ec,0xd84d21ee,
	0xde8921ee,0xd84d21f0,0xde8b21f0,0xd84d21f2,0xde8b21f2,0xd84d21f4,0xde8d21f4,0xd84d21f6,
	0xde8d21f6,0xd84d21f8,0xde8f21f8,0xd84d21fa,0xde8f21fa,0xd84d21fc,0xde9121fc,0xd84d21fe,
	0xde9121fe,0xd84d2200,0xde932200,0xd84d2202,0xde932202,0xd84d2204,0xde952204,0xd84d2206,
	0xde952206,0xd84d2208,0xde972208,0xd84d220a,0xde97220a,0xd84d220c,0xde99220c,0xd84d220e,
	0xde99220e,0xd84d2210,0xde9b2210,0xd84d2212,0xde9b2212,0xd84d2214,0xde9d2214,0xd84d2216,
	0xde9d2216,0xd84d2218,0xde9f2218,0xd84d221a,0xde9f221a,0xd84d221c,0xdea1221c,0xd84d221e,
	0xdea1221e,0xd84d2220,0xdea32220,0xd84d2222,0xdea32222,0xd84d2224,0xdea52224,0xd84d2226,
	0xdea52226,0xd84d2228,0xdea72228,0xd84d222a,0xdea7222a,0xd84d222c,0xdea9222c,0xd84d222e,
	0xdea9222e,0xd84d2230,0xdeab2230,0xd84d2232,0xdeab2232,0xd84d2234,0xdead2234,0xd84d2236,
	0xdead2236,0xd84d2238,0xdeaf2238,0xd84d223a,0xdeaf223a,0xd84d223c,0xdeb1223c,0xd84d223e,
	0xdeb1223e,0xd84d2240,0xdeb32240,0xd84d2242,0xdeb32242,0xd84d2244,0xdeb52244,0xd84d2246,
	0xdeb52246,0xd84d2248,0xdeb72248,0xd84d224a,0xdeb7224a,0xd84d224c,0xdeb9224c,0xd84d224e,
	0xdeb9224e,0xd84d2250,0xdebb2250,0xd84d2252,0xdebb2252,0xd84d2254,0xdebd2254,0xd84d2256,
	0xdebd2256,0xd84d2258,0xdebf2258,0xd84d225a,0xdebf225a,0xd84d225c,0xdec1225c,0xd84d225e,
	0xdec1225e,0xd84d2260,0xdec32260,0xd84d2262,0xdec32262,0xd84d2264,0xdec52264,0xd84d2266,
	0xdec52266,0xd84d2268,0xdec72268,0xd84d226a,0xdec7226a,0xd84d226c,0xdec9226c,0xd84d226e,
	0xdec9226e,0xd84d2270,0xdecb2270,0xd84d2272,0xdecb2272,0xd84d2274,0xdecd2274,0xd84d2276,
	0xdecd2276,0xd84d2278,0xdecf2278,0xd84d227a,0xdecf227a,0xd84d227c,0xded1227c,0xd84d227e,
	0xded1227e,0xd84d2280,0xded32280,0xd84d2282,0xded32282,0xd84d2284,0xded52284,0xd84d2286,
	0xded52286,0xd84d2288,0xded72288,0xd84d228a,0xded7228a,0xd84d228c,0xded9228c,0xd84d228e,
	0xded9228e,0xd84d2290,0xdedb2290,0xd84d2292,0xdedb2292,0xd84d2294,0xdedd2294,0xd84d2296,
	0xdedd2296,0xd84d2298,0xdedf2298,0xd84d229a,0xdedf229a,0xd84d229c,0xdee1229c,0xd84d229e,
	0xdee1229e,0xd84d22a0,0xdee322a0,0xd84d22a2,0xdee322a2,0xd84d22a4,0xdee522a4,0xd84d22a6,
	0xdee522a6,0xd84d22a8,0xdee722a8,0xd84d22aa,0xdee722aa,0xd84d22ac,0xdee922ac,0xd84d22ae,
	0xdee922ae,0xd84d22b0,0xdeeb22b0,0xd84d22b2,0xdeeb22b2,0xd84d22b4,0xdeed22b4,0xd84d22b6,
	0xdeed22b6,0xd84d22b8,0xdeef22b8,0xd84d22ba,0xdeef22ba,0xd84d22bc,0xdef122bc,0xd84d22be,
	0xdef122be,0xd84d22c0,0xdef322c0,0xd84d22c2,0xdef322c2,0xd84d22c4,0xdef522c4,0xd84d22c6,
	0xdef522c6,0xd84d22c8,0xdef722c8,0xd84d22ca,0xdef722ca,0xd84d22cc,0xdef922cc,0xd84d22ce,
	0xdef922ce,0xd84d22d0,0xdefb22d0,0xd84d22d2,0xdefb22d2,0xd84d22d4,0xdefd22d4,0xd84d22d6,
	0xdefd22d6,0xd84d22d8,0xdeff22d8,0xd84d22da,0xdeff22da,0xd84d22dc,0xdf0122dc,0xd84d22de,
	0xdf0122de,0xd84d22e0,0xdf0322e0,0xd84d22e2,0xdf0322e2,0xd84d22e4,0xdf0522e4,0xd84d22e6,
	0xdf0522e6,0xd84d22e8,0xdf0722e8,0xd84d22ea,0xdf0722ea,0xd84d22ec,0xdf0922ec,0xd84d22ee,
	0xdf0922ee,0xd84d22f0,0xdf0b22f0,0xd84d22f2,0xdf0b22f2,0xd84d22f4,0xdf0d22f4,0xd84d22f6,
	0xdf0d22f6,0xd84d22f8,0xdf0f22f8,0xd84d22fa,0xdf0f22fa,0xd84d22fc,0xdf1122fc,0xd84d22fe,
	0xdf1122fe,0xd84d2300,0xdf132300,0xd84d2302,0xdf132302,0xd84d2304,0xdf152304,0xd84d2306,
	0xdf152306,0xd84d2308,0xdf172308,0xd84d230a,0xdf17230a,0xd84d230c,0xdf19230c,0xd84d230e,
	0xdf19230e,0xd84d2310,0xdf1b2310,0xd84d2312,0xdf1b2312,0xd84d2314,0xdf1d2314,0xd84d2316,
	0xdf1d2316,0xd84d2318,0xdf1f2318,0xd84d231a,0xdf1f231a,0xd84d231c,0xdf21231c,0xd84d231e,
	0xdf21231e,0xd84d2320,0xdf232320,0xd84d2322,0xdf232322,0xd84d2324,0xdf252324,0xd84d2326,
	0xdf252326,0xd84d2328,0xdf272328,0xd84d232a,0xdf27232a,0xd84d232c,0xdf29232c,0xd84d232e,
	0xdf29232e,0xd84d2330,0xdf2b2330,0xd84d2332,0xdf2b2332,0xd84d2334,0xdf2d2334,0xd84d2336,
	0xdf2d2336,0xd84d2338,0xdf2f2338,0xd84d233a,0xdf2f233a,0xd84d233c,0xdf31233c,0xd84d233e,
	0xdf31233e,0xd84d2340,0xdf332340,0xd84d2342,0xdf332342,0xd84d2344,0xdf352344,0xd84d2346,
	0xdf352346,0xd84d2348,0xdf372348,0xd84d234a,0xdf37234a,0xd84d234c,0xdf39234c,0xd84d234e,
	0xdf39234e,0xd84d2350,0xdf3b2350,0xd84d2352,0xdf3b2352,0xd84d2354,0xdf3d2354,0xd84d2356,
	0xdf3d2356,0xd84d2358,0xdf3f2358,0xd84d235a,0xdf3f235a,0xd84d235c,0xdf41235c,0xd84d235e,
	0xdf41235e,0xd84d2360,0xdf432360,0xd84d2362,0xdf432362,0xd84d2364,0xdf452364,0xd84d2366,
	0xdf452366,0xd84d2368,0xdf472368,0xd84d236a,0xdf47236a,0xd84d236c,0xdf49236c,0xd84d236e,
	0xdf49236e,0xd84d2370,0xdf4b2370,0xd84d2372,0xdf4b2372,0xd84d2374,0xdf4d2374,0xd84d2376,
	0xdf4d2376,0xd84d2378,0xdf4f2378,0xd84d237a,0xdf4f237a,0xd84d237c,0xdf51237c,0xd84d237e,
	0xdf51237e,0xd84d2380,0xdf532380,0xd84d2382,0xdf532382,0xd84d2384,0xdf552384,0xd84d2386,
	0xdf552386,0xd84d2388,0xdf572388,0xd84d238a,0xdf57238a,0xd84d238c,0xdf59238c,0xd84d238e,
	0xdf59238e,0xd84d2390,0xdf5b2390,0xd84d2392,0xdf5b2392,0xd84d2394,0xdf5d2394,0xd84d2396,
	0xdf5d2396,0xd84d2398,0xdf5f2398,0xd84d239a,0xdf5f239a,0xd84d239c,0xdf61239c,0xd84d239e,
	0xdf61239e,0xd84d23a0,0xdf6323a0,0xd84d23a2,0xdf6323a2,0xd84d23a4,0xdf6523a4,0xd84d23a6,
	0xdf6523a6,0xd84d23a8,0xdf6723a8,0xd84d23aa,0xdf6723aa,0xd84d23ac,0xdf6923ac,0xd84d23ae,
	0xdf6923ae,0xd84d23b0,0xdf6b23b0,0xd84d23b2,0xdf6b23b2,0xd84d23b4,0xdf6d23b4,0xd84d23b6,
	0xdf6d23b6,0xd84d23b8,0xdf6f23b8,0xd84d23ba,0xdf6f23ba,0xd84d23bc,0xdf7123bc,0xd84d23be,
	0xdf7123be,0xd84d23c0,0xdf7323c0,0xd84d23c2,0xdf7323c2,0xd84d23c4,0xdf7523c4,0xd84d23c6,
	0xdf7523c6,0xd84d23c8,0xdf7723c8,0xd84d23ca,0xdf7723ca,0xd84d23cc,0xdf7923cc,0xd84d23ce,
	0xdf7923ce,0xd84d23d0,0xdf7b23d0,0xd84d23d2,0xdf7b23d2,0xd84d23d4,0xdf7d23d4,0xd84d23d6,
	0xdf7d23d6,0xd84d23d8,0xdf7f23d8,0xd84d23da,0xdf7f23da,0xd84d23dc,0xdf8123dc,0xd84d23de,
	0xdf8123de,0xd84d23e0,0xdf8323e0,0xd84d23e2,0xdf8323e2,0xd84d23e4,0xdf8523e4,0xd84d23e6,
	0xdf8523e6,0xd84d23e8,0xdf8723e8,0xd84d23ea,0xdf8723ea,0xd84d23ec,0xdf8923ec,0xd84d23ee,
	0xdf8923ee,0xd84d23f0,0xdf8b23f0,0xd84d23f2,0xdf8b23f2,0xd84d23f4,0xdf8d23f4,0xd84d23f6,
	0xdf8d23f6,0xd84d23f8,0xdf8f23f8,0xd84d23fa,0xdf8f23fa,0xd84d23fc,0xdf9123fc,0xd84d23fe,
	0xdf9123fe,0xd84d2400,0xdf932400,0xd84d2402,0xdf932402,0xd84d2404,0xdf952404,0xd84d2406,
	0xdf952406,0xd84d2408,0xdf972408,0xd84d240a,0xdf97240a,0xd84d240c,0xdf99240c,0xd84d240e,
	0xdf99240e,0xd84d2410,0xdf9b2410,0xd84d2412,0xdf9b2412,0xd84d2414,0xdf9d2414,0xd84d2416,
	0xdf9d2416,0xd84d2418,0xdf9f2418,0xd84d241a,0xdf9f241a,0xd84d241c,0xdfa1241c,0xd84d241e,
	0xdfa1241e,0xd84d2420,0xdfa32420,0xd84d2422,0xdfa32422,0xd84d2424,0xdfa52424,0xd84d2426,
	0xdfa52426,0xd84d2428,0xdfa72428,0xd84d242a,0xdfa7242a,0xd84d242c,0xdfa9242c,0xd84d242e,
	0xdfa9242e,0xd84d2430,0xdfab2430,0xd84d2432,0xdfab2432,0xd84d2434,0xdfad2434,0xd84d2436,
	0xdfad2436,0xd84d2438,0xdfaf2438,0xd84d243a,0xdfaf243a,0xd84d243c,0xdfb1243c,0xd84d243e,
	0xdfb1243e,0xd84d2440,0xdfb32440,0xd84d2442,0xdfb32442,0xd84d2444,0xdfb52444,0xd84d2446,
	0xdfb52446,0xd84d2448,0xdfb72448,0xd84d244a,0xdfb7244a,0xd84d244c,0xdfb9244c,0xd84d244e,
	0xdfb9244e,0xd84d2450,0xdfbb2450,0xd84d2452,0xdfbb2452,0xd84d2454,0xdfbd2454,0xd84d2456,
	0xdfbd2456,0xd84d2458,0xdfbf2458,0xd84d245a,0xdfbf245a,0xd84d245c,0xdfc1245c,0xd84d245e,
	0xdfc1245e,0xd84d2460,0xdfc32460,0xd84d2462,0xdfc32462,0xd84d2464,0xdfc52464,0xd84d2466,
	0xdfc52466,0xd84d2468,0xdfc72468,0xd84d246a,0xdfc7246a,0xd84d246c,0xdfc9246c,0xd84d246e,
	0xdfc9246e,0xd84d2470,0xdfcb2470,0xd84d2472,0xdfcb2472,0xd84d2474,0xdfcd2474,0xd84d2476,
	0xdfcd2476,0xd84d2478,0xdfcf2478,0xd84d247a,0xdfcf247a,0xd84d247c,0xdfd1247c,0xd84d247e,
	0xdfd1247e,0xd84d2480,0xdfd32480,0xd84d2482,0xdfd32482,0xd84d2484,0xdfd52484,0xd84d2486,
	0xdfd52486,0xd84d2488,0xdfd72488,0xd84d248a,0xdfd7248a,0xd84d248c,0xdfd9248c,0xd84d248e,
	0xdfd9248e,0xd84d2490,0xdfdb2490,0xd84d2492,0xdfdb2492,0xd84d2494,0xdfdd2494,0xd84d2496,
	0xdfdd2496,0xd84d2498,0xdfdf2498,0xd84d249a,0xdfdf249a,0xd84d249c,0xdfe1249c,0xd84d249e,
	0xdfe1249e,0xd84d24a0,0xdfe324a0,0xd84d24a2,0xdfe324a2,0xd84d24a4,0xdfe524a4,0xd84d24a6,
	0xdfe524a6,0xd84d24a8,0xdfe724a8,0xd84d24aa,0xdfe724aa,0xd84d24ac,0xdfe924ac,0xd84d24ae,
	0xdfe924ae,0xd84d24b0,0xdfeb24b0,0xd84d24b2,0xdfeb24b2,0xd84d24b4,0xdfed24b4,0xd84d24b6,
	0xdfed24b6,0xd84d24b8,0xdfef24b8,0xd84d24ba,0xdfef24ba,0xd84d24bc,0xdff124bc,0xd84d24be,
	0xdff124be,0xd84d24c0,0xdff324c0,0xd84d24c2,0xdff324c2,0xd84d24c4,0xdff524c4,0xd84d24c6,
	0xdff524c6,0xd84d24c8,0xdff724c8,0xd84d24ca,0xdff724ca,0xd84d24cc,0xdff924cc,0xd84d24ce,
	0xdff924ce,0xd84d24d0,0xdffb24d0,0xd84d24d2,0xdffb24d2,0xd84d24d4,0xdffd24d4,0xd84d24d6,
	0xdffd24d6,0xd84d24d8,0xdfff24d8,0xd84d24da,0xdfff24da,0xd84d24dc,0xdc0124dc,0xd84d24de,
	0xdc0124de,0xd84d24e0,0xdc0324e0,0xd84d24e2,0xdc0324e2,0xd84d24e4,0xdc0524e4,0xd84d24e6,
	0xdc0524e6,0xd84d24e8,0xdc0724e8,0xd84d24ea,0xdc0724ea,0xd84d24ec,0xdc0924ec,0xd84d24ee,
	0xdc0924ee,0xd84d24f0,0xdc0b24f0,0xd84d24f2,0xdc0b24f2,0xd84d24f4,0xdc0d24f4,0xd84d24f6,
	0xdc0d24f6,0xd84d24f8,0xdc0f24f8,0xd84d24fa,0xdc0f24fa,0xd84d24fc,0xdc1124fc,0xd84d24fe,
	0xdc1124fe,0xd84d2500,0xdc132500,0xd84d2502,0xdc132502,0xd84d2504,0xdc152504,0xd84d2506,
	0xdc152506,0xd84d2508,0xdc172508,0xd84d250a,0xdc17250a,0xd84d250c,0xdc19250c,0xd84d250e,
	0xdc19250e,0xd84d2510,0xdc1b2510,0xd84d2512,0xdc1b2512,0xd84d2514,0xdc1d2514,0xd84d2516,
	0xdc1d2516,0xd84d2518,0xdc1f2518,0xd84d251a,0xdc1f251a,0xd84d251c,0xdc21251c,0xd84d251e,
	0xdc21251e,0xd84d2520,0xdc232520,0xd84d2522,0xdc232522,0xd84d2524,0xdc252524,0xd84d2526,
	0xdc252526,0xd84d2528,0xdc272528,0xd84d252a,0xdc27252a,0xd84d252c,0xdc29252c,0xd84d252e,
	0xdc29252e,0xd84d2530,0xdc2b2530,0xd84d2532,0xdc2b2532,0xd84d2534,0xdc2d2534,0xd84d2536,
	0xdc2d2536,0xd84d2538,0xdc2f2538,0xd84d253a,0xdc2f253a,0xd84d253c,0xdc31253c,0xd84d253e,
	0xdc31253e,0xd84d2540,0xdc332540,0xd84d2542,0xdc332542,0xd84d2544,0xdc352544,0xd84d2546,
	0xdc352546,0xd84d2548,0xdc372548,0xd84d254a,0xdc37254a,0xd84d254c,0xdc39254c,0xd84d254e,
	0xdc39254e,0xd84d2550,0xdc3b2550,0xd84d2552,0xdc3b2552,0xd84d2554,0xdc3d2554,0xd84d2556,
	0xdc3d2556,0xd84d2558,0xdc3f2558,0xd84d255a,0xdc3f255a,0xd84d255c,0xdc41255c,0xd84d255e,
	0xdc41255e,0xd84d2560,0xdc432560,0xd84d2562,0xdc432562,0xd84d2564,0xdc452564,0xd84d2566,
	0xdc452566,0xd84d2568,0xdc472568,0xd84d256a,0xdc47256a,0xd84d256c,0xdc49256c,0xd84d256e,
	0xdc49256e,0xd84d2570,0xdc4b2570,0xd84d2572,0xdc4b2572,0xd84d2574,0xdc4d2574,0xd84d2576,
	0xdc4d2576,0xd84d2578,0xdc4f2578,0xd84d257a,0xdc4f257a,0xd84d257c,0xdc51257c,0xd84d257e,
	0xdc51257e,0xd84d2580,0xdc532580,0xd84d2582,0xdc532582,0xd84d2584,0xdc552584,0xd84d2586,
	0xdc552586,0xd84d2588,0xdc572588,0xd84d258a,0xdc57258a,0xd84d258c,0xdc59258c,0xd84d258e,
	0xdc59258e,0xd84d2590,0xdc5b2590,0xd84d2592,0xdc5b2592,0xd84d2594,0xdc5d2594,0xd84d2596,
	0xdc5d2596,0xd84d2598,0xdc5f2598,0xd84d259a,0xdc5f259a,0xd84d259c,0xdc61259c,0xd84d259e,
	0xdc61259e,0xd84d25a0,0xdc6325a0,0xd84d25a2,0xdc6325a2,0xd84d25a4,0xdc6525a4,0xd84d25a6,
	0xdc6525a6,0xd84d25a8,0xdc6725a8,0xd84d25aa,0xdc6725aa,0xd84d25ac,0xdc6925ac,0xd84d25ae,
	0xdc6925ae,0xd84d25b0,0xdc6b25b0,0xd84d25b2,0xdc6b25b2,0xd84d25b4,0xdc6d25b4,0xd84d25b6,
	0xdc6d25b6,0xd84d25b8,0xdc6f25b8,0xd84d25ba,0xdc6f25ba,0xd84d25bc,0xdc7125bc,0xd84d25be,
	0xdc7125be,0xd84d25c0,0xdc7325c0,0xd84d25c2,0xdc7325c2,0xd84d25c4,0xdc7525c4,0xd84d25c6,
	0xdc7525c6,0xd84d25c8,0xdc7725c8,0xd84d25ca,0xdc7725ca,0xd84d25cc,0xdc7925cc,0xd84d25ce,
	0xdc7925ce,0xd84d25d0,0xdc7b25d0,0xd84d25d2,0xdc7b25d2,0xd84d25d4,0xdc7d25d4,0xd84d25d6,
	0xdc7d25d6,0xd84d25d8,0xdc7f25d8,0xd84d25da,0xdc7f25da,0xd84d25dc,0xdc8125dc,0xd84d25de,
	0xdc8125de,0xd84d25e0,0xdc8325e0,0xd84d25e2,0xdc8325e2,0xd84d25e4,0xdc8525e4,0xd84d25e6,
	0xdc8525e6,0xd84d25e8,0xdc8725e8,0xd84d25ea,0xdc8725ea,0xd84d25ec,0xdc8925ec,0xd84d25ee,
	0xdc8925ee,0xd84d25f0,0xdc8b25f0,0xd84d25f2,0xdc8b25f2,0xd84d25f4,0xdc8d25f4,0xd84d25f6,
	0xdc8d25f6,0xd84d25f8,0xdc8f25f8,0xd84d25fa,0xdc8f25fa,0xd84d25fc,0xdc9125fc,0xd84d25fe,
	0xdc9125fe,0xd84d2600,0xdc932600,0xd84d2602,0xdc932602,0xd84d2604,0xdc952604,0xd84d2606,
	0xdc952606,0xd84d2608,0xdc972608,0xd84d260a,0xdc97260a,0xd84d260c,0xdc99260c,0xd84d260e,
	0xdc99260e,0xd84d2610,0xdc9b2610,0xd84d2612,0xdc9b2612,0xd84d2614,0xdc9d2614,0xd84d2616,
	0xdc9d2616,0xd84d2618,0xdc9f2618,0xd84d261a,0xdc9f261a,0xd84d261c,0xdca1261c,0xd84d261e,
	0xdca1261e,0xd84d2620,0xdca32620,0xd84d2622,0xdca32622,0xd84d2624,0xdca52624,0xd84d2626,
	0xdca52626,0xd84d2628,0xdca72628,0xd84d262a,0xdca7262a,0xd84d262c,0xdca9262c,0xd84d262e,
	0xdca9262e,0xd84d2630,0xdcab2630,0xd84d2632,0xdcab2632,0xd84d2634,0xdcad2634,0xd84d2636,
	0xdcad2636,0xd84d2638,0xdcaf2638,0xd84d263a,0xdcaf263a,0xd84d263c,0xdcb1263c,0xd84d263e,
	0xdcb1263e,0xd84d2640,0xdcb32640,0xd84d2642,0xdcb32642,0xd84d2644,0xdcb52644,0xd84d2646,
	0xdcb52646,0xd84d2648,0xdcb72648,0xd84d264a,0xdcb7264a,0xd84d264c,0xdcb9264c,0xd84d264e,
	0xdcb9264e,0xd84d2650,0xdcbb2650,0xd84d2652,0xdcbb2652,0xd84d2654,0xdcbd2654,0xd84d2656,
	0xdcbd2656,0xd84d2658,0xdcbf2658,0xd84d265a,0xdcbf265a,0xd84d265c,0xdcc1265c,0xd84d265e,
	0xdcc1265e,0xd84d2660,0xdcc32660,0xd84d2662,0xdcc32662,0xd84d2664,0xdcc52664,0xd84d2666,
	0xdcc52666,0xd84d2668,0xdcc72668,0xd84d266a,0xdcc7266a,0xd84d266c,0xdcc9266c,0xd84d266e,
	0xdcc9266e,0xd84d2670,0xdccb2670,0xd84d2672,0xdccb2672,0xd84d2674,0xdccd2674,0xd84d2676,
	0xdccd2676,0xd84d2678,0xdccf2678,0xd84d267a,0xdccf267a,0xd84d267c,0xdcd1267c,0xd84d267e,
	0xdcd1267e,0xd84d2680,0xdcd32680,0xd84d2682,0xdcd32682,0xd84d2684,0xdcd52684,0xd84d2686,
	0xdcd52686,0xd84d2688,0xdcd72688,0xd84d268a,0xdcd7268a,0xd84d268c,0xdcd9268c,0xd84d268e,
	0xdcd9268e,0xd84d2690,0xdcdb2690,0xd84d2692,0xdcdb2692,0xd84d2694,0xdcdd2694,0xd84d2696,
	0xdcdd2696,0xd84d2698,0xdcdf2698,0xd84d269a,0xdcdf269a,0xd84d269c,0xdce1269c,0xd84d269e,
	0xdce1269e,0xd84d26a0,0xdce326a0,0xd84d26a2,0xdce326a2,0xd84d26a4,0xdce526a4,0xd84d26a6,
	0xdce526a6,0xd84d26a8,0xdce726a8,0xd84d26aa,0xdce726aa,0xd84d26ac,0xdce926ac,0xd84d26ae,
	0xdce926ae,0xd84d26b0,0xdceb26b0,0xd84d26b2,0xdceb26b2,0xd84d26b4,0xdced26b4,0xd84d26b6,
	0xdced26b6,0xd84d26b8,0xdcef26b8,0xd84d26ba,0xdcef26ba,0xd84d26bc,0xdcf126bc,0xd84d26be,
	0xdcf126be,0xd84d26c0,0xdcf326c0,0xd84d26c2,0xdcf326c2,0xd84d26c4,0xdcf526c4,0xd84d26c6,
	0xdcf526c6,0xd84d26c8,0xdcf726c8,0xd84d26ca,0xdcf726ca,0xd84d26cc,0xdcf926cc,0xd84d26ce,
	0xdcf926ce,0xd84d26d0,0xdcfb26d0,0xd84d26d2,0xdcfb26d2,0xd84d26d4,0xdcfd26d4,0xd84d26d6,
	0xdcfd26d6,0xd84d26d8,0xdcff26d8,0xd84d26da,0xdcff26da,0xd84d26dc,0xdd0126dc,0xd84d26de,
	0xdd0126de,0xd84d26e0,0xdd0326e0,0xd84d26e2,0xdd0326e2,0xd84d26e4,0xdd0526e4,0xd84d26e6,
	0xdd0526e6,0xd84d26e8,0xdd0726e8,0xd84d26ea,0xdd0726ea,0xd84d26ec,0xdd0926ec,0xd84d26ee,
	0xdd0926ee,0xd84d26f0,0xdd0b26f0,0xd84d26f2,0xdd0b26f2,0xd84d26f4,0xdd0d26f4,0xd84d26f6,
	0xdd0d26f6,0xd84d26f8,0xdd0f26f8,0xd84d26fa,0xdd0f26fa,0xd84d26fc,0xdd1126fc,0xd84d26fe,
	0xdd1126fe,0xd84d2700,0xdd132700,0xd84d2702,0xdd132702,0xd84d2704,0xdd152704,0xd84d2706,
	0xdd152706,0xd84d2708,0xdd172708,0xd84d270a,0xdd17270a,0xd84d270c,0xdd19270c,0xd84d270e,
	0xdd19270e,0xd84d2710,0xdd1b2710,0xd84d2712,0xdd1b2712,0xd84d2714,0xdd1d2714,0xd84d2716,
	0xdd1d2716,0xd84d2718,0xdd1f2718,0xd84d271a,0xdd1f271a,0xd84d271c,0xdd21271c,0xd84d271e,
	0xdd21271e,0xd84d2720,0xdd232720,0xd84d2722,0xdd232722,0xd84d2724,0xdd252724,0xd84d2726,
	0xdd252726,0xd84d2728,0xdd272728,0xd84d272a,0xdd27272a,0xd84d272c,0xdd29272c,0xd84d272e,
	0xdd29272e,0xd84d2730,0xdd2b2730,0xd84d2732,0xdd2b2732,0xd84d2734,0xdd2d2734,0xd84d2736,
	0xdd2d2736,0xd84d2738,0xdd2f2738,0xd84d273a,0xdd2f273a,0xd84d273c,0xdd31273c,0xd84d273e,
	0xdd31273e,0xd84d2740,0xdd332740,0xd84d2742,0xdd332742,0xd84d2744,0xdd352744,0xd84d2746,
	0xdd352746,0xd84d2748,0xdd372748,0xd84d274a,0xdd37274a,0xd84d274c,0xdd39274c,0xd84d274e,
	0xdd39274e,0xd84d2750,0xdd3b2750,0xd84d2752,0xdd3b2752,0xd84d2754,0xdd3d2754,0xd84d2756,
	0xdd3d2756,0xd84d2758,0xdd3f2758,0xd84d275a,0xdd3f275a,0xd84d275c,0xdd41275c,0xd84d275e,
	0xdd41275e,0xd84d2760,0xdd432760,0xd84d2762,0xdd432762,0xd84d2764,0xdd452764,0xd84d2766,
	0xdd452766,0xd84d2768,0xdd472768,0xd84d276a,0xdd47276a,0xd84d276c,0xdd49276c,0xd84d276e,
	0xdd49276e,0xd84d2770,0xdd4b2770,0xd84d2772,0xdd4b2772,0xd84d2774,0xdd4d2774,0xd84d2776,
	0xdd4d2776,0xd84d2778,0xdd4f2778,0xd84d277a,0xdd4f277a,0xd84d277c,0xdd51277c,0xd84d277e,
	0xdd51277e,0xd84d2780,0xdd532780,0xd84d2782,0xdd532782,0xd84d2784,0xdd552784,0xd84d2786,
	0xdd552786,0xd84d2788,0xdd572788,0xd84d278a,0xdd57278a,0xd84d278c,0xdd59278c,0xd84d278e,
	0xdd59278e,0xd84d2790,0xdd5b2790,0xd84d2792,0xdd5b2792,0xd84d2794,0xdd5d2794,0xd84d2796,
	0xdd5d2796,0xd84d2798,0xdd5f2798,0xd84d279a,0xdd5f279a,0xd84d279c,0xdd61279c,0xd84d279e,
	0xdd61279e,0xd84d27a0,0xdd6327a0,0xd84d27a2,0xdd6327a2,0xd84d27a4,0xdd6527a4,0xd84d27a6,
	0xdd6527a6,0xd84d27a8,0xdd6727a8,0xd84d27aa,0xdd6727aa,0xd84d27ac,0xdd6927ac,0xd84d27ae,
	0xdd6927ae,0xd84d27b0,0xdd6b27b0,0xd84d27b2,0xdd6b27b2,0xd84d27b4,0xdd6d27b4,0xd84d27b6,
	0xdd6d27b6,0xd84d27b8,0xdd6f27b8,0xd84d27ba,0xdd6f27ba,0xd84d27bc,0xdd7127bc,0xd84d27be,
	0xdd7127be,0xd84d27c0,0xdd7327c0,0xd84d27c2,0xdd7327c2,0xd84d27c4,0xdd7527c4,0xd84d27c6,
	0xdd7527c6,0xd84d27c8,0xdd7727c8,0xd84d27ca,0xdd7727ca,0xd84d27cc,0xdd7927cc,0xd84d27ce,
	0xdd7927ce,0xd84d27d0,0xdd7b27d0,0xd84d27d2,0xdd7b27d2,0xd84d27d4,0xdd7d27d4,0xd84d27d6,
	0xdd7d27d6,0xd84d27d8,0xdd7f27d8,0xd84d27da,0xdd7f27da,0xd84d27dc,0xdd8127dc,0xd84d27de,
	0xdd8127de,0xd84d27e0,0xdd8327e0,0xd84d27e2,0xdd8327e2,0xd84d27e4,0xdd8527e4,0xd84d27e6,
	0xdd8527e6,0xd84d27e8,0xdd8727e8,0xd84d27ea,0xdd8727ea,0xd84d27ec,0xdd8927ec,0xd84d27ee,
	0xdd8927ee,0xd84d27f0,0xdd8b27f0,0xd84d27f2,0xdd8b27f2,0xd84d27f4,0xdd8d27f4,0xd84d27f6,
	0xdd8d27f6,0xd84d27f8,0xdd8f27f8,0xd84d27fa,0xdd8f27fa,0xd84d27fc,0xdd9127fc,0xd84d27fe,
	0xdd9127fe,0xd84d2800,0xdd932800,0xd84d2802,0xdd932802,0xd84d2804,0xdd952804,0xd84d2806,
	0xdd952806,0xd84d2808,0xdd972808,0xd84d280a,0xdd97280a,0xd84d280c,0xdd99280c,0xd84d280e,
	0xdd99280e,0xd84d2810,0xdd9b2810,0xd84d2812,0xdd9b2812,0xd84d2814,0xdd9d2814,0xd84d2816,
	0xdd9d2816,0xd84d2818,0xdd9f2818,0xd84d281a,0xdd9f281a,0xd84d281c,0xdda1281c,0xd84d281e,
	0xdda1281e,0xd84d2820,0xdda32820,0xd84d2822,0xdda32822,0xd84d2824,0xdda52824,0xd84d2826,
	0xdda52826,0xd84d2828,0xdda72828,0xd84d282a,0xdda7282a,0xd84d282c,0xdda9282c,0xd84d282e,
	0xdda9282e,0xd84d2830,0xddab2830,0xd84d2832,0xddab2832,0xd84d2834,0xddad2834,0xd84d2836,
	0xddad2836,0xd84d2838,0xddaf2838,0xd84d283a,0xddaf283a,0xd84d283c,0xddb1283c,0xd84d283e,
	0xddb1283e,0xd84d2840,0xddb32840,0xd84d2842,0xddb32842,0xd84d2844,0xddb52844,0xd84d2846,
	0xddb52846,0xd84d2848,0xddb72848,0xd84d284a,0xddb7284a,0xd84d284c,0xddb9284c,0xd84d284e,
	0xddb9284e,0xd84d2850,0xddbb2850,0xd84d2852,0xddbb2852,0xd84d2854,0xddbd2854,0xd84d2856,
	0xddbd2856,0xd84d2858,0xddbf2858,0xd84d285a,0xddbf285a,0xd84d285c,0xddc1285c,0xd84d285e,
	0xddc1285e,0xd84d2860,0xddc32860,0xd84d2862,0xddc32862,0xd84d2864,0xddc52864,0xd84d2866,
	0xddc52866,0xd84d2868,0xddc72868,0xd84d286a,0xddc7286a,0xd84d286c,0xddc9286c,0xd84d286e,
	0xddc9286e,0xd84d2870,0xddcb2870,0xd84d2872,0xddcb2872,0xd84d2874,0xddcd2874,0xd84d2876,
	0xddcd2876,0xd84d2878,0xddcf2878,0xd84d287a,0xddcf287a,0xd84d287c,0xddd1287c,0xd84d287e,
	0xddd1287e,0xd84d2880,0xddd32880,0xd84d2882,0xddd32882,0xd84d2884,0xddd52884,0xd84d2886,
	0xddd52886,0xd84d2888,0xddd72888,0xd84d288a,0xddd7288a,0xd84d288c,0xddd9288c,0xd84d288e,
	0xddd9288e,0xd84d2890,0xdddb2890,0xd84d2892,0xdddb2892,0xd84d2894,0xdddd2894,0xd84d2896,
	0xdddd2896,0xd84d2898,0xdddf2898,0xd84d289a,0xdddf289a,0xd84d289c,0xdde1289c,0xd84d289e,
	0xdde1289e,0xd84d28a0,0xdde328a0,0xd84d28a2,0xdde328a2,0xd84d28a4,0xdde528a4,0xd84d28a6,
	0xdde528a6,0xd84d28a8,0xdde728a8,0xd84d28aa,0xdde728aa,0xd84d28ac,0xdde928ac,0xd84d28ae,
	0xdde928ae,0xd84d28b0,0xddeb28b0,0xd84d28b2,0xddeb28b2,0xd84d28b4,0xdded28b4,0xd84d28b6,
	0xdded28b6,0xd84d28b8,0xddef28b8,0xd84d28ba,0xddef28ba,0xd84d28bc,0xddf128bc,0xd84d28be,
	0xddf128be,0xd84d28c0,0xddf328c0,0xd84d28c2,0xddf328c2,0xd84d28c4,0xddf528c4,0xd84d28c6,
	0xddf528c6,0xd84d28c8,0xddf728c8,0xd84d28ca,0xddf728ca,0xd84d28cc,0xddf928cc,0xd84d28ce,
	0xddf928ce,0xd84d28d0,0xddfb28d0,0xd84d28d2,0xddfb28d2,0xd84d28d4,0xddfd28d4,0xd84d28d6,
	0xddfd28d6,0xd84d28d8,0xddff28d8,0xd84d28da,0xddff28da,0xd84d28dc,0xde0128dc,0xd84d28de,
	0xde0128de,0xd84d28e0,0xde0328e0,0xd84d28e2,0xde0328e2,0xd84d28e4,0xde0528e4,0xd84d28e6,
	0xde0528e6,0xd84d28e8,0xde0728e8,0xd84d28ea,0xde0728ea,0xd84d28ec,0xde0928ec,0xd84d28ee,
	0xde0928ee,0xd84d28f0,0xde0b28f0,0xd84d28f2,0xde0b28f2,0xd84d28f4,0xde0d28f4,0xd84d28f6,
	0xde0d28f6,0xd84d28f8,0xde0f28f8,0xd84d28fa,0xde0f28fa,0xd84d28fc,0xde1128fc,0xd84d28fe,
	0xde1128fe,0xd84d2900,0xde132900,0xd84d2902,0xde132902,0xd84d2904,0xde152904,0xd84d2906,
	0xde152906,0xd84d2908,0xde172908,0xd84d290a,0xde17290a,0xd84d290c,0xde19290c,0xd84d290e,
	0xde19290e,0xd84d2910,0xde1b2910,0xd84d2912,0xde1b2912,0xd84d2914,0xde1d2914,0xd84d2916,
	0xde1d2916,0xd84d2918,0xde1f2918,0xd84d291a,0xde1f291a,0xd84d291c,0xde21291c,0xd84d291e,
	0xde21291e,0xd84d2920,0xde232920,0xd84d2922,0xde232922,0xd84d2924,0xde252924,0xd84d2926,
	0xde252926,0xd84d2928,0xde272928,0xd84d292a,0xde27292a,0xd84d292c,0xde29292c,0xd84d292e,
	0xde29292e,0xd84d2930,0xde2b2930,0xd84d2932,0xde2b2932,0xd84d2934,0xde2d2934,0xd84d2936,
	0xde2d2936,0xd84d2938,0xde2f2938,0xd84d293a,0xde2f293a,0xd84d293c,0xde31293c,0xd84d293e,
	0xde31293e,0xd84d2940,0xde332940,0xd84d2942,0xde332942,0xd84d2944,0xde352944,0xd84d2946,
	0xde352946,0xd84d2948,0xde372948,0xd84d294a,0xde37294a,0xd84d294c,0xde39294c,0xd84d294e,
	0xde39294e,0xd84d2950,0xde3b2950,0xd84d2952,0xde3b2952,0xd84d2954,0xde3d2954,0xd84d2956,
	0xde3d2956,0xd84d2958,0xde3f2958,0xd84d295a,0xde3f295a,0xd84d295c,0xde41295c,0xd84d295e,
	0xde41295e,0xd84d2960,0xde432960,0xd84d2962,0xde432962,0xd84d2964,0xde452964,0xd84d2966,
	0xde452966,0xd84d2968,0xde472968,0xd84d296a,0xde47296a,0xd84d296c,0xde49296c,0xd84d296e,
	0xde49296e,0xd84d2970,0xde4b2970,0xd84d2972,0xde4b2972,0xd84d2974,0xde4d2974,0xd84d2976,
	0xde4d2976,0xd84d2978,0xde4f2978,0xd84d297a,0xde4f297a,0xd84d297c,0xde51297c,0xd84d297e,
	0xde51297e,0xd84d2980,0xde532980,0xd84d2982,0xde532982,0xd84d2984,0xde552984,0xd84d2986,
	0xde552986,0xd84d2988,0xde572988,0xd84d298a,0xde57298a,0xd84d298c,0xde59298c,0xd84d298e,
	0xde59298e,0xd84d2990,0xde5b2990,0xd84d2992,0xde5b2992,0xd84d2994,0xde5d2994,0xd84d2996,
	0xde5d2996,0xd84d2998,0xde5f2998,0xd84d299a,0xde5f299a,0xd84d299c,0xde61299c,0xd84d299e,
	0xde61299e,0xd84d29a0,0xde6329a0,0xd84d29a2,0xde6329a2,0xd84d29a4,0xde6529a4,0xd84d29a6,
	0xde6529a6,0xd84d29a8,0xde6729a8,0xd84d29aa,0xde6729aa,0xd84d29ac,0xde6929ac,0xd84d29ae,
	0xde6929ae,0xd84d29b0,0xde6b29b0,0xd84d29b2,0xde6b29b2,0xd84d29b4,0xde6d29b4,0xd84d29b6,
	0xde6d29b6,0xd84d29b8,0xde6f29b8,0xd84d29ba,0xde6f29ba,0xd84d29bc,0xde7129bc,0xd84d29be,
	0xde7129be,0xd84d29c0,0xde7329c0,0xd84d29c2,0xde7329c2,0xd84d29c4,0xde7529c4,0xd84d29c6,
	0xde7529c6,0xd84d29c8,0xde7729c8,0xd84d29ca,0xde7729ca,0xd84d29cc,0xde7929cc,0xd84d29ce,
	0xde7929ce,0xd84d29d0,0xde7b29d0,0xd84d29d2,0xde7b29d2,0xd84d29d4,0xde7d29d4,0xd84d29d6,
	0xde7d29d6,0xd84d29d8,0xde7f29d8,0xd84d29da,0xde7f29da,0xd84d29dc,0xde8129dc,0xd84d29de,
	0xde8129de,0xd84d29e0,0xde8329e0,0xd84d29e2,0xde8329e2,0xd84d29e4,0xde8529e4,0xd84d29e6,
	0xde8529e6,0xd84d29e8,0xde8729e8,0xd84d29ea,0xde8729ea,0xd84d29ec,0xde8929ec,0xd84d29ee,
	0xde8929ee,0xd84d29f0,0xde8b29f0,0xd84d29f2,0xde8b29f2,0xd84d29f4,0xde8d29f4,0xd84d29f6,
	0xde8d29f6,0xd84d29f8,0xde8f29f8,0xd84d29fa,0xde8f29fa,0xd84d29fc,0xde9129fc,0xd84d29fe,
	0xde9129fe,0xd84d2a00,0xde932a00,0xd84d2a02,0xde932a02,0xd84d2a04,0xde952a04,0xd84d2a06,
	0xde952a06,0xd84d2a08,0xde972a08,0xd84d2a0a,0xde972a0a,0xd84d2a0c,0xde992a0c,0xd84d2a0e,
	0xde992a0e,0xd84d2a10,0xde9b2a10,0xd84d2a12,0xde9b2a12,0xd84d2a14,0xde9d2a14,0xd84d2a16,
	0xde9d2a16,0xd84d2a18,0xde9f2a18,0xd84d2a1a,0xde9f2a1a,0xd84d2a1c,0xdea12a1c,0xd84d2a1e,
	0xdea12a1e,0xd84d2a20,0xdea32a20,0xd84d2a22,0xdea32a22,0xd84d2a24,0xdea52a24,0xd84d2a26,
	0xdea52a26,0xd84d2a28,0xdea72a28,0xd84d2a2a,0xdea72a2a,0xd84d2a2c,0xdea92a2c,0xd84d2a2e,
	0xdea92a2e,0xd84d2a30,0xdeab2a30,0xd84d2a32,0xdeab2a32,0xd84d2a34,0xdead2a34,0xd84d2a36,
	0xdead2a36,0xd84d2a38,0xdeaf2a38,0xd84d2a3a,0xdeaf2a3a,0xd84d2a3c,0xdeb12a3c,0xd84d2a3e,
	0xdeb12a3e,0xd84d2a40,0xdeb32a40,0xd84d2a42,0xdeb32a42,0xd84d2a44,0xdeb52a44,0xd84d2a46,
	0xdeb52a46,0xd84d2a48,0xdeb72a48,0xd84d2a4a,0xdeb72a4a,0xd84d2a4c,0xdeb92a4c,0xd84d2a4e,
	0xdeb92a4e,0xd84d2a50,0xdebb2a50,0xd84d2a52,0xdebb2a52,0xd84d2a54,0xdebd2a54,0xd84d2a56,
	0xdebd2a56,0xd84d2a58,0xdebf2a58,0xd84d2a5a,0xdebf2a5a,0xd84d2a5c,0xdec12a5c,0xd84d2a5e,
	0xdec12a5e,0xd84d2a60,0xdec32a60,0xd84d2a62,0xdec32a62,0xd84d2a64,0xdec52a64,0xd84d2a66,
	0xdec52a66,0xd84d2a68,0xdec72a68,0xd84d2a6a,0xdec72a6a,0xd84d2a6c,0xdec92a6c,0xd84d2a6e,
	0xdec92a6e,0xd84d2a70,0xdecb2a70,0xd84d2a72,0xdecb2a72,0xd84d2a74,0xdecd2a74,0xd84d2a76,
	0xdecd2a76,0xd84d2a78,0xdecf2a78,0xd84d2a7a,0xdecf2a7a,0xd84d2a7c,0xded12a7c,0xd84d2a7e,
	0xded12a7e,0xd84d2a80,0xded32a80,0xd84d2a82,0xded32a82,0xd84d2a84,0xded52a84,0xd84d2a86,
	0xded52a86,0xd84d2a88,0xded72a88,0xd84d2a8a,0xded72a8a,0xd84d2a8c,0xded92a8c,0xd84d2a8e,
	0xded92a8e,0xd84d2a90,0xdedb2a90,0xd84d2a92,0xdedb2a92,0xd84d2a94,0xdedd2a94,0xd84d2a96,
	0xdedd2a96,0xd84d2a98,0xdedf2a98,0xd84d2a9a,0xdedf2a9a,0xd84d2a9c,0xdee12a9c,0xd84d2a9e,
	0xdee12a9e,0xd84d2aa0,0xdee32aa0,0xd84d2aa2,0xdee32aa2,0xd84d2aa4,0xdee52aa4,0xd84d2aa6,
	0xdee52aa6,0xd84d2aa8,0xdee72aa8,0xd84d2aaa,0xdee72aaa,0xd84d2aac,0xdee92aac,0xd84d2aae,
	0xdee92aae,0xd84d2ab0,0xdeeb2ab0,0xd84d2ab2,0xdeeb2ab2,0xd84d2ab4,0xdeed2ab4,0xd84d2ab6,
	0xdeed2ab6,0xd84d2ab8,0xdeef2ab8,0xd84d2aba,0xdeef2aba,0xd84d2abc,0xdef12abc,0xd84d2abe,
	0xdef12abe,0xd84d2ac0,0xdef32ac0,0xd84d2ac2,0xdef32ac2,0xd84d2ac4,0xdef52ac4,0xd84d2ac6,
	0xdef52ac6,0xd84d2ac8,0xdef72ac8,0xd84d2aca,0xdef72aca,0xd84d2acc,0xdef92acc,0xd84d2ace,
	0xdef92ace,0xd84d2ad0,0xdefb2ad0,0xd84d2ad2,0xdefb2ad2,0xd84d2ad4,0xdefd2ad4,0xd84d2ad6,
	0xdefd2ad6,0xd84d2ad8,0xdeff2ad8,0xd84d2ada,0xdeff2ada,0xd84d2adc,0xdf012adc,0xd84d2ade,
	0xdf012ade,0xd84d2ae0,0xdf032ae0,0xd84d2ae2,0xdf032ae2,0xd84d2ae4,0xdf052ae4,0xd84d2ae6,
	0xdf052ae6,0xd84d2ae8,0xdf072ae8,0xd84d2aea,0xdf072aea,0xd84d2aec,0xdf092aec,0xd84d2aee,
	0xdf092aee,0xd84d2af0,0xdf0b2af0,0xd84d2af2,0xdf0b2af2,0xd84d2af4,0xdf0d2af4,0xd84d2af6,
	0xdf0d2af6,0xd84d2af8,0xdf0f2af8,0xd84d2afa,0xdf0f2afa,0xd84d2afc,0xdf112afc,0xd84d2afe,
	0xdf112afe,0xd84d2b00,0xdf132b00,0xd84d2b02,0xdf132b02,0xd84d2b04,0xdf152b04,0xd84d2b06,
	0xdf152b06,0xd84d2b08,0xdf172b08,0xd84d2b0a,0xdf172b0a,0xd84d2b0c,0xdf192b0c,0xd84d2b0e,
	0xdf192b0e,0xd84d2b10,0xdf1b2b10,0xd84d2b12,0xdf1b2b12,0xd84d2b14,0xdf1d2b14,0xd84d2b16,
	0xdf1d2b16,0xd84d2b18,0xdf1f2b18,0xd84d2b1a,0xdf1f2b1a,0xd84d2b1c,0xdf212b1c,0xd84d2b1e,
	0xdf212b1e,0xd84d2b20,0xdf232b20,0xd84d2b22,0xdf232b22,0xd84d2b24,0xdf252b24,0xd84d2b26,
	0xdf252b26,0xd84d2b28,0xdf272b28,0xd84d2b2a,0xdf272b2a,0xd84d2b2c,0xdf292b2c,0xd84d2b2e,
	0xdf292b2e,0xd84d2b30,0xdf2b2b30,0xd84d2b32,0xdf2b2b32,0xd84d2b34,0xdf2d2b34,0xd84d2b36,
	0xdf2d2b36,0xd84d2b38,0xdf2f2b38,0xd84d2b3a,0xdf2f2b3a,0xd84d2b3c,0xdf312b3c,0xd84d2b3e,
	0xdf312b3e,0xd84d2b40,0xdf332b40,0xd84d2b42,0xdf332b42,0xd84d2b44,0xdf352b44,0xd84d2b46,
	0xdf352b46,0xd84d2b48,0xdf372b48,0xd84d2b4a,0xdf372b4a,0xd84d2b4c,0xdf392b4c,0xd84d2b4e,
	0xdf392b4e,0xd84d2b50,0xdf3b2b50,0xd84d2b52,0xdf3b2b52,0xd84d2b54,0xdf3d2b54,0xd84d2b56,
	0xdf3d2b56,0xd84d2b58,0xdf3f2b58,0xd84d2b5a,0xdf3f2b5a,0xd84d2b5c,0xdf412b5c,0xd84d2b5e,
	0xdf412b5e,0xd84d2b60,0xdf432b60,0xd84d2b62,0xdf432b62,0xd84d2b64,0xdf452b64,0xd84d2b66,
	0xdf452b66,0xd84d2b68,0xdf472b68,0xd84d2b6a,0xdf472b6a,0xd84d2b6c,0xdf492b6c,0xd84d2b6e,
	0xdf492b6e,0xd84d2b70,0xdf4b2b70,0xd84d2b72,0xdf4b2b72,0xd84d2b74,0xdf4d2b74,0xd84d2b76,
	0xdf4d2b76,0xd84d2b78,0xdf4f2b78,0xd84d2b7a,0xdf4f2b7a,0xd84d2b7c,0xdf512b7c,0xd84d2b7e,
	0xdf512b7e,0xd84d2b80,0xdf532b80,0xd84d2b82,0xdf532b82,0xd84d2b84,0xdf552b84,0xd84d2b86,
	0xdf552b86,0xd84d2b88,0xdf572b88,0xd84d2b8a,0xdf572b8a,0xd84d2b8c,0xdf592b8c,0xd84d2b8e,
	0xdf592b8e,0xd84d2b90,0xdf5b2b90,0xd84d2b92,0xdf5b2b92,0xd84d2b94,0xdf5d2b94,0xd84d2b96,
	0xdf5d2b96,0xd84d2b98,0xdf5f2b98,0xd84d2b9a,0xdf5f2b9a,0xd84d2b9c,0xdf612b9c,0xd84d2b9e,
	0xdf612b9e,0xd84d2ba0,0xdf632ba0,0xd84d2ba2,0xdf632ba2,0xd84d2ba4,0xdf652ba4,0xd84d2ba6,
	0xdf652ba6,0xd84d2ba8,0xdf672ba8,0xd84d2baa,0xdf672baa,0xd84d2bac,0xdf692bac,0xd84d2bae,
	0xdf692bae,0xd84d2bb0,0xdf6b2bb0,0xd84d2bb2,0xdf6b2bb2,0xd84d2bb4,0xdf6d2bb4,0xd84d2bb6,
	0xdf6d2bb6,0xd84d2bb8,0xdf6f2bb8,0xd84d2bba,0xdf6f2bba,0xd84d2bbc,0xdf712bbc,0xd84d2bbe,
	0xdf712bbe,0xd84d2bc0,0xdf732bc0,0xd84d2bc2,0xdf732bc2,0xd84d2bc4,0xdf752bc4,0xd84d2bc6,
	0xdf752bc6,0xd84d2bc8,0xdf772bc8,0xd84d2bca,0xdf772bca,0xd84d2bcc,0xdf792bcc,0xd84d2bce,
	0xdf792bce,0xd84d2bd0,0xdf7b2bd0,0xd84d2bd2,0xdf7b2bd2,0xd84d2bd4,0xdf7d2bd4,0xd84d2bd6,
	0xdf7d2bd6,0xd84d2bd8,0xdf7f2bd8,0xd84d2bda,0xdf7f2bda,0xd84d2bdc,0xdf812bdc,0xd84d2bde,
	0xdf812bde,0xd84d2be0,0xdf832be0,0xd84d2be2,0xdf832be2,0xd84d2be4,0xdf852be4,0xd84d2be6,
	0xdf852be6,0xd84d2be8,0xdf872be8,0xd84d2bea,0xdf872bea,0xd84d2bec,0xdf892bec,0xd84d2bee,
	0xdf892bee,0xd84d2bf0,0xdf8b2bf0,0xd84d2bf2,0xdf8b2bf2,0xd84d2bf4,0xdf8d2bf4,0xd84d2bf6,
	0xdf8d2bf6,0xd84d2bf8,0xdf8f2bf8,0xd84d2bfa,0xdf8f2bfa,0xd84d2bfc,0xdf912bfc,0xd84d2bfe,
	0xdf912bfe,0xd84d2c00,0xdf932c00,0xd84d2c02,0xdf932c02,0xd84d2c04,0xdf952c04,0xd84d2c06,
	0xdf952c06,0xd84d2c08,0xdf972c08,0xd84d2c0a,0xdf972c0a,0xd84d2c0c,0xdf992c0c,0xd84d2c0e,
	0xdf992c0e,0xd84d2c10,0xdf9b2c10,0xd84d2c12,0xdf9b2c12,0xd84d2c14,0xdf9d2c14,0xd84d2c16,
	0xdf9d2c16,0xd84d2c18,0xdf9f2c18,0xd84d2c1a,0xdf9f2c1a,0xd84d2c1c,0xdfa12c1c,0xd84d2c1e,
	0xdfa12c1e,0xd84d2c20,0xdfa32c20,0xd84d2c22,0xdfa32c22,0xd84d2c24,0xdfa52c24,0xd84d2c26,
	0xdfa52c26,0xd84d2c28,0xdfa72c28,0xd84d2c2a,0xdfa72c2a,0xd84d2c2c,0xdfa92c2c,0xd84d2c2e,
	0xdfa92c2e,0xd84d2c30,0xdfab2c30,0xd84d2c32,0xdfab2c32,0xd84d2c34,0xdfad2c34,0xd84d2c36,
	0xdfad2c36,0xd84d2c38,0xdfaf2c38,0xd84d2c3a,0xdfaf2c3a,0xd84d2c3c,0xdfb12c3c,0xd84d2c3e,
	0xdfb12c3e,0xd84d2c40,0xdfb32c40,0xd84d2c42,0xdfb32c42,0xd84d2c44,0xdfb52c44,0xd84d2c46,
	0xdfb52c46,0xd84d2c48,0xdfb72c48,0xd84d2c4a,0xdfb72c4a,0xd84d2c4c,0xdfb92c4c,0xd84d2c4e,
	0xdfb92c4e,0xd84d2c50,0xdfbb2c50,0xd84d2c52,0xdfbb2c52,0xd84d2c54,0xdfbd2c54,0xd84d2c56,
	0xdfbd2c56,0xd84d2c58,0xdfbf2c58,0xd84d2c5a,0xdfbf2c5a,0xd84d2c5c,0xdfc12c5c,0xd84d2c5e,
	0xdfc12c5e,0xd84d2c60,0xdfc32c60,0xd84d2c62,0xdfc32c62,0xd84d2c64,0xdfc52c64,0xd84d2c66,
	0xdfc52c66,0xd84d2c68,0xdfc72c68,0xd84d2c6a,0xdfc72c6a,0xd84d2c6c,0xdfc92c6c,0xd84d2c6e,
	0xdfc92c6e,0xd84d2c70,0xdfcb2c70,0xd84d2c72,0xdfcb2c72,0xd84d2c74,0xdfcd2c74,0xd84d2c76,
	0xdfcd2c76,0xd84d2c78,0xdfcf2c78,0xd84d2c7a,0xdfcf2c7a,0xd84d2c7c,0xdfd12c7c,0xd84d2c7e,
	0xdfd12c7e,0xd84d2c80,0xdfd32c80,0xd84d2c82,0xdfd32c82,0xd84d2c84,0xdfd52c84,0xd84d2c86,
	0xdfd52c86,0xd84d2c88,0xdfd72c88,0xd84d2c8a,0xdfd72c8a,0xd84d2c8c,0xdfd92c8c,0xd84d2c8e,
	0xdfd92c8e,0xd84d2c90,0xdfdb2c90,0xd84d2c92,0xdfdb2c92,0xd84d2c94,0xdfdd2c94,0xd84d2c96,
	0xdfdd2c96,0xd84d2c98,0xdfdf2c98,0xd84d2c9a,0xdfdf2c9a,0xd84d2c9c,0xdfe12c9c,0xd84d2c9e,
	0xdfe12c9e,0xd84d2ca0,0xdfe32ca0,0xd84d2ca2,0xdfe32ca2,0xd84d2ca4,0xdfe52ca4,0xd84d2ca6,
	0xdfe52ca6,0xd84d2ca8,0xdfe72ca8,0xd84d2caa,0xdfe72caa,0xd84d2cac,0xdfe92cac,0xd84d2cae,
	0xdfe92cae,0xd84d2cb0,0xdfeb2cb0,0xd84d2cb2,0xdfeb2cb2,0xd84d2cb4,0xdfed2cb4,0xd84d2cb6,
	0xdfed2cb6,0xd84d2cb8,0xdfef2cb8,0xd84d2cba,0xdfef2cba,0xd84d2cbc,0xdff12cbc,0xd84d2cbe,
	0xdff12cbe,0xd84d2cc0,0xdff32cc0,0xd84d2cc2,0xdff32cc2,0xd84d2cc4,0xdff52cc4,0xd84d2cc6,
	0xdff52cc6,0xd84d2cc8,0xdff72cc8,0xd84d2cca,0xdff72cca,0xd84d2ccc,0xdff92ccc,0xd84d2cce,
	0xdff92cce,0xd84d2cd0,0xdffb2cd0,0xd84d2cd2,0xdffb2cd2,0xd84d2cd4,0xdffd2cd4,0xd84d2cd6,
	0xdffd2cd6,0xd84d2cd8,0xdfff2cd8,0xd84d2cda,0xdfff2cda,0xd84f2cdc,0xdc012cdc,0xd84f2cde,
	0xdc012cde,0xd84f2ce0,0xdc032ce0,0xd84f2ce2,0xdc032ce2,0xd84f2ce4,0xdc052ce4,0xd84f2ce6,
	0xdc052ce6,0xd84f2ce8,0xdc072ce8,0xd84f2cea,0xdc072cea,0xd84f2cec,0xdc092cec,0xd84f2cee,
	0xdc092cee,0xd84f2cf0,0xdc0b2cf0,0xd84f2cf2,0xdc0b2cf2,0xd84f2cf4,0xdc0d2cf4,0xd84f2cf6,
	0xdc0d2cf6,0xd84f2cf8,0xdc0f2cf8,0xd84f2cfa,0xdc0f2cfa,0xd84f2cfc,0xdc112cfc,0xd84f2cfe,
	0xdc112cfe,0xd84f2d00,0xdc132d00,0xd84f2d02,0xdc132d02,0xd84f2d04,0xdc152d04,0xd84f2d06,
	0xdc152d06,0xd84f2d08,0xdc172d08,0xd84f2d0a,0xdc172d0a,0xd84f2d0c,0xdc192d0c,0xd84f2d0e,
	0xdc192d0e,0xd84f2d10,0xdc1b2d10,0xd84f2d12,0xdc1b2d12,0xd84f2d14,0xdc1d2d14,0xd84f2d16,
	0xdc1d2d16,0xd84f2d18,0xdc1f2d18,0xd84f2d1a,0xdc1f2d1a,0xd84f2d1c,0xdc212d1c,0xd84f2d1e,
	0xdc212d1e,0xd84f2d20,0xdc232d20,0xd84f2d22,0xdc232d22,0xd84f2d24,0xdc252d24,0xd84f2d26,
	0xdc252d26,0xd84f2d28,0xdc272d28,0xd84f2d2a,0xdc272d2a,0xd84f2d2c,0xdc292d2c,0xd84f2d2e,
	0xdc292d2e,0xd84f2d30,0xdc2b2d30,0xd84f2d32,0xdc2b2d32,0xd84f2d34,0xdc2d2d34,0xd84f2d36,
	0xdc2d2d36,0xd84f2d38,0xdc2f2d38,0xd84f2d3a,0xdc2f2d3a,0xd84f2d3c,0xdc312d3c,0xd84f2d3e,
	0xdc312d3e,0xd84f2d40,0xdc332d40,0xd84f2d42,0xdc332d42,0xd84f2d44,0xdc352d44,0xd84f2d46,
	0xdc352d46,0xd84f2d48,0xdc372d48,0xd84f2d4a,0xdc372d4a,0xd84f2d4c,0xdc392d4c,0xd84f2d4e,
	0xdc392d4e,0xd84f2d50,0xdc3b2d50,0xd84f2d52,0xdc3b2d52,0xd84f2d54,0xdc3d2d54,0xd84f2d56,
	0xdc3d2d56,0xd84f2d58,0xdc3f2d58,0xd84f2d5a,0xdc3f2d5a,0xd84f2d5c,0xdc412d5c,0xd84f2d5e,
	0xdc412d5e,0xd84f2d60,0xdc432d60,0xd84f2d62,0xdc432d62,0xd84f2d64,0xdc452d64,0xd84f2d66,
	0xdc452d66,0xd84f2d68,0xdc472d68,0xd84f2d6a,0xdc472d6a,0xd84f2d6c,0xdc492d6c,0xd84f2d6e,
	0xdc492d6e,0xd84f2d70,0xdc4b2d70,0xd84f2d72,0xdc4b2d72,0xd84f2d74,0xdc4d2d74,0xd84f2d76,
	0xdc4d2d76,0xd84f2d78,0xdc4f2d78,0xd84f2d7a,0xdc4f2d7a,0xd84f2d7c,0xdc512d7c,0xd84f2d7e,
	0xdc512d7e,0xd84f2d80,0xdc532d80,0xd84f2d82,0xdc532d82,0xd84f2d84,0xdc552d84,0xd84f2d86,
	0xdc552d86,0xd84f2d88,0xdc572d88,0xd84f2d8a,0xdc572d8a,0xd84f2d8c,0xdc592d8c,0xd84f2d8e,
	0xdc592d8e,0xd84f2d90,0xdc5b2d90,0xd84f2d92,0xdc5b2d92,0xd84f2d94,0xdc5d2d94,0xd84f2d96,
	0xdc5d2d96,0xd84f2d98,0xdc5f2d98,0xd84f2d9a,0xdc5f2d9a,0xd84f2d9c,0xdc612d9c,0xd84f2d9e,
	0xdc612d9e,0xd84f2da0,0xdc632da0,0xd84f2da2,0xdc632da2,0xd84f2da4,0xdc652da4,0xd84f2da6,
	0xdc652da6,0xd84f2da8,0xdc672da8,0xd84f2daa,0xdc672daa,0xd84f2dac,0xdc692dac,0xd84f2dae,
	0xdc692dae,0xd84f2db0,0xdc6b2db0,0xd84f2db2,0xdc6b2db2,0xd84f2db4,0xdc6d2db4,0xd84f2db6,
	0xdc6d2db6,0xd84f2db8,0xdc6f2db8,0xd84f2dba,0xdc6f2dba,0xd84f2dbc,0xdc712dbc,0xd84f2dbe,
	0xdc712dbe,0xd84f2dc0,0xdc732dc0,0xd84f2dc2,0xdc732dc2,0xd84f2dc4,0xdc752dc4,0xd84f2dc6,
	0xdc752dc6,0xd84f2dc8,0xdc772dc8,0xd84f2dca,0xdc772dca,0xd84f2dcc,0xdc792dcc,0xd84f2dce,
	0xdc792dce,0xd84f2dd0,0xdc7b2dd0,0xd84f2dd2,0xdc7b2dd2,0xd84f2dd4,0xdc7d2dd4,0xd84f2dd6,
	0xdc7d2dd6,0xd84f2dd8,0xdc7f2dd8,0xd84f2dda,0xdc7f2dda,0xd84f2ddc,0xdc812ddc,0xd84f2dde,
	0xdc812dde,0xd84f2de0,0xdc832de0,0xd84f2de2,0xdc832de2,0xd84f2de4,0xdc852de4,0xd84f2de6,
	0xdc852de6,0xd84f2de8,0xdc872de8,0xd84f2dea,0xdc872dea,0xd84f2dec,0xdc892dec,0xd84f2dee,
	0xdc892dee,0xd84f2df0,0xdc8b2df0,0xd84f2df2,0xdc8b2df2,0xd84f2df4,0xdc8d2df4,0xd84f2df6,
	0xdc8d2df6,0xd84f2df8,0xdc8f2df8,0xd84f2dfa,0xdc8f2dfa,0xd84f2dfc,0xdc912dfc,0xd84f2dfe,
	0xdc912dfe,0xd84f2e00,0xdc932e00,0xd84f2e02,0xdc932e02,0xd84f2e04,0xdc952e04,0xd84f2e06,
	0xdc952e06,0xd84f2e08,0xdc972e08,0xd84f2e0a,0xdc972e0a,0xd84f2e0c,0xdc992e0c,0xd84f2e0e,
	0xdc992e0e,0xd84f2e10,0xdc9b2e10,0xd84f2e12,0xdc9b2e12,0xd84f2e14,0xdc9d2e14,0xd84f2e16,
	0xdc9d2e16,0xd84f2e18,0xdc9f2e18,0xd84f2e1a,0xdc9f2e1a,0xd84f2e1c,0xdca12e1c,0xd84f2e1e,
	0xdca12e1e,0xd84f2e20,0xdca32e20,0xd84f2e22,0xdca32e22,0xd84f2e24,0xdca52e24,0xd84f2e26,
	0xdca52e26,0xd84f2e28,0xdca72e28,0xd84f2e2a,0xdca72e2a,0xd84f2e2c,0xdca92e2c,0xd84f2e2e,
	0xdca92e2e,0xd84f2e30,0xdcab2e30,0xd84f2e32,0xdcab2e32,0xd84f2e34,0xdcad2e34,0xd84f2e36,
	0xdcad2e36,0xd84f2e38,0xdcaf2e38,0xd84f2e3a,0xdcaf2e3a,0xd84f2e3c,0xdcb12e3c,0xd84f2e3e,
	0xdcb12e3e,0xd84f2e40,0xdcb32e40,0xd84f2e42,0xdcb32e42,0xd84f2e44,0xdcb52e44,0xd84f2e46,
	0xdcb52e46,0xd84f2e48,0xdcb72e48,0xd84f2e4a,0xdcb72e4a,0xd84f2e4c,0xdcb92e4c,0xd84f2e4e,
	0xdcb92e4e,0xd84f2e50,0xdcbb2e50,0xd84f2e52,0xdcbb2e52,0xd84f2e54,0xdcbd2e54,0xd84f2e56,
	0xdcbd2e56,0xd84f2e58,0xdcbf2e58,0xd84f2e5a,0xdcbf2e5a,0xd84f2e5c,0xdcc12e5c,0xd84f2e5e,
	0xdcc12e5e,0xd84f2e60,0xdcc32e60,0xd84f2e62,0xdcc32e62,0xd84f2e64,0xdcc52e64,0xd84f2e66,
	0xdcc52e66,0xd84f2e68,0xdcc72e68,0xd84f2e6a,0xdcc72e6a,0xd84f2e6c,0xdcc92e6c,0xd84f2e6e,
	0xdcc92e6e,0xd84f2e70,0xdccb2e70,0xd84f2e72,0xdccb2e72,0xd84f2e74,0xdccd2e74,0xd84f2e76,
	0xdccd2e76,0xd84f2e78,0xdccf2e78,0xd84f2e7a,0xdccf2e7a,0xd84f2e7c,0xdcd12e7c,0xd84f2e7e,
	0xdcd12e7e,0xd84f2e80,0xdcd32e80,0xd84f2e82,0xdcd32e82,0xd84f2e84,0xdcd52e84,0xd84f2e86,
	0xdcd52e86,0xd84f2e88,0xdcd72e88,0xd84f2e8a,0xdcd72e8a,0xd84f2e8c,0xdcd92e8c,0xd84f2e8e,
	0xdcd92e8e,0xd84f2e90,0xdcdb2e90,0xd84f2e92,0xdcdb2e92,0xd84f2e94,0xdcdd2e94,0xd84f2e96,
	0xdcdd2e96,0xd84f2e98,0xdcdf2e98,0xd84f2e9a,0xdcdf2e9a,0xd84f2e9c,0xdce12e9c,0xd84f2e9e,
	0xdce12e9e,0xd84f2ea0,0xdce32ea0,0xd84f2ea2,0xdce32ea2,0xd84f2ea4,0xdce52ea4,0xd84f2ea6,
	0xdce52ea6,0xd84f2ea8,0xdce72ea8,0xd84f2eaa,0xdce72eaa,0xd84f2eac,0xdce92eac,0xd84f2eae,
	0xdce92eae,0xd84f2eb0,0xdceb2eb0,0xd84f2eb2,0xdceb2eb2,0xd84f2eb4,0xdced2eb4,0xd84f2eb6,
	0xdced2eb6,0xd84f2eb8,0xdcef2eb8,0xd84f2eba,0xdcef2eba,0xd84f2ebc,0xdcf12ebc,0xd84f2ebe,
	0xdcf12ebe,0xd84f2ec0,0xdcf32ec0,0xd84f2ec2,0xdcf32ec2,0xd84f2ec4,0xdcf52ec4,0xd84f2ec6,
	0xdcf52ec6,0xd84f2ec8,0xdcf72ec8,0xd84f2eca,0xdcf72eca,0xd84f2ecc,0xdcf92ecc,0xd84f2ece,
	0xdcf92ece,0xd84f2ed0,0xdcfb2ed0,0xd84f2ed2,0xdcfb2ed2,0xd84f2ed4,0xdcfd2ed4,0xd84f2ed6,
	0xdcfd2ed6,0xd84f2ed8,0xdcff2ed8,0xd84f2eda,0xdcff2eda,0xd84f2edc,0xdd012edc,0xd84f2ede,
	0xdd012ede,0xd84f2ee0,0xdd032ee0,0xd84f2ee2,0xdd032ee2,0xd84f2ee4,0xdd052ee4,0xd84f2ee6,
	0xdd052ee6,0xd84f2ee8,0xdd072ee8,0xd84f2eea,0xdd072eea,0xd84f2eec,0xdd092eec,0xd84f2eee,
	0xdd092eee,0xd84f2ef0,0xdd0b2ef0,0xd84f2ef2,0xdd0b2ef2,0xd84f2ef4,0xdd0d2ef4,0xd84f2ef6,
	0xdd0d2ef6,0xd84f2ef8,0xdd0f2ef8,0xd84f2efa,0xdd0f2efa,0xd84f2efc,0xdd112efc,0xd84f2efe,
	0xdd112efe,0xd84f2f00,0xdd132f00,0xd84f2f02,0xdd132f02,0xd84f2f04,0xdd152f04,0xd84f2f06,
	0xdd152f06,0xd84f2f08,0xdd172f08,0xd84f2f0a,0xdd172f0a,0xd84f2f0c,0xdd192f0c,0xd84f2f0e,
	0xdd192f0e,0xd84f2f10,0xdd1b2f10,0xd84f2f12,0xdd1b2f12,0xd84f2f14,0xdd1d2f14,0xd84f2f16,
	0xdd1d2f16,0xd84f2f18,0xdd1f2f18,0xd84f2f1a,0xdd1f2f1a,0xd84f2f1c,0xdd212f1c,0xd84f2f1e,
	0xdd212f1e,0xd84f2f20,0xdd232f20,0xd84f2f22,0xdd232f22,0xd84f2f24,0xdd252f24,0xd84f2f26,
	0xdd252f26,0xd84f2f28,0xdd272f28,0xd84f2f2a,0xdd272f2a,0xd84f2f2c,0xdd292f2c,0xd84f2f2e,
	0xdd292f2e,0xd84f2f30,0xdd2b2f30,0xd84f2f32,0xdd2b2f32,0xd84f2f34,0xdd2d2f34,0xd84f2f36,
	0xdd2d2f36,0xd84f2f38,0xdd2f2f38,0xd84f2f3a,0xdd2f2f3a,0xd84f2f3c,0xdd312f3c,0xd84f2f3e,
	0xdd312f3e,0xd84f2f40,0xdd332f40,0xd84f2f42,0xdd332f42,0xd84f2f44,0xdd352f44,0xd84f2f46,
	0xdd352f46,0xd84f2f48,0xdd372f48,0xd84f2f4a,0xdd372f4a,0xd84f2f4c,0xdd392f4c,0xd84f2f4e,
	0xdd392f4e,0xd84f2f50,0xdd3b2f50,0xd84f2f52,0xdd3b2f52,0xd84f2f54,0xdd3d2f54,0xd84f2f56,
	0xdd3d2f56,0xd84f2f58,0xdd3f2f58,0xd84f2f5a,0xdd3f2f5a,0xd84f2f5c,0xdd412f5c,0xd84f2f5e,
	0xdd412f5e,0xd84f2f60,0xdd432f60,0xd84f2f62,0xdd432f62,0xd84f2f64,0xdd452f64,0xd84f2f66,
	0xdd452f66,0xd84f2f68,0xdd472f68,0xd84f2f6a,0xdd472f6a,0xd84f2f6c,0xdd492f6c,0xd84f2f6e,
	0xdd492f6e,0xd84f2f70,0xdd4b2f70,0xd84f2f72,0xdd4b2f72,0xd84f2f74,0xdd4d2f74,0xd84f2f76,
	0xdd4d2f76,0xd84f2f78,0xdd4f2f78,0xd84f2f7a,0xdd4f2f7a,0xd84f2f7c,0xdd512f7c,0xd84f2f7e,
	0xdd512f7e,0xd84f2f80,0xdd532f80,0xd84f2f82,0xdd532f82,0xd84f2f84,0xdd552f84,0xd84f2f86,
	0xdd552f86,0xd84f2f88,0xdd572f88,0xd84f2f8a,0xdd572f8a,0xd84f2f8c,0xdd592f8c,0xd84f2f8e,
	0xdd592f8e,0xd84f2f90,0xdd5b2f90,0xd84f2f92,0xdd5b2f92,0xd84f2f94,0xdd5d2f94,0xd84f2f96,
	0xdd5d2f96,0xd84f2f98,0xdd5f2f98,0xd84f2f9a,0xdd5f2f9a,0xd84f2f9c,0xdd612f9c,0xd84f2f9e,
	0xdd612f9e,0xd84f2fa0,0xdd632fa0,0xd84f2fa2,0xdd632fa2,0xd84f2fa4,0xdd652fa4,0xd84f2fa6,
	0xdd652fa6,0xd84f2fa8,0xdd672fa8,0xd84f2faa,0xdd672faa,0xd84f2fac,0xdd692fac,0xd84f2fae,
	0xdd692fae,0xd84f2fb0,0xdd6b2fb0,0xd84f2fb2,0xdd6b2fb2,0xd84f2fb4,0xdd6d2fb4,0xd84f2fb6,
	0xdd6d2fb6,0xd84f2fb8,0xdd6f2fb8,0xd84f2fba,0xdd6f2fba,0xd84f2fbc,0xdd712fbc,0xd84f2fbe,
	0xdd712fbe,0xd84f2fc0,0xdd732fc0,0xd84f2fc2,0xdd732fc2,0xd84f2fc4,0xdd752fc4,0xd84f2fc6,
	0xdd752fc6,0xd84f2fc8,0xdd772fc8,0xd84f2fca,0xdd772fca,0xd84f2fcc,0xdd792fcc,0xd84f2fce,
	0xdd792fce,0xd84f2fd0,0xdd7b2fd0,0xd84f2fd2,0xdd7b2fd2,0xd84f2fd4,0xdd7d2fd4,0xd84f2fd6,
	0xdd7d2fd6,0xd84f2fd8,0xdd7f2fd8,0xd84f2fda,0xdd7f2fda,0xd84f2fdc,0xdd812fdc,0xd84f2fde,
	0xdd812fde,0xd84f2fe0,0xdd832fe0,0xd84f2fe2,0xdd832fe2,0xd84f2fe4,0xdd852fe4,0xd84f2fe6,
	0xdd852fe6,0xd84f2fe8,0xdd872fe8,0xd84f2fea,0xdd872fea,0xd84f2fec,0xdd892fec,0xd84f2fee,
	0xdd892fee,0xd84f2ff0,0xdd8b2ff0,0xd84f2ff2,0xdd8b2ff2,0xd84f2ff4,0xdd8d2ff4,0xd84f2ff6,
	0xdd8d2ff6,0xd84f2ff8,0xdd8f2ff8,0xd84f2ffa,0xdd8f2ffa,0xd84f2ffc,0xdd912ffc,0xd84f2ffe,
	0xdd912ffe,0xd84f3000,0xdd933000,0xd84f3002,0xdd933002,0xd84f3004,0xdd953004,0xd84f3006,
	0xdd953006,0xd84f3008,0xdd973008,0xd84f300a,0xdd97300a,0xd84f300c,0xdd99300c,0xd84f300e,
	0xdd99300e,0xd84f3010,0xdd9b3010,0xd84f3012,0xdd9b3012,0xd84f3014,0xdd9d3014,0xd84f3016,
	0xdd9d3016,0xd84f3018,0xdd9f3018,0xd84f301a,0xdd9f301a,0xd84f301c,0xdda1301c,0xd84f301e,
	0xdda1301e,0xd84f3020,0xdda33020,0xd84f3022,0xdda33022,0xd84f3024,0xdda53024,0xd84f3026,
	0xdda53026,0xd84f3028,0xdda73028,0xd84f302a,0xdda7302a,0xd84f302c,0xdda9302c,0xd84f302e,
	0xdda9302e,0xd84f3030,0xddab3030,0xd84f3032,0xddab3032,0xd84f3034,0xddad3034,0xd84f3036,
	0xddad3036,0xd84f3038,0xddaf3038,0xd84f303a,0xddaf303a,0xd84f303c,0xddb1303c,0xd84f303e,
	0xddb1303e,0xd84f3040,0xddb33040,0xd84f3042,0xddb33042,0xd84f3044,0xddb53044,0xd84f3046,
	0xddb53046,0xd84f3048,0xddb73048,0xd84f304a,0xddb7304a,0xd84f304c,0xddb9304c,0xd84f304e,
	0xddb9304e,0xd84f3050,0xddbb3050,0xd84f3052,0xddbb3052,0xd84f3054,0xddbd3054,0xd84f3056,
	0xddbd3056,0xd84f3058,0xddbf3058,0xd84f305a,0xddbf305a,0xd84f305c,0xddc1305c,0xd84f305e,
	0xddc1305e,0xd84f3060,0xddc33060,0xd84f3062,0xddc33062,0xd84f3064,0xddc53064,0xd84f3066,
	0xddc53066,0xd84f3068,0xddc73068,0xd84f306a,0xddc7306a,0xd84f306c,0xddc9306c,0xd84f306e,
	0xddc9306e,0xd84f3070,0xddcb3070,0xd84f3072,0xddcb3072,0xd84f3074,0xddcd3074,0xd84f3076,
	0xddcd3076,0xd84f3078,0xddcf3078,0xd84f307a,0xddcf307a,0xd84f307c,0xddd1307c,0xd84f307e,
	0xddd1307e,0xd84f3080,0xddd33080,0xd84f3082,0xddd33082,0xd84f3084,0xddd53084,0xd84f3086,
	0xddd53086,0xd84f3088,0xddd73088,0xd84f308a,0xddd7308a,0xd84f308c,0xddd9308c,0xd84f308e,
	0xddd9308e,0xd84f3090,0xdddb3090,0xd84f3092,0xdddb3092,0xd84f3094,0xdddd3094,0xd84f3096,
	0xdddd3096,0xd84f3098,0xdddf3098,0xd84f309a,0xdddf309a,0xd84f309c,0xdde1309c,0xd84f309e,
	0xdde1309e,0xd84f30a0,0xdde330a0,0xd84f30a2,0xdde330a2,0xd84f30a4,0xdde530a4,0xd84f30a6,
	0xdde530a6,0xd84f30a8,0xdde730a8,0xd84f30aa,0xdde730aa,0xd84f30ac,0xdde930ac,0xd84f30ae,
	0xdde930ae,0xd84f30b0,0xddeb30b0,0xd84f30b2,0xddeb30b2,0xd84f30b4,0xdded30b4,0xd84f30b6,
	0xdded30b6,0xd84f30b8,0xddef30b8,0xd84f30ba,0xddef30ba,0xd84f30bc,0xddf130bc,0xd84f30be,
	0xddf130be,0xd84f30c0,0xddf330c0,0xd84f30c2,0xddf330c2,0xd84f30c4,0xddf530c4,0xd84f30c6,
	0xddf530c6,0xd84f30c8,0xddf730c8,0xd84f30ca,0xddf730ca,0xd84f30cc,0xddf930cc,0xd84f30ce,
	0xddf930ce,0xd84f30d0,0xddfb30d0,0xd84f30d2,0xddfb30d2,0xd84f30d4,0xddfd30d4,0xd84f30d6,
	0xddfd30d6,0xd84f30d8,0xddff30d8,0xd84f30da,0xddff30da,0xd84f30dc,0xde0130dc,0xd84f30de,
	0xde0130de,0xd84f30e0,0xde0330e0,0xd84f30e2,0xde0330e2,0xd84f30e4,0xde0530e4,0xd84f30e6,
	0xde0530e6,0xd84f30e8,0xde0730e8,0xd84f30ea,0xde0730ea,0xd84f30ec,0xde0930ec,0xd84f30ee,
	0xde0930ee,0xd84f30f0,0xde0b30f0,0xd84f30f2,0xde0b30f2,0xd84f30f4,0xde0d30f4,0xd84f30f6,
	0xde0d30f6,0xd84f30f8,0xde0f30f8,0xd84f30fa,0xde0f30fa,0xd84f30fc,0xde1130fc,0xd84f30fe,
	0xde1130fe,0xd84f3100,0xde133100,0xd84f3102,0xde133102,0xd84f3104,0xde153104,0xd84f3106,
	0xde153106,0xd84f3108,0xde173108,0xd84f310a,0xde17310a,0xd84f310c,0xde19310c,0xd84f310e,
	0xde19310e,0xd84f3110,0xde1b3110,0xd84f3112,0xde1b3112,0xd84f3114,0xde1d3114,0xd84f3116,
	0xde1d3116,0xd84f3118,0xde1f3118,0xd84f311a,0xde1f311a,0xd84f311c,0xde21311c,0xd84f311e,
	0xde21311e,0xd84f3120,0xde233120,0xd84f3122,0xde233122,0xd84f3124,0xde253124,0xd84f3126,
	0xde253126,0xd84f3128,0xde273128,0xd84f312a,0xde27312a,0xd84f312c,0xde29312c,0xd84f312e,
	0xde29312e,0xd84f3130,0xde2b3130,0xd84f3132,0xde2b3132,0xd84f3134,0xde2d3134,0xd84f3136,
	0xde2d3136,0xd84f3138,0xde2f3138,0xd84f313a,0xde2f313a,0xd84f313c,0xde31313c,0xd84f313e,
	0xde31313e,0xd84f3140,0xde333140,0xd84f3142,0xde333142,0xd84f3144,0xde353144,0xd84f3146,
	0xde353146,0xd84f3148,0xde373148,0xd84f314a,0xde37314a,0xd84f314c,0xde39314c,0xd84f314e,
	0xde39314e,0xd84f3150,0xde3b3150,0xd84f3152,0xde3b3152,0xd84f3154,0xde3d3154,0xd84f3156,
	0xde3d3156,0xd84f3158,0xde3f3158,0xd84f315a,0xde3f315a,0xd84f315c,0xde41315c,0xd84f315e,
	0xde41315e,0xd84f3160,0xde433160,0xd84f3162,0xde433162,0xd84f3164,0xde453164,0xd84f3166,
	0xde453166,0xd84f3168,0xde473168,0xd84f316a,0xde47316a,0xd84f316c,0xde49316c,0xd84f316e,
	0xde49316e,0xd84f3170,0xde4b3170,0xd84f3172,0xde4b3172,0xd84f3174,0xde4d3174,0xd84f3176,
	0xde4d3176,0xd84f3178,0xde4f3178,0xd84f317a,0xde4f317a,0xd84f317c,0xde51317c,0xd84f317e,
	0xde51317e,0xd84f3180,0xde533180,0xd84f3182,0xde533182,0xd84f3184,0xde553184,0xd84f3186,
	0xde553186,0xd84f3188,0xde573188,0xd84f318a,0xde57318a,0xd84f318c,0xde59318c,0xd84f318e,
	0xde59318e,0xd84f3190,0xde5b3190,0xd84f3192,0xde5b3192,0xd84f3194,0xde5d3194,0xd84f3196,
	0xde5d3196,0xd84f3198,0xde5f3198,0xd84f319a,0xde5f319a,0xd84f319c,0xde61319c,0xd84f319e,
	0xde61319e,0xd84f31a0,0xde6331a0,0xd84f31a2,0xde6331a2,0xd84f31a4,0xde6531a4,0xd84f31a6,
	0xde6531a6,0xd84f31a8,0xde6731a8,0xd84f31aa,0xde6731aa,0xd84f31ac,0xde6931ac,0xd84f31ae,
	0xde6931ae,0xd84f31b0,0xde6b31b0,0xd84f31b2,0xde6b31b2,0xd84f31b4,0xde6d31b4,0xd84f31b6,
	0xde6d31b6,0xd84f31b8,0xde6f31b8,0xd84f31ba,0xde6f31ba,0xd84f31bc,0xde7131bc,0xd84f31be,
	0xde7131be,0xd84f31c0,0xde7331c0,0xd84f31c2,0xde7331c2,0xd84f31c4,0xde7531c4,0xd84f31c6,
	0xde7531c6,0xd84f31c8,0xde7731c8,0xd84f31ca,0xde7731ca,0xd84f31cc,0xde7931cc,0xd84f31ce,
	0xde7931ce,0xd84f31d0,0xde7b31d0,0xd84f31d2,0xde7b31d2,0xd84f31d4,0xde7d31d4,0xd84f31d6,
	0xde7d31d6,0xd84f31d8,0xde7f31d8,0xd84f31da,0xde7f31da,0xd84f31dc,0xde8131dc,0xd84f31de,
	0xde8131de,0xd84f31e0,0xde8331e0,0xd84f31e2,0xde8331e2,0xd84f31e4,0xde8531e4,0xd84f31e6,
	0xde8531e6,0xd84f31e8,0xde8731e8,0xd84f31ea,0xde8731ea,0xd84f31ec,0xde8931ec,0xd84f31ee,
	0xde8931ee,0xd84f31f0,0xde8b31f0,0xd84f31f2,0xde8b31f2,0xd84f31f4,0xde8d31f4,0xd84f31f6,
	0xde8d31f6,0xd84f31f8,0xde8f31f8,0xd84f31fa,0xde8f31fa,0xd84f31fc,0xde9131fc,0xd84f31fe,
	0xde9131fe,0xd84f3200,0xde933200,0xd84f3202,0xde933202,0xd84f3204,0xde953204,0xd84f3206,
	0xde953206,0xd84f3208,0xde973208,0xd84f320a,0xde97320a,0xd84f320c,0xde99320c,0xd84f320e,
	0xde99320e,0xd84f3210,0xde9b3210,0xd84f3212,0xde9b3212,0xd84f3214,0xde9d3214,0xd84f3216,
	0xde9d3216,0xd84f3218,0xde9f3218,0xd84f321a,0xde9f321a,0xd84f321c,0xdea1321c,0xd84f321e,
	0xdea1321e,0xd84f3220,0xdea33220,0xd84f3222,0xdea33222,0xd84f3224,0xdea53224,0xd84f3226,
	0xdea53226,0xd84f3228,0xdea73228,0xd84f322a,0xdea7322a,0xd84f322c,0xdea9322c,0xd84f322e,
	0xdea9322e,0xd84f3230,0xdeab3230,0xd84f3232,0xdeab3232,0xd84f3234,0xdead3234,0xd84f3236,
	0xdead3236,0xd84f3238,0xdeaf3238,0xd84f323a,0xdeaf323a,0xd84f323c,0xdeb1323c,0xd84f323e,
	0xdeb1323e,0xd84f3240,0xdeb33240,0xd84f3242,0xdeb33242,0xd84f3244,0xdeb53244,0xd84f3246,
	0xdeb53246,0xd84f3248,0xdeb73248,0xd84f324a,0xdeb7324a,0xd84f324c,0xdeb9324c,0xd84f324e,
	0xdeb9324e,0xd84f3250,0xdebb3250,0xd84f3252,0xdebb3252,0xd84f3254,0xdebd3254,0xd84f3256,
	0xdebd3256,0xd84f3258,0xdebf3258,0xd84f325a,0xdebf325a,0xd84f325c,0xdec1325c,0xd84f325e,
	0xdec1325e,0xd84f3260,0xdec33260,0xd84f3262,0xdec33262,0xd84f3264,0xdec53264,0xd84f3266,
	0xdec53266,0xd84f3268,0xdec73268,0xd84f326a,0xdec7326a,0xd84f326c,0xdec9326c,0xd84f326e,
	0xdec9326e,0xd84f3270,0xdecb3270,0xd84f3272,0xdecb3272,0xd84f3274,0xdecd3274,0xd84f3276,
	0xdecd3276,0xd84f3278,0xdecf3278,0xd84f327a,0xdecf327a,0xd84f327c,0xded1327c,0xd84f327e,
	0xded1327e,0xd84f3280,0xded33280,0xd84f3282,0xded33282,0xd84f3284,0xded53284,0xd84f3286,
	0xded53286,0xd84f3288,0xded73288,0xd84f328a,0xded7328a,0xd84f328c,0xded9328c,0xd84f328e,
	0xded9328e,0xd84f3290,0xdedb3290,0xd84f3292,0xdedb3292,0xd84f3294,0xdedd3294,0xd84f3296,
	0xdedd3296,0xd84f3298,0xdedf3298,0xd84f329a,0xdedf329a,0xd84f329c,0xdee1329c,0xd84f329e,
	0xdee1329e,0xd84f32a0,0xdee332a0,0xd84f32a2,0xdee332a2,0xd84f32a4,0xdee532a4,0xd84f32a6,
	0xdee532a6,0xd84f32a8,0xdee732a8,0xd84f32aa,0xdee732aa,0xd84f32ac,0xdee932ac,0xd84f32ae,
	0xdee932ae,0xd84f32b0,0xdeeb32b0,0xd84f32b2,0xdeeb32b2,0xd84f32b4,0xdeed32b4,0xd84f32b6,
	0xdeed32b6,0xd84f32b8,0xdeef32b8,0xd84f32ba,0xdeef32ba,0xd84f32bc,0xdef132bc,0xd84f32be,
	0xdef132be,0xd84f32c0,0xdef332c0,0xd84f32c2,0xdef332c2,0xd84f32c4,0xdef532c4,0xd84f32c6,
	0xdef532c6,0xd84f32c8,0xdef732c8,0xd84f32ca,0xdef732ca,0xd84f32cc,0xdef932cc,0xd84f32ce,
	0xdef932ce,0xd84f32d0,0xdefb32d0,0xd84f32d2,0xdefb32d2,0xd84f32d4,0xdefd32d4,0xd84f32d6,
	0xdefd32d6,0xd84f32d8,0xdeff32d8,0xd84f32da,0xdeff32da,0xd84f32dc,0xdf0132dc,0xd84f32de,
	0xdf0132de,0xd84f32e0,0xdf0332e0,0xd84f32e2,0xdf0332e2,0xd84f32e4,0xdf0532e4,0xd84f32e6,
	0xdf0532e6,0xd84f32e8,0xdf0732e8,0xd84f32ea,0xdf0732ea,0xd84f32ec,0xdf0932ec,0xd84f32ee,
	0xdf0932ee,0xd84f32f0,0xdf0b32f0,0xd84f32f2,0xdf0b32f2,0xd84f32f4,0xdf0d32f4,0xd84f32f6,
	0xdf0d32f6,0xd84f32f8,0xdf0f32f8,0xd84f32fa,0xdf0f32fa,0xd84f32fc,0xdf1132fc,0xd84f32fe,
	0xdf1132fe,0xd84f3300,0xdf133300,0xd84f3302,0xdf133302,0xd84f3304,0xdf153304,0xd84f3306,
	0xdf153306,0xd84f3308,0xdf173308,0xd84f330a,0xdf17330a,0xd84f330c,0xdf19330c,0xd84f330e,
	0xdf19330e,0xd84f3310,0xdf1b3310,0xd84f3312,0xdf1b3312,0xd84f3314,0xdf1d3314,0xd84f3316,
	0xdf1d3316,0xd84f3318,0xdf1f3318,0xd84f331a,0xdf1f331a,0xd84f331c,0xdf21331c,0xd84f331e,
	0xdf21331e,0xd84f3320,0xdf233320,0xd84f3322,0xdf233322,0xd84f3324,0xdf253324,0xd84f3326,
	0xdf253326,0xd84f3328,0xdf273328,0xd84f332a,0xdf27332a,0xd84f332c,0xdf29332c,0xd84f332e,
	0xdf29332e,0xd84f3330,0xdf2b3330,0xd84f3332,0xdf2b3332,0xd84f3334,0xdf2d3334,0xd84f3336,
	0xdf2d3336,0xd84f3338,0xdf2f3338,0xd84f333a,0xdf2f333a,0xd84f333c,0xdf31333c,0xd84f333e,
	0xdf31333e,0xd84f3340,0xdf333340,0xd84f3342,0xdf333342,0xd84f3344,0xdf353344,0xd84f3346,
	0xdf353346,0xd84f3348,0xdf373348,0xd84f334a,0xdf37334a,0xd84f334c,0xdf39334c,0xd84f334e,
	0xdf39334e,0xd84f3350,0xdf3b3350,0xd84f3352,0xdf3b3352,0xd84f3354,0xdf3d3354,0xd84f3356,
	0xdf3d3356,0xd84f3358,0xdf3f3358,0xd84f335a,0xdf3f335a,0xd84f335c,0xdf41335c,0xd84f335e,
	0xdf41335e,0xd84f3360,0xdf433360,0xd84f3362,0xdf433362,0xd84f3364,0xdf453364,0xd84f3366,
	0xdf453366,0xd84f3368,0xdf473368,0xd84f336a,0xdf47336a,0xd84f336c,0xdf49336c,0xd84f336e,
	0xdf49336e,0xd84f3370,0xdf4b3370,0xd84f3372,0xdf4b3372,0xd84f3374,0xdf4d3374,0xd84f3376,
	0xdf4d3376,0xd84f3378,0xdf4f3378,0xd84f337a,0xdf4f337a,0xd84f337c,0xdf51337c,0xd84f337e,
	0xdf51337e,0xd84f3380,0xdf533380,0xd84f3382,0xdf533382,0xd84f3384,0xdf553384,0xd84f3386,
	0xdf553386,0xd84f3388,0xdf573388,0xd84f338a,0xdf57338a,0xd84f338c,0xdf59338c,0xd84f338e,
	0xdf59338e,0xd84f3390,0xdf5b3390,0xd84f3392,0xdf5b3392,0xd84f3394,0xdf5d3394,0xd84f3396,
	0xdf5d3396,0xd84f3398,0xdf5f3398,0xd84f339a,0xdf5f339a,0xd84f339c,0xdf61339c,0xd84f339e,
	0xdf61339e,0xd84f33a0,0xdf6333a0,0xd84f33a2,0xdf6333a2,0xd84f33a4,0xdf6533a4,0xd84f33a6,
	0xdf6533a6,0xd84f33a8,0xdf6733a8,0xd84f33aa,0xdf6733aa,0xd84f33ac,0xdf6933ac,0xd84f33ae,
	0xdf6933ae,0xd84f33b0,0xdf6b33b0,0xd84f33b2,0xdf6b33b2,0xd84f33b4,0xdf6d33b4,0xd84f33b6,
	0xdf6d33b6,0xd84f33b8,0xdf6f33b8,0xd84f33ba,0xdf6f33ba,0xd84f33bc,0xdf7133bc,0xd84f33be,
	0xdf7133be,0xd84f33c0,0xdf7333c0,0xd84f33c2,0xdf7333c2,0xd84f33c4,0xdf7533c4,0xd84f33c6,
	0xdf7533c6,0xd84f33c8,0xdf7733c8,0xd84f33ca,0xdf7733ca,0xd84f33cc,0xdf7933cc,0xd84f33ce,
	0xdf7933ce,0xd84f33d0,0xdf7b33d0,0xd84f33d2,0xdf7b33d2,0xd84f33d4,0xdf7d33d4,0xd84f33d6,
	0xdf7d33d6,0xd84f33d8,0xdf7f33d8,0xd84f33da,0xdf7f33da,0xd84f33dc,0xdf8133dc,0xd84f33de,
	0xdf8133de,0xd84f33e0,0xdf8333e0,0xd84f33e2,0xdf8333e2,0xd84f33e4,0xdf8533e4,0xd84f33e6,
	0xdf8533e6,0xd84f33e8,0xdf8733e8,0xd84f33ea,0xdf8733ea,0xd84f33ec,0xdf8933ec,0xd84f33ee,
	0xdf8933ee,0xd84f33f0,0xdf8b33f0,0xd84f33f2,0xdf8b33f2,0xd84f33f4,0xdf8d33f4,0xd84f33f6,
	0xdf8d33f6,0xd84f33f8,0xdf8f33f8,0xd84f33fa,0xdf8f33fa,0xd84f33fc,0xdf9133fc,0xd84f33fe,
	0xdf9133fe,0xd84f3400,0xdf933400,0xd84f3402,0xdf933402,0xd84f3404,0xdf953404,0xd84f3406,
	0xdf953406,0xd84f3408,0xdf973408,0xd84f340a,0xdf97340a,0xd84f340c,0xdf99340c,0xd84f340e,
	0xdf99340e,0xd84f3410,0xdf9b3410,0xd84f3412,0xdf9b3412,0xd84f3414,0xdf9d3414,0xd84f3416,
	0xdf9d3416,0xd84f3418,0xdf9f3418,0xd84f341a,0xdf9f341a,0xd84f341c,0xdfa1341c,0xd84f341e,
	0xdfa1341e,0xd84f3420,0xdfa33420,0xd84f3422,0xdfa33422,0xd84f3424,0xdfa53424,0xd84f3426,
	0xdfa53426,0xd84f3428,0xdfa73428,0xd84f342a,0xdfa7342a,0xd84f342c,0xdfa9342c,0xd84f342e,
	0xdfa9342e,0xd84f3430,0xdfab3430,0xd84f3432,0xdfab3432,0xd84f3434,0xdfad3434,0xd84f3436,
	0xdfad3436,0xd84f3438,0xdfaf3438,0xd84f343a,0xdfaf343a,0xd84f343c,0xdfb1343c,0xd84f343e,
	0xdfb1343e,0xd84f3440,0xdfb33440,0xd84f3442,0xdfb33442,0xd84f3444,0xdfb53444,0xd84f3446,
	0xdfb53446,0xd84f3448,0xdfb73448,0xd84f344a,0xdfb7344a,0xd84f344c,0xdfb9344c,0xd84f344e,
	0xdfb9344e,0xd84f3450,0xdfbb3450,0xd84f3452,0xdfbb3452,0xd84f3454,0xdfbd3454,0xd84f3456,
	0xdfbd3456,0xd84f3458,0xdfbf3458,0xd84f345a,0xdfbf345a,0xd84f345c,0xdfc1345c,0xd84f345e,
	0xdfc1345e,0xd84f3460,0xdfc33460,0xd84f3462,0xdfc33462,0xd84f3464,0xdfc53464,0xd84f3466,
	0xdfc53466,0xd84f3468,0xdfc73468,0xd84f346a,0xdfc7346a,0xd84f346c,0xdfc9346c,0xd84f346e,
	0xdfc9346e,0xd84f3470,0xdfcb3470,0xd84f3472,0xdfcb3472,0xd84f3474,0xdfcd3474,0xd84f3476,
	0xdfcd3476,0xd84f3478,0xdfcf3478,0xd84f347a,0xdfcf347a,0xd84f347c,0xdfd1347c,0xd84f347e,
	0xdfd1347e,0xd84f3480,0xdfd33480,0xd84f3482,0xdfd33482,0xd84f3484,0xdfd53484,0xd84f3486,
	0xdfd53486,0xd84f3488,0xdfd73488,0xd84f348a,0xdfd7348a,0xd84f348c,0xdfd9348c,0xd84f348e,
	0xdfd9348e,0xd84f3490,0xdfdb3490,0xd84f3492,0xdfdb3492,0xd84f3494,0xdfdd3494,0xd84f3496,
	0xdfdd3496,0xd84f3498,0xdfdf3498,0xd84f349a,0xdfdf349a,0xd84f349c,0xdfe1349c,0xd84f349e,
	0xdfe1349e,0xd84f34a0,0xdfe334a0,0xd84f34a2,0xdfe334a2,0xd84f34a4,0xdfe534a4,0xd84f34a6,
	0xdfe534a6,0xd84f34a8,0xdfe734a8,0xd84f34aa,0xdfe734aa,0xd84f34ac,0xdfe934ac,0xd84f34ae,
	0xdfe934ae,0xd84f34b0,0xdfeb34b0,0xd84f34b2,0xdfeb34b2,0xd84f34b4,0xdfed34b4,0xd84f34b6,
	0xdfed34b6,0xd84f34b8,0xdfef34b8,0xd84f34ba,0xdfef34ba,0xd84f34bc,0xdff134bc,0xd84f34be,
	0xdff134be,0xd84f34c0,0xdff334c0,0xd84f34c2,0xdff334c2,0xd84f34c4,0xdff534c4,0xd84f34c6,
	0xdff534c6,0xd84f34c8,0xdff734c8,0xd84f34ca,0xdff734ca,0xd84f34cc,0xdff934cc,0xd84f34ce,
	0xdff934ce,0xd84f34d0,0xdffb34d0,0xd84f34d2,0xdffb34d2,0xd84f34d4,0xdffd34d4,0xd84f34d6,
	0xdffd34d6,0xd84f34d8,0xdfff34d8,0xd84f34da,0xdfff34da,0xd84f34dc,0xdc0134dc,0xd84f34de,
	0xdc0134de,0xd84f34e0,0xdc0334e0,0xd84f34e2,0xdc0334e2,0xd84f34e4,0xdc0534e4,0xd84f34e6,
	0xdc0534e6,0xd84f34e8,0xdc0734e8,0xd84f34ea,0xdc0734ea,0xd84f34ec,0xdc0934ec,0xd84f34ee,
	0xdc0934ee,0xd84f34f0,0xdc0b34f0,0xd84f34f2,0xdc0b34f2,0xd84f34f4,0xdc0d34f4,0xd84f34f6,
	0xdc0d34f6,0xd84f34f8,0xdc0f34f8,0xd84f34fa,0xdc0f34fa,0xd84f34fc,0xdc1134fc,0xd84f34fe,
	0xdc1134fe,0xd84f3500,0xdc133500,0xd84f3502,0xdc133502,0xd84f3504,0xdc153504,0xd84f3506,
	0xdc153506,0xd84f3508,0xdc173508,0xd84f350a,0xdc17350a,0xd84f350c,0xdc19350c,0xd84f350e,
	0xdc19350e,0xd84f3510,0xdc1b3510,0xd84f3512,0xdc1b3512,0xd84f3514,0xdc1d3514,0xd84f3516,
	0xdc1d3516,0xd84f3518,0xdc1f3518,0xd84f351a,0xdc1f351a,0xd84f351c,0xdc21351c,0xd84f351e,
	0xdc21351e,0xd84f3520,0xdc233520,0xd84f3522,0xdc233522,0xd84f3524,0xdc253524,0xd84f3526,
	0xdc253526,0xd84f3528,0xdc273528,0xd84f352a,0xdc27352a,0xd84f352c,0xdc29352c,0xd84f352e,
	0xdc29352e,0xd84f3530,0xdc2b3530,0xd84f3532,0xdc2b3532,0xd84f3534,0xdc2d3534,0xd84f3536,
	0xdc2d3536,0xd84f3538,0xdc2f3538,0xd84f353a,0xdc2f353a,0xd84f353c,0xdc31353c,0xd84f353e,
	0xdc31353e,0xd84f3540,0xdc333540,0xd84f3542,0xdc333542,0xd84f3544,0xdc353544,0xd84f3546,
	0xdc353546,0xd84f3548,0xdc373548,0xd84f354a,0xdc37354a,0xd84f354c,0xdc39354c,0xd84f354e,
	0xdc39354e,0xd84f3550,0xdc3b3550,0xd84f3552,0xdc3b3552,0xd84f3554,0xdc3d3554,0xd84f3556,
	0xdc3d3556,0xd84f3558,0xdc3f3558,0xd84f355a,0xdc3f355a,0xd84f355c,0xdc41355c,0xd84f355e,
	0xdc41355e,0xd84f3560,0xdc433560,0xd84f3562,0xdc433562,0xd84f3564,0xdc453564,0xd84f3566,
	0xdc453566,0xd84f3568,0xdc473568,0xd84f356a,0xdc47356a,0xd84f356c,0xdc49356c,0xd84f356e,
	0xdc49356e,0xd84f3570,0xdc4b3570,0xd84f3572,0xdc4b3572,0xd84f3574,0xdc4d3574,0xd84f3576,
	0xdc4d3576,0xd84f3578,0xdc4f3578,0xd84f357a,0xdc4f357a,0xd84f357c,0xdc51357c,0xd84f357e,
	0xdc51357e,0xd84f3580,0xdc533580,0xd84f3582,0xdc533582,0xd84f3584,0xdc553584,0xd84f3586,
	0xdc553586,0xd84f3588,0xdc573588,0xd84f358a,0xdc57358a,0xd84f358c,0xdc59358c,0xd84f358e,
	0xdc59358e,0xd84f3590,0xdc5b3590,0xd84f3592,0xdc5b3592,0xd84f3594,0xdc5d3594,0xd84f3596,
	0xdc5d3596,0xd84f3598,0xdc5f3598,0xd84f359a,0xdc5f359a,0xd84f359c,0xdc61359c,0xd84f359e,
	0xdc61359e,0xd84f35a0,0xdc6335a0,0xd84f35a2,0xdc6335a2,0xd84f35a4,0xdc6535a4,0xd84f35a6,
	0xdc6535a6,0xd84f35a8,0xdc6735a8,0xd84f35aa,0xdc6735aa,0xd84f35ac,0xdc6935ac,0xd84f35ae,
	0xdc6935ae,0xd84f35b0,0xdc6b35b0,0xd84f35b2,0xdc6b35b2,0xd84f35b4,0xdc6d35b4,0xd84f35b6,
	0xdc6d35b6,0xd84f35b8,0xdc6f35b8,0xd84f35ba,0xdc6f35ba,0xd84f35bc,0xdc7135bc,0xd84f35be,
	0xdc7135be,0xd84f35c0,0xdc7335c0,0xd84f35c2,0xdc7335c2,0xd84f35c4,0xdc7535c4,0xd84f35c6,
	0xdc7535c6,0xd84f35c8,0xdc7735c8,0xd84f35ca,0xdc7735ca,0xd84f35cc,0xdc7935cc,0xd84f35ce,
	0xdc7935ce,0xd84f35d0,0xdc7b35d0,0xd84f35d2,0xdc7b35d2,0xd84f35d4,0xdc7d35d4,0xd84f35d6,
	0xdc7d35d6,0xd84f35d8,0xdc7f35d8,0xd84f35da,0xdc7f35da,0xd84f35dc,0xdc8135dc,0xd84f35de,
	0xdc8135de,0xd84f35e0,0xdc8335e0,0xd84f35e2,0xdc8335e2,0xd84f35e4,0xdc8535e4,0xd84f35e6,
	0xdc8535e6,0xd84f35e8,0xdc8735e8,0xd84f35ea,0xdc8735ea,0xd84f35ec,0xdc8935ec,0xd84f35ee,
	0xdc8935ee,0xd84f35f0,0xdc8b35f0,0xd84f35f2,0xdc8b35f2,0xd84f35f4,0xdc8d35f4,0xd84f35f6,
	0xdc8d35f6,0xd84f35f8,0xdc8f35f8,0xd84f35fa,0xdc8f35fa,0xd84f35fc,0xdc9135fc,0xd84f35fe,
	0xdc9135fe,0xd84f3600,0xdc933600,0xd84f3602,0xdc933602,0xd84f3604,0xdc953604,0xd84f3606,
	0xdc953606,0xd84f3608,0xdc973608,0xd84f360a,0xdc97360a,0xd84f360c,0xdc99360c,0xd84f360e,
	0xdc99360e,0xd84f3610,0xdc9b3610,0xd84f3612,0xdc9b3612,0xd84f3614,0xdc9d3614,0xd84f3616,
	0xdc9d3616,0xd84f3618,0xdc9f3618,0xd84f361a,0xdc9f361a,0xd84f361c,0xdca1361c,0xd84f361e,
	0xdca1361e,0xd84f3620,0xdca33620,0xd84f3622,0xdca33622,0xd84f3624,0xdca53624,0xd84f3626,
	0xdca53626,0xd84f3628,0xdca73628,0xd84f362a,0xdca7362a,0xd84f362c,0xdca9362c,0xd84f362e,
	0xdca9362e,0xd84f3630,0xdcab3630,0xd84f3632,0xdcab3632,0xd84f3634,0xdcad3634,0xd84f3636,
	0xdcad3636,0xd84f3638,0xdcaf3638,0xd84f363a,0xdcaf363a,0xd84f363c,0xdcb1363c,0xd84f363e,
	0xdcb1363e,0xd84f3640,0xdcb33640,0xd84f3642,0xdcb33642,0xd84f3644,0xdcb53644,0xd84f3646,
	0xdcb53646,0xd84f3648,0xdcb73648,0xd84f364a,0xdcb7364a,0xd84f364c,0xdcb9364c,0xd84f364e,
	0xdcb9364e,0xd84f3650,0xdcbb3650,0xd84f3652,0xdcbb3652,0xd84f3654,0xdcbd3654,0xd84f3656,
	0xdcbd3656,0xd84f3658,0xdcbf3658,0xd84f365a,0xdcbf365a,0xd84f365c,0xdcc1365c,0xd84f365e,
	0xdcc1365e,0xd84f3660,0xdcc33660,0xd84f3662,0xdcc33662,0xd84f3664,0xdcc53664,0xd84f3666,
	0xdcc53666,0xd84f3668,0xdcc73668,0xd84f366a,0xdcc7366a,0xd84f366c,0xdcc9366c,0xd84f366e,
	0xdcc9366e,0xd84f3670,0xdccb3670,0xd84f3672,0xdccb3672,0xd84f3674,0xdccd3674,0xd84f3676,
	0xdccd3676,0xd84f3678,0xdccf3678,0xd84f367a,0xdccf367a,0xd84f367c,0xdcd1367c,0xd84f367e,
	0xdcd1367e,0xd84f3680,0xdcd33680,0xd84f3682,0xdcd33682,0xd84f3684,0xdcd53684,0xd84f3686,
	0xdcd53686,0xd84f3688,0xdcd73688,0xd84f368a,0xdcd7368a,0xd84f368c,0xdcd9368c,0xd84f368e,
	0xdcd9368e,0xd84f3690,0xdcdb3690,0xd84f3692,0xdcdb3692,0xd84f3694,0xdcdd3694,0xd84f3696,
	0xdcdd3696,0xd84f3698,0xdcdf3698,0xd84f369a,0xdcdf369a,0xd84f369c,0xdce1369c,0xd84f369e,
	0xdce1369e,0xd84f36a0,0xdce336a0,0xd84f36a2,0xdce336a2,0xd84f36a4,0xdce536a4,0xd84f36a6,
	0xdce536a6,0xd84f36a8,0xdce736a8,0xd84f36aa,0xdce736aa,0xd84f36ac,0xdce936ac,0xd84f36ae,
	0xdce936ae,0xd84f36b0,0xdceb36b0,0xd84f36b2,0xdceb36b2,0xd84f36b4,0xdced36b4,0xd84f36b6,
	0xdced36b6,0xd84f36b8,0xdcef36b8,0xd84f36ba,0xdcef36ba,0xd84f36bc,0xdcf136bc,0xd84f36be,
	0xdcf136be,0xd84f36c0,0xdcf336c0,0xd84f36c2,0xdcf336c2,0xd84f36c4,0xdcf536c4,0xd84f36c6,
	0xdcf536c6,0xd84f36c8,0xdcf736c8,0xd84f36ca,0xdcf736ca,0xd84f36cc,0xdcf936cc,0xd84f36ce,
	0xdcf936ce,0xd84f36d0,0xdcfb36d0,0xd84f36d2,0xdcfb36d2,0xd84f36d4,0xdcfd36d4,0xd84f36d6,
	0xdcfd36d6,0xd84f36d8,0xdcff36d8,0xd84f36da,0xdcff36da,0xd84f36dc,0xdd0136dc,0xd84f36de,
	0xdd0136de,0xd84f36e0,0xdd0336e0,0xd84f36e2,0xdd0336e2,0xd84f36e4,0xdd0536e4,0xd84f36e6,
	0xdd0536e6,0xd84f36e8,0xdd0736e8,0xd84f36ea,0xdd0736ea,0xd84f36ec,0xdd0936ec,0xd84f36ee,
	0xdd0936ee,0xd84f36f0,0xdd0b36f0,0xd84f36f2,0xdd0b36f2,0xd84f36f4,0xdd0d36f4,0xd84f36f6,
	0xdd0d36f6,0xd84f36f8,0xdd0f36f8,0xd84f36fa,0xdd0f36fa,0xd84f36fc,0xdd1136fc,0xd84f36fe,
	0xdd1136fe,0xd84f3700,0xdd133700,0xd84f3702,0xdd133702,0xd84f3704,0xdd153704,0xd84f3706,
	0xdd153706,0xd84f3708,0xdd173708,0xd84f370a,0xdd17370a,0xd84f370c,0xdd19370c,0xd84f370e,
	0xdd19370e,0xd84f3710,0xdd1b3710,0xd84f3712,0xdd1b3712,0xd84f3714,0xdd1d3714,0xd84f3716,
	0xdd1d3716,0xd84f3718,0xdd1f3718,0xd84f371a,0xdd1f371a,0xd84f371c,0xdd21371c,0xd84f371e,
	0xdd21371e,0xd84f3720,0xdd233720,0xd84f3722,0xdd233722,0xd84f3724,0xdd253724,0xd84f3726,
	0xdd253726,0xd84f3728,0xdd273728,0xd84f372a,0xdd27372a,0xd84f372c,0xdd29372c,0xd84f372e,
	0xdd29372e,0xd84f3730,0xdd2b3730,0xd84f3732,0xdd2b3732,0xd84f3734,0xdd2d3734,0xd84f3736,
	0xdd2d3736,0xd84f3738,0xdd2f3738,0xd84f373a,0xdd2f373a,0xd84f373c,0xdd31373c,0xd84f373e,
	0xdd31373e,0xd84f3740,0xdd333740,0xd84f3742,0xdd333742,0xd84f3744,0xdd353744,0xd84f3746,
	0xdd353746,0xd84f3748,0xdd373748,0xd84f374a,0xdd37374a,0xd84f374c,0xdd39374c,0xd84f374e,
	0xdd39374e,0xd84f3750,0xdd3b3750,0xd84f3752,0xdd3b3752,0xd84f3754,0xdd3d3754,0xd84f3756,
	0xdd3d3756,0xd84f3758,0xdd3f3758,0xd84f375a,0xdd3f375a,0xd84f375c,0xdd41375c,0xd84f375e,
	0xdd41375e,0xd84f3760,0xdd433760,0xd84f3762,0xdd433762,0xd84f3764,0xdd453764,0xd84f3766,
	0xdd453766,0xd84f3768,0xdd473768,0xd84f376a,0xdd47376a,0xd84f376c,0xdd49376c,0xd84f376e,
	0xdd49376e,0xd84f3770,0xdd4b3770,0xd84f3772,0xdd4b3772,0xd84f3774,0xdd4d3774,0xd84f3776,
	0xdd4d3776,0xd84f3778,0xdd4f3778,0xd84f377a,0xdd4f377a,0xd84f377c,0xdd51377c,0xd84f377e,
	0xdd51377e,0xd84f3780,0xdd533780,0xd84f3782,0xdd533782,0xd84f3784,0xdd553784,0xd84f3786,
	0xdd553786,0xd84f3788,0xdd573788,0xd84f378a,0xdd57378a,0xd84f378c,0xdd59378c,0xd84f378e,
	0xdd59378e,0xd84f3790,0xdd5b3790,0xd84f3792,0xdd5b3792,0xd84f3794,0xdd5d3794,0xd84f3796,
	0xdd5d3796,0xd84f3798,0xdd5f3798,0xd84f379a,0xdd5f379a,0xd84f379c,0xdd61379c,0xd84f379e,
	0xdd61379e,0xd84f37a0,0xdd6337a0,0xd84f37a2,0xdd6337a2,0xd84f37a4,0xdd6537a4,0xd84f37a6,
	0xdd6537a6,0xd84f37a8,0xdd6737a8,0xd84f37aa,0xdd6737aa,0xd84f37ac,0xdd6937ac,0xd84f37ae,
	0xdd6937ae,0xd84f37b0,0xdd6b37b0,0xd84f37b2,0xdd6b37b2,0xd84f37b4,0xdd6d37b4,0xd84f37b6,
	0xdd6d37b6,0xd84f37b8,0xdd6f37b8,0xd84f37ba,0xdd6f37ba,0xd84f37bc,0xdd7137bc,0xd84f37be,
	0xdd7137be,0xd84f37c0,0xdd7337c0,0xd84f37c2,0xdd7337c2,0xd84f37c4,0xdd7537c4,0xd84f37c6,
	0xdd7537c6,0xd84f37c8,0xdd7737c8,0xd84f37ca,0xdd7737ca,0xd84f37cc,0xdd7937cc,0xd84f37ce,
	0xdd7937ce,0xd84f37d0,0xdd7b37d0,0xd84f37d2,0xdd7b37d2,0xd84f37d4,0xdd7d37d4,0xd84f37d6,
	0xdd7d37d6,0xd84f37d8,0xdd7f37d8,0xd84f37da,0xdd7f37da,0xd84f37dc,0xdd8137dc,0xd84f37de,
	0xdd8137de,0xd84f37e0,0xdd8337e0,0xd84f37e2,0xdd8337e2,0xd84f37e4,0xdd8537e4,0xd84f37e6,
	0xdd8537e6,0xd84f37e8,0xdd8737e8,0xd84f37ea,0xdd8737ea,0xd84f37ec,0xdd8937ec,0xd84f37ee,
	0xdd8937ee,0xd84f37f0,0xdd8b37f0,0xd84f37f2,0xdd8b37f2,0xd84f37f4,0xdd8d37f4,0xd84f37f6,
	0xdd8d37f6,0xd84f37f8,0xdd8f37f8,0xd84f37fa,0xdd8f37fa,0xd84f37fc,0xdd9137fc,0xd84f37fe,
	0xdd9137fe,0xd84f3800,0xdd933800,0xd84f3802,0xdd933802,0xd84f3804,0xdd953804,0xd84f3806,
	0xdd953806,0xd84f3808,0xdd973808,0xd84f380a,0xdd97380a,0xd84f380c,0xdd99380c,0xd84f380e,
	0xdd99380e,0xd84f3810,0xdd9b3810,0xd84f3812,0xdd9b3812,0xd84f3814,0xdd9d3814,0xd84f3816,
	0xdd9d3816,0xd84f3818,0xdd9f3818,0xd84f381a,0xdd9f381a,0xd84f381c,0xdda1381c,0xd84f381e,
	0xdda1381e,0xd84f3820,0xdda33820,0xd84f3822,0xdda33822,0xd84f3824,0xdda53824,0xd84f3826,
	0xdda53826,0xd84f3828,0xdda73828,0xd84f382a,0xdda7382a,0xd84f382c,0xdda9382c,0xd84f382e,
	0xdda9382e,0xd84f3830,0xddab3830,0xd84f3832,0xddab3832,0xd84f3834,0xddad3834,0xd84f3836,
	0xddad3836,0xd84f3838,0xddaf3838,0xd84f383a,0xddaf383a,0xd84f383c,0xddb1383c,0xd84f383e,
	0xddb1383e,0xd84f3840,0xddb33840,0xd84f3842,0xddb33842,0xd84f3844,0xddb53844,0xd84f3846,
	0xddb53846,0xd84f3848,0xddb73848,0xd84f384a,0xddb7384a,0xd84f384c,0xddb9384c,0xd84f384e,
	0xddb9384e,0xd84f3850,0xddbb3850,0xd84f3852,0xddbb3852,0xd84f3854,0xddbd3854,0xd84f3856,
	0xddbd3856,0xd84f3858,0xddbf3858,0xd84f385a,0xddbf385a,0xd84f385c,0xddc1385c,0xd84f385e,
	0xddc1385e,0xd84f3860,0xddc33860,0xd84f3862,0xddc33862,0xd84f3864,0xddc53864,0xd84f3866,
	0xddc53866,0xd84f3868,0xddc73868,0xd84f386a,0xddc7386a,0xd84f386c,0xddc9386c,0xd84f386e,
	0xddc9386e,0xd84f3870,0xddcb3870,0xd84f3872,0xddcb3872,0xd84f3874,0xddcd3874,0xd84f3876,
	0xddcd3876,0xd84f3878,0xddcf3878,0xd84f387a,0xddcf387a,0xd84f387c,0xddd1387c,0xd84f387e,
	0xddd1387e,0xd84f3880,0xddd33880,0xd84f3882,0xddd33882,0xd84f3884,0xddd53884,0xd84f3886,
	0xddd53886,0xd84f3888,0xddd73888,0xd84f388a,0xddd7388a,0xd84f388c,0xddd9388c,0xd84f388e,
	0xddd9388e,0xd84f3890,0xdddb3890,0xd84f3892,0xdddb3892,0xd84f3894,0xdddd3894,0xd84f3896,
	0xdddd3896,0xd84f3898,0xdddf3898,0xd84f389a,0xdddf389a,0xd84f389c,0xdde1389c,0xd84f389e,
	0xdde1389e,0xd84f38a0,0xdde338a0,0xd84f38a2,0xdde338a2,0xd84f38a4,0xdde538a4,0xd84f38a6,
	0xdde538a6,0xd84f38a8,0xdde738a8,0xd84f38aa,0xdde738aa,0xd84f38ac,0xdde938ac,0xd84f38ae,
	0xdde938ae,0xd84f38b0,0xddeb38b0,0xd84f38b2,0xddeb38b2,0xd84f38b4,0xdded38b4,0xd84f38b6,
	0xdded38b6,0xd84f38b8,0xddef38b8,0xd84f38ba,0xddef38ba,0xd84f38bc,0xddf138bc,0xd84f38be,
	0xddf138be,0xd84f38c0,0xddf338c0,0xd84f38c2,0xddf338c2,0xd84f38c4,0xddf538c4,0xd84f38c6,
	0xddf538c6,0xd84f38c8,0xddf738c8,0xd84f38ca,0xddf738ca,0xd84f38cc,0xddf938cc,0xd84f38ce,
	0xddf938ce,0xd84f38d0,0xddfb38d0,0xd84f38d2,0xddfb38d2,0xd84f38d4,0xddfd38d4,0xd84f38d6,
	0xddfd38d6,0xd84f38d8,0xddff38d8,0xd84f38da,0xddff38da,0xd84f38dc,0xde0138dc,0xd84f38de,
	0xde0138de,0xd84f38e0,0xde0338e0,0xd84f38e2,0xde0338e2,0xd84f38e4,0xde0538e4,0xd84f38e6,
	0xde0538e6,0xd84f38e8,0xde0738e8,0xd84f38ea,0xde0738ea,0xd84f38ec,0xde0938ec,0xd84f38ee,
	0xde0938ee,0xd84f38f0,0xde0b38f0,0xd84f38f2,0xde0b38f2,0xd84f38f4,0xde0d38f4,0xd84f38f6,
	0xde0d38f6,0xd84f38f8,0xde0f38f8,0xd84f38fa,0xde0f38fa,0xd84f38fc,0xde1138fc,0xd84f38fe,
	0xde1138fe,0xd84f3900,0xde133900,0xd84f3902,0xde133902,0xd84f3904,0xde153904,0xd84f3906,
	0xde153906,0xd84f3908,0xde173908,0xd84f390a,0xde17390a,0xd84f390c,0xde19390c,0xd84f390e,
	0xde19390e,0xd84f3910,0xde1b3910,0xd84f3912,0xde1b3912,0xd84f3914,0xde1d3914,0xd84f3916,
	0xde1d3916,0xd84f3918,0xde1f3918,0xd84f391a,0xde1f391a,0xd84f391c,0xde21391c,0xd84f391e,
	0xde21391e,0xd84f3920,0xde233920,0xd84f3922,0xde233922,0xd84f3924,0xde253924,0xd84f3926,
	0xde253926,0xd84f3928,0xde273928,0xd84f392a,0xde27392a,0xd84f392c,0xde29392c,0xd84f392e,
	0xde29392e,0xd84f3930,0xde2b3930,0xd84f3932,0xde2b3932,0xd84f3934,0xde2d3934,0xd84f3936,
	0xde2d3936,0xd84f3938,0xde2f3938,0xd84f393a,0xde2f393a,0xd84f393c,0xde31393c,0xd84f393e,
	0xde31393e,0xd84f3940,0xde333940,0xd84f3942,0xde333942,0xd84f3944,0xde353944,0xd84f3946,
	0xde353946,0xd84f3948,0xde373948,0xd84f394a,0xde37394a,0xd84f394c,0xde39394c,0xd84f394e,
	0xde39394e,0xd84f3950,0xde3b3950,0xd84f3952,0xde3b3952,0xd84f3954,0xde3d3954,0xd84f3956,
	0xde3d3956,0xd84f3958,0xde3f3958,0xd84f395a,0xde3f395a,0xd84f395c,0xde41395c,0xd84f395e,
	0xde41395e,0xd84f3960,0xde433960,0xd84f3962,0xde433962,0xd84f3964,0xde453964,0xd84f3966,
	0xde453966,0xd84f3968,0xde473968,0xd84f396a,0xde47396a,0xd84f396c,0xde49396c,0xd84f396e,
	0xde49396e,0xd84f3970,0xde4b3970,0xd84f3972,0xde4b3972,0xd84f3974,0xde4d3974,0xd84f3976,
	0xde4d3976,0xd84f3978,0xde4f3978,0xd84f397a,0xde4f397a,0xd84f397c,0xde51397c,0xd84f397e,
	0xde51397e,0xd84f3980,0xde533980,0xd84f3982,0xde533982,0xd84f3984,0xde553984,0xd84f3986,
	0xde553986,0xd84f3988,0xde573988,0xd84f398a,0xde57398a,0xd84f398c,0xde59398c,0xd84f398e,
	0xde59398e,0xd84f3990,0xde5b3990,0xd84f3992,0xde5b3992,0xd84f3994,0xde5d3994,0xd84f3996,
	0xde5d3996,0xd84f3998,0xde5f3998,0xd84f399a,0xde5f399a,0xd84f399c,0xde61399c,0xd84f399e,
	0xde61399e,0xd84f39a0,0xde6339a0,0xd84f39a2,0xde6339a2,0xd84f39a4,0xde6539a4,0xd84f39a6,
	0xde6539a6,0xd84f39a8,0xde6739a8,0xd84f39aa,0xde6739aa,0xd84f39ac,0xde6939ac,0xd84f39ae,
	0xde6939ae,0xd84f39b0,0xde6b39b0,0xd84f39b2,0xde6b39b2,0xd84f39b4,0xde6d39b4,0xd84f39b6,
	0xde6d39b6,0xd84f39b8,0xde6f39b8,0xd84f39ba,0xde6f39ba,0xd84f39bc,0xde7139bc,0xd84f39be,
	0xde7139be,0xd84f39c0,0xde7339c0,0xd84f39c2,0xde7339c2,0xd84f39c4,0xde7539c4,0xd84f39c6,
	0xde7539c6,0xd84f39c8,0xde7739c8,0xd84f39ca,0xde7739ca,0xd84f39cc,0xde7939cc,0xd84f39ce,
	0xde7939ce,0xd84f39d0,0xde7b39d0,0xd84f39d2,0xde7b39d2,0xd84f39d4,0xde7d39d4,0xd84f39d6,
	0xde7d39d6,0xd84f39d8,0xde7f39d8,0xd84f39da,0xde7f39da,0xd84f39dc,0xde8139dc,0xd84f39de,
	0xde8139de,0xd84f39e0,0xde8339e0,0xd84f39e2,0xde8339e2,0xd84f39e4,0xde8539e4,0xd84f39e6,
	0xde8539e6,0xd84f39e8,0xde8739e8,0xd84f39ea,0xde8739ea,0xd84f39ec,0xde8939ec,0xd84f39ee,
	0xde8939ee,0xd84f39f0,0xde8b39f0,0xd84f39f2,0xde8b39f2,0xd84f39f4,0xde8d39f4,0xd84f39f6,
	0xde8d39f6,0xd84f39f8,0xde8f39f8,0xd84f39fa,0xde8f39fa,0xd84f39fc,0xde9139fc,0xd84f39fe,
	0xde9139fe,0xd84f3a00,0xde933a00,0xd84f3a02,0xde933a02,0xd84f3a04,0xde953a04,0xd84f3a06,
	0xde953a06,0xd84f3a08,0xde973a08,0xd84f3a0a,0xde973a0a,0xd84f3a0c,0xde993a0c,0xd84f3a0e,
	0xde993a0e,0xd84f3a10,0xde9b3a10,0xd84f3a12,0xde9b3a12,0xd84f3a14,0xde9d3a14,0xd84f3a16,
	0xde9d3a16,0xd84f3a18,0xde9f3a18,0xd84f3a1a,0xde9f3a1a,0xd84f3a1c,0xdea13a1c,0xd84f3a1e,
	0xdea13a1e,0xd84f3a20,0xdea33a20,0xd84f3a22,0xdea33a22,0xd84f3a24,0xdea53a24,0xd84f3a26,
	0xdea53a26,0xd84f3a28,0xdea73a28,0xd84f3a2a,0xdea73a2a,0xd84f3a2c,0xdea93a2c,0xd84f3a2e,
	0xdea93a2e,0xd84f3a30,0xdeab3a30,0xd84f3a32,0xdeab3a32,0xd84f3a34,0xdead3a34,0xd84f3a36,
	0xdead3a36,0xd84f3a38,0xdeaf3a38,0xd84f3a3a,0xdeaf3a3a,0xd84f3a3c,0xdeb13a3c,0xd84f3a3e,
	0xdeb13a3e,0xd84f3a40,0xdeb33a40,0xd84f3a42,0xdeb33a42,0xd84f3a44,0xdeb53a44,0xd84f3a46,
	0xdeb53a46,0xd84f3a48,0xdeb73a48,0xd84f3a4a,0xdeb73a4a,0xd84f3a4c,0xdeb93a4c,0xd84f3a4e,
	0xdeb93a4e,0xd84f3a50,0xdebb3a50,0xd84f3a52,0xdebb3a52,0xd84f3a54,0xdebd3a54,0xd84f3a56,
	0xdebd3a56,0xd84f3a58,0xdebf3a58,0xd84f3a5a,0xdebf3a5a,0xd84f3a5c,0xdec13a5c,0xd84f3a5e,
	0xdec13a5e,0xd84f3a60,0xdec33a60,0xd84f3a62,0xdec33a62,0xd84f3a64,0xdec53a64,0xd84f3a66,
	0xdec53a66,0xd84f3a68,0xdec73a68,0xd84f3a6a,0xdec73a6a,0xd84f3a6c,0xdec93a6c,0xd84f3a6e,
	0xdec93a6e,0xd84f3a70,0xdecb3a70,0xd84f3a72,0xdecb3a72,0xd84f3a74,0xdecd3a74,0xd84f3a76,
	0xdecd3a76,0xd84f3a78,0xdecf3a78,0xd84f3a7a,0xdecf3a7a,0xd84f3a7c,0xded13a7c,0xd84f3a7e,
	0xded13a7e,0xd84f3a80,0xded33a80,0xd84f3a82,0xded33a82,0xd84f3a84,0xded53a84,0xd84f3a86,
	0xded53a86,0xd84f3a88,0xded73a88,0xd84f3a8a,0xded73a8a,0xd84f3a8c,0xded93a8c,0xd84f3a8e,
	0xded93a8e,0xd84f3a90,0xdedb3a90,0xd84f3a92,0xdedb3a92,0xd84f3a94,0xdedd3a94,0xd84f3a96,
	0xdedd3a96,0xd84f3a98,0xdedf3a98,0xd84f3a9a,0xdedf3a9a,0xd84f3a9c,0xdee13a9c,0xd84f3a9e,
	0xdee13a9e,0xd84f3aa0,0xdee33aa0,0xd84f3aa2,0xdee33aa2,0xd84f3aa4,0xdee53aa4,0xd84f3aa6,
	0xdee53aa6,0xd84f3aa8,0xdee73aa8,0xd84f3aaa,0xdee73aaa,0xd84f3aac,0xdee93aac,0xd84f3aae,
	0xdee93aae,0xd84f3ab0,0xdeeb3ab0,0xd84f3ab2,0xdeeb3ab2,0xd84f3ab4,0xdeed3ab4,0xd84f3ab6,
	0xdeed3ab6,0xd84f3ab8,0xdeef3ab8,0xd84f3aba,0xdeef3aba,0xd84f3abc,0xdef13abc,0xd84f3abe,
	0xdef13abe,0xd84f3ac0,0xdef33ac0,0xd84f3ac2,0xdef33ac2,0xd84f3ac4,0xdef53ac4,0xd84f3ac6,
	0xdef53ac6,0xd84f3ac8,0xdef73ac8,0xd84f3aca,0xdef73aca,0xd84f3acc,0xdef93acc,0xd84f3ace,
	0xdef93ace,0xd84f3ad0,0xdefb3ad0,0xd84f3ad2,0xdefb3ad2,0xd84f3ad4,0xdefd3ad4,0xd84f3ad6,
	0xdefd3ad6,0xd84f3ad8,0xdeff3ad8,0xd84f3ada,0xdeff3ada,0xd84f3adc,0xdf013adc,0xd84f3ade,
	0xdf013ade,0xd84f3ae0,0xdf033ae0,0xd84f3ae2,0xdf033ae2,0xd84f3ae4,0xdf053ae4,0xd84f3ae6,
	0xdf053ae6,0xd84f3ae8,0xdf073ae8,0xd84f3aea,0xdf073aea,0xd84f3aec,0xdf093aec,0xd84f3aee,
	0xdf093aee,0xd84f3af0,0xdf0b3af0,0xd84f3af2,0xdf0b3af2,0xd84f3af4,0xdf0d3af4,0xd84f3af6,
	0xdf0d3af6,0xd84f3af8,0xdf0f3af8,0xd84f3afa,0xdf0f3afa,0xd84f3afc,0xdf113afc,0xd84f3afe,
	0xdf113afe,0xd84f3b00,0xdf133b00,0xd84f3b02,0xdf133b02,0xd84f3b04,0xdf153b04,0xd84f3b06,
	0xdf153b06,0xd84f3b08,0xdf173b08,0xd84f3b0a,0xdf173b0a,0xd84f3b0c,0xdf193b0c,0xd84f3b0e,
	0xdf193b0e,0xd84f3b10,0xdf1b3b10,0xd84f3b12,0xdf1b3b12,0xd84f3b14,0xdf1d3b14,0xd84f3b16,
	0xdf1d3b16,0xd84f3b18,0xdf1f3b18,0xd84f3b1a,0xdf1f3b1a,0xd84f3b1c,0xdf213b1c,0xd84f3b1e,
	0xdf213b1e,0xd84f3b20,0xdf233b20,0xd84f3b22,0xdf233b22,0xd84f3b24,0xdf253b24,0xd84f3b26,
	0xdf253b26,0xd84f3b28,0xdf273b28,0xd84f3b2a,0xdf273b2a,0xd84f3b2c,0xdf293b2c,0xd84f3b2e,
	0xdf293b2e,0xd84f3b30,0xdf2b3b30,0xd84f3b32,0xdf2b3b32,0xd84f3b34,0xdf2d3b34,0xd84f3b36,
	0xdf2d3b36,0xd84f3b38,0xdf2f3b38,0xd84f3b3a,0xdf2f3b3a,0xd84f3b3c,0xdf313b3c,0xd84f3b3e,
	0xdf313b3e,0xd84f3b40,0xdf333b40,0xd84f3b42,0xdf333b42,0xd84f3b44,0xdf353b44,0xd84f3b46,
	0xdf353b46,0xd84f3b48,0xdf373b48,0xd84f3b4a,0xdf373b4a,0xd84f3b4c,0xdf393b4c,0xd84f3b4e,
	0xdf393b4e,0xd84f3b50,0xdf3b3b50,0xd84f3b52,0xdf3b3b52,0xd84f3b54,0xdf3d3b54,0xd84f3b56,
	0xdf3d3b56,0xd84f3b58,0xdf3f3b58,0xd84f3b5a,0xdf3f3b5a,0xd84f3b5c,0xdf413b5c,0xd84f3b5e,
	0xdf413b5e,0xd84f3b60,0xdf433b60,0xd84f3b62,0xdf433b62,0xd84f3b64,0xdf453b64,0xd84f3b66,
	0xdf453b66,0xd84f3b68,0xdf473b68,0xd84f3b6a,0xdf473b6a,0xd84f3b6c,0xdf493b6c,0xd84f3b6e,
	0xdf493b6e,0xd84f3b70,0xdf4b3b70,0xd84f3b72,0xdf4b3b72,0xd84f3b74,0xdf4d3b74,0xd84f3b76,
	0xdf4d3b76,0xd84f3b78,0xdf4f3b78,0xd84f3b7a,0xdf4f3b7a,0xd84f3b7c,0xdf513b7c,0xd84f3b7e,
	0xdf513b7e,0xd84f3b80,0xdf533b80,0xd84f3b82,0xdf533b82,0xd84f3b84,0xdf553b84,0xd84f3b86,
	0xdf553b86,0xd84f3b88,0xdf573b88,0xd84f3b8a,0xdf573b8a,0xd84f3b8c,0xdf593b8c,0xd84f3b8e,
	0xdf593b8e,0xd84f3b90,0xdf5b3b90,0xd84f3b92,0xdf5b3b92,0xd84f3b94,0xdf5d3b94,0xd84f3b96,
	0xdf5d3b96,0xd84f3b98,0xdf5f3b98,0xd84f3b9a,0xdf5f3b9a,0xd84f3b9c,0xdf613b9c,0xd84f3b9e,
	0xdf613b9e,0xd84f3ba0,0xdf633ba0,0xd84f3ba2,0xdf633ba2,0xd84f3ba4,0xdf653ba4,0xd84f3ba6,
	0xdf653ba6,0xd84f3ba8,0xdf673ba8,0xd84f3baa,0xdf673baa,0xd84f3bac,0xdf693bac,0xd84f3bae,
	0xdf693bae,0xd84f3bb0,0xdf6b3bb0,0xd84f3bb2,0xdf6b3bb2,0xd84f3bb4,0xdf6d3bb4,0xd84f3bb6,
	0xdf6d3bb6,0xd84f3bb8,0xdf6f3bb8,0xd84f3bba,0xdf6f3bba,0xd84f3bbc,0xdf713bbc,0xd84f3bbe,
	0xdf713bbe,0xd84f3bc0,0xdf733bc0,0xd84f3bc2,0xdf733bc2,0xd84f3bc4,0xdf753bc4,0xd84f3bc6,
	0xdf753bc6,0xd84f3bc8,0xdf773bc8,0xd84f3bca,0xdf773bca,0xd84f3bcc,0xdf793bcc,0xd84f3bce,
	0xdf793bce,0xd84f3bd0,0xdf7b3bd0,0xd84f3bd2,0xdf7b3bd2,0xd84f3bd4,0xdf7d3bd4,0xd84f3bd6,
	0xdf7d3bd6,0xd84f3bd8,0xdf7f3bd8,0xd84f3bda,0xdf7f3bda,0xd84f3bdc,0xdf813bdc,0xd84f3bde,
	0xdf813bde,0xd84f3be0,0xdf833be0,0xd84f3be2,0xdf833be2,0xd84f3be4,0xdf853be4,0xd84f3be6,
	0xdf853be6,0xd84f3be8,0xdf873be8,0xd84f3bea,0xdf873bea,0xd84f3bec,0xdf893bec,0xd84f3bee,
	0xdf893bee,0xd84f3bf0,0xdf8b3bf0,0xd84f3bf2,0xdf8b3bf2,0xd84f3bf4,0xdf8d3bf4,0xd84f3bf6,
	0xdf8d3bf6,0xd84f3bf8,0xdf8f3bf8,0xd84f3bfa,0xdf8f3bfa,0xd84f3bfc,0xdf913bfc,0xd84f3bfe,
	0xdf913bfe,0xd84f3c00,0xdf933c00,0xd84f3c02,0xdf933c02,0xd84f3c04,0xdf953c04,0xd84f3c06,
	0xdf953c06,0xd84f3c08,0xdf973c08,0xd84f3c0a,0xdf973c0a,0xd84f3c0c,0xdf993c0c,0xd84f3c0e,
	0xdf993c0e,0xd84f3c10,0xdf9b3c10,0xd84f3c12,0xdf9b3c12,0xd84f3c14,0xdf9d3c14,0xd84f3c16,
	0xdf9d3c16,0xd84f3c18,0xdf9f3c18,0xd84f3c1a,0xdf9f3c1a,0xd84f3c1c,0xdfa13c1c,0xd84f3c1e,
	0xdfa13c1e,0xd84f3c20,0xdfa33c20,0xd84f3c22,0xdfa33c22,0xd84f3c24,0xdfa53c24,0xd84f3c26,
	0xdfa53c26,0xd84f3c28,0xdfa73c28,0xd84f3c2a,0xdfa73c2a,0xd84f3c2c,0xdfa93c2c,0xd84f3c2e,
	0xdfa93c2e,0xd84f3c30,0xdfab3c30,0xd84f3c32,0xdfab3c32,0xd84f3c34,0xdfad3c34,0xd84f3c36,
	0xdfad3c36,0xd84f3c38,0xdfaf3c38,0xd84f3c3a,0xdfaf3c3a,0xd84f3c3c,0xdfb13c3c,0xd84f3c3e,
	0xdfb13c3e,0xd84f3c40,0xdfb33c40,0xd84f3c42,0xdfb33c42,0xd84f3c44,0xdfb53c44,0xd84f3c46,
	0xdfb53c46,0xd84f3c48,0xdfb73c48,0xd84f3c4a,0xdfb73c4a,0xd84f3c4c,0xdfb93c4c,0xd84f3c4e,
	0xdfb93c4e,0xd84f3c50,0xdfbb3c50,0xd84f3c52,0xdfbb3c52,0xd84f3c54,0xdfbd3c54,0xd84f3c56,
	0xdfbd3c56,0xd84f3c58,0xdfbf3c58,0xd84f3c5a,0xdfbf3c5a,0xd84f3c5c,0xdfc13c5c,0xd84f3c5e,
	0xdfc13c5e,0xd84f3c60,0xdfc33c60,0xd84f3c62,0xdfc33c62,0xd84f3c64,0xdfc53c64,0xd84f3c66,
	0xdfc53c66,0xd84f3c68,0xdfc73c68,0xd84f3c6a,0xdfc73c6a,0xd84f3c6c,0xdfc93c6c,0xd84f3c6e,
	0xdfc93c6e,0xd84f3c70,0xdfcb3c70,0xd84f3c72,0xdfcb3c72,0xd84f3c74,0xdfcd3c74,0xd84f3c76,
	0xdfcd3c76,0xd84f3c78,0xdfcf3c78,0xd84f3c7a,0xdfcf3c7a,0xd84f3c7c,0xdfd13c7c,0xd84f3c7e,
	0xdfd13c7e,0xd84f3c80,0xdfd33c80,0xd84f3c82,0xdfd33c82,0xd84f3c84,0xdfd53c84,0xd84f3c86,
	0xdfd53c86,0xd84f3c88,0xdfd73c88,0xd84f3c8a,0xdfd73c8a,0xd84f3c8c,0xdfd93c8c,0xd84f3c8e,
	0xdfd93c8e,0xd84f3c90,0xdfdb3c90,0xd84f3c92,0xdfdb3c92,0xd84f3c94,0xdfdd3c94,0xd84f3c96,
	0xdfdd3c96,0xd84f3c98,0xdfdf3c98,0xd84f3c9a,0xdfdf3c9a,0xd84f3c9c,0xdfe13c9c,0xd84f3c9e,
	0xdfe13c9e,0xd84f3ca0,0xdfe33ca0,0xd84f3ca2,0xdfe33ca2,0xd84f3ca4,0xdfe53ca4,0xd84f3ca6,
	0xdfe53ca6,0xd84f3ca8,0xdfe73ca8,0xd84f3caa,0xdfe73caa,0xd84f3cac,0xdfe93cac,0xd84f3cae,
	0xdfe93cae,0xd84f3cb0,0xdfeb3cb0,0xd84f3cb2,0xdfeb3cb2,0xd84f3cb4,0xdfed3cb4,0xd84f3cb6,
	0xdfed3cb6,0xd84f3cb8,0xdfef3cb8,0xd84f3cba,0xdfef3cba,0xd84f3cbc,0xdff13cbc,0xd84f3cbe,
	0xdff13cbe,0xd84f3cc0,0xdff33cc0,0xd84f3cc2,0xdff33cc2,0xd84f3cc4,0xdff53cc4,0xd84f3cc6,
	0xdff53cc6,0xd84f3cc8,0xdff73cc8,0xd84f3cca,0xdff73cca,0xd84f3ccc,0xdff93ccc,0xd84f3cce,
	0xdff93cce,0xd84f3cd0,0xdffb3cd0,0xd84f3cd2,0xdffb3cd2,0xd84f3cd4,0xdffd3cd4,0xd84f3cd6,
	0xdffd3cd6,0xd84f3cd8,0xdfff3cd8,0xd84f3cda,0xdfff3cda,0xd8513cdc,0xdc013cdc,0xd8513cde,
	0xdc013cde,0xd8513ce0,0xdc033ce0,0xd8513ce2,0xdc033ce2,0xd8513ce4,0xdc053ce4,0xd8513ce6,
	0xdc053ce6,0xd8513ce8,0xdc073ce8,0xd8513cea,0xdc073cea,0xd8513cec,0xdc093cec,0xd8513cee,
	0xdc093cee,0xd8513cf0,0xdc0b3cf0,0xd8513cf2,0xdc0b3cf2,0xd8513cf4,0xdc0d3cf4,0xd8513cf6,
	0xdc0d3cf6,0xd8513cf8,0xdc0f3cf8,0xd8513cfa,0xdc0f3cfa,0xd8513cfc,0xdc113cfc,0xd8513cfe,
	0xdc113cfe,0xd8513d00,0xdc133d00,0xd8513d02,0xdc133d02,0xd8513d04,0xdc153d04,0xd8513d06,
	0xdc153d06,0xd8513d08,0xdc173d08,0xd8513d0a,0xdc173d0a,0xd8513d0c,0xdc193d0c,0xd8513d0e,
	0xdc193d0e,0xd8513d10,0xdc1b3d10,0xd8513d12,0xdc1b3d12,0xd8513d14,0xdc1d3d14,0xd8513d16,
	0xdc1d3d16,0xd8513d18,0xdc1f3d18,0xd8513d1a,0xdc1f3d1a,0xd8513d1c,0xdc213d1c,0xd8513d1e,
	0xdc213d1e,0xd8513d20,0xdc233d20,0xd8513d22,0xdc233d22,0xd8513d24,0xdc253d24,0xd8513d26,
	0xdc253d26,0xd8513d28,0xdc273d28,0xd8513d2a,0xdc273d2a,0xd8513d2c,0xdc293d2c,0xd8513d2e,
	0xdc293d2e,0xd8513d30,0xdc2b3d30,0xd8513d32,0xdc2b3d32,0xd8513d34,0xdc2d3d34,0xd8513d36,
	0xdc2d3d36,0xd8513d38,0xdc2f3d38,0xd8513d3a,0xdc2f3d3a,0xd8513d3c,0xdc313d3c,0xd8513d3e,
	0xdc313d3e,0xd8513d40,0xdc333d40,0xd8513d42,0xdc333d42,0xd8513d44,0xdc353d44,0xd8513d46,
	0xdc353d46,0xd8513d48,0xdc373d48,0xd8513d4a,0xdc373d4a,0xd8513d4c,0xdc393d4c,0xd8513d4e,
	0xdc393d4e,0xd8513d50,0xdc3b3d50,0xd8513d52,0xdc3b3d52,0xd8513d54,0xdc3d3d54,0xd8513d56,
	0xdc3d3d56,0xd8513d58,0xdc3f3d58,0xd8513d5a,0xdc3f3d5a,0xd8513d5c,0xdc413d5c,0xd8513d5e,
	0xdc413d5e,0xd8513d60,0xdc433d60,0xd8513d62,0xdc433d62,0xd8513d64,0xdc453d64,0xd8513d66,
	0xdc453d66,0xd8513d68,0xdc473d68,0xd8513d6a,0xdc473d6a,0xd8513d6c,0xdc493d6c,0xd8513d6e,
	0xdc493d6e,0xd8513d70,0xdc4b3d70,0xd8513d72,0xdc4b3d72,0xd8513d74,0xdc4d3d74,0xd8513d76,
	0xdc4d3d76,0xd8513d78,0xdc4f3d78,0xd8513d7a,0xdc4f3d7a,0xd8513d7c,0xdc513d7c,0xd8513d7e,
	0xdc513d7e,0xd8513d80,0xdc533d80,0xd8513d82,0xdc533d82,0xd8513d84,0xdc553d84,0xd8513d86,
	0xdc553d86,0xd8513d88,0xdc573d88,0xd8513d8a,0xdc573d8a,0xd8513d8c,0xdc593d8c,0xd8513d8e,
	0xdc593d8e,0xd8513d90,0xdc5b3d90,0xd8513d92,0xdc5b3d92,0xd8513d94,0xdc5d3d94,0xd8513d96,
	0xdc5d3d96,0xd8513d98,0xdc5f3d98,0xd8513d9a,0xdc5f3d9a,0xd8513d9c,0xdc613d9c,0xd8513d9e,
	0xdc613d9e,0xd8513da0,0xdc633da0,0xd8513da2,0xdc633da2,0xd8513da4,0xdc653da4,0xd8513da6,
	0xdc653da6,0xd8513da8,0xdc673da8,0xd8513daa,0xdc673daa,0xd8513dac,0xdc693dac,0xd8513dae,
	0xdc693dae,0xd8513db0,0xdc6b3db0,0xd8513db2,0xdc6b3db2,0xd8513db4,0xdc6d3db4,0xd8513db6,
	0xdc6d3db6,0xd8513db8,0xdc6f3db8,0xd8513dba,0xdc6f3dba,0xd8513dbc,0xdc713dbc,0xd8513dbe,
	0xdc713dbe,0xd8513dc0,0xdc733dc0,0xd8513dc2,0xdc733dc2,0xd8513dc4,0xdc753dc4,0xd8513dc6,
	0xdc753dc6,0xd8513dc8,0xdc773dc8,0xd8513dca,0xdc773dca,0xd8513dcc,0xdc793dcc,0xd8513dce,
	0xdc793dce,0xd8513dd0,0xdc7b3dd0,0xd8513dd2,0xdc7b3dd2,0xd8513dd4,0xdc7d3dd4,0xd8513dd6,
	0xdc7d3dd6,0xd8513dd8,0xdc7f3dd8,0xd8513dda,0xdc7f3dda,0xd8513ddc,0xdc813ddc,0xd8513dde,
	0xdc813dde,0xd8513de0,0xdc833de0,0xd8513de2,0xdc833de2,0xd8513de4,0xdc853de4,0xd8513de6,
	0xdc853de6,0xd8513de8,0xdc873de8,0xd8513dea,0xdc873dea,0xd8513dec,0xdc893dec,0xd8513dee,
	0xdc893dee,0xd8513df0,0xdc8b3df0,0xd8513df2,0xdc8b3df2,0xd8513df4,0xdc8d3df4,0xd8513df6,
	0xdc8d3df6,0xd8513df8,0xdc8f3df8,0xd8513dfa,0xdc8f3dfa,0xd8513dfc,0xdc913dfc,0xd8513dfe,
	0xdc913dfe,0xd8513e00,0xdc933e00,0xd8513e02,0xdc933e02,0xd8513e04,0xdc953e04,0xd8513e06,
	0xdc953e06,0xd8513e08,0xdc973e08,0xd8513e0a,0xdc973e0a,0xd8513e0c,0xdc993e0c,0xd8513e0e,
	0xdc993e0e,0xd8513e10,0xdc9b3e10,0xd8513e12,0xdc9b3e12,0xd8513e14,0xdc9d3e14,0xd8513e16,
	0xdc9d3e16,0xd8513e18,0xdc9f3e18,0xd8513e1a,0xdc9f3e1a,0xd8513e1c,0xdca13e1c,0xd8513e1e,
	0xdca13e1e,0xd8513e20,0xdca33e20,0xd8513e22,0xdca33e22,0xd8513e24,0xdca53e24,0xd8513e26,
	0xdca53e26,0xd8513e28,0xdca73e28,0xd8513e2a,0xdca73e2a,0xd8513e2c,0xdca93e2c,0xd8513e2e,
	0xdca93e2e,0xd8513e30,0xdcab3e30,0xd8513e32,0xdcab3e32,0xd8513e34,0xdcad3e34,0xd8513e36,
	0xdcad3e36,0xd8513e38,0xdcaf3e38,0xd8513e3a,0xdcaf3e3a,0xd8513e3c,0xdcb13e3c,0xd8513e3e,
	0xdcb13e3e,0xd8513e40,0xdcb33e40,0xd8513e42,0xdcb33e42,0xd8513e44,0xdcb53e44,0xd8513e46,
	0xdcb53e46,0xd8513e48,0xdcb73e48,0xd8513e4a,0xdcb73e4a,0xd8513e4c,0xdcb93e4c,0xd8513e4e,
	0xdcb93e4e,0xd8513e50,0xdcbb3e50,0xd8513e52,0xdcbb3e52,0xd8513e54,0xdcbd3e54,0xd8513e56,
	0xdcbd3e56,0xd8513e58,0xdcbf3e58,0xd8513e5a,0xdcbf3e5a,0xd8513e5c,0xdcc13e5c,0xd8513e5e,
	0xdcc13e5e,0xd8513e60,0xdcc33e60,0xd8513e62,0xdcc33e62,0xd8513e64,0xdcc53e64,0xd8513e66,
	0xdcc53e66,0xd8513e68,0xdcc73e68,0xd8513e6a,0xdcc73e6a,0xd8513e6c,0xdcc93e6c,0xd8513e6e,
	0xdcc93e6e,0xd8513e70,0xdccb3e70,0xd8513e72,0xdccb3e72,0xd8513e74,0xdccd3e74,0xd8513e76,
	0xdccd3e76,0xd8513e78,0xdccf3e78,0xd8513e7a,0xdccf3e7a,0xd8513e7c,0xdcd13e7c,0xd8513e7e,
	0xdcd13e7e,0xd8513e80,0xdcd33e80,0xd8513e82,0xdcd33e82,0xd8513e84,0xdcd53e84,0xd8513e86,
	0xdcd53e86,0xd8513e88,0xdcd73e88,0xd8513e8a,0xdcd73e8a,0xd8513e8c,0xdcd93e8c,0xd8513e8e,
	0xdcd93e8e,0xd8513e90,0xdcdb3e90,0xd8513e92,0xdcdb3e92,0xd8513e94,0xdcdd3e94,0xd8513e96,
	0xdcdd3e96,0xd8513e98,0xdcdf3e98,0xd8513e9a,0xdcdf3e9a,0xd8513e9c,0xdce13e9c,0xd8513e9e,
	0xdce13e9e,0xd8513ea0,0xdce33ea0,0xd8513ea2,0xdce33ea2,0xd8513ea4,0xdce53ea4,0xd8513ea6,
	0xdce53ea6,0xd8513ea8,0xdce73ea8,0xd8513eaa,0xdce73eaa,0xd8513eac,0xdce93eac,0xd8513eae,
	0xdce93eae,0xd8513eb0,0xdceb3eb0,0xd8513eb2,0xdceb3eb2,0xd8513eb4,0xdced3eb4,0xd8513eb6,
	0xdced3eb6,0xd8513eb8,0xdcef3eb8,0xd8513eba,0xdcef3eba,0xd8513ebc,0xdcf13ebc,0xd8513ebe,
	0xdcf13ebe,0xd8513ec0,0xdcf33ec0,0xd8513ec2,0xdcf33ec2,0xd8513ec4,0xdcf53ec4,0xd8513ec6,
	0xdcf53ec6,0xd8513ec8,0xdcf73ec8,0xd8513eca,0xdcf73eca,0xd8513ecc,0xdcf93ecc,0xd8513ece,
	0xdcf93ece,0xd8513ed0,0xdcfb3ed0,0xd8513ed2,0xdcfb3ed2,0xd8513ed4,0xdcfd3ed4,0xd8513ed6,
	0xdcfd3ed6,0xd8513ed8,0xdcff3ed8,0xd8513eda,0xdcff3eda,0xd8513edc,0xdd013edc,0xd8513ede,
	0xdd013ede,0xd8513ee0,0xdd033ee0,0xd8513ee2,0xdd033ee2,0xd8513ee4,0xdd053ee4,0xd8513ee6,
	0xdd053ee6,0xd8513ee8,0xdd073ee8,0xd8513eea,0xdd073eea,0xd8513eec,0xdd093eec,0xd8513eee,
	0xdd093eee,0xd8513ef0,0xdd0b3ef0,0xd8513ef2,0xdd0b3ef2,0xd8513ef4,0xdd0d3ef4,0xd8513ef6,
	0xdd0d3ef6,0xd8513ef8,0xdd0f3ef8,0xd8513efa,0xdd0f3efa,0xd8513efc,0xdd113efc,0xd8513efe,
	0xdd113efe,0xd8513f00,0xdd133f00,0xd8513f02,0xdd133f02,0xd8513f04,0xdd153f04,0xd8513f06,
	0xdd153f06,0xd8513f08,0xdd173f08,0xd8513f0a,0xdd173f0a,0xd8513f0c,0xdd193f0c,0xd8513f0e,
	0xdd193f0e,0xd8513f10,0xdd1b3f10,0xd8513f12,0xdd1b3f12,0xd8513f14,0xdd1d3f14,0xd8513f16,
	0xdd1d3f16,0xd8513f18,0xdd1f3f18,0xd8513f1a,0xdd1f3f1a,0xd8513f1c,0xdd213f1c,0xd8513f1e,
	0xdd213f1e,0xd8513f20,0xdd233f20,0xd8513f22,0xdd233f22,0xd8513f24,0xdd253f24,0xd8513f26,
	0xdd253f26,0xd8513f28,0xdd273f28,0xd8513f2a,0xdd273f2a,0xd8513f2c,0xdd293f2c,0xd8513f2e,
	0xdd293f2e,0xd8513f30,0xdd2b3f30,0xd8513f32,0xdd2b3f32,0xd8513f34,0xdd2d3f34,0xd8513f36,
	0xdd2d3f36,0xd8513f38,0xdd2f3f38,0xd8513f3a,0xdd2f3f3a,0xd8513f3c,0xdd313f3c,0xd8513f3e,
	0xdd313f3e,0xd8513f40,0xdd333f40,0xd8513f42,0xdd333f42,0xd8513f44,0xdd353f44,0xd8513f46,
	0xdd353f46,0xd8513f48,0xdd373f48,0xd8513f4a,0xdd373f4a,0xd8513f4c,0xdd393f4c,0xd8513f4e,
	0xdd393f4e,0xd8513f50,0xdd3b3f50,0xd8513f52,0xdd3b3f52,0xd8513f54,0xdd3d3f54,0xd8513f56,
	0xdd3d3f56,0xd8513f58,0xdd3f3f58,0xd8513f5a,0xdd3f3f5a,0xd8513f5c,0xdd413f5c,0xd8513f5e,
	0xdd413f5e,0xd8513f60,0xdd433f60,0xd8513f62,0xdd433f62,0xd8513f64,0xdd453f64,0xd8513f66,
	0xdd453f66,0xd8513f68,0xdd473f68,0xd8513f6a,0xdd473f6a,0xd8513f6c,0xdd493f6c,0xd8513f6e,
	0xdd493f6e,0xd8513f70,0xdd4b3f70,0xd8513f72,0xdd4b3f72,0xd8513f74,0xdd4d3f74,0xd8513f76,
	0xdd4d3f76,0xd8513f78,0xdd4f3f78,0xd8513f7a,0xdd4f3f7a,0xd8513f7c,0xdd513f7c,0xd8513f7e,
	0xdd513f7e,0xd8513f80,0xdd533f80,0xd8513f82,0xdd533f82,0xd8513f84,0xdd553f84,0xd8513f86,
	0xdd553f86,0xd8513f88,0xdd573f88,0xd8513f8a,0xdd573f8a,0xd8513f8c,0xdd593f8c,0xd8513f8e,
	0xdd593f8e,0xd8513f90,0xdd5b3f90,0xd8513f92,0xdd5b3f92,0xd8513f94,0xdd5d3f94,0xd8513f96,
	0xdd5d3f96,0xd8513f98,0xdd5f3f98,0xd8513f9a,0xdd5f3f9a,0xd8513f9c,0xdd613f9c,0xd8513f9e,
	0xdd613f9e,0xd8513fa0,0xdd633fa0,0xd8513fa2,0xdd633fa2,0xd8513fa4,0xdd653fa4,0xd8513fa6,
	0xdd653fa6,0xd8513fa8,0xdd673fa8,0xd8513faa,0xdd673faa,0xd8513fac,0xdd693fac,0xd8513fae,
	0xdd693fae,0xd8513fb0,0xdd6b3fb0,0xd8513fb2,0xdd6b3fb2,0xd8513fb4,0xdd6d3fb4,0xd8513fb6,
	0xdd6d3fb6,0xd8513fb8,0xdd6f3fb8,0xd8513fba,0xdd6f3fba,0xd8513fbc,0xdd713fbc,0xd8513fbe,
	0xdd713fbe,0xd8513fc0,0xdd733fc0,0xd8513fc2,0xdd733fc2,0xd8513fc4,0xdd753fc4,0xd8513fc6,
	0xdd753fc6,0xd8513fc8,0xdd773fc8,0xd8513fca,0xdd773fca,0xd8513fcc,0xdd793fcc,0xd8513fce,
	0xdd793fce,0xd8513fd0,0xdd7b3fd0,0xd8513fd2,0xdd7b3fd2,0xd8513fd4,0xdd7d3fd4,0xd8513fd6,
	0xdd7d3fd6,0xd8513fd8,0xdd7f3fd8,0xd8513fda,0xdd7f3fda,0xd8513fdc,0xdd813fdc,0xd8513fde,
	0xdd813fde,0xd8513fe0,0xdd833fe0,0xd8513fe2,0xdd833fe2,0xd8513fe4,0xdd853fe4,0xd8513fe6,
	0xdd853fe6,0xd8513fe8,0xdd873fe8,0xd8513fea,0xdd873fea,0xd8513fec,0xdd893fec,0xd8513fee,
	0xdd893fee,0xd8513ff0,0xdd8b3ff0,0xd8513ff2,0xdd8b3ff2,0xd8513ff4,0xdd8d3ff4,0xd8513ff6,
	0xdd8d3ff6,0xd8513ff8,0xdd8f3ff8,0xd8513ffa,0xdd8f3ffa,0xd8513ffc,0xdd913ffc,0xd8513ffe,
	0xdd913ffe,0xd8514000,0xdd934000,0xd8514002,0xdd934002,0xd8514004,0xdd954004,0xd8514006,
	0xdd954006,0xd8514008,0xdd974008,0xd851400a,0xdd97400a,0xd851400c,0xdd99400c,0xd851400e,
	0xdd99400e,0xd8514010,0xdd9b4010,0xd8514012,0xdd9b4012,0xd8514014,0xdd9d4014,0xd8514016,
	0xdd9d4016,0xd8514018,0xdd9f4018,0xd851401a,0xdd9f401a,0xd851401c,0xdda1401c,0xd851401e,
	0xdda1401e,0xd8514020,0xdda34020,0xd8514022,0xdda34022,0xd8514024,0xdda54024,0xd8514026,
	0xdda54026,0xd8514028,0xdda74028,0xd851402a,0xdda7402a,0xd851402c,0xdda9402c,0xd851402e,
	0xdda9402e,0xd8514030,0xddab4030,0xd8514032,0xddab4032,0xd8514034,0xddad4034,0xd8514036,
	0xddad4036,0xd8514038,0xddaf4038,0xd851403a,0xddaf403a,0xd851403c,0xddb1403c,0xd851403e,
	0xddb1403e,0xd8514040,0xddb34040,0xd8514042,0xddb34042,0xd8514044,0xddb54044,0xd8514046,
	0xddb54046,0xd8514048,0xddb74048,0xd851404a,0xddb7404a,0xd851404c,0xddb9404c,0xd851404e,
	0xddb9404e,0xd8514050,0xddbb4050,0xd8514052,0xddbb4052,0xd8514054,0xddbd4054,0xd8514056,
	0xddbd4056,0xd8514058,0xddbf4058,0xd851405a,0xddbf405a,0xd851405c,0xddc1405c,0xd851405e,
	0xddc1405e,0xd8514060,0xddc34060,0xd8514062,0xddc34062,0xd8514064,0xddc54064,0xd8514066,
	0xddc54066,0xd8514068,0xddc74068,0xd851406a,0xddc7406a,0xd851406c,0xddc9406c,0xd851406e,
	0xddc9406e,0xd8514070,0xddcb4070,0xd8514072,0xddcb4072,0xd8514074,0xddcd4074,0xd8514076,
	0xddcd4076,0xd8514078,0xddcf4078,0xd851407a,0xddcf407a,0xd851407c,0xddd1407c,0xd851407e,
	0xddd1407e,0xd8514080,0xddd34080,0xd8514082,0xddd34082,0xd8514084,0xddd54084,0xd8514086,
	0xddd54086,0xd8514088,0xddd74088,0xd851408a,0xddd7408a,0xd851408c,0xddd9408c,0xd851408e,
	0xddd9408e,0xd8514090,0xdddb4090,0xd8514092,0xdddb4092,0xd8514094,0xdddd4094,0xd8514096,
	0xdddd4096,0xd8514098,0xdddf4098,0xd851409a,0xdddf409a,0xd851409c,0xdde1409c,0xd851409e,
	0xdde1409e,0xd85140a0,0xdde340a0,0xd85140a2,0xdde340a2,0xd85140a4,0xdde540a4,0xd85140a6,
	0xdde540a6,0xd85140a8,0xdde740a8,0xd85140aa,0xdde740aa,0xd85140ac,0xdde940ac,0xd85140ae,
	0xdde940ae,0xd85140b0,0xddeb40b0,0xd85140b2,0xddeb40b2,0xd85140b4,0xdded40b4,0xd85140b6,
	0xdded40b6,0xd85140b8,0xddef40b8,0xd85140ba,0xddef40ba,0xd85140bc,0xddf140bc,0xd85140be,
	0xddf140be,0xd85140c0,0xddf340c0,0xd85140c2,0xddf340c2,0xd85140c4,0xddf540c4,0xd85140c6,
	0xddf540c6,0xd85140c8,0xddf740c8,0xd85140ca,0xddf740ca,0xd85140cc,0xddf940cc,0xd85140ce,
	0xddf940ce,0xd85140d0,0xddfb40d0,0xd85140d2,0xddfb40d2,0xd85140d4,0xddfd40d4,0xd85140d6,
	0xddfd40d6,0xd85140d8,0xddff40d8,0xd85140da,0xddff40da,0xd85140dc,0xde0140dc,0xd85140de,
	0xde0140de,0xd85140e0,0xde0340e0,0xd85140e2,0xde0340e2,0xd85140e4,0xde0540e4,0xd85140e6,
	0xde0540e6,0xd85140e8,0xde0740e8,0xd85140ea,0xde0740ea,0xd85140ec,0xde0940ec,0xd85140ee,
	0xde0940ee,0xd85140f0,0xde0b40f0,0xd85140f2,0xde0b40f2,0xd85140f4,0xde0d40f4,0xd85140f6,
	0xde0d40f6,0xd85140f8,0xde0f40f8,0xd85140fa,0xde0f40fa,0xd85140fc,0xde1140fc,0xd85140fe,
	0xde1140fe,0xd8514100,0xde134100,0xd8514102,0xde134102,0xd8514104,0xde154104,0xd8514106,
	0xde154106,0xd8514108,0xde174108,0xd851410a,0xde17410a,0xd851410c,0xde19410c,0xd851410e,
	0xde19410e,0xd8514110,0xde1b4110,0xd8514112,0xde1b4112,0xd8514114,0xde1d4114,0xd8514116,
	0xde1d4116,0xd8514118,0xde1f4118,0xd851411a,0xde1f411a,0xd851411c,0xde21411c,0xd851411e,
	0xde21411e,0xd8514120,0xde234120,0xd8514122,0xde234122,0xd8514124,0xde254124,0xd8514126,
	0xde254126,0xd8514128,0xde274128,0xd851412a,0xde27412a,0xd851412c,0xde29412c,0xd851412e,
	0xde29412e,0xd8514130,0xde2b4130,0xd8514132,0xde2b4132,0xd8514134,0xde2d4134,0xd8514136,
	0xde2d4136,0xd8514138,0xde2f4138,0xd851413a,0xde2f413a,0xd851413c,0xde31413c,0xd851413e,
	0xde31413e,0xd8514140,0xde334140,0xd8514142,0xde334142,0xd8514144,0xde354144,0xd8514146,
	0xde354146,0xd8514148,0xde374148,0xd851414a,0xde37414a,0xd851414c,0xde39414c,0xd851414e,
	0xde39414e,0xd8514150,0xde3b4150,0xd8514152,0xde3b4152,0xd8514154,0xde3d4154,0xd8514156,
	0xde3d4156,0xd8514158,0xde3f4158,0xd851415a,0xde3f415a,0xd851415c,0xde41415c,0xd851415e,
	0xde41415e,0xd8514160,0xde434160,0xd8514162,0xde434162,0xd8514164,0xde454164,0xd8514166,
	0xde454166,0xd8514168,0xde474168,0xd851416a,0xde47416a,0xd851416c,0xde49416c,0xd851416e,
	0xde49416e,0xd8514170,0xde4b4170,0xd8514172,0xde4b4172,0xd8514174,0xde4d4174,0xd8514176,
	0xde4d4176,0xd8514178,0xde4f4178,0xd851417a,0xde4f417a,0xd851417c,0xde51417c,0xd851417e,
	0xde51417e,0xd8514180,0xde534180,0xd8514182,0xde534182,0xd8514184,0xde554184,0xd8514186,
	0xde554186,0xd8514188,0xde574188,0xd851418a,0xde57418a,0xd851418c,0xde59418c,0xd851418e,
	0xde59418e,0xd8514190,0xde5b4190,0xd8514192,0xde5b4192,0xd8514194,0xde5d4194,0xd8514196,
	0xde5d4196,0xd8514198,0xde5f4198,0xd851419a,0xde5f419a,0xd851419c,0xde61419c,0xd851419e,
	0xde61419e,0xd85141a0,0xde6341a0,0xd85141a2,0xde6341a2,0xd85141a4,0xde6541a4,0xd85141a6,
	0xde6541a6,0xd85141a8,0xde6741a8,0xd85141aa,0xde6741aa,0xd85141ac,0xde6941ac,0xd85141ae,
	0xde6941ae,0xd85141b0,0xde6b41b0,0xd85141b2,0xde6b41b2,0xd85141b4,0xde6d41b4,0xd85141b6,
	0xde6d41b6,0xd85141b8,0xde6f41b8,0xd85141ba,0xde6f41ba,0xd85141bc,0xde7141bc,0xd85141be,
	0xde7141be,0xd85141c0,0xde7341c0,0xd85141c2,0xde7341c2,0xd85141c4,0xde7541c4,0xd85141c6,
	0xde7541c6,0xd85141c8,0xde7741c8,0xd85141ca,0xde7741ca,0xd85141cc,0xde7941cc,0xd85141ce,
	0xde7941ce,0xd85141d0,0xde7b41d0,0xd85141d2,0xde7b41d2,0xd85141d4,0xde7d41d4,0xd85141d6,
	0xde7d41d6,0xd85141d8,0xde7f41d8,0xd85141da,0xde7f41da,0xd85141dc,0xde8141dc,0xd85141de,
	0xde8141de,0xd85141e0,0xde8341e0,0xd85141e2,0xde8341e2,0xd85141e4,0xde8541e4,0xd85141e6,
	0xde8541e6,0xd85141e8,0xde8741e8,0xd85141ea,0xde8741ea,0xd85141ec,0xde8941ec,0xd85141ee,
	0xde8941ee,0xd85141f0,0xde8b41f0,0xd85141f2,0xde8b41f2,0xd85141f4,0xde8d41f4,0xd85141f6,
	0xde8d41f6,0xd85141f8,0xde8f41f8,0xd85141fa,0xde8f41fa,0xd85141fc,0xde9141fc,0xd85141fe,
	0xde9141fe,0xd8514200,0xde934200,0xd8514202,0xde934202,0xd8514204,0xde954204,0xd8514206,
	0xde954206,0xd8514208,0xde974208,0xd851420a,0xde97420a,0xd851420c,0xde99420c,0xd851420e,
	0xde99420e,0xd8514210,0xde9b4210,0xd8514212,0xde9b4212,0xd8514214,0xde9d4214,0xd8514216,
	0xde9d4216,0xd8514218,0xde9f4218,0xd851421a,0xde9f421a,0xd851421c,0xdea1421c,0xd851421e,
	0xdea1421e,0xd8514220,0xdea34220,0xd8514222,0xdea34222,0xd8514224,0xdea54224,0xd8514226,
	0xdea54226,0xd8514228,0xdea74228,0xd851422a,0xdea7422a,0xd851422c,0xdea9422c,0xd851422e,
	0xdea9422e,0xd8514230,0xdeab4230,0xd8514232,0xdeab4232,0xd8514234,0xdead4234,0xd8514236,
	0xdead4236,0xd8514238,0xdeaf4238,0xd851423a,0xdeaf423a,0xd851423c,0xdeb1423c,0xd851423e,
	0xdeb1423e,0xd8514240,0xdeb34240,0xd8514242,0xdeb34242,0xd8514244,0xdeb54244,0xd8514246,
	0xdeb54246,0xd8514248,0xdeb74248,0xd851424a,0xdeb7424a,0xd851424c,0xdeb9424c,0xd851424e,
	0xdeb9424e,0xd8514250,0xdebb4250,0xd8514252,0xdebb4252,0xd8514254,0xdebd4254,0xd8514256,
	0xdebd4256,0xd8514258,0xdebf4258,0xd851425a,0xdebf425a,0xd851425c,0xdec1425c,0xd851425e,
	0xdec1425e,0xd8514260,0xdec34260,0xd8514262,0xdec34262,0xd8514264,0xdec54264,0xd8514266,
	0xdec54266,0xd8514268,0xdec74268,0xd851426a,0xdec7426a,0xd851426c,0xdec9426c,0xd851426e,
	0xdec9426e,0xd8514270,0xdecb4270,0xd8514272,0xdecb4272,0xd8514274,0xdecd4274,0xd8514276,
	0xdecd4276,0xd8514278,0xdecf4278,0xd851427a,0xdecf427a,0xd851427c,0xded1427c,0xd851427e,
	0xded1427e,0xd8514280,0xded34280,0xd8514282,0xded34282,0xd8514284,0xded54284,0xd8514286,
	0xded54286,0xd8514288,0xded74288,0xd851428a,0xded7428a,0xd851428c,0xded9428c,0xd851428e,
	0xded9428e,0xd8514290,0xdedb4290,0xd8514292,0xdedb4292,0xd8514294,0xdedd4294,0xd8514296,
	0xdedd4296,0xd8514298,0xdedf4298,0xd851429a,0xdedf429a,0xd851429c,0xdee1429c,0xd851429e,
	0xdee1429e,0xd85142a0,0xdee342a0,0xd85142a2,0xdee342a2,0xd85142a4,0xdee542a4,0xd85142a6,
	0xdee542a6,0xd85142a8,0xdee742a8,0xd85142aa,0xdee742aa,0xd85142ac,0xdee942ac,0xd85142ae,
	0xdee942ae,0xd85142b0,0xdeeb42b0,0xd85142b2,0xdeeb42b2,0xd85142b4,0xdeed42b4,0xd85142b6,
	0xdeed42b6,0xd85142b8,0xdeef42b8,0xd85142ba,0xdeef42ba,0xd85142bc,0xdef142bc,0xd85142be,
	0xdef142be,0xd85142c0,0xdef342c0,0xd85142c2,0xdef342c2,0xd85142c4,0xdef542c4,0xd85142c6,
	0xdef542c6,0xd85142c8,0xdef742c8,0xd85142ca,0xdef742ca,0xd85142cc,0xdef942cc,0xd85142ce,
	0xdef942ce,0xd85142d0,0xdefb42d0,0xd85142d2,0xdefb42d2,0xd85142d4,0xdefd42d4,0xd85142d6,
	0xdefd42d6,0xd85142d8,0xdeff42d8,0xd85142da,0xdeff42da,0xd85142dc,0xdf0142dc,0xd85142de,
	0xdf0142de,0xd85142e0,0xdf0342e0,0xd85142e2,0xdf0342e2,0xd85142e4,0xdf0542e4,0xd85142e6,
	0xdf0542e6,0xd85142e8,0xdf0742e8,0xd85142ea,0xdf0742ea,0xd85142ec,0xdf0942ec,0xd85142ee,
	0xdf0942ee,0xd85142f0,0xdf0b42f0,0xd85142f2,0xdf0b42f2,0xd85142f4,0xdf0d42f4,0xd85142f6,
	0xdf0d42f6,0xd85142f8,0xdf0f42f8,0xd85142fa,0xdf0f42fa,0xd85142fc,0xdf1142fc,0xd85142fe,
	0xdf1142fe,0xd8514300,0xdf134300,0xd8514302,0xdf134302,0xd8514304,0xdf154304,0xd8514306,
	0xdf154306,0xd8514308,0xdf174308,0xd851430a,0xdf17430a,0xd851430c,0xdf19430c,0xd851430e,
	0xdf19430e,0xd8514310,0xdf1b4310,0xd8514312,0xdf1b4312,0xd8514314,0xdf1d4314,0xd8514316,
	0xdf1d4316,0xd8514318,0xdf1f4318,0xd851431a,0xdf1f431a,0xd851431c,0xdf21431c,0xd851431e,
	0xdf21431e,0xd8514320,0xdf234320,0xd8514322,0xdf234322,0xd8514324,0xdf254324,0xd8514326,
	0xdf254326,0xd8514328,0xdf274328,0xd851432a,0xdf27432a,0xd851432c,0xdf29432c,0xd851432e,
	0xdf29432e,0xd8514330,0xdf2b4330,0xd8514332,0xdf2b4332,0xd8514334,0xdf2d4334,0xd8514336,
	0xdf2d4336,0xd8514338,0xdf2f4338,0xd851433a,0xdf2f433a,0xd851433c,0xdf31433c,0xd851433e,
	0xdf31433e,0xd8514340,0xdf334340,0xd8514342,0xdf334342,0xd8514344,0xdf354344,0xd8514346,
	0xdf354346,0xd8514348,0xdf374348,0xd851434a,0xdf37434a,0xd851434c,0xdf39434c,0xd851434e,
	0xdf39434e,0xd8514350,0xdf3b4350,0xd8514352,0xdf3b4352,0xd8514354,0xdf3d4354,0xd8514356,
	0xdf3d4356,0xd8514358,0xdf3f4358,0xd851435a,0xdf3f435a,0xd851435c,0xdf41435c,0xd851435e,
	0xdf41435e,0xd8514360,0xdf434360,0xd8514362,0xdf434362,0xd8514364,0xdf454364,0xd8514366,
	0xdf454366,0xd8514368,0xdf474368,0xd851436a,0xdf47436a,0xd851436c,0xdf49436c,0xd851436e,
	0xdf49436e,0xd8514370,0xdf4b4370,0xd8514372,0xdf4b4372,0xd8514374,0xdf4d4374,0xd8514376,
	0xdf4d4376,0xd8514378,0xdf4f4378,0xd851437a,0xdf4f437a,0xd851437c,0xdf51437c,0xd851437e,
	0xdf51437e,0xd8514380,0xdf534380,0xd8514382,0xdf534382,0xd8514384,0xdf554384,0xd8514386,
	0xdf554386,0xd8514388,0xdf574388,0xd851438a,0xdf57438a,0xd851438c,0xdf59438c,0xd851438e,
	0xdf59438e,0xd8514390,0xdf5b4390,0xd8514392,0xdf5b4392,0xd8514394,0xdf5d4394,0xd8514396,
	0xdf5d4396,0xd8514398,0xdf5f4398,0xd851439a,0xdf5f439a,0xd851439c,0xdf61439c,0xd851439e,
	0xdf61439e,0xd85143a0,0xdf6343a0,0xd85143a2,0xdf6343a2,0xd85143a4,0xdf6543a4,0xd85143a6,
	0xdf6543a6,0xd85143a8,0xdf6743a8,0xd85143aa,0xdf6743aa,0xd85143ac,0xdf6943ac,0xd85143ae,
	0xdf6943ae,0xd85143b0,0xdf6b43b0,0xd85143b2,0xdf6b43b2,0xd85143b4,0xdf6d43b4,0xd85143b6,
	0xdf6d43b6,0xd85143b8,0xdf6f43b8,0xd85143ba,0xdf6f43ba,0xd85143bc,0xdf7143bc,0xd85143be,
	0xdf7143be,0xd85143c0,0xdf7343c0,0xd85143c2,0xdf7343c2,0xd85143c4,0xdf7543c4,0xd85143c6,
	0xdf7543c6,0xd85143c8,0xdf7743c8,0xd85143ca,0xdf7743ca,0xd85143cc,0xdf7943cc,0xd85143ce,
	0xdf7943ce,0xd85143d0,0xdf7b43d0,0xd85143d2,0xdf7b43d2,0xd85143d4,0xdf7d43d4,0xd85143d6,
	0xdf7d43d6,0xd85143d8,0xdf7f43d8,0xd85143da,0xdf7f43da,0xd85143dc,0xdf8143dc,0xd85143de,
	0xdf8143de,0xd85143e0,0xdf8343e0,0xd85143e2,0xdf8343e2,0xd85143e4,0xdf8543e4,0xd85143e6,
	0xdf8543e6,0xd85143e8,0xdf8743e8,0xd85143ea,0xdf8743ea,0xd85143ec,0xdf8943ec,0xd85143ee,
	0xdf8943ee,0xd85143f0,0xdf8b43f0,0xd85143f2,0xdf8b43f2,0xd85143f4,0xdf8d43f4,0xd85143f6,
	0xdf8d43f6,0xd85143f8,0xdf8f43f8,0xd85143fa,0xdf8f43fa,0xd85143fc,0xdf9143fc,0xd85143fe,
	0xdf9143fe,0xd8514400,0xdf934400,0xd8514402,0xdf934402,0xd8514404,0xdf954404,0xd8514406,
	0xdf954406,0xd8514408,0xdf974408,0xd851440a,0xdf97440a,0xd851440c,0xdf99440c,0xd851440e,
	0xdf99440e,0xd8514410,0xdf9b4410,0xd8514412,0xdf9b4412,0xd8514414,0xdf9d4414,0xd8514416,
	0xdf9d4416,0xd8514418,0xdf9f4418,0xd851441a,0xdf9f441a,0xd851441c,0xdfa1441c,0xd851441e,
	0xdfa1441e,0xd8514420,0xdfa34420,0xd8514422,0xdfa34422,0xd8514424,0xdfa54424,0xd8514426,
	0xdfa54426,0xd8514428,0xdfa74428,0xd851442a,0xdfa7442a,0xd851442c,0xdfa9442c,0xd851442e,
	0xdfa9442e,0xd8514430,0xdfab4430,0xd8514432,0xdfab4432,0xd8514434,0xdfad4434,0xd8514436,
	0xdfad4436,0xd8514438,0xdfaf4438,0xd851443a,0xdfaf443a,0xd851443c,0xdfb1443c,0xd851443e,
	0xdfb1443e,0xd8514440,0xdfb34440,0xd8514442,0xdfb34442,0xd8514444,0xdfb54444,0xd8514446,
	0xdfb54446,0xd8514448,0xdfb74448,0xd851444a,0xdfb7444a,0xd851444c,0xdfb9444c,0xd851444e,
	0xdfb9444e,0xd8514450,0xdfbb4450,0xd8514452,0xdfbb4452,0xd8514454,0xdfbd4454,0xd8514456,
	0xdfbd4456,0xd8514458,0xdfbf4458,0xd851445a,0xdfbf445a,0xd851445c,0xdfc1445c,0xd851445e,
	0xdfc1445e,0xd8514460,0xdfc34460,0xd8514462,0xdfc34462,0xd8514464,0xdfc54464,0xd8514466,
	0xdfc54466,0xd8514468,0xdfc74468,0xd851446a,0xdfc7446a,0xd851446c,0xdfc9446c,0xd851446e,
	0xdfc9446e,0xd8514470,0xdfcb4470,0xd8514472,0xdfcb4472,0xd8514474,0xdfcd4474,0xd8514476,
	0xdfcd4476,0xd8514478,0xdfcf4478,0xd851447a,0xdfcf447a,0xd851447c,0xdfd1447c,0xd851447e,
	0xdfd1447e,0xd8514480,0xdfd34480,0xd8514482,0xdfd34482,0xd8514484,0xdfd54484,0xd8514486,
	0xdfd54486,0xd8514488,0xdfd74488,0xd851448a,0xdfd7448a,0xd851448c,0xdfd9448c,0xd851448e,
	0xdfd9448e,0xd8514490,0xdfdb4490,0xd8514492,0xdfdb4492,0xd8514494,0xdfdd4494,0xd8514496,
	0xdfdd4496,0xd8514498,0xdfdf4498,0xd851449a,0xdfdf449a,0xd851449c,0xdfe1449c,0xd851449e,
	0xdfe1449e,0xd85144a0,0xdfe344a0,0xd85144a2,0xdfe344a2,0xd85144a4,0xdfe544a4,0xd85144a6,
	0xdfe544a6,0xd85144a8,0xdfe744a8,0xd85144aa,0xdfe744aa,0xd85144ac,0xdfe944ac,0xd85144ae,
	0xdfe944ae,0xd85144b0,0xdfeb44b0,0xd85144b2,0xdfeb44b2,0xd85144b4,0xdfed44b4,0xd85144b6,
	0xdfed44b6,0xd85144b8,0xdfef44b8,0xd85144ba,0xdfef44ba,0xd85144bc,0xdff144bc,0xd85144be,
	0xdff144be,0xd85144c0,0xdff344c0,0xd85144c2,0xdff344c2,0xd85144c4,0xdff544c4,0xd85144c6,
	0xdff544c6,0xd85144c8,0xdff744c8,0xd85144ca,0xdff744ca,0xd85144cc,0xdff944cc,0xd85144ce,
	0xdff944ce,0xd85144d0,0xdffb44d0,0xd85144d2,0xdffb44d2,0xd85144d4,0xdffd44d4,0xd85144d6,
	0xdffd44d6,0xd85144d8,0xdfff44d8,0xd85144da,0xdfff44da,0xd85144dc,0xdc0144dc,0xd85144de,
	0xdc0144de,0xd85144e0,0xdc0344e0,0xd85144e2,0xdc0344e2,0xd85144e4,0xdc0544e4,0xd85144e6,
	0xdc0544e6,0xd85144e8,0xdc0744e8,0xd85144ea,0xdc0744ea,0xd85144ec,0xdc0944ec,0xd85144ee,
	0xdc0944ee,0xd85144f0,0xdc0b44f0,0xd85144f2,0xdc0b44f2,0xd85144f4,0xdc0d44f4,0xd85144f6,
	0xdc0d44f6,0xd85144f8,0xdc0f44f8,0xd85144fa,0xdc0f44fa,0xd85144fc,0xdc1144fc,0xd85144fe,
	0xdc1144fe,0xd8514500,0xdc134500,0xd8514502,0xdc134502,0xd8514504,0xdc154504,0xd8514506,
	0xdc154506,0xd8514508,0xdc174508,0xd851450a,0xdc17450a,0xd851450c,0xdc19450c,0xd851450e,
	0xdc19450e,0xd8514510,0xdc1b4510,0xd8514512,0xdc1b4512,0xd8514514,0xdc1d4514,0xd8514516,
	0xdc1d4516,0xd8514518,0xdc1f4518,0xd851451a,0xdc1f451a,0xd851451c,0xdc21451c,0xd851451e,
	0xdc21451e,0xd8514520,0xdc234520,0xd8514522,0xdc234522,0xd8514524,0xdc254524,0xd8514526,
	0xdc254526,0xd8514528,0xdc274528,0xd851452a,0xdc27452a,0xd851452c,0xdc29452c,0xd851452e,
	0xdc29452e,0xd8514530,0xdc2b4530,0xd8514532,0xdc2b4532,0xd8514534,0xdc2d4534,0xd8514536,
	0xdc2d4536,0xd8514538,0xdc2f4538,0xd851453a,0xdc2f453a,0xd851453c,0xdc31453c,0xd851453e,
	0xdc31453e,0xd8514540,0xdc334540,0xd8514542,0xdc334542,0xd8514544,0xdc354544,0xd8514546,
	0xdc354546,0xd8514548,0xdc374548,0xd851454a,0xdc37454a,0xd851454c,0xdc39454c,0xd851454e,
	0xdc39454e,0xd8514550,0xdc3b4550,0xd8514552,0xdc3b4552,0xd8514554,0xdc3d4554,0xd8514556,
	0xdc3d4556,0xd8514558,0xdc3f4558,0xd851455a,0xdc3f455a,0xd851455c,0xdc41455c,0xd851455e,
	0xdc41455e,0xd8514560,0xdc434560,0xd8514562,0xdc434562,0xd8514564,0xdc454564,0xd8514566,
	0xdc454566,0xd8514568,0xdc474568,0xd851456a,0xdc47456a,0xd851456c,0xdc49456c,0xd851456e,
	0xdc49456e,0xd8514570,0xdc4b4570,0xd8514572,0xdc4b4572,0xd8514574,0xdc4d4574,0xd8514576,
	0xdc4d4576,0xd8514578,0xdc4f4578,0xd851457a,0xdc4f457a,0xd851457c,0xdc51457c,0xd851457e,
	0xdc51457e,0xd8514580,0xdc534580,0xd8514582,0xdc534582,0xd8514584,0xdc554584,0xd8514586,
	0xdc554586,0xd8514588,0xdc574588,0xd851458a,0xdc57458a,0xd851458c,0xdc59458c,0xd851458e,
	0xdc59458e,0xd8514590,0xdc5b4590,0xd8514592,0xdc5b4592,0xd8514594,0xdc5d4594,0xd8514596,
	0xdc5d4596,0xd8514598,0xdc5f4598,0xd851459a,0xdc5f459a,0xd851459c,0xdc61459c,0xd851459e,
	0xdc61459e,0xd85145a0,0xdc6345a0,0xd85145a2,0xdc6345a2,0xd85145a4,0xdc6545a4,0xd85145a6,
	0xdc6545a6,0xd85145a8,0xdc6745a8,0xd85145aa,0xdc6745aa,0xd85145ac,0xdc6945ac,0xd85145ae,
	0xdc6945ae,0xd85145b0,0xdc6b45b0,0xd85145b2,0xdc6b45b2,0xd85145b4,0xdc6d45b4,0xd85145b6,
	0xdc6d45b6,0xd85145b8,0xdc6f45b8,0xd85145ba,0xdc6f45ba,0xd85145bc,0xdc7145bc,0xd85145be,
	0xdc7145be,0xd85145c0,0xdc7345c0,0xd85145c2,0xdc7345c2,0xd85145c4,0xdc7545c4,0xd85145c6,
	0xdc7545c6,0xd85145c8,0xdc7745c8,0xd85145ca,0xdc7745ca,0xd85145cc,0xdc7945cc,0xd85145ce,
	0xdc7945ce,0xd85145d0,0xdc7b45d0,0xd85145d2,0xdc7b45d2,0xd85145d4,0xdc7d45d4,0xd85145d6,
	0xdc7d45d6,0xd85145d8,0xdc7f45d8,0xd85145da,0xdc7f45da,0xd85145dc,0xdc8145dc,0xd85145de,
	0xdc8145de,0xd85145e0,0xdc8345e0,0xd85145e2,0xdc8345e2,0xd85145e4,0xdc8545e4,0xd85145e6,
	0xdc8545e6,0xd85145e8,0xdc8745e8,0xd85145ea,0xdc8745ea,0xd85145ec,0xdc8945ec,0xd85145ee,
	0xdc8945ee,0xd85145f0,0xdc8b45f0,0xd85145f2,0xdc8b45f2,0xd85145f4,0xdc8d45f4,0xd85145f6,
	0xdc8d45f6,0xd85145f8,0xdc8f45f8,0xd85145fa,0xdc8f45fa,0xd85145fc,0xdc9145fc,0xd85145fe,
	0xdc9145fe,0xd8514600,0xdc934600,0xd8514602,0xdc934602,0xd8514604,0xdc954604,0xd8514606,
	0xdc954606,0xd8514608,0xdc974608,0xd851460a,0xdc97460a,0xd851460c,0xdc99460c,0xd851460e,
	0xdc99460e,0xd8514610,0xdc9b4610,0xd8514612,0xdc9b4612,0xd8514614,0xdc9d4614,0xd8514616,
	0xdc9d4616,0xd8514618,0xdc9f4618,0xd851461a,0xdc9f461a,0xd851461c,0xdca1461c,0xd851461e,
	0xdca1461e,0xd8514620,0xdca34620,0xd8514622,0xdca34622,0xd8514624,0xdca54624,0xd8514626,
	0xdca54626,0xd8514628,0xdca74628,0xd851462a,0xdca7462a,0xd851462c,0xdca9462c,0xd851462e,
	0xdca9462e,0xd8514630,0xdcab4630,0xd8514632,0xdcab4632,0xd8514634,0xdcad4634,0xd8514636,
	0xdcad4636,0xd8514638,0xdcaf4638,0xd851463a,0xdcaf463a,0xd851463c,0xdcb1463c,0xd851463e,
	0xdcb1463e,0xd8514640,0xdcb34640,0xd8514642,0xdcb34642,0xd8514644,0xdcb54644,0xd8514646,
	0xdcb54646,0xd8514648,0xdcb74648,0xd851464a,0xdcb7464a,0xd851464c,0xdcb9464c,0xd851464e,
	0xdcb9464e,0xd8514650,0xdcbb4650,0xd8514652,0xdcbb4652,0xd8514654,0xdcbd4654,0xd8514656,
	0xdcbd4656,0xd8514658,0xdcbf4658,0xd851465a,0xdcbf465a,0xd851465c,0xdcc1465c,0xd851465e,
	0xdcc1465e,0xd8514660,0xdcc34660,0xd8514662,0xdcc34662,0xd8514664,0xdcc54664,0xd8514666,
	0xdcc54666,0xd8514668,0xdcc74668,0xd851466a,0xdcc7466a,0xd851466c,0xdcc9466c,0xd851466e,
	0xdcc9466e,0xd8514670,0xdccb4670,0xd8514672,0xdccb4672,0xd8514674,0xdccd4674,0xd8514676,
	0xdccd4676,0xd8514678,0xdccf4678,0xd851467a,0xdccf467a,0xd851467c,0xdcd1467c,0xd851467e,
	0xdcd1467e,0xd8514680,0xdcd34680,0xd8514682,0xdcd34682,0xd8514684,0xdcd54684,0xd8514686,
	0xdcd54686,0xd8514688,0xdcd74688,0xd851468a,0xdcd7468a,0xd851468c,0xdcd9468c,0xd851468e,
	0xdcd9468e,0xd8514690,0xdcdb4690,0xd8514692,0xdcdb4692,0xd8514694,0xdcdd4694,0xd8514696,
	0xdcdd4696,0xd8514698,0xdcdf4698,0xd851469a,0xdcdf469a,0xd851469c,0xdce1469c,0xd851469e,
	0xdce1469e,0xd85146a0,0xdce346a0,0xd85146a2,0xdce346a2,0xd85146a4,0xdce546a4,0xd85146a6,
	0xdce546a6,0xd85146a8,0xdce746a8,0xd85146aa,0xdce746aa,0xd85146ac,0xdce946ac,0xd85146ae,
	0xdce946ae,0xd85146b0,0xdceb46b0,0xd85146b2,0xdceb46b2,0xd85146b4,0xdced46b4,0xd85146b6,
	0xdced46b6,0xd85146b8,0xdcef46b8,0xd85146ba,0xdcef46ba,0xd85146bc,0xdcf146bc,0xd85146be,
	0xdcf146be,0xd85146c0,0xdcf346c0,0xd85146c2,0xdcf346c2,0xd85146c4,0xdcf546c4,0xd85146c6,
	0xdcf546c6,0xd85146c8,0xdcf746c8,0xd85146ca,0xdcf746ca,0xd85146cc,0xdcf946cc,0xd85146ce,
	0xdcf946ce,0xd85146d0,0xdcfb46d0,0xd85146d2,0xdcfb46d2,0xd85146d4,0xdcfd46d4,0xd85146d6,
	0xdcfd46d6,0xd85146d8,0xdcff46d8,0xd85146da,0xdcff46da,0xd85146dc,0xdd0146dc,0xd85146de,
	0xdd0146de,0xd85146e0,0xdd0346e0,0xd85146e2,0xdd0346e2,0xd85146e4,0xdd0546e4,0xd85146e6,
	0xdd0546e6,0xd85146e8,0xdd0746e8,0xd85146ea,0xdd0746ea,0xd85146ec,0xdd0946ec,0xd85146ee,
	0xdd0946ee,0xd85146f0,0xdd0b46f0,0xd85146f2,0xdd0b46f2,0xd85146f4,0xdd0d46f4,0xd85146f6,
	0xdd0d46f6,0xd85146f8,0xdd0f46f8,0xd85146fa,0xdd0f46fa,0xd85146fc,0xdd1146fc,0xd85146fe,
	0xdd1146fe,0xd8514700,0xdd134700,0xd8514702,0xdd134702,0xd8514704,0xdd154704,0xd8514706,
	0xdd154706,0xd8514708,0xdd174708,0xd851470a,0xdd17470a,0xd851470c,0xdd19470c,0xd851470e,
	0xdd19470e,0xd8514710,0xdd1b4710,0xd8514712,0xdd1b4712,0xd8514714,0xdd1d4714,0xd8514716,
	0xdd1d4716,0xd8514718,0xdd1f4718,0xd851471a,0xdd1f471a,0xd851471c,0xdd21471c,0xd851471e,
	0xdd21471e,0xd8514720,0xdd234720,0xd8514722,0xdd234722,0xd8514724,0xdd254724,0xd8514726,
	0xdd254726,0xd8514728,0xdd274728,0xd851472a,0xdd27472a,0xd851472c,0xdd29472c,0xd851472e,
	0xdd29472e,0xd8514730,0xdd2b4730,0xd8514732,0xdd2b4732,0xd8514734,0xdd2d4734,0xd8514736,
	0xdd2d4736,0xd8514738,0xdd2f4738,0xd851473a,0xdd2f473a,0xd851473c,0xdd31473c,0xd851473e,
	0xdd31473e,0xd8514740,0xdd334740,0xd8514742,0xdd334742,0xd8514744,0xdd354744,0xd8514746,
	0xdd354746,0xd8514748,0xdd374748,0xd851474a,0xdd37474a,0xd851474c,0xdd39474c,0xd851474e,
	0xdd39474e,0xd8514750,0xdd3b4750,0xd8514752,0xdd3b4752,0xd8514754,0xdd3d4754,0xd8514756,
	0xdd3d4756,0xd8514758,0xdd3f4758,0xd851475a,0xdd3f475a,0xd851475c,0xdd41475c,0xd851475e,
	0xdd41475e,0xd8514760,0xdd434760,0xd8514762,0xdd434762,0xd8514764,0xdd454764,0xd8514766,
	0xdd454766,0xd8514768,0xdd474768,0xd851476a,0xdd47476a,0xd851476c,0xdd49476c,0xd851476e,
	0xdd49476e,0xd8514770,0xdd4b4770,0xd8514772,0xdd4b4772,0xd8514774,0xdd4d4774,0xd8514776,
	0xdd4d4776,0xd8514778,0xdd4f4778,0xd851477a,0xdd4f477a,0xd851477c,0xdd51477c,0xd851477e,
	0xdd51477e,0xd8514780,0xdd534780,0xd8514782,0xdd534782,0xd8514784,0xdd554784,0xd8514786,
	0xdd554786,0xd8514788,0xdd574788,0xd851478a,0xdd57478a,0xd851478c,0xdd59478c,0xd851478e,
	0xdd59478e,0xd8514790,0xdd5b4790,0xd8514792,0xdd5b4792,0xd8514794,0xdd5d4794,0xd8514796,
	0xdd5d4796,0xd8514798,0xdd5f4798,0xd851479a,0xdd5f479a,0xd851479c,0xdd61479c,0xd851479e,
	0xdd61479e,0xd85147a0,0xdd6347a0,0xd85147a2,0xdd6347a2,0xd85147a4,0xdd6547a4,0xd85147a6,
	0xdd6547a6,0xd85147a8,0xdd6747a8,0xd85147aa,0xdd6747aa,0xd85147ac,0xdd6947ac,0xd85147ae,
	0xdd6947ae,0xd85147b0,0xdd6b47b0,0xd85147b2,0xdd6b47b2,0xd85147b4,0xdd6d47b4,0xd85147b6,
	0xdd6d47b6,0xd85147b8,0xdd6f47b8,0xd85147ba,0xdd6f47ba,0xd85147bc,0xdd7147bc,0xd85147be,
	0xdd7147be,0xd85147c0,0xdd7347c0,0xd85147c2,0xdd7347c2,0xd85147c4,0xdd7547c4,0xd85147c6,
	0xdd7547c6,0xd85147c8,0xdd7747c8,0xd85147ca,0xdd7747ca,0xd85147cc,0xdd7947cc,0xd85147ce,
	0xdd7947ce,0xd85147d0,0xdd7b47d0,0xd85147d2,0xdd7b47d2,0xd85147d4,0xdd7d47d4,0xd85147d6,
	0xdd7d47d6,0xd85147d8,0xdd7f47d8,0xd85147da,0xdd7f47da,0xd85147dc,0xdd8147dc,0xd85147de,
	0xdd8147de,0xd85147e0,0xdd8347e0,0xd85147e2,0xdd8347e2,0xd85147e4,0xdd8547e4,0xd85147e6,
	0xdd8547e6,0xd85147e8,0xdd8747e8,0xd85147ea,0xdd8747ea,0xd85147ec,0xdd8947ec,0xd85147ee,
	0xdd8947ee,0xd85147f0,0xdd8b47f0,0xd85147f2,0xdd8b47f2,0xd85147f4,0xdd8d47f4,0xd85147f6,
	0xdd8d47f6,0xd85147f8,0xdd8f47f8,0xd85147fa,0xdd8f47fa,0xd85147fc,0xdd9147fc,0xd85147fe,
	0xdd9147fe,0xd8514800,0xdd934800,0xd8514802,0xdd934802,0xd8514804,0xdd954804,0xd8514806,
	0xdd954806,0xd8514808,0xdd974808,0xd851480a,0xdd97480a,0xd851480c,0xdd99480c,0xd851480e,
	0xdd99480e,0xd8514810,0xdd9b4810,0xd8514812,0xdd9b4812,0xd8514814,0xdd9d4814,0xd8514816,
	0xdd9d4816,0xd8514818,0xdd9f4818,0xd851481a,0xdd9f481a,0xd851481c,0xdda1481c,0xd851481e,
	0xdda1481e,0xd8514820,0xdda34820,0xd8514822,0xdda34822,0xd8514824,0xdda54824,0xd8514826,
	0xdda54826,0xd8514828,0xdda74828,0xd851482a,0xdda7482a,0xd851482c,0xdda9482c,0xd851482e,
	0xdda9482e,0xd8514830,0xddab4830,0xd8514832,0xddab4832,0xd8514834,0xddad4834,0xd8514836,
	0xddad4836,0xd8514838,0xddaf4838,0xd851483a,0xddaf483a,0xd851483c,0xddb1483c,0xd851483e,
	0xddb1483e,0xd8514840,0xddb34840,0xd8514842,0xddb34842,0xd8514844,0xddb54844,0xd8514846,
	0xddb54846,0xd8514848,0xddb74848,0xd851484a,0xddb7484a,0xd851484c,0xddb9484c,0xd851484e,
	0xddb9484e,0xd8514850,0xddbb4850,0xd8514852,0xddbb4852,0xd8514854,0xddbd4854,0xd8514856,
	0xddbd4856,0xd8514858,0xddbf4858,0xd851485a,0xddbf485a,0xd851485c,0xddc1485c,0xd851485e,
	0xddc1485e,0xd8514860,0xddc34860,0xd8514862,0xddc34862,0xd8514864,0xddc54864,0xd8514866,
	0xddc54866,0xd8514868,0xddc74868,0xd851486a,0xddc7486a,0xd851486c,0xddc9486c,0xd851486e,
	0xddc9486e,0xd8514870,0xddcb4870,0xd8514872,0xddcb4872,0xd8514874,0xddcd4874,0xd8514876,
	0xddcd4876,0xd8514878,0xddcf4878,0xd851487a,0xddcf487a,0xd851487c,0xddd1487c,0xd851487e,
	0xddd1487e,0xd8514880,0xddd34880,0xd8514882,0xddd34882,0xd8514884,0xddd54884,0xd8514886,
	0xddd54886,0xd8514888,0xddd74888,0xd851488a,0xddd7488a,0xd851488c,0xddd9488c,0xd851488e,
	0xddd9488e,0xd8514890,0xdddb4890,0xd8514892,0xdddb4892,0xd8514894,0xdddd4894,0xd8514896,
	0xdddd4896,0xd8514898,0xdddf4898,0xd851489a,0xdddf489a,0xd851489c,0xdde1489c,0xd851489e,
	0xdde1489e,0xd85148a0,0xdde348a0,0xd85148a2,0xdde348a2,0xd85148a4,0xdde548a4,0xd85148a6,
	0xdde548a6,0xd85148a8,0xdde748a8,0xd85148aa,0xdde748aa,0xd85148ac,0xdde948ac,0xd85148ae,
	0xdde948ae,0xd85148b0,0xddeb48b0,0xd85148b2,0xddeb48b2,0xd85148b4,0xdded48b4,0xd85148b6,
	0xdded48b6,0xd85148b8,0xddef48b8,0xd85148ba,0xddef48ba,0xd85148bc,0xddf148bc,0xd85148be,
	0xddf148be,0xd85148c0,0xddf348c0,0xd85148c2,0xddf348c2,0xd85148c4,0xddf548c4,0xd85148c6,
	0xddf548c6,0xd85148c8,0xddf748c8,0xd85148ca,0xddf748ca,0xd85148cc,0xddf948cc,0xd85148ce,
	0xddf948ce,0xd85148d0,0xddfb48d0,0xd85148d2,0xddfb48d2,0xd85148d4,0xddfd48d4,0xd85148d6,
	0xddfd48d6,0xd85148d8,0xddff48d8,0xd85148da,0xddff48da,0xd85148dc,0xde0148dc,0xd85148de,
	0xde0148de,0xd85148e0,0xde0348e0,0xd85148e2,0xde0348e2,0xd85148e4,0xde0548e4,0xd85148e6,
	0xde0548e6,0xd85148e8,0xde0748e8,0xd85148ea,0xde0748ea,0xd85148ec,0xde0948ec,0xd85148ee,
	0xde0948ee,0xd85148f0,0xde0b48f0,0xd85148f2,0xde0b48f2,0xd85148f4,0xde0d48f4,0xd85148f6,
	0xde0d48f6,0xd85148f8,0xde0f48f8,0xd85148fa,0xde0f48fa,0xd85148fc,0xde1148fc,0xd85148fe,
	0xde1148fe,0xd8514900,0xde134900,0xd8514902,0xde134902,0xd8514904,0xde154904,0xd8514906,
	0xde154906,0xd8514908,0xde174908,0xd851490a,0xde17490a,0xd851490c,0xde19490c,0xd851490e,
	0xde19490e,0xd8514910,0xde1b4910,0xd8514912,0xde1b4912,0xd8514914,0xde1d4914,0xd8514916,
	0xde1d4916,0xd8514918,0xde1f4918,0xd851491a,0xde1f491a,0xd851491c,0xde21491c,0xd851491e,
	0xde21491e,0xd8514920,0xde234920,0xd8514922,0xde234922,0xd8514924,0xde254924,0xd8514926,
	0xde254926,0xd8514928,0xde274928,0xd851492a,0xde27492a,0xd851492c,0xde29492c,0xd851492e,
	0xde29492e,0xd8514930,0xde2b4930,0xd8514932,0xde2b4932,0xd8514934,0xde2d4934,0xd8514936,
	0xde2d4936,0xd8514938,0xde2f4938,0xd851493a,0xde2f493a,0xd851493c,0xde31493c,0xd851493e,
	0xde31493e,0xd8514940,0xde334940,0xd8514942,0xde334942,0xd8514944,0xde354944,0xd8514946,
	0xde354946,0xd8514948,0xde374948,0xd851494a,0xde37494a,0xd851494c,0xde39494c,0xd851494e,
	0xde39494e,0xd8514950,0xde3b4950,0xd8514952,0xde3b4952,0xd8514954,0xde3d4954,0xd8514956,
	0xde3d4956,0xd8514958,0xde3f4958,0xd851495a,0xde3f495a,0xd851495c,0xde41495c,0xd851495e,
	0xde41495e,0xd8514960,0xde434960,0xd8514962,0xde434962,0xd8514964,0xde454964,0xd8514966,
	0xde454966,0xd8514968,0xde474968,0xd851496a,0xde47496a,0xd851496c,0xde49496c,0xd851496e,
	0xde49496e,0xd8514970,0xde4b4970,0xd8514972,0xde4b4972,0xd8514974,0xde4d4974,0xd8514976,
	0xde4d4976,0xd8514978,0xde4f4978,0xd851497a,0xde4f497a,0xd851497c,0xde51497c,0xd851497e,
	0xde51497e,0xd8514980,0xde534980,0xd8514982,0xde534982,0xd8514984,0xde554984,0xd8514986,
	0xde554986,0xd8514988,0xde574988,0xd851498a,0xde57498a,0xd851498c,0xde59498c,0xd851498e,
	0xde59498e,0xd8514990,0xde5b4990,0xd8514992,0xde5b4992,0xd8514994,0xde5d4994,0xd8514996,
	0xde5d4996,0xd8514998,0xde5f4998,0xd851499a,0xde5f499a,0xd851499c,0xde61499c,0xd851499e,
	0xde61499e,0xd85149a0,0xde6349a0,0xd85149a2,0xde6349a2,0xd85149a4,0xde6549a4,0xd85149a6,
	0xde6549a6,0xd85149a8,0xde6749a8,0xd85149aa,0xde6749aa,0xd85149ac,0xde6949ac,0xd85149ae,
	0xde6949ae,0xd85149b0,0xde6b49b0,0xd85149b2,0xde6b49b2,0xd85149b4,0xde6d49b4,0xd85149b6,
	0xde6d49b6,0xd85149b8,0xde6f49b8,0xd85149ba,0xde6f49ba,0xd85149bc,0xde7149bc,0xd85149be,
	0xde7149be,0xd85149c0,0xde7349c0,0xd85149c2,0xde7349c2,0xd85149c4,0xde7549c4,0xd85149c6,
	0xde7549c6,0xd85149c8,0xde7749c8,0xd85149ca,0xde7749ca,0xd85149cc,0xde7949cc,0xd85149ce,
	0xde7949ce,0xd85149d0,0xde7b49d0,0xd85149d2,0xde7b49d2,0xd85149d4,0xde7d49d4,0xd85149d6,
	0xde7d49d6,0xd85149d8,0xde7f49d8,0xd85149da,0xde7f49da,0xd85149dc,0xde8149dc,0xd85149de,
	0xde8149de,0xd85149e0,0xde8349e0,0xd85149e2,0xde8349e2,0xd85149e4,0xde8549e4,0xd85149e6,
	0xde8549e6,0xd85149e8,0xde8749e8,0xd85149ea,0xde8749ea,0xd85149ec,0xde8949ec,0xd85149ee,
	0xde8949ee,0xd85149f0,0xde8b49f0,0xd85149f2,0xde8b49f2,0xd85149f4,0xde8d49f4,0xd85149f6,
	0xde8d49f6,0xd85149f8,0xde8f49f8,0xd85149fa,0xde8f49fa,0xd85149fc,0xde9149fc,0xd85149fe,
	0xde9149fe,0xd8514a00,0xde934a00,0xd8514a02,0xde934a02,0xd8514a04,0xde954a04,0xd8514a06,
	0xde954a06,0xd8514a08,0xde974a08,0xd8514a0a,0xde974a0a,0xd8514a0c,0xde994a0c,0xd8514a0e,
	0xde994a0e,0xd8514a10,0xde9b4a10,0xd8514a12,0xde9b4a12,0xd8514a14,0xde9d4a14,0xd8514a16,
	0xde9d4a16,0xd8514a18,0xde9f4a18,0xd8514a1a,0xde9f4a1a,0xd8514a1c,0xdea14a1c,0xd8514a1e,
	0xdea14a1e,0xd8514a20,0xdea34a20,0xd8514a22,0xdea34a22,0xd8514a24,0xdea54a24,0xd8514a26,
	0xdea54a26,0xd8514a28,0xdea74a28,0xd8514a2a,0xdea74a2a,0xd8514a2c,0xdea94a2c,0xd8514a2e,
	0xdea94a2e,0xd8514a30,0xdeab4a30,0xd8514a32,0xdeab4a32,0xd8514a34,0xdead4a34,0xd8514a36,
	0xdead4a36,0xd8514a38,0xdeaf4a38,0xd8514a3a,0xdeaf4a3a,0xd8514a3c,0xdeb14a3c,0xd8514a3e,
	0xdeb14a3e,0xd8514a40,0xdeb34a40,0xd8514a42,0xdeb34a42,0xd8514a44,0xdeb54a44,0xd8514a46,
	0xdeb54a46,0xd8514a48,0xdeb74a48,0xd8514a4a,0xdeb74a4a,0xd8514a4c,0xdeb94a4c,0xd8514a4e,
	0xdeb94a4e,0xd8514a50,0xdebb4a50,0xd8514a52,0xdebb4a52,0xd8514a54,0xdebd4a54,0xd8514a56,
	0xdebd4a56,0xd8514a58,0xdebf4a58,0xd8514a5a,0xdebf4a5a,0xd8514a5c,0xdec14a5c,0xd8514a5e,
	0xdec14a5e,0xd8514a60,0xdec34a60,0xd8514a62,0xdec34a62,0xd8514a64,0xdec54a64,0xd8514a66,
	0xdec54a66,0xd8514a68,0xdec74a68,0xd8514a6a,0xdec74a6a,0xd8514a6c,0xdec94a6c,0xd8514a6e,
	0xdec94a6e,0xd8514a70,0xdecb4a70,0xd8514a72,0xdecb4a72,0xd8514a74,0xdecd4a74,0xd8514a76,
	0xdecd4a76,0xd8514a78,0xdecf4a78,0xd8514a7a,0xdecf4a7a,0xd8514a7c,0xded14a7c,0xd8514a7e,
	0xded14a7e,0xd8514a80,0xded34a80,0xd8514a82,0xded34a82,0xd8514a84,0xded54a84,0xd8514a86,
	0xded54a86,0xd8514a88,0xded74a88,0xd8514a8a,0xded74a8a,0xd8514a8c,0xded94a8c,0xd8514a8e,
	0xded94a8e,0xd8514a90,0xdedb4a90,0xd8514a92,0xdedb4a92,0xd8514a94,0xdedd4a94,0xd8514a96,
	0xdedd4a96,0xd8514a98,0xdedf4a98,0xd8514a9a,0xdedf4a9a,0xd8514a9c,0xdee14a9c,0xd8514a9e,
	0xdee14a9e,0xd8514aa0,0xdee34aa0,0xd8514aa2,0xdee34aa2,0xd8514aa4,0xdee54aa4,0xd8514aa6,
	0xdee54aa6,0xd8514aa8,0xdee74aa8,0xd8514aaa,0xdee74aaa,0xd8514aac,0xdee94aac,0xd8514aae,
	0xdee94aae,0xd8514ab0,0xdeeb4ab0,0xd8514ab2,0xdeeb4ab2,0xd8514ab4,0xdeed4ab4,0xd8514ab6,
	0xdeed4ab6,0xd8514ab8,0xdeef4ab8,0xd8514aba,0xdeef4aba,0xd8514abc,0xdef14abc,0xd8514abe,
	0xdef14abe,0xd8514ac0,0xdef34ac0,0xd8514ac2,0xdef34ac2,0xd8514ac4,0xdef54ac4,0xd8514ac6,
	0xdef54ac6,0xd8514ac8,0xdef74ac8,0xd8514aca,0xdef74aca,0xd8514acc,0xdef94acc,0xd8514ace,
	0xdef94ace,0xd8514ad0,0xdefb4ad0,0xd8514ad2,0xdefb4ad2,0xd8514ad4,0xdefd4ad4,0xd8514ad6,
	0xdefd4ad6,0xd8514ad8,0xdeff4ad8,0xd8514ada,0xdeff4ada,0xd8514adc,0xdf014adc,0xd8514ade,
	0xdf014ade,0xd8514ae0,0xdf034ae0,0xd8514ae2,0xdf034ae2,0xd8514ae4,0xdf054ae4,0xd8514ae6,
	0xdf054ae6,0xd8514ae8,0xdf074ae8,0xd8514aea,0xdf074aea,0xd8514aec,0xdf094aec,0xd8514aee,
	0xdf094aee,0xd8514af0,0xdf0b4af0,0xd8514af2,0xdf0b4af2,0xd8514af4,0xdf0d4af4,0xd8514af6,
	0xdf0d4af6,0xd8514af8,0xdf0f4af8,0xd8514afa,0xdf0f4afa,0xd8514afc,0xdf114afc,0xd8514afe,
	0xdf114afe,0xd8514b00,0xdf134b00,0xd8514b02,0xdf134b02,0xd8514b04,0xdf154b04,0xd8514b06,
	0xdf154b06,0xd8514b08,0xdf174b08,0xd8514b0a,0xdf174b0a,0xd8514b0c,0xdf194b0c,0xd8514b0e,
	0xdf194b0e,0xd8514b10,0xdf1b4b10,0xd8514b12,0xdf1b4b12,0xd8514b14,0xdf1d4b14,0xd8514b16,
	0xdf1d4b16,0xd8514b18,0xdf1f4b18,0xd8514b1a,0xdf1f4b1a,0xd8514b1c,0xdf214b1c,0xd8514b1e,
	0xdf214b1e,0xd8514b20,0xdf234b20,0xd8514b22,0xdf234b22,0xd8514b24,0xdf254b24,0xd8514b26,
	0xdf254b26,0xd8514b28,0xdf274b28,0xd8514b2a,0xdf274b2a,0xd8514b2c,0xdf294b2c,0xd8514b2e,
	0xdf294b2e,0xd8514b30,0xdf2b4b30,0xd8514b32,0xdf2b4b32,0xd8514b34,0xdf2d4b34,0xd8514b36,
	0xdf2d4b36,0xd8514b38,0xdf2f4b38,0xd8514b3a,0xdf2f4b3a,0xd8514b3c,0xdf314b3c,0xd8514b3e,
	0xdf314b3e,0xd8514b40,0xdf334b40,0xd8514b42,0xdf334b42,0xd8514b44,0xdf354b44,0xd8514b46,
	0xdf354b46,0xd8514b48,0xdf374b48,0xd8514b4a,0xdf374b4a,0xd8514b4c,0xdf394b4c,0xd8514b4e,
	0xdf394b4e,0xd8514b50,0xdf3b4b50,0xd8514b52,0xdf3b4b52,0xd8514b54,0xdf3d4b54,0xd8514b56,
	0xdf3d4b56,0xd8514b58,0xdf3f4b58,0xd8514b5a,0xdf3f4b5a,0xd8514b5c,0xdf414b5c,0xd8514b5e,
	0xdf414b5e,0xd8514b60,0xdf434b60,0xd8514b62,0xdf434b62,0xd8514b64,0xdf454b64,0xd8514b66,
	0xdf454b66,0xd8514b68,0xdf474b68,0xd8514b6a,0xdf474b6a,0xd8514b6c,0xdf494b6c,0xd8514b6e,
	0xdf494b6e,0xd8514b70,0xdf4b4b70,0xd8514b72,0xdf4b4b72,0xd8514b74,0xdf4d4b74,0xd8514b76,
	0xdf4d4b76,0xd8514b78,0xdf4f4b78,0xd8514b7a,0xdf4f4b7a,0xd8514b7c,0xdf514b7c,0xd8514b7e,
	0xdf514b7e,0xd8514b80,0xdf534b80,0xd8514b82,0xdf534b82,0xd8514b84,0xdf554b84,0xd8514b86,
	0xdf554b86,0xd8514b88,0xdf574b88,0xd8514b8a,0xdf574b8a,0xd8514b8c,0xdf594b8c,0xd8514b8e,
	0xdf594b8e,0xd8514b90,0xdf5b4b90,0xd8514b92,0xdf5b4b92,0xd8514b94,0xdf5d4b94,0xd8514b96,
	0xdf5d4b96,0xd8514b98,0xdf5f4b98,0xd8514b9a,0xdf5f4b9a,0xd8514b9c,0xdf614b9c,0xd8514b9e,
	0xdf614b9e,0xd8514ba0,0xdf634ba0,0xd8514ba2,0xdf634ba2,0xd8514ba4,0xdf654ba4,0xd8514ba6,
	0xdf654ba6,0xd8514ba8,0xdf674ba8,0xd8514baa,0xdf674baa,0xd8514bac,0xdf694bac,0xd8514bae,
	0xdf694bae,0xd8514bb0,0xdf6b4bb0,0xd8514bb2,0xdf6b4bb2,0xd8514bb4,0xdf6d4bb4,0xd8514bb6,
	0xdf6d4bb6,0xd8514bb8,0xdf6f4bb8,0xd8514bba,0xdf6f4bba,0xd8514bbc,0xdf714bbc,0xd8514bbe,
	0xdf714bbe,0xd8514bc0,0xdf734bc0,0xd8514bc2,0xdf734bc2,0xd8514bc4,0xdf754bc4,0xd8514bc6,
	0xdf754bc6,0xd8514bc8,0xdf774bc8,0xd8514bca,0xdf774bca,0xd8514bcc,0xdf794bcc,0xd8514bce,
	0xdf794bce,0xd8514bd0,0xdf7b4bd0,0xd8514bd2,0xdf7b4bd2,0xd8514bd4,0xdf7d4bd4,0xd8514bd6,
	0xdf7d4bd6,0xd8514bd8,0xdf7f4bd8,0xd8514bda,0xdf7f4bda,0xd8514bdc,0xdf814bdc,0xd8514bde,
	0xdf814bde,0xd8514be0,0xdf834be0,0xd8514be2,0xdf834be2,0xd8514be4,0xdf854be4,0xd8514be6,
	0xdf854be6,0xd8514be8,0xdf874be8,0xd8514bea,0xdf874bea,0xd8514bec,0xdf894bec,0xd8514bee,
	0xdf894bee,0xd8514bf0,0xdf8b4bf0,0xd8514bf2,0xdf8b4bf2,0xd8514bf4,0xdf8d4bf4,0xd8514bf6,
	0xdf8d4bf6,0xd8514bf8,0xdf8f4bf8,0xd8514bfa,0xdf8f4bfa,0xd8514bfc,0xdf914bfc,0xd8514bfe,
	0xdf914bfe,0xd8514c00,0xdf934c00,0xd8514c02,0xdf934c02,0xd8514c04,0xdf954c04,0xd8514c06,
	0xdf954c06,0xd8514c08,0xdf974c08,0xd8514c0a,0xdf974c0a,0xd8514c0c,0xdf994c0c,0xd8514c0e,
	0xdf994c0e,0xd8514c10,0xdf9b4c10,0xd8514c12,0xdf9b4c12,0xd8514c14,0xdf9d4c14,0xd8514c16,
	0xdf9d4c16,0xd8514c18,0xdf9f4c18,0xd8514c1a,0xdf9f4c1a,0xd8514c1c,0xdfa14c1c,0xd8514c1e,
	0xdfa14c1e,0xd8514c20,0xdfa34c20,0xd8514c22,0xdfa34c22,0xd8514c24,0xdfa54c24,0xd8514c26,
	0xdfa54c26,0xd8514c28,0xdfa74c28,0xd8514c2a,0xdfa74c2a,0xd8514c2c,0xdfa94c2c,0xd8514c2e,
	0xdfa94c2e,0xd8514c30,0xdfab4c30,0xd8514c32,0xdfab4c32,0xd8514c34,0xdfad4c34,0xd8514c36,
	0xdfad4c36,0xd8514c38,0xdfaf4c38,0xd8514c3a,0xdfaf4c3a,0xd8514c3c,0xdfb14c3c,0xd8514c3e,
	0xdfb14c3e,0xd8514c40,0xdfb34c40,0xd8514c42,0xdfb34c42,0xd8514c44,0xdfb54c44,0xd8514c46,
	0xdfb54c46,0xd8514c48,0xdfb74c48,0xd8514c4a,0xdfb74c4a,0xd8514c4c,0xdfb94c4c,0xd8514c4e,
	0xdfb94c4e,0xd8514c50,0xdfbb4c50,0xd8514c52,0xdfbb4c52,0xd8514c54,0xdfbd4c54,0xd8514c56,
	0xdfbd4c56,0xd8514c58,0xdfbf4c58,0xd8514c5a,0xdfbf4c5a,0xd8514c5c,0xdfc14c5c,0xd8514c5e,
	0xdfc14c5e,0xd8514c60,0xdfc34c60,0xd8514c62,0xdfc34c62,0xd8514c64,0xdfc54c64,0xd8514c66,
	0xdfc54c66,0xd8514c68,0xdfc74c68,0xd8514c6a,0xdfc74c6a,0xd8514c6c,0xdfc94c6c,0xd8514c6e,
	0xdfc94c6e,0xd8514c70,0xdfcb4c70,0xd8514c72,0xdfcb4c72,0xd8514c74,0xdfcd4c74,0xd8514c76,
	0xdfcd4c76,0xd8514c78,0xdfcf4c78,0xd8514c7a,0xdfcf4c7a,0xd8514c7c,0xdfd14c7c,0xd8514c7e,
	0xdfd14c7e,0xd8514c80,0xdfd34c80,0xd8514c82,0xdfd34c82,0xd8514c84,0xdfd54c84,0xd8514c86,
	0xdfd54c86,0xd8514c88,0xdfd74c88,0xd8514c8a,0xdfd74c8a,0xd8514c8c,0xdfd94c8c,0xd8514c8e,
	0xdfd94c8e,0xd8514c90,0xdfdb4c90,0xd8514c92,0xdfdb4c92,0xd8514c94,0xdfdd4c94,0xd8514c96,
	0xdfdd4c96,0xd8514c98,0xdfdf4c98,0xd8514c9a,0xdfdf4c9a,0xd8514c9c,0xdfe14c9c,0xd8514c9e,
	0xdfe14c9e,0xd8514ca0,0xdfe34ca0,0xd8514ca2,0xdfe34ca2,0xd8514ca4,0xdfe54ca4,0xd8514ca6,
	0xdfe54ca6,0xd8514ca8,0xdfe74ca8,0xd8514caa,0xdfe74caa,0xd8514cac,0xdfe94cac,0xd8514cae,
	0xdfe94cae,0xd8514cb0,0xdfeb4cb0,0xd8514cb2,0xdfeb4cb2,0xd8514cb4,0xdfed4cb4,0xd8514cb6,
	0xdfed4cb6,0xd8514cb8,0xdfef4cb8,0xd8514cba,0xdfef4cba,0xd8514cbc,0xdff14cbc,0xd8514cbe,
	0xdff14cbe,0xd8514cc0,0xdff34cc0,0xd8514cc2,0xdff34cc2,0xd8514cc4,0xdff54cc4,0xd8514cc6,
	0xdff54cc6,0xd8514cc8,0xdff74cc8,0xd8514cca,0xdff74cca,0xd8514ccc,0xdff94ccc,0xd8514cce,
	0xdff94cce,0xd8514cd0,0xdffb4cd0,0xd8514cd2,0xdffb4cd2,0xd8514cd4,0xdffd4cd4,0xd8514cd6,
	0xdffd4cd6,0xd8514cd8,0xdfff4cd8,0xd8514cda,0xdfff4cda,0xd8534cdc,0xdc014cdc,0xd8534cde,
	0xdc014cde,0xd8534ce0,0xdc034ce0,0xd8534ce2,0xdc034ce2,0xd8534ce4,0xdc054ce4,0xd8534ce6,
	0xdc054ce6,0xd8534ce8,0xdc074ce8,0xd8534cea,0xdc074cea,0xd8534cec,0xdc094cec,0xd8534cee,
	0xdc094cee,0xd8534cf0,0xdc0b4cf0,0xd8534cf2,0xdc0b4cf2,0xd8534cf4,0xdc0d4cf4,0xd8534cf6,
	0xdc0d4cf6,0xd8534cf8,0xdc0f4cf8,0xd8534cfa,0xdc0f4cfa,0xd8534cfc,0xdc114cfc,0xd8534cfe,
	0xdc114cfe,0xd8534d00,0xdc134d00,0xd8534d02,0xdc134d02,0xd8534d04,0xdc154d04,0xd8534d06,
	0xdc154d06,0xd8534d08,0xdc174d08,0xd8534d0a,0xdc174d0a,0xd8534d0c,0xdc194d0c,0xd8534d0e,
	0xdc194d0e,0xd8534d10,0xdc1b4d10,0xd8534d12,0xdc1b4d12,0xd8534d14,0xdc1d4d14,0xd8534d16,
	0xdc1d4d16,0xd8534d18,0xdc1f4d18,0xd8534d1a,0xdc1f4d1a,0xd8534d1c,0xdc214d1c,0xd8534d1e,
	0xdc214d1e,0xd8534d20,0xdc234d20,0xd8534d22,0xdc234d22,0xd8534d24,0xdc254d24,0xd8534d26,
	0xdc254d26,0xd8534d28,0xdc274d28,0xd8534d2a,0xdc274d2a,0xd8534d2c,0xdc294d2c,0xd8534d2e,
	0xdc294d2e,0xd8534d30,0xdc2b4d30,0xd8534d32,0xdc2b4d32,0xd8534d34,0xdc2d4d34,0xd8534d36,
	0xdc2d4d36,0xd8534d38,0xdc2f4d38,0xd8534d3a,0xdc2f4d3a,0xd8534d3c,0xdc314d3c,0xd8534d3e,
	0xdc314d3e,0xd8534d40,0xdc334d40,0xd8534d42,0xdc334d42,0xd8534d44,0xdc354d44,0xd8534d46,
	0xdc354d46,0xd8534d48,0xdc374d48,0xd8534d4a,0xdc374d4a,0xd8534d4c,0xdc394d4c,0xd8534d4e,
	0xdc394d4e,0xd8534d50,0xdc3b4d50,0xd8534d52,0xdc3b4d52,0xd8534d54,0xdc3d4d54,0xd8534d56,
	0xdc3d4d56,0xd8534d58,0xdc3f4d58,0xd8534d5a,0xdc3f4d5a,0xd8534d5c,0xdc414d5c,0xd8534d5e,
	0xdc414d5e,0xd8534d60,0xdc434d60,0xd8534d62,0xdc434d62,0xd8534d64,0xdc454d64,0xd8534d66,
	0xdc454d66,0xd8534d68,0xdc474d68,0xd8534d6a,0xdc474d6a,0xd8534d6c,0xdc494d6c,0xd8534d6e,
	0xdc494d6e,0xd8534d70,0xdc4b4d70,0xd8534d72,0xdc4b4d72,0xd8534d74,0xdc4d4d74,0xd8534d76,
	0xdc4d4d76,0xd8534d78,0xdc4f4d78,0xd8534d7a,0xdc4f4d7a,0xd8534d7c,0xdc514d7c,0xd8534d7e,
	0xdc514d7e,0xd8534d80,0xdc534d80,0xd8534d82,0xdc534d82,0xd8534d84,0xdc554d84,0xd8534d86,
	0xdc554d86,0xd8534d88,0xdc574d88,0xd8534d8a,0xdc574d8a,0xd8534d8c,0xdc594d8c,0xd8534d8e,
	0xdc594d8e,0xd8534d90,0xdc5b4d90,0xd8534d92,0xdc5b4d92,0xd8534d94,0xdc5d4d94,0xd8534d96,
	0xdc5d4d96,0xd8534d98,0xdc5f4d98,0xd8534d9a,0xdc5f4d9a,0xd8534d9c,0xdc614d9c,0xd8534d9e,
	0xdc614d9e,0xd8534da0,0xdc634da0,0xd8534da2,0xdc634da2,0xd8534da4,0xdc654da4,0xd8534da6,
	0xdc654da6,0xd8534da8,0xdc674da8,0xd8534daa,0xdc674daa,0xd8534dac,0xdc694dac,0xd8534dae,
	0xdc694dae,0xd8534db0,0xdc6b4db0,0xd8534db2,0xdc6b4db2,0xd8534db4,0xdc6d4db4,0xd8534db6,
	0xdc6d4db6,0xd8534db8,0xdc6f4db8,0xd8534dba,0xdc6f4dba,0xd8534dbc,0xdc714dbc,0xd8534dbe,
	0xdc714dbe,0xd8534dc0,0xdc734dc0,0xd8534dc2,0xdc734dc2,0xd8534dc4,0xdc754dc4,0xd8534dc6,
	0xdc754dc6,0xd8534dc8,0xdc774dc8,0xd8534dca,0xdc774dca,0xd8534dcc,0xdc794dcc,0xd8534dce,
	0xdc794dce,0xd8534dd0,0xdc7b4dd0,0xd8534dd2,0xdc7b4dd2,0xd8534dd4,0xdc7d4dd4,0xd8534dd6,
	0xdc7d4dd6,0xd8534dd8,0xdc7f4dd8,0xd8534dda,0xdc7f4dda,0xd8534ddc,0xdc814ddc,0xd8534dde,
	0xdc814dde,0xd8534de0,0xdc834de0,0xd8534de2,0xdc834de2,0xd8534de4,0xdc854de4,0xd8534de6,
	0xdc854de6,0xd8534de8,0xdc874de8,0xd8534dea,0xdc874dea,0xd8534dec,0xdc894dec,0xd8534dee,
	0xdc894dee,0xd8534df0,0xdc8b4df0,0xd8534df2,0xdc8b4df2,0xd8534df4,0xdc8d4df4,0xd8534df6,
	0xdc8d4df6,0xd8534df8,0xdc8f4df8,0xd8534dfa,0xdc8f4dfa,0xd8534dfc,0xdc914dfc,0xd8534dfe,
	0xdc914dfe,0xd8534e00,0xdc934e00,0xd8534e02,0xdc934e02,0xd8534e04,0xdc954e04,0xd8534e06,
	0xdc954e06,0xd8534e08,0xdc974e08,0xd8534e0a,0xdc974e0a,0xd8534e0c,0xdc994e0c,0xd8534e0e,
	0xdc994e0e,0xd8534e10,0xdc9b4e10,0xd8534e12,0xdc9b4e12,0xd8534e14,0xdc9d4e14,0xd8534e16,
	0xdc9d4e16,0xd8534e18,0xdc9f4e18,0xd8534e1a,0xdc9f4e1a,0xd8534e1c,0xdca14e1c,0xd8534e1e,
	0xdca14e1e,0xd8534e20,0xdca34e20,0xd8534e22,0xdca34e22,0xd8534e24,0xdca54e24,0xd8534e26,
	0xdca54e26,0xd8534e28,0xdca74e28,0xd8534e2a,0xdca74e2a,0xd8534e2c,0xdca94e2c,0xd8534e2e,
	0xdca94e2e,0xd8534e30,0xdcab4e30,0xd8534e32,0xdcab4e32,0xd8534e34,0xdcad4e34,0xd8534e36,
	0xdcad4e36,0xd8534e38,0xdcaf4e38,0xd8534e3a,0xdcaf4e3a,0xd8534e3c,0xdcb14e3c,0xd8534e3e,
	0xdcb14e3e,0xd8534e40,0xdcb34e40,0xd8534e42,0xdcb34e42,0xd8534e44,0xdcb54e44,0xd8534e46,
	0xdcb54e46,0xd8534e48,0xdcb74e48,0xd8534e4a,0xdcb74e4a,0xd8534e4c,0xdcb94e4c,0xd8534e4e,
	0xdcb94e4e,0xd8534e50,0xdcbb4e50,0xd8534e52,0xdcbb4e52,0xd8534e54,0xdcbd4e54,0xd8534e56,
	0xdcbd4e56,0xd8534e58,0xdcbf4e58,0xd8534e5a,0xdcbf4e5a,0xd8534e5c,0xdcc14e5c,0xd8534e5e,
	0xdcc14e5e,0xd8534e60,0xdcc34e60,0xd8534e62,0xdcc34e62,0xd8534e64,0xdcc54e64,0xd8534e66,
	0xdcc54e66,0xd8534e68,0xdcc74e68,0xd8534e6a,0xdcc74e6a,0xd8534e6c,0xdcc94e6c,0xd8534e6e,
	0xdcc94e6e,0xd8534e70,0xdccb4e70,0xd8534e72,0xdccb4e72,0xd8534e74,0xdccd4e74,0xd8534e76,
	0xdccd4e76,0xd8534e78,0xdccf4e78,0xd8534e7a,0xdccf4e7a,0xd8534e7c,0xdcd14e7c,0xd8534e7e,
	0xdcd14e7e,0xd8534e80,0xdcd34e80,0xd8534e82,0xdcd34e82,0xd8534e84,0xdcd54e84,0xd8534e86,
	0xdcd54e86,0xd8534e88,0xdcd74e88,0xd8534e8a,0xdcd74e8a,0xd8534e8c,0xdcd94e8c,0xd8534e8e,
	0xdcd94e8e,0xd8534e90,0xdcdb4e90,0xd8534e92,0xdcdb4e92,0xd8534e94,0xdcdd4e94,0xd8534e96,
	0xdcdd4e96,0xd8534e98,0xdcdf4e98,0xd8534e9a,0xdcdf4e9a,0xd8534e9c,0xdce14e9c,0xd8534e9e,
	0xdce14e9e,0xd8534ea0,0xdce34ea0,0xd8534ea2,0xdce34ea2,0xd8534ea4,0xdce54ea4,0xd8534ea6,
	0xdce54ea6,0xd8534ea8,0xdce74ea8,0xd8534eaa,0xdce74eaa,0xd8534eac,0xdce94eac,0xd8534eae,
	0xdce94eae,0xd8534eb0,0xdceb4eb0,0xd8534eb2,0xdceb4eb2,0xd8534eb4,0xdced4eb4,0xd8534eb6,
	0xdced4eb6,0xd8534eb8,0xdcef4eb8,0xd8534eba,0xdcef4eba,0xd8534ebc,0xdcf14ebc,0xd8534ebe,
	0xdcf14ebe,0xd8534ec0,0xdcf34ec0,0xd8534ec2,0xdcf34ec2,0xd8534ec4,0xdcf54ec4,0xd8534ec6,
	0xdcf54ec6,0xd8534ec8,0xdcf74ec8,0xd8534eca,0xdcf74eca,0xd8534ecc,0xdcf94ecc,0xd8534ece,
	0xdcf94ece,0xd8534ed0,0xdcfb4ed0,0xd8534ed2,0xdcfb4ed2,0xd8534ed4,0xdcfd4ed4,0xd8534ed6,
	0xdcfd4ed6,0xd8534ed8,0xdcff4ed8,0xd8534eda,0xdcff4eda,0xd8534edc,0xdd014edc,0xd8534ede,
	0xdd014ede,0xd8534ee0,0xdd034ee0,0xd8534ee2,0xdd034ee2,0xd8534ee4,0xdd054ee4,0xd8534ee6,
	0xdd054ee6,0xd8534ee8,0xdd074ee8,0xd8534eea,0xdd074eea,0xd8534eec,0xdd094eec,0xd8534eee,
	0xdd094eee,0xd8534ef0,0xdd0b4ef0,0xd8534ef2,0xdd0b4ef2,0xd8534ef4,0xdd0d4ef4,0xd8534ef6,
	0xdd0d4ef6,0xd8534ef8,0xdd0f4ef8,0xd8534efa,0xdd0f4efa,0xd8534efc,0xdd114efc,0xd8534efe,
	0xdd114efe,0xd8534f00,0xdd134f00,0xd8534f02,0xdd134f02,0xd8534f04,0xdd154f04,0xd8534f06,
	0xdd154f06,0xd8534f08,0xdd174f08,0xd8534f0a,0xdd174f0a,0xd8534f0c,0xdd194f0c,0xd8534f0e,
	0xdd194f0e,0xd8534f10,0xdd1b4f10,0xd8534f12,0xdd1b4f12,0xd8534f14,0xdd1d4f14,0xd8534f16,
	0xdd1d4f16,0xd8534f18,0xdd1f4f18,0xd8534f1a,0xdd1f4f1a,0xd8534f1c,0xdd214f1c,0xd8534f1e,
	0xdd214f1e,0xd8534f20,0xdd234f20,0xd8534f22,0xdd234f22,0xd8534f24,0xdd254f24,0xd8534f26,
	0xdd254f26,0xd8534f28,0xdd274f28,0xd8534f2a,0xdd274f2a,0xd8534f2c,0xdd294f2c,0xd8534f2e,
	0xdd294f2e,0xd8534f30,0xdd2b4f30,0xd8534f32,0xdd2b4f32,0xd8534f34,0xdd2d4f34,0xd8534f36,
	0xdd2d4f36,0xd8534f38,0xdd2f4f38,0xd8534f3a,0xdd2f4f3a,0xd8534f3c,0xdd314f3c,0xd8534f3e,
	0xdd314f3e,0xd8534f40,0xdd334f40,0xd8534f42,0xdd334f42,0xd8534f44,0xdd354f44,0xd8534f46,
	0xdd354f46,0xd8534f48,0xdd374f48,0xd8534f4a,0xdd374f4a,0xd8534f4c,0xdd394f4c,0xd8534f4e,
	0xdd394f4e,0xd8534f50,0xdd3b4f50,0xd8534f52,0xdd3b4f52,0xd8534f54,0xdd3d4f54,0xd8534f56,
	0xdd3d4f56,0xd8534f58,0xdd3f4f58,0xd8534f5a,0xdd3f4f5a,0xd8534f5c,0xdd414f5c,0xd8534f5e,
	0xdd414f5e,0xd8534f60,0xdd434f60,0xd8534f62,0xdd434f62,0xd8534f64,0xdd454f64,0xd8534f66,
	0xdd454f66,0xd8534f68,0xdd474f68,0xd8534f6a,0xdd474f6a,0xd8534f6c,0xdd494f6c,0xd8534f6e,
	0xdd494f6e,0xd8534f70,0xdd4b4f70,0xd8534f72,0xdd4b4f72,0xd8534f74,0xdd4d4f74,0xd8534f76,
	0xdd4d4f76,0xd8534f78,0xdd4f4f78,0xd8534f7a,0xdd4f4f7a,0xd8534f7c,0xdd514f7c,0xd8534f7e,
	0xdd514f7e,0xd8534f80,0xdd534f80,0xd8534f82,0xdd534f82,0xd8534f84,0xdd554f84,0xd8534f86,
	0xdd554f86,0xd8534f88,0xdd574f88,0xd8534f8a,0xdd574f8a,0xd8534f8c,0xdd594f8c,0xd8534f8e,
	0xdd594f8e,0xd8534f90,0xdd5b4f90,0xd8534f92,0xdd5b4f92,0xd8534f94,0xdd5d4f94,0xd8534f96,
	0xdd5d4f96,0xd8534f98,0xdd5f4f98,0xd8534f9a,0xdd5f4f9a,0xd8534f9c,0xdd614f9c,0xd8534f9e,
	0xdd614f9e,0xd8534fa0,0xdd634fa0,0xd8534fa2,0xdd634fa2,0xd8534fa4,0xdd654fa4,0xd8534fa6,
	0xdd654fa6,0xd8534fa8,0xdd674fa8,0xd8534faa,0xdd674faa,0xd8534fac,0xdd694fac,0xd8534fae,
	0xdd694fae,0xd8534fb0,0xdd6b4fb0,0xd8534fb2,0xdd6b4fb2,0xd8534fb4,0xdd6d4fb4,0xd8534fb6,
	0xdd6d4fb6,0xd8534fb8,0xdd6f4fb8,0xd8534fba,0xdd6f4fba,0xd8534fbc,0xdd714fbc,0xd8534fbe,
	0xdd714fbe,0xd8534fc0,0xdd734fc0,0xd8534fc2,0xdd734fc2,0xd8534fc4,0xdd754fc4,0xd8534fc6,
	0xdd754fc6,0xd8534fc8,0xdd774fc8,0xd8534fca,0xdd774fca,0xd8534fcc,0xdd794fcc,0xd8534fce,
	0xdd794fce,0xd8534fd0,0xdd7b4fd0,0xd8534fd2,0xdd7b4fd2,0xd8534fd4,0xdd7d4fd4,0xd8534fd6,
	0xdd7d4fd6,0xd8534fd8,0xdd7f4fd8,0xd8534fda,0xdd7f4fda,0xd8534fdc,0xdd814fdc,0xd8534fde,
	0xdd814fde,0xd8534fe0,0xdd834fe0,0xd8534fe2,0xdd834fe2,0xd8534fe4,0xdd854fe4,0xd8534fe6,
	0xdd854fe6,0xd8534fe8,0xdd874fe8,0xd8534fea,0xdd874fea,0xd8534fec,0xdd894fec,0xd8534fee,
	0xdd894fee,0xd8534ff0,0xdd8b4ff0,0xd8534ff2,0xdd8b4ff2,0xd8534ff4,0xdd8d4ff4,0xd8534ff6,
	0xdd8d4ff6,0xd8534ff8,0xdd8f4ff8,0xd8534ffa,0xdd8f4ffa,0xd8534ffc,0xdd914ffc,0xd8534ffe,
	0xdd914ffe,0xd8535000,0xdd935000,0xd8535002,0xdd935002,0xd8535004,0xdd955004,0xd8535006,
	0xdd955006,0xd8535008,0xdd975008,0xd853500a,0xdd97500a,0xd853500c,0xdd99500c,0xd853500e,
	0xdd99500e,0xd8535010,0xdd9b5010,0xd8535012,0xdd9b5012,0xd8535014,0xdd9d5014,0xd8535016,
	0xdd9d5016,0xd8535018,0xdd9f5018,0xd853501a,0xdd9f501a,0xd853501c,0xdda1501c,0xd853501e,
	0xdda1501e,0xd8535020,0xdda35020,0xd8535022,0xdda35022,0xd8535024,0xdda55024,0xd8535026,
	0xdda55026,0xd8535028,0xdda75028,0xd853502a,0xdda7502a,0xd853502c,0xdda9502c,0xd853502e,
	0xdda9502e,0xd8535030,0xddab5030,0xd8535032,0xddab5032,0xd8535034,0xddad5034,0xd8535036,
	0xddad5036,0xd8535038,0xddaf5038,0xd853503a,0xddaf503a,0xd853503c,0xddb1503c,0xd853503e,
	0xddb1503e,0xd8535040,0xddb35040,0xd8535042,0xddb35042,0xd8535044,0xddb55044,0xd8535046,
	0xddb55046,0xd8535048,0xddb75048,0xd853504a,0xddb7504a,0xd853504c,0xddb9504c,0xd853504e,
	0xddb9504e,0xd8535050,0xddbb5050,0xd8535052,0xddbb5052,0xd8535054,0xddbd5054,0xd8535056,
	0xddbd5056,0xd8535058,0xddbf5058,0xd853505a,0xddbf505a,0xd853505c,0xddc1505c,0xd853505e,
	0xddc1505e,0xd8535060,0xddc35060,0xd8535062,0xddc35062,0xd8535064,0xddc55064,0xd8535066,
	0xddc55066,0xd8535068,0xddc75068,0xd853506a,0xddc7506a,0xd853506c,0xddc9506c,0xd853506e,
	0xddc9506e,0xd8535070,0xddcb5070,0xd8535072,0xddcb5072,0xd8535074,0xddcd5074,0xd8535076,
	0xddcd5076,0xd8535078,0xddcf5078,0xd853507a,0xddcf507a,0xd853507c,0xddd1507c,0xd853507e,
	0xddd1507e,0xd8535080,0xddd35080,0xd8535082,0xddd35082,0xd8535084,0xddd55084,0xd8535086,
	0xddd55086,0xd8535088,0xddd75088,0xd853508a,0xddd7508a,0xd853508c,0xddd9508c,0xd853508e,
	0xddd9508e,0xd8535090,0xdddb5090,0xd8535092,0xdddb5092,0xd8535094,0xdddd5094,0xd8535096,
	0xdddd5096,0xd8535098,0xdddf5098,0xd853509a,0xdddf509a,0xd853509c,0xdde1509c,0xd853509e,
	0xdde1509e,0xd85350a0,0xdde350a0,0xd85350a2,0xdde350a2,0xd85350a4,0xdde550a4,0xd85350a6,
	0xdde550a6,0xd85350a8,0xdde750a8,0xd85350aa,0xdde750aa,0xd85350ac,0xdde950ac,0xd85350ae,
	0xdde950ae,0xd85350b0,0xddeb50b0,0xd85350b2,0xddeb50b2,0xd85350b4,0xdded50b4,0xd85350b6,
	0xdded50b6,0xd85350b8,0xddef50b8,0xd85350ba,0xddef50ba,0xd85350bc,0xddf150bc,0xd85350be,
	0xddf150be,0xd85350c0,0xddf350c0,0xd85350c2,0xddf350c2,0xd85350c4,0xddf550c4,0xd85350c6,
	0xddf550c6,0xd85350c8,0xddf750c8,0xd85350ca,0xddf750ca,0xd85350cc,0xddf950cc,0xd85350ce,
	0xddf950ce,0xd85350d0,0xddfb50d0,0xd85350d2,0xddfb50d2,0xd85350d4,0xddfd50d4,0xd85350d6,
	0xddfd50d6,0xd85350d8,0xddff50d8,0xd85350da,0xddff50da,0xd85350dc,0xde0150dc,0xd85350de,
	0xde0150de,0xd85350e0,0xde0350e0,0xd85350e2,0xde0350e2,0xd85350e4,0xde0550e4,0xd85350e6,
	0xde0550e6,0xd85350e8,0xde0750e8,0xd85350ea,0xde0750ea,0xd85350ec,0xde0950ec,0xd85350ee,
	0xde0950ee,0xd85350f0,0xde0b50f0,0xd85350f2,0xde0b50f2,0xd85350f4,0xde0d50f4,0xd85350f6,
	0xde0d50f6,0xd85350f8,0xde0f50f8,0xd85350fa,0xde0f50fa,0xd85350fc,0xde1150fc,0xd85350fe,
	0xde1150fe,0xd8535100,0xde135100,0xd8535102,0xde135102,0xd8535104,0xde155104,0xd8535106,
	0xde155106,0xd8535108,0xde175108,0xd853510a,0xde17510a,0xd853510c,0xde19510c,0xd853510e,
	0xde19510e,0xd8535110,0xde1b5110,0xd8535112,0xde1b5112,0xd8535114,0xde1d5114,0xd8535116,
	0xde1d5116,0xd8535118,0xde1f5118,0xd853511a,0xde1f511a,0xd853511c,0xde21511c,0xd853511e,
	0xde21511e,0xd8535120,0xde235120,0xd8535122,0xde235122,0xd8535124,0xde255124,0xd8535126,
	0xde255126,0xd8535128,0xde275128,0xd853512a,0xde27512a,0xd853512c,0xde29512c,0xd853512e,
	0xde29512e,0xd8535130,0xde2b5130,0xd8535132,0xde2b5132,0xd8535134,0xde2d5134,0xd8535136,
	0xde2d5136,0xd8535138,0xde2f5138,0xd853513a,0xde2f513a,0xd853513c,0xde31513c,0xd853513e,
	0xde31513e,0xd8535140,0xde335140,0xd8535142,0xde335142,0xd8535144,0xde355144,0xd8535146,
	0xde355146,0xd8535148,0xde375148,0xd853514a,0xde37514a,0xd853514c,0xde39514c,0xd853514e,
	0xde39514e,0xd8535150,0xde3b5150,0xd8535152,0xde3b5152,0xd8535154,0xde3d5154,0xd8535156,
	0xde3d5156,0xd8535158,0xde3f5158,0xd853515a,0xde3f515a,0xd853515c,0xde41515c,0xd853515e,
	0xde41515e,0xd8535160,0xde435160,0xd8535162,0xde435162,0xd8535164,0xde455164,0xd8535166,
	0xde455166,0xd8535168,0xde475168,0xd853516a,0xde47516a,0xd853516c,0xde49516c,0xd853516e,
	0xde49516e,0xd8535170,0xde4b5170,0xd8535172,0xde4b5172,0xd8535174,0xde4d5174,0xd8535176,
	0xde4d5176,0xd8535178,0xde4f5178,0xd853517a,0xde4f517a,0xd853517c,0xde51517c,0xd853517e,
	0xde51517e,0xd8535180,0xde535180,0xd8535182,0xde535182,0xd8535184,0xde555184,0xd8535186,
	0xde555186,0xd8535188,0xde575188,0xd853518a,0xde57518a,0xd853518c,0xde59518c,0xd853518e,
	0xde59518e,0xd8535190,0xde5b5190,0xd8535192,0xde5b5192,0xd8535194,0xde5d5194,0xd8535196,
	0xde5d5196,0xd8535198,0xde5f5198,0xd853519a,0xde5f519a,0xd853519c,0xde61519c,0xd853519e,
	0xde61519e,0xd85351a0,0xde6351a0,0xd85351a2,0xde6351a2,0xd85351a4,0xde6551a4,0xd85351a6,
	0xde6551a6,0xd85351a8,0xde6751a8,0xd85351aa,0xde6751aa,0xd85351ac,0xde6951ac,0xd85351ae,
	0xde6951ae,0xd85351b0,0xde6b51b0,0xd85351b2,0xde6b51b2,0xd85351b4,0xde6d51b4,0xd85351b6,
	0xde6d51b6,0xd85351b8,0xde6f51b8,0xd85351ba,0xde6f51ba,0xd85351bc,0xde7151bc,0xd85351be,
	0xde7151be,0xd85351c0,0xde7351c0,0xd85351c2,0xde7351c2,0xd85351c4,0xde7551c4,0xd85351c6,
	0xde7551c6,0xd85351c8,0xde7751c8,0xd85351ca,0xde7751ca,0xd85351cc,0xde7951cc,0xd85351ce,
	0xde7951ce,0xd85351d0,0xde7b51d0,0xd85351d2,0xde7b51d2,0xd85351d4,0xde7d51d4,0xd85351d6,
	0xde7d51d6,0xd85351d8,0xde7f51d8,0xd85351da,0xde7f51da,0xd85351dc,0xde8151dc,0xd85351de,
	0xde8151de,0xd85351e0,0xde8351e0,0xd85351e2,0xde8351e2,0xd85351e4,0xde8551e4,0xd85351e6,
	0xde8551e6,0xd85351e8,0xde8751e8,0xd85351ea,0xde8751ea,0xd85351ec,0xde8951ec,0xd85351ee,
	0xde8951ee,0xd85351f0,0xde8b51f0,0xd85351f2,0xde8b51f2,0xd85351f4,0xde8d51f4,0xd85351f6,
	0xde8d51f6,0xd85351f8,0xde8f51f8,0xd85351fa,0xde8f51fa,0xd85351fc,0xde9151fc,0xd85351fe,
	0xde9151fe,0xd8535200,0xde935200,0xd8535202,0xde935202,0xd8535204,0xde955204,0xd8535206,
	0xde955206,0xd8535208,0xde975208,0xd853520a,0xde97520a,0xd853520c,0xde99520c,0xd853520e,
	0xde99520e,0xd8535210,0xde9b5210,0xd8535212,0xde9b5212,0xd8535214,0xde9d5214,0xd8535216,
	0xde9d5216,0xd8535218,0xde9f5218,0xd853521a,0xde9f521a,0xd853521c,0xdea1521c,0xd853521e,
	0xdea1521e,0xd8535220,0xdea35220,0xd8535222,0xdea35222,0xd8535224,0xdea55224,0xd8535226,
	0xdea55226,0xd8535228,0xdea75228,0xd853522a,0xdea7522a,0xd853522c,0xdea9522c,0xd853522e,
	0xdea9522e,0xd8535230,0xdeab5230,0xd8535232,0xdeab5232,0xd8535234,0xdead5234,0xd8535236,
	0xdead5236,0xd8535238,0xdeaf5238,0xd853523a,0xdeaf523a,0xd853523c,0xdeb1523c,0xd853523e,
	0xdeb1523e,0xd8535240,0xdeb35240,0xd8535242,0xdeb35242,0xd8535244,0xdeb55244,0xd8535246,
	0xdeb55246,0xd8535248,0xdeb75248,0xd853524a,0xdeb7524a,0xd853524c,0xdeb9524c,0xd853524e,
	0xdeb9524e,0xd8535250,0xdebb5250,0xd8535252,0xdebb5252,0xd8535254,0xdebd5254,0xd8535256,
	0xdebd5256,0xd8535258,0xdebf5258,0xd853525a,0xdebf525a,0xd853525c,0xdec1525c,0xd853525e,
	0xdec1525e,0xd8535260,0xdec35260,0xd8535262,0xdec35262,0xd8535264,0xdec55264,0xd8535266,
	0xdec55266,0xd8535268,0xdec75268,0xd853526a,0xdec7526a,0xd853526c,0xdec9526c,0xd853526e,
	0xdec9526e,0xd8535270,0xdecb5270,0xd8535272,0xdecb5272,0xd8535274,0xdecd5274,0xd8535276,
	0xdecd5276,0xd8535278,0xdecf5278,0xd853527a,0xdecf527a,0xd853527c,0xded1527c,0xd853527e,
	0xded1527e,0xd8535280,0xded35280,0xd8535282,0xded35282,0xd8535284,0xded55284,0xd8535286,
	0xded55286,0xd8535288,0xded75288,0xd853528a,0xded7528a,0xd853528c,0xded9528c,0xd853528e,
	0xded9528e,0xd8535290,0xdedb5290,0xd8535292,0xdedb5292,0xd8535294,0xdedd5294,0xd8535296,
	0xdedd5296,0xd8535298,0xdedf5298,0xd853529a,0xdedf529a,0xd853529c,0xdee1529c,0xd853529e,
	0xdee1529e,0xd85352a0,0xdee352a0,0xd85352a2,0xdee352a2,0xd85352a4,0xdee552a4,0xd85352a6,
	0xdee552a6,0xd85352a8,0xdee752a8,0xd85352aa,0xdee752aa,0xd85352ac,0xdee952ac,0xd85352ae,
	0xdee952ae,0xd85352b0,0xdeeb52b0,0xd85352b2,0xdeeb52b2,0xd85352b4,0xdeed52b4,0xd85352b6,
	0xdeed52b6,0xd85352b8,0xdeef52b8,0xd85352ba,0xdeef52ba,0xd85352bc,0xdef152bc,0xd85352be,
	0xdef152be,0xd85352c0,0xdef352c0,0xd85352c2,0xdef352c2,0xd85352c4,0xdef552c4,0xd85352c6,
	0xdef552c6,0xd85352c8,0xdef752c8,0xd85352ca,0xdef752ca,0xd85352cc,0xdef952cc,0xd85352ce,
	0xdef952ce,0xd85352d0,0xdefb52d0,0xd85352d2,0xdefb52d2,0xd85352d4,0xdefd52d4,0xd85352d6,
	0xdefd52d6,0xd85352d8,0xdeff52d8,0xd85352da,0xdeff52da,0xd85352dc,0xdf0152dc,0xd85352de,
	0xdf0152de,0xd85352e0,0xdf0352e0,0xd85352e2,0xdf0352e2,0xd85352e4,0xdf0552e4,0xd85352e6,
	0xdf0552e6,0xd85352e8,0xdf0752e8,0xd85352ea,0xdf0752ea,0xd85352ec,0xdf0952ec,0xd85352ee,
	0xdf0952ee,0xd85352f0,0xdf0b52f0,0xd85352f2,0xdf0b52f2,0xd85352f4,0xdf0d52f4,0xd85352f6,
	0xdf0d52f6,0xd85352f8,0xdf0f52f8,0xd85352fa,0xdf0f52fa,0xd85352fc,0xdf1152fc,0xd85352fe,
	0xdf1152fe,0xd8535300,0xdf135300,0xd8535302,0xdf135302,0xd8535304,0xdf155304,0xd8535306,
	0xdf155306,0xd8535308,0xdf175308,0xd853530a,0xdf17530a,0xd853530c,0xdf19530c,0xd853530e,
	0xdf19530e,0xd8535310,0xdf1b5310,0xd8535312,0xdf1b5312,0xd8535314,0xdf1d5314,0xd8535316,
	0xdf1d5316,0xd8535318,0xdf1f5318,0xd853531a,0xdf1f531a,0xd853531c,0xdf21531c,0xd853531e,
	0xdf21531e,0xd8535320,0xdf235320,0xd8535322,0xdf235322,0xd8535324,0xdf255324,0xd8535326,
	0xdf255326,0xd8535328,0xdf275328,0xd853532a,0xdf27532a,0xd853532c,0xdf29532c,0xd853532e,
	0xdf29532e,0xd8535330,0xdf2b5330,0xd8535332,0xdf2b5332,0xd8535334,0xdf2d5334,0xd8535336,
	0xdf2d5336,0xd8535338,0xdf2f5338,0xd853533a,0xdf2f533a,0xd853533c,0xdf31533c,0xd853533e,
	0xdf31533e,0xd8535340,0xdf335340,0xd8535342,0xdf335342,0xd8535344,0xdf355344,0xd8535346,
	0xdf355346,0xd8535348,0xdf375348,0xd853534a,0xdf37534a,0xd853534c,0xdf39534c,0xd853534e,
	0xdf39534e,0xd8535350,0xdf3b5350,0xd8535352,0xdf3b5352,0xd8535354,0xdf3d5354,0xd8535356,
	0xdf3d5356,0xd8535358,0xdf3f5358,0xd853535a,0xdf3f535a,0xd853535c,0xdf41535c,0xd853535e,
	0xdf41535e,0xd8535360,0xdf435360,0xd8535362,0xdf435362,0xd8535364,0xdf455364,0xd8535366,
	0xdf455366,0xd8535368,0xdf475368,0xd853536a,0xdf47536a,0xd853536c,0xdf49536c,0xd853536e,
	0xdf49536e,0xd8535370,0xdf4b5370,0xd8535372,0xdf4b5372,0xd8535374,0xdf4d5374,0xd8535376,
	0xdf4d5376,0xd8535378,0xdf4f5378,0xd853537a,0xdf4f537a,0xd853537c,0xdf51537c,0xd853537e,
	0xdf51537e,0xd8535380,0xdf535380,0xd8535382,0xdf535382,0xd8535384,0xdf555384,0xd8535386,
	0xdf555386,0xd8535388,0xdf575388,0xd853538a,0xdf57538a,0xd853538c,0xdf59538c,0xd853538e,
	0xdf59538e,0xd8535390,0xdf5b5390,0xd8535392,0xdf5b5392,0xd8535394,0xdf5d5394,0xd8535396,
	0xdf5d5396,0xd8535398,0xdf5f5398,0xd853539a,0xdf5f539a,0xd853539c,0xdf61539c,0xd853539e,
	0xdf61539e,0xd85353a0,0xdf6353a0,0xd85353a2,0xdf6353a2,0xd85353a4,0xdf6553a4,0xd85353a6,
	0xdf6553a6,0xd85353a8,0xdf6753a8,0xd85353aa,0xdf6753aa,0xd85353ac,0xdf6953ac,0xd85353ae,
	0xdf6953ae,0xd85353b0,0xdf6b53b0,0xd85353b2,0xdf6b53b2,0xd85353b4,0xdf6d53b4,0xd85353b6,
	0xdf6d53b6,0xd85353b8,0xdf6f53b8,0xd85353ba,0xdf6f53ba,0xd85353bc,0xdf7153bc,0xd85353be,
	0xdf7153be,0xd85353c0,0xdf7353c0,0xd85353c2,0xdf7353c2,0xd85353c4,0xdf7553c4,0xd85353c6,
	0xdf7553c6,0xd85353c8,0xdf7753c8,0xd85353ca,0xdf7753ca,0xd85353cc,0xdf7953cc,0xd85353ce,
	0xdf7953ce,0xd85353d0,0xdf7b53d0,0xd85353d2,0xdf7b53d2,0xd85353d4,0xdf7d53d4,0xd85353d6,
	0xdf7d53d6,0xd85353d8,0xdf7f53d8,0xd85353da,0xdf7f53da,0xd85353dc,0xdf8153dc,0xd85353de,
	0xdf8153de,0xd85353e0,0xdf8353e0,0xd85353e2,0xdf8353e2,0xd85353e4,0xdf8553e4,0xd85353e6,
	0xdf8553e6,0xd85353e8,0xdf8753e8,0xd85353ea,0xdf8753ea,0xd85353ec,0xdf8953ec,0xd85353ee,
	0xdf8953ee,0xd85353f0,0xdf8b53f0,0xd85353f2,0xdf8b53f2,0xd85353f4,0xdf8d53f4,0xd85353f6,
	0xdf8d53f6,0xd85353f8,0xdf8f53f8,0xd85353fa,0xdf8f53fa,0xd85353fc,0xdf9153fc,0xd85353fe,
	0xdf9153fe,0xd8535400,0xdf935400,0xd8535402,0xdf935402,0xd8535404,0xdf955404,0xd8535406,
	0xdf955406,0xd8535408,0xdf975408,0xd853540a,0xdf97540a,0xd853540c,0xdf99540c,0xd853540e,
	0xdf99540e,0xd8535410,0xdf9b5410,0xd8535412,0xdf9b5412,0xd8535414,0xdf9d5414,0xd8535416,
	0xdf9d5416,0xd8535418,0xdf9f5418,0xd853541a,0xdf9f541a,0xd853541c,0xdfa1541c,0xd853541e,
	0xdfa1541e,0xd8535420,0xdfa35420,0xd8535422,0xdfa35422,0xd8535424,0xdfa55424,0xd8535426,
	0xdfa55426,0xd8535428,0xdfa75428,0xd853542a,0xdfa7542a,0xd853542c,0xdfa9542c,0xd853542e,
	0xdfa9542e,0xd8535430,0xdfab5430,0xd8535432,0xdfab5432,0xd8535434,0xdfad5434,0xd8535436,
	0xdfad5436,0xd8535438,0xdfaf5438,0xd853543a,0xdfaf543a,0xd853543c,0xdfb1543c,0xd853543e,
	0xdfb1543e,0xd8535440,0xdfb35440,0xd8535442,0xdfb35442,0xd8535444,0xdfb55444,0xd8535446,
	0xdfb55446,0xd8535448,0xdfb75448,0xd853544a,0xdfb7544a,0xd853544c,0xdfb9544c,0xd853544e,
	0xdfb9544e,0xd8535450,0xdfbb5450,0xd8535452,0xdfbb5452,0xd8535454,0xdfbd5454,0xd8535456,
	0xdfbd5456,0xd8535458,0xdfbf5458,0xd853545a,0xdfbf545a,0xd853545c,0xdfc1545c,0xd853545e,
	0xdfc1545e,0xd8535460,0xdfc35460,0xd8535462,0xdfc35462,0xd8535464,0xdfc55464,0xd8535466,
	0xdfc55466,0xd8535468,0xdfc75468,0xd853546a,0xdfc7546a,0xd853546c,0xdfc9546c,0xd853546e,
	0xdfc9546e,0xd8535470,0xdfcb5470,0xd8535472,0xdfcb5472,0xd8535474,0xdfcd5474,0xd8535476,
	0xdfcd5476,0xd8535478,0xdfcf5478,0xd853547a,0xdfcf547a,0xd853547c,0xdfd1547c,0xd853547e,
	0xdfd1547e,0xd8535480,0xdfd35480,0xd8535482,0xdfd35482,0xd8535484,0xdfd55484,0xd8535486,
	0xdfd55486,0xd8535488,0xdfd75488,0xd853548a,0xdfd7548a,0xd853548c,0xdfd9548c,0xd853548e,
	0xdfd9548e,0xd8535490,0xdfdb5490,0xd8535492,0xdfdb5492,0xd8535494,0xdfdd5494,0xd8535496,
	0xdfdd5496,0xd8535498,0xdfdf5498,0xd853549a,0xdfdf549a,0xd853549c,0xdfe1549c,0xd853549e,
	0xdfe1549e,0xd85354a0,0xdfe354a0,0xd85354a2,0xdfe354a2,0xd85354a4,0xdfe554a4,0xd85354a6,
	0xdfe554a6,0xd85354a8,0xdfe754a8,0xd85354aa,0xdfe754aa,0xd85354ac,0xdfe954ac,0xd85354ae,
	0xdfe954ae,0xd85354b0,0xdfeb54b0,0xd85354b2,0xdfeb54b2,0xd85354b4,0xdfed54b4,0xd85354b6,
	0xdfed54b6,0xd85354b8,0xdfef54b8,0xd85354ba,0xdfef54ba,0xd85354bc,0xdff154bc,0xd85354be,
	0xdff154be,0xd85354c0,0xdff354c0,0xd85354c2,0xdff354c2,0xd85354c4,0xdff554c4,0xd85354c6,
	0xdff554c6,0xd85354c8,0xdff754c8,0xd85354ca,0xdff754ca,0xd85354cc,0xdff954cc,0xd85354ce,
	0xdff954ce,0xd85354d0,0xdffb54d0,0xd85354d2,0xdffb54d2,0xd85354d4,0xdffd54d4,0xd85354d6,
	0xdffd54d6,0xd85354d8,0xdfff54d8,0xd85354da,0xdfff54da,0xd85354dc,0xdc0154dc,0xd85354de,
	0xdc0154de,0xd85354e0,0xdc0354e0,0xd85354e2,0xdc0354e2,0xd85354e4,0xdc0554e4,0xd85354e6,
	0xdc0554e6,0xd85354e8,0xdc0754e8,0xd85354ea,0xdc0754ea,0xd85354ec,0xdc0954ec,0xd85354ee,
	0xdc0954ee,0xd85354f0,0xdc0b54f0,0xd85354f2,0xdc0b54f2,0xd85354f4,0xdc0d54f4,0xd85354f6,
	0xdc0d54f6,0xd85354f8,0xdc0f54f8,0xd85354fa,0xdc0f54fa,0xd85354fc,0xdc1154fc,0xd85354fe,
	0xdc1154fe,0xd8535500,0xdc135500,0xd8535502,0xdc135502,0xd8535504,0xdc155504,0xd8535506,
	0xdc155506,0xd8535508,0xdc175508,0xd853550a,0xdc17550a,0xd853550c,0xdc19550c,0xd853550e,
	0xdc19550e,0xd8535510,0xdc1b5510,0xd8535512,0xdc1b5512,0xd8535514,0xdc1d5514,0xd8535516,
	0xdc1d5516,0xd8535518,0xdc1f5518,0xd853551a,0xdc1f551a,0xd853551c,0xdc21551c,0xd853551e,
	0xdc21551e,0xd8535520,0xdc235520,0xd8535522,0xdc235522,0xd8535524,0xdc255524,0xd8535526,
	0xdc255526,0xd8535528,0xdc275528,0xd853552a,0xdc27552a,0xd853552c,0xdc29552c,0xd853552e,
	0xdc29552e,0xd8535530,0xdc2b5530,0xd8535532,0xdc2b5532,0xd8535534,0xdc2d5534,0xd8535536,
	0xdc2d5536,0xd8535538,0xdc2f5538,0xd853553a,0xdc2f553a,0xd853553c,0xdc31553c,0xd853553e,
	0xdc31553e,0xd8535540,0xdc335540,0xd8535542,0xdc335542,0xd8535544,0xdc355544,0xd8535546,
	0xdc355546,0xd8535548,0xdc375548,0xd853554a,0xdc37554a,0xd853554c,0xdc39554c,0xd853554e,
	0xdc39554e,0xd8535550,0xdc3b5550,0xd8535552,0xdc3b5552,0xd8535554,0xdc3d5554,0xd8535556,
	0xdc3d5556,0xd8535558,0xdc3f5558,0xd853555a,0xdc3f555a,0xd853555c,0xdc41555c,0xd853555e,
	0xdc41555e,0xd8535560,0xdc435560,0xd8535562,0xdc435562,0xd8535564,0xdc455564,0xd8535566,
	0xdc455566,0xd8535568,0xdc475568,0xd853556a,0xdc47556a,0xd853556c,0xdc49556c,0xd853556e,
	0xdc49556e,0xd8535570,0xdc4b5570,0xd8535572,0xdc4b5572,0xd8535574,0xdc4d5574,0xd8535576,
	0xdc4d5576,0xd8535578,0xdc4f5578,0xd853557a,0xdc4f557a,0xd853557c,0xdc51557c,0xd853557e,
	0xdc51557e,0xd8535580,0xdc535580,0xd8535582,0xdc535582,0xd8535584,0xdc555584,0xd8535586,
	0xdc555586,0xd8535588,0xdc575588,0xd853558a,0xdc57558a,0xd853558c,0xdc59558c,0xd853558e,
	0xdc59558e,0xd8535590,0xdc5b5590,0xd8535592,0xdc5b5592,0xd8535594,0xdc5d5594,0xd8535596,
	0xdc5d5596,0xd8535598,0xdc5f5598,0xd853559a,0xdc5f559a,0xd853559c,0xdc61559c,0xd853559e,
	0xdc61559e,0xd85355a0,0xdc6355a0,0xd85355a2,0xdc6355a2,0xd85355a4,0xdc6555a4,0xd85355a6,
	0xdc6555a6,0xd85355a8,0xdc6755a8,0xd85355aa,0xdc6755aa,0xd85355ac,0xdc6955ac,0xd85355ae,
	0xdc6955ae,0xd85355b0,0xdc6b55b0,0xd85355b2,0xdc6b55b2,0xd85355b4,0xdc6d55b4,0xd85355b6,
	0xdc6d55b6,0xd85355b8,0xdc6f55b8,0xd85355ba,0xdc6f55ba,0xd85355bc,0xdc7155bc,0xd85355be,
	0xdc7155be,0xd85355c0,0xdc7355c0,0xd85355c2,0xdc7355c2,0xd85355c4,0xdc7555c4,0xd85355c6,
	0xdc7555c6,0xd85355c8,0xdc7755c8,0xd85355ca,0xdc7755ca,0xd85355cc,0xdc7955cc,0xd85355ce,
	0xdc7955ce,0xd85355d0,0xdc7b55d0,0xd85355d2,0xdc7b55d2,0xd85355d4,0xdc7d55d4,0xd85355d6,
	0xdc7d55d6,0xd85355d8,0xdc7f55d8,0xd85355da,0xdc7f55da,0xd85355dc,0xdc8155dc,0xd85355de,
	0xdc8155de,0xd85355e0,0xdc8355e0,0xd85355e2,0xdc8355e2,0xd85355e4,0xdc8555e4,0xd85355e6,
	0xdc8555e6,0xd85355e8,0xdc8755e8,0xd85355ea,0xdc8755ea,0xd85355ec,0xdc8955ec,0xd85355ee,
	0xdc8955ee,0xd85355f0,0xdc8b55f0,0xd85355f2,0xdc8b55f2,0xd85355f4,0xdc8d55f4,0xd85355f6,
	0xdc8d55f6,0xd85355f8,0xdc8f55f8,0xd85355fa,0xdc8f55fa,0xd85355fc,0xdc9155fc,0xd85355fe,
	0xdc9155fe,0xd8535600,0xdc935600,0xd8535602,0xdc935602,0xd8535604,0xdc955604,0xd8535606,
	0xdc955606,0xd8535608,0xdc975608,0xd853560a,0xdc97560a,0xd853560c,0xdc99560c,0xd853560e,
	0xdc99560e,0xd8535610,0xdc9b5610,0xd8535612,0xdc9b5612,0xd8535614,0xdc9d5614,0xd8535616,
	0xdc9d5616,0xd8535618,0xdc9f5618,0xd853561a,0xdc9f561a,0xd853561c,0xdca1561c,0xd853561e,
	0xdca1561e,0xd8535620,0xdca35620,0xd8535622,0xdca35622,0xd8535624,0xdca55624,0xd8535626,
	0xdca55626,0xd8535628,0xdca75628,0xd853562a,0xdca7562a,0xd853562c,0xdca9562c,0xd853562e,
	0xdca9562e,0xd8535630,0xdcab5630,0xd8535632,0xdcab5632,0xd8535634,0xdcad5634,0xd8535636,
	0xdcad5636,0xd8535638,0xdcaf5638,0xd853563a,0xdcaf563a,0xd853563c,0xdcb1563c,0xd853563e,
	0xdcb1563e,0xd8535640,0xdcb35640,0xd8535642,0xdcb35642,0xd8535644,0xdcb55644,0xd8535646,
	0xdcb55646,0xd8535648,0xdcb75648,0xd853564a,0xdcb7564a,0xd853564c,0xdcb9564c,0xd853564e,
	0xdcb9564e,0xd8535650,0xdcbb5650,0xd8535652,0xdcbb5652,0xd8535654,0xdcbd5654,0xd8535656,
	0xdcbd5656,0xd8535658,0xdcbf5658,0xd853565a,0xdcbf565a,0xd853565c,0xdcc1565c,0xd853565e,
	0xdcc1565e,0xd8535660,0xdcc35660,0xd8535662,0xdcc35662,0xd8535664,0xdcc55664,0xd8535666,
	0xdcc55666,0xd8535668,0xdcc75668,0xd853566a,0xdcc7566a,0xd853566c,0xdcc9566c,0xd853566e,
	0xdcc9566e,0xd8535670,0xdccb5670,0xd8535672,0xdccb5672,0xd8535674,0xdccd5674,0xd8535676,
	0xdccd5676,0xd8535678,0xdccf5678,0xd853567a,0xdccf567a,0xd853567c,0xdcd1567c,0xd853567e,
	0xdcd1567e,0xd8535680,0xdcd35680,0xd8535682,0xdcd35682,0xd8535684,0xdcd55684,0xd8535686,
	0xdcd55686,0xd8535688,0xdcd75688,0xd853568a,0xdcd7568a,0xd853568c,0xdcd9568c,0xd853568e,
	0xdcd9568e,0xd8535690,0xdcdb5690,0xd8535692,0xdcdb5692,0xd8535694,0xdcdd5694,0xd8535696,
	0xdcdd5696,0xd8535698,0xdcdf5698,0xd853569a,0xdcdf569a,0xd853569c,0xdce1569c,0xd853569e,
	0xdce1569e,0xd85356a0,0xdce356a0,0xd85356a2,0xdce356a2,0xd85356a4,0xdce556a4,0xd85356a6,
	0xdce556a6,0xd85356a8,0xdce756a8,0xd85356aa,0xdce756aa,0xd85356ac,0xdce956ac,0xd85356ae,
	0xdce956ae,0xd85356b0,0xdceb56b0,0xd85356b2,0xdceb56b2,0xd85356b4,0xdced56b4,0xd85356b6,
	0xdced56b6,0xd85356b8,0xdcef56b8,0xd85356ba,0xdcef56ba,0xd85356bc,0xdcf156bc,0xd85356be,
	0xdcf156be,0xd85356c0,0xdcf356c0,0xd85356c2,0xdcf356c2,0xd85356c4,0xdcf556c4,0xd85356c6,
	0xdcf556c6,0xd85356c8,0xdcf756c8,0xd85356ca,0xdcf756ca,0xd85356cc,0xdcf956cc,0xd85356ce,
	0xdcf956ce,0xd85356d0,0xdcfb56d0,0xd85356d2,0xdcfb56d2,0xd85356d4,0xdcfd56d4,0xd85356d6,
	0xdcfd56d6,0xd85356d8,0xdcff56d8,0xd85356da,0xdcff56da,0xd85356dc,0xdd0156dc,0xd85356de,
	0xdd0156de,0xd85356e0,0xdd0356e0,0xd85356e2,0xdd0356e2,0xd85356e4,0xdd0556e4,0xd85356e6,
	0xdd0556e6,0xd85356e8,0xdd0756e8,0xd85356ea,0xdd0756ea,0xd85356ec,0xdd0956ec,0xd85356ee,
	0xdd0956ee,0xd85356f0,0xdd0b56f0,0xd85356f2,0xdd0b56f2,0xd85356f4,0xdd0d56f4,0xd85356f6,
	0xdd0d56f6,0xd85356f8,0xdd0f56f8,0xd85356fa,0xdd0f56fa,0xd85356fc,0xdd1156fc,0xd85356fe,
	0xdd1156fe,0xd8535700,0xdd135700,0xd8535702,0xdd135702,0xd8535704,0xdd155704,0xd8535706,
	0xdd155706,0xd8535708,0xdd175708,0xd853570a,0xdd17570a,0xd853570c,0xdd19570c,0xd853570e,
	0xdd19570e,0xd8535710,0xdd1b5710,0xd8535712,0xdd1b5712,0xd8535714,0xdd1d5714,0xd8535716,
	0xdd1d5716,0xd8535718,0xdd1f5718,0xd853571a,0xdd1f571a,0xd853571c,0xdd21571c,0xd853571e,
	0xdd21571e,0xd8535720,0xdd235720,0xd8535722,0xdd235722,0xd8535724,0xdd255724,0xd8535726,
	0xdd255726,0xd8535728,0xdd275728,0xd853572a,0xdd27572a,0xd853572c,0xdd29572c,0xd853572e,
	0xdd29572e,0xd8535730,0xdd2b5730,0xd8535732,0xdd2b5732,0xd8535734,0xdd2d5734,0xd8535736,
	0xdd2d5736,0xd8535738,0xdd2f5738,0xd853573a,0xdd2f573a,0xd853573c,0xdd31573c,0xd853573e,
	0xdd31573e,0xd8535740,0xdd335740,0xd8535742,0xdd335742,0xd8535744,0xdd355744,0xd8535746,
	0xdd355746,0xd8535748,0xdd375748,0xd853574a,0xdd37574a,0xd853574c,0xdd39574c,0xd853574e,
	0xdd39574e,0xd8535750,0xdd3b5750,0xd8535752,0xdd3b5752,0xd8535754,0xdd3d5754,0xd8535756,
	0xdd3d5756,0xd8535758,0xdd3f5758,0xd853575a,0xdd3f575a,0xd853575c,0xdd41575c,0xd853575e,
	0xdd41575e,0xd8535760,0xdd435760,0xd8535762,0xdd435762,0xd8535764,0xdd455764,0xd8535766,
	0xdd455766,0xd8535768,0xdd475768,0xd853576a,0xdd47576a,0xd853576c,0xdd49576c,0xd853576e,
	0xdd49576e,0xd8535770,0xdd4b5770,0xd8535772,0xdd4b5772,0xd8535774,0xdd4d5774,0xd8535776,
	0xdd4d5776,0xd8535778,0xdd4f5778,0xd853577a,0xdd4f577a,0xd853577c,0xdd51577c,0xd853577e,
	0xdd51577e,0xd8535780,0xdd535780,0xd8535782,0xdd535782,0xd8535784,0xdd555784,0xd8535786,
	0xdd555786,0xd8535788,0xdd575788,0xd853578a,0xdd57578a,0xd853578c,0xdd59578c,0xd853578e,
	0xdd59578e,0xd8535790,0xdd5b5790,0xd8535792,0xdd5b5792,0xd8535794,0xdd5d5794,0xd8535796,
	0xdd5d5796,0xd8535798,0xdd5f5798,0xd853579a,0xdd5f579a,0xd853579c,0xdd61579c,0xd853579e,
	0xdd61579e,0xd85357a0,0xdd6357a0,0xd85357a2,0xdd6357a2,0xd85357a4,0xdd6557a4,0xd85357a6,
	0xdd6557a6,0xd85357a8,0xdd6757a8,0xd85357aa,0xdd6757aa,0xd85357ac,0xdd6957ac,0xd85357ae,
	0xdd6957ae,0xd85357b0,0xdd6b57b0,0xd85357b2,0xdd6b57b2,0xd85357b4,0xdd6d57b4,0xd85357b6,
	0xdd6d57b6,0xd85357b8,0xdd6f57b8,0xd85357ba,0xdd6f57ba,0xd85357bc,0xdd7157bc,0xd85357be,
	0xdd7157be,0xd85357c0,0xdd7357c0,0xd85357c2,0xdd7357c2,0xd85357c4,0xdd7557c4,0xd85357c6,
	0xdd7557c6,0xd85357c8,0xdd7757c8,0xd85357ca,0xdd7757ca,0xd85357cc,0xdd7957cc,0xd85357ce,
	0xdd7957ce,0xd85357d0,0xdd7b57d0,0xd85357d2,0xdd7b57d2,0xd85357d4,0xdd7d57d4,0xd85357d6,
	0xdd7d57d6,0xd85357d8,0xdd7f57d8,0xd85357da,0xdd7f57da,0xd85357dc,0xdd8157dc,0xd85357de,
	0xdd8157de,0xd85357e0,0xdd8357e0,0xd85357e2,0xdd8357e2,0xd85357e4,0xdd8557e4,0xd85357e6,
	0xdd8557e6,0xd85357e8,0xdd8757e8,0xd85357ea,0xdd8757ea,0xd85357ec,0xdd8957ec,0xd85357ee,
	0xdd8957ee,0xd85357f0,0xdd8b57f0,0xd85357f2,0xdd8b57f2,0xd85357f4,0xdd8d57f4,0xd85357f6,
	0xdd8d57f6,0xd85357f8,0xdd8f57f8,0xd85357fa,0xdd8f57fa,0xd85357fc,0xdd9157fc,0xd85357fe,
	0xdd9157fe,0xd8535800,0xdd935800,0xd8535802,0xdd935802,0xd8535804,0xdd955804,0xd8535806,
	0xdd955806,0xd8535808,0xdd975808,0xd853580a,0xdd97580a,0xd853580c,0xdd99580c,0xd853580e,
	0xdd99580e,0xd8535810,0xdd9b5810,0xd8535812,0xdd9b5812,0xd8535814,0xdd9d5814,0xd8535816,
	0xdd9d5816,0xd8535818,0xdd9f5818,0xd853581a,0xdd9f581a,0xd853581c,0xdda1581c,0xd853581e,
	0xdda1581e,0xd8535820,0xdda35820,0xd8535822,0xdda35822,0xd8535824,0xdda55824,0xd8535826,
	0xdda55826,0xd8535828,0xdda75828,0xd853582a,0xdda7582a,0xd853582c,0xdda9582c,0xd853582e,
	0xdda9582e,0xd8535830,0xddab5830,0xd8535832,0xddab5832,0xd8535834,0xddad5834,0xd8535836,
	0xddad5836,0xd8535838,0xddaf5838,0xd853583a,0xddaf583a,0xd853583c,0xddb1583c,0xd853583e,
	0xddb1583e,0xd8535840,0xddb35840,0xd8535842,0xddb35842,0xd8535844,0xddb55844,0xd8535846,
	0xddb55846,0xd8535848,0xddb75848,0xd853584a,0xddb7584a,0xd853584c,0xddb9584c,0xd853584e,
	0xddb9584e,0xd8535850,0xddbb5850,0xd8535852,0xddbb5852,0xd8535854,0xddbd5854,0xd8535856,
	0xddbd5856,0xd8535858,0xddbf5858,0xd853585a,0xddbf585a,0xd853585c,0xddc1585c,0xd853585e,
	0xddc1585e,0xd8535860,0xddc35860,0xd8535862,0xddc35862,0xd8535864,0xddc55864,0xd8535866,
	0xddc55866,0xd8535868,0xddc75868,0xd853586a,0xddc7586a,0xd853586c,0xddc9586c,0xd853586e,
	0xddc9586e,0xd8535870,0xddcb5870,0xd8535872,0xddcb5872,0xd8535874,0xddcd5874,0xd8535876,
	0xddcd5876,0xd8535878,0xddcf5878,0xd853587a,0xddcf587a,0xd853587c,0xddd1587c,0xd853587e,
	0xddd1587e,0xd8535880,0xddd35880,0xd8535882,0xddd35882,0xd8535884,0xddd55884,0xd8535886,
	0xddd55886,0xd8535888,0xddd75888,0xd853588a,0xddd7588a,0xd853588c,0xddd9588c,0xd853588e,
	0xddd9588e,0xd8535890,0xdddb5890,0xd8535892,0xdddb5892,0xd8535894,0xdddd5894,0xd8535896,
	0xdddd5896,0xd8535898,0xdddf5898,0xd853589a,0xdddf589a,0xd853589c,0xdde1589c,0xd853589e,
	0xdde1589e,0xd85358a0,0xdde358a0,0xd85358a2,0xdde358a2,0xd85358a4,0xdde558a4,0xd85358a6,
	0xdde558a6,0xd85358a8,0xdde758a8,0xd85358aa,0xdde758aa,0xd85358ac,0xdde958ac,0xd85358ae,
	0xdde958ae,0xd85358b0,0xddeb58b0,0xd85358b2,0xddeb58b2,0xd85358b4,0xdded58b4,0xd85358b6,
	0xdded58b6,0xd85358b8,0xddef58b8,0xd85358ba,0xddef58ba,0xd85358bc,0xddf158bc,0xd85358be,
	0xddf158be,0xd85358c0,0xddf358c0,0xd85358c2,0xddf358c2,0xd85358c4,0xddf558c4,0xd85358c6,
	0xddf558c6,0xd85358c8,0xddf758c8,0xd85358ca,0xddf758ca,0xd85358cc,0xddf958cc,0xd85358ce,
	0xddf958ce,0xd85358d0,0xddfb58d0,0xd85358d2,0xddfb58d2,0xd85358d4,0xddfd58d4,0xd85358d6,
	0xddfd58d6,0xd85358d8,0xddff58d8,0xd85358da,0xddff58da,0xd85358dc,0xde0158dc,0xd85358de,
	0xde0158de,0xd85358e0,0xde0358e0,0xd85358e2,0xde0358e2,0xd85358e4,0xde0558e4,0xd85358e6,
	0xde0558e6,0xd85358e8,0xde0758e8,0xd85358ea,0xde0758ea,0xd85358ec,0xde0958ec,0xd85358ee,
	0xde0958ee,0xd85358f0,0xde0b58f0,0xd85358f2,0xde0b58f2,0xd85358f4,0xde0d58f4,0xd85358f6,
	0xde0d58f6,0xd85358f8,0xde0f58f8,0xd85358fa,0xde0f58fa,0xd85358fc,0xde1158fc,0xd85358fe,
	0xde1158fe,0xd8535900,0xde135900,0xd8535902,0xde135902,0xd8535904,0xde155904,0xd8535906,
	0xde155906,0xd8535908,0xde175908,0xd853590a,0xde17590a,0xd853590c,0xde19590c,0xd853590e,
	0xde19590e,0xd8535910,0xde1b5910,0xd8535912,0xde1b5912,0xd8535914,0xde1d5914,0xd8535916,
	0xde1d5916,0xd8535918,0xde1f5918,0xd853591a,0xde1f591a,0xd853591c,0xde21591c,0xd853591e,
	0xde21591e,0xd8535920,0xde235920,0xd8535922,0xde235922,0xd8535924,0xde255924,0xd8535926,
	0xde255926,0xd8535928,0xde275928,0xd853592a,0xde27592a,0xd853592c,0xde29592c,0xd853592e,
	0xde29592e,0xd8535930,0xde2b5930,0xd8535932,0xde2b5932,0xd8535934,0xde2d5934,0xd8535936,
	0xde2d5936,0xd8535938,0xde2f5938,0xd853593a,0xde2f593a,0xd853593c,0xde31593c,0xd853593e,
	0xde31593e,0xd8535940,0xde335940,0xd8535942,0xde335942,0xd8535944,0xde355944,0xd8535946,
	0xde355946,0xd8535948,0xde375948,0xd853594a,0xde37594a,0xd853594c,0xde39594c,0xd853594e,
	0xde39594e,0xd8535950,0xde3b5950,0xd8535952,0xde3b5952,0xd8535954,0xde3d5954,0xd8535956,
	0xde3d5956,0xd8535958,0xde3f5958,0xd853595a,0xde3f595a,0xd853595c,0xde41595c,0xd853595e,
	0xde41595e,0xd8535960,0xde435960,0xd8535962,0xde435962,0xd8535964,0xde455964,0xd8535966,
	0xde455966,0xd8535968,0xde475968,0xd853596a,0xde47596a,0xd853596c,0xde49596c,0xd853596e,
	0xde49596e,0xd8535970,0xde4b5970,0xd8535972,0xde4b5972,0xd8535974,0xde4d5974,0xd8535976,
	0xde4d5976,0xd8535978,0xde4f5978,0xd853597a,0xde4f597a,0xd853597c,0xde51597c,0xd853597e,
	0xde51597e,0xd8535980,0xde535980,0xd8535982,0xde535982,0xd8535984,0xde555984,0xd8535986,
	0xde555986,0xd8535988,0xde575988,0xd853598a,0xde57598a,0xd853598c,0xde59598c,0xd853598e,
	0xde59598e,0xd8535990,0xde5b5990,0xd8535992,0xde5b5992,0xd8535994,0xde5d5994,0xd8535996,
	0xde5d5996,0xd8535998,0xde5f5998,0xd853599a,0xde5f599a,0xd853599c,0xde61599c,0xd853599e,
	0xde61599e,0xd85359a0,0xde6359a0,0xd85359a2,0xde6359a2,0xd85359a4,0xde6559a4,0xd85359a6,
	0xde6559a6,0xd85359a8,0xde6759a8,0xd85359aa,0xde6759aa,0xd85359ac,0xde6959ac,0xd85359ae,
	0xde6959ae,0xd85359b0,0xde6b59b0,0xd85359b2,0xde6b59b2,0xd85359b4,0xde6d59b4,0xd85359b6,
	0xde6d59b6,0xd85359b8,0xde6f59b8,0xd85359ba,0xde6f59ba,0xd85359bc,0xde7159bc,0xd85359be,
	0xde7159be,0xd85359c0,0xde7359c0,0xd85359c2,0xde7359c2,0xd85359c4,0xde7559c4,0xd85359c6,
	0xde7559c6,0xd85359c8,0xde7759c8,0xd85359ca,0xde7759ca,0xd85359cc,0xde7959cc,0xd85359ce,
	0xde7959ce,0xd85359d0,0xde7b59d0,0xd85359d2,0xde7b59d2,0xd85359d4,0xde7d59d4,0xd85359d6,
	0xde7d59d6,0xd85359d8,0xde7f59d8,0xd85359da,0xde7f59da,0xd85359dc,0xde8159dc,0xd85359de,
	0xde8159de,0xd85359e0,0xde8359e0,0xd85359e2,0xde8359e2,0xd85359e4,0xde8559e4,0xd85359e6,
	0xde8559e6,0xd85359e8,0xde8759e8,0xd85359ea,0xde8759ea,0xd85359ec,0xde8959ec,0xd85359ee,
	0xde8959ee,0xd85359f0,0xde8b59f0,0xd85359f2,0xde8b59f2,0xd85359f4,0xde8d59f4,0xd85359f6,
	0xde8d59f6,0xd85359f8,0xde8f59f8,0xd85359fa,0xde8f59fa,0xd85359fc,0xde9159fc,0xd85359fe,
	0xde9159fe,0xd8535a00,0xde935a00,0xd8535a02,0xde935a02,0xd8535a04,0xde955a04,0xd8535a06,
	0xde955a06,0xd8535a08,0xde975a08,0xd8535a0a,0xde975a0a,0xd8535a0c,0xde995a0c,0xd8535a0e,
	0xde995a0e,0xd8535a10,0xde9b5a10,0xd8535a12,0xde9b5a12,0xd8535a14,0xde9d5a14,0xd8535a16,
	0xde9d5a16,0xd8535a18,0xde9f5a18,0xd8535a1a,0xde9f5a1a,0xd8535a1c,0xdea15a1c,0xd8535a1e,
	0xdea15a1e,0xd8535a20,0xdea35a20,0xd8535a22,0xdea35a22,0xd8535a24,0xdea55a24,0xd8535a26,
	0xdea55a26,0xd8535a28,0xdea75a28,0xd8535a2a,0xdea75a2a,0xd8535a2c,0xdea95a2c,0xd8535a2e,
	0xdea95a2e,0xd8535a30,0xdeab5a30,0xd8535a32,0xdeab5a32,0xd8535a34,0xdead5a34,0xd8535a36,
	0xdead5a36,0xd8535a38,0xdeaf5a38,0xd8535a3a,0xdeaf5a3a,0xd8535a3c,0xdeb15a3c,0xd8535a3e,
	0xdeb15a3e,0xd8535a40,0xdeb35a40,0xd8535a42,0xdeb35a42,0xd8535a44,0xdeb55a44,0xd8535a46,
	0xdeb55a46,0xd8535a48,0xdeb75a48,0xd8535a4a,0xdeb75a4a,0xd8535a4c,0xdeb95a4c,0xd8535a4e,
	0xdeb95a4e,0xd8535a50,0xdebb5a50,0xd8535a52,0xdebb5a52,0xd8535a54,0xdebd5a54,0xd8535a56,
	0xdebd5a56,0xd8535a58,0xdebf5a58,0xd8535a5a,0xdebf5a5a,0xd8535a5c,0xdec15a5c,0xd8535a5e,
	0xdec15a5e,0xd8535a60,0xdec35a60,0xd8535a62,0xdec35a62,0xd8535a64,0xdec55a64,0xd8535a66,
	0xdec55a66,0xd8535a68,0xdec75a68,0xd8535a6a,0xdec75a6a,0xd8535a6c,0xdec95a6c,0xd8535a6e,
	0xdec95a6e,0xd8535a70,0xdecb5a70,0xd8535a72,0xdecb5a72,0xd8535a74,0xdecd5a74,0xd8535a76,
	0xdecd5a76,0xd8535a78,0xdecf5a78,0xd8535a7a,0xdecf5a7a,0xd8535a7c,0xded15a7c,0xd8535a7e,
	0xded15a7e,0xd8535a80,0xded35a80,0xd8535a82,0xded35a82,0xd8535a84,0xded55a84,0xd8535a86,
	0xded55a86,0xd8535a88,0xded75a88,0xd8535a8a,0xded75a8a,0xd8535a8c,0xded95a8c,0xd8535a8e,
	0xded95a8e,0xd8535a90,0xdedb5a90,0xd8535a92,0xdedb5a92,0xd8535a94,0xdedd5a94,0xd8535a96,
	0xdedd5a96,0xd8535a98,0xdedf5a98,0xd8535a9a,0xdedf5a9a,0xd8535a9c,0xdee15a9c,0xd8535a9e,
	0xdee15a9e,0xd8535aa0,0xdee35aa0,0xd8535aa2,0xdee35aa2,0xd8535aa4,0xdee55aa4,0xd8535aa6,
	0xdee55aa6,0xd8535aa8,0xdee75aa8,0xd8535aaa,0xdee75aaa,0xd8535aac,0xdee95aac,0xd8535aae,
	0xdee95aae,0xd8535ab0,0xdeeb5ab0,0xd8535ab2,0xdeeb5ab2,0xd8535ab4,0xdeed5ab4,0xd8535ab6,
	0xdeed5ab6,0xd8535ab8,0xdeef5ab8,0xd8535aba,0xdeef5aba,0xd8535abc,0xdef15abc,0xd8535abe,
	0xdef15abe,0xd8535ac0,0xdef35ac0,0xd8535ac2,0xdef35ac2,0xd8535ac4,0xdef55ac4,0xd8535ac6,
	0xdef55ac6,0xd8535ac8,0xdef75ac8,0xd8535aca,0xdef75aca,0xd8535acc,0xdef95acc,0xd8535ace,
	0xdef95ace,0xd8535ad0,0xdefb5ad0,0xd8535ad2,0xdefb5ad2,0xd8535ad4,0xdefd5ad4,0xd8535ad6,
	0xdefd5ad6,0xd8535ad8,0xdeff5ad8,0xd8535ada,0xdeff5ada,0xd8535adc,0xdf015adc,0xd8535ade,
	0xdf015ade,0xd8535ae0,0xdf035ae0,0xd8535ae2,0xdf035ae2,0xd8535ae4,0xdf055ae4,0xd8535ae6,
	0xdf055ae6,0xd8535ae8,0xdf075ae8,0xd8535aea,0xdf075aea,0xd8535aec,0xdf095aec,0xd8535aee,
	0xdf095aee,0xd8535af0,0xdf0b5af0,0xd8535af2,0xdf0b5af2,0xd8535af4,0xdf0d5af4,0xd8535af6,
	0xdf0d5af6,0xd8535af8,0xdf0f5af8,0xd8535afa,0xdf0f5afa,0xd8535afc,0xdf115afc,0xd8535afe,
	0xdf115afe,0xd8535b00,0xdf135b00,0xd8535b02,0xdf135b02,0xd8535b04,0xdf155b04,0xd8535b06,
	0xdf155b06,0xd8535b08,0xdf175b08,0xd8535b0a,0xdf175b0a,0xd8535b0c,0xdf195b0c,0xd8535b0e,
	0xdf195b0e,0xd8535b10,0xdf1b5b10,0xd8535b12,0xdf1b5b12,0xd8535b14,0xdf1d5b14,0xd8535b16,
	0xdf1d5b16,0xd8535b18,0xdf1f5b18,0xd8535b1a,0xdf1f5b1a,0xd8535b1c,0xdf215b1c,0xd8535b1e,
	0xdf215b1e,0xd8535b20,0xdf235b20,0xd8535b22,0xdf235b22,0xd8535b24,0xdf255b24,0xd8535b26,
	0xdf255b26,0xd8535b28,0xdf275b28,0xd8535b2a,0xdf275b2a,0xd8535b2c,0xdf295b2c,0xd8535b2e,
	0xdf295b2e,0xd8535b30,0xdf2b5b30,0xd8535b32,0xdf2b5b32,0xd8535b34,0xdf2d5b34,0xd8535b36,
	0xdf2d5b36,0xd8535b38,0xdf2f5b38,0xd8535b3a,0xdf2f5b3a,0xd8535b3c,0xdf315b3c,0xd8535b3e,
	0xdf315b3e,0xd8535b40,0xdf335b40,0xd8535b42,0xdf335b42,0xd8535b44,0xdf355b44,0xd8535b46,
	0xdf355b46,0xd8535b48,0xdf375b48,0xd8535b4a,0xdf375b4a,0xd8535b4c,0xdf395b4c,0xd8535b4e,
	0xdf395b4e,0xd8535b50,0xdf3b5b50,0xd8535b52,0xdf3b5b52,0xd8535b54,0xdf3d5b54,0xd8535b56,
	0xdf3d5b56,0xd8535b58,0xdf3f5b58,0xd8535b5a,0xdf3f5b5a,0xd8535b5c,0xdf415b5c,0xd8535b5e,
	0xdf415b5e,0xd8535b60,0xdf435b60,0xd8535b62,0xdf435b62,0xd8535b64,0xdf455b64,0xd8535b66,
	0xdf455b66,0xd8535b68,0xdf475b68,0xd8535b6a,0xdf475b6a,0xd8535b6c,0xdf495b6c,0xd8535b6e,
	0xdf495b6e,0xd8535b70,0xdf4b5b70,0xd8535b72,0xdf4b5b72,0xd8535b74,0xdf4d5b74,0xd8535b76,
	0xdf4d5b76,0xd8535b78,0xdf4f5b78,0xd8535b7a,0xdf4f5b7a,0xd8535b7c,0xdf515b7c,0xd8535b7e,
	0xdf515b7e,0xd8535b80,0xdf535b80,0xd8535b82,0xdf535b82,0xd8535b84,0xdf555b84,0xd8535b86,
	0xdf555b86,0xd8535b88,0xdf575b88,0xd8535b8a,0xdf575b8a,0xd8535b8c,0xdf595b8c,0xd8535b8e,
	0xdf595b8e,0xd8535b90,0xdf5b5b90,0xd8535b92,0xdf5b5b92,0xd8535b94,0xdf5d5b94,0xd8535b96,
	0xdf5d5b96,0xd8535b98,0xdf5f5b98,0xd8535b9a,0xdf5f5b9a,0xd8535b9c,0xdf615b9c,0xd8535b9e,
	0xdf615b9e,0xd8535ba0,0xdf635ba0,0xd8535ba2,0xdf635ba2,0xd8535ba4,0xdf655ba4,0xd8535ba6,
	0xdf655ba6,0xd8535ba8,0xdf675ba8,0xd8535baa,0xdf675baa,0xd8535bac,0xdf695bac,0xd8535bae,
	0xdf695bae,0xd8535bb0,0xdf6b5bb0,0xd8535bb2,0xdf6b5bb2,0xd8535bb4,0xdf6d5bb4,0xd8535bb6,
	0xdf6d5bb6,0xd8535bb8,0xdf6f5bb8,0xd8535bba,0xdf6f5bba,0xd8535bbc,0xdf715bbc,0xd8535bbe,
	0xdf715bbe,0xd8535bc0,0xdf735bc0,0xd8535bc2,0xdf735bc2,0xd8535bc4,0xdf755bc4,0xd8535bc6,
	0xdf755bc6,0xd8535bc8,0xdf775bc8,0xd8535bca,0xdf775bca,0xd8535bcc,0xdf795bcc,0xd8535bce,
	0xdf795bce,0xd8535bd0,0xdf7b5bd0,0xd8535bd2,0xdf7b5bd2,0xd8535bd4,0xdf7d5bd4,0xd8535bd6,
	0xdf7d5bd6,0xd8535bd8,0xdf7f5bd8,0xd8535bda,0xdf7f5bda,0xd8535bdc,0xdf815bdc,0xd8535bde,
	0xdf815bde,0xd8535be0,0xdf835be0,0xd8535be2,0xdf835be2,0xd8535be4,0xdf855be4,0xd8535be6,
	0xdf855be6,0xd8535be8,0xdf875be8,0xd8535bea,0xdf875bea,0xd8535bec,0xdf895bec,0xd8535bee,
	0xdf895bee,0xd8535bf0,0xdf8b5bf0,0xd8535bf2,0xdf8b5bf2,0xd8535bf4,0xdf8d5bf4,0xd8535bf6,
	0xdf8d5bf6,0xd8535bf8,0xdf8f5bf8,0xd8535bfa,0xdf8f5bfa,0xd8535bfc,0xdf915bfc,0xd8535bfe,
	0xdf915bfe,0xd8535c00,0xdf935c00,0xd8535c02,0xdf935c02,0xd8535c04,0xdf955c04,0xd8535c06,
	0xdf955c06,0xd8535c08,0xdf975c08,0xd8535c0a,0xdf975c0a,0xd8535c0c,0xdf995c0c,0xd8535c0e,
	0xdf995c0e,0xd8535c10,0xdf9b5c10,0xd8535c12,0xdf9b5c12,0xd8535c14,0xdf9d5c14,0xd8535c16,
	0xdf9d5c16,0xd8535c18,0xdf9f5c18,0xd8535c1a,0xdf9f5c1a,0xd8535c1c,0xdfa15c1c,0xd8535c1e,
	0xdfa15c1e,0xd8535c20,0xdfa35c20,0xd8535c22,0xdfa35c22,0xd8535c24,0xdfa55c24,0xd8535c26,
	0xdfa55c26,0xd8535c28,0xdfa75c28,0xd8535c2a,0xdfa75c2a,0xd8535c2c,0xdfa95c2c,0xd8535c2e,
	0xdfa95c2e,0xd8535c30,0xdfab5c30,0xd8535c32,0xdfab5c32,0xd8535c34,0xdfad5c34,0xd8535c36,
	0xdfad5c36,0xd8535c38,0xdfaf5c38,0xd8535c3a,0xdfaf5c3a,0xd8535c3c,0xdfb15c3c,0xd8535c3e,
	0xdfb15c3e,0xd8535c40,0xdfb35c40,0xd8535c42,0xdfb35c42,0xd8535c44,0xdfb55c44,0xd8535c46,
	0xdfb55c46,0xd8535c48,0xdfb75c48,0xd8535c4a,0xdfb75c4a,0xd8535c4c,0xdfb95c4c,0xd8535c4e,
	0xdfb95c4e,0xd8535c50,0xdfbb5c50,0xd8535c52,0xdfbb5c52,0xd8535c54,0xdfbd5c54,0xd8535c56,
	0xdfbd5c56,0xd8535c58,0xdfbf5c58,0xd8535c5a,0xdfbf5c5a,0xd8535c5c,0xdfc15c5c,0xd8535c5e,
	0xdfc15c5e,0xd8535c60,0xdfc35c60,0xd8535c62,0xdfc35c62,0xd8535c64,0xdfc55c64,0xd8535c66,
	0xdfc55c66,0xd8535c68,0xdfc75c68,0xd8535c6a,0xdfc75c6a,0xd8535c6c,0xdfc95c6c,0xd8535c6e,
	0xdfc95c6e,0xd8535c70,0xdfcb5c70,0xd8535c72,0xdfcb5c72,0xd8535c74,0xdfcd5c74,0xd8535c76,
	0xdfcd5c76,0xd8535c78,0xdfcf5c78,0xd8535c7a,0xdfcf5c7a,0xd8535c7c,0xdfd15c7c,0xd8535c7e,
	0xdfd15c7e,0xd8535c80,0xdfd35c80,0xd8535c82,0xdfd35c82,0xd8535c84,0xdfd55c84,0xd8535c86,
	0xdfd55c86,0xd8535c88,0xdfd75c88,0xd8535c8a,0xdfd75c8a,0xd8535c8c,0xdfd95c8c,0xd8535c8e,
	0xdfd95c8e,0xd8535c90,0xdfdb5c90,0xd8535c92,0xdfdb5c92,0xd8535c94,0xdfdd5c94,0xd8535c96,
	0xdfdd5c96,0xd8535c98,0xdfdf5c98,0xd8535c9a,0xdfdf5c9a,0xd8535c9c,0xdfe15c9c,0xd8535c9e,
	0xdfe15c9e,0xd8535ca0,0xdfe35ca0,0xd8535ca2,0xdfe35ca2,0xd8535ca4,0xdfe55ca4,0xd8535ca6,
	0xdfe55ca6,0xd8535ca8,0xdfe75ca8,0xd8535caa,0xdfe75caa,0xd8535cac,0xdfe95cac,0xd8535cae,
	0xdfe95cae,0xd8535cb0,0xdfeb5cb0,0xd8535cb2,0xdfeb5cb2,0xd8535cb4,0xdfed5cb4,0xd8535cb6,
	0xdfed5cb6,0xd8535cb8,0xdfef5cb8,0xd8535cba,0xdfef5cba,0xd8535cbc,0xdff15cbc,0xd8535cbe,
	0xdff15cbe,0xd8535cc0,0xdff35cc0,0xd8535cc2,0xdff35cc2,0xd8535cc4,0xdff55cc4,0xd8535cc6,
	0xdff55cc6,0xd8535cc8,0xdff75cc8,0xd8535cca,0xdff75cca,0xd8535ccc,0xdff95ccc,0xd8535cce,
	0xdff95cce,0xd8535cd0,0xdffb5cd0,0xd8535cd2,0xdffb5cd2,0xd8535cd4,0xdffd5cd4,0xd8535cd6,
	0xdffd5cd6,0xd8535cd8,0xdfff5cd8,0xd8535cda,0xdfff5cda,0xd8555cdc,0xdc015cdc,0xd8555cde,
	0xdc015cde,0xd8555ce0,0xdc035ce0,0xd8555ce2,0xdc035ce2,0xd8555ce4,0xdc055ce4,0xd8555ce6,
	0xdc055ce6,0xd8555ce8,0xdc075ce8,0xd8555cea,0xdc075cea,0xd8555cec,0xdc095cec,0xd8555cee,
	0xdc095cee,0xd8555cf0,0xdc0b5cf0,0xd8555cf2,0xdc0b5cf2,0xd8555cf4,0xdc0d5cf4,0xd8555cf6,
	0xdc0d5cf6,0xd8555cf8,0xdc0f5cf8,0xd8555cfa,0xdc0f5cfa,0xd8555cfc,0xdc115cfc,0xd8555cfe,
	0xdc115cfe,0xd8555d00,0xdc135d00,0xd8555d02,0xdc135d02,0xd8555d04,0xdc155d04,0xd8555d06,
	0xdc155d06,0xd8555d08,0xdc175d08,0xd8555d0a,0xdc175d0a,0xd8555d0c,0xdc195d0c,0xd8555d0e,
	0xdc195d0e,0xd8555d10,0xdc1b5d10,0xd8555d12,0xdc1b5d12,0xd8555d14,0xdc1d5d14,0xd8555d16,
	0xdc1d5d16,0xd8555d18,0xdc1f5d18,0xd8555d1a,0xdc1f5d1a,0xd8555d1c,0xdc215d1c,0xd8555d1e,
	0xdc215d1e,0xd8555d20,0xdc235d20,0xd8555d22,0xdc235d22,0xd8555d24,0xdc255d24,0xd8555d26,
	0xdc255d26,0xd8555d28,0xdc275d28,0xd8555d2a,0xdc275d2a,0xd8555d2c,0xdc295d2c,0xd8555d2e,
	0xdc295d2e,0xd8555d30,0xdc2b5d30,0xd8555d32,0xdc2b5d32,0xd8555d34,0xdc2d5d34,0xd8555d36,
	0xdc2d5d36,0xd8555d38,0xdc2f5d38,0xd8555d3a,0xdc2f5d3a,0xd8555d3c,0xdc315d3c,0xd8555d3e,
	0xdc315d3e,0xd8555d40,0xdc335d40,0xd8555d42,0xdc335d42,0xd8555d44,0xdc355d44,0xd8555d46,
	0xdc355d46,0xd8555d48,0xdc375d48,0xd8555d4a,0xdc375d4a,0xd8555d4c,0xdc395d4c,0xd8555d4e,
	0xdc395d4e,0xd8555d50,0xdc3b5d50,0xd8555d52,0xdc3b5d52,0xd8555d54,0xdc3d5d54,0xd8555d56,
	0xdc3d5d56,0xd8555d58,0xdc3f5d58,0xd8555d5a,0xdc3f5d5a,0xd8555d5c,0xdc415d5c,0xd8555d5e,
	0xdc415d5e,0xd8555d60,0xdc435d60,0xd8555d62,0xdc435d62,0xd8555d64,0xdc455d64,0xd8555d66,
	0xdc455d66,0xd8555d68,0xdc475d68,0xd8555d6a,0xdc475d6a,0xd8555d6c,0xdc495d6c,0xd8555d6e,
	0xdc495d6e,0xd8555d70,0xdc4b5d70,0xd8555d72,0xdc4b5d72,0xd8555d74,0xdc4d5d74,0xd8555d76,
	0xdc4d5d76,0xd8555d78,0xdc4f5d78,0xd8555d7a,0xdc4f5d7a,0xd8555d7c,0xdc515d7c,0xd8555d7e,
	0xdc515d7e,0xd8555d80,0xdc535d80,0xd8555d82,0xdc535d82,0xd8555d84,0xdc555d84,0xd8555d86,
	0xdc555d86,0xd8555d88,0xdc575d88,0xd8555d8a,0xdc575d8a,0xd8555d8c,0xdc595d8c,0xd8555d8e,
	0xdc595d8e,0xd8555d90,0xdc5b5d90,0xd8555d92,0xdc5b5d92,0xd8555d94,0xdc5d5d94,0xd8555d96,
	0xdc5d5d96,0xd8555d98,0xdc5f5d98,0xd8555d9a,0xdc5f5d9a,0xd8555d9c,0xdc615d9c,0xd8555d9e,
	0xdc615d9e,0xd8555da0,0xdc635da0,0xd8555da2,0xdc635da2,0xd8555da4,0xdc655da4,0xd8555da6,
	0xdc655da6,0xd8555da8,0xdc675da8,0xd8555daa,0xdc675daa,0xd8555dac,0xdc695dac,0xd8555dae,
	0xdc695dae,0xd8555db0,0xdc6b5db0,0xd8555db2,0xdc6b5db2,0xd8555db4,0xdc6d5db4,0xd8555db6,
	0xdc6d5db6,0xd8555db8,0xdc6f5db8,0xd8555dba,0xdc6f5dba,0xd8555dbc,0xdc715dbc,0xd8555dbe,
	0xdc715dbe,0xd8555dc0,0xdc735dc0,0xd8555dc2,0xdc735dc2,0xd8555dc4,0xdc755dc4,0xd8555dc6,
	0xdc755dc6,0xd8555dc8,0xdc775dc8,0xd8555dca,0xdc775dca,0xd8555dcc,0xdc795dcc,0xd8555dce,
	0xdc795dce,0xd8555dd0,0xdc7b5dd0,0xd8555dd2,0xdc7b5dd2,0xd8555dd4,0xdc7d5dd4,0xd8555dd6,
	0xdc7d5dd6,0xd8555dd8,0xdc7f5dd8,0xd8555dda,0xdc7f5dda,0xd8555ddc,0xdc815ddc,0xd8555dde,
	0xdc815dde,0xd8555de0,0xdc835de0,0xd8555de2,0xdc835de2,0xd8555de4,0xdc855de4,0xd8555de6,
	0xdc855de6,0xd8555de8,0xdc875de8,0xd8555dea,0xdc875dea,0xd8555dec,0xdc895dec,0xd8555dee,
	0xdc895dee,0xd8555df0,0xdc8b5df0,0xd8555df2,0xdc8b5df2,0xd8555df4,0xdc8d5df4,0xd8555df6,
	0xdc8d5df6,0xd8555df8,0xdc8f5df8,0xd8555dfa,0xdc8f5dfa,0xd8555dfc,0xdc915dfc,0xd8555dfe,
	0xdc915dfe,0xd8555e00,0xdc935e00,0xd8555e02,0xdc935e02,0xd8555e04,0xdc955e04,0xd8555e06,
	0xdc955e06,0xd8555e08,0xdc975e08,0xd8555e0a,0xdc975e0a,0xd8555e0c,0xdc995e0c,0xd8555e0e,
	0xdc995e0e,0xd8555e10,0xdc9b5e10,0xd8555e12,0xdc9b5e12,0xd8555e14,0xdc9d5e14,0xd8555e16,
	0xdc9d5e16,0xd8555e18,0xdc9f5e18,0xd8555e1a,0xdc9f5e1a,0xd8555e1c,0xdca15e1c,0xd8555e1e,
	0xdca15e1e,0xd8555e20,0xdca35e20,0xd8555e22,0xdca35e22,0xd8555e24,0xdca55e24,0xd8555e26,
	0xdca55e26,0xd8555e28,0xdca75e28,0xd8555e2a,0xdca75e2a,0xd8555e2c,0xdca95e2c,0xd8555e2e,
	0xdca95e2e,0xd8555e30,0xdcab5e30,0xd8555e32,0xdcab5e32,0xd8555e34,0xdcad5e34,0xd8555e36,
	0xdcad5e36,0xd8555e38,0xdcaf5e38,0xd8555e3a,0xdcaf5e3a,0xd8555e3c,0xdcb15e3c,0xd8555e3e,
	0xdcb15e3e,0xd8555e40,0xdcb35e40,0xd8555e42,0xdcb35e42,0xd8555e44,0xdcb55e44,0xd8555e46,
	0xdcb55e46,0xd8555e48,0xdcb75e48,0xd8555e4a,0xdcb75e4a,0xd8555e4c,0xdcb95e4c,0xd8555e4e,
	0xdcb95e4e,0xd8555e50,0xdcbb5e50,0xd8555e52,0xdcbb5e52,0xd8555e54,0xdcbd5e54,0xd8555e56,
	0xdcbd5e56,0xd8555e58,0xdcbf5e58,0xd8555e5a,0xdcbf5e5a,0xd8555e5c,0xdcc15e5c,0xd8555e5e,
	0xdcc15e5e,0xd8555e60,0xdcc35e60,0xd8555e62,0xdcc35e62,0xd8555e64,0xdcc55e64,0xd8555e66,
	0xdcc55e66,0xd8555e68,0xdcc75e68,0xd8555e6a,0xdcc75e6a,0xd8555e6c,0xdcc95e6c,0xd8555e6e,
	0xdcc95e6e,0xd8555e70,0xdccb5e70,0xd8555e72,0xdccb5e72,0xd8555e74,0xdccd5e74,0xd8555e76,
	0xdccd5e76,0xd8555e78,0xdccf5e78,0xd8555e7a,0xdccf5e7a,0xd8555e7c,0xdcd15e7c,0xd8555e7e,
	0xdcd15e7e,0xd8555e80,0xdcd35e80,0xd8555e82,0xdcd35e82,0xd8555e84,0xdcd55e84,0xd8555e86,
	0xdcd55e86,0xd8555e88,0xdcd75e88,0xd8555e8a,0xdcd75e8a,0xd8555e8c,0xdcd95e8c,0xd8555e8e,
	0xdcd95e8e,0xd8555e90,0xdcdb5e90,0xd8555e92,0xdcdb5e92,0xd8555e94,0xdcdd5e94,0xd8555e96,
	0xdcdd5e96,0xd8555e98,0xdcdf5e98,0xd8555e9a,0xdcdf5e9a,0xd8555e9c,0xdce15e9c,0xd8555e9e,
	0xdce15e9e,0xd8555ea0,0xdce35ea0,0xd8555ea2,0xdce35ea2,0xd8555ea4,0xdce55ea4,0xd8555ea6,
	0xdce55ea6,0xd8555ea8,0xdce75ea8,0xd8555eaa,0xdce75eaa,0xd8555eac,0xdce95eac,0xd8555eae,
	0xdce95eae,0xd8555eb0,0xdceb5eb0,0xd8555eb2,0xdceb5eb2,0xd8555eb4,0xdced5eb4,0xd8555eb6,
	0xdced5eb6,0xd8555eb8,0xdcef5eb8,0xd8555eba,0xdcef5eba,0xd8555ebc,0xdcf15ebc,0xd8555ebe,
	0xdcf15ebe,0xd8555ec0,0xdcf35ec0,0xd8555ec2,0xdcf35ec2,0xd8555ec4,0xdcf55ec4,0xd8555ec6,
	0xdcf55ec6,0xd8555ec8,0xdcf75ec8,0xd8555eca,0xdcf75eca,0xd8555ecc,0xdcf95ecc,0xd8555ece,
	0xdcf95ece,0xd8555ed0,0xdcfb5ed0,0xd8555ed2,0xdcfb5ed2,0xd8555ed4,0xdcfd5ed4,0xd8555ed6,
	0xdcfd5ed6,0xd8555ed8,0xdcff5ed8,0xd8555eda,0xdcff5eda,0xd8555edc,0xdd015edc,0xd8555ede,
	0xdd015ede,0xd8555ee0,0xdd035ee0,0xd8555ee2,0xdd035ee2,0xd8555ee4,0xdd055ee4,0xd8555ee6,
	0xdd055ee6,0xd8555ee8,0xdd075ee8,0xd8555eea,0xdd075eea,0xd8555eec,0xdd095eec,0xd8555eee,
	0xdd095eee,0xd8555ef0,0xdd0b5ef0,0xd8555ef2,0xdd0b5ef2,0xd8555ef4,0xdd0d5ef4,0xd8555ef6,
	0xdd0d5ef6,0xd8555ef8,0xdd0f5ef8,0xd8555efa,0xdd0f5efa,0xd8555efc,0xdd115efc,0xd8555efe,
	0xdd115efe,0xd8555f00,0xdd135f00,0xd8555f02,0xdd135f02,0xd8555f04,0xdd155f04,0xd8555f06,
	0xdd155f06,0xd8555f08,0xdd175f08,0xd8555f0a,0xdd175f0a,0xd8555f0c,0xdd195f0c,0xd8555f0e,
	0xdd195f0e,0xd8555f10,0xdd1b5f10,0xd8555f12,0xdd1b5f12,0xd8555f14,0xdd1d5f14,0xd8555f16,
	0xdd1d5f16,0xd8555f18,0xdd1f5f18,0xd8555f1a,0xdd1f5f1a,0xd8555f1c,0xdd215f1c,0xd8555f1e,
	0xdd215f1e,0xd8555f20,0xdd235f20,0xd8555f22,0xdd235f22,0xd8555f24,0xdd255f24,0xd8555f26,
	0xdd255f26,0xd8555f28,0xdd275f28,0xd8555f2a,0xdd275f2a,0xd8555f2c,0xdd295f2c,0xd8555f2e,
	0xdd295f2e,0xd8555f30,0xdd2b5f30,0xd8555f32,0xdd2b5f32,0xd8555f34,0xdd2d5f34,0xd8555f36,
	0xdd2d5f36,0xd8555f38,0xdd2f5f38,0xd8555f3a,0xdd2f5f3a,0xd8555f3c,0xdd315f3c,0xd8555f3e,
	0xdd315f3e,0xd8555f40,0xdd335f40,0xd8555f42,0xdd335f42,0xd8555f44,0xdd355f44,0xd8555f46,
	0xdd355f46,0xd8555f48,0xdd375f48,0xd8555f4a,0xdd375f4a,0xd8555f4c,0xdd395f4c,0xd8555f4e,
	0xdd395f4e,0xd8555f50,0xdd3b5f50,0xd8555f52,0xdd3b5f52,0xd8555f54,0xdd3d5f54,0xd8555f56,
	0xdd3d5f56,0xd8555f58,0xdd3f5f58,0xd8555f5a,0xdd3f5f5a,0xd8555f5c,0xdd415f5c,0xd8555f5e,
	0xdd415f5e,0xd8555f60,0xdd435f60,0xd8555f62,0xdd435f62,0xd8555f64,0xdd455f64,0xd8555f66,
	0xdd455f66,0xd8555f68,0xdd475f68,0xd8555f6a,0xdd475f6a,0xd8555f6c,0xdd495f6c,0xd8555f6e,
	0xdd495f6e,0xd8555f70,0xdd4b5f70,0xd8555f72,0xdd4b5f72,0xd8555f74,0xdd4d5f74,0xd8555f76,
	0xdd4d5f76,0xd8555f78,0xdd4f5f78,0xd8555f7a,0xdd4f5f7a,0xd8555f7c,0xdd515f7c,0xd8555f7e,
	0xdd515f7e,0xd8555f80,0xdd535f80,0xd8555f82,0xdd535f82,0xd8555f84,0xdd555f84,0xd8555f86,
	0xdd555f86,0xd8555f88,0xdd575f88,0xd8555f8a,0xdd575f8a,0xd8555f8c,0xdd595f8c,0xd8555f8e,
	0xdd595f8e,0xd8555f90,0xdd5b5f90,0xd8555f92,0xdd5b5f92,0xd8555f94,0xdd5d5f94,0xd8555f96,
	0xdd5d5f96,0xd8555f98,0xdd5f5f98,0xd8555f9a,0xdd5f5f9a,0xd8555f9c,0xdd615f9c,0xd8555f9e,
	0xdd615f9e,0xd8555fa0,0xdd635fa0,0xd8555fa2,0xdd635fa2,0xd8555fa4,0xdd655fa4,0xd8555fa6,
	0xdd655fa6,0xd8555fa8,0xdd675fa8,0xd8555faa,0xdd675faa,0xd8555fac,0xdd695fac,0xd8555fae,
	0xdd695fae,0xd8555fb0,0xdd6b5fb0,0xd8555fb2,0xdd6b5fb2,0xd8555fb4,0xdd6d5fb4,0xd8555fb6,
	0xdd6d5fb6,0xd8555fb8,0xdd6f5fb8,0xd8555fba,0xdd6f5fba,0xd8555fbc,0xdd715fbc,0xd8555fbe,
	0xdd715fbe,0xd8555fc0,0xdd735fc0,0xd8555fc2,0xdd735fc2,0xd8555fc4,0xdd755fc4,0xd8555fc6,
	0xdd755fc6,0xd8555fc8,0xdd775fc8,0xd8555fca,0xdd775fca,0xd8555fcc,0xdd795fcc,0xd8555fce,
	0xdd795fce,0xd8555fd0,0xdd7b5fd0,0xd8555fd2,0xdd7b5fd2,0xd8555fd4,0xdd7d5fd4,0xd8555fd6,
	0xdd7d5fd6,0xd8555fd8,0xdd7f5fd8,0xd8555fda,0xdd7f5fda,0xd8555fdc,0xdd815fdc,0xd8555fde,
	0xdd815fde,0xd8555fe0,0xdd835fe0,0xd8555fe2,0xdd835fe2,0xd8555fe4,0xdd855fe4,0xd8555fe6,
	0xdd855fe6,0xd8555fe8,0xdd875fe8,0xd8555fea,0xdd875fea,0xd8555fec,0xdd895fec,0xd8555fee,
	0xdd895fee,0xd8555ff0,0xdd8b5ff0,0xd8555ff2,0xdd8b5ff2,0xd8555ff4,0xdd8d5ff4,0xd8555ff6,
	0xdd8d5ff6,0xd8555ff8,0xdd8f5ff8,0xd8555ffa,0xdd8f5ffa,0xd8555ffc,0xdd915ffc,0xd8555ffe,
	0xdd915ffe,0xd8556000,0xdd936000,0xd8556002,0xdd936002,0xd8556004,0xdd956004,0xd8556006,
	0xdd956006,0xd8556008,0xdd976008,0xd855600a,0xdd97600a,0xd855600c,0xdd99600c,0xd855600e,
	0xdd99600e,0xd8556010,0xdd9b6010,0xd8556012,0xdd9b6012,0xd8556014,0xdd9d6014,0xd8556016,
	0xdd9d6016,0xd8556018,0xdd9f6018,0xd855601a,0xdd9f601a,0xd855601c,0xdda1601c,0xd855601e,
	0xdda1601e,0xd8556020,0xdda36020,0xd8556022,0xdda36022,0xd8556024,0xdda56024,0xd8556026,
	0xdda56026,0xd8556028,0xdda76028,0xd855602a,0xdda7602a,0xd855602c,0xdda9602c,0xd855602e,
	0xdda9602e,0xd8556030,0xddab6030,0xd8556032,0xddab6032,0xd8556034,0xddad6034,0xd8556036,
	0xddad6036,0xd8556038,0xddaf6038,0xd855603a,0xddaf603a,0xd855603c,0xddb1603c,0xd855603e,
	0xddb1603e,0xd8556040,0xddb36040,0xd8556042,0xddb36042,0xd8556044,0xddb56044,0xd8556046,
	0xddb56046,0xd8556048,0xddb76048,0xd855604a,0xddb7604a,0xd855604c,0xddb9604c,0xd855604e,
	0xddb9604e,0xd8556050,0xddbb6050,0xd8556052,0xddbb6052,0xd8556054,0xddbd6054,0xd8556056,
	0xddbd6056,0xd8556058,0xddbf6058,0xd855605a,0xddbf605a,0xd855605c,0xddc1605c,0xd855605e,
	0xddc1605e,0xd8556060,0xddc36060,0xd8556062,0xddc36062,0xd8556064,0xddc56064,0xd8556066,
	0xddc56066,0xd8556068,0xddc76068,0xd855606a,0xddc7606a,0xd855606c,0xddc9606c,0xd855606e,
	0xddc9606e,0xd8556070,0xddcb6070,0xd8556072,0xddcb6072,0xd8556074,0xddcd6074,0xd8556076,
	0xddcd6076,0xd8556078,0xddcf6078,0xd855607a,0xddcf607a,0xd855607c,0xddd1607c,0xd855607e,
	0xddd1607e,0xd8556080,0xddd36080,0xd8556082,0xddd36082,0xd8556084,0xddd56084,0xd8556086,
	0xddd56086,0xd8556088,0xddd76088,0xd855608a,0xddd7608a,0xd855608c,0xddd9608c,0xd855608e,
	0xddd9608e,0xd8556090,0xdddb6090,0xd8556092,0xdddb6092,0xd8556094,0xdddd6094,0xd8556096,
	0xdddd6096,0xd8556098,0xdddf6098,0xd855609a,0xdddf609a,0xd855609c,0xdde1609c,0xd855609e,
	0xdde1609e,0xd85560a0,0xdde360a0,0xd85560a2,0xdde360a2,0xd85560a4,0xdde560a4,0xd85560a6,
	0xdde560a6,0xd85560a8,0xdde760a8,0xd85560aa,0xdde760aa,0xd85560ac,0xdde960ac,0xd85560ae,
	0xdde960ae,0xd85560b0,0xddeb60b0,0xd85560b2,0xddeb60b2,0xd85560b4,0xdded60b4,0xd85560b6,
	0xdded60b6,0xd85560b8,0xddef60b8,0xd85560ba,0xddef60ba,0xd85560bc,0xddf160bc,0xd85560be,
	0xddf160be,0xd85560c0,0xddf360c0,0xd85560c2,0xddf360c2,0xd85560c4,0xddf560c4,0xd85560c6,
	0xddf560c6,0xd85560c8,0xddf760c8,0xd85560ca,0xddf760ca,0xd85560cc,0xddf960cc,0xd85560ce,
	0xddf960ce,0xd85560d0,0xddfb60d0,0xd85560d2,0xddfb60d2,0xd85560d4,0xddfd60d4,0xd85560d6,
	0xddfd60d6,0xd85560d8,0xddff60d8,0xd85560da,0xddff60da,0xd85560dc,0xde0160dc,0xd85560de,
	0xde0160de,0xd85560e0,0xde0360e0,0xd85560e2,0xde0360e2,0xd85560e4,0xde0560e4,0xd85560e6,
	0xde0560e6,0xd85560e8,0xde0760e8,0xd85560ea,0xde0760ea,0xd85560ec,0xde0960ec,0xd85560ee,
	0xde0960ee,0xd85560f0,0xde0b60f0,0xd85560f2,0xde0b60f2,0xd85560f4,0xde0d60f4,0xd85560f6,
	0xde0d60f6,0xd85560f8,0xde0f60f8,0xd85560fa,0xde0f60fa,0xd85560fc,0xde1160fc,0xd85560fe,
	0xde1160fe,0xd8556100,0xde136100,0xd8556102,0xde136102,0xd8556104,0xde156104,0xd8556106,
	0xde156106,0xd8556108,0xde176108,0xd855610a,0xde17610a,0xd855610c,0xde19610c,0xd855610e,
	0xde19610e,0xd8556110,0xde1b6110,0xd8556112,0xde1b6112,0xd8556114,0xde1d6114,0xd8556116,
	0xde1d6116,0xd8556118,0xde1f6118,0xd855611a,0xde1f611a,0xd855611c,0xde21611c,0xd855611e,
	0xde21611e,0xd8556120,0xde236120,0xd8556122,0xde236122,0xd8556124,0xde256124,0xd8556126,
	0xde256126,0xd8556128,0xde276128,0xd855612a,0xde27612a,0xd855612c,0xde29612c,0xd855612e,
	0xde29612e,0xd8556130,0xde2b6130,0xd8556132,0xde2b6132,0xd8556134,0xde2d6134,0xd8556136,
	0xde2d6136,0xd8556138,0xde2f6138,0xd855613a,0xde2f613a,0xd855613c,0xde31613c,0xd855613e,
	0xde31613e,0xd8556140,0xde336140,0xd8556142,0xde336142,0xd8556144,0xde356144,0xd8556146,
	0xde356146,0xd8556148,0xde376148,0xd855614a,0xde37614a,0xd855614c,0xde39614c,0xd855614e,
	0xde39614e,0xd8556150,0xde3b6150,0xd8556152,0xde3b6152,0xd8556154,0xde3d6154,0xd8556156,
	0xde3d6156,0xd8556158,0xde3f6158,0xd855615a,0xde3f615a,0xd855615c,0xde41615c,0xd855615e,
	0xde41615e,0xd8556160,0xde436160,0xd8556162,0xde436162,0xd8556164,0xde456164,0xd8556166,
	0xde456166,0xd8556168,0xde476168,0xd855616a,0xde47616a,0xd855616c,0xde49616c,0xd855616e,
	0xde49616e,0xd8556170,0xde4b6170,0xd8556172,0xde4b6172,0xd8556174,0xde4d6174,0xd8556176,
	0xde4d6176,0xd8556178,0xde4f6178,0xd855617a,0xde4f617a,0xd855617c,0xde51617c,0xd855617e,
	0xde51617e,0xd8556180,0xde536180,0xd8556182,0xde536182,0xd8556184,0xde556184,0xd8556186,
	0xde556186,0xd8556188,0xde576188,0xd855618a,0xde57618a,0xd855618c,0xde59618c,0xd855618e,
	0xde59618e,0xd8556190,0xde5b6190,0xd8556192,0xde5b6192,0xd8556194,0xde5d6194,0xd8556196,
	0xde5d6196,0xd8556198,0xde5f6198,0xd855619a,0xde5f619a,0xd855619c,0xde61619c,0xd855619e,
	0xde61619e,0xd85561a0,0xde6361a0,0xd85561a2,0xde6361a2,0xd85561a4,0xde6561a4,0xd85561a6,
	0xde6561a6,0xd85561a8,0xde6761a8,0xd85561aa,0xde6761aa,0xd85561ac,0xde6961ac,0xd85561ae,
	0xde6961ae,0xd85561b0,0xde6b61b0,0xd85561b2,0xde6b61b2,0xd85561b4,0xde6d61b4,0xd85561b6,
	0xde6d61b6,0xd85561b8,0xde6f61b8,0xd85561ba,0xde6f61ba,0xd85561bc,0xde7161bc,0xd85561be,
	0xde7161be,0xd85561c0,0xde7361c0,0xd85561c2,0xde7361c2,0xd85561c4,0xde7561c4,0xd85561c6,
	0xde7561c6,0xd85561c8,0xde7761c8,0xd85561ca,0xde7761ca,0xd85561cc,0xde7961cc,0xd85561ce,
	0xde7961ce,0xd85561d0,0xde7b61d0,0xd85561d2,0xde7b61d2,0xd85561d4,0xde7d61d4,0xd85561d6,
	0xde7d61d6,0xd85561d8,0xde7f61d8,0xd85561da,0xde7f61da,0xd85561dc,0xde8161dc,0xd85561de,
	0xde8161de,0xd85561e0,0xde8361e0,0xd85561e2,0xde8361e2,0xd85561e4,0xde8561e4,0xd85561e6,
	0xde8561e6,0xd85561e8,0xde8761e8,0xd85561ea,0xde8761ea,0xd85561ec,0xde8961ec,0xd85561ee,
	0xde8961ee,0xd85561f0,0xde8b61f0,0xd85561f2,0xde8b61f2,0xd85561f4,0xde8d61f4,0xd85561f6,
	0xde8d61f6,0xd85561f8,0xde8f61f8,0xd85561fa,0xde8f61fa,0xd85561fc,0xde9161fc,0xd85561fe,
	0xde9161fe,0xd8556200,0xde936200,0xd8556202,0xde936202,0xd8556204,0xde956204,0xd8556206,
	0xde956206,0xd8556208,0xde976208,0xd855620a,0xde97620a,0xd855620c,0xde99620c,0xd855620e,
	0xde99620e,0xd8556210,0xde9b6210,0xd8556212,0xde9b6212,0xd8556214,0xde9d6214,0xd8556216,
	0xde9d6216,0xd8556218,0xde9f6218,0xd855621a,0xde9f621a,0xd855621c,0xdea1621c,0xd855621e,
	0xdea1621e,0xd8556220,0xdea36220,0xd8556222,0xdea36222,0xd8556224,0xdea56224,0xd8556226,
	0xdea56226,0xd8556228,0xdea76228,0xd855622a,0xdea7622a,0xd855622c,0xdea9622c,0xd855622e,
	0xdea9622e,0xd8556230,0xdeab6230,0xd8556232,0xdeab6232,0xd8556234,0xdead6234,0xd8556236,
	0xdead6236,0xd8556238,0xdeaf6238,0xd855623a,0xdeaf623a,0xd855623c,0xdeb1623c,0xd855623e,
	0xdeb1623e,0xd8556240,0xdeb36240,0xd8556242,0xdeb36242,0xd8556244,0xdeb56244,0xd8556246,
	0xdeb56246,0xd8556248,0xdeb76248,0xd855624a,0xdeb7624a,0xd855624c,0xdeb9624c,0xd855624e,
	0xdeb9624e,0xd8556250,0xdebb6250,0xd8556252,0xdebb6252,0xd8556254,0xdebd6254,0xd8556256,
	0xdebd6256,0xd8556258,0xdebf6258,0xd855625a,0xdebf625a,0xd855625c,0xdec1625c,0xd855625e,
	0xdec1625e,0xd8556260,0xdec36260,0xd8556262,0xdec36262,0xd8556264,0xdec56264,0xd8556266,
	0xdec56266,0xd8556268,0xdec76268,0xd855626a,0xdec7626a,0xd855626c,0xdec9626c,0xd855626e,
	0xdec9626e,0xd8556270,0xdecb6270,0xd8556272,0xdecb6272,0xd8556274,0xdecd6274,0xd8556276,
	0xdecd6276,0xd8556278,0xdecf6278,0xd855627a,0xdecf627a,0xd855627c,0xded1627c,0xd855627e,
	0xded1627e,0xd8556280,0xded36280,0xd8556282,0xded36282,0xd8556284,0xded56284,0xd8556286,
	0xded56286,0xd8556288,0xded76288,0xd855628a,0xded7628a,0xd855628c,0xded9628c,0xd855628e,
	0xded9628e,0xd8556290,0xdedb6290,0xd8556292,0xdedb6292,0xd8556294,0xdedd6294,0xd8556296,
	0xdedd6296,0xd8556298,0xdedf6298,0xd855629a,0xdedf629a,0xd855629c,0xdee1629c,0xd855629e,
	0xdee1629e,0xd85562a0,0xdee362a0,0xd85562a2,0xdee362a2,0xd85562a4,0xdee562a4,0xd85562a6,
	0xdee562a6,0xd85562a8,0xdee762a8,0xd85562aa,0xdee762aa,0xd85562ac,0xdee962ac,0xd85562ae,
	0xdee962ae,0xd85562b0,0xdeeb62b0,0xd85562b2,0xdeeb62b2,0xd85562b4,0xdeed62b4,0xd85562b6,
	0xdeed62b6,0xd85562b8,0xdeef62b8,0xd85562ba,0xdeef62ba,0xd85562bc,0xdef162bc,0xd85562be,
	0xdef162be,0xd85562c0,0xdef362c0,0xd85562c2,0xdef362c2,0xd85562c4,0xdef562c4,0xd85562c6,
	0xdef562c6,0xd85562c8,0xdef762c8,0xd85562ca,0xdef762ca,0xd85562cc,0xdef962cc,0xd85562ce,
	0xdef962ce,0xd85562d0,0xdefb62d0,0xd85562d2,0xdefb62d2,0xd85562d4,0xdefd62d4,0xd85562d6,
	0xdefd62d6,0xd85562d8,0xdeff62d8,0xd85562da,0xdeff62da,0xd85562dc,0xdf0162dc,0xd85562de,
	0xdf0162de,0xd85562e0,0xdf0362e0,0xd85562e2,0xdf0362e2,0xd85562e4,0xdf0562e4,0xd85562e6,
	0xdf0562e6,0xd85562e8,0xdf0762e8,0xd85562ea,0xdf0762ea,0xd85562ec,0xdf0962ec,0xd85562ee,
	0xdf0962ee,0xd85562f0,0xdf0b62f0,0xd85562f2,0xdf0b62f2,0xd85562f4,0xdf0d62f4,0xd85562f6,
	0xdf0d62f6,0xd85562f8,0xdf0f62f8,0xd85562fa,0xdf0f62fa,0xd85562fc,0xdf1162fc,0xd85562fe,
	0xdf1162fe,0xd8556300,0xdf136300,0xd8556302,0xdf136302,0xd8556304,0xdf156304,0xd8556306,
	0xdf156306,0xd8556308,0xdf176308,0xd855630a,0xdf17630a,0xd855630c,0xdf19630c,0xd855630e,
	0xdf19630e,0xd8556310,0xdf1b6310,0xd8556312,0xdf1b6312,0xd8556314,0xdf1d6314,0xd8556316,
	0xdf1d6316,0xd8556318,0xdf1f6318,0xd855631a,0xdf1f631a,0xd855631c,0xdf21631c,0xd855631e,
	0xdf21631e,0xd8556320,0xdf236320,0xd8556322,0xdf236322,0xd8556324,0xdf256324,0xd8556326,
	0xdf256326,0xd8556328,0xdf276328,0xd855632a,0xdf27632a,0xd855632c,0xdf29632c,0xd855632e,
	0xdf29632e,0xd8556330,0xdf2b6330,0xd8556332,0xdf2b6332,0xd8556334,0xdf2d6334,0xd8556336,
	0xdf2d6336,0xd8556338,0xdf2f6338,0xd855633a,0xdf2f633a,0xd855633c,0xdf31633c,0xd855633e,
	0xdf31633e,0xd8556340,0xdf336340,0xd8556342,0xdf336342,0xd8556344,0xdf356344,0xd8556346,
	0xdf356346,0xd8556348,0xdf376348,0xd855634a,0xdf37634a,0xd855634c,0xdf39634c,0xd855634e,
	0xdf39634e,0xd8556350,0xdf3b6350,0xd8556352,0xdf3b6352,0xd8556354,0xdf3d6354,0xd8556356,
	0xdf3d6356,0xd8556358,0xdf3f6358,0xd855635a,0xdf3f635a,0xd855635c,0xdf41635c,0xd855635e,
	0xdf41635e,0xd8556360,0xdf436360,0xd8556362,0xdf436362,0xd8556364,0xdf456364,0xd8556366,
	0xdf456366,0xd8556368,0xdf476368,0xd855636a,0xdf47636a,0xd855636c,0xdf49636c,0xd855636e,
	0xdf49636e,0xd8556370,0xdf4b6370,0xd8556372,0xdf4b6372,0xd8556374,0xdf4d6374,0xd8556376,
	0xdf4d6376,0xd8556378,0xdf4f6378,0xd855637a,0xdf4f637a,0xd855637c,0xdf51637c,0xd855637e,
	0xdf51637e,0xd8556380,0xdf536380,0xd8556382,0xdf536382,0xd8556384,0xdf556384,0xd8556386,
	0xdf556386,0xd8556388,0xdf576388,0xd855638a,0xdf57638a,0xd855638c,0xdf59638c,0xd855638e,
	0xdf59638e,0xd8556390,0xdf5b6390,0xd8556392,0xdf5b6392,0xd8556394,0xdf5d6394,0xd8556396,
	0xdf5d6396,0xd8556398,0xdf5f6398,0xd855639a,0xdf5f639a,0xd855639c,0xdf61639c,0xd855639e,
	0xdf61639e,0xd85563a0,0xdf6363a0,0xd85563a2,0xdf6363a2,0xd85563a4,0xdf6563a4,0xd85563a6,
	0xdf6563a6,0xd85563a8,0xdf6763a8,0xd85563aa,0xdf6763aa,0xd85563ac,0xdf6963ac,0xd85563ae,
	0xdf6963ae,0xd85563b0,0xdf6b63b0,0xd85563b2,0xdf6b63b2,0xd85563b4,0xdf6d63b4,0xd85563b6,
	0xdf6d63b6,0xd85563b8,0xdf6f63b8,0xd85563ba,0xdf6f63ba,0xd85563bc,0xdf7163bc,0xd85563be,
	0xdf7163be,0xd85563c0,0xdf7363c0,0xd85563c2,0xdf7363c2,0xd85563c4,0xdf7563c4,0xd85563c6,
	0xdf7563c6,0xd85563c8,0xdf7763c8,0xd85563ca,0xdf7763ca,0xd85563cc,0xdf7963cc,0xd85563ce,
	0xdf7963ce,0xd85563d0,0xdf7b63d0,0xd85563d2,0xdf7b63d2,0xd85563d4,0xdf7d63d4,0xd85563d6,
	0xdf7d63d6,0xd85563d8,0xdf7f63d8,0xd85563da,0xdf7f63da,0xd85563dc,0xdf8163dc,0xd85563de,
	0xdf8163de,0xd85563e0,0xdf8363e0,0xd85563e2,0xdf8363e2,0xd85563e4,0xdf8563e4,0xd85563e6,
	0xdf8563e6,0xd85563e8,0xdf8763e8,0xd85563ea,0xdf8763ea,0xd85563ec,0xdf8963ec,0xd85563ee,
	0xdf8963ee,0xd85563f0,0xdf8b63f0,0xd85563f2,0xdf8b63f2,0xd85563f4,0xdf8d63f4,0xd85563f6,
	0xdf8d63f6,0xd85563f8,0xdf8f63f8,0xd85563fa,0xdf8f63fa,0xd85563fc,0xdf9163fc,0xd85563fe,
	0xdf9163fe,0xd8556400,0xdf936400,0xd8556402,0xdf936402,0xd8556404,0xdf956404,0xd8556406,
	0xdf956406,0xd8556408,0xdf976408,0xd855640a,0xdf97640a,0xd855640c,0xdf99640c,0xd855640e,
	0xdf99640e,0xd8556410,0xdf9b6410,0xd8556412,0xdf9b6412,0xd8556414,0xdf9d6414,0xd8556416,
	0xdf9d6416,0xd8556418,0xdf9f6418,0xd855641a,0xdf9f641a,0xd855641c,0xdfa1641c,0xd855641e,
	0xdfa1641e,0xd8556420,0xdfa36420,0xd8556422,0xdfa36422,0xd8556424,0xdfa56424,0xd8556426,
	0xdfa56426,0xd8556428,0xdfa76428,0xd855642a,0xdfa7642a,0xd855642c,0xdfa9642c,0xd855642e,
	0xdfa9642e,0xd8556430,0xdfab6430,0xd8556432,0xdfab6432,0xd8556434,0xdfad6434,0xd8556436,
	0xdfad6436,0xd8556438,0xdfaf6438,0xd855643a,0xdfaf643a,0xd855643c,0xdfb1643c,0xd855643e,
	0xdfb1643e,0xd8556440,0xdfb36440,0xd8556442,0xdfb36442,0xd8556444,0xdfb56444,0xd8556446,
	0xdfb56446,0xd8556448,0xdfb76448,0xd855644a,0xdfb7644a,0xd855644c,0xdfb9644c,0xd855644e,
	0xdfb9644e,0xd8556450,0xdfbb6450,0xd8556452,0xdfbb6452,0xd8556454,0xdfbd6454,0xd8556456,
	0xdfbd6456,0xd8556458,0xdfbf6458,0xd855645a,0xdfbf645a,0xd855645c,0xdfc1645c,0xd855645e,
	0xdfc1645e,0xd8556460,0xdfc36460,0xd8556462,0xdfc36462,0xd8556464,0xdfc56464,0xd8556466,
	0xdfc56466,0xd8556468,0xdfc76468,0xd855646a,0xdfc7646a,0xd855646c,0xdfc9646c,0xd855646e,
	0xdfc9646e,0xd8556470,0xdfcb6470,0xd8556472,0xdfcb6472,0xd8556474,0xdfcd6474,0xd8556476,
	0xdfcd6476,0xd8556478,0xdfcf6478,0xd855647a,0xdfcf647a,0xd855647c,0xdfd1647c,0xd855647e,
	0xdfd1647e,0xd8556480,0xdfd36480,0xd8556482,0xdfd36482,0xd8556484,0xdfd56484,0xd8556486,
	0xdfd56486,0xd8556488,0xdfd76488,0xd855648a,0xdfd7648a,0xd855648c,0xdfd9648c,0xd855648e,
	0xdfd9648e,0xd8556490,0xdfdb6490,0xd8556492,0xdfdb6492,0xd8556494,0xdfdd6494,0xd8556496,
	0xdfdd6496,0xd8556498,0xdfdf6498,0xd855649a,0xdfdf649a,0xd855649c,0xdfe1649c,0xd855649e,
	0xdfe1649e,0xd85564a0,0xdfe364a0,0xd85564a2,0xdfe364a2,0xd85564a4,0xdfe564a4,0xd85564a6,
	0xdfe564a6,0xd85564a8,0xdfe764a8,0xd85564aa,0xdfe764aa,0xd85564ac,0xdfe964ac,0xd85564ae,
	0xdfe964ae,0xd85564b0,0xdfeb64b0,0xd85564b2,0xdfeb64b2,0xd85564b4,0xdfed64b4,0xd85564b6,
	0xdfed64b6,0xd85564b8,0xdfef64b8,0xd85564ba,0xdfef64ba,0xd85564bc,0xdff164bc,0xd85564be,
	0xdff164be,0xd85564c0,0xdff364c0,0xd85564c2,0xdff364c2,0xd85564c4,0xdff564c4,0xd85564c6,
	0xdff564c6,0xd85564c8,0xdff764c8,0xd85564ca,0xdff764ca,0xd85564cc,0xdff964cc,0xd85564ce,
	0xdff964ce,0xd85564d0,0xdffb64d0,0xd85564d2,0xdffb64d2,0xd85564d4,0xdffd64d4,0xd85564d6,
	0xdffd64d6,0xd85564d8,0xdfff64d8,0xd85564da,0xdfff64da,0xd85564dc,0xdc0164dc,0xd85564de,
	0xdc0164de,0xd85564e0,0xdc0364e0,0xd85564e2,0xdc0364e2,0xd85564e4,0xdc0564e4,0xd85564e6,
	0xdc0564e6,0xd85564e8,0xdc0764e8,0xd85564ea,0xdc0764ea,0xd85564ec,0xdc0964ec,0xd85564ee,
	0xdc0964ee,0xd85564f0,0xdc0b64f0,0xd85564f2,0xdc0b64f2,0xd85564f4,0xdc0d64f4,0xd85564f6,
	0xdc0d64f6,0xd85564f8,0xdc0f64f8,0xd85564fa,0xdc0f64fa,0xd85564fc,0xdc1164fc,0xd85564fe,
	0xdc1164fe,0xd8556500,0xdc136500,0xd8556502,0xdc136502,0xd8556504,0xdc156504,0xd8556506,
	0xdc156506,0xd8556508,0xdc176508,0xd855650a,0xdc17650a,0xd855650c,0xdc19650c,0xd855650e,
	0xdc19650e,0xd8556510,0xdc1b6510,0xd8556512,0xdc1b6512,0xd8556514,0xdc1d6514,0xd8556516,
	0xdc1d6516,0xd8556518,0xdc1f6518,0xd855651a,0xdc1f651a,0xd855651c,0xdc21651c,0xd855651e,
	0xdc21651e,0xd8556520,0xdc236520,0xd8556522,0xdc236522,0xd8556524,0xdc256524,0xd8556526,
	0xdc256526,0xd8556528,0xdc276528,0xd855652a,0xdc27652a,0xd855652c,0xdc29652c,0xd855652e,
	0xdc29652e,0xd8556530,0xdc2b6530,0xd8556532,0xdc2b6532,0xd8556534,0xdc2d6534,0xd8556536,
	0xdc2d6536,0xd8556538,0xdc2f6538,0xd855653a,0xdc2f653a,0xd855653c,0xdc31653c,0xd855653e,
	0xdc31653e,0xd8556540,0xdc336540,0xd8556542,0xdc336542,0xd8556544,0xdc356544,0xd8556546,
	0xdc356546,0xd8556548,0xdc376548,0xd855654a,0xdc37654a,0xd855654c,0xdc39654c,0xd855654e,
	0xdc39654e,0xd8556550,0xdc3b6550,0xd8556552,0xdc3b6552,0xd8556554,0xdc3d6554,0xd8556556,
	0xdc3d6556,0xd8556558,0xdc3f6558,0xd855655a,0xdc3f655a,0xd855655c,0xdc41655c,0xd855655e,
	0xdc41655e,0xd8556560,0xdc436560,0xd8556562,0xdc436562,0xd8556564,0xdc456564,0xd8556566,
	0xdc456566,0xd8556568,0xdc476568,0xd855656a,0xdc47656a,0xd855656c,0xdc49656c,0xd855656e,
	0xdc49656e,0xd8556570,0xdc4b6570,0xd8556572,0xdc4b6572,0xd8556574,0xdc4d6574,0xd8556576,
	0xdc4d6576,0xd8556578,0xdc4f6578,0xd855657a,0xdc4f657a,0xd855657c,0xdc51657c,0xd855657e,
	0xdc51657e,0xd8556580,0xdc536580,0xd8556582,0xdc536582,0xd8556584,0xdc556584,0xd8556586,
	0xdc556586,0xd8556588,0xdc576588,0xd855658a,0xdc57658a,0xd855658c,0xdc59658c,0xd855658e,
	0xdc59658e,0xd8556590,0xdc5b6590,0xd8556592,0xdc5b6592,0xd8556594,0xdc5d6594,0xd8556596,
	0xdc5d6596,0xd8556598,0xdc5f6598,0xd855659a,0xdc5f659a,0xd855659c,0xdc61659c,0xd855659e,
	0xdc61659e,0xd85565a0,0xdc6365a0,0xd85565a2,0xdc6365a2,0xd85565a4,0xdc6565a4,0xd85565a6,
	0xdc6565a6,0xd85565a8,0xdc6765a8,0xd85565aa,0xdc6765aa,0xd85565ac,0xdc6965ac,0xd85565ae,
	0xdc6965ae,0xd85565b0,0xdc6b65b0,0xd85565b2,0xdc6b65b2,0xd85565b4,0xdc6d65b4,0xd85565b6,
	0xdc6d65b6,0xd85565b8,0xdc6f65b8,0xd85565ba,0xdc6f65ba,0xd85565bc,0xdc7165bc,0xd85565be,
	0xdc7165be,0xd85565c0,0xdc7365c0,0xd85565c2,0xdc7365c2,0xd85565c4,0xdc7565c4,0xd85565c6,
	0xdc7565c6,0xd85565c8,0xdc7765c8,0xd85565ca,0xdc7765ca,0xd85565cc,0xdc7965cc,0xd85565ce,
	0xdc7965ce,0xd85565d0,0xdc7b65d0,0xd85565d2,0xdc7b65d2,0xd85565d4,0xdc7d65d4,0xd85565d6,
	0xdc7d65d6,0xd85565d8,0xdc7f65d8,0xd85565da,0xdc7f65da,0xd85565dc,0xdc8165dc,0xd85565de,
	0xdc8165de,0xd85565e0,0xdc8365e0,0xd85565e2,0xdc8365e2,0xd85565e4,0xdc8565e4,0xd85565e6,
	0xdc8565e6,0xd85565e8,0xdc8765e8,0xd85565ea,0xdc8765ea,0xd85565ec,0xdc8965ec,0xd85565ee,
	0xdc8965ee,0xd85565f0,0xdc8b65f0,0xd85565f2,0xdc8b65f2,0xd85565f4,0xdc8d65f4,0xd85565f6,
	0xdc8d65f6,0xd85565f8,0xdc8f65f8,0xd85565fa,0xdc8f65fa,0xd85565fc,0xdc9165fc,0xd85565fe,
	0xdc9165fe,0xd8556600,0xdc936600,0xd8556602,0xdc936602,0xd8556604,0xdc956604,0xd8556606,
	0xdc956606,0xd8556608,0xdc976608,0xd855660a,0xdc97660a,0xd855660c,0xdc99660c,0xd855660e,
	0xdc99660e,0xd8556610,0xdc9b6610,0xd8556612,0xdc9b6612,0xd8556614,0xdc9d6614,0xd8556616,
	0xdc9d6616,0xd8556618,0xdc9f6618,0xd855661a,0xdc9f661a,0xd855661c,0xdca1661c,0xd855661e,
	0xdca1661e,0xd8556620,0xdca36620,0xd8556622,0xdca36622,0xd8556624,0xdca56624,0xd8556626,
	0xdca56626,0xd8556628,0xdca76628,0xd855662a,0xdca7662a,0xd855662c,0xdca9662c,0xd855662e,
	0xdca9662e,0xd8556630,0xdcab6630,0xd8556632,0xdcab6632,0xd8556634,0xdcad6634,0xd8556636,
	0xdcad6636,0xd8556638,0xdcaf6638,0xd855663a,0xdcaf663a,0xd855663c,0xdcb1663c,0xd855663e,
	0xdcb1663e,0xd8556640,0xdcb36640,0xd8556642,0xdcb36642,0xd8556644,0xdcb56644,0xd8556646,
	0xdcb56646,0xd8556648,0xdcb76648,0xd855664a,0xdcb7664a,0xd855664c,0xdcb9664c,0xd855664e,
	0xdcb9664e,0xd8556650,0xdcbb6650,0xd8556652,0xdcbb6652,0xd8556654,0xdcbd6654,0xd8556656,
	0xdcbd6656,0xd8556658,0xdcbf6658,0xd855665a,0xdcbf665a,0xd855665c,0xdcc1665c,0xd855665e,
	0xdcc1665e,0xd8556660,0xdcc36660,0xd8556662,0xdcc36662,0xd8556664,0xdcc56664,0xd8556666,
	0xdcc56666,0xd8556668,0xdcc76668,0xd855666a,0xdcc7666a,0xd855666c,0xdcc9666c,0xd855666e,
	0xdcc9666e,0xd8556670,0xdccb6670,0xd8556672,0xdccb6672,0xd8556674,0xdccd6674,0xd8556676,
	0xdccd6676,0xd8556678,0xdccf6678,0xd855667a,0xdccf667a,0xd855667c,0xdcd1667c,0xd855667e,
	0xdcd1667e,0xd8556680,0xdcd36680,0xd8556682,0xdcd36682,0xd8556684,0xdcd56684,0xd8556686,
	0xdcd56686,0xd8556688,0xdcd76688,0xd855668a,0xdcd7668a,0xd855668c,0xdcd9668c,0xd855668e,
	0xdcd9668e,0xd8556690,0xdcdb6690,0xd8556692,0xdcdb6692,0xd8556694,0xdcdd6694,0xd8556696,
	0xdcdd6696,0xd8556698,0xdcdf6698,0xd855669a,0xdcdf669a,0xd855669c,0xdce1669c,0xd855669e,
	0xdce1669e,0xd85566a0,0xdce366a0,0xd85566a2,0xdce366a2,0xd85566a4,0xdce566a4,0xd85566a6,
	0xdce566a6,0xd85566a8,0xdce766a8,0xd85566aa,0xdce766aa,0xd85566ac,0xdce966ac,0xd85566ae,
	0xdce966ae,0xd85566b0,0xdceb66b0,0xd85566b2,0xdceb66b2,0xd85566b4,0xdced66b4,0xd85566b6,
	0xdced66b6,0xd85566b8,0xdcef66b8,0xd85566ba,0xdcef66ba,0xd85566bc,0xdcf166bc,0xd85566be,
	0xdcf166be,0xd85566c0,0xdcf366c0,0xd85566c2,0xdcf366c2,0xd85566c4,0xdcf566c4,0xd85566c6,
	0xdcf566c6,0xd85566c8,0xdcf766c8,0xd85566ca,0xdcf766ca,0xd85566cc,0xdcf966cc,0xd85566ce,
	0xdcf966ce,0xd85566d0,0xdcfb66d0,0xd85566d2,0xdcfb66d2,0xd85566d4,0xdcfd66d4,0xd85566d6,
	0xdcfd66d6,0xd85566d8,0xdcff66d8,0xd85566da,0xdcff66da,0xd85566dc,0xdd0166dc,0xd85566de,
	0xdd0166de,0xd85566e0,0xdd0366e0,0xd85566e2,0xdd0366e2,0xd85566e4,0xdd0566e4,0xd85566e6,
	0xdd0566e6,0xd85566e8,0xdd0766e8,0xd85566ea,0xdd0766ea,0xd85566ec,0xdd0966ec,0xd85566ee,
	0xdd0966ee,0xd85566f0,0xdd0b66f0,0xd85566f2,0xdd0b66f2,0xd85566f4,0xdd0d66f4,0xd85566f6,
	0xdd0d66f6,0xd85566f8,0xdd0f66f8,0xd85566fa,0xdd0f66fa,0xd85566fc,0xdd1166fc,0xd85566fe,
	0xdd1166fe,0xd8556700,0xdd136700,0xd8556702,0xdd136702,0xd8556704,0xdd156704,0xd8556706,
	0xdd156706,0xd8556708,0xdd176708,0xd855670a,0xdd17670a,0xd855670c,0xdd19670c,0xd855670e,
	0xdd19670e,0xd8556710,0xdd1b6710,0xd8556712,0xdd1b6712,0xd8556714,0xdd1d6714,0xd8556716,
	0xdd1d6716,0xd8556718,0xdd1f6718,0xd855671a,0xdd1f671a,0xd855671c,0xdd21671c,0xd855671e,
	0xdd21671e,0xd8556720,0xdd236720,0xd8556722,0xdd236722,0xd8556724,0xdd256724,0xd8556726,
	0xdd256726,0xd8556728,0xdd276728,0xd855672a,0xdd27672a,0xd855672c,0xdd29672c,0xd855672e,
	0xdd29672e,0xd8556730,0xdd2b6730,0xd8556732,0xdd2b6732,0xd8556734,0xdd2d6734,0xd8556736,
	0xdd2d6736,0xd8556738,0xdd2f6738,0xd855673a,0xdd2f673a,0xd855673c,0xdd31673c,0xd855673e,
	0xdd31673e,0xd8556740,0xdd336740,0xd8556742,0xdd336742,0xd8556744,0xdd356744,0xd8556746,
	0xdd356746,0xd8556748,0xdd376748,0xd855674a,0xdd37674a,0xd855674c,0xdd39674c,0xd855674e,
	0xdd39674e,0xd8556750,0xdd3b6750,0xd8556752,0xdd3b6752,0xd8556754,0xdd3d6754,0xd8556756,
	0xdd3d6756,0xd8556758,0xdd3f6758,0xd855675a,0xdd3f675a,0xd855675c,0xdd41675c,0xd855675e,
	0xdd41675e,0xd8556760,0xdd436760,0xd8556762,0xdd436762,0xd8556764,0xdd456764,0xd8556766,
	0xdd456766,0xd8556768,0xdd476768,0xd855676a,0xdd47676a,0xd855676c,0xdd49676c,0xd855676e,
	0xdd49676e,0xd8556770,0xdd4b6770,0xd8556772,0xdd4b6772,0xd8556774,0xdd4d6774,0xd8556776,
	0xdd4d6776,0xd8556778,0xdd4f6778,0xd855677a,0xdd4f677a,0xd855677c,0xdd51677c,0xd855677e,
	0xdd51677e,0xd8556780,0xdd536780,0xd8556782,0xdd536782,0xd8556784,0xdd556784,0xd8556786,
	0xdd556786,0xd8556788,0xdd576788,0xd855678a,0xdd57678a,0xd855678c,0xdd59678c,0xd855678e,
	0xdd59678e,0xd8556790,0xdd5b6790,0xd8556792,0xdd5b6792,0xd8556794,0xdd5d6794,0xd8556796,
	0xdd5d6796,0xd8556798,0xdd5f6798,0xd855679a,0xdd5f679a,0xd855679c,0xdd61679c,0xd855679e,
	0xdd61679e,0xd85567a0,0xdd6367a0,0xd85567a2,0xdd6367a2,0xd85567a4,0xdd6567a4,0xd85567a6,
	0xdd6567a6,0xd85567a8,0xdd6767a8,0xd85567aa,0xdd6767aa,0xd85567ac,0xdd6967ac,0xd85567ae,
	0xdd6967ae,0xd85567b0,0xdd6b67b0,0xd85567b2,0xdd6b67b2,0xd85567b4,0xdd6d67b4,0xd85567b6,
	0xdd6d67b6,0xd85567b8,0xdd6f67b8,0xd85567ba,0xdd6f67ba,0xd85567bc,0xdd7167bc,0xd85567be,
	0xdd7167be,0xd85567c0,0xdd7367c0,0xd85567c2,0xdd7367c2,0xd85567c4,0xdd7567c4,0xd85567c6,
	0xdd7567c6,0xd85567c8,0xdd7767c8,0xd85567ca,0xdd7767ca,0xd85567cc,0xdd7967cc,0xd85567ce,
	0xdd7967ce,0xd85567d0,0xdd7b67d0,0xd85567d2,0xdd7b67d2,0xd85567d4,0xdd7d67d4,0xd85567d6,
	0xdd7d67d6,0xd85567d8,0xdd7f67d8,0xd85567da,0xdd7f67da,0xd85567dc,0xdd8167dc,0xd85567de,
	0xdd8167de,0xd85567e0,0xdd8367e0,0xd85567e2,0xdd8367e2,0xd85567e4,0xdd8567e4,0xd85567e6,
	0xdd8567e6,0xd85567e8,0xdd8767e8,0xd85567ea,0xdd8767ea,0xd85567ec,0xdd8967ec,0xd85567ee,
	0xdd8967ee,0xd85567f0,0xdd8b67f0,0xd85567f2,0xdd8b67f2,0xd85567f4,0xdd8d67f4,0xd85567f6,
	0xdd8d67f6,0xd85567f8,0xdd8f67f8,0xd85567fa,0xdd8f67fa,0xd85567fc,0xdd9167fc,0xd85567fe,
	0xdd9167fe,0xd8556800,0xdd936800,0xd8556802,0xdd936802,0xd8556804,0xdd956804,0xd8556806,
	0xdd956806,0xd8556808,0xdd976808,0xd855680a,0xdd97680a,0xd855680c,0xdd99680c,0xd855680e,
	0xdd99680e,0xd8556810,0xdd9b6810,0xd8556812,0xdd9b6812,0xd8556814,0xdd9d6814,0xd8556816,
	0xdd9d6816,0xd8556818,0xdd9f6818,0xd855681a,0xdd9f681a,0xd855681c,0xdda1681c,0xd855681e,
	0xdda1681e,0xd8556820,0xdda36820,0xd8556822,0xdda36822,0xd8556824,0xdda56824,0xd8556826,
	0xdda56826,0xd8556828,0xdda76828,0xd855682a,0xdda7682a,0xd855682c,0xdda9682c,0xd855682e,
	0xdda9682e,0xd8556830,0xddab6830,0xd8556832,0xddab6832,0xd8556834,0xddad6834,0xd8556836,
	0xddad6836,0xd8556838,0xddaf6838,0xd855683a,0xddaf683a,0xd855683c,0xddb1683c,0xd855683e,
	0xddb1683e,0xd8556840,0xddb36840,0xd8556842,0xddb36842,0xd8556844,0xddb56844,0xd8556846,
	0xddb56846,0xd8556848,0xddb76848,0xd855684a,0xddb7684a,0xd855684c,0xddb9684c,0xd855684e,
	0xddb9684e,0xd8556850,0xddbb6850,0xd8556852,0xddbb6852,0xd8556854,0xddbd6854,0xd8556856,
	0xddbd6856,0xd8556858,0xddbf6858,0xd855685a,0xddbf685a,0xd855685c,0xddc1685c,0xd855685e,
	0xddc1685e,0xd8556860,0xddc36860,0xd8556862,0xddc36862,0xd8556864,0xddc56864,0xd8556866,
	0xddc56866,0xd8556868,0xddc76868,0xd855686a,0xddc7686a,0xd855686c,0xddc9686c,0xd855686e,
	0xddc9686e,0xd8556870,0xddcb6870,0xd8556872,0xddcb6872,0xd8556874,0xddcd6874,0xd8556876,
	0xddcd6876,0xd8556878,0xddcf6878,0xd855687a,0xddcf687a,0xd855687c,0xddd1687c,0xd855687e,
	0xddd1687e,0xd8556880,0xddd36880,0xd8556882,0xddd36882,0xd8556884,0xddd56884,0xd8556886,
	0xddd56886,0xd8556888,0xddd76888,0xd855688a,0xddd7688a,0xd855688c,0xddd9688c,0xd855688e,
	0xddd9688e,0xd8556890,0xdddb6890,0xd8556892,0xdddb6892,0xd8556894,0xdddd6894,0xd8556896,
	0xdddd6896,0xd8556898,0xdddf6898,0xd855689a,0xdddf689a,0xd855689c,0xdde1689c,0xd855689e,
	0xdde1689e,0xd85568a0,0xdde368a0,0xd85568a2,0xdde368a2,0xd85568a4,0xdde568a4,0xd85568a6,
	0xdde568a6,0xd85568a8,0xdde768a8,0xd85568aa,0xdde768aa,0xd85568ac,0xdde968ac,0xd85568ae,
	0xdde968ae,0xd85568b0,0xddeb68b0,0xd85568b2,0xddeb68b2,0xd85568b4,0xdded68b4,0xd85568b6,
	0xdded68b6,0xd85568b8,0xddef68b8,0xd85568ba,0xddef68ba,0xd85568bc,0xddf168bc,0xd85568be,
	0xddf168be,0xd85568c0,0xddf368c0,0xd85568c2,0xddf368c2,0xd85568c4,0xddf568c4,0xd85568c6,
	0xddf568c6,0xd85568c8,0xddf768c8,0xd85568ca,0xddf768ca,0xd85568cc,0xddf968cc,0xd85568ce,
	0xddf968ce,0xd85568d0,0xddfb68d0,0xd85568d2,0xddfb68d2,0xd85568d4,0xddfd68d4,0xd85568d6,
	0xddfd68d6,0xd85568d8,0xddff68d8,0xd85568da,0xddff68da,0xd85568dc,0xde0168dc,0xd85568de,
	0xde0168de,0xd85568e0,0xde0368e0,0xd85568e2,0xde0368e2,0xd85568e4,0xde0568e4,0xd85568e6,
	0xde0568e6,0xd85568e8,0xde0768e8,0xd85568ea,0xde0768ea,0xd85568ec,0xde0968ec,0xd85568ee,
	0xde0968ee,0xd85568f0,0xde0b68f0,0xd85568f2,0xde0b68f2,0xd85568f4,0xde0d68f4,0xd85568f6,
	0xde0d68f6,0xd85568f8,0xde0f68f8,0xd85568fa,0xde0f68fa,0xd85568fc,0xde1168fc,0xd85568fe,
	0xde1168fe,0xd8556900,0xde136900,0xd8556902,0xde136902,0xd8556904,0xde156904,0xd8556906,
	0xde156906,0xd8556908,0xde176908,0xd855690a,0xde17690a,0xd855690c,0xde19690c,0xd855690e,
	0xde19690e,0xd8556910,0xde1b6910,0xd8556912,0xde1b6912,0xd8556914,0xde1d6914,0xd8556916,
	0xde1d6916,0xd8556918,0xde1f6918,0xd855691a,0xde1f691a,0xd855691c,0xde21691c,0xd855691e,
	0xde21691e,0xd8556920,0xde236920,0xd8556922,0xde236922,0xd8556924,0xde256924,0xd8556926,
	0xde256926,0xd8556928,0xde276928,0xd855692a,0xde27692a,0xd855692c,0xde29692c,0xd855692e,
	0xde29692e,0xd8556930,0xde2b6930,0xd8556932,0xde2b6932,0xd8556934,0xde2d6934,0xd8556936,
	0xde2d6936,0xd8556938,0xde2f6938,0xd855693a,0xde2f693a,0xd855693c,0xde31693c,0xd855693e,
	0xde31693e,0xd8556940,0xde336940,0xd8556942,0xde336942,0xd8556944,0xde356944,0xd8556946,
	0xde356946,0xd8556948,0xde376948,0xd855694a,0xde37694a,0xd855694c,0xde39694c,0xd855694e,
	0xde39694e,0xd8556950,0xde3b6950,0xd8556952,0xde3b6952,0xd8556954,0xde3d6954,0xd8556956,
	0xde3d6956,0xd8556958,0xde3f6958,0xd855695a,0xde3f695a,0xd855695c,0xde41695c,0xd855695e,
	0xde41695e,0xd8556960,0xde436960,0xd8556962,0xde436962,0xd8556964,0xde456964,0xd8556966,
	0xde456966,0xd8556968,0xde476968,0xd855696a,0xde47696a,0xd855696c,0xde49696c,0xd855696e,
	0xde49696e,0xd8556970,0xde4b6970,0xd8556972,0xde4b6972,0xd8556974,0xde4d6974,0xd8556976,
	0xde4d6976,0xd8556978,0xde4f6978,0xd855697a,0xde4f697a,0xd855697c,0xde51697c,0xd855697e,
	0xde51697e,0xd8556980,0xde536980,0xd8556982,0xde536982,0xd8556984,0xde556984,0xd8556986,
	0xde556986,0xd8556988,0xde576988,0xd855698a,0xde57698a,0xd855698c,0xde59698c,0xd855698e,
	0xde59698e,0xd8556990,0xde5b6990,0xd8556992,0xde5b6992,0xd8556994,0xde5d6994,0xd8556996,
	0xde5d6996,0xd8556998,0xde5f6998,0xd855699a,0xde5f699a,0xd855699c,0xde61699c,0xd855699e,
	0xde61699e,0xd85569a0,0xde6369a0,0xd85569a2,0xde6369a2,0xd85569a4,0xde6569a4,0xd85569a6,
	0xde6569a6,0xd85569a8,0xde6769a8,0xd85569aa,0xde6769aa,0xd85569ac,0xde6969ac,0xd85569ae,
	0xde6969ae,0xd85569b0,0xde6b69b0,0xd85569b2,0xde6b69b2,0xd85569b4,0xde6d69b4,0xd85569b6,
	0xde6d69b6,0xd85569b8,0xde6f69b8,0xd85569ba,0xde6f69ba,0xd85569bc,0xde7169bc,0xd85569be,
	0xde7169be,0xd85569c0,0xde7369c0,0xd85569c2,0xde7369c2,0xd85569c4,0xde7569c4,0xd85569c6,
	0xde7569c6,0xd85569c8,0xde7769c8,0xd85569ca,0xde7769ca,0xd85569cc,0xde7969cc,0xd85569ce,
	0xde7969ce,0xd85569d0,0xde7b69d0,0xd85569d2,0xde7b69d2,0xd85569d4,0xde7d69d4,0xd85569d6,
	0xde7d69d6,0xd85569d8,0xde7f69d8,0xd85569da,0xde7f69da,0xd85569dc,0xde8169dc,0xd85569de,
	0xde8169de,0xd85569e0,0xde8369e0,0xd85569e2,0xde8369e2,0xd85569e4,0xde8569e4,0xd85569e6,
	0xde8569e6,0xd85569e8,0xde8769e8,0xd85569ea,0xde8769ea,0xd85569ec,0xde8969ec,0xd85569ee,
	0xde8969ee,0xd85569f0,0xde8b69f0,0xd85569f2,0xde8b69f2,0xd85569f4,0xde8d69f4,0xd85569f6,
	0xde8d69f6,0xd85569f8,0xde8f69f8,0xd85569fa,0xde8f69fa,0xd85569fc,0xde9169fc,0xd85569fe,
	0xde9169fe,0xd8556a00,0xde936a00,0xd8556a02,0xde936a02,0xd8556a04,0xde956a04,0xd8556a06,
	0xde956a06,0xd8556a08,0xde976a08,0xd8556a0a,0xde976a0a,0xd8556a0c,0xde996a0c,0xd8556a0e,
	0xde996a0e,0xd8556a10,0xde9b6a10,0xd8556a12,0xde9b6a12,0xd8556a14,0xde9d6a14,0xd8556a16,
	0xde9d6a16,0xd8556a18,0xde9f6a18,0xd8556a1a,0xde9f6a1a,0xd8556a1c,0xdea16a1c,0xd8556a1e,
	0xdea16a1e,0xd8556a20,0xdea36a20,0xd8556a22,0xdea36a22,0xd8556a24,0xdea56a24,0xd8556a26,
	0xdea56a26,0xd8556a28,0xdea76a28,0xd8556a2a,0xdea76a2a,0xd8556a2c,0xdea96a2c,0xd8556a2e,
	0xdea96a2e,0xd8556a30,0xdeab6a30,0xd8556a32,0xdeab6a32,0xd8556a34,0xdead6a34,0xd8556a36,
	0xdead6a36,0xd8556a38,0xdeaf6a38,0xd8556a3a,0xdeaf6a3a,0xd8556a3c,0xdeb16a3c,0xd8556a3e,
	0xdeb16a3e,0xd8556a40,0xdeb36a40,0xd8556a42,0xdeb36a42,0xd8556a44,0xdeb56a44,0xd8556a46,
	0xdeb56a46,0xd8556a48,0xdeb76a48,0xd8556a4a,0xdeb76a4a,0xd8556a4c,0xdeb96a4c,0xd8556a4e,
	0xdeb96a4e,0xd8556a50,0xdebb6a50,0xd8556a52,0xdebb6a52,0xd8556a54,0xdebd6a54,0xd8556a56,
	0xdebd6a56,0xd8556a58,0xdebf6a58,0xd8556a5a,0xdebf6a5a,0xd8556a5c,0xdec16a5c,0xd8556a5e,
	0xdec16a5e,0xd8556a60,0xdec36a60,0xd8556a62,0xdec36a62,0xd8556a64,0xdec56a64,0xd8556a66,
	0xdec56a66,0xd8556a68,0xdec76a68,0xd8556a6a,0xdec76a6a,0xd8556a6c,0xdec96a6c,0xd8556a6e,
	0xdec96a6e,0xd8556a70,0xdecb6a70,0xd8556a72,0xdecb6a72,0xd8556a74,0xdecd6a74,0xd8556a76,
	0xdecd6a76,0xd8556a78,0xdecf6a78,0xd8556a7a,0xdecf6a7a,0xd8556a7c,0xded16a7c,0xd8556a7e,
	0xded16a7e,0xd8556a80,0xded36a80,0xd8556a82,0xded36a82,0xd8556a84,0xded56a84,0xd8556a86,
	0xded56a86,0xd8556a88,0xded76a88,0xd8556a8a,0xded76a8a,0xd8556a8c,0xded96a8c,0xd8556a8e,
	0xded96a8e,0xd8556a90,0xdedb6a90,0xd8556a92,0xdedb6a92,0xd8556a94,0xdedd6a94,0xd8556a96,
	0xdedd6a96,0xd8556a98,0xdedf6a98,0xd8556a9a,0xdedf6a9a,0xd8556a9c,0xdee16a9c,0xd8556a9e,
	0xdee16a9e,0xd8556aa0,0xdee36aa0,0xd8556aa2,0xdee36aa2,0xd8556aa4,0xdee56aa4,0xd8556aa6,
	0xdee56aa6,0xd8556aa8,0xdee76aa8,0xd8556aaa,0xdee76aaa,0xd8556aac,0xdee96aac,0xd8556aae,
	0xdee96aae,0xd8556ab0,0xdeeb6ab0,0xd8556ab2,0xdeeb6ab2,0xd8556ab4,0xdeed6ab4,0xd8556ab6,
	0xdeed6ab6,0xd8556ab8,0xdeef6ab8,0xd8556aba,0xdeef6aba,0xd8556abc,0xdef16abc,0xd8556abe,
	0xdef16abe,0xd8556ac0,0xdef36ac0,0xd8556ac2,0xdef36ac2,0xd8556ac4,0xdef56ac4,0xd8556ac6,
	0xdef56ac6,0xd8556ac8,0xdef76ac8,0xd8556aca,0xdef76aca,0xd8556acc,0xdef96acc,0xd8556ace,
	0xdef96ace,0xd8556ad0,0xdefb6ad0,0xd8556ad2,0xdefb6ad2,0xd8556ad4,0xdefd6ad4,0xd8556ad6,
	0xdefd6ad6,0xd8556ad8,0xdeff6ad8,0xd8556ada,0xdeff6ada,0xd8556adc,0xdf016adc,0xd8556ade,
	0xdf016ade,0xd8556ae0,0xdf036ae0,0xd8556ae2,0xdf036ae2,0xd8556ae4,0xdf056ae4,0xd8556ae6,
	0xdf056ae6,0xd8556ae8,0xdf076ae8,0xd8556aea,0xdf076aea,0xd8556aec,0xdf096aec,0xd8556aee,
	0xdf096aee,0xd8556af0,0xdf0b6af0,0xd8556af2,0xdf0b6af2,0xd8556af4,0xdf0d6af4,0xd8556af6,
	0xdf0d6af6,0xd8556af8,0xdf0f6af8,0xd8556afa,0xdf0f6afa,0xd8556afc,0xdf116afc,0xd8556afe,
	0xdf116afe,0xd8556b00,0xdf136b00,0xd8556b02,0xdf136b02,0xd8556b04,0xdf156b04,0xd8556b06,
	0xdf156b06,0xd8556b08,0xdf176b08,0xd8556b0a,0xdf176b0a,0xd8556b0c,0xdf196b0c,0xd8556b0e,
	0xdf196b0e,0xd8556b10,0xdf1b6b10,0xd8556b12,0xdf1b6b12,0xd8556b14,0xdf1d6b14,0xd8556b16,
	0xdf1d6b16,0xd8556b18,0xdf1f6b18,0xd8556b1a,0xdf1f6b1a,0xd8556b1c,0xdf216b1c,0xd8556b1e,
	0xdf216b1e,0xd8556b20,0xdf236b20,0xd8556b22,0xdf236b22,0xd8556b24,0xdf256b24,0xd8556b26,
	0xdf256b26,0xd8556b28,0xdf276b28,0xd8556b2a,0xdf276b2a,0xd8556b2c,0xdf296b2c,0xd8556b2e,
	0xdf296b2e,0xd8556b30,0xdf2b6b30,0xd8556b32,0xdf2b6b32,0xd8556b34,0xdf2d6b34,0xd8556b36,
	0xdf2d6b36,0xd8556b38,0xdf2f6b38,0xd8556b3a,0xdf2f6b3a,0xd8556b3c,0xdf316b3c,0xd8556b3e,
	0xdf316b3e,0xd8556b40,0xdf336b40,0xd8556b42,0xdf336b42,0xd8556b44,0xdf356b44,0xd8556b46,
	0xdf356b46,0xd8556b48,0xdf376b48,0xd8556b4a,0xdf376b4a,0xd8556b4c,0xdf396b4c,0xd8556b4e,
	0xdf396b4e,0xd8556b50,0xdf3b6b50,0xd8556b52,0xdf3b6b52,0xd8556b54,0xdf3d6b54,0xd8556b56,
	0xdf3d6b56,0xd8556b58,0xdf3f6b58,0xd8556b5a,0xdf3f6b5a,0xd8556b5c,0xdf416b5c,0xd8556b5e,
	0xdf416b5e,0xd8556b60,0xdf436b60,0xd8556b62,0xdf436b62,0xd8556b64,0xdf456b64,0xd8556b66,
	0xdf456b66,0xd8556b68,0xdf476b68,0xd8556b6a,0xdf476b6a,0xd8556b6c,0xdf496b6c,0xd8556b6e,
	0xdf496b6e,0xd8556b70,0xdf4b6b70,0xd8556b72,0xdf4b6b72,0xd8556b74,0xdf4d6b74,0xd8556b76,
	0xdf4d6b76,0xd8556b78,0xdf4f6b78,0xd8556b7a,0xdf4f6b7a,0xd8556b7c,0xdf516b7c,0xd8556b7e,
	0xdf516b7e,0xd8556b80,0xdf536b80,0xd8556b82,0xdf536b82,0xd8556b84,0xdf556b84,0xd8556b86,
	0xdf556b86,0xd8556b88,0xdf576b88,0xd8556b8a,0xdf576b8a,0xd8556b8c,0xdf596b8c,0xd8556b8e,
	0xdf596b8e,0xd8556b90,0xdf5b6b90,0xd8556b92,0xdf5b6b92,0xd8556b94,0xdf5d6b94,0xd8556b96,
	0xdf5d6b96,0xd8556b98,0xdf5f6b98,0xd8556b9a,0xdf5f6b9a,0xd8556b9c,0xdf616b9c,0xd8556b9e,
	0xdf616b9e,0xd8556ba0,0xdf636ba0,0xd8556ba2,0xdf636ba2,0xd8556ba4,0xdf656ba4,0xd8556ba6,
	0xdf656ba6,0xd8556ba8,0xdf676ba8,0xd8556baa,0xdf676baa,0xd8556bac,0xdf696bac,0xd8556bae,
	0xdf696bae,0xd8556bb0,0xdf6b6bb0,0xd8556bb2,0xdf6b6bb2,0xd8556bb4,0xdf6d6bb4,0xd8556bb6,
	0xdf6d6bb6,0xd8556bb8,0xdf6f6bb8,0xd8556bba,0xdf6f6bba,0xd8556bbc,0xdf716bbc,0xd8556bbe,
	0xdf716bbe,0xd8556bc0,0xdf736bc0,0xd8556bc2,0xdf736bc2,0xd8556bc4,0xdf756bc4,0xd8556bc6,
	0xdf756bc6,0xd8556bc8,0xdf776bc8,0xd8556bca,0xdf776bca,0xd8556bcc,0xdf796bcc,0xd8556bce,
	0xdf796bce,0xd8556bd0,0xdf7b6bd0,0xd8556bd2,0xdf7b6bd2,0xd8556bd4,0xdf7d6bd4,0xd8556bd6,
	0xdf7d6bd6,0xd8556bd8,0xdf7f6bd8,0xd8556bda,0xdf7f6bda,0xd8556bdc,0xdf816bdc,0xd8556bde,
	0xdf816bde,0xd8556be0,0xdf836be0,0xd8556be2,0xdf836be2,0xd8556be4,0xdf856be4,0xd8556be6,
	0xdf856be6,0xd8556be8,0xdf876be8,0xd8556bea,0xdf876bea,0xd8556bec,0xdf896bec,0xd8556bee,
	0xdf896bee,0xd8556bf0,0xdf8b6bf0,0xd8556bf2,0xdf8b6bf2,0xd8556bf4,0xdf8d6bf4,0xd8556bf6,
	0xdf8d6bf6,0xd8556bf8,0xdf8f6bf8,0xd8556bfa,0xdf8f6bfa,0xd8556bfc,0xdf916bfc,0xd8556bfe,
	0xdf916bfe,0xd8556c00,0xdf936c00,0xd8556c02,0xdf936c02,0xd8556c04,0xdf956c04,0xd8556c06,
	0xdf956c06,0xd8556c08,0xdf976c08,0xd8556c0a,0xdf976c0a,0xd8556c0c,0xdf996c0c,0xd8556c0e,
	0xdf996c0e,0xd8556c10,0xdf9b6c10,0xd8556c12,0xdf9b6c12,0xd8556c14,0xdf9d6c14,0xd8556c16,
	0xdf9d6c16,0xd8556c18,0xdf9f6c18,0xd8556c1a,0xdf9f6c1a,0xd8556c1c,0xdfa16c1c,0xd8556c1e,
	0xdfa16c1e,0xd8556c20,0xdfa36c20,0xd8556c22,0xdfa36c22,0xd8556c24,0xdfa56c24,0xd8556c26,
	0xdfa56c26,0xd8556c28,0xdfa76c28,0xd8556c2a,0xdfa76c2a,0xd8556c2c,0xdfa96c2c,0xd8556c2e,
	0xdfa96c2e,0xd8556c30,0xdfab6c30,0xd8556c32,0xdfab6c32,0xd8556c34,0xdfad6c34,0xd8556c36,
	0xdfad6c36,0xd8556c38,0xdfaf6c38,0xd8556c3a,0xdfaf6c3a,0xd8556c3c,0xdfb16c3c,0xd8556c3e,
	0xdfb16c3e,0xd8556c40,0xdfb36c40,0xd8556c42,0xdfb36c42,0xd8556c44,0xdfb56c44,0xd8556c46,
	0xdfb56c46,0xd8556c48,0xdfb76c48,0xd8556c4a,0xdfb76c4a,0xd8556c4c,0xdfb96c4c,0xd8556c4e,
	0xdfb96c4e,0xd8556c50,0xdfbb6c50,0xd8556c52,0xdfbb6c52,0xd8556c54,0xdfbd6c54,0xd8556c56,
	0xdfbd6c56,0xd8556c58,0xdfbf6c58,0xd8556c5a,0xdfbf6c5a,0xd8556c5c,0xdfc16c5c,0xd8556c5e,
	0xdfc16c5e,0xd8556c60,0xdfc36c60,0xd8556c62,0xdfc36c62,0xd8556c64,0xdfc56c64,0xd8556c66,
	0xdfc56c66,0xd8556c68,0xdfc76c68,0xd8556c6a,0xdfc76c6a,0xd8556c6c,0xdfc96c6c,0xd8556c6e,
	0xdfc96c6e,0xd8556c70,0xdfcb6c70,0xd8556c72,0xdfcb6c72,0xd8556c74,0xdfcd6c74,0xd8556c76,
	0xdfcd6c76,0xd8556c78,0xdfcf6c78,0xd8556c7a,0xdfcf6c7a,0xd8556c7c,0xdfd16c7c,0xd8556c7e,
	0xdfd16c7e,0xd8556c80,0xdfd36c80,0xd8556c82,0xdfd36c82,0xd8556c84,0xdfd56c84,0xd8556c86,
	0xdfd56c86,0xd8556c88,0xdfd76c88,0xd8556c8a,0xdfd76c8a,0xd8556c8c,0xdfd96c8c,0xd8556c8e,
	0xdfd96c8e,0xd8556c90,0xdfdb6c90,0xd8556c92,0xdfdb6c92,0xd8556c94,0xdfdd6c94,0xd8556c96,
	0xdfdd6c96,0xd8556c98,0xdfdf6c98,0xd8556c9a,0xdfdf6c9a,0xd8556c9c,0xdfe16c9c,0xd8556c9e,
	0xdfe16c9e,0xd8556ca0,0xdfe36ca0,0xd8556ca2,0xdfe36ca2,0xd8556ca4,0xdfe56ca4,0xd8556ca6,
	0xdfe56ca6,0xd8556ca8,0xdfe76ca8,0xd8556caa,0xdfe76caa,0xd8556cac,0xdfe96cac,0xd8556cae,
	0xdfe96cae,0xd8556cb0,0xdfeb6cb0,0xd8556cb2,0xdfeb6cb2,0xd8556cb4,0xdfed6cb4,0xd8556cb6,
	0xdfed6cb6,0xd8556cb8,0xdfef6cb8,0xd8556cba,0xdfef6cba,0xd8556cbc,0xdff16cbc,0xd8556cbe,
	0xdff16cbe,0xd8556cc0,0xdff36cc0,0xd8556cc2,0xdff36cc2,0xd8556cc4,0xdff56cc4,0xd8556cc6,
	0xdff56cc6,0xd8556cc8,0xdff76cc8,0xd8556cca,0xdff76cca,0xd8556ccc,0xdff96ccc,0xd8556cce,
	0xdff96cce,0xd8556cd0,0xdffb6cd0,0xd8556cd2,0xdffb6cd2,0xd8556cd4,0xdffd6cd4,0xd8556cd6,
	0xdffd6cd6,0xd8556cd8,0xdfff6cd8,0xd8556cda,0xdfff6cda,0xd8576cdc,0xdc016cdc,0xd8576cde,
	0xdc016cde,0xd8576ce0,0xdc036ce0,0xd8576ce2,0xdc036ce2,0xd8576ce4,0xdc056ce4,0xd8576ce6,
	0xdc056ce6,0xd8576ce8,0xdc076ce8,0xd8576cea,0xdc076cea,0xd8576cec,0xdc096cec,0xd8576cee,
	0xdc096cee,0xd8576cf0,0xdc0b6cf0,0xd8576cf2,0xdc0b6cf2,0xd8576cf4,0xdc0d6cf4,0xd8576cf6,
	0xdc0d6cf6,0xd8576cf8,0xdc0f6cf8,0xd8576cfa,0xdc0f6cfa,0xd8576cfc,0xdc116cfc,0xd8576cfe,
	0xdc116cfe,0xd8576d00,0xdc136d00,0xd8576d02,0xdc136d02,0xd8576d04,0xdc156d04,0xd8576d06,
	0xdc156d06,0xd8576d08,0xdc176d08,0xd8576d0a,0xdc176d0a,0xd8576d0c,0xdc196d0c,0xd8576d0e,
	0xdc196d0e,0xd8576d10,0xdc1b6d10,0xd8576d12,0xdc1b6d12,0xd8576d14,0xdc1d6d14,0xd8576d16,
	0xdc1d6d16,0xd8576d18,0xdc1f6d18,0xd8576d1a,0xdc1f6d1a,0xd8576d1c,0xdc216d1c,0xd8576d1e,
	0xdc216d1e,0xd8576d20,0xdc236d20,0xd8576d22,0xdc236d22,0xd8576d24,0xdc256d24,0xd8576d26,
	0xdc256d26,0xd8576d28,0xdc276d28,0xd8576d2a,0xdc276d2a,0xd8576d2c,0xdc296d2c,0xd8576d2e,
	0xdc296d2e,0xd8576d30,0xdc2b6d30,0xd8576d32,0xdc2b6d32,0xd8576d34,0xdc2d6d34,0xd8576d36,
	0xdc2d6d36,0xd8576d38,0xdc2f6d38,0xd8576d3a,0xdc2f6d3a,0xd8576d3c,0xdc316d3c,0xd8576d3e,
	0xdc316d3e,0xd8576d40,0xdc336d40,0xd8576d42,0xdc336d42,0xd8576d44,0xdc356d44,0xd8576d46,
	0xdc356d46,0xd8576d48,0xdc376d48,0xd8576d4a,0xdc376d4a,0xd8576d4c,0xdc396d4c,0xd8576d4e,
	0xdc396d4e,0xd8576d50,0xdc3b6d50,0xd8576d52,0xdc3b6d52,0xd8576d54,0xdc3d6d54,0xd8576d56,
	0xdc3d6d56,0xd8576d58,0xdc3f6d58,0xd8576d5a,0xdc3f6d5a,0xd8576d5c,0xdc416d5c,0xd8576d5e,
	0xdc416d5e,0xd8576d60,0xdc436d60,0xd8576d62,0xdc436d62,0xd8576d64,0xdc456d64,0xd8576d66,
	0xdc456d66,0xd8576d68,0xdc476d68,0xd8576d6a,0xdc476d6a,0xd8576d6c,0xdc496d6c,0xd8576d6e,
	0xdc496d6e,0xd8576d70,0xdc4b6d70,0xd8576d72,0xdc4b6d72,0xd8576d74,0xdc4d6d74,0xd8576d76,
	0xdc4d6d76,0xd8576d78,0xdc4f6d78,0xd8576d7a,0xdc4f6d7a,0xd8576d7c,0xdc516d7c,0xd8576d7e,
	0xdc516d7e,0xd8576d80,0xdc536d80,0xd8576d82,0xdc536d82,0xd8576d84,0xdc556d84,0xd8576d86,
	0xdc556d86,0xd8576d88,0xdc576d88,0xd8576d8a,0xdc576d8a,0xd8576d8c,0xdc596d8c,0xd8576d8e,
	0xdc596d8e,0xd8576d90,0xdc5b6d90,0xd8576d92,0xdc5b6d92,0xd8576d94,0xdc5d6d94,0xd8576d96,
	0xdc5d6d96,0xd8576d98,0xdc5f6d98,0xd8576d9a,0xdc5f6d9a,0xd8576d9c,0xdc616d9c,0xd8576d9e,
	0xdc616d9e,0xd8576da0,0xdc636da0,0xd8576da2,0xdc636da2,0xd8576da4,0xdc656da4,0xd8576da6,
	0xdc656da6,0xd8576da8,0xdc676da8,0xd8576daa,0xdc676daa,0xd8576dac,0xdc696dac,0xd8576dae,
	0xdc696dae,0xd8576db0,0xdc6b6db0,0xd8576db2,0xdc6b6db2,0xd8576db4,0xdc6d6db4,0xd8576db6,
	0xdc6d6db6,0xd8576db8,0xdc6f6db8,0xd8576dba,0xdc6f6dba,0xd8576dbc,0xdc716dbc,0xd8576dbe,
	0xdc716dbe,0xd8576dc0,0xdc736dc0,0xd8576dc2,0xdc736dc2,0xd8576dc4,0xdc756dc4,0xd8576dc6,
	0xdc756dc6,0xd8576dc8,0xdc776dc8,0xd8576dca,0xdc776dca,0xd8576dcc,0xdc796dcc,0xd8576dce,
	0xdc796dce,0xd8576dd0,0xdc7b6dd0,0xd8576dd2,0xdc7b6dd2,0xd8576dd4,0xdc7d6dd4,0xd8576dd6,
	0xdc7d6dd6,0xd8576dd8,0xdc7f6dd8,0xd8576dda,0xdc7f6dda,0xd8576ddc,0xdc816ddc,0xd8576dde,
	0xdc816dde,0xd8576de0,0xdc836de0,0xd8576de2,0xdc836de2,0xd8576de4,0xdc856de4,0xd8576de6,
	0xdc856de6,0xd8576de8,0xdc876de8,0xd8576dea,0xdc876dea,0xd8576dec,0xdc896dec,0xd8576dee,
	0xdc896dee,0xd8576df0,0xdc8b6df0,0xd8576df2,0xdc8b6df2,0xd8576df4,0xdc8d6df4,0xd8576df6,
	0xdc8d6df6,0xd8576df8,0xdc8f6df8,0xd8576dfa,0xdc8f6dfa,0xd8576dfc,0xdc916dfc,0xd8576dfe,
	0xdc916dfe,0xd8576e00,0xdc936e00,0xd8576e02,0xdc936e02,0xd8576e04,0xdc956e04,0xd8576e06,
	0xdc956e06,0xd8576e08,0xdc976e08,0xd8576e0a,0xdc976e0a,0xd8576e0c,0xdc996e0c,0xd8576e0e,
	0xdc996e0e,0xd8576e10,0xdc9b6e10,0xd8576e12,0xdc9b6e12,0xd8576e14,0xdc9d6e14,0xd8576e16,
	0xdc9d6e16,0xd8576e18,0xdc9f6e18,0xd8576e1a,0xdc9f6e1a,0xd8576e1c,0xdca16e1c,0xd8576e1e,
	0xdca16e1e,0xd8576e20,0xdca36e20,0xd8576e22,0xdca36e22,0xd8576e24,0xdca56e24,0xd8576e26,
	0xdca56e26,0xd8576e28,0xdca76e28,0xd8576e2a,0xdca76e2a,0xd8576e2c,0xdca96e2c,0xd8576e2e,
	0xdca96e2e,0xd8576e30,0xdcab6e30,0xd8576e32,0xdcab6e32,0xd8576e34,0xdcad6e34,0xd8576e36,
	0xdcad6e36,0xd8576e38,0xdcaf6e38,0xd8576e3a,0xdcaf6e3a,0xd8576e3c,0xdcb16e3c,0xd8576e3e,
	0xdcb16e3e,0xd8576e40,0xdcb36e40,0xd8576e42,0xdcb36e42,0xd8576e44,0xdcb56e44,0xd8576e46,
	0xdcb56e46,0xd8576e48,0xdcb76e48,0xd8576e4a,0xdcb76e4a,0xd8576e4c,0xdcb96e4c,0xd8576e4e,
	0xdcb96e4e,0xd8576e50,0xdcbb6e50,0xd8576e52,0xdcbb6e52,0xd8576e54,0xdcbd6e54,0xd8576e56,
	0xdcbd6e56,0xd8576e58,0xdcbf6e58,0xd8576e5a,0xdcbf6e5a,0xd8576e5c,0xdcc16e5c,0xd8576e5e,
	0xdcc16e5e,0xd8576e60,0xdcc36e60,0xd8576e62,0xdcc36e62,0xd8576e64,0xdcc56e64,0xd8576e66,
	0xdcc56e66,0xd8576e68,0xdcc76e68,0xd8576e6a,0xdcc76e6a,0xd8576e6c,0xdcc96e6c,0xd8576e6e,
	0xdcc96e6e,0xd8576e70,0xdccb6e70,0xd8576e72,0xdccb6e72,0xd8576e74,0xdccd6e74,0xd8576e76,
	0xdccd6e76,0xd8576e78,0xdccf6e78,0xd8576e7a,0xdccf6e7a,0xd8576e7c,0xdcd16e7c,0xd8576e7e,
	0xdcd16e7e,0xd8576e80,0xdcd36e80,0xd8576e82,0xdcd36e82,0xd8576e84,0xdcd56e84,0xd8576e86,
	0xdcd56e86,0xd8576e88,0xdcd76e88,0xd8576e8a,0xdcd76e8a,0xd8576e8c,0xdcd96e8c,0xd8576e8e,
	0xdcd96e8e,0xd8576e90,0xdcdb6e90,0xd8576e92,0xdcdb6e92,0xd8576e94,0xdcdd6e94,0xd8576e96,
	0xdcdd6e96,0xd8576e98,0xdcdf6e98,0xd8576e9a,0xdcdf6e9a,0xd8576e9c,0xdce16e9c,0xd8576e9e,
	0xdce16e9e,0xd8576ea0,0xdce36ea0,0xd8576ea2,0xdce36ea2,0xd8576ea4,0xdce56ea4,0xd8576ea6,
	0xdce56ea6,0xd8576ea8,0xdce76ea8,0xd8576eaa,0xdce76eaa,0xd8576eac,0xdce96eac,0xd8576eae,
	0xdce96eae,0xd8576eb0,0xdceb6eb0,0xd8576eb2,0xdceb6eb2,0xd8576eb4,0xdced6eb4,0xd8576eb6,
	0xdced6eb6,0xd8576eb8,0xdcef6eb8,0xd8576eba,0xdcef6eba,0xd8576ebc,0xdcf16ebc,0xd8576ebe,
	0xdcf16ebe,0xd8576ec0,0xdcf36ec0,0xd8576ec2,0xdcf36ec2,0xd8576ec4,0xdcf56ec4,0xd8576ec6,
	0xdcf56ec6,0xd8576ec8,0xdcf76ec8,0xd8576eca,0xdcf76eca,0xd8576ecc,0xdcf96ecc,0xd8576ece,
	0xdcf96ece,0xd8576ed0,0xdcfb6ed0,0xd8576ed2,0xdcfb6ed2,0xd8576ed4,0xdcfd6ed4,0xd8576ed6,
	0xdcfd6ed6,0xd8576ed8,0xdcff6ed8,0xd8576eda,0xdcff6eda,0xd8576edc,0xdd016edc,0xd8576ede,
	0xdd016ede,0xd8576ee0,0xdd036ee0,0xd8576ee2,0xdd036ee2,0xd8576ee4,0xdd056ee4,0xd8576ee6,
	0xdd056ee6,0xd8576ee8,0xdd076ee8,0xd8576eea,0xdd076eea,0xd8576eec,0xdd096eec,0xd8576eee,
	0xdd096eee,0xd8576ef0,0xdd0b6ef0,0xd8576ef2,0xdd0b6ef2,0xd8576ef4,0xdd0d6ef4,0xd8576ef6,
	0xdd0d6ef6,0xd8576ef8,0xdd0f6ef8,0xd8576efa,0xdd0f6efa,0xd8576efc,0xdd116efc,0xd8576efe,
	0xdd116efe,0xd8576f00,0xdd136f00,0xd8576f02,0xdd136f02,0xd8576f04,0xdd156f04,0xd8576f06,
	0xdd156f06,0xd8576f08,0xdd176f08,0xd8576f0a,0xdd176f0a,0xd8576f0c,0xdd196f0c,0xd8576f0e,
	0xdd196f0e,0xd8576f10,0xdd1b6f10,0xd8576f12,0xdd1b6f12,0xd8576f14,0xdd1d6f14,0xd8576f16,
	0xdd1d6f16,0xd8576f18,0xdd1f6f18,0xd8576f1a,0xdd1f6f1a,0xd8576f1c,0xdd216f1c,0xd8576f1e,
	0xdd216f1e,0xd8576f20,0xdd236f20,0xd8576f22,0xdd236f22,0xd8576f24,0xdd256f24,0xd8576f26,
	0xdd256f26,0xd8576f28,0xdd276f28,0xd8576f2a,0xdd276f2a,0xd8576f2c,0xdd296f2c,0xd8576f2e,
	0xdd296f2e,0xd8576f30,0xdd2b6f30,0xd8576f32,0xdd2b6f32,0xd8576f34,0xdd2d6f34,0xd8576f36,
	0xdd2d6f36,0xd8576f38,0xdd2f6f38,0xd8576f3a,0xdd2f6f3a,0xd8576f3c,0xdd316f3c,0xd8576f3e,
	0xdd316f3e,0xd8576f40,0xdd336f40,0xd8576f42,0xdd336f42,0xd8576f44,0xdd356f44,0xd8576f46,
	0xdd356f46,0xd8576f48,0xdd376f48,0xd8576f4a,0xdd376f4a,0xd8576f4c,0xdd396f4c,0xd8576f4e,
	0xdd396f4e,0xd8576f50,0xdd3b6f50,0xd8576f52,0xdd3b6f52,0xd8576f54,0xdd3d6f54,0xd8576f56,
	0xdd3d6f56,0xd8576f58,0xdd3f6f58,0xd8576f5a,0xdd3f6f5a,0xd8576f5c,0xdd416f5c,0xd8576f5e,
	0xdd416f5e,0xd8576f60,0xdd436f60,0xd8576f62,0xdd436f62,0xd8576f64,0xdd456f64,0xd8576f66,
	0xdd456f66,0xd8576f68,0xdd476f68,0xd8576f6a,0xdd476f6a,0xd8576f6c,0xdd496f6c,0xd8576f6e,
	0xdd496f6e,0xd8576f70,0xdd4b6f70,0xd8576f72,0xdd4b6f72,0xd8576f74,0xdd4d6f74,0xd8576f76,
	0xdd4d6f76,0xd8576f78,0xdd4f6f78,0xd8576f7a,0xdd4f6f7a,0xd8576f7c,0xdd516f7c,0xd8576f7e,
	0xdd516f7e,0xd8576f80,0xdd536f80,0xd8576f82,0xdd536f82,0xd8576f84,0xdd556f84,0xd8576f86,
	0xdd556f86,0xd8576f88,0xdd576f88,0xd8576f8a,0xdd576f8a,0xd8576f8c,0xdd596f8c,0xd8576f8e,
	0xdd596f8e,0xd8576f90,0xdd5b6f90,0xd8576f92,0xdd5b6f92,0xd8576f94,0xdd5d6f94,0xd8576f96,
	0xdd5d6f96,0xd8576f98,0xdd5f6f98,0xd8576f9a,0xdd5f6f9a,0xd8576f9c,0xdd616f9c,0xd8576f9e,
	0xdd616f9e,0xd8576fa0,0xdd636fa0,0xd8576fa2,0xdd636fa2,0xd8576fa4,0xdd656fa4,0xd8576fa6,
	0xdd656fa6,0xd8576fa8,0xdd676fa8,0xd8576faa,0xdd676faa,0xd8576fac,0xdd696fac,0xd8576fae,
	0xdd696fae,0xd8576fb0,0xdd6b6fb0,0xd8576fb2,0xdd6b6fb2,0xd8576fb4,0xdd6d6fb4,0xd8576fb6,
	0xdd6d6fb6,0xd8576fb8,0xdd6f6fb8,0xd8576fba,0xdd6f6fba,0xd8576fbc,0xdd716fbc,0xd8576fbe,
	0xdd716fbe,0xd8576fc0,0xdd736fc0,0xd8576fc2,0xdd736fc2,0xd8576fc4,0xdd756fc4,0xd8576fc6,
	0xdd756fc6,0xd8576fc8,0xdd776fc8,0xd8576fca,0xdd776fca,0xd8576fcc,0xdd796fcc,0xd8576fce,
	0xdd796fce,0xd8576fd0,0xdd7b6fd0,0xd8576fd2,0xdd7b6fd2,0xd8576fd4,0xdd7d6fd4,0xd8576fd6,
	0xdd7d6fd6,0xd8576fd8,0xdd7f6fd8,0xd8576fda,0xdd7f6fda,0xd8576fdc,0xdd816fdc,0xd8576fde,
	0xdd816fde,0xd8576fe0,0xdd836fe0,0xd8576fe2,0xdd836fe2,0xd8576fe4,0xdd856fe4,0xd8576fe6,
	0xdd856fe6,0xd8576fe8,0xdd876fe8,0xd8576fea,0xdd876fea,0xd8576fec,0xdd896fec,0xd8576fee,
	0xdd896fee,0xd8576ff0,0xdd8b6ff0,0xd8576ff2,0xdd8b6ff2,0xd8576ff4,0xdd8d6ff4,0xd8576ff6,
	0xdd8d6ff6,0xd8576ff8,0xdd8f6ff8,0xd8576ffa,0xdd8f6ffa,0xd8576ffc,0xdd916ffc,0xd8576ffe,
	0xdd916ffe,0xd8577000,0xdd937000,0xd8577002,0xdd937002,0xd8577004,0xdd957004,0xd8577006,
	0xdd957006,0xd8577008,0xdd977008,0xd857700a,0xdd97700a,0xd857700c,0xdd99700c,0xd857700e,
	0xdd99700e,0xd8577010,0xdd9b7010,0xd8577012,0xdd9b7012,0xd8577014,0xdd9d7014,0xd8577016,
	0xdd9d7016,0xd8577018,0xdd9f7018,0xd857701a,0xdd9f701a,0xd857701c,0xdda1701c,0xd857701e,
	0xdda1701e,0xd8577020,0xdda37020,0xd8577022,0xdda37022,0xd8577024,0xdda57024,0xd8577026,
	0xdda57026,0xd8577028,0xdda77028,0xd857702a,0xdda7702a,0xd857702c,0xdda9702c,0xd857702e,
	0xdda9702e,0xd8577030,0xddab7030,0xd8577032,0xddab7032,0xd8577034,0xddad7034,0xd8577036,
	0xddad7036,0xd8577038,0xddaf7038,0xd857703a,0xddaf703a,0xd857703c,0xddb1703c,0xd857703e,
	0xddb1703e,0xd8577040,0xddb37040,0xd8577042,0xddb37042,0xd8577044,0xddb57044,0xd8577046,
	0xddb57046,0xd8577048,0xddb77048,0xd857704a,0xddb7704a,0xd857704c,0xddb9704c,0xd857704e,
	0xddb9704e,0xd8577050,0xddbb7050,0xd8577052,0xddbb7052,0xd8577054,0xddbd7054,0xd8577056,
	0xddbd7056,0xd8577058,0xddbf7058,0xd857705a,0xddbf705a,0xd857705c,0xddc1705c,0xd857705e,
	0xddc1705e,0xd8577060,0xddc37060,0xd8577062,0xddc37062,0xd8577064,0xddc57064,0xd8577066,
	0xddc57066,0xd8577068,0xddc77068,0xd857706a,0xddc7706a,0xd857706c,0xddc9706c,0xd857706e,
	0xddc9706e,0xd8577070,0xddcb7070,0xd8577072,0xddcb7072,0xd8577074,0xddcd7074,0xd8577076,
	0xddcd7076,0xd8577078,0xddcf7078,0xd857707a,0xddcf707a,0xd857707c,0xddd1707c,0xd857707e,
	0xddd1707e,0xd8577080,0xddd37080,0xd8577082,0xddd37082,0xd8577084,0xddd57084,0xd8577086,
	0xddd57086,0xd8577088,0xddd77088,0xd857708a,0xddd7708a,0xd857708c,0xddd9708c,0xd857708e,
	0xddd9708e,0xd8577090,0xdddb7090,0xd8577092,0xdddb7092,0xd8577094,0xdddd7094,0xd8577096,
	0xdddd7096,0xd8577098,0xdddf7098,0xd857709a,0xdddf709a,0xd857709c,0xdde1709c,0xd857709e,
	0xdde1709e,0xd85770a0,0xdde370a0,0xd85770a2,0xdde370a2,0xd85770a4,0xdde570a4,0xd85770a6,
	0xdde570a6,0xd85770a8,0xdde770a8,0xd85770aa,0xdde770aa,0xd85770ac,0xdde970ac,0xd85770ae,
	0xdde970ae,0xd85770b0,0xddeb70b0,0xd85770b2,0xddeb70b2,0xd85770b4,0xdded70b4,0xd85770b6,
	0xdded70b6,0xd85770b8,0xddef70b8,0xd85770ba,0xddef70ba,0xd85770bc,0xddf170bc,0xd85770be,
	0xddf170be,0xd85770c0,0xddf370c0,0xd85770c2,0xddf370c2,0xd85770c4,0xddf570c4,0xd85770c6,
	0xddf570c6,0xd85770c8,0xddf770c8,0xd85770ca,0xddf770ca,0xd85770cc,0xddf970cc,0xd85770ce,
	0xddf970ce,0xd85770d0,0xddfb70d0,0xd85770d2,0xddfb70d2,0xd85770d4,0xddfd70d4,0xd85770d6,
	0xddfd70d6,0xd85770d8,0xddff70d8,0xd85770da,0xddff70da,0xd85770dc,0xde0170dc,0xd85770de,
	0xde0170de,0xd85770e0,0xde0370e0,0xd85770e2,0xde0370e2,0xd85770e4,0xde0570e4,0xd85770e6,
	0xde0570e6,0xd85770e8,0xde0770e8,0xd85770ea,0xde0770ea,0xd85770ec,0xde0970ec,0xd85770ee,
	0xde0970ee,0xd85770f0,0xde0b70f0,0xd85770f2,0xde0b70f2,0xd85770f4,0xde0d70f4,0xd85770f6,
	0xde0d70f6,0xd85770f8,0xde0f70f8,0xd85770fa,0xde0f70fa,0xd85770fc,0xde1170fc,0xd85770fe,
	0xde1170fe,0xd8577100,0xde137100,0xd8577102,0xde137102,0xd8577104,0xde157104,0xd8577106,
	0xde157106,0xd8577108,0xde177108,0xd857710a,0xde17710a,0xd857710c,0xde19710c,0xd857710e,
	0xde19710e,0xd8577110,0xde1b7110,0xd8577112,0xde1b7112,0xd8577114,0xde1d7114,0xd8577116,
	0xde1d7116,0xd8577118,0xde1f7118,0xd857711a,0xde1f711a,0xd857711c,0xde21711c,0xd857711e,
	0xde21711e,0xd8577120,0xde237120,0xd8577122,0xde237122,0xd8577124,0xde257124,0xd8577126,
	0xde257126,0xd8577128,0xde277128,0xd857712a,0xde27712a,0xd857712c,0xde29712c,0xd857712e,
	0xde29712e,0xd8577130,0xde2b7130,0xd8577132,0xde2b7132,0xd8577134,0xde2d7134,0xd8577136,
	0xde2d7136,0xd8577138,0xde2f7138,0xd857713a,0xde2f713a,0xd857713c,0xde31713c,0xd857713e,
	0xde31713e,0xd8577140,0xde337140,0xd8577142,0xde337142,0xd8577144,0xde357144,0xd8577146,
	0xde357146,0xd8577148,0xde377148,0xd857714a,0xde37714a,0xd857714c,0xde39714c,0xd857714e,
	0xde39714e,0xd8577150,0xde3b7150,0xd8577152,0xde3b7152,0xd8577154,0xde3d7154,0xd8577156,
	0xde3d7156,0xd8577158,0xde3f7158,0xd857715a,0xde3f715a,0xd857715c,0xde41715c,0xd857715e,
	0xde41715e,0xd8577160,0xde437160,0xd8577162,0xde437162,0xd8577164,0xde457164,0xd8577166,
	0xde457166,0xd8577168,0xde477168,0xd857716a,0xde47716a,0xd857716c,0xde49716c,0xd857716e,
	0xde49716e,0xd8577170,0xde4b7170,0xd8577172,0xde4b7172,0xd8577174,0xde4d7174,0xd8577176,
	0xde4d7176,0xd8577178,0xde4f7178,0xd857717a,0xde4f717a,0xd857717c,0xde51717c,0xd857717e,
	0xde51717e,0xd8577180,0xde537180,0xd8577182,0xde537182,0xd8577184,0xde557184,0xd8577186,
	0xde557186,0xd8577188,0xde577188,0xd857718a,0xde57718a,0xd857718c,0xde59718c,0xd857718e,
	0xde59718e,0xd8577190,0xde5b7190,0xd8577192,0xde5b7192,0xd8577194,0xde5d7194,0xd8577196,
	0xde5d7196,0xd8577198,0xde5f7198,0xd857719a,0xde5f719a,0xd857719c,0xde61719c,0xd857719e,
	0xde61719e,0xd85771a0,0xde6371a0,0xd85771a2,0xde6371a2,0xd85771a4,0xde6571a4,0xd85771a6,
	0xde6571a6,0xd85771a8,0xde6771a8,0xd85771aa,0xde6771aa,0xd85771ac,0xde6971ac,0xd85771ae,
	0xde6971ae,0xd85771b0,0xde6b71b0,0xd85771b2,0xde6b71b2,0xd85771b4,0xde6d71b4,0xd85771b6,
	0xde6d71b6,0xd85771b8,0xde6f71b8,0xd85771ba,0xde6f71ba,0xd85771bc,0xde7171bc,0xd85771be,
	0xde7171be,0xd85771c0,0xde7371c0,0xd85771c2,0xde7371c2,0xd85771c4,0xde7571c4,0xd85771c6,
	0xde7571c6,0xd85771c8,0xde7771c8,0xd85771ca,0xde7771ca,0xd85771cc,0xde7971cc,0xd85771ce,
	0xde7971ce,0xd85771d0,0xde7b71d0,0xd85771d2,0xde7b71d2,0xd85771d4,0xde7d71d4,0xd85771d6,
	0xde7d71d6,0xd85771d8,0xde7f71d8,0xd85771da,0xde7f71da,0xd85771dc,0xde8171dc,0xd85771de,
	0xde8171de,0xd85771e0,0xde8371e0,0xd85771e2,0xde8371e2,0xd85771e4,0xde8571e4,0xd85771e6,
	0xde8571e6,0xd85771e8,0xde8771e8,0xd85771ea,0xde8771ea,0xd85771ec,0xde8971ec,0xd85771ee,
	0xde8971ee,0xd85771f0,0xde8b71f0,0xd85771f2,0xde8b71f2,0xd85771f4,0xde8d71f4,0xd85771f6,
	0xde8d71f6,0xd85771f8,0xde8f71f8,0xd85771fa,0xde8f71fa,0xd85771fc,0xde9171fc,0xd85771fe,
	0xde9171fe,0xd8577200,0xde937200,0xd8577202,0xde937202,0xd8577204,0xde957204,0xd8577206,
	0xde957206,0xd8577208,0xde977208,0xd857720a,0xde97720a,0xd857720c,0xde99720c,0xd857720e,
	0xde99720e,0xd8577210,0xde9b7210,0xd8577212,0xde9b7212,0xd8577214,0xde9d7214,0xd8577216,
	0xde9d7216,0xd8577218,0xde9f7218,0xd857721a,0xde9f721a,0xd857721c,0xdea1721c,0xd857721e,
	0xdea1721e,0xd8577220,0xdea37220,0xd8577222,0xdea37222,0xd8577224,0xdea57224,0xd8577226,
	0xdea57226,0xd8577228,0xdea77228,0xd857722a,0xdea7722a,0xd857722c,0xdea9722c,0xd857722e,
	0xdea9722e,0xd8577230,0xdeab7230,0xd8577232,0xdeab7232,0xd8577234,0xdead7234,0xd8577236,
	0xdead7236,0xd8577238,0xdeaf7238,0xd857723a,0xdeaf723a,0xd857723c,0xdeb1723c,0xd857723e,
	0xdeb1723e,0xd8577240,0xdeb37240,0xd8577242,0xdeb37242,0xd8577244,0xdeb57244,0xd8577246,
	0xdeb57246,0xd8577248,0xdeb77248,0xd857724a,0xdeb7724a,0xd857724c,0xdeb9724c,0xd857724e,
	0xdeb9724e,0xd8577250,0xdebb7250,0xd8577252,0xdebb7252,0xd8577254,0xdebd7254,0xd8577256,
	0xdebd7256,0xd8577258,0xdebf7258,0xd857725a,0xdebf725a,0xd857725c,0xdec1725c,0xd857725e,
	0xdec1725e,0xd8577260,0xdec37260,0xd8577262,0xdec37262,0xd8577264,0xdec57264,0xd8577266,
	0xdec57266,0xd8577268,0xdec77268,0xd857726a,0xdec7726a,0xd857726c,0xdec9726c,0xd857726e,
	0xdec9726e,0xd8577270,0xdecb7270,0xd8577272,0xdecb7272,0xd8577274,0xdecd7274,0xd8577276,
	0xdecd7276,0xd8577278,0xdecf7278,0xd857727a,0xdecf727a,0xd857727c,0xded1727c,0xd857727e,
	0xded1727e,0xd8577280,0xded37280,0xd8577282,0xded37282,0xd8577284,0xded57284,0xd8577286,
	0xded57286,0xd8577288,0xded77288,0xd857728a,0xded7728a,0xd857728c,0xded9728c,0xd857728e,
	0xded9728e,0xd8577290,0xdedb7290,0xd8577292,0xdedb7292,0xd8577294,0xdedd7294,0xd8577296,
	0xdedd7296,0xd8577298,0xdedf7298,0xd857729a,0xdedf729a,0xd857729c,0xdee1729c,0xd857729e,
	0xdee1729e,0xd85772a0,0xdee372a0,0xd85772a2,0xdee372a2,0xd85772a4,0xdee572a4,0xd85772a6,
	0xdee572a6,0xd85772a8,0xdee772a8,0xd85772aa,0xdee772aa,0xd85772ac,0xdee972ac,0xd85772ae,
	0xdee972ae,0xd85772b0,0xdeeb72b0,0xd85772b2,0xdeeb72b2,0xd85772b4,0xdeed72b4,0xd85772b6,
	0xdeed72b6,0xd85772b8,0xdeef72b8,0xd85772ba,0xdeef72ba,0xd85772bc,0xdef172bc,0xd85772be,
	0xdef172be,0xd85772c0,0xdef372c0,0xd85772c2,0xdef372c2,0xd85772c4,0xdef572c4,0xd85772c6,
	0xdef572c6,0xd85772c8,0xdef772c8,0xd85772ca,0xdef772ca,0xd85772cc,0xdef972cc,0xd85772ce,
	0xdef972ce,0xd85772d0,0xdefb72d0,0xd85772d2,0xdefb72d2,0xd85772d4,0xdefd72d4,0xd85772d6,
	0xdefd72d6,0xd85772d8,0xdeff72d8,0xd85772da,0xdeff72da,0xd85772dc,0xdf0172dc,0xd85772de,
	0xdf0172de,0xd85772e0,0xdf0372e0,0xd85772e2,0xdf0372e2,0xd85772e4,0xdf0572e4,0xd85772e6,
	0xdf0572e6,0xd85772e8,0xdf0772e8,0xd85772ea,0xdf0772ea,0xd85772ec,0xdf0972ec,0xd85772ee,
	0xdf0972ee,0xd85772f0,0xdf0b72f0,0xd85772f2,0xdf0b72f2,0xd85772f4,0xdf0d72f4,0xd85772f6,
	0xdf0d72f6,0xd85772f8,0xdf0f72f8,0xd85772fa,0xdf0f72fa,0xd85772fc,0xdf1172fc,0xd85772fe,
	0xdf1172fe,0xd8577300,0xdf137300,0xd8577302,0xdf137302,0xd8577304,0xdf157304,0xd8577306,
	0xdf157306,0xd8577308,0xdf177308,0xd857730a,0xdf17730a,0xd857730c,0xdf19730c,0xd857730e,
	0xdf19730e,0xd8577310,0xdf1b7310,0xd8577312,0xdf1b7312,0xd8577314,0xdf1d7314,0xd8577316,
	0xdf1d7316,0xd8577318,0xdf1f7318,0xd857731a,0xdf1f731a,0xd857731c,0xdf21731c,0xd857731e,
	0xdf21731e,0xd8577320,0xdf237320,0xd8577322,0xdf237322,0xd8577324,0xdf257324,0xd8577326,
	0xdf257326,0xd8577328,0xdf277328,0xd857732a,0xdf27732a,0xd857732c,0xdf29732c,0xd857732e,
	0xdf29732e,0xd8577330,0xdf2b7330,0xd8577332,0xdf2b7332,0xd8577334,0xdf2d7334,0xd8577336,
	0xdf2d7336,0xd8577338,0xdf2f7338,0xd857733a,0xdf2f733a,0xd857733c,0xdf31733c,0xd857733e,
	0xdf31733e,0xd8577340,0xdf337340,0xd8577342,0xdf337342,0xd8577344,0xdf357344,0xd8577346,
	0xdf357346,0xd8577348,0xdf377348,0xd857734a,0xdf37734a,0xd857734c,0xdf39734c,0xd857734e,
	0xdf39734e,0xd8577350,0xdf3b7350,0xd8577352,0xdf3b7352,0xd8577354,0xdf3d7354,0xd8577356,
	0xdf3d7356,0xd8577358,0xdf3f7358,0xd857735a,0xdf3f735a,0xd857735c,0xdf41735c,0xd857735e,
	0xdf41735e,0xd8577360,0xdf437360,0xd8577362,0xdf437362,0xd8577364,0xdf457364,0xd8577366,
	0xdf457366,0xd8577368,0xdf477368,0xd857736a,0xdf47736a,0xd857736c,0xdf49736c,0xd857736e,
	0xdf49736e,0xd8577370,0xdf4b7370,0xd8577372,0xdf4b7372,0xd8577374,0xdf4d7374,0xd8577376,
	0xdf4d7376,0xd8577378,0xdf4f7378,0xd857737a,0xdf4f737a,0xd857737c,0xdf51737c,0xd857737e,
	0xdf51737e,0xd8577380,0xdf537380,0xd8577382,0xdf537382,0xd8577384,0xdf557384,0xd8577386,
	0xdf557386,0xd8577388,0xdf577388,0xd857738a,0xdf57738a,0xd857738c,0xdf59738c,0xd857738e,
	0xdf59738e,0xd8577390,0xdf5b7390,0xd8577392,0xdf5b7392,0xd8577394,0xdf5d7394,0xd8577396,
	0xdf5d7396,0xd8577398,0xdf5f7398,0xd857739a,0xdf5f739a,0xd857739c,0xdf61739c,0xd857739e,
	0xdf61739e,0xd85773a0,0xdf6373a0,0xd85773a2,0xdf6373a2,0xd85773a4,0xdf6573a4,0xd85773a6,
	0xdf6573a6,0xd85773a8,0xdf6773a8,0xd85773aa,0xdf6773aa,0xd85773ac,0xdf6973ac,0xd85773ae,
	0xdf6973ae,0xd85773b0,0xdf6b73b0,0xd85773b2,0xdf6b73b2,0xd85773b4,0xdf6d73b4,0xd85773b6,
	0xdf6d73b6,0xd85773b8,0xdf6f73b8,0xd85773ba,0xdf6f73ba,0xd85773bc,0xdf7173bc,0xd85773be,
	0xdf7173be,0xd85773c0,0xdf7373c0,0xd85773c2,0xdf7373c2,0xd85773c4,0xdf7573c4,0xd85773c6,
	0xdf7573c6,0xd85773c8,0xdf7773c8,0xd85773ca,0xdf7773ca,0xd85773cc,0xdf7973cc,0xd85773ce,
	0xdf7973ce,0xd85773d0,0xdf7b73d0,0xd85773d2,0xdf7b73d2,0xd85773d4,0xdf7d73d4,0xd85773d6,
	0xdf7d73d6,0xd85773d8,0xdf7f73d8,0xd85773da,0xdf7f73da,0xd85773dc,0xdf8173dc,0xd85773de,
	0xdf8173de,0xd85773e0,0xdf8373e0,0xd85773e2,0xdf8373e2,0xd85773e4,0xdf8573e4,0xd85773e6,
	0xdf8573e6,0xd85773e8,0xdf8773e8,0xd85773ea,0xdf8773ea,0xd85773ec,0xdf8973ec,0xd85773ee,
	0xdf8973ee,0xd85773f0,0xdf8b73f0,0xd85773f2,0xdf8b73f2,0xd85773f4,0xdf8d73f4,0xd85773f6,
	0xdf8d73f6,0xd85773f8,0xdf8f73f8,0xd85773fa,0xdf8f73fa,0xd85773fc,0xdf9173fc,0xd85773fe,
	0xdf9173fe,0xd8577400,0xdf937400,0xd8577402,0xdf937402,0xd8577404,0xdf957404,0xd8577406,
	0xdf957406,0xd8577408,0xdf977408,0xd857740a,0xdf97740a,0xd857740c,0xdf99740c,0xd857740e,
	0xdf99740e,0xd8577410,0xdf9b7410,0xd8577412,0xdf9b7412,0xd8577414,0xdf9d7414,0xd8577416,
	0xdf9d7416,0xd8577418,0xdf9f7418,0xd857741a,0xdf9f741a,0xd857741c,0xdfa1741c,0xd857741e,
	0xdfa1741e,0xd8577420,0xdfa37420,0xd8577422,0xdfa37422,0xd8577424,0xdfa57424,0xd8577426,
	0xdfa57426,0xd8577428,0xdfa77428,0xd857742a,0xdfa7742a,0xd857742c,0xdfa9742c,0xd857742e,
	0xdfa9742e,0xd8577430,0xdfab7430,0xd8577432,0xdfab7432,0xd8577434,0xdfad7434,0xd8577436,
	0xdfad7436,0xd8577438,0xdfaf7438,0xd857743a,0xdfaf743a,0xd857743c,0xdfb1743c,0xd857743e,
	0xdfb1743e,0xd8577440,0xdfb37440,0xd8577442,0xdfb37442,0xd8577444,0xdfb57444,0xd8577446,
	0xdfb57446,0xd8577448,0xdfb77448,0xd857744a,0xdfb7744a,0xd857744c,0xdfb9744c,0xd857744e,
	0xdfb9744e,0xd8577450,0xdfbb7450,0xd8577452,0xdfbb7452,0xd8577454,0xdfbd7454,0xd8577456,
	0xdfbd7456,0xd8577458,0xdfbf7458,0xd857745a,0xdfbf745a,0xd857745c,0xdfc1745c,0xd857745e,
	0xdfc1745e,0xd8577460,0xdfc37460,0xd8577462,0xdfc37462,0xd8577464,0xdfc57464,0xd8577466,
	0xdfc57466,0xd8577468,0xdfc77468,0xd857746a,0xdfc7746a,0xd857746c,0xdfc9746c,0xd857746e,
	0xdfc9746e,0xd8577470,0xdfcb7470,0xd8577472,0xdfcb7472,0xd8577474,0xdfcd7474,0xd8577476,
	0xdfcd7476,0xd8577478,0xdfcf7478,0xd857747a,0xdfcf747a,0xd857747c,0xdfd1747c,0xd857747e,
	0xdfd1747e,0xd8577480,0xdfd37480,0xd8577482,0xdfd37482,0xd8577484,0xdfd57484,0xd8577486,
	0xdfd57486,0xd8577488,0xdfd77488,0xd857748a,0xdfd7748a,0xd857748c,0xdfd9748c,0xd857748e,
	0xdfd9748e,0xd8577490,0xdfdb7490,0xd8577492,0xdfdb7492,0xd8577494,0xdfdd7494,0xd8577496,
	0xdfdd7496,0xd8577498,0xdfdf7498,0xd857749a,0xdfdf749a,0xd857749c,0xdfe1749c,0xd857749e,
	0xdfe1749e,0xd85774a0,0xdfe374a0,0xd85774a2,0xdfe374a2,0xd85774a4,0xdfe574a4,0xd85774a6,
	0xdfe574a6,0xd85774a8,0xdfe774a8,0xd85774aa,0xdfe774aa,0xd85774ac,0xdfe974ac,0xd85774ae,
	0xdfe974ae,0xd85774b0,0xdfeb74b0,0xd85774b2,0xdfeb74b2,0xd85774b4,0xdfed74b4,0xd85774b6,
	0xdfed74b6,0xd85774b8,0xdfef74b8,0xd85774ba,0xdfef74ba,0xd85774bc,0xdff174bc,0xd85774be,
	0xdff174be,0xd85774c0,0xdff374c0,0xd85774c2,0xdff374c2,0xd85774c4,0xdff574c4,0xd85774c6,
	0xdff574c6,0xd85774c8,0xdff774c8,0xd85774ca,0xdff774ca,0xd85774cc,0xdff974cc,0xd85774ce,
	0xdff974ce,0xd85774d0,0xdffb74d0,0xd85774d2,0xdffb74d2,0xd85774d4,0xdffd74d4,0xd85774d6,
	0xdffd74d6,0xd85774d8,0xdfff74d8,0xd85774da,0xdfff74da,0xd85774dc,0xdc0174dc,0xd85774de,
	0xdc0174de,0xd85774e0,0xdc0374e0,0xd85774e2,0xdc0374e2,0xd85774e4,0xdc0574e4,0xd85774e6,
	0xdc0574e6,0xd85774e8,0xdc0774e8,0xd85774ea,0xdc0774ea,0xd85774ec,0xdc0974ec,0xd85774ee,
	0xdc0974ee,0xd85774f0,0xdc0b74f0,0xd85774f2,0xdc0b74f2,0xd85774f4,0xdc0d74f4,0xd85774f6,
	0xdc0d74f6,0xd85774f8,0xdc0f74f8,0xd85774fa,0xdc0f74fa,0xd85774fc,0xdc1174fc,0xd85774fe,
	0xdc1174fe,0xd8577500,0xdc137500,0xd8577502,0xdc137502,0xd8577504,0xdc157504,0xd8577506,
	0xdc157506,0xd8577508,0xdc177508,0xd857750a,0xdc17750a,0xd857750c,0xdc19750c,0xd857750e,
	0xdc19750e,0xd8577510,0xdc1b7510,0xd8577512,0xdc1b7512,0xd8577514,0xdc1d7514,0xd8577516,
	0xdc1d7516,0xd8577518,0xdc1f7518,0xd857751a,0xdc1f751a,0xd857751c,0xdc21751c,0xd857751e,
	0xdc21751e,0xd8577520,0xdc237520,0xd8577522,0xdc237522,0xd8577524,0xdc257524,0xd8577526,
	0xdc257526,0xd8577528,0xdc277528,0xd857752a,0xdc27752a,0xd857752c,0xdc29752c,0xd857752e,
	0xdc29752e,0xd8577530,0xdc2b7530,0xd8577532,0xdc2b7532,0xd8577534,0xdc2d7534,0xd8577536,
	0xdc2d7536,0xd8577538,0xdc2f7538,0xd857753a,0xdc2f753a,0xd857753c,0xdc31753c,0xd857753e,
	0xdc31753e,0xd8577540,0xdc337540,0xd8577542,0xdc337542,0xd8577544,0xdc357544,0xd8577546,
	0xdc357546,0xd8577548,0xdc377548,0xd857754a,0xdc37754a,0xd857754c,0xdc39754c,0xd857754e,
	0xdc39754e,0xd8577550,0xdc3b7550,0xd8577552,0xdc3b7552,0xd8577554,0xdc3d7554,0xd8577556,
	0xdc3d7556,0xd8577558,0xdc3f7558,0xd857755a,0xdc3f755a,0xd857755c,0xdc41755c,0xd857755e,
	0xdc41755e,0xd8577560,0xdc437560,0xd8577562,0xdc437562,0xd8577564,0xdc457564,0xd8577566,
	0xdc457566,0xd8577568,0xdc477568,0xd857756a,0xdc47756a,0xd857756c,0xdc49756c,0xd857756e,
	0xdc49756e,0xd8577570,0xdc4b7570,0xd8577572,0xdc4b7572,0xd8577574,0xdc4d7574,0xd8577576,
	0xdc4d7576,0xd8577578,0xdc4f7578,0xd857757a,0xdc4f757a,0xd857757c,0xdc51757c,0xd857757e,
	0xdc51757e,0xd8577580,0xdc537580,0xd8577582,0xdc537582,0xd8577584,0xdc557584,0xd8577586,
	0xdc557586,0xd8577588,0xdc577588,0xd857758a,0xdc57758a,0xd857758c,0xdc59758c,0xd857758e,
	0xdc59758e,0xd8577590,0xdc5b7590,0xd8577592,0xdc5b7592,0xd8577594,0xdc5d7594,0xd8577596,
	0xdc5d7596,0xd8577598,0xdc5f7598,0xd857759a,0xdc5f759a,0xd857759c,0xdc61759c,0xd857759e,
	0xdc61759e,0xd85775a0,0xdc6375a0,0xd85775a2,0xdc6375a2,0xd85775a4,0xdc6575a4,0xd85775a6,
	0xdc6575a6,0xd85775a8,0xdc6775a8,0xd85775aa,0xdc6775aa,0xd85775ac,0xdc6975ac,0xd85775ae,
	0xdc6975ae,0xd85775b0,0xdc6b75b0,0xd85775b2,0xdc6b75b2,0xd85775b4,0xdc6d75b4,0xd85775b6,
	0xdc6d75b6,0xd85775b8,0xdc6f75b8,0xd85775ba,0xdc6f75ba,0xd85775bc,0xdc7175bc,0xd85775be,
	0xdc7175be,0xd85775c0,0xdc7375c0,0xd85775c2,0xdc7375c2,0xd85775c4,0xdc7575c4,0xd85775c6,
	0xdc7575c6,0xd85775c8,0xdc7775c8,0xd85775ca,0xdc7775ca,0xd85775cc,0xdc7975cc,0xd85775ce,
	0xdc7975ce,0xd85775d0,0xdc7b75d0,0xd85775d2,0xdc7b75d2,0xd85775d4,0xdc7d75d4,0xd85775d6,
	0xdc7d75d6,0xd85775d8,0xdc7f75d8,0xd85775da,0xdc7f75da,0xd85775dc,0xdc8175dc,0xd85775de,
	0xdc8175de,0xd85775e0,0xdc8375e0,0xd85775e2,0xdc8375e2,0xd85775e4,0xdc8575e4,0xd85775e6,
	0xdc8575e6,0xd85775e8,0xdc8775e8,0xd85775ea,0xdc8775ea,0xd85775ec,0xdc8975ec,0xd85775ee,
	0xdc8975ee,0xd85775f0,0xdc8b75f0,0xd85775f2,0xdc8b75f2,0xd85775f4,0xdc8d75f4,0xd85775f6,
	0xdc8d75f6,0xd85775f8,0xdc8f75f8,0xd85775fa,0xdc8f75fa,0xd85775fc,0xdc9175fc,0xd85775fe,
	0xdc9175fe,0xd8577600,0xdc937600,0xd8577602,0xdc937602,0xd8577604,0xdc957604,0xd8577606,
	0xdc957606,0xd8577608,0xdc977608,0xd857760a,0xdc97760a,0xd857760c,0xdc99760c,0xd857760e,
	0xdc99760e,0xd8577610,0xdc9b7610,0xd8577612,0xdc9b7612,0xd8577614,0xdc9d7614,0xd8577616,
	0xdc9d7616,0xd8577618,0xdc9f7618,0xd857761a,0xdc9f761a,0xd857761c,0xdca1761c,0xd857761e,
	0xdca1761e,0xd8577620,0xdca37620,0xd8577622,0xdca37622,0xd8577624,0xdca57624,0xd8577626,
	0xdca57626,0xd8577628,0xdca77628,0xd857762a,0xdca7762a,0xd857762c,0xdca9762c,0xd857762e,
	0xdca9762e,0xd8577630,0xdcab7630,0xd8577632,0xdcab7632,0xd8577634,0xdcad7634,0xd8577636,
	0xdcad7636,0xd8577638,0xdcaf7638,0xd857763a,0xdcaf763a,0xd857763c,0xdcb1763c,0xd857763e,
	0xdcb1763e,0xd8577640,0xdcb37640,0xd8577642,0xdcb37642,0xd8577644,0xdcb57644,0xd8577646,
	0xdcb57646,0xd8577648,0xdcb77648,0xd857764a,0xdcb7764a,0xd857764c,0xdcb9764c,0xd857764e,
	0xdcb9764e,0xd8577650,0xdcbb7650,0xd8577652,0xdcbb7652,0xd8577654,0xdcbd7654,0xd8577656,
	0xdcbd7656,0xd8577658,0xdcbf7658,0xd857765a,0xdcbf765a,0xd857765c,0xdcc1765c,0xd857765e,
	0xdcc1765e,0xd8577660,0xdcc37660,0xd8577662,0xdcc37662,0xd8577664,0xdcc57664,0xd8577666,
	0xdcc57666,0xd8577668,0xdcc77668,0xd857766a,0xdcc7766a,0xd857766c,0xdcc9766c,0xd857766e,
	0xdcc9766e,0xd8577670,0xdccb7670,0xd8577672,0xdccb7672,0xd8577674,0xdccd7674,0xd8577676,
	0xdccd7676,0xd8577678,0xdccf7678,0xd857767a,0xdccf767a,0xd857767c,0xdcd1767c,0xd857767e,
	0xdcd1767e,0xd8577680,0xdcd37680,0xd8577682,0xdcd37682,0xd8577684,0xdcd57684,0xd8577686,
	0xdcd57686,0xd8577688,0xdcd77688,0xd857768a,0xdcd7768a,0xd857768c,0xdcd9768c,0xd857768e,
	0xdcd9768e,0xd8577690,0xdcdb7690,0xd8577692,0xdcdb7692,0xd8577694,0xdcdd7694,0xd8577696,
	0xdcdd7696,0xd8577698,0xdcdf7698,0xd857769a,0xdcdf769a,0xd857769c,0xdce1769c,0xd857769e,
	0xdce1769e,0xd85776a0,0xdce376a0,0xd85776a2,0xdce376a2,0xd85776a4,0xdce576a4,0xd85776a6,
	0xdce576a6,0xd85776a8,0xdce776a8,0xd85776aa,0xdce776aa,0xd85776ac,0xdce976ac,0xd85776ae,
	0xdce976ae,0xd85776b0,0xdceb76b0,0xd85776b2,0xdceb76b2,0xd85776b4,0xdced76b4,0xd85776b6,
	0xdced76b6,0xd85776b8,0xdcef76b8,0xd85776ba,0xdcef76ba,0xd85776bc,0xdcf176bc,0xd85776be,
	0xdcf176be,0xd85776c0,0xdcf376c0,0xd85776c2,0xdcf376c2,0xd85776c4,0xdcf576c4,0xd85776c6,
	0xdcf576c6,0xd85776c8,0xdcf776c8,0xd85776ca,0xdcf776ca,0xd85776cc,0xdcf976cc,0xd85776ce,
	0xdcf976ce,0xd85776d0,0xdcfb76d0,0xd85776d2,0xdcfb76d2,0xd85776d4,0xdcfd76d4,0xd85776d6,
	0xdcfd76d6,0xd85776d8,0xdcff76d8,0xd85776da,0xdcff76da,0xd85776dc,0xdd0176dc,0xd85776de,
	0xdd0176de,0xd85776e0,0xdd0376e0,0xd85776e2,0xdd0376e2,0xd85776e4,0xdd0576e4,0xd85776e6,
	0xdd0576e6,0xd85776e8,0xdd0776e8,0xd85776ea,0xdd0776ea,0xd85776ec,0xdd0976ec,0xd85776ee,
	0xdd0976ee,0xd85776f0,0xdd0b76f0,0xd85776f2,0xdd0b76f2,0xd85776f4,0xdd0d76f4,0xd85776f6,
	0xdd0d76f6,0xd85776f8,0xdd0f76f8,0xd85776fa,0xdd0f76fa,0xd85776fc,0xdd1176fc,0xd85776fe,
	0xdd1176fe,0xd8577700,0xdd137700,0xd8577702,0xdd137702,0xd8577704,0xdd157704,0xd8577706,
	0xdd157706,0xd8577708,0xdd177708,0xd857770a,0xdd17770a,0xd857770c,0xdd19770c,0xd857770e,
	0xdd19770e,0xd8577710,0xdd1b7710,0xd8577712,0xdd1b7712,0xd8577714,0xdd1d7714,0xd8577716,
	0xdd1d7716,0xd8577718,0xdd1f7718,0xd857771a,0xdd1f771a,0xd857771c,0xdd21771c,0xd857771e,
	0xdd21771e,0xd8577720,0xdd237720,0xd8577722,0xdd237722,0xd8577724,0xdd257724,0xd8577726,
	0xdd257726,0xd8577728,0xdd277728,0xd857772a,0xdd27772a,0xd857772c,0xdd29772c,0xd857772e,
	0xdd29772e,0xd8577730,0xdd2b7730,0xd8577732,0xdd2b7732,0xd8577734,0xdd2d7734,0xd8577736,
	0xdd2d7736,0xd8577738,0xdd2f7738,0xd857773a,0xdd2f773a,0xd857773c,0xdd31773c,0xd857773e,
	0xdd31773e,0xd8577740,0xdd337740,0xd8577742,0xdd337742,0xd8577744,0xdd357744,0xd8577746,
	0xdd357746,0xd8577748,0xdd377748,0xd857774a,0xdd37774a,0xd857774c,0xdd39774c,0xd857774e,
	0xdd39774e,0xd8577750,0xdd3b7750,0xd8577752,0xdd3b7752,0xd8577754,0xdd3d7754,0xd8577756,
	0xdd3d7756,0xd8577758,0xdd3f7758,0xd857775a,0xdd3f775a,0xd857775c,0xdd41775c,0xd857775e,
	0xdd41775e,0xd8577760,0xdd437760,0xd8577762,0xdd437762,0xd8577764,0xdd457764,0xd8577766,
	0xdd457766,0xd8577768,0xdd477768,0xd857776a,0xdd47776a,0xd857776c,0xdd49776c,0xd857776e,
	0xdd49776e,0xd8577770,0xdd4b7770,0xd8577772,0xdd4b7772,0xd8577774,0xdd4d7774,0xd8577776,
	0xdd4d7776,0xd8577778,0xdd4f7778,0xd857777a,0xdd4f777a,0xd857777c,0xdd51777c,0xd857777e,
	0xdd51777e,0xd8577780,0xdd537780,0xd8577782,0xdd537782,0xd8577784,0xdd557784,0xd8577786,
	0xdd557786,0xd8577788,0xdd577788,0xd857778a,0xdd57778a,0xd857778c,0xdd59778c,0xd857778e,
	0xdd59778e,0xd8577790,0xdd5b7790,0xd8577792,0xdd5b7792,0xd8577794,0xdd5d7794,0xd8577796,
	0xdd5d7796,0xd8577798,0xdd5f7798,0xd857779a,0xdd5f779a,0xd857779c,0xdd61779c,0xd857779e,
	0xdd61779e,0xd85777a0,0xdd6377a0,0xd85777a2,0xdd6377a2,0xd85777a4,0xdd6577a4,0xd85777a6,
	0xdd6577a6,0xd85777a8,0xdd6777a8,0xd85777aa,0xdd6777aa,0xd85777ac,0xdd6977ac,0xd85777ae,
	0xdd6977ae,0xd85777b0,0xdd6b77b0,0xd85777b2,0xdd6b77b2,0xd85777b4,0xdd6d77b4,0xd85777b6,
	0xdd6d77b6,0xd85777b8,0xdd6f77b8,0xd85777ba,0xdd6f77ba,0xd85777bc,0xdd7177bc,0xd85777be,
	0xdd7177be,0xd85777c0,0xdd7377c0,0xd85777c2,0xdd7377c2,0xd85777c4,0xdd7577c4,0xd85777c6,
	0xdd7577c6,0xd85777c8,0xdd7777c8,0xd85777ca,0xdd7777ca,0xd85777cc,0xdd7977cc,0xd85777ce,
	0xdd7977ce,0xd85777d0,0xdd7b77d0,0xd85777d2,0xdd7b77d2,0xd85777d4,0xdd7d77d4,0xd85777d6,
	0xdd7d77d6,0xd85777d8,0xdd7f77d8,0xd85777da,0xdd7f77da,0xd85777dc,0xdd8177dc,0xd85777de,
	0xdd8177de,0xd85777e0,0xdd8377e0,0xd85777e2,0xdd8377e2,0xd85777e4,0xdd8577e4,0xd85777e6,
	0xdd8577e6,0xd85777e8,0xdd8777e8,0xd85777ea,0xdd8777ea,0xd85777ec,0xdd8977ec,0xd85777ee,
	0xdd8977ee,0xd85777f0,0xdd8b77f0,0xd85777f2,0xdd8b77f2,0xd85777f4,0xdd8d77f4,0xd85777f6,
	0xdd8d77f6,0xd85777f8,0xdd8f77f8,0xd85777fa,0xdd8f77fa,0xd85777fc,0xdd9177fc,0xd85777fe,
	0xdd9177fe,0xd8577800,0xdd937800,0xd8577802,0xdd937802,0xd8577804,0xdd957804,0xd8577806,
	0xdd957806,0xd8577808,0xdd977808,0xd857780a,0xdd97780a,0xd857780c,0xdd99780c,0xd857780e,
	0xdd99780e,0xd8577810,0xdd9b7810,0xd8577812,0xdd9b7812,0xd8577814,0xdd9d7814,0xd8577816,
	0xdd9d7816,0xd8577818,0xdd9f7818,0xd857781a,0xdd9f781a,0xd857781c,0xdda1781c,0xd857781e,
	0xdda1781e,0xd8577820,0xdda37820,0xd8577822,0xdda37822,0xd8577824,0xdda57824,0xd8577826,
	0xdda57826,0xd8577828,0xdda77828,0xd857782a,0xdda7782a,0xd857782c,0xdda9782c,0xd857782e,
	0xdda9782e,0xd8577830,0xddab7830,0xd8577832,0xddab7832,0xd8577834,0xddad7834,0xd8577836,
	0xddad7836,0xd8577838,0xddaf7838,0xd857783a,0xddaf783a,0xd857783c,0xddb1783c,0xd857783e,
	0xddb1783e,0xd8577840,0xddb37840,0xd8577842,0xddb37842,0xd8577844,0xddb57844,0xd8577846,
	0xddb57846,0xd8577848,0xddb77848,0xd857784a,0xddb7784a,0xd857784c,0xddb9784c,0xd857784e,
	0xddb9784e,0xd8577850,0xddbb7850,0xd8577852,0xddbb7852,0xd8577854,0xddbd7854,0xd8577856,
	0xddbd7856,0xd8577858,0xddbf7858,0xd857785a,0xddbf785a,0xd857785c,0xddc1785c,0xd857785e,
	0xddc1785e,0xd8577860,0xddc37860,0xd8577862,0xddc37862,0xd8577864,0xddc57864,0xd8577866,
	0xddc57866,0xd8577868,0xddc77868,0xd857786a,0xddc7786a,0xd857786c,0xddc9786c,0xd857786e,
	0xddc9786e,0xd8577870,0xddcb7870,0xd8577872,0xddcb7872,0xd8577874,0xddcd7874,0xd8577876,
	0xddcd7876,0xd8577878,0xddcf7878,0xd857787a,0xddcf787a,0xd857787c,0xddd1787c,0xd857787e,
	0xddd1787e,0xd8577880,0xddd37880,0xd8577882,0xddd37882,0xd8577884,0xddd57884,0xd8577886,
	0xddd57886,0xd8577888,0xddd77888,0xd857788a,0xddd7788a,0xd857788c,0xddd9788c,0xd857788e,
	0xddd9788e,0xd8577890,0xdddb7890,0xd8577892,0xdddb7892,0xd8577894,0xdddd7894,0xd8577896,
	0xdddd7896,0xd8577898,0xdddf7898,0xd857789a,0xdddf789a,0xd857789c,0xdde1789c,0xd857789e,
	0xdde1789e,0xd85778a0,0xdde378a0,0xd85778a2,0xdde378a2,0xd85778a4,0xdde578a4,0xd85778a6,
	0xdde578a6,0xd85778a8,0xdde778a8,0xd85778aa,0xdde778aa,0xd85778ac,0xdde978ac,0xd85778ae,
	0xdde978ae,0xd85778b0,0xddeb78b0,0xd85778b2,0xddeb78b2,0xd85778b4,0xdded78b4,0xd85778b6,
	0xdded78b6,0xd85778b8,0xddef78b8,0xd85778ba,0xddef78ba,0xd85778bc,0xddf178bc,0xd85778be,
	0xddf178be,0xd85778c0,0xddf378c0,0xd85778c2,0xddf378c2,0xd85778c4,0xddf578c4,0xd85778c6,
	0xddf578c6,0xd85778c8,0xddf778c8,0xd85778ca,0xddf778ca,0xd85778cc,0xddf978cc,0xd85778ce,
	0xddf978ce,0xd85778d0,0xddfb78d0,0xd85778d2,0xddfb78d2,0xd85778d4,0xddfd78d4,0xd85778d6,
	0xddfd78d6,0xd85778d8,0xddff78d8,0xd85778da,0xddff78da,0xd85778dc,0xde0178dc,0xd85778de,
	0xde0178de,0xd85778e0,0xde0378e0,0xd85778e2,0xde0378e2,0xd85778e4,0xde0578e4,0xd85778e6,
	0xde0578e6,0xd85778e8,0xde0778e8,0xd85778ea,0xde0778ea,0xd85778ec,0xde0978ec,0xd85778ee,
	0xde0978ee,0xd85778f0,0xde0b78f0,0xd85778f2,0xde0b78f2,0xd85778f4,0xde0d78f4,0xd85778f6,
	0xde0d78f6,0xd85778f8,0xde0f78f8,0xd85778fa,0xde0f78fa,0xd85778fc,0xde1178fc,0xd85778fe,
	0xde1178fe,0xd8577900,0xde137900,0xd8577902,0xde137902,0xd8577904,0xde157904,0xd8577906,
	0xde157906,0xd8577908,0xde177908,0xd857790a,0xde17790a,0xd857790c,0xde19790c,0xd857790e,
	0xde19790e,0xd8577910,0xde1b7910,0xd8577912,0xde1b7912,0xd8577914,0xde1d7914,0xd8577916,
	0xde1d7916,0xd8577918,0xde1f7918,0xd857791a,0xde1f791a,0xd857791c,0xde21791c,0xd857791e,
	0xde21791e,0xd8577920,0xde237920,0xd8577922,0xde237922,0xd8577924,0xde257924,0xd8577926,
	0xde257926,0xd8577928,0xde277928,0xd857792a,0xde27792a,0xd857792c,0xde29792c,0xd857792e,
	0xde29792e,0xd8577930,0xde2b7930,0xd8577932,0xde2b7932,0xd8577934,0xde2d7934,0xd8577936,
	0xde2d7936,0xd8577938,0xde2f7938,0xd857793a,0xde2f793a,0xd857793c,0xde31793c,0xd857793e,
	0xde31793e,0xd8577940,0xde337940,0xd8577942,0xde337942,0xd8577944,0xde357944,0xd8577946,
	0xde357946,0xd8577948,0xde377948,0xd857794a,0xde37794a,0xd857794c,0xde39794c,0xd857794e,
	0xde39794e,0xd8577950,0xde3b7950,0xd8577952,0xde3b7952,0xd8577954,0xde3d7954,0xd8577956,
	0xde3d7956,0xd8577958,0xde3f7958,0xd857795a,0xde3f795a,0xd857795c,0xde41795c,0xd857795e,
	0xde41795e,0xd8577960,0xde437960,0xd8577962,0xde437962,0xd8577964,0xde457964,0xd8577966,
	0xde457966,0xd8577968,0xde477968,0xd857796a,0xde47796a,0xd857796c,0xde49796c,0xd857796e,
	0xde49796e,0xd8577970,0xde4b7970,0xd8577972,0xde4b7972,0xd8577974,0xde4d7974,0xd8577976,
	0xde4d7976,0xd8577978,0xde4f7978,0xd857797a,0xde4f797a,0xd857797c,0xde51797c,0xd857797e,
	0xde51797e,0xd8577980,0xde537980,0xd8577982,0xde537982,0xd8577984,0xde557984,0xd8577986,
	0xde557986,0xd8577988,0xde577988,0xd857798a,0xde57798a,0xd857798c,0xde59798c,0xd857798e,
	0xde59798e,0xd8577990,0xde5b7990,0xd8577992,0xde5b7992,0xd8577994,0xde5d7994,0xd8577996,
	0xde5d7996,0xd8577998,0xde5f7998,0xd857799a,0xde5f799a,0xd857799c,0xde61799c,0xd857799e,
	0xde61799e,0xd85779a0,0xde6379a0,0xd85779a2,0xde6379a2,0xd85779a4,0xde6579a4,0xd85779a6,
	0xde6579a6,0xd85779a8,0xde6779a8,0xd85779aa,0xde6779aa,0xd85779ac,0xde6979ac,0xd85779ae,
	0xde6979ae,0xd85779b0,0xde6b79b0,0xd85779b2,0xde6b79b2,0xd85779b4,0xde6d79b4,0xd85779b6,
	0xde6d79b6,0xd85779b8,0xde6f79b8,0xd85779ba,0xde6f79ba,0xd85779bc,0xde7179bc,0xd85779be,
	0xde7179be,0xd85779c0,0xde7379c0,0xd85779c2,0xde7379c2,0xd85779c4,0xde7579c4,0xd85779c6,
	0xde7579c6,0xd85779c8,0xde7779c8,0xd85779ca,0xde7779ca,0xd85779cc,0xde7979cc,0xd85779ce,
	0xde7979ce,0xd85779d0,0xde7b79d0,0xd85779d2,0xde7b79d2,0xd85779d4,0xde7d79d4,0xd85779d6,
	0xde7d79d6,0xd85779d8,0xde7f79d8,0xd85779da,0xde7f79da,0xd85779dc,0xde8179dc,0xd85779de,
	0xde8179de,0xd85779e0,0xde8379e0,0xd85779e2,0xde8379e2,0xd85779e4,0xde8579e4,0xd85779e6,
	0xde8579e6,0xd85779e8,0xde8779e8,0xd85779ea,0xde8779ea,0xd85779ec,0xde8979ec,0xd85779ee,
	0xde8979ee,0xd85779f0,0xde8b79f0,0xd85779f2,0xde8b79f2,0xd85779f4,0xde8d79f4,0xd85779f6,
	0xde8d79f6,0xd85779f8,0xde8f79f8,0xd85779fa,0xde8f79fa,0xd85779fc,0xde9179fc,0xd85779fe,
	0xde9179fe,0xd8577a00,0xde937a00,0xd8577a02,0xde937a02,0xd8577a04,0xde957a04,0xd8577a06,
	0xde957a06,0xd8577a08,0xde977a08,0xd8577a0a,0xde977a0a,0xd8577a0c,0xde997a0c,0xd8577a0e,
	0xde997a0e,0xd8577a10,0xde9b7a10,0xd8577a12,0xde9b7a12,0xd8577a14,0xde9d7a14,0xd8577a16,
	0xde9d7a16,0xd8577a18,0xde9f7a18,0xd8577a1a,0xde9f7a1a,0xd8577a1c,0xdea17a1c,0xd8577a1e,
	0xdea17a1e,0xd8577a20,0xdea37a20,0xd8577a22,0xdea37a22,0xd8577a24,0xdea57a24,0xd8577a26,
	0xdea57a26,0xd8577a28,0xdea77a28,0xd8577a2a,0xdea77a2a,0xd8577a2c,0xdea97a2c,0xd8577a2e,
	0xdea97a2e,0xd8577a30,0xdeab7a30,0xd8577a32,0xdeab7a32,0xd8577a34,0xdead7a34,0xd8577a36,
	0xdead7a36,0xd8577a38,0xdeaf7a38,0xd8577a3a,0xdeaf7a3a,0xd8577a3c,0xdeb17a3c,0xd8577a3e,
	0xdeb17a3e,0xd8577a40,0xdeb37a40,0xd8577a42,0xdeb37a42,0xd8577a44,0xdeb57a44,0xd8577a46,
	0xdeb57a46,0xd8577a48,0xdeb77a48,0xd8577a4a,0xdeb77a4a,0xd8577a4c,0xdeb97a4c,0xd8577a4e,
	0xdeb97a4e,0xd8577a50,0xdebb7a50,0xd8577a52,0xdebb7a52,0xd8577a54,0xdebd7a54,0xd8577a56,
	0xdebd7a56,0xd8577a58,0xdebf7a58,0xd8577a5a,0xdebf7a5a,0xd8577a5c,0xdec17a5c,0xd8577a5e,
	0xdec17a5e,0xd8577a60,0xdec37a60,0xd8577a62,0xdec37a62,0xd8577a64,0xdec57a64,0xd8577a66,
	0xdec57a66,0xd8577a68,0xdec77a68,0xd8577a6a,0xdec77a6a,0xd8577a6c,0xdec97a6c,0xd8577a6e,
	0xdec97a6e,0xd8577a70,0xdecb7a70,0xd8577a72,0xdecb7a72,0xd8577a74,0xdecd7a74,0xd8577a76,
	0xdecd7a76,0xd8577a78,0xdecf7a78,0xd8577a7a,0xdecf7a7a,0xd8577a7c,0xded17a7c,0xd8577a7e,
	0xded17a7e,0xd8577a80,0xded37a80,0xd8577a82,0xded37a82,0xd8577a84,0xded57a84,0xd8577a86,
	0xded57a86,0xd8577a88,0xded77a88,0xd8577a8a,0xded77a8a,0xd8577a8c,0xded97a8c,0xd8577a8e,
	0xded97a8e,0xd8577a90,0xdedb7a90,0xd8577a92,0xdedb7a92,0xd8577a94,0xdedd7a94,0xd8577a96,
	0xdedd7a96,0xd8577a98,0xdedf7a98,0xd8577a9a,0xdedf7a9a,0xd8577a9c,0xdee17a9c,0xd8577a9e,
	0xdee17a9e,0xd8577aa0,0xdee37aa0,0xd8577aa2,0xdee37aa2,0xd8577aa4,0xdee57aa4,0xd8577aa6,
	0xdee57aa6,0xd8577aa8,0xdee77aa8,0xd8577aaa,0xdee77aaa,0xd8577aac,0xdee97aac,0xd8577aae,
	0xdee97aae,0xd8577ab0,0xdeeb7ab0,0xd8577ab2,0xdeeb7ab2,0xd8577ab4,0xdeed7ab4,0xd8577ab6,
	0xdeed7ab6,0xd8577ab8,0xdeef7ab8,0xd8577aba,0xdeef7aba,0xd8577abc,0xdef17abc,0xd8577abe,
	0xdef17abe,0xd8577ac0,0xdef37ac0,0xd8577ac2,0xdef37ac2,0xd8577ac4,0xdef57ac4,0xd8577ac6,
	0xdef57ac6,0xd8577ac8,0xdef77ac8,0xd8577aca,0xdef77aca,0xd8577acc,0xdef97acc,0xd8577ace,
	0xdef97ace,0xd8577ad0,0xdefb7ad0,0xd8577ad2,0xdefb7ad2,0xd8577ad4,0xdefd7ad4,0xd8577ad6,
	0xdefd7ad6,0xd8577ad8,0xdeff7ad8,0xd8577ada,0xdeff7ada,0xd8577adc,0xdf017adc,0xd8577ade,
	0xdf017ade,0xd8577ae0,0xdf037ae0,0xd8577ae2,0xdf037ae2,0xd8577ae4,0xdf057ae4,0xd8577ae6,
	0xdf057ae6,0xd8577ae8,0xdf077ae8,0xd8577aea,0xdf077aea,0xd8577aec,0xdf097aec,0xd8577aee,
	0xdf097aee,0xd8577af0,0xdf0b7af0,0xd8577af2,0xdf0b7af2,0xd8577af4,0xdf0d7af4,0xd8577af6,
	0xdf0d7af6,0xd8577af8,0xdf0f7af8,0xd8577afa,0xdf0f7afa,0xd8577afc,0xdf117afc,0xd8577afe,
	0xdf117afe,0xd8577b00,0xdf137b00,0xd8577b02,0xdf137b02,0xd8577b04,0xdf157b04,0xd8577b06,
	0xdf157b06,0xd8577b08,0xdf177b08,0xd8577b0a,0xdf177b0a,0xd8577b0c,0xdf197b0c,0xd8577b0e,
	0xdf197b0e,0xd8577b10,0xdf1b7b10,0xd8577b12,0xdf1b7b12,0xd8577b14,0xdf1d7b14,0xd8577b16,
	0xdf1d7b16,0xd8577b18,0xdf1f7b18,0xd8577b1a,0xdf1f7b1a,0xd8577b1c,0xdf217b1c,0xd8577b1e,
	0xdf217b1e,0xd8577b20,0xdf237b20,0xd8577b22,0xdf237b22,0xd8577b24,0xdf257b24,0xd8577b26,
	0xdf257b26,0xd8577b28,0xdf277b28,0xd8577b2a,0xdf277b2a,0xd8577b2c,0xdf297b2c,0xd8577b2e,
	0xdf297b2e,0xd8577b30,0xdf2b7b30,0xd8577b32,0xdf2b7b32,0xd8577b34,0xdf2d7b34,0xd8577b36,
	0xdf2d7b36,0xd8577b38,0xdf2f7b38,0xd8577b3a,0xdf2f7b3a,0xd8577b3c,0xdf317b3c,0xd8577b3e,
	0xdf317b3e,0xd8577b40,0xdf337b40,0xd8577b42,0xdf337b42,0xd8577b44,0xdf357b44,0xd8577b46,
	0xdf357b46,0xd8577b48,0xdf377b48,0xd8577b4a,0xdf377b4a,0xd8577b4c,0xdf397b4c,0xd8577b4e,
	0xdf397b4e,0xd8577b50,0xdf3b7b50,0xd8577b52,0xdf3b7b52,0xd8577b54,0xdf3d7b54,0xd8577b56,
	0xdf3d7b56,0xd8577b58,0xdf3f7b58,0xd8577b5a,0xdf3f7b5a,0xd8577b5c,0xdf417b5c,0xd8577b5e,
	0xdf417b5e,0xd8577b60,0xdf437b60,0xd8577b62,0xdf437b62,0xd8577b64,0xdf457b64,0xd8577b66,
	0xdf457b66,0xd8577b68,0xdf477b68,0xd8577b6a,0xdf477b6a,0xd8577b6c,0xdf497b6c,0xd8577b6e,
	0xdf497b6e,0xd8577b70,0xdf4b7b70,0xd8577b72,0xdf4b7b72,0xd8577b74,0xdf4d7b74,0xd8577b76,
	0xdf4d7b76,0xd8577b78,0xdf4f7b78,0xd8577b7a,0xdf4f7b7a,0xd8577b7c,0xdf517b7c,0xd8577b7e,
	0xdf517b7e,0xd8577b80,0xdf537b80,0xd8577b82,0xdf537b82,0xd8577b84,0xdf557b84,0xd8577b86,
	0xdf557b86,0xd8577b88,0xdf577b88,0xd8577b8a,0xdf577b8a,0xd8577b8c,0xdf597b8c,0xd8577b8e,
	0xdf597b8e,0xd8577b90,0xdf5b7b90,0xd8577b92,0xdf5b7b92,0xd8577b94,0xdf5d7b94,0xd8577b96,
	0xdf5d7b96,0xd8577b98,0xdf5f7b98,0xd8577b9a,0xdf5f7b9a,0xd8577b9c,0xdf617b9c,0xd8577b9e,
	0xdf617b9e,0xd8577ba0,0xdf637ba0,0xd8577ba2,0xdf637ba2,0xd8577ba4,0xdf657ba4,0xd8577ba6,
	0xdf657ba6,0xd8577ba8,0xdf677ba8,0xd8577baa,0xdf677baa,0xd8577bac,0xdf697bac,0xd8577bae,
	0xdf697bae,0xd8577bb0,0xdf6b7bb0,0xd8577bb2,0xdf6b7bb2,0xd8577bb4,0xdf6d7bb4,0xd8577bb6,
	0xdf6d7bb6,0xd8577bb8,0xdf6f7bb8,0xd8577bba,0xdf6f7bba,0xd8577bbc,0xdf717bbc,0xd8577bbe,
	0xdf717bbe,0xd8577bc0,0xdf737bc0,0xd8577bc2,0xdf737bc2,0xd8577bc4,0xdf757bc4,0xd8577bc6,
	0xdf757bc6,0xd8577bc8,0xdf777bc8,0xd8577bca,0xdf777bca,0xd8577bcc,0xdf797bcc,0xd8577bce,
	0xdf797bce,0xd8577bd0,0xdf7b7bd0,0xd8577bd2,0xdf7b7bd2,0xd8577bd4,0xdf7d7bd4,0xd8577bd6,
	0xdf7d7bd6,0xd8577bd8,0xdf7f7bd8,0xd8577bda,0xdf7f7bda,0xd8577bdc,0xdf817bdc,0xd8577bde,
	0xdf817bde,0xd8577be0,0xdf837be0,0xd8577be2,0xdf837be2,0xd8577be4,0xdf857be4,0xd8577be6,
	0xdf857be6,0xd8577be8,0xdf877be8,0xd8577bea,0xdf877bea,0xd8577bec,0xdf897bec,0xd8577bee,
	0xdf897bee,0xd8577bf0,0xdf8b7bf0,0xd8577bf2,0xdf8b7bf2,0xd8577bf4,0xdf8d7bf4,0xd8577bf6,
	0xdf8d7bf6,0xd8577bf8,0xdf8f7bf8,0xd8577bfa,0xdf8f7bfa,0xd8577bfc,0xdf917bfc,0xd8577bfe,
	0xdf917bfe,0xd8577c00,0xdf937c00,0xd8577c02,0xdf937c02,0xd8577c04,0xdf957c04,0xd8577c06,
	0xdf957c06,0xd8577c08,0xdf977c08,0xd8577c0a,0xdf977c0a,0xd8577c0c,0xdf997c0c,0xd8577c0e,
	0xdf997c0e,0xd8577c10,0xdf9b7c10,0xd8577c12,0xdf9b7c12,0xd8577c14,0xdf9d7c14,0xd8577c16,
	0xdf9d7c16,0xd8577c18,0xdf9f7c18,0xd8577c1a,0xdf9f7c1a,0xd8577c1c,0xdfa17c1c,0xd8577c1e,
	0xdfa17c1e,0xd8577c20,0xdfa37c20,0xd8577c22,0xdfa37c22,0xd8577c24,0xdfa57c24,0xd8577c26,
	0xdfa57c26,0xd8577c28,0xdfa77c28,0xd8577c2a,0xdfa77c2a,0xd8577c2c,0xdfa97c2c,0xd8577c2e,
	0xdfa97c2e,0xd8577c30,0xdfab7c30,0xd8577c32,0xdfab7c32,0xd8577c34,0xdfad7c34,0xd8577c36,
	0xdfad7c36,0xd8577c38,0xdfaf7c38,0xd8577c3a,0xdfaf7c3a,0xd8577c3c,0xdfb17c3c,0xd8577c3e,
	0xdfb17c3e,0xd8577c40,0xdfb37c40,0xd8577c42,0xdfb37c42,0xd8577c44,0xdfb57c44,0xd8577c46,
	0xdfb57c46,0xd8577c48,0xdfb77c48,0xd8577c4a,0xdfb77c4a,0xd8577c4c,0xdfb97c4c,0xd8577c4e,
	0xdfb97c4e,0xd8577c50,0xdfbb7c50,0xd8577c52,0xdfbb7c52,0xd8577c54,0xdfbd7c54,0xd8577c56,
	0xdfbd7c56,0xd8577c58,0xdfbf7c58,0xd8577c5a,0xdfbf7c5a,0xd8577c5c,0xdfc17c5c,0xd8577c5e,
	0xdfc17c5e,0xd8577c60,0xdfc37c60,0xd8577c62,0xdfc37c62,0xd8577c64,0xdfc57c64,0xd8577c66,
	0xdfc57c66,0xd8577c68,0xdfc77c68,0xd8577c6a,0xdfc77c6a,0xd8577c6c,0xdfc97c6c,0xd8577c6e,
	0xdfc97c6e,0xd8577c70,0xdfcb7c70,0xd8577c72,0xdfcb7c72,0xd8577c74,0xdfcd7c74,0xd8577c76,
	0xdfcd7c76,0xd8577c78,0xdfcf7c78,0xd8577c7a,0xdfcf7c7a,0xd8577c7c,0xdfd17c7c,0xd8577c7e,
	0xdfd17c7e,0xd8577c80,0xdfd37c80,0xd8577c82,0xdfd37c82,0xd8577c84,0xdfd57c84,0xd8577c86,
	0xdfd57c86,0xd8577c88,0xdfd77c88,0xd8577c8a,0xdfd77c8a,0xd8577c8c,0xdfd97c8c,0xd8577c8e,
	0xdfd97c8e,0xd8577c90,0xdfdb7c90,0xd8577c92,0xdfdb7c92,0xd8577c94,0xdfdd7c94,0xd8577c96,
	0xdfdd7c96,0xd8577c98,0xdfdf7c98,0xd8577c9a,0xdfdf7c9a,0xd8577c9c,0xdfe17c9c,0xd8577c9e,
	0xdfe17c9e,0xd8577ca0,0xdfe37ca0,0xd8577ca2,0xdfe37ca2,0xd8577ca4,0xdfe57ca4,0xd8577ca6,
	0xdfe57ca6,0xd8577ca8,0xdfe77ca8,0xd8577caa,0xdfe77caa,0xd8577cac,0xdfe97cac,0xd8577cae,
	0xdfe97cae,0xd8577cb0,0xdfeb7cb0,0xd8577cb2,0xdfeb7cb2,0xd8577cb4,0xdfed7cb4,0xd8577cb6,
	0xdfed7cb6,0xd8577cb8,0xdfef7cb8,0xd8577cba,0xdfef7cba,0xd8577cbc,0xdff17cbc,0xd8577cbe,
	0xdff17cbe,0xd8577cc0,0xdff37cc0,0xd8577cc2,0xdff37cc2,0xd8577cc4,0xdff57cc4,0xd8577cc6,
	0xdff57cc6,0xd8577cc8,0xdff77cc8,0xd8577cca,0xdff77cca,0xd8577ccc,0xdff97ccc,0xd8577cce,
	0xdff97cce,0xd8577cd0,0xdffb7cd0,0xd8577cd2,0xdffb7cd2,0xd8577cd4,0xdffd7cd4,0xd8577cd6,
	0xdffd7cd6,0xd8577cd8,0xdfff7cd8,0xd8577cda,0xdfff7cda,0xd8597cdc,0xdc017cdc,0xd8597cde,
	0xdc017cde,0xd8597ce0,0xdc037ce0,0xd8597ce2,0xdc037ce2,0xd8597ce4,0xdc057ce4,0xd8597ce6,
	0xdc057ce6,0xd8597ce8,0xdc077ce8,0xd8597cea,0xdc077cea,0xd8597cec,0xdc097cec,0xd8597cee,
	0xdc097cee,0xd8597cf0,0xdc0b7cf0,0xd8597cf2,0xdc0b7cf2,0xd8597cf4,0xdc0d7cf4,0xd8597cf6,
	0xdc0d7cf6,0xd8597cf8,0xdc0f7cf8,0xd8597cfa,0xdc0f7cfa,0xd8597cfc,0xdc117cfc,0xd8597cfe,
	0xdc117cfe,0xd8597d00,0xdc137d00,0xd8597d02,0xdc137d02,0xd8597d04,0xdc157d04,0xd8597d06,
	0xdc157d06,0xd8597d08,0xdc177d08,0xd8597d0a,0xdc177d0a,0xd8597d0c,0xdc197d0c,0xd8597d0e,
	0xdc197d0e,0xd8597d10,0xdc1b7d10,0xd8597d12,0xdc1b7d12,0xd8597d14,0xdc1d7d14,0xd8597d16,
	0xdc1d7d16,0xd8597d18,0xdc1f7d18,0xd8597d1a,0xdc1f7d1a,0xd8597d1c,0xdc217d1c,0xd8597d1e,
	0xdc217d1e,0xd8597d20,0xdc237d20,0xd8597d22,0xdc237d22,0xd8597d24,0xdc257d24,0xd8597d26,
	0xdc257d26,0xd8597d28,0xdc277d28,0xd8597d2a,0xdc277d2a,0xd8597d2c,0xdc297d2c,0xd8597d2e,
	0xdc297d2e,0xd8597d30,0xdc2b7d30,0xd8597d32,0xdc2b7d32,0xd8597d34,0xdc2d7d34,0xd8597d36,
	0xdc2d7d36,0xd8597d38,0xdc2f7d38,0xd8597d3a,0xdc2f7d3a,0xd8597d3c,0xdc317d3c,0xd8597d3e,
	0xdc317d3e,0xd8597d40,0xdc337d40,0xd8597d42,0xdc337d42,0xd8597d44,0xdc357d44,0xd8597d46,
	0xdc357d46,0xd8597d48,0xdc377d48,0xd8597d4a,0xdc377d4a,0xd8597d4c,0xdc397d4c,0xd8597d4e,
	0xdc397d4e,0xd8597d50,0xdc3b7d50,0xd8597d52,0xdc3b7d52,0xd8597d54,0xdc3d7d54,0xd8597d56,
	0xdc3d7d56,0xd8597d58,0xdc3f7d58,0xd8597d5a,0xdc3f7d5a,0xd8597d5c,0xdc417d5c,0xd8597d5e,
	0xdc417d5e,0xd8597d60,0xdc437d60,0xd8597d62,0xdc437d62,0xd8597d64,0xdc457d64,0xd8597d66,
	0xdc457d66,0xd8597d68,0xdc477d68,0xd8597d6a,0xdc477d6a,0xd8597d6c,0xdc497d6c,0xd8597d6e,
	0xdc497d6e,0xd8597d70,0xdc4b7d70,0xd8597d72,0xdc4b7d72,0xd8597d74,0xdc4d7d74,0xd8597d76,
	0xdc4d7d76,0xd8597d78,0xdc4f7d78,0xd8597d7a,0xdc4f7d7a,0xd8597d7c,0xdc517d7c,0xd8597d7e,
	0xdc517d7e,0xd8597d80,0xdc537d80,0xd8597d82,0xdc537d82,0xd8597d84,0xdc557d84,0xd8597d86,
	0xdc557d86,0xd8597d88,0xdc577d88,0xd8597d8a,0xdc577d8a,0xd8597d8c,0xdc597d8c,0xd8597d8e,
	0xdc597d8e,0xd8597d90,0xdc5b7d90,0xd8597d92,0xdc5b7d92,0xd8597d94,0xdc5d7d94,0xd8597d96,
	0xdc5d7d96,0xd8597d98,0xdc5f7d98,0xd8597d9a,0xdc5f7d9a,0xd8597d9c,0xdc617d9c,0xd8597d9e,
	0xdc617d9e,0xd8597da0,0xdc637da0,0xd8597da2,0xdc637da2,0xd8597da4,0xdc657da4,0xd8597da6,
	0xdc657da6,0xd8597da8,0xdc677da8,0xd8597daa,0xdc677daa,0xd8597dac,0xdc697dac,0xd8597dae,
	0xdc697dae,0xd8597db0,0xdc6b7db0,0xd8597db2,0xdc6b7db2,0xd8597db4,0xdc6d7db4,0xd8597db6,
	0xdc6d7db6,0xd8597db8,0xdc6f7db8,0xd8597dba,0xdc6f7dba,0xd8597dbc,0xdc717dbc,0xd8597dbe,
	0xdc717dbe,0xd8597dc0,0xdc737dc0,0xd8597dc2,0xdc737dc2,0xd8597dc4,0xdc757dc4,0xd8597dc6,
	0xdc757dc6,0xd8597dc8,0xdc777dc8,0xd8597dca,0xdc777dca,0xd8597dcc,0xdc797dcc,0xd8597dce,
	0xdc797dce,0xd8597dd0,0xdc7b7dd0,0xd8597dd2,0xdc7b7dd2,0xd8597dd4,0xdc7d7dd4,0xd8597dd6,
	0xdc7d7dd6,0xd8597dd8,0xdc7f7dd8,0xd8597dda,0xdc7f7dda,0xd8597ddc,0xdc817ddc,0xd8597dde,
	0xdc817dde,0xd8597de0,0xdc837de0,0xd8597de2,0xdc837de2,0xd8597de4,0xdc857de4,0xd8597de6,
	0xdc857de6,0xd8597de8,0xdc877de8,0xd8597dea,0xdc877dea,0xd8597dec,0xdc897dec,0xd8597dee,
	0xdc897dee,0xd8597df0,0xdc8b7df0,0xd8597df2,0xdc8b7df2,0xd8597df4,0xdc8d7df4,0xd8597df6,
	0xdc8d7df6,0xd8597df8,0xdc8f7df8,0xd8597dfa,0xdc8f7dfa,0xd8597dfc,0xdc917dfc,0xd8597dfe,
	0xdc917dfe,0xd8597e00,0xdc937e00,0xd8597e02,0xdc937e02,0xd8597e04,0xdc957e04,0xd8597e06,
	0xdc957e06,0xd8597e08,0xdc977e08,0xd8597e0a,0xdc977e0a,0xd8597e0c,0xdc997e0c,0xd8597e0e,
	0xdc997e0e,0xd8597e10,0xdc9b7e10,0xd8597e12,0xdc9b7e12,0xd8597e14,0xdc9d7e14,0xd8597e16,
	0xdc9d7e16,0xd8597e18,0xdc9f7e18,0xd8597e1a,0xdc9f7e1a,0xd8597e1c,0xdca17e1c,0xd8597e1e,
	0xdca17e1e,0xd8597e20,0xdca37e20,0xd8597e22,0xdca37e22,0xd8597e24,0xdca57e24,0xd8597e26,
	0xdca57e26,0xd8597e28,0xdca77e28,0xd8597e2a,0xdca77e2a,0xd8597e2c,0xdca97e2c,0xd8597e2e,
	0xdca97e2e,0xd8597e30,0xdcab7e30,0xd8597e32,0xdcab7e32,0xd8597e34,0xdcad7e34,0xd8597e36,
	0xdcad7e36,0xd8597e38,0xdcaf7e38,0xd8597e3a,0xdcaf7e3a,0xd8597e3c,0xdcb17e3c,0xd8597e3e,
	0xdcb17e3e,0xd8597e40,0xdcb37e40,0xd8597e42,0xdcb37e42,0xd8597e44,0xdcb57e44,0xd8597e46,
	0xdcb57e46,0xd8597e48,0xdcb77e48,0xd8597e4a,0xdcb77e4a,0xd8597e4c,0xdcb97e4c,0xd8597e4e,
	0xdcb97e4e,0xd8597e50,0xdcbb7e50,0xd8597e52,0xdcbb7e52,0xd8597e54,0xdcbd7e54,0xd8597e56,
	0xdcbd7e56,0xd8597e58,0xdcbf7e58,0xd8597e5a,0xdcbf7e5a,0xd8597e5c,0xdcc17e5c,0xd8597e5e,
	0xdcc17e5e,0xd8597e60,0xdcc37e60,0xd8597e62,0xdcc37e62,0xd8597e64,0xdcc57e64,0xd8597e66,
	0xdcc57e66,0xd8597e68,0xdcc77e68,0xd8597e6a,0xdcc77e6a,0xd8597e6c,0xdcc97e6c,0xd8597e6e,
	0xdcc97e6e,0xd8597e70,0xdccb7e70,0xd8597e72,0xdccb7e72,0xd8597e74,0xdccd7e74,0xd8597e76,
	0xdccd7e76,0xd8597e78,0xdccf7e78,0xd8597e7a,0xdccf7e7a,0xd8597e7c,0xdcd17e7c,0xd8597e7e,
	0xdcd17e7e,0xd8597e80,0xdcd37e80,0xd8597e82,0xdcd37e82,0xd8597e84,0xdcd57e84,0xd8597e86,
	0xdcd57e86,0xd8597e88,0xdcd77e88,0xd8597e8a,0xdcd77e8a,0xd8597e8c,0xdcd97e8c,0xd8597e8e,
	0xdcd97e8e,0xd8597e90,0xdcdb7e90,0xd8597e92,0xdcdb7e92,0xd8597e94,0xdcdd7e94,0xd8597e96,
	0xdcdd7e96,0xd8597e98,0xdcdf7e98,0xd8597e9a,0xdcdf7e9a,0xd8597e9c,0xdce17e9c,0xd8597e9e,
	0xdce17e9e,0xd8597ea0,0xdce37ea0,0xd8597ea2,0xdce37ea2,0xd8597ea4,0xdce57ea4,0xd8597ea6,
	0xdce57ea6,0xd8597ea8,0xdce77ea8,0xd8597eaa,0xdce77eaa,0xd8597eac,0xdce97eac,0xd8597eae,
	0xdce97eae,0xd8597eb0,0xdceb7eb0,0xd8597eb2,0xdceb7eb2,0xd8597eb4,0xdced7eb4,0xd8597eb6,
	0xdced7eb6,0xd8597eb8,0xdcef7eb8,0xd8597eba,0xdcef7eba,0xd8597ebc,0xdcf17ebc,0xd8597ebe,
	0xdcf17ebe,0xd8597ec0,0xdcf37ec0,0xd8597ec2,0xdcf37ec2,0xd8597ec4,0xdcf57ec4,0xd8597ec6,
	0xdcf57ec6,0xd8597ec8,0xdcf77ec8,0xd8597eca,0xdcf77eca,0xd8597ecc,0xdcf97ecc,0xd8597ece,
	0xdcf97ece,0xd8597ed0,0xdcfb7ed0,0xd8597ed2,0xdcfb7ed2,0xd8597ed4,0xdcfd7ed4,0xd8597ed6,
	0xdcfd7ed6,0xd8597ed8,0xdcff7ed8,0xd8597eda,0xdcff7eda,0xd8597edc,0xdd017edc,0xd8597ede,
	0xdd017ede,0xd8597ee0,0xdd037ee0,0xd8597ee2,0xdd037ee2,0xd8597ee4,0xdd057ee4,0xd8597ee6,
	0xdd057ee6,0xd8597ee8,0xdd077ee8,0xd8597eea,0xdd077eea,0xd8597eec,0xdd097eec,0xd8597eee,
	0xdd097eee,0xd8597ef0,0xdd0b7ef0,0xd8597ef2,0xdd0b7ef2,0xd8597ef4,0xdd0d7ef4,0xd8597ef6,
	0xdd0d7ef6,0xd8597ef8,0xdd0f7ef8,0xd8597efa,0xdd0f7efa,0xd8597efc,0xdd117efc,0xd8597efe,
	0xdd117efe,0xd8597f00,0xdd137f00,0xd8597f02,0xdd137f02,0xd8597f04,0xdd157f04,0xd8597f06,
	0xdd157f06,0xd8597f08,0xdd177f08,0xd8597f0a,0xdd177f0a,0xd8597f0c,0xdd197f0c,0xd8597f0e,
	0xdd197f0e,0xd8597f10,0xdd1b7f10,0xd8597f12,0xdd1b7f12,0xd8597f14,0xdd1d7f14,0xd8597f16,
	0xdd1d7f16,0xd8597f18,0xdd1f7f18,0xd8597f1a,0xdd1f7f1a,0xd8597f1c,0xdd217f1c,0xd8597f1e,
	0xdd217f1e,0xd8597f20,0xdd237f20,0xd8597f22,0xdd237f22,0xd8597f24,0xdd257f24,0xd8597f26,
	0xdd257f26,0xd8597f28,0xdd277f28,0xd8597f2a,0xdd277f2a,0xd8597f2c,0xdd297f2c,0xd8597f2e,
	0xdd297f2e,0xd8597f30,0xdd2b7f30,0xd8597f32,0xdd2b7f32,0xd8597f34,0xdd2d7f34,0xd8597f36,
	0xdd2d7f36,0xd8597f38,0xdd2f7f38,0xd8597f3a,0xdd2f7f3a,0xd8597f3c,0xdd317f3c,0xd8597f3e,
	0xdd317f3e,0xd8597f40,0xdd337f40,0xd8597f42,0xdd337f42,0xd8597f44,0xdd357f44,0xd8597f46,
	0xdd357f46,0xd8597f48,0xdd377f48,0xd8597f4a,0xdd377f4a,0xd8597f4c,0xdd397f4c,0xd8597f4e,
	0xdd397f4e,0xd8597f50,0xdd3b7f50,0xd8597f52,0xdd3b7f52,0xd8597f54,0xdd3d7f54,0xd8597f56,
	0xdd3d7f56,0xd8597f58,0xdd3f7f58,0xd8597f5a,0xdd3f7f5a,0xd8597f5c,0xdd417f5c,0xd8597f5e,
	0xdd417f5e,0xd8597f60,0xdd437f60,0xd8597f62,0xdd437f62,0xd8597f64,0xdd457f64,0xd8597f66,
	0xdd457f66,0xd8597f68,0xdd477f68,0xd8597f6a,0xdd477f6a,0xd8597f6c,0xdd497f6c,0xd8597f6e,
	0xdd497f6e,0xd8597f70,0xdd4b7f70,0xd8597f72,0xdd4b7f72,0xd8597f74,0xdd4d7f74,0xd8597f76,
	0xdd4d7f76,0xd8597f78,0xdd4f7f78,0xd8597f7a,0xdd4f7f7a,0xd8597f7c,0xdd517f7c,0xd8597f7e,
	0xdd517f7e,0xd8597f80,0xdd537f80,0xd8597f82,0xdd537f82,0xd8597f84,0xdd557f84,0xd8597f86,
	0xdd557f86,0xd8597f88,0xdd577f88,0xd8597f8a,0xdd577f8a,0xd8597f8c,0xdd597f8c,0xd8597f8e,
	0xdd597f8e,0xd8597f90,0xdd5b7f90,0xd8597f92,0xdd5b7f92,0xd8597f94,0xdd5d7f94,0xd8597f96,
	0xdd5d7f96,0xd8597f98,0xdd5f7f98,0xd8597f9a,0xdd5f7f9a,0xd8597f9c,0xdd617f9c,0xd8597f9e,
	0xdd617f9e,0xd8597fa0,0xdd637fa0,0xd8597fa2,0xdd637fa2,0xd8597fa4,0xdd657fa4,0xd8597fa6,
	0xdd657fa6,0xd8597fa8,0xdd677fa8,0xd8597faa,0xdd677faa,0xd8597fac,0xdd697fac,0xd8597fae,
	0xdd697fae,0xd8597fb0,0xdd6b7fb0,0xd8597fb2,0xdd6b7fb2,0xd8597fb4,0xdd6d7fb4,0xd8597fb6,
	0xdd6d7fb6,0xd8597fb8,0xdd6f7fb8,0xd8597fba,0xdd6f7fba,0xd8597fbc,0xdd717fbc,0xd8597fbe,
	0xdd717fbe,0xd8597fc0,0xdd737fc0,0xd8597fc2,0xdd737fc2,0xd8597fc4,0xdd757fc4,0xd8597fc6,
	0xdd757fc6,0xd8597fc8,0xdd777fc8,0xd8597fca,0xdd777fca,0xd8597fcc,0xdd797fcc,0xd8597fce,
	0xdd797fce,0xd8597fd0,0xdd7b7fd0,0xd8597fd2,0xdd7b7fd2,0xd8597fd4,0xdd7d7fd4,0xd8597fd6,
	0xdd7d7fd6,0xd8597fd8,0xdd7f7fd8,0xd8597fda,0xdd7f7fda,0xd8597fdc,0xdd817fdc,0xd8597fde,
	0xdd817fde,0xd8597fe0,0xdd837fe0,0xd8597fe2,0xdd837fe2,0xd8597fe4,0xdd857fe4,0xd8597fe6,
	0xdd857fe6,0xd8597fe8,0xdd877fe8,0xd8597fea,0xdd877fea,0xd8597fec,0xdd897fec,0xd8597fee,
	0xdd897fee,0xd8597ff0,0xdd8b7ff0,0xd8597ff2,0xdd8b7ff2,0xd8597ff4,0xdd8d7ff4,0xd8597ff6,
	0xdd8d7ff6,0xd8597ff8,0xdd8f7ff8,0xd8597ffa,0xdd8f7ffa,0xd8597ffc,0xdd917ffc,0xd8597ffe,
	0xdd917ffe,0xd8598000,0xdd938000,0xd8598002,0xdd938002,0xd8598004,0xdd958004,0xd8598006,
	0xdd958006,0xd8598008,0xdd978008,0xd859800a,0xdd97800a,0xd859800c,0xdd99800c,0xd859800e,
	0xdd99800e,0xd8598010,0xdd9b8010,0xd8598012,0xdd9b8012,0xd8598014,0xdd9d8014,0xd8598016,
	0xdd9d8016,0xd8598018,0xdd9f8018,0xd859801a,0xdd9f801a,0xd859801c,0xdda1801c,0xd859801e,
	0xdda1801e,0xd8598020,0xdda38020,0xd8598022,0xdda38022,0xd8598024,0xdda58024,0xd8598026,
	0xdda58026,0xd8598028,0xdda78028,0xd859802a,0xdda7802a,0xd859802c,0xdda9802c,0xd859802e,
	0xdda9802e,0xd8598030,0xddab8030,0xd8598032,0xddab8032,0xd8598034,0xddad8034,0xd8598036,
	0xddad8036,0xd8598038,0xddaf8038,0xd859803a,0xddaf803a,0xd859803c,0xddb1803c,0xd859803e,
	0xddb1803e,0xd8598040,0xddb38040,0xd8598042,0xddb38042,0xd8598044,0xddb58044,0xd8598046,
	0xddb58046,0xd8598048,0xddb78048,0xd859804a,0xddb7804a,0xd859804c,0xddb9804c,0xd859804e,
	0xddb9804e,0xd8598050,0xddbb8050,0xd8598052,0xddbb8052,0xd8598054,0xddbd8054,0xd8598056,
	0xddbd8056,0xd8598058,0xddbf8058,0xd859805a,0xddbf805a,0xd859805c,0xddc1805c,0xd859805e,
	0xddc1805e,0xd8598060,0xddc38060,0xd8598062,0xddc38062,0xd8598064,0xddc58064,0xd8598066,
	0xddc58066,0xd8598068,0xddc78068,0xd859806a,0xddc7806a,0xd859806c,0xddc9806c,0xd859806e,
	0xddc9806e,0xd8598070,0xddcb8070,0xd8598072,0xddcb8072,0xd8598074,0xddcd8074,0xd8598076,
	0xddcd8076,0xd8598078,0xddcf8078,0xd859807a,0xddcf807a,0xd859807c,0xddd1807c,0xd859807e,
	0xddd1807e,0xd8598080,0xddd38080,0xd8598082,0xddd38082,0xd8598084,0xddd58084,0xd8598086,
	0xddd58086,0xd8598088,0xddd78088,0xd859808a,0xddd7808a,0xd859808c,0xddd9808c,0xd859808e,
	0xddd9808e,0xd8598090,0xdddb8090,0xd8598092,0xdddb8092,0xd8598094,0xdddd8094,0xd8598096,
	0xdddd8096,0xd8598098,0xdddf8098,0xd859809a,0xdddf809a,0xd859809c,0xdde1809c,0xd859809e,
	0xdde1809e,0xd85980a0,0xdde380a0,0xd85980a2,0xdde380a2,0xd85980a4,0xdde580a4,0xd85980a6,
	0xdde580a6,0xd85980a8,0xdde780a8,0xd85980aa,0xdde780aa,0xd85980ac,0xdde980ac,0xd85980ae,
	0xdde980ae,0xd85980b0,0xddeb80b0,0xd85980b2,0xddeb80b2,0xd85980b4,0xdded80b4,0xd85980b6,
	0xdded80b6,0xd85980b8,0xddef80b8,0xd85980ba,0xddef80ba,0xd85980bc,0xddf180bc,0xd85980be,
	0xddf180be,0xd85980c0,0xddf380c0,0xd85980c2,0xddf380c2,0xd85980c4,0xddf580c4,0xd85980c6,
	0xddf580c6,0xd85980c8,0xddf780c8,0xd85980ca,0xddf780ca,0xd85980cc,0xddf980cc,0xd85980ce,
	0xddf980ce,0xd85980d0,0xddfb80d0,0xd85980d2,0xddfb80d2,0xd85980d4,0xddfd80d4,0xd85980d6,
	0xddfd80d6,0xd85980d8,0xddff80d8,0xd85980da,0xddff80da,0xd85980dc,0xde0180dc,0xd85980de,
	0xde0180de,0xd85980e0,0xde0380e0,0xd85980e2,0xde0380e2,0xd85980e4,0xde0580e4,0xd85980e6,
	0xde0580e6,0xd85980e8,0xde0780e8,0xd85980ea,0xde0780ea,0xd85980ec,0xde0980ec,0xd85980ee,
	0xde0980ee,0xd85980f0,0xde0b80f0,0xd85980f2,0xde0b80f2,0xd85980f4,0xde0d80f4,0xd85980f6,
	0xde0d80f6,0xd85980f8,0xde0f80f8,0xd85980fa,0xde0f80fa,0xd85980fc,0xde1180fc,0xd85980fe,
	0xde1180fe,0xd8598100,0xde138100,0xd8598102,0xde138102,0xd8598104,0xde158104,0xd8598106,
	0xde158106,0xd8598108,0xde178108,0xd859810a,0xde17810a,0xd859810c,0xde19810c,0xd859810e,
	0xde19810e,0xd8598110,0xde1b8110,0xd8598112,0xde1b8112,0xd8598114,0xde1d8114,0xd8598116,
	0xde1d8116,0xd8598118,0xde1f8118,0xd859811a,0xde1f811a,0xd859811c,0xde21811c,0xd859811e,
	0xde21811e,0xd8598120,0xde238120,0xd8598122,0xde238122,0xd8598124,0xde258124,0xd8598126,
	0xde258126,0xd8598128,0xde278128,0xd859812a,0xde27812a,0xd859812c,0xde29812c,0xd859812e,
	0xde29812e,0xd8598130,0xde2b8130,0xd8598132,0xde2b8132,0xd8598134,0xde2d8134,0xd8598136,
	0xde2d8136,0xd8598138,0xde2f8138,0xd859813a,0xde2f813a,0xd859813c,0xde31813c,0xd859813e,
	0xde31813e,0xd8598140,0xde338140,0xd8598142,0xde338142,0xd8598144,0xde358144,0xd8598146,
	0xde358146,0xd8598148,0xde378148,0xd859814a,0xde37814a,0xd859814c,0xde39814c,0xd859814e,
	0xde39814e,0xd8598150,0xde3b8150,0xd8598152,0xde3b8152,0xd8598154,0xde3d8154,0xd8598156,
	0xde3d8156,0xd8598158,0xde3f8158,0xd859815a,0xde3f815a,0xd859815c,0xde41815c,0xd859815e,
	0xde41815e,0xd8598160,0xde438160,0xd8598162,0xde438162,0xd8598164,0xde458164,0xd8598166,
	0xde458166,0xd8598168,0xde478168,0xd859816a,0xde47816a,0xd859816c,0xde49816c,0xd859816e,
	0xde49816e,0xd8598170,0xde4b8170,0xd8598172,0xde4b8172,0xd8598174,0xde4d8174,0xd8598176,
	0xde4d8176,0xd8598178,0xde4f8178,0xd859817a,0xde4f817a,0xd859817c,0xde51817c,0xd859817e,
	0xde51817e,0xd8598180,0xde538180,0xd8598182,0xde538182,0xd8598184,0xde558184,0xd8598186,
	0xde558186,0xd8598188,0xde578188,0xd859818a,0xde57818a,0xd859818c,0xde59818c,0xd859818e,
	0xde59818e,0xd8598190,0xde5b8190,0xd8598192,0xde5b8192,0xd8598194,0xde5d8194,0xd8598196,
	0xde5d8196,0xd8598198,0xde5f8198,0xd859819a,0xde5f819a,0xd859819c,0xde61819c,0xd859819e,
	0xde61819e,0xd85981a0,0xde6381a0,0xd85981a2,0xde6381a2,0xd85981a4,0xde6581a4,0xd85981a6,
	0xde6581a6,0xd85981a8,0xde6781a8,0xd85981aa,0xde6781aa,0xd85981ac,0xde6981ac,0xd85981ae,
	0xde6981ae,0xd85981b0,0xde6b81b0,0xd85981b2,0xde6b81b2,0xd85981b4,0xde6d81b4,0xd85981b6,
	0xde6d81b6,0xd85981b8,0xde6f81b8,0xd85981ba,0xde6f81ba,0xd85981bc,0xde7181bc,0xd85981be,
	0xde7181be,0xd85981c0,0xde7381c0,0xd85981c2,0xde7381c2,0xd85981c4,0xde7581c4,0xd85981c6,
	0xde7581c6,0xd85981c8,0xde7781c8,0xd85981ca,0xde7781ca,0xd85981cc,0xde7981cc,0xd85981ce,
	0xde7981ce,0xd85981d0,0xde7b81d0,0xd85981d2,0xde7b81d2,0xd85981d4,0xde7d81d4,0xd85981d6,
	0xde7d81d6,0xd85981d8,0xde7f81d8,0xd85981da,0xde7f81da,0xd85981dc,0xde8181dc,0xd85981de,
	0xde8181de,0xd85981e0,0xde8381e0,0xd85981e2,0xde8381e2,0xd85981e4,0xde8581e4,0xd85981e6,
	0xde8581e6,0xd85981e8,0xde8781e8,0xd85981ea,0xde8781ea,0xd85981ec,0xde8981ec,0xd85981ee,
	0xde8981ee,0xd85981f0,0xde8b81f0,0xd85981f2,0xde8b81f2,0xd85981f4,0xde8d81f4,0xd85981f6,
	0xde8d81f6,0xd85981f8,0xde8f81f8,0xd85981fa,0xde8f81fa,0xd85981fc,0xde9181fc,0xd85981fe,
	0xde9181fe,0xd8598200,0xde938200,0xd8598202,0xde938202,0xd8598204,0xde958204,0xd8598206,
	0xde958206,0xd8598208,0xde978208,0xd859820a,0xde97820a,0xd859820c,0xde99820c,0xd859820e,
	0xde99820e,0xd8598210,0xde9b8210,0xd8598212,0xde9b8212,0xd8598214,0xde9d8214,0xd8598216,
	0xde9d8216,0xd8598218,0xde9f8218,0xd859821a,0xde9f821a,0xd859821c,0xdea1821c,0xd859821e,
	0xdea1821e,0xd8598220,0xdea38220,0xd8598222,0xdea38222,0xd8598224,0xdea58224,0xd8598226,
	0xdea58226,0xd8598228,0xdea78228,0xd859822a,0xdea7822a,0xd859822c,0xdea9822c,0xd859822e,
	0xdea9822e,0xd8598230,0xdeab8230,0xd8598232,0xdeab8232,0xd8598234,0xdead8234,0xd8598236,
	0xdead8236,0xd8598238,0xdeaf8238,0xd859823a,0xdeaf823a,0xd859823c,0xdeb1823c,0xd859823e,
	0xdeb1823e,0xd8598240,0xdeb38240,0xd8598242,0xdeb38242,0xd8598244,0xdeb58244,0xd8598246,
	0xdeb58246,0xd8598248,0xdeb78248,0xd859824a,0xdeb7824a,0xd859824c,0xdeb9824c,0xd859824e,
	0xdeb9824e,0xd8598250,0xdebb8250,0xd8598252,0xdebb8252,0xd8598254,0xdebd8254,0xd8598256,
	0xdebd8256,0xd8598258,0xdebf8258,0xd859825a,0xdebf825a,0xd859825c,0xdec1825c,0xd859825e,
	0xdec1825e,0xd8598260,0xdec38260,0xd8598262,0xdec38262,0xd8598264,0xdec58264,0xd8598266,
	0xdec58266,0xd8598268,0xdec78268,0xd859826a,0xdec7826a,0xd859826c,0xdec9826c,0xd859826e,
	0xdec9826e,0xd8598270,0xdecb8270,0xd8598272,0xdecb8272,0xd8598274,0xdecd8274,0xd8598276,
	0xdecd8276,0xd8598278,0xdecf8278,0xd859827a,0xdecf827a,0xd859827c,0xded1827c,0xd859827e,
	0xded1827e,0xd8598280,0xded38280,0xd8598282,0xded38282,0xd8598284,0xded58284,0xd8598286,
	0xded58286,0xd8598288,0xded78288,0xd859828a,0xded7828a,0xd859828c,0xded9828c,0xd859828e,
	0xded9828e,0xd8598290,0xdedb8290,0xd8598292,0xdedb8292,0xd8598294,0xdedd8294,0xd8598296,
	0xdedd8296,0xd8598298,0xdedf8298,0xd859829a,0xdedf829a,0xd859829c,0xdee1829c,0xd859829e,
	0xdee1829e,0xd85982a0,0xdee382a0,0xd85982a2,0xdee382a2,0xd85982a4,0xdee582a4,0xd85982a6,
	0xdee582a6,0xd85982a8,0xdee782a8,0xd85982aa,0xdee782aa,0xd85982ac,0xdee982ac,0xd85982ae,
	0xdee982ae,0xd85982b0,0xdeeb82b0,0xd85982b2,0xdeeb82b2,0xd85982b4,0xdeed82b4,0xd85982b6,
	0xdeed82b6,0xd85982b8,0xdeef82b8,0xd85982ba,0xdeef82ba,0xd85982bc,0xdef182bc,0xd85982be,
	0xdef182be,0xd85982c0,0xdef382c0,0xd85982c2,0xdef382c2,0xd85982c4,0xdef582c4,0xd85982c6,
	0xdef582c6,0xd85982c8,0xdef782c8,0xd85982ca,0xdef782ca,0xd85982cc,0xdef982cc,0xd85982ce,
	0xdef982ce,0xd85982d0,0xdefb82d0,0xd85982d2,0xdefb82d2,0xd85982d4,0xdefd82d4,0xd85982d6,
	0xdefd82d6,0xd85982d8,0xdeff82d8,0xd85982da,0xdeff82da,0xd85982dc,0xdf0182dc,0xd85982de,
	0xdf0182de,0xd85982e0,0xdf0382e0,0xd85982e2,0xdf0382e2,0xd85982e4,0xdf0582e4,0xd85982e6,
	0xdf0582e6,0xd85982e8,0xdf0782e8,0xd85982ea,0xdf0782ea,0xd85982ec,0xdf0982ec,0xd85982ee,
	0xdf0982ee,0xd85982f0,0xdf0b82f0,0xd85982f2,0xdf0b82f2,0xd85982f4,0xdf0d82f4,0xd85982f6,
	0xdf0d82f6,0xd85982f8,0xdf0f82f8,0xd85982fa,0xdf0f82fa,0xd85982fc,0xdf1182fc,0xd85982fe,
	0xdf1182fe,0xd8598300,0xdf138300,0xd8598302,0xdf138302,0xd8598304,0xdf158304,0xd8598306,
	0xdf158306,0xd8598308,0xdf178308,0xd859830a,0xdf17830a,0xd859830c,0xdf19830c,0xd859830e,
	0xdf19830e,0xd8598310,0xdf1b8310,0xd8598312,0xdf1b8312,0xd8598314,0xdf1d8314,0xd8598316,
	0xdf1d8316,0xd8598318,0xdf1f8318,0xd859831a,0xdf1f831a,0xd859831c,0xdf21831c,0xd859831e,
	0xdf21831e,0xd8598320,0xdf238320,0xd8598322,0xdf238322,0xd8598324,0xdf258324,0xd8598326,
	0xdf258326,0xd8598328,0xdf278328,0xd859832a,0xdf27832a,0xd859832c,0xdf29832c,0xd859832e,
	0xdf29832e,0xd8598330,0xdf2b8330,0xd8598332,0xdf2b8332,0xd8598334,0xdf2d8334,0xd8598336,
	0xdf2d8336,0xd8598338,0xdf2f8338,0xd859833a,0xdf2f833a,0xd859833c,0xdf31833c,0xd859833e,
	0xdf31833e,0xd8598340,0xdf338340,0xd8598342,0xdf338342,0xd8598344,0xdf358344,0xd8598346,
	0xdf358346,0xd8598348,0xdf378348,0xd859834a,0xdf37834a,0xd859834c,0xdf39834c,0xd859834e,
	0xdf39834e,0xd8598350,0xdf3b8350,0xd8598352,0xdf3b8352,0xd8598354,0xdf3d8354,0xd8598356,
	0xdf3d8356,0xd8598358,0xdf3f8358,0xd859835a,0xdf3f835a,0xd859835c,0xdf41835c,0xd859835e,
	0xdf41835e,0xd8598360,0xdf438360,0xd8598362,0xdf438362,0xd8598364,0xdf458364,0xd8598366,
	0xdf458366,0xd8598368,0xdf478368,0xd859836a,0xdf47836a,0xd859836c,0xdf49836c,0xd859836e,
	0xdf49836e,0xd8598370,0xdf4b8370,0xd8598372,0xdf4b8372,0xd8598374,0xdf4d8374,0xd8598376,
	0xdf4d8376,0xd8598378,0xdf4f8378,0xd859837a,0xdf4f837a,0xd859837c,0xdf51837c,0xd859837e,
	0xdf51837e,0xd8598380,0xdf538380,0xd8598382,0xdf538382,0xd8598384,0xdf558384,0xd8598386,
	0xdf558386,0xd8598388,0xdf578388,0xd859838a,0xdf57838a,0xd859838c,0xdf59838c,0xd859838e,
	0xdf59838e,0xd8598390,0xdf5b8390,0xd8598392,0xdf5b8392,0xd8598394,0xdf5d8394,0xd8598396,
	0xdf5d8396,0xd8598398,0xdf5f8398,0xd859839a,0xdf5f839a,0xd859839c,0xdf61839c,0xd859839e,
	0xdf61839e,0xd85983a0,0xdf6383a0,0xd85983a2,0xdf6383a2,0xd85983a4,0xdf6583a4,0xd85983a6,
	0xdf6583a6,0xd85983a8,0xdf6783a8,0xd85983aa,0xdf6783aa,0xd85983ac,0xdf6983ac,0xd85983ae,
	0xdf6983ae,0xd85983b0,0xdf6b83b0,0xd85983b2,0xdf6b83b2,0xd85983b4,0xdf6d83b4,0xd85983b6,
	0xdf6d83b6,0xd85983b8,0xdf6f83b8,0xd85983ba,0xdf6f83ba,0xd85983bc,0xdf7183bc,0xd85983be,
	0xdf7183be,0xd85983c0,0xdf7383c0,0xd85983c2,0xdf7383c2,0xd85983c4,0xdf7583c4,0xd85983c6,
	0xdf7583c6,0xd85983c8,0xdf7783c8,0xd85983ca,0xdf7783ca,0xd85983cc,0xdf7983cc,0xd85983ce,
	0xdf7983ce,0xd85983d0,0xdf7b83d0,0xd85983d2,0xdf7b83d2,0xd85983d4,0xdf7d83d4,0xd85983d6,
	0xdf7d83d6,0xd85983d8,0xdf7f83d8,0xd85983da,0xdf7f83da,0xd85983dc,0xdf8183dc,0xd85983de,
	0xdf8183de,0xd85983e0,0xdf8383e0,0xd85983e2,0xdf8383e2,0xd85983e4,0xdf8583e4,0xd85983e6,
	0xdf8583e6,0xd85983e8,0xdf8783e8,0xd85983ea,0xdf8783ea,0xd85983ec,0xdf8983ec,0xd85983ee,
	0xdf8983ee,0xd85983f0,0xdf8b83f0,0xd85983f2,0xdf8b83f2,0xd85983f4,0xdf8d83f4,0xd85983f6,
	0xdf8d83f6,0xd85983f8,0xdf8f83f8,0xd85983fa,0xdf8f83fa,0xd85983fc,0xdf9183fc,0xd85983fe,
	0xdf9183fe,0xd8598400,0xdf938400,0xd8598402,0xdf938402,0xd8598404,0xdf958404,0xd8598406,
	0xdf958406,0xd8598408,0xdf978408,0xd859840a,0xdf97840a,0xd859840c,0xdf99840c,0xd859840e,
	0xdf99840e,0xd8598410,0xdf9b8410,0xd8598412,0xdf9b8412,0xd8598414,0xdf9d8414,0xd8598416,
	0xdf9d8416,0xd8598418,0xdf9f8418,0xd859841a,0xdf9f841a,0xd859841c,0xdfa1841c,0xd859841e,
	0xdfa1841e,0xd8598420,0xdfa38420,0xd8598422,0xdfa38422,0xd8598424,0xdfa58424,0xd8598426,
	0xdfa58426,0xd8598428,0xdfa78428,0xd859842a,0xdfa7842a,0xd859842c,0xdfa9842c,0xd859842e,
	0xdfa9842e,0xd8598430,0xdfab8430,0xd8598432,0xdfab8432,0xd8598434,0xdfad8434,0xd8598436,
	0xdfad8436,0xd8598438,0xdfaf8438,0xd859843a,0xdfaf843a,0xd859843c,0xdfb1843c,0xd859843e,
	0xdfb1843e,0xd8598440,0xdfb38440,0xd8598442,0xdfb38442,0xd8598444,0xdfb58444,0xd8598446,
	0xdfb58446,0xd8598448,0xdfb78448,0xd859844a,0xdfb7844a,0xd859844c,0xdfb9844c,0xd859844e,
	0xdfb9844e,0xd8598450,0xdfbb8450,0xd8598452,0xdfbb8452,0xd8598454,0xdfbd8454,0xd8598456,
	0xdfbd8456,0xd8598458,0xdfbf8458,0xd859845a,0xdfbf845a,0xd859845c,0xdfc1845c,0xd859845e,
	0xdfc1845e,0xd8598460,0xdfc38460,0xd8598462,0xdfc38462,0xd8598464,0xdfc58464,0xd8598466,
	0xdfc58466,0xd8598468,0xdfc78468,0xd859846a,0xdfc7846a,0xd859846c,0xdfc9846c,0xd859846e,
	0xdfc9846e,0xd8598470,0xdfcb8470,0xd8598472,0xdfcb8472,0xd8598474,0xdfcd8474,0xd8598476,
	0xdfcd8476,0xd8598478,0xdfcf8478,0xd859847a,0xdfcf847a,0xd859847c,0xdfd1847c,0xd859847e,
	0xdfd1847e,0xd8598480,0xdfd38480,0xd8598482,0xdfd38482,0xd8598484,0xdfd58484,0xd8598486,
	0xdfd58486,0xd8598488,0xdfd78488,0xd859848a,0xdfd7848a,0xd859848c,0xdfd9848c,0xd859848e,
	0xdfd9848e,0xd8598490,0xdfdb8490,0xd8598492,0xdfdb8492,0xd8598494,0xdfdd8494,0xd8598496,
	0xdfdd8496,0xd8598498,0xdfdf8498,0xd859849a,0xdfdf849a,0xd859849c,0xdfe1849c,0xd859849e,
	0xdfe1849e,0xd85984a0,0xdfe384a0,0xd85984a2,0xdfe384a2,0xd85984a4,0xdfe584a4,0xd85984a6,
	0xdfe584a6,0xd85984a8,0xdfe784a8,0xd85984aa,0xdfe784aa,0xd85984ac,0xdfe984ac,0xd85984ae,
	0xdfe984ae,0xd85984b0,0xdfeb84b0,0xd85984b2,0xdfeb84b2,0xd85984b4,0xdfed84b4,0xd85984b6,
	0xdfed84b6,0xd85984b8,0xdfef84b8,0xd85984ba,0xdfef84ba,0xd85984bc,0xdff184bc,0xd85984be,
	0xdff184be,0xd85984c0,0xdff384c0,0xd85984c2,0xdff384c2,0xd85984c4,0xdff584c4,0xd85984c6,
	0xdff584c6,0xd85984c8,0xdff784c8,0xd85984ca,0xdff784ca,0xd85984cc,0xdff984cc,0xd85984ce,
	0xdff984ce,0xd85984d0,0xdffb84d0,0xd85984d2,0xdffb84d2,0xd85984d4,0xdffd84d4,0xd85984d6,
	0xdffd84d6,0xd85984d8,0xdfff84d8,0xd85984da,0xdfff84da,0xd85984dc,0xdc0184dc,0xd85984de,
	0xdc0184de,0xd85984e0,0xdc0384e0,0xd85984e2,0xdc0384e2,0xd85984e4,0xdc0584e4,0xd85984e6,
	0xdc0584e6,0xd85984e8,0xdc0784e8,0xd85984ea,0xdc0784ea,0xd85984ec,0xdc0984ec,0xd85984ee,
	0xdc0984ee,0xd85984f0,0xdc0b84f0,0xd85984f2,0xdc0b84f2,0xd85984f4,0xdc0d84f4,0xd85984f6,
	0xdc0d84f6,0xd85984f8,0xdc0f84f8,0xd85984fa,0xdc0f84fa,0xd85984fc,0xdc1184fc,0xd85984fe,
	0xdc1184fe,0xd8598500,0xdc138500,0xd8598502,0xdc138502,0xd8598504,0xdc158504,0xd8598506,
	0xdc158506,0xd8598508,0xdc178508,0xd859850a,0xdc17850a,0xd859850c,0xdc19850c,0xd859850e,
	0xdc19850e,0xd8598510,0xdc1b8510,0xd8598512,0xdc1b8512,0xd8598514,0xdc1d8514,0xd8598516,
	0xdc1d8516,0xd8598518,0xdc1f8518,0xd859851a,0xdc1f851a,0xd859851c,0xdc21851c,0xd859851e,
	0xdc21851e,0xd8598520,0xdc238520,0xd8598522,0xdc238522,0xd8598524,0xdc258524,0xd8598526,
	0xdc258526,0xd8598528,0xdc278528,0xd859852a,0xdc27852a,0xd859852c,0xdc29852c,0xd859852e,
	0xdc29852e,0xd8598530,0xdc2b8530,0xd8598532,0xdc2b8532,0xd8598534,0xdc2d8534,0xd8598536,
	0xdc2d8536,0xd8598538,0xdc2f8538,0xd859853a,0xdc2f853a,0xd859853c,0xdc31853c,0xd859853e,
	0xdc31853e,0xd8598540,0xdc338540,0xd8598542,0xdc338542,0xd8598544,0xdc358544,0xd8598546,
	0xdc358546,0xd8598548,0xdc378548,0xd859854a,0xdc37854a,0xd859854c,0xdc39854c,0xd859854e,
	0xdc39854e,0xd8598550,0xdc3b8550,0xd8598552,0xdc3b8552,0xd8598554,0xdc3d8554,0xd8598556,
	0xdc3d8556,0xd8598558,0xdc3f8558,0xd859855a,0xdc3f855a,0xd859855c,0xdc41855c,0xd859855e,
	0xdc41855e,0xd8598560,0xdc438560,0xd8598562,0xdc438562,0xd8598564,0xdc458564,0xd8598566,
	0xdc458566,0xd8598568,0xdc478568,0xd859856a,0xdc47856a,0xd859856c,0xdc49856c,0xd859856e,
	0xdc49856e,0xd8598570,0xdc4b8570,0xd8598572,0xdc4b8572,0xd8598574,0xdc4d8574,0xd8598576,
	0xdc4d8576,0xd8598578,0xdc4f8578,0xd859857a,0xdc4f857a,0xd859857c,0xdc51857c,0xd859857e,
	0xdc51857e,0xd8598580,0xdc538580,0xd8598582,0xdc538582,0xd8598584,0xdc558584,0xd8598586,
	0xdc558586,0xd8598588,0xdc578588,0xd859858a,0xdc57858a,0xd859858c,0xdc59858c,0xd859858e,
	0xdc59858e,0xd8598590,0xdc5b8590,0xd8598592,0xdc5b8592,0xd8598594,0xdc5d8594,0xd8598596,
	0xdc5d8596,0xd8598598,0xdc5f8598,0xd859859a,0xdc5f859a,0xd859859c,0xdc61859c,0xd859859e,
	0xdc61859e,0xd85985a0,0xdc6385a0,0xd85985a2,0xdc6385a2,0xd85985a4,0xdc6585a4,0xd85985a6,
	0xdc6585a6,0xd85985a8,0xdc6785a8,0xd85985aa,0xdc6785aa,0xd85985ac,0xdc6985ac,0xd85985ae,
	0xdc6985ae,0xd85985b0,0xdc6b85b0,0xd85985b2,0xdc6b85b2,0xd85985b4,0xdc6d85b4,0xd85985b6,
	0xdc6d85b6,0xd85985b8,0xdc6f85b8,0xd85985ba,0xdc6f85ba,0xd85985bc,0xdc7185bc,0xd85985be,
	0xdc7185be,0xd85985c0,0xdc7385c0,0xd85985c2,0xdc7385c2,0xd85985c4,0xdc7585c4,0xd85985c6,
	0xdc7585c6,0xd85985c8,0xdc7785c8,0xd85985ca,0xdc7785ca,0xd85985cc,0xdc7985cc,0xd85985ce,
	0xdc7985ce,0xd85985d0,0xdc7b85d0,0xd85985d2,0xdc7b85d2,0xd85985d4,0xdc7d85d4,0xd85985d6,
	0xdc7d85d6,0xd85985d8,0xdc7f85d8,0xd85985da,0xdc7f85da,0xd85985dc,0xdc8185dc,0xd85985de,
	0xdc8185de,0xd85985e0,0xdc8385e0,0xd85985e2,0xdc8385e2,0xd85985e4,0xdc8585e4,0xd85985e6,
	0xdc8585e6,0xd85985e8,0xdc8785e8,0xd85985ea,0xdc8785ea,0xd85985ec,0xdc8985ec,0xd85985ee,
	0xdc8985ee,0xd85985f0,0xdc8b85f0,0xd85985f2,0xdc8b85f2,0xd85985f4,0xdc8d85f4,0xd85985f6,
	0xdc8d85f6,0xd85985f8,0xdc8f85f8,0xd85985fa,0xdc8f85fa,0xd85985fc,0xdc9185fc,0xd85985fe,
	0xdc9185fe,0xd8598600,0xdc938600,0xd8598602,0xdc938602,0xd8598604,0xdc958604,0xd8598606,
	0xdc958606,0xd8598608,0xdc978608,0xd859860a,0xdc97860a,0xd859860c,0xdc99860c,0xd859860e,
	0xdc99860e,0xd8598610,0xdc9b8610,0xd8598612,0xdc9b8612,0xd8598614,0xdc9d8614,0xd8598616,
	0xdc9d8616,0xd8598618,0xdc9f8618,0xd859861a,0xdc9f861a,0xd859861c,0xdca1861c,0xd859861e,
	0xdca1861e,0xd8598620,0xdca38620,0xd8598622,0xdca38622,0xd8598624,0xdca58624,0xd8598626,
	0xdca58626,0xd8598628,0xdca78628,0xd859862a,0xdca7862a,0xd859862c,0xdca9862c,0xd859862e,
	0xdca9862e,0xd8598630,0xdcab8630,0xd8598632,0xdcab8632,0xd8598634,0xdcad8634,0xd8598636,
	0xdcad8636,0xd8598638,0xdcaf8638,0xd859863a,0xdcaf863a,0xd859863c,0xdcb1863c,0xd859863e,
	0xdcb1863e,0xd8598640,0xdcb38640,0xd8598642,0xdcb38642,0xd8598644,0xdcb58644,0xd8598646,
	0xdcb58646,0xd8598648,0xdcb78648,0xd859864a,0xdcb7864a,0xd859864c,0xdcb9864c,0xd859864e,
	0xdcb9864e,0xd8598650,0xdcbb8650,0xd8598652,0xdcbb8652,0xd8598654,0xdcbd8654,0xd8598656,
	0xdcbd8656,0xd8598658,0xdcbf8658,0xd859865a,0xdcbf865a,0xd859865c,0xdcc1865c,0xd859865e,
	0xdcc1865e,0xd8598660,0xdcc38660,0xd8598662,0xdcc38662,0xd8598664,0xdcc58664,0xd8598666,
	0xdcc58666,0xd8598668,0xdcc78668,0xd859866a,0xdcc7866a,0xd859866c,0xdcc9866c,0xd859866e,
	0xdcc9866e,0xd8598670,0xdccb8670,0xd8598672,0xdccb8672,0xd8598674,0xdccd8674,0xd8598676,
	0xdccd8676,0xd8598678,0xdccf8678,0xd859867a,0xdccf867a,0xd859867c,0xdcd1867c,0xd859867e,
	0xdcd1867e,0xd8598680,0xdcd38680,0xd8598682,0xdcd38682,0xd8598684,0xdcd58684,0xd8598686,
	0xdcd58686,0xd8598688,0xdcd78688,0xd859868a,0xdcd7868a,0xd859868c,0xdcd9868c,0xd859868e,
	0xdcd9868e,0xd8598690,0xdcdb8690,0xd8598692,0xdcdb8692,0xd8598694,0xdcdd8694,0xd8598696,
	0xdcdd8696,0xd8598698,0xdcdf8698,0xd859869a,0xdcdf869a,0xd859869c,0xdce1869c,0xd859869e,
	0xdce1869e,0xd85986a0,0xdce386a0,0xd85986a2,0xdce386a2,0xd85986a4,0xdce586a4,0xd85986a6,
	0xdce586a6,0xd85986a8,0xdce786a8,0xd85986aa,0xdce786aa,0xd85986ac,0xdce986ac,0xd85986ae,
	0xdce986ae,0xd85986b0,0xdceb86b0,0xd85986b2,0xdceb86b2,0xd85986b4,0xdced86b4,0xd85986b6,
	0xdced86b6,0xd85986b8,0xdcef86b8,0xd85986ba,0xdcef86ba,0xd85986bc,0xdcf186bc,0xd85986be,
	0xdcf186be,0xd85986c0,0xdcf386c0,0xd85986c2,0xdcf386c2,0xd85986c4,0xdcf586c4,0xd85986c6,
	0xdcf586c6,0xd85986c8,0xdcf786c8,0xd85986ca,0xdcf786ca,0xd85986cc,0xdcf986cc,0xd85986ce,
	0xdcf986ce,0xd85986d0,0xdcfb86d0,0xd85986d2,0xdcfb86d2,0xd85986d4,0xdcfd86d4,0xd85986d6,
	0xdcfd86d6,0xd85986d8,0xdcff86d8,0xd85986da,0xdcff86da,0xd85986dc,0xdd0186dc,0xd85986de,
	0xdd0186de,0xd85986e0,0xdd0386e0,0xd85986e2,0xdd0386e2,0xd85986e4,0xdd0586e4,0xd85986e6,
	0xdd0586e6,0xd85986e8,0xdd0786e8,0xd85986ea,0xdd0786ea,0xd85986ec,0xdd0986ec,0xd85986ee,
	0xdd0986ee,0xd85986f0,0xdd0b86f0,0xd85986f2,0xdd0b86f2,0xd85986f4,0xdd0d86f4,0xd85986f6,
	0xdd0d86f6,0xd85986f8,0xdd0f86f8,0xd85986fa,0xdd0f86fa,0xd85986fc,0xdd1186fc,0xd85986fe,
	0xdd1186fe,0xd8598700,0xdd138700,0xd8598702,0xdd138702,0xd8598704,0xdd158704,0xd8598706,
	0xdd158706,0xd8598708,0xdd178708,0xd859870a,0xdd17870a,0xd859870c,0xdd19870c,0xd859870e,
	0xdd19870e,0xd8598710,0xdd1b8710,0xd8598712,0xdd1b8712,0xd8598714,0xdd1d8714,0xd8598716,
	0xdd1d8716,0xd8598718,0xdd1f8718,0xd859871a,0xdd1f871a,0xd859871c,0xdd21871c,0xd859871e,
	0xdd21871e,0xd8598720,0xdd238720,0xd8598722,0xdd238722,0xd8598724,0xdd258724,0xd8598726,
	0xdd258726,0xd8598728,0xdd278728,0xd859872a,0xdd27872a,0xd859872c,0xdd29872c,0xd859872e,
	0xdd29872e,0xd8598730,0xdd2b8730,0xd8598732,0xdd2b8732,0xd8598734,0xdd2d8734,0xd8598736,
	0xdd2d8736,0xd8598738,0xdd2f8738,0xd859873a,0xdd2f873a,0xd859873c,0xdd31873c,0xd859873e,
	0xdd31873e,0xd8598740,0xdd338740,0xd8598742,0xdd338742,0xd8598744,0xdd358744,0xd8598746,
	0xdd358746,0xd8598748,0xdd378748,0xd859874a,0xdd37874a,0xd859874c,0xdd39874c,0xd859874e,
	0xdd39874e,0xd8598750,0xdd3b8750,0xd8598752,0xdd3b8752,0xd8598754,0xdd3d8754,0xd8598756,
	0xdd3d8756,0xd8598758,0xdd3f8758,0xd859875a,0xdd3f875a,0xd859875c,0xdd41875c,0xd859875e,
	0xdd41875e,0xd8598760,0xdd438760,0xd8598762,0xdd438762,0xd8598764,0xdd458764,0xd8598766,
	0xdd458766,0xd8598768,0xdd478768,0xd859876a,0xdd47876a,0xd859876c,0xdd49876c,0xd859876e,
	0xdd49876e,0xd8598770,0xdd4b8770,0xd8598772,0xdd4b8772,0xd8598774,0xdd4d8774,0xd8598776,
	0xdd4d8776,0xd8598778,0xdd4f8778,0xd859877a,0xdd4f877a,0xd859877c,0xdd51877c,0xd859877e,
	0xdd51877e,0xd8598780,0xdd538780,0xd8598782,0xdd538782,0xd8598784,0xdd558784,0xd8598786,
	0xdd558786,0xd8598788,0xdd578788,0xd859878a,0xdd57878a,0xd859878c,0xdd59878c,0xd859878e,
	0xdd59878e,0xd8598790,0xdd5b8790,0xd8598792,0xdd5b8792,0xd8598794,0xdd5d8794,0xd8598796,
	0xdd5d8796,0xd8598798,0xdd5f8798,0xd859879a,0xdd5f879a,0xd859879c,0xdd61879c,0xd859879e,
	0xdd61879e,0xd85987a0,0xdd6387a0,0xd85987a2,0xdd6387a2,0xd85987a4,0xdd6587a4,0xd85987a6,
	0xdd6587a6,0xd85987a8,0xdd6787a8,0xd85987aa,0xdd6787aa,0xd85987ac,0xdd6987ac,0xd85987ae,
	0xdd6987ae,0xd85987b0,0xdd6b87b0,0xd85987b2,0xdd6b87b2,0xd85987b4,0xdd6d87b4,0xd85987b6,
	0xdd6d87b6,0xd85987b8,0xdd6f87b8,0xd85987ba,0xdd6f87ba,0xd85987bc,0xdd7187bc,0xd85987be,
	0xdd7187be,0xd85987c0,0xdd7387c0,0xd85987c2,0xdd7387c2,0xd85987c4,0xdd7587c4,0xd85987c6,
	0xdd7587c6,0xd85987c8,0xdd7787c8,0xd85987ca,0xdd7787ca,0xd85987cc,0xdd7987cc,0xd85987ce,
	0xdd7987ce,0xd85987d0,0xdd7b87d0,0xd85987d2,0xdd7b87d2,0xd85987d4,0xdd7d87d4,0xd85987d6,
	0xdd7d87d6,0xd85987d8,0xdd7f87d8,0xd85987da,0xdd7f87da,0xd85987dc,0xdd8187dc,0xd85987de,
	0xdd8187de,0xd85987e0,0xdd8387e0,0xd85987e2,0xdd8387e2,0xd85987e4,0xdd8587e4,0xd85987e6,
	0xdd8587e6,0xd85987e8,0xdd8787e8,0xd85987ea,0xdd8787ea,0xd85987ec,0xdd8987ec,0xd85987ee,
	0xdd8987ee,0xd85987f0,0xdd8b87f0,0xd85987f2,0xdd8b87f2,0xd85987f4,0xdd8d87f4,0xd85987f6,
	0xdd8d87f6,0xd85987f8,0xdd8f87f8,0xd85987fa,0xdd8f87fa,0xd85987fc,0xdd9187fc,0xd85987fe,
	0xdd9187fe,0xd8598800,0xdd938800,0xd8598802,0xdd938802,0xd8598804,0xdd958804,0xd8598806,
	0xdd958806,0xd8598808,0xdd978808,0xd859880a,0xdd97880a,0xd859880c,0xdd99880c,0xd859880e,
	0xdd99880e,0xd8598810,0xdd9b8810,0xd8598812,0xdd9b8812,0xd8598814,0xdd9d8814,0xd8598816,
	0xdd9d8816,0xd8598818,0xdd9f8818,0xd859881a,0xdd9f881a,0xd859881c,0xdda1881c,0xd859881e,
	0xdda1881e,0xd8598820,0xdda38820,0xd8598822,0xdda38822,0xd8598824,0xdda58824,0xd8598826,
	0xdda58826,0xd8598828,0xdda78828,0xd859882a,0xdda7882a,0xd859882c,0xdda9882c,0xd859882e,
	0xdda9882e,0xd8598830,0xddab8830,0xd8598832,0xddab8832,0xd8598834,0xddad8834,0xd8598836,
	0xddad8836,0xd8598838,0xddaf8838,0xd859883a,0xddaf883a,0xd859883c,0xddb1883c,0xd859883e,
	0xddb1883e,0xd8598840,0xddb38840,0xd8598842,0xddb38842,0xd8598844,0xddb58844,0xd8598846,
	0xddb58846,0xd8598848,0xddb78848,0xd859884a,0xddb7884a,0xd859884c,0xddb9884c,0xd859884e,
	0xddb9884e,0xd8598850,0xddbb8850,0xd8598852,0xddbb8852,0xd8598854,0xddbd8854,0xd8598856,
	0xddbd8856,0xd8598858,0xddbf8858,0xd859885a,0xddbf885a,0xd859885c,0xddc1885c,0xd859885e,
	0xddc1885e,0xd8598860,0xddc38860,0xd8598862,0xddc38862,0xd8598864,0xddc58864,0xd8598866,
	0xddc58866,0xd8598868,0xddc78868,0xd859886a,0xddc7886a,0xd859886c,0xddc9886c,0xd859886e,
	0xddc9886e,0xd8598870,0xddcb8870,0xd8598872,0xddcb8872,0xd8598874,0xddcd8874,0xd8598876,
	0xddcd8876,0xd8598878,0xddcf8878,0xd859887a,0xddcf887a,0xd859887c,0xddd1887c,0xd859887e,
	0xddd1887e,0xd8598880,0xddd38880,0xd8598882,0xddd38882,0xd8598884,0xddd58884,0xd8598886,
	0xddd58886,0xd8598888,0xddd78888,0xd859888a,0xddd7888a,0xd859888c,0xddd9888c,0xd859888e,
	0xddd9888e,0xd8598890,0xdddb8890,0xd8598892,0xdddb8892,0xd8598894,0xdddd8894,0xd8598896,
	0xdddd8896,0xd8598898,0xdddf8898,0xd859889a,0xdddf889a,0xd859889c,0xdde1889c,0xd859889e,
	0xdde1889e,0xd85988a0,0xdde388a0,0xd85988a2,0xdde388a2,0xd85988a4,0xdde588a4,0xd85988a6,
	0xdde588a6,0xd85988a8,0xdde788a8,0xd85988aa,0xdde788aa,0xd85988ac,0xdde988ac,0xd85988ae,
	0xdde988ae,0xd85988b0,0xddeb88b0,0xd85988b2,0xddeb88b2,0xd85988b4,0xdded88b4,0xd85988b6,
	0xdded88b6,0xd85988b8,0xddef88b8,0xd85988ba,0xddef88ba,0xd85988bc,0xddf188bc,0xd85988be,
	0xddf188be,0xd85988c0,0xddf388c0,0xd85988c2,0xddf388c2,0xd85988c4,0xddf588c4,0xd85988c6,
	0xddf588c6,0xd85988c8,0xddf788c8,0xd85988ca,0xddf788ca,0xd85988cc,0xddf988cc,0xd85988ce,
	0xddf988ce,0xd85988d0,0xddfb88d0,0xd85988d2,0xddfb88d2,0xd85988d4,0xddfd88d4,0xd85988d6,
	0xddfd88d6,0xd85988d8,0xddff88d8,0xd85988da,0xddff88da,0xd85988dc,0xde0188dc,0xd85988de,
	0xde0188de,0xd85988e0,0xde0388e0,0xd85988e2,0xde0388e2,0xd85988e4,0xde0588e4,0xd85988e6,
	0xde0588e6,0xd85988e8,0xde0788e8,0xd85988ea,0xde0788ea,0xd85988ec,0xde0988ec,0xd85988ee,
	0xde0988ee,0xd85988f0,0xde0b88f0,0xd85988f2,0xde0b88f2,0xd85988f4,0xde0d88f4,0xd85988f6,
	0xde0d88f6,0xd85988f8,0xde0f88f8,0xd85988fa,0xde0f88fa,0xd85988fc,0xde1188fc,0xd85988fe,
	0xde1188fe,0xd8598900,0xde138900,0xd8598902,0xde138902,0xd8598904,0xde158904,0xd8598906,
	0xde158906,0xd8598908,0xde178908,0xd859890a,0xde17890a,0xd859890c,0xde19890c,0xd859890e,
	0xde19890e,0xd8598910,0xde1b8910,0xd8598912,0xde1b8912,0xd8598914,0xde1d8914,0xd8598916,
	0xde1d8916,0xd8598918,0xde1f8918,0xd859891a,0xde1f891a,0xd859891c,0xde21891c,0xd859891e,
	0xde21891e,0xd8598920,0xde238920,0xd8598922,0xde238922,0xd8598924,0xde258924,0xd8598926,
	0xde258926,0xd8598928,0xde278928,0xd859892a,0xde27892a,0xd859892c,0xde29892c,0xd859892e,
	0xde29892e,0xd8598930,0xde2b8930,0xd8598932,0xde2b8932,0xd8598934,0xde2d8934,0xd8598936,
	0xde2d8936,0xd8598938,0xde2f8938,0xd859893a,0xde2f893a,0xd859893c,0xde31893c,0xd859893e,
	0xde31893e,0xd8598940,0xde338940,0xd8598942,0xde338942,0xd8598944,0xde358944,0xd8598946,
	0xde358946,0xd8598948,0xde378948,0xd859894a,0xde37894a,0xd859894c,0xde39894c,0xd859894e,
	0xde39894e,0xd8598950,0xde3b8950,0xd8598952,0xde3b8952,0xd8598954,0xde3d8954,0xd8598956,
	0xde3d8956,0xd8598958,0xde3f8958,0xd859895a,0xde3f895a,0xd859895c,0xde41895c,0xd859895e,
	0xde41895e,0xd8598960,0xde438960,0xd8598962,0xde438962,0xd8598964,0xde458964,0xd8598966,
	0xde458966,0xd8598968,0xde478968,0xd859896a,0xde47896a,0xd859896c,0xde49896c,0xd859896e,
	0xde49896e,0xd8598970,0xde4b8970,0xd8598972,0xde4b8972,0xd8598974,0xde4d8974,0xd8598976,
	0xde4d8976,0xd8598978,0xde4f8978,0xd859897a,0xde4f897a,0xd859897c,0xde51897c,0xd859897e,
	0xde51897e,0xd8598980,0xde538980,0xd8598982,0xde538982,0xd8598984,0xde558984,0xd8598986,
	0xde558986,0xd8598988,0xde578988,0xd859898a,0xde57898a,0xd859898c,0xde59898c,0xd859898e,
	0xde59898e,0xd8598990,0xde5b8990,0xd8598992,0xde5b8992,0xd8598994,0xde5d8994,0xd8598996,
	0xde5d8996,0xd8598998,0xde5f8998,0xd859899a,0xde5f899a,0xd859899c,0xde61899c,0xd859899e,
	0xde61899e,0xd85989a0,0xde6389a0,0xd85989a2,0xde6389a2,0xd85989a4,0xde6589a4,0xd85989a6,
	0xde6589a6,0xd85989a8,0xde6789a8,0xd85989aa,0xde6789aa,0xd85989ac,0xde6989ac,0xd85989ae,
	0xde6989ae,0xd85989b0,0xde6b89b0,0xd85989b2,0xde6b89b2,0xd85989b4,0xde6d89b4,0xd85989b6,
	0xde6d89b6,0xd85989b8,0xde6f89b8,0xd85989ba,0xde6f89ba,0xd85989bc,0xde7189bc,0xd85989be,
	0xde7189be,0xd85989c0,0xde7389c0,0xd85989c2,0xde7389c2,0xd85989c4,0xde7589c4,0xd85989c6,
	0xde7589c6,0xd85989c8,0xde7789c8,0xd85989ca,0xde7789ca,0xd85989cc,0xde7989cc,0xd85989ce,
	0xde7989ce,0xd85989d0,0xde7b89d0,0xd85989d2,0xde7b89d2,0xd85989d4,0xde7d89d4,0xd85989d6,
	0xde7d89d6,0xd85989d8,0xde7f89d8,0xd85989da,0xde7f89da,0xd85989dc,0xde8189dc,0xd85989de,
	0xde8189de,0xd85989e0,0xde8389e0,0xd85989e2,0xde8389e2,0xd85989e4,0xde8589e4,0xd85989e6,
	0xde8589e6,0xd85989e8,0xde8789e8,0xd85989ea,0xde8789ea,0xd85989ec,0xde8989ec,0xd85989ee,
	0xde8989ee,0xd85989f0,0xde8b89f0,0xd85989f2,0xde8b89f2,0xd85989f4,0xde8d89f4,0xd85989f6,
	0xde8d89f6,0xd85989f8,0xde8f89f8,0xd85989fa,0xde8f89fa,0xd85989fc,0xde9189fc,0xd85989fe,
	0xde9189fe,0xd8598a00,0xde938a00,0xd8598a02,0xde938a02,0xd8598a04,0xde958a04,0xd8598a06,
	0xde958a06,0xd8598a08,0xde978a08,0xd8598a0a,0xde978a0a,0xd8598a0c,0xde998a0c,0xd8598a0e,
	0xde998a0e,0xd8598a10,0xde9b8a10,0xd8598a12,0xde9b8a12,0xd8598a14,0xde9d8a14,0xd8598a16,
	0xde9d8a16,0xd8598a18,0xde9f8a18,0xd8598a1a,0xde9f8a1a,0xd8598a1c,0xdea18a1c,0xd8598a1e,
	0xdea18a1e,0xd8598a20,0xdea38a20,0xd8598a22,0xdea38a22,0xd8598a24,0xdea58a24,0xd8598a26,
	0xdea58a26,0xd8598a28,0xdea78a28,0xd8598a2a,0xdea78a2a,0xd8598a2c,0xdea98a2c,0xd8598a2e,
	0xdea98a2e,0xd8598a30,0xdeab8a30,0xd8598a32,0xdeab8a32,0xd8598a34,0xdead8a34,0xd8598a36,
	0xdead8a36,0xd8598a38,0xdeaf8a38,0xd8598a3a,0xdeaf8a3a,0xd8598a3c,0xdeb18a3c,0xd8598a3e,
	0xdeb18a3e,0xd8598a40,0xdeb38a40,0xd8598a42,0xdeb38a42,0xd8598a44,0xdeb58a44,0xd8598a46,
	0xdeb58a46,0xd8598a48,0xdeb78a48,0xd8598a4a,0xdeb78a4a,0xd8598a4c,0xdeb98a4c,0xd8598a4e,
	0xdeb98a4e,0xd8598a50,0xdebb8a50,0xd8598a52,0xdebb8a52,0xd8598a54,0xdebd8a54,0xd8598a56,
	0xdebd8a56,0xd8598a58,0xdebf8a58,0xd8598a5a,0xdebf8a5a,0xd8598a5c,0xdec18a5c,0xd8598a5e,
	0xdec18a5e,0xd8598a60,0xdec38a60,0xd8598a62,0xdec38a62,0xd8598a64,0xdec58a64,0xd8598a66,
	0xdec58a66,0xd8598a68,0xdec78a68,0xd8598a6a,0xdec78a6a,0xd8598a6c,0xdec98a6c,0xd8598a6e,
	0xdec98a6e,0xd8598a70,0xdecb8a70,0xd8598a72,0xdecb8a72,0xd8598a74,0xdecd8a74,0xd8598a76,
	0xdecd8a76,0xd8598a78,0xdecf8a78,0xd8598a7a,0xdecf8a7a,0xd8598a7c,0xded18a7c,0xd8598a7e,
	0xded18a7e,0xd8598a80,0xded38a80,0xd8598a82,0xded38a82,0xd8598a84,0xded58a84,0xd8598a86,
	0xded58a86,0xd8598a88,0xded78a88,0xd8598a8a,0xded78a8a,0xd8598a8c,0xded98a8c,0xd8598a8e,
	0xded98a8e,0xd8598a90,0xdedb8a90,0xd8598a92,0xdedb8a92,0xd8598a94,0xdedd8a94,0xd8598a96,
	0xdedd8a96,0xd8598a98,0xdedf8a98,0xd8598a9a,0xdedf8a9a,0xd8598a9c,0xdee18a9c,0xd8598a9e,
	0xdee18a9e,0xd8598aa0,0xdee38aa0,0xd8598aa2,0xdee38aa2,0xd8598aa4,0xdee58aa4,0xd8598aa6,
	0xdee58aa6,0xd8598aa8,0xdee78aa8,0xd8598aaa,0xdee78aaa,0xd8598aac,0xdee98aac,0xd8598aae,
	0xdee98aae,0xd8598ab0,0xdeeb8ab0,0xd8598ab2,0xdeeb8ab2,0xd8598ab4,0xdeed8ab4,0xd8598ab6,
	0xdeed8ab6,0xd8598ab8,0xdeef8ab8,0xd8598aba,0xdeef8aba,0xd8598abc,0xdef18abc,0xd8598abe,
	0xdef18abe,0xd8598ac0,0xdef38ac0,0xd8598ac2,0xdef38ac2,0xd8598ac4,0xdef58ac4,0xd8598ac6,
	0xdef58ac6,0xd8598ac8,0xdef78ac8,0xd8598aca,0xdef78aca,0xd8598acc,0xdef98acc,0xd8598ace,
	0xdef98ace,0xd8598ad0,0xdefb8ad0,0xd8598ad2,0xdefb8ad2,0xd8598ad4,0xdefd8ad4,0xd8598ad6,
	0xdefd8ad6,0xd8598ad8,0xdeff8ad8,0xd8598ada,0xdeff8ada,0xd8598adc,0xdf018adc,0xd8598ade,
	0xdf018ade,0xd8598ae0,0xdf038ae0,0xd8598ae2,0xdf038ae2,0xd8598ae4,0xdf058ae4,0xd8598ae6,
	0xdf058ae6,0xd8598ae8,0xdf078ae8,0xd8598aea,0xdf078aea,0xd8598aec,0xdf098aec,0xd8598aee,
	0xdf098aee,0xd8598af0,0xdf0b8af0,0xd8598af2,0xdf0b8af2,0xd8598af4,0xdf0d8af4,0xd8598af6,
	0xdf0d8af6,0xd8598af8,0xdf0f8af8,0xd8598afa,0xdf0f8afa,0xd8598afc,0xdf118afc,0xd8598afe,
	0xdf118afe,0xd8598b00,0xdf138b00,0xd8598b02,0xdf138b02,0xd8598b04,0xdf158b04,0xd8598b06,
	0xdf158b06,0xd8598b08,0xdf178b08,0xd8598b0a,0xdf178b0a,0xd8598b0c,0xdf198b0c,0xd8598b0e,
	0xdf198b0e,0xd8598b10,0xdf1b8b10,0xd8598b12,0xdf1b8b12,0xd8598b14,0xdf1d8b14,0xd8598b16,
	0xdf1d8b16,0xd8598b18,0xdf1f8b18,0xd8598b1a,0xdf1f8b1a,0xd8598b1c,0xdf218b1c,0xd8598b1e,
	0xdf218b1e,0xd8598b20,0xdf238b20,0xd8598b22,0xdf238b22,0xd8598b24,0xdf258b24,0xd8598b26,
	0xdf258b26,0xd8598b28,0xdf278b28,0xd8598b2a,0xdf278b2a,0xd8598b2c,0xdf298b2c,0xd8598b2e,
	0xdf298b2e,0xd8598b30,0xdf2b8b30,0xd8598b32,0xdf2b8b32,0xd8598b34,0xdf2d8b34,0xd8598b36,
	0xdf2d8b36,0xd8598b38,0xdf2f8b38,0xd8598b3a,0xdf2f8b3a,0xd8598b3c,0xdf318b3c,0xd8598b3e,
	0xdf318b3e,0xd8598b40,0xdf338b40,0xd8598b42,0xdf338b42,0xd8598b44,0xdf358b44,0xd8598b46,
	0xdf358b46,0xd8598b48,0xdf378b48,0xd8598b4a,0xdf378b4a,0xd8598b4c,0xdf398b4c,0xd8598b4e,
	0xdf398b4e,0xd8598b50,0xdf3b8b50,0xd8598b52,0xdf3b8b52,0xd8598b54,0xdf3d8b54,0xd8598b56,
	0xdf3d8b56,0xd8598b58,0xdf3f8b58,0xd8598b5a,0xdf3f8b5a,0xd8598b5c,0xdf418b5c,0xd8598b5e,
	0xdf418b5e,0xd8598b60,0xdf438b60,0xd8598b62,0xdf438b62,0xd8598b64,0xdf458b64,0xd8598b66,
	0xdf458b66,0xd8598b68,0xdf478b68,0xd8598b6a,0xdf478b6a,0xd8598b6c,0xdf498b6c,0xd8598b6e,
	0xdf498b6e,0xd8598b70,0xdf4b8b70,0xd8598b72,0xdf4b8b72,0xd8598b74,0xdf4d8b74,0xd8598b76,
	0xdf4d8b76,0xd8598b78,0xdf4f8b78,0xd8598b7a,0xdf4f8b7a,0xd8598b7c,0xdf518b7c,0xd8598b7e,
	0xdf518b7e,0xd8598b80,0xdf538b80,0xd8598b82,0xdf538b82,0xd8598b84,0xdf558b84,0xd8598b86,
	0xdf558b86,0xd8598b88,0xdf578b88,0xd8598b8a,0xdf578b8a,0xd8598b8c,0xdf598b8c,0xd8598b8e,
	0xdf598b8e,0xd8598b90,0xdf5b8b90,0xd8598b92,0xdf5b8b92,0xd8598b94,0xdf5d8b94,0xd8598b96,
	0xdf5d8b96,0xd8598b98,0xdf5f8b98,0xd8598b9a,0xdf5f8b9a,0xd8598b9c,0xdf618b9c,0xd8598b9e,
	0xdf618b9e,0xd8598ba0,0xdf638ba0,0xd8598ba2,0xdf638ba2,0xd8598ba4,0xdf658ba4,0xd8598ba6,
	0xdf658ba6,0xd8598ba8,0xdf678ba8,0xd8598baa,0xdf678baa,0xd8598bac,0xdf698bac,0xd8598bae,
	0xdf698bae,0xd8598bb0,0xdf6b8bb0,0xd8598bb2,0xdf6b8bb2,0xd8598bb4,0xdf6d8bb4,0xd8598bb6,
	0xdf6d8bb6,0xd8598bb8,0xdf6f8bb8,0xd8598bba,0xdf6f8bba,0xd8598bbc,0xdf718bbc,0xd8598bbe,
	0xdf718bbe,0xd8598bc0,0xdf738bc0,0xd8598bc2,0xdf738bc2,0xd8598bc4,0xdf758bc4,0xd8598bc6,
	0xdf758bc6,0xd8598bc8,0xdf778bc8,0xd8598bca,0xdf778bca,0xd8598bcc,0xdf798bcc,0xd8598bce,
	0xdf798bce,0xd8598bd0,0xdf7b8bd0,0xd8598bd2,0xdf7b8bd2,0xd8598bd4,0xdf7d8bd4,0xd8598bd6,
	0xdf7d8bd6,0xd8598bd8,0xdf7f8bd8,0xd8598bda,0xdf7f8bda,0xd8598bdc,0xdf818bdc,0xd8598bde,
	0xdf818bde,0xd8598be0,0xdf838be0,0xd8598be2,0xdf838be2,0xd8598be4,0xdf858be4,0xd8598be6,
	0xdf858be6,0xd8598be8,0xdf878be8,0xd8598bea,0xdf878bea,0xd8598bec,0xdf898bec,0xd8598bee,
	0xdf898bee,0xd8598bf0,0xdf8b8bf0,0xd8598bf2,0xdf8b8bf2,0xd8598bf4,0xdf8d8bf4,0xd8598bf6,
	0xdf8d8bf6,0xd8598bf8,0xdf8f8bf8,0xd8598bfa,0xdf8f8bfa,0xd8598bfc,0xdf918bfc,0xd8598bfe,
	0xdf918bfe,0xd8598c00,0xdf938c00,0xd8598c02,0xdf938c02,0xd8598c04,0xdf958c04,0xd8598c06,
	0xdf958c06,0xd8598c08,0xdf978c08,0xd8598c0a,0xdf978c0a,0xd8598c0c,0xdf998c0c,0xd8598c0e,
	0xdf998c0e,0xd8598c10,0xdf9b8c10,0xd8598c12,0xdf9b8c12,0xd8598c14,0xdf9d8c14,0xd8598c16,
	0xdf9d8c16,0xd8598c18,0xdf9f8c18,0xd8598c1a,0xdf9f8c1a,0xd8598c1c,0xdfa18c1c,0xd8598c1e,
	0xdfa18c1e,0xd8598c20,0xdfa38c20,0xd8598c22,0xdfa38c22,0xd8598c24,0xdfa58c24,0xd8598c26,
	0xdfa58c26,0xd8598c28,0xdfa78c28,0xd8598c2a,0xdfa78c2a,0xd8598c2c,0xdfa98c2c,0xd8598c2e,
	0xdfa98c2e,0xd8598c30,0xdfab8c30,0xd8598c32,0xdfab8c32,0xd8598c34,0xdfad8c34,0xd8598c36,
	0xdfad8c36,0xd8598c38,0xdfaf8c38,0xd8598c3a,0xdfaf8c3a,0xd8598c3c,0xdfb18c3c,0xd8598c3e,
	0xdfb18c3e,0xd8598c40,0xdfb38c40,0xd8598c42,0xdfb38c42,0xd8598c44,0xdfb58c44,0xd8598c46,
	0xdfb58c46,0xd8598c48,0xdfb78c48,0xd8598c4a,0xdfb78c4a,0xd8598c4c,0xdfb98c4c,0xd8598c4e,
	0xdfb98c4e,0xd8598c50,0xdfbb8c50,0xd8598c52,0xdfbb8c52,0xd8598c54,0xdfbd8c54,0xd8598c56,
	0xdfbd8c56,0xd8598c58,0xdfbf8c58,0xd8598c5a,0xdfbf8c5a,0xd8598c5c,0xdfc18c5c,0xd8598c5e,
	0xdfc18c5e,0xd8598c60,0xdfc38c60,0xd8598c62,0xdfc38c62,0xd8598c64,0xdfc58c64,0xd8598c66,
	0xdfc58c66,0xd8598c68,0xdfc78c68,0xd8598c6a,0xdfc78c6a,0xd8598c6c,0xdfc98c6c,0xd8598c6e,
	0xdfc98c6e,0xd8598c70,0xdfcb8c70,0xd8598c72,0xdfcb8c72,0xd8598c74,0xdfcd8c74,0xd8598c76,
	0xdfcd8c76,0xd8598c78,0xdfcf8c78,0xd8598c7a,0xdfcf8c7a,0xd8598c7c,0xdfd18c7c,0xd8598c7e,
	0xdfd18c7e,0xd8598c80,0xdfd38c80,0xd8598c82,0xdfd38c82,0xd8598c84,0xdfd58c84,0xd8598c86,
	0xdfd58c86,0xd8598c88,0xdfd78c88,0xd8598c8a,0xdfd78c8a,0xd8598c8c,0xdfd98c8c,0xd8598c8e,
	0xdfd98c8e,0xd8598c90,0xdfdb8c90,0xd8598c92,0xdfdb8c92,0xd8598c94,0xdfdd8c94,0xd8598c96,
	0xdfdd8c96,0xd8598c98,0xdfdf8c98,0xd8598c9a,0xdfdf8c9a,0xd8598c9c,0xdfe18c9c,0xd8598c9e,
	0xdfe18c9e,0xd8598ca0,0xdfe38ca0,0xd8598ca2,0xdfe38ca2,0xd8598ca4,0xdfe58ca4,0xd8598ca6,
	0xdfe58ca6,0xd8598ca8,0xdfe78ca8,0xd8598caa,0xdfe78caa,0xd8598cac,0xdfe98cac,0xd8598cae,
	0xdfe98cae,0xd8598cb0,0xdfeb8cb0,0xd8598cb2,0xdfeb8cb2,0xd8598cb4,0xdfed8cb4,0xd8598cb6,
	0xdfed8cb6,0xd8598cb8,0xdfef8cb8,0xd8598cba,0xdfef8cba,0xd8598cbc,0xdff18cbc,0xd8598cbe,
	0xdff18cbe,0xd8598cc0,0xdff38cc0,0xd8598cc2,0xdff38cc2,0xd8598cc4,0xdff58cc4,0xd8598cc6,
	0xdff58cc6,0xd8598cc8,0xdff78cc8,0xd8598cca,0xdff78cca,0xd8598ccc,0xdff98ccc,0xd8598cce,
	0xdff98cce,0xd8598cd0,0xdffb8cd0,0xd8598cd2,0xdffb8cd2,0xd8598cd4,0xdffd8cd4,0xd8598cd6,
	0xdffd8cd6,0xd8598cd8,0xdfff8cd8,0xd8598cda,0xdfff8cda,0xd85b8cdc,0xdc018cdc,0xd85b8cde,
	0xdc018cde,0xd85b8ce0,0xdc038ce0,0xd85b8ce2,0xdc038ce2,0xd85b8ce4,0xdc058ce4,0xd85b8ce6,
	0xdc058ce6,0xd85b8ce8,0xdc078ce8,0xd85b8cea,0xdc078cea,0xd85b8cec,0xdc098cec,0xd85b8cee,
	0xdc098cee,0xd85b8cf0,0xdc0b8cf0,0xd85b8cf2,0xdc0b8cf2,0xd85b8cf4,0xdc0d8cf4,0xd85b8cf6,
	0xdc0d8cf6,0xd85b8cf8,0xdc0f8cf8,0xd85b8cfa,0xdc0f8cfa,0xd85b8cfc,0xdc118cfc,0xd85b8cfe,
	0xdc118cfe,0xd85b8d00,0xdc138d00,0xd85b8d02,0xdc138d02,0xd85b8d04,0xdc158d04,0xd85b8d06,
	0xdc158d06,0xd85b8d08,0xdc178d08,0xd85b8d0a,0xdc178d0a,0xd85b8d0c,0xdc198d0c,0xd85b8d0e,
	0xdc198d0e,0xd85b8d10,0xdc1b8d10,0xd85b8d12,0xdc1b8d12,0xd85b8d14,0xdc1d8d14,0xd85b8d16,
	0xdc1d8d16,0xd85b8d18,0xdc1f8d18,0xd85b8d1a,0xdc1f8d1a,0xd85b8d1c,0xdc218d1c,0xd85b8d1e,
	0xdc218d1e,0xd85b8d20,0xdc238d20,0xd85b8d22,0xdc238d22,0xd85b8d24,0xdc258d24,0xd85b8d26,
	0xdc258d26,0xd85b8d28,0xdc278d28,0xd85b8d2a,0xdc278d2a,0xd85b8d2c,0xdc298d2c,0xd85b8d2e,
	0xdc298d2e,0xd85b8d30,0xdc2b8d30,0xd85b8d32,0xdc2b8d32,0xd85b8d34,0xdc2d8d34,0xd85b8d36,
	0xdc2d8d36,0xd85b8d38,0xdc2f8d38,0xd85b8d3a,0xdc2f8d3a,0xd85b8d3c,0xdc318d3c,0xd85b8d3e,
	0xdc318d3e,0xd85b8d40,0xdc338d40,0xd85b8d42,0xdc338d42,0xd85b8d44,0xdc358d44,0xd85b8d46,
	0xdc358d46,0xd85b8d48,0xdc378d48,0xd85b8d4a,0xdc378d4a,0xd85b8d4c,0xdc398d4c,0xd85b8d4e,
	0xdc398d4e,0xd85b8d50,0xdc3b8d50,0xd85b8d52,0xdc3b8d52,0xd85b8d54,0xdc3d8d54,0xd85b8d56,
	0xdc3d8d56,0xd85b8d58,0xdc3f8d58,0xd85b8d5a,0xdc3f8d5a,0xd85b8d5c,0xdc418d5c,0xd85b8d5e,
	0xdc418d5e,0xd85b8d60,0xdc438d60,0xd85b8d62,0xdc438d62,0xd85b8d64,0xdc458d64,0xd85b8d66,
	0xdc458d66,0xd85b8d68,0xdc478d68,0xd85b8d6a,0xdc478d6a,0xd85b8d6c,0xdc498d6c,0xd85b8d6e,
	0xdc498d6e,0xd85b8d70,0xdc4b8d70,0xd85b8d72,0xdc4b8d72,0xd85b8d74,0xdc4d8d74,0xd85b8d76,
	0xdc4d8d76,0xd85b8d78,0xdc4f8d78,0xd85b8d7a,0xdc4f8d7a,0xd85b8d7c,0xdc518d7c,0xd85b8d7e,
	0xdc518d7e,0xd85b8d80,0xdc538d80,0xd85b8d82,0xdc538d82,0xd85b8d84,0xdc558d84,0xd85b8d86,
	0xdc558d86,0xd85b8d88,0xdc578d88,0xd85b8d8a,0xdc578d8a,0xd85b8d8c,0xdc598d8c,0xd85b8d8e,
	0xdc598d8e,0xd85b8d90,0xdc5b8d90,0xd85b8d92,0xdc5b8d92,0xd85b8d94,0xdc5d8d94,0xd85b8d96,
	0xdc5d8d96,0xd85b8d98,0xdc5f8d98,0xd85b8d9a,0xdc5f8d9a,0xd85b8d9c,0xdc618d9c,0xd85b8d9e,
	0xdc618d9e,0xd85b8da0,0xdc638da0,0xd85b8da2,0xdc638da2,0xd85b8da4,0xdc658da4,0xd85b8da6,
	0xdc658da6,0xd85b8da8,0xdc678da8,0xd85b8daa,0xdc678daa,0xd85b8dac,0xdc698dac,0xd85b8dae,
	0xdc698dae,0xd85b8db0,0xdc6b8db0,0xd85b8db2,0xdc6b8db2,0xd85b8db4,0xdc6d8db4,0xd85b8db6,
	0xdc6d8db6,0xd85b8db8,0xdc6f8db8,0xd85b8dba,0xdc6f8dba,0xd85b8dbc,0xdc718dbc,0xd85b8dbe,
	0xdc718dbe,0xd85b8dc0,0xdc738dc0,0xd85b8dc2,0xdc738dc2,0xd85b8dc4,0xdc758dc4,0xd85b8dc6,
	0xdc758dc6,0xd85b8dc8,0xdc778dc8,0xd85b8dca,0xdc778dca,0xd85b8dcc,0xdc798dcc,0xd85b8dce,
	0xdc798dce,0xd85b8dd0,0xdc7b8dd0,0xd85b8dd2,0xdc7b8dd2,0xd85b8dd4,0xdc7d8dd4,0xd85b8dd6,
	0xdc7d8dd6,0xd85b8dd8,0xdc7f8dd8,0xd85b8dda,0xdc7f8dda,0xd85b8ddc,0xdc818ddc,0xd85b8dde,
	0xdc818dde,0xd85b8de0,0xdc838de0,0xd85b8de2,0xdc838de2,0xd85b8de4,0xdc858de4,0xd85b8de6,
	0xdc858de6,0xd85b8de8,0xdc878de8,0xd85b8dea,0xdc878dea,0xd85b8dec,0xdc898dec,0xd85b8dee,
	0xdc898dee,0xd85b8df0,0xdc8b8df0,0xd85b8df2,0xdc8b8df2,0xd85b8df4,0xdc8d8df4,0xd85b8df6,
	0xdc8d8df6,0xd85b8df8,0xdc8f8df8,0xd85b8dfa,0xdc8f8dfa,0xd85b8dfc,0xdc918dfc,0xd85b8dfe,
	0xdc918dfe,0xd85b8e00,0xdc938e00,0xd85b8e02,0xdc938e02,0xd85b8e04,0xdc958e04,0xd85b8e06,
	0xdc958e06,0xd85b8e08,0xdc978e08,0xd85b8e0a,0xdc978e0a,0xd85b8e0c,0xdc998e0c,0xd85b8e0e,
	0xdc998e0e,0xd85b8e10,0xdc9b8e10,0xd85b8e12,0xdc9b8e12,0xd85b8e14,0xdc9d8e14,0xd85b8e16,
	0xdc9d8e16,0xd85b8e18,0xdc9f8e18,0xd85b8e1a,0xdc9f8e1a,0xd85b8e1c,0xdca18e1c,0xd85b8e1e,
	0xdca18e1e,0xd85b8e20,0xdca38e20,0xd85b8e22,0xdca38e22,0xd85b8e24,0xdca58e24,0xd85b8e26,
	0xdca58e26,0xd85b8e28,0xdca78e28,0xd85b8e2a,0xdca78e2a,0xd85b8e2c,0xdca98e2c,0xd85b8e2e,
	0xdca98e2e,0xd85b8e30,0xdcab8e30,0xd85b8e32,0xdcab8e32,0xd85b8e34,0xdcad8e34,0xd85b8e36,
	0xdcad8e36,0xd85b8e38,0xdcaf8e38,0xd85b8e3a,0xdcaf8e3a,0xd85b8e3c,0xdcb18e3c,0xd85b8e3e,
	0xdcb18e3e,0xd85b8e40,0xdcb38e40,0xd85b8e42,0xdcb38e42,0xd85b8e44,0xdcb58e44,0xd85b8e46,
	0xdcb58e46,0xd85b8e48,0xdcb78e48,0xd85b8e4a,0xdcb78e4a,0xd85b8e4c,0xdcb98e4c,0xd85b8e4e,
	0xdcb98e4e,0xd85b8e50,0xdcbb8e50,0xd85b8e52,0xdcbb8e52,0xd85b8e54,0xdcbd8e54,0xd85b8e56,
	0xdcbd8e56,0xd85b8e58,0xdcbf8e58,0xd85b8e5a,0xdcbf8e5a,0xd85b8e5c,0xdcc18e5c,0xd85b8e5e,
	0xdcc18e5e,0xd85b8e60,0xdcc38e60,0xd85b8e62,0xdcc38e62,0xd85b8e64,0xdcc58e64,0xd85b8e66,
	0xdcc58e66,0xd85b8e68,0xdcc78e68,0xd85b8e6a,0xdcc78e6a,0xd85b8e6c,0xdcc98e6c,0xd85b8e6e,
	0xdcc98e6e,0xd85b8e70,0xdccb8e70,0xd85b8e72,0xdccb8e72,0xd85b8e74,0xdccd8e74,0xd85b8e76,
	0xdccd8e76,0xd85b8e78,0xdccf8e78,0xd85b8e7a,0xdccf8e7a,0xd85b8e7c,0xdcd18e7c,0xd85b8e7e,
	0xdcd18e7e,0xd85b8e80,0xdcd38e80,0xd85b8e82,0xdcd38e82,0xd85b8e84,0xdcd58e84,0xd85b8e86,
	0xdcd58e86,0xd85b8e88,0xdcd78e88,0xd85b8e8a,0xdcd78e8a,0xd85b8e8c,0xdcd98e8c,0xd85b8e8e,
	0xdcd98e8e,0xd85b8e90,0xdcdb8e90,0xd85b8e92,0xdcdb8e92,0xd85b8e94,0xdcdd8e94,0xd85b8e96,
	0xdcdd8e96,0xd85b8e98,0xdcdf8e98,0xd85b8e9a,0xdcdf8e9a,0xd85b8e9c,0xdce18e9c,0xd85b8e9e,
	0xdce18e9e,0xd85b8ea0,0xdce38ea0,0xd85b8ea2,0xdce38ea2,0xd85b8ea4,0xdce58ea4,0xd85b8ea6,
	0xdce58ea6,0xd85b8ea8,0xdce78ea8,0xd85b8eaa,0xdce78eaa,0xd85b8eac,0xdce98eac,0xd85b8eae,
	0xdce98eae,0xd85b8eb0,0xdceb8eb0,0xd85b8eb2,0xdceb8eb2,0xd85b8eb4,0xdced8eb4,0xd85b8eb6,
	0xdced8eb6,0xd85b8eb8,0xdcef8eb8,0xd85b8eba,0xdcef8eba,0xd85b8ebc,0xdcf18ebc,0xd85b8ebe,
	0xdcf18ebe,0xd85b8ec0,0xdcf38ec0,0xd85b8ec2,0xdcf38ec2,0xd85b8ec4,0xdcf58ec4,0xd85b8ec6,
	0xdcf58ec6,0xd85b8ec8,0xdcf78ec8,0xd85b8eca,0xdcf78eca,0xd85b8ecc,0xdcf98ecc,0xd85b8ece,
	0xdcf98ece,0xd85b8ed0,0xdcfb8ed0,0xd85b8ed2,0xdcfb8ed2,0xd85b8ed4,0xdcfd8ed4,0xd85b8ed6,
	0xdcfd8ed6,0xd85b8ed8,0xdcff8ed8,0xd85b8eda,0xdcff8eda,0xd85b8edc,0xdd018edc,0xd85b8ede,
	0xdd018ede,0xd85b8ee0,0xdd038ee0,0xd85b8ee2,0xdd038ee2,0xd85b8ee4,0xdd058ee4,0xd85b8ee6,
	0xdd058ee6,0xd85b8ee8,0xdd078ee8,0xd85b8eea,0xdd078eea,0xd85b8eec,0xdd098eec,0xd85b8eee,
	0xdd098eee,0xd85b8ef0,0xdd0b8ef0,0xd85b8ef2,0xdd0b8ef2,0xd85b8ef4,0xdd0d8ef4,0xd85b8ef6,
	0xdd0d8ef6,0xd85b8ef8,0xdd0f8ef8,0xd85b8efa,0xdd0f8efa,0xd85b8efc,0xdd118efc,0xd85b8efe,
	0xdd118efe,0xd85b8f00,0xdd138f00,0xd85b8f02,0xdd138f02,0xd85b8f04,0xdd158f04,0xd85b8f06,
	0xdd158f06,0xd85b8f08,0xdd178f08,0xd85b8f0a,0xdd178f0a,0xd85b8f0c,0xdd198f0c,0xd85b8f0e,
	0xdd198f0e,0xd85b8f10,0xdd1b8f10,0xd85b8f12,0xdd1b8f12,0xd85b8f14,0xdd1d8f14,0xd85b8f16,
	0xdd1d8f16,0xd85b8f18,0xdd1f8f18,0xd85b8f1a,0xdd1f8f1a,0xd85b8f1c,0xdd218f1c,0xd85b8f1e,
	0xdd218f1e,0xd85b8f20,0xdd238f20,0xd85b8f22,0xdd238f22,0xd85b8f24,0xdd258f24,0xd85b8f26,
	0xdd258f26,0xd85b8f28,0xdd278f28,0xd85b8f2a,0xdd278f2a,0xd85b8f2c,0xdd298f2c,0xd85b8f2e,
	0xdd298f2e,0xd85b8f30,0xdd2b8f30,0xd85b8f32,0xdd2b8f32,0xd85b8f34,0xdd2d8f34,0xd85b8f36,
	0xdd2d8f36,0xd85b8f38,0xdd2f8f38,0xd85b8f3a,0xdd2f8f3a,0xd85b8f3c,0xdd318f3c,0xd85b8f3e,
	0xdd318f3e,0xd85b8f40,0xdd338f40,0xd85b8f42,0xdd338f42,0xd85b8f44,0xdd358f44,0xd85b8f46,
	0xdd358f46,0xd85b8f48,0xdd378f48,0xd85b8f4a,0xdd378f4a,0xd85b8f4c,0xdd398f4c,0xd85b8f4e,
	0xdd398f4e,0xd85b8f50,0xdd3b8f50,0xd85b8f52,0xdd3b8f52,0xd85b8f54,0xdd3d8f54,0xd85b8f56,
	0xdd3d8f56,0xd85b8f58,0xdd3f8f58,0xd85b8f5a,0xdd3f8f5a,0xd85b8f5c,0xdd418f5c,0xd85b8f5e,
	0xdd418f5e,0xd85b8f60,0xdd438f60,0xd85b8f62,0xdd438f62,0xd85b8f64,0xdd458f64,0xd85b8f66,
	0xdd458f66,0xd85b8f68,0xdd478f68,0xd85b8f6a,0xdd478f6a,0xd85b8f6c,0xdd498f6c,0xd85b8f6e,
	0xdd498f6e,0xd85b8f70,0xdd4b8f70,0xd85b8f72,0xdd4b8f72,0xd85b8f74,0xdd4d8f74,0xd85b8f76,
	0xdd4d8f76,0xd85b8f78,0xdd4f8f78,0xd85b8f7a,0xdd4f8f7a,0xd85b8f7c,0xdd518f7c,0xd85b8f7e,
	0xdd518f7e,0xd85b8f80,0xdd538f80,0xd85b8f82,0xdd538f82,0xd85b8f84,0xdd558f84,0xd85b8f86,
	0xdd558f86,0xd85b8f88,0xdd578f88,0xd85b8f8a,0xdd578f8a,0xd85b8f8c,0xdd598f8c,0xd85b8f8e,
	0xdd598f8e,0xd85b8f90,0xdd5b8f90,0xd85b8f92,0xdd5b8f92,0xd85b8f94,0xdd5d8f94,0xd85b8f96,
	0xdd5d8f96,0xd85b8f98,0xdd5f8f98,0xd85b8f9a,0xdd5f8f9a,0xd85b8f9c,0xdd618f9c,0xd85b8f9e,
	0xdd618f9e,0xd85b8fa0,0xdd638fa0,0xd85b8fa2,0xdd638fa2,0xd85b8fa4,0xdd658fa4,0xd85b8fa6,
	0xdd658fa6,0xd85b8fa8,0xdd678fa8,0xd85b8faa,0xdd678faa,0xd85b8fac,0xdd698fac,0xd85b8fae,
	0xdd698fae,0xd85b8fb0,0xdd6b8fb0,0xd85b8fb2,0xdd6b8fb2,0xd85b8fb4,0xdd6d8fb4,0xd85b8fb6,
	0xdd6d8fb6,0xd85b8fb8,0xdd6f8fb8,0xd85b8fba,0xdd6f8fba,0xd85b8fbc,0xdd718fbc,0xd85b8fbe,
	0xdd718fbe,0xd85b8fc0,0xdd738fc0,0xd85b8fc2,0xdd738fc2,0xd85b8fc4,0xdd758fc4,0xd85b8fc6,
	0xdd758fc6,0xd85b8fc8,0xdd778fc8,0xd85b8fca,0xdd778fca,0xd85b8fcc,0xdd798fcc,0xd85b8fce,
	0xdd798fce,0xd85b8fd0,0xdd7b8fd0,0xd85b8fd2,0xdd7b8fd2,0xd85b8fd4,0xdd7d8fd4,0xd85b8fd6,
	0xdd7d8fd6,0xd85b8fd8,0xdd7f8fd8,0xd85b8fda,0xdd7f8fda,0xd85b8fdc,0xdd818fdc,0xd85b8fde,
	0xdd818fde,0xd85b8fe0,0xdd838fe0,0xd85b8fe2,0xdd838fe2,0xd85b8fe4,0xdd858fe4,0xd85b8fe6,
	0xdd858fe6,0xd85b8fe8,0xdd878fe8,0xd85b8fea,0xdd878fea,0xd85b8fec,0xdd898fec,0xd85b8fee,
	0xdd898fee,0xd85b8ff0,0xdd8b8ff0,0xd85b8ff2,0xdd8b8ff2,0xd85b8ff4,0xdd8d8ff4,0xd85b8ff6,
	0xdd8d8ff6,0xd85b8ff8,0xdd8f8ff8,0xd85b8ffa,0xdd8f8ffa,0xd85b8ffc,0xdd918ffc,0xd85b8ffe,
	0xdd918ffe,0xd85b9000,0xdd939000,0xd85b9002,0xdd939002,0xd85b9004,0xdd959004,0xd85b9006,
	0xdd959006,0xd85b9008,0xdd979008,0xd85b900a,0xdd97900a,0xd85b900c,0xdd99900c,0xd85b900e,
	0xdd99900e,0xd85b9010,0xdd9b9010,0xd85b9012,0xdd9b9012,0xd85b9014,0xdd9d9014,0xd85b9016,
	0xdd9d9016,0xd85b9018,0xdd9f9018,0xd85b901a,0xdd9f901a,0xd85b901c,0xdda1901c,0xd85b901e,
	0xdda1901e,0xd85b9020,0xdda39020,0xd85b9022,0xdda39022,0xd85b9024,0xdda59024,0xd85b9026,
	0xdda59026,0xd85b9028,0xdda79028,0xd85b902a,0xdda7902a,0xd85b902c,0xdda9902c,0xd85b902e,
	0xdda9902e,0xd85b9030,0xddab9030,0xd85b9032,0xddab9032,0xd85b9034,0xddad9034,0xd85b9036,
	0xddad9036,0xd85b9038,0xddaf9038,0xd85b903a,0xddaf903a,0xd85b903c,0xddb1903c,0xd85b903e,
	0xddb1903e,0xd85b9040,0xddb39040,0xd85b9042,0xddb39042,0xd85b9044,0xddb59044,0xd85b9046,
	0xddb59046,0xd85b9048,0xddb79048,0xd85b904a,0xddb7904a,0xd85b904c,0xddb9904c,0xd85b904e,
	0xddb9904e,0xd85b9050,0xddbb9050,0xd85b9052,0xddbb9052,0xd85b9054,0xddbd9054,0xd85b9056,
	0xddbd9056,0xd85b9058,0xddbf9058,0xd85b905a,0xddbf905a,0xd85b905c,0xddc1905c,0xd85b905e,
	0xddc1905e,0xd85b9060,0xddc39060,0xd85b9062,0xddc39062,0xd85b9064,0xddc59064,0xd85b9066,
	0xddc59066,0xd85b9068,0xddc79068,0xd85b906a,0xddc7906a,0xd85b906c,0xddc9906c,0xd85b906e,
	0xddc9906e,0xd85b9070,0xddcb9070,0xd85b9072,0xddcb9072,0xd85b9074,0xddcd9074,0xd85b9076,
	0xddcd9076,0xd85b9078,0xddcf9078,0xd85b907a,0xddcf907a,0xd85b907c,0xddd1907c,0xd85b907e,
	0xddd1907e,0xd85b9080,0xddd39080,0xd85b9082,0xddd39082,0xd85b9084,0xddd59084,0xd85b9086,
	0xddd59086,0xd85b9088,0xddd79088,0xd85b908a,0xddd7908a,0xd85b908c,0xddd9908c,0xd85b908e,
	0xddd9908e,0xd85b9090,0xdddb9090,0xd85b9092,0xdddb9092,0xd85b9094,0xdddd9094,0xd85b9096,
	0xdddd9096,0xd85b9098,0xdddf9098,0xd85b909a,0xdddf909a,0xd85b909c,0xdde1909c,0xd85b909e,
	0xdde1909e,0xd85b90a0,0xdde390a0,0xd85b90a2,0xdde390a2,0xd85b90a4,0xdde590a4,0xd85b90a6,
	0xdde590a6,0xd85b90a8,0xdde790a8,0xd85b90aa,0xdde790aa,0xd85b90ac,0xdde990ac,0xd85b90ae,
	0xdde990ae,0xd85b90b0,0xddeb90b0,0xd85b90b2,0xddeb90b2,0xd85b90b4,0xdded90b4,0xd85b90b6,
	0xdded90b6,0xd85b90b8,0xddef90b8,0xd85b90ba,0xddef90ba,0xd85b90bc,0xddf190bc,0xd85b90be,
	0xddf190be,0xd85b90c0,0xddf390c0,0xd85b90c2,0xddf390c2,0xd85b90c4,0xddf590c4,0xd85b90c6,
	0xddf590c6,0xd85b90c8,0xddf790c8,0xd85b90ca,0xddf790ca,0xd85b90cc,0xddf990cc,0xd85b90ce,
	0xddf990ce,0xd85b90d0,0xddfb90d0,0xd85b90d2,0xddfb90d2,0xd85b90d4,0xddfd90d4,0xd85b90d6,
	0xddfd90d6,0xd85b90d8,0xddff90d8,0xd85b90da,0xddff90da,0xd85b90dc,0xde0190dc,0xd85b90de,
	0xde0190de,0xd85b90e0,0xde0390e0,0xd85b90e2,0xde0390e2,0xd85b90e4,0xde0590e4,0xd85b90e6,
	0xde0590e6,0xd85b90e8,0xde0790e8,0xd85b90ea,0xde0790ea,0xd85b90ec,0xde0990ec,0xd85b90ee,
	0xde0990ee,0xd85b90f0,0xde0b90f0,0xd85b90f2,0xde0b90f2,0xd85b90f4,0xde0d90f4,0xd85b90f6,
	0xde0d90f6,0xd85b90f8,0xde0f90f8,0xd85b90fa,0xde0f90fa,0xd85b90fc,0xde1190fc,0xd85b90fe,
	0xde1190fe,0xd85b9100,0xde139100,0xd85b9102,0xde139102,0xd85b9104,0xde159104,0xd85b9106,
	0xde159106,0xd85b9108,0xde179108,0xd85b910a,0xde17910a,0xd85b910c,0xde19910c,0xd85b910e,
	0xde19910e,0xd85b9110,0xde1b9110,0xd85b9112,0xde1b9112,0xd85b9114,0xde1d9114,0xd85b9116,
	0xde1d9116,0xd85b9118,0xde1f9118,0xd85b911a,0xde1f911a,0xd85b911c,0xde21911c,0xd85b911e,
	0xde21911e,0xd85b9120,0xde239120,0xd85b9122,0xde239122,0xd85b9124,0xde259124,0xd85b9126,
	0xde259126,0xd85b9128,0xde279128,0xd85b912a,0xde27912a,0xd85b912c,0xde29912c,0xd85b912e,
	0xde29912e,0xd85b9130,0xde2b9130,0xd85b9132,0xde2b9132,0xd85b9134,0xde2d9134,0xd85b9136,
	0xde2d9136,0xd85b9138,0xde2f9138,0xd85b913a,0xde2f913a,0xd85b913c,0xde31913c,0xd85b913e,
	0xde31913e,0xd85b9140,0xde339140,0xd85b9142,0xde339142,0xd85b9144,0xde359144,0xd85b9146,
	0xde359146,0xd85b9148,0xde379148,0xd85b914a,0xde37914a,0xd85b914c,0xde39914c,0xd85b914e,
	0xde39914e,0xd85b9150,0xde3b9150,0xd85b9152,0xde3b9152,0xd85b9154,0xde3d9154,0xd85b9156,
	0xde3d9156,0xd85b9158,0xde3f9158,0xd85b915a,0xde3f915a,0xd85b915c,0xde41915c,0xd85b915e,
	0xde41915e,0xd85b9160,0xde439160,0xd85b9162,0xde439162,0xd85b9164,0xde459164,0xd85b9166,
	0xde459166,0xd85b9168,0xde479168,0xd85b916a,0xde47916a,0xd85b916c,0xde49916c,0xd85b916e,
	0xde49916e,0xd85b9170,0xde4b9170,0xd85b9172,0xde4b9172,0xd85b9174,0xde4d9174,0xd85b9176,
	0xde4d9176,0xd85b9178,0xde4f9178,0xd85b917a,0xde4f917a,0xd85b917c,0xde51917c,0xd85b917e,
	0xde51917e,0xd85b9180,0xde539180,0xd85b9182,0xde539182,0xd85b9184,0xde559184,0xd85b9186,
	0xde559186,0xd85b9188,0xde579188,0xd85b918a,0xde57918a,0xd85b918c,0xde59918c,0xd85b918e,
	0xde59918e,0xd85b9190,0xde5b9190,0xd85b9192,0xde5b9192,0xd85b9194,0xde5d9194,0xd85b9196,
	0xde5d9196,0xd85b9198,0xde5f9198,0xd85b919a,0xde5f919a,0xd85b919c,0xde61919c,0xd85b919e,
	0xde61919e,0xd85b91a0,0xde6391a0,0xd85b91a2,0xde6391a2,0xd85b91a4,0xde6591a4,0xd85b91a6,
	0xde6591a6,0xd85b91a8,0xde6791a8,0xd85b91aa,0xde6791aa,0xd85b91ac,0xde6991ac,0xd85b91ae,
	0xde6991ae,0xd85b91b0,0xde6b91b0,0xd85b91b2,0xde6b91b2,0xd85b91b4,0xde6d91b4,0xd85b91b6,
	0xde6d91b6,0xd85b91b8,0xde6f91b8,0xd85b91ba,0xde6f91ba,0xd85b91bc,0xde7191bc,0xd85b91be,
	0xde7191be,0xd85b91c0,0xde7391c0,0xd85b91c2,0xde7391c2,0xd85b91c4,0xde7591c4,0xd85b91c6,
	0xde7591c6,0xd85b91c8,0xde7791c8,0xd85b91ca,0xde7791ca,0xd85b91cc,0xde7991cc,0xd85b91ce,
	0xde7991ce,0xd85b91d0,0xde7b91d0,0xd85b91d2,0xde7b91d2,0xd85b91d4,0xde7d91d4,0xd85b91d6,
	0xde7d91d6,0xd85b91d8,0xde7f91d8,0xd85b91da,0xde7f91da,0xd85b91dc,0xde8191dc,0xd85b91de,
	0xde8191de,0xd85b91e0,0xde8391e0,0xd85b91e2,0xde8391e2,0xd85b91e4,0xde8591e4,0xd85b91e6,
	0xde8591e6,0xd85b91e8,0xde8791e8,0xd85b91ea,0xde8791ea,0xd85b91ec,0xde8991ec,0xd85b91ee,
	0xde8991ee,0xd85b91f0,0xde8b91f0,0xd85b91f2,0xde8b91f2,0xd85b91f4,0xde8d91f4,0xd85b91f6,
	0xde8d91f6,0xd85b91f8,0xde8f91f8,0xd85b91fa,0xde8f91fa,0xd85b91fc,0xde9191fc,0xd85b91fe,
	0xde9191fe,0xd85b9200,0xde939200,0xd85b9202,0xde939202,0xd85b9204,0xde959204,0xd85b9206,
	0xde959206,0xd85b9208,0xde979208,0xd85b920a,0xde97920a,0xd85b920c,0xde99920c,0xd85b920e,
	0xde99920e,0xd85b9210,0xde9b9210,0xd85b9212,0xde9b9212,0xd85b9214,0xde9d9214,0xd85b9216,
	0xde9d9216,0xd85b9218,0xde9f9218,0xd85b921a,0xde9f921a,0xd85b921c,0xdea1921c,0xd85b921e,
	0xdea1921e,0xd85b9220,0xdea39220,0xd85b9222,0xdea39222,0xd85b9224,0xdea59224,0xd85b9226,
	0xdea59226,0xd85b9228,0xdea79228,0xd85b922a,0xdea7922a,0xd85b922c,0xdea9922c,0xd85b922e,
	0xdea9922e,0xd85b9230,0xdeab9230,0xd85b9232,0xdeab9232,0xd85b9234,0xdead9234,0xd85b9236,
	0xdead9236,0xd85b9238,0xdeaf9238,0xd85b923a,0xdeaf923a,0xd85b923c,0xdeb1923c,0xd85b923e,
	0xdeb1923e,0xd85b9240,0xdeb39240,0xd85b9242,0xdeb39242,0xd85b9244,0xdeb59244,0xd85b9246,
	0xdeb59246,0xd85b9248,0xdeb79248,0xd85b924a,0xdeb7924a,0xd85b924c,0xdeb9924c,0xd85b924e,
	0xdeb9924e,0xd85b9250,0xdebb9250,0xd85b9252,0xdebb9252,0xd85b9254,0xdebd9254,0xd85b9256,
	0xdebd9256,0xd85b9258,0xdebf9258,0xd85b925a,0xdebf925a,0xd85b925c,0xdec1925c,0xd85b925e,
	0xdec1925e,0xd85b9260,0xdec39260,0xd85b9262,0xdec39262,0xd85b9264,0xdec59264,0xd85b9266,
	0xdec59266,0xd85b9268,0xdec79268,0xd85b926a,0xdec7926a,0xd85b926c,0xdec9926c,0xd85b926e,
	0xdec9926e,0xd85b9270,0xdecb9270,0xd85b9272,0xdecb9272,0xd85b9274,0xdecd9274,0xd85b9276,
	0xdecd9276,0xd85b9278,0xdecf9278,0xd85b927a,0xdecf927a,0xd85b927c,0xded1927c,0xd85b927e,
	0xded1927e,0xd85b9280,0xded39280,0xd85b9282,0xded39282,0xd85b9284,0xded59284,0xd85b9286,
	0xded59286,0xd85b9288,0xded79288,0xd85b928a,0xded7928a,0xd85b928c,0xded9928c,0xd85b928e,
	0xded9928e,0xd85b9290,0xdedb9290,0xd85b9292,0xdedb9292,0xd85b9294,0xdedd9294,0xd85b9296,
	0xdedd9296,0xd85b9298,0xdedf9298,0xd85b929a,0xdedf929a,0xd85b929c,0xdee1929c,0xd85b929e,
	0xdee1929e,0xd85b92a0,0xdee392a0,0xd85b92a2,0xdee392a2,0xd85b92a4,0xdee592a4,0xd85b92a6,
	0xdee592a6,0xd85b92a8,0xdee792a8,0xd85b92aa,0xdee792aa,0xd85b92ac,0xdee992ac,0xd85b92ae,
	0xdee992ae,0xd85b92b0,0xdeeb92b0,0xd85b92b2,0xdeeb92b2,0xd85b92b4,0xdeed92b4,0xd85b92b6,
	0xdeed92b6,0xd85b92b8,0xdeef92b8,0xd85b92ba,0xdeef92ba,0xd85b92bc,0xdef192bc,0xd85b92be,
	0xdef192be,0xd85b92c0,0xdef392c0,0xd85b92c2,0xdef392c2,0xd85b92c4,0xdef592c4,0xd85b92c6,
	0xdef592c6,0xd85b92c8,0xdef792c8,0xd85b92ca,0xdef792ca,0xd85b92cc,0xdef992cc,0xd85b92ce,
	0xdef992ce,0xd85b92d0,0xdefb92d0,0xd85b92d2,0xdefb92d2,0xd85b92d4,0xdefd92d4,0xd85b92d6,
	0xdefd92d6,0xd85b92d8,0xdeff92d8,0xd85b92da,0xdeff92da,0xd85b92dc,0xdf0192dc,0xd85b92de,
	0xdf0192de,0xd85b92e0,0xdf0392e0,0xd85b92e2,0xdf0392e2,0xd85b92e4,0xdf0592e4,0xd85b92e6,
	0xdf0592e6,0xd85b92e8,0xdf0792e8,0xd85b92ea,0xdf0792ea,0xd85b92ec,0xdf0992ec,0xd85b92ee,
	0xdf0992ee,0xd85b92f0,0xdf0b92f0,0xd85b92f2,0xdf0b92f2,0xd85b92f4,0xdf0d92f4,0xd85b92f6,
	0xdf0d92f6,0xd85b92f8,0xdf0f92f8,0xd85b92fa,0xdf0f92fa,0xd85b92fc,0xdf1192fc,0xd85b92fe,
	0xdf1192fe,0xd85b9300,0xdf139300,0xd85b9302,0xdf139302,0xd85b9304,0xdf159304,0xd85b9306,
	0xdf159306,0xd85b9308,0xdf179308,0xd85b930a,0xdf17930a,0xd85b930c,0xdf19930c,0xd85b930e,
	0xdf19930e,0xd85b9310,0xdf1b9310,0xd85b9312,0xdf1b9312,0xd85b9314,0xdf1d9314,0xd85b9316,
	0xdf1d9316,0xd85b9318,0xdf1f9318,0xd85b931a,0xdf1f931a,0xd85b931c,0xdf21931c,0xd85b931e,
	0xdf21931e,0xd85b9320,0xdf239320,0xd85b9322,0xdf239322,0xd85b9324,0xdf259324,0xd85b9326,
	0xdf259326,0xd85b9328,0xdf279328,0xd85b932a,0xdf27932a,0xd85b932c,0xdf29932c,0xd85b932e,
	0xdf29932e,0xd85b9330,0xdf2b9330,0xd85b9332,0xdf2b9332,0xd85b9334,0xdf2d9334,0xd85b9336,
	0xdf2d9336,0xd85b9338,0xdf2f9338,0xd85b933a,0xdf2f933a,0xd85b933c,0xdf31933c,0xd85b933e,
	0xdf31933e,0xd85b9340,0xdf339340,0xd85b9342,0xdf339342,0xd85b9344,0xdf359344,0xd85b9346,
	0xdf359346,0xd85b9348,0xdf379348,0xd85b934a,0xdf37934a,0xd85b934c,0xdf39934c,0xd85b934e,
	0xdf39934e,0xd85b9350,0xdf3b9350,0xd85b9352,0xdf3b9352,0xd85b9354,0xdf3d9354,0xd85b9356,
	0xdf3d9356,0xd85b9358,0xdf3f9358,0xd85b935a,0xdf3f935a,0xd85b935c,0xdf41935c,0xd85b935e,
	0xdf41935e,0xd85b9360,0xdf439360,0xd85b9362,0xdf439362,0xd85b9364,0xdf459364,0xd85b9366,
	0xdf459366,0xd85b9368,0xdf479368,0xd85b936a,0xdf47936a,0xd85b936c,0xdf49936c,0xd85b936e,
	0xdf49936e,0xd85b9370,0xdf4b9370,0xd85b9372,0xdf4b9372,0xd85b9374,0xdf4d9374,0xd85b9376,
	0xdf4d9376,0xd85b9378,0xdf4f9378,0xd85b937a,0xdf4f937a,0xd85b937c,0xdf51937c,0xd85b937e,
	0xdf51937e,0xd85b9380,0xdf539380,0xd85b9382,0xdf539382,0xd85b9384,0xdf559384,0xd85b9386,
	0xdf559386,0xd85b9388,0xdf579388,0xd85b938a,0xdf57938a,0xd85b938c,0xdf59938c,0xd85b938e,
	0xdf59938e,0xd85b9390,0xdf5b9390,0xd85b9392,0xdf5b9392,0xd85b9394,0xdf5d9394,0xd85b9396,
	0xdf5d9396,0xd85b9398,0xdf5f9398,0xd85b939a,0xdf5f939a,0xd85b939c,0xdf61939c,0xd85b939e,
	0xdf61939e,0xd85b93a0,0xdf6393a0,0xd85b93a2,0xdf6393a2,0xd85b93a4,0xdf6593a4,0xd85b93a6,
	0xdf6593a6,0xd85b93a8,0xdf6793a8,0xd85b93aa,0xdf6793aa,0xd85b93ac,0xdf6993ac,0xd85b93ae,
	0xdf6993ae,0xd85b93b0,0xdf6b93b0,0xd85b93b2,0xdf6b93b2,0xd85b93b4,0xdf6d93b4,0xd85b93b6,
	0xdf6d93b6,0xd85b93b8,0xdf6f93b8,0xd85b93ba,0xdf6f93ba,0xd85b93bc,0xdf7193bc,0xd85b93be,
	0xdf7193be,0xd85b93c0,0xdf7393c0,0xd85b93c2,0xdf7393c2,0xd85b93c4,0xdf7593c4,0xd85b93c6,
	0xdf7593c6,0xd85b93c8,0xdf7793c8,0xd85b93ca,0xdf7793ca,0xd85b93cc,0xdf7993cc,0xd85b93ce,
	0xdf7993ce,0xd85b93d0,0xdf7b93d0,0xd85b93d2,0xdf7b93d2,0xd85b93d4,0xdf7d93d4,0xd85b93d6,
	0xdf7d93d6,0xd85b93d8,0xdf7f93d8,0xd85b93da,0xdf7f93da,0xd85b93dc,0xdf8193dc,0xd85b93de,
	0xdf8193de,0xd85b93e0,0xdf8393e0,0xd85b93e2,0xdf8393e2,0xd85b93e4,0xdf8593e4,0xd85b93e6,
	0xdf8593e6,0xd85b93e8,0xdf8793e8,0xd85b93ea,0xdf8793ea,0xd85b93ec,0xdf8993ec,0xd85b93ee,
	0xdf8993ee,0xd85b93f0,0xdf8b93f0,0xd85b93f2,0xdf8b93f2,0xd85b93f4,0xdf8d93f4,0xd85b93f6,
	0xdf8d93f6,0xd85b93f8,0xdf8f93f8,0xd85b93fa,0xdf8f93fa,0xd85b93fc,0xdf9193fc,0xd85b93fe,
	0xdf9193fe,0xd85b9400,0xdf939400,0xd85b9402,0xdf939402,0xd85b9404,0xdf959404,0xd85b9406,
	0xdf959406,0xd85b9408,0xdf979408,0xd85b940a,0xdf97940a,0xd85b940c,0xdf99940c,0xd85b940e,
	0xdf99940e,0xd85b9410,0xdf9b9410,0xd85b9412,0xdf9b9412,0xd85b9414,0xdf9d9414,0xd85b9416,
	0xdf9d9416,0xd85b9418,0xdf9f9418,0xd85b941a,0xdf9f941a,0xd85b941c,0xdfa1941c,0xd85b941e,
	0xdfa1941e,0xd85b9420,0xdfa39420,0xd85b9422,0xdfa39422,0xd85b9424,0xdfa59424,0xd85b9426,
	0xdfa59426,0xd85b9428,0xdfa79428,0xd85b942a,0xdfa7942a,0xd85b942c,0xdfa9942c,0xd85b942e,
	0xdfa9942e,0xd85b9430,0xdfab9430,0xd85b9432,0xdfab9432,0xd85b9434,0xdfad9434,0xd85b9436,
	0xdfad9436,0xd85b9438,0xdfaf9438,0xd85b943a,0xdfaf943a,0xd85b943c,0xdfb1943c,0xd85b943e,
	0xdfb1943e,0xd85b9440,0xdfb39440,0xd85b9442,0xdfb39442,0xd85b9444,0xdfb59444,0xd85b9446,
	0xdfb59446,0xd85b9448,0xdfb79448,0xd85b944a,0xdfb7944a,0xd85b944c,0xdfb9944c,0xd85b944e,
	0xdfb9944e,0xd85b9450,0xdfbb9450,0xd85b9452,0xdfbb9452,0xd85b9454,0xdfbd9454,0xd85b9456,
	0xdfbd9456,0xd85b9458,0xdfbf9458,0xd85b945a,0xdfbf945a,0xd85b945c,0xdfc1945c,0xd85b945e,
	0xdfc1945e,0xd85b9460,0xdfc39460,0xd85b9462,0xdfc39462,0xd85b9464,0xdfc59464,0xd85b9466,
	0xdfc59466,0xd85b9468,0xdfc79468,0xd85b946a,0xdfc7946a,0xd85b946c,0xdfc9946c,0xd85b946e,
	0xdfc9946e,0xd85b9470,0xdfcb9470,0xd85b9472,0xdfcb9472,0xd85b9474,0xdfcd9474,0xd85b9476,
	0xdfcd9476,0xd85b9478,0xdfcf9478,0xd85b947a,0xdfcf947a,0xd85b947c,0xdfd1947c,0xd85b947e,
	0xdfd1947e,0xd85b9480,0xdfd39480,0xd85b9482,0xdfd39482,0xd85b9484,0xdfd59484,0xd85b9486,
	0xdfd59486,0xd85b9488,0xdfd79488,0xd85b948a,0xdfd7948a,0xd85b948c,0xdfd9948c,0xd85b948e,
	0xdfd9948e,0xd85b9490,0xdfdb9490,0xd85b9492,0xdfdb9492,0xd85b9494,0xdfdd9494,0xd85b9496,
	0xdfdd9496,0xd85b9498,0xdfdf9498,0xd85b949a,0xdfdf949a,0xd85b949c,0xdfe1949c,0xd85b949e,
	0xdfe1949e,0xd85b94a0,0xdfe394a0,0xd85b94a2,0xdfe394a2,0xd85b94a4,0xdfe594a4,0xd85b94a6,
	0xdfe594a6,0xd85b94a8,0xdfe794a8,0xd85b94aa,0xdfe794aa,0xd85b94ac,0xdfe994ac,0xd85b94ae,
	0xdfe994ae,0xd85b94b0,0xdfeb94b0,0xd85b94b2,0xdfeb94b2,0xd85b94b4,0xdfed94b4,0xd85b94b6,
	0xdfed94b6,0xd85b94b8,0xdfef94b8,0xd85b94ba,0xdfef94ba,0xd85b94bc,0xdff194bc,0xd85b94be,
	0xdff194be,0xd85b94c0,0xdff394c0,0xd85b94c2,0xdff394c2,0xd85b94c4,0xdff594c4,0xd85b94c6,
	0xdff594c6,0xd85b94c8,0xdff794c8,0xd85b94ca,0xdff794ca,0xd85b94cc,0xdff994cc,0xd85b94ce,
	0xdff994ce,0xd85b94d0,0xdffb94d0,0xd85b94d2,0xdffb94d2,0xd85b94d4,0xdffd94d4,0xd85b94d6,
	0xdffd94d6,0xd85b94d8,0xdfff94d8,0xd85b94da,0xdfff94da,0xd85b94dc,0xdc0194dc,0xd85b94de,
	0xdc0194de,0xd85b94e0,0xdc0394e0,0xd85b94e2,0xdc0394e2,0xd85b94e4,0xdc0594e4,0xd85b94e6,
	0xdc0594e6,0xd85b94e8,0xdc0794e8,0xd85b94ea,0xdc0794ea,0xd85b94ec,0xdc0994ec,0xd85b94ee,
	0xdc0994ee,0xd85b94f0,0xdc0b94f0,0xd85b94f2,0xdc0b94f2,0xd85b94f4,0xdc0d94f4,0xd85b94f6,
	0xdc0d94f6,0xd85b94f8,0xdc0f94f8,0xd85b94fa,0xdc0f94fa,0xd85b94fc,0xdc1194fc,0xd85b94fe,
	0xdc1194fe,0xd85b9500,0xdc139500,0xd85b9502,0xdc139502,0xd85b9504,0xdc159504,0xd85b9506,
	0xdc159506,0xd85b9508,0xdc179508,0xd85b950a,0xdc17950a,0xd85b950c,0xdc19950c,0xd85b950e,
	0xdc19950e,0xd85b9510,0xdc1b9510,0xd85b9512,0xdc1b9512,0xd85b9514,0xdc1d9514,0xd85b9516,
	0xdc1d9516,0xd85b9518,0xdc1f9518,0xd85b951a,0xdc1f951a,0xd85b951c,0xdc21951c,0xd85b951e,
	0xdc21951e,0xd85b9520,0xdc239520,0xd85b9522,0xdc239522,0xd85b9524,0xdc259524,0xd85b9526,
	0xdc259526,0xd85b9528,0xdc279528,0xd85b952a,0xdc27952a,0xd85b952c,0xdc29952c,0xd85b952e,
	0xdc29952e,0xd85b9530,0xdc2b9530,0xd85b9532,0xdc2b9532,0xd85b9534,0xdc2d9534,0xd85b9536,
	0xdc2d9536,0xd85b9538,0xdc2f9538,0xd85b953a,0xdc2f953a,0xd85b953c,0xdc31953c,0xd85b953e,
	0xdc31953e,0xd85b9540,0xdc339540,0xd85b9542,0xdc339542,0xd85b9544,0xdc359544,0xd85b9546,
	0xdc359546,0xd85b9548,0xdc379548,0xd85b954a,0xdc37954a,0xd85b954c,0xdc39954c,0xd85b954e,
	0xdc39954e,0xd85b9550,0xdc3b9550,0xd85b9552,0xdc3b9552,0xd85b9554,0xdc3d9554,0xd85b9556,
	0xdc3d9556,0xd85b9558,0xdc3f9558,0xd85b955a,0xdc3f955a,0xd85b955c,0xdc41955c,0xd85b955e,
	0xdc41955e,0xd85b9560,0xdc439560,0xd85b9562,0xdc439562,0xd85b9564,0xdc459564,0xd85b9566,
	0xdc459566,0xd85b9568,0xdc479568,0xd85b956a,0xdc47956a,0xd85b956c,0xdc49956c,0xd85b956e,
	0xdc49956e,0xd85b9570,0xdc4b9570,0xd85b9572,0xdc4b9572,0xd85b9574,0xdc4d9574,0xd85b9576,
	0xdc4d9576,0xd85b9578,0xdc4f9578,0xd85b957a,0xdc4f957a,0xd85b957c,0xdc51957c,0xd85b957e,
	0xdc51957e,0xd85b9580,0xdc539580,0xd85b9582,0xdc539582,0xd85b9584,0xdc559584,0xd85b9586,
	0xdc559586,0xd85b9588,0xdc579588,0xd85b958a,0xdc57958a,0xd85b958c,0xdc59958c,0xd85b958e,
	0xdc59958e,0xd85b9590,0xdc5b9590,0xd85b9592,0xdc5b9592,0xd85b9594,0xdc5d9594,0xd85b9596,
	0xdc5d9596,0xd85b9598,0xdc5f9598,0xd85b959a,0xdc5f959a,0xd85b959c,0xdc61959c,0xd85b959e,
	0xdc61959e,0xd85b95a0,0xdc6395a0,0xd85b95a2,0xdc6395a2,0xd85b95a4,0xdc6595a4,0xd85b95a6,
	0xdc6595a6,0xd85b95a8,0xdc6795a8,0xd85b95aa,0xdc6795aa,0xd85b95ac,0xdc6995ac,0xd85b95ae,
	0xdc6995ae,0xd85b95b0,0xdc6b95b0,0xd85b95b2,0xdc6b95b2,0xd85b95b4,0xdc6d95b4,0xd85b95b6,
	0xdc6d95b6,0xd85b95b8,0xdc6f95b8,0xd85b95ba,0xdc6f95ba,0xd85b95bc,0xdc7195bc,0xd85b95be,
	0xdc7195be,0xd85b95c0,0xdc7395c0,0xd85b95c2,0xdc7395c2,0xd85b95c4,0xdc7595c4,0xd85b95c6,
	0xdc7595c6,0xd85b95c8,0xdc7795c8,0xd85b95ca,0xdc7795ca,0xd85b95cc,0xdc7995cc,0xd85b95ce,
	0xdc7995ce,0xd85b95d0,0xdc7b95d0,0xd85b95d2,0xdc7b95d2,0xd85b95d4,0xdc7d95d4,0xd85b95d6,
	0xdc7d95d6,0xd85b95d8,0xdc7f95d8,0xd85b95da,0xdc7f95da,0xd85b95dc,0xdc8195dc,0xd85b95de,
	0xdc8195de,0xd85b95e0,0xdc8395e0,0xd85b95e2,0xdc8395e2,0xd85b95e4,0xdc8595e4,0xd85b95e6,
	0xdc8595e6,0xd85b95e8,0xdc8795e8,0xd85b95ea,0xdc8795ea,0xd85b95ec,0xdc8995ec,0xd85b95ee,
	0xdc8995ee,0xd85b95f0,0xdc8b95f0,0xd85b95f2,0xdc8b95f2,0xd85b95f4,0xdc8d95f4,0xd85b95f6,
	0xdc8d95f6,0xd85b95f8,0xdc8f95f8,0xd85b95fa,0xdc8f95fa,0xd85b95fc,0xdc9195fc,0xd85b95fe,
	0xdc9195fe,0xd85b9600,0xdc939600,0xd85b9602,0xdc939602,0xd85b9604,0xdc959604,0xd85b9606,
	0xdc959606,0xd85b9608,0xdc979608,0xd85b960a,0xdc97960a,0xd85b960c,0xdc99960c,0xd85b960e,
	0xdc99960e,0xd85b9610,0xdc9b9610,0xd85b9612,0xdc9b9612,0xd85b9614,0xdc9d9614,0xd85b9616,
	0xdc9d9616,0xd85b9618,0xdc9f9618,0xd85b961a,0xdc9f961a,0xd85b961c,0xdca1961c,0xd85b961e,
	0xdca1961e,0xd85b9620,0xdca39620,0xd85b9622,0xdca39622,0xd85b9624,0xdca59624,0xd85b9626,
	0xdca59626,0xd85b9628,0xdca79628,0xd85b962a,0xdca7962a,0xd85b962c,0xdca9962c,0xd85b962e,
	0xdca9962e,0xd85b9630,0xdcab9630,0xd85b9632,0xdcab9632,0xd85b9634,0xdcad9634,0xd85b9636,
	0xdcad9636,0xd85b9638,0xdcaf9638,0xd85b963a,0xdcaf963a,0xd85b963c,0xdcb1963c,0xd85b963e,
	0xdcb1963e,0xd85b9640,0xdcb39640,0xd85b9642,0xdcb39642,0xd85b9644,0xdcb59644,0xd85b9646,
	0xdcb59646,0xd85b9648,0xdcb79648,0xd85b964a,0xdcb7964a,0xd85b964c,0xdcb9964c,0xd85b964e,
	0xdcb9964e,0xd85b9650,0xdcbb9650,0xd85b9652,0xdcbb9652,0xd85b9654,0xdcbd9654,0xd85b9656,
	0xdcbd9656,0xd85b9658,0xdcbf9658,0xd85b965a,0xdcbf965a,0xd85b965c,0xdcc1965c,0xd85b965e,
	0xdcc1965e,0xd85b9660,0xdcc39660,0xd85b9662,0xdcc39662,0xd85b9664,0xdcc59664,0xd85b9666,
	0xdcc59666,0xd85b9668,0xdcc79668,0xd85b966a,0xdcc7966a,0xd85b966c,0xdcc9966c,0xd85b966e,
	0xdcc9966e,0xd85b9670,0xdccb9670,0xd85b9672,0xdccb9672,0xd85b9674,0xdccd9674,0xd85b9676,
	0xdccd9676,0xd85b9678,0xdccf9678,0xd85b967a,0xdccf967a,0xd85b967c,0xdcd1967c,0xd85b967e,
	0xdcd1967e,0xd85b9680,0xdcd39680,0xd85b9682,0xdcd39682,0xd85b9684,0xdcd59684,0xd85b9686,
	0xdcd59686,0xd85b9688,0xdcd79688,0xd85b968a,0xdcd7968a,0xd85b968c,0xdcd9968c,0xd85b968e,
	0xdcd9968e,0xd85b9690,0xdcdb9690,0xd85b9692,0xdcdb9692,0xd85b9694,0xdcdd9694,0xd85b9696,
	0xdcdd9696,0xd85b9698,0xdcdf9698,0xd85b969a,0xdcdf969a,0xd85b969c,0xdce1969c,0xd85b969e,
	0xdce1969e,0xd85b96a0,0xdce396a0,0xd85b96a2,0xdce396a2,0xd85b96a4,0xdce596a4,0xd85b96a6,
	0xdce596a6,0xd85b96a8,0xdce796a8,0xd85b96aa,0xdce796aa,0xd85b96ac,0xdce996ac,0xd85b96ae,
	0xdce996ae,0xd85b96b0,0xdceb96b0,0xd85b96b2,0xdceb96b2,0xd85b96b4,0xdced96b4,0xd85b96b6,
	0xdced96b6,0xd85b96b8,0xdcef96b8,0xd85b96ba,0xdcef96ba,0xd85b96bc,0xdcf196bc,0xd85b96be,
	0xdcf196be,0xd85b96c0,0xdcf396c0,0xd85b96c2,0xdcf396c2,0xd85b96c4,0xdcf596c4,0xd85b96c6,
	0xdcf596c6,0xd85b96c8,0xdcf796c8,0xd85b96ca,0xdcf796ca,0xd85b96cc,0xdcf996cc,0xd85b96ce,
	0xdcf996ce,0xd85b96d0,0xdcfb96d0,0xd85b96d2,0xdcfb96d2,0xd85b96d4,0xdcfd96d4,0xd85b96d6,
	0xdcfd96d6,0xd85b96d8,0xdcff96d8,0xd85b96da,0xdcff96da,0xd85b96dc,0xdd0196dc,0xd85b96de,
	0xdd0196de,0xd85b96e0,0xdd0396e0,0xd85b96e2,0xdd0396e2,0xd85b96e4,0xdd0596e4,0xd85b96e6,
	0xdd0596e6,0xd85b96e8,0xdd0796e8,0xd85b96ea,0xdd0796ea,0xd85b96ec,0xdd0996ec,0xd85b96ee,
	0xdd0996ee,0xd85b96f0,0xdd0b96f0,0xd85b96f2,0xdd0b96f2,0xd85b96f4,0xdd0d96f4,0xd85b96f6,
	0xdd0d96f6,0xd85b96f8,0xdd0f96f8,0xd85b96fa,0xdd0f96fa,0xd85b96fc,0xdd1196fc,0xd85b96fe,
	0xdd1196fe,0xd85b9700,0xdd139700,0xd85b9702,0xdd139702,0xd85b9704,0xdd159704,0xd85b9706,
	0xdd159706,0xd85b9708,0xdd179708,0xd85b970a,0xdd17970a,0xd85b970c,0xdd19970c,0xd85b970e,
	0xdd19970e,0xd85b9710,0xdd1b9710,0xd85b9712,0xdd1b9712,0xd85b9714,0xdd1d9714,0xd85b9716,
	0xdd1d9716,0xd85b9718,0xdd1f9718,0xd85b971a,0xdd1f971a,0xd85b971c,0xdd21971c,0xd85b971e,
	0xdd21971e,0xd85b9720,0xdd239720,0xd85b9722,0xdd239722,0xd85b9724,0xdd259724,0xd85b9726,
	0xdd259726,0xd85b9728,0xdd279728,0xd85b972a,0xdd27972a,0xd85b972c,0xdd29972c,0xd85b972e,
	0xdd29972e,0xd85b9730,0xdd2b9730,0xd85b9732,0xdd2b9732,0xd85b9734,0xdd2d9734,0xd85b9736,
	0xdd2d9736,0xd85b9738,0xdd2f9738,0xd85b973a,0xdd2f973a,0xd85b973c,0xdd31973c,0xd85b973e,
	0xdd31973e,0xd85b9740,0xdd339740,0xd85b9742,0xdd339742,0xd85b9744,0xdd359744,0xd85b9746,
	0xdd359746,0xd85b9748,0xdd379748,0xd85b974a,0xdd37974a,0xd85b974c,0xdd39974c,0xd85b974e,
	0xdd39974e,0xd85b9750,0xdd3b9750,0xd85b9752,0xdd3b9752,0xd85b9754,0xdd3d9754,0xd85b9756,
	0xdd3d9756,0xd85b9758,0xdd3f9758,0xd85b975a,0xdd3f975a,0xd85b975c,0xdd41975c,0xd85b975e,
	0xdd41975e,0xd85b9760,0xdd439760,0xd85b9762,0xdd439762,0xd85b9764,0xdd459764,0xd85b9766,
	0xdd459766,0xd85b9768,0xdd479768,0xd85b976a,0xdd47976a,0xd85b976c,0xdd49976c,0xd85b976e,
	0xdd49976e,0xd85b9770,0xdd4b9770,0xd85b9772,0xdd4b9772,0xd85b9774,0xdd4d9774,0xd85b9776,
	0xdd4d9776,0xd85b9778,0xdd4f9778,0xd85b977a,0xdd4f977a,0xd85b977c,0xdd51977c,0xd85b977e,
	0xdd51977e,0xd85b9780,0xdd539780,0xd85b9782,0xdd539782,0xd85b9784,0xdd559784,0xd85b9786,
	0xdd559786,0xd85b9788,0xdd579788,0xd85b978a,0xdd57978a,0xd85b978c,0xdd59978c,0xd85b978e,
	0xdd59978e,0xd85b9790,0xdd5b9790,0xd85b9792,0xdd5b9792,0xd85b9794,0xdd5d9794,0xd85b9796,
	0xdd5d9796,0xd85b9798,0xdd5f9798,0xd85b979a,0xdd5f979a,0xd85b979c,0xdd61979c,0xd85b979e,
	0xdd61979e,0xd85b97a0,0xdd6397a0,0xd85b97a2,0xdd6397a2,0xd85b97a4,0xdd6597a4,0xd85b97a6,
	0xdd6597a6,0xd85b97a8,0xdd6797a8,0xd85b97aa,0xdd6797aa,0xd85b97ac,0xdd6997ac,0xd85b97ae,
	0xdd6997ae,0xd85b97b0,0xdd6b97b0,0xd85b97b2,0xdd6b97b2,0xd85b97b4,0xdd6d97b4,0xd85b97b6,
	0xdd6d97b6,0xd85b97b8,0xdd6f97b8,0xd85b97ba,0xdd6f97ba,0xd85b97bc,0xdd7197bc,0xd85b97be,
	0xdd7197be,0xd85b97c0,0xdd7397c0,0xd85b97c2,0xdd7397c2,0xd85b97c4,0xdd7597c4,0xd85b97c6,
	0xdd7597c6,0xd85b97c8,0xdd7797c8,0xd85b97ca,0xdd7797ca,0xd85b97cc,0xdd7997cc,0xd85b97ce,
	0xdd7997ce,0xd85b97d0,0xdd7b97d0,0xd85b97d2,0xdd7b97d2,0xd85b97d4,0xdd7d97d4,0xd85b97d6,
	0xdd7d97d6,0xd85b97d8,0xdd7f97d8,0xd85b97da,0xdd7f97da,0xd85b97dc,0xdd8197dc,0xd85b97de,
	0xdd8197de,0xd85b97e0,0xdd8397e0,0xd85b97e2,0xdd8397e2,0xd85b97e4,0xdd8597e4,0xd85b97e6,
	0xdd8597e6,0xd85b97e8,0xdd8797e8,0xd85b97ea,0xdd8797ea,0xd85b97ec,0xdd8997ec,0xd85b97ee,
	0xdd8997ee,0xd85b97f0,0xdd8b97f0,0xd85b97f2,0xdd8b97f2,0xd85b97f4,0xdd8d97f4,0xd85b97f6,
	0xdd8d97f6,0xd85b97f8,0xdd8f97f8,0xd85b97fa,0xdd8f97fa,0xd85b97fc,0xdd9197fc,0xd85b97fe,
	0xdd9197fe,0xd85b9800,0xdd939800,0xd85b9802,0xdd939802,0xd85b9804,0xdd959804,0xd85b9806,
	0xdd959806,0xd85b9808,0xdd979808,0xd85b980a,0xdd97980a,0xd85b980c,0xdd99980c,0xd85b980e,
	0xdd99980e,0xd85b9810,0xdd9b9810,0xd85b9812,0xdd9b9812,0xd85b9814,0xdd9d9814,0xd85b9816,
	0xdd9d9816,0xd85b9818,0xdd9f9818,0xd85b981a,0xdd9f981a,0xd85b981c,0xdda1981c,0xd85b981e,
	0xdda1981e,0xd85b9820,0xdda39820,0xd85b9822,0xdda39822,0xd85b9824,0xdda59824,0xd85b9826,
	0xdda59826,0xd85b9828,0xdda79828,0xd85b982a,0xdda7982a,0xd85b982c,0xdda9982c,0xd85b982e,
	0xdda9982e,0xd85b9830,0xddab9830,0xd85b9832,0xddab9832,0xd85b9834,0xddad9834,0xd85b9836,
	0xddad9836,0xd85b9838,0xddaf9838,0xd85b983a,0xddaf983a,0xd85b983c,0xddb1983c,0xd85b983e,
	0xddb1983e,0xd85b9840,0xddb39840,0xd85b9842,0xddb39842,0xd85b9844,0xddb59844,0xd85b9846,
	0xddb59846,0xd85b9848,0xddb79848,0xd85b984a,0xddb7984a,0xd85b984c,0xddb9984c,0xd85b984e,
	0xddb9984e,0xd85b9850,0xddbb9850,0xd85b9852,0xddbb9852,0xd85b9854,0xddbd9854,0xd85b9856,
	0xddbd9856,0xd85b9858,0xddbf9858,0xd85b985a,0xddbf985a,0xd85b985c,0xddc1985c,0xd85b985e,
	0xddc1985e,0xd85b9860,0xddc39860,0xd85b9862,0xddc39862,0xd85b9864,0xddc59864,0xd85b9866,
	0xddc59866,0xd85b9868,0xddc79868,0xd85b986a,0xddc7986a,0xd85b986c,0xddc9986c,0xd85b986e,
	0xddc9986e,0xd85b9870,0xddcb9870,0xd85b9872,0xddcb9872,0xd85b9874,0xddcd9874,0xd85b9876,
	0xddcd9876,0xd85b9878,0xddcf9878,0xd85b987a,0xddcf987a,0xd85b987c,0xddd1987c,0xd85b987e,
	0xddd1987e,0xd85b9880,0xddd39880,0xd85b9882,0xddd39882,0xd85b9884,0xddd59884,0xd85b9886,
	0xddd59886,0xd85b9888,0xddd79888,0xd85b988a,0xddd7988a,0xd85b988c,0xddd9988c,0xd85b988e,
	0xddd9988e,0xd85b9890,0xdddb9890,0xd85b9892,0xdddb9892,0xd85b9894,0xdddd9894,0xd85b9896,
	0xdddd9896,0xd85b9898,0xdddf9898,0xd85b989a,0xdddf989a,0xd85b989c,0xdde1989c,0xd85b989e,
	0xdde1989e,0xd85b98a0,0xdde398a0,0xd85b98a2,0xdde398a2,0xd85b98a4,0xdde598a4,0xd85b98a6,
	0xdde598a6,0xd85b98a8,0xdde798a8,0xd85b98aa,0xdde798aa,0xd85b98ac,0xdde998ac,0xd85b98ae,
	0xdde998ae,0xd85b98b0,0xddeb98b0,0xd85b98b2,0xddeb98b2,0xd85b98b4,0xdded98b4,0xd85b98b6,
	0xdded98b6,0xd85b98b8,0xddef98b8,0xd85b98ba,0xddef98ba,0xd85b98bc,0xddf198bc,0xd85b98be,
	0xddf198be,0xd85b98c0,0xddf398c0,0xd85b98c2,0xddf398c2,0xd85b98c4,0xddf598c4,0xd85b98c6,
	0xddf598c6,0xd85b98c8,0xddf798c8,0xd85b98ca,0xddf798ca,0xd85b98cc,0xddf998cc,0xd85b98ce,
	0xddf998ce,0xd85b98d0,0xddfb98d0,0xd85b98d2,0xddfb98d2,0xd85b98d4,0xddfd98d4,0xd85b98d6,
	0xddfd98d6,0xd85b98d8,0xddff98d8,0xd85b98da,0xddff98da,0xd85b98dc,0xde0198dc,0xd85b98de,
	0xde0198de,0xd85b98e0,0xde0398e0,0xd85b98e2,0xde0398e2,0xd85b98e4,0xde0598e4,0xd85b98e6,
	0xde0598e6,0xd85b98e8,0xde0798e8,0xd85b98ea,0xde0798ea,0xd85b98ec,0xde0998ec,0xd85b98ee,
	0xde0998ee,0xd85b98f0,0xde0b98f0,0xd85b98f2,0xde0b98f2,0xd85b98f4,0xde0d98f4,0xd85b98f6,
	0xde0d98f6,0xd85b98f8,0xde0f98f8,0xd85b98fa,0xde0f98fa,0xd85b98fc,0xde1198fc,0xd85b98fe,
	0xde1198fe,0xd85b9900,0xde139900,0xd85b9902,0xde139902,0xd85b9904,0xde159904,0xd85b9906,
	0xde159906,0xd85b9908,0xde179908,0xd85b990a,0xde17990a,0xd85b990c,0xde19990c,0xd85b990e,
	0xde19990e,0xd85b9910,0xde1b9910,0xd85b9912,0xde1b9912,0xd85b9914,0xde1d9914,0xd85b9916,
	0xde1d9916,0xd85b9918,0xde1f9918,0xd85b991a,0xde1f991a,0xd85b991c,0xde21991c,0xd85b991e,
	0xde21991e,0xd85b9920,0xde239920,0xd85b9922,0xde239922,0xd85b9924,0xde259924,0xd85b9926,
	0xde259926,0xd85b9928,0xde279928,0xd85b992a,0xde27992a,0xd85b992c,0xde29992c,0xd85b992e,
	0xde29992e,0xd85b9930,0xde2b9930,0xd85b9932,0xde2b9932,0xd85b9934,0xde2d9934,0xd85b9936,
	0xde2d9936,0xd85b9938,0xde2f9938,0xd85b993a,0xde2f993a,0xd85b993c,0xde31993c,0xd85b993e,
	0xde31993e,0xd85b9940,0xde339940,0xd85b9942,0xde339942,0xd85b9944,0xde359944,0xd85b9946,
	0xde359946,0xd85b9948,0xde379948,0xd85b994a,0xde37994a,0xd85b994c,0xde39994c,0xd85b994e,
	0xde39994e,0xd85b9950,0xde3b9950,0xd85b9952,0xde3b9952,0xd85b9954,0xde3d9954,0xd85b9956,
	0xde3d9956,0xd85b9958,0xde3f9958,0xd85b995a,0xde3f995a,0xd85b995c,0xde41995c,0xd85b995e,
	0xde41995e,0xd85b9960,0xde439960,0xd85b9962,0xde439962,0xd85b9964,0xde459964,0xd85b9966,
	0xde459966,0xd85b9968,0xde479968,0xd85b996a,0xde47996a,0xd85b996c,0xde49996c,0xd85b996e,
	0xde49996e,0xd85b9970,0xde4b9970,0xd85b9972,0xde4b9972,0xd85b9974,0xde4d9974,0xd85b9976,
	0xde4d9976,0xd85b9978,0xde4f9978,0xd85b997a,0xde4f997a,0xd85b997c,0xde51997c,0xd85b997e,
	0xde51997e,0xd85b9980,0xde539980,0xd85b9982,0xde539982,0xd85b9984,0xde559984,0xd85b9986,
	0xde559986,0xd85b9988,0xde579988,0xd85b998a,0xde57998a,0xd85b998c,0xde59998c,0xd85b998e,
	0xde59998e,0xd85b9990,0xde5b9990,0xd85b9992,0xde5b9992,0xd85b9994,0xde5d9994,0xd85b9996,
	0xde5d9996,0xd85b9998,0xde5f9998,0xd85b999a,0xde5f999a,0xd85b999c,0xde61999c,0xd85b999e,
	0xde61999e,0xd85b99a0,0xde6399a0,0xd85b99a2,0xde6399a2,0xd85b99a4,0xde6599a4,0xd85b99a6,
	0xde6599a6,0xd85b99a8,0xde6799a8,0xd85b99aa,0xde6799aa,0xd85b99ac,0xde6999ac,0xd85b99ae,
	0xde6999ae,0xd85b99b0,0xde6b99b0,0xd85b99b2,0xde6b99b2,0xd85b99b4,0xde6d99b4,0xd85b99b6,
	0xde6d99b6,0xd85b99b8,0xde6f99b8,0xd85b99ba,0xde6f99ba,0xd85b99bc,0xde7199bc,0xd85b99be,
	0xde7199be,0xd85b99c0,0xde7399c0,0xd85b99c2,0xde7399c2,0xd85b99c4,0xde7599c4,0xd85b99c6,
	0xde7599c6,0xd85b99c8,0xde7799c8,0xd85b99ca,0xde7799ca,0xd85b99cc,0xde7999cc,0xd85b99ce,
	0xde7999ce,0xd85b99d0,0xde7b99d0,0xd85b99d2,0xde7b99d2,0xd85b99d4,0xde7d99d4,0xd85b99d6,
	0xde7d99d6,0xd85b99d8,0xde7f99d8,0xd85b99da,0xde7f99da,0xd85b99dc,0xde8199dc,0xd85b99de,
	0xde8199de,0xd85b99e0,0xde8399e0,0xd85b99e2,0xde8399e2,0xd85b99e4,0xde8599e4,0xd85b99e6,
	0xde8599e6,0xd85b99e8,0xde8799e8,0xd85b99ea,0xde8799ea,0xd85b99ec,0xde8999ec,0xd85b99ee,
	0xde8999ee,0xd85b99f0,0xde8b99f0,0xd85b99f2,0xde8b99f2,0xd85b99f4,0xde8d99f4,0xd85b99f6,
	0xde8d99f6,0xd85b99f8,0xde8f99f8,0xd85b99fa,0xde8f99fa,0xd85b99fc,0xde9199fc,0xd85b99fe,
	0xde9199fe,0xd85b9a00,0xde939a00,0xd85b9a02,0xde939a02,0xd85b9a04,0xde959a04,0xd85b9a06,
	0xde959a06,0xd85b9a08,0xde979a08,0xd85b9a0a,0xde979a0a,0xd85b9a0c,0xde999a0c,0xd85b9a0e,
	0xde999a0e,0xd85b9a10,0xde9b9a10,0xd85b9a12,0xde9b9a12,0xd85b9a14,0xde9d9a14,0xd85b9a16,
	0xde9d9a16,0xd85b9a18,0xde9f9a18,0xd85b9a1a,0xde9f9a1a,0xd85b9a1c,0xdea19a1c,0xd85b9a1e,
	0xdea19a1e,0xd85b9a20,0xdea39a20,0xd85b9a22,0xdea39a22,0xd85b9a24,0xdea59a24,0xd85b9a26,
	0xdea59a26,0xd85b9a28,0xdea79a28,0xd85b9a2a,0xdea79a2a,0xd85b9a2c,0xdea99a2c,0xd85b9a2e,
	0xdea99a2e,0xd85b9a30,0xdeab9a30,0xd85b9a32,0xdeab9a32,0xd85b9a34,0xdead9a34,0xd85b9a36,
	0xdead9a36,0xd85b9a38,0xdeaf9a38,0xd85b9a3a,0xdeaf9a3a,0xd85b9a3c,0xdeb19a3c,0xd85b9a3e,
	0xdeb19a3e,0xd85b9a40,0xdeb39a40,0xd85b9a42,0xdeb39a42,0xd85b9a44,0xdeb59a44,0xd85b9a46,
	0xdeb59a46,0xd85b9a48,0xdeb79a48,0xd85b9a4a,0xdeb79a4a,0xd85b9a4c,0xdeb99a4c,0xd85b9a4e,
	0xdeb99a4e,0xd85b9a50,0xdebb9a50,0xd85b9a52,0xdebb9a52,0xd85b9a54,0xdebd9a54,0xd85b9a56,
	0xdebd9a56,0xd85b9a58,0xdebf9a58,0xd85b9a5a,0xdebf9a5a,0xd85b9a5c,0xdec19a5c,0xd85b9a5e,
	0xdec19a5e,0xd85b9a60,0xdec39a60,0xd85b9a62,0xdec39a62,0xd85b9a64,0xdec59a64,0xd85b9a66,
	0xdec59a66,0xd85b9a68,0xdec79a68,0xd85b9a6a,0xdec79a6a,0xd85b9a6c,0xdec99a6c,0xd85b9a6e,
	0xdec99a6e,0xd85b9a70,0xdecb9a70,0xd85b9a72,0xdecb9a72,0xd85b9a74,0xdecd9a74,0xd85b9a76,
	0xdecd9a76,0xd85b9a78,0xdecf9a78,0xd85b9a7a,0xdecf9a7a,0xd85b9a7c,0xded19a7c,0xd85b9a7e,
	0xded19a7e,0xd85b9a80,0xded39a80,0xd85b9a82,0xded39a82,0xd85b9a84,0xded59a84,0xd85b9a86,
	0xded59a86,0xd85b9a88,0xded79a88,0xd85b9a8a,0xded79a8a,0xd85b9a8c,0xded99a8c,0xd85b9a8e,
	0xded99a8e,0xd85b9a90,0xdedb9a90,0xd85b9a92,0xdedb9a92,0xd85b9a94,0xdedd9a94,0xd85b9a96,
	0xdedd9a96,0xd85b9a98,0xdedf9a98,0xd85b9a9a,0xdedf9a9a,0xd85b9a9c,0xdee19a9c,0xd85b9a9e,
	0xdee19a9e,0xd85b9aa0,0xdee39aa0,0xd85b9aa2,0xdee39aa2,0xd85b9aa4,0xdee59aa4,0xd85b9aa6,
	0xdee59aa6,0xd85b9aa8,0xdee79aa8,0xd85b9aaa,0xdee79aaa,0xd85b9aac,0xdee99aac,0xd85b9aae,
	0xdee99aae,0xd85b9ab0,0xdeeb9ab0,0xd85b9ab2,0xdeeb9ab2,0xd85b9ab4,0xdeed9ab4,0xd85b9ab6,
	0xdeed9ab6,0xd85b9ab8,0xdeef9ab8,0xd85b9aba,0xdeef9aba,0xd85b9abc,0xdef19abc,0xd85b9abe,
	0xdef19abe,0xd85b9ac0,0xdef39ac0,0xd85b9ac2,0xdef39ac2,0xd85b9ac4,0xdef59ac4,0xd85b9ac6,
	0xdef59ac6,0xd85b9ac8,0xdef79ac8,0xd85b9aca,0xdef79aca,0xd85b9acc,0xdef99acc,0xd85b9ace,
	0xdef99ace,0xd85b9ad0,0xdefb9ad0,0xd85b9ad2,0xdefb9ad2,0xd85b9ad4,0xdefd9ad4,0xd85b9ad6,
	0xdefd9ad6,0xd85b9ad8,0xdeff9ad8,0xd85b9ada,0xdeff9ada,0xd85b9adc,0xdf019adc,0xd85b9ade,
	0xdf019ade,0xd85b9ae0,0xdf039ae0,0xd85b9ae2,0xdf039ae2,0xd85b9ae4,0xdf059ae4,0xd85b9ae6,
	0xdf059ae6,0xd85b9ae8,0xdf079ae8,0xd85b9aea,0xdf079aea,0xd85b9aec,0xdf099aec,0xd85b9aee,
	0xdf099aee,0xd85b9af0,0xdf0b9af0,0xd85b9af2,0xdf0b9af2,0xd85b9af4,0xdf0d9af4,0xd85b9af6,
	0xdf0d9af6,0xd85b9af8,0xdf0f9af8,0xd85b9afa,0xdf0f9afa,0xd85b9afc,0xdf119afc,0xd85b9afe,
	0xdf119afe,0xd85b9b00,0xdf139b00,0xd85b9b02,0xdf139b02,0xd85b9b04,0xdf159b04,0xd85b9b06,
	0xdf159b06,0xd85b9b08,0xdf179b08,0xd85b9b0a,0xdf179b0a,0xd85b9b0c,0xdf199b0c,0xd85b9b0e,
	0xdf199b0e,0xd85b9b10,0xdf1b9b10,0xd85b9b12,0xdf1b9b12,0xd85b9b14,0xdf1d9b14,0xd85b9b16,
	0xdf1d9b16,0xd85b9b18,0xdf1f9b18,0xd85b9b1a,0xdf1f9b1a,0xd85b9b1c,0xdf219b1c,0xd85b9b1e,
	0xdf219b1e,0xd85b9b20,0xdf239b20,0xd85b9b22,0xdf239b22,0xd85b9b24,0xdf259b24,0xd85b9b26,
	0xdf259b26,0xd85b9b28,0xdf279b28,0xd85b9b2a,0xdf279b2a,0xd85b9b2c,0xdf299b2c,0xd85b9b2e,
	0xdf299b2e,0xd85b9b30,0xdf2b9b30,0xd85b9b32,0xdf2b9b32,0xd85b9b34,0xdf2d9b34,0xd85b9b36,
	0xdf2d9b36,0xd85b9b38,0xdf2f9b38,0xd85b9b3a,0xdf2f9b3a,0xd85b9b3c,0xdf319b3c,0xd85b9b3e,
	0xdf319b3e,0xd85b9b40,0xdf339b40,0xd85b9b42,0xdf339b42,0xd85b9b44,0xdf359b44,0xd85b9b46,
	0xdf359b46,0xd85b9b48,0xdf379b48,0xd85b9b4a,0xdf379b4a,0xd85b9b4c,0xdf399b4c,0xd85b9b4e,
	0xdf399b4e,0xd85b9b50,0xdf3b9b50,0xd85b9b52,0xdf3b9b52,0xd85b9b54,0xdf3d9b54,0xd85b9b56,
	0xdf3d9b56,0xd85b9b58,0xdf3f9b58,0xd85b9b5a,0xdf3f9b5a,0xd85b9b5c,0xdf419b5c,0xd85b9b5e,
	0xdf419b5e,0xd85b9b60,0xdf439b60,0xd85b9b62,0xdf439b62,0xd85b9b64,0xdf459b64,0xd85b9b66,
	0xdf459b66,0xd85b9b68,0xdf479b68,0xd85b9b6a,0xdf479b6a,0xd85b9b6c,0xdf499b6c,0xd85b9b6e,
	0xdf499b6e,0xd85b9b70,0xdf4b9b70,0xd85b9b72,0xdf4b9b72,0xd85b9b74,0xdf4d9b74,0xd85b9b76,
	0xdf4d9b76,0xd85b9b78,0xdf4f9b78,0xd85b9b7a,0xdf4f9b7a,0xd85b9b7c,0xdf519b7c,0xd85b9b7e,
	0xdf519b7e,0xd85b9b80,0xdf539b80,0xd85b9b82,0xdf539b82,0xd85b9b84,0xdf559b84,0xd85b9b86,
	0xdf559b86,0xd85b9b88,0xdf579b88,0xd85b9b8a,0xdf579b8a,0xd85b9b8c,0xdf599b8c,0xd85b9b8e,
	0xdf599b8e,0xd85b9b90,0xdf5b9b90,0xd85b9b92,0xdf5b9b92,0xd85b9b94,0xdf5d9b94,0xd85b9b96,
	0xdf5d9b96,0xd85b9b98,0xdf5f9b98,0xd85b9b9a,0xdf5f9b9a,0xd85b9b9c,0xdf619b9c,0xd85b9b9e,
	0xdf619b9e,0xd85b9ba0,0xdf639ba0,0xd85b9ba2,0xdf639ba2,0xd85b9ba4,0xdf659ba4,0xd85b9ba6,
	0xdf659ba6,0xd85b9ba8,0xdf679ba8,0xd85b9baa,0xdf679baa,0xd85b9bac,0xdf699bac,0xd85b9bae,
	0xdf699bae,0xd85b9bb0,0xdf6b9bb0,0xd85b9bb2,0xdf6b9bb2,0xd85b9bb4,0xdf6d9bb4,0xd85b9bb6,
	0xdf6d9bb6,0xd85b9bb8,0xdf6f9bb8,0xd85b9bba,0xdf6f9bba,0xd85b9bbc,0xdf719bbc,0xd85b9bbe,
	0xdf719bbe,0xd85b9bc0,0xdf739bc0,0xd85b9bc2,0xdf739bc2,0xd85b9bc4,0xdf759bc4,0xd85b9bc6,
	0xdf759bc6,0xd85b9bc8,0xdf779bc8,0xd85b9bca,0xdf779bca,0xd85b9bcc,0xdf799bcc,0xd85b9bce,
	0xdf799bce,0xd85b9bd0,0xdf7b9bd0,0xd85b9bd2,0xdf7b9bd2,0xd85b9bd4,0xdf7d9bd4,0xd85b9bd6,
	0xdf7d9bd6,0xd85b9bd8,0xdf7f9bd8,0xd85b9bda,0xdf7f9bda,0xd85b9bdc,0xdf819bdc,0xd85b9bde,
	0xdf819bde,0xd85b9be0,0xdf839be0,0xd85b9be2,0xdf839be2,0xd85b9be4,0xdf859be4,0xd85b9be6,
	0xdf859be6,0xd85b9be8,0xdf879be8,0xd85b9bea,0xdf879bea,0xd85b9bec,0xdf899bec,0xd85b9bee,
	0xdf899bee,0xd85b9bf0,0xdf8b9bf0,0xd85b9bf2,0xdf8b9bf2,0xd85b9bf4,0xdf8d9bf4,0xd85b9bf6,
	0xdf8d9bf6,0xd85b9bf8,0xdf8f9bf8,0xd85b9bfa,0xdf8f9bfa,0xd85b9bfc,0xdf919bfc,0xd85b9bfe,
	0xdf919bfe,0xd85b9c00,0xdf939c00,0xd85b9c02,0xdf939c02,0xd85b9c04,0xdf959c04,0xd85b9c06,
	0xdf959c06,0xd85b9c08,0xdf979c08,0xd85b9c0a,0xdf979c0a,0xd85b9c0c,0xdf999c0c,0xd85b9c0e,
	0xdf999c0e,0xd85b9c10,0xdf9b9c10,0xd85b9c12,0xdf9b9c12,0xd85b9c14,0xdf9d9c14,0xd85b9c16,
	0xdf9d9c16,0xd85b9c18,0xdf9f9c18,0xd85b9c1a,0xdf9f9c1a,0xd85b9c1c,0xdfa19c1c,0xd85b9c1e,
	0xdfa19c1e,0xd85b9c20,0xdfa39c20,0xd85b9c22,0xdfa39c22,0xd85b9c24,0xdfa59c24,0xd85b9c26,
	0xdfa59c26,0xd85b9c28,0xdfa79c28,0xd85b9c2a,0xdfa79c2a,0xd85b9c2c,0xdfa99c2c,0xd85b9c2e,
	0xdfa99c2e,0xd85b9c30,0xdfab9c30,0xd85b9c32,0xdfab9c32,0xd85b9c34,0xdfad9c34,0xd85b9c36,
	0xdfad9c36,0xd85b9c38,0xdfaf9c38,0xd85b9c3a,0xdfaf9c3a,0xd85b9c3c,0xdfb19c3c,0xd85b9c3e,
	0xdfb19c3e,0xd85b9c40,0xdfb39c40,0xd85b9c42,0xdfb39c42,0xd85b9c44,0xdfb59c44,0xd85b9c46,
	0xdfb59c46,0xd85b9c48,0xdfb79c48,0xd85b9c4a,0xdfb79c4a,0xd85b9c4c,0xdfb99c4c,0xd85b9c4e,
	0xdfb99c4e,0xd85b9c50,0xdfbb9c50,0xd85b9c52,0xdfbb9c52,0xd85b9c54,0xdfbd9c54,0xd85b9c56,
	0xdfbd9c56,0xd85b9c58,0xdfbf9c58,0xd85b9c5a,0xdfbf9c5a,0xd85b9c5c,0xdfc19c5c,0xd85b9c5e,
	0xdfc19c5e,0xd85b9c60,0xdfc39c60,0xd85b9c62,0xdfc39c62,0xd85b9c64,0xdfc59c64,0xd85b9c66,
	0xdfc59c66,0xd85b9c68,0xdfc79c68,0xd85b9c6a,0xdfc79c6a,0xd85b9c6c,0xdfc99c6c,0xd85b9c6e,
	0xdfc99c6e,0xd85b9c70,0xdfcb9c70,0xd85b9c72,0xdfcb9c72,0xd85b9c74,0xdfcd9c74,0xd85b9c76,
	0xdfcd9c76,0xd85b9c78,0xdfcf9c78,0xd85b9c7a,0xdfcf9c7a,0xd85b9c7c,0xdfd19c7c,0xd85b9c7e,
	0xdfd19c7e,0xd85b9c80,0xdfd39c80,0xd85b9c82,0xdfd39c82,0xd85b9c84,0xdfd59c84,0xd85b9c86,
	0xdfd59c86,0xd85b9c88,0xdfd79c88,0xd85b9c8a,0xdfd79c8a,0xd85b9c8c,0xdfd99c8c,0xd85b9c8e,
	0xdfd99c8e,0xd85b9c90,0xdfdb9c90,0xd85b9c92,0xdfdb9c92,0xd85b9c94,0xdfdd9c94,0xd85b9c96,
	0xdfdd9c96,0xd85b9c98,0xdfdf9c98,0xd85b9c9a,0xdfdf9c9a,0xd85b9c9c,0xdfe19c9c,0xd85b9c9e,
	0xdfe19c9e,0xd85b9ca0,0xdfe39ca0,0xd85b9ca2,0xdfe39ca2,0xd85b9ca4,0xdfe59ca4,0xd85b9ca6,
	0xdfe59ca6,0xd85b9ca8,0xdfe79ca8,0xd85b9caa,0xdfe79caa,0xd85b9cac,0xdfe99cac,0xd85b9cae,
	0xdfe99cae,0xd85b9cb0,0xdfeb9cb0,0xd85b9cb2,0xdfeb9cb2,0xd85b9cb4,0xdfed9cb4,0xd85b9cb6,
	0xdfed9cb6,0xd85b9cb8,0xdfef9cb8,0xd85b9cba,0xdfef9cba,0xd85b9cbc,0xdff19cbc,0xd85b9cbe,
	0xdff19cbe,0xd85b9cc0,0xdff39cc0,0xd85b9cc2,0xdff39cc2,0xd85b9cc4,0xdff59cc4,0xd85b9cc6,
	0xdff59cc6,0xd85b9cc8,0xdff79cc8,0xd85b9cca,0xdff79cca,0xd85b9ccc,0xdff99ccc,0xd85b9cce,
	0xdff99cce,0xd85b9cd0,0xdffb9cd0,0xd85b9cd2,0xdffb9cd2,0xd85b9cd4,0xdffd9cd4,0xd85b9cd6,
	0xdffd9cd6,0xd85b9cd8,0xdfff9cd8,0xd85b9cda,0xdfff9cda,0xd85d9cdc,0xdc019cdc,0xd85d9cde,
	0xdc019cde,0xd85d9ce0,0xdc039ce0,0xd85d9ce2,0xdc039ce2,0xd85d9ce4,0xdc059ce4,0xd85d9ce6,
	0xdc059ce6,0xd85d9ce8,0xdc079ce8,0xd85d9cea,0xdc079cea,0xd85d9cec,0xdc099cec,0xd85d9cee,
	0xdc099cee,0xd85d9cf0,0xdc0b9cf0,0xd85d9cf2,0xdc0b9cf2,0xd85d9cf4,0xdc0d9cf4,0xd85d9cf6,
	0xdc0d9cf6,0xd85d9cf8,0xdc0f9cf8,0xd85d9cfa,0xdc0f9cfa,0xd85d9cfc,0xdc119cfc,0xd85d9cfe,
	0xdc119cfe,0xd85d9d00,0xdc139d00,0xd85d9d02,0xdc139d02,0xd85d9d04,0xdc159d04,0xd85d9d06,
	0xdc159d06,0xd85d9d08,0xdc179d08,0xd85d9d0a,0xdc179d0a,0xd85d9d0c,0xdc199d0c,0xd85d9d0e,
	0xdc199d0e,0xd85d9d10,0xdc1b9d10,0xd85d9d12,0xdc1b9d12,0xd85d9d14,0xdc1d9d14,0xd85d9d16,
	0xdc1d9d16,0xd85d9d18,0xdc1f9d18,0xd85d9d1a,0xdc1f9d1a,0xd85d9d1c,0xdc219d1c,0xd85d9d1e,
	0xdc219d1e,0xd85d9d20,0xdc239d20,0xd85d9d22,0xdc239d22,0xd85d9d24,0xdc259d24,0xd85d9d26,
	0xdc259d26,0xd85d9d28,0xdc279d28,0xd85d9d2a,0xdc279d2a,0xd85d9d2c,0xdc299d2c,0xd85d9d2e,
	0xdc299d2e,0xd85d9d30,0xdc2b9d30,0xd85d9d32,0xdc2b9d32,0xd85d9d34,0xdc2d9d34,0xd85d9d36,
	0xdc2d9d36,0xd85d9d38,0xdc2f9d38,0xd85d9d3a,0xdc2f9d3a,0xd85d9d3c,0xdc319d3c,0xd85d9d3e,
	0xdc319d3e,0xd85d9d40,0xdc339d40,0xd85d9d42,0xdc339d42,0xd85d9d44,0xdc359d44,0xd85d9d46,
	0xdc359d46,0xd85d9d48,0xdc379d48,0xd85d9d4a,0xdc379d4a,0xd85d9d4c,0xdc399d4c,0xd85d9d4e,
	0xdc399d4e,0xd85d9d50,0xdc3b9d50,0xd85d9d52,0xdc3b9d52,0xd85d9d54,0xdc3d9d54,0xd85d9d56,
	0xdc3d9d56,0xd85d9d58,0xdc3f9d58,0xd85d9d5a,0xdc3f9d5a,0xd85d9d5c,0xdc419d5c,0xd85d9d5e,
	0xdc419d5e,0xd85d9d60,0xdc439d60,0xd85d9d62,0xdc439d62,0xd85d9d64,0xdc459d64,0xd85d9d66,
	0xdc459d66,0xd85d9d68,0xdc479d68,0xd85d9d6a,0xdc479d6a,0xd85d9d6c,0xdc499d6c,0xd85d9d6e,
	0xdc499d6e,0xd85d9d70,0xdc4b9d70,0xd85d9d72,0xdc4b9d72,0xd85d9d74,0xdc4d9d74,0xd85d9d76,
	0xdc4d9d76,0xd85d9d78,0xdc4f9d78,0xd85d9d7a,0xdc4f9d7a,0xd85d9d7c,0xdc519d7c,0xd85d9d7e,
	0xdc519d7e,0xd85d9d80,0xdc539d80,0xd85d9d82,0xdc539d82,0xd85d9d84,0xdc559d84,0xd85d9d86,
	0xdc559d86,0xd85d9d88,0xdc579d88,0xd85d9d8a,0xdc579d8a,0xd85d9d8c,0xdc599d8c,0xd85d9d8e,
	0xdc599d8e,0xd85d9d90,0xdc5b9d90,0xd85d9d92,0xdc5b9d92,0xd85d9d94,0xdc5d9d94,0xd85d9d96,
	0xdc5d9d96,0xd85d9d98,0xdc5f9d98,0xd85d9d9a,0xdc5f9d9a,0xd85d9d9c,0xdc619d9c,0xd85d9d9e,
	0xdc619d9e,0xd85d9da0,0xdc639da0,0xd85d9da2,0xdc639da2,0xd85d9da4,0xdc659da4,0xd85d9da6,
	0xdc659da6,0xd85d9da8,0xdc679da8,0xd85d9daa,0xdc679daa,0xd85d9dac,0xdc699dac,0xd85d9dae,
	0xdc699dae,0xd85d9db0,0xdc6b9db0,0xd85d9db2,0xdc6b9db2,0xd85d9db4,0xdc6d9db4,0xd85d9db6,
	0xdc6d9db6,0xd85d9db8,0xdc6f9db8,0xd85d9dba,0xdc6f9dba,0xd85d9dbc,0xdc719dbc,0xd85d9dbe,
	0xdc719dbe,0xd85d9dc0,0xdc739dc0,0xd85d9dc2,0xdc739dc2,0xd85d9dc4,0xdc759dc4,0xd85d9dc6,
	0xdc759dc6,0xd85d9dc8,0xdc779dc8,0xd85d9dca,0xdc779dca,0xd85d9dcc,0xdc799dcc,0xd85d9dce,
	0xdc799dce,0xd85d9dd0,0xdc7b9dd0,0xd85d9dd2,0xdc7b9dd2,0xd85d9dd4,0xdc7d9dd4,0xd85d9dd6,
	0xdc7d9dd6,0xd85d9dd8,0xdc7f9dd8,0xd85d9dda,0xdc7f9dda,0xd85d9ddc,0xdc819ddc,0xd85d9dde,
	0xdc819dde,0xd85d9de0,0xdc839de0,0xd85d9de2,0xdc839de2,0xd85d9de4,0xdc859de4,0xd85d9de6,
	0xdc859de6,0xd85d9de8,0xdc879de8,0xd85d9dea,0xdc879dea,0xd85d9dec,0xdc899dec,0xd85d9dee,
	0xdc899dee,0xd85d9df0,0xdc8b9df0,0xd85d9df2,0xdc8b9df2,0xd85d9df4,0xdc8d9df4,0xd85d9df6,
	0xdc8d9df6,0xd85d9df8,0xdc8f9df8,0xd85d9dfa,0xdc8f9dfa,0xd85d9dfc,0xdc919dfc,0xd85d9dfe,
	0xdc919dfe,0xd85d9e00,0xdc939e00,0xd85d9e02,0xdc939e02,0xd85d9e04,0xdc959e04,0xd85d9e06,
	0xdc959e06,0xd85d9e08,0xdc979e08,0xd85d9e0a,0xdc979e0a,0xd85d9e0c,0xdc999e0c,0xd85d9e0e,
	0xdc999e0e,0xd85d9e10,0xdc9b9e10,0xd85d9e12,0xdc9b9e12,0xd85d9e14,0xdc9d9e14,0xd85d9e16,
	0xdc9d9e16,0xd85d9e18,0xdc9f9e18,0xd85d9e1a,0xdc9f9e1a,0xd85d9e1c,0xdca19e1c,0xd85d9e1e,
	0xdca19e1e,0xd85d9e20,0xdca39e20,0xd85d9e22,0xdca39e22,0xd85d9e24,0xdca59e24,0xd85d9e26,
	0xdca59e26,0xd85d9e28,0xdca79e28,0xd85d9e2a,0xdca79e2a,0xd85d9e2c,0xdca99e2c,0xd85d9e2e,
	0xdca99e2e,0xd85d9e30,0xdcab9e30,0xd85d9e32,0xdcab9e32,0xd85d9e34,0xdcad9e34,0xd85d9e36,
	0xdcad9e36,0xd85d9e38,0xdcaf9e38,0xd85d9e3a,0xdcaf9e3a,0xd85d9e3c,0xdcb19e3c,0xd85d9e3e,
	0xdcb19e3e,0xd85d9e40,0xdcb39e40,0xd85d9e42,0xdcb39e42,0xd85d9e44,0xdcb59e44,0xd85d9e46,
	0xdcb59e46,0xd85d9e48,0xdcb79e48,0xd85d9e4a,0xdcb79e4a,0xd85d9e4c,0xdcb99e4c,0xd85d9e4e,
	0xdcb99e4e,0xd85d9e50,0xdcbb9e50,0xd85d9e52,0xdcbb9e52,0xd85d9e54,0xdcbd9e54,0xd85d9e56,
	0xdcbd9e56,0xd85d9e58,0xdcbf9e58,0xd85d9e5a,0xdcbf9e5a,0xd85d9e5c,0xdcc19e5c,0xd85d9e5e,
	0xdcc19e5e,0xd85d9e60,0xdcc39e60,0xd85d9e62,0xdcc39e62,0xd85d9e64,0xdcc59e64,0xd85d9e66,
	0xdcc59e66,0xd85d9e68,0xdcc79e68,0xd85d9e6a,0xdcc79e6a,0xd85d9e6c,0xdcc99e6c,0xd85d9e6e,
	0xdcc99e6e,0xd85d9e70,0xdccb9e70,0xd85d9e72,0xdccb9e72,0xd85d9e74,0xdccd9e74,0xd85d9e76,
	0xdccd9e76,0xd85d9e78,0xdccf9e78,0xd85d9e7a,0xdccf9e7a,0xd85d9e7c,0xdcd19e7c,0xd85d9e7e,
	0xdcd19e7e,0xd85d9e80,0xdcd39e80,0xd85d9e82,0xdcd39e82,0xd85d9e84,0xdcd59e84,0xd85d9e86,
	0xdcd59e86,0xd85d9e88,0xdcd79e88,0xd85d9e8a,0xdcd79e8a,0xd85d9e8c,0xdcd99e8c,0xd85d9e8e,
	0xdcd99e8e,0xd85d9e90,0xdcdb9e90,0xd85d9e92,0xdcdb9e92,0xd85d9e94,0xdcdd9e94,0xd85d9e96,
	0xdcdd9e96,0xd85d9e98,0xdcdf9e98,0xd85d9e9a,0xdcdf9e9a,0xd85d9e9c,0xdce19e9c,0xd85d9e9e,
	0xdce19e9e,0xd85d9ea0,0xdce39ea0,0xd85d9ea2,0xdce39ea2,0xd85d9ea4,0xdce59ea4,0xd85d9ea6,
	0xdce59ea6,0xd85d9ea8,0xdce79ea8,0xd85d9eaa,0xdce79eaa,0xd85d9eac,0xdce99eac,0xd85d9eae,
	0xdce99eae,0xd85d9eb0,0xdceb9eb0,0xd85d9eb2,0xdceb9eb2,0xd85d9eb4,0xdced9eb4,0xd85d9eb6,
	0xdced9eb6,0xd85d9eb8,0xdcef9eb8,0xd85d9eba,0xdcef9eba,0xd85d9ebc,0xdcf19ebc,0xd85d9ebe,
	0xdcf19ebe,0xd85d9ec0,0xdcf39ec0,0xd85d9ec2,0xdcf39ec2,0xd85d9ec4,0xdcf59ec4,0xd85d9ec6,
	0xdcf59ec6,0xd85d9ec8,0xdcf79ec8,0xd85d9eca,0xdcf79eca,0xd85d9ecc,0xdcf99ecc,0xd85d9ece,
	0xdcf99ece,0xd85d9ed0,0xdcfb9ed0,0xd85d9ed2,0xdcfb9ed2,0xd85d9ed4,0xdcfd9ed4,0xd85d9ed6,
	0xdcfd9ed6,0xd85d9ed8,0xdcff9ed8,0xd85d9eda,0xdcff9eda,0xd85d9edc,0xdd019edc,0xd85d9ede,
	0xdd019ede,0xd85d9ee0,0xdd039ee0,0xd85d9ee2,0xdd039ee2,0xd85d9ee4,0xdd059ee4,0xd85d9ee6,
	0xdd059ee6,0xd85d9ee8,0xdd079ee8,0xd85d9eea,0xdd079eea,0xd85d9eec,0xdd099eec,0xd85d9eee,
	0xdd099eee,0xd85d9ef0,0xdd0b9ef0,0xd85d9ef2,0xdd0b9ef2,0xd85d9ef4,0xdd0d9ef4,0xd85d9ef6,
	0xdd0d9ef6,0xd85d9ef8,0xdd0f9ef8,0xd85d9efa,0xdd0f9efa,0xd85d9efc,0xdd119efc,0xd85d9efe,
	0xdd119efe,0xd85d9f00,0xdd139f00,0xd85d9f02,0xdd139f02,0xd85d9f04,0xdd159f04,0xd85d9f06,
	0xdd159f06,0xd85d9f08,0xdd179f08,0xd85d9f0a,0xdd179f0a,0xd85d9f0c,0xdd199f0c,0xd85d9f0e,
	0xdd199f0e,0xd85d9f10,0xdd1b9f10,0xd85d9f12,0xdd1b9f12,0xd85d9f14,0xdd1d9f14,0xd85d9f16,
	0xdd1d9f16,0xd85d9f18,0xdd1f9f18,0xd85d9f1a,0xdd1f9f1a,0xd85d9f1c,0xdd219f1c,0xd85d9f1e,
	0xdd219f1e,0xd85d9f20,0xdd239f20,0xd85d9f22,0xdd239f22,0xd85d9f24,0xdd259f24,0xd85d9f26,
	0xdd259f26,0xd85d9f28,0xdd279f28,0xd85d9f2a,0xdd279f2a,0xd85d9f2c,0xdd299f2c,0xd85d9f2e,
	0xdd299f2e,0xd85d9f30,0xdd2b9f30,0xd85d9f32,0xdd2b9f32,0xd85d9f34,0xdd2d9f34,0xd85d9f36,
	0xdd2d9f36,0xd85d9f38,0xdd2f9f38,0xd85d9f3a,0xdd2f9f3a,0xd85d9f3c,0xdd319f3c,0xd85d9f3e,
	0xdd319f3e,0xd85d9f40,0xdd339f40,0xd85d9f42,0xdd339f42,0xd85d9f44,0xdd359f44,0xd85d9f46,
	0xdd359f46,0xd85d9f48,0xdd379f48,0xd85d9f4a,0xdd379f4a,0xd85d9f4c,0xdd399f4c,0xd85d9f4e,
	0xdd399f4e,0xd85d9f50,0xdd3b9f50,0xd85d9f52,0xdd3b9f52,0xd85d9f54,0xdd3d9f54,0xd85d9f56,
	0xdd3d9f56,0xd85d9f58,0xdd3f9f58,0xd85d9f5a,0xdd3f9f5a,0xd85d9f5c,0xdd419f5c,0xd85d9f5e,
	0xdd419f5e,0xd85d9f60,0xdd439f60,0xd85d9f62,0xdd439f62,0xd85d9f64,0xdd459f64,0xd85d9f66,
	0xdd459f66,0xd85d9f68,0xdd479f68,0xd85d9f6a,0xdd479f6a,0xd85d9f6c,0xdd499f6c,0xd85d9f6e,
	0xdd499f6e,0xd85d9f70,0xdd4b9f70,0xd85d9f72,0xdd4b9f72,0xd85d9f74,0xdd4d9f74,0xd85d9f76,
	0xdd4d9f76,0xd85d9f78,0xdd4f9f78,0xd85d9f7a,0xdd4f9f7a,0xd85d9f7c,0xdd519f7c,0xd85d9f7e,
	0xdd519f7e,0xd85d9f80,0xdd539f80,0xd85d9f82,0xdd539f82,0xd85d9f84,0xdd559f84,0xd85d9f86,
	0xdd559f86,0xd85d9f88,0xdd579f88,0xd85d9f8a,0xdd579f8a,0xd85d9f8c,0xdd599f8c,0xd85d9f8e,
	0xdd599f8e,0xd85d9f90,0xdd5b9f90,0xd85d9f92,0xdd5b9f92,0xd85d9f94,0xdd5d9f94,0xd85d9f96,
	0xdd5d9f96,0xd85d9f98,0xdd5f9f98,0xd85d9f9a,0xdd5f9f9a,0xd85d9f9c,0xdd619f9c,0xd85d9f9e,
	0xdd619f9e,0xd85d9fa0,0xdd639fa0,0xd85d9fa2,0xdd639fa2,0xd85d9fa4,0xdd659fa4,0xd85d9fa6,
	0xdd659fa6,0xd85d9fa8,0xdd679fa8,0xd85d9faa,0xdd679faa,0xd85d9fac,0xdd699fac,0xd85d9fae,
	0xdd699fae,0xd85d9fb0,0xdd6b9fb0,0xd85d9fb2,0xdd6b9fb2,0xd85d9fb4,0xdd6d9fb4,0xd85d9fb6,
	0xdd6d9fb6,0xd85d9fb8,0xdd6f9fb8,0xd85d9fba,0xdd6f9fba,0xd85d9fbc,0xdd719fbc,0xd85d9fbe,
	0xdd719fbe,0xd85d9fc0,0xdd739fc0,0xd85d9fc2,0xdd739fc2,0xd85d9fc4,0xdd759fc4,0xd85d9fc6,
	0xdd759fc6,0xd85d9fc8,0xdd779fc8,0xd85d9fca,0xdd779fca,0xd85d9fcc,0xdd799fcc,0xd85d9fce,
	0xdd799fce,0xd85d9fd0,0xdd7b9fd0,0xd85d9fd2,0xdd7b9fd2,0xd85d9fd4,0xdd7d9fd4,0xd85d9fd6,
	0xdd7d9fd6,0xd85d9fd8,0xdd7f9fd8,0xd85d9fda,0xdd7f9fda,0xd85d9fdc,0xdd819fdc,0xd85d9fde,
	0xdd819fde,0xd85d9fe0,0xdd839fe0,0xd85d9fe2,0xdd839fe2,0xd85d9fe4,0xdd859fe4,0xd85d9fe6,
	0xdd859fe6,0xd85d9fe8,0xdd879fe8,0xd85d9fea,0xdd879fea,0xd85d9fec,0xdd899fec,0xd85d9fee,
	0xdd899fee,0xd85d9ff0,0xdd8b9ff0,0xd85d9ff2,0xdd8b9ff2,0xd85d9ff4,0xdd8d9ff4,0xd85d9ff6,
	0xdd8d9ff6,0xd85d9ff8,0xdd8f9ff8,0xd85d9ffa,0xdd8f9ffa,0xd85d9ffc,0xdd919ffc,0xd85d9ffe,
	0xdd919ffe,0xd85da000,0xdd93a000,0xd85da002,0xdd93a002,0xd85da004,0xdd95a004,0xd85da006,
	0xdd95a006,0xd85da008,0xdd97a008,0xd85da00a,0xdd97a00a,0xd85da00c,0xdd99a00c,0xd85da00e,
	0xdd99a00e,0xd85da010,0xdd9ba010,0xd85da012,0xdd9ba012,0xd85da014,0xdd9da014,0xd85da016,
	0xdd9da016,0xd85da018,0xdd9fa018,0xd85da01a,0xdd9fa01a,0xd85da01c,0xdda1a01c,0xd85da01e,
	0xdda1a01e,0xd85da020,0xdda3a020,0xd85da022,0xdda3a022,0xd85da024,0xdda5a024,0xd85da026,
	0xdda5a026,0xd85da028,0xdda7a028,0xd85da02a,0xdda7a02a,0xd85da02c,0xdda9a02c,0xd85da02e,
	0xdda9a02e,0xd85da030,0xddaba030,0xd85da032,0xddaba032,0xd85da034,0xddada034,0xd85da036,
	0xddada036,0xd85da038,0xddafa038,0xd85da03a,0xddafa03a,0xd85da03c,0xddb1a03c,0xd85da03e,
	0xddb1a03e,0xd85da040,0xddb3a040,0xd85da042,0xddb3a042,0xd85da044,0xddb5a044,0xd85da046,
	0xddb5a046,0xd85da048,0xddb7a048,0xd85da04a,0xddb7a04a,0xd85da04c,0xddb9a04c,0xd85da04e,
	0xddb9a04e,0xd85da050,0xddbba050,0xd85da052,0xddbba052,0xd85da054,0xddbda054,0xd85da056,
	0xddbda056,0xd85da058,0xddbfa058,0xd85da05a,0xddbfa05a,0xd85da05c,0xddc1a05c,0xd85da05e,
	0xddc1a05e,0xd85da060,0xddc3a060,0xd85da062,0xddc3a062,0xd85da064,0xddc5a064,0xd85da066,
	0xddc5a066,0xd85da068,0xddc7a068,0xd85da06a,0xddc7a06a,0xd85da06c,0xddc9a06c,0xd85da06e,
	0xddc9a06e,0xd85da070,0xddcba070,0xd85da072,0xddcba072,0xd85da074,0xddcda074,0xd85da076,
	0xddcda076,0xd85da078,0xddcfa078,0xd85da07a,0xddcfa07a,0xd85da07c,0xddd1a07c,0xd85da07e,
	0xddd1a07e,0xd85da080,0xddd3a080,0xd85da082,0xddd3a082,0xd85da084,0xddd5a084,0xd85da086,
	0xddd5a086,0xd85da088,0xddd7a088,0xd85da08a,0xddd7a08a,0xd85da08c,0xddd9a08c,0xd85da08e,
	0xddd9a08e,0xd85da090,0xdddba090,0xd85da092,0xdddba092,0xd85da094,0xdddda094,0xd85da096,
	0xdddda096,0xd85da098,0xdddfa098,0xd85da09a,0xdddfa09a,0xd85da09c,0xdde1a09c,0xd85da09e,
	0xdde1a09e,0xd85da0a0,0xdde3a0a0,0xd85da0a2,0xdde3a0a2,0xd85da0a4,0xdde5a0a4,0xd85da0a6,
	0xdde5a0a6,0xd85da0a8,0xdde7a0a8,0xd85da0aa,0xdde7a0aa,0xd85da0ac,0xdde9a0ac,0xd85da0ae,
	0xdde9a0ae,0xd85da0b0,0xddeba0b0,0xd85da0b2,0xddeba0b2,0xd85da0b4,0xddeda0b4,0xd85da0b6,
	0xddeda0b6,0xd85da0b8,0xddefa0b8,0xd85da0ba,0xddefa0ba,0xd85da0bc,0xddf1a0bc,0xd85da0be,
	0xddf1a0be,0xd85da0c0,0xddf3a0c0,0xd85da0c2,0xddf3a0c2,0xd85da0c4,0xddf5a0c4,0xd85da0c6,
	0xddf5a0c6,0xd85da0c8,0xddf7a0c8,0xd85da0ca,0xddf7a0ca,0xd85da0cc,0xddf9a0cc,0xd85da0ce,
	0xddf9a0ce,0xd85da0d0,0xddfba0d0,0xd85da0d2,0xddfba0d2,0xd85da0d4,0xddfda0d4,0xd85da0d6,
	0xddfda0d6,0xd85da0d8,0xddffa0d8,0xd85da0da,0xddffa0da,0xd85da0dc,0xde01a0dc,0xd85da0de,
	0xde01a0de,0xd85da0e0,0xde03a0e0,0xd85da0e2,0xde03a0e2,0xd85da0e4,0xde05a0e4,0xd85da0e6,
	0xde05a0e6,0xd85da0e8,0xde07a0e8,0xd85da0ea,0xde07a0ea,0xd85da0ec,0xde09a0ec,0xd85da0ee,
	0xde09a0ee,0xd85da0f0,0xde0ba0f0,0xd85da0f2,0xde0ba0f2,0xd85da0f4,0xde0da0f4,0xd85da0f6,
	0xde0da0f6,0xd85da0f8,0xde0fa0f8,0xd85da0fa,0xde0fa0fa,0xd85da0fc,0xde11a0fc,0xd85da0fe,
	0xde11a0fe,0xd85da100,0xde13a100,0xd85da102,0xde13a102,0xd85da104,0xde15a104,0xd85da106,
	0xde15a106,0xd85da108,0xde17a108,0xd85da10a,0xde17a10a,0xd85da10c,0xde19a10c,0xd85da10e,
	0xde19a10e,0xd85da110,0xde1ba110,0xd85da112,0xde1ba112,0xd85da114,0xde1da114,0xd85da116,
	0xde1da116,0xd85da118,0xde1fa118,0xd85da11a,0xde1fa11a,0xd85da11c,0xde21a11c,0xd85da11e,
	0xde21a11e,0xd85da120,0xde23a120,0xd85da122,0xde23a122,0xd85da124,0xde25a124,0xd85da126,
	0xde25a126,0xd85da128,0xde27a128,0xd85da12a,0xde27a12a,0xd85da12c,0xde29a12c,0xd85da12e,
	0xde29a12e,0xd85da130,0xde2ba130,0xd85da132,0xde2ba132,0xd85da134,0xde2da134,0xd85da136,
	0xde2da136,0xd85da138,0xde2fa138,0xd85da13a,0xde2fa13a,0xd85da13c,0xde31a13c,0xd85da13e,
	0xde31a13e,0xd85da140,0xde33a140,0xd85da142,0xde33a142,0xd85da144,0xde35a144,0xd85da146,
	0xde35a146,0xd85da148,0xde37a148,0xd85da14a,0xde37a14a,0xd85da14c,0xde39a14c,0xd85da14e,
	0xde39a14e,0xd85da150,0xde3ba150,0xd85da152,0xde3ba152,0xd85da154,0xde3da154,0xd85da156,
	0xde3da156,0xd85da158,0xde3fa158,0xd85da15a,0xde3fa15a,0xd85da15c,0xde41a15c,0xd85da15e,
	0xde41a15e,0xd85da160,0xde43a160,0xd85da162,0xde43a162,0xd85da164,0xde45a164,0xd85da166,
	0xde45a166,0xd85da168,0xde47a168,0xd85da16a,0xde47a16a,0xd85da16c,0xde49a16c,0xd85da16e,
	0xde49a16e,0xd85da170,0xde4ba170,0xd85da172,0xde4ba172,0xd85da174,0xde4da174,0xd85da176,
	0xde4da176,0xd85da178,0xde4fa178,0xd85da17a,0xde4fa17a,0xd85da17c,0xde51a17c,0xd85da17e,
	0xde51a17e,0xd85da180,0xde53a180,0xd85da182,0xde53a182,0xd85da184,0xde55a184,0xd85da186,
	0xde55a186,0xd85da188,0xde57a188,0xd85da18a,0xde57a18a,0xd85da18c,0xde59a18c,0xd85da18e,
	0xde59a18e,0xd85da190,0xde5ba190,0xd85da192,0xde5ba192,0xd85da194,0xde5da194,0xd85da196,
	0xde5da196,0xd85da198,0xde5fa198,0xd85da19a,0xde5fa19a,0xd85da19c,0xde61a19c,0xd85da19e,
	0xde61a19e,0xd85da1a0,0xde63a1a0,0xd85da1a2,0xde63a1a2,0xd85da1a4,0xde65a1a4,0xd85da1a6,
	0xde65a1a6,0xd85da1a8,0xde67a1a8,0xd85da1aa,0xde67a1aa,0xd85da1ac,0xde69a1ac,0xd85da1ae,
	0xde69a1ae,0xd85da1b0,0xde6ba1b0,0xd85da1b2,0xde6ba1b2,0xd85da1b4,0xde6da1b4,0xd85da1b6,
	0xde6da1b6,0xd85da1b8,0xde6fa1b8,0xd85da1ba,0xde6fa1ba,0xd85da1bc,0xde71a1bc,0xd85da1be,
	0xde71a1be,0xd85da1c0,0xde73a1c0,0xd85da1c2,0xde73a1c2,0xd85da1c4,0xde75a1c4,0xd85da1c6,
	0xde75a1c6,0xd85da1c8,0xde77a1c8,0xd85da1ca,0xde77a1ca,0xd85da1cc,0xde79a1cc,0xd85da1ce,
	0xde79a1ce,0xd85da1d0,0xde7ba1d0,0xd85da1d2,0xde7ba1d2,0xd85da1d4,0xde7da1d4,0xd85da1d6,
	0xde7da1d6,0xd85da1d8,0xde7fa1d8,0xd85da1da,0xde7fa1da,0xd85da1dc,0xde81a1dc,0xd85da1de,
	0xde81a1de,0xd85da1e0,0xde83a1e0,0xd85da1e2,0xde83a1e2,0xd85da1e4,0xde85a1e4,0xd85da1e6,
	0xde85a1e6,0xd85da1e8,0xde87a1e8,0xd85da1ea,0xde87a1ea,0xd85da1ec,0xde89a1ec,0xd85da1ee,
	0xde89a1ee,0xd85da1f0,0xde8ba1f0,0xd85da1f2,0xde8ba1f2,0xd85da1f4,0xde8da1f4,0xd85da1f6,
	0xde8da1f6,0xd85da1f8,0xde8fa1f8,0xd85da1fa,0xde8fa1fa,0xd85da1fc,0xde91a1fc,0xd85da1fe,
	0xde91a1fe,0xd85da200,0xde93a200,0xd85da202,0xde93a202,0xd85da204,0xde95a204,0xd85da206,
	0xde95a206,0xd85da208,0xde97a208,0xd85da20a,0xde97a20a,0xd85da20c,0xde99a20c,0xd85da20e,
	0xde99a20e,0xd85da210,0xde9ba210,0xd85da212,0xde9ba212,0xd85da214,0xde9da214,0xd85da216,
	0xde9da216,0xd85da218,0xde9fa218,0xd85da21a,0xde9fa21a,0xd85da21c,0xdea1a21c,0xd85da21e,
	0xdea1a21e,0xd85da220,0xdea3a220,0xd85da222,0xdea3a222,0xd85da224,0xdea5a224,0xd85da226,
	0xdea5a226,0xd85da228,0xdea7a228,0xd85da22a,0xdea7a22a,0xd85da22c,0xdea9a22c,0xd85da22e,
	0xdea9a22e,0xd85da230,0xdeaba230,0xd85da232,0xdeaba232,0xd85da234,0xdeada234,0xd85da236,
	0xdeada236,0xd85da238,0xdeafa238,0xd85da23a,0xdeafa23a,0xd85da23c,0xdeb1a23c,0xd85da23e,
	0xdeb1a23e,0xd85da240,0xdeb3a240,0xd85da242,0xdeb3a242,0xd85da244,0xdeb5a244,0xd85da246,
	0xdeb5a246,0xd85da248,0xdeb7a248,0xd85da24a,0xdeb7a24a,0xd85da24c,0xdeb9a24c,0xd85da24e,
	0xdeb9a24e,0xd85da250,0xdebba250,0xd85da252,0xdebba252,0xd85da254,0xdebda254,0xd85da256,
	0xdebda256,0xd85da258,0xdebfa258,0xd85da25a,0xdebfa25a,0xd85da25c,0xdec1a25c,0xd85da25e,
	0xdec1a25e,0xd85da260,0xdec3a260,0xd85da262,0xdec3a262,0xd85da264,0xdec5a264,0xd85da266,
	0xdec5a266,0xd85da268,0xdec7a268,0xd85da26a,0xdec7a26a,0xd85da26c,0xdec9a26c,0xd85da26e,
	0xdec9a26e,0xd85da270,0xdecba270,0xd85da272,0xdecba272,0xd85da274,0xdecda274,0xd85da276,
	0xdecda276,0xd85da278,0xdecfa278,0xd85da27a,0xdecfa27a,0xd85da27c,0xded1a27c,0xd85da27e,
	0xded1a27e,0xd85da280,0xded3a280,0xd85da282,0xded3a282,0xd85da284,0xded5a284,0xd85da286,
	0xded5a286,0xd85da288,0xded7a288,0xd85da28a,0xded7a28a,0xd85da28c,0xded9a28c,0xd85da28e,
	0xded9a28e,0xd85da290,0xdedba290,0xd85da292,0xdedba292,0xd85da294,0xdedda294,0xd85da296,
	0xdedda296,0xd85da298,0xdedfa298,0xd85da29a,0xdedfa29a,0xd85da29c,0xdee1a29c,0xd85da29e,
	0xdee1a29e,0xd85da2a0,0xdee3a2a0,0xd85da2a2,0xdee3a2a2,0xd85da2a4,0xdee5a2a4,0xd85da2a6,
	0xdee5a2a6,0xd85da2a8,0xdee7a2a8,0xd85da2aa,0xdee7a2aa,0xd85da2ac,0xdee9a2ac,0xd85da2ae,
	0xdee9a2ae,0xd85da2b0,0xdeeba2b0,0xd85da2b2,0xdeeba2b2,0xd85da2b4,0xdeeda2b4,0xd85da2b6,
	0xdeeda2b6,0xd85da2b8,0xdeefa2b8,0xd85da2ba,0xdeefa2ba,0xd85da2bc,0xdef1a2bc,0xd85da2be,
	0xdef1a2be,0xd85da2c0,0xdef3a2c0,0xd85da2c2,0xdef3a2c2,0xd85da2c4,0xdef5a2c4,0xd85da2c6,
	0xdef5a2c6,0xd85da2c8,0xdef7a2c8,0xd85da2ca,0xdef7a2ca,0xd85da2cc,0xdef9a2cc,0xd85da2ce,
	0xdef9a2ce,0xd85da2d0,0xdefba2d0,0xd85da2d2,0xdefba2d2,0xd85da2d4,0xdefda2d4,0xd85da2d6,
	0xdefda2d6,0xd85da2d8,0xdeffa2d8,0xd85da2da,0xdeffa2da,0xd85da2dc,0xdf01a2dc,0xd85da2de,
	0xdf01a2de,0xd85da2e0,0xdf03a2e0,0xd85da2e2,0xdf03a2e2,0xd85da2e4,0xdf05a2e4,0xd85da2e6,
	0xdf05a2e6,0xd85da2e8,0xdf07a2e8,0xd85da2ea,0xdf07a2ea,0xd85da2ec,0xdf09a2ec,0xd85da2ee,
	0xdf09a2ee,0xd85da2f0,0xdf0ba2f0,0xd85da2f2,0xdf0ba2f2,0xd85da2f4,0xdf0da2f4,0xd85da2f6,
	0xdf0da2f6,0xd85da2f8,0xdf0fa2f8,0xd85da2fa,0xdf0fa2fa,0xd85da2fc,0xdf11a2fc,0xd85da2fe,
	0xdf11a2fe,0xd85da300,0xdf13a300,0xd85da302,0xdf13a302,0xd85da304,0xdf15a304,0xd85da306,
	0xdf15a306,0xd85da308,0xdf17a308,0xd85da30a,0xdf17a30a,0xd85da30c,0xdf19a30c,0xd85da30e,
	0xdf19a30e,0xd85da310,0xdf1ba310,0xd85da312,0xdf1ba312,0xd85da314,0xdf1da314,0xd85da316,
	0xdf1da316,0xd85da318,0xdf1fa318,0xd85da31a,0xdf1fa31a,0xd85da31c,0xdf21a31c,0xd85da31e,
	0xdf21a31e,0xd85da320,0xdf23a320,0xd85da322,0xdf23a322,0xd85da324,0xdf25a324,0xd85da326,
	0xdf25a326,0xd85da328,0xdf27a328,0xd85da32a,0xdf27a32a,0xd85da32c,0xdf29a32c,0xd85da32e,
	0xdf29a32e,0xd85da330,0xdf2ba330,0xd85da332,0xdf2ba332,0xd85da334,0xdf2da334,0xd85da336,
	0xdf2da336,0xd85da338,0xdf2fa338,0xd85da33a,0xdf2fa33a,0xd85da33c,0xdf31a33c,0xd85da33e,
	0xdf31a33e,0xd85da340,0xdf33a340,0xd85da342,0xdf33a342,0xd85da344,0xdf35a344,0xd85da346,
	0xdf35a346,0xd85da348,0xdf37a348,0xd85da34a,0xdf37a34a,0xd85da34c,0xdf39a34c,0xd85da34e,
	0xdf39a34e,0xd85da350,0xdf3ba350,0xd85da352,0xdf3ba352,0xd85da354,0xdf3da354,0xd85da356,
	0xdf3da356,0xd85da358,0xdf3fa358,0xd85da35a,0xdf3fa35a,0xd85da35c,0xdf41a35c,0xd85da35e,
	0xdf41a35e,0xd85da360,0xdf43a360,0xd85da362,0xdf43a362,0xd85da364,0xdf45a364,0xd85da366,
	0xdf45a366,0xd85da368,0xdf47a368,0xd85da36a,0xdf47a36a,0xd85da36c,0xdf49a36c,0xd85da36e,
	0xdf49a36e,0xd85da370,0xdf4ba370,0xd85da372,0xdf4ba372,0xd85da374,0xdf4da374,0xd85da376,
	0xdf4da376,0xd85da378,0xdf4fa378,0xd85da37a,0xdf4fa37a,0xd85da37c,0xdf51a37c,0xd85da37e,
	0xdf51a37e,0xd85da380,0xdf53a380,0xd85da382,0xdf53a382,0xd85da384,0xdf55a384,0xd85da386,
	0xdf55a386,0xd85da388,0xdf57a388,0xd85da38a,0xdf57a38a,0xd85da38c,0xdf59a38c,0xd85da38e,
	0xdf59a38e,0xd85da390,0xdf5ba390,0xd85da392,0xdf5ba392,0xd85da394,0xdf5da394,0xd85da396,
	0xdf5da396,0xd85da398,0xdf5fa398,0xd85da39a,0xdf5fa39a,0xd85da39c,0xdf61a39c,0xd85da39e,
	0xdf61a39e,0xd85da3a0,0xdf63a3a0,0xd85da3a2,0xdf63a3a2,0xd85da3a4,0xdf65a3a4,0xd85da3a6,
	0xdf65a3a6,0xd85da3a8,0xdf67a3a8,0xd85da3aa,0xdf67a3aa,0xd85da3ac,0xdf69a3ac,0xd85da3ae,
	0xdf69a3ae,0xd85da3b0,0xdf6ba3b0,0xd85da3b2,0xdf6ba3b2,0xd85da3b4,0xdf6da3b4,0xd85da3b6,
	0xdf6da3b6,0xd85da3b8,0xdf6fa3b8,0xd85da3ba,0xdf6fa3ba,0xd85da3bc,0xdf71a3bc,0xd85da3be,
	0xdf71a3be,0xd85da3c0,0xdf73a3c0,0xd85da3c2,0xdf73a3c2,0xd85da3c4,0xdf75a3c4,0xd85da3c6,
	0xdf75a3c6,0xd85da3c8,0xdf77a3c8,0xd85da3ca,0xdf77a3ca,0xd85da3cc,0xdf79a3cc,0xd85da3ce,
	0xdf79a3ce,0xd85da3d0,0xdf7ba3d0,0xd85da3d2,0xdf7ba3d2,0xd85da3d4,0xdf7da3d4,0xd85da3d6,
	0xdf7da3d6,0xd85da3d8,0xdf7fa3d8,0xd85da3da,0xdf7fa3da,0xd85da3dc,0xdf81a3dc,0xd85da3de,
	0xdf81a3de,0xd85da3e0,0xdf83a3e0,0xd85da3e2,0xdf83a3e2,0xd85da3e4,0xdf85a3e4,0xd85da3e6,
	0xdf85a3e6,0xd85da3e8,0xdf87a3e8,0xd85da3ea,0xdf87a3ea,0xd85da3ec,0xdf89a3ec,0xd85da3ee,
	0xdf89a3ee,0xd85da3f0,0xdf8ba3f0,0xd85da3f2,0xdf8ba3f2,0xd85da3f4,0xdf8da3f4,0xd85da3f6,
	0xdf8da3f6,0xd85da3f8,0xdf8fa3f8,0xd85da3fa,0xdf8fa3fa,0xd85da3fc,0xdf91a3fc,0xd85da3fe,
	0xdf91a3fe,0xd85da400,0xdf93a400,0xd85da402,0xdf93a402,0xd85da404,0xdf95a404,0xd85da406,
	0xdf95a406,0xd85da408,0xdf97a408,0xd85da40a,0xdf97a40a,0xd85da40c,0xdf99a40c,0xd85da40e,
	0xdf99a40e,0xd85da410,0xdf9ba410,0xd85da412,0xdf9ba412,0xd85da414,0xdf9da414,0xd85da416,
	0xdf9da416,0xd85da418,0xdf9fa418,0xd85da41a,0xdf9fa41a,0xd85da41c,0xdfa1a41c,0xd85da41e,
	0xdfa1a41e,0xd85da420,0xdfa3a420,0xd85da422,0xdfa3a422,0xd85da424,0xdfa5a424,0xd85da426,
	0xdfa5a426,0xd85da428,0xdfa7a428,0xd85da42a,0xdfa7a42a,0xd85da42c,0xdfa9a42c,0xd85da42e,
	0xdfa9a42e,0xd85da430,0xdfaba430,0xd85da432,0xdfaba432,0xd85da434,0xdfada434,0xd85da436,
	0xdfada436,0xd85da438,0xdfafa438,0xd85da43a,0xdfafa43a,0xd85da43c,0xdfb1a43c,0xd85da43e,
	0xdfb1a43e,0xd85da440,0xdfb3a440,0xd85da442,0xdfb3a442,0xd85da444,0xdfb5a444,0xd85da446,
	0xdfb5a446,0xd85da448,0xdfb7a448,0xd85da44a,0xdfb7a44a,0xd85da44c,0xdfb9a44c,0xd85da44e,
	0xdfb9a44e,0xd85da450,0xdfbba450,0xd85da452,0xdfbba452,0xd85da454,0xdfbda454,0xd85da456,
	0xdfbda456,0xd85da458,0xdfbfa458,0xd85da45a,0xdfbfa45a,0xd85da45c,0xdfc1a45c,0xd85da45e,
	0xdfc1a45e,0xd85da460,0xdfc3a460,0xd85da462,0xdfc3a462,0xd85da464,0xdfc5a464,0xd85da466,
	0xdfc5a466,0xd85da468,0xdfc7a468,0xd85da46a,0xdfc7a46a,0xd85da46c,0xdfc9a46c,0xd85da46e,
	0xdfc9a46e,0xd85da470,0xdfcba470,0xd85da472,0xdfcba472,0xd85da474,0xdfcda474,0xd85da476,
	0xdfcda476,0xd85da478,0xdfcfa478,0xd85da47a,0xdfcfa47a,0xd85da47c,0xdfd1a47c,0xd85da47e,
	0xdfd1a47e,0xd85da480,0xdfd3a480,0xd85da482,0xdfd3a482,0xd85da484,0xdfd5a484,0xd85da486,
	0xdfd5a486,0xd85da488,0xdfd7a488,0xd85da48a,0xdfd7a48a,0xd85da48c,0xdfd9a48c,0xd85da48e,
	0xdfd9a48e,0xd85da490,0xdfdba490,0xd85da492,0xdfdba492,0xd85da494,0xdfdda494,0xd85da496,
	0xdfdda496,0xd85da498,0xdfdfa498,0xd85da49a,0xdfdfa49a,0xd85da49c,0xdfe1a49c,0xd85da49e,
	0xdfe1a49e,0xd85da4a0,0xdfe3a4a0,0xd85da4a2,0xdfe3a4a2,0xd85da4a4,0xdfe5a4a4,0xd85da4a6,
	0xdfe5a4a6,0xd85da4a8,0xdfe7a4a8,0xd85da4aa,0xdfe7a4aa,0xd85da4ac,0xdfe9a4ac,0xd85da4ae,
	0xdfe9a4ae,0xd85da4b0,0xdfeba4b0,0xd85da4b2,0xdfeba4b2,0xd85da4b4,0xdfeda4b4,0xd85da4b6,
	0xdfeda4b6,0xd85da4b8,0xdfefa4b8,0xd85da4ba,0xdfefa4ba,0xd85da4bc,0xdff1a4bc,0xd85da4be,
	0xdff1a4be,0xd85da4c0,0xdff3a4c0,0xd85da4c2,0xdff3a4c2,0xd85da4c4,0xdff5a4c4,0xd85da4c6,
	0xdff5a4c6,0xd85da4c8,0xdff7a4c8,0xd85da4ca,0xdff7a4ca,0xd85da4cc,0xdff9a4cc,0xd85da4ce,
	0xdff9a4ce,0xd85da4d0,0xdffba4d0,0xd85da4d2,0xdffba4d2,0xd85da4d4,0xdffda4d4,0xd85da4d6,
	0xdffda4d6,0xd85da4d8,0xdfffa4d8,0xd85da4da,0xdfffa4da,0xd85da4dc,0xdc01a4dc,0xd85da4de,
	0xdc01a4de,0xd85da4e0,0xdc03a4e0,0xd85da4e2,0xdc03a4e2,0xd85da4e4,0xdc05a4e4,0xd85da4e6,
	0xdc05a4e6,0xd85da4e8,0xdc07a4e8,0xd85da4ea,0xdc07a4ea,0xd85da4ec,0xdc09a4ec,0xd85da4ee,
	0xdc09a4ee,0xd85da4f0,0xdc0ba4f0,0xd85da4f2,0xdc0ba4f2,0xd85da4f4,0xdc0da4f4,0xd85da4f6,
	0xdc0da4f6,0xd85da4f8,0xdc0fa4f8,0xd85da4fa,0xdc0fa4fa,0xd85da4fc,0xdc11a4fc,0xd85da4fe,
	0xdc11a4fe,0xd85da500,0xdc13a500,0xd85da502,0xdc13a502,0xd85da504,0xdc15a504,0xd85da506,
	0xdc15a506,0xd85da508,0xdc17a508,0xd85da50a,0xdc17a50a,0xd85da50c,0xdc19a50c,0xd85da50e,
	0xdc19a50e,0xd85da510,0xdc1ba510,0xd85da512,0xdc1ba512,0xd85da514,0xdc1da514,0xd85da516,
	0xdc1da516,0xd85da518,0xdc1fa518,0xd85da51a,0xdc1fa51a,0xd85da51c,0xdc21a51c,0xd85da51e,
	0xdc21a51e,0xd85da520,0xdc23a520,0xd85da522,0xdc23a522,0xd85da524,0xdc25a524,0xd85da526,
	0xdc25a526,0xd85da528,0xdc27a528,0xd85da52a,0xdc27a52a,0xd85da52c,0xdc29a52c,0xd85da52e,
	0xdc29a52e,0xd85da530,0xdc2ba530,0xd85da532,0xdc2ba532,0xd85da534,0xdc2da534,0xd85da536,
	0xdc2da536,0xd85da538,0xdc2fa538,0xd85da53a,0xdc2fa53a,0xd85da53c,0xdc31a53c,0xd85da53e,
	0xdc31a53e,0xd85da540,0xdc33a540,0xd85da542,0xdc33a542,0xd85da544,0xdc35a544,0xd85da546,
	0xdc35a546,0xd85da548,0xdc37a548,0xd85da54a,0xdc37a54a,0xd85da54c,0xdc39a54c,0xd85da54e,
	0xdc39a54e,0xd85da550,0xdc3ba550,0xd85da552,0xdc3ba552,0xd85da554,0xdc3da554,0xd85da556,
	0xdc3da556,0xd85da558,0xdc3fa558,0xd85da55a,0xdc3fa55a,0xd85da55c,0xdc41a55c,0xd85da55e,
	0xdc41a55e,0xd85da560,0xdc43a560,0xd85da562,0xdc43a562,0xd85da564,0xdc45a564,0xd85da566,
	0xdc45a566,0xd85da568,0xdc47a568,0xd85da56a,0xdc47a56a,0xd85da56c,0xdc49a56c,0xd85da56e,
	0xdc49a56e,0xd85da570,0xdc4ba570,0xd85da572,0xdc4ba572,0xd85da574,0xdc4da574,0xd85da576,
	0xdc4da576,0xd85da578,0xdc4fa578,0xd85da57a,0xdc4fa57a,0xd85da57c,0xdc51a57c,0xd85da57e,
	0xdc51a57e,0xd85da580,0xdc53a580,0xd85da582,0xdc53a582,0xd85da584,0xdc55a584,0xd85da586,
	0xdc55a586,0xd85da588,0xdc57a588,0xd85da58a,0xdc57a58a,0xd85da58c,0xdc59a58c,0xd85da58e,
	0xdc59a58e,0xd85da590,0xdc5ba590,0xd85da592,0xdc5ba592,0xd85da594,0xdc5da594,0xd85da596,
	0xdc5da596,0xd85da598,0xdc5fa598,0xd85da59a,0xdc5fa59a,0xd85da59c,0xdc61a59c,0xd85da59e,
	0xdc61a59e,0xd85da5a0,0xdc63a5a0,0xd85da5a2,0xdc63a5a2,0xd85da5a4,0xdc65a5a4,0xd85da5a6,
	0xdc65a5a6,0xd85da5a8,0xdc67a5a8,0xd85da5aa,0xdc67a5aa,0xd85da5ac,0xdc69a5ac,0xd85da5ae,
	0xdc69a5ae,0xd85da5b0,0xdc6ba5b0,0xd85da5b2,0xdc6ba5b2,0xd85da5b4,0xdc6da5b4,0xd85da5b6,
	0xdc6da5b6,0xd85da5b8,0xdc6fa5b8,0xd85da5ba,0xdc6fa5ba,0xd85da5bc,0xdc71a5bc,0xd85da5be,
	0xdc71a5be,0xd85da5c0,0xdc73a5c0,0xd85da5c2,0xdc73a5c2,0xd85da5c4,0xdc75a5c4,0xd85da5c6,
	0xdc75a5c6,0xd85da5c8,0xdc77a5c8,0xd85da5ca,0xdc77a5ca,0xd85da5cc,0xdc79a5cc,0xd85da5ce,
	0xdc79a5ce,0xd85da5d0,0xdc7ba5d0,0xd85da5d2,0xdc7ba5d2,0xd85da5d4,0xdc7da5d4,0xd85da5d6,
	0xdc7da5d6,0xd85da5d8,0xdc7fa5d8,0xd85da5da,0xdc7fa5da,0xd85da5dc,0xdc81a5dc,0xd85da5de,
	0xdc81a5de,0xd85da5e0,0xdc83a5e0,0xd85da5e2,0xdc83a5e2,0xd85da5e4,0xdc85a5e4,0xd85da5e6,
	0xdc85a5e6,0xd85da5e8,0xdc87a5e8,0xd85da5ea,0xdc87a5ea,0xd85da5ec,0xdc89a5ec,0xd85da5ee,
	0xdc89a5ee,0xd85da5f0,0xdc8ba5f0,0xd85da5f2,0xdc8ba5f2,0xd85da5f4,0xdc8da5f4,0xd85da5f6,
	0xdc8da5f6,0xd85da5f8,0xdc8fa5f8,0xd85da5fa,0xdc8fa5fa,0xd85da5fc,0xdc91a5fc,0xd85da5fe,
	0xdc91a5fe,0xd85da600,0xdc93a600,0xd85da602,0xdc93a602,0xd85da604,0xdc95a604,0xd85da606,
	0xdc95a606,0xd85da608,0xdc97a608,0xd85da60a,0xdc97a60a,0xd85da60c,0xdc99a60c,0xd85da60e,
	0xdc99a60e,0xd85da610,0xdc9ba610,0xd85da612,0xdc9ba612,0xd85da614,0xdc9da614,0xd85da616,
	0xdc9da616,0xd85da618,0xdc9fa618,0xd85da61a,0xdc9fa61a,0xd85da61c,0xdca1a61c,0xd85da61e,
	0xdca1a61e,0xd85da620,0xdca3a620,0xd85da622,0xdca3a622,0xd85da624,0xdca5a624,0xd85da626,
	0xdca5a626,0xd85da628,0xdca7a628,0xd85da62a,0xdca7a62a,0xd85da62c,0xdca9a62c,0xd85da62e,
	0xdca9a62e,0xd85da630,0xdcaba630,0xd85da632,0xdcaba632,0xd85da634,0xdcada634,0xd85da636,
	0xdcada636,0xd85da638,0xdcafa638,0xd85da63a,0xdcafa63a,0xd85da63c,0xdcb1a63c,0xd85da63e,
	0xdcb1a63e,0xd85da640,0xdcb3a640,0xd85da642,0xdcb3a642,0xd85da644,0xdcb5a644,0xd85da646,
	0xdcb5a646,0xd85da648,0xdcb7a648,0xd85da64a,0xdcb7a64a,0xd85da64c,0xdcb9a64c,0xd85da64e,
	0xdcb9a64e,0xd85da650,0xdcbba650,0xd85da652,0xdcbba652,0xd85da654,0xdcbda654,0xd85da656,
	0xdcbda656,0xd85da658,0xdcbfa658,0xd85da65a,0xdcbfa65a,0xd85da65c,0xdcc1a65c,0xd85da65e,
	0xdcc1a65e,0xd85da660,0xdcc3a660,0xd85da662,0xdcc3a662,0xd85da664,0xdcc5a664,0xd85da666,
	0xdcc5a666,0xd85da668,0xdcc7a668,0xd85da66a,0xdcc7a66a,0xd85da66c,0xdcc9a66c,0xd85da66e,
	0xdcc9a66e,0xd85da670,0xdccba670,0xd85da672,0xdccba672,0xd85da674,0xdccda674,0xd85da676,
	0xdccda676,0xd85da678,0xdccfa678,0xd85da67a,0xdccfa67a,0xd85da67c,0xdcd1a67c,0xd85da67e,
	0xdcd1a67e,0xd85da680,0xdcd3a680,0xd85da682,0xdcd3a682,0xd85da684,0xdcd5a684,0xd85da686,
	0xdcd5a686,0xd85da688,0xdcd7a688,0xd85da68a,0xdcd7a68a,0xd85da68c,0xdcd9a68c,0xd85da68e,
	0xdcd9a68e,0xd85da690,0xdcdba690,0xd85da692,0xdcdba692,0xd85da694,0xdcdda694,0xd85da696,
	0xdcdda696,0xd85da698,0xdcdfa698,0xd85da69a,0xdcdfa69a,0xd85da69c,0xdce1a69c,0xd85da69e,
	0xdce1a69e,0xd85da6a0,0xdce3a6a0,0xd85da6a2,0xdce3a6a2,0xd85da6a4,0xdce5a6a4,0xd85da6a6,
	0xdce5a6a6,0xd85da6a8,0xdce7a6a8,0xd85da6aa,0xdce7a6aa,0xd85da6ac,0xdce9a6ac,0xd85da6ae,
	0xdce9a6ae,0xd85da6b0,0xdceba6b0,0xd85da6b2,0xdceba6b2,0xd85da6b4,0xdceda6b4,0xd85da6b6,
	0xdceda6b6,0xd85da6b8,0xdcefa6b8,0xd85da6ba,0xdcefa6ba,0xd85da6bc,0xdcf1a6bc,0xd85da6be,
	0xdcf1a6be,0xd85da6c0,0xdcf3a6c0,0xd85da6c2,0xdcf3a6c2,0xd85da6c4,0xdcf5a6c4,0xd85da6c6,
	0xdcf5a6c6,0xd85da6c8,0xdcf7a6c8,0xd85da6ca,0xdcf7a6ca,0xd85da6cc,0xdcf9a6cc,0xd85da6ce,
	0xdcf9a6ce,0xd85da6d0,0xdcfba6d0,0xd85da6d2,0xdcfba6d2,0xd85da6d4,0xdcfda6d4,0xd85da6d6,
	0xdcfda6d6,0xd85da6d8,0xdcffa6d8,0xd85da6da,0xdcffa6da,0xd85da6dc,0xdd01a6dc,0xd85da6de,
	0xdd01a6de,0xd85da6e0,0xdd03a6e0,0xd85da6e2,0xdd03a6e2,0xd85da6e4,0xdd05a6e4,0xd85da6e6,
	0xdd05a6e6,0xd85da6e8,0xdd07a6e8,0xd85da6ea,0xdd07a6ea,0xd85da6ec,0xdd09a6ec,0xd85da6ee,
	0xdd09a6ee,0xd85da6f0,0xdd0ba6f0,0xd85da6f2,0xdd0ba6f2,0xd85da6f4,0xdd0da6f4,0xd85da6f6,
	0xdd0da6f6,0xd85da6f8,0xdd0fa6f8,0xd85da6fa,0xdd0fa6fa,0xd85da6fc,0xdd11a6fc,0xd85da6fe,
	0xdd11a6fe,0xd85da700,0xdd13a700,0xd85da702,0xdd13a702,0xd85da704,0xdd15a704,0xd85da706,
	0xdd15a706,0xd85da708,0xdd17a708,0xd85da70a,0xdd17a70a,0xd85da70c,0xdd19a70c,0xd85da70e,
	0xdd19a70e,0xd85da710,0xdd1ba710,0xd85da712,0xdd1ba712,0xd85da714,0xdd1da714,0xd85da716,
	0xdd1da716,0xd85da718,0xdd1fa718,0xd85da71a,0xdd1fa71a,0xd85da71c,0xdd21a71c,0xd85da71e,
	0xdd21a71e,0xd85da720,0xdd23a720,0xd85da722,0xdd23a722,0xd85da724,0xdd25a724,0xd85da726,
	0xdd25a726,0xd85da728,0xdd27a728,0xd85da72a,0xdd27a72a,0xd85da72c,0xdd29a72c,0xd85da72e,
	0xdd29a72e,0xd85da730,0xdd2ba730,0xd85da732,0xdd2ba732,0xd85da734,0xdd2da734,0xd85da736,
	0xdd2da736,0xd85da738,0xdd2fa738,0xd85da73a,0xdd2fa73a,0xd85da73c,0xdd31a73c,0xd85da73e,
	0xdd31a73e,0xd85da740,0xdd33a740,0xd85da742,0xdd33a742,0xd85da744,0xdd35a744,0xd85da746,
	0xdd35a746,0xd85da748,0xdd37a748,0xd85da74a,0xdd37a74a,0xd85da74c,0xdd39a74c,0xd85da74e,
	0xdd39a74e,0xd85da750,0xdd3ba750,0xd85da752,0xdd3ba752,0xd85da754,0xdd3da754,0xd85da756,
	0xdd3da756,0xd85da758,0xdd3fa758,0xd85da75a,0xdd3fa75a,0xd85da75c,0xdd41a75c,0xd85da75e,
	0xdd41a75e,0xd85da760,0xdd43a760,0xd85da762,0xdd43a762,0xd85da764,0xdd45a764,0xd85da766,
	0xdd45a766,0xd85da768,0xdd47a768,0xd85da76a,0xdd47a76a,0xd85da76c,0xdd49a76c,0xd85da76e,
	0xdd49a76e,0xd85da770,0xdd4ba770,0xd85da772,0xdd4ba772,0xd85da774,0xdd4da774,0xd85da776,
	0xdd4da776,0xd85da778,0xdd4fa778,0xd85da77a,0xdd4fa77a,0xd85da77c,0xdd51a77c,0xd85da77e,
	0xdd51a77e,0xd85da780,0xdd53a780,0xd85da782,0xdd53a782,0xd85da784,0xdd55a784,0xd85da786,
	0xdd55a786,0xd85da788,0xdd57a788,0xd85da78a,0xdd57a78a,0xd85da78c,0xdd59a78c,0xd85da78e,
	0xdd59a78e,0xd85da790,0xdd5ba790,0xd85da792,0xdd5ba792,0xd85da794,0xdd5da794,0xd85da796,
	0xdd5da796,0xd85da798,0xdd5fa798,0xd85da79a,0xdd5fa79a,0xd85da79c,0xdd61a79c,0xd85da79e,
	0xdd61a79e,0xd85da7a0,0xdd63a7a0,0xd85da7a2,0xdd63a7a2,0xd85da7a4,0xdd65a7a4,0xd85da7a6,
	0xdd65a7a6,0xd85da7a8,0xdd67a7a8,0xd85da7aa,0xdd67a7aa,0xd85da7ac,0xdd69a7ac,0xd85da7ae,
	0xdd69a7ae,0xd85da7b0,0xdd6ba7b0,0xd85da7b2,0xdd6ba7b2,0xd85da7b4,0xdd6da7b4,0xd85da7b6,
	0xdd6da7b6,0xd85da7b8,0xdd6fa7b8,0xd85da7ba,0xdd6fa7ba,0xd85da7bc,0xdd71a7bc,0xd85da7be,
	0xdd71a7be,0xd85da7c0,0xdd73a7c0,0xd85da7c2,0xdd73a7c2,0xd85da7c4,0xdd75a7c4,0xd85da7c6,
	0xdd75a7c6,0xd85da7c8,0xdd77a7c8,0xd85da7ca,0xdd77a7ca,0xd85da7cc,0xdd79a7cc,0xd85da7ce,
	0xdd79a7ce,0xd85da7d0,0xdd7ba7d0,0xd85da7d2,0xdd7ba7d2,0xd85da7d4,0xdd7da7d4,0xd85da7d6,
	0xdd7da7d6,0xd85da7d8,0xdd7fa7d8,0xd85da7da,0xdd7fa7da,0xd85da7dc,0xdd81a7dc,0xd85da7de,
	0xdd81a7de,0xd85da7e0,0xdd83a7e0,0xd85da7e2,0xdd83a7e2,0xd85da7e4,0xdd85a7e4,0xd85da7e6,
	0xdd85a7e6,0xd85da7e8,0xdd87a7e8,0xd85da7ea,0xdd87a7ea,0xd85da7ec,0xdd89a7ec,0xd85da7ee,
	0xdd89a7ee,0xd85da7f0,0xdd8ba7f0,0xd85da7f2,0xdd8ba7f2,0xd85da7f4,0xdd8da7f4,0xd85da7f6,
	0xdd8da7f6,0xd85da7f8,0xdd8fa7f8,0xd85da7fa,0xdd8fa7fa,0xd85da7fc,0xdd91a7fc,0xd85da7fe,
	0xdd91a7fe,0xd85da800,0xdd93a800,0xd85da802,0xdd93a802,0xd85da804,0xdd95a804,0xd85da806,
	0xdd95a806,0xd85da808,0xdd97a808,0xd85da80a,0xdd97a80a,0xd85da80c,0xdd99a80c,0xd85da80e,
	0xdd99a80e,0xd85da810,0xdd9ba810,0xd85da812,0xdd9ba812,0xd85da814,0xdd9da814,0xd85da816,
	0xdd9da816,0xd85da818,0xdd9fa818,0xd85da81a,0xdd9fa81a,0xd85da81c,0xdda1a81c,0xd85da81e,
	0xdda1a81e,0xd85da820,0xdda3a820,0xd85da822,0xdda3a822,0xd85da824,0xdda5a824,0xd85da826,
	0xdda5a826,0xd85da828,0xdda7a828,0xd85da82a,0xdda7a82a,0xd85da82c,0xdda9a82c,0xd85da82e,
	0xdda9a82e,0xd85da830,0xddaba830,0xd85da832,0xddaba832,0xd85da834,0xddada834,0xd85da836,
	0xddada836,0xd85da838,0xddafa838,0xd85da83a,0xddafa83a,0xd85da83c,0xddb1a83c,0xd85da83e,
	0xddb1a83e,0xd85da840,0xddb3a840,0xd85da842,0xddb3a842,0xd85da844,0xddb5a844,0xd85da846,
	0xddb5a846,0xd85da848,0xddb7a848,0xd85da84a,0xddb7a84a,0xd85da84c,0xddb9a84c,0xd85da84e,
	0xddb9a84e,0xd85da850,0xddbba850,0xd85da852,0xddbba852,0xd85da854,0xddbda854,0xd85da856,
	0xddbda856,0xd85da858,0xddbfa858,0xd85da85a,0xddbfa85a,0xd85da85c,0xddc1a85c,0xd85da85e,
	0xddc1a85e,0xd85da860,0xddc3a860,0xd85da862,0xddc3a862,0xd85da864,0xddc5a864,0xd85da866,
	0xddc5a866,0xd85da868,0xddc7a868,0xd85da86a,0xddc7a86a,0xd85da86c,0xddc9a86c,0xd85da86e,
	0xddc9a86e,0xd85da870,0xddcba870,0xd85da872,0xddcba872,0xd85da874,0xddcda874,0xd85da876,
	0xddcda876,0xd85da878,0xddcfa878,0xd85da87a,0xddcfa87a,0xd85da87c,0xddd1a87c,0xd85da87e,
	0xddd1a87e,0xd85da880,0xddd3a880,0xd85da882,0xddd3a882,0xd85da884,0xddd5a884,0xd85da886,
	0xddd5a886,0xd85da888,0xddd7a888,0xd85da88a,0xddd7a88a,0xd85da88c,0xddd9a88c,0xd85da88e,
	0xddd9a88e,0xd85da890,0xdddba890,0xd85da892,0xdddba892,0xd85da894,0xdddda894,0xd85da896,
	0xdddda896,0xd85da898,0xdddfa898,0xd85da89a,0xdddfa89a,0xd85da89c,0xdde1a89c,0xd85da89e,
	0xdde1a89e,0xd85da8a0,0xdde3a8a0,0xd85da8a2,0xdde3a8a2,0xd85da8a4,0xdde5a8a4,0xd85da8a6,
	0xdde5a8a6,0xd85da8a8,0xdde7a8a8,0xd85da8aa,0xdde7a8aa,0xd85da8ac,0xdde9a8ac,0xd85da8ae,
	0xdde9a8ae,0xd85da8b0,0xddeba8b0,0xd85da8b2,0xddeba8b2,0xd85da8b4,0xddeda8b4,0xd85da8b6,
	0xddeda8b6,0xd85da8b8,0xddefa8b8,0xd85da8ba,0xddefa8ba,0xd85da8bc,0xddf1a8bc,0xd85da8be,
	0xddf1a8be,0xd85da8c0,0xddf3a8c0,0xd85da8c2,0xddf3a8c2,0xd85da8c4,0xddf5a8c4,0xd85da8c6,
	0xddf5a8c6,0xd85da8c8,0xddf7a8c8,0xd85da8ca,0xddf7a8ca,0xd85da8cc,0xddf9a8cc,0xd85da8ce,
	0xddf9a8ce,0xd85da8d0,0xddfba8d0,0xd85da8d2,0xddfba8d2,0xd85da8d4,0xddfda8d4,0xd85da8d6,
	0xddfda8d6,0xd85da8d8,0xddffa8d8,0xd85da8da,0xddffa8da,0xd85da8dc,0xde01a8dc,0xd85da8de,
	0xde01a8de,0xd85da8e0,0xde03a8e0,0xd85da8e2,0xde03a8e2,0xd85da8e4,0xde05a8e4,0xd85da8e6,
	0xde05a8e6,0xd85da8e8,0xde07a8e8,0xd85da8ea,0xde07a8ea,0xd85da8ec,0xde09a8ec,0xd85da8ee,
	0xde09a8ee,0xd85da8f0,0xde0ba8f0,0xd85da8f2,0xde0ba8f2,0xd85da8f4,0xde0da8f4,0xd85da8f6,
	0xde0da8f6,0xd85da8f8,0xde0fa8f8,0xd85da8fa,0xde0fa8fa,0xd85da8fc,0xde11a8fc,0xd85da8fe,
	0xde11a8fe,0xd85da900,0xde13a900,0xd85da902,0xde13a902,0xd85da904,0xde15a904,0xd85da906,
	0xde15a906,0xd85da908,0xde17a908,0xd85da90a,0xde17a90a,0xd85da90c,0xde19a90c,0xd85da90e,
	0xde19a90e,0xd85da910,0xde1ba910,0xd85da912,0xde1ba912,0xd85da914,0xde1da914,0xd85da916,
	0xde1da916,0xd85da918,0xde1fa918,0xd85da91a,0xde1fa91a,0xd85da91c,0xde21a91c,0xd85da91e,
	0xde21a91e,0xd85da920,0xde23a920,0xd85da922,0xde23a922,0xd85da924,0xde25a924,0xd85da926,
	0xde25a926,0xd85da928,0xde27a928,0xd85da92a,0xde27a92a,0xd85da92c,0xde29a92c,0xd85da92e,
	0xde29a92e,0xd85da930,0xde2ba930,0xd85da932,0xde2ba932,0xd85da934,0xde2da934,0xd85da936,
	0xde2da936,0xd85da938,0xde2fa938,0xd85da93a,0xde2fa93a,0xd85da93c,0xde31a93c,0xd85da93e,
	0xde31a93e,0xd85da940,0xde33a940,0xd85da942,0xde33a942,0xd85da944,0xde35a944,0xd85da946,
	0xde35a946,0xd85da948,0xde37a948,0xd85da94a,0xde37a94a,0xd85da94c,0xde39a94c,0xd85da94e,
	0xde39a94e,0xd85da950,0xde3ba950,0xd85da952,0xde3ba952,0xd85da954,0xde3da954,0xd85da956,
	0xde3da956,0xd85da958,0xde3fa958,0xd85da95a,0xde3fa95a,0xd85da95c,0xde41a95c,0xd85da95e,
	0xde41a95e,0xd85da960,0xde43a960,0xd85da962,0xde43a962,0xd85da964,0xde45a964,0xd85da966,
	0xde45a966,0xd85da968,0xde47a968,0xd85da96a,0xde47a96a,0xd85da96c,0xde49a96c,0xd85da96e,
	0xde49a96e,0xd85da970,0xde4ba970,0xd85da972,0xde4ba972,0xd85da974,0xde4da974,0xd85da976,
	0xde4da976,0xd85da978,0xde4fa978,0xd85da97a,0xde4fa97a,0xd85da97c,0xde51a97c,0xd85da97e,
	0xde51a97e,0xd85da980,0xde53a980,0xd85da982,0xde53a982,0xd85da984,0xde55a984,0xd85da986,
	0xde55a986,0xd85da988,0xde57a988,0xd85da98a,0xde57a98a,0xd85da98c,0xde59a98c,0xd85da98e,
	0xde59a98e,0xd85da990,0xde5ba990,0xd85da992,0xde5ba992,0xd85da994,0xde5da994,0xd85da996,
	0xde5da996,0xd85da998,0xde5fa998,0xd85da99a,0xde5fa99a,0xd85da99c,0xde61a99c,0xd85da99e,
	0xde61a99e,0xd85da9a0,0xde63a9a0,0xd85da9a2,0xde63a9a2,0xd85da9a4,0xde65a9a4,0xd85da9a6,
	0xde65a9a6,0xd85da9a8,0xde67a9a8,0xd85da9aa,0xde67a9aa,0xd85da9ac,0xde69a9ac,0xd85da9ae,
	0xde69a9ae,0xd85da9b0,0xde6ba9b0,0xd85da9b2,0xde6ba9b2,0xd85da9b4,0xde6da9b4,0xd85da9b6,
	0xde6da9b6,0xd85da9b8,0xde6fa9b8,0xd85da9ba,0xde6fa9ba,0xd85da9bc,0xde71a9bc,0xd85da9be,
	0xde71a9be,0xd85da9c0,0xde73a9c0,0xd85da9c2,0xde73a9c2,0xd85da9c4,0xde75a9c4,0xd85da9c6,
	0xde75a9c6,0xd85da9c8,0xde77a9c8,0xd85da9ca,0xde77a9ca,0xd85da9cc,0xde79a9cc,0xd85da9ce,
	0xde79a9ce,0xd85da9d0,0xde7ba9d0,0xd85da9d2,0xde7ba9d2,0xd85da9d4,0xde7da9d4,0xd85da9d6,
	0xde7da9d6,0xd85da9d8,0xde7fa9d8,0xd85da9da,0xde7fa9da,0xd85da9dc,0xde81a9dc,0xd85da9de,
	0xde81a9de,0xd85da9e0,0xde83a9e0,0xd85da9e2,0xde83a9e2,0xd85da9e4,0xde85a9e4,0xd85da9e6,
	0xde85a9e6,0xd85da9e8,0xde87a9e8,0xd85da9ea,0xde87a9ea,0xd85da9ec,0xde89a9ec,0xd85da9ee,
	0xde89a9ee,0xd85da9f0,0xde8ba9f0,0xd85da9f2,0xde8ba9f2,0xd85da9f4,0xde8da9f4,0xd85da9f6,
	0xde8da9f6,0xd85da9f8,0xde8fa9f8,0xd85da9fa,0xde8fa9fa,0xd85da9fc,0xde91a9fc,0xd85da9fe,
	0xde91a9fe,0xd85daa00,0xde93aa00,0xd85daa02,0xde93aa02,0xd85daa04,0xde95aa04,0xd85daa06,
	0xde95aa06,0xd85daa08,0xde97aa08,0xd85daa0a,0xde97aa0a,0xd85daa0c,0xde99aa0c,0xd85daa0e,
	0xde99aa0e,0xd85daa10,0xde9baa10,0xd85daa12,0xde9baa12,0xd85daa14,0xde9daa14,0xd85daa16,
	0xde9daa16,0xd85daa18,0xde9faa18,0xd85daa1a,0xde9faa1a,0xd85daa1c,0xdea1aa1c,0xd85daa1e,
	0xdea1aa1e,0xd85daa20,0xdea3aa20,0xd85daa22,0xdea3aa22,0xd85daa24,0xdea5aa24,0xd85daa26,
	0xdea5aa26,0xd85daa28,0xdea7aa28,0xd85daa2a,0xdea7aa2a,0xd85daa2c,0xdea9aa2c,0xd85daa2e,
	0xdea9aa2e,0xd85daa30,0xdeabaa30,0xd85daa32,0xdeabaa32,0xd85daa34,0xdeadaa34,0xd85daa36,
	0xdeadaa36,0xd85daa38,0xdeafaa38,0xd85daa3a,0xdeafaa3a,0xd85daa3c,0xdeb1aa3c,0xd85daa3e,
	0xdeb1aa3e,0xd85daa40,0xdeb3aa40,0xd85daa42,0xdeb3aa42,0xd85daa44,0xdeb5aa44,0xd85daa46,
	0xdeb5aa46,0xd85daa48,0xdeb7aa48,0xd85daa4a,0xdeb7aa4a,0xd85daa4c,0xdeb9aa4c,0xd85daa4e,
	0xdeb9aa4e,0xd85daa50,0xdebbaa50,0xd85daa52,0xdebbaa52,0xd85daa54,0xdebdaa54,0xd85daa56,
	0xdebdaa56,0xd85daa58,0xdebfaa58,0xd85daa5a,0xdebfaa5a,0xd85daa5c,0xdec1aa5c,0xd85daa5e,
	0xdec1aa5e,0xd85daa60,0xdec3aa60,0xd85daa62,0xdec3aa62,0xd85daa64,0xdec5aa64,0xd85daa66,
	0xdec5aa66,0xd85daa68,0xdec7aa68,0xd85daa6a,0xdec7aa6a,0xd85daa6c,0xdec9aa6c,0xd85daa6e,
	0xdec9aa6e,0xd85daa70,0xdecbaa70,0xd85daa72,0xdecbaa72,0xd85daa74,0xdecdaa74,0xd85daa76,
	0xdecdaa76,0xd85daa78,0xdecfaa78,0xd85daa7a,0xdecfaa7a,0xd85daa7c,0xded1aa7c,0xd85daa7e,
	0xded1aa7e,0xd85daa80,0xded3aa80,0xd85daa82,0xded3aa82,0xd85daa84,0xded5aa84,0xd85daa86,
	0xded5aa86,0xd85daa88,0xded7aa88,0xd85daa8a,0xded7aa8a,0xd85daa8c,0xded9aa8c,0xd85daa8e,
	0xded9aa8e,0xd85daa90,0xdedbaa90,0xd85daa92,0xdedbaa92,0xd85daa94,0xdeddaa94,0xd85daa96,
	0xdeddaa96,0xd85daa98,0xdedfaa98,0xd85daa9a,0xdedfaa9a,0xd85daa9c,0xdee1aa9c,0xd85daa9e,
	0xdee1aa9e,0xd85daaa0,0xdee3aaa0,0xd85daaa2,0xdee3aaa2,0xd85daaa4,0xdee5aaa4,0xd85daaa6,
	0xdee5aaa6,0xd85daaa8,0xdee7aaa8,0xd85daaaa,0xdee7aaaa,0xd85daaac,0xdee9aaac,0xd85daaae,
	0xdee9aaae,0xd85daab0,0xdeebaab0,0xd85daab2,0xdeebaab2,0xd85daab4,0xdeedaab4,0xd85daab6,
	0xdeedaab6,0xd85daab8,0xdeefaab8,0xd85daaba,0xdeefaaba,0xd85daabc,0xdef1aabc,0xd85daabe,
	0xdef1aabe,0xd85daac0,0xdef3aac0,0xd85daac2,0xdef3aac2,0xd85daac4,0xdef5aac4,0xd85daac6,
	0xdef5aac6,0xd85daac8,0xdef7aac8,0xd85daaca,0xdef7aaca,0xd85daacc,0xdef9aacc,0xd85daace,
	0xdef9aace,0xd85daad0,0xdefbaad0,0xd85daad2,0xdefbaad2,0xd85daad4,0xdefdaad4,0xd85daad6,
	0xdefdaad6,0xd85daad8,0xdeffaad8,0xd85daada,0xdeffaada,0xd85daadc,0xdf01aadc,0xd85daade,
	0xdf01aade,0xd85daae0,0xdf03aae0,0xd85daae2,0xdf03aae2,0xd85daae4,0xdf05aae4,0xd85daae6,
	0xdf05aae6,0xd85daae8,0xdf07aae8,0xd85daaea,0xdf07aaea,0xd85daaec,0xdf09aaec,0xd85daaee,
	0xdf09aaee,0xd85daaf0,0xdf0baaf0,0xd85daaf2,0xdf0baaf2,0xd85daaf4,0xdf0daaf4,0xd85daaf6,
	0xdf0daaf6,0xd85daaf8,0xdf0faaf8,0xd85daafa,0xdf0faafa,0xd85daafc,0xdf11aafc,0xd85daafe,
	0xdf11aafe,0xd85dab00,0xdf13ab00,0xd85dab02,0xdf13ab02,0xd85dab04,0xdf15ab04,0xd85dab06,
	0xdf15ab06,0xd85dab08,0xdf17ab08,0xd85dab0a,0xdf17ab0a,0xd85dab0c,0xdf19ab0c,0xd85dab0e,
	0xdf19ab0e,0xd85dab10,0xdf1bab10,0xd85dab12,0xdf1bab12,0xd85dab14,0xdf1dab14,0xd85dab16,
	0xdf1dab16,0xd85dab18,0xdf1fab18,0xd85dab1a,0xdf1fab1a,0xd85dab1c,0xdf21ab1c,0xd85dab1e,
	0xdf21ab1e,0xd85dab20,0xdf23ab20,0xd85dab22,0xdf23ab22,0xd85dab24,0xdf25ab24,0xd85dab26,
	0xdf25ab26,0xd85dab28,0xdf27ab28,0xd85dab2a,0xdf27ab2a,0xd85dab2c,0xdf29ab2c,0xd85dab2e,
	0xdf29ab2e,0xd85dab30,0xdf2bab30,0xd85dab32,0xdf2bab32,0xd85dab34,0xdf2dab34,0xd85dab36,
	0xdf2dab36,0xd85dab38,0xdf2fab38,0xd85dab3a,0xdf2fab3a,0xd85dab3c,0xdf31ab3c,0xd85dab3e,
	0xdf31ab3e,0xd85dab40,0xdf33ab40,0xd85dab42,0xdf33ab42,0xd85dab44,0xdf35ab44,0xd85dab46,
	0xdf35ab46,0xd85dab48,0xdf37ab48,0xd85dab4a,0xdf37ab4a,0xd85dab4c,0xdf39ab4c,0xd85dab4e,
	0xdf39ab4e,0xd85dab50,0xdf3bab50,0xd85dab52,0xdf3bab52,0xd85dab54,0xdf3dab54,0xd85dab56,
	0xdf3dab56,0xd85dab58,0xdf3fab58,0xd85dab5a,0xdf3fab5a,0xd85dab5c,0xdf41ab5c,0xd85dab5e,
	0xdf41ab5e,0xd85dab60,0xdf43ab60,0xd85dab62,0xdf43ab62,0xd85dab64,0xdf45ab64,0xd85dab66,
	0xdf45ab66,0xd85dab68,0xdf47ab68,0xd85dab6a,0xdf47ab6a,0xd85dab6c,0xdf49ab6c,0xd85dab6e,
	0xdf49ab6e,0xd85dab70,0xdf4bab70,0xd85dab72,0xdf4bab72,0xd85dab74,0xdf4dab74,0xd85dab76,
	0xdf4dab76,0xd85dab78,0xdf4fab78,0xd85dab7a,0xdf4fab7a,0xd85dab7c,0xdf51ab7c,0xd85dab7e,
	0xdf51ab7e,0xd85dab80,0xdf53ab80,0xd85dab82,0xdf53ab82,0xd85dab84,0xdf55ab84,0xd85dab86,
	0xdf55ab86,0xd85dab88,0xdf57ab88,0xd85dab8a,0xdf57ab8a,0xd85dab8c,0xdf59ab8c,0xd85dab8e,
	0xdf59ab8e,0xd85dab90,0xdf5bab90,0xd85dab92,0xdf5bab92,0xd85dab94,0xdf5dab94,0xd85dab96,
	0xdf5dab96,0xd85dab98,0xdf5fab98,0xd85dab9a,0xdf5fab9a,0xd85dab9c,0xdf61ab9c,0xd85dab9e,
	0xdf61ab9e,0xd85daba0,0xdf63aba0,0xd85daba2,0xdf63aba2,0xd85daba4,0xdf65aba4,0xd85daba6,
	0xdf65aba6,0xd85daba8,0xdf67aba8,0xd85dabaa,0xdf67abaa,0xd85dabac,0xdf69abac,0xd85dabae,
	0xdf69abae,0xd85dabb0,0xdf6babb0,0xd85dabb2,0xdf6babb2,0xd85dabb4,0xdf6dabb4,0xd85dabb6,
	0xdf6dabb6,0xd85dabb8,0xdf6fabb8,0xd85dabba,0xdf6fabba,0xd85dabbc,0xdf71abbc,0xd85dabbe,
	0xdf71abbe,0xd85dabc0,0xdf73abc0,0xd85dabc2,0xdf73abc2,0xd85dabc4,0xdf75abc4,0xd85dabc6,
	0xdf75abc6,0xd85dabc8,0xdf77abc8,0xd85dabca,0xdf77abca,0xd85dabcc,0xdf79abcc,0xd85dabce,
	0xdf79abce,0xd85dabd0,0xdf7babd0,0xd85dabd2,0xdf7babd2,0xd85dabd4,0xdf7dabd4,0xd85dabd6,
	0xdf7dabd6,0xd85dabd8,0xdf7fabd8,0xd85dabda,0xdf7fabda,0xd85dabdc,0xdf81abdc,0xd85dabde,
	0xdf81abde,0xd85dabe0,0xdf83abe0,0xd85dabe2,0xdf83abe2,0xd85dabe4,0xdf85abe4,0xd85dabe6,
	0xdf85abe6,0xd85dabe8,0xdf87abe8,0xd85dabea,0xdf87abea,0xd85dabec,0xdf89abec,0xd85dabee,
	0xdf89abee,0xd85dabf0,0xdf8babf0,0xd85dabf2,0xdf8babf2,0xd85dabf4,0xdf8dabf4,0xd85dabf6,
	0xdf8dabf6,0xd85dabf8,0xdf8fabf8,0xd85dabfa,0xdf8fabfa,0xd85dabfc,0xdf91abfc,0xd85dabfe,
	0xdf91abfe,0xd85dac00,0xdf93ac00,0xd85dac02,0xdf93ac02,0xd85dac04,0xdf95ac04,0xd85dac06,
	0xdf95ac06,0xd85dac08,0xdf97ac08,0xd85dac0a,0xdf97ac0a,0xd85dac0c,0xdf99ac0c,0xd85dac0e,
	0xdf99ac0e,0xd85dac10,0xdf9bac10,0xd85dac12,0xdf9bac12,0xd85dac14,0xdf9dac14,0xd85dac16,
	0xdf9dac16,0xd85dac18,0xdf9fac18,0xd85dac1a,0xdf9fac1a,0xd85dac1c,0xdfa1ac1c,0xd85dac1e,
	0xdfa1ac1e,0xd85dac20,0xdfa3ac20,0xd85dac22,0xdfa3ac22,0xd85dac24,0xdfa5ac24,0xd85dac26,
	0xdfa5ac26,0xd85dac28,0xdfa7ac28,0xd85dac2a,0xdfa7ac2a,0xd85dac2c,0xdfa9ac2c,0xd85dac2e,
	0xdfa9ac2e,0xd85dac30,0xdfabac30,0xd85dac32,0xdfabac32,0xd85dac34,0xdfadac34,0xd85dac36,
	0xdfadac36,0xd85dac38,0xdfafac38,0xd85dac3a,0xdfafac3a,0xd85dac3c,0xdfb1ac3c,0xd85dac3e,
	0xdfb1ac3e,0xd85dac40,0xdfb3ac40,0xd85dac42,0xdfb3ac42,0xd85dac44,0xdfb5ac44,0xd85dac46,
	0xdfb5ac46,0xd85dac48,0xdfb7ac48,0xd85dac4a,0xdfb7ac4a,0xd85dac4c,0xdfb9ac4c,0xd85dac4e,
	0xdfb9ac4e,0xd85dac50,0xdfbbac50,0xd85dac52,0xdfbbac52,0xd85dac54,0xdfbdac54,0xd85dac56,
	0xdfbdac56,0xd85dac58,0xdfbfac58,0xd85dac5a,0xdfbfac5a,0xd85dac5c,0xdfc1ac5c,0xd85dac5e,
	0xdfc1ac5e,0xd85dac60,0xdfc3ac60,0xd85dac62,0xdfc3ac62,0xd85dac64,0xdfc5ac64,0xd85dac66,
	0xdfc5ac66,0xd85dac68,0xdfc7ac68,0xd85dac6a,0xdfc7ac6a,0xd85dac6c,0xdfc9ac6c,0xd85dac6e,
	0xdfc9ac6e,0xd85dac70,0xdfcbac70,0xd85dac72,0xdfcbac72,0xd85dac74,0xdfcdac74,0xd85dac76,
	0xdfcdac76,0xd85dac78,0xdfcfac78,0xd85dac7a,0xdfcfac7a,0xd85dac7c,0xdfd1ac7c,0xd85dac7e,
	0xdfd1ac7e,0xd85dac80,0xdfd3ac80,0xd85dac82,0xdfd3ac82,0xd85dac84,0xdfd5ac84,0xd85dac86,
	0xdfd5ac86,0xd85dac88,0xdfd7ac88,0xd85dac8a,0xdfd7ac8a,0xd85dac8c,0xdfd9ac8c,0xd85dac8e,
	0xdfd9ac8e,0xd85dac90,0xdfdbac90,0xd85dac92,0xdfdbac92,0xd85dac94,0xdfddac94,0xd85dac96,
	0xdfddac96,0xd85dac98,0xdfdfac98,0xd85dac9a,0xdfdfac9a,0xd85dac9c,0xdfe1ac9c,0xd85dac9e,
	0xdfe1ac9e,0xd85daca0,0xdfe3aca0,0xd85daca2,0xdfe3aca2,0xd85daca4,0xdfe5aca4,0xd85daca6,
	0xdfe5aca6,0xd85daca8,0xdfe7aca8,0xd85dacaa,0xdfe7acaa,0xd85dacac,0xdfe9acac,0xd85dacae,
	0xdfe9acae,0xd85dacb0,0xdfebacb0,0xd85dacb2,0xdfebacb2,0xd85dacb4,0xdfedacb4,0xd85dacb6,
	0xdfedacb6,0xd85dacb8,0xdfefacb8,0xd85dacba,0xdfefacba,0xd85dacbc,0xdff1acbc,0xd85dacbe,
	0xdff1acbe,0xd85dacc0,0xdff3acc0,0xd85dacc2,0xdff3acc2,0xd85dacc4,0xdff5acc4,0xd85dacc6,
	0xdff5acc6,0xd85dacc8,0xdff7acc8,0xd85dacca,0xdff7acca,0xd85daccc,0xdff9accc,0xd85dacce,
	0xdff9acce,0xd85dacd0,0xdffbacd0,0xd85dacd2,0xdffbacd2,0xd85dacd4,0xdffdacd4,0xd85dacd6,
	0xdffdacd6,0xd85dacd8,0xdfffacd8,0xd85dacda,0xdfffacda,0xd85facdc,0xdc01acdc,0xd85facde,
	0xdc01acde,0xd85face0,0xdc03ace0,0xd85face2,0xdc03ace2,0xd85face4,0xdc05ace4,0xd85face6,
	0xdc05ace6,0xd85face8,0xdc07ace8,0xd85facea,0xdc07acea,0xd85facec,0xdc09acec,0xd85facee,
	0xdc09acee,0xd85facf0,0xdc0bacf0,0xd85facf2,0xdc0bacf2,0xd85facf4,0xdc0dacf4,0xd85facf6,
	0xdc0dacf6,0xd85facf8,0xdc0facf8,0xd85facfa,0xdc0facfa,0xd85facfc,0xdc11acfc,0xd85facfe,
	0xdc11acfe,0xd85fad00,0xdc13ad00,0xd85fad02,0xdc13ad02,0xd85fad04,0xdc15ad04,0xd85fad06,
	0xdc15ad06,0xd85fad08,0xdc17ad08,0xd85fad0a,0xdc17ad0a,0xd85fad0c,0xdc19ad0c,0xd85fad0e,
	0xdc19ad0e,0xd85fad10,0xdc1bad10,0xd85fad12,0xdc1bad12,0xd85fad14,0xdc1dad14,0xd85fad16,
	0xdc1dad16,0xd85fad18,0xdc1fad18,0xd85fad1a,0xdc1fad1a,0xd85fad1c,0xdc21ad1c,0xd85fad1e,
	0xdc21ad1e,0xd85fad20,0xdc23ad20,0xd85fad22,0xdc23ad22,0xd85fad24,0xdc25ad24,0xd85fad26,
	0xdc25ad26,0xd85fad28,0xdc27ad28,0xd85fad2a,0xdc27ad2a,0xd85fad2c,0xdc29ad2c,0xd85fad2e,
	0xdc29ad2e,0xd85fad30,0xdc2bad30,0xd85fad32,0xdc2bad32,0xd85fad34,0xdc2dad34,0xd85fad36,
	0xdc2dad36,0xd85fad38,0xdc2fad38,0xd85fad3a,0xdc2fad3a,0xd85fad3c,0xdc31ad3c,0xd85fad3e,
	0xdc31ad3e,0xd85fad40,0xdc33ad40,0xd85fad42,0xdc33ad42,0xd85fad44,0xdc35ad44,0xd85fad46,
	0xdc35ad46,0xd85fad48,0xdc37ad48,0xd85fad4a,0xdc37ad4a,0xd85fad4c,0xdc39ad4c,0xd85fad4e,
	0xdc39ad4e,0xd85fad50,0xdc3bad50,0xd85fad52,0xdc3bad52,0xd85fad54,0xdc3dad54,0xd85fad56,
	0xdc3dad56,0xd85fad58,0xdc3fad58,0xd85fad5a,0xdc3fad5a,0xd85fad5c,0xdc41ad5c,0xd85fad5e,
	0xdc41ad5e,0xd85fad60,0xdc43ad60,0xd85fad62,0xdc43ad62,0xd85fad64,0xdc45ad64,0xd85fad66,
	0xdc45ad66,0xd85fad68,0xdc47ad68,0xd85fad6a,0xdc47ad6a,0xd85fad6c,0xdc49ad6c,0xd85fad6e,
	0xdc49ad6e,0xd85fad70,0xdc4bad70,0xd85fad72,0xdc4bad72,0xd85fad74,0xdc4dad74,0xd85fad76,
	0xdc4dad76,0xd85fad78,0xdc4fad78,0xd85fad7a,0xdc4fad7a,0xd85fad7c,0xdc51ad7c,0xd85fad7e,
	0xdc51ad7e,0xd85fad80,0xdc53ad80,0xd85fad82,0xdc53ad82,0xd85fad84,0xdc55ad84,0xd85fad86,
	0xdc55ad86,0xd85fad88,0xdc57ad88,0xd85fad8a,0xdc57ad8a,0xd85fad8c,0xdc59ad8c,0xd85fad8e,
	0xdc59ad8e,0xd85fad90,0xdc5bad90,0xd85fad92,0xdc5bad92,0xd85fad94,0xdc5dad94,0xd85fad96,
	0xdc5dad96,0xd85fad98,0xdc5fad98,0xd85fad9a,0xdc5fad9a,0xd85fad9c,0xdc61ad9c,0xd85fad9e,
	0xdc61ad9e,0xd85fada0,0xdc63ada0,0xd85fada2,0xdc63ada2,0xd85fada4,0xdc65ada4,0xd85fada6,
	0xdc65ada6,0xd85fada8,0xdc67ada8,0xd85fadaa,0xdc67adaa,0xd85fadac,0xdc69adac,0xd85fadae,
	0xdc69adae,0xd85fadb0,0xdc6badb0,0xd85fadb2,0xdc6badb2,0xd85fadb4,0xdc6dadb4,0xd85fadb6,
	0xdc6dadb6,0xd85fadb8,0xdc6fadb8,0xd85fadba,0xdc6fadba,0xd85fadbc,0xdc71adbc,0xd85fadbe,
	0xdc71adbe,0xd85fadc0,0xdc73adc0,0xd85fadc2,0xdc73adc2,0xd85fadc4,0xdc75adc4,0xd85fadc6,
	0xdc75adc6,0xd85fadc8,0xdc77adc8,0xd85fadca,0xdc77adca,0xd85fadcc,0xdc79adcc,0xd85fadce,
	0xdc79adce,0xd85fadd0,0xdc7badd0,0xd85fadd2,0xdc7badd2,0xd85fadd4,0xdc7dadd4,0xd85fadd6,
	0xdc7dadd6,0xd85fadd8,0xdc7fadd8,0xd85fadda,0xdc7fadda,0xd85faddc,0xdc81addc,0xd85fadde,
	0xdc81adde,0xd85fade0,0xdc83ade0,0xd85fade2,0xdc83ade2,0xd85fade4,0xdc85ade4,0xd85fade6,
	0xdc85ade6,0xd85fade8,0xdc87ade8,0xd85fadea,0xdc87adea,0xd85fadec,0xdc89adec,0xd85fadee,
	0xdc89adee,0xd85fadf0,0xdc8badf0,0xd85fadf2,0xdc8badf2,0xd85fadf4,0xdc8dadf4,0xd85fadf6,
	0xdc8dadf6,0xd85fadf8,0xdc8fadf8,0xd85fadfa,0xdc8fadfa,0xd85fadfc,0xdc91adfc,0xd85fadfe,
	0xdc91adfe,0xd85fae00,0xdc93ae00,0xd85fae02,0xdc93ae02,0xd85fae04,0xdc95ae04,0xd85fae06,
	0xdc95ae06,0xd85fae08,0xdc97ae08,0xd85fae0a,0xdc97ae0a,0xd85fae0c,0xdc99ae0c,0xd85fae0e,
	0xdc99ae0e,0xd85fae10,0xdc9bae10,0xd85fae12,0xdc9bae12,0xd85fae14,0xdc9dae14,0xd85fae16,
	0xdc9dae16,0xd85fae18,0xdc9fae18,0xd85fae1a,0xdc9fae1a,0xd85fae1c,0xdca1ae1c,0xd85fae1e,
	0xdca1ae1e,0xd85fae20,0xdca3ae20,0xd85fae22,0xdca3ae22,0xd85fae24,0xdca5ae24,0xd85fae26,
	0xdca5ae26,0xd85fae28,0xdca7ae28,0xd85fae2a,0xdca7ae2a,0xd85fae2c,0xdca9ae2c,0xd85fae2e,
	0xdca9ae2e,0xd85fae30,0xdcabae30,0xd85fae32,0xdcabae32,0xd85fae34,0xdcadae34,0xd85fae36,
	0xdcadae36,0xd85fae38,0xdcafae38,0xd85fae3a,0xdcafae3a,0xd85fae3c,0xdcb1ae3c,0xd85fae3e,
	0xdcb1ae3e,0xd85fae40,0xdcb3ae40,0xd85fae42,0xdcb3ae42,0xd85fae44,0xdcb5ae44,0xd85fae46,
	0xdcb5ae46,0xd85fae48,0xdcb7ae48,0xd85fae4a,0xdcb7ae4a,0xd85fae4c,0xdcb9ae4c,0xd85fae4e,
	0xdcb9ae4e,0xd85fae50,0xdcbbae50,0xd85fae52,0xdcbbae52,0xd85fae54,0xdcbdae54,0xd85fae56,
	0xdcbdae56,0xd85fae58,0xdcbfae58,0xd85fae5a,0xdcbfae5a,0xd85fae5c,0xdcc1ae5c,0xd85fae5e,
	0xdcc1ae5e,0xd85fae60,0xdcc3ae60,0xd85fae62,0xdcc3ae62,0xd85fae64,0xdcc5ae64,0xd85fae66,
	0xdcc5ae66,0xd85fae68,0xdcc7ae68,0xd85fae6a,0xdcc7ae6a,0xd85fae6c,0xdcc9ae6c,0xd85fae6e,
	0xdcc9ae6e,0xd85fae70,0xdccbae70,0xd85fae72,0xdccbae72,0xd85fae74,0xdccdae74,0xd85fae76,
	0xdccdae76,0xd85fae78,0xdccfae78,0xd85fae7a,0xdccfae7a,0xd85fae7c,0xdcd1ae7c,0xd85fae7e,
	0xdcd1ae7e,0xd85fae80,0xdcd3ae80,0xd85fae82,0xdcd3ae82,0xd85fae84,0xdcd5ae84,0xd85fae86,
	0xdcd5ae86,0xd85fae88,0xdcd7ae88,0xd85fae8a,0xdcd7ae8a,0xd85fae8c,0xdcd9ae8c,0xd85fae8e,
	0xdcd9ae8e,0xd85fae90,0xdcdbae90,0xd85fae92,0xdcdbae92,0xd85fae94,0xdcddae94,0xd85fae96,
	0xdcddae96,0xd85fae98,0xdcdfae98,0xd85fae9a,0xdcdfae9a,0xd85fae9c,0xdce1ae9c,0xd85fae9e,
	0xdce1ae9e,0xd85faea0,0xdce3aea0,0xd85faea2,0xdce3aea2,0xd85faea4,0xdce5aea4,0xd85faea6,
	0xdce5aea6,0xd85faea8,0xdce7aea8,0xd85faeaa,0xdce7aeaa,0xd85faeac,0xdce9aeac,0xd85faeae,
	0xdce9aeae,0xd85faeb0,0xdcebaeb0,0xd85faeb2,0xdcebaeb2,0xd85faeb4,0xdcedaeb4,0xd85faeb6,
	0xdcedaeb6,0xd85faeb8,0xdcefaeb8,0xd85faeba,0xdcefaeba,0xd85faebc,0xdcf1aebc,0xd85faebe,
	0xdcf1aebe,0xd85faec0,0xdcf3aec0,0xd85faec2,0xdcf3aec2,0xd85faec4,0xdcf5aec4,0xd85faec6,
	0xdcf5aec6,0xd85faec8,0xdcf7aec8,0xd85faeca,0xdcf7aeca,0xd85faecc,0xdcf9aecc,0xd85faece,
	0xdcf9aece,0xd85faed0,0xdcfbaed0,0xd85faed2,0xdcfbaed2,0xd85faed4,0xdcfdaed4,0xd85faed6,
	0xdcfdaed6,0xd85faed8,0xdcffaed8,0xd85faeda,0xdcffaeda,0xd85faedc,0xdd01aedc,0xd85faede,
	0xdd01aede,0xd85faee0,0xdd03aee0,0xd85faee2,0xdd03aee2,0xd85faee4,0xdd05aee4,0xd85faee6,
	0xdd05aee6,0xd85faee8,0xdd07aee8,0xd85faeea,0xdd07aeea,0xd85faeec,0xdd09aeec,0xd85faeee,
	0xdd09aeee,0xd85faef0,0xdd0baef0,0xd85faef2,0xdd0baef2,0xd85faef4,0xdd0daef4,0xd85faef6,
	0xdd0daef6,0xd85faef8,0xdd0faef8,0xd85faefa,0xdd0faefa,0xd85faefc,0xdd11aefc,0xd85faefe,
	0xdd11aefe,0xd85faf00,0xdd13af00,0xd85faf02,0xdd13af02,0xd85faf04,0xdd15af04,0xd85faf06,
	0xdd15af06,0xd85faf08,0xdd17af08,0xd85faf0a,0xdd17af0a,0xd85faf0c,0xdd19af0c,0xd85faf0e,
	0xdd19af0e,0xd85faf10,0xdd1baf10,0xd85faf12,0xdd1baf12,0xd85faf14,0xdd1daf14,0xd85faf16,
	0xdd1daf16,0xd85faf18,0xdd1faf18,0xd85faf1a,0xdd1faf1a,0xd85faf1c,0xdd21af1c,0xd85faf1e,
	0xdd21af1e,0xd85faf20,0xdd23af20,0xd85faf22,0xdd23af22,0xd85faf24,0xdd25af24,0xd85faf26,
	0xdd25af26,0xd85faf28,0xdd27af28,0xd85faf2a,0xdd27af2a,0xd85faf2c,0xdd29af2c,0xd85faf2e,
	0xdd29af2e,0xd85faf30,0xdd2baf30,0xd85faf32,0xdd2baf32,0xd85faf34,0xdd2daf34,0xd85faf36,
	0xdd2daf36,0xd85faf38,0xdd2faf38,0xd85faf3a,0xdd2faf3a,0xd85faf3c,0xdd31af3c,0xd85faf3e,
	0xdd31af3e,0xd85faf40,0xdd33af40,0xd85faf42,0xdd33af42,0xd85faf44,0xdd35af44,0xd85faf46,
	0xdd35af46,0xd85faf48,0xdd37af48,0xd85faf4a,0xdd37af4a,0xd85faf4c,0xdd39af4c,0xd85faf4e,
	0xdd39af4e,0xd85faf50,0xdd3baf50,0xd85faf52,0xdd3baf52,0xd85faf54,0xdd3daf54,0xd85faf56,
	0xdd3daf56,0xd85faf58,0xdd3faf58,0xd85faf5a,0xdd3faf5a,0xd85faf5c,0xdd41af5c,0xd85faf5e,
	0xdd41af5e,0xd85faf60,0xdd43af60,0xd85faf62,0xdd43af62,0xd85faf64,0xdd45af64,0xd85faf66,
	0xdd45af66,0xd85faf68,0xdd47af68,0xd85faf6a,0xdd47af6a,0xd85faf6c,0xdd49af6c,0xd85faf6e,
	0xdd49af6e,0xd85faf70,0xdd4baf70,0xd85faf72,0xdd4baf72,0xd85faf74,0xdd4daf74,0xd85faf76,
	0xdd4daf76,0xd85faf78,0xdd4faf78,0xd85faf7a,0xdd4faf7a,0xd85faf7c,0xdd51af7c,0xd85faf7e,
	0xdd51af7e,0xd85faf80,0xdd53af80,0xd85faf82,0xdd53af82,0xd85faf84,0xdd55af84,0xd85faf86,
	0xdd55af86,0xd85faf88,0xdd57af88,0xd85faf8a,0xdd57af8a,0xd85faf8c,0xdd59af8c,0xd85faf8e,
	0xdd59af8e,0xd85faf90,0xdd5baf90,0xd85faf92,0xdd5baf92,0xd85faf94,0xdd5daf94,0xd85faf96,
	0xdd5daf96,0xd85faf98,0xdd5faf98,0xd85faf9a,0xdd5faf9a,0xd85faf9c,0xdd61af9c,0xd85faf9e,
	0xdd61af9e,0xd85fafa0,0xdd63afa0,0xd85fafa2,0xdd63afa2,0xd85fafa4,0xdd65afa4,0xd85fafa6,
	0xdd65afa6,0xd85fafa8,0xdd67afa8,0xd85fafaa,0xdd67afaa,0xd85fafac,0xdd69afac,0xd85fafae,
	0xdd69afae,0xd85fafb0,0xdd6bafb0,0xd85fafb2,0xdd6bafb2,0xd85fafb4,0xdd6dafb4,0xd85fafb6,
	0xdd6dafb6,0xd85fafb8,0xdd6fafb8,0xd85fafba,0xdd6fafba,0xd85fafbc,0xdd71afbc,0xd85fafbe,
	0xdd71afbe,0xd85fafc0,0xdd73afc0,0xd85fafc2,0xdd73afc2,0xd85fafc4,0xdd75afc4,0xd85fafc6,
	0xdd75afc6,0xd85fafc8,0xdd77afc8,0xd85fafca,0xdd77afca,0xd85fafcc,0xdd79afcc,0xd85fafce,
	0xdd79afce,0xd85fafd0,0xdd7bafd0,0xd85fafd2,0xdd7bafd2,0xd85fafd4,0xdd7dafd4,0xd85fafd6,
	0xdd7dafd6,0xd85fafd8,0xdd7fafd8,0xd85fafda,0xdd7fafda,0xd85fafdc,0xdd81afdc,0xd85fafde,
	0xdd81afde,0xd85fafe0,0xdd83afe0,0xd85fafe2,0xdd83afe2,0xd85fafe4,0xdd85afe4,0xd85fafe6,
	0xdd85afe6,0xd85fafe8,0xdd87afe8,0xd85fafea,0xdd87afea,0xd85fafec,0xdd89afec,0xd85fafee,
	0xdd89afee,0xd85faff0,0xdd8baff0,0xd85faff2,0xdd8baff2,0xd85faff4,0xdd8daff4,0xd85faff6,
	0xdd8daff6,0xd85faff8,0xdd8faff8,0xd85faffa,0xdd8faffa,0xd85faffc,0xdd91affc,0xd85faffe,
	0xdd91affe,0xd85fb000,0xdd93b000,0xd85fb002,0xdd93b002,0xd85fb004,0xdd95b004,0xd85fb006,
	0xdd95b006,0xd85fb008,0xdd97b008,0xd85fb00a,0xdd97b00a,0xd85fb00c,0xdd99b00c,0xd85fb00e,
	0xdd99b00e,0xd85fb010,0xdd9bb010,0xd85fb012,0xdd9bb012,0xd85fb014,0xdd9db014,0xd85fb016,
	0xdd9db016,0xd85fb018,0xdd9fb018,0xd85fb01a,0xdd9fb01a,0xd85fb01c,0xdda1b01c,0xd85fb01e,
	0xdda1b01e,0xd85fb020,0xdda3b020,0xd85fb022,0xdda3b022,0xd85fb024,0xdda5b024,0xd85fb026,
	0xdda5b026,0xd85fb028,0xdda7b028,0xd85fb02a,0xdda7b02a,0xd85fb02c,0xdda9b02c,0xd85fb02e,
	0xdda9b02e,0xd85fb030,0xddabb030,0xd85fb032,0xddabb032,0xd85fb034,0xddadb034,0xd85fb036,
	0xddadb036,0xd85fb038,0xddafb038,0xd85fb03a,0xddafb03a,0xd85fb03c,0xddb1b03c,0xd85fb03e,
	0xddb1b03e,0xd85fb040,0xddb3b040,0xd85fb042,0xddb3b042,0xd85fb044,0xddb5b044,0xd85fb046,
	0xddb5b046,0xd85fb048,0xddb7b048,0xd85fb04a,0xddb7b04a,0xd85fb04c,0xddb9b04c,0xd85fb04e,
	0xddb9b04e,0xd85fb050,0xddbbb050,0xd85fb052,0xddbbb052,0xd85fb054,0xddbdb054,0xd85fb056,
	0xddbdb056,0xd85fb058,0xddbfb058,0xd85fb05a,0xddbfb05a,0xd85fb05c,0xddc1b05c,0xd85fb05e,
	0xddc1b05e,0xd85fb060,0xddc3b060,0xd85fb062,0xddc3b062,0xd85fb064,0xddc5b064,0xd85fb066,
	0xddc5b066,0xd85fb068,0xddc7b068,0xd85fb06a,0xddc7b06a,0xd85fb06c,0xddc9b06c,0xd85fb06e,
	0xddc9b06e,0xd85fb070,0xddcbb070,0xd85fb072,0xddcbb072,0xd85fb074,0xddcdb074,0xd85fb076,
	0xddcdb076,0xd85fb078,0xddcfb078,0xd85fb07a,0xddcfb07a,0xd85fb07c,0xddd1b07c,0xd85fb07e,
	0xddd1b07e,0xd85fb080,0xddd3b080,0xd85fb082,0xddd3b082,0xd85fb084,0xddd5b084,0xd85fb086,
	0xddd5b086,0xd85fb088,0xddd7b088,0xd85fb08a,0xddd7b08a,0xd85fb08c,0xddd9b08c,0xd85fb08e,
	0xddd9b08e,0xd85fb090,0xdddbb090,0xd85fb092,0xdddbb092,0xd85fb094,0xddddb094,0xd85fb096,
	0xddddb096,0xd85fb098,0xdddfb098,0xd85fb09a,0xdddfb09a,0xd85fb09c,0xdde1b09c,0xd85fb09e,
	0xdde1b09e,0xd85fb0a0,0xdde3b0a0,0xd85fb0a2,0xdde3b0a2,0xd85fb0a4,0xdde5b0a4,0xd85fb0a6,
	0xdde5b0a6,0xd85fb0a8,0xdde7b0a8,0xd85fb0aa,0xdde7b0aa,0xd85fb0ac,0xdde9b0ac,0xd85fb0ae,
	0xdde9b0ae,0xd85fb0b0,0xddebb0b0,0xd85fb0b2,0xddebb0b2,0xd85fb0b4,0xddedb0b4,0xd85fb0b6,
	0xddedb0b6,0xd85fb0b8,0xddefb0b8,0xd85fb0ba,0xddefb0ba,0xd85fb0bc,0xddf1b0bc,0xd85fb0be,
	0xddf1b0be,0xd85fb0c0,0xddf3b0c0,0xd85fb0c2,0xddf3b0c2,0xd85fb0c4,0xddf5b0c4,0xd85fb0c6,
	0xddf5b0c6,0xd85fb0c8,0xddf7b0c8,0xd85fb0ca,0xddf7b0ca,0xd85fb0cc,0xddf9b0cc,0xd85fb0ce,
	0xddf9b0ce,0xd85fb0d0,0xddfbb0d0,0xd85fb0d2,0xddfbb0d2,0xd85fb0d4,0xddfdb0d4,0xd85fb0d6,
	0xddfdb0d6,0xd85fb0d8,0xddffb0d8,0xd85fb0da,0xddffb0da,0xd85fb0dc,0xde01b0dc,0xd85fb0de,
	0xde01b0de,0xd85fb0e0,0xde03b0e0,0xd85fb0e2,0xde03b0e2,0xd85fb0e4,0xde05b0e4,0xd85fb0e6,
	0xde05b0e6,0xd85fb0e8,0xde07b0e8,0xd85fb0ea,0xde07b0ea,0xd85fb0ec,0xde09b0ec,0xd85fb0ee,
	0xde09b0ee,0xd85fb0f0,0xde0bb0f0,0xd85fb0f2,0xde0bb0f2,0xd85fb0f4,0xde0db0f4,0xd85fb0f6,
	0xde0db0f6,0xd85fb0f8,0xde0fb0f8,0xd85fb0fa,0xde0fb0fa,0xd85fb0fc,0xde11b0fc,0xd85fb0fe,
	0xde11b0fe,0xd85fb100,0xde13b100,0xd85fb102,0xde13b102,0xd85fb104,0xde15b104,0xd85fb106,
	0xde15b106,0xd85fb108,0xde17b108,0xd85fb10a,0xde17b10a,0xd85fb10c,0xde19b10c,0xd85fb10e,
	0xde19b10e,0xd85fb110,0xde1bb110,0xd85fb112,0xde1bb112,0xd85fb114,0xde1db114,0xd85fb116,
	0xde1db116,0xd85fb118,0xde1fb118,0xd85fb11a,0xde1fb11a,0xd85fb11c,0xde21b11c,0xd85fb11e,
	0xde21b11e,0xd85fb120,0xde23b120,0xd85fb122,0xde23b122,0xd85fb124,0xde25b124,0xd85fb126,
	0xde25b126,0xd85fb128,0xde27b128,0xd85fb12a,0xde27b12a,0xd85fb12c,0xde29b12c,0xd85fb12e,
	0xde29b12e,0xd85fb130,0xde2bb130,0xd85fb132,0xde2bb132,0xd85fb134,0xde2db134,0xd85fb136,
	0xde2db136,0xd85fb138,0xde2fb138,0xd85fb13a,0xde2fb13a,0xd85fb13c,0xde31b13c,0xd85fb13e,
	0xde31b13e,0xd85fb140,0xde33b140,0xd85fb142,0xde33b142,0xd85fb144,0xde35b144,0xd85fb146,
	0xde35b146,0xd85fb148,0xde37b148,0xd85fb14a,0xde37b14a,0xd85fb14c,0xde39b14c,0xd85fb14e,
	0xde39b14e,0xd85fb150,0xde3bb150,0xd85fb152,0xde3bb152,0xd85fb154,0xde3db154,0xd85fb156,
	0xde3db156,0xd85fb158,0xde3fb158,0xd85fb15a,0xde3fb15a,0xd85fb15c,0xde41b15c,0xd85fb15e,
	0xde41b15e,0xd85fb160,0xde43b160,0xd85fb162,0xde43b162,0xd85fb164,0xde45b164,0xd85fb166,
	0xde45b166,0xd85fb168,0xde47b168,0xd85fb16a,0xde47b16a,0xd85fb16c,0xde49b16c,0xd85fb16e,
	0xde49b16e,0xd85fb170,0xde4bb170,0xd85fb172,0xde4bb172,0xd85fb174,0xde4db174,0xd85fb176,
	0xde4db176,0xd85fb178,0xde4fb178,0xd85fb17a,0xde4fb17a,0xd85fb17c,0xde51b17c,0xd85fb17e,
	0xde51b17e,0xd85fb180,0xde53b180,0xd85fb182,0xde53b182,0xd85fb184,0xde55b184,0xd85fb186,
	0xde55b186,0xd85fb188,0xde57b188,0xd85fb18a,0xde57b18a,0xd85fb18c,0xde59b18c,0xd85fb18e,
	0xde59b18e,0xd85fb190,0xde5bb190,0xd85fb192,0xde5bb192,0xd85fb194,0xde5db194,0xd85fb196,
	0xde5db196,0xd85fb198,0xde5fb198,0xd85fb19a,0xde5fb19a,0xd85fb19c,0xde61b19c,0xd85fb19e,
	0xde61b19e,0xd85fb1a0,0xde63b1a0,0xd85fb1a2,0xde63b1a2,0xd85fb1a4,0xde65b1a4,0xd85fb1a6,
	0xde65b1a6,0xd85fb1a8,0xde67b1a8,0xd85fb1aa,0xde67b1aa,0xd85fb1ac,0xde69b1ac,0xd85fb1ae,
	0xde69b1ae,0xd85fb1b0,0xde6bb1b0,0xd85fb1b2,0xde6bb1b2,0xd85fb1b4,0xde6db1b4,0xd85fb1b6,
	0xde6db1b6,0xd85fb1b8,0xde6fb1b8,0xd85fb1ba,0xde6fb1ba,0xd85fb1bc,0xde71b1bc,0xd85fb1be,
	0xde71b1be,0xd85fb1c0,0xde73b1c0,0xd85fb1c2,0xde73b1c2,0xd85fb1c4,0xde75b1c4,0xd85fb1c6,
	0xde75b1c6,0xd85fb1c8,0xde77b1c8,0xd85fb1ca,0xde77b1ca,0xd85fb1cc,0xde79b1cc,0xd85fb1ce,
	0xde79b1ce,0xd85fb1d0,0xde7bb1d0,0xd85fb1d2,0xde7bb1d2,0xd85fb1d4,0xde7db1d4,0xd85fb1d6,
	0xde7db1d6,0xd85fb1d8,0xde7fb1d8,0xd85fb1da,0xde7fb1da,0xd85fb1dc,0xde81b1dc,0xd85fb1de,
	0xde81b1de,0xd85fb1e0,0xde83b1e0,0xd85fb1e2,0xde83b1e2,0xd85fb1e4,0xde85b1e4,0xd85fb1e6,
	0xde85b1e6,0xd85fb1e8,0xde87b1e8,0xd85fb1ea,0xde87b1ea,0xd85fb1ec,0xde89b1ec,0xd85fb1ee,
	0xde89b1ee,0xd85fb1f0,0xde8bb1f0,0xd85fb1f2,0xde8bb1f2,0xd85fb1f4,0xde8db1f4,0xd85fb1f6,
	0xde8db1f6,0xd85fb1f8,0xde8fb1f8,0xd85fb1fa,0xde8fb1fa,0xd85fb1fc,0xde91b1fc,0xd85fb1fe,
	0xde91b1fe,0xd85fb200,0xde93b200,0xd85fb202,0xde93b202,0xd85fb204,0xde95b204,0xd85fb206,
	0xde95b206,0xd85fb208,0xde97b208,0xd85fb20a,0xde97b20a,0xd85fb20c,0xde99b20c,0xd85fb20e,
	0xde99b20e,0xd85fb210,0xde9bb210,0xd85fb212,0xde9bb212,0xd85fb214,0xde9db214,0xd85fb216,
	0xde9db216,0xd85fb218,0xde9fb218,0xd85fb21a,0xde9fb21a,0xd85fb21c,0xdea1b21c,0xd85fb21e,
	0xdea1b21e,0xd85fb220,0xdea3b220,0xd85fb222,0xdea3b222,0xd85fb224,0xdea5b224,0xd85fb226,
	0xdea5b226,0xd85fb228,0xdea7b228,0xd85fb22a,0xdea7b22a,0xd85fb22c,0xdea9b22c,0xd85fb22e,
	0xdea9b22e,0xd85fb230,0xdeabb230,0xd85fb232,0xdeabb232,0xd85fb234,0xdeadb234,0xd85fb236,
	0xdeadb236,0xd85fb238,0xdeafb238,0xd85fb23a,0xdeafb23a,0xd85fb23c,0xdeb1b23c,0xd85fb23e,
	0xdeb1b23e,0xd85fb240,0xdeb3b240,0xd85fb242,0xdeb3b242,0xd85fb244,0xdeb5b244,0xd85fb246,
	0xdeb5b246,0xd85fb248,0xdeb7b248,0xd85fb24a,0xdeb7b24a,0xd85fb24c,0xdeb9b24c,0xd85fb24e,
	0xdeb9b24e,0xd85fb250,0xdebbb250,0xd85fb252,0xdebbb252,0xd85fb254,0xdebdb254,0xd85fb256,
	0xdebdb256,0xd85fb258,0xdebfb258,0xd85fb25a,0xdebfb25a,0xd85fb25c,0xdec1b25c,0xd85fb25e,
	0xdec1b25e,0xd85fb260,0xdec3b260,0xd85fb262,0xdec3b262,0xd85fb264,0xdec5b264,0xd85fb266,
	0xdec5b266,0xd85fb268,0xdec7b268,0xd85fb26a,0xdec7b26a,0xd85fb26c,0xdec9b26c,0xd85fb26e,
	0xdec9b26e,0xd85fb270,0xdecbb270,0xd85fb272,0xdecbb272,0xd85fb274,0xdecdb274,0xd85fb276,
	0xdecdb276,0xd85fb278,0xdecfb278,0xd85fb27a,0xdecfb27a,0xd85fb27c,0xded1b27c,0xd85fb27e,
	0xded1b27e,0xd85fb280,0xded3b280,0xd85fb282,0xded3b282,0xd85fb284,0xded5b284,0xd85fb286,
	0xded5b286,0xd85fb288,0xded7b288,0xd85fb28a,0xded7b28a,0xd85fb28c,0xded9b28c,0xd85fb28e,
	0xded9b28e,0xd85fb290,0xdedbb290,0xd85fb292,0xdedbb292,0xd85fb294,0xdeddb294,0xd85fb296,
	0xdeddb296,0xd85fb298,0xdedfb298,0xd85fb29a,0xdedfb29a,0xd85fb29c,0xdee1b29c,0xd85fb29e,
	0xdee1b29e,0xd85fb2a0,0xdee3b2a0,0xd85fb2a2,0xdee3b2a2,0xd85fb2a4,0xdee5b2a4,0xd85fb2a6,
	0xdee5b2a6,0xd85fb2a8,0xdee7b2a8,0xd85fb2aa,0xdee7b2aa,0xd85fb2ac,0xdee9b2ac,0xd85fb2ae,
	0xdee9b2ae,0xd85fb2b0,0xdeebb2b0,0xd85fb2b2,0xdeebb2b2,0xd85fb2b4,0xdeedb2b4,0xd85fb2b6,
	0xdeedb2b6,0xd85fb2b8,0xdeefb2b8,0xd85fb2ba,0xdeefb2ba,0xd85fb2bc,0xdef1b2bc,0xd85fb2be,
	0xdef1b2be,0xd85fb2c0,0xdef3b2c0,0xd85fb2c2,0xdef3b2c2,0xd85fb2c4,0xdef5b2c4,0xd85fb2c6,
	0xdef5b2c6,0xd85fb2c8,0xdef7b2c8,0xd85fb2ca,0xdef7b2ca,0xd85fb2cc,0xdef9b2cc,0xd85fb2ce,
	0xdef9b2ce,0xd85fb2d0,0xdefbb2d0,0xd85fb2d2,0xdefbb2d2,0xd85fb2d4,0xdefdb2d4,0xd85fb2d6,
	0xdefdb2d6,0xd85fb2d8,0xdeffb2d8,0xd85fb2da,0xdeffb2da,0xd85fb2dc,0xdf01b2dc,0xd85fb2de,
	0xdf01b2de,0xd85fb2e0,0xdf03b2e0,0xd85fb2e2,0xdf03b2e2,0xd85fb2e4,0xdf05b2e4,0xd85fb2e6,
	0xdf05b2e6,0xd85fb2e8,0xdf07b2e8,0xd85fb2ea,0xdf07b2ea,0xd85fb2ec,0xdf09b2ec,0xd85fb2ee,
	0xdf09b2ee,0xd85fb2f0,0xdf0bb2f0,0xd85fb2f2,0xdf0bb2f2,0xd85fb2f4,0xdf0db2f4,0xd85fb2f6,
	0xdf0db2f6,0xd85fb2f8,0xdf0fb2f8,0xd85fb2fa,0xdf0fb2fa,0xd85fb2fc,0xdf11b2fc,0xd85fb2fe,
	0xdf11b2fe,0xd85fb300,0xdf13b300,0xd85fb302,0xdf13b302,0xd85fb304,0xdf15b304,0xd85fb306,
	0xdf15b306,0xd85fb308,0xdf17b308,0xd85fb30a,0xdf17b30a,0xd85fb30c,0xdf19b30c,0xd85fb30e,
	0xdf19b30e,0xd85fb310,0xdf1bb310,0xd85fb312,0xdf1bb312,0xd85fb314,0xdf1db314,0xd85fb316,
	0xdf1db316,0xd85fb318,0xdf1fb318,0xd85fb31a,0xdf1fb31a,0xd85fb31c,0xdf21b31c,0xd85fb31e,
	0xdf21b31e,0xd85fb320,0xdf23b320,0xd85fb322,0xdf23b322,0xd85fb324,0xdf25b324,0xd85fb326,
	0xdf25b326,0xd85fb328,0xdf27b328,0xd85fb32a,0xdf27b32a,0xd85fb32c,0xdf29b32c,0xd85fb32e,
	0xdf29b32e,0xd85fb330,0xdf2bb330,0xd85fb332,0xdf2bb332,0xd85fb334,0xdf2db334,0xd85fb336,
	0xdf2db336,0xd85fb338,0xdf2fb338,0xd85fb33a,0xdf2fb33a,0xd85fb33c,0xdf31b33c,0xd85fb33e,
	0xdf31b33e,0xd85fb340,0xdf33b340,0xd85fb342,0xdf33b342,0xd85fb344,0xdf35b344,0xd85fb346,
	0xdf35b346,0xd85fb348,0xdf37b348,0xd85fb34a,0xdf37b34a,0xd85fb34c,0xdf39b34c,0xd85fb34e,
	0xdf39b34e,0xd85fb350,0xdf3bb350,0xd85fb352,0xdf3bb352,0xd85fb354,0xdf3db354,0xd85fb356,
	0xdf3db356,0xd85fb358,0xdf3fb358,0xd85fb35a,0xdf3fb35a,0xd85fb35c,0xdf41b35c,0xd85fb35e,
	0xdf41b35e,0xd85fb360,0xdf43b360,0xd85fb362,0xdf43b362,0xd85fb364,0xdf45b364,0xd85fb366,
	0xdf45b366,0xd85fb368,0xdf47b368,0xd85fb36a,0xdf47b36a,0xd85fb36c,0xdf49b36c,0xd85fb36e,
	0xdf49b36e,0xd85fb370,0xdf4bb370,0xd85fb372,0xdf4bb372,0xd85fb374,0xdf4db374,0xd85fb376,
	0xdf4db376,0xd85fb378,0xdf4fb378,0xd85fb37a,0xdf4fb37a,0xd85fb37c,0xdf51b37c,0xd85fb37e,
	0xdf51b37e,0xd85fb380,0xdf53b380,0xd85fb382,0xdf53b382,0xd85fb384,0xdf55b384,0xd85fb386,
	0xdf55b386,0xd85fb388,0xdf57b388,0xd85fb38a,0xdf57b38a,0xd85fb38c,0xdf59b38c,0xd85fb38e,
	0xdf59b38e,0xd85fb390,0xdf5bb390,0xd85fb392,0xdf5bb392,0xd85fb394,0xdf5db394,0xd85fb396,
	0xdf5db396,0xd85fb398,0xdf5fb398,0xd85fb39a,0xdf5fb39a,0xd85fb39c,0xdf61b39c,0xd85fb39e,
	0xdf61b39e,0xd85fb3a0,0xdf63b3a0,0xd85fb3a2,0xdf63b3a2,0xd85fb3a4,0xdf65b3a4,0xd85fb3a6,
	0xdf65b3a6,0xd85fb3a8,0xdf67b3a8,0xd85fb3aa,0xdf67b3aa,0xd85fb3ac,0xdf69b3ac,0xd85fb3ae,
	0xdf69b3ae,0xd85fb3b0,0xdf6bb3b0,0xd85fb3b2,0xdf6bb3b2,0xd85fb3b4,0xdf6db3b4,0xd85fb3b6,
	0xdf6db3b6,0xd85fb3b8,0xdf6fb3b8,0xd85fb3ba,0xdf6fb3ba,0xd85fb3bc,0xdf71b3bc,0xd85fb3be,
	0xdf71b3be,0xd85fb3c0,0xdf73b3c0,0xd85fb3c2,0xdf73b3c2,0xd85fb3c4,0xdf75b3c4,0xd85fb3c6,
	0xdf75b3c6,0xd85fb3c8,0xdf77b3c8,0xd85fb3ca,0xdf77b3ca,0xd85fb3cc,0xdf79b3cc,0xd85fb3ce,
	0xdf79b3ce,0xd85fb3d0,0xdf7bb3d0,0xd85fb3d2,0xdf7bb3d2,0xd85fb3d4,0xdf7db3d4,0xd85fb3d6,
	0xdf7db3d6,0xd85fb3d8,0xdf7fb3d8,0xd85fb3da,0xdf7fb3da,0xd85fb3dc,0xdf81b3dc,0xd85fb3de,
	0xdf81b3de,0xd85fb3e0,0xdf83b3e0,0xd85fb3e2,0xdf83b3e2,0xd85fb3e4,0xdf85b3e4,0xd85fb3e6,
	0xdf85b3e6,0xd85fb3e8,0xdf87b3e8,0xd85fb3ea,0xdf87b3ea,0xd85fb3ec,0xdf89b3ec,0xd85fb3ee,
	0xdf89b3ee,0xd85fb3f0,0xdf8bb3f0,0xd85fb3f2,0xdf8bb3f2,0xd85fb3f4,0xdf8db3f4,0xd85fb3f6,
	0xdf8db3f6,0xd85fb3f8,0xdf8fb3f8,0xd85fb3fa,0xdf8fb3fa,0xd85fb3fc,0xdf91b3fc,0xd85fb3fe,
	0xdf91b3fe,0xd85fb400,0xdf93b400,0xd85fb402,0xdf93b402,0xd85fb404,0xdf95b404,0xd85fb406,
	0xdf95b406,0xd85fb408,0xdf97b408,0xd85fb40a,0xdf97b40a,0xd85fb40c,0xdf99b40c,0xd85fb40e,
	0xdf99b40e,0xd85fb410,0xdf9bb410,0xd85fb412,0xdf9bb412,0xd85fb414,0xdf9db414,0xd85fb416,
	0xdf9db416,0xd85fb418,0xdf9fb418,0xd85fb41a,0xdf9fb41a,0xd85fb41c,0xdfa1b41c,0xd85fb41e,
	0xdfa1b41e,0xd85fb420,0xdfa3b420,0xd85fb422,0xdfa3b422,0xd85fb424,0xdfa5b424,0xd85fb426,
	0xdfa5b426,0xd85fb428,0xdfa7b428,0xd85fb42a,0xdfa7b42a,0xd85fb42c,0xdfa9b42c,0xd85fb42e,
	0xdfa9b42e,0xd85fb430,0xdfabb430,0xd85fb432,0xdfabb432,0xd85fb434,0xdfadb434,0xd85fb436,
	0xdfadb436,0xd85fb438,0xdfafb438,0xd85fb43a,0xdfafb43a,0xd85fb43c,0xdfb1b43c,0xd85fb43e,
	0xdfb1b43e,0xd85fb440,0xdfb3b440,0xd85fb442,0xdfb3b442,0xd85fb444,0xdfb5b444,0xd85fb446,
	0xdfb5b446,0xd85fb448,0xdfb7b448,0xd85fb44a,0xdfb7b44a,0xd85fb44c,0xdfb9b44c,0xd85fb44e,
	0xdfb9b44e,0xd85fb450,0xdfbbb450,0xd85fb452,0xdfbbb452,0xd85fb454,0xdfbdb454,0xd85fb456,
	0xdfbdb456,0xd85fb458,0xdfbfb458,0xd85fb45a,0xdfbfb45a,0xd85fb45c,0xdfc1b45c,0xd85fb45e,
	0xdfc1b45e,0xd85fb460,0xdfc3b460,0xd85fb462,0xdfc3b462,0xd85fb464,0xdfc5b464,0xd85fb466,
	0xdfc5b466,0xd85fb468,0xdfc7b468,0xd85fb46a,0xdfc7b46a,0xd85fb46c,0xdfc9b46c,0xd85fb46e,
	0xdfc9b46e,0xd85fb470,0xdfcbb470,0xd85fb472,0xdfcbb472,0xd85fb474,0xdfcdb474,0xd85fb476,
	0xdfcdb476,0xd85fb478,0xdfcfb478,0xd85fb47a,0xdfcfb47a,0xd85fb47c,0xdfd1b47c,0xd85fb47e,
	0xdfd1b47e,0xd85fb480,0xdfd3b480,0xd85fb482,0xdfd3b482,0xd85fb484,0xdfd5b484,0xd85fb486,
	0xdfd5b486,0xd85fb488,0xdfd7b488,0xd85fb48a,0xdfd7b48a,0xd85fb48c,0xdfd9b48c,0xd85fb48e,
	0xdfd9b48e,0xd85fb490,0xdfdbb490,0xd85fb492,0xdfdbb492,0xd85fb494,0xdfddb494,0xd85fb496,
	0xdfddb496,0xd85fb498,0xdfdfb498,0xd85fb49a,0xdfdfb49a,0xd85fb49c,0xdfe1b49c,0xd85fb49e,
	0xdfe1b49e,0xd85fb4a0,0xdfe3b4a0,0xd85fb4a2,0xdfe3b4a2,0xd85fb4a4,0xdfe5b4a4,0xd85fb4a6,
	0xdfe5b4a6,0xd85fb4a8,0xdfe7b4a8,0xd85fb4aa,0xdfe7b4aa,0xd85fb4ac,0xdfe9b4ac,0xd85fb4ae,
	0xdfe9b4ae,0xd85fb4b0,0xdfebb4b0,0xd85fb4b2,0xdfebb4b2,0xd85fb4b4,0xdfedb4b4,0xd85fb4b6,
	0xdfedb4b6,0xd85fb4b8,0xdfefb4b8,0xd85fb4ba,0xdfefb4ba,0xd85fb4bc,0xdff1b4bc,0xd85fb4be,
	0xdff1b4be,0xd85fb4c0,0xdff3b4c0,0xd85fb4c2,0xdff3b4c2,0xd85fb4c4,0xdff5b4c4,0xd85fb4c6,
	0xdff5b4c6,0xd85fb4c8,0xdff7b4c8,0xd85fb4ca,0xdff7b4ca,0xd85fb4cc,0xdff9b4cc,0xd85fb4ce,
	0xdff9b4ce,0xd85fb4d0,0xdffbb4d0,0xd85fb4d2,0xdffbb4d2,0xd85fb4d4,0xdffdb4d4,0xd85fb4d6,
	0xdffdb4d6,0xd85fb4d8,0xdfffb4d8,0xd85fb4da,0xdfffb4da,0xd85fb4dc,0xdc01b4dc,0xd85fb4de,
	0xdc01b4de,0xd85fb4e0,0xdc03b4e0,0xd85fb4e2,0xdc03b4e2,0xd85fb4e4,0xdc05b4e4,0xd85fb4e6,
	0xdc05b4e6,0xd85fb4e8,0xdc07b4e8,0xd85fb4ea,0xdc07b4ea,0xd85fb4ec,0xdc09b4ec,0xd85fb4ee,
	0xdc09b4ee,0xd85fb4f0,0xdc0bb4f0,0xd85fb4f2,0xdc0bb4f2,0xd85fb4f4,0xdc0db4f4,0xd85fb4f6,
	0xdc0db4f6,0xd85fb4f8,0xdc0fb4f8,0xd85fb4fa,0xdc0fb4fa,0xd85fb4fc,0xdc11b4fc,0xd85fb4fe,
	0xdc11b4fe,0xd85fb500,0xdc13b500,0xd85fb502,0xdc13b502,0xd85fb504,0xdc15b504,0xd85fb506,
	0xdc15b506,0xd85fb508,0xdc17b508,0xd85fb50a,0xdc17b50a,0xd85fb50c,0xdc19b50c,0xd85fb50e,
	0xdc19b50e,0xd85fb510,0xdc1bb510,0xd85fb512,0xdc1bb512,0xd85fb514,0xdc1db514,0xd85fb516,
	0xdc1db516,0xd85fb518,0xdc1fb518,0xd85fb51a,0xdc1fb51a,0xd85fb51c,0xdc21b51c,0xd85fb51e,
	0xdc21b51e,0xd85fb520,0xdc23b520,0xd85fb522,0xdc23b522,0xd85fb524,0xdc25b524,0xd85fb526,
	0xdc25b526,0xd85fb528,0xdc27b528,0xd85fb52a,0xdc27b52a,0xd85fb52c,0xdc29b52c,0xd85fb52e,
	0xdc29b52e,0xd85fb530,0xdc2bb530,0xd85fb532,0xdc2bb532,0xd85fb534,0xdc2db534,0xd85fb536,
	0xdc2db536,0xd85fb538,0xdc2fb538,0xd85fb53a,0xdc2fb53a,0xd85fb53c,0xdc31b53c,0xd85fb53e,
	0xdc31b53e,0xd85fb540,0xdc33b540,0xd85fb542,0xdc33b542,0xd85fb544,0xdc35b544,0xd85fb546,
	0xdc35b546,0xd85fb548,0xdc37b548,0xd85fb54a,0xdc37b54a,0xd85fb54c,0xdc39b54c,0xd85fb54e,
	0xdc39b54e,0xd85fb550,0xdc3bb550,0xd85fb552,0xdc3bb552,0xd85fb554,0xdc3db554,0xd85fb556,
	0xdc3db556,0xd85fb558,0xdc3fb558,0xd85fb55a,0xdc3fb55a,0xd85fb55c,0xdc41b55c,0xd85fb55e,
	0xdc41b55e,0xd85fb560,0xdc43b560,0xd85fb562,0xdc43b562,0xd85fb564,0xdc45b564,0xd85fb566,
	0xdc45b566,0xd85fb568,0xdc47b568,0xd85fb56a,0xdc47b56a,0xd85fb56c,0xdc49b56c,0xd85fb56e,
	0xdc49b56e,0xd85fb570,0xdc4bb570,0xd85fb572,0xdc4bb572,0xd85fb574,0xdc4db574,0xd85fb576,
	0xdc4db576,0xd85fb578,0xdc4fb578,0xd85fb57a,0xdc4fb57a,0xd85fb57c,0xdc51b57c,0xd85fb57e,
	0xdc51b57e,0xd85fb580,0xdc53b580,0xd85fb582,0xdc53b582,0xd85fb584,0xdc55b584,0xd85fb586,
	0xdc55b586,0xd85fb588,0xdc57b588,0xd85fb58a,0xdc57b58a,0xd85fb58c,0xdc59b58c,0xd85fb58e,
	0xdc59b58e,0xd85fb590,0xdc5bb590,0xd85fb592,0xdc5bb592,0xd85fb594,0xdc5db594,0xd85fb596,
	0xdc5db596,0xd85fb598,0xdc5fb598,0xd85fb59a,0xdc5fb59a,0xd85fb59c,0xdc61b59c,0xd85fb59e,
	0xdc61b59e,0xd85fb5a0,0xdc63b5a0,0xd85fb5a2,0xdc63b5a2,0xd85fb5a4,0xdc65b5a4,0xd85fb5a6,
	0xdc65b5a6,0xd85fb5a8,0xdc67b5a8,0xd85fb5aa,0xdc67b5aa,0xd85fb5ac,0xdc69b5ac,0xd85fb5ae,
	0xdc69b5ae,0xd85fb5b0,0xdc6bb5b0,0xd85fb5b2,0xdc6bb5b2,0xd85fb5b4,0xdc6db5b4,0xd85fb5b6,
	0xdc6db5b6,0xd85fb5b8,0xdc6fb5b8,0xd85fb5ba,0xdc6fb5ba,0xd85fb5bc,0xdc71b5bc,0xd85fb5be,
	0xdc71b5be,0xd85fb5c0,0xdc73b5c0,0xd85fb5c2,0xdc73b5c2,0xd85fb5c4,0xdc75b5c4,0xd85fb5c6,
	0xdc75b5c6,0xd85fb5c8,0xdc77b5c8,0xd85fb5ca,0xdc77b5ca,0xd85fb5cc,0xdc79b5cc,0xd85fb5ce,
	0xdc79b5ce,0xd85fb5d0,0xdc7bb5d0,0xd85fb5d2,0xdc7bb5d2,0xd85fb5d4,0xdc7db5d4,0xd85fb5d6,
	0xdc7db5d6,0xd85fb5d8,0xdc7fb5d8,0xd85fb5da,0xdc7fb5da,0xd85fb5dc,0xdc81b5dc,0xd85fb5de,
	0xdc81b5de,0xd85fb5e0,0xdc83b5e0,0xd85fb5e2,0xdc83b5e2,0xd85fb5e4,0xdc85b5e4,0xd85fb5e6,
	0xdc85b5e6,0xd85fb5e8,0xdc87b5e8,0xd85fb5ea,0xdc87b5ea,0xd85fb5ec,0xdc89b5ec,0xd85fb5ee,
	0xdc89b5ee,0xd85fb5f0,0xdc8bb5f0,0xd85fb5f2,0xdc8bb5f2,0xd85fb5f4,0xdc8db5f4,0xd85fb5f6,
	0xdc8db5f6,0xd85fb5f8,0xdc8fb5f8,0xd85fb5fa,0xdc8fb5fa,0xd85fb5fc,0xdc91b5fc,0xd85fb5fe,
	0xdc91b5fe,0xd85fb600,0xdc93b600,0xd85fb602,0xdc93b602,0xd85fb604,0xdc95b604,0xd85fb606,
	0xdc95b606,0xd85fb608,0xdc97b608,0xd85fb60a,0xdc97b60a,0xd85fb60c,0xdc99b60c,0xd85fb60e,
	0xdc99b60e,0xd85fb610,0xdc9bb610,0xd85fb612,0xdc9bb612,0xd85fb614,0xdc9db614,0xd85fb616,
	0xdc9db616,0xd85fb618,0xdc9fb618,0xd85fb61a,0xdc9fb61a,0xd85fb61c,0xdca1b61c,0xd85fb61e,
	0xdca1b61e,0xd85fb620,0xdca3b620,0xd85fb622,0xdca3b622,0xd85fb624,0xdca5b624,0xd85fb626,
	0xdca5b626,0xd85fb628,0xdca7b628,0xd85fb62a,0xdca7b62a,0xd85fb62c,0xdca9b62c,0xd85fb62e,
	0xdca9b62e,0xd85fb630,0xdcabb630,0xd85fb632,0xdcabb632,0xd85fb634,0xdcadb634,0xd85fb636,
	0xdcadb636,0xd85fb638,0xdcafb638,0xd85fb63a,0xdcafb63a,0xd85fb63c,0xdcb1b63c,0xd85fb63e,
	0xdcb1b63e,0xd85fb640,0xdcb3b640,0xd85fb642,0xdcb3b642,0xd85fb644,0xdcb5b644,0xd85fb646,
	0xdcb5b646,0xd85fb648,0xdcb7b648,0xd85fb64a,0xdcb7b64a,0xd85fb64c,0xdcb9b64c,0xd85fb64e,
	0xdcb9b64e,0xd85fb650,0xdcbbb650,0xd85fb652,0xdcbbb652,0xd85fb654,0xdcbdb654,0xd85fb656,
	0xdcbdb656,0xd85fb658,0xdcbfb658,0xd85fb65a,0xdcbfb65a,0xd85fb65c,0xdcc1b65c,0xd85fb65e,
	0xdcc1b65e,0xd85fb660,0xdcc3b660,0xd85fb662,0xdcc3b662,0xd85fb664,0xdcc5b664,0xd85fb666,
	0xdcc5b666,0xd85fb668,0xdcc7b668,0xd85fb66a,0xdcc7b66a,0xd85fb66c,0xdcc9b66c,0xd85fb66e,
	0xdcc9b66e,0xd85fb670,0xdccbb670,0xd85fb672,0xdccbb672,0xd85fb674,0xdccdb674,0xd85fb676,
	0xdccdb676,0xd85fb678,0xdccfb678,0xd85fb67a,0xdccfb67a,0xd85fb67c,0xdcd1b67c,0xd85fb67e,
	0xdcd1b67e,0xd85fb680,0xdcd3b680,0xd85fb682,0xdcd3b682,0xd85fb684,0xdcd5b684,0xd85fb686,
	0xdcd5b686,0xd85fb688,0xdcd7b688,0xd85fb68a,0xdcd7b68a,0xd85fb68c,0xdcd9b68c,0xd85fb68e,
	0xdcd9b68e,0xd85fb690,0xdcdbb690,0xd85fb692,0xdcdbb692,0xd85fb694,0xdcddb694,0xd85fb696,
	0xdcddb696,0xd85fb698,0xdcdfb698,0xd85fb69a,0xdcdfb69a,0xd85fb69c,0xdce1b69c,0xd85fb69e,
	0xdce1b69e,0xd85fb6a0,0xdce3b6a0,0xd85fb6a2,0xdce3b6a2,0xd85fb6a4,0xdce5b6a4,0xd85fb6a6,
	0xdce5b6a6,0xd85fb6a8,0xdce7b6a8,0xd85fb6aa,0xdce7b6aa,0xd85fb6ac,0xdce9b6ac,0xd85fb6ae,
	0xdce9b6ae,0xd85fb6b0,0xdcebb6b0,0xd85fb6b2,0xdcebb6b2,0xd85fb6b4,0xdcedb6b4,0xd85fb6b6,
	0xdcedb6b6,0xd85fb6b8,0xdcefb6b8,0xd85fb6ba,0xdcefb6ba,0xd85fb6bc,0xdcf1b6bc,0xd85fb6be,
	0xdcf1b6be,0xd85fb6c0,0xdcf3b6c0,0xd85fb6c2,0xdcf3b6c2,0xd85fb6c4,0xdcf5b6c4,0xd85fb6c6,
	0xdcf5b6c6,0xd85fb6c8,0xdcf7b6c8,0xd85fb6ca,0xdcf7b6ca,0xd85fb6cc,0xdcf9b6cc,0xd85fb6ce,
	0xdcf9b6ce,0xd85fb6d0,0xdcfbb6d0,0xd85fb6d2,0xdcfbb6d2,0xd85fb6d4,0xdcfdb6d4,0xd85fb6d6,
	0xdcfdb6d6,0xd85fb6d8,0xdcffb6d8,0xd85fb6da,0xdcffb6da,0xd85fb6dc,0xdd01b6dc,0xd85fb6de,
	0xdd01b6de,0xd85fb6e0,0xdd03b6e0,0xd85fb6e2,0xdd03b6e2,0xd85fb6e4,0xdd05b6e4,0xd85fb6e6,
	0xdd05b6e6,0xd85fb6e8,0xdd07b6e8,0xd85fb6ea,0xdd07b6ea,0xd85fb6ec,0xdd09b6ec,0xd85fb6ee,
	0xdd09b6ee,0xd85fb6f0,0xdd0bb6f0,0xd85fb6f2,0xdd0bb6f2,0xd85fb6f4,0xdd0db6f4,0xd85fb6f6,
	0xdd0db6f6,0xd85fb6f8,0xdd0fb6f8,0xd85fb6fa,0xdd0fb6fa,0xd85fb6fc,0xdd11b6fc,0xd85fb6fe,
	0xdd11b6fe,0xd85fb700,0xdd13b700,0xd85fb702,0xdd13b702,0xd85fb704,0xdd15b704,0xd85fb706,
	0xdd15b706,0xd85fb708,0xdd17b708,0xd85fb70a,0xdd17b70a,0xd85fb70c,0xdd19b70c,0xd85fb70e,
	0xdd19b70e,0xd85fb710,0xdd1bb710,0xd85fb712,0xdd1bb712,0xd85fb714,0xdd1db714,0xd85fb716,
	0xdd1db716,0xd85fb718,0xdd1fb718,0xd85fb71a,0xdd1fb71a,0xd85fb71c,0xdd21b71c,0xd85fb71e,
	0xdd21b71e,0xd85fb720,0xdd23b720,0xd85fb722,0xdd23b722,0xd85fb724,0xdd25b724,0xd85fb726,
	0xdd25b726,0xd85fb728,0xdd27b728,0xd85fb72a,0xdd27b72a,0xd85fb72c,0xdd29b72c,0xd85fb72e,
	0xdd29b72e,0xd85fb730,0xdd2bb730,0xd85fb732,0xdd2bb732,0xd85fb734,0xdd2db734,0xd85fb736,
	0xdd2db736,0xd85fb738,0xdd2fb738,0xd85fb73a,0xdd2fb73a,0xd85fb73c,0xdd31b73c,0xd85fb73e,
	0xdd31b73e,0xd85fb740,0xdd33b740,0xd85fb742,0xdd33b742,0xd85fb744,0xdd35b744,0xd85fb746,
	0xdd35b746,0xd85fb748,0xdd37b748,0xd85fb74a,0xdd37b74a,0xd85fb74c,0xdd39b74c,0xd85fb74e,
	0xdd39b74e,0xd85fb750,0xdd3bb750,0xd85fb752,0xdd3bb752,0xd85fb754,0xdd3db754,0xd85fb756,
	0xdd3db756,0xd85fb758,0xdd3fb758,0xd85fb75a,0xdd3fb75a,0xd85fb75c,0xdd41b75c,0xd85fb75e,
	0xdd41b75e,0xd85fb760,0xdd43b760,0xd85fb762,0xdd43b762,0xd85fb764,0xdd45b764,0xd85fb766,
	0xdd45b766,0xd85fb768,0xdd47b768,0xd85fb76a,0xdd47b76a,0xd85fb76c,0xdd49b76c,0xd85fb76e,
	0xdd49b76e,0xd85fb770,0xdd4bb770,0xd85fb772,0xdd4bb772,0xd85fb774,0xdd4db774,0xd85fb776,
	0xdd4db776,0xd85fb778,0xdd4fb778,0xd85fb77a,0xdd4fb77a,0xd85fb77c,0xdd51b77c,0xd85fb77e,
	0xdd51b77e,0xd85fb780,0xdd53b780,0xd85fb782,0xdd53b782,0xd85fb784,0xdd55b784,0xd85fb786,
	0xdd55b786,0xd85fb788,0xdd57b788,0xd85fb78a,0xdd57b78a,0xd85fb78c,0xdd59b78c,0xd85fb78e,
	0xdd59b78e,0xd85fb790,0xdd5bb790,0xd85fb792,0xdd5bb792,0xd85fb794,0xdd5db794,0xd85fb796,
	0xdd5db796,0xd85fb798,0xdd5fb798,0xd85fb79a,0xdd5fb79a,0xd85fb79c,0xdd61b79c,0xd85fb79e,
	0xdd61b79e,0xd85fb7a0,0xdd63b7a0,0xd85fb7a2,0xdd63b7a2,0xd85fb7a4,0xdd65b7a4,0xd85fb7a6,
	0xdd65b7a6,0xd85fb7a8,0xdd67b7a8,0xd85fb7aa,0xdd67b7aa,0xd85fb7ac,0xdd69b7ac,0xd85fb7ae,
	0xdd69b7ae,0xd85fb7b0,0xdd6bb7b0,0xd85fb7b2,0xdd6bb7b2,0xd85fb7b4,0xdd6db7b4,0xd85fb7b6,
	0xdd6db7b6,0xd85fb7b8,0xdd6fb7b8,0xd85fb7ba,0xdd6fb7ba,0xd85fb7bc,0xdd71b7bc,0xd85fb7be,
	0xdd71b7be,0xd85fb7c0,0xdd73b7c0,0xd85fb7c2,0xdd73b7c2,0xd85fb7c4,0xdd75b7c4,0xd85fb7c6,
	0xdd75b7c6,0xd85fb7c8,0xdd77b7c8,0xd85fb7ca,0xdd77b7ca,0xd85fb7cc,0xdd79b7cc,0xd85fb7ce,
	0xdd79b7ce,0xd85fb7d0,0xdd7bb7d0,0xd85fb7d2,0xdd7bb7d2,0xd85fb7d4,0xdd7db7d4,0xd85fb7d6,
	0xdd7db7d6,0xd85fb7d8,0xdd7fb7d8,0xd85fb7da,0xdd7fb7da,0xd85fb7dc,0xdd81b7dc,0xd85fb7de,
	0xdd81b7de,0xd85fb7e0,0xdd83b7e0,0xd85fb7e2,0xdd83b7e2,0xd85fb7e4,0xdd85b7e4,0xd85fb7e6,
	0xdd85b7e6,0xd85fb7e8,0xdd87b7e8,0xd85fb7ea,0xdd87b7ea,0xd85fb7ec,0xdd89b7ec,0xd85fb7ee,
	0xdd89b7ee,0xd85fb7f0,0xdd8bb7f0,0xd85fb7f2,0xdd8bb7f2,0xd85fb7f4,0xdd8db7f4,0xd85fb7f6,
	0xdd8db7f6,0xd85fb7f8,0xdd8fb7f8,0xd85fb7fa,0xdd8fb7fa,0xd85fb7fc,0xdd91b7fc,0xd85fb7fe,
	0xdd91b7fe,0xd85fb800,0xdd93b800,0xd85fb802,0xdd93b802,0xd85fb804,0xdd95b804,0xd85fb806,
	0xdd95b806,0xd85fb808,0xdd97b808,0xd85fb80a,0xdd97b80a,0xd85fb80c,0xdd99b80c,0xd85fb80e,
	0xdd99b80e,0xd85fb810,0xdd9bb810,0xd85fb812,0xdd9bb812,0xd85fb814,0xdd9db814,0xd85fb816,
	0xdd9db816,0xd85fb818,0xdd9fb818,0xd85fb81a,0xdd9fb81a,0xd85fb81c,0xdda1b81c,0xd85fb81e,
	0xdda1b81e,0xd85fb820,0xdda3b820,0xd85fb822,0xdda3b822,0xd85fb824,0xdda5b824,0xd85fb826,
	0xdda5b826,0xd85fb828,0xdda7b828,0xd85fb82a,0xdda7b82a,0xd85fb82c,0xdda9b82c,0xd85fb82e,
	0xdda9b82e,0xd85fb830,0xddabb830,0xd85fb832,0xddabb832,0xd85fb834,0xddadb834,0xd85fb836,
	0xddadb836,0xd85fb838,0xddafb838,0xd85fb83a,0xddafb83a,0xd85fb83c,0xddb1b83c,0xd85fb83e,
	0xddb1b83e,0xd85fb840,0xddb3b840,0xd85fb842,0xddb3b842,0xd85fb844,0xddb5b844,0xd85fb846,
	0xddb5b846,0xd85fb848,0xddb7b848,0xd85fb84a,0xddb7b84a,0xd85fb84c,0xddb9b84c,0xd85fb84e,
	0xddb9b84e,0xd85fb850,0xddbbb850,0xd85fb852,0xddbbb852,0xd85fb854,0xddbdb854,0xd85fb856,
	0xddbdb856,0xd85fb858,0xddbfb858,0xd85fb85a,0xddbfb85a,0xd85fb85c,0xddc1b85c,0xd85fb85e,
	0xddc1b85e,0xd85fb860,0xddc3b860,0xd85fb862,0xddc3b862,0xd85fb864,0xddc5b864,0xd85fb866,
	0xddc5b866,0xd85fb868,0xddc7b868,0xd85fb86a,0xddc7b86a,0xd85fb86c,0xddc9b86c,0xd85fb86e,
	0xddc9b86e,0xd85fb870,0xddcbb870,0xd85fb872,0xddcbb872,0xd85fb874,0xddcdb874,0xd85fb876,
	0xddcdb876,0xd85fb878,0xddcfb878,0xd85fb87a,0xddcfb87a,0xd85fb87c,0xddd1b87c,0xd85fb87e,
	0xddd1b87e,0xd85fb880,0xddd3b880,0xd85fb882,0xddd3b882,0xd85fb884,0xddd5b884,0xd85fb886,
	0xddd5b886,0xd85fb888,0xddd7b888,0xd85fb88a,0xddd7b88a,0xd85fb88c,0xddd9b88c,0xd85fb88e,
	0xddd9b88e,0xd85fb890,0xdddbb890,0xd85fb892,0xdddbb892,0xd85fb894,0xddddb894,0xd85fb896,
	0xddddb896,0xd85fb898,0xdddfb898,0xd85fb89a,0xdddfb89a,0xd85fb89c,0xdde1b89c,0xd85fb89e,
	0xdde1b89e,0xd85fb8a0,0xdde3b8a0,0xd85fb8a2,0xdde3b8a2,0xd85fb8a4,0xdde5b8a4,0xd85fb8a6,
	0xdde5b8a6,0xd85fb8a8,0xdde7b8a8,0xd85fb8aa,0xdde7b8aa,0xd85fb8ac,0xdde9b8ac,0xd85fb8ae,
	0xdde9b8ae,0xd85fb8b0,0xddebb8b0,0xd85fb8b2,0xddebb8b2,0xd85fb8b4,0xddedb8b4,0xd85fb8b6,
	0xddedb8b6,0xd85fb8b8,0xddefb8b8,0xd85fb8ba,0xddefb8ba,0xd85fb8bc,0xddf1b8bc,0xd85fb8be,
	0xddf1b8be,0xd85fb8c0,0xddf3b8c0,0xd85fb8c2,0xddf3b8c2,0xd85fb8c4,0xddf5b8c4,0xd85fb8c6,
	0xddf5b8c6,0xd85fb8c8,0xddf7b8c8,0xd85fb8ca,0xddf7b8ca,0xd85fb8cc,0xddf9b8cc,0xd85fb8ce,
	0xddf9b8ce,0xd85fb8d0,0xddfbb8d0,0xd85fb8d2,0xddfbb8d2,0xd85fb8d4,0xddfdb8d4,0xd85fb8d6,
	0xddfdb8d6,0xd85fb8d8,0xddffb8d8,0xd85fb8da,0xddffb8da,0xd85fb8dc,0xde01b8dc,0xd85fb8de,
	0xde01b8de,0xd85fb8e0,0xde03b8e0,0xd85fb8e2,0xde03b8e2,0xd85fb8e4,0xde05b8e4,0xd85fb8e6,
	0xde05b8e6,0xd85fb8e8,0xde07b8e8,0xd85fb8ea,0xde07b8ea,0xd85fb8ec,0xde09b8ec,0xd85fb8ee,
	0xde09b8ee,0xd85fb8f0,0xde0bb8f0,0xd85fb8f2,0xde0bb8f2,0xd85fb8f4,0xde0db8f4,0xd85fb8f6,
	0xde0db8f6,0xd85fb8f8,0xde0fb8f8,0xd85fb8fa,0xde0fb8fa,0xd85fb8fc,0xde11b8fc,0xd85fb8fe,
	0xde11b8fe,0xd85fb900,0xde13b900,0xd85fb902,0xde13b902,0xd85fb904,0xde15b904,0xd85fb906,
	0xde15b906,0xd85fb908,0xde17b908,0xd85fb90a,0xde17b90a,0xd85fb90c,0xde19b90c,0xd85fb90e,
	0xde19b90e,0xd85fb910,0xde1bb910,0xd85fb912,0xde1bb912,0xd85fb914,0xde1db914,0xd85fb916,
	0xde1db916,0xd85fb918,0xde1fb918,0xd85fb91a,0xde1fb91a,0xd85fb91c,0xde21b91c,0xd85fb91e,
	0xde21b91e,0xd85fb920,0xde23b920,0xd85fb922,0xde23b922,0xd85fb924,0xde25b924,0xd85fb926,
	0xde25b926,0xd85fb928,0xde27b928,0xd85fb92a,0xde27b92a,0xd85fb92c,0xde29b92c,0xd85fb92e,
	0xde29b92e,0xd85fb930,0xde2bb930,0xd85fb932,0xde2bb932,0xd85fb934,0xde2db934,0xd85fb936,
	0xde2db936,0xd85fb938,0xde2fb938,0xd85fb93a,0xde2fb93a,0xd85fb93c,0xde31b93c,0xd85fb93e,
	0xde31b93e,0xd85fb940,0xde33b940,0xd85fb942,0xde33b942,0xd85fb944,0xde35b944,0xd85fb946,
	0xde35b946,0xd85fb948,0xde37b948,0xd85fb94a,0xde37b94a,0xd85fb94c,0xde39b94c,0xd85fb94e,
	0xde39b94e,0xd85fb950,0xde3bb950,0xd85fb952,0xde3bb952,0xd85fb954,0xde3db954,0xd85fb956,
	0xde3db956,0xd85fb958,0xde3fb958,0xd85fb95a,0xde3fb95a,0xd85fb95c,0xde41b95c,0xd85fb95e,
	0xde41b95e,0xd85fb960,0xde43b960,0xd85fb962,0xde43b962,0xd85fb964,0xde45b964,0xd85fb966,
	0xde45b966,0xd85fb968,0xde47b968,0xd85fb96a,0xde47b96a,0xd85fb96c,0xde49b96c,0xd85fb96e,
	0xde49b96e,0xd85fb970,0xde4bb970,0xd85fb972,0xde4bb972,0xd85fb974,0xde4db974,0xd85fb976,
	0xde4db976,0xd85fb978,0xde4fb978,0xd85fb97a,0xde4fb97a,0xd85fb97c,0xde51b97c,0xd85fb97e,
	0xde51b97e,0xd85fb980,0xde53b980,0xd85fb982,0xde53b982,0xd85fb984,0xde55b984,0xd85fb986,
	0xde55b986,0xd85fb988,0xde57b988,0xd85fb98a,0xde57b98a,0xd85fb98c,0xde59b98c,0xd85fb98e,
	0xde59b98e,0xd85fb990,0xde5bb990,0xd85fb992,0xde5bb992,0xd85fb994,0xde5db994,0xd85fb996,
	0xde5db996,0xd85fb998,0xde5fb998,0xd85fb99a,0xde5fb99a,0xd85fb99c,0xde61b99c,0xd85fb99e,
	0xde61b99e,0xd85fb9a0,0xde63b9a0,0xd85fb9a2,0xde63b9a2,0xd85fb9a4,0xde65b9a4,0xd85fb9a6,
	0xde65b9a6,0xd85fb9a8,0xde67b9a8,0xd85fb9aa,0xde67b9aa,0xd85fb9ac,0xde69b9ac,0xd85fb9ae,
	0xde69b9ae,0xd85fb9b0,0xde6bb9b0,0xd85fb9b2,0xde6bb9b2,0xd85fb9b4,0xde6db9b4,0xd85fb9b6,
	0xde6db9b6,0xd85fb9b8,0xde6fb9b8,0xd85fb9ba,0xde6fb9ba,0xd85fb9bc,0xde71b9bc,0xd85fb9be,
	0xde71b9be,0xd85fb9c0,0xde73b9c0,0xd85fb9c2,0xde73b9c2,0xd85fb9c4,0xde75b9c4,0xd85fb9c6,
	0xde75b9c6,0xd85fb9c8,0xde77b9c8,0xd85fb9ca,0xde77b9ca,0xd85fb9cc,0xde79b9cc,0xd85fb9ce,
	0xde79b9ce,0xd85fb9d0,0xde7bb9d0,0xd85fb9d2,0xde7bb9d2,0xd85fb9d4,0xde7db9d4,0xd85fb9d6,
	0xde7db9d6,0xd85fb9d8,0xde7fb9d8,0xd85fb9da,0xde7fb9da,0xd85fb9dc,0xde81b9dc,0xd85fb9de,
	0xde81b9de,0xd85fb9e0,0xde83b9e0,0xd85fb9e2,0xde83b9e2,0xd85fb9e4,0xde85b9e4,0xd85fb9e6,
	0xde85b9e6,0xd85fb9e8,0xde87b9e8,0xd85fb9ea,0xde87b9ea,0xd85fb9ec,0xde89b9ec,0xd85fb9ee,
	0xde89b9ee,0xd85fb9f0,0xde8bb9f0,0xd85fb9f2,0xde8bb9f2,0xd85fb9f4,0xde8db9f4,0xd85fb9f6,
	0xde8db9f6,0xd85fb9f8,0xde8fb9f8,0xd85fb9fa,0xde8fb9fa,0xd85fb9fc,0xde91b9fc,0xd85fb9fe,
	0xde91b9fe,0xd85fba00,0xde93ba00,0xd85fba02,0xde93ba02,0xd85fba04,0xde95ba04,0xd85fba06,
	0xde95ba06,0xd85fba08,0xde97ba08,0xd85fba0a,0xde97ba0a,0xd85fba0c,0xde99ba0c,0xd85fba0e,
	0xde99ba0e,0xd85fba10,0xde9bba10,0xd85fba12,0xde9bba12,0xd85fba14,0xde9dba14,0xd85fba16,
	0xde9dba16,0xd85fba18,0xde9fba18,0xd85fba1a,0xde9fba1a,0xd85fba1c,0xdea1ba1c,0xd85fba1e,
	0xdea1ba1e,0xd85fba20,0xdea3ba20,0xd85fba22,0xdea3ba22,0xd85fba24,0xdea5ba24,0xd85fba26,
	0xdea5ba26,0xd85fba28,0xdea7ba28,0xd85fba2a,0xdea7ba2a,0xd85fba2c,0xdea9ba2c,0xd85fba2e,
	0xdea9ba2e,0xd85fba30,0xdeabba30,0xd85fba32,0xdeabba32,0xd85fba34,0xdeadba34,0xd85fba36,
	0xdeadba36,0xd85fba38,0xdeafba38,0xd85fba3a,0xdeafba3a,0xd85fba3c,0xdeb1ba3c,0xd85fba3e,
	0xdeb1ba3e,0xd85fba40,0xdeb3ba40,0xd85fba42,0xdeb3ba42,0xd85fba44,0xdeb5ba44,0xd85fba46,
	0xdeb5ba46,0xd85fba48,0xdeb7ba48,0xd85fba4a,0xdeb7ba4a,0xd85fba4c,0xdeb9ba4c,0xd85fba4e,
	0xdeb9ba4e,0xd85fba50,0xdebbba50,0xd85fba52,0xdebbba52,0xd85fba54,0xdebdba54,0xd85fba56,
	0xdebdba56,0xd85fba58,0xdebfba58,0xd85fba5a,0xdebfba5a,0xd85fba5c,0xdec1ba5c,0xd85fba5e,
	0xdec1ba5e,0xd85fba60,0xdec3ba60,0xd85fba62,0xdec3ba62,0xd85fba64,0xdec5ba64,0xd85fba66,
	0xdec5ba66,0xd85fba68,0xdec7ba68,0xd85fba6a,0xdec7ba6a,0xd85fba6c,0xdec9ba6c,0xd85fba6e,
	0xdec9ba6e,0xd85fba70,0xdecbba70,0xd85fba72,0xdecbba72,0xd85fba74,0xdecdba74,0xd85fba76,
	0xdecdba76,0xd85fba78,0xdecfba78,0xd85fba7a,0xdecfba7a,0xd85fba7c,0xded1ba7c,0xd85fba7e,
	0xded1ba7e,0xd85fba80,0xded3ba80,0xd85fba82,0xded3ba82,0xd85fba84,0xded5ba84,0xd85fba86,
	0xded5ba86,0xd85fba88,0xded7ba88,0xd85fba8a,0xded7ba8a,0xd85fba8c,0xded9ba8c,0xd85fba8e,
	0xded9ba8e,0xd85fba90,0xdedbba90,0xd85fba92,0xdedbba92,0xd85fba94,0xdeddba94,0xd85fba96,
	0xdeddba96,0xd85fba98,0xdedfba98,0xd85fba9a,0xdedfba9a,0xd85fba9c,0xdee1ba9c,0xd85fba9e,
	0xdee1ba9e,0xd85fbaa0,0xdee3baa0,0xd85fbaa2,0xdee3baa2,0xd85fbaa4,0xdee5baa4,0xd85fbaa6,
	0xdee5baa6,0xd85fbaa8,0xdee7baa8,0xd85fbaaa,0xdee7baaa,0xd85fbaac,0xdee9baac,0xd85fbaae,
	0xdee9baae,0xd85fbab0,0xdeebbab0,0xd85fbab2,0xdeebbab2,0xd85fbab4,0xdeedbab4,0xd85fbab6,
	0xdeedbab6,0xd85fbab8,0xdeefbab8,0xd85fbaba,0xdeefbaba,0xd85fbabc,0xdef1babc,0xd85fbabe,
	0xdef1babe,0xd85fbac0,0xdef3bac0,0xd85fbac2,0xdef3bac2,0xd85fbac4,0xdef5bac4,0xd85fbac6,
	0xdef5bac6,0xd85fbac8,0xdef7bac8,0xd85fbaca,0xdef7baca,0xd85fbacc,0xdef9bacc,0xd85fbace,
	0xdef9bace,0xd85fbad0,0xdefbbad0,0xd85fbad2,0xdefbbad2,0xd85fbad4,0xdefdbad4,0xd85fbad6,
	0xdefdbad6,0xd85fbad8,0xdeffbad8,0xd85fbada,0xdeffbada,0xd85fbadc,0xdf01badc,0xd85fbade,
	0xdf01bade,0xd85fbae0,0xdf03bae0,0xd85fbae2,0xdf03bae2,0xd85fbae4,0xdf05bae4,0xd85fbae6,
	0xdf05bae6,0xd85fbae8,0xdf07bae8,0xd85fbaea,0xdf07baea,0xd85fbaec,0xdf09baec,0xd85fbaee,
	0xdf09baee,0xd85fbaf0,0xdf0bbaf0,0xd85fbaf2,0xdf0bbaf2,0xd85fbaf4,0xdf0dbaf4,0xd85fbaf6,
	0xdf0dbaf6,0xd85fbaf8,0xdf0fbaf8,0xd85fbafa,0xdf0fbafa,0xd85fbafc,0xdf11bafc,0xd85fbafe,
	0xdf11bafe,0xd85fbb00,0xdf13bb00,0xd85fbb02,0xdf13bb02,0xd85fbb04,0xdf15bb04,0xd85fbb06,
	0xdf15bb06,0xd85fbb08,0xdf17bb08,0xd85fbb0a,0xdf17bb0a,0xd85fbb0c,0xdf19bb0c,0xd85fbb0e,
	0xdf19bb0e,0xd85fbb10,0xdf1bbb10,0xd85fbb12,0xdf1bbb12,0xd85fbb14,0xdf1dbb14,0xd85fbb16,
	0xdf1dbb16,0xd85fbb18,0xdf1fbb18,0xd85fbb1a,0xdf1fbb1a,0xd85fbb1c,0xdf21bb1c,0xd85fbb1e,
	0xdf21bb1e,0xd85fbb20,0xdf23bb20,0xd85fbb22,0xdf23bb22,0xd85fbb24,0xdf25bb24,0xd85fbb26,
	0xdf25bb26,0xd85fbb28,0xdf27bb28,0xd85fbb2a,0xdf27bb2a,0xd85fbb2c,0xdf29bb2c,0xd85fbb2e,
	0xdf29bb2e,0xd85fbb30,0xdf2bbb30,0xd85fbb32,0xdf2bbb32,0xd85fbb34,0xdf2dbb34,0xd85fbb36,
	0xdf2dbb36,0xd85fbb38,0xdf2fbb38,0xd85fbb3a,0xdf2fbb3a,0xd85fbb3c,0xdf31bb3c,0xd85fbb3e,
	0xdf31bb3e,0xd85fbb40,0xdf33bb40,0xd85fbb42,0xdf33bb42,0xd85fbb44,0xdf35bb44,0xd85fbb46,
	0xdf35bb46,0xd85fbb48,0xdf37bb48,0xd85fbb4a,0xdf37bb4a,0xd85fbb4c,0xdf39bb4c,0xd85fbb4e,
	0xdf39bb4e,0xd85fbb50,0xdf3bbb50,0xd85fbb52,0xdf3bbb52,0xd85fbb54,0xdf3dbb54,0xd85fbb56,
	0xdf3dbb56,0xd85fbb58,0xdf3fbb58,0xd85fbb5a,0xdf3fbb5a,0xd85fbb5c,0xdf41bb5c,0xd85fbb5e,
	0xdf41bb5e,0xd85fbb60,0xdf43bb60,0xd85fbb62,0xdf43bb62,0xd85fbb64,0xdf45bb64,0xd85fbb66,
	0xdf45bb66,0xd85fbb68,0xdf47bb68,0xd85fbb6a,0xdf47bb6a,0xd85fbb6c,0xdf49bb6c,0xd85fbb6e,
	0xdf49bb6e,0xd85fbb70,0xdf4bbb70,0xd85fbb72,0xdf4bbb72,0xd85fbb74,0xdf4dbb74,0xd85fbb76,
	0xdf4dbb76,0xd85fbb78,0xdf4fbb78,0xd85fbb7a,0xdf4fbb7a,0xd85fbb7c,0xdf51bb7c,0xd85fbb7e,
	0xdf51bb7e,0xd85fbb80,0xdf53bb80,0xd85fbb82,0xdf53bb82,0xd85fbb84,0xdf55bb84,0xd85fbb86,
	0xdf55bb86,0xd85fbb88,0xdf57bb88,0xd85fbb8a,0xdf57bb8a,0xd85fbb8c,0xdf59bb8c,0xd85fbb8e,
	0xdf59bb8e,0xd85fbb90,0xdf5bbb90,0xd85fbb92,0xdf5bbb92,0xd85fbb94,0xdf5dbb94,0xd85fbb96,
	0xdf5dbb96,0xd85fbb98,0xdf5fbb98,0xd85fbb9a,0xdf5fbb9a,0xd85fbb9c,0xdf61bb9c,0xd85fbb9e,
	0xdf61bb9e,0xd85fbba0,0xdf63bba0,0xd85fbba2,0xdf63bba2,0xd85fbba4,0xdf65bba4,0xd85fbba6,
	0xdf65bba6,0xd85fbba8,0xdf67bba8,0xd85fbbaa,0xdf67bbaa,0xd85fbbac,0xdf69bbac,0xd85fbbae,
	0xdf69bbae,0xd85fbbb0,0xdf6bbbb0,0xd85fbbb2,0xdf6bbbb2,0xd85fbbb4,0xdf6dbbb4,0xd85fbbb6,
	0xdf6dbbb6,0xd85fbbb8,0xdf6fbbb8,0xd85fbbba,0xdf6fbbba,0xd85fbbbc,0xdf71bbbc,0xd85fbbbe,
	0xdf71bbbe,0xd85fbbc0,0xdf73bbc0,0xd85fbbc2,0xdf73bbc2,0xd85fbbc4,0xdf75bbc4,0xd85fbbc6,
	0xdf75bbc6,0xd85fbbc8,0xdf77bbc8,0xd85fbbca,0xdf77bbca,0xd85fbbcc,0xdf79bbcc,0xd85fbbce,
	0xdf79bbce,0xd85fbbd0,0xdf7bbbd0,0xd85fbbd2,0xdf7bbbd2,0xd85fbbd4,0xdf7dbbd4,0xd85fbbd6,
	0xdf7dbbd6,0xd85fbbd8,0xdf7fbbd8,0xd85fbbda,0xdf7fbbda,0xd85fbbdc,0xdf81bbdc,0xd85fbbde,
	0xdf81bbde,0xd85fbbe0,0xdf83bbe0,0xd85fbbe2,0xdf83bbe2,0xd85fbbe4,0xdf85bbe4,0xd85fbbe6,
	0xdf85bbe6,0xd85fbbe8,0xdf87bbe8,0xd85fbbea,0xdf87bbea,0xd85fbbec,0xdf89bbec,0xd85fbbee,
	0xdf89bbee,0xd85fbbf0,0xdf8bbbf0,0xd85fbbf2,0xdf8bbbf2,0xd85fbbf4,0xdf8dbbf4,0xd85fbbf6,
	0xdf8dbbf6,0xd85fbbf8,0xdf8fbbf8,0xd85fbbfa,0xdf8fbbfa,0xd85fbbfc,0xdf91bbfc,0xd85fbbfe,
	0xdf91bbfe,0xd85fbc00,0xdf93bc00,0xd85fbc02,0xdf93bc02,0xd85fbc04,0xdf95bc04,0xd85fbc06,
	0xdf95bc06,0xd85fbc08,0xdf97bc08,0xd85fbc0a,0xdf97bc0a,0xd85fbc0c,0xdf99bc0c,0xd85fbc0e,
	0xdf99bc0e,0xd85fbc10,0xdf9bbc10,0xd85fbc12,0xdf9bbc12,0xd85fbc14,0xdf9dbc14,0xd85fbc16,
	0xdf9dbc16,0xd85fbc18,0xdf9fbc18,0xd85fbc1a,0xdf9fbc1a,0xd85fbc1c,0xdfa1bc1c,0xd85fbc1e,
	0xdfa1bc1e,0xd85fbc20,0xdfa3bc20,0xd85fbc22,0xdfa3bc22,0xd85fbc24,0xdfa5bc24,0xd85fbc26,
	0xdfa5bc26,0xd85fbc28,0xdfa7bc28,0xd85fbc2a,0xdfa7bc2a,0xd85fbc2c,0xdfa9bc2c,0xd85fbc2e,
	0xdfa9bc2e,0xd85fbc30,0xdfabbc30,0xd85fbc32,0xdfabbc32,0xd85fbc34,0xdfadbc34,0xd85fbc36,
	0xdfadbc36,0xd85fbc38,0xdfafbc38,0xd85fbc3a,0xdfafbc3a,0xd85fbc3c,0xdfb1bc3c,0xd85fbc3e,
	0xdfb1bc3e,0xd85fbc40,0xdfb3bc40,0xd85fbc42,0xdfb3bc42,0xd85fbc44,0xdfb5bc44,0xd85fbc46,
	0xdfb5bc46,0xd85fbc48,0xdfb7bc48,0xd85fbc4a,0xdfb7bc4a,0xd85fbc4c,0xdfb9bc4c,0xd85fbc4e,
	0xdfb9bc4e,0xd85fbc50,0xdfbbbc50,0xd85fbc52,0xdfbbbc52,0xd85fbc54,0xdfbdbc54,0xd85fbc56,
	0xdfbdbc56,0xd85fbc58,0xdfbfbc58,0xd85fbc5a,0xdfbfbc5a,0xd85fbc5c,0xdfc1bc5c,0xd85fbc5e,
	0xdfc1bc5e,0xd85fbc60,0xdfc3bc60,0xd85fbc62,0xdfc3bc62,0xd85fbc64,0xdfc5bc64,0xd85fbc66,
	0xdfc5bc66,0xd85fbc68,0xdfc7bc68,0xd85fbc6a,0xdfc7bc6a,0xd85fbc6c,0xdfc9bc6c,0xd85fbc6e,
	0xdfc9bc6e,0xd85fbc70,0xdfcbbc70,0xd85fbc72,0xdfcbbc72,0xd85fbc74,0xdfcdbc74,0xd85fbc76,
	0xdfcdbc76,0xd85fbc78,0xdfcfbc78,0xd85fbc7a,0xdfcfbc7a,0xd85fbc7c,0xdfd1bc7c,0xd85fbc7e,
	0xdfd1bc7e,0xd85fbc80,0xdfd3bc80,0xd85fbc82,0xdfd3bc82,0xd85fbc84,0xdfd5bc84,0xd85fbc86,
	0xdfd5bc86,0xd85fbc88,0xdfd7bc88,0xd85fbc8a,0xdfd7bc8a,0xd85fbc8c,0xdfd9bc8c,0xd85fbc8e,
	0xdfd9bc8e,0xd85fbc90,0xdfdbbc90,0xd85fbc92,0xdfdbbc92,0xd85fbc94,0xdfddbc94,0xd85fbc96,
	0xdfddbc96,0xd85fbc98,0xdfdfbc98,0xd85fbc9a,0xdfdfbc9a,0xd85fbc9c,0xdfe1bc9c,0xd85fbc9e,
	0xdfe1bc9e,0xd85fbca0,0xdfe3bca0,0xd85fbca2,0xdfe3bca2,0xd85fbca4,0xdfe5bca4,0xd85fbca6,
	0xdfe5bca6,0xd85fbca8,0xdfe7bca8,0xd85fbcaa,0xdfe7bcaa,0xd85fbcac,0xdfe9bcac,0xd85fbcae,
	0xdfe9bcae,0xd85fbcb0,0xdfebbcb0,0xd85fbcb2,0xdfebbcb2,0xd85fbcb4,0xdfedbcb4,0xd85fbcb6,
	0xdfedbcb6,0xd85fbcb8,0xdfefbcb8,0xd85fbcba,0xdfefbcba,0xd85fbcbc,0xdff1bcbc,0xd85fbcbe,
	0xdff1bcbe,0xd85fbcc0,0xdff3bcc0,0xd85fbcc2,0xdff3bcc2,0xd85fbcc4,0xdff5bcc4,0xd85fbcc6,
	0xdff5bcc6,0xd85fbcc8,0xdff7bcc8,0xd85fbcca,0xdff7bcca,0xd85fbccc,0xdff9bccc,0xd85fbcce,
	0xdff9bcce,0xd85fbcd0,0xdffbbcd0,0xd85fbcd2,0xdffbbcd2,0xd85fbcd4,0xdffdbcd4,0xd85fbcd6,
	0xdffdbcd6,0xd85fbcd8,0xdfffbcd8,0xd85fbcda,0xdfffbcda,0xd861bcdc,0xdc01bcdc,0xd861bcde,
	0xdc01bcde,0xd861bce0,0xdc03bce0,0xd861bce2,0xdc03bce2,0xd861bce4,0xdc05bce4,0xd861bce6,
	0xdc05bce6,0xd861bce8,0xdc07bce8,0xd861bcea,0xdc07bcea,0xd861bcec,0xdc09bcec,0xd861bcee,
	0xdc09bcee,0xd861bcf0,0xdc0bbcf0,0xd861bcf2,0xdc0bbcf2,0xd861bcf4,0xdc0dbcf4,0xd861bcf6,
	0xdc0dbcf6,0xd861bcf8,0xdc0fbcf8,0xd861bcfa,0xdc0fbcfa,0xd861bcfc,0xdc11bcfc,0xd861bcfe,
	0xdc11bcfe,0xd861bd00,0xdc13bd00,0xd861bd02,0xdc13bd02,0xd861bd04,0xdc15bd04,0xd861bd06,
	0xdc15bd06,0xd861bd08,0xdc17bd08,0xd861bd0a,0xdc17bd0a,0xd861bd0c,0xdc19bd0c,0xd861bd0e,
	0xdc19bd0e,0xd861bd10,0xdc1bbd10,0xd861bd12,0xdc1bbd12,0xd861bd14,0xdc1dbd14,0xd861bd16,
	0xdc1dbd16,0xd861bd18,0xdc1fbd18,0xd861bd1a,0xdc1fbd1a,0xd861bd1c,0xdc21bd1c,0xd861bd1e,
	0xdc21bd1e,0xd861bd20,0xdc23bd20,0xd861bd22,0xdc23bd22,0xd861bd24,0xdc25bd24,0xd861bd26,
	0xdc25bd26,0xd861bd28,0xdc27bd28,0xd861bd2a,0xdc27bd2a,0xd861bd2c,0xdc29bd2c,0xd861bd2e,
	0xdc29bd2e,0xd861bd30,0xdc2bbd30,0xd861bd32,0xdc2bbd32,0xd861bd34,0xdc2dbd34,0xd861bd36,
	0xdc2dbd36,0xd861bd38,0xdc2fbd38,0xd861bd3a,0xdc2fbd3a,0xd861bd3c,0xdc31bd3c,0xd861bd3e,
	0xdc31bd3e,0xd861bd40,0xdc33bd40,0xd861bd42,0xdc33bd42,0xd861bd44,0xdc35bd44,0xd861bd46,
	0xdc35bd46,0xd861bd48,0xdc37bd48,0xd861bd4a,0xdc37bd4a,0xd861bd4c,0xdc39bd4c,0xd861bd4e,
	0xdc39bd4e,0xd861bd50,0xdc3bbd50,0xd861bd52,0xdc3bbd52,0xd861bd54,0xdc3dbd54,0xd861bd56,
	0xdc3dbd56,0xd861bd58,0xdc3fbd58,0xd861bd5a,0xdc3fbd5a,0xd861bd5c,0xdc41bd5c,0xd861bd5e,
	0xdc41bd5e,0xd861bd60,0xdc43bd60,0xd861bd62,0xdc43bd62,0xd861bd64,0xdc45bd64,0xd861bd66,
	0xdc45bd66,0xd861bd68,0xdc47bd68,0xd861bd6a,0xdc47bd6a,0xd861bd6c,0xdc49bd6c,0xd861bd6e,
	0xdc49bd6e,0xd861bd70,0xdc4bbd70,0xd861bd72,0xdc4bbd72,0xd861bd74,0xdc4dbd74,0xd861bd76,
	0xdc4dbd76,0xd861bd78,0xdc4fbd78,0xd861bd7a,0xdc4fbd7a,0xd861bd7c,0xdc51bd7c,0xd861bd7e,
	0xdc51bd7e,0xd861bd80,0xdc53bd80,0xd861bd82,0xdc53bd82,0xd861bd84,0xdc55bd84,0xd861bd86,
	0xdc55bd86,0xd861bd88,0xdc57bd88,0xd861bd8a,0xdc57bd8a,0xd861bd8c,0xdc59bd8c,0xd861bd8e,
	0xdc59bd8e,0xd861bd90,0xdc5bbd90,0xd861bd92,0xdc5bbd92,0xd861bd94,0xdc5dbd94,0xd861bd96,
	0xdc5dbd96,0xd861bd98,0xdc5fbd98,0xd861bd9a,0xdc5fbd9a,0xd861bd9c,0xdc61bd9c,0xd861bd9e,
	0xdc61bd9e,0xd861bda0,0xdc63bda0,0xd861bda2,0xdc63bda2,0xd861bda4,0xdc65bda4,0xd861bda6,
	0xdc65bda6,0xd861bda8,0xdc67bda8,0xd861bdaa,0xdc67bdaa,0xd861bdac,0xdc69bdac,0xd861bdae,
	0xdc69bdae,0xd861bdb0,0xdc6bbdb0,0xd861bdb2,0xdc6bbdb2,0xd861bdb4,0xdc6dbdb4,0xd861bdb6,
	0xdc6dbdb6,0xd861bdb8,0xdc6fbdb8,0xd861bdba,0xdc6fbdba,0xd861bdbc,0xdc71bdbc,0xd861bdbe,
	0xdc71bdbe,0xd861bdc0,0xdc73bdc0,0xd861bdc2,0xdc73bdc2,0xd861bdc4,0xdc75bdc4,0xd861bdc6,
	0xdc75bdc6,0xd861bdc8,0xdc77bdc8,0xd861bdca,0xdc77bdca,0xd861bdcc,0xdc79bdcc,0xd861bdce,
	0xdc79bdce,0xd861bdd0,0xdc7bbdd0,0xd861bdd2,0xdc7bbdd2,0xd861bdd4,0xdc7dbdd4,0xd861bdd6,
	0xdc7dbdd6,0xd861bdd8,0xdc7fbdd8,0xd861bdda,0xdc7fbdda,0xd861bddc,0xdc81bddc,0xd861bdde,
	0xdc81bdde,0xd861bde0,0xdc83bde0,0xd861bde2,0xdc83bde2,0xd861bde4,0xdc85bde4,0xd861bde6,
	0xdc85bde6,0xd861bde8,0xdc87bde8,0xd861bdea,0xdc87bdea,0xd861bdec,0xdc89bdec,0xd861bdee,
	0xdc89bdee,0xd861bdf0,0xdc8bbdf0,0xd861bdf2,0xdc8bbdf2,0xd861bdf4,0xdc8dbdf4,0xd861bdf6,
	0xdc8dbdf6,0xd861bdf8,0xdc8fbdf8,0xd861bdfa,0xdc8fbdfa,0xd861bdfc,0xdc91bdfc,0xd861bdfe,
	0xdc91bdfe,0xd861be00,0xdc93be00,0xd861be02,0xdc93be02,0xd861be04,0xdc95be04,0xd861be06,
	0xdc95be06,0xd861be08,0xdc97be08,0xd861be0a,0xdc97be0a,0xd861be0c,0xdc99be0c,0xd861be0e,
	0xdc99be0e,0xd861be10,0xdc9bbe10,0xd861be12,0xdc9bbe12,0xd861be14,0xdc9dbe14,0xd861be16,
	0xdc9dbe16,0xd861be18,0xdc9fbe18,0xd861be1a,0xdc9fbe1a,0xd861be1c,0xdca1be1c,0xd861be1e,
	0xdca1be1e,0xd861be20,0xdca3be20,0xd861be22,0xdca3be22,0xd861be24,0xdca5be24,0xd861be26,
	0xdca5be26,0xd861be28,0xdca7be28,0xd861be2a,0xdca7be2a,0xd861be2c,0xdca9be2c,0xd861be2e,
	0xdca9be2e,0xd861be30,0xdcabbe30,0xd861be32,0xdcabbe32,0xd861be34,0xdcadbe34,0xd861be36,
	0xdcadbe36,0xd861be38,0xdcafbe38,0xd861be3a,0xdcafbe3a,0xd861be3c,0xdcb1be3c,0xd861be3e,
	0xdcb1be3e,0xd861be40,0xdcb3be40,0xd861be42,0xdcb3be42,0xd861be44,0xdcb5be44,0xd861be46,
	0xdcb5be46,0xd861be48,0xdcb7be48,0xd861be4a,0xdcb7be4a,0xd861be4c,0xdcb9be4c,0xd861be4e,
	0xdcb9be4e,0xd861be50,0xdcbbbe50,0xd861be52,0xdcbbbe52,0xd861be54,0xdcbdbe54,0xd861be56,
	0xdcbdbe56,0xd861be58,0xdcbfbe58,0xd861be5a,0xdcbfbe5a,0xd861be5c,0xdcc1be5c,0xd861be5e,
	0xdcc1be5e,0xd861be60,0xdcc3be60,0xd861be62,0xdcc3be62,0xd861be64,0xdcc5be64,0xd861be66,
	0xdcc5be66,0xd861be68,0xdcc7be68,0xd861be6a,0xdcc7be6a,0xd861be6c,0xdcc9be6c,0xd861be6e,
	0xdcc9be6e,0xd861be70,0xdccbbe70,0xd861be72,0xdccbbe72,0xd861be74,0xdccdbe74,0xd861be76,
	0xdccdbe76,0xd861be78,0xdccfbe78,0xd861be7a,0xdccfbe7a,0xd861be7c,0xdcd1be7c,0xd861be7e,
	0xdcd1be7e,0xd861be80,0xdcd3be80,0xd861be82,0xdcd3be82,0xd861be84,0xdcd5be84,0xd861be86,
	0xdcd5be86,0xd861be88,0xdcd7be88,0xd861be8a,0xdcd7be8a,0xd861be8c,0xdcd9be8c,0xd861be8e,
	0xdcd9be8e,0xd861be90,0xdcdbbe90,0xd861be92,0xdcdbbe92,0xd861be94,0xdcddbe94,0xd861be96,
	0xdcddbe96,0xd861be98,0xdcdfbe98,0xd861be9a,0xdcdfbe9a,0xd861be9c,0xdce1be9c,0xd861be9e,
	0xdce1be9e,0xd861bea0,0xdce3bea0,0xd861bea2,0xdce3bea2,0xd861bea4,0xdce5bea4,0xd861bea6,
	0xdce5bea6,0xd861bea8,0xdce7bea8,0xd861beaa,0xdce7beaa,0xd861beac,0xdce9beac,0xd861beae,
	0xdce9beae,0xd861beb0,0xdcebbeb0,0xd861beb2,0xdcebbeb2,0xd861beb4,0xdcedbeb4,0xd861beb6,
	0xdcedbeb6,0xd861beb8,0xdcefbeb8,0xd861beba,0xdcefbeba,0xd861bebc,0xdcf1bebc,0xd861bebe,
	0xdcf1bebe,0xd861bec0,0xdcf3bec0,0xd861bec2,0xdcf3bec2,0xd861bec4,0xdcf5bec4,0xd861bec6,
	0xdcf5bec6,0xd861bec8,0xdcf7bec8,0xd861beca,0xdcf7beca,0xd861becc,0xdcf9becc,0xd861bece,
	0xdcf9bece,0xd861bed0,0xdcfbbed0,0xd861bed2,0xdcfbbed2,0xd861bed4,0xdcfdbed4,0xd861bed6,
	0xdcfdbed6,0xd861bed8,0xdcffbed8,0xd861beda,0xdcffbeda,0xd861bedc,0xdd01bedc,0xd861bede,
	0xdd01bede,0xd861bee0,0xdd03bee0,0xd861bee2,0xdd03bee2,0xd861bee4,0xdd05bee4,0xd861bee6,
	0xdd05bee6,0xd861bee8,0xdd07bee8,0xd861beea,0xdd07beea,0xd861beec,0xdd09beec,0xd861beee,
	0xdd09beee,0xd861bef0,0xdd0bbef0,0xd861bef2,0xdd0bbef2,0xd861bef4,0xdd0dbef4,0xd861bef6,
	0xdd0dbef6,0xd861bef8,0xdd0fbef8,0xd861befa,0xdd0fbefa,0xd861befc,0xdd11befc,0xd861befe,
	0xdd11befe,0xd861bf00,0xdd13bf00,0xd861bf02,0xdd13bf02,0xd861bf04,0xdd15bf04,0xd861bf06,
	0xdd15bf06,0xd861bf08,0xdd17bf08,0xd861bf0a,0xdd17bf0a,0xd861bf0c,0xdd19bf0c,0xd861bf0e,
	0xdd19bf0e,0xd861bf10,0xdd1bbf10,0xd861bf12,0xdd1bbf12,0xd861bf14,0xdd1dbf14,0xd861bf16,
	0xdd1dbf16,0xd861bf18,0xdd1fbf18,0xd861bf1a,0xdd1fbf1a,0xd861bf1c,0xdd21bf1c,0xd861bf1e,
	0xdd21bf1e,0xd861bf20,0xdd23bf20,0xd861bf22,0xdd23bf22,0xd861bf24,0xdd25bf24,0xd861bf26,
	0xdd25bf26,0xd861bf28,0xdd27bf28,0xd861bf2a,0xdd27bf2a,0xd861bf2c,0xdd29bf2c,0xd861bf2e,
	0xdd29bf2e,0xd861bf30,0xdd2bbf30,0xd861bf32,0xdd2bbf32,0xd861bf34,0xdd2dbf34,0xd861bf36,
	0xdd2dbf36,0xd861bf38,0xdd2fbf38,0xd861bf3a,0xdd2fbf3a,0xd861bf3c,0xdd31bf3c,0xd861bf3e,
	0xdd31bf3e,0xd861bf40,0xdd33bf40,0xd861bf42,0xdd33bf42,0xd861bf44,0xdd35bf44,0xd861bf46,
	0xdd35bf46,0xd861bf48,0xdd37bf48,0xd861bf4a,0xdd37bf4a,0xd861bf4c,0xdd39bf4c,0xd861bf4e,
	0xdd39bf4e,0xd861bf50,0xdd3bbf50,0xd861bf52,0xdd3bbf52,0xd861bf54,0xdd3dbf54,0xd861bf56,
	0xdd3dbf56,0xd861bf58,0xdd3fbf58,0xd861bf5a,0xdd3fbf5a,0xd861bf5c,0xdd41bf5c,0xd861bf5e,
	0xdd41bf5e,0xd861bf60,0xdd43bf60,0xd861bf62,0xdd43bf62,0xd861bf64,0xdd45bf64,0xd861bf66,
	0xdd45bf66,0xd861bf68,0xdd47bf68,0xd861bf6a,0xdd47bf6a,0xd861bf6c,0xdd49bf6c,0xd861bf6e,
	0xdd49bf6e,0xd861bf70,0xdd4bbf70,0xd861bf72,0xdd4bbf72,0xd861bf74,0xdd4dbf74,0xd861bf76,
	0xdd4dbf76,0xd861bf78,0xdd4fbf78,0xd861bf7a,0xdd4fbf7a,0xd861bf7c,0xdd51bf7c,0xd861bf7e,
	0xdd51bf7e,0xd861bf80,0xdd53bf80,0xd861bf82,0xdd53bf82,0xd861bf84,0xdd55bf84,0xd861bf86,
	0xdd55bf86,0xd861bf88,0xdd57bf88,0xd861bf8a,0xdd57bf8a,0xd861bf8c,0xdd59bf8c,0xd861bf8e,
	0xdd59bf8e,0xd861bf90,0xdd5bbf90,0xd861bf92,0xdd5bbf92,0xd861bf94,0xdd5dbf94,0xd861bf96,
	0xdd5dbf96,0xd861bf98,0xdd5fbf98,0xd861bf9a,0xdd5fbf9a,0xd861bf9c,0xdd61bf9c,0xd861bf9e,
	0xdd61bf9e,0xd861bfa0,0xdd63bfa0,0xd861bfa2,0xdd63bfa2,0xd861bfa4,0xdd65bfa4,0xd861bfa6,
	0xdd65bfa6,0xd861bfa8,0xdd67bfa8,0xd861bfaa,0xdd67bfaa,0xd861bfac,0xdd69bfac,0xd861bfae,
	0xdd69bfae,0xd861bfb0,0xdd6bbfb0,0xd861bfb2,0xdd6bbfb2,0xd861bfb4,0xdd6dbfb4,0xd861bfb6,
	0xdd6dbfb6,0xd861bfb8,0xdd6fbfb8,0xd861bfba,0xdd6fbfba,0xd861bfbc,0xdd71bfbc,0xd861bfbe,
	0xdd71bfbe,0xd861bfc0,0xdd73bfc0,0xd861bfc2,0xdd73bfc2,0xd861bfc4,0xdd75bfc4,0xd861bfc6,
	0xdd75bfc6,0xd861bfc8,0xdd77bfc8,0xd861bfca,0xdd77bfca,0xd861bfcc,0xdd79bfcc,0xd861bfce,
	0xdd79bfce,0xd861bfd0,0xdd7bbfd0,0xd861bfd2,0xdd7bbfd2,0xd861bfd4,0xdd7dbfd4,0xd861bfd6,
	0xdd7dbfd6,0xd861bfd8,0xdd7fbfd8,0xd861bfda,0xdd7fbfda,0xd861bfdc,0xdd81bfdc,0xd861bfde,
	0xdd81bfde,0xd861bfe0,0xdd83bfe0,0xd861bfe2,0xdd83bfe2,0xd861bfe4,0xdd85bfe4,0xd861bfe6,
	0xdd85bfe6,0xd861bfe8,0xdd87bfe8,0xd861bfea,0xdd87bfea,0xd861bfec,0xdd89bfec,0xd861bfee,
	0xdd89bfee,0xd861bff0,0xdd8bbff0,0xd861bff2,0xdd8bbff2,0xd861bff4,0xdd8dbff4,0xd861bff6,
	0xdd8dbff6,0xd861bff8,0xdd8fbff8,0xd861bffa,0xdd8fbffa,0xd861bffc,0xdd91bffc,0xd861bffe,
	0xdd91bffe,0xd861c000,0xdd93c000,0xd861c002,0xdd93c002,0xd861c004,0xdd95c004,0xd861c006,
	0xdd95c006,0xd861c008,0xdd97c008,0xd861c00a,0xdd97c00a,0xd861c00c,0xdd99c00c,0xd861c00e,
	0xdd99c00e,0xd861c010,0xdd9bc010,0xd861c012,0xdd9bc012,0xd861c014,0xdd9dc014,0xd861c016,
	0xdd9dc016,0xd861c018,0xdd9fc018,0xd861c01a,0xdd9fc01a,0xd861c01c,0xdda1c01c,0xd861c01e,
	0xdda1c01e,0xd861c020,0xdda3c020,0xd861c022,0xdda3c022,0xd861c024,0xdda5c024,0xd861c026,
	0xdda5c026,0xd861c028,0xdda7c028,0xd861c02a,0xdda7c02a,0xd861c02c,0xdda9c02c,0xd861c02e,
	0xdda9c02e,0xd861c030,0xddabc030,0xd861c032,0xddabc032,0xd861c034,0xddadc034,0xd861c036,
	0xddadc036,0xd861c038,0xddafc038,0xd861c03a,0xddafc03a,0xd861c03c,0xddb1c03c,0xd861c03e,
	0xddb1c03e,0xd861c040,0xddb3c040,0xd861c042,0xddb3c042,0xd861c044,0xddb5c044,0xd861c046,
	0xddb5c046,0xd861c048,0xddb7c048,0xd861c04a,0xddb7c04a,0xd861c04c,0xddb9c04c,0xd861c04e,
	0xddb9c04e,0xd861c050,0xddbbc050,0xd861c052,0xddbbc052,0xd861c054,0xddbdc054,0xd861c056,
	0xddbdc056,0xd861c058,0xddbfc058,0xd861c05a,0xddbfc05a,0xd861c05c,0xddc1c05c,0xd861c05e,
	0xddc1c05e,0xd861c060,0xddc3c060,0xd861c062,0xddc3c062,0xd861c064,0xddc5c064,0xd861c066,
	0xddc5c066,0xd861c068,0xddc7c068,0xd861c06a,0xddc7c06a,0xd861c06c,0xddc9c06c,0xd861c06e,
	0xddc9c06e,0xd861c070,0xddcbc070,0xd861c072,0xddcbc072,0xd861c074,0xddcdc074,0xd861c076,
	0xddcdc076,0xd861c078,0xddcfc078,0xd861c07a,0xddcfc07a,0xd861c07c,0xddd1c07c,0xd861c07e,
	0xddd1c07e,0xd861c080,0xddd3c080,0xd861c082,0xddd3c082,0xd861c084,0xddd5c084,0xd861c086,
	0xddd5c086,0xd861c088,0xddd7c088,0xd861c08a,0xddd7c08a,0xd861c08c,0xddd9c08c,0xd861c08e,
	0xddd9c08e,0xd861c090,0xdddbc090,0xd861c092,0xdddbc092,0xd861c094,0xddddc094,0xd861c096,
	0xddddc096,0xd861c098,0xdddfc098,0xd861c09a,0xdddfc09a,0xd861c09c,0xdde1c09c,0xd861c09e,
	0xdde1c09e,0xd861c0a0,0xdde3c0a0,0xd861c0a2,0xdde3c0a2,0xd861c0a4,0xdde5c0a4,0xd861c0a6,
	0xdde5c0a6,0xd861c0a8,0xdde7c0a8,0xd861c0aa,0xdde7c0aa,0xd861c0ac,0xdde9c0ac,0xd861c0ae,
	0xdde9c0ae,0xd861c0b0,0xddebc0b0,0xd861c0b2,0xddebc0b2,0xd861c0b4,0xddedc0b4,0xd861c0b6,
	0xddedc0b6,0xd861c0b8,0xddefc0b8,0xd861c0ba,0xddefc0ba,0xd861c0bc,0xddf1c0bc,0xd861c0be,
	0xddf1c0be,0xd861c0c0,0xddf3c0c0,0xd861c0c2,0xddf3c0c2,0xd861c0c4,0xddf5c0c4,0xd861c0c6,
	0xddf5c0c6,0xd861c0c8,0xddf7c0c8,0xd861c0ca,0xddf7c0ca,0xd861c0cc,0xddf9c0cc,0xd861c0ce,
	0xddf9c0ce,0xd861c0d0,0xddfbc0d0,0xd861c0d2,0xddfbc0d2,0xd861c0d4,0xddfdc0d4,0xd861c0d6,
	0xddfdc0d6,0xd861c0d8,0xddffc0d8,0xd861c0da,0xddffc0da,0xd861c0dc,0xde01c0dc,0xd861c0de,
	0xde01c0de,0xd861c0e0,0xde03c0e0,0xd861c0e2,0xde03c0e2,0xd861c0e4,0xde05c0e4,0xd861c0e6,
	0xde05c0e6,0xd861c0e8,0xde07c0e8,0xd861c0ea,0xde07c0ea,0xd861c0ec,0xde09c0ec,0xd861c0ee,
	0xde09c0ee,0xd861c0f0,0xde0bc0f0,0xd861c0f2,0xde0bc0f2,0xd861c0f4,0xde0dc0f4,0xd861c0f6,
	0xde0dc0f6,0xd861c0f8,0xde0fc0f8,0xd861c0fa,0xde0fc0fa,0xd861c0fc,0xde11c0fc,0xd861c0fe,
	0xde11c0fe,0xd861c100,0xde13c100,0xd861c102,0xde13c102,0xd861c104,0xde15c104,0xd861c106,
	0xde15c106,0xd861c108,0xde17c108,0xd861c10a,0xde17c10a,0xd861c10c,0xde19c10c,0xd861c10e,
	0xde19c10e,0xd861c110,0xde1bc110,0xd861c112,0xde1bc112,0xd861c114,0xde1dc114,0xd861c116,
	0xde1dc116,0xd861c118,0xde1fc118,0xd861c11a,0xde1fc11a,0xd861c11c,0xde21c11c,0xd861c11e,
	0xde21c11e,0xd861c120,0xde23c120,0xd861c122,0xde23c122,0xd861c124,0xde25c124,0xd861c126,
	0xde25c126,0xd861c128,0xde27c128,0xd861c12a,0xde27c12a,0xd861c12c,0xde29c12c,0xd861c12e,
	0xde29c12e,0xd861c130,0xde2bc130,0xd861c132,0xde2bc132,0xd861c134,0xde2dc134,0xd861c136,
	0xde2dc136,0xd861c138,0xde2fc138,0xd861c13a,0xde2fc13a,0xd861c13c,0xde31c13c,0xd861c13e,
	0xde31c13e,0xd861c140,0xde33c140,0xd861c142,0xde33c142,0xd861c144,0xde35c144,0xd861c146,
	0xde35c146,0xd861c148,0xde37c148,0xd861c14a,0xde37c14a,0xd861c14c,0xde39c14c,0xd861c14e,
	0xde39c14e,0xd861c150,0xde3bc150,0xd861c152,0xde3bc152,0xd861c154,0xde3dc154,0xd861c156,
	0xde3dc156,0xd861c158,0xde3fc158,0xd861c15a,0xde3fc15a,0xd861c15c,0xde41c15c,0xd861c15e,
	0xde41c15e,0xd861c160,0xde43c160,0xd861c162,0xde43c162,0xd861c164,0xde45c164,0xd861c166,
	0xde45c166,0xd861c168,0xde47c168,0xd861c16a,0xde47c16a,0xd861c16c,0xde49c16c,0xd861c16e,
	0xde49c16e,0xd861c170,0xde4bc170,0xd861c172,0xde4bc172,0xd861c174,0xde4dc174,0xd861c176,
	0xde4dc176,0xd861c178,0xde4fc178,0xd861c17a,0xde4fc17a,0xd861c17c,0xde51c17c,0xd861c17e,
	0xde51c17e,0xd861c180,0xde53c180,0xd861c182,0xde53c182,0xd861c184,0xde55c184,0xd861c186,
	0xde55c186,0xd861c188,0xde57c188,0xd861c18a,0xde57c18a,0xd861c18c,0xde59c18c,0xd861c18e,
	0xde59c18e,0xd861c190,0xde5bc190,0xd861c192,0xde5bc192,0xd861c194,0xde5dc194,0xd861c196,
	0xde5dc196,0xd861c198,0xde5fc198,0xd861c19a,0xde5fc19a,0xd861c19c,0xde61c19c,0xd861c19e,
	0xde61c19e,0xd861c1a0,0xde63c1a0,0xd861c1a2,0xde63c1a2,0xd861c1a4,0xde65c1a4,0xd861c1a6,
	0xde65c1a6,0xd861c1a8,0xde67c1a8,0xd861c1aa,0xde67c1aa,0xd861c1ac,0xde69c1ac,0xd861c1ae,
	0xde69c1ae,0xd861c1b0,0xde6bc1b0,0xd861c1b2,0xde6bc1b2,0xd861c1b4,0xde6dc1b4,0xd861c1b6,
	0xde6dc1b6,0xd861c1b8,0xde6fc1b8,0xd861c1ba,0xde6fc1ba,0xd861c1bc,0xde71c1bc,0xd861c1be,
	0xde71c1be,0xd861c1c0,0xde73c1c0,0xd861c1c2,0xde73c1c2,0xd861c1c4,0xde75c1c4,0xd861c1c6,
	0xde75c1c6,0xd861c1c8,0xde77c1c8,0xd861c1ca,0xde77c1ca,0xd861c1cc,0xde79c1cc,0xd861c1ce,
	0xde79c1ce,0xd861c1d0,0xde7bc1d0,0xd861c1d2,0xde7bc1d2,0xd861c1d4,0xde7dc1d4,0xd861c1d6,
	0xde7dc1d6,0xd861c1d8,0xde7fc1d8,0xd861c1da,0xde7fc1da,0xd861c1dc,0xde81c1dc,0xd861c1de,
	0xde81c1de,0xd861c1e0,0xde83c1e0,0xd861c1e2,0xde83c1e2,0xd861c1e4,0xde85c1e4,0xd861c1e6,
	0xde85c1e6,0xd861c1e8,0xde87c1e8,0xd861c1ea,0xde87c1ea,0xd861c1ec,0xde89c1ec,0xd861c1ee,
	0xde89c1ee,0xd861c1f0,0xde8bc1f0,0xd861c1f2,0xde8bc1f2,0xd861c1f4,0xde8dc1f4,0xd861c1f6,
	0xde8dc1f6,0xd861c1f8,0xde8fc1f8,0xd861c1fa,0xde8fc1fa,0xd861c1fc,0xde91c1fc,0xd861c1fe,
	0xde91c1fe,0xd861c200,0xde93c200,0xd861c202,0xde93c202,0xd861c204,0xde95c204,0xd861c206,
	0xde95c206,0xd861c208,0xde97c208,0xd861c20a,0xde97c20a,0xd861c20c,0xde99c20c,0xd861c20e,
	0xde99c20e,0xd861c210,0xde9bc210,0xd861c212,0xde9bc212,0xd861c214,0xde9dc214,0xd861c216,
	0xde9dc216,0xd861c218,0xde9fc218,0xd861c21a,0xde9fc21a,0xd861c21c,0xdea1c21c,0xd861c21e,
	0xdea1c21e,0xd861c220,0xdea3c220,0xd861c222,0xdea3c222,0xd861c224,0xdea5c224,0xd861c226,
	0xdea5c226,0xd861c228,0xdea7c228,0xd861c22a,0xdea7c22a,0xd861c22c,0xdea9c22c,0xd861c22e,
	0xdea9c22e,0xd861c230,0xdeabc230,0xd861c232,0xdeabc232,0xd861c234,0xdeadc234,0xd861c236,
	0xdeadc236,0xd861c238,0xdeafc238,0xd861c23a,0xdeafc23a,0xd861c23c,0xdeb1c23c,0xd861c23e,
	0xdeb1c23e,0xd861c240,0xdeb3c240,0xd861c242,0xdeb3c242,0xd861c244,0xdeb5c244,0xd861c246,
	0xdeb5c246,0xd861c248,0xdeb7c248,0xd861c24a,0xdeb7c24a,0xd861c24c,0xdeb9c24c,0xd861c24e,
	0xdeb9c24e,0xd861c250,0xdebbc250,0xd861c252,0xdebbc252,0xd861c254,0xdebdc254,0xd861c256,
	0xdebdc256,0xd861c258,0xdebfc258,0xd861c25a,0xdebfc25a,0xd861c25c,0xdec1c25c,0xd861c25e,
	0xdec1c25e,0xd861c260,0xdec3c260,0xd861c262,0xdec3c262,0xd861c264,0xdec5c264,0xd861c266,
	0xdec5c266,0xd861c268,0xdec7c268,0xd861c26a,0xdec7c26a,0xd861c26c,0xdec9c26c,0xd861c26e,
	0xdec9c26e,0xd861c270,0xdecbc270,0xd861c272,0xdecbc272,0xd861c274,0xdecdc274,0xd861c276,
	0xdecdc276,0xd861c278,0xdecfc278,0xd861c27a,0xdecfc27a,0xd861c27c,0xded1c27c,0xd861c27e,
	0xded1c27e,0xd861c280,0xded3c280,0xd861c282,0xded3c282,0xd861c284,0xded5c284,0xd861c286,
	0xded5c286,0xd861c288,0xded7c288,0xd861c28a,0xded7c28a,0xd861c28c,0xded9c28c,0xd861c28e,
	0xded9c28e,0xd861c290,0xdedbc290,0xd861c292,0xdedbc292,0xd861c294,0xdeddc294,0xd861c296,
	0xdeddc296,0xd861c298,0xdedfc298,0xd861c29a,0xdedfc29a,0xd861c29c,0xdee1c29c,0xd861c29e,
	0xdee1c29e,0xd861c2a0,0xdee3c2a0,0xd861c2a2,0xdee3c2a2,0xd861c2a4,0xdee5c2a4,0xd861c2a6,
	0xdee5c2a6,0xd861c2a8,0xdee7c2a8,0xd861c2aa,0xdee7c2aa,0xd861c2ac,0xdee9c2ac,0xd861c2ae,
	0xdee9c2ae,0xd861c2b0,0xdeebc2b0,0xd861c2b2,0xdeebc2b2,0xd861c2b4,0xdeedc2b4,0xd861c2b6,
	0xdeedc2b6,0xd861c2b8,0xdeefc2b8,0xd861c2ba,0xdeefc2ba,0xd861c2bc,0xdef1c2bc,0xd861c2be,
	0xdef1c2be,0xd861c2c0,0xdef3c2c0,0xd861c2c2,0xdef3c2c2,0xd861c2c4,0xdef5c2c4,0xd861c2c6,
	0xdef5c2c6,0xd861c2c8,0xdef7c2c8,0xd861c2ca,0xdef7c2ca,0xd861c2cc,0xdef9c2cc,0xd861c2ce,
	0xdef9c2ce,0xd861c2d0,0xdefbc2d0,0xd861c2d2,0xdefbc2d2,0xd861c2d4,0xdefdc2d4,0xd861c2d6,
	0xdefdc2d6,0xd861c2d8,0xdeffc2d8,0xd861c2da,0xdeffc2da,0xd861c2dc,0xdf01c2dc,0xd861c2de,
	0xdf01c2de,0xd861c2e0,0xdf03c2e0,0xd861c2e2,0xdf03c2e2,0xd861c2e4,0xdf05c2e4,0xd861c2e6,
	0xdf05c2e6,0xd861c2e8,0xdf07c2e8,0xd861c2ea,0xdf07c2ea,0xd861c2ec,0xdf09c2ec,0xd861c2ee,
	0xdf09c2ee,0xd861c2f0,0xdf0bc2f0,0xd861c2f2,0xdf0bc2f2,0xd861c2f4,0xdf0dc2f4,0xd861c2f6,
	0xdf0dc2f6,0xd861c2f8,0xdf0fc2f8,0xd861c2fa,0xdf0fc2fa,0xd861c2fc,0xdf11c2fc,0xd861c2fe,
	0xdf11c2fe,0xd861c300,0xdf13c300,0xd861c302,0xdf13c302,0xd861c304,0xdf15c304,0xd861c306,
	0xdf15c306,0xd861c308,0xdf17c308,0xd861c30a,0xdf17c30a,0xd861c30c,0xdf19c30c,0xd861c30e,
	0xdf19c30e,0xd861c310,0xdf1bc310,0xd861c312,0xdf1bc312,0xd861c314,0xdf1dc314,0xd861c316,
	0xdf1dc316,0xd861c318,0xdf1fc318,0xd861c31a,0xdf1fc31a,0xd861c31c,0xdf21c31c,0xd861c31e,
	0xdf21c31e,0xd861c320,0xdf23c320,0xd861c322,0xdf23c322,0xd861c324,0xdf25c324,0xd861c326,
	0xdf25c326,0xd861c328,0xdf27c328,0xd861c32a,0xdf27c32a,0xd861c32c,0xdf29c32c,0xd861c32e,
	0xdf29c32e,0xd861c330,0xdf2bc330,0xd861c332,0xdf2bc332,0xd861c334,0xdf2dc334,0xd861c336,
	0xdf2dc336,0xd861c338,0xdf2fc338,0xd861c33a,0xdf2fc33a,0xd861c33c,0xdf31c33c,0xd861c33e,
	0xdf31c33e,0xd861c340,0xdf33c340,0xd861c342,0xdf33c342,0xd861c344,0xdf35c344,0xd861c346,
	0xdf35c346,0xd861c348,0xdf37c348,0xd861c34a,0xdf37c34a,0xd861c34c,0xdf39c34c,0xd861c34e,
	0xdf39c34e,0xd861c350,0xdf3bc350,0xd861c352,0xdf3bc352,0xd861c354,0xdf3dc354,0xd861c356,
	0xdf3dc356,0xd861c358,0xdf3fc358,0xd861c35a,0xdf3fc35a,0xd861c35c,0xdf41c35c,0xd861c35e,
	0xdf41c35e,0xd861c360,0xdf43c360,0xd861c362,0xdf43c362,0xd861c364,0xdf45c364,0xd861c366,
	0xdf45c366,0xd861c368,0xdf47c368,0xd861c36a,0xdf47c36a,0xd861c36c,0xdf49c36c,0xd861c36e,
	0xdf49c36e,0xd861c370,0xdf4bc370,0xd861c372,0xdf4bc372,0xd861c374,0xdf4dc374,0xd861c376,
	0xdf4dc376,0xd861c378,0xdf4fc378,0xd861c37a,0xdf4fc37a,0xd861c37c,0xdf51c37c,0xd861c37e,
	0xdf51c37e,0xd861c380,0xdf53c380,0xd861c382,0xdf53c382,0xd861c384,0xdf55c384,0xd861c386,
	0xdf55c386,0xd861c388,0xdf57c388,0xd861c38a,0xdf57c38a,0xd861c38c,0xdf59c38c,0xd861c38e,
	0xdf59c38e,0xd861c390,0xdf5bc390,0xd861c392,0xdf5bc392,0xd861c394,0xdf5dc394,0xd861c396,
	0xdf5dc396,0xd861c398,0xdf5fc398,0xd861c39a,0xdf5fc39a,0xd861c39c,0xdf61c39c,0xd861c39e,
	0xdf61c39e,0xd861c3a0,0xdf63c3a0,0xd861c3a2,0xdf63c3a2,0xd861c3a4,0xdf65c3a4,0xd861c3a6,
	0xdf65c3a6,0xd861c3a8,0xdf67c3a8,0xd861c3aa,0xdf67c3aa,0xd861c3ac,0xdf69c3ac,0xd861c3ae,
	0xdf69c3ae,0xd861c3b0,0xdf6bc3b0,0xd861c3b2,0xdf6bc3b2,0xd861c3b4,0xdf6dc3b4,0xd861c3b6,
	0xdf6dc3b6,0xd861c3b8,0xdf6fc3b8,0xd861c3ba,0xdf6fc3ba,0xd861c3bc,0xdf71c3bc,0xd861c3be,
	0xdf71c3be,0xd861c3c0,0xdf73c3c0,0xd861c3c2,0xdf73c3c2,0xd861c3c4,0xdf75c3c4,0xd861c3c6,
	0xdf75c3c6,0xd861c3c8,0xdf77c3c8,0xd861c3ca,0xdf77c3ca,0xd861c3cc,0xdf79c3cc,0xd861c3ce,
	0xdf79c3ce,0xd861c3d0,0xdf7bc3d0,0xd861c3d2,0xdf7bc3d2,0xd861c3d4,0xdf7dc3d4,0xd861c3d6,
	0xdf7dc3d6,0xd861c3d8,0xdf7fc3d8,0xd861c3da,0xdf7fc3da,0xd861c3dc,0xdf81c3dc,0xd861c3de,
	0xdf81c3de,0xd861c3e0,0xdf83c3e0,0xd861c3e2,0xdf83c3e2,0xd861c3e4,0xdf85c3e4,0xd861c3e6,
	0xdf85c3e6,0xd861c3e8,0xdf87c3e8,0xd861c3ea,0xdf87c3ea,0xd861c3ec,0xdf89c3ec,0xd861c3ee,
	0xdf89c3ee,0xd861c3f0,0xdf8bc3f0,0xd861c3f2,0xdf8bc3f2,0xd861c3f4,0xdf8dc3f4,0xd861c3f6,
	0xdf8dc3f6,0xd861c3f8,0xdf8fc3f8,0xd861c3fa,0xdf8fc3fa,0xd861c3fc,0xdf91c3fc,0xd861c3fe,
	0xdf91c3fe,0xd861c400,0xdf93c400,0xd861c402,0xdf93c402,0xd861c404,0xdf95c404,0xd861c406,
	0xdf95c406,0xd861c408,0xdf97c408,0xd861c40a,0xdf97c40a,0xd861c40c,0xdf99c40c,0xd861c40e,
	0xdf99c40e,0xd861c410,0xdf9bc410,0xd861c412,0xdf9bc412,0xd861c414,0xdf9dc414,0xd861c416,
	0xdf9dc416,0xd861c418,0xdf9fc418,0xd861c41a,0xdf9fc41a,0xd861c41c,0xdfa1c41c,0xd861c41e,
	0xdfa1c41e,0xd861c420,0xdfa3c420,0xd861c422,0xdfa3c422,0xd861c424,0xdfa5c424,0xd861c426,
	0xdfa5c426,0xd861c428,0xdfa7c428,0xd861c42a,0xdfa7c42a,0xd861c42c,0xdfa9c42c,0xd861c42e,
	0xdfa9c42e,0xd861c430,0xdfabc430,0xd861c432,0xdfabc432,0xd861c434,0xdfadc434,0xd861c436,
	0xdfadc436,0xd861c438,0xdfafc438,0xd861c43a,0xdfafc43a,0xd861c43c,0xdfb1c43c,0xd861c43e,
	0xdfb1c43e,0xd861c440,0xdfb3c440,0xd861c442,0xdfb3c442,0xd861c444,0xdfb5c444,0xd861c446,
	0xdfb5c446,0xd861c448,0xdfb7c448,0xd861c44a,0xdfb7c44a,0xd861c44c,0xdfb9c44c,0xd861c44e,
	0xdfb9c44e,0xd861c450,0xdfbbc450,0xd861c452,0xdfbbc452,0xd861c454,0xdfbdc454,0xd861c456,
	0xdfbdc456,0xd861c458,0xdfbfc458,0xd861c45a,0xdfbfc45a,0xd861c45c,0xdfc1c45c,0xd861c45e,
	0xdfc1c45e,0xd861c460,0xdfc3c460,0xd861c462,0xdfc3c462,0xd861c464,0xdfc5c464,0xd861c466,
	0xdfc5c466,0xd861c468,0xdfc7c468,0xd861c46a,0xdfc7c46a,0xd861c46c,0xdfc9c46c,0xd861c46e,
	0xdfc9c46e,0xd861c470,0xdfcbc470,0xd861c472,0xdfcbc472,0xd861c474,0xdfcdc474,0xd861c476,
	0xdfcdc476,0xd861c478,0xdfcfc478,0xd861c47a,0xdfcfc47a,0xd861c47c,0xdfd1c47c,0xd861c47e,
	0xdfd1c47e,0xd861c480,0xdfd3c480,0xd861c482,0xdfd3c482,0xd861c484,0xdfd5c484,0xd861c486,
	0xdfd5c486,0xd861c488,0xdfd7c488,0xd861c48a,0xdfd7c48a,0xd861c48c,0xdfd9c48c,0xd861c48e,
	0xdfd9c48e,0xd861c490,0xdfdbc490,0xd861c492,0xdfdbc492,0xd861c494,0xdfddc494,0xd861c496,
	0xdfddc496,0xd861c498,0xdfdfc498,0xd861c49a,0xdfdfc49a,0xd861c49c,0xdfe1c49c,0xd861c49e,
	0xdfe1c49e,0xd861c4a0,0xdfe3c4a0,0xd861c4a2,0xdfe3c4a2,0xd861c4a4,0xdfe5c4a4,0xd861c4a6,
	0xdfe5c4a6,0xd861c4a8,0xdfe7c4a8,0xd861c4aa,0xdfe7c4aa,0xd861c4ac,0xdfe9c4ac,0xd861c4ae,
	0xdfe9c4ae,0xd861c4b0,0xdfebc4b0,0xd861c4b2,0xdfebc4b2,0xd861c4b4,0xdfedc4b4,0xd861c4b6,
	0xdfedc4b6,0xd861c4b8,0xdfefc4b8,0xd861c4ba,0xdfefc4ba,0xd861c4bc,0xdff1c4bc,0xd861c4be,
	0xdff1c4be,0xd861c4c0,0xdff3c4c0,0xd861c4c2,0xdff3c4c2,0xd861c4c4,0xdff5c4c4,0xd861c4c6,
	0xdff5c4c6,0xd861c4c8,0xdff7c4c8,0xd861c4ca,0xdff7c4ca,0xd861c4cc,0xdff9c4cc,0xd861c4ce,
	0xdff9c4ce,0xd861c4d0,0xdffbc4d0,0xd861c4d2,0xdffbc4d2,0xd861c4d4,0xdffdc4d4,0xd861c4d6,
	0xdffdc4d6,0xd861c4d8,0xdfffc4d8,0xd861c4da,0xdfffc4da,0xd861c4dc,0xdc01c4dc,0xd861c4de,
	0xdc01c4de,0xd861c4e0,0xdc03c4e0,0xd861c4e2,0xdc03c4e2,0xd861c4e4,0xdc05c4e4,0xd861c4e6,
	0xdc05c4e6,0xd861c4e8,0xdc07c4e8,0xd861c4ea,0xdc07c4ea,0xd861c4ec,0xdc09c4ec,0xd861c4ee,
	0xdc09c4ee,0xd861c4f0,0xdc0bc4f0,0xd861c4f2,0xdc0bc4f2,0xd861c4f4,0xdc0dc4f4,0xd861c4f6,
	0xdc0dc4f6,0xd861c4f8,0xdc0fc4f8,0xd861c4fa,0xdc0fc4fa,0xd861c4fc,0xdc11c4fc,0xd861c4fe,
	0xdc11c4fe,0xd861c500,0xdc13c500,0xd861c502,0xdc13c502,0xd861c504,0xdc15c504,0xd861c506,
	0xdc15c506,0xd861c508,0xdc17c508,0xd861c50a,0xdc17c50a,0xd861c50c,0xdc19c50c,0xd861c50e,
	0xdc19c50e,0xd861c510,0xdc1bc510,0xd861c512,0xdc1bc512,0xd861c514,0xdc1dc514,0xd861c516,
	0xdc1dc516,0xd861c518,0xdc1fc518,0xd861c51a,0xdc1fc51a,0xd861c51c,0xdc21c51c,0xd861c51e,
	0xdc21c51e,0xd861c520,0xdc23c520,0xd861c522,0xdc23c522,0xd861c524,0xdc25c524,0xd861c526,
	0xdc25c526,0xd861c528,0xdc27c528,0xd861c52a,0xdc27c52a,0xd861c52c,0xdc29c52c,0xd861c52e,
	0xdc29c52e,0xd861c530,0xdc2bc530,0xd861c532,0xdc2bc532,0xd861c534,0xdc2dc534,0xd861c536,
	0xdc2dc536,0xd861c538,0xdc2fc538,0xd861c53a,0xdc2fc53a,0xd861c53c,0xdc31c53c,0xd861c53e,
	0xdc31c53e,0xd861c540,0xdc33c540,0xd861c542,0xdc33c542,0xd861c544,0xdc35c544,0xd861c546,
	0xdc35c546,0xd861c548,0xdc37c548,0xd861c54a,0xdc37c54a,0xd861c54c,0xdc39c54c,0xd861c54e,
	0xdc39c54e,0xd861c550,0xdc3bc550,0xd861c552,0xdc3bc552,0xd861c554,0xdc3dc554,0xd861c556,
	0xdc3dc556,0xd861c558,0xdc3fc558,0xd861c55a,0xdc3fc55a,0xd861c55c,0xdc41c55c,0xd861c55e,
	0xdc41c55e,0xd861c560,0xdc43c560,0xd861c562,0xdc43c562,0xd861c564,0xdc45c564,0xd861c566,
	0xdc45c566,0xd861c568,0xdc47c568,0xd861c56a,0xdc47c56a,0xd861c56c,0xdc49c56c,0xd861c56e,
	0xdc49c56e,0xd861c570,0xdc4bc570,0xd861c572,0xdc4bc572,0xd861c574,0xdc4dc574,0xd861c576,
	0xdc4dc576,0xd861c578,0xdc4fc578,0xd861c57a,0xdc4fc57a,0xd861c57c,0xdc51c57c,0xd861c57e,
	0xdc51c57e,0xd861c580,0xdc53c580,0xd861c582,0xdc53c582,0xd861c584,0xdc55c584,0xd861c586,
	0xdc55c586,0xd861c588,0xdc57c588,0xd861c58a,0xdc57c58a,0xd861c58c,0xdc59c58c,0xd861c58e,
	0xdc59c58e,0xd861c590,0xdc5bc590,0xd861c592,0xdc5bc592,0xd861c594,0xdc5dc594,0xd861c596,
	0xdc5dc596,0xd861c598,0xdc5fc598,0xd861c59a,0xdc5fc59a,0xd861c59c,0xdc61c59c,0xd861c59e,
	0xdc61c59e,0xd861c5a0,0xdc63c5a0,0xd861c5a2,0xdc63c5a2,0xd861c5a4,0xdc65c5a4,0xd861c5a6,
	0xdc65c5a6,0xd861c5a8,0xdc67c5a8,0xd861c5aa,0xdc67c5aa,0xd861c5ac,0xdc69c5ac,0xd861c5ae,
	0xdc69c5ae,0xd861c5b0,0xdc6bc5b0,0xd861c5b2,0xdc6bc5b2,0xd861c5b4,0xdc6dc5b4,0xd861c5b6,
	0xdc6dc5b6,0xd861c5b8,0xdc6fc5b8,0xd861c5ba,0xdc6fc5ba,0xd861c5bc,0xdc71c5bc,0xd861c5be,
	0xdc71c5be,0xd861c5c0,0xdc73c5c0,0xd861c5c2,0xdc73c5c2,0xd861c5c4,0xdc75c5c4,0xd861c5c6,
	0xdc75c5c6,0xd861c5c8,0xdc77c5c8,0xd861c5ca,0xdc77c5ca,0xd861c5cc,0xdc79c5cc,0xd861c5ce,
	0xdc79c5ce,0xd861c5d0,0xdc7bc5d0,0xd861c5d2,0xdc7bc5d2,0xd861c5d4,0xdc7dc5d4,0xd861c5d6,
	0xdc7dc5d6,0xd861c5d8,0xdc7fc5d8,0xd861c5da,0xdc7fc5da,0xd861c5dc,0xdc81c5dc,0xd861c5de,
	0xdc81c5de,0xd861c5e0,0xdc83c5e0,0xd861c5e2,0xdc83c5e2,0xd861c5e4,0xdc85c5e4,0xd861c5e6,
	0xdc85c5e6,0xd861c5e8,0xdc87c5e8,0xd861c5ea,0xdc87c5ea,0xd861c5ec,0xdc89c5ec,0xd861c5ee,
	0xdc89c5ee,0xd861c5f0,0xdc8bc5f0,0xd861c5f2,0xdc8bc5f2,0xd861c5f4,0xdc8dc5f4,0xd861c5f6,
	0xdc8dc5f6,0xd861c5f8,0xdc8fc5f8,0xd861c5fa,0xdc8fc5fa,0xd861c5fc,0xdc91c5fc,0xd861c5fe,
	0xdc91c5fe,0xd861c600,0xdc93c600,0xd861c602,0xdc93c602,0xd861c604,0xdc95c604,0xd861c606,
	0xdc95c606,0xd861c608,0xdc97c608,0xd861c60a,0xdc97c60a,0xd861c60c,0xdc99c60c,0xd861c60e,
	0xdc99c60e,0xd861c610,0xdc9bc610,0xd861c612,0xdc9bc612,0xd861c614,0xdc9dc614,0xd861c616,
	0xdc9dc616,0xd861c618,0xdc9fc618,0xd861c61a,0xdc9fc61a,0xd861c61c,0xdca1c61c,0xd861c61e,
	0xdca1c61e,0xd861c620,0xdca3c620,0xd861c622,0xdca3c622,0xd861c624,0xdca5c624,0xd861c626,
	0xdca5c626,0xd861c628,0xdca7c628,0xd861c62a,0xdca7c62a,0xd861c62c,0xdca9c62c,0xd861c62e,
	0xdca9c62e,0xd861c630,0xdcabc630,0xd861c632,0xdcabc632,0xd861c634,0xdcadc634,0xd861c636,
	0xdcadc636,0xd861c638,0xdcafc638,0xd861c63a,0xdcafc63a,0xd861c63c,0xdcb1c63c,0xd861c63e,
	0xdcb1c63e,0xd861c640,0xdcb3c640,0xd861c642,0xdcb3c642,0xd861c644,0xdcb5c644,0xd861c646,
	0xdcb5c646,0xd861c648,0xdcb7c648,0xd861c64a,0xdcb7c64a,0xd861c64c,0xdcb9c64c,0xd861c64e,
	0xdcb9c64e,0xd861c650,0xdcbbc650,0xd861c652,0xdcbbc652,0xd861c654,0xdcbdc654,0xd861c656,
	0xdcbdc656,0xd861c658,0xdcbfc658,0xd861c65a,0xdcbfc65a,0xd861c65c,0xdcc1c65c,0xd861c65e,
	0xdcc1c65e,0xd861c660,0xdcc3c660,0xd861c662,0xdcc3c662,0xd861c664,0xdcc5c664,0xd861c666,
	0xdcc5c666,0xd861c668,0xdcc7c668,0xd861c66a,0xdcc7c66a,0xd861c66c,0xdcc9c66c,0xd861c66e,
	0xdcc9c66e,0xd861c670,0xdccbc670,0xd861c672,0xdccbc672,0xd861c674,0xdccdc674,0xd861c676,
	0xdccdc676,0xd861c678,0xdccfc678,0xd861c67a,0xdccfc67a,0xd861c67c,0xdcd1c67c,0xd861c67e,
	0xdcd1c67e,0xd861c680,0xdcd3c680,0xd861c682,0xdcd3c682,0xd861c684,0xdcd5c684,0xd861c686,
	0xdcd5c686,0xd861c688,0xdcd7c688,0xd861c68a,0xdcd7c68a,0xd861c68c,0xdcd9c68c,0xd861c68e,
	0xdcd9c68e,0xd861c690,0xdcdbc690,0xd861c692,0xdcdbc692,0xd861c694,0xdcddc694,0xd861c696,
	0xdcddc696,0xd861c698,0xdcdfc698,0xd861c69a,0xdcdfc69a,0xd861c69c,0xdce1c69c,0xd861c69e,
	0xdce1c69e,0xd861c6a0,0xdce3c6a0,0xd861c6a2,0xdce3c6a2,0xd861c6a4,0xdce5c6a4,0xd861c6a6,
	0xdce5c6a6,0xd861c6a8,0xdce7c6a8,0xd861c6aa,0xdce7c6aa,0xd861c6ac,0xdce9c6ac,0xd861c6ae,
	0xdce9c6ae,0xd861c6b0,0xdcebc6b0,0xd861c6b2,0xdcebc6b2,0xd861c6b4,0xdcedc6b4,0xd861c6b6,
	0xdcedc6b6,0xd861c6b8,0xdcefc6b8,0xd861c6ba,0xdcefc6ba,0xd861c6bc,0xdcf1c6bc,0xd861c6be,
	0xdcf1c6be,0xd861c6c0,0xdcf3c6c0,0xd861c6c2,0xdcf3c6c2,0xd861c6c4,0xdcf5c6c4,0xd861c6c6,
	0xdcf5c6c6,0xd861c6c8,0xdcf7c6c8,0xd861c6ca,0xdcf7c6ca,0xd861c6cc,0xdcf9c6cc,0xd861c6ce,
	0xdcf9c6ce,0xd861c6d0,0xdcfbc6d0,0xd861c6d2,0xdcfbc6d2,0xd861c6d4,0xdcfdc6d4,0xd861c6d6,
	0xdcfdc6d6,0xd861c6d8,0xdcffc6d8,0xd861c6da,0xdcffc6da,0xd861c6dc,0xdd01c6dc,0xd861c6de,
	0xdd01c6de,0xd861c6e0,0xdd03c6e0,0xd861c6e2,0xdd03c6e2,0xd861c6e4,0xdd05c6e4,0xd861c6e6,
	0xdd05c6e6,0xd861c6e8,0xdd07c6e8,0xd861c6ea,0xdd07c6ea,0xd861c6ec,0xdd09c6ec,0xd861c6ee,
	0xdd09c6ee,0xd861c6f0,0xdd0bc6f0,0xd861c6f2,0xdd0bc6f2,0xd861c6f4,0xdd0dc6f4,0xd861c6f6,
	0xdd0dc6f6,0xd861c6f8,0xdd0fc6f8,0xd861c6fa,0xdd0fc6fa,0xd861c6fc,0xdd11c6fc,0xd861c6fe,
	0xdd11c6fe,0xd861c700,0xdd13c700,0xd861c702,0xdd13c702,0xd861c704,0xdd15c704,0xd861c706,
	0xdd15c706,0xd861c708,0xdd17c708,0xd861c70a,0xdd17c70a,0xd861c70c,0xdd19c70c,0xd861c70e,
	0xdd19c70e,0xd861c710,0xdd1bc710,0xd861c712,0xdd1bc712,0xd861c714,0xdd1dc714,0xd861c716,
	0xdd1dc716,0xd861c718,0xdd1fc718,0xd861c71a,0xdd1fc71a,0xd861c71c,0xdd21c71c,0xd861c71e,
	0xdd21c71e,0xd861c720,0xdd23c720,0xd861c722,0xdd23c722,0xd861c724,0xdd25c724,0xd861c726,
	0xdd25c726,0xd861c728,0xdd27c728,0xd861c72a,0xdd27c72a,0xd861c72c,0xdd29c72c,0xd861c72e,
	0xdd29c72e,0xd861c730,0xdd2bc730,0xd861c732,0xdd2bc732,0xd861c734,0xdd2dc734,0xd861c736,
	0xdd2dc736,0xd861c738,0xdd2fc738,0xd861c73a,0xdd2fc73a,0xd861c73c,0xdd31c73c,0xd861c73e,
	0xdd31c73e,0xd861c740,0xdd33c740,0xd861c742,0xdd33c742,0xd861c744,0xdd35c744,0xd861c746,
	0xdd35c746,0xd861c748,0xdd37c748,0xd861c74a,0xdd37c74a,0xd861c74c,0xdd39c74c,0xd861c74e,
	0xdd39c74e,0xd861c750,0xdd3bc750,0xd861c752,0xdd3bc752,0xd861c754,0xdd3dc754,0xd861c756,
	0xdd3dc756,0xd861c758,0xdd3fc758,0xd861c75a,0xdd3fc75a,0xd861c75c,0xdd41c75c,0xd861c75e,
	0xdd41c75e,0xd861c760,0xdd43c760,0xd861c762,0xdd43c762,0xd861c764,0xdd45c764,0xd861c766,
	0xdd45c766,0xd861c768,0xdd47c768,0xd861c76a,0xdd47c76a,0xd861c76c,0xdd49c76c,0xd861c76e,
	0xdd49c76e,0xd861c770,0xdd4bc770,0xd861c772,0xdd4bc772,0xd861c774,0xdd4dc774,0xd861c776,
	0xdd4dc776,0xd861c778,0xdd4fc778,0xd861c77a,0xdd4fc77a,0xd861c77c,0xdd51c77c,0xd861c77e,
	0xdd51c77e,0xd861c780,0xdd53c780,0xd861c782,0xdd53c782,0xd861c784,0xdd55c784,0xd861c786,
	0xdd55c786,0xd861c788,0xdd57c788,0xd861c78a,0xdd57c78a,0xd861c78c,0xdd59c78c,0xd861c78e,
	0xdd59c78e,0xd861c790,0xdd5bc790,0xd861c792,0xdd5bc792,0xd861c794,0xdd5dc794,0xd861c796,
	0xdd5dc796,0xd861c798,0xdd5fc798,0xd861c79a,0xdd5fc79a,0xd861c79c,0xdd61c79c,0xd861c79e,
	0xdd61c79e,0xd861c7a0,0xdd63c7a0,0xd861c7a2,0xdd63c7a2,0xd861c7a4,0xdd65c7a4,0xd861c7a6,
	0xdd65c7a6,0xd861c7a8,0xdd67c7a8,0xd861c7aa,0xdd67c7aa,0xd861c7ac,0xdd69c7ac,0xd861c7ae,
	0xdd69c7ae,0xd861c7b0,0xdd6bc7b0,0xd861c7b2,0xdd6bc7b2,0xd861c7b4,0xdd6dc7b4,0xd861c7b6,
	0xdd6dc7b6,0xd861c7b8,0xdd6fc7b8,0xd861c7ba,0xdd6fc7ba,0xd861c7bc,0xdd71c7bc,0xd861c7be,
	0xdd71c7be,0xd861c7c0,0xdd73c7c0,0xd861c7c2,0xdd73c7c2,0xd861c7c4,0xdd75c7c4,0xd861c7c6,
	0xdd75c7c6,0xd861c7c8,0xdd77c7c8,0xd861c7ca,0xdd77c7ca,0xd861c7cc,0xdd79c7cc,0xd861c7ce,
	0xdd79c7ce,0xd861c7d0,0xdd7bc7d0,0xd861c7d2,0xdd7bc7d2,0xd861c7d4,0xdd7dc7d4,0xd861c7d6,
	0xdd7dc7d6,0xd861c7d8,0xdd7fc7d8,0xd861c7da,0xdd7fc7da,0xd861c7dc,0xdd81c7dc,0xd861c7de,
	0xdd81c7de,0xd861c7e0,0xdd83c7e0,0xd861c7e2,0xdd83c7e2,0xd861c7e4,0xdd85c7e4,0xd861c7e6,
	0xdd85c7e6,0xd861c7e8,0xdd87c7e8,0xd861c7ea,0xdd87c7ea,0xd861c7ec,0xdd89c7ec,0xd861c7ee,
	0xdd89c7ee,0xd861c7f0,0xdd8bc7f0,0xd861c7f2,0xdd8bc7f2,0xd861c7f4,0xdd8dc7f4,0xd861c7f6,
	0xdd8dc7f6,0xd861c7f8,0xdd8fc7f8,0xd861c7fa,0xdd8fc7fa,0xd861c7fc,0xdd91c7fc,0xd861c7fe,
	0xdd91c7fe,0xd861c800,0xdd93c800,0xd861c802,0xdd93c802,0xd861c804,0xdd95c804,0xd861c806,
	0xdd95c806,0xd861c808,0xdd97c808,0xd861c80a,0xdd97c80a,0xd861c80c,0xdd99c80c,0xd861c80e,
	0xdd99c80e,0xd861c810,0xdd9bc810,0xd861c812,0xdd9bc812,0xd861c814,0xdd9dc814,0xd861c816,
	0xdd9dc816,0xd861c818,0xdd9fc818,0xd861c81a,0xdd9fc81a,0xd861c81c,0xdda1c81c,0xd861c81e,
	0xdda1c81e,0xd861c820,0xdda3c820,0xd861c822,0xdda3c822,0xd861c824,0xdda5c824,0xd861c826,
	0xdda5c826,0xd861c828,0xdda7c828,0xd861c82a,0xdda7c82a,0xd861c82c,0xdda9c82c,0xd861c82e,
	0xdda9c82e,0xd861c830,0xddabc830,0xd861c832,0xddabc832,0xd861c834,0xddadc834,0xd861c836,
	0xddadc836,0xd861c838,0xddafc838,0xd861c83a,0xddafc83a,0xd861c83c,0xddb1c83c,0xd861c83e,
	0xddb1c83e,0xd861c840,0xddb3c840,0xd861c842,0xddb3c842,0xd861c844,0xddb5c844,0xd861c846,
	0xddb5c846,0xd861c848,0xddb7c848,0xd861c84a,0xddb7c84a,0xd861c84c,0xddb9c84c,0xd861c84e,
	0xddb9c84e,0xd861c850,0xddbbc850,0xd861c852,0xddbbc852,0xd861c854,0xddbdc854,0xd861c856,
	0xddbdc856,0xd861c858,0xddbfc858,0xd861c85a,0xddbfc85a,0xd861c85c,0xddc1c85c,0xd861c85e,
	0xddc1c85e,0xd861c860,0xddc3c860,0xd861c862,0xddc3c862,0xd861c864,0xddc5c864,0xd861c866,
	0xddc5c866,0xd861c868,0xddc7c868,0xd861c86a,0xddc7c86a,0xd861c86c,0xddc9c86c,0xd861c86e,
	0xddc9c86e,0xd861c870,0xddcbc870,0xd861c872,0xddcbc872,0xd861c874,0xddcdc874,0xd861c876,
	0xddcdc876,0xd861c878,0xddcfc878,0xd861c87a,0xddcfc87a,0xd861c87c,0xddd1c87c,0xd861c87e,
	0xddd1c87e,0xd861c880,0xddd3c880,0xd861c882,0xddd3c882,0xd861c884,0xddd5c884,0xd861c886,
	0xddd5c886,0xd861c888,0xddd7c888,0xd861c88a,0xddd7c88a,0xd861c88c,0xddd9c88c,0xd861c88e,
	0xddd9c88e,0xd861c890,0xdddbc890,0xd861c892,0xdddbc892,0xd861c894,0xddddc894,0xd861c896,
	0xddddc896,0xd861c898,0xdddfc898,0xd861c89a,0xdddfc89a,0xd861c89c,0xdde1c89c,0xd861c89e,
	0xdde1c89e,0xd861c8a0,0xdde3c8a0,0xd861c8a2,0xdde3c8a2,0xd861c8a4,0xdde5c8a4,0xd861c8a6,
	0xdde5c8a6,0xd861c8a8,0xdde7c8a8,0xd861c8aa,0xdde7c8aa,0xd861c8ac,0xdde9c8ac,0xd861c8ae,
	0xdde9c8ae,0xd861c8b0,0xddebc8b0,0xd861c8b2,0xddebc8b2,0xd861c8b4,0xddedc8b4,0xd861c8b6,
	0xddedc8b6,0xd861c8b8,0xddefc8b8,0xd861c8ba,0xddefc8ba,0xd861c8bc,0xddf1c8bc,0xd861c8be,
	0xddf1c8be,0xd861c8c0,0xddf3c8c0,0xd861c8c2,0xddf3c8c2,0xd861c8c4,0xddf5c8c4,0xd861c8c6,
	0xddf5c8c6,0xd861c8c8,0xddf7c8c8,0xd861c8ca,0xddf7c8ca,0xd861c8cc,0xddf9c8cc,0xd861c8ce,
	0xddf9c8ce,0xd861c8d0,0xddfbc8d0,0xd861c8d2,0xddfbc8d2,0xd861c8d4,0xddfdc8d4,0xd861c8d6,
	0xddfdc8d6,0xd861c8d8,0xddffc8d8,0xd861c8da,0xddffc8da,0xd861c8dc,0xde01c8dc,0xd861c8de,
	0xde01c8de,0xd861c8e0,0xde03c8e0,0xd861c8e2,0xde03c8e2,0xd861c8e4,0xde05c8e4,0xd861c8e6,
	0xde05c8e6,0xd861c8e8,0xde07c8e8,0xd861c8ea,0xde07c8ea,0xd861c8ec,0xde09c8ec,0xd861c8ee,
	0xde09c8ee,0xd861c8f0,0xde0bc8f0,0xd861c8f2,0xde0bc8f2,0xd861c8f4,0xde0dc8f4,0xd861c8f6,
	0xde0dc8f6,0xd861c8f8,0xde0fc8f8,0xd861c8fa,0xde0fc8fa,0xd861c8fc,0xde11c8fc,0xd861c8fe,
	0xde11c8fe,0xd861c900,0xde13c900,0xd861c902,0xde13c902,0xd861c904,0xde15c904,0xd861c906,
	0xde15c906,0xd861c908,0xde17c908,0xd861c90a,0xde17c90a,0xd861c90c,0xde19c90c,0xd861c90e,
	0xde19c90e,0xd861c910,0xde1bc910,0xd861c912,0xde1bc912,0xd861c914,0xde1dc914,0xd861c916,
	0xde1dc916,0xd861c918,0xde1fc918,0xd861c91a,0xde1fc91a,0xd861c91c,0xde21c91c,0xd861c91e,
	0xde21c91e,0xd861c920,0xde23c920,0xd861c922,0xde23c922,0xd861c924,0xde25c924,0xd861c926,
	0xde25c926,0xd861c928,0xde27c928,0xd861c92a,0xde27c92a,0xd861c92c,0xde29c92c,0xd861c92e,
	0xde29c92e,0xd861c930,0xde2bc930,0xd861c932,0xde2bc932,0xd861c934,0xde2dc934,0xd861c936,
	0xde2dc936,0xd861c938,0xde2fc938,0xd861c93a,0xde2fc93a,0xd861c93c,0xde31c93c,0xd861c93e,
	0xde31c93e,0xd861c940,0xde33c940,0xd861c942,0xde33c942,0xd861c944,0xde35c944,0xd861c946,
	0xde35c946,0xd861c948,0xde37c948,0xd861c94a,0xde37c94a,0xd861c94c,0xde39c94c,0xd861c94e,
	0xde39c94e,0xd861c950,0xde3bc950,0xd861c952,0xde3bc952,0xd861c954,0xde3dc954,0xd861c956,
	0xde3dc956,0xd861c958,0xde3fc958,0xd861c95a,0xde3fc95a,0xd861c95c,0xde41c95c,0xd861c95e,
	0xde41c95e,0xd861c960,0xde43c960,0xd861c962,0xde43c962,0xd861c964,0xde45c964,0xd861c966,
	0xde45c966,0xd861c968,0xde47c968,0xd861c96a,0xde47c96a,0xd861c96c,0xde49c96c,0xd861c96e,
	0xde49c96e,0xd861c970,0xde4bc970,0xd861c972,0xde4bc972,0xd861c974,0xde4dc974,0xd861c976,
	0xde4dc976,0xd861c978,0xde4fc978,0xd861c97a,0xde4fc97a,0xd861c97c,0xde51c97c,0xd861c97e,
	0xde51c97e,0xd861c980,0xde53c980,0xd861c982,0xde53c982,0xd861c984,0xde55c984,0xd861c986,
	0xde55c986,0xd861c988,0xde57c988,0xd861c98a,0xde57c98a,0xd861c98c,0xde59c98c,0xd861c98e,
	0xde59c98e,0xd861c990,0xde5bc990,0xd861c992,0xde5bc992,0xd861c994,0xde5dc994,0xd861c996,
	0xde5dc996,0xd861c998,0xde5fc998,0xd861c99a,0xde5fc99a,0xd861c99c,0xde61c99c,0xd861c99e,
	0xde61c99e,0xd861c9a0,0xde63c9a0,0xd861c9a2,0xde63c9a2,0xd861c9a4,0xde65c9a4,0xd861c9a6,
	0xde65c9a6,0xd861c9a8,0xde67c9a8,0xd861c9aa,0xde67c9aa,0xd861c9ac,0xde69c9ac,0xd861c9ae,
	0xde69c9ae,0xd861c9b0,0xde6bc9b0,0xd861c9b2,0xde6bc9b2,0xd861c9b4,0xde6dc9b4,0xd861c9b6,
	0xde6dc9b6,0xd861c9b8,0xde6fc9b8,0xd861c9ba,0xde6fc9ba,0xd861c9bc,0xde71c9bc,0xd861c9be,
	0xde71c9be,0xd861c9c0,0xde73c9c0,0xd861c9c2,0xde73c9c2,0xd861c9c4,0xde75c9c4,0xd861c9c6,
	0xde75c9c6,0xd861c9c8,0xde77c9c8,0xd861c9ca,0xde77c9ca,0xd861c9cc,0xde79c9cc,0xd861c9ce,
	0xde79c9ce,0xd861c9d0,0xde7bc9d0,0xd861c9d2,0xde7bc9d2,0xd861c9d4,0xde7dc9d4,0xd861c9d6,
	0xde7dc9d6,0xd861c9d8,0xde7fc9d8,0xd861c9da,0xde7fc9da,0xd861c9dc,0xde81c9dc,0xd861c9de,
	0xde81c9de,0xd861c9e0,0xde83c9e0,0xd861c9e2,0xde83c9e2,0xd861c9e4,0xde85c9e4,0xd861c9e6,
	0xde85c9e6,0xd861c9e8,0xde87c9e8,0xd861c9ea,0xde87c9ea,0xd861c9ec,0xde89c9ec,0xd861c9ee,
	0xde89c9ee,0xd861c9f0,0xde8bc9f0,0xd861c9f2,0xde8bc9f2,0xd861c9f4,0xde8dc9f4,0xd861c9f6,
	0xde8dc9f6,0xd861c9f8,0xde8fc9f8,0xd861c9fa,0xde8fc9fa,0xd861c9fc,0xde91c9fc,0xd861c9fe,
	0xde91c9fe,0xd861ca00,0xde93ca00,0xd861ca02,0xde93ca02,0xd861ca04,0xde95ca04,0xd861ca06,
	0xde95ca06,0xd861ca08,0xde97ca08,0xd861ca0a,0xde97ca0a,0xd861ca0c,0xde99ca0c,0xd861ca0e,
	0xde99ca0e,0xd861ca10,0xde9bca10,0xd861ca12,0xde9bca12,0xd861ca14,0xde9dca14,0xd861ca16,
	0xde9dca16,0xd861ca18,0xde9fca18,0xd861ca1a,0xde9fca1a,0xd861ca1c,0xdea1ca1c,0xd861ca1e,
	0xdea1ca1e,0xd861ca20,0xdea3ca20,0xd861ca22,0xdea3ca22,0xd861ca24,0xdea5ca24,0xd861ca26,
	0xdea5ca26,0xd861ca28,0xdea7ca28,0xd861ca2a,0xdea7ca2a,0xd861ca2c,0xdea9ca2c,0xd861ca2e,
	0xdea9ca2e,0xd861ca30,0xdeabca30,0xd861ca32,0xdeabca32,0xd861ca34,0xdeadca34,0xd861ca36,
	0xdeadca36,0xd861ca38,0xdeafca38,0xd861ca3a,0xdeafca3a,0xd861ca3c,0xdeb1ca3c,0xd861ca3e,
	0xdeb1ca3e,0xd861ca40,0xdeb3ca40,0xd861ca42,0xdeb3ca42,0xd861ca44,0xdeb5ca44,0xd861ca46,
	0xdeb5ca46,0xd861ca48,0xdeb7ca48,0xd861ca4a,0xdeb7ca4a,0xd861ca4c,0xdeb9ca4c,0xd861ca4e,
	0xdeb9ca4e,0xd861ca50,0xdebbca50,0xd861ca52,0xdebbca52,0xd861ca54,0xdebdca54,0xd861ca56,
	0xdebdca56,0xd861ca58,0xdebfca58,0xd861ca5a,0xdebfca5a,0xd861ca5c,0xdec1ca5c,0xd861ca5e,
	0xdec1ca5e,0xd861ca60,0xdec3ca60,0xd861ca62,0xdec3ca62,0xd861ca64,0xdec5ca64,0xd861ca66,
	0xdec5ca66,0xd861ca68,0xdec7ca68,0xd861ca6a,0xdec7ca6a,0xd861ca6c,0xdec9ca6c,0xd861ca6e,
	0xdec9ca6e,0xd861ca70,0xdecbca70,0xd861ca72,0xdecbca72,0xd861ca74,0xdecdca74,0xd861ca76,
	0xdecdca76,0xd861ca78,0xdecfca78,0xd861ca7a,0xdecfca7a,0xd861ca7c,0xded1ca7c,0xd861ca7e,
	0xded1ca7e,0xd861ca80,0xded3ca80,0xd861ca82,0xded3ca82,0xd861ca84,0xded5ca84,0xd861ca86,
	0xded5ca86,0xd861ca88,0xded7ca88,0xd861ca8a,0xded7ca8a,0xd861ca8c,0xded9ca8c,0xd861ca8e,
	0xded9ca8e,0xd861ca90,0xdedbca90,0xd861ca92,0xdedbca92,0xd861ca94,0xdeddca94,0xd861ca96,
	0xdeddca96,0xd861ca98,0xdedfca98,0xd861ca9a,0xdedfca9a,0xd861ca9c,0xdee1ca9c,0xd861ca9e,
	0xdee1ca9e,0xd861caa0,0xdee3caa0,0xd861caa2,0xdee3caa2,0xd861caa4,0xdee5caa4,0xd861caa6,
	0xdee5caa6,0xd861caa8,0xdee7caa8,0xd861caaa,0xdee7caaa,0xd861caac,0xdee9caac,0xd861caae,
	0xdee9caae,0xd861cab0,0xdeebcab0,0xd861cab2,0xdeebcab2,0xd861cab4,0xdeedcab4,0xd861cab6,
	0xdeedcab6,0xd861cab8,0xdeefcab8,0xd861caba,0xdeefcaba,0xd861cabc,0xdef1cabc,0xd861cabe,
	0xdef1cabe,0xd861cac0,0xdef3cac0,0xd861cac2,0xdef3cac2,0xd861cac4,0xdef5cac4,0xd861cac6,
	0xdef5cac6,0xd861cac8,0xdef7cac8,0xd861caca,0xdef7caca,0xd861cacc,0xdef9cacc,0xd861cace,
	0xdef9cace,0xd861cad0,0xdefbcad0,0xd861cad2,0xdefbcad2,0xd861cad4,0xdefdcad4,0xd861cad6,
	0xdefdcad6,0xd861cad8,0xdeffcad8,0xd861cada,0xdeffcada,0xd861cadc,0xdf01cadc,0xd861cade,
	0xdf01cade,0xd861cae0,0xdf03cae0,0xd861cae2,0xdf03cae2,0xd861cae4,0xdf05cae4,0xd861cae6,
	0xdf05cae6,0xd861cae8,0xdf07cae8,0xd861caea,0xdf07caea,0xd861caec,0xdf09caec,0xd861caee,
	0xdf09caee,0xd861caf0,0xdf0bcaf0,0xd861caf2,0xdf0bcaf2,0xd861caf4,0xdf0dcaf4,0xd861caf6,
	0xdf0dcaf6,0xd861caf8,0xdf0fcaf8,0xd861cafa,0xdf0fcafa,0xd861cafc,0xdf11cafc,0xd861cafe,
	0xdf11cafe,0xd861cb00,0xdf13cb00,0xd861cb02,0xdf13cb02,0xd861cb04,0xdf15cb04,0xd861cb06,
	0xdf15cb06,0xd861cb08,0xdf17cb08,0xd861cb0a,0xdf17cb0a,0xd861cb0c,0xdf19cb0c,0xd861cb0e,
	0xdf19cb0e,0xd861cb10,0xdf1bcb10,0xd861cb12,0xdf1bcb12,0xd861cb14,0xdf1dcb14,0xd861cb16,
	0xdf1dcb16,0xd861cb18,0xdf1fcb18,0xd861cb1a,0xdf1fcb1a,0xd861cb1c,0xdf21cb1c,0xd861cb1e,
	0xdf21cb1e,0xd861cb20,0xdf23cb20,0xd861cb22,0xdf23cb22,0xd861cb24,0xdf25cb24,0xd861cb26,
	0xdf25cb26,0xd861cb28,0xdf27cb28,0xd861cb2a,0xdf27cb2a,0xd861cb2c,0xdf29cb2c,0xd861cb2e,
	0xdf29cb2e,0xd861cb30,0xdf2bcb30,0xd861cb32,0xdf2bcb32,0xd861cb34,0xdf2dcb34,0xd861cb36,
	0xdf2dcb36,0xd861cb38,0xdf2fcb38,0xd861cb3a,0xdf2fcb3a,0xd861cb3c,0xdf31cb3c,0xd861cb3e,
	0xdf31cb3e,0xd861cb40,0xdf33cb40,0xd861cb42,0xdf33cb42,0xd861cb44,0xdf35cb44,0xd861cb46,
	0xdf35cb46,0xd861cb48,0xdf37cb48,0xd861cb4a,0xdf37cb4a,0xd861cb4c,0xdf39cb4c,0xd861cb4e,
	0xdf39cb4e,0xd861cb50,0xdf3bcb50,0xd861cb52,0xdf3bcb52,0xd861cb54,0xdf3dcb54,0xd861cb56,
	0xdf3dcb56,0xd861cb58,0xdf3fcb58,0xd861cb5a,0xdf3fcb5a,0xd861cb5c,0xdf41cb5c,0xd861cb5e,
	0xdf41cb5e,0xd861cb60,0xdf43cb60,0xd861cb62,0xdf43cb62,0xd861cb64,0xdf45cb64,0xd861cb66,
	0xdf45cb66,0xd861cb68,0xdf47cb68,0xd861cb6a,0xdf47cb6a,0xd861cb6c,0xdf49cb6c,0xd861cb6e,
	0xdf49cb6e,0xd861cb70,0xdf4bcb70,0xd861cb72,0xdf4bcb72,0xd861cb74,0xdf4dcb74,0xd861cb76,
	0xdf4dcb76,0xd861cb78,0xdf4fcb78,0xd861cb7a,0xdf4fcb7a,0xd861cb7c,0xdf51cb7c,0xd861cb7e,
	0xdf51cb7e,0xd861cb80,0xdf53cb80,0xd861cb82,0xdf53cb82,0xd861cb84,0xdf55cb84,0xd861cb86,
	0xdf55cb86,0xd861cb88,0xdf57cb88,0xd861cb8a,0xdf57cb8a,0xd861cb8c,0xdf59cb8c,0xd861cb8e,
	0xdf59cb8e,0xd861cb90,0xdf5bcb90,0xd861cb92,0xdf5bcb92,0xd861cb94,0xdf5dcb94,0xd861cb96,
	0xdf5dcb96,0xd861cb98,0xdf5fcb98,0xd861cb9a,0xdf5fcb9a,0xd861cb9c,0xdf61cb9c,0xd861cb9e,
	0xdf61cb9e,0xd861cba0,0xdf63cba0,0xd861cba2,0xdf63cba2,0xd861cba4,0xdf65cba4,0xd861cba6,
	0xdf65cba6,0xd861cba8,0xdf67cba8,0xd861cbaa,0xdf67cbaa,0xd861cbac,0xdf69cbac,0xd861cbae,
	0xdf69cbae,0xd861cbb0,0xdf6bcbb0,0xd861cbb2,0xdf6bcbb2,0xd861cbb4,0xdf6dcbb4,0xd861cbb6,
	0xdf6dcbb6,0xd861cbb8,0xdf6fcbb8,0xd861cbba,0xdf6fcbba,0xd861cbbc,0xdf71cbbc,0xd861cbbe,
	0xdf71cbbe,0xd861cbc0,0xdf73cbc0,0xd861cbc2,0xdf73cbc2,0xd861cbc4,0xdf75cbc4,0xd861cbc6,
	0xdf75cbc6,0xd861cbc8,0xdf77cbc8,0xd861cbca,0xdf77cbca,0xd861cbcc,0xdf79cbcc,0xd861cbce,
	0xdf79cbce,0xd861cbd0,0xdf7bcbd0,0xd861cbd2,0xdf7bcbd2,0xd861cbd4,0xdf7dcbd4,0xd861cbd6,
	0xdf7dcbd6,0xd861cbd8,0xdf7fcbd8,0xd861cbda,0xdf7fcbda,0xd861cbdc,0xdf81cbdc,0xd861cbde,
	0xdf81cbde,0xd861cbe0,0xdf83cbe0,0xd861cbe2,0xdf83cbe2,0xd861cbe4,0xdf85cbe4,0xd861cbe6,
	0xdf85cbe6,0xd861cbe8,0xdf87cbe8,0xd861cbea,0xdf87cbea,0xd861cbec,0xdf89cbec,0xd861cbee,
	0xdf89cbee,0xd861cbf0,0xdf8bcbf0,0xd861cbf2,0xdf8bcbf2,0xd861cbf4,0xdf8dcbf4,0xd861cbf6,
	0xdf8dcbf6,0xd861cbf8,0xdf8fcbf8,0xd861cbfa,0xdf8fcbfa,0xd861cbfc,0xdf91cbfc,0xd861cbfe,
	0xdf91cbfe,0xd861cc00,0xdf93cc00,0xd861cc02,0xdf93cc02,0xd861cc04,0xdf95cc04,0xd861cc06,
	0xdf95cc06,0xd861cc08,0xdf97cc08,0xd861cc0a,0xdf97cc0a,0xd861cc0c,0xdf99cc0c,0xd861cc0e,
	0xdf99cc0e,0xd861cc10,0xdf9bcc10,0xd861cc12,0xdf9bcc12,0xd861cc14,0xdf9dcc14,0xd861cc16,
	0xdf9dcc16,0xd861cc18,0xdf9fcc18,0xd861cc1a,0xdf9fcc1a,0xd861cc1c,0xdfa1cc1c,0xd861cc1e,
	0xdfa1cc1e,0xd861cc20,0xdfa3cc20,0xd861cc22,0xdfa3cc22,0xd861cc24,0xdfa5cc24,0xd861cc26,
	0xdfa5cc26,0xd861cc28,0xdfa7cc28,0xd861cc2a,0xdfa7cc2a,0xd861cc2c,0xdfa9cc2c,0xd861cc2e,
	0xdfa9cc2e,0xd861cc30,0xdfabcc30,0xd861cc32,0xdfabcc32,0xd861cc34,0xdfadcc34,0xd861cc36,
	0xdfadcc36,0xd861cc38,0xdfafcc38,0xd861cc3a,0xdfafcc3a,0xd861cc3c,0xdfb1cc3c,0xd861cc3e,
	0xdfb1cc3e,0xd861cc40,0xdfb3cc40,0xd861cc42,0xdfb3cc42,0xd861cc44,0xdfb5cc44,0xd861cc46,
	0xdfb5cc46,0xd861cc48,0xdfb7cc48,0xd861cc4a,0xdfb7cc4a,0xd861cc4c,0xdfb9cc4c,0xd861cc4e,
	0xdfb9cc4e,0xd861cc50,0xdfbbcc50,0xd861cc52,0xdfbbcc52,0xd861cc54,0xdfbdcc54,0xd861cc56,
	0xdfbdcc56,0xd861cc58,0xdfbfcc58,0xd861cc5a,0xdfbfcc5a,0xd861cc5c,0xdfc1cc5c,0xd861cc5e,
	0xdfc1cc5e,0xd861cc60,0xdfc3cc60,0xd861cc62,0xdfc3cc62,0xd861cc64,0xdfc5cc64,0xd861cc66,
	0xdfc5cc66,0xd861cc68,0xdfc7cc68,0xd861cc6a,0xdfc7cc6a,0xd861cc6c,0xdfc9cc6c,0xd861cc6e,
	0xdfc9cc6e,0xd861cc70,0xdfcbcc70,0xd861cc72,0xdfcbcc72,0xd861cc74,0xdfcdcc74,0xd861cc76,
	0xdfcdcc76,0xd861cc78,0xdfcfcc78,0xd861cc7a,0xdfcfcc7a,0xd861cc7c,0xdfd1cc7c,0xd861cc7e,
	0xdfd1cc7e,0xd861cc80,0xdfd3cc80,0xd861cc82,0xdfd3cc82,0xd861cc84,0xdfd5cc84,0xd861cc86,
	0xdfd5cc86,0xd861cc88,0xdfd7cc88,0xd861cc8a,0xdfd7cc8a,0xd861cc8c,0xdfd9cc8c,0xd861cc8e,
	0xdfd9cc8e,0xd861cc90,0xdfdbcc90,0xd861cc92,0xdfdbcc92,0xd861cc94,0xdfddcc94,0xd861cc96,
	0xdfddcc96,0xd861cc98,0xdfdfcc98,0xd861cc9a,0xdfdfcc9a,0xd861cc9c,0xdfe1cc9c,0xd861cc9e,
	0xdfe1cc9e,0xd861cca0,0xdfe3cca0,0xd861cca2,0xdfe3cca2,0xd861cca4,0xdfe5cca4,0xd861cca6,
	0xdfe5cca6,0xd861cca8,0xdfe7cca8,0xd861ccaa,0xdfe7ccaa,0xd861ccac,0xdfe9ccac,0xd861ccae,
	0xdfe9ccae,0xd861ccb0,0xdfebccb0,0xd861ccb2,0xdfebccb2,0xd861ccb4,0xdfedccb4,0xd861ccb6,
	0xdfedccb6,0xd861ccb8,0xdfefccb8,0xd861ccba,0xdfefccba,0xd861ccbc,0xdff1ccbc,0xd861ccbe,
	0xdff1ccbe,0xd861ccc0,0xdff3ccc0,0xd861ccc2,0xdff3ccc2,0xd861ccc4,0xdff5ccc4,0xd861ccc6,
	0xdff5ccc6,0xd861ccc8,0xdff7ccc8,0xd861ccca,0xdff7ccca,0xd861cccc,0xdff9cccc,0xd861ccce,
	0xdff9ccce,0xd861ccd0,0xdffbccd0,0xd861ccd2,0xdffbccd2,0xd861ccd4,0xdffdccd4,0xd861ccd6,
	0xdffdccd6,0xd861ccd8,0xdfffccd8,0xd861ccda,0xdfffccda,0xd863ccdc,0xdc01ccdc,0xd863ccde,
	0xdc01ccde,0xd863cce0,0xdc03cce0,0xd863cce2,0xdc03cce2,0xd863cce4,0xdc05cce4,0xd863cce6,
	0xdc05cce6,0xd863cce8,0xdc07cce8,0xd863ccea,0xdc07ccea,0xd863ccec,0xdc09ccec,0xd863ccee,
	0xdc09ccee,0xd863ccf0,0xdc0bccf0,0xd863ccf2,0xdc0bccf2,0xd863ccf4,0xdc0dccf4,0xd863ccf6,
	0xdc0dccf6,0xd863ccf8,0xdc0fccf8,0xd863ccfa,0xdc0fccfa,0xd863ccfc,0xdc11ccfc,0xd863ccfe,
	0xdc11ccfe,0xd863cd00,0xdc13cd00,0xd863cd02,0xdc13cd02,0xd863cd04,0xdc15cd04,0xd863cd06,
	0xdc15cd06,0xd863cd08,0xdc17cd08,0xd863cd0a,0xdc17cd0a,0xd863cd0c,0xdc19cd0c,0xd863cd0e,
	0xdc19cd0e,0xd863cd10,0xdc1bcd10,0xd863cd12,0xdc1bcd12,0xd863cd14,0xdc1dcd14,0xd863cd16,
	0xdc1dcd16,0xd863cd18,0xdc1fcd18,0xd863cd1a,0xdc1fcd1a,0xd863cd1c,0xdc21cd1c,0xd863cd1e,
	0xdc21cd1e,0xd863cd20,0xdc23cd20,0xd863cd22,0xdc23cd22,0xd863cd24,0xdc25cd24,0xd863cd26,
	0xdc25cd26,0xd863cd28,0xdc27cd28,0xd863cd2a,0xdc27cd2a,0xd863cd2c,0xdc29cd2c,0xd863cd2e,
	0xdc29cd2e,0xd863cd30,0xdc2bcd30,0xd863cd32,0xdc2bcd32,0xd863cd34,0xdc2dcd34,0xd863cd36,
	0xdc2dcd36,0xd863cd38,0xdc2fcd38,0xd863cd3a,0xdc2fcd3a,0xd863cd3c,0xdc31cd3c,0xd863cd3e,
	0xdc31cd3e,0xd863cd40,0xdc33cd40,0xd863cd42,0xdc33cd42,0xd863cd44,0xdc35cd44,0xd863cd46,
	0xdc35cd46,0xd863cd48,0xdc37cd48,0xd863cd4a,0xdc37cd4a,0xd863cd4c,0xdc39cd4c,0xd863cd4e,
	0xdc39cd4e,0xd863cd50,0xdc3bcd50,0xd863cd52,0xdc3bcd52,0xd863cd54,0xdc3dcd54,0xd863cd56,
	0xdc3dcd56,0xd863cd58,0xdc3fcd58,0xd863cd5a,0xdc3fcd5a,0xd863cd5c,0xdc41cd5c,0xd863cd5e,
	0xdc41cd5e,0xd863cd60,0xdc43cd60,0xd863cd62,0xdc43cd62,0xd863cd64,0xdc45cd64,0xd863cd66,
	0xdc45cd66,0xd863cd68,0xdc47cd68,0xd863cd6a,0xdc47cd6a,0xd863cd6c,0xdc49cd6c,0xd863cd6e,
	0xdc49cd6e,0xd863cd70,0xdc4bcd70,0xd863cd72,0xdc4bcd72,0xd863cd74,0xdc4dcd74,0xd863cd76,
	0xdc4dcd76,0xd863cd78,0xdc4fcd78,0xd863cd7a,0xdc4fcd7a,0xd863cd7c,0xdc51cd7c,0xd863cd7e,
	0xdc51cd7e,0xd863cd80,0xdc53cd80,0xd863cd82,0xdc53cd82,0xd863cd84,0xdc55cd84,0xd863cd86,
	0xdc55cd86,0xd863cd88,0xdc57cd88,0xd863cd8a,0xdc57cd8a,0xd863cd8c,0xdc59cd8c,0xd863cd8e,
	0xdc59cd8e,0xd863cd90,0xdc5bcd90,0xd863cd92,0xdc5bcd92,0xd863cd94,0xdc5dcd94,0xd863cd96,
	0xdc5dcd96,0xd863cd98,0xdc5fcd98,0xd863cd9a,0xdc5fcd9a,0xd863cd9c,0xdc61cd9c,0xd863cd9e,
	0xdc61cd9e,0xd863cda0,0xdc63cda0,0xd863cda2,0xdc63cda2,0xd863cda4,0xdc65cda4,0xd863cda6,
	0xdc65cda6,0xd863cda8,0xdc67cda8,0xd863cdaa,0xdc67cdaa,0xd863cdac,0xdc69cdac,0xd863cdae,
	0xdc69cdae,0xd863cdb0,0xdc6bcdb0,0xd863cdb2,0xdc6bcdb2,0xd863cdb4,0xdc6dcdb4,0xd863cdb6,
	0xdc6dcdb6,0xd863cdb8,0xdc6fcdb8,0xd863cdba,0xdc6fcdba,0xd863cdbc,0xdc71cdbc,0xd863cdbe,
	0xdc71cdbe,0xd863cdc0,0xdc73cdc0,0xd863cdc2,0xdc73cdc2,0xd863cdc4,0xdc75cdc4,0xd863cdc6,
	0xdc75cdc6,0xd863cdc8,0xdc77cdc8,0xd863cdca,0xdc77cdca,0xd863cdcc,0xdc79cdcc,0xd863cdce,
	0xdc79cdce,0xd863cdd0,0xdc7bcdd0,0xd863cdd2,0xdc7bcdd2,0xd863cdd4,0xdc7dcdd4,0xd863cdd6,
	0xdc7dcdd6,0xd863cdd8,0xdc7fcdd8,0xd863cdda,0xdc7fcdda,0xd863cddc,0xdc81cddc,0xd863cdde,
	0xdc81cdde,0xd863cde0,0xdc83cde0,0xd863cde2,0xdc83cde2,0xd863cde4,0xdc85cde4,0xd863cde6,
	0xdc85cde6,0xd863cde8,0xdc87cde8,0xd863cdea,0xdc87cdea,0xd863cdec,0xdc89cdec,0xd863cdee,
	0xdc89cdee,0xd863cdf0,0xdc8bcdf0,0xd863cdf2,0xdc8bcdf2,0xd863cdf4,0xdc8dcdf4,0xd863cdf6,
	0xdc8dcdf6,0xd863cdf8,0xdc8fcdf8,0xd863cdfa,0xdc8fcdfa,0xd863cdfc,0xdc91cdfc,0xd863cdfe,
	0xdc91cdfe,0xd863ce00,0xdc93ce00,0xd863ce02,0xdc93ce02,0xd863ce04,0xdc95ce04,0xd863ce06,
	0xdc95ce06,0xd863ce08,0xdc97ce08,0xd863ce0a,0xdc97ce0a,0xd863ce0c,0xdc99ce0c,0xd863ce0e,
	0xdc99ce0e,0xd863ce10,0xdc9bce10,0xd863ce12,0xdc9bce12,0xd863ce14,0xdc9dce14,0xd863ce16,
	0xdc9dce16,0xd863ce18,0xdc9fce18,0xd863ce1a,0xdc9fce1a,0xd863ce1c,0xdca1ce1c,0xd863ce1e,
	0xdca1ce1e,0xd863ce20,0xdca3ce20,0xd863ce22,0xdca3ce22,0xd863ce24,0xdca5ce24,0xd863ce26,
	0xdca5ce26,0xd863ce28,0xdca7ce28,0xd863ce2a,0xdca7ce2a,0xd863ce2c,0xdca9ce2c,0xd863ce2e,
	0xdca9ce2e,0xd863ce30,0xdcabce30,0xd863ce32,0xdcabce32,0xd863ce34,0xdcadce34,0xd863ce36,
	0xdcadce36,0xd863ce38,0xdcafce38,0xd863ce3a,0xdcafce3a,0xd863ce3c,0xdcb1ce3c,0xd863ce3e,
	0xdcb1ce3e,0xd863ce40,0xdcb3ce40,0xd863ce42,0xdcb3ce42,0xd863ce44,0xdcb5ce44,0xd863ce46,
	0xdcb5ce46,0xd863ce48,0xdcb7ce48,0xd863ce4a,0xdcb7ce4a,0xd863ce4c,0xdcb9ce4c,0xd863ce4e,
	0xdcb9ce4e,0xd863ce50,0xdcbbce50,0xd863ce52,0xdcbbce52,0xd863ce54,0xdcbdce54,0xd863ce56,
	0xdcbdce56,0xd863ce58,0xdcbfce58,0xd863ce5a,0xdcbfce5a,0xd863ce5c,0xdcc1ce5c,0xd863ce5e,
	0xdcc1ce5e,0xd863ce60,0xdcc3ce60,0xd863ce62,0xdcc3ce62,0xd863ce64,0xdcc5ce64,0xd863ce66,
	0xdcc5ce66,0xd863ce68,0xdcc7ce68,0xd863ce6a,0xdcc7ce6a,0xd863ce6c,0xdcc9ce6c,0xd863ce6e,
	0xdcc9ce6e,0xd863ce70,0xdccbce70,0xd863ce72,0xdccbce72,0xd863ce74,0xdccdce74,0xd863ce76,
	0xdccdce76,0xd863ce78,0xdccfce78,0xd863ce7a,0xdccfce7a,0xd863ce7c,0xdcd1ce7c,0xd863ce7e,
	0xdcd1ce7e,0xd863ce80,0xdcd3ce80,0xd863ce82,0xdcd3ce82,0xd863ce84,0xdcd5ce84,0xd863ce86,
	0xdcd5ce86,0xd863ce88,0xdcd7ce88,0xd863ce8a,0xdcd7ce8a,0xd863ce8c,0xdcd9ce8c,0xd863ce8e,
	0xdcd9ce8e,0xd863ce90,0xdcdbce90,0xd863ce92,0xdcdbce92,0xd863ce94,0xdcddce94,0xd863ce96,
	0xdcddce96,0xd863ce98,0xdcdfce98,0xd863ce9a,0xdcdfce9a,0xd863ce9c,0xdce1ce9c,0xd863ce9e,
	0xdce1ce9e,0xd863cea0,0xdce3cea0,0xd863cea2,0xdce3cea2,0xd863cea4,0xdce5cea4,0xd863cea6,
	0xdce5cea6,0xd863cea8,0xdce7cea8,0xd863ceaa,0xdce7ceaa,0xd863ceac,0xdce9ceac,0xd863ceae,
	0xdce9ceae,0xd863ceb0,0xdcebceb0,0xd863ceb2,0xdcebceb2,0xd863ceb4,0xdcedceb4,0xd863ceb6,
	0xdcedceb6,0xd863ceb8,0xdcefceb8,0xd863ceba,0xdcefceba,0xd863cebc,0xdcf1cebc,0xd863cebe,
	0xdcf1cebe,0xd863cec0,0xdcf3cec0,0xd863cec2,0xdcf3cec2,0xd863cec4,0xdcf5cec4,0xd863cec6,
	0xdcf5cec6,0xd863cec8,0xdcf7cec8,0xd863ceca,0xdcf7ceca,0xd863cecc,0xdcf9cecc,0xd863cece,
	0xdcf9cece,0xd863ced0,0xdcfbced0,0xd863ced2,0xdcfbced2,0xd863ced4,0xdcfdced4,0xd863ced6,
	0xdcfdced6,0xd863ced8,0xdcffced8,0xd863ceda,0xdcffceda,0xd863cedc,0xdd01cedc,0xd863cede,
	0xdd01cede,0xd863cee0,0xdd03cee0,0xd863cee2,0xdd03cee2,0xd863cee4,0xdd05cee4,0xd863cee6,
	0xdd05cee6,0xd863cee8,0xdd07cee8,0xd863ceea,0xdd07ceea,0xd863ceec,0xdd09ceec,0xd863ceee,
	0xdd09ceee,0xd863cef0,0xdd0bcef0,0xd863cef2,0xdd0bcef2,0xd863cef4,0xdd0dcef4,0xd863cef6,
	0xdd0dcef6,0xd863cef8,0xdd0fcef8,0xd863cefa,0xdd0fcefa,0xd863cefc,0xdd11cefc,0xd863cefe,
	0xdd11cefe,0xd863cf00,0xdd13cf00,0xd863cf02,0xdd13cf02,0xd863cf04,0xdd15cf04,0xd863cf06,
	0xdd15cf06,0xd863cf08,0xdd17cf08,0xd863cf0a,0xdd17cf0a,0xd863cf0c,0xdd19cf0c,0xd863cf0e,
	0xdd19cf0e,0xd863cf10,0xdd1bcf10,0xd863cf12,0xdd1bcf12,0xd863cf14,0xdd1dcf14,0xd863cf16,
	0xdd1dcf16,0xd863cf18,0xdd1fcf18,0xd863cf1a,0xdd1fcf1a,0xd863cf1c,0xdd21cf1c,0xd863cf1e,
	0xdd21cf1e,0xd863cf20,0xdd23cf20,0xd863cf22,0xdd23cf22,0xd863cf24,0xdd25cf24,0xd863cf26,
	0xdd25cf26,0xd863cf28,0xdd27cf28,0xd863cf2a,0xdd27cf2a,0xd863cf2c,0xdd29cf2c,0xd863cf2e,
	0xdd29cf2e,0xd863cf30,0xdd2bcf30,0xd863cf32,0xdd2bcf32,0xd863cf34,0xdd2dcf34,0xd863cf36,
	0xdd2dcf36,0xd863cf38,0xdd2fcf38,0xd863cf3a,0xdd2fcf3a,0xd863cf3c,0xdd31cf3c,0xd863cf3e,
	0xdd31cf3e,0xd863cf40,0xdd33cf40,0xd863cf42,0xdd33cf42,0xd863cf44,0xdd35cf44,0xd863cf46,
	0xdd35cf46,0xd863cf48,0xdd37cf48,0xd863cf4a,0xdd37cf4a,0xd863cf4c,0xdd39cf4c,0xd863cf4e,
	0xdd39cf4e,0xd863cf50,0xdd3bcf50,0xd863cf52,0xdd3bcf52,0xd863cf54,0xdd3dcf54,0xd863cf56,
	0xdd3dcf56,0xd863cf58,0xdd3fcf58,0xd863cf5a,0xdd3fcf5a,0xd863cf5c,0xdd41cf5c,0xd863cf5e,
	0xdd41cf5e,0xd863cf60,0xdd43cf60,0xd863cf62,0xdd43cf62,0xd863cf64,0xdd45cf64,0xd863cf66,
	0xdd45cf66,0xd863cf68,0xdd47cf68,0xd863cf6a,0xdd47cf6a,0xd863cf6c,0xdd49cf6c,0xd863cf6e,
	0xdd49cf6e,0xd863cf70,0xdd4bcf70,0xd863cf72,0xdd4bcf72,0xd863cf74,0xdd4dcf74,0xd863cf76,
	0xdd4dcf76,0xd863cf78,0xdd4fcf78,0xd863cf7a,0xdd4fcf7a,0xd863cf7c,0xdd51cf7c,0xd863cf7e,
	0xdd51cf7e,0xd863cf80,0xdd53cf80,0xd863cf82,0xdd53cf82,0xd863cf84,0xdd55cf84,0xd863cf86,
	0xdd55cf86,0xd863cf88,0xdd57cf88,0xd863cf8a,0xdd57cf8a,0xd863cf8c,0xdd59cf8c,0xd863cf8e,
	0xdd59cf8e,0xd863cf90,0xdd5bcf90,0xd863cf92,0xdd5bcf92,0xd863cf94,0xdd5dcf94,0xd863cf96,
	0xdd5dcf96,0xd863cf98,0xdd5fcf98,0xd863cf9a,0xdd5fcf9a,0xd863cf9c,0xdd61cf9c,0xd863cf9e,
	0xdd61cf9e,0xd863cfa0,0xdd63cfa0,0xd863cfa2,0xdd63cfa2,0xd863cfa4,0xdd65cfa4,0xd863cfa6,
	0xdd65cfa6,0xd863cfa8,0xdd67cfa8,0xd863cfaa,0xdd67cfaa,0xd863cfac,0xdd69cfac,0xd863cfae,
	0xdd69cfae,0xd863cfb0,0xdd6bcfb0,0xd863cfb2,0xdd6bcfb2,0xd863cfb4,0xdd6dcfb4,0xd863cfb6,
	0xdd6dcfb6,0xd863cfb8,0xdd6fcfb8,0xd863cfba,0xdd6fcfba,0xd863cfbc,0xdd71cfbc,0xd863cfbe,
	0xdd71cfbe,0xd863cfc0,0xdd73cfc0,0xd863cfc2,0xdd73cfc2,0xd863cfc4,0xdd75cfc4,0xd863cfc6,
	0xdd75cfc6,0xd863cfc8,0xdd77cfc8,0xd863cfca,0xdd77cfca,0xd863cfcc,0xdd79cfcc,0xd863cfce,
	0xdd79cfce,0xd863cfd0,0xdd7bcfd0,0xd863cfd2,0xdd7bcfd2,0xd863cfd4,0xdd7dcfd4,0xd863cfd6,
	0xdd7dcfd6,0xd863cfd8,0xdd7fcfd8,0xd863cfda,0xdd7fcfda,0xd863cfdc,0xdd81cfdc,0xd863cfde,
	0xdd81cfde,0xd863cfe0,0xdd83cfe0,0xd863cfe2,0xdd83cfe2,0xd863cfe4,0xdd85cfe4,0xd863cfe6,
	0xdd85cfe6,0xd863cfe8,0xdd87cfe8,0xd863cfea,0xdd87cfea,0xd863cfec,0xdd89cfec,0xd863cfee,
	0xdd89cfee,0xd863cff0,0xdd8bcff0,0xd863cff2,0xdd8bcff2,0xd863cff4,0xdd8dcff4,0xd863cff6,
	0xdd8dcff6,0xd863cff8,0xdd8fcff8,0xd863cffa,0xdd8fcffa,0xd863cffc,0xdd91cffc,0xd863cffe,
	0xdd91cffe,0xd863d000,0xdd93d000,0xd863d002,0xdd93d002,0xd863d004,0xdd95d004,0xd863d006,
	0xdd95d006,0xd863d008,0xdd97d008,0xd863d00a,0xdd97d00a,0xd863d00c,0xdd99d00c,0xd863d00e,
	0xdd99d00e,0xd863d010,0xdd9bd010,0xd863d012,0xdd9bd012,0xd863d014,0xdd9dd014,0xd863d016,
	0xdd9dd016,0xd863d018,0xdd9fd018,0xd863d01a,0xdd9fd01a,0xd863d01c,0xdda1d01c,0xd863d01e,
	0xdda1d01e,0xd863d020,0xdda3d020,0xd863d022,0xdda3d022,0xd863d024,0xdda5d024,0xd863d026,
	0xdda5d026,0xd863d028,0xdda7d028,0xd863d02a,0xdda7d02a,0xd863d02c,0xdda9d02c,0xd863d02e,
	0xdda9d02e,0xd863d030,0xddabd030,0xd863d032,0xddabd032,0xd863d034,0xddadd034,0xd863d036,
	0xddadd036,0xd863d038,0xddafd038,0xd863d03a,0xddafd03a,0xd863d03c,0xddb1d03c,0xd863d03e,
	0xddb1d03e,0xd863d040,0xddb3d040,0xd863d042,0xddb3d042,0xd863d044,0xddb5d044,0xd863d046,
	0xddb5d046,0xd863d048,0xddb7d048,0xd863d04a,0xddb7d04a,0xd863d04c,0xddb9d04c,0xd863d04e,
	0xddb9d04e,0xd863d050,0xddbbd050,0xd863d052,0xddbbd052,0xd863d054,0xddbdd054,0xd863d056,
	0xddbdd056,0xd863d058,0xddbfd058,0xd863d05a,0xddbfd05a,0xd863d05c,0xddc1d05c,0xd863d05e,
	0xddc1d05e,0xd863d060,0xddc3d060,0xd863d062,0xddc3d062,0xd863d064,0xddc5d064,0xd863d066,
	0xddc5d066,0xd863d068,0xddc7d068,0xd863d06a,0xddc7d06a,0xd863d06c,0xddc9d06c,0xd863d06e,
	0xddc9d06e,0xd863d070,0xddcbd070,0xd863d072,0xddcbd072,0xd863d074,0xddcdd074,0xd863d076,
	0xddcdd076,0xd863d078,0xddcfd078,0xd863d07a,0xddcfd07a,0xd863d07c,0xddd1d07c,0xd863d07e,
	0xddd1d07e,0xd863d080,0xddd3d080,0xd863d082,0xddd3d082,0xd863d084,0xddd5d084,0xd863d086,
	0xddd5d086,0xd863d088,0xddd7d088,0xd863d08a,0xddd7d08a,0xd863d08c,0xddd9d08c,0xd863d08e,
	0xddd9d08e,0xd863d090,0xdddbd090,0xd863d092,0xdddbd092,0xd863d094,0xddddd094,0xd863d096,
	0xddddd096,0xd863d098,0xdddfd098,0xd863d09a,0xdddfd09a,0xd863d09c,0xdde1d09c,0xd863d09e,
	0xdde1d09e,0xd863d0a0,0xdde3d0a0,0xd863d0a2,0xdde3d0a2,0xd863d0a4,0xdde5d0a4,0xd863d0a6,
	0xdde5d0a6,0xd863d0a8,0xdde7d0a8,0xd863d0aa,0xdde7d0aa,0xd863d0ac,0xdde9d0ac,0xd863d0ae,
	0xdde9d0ae,0xd863d0b0,0xddebd0b0,0xd863d0b2,0xddebd0b2,0xd863d0b4,0xddedd0b4,0xd863d0b6,
	0xddedd0b6,0xd863d0b8,0xddefd0b8,0xd863d0ba,0xddefd0ba,0xd863d0bc,0xddf1d0bc,0xd863d0be,
	0xddf1d0be,0xd863d0c0,0xddf3d0c0,0xd863d0c2,0xddf3d0c2,0xd863d0c4,0xddf5d0c4,0xd863d0c6,
	0xddf5d0c6,0xd863d0c8,0xddf7d0c8,0xd863d0ca,0xddf7d0ca,0xd863d0cc,0xddf9d0cc,0xd863d0ce,
	0xddf9d0ce,0xd863d0d0,0xddfbd0d0,0xd863d0d2,0xddfbd0d2,0xd863d0d4,0xddfdd0d4,0xd863d0d6,
	0xddfdd0d6,0xd863d0d8,0xddffd0d8,0xd863d0da,0xddffd0da,0xd863d0dc,0xde01d0dc,0xd863d0de,
	0xde01d0de,0xd863d0e0,0xde03d0e0,0xd863d0e2,0xde03d0e2,0xd863d0e4,0xde05d0e4,0xd863d0e6,
	0xde05d0e6,0xd863d0e8,0xde07d0e8,0xd863d0ea,0xde07d0ea,0xd863d0ec,0xde09d0ec,0xd863d0ee,
	0xde09d0ee,0xd863d0f0,0xde0bd0f0,0xd863d0f2,0xde0bd0f2,0xd863d0f4,0xde0dd0f4,0xd863d0f6,
	0xde0dd0f6,0xd863d0f8,0xde0fd0f8,0xd863d0fa,0xde0fd0fa,0xd863d0fc,0xde11d0fc,0xd863d0fe,
	0xde11d0fe,0xd863d100,0xde13d100,0xd863d102,0xde13d102,0xd863d104,0xde15d104,0xd863d106,
	0xde15d106,0xd863d108,0xde17d108,0xd863d10a,0xde17d10a,0xd863d10c,0xde19d10c,0xd863d10e,
	0xde19d10e,0xd863d110,0xde1bd110,0xd863d112,0xde1bd112,0xd863d114,0xde1dd114,0xd863d116,
	0xde1dd116,0xd863d118,0xde1fd118,0xd863d11a,0xde1fd11a,0xd863d11c,0xde21d11c,0xd863d11e,
	0xde21d11e,0xd863d120,0xde23d120,0xd863d122,0xde23d122,0xd863d124,0xde25d124,0xd863d126,
	0xde25d126,0xd863d128,0xde27d128,0xd863d12a,0xde27d12a,0xd863d12c,0xde29d12c,0xd863d12e,
	0xde29d12e,0xd863d130,0xde2bd130,0xd863d132,0xde2bd132,0xd863d134,0xde2dd134,0xd863d136,
	0xde2dd136,0xd863d138,0xde2fd138,0xd863d13a,0xde2fd13a,0xd863d13c,0xde31d13c,0xd863d13e,
	0xde31d13e,0xd863d140,0xde33d140,0xd863d142,0xde33d142,0xd863d144,0xde35d144,0xd863d146,
	0xde35d146,0xd863d148,0xde37d148,0xd863d14a,0xde37d14a,0xd863d14c,0xde39d14c,0xd863d14e,
	0xde39d14e,0xd863d150,0xde3bd150,0xd863d152,0xde3bd152,0xd863d154,0xde3dd154,0xd863d156,
	0xde3dd156,0xd863d158,0xde3fd158,0xd863d15a,0xde3fd15a,0xd863d15c,0xde41d15c,0xd863d15e,
	0xde41d15e,0xd863d160,0xde43d160,0xd863d162,0xde43d162,0xd863d164,0xde45d164,0xd863d166,
	0xde45d166,0xd863d168,0xde47d168,0xd863d16a,0xde47d16a,0xd863d16c,0xde49d16c,0xd863d16e,
	0xde49d16e,0xd863d170,0xde4bd170,0xd863d172,0xde4bd172,0xd863d174,0xde4dd174,0xd863d176,
	0xde4dd176,0xd863d178,0xde4fd178,0xd863d17a,0xde4fd17a,0xd863d17c,0xde51d17c,0xd863d17e,
	0xde51d17e,0xd863d180,0xde53d180,0xd863d182,0xde53d182,0xd863d184,0xde55d184,0xd863d186,
	0xde55d186,0xd863d188,0xde57d188,0xd863d18a,0xde57d18a,0xd863d18c,0xde59d18c,0xd863d18e,
	0xde59d18e,0xd863d190,0xde5bd190,0xd863d192,0xde5bd192,0xd863d194,0xde5dd194,0xd863d196,
	0xde5dd196,0xd863d198,0xde5fd198,0xd863d19a,0xde5fd19a,0xd863d19c,0xde61d19c,0xd863d19e,
	0xde61d19e,0xd863d1a0,0xde63d1a0,0xd863d1a2,0xde63d1a2,0xd863d1a4,0xde65d1a4,0xd863d1a6,
	0xde65d1a6,0xd863d1a8,0xde67d1a8,0xd863d1aa,0xde67d1aa,0xd863d1ac,0xde69d1ac,0xd863d1ae,
	0xde69d1ae,0xd863d1b0,0xde6bd1b0,0xd863d1b2,0xde6bd1b2,0xd863d1b4,0xde6dd1b4,0xd863d1b6,
	0xde6dd1b6,0xd863d1b8,0xde6fd1b8,0xd863d1ba,0xde6fd1ba,0xd863d1bc,0xde71d1bc,0xd863d1be,
	0xde71d1be,0xd863d1c0,0xde73d1c0,0xd863d1c2,0xde73d1c2,0xd863d1c4,0xde75d1c4,0xd863d1c6,
	0xde75d1c6,0xd863d1c8,0xde77d1c8,0xd863d1ca,0xde77d1ca,0xd863d1cc,0xde79d1cc,0xd863d1ce,
	0xde79d1ce,0xd863d1d0,0xde7bd1d0,0xd863d1d2,0xde7bd1d2,0xd863d1d4,0xde7dd1d4,0xd863d1d6,
	0xde7dd1d6,0xd863d1d8,0xde7fd1d8,0xd863d1da,0xde7fd1da,0xd863d1dc,0xde81d1dc,0xd863d1de,
	0xde81d1de,0xd863d1e0,0xde83d1e0,0xd863d1e2,0xde83d1e2,0xd863d1e4,0xde85d1e4,0xd863d1e6,
	0xde85d1e6,0xd863d1e8,0xde87d1e8,0xd863d1ea,0xde87d1ea,0xd863d1ec,0xde89d1ec,0xd863d1ee,
	0xde89d1ee,0xd863d1f0,0xde8bd1f0,0xd863d1f2,0xde8bd1f2,0xd863d1f4,0xde8dd1f4,0xd863d1f6,
	0xde8dd1f6,0xd863d1f8,0xde8fd1f8,0xd863d1fa,0xde8fd1fa,0xd863d1fc,0xde91d1fc,0xd863d1fe,
	0xde91d1fe,0xd863d200,0xde93d200,0xd863d202,0xde93d202,0xd863d204,0xde95d204,0xd863d206,
	0xde95d206,0xd863d208,0xde97d208,0xd863d20a,0xde97d20a,0xd863d20c,0xde99d20c,0xd863d20e,
	0xde99d20e,0xd863d210,0xde9bd210,0xd863d212,0xde9bd212,0xd863d214,0xde9dd214,0xd863d216,
	0xde9dd216,0xd863d218,0xde9fd218,0xd863d21a,0xde9fd21a,0xd863d21c,0xdea1d21c,0xd863d21e,
	0xdea1d21e,0xd863d220,0xdea3d220,0xd863d222,0xdea3d222,0xd863d224,0xdea5d224,0xd863d226,
	0xdea5d226,0xd863d228,0xdea7d228,0xd863d22a,0xdea7d22a,0xd863d22c,0xdea9d22c,0xd863d22e,
	0xdea9d22e,0xd863d230,0xdeabd230,0xd863d232,0xdeabd232,0xd863d234,0xdeadd234,0xd863d236,
	0xdeadd236,0xd863d238,0xdeafd238,0xd863d23a,0xdeafd23a,0xd863d23c,0xdeb1d23c,0xd863d23e,
	0xdeb1d23e,0xd863d240,0xdeb3d240,0xd863d242,0xdeb3d242,0xd863d244,0xdeb5d244,0xd863d246,
	0xdeb5d246,0xd863d248,0xdeb7d248,0xd863d24a,0xdeb7d24a,0xd863d24c,0xdeb9d24c,0xd863d24e,
	0xdeb9d24e,0xd863d250,0xdebbd250,0xd863d252,0xdebbd252,0xd863d254,0xdebdd254,0xd863d256,
	0xdebdd256,0xd863d258,0xdebfd258,0xd863d25a,0xdebfd25a,0xd863d25c,0xdec1d25c,0xd863d25e,
	0xdec1d25e,0xd863d260,0xdec3d260,0xd863d262,0xdec3d262,0xd863d264,0xdec5d264,0xd863d266,
	0xdec5d266,0xd863d268,0xdec7d268,0xd863d26a,0xdec7d26a,0xd863d26c,0xdec9d26c,0xd863d26e,
	0xdec9d26e,0xd863d270,0xdecbd270,0xd863d272,0xdecbd272,0xd863d274,0xdecdd274,0xd863d276,
	0xdecdd276,0xd863d278,0xdecfd278,0xd863d27a,0xdecfd27a,0xd863d27c,0xded1d27c,0xd863d27e,
	0xded1d27e,0xd863d280,0xded3d280,0xd863d282,0xded3d282,0xd863d284,0xded5d284,0xd863d286,
	0xded5d286,0xd863d288,0xded7d288,0xd863d28a,0xded7d28a,0xd863d28c,0xded9d28c,0xd863d28e,
	0xded9d28e,0xd863d290,0xdedbd290,0xd863d292,0xdedbd292,0xd863d294,0xdeddd294,0xd863d296,
	0xdeddd296,0xd863d298,0xdedfd298,0xd863d29a,0xdedfd29a,0xd863d29c,0xdee1d29c,0xd863d29e,
	0xdee1d29e,0xd863d2a0,0xdee3d2a0,0xd863d2a2,0xdee3d2a2,0xd863d2a4,0xdee5d2a4,0xd863d2a6,
	0xdee5d2a6,0xd863d2a8,0xdee7d2a8,0xd863d2aa,0xdee7d2aa,0xd863d2ac,0xdee9d2ac,0xd863d2ae,
	0xdee9d2ae,0xd863d2b0,0xdeebd2b0,0xd863d2b2,0xdeebd2b2,0xd863d2b4,0xdeedd2b4,0xd863d2b6,
	0xdeedd2b6,0xd863d2b8,0xdeefd2b8,0xd863d2ba,0xdeefd2ba,0xd863d2bc,0xdef1d2bc,0xd863d2be,
	0xdef1d2be,0xd863d2c0,0xdef3d2c0,0xd863d2c2,0xdef3d2c2,0xd863d2c4,0xdef5d2c4,0xd863d2c6,
	0xdef5d2c6,0xd863d2c8,0xdef7d2c8,0xd863d2ca,0xdef7d2ca,0xd863d2cc,0xdef9d2cc,0xd863d2ce,
	0xdef9d2ce,0xd863d2d0,0xdefbd2d0,0xd863d2d2,0xdefbd2d2,0xd863d2d4,0xdefdd2d4,0xd863d2d6,
	0xdefdd2d6,0xd863d2d8,0xdeffd2d8,0xd863d2da,0xdeffd2da,0xd863d2dc,0xdf01d2dc,0xd863d2de,
	0xdf01d2de,0xd863d2e0,0xdf03d2e0,0xd863d2e2,0xdf03d2e2,0xd863d2e4,0xdf05d2e4,0xd863d2e6,
	0xdf05d2e6,0xd863d2e8,0xdf07d2e8,0xd863d2ea,0xdf07d2ea,0xd863d2ec,0xdf09d2ec,0xd863d2ee,
	0xdf09d2ee,0xd863d2f0,0xdf0bd2f0,0xd863d2f2,0xdf0bd2f2,0xd863d2f4,0xdf0dd2f4,0xd863d2f6,
	0xdf0dd2f6,0xd863d2f8,0xdf0fd2f8,0xd863d2fa,0xdf0fd2fa,0xd863d2fc,0xdf11d2fc,0xd863d2fe,
	0xdf11d2fe,0xd863d300,0xdf13d300,0xd863d302,0xdf13d302,0xd863d304,0xdf15d304,0xd863d306,
	0xdf15d306,0xd863d308,0xdf17d308,0xd863d30a,0xdf17d30a,0xd863d30c,0xdf19d30c,0xd863d30e,
	0xdf19d30e,0xd863d310,0xdf1bd310,0xd863d312,0xdf1bd312,0xd863d314,0xdf1dd314,0xd863d316,
	0xdf1dd316,0xd863d318,0xdf1fd318,0xd863d31a,0xdf1fd31a,0xd863d31c,0xdf21d31c,0xd863d31e,
	0xdf21d31e,0xd863d320,0xdf23d320,0xd863d322,0xdf23d322,0xd863d324,0xdf25d324,0xd863d326,
	0xdf25d326,0xd863d328,0xdf27d328,0xd863d32a,0xdf27d32a,0xd863d32c,0xdf29d32c,0xd863d32e,
	0xdf29d32e,0xd863d330,0xdf2bd330,0xd863d332,0xdf2bd332,0xd863d334,0xdf2dd334,0xd863d336,
	0xdf2dd336,0xd863d338,0xdf2fd338,0xd863d33a,0xdf2fd33a,0xd863d33c,0xdf31d33c,0xd863d33e,
	0xdf31d33e,0xd863d340,0xdf33d340,0xd863d342,0xdf33d342,0xd863d344,0xdf35d344,0xd863d346,
	0xdf35d346,0xd863d348,0xdf37d348,0xd863d34a,0xdf37d34a,0xd863d34c,0xdf39d34c,0xd863d34e,
	0xdf39d34e,0xd863d350,0xdf3bd350,0xd863d352,0xdf3bd352,0xd863d354,0xdf3dd354,0xd863d356,
	0xdf3dd356,0xd863d358,0xdf3fd358,0xd863d35a,0xdf3fd35a,0xd863d35c,0xdf41d35c,0xd863d35e,
	0xdf41d35e,0xd863d360,0xdf43d360,0xd863d362,0xdf43d362,0xd863d364,0xdf45d364,0xd863d366,
	0xdf45d366,0xd863d368,0xdf47d368,0xd863d36a,0xdf47d36a,0xd863d36c,0xdf49d36c,0xd863d36e,
	0xdf49d36e,0xd863d370,0xdf4bd370,0xd863d372,0xdf4bd372,0xd863d374,0xdf4dd374,0xd863d376,
	0xdf4dd376,0xd863d378,0xdf4fd378,0xd863d37a,0xdf4fd37a,0xd863d37c,0xdf51d37c,0xd863d37e,
	0xdf51d37e,0xd863d380,0xdf53d380,0xd863d382,0xdf53d382,0xd863d384,0xdf55d384,0xd863d386,
	0xdf55d386,0xd863d388,0xdf57d388,0xd863d38a,0xdf57d38a,0xd863d38c,0xdf59d38c,0xd863d38e,
	0xdf59d38e,0xd863d390,0xdf5bd390,0xd863d392,0xdf5bd392,0xd863d394,0xdf5dd394,0xd863d396,
	0xdf5dd396,0xd863d398,0xdf5fd398,0xd863d39a,0xdf5fd39a,0xd863d39c,0xdf61d39c,0xd863d39e,
	0xdf61d39e,0xd863d3a0,0xdf63d3a0,0xd863d3a2,0xdf63d3a2,0xd863d3a4,0xdf65d3a4,0xd863d3a6,
	0xdf65d3a6,0xd863d3a8,0xdf67d3a8,0xd863d3aa,0xdf67d3aa,0xd863d3ac,0xdf69d3ac,0xd863d3ae,
	0xdf69d3ae,0xd863d3b0,0xdf6bd3b0,0xd863d3b2,0xdf6bd3b2,0xd863d3b4,0xdf6dd3b4,0xd863d3b6,
	0xdf6dd3b6,0xd863d3b8,0xdf6fd3b8,0xd863d3ba,0xdf6fd3ba,0xd863d3bc,0xdf71d3bc,0xd863d3be,
	0xdf71d3be,0xd863d3c0,0xdf73d3c0,0xd863d3c2,0xdf73d3c2,0xd863d3c4,0xdf75d3c4,0xd863d3c6,
	0xdf75d3c6,0xd863d3c8,0xdf77d3c8,0xd863d3ca,0xdf77d3ca,0xd863d3cc,0xdf79d3cc,0xd863d3ce,
	0xdf79d3ce,0xd863d3d0,0xdf7bd3d0,0xd863d3d2,0xdf7bd3d2,0xd863d3d4,0xdf7dd3d4,0xd863d3d6,
	0xdf7dd3d6,0xd863d3d8,0xdf7fd3d8,0xd863d3da,0xdf7fd3da,0xd863d3dc,0xdf81d3dc,0xd863d3de,
	0xdf81d3de,0xd863d3e0,0xdf83d3e0,0xd863d3e2,0xdf83d3e2,0xd863d3e4,0xdf85d3e4,0xd863d3e6,
	0xdf85d3e6,0xd863d3e8,0xdf87d3e8,0xd863d3ea,0xdf87d3ea,0xd863d3ec,0xdf89d3ec,0xd863d3ee,
	0xdf89d3ee,0xd863d3f0,0xdf8bd3f0,0xd863d3f2,0xdf8bd3f2,0xd863d3f4,0xdf8dd3f4,0xd863d3f6,
	0xdf8dd3f6,0xd863d3f8,0xdf8fd3f8,0xd863d3fa,0xdf8fd3fa,0xd863d3fc,0xdf91d3fc,0xd863d3fe,
	0xdf91d3fe,0xd863d400,0xdf93d400,0xd863d402,0xdf93d402,0xd863d404,0xdf95d404,0xd863d406,
	0xdf95d406,0xd863d408,0xdf97d408,0xd863d40a,0xdf97d40a,0xd863d40c,0xdf99d40c,0xd863d40e,
	0xdf99d40e,0xd863d410,0xdf9bd410,0xd863d412,0xdf9bd412,0xd863d414,0xdf9dd414,0xd863d416,
	0xdf9dd416,0xd863d418,0xdf9fd418,0xd863d41a,0xdf9fd41a,0xd863d41c,0xdfa1d41c,0xd863d41e,
	0xdfa1d41e,0xd863d420,0xdfa3d420,0xd863d422,0xdfa3d422,0xd863d424,0xdfa5d424,0xd863d426,
	0xdfa5d426,0xd863d428,0xdfa7d428,0xd863d42a,0xdfa7d42a,0xd863d42c,0xdfa9d42c,0xd863d42e,
	0xdfa9d42e,0xd863d430,0xdfabd430,0xd863d432,0xdfabd432,0xd863d434,0xdfadd434,0xd863d436,
	0xdfadd436,0xd863d438,0xdfafd438,0xd863d43a,0xdfafd43a,0xd863d43c,0xdfb1d43c,0xd863d43e,
	0xdfb1d43e,0xd863d440,0xdfb3d440,0xd863d442,0xdfb3d442,0xd863d444,0xdfb5d444,0xd863d446,
	0xdfb5d446,0xd863d448,0xdfb7d448,0xd863d44a,0xdfb7d44a,0xd863d44c,0xdfb9d44c,0xd863d44e,
	0xdfb9d44e,0xd863d450,0xdfbbd450,0xd863d452,0xdfbbd452,0xd863d454,0xdfbdd454,0xd863d456,
	0xdfbdd456,0xd863d458,0xdfbfd458,0xd863d45a,0xdfbfd45a,0xd863d45c,0xdfc1d45c,0xd863d45e,
	0xdfc1d45e,0xd863d460,0xdfc3d460,0xd863d462,0xdfc3d462,0xd863d464,0xdfc5d464,0xd863d466,
	0xdfc5d466,0xd863d468,0xdfc7d468,0xd863d46a,0xdfc7d46a,0xd863d46c,0xdfc9d46c,0xd863d46e,
	0xdfc9d46e,0xd863d470,0xdfcbd470,0xd863d472,0xdfcbd472,0xd863d474,0xdfcdd474,0xd863d476,
	0xdfcdd476,0xd863d478,0xdfcfd478,0xd863d47a,0xdfcfd47a,0xd863d47c,0xdfd1d47c,0xd863d47e,
	0xdfd1d47e,0xd863d480,0xdfd3d480,0xd863d482,0xdfd3d482,0xd863d484,0xdfd5d484,0xd863d486,
	0xdfd5d486,0xd863d488,0xdfd7d488,0xd863d48a,0xdfd7d48a,0xd863d48c,0xdfd9d48c,0xd863d48e,
	0xdfd9d48e,0xd863d490,0xdfdbd490,0xd863d492,0xdfdbd492,0xd863d494,0xdfddd494,0xd863d496,
	0xdfddd496,0xd863d498,0xdfdfd498,0xd863d49a,0xdfdfd49a,0xd863d49c,0xdfe1d49c,0xd863d49e,
	0xdfe1d49e,0xd863d4a0,0xdfe3d4a0,0xd863d4a2,0xdfe3d4a2,0xd863d4a4,0xdfe5d4a4,0xd863d4a6,
	0xdfe5d4a6,0xd863d4a8,0xdfe7d4a8,0xd863d4aa,0xdfe7d4aa,0xd863d4ac,0xdfe9d4ac,0xd863d4ae,
	0xdfe9d4ae,0xd863d4b0,0xdfebd4b0,0xd863d4b2,0xdfebd4b2,0xd863d4b4,0xdfedd4b4,0xd863d4b6,
	0xdfedd4b6,0xd863d4b8,0xdfefd4b8,0xd863d4ba,0xdfefd4ba,0xd863d4bc,0xdff1d4bc,0xd863d4be,
	0xdff1d4be,0xd863d4c0,0xdff3d4c0,0xd863d4c2,0xdff3d4c2,0xd863d4c4,0xdff5d4c4,0xd863d4c6,
	0xdff5d4c6,0xd863d4c8,0xdff7d4c8,0xd863d4ca,0xdff7d4ca,0xd863d4cc,0xdff9d4cc,0xd863d4ce,
	0xdff9d4ce,0xd863d4d0,0xdffbd4d0,0xd863d4d2,0xdffbd4d2,0xd863d4d4,0xdffdd4d4,0xd863d4d6,
	0xdffdd4d6,0xd863d4d8,0xdfffd4d8,0xd863d4da,0xdfffd4da,0xd863d4dc,0xdc01d4dc,0xd863d4de,
	0xdc01d4de,0xd863d4e0,0xdc03d4e0,0xd863d4e2,0xdc03d4e2,0xd863d4e4,0xdc05d4e4,0xd863d4e6,
	0xdc05d4e6,0xd863d4e8,0xdc07d4e8,0xd863d4ea,0xdc07d4ea,0xd863d4ec,0xdc09d4ec,0xd863d4ee,
	0xdc09d4ee,0xd863d4f0,0xdc0bd4f0,0xd863d4f2,0xdc0bd4f2,0xd863d4f4,0xdc0dd4f4,0xd863d4f6,
	0xdc0dd4f6,0xd863d4f8,0xdc0fd4f8,0xd863d4fa,0xdc0fd4fa,0xd863d4fc,0xdc11d4fc,0xd863d4fe,
	0xdc11d4fe,0xd863d500,0xdc13d500,0xd863d502,0xdc13d502,0xd863d504,0xdc15d504,0xd863d506,
	0xdc15d506,0xd863d508,0xdc17d508,0xd863d50a,0xdc17d50a,0xd863d50c,0xdc19d50c,0xd863d50e,
	0xdc19d50e,0xd863d510,0xdc1bd510,0xd863d512,0xdc1bd512,0xd863d514,0xdc1dd514,0xd863d516,
	0xdc1dd516,0xd863d518,0xdc1fd518,0xd863d51a,0xdc1fd51a,0xd863d51c,0xdc21d51c,0xd863d51e,
	0xdc21d51e,0xd863d520,0xdc23d520,0xd863d522,0xdc23d522,0xd863d524,0xdc25d524,0xd863d526,
	0xdc25d526,0xd863d528,0xdc27d528,0xd863d52a,0xdc27d52a,0xd863d52c,0xdc29d52c,0xd863d52e,
	0xdc29d52e,0xd863d530,0xdc2bd530,0xd863d532,0xdc2bd532,0xd863d534,0xdc2dd534,0xd863d536,
	0xdc2dd536,0xd863d538,0xdc2fd538,0xd863d53a,0xdc2fd53a,0xd863d53c,0xdc31d53c,0xd863d53e,
	0xdc31d53e,0xd863d540,0xdc33d540,0xd863d542,0xdc33d542,0xd863d544,0xdc35d544,0xd863d546,
	0xdc35d546,0xd863d548,0xdc37d548,0xd863d54a,0xdc37d54a,0xd863d54c,0xdc39d54c,0xd863d54e,
	0xdc39d54e,0xd863d550,0xdc3bd550,0xd863d552,0xdc3bd552,0xd863d554,0xdc3dd554,0xd863d556,
	0xdc3dd556,0xd863d558,0xdc3fd558,0xd863d55a,0xdc3fd55a,0xd863d55c,0xdc41d55c,0xd863d55e,
	0xdc41d55e,0xd863d560,0xdc43d560,0xd863d562,0xdc43d562,0xd863d564,0xdc45d564,0xd863d566,
	0xdc45d566,0xd863d568,0xdc47d568,0xd863d56a,0xdc47d56a,0xd863d56c,0xdc49d56c,0xd863d56e,
	0xdc49d56e,0xd863d570,0xdc4bd570,0xd863d572,0xdc4bd572,0xd863d574,0xdc4dd574,0xd863d576,
	0xdc4dd576,0xd863d578,0xdc4fd578,0xd863d57a,0xdc4fd57a,0xd863d57c,0xdc51d57c,0xd863d57e,
	0xdc51d57e,0xd863d580,0xdc53d580,0xd863d582,0xdc53d582,0xd863d584,0xdc55d584,0xd863d586,
	0xdc55d586,0xd863d588,0xdc57d588,0xd863d58a,0xdc57d58a,0xd863d58c,0xdc59d58c,0xd863d58e,
	0xdc59d58e,0xd863d590,0xdc5bd590,0xd863d592,0xdc5bd592,0xd863d594,0xdc5dd594,0xd863d596,
	0xdc5dd596,0xd863d598,0xdc5fd598,0xd863d59a,0xdc5fd59a,0xd863d59c,0xdc61d59c,0xd863d59e,
	0xdc61d59e,0xd863d5a0,0xdc63d5a0,0xd863d5a2,0xdc63d5a2,0xd863d5a4,0xdc65d5a4,0xd863d5a6,
	0xdc65d5a6,0xd863d5a8,0xdc67d5a8,0xd863d5aa,0xdc67d5aa,0xd863d5ac,0xdc69d5ac,0xd863d5ae,
	0xdc69d5ae,0xd863d5b0,0xdc6bd5b0,0xd863d5b2,0xdc6bd5b2,0xd863d5b4,0xdc6dd5b4,0xd863d5b6,
	0xdc6dd5b6,0xd863d5b8,0xdc6fd5b8,0xd863d5ba,0xdc6fd5ba,0xd863d5bc,0xdc71d5bc,0xd863d5be,
	0xdc71d5be,0xd863d5c0,0xdc73d5c0,0xd863d5c2,0xdc73d5c2,0xd863d5c4,0xdc75d5c4,0xd863d5c6,
	0xdc75d5c6,0xd863d5c8,0xdc77d5c8,0xd863d5ca,0xdc77d5ca,0xd863d5cc,0xdc79d5cc,0xd863d5ce,
	0xdc79d5ce,0xd863d5d0,0xdc7bd5d0,0xd863d5d2,0xdc7bd5d2,0xd863d5d4,0xdc7dd5d4,0xd863d5d6,
	0xdc7dd5d6,0xd863d5d8,0xdc7fd5d8,0xd863d5da,0xdc7fd5da,0xd863d5dc,0xdc81d5dc,0xd863d5de,
	0xdc81d5de,0xd863d5e0,0xdc83d5e0,0xd863d5e2,0xdc83d5e2,0xd863d5e4,0xdc85d5e4,0xd863d5e6,
	0xdc85d5e6,0xd863d5e8,0xdc87d5e8,0xd863d5ea,0xdc87d5ea,0xd863d5ec,0xdc89d5ec,0xd863d5ee,
	0xdc89d5ee,0xd863d5f0,0xdc8bd5f0,0xd863d5f2,0xdc8bd5f2,0xd863d5f4,0xdc8dd5f4,0xd863d5f6,
	0xdc8dd5f6,0xd863d5f8,0xdc8fd5f8,0xd863d5fa,0xdc8fd5fa,0xd863d5fc,0xdc91d5fc,0xd863d5fe,
	0xdc91d5fe,0xd863d600,0xdc93d600,0xd863d602,0xdc93d602,0xd863d604,0xdc95d604,0xd863d606,
	0xdc95d606,0xd863d608,0xdc97d608,0xd863d60a,0xdc97d60a,0xd863d60c,0xdc99d60c,0xd863d60e,
	0xdc99d60e,0xd863d610,0xdc9bd610,0xd863d612,0xdc9bd612,0xd863d614,0xdc9dd614,0xd863d616,
	0xdc9dd616,0xd863d618,0xdc9fd618,0xd863d61a,0xdc9fd61a,0xd863d61c,0xdca1d61c,0xd863d61e,
	0xdca1d61e,0xd863d620,0xdca3d620,0xd863d622,0xdca3d622,0xd863d624,0xdca5d624,0xd863d626,
	0xdca5d626,0xd863d628,0xdca7d628,0xd863d62a,0xdca7d62a,0xd863d62c,0xdca9d62c,0xd863d62e,
	0xdca9d62e,0xd863d630,0xdcabd630,0xd863d632,0xdcabd632,0xd863d634,0xdcadd634,0xd863d636,
	0xdcadd636,0xd863d638,0xdcafd638,0xd863d63a,0xdcafd63a,0xd863d63c,0xdcb1d63c,0xd863d63e,
	0xdcb1d63e,0xd863d640,0xdcb3d640,0xd863d642,0xdcb3d642,0xd863d644,0xdcb5d644,0xd863d646,
	0xdcb5d646,0xd863d648,0xdcb7d648,0xd863d64a,0xdcb7d64a,0xd863d64c,0xdcb9d64c,0xd863d64e,
	0xdcb9d64e,0xd863d650,0xdcbbd650,0xd863d652,0xdcbbd652,0xd863d654,0xdcbdd654,0xd863d656,
	0xdcbdd656,0xd863d658,0xdcbfd658,0xd863d65a,0xdcbfd65a,0xd863d65c,0xdcc1d65c,0xd863d65e,
	0xdcc1d65e,0xd863d660,0xdcc3d660,0xd863d662,0xdcc3d662,0xd863d664,0xdcc5d664,0xd863d666,
	0xdcc5d666,0xd863d668,0xdcc7d668,0xd863d66a,0xdcc7d66a,0xd863d66c,0xdcc9d66c,0xd863d66e,
	0xdcc9d66e,0xd863d670,0xdccbd670,0xd863d672,0xdccbd672,0xd863d674,0xdccdd674,0xd863d676,
	0xdccdd676,0xd863d678,0xdccfd678,0xd863d67a,0xdccfd67a,0xd863d67c,0xdcd1d67c,0xd863d67e,
	0xdcd1d67e,0xd863d680,0xdcd3d680,0xd863d682,0xdcd3d682,0xd863d684,0xdcd5d684,0xd863d686,
	0xdcd5d686,0xd863d688,0xdcd7d688,0xd863d68a,0xdcd7d68a,0xd863d68c,0xdcd9d68c,0xd863d68e,
	0xdcd9d68e,0xd863d690,0xdcdbd690,0xd863d692,0xdcdbd692,0xd863d694,0xdcddd694,0xd863d696,
	0xdcddd696,0xd863d698,0xdcdfd698,0xd863d69a,0xdcdfd69a,0xd863d69c,0xdce1d69c,0xd863d69e,
	0xdce1d69e,0xd863d6a0,0xdce3d6a0,0xd863d6a2,0xdce3d6a2,0xd863d6a4,0xdce5d6a4,0xd863d6a6,
	0xdce5d6a6,0xd863d6a8,0xdce7d6a8,0xd863d6aa,0xdce7d6aa,0xd863d6ac,0xdce9d6ac,0xd863d6ae,
	0xdce9d6ae,0xd863d6b0,0xdcebd6b0,0xd863d6b2,0xdcebd6b2,0xd863d6b4,0xdcedd6b4,0xd863d6b6,
	0xdcedd6b6,0xd863d6b8,0xdcefd6b8,0xd863d6ba,0xdcefd6ba,0xd863d6bc,0xdcf1d6bc,0xd863d6be,
	0xdcf1d6be,0xd863d6c0,0xdcf3d6c0,0xd863d6c2,0xdcf3d6c2,0xd863d6c4,0xdcf5d6c4,0xd863d6c6,
	0xdcf5d6c6,0xd863d6c8,0xdcf7d6c8,0xd863d6ca,0xdcf7d6ca,0xd863d6cc,0xdcf9d6cc,0xd863d6ce,
	0xdcf9d6ce,0xd863d6d0,0xdcfbd6d0,0xd863d6d2,0xdcfbd6d2,0xd863d6d4,0xdcfdd6d4,0xd863d6d6,
	0xdcfdd6d6,0xd863d6d8,0xdcffd6d8,0xd863d6da,0xdcffd6da,0xd863d6dc,0xdd01d6dc,0xd863d6de,
	0xdd01d6de,0xd863d6e0,0xdd03d6e0,0xd863d6e2,0xdd03d6e2,0xd863d6e4,0xdd05d6e4,0xd863d6e6,
	0xdd05d6e6,0xd863d6e8,0xdd07d6e8,0xd863d6ea,0xdd07d6ea,0xd863d6ec,0xdd09d6ec,0xd863d6ee,
	0xdd09d6ee,0xd863d6f0,0xdd0bd6f0,0xd863d6f2,0xdd0bd6f2,0xd863d6f4,0xdd0dd6f4,0xd863d6f6,
	0xdd0dd6f6,0xd863d6f8,0xdd0fd6f8,0xd863d6fa,0xdd0fd6fa,0xd863d6fc,0xdd11d6fc,0xd863d6fe,
	0xdd11d6fe,0xd863d700,0xdd13d700,0xd863d702,0xdd13d702,0xd863d704,0xdd15d704,0xd863d706,
	0xdd15d706,0xd863d708,0xdd17d708,0xd863d70a,0xdd17d70a,0xd863d70c,0xdd19d70c,0xd863d70e,
	0xdd19d70e,0xd863d710,0xdd1bd710,0xd863d712,0xdd1bd712,0xd863d714,0xdd1dd714,0xd863d716,
	0xdd1dd716,0xd863d718,0xdd1fd718,0xd863d71a,0xdd1fd71a,0xd863d71c,0xdd21d71c,0xd863d71e,
	0xdd21d71e,0xd863d720,0xdd23d720,0xd863d722,0xdd23d722,0xd863d724,0xdd25d724,0xd863d726,
	0xdd25d726,0xd863d728,0xdd27d728,0xd863d72a,0xdd27d72a,0xd863d72c,0xdd29d72c,0xd863d72e,
	0xdd29d72e,0xd863d730,0xdd2bd730,0xd863d732,0xdd2bd732,0xd863d734,0xdd2dd734,0xd863d736,
	0xdd2dd736,0xd863d738,0xdd2fd738,0xd863d73a,0xdd2fd73a,0xd863d73c,0xdd31d73c,0xd863d73e,
	0xdd31d73e,0xd863d740,0xdd33d740,0xd863d742,0xdd33d742,0xd863d744,0xdd35d744,0xd863d746,
	0xdd35d746,0xd863d748,0xdd37d748,0xd863d74a,0xdd37d74a,0xd863d74c,0xdd39d74c,0xd863d74e,
	0xdd39d74e,0xd863d750,0xdd3bd750,0xd863d752,0xdd3bd752,0xd863d754,0xdd3dd754,0xd863d756,
	0xdd3dd756,0xd863d758,0xdd3fd758,0xd863d75a,0xdd3fd75a,0xd863d75c,0xdd41d75c,0xd863d75e,
	0xdd41d75e,0xd863d760,0xdd43d760,0xd863d762,0xdd43d762,0xd863d764,0xdd45d764,0xd863d766,
	0xdd45d766,0xd863d768,0xdd47d768,0xd863d76a,0xdd47d76a,0xd863d76c,0xdd49d76c,0xd863d76e,
	0xdd49d76e,0xd863d770,0xdd4bd770,0xd863d772,0xdd4bd772,0xd863d774,0xdd4dd774,0xd863d776,
	0xdd4dd776,0xd863d778,0xdd4fd778,0xd863d77a,0xdd4fd77a,0xd863d77c,0xdd51d77c,0xd863d77e,
	0xdd51d77e,0xd863d780,0xdd53d780,0xd863d782,0xdd53d782,0xd863d784,0xdd55d784,0xd863d786,
	0xdd55d786,0xd863d788,0xdd57d788,0xd863d78a,0xdd57d78a,0xd863d78c,0xdd59d78c,0xd863d78e,
	0xdd59d78e,0xd863d790,0xdd5bd790,0xd863d792,0xdd5bd792,0xd863d794,0xdd5dd794,0xd863d796,
	0xdd5dd796,0xd863d798,0xdd5fd798,0xd863d79a,0xdd5fd79a,0xd863d79c,0xdd61d79c,0xd863d79e,
	0xdd61d79e,0xd863d7a0,0xdd63d7a0,0xd863d7a2,0xdd63d7a2,0xd863d7a4,0xdd65d7a4,0xd863d7a6,
	0xdd65d7a6,0xd863d7a8,0xdd67d7a8,0xd863d7aa,0xdd67d7aa,0xd863d7ac,0xdd69d7ac,0xd863d7ae,
	0xdd69d7ae,0xd863d7b0,0xdd6bd7b0,0xd863d7b2,0xdd6bd7b2,0xd863d7b4,0xdd6dd7b4,0xd863d7b6,
	0xdd6dd7b6,0xd863d7b8,0xdd6fd7b8,0xd863d7ba,0xdd6fd7ba,0xd863d7bc,0xdd71d7bc,0xd863d7be,
	0xdd71d7be,0xd863d7c0,0xdd73d7c0,0xd863d7c2,0xdd73d7c2,0xd863d7c4,0xdd75d7c4,0xd863d7c6,
	0xdd75d7c6,0xd863d7c8,0xdd77d7c8,0xd863d7ca,0xdd77d7ca,0xd863d7cc,0xdd79d7cc,0xd863d7ce,
	0xdd79d7ce,0xd863d7d0,0xdd7bd7d0,0xd863d7d2,0xdd7bd7d2,0xd863d7d4,0xdd7dd7d4,0xd863d7d6,
	0xdd7dd7d6,0xd863d7d8,0xdd7fd7d8,0xd863d7da,0xdd7fd7da,0xd863d7dc,0xdd81d7dc,0xd863d7de,
	0xdd81d7de,0xd863d7e0,0xdd83d7e0,0xd863d7e2,0xdd83d7e2,0xd863d7e4,0xdd85d7e4,0xd863d7e6,
	0xdd85d7e6,0xd863d7e8,0xdd87d7e8,0xd863d7ea,0xdd87d7ea,0xd863d7ec,0xdd89d7ec,0xd863d7ee,
	0xdd89d7ee,0xd863d7f0,0xdd8bd7f0,0xd863d7f2,0xdd8bd7f2,0xd863d7f4,0xdd8dd7f4,0xd863d7f6,
	0xdd8dd7f6,0xd863d7f8,0xdd8fd7f8,0xd863d7fa,0xdd8fd7fa,0xd863d7fc,0xdd91d7fc,0xd863d7fe,
	0xdd91d7fe,0xd863d800,0xdd93d800,0xd863d802,0xdd93d802,0xd863d804,0xdd95d804,0xd863d806,
	0xdd95d806,0xd863d808,0xdd97d808,0xd863d80a,0xdd97d80a,0xd863d80c,0xdd99d80c,0xd863d80e,
	0xdd99d80e,0xd863d810,0xdd9bd810,0xd863d812,0xdd9bd812,0xd863d814,0xdd9dd814,0xd863d816,
	0xdd9dd816,0xd863d818,0xdd9fd818,0xd863d81a,0xdd9fd81a,0xd863d81c,0xdda1d81c,0xd863d81e,
	0xdda1d81e,0xd863d820,0xdda3d820,0xd863d822,0xdda3d822,0xd863d824,0xdda5d824,0xd863d826,
	0xdda5d826,0xd863d828,0xdda7d828,0xd863d82a,0xdda7d82a,0xd863d82c,0xdda9d82c,0xd863d82e,
	0xdda9d82e,0xd863d830,0xddabd830,0xd863d832,0xddabd832,0xd863d834,0xddadd834,0xd863d836,
	0xddadd836,0xd863d838,0xddafd838,0xd863d83a,0xddafd83a,0xd863d83c,0xddb1d83c,0xd863d83e,
	0xddb1d83e,0xd863d840,0xddb3d840,0xd863d842,0xddb3d842,0xd863d844,0xddb5d844,0xd863d846,
	0xddb5d846,0xd863d848,0xddb7d848,0xd863d84a,0xddb7d84a,0xd863d84c,0xddb9d84c,0xd863d84e,
	0xddb9d84e,0xd863d850,0xddbbd850,0xd863d852,0xddbbd852,0xd863d854,0xddbdd854,0xd863d856,
	0xddbdd856,0xd863d858,0xddbfd858,0xd863d85a,0xddbfd85a,0xd863d85c,0xddc1d85c,0xd863d85e,
	0xddc1d85e,0xd863d860,0xddc3d860,0xd863d862,0xddc3d862,0xd863d864,0xddc5d864,0xd863d866,
	0xddc5d866,0xd863d868,0xddc7d868,0xd863d86a,0xddc7d86a,0xd863d86c,0xddc9d86c,0xd863d86e,
	0xddc9d86e,0xd863d870,0xddcbd870,0xd863d872,0xddcbd872,0xd863d874,0xddcdd874,0xd863d876,
	0xddcdd876,0xd863d878,0xddcfd878,0xd863d87a,0xddcfd87a,0xd863d87c,0xddd1d87c,0xd863d87e,
	0xddd1d87e,0xd863d880,0xddd3d880,0xd863d882,0xddd3d882,0xd863d884,0xddd5d884,0xd863d886,
	0xddd5d886,0xd863d888,0xddd7d888,0xd863d88a,0xddd7d88a,0xd863d88c,0xddd9d88c,0xd863d88e,
	0xddd9d88e,0xd863d890,0xdddbd890,0xd863d892,0xdddbd892,0xd863d894,0xddddd894,0xd863d896,
	0xddddd896,0xd863d898,0xdddfd898,0xd863d89a,0xdddfd89a,0xd863d89c,0xdde1d89c,0xd863d89e,
	0xdde1d89e,0xd863d8a0,0xdde3d8a0,0xd863d8a2,0xdde3d8a2,0xd863d8a4,0xdde5d8a4,0xd863d8a6,
	0xdde5d8a6,0xd863d8a8,0xdde7d8a8,0xd863d8aa,0xdde7d8aa,0xd863d8ac,0xdde9d8ac,0xd863d8ae,
	0xdde9d8ae,0xd863d8b0,0xddebd8b0,0xd863d8b2,0xddebd8b2,0xd863d8b4,0xddedd8b4,0xd863d8b6,
	0xddedd8b6,0xd863d8b8,0xddefd8b8,0xd863d8ba,0xddefd8ba,0xd863d8bc,0xddf1d8bc,0xd863d8be,
	0xddf1d8be,0xd863d8c0,0xddf3d8c0,0xd863d8c2,0xddf3d8c2,0xd863d8c4,0xddf5d8c4,0xd863d8c6,
	0xddf5d8c6,0xd863d8c8,0xddf7d8c8,0xd863d8ca,0xddf7d8ca,0xd863d8cc,0xddf9d8cc,0xd863d8ce,
	0xddf9d8ce,0xd863d8d0,0xddfbd8d0,0xd863d8d2,0xddfbd8d2,0xd863d8d4,0xddfdd8d4,0xd863d8d6,
	0xddfdd8d6,0xd863d8d8,0xddffd8d8,0xd863d8da,0xddffd8da,0xd863d8dc,0xde01d8dc,0xd863d8de,
	0xde01d8de,0xd863d8e0,0xde03d8e0,0xd863d8e2,0xde03d8e2,0xd863d8e4,0xde05d8e4,0xd863d8e6,
	0xde05d8e6,0xd863d8e8,0xde07d8e8,0xd863d8ea,0xde07d8ea,0xd863d8ec,0xde09d8ec,0xd863d8ee,
	0xde09d8ee,0xd863d8f0,0xde0bd8f0,0xd863d8f2,0xde0bd8f2,0xd863d8f4,0xde0dd8f4,0xd863d8f6,
	0xde0dd8f6,0xd863d8f8,0xde0fd8f8,0xd863d8fa,0xde0fd8fa,0xd863d8fc,0xde11d8fc,0xd863d8fe,
	0xde11d8fe,0xd863d900,0xde13d900,0xd863d902,0xde13d902,0xd863d904,0xde15d904,0xd863d906,
	0xde15d906,0xd863d908,0xde17d908,0xd863d90a,0xde17d90a,0xd863d90c,0xde19d90c,0xd863d90e,
	0xde19d90e,0xd863d910,0xde1bd910,0xd863d912,0xde1bd912,0xd863d914,0xde1dd914,0xd863d916,
	0xde1dd916,0xd863d918,0xde1fd918,0xd863d91a,0xde1fd91a,0xd863d91c,0xde21d91c,0xd863d91e,
	0xde21d91e,0xd863d920,0xde23d920,0xd863d922,0xde23d922,0xd863d924,0xde25d924,0xd863d926,
	0xde25d926,0xd863d928,0xde27d928,0xd863d92a,0xde27d92a,0xd863d92c,0xde29d92c,0xd863d92e,
	0xde29d92e,0xd863d930,0xde2bd930,0xd863d932,0xde2bd932,0xd863d934,0xde2dd934,0xd863d936,
	0xde2dd936,0xd863d938,0xde2fd938,0xd863d93a,0xde2fd93a,0xd863d93c,0xde31d93c,0xd863d93e,
	0xde31d93e,0xd863d940,0xde33d940,0xd863d942,0xde33d942,0xd863d944,0xde35d944,0xd863d946,
	0xde35d946,0xd863d948,0xde37d948,0xd863d94a,0xde37d94a,0xd863d94c,0xde39d94c,0xd863d94e,
	0xde39d94e,0xd863d950,0xde3bd950,0xd863d952,0xde3bd952,0xd863d954,0xde3dd954,0xd863d956,
	0xde3dd956,0xd863d958,0xde3fd958,0xd863d95a,0xde3fd95a,0xd863d95c,0xde41d95c,0xd863d95e,
	0xde41d95e,0xd863d960,0xde43d960,0xd863d962,0xde43d962,0xd863d964,0xde45d964,0xd863d966,
	0xde45d966,0xd863d968,0xde47d968,0xd863d96a,0xde47d96a,0xd863d96c,0xde49d96c,0xd863d96e,
	0xde49d96e,0xd863d970,0xde4bd970,0xd863d972,0xde4bd972,0xd863d974,0xde4dd974,0xd863d976,
	0xde4dd976,0xd863d978,0xde4fd978,0xd863d97a,0xde4fd97a,0xd863d97c,0xde51d97c,0xd863d97e,
	0xde51d97e,0xd863d980,0xde53d980,0xd863d982,0xde53d982,0xd863d984,0xde55d984,0xd863d986,
	0xde55d986,0xd863d988,0xde57d988,0xd863d98a,0xde57d98a,0xd863d98c,0xde59d98c,0xd863d98e,
	0xde59d98e,0xd863d990,0xde5bd990,0xd863d992,0xde5bd992,0xd863d994,0xde5dd994,0xd863d996,
	0xde5dd996,0xd863d998,0xde5fd998,0xd863d99a,0xde5fd99a,0xd863d99c,0xde61d99c,0xd863d99e,
	0xde61d99e,0xd863d9a0,0xde63d9a0,0xd863d9a2,0xde63d9a2,0xd863d9a4,0xde65d9a4,0xd863d9a6,
	0xde65d9a6,0xd863d9a8,0xde67d9a8,0xd863d9aa,0xde67d9aa,0xd863d9ac,0xde69d9ac,0xd863d9ae,
	0xde69d9ae,0xd863d9b0,0xde6bd9b0,0xd863d9b2,0xde6bd9b2,0xd863d9b4,0xde6dd9b4,0xd863d9b6,
	0xde6dd9b6,0xd863d9b8,0xde6fd9b8,0xd863d9ba,0xde6fd9ba,0xd863d9bc,0xde71d9bc,0xd863d9be,
	0xde71d9be,0xd863d9c0,0xde73d9c0,0xd863d9c2,0xde73d9c2,0xd863d9c4,0xde75d9c4,0xd863d9c6,
	0xde75d9c6,0xd863d9c8,0xde77d9c8,0xd863d9ca,0xde77d9ca,0xd863d9cc,0xde79d9cc,0xd863d9ce,
	0xde79d9ce,0xd863d9d0,0xde7bd9d0,0xd863d9d2,0xde7bd9d2,0xd863d9d4,0xde7dd9d4,0xd863d9d6,
	0xde7dd9d6,0xd863d9d8,0xde7fd9d8,0xd863d9da,0xde7fd9da,0xd863d9dc,0xde81d9dc,0xd863d9de,
	0xde81d9de,0xd863d9e0,0xde83d9e0,0xd863d9e2,0xde83d9e2,0xd863d9e4,0xde85d9e4,0xd863d9e6,
	0xde85d9e6,0xd863d9e8,0xde87d9e8,0xd863d9ea,0xde87d9ea,0xd863d9ec,0xde89d9ec,0xd863d9ee,
	0xde89d9ee,0xd863d9f0,0xde8bd9f0,0xd863d9f2,0xde8bd9f2,0xd863d9f4,0xde8dd9f4,0xd863d9f6,
	0xde8dd9f6,0xd863d9f8,0xde8fd9f8,0xd863d9fa,0xde8fd9fa,0xd863d9fc,0xde91d9fc,0xd863d9fe,
	0xde91d9fe,0xd863da00,0xde93da00,0xd863da02,0xde93da02,0xd863da04,0xde95da04,0xd863da06,
	0xde95da06,0xd863da08,0xde97da08,0xd863da0a,0xde97da0a,0xd863da0c,0xde99da0c,0xd863da0e,
	0xde99da0e,0xd863da10,0xde9bda10,0xd863da12,0xde9bda12,0xd863da14,0xde9dda14,0xd863da16,
	0xde9dda16,0xd863da18,0xde9fda18,0xd863da1a,0xde9fda1a,0xd863da1c,0xdea1da1c,0xd863da1e,
	0xdea1da1e,0xd863da20,0xdea3da20,0xd863da22,0xdea3da22,0xd863da24,0xdea5da24,0xd863da26,
	0xdea5da26,0xd863da28,0xdea7da28,0xd863da2a,0xdea7da2a,0xd863da2c,0xdea9da2c,0xd863da2e,
	0xdea9da2e,0xd863da30,0xdeabda30,0xd863da32,0xdeabda32,0xd863da34,0xdeadda34,0xd863da36,
	0xdeadda36,0xd863da38,0xdeafda38,0xd863da3a,0xdeafda3a,0xd863da3c,0xdeb1da3c,0xd863da3e,
	0xdeb1da3e,0xd863da40,0xdeb3da40,0xd863da42,0xdeb3da42,0xd863da44,0xdeb5da44,0xd863da46,
	0xdeb5da46,0xd863da48,0xdeb7da48,0xd863da4a,0xdeb7da4a,0xd863da4c,0xdeb9da4c,0xd863da4e,
	0xdeb9da4e,0xd863da50,0xdebbda50,0xd863da52,0xdebbda52,0xd863da54,0xdebdda54,0xd863da56,
	0xdebdda56,0xd863da58,0xdebfda58,0xd863da5a,0xdebfda5a,0xd863da5c,0xdec1da5c,0xd863da5e,
	0xdec1da5e,0xd863da60,0xdec3da60,0xd863da62,0xdec3da62,0xd863da64,0xdec5da64,0xd863da66,
	0xdec5da66,0xd863da68,0xdec7da68,0xd863da6a,0xdec7da6a,0xd863da6c,0xdec9da6c,0xd863da6e,
	0xdec9da6e,0xd863da70,0xdecbda70,0xd863da72,0xdecbda72,0xd863da74,0xdecdda74,0xd863da76,
	0xdecdda76,0xd863da78,0xdecfda78,0xd863da7a,0xdecfda7a,0xd863da7c,0xded1da7c,0xd863da7e,
	0xded1da7e,0xd863da80,0xded3da80,0xd863da82,0xded3da82,0xd863da84,0xded5da84,0xd863da86,
	0xded5da86,0xd863da88,0xded7da88,0xd863da8a,0xded7da8a,0xd863da8c,0xded9da8c,0xd863da8e,
	0xded9da8e,0xd863da90,0xdedbda90,0xd863da92,0xdedbda92,0xd863da94,0xdeddda94,0xd863da96,
	0xdeddda96,0xd863da98,0xdedfda98,0xd863da9a,0xdedfda9a,0xd863da9c,0xdee1da9c,0xd863da9e,
	0xdee1da9e,0xd863daa0,0xdee3daa0,0xd863daa2,0xdee3daa2,0xd863daa4,0xdee5daa4,0xd863daa6,
	0xdee5daa6,0xd863daa8,0xdee7daa8,0xd863daaa,0xdee7daaa,0xd863daac,0xdee9daac,0xd863daae,
	0xdee9daae,0xd863dab0,0xdeebdab0,0xd863dab2,0xdeebdab2,0xd863dab4,0xdeeddab4,0xd863dab6,
	0xdeeddab6,0xd863dab8,0xdeefdab8,0xd863daba,0xdeefdaba,0xd863dabc,0xdef1dabc,0xd863dabe,
	0xdef1dabe,0xd863dac0,0xdef3dac0,0xd863dac2,0xdef3dac2,0xd863dac4,0xdef5dac4,0xd863dac6,
	0xdef5dac6,0xd863dac8,0xdef7dac8,0xd863daca,0xdef7daca,0xd863dacc,0xdef9dacc,0xd863dace,
	0xdef9dace,0xd863dad0,0xdefbdad0,0xd863dad2,0xdefbdad2,0xd863dad4,0xdefddad4,0xd863dad6,
	0xdefddad6,0xd863dad8,0xdeffdad8,0xd863dada,0xdeffdada,0xd863dadc,0xdf01dadc,0xd863dade,
	0xdf01dade,0xd863dae0,0xdf03dae0,0xd863dae2,0xdf03dae2,0xd863dae4,0xdf05dae4,0xd863dae6,
	0xdf05dae6,0xd863dae8,0xdf07dae8,0xd863daea,0xdf07daea,0xd863daec,0xdf09daec,0xd863daee,
	0xdf09daee,0xd863daf0,0xdf0bdaf0,0xd863daf2,0xdf0bdaf2,0xd863daf4,0xdf0ddaf4,0xd863daf6,
	0xdf0ddaf6,0xd863daf8,0xdf0fdaf8,0xd863dafa,0xdf0fdafa,0xd863dafc,0xdf11dafc,0xd863dafe,
	0xdf11dafe,0xd863db00,0xdf13db00,0xd863db02,0xdf13db02,0xd863db04,0xdf15db04,0xd863db06,
	0xdf15db06,0xd863db08,0xdf17db08,0xd863db0a,0xdf17db0a,0xd863db0c,0xdf19db0c,0xd863db0e,
	0xdf19db0e,0xd863db10,0xdf1bdb10,0xd863db12,0xdf1bdb12,0xd863db14,0xdf1ddb14,0xd863db16,
	0xdf1ddb16,0xd863db18,0xdf1fdb18,0xd863db1a,0xdf1fdb1a,0xd863db1c,0xdf21db1c,0xd863db1e,
	0xdf21db1e,0xd863db20,0xdf23db20,0xd863db22,0xdf23db22,0xd863db24,0xdf25db24,0xd863db26,
	0xdf25db26,0xd863db28,0xdf27db28,0xd863db2a,0xdf27db2a,0xd863db2c,0xdf29db2c,0xd863db2e,
	0xdf29db2e,0xd863db30,0xdf2bdb30,0xd863db32,0xdf2bdb32,0xd863db34,0xdf2ddb34,0xd863db36,
	0xdf2ddb36,0xd863db38,0xdf2fdb38,0xd863db3a,0xdf2fdb3a,0xd863db3c,0xdf31db3c,0xd863db3e,
	0xdf31db3e,0xd863db40,0xdf33db40,0xd863db42,0xdf33db42,0xd863db44,0xdf35db44,0xd863db46,
	0xdf35db46,0xd863db48,0xdf37db48,0xd863db4a,0xdf37db4a,0xd863db4c,0xdf39db4c,0xd863db4e,
	0xdf39db4e,0xd863db50,0xdf3bdb50,0xd863db52,0xdf3bdb52,0xd863db54,0xdf3ddb54,0xd863db56,
	0xdf3ddb56,0xd863db58,0xdf3fdb58,0xd863db5a,0xdf3fdb5a,0xd863db5c,0xdf41db5c,0xd863db5e,
	0xdf41db5e,0xd863db60,0xdf43db60,0xd863db62,0xdf43db62,0xd863db64,0xdf45db64,0xd863db66,
	0xdf45db66,0xd863db68,0xdf47db68,0xd863db6a,0xdf47db6a,0xd863db6c,0xdf49db6c,0xd863db6e,
	0xdf49db6e,0xd863db70,0xdf4bdb70,0xd863db72,0xdf4bdb72,0xd863db74,0xdf4ddb74,0xd863db76,
	0xdf4ddb76,0xd863db78,0xdf4fdb78,0xd863db7a,0xdf4fdb7a,0xd863db7c,0xdf51db7c,0xd863db7e,
	0xdf51db7e,0xd863db80,0xdf53db80,0xd863db82,0xdf53db82,0xd863db84,0xdf55db84,0xd863db86,
	0xdf55db86,0xd863db88,0xdf57db88,0xd863db8a,0xdf57db8a,0xd863db8c,0xdf59db8c,0xd863db8e,
	0xdf59db8e,0xd863db90,0xdf5bdb90,0xd863db92,0xdf5bdb92,0xd863db94,0xdf5ddb94,0xd863db96,
	0xdf5ddb96,0xd863db98,0xdf5fdb98,0xd863db9a,0xdf5fdb9a,0xd863db9c,0xdf61db9c,0xd863db9e,
	0xdf61db9e,0xd863dba0,0xdf63dba0,0xd863dba2,0xdf63dba2,0xd863dba4,0xdf65dba4,0xd863dba6,
	0xdf65dba6,0xd863dba8,0xdf67dba8,0xd863dbaa,0xdf67dbaa,0xd863dbac,0xdf69dbac,0xd863dbae,
	0xdf69dbae,0xd863dbb0,0xdf6bdbb0,0xd863dbb2,0xdf6bdbb2,0xd863dbb4,0xdf6ddbb4,0xd863dbb6,
	0xdf6ddbb6,0xd863dbb8,0xdf6fdbb8,0xd863dbba,0xdf6fdbba,0xd863dbbc,0xdf71dbbc,0xd863dbbe,
	0xdf71dbbe,0xd863dbc0,0xdf73dbc0,0xd863dbc2,0xdf73dbc2,0xd863dbc4,0xdf75dbc4,0xd863dbc6,
	0xdf75dbc6,0xd863dbc8,0xdf77dbc8,0xd863dbca,0xdf77dbca,0xd863dbcc,0xdf79dbcc,0xd863dbce,
	0xdf79dbce,0xd863dbd0,0xdf7bdbd0,0xd863dbd2,0xdf7bdbd2,0xd863dbd4,0xdf7ddbd4,0xd863dbd6,
	0xdf7ddbd6,0xd863dbd8,0xdf7fdbd8,0xd863dbda,0xdf7fdbda,0xd863dbdc,0xdf81dbdc,0xd863dbde,
	0xdf81dbde,0xd863dbe0,0xdf83dbe0,0xd863dbe2,0xdf83dbe2,0xd863dbe4,0xdf85dbe4,0xd863dbe6,
	0xdf85dbe6,0xd863dbe8,0xdf87dbe8,0xd863dbea,0xdf87dbea,0xd863dbec,0xdf89dbec,0xd863dbee,
	0xdf89dbee,0xd863dbf0,0xdf8bdbf0,0xd863dbf2,0xdf8bdbf2,0xd863dbf4,0xdf8ddbf4,0xd863dbf6,
	0xdf8ddbf6,0xd863dbf8,0xdf8fdbf8,0xd863dbfa,0xdf8fdbfa,0xd863dbfc,0xdf91dbfc,0xd863dbfe,
	0xdf91dbfe,0xd863dc00,0xdf93dc00,0xd863dc02,0xdf93dc02,0xd863dc04,0xdf95dc04,0xd863dc06,
	0xdf95dc06,0xd863dc08,0xdf97dc08,0xd863dc0a,0xdf97dc0a,0xd863dc0c,0xdf99dc0c,0xd863dc0e,
	0xdf99dc0e,0xd863dc10,0xdf9bdc10,0xd863dc12,0xdf9bdc12,0xd863dc14,0xdf9ddc14,0xd863dc16,
	0xdf9ddc16,0xd863dc18,0xdf9fdc18,0xd863dc1a,0xdf9fdc1a,0xd863dc1c,0xdfa1dc1c,0xd863dc1e,
	0xdfa1dc1e,0xd863dc20,0xdfa3dc20,0xd863dc22,0xdfa3dc22,0xd863dc24,0xdfa5dc24,0xd863dc26,
	0xdfa5dc26,0xd863dc28,0xdfa7dc28,0xd863dc2a,0xdfa7dc2a,0xd863dc2c,0xdfa9dc2c,0xd863dc2e,
	0xdfa9dc2e,0xd863dc30,0xdfabdc30,0xd863dc32,0xdfabdc32,0xd863dc34,0xdfaddc34,0xd863dc36,
	0xdfaddc36,0xd863dc38,0xdfafdc38,0xd863dc3a,0xdfafdc3a,0xd863dc3c,0xdfb1dc3c,0xd863dc3e,
	0xdfb1dc3e,0xd863dc40,0xdfb3dc40,0xd863dc42,0xdfb3dc42,0xd863dc44,0xdfb5dc44,0xd863dc46,
	0xdfb5dc46,0xd863dc48,0xdfb7dc48,0xd863dc4a,0xdfb7dc4a,0xd863dc4c,0xdfb9dc4c,0xd863dc4e,
	0xdfb9dc4e,0xd863dc50,0xdfbbdc50,0xd863dc52,0xdfbbdc52,0xd863dc54,0xdfbddc54,0xd863dc56,
	0xdfbddc56,0xd863dc58,0xdfbfdc58,0xd863dc5a,0xdfbfdc5a,0xd863dc5c,0xdfc1dc5c,0xd863dc5e,
	0xdfc1dc5e,0xd863dc60,0xdfc3dc60,0xd863dc62,0xdfc3dc62,0xd863dc64,0xdfc5dc64,0xd863dc66,
	0xdfc5dc66,0xd863dc68,0xdfc7dc68,0xd863dc6a,0xdfc7dc6a,0xd863dc6c,0xdfc9dc6c,0xd863dc6e,
	0xdfc9dc6e,0xd863dc70,0xdfcbdc70,0xd863dc72,0xdfcbdc72,0xd863dc74,0xdfcddc74,0xd863dc76,
	0xdfcddc76,0xd863dc78,0xdfcfdc78,0xd863dc7a,0xdfcfdc7a,0xd863dc7c,0xdfd1dc7c,0xd863dc7e,
	0xdfd1dc7e,0xd863dc80,0xdfd3dc80,0xd863dc82,0xdfd3dc82,0xd863dc84,0xdfd5dc84,0xd863dc86,
	0xdfd5dc86,0xd863dc88,0xdfd7dc88,0xd863dc8a,0xdfd7dc8a,0xd863dc8c,0xdfd9dc8c,0xd863dc8e,
	0xdfd9dc8e,0xd863dc90,0xdfdbdc90,0xd863dc92,0xdfdbdc92,0xd863dc94,0xdfdddc94,0xd863dc96,
	0xdfdddc96,0xd863dc98,0xdfdfdc98,0xd863dc9a,0xdfdfdc9a,0xd863dc9c,0xdfe1dc9c,0xd863dc9e,
	0xdfe1dc9e,0xd863dca0,0xdfe3dca0,0xd863dca2,0xdfe3dca2,0xd863dca4,0xdfe5dca4,0xd863dca6,
	0xdfe5dca6,0xd863dca8,0xdfe7dca8,0xd863dcaa,0xdfe7dcaa,0xd863dcac,0xdfe9dcac,0xd863dcae,
	0xdfe9dcae,0xd863dcb0,0xdfebdcb0,0xd863dcb2,0xdfebdcb2,0xd863dcb4,0xdfeddcb4,0xd863dcb6,
	0xdfeddcb6,0xd863dcb8,0xdfefdcb8,0xd863dcba,0xdfefdcba,0xd863dcbc,0xdff1dcbc,0xd863dcbe,
	0xdff1dcbe,0xd863dcc0,0xdff3dcc0,0xd863dcc2,0xdff3dcc2,0xd863dcc4,0xdff5dcc4,0xd863dcc6,
	0xdff5dcc6,0xd863dcc8,0xdff7dcc8,0xd863dcca,0xdff7dcca,0xd863dccc,0xdff9dccc,0xd863dcce,
	0xdff9dcce,0xd863dcd0,0xdffbdcd0,0xd863dcd2,0xdffbdcd2,0xd863dcd4,0xdffddcd4,0xd863dcd6,
	0xdffddcd6,0xd863dcd8,0xdfffdcd8,0xd863dcda,0xdfffdcda,0xd865dcdc,0xdc01dcdc,0xd865dcde,
	0xdc01dcde,0xd865dce0,0xdc03dce0,0xd865dce2,0xdc03dce2,0xd865dce4,0xdc05dce4,0xd865dce6,
	0xdc05dce6,0xd865dce8,0xdc07dce8,0xd865dcea,0xdc07dcea,0xd865dcec,0xdc09dcec,0xd865dcee,
	0xdc09dcee,0xd865dcf0,0xdc0bdcf0,0xd865dcf2,0xdc0bdcf2,0xd865dcf4,0xdc0ddcf4,0xd865dcf6,
	0xdc0ddcf6,0xd865dcf8,0xdc0fdcf8,0xd865dcfa,0xdc0fdcfa,0xd865dcfc,0xdc11dcfc,0xd865dcfe,
	0xdc11dcfe,0xd865dd00,0xdc13dd00,0xd865dd02,0xdc13dd02,0xd865dd04,0xdc15dd04,0xd865dd06,
	0xdc15dd06,0xd865dd08,0xdc17dd08,0xd865dd0a,0xdc17dd0a,0xd865dd0c,0xdc19dd0c,0xd865dd0e,
	0xdc19dd0e,0xd865dd10,0xdc1bdd10,0xd865dd12,0xdc1bdd12,0xd865dd14,0xdc1ddd14,0xd865dd16,
	0xdc1ddd16,0xd865dd18,0xdc1fdd18,0xd865dd1a,0xdc1fdd1a,0xd865dd1c,0xdc21dd1c,0xd865dd1e,
	0xdc21dd1e,0xd865dd20,0xdc23dd20,0xd865dd22,0xdc23dd22,0xd865dd24,0xdc25dd24,0xd865dd26,
	0xdc25dd26,0xd865dd28,0xdc27dd28,0xd865dd2a,0xdc27dd2a,0xd865dd2c,0xdc29dd2c,0xd865dd2e,
	0xdc29dd2e,0xd865dd30,0xdc2bdd30,0xd865dd32,0xdc2bdd32,0xd865dd34,0xdc2ddd34,0xd865dd36,
	0xdc2ddd36,0xd865dd38,0xdc2fdd38,0xd865dd3a,0xdc2fdd3a,0xd865dd3c,0xdc31dd3c,0xd865dd3e,
	0xdc31dd3e,0xd865dd40,0xdc33dd40,0xd865dd42,0xdc33dd42,0xd865dd44,0xdc35dd44,0xd865dd46,
	0xdc35dd46,0xd865dd48,0xdc37dd48,0xd865dd4a,0xdc37dd4a,0xd865dd4c,0xdc39dd4c,0xd865dd4e,
	0xdc39dd4e,0xd865dd50,0xdc3bdd50,0xd865dd52,0xdc3bdd52,0xd865dd54,0xdc3ddd54,0xd865dd56,
	0xdc3ddd56,0xd865dd58,0xdc3fdd58,0xd865dd5a,0xdc3fdd5a,0xd865dd5c,0xdc41dd5c,0xd865dd5e,
	0xdc41dd5e,0xd865dd60,0xdc43dd60,0xd865dd62,0xdc43dd62,0xd865dd64,0xdc45dd64,0xd865dd66,
	0xdc45dd66,0xd865dd68,0xdc47dd68,0xd865dd6a,0xdc47dd6a,0xd865dd6c,0xdc49dd6c,0xd865dd6e,
	0xdc49dd6e,0xd865dd70,0xdc4bdd70,0xd865dd72,0xdc4bdd72,0xd865dd74,0xdc4ddd74,0xd865dd76,
	0xdc4ddd76,0xd865dd78,0xdc4fdd78,0xd865dd7a,0xdc4fdd7a,0xd865dd7c,0xdc51dd7c,0xd865dd7e,
	0xdc51dd7e,0xd865dd80,0xdc53dd80,0xd865dd82,0xdc53dd82,0xd865dd84,0xdc55dd84,0xd865dd86,
	0xdc55dd86,0xd865dd88,0xdc57dd88,0xd865dd8a,0xdc57dd8a,0xd865dd8c,0xdc59dd8c,0xd865dd8e,
	0xdc59dd8e,0xd865dd90,0xdc5bdd90,0xd865dd92,0xdc5bdd92,0xd865dd94,0xdc5ddd94,0xd865dd96,
	0xdc5ddd96,0xd865dd98,0xdc5fdd98,0xd865dd9a,0xdc5fdd9a,0xd865dd9c,0xdc61dd9c,0xd865dd9e,
	0xdc61dd9e,0xd865dda0,0xdc63dda0,0xd865dda2,0xdc63dda2,0xd865dda4,0xdc65dda4,0xd865dda6,
	0xdc65dda6,0xd865dda8,0xdc67dda8,0xd865ddaa,0xdc67ddaa,0xd865ddac,0xdc69ddac,0xd865ddae,
	0xdc69ddae,0xd865ddb0,0xdc6bddb0,0xd865ddb2,0xdc6bddb2,0xd865ddb4,0xdc6dddb4,0xd865ddb6,
	0xdc6dddb6,0xd865ddb8,0xdc6fddb8,0xd865ddba,0xdc6fddba,0xd865ddbc,0xdc71ddbc,0xd865ddbe,
	0xdc71ddbe,0xd865ddc0,0xdc73ddc0,0xd865ddc2,0xdc73ddc2,0xd865ddc4,0xdc75ddc4,0xd865ddc6,
	0xdc75ddc6,0xd865ddc8,0xdc77ddc8,0xd865ddca,0xdc77ddca,0xd865ddcc,0xdc79ddcc,0xd865ddce,
	0xdc79ddce,0xd865ddd0,0xdc7bddd0,0xd865ddd2,0xdc7bddd2,0xd865ddd4,0xdc7dddd4,0xd865ddd6,
	0xdc7dddd6,0xd865ddd8,0xdc7fddd8,0xd865ddda,0xdc7fddda,0xd865dddc,0xdc81dddc,0xd865ddde,
	0xdc81ddde,0xd865dde0,0xdc83dde0,0xd865dde2,0xdc83dde2,0xd865dde4,0xdc85dde4,0xd865dde6,
	0xdc85dde6,0xd865dde8,0xdc87dde8,0xd865ddea,0xdc87ddea,0xd865ddec,0xdc89ddec,0xd865ddee,
	0xdc89ddee,0xd865ddf0,0xdc8bddf0,0xd865ddf2,0xdc8bddf2,0xd865ddf4,0xdc8dddf4,0xd865ddf6,
	0xdc8dddf6,0xd865ddf8,0xdc8fddf8,0xd865ddfa,0xdc8fddfa,0xd865ddfc,0xdc91ddfc,0xd865ddfe,
	0xdc91ddfe,0xd865de00,0xdc93de00,0xd865de02,0xdc93de02,0xd865de04,0xdc95de04,0xd865de06,
	0xdc95de06,0xd865de08,0xdc97de08,0xd865de0a,0xdc97de0a,0xd865de0c,0xdc99de0c,0xd865de0e,
	0xdc99de0e,0xd865de10,0xdc9bde10,0xd865de12,0xdc9bde12,0xd865de14,0xdc9dde14,0xd865de16,
	0xdc9dde16,0xd865de18,0xdc9fde18,0xd865de1a,0xdc9fde1a,0xd865de1c,0xdca1de1c,0xd865de1e,
	0xdca1de1e,0xd865de20,0xdca3de20,0xd865de22,0xdca3de22,0xd865de24,0xdca5de24,0xd865de26,
	0xdca5de26,0xd865de28,0xdca7de28,0xd865de2a,0xdca7de2a,0xd865de2c,0xdca9de2c,0xd865de2e,
	0xdca9de2e,0xd865de30,0xdcabde30,0xd865de32,0xdcabde32,0xd865de34,0xdcadde34,0xd865de36,
	0xdcadde36,0xd865de38,0xdcafde38,0xd865de3a,0xdcafde3a,0xd865de3c,0xdcb1de3c,0xd865de3e,
	0xdcb1de3e,0xd865de40,0xdcb3de40,0xd865de42,0xdcb3de42,0xd865de44,0xdcb5de44,0xd865de46,
	0xdcb5de46,0xd865de48,0xdcb7de48,0xd865de4a,0xdcb7de4a,0xd865de4c,0xdcb9de4c,0xd865de4e,
	0xdcb9de4e,0xd865de50,0xdcbbde50,0xd865de52,0xdcbbde52,0xd865de54,0xdcbdde54,0xd865de56,
	0xdcbdde56,0xd865de58,0xdcbfde58,0xd865de5a,0xdcbfde5a,0xd865de5c,0xdcc1de5c,0xd865de5e,
	0xdcc1de5e,0xd865de60,0xdcc3de60,0xd865de62,0xdcc3de62,0xd865de64,0xdcc5de64,0xd865de66,
	0xdcc5de66,0xd865de68,0xdcc7de68,0xd865de6a,0xdcc7de6a,0xd865de6c,0xdcc9de6c,0xd865de6e,
	0xdcc9de6e,0xd865de70,0xdccbde70,0xd865de72,0xdccbde72,0xd865de74,0xdccdde74,0xd865de76,
	0xdccdde76,0xd865de78,0xdccfde78,0xd865de7a,0xdccfde7a,0xd865de7c,0xdcd1de7c,0xd865de7e,
	0xdcd1de7e,0xd865de80,0xdcd3de80,0xd865de82,0xdcd3de82,0xd865de84,0xdcd5de84,0xd865de86,
	0xdcd5de86,0xd865de88,0xdcd7de88,0xd865de8a,0xdcd7de8a,0xd865de8c,0xdcd9de8c,0xd865de8e,
	0xdcd9de8e,0xd865de90,0xdcdbde90,0xd865de92,0xdcdbde92,0xd865de94,0xdcddde94,0xd865de96,
	0xdcddde96,0xd865de98,0xdcdfde98,0xd865de9a,0xdcdfde9a,0xd865de9c,0xdce1de9c,0xd865de9e,
	0xdce1de9e,0xd865dea0,0xdce3dea0,0xd865dea2,0xdce3dea2,0xd865dea4,0xdce5dea4,0xd865dea6,
	0xdce5dea6,0xd865dea8,0xdce7dea8,0xd865deaa,0xdce7deaa,0xd865deac,0xdce9deac,0xd865deae,
	0xdce9deae,0xd865deb0,0xdcebdeb0,0xd865deb2,0xdcebdeb2,0xd865deb4,0xdceddeb4,0xd865deb6,
	0xdceddeb6,0xd865deb8,0xdcefdeb8,0xd865deba,0xdcefdeba,0xd865debc,0xdcf1debc,0xd865debe,
	0xdcf1debe,0xd865dec0,0xdcf3dec0,0xd865dec2,0xdcf3dec2,0xd865dec4,0xdcf5dec4,0xd865dec6,
	0xdcf5dec6,0xd865dec8,0xdcf7dec8,0xd865deca,0xdcf7deca,0xd865decc,0xdcf9decc,0xd865dece,
	0xdcf9dece,0xd865ded0,0xdcfbded0,0xd865ded2,0xdcfbded2,0xd865ded4,0xdcfdded4,0xd865ded6,
	0xdcfdded6,0xd865ded8,0xdcffded8,0xd865deda,0xdcffdeda,0xd865dedc,0xdd01dedc,0xd865dede,
	0xdd01dede,0xd865dee0,0xdd03dee0,0xd865dee2,0xdd03dee2,0xd865dee4,0xdd05dee4,0xd865dee6,
	0xdd05dee6,0xd865dee8,0xdd07dee8,0xd865deea,0xdd07deea,0xd865deec,0xdd09deec,0xd865deee,
	0xdd09deee,0xd865def0,0xdd0bdef0,0xd865def2,0xdd0bdef2,0xd865def4,0xdd0ddef4,0xd865def6,
	0xdd0ddef6,0xd865def8,0xdd0fdef8,0xd865defa,0xdd0fdefa,0xd865defc,0xdd11defc,0xd865defe,
	0xdd11defe,0xd865df00,0xdd13df00,0xd865df02,0xdd13df02,0xd865df04,0xdd15df04,0xd865df06,
	0xdd15df06,0xd865df08,0xdd17df08,0xd865df0a,0xdd17df0a,0xd865df0c,0xdd19df0c,0xd865df0e,
	0xdd19df0e,0xd865df10,0xdd1bdf10,0xd865df12,0xdd1bdf12,0xd865df14,0xdd1ddf14,0xd865df16,
	0xdd1ddf16,0xd865df18,0xdd1fdf18,0xd865df1a,0xdd1fdf1a,0xd865df1c,0xdd21df1c,0xd865df1e,
	0xdd21df1e,0xd865df20,0xdd23df20,0xd865df22,0xdd23df22,0xd865df24,0xdd25df24,0xd865df26,
	0xdd25df26,0xd865df28,0xdd27df28,0xd865df2a,0xdd27df2a,0xd865df2c,0xdd29df2c,0xd865df2e,
	0xdd29df2e,0xd865df30,0xdd2bdf30,0xd865df32,0xdd2bdf32,0xd865df34,0xdd2ddf34,0xd865df36,
	0xdd2ddf36,0xd865df38,0xdd2fdf38,0xd865df3a,0xdd2fdf3a,0xd865df3c,0xdd31df3c,0xd865df3e,
	0xdd31df3e,0xd865df40,0xdd33df40,0xd865df42,0xdd33df42,0xd865df44,0xdd35df44,0xd865df46,
	0xdd35df46,0xd865df48,0xdd37df48,0xd865df4a,0xdd37df4a,0xd865df4c,0xdd39df4c,0xd865df4e,
	0xdd39df4e,0xd865df50,0xdd3bdf50,0xd865df52,0xdd3bdf52,0xd865df54,0xdd3ddf54,0xd865df56,
	0xdd3ddf56,0xd865df58,0xdd3fdf58,0xd865df5a,0xdd3fdf5a,0xd865df5c,0xdd41df5c,0xd865df5e,
	0xdd41df5e,0xd865df60,0xdd43df60,0xd865df62,0xdd43df62,0xd865df64,0xdd45df64,0xd865df66,
	0xdd45df66,0xd865df68,0xdd47df68,0xd865df6a,0xdd47df6a,0xd865df6c,0xdd49df6c,0xd865df6e,
	0xdd49df6e,0xd865df70,0xdd4bdf70,0xd865df72,0xdd4bdf72,0xd865df74,0xdd4ddf74,0xd865df76,
	0xdd4ddf76,0xd865df78,0xdd4fdf78,0xd865df7a,0xdd4fdf7a,0xd865df7c,0xdd51df7c,0xd865df7e,
	0xdd51df7e,0xd865df80,0xdd53df80,0xd865df82,0xdd53df82,0xd865df84,0xdd55df84,0xd865df86,
	0xdd55df86,0xd865df88,0xdd57df88,0xd865df8a,0xdd57df8a,0xd865df8c,0xdd59df8c,0xd865df8e,
	0xdd59df8e,0xd865df90,0xdd5bdf90,0xd865df92,0xdd5bdf92,0xd865df94,0xdd5ddf94,0xd865df96,
	0xdd5ddf96,0xd865df98,0xdd5fdf98,0xd865df9a,0xdd5fdf9a,0xd865df9c,0xdd61df9c,0xd865df9e,
	0xdd61df9e,0xd865dfa0,0xdd63dfa0,0xd865dfa2,0xdd63dfa2,0xd865dfa4,0xdd65dfa4,0xd865dfa6,
	0xdd65dfa6,0xd865dfa8,0xdd67dfa8,0xd865dfaa,0xdd67dfaa,0xd865dfac,0xdd69dfac,0xd865dfae,
	0xdd69dfae,0xd865dfb0,0xdd6bdfb0,0xd865dfb2,0xdd6bdfb2,0xd865dfb4,0xdd6ddfb4,0xd865dfb6,
	0xdd6ddfb6,0xd865dfb8,0xdd6fdfb8,0xd865dfba,0xdd6fdfba,0xd865dfbc,0xdd71dfbc,0xd865dfbe,
	0xdd71dfbe,0xd865dfc0,0xdd73dfc0,0xd865dfc2,0xdd73dfc2,0xd865dfc4,0xdd75dfc4,0xd865dfc6,
	0xdd75dfc6,0xd865dfc8,0xdd77dfc8,0xd865dfca,0xdd77dfca,0xd865dfcc,0xdd79dfcc,0xd865dfce,
	0xdd79dfce,0xd865dfd0,0xdd7bdfd0,0xd865dfd2,0xdd7bdfd2,0xd865dfd4,0xdd7ddfd4,0xd865dfd6,
	0xdd7ddfd6,0xd865dfd8,0xdd7fdfd8,0xd865dfda,0xdd7fdfda,0xd865dfdc,0xdd81dfdc,0xd865dfde,
	0xdd81dfde,0xd865dfe0,0xdd83dfe0,0xd865dfe2,0xdd83dfe2,0xd865dfe4,0xdd85dfe4,0xd865dfe6,
	0xdd85dfe6,0xd865dfe8,0xdd87dfe8,0xd865dfea,0xdd87dfea,0xd865dfec,0xdd89dfec,0xd865dfee,
	0xdd89dfee,0xd865dff0,0xdd8bdff0,0xd865dff2,0xdd8bdff2,0xd865dff4,0xdd8ddff4,0xd865dff6,
	0xdd8ddff6,0xd865dff8,0xdd8fdff8,0xd865dffa,0xdd8fdffa,0xd865dffc,0xdd91dffc,0xd865dffe,
	0xdd91dffe,0xd865e000,0xdd93e000,0xd865e002,0xdd93e002,0xd865e004,0xdd95e004,0xd865e006,
	0xdd95e006,0xd865e008,0xdd97e008,0xd865e00a,0xdd97e00a,0xd865e00c,0xdd99e00c,0xd865e00e,
	0xdd99e00e,0xd865e010,0xdd9be010,0xd865e012,0xdd9be012,0xd865e014,0xdd9de014,0xd865e016,
	0xdd9de016,0xd865e018,0xdd9fe018,0xd865e01a,0xdd9fe01a,0xd865e01c,0xdda1e01c,0xd865e01e,
	0xdda1e01e,0xd865e020,0xdda3e020,0xd865e022,0xdda3e022,0xd865e024,0xdda5e024,0xd865e026,
	0xdda5e026,0xd865e028,0xdda7e028,0xd865e02a,0xdda7e02a,0xd865e02c,0xdda9e02c,0xd865e02e,
	0xdda9e02e,0xd865e030,0xddabe030,0xd865e032,0xddabe032,0xd865e034,0xddade034,0xd865e036,
	0xddade036,0xd865e038,0xddafe038,0xd865e03a,0xddafe03a,0xd865e03c,0xddb1e03c,0xd865e03e,
	0xddb1e03e,0xd865e040,0xddb3e040,0xd865e042,0xddb3e042,0xd865e044,0xddb5e044,0xd865e046,
	0xddb5e046,0xd865e048,0xddb7e048,0xd865e04a,0xddb7e04a,0xd865e04c,0xddb9e04c,0xd865e04e,
	0xddb9e04e,0xd865e050,0xddbbe050,0xd865e052,0xddbbe052,0xd865e054,0xddbde054,0xd865e056,
	0xddbde056,0xd865e058,0xddbfe058,0xd865e05a,0xddbfe05a,0xd865e05c,0xddc1e05c,0xd865e05e,
	0xddc1e05e,0xd865e060,0xddc3e060,0xd865e062,0xddc3e062,0xd865e064,0xddc5e064,0xd865e066,
	0xddc5e066,0xd865e068,0xddc7e068,0xd865e06a,0xddc7e06a,0xd865e06c,0xddc9e06c,0xd865e06e,
	0xddc9e06e,0xd865e070,0xddcbe070,0xd865e072,0xddcbe072,0xd865e074,0xddcde074,0xd865e076,
	0xddcde076,0xd865e078,0xddcfe078,0xd865e07a,0xddcfe07a,0xd865e07c,0xddd1e07c,0xd865e07e,
	0xddd1e07e,0xd865e080,0xddd3e080,0xd865e082,0xddd3e082,0xd865e084,0xddd5e084,0xd865e086,
	0xddd5e086,0xd865e088,0xddd7e088,0xd865e08a,0xddd7e08a,0xd865e08c,0xddd9e08c,0xd865e08e,
	0xddd9e08e,0xd865e090,0xdddbe090,0xd865e092,0xdddbe092,0xd865e094,0xdddde094,0xd865e096,
	0xdddde096,0xd865e098,0xdddfe098,0xd865e09a,0xdddfe09a,0xd865e09c,0xdde1e09c,0xd865e09e,
	0xdde1e09e,0xd865e0a0,0xdde3e0a0,0xd865e0a2,0xdde3e0a2,0xd865e0a4,0xdde5e0a4,0xd865e0a6,
	0xdde5e0a6,0xd865e0a8,0xdde7e0a8,0xd865e0aa,0xdde7e0aa,0xd865e0ac,0xdde9e0ac,0xd865e0ae,
	0xdde9e0ae,0xd865e0b0,0xddebe0b0,0xd865e0b2,0xddebe0b2,0xd865e0b4,0xddede0b4,0xd865e0b6,
	0xddede0b6,0xd865e0b8,0xddefe0b8,0xd865e0ba,0xddefe0ba,0xd865e0bc,0xddf1e0bc,0xd865e0be,
	0xddf1e0be,0xd865e0c0,0xddf3e0c0,0xd865e0c2,0xddf3e0c2,0xd865e0c4,0xddf5e0c4,0xd865e0c6,
	0xddf5e0c6,0xd865e0c8,0xddf7e0c8,0xd865e0ca,0xddf7e0ca,0xd865e0cc,0xddf9e0cc,0xd865e0ce,
	0xddf9e0ce,0xd865e0d0,0xddfbe0d0,0xd865e0d2,0xddfbe0d2,0xd865e0d4,0xddfde0d4,0xd865e0d6,
	0xddfde0d6,0xd865e0d8,0xddffe0d8,0xd865e0da,0xddffe0da,0xd865e0dc,0xde01e0dc,0xd865e0de,
	0xde01e0de,0xd865e0e0,0xde03e0e0,0xd865e0e2,0xde03e0e2,0xd865e0e4,0xde05e0e4,0xd865e0e6,
	0xde05e0e6,0xd865e0e8,0xde07e0e8,0xd865e0ea,0xde07e0ea,0xd865e0ec,0xde09e0ec,0xd865e0ee,
	0xde09e0ee,0xd865e0f0,0xde0be0f0,0xd865e0f2,0xde0be0f2,0xd865e0f4,0xde0de0f4,0xd865e0f6,
	0xde0de0f6,0xd865e0f8,0xde0fe0f8,0xd865e0fa,0xde0fe0fa,0xd865e0fc,0xde11e0fc,0xd865e0fe,
	0xde11e0fe,0xd865e100,0xde13e100,0xd865e102,0xde13e102,0xd865e104,0xde15e104,0xd865e106,
	0xde15e106,0xd865e108,0xde17e108,0xd865e10a,0xde17e10a,0xd865e10c,0xde19e10c,0xd865e10e,
	0xde19e10e,0xd865e110,0xde1be110,0xd865e112,0xde1be112,0xd865e114,0xde1de114,0xd865e116,
	0xde1de116,0xd865e118,0xde1fe118,0xd865e11a,0xde1fe11a,0xd865e11c,0xde21e11c,0xd865e11e,
	0xde21e11e,0xd865e120,0xde23e120,0xd865e122,0xde23e122,0xd865e124,0xde25e124,0xd865e126,
	0xde25e126,0xd865e128,0xde27e128,0xd865e12a,0xde27e12a,0xd865e12c,0xde29e12c,0xd865e12e,
	0xde29e12e,0xd865e130,0xde2be130,0xd865e132,0xde2be132,0xd865e134,0xde2de134,0xd865e136,
	0xde2de136,0xd865e138,0xde2fe138,0xd865e13a,0xde2fe13a,0xd865e13c,0xde31e13c,0xd865e13e,
	0xde31e13e,0xd865e140,0xde33e140,0xd865e142,0xde33e142,0xd865e144,0xde35e144,0xd865e146,
	0xde35e146,0xd865e148,0xde37e148,0xd865e14a,0xde37e14a,0xd865e14c,0xde39e14c,0xd865e14e,
	0xde39e14e,0xd865e150,0xde3be150,0xd865e152,0xde3be152,0xd865e154,0xde3de154,0xd865e156,
	0xde3de156,0xd865e158,0xde3fe158,0xd865e15a,0xde3fe15a,0xd865e15c,0xde41e15c,0xd865e15e,
	0xde41e15e,0xd865e160,0xde43e160,0xd865e162,0xde43e162,0xd865e164,0xde45e164,0xd865e166,
	0xde45e166,0xd865e168,0xde47e168,0xd865e16a,0xde47e16a,0xd865e16c,0xde49e16c,0xd865e16e,
	0xde49e16e,0xd865e170,0xde4be170,0xd865e172,0xde4be172,0xd865e174,0xde4de174,0xd865e176,
	0xde4de176,0xd865e178,0xde4fe178,0xd865e17a,0xde4fe17a,0xd865e17c,0xde51e17c,0xd865e17e,
	0xde51e17e,0xd865e180,0xde53e180,0xd865e182,0xde53e182,0xd865e184,0xde55e184,0xd865e186,
	0xde55e186,0xd865e188,0xde57e188,0xd865e18a,0xde57e18a,0xd865e18c,0xde59e18c,0xd865e18e,
	0xde59e18e,0xd865e190,0xde5be190,0xd865e192,0xde5be192,0xd865e194,0xde5de194,0xd865e196,
	0xde5de196,0xd865e198,0xde5fe198,0xd865e19a,0xde5fe19a,0xd865e19c,0xde61e19c,0xd865e19e,
	0xde61e19e,0xd865e1a0,0xde63e1a0,0xd865e1a2,0xde63e1a2,0xd865e1a4,0xde65e1a4,0xd865e1a6,
	0xde65e1a6,0xd865e1a8,0xde67e1a8,0xd865e1aa,0xde67e1aa,0xd865e1ac,0xde69e1ac,0xd865e1ae,
	0xde69e1ae,0xd865e1b0,0xde6be1b0,0xd865e1b2,0xde6be1b2,0xd865e1b4,0xde6de1b4,0xd865e1b6,
	0xde6de1b6,0xd865e1b8,0xde6fe1b8,0xd865e1ba,0xde6fe1ba,0xd865e1bc,0xde71e1bc,0xd865e1be,
	0xde71e1be,0xd865e1c0,0xde73e1c0,0xd865e1c2,0xde73e1c2,0xd865e1c4,0xde75e1c4,0xd865e1c6,
	0xde75e1c6,0xd865e1c8,0xde77e1c8,0xd865e1ca,0xde77e1ca,0xd865e1cc,0xde79e1cc,0xd865e1ce,
	0xde79e1ce,0xd865e1d0,0xde7be1d0,0xd865e1d2,0xde7be1d2,0xd865e1d4,0xde7de1d4,0xd865e1d6,
	0xde7de1d6,0xd865e1d8,0xde7fe1d8,0xd865e1da,0xde7fe1da,0xd865e1dc,0xde81e1dc,0xd865e1de,
	0xde81e1de,0xd865e1e0,0xde83e1e0,0xd865e1e2,0xde83e1e2,0xd865e1e4,0xde85e1e4,0xd865e1e6,
	0xde85e1e6,0xd865e1e8,0xde87e1e8,0xd865e1ea,0xde87e1ea,0xd865e1ec,0xde89e1ec,0xd865e1ee,
	0xde89e1ee,0xd865e1f0,0xde8be1f0,0xd865e1f2,0xde8be1f2,0xd865e1f4,0xde8de1f4,0xd865e1f6,
	0xde8de1f6,0xd865e1f8,0xde8fe1f8,0xd865e1fa,0xde8fe1fa,0xd865e1fc,0xde91e1fc,0xd865e1fe,
	0xde91e1fe,0xd865e200,0xde93e200,0xd865e202,0xde93e202,0xd865e204,0xde95e204,0xd865e206,
	0xde95e206,0xd865e208,0xde97e208,0xd865e20a,0xde97e20a,0xd865e20c,0xde99e20c,0xd865e20e,
	0xde99e20e,0xd865e210,0xde9be210,0xd865e212,0xde9be212,0xd865e214,0xde9de214,0xd865e216,
	0xde9de216,0xd865e218,0xde9fe218,0xd865e21a,0xde9fe21a,0xd865e21c,0xdea1e21c,0xd865e21e,
	0xdea1e21e,0xd865e220,0xdea3e220,0xd865e222,0xdea3e222,0xd865e224,0xdea5e224,0xd865e226,
	0xdea5e226,0xd865e228,0xdea7e228,0xd865e22a,0xdea7e22a,0xd865e22c,0xdea9e22c,0xd865e22e,
	0xdea9e22e,0xd865e230,0xdeabe230,0xd865e232,0xdeabe232,0xd865e234,0xdeade234,0xd865e236,
	0xdeade236,0xd865e238,0xdeafe238,0xd865e23a,0xdeafe23a,0xd865e23c,0xdeb1e23c,0xd865e23e,
	0xdeb1e23e,0xd865e240,0xdeb3e240,0xd865e242,0xdeb3e242,0xd865e244,0xdeb5e244,0xd865e246,
	0xdeb5e246,0xd865e248,0xdeb7e248,0xd865e24a,0xdeb7e24a,0xd865e24c,0xdeb9e24c,0xd865e24e,
	0xdeb9e24e,0xd865e250,0xdebbe250,0xd865e252,0xdebbe252,0xd865e254,0xdebde254,0xd865e256,
	0xdebde256,0xd865e258,0xdebfe258,0xd865e25a,0xdebfe25a,0xd865e25c,0xdec1e25c,0xd865e25e,
	0xdec1e25e,0xd865e260,0xdec3e260,0xd865e262,0xdec3e262,0xd865e264,0xdec5e264,0xd865e266,
	0xdec5e266,0xd865e268,0xdec7e268,0xd865e26a,0xdec7e26a,0xd865e26c,0xdec9e26c,0xd865e26e,
	0xdec9e26e,0xd865e270,0xdecbe270,0xd865e272,0xdecbe272,0xd865e274,0xdecde274,0xd865e276,
	0xdecde276,0xd865e278,0xdecfe278,0xd865e27a,0xdecfe27a,0xd865e27c,0xded1e27c,0xd865e27e,
	0xded1e27e,0xd865e280,0xded3e280,0xd865e282,0xded3e282,0xd865e284,0xded5e284,0xd865e286,
	0xded5e286,0xd865e288,0xded7e288,0xd865e28a,0xded7e28a,0xd865e28c,0xded9e28c,0xd865e28e,
	0xded9e28e,0xd865e290,0xdedbe290,0xd865e292,0xdedbe292,0xd865e294,0xdedde294,0xd865e296,
	0xdedde296,0xd865e298,0xdedfe298,0xd865e29a,0xdedfe29a,0xd865e29c,0xdee1e29c,0xd865e29e,
	0xdee1e29e,0xd865e2a0,0xdee3e2a0,0xd865e2a2,0xdee3e2a2,0xd865e2a4,0xdee5e2a4,0xd865e2a6,
	0xdee5e2a6,0xd865e2a8,0xdee7e2a8,0xd865e2aa,0xdee7e2aa,0xd865e2ac,0xdee9e2ac,0xd865e2ae,
	0xdee9e2ae,0xd865e2b0,0xdeebe2b0,0xd865e2b2,0xdeebe2b2,0xd865e2b4,0xdeede2b4,0xd865e2b6,
	0xdeede2b6,0xd865e2b8,0xdeefe2b8,0xd865e2ba,0xdeefe2ba,0xd865e2bc,0xdef1e2bc,0xd865e2be,
	0xdef1e2be,0xd865e2c0,0xdef3e2c0,0xd865e2c2,0xdef3e2c2,0xd865e2c4,0xdef5e2c4,0xd865e2c6,
	0xdef5e2c6,0xd865e2c8,0xdef7e2c8,0xd865e2ca,0xdef7e2ca,0xd865e2cc,0xdef9e2cc,0xd865e2ce,
	0xdef9e2ce,0xd865e2d0,0xdefbe2d0,0xd865e2d2,0xdefbe2d2,0xd865e2d4,0xdefde2d4,0xd865e2d6,
	0xdefde2d6,0xd865e2d8,0xdeffe2d8,0xd865e2da,0xdeffe2da,0xd865e2dc,0xdf01e2dc,0xd865e2de,
	0xdf01e2de,0xd865e2e0,0xdf03e2e0,0xd865e2e2,0xdf03e2e2,0xd865e2e4,0xdf05e2e4,0xd865e2e6,
	0xdf05e2e6,0xd865e2e8,0xdf07e2e8,0xd865e2ea,0xdf07e2ea,0xd865e2ec,0xdf09e2ec,0xd865e2ee,
	0xdf09e2ee,0xd865e2f0,0xdf0be2f0,0xd865e2f2,0xdf0be2f2,0xd865e2f4,0xdf0de2f4,0xd865e2f6,
	0xdf0de2f6,0xd865e2f8,0xdf0fe2f8,0xd865e2fa,0xdf0fe2fa,0xd865e2fc,0xdf11e2fc,0xd865e2fe,
	0xdf11e2fe,0xd865e300,0xdf13e300,0xd865e302,0xdf13e302,0xd865e304,0xdf15e304,0xd865e306,
	0xdf15e306,0xd865e308,0xdf17e308,0xd865e30a,0xdf17e30a,0xd865e30c,0xdf19e30c,0xd865e30e,
	0xdf19e30e,0xd865e310,0xdf1be310,0xd865e312,0xdf1be312,0xd865e314,0xdf1de314,0xd865e316,
	0xdf1de316,0xd865e318,0xdf1fe318,0xd865e31a,0xdf1fe31a,0xd865e31c,0xdf21e31c,0xd865e31e,
	0xdf21e31e,0xd865e320,0xdf23e320,0xd865e322,0xdf23e322,0xd865e324,0xdf25e324,0xd865e326,
	0xdf25e326,0xd865e328,0xdf27e328,0xd865e32a,0xdf27e32a,0xd865e32c,0xdf29e32c,0xd865e32e,
	0xdf29e32e,0xd865e330,0xdf2be330,0xd865e332,0xdf2be332,0xd865e334,0xdf2de334,0xd865e336,
	0xdf2de336,0xd865e338,0xdf2fe338,0xd865e33a,0xdf2fe33a,0xd865e33c,0xdf31e33c,0xd865e33e,
	0xdf31e33e,0xd865e340,0xdf33e340,0xd865e342,0xdf33e342,0xd865e344,0xdf35e344,0xd865e346,
	0xdf35e346,0xd865e348,0xdf37e348,0xd865e34a,0xdf37e34a,0xd865e34c,0xdf39e34c,0xd865e34e,
	0xdf39e34e,0xd865e350,0xdf3be350,0xd865e352,0xdf3be352,0xd865e354,0xdf3de354,0xd865e356,
	0xdf3de356,0xd865e358,0xdf3fe358,0xd865e35a,0xdf3fe35a,0xd865e35c,0xdf41e35c,0xd865e35e,
	0xdf41e35e,0xd865e360,0xdf43e360,0xd865e362,0xdf43e362,0xd865e364,0xdf45e364,0xd865e366,
	0xdf45e366,0xd865e368,0xdf47e368,0xd865e36a,0xdf47e36a,0xd865e36c,0xdf49e36c,0xd865e36e,
	0xdf49e36e,0xd865e370,0xdf4be370,0xd865e372,0xdf4be372,0xd865e374,0xdf4de374,0xd865e376,
	0xdf4de376,0xd865e378,0xdf4fe378,0xd865e37a,0xdf4fe37a,0xd865e37c,0xdf51e37c,0xd865e37e,
	0xdf51e37e,0xd865e380,0xdf53e380,0xd865e382,0xdf53e382,0xd865e384,0xdf55e384,0xd865e386,
	0xdf55e386,0xd865e388,0xdf57e388,0xd865e38a,0xdf57e38a,0xd865e38c,0xdf59e38c,0xd865e38e,
	0xdf59e38e,0xd865e390,0xdf5be390,0xd865e392,0xdf5be392,0xd865e394,0xdf5de394,0xd865e396,
	0xdf5de396,0xd865e398,0xdf5fe398,0xd865e39a,0xdf5fe39a,0xd865e39c,0xdf61e39c,0xd865e39e,
	0xdf61e39e,0xd865e3a0,0xdf63e3a0,0xd865e3a2,0xdf63e3a2,0xd865e3a4,0xdf65e3a4,0xd865e3a6,
	0xdf65e3a6,0xd865e3a8,0xdf67e3a8,0xd865e3aa,0xdf67e3aa,0xd865e3ac,0xdf69e3ac,0xd865e3ae,
	0xdf69e3ae,0xd865e3b0,0xdf6be3b0,0xd865e3b2,0xdf6be3b2,0xd865e3b4,0xdf6de3b4,0xd865e3b6,
	0xdf6de3b6,0xd865e3b8,0xdf6fe3b8,0xd865e3ba,0xdf6fe3ba,0xd865e3bc,0xdf71e3bc,0xd865e3be,
	0xdf71e3be,0xd865e3c0,0xdf73e3c0,0xd865e3c2,0xdf73e3c2,0xd865e3c4,0xdf75e3c4,0xd865e3c6,
	0xdf75e3c6,0xd865e3c8,0xdf77e3c8,0xd865e3ca,0xdf77e3ca,0xd865e3cc,0xdf79e3cc,0xd865e3ce,
	0xdf79e3ce,0xd865e3d0,0xdf7be3d0,0xd865e3d2,0xdf7be3d2,0xd865e3d4,0xdf7de3d4,0xd865e3d6,
	0xdf7de3d6,0xd865e3d8,0xdf7fe3d8,0xd865e3da,0xdf7fe3da,0xd865e3dc,0xdf81e3dc,0xd865e3de,
	0xdf81e3de,0xd865e3e0,0xdf83e3e0,0xd865e3e2,0xdf83e3e2,0xd865e3e4,0xdf85e3e4,0xd865e3e6,
	0xdf85e3e6,0xd865e3e8,0xdf87e3e8,0xd865e3ea,0xdf87e3ea,0xd865e3ec,0xdf89e3ec,0xd865e3ee,
	0xdf89e3ee,0xd865e3f0,0xdf8be3f0,0xd865e3f2,0xdf8be3f2,0xd865e3f4,0xdf8de3f4,0xd865e3f6,
	0xdf8de3f6,0xd865e3f8,0xdf8fe3f8,0xd865e3fa,0xdf8fe3fa,0xd865e3fc,0xdf91e3fc,0xd865e3fe,
	0xdf91e3fe,0xd865e400,0xdf93e400,0xd865e402,0xdf93e402,0xd865e404,0xdf95e404,0xd865e406,
	0xdf95e406,0xd865e408,0xdf97e408,0xd865e40a,0xdf97e40a,0xd865e40c,0xdf99e40c,0xd865e40e,
	0xdf99e40e,0xd865e410,0xdf9be410,0xd865e412,0xdf9be412,0xd865e414,0xdf9de414,0xd865e416,
	0xdf9de416,0xd865e418,0xdf9fe418,0xd865e41a,0xdf9fe41a,0xd865e41c,0xdfa1e41c,0xd865e41e,
	0xdfa1e41e,0xd865e420,0xdfa3e420,0xd865e422,0xdfa3e422,0xd865e424,0xdfa5e424,0xd865e426,
	0xdfa5e426,0xd865e428,0xdfa7e428,0xd865e42a,0xdfa7e42a,0xd865e42c,0xdfa9e42c,0xd865e42e,
	0xdfa9e42e,0xd865e430,0xdfabe430,0xd865e432,0xdfabe432,0xd865e434,0xdfade434,0xd865e436,
	0xdfade436,0xd865e438,0xdfafe438,0xd865e43a,0xdfafe43a,0xd865e43c,0xdfb1e43c,0xd865e43e,
	0xdfb1e43e,0xd865e440,0xdfb3e440,0xd865e442,0xdfb3e442,0xd865e444,0xdfb5e444,0xd865e446,
	0xdfb5e446,0xd865e448,0xdfb7e448,0xd865e44a,0xdfb7e44a,0xd865e44c,0xdfb9e44c,0xd865e44e,
	0xdfb9e44e,0xd865e450,0xdfbbe450,0xd865e452,0xdfbbe452,0xd865e454,0xdfbde454,0xd865e456,
	0xdfbde456,0xd865e458,0xdfbfe458,0xd865e45a,0xdfbfe45a,0xd865e45c,0xdfc1e45c,0xd865e45e,
	0xdfc1e45e,0xd865e460,0xdfc3e460,0xd865e462,0xdfc3e462,0xd865e464,0xdfc5e464,0xd865e466,
	0xdfc5e466,0xd865e468,0xdfc7e468,0xd865e46a,0xdfc7e46a,0xd865e46c,0xdfc9e46c,0xd865e46e,
	0xdfc9e46e,0xd865e470,0xdfcbe470,0xd865e472,0xdfcbe472,0xd865e474,0xdfcde474,0xd865e476,
	0xdfcde476,0xd865e478,0xdfcfe478,0xd865e47a,0xdfcfe47a,0xd865e47c,0xdfd1e47c,0xd865e47e,
	0xdfd1e47e,0xd865e480,0xdfd3e480,0xd865e482,0xdfd3e482,0xd865e484,0xdfd5e484,0xd865e486,
	0xdfd5e486,0xd865e488,0xdfd7e488,0xd865e48a,0xdfd7e48a,0xd865e48c,0xdfd9e48c,0xd865e48e,
	0xdfd9e48e,0xd865e490,0xdfdbe490,0xd865e492,0xdfdbe492,0xd865e494,0xdfdde494,0xd865e496,
	0xdfdde496,0xd865e498,0xdfdfe498,0xd865e49a,0xdfdfe49a,0xd865e49c,0xdfe1e49c,0xd865e49e,
	0xdfe1e49e,0xd865e4a0,0xdfe3e4a0,0xd865e4a2,0xdfe3e4a2,0xd865e4a4,0xdfe5e4a4,0xd865e4a6,
	0xdfe5e4a6,0xd865e4a8,0xdfe7e4a8,0xd865e4aa,0xdfe7e4aa,0xd865e4ac,0xdfe9e4ac,0xd865e4ae,
	0xdfe9e4ae,0xd865e4b0,0xdfebe4b0,0xd865e4b2,0xdfebe4b2,0xd865e4b4,0xdfede4b4,0xd865e4b6,
	0xdfede4b6,0xd865e4b8,0xdfefe4b8,0xd865e4ba,0xdfefe4ba,0xd865e4bc,0xdff1e4bc,0xd865e4be,
	0xdff1e4be,0xd865e4c0,0xdff3e4c0,0xd865e4c2,0xdff3e4c2,0xd865e4c4,0xdff5e4c4,0xd865e4c6,
	0xdff5e4c6,0xd865e4c8,0xdff7e4c8,0xd865e4ca,0xdff7e4ca,0xd865e4cc,0xdff9e4cc,0xd865e4ce,
	0xdff9e4ce,0xd865e4d0,0xdffbe4d0,0xd865e4d2,0xdffbe4d2,0xd865e4d4,0xdffde4d4,0xd865e4d6,
	0xdffde4d6,0xd865e4d8,0xdfffe4d8,0xd865e4da,0xdfffe4da,0xd865e4dc,0xdc01e4dc,0xd865e4de,
	0xdc01e4de,0xd865e4e0,0xdc03e4e0,0xd865e4e2,0xdc03e4e2,0xd865e4e4,0xdc05e4e4,0xd865e4e6,
	0xdc05e4e6,0xd865e4e8,0xdc07e4e8,0xd865e4ea,0xdc07e4ea,0xd865e4ec,0xdc09e4ec,0xd865e4ee,
	0xdc09e4ee,0xd865e4f0,0xdc0be4f0,0xd865e4f2,0xdc0be4f2,0xd865e4f4,0xdc0de4f4,0xd865e4f6,
	0xdc0de4f6,0xd865e4f8,0xdc0fe4f8,0xd865e4fa,0xdc0fe4fa,0xd865e4fc,0xdc11e4fc,0xd865e4fe,
	0xdc11e4fe,0xd865e500,0xdc13e500,0xd865e502,0xdc13e502,0xd865e504,0xdc15e504,0xd865e506,
	0xdc15e506,0xd865e508,0xdc17e508,0xd865e50a,0xdc17e50a,0xd865e50c,0xdc19e50c,0xd865e50e,
	0xdc19e50e,0xd865e510,0xdc1be510,0xd865e512,0xdc1be512,0xd865e514,0xdc1de514,0xd865e516,
	0xdc1de516,0xd865e518,0xdc1fe518,0xd865e51a,0xdc1fe51a,0xd865e51c,0xdc21e51c,0xd865e51e,
	0xdc21e51e,0xd865e520,0xdc23e520,0xd865e522,0xdc23e522,0xd865e524,0xdc25e524,0xd865e526,
	0xdc25e526,0xd865e528,0xdc27e528,0xd865e52a,0xdc27e52a,0xd865e52c,0xdc29e52c,0xd865e52e,
	0xdc29e52e,0xd865e530,0xdc2be530,0xd865e532,0xdc2be532,0xd865e534,0xdc2de534,0xd865e536,
	0xdc2de536,0xd865e538,0xdc2fe538,0xd865e53a,0xdc2fe53a,0xd865e53c,0xdc31e53c,0xd865e53e,
	0xdc31e53e,0xd865e540,0xdc33e540,0xd865e542,0xdc33e542,0xd865e544,0xdc35e544,0xd865e546,
	0xdc35e546,0xd865e548,0xdc37e548,0xd865e54a,0xdc37e54a,0xd865e54c,0xdc39e54c,0xd865e54e,
	0xdc39e54e,0xd865e550,0xdc3be550,0xd865e552,0xdc3be552,0xd865e554,0xdc3de554,0xd865e556,
	0xdc3de556,0xd865e558,0xdc3fe558,0xd865e55a,0xdc3fe55a,0xd865e55c,0xdc41e55c,0xd865e55e,
	0xdc41e55e,0xd865e560,0xdc43e560,0xd865e562,0xdc43e562,0xd865e564,0xdc45e564,0xd865e566,
	0xdc45e566,0xd865e568,0xdc47e568,0xd865e56a,0xdc47e56a,0xd865e56c,0xdc49e56c,0xd865e56e,
	0xdc49e56e,0xd865e570,0xdc4be570,0xd865e572,0xdc4be572,0xd865e574,0xdc4de574,0xd865e576,
	0xdc4de576,0xd865e578,0xdc4fe578,0xd865e57a,0xdc4fe57a,0xd865e57c,0xdc51e57c,0xd865e57e,
	0xdc51e57e,0xd865e580,0xdc53e580,0xd865e582,0xdc53e582,0xd865e584,0xdc55e584,0xd865e586,
	0xdc55e586,0xd865e588,0xdc57e588,0xd865e58a,0xdc57e58a,0xd865e58c,0xdc59e58c,0xd865e58e,
	0xdc59e58e,0xd865e590,0xdc5be590,0xd865e592,0xdc5be592,0xd865e594,0xdc5de594,0xd865e596,
	0xdc5de596,0xd865e598,0xdc5fe598,0xd865e59a,0xdc5fe59a,0xd865e59c,0xdc61e59c,0xd865e59e,
	0xdc61e59e,0xd865e5a0,0xdc63e5a0,0xd865e5a2,0xdc63e5a2,0xd865e5a4,0xdc65e5a4,0xd865e5a6,
	0xdc65e5a6,0xd865e5a8,0xdc67e5a8,0xd865e5aa,0xdc67e5aa,0xd865e5ac,0xdc69e5ac,0xd865e5ae,
	0xdc69e5ae,0xd865e5b0,0xdc6be5b0,0xd865e5b2,0xdc6be5b2,0xd865e5b4,0xdc6de5b4,0xd865e5b6,
	0xdc6de5b6,0xd865e5b8,0xdc6fe5b8,0xd865e5ba,0xdc6fe5ba,0xd865e5bc,0xdc71e5bc,0xd865e5be,
	0xdc71e5be,0xd865e5c0,0xdc73e5c0,0xd865e5c2,0xdc73e5c2,0xd865e5c4,0xdc75e5c4,0xd865e5c6,
	0xdc75e5c6,0xd865e5c8,0xdc77e5c8,0xd865e5ca,0xdc77e5ca,0xd865e5cc,0xdc79e5cc,0xd865e5ce,
	0xdc79e5ce,0xd865e5d0,0xdc7be5d0,0xd865e5d2,0xdc7be5d2,0xd865e5d4,0xdc7de5d4,0xd865e5d6,
	0xdc7de5d6,0xd865e5d8,0xdc7fe5d8,0xd865e5da,0xdc7fe5da,0xd865e5dc,0xdc81e5dc,0xd865e5de,
	0xdc81e5de,0xd865e5e0,0xdc83e5e0,0xd865e5e2,0xdc83e5e2,0xd865e5e4,0xdc85e5e4,0xd865e5e6,
	0xdc85e5e6,0xd865e5e8,0xdc87e5e8,0xd865e5ea,0xdc87e5ea,0xd865e5ec,0xdc89e5ec,0xd865e5ee,
	0xdc89e5ee,0xd865e5f0,0xdc8be5f0,0xd865e5f2,0xdc8be5f2,0xd865e5f4,0xdc8de5f4,0xd865e5f6,
	0xdc8de5f6,0xd865e5f8,0xdc8fe5f8,0xd865e5fa,0xdc8fe5fa,0xd865e5fc,0xdc91e5fc,0xd865e5fe,
	0xdc91e5fe,0xd865e600,0xdc93e600,0xd865e602,0xdc93e602,0xd865e604,0xdc95e604,0xd865e606,
	0xdc95e606,0xd865e608,0xdc97e608,0xd865e60a,0xdc97e60a,0xd865e60c,0xdc99e60c,0xd865e60e,
	0xdc99e60e,0xd865e610,0xdc9be610,0xd865e612,0xdc9be612,0xd865e614,0xdc9de614,0xd865e616,
	0xdc9de616,0xd865e618,0xdc9fe618,0xd865e61a,0xdc9fe61a,0xd865e61c,0xdca1e61c,0xd865e61e,
	0xdca1e61e,0xd865e620,0xdca3e620,0xd865e622,0xdca3e622,0xd865e624,0xdca5e624,0xd865e626,
	0xdca5e626,0xd865e628,0xdca7e628,0xd865e62a,0xdca7e62a,0xd865e62c,0xdca9e62c,0xd865e62e,
	0xdca9e62e,0xd865e630,0xdcabe630,0xd865e632,0xdcabe632,0xd865e634,0xdcade634,0xd865e636,
	0xdcade636,0xd865e638,0xdcafe638,0xd865e63a,0xdcafe63a,0xd865e63c,0xdcb1e63c,0xd865e63e,
	0xdcb1e63e,0xd865e640,0xdcb3e640,0xd865e642,0xdcb3e642,0xd865e644,0xdcb5e644,0xd865e646,
	0xdcb5e646,0xd865e648,0xdcb7e648,0xd865e64a,0xdcb7e64a,0xd865e64c,0xdcb9e64c,0xd865e64e,
	0xdcb9e64e,0xd865e650,0xdcbbe650,0xd865e652,0xdcbbe652,0xd865e654,0xdcbde654,0xd865e656,
	0xdcbde656,0xd865e658,0xdcbfe658,0xd865e65a,0xdcbfe65a,0xd865e65c,0xdcc1e65c,0xd865e65e,
	0xdcc1e65e,0xd865e660,0xdcc3e660,0xd865e662,0xdcc3e662,0xd865e664,0xdcc5e664,0xd865e666,
	0xdcc5e666,0xd865e668,0xdcc7e668,0xd865e66a,0xdcc7e66a,0xd865e66c,0xdcc9e66c,0xd865e66e,
	0xdcc9e66e,0xd865e670,0xdccbe670,0xd865e672,0xdccbe672,0xd865e674,0xdccde674,0xd865e676,
	0xdccde676,0xd865e678,0xdccfe678,0xd865e67a,0xdccfe67a,0xd865e67c,0xdcd1e67c,0xd865e67e,
	0xdcd1e67e,0xd865e680,0xdcd3e680,0xd865e682,0xdcd3e682,0xd865e684,0xdcd5e684,0xd865e686,
	0xdcd5e686,0xd865e688,0xdcd7e688,0xd865e68a,0xdcd7e68a,0xd865e68c,0xdcd9e68c,0xd865e68e,
	0xdcd9e68e,0xd865e690,0xdcdbe690,0xd865e692,0xdcdbe692,0xd865e694,0xdcdde694,0xd865e696,
	0xdcdde696,0xd865e698,0xdcdfe698,0xd865e69a,0xdcdfe69a,0xd865e69c,0xdce1e69c,0xd865e69e,
	0xdce1e69e,0xd865e6a0,0xdce3e6a0,0xd865e6a2,0xdce3e6a2,0xd865e6a4,0xdce5e6a4,0xd865e6a6,
	0xdce5e6a6,0xd865e6a8,0xdce7e6a8,0xd865e6aa,0xdce7e6aa,0xd865e6ac,0xdce9e6ac,0xd865e6ae,
	0xdce9e6ae,0xd865e6b0,0xdcebe6b0,0xd865e6b2,0xdcebe6b2,0xd865e6b4,0xdcede6b4,0xd865e6b6,
	0xdcede6b6,0xd865e6b8,0xdcefe6b8,0xd865e6ba,0xdcefe6ba,0xd865e6bc,0xdcf1e6bc,0xd865e6be,
	0xdcf1e6be,0xd865e6c0,0xdcf3e6c0,0xd865e6c2,0xdcf3e6c2,0xd865e6c4,0xdcf5e6c4,0xd865e6c6,
	0xdcf5e6c6,0xd865e6c8,0xdcf7e6c8,0xd865e6ca,0xdcf7e6ca,0xd865e6cc,0xdcf9e6cc,0xd865e6ce,
	0xdcf9e6ce,0xd865e6d0,0xdcfbe6d0,0xd865e6d2,0xdcfbe6d2,0xd865e6d4,0xdcfde6d4,0xd865e6d6,
	0xdcfde6d6,0xd865e6d8,0xdcffe6d8,0xd865e6da,0xdcffe6da,0xd865e6dc,0xdd01e6dc,0xd865e6de,
	0xdd01e6de,0xd865e6e0,0xdd03e6e0,0xd865e6e2,0xdd03e6e2,0xd865e6e4,0xdd05e6e4,0xd865e6e6,
	0xdd05e6e6,0xd865e6e8,0xdd07e6e8,0xd865e6ea,0xdd07e6ea,0xd865e6ec,0xdd09e6ec,0xd865e6ee,
	0xdd09e6ee,0xd865e6f0,0xdd0be6f0,0xd865e6f2,0xdd0be6f2,0xd865e6f4,0xdd0de6f4,0xd865e6f6,
	0xdd0de6f6,0xd865e6f8,0xdd0fe6f8,0xd865e6fa,0xdd0fe6fa,0xd865e6fc,0xdd11e6fc,0xd865e6fe,
	0xdd11e6fe,0xd865e700,0xdd13e700,0xd865e702,0xdd13e702,0xd865e704,0xdd15e704,0xd865e706,
	0xdd15e706,0xd865e708,0xdd17e708,0xd865e70a,0xdd17e70a,0xd865e70c,0xdd19e70c,0xd865e70e,
	0xdd19e70e,0xd865e710,0xdd1be710,0xd865e712,0xdd1be712,0xd865e714,0xdd1de714,0xd865e716,
	0xdd1de716,0xd865e718,0xdd1fe718,0xd865e71a,0xdd1fe71a,0xd865e71c,0xdd21e71c,0xd865e71e,
	0xdd21e71e,0xd865e720,0xdd23e720,0xd865e722,0xdd23e722,0xd865e724,0xdd25e724,0xd865e726,
	0xdd25e726,0xd865e728,0xdd27e728,0xd865e72a,0xdd27e72a,0xd865e72c,0xdd29e72c,0xd865e72e,
	0xdd29e72e,0xd865e730,0xdd2be730,0xd865e732,0xdd2be732,0xd865e734,0xdd2de734,0xd865e736,
	0xdd2de736,0xd865e738,0xdd2fe738,0xd865e73a,0xdd2fe73a,0xd865e73c,0xdd31e73c,0xd865e73e,
	0xdd31e73e,0xd865e740,0xdd33e740,0xd865e742,0xdd33e742,0xd865e744,0xdd35e744,0xd865e746,
	0xdd35e746,0xd865e748,0xdd37e748,0xd865e74a,0xdd37e74a,0xd865e74c,0xdd39e74c,0xd865e74e,
	0xdd39e74e,0xd865e750,0xdd3be750,0xd865e752,0xdd3be752,0xd865e754,0xdd3de754,0xd865e756,
	0xdd3de756,0xd865e758,0xdd3fe758,0xd865e75a,0xdd3fe75a,0xd865e75c,0xdd41e75c,0xd865e75e,
	0xdd41e75e,0xd865e760,0xdd43e760,0xd865e762,0xdd43e762,0xd865e764,0xdd45e764,0xd865e766,
	0xdd45e766,0xd865e768,0xdd47e768,0xd865e76a,0xdd47e76a,0xd865e76c,0xdd49e76c,0xd865e76e,
	0xdd49e76e,0xd865e770,0xdd4be770,0xd865e772,0xdd4be772,0xd865e774,0xdd4de774,0xd865e776,
	0xdd4de776,0xd865e778,0xdd4fe778,0xd865e77a,0xdd4fe77a,0xd865e77c,0xdd51e77c,0xd865e77e,
	0xdd51e77e,0xd865e780,0xdd53e780,0xd865e782,0xdd53e782,0xd865e784,0xdd55e784,0xd865e786,
	0xdd55e786,0xd865e788,0xdd57e788,0xd865e78a,0xdd57e78a,0xd865e78c,0xdd59e78c,0xd865e78e,
	0xdd59e78e,0xd865e790,0xdd5be790,0xd865e792,0xdd5be792,0xd865e794,0xdd5de794,0xd865e796,
	0xdd5de796,0xd865e798,0xdd5fe798,0xd865e79a,0xdd5fe79a,0xd865e79c,0xdd61e79c,0xd865e79e,
	0xdd61e79e,0xd865e7a0,0xdd63e7a0,0xd865e7a2,0xdd63e7a2,0xd865e7a4,0xdd65e7a4,0xd865e7a6,
	0xdd65e7a6,0xd865e7a8,0xdd67e7a8,0xd865e7aa,0xdd67e7aa,0xd865e7ac,0xdd69e7ac,0xd865e7ae,
	0xdd69e7ae,0xd865e7b0,0xdd6be7b0,0xd865e7b2,0xdd6be7b2,0xd865e7b4,0xdd6de7b4,0xd865e7b6,
	0xdd6de7b6,0xd865e7b8,0xdd6fe7b8,0xd865e7ba,0xdd6fe7ba,0xd865e7bc,0xdd71e7bc,0xd865e7be,
	0xdd71e7be,0xd865e7c0,0xdd73e7c0,0xd865e7c2,0xdd73e7c2,0xd865e7c4,0xdd75e7c4,0xd865e7c6,
	0xdd75e7c6,0xd865e7c8,0xdd77e7c8,0xd865e7ca,0xdd77e7ca,0xd865e7cc,0xdd79e7cc,0xd865e7ce,
	0xdd79e7ce,0xd865e7d0,0xdd7be7d0,0xd865e7d2,0xdd7be7d2,0xd865e7d4,0xdd7de7d4,0xd865e7d6,
	0xdd7de7d6,0xd865e7d8,0xdd7fe7d8,0xd865e7da,0xdd7fe7da,0xd865e7dc,0xdd81e7dc,0xd865e7de,
	0xdd81e7de,0xd865e7e0,0xdd83e7e0,0xd865e7e2,0xdd83e7e2,0xd865e7e4,0xdd85e7e4,0xd865e7e6,
	0xdd85e7e6,0xd865e7e8,0xdd87e7e8,0xd865e7ea,0xdd87e7ea,0xd865e7ec,0xdd89e7ec,0xd865e7ee,
	0xdd89e7ee,0xd865e7f0,0xdd8be7f0,0xd865e7f2,0xdd8be7f2,0xd865e7f4,0xdd8de7f4,0xd865e7f6,
	0xdd8de7f6,0xd865e7f8,0xdd8fe7f8,0xd865e7fa,0xdd8fe7fa,0xd865e7fc,0xdd91e7fc,0xd865e7fe,
	0xdd91e7fe,0xd865e800,0xdd93e800,0xd865e802,0xdd93e802,0xd865e804,0xdd95e804,0xd865e806,
	0xdd95e806,0xd865e808,0xdd97e808,0xd865e80a,0xdd97e80a,0xd865e80c,0xdd99e80c,0xd865e80e,
	0xdd99e80e,0xd865e810,0xdd9be810,0xd865e812,0xdd9be812,0xd865e814,0xdd9de814,0xd865e816,
	0xdd9de816,0xd865e818,0xdd9fe818,0xd865e81a,0xdd9fe81a,0xd865e81c,0xdda1e81c,0xd865e81e,
	0xdda1e81e,0xd865e820,0xdda3e820,0xd865e822,0xdda3e822,0xd865e824,0xdda5e824,0xd865e826,
	0xdda5e826,0xd865e828,0xdda7e828,0xd865e82a,0xdda7e82a,0xd865e82c,0xdda9e82c,0xd865e82e,
	0xdda9e82e,0xd865e830,0xddabe830,0xd865e832,0xddabe832,0xd865e834,0xddade834,0xd865e836,
	0xddade836,0xd865e838,0xddafe838,0xd865e83a,0xddafe83a,0xd865e83c,0xddb1e83c,0xd865e83e,
	0xddb1e83e,0xd865e840,0xddb3e840,0xd865e842,0xddb3e842,0xd865e844,0xddb5e844,0xd865e846,
	0xddb5e846,0xd865e848,0xddb7e848,0xd865e84a,0xddb7e84a,0xd865e84c,0xddb9e84c,0xd865e84e,
	0xddb9e84e,0xd865e850,0xddbbe850,0xd865e852,0xddbbe852,0xd865e854,0xddbde854,0xd865e856,
	0xddbde856,0xd865e858,0xddbfe858,0xd865e85a,0xddbfe85a,0xd865e85c,0xddc1e85c,0xd865e85e,
	0xddc1e85e,0xd865e860,0xddc3e860,0xd865e862,0xddc3e862,0xd865e864,0xddc5e864,0xd865e866,
	0xddc5e866,0xd865e868,0xddc7e868,0xd865e86a,0xddc7e86a,0xd865e86c,0xddc9e86c,0xd865e86e,
	0xddc9e86e,0xd865e870,0xddcbe870,0xd865e872,0xddcbe872,0xd865e874,0xddcde874,0xd865e876,
	0xddcde876,0xd865e878,0xddcfe878,0xd865e87a,0xddcfe87a,0xd865e87c,0xddd1e87c,0xd865e87e,
	0xddd1e87e,0xd865e880,0xddd3e880,0xd865e882,0xddd3e882,0xd865e884,0xddd5e884,0xd865e886,
	0xddd5e886,0xd865e888,0xddd7e888,0xd865e88a,0xddd7e88a,0xd865e88c,0xddd9e88c,0xd865e88e,
	0xddd9e88e,0xd865e890,0xdddbe890,0xd865e892,0xdddbe892,0xd865e894,0xdddde894,0xd865e896,
	0xdddde896,0xd865e898,0xdddfe898,0xd865e89a,0xdddfe89a,0xd865e89c,0xdde1e89c,0xd865e89e,
	0xdde1e89e,0xd865e8a0,0xdde3e8a0,0xd865e8a2,0xdde3e8a2,0xd865e8a4,0xdde5e8a4,0xd865e8a6,
	0xdde5e8a6,0xd865e8a8,0xdde7e8a8,0xd865e8aa,0xdde7e8aa,0xd865e8ac,0xdde9e8ac,0xd865e8ae,
	0xdde9e8ae,0xd865e8b0,0xddebe8b0,0xd865e8b2,0xddebe8b2,0xd865e8b4,0xddede8b4,0xd865e8b6,
	0xddede8b6,0xd865e8b8,0xddefe8b8,0xd865e8ba,0xddefe8ba,0xd865e8bc,0xddf1e8bc,0xd865e8be,
	0xddf1e8be,0xd865e8c0,0xddf3e8c0,0xd865e8c2,0xddf3e8c2,0xd865e8c4,0xddf5e8c4,0xd865e8c6,
	0xddf5e8c6,0xd865e8c8,0xddf7e8c8,0xd865e8ca,0xddf7e8ca,0xd865e8cc,0xddf9e8cc,0xd865e8ce,
	0xddf9e8ce,0xd865e8d0,0xddfbe8d0,0xd865e8d2,0xddfbe8d2,0xd865e8d4,0xddfde8d4,0xd865e8d6,
	0xddfde8d6,0xd865e8d8,0xddffe8d8,0xd865e8da,0xddffe8da,0xd865e8dc,0xde01e8dc,0xd865e8de,
	0xde01e8de,0xd865e8e0,0xde03e8e0,0xd865e8e2,0xde03e8e2,0xd865e8e4,0xde05e8e4,0xd865e8e6,
	0xde05e8e6,0xd865e8e8,0xde07e8e8,0xd865e8ea,0xde07e8ea,0xd865e8ec,0xde09e8ec,0xd865e8ee,
	0xde09e8ee,0xd865e8f0,0xde0be8f0,0xd865e8f2,0xde0be8f2,0xd865e8f4,0xde0de8f4,0xd865e8f6,
	0xde0de8f6,0xd865e8f8,0xde0fe8f8,0xd865e8fa,0xde0fe8fa,0xd865e8fc,0xde11e8fc,0xd865e8fe,
	0xde11e8fe,0xd865e900,0xde13e900,0xd865e902,0xde13e902,0xd865e904,0xde15e904,0xd865e906,
	0xde15e906,0xd865e908,0xde17e908,0xd865e90a,0xde17e90a,0xd865e90c,0xde19e90c,0xd865e90e,
	0xde19e90e,0xd865e910,0xde1be910,0xd865e912,0xde1be912,0xd865e914,0xde1de914,0xd865e916,
	0xde1de916,0xd865e918,0xde1fe918,0xd865e91a,0xde1fe91a,0xd865e91c,0xde21e91c,0xd865e91e,
	0xde21e91e,0xd865e920,0xde23e920,0xd865e922,0xde23e922,0xd865e924,0xde25e924,0xd865e926,
	0xde25e926,0xd865e928,0xde27e928,0xd865e92a,0xde27e92a,0xd865e92c,0xde29e92c,0xd865e92e,
	0xde29e92e,0xd865e930,0xde2be930,0xd865e932,0xde2be932,0xd865e934,0xde2de934,0xd865e936,
	0xde2de936,0xd865e938,0xde2fe938,0xd865e93a,0xde2fe93a,0xd865e93c,0xde31e93c,0xd865e93e,
	0xde31e93e,0xd865e940,0xde33e940,0xd865e942,0xde33e942,0xd865e944,0xde35e944,0xd865e946,
	0xde35e946,0xd865e948,0xde37e948,0xd865e94a,0xde37e94a,0xd865e94c,0xde39e94c,0xd865e94e,
	0xde39e94e,0xd865e950,0xde3be950,0xd865e952,0xde3be952,0xd865e954,0xde3de954,0xd865e956,
	0xde3de956,0xd865e958,0xde3fe958,0xd865e95a,0xde3fe95a,0xd865e95c,0xde41e95c,0xd865e95e,
	0xde41e95e,0xd865e960,0xde43e960,0xd865e962,0xde43e962,0xd865e964,0xde45e964,0xd865e966,
	0xde45e966,0xd865e968,0xde47e968,0xd865e96a,0xde47e96a,0xd865e96c,0xde49e96c,0xd865e96e,
	0xde49e96e,0xd865e970,0xde4be970,0xd865e972,0xde4be972,0xd865e974,0xde4de974,0xd865e976,
	0xde4de976,0xd865e978,0xde4fe978,0xd865e97a,0xde4fe97a,0xd865e97c,0xde51e97c,0xd865e97e,
	0xde51e97e,0xd865e980,0xde53e980,0xd865e982,0xde53e982,0xd865e984,0xde55e984,0xd865e986,
	0xde55e986,0xd865e988,0xde57e988,0xd865e98a,0xde57e98a,0xd865e98c,0xde59e98c,0xd865e98e,
	0xde59e98e,0xd865e990,0xde5be990,0xd865e992,0xde5be992,0xd865e994,0xde5de994,0xd865e996,
	0xde5de996,0xd865e998,0xde5fe998,0xd865e99a,0xde5fe99a,0xd865e99c,0xde61e99c,0xd865e99e,
	0xde61e99e,0xd865e9a0,0xde63e9a0,0xd865e9a2,0xde63e9a2,0xd865e9a4,0xde65e9a4,0xd865e9a6,
	0xde65e9a6,0xd865e9a8,0xde67e9a8,0xd865e9aa,0xde67e9aa,0xd865e9ac,0xde69e9ac,0xd865e9ae,
	0xde69e9ae,0xd865e9b0,0xde6be9b0,0xd865e9b2,0xde6be9b2,0xd865e9b4,0xde6de9b4,0xd865e9b6,
	0xde6de9b6,0xd865e9b8,0xde6fe9b8,0xd865e9ba,0xde6fe9ba,0xd865e9bc,0xde71e9bc,0xd865e9be,
	0xde71e9be,0xd865e9c0,0xde73e9c0,0xd865e9c2,0xde73e9c2,0xd865e9c4,0xde75e9c4,0xd865e9c6,
	0xde75e9c6,0xd865e9c8,0xde77e9c8,0xd865e9ca,0xde77e9ca,0xd865e9cc,0xde79e9cc,0xd865e9ce,
	0xde79e9ce,0xd865e9d0,0xde7be9d0,0xd865e9d2,0xde7be9d2,0xd865e9d4,0xde7de9d4,0xd865e9d6,
	0xde7de9d6,0xd865e9d8,0xde7fe9d8,0xd865e9da,0xde7fe9da,0xd865e9dc,0xde81e9dc,0xd865e9de,
	0xde81e9de,0xd865e9e0,0xde83e9e0,0xd865e9e2,0xde83e9e2,0xd865e9e4,0xde85e9e4,0xd865e9e6,
	0xde85e9e6,0xd865e9e8,0xde87e9e8,0xd865e9ea,0xde87e9ea,0xd865e9ec,0xde89e9ec,0xd865e9ee,
	0xde89e9ee,0xd865e9f0,0xde8be9f0,0xd865e9f2,0xde8be9f2,0xd865e9f4,0xde8de9f4,0xd865e9f6,
	0xde8de9f6,0xd865e9f8,0xde8fe9f8,0xd865e9fa,0xde8fe9fa,0xd865e9fc,0xde91e9fc,0xd865e9fe,
	0xde91e9fe,0xd865ea00,0xde93ea00,0xd865ea02,0xde93ea02,0xd865ea04,0xde95ea04,0xd865ea06,
	0xde95ea06,0xd865ea08,0xde97ea08,0xd865ea0a,0xde97ea0a,0xd865ea0c,0xde99ea0c,0xd865ea0e,
	0xde99ea0e,0xd865ea10,0xde9bea10,0xd865ea12,0xde9bea12,0xd865ea14,0xde9dea14,0xd865ea16,
	0xde9dea16,0xd865ea18,0xde9fea18,0xd865ea1a,0xde9fea1a,0xd865ea1c,0xdea1ea1c,0xd865ea1e,
	0xdea1ea1e,0xd865ea20,0xdea3ea20,0xd865ea22,0xdea3ea22,0xd865ea24,0xdea5ea24,0xd865ea26,
	0xdea5ea26,0xd865ea28,0xdea7ea28,0xd865ea2a,0xdea7ea2a,0xd865ea2c,0xdea9ea2c,0xd865ea2e,
	0xdea9ea2e,0xd865ea30,0xdeabea30,0xd865ea32,0xdeabea32,0xd865ea34,0xdeadea34,0xd865ea36,
	0xdeadea36,0xd865ea38,0xdeafea38,0xd865ea3a,0xdeafea3a,0xd865ea3c,0xdeb1ea3c,0xd865ea3e,
	0xdeb1ea3e,0xd865ea40,0xdeb3ea40,0xd865ea42,0xdeb3ea42,0xd865ea44,0xdeb5ea44,0xd865ea46,
	0xdeb5ea46,0xd865ea48,0xdeb7ea48,0xd865ea4a,0xdeb7ea4a,0xd865ea4c,0xdeb9ea4c,0xd865ea4e,
	0xdeb9ea4e,0xd865ea50,0xdebbea50,0xd865ea52,0xdebbea52,0xd865ea54,0xdebdea54,0xd865ea56,
	0xdebdea56,0xd865ea58,0xdebfea58,0xd865ea5a,0xdebfea5a,0xd865ea5c,0xdec1ea5c,0xd865ea5e,
	0xdec1ea5e,0xd865ea60,0xdec3ea60,0xd865ea62,0xdec3ea62,0xd865ea64,0xdec5ea64,0xd865ea66,
	0xdec5ea66,0xd865ea68,0xdec7ea68,0xd865ea6a,0xdec7ea6a,0xd865ea6c,0xdec9ea6c,0xd865ea6e,
	0xdec9ea6e,0xd865ea70,0xdecbea70,0xd865ea72,0xdecbea72,0xd865ea74,0xdecdea74,0xd865ea76,
	0xdecdea76,0xd865ea78,0xdecfea78,0xd865ea7a,0xdecfea7a,0xd865ea7c,0xded1ea7c,0xd865ea7e,
	0xded1ea7e,0xd865ea80,0xded3ea80,0xd865ea82,0xded3ea82,0xd865ea84,0xded5ea84,0xd865ea86,
	0xded5ea86,0xd865ea88,0xded7ea88,0xd865ea8a,0xded7ea8a,0xd865ea8c,0xded9ea8c,0xd865ea8e,
	0xded9ea8e,0xd865ea90,0xdedbea90,0xd865ea92,0xdedbea92,0xd865ea94,0xdeddea94,0xd865ea96,
	0xdeddea96,0xd865ea98,0xdedfea98,0xd865ea9a,0xdedfea9a,0xd865ea9c,0xdee1ea9c,0xd865ea9e,
	0xdee1ea9e,0xd865eaa0,0xdee3eaa0,0xd865eaa2,0xdee3eaa2,0xd865eaa4,0xdee5eaa4,0xd865eaa6,
	0xdee5eaa6,0xd865eaa8,0xdee7eaa8,0xd865eaaa,0xdee7eaaa,0xd865eaac,0xdee9eaac,0xd865eaae,
	0xdee9eaae,0xd865eab0,0xdeebeab0,0xd865eab2,0xdeebeab2,0xd865eab4,0xdeedeab4,0xd865eab6,
	0xdeedeab6,0xd865eab8,0xdeefeab8,0xd865eaba,0xdeefeaba,0xd865eabc,0xdef1eabc,0xd865eabe,
	0xdef1eabe,0xd865eac0,0xdef3eac0,0xd865eac2,0xdef3eac2,0xd865eac4,0xdef5eac4,0xd865eac6,
	0xdef5eac6,0xd865eac8,0xdef7eac8,0xd865eaca,0xdef7eaca,0xd865eacc,0xdef9eacc,0xd865eace,
	0xdef9eace,0xd865ead0,0xdefbead0,0xd865ead2,0xdefbead2,0xd865ead4,0xdefdead4,0xd865ead6,
	0xdefdead6,0xd865ead8,0xdeffead8,0xd865eada,0xdeffeada,0xd865eadc,0xdf01eadc,0xd865eade,
	0xdf01eade,0xd865eae0,0xdf03eae0,0xd865eae2,0xdf03eae2,0xd865eae4,0xdf05eae4,0xd865eae6,
	0xdf05eae6,0xd865eae8,0xdf07eae8,0xd865eaea,0xdf07eaea,0xd865eaec,0xdf09eaec,0xd865eaee,
	0xdf09eaee,0xd865eaf0,0xdf0beaf0,0xd865eaf2,0xdf0beaf2,0xd865eaf4,0xdf0deaf4,0xd865eaf6,
	0xdf0deaf6,0xd865eaf8,0xdf0feaf8,0xd865eafa,0xdf0feafa,0xd865eafc,0xdf11eafc,0xd865eafe,
	0xdf11eafe,0xd865eb00,0xdf13eb00,0xd865eb02,0xdf13eb02,0xd865eb04,0xdf15eb04,0xd865eb06,
	0xdf15eb06,0xd865eb08,0xdf17eb08,0xd865eb0a,0xdf17eb0a,0xd865eb0c,0xdf19eb0c,0xd865eb0e,
	0xdf19eb0e,0xd865eb10,0xdf1beb10,0xd865eb12,0xdf1beb12,0xd865eb14,0xdf1deb14,0xd865eb16,
	0xdf1deb16,0xd865eb18,0xdf1feb18,0xd865eb1a,0xdf1feb1a,0xd865eb1c,0xdf21eb1c,0xd865eb1e,
	0xdf21eb1e,0xd865eb20,0xdf23eb20,0xd865eb22,0xdf23eb22,0xd865eb24,0xdf25eb24,0xd865eb26,
	0xdf25eb26,0xd865eb28,0xdf27eb28,0xd865eb2a,0xdf27eb2a,0xd865eb2c,0xdf29eb2c,0xd865eb2e,
	0xdf29eb2e,0xd865eb30,0xdf2beb30,0xd865eb32,0xdf2beb32,0xd865eb34,0xdf2deb34,0xd865eb36,
	0xdf2deb36,0xd865eb38,0xdf2feb38,0xd865eb3a,0xdf2feb3a,0xd865eb3c,0xdf31eb3c,0xd865eb3e,
	0xdf31eb3e,0xd865eb40,0xdf33eb40,0xd865eb42,0xdf33eb42,0xd865eb44,0xdf35eb44,0xd865eb46,
	0xdf35eb46,0xd865eb48,0xdf37eb48,0xd865eb4a,0xdf37eb4a,0xd865eb4c,0xdf39eb4c,0xd865eb4e,
	0xdf39eb4e,0xd865eb50,0xdf3beb50,0xd865eb52,0xdf3beb52,0xd865eb54,0xdf3deb54,0xd865eb56,
	0xdf3deb56,0xd865eb58,0xdf3feb58,0xd865eb5a,0xdf3feb5a,0xd865eb5c,0xdf41eb5c,0xd865eb5e,
	0xdf41eb5e,0xd865eb60,0xdf43eb60,0xd865eb62,0xdf43eb62,0xd865eb64,0xdf45eb64,0xd865eb66,
	0xdf45eb66,0xd865eb68,0xdf47eb68,0xd865eb6a,0xdf47eb6a,0xd865eb6c,0xdf49eb6c,0xd865eb6e,
	0xdf49eb6e,0xd865eb70,0xdf4beb70,0xd865eb72,0xdf4beb72,0xd865eb74,0xdf4deb74,0xd865eb76,
	0xdf4deb76,0xd865eb78,0xdf4feb78,0xd865eb7a,0xdf4feb7a,0xd865eb7c,0xdf51eb7c,0xd865eb7e,
	0xdf51eb7e,0xd865eb80,0xdf53eb80,0xd865eb82,0xdf53eb82,0xd865eb84,0xdf55eb84,0xd865eb86,
	0xdf55eb86,0xd865eb88,0xdf57eb88,0xd865eb8a,0xdf57eb8a,0xd865eb8c,0xdf59eb8c,0xd865eb8e,
	0xdf59eb8e,0xd865eb90,0xdf5beb90,0xd865eb92,0xdf5beb92,0xd865eb94,0xdf5deb94,0xd865eb96,
	0xdf5deb96,0xd865eb98,0xdf5feb98,0xd865eb9a,0xdf5feb9a,0xd865eb9c,0xdf61eb9c,0xd865eb9e,
	0xdf61eb9e,0xd865eba0,0xdf63eba0,0xd865eba2,0xdf63eba2,0xd865eba4,0xdf65eba4,0xd865eba6,
	0xdf65eba6,0xd865eba8,0xdf67eba8,0xd865ebaa,0xdf67ebaa,0xd865ebac,0xdf69ebac,0xd865ebae,
	0xdf69ebae,0xd865ebb0,0xdf6bebb0,0xd865ebb2,0xdf6bebb2,0xd865ebb4,0xdf6debb4,0xd865ebb6,
	0xdf6debb6,0xd865ebb8,0xdf6febb8,0xd865ebba,0xdf6febba,0xd865ebbc,0xdf71ebbc,0xd865ebbe,
	0xdf71ebbe,0xd865ebc0,0xdf73ebc0,0xd865ebc2,0xdf73ebc2,0xd865ebc4,0xdf75ebc4,0xd865ebc6,
	0xdf75ebc6,0xd865ebc8,0xdf77ebc8,0xd865ebca,0xdf77ebca,0xd865ebcc,0xdf79ebcc,0xd865ebce,
	0xdf79ebce,0xd865ebd0,0xdf7bebd0,0xd865ebd2,0xdf7bebd2,0xd865ebd4,0xdf7debd4,0xd865ebd6,
	0xdf7debd6,0xd865ebd8,0xdf7febd8,0xd865ebda,0xdf7febda,0xd865ebdc,0xdf81ebdc,0xd865ebde,
	0xdf81ebde,0xd865ebe0,0xdf83ebe0,0xd865ebe2,0xdf83ebe2,0xd865ebe4,0xdf85ebe4,0xd865ebe6,
	0xdf85ebe6,0xd865ebe8,0xdf87ebe8,0xd865ebea,0xdf87ebea,0xd865ebec,0xdf89ebec,0xd865ebee,
	0xdf89ebee,0xd865ebf0,0xdf8bebf0,0xd865ebf2,0xdf8bebf2,0xd865ebf4,0xdf8debf4,0xd865ebf6,
	0xdf8debf6,0xd865ebf8,0xdf8febf8,0xd865ebfa,0xdf8febfa,0xd865ebfc,0xdf91ebfc,0xd865ebfe,
	0xdf91ebfe,0xd865ec00,0xdf93ec00,0xd865ec02,0xdf93ec02,0xd865ec04,0xdf95ec04,0xd865ec06,
	0xdf95ec06,0xd865ec08,0xdf97ec08,0xd865ec0a,0xdf97ec0a,0xd865ec0c,0xdf99ec0c,0xd865ec0e,
	0xdf99ec0e,0xd865ec10,0xdf9bec10,0xd865ec12,0xdf9bec12,0xd865ec14,0xdf9dec14,0xd865ec16,
	0xdf9dec16,0xd865ec18,0xdf9fec18,0xd865ec1a,0xdf9fec1a,0xd865ec1c,0xdfa1ec1c,0xd865ec1e,
	0xdfa1ec1e,0xd865ec20,0xdfa3ec20,0xd865ec22,0xdfa3ec22,0xd865ec24,0xdfa5ec24,0xd865ec26,
	0xdfa5ec26,0xd865ec28,0xdfa7ec28,0xd865ec2a,0xdfa7ec2a,0xd865ec2c,0xdfa9ec2c,0xd865ec2e,
	0xdfa9ec2e,0xd865ec30,0xdfabec30,0xd865ec32,0xdfabec32,0xd865ec34,0xdfadec34,0xd865ec36,
	0xdfadec36,0xd865ec38,0xdfafec38,0xd865ec3a,0xdfafec3a,0xd865ec3c,0xdfb1ec3c,0xd865ec3e,
	0xdfb1ec3e,0xd865ec40,0xdfb3ec40,0xd865ec42,0xdfb3ec42,0xd865ec44,0xdfb5ec44,0xd865ec46,
	0xdfb5ec46,0xd865ec48,0xdfb7ec48,0xd865ec4a,0xdfb7ec4a,0xd865ec4c,0xdfb9ec4c,0xd865ec4e,
	0xdfb9ec4e,0xd865ec50,0xdfbbec50,0xd865ec52,0xdfbbec52,0xd865ec54,0xdfbdec54,0xd865ec56,
	0xdfbdec56,0xd865ec58,0xdfbfec58,0xd865ec5a,0xdfbfec5a,0xd865ec5c,0xdfc1ec5c,0xd865ec5e,
	0xdfc1ec5e,0xd865ec60,0xdfc3ec60,0xd865ec62,0xdfc3ec62,0xd865ec64,0xdfc5ec64,0xd865ec66,
	0xdfc5ec66,0xd865ec68,0xdfc7ec68,0xd865ec6a,0xdfc7ec6a,0xd865ec6c,0xdfc9ec6c,0xd865ec6e,
	0xdfc9ec6e,0xd865ec70,0xdfcbec70,0xd865ec72,0xdfcbec72,0xd865ec74,0xdfcdec74,0xd865ec76,
	0xdfcdec76,0xd865ec78,0xdfcfec78,0xd865ec7a,0xdfcfec7a,0xd865ec7c,0xdfd1ec7c,0xd865ec7e,
	0xdfd1ec7e,0xd865ec80,0xdfd3ec80,0xd865ec82,0xdfd3ec82,0xd865ec84,0xdfd5ec84,0xd865ec86,
	0xdfd5ec86,0xd865ec88,0xdfd7ec88,0xd865ec8a,0xdfd7ec8a,0xd865ec8c,0xdfd9ec8c,0xd865ec8e,
	0xdfd9ec8e,0xd865ec90,0xdfdbec90,0xd865ec92,0xdfdbec92,0xd865ec94,0xdfddec94,0xd865ec96,
	0xdfddec96,0xd865ec98,0xdfdfec98,0xd865ec9a,0xdfdfec9a,0xd865ec9c,0xdfe1ec9c,0xd865ec9e,
	0xdfe1ec9e,0xd865eca0,0xdfe3eca0,0xd865eca2,0xdfe3eca2,0xd865eca4,0xdfe5eca4,0xd865eca6,
	0xdfe5eca6,0xd865eca8,0xdfe7eca8,0xd865ecaa,0xdfe7ecaa,0xd865ecac,0xdfe9ecac,0xd865ecae,
	0xdfe9ecae,0xd865ecb0,0xdfebecb0,0xd865ecb2,0xdfebecb2,0xd865ecb4,0xdfedecb4,0xd865ecb6,
	0xdfedecb6,0xd865ecb8,0xdfefecb8,0xd865ecba,0xdfefecba,0xd865ecbc,0xdff1ecbc,0xd865ecbe,
	0xdff1ecbe,0xd865ecc0,0xdff3ecc0,0xd865ecc2,0xdff3ecc2,0xd865ecc4,0xdff5ecc4,0xd865ecc6,
	0xdff5ecc6,0xd865ecc8,0xdff7ecc8,0xd865ecca,0xdff7ecca,0xd865eccc,0xdff9eccc,0xd865ecce,
	0xdff9ecce,0xd865ecd0,0xdffbecd0,0xd865ecd2,0xdffbecd2,0xd865ecd4,0xdffdecd4,0xd865ecd6,
	0xdffdecd6,0xd865ecd8,0xdfffecd8,0xd865ecda,0xdfffecda,0xd867ecdc,0xdc01ecdc,0xd867ecde,
	0xdc01ecde,0xd867ece0,0xdc03ece0,0xd867ece2,0xdc03ece2,0xd867ece4,0xdc05ece4,0xd867ece6,
	0xdc05ece6,0xd867ece8,0xdc07ece8,0xd867ecea,0xdc07ecea,0xd867ecec,0xdc09ecec,0xd867ecee,
	0xdc09ecee,0xd867ecf0,0xdc0becf0,0xd867ecf2,0xdc0becf2,0xd867ecf4,0xdc0decf4,0xd867ecf6,
	0xdc0decf6,0xd867ecf8,0xdc0fecf8,0xd867ecfa,0xdc0fecfa,0xd867ecfc,0xdc11ecfc,0xd867ecfe,
	0xdc11ecfe,0xd867ed00,0xdc13ed00,0xd867ed02,0xdc13ed02,0xd867ed04,0xdc15ed04,0xd867ed06,
	0xdc15ed06,0xd867ed08,0xdc17ed08,0xd867ed0a,0xdc17ed0a,0xd867ed0c,0xdc19ed0c,0xd867ed0e,
	0xdc19ed0e,0xd867ed10,0xdc1bed10,0xd867ed12,0xdc1bed12,0xd867ed14,0xdc1ded14,0xd867ed16,
	0xdc1ded16,0xd867ed18,0xdc1fed18,0xd867ed1a,0xdc1fed1a,0xd867ed1c,0xdc21ed1c,0xd867ed1e,
	0xdc21ed1e,0xd867ed20,0xdc23ed20,0xd867ed22,0xdc23ed22,0xd867ed24,0xdc25ed24,0xd867ed26,
	0xdc25ed26,0xd867ed28,0xdc27ed28,0xd867ed2a,0xdc27ed2a,0xd867ed2c,0xdc29ed2c,0xd867ed2e,
	0xdc29ed2e,0xd867ed30,0xdc2bed30,0xd867ed32,0xdc2bed32,0xd867ed34,0xdc2ded34,0xd867ed36,
	0xdc2ded36,0xd867ed38,0xdc2fed38,0xd867ed3a,0xdc2fed3a,0xd867ed3c,0xdc31ed3c,0xd867ed3e,
	0xdc31ed3e,0xd867ed40,0xdc33ed40,0xd867ed42,0xdc33ed42,0xd867ed44,0xdc35ed44,0xd867ed46,
	0xdc35ed46,0xd867ed48,0xdc37ed48,0xd867ed4a,0xdc37ed4a,0xd867ed4c,0xdc39ed4c,0xd867ed4e,
	0xdc39ed4e,0xd867ed50,0xdc3bed50,0xd867ed52,0xdc3bed52,0xd867ed54,0xdc3ded54,0xd867ed56,
	0xdc3ded56,0xd867ed58,0xdc3fed58,0xd867ed5a,0xdc3fed5a,0xd867ed5c,0xdc41ed5c,0xd867ed5e,
	0xdc41ed5e,0xd867ed60,0xdc43ed60,0xd867ed62,0xdc43ed62,0xd867ed64,0xdc45ed64,0xd867ed66,
	0xdc45ed66,0xd867ed68,0xdc47ed68,0xd867ed6a,0xdc47ed6a,0xd867ed6c,0xdc49ed6c,0xd867ed6e,
	0xdc49ed6e,0xd867ed70,0xdc4bed70,0xd867ed72,0xdc4bed72,0xd867ed74,0xdc4ded74,0xd867ed76,
	0xdc4ded76,0xd867ed78,0xdc4fed78,0xd867ed7a,0xdc4fed7a,0xd867ed7c,0xdc51ed7c,0xd867ed7e,
	0xdc51ed7e,0xd867ed80,0xdc53ed80,0xd867ed82,0xdc53ed82,0xd867ed84,0xdc55ed84,0xd867ed86,
	0xdc55ed86,0xd867ed88,0xdc57ed88,0xd867ed8a,0xdc57ed8a,0xd867ed8c,0xdc59ed8c,0xd867ed8e,
	0xdc59ed8e,0xd867ed90,0xdc5bed90,0xd867ed92,0xdc5bed92,0xd867ed94,0xdc5ded94,0xd867ed96,
	0xdc5ded96,0xd867ed98,0xdc5fed98,0xd867ed9a,0xdc5fed9a,0xd867ed9c,0xdc61ed9c,0xd867ed9e,
	0xdc61ed9e,0xd867eda0,0xdc63eda0,0xd867eda2,0xdc63eda2,0xd867eda4,0xdc65eda4,0xd867eda6,
	0xdc65eda6,0xd867eda8,0xdc67eda8,0xd867edaa,0xdc67edaa,0xd867edac,0xdc69edac,0xd867edae,
	0xdc69edae,0xd867edb0,0xdc6bedb0,0xd867edb2,0xdc6bedb2,0xd867edb4,0xdc6dedb4,0xd867edb6,
	0xdc6dedb6,0xd867edb8,0xdc6fedb8,0xd867edba,0xdc6fedba,0xd867edbc,0xdc71edbc,0xd867edbe,
	0xdc71edbe,0xd867edc0,0xdc73edc0,0xd867edc2,0xdc73edc2,0xd867edc4,0xdc75edc4,0xd867edc6,
	0xdc75edc6,0xd867edc8,0xdc77edc8,0xd867edca,0xdc77edca,0xd867edcc,0xdc79edcc,0xd867edce,
	0xdc79edce,0xd867edd0,0xdc7bedd0,0xd867edd2,0xdc7bedd2,0xd867edd4,0xdc7dedd4,0xd867edd6,
	0xdc7dedd6,0xd867edd8,0xdc7fedd8,0xd867edda,0xdc7fedda,0xd867eddc,0xdc81eddc,0xd867edde,
	0xdc81edde,0xd867ede0,0xdc83ede0,0xd867ede2,0xdc83ede2,0xd867ede4,0xdc85ede4,0xd867ede6,
	0xdc85ede6,0xd867ede8,0xdc87ede8,0xd867edea,0xdc87edea,0xd867edec,0xdc89edec,0xd867edee,
	0xdc89edee,0xd867edf0,0xdc8bedf0,0xd867edf2,0xdc8bedf2,0xd867edf4,0xdc8dedf4,0xd867edf6,
	0xdc8dedf6,0xd867edf8,0xdc8fedf8,0xd867edfa,0xdc8fedfa,0xd867edfc,0xdc91edfc,0xd867edfe,
	0xdc91edfe,0xd867ee00,0xdc93ee00,0xd867ee02,0xdc93ee02,0xd867ee04,0xdc95ee04,0xd867ee06,
	0xdc95ee06,0xd867ee08,0xdc97ee08,0xd867ee0a,0xdc97ee0a,0xd867ee0c,0xdc99ee0c,0xd867ee0e,
	0xdc99ee0e,0xd867ee10,0xdc9bee10,0xd867ee12,0xdc9bee12,0xd867ee14,0xdc9dee14,0xd867ee16,
	0xdc9dee16,0xd867ee18,0xdc9fee18,0xd867ee1a,0xdc9fee1a,0xd867ee1c,0xdca1ee1c,0xd867ee1e,
	0xdca1ee1e,0xd867ee20,0xdca3ee20,0xd867ee22,0xdca3ee22,0xd867ee24,0xdca5ee24,0xd867ee26,
	0xdca5ee26,0xd867ee28,0xdca7ee28,0xd867ee2a,0xdca7ee2a,0xd867ee2c,0xdca9ee2c,0xd867ee2e,
	0xdca9ee2e,0xd867ee30,0xdcabee30,0xd867ee32,0xdcabee32,0xd867ee34,0xdcadee34,0xd867ee36,
	0xdcadee36,0xd867ee38,0xdcafee38,0xd867ee3a,0xdcafee3a,0xd867ee3c,0xdcb1ee3c,0xd867ee3e,
	0xdcb1ee3e,0xd867ee40,0xdcb3ee40,0xd867ee42,0xdcb3ee42,0xd867ee44,0xdcb5ee44,0xd867ee46,
	0xdcb5ee46,0xd867ee48,0xdcb7ee48,0xd867ee4a,0xdcb7ee4a,0xd867ee4c,0xdcb9ee4c,0xd867ee4e,
	0xdcb9ee4e,0xd867ee50,0xdcbbee50,0xd867ee52,0xdcbbee52,0xd867ee54,0xdcbdee54,0xd867ee56,
	0xdcbdee56,0xd867ee58,0xdcbfee58,0xd867ee5a,0xdcbfee5a,0xd867ee5c,0xdcc1ee5c,0xd867ee5e,
	0xdcc1ee5e,0xd867ee60,0xdcc3ee60,0xd867ee62,0xdcc3ee62,0xd867ee64,0xdcc5ee64,0xd867ee66,
	0xdcc5ee66,0xd867ee68,0xdcc7ee68,0xd867ee6a,0xdcc7ee6a,0xd867ee6c,0xdcc9ee6c,0xd867ee6e,
	0xdcc9ee6e,0xd867ee70,0xdccbee70,0xd867ee72,0xdccbee72,0xd867ee74,0xdccdee74,0xd867ee76,
	0xdccdee76,0xd867ee78,0xdccfee78,0xd867ee7a,0xdccfee7a,0xd867ee7c,0xdcd1ee7c,0xd867ee7e,
	0xdcd1ee7e,0xd867ee80,0xdcd3ee80,0xd867ee82,0xdcd3ee82,0xd867ee84,0xdcd5ee84,0xd867ee86,
	0xdcd5ee86,0xd867ee88,0xdcd7ee88,0xd867ee8a,0xdcd7ee8a,0xd867ee8c,0xdcd9ee8c,0xd867ee8e,
	0xdcd9ee8e,0xd867ee90,0xdcdbee90,0xd867ee92,0xdcdbee92,0xd867ee94,0xdcddee94,0xd867ee96,
	0xdcddee96,0xd867ee98,0xdcdfee98,0xd867ee9a,0xdcdfee9a,0xd867ee9c,0xdce1ee9c,0xd867ee9e,
	0xdce1ee9e,0xd867eea0,0xdce3eea0,0xd867eea2,0xdce3eea2,0xd867eea4,0xdce5eea4,0xd867eea6,
	0xdce5eea6,0xd867eea8,0xdce7eea8,0xd867eeaa,0xdce7eeaa,0xd867eeac,0xdce9eeac,0xd867eeae,
	0xdce9eeae,0xd867eeb0,0xdcebeeb0,0xd867eeb2,0xdcebeeb2,0xd867eeb4,0xdcedeeb4,0xd867eeb6,
	0xdcedeeb6,0xd867eeb8,0xdcefeeb8,0xd867eeba,0xdcefeeba,0xd867eebc,0xdcf1eebc,0xd867eebe,
	0xdcf1eebe,0xd867eec0,0xdcf3eec0,0xd867eec2,0xdcf3eec2,0xd867eec4,0xdcf5eec4,0xd867eec6,
	0xdcf5eec6,0xd867eec8,0xdcf7eec8,0xd867eeca,0xdcf7eeca,0xd867eecc,0xdcf9eecc,0xd867eece,
	0xdcf9eece,0xd867eed0,0xdcfbeed0,0xd867eed2,0xdcfbeed2,0xd867eed4,0xdcfdeed4,0xd867eed6,
	0xdcfdeed6,0xd867eed8,0xdcffeed8,0xd867eeda,0xdcffeeda,0xd867eedc,0xdd01eedc,0xd867eede,
	0xdd01eede,0xd867eee0,0xdd03eee0,0xd867eee2,0xdd03eee2,0xd867eee4,0xdd05eee4,0xd867eee6,
	0xdd05eee6,0xd867eee8,0xdd07eee8,0xd867eeea,0xdd07eeea,0xd867eeec,0xdd09eeec,0xd867eeee,
	0xdd09eeee,0xd867eef0,0xdd0beef0,0xd867eef2,0xdd0beef2,0xd867eef4,0xdd0deef4,0xd867eef6,
	0xdd0deef6,0xd867eef8,0xdd0feef8,0xd867eefa,0xdd0feefa,0xd867eefc,0xdd11eefc,0xd867eefe,
	0xdd11eefe,0xd867ef00,0xdd13ef00,0xd867ef02,0xdd13ef02,0xd867ef04,0xdd15ef04,0xd867ef06,
	0xdd15ef06,0xd867ef08,0xdd17ef08,0xd867ef0a,0xdd17ef0a,0xd867ef0c,0xdd19ef0c,0xd867ef0e,
	0xdd19ef0e,0xd867ef10,0xdd1bef10,0xd867ef12,0xdd1bef12,0xd867ef14,0xdd1def14,0xd867ef16,
	0xdd1def16,0xd867ef18,0xdd1fef18,0xd867ef1a,0xdd1fef1a,0xd867ef1c,0xdd21ef1c,0xd867ef1e,
	0xdd21ef1e,0xd867ef20,0xdd23ef20,0xd867ef22,0xdd23ef22,0xd867ef24,0xdd25ef24,0xd867ef26,
	0xdd25ef26,0xd867ef28,0xdd27ef28,0xd867ef2a,0xdd27ef2a,0xd867ef2c,0xdd29ef2c,0xd867ef2e,
	0xdd29ef2e,0xd867ef30,0xdd2bef30,0xd867ef32,0xdd2bef32,0xd867ef34,0xdd2def34,0xd867ef36,
	0xdd2def36,0xd867ef38,0xdd2fef38,0xd867ef3a,0xdd2fef3a,0xd867ef3c,0xdd31ef3c,0xd867ef3e,
	0xdd31ef3e,0xd867ef40,0xdd33ef40,0xd867ef42,0xdd33ef42,0xd867ef44,0xdd35ef44,0xd867ef46,
	0xdd35ef46,0xd867ef48,0xdd37ef48,0xd867ef4a,0xdd37ef4a,0xd867ef4c,0xdd39ef4c,0xd867ef4e,
	0xdd39ef4e,0xd867ef50,0xdd3bef50,0xd867ef52,0xdd3bef52,0xd867ef54,0xdd3def54,0xd867ef56,
	0xdd3def56,0xd867ef58,0xdd3fef58,0xd867ef5a,0xdd3fef5a,0xd867ef5c,0xdd41ef5c,0xd867ef5e,
	0xdd41ef5e,0xd867ef60,0xdd43ef60,0xd867ef62,0xdd43ef62,0xd867ef64,0xdd45ef64,0xd867ef66,
	0xdd45ef66,0xd867ef68,0xdd47ef68,0xd867ef6a,0xdd47ef6a,0xd867ef6c,0xdd49ef6c,0xd867ef6e,
	0xdd49ef6e,0xd867ef70,0xdd4bef70,0xd867ef72,0xdd4bef72,0xd867ef74,0xdd4def74,0xd867ef76,
	0xdd4def76,0xd867ef78,0xdd4fef78,0xd867ef7a,0xdd4fef7a,0xd867ef7c,0xdd51ef7c,0xd867ef7e,
	0xdd51ef7e,0xd867ef80,0xdd53ef80,0xd867ef82,0xdd53ef82,0xd867ef84,0xdd55ef84,0xd867ef86,
	0xdd55ef86,0xd867ef88,0xdd57ef88,0xd867ef8a,0xdd57ef8a,0xd867ef8c,0xdd59ef8c,0xd867ef8e,
	0xdd59ef8e,0xd867ef90,0xdd5bef90,0xd867ef92,0xdd5bef92,0xd867ef94,0xdd5def94,0xd867ef96,
	0xdd5def96,0xd867ef98,0xdd5fef98,0xd867ef9a,0xdd5fef9a,0xd867ef9c,0xdd61ef9c,0xd867ef9e,
	0xdd61ef9e,0xd867efa0,0xdd63efa0,0xd867efa2,0xdd63efa2,0xd867efa4,0xdd65efa4,0xd867efa6,
	0xdd65efa6,0xd867efa8,0xdd67efa8,0xd867efaa,0xdd67efaa,0xd867efac,0xdd69efac,0xd867efae,
	0xdd69efae,0xd867efb0,0xdd6befb0,0xd867efb2,0xdd6befb2,0xd867efb4,0xdd6defb4,0xd867efb6,
	0xdd6defb6,0xd867efb8,0xdd6fefb8,0xd867efba,0xdd6fefba,0xd867efbc,0xdd71efbc,0xd867efbe,
	0xdd71efbe,0xd867efc0,0xdd73efc0,0xd867efc2,0xdd73efc2,0xd867efc4,0xdd75efc4,0xd867efc6,
	0xdd75efc6,0xd867efc8,0xdd77efc8,0xd867efca,0xdd77efca,0xd867efcc,0xdd79efcc,0xd867efce,
	0xdd79efce,0xd867efd0,0xdd7befd0,0xd867efd2,0xdd7befd2,0xd867efd4,0xdd7defd4,0xd867efd6,
	0xdd7defd6,0xd867efd8,0xdd7fefd8,0xd867efda,0xdd7fefda,0xd867efdc,0xdd81efdc,0xd867efde,
	0xdd81efde,0xd867efe0,0xdd83efe0,0xd867efe2,0xdd83efe2,0xd867efe4,0xdd85efe4,0xd867efe6,
	0xdd85efe6,0xd867efe8,0xdd87efe8,0xd867efea,0xdd87efea,0xd867efec,0xdd89efec,0xd867efee,
	0xdd89efee,0xd867eff0,0xdd8beff0,0xd867eff2,0xdd8beff2,0xd867eff4,0xdd8deff4,0xd867eff6,
	0xdd8deff6,0xd867eff8,0xdd8feff8,0xd867effa,0xdd8feffa,0xd867effc,0xdd91effc,0xd867effe,
	0xdd91effe,0xd867f000,0xdd93f000,0xd867f002,0xdd93f002,0xd867f004,0xdd95f004,0xd867f006,
	0xdd95f006,0xd867f008,0xdd97f008,0xd867f00a,0xdd97f00a,0xd867f00c,0xdd99f00c,0xd867f00e,
	0xdd99f00e,0xd867f010,0xdd9bf010,0xd867f012,0xdd9bf012,0xd867f014,0xdd9df014,0xd867f016,
	0xdd9df016,0xd867f018,0xdd9ff018,0xd867f01a,0xdd9ff01a,0xd867f01c,0xdda1f01c,0xd867f01e,
	0xdda1f01e,0xd867f020,0xdda3f020,0xd867f022,0xdda3f022,0xd867f024,0xdda5f024,0xd867f026,
	0xdda5f026,0xd867f028,0xdda7f028,0xd867f02a,0xdda7f02a,0xd867f02c,0xdda9f02c,0xd867f02e,
	0xdda9f02e,0xd867f030,0xddabf030,0xd867f032,0xddabf032,0xd867f034,0xddadf034,0xd867f036,
	0xddadf036,0xd867f038,0xddaff038,0xd867f03a,0xddaff03a,0xd867f03c,0xddb1f03c,0xd867f03e,
	0xddb1f03e,0xd867f040,0xddb3f040,0xd867f042,0xddb3f042,0xd867f044,0xddb5f044,0xd867f046,
	0xddb5f046,0xd867f048,0xddb7f048,0xd867f04a,0xddb7f04a,0xd867f04c,0xddb9f04c,0xd867f04e,
	0xddb9f04e,0xd867f050,0xddbbf050,0xd867f052,0xddbbf052,0xd867f054,0xddbdf054,0xd867f056,
	0xddbdf056,0xd867f058,0xddbff058,0xd867f05a,0xddbff05a,0xd867f05c,0xddc1f05c,0xd867f05e,
	0xddc1f05e,0xd867f060,0xddc3f060,0xd867f062,0xddc3f062,0xd867f064,0xddc5f064,0xd867f066,
	0xddc5f066,0xd867f068,0xddc7f068,0xd867f06a,0xddc7f06a,0xd867f06c,0xddc9f06c,0xd867f06e,
	0xddc9f06e,0xd867f070,0xddcbf070,0xd867f072,0xddcbf072,0xd867f074,0xddcdf074,0xd867f076,
	0xddcdf076,0xd867f078,0xddcff078,0xd867f07a,0xddcff07a,0xd867f07c,0xddd1f07c,0xd867f07e,
	0xddd1f07e,0xd867f080,0xddd3f080,0xd867f082,0xddd3f082,0xd867f084,0xddd5f084,0xd867f086,
	0xddd5f086,0xd867f088,0xddd7f088,0xd867f08a,0xddd7f08a,0xd867f08c,0xddd9f08c,0xd867f08e,
	0xddd9f08e,0xd867f090,0xdddbf090,0xd867f092,0xdddbf092,0xd867f094,0xddddf094,0xd867f096,
	0xddddf096,0xd867f098,0xdddff098,0xd867f09a,0xdddff09a,0xd867f09c,0xdde1f09c,0xd867f09e,
	0xdde1f09e,0xd867f0a0,0xdde3f0a0,0xd867f0a2,0xdde3f0a2,0xd867f0a4,0xdde5f0a4,0xd867f0a6,
	0xdde5f0a6,0xd867f0a8,0xdde7f0a8,0xd867f0aa,0xdde7f0aa,0xd867f0ac,0xdde9f0ac,0xd867f0ae,
	0xdde9f0ae,0xd867f0b0,0xddebf0b0,0xd867f0b2,0xddebf0b2,0xd867f0b4,0xddedf0b4,0xd867f0b6,
	0xddedf0b6,0xd867f0b8,0xddeff0b8,0xd867f0ba,0xddeff0ba,0xd867f0bc,0xddf1f0bc,0xd867f0be,
	0xddf1f0be,0xd867f0c0,0xddf3f0c0,0xd867f0c2,0xddf3f0c2,0xd867f0c4,0xddf5f0c4,0xd867f0c6,
	0xddf5f0c6,0xd867f0c8,0xddf7f0c8,0xd867f0ca,0xddf7f0ca,0xd867f0cc,0xddf9f0cc,0xd867f0ce,
	0xddf9f0ce,0xd867f0d0,0xddfbf0d0,0xd867f0d2,0xddfbf0d2,0xd867f0d4,0xddfdf0d4,0xd867f0d6,
	0xddfdf0d6,0xd867f0d8,0xddfff0d8,0xd867f0da,0xddfff0da,0xd867f0dc,0xde01f0dc,0xd867f0de,
	0xde01f0de,0xd867f0e0,0xde03f0e0,0xd867f0e2,0xde03f0e2,0xd867f0e4,0xde05f0e4,0xd867f0e6,
	0xde05f0e6,0xd867f0e8,0xde07f0e8,0xd867f0ea,0xde07f0ea,0xd867f0ec,0xde09f0ec,0xd867f0ee,
	0xde09f0ee,0xd867f0f0,0xde0bf0f0,0xd867f0f2,0xde0bf0f2,0xd867f0f4,0xde0df0f4,0xd867f0f6,
	0xde0df0f6,0xd867f0f8,0xde0ff0f8,0xd867f0fa,0xde0ff0fa,0xd867f0fc,0xde11f0fc,0xd867f0fe,
	0xde11f0fe,0xd867f100,0xde13f100,0xd867f102,0xde13f102,0xd867f104,0xde15f104,0xd867f106,
	0xde15f106,0xd867f108,0xde17f108,0xd867f10a,0xde17f10a,0xd867f10c,0xde19f10c,0xd867f10e,
	0xde19f10e,0xd867f110,0xde1bf110,0xd867f112,0xde1bf112,0xd867f114,0xde1df114,0xd867f116,
	0xde1df116,0xd867f118,0xde1ff118,0xd867f11a,0xde1ff11a,0xd867f11c,0xde21f11c,0xd867f11e,
	0xde21f11e,0xd867f120,0xde23f120,0xd867f122,0xde23f122,0xd867f124,0xde25f124,0xd867f126,
	0xde25f126,0xd867f128,0xde27f128,0xd867f12a,0xde27f12a,0xd867f12c,0xde29f12c,0xd867f12e,
	0xde29f12e,0xd867f130,0xde2bf130,0xd867f132,0xde2bf132,0xd867f134,0xde2df134,0xd867f136,
	0xde2df136,0xd867f138,0xde2ff138,0xd867f13a,0xde2ff13a,0xd867f13c,0xde31f13c,0xd867f13e,
	0xde31f13e,0xd867f140,0xde33f140,0xd867f142,0xde33f142,0xd867f144,0xde35f144,0xd867f146,
	0xde35f146,0xd867f148,0xde37f148,0xd867f14a,0xde37f14a,0xd867f14c,0xde39f14c,0xd867f14e,
	0xde39f14e,0xd867f150,0xde3bf150,0xd867f152,0xde3bf152,0xd867f154,0xde3df154,0xd867f156,
	0xde3df156,0xd867f158,0xde3ff158,0xd867f15a,0xde3ff15a,0xd867f15c,0xde41f15c,0xd867f15e,
	0xde41f15e,0xd867f160,0xde43f160,0xd867f162,0xde43f162,0xd867f164,0xde45f164,0xd867f166,
	0xde45f166,0xd867f168,0xde47f168,0xd867f16a,0xde47f16a,0xd867f16c,0xde49f16c,0xd867f16e,
	0xde49f16e,0xd867f170,0xde4bf170,0xd867f172,0xde4bf172,0xd867f174,0xde4df174,0xd867f176,
	0xde4df176,0xd867f178,0xde4ff178,0xd867f17a,0xde4ff17a,0xd867f17c,0xde51f17c,0xd867f17e,
	0xde51f17e,0xd867f180,0xde53f180,0xd867f182,0xde53f182,0xd867f184,0xde55f184,0xd867f186,
	0xde55f186,0xd867f188,0xde57f188,0xd867f18a,0xde57f18a,0xd867f18c,0xde59f18c,0xd867f18e,
	0xde59f18e,0xd867f190,0xde5bf190,0xd867f192,0xde5bf192,0xd867f194,0xde5df194,0xd867f196,
	0xde5df196,0xd867f198,0xde5ff198,0xd867f19a,0xde5ff19a,0xd867f19c,0xde61f19c,0xd867f19e,
	0xde61f19e,0xd867f1a0,0xde63f1a0,0xd867f1a2,0xde63f1a2,0xd867f1a4,0xde65f1a4,0xd867f1a6,
	0xde65f1a6,0xd867f1a8,0xde67f1a8,0xd867f1aa,0xde67f1aa,0xd867f1ac,0xde69f1ac,0xd867f1ae,
	0xde69f1ae,0xd867f1b0,0xde6bf1b0,0xd867f1b2,0xde6bf1b2,0xd867f1b4,0xde6df1b4,0xd867f1b6,
	0xde6df1b6,0xd867f1b8,0xde6ff1b8,0xd867f1ba,0xde6ff1ba,0xd867f1bc,0xde71f1bc,0xd867f1be,
	0xde71f1be,0xd867f1c0,0xde73f1c0,0xd867f1c2,0xde73f1c2,0xd867f1c4,0xde75f1c4,0xd867f1c6,
	0xde75f1c6,0xd867f1c8,0xde77f1c8,0xd867f1ca,0xde77f1ca,0xd867f1cc,0xde79f1cc,0xd867f1ce,
	0xde79f1ce,0xd867f1d0,0xde7bf1d0,0xd867f1d2,0xde7bf1d2,0xd867f1d4,0xde7df1d4,0xd867f1d6,
	0xde7df1d6,0xd867f1d8,0xde7ff1d8,0xd867f1da,0xde7ff1da,0xd867f1dc,0xde81f1dc,0xd867f1de,
	0xde81f1de,0xd867f1e0,0xde83f1e0,0xd867f1e2,0xde83f1e2,0xd867f1e4,0xde85f1e4,0xd867f1e6,
	0xde85f1e6,0xd867f1e8,0xde87f1e8,0xd867f1ea,0xde87f1ea,0xd867f1ec,0xde89f1ec,0xd867f1ee,
	0xde89f1ee,0xd867f1f0,0xde8bf1f0,0xd867f1f2,0xde8bf1f2,0xd867f1f4,0xde8df1f4,0xd867f1f6,
	0xde8df1f6,0xd867f1f8,0xde8ff1f8,0xd867f1fa,0xde8ff1fa,0xd867f1fc,0xde91f1fc,0xd867f1fe,
	0xde91f1fe,0xd867f200,0xde93f200,0xd867f202,0xde93f202,0xd867f204,0xde95f204,0xd867f206,
	0xde95f206,0xd867f208,0xde97f208,0xd867f20a,0xde97f20a,0xd867f20c,0xde99f20c,0xd867f20e,
	0xde99f20e,0xd867f210,0xde9bf210,0xd867f212,0xde9bf212,0xd867f214,0xde9df214,0xd867f216,
	0xde9df216,0xd867f218,0xde9ff218,0xd867f21a,0xde9ff21a,0xd867f21c,0xdea1f21c,0xd867f21e,
	0xdea1f21e,0xd867f220,0xdea3f220,0xd867f222,0xdea3f222,0xd867f224,0xdea5f224,0xd867f226,
	0xdea5f226,0xd867f228,0xdea7f228,0xd867f22a,0xdea7f22a,0xd867f22c,0xdea9f22c,0xd867f22e,
	0xdea9f22e,0xd867f230,0xdeabf230,0xd867f232,0xdeabf232,0xd867f234,0xdeadf234,0xd867f236,
	0xdeadf236,0xd867f238,0xdeaff238,0xd867f23a,0xdeaff23a,0xd867f23c,0xdeb1f23c,0xd867f23e,
	0xdeb1f23e,0xd867f240,0xdeb3f240,0xd867f242,0xdeb3f242,0xd867f244,0xdeb5f244,0xd867f246,
	0xdeb5f246,0xd867f248,0xdeb7f248,0xd867f24a,0xdeb7f24a,0xd867f24c,0xdeb9f24c,0xd867f24e,
	0xdeb9f24e,0xd867f250,0xdebbf250,0xd867f252,0xdebbf252,0xd867f254,0xdebdf254,0xd867f256,
	0xdebdf256,0xd867f258,0xdebff258,0xd867f25a,0xdebff25a,0xd867f25c,0xdec1f25c,0xd867f25e,
	0xdec1f25e,0xd867f260,0xdec3f260,0xd867f262,0xdec3f262,0xd867f264,0xdec5f264,0xd867f266,
	0xdec5f266,0xd867f268,0xdec7f268,0xd867f26a,0xdec7f26a,0xd867f26c,0xdec9f26c,0xd867f26e,
	0xdec9f26e,0xd867f270,0xdecbf270,0xd867f272,0xdecbf272,0xd867f274,0xdecdf274,0xd867f276,
	0xdecdf276,0xd867f278,0xdecff278,0xd867f27a,0xdecff27a,0xd867f27c,0xded1f27c,0xd867f27e,
	0xded1f27e,0xd867f280,0xded3f280,0xd867f282,0xded3f282,0xd867f284,0xded5f284,0xd867f286,
	0xded5f286,0xd867f288,0xded7f288,0xd867f28a,0xded7f28a,0xd867f28c,0xded9f28c,0xd867f28e,
	0xded9f28e,0xd867f290,0xdedbf290,0xd867f292,0xdedbf292,0xd867f294,0xdeddf294,0xd867f296,
	0xdeddf296,0xd867f298,0xdedff298,0xd867f29a,0xdedff29a,0xd867f29c,0xdee1f29c,0xd867f29e,
	0xdee1f29e,0xd867f2a0,0xdee3f2a0,0xd867f2a2,0xdee3f2a2,0xd867f2a4,0xdee5f2a4,0xd867f2a6,
	0xdee5f2a6,0xd867f2a8,0xdee7f2a8,0xd867f2aa,0xdee7f2aa,0xd867f2ac,0xdee9f2ac,0xd867f2ae,
	0xdee9f2ae,0xd867f2b0,0xdeebf2b0,0xd867f2b2,0xdeebf2b2,0xd867f2b4,0xdeedf2b4,0xd867f2b6,
	0xdeedf2b6,0xd867f2b8,0xdeeff2b8,0xd867f2ba,0xdeeff2ba,0xd867f2bc,0xdef1f2bc,0xd867f2be,
	0xdef1f2be,0xd867f2c0,0xdef3f2c0,0xd867f2c2,0xdef3f2c2,0xd867f2c4,0xdef5f2c4,0xd867f2c6,
	0xdef5f2c6,0xd867f2c8,0xdef7f2c8,0xd867f2ca,0xdef7f2ca,0xd867f2cc,0xdef9f2cc,0xd867f2ce,
	0xdef9f2ce,0xd867f2d0,0xdefbf2d0,0xd867f2d2,0xdefbf2d2,0xd867f2d4,0xdefdf2d4,0xd867f2d6,
	0xdefdf2d6,0xd867f2d8,0xdefff2d8,0xd867f2da,0xdefff2da,0xd867f2dc,0xdf01f2dc,0xd867f2de,
	0xdf01f2de,0xd867f2e0,0xdf03f2e0,0xd867f2e2,0xdf03f2e2,0xd867f2e4,0xdf05f2e4,0xd867f2e6,
	0xdf05f2e6,0xd867f2e8,0xdf07f2e8,0xd867f2ea,0xdf07f2ea,0xd867f2ec,0xdf09f2ec,0xd867f2ee,
	0xdf09f2ee,0xd867f2f0,0xdf0bf2f0,0xd867f2f2,0xdf0bf2f2,0xd867f2f4,0xdf0df2f4,0xd867f2f6,
	0xdf0df2f6,0xd867f2f8,0xdf0ff2f8,0xd867f2fa,0xdf0ff2fa,0xd867f2fc,0xdf11f2fc,0xd867f2fe,
	0xdf11f2fe,0xd867f300,0xdf13f300,0xd867f302,0xdf13f302,0xd867f304,0xdf15f304,0xd867f306,
	0xdf15f306,0xd867f308,0xdf17f308,0xd867f30a,0xdf17f30a,0xd867f30c,0xdf19f30c,0xd867f30e,
	0xdf19f30e,0xd867f310,0xdf1bf310,0xd867f312,0xdf1bf312,0xd867f314,0xdf1df314,0xd867f316,
	0xdf1df316,0xd867f318,0xdf1ff318,0xd867f31a,0xdf1ff31a,0xd867f31c,0xdf21f31c,0xd867f31e,
	0xdf21f31e,0xd867f320,0xdf23f320,0xd867f322,0xdf23f322,0xd867f324,0xdf25f324,0xd867f326,
	0xdf25f326,0xd867f328,0xdf27f328,0xd867f32a,0xdf27f32a,0xd867f32c,0xdf29f32c,0xd867f32e,
	0xdf29f32e,0xd867f330,0xdf2bf330,0xd867f332,0xdf2bf332,0xd867f334,0xdf2df334,0xd867f336,
	0xdf2df336,0xd867f338,0xdf2ff338,0xd867f33a,0xdf2ff33a,0xd867f33c,0xdf31f33c,0xd867f33e,
	0xdf31f33e,0xd867f340,0xdf33f340,0xd867f342,0xdf33f342,0xd867f344,0xdf35f344,0xd867f346,
	0xdf35f346,0xd867f348,0xdf37f348,0xd867f34a,0xdf37f34a,0xd867f34c,0xdf39f34c,0xd867f34e,
	0xdf39f34e,0xd867f350,0xdf3bf350,0xd867f352,0xdf3bf352,0xd867f354,0xdf3df354,0xd867f356,
	0xdf3df356,0xd867f358,0xdf3ff358,0xd867f35a,0xdf3ff35a,0xd867f35c,0xdf41f35c,0xd867f35e,
	0xdf41f35e,0xd867f360,0xdf43f360,0xd867f362,0xdf43f362,0xd867f364,0xdf45f364,0xd867f366,
	0xdf45f366,0xd867f368,0xdf47f368,0xd867f36a,0xdf47f36a,0xd867f36c,0xdf49f36c,0xd867f36e,
	0xdf49f36e,0xd867f370,0xdf4bf370,0xd867f372,0xdf4bf372,0xd867f374,0xdf4df374,0xd867f376,
	0xdf4df376,0xd867f378,0xdf4ff378,0xd867f37a,0xdf4ff37a,0xd867f37c,0xdf51f37c,0xd867f37e,
	0xdf51f37e,0xd867f380,0xdf53f380,0xd867f382,0xdf53f382,0xd867f384,0xdf55f384,0xd867f386,
	0xdf55f386,0xd867f388,0xdf57f388,0xd867f38a,0xdf57f38a,0xd867f38c,0xdf59f38c,0xd867f38e,
	0xdf59f38e,0xd867f390,0xdf5bf390,0xd867f392,0xdf5bf392,0xd867f394,0xdf5df394,0xd867f396,
	0xdf5df396,0xd867f398,0xdf5ff398,0xd867f39a,0xdf5ff39a,0xd867f39c,0xdf61f39c,0xd867f39e,
	0xdf61f39e,0xd867f3a0,0xdf63f3a0,0xd867f3a2,0xdf63f3a2,0xd867f3a4,0xdf65f3a4,0xd867f3a6,
	0xdf65f3a6,0xd867f3a8,0xdf67f3a8,0xd867f3aa,0xdf67f3aa,0xd867f3ac,0xdf69f3ac,0xd867f3ae,
	0xdf69f3ae,0xd867f3b0,0xdf6bf3b0,0xd867f3b2,0xdf6bf3b2,0xd867f3b4,0xdf6df3b4,0xd867f3b6,
	0xdf6df3b6,0xd867f3b8,0xdf6ff3b8,0xd867f3ba,0xdf6ff3ba,0xd867f3bc,0xdf71f3bc,0xd867f3be,
	0xdf71f3be,0xd867f3c0,0xdf73f3c0,0xd867f3c2,0xdf73f3c2,0xd867f3c4,0xdf75f3c4,0xd867f3c6,
	0xdf75f3c6,0xd867f3c8,0xdf77f3c8,0xd867f3ca,0xdf77f3ca,0xd867f3cc,0xdf79f3cc,0xd867f3ce,
	0xdf79f3ce,0xd867f3d0,0xdf7bf3d0,0xd867f3d2,0xdf7bf3d2,0xd867f3d4,0xdf7df3d4,0xd867f3d6,
	0xdf7df3d6,0xd867f3d8,0xdf7ff3d8,0xd867f3da,0xdf7ff3da,0xd867f3dc,0xdf81f3dc,0xd867f3de,
	0xdf81f3de,0xd867f3e0,0xdf83f3e0,0xd867f3e2,0xdf83f3e2,0xd867f3e4,0xdf85f3e4,0xd867f3e6,
	0xdf85f3e6,0xd867f3e8,0xdf87f3e8,0xd867f3ea,0xdf87f3ea,0xd867f3ec,0xdf89f3ec,0xd867f3ee,
	0xdf89f3ee,0xd867f3f0,0xdf8bf3f0,0xd867f3f2,0xdf8bf3f2,0xd867f3f4,0xdf8df3f4,0xd867f3f6,
	0xdf8df3f6,0xd867f3f8,0xdf8ff3f8,0xd867f3fa,0xdf8ff3fa,0xd867f3fc,0xdf91f3fc,0xd867f3fe,
	0xdf91f3fe,0xd867f400,0xdf93f400,0xd867f402,0xdf93f402,0xd867f404,0xdf95f404,0xd867f406,
	0xdf95f406,0xd867f408,0xdf97f408,0xd867f40a,0xdf97f40a,0xd867f40c,0xdf99f40c,0xd867f40e,
	0xdf99f40e,0xd867f410,0xdf9bf410,0xd867f412,0xdf9bf412,0xd867f414,0xdf9df414,0xd867f416,
	0xdf9df416,0xd867f418,0xdf9ff418,0xd867f41a,0xdf9ff41a,0xd867f41c,0xdfa1f41c,0xd867f41e,
	0xdfa1f41e,0xd867f420,0xdfa3f420,0xd867f422,0xdfa3f422,0xd867f424,0xdfa5f424,0xd867f426,
	0xdfa5f426,0xd867f428,0xdfa7f428,0xd867f42a,0xdfa7f42a,0xd867f42c,0xdfa9f42c,0xd867f42e,
	0xdfa9f42e,0xd867f430,0xdfabf430,0xd867f432,0xdfabf432,0xd867f434,0xdfadf434,0xd867f436,
	0xdfadf436,0xd867f438,0xdfaff438,0xd867f43a,0xdfaff43a,0xd867f43c,0xdfb1f43c,0xd867f43e,
	0xdfb1f43e,0xd867f440,0xdfb3f440,0xd867f442,0xdfb3f442,0xd867f444,0xdfb5f444,0xd867f446,
	0xdfb5f446,0xd867f448,0xdfb7f448,0xd867f44a,0xdfb7f44a,0xd867f44c,0xdfb9f44c,0xd867f44e,
	0xdfb9f44e,0xd867f450,0xdfbbf450,0xd867f452,0xdfbbf452,0xd867f454,0xdfbdf454,0xd867f456,
	0xdfbdf456,0xd867f458,0xdfbff458,0xd867f45a,0xdfbff45a,0xd867f45c,0xdfc1f45c,0xd867f45e,
	0xdfc1f45e,0xd867f460,0xdfc3f460,0xd867f462,0xdfc3f462,0xd867f464,0xdfc5f464,0xd867f466,
	0xdfc5f466,0xd867f468,0xdfc7f468,0xd867f46a,0xdfc7f46a,0xd867f46c,0xdfc9f46c,0xd867f46e,
	0xdfc9f46e,0xd867f470,0xdfcbf470,0xd867f472,0xdfcbf472,0xd867f474,0xdfcdf474,0xd867f476,
	0xdfcdf476,0xd867f478,0xdfcff478,0xd867f47a,0xdfcff47a,0xd867f47c,0xdfd1f47c,0xd867f47e,
	0xdfd1f47e,0xd867f480,0xdfd3f480,0xd867f482,0xdfd3f482,0xd867f484,0xdfd5f484,0xd867f486,
	0xdfd5f486,0xd867f488,0xdfd7f488,0xd867f48a,0xdfd7f48a,0xd867f48c,0xdfd9f48c,0xd867f48e,
	0xdfd9f48e,0xd867f490,0xdfdbf490,0xd867f492,0xdfdbf492,0xd867f494,0xdfddf494,0xd867f496,
	0xdfddf496,0xd867f498,0xdfdff498,0xd867f49a,0xdfdff49a,0xd867f49c,0xdfe1f49c,0xd867f49e,
	0xdfe1f49e,0xd867f4a0,0xdfe3f4a0,0xd867f4a2,0xdfe3f4a2,0xd867f4a4,0xdfe5f4a4,0xd867f4a6,
	0xdfe5f4a6,0xd867f4a8,0xdfe7f4a8,0xd867f4aa,0xdfe7f4aa,0xd867f4ac,0xdfe9f4ac,0xd867f4ae,
	0xdfe9f4ae,0xd867f4b0,0xdfebf4b0,0xd867f4b2,0xdfebf4b2,0xd867f4b4,0xdfedf4b4,0xd867f4b6,
	0xdfedf4b6,0xd867f4b8,0xdfeff4b8,0xd867f4ba,0xdfeff4ba,0xd867f4bc,0xdff1f4bc,0xd867f4be,
	0xdff1f4be,0xd867f4c0,0xdff3f4c0,0xd867f4c2,0xdff3f4c2,0xd867f4c4,0xdff5f4c4,0xd867f4c6,
	0xdff5f4c6,0xd867f4c8,0xdff7f4c8,0xd867f4ca,0xdff7f4ca,0xd867f4cc,0xdff9f4cc,0xd867f4ce,
	0xdff9f4ce,0xd867f4d0,0xdffbf4d0,0xd867f4d2,0xdffbf4d2,0xd867f4d4,0xdffdf4d4,0xd867f4d6,
	0xdffdf4d6,0xd867f4d8,0xdffff4d8,0xd867f4da,0xdffff4da,0xd867f4dc,0xdc01f4dc,0xd867f4de,
	0xdc01f4de,0xd867f4e0,0xdc03f4e0,0xd867f4e2,0xdc03f4e2,0xd867f4e4,0xdc05f4e4,0xd867f4e6,
	0xdc05f4e6,0xd867f4e8,0xdc07f4e8,0xd867f4ea,0xdc07f4ea,0xd867f4ec,0xdc09f4ec,0xd867f4ee,
	0xdc09f4ee,0xd867f4f0,0xdc0bf4f0,0xd867f4f2,0xdc0bf4f2,0xd867f4f4,0xdc0df4f4,0xd867f4f6,
	0xdc0df4f6,0xd867f4f8,0xdc0ff4f8,0xd867f4fa,0xdc0ff4fa,0xd867f4fc,0xdc11f4fc,0xd867f4fe,
	0xdc11f4fe,0xd867f500,0xdc13f500,0xd867f502,0xdc13f502,0xd867f504,0xdc15f504,0xd867f506,
	0xdc15f506,0xd867f508,0xdc17f508,0xd867f50a,0xdc17f50a,0xd867f50c,0xdc19f50c,0xd867f50e,
	0xdc19f50e,0xd867f510,0xdc1bf510,0xd867f512,0xdc1bf512,0xd867f514,0xdc1df514,0xd867f516,
	0xdc1df516,0xd867f518,0xdc1ff518,0xd867f51a,0xdc1ff51a,0xd867f51c,0xdc21f51c,0xd867f51e,
	0xdc21f51e,0xd867f520,0xdc23f520,0xd867f522,0xdc23f522,0xd867f524,0xdc25f524,0xd867f526,
	0xdc25f526,0xd867f528,0xdc27f528,0xd867f52a,0xdc27f52a,0xd867f52c,0xdc29f52c,0xd867f52e,
	0xdc29f52e,0xd867f530,0xdc2bf530,0xd867f532,0xdc2bf532,0xd867f534,0xdc2df534,0xd867f536,
	0xdc2df536,0xd867f538,0xdc2ff538,0xd867f53a,0xdc2ff53a,0xd867f53c,0xdc31f53c,0xd867f53e,
	0xdc31f53e,0xd867f540,0xdc33f540,0xd867f542,0xdc33f542,0xd867f544,0xdc35f544,0xd867f546,
	0xdc35f546,0xd867f548,0xdc37f548,0xd867f54a,0xdc37f54a,0xd867f54c,0xdc39f54c,0xd867f54e,
	0xdc39f54e,0xd867f550,0xdc3bf550,0xd867f552,0xdc3bf552,0xd867f554,0xdc3df554,0xd867f556,
	0xdc3df556,0xd867f558,0xdc3ff558,0xd867f55a,0xdc3ff55a,0xd867f55c,0xdc41f55c,0xd867f55e,
	0xdc41f55e,0xd867f560,0xdc43f560,0xd867f562,0xdc43f562,0xd867f564,0xdc45f564,0xd867f566,
	0xdc45f566,0xd867f568,0xdc47f568,0xd867f56a,0xdc47f56a,0xd867f56c,0xdc49f56c,0xd867f56e,
	0xdc49f56e,0xd867f570,0xdc4bf570,0xd867f572,0xdc4bf572,0xd867f574,0xdc4df574,0xd867f576,
	0xdc4df576,0xd867f578,0xdc4ff578,0xd867f57a,0xdc4ff57a,0xd867f57c,0xdc51f57c,0xd867f57e,
	0xdc51f57e,0xd867f580,0xdc53f580,0xd867f582,0xdc53f582,0xd867f584,0xdc55f584,0xd867f586,
	0xdc55f586,0xd867f588,0xdc57f588,0xd867f58a,0xdc57f58a,0xd867f58c,0xdc59f58c,0xd867f58e,
	0xdc59f58e,0xd867f590,0xdc5bf590,0xd867f592,0xdc5bf592,0xd867f594,0xdc5df594,0xd867f596,
	0xdc5df596,0xd867f598,0xdc5ff598,0xd867f59a,0xdc5ff59a,0xd867f59c,0xdc61f59c,0xd867f59e,
	0xdc61f59e,0xd867f5a0,0xdc63f5a0,0xd867f5a2,0xdc63f5a2,0xd867f5a4,0xdc65f5a4,0xd867f5a6,
	0xdc65f5a6,0xd867f5a8,0xdc67f5a8,0xd867f5aa,0xdc67f5aa,0xd867f5ac,0xdc69f5ac,0xd867f5ae,
	0xdc69f5ae,0xd867f5b0,0xdc6bf5b0,0xd867f5b2,0xdc6bf5b2,0xd867f5b4,0xdc6df5b4,0xd867f5b6,
	0xdc6df5b6,0xd867f5b8,0xdc6ff5b8,0xd867f5ba,0xdc6ff5ba,0xd867f5bc,0xdc71f5bc,0xd867f5be,
	0xdc71f5be,0xd867f5c0,0xdc73f5c0,0xd867f5c2,0xdc73f5c2,0xd867f5c4,0xdc75f5c4,0xd867f5c6,
	0xdc75f5c6,0xd867f5c8,0xdc77f5c8,0xd867f5ca,0xdc77f5ca,0xd867f5cc,0xdc79f5cc,0xd867f5ce,
	0xdc79f5ce,0xd867f5d0,0xdc7bf5d0,0xd867f5d2,0xdc7bf5d2,0xd867f5d4,0xdc7df5d4,0xd867f5d6,
	0xdc7df5d6,0xd867f5d8,0xdc7ff5d8,0xd867f5da,0xdc7ff5da,0xd867f5dc,0xdc81f5dc,0xd867f5de,
	0xdc81f5de,0xd867f5e0,0xdc83f5e0,0xd867f5e2,0xdc83f5e2,0xd867f5e4,0xdc85f5e4,0xd867f5e6,
	0xdc85f5e6,0xd867f5e8,0xdc87f5e8,0xd867f5ea,0xdc87f5ea,0xd867f5ec,0xdc89f5ec,0xd867f5ee,
	0xdc89f5ee,0xd867f5f0,0xdc8bf5f0,0xd867f5f2,0xdc8bf5f2,0xd867f5f4,0xdc8df5f4,0xd867f5f6,
	0xdc8df5f6,0xd867f5f8,0xdc8ff5f8,0xd867f5fa,0xdc8ff5fa,0xd867f5fc,0xdc91f5fc,0xd867f5fe,
	0xdc91f5fe,0xd867f600,0xdc93f600,0xd867f602,0xdc93f602,0xd867f604,0xdc95f604,0xd867f606,
	0xdc95f606,0xd867f608,0xdc97f608,0xd867f60a,0xdc97f60a,0xd867f60c,0xdc99f60c,0xd867f60e,
	0xdc99f60e,0xd867f610,0xdc9bf610,0xd867f612,0xdc9bf612,0xd867f614,0xdc9df614,0xd867f616,
	0xdc9df616,0xd867f618,0xdc9ff618,0xd867f61a,0xdc9ff61a,0xd867f61c,0xdca1f61c,0xd867f61e,
	0xdca1f61e,0xd867f620,0xdca3f620,0xd867f622,0xdca3f622,0xd867f624,0xdca5f624,0xd867f626,
	0xdca5f626,0xd867f628,0xdca7f628,0xd867f62a,0xdca7f62a,0xd867f62c,0xdca9f62c,0xd867f62e,
	0xdca9f62e,0xd867f630,0xdcabf630,0xd867f632,0xdcabf632,0xd867f634,0xdcadf634,0xd867f636,
	0xdcadf636,0xd867f638,0xdcaff638,0xd867f63a,0xdcaff63a,0xd867f63c,0xdcb1f63c,0xd867f63e,
	0xdcb1f63e,0xd867f640,0xdcb3f640,0xd867f642,0xdcb3f642,0xd867f644,0xdcb5f644,0xd867f646,
	0xdcb5f646,0xd867f648,0xdcb7f648,0xd867f64a,0xdcb7f64a,0xd867f64c,0xdcb9f64c,0xd867f64e,
	0xdcb9f64e,0xd867f650,0xdcbbf650,0xd867f652,0xdcbbf652,0xd867f654,0xdcbdf654,0xd867f656,
	0xdcbdf656,0xd867f658,0xdcbff658,0xd867f65a,0xdcbff65a,0xd867f65c,0xdcc1f65c,0xd867f65e,
	0xdcc1f65e,0xd867f660,0xdcc3f660,0xd867f662,0xdcc3f662,0xd867f664,0xdcc5f664,0xd867f666,
	0xdcc5f666,0xd867f668,0xdcc7f668,0xd867f66a,0xdcc7f66a,0xd867f66c,0xdcc9f66c,0xd867f66e,
	0xdcc9f66e,0xd867f670,0xdccbf670,0xd867f672,0xdccbf672,0xd867f674,0xdccdf674,0xd867f676,
	0xdccdf676,0xd867f678,0xdccff678,0xd867f67a,0xdccff67a,0xd867f67c,0xdcd1f67c,0xd867f67e,
	0xdcd1f67e,0xd867f680,0xdcd3f680,0xd867f682,0xdcd3f682,0xd867f684,0xdcd5f684,0xd867f686,
	0xdcd5f686,0xd867f688,0xdcd7f688,0xd867f68a,0xdcd7f68a,0xd867f68c,0xdcd9f68c,0xd867f68e,
	0xdcd9f68e,0xd867f690,0xdcdbf690,0xd867f692,0xdcdbf692,0xd867f694,0xdcddf694,0xd867f696,
	0xdcddf696,0xd867f698,0xdcdff698,0xd867f69a,0xdcdff69a,0xd867f69c,0xdce1f69c,0xd867f69e,
	0xdce1f69e,0xd867f6a0,0xdce3f6a0,0xd867f6a2,0xdce3f6a2,0xd867f6a4,0xdce5f6a4,0xd867f6a6,
	0xdce5f6a6,0xd867f6a8,0xdce7f6a8,0xd867f6aa,0xdce7f6aa,0xd867f6ac,0xdce9f6ac,0xd867f6ae,
	0xdce9f6ae,0xd867f6b0,0xdcebf6b0,0xd867f6b2,0xdcebf6b2,0xd867f6b4,0xdcedf6b4,0xd867f6b6,
	0xdcedf6b6,0xd867f6b8,0xdceff6b8,0xd867f6ba,0xdceff6ba,0xd867f6bc,0xdcf1f6bc,0xd867f6be,
	0xdcf1f6be,0xd867f6c0,0xdcf3f6c0,0xd867f6c2,0xdcf3f6c2,0xd867f6c4,0xdcf5f6c4,0xd867f6c6,
	0xdcf5f6c6,0xd867f6c8,0xdcf7f6c8,0xd867f6ca,0xdcf7f6ca,0xd867f6cc,0xdcf9f6cc,0xd867f6ce,
	0xdcf9f6ce,0xd867f6d0,0xdcfbf6d0,0xd867f6d2,0xdcfbf6d2,0xd867f6d4,0xdcfdf6d4,0xd867f6d6,
	0xdcfdf6d6,0xd867f6d8,0xdcfff6d8,0xd867f6da,0xdcfff6da,0xd867f6dc,0xdd01f6dc,0xd867f6de,
	0xdd01f6de,0xd867f6e0,0xdd03f6e0,0xd867f6e2,0xdd03f6e2,0xd867f6e4,0xdd05f6e4,0xd867f6e6,
	0xdd05f6e6,0xd867f6e8,0xdd07f6e8,0xd867f6ea,0xdd07f6ea,0xd867f6ec,0xdd09f6ec,0xd867f6ee,
	0xdd09f6ee,0xd867f6f0,0xdd0bf6f0,0xd867f6f2,0xdd0bf6f2,0xd867f6f4,0xdd0df6f4,0xd867f6f6,
	0xdd0df6f6,0xd867f6f8,0xdd0ff6f8,0xd867f6fa,0xdd0ff6fa,0xd867f6fc,0xdd11f6fc,0xd867f6fe,
	0xdd11f6fe,0xd867f700,0xdd13f700,0xd867f702,0xdd13f702,0xd867f704,0xdd15f704,0xd867f706,
	0xdd15f706,0xd867f708,0xdd17f708,0xd867f70a,0xdd17f70a,0xd867f70c,0xdd19f70c,0xd867f70e,
	0xdd19f70e,0xd867f710,0xdd1bf710,0xd867f712,0xdd1bf712,0xd867f714,0xdd1df714,0xd867f716,
	0xdd1df716,0xd867f718,0xdd1ff718,0xd867f71a,0xdd1ff71a,0xd867f71c,0xdd21f71c,0xd867f71e,
	0xdd21f71e,0xd867f720,0xdd23f720,0xd867f722,0xdd23f722,0xd867f724,0xdd25f724,0xd867f726,
	0xdd25f726,0xd867f728,0xdd27f728,0xd867f72a,0xdd27f72a,0xd867f72c,0xdd29f72c,0xd867f72e,
	0xdd29f72e,0xd867f730,0xdd2bf730,0xd867f732,0xdd2bf732,0xd867f734,0xdd2df734,0xd867f736,
	0xdd2df736,0xd867f738,0xdd2ff738,0xd867f73a,0xdd2ff73a,0xd867f73c,0xdd31f73c,0xd867f73e,
	0xdd31f73e,0xd867f740,0xdd33f740,0xd867f742,0xdd33f742,0xd867f744,0xdd35f744,0xd867f746,
	0xdd35f746,0xd867f748,0xdd37f748,0xd867f74a,0xdd37f74a,0xd867f74c,0xdd39f74c,0xd867f74e,
	0xdd39f74e,0xd867f750,0xdd3bf750,0xd867f752,0xdd3bf752,0xd867f754,0xdd3df754,0xd867f756,
	0xdd3df756,0xd867f758,0xdd3ff758,0xd867f75a,0xdd3ff75a,0xd867f75c,0xdd41f75c,0xd867f75e,
	0xdd41f75e,0xd867f760,0xdd43f760,0xd867f762,0xdd43f762,0xd867f764,0xdd45f764,0xd867f766,
	0xdd45f766,0xd867f768,0xdd47f768,0xd867f76a,0xdd47f76a,0xd867f76c,0xdd49f76c,0xd867f76e,
	0xdd49f76e,0xd867f770,0xdd4bf770,0xd867f772,0xdd4bf772,0xd867f774,0xdd4df774,0xd867f776,
	0xdd4df776,0xd867f778,0xdd4ff778,0xd867f77a,0xdd4ff77a,0xd867f77c,0xdd51f77c,0xd867f77e,
	0xdd51f77e,0xd867f780,0xdd53f780,0xd867f782,0xdd53f782,0xd867f784,0xdd55f784,0xd867f786,
	0xdd55f786,0xd867f788,0xdd57f788,0xd867f78a,0xdd57f78a,0xd867f78c,0xdd59f78c,0xd867f78e,
	0xdd59f78e,0xd867f790,0xdd5bf790,0xd867f792,0xdd5bf792,0xd867f794,0xdd5df794,0xd867f796,
	0xdd5df796,0xd867f798,0xdd5ff798,0xd867f79a,0xdd5ff79a,0xd867f79c,0xdd61f79c,0xd867f79e,
	0xdd61f79e,0xd867f7a0,0xdd63f7a0,0xd867f7a2,0xdd63f7a2,0xd867f7a4,0xdd65f7a4,0xd867f7a6,
	0xdd65f7a6,0xd867f7a8,0xdd67f7a8,0xd867f7aa,0xdd67f7aa,0xd867f7ac,0xdd69f7ac,0xd867f7ae,
	0xdd69f7ae,0xd867f7b0,0xdd6bf7b0,0xd867f7b2,0xdd6bf7b2,0xd867f7b4,0xdd6df7b4,0xd867f7b6,
	0xdd6df7b6,0xd867f7b8,0xdd6ff7b8,0xd867f7ba,0xdd6ff7ba,0xd867f7bc,0xdd71f7bc,0xd867f7be,
	0xdd71f7be,0xd867f7c0,0xdd73f7c0,0xd867f7c2,0xdd73f7c2,0xd867f7c4,0xdd75f7c4,0xd867f7c6,
	0xdd75f7c6,0xd867f7c8,0xdd77f7c8,0xd867f7ca,0xdd77f7ca,0xd867f7cc,0xdd79f7cc,0xd867f7ce,
	0xdd79f7ce,0xd867f7d0,0xdd7bf7d0,0xd867f7d2,0xdd7bf7d2,0xd867f7d4,0xdd7df7d4,0xd867f7d6,
	0xdd7df7d6,0xd867f7d8,0xdd7ff7d8,0xd867f7da,0xdd7ff7da,0xd867f7dc,0xdd81f7dc,0xd867f7de,
	0xdd81f7de,0xd867f7e0,0xdd83f7e0,0xd867f7e2,0xdd83f7e2,0xd867f7e4,0xdd85f7e4,0xd867f7e6,
	0xdd85f7e6,0xd867f7e8,0xdd87f7e8,0xd867f7ea,0xdd87f7ea,0xd867f7ec,0xdd89f7ec,0xd867f7ee,
	0xdd89f7ee,0xd867f7f0,0xdd8bf7f0,0xd867f7f2,0xdd8bf7f2,0xd867f7f4,0xdd8df7f4,0xd867f7f6,
	0xdd8df7f6,0xd867f7f8,0xdd8ff7f8,0xd867f7fa,0xdd8ff7fa,0xd867f7fc,0xdd91f7fc,0xd867f7fe,
	0xdd91f7fe,0xd867f800,0xdd93f800,0xd867f802,0xdd93f802,0xd867f804,0xdd95f804,0xd867f806,
	0xdd95f806,0xd867f808,0xdd97f808,0xd867f80a,0xdd97f80a,0xd867f80c,0xdd99f80c,0xd867f80e,
	0xdd99f80e,0xd867f810,0xdd9bf810,0xd867f812,0xdd9bf812,0xd867f814,0xdd9df814,0xd867f816,
	0xdd9df816,0xd867f818,0xdd9ff818,0xd867f81a,0xdd9ff81a,0xd867f81c,0xdda1f81c,0xd867f81e,
	0xdda1f81e,0xd867f820,0xdda3f820,0xd867f822,0xdda3f822,0xd867f824,0xdda5f824,0xd867f826,
	0xdda5f826,0xd867f828,0xdda7f828,0xd867f82a,0xdda7f82a,0xd867f82c,0xdda9f82c,0xd867f82e,
	0xdda9f82e,0xd867f830,0xddabf830,0xd867f832,0xddabf832,0xd867f834,0xddadf834,0xd867f836,
	0xddadf836,0xd867f838,0xddaff838,0xd867f83a,0xddaff83a,0xd867f83c,0xddb1f83c,0xd867f83e,
	0xddb1f83e,0xd867f840,0xddb3f840,0xd867f842,0xddb3f842,0xd867f844,0xddb5f844,0xd867f846,
	0xddb5f846,0xd867f848,0xddb7f848,0xd867f84a,0xddb7f84a,0xd867f84c,0xddb9f84c,0xd867f84e,
	0xddb9f84e,0xd867f850,0xddbbf850,0xd867f852,0xddbbf852,0xd867f854,0xddbdf854,0xd867f856,
	0xddbdf856,0xd867f858,0xddbff858,0xd867f85a,0xddbff85a,0xd867f85c,0xddc1f85c,0xd867f85e,
	0xddc1f85e,0xd867f860,0xddc3f860,0xd867f862,0xddc3f862,0xd867f864,0xddc5f864,0xd867f866,
	0xddc5f866,0xd867f868,0xddc7f868,0xd867f86a,0xddc7f86a,0xd867f86c,0xddc9f86c,0xd867f86e,
	0xddc9f86e,0xd867f870,0xddcbf870,0xd867f872,0xddcbf872,0xd867f874,0xddcdf874,0xd867f876,
	0xddcdf876,0xd867f878,0xddcff878,0xd867f87a,0xddcff87a,0xd867f87c,0xddd1f87c,0xd867f87e,
	0xddd1f87e,0xd867f880,0xddd3f880,0xd867f882,0xddd3f882,0xd867f884,0xddd5f884,0xd867f886,
	0xddd5f886,0xd867f888,0xddd7f888,0xd867f88a,0xddd7f88a,0xd867f88c,0xddd9f88c,0xd867f88e,
	0xddd9f88e,0xd867f890,0xdddbf890,0xd867f892,0xdddbf892,0xd867f894,0xddddf894,0xd867f896,
	0xddddf896,0xd867f898,0xdddff898,0xd867f89a,0xdddff89a,0xd867f89c,0xdde1f89c,0xd867f89e,
	0xdde1f89e,0xd867f8a0,0xdde3f8a0,0xd867f8a2,0xdde3f8a2,0xd867f8a4,0xdde5f8a4,0xd867f8a6,
	0xdde5f8a6,0xd867f8a8,0xdde7f8a8,0xd867f8aa,0xdde7f8aa,0xd867f8ac,0xdde9f8ac,0xd867f8ae,
	0xdde9f8ae,0xd867f8b0,0xddebf8b0,0xd867f8b2,0xddebf8b2,0xd867f8b4,0xddedf8b4,0xd867f8b6,
	0xddedf8b6,0xd867f8b8,0xddeff8b8,0xd867f8ba,0xddeff8ba,0xd867f8bc,0xddf1f8bc,0xd867f8be,
	0xddf1f8be,0xd867f8c0,0xddf3f8c0,0xd867f8c2,0xddf3f8c2,0xd867f8c4,0xddf5f8c4,0xd867f8c6,
	0xddf5f8c6,0xd867f8c8,0xddf7f8c8,0xd867f8ca,0xddf7f8ca,0xd867f8cc,0xddf9f8cc,0xd867f8ce,
	0xddf9f8ce,0xd867f8d0,0xddfbf8d0,0xd867f8d2,0xddfbf8d2,0xd867f8d4,0xddfdf8d4,0xd867f8d6,
	0xddfdf8d6,0xd867f8d8,0xddfff8d8,0xd867f8da,0xddfff8da,0xd867f8dc,0xde01f8dc,0xd867f8de,
	0xde01f8de,0xd867f8e0,0xde03f8e0,0xd867f8e2,0xde03f8e2,0xd867f8e4,0xde05f8e4,0xd867f8e6,
	0xde05f8e6,0xd867f8e8,0xde07f8e8,0xd867f8ea,0xde07f8ea,0xd867f8ec,0xde09f8ec,0xd867f8ee,
	0xde09f8ee,0xd867f8f0,0xde0bf8f0,0xd867f8f2,0xde0bf8f2,0xd867f8f4,0xde0df8f4,0xd867f8f6,
	0xde0df8f6,0xd867f8f8,0xde0ff8f8,0xd867f8fa,0xde0ff8fa,0xd867f8fc,0xde11f8fc,0xd867f8fe,
	0xde11f8fe,0xd867f900,0xde13f900,0xd867f902,0xde13f902,0xd867f904,0xde15f904,0xd867f906,
	0xde15f906,0xd867f908,0xde17f908,0xd867f90a,0xde17f90a,0xd867f90c,0xde19f90c,0xd867f90e,
	0xde19f90e,0xd867f910,0xde1bf910,0xd867f912,0xde1bf912,0xd867f914,0xde1df914,0xd867f916,
	0xde1df916,0xd867f918,0xde1ff918,0xd867f91a,0xde1ff91a,0xd867f91c,0xde21f91c,0xd867f91e,
	0xde21f91e,0xd867f920,0xde23f920,0xd867f922,0xde23f922,0xd867f924,0xde25f924,0xd867f926,
	0xde25f926,0xd867f928,0xde27f928,0xd867f92a,0xde27f92a,0xd867f92c,0xde29f92c,0xd867f92e,
	0xde29f92e,0xd867f930,0xde2bf930,0xd867f932,0xde2bf932,0xd867f934,0xde2df934,0xd867f936,
	0xde2df936,0xd867f938,0xde2ff938,0xd867f93a,0xde2ff93a,0xd867f93c,0xde31f93c,0xd867f93e,
	0xde31f93e,0xd867f940,0xde33f940,0xd867f942,0xde33f942,0xd867f944,0xde35f944,0xd867f946,
	0xde35f946,0xd867f948,0xde37f948,0xd867f94a,0xde37f94a,0xd867f94c,0xde39f94c,0xd867f94e,
	0xde39f94e,0xd867f950,0xde3bf950,0xd867f952,0xde3bf952,0xd867f954,0xde3df954,0xd867f956,
	0xde3df956,0xd867f958,0xde3ff958,0xd867f95a,0xde3ff95a,0xd867f95c,0xde41f95c,0xd867f95e,
	0xde41f95e,0xd867f960,0xde43f960,0xd867f962,0xde43f962,0xd867f964,0xde45f964,0xd867f966,
	0xde45f966,0xd867f968,0xde47f968,0xd867f96a,0xde47f96a,0xd867f96c,0xde49f96c,0xd867f96e,
	0xde49f96e,0xd867f970,0xde4bf970,0xd867f972,0xde4bf972,0xd867f974,0xde4df974,0xd867f976,
	0xde4df976,0xd867f978,0xde4ff978,0xd867f97a,0xde4ff97a,0xd867f97c,0xde51f97c,0xd867f97e,
	0xde51f97e,0xd867f980,0xde53f980,0xd867f982,0xde53f982,0xd867f984,0xde55f984,0xd867f986,
	0xde55f986,0xd867f988,0xde57f988,0xd867f98a,0xde57f98a,0xd867f98c,0xde59f98c,0xd867f98e,
	0xde59f98e,0xd867f990,0xde5bf990,0xd867f992,0xde5bf992,0xd867f994,0xde5df994,0xd867f996,
	0xde5df996,0xd867f998,0xde5ff998,0xd867f99a,0xde5ff99a,0xd867f99c,0xde61f99c,0xd867f99e,
	0xde61f99e,0xd867f9a0,0xde63f9a0,0xd867f9a2,0xde63f9a2,0xd867f9a4,0xde65f9a4,0xd867f9a6,
	0xde65f9a6,0xd867f9a8,0xde67f9a8,0xd867f9aa,0xde67f9aa,0xd867f9ac,0xde69f9ac,0xd867f9ae,
	0xde69f9ae,0xd867f9b0,0xde6bf9b0,0xd867f9b2,0xde6bf9b2,0xd867f9b4,0xde6df9b4,0xd867f9b6,
	0xde6df9b6,0xd867f9b8,0xde6ff9b8,0xd867f9ba,0xde6ff9ba,0xd867f9bc,0xde71f9bc,0xd867f9be,
	0xde71f9be,0xd867f9c0,0xde73f9c0,0xd867f9c2,0xde73f9c2,0xd867f9c4,0xde75f9c4,0xd867f9c6,
	0xde75f9c6,0xd867f9c8,0xde77f9c8,0xd867f9ca,0xde77f9ca,0xd867f9cc,0xde79f9cc,0xd867f9ce,
	0xde79f9ce,0xd867f9d0,0xde7bf9d0,0xd867f9d2,0xde7bf9d2,0xd867f9d4,0xde7df9d4,0xd867f9d6,
	0xde7df9d6,0xd867f9d8,0xde7ff9d8,0xd867f9da,0xde7ff9da,0xd867f9dc,0xde81f9dc,0xd867f9de,
	0xde81f9de,0xd867f9e0,0xde83f9e0,0xd867f9e2,0xde83f9e2,0xd867f9e4,0xde85f9e4,0xd867f9e6,
	0xde85f9e6,0xd867f9e8,0xde87f9e8,0xd867f9ea,0xde87f9ea,0xd867f9ec,0xde89f9ec,0xd867f9ee,
	0xde89f9ee,0xd867f9f0,0xde8bf9f0,0xd867f9f2,0xde8bf9f2,0xd867f9f4,0xde8df9f4,0xd867f9f6,
	0xde8df9f6,0xd867f9f8,0xde8ff9f8,0xd867f9fa,0xde8ff9fa,0xd867f9fc,0xde91f9fc,0xd867f9fe,
	0xde91f9fe,0xd867fa00,0xde93fa00,0xd867fa02,0xde93fa02,0xd867fa04,0xde95fa04,0xd867fa06,
	0xde95fa06,0xd867fa08,0xde97fa08,0xd867fa0a,0xde97fa0a,0xd867fa0c,0xde99fa0c,0xd867fa0e,
	0xde99fa0e,0xd867fa10,0xde9bfa10,0xd867fa12,0xde9bfa12,0xd867fa14,0xde9dfa14,0xd867fa16,
	0xde9dfa16,0xd867fa18,0xde9ffa18,0xd867fa1a,0xde9ffa1a,0xd867fa1c,0xdea1fa1c,0xd867fa1e,
	0xdea1fa1e,0xd867fa20,0xdea3fa20,0xd867fa22,0xdea3fa22,0xd867fa24,0xdea5fa24,0xd867fa26,
	0xdea5fa26,0xd867fa28,0xdea7fa28,0xd867fa2a,0xdea7fa2a,0xd867fa2c,0xdea9fa2c,0xd867fa2e,
	0xdea9fa2e,0xd867fa30,0xdeabfa30,0xd867fa32,0xdeabfa32,0xd867fa34,0xdeadfa34,0xd867fa36,
	0xdeadfa36,0xd867fa38,0xdeaffa38,0xd867fa3a,0xdeaffa3a,0xd867fa3c,0xdeb1fa3c,0xd867fa3e,
	0xdeb1fa3e,0xd867fa40,0xdeb3fa40,0xd867fa42,0xdeb3fa42,0xd867fa44,0xdeb5fa44,0xd867fa46,
	0xdeb5fa46,0xd867fa48,0xdeb7fa48,0xd867fa4a,0xdeb7fa4a,0xd867fa4c,0xdeb9fa4c,0xd867fa4e,
	0xdeb9fa4e,0xd867fa50,0xdebbfa50,0xd867fa52,0xdebbfa52,0xd867fa54,0xdebdfa54,0xd867fa56,
	0xdebdfa56,0xd867fa58,0xdebffa58,0xd867fa5a,0xdebffa5a,0xd867fa5c,0xdec1fa5c,0xd867fa5e,
	0xdec1fa5e,0xd867fa60,0xdec3fa60,0xd867fa62,0xdec3fa62,0xd867fa64,0xdec5fa64,0xd867fa66,
	0xdec5fa66,0xd867fa68,0xdec7fa68,0xd867fa6a,0xdec7fa6a,0xd867fa6c,0xdec9fa6c,0xd867fa6e,
	0xdec9fa6e,0xd867fa70,0xdecbfa70,0xd867fa72,0xdecbfa72,0xd867fa74,0xdecdfa74,0xd867fa76,
	0xdecdfa76,0xd867fa78,0xdecffa78,0xd867fa7a,0xdecffa7a,0xd867fa7c,0xded1fa7c,0xd867fa7e,
	0xded1fa7e,0xd867fa80,0xded3fa80,0xd867fa82,0xded3fa82,0xd867fa84,0xded5fa84,0xd867fa86,
	0xded5fa86,0xd867fa88,0xded7fa88,0xd867fa8a,0xded7fa8a,0xd867fa8c,0xded9fa8c,0xd867fa8e,
	0xded9fa8e,0xd867fa90,0xdedbfa90,0xd867fa92,0xdedbfa92,0xd867fa94,0xdeddfa94,0xd867fa96,
	0xdeddfa96,0xd867fa98,0xdedffa98,0xd867fa9a,0xdedffa9a,0xd867fa9c,0xdee1fa9c,0xd867fa9e,
	0xdee1fa9e,0xd867faa0,0xdee3faa0,0xd867faa2,0xdee3faa2,0xd867faa4,0xdee5faa4,0xd867faa6,
	0xdee5faa6,0xd867faa8,0xdee7faa8,0xd867faaa,0xdee7faaa,0xd867faac,0xdee9faac,0xd867faae,
	0xdee9faae,0xd867fab0,0xdeebfab0,0xd867fab2,0xdeebfab2,0xd867fab4,0xdeedfab4,0xd867fab6,
	0xdeedfab6,0xd867fab8,0xdeeffab8,0xd867faba,0xdeeffaba,0xd867fabc,0xdef1fabc,0xd867fabe,
	0xdef1fabe,0xd867fac0,0xdef3fac0,0xd867fac2,0xdef3fac2,0xd867fac4,0xdef5fac4,0xd867fac6,
	0xdef5fac6,0xd867fac8,0xdef7fac8,0xd867faca,0xdef7faca,0xd867facc,0xdef9facc,0xd867face,
	0xdef9face,0xd867fad0,0xdefbfad0,0xd867fad2,0xdefbfad2,0xd867fad4,0xdefdfad4,0xd867fad6,
	0xdefdfad6,0xd867fad8,0xdefffad8,0xd867fada,0xdefffada,0xd867fadc,0xdf01fadc,0xd867fade,
	0xdf01fade,0xd867fae0,0xdf03fae0,0xd867fae2,0xdf03fae2,0xd867fae4,0xdf05fae4,0xd867fae6,
	0xdf05fae6,0xd867fae8,0xdf07fae8,0xd867faea,0xdf07faea,0xd867faec,0xdf09faec,0xd867faee,
	0xdf09faee,0xd867faf0,0xdf0bfaf0,0xd867faf2,0xdf0bfaf2,0xd867faf4,0xdf0dfaf4,0xd867faf6,
	0xdf0dfaf6,0xd867faf8,0xdf0ffaf8,0xd867fafa,0xdf0ffafa,0xd867fafc,0xdf11fafc,0xd867fafe,
	0xdf11fafe,0xd867fb00,0xdf13fb00,0xd867fb02,0xdf13fb02,0xd867fb04,0xdf15fb04,0xd867fb06,
	0xdf15fb06,0xd867fb08,0xdf17fb08,0xd867fb0a,0xdf17fb0a,0xd867fb0c,0xdf19fb0c,0xd867fb0e,
	0xdf19fb0e,0xd867fb10,0xdf1bfb10,0xd867fb12,0xdf1bfb12,0xd867fb14,0xdf1dfb14,0xd867fb16,
	0xdf1dfb16,0xd867fb18,0xdf1ffb18,0xd867fb1a,0xdf1ffb1a,0xd867fb1c,0xdf21fb1c,0xd867fb1e,
	0xdf21fb1e,0xd867fb20,0xdf23fb20,0xd867fb22,0xdf23fb22,0xd867fb24,0xdf25fb24,0xd867fb26,
	0xdf25fb26,0xd867fb28,0xdf27fb28,0xd867fb2a,0xdf27fb2a,0xd867fb2c,0xdf29fb2c,0xd867fb2e,
	0xdf29fb2e,0xd867fb30,0xdf2bfb30,0xd867fb32,0xdf2bfb32,0xd867fb34,0xdf2dfb34,0xd867fb36,
	0xdf2dfb36,0xd867fb38,0xdf2ffb38,0xd867fb3a,0xdf2ffb3a,0xd867fb3c,0xdf31fb3c,0xd867fb3e,
	0xdf31fb3e,0xd867fb40,0xdf33fb40,0xd867fb42,0xdf33fb42,0xd867fb44,0xdf35fb44,0xd867fb46,
	0xdf35fb46,0xd867fb48,0xdf37fb48,0xd867fb4a,0xdf37fb4a,0xd867fb4c,0xdf39fb4c,0xd867fb4e,
	0xdf39fb4e,0xd867fb50,0xdf3bfb50,0xd867fb52,0xdf3bfb52,0xd867fb54,0xdf3dfb54,0xd867fb56,
	0xdf3dfb56,0xd867fb58,0xdf3ffb58,0xd867fb5a,0xdf3ffb5a,0xd867fb5c,0xdf41fb5c,0xd867fb5e,
	0xdf41fb5e,0xd867fb60,0xdf43fb60,0xd867fb62,0xdf43fb62,0xd867fb64,0xdf45fb64,0xd867fb66,
	0xdf45fb66,0xd867fb68,0xdf47fb68,0xd867fb6a,0xdf47fb6a,0xd867fb6c,0xdf49fb6c,0xd867fb6e,
	0xdf49fb6e,0xd867fb70,0xdf4bfb70,0xd867fb72,0xdf4bfb72,0xd867fb74,0xdf4dfb74,0xd867fb76,
	0xdf4dfb76,0xd867fb78,0xdf4ffb78,0xd867fb7a,0xdf4ffb7a,0xd867fb7c,0xdf51fb7c,0xd867fb7e,
	0xdf51fb7e,0xd867fb80,0xdf53fb80,0xd867fb82,0xdf53fb82,0xd867fb84,0xdf55fb84,0xd867fb86,
	0xdf55fb86,0xd867fb88,0xdf57fb88,0xd867fb8a,0xdf57fb8a,0xd867fb8c,0xdf59fb8c,0xd867fb8e,
	0xdf59fb8e,0xd867fb90,0xdf5bfb90,0xd867fb92,0xdf5bfb92,0xd867fb94,0xdf5dfb94,0xd867fb96,
	0xdf5dfb96,0xd867fb98,0xdf5ffb98,0xd867fb9a,0xdf5ffb9a,0xd867fb9c,0xdf61fb9c,0xd867fb9e,
	0xdf61fb9e,0xd867fba0,0xdf63fba0,0xd867fba2,0xdf63fba2,0xd867fba4,0xdf65fba4,0xd867fba6,
	0xdf65fba6,0xd867fba8,0xdf67fba8,0xd867fbaa,0xdf67fbaa,0xd867fbac,0xdf69fbac,0xd867fbae,
	0xdf69fbae,0xd867fbb0,0xdf6bfbb0,0xd867fbb2,0xdf6bfbb2,0xd867fbb4,0xdf6dfbb4,0xd867fbb6,
	0xdf6dfbb6,0xd867fbb8,0xdf6ffbb8,0xd867fbba,0xdf6ffbba,0xd867fbbc,0xdf71fbbc,0xd867fbbe,
	0xdf71fbbe,0xd867fbc0,0xdf73fbc0,0xd867fbc2,0xdf73fbc2,0xd867fbc4,0xdf75fbc4,0xd867fbc6,
	0xdf75fbc6,0xd867fbc8,0xdf77fbc8,0xd867fbca,0xdf77fbca,0xd867fbcc,0xdf79fbcc,0xd867fbce,
	0xdf79fbce,0xd867fbd0,0xdf7bfbd0,0xd867fbd2,0xdf7bfbd2,0xd867fbd4,0xdf7dfbd4,0xd867fbd6,
	0xdf7dfbd6,0xd867fbd8,0xdf7ffbd8,0xd867fbda,0xdf7ffbda,0xd867fbdc,0xdf81fbdc,0xd867fbde,
	0xdf81fbde,0xd867fbe0,0xdf83fbe0,0xd867fbe2,0xdf83fbe2,0xd867fbe4,0xdf85fbe4,0xd867fbe6,
	0xdf85fbe6,0xd867fbe8,0xdf87fbe8,0xd867fbea,0xdf87fbea,0xd867fbec,0xdf89fbec,0xd867fbee,
	0xdf89fbee,0xd867fbf0,0xdf8bfbf0,0xd867fbf2,0xdf8bfbf2,0xd867fbf4,0xdf8dfbf4,0xd867fbf6,
	0xdf8dfbf6,0xd867fbf8,0xdf8ffbf8,0xd867fbfa,0xdf8ffbfa,0xd867fbfc,0xdf91fbfc,0xd867fbfe,
	0xdf91fbfe,0xd867fc00,0xdf93fc00,0xd867fc02,0xdf93fc02,0xd867fc04,0xdf95fc04,0xd867fc06,
	0xdf95fc06,0xd867fc08,0xdf97fc08,0xd867fc0a,0xdf97fc0a,0xd867fc0c,0xdf99fc0c,0xd867fc0e,
	0xdf99fc0e,0xd867fc10,0xdf9bfc10,0xd867fc12,0xdf9bfc12,0xd867fc14,0xdf9dfc14,0xd867fc16,
	0xdf9dfc16,0xd867fc18,0xdf9ffc18,0xd867fc1a,0xdf9ffc1a,0xd867fc1c,0xdfa1fc1c,0xd867fc1e,
	0xdfa1fc1e,0xd867fc20,0xdfa3fc20,0xd867fc22,0xdfa3fc22,0xd867fc24,0xdfa5fc24,0xd867fc26,
	0xdfa5fc26,0xd867fc28,0xdfa7fc28,0xd867fc2a,0xdfa7fc2a,0xd867fc2c,0xdfa9fc2c,0xd867fc2e,
	0xdfa9fc2e,0xd867fc30,0xdfabfc30,0xd867fc32,0xdfabfc32,0xd867fc34,0xdfadfc34,0xd867fc36,
	0xdfadfc36,0xd867fc38,0xdfaffc38,0xd867fc3a,0xdfaffc3a,0xd867fc3c,0xdfb1fc3c,0xd867fc3e,
	0xdfb1fc3e,0xd867fc40,0xdfb3fc40,0xd867fc42,0xdfb3fc42,0xd867fc44,0xdfb5fc44,0xd867fc46,
	0xdfb5fc46,0xd867fc48,0xdfb7fc48,0xd867fc4a,0xdfb7fc4a,0xd867fc4c,0xdfb9fc4c,0xd867fc4e,
	0xdfb9fc4e,0xd867fc50,0xdfbbfc50,0xd867fc52,0xdfbbfc52,0xd867fc54,0xdfbdfc54,0xd867fc56,
	0xdfbdfc56,0xd867fc58,0xdfbffc58,0xd867fc5a,0xdfbffc5a,0xd867fc5c,0xdfc1fc5c,0xd867fc5e,
	0xdfc1fc5e,0xd867fc60,0xdfc3fc60,0xd867fc62,0xdfc3fc62,0xd867fc64,0xdfc5fc64,0xd867fc66,
	0xdfc5fc66,0xd867fc68,0xdfc7fc68,0xd867fc6a,0xdfc7fc6a,0xd867fc6c,0xdfc9fc6c,0xd867fc6e,
	0xdfc9fc6e,0xd867fc70,0xdfcbfc70,0xd867fc72,0xdfcbfc72,0xd867fc74,0xdfcdfc74,0xd867fc76,
	0xdfcdfc76,0xd867fc78,0xdfcffc78,0xd867fc7a,0xdfcffc7a,0xd867fc7c,0xdfd1fc7c,0xd867fc7e,
	0xdfd1fc7e,0xd867fc80,0xdfd3fc80,0xd867fc82,0xdfd3fc82,0xd867fc84,0xdfd5fc84,0xd867fc86,
	0xdfd5fc86,0xd867fc88,0xdfd7fc88,0xd867fc8a,0xdfd7fc8a,0xd867fc8c,0xdfd9fc8c,0xd867fc8e,
	0xdfd9fc8e,0xd867fc90,0xdfdbfc90,0xd867fc92,0xdfdbfc92,0xd867fc94,0xdfddfc94,0xd867fc96,
	0xdfddfc96,0xd867fc98,0xdfdffc98,0xd867fc9a,0xdfdffc9a,0xd867fc9c,0xdfe1fc9c,0xd867fc9e,
	0xdfe1fc9e,0xd867fca0,0xdfe3fca0,0xd867fca2,0xdfe3fca2,0xd867fca4,0xdfe5fca4,0xd867fca6,
	0xdfe5fca6,0xd867fca8,0xdfe7fca8,0xd867fcaa,0xdfe7fcaa,0xd867fcac,0xdfe9fcac,0xd867fcae,
	0xdfe9fcae,0xd867fcb0,0xdfebfcb0,0xd867fcb2,0xdfebfcb2,0xd867fcb4,0xdfedfcb4,0xd867fcb6,
	0xdfedfcb6,0xd867fcb8,0xdfeffcb8,0xd867fcba,0xdfeffcba,0xd867fcbc,0xdff1fcbc,0xd867fcbe,
	0xdff1fcbe,0xd867fcc0,0xdff3fcc0,0xd867fcc2,0xdff3fcc2,0xd867fcc4,0xdff5fcc4,0xd867fcc6,
	0xdff5fcc6,0xd867fcc8,0xdff7fcc8,0xd867fcca,0xdff7fcca,0xd867fccc,0xdff9fccc,0xd867fcce,
	0xdff9fcce,0xd867fcd0,0xdffbfcd0,0xd867fcd2,0xdffbfcd2,0xd867fcd4,0xdffdfcd4,0xd867fcd6,
	0xdffdfcd6,0xd867fcd8,0xdffffcd8,0xd867fcda,0xdffffcda,0xd869fcdc,0xdc01fcdc,0xd869fcde,
	0xdc01fcde,0xd869fce0,0xdc03fce0,0xd869fce2,0xdc03fce2,0xd869fce4,0xdc05fce4,0xd869fce6,
	0xdc05fce6,0xd869fce8,0xdc07fce8,0xd869fcea,0xdc07fcea,0xd869fcec,0xdc09fcec,0xd869fcee,
	0xdc09fcee,0xd869fcf0,0xdc0bfcf0,0xd869fcf2,0xdc0bfcf2,0xd869fcf4,0xdc0dfcf4,0xd869fcf6,
	0xdc0dfcf6,0xd869fcf8,0xdc0ffcf8,0xd869fcfa,0xdc0ffcfa,0xd869fcfc,0xdc11fcfc,0xd869fcfe,
	0xdc11fcfe,0xd869fd00,0xdc13fd00,0xd869fd02,0xdc13fd02,0xd869fd04,0xdc15fd04,0xd869fd06,
	0xdc15fd06,0xd869fd08,0xdc17fd08,0xd869fd0a,0xdc17fd0a,0xd869fd0c,0xdc19fd0c,0xd869fd0e,
	0xdc19fd0e,0xd869fd10,0xdc1bfd10,0xd869fd12,0xdc1bfd12,0xd869fd14,0xdc1dfd14,0xd869fd16,
	0xdc1dfd16,0xd869fd18,0xdc1ffd18,0xd869fd1a,0xdc1ffd1a,0xd869fd1c,0xdc21fd1c,0xd869fd1e,
	0xdc21fd1e,0xd869fd20,0xdc23fd20,0xd869fd22,0xdc23fd22,0xd869fd24,0xdc25fd24,0xd869fd26,
	0xdc25fd26,0xd869fd28,0xdc27fd28,0xd869fd2a,0xdc27fd2a,0xd869fd2c,0xdc29fd2c,0xd869fd2e,
	0xdc29fd2e,0xd869fd30,0xdc2bfd30,0xd869fd32,0xdc2bfd32,0xd869fd34,0xdc2dfd34,0xd869fd36,
	0xdc2dfd36,0xd869fd38,0xdc2ffd38,0xd869fd3a,0xdc2ffd3a,0xd869fd3c,0xdc31fd3c,0xd869fd3e,
	0xdc31fd3e,0xd869fd40,0xdc33fd40,0xd869fd42,0xdc33fd42,0xd869fd44,0xdc35fd44,0xd869fd46,
	0xdc35fd46,0xd869fd48,0xdc37fd48,0xd869fd4a,0xdc37fd4a,0xd869fd4c,0xdc39fd4c,0xd869fd4e,
	0xdc39fd4e,0xd869fd50,0xdc3bfd50,0xd869fd52,0xdc3bfd52,0xd869fd54,0xdc3dfd54,0xd869fd56,
	0xdc3dfd56,0xd869fd58,0xdc3ffd58,0xd869fd5a,0xdc3ffd5a,0xd869fd5c,0xdc41fd5c,0xd869fd5e,
	0xdc41fd5e,0xd869fd60,0xdc43fd60,0xd869fd62,0xdc43fd62,0xd869fd64,0xdc45fd64,0xd869fd66,
	0xdc45fd66,0xd869fd68,0xdc47fd68,0xd869fd6a,0xdc47fd6a,0xd869fd6c,0xdc49fd6c,0xd869fd6e,
	0xdc49fd6e,0xd869fd70,0xdc4bfd70,0xd869fd72,0xdc4bfd72,0xd869fd74,0xdc4dfd74,0xd869fd76,
	0xdc4dfd76,0xd869fd78,0xdc4ffd78,0xd869fd7a,0xdc4ffd7a,0xd869fd7c,0xdc51fd7c,0xd869fd7e,
	0xdc51fd7e,0xd869fd80,0xdc53fd80,0xd869fd82,0xdc53fd82,0xd869fd84,0xdc55fd84,0xd869fd86,
	0xdc55fd86,0xd869fd88,0xdc57fd88,0xd869fd8a,0xdc57fd8a,0xd869fd8c,0xdc59fd8c,0xd869fd8e,
	0xdc59fd8e,0xd869fd90,0xdc5bfd90,0xd869fd92,0xdc5bfd92,0xd869fd94,0xdc5dfd94,0xd869fd96,
	0xdc5dfd96,0xd869fd98,0xdc5ffd98,0xd869fd9a,0xdc5ffd9a,0xd869fd9c,0xdc61fd9c,0xd869fd9e,
	0xdc61fd9e,0xd869fda0,0xdc63fda0,0xd869fda2,0xdc63fda2,0xd869fda4,0xdc65fda4,0xd869fda6,
	0xdc65fda6,0xd869fda8,0xdc67fda8,0xd869fdaa,0xdc67fdaa,0xd869fdac,0xdc69fdac,0xd869fdae,
	0xdc69fdae,0xd869fdb0,0xdc6bfdb0,0xd869fdb2,0xdc6bfdb2,0xd869fdb4,0xdc6dfdb4,0xd869fdb6,
	0xdc6dfdb6,0xd869fdb8,0xdc6ffdb8,0xd869fdba,0xdc6ffdba,0xd869fdbc,0xdc71fdbc,0xd869fdbe,
	0xdc71fdbe,0xd869fdc0,0xdc73fdc0,0xd869fdc2,0xdc73fdc2,0xd869fdc4,0xdc75fdc4,0xd869fdc6,
	0xdc75fdc6,0xd869fdc8,0xdc77fdc8,0xd869fdca,0xdc77fdca,0xd869fdcc,0xdc79fdcc,0xd869fdce,
	0xdc79fdce,0xd869fdd0,0xdc7bfdd0,0xd869fdd2,0xdc7bfdd2,0xd869fdd4,0xdc7dfdd4,0xd869fdd6,
	0xdc7dfdd6,0xd869fdd8,0xdc7ffdd8,0xd869fdda,0xdc7ffdda,0xd869fddc,0xdc81fddc,0xd869fdde,
	0xdc81fdde,0xd869fde0,0xdc83fde0,0xd869fde2,0xdc83fde2,0xd869fde4,0xdc85fde4,0xd869fde6,
	0xdc85fde6,0xd869fde8,0xdc87fde8,0xd869fdea,0xdc87fdea,0xd869fdec,0xdc89fdec,0xd869fdee,
	0xdc89fdee,0xd869fdf0,0xdc8bfdf0,0xd869fdf2,0xdc8bfdf2,0xd869fdf4,0xdc8dfdf4,0xd869fdf6,
	0xdc8dfdf6,0xd869fdf8,0xdc8ffdf8,0xd869fdfa,0xdc8ffdfa,0xd869fdfc,0xdc91fdfc,0xd869fdfe,
	0xdc91fdfe,0xd869fe00,0xdc93fe00,0xd869fe02,0xdc93fe02,0xd869fe04,0xdc95fe04,0xd869fe06,
	0xdc95fe06,0xd869fe08,0xdc97fe08,0xd869fe0a,0xdc97fe0a,0xd869fe0c,0xdc99fe0c,0xd869fe0e,
	0xdc99fe0e,0xd869fe10,0xdc9bfe10,0xd869fe12,0xdc9bfe12,0xd869fe14,0xdc9dfe14,0xd869fe16,
	0xdc9dfe16,0xd869fe18,0xdc9ffe18,0xd869fe1a,0xdc9ffe1a,0xd869fe1c,0xdca1fe1c,0xd869fe1e,
	0xdca1fe1e,0xd869fe20,0xdca3fe20,0xd869fe22,0xdca3fe22,0xd869fe24,0xdca5fe24,0xd869fe26,
	0xdca5fe26,0xd869fe28,0xdca7fe28,0xd869fe2a,0xdca7fe2a,0xd869fe2c,0xdca9fe2c,0xd869fe2e,
	0xdca9fe2e,0xd869fe30,0xdcabfe30,0xd869fe32,0xdcabfe32,0xd869fe34,0xdcadfe34,0xd869fe36,
	0xdcadfe36,0xd869fe38,0xdcaffe38,0xd869fe3a,0xdcaffe3a,0xd869fe3c,0xdcb1fe3c,0xd869fe3e,
	0xdcb1fe3e,0xd869fe40,0xdcb3fe40,0xd869fe42,0xdcb3fe42,0xd869fe44,0xdcb5fe44,0xd869fe46,
	0xdcb5fe46,0xd869fe48,0xdcb7fe48,0xd869fe4a,0xdcb7fe4a,0xd869fe4c,0xdcb9fe4c,0xd869fe4e,
	0xdcb9fe4e,0xd869fe50,0xdcbbfe50,0xd869fe52,0xdcbbfe52,0xd869fe54,0xdcbdfe54,0xd869fe56,
	0xdcbdfe56,0xd869fe58,0xdcbffe58,0xd869fe5a,0xdcbffe5a,0xd869fe5c,0xdcc1fe5c,0xd869fe5e,
	0xdcc1fe5e,0xd869fe60,0xdcc3fe60,0xd869fe62,0xdcc3fe62,0xd869fe64,0xdcc5fe64,0xd869fe66,
	0xdcc5fe66,0xd869fe68,0xdcc7fe68,0xd869fe6a,0xdcc7fe6a,0xd869fe6c,0xdcc9fe6c,0xd869fe6e,
	0xdcc9fe6e,0xd869fe70,0xdccbfe70,0xd869fe72,0xdccbfe72,0xd869fe74,0xdccdfe74,0xd869fe76,
	0xdccdfe76,0xd869fe78,0xdccffe78,0xd869fe7a,0xdccffe7a,0xd869fe7c,0xdcd1fe7c,0xd869fe7e,
	0xdcd1fe7e,0xd869fe80,0xdcd3fe80,0xd869fe82,0xdcd3fe82,0xd869fe84,0xdcd5fe84,0xd869fe86,
	0xdcd5fe86,0xd869fe88,0xdcd7fe88,0xd869fe8a,0xdcd7fe8a,0xd869fe8c,0xdcd9fe8c,0xd869fe8e,
	0xdcd9fe8e,0xd869fe90,0xdcdbfe90,0xd869fe92,0xdcdbfe92,0xd869fe94,0xdcddfe94,0xd869fe96,
	0xdcddfe96,0xd869fe98,0xdcdffe98,0xd869fe9a,0xdcdffe9a,0xd869fe9c,0xdce1fe9c,0xd869fe9e,
	0xdce1fe9e,0xd869fea0,0xdce3fea0,0xd869fea2,0xdce3fea2,0xd869fea4,0xdce5fea4,0xd869fea6,
	0xdce5fea6,0xd869fea8,0xdce7fea8,0xd869feaa,0xdce7feaa,0xd869feac,0xdce9feac,0xd869feae,
	0xdce9feae,0xd869feb0,0xdcebfeb0,0xd869feb2,0xdcebfeb2,0xd869feb4,0xdcedfeb4,0xd869feb6,
	0xdcedfeb6,0xd869feb8,0xdceffeb8,0xd869feba,0xdceffeba,0xd869febc,0xdcf1febc,0xd869febe,
	0xdcf1febe,0xd869fec0,0xdcf3fec0,0xd869fec2,0xdcf3fec2,0xd869fec4,0xdcf5fec4,0xd869fec6,
	0xdcf5fec6,0xd869fec8,0xdcf7fec8,0xd869feca,0xdcf7feca,0xd869fecc,0xdcf9fecc,0xd869fece,
	0xdcf9fece,0xd869fed0,0xdcfbfed0,0xd869fed2,0xdcfbfed2,0xd869fed4,0xdcfdfed4,0xd869fed6,
	0xdcfdfed6,0xd869fed8,0xdcfffed8,0xd869feda,0xdcfffeda,0xd869fedc,0xdd01fedc,0xd869fede,
	0xdd01fede,0xd869fee0,0xdd03fee0,0xd869fee2,0xdd03fee2,0xd869fee4,0xdd05fee4,0xd869fee6,
	0xdd05fee6,0xd869fee8,0xdd07fee8,0xd869feea,0xdd07feea,0xd869feec,0xdd09feec,0xd869feee,
	0xdd09feee,0xd869fef0,0xdd0bfef0,0xd869fef2,0xdd0bfef2,0xd869fef4,0xdd0dfef4,0xd869fef6,
	0xdd0dfef6,0xd869fef8,0xdd0ffef8,0xd869fefa,0xdd0ffefa,0xd869fefc,0xdd11fefc,0xd869fefe,
	0xdd11fefe,0xd869ff00,0xdd13ff00,0xd869ff02,0xdd13ff02,0xd869ff04,0xdd15ff04,0xd869ff06,
	0xdd15ff06,0xd869ff08,0xdd17ff08,0xd869ff0a,0xdd17ff0a,0xd869ff0c,0xdd19ff0c,0xd869ff0e,
	0xdd19ff0e,0xd869ff10,0xdd1bff10,0xd869ff12,0xdd1bff12,0xd869ff14,0xdd1dff14,0xd869ff16,
	0xdd1dff16,0xd869ff18,0xdd1fff18,0xd869ff1a,0xdd1fff1a,0xd869ff1c,0xdd21ff1c,0xd869ff1e,
	0xdd21ff1e,0xd869ff20,0xdd23ff20,0xd869ff22,0xdd23ff22,0xd869ff24,0xdd25ff24,0xd869ff26,
	0xdd25ff26,0xd869ff28,0xdd27ff28,0xd869ff2a,0xdd27ff2a,0xd869ff2c,0xdd29ff2c,0xd869ff2e,
	0xdd29ff2e,0xd869ff30,0xdd2bff30,0xd869ff32,0xdd2bff32,0xd869ff34,0xdd2dff34,0xd869ff36,
	0xdd2dff36,0xd869ff38,0xdd2fff38,0xd869ff3a,0xdd2fff3a,0xd869ff3c,0xdd31ff3c,0xd869ff3e,
	0xdd31ff3e,0xd869ff40,0xdd33ff40,0xd869ff42,0xdd33ff42,0xd869ff44,0xdd35ff44,0xd869ff46,
	0xdd35ff46,0xd869ff48,0xdd37ff48,0xd869ff4a,0xdd37ff4a,0xd869ff4c,0xdd39ff4c,0xd869ff4e,
	0xdd39ff4e,0xd869ff50,0xdd3bff50,0xd869ff52,0xdd3bff52,0xd869ff54,0xdd3dff54,0xd869ff56,
	0xdd3dff56,0xd869ff58,0xdd3fff58,0xd869ff5a,0xdd3fff5a,0xd869ff5c,0xdd41ff5c,0xd869ff5e,
	0xdd41ff5e,0xd869ff60,0xdd43ff60,0xd869ff62,0xdd43ff62,0xd869ff64,0xdd45ff64,0xd869ff66,
	0xdd45ff66,0xd869ff68,0xdd47ff68,0xd869ff6a,0xdd47ff6a,0xd869ff6c,0xdd49ff6c,0xd869ff6e,
	0xdd49ff6e,0xd869ff70,0xdd4bff70,0xd869ff72,0xdd4bff72,0xd869ff74,0xdd4dff74,0xd869ff76,
	0xdd4dff76,0xd869ff78,0xdd4fff78,0xd869ff7a,0xdd4fff7a,0xd869ff7c,0xdd51ff7c,0xd869ff7e,
	0xdd51ff7e,0xd869ff80,0xdd53ff80,0xd869ff82,0xdd53ff82,0xd869ff84,0xdd55ff84,0xd869ff86,
	0xdd55ff86,0xd869ff88,0xdd57ff88,0xd869ff8a,0xdd57ff8a,0xd869ff8c,0xdd59ff8c,0xd869ff8e,
	0xdd59ff8e,0xd869ff90,0xdd5bff90,0xd869ff92,0xdd5bff92,0xd869ff94,0xdd5dff94,0xd869ff96,
	0xdd5dff96,0xd869ff98,0xdd5fff98,0xd869ff9a,0xdd5fff9a,0xd869ff9c,0xdd61ff9c,0xd869ff9e,
	0xdd61ff9e,0xd869ffa0,0xdd63ffa0,0xd869ffa2,0xdd63ffa2,0xd869ffa4,0xdd65ffa4,0xd869ffa6,
	0xdd65ffa6,0xd869ffa8,0xdd67ffa8,0xd869ffaa,0xdd67ffaa,0xd869ffac,0xdd69ffac,0xd869ffae,
	0xdd69ffae,0xd869ffb0,0xdd6bffb0,0xd869ffb2,0xdd6bffb2,0xd869ffb4,0xdd6dffb4,0xd869ffb6,
	0xdd6dffb6,0xd869ffb8,0xdd6fffb8,0xd869ffba,0xdd6fffba,0xd869ffbc,0xdd71ffbc,0xd869ffbe,
	0xdd71ffbe,0xd869ffc0,0xdd73ffc0,0xd869ffc2,0xdd73ffc2,0xd869ffc4,0xdd75ffc4,0xd869ffc6,
	0xdd75ffc6,0xd869ffc8,0xdd77ffc8,0xd869ffca,0xdd77ffca,0xd869ffcc,0xdd79ffcc,0xd869ffce,
	0xdd79ffce,0xd869ffd0,0xdd7bffd0,0xd869ffd2,0xdd7bffd2,0xd869ffd4,0xdd7dffd4,0xd869ffd6,
	0xdd7dffd6,0xd869ffd8,0xdd7fffd8,0xd869ffda,0xdd7fffda,0xd869ffdc,0xdd81ffdc,0xd869ffde,
	0xdd81ffde,0xd869ffe0,0xdd83ffe0,0xd869ffe2,0xdd83ffe2,0xd869ffe4,0xdd85ffe4,0xd869ffe6,
	0xdd85ffe6,0xd869ffe8,0xdd87ffe8,0xd869ffea,0xdd87ffea,0xd869ffec,0xdd89ffec,0xd869ffee,
	0xdd89ffee,0xd869fff0,0xdd8bfff0,0xd869fff2,0xdd8bfff2,0xd869fff4,0xdd8dfff4,0xd869fff6,
	0xdd8dfff6,0xd869fff8,0xdd8ffff8,0xd869fffa,0xdd8ffffa,0xd869fffc,0xdd91fffc,0xd869fffe,
	0xdd91fffe,0xd86a0000,0xdd920000,0xd86a0002,0xdd930002,0xd86a0004,0xdd960004,0xd86a0006,
	0xdd970006,0xd86a0008,0xdd960008,0xd86a000a,0xdd97000a,0xd86a000c,0xdd9a000c,0xd86a000e,
	0xdd9b000e,0xd86a0010,0xdd9a0010,0xd86a0012,0xdd9b0012,0xd86a0014,0xdd9e0014,0xd86a0016,
	0xdd9f0016,0xd86a0018,0xdd9e0018,0xd86a001a,0xdd9f001a,0xd86a001c,0xdda2001c,0xd86a001e,
	0xdda3001e,0xd86a0020,0xdda20020,0xd86a0022,0xdda30022,0xd86a0024,0xdda60024,0xd86a0026,
	0xdda70026,0xd86a0028,0xdda60028,0xd86a002a,0xdda7002a,0xd86a002c,0xddaa002c,0xd86a002e,
	0xddab002e,0xd86a0030,0xddaa0030,0xd86a0032,0xddab0032,0xd86a0034,0xddae0034,0xd86a0036,
	0xddaf0036,0xd86a0038,0xddae0038,0xd86a003a,0xddaf003a,0xd86a003c,0xddb2003c,0xd86a003e,
	0xddb3003e,0xd86a0040,0xddb20040,0xd86a0042,0xddb30042,0xd86a0044,0xddb60044,0xd86a0046,
	0xddb70046,0xd86a0048,0xddb60048,0xd86a004a,0xddb7004a,0xd86a004c,0xddba004c,0xd86a004e,
	0xddbb004e,0xd86a0050,0xddba0050,0xd86a0052,0xddbb0052,0xd86a0054,0xddbe0054,0xd86a0056,
	0xddbf0056,0xd86a0058,0xddbe0058,0xd86a005a,0xddbf005a,0xd86a005c,0xddc2005c,0xd86a005e,
	0xddc3005e,0xd86a0060,0xddc20060,0xd86a0062,0xddc30062,0xd86a0064,0xddc60064,0xd86a0066,
	0xddc70066,0xd86a0068,0xddc60068,0xd86a006a,0xddc7006a,0xd86a006c,0xddca006c,0xd86a006e,
	0xddcb006e,0xd86a0070,0xddca0070,0xd86a0072,0xddcb0072,0xd86a0074,0xddce0074,0xd86a0076,
	0xddcf0076,0xd86a0078,0xddce0078,0xd86a007a,0xddcf007a,0xd86a007c,0xddd2007c,0xd86a007e,
	0xddd3007e,0xd86a0080,0xddd20080,0xd86a0082,0xddd30082,0xd86a0084,0xddd60084,0xd86a0086,
	0xddd70086,0xd86a0088,0xddd60088,0xd86a008a,0xddd7008a,0xd86a008c,0xddda008c,0xd86a008e,
	0xdddb008e,0xd86a0090,0xddda0090,0xd86a0092,0xdddb0092,0xd86a0094,0xddde0094,0xd86a0096,
	0xdddf0096,0xd86a0098,0xddde0098,0xd86a009a,0xdddf009a,0xd86a009c,0xdde2009c,0xd86a009e,
	0xdde3009e,0xd86a00a0,0xdde200a0,0xd86a00a2,0xdde300a2,0xd86a00a4,0xdde600a4,0xd86a00a6,
	0xdde700a6,0xd86a00a8,0xdde600a8,0xd86a00aa,0xdde700aa,0xd86a00ac,0xddea00ac,0xd86a00ae,
	0xddeb00ae,0xd86a00b0,0xddea00b0,0xd86a00b2,0xddeb00b2,0xd86a00b4,0xddee00b4,0xd86a00b6,
	0xddef00b6,0xd86a00b8,0xddee00b8,0xd86a00ba,0xddef00ba,0xd86a00bc,0xddf200bc,0xd86a00be,
	0xddf300be,0xd86a00c0,0xddf200c0,0xd86a00c2,0xddf300c2,0xd86a00c4,0xddf600c4,0xd86a00c6,
	0xddf700c6,0xd86a00c8,0xddf600c8,0xd86a00ca,0xddf700ca,0xd86a00cc,0xddfa00cc,0xd86a00ce,
	0xddfb00ce,0xd86a00d0,0xddfa00d0,0xd86a00d2,0xddfb00d2,0xd86a00d4,0xddfe00d4,0xd86a00d6,
	0xddff00d6,0xd86a00d8,0xddfe00d8,0xd86a00da,0xddff00da,0xd86a00dc,0xde0200dc,0xd86a00de,
	0xde0300de,0xd86a00e0,0xde0200e0,0xd86a00e2,0xde0300e2,0xd86a00e4,0xde0600e4,0xd86a00e6,
	0xde0700e6,0xd86a00e8,0xde0600e8,0xd86a00ea,0xde0700ea,0xd86a00ec,0xde0a00ec,0xd86a00ee,
	0xde0b00ee,0xd86a00f0,0xde0a00f0,0xd86a00f2,0xde0b00f2,0xd86a00f4,0xde0e00f4,0xd86a00f6,
	0xde0f00f6,0xd86a00f8,0xde0e00f8,0xd86a00fa,0xde0f00fa,0xd86a00fc,0xde1200fc,0xd86a00fe,
	0xde1300fe,0xd86a0100,0xde120100,0xd86a0102,0xde130102,0xd86a0104,0xde160104,0xd86a0106,
	0xde170106,0xd86a0108,0xde160108,0xd86a010a,0xde17010a,0xd86a010c,0xde1a010c,0xd86a010e,
	0xde1b010e,0xd86a0110,0xde1a0110,0xd86a0112,0xde1b0112,0xd86a0114,0xde1e0114,0xd86a0116,
	0xde1f0116,0xd86a0118,0xde1e0118,0xd86a011a,0xde1f011a,0xd86a011c,0xde22011c,0xd86a011e,
	0xde23011e,0xd86a0120,0xde220120,0xd86a0122,0xde230122,0xd86a0124,0xde260124,0xd86a0126,
	0xde270126,0xd86a0128,0xde260128,0xd86a012a,0xde27012a,0xd86a012c,0xde2a012c,0xd86a012e,
	0xde2b012e,0xd86a0130,0xde2a0130,0xd86a0132,0xde2b0132,0xd86a0134,0xde2e0134,0xd86a0136,
	0xde2f0136,0xd86a0138,0xde2e0138,0xd86a013a,0xde2f013a,0xd86a013c,0xde32013c,0xd86a013e,
	0xde33013e,0xd86a0140,0xde320140,0xd86a0142,0xde330142,0xd86a0144,0xde360144,0xd86a0146,
	0xde370146,0xd86a0148,0xde360148,0xd86a014a,0xde37014a,0xd86a014c,0xde3a014c,0xd86a014e,
	0xde3b014e,0xd86a0150,0xde3a0150,0xd86a0152,0xde3b0152,0xd86a0154,0xde3e0154,0xd86a0156,
	0xde3f0156,0xd86a0158,0xde3e0158,0xd86a015a,0xde3f015a,0xd86a015c,0xde42015c,0xd86a015e,
	0xde43015e,0xd86a0160,0xde420160,0xd86a0162,0xde430162,0xd86a0164,0xde460164,0xd86a0166,
	0xde470166,0xd86a0168,0xde460168,0xd86a016a,0xde47016a,0xd86a016c,0xde4a016c,0xd86a016e,
	0xde4b016e,0xd86a0170,0xde4a0170,0xd86a0172,0xde4b0172,0xd86a0174,0xde4e0174,0xd86a0176,
	0xde4f0176,0xd86a0178,0xde4e0178,0xd86a017a,0xde4f017a,0xd86a017c,0xde52017c,0xd86a017e,
	0xde53017e,0xd86a0180,0xde520180,0xd86a0182,0xde530182,0xd86a0184,0xde560184,0xd86a0186,
	0xde570186,0xd86a0188,0xde560188,0xd86a018a,0xde57018a,0xd86a018c,0xde5a018c,0xd86a018e,
	0xde5b018e,0xd86a0190,0xde5a0190,0xd86a0192,0xde5b0192,0xd86a0194,0xde5e0194,0xd86a0196,
	0xde5f0196,0xd86a0198,0xde5e0198,0xd86a019a,0xde5f019a,0xd86a019c,0xde62019c,0xd86a019e,
	0xde63019e,0xd86a01a0,0xde6201a0,0xd86a01a2,0xde6301a2,0xd86a01a4,0xde6601a4,0xd86a01a6,
	0xde6701a6,0xd86a01a8,0xde6601a8,0xd86a01aa,0xde6701aa,0xd86a01ac,0xde6a01ac,0xd86a01ae,
	0xde6b01ae,0xd86a01b0,0xde6a01b0,0xd86a01b2,0xde6b01b2,0xd86a01b4,0xde6e01b4,0xd86a01b6,
	0xde6f01b6,0xd86a01b8,0xde6e01b8,0xd86a01ba,0xde6f01ba,0xd86a01bc,0xde7201bc,0xd86a01be,
	0xde7301be,0xd86a01c0,0xde7201c0,0xd86a01c2,0xde7301c2,0xd86a01c4,0xde7601c4,0xd86a01c6,
	0xde7701c6,0xd86a01c8,0xde7601c8,0xd86a01ca,0xde7701ca,0xd86a01cc,0xde7a01cc,0xd86a01ce,
	0xde7b01ce,0xd86a01d0,0xde7a01d0,0xd86a01d2,0xde7b01d2,0xd86a01d4,0xde7e01d4,0xd86a01d6,
	0xde7f01d6,0xd86a01d8,0xde7e01d8,0xd86a01da,0xde7f01da,0xd86a01dc,0xde8201dc,0xd86a01de,
	0xde8301de,0xd86a01e0,0xde8201e0,0xd86a01e2,0xde8301e2,0xd86a01e4,0xde8601e4,0xd86a01e6,
	0xde8701e6,0xd86a01e8,0xde8601e8,0xd86a01ea,0xde8701ea,0xd86a01ec,0xde8a01ec,0xd86a01ee,
	0xde8b01ee,0xd86a01f0,0xde8a01f0,0xd86a01f2,0xde8b01f2,0xd86a01f4,0xde8e01f4,0xd86a01f6,
	0xde8f01f6,0xd86a01f8,0xde8e01f8,0xd86a01fa,0xde8f01fa,0xd86a01fc,0xde9201fc,0xd86a01fe,
	0xde9301fe,0xd86a0200,0xde920200,0xd86a0202,0xde930202,0xd86a0204,0xde960204,0xd86a0206,
	0xde970206,0xd86a0208,0xde960208,0xd86a020a,0xde97020a,0xd86a020c,0xde9a020c,0xd86a020e,
	0xde9b020e,0xd86a0210,0xde9a0210,0xd86a0212,0xde9b0212,0xd86a0214,0xde9e0214,0xd86a0216,
	0xde9f0216,0xd86a0218,0xde9e0218,0xd86a021a,0xde9f021a,0xd86a021c,0xdea2021c,0xd86a021e,
	0xdea3021e,0xd86a0220,0xdea20220,0xd86a0222,0xdea30222,0xd86a0224,0xdea60224,0xd86a0226,
	0xdea70226,0xd86a0228,0xdea60228,0xd86a022a,0xdea7022a,0xd86a022c,0xdeaa022c,0xd86a022e,
	0xdeab022e,0xd86a0230,0xdeaa0230,0xd86a0232,0xdeab0232,0xd86a0234,0xdeae0234,0xd86a0236,
	0xdeaf0236,0xd86a0238,0xdeae0238,0xd86a023a,0xdeaf023a,0xd86a023c,0xdeb2023c,0xd86a023e,
	0xdeb3023e,0xd86a0240,0xdeb20240,0xd86a0242,0xdeb30242,0xd86a0244,0xdeb60244,0xd86a0246,
	0xdeb70246,0xd86a0248,0xdeb60248,0xd86a024a,0xdeb7024a,0xd86a024c,0xdeba024c,0xd86a024e,
	0xdebb024e,0xd86a0250,0xdeba0250,0xd86a0252,0xdebb0252,0xd86a0254,0xdebe0254,0xd86a0256,
	0xdebf0256,0xd86a0258,0xdebe0258,0xd86a025a,0xdebf025a,0xd86a025c,0xdec2025c,0xd86a025e,
	0xdec3025e,0xd86a0260,0xdec20260,0xd86a0262,0xdec30262,0xd86a0264,0xdec60264,0xd86a0266,
	0xdec70266,0xd86a0268,0xdec60268,0xd86a026a,0xdec7026a,0xd86a026c,0xdeca026c,0xd86a026e,
	0xdecb026e,0xd86a0270,0xdeca0270,0xd86a0272,0xdecb0272,0xd86a0274,0xdece0274,0xd86a0276,
	0xdecf0276,0xd86a0278,0xdece0278,0xd86a027a,0xdecf027a,0xd86a027c,0xded2027c,0xd86a027e,
	0xded3027e,0xd86a0280,0xded20280,0xd86a0282,0xded30282,0xd86a0284,0xded60284,0xd86a0286,
	0xded70286,0xd86a0288,0xded60288,0xd86a028a,0xded7028a,0xd86a028c,0xdeda028c,0xd86a028e,
	0xdedb028e,0xd86a0290,0xdeda0290,0xd86a0292,0xdedb0292,0xd86a0294,0xdede0294,0xd86a0296,
	0xdedf0296,0xd86a0298,0xdede0298,0xd86a029a,0xdedf029a,0xd86a029c,0xdee2029c,0xd86a029e,
	0xdee3029e,0xd86a02a0,0xdee202a0,0xd86a02a2,0xdee302a2,0xd86a02a4,0xdee602a4,0xd86a02a6,
	0xdee702a6,0xd86a02a8,0xdee602a8,0xd86a02aa,0xdee702aa,0xd86a02ac,0xdeea02ac,0xd86a02ae,
	0xdeeb02ae,0xd86a02b0,0xdeea02b0,0xd86a02b2,0xdeeb02b2,0xd86a02b4,0xdeee02b4,0xd86a02b6,
	0xdeef02b6,0xd86a02b8,0xdeee02b8,0xd86a02ba,0xdeef02ba,0xd86a02bc,0xdef202bc,0xd86a02be,
	0xdef302be,0xd86a02c0,0xdef202c0,0xd86a02c2,0xdef302c2,0xd86a02c4,0xdef602c4,0xd86a02c6,
	0xdef702c6,0xd86a02c8,0xdef602c8,0xd86a02ca,0xdef702ca,0xd86a02cc,0xdefa02cc,0xd86a02ce,
	0xdefb02ce,0xd86a02d0,0xdefa02d0,0xd86a02d2,0xdefb02d2,0xd86a02d4,0xdefe02d4,0xd86a02d6,
	0xdeff02d6,0xd86a02d8,0xdefe02d8,0xd86a02da,0xdeff02da,0xd86a02dc,0xdf0202dc,0xd86a02de,
	0xdf0302de,0xd86a02e0,0xdf0202e0,0xd86a02e2,0xdf0302e2,0xd86a02e4,0xdf0602e4,0xd86a02e6,
	0xdf0702e6,0xd86a02e8,0xdf0602e8,0xd86a02ea,0xdf0702ea,0xd86a02ec,0xdf0a02ec,0xd86a02ee,
	0xdf0b02ee,0xd86a02f0,0xdf0a02f0,0xd86a02f2,0xdf0b02f2,0xd86a02f4,0xdf0e02f4,0xd86a02f6,
	0xdf0f02f6,0xd86a02f8,0xdf0e02f8,0xd86a02fa,0xdf0f02fa,0xd86a02fc,0xdf1202fc,0xd86a02fe,
	0xdf1302fe,0xd86a0300,0xdf120300,0xd86a0302,0xdf130302,0xd86a0304,0xdf160304,0xd86a0306,
	0xdf170306,0xd86a0308,0xdf160308,0xd86a030a,0xdf17030a,0xd86a030c,0xdf1a030c,0xd86a030e,
	0xdf1b030e,0xd86a0310,0xdf1a0310,0xd86a0312,0xdf1b0312,0xd86a0314,0xdf1e0314,0xd86a0316,
	0xdf1f0316,0xd86a0318,0xdf1e0318,0xd86a031a,0xdf1f031a,0xd86a031c,0xdf22031c,0xd86a031e,
	0xdf23031e,0xd86a0320,0xdf220320,0xd86a0322,0xdf230322,0xd86a0324,0xdf260324,0xd86a0326,
	0xdf270326,0xd86a0328,0xdf260328,0xd86a032a,0xdf27032a,0xd86a032c,0xdf2a032c,0xd86a032e,
	0xdf2b032e,0xd86a0330,0xdf2a0330,0xd86a0332,0xdf2b0332,0xd86a0334,0xdf2e0334,0xd86a0336,
	0xdf2f0336,0xd86a0338,0xdf2e0338,0xd86a033a,0xdf2f033a,0xd86a033c,0xdf32033c,0xd86a033e,
	0xdf33033e,0xd86a0340,0xdf320340,0xd86a0342,0xdf330342,0xd86a0344,0xdf360344,0xd86a0346,
	0xdf370346,0xd86a0348,0xdf360348,0xd86a034a,0xdf37034a,0xd86a034c,0xdf3a034c,0xd86a034e,
	0xdf3b034e,0xd86a0350,0xdf3a0350,0xd86a0352,0xdf3b0352,0xd86a0354,0xdf3e0354,0xd86a0356,
	0xdf3f0356,0xd86a0358,0xdf3e0358,0xd86a035a,0xdf3f035a,0xd86a035c,0xdf42035c,0xd86a035e,
	0xdf43035e,0xd86a0360,0xdf420360,0xd86a0362,0xdf430362,0xd86a0364,0xdf460364,0xd86a0366,
	0xdf470366,0xd86a0368,0xdf460368,0xd86a036a,0xdf47036a,0xd86a036c,0xdf4a036c,0xd86a036e,
	0xdf4b036e,0xd86a0370,0xdf4a0370,0xd86a0372,0xdf4b0372,0xd86a0374,0xdf4e0374,0xd86a0376,
	0xdf4f0376,0xd86a0378,0xdf4e0378,0xd86a037a,0xdf4f037a,0xd86a037c,0xdf52037c,0xd86a037e,
	0xdf53037e,0xd86a0380,0xdf520380,0xd86a0382,0xdf530382,0xd86a0384,0xdf560384,0xd86a0386,
	0xdf570386,0xd86a0388,0xdf560388,0xd86a038a,0xdf57038a,0xd86a038c,0xdf5a038c,0xd86a038e,
	0xdf5b038e,0xd86a0390,0xdf5a0390,0xd86a0392,0xdf5b0392,0xd86a0394,0xdf5e0394,0xd86a0396,
	0xdf5f0396,0xd86a0398,0xdf5e0398,0xd86a039a,0xdf5f039a,0xd86a039c,0xdf62039c,0xd86a039e,
	0xdf63039e,0xd86a03a0,0xdf6203a0,0xd86a03a2,0xdf6303a2,0xd86a03a4,0xdf6603a4,0xd86a03a6,
	0xdf6703a6,0xd86a03a8,0xdf6603a8,0xd86a03aa,0xdf6703aa,0xd86a03ac,0xdf6a03ac,0xd86a03ae,
	0xdf6b03ae,0xd86a03b0,0xdf6a03b0,0xd86a03b2,0xdf6b03b2,0xd86a03b4,0xdf6e03b4,0xd86a03b6,
	0xdf6f03b6,0xd86a03b8,0xdf6e03b8,0xd86a03ba,0xdf6f03ba,0xd86a03bc,0xdf7203bc,0xd86a03be,
	0xdf7303be,0xd86a03c0,0xdf7203c0,0xd86a03c2,0xdf7303c2,0xd86a03c4,0xdf7603c4,0xd86a03c6,
	0xdf7703c6,0xd86a03c8,0xdf7603c8,0xd86a03ca,0xdf7703ca,0xd86a03cc,0xdf7a03cc,0xd86a03ce,
	0xdf7b03ce,0xd86a03d0,0xdf7a03d0,0xd86a03d2,0xdf7b03d2,0xd86a03d4,0xdf7e03d4,0xd86a03d6,
	0xdf7f03d6,0xd86a03d8,0xdf7e03d8,0xd86a03da,0xdf7f03da,0xd86a03dc,0xdf8203dc,0xd86a03de,
	0xdf8303de,0xd86a03e0,0xdf8203e0,0xd86a03e2,0xdf8303e2,0xd86a03e4,0xdf8603e4,0xd86a03e6,
	0xdf8703e6,0xd86a03e8,0xdf8603e8,0xd86a03ea,0xdf8703ea,0xd86a03ec,0xdf8a03ec,0xd86a03ee,
	0xdf8b03ee,0xd86a03f0,0xdf8a03f0,0xd86a03f2,0xdf8b03f2,0xd86a03f4,0xdf8e03f4,0xd86a03f6,
	0xdf8f03f6,0xd86a03f8,0xdf8e03f8,0xd86a03fa,0xdf8f03fa,0xd86a03fc,0xdf9203fc,0xd86a03fe,
	0xdf9303fe,0xd86a0400,0xdf920400,0xd86a0402,0xdf930402,0xd86a0404,0xdf960404,0xd86a0406,
	0xdf970406,0xd86a0408,0xdf960408,0xd86a040a,0xdf97040a,0xd86a040c,0xdf9a040c,0xd86a040e,
	0xdf9b040e,0xd86a0410,0xdf9a0410,0xd86a0412,0xdf9b0412,0xd86a0414,0xdf9e0414,0xd86a0416,
	0xdf9f0416,0xd86a0418,0xdf9e0418,0xd86a041a,0xdf9f041a,0xd86a041c,0xdfa2041c,0xd86a041e,
	0xdfa3041e,0xd86a0420,0xdfa20420,0xd86a0422,0xdfa30422,0xd86a0424,0xdfa60424,0xd86a0426,
	0xdfa70426,0xd86a0428,0xdfa60428,0xd86a042a,0xdfa7042a,0xd86a042c,0xdfaa042c,0xd86a042e,
	0xdfab042e,0xd86a0430,0xdfaa0430,0xd86a0432,0xdfab0432,0xd86a0434,0xdfae0434,0xd86a0436,
	0xdfaf0436,0xd86a0438,0xdfae0438,0xd86a043a,0xdfaf043a,0xd86a043c,0xdfb2043c,0xd86a043e,
	0xdfb3043e,0xd86a0440,0xdfb20440,0xd86a0442,0xdfb30442,0xd86a0444,0xdfb60444,0xd86a0446,
	0xdfb70446,0xd86a0448,0xdfb60448,0xd86a044a,0xdfb7044a,0xd86a044c,0xdfba044c,0xd86a044e,
	0xdfbb044e,0xd86a0450,0xdfba0450,0xd86a0452,0xdfbb0452,0xd86a0454,0xdfbe0454,0xd86a0456,
	0xdfbf0456,0xd86a0458,0xdfbe0458,0xd86a045a,0xdfbf045a,0xd86a045c,0xdfc2045c,0xd86a045e,
	0xdfc3045e,0xd86a0460,0xdfc20460,0xd86a0462,0xdfc30462,0xd86a0464,0xdfc60464,0xd86a0466,
	0xdfc70466,0xd86a0468,0xdfc60468,0xd86a046a,0xdfc7046a,0xd86a046c,0xdfca046c,0xd86a046e,
	0xdfcb046e,0xd86a0470,0xdfca0470,0xd86a0472,0xdfcb0472,0xd86a0474,0xdfce0474,0xd86a0476,
	0xdfcf0476,0xd86a0478,0xdfce0478,0xd86a047a,0xdfcf047a,0xd86a047c,0xdfd2047c,0xd86a047e,
	0xdfd3047e,0xd86a0480,0xdfd20480,0xd86a0482,0xdfd30482,0xd86a0484,0xdfd60484,0xd86a0486,
	0xdfd70486,0xd86a0488,0xdfd60488,0xd86a048a,0xdfd7048a,0xd86a048c,0xdfda048c,0xd86a048e,
	0xdfdb048e,0xd86a0490,0xdfda0490,0xd86a0492,0xdfdb0492,0xd86a0494,0xdfde0494,0xd86a0496,
	0xdfdf0496,0xd86a0498,0xdfde0498,0xd86a049a,0xdfdf049a,0xd86a049c,0xdfe2049c,0xd86a049e,
	0xdfe3049e,0xd86a04a0,0xdfe204a0,0xd86a04a2,0xdfe304a2,0xd86a04a4,0xdfe604a4,0xd86a04a6,
	0xdfe704a6,0xd86a04a8,0xdfe604a8,0xd86a04aa,0xdfe704aa,0xd86a04ac,0xdfea04ac,0xd86a04ae,
	0xdfeb04ae,0xd86a04b0,0xdfea04b0,0xd86a04b2,0xdfeb04b2,0xd86a04b4,0xdfee04b4,0xd86a04b6,
	0xdfef04b6,0xd86a04b8,0xdfee04b8,0xd86a04ba,0xdfef04ba,0xd86a04bc,0xdff204bc,0xd86a04be,
	0xdff304be,0xd86a04c0,0xdff204c0,0xd86a04c2,0xdff304c2,0xd86a04c4,0xdff604c4,0xd86a04c6,
	0xdff704c6,0xd86a04c8,0xdff604c8,0xd86a04ca,0xdff704ca,0xd86a04cc,0xdffa04cc,0xd86a04ce,
	0xdffb04ce,0xd86a04d0,0xdffa04d0,0xd86a04d2,0xdffb04d2,0xd86a04d4,0xdffe04d4,0xd86a04d6,
	0xdfff04d6,0xd86a04d8,0xdffe04d8,0xd86a04da,0xdfff04da,0xd86b04dc,0xdc0204dc,0xd86b04de,
	0xdc0304de,0xd86b04e0,0xdc0204e0,0xd86b04e2,0xdc0304e2,0xd86b04e4,0xdc0604e4,0xd86b04e6,
	0xdc0704e6,0xd86b04e8,0xdc0604e8,0xd86b04ea,0xdc0704ea,0xd86b04ec,0xdc0a04ec,0xd86b04ee,
	0xdc0b04ee,0xd86b04f0,0xdc0a04f0,0xd86b04f2,0xdc0b04f2,0xd86b04f4,0xdc0e04f4,0xd86b04f6,
	0xdc0f04f6,0xd86b04f8,0xdc0e04f8,0xd86b04fa,0xdc0f04fa,0xd86b04fc,0xdc1204fc,0xd86b04fe,
	0xdc1304fe,0xd86b0500,0xdc120500,0xd86b0502,0xdc130502,0xd86b0504,0xdc160504,0xd86b0506,
	0xdc170506,0xd86b0508,0xdc160508,0xd86b050a,0xdc17050a,0xd86b050c,0xdc1a050c,0xd86b050e,
	0xdc1b050e,0xd86b0510,0xdc1a0510,0xd86b0512,0xdc1b0512,0xd86b0514,0xdc1e0514,0xd86b0516,
	0xdc1f0516,0xd86b0518,0xdc1e0518,0xd86b051a,0xdc1f051a,0xd86b051c,0xdc22051c,0xd86b051e,
	0xdc23051e,0xd86b0520,0xdc220520,0xd86b0522,0xdc230522,0xd86b0524,0xdc260524,0xd86b0526,
	0xdc270526,0xd86b0528,0xdc260528,0xd86b052a,0xdc27052a,0xd86b052c,0xdc2a052c,0xd86b052e,
	0xdc2b052e,0xd86b0530,0xdc2a0530,0xd86b0532,0xdc2b0532,0xd86b0534,0xdc2e0534,0xd86b0536,
	0xdc2f0536,0xd86b0538,0xdc2e0538,0xd86b053a,0xdc2f053a,0xd86b053c,0xdc32053c,0xd86b053e,
	0xdc33053e,0xd86b0540,0xdc320540,0xd86b0542,0xdc330542,0xd86b0544,0xdc360544,0xd86b0546,
	0xdc370546,0xd86b0548,0xdc360548,0xd86b054a,0xdc37054a,0xd86b054c,0xdc3a054c,0xd86b054e,
	0xdc3b054e,0xd86b0550,0xdc3a0550,0xd86b0552,0xdc3b0552,0xd86b0554,0xdc3e0554,0xd86b0556,
	0xdc3f0556,0xd86b0558,0xdc3e0558,0xd86b055a,0xdc3f055a,0xd86b055c,0xdc42055c,0xd86b055e,
	0xdc43055e,0xd86b0560,0xdc420560,0xd86b0562,0xdc430562,0xd86b0564,0xdc460564,0xd86b0566,
	0xdc470566,0xd86b0568,0xdc460568,0xd86b056a,0xdc47056a,0xd86b056c,0xdc4a056c,0xd86b056e,
	0xdc4b056e,0xd86b0570,0xdc4a0570,0xd86b0572,0xdc4b0572,0xd86b0574,0xdc4e0574,0xd86b0576,
	0xdc4f0576,0xd86b0578,0xdc4e0578,0xd86b057a,0xdc4f057a,0xd86b057c,0xdc52057c,0xd86b057e,
	0xdc53057e,0xd86b0580,0xdc520580,0xd86b0582,0xdc530582,0xd86b0584,0xdc560584,0xd86b0586,
	0xdc570586,0xd86b0588,0xdc560588,0xd86b058a,0xdc57058a,0xd86b058c,0xdc5a058c,0xd86b058e,
	0xdc5b058e,0xd86b0590,0xdc5a0590,0xd86b0592,0xdc5b0592,0xd86b0594,0xdc5e0594,0xd86b0596,
	0xdc5f0596,0xd86b0598,0xdc5e0598,0xd86b059a,0xdc5f059a,0xd86b059c,0xdc62059c,0xd86b059e,
	0xdc63059e,0xd86b05a0,0xdc6205a0,0xd86b05a2,0xdc6305a2,0xd86b05a4,0xdc6605a4,0xd86b05a6,
	0xdc6705a6,0xd86b05a8,0xdc6605a8,0xd86b05aa,0xdc6705aa,0xd86b05ac,0xdc6a05ac,0xd86b05ae,
	0xdc6b05ae,0xd86b05b0,0xdc6a05b0,0xd86b05b2,0xdc6b05b2,0xd86b05b4,0xdc6e05b4,0xd86b05b6,
	0xdc6f05b6,0xd86b05b8,0xdc6e05b8,0xd86b05ba,0xdc6f05ba,0xd86b05bc,0xdc7205bc,0xd86b05be,
	0xdc7305be,0xd86b05c0,0xdc7205c0,0xd86b05c2,0xdc7305c2,0xd86b05c4,0xdc7605c4,0xd86b05c6,
	0xdc7705c6,0xd86b05c8,0xdc7605c8,0xd86b05ca,0xdc7705ca,0xd86b05cc,0xdc7a05cc,0xd86b05ce,
	0xdc7b05ce,0xd86b05d0,0xdc7a05d0,0xd86b05d2,0xdc7b05d2,0xd86b05d4,0xdc7e05d4,0xd86b05d6,
	0xdc7f05d6,0xd86b05d8,0xdc7e05d8,0xd86b05da,0xdc7f05da,0xd86b05dc,0xdc8205dc,0xd86b05de,
	0xdc8305de,0xd86b05e0,0xdc8205e0,0xd86b05e2,0xdc8305e2,0xd86b05e4,0xdc8605e4,0xd86b05e6,
	0xdc8705e6,0xd86b05e8,0xdc8605e8,0xd86b05ea,0xdc8705ea,0xd86b05ec,0xdc8a05ec,0xd86b05ee,
	0xdc8b05ee,0xd86b05f0,0xdc8a05f0,0xd86b05f2,0xdc8b05f2,0xd86b05f4,0xdc8e05f4,0xd86b05f6,
	0xdc8f05f6,0xd86b05f8,0xdc8e05f8,0xd86b05fa,0xdc8f05fa,0xd86b05fc,0xdc9205fc,0xd86b05fe,
	0xdc9305fe,0xd86b0600,0xdc920600,0xd86b0602,0xdc930602,0xd86b0604,0xdc960604,0xd86b0606,
	0xdc970606,0xd86b0608,0xdc960608,0xd86b060a,0xdc97060a,0xd86b060c,0xdc9a060c,0xd86b060e,
	0xdc9b060e,0xd86b0610,0xdc9a0610,0xd86b0612,0xdc9b0612,0xd86b0614,0xdc9e0614,0xd86b0616,
	0xdc9f0616,0xd86b0618,0xdc9e0618,0xd86b061a,0xdc9f061a,0xd86b061c,0xdca2061c,0xd86b061e,
	0xdca3061e,0xd86b0620,0xdca20620,0xd86b0622,0xdca30622,0xd86b0624,0xdca60624,0xd86b0626,
	0xdca70626,0xd86b0628,0xdca60628,0xd86b062a,0xdca7062a,0xd86b062c,0xdcaa062c,0xd86b062e,
	0xdcab062e,0xd86b0630,0xdcaa0630,0xd86b0632,0xdcab0632,0xd86b0634,0xdcae0634,0xd86b0636,
	0xdcaf0636,0xd86b0638,0xdcae0638,0xd86b063a,0xdcaf063a,0xd86b063c,0xdcb2063c,0xd86b063e,
	0xdcb3063e,0xd86b0640,0xdcb20640,0xd86b0642,0xdcb30642,0xd86b0644,0xdcb60644,0xd86b0646,
	0xdcb70646,0xd86b0648,0xdcb60648,0xd86b064a,0xdcb7064a,0xd86b064c,0xdcba064c,0xd86b064e,
	0xdcbb064e,0xd86b0650,0xdcba0650,0xd86b0652,0xdcbb0652,0xd86b0654,0xdcbe0654,0xd86b0656,
	0xdcbf0656,0xd86b0658,0xdcbe0658,0xd86b065a,0xdcbf065a,0xd86b065c,0xdcc2065c,0xd86b065e,
	0xdcc3065e,0xd86b0660,0xdcc20660,0xd86b0662,0xdcc30662,0xd86b0664,0xdcc60664,0xd86b0666,
	0xdcc70666,0xd86b0668,0xdcc60668,0xd86b066a,0xdcc7066a,0xd86b066c,0xdcca066c,0xd86b066e,
	0xdccb066e,0xd86b0670,0xdcca0670,0xd86b0672,0xdccb0672,0xd86b0674,0xdcce0674,0xd86b0676,
	0xdccf0676,0xd86b0678,0xdcce0678,0xd86b067a,0xdccf067a,0xd86b067c,0xdcd2067c,0xd86b067e,
	0xdcd3067e,0xd86b0680,0xdcd20680,0xd86b0682,0xdcd30682,0xd86b0684,0xdcd60684,0xd86b0686,
	0xdcd70686,0xd86b0688,0xdcd60688,0xd86b068a,0xdcd7068a,0xd86b068c,0xdcda068c,0xd86b068e,
	0xdcdb068e,0xd86b0690,0xdcda0690,0xd86b0692,0xdcdb0692,0xd86b0694,0xdcde0694,0xd86b0696,
	0xdcdf0696,0xd86b0698,0xdcde0698,0xd86b069a,0xdcdf069a,0xd86b069c,0xdce2069c,0xd86b069e,
	0xdce3069e,0xd86b06a0,0xdce206a0,0xd86b06a2,0xdce306a2,0xd86b06a4,0xdce606a4,0xd86b06a6,
	0xdce706a6,0xd86b06a8,0xdce606a8,0xd86b06aa,0xdce706aa,0xd86b06ac,0xdcea06ac,0xd86b06ae,
	0xdceb06ae,0xd86b06b0,0xdcea06b0,0xd86b06b2,0xdceb06b2,0xd86b06b4,0xdcee06b4,0xd86b06b6,
	0xdcef06b6,0xd86b06b8,0xdcee06b8,0xd86b06ba,0xdcef06ba,0xd86b06bc,0xdcf206bc,0xd86b06be,
	0xdcf306be,0xd86b06c0,0xdcf206c0,0xd86b06c2,0xdcf306c2,0xd86b06c4,0xdcf606c4,0xd86b06c6,
	0xdcf706c6,0xd86b06c8,0xdcf606c8,0xd86b06ca,0xdcf706ca,0xd86b06cc,0xdcfa06cc,0xd86b06ce,
	0xdcfb06ce,0xd86b06d0,0xdcfa06d0,0xd86b06d2,0xdcfb06d2,0xd86b06d4,0xdcfe06d4,0xd86b06d6,
	0xdcff06d6,0xd86b06d8,0xdcfe06d8,0xd86b06da,0xdcff06da,0xd86b06dc,0xdd0206dc,0xd86b06de,
	0xdd0306de,0xd86b06e0,0xdd0206e0,0xd86b06e2,0xdd0306e2,0xd86b06e4,0xdd0606e4,0xd86b06e6,
	0xdd0706e6,0xd86b06e8,0xdd0606e8,0xd86b06ea,0xdd0706ea,0xd86b06ec,0xdd0a06ec,0xd86b06ee,
	0xdd0b06ee,0xd86b06f0,0xdd0a06f0,0xd86b06f2,0xdd0b06f2,0xd86b06f4,0xdd0e06f4,0xd86b06f6,
	0xdd0f06f6,0xd86b06f8,0xdd0e06f8,0xd86b06fa,0xdd0f06fa,0xd86b06fc,0xdd1206fc,0xd86b06fe,
	0xdd1306fe,0xd86b0700,0xdd120700,0xd86b0702,0xdd130702,0xd86b0704,0xdd160704,0xd86b0706,
	0xdd170706,0xd86b0708,0xdd160708,0xd86b070a,0xdd17070a,0xd86b070c,0xdd1a070c,0xd86b070e,
	0xdd1b070e,0xd86b0710,0xdd1a0710,0xd86b0712,0xdd1b0712,0xd86b0714,0xdd1e0714,0xd86b0716,
	0xdd1f0716,0xd86b0718,0xdd1e0718,0xd86b071a,0xdd1f071a,0xd86b071c,0xdd22071c,0xd86b071e,
	0xdd23071e,0xd86b0720,0xdd220720,0xd86b0722,0xdd230722,0xd86b0724,0xdd260724,0xd86b0726,
	0xdd270726,0xd86b0728,0xdd260728,0xd86b072a,0xdd27072a,0xd86b072c,0xdd2a072c,0xd86b072e,
	0xdd2b072e,0xd86b0730,0xdd2a0730,0xd86b0732,0xdd2b0732,0xd86b0734,0xdd2e0734,0xd86b0736,
	0xdd2f0736,0xd86b0738,0xdd2e0738,0xd86b073a,0xdd2f073a,0xd86b073c,0xdd32073c,0xd86b073e,
	0xdd33073e,0xd86b0740,0xdd320740,0xd86b0742,0xdd330742,0xd86b0744,0xdd360744,0xd86b0746,
	0xdd370746,0xd86b0748,0xdd360748,0xd86b074a,0xdd37074a,0xd86b074c,0xdd3a074c,0xd86b074e,
	0xdd3b074e,0xd86b0750,0xdd3a0750,0xd86b0752,0xdd3b0752,0xd86b0754,0xdd3e0754,0xd86b0756,
	0xdd3f0756,0xd86b0758,0xdd3e0758,0xd86b075a,0xdd3f075a,0xd86b075c,0xdd42075c,0xd86b075e,
	0xdd43075e,0xd86b0760,0xdd420760,0xd86b0762,0xdd430762,0xd86b0764,0xdd460764,0xd86b0766,
	0xdd470766,0xd86b0768,0xdd460768,0xd86b076a,0xdd47076a,0xd86b076c,0xdd4a076c,0xd86b076e,
	0xdd4b076e,0xd86b0770,0xdd4a0770,0xd86b0772,0xdd4b0772,0xd86b0774,0xdd4e0774,0xd86b0776,
	0xdd4f0776,0xd86b0778,0xdd4e0778,0xd86b077a,0xdd4f077a,0xd86b077c,0xdd52077c,0xd86b077e,
	0xdd53077e,0xd86b0780,0xdd520780,0xd86b0782,0xdd530782,0xd86b0784,0xdd560784,0xd86b0786,
	0xdd570786,0xd86b0788,0xdd560788,0xd86b078a,0xdd57078a,0xd86b078c,0xdd5a078c,0xd86b078e,
	0xdd5b078e,0xd86b0790,0xdd5a0790,0xd86b0792,0xdd5b0792,0xd86b0794,0xdd5e0794,0xd86b0796,
	0xdd5f0796,0xd86b0798,0xdd5e0798,0xd86b079a,0xdd5f079a,0xd86b079c,0xdd62079c,0xd86b079e,
	0xdd63079e,0xd86b07a0,0xdd6207a0,0xd86b07a2,0xdd6307a2,0xd86b07a4,0xdd6607a4,0xd86b07a6,
	0xdd6707a6,0xd86b07a8,0xdd6607a8,0xd86b07aa,0xdd6707aa,0xd86b07ac,0xdd6a07ac,0xd86b07ae,
	0xdd6b07ae,0xd86b07b0,0xdd6a07b0,0xd86b07b2,0xdd6b07b2,0xd86b07b4,0xdd6e07b4,0xd86b07b6,
	0xdd6f07b6,0xd86b07b8,0xdd6e07b8,0xd86b07ba,0xdd6f07ba,0xd86b07bc,0xdd7207bc,0xd86b07be,
	0xdd7307be,0xd86b07c0,0xdd7207c0,0xd86b07c2,0xdd7307c2,0xd86b07c4,0xdd7607c4,0xd86b07c6,
	0xdd7707c6,0xd86b07c8,0xdd7607c8,0xd86b07ca,0xdd7707ca,0xd86b07cc,0xdd7a07cc,0xd86b07ce,
	0xdd7b07ce,0xd86b07d0,0xdd7a07d0,0xd86b07d2,0xdd7b07d2,0xd86b07d4,0xdd7e07d4,0xd86b07d6,
	0xdd7f07d6,0xd86b07d8,0xdd7e07d8,0xd86b07da,0xdd7f07da,0xd86b07dc,0xdd8207dc,0xd86b07de,
	0xdd8307de,0xd86b07e0,0xdd8207e0,0xd86b07e2,0xdd8307e2,0xd86b07e4,0xdd8607e4,0xd86b07e6,
	0xdd8707e6,0xd86b07e8,0xdd8607e8,0xd86b07ea,0xdd8707ea,0xd86b07ec,0xdd8a07ec,0xd86b07ee,
	0xdd8b07ee,0xd86b07f0,0xdd8a07f0,0xd86b07f2,0xdd8b07f2,0xd86b07f4,0xdd8e07f4,0xd86b07f6,
	0xdd8f07f6,0xd86b07f8,0xdd8e07f8,0xd86b07fa,0xdd8f07fa,0xd86b07fc,0xdd9207fc,0xd86b07fe,
	0xdd9307fe,0xd86b0800,0xdd920800,0xd86b0802,0xdd930802,0xd86b0804,0xdd960804,0xd86b0806,
	0xdd970806,0xd86b0808,0xdd960808,0xd86b080a,0xdd97080a,0xd86b080c,0xdd9a080c,0xd86b080e,
	0xdd9b080e,0xd86b0810,0xdd9a0810,0xd86b0812,0xdd9b0812,0xd86b0814,0xdd9e0814,0xd86b0816,
	0xdd9f0816,0xd86b0818,0xdd9e0818,0xd86b081a,0xdd9f081a,0xd86b081c,0xdda2081c,0xd86b081e,
	0xdda3081e,0xd86b0820,0xdda20820,0xd86b0822,0xdda30822,0xd86b0824,0xdda60824,0xd86b0826,
	0xdda70826,0xd86b0828,0xdda60828,0xd86b082a,0xdda7082a,0xd86b082c,0xddaa082c,0xd86b082e,
	0xddab082e,0xd86b0830,0xddaa0830,0xd86b0832,0xddab0832,0xd86b0834,0xddae0834,0xd86b0836,
	0xddaf0836,0xd86b0838,0xddae0838,0xd86b083a,0xddaf083a,0xd86b083c,0xddb2083c,0xd86b083e,
	0xddb3083e,0xd86b0840,0xddb20840,0xd86b0842,0xddb30842,0xd86b0844,0xddb60844,0xd86b0846,
	0xddb70846,0xd86b0848,0xddb60848,0xd86b084a,0xddb7084a,0xd86b084c,0xddba084c,0xd86b084e,
	0xddbb084e,0xd86b0850,0xddba0850,0xd86b0852,0xddbb0852,0xd86b0854,0xddbe0854,0xd86b0856,
	0xddbf0856,0xd86b0858,0xddbe0858,0xd86b085a,0xddbf085a,0xd86b085c,0xddc2085c,0xd86b085e,
	0xddc3085e,0xd86b0860,0xddc20860,0xd86b0862,0xddc30862,0xd86b0864,0xddc60864,0xd86b0866,
	0xddc70866,0xd86b0868,0xddc60868,0xd86b086a,0xddc7086a,0xd86b086c,0xddca086c,0xd86b086e,
	0xddcb086e,0xd86b0870,0xddca0870,0xd86b0872,0xddcb0872,0xd86b0874,0xddce0874,0xd86b0876,
	0xddcf0876,0xd86b0878,0xddce0878,0xd86b087a,0xddcf087a,0xd86b087c,0xddd2087c,0xd86b087e,
	0xddd3087e,0xd86b0880,0xddd20880,0xd86b0882,0xddd30882,0xd86b0884,0xddd60884,0xd86b0886,
	0xddd70886,0xd86b0888,0xddd60888,0xd86b088a,0xddd7088a,0xd86b088c,0xddda088c,0xd86b088e,
	0xdddb088e,0xd86b0890,0xddda0890,0xd86b0892,0xdddb0892,0xd86b0894,0xddde0894,0xd86b0896,
	0xdddf0896,0xd86b0898,0xddde0898,0xd86b089a,0xdddf089a,0xd86b089c,0xdde2089c,0xd86b089e,
	0xdde3089e,0xd86b08a0,0xdde208a0,0xd86b08a2,0xdde308a2,0xd86b08a4,0xdde608a4,0xd86b08a6,
	0xdde708a6,0xd86b08a8,0xdde608a8,0xd86b08aa,0xdde708aa,0xd86b08ac,0xddea08ac,0xd86b08ae,
	0xddeb08ae,0xd86b08b0,0xddea08b0,0xd86b08b2,0xddeb08b2,0xd86b08b4,0xddee08b4,0xd86b08b6,
	0xddef08b6,0xd86b08b8,0xddee08b8,0xd86b08ba,0xddef08ba,0xd86b08bc,0xddf208bc,0xd86b08be,
	0xddf308be,0xd86b08c0,0xddf208c0,0xd86b08c2,0xddf308c2,0xd86b08c4,0xddf608c4,0xd86b08c6,
	0xddf708c6,0xd86b08c8,0xddf608c8,0xd86b08ca,0xddf708ca,0xd86b08cc,0xddfa08cc,0xd86b08ce,
	0xddfb08ce,0xd86b08d0,0xddfa08d0,0xd86b08d2,0xddfb08d2,0xd86b08d4,0xddfe08d4,0xd86b08d6,
	0xddff08d6,0xd86b08d8,0xddfe08d8,0xd86b08da,0xddff08da,0xd86b08dc,0xde0208dc,0xd86b08de,
	0xde0308de,0xd86b08e0,0xde0208e0,0xd86b08e2,0xde0308e2,0xd86b08e4,0xde0608e4,0xd86b08e6,
	0xde0708e6,0xd86b08e8,0xde0608e8,0xd86b08ea,0xde0708ea,0xd86b08ec,0xde0a08ec,0xd86b08ee,
	0xde0b08ee,0xd86b08f0,0xde0a08f0,0xd86b08f2,0xde0b08f2,0xd86b08f4,0xde0e08f4,0xd86b08f6,
	0xde0f08f6,0xd86b08f8,0xde0e08f8,0xd86b08fa,0xde0f08fa,0xd86b08fc,0xde1208fc,0xd86b08fe,
	0xde1308fe,0xd86b0900,0xde120900,0xd86b0902,0xde130902,0xd86b0904,0xde160904,0xd86b0906,
	0xde170906,0xd86b0908,0xde160908,0xd86b090a,0xde17090a,0xd86b090c,0xde1a090c,0xd86b090e,
	0xde1b090e,0xd86b0910,0xde1a0910,0xd86b0912,0xde1b0912,0xd86b0914,0xde1e0914,0xd86b0916,
	0xde1f0916,0xd86b0918,0xde1e0918,0xd86b091a,0xde1f091a,0xd86b091c,0xde22091c,0xd86b091e,
	0xde23091e,0xd86b0920,0xde220920,0xd86b0922,0xde230922,0xd86b0924,0xde260924,0xd86b0926,
	0xde270926,0xd86b0928,0xde260928,0xd86b092a,0xde27092a,0xd86b092c,0xde2a092c,0xd86b092e,
	0xde2b092e,0xd86b0930,0xde2a0930,0xd86b0932,0xde2b0932,0xd86b0934,0xde2e0934,0xd86b0936,
	0xde2f0936,0xd86b0938,0xde2e0938,0xd86b093a,0xde2f093a,0xd86b093c,0xde32093c,0xd86b093e,
	0xde33093e,0xd86b0940,0xde320940,0xd86b0942,0xde330942,0xd86b0944,0xde360944,0xd86b0946,
	0xde370946,0xd86b0948,0xde360948,0xd86b094a,0xde37094a,0xd86b094c,0xde3a094c,0xd86b094e,
	0xde3b094e,0xd86b0950,0xde3a0950,0xd86b0952,0xde3b0952,0xd86b0954,0xde3e0954,0xd86b0956,
	0xde3f0956,0xd86b0958,0xde3e0958,0xd86b095a,0xde3f095a,0xd86b095c,0xde42095c,0xd86b095e,
	0xde43095e,0xd86b0960,0xde420960,0xd86b0962,0xde430962,0xd86b0964,0xde460964,0xd86b0966,
	0xde470966,0xd86b0968,0xde460968,0xd86b096a,0xde47096a,0xd86b096c,0xde4a096c,0xd86b096e,
	0xde4b096e,0xd86b0970,0xde4a0970,0xd86b0972,0xde4b0972,0xd86b0974,0xde4e0974,0xd86b0976,
	0xde4f0976,0xd86b0978,0xde4e0978,0xd86b097a,0xde4f097a,0xd86b097c,0xde52097c,0xd86b097e,
	0xde53097e,0xd86b0980,0xde520980,0xd86b0982,0xde530982,0xd86b0984,0xde560984,0xd86b0986,
	0xde570986,0xd86b0988,0xde560988,0xd86b098a,0xde57098a,0xd86b098c,0xde5a098c,0xd86b098e,
	0xde5b098e,0xd86b0990,0xde5a0990,0xd86b0992,0xde5b0992,0xd86b0994,0xde5e0994,0xd86b0996,
	0xde5f0996,0xd86b0998,0xde5e0998,0xd86b099a,0xde5f099a,0xd86b099c,0xde62099c,0xd86b099e,
	0xde63099e,0xd86b09a0,0xde6209a0,0xd86b09a2,0xde6309a2,0xd86b09a4,0xde6609a4,0xd86b09a6,
	0xde6709a6,0xd86b09a8,0xde6609a8,0xd86b09aa,0xde6709aa,0xd86b09ac,0xde6a09ac,0xd86b09ae,
	0xde6b09ae,0xd86b09b0,0xde6a09b0,0xd86b09b2,0xde6b09b2,0xd86b09b4,0xde6e09b4,0xd86b09b6,
	0xde6f09b6,0xd86b09b8,0xde6e09b8,0xd86b09ba,0xde6f09ba,0xd86b09bc,0xde7209bc,0xd86b09be,
	0xde7309be,0xd86b09c0,0xde7209c0,0xd86b09c2,0xde7309c2,0xd86b09c4,0xde7609c4,0xd86b09c6,
	0xde7709c6,0xd86b09c8,0xde7609c8,0xd86b09ca,0xde7709ca,0xd86b09cc,0xde7a09cc,0xd86b09ce,
	0xde7b09ce,0xd86b09d0,0xde7a09d0,0xd86b09d2,0xde7b09d2,0xd86b09d4,0xde7e09d4,0xd86b09d6,
	0xde7f09d6,0xd86b09d8,0xde7e09d8,0xd86b09da,0xde7f09da,0xd86b09dc,0xde8209dc,0xd86b09de,
	0xde8309de,0xd86b09e0,0xde8209e0,0xd86b09e2,0xde8309e2,0xd86b09e4,0xde8609e4,0xd86b09e6,
	0xde8709e6,0xd86b09e8,0xde8609e8,0xd86b09ea,0xde8709ea,0xd86b09ec,0xde8a09ec,0xd86b09ee,
	0xde8b09ee,0xd86b09f0,0xde8a09f0,0xd86b09f2,0xde8b09f2,0xd86b09f4,0xde8e09f4,0xd86b09f6,
	0xde8f09f6,0xd86b09f8,0xde8e09f8,0xd86b09fa,0xde8f09fa,0xd86b09fc,0xde9209fc,0xd86b09fe,
	0xde9309fe,0xd86b0a00,0xde920a00,0xd86b0a02,0xde930a02,0xd86b0a04,0xde960a04,0xd86b0a06,
	0xde970a06,0xd86b0a08,0xde960a08,0xd86b0a0a,0xde970a0a,0xd86b0a0c,0xde9a0a0c,0xd86b0a0e,
	0xde9b0a0e,0xd86b0a10,0xde9a0a10,0xd86b0a12,0xde9b0a12,0xd86b0a14,0xde9e0a14,0xd86b0a16,
	0xde9f0a16,0xd86b0a18,0xde9e0a18,0xd86b0a1a,0xde9f0a1a,0xd86b0a1c,0xdea20a1c,0xd86b0a1e,
	0xdea30a1e,0xd86b0a20,0xdea20a20,0xd86b0a22,0xdea30a22,0xd86b0a24,0xdea60a24,0xd86b0a26,
	0xdea70a26,0xd86b0a28,0xdea60a28,0xd86b0a2a,0xdea70a2a,0xd86b0a2c,0xdeaa0a2c,0xd86b0a2e,
	0xdeab0a2e,0xd86b0a30,0xdeaa0a30,0xd86b0a32,0xdeab0a32,0xd86b0a34,0xdeae0a34,0xd86b0a36,
	0xdeaf0a36,0xd86b0a38,0xdeae0a38,0xd86b0a3a,0xdeaf0a3a,0xd86b0a3c,0xdeb20a3c,0xd86b0a3e,
	0xdeb30a3e,0xd86b0a40,0xdeb20a40,0xd86b0a42,0xdeb30a42,0xd86b0a44,0xdeb60a44,0xd86b0a46,
	0xdeb70a46,0xd86b0a48,0xdeb60a48,0xd86b0a4a,0xdeb70a4a,0xd86b0a4c,0xdeba0a4c,0xd86b0a4e,
	0xdebb0a4e,0xd86b0a50,0xdeba0a50,0xd86b0a52,0xdebb0a52,0xd86b0a54,0xdebe0a54,0xd86b0a56,
	0xdebf0a56,0xd86b0a58,0xdebe0a58,0xd86b0a5a,0xdebf0a5a,0xd86b0a5c,0xdec20a5c,0xd86b0a5e,
	0xdec30a5e,0xd86b0a60,0xdec20a60,0xd86b0a62,0xdec30a62,0xd86b0a64,0xdec60a64,0xd86b0a66,
	0xdec70a66,0xd86b0a68,0xdec60a68,0xd86b0a6a,0xdec70a6a,0xd86b0a6c,0xdeca0a6c,0xd86b0a6e,
	0xdecb0a6e,0xd86b0a70,0xdeca0a70,0xd86b0a72,0xdecb0a72,0xd86b0a74,0xdece0a74,0xd86b0a76,
	0xdecf0a76,0xd86b0a78,0xdece0a78,0xd86b0a7a,0xdecf0a7a,0xd86b0a7c,0xded20a7c,0xd86b0a7e,
	0xded30a7e,0xd86b0a80,0xded20a80,0xd86b0a82,0xded30a82,0xd86b0a84,0xded60a84,0xd86b0a86,
	0xded70a86,0xd86b0a88,0xded60a88,
	};	 // 113027 entries

static const TCollationKeyTable TheGB18030Table = 
	{ TheGB18030Key, TheGB18030Index, 113027, 0, 0, 0 };

static const TCollationMethod TheGB18030Method = 
	{
		0, // the method for the locale
		NULL, // use the standard table as the main table
		&TheGB18030Table, // the locale values override the standard values
		0 // the flags are standard
	};
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////

void test_unicode_collations()
	{
	/*
	Note: the curious method of getting values greater than 255 into the strings is needed
	because GCC doesn't handle the \x notation properly: it zeroes the high byte, so
	L"\x2011" puts the value 0x0011, not 0x2011, into the string.
	*/
	TBuf<32> x, y;

	test.Next(_L("collation"));
	test_unicode_collation(_L("Graham"),_L("Graham"),0);		// equality
	test_unicode_collation(_L("something"),_L(""),1);			// something after nothing
	test_unicode_collation(_L("inform"),_L("information"),-1);	// prefixes come first
 	test_unicode_collation(_L("\x30CB\x30e5\x30fc"),_L("\x30CB\x30e5\x30fc\x30e0"),-1); // Katakana 
	test_unicode_collation(_L("\x3042\x308a"),_L("\x3042\x308a\x304c\x3068"),-1); // Hiragana
	test_unicode_collation(_L("unicode"),_L("consortium"),1);	// standard alphabetic ordering: lowercase
	test_unicode_collation(_L("AARON"),_L("AARDVARK"),1);		// standard alphabetic ordering: uppercase
	test_unicode_collation(_L("AARON"),_L("AARDVARK"),1,0);		// alphabetic ordering is significant at level 0
	test_unicode_collation(_L("hello"),_L("Hello"),1,3,&TheSwapCaseMethod);			// lowercase before uppercase
	test_unicode_collation(_L("hello"),_L("Hello"),-1,2);		// case is significant at level 2
	//test_unicode_collation(_L("hello"),_L("Hello"),0,1);		// ignore case at level 1
	test_unicode_collation(_L("hello"),_L("Hello"),0,1);
	test_unicode_collation(_L("hello"),_L("[H E L L O !]"),0,1);// ignore punctuation and case at level 1
	test_unicode_collation(_L("1234"),_L("abc"),-1);			// digits before letters
	x = _L("week day"); x[4] = 0x2011;							// (week-day with non-breaking hyphen)
	test_unicode_collation(x,_L("weekend"),-1);					// ignore punctuation if letters differ; d < e
	test_unicode_collation(x,_L("week-day"),1);					// compare punctuation if otherwise equal
	test_unicode_collation(x,_L("week-day"),0,2);				// ignore difference between hyphens at level 2
	test_unicode_collation(x,_L("Week-day"),-1);				// case difference outweighs punctuation ordering
	y = _L("a ngstro m"); y[1] = 0x30a; y[8] = 0x308;			// (a<ring>ngstro<umlaut>m with piece accents)
	test_unicode_collation(_L("\xe5ngstr\xf6m"),y,0);			// base+accent = precomposed (a<ring>ngstro<umlaut>m)
	test_unicode_collation(_L("Liege"),_L("Li\xe8ge"),-1);		// unaccented before accented...
	test_unicode_collation(_L("Liege"),_L("Li\xe8ge"),-1,1);	// accents are significant at level 1
	test_unicode_collation(_L("Liege"),_L("li\xe8ge"),-1);		// accentuation outweighs case
	test_unicode_collation(_L("Liege"),_L("li\xe8ge"),0,0);		// ignore accents and case at level 0
	test_unicode_collation(_L("\xe5ngstr\xf6m"),_L("zoo"),-1);	// a-ring before z by default
#undef USE_SWEDISH_TABLE
#ifdef USE_SWEDISH_TABLE
	test_unicode_collation(_L("\xe5ngstr\xf6m"),_L("zoo"),1,3,&TheSwedishMethod);	// a-ring after z in Sweden
#endif
	test_unicode_collation(_L("Antidisestablishmentarian"),_L("antidisestablishmentariac"),1); 	// long strings that don't have identical prefixes
	
	test_unicode_collation(_L("\xFF86"),_L(""),1);
	test_unicode_collation(_L("\x30B9"),_L(""),1);
	// Equality 
	test_unicode_collation(_L("\xFF71"),_L("\xFF71"),0,3,&TheKanaMethod);  // Same Halfwidth Katakana characters
	test_unicode_collation(_L("\x30A2"),_L("\x30A2"),0,3,&TheKanaMethod);  // Same Katakana characters
	test_unicode_collation(_L("\xFF71\xFF67"),_L("\xFF71\xFF67"),0,3,&TheKanaMethod); // Same katakana Characters with small letter
	test_unicode_collation(_L("\x30b9\x30af\x30ea\x30fc\x30e0"),_L("\x30b9\x30af\x30ea\x30fc\x30e0"),0,3,&TheKanaMethod); // Scream in Katakana
	test_unicode_collation(_L("\x3042"),_L("\x3042"),0,3,&TheKanaMethod);  // same Hiragana characters
	test_unicode_collation(_L("\x3042\x308A"),_L("\x3042\x308A"),0,3,&TheKanaMethod); 
	test_unicode_collation(_L("\x304C"),_L("\x304B\x3099"),0,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3072\x3063\x304f"),_L("\x3072\x3063\x304f"),0,3,&TheKanaMethod); // Hiragana word Hyaku
	// 1/2 width Katakana is sorted before Katakana 
	test_unicode_collation(_L("\xFF71"),_L("\x30A2"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x30A2"),_L("\xFF71"), 1,3,&TheKanaMethod);
	test_unicode_collation(_L("\xFF86\xFF6D\xFF70\xFF7D"),_L("\x30CB\x30E5\x30FC\x30B9"),-1,3,&TheKanaMethod); // News Katakan 1/2 width & full width..this shoule be -1
	// 1/2 width katakana sorted before half width with small character;
	test_unicode_collation(_L("\xFF71"),_L("\xFF71\xFF67"),-1,3,&TheKanaMethod);  // same katakana characcter, one with a small character
	test_unicode_collation(_L("\xFF77"),_L("\xFF71\xFF6C"),1,3,&TheKanaMethod); // different Katakana character, one with small character 
	// Hiragana before Katakana & 1/2 width Katakana
	test_unicode_collation(_L("\x3042"),_L("\x30a2"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3042"),_L("\x30a2"),1,3,&TheSwapKanaMethod);
	test_unicode_collation(_L("\x3042"),_L("\x30cb"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x308a"),_L("\x30a2\x30ea\x30e0"),1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3042\x308a"),_L("\x30a2\x30ea"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3042\x308a\x304c\x3068"),_L("\xFF71\xFF86\xFF6D\xFF70\xff7D"),1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3042"),_L("\xFF71"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3042\x308a"),_L("\x30a2\x30ea\x30e0\x30b9"),-1,3,&TheKanaMethod);

	// Test Katakana Nigori and Maru Sort sequence
	test_unicode_collation(_L("\xff8a"),_L("\xff8A\xff9e"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\xff8a"),_L("\xff8a\xff9f"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\xff8a\xff9e"),_L("\xff8a\xff9f"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x30cf"),_L("\x30d0"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x30cf"),_L("\x30d1"),-1,3,&TheKanaMethod); 
	test_unicode_collation(_L("\x30d1"),_L("\x30d0"),1,3,&TheKanaMethod); 
	test_unicode_collation(_L("\x306f"),_L("\x3070"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x306f"),_L("\x3071"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3070"),_L("\x3071"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\xff8a"),_L("\x30cf"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3071"),_L("\x3068"),1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x306f"),_L("\x30d1"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x306f"),_L("\xff8a"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3070"),_L("\xff8a\xff9e"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3071"),_L("\xff8a\xff9f"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3042\x304e\x3089"),_L("\x3044"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3070"),_L("\x30cf"),1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x30cf"),_L("\x3070"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3070"),_L("\xff8a"),1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x3071"),_L("\xff8a"),1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x30d1"),_L("\x3071"),1,3,&TheKanaMethod);
	
	test_unicode_collation(_L("\x30ac"),_L("\x30ab"),1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x30a2\x30ac"),_L("\x30a2\x30ab"),1,3,&TheKanaMethod); // ka comes before Ga
	test_unicode_collation(_L("\x306b\x3058\x3085"),_L("\x306b\x3057\x308c"),-1,3,&TheKanaMethod);
	
	// Triplet support test 
	test_unicode_collation(_L("\x30ac\x30fc"),_L("\x30ae\x30fc"),-1,3,&TheKanaMethod);
	test_unicode_collation(_L("\x30ab\x30fc"),_L("\x30ab\x30fc"),0,3,&TheKanaMethod);
	test_unicode_collation(_L("\x30ab\x30a4"),_L("\x30ab\x30fc"),1,3,&TheKanaMethod);  
	test_unicode_collation(_L("\x30ab\x30a4"),_L("\x30ac"),1,3,&TheKanaMethod);
	
	//Simplified Chinese test starts. Note that the Chinese collation table only includes the ideological characters. 
	//The characters which are in the base table should come to the front while the characters which are not either 
	//in base table or client table should come after
	test.Next(_L("Simplified Chinese test starts"));
	test_unicode_collation(_L("\x554a"),_L("\x4f4f"),-1,3,&TheChineseMethod);//different sound
	test_unicode_collation(_L("\x54ed"),_L("\x5757"),-1,3,&TheChineseMethod);//different sound
	test_unicode_collation(_L("\x8d30"),_L("\x82ac"),-1,3,&TheChineseMethod);//different sound
	test_unicode_collation(_L("\x62bd"),_L("\x6577"),-1,3,&TheChineseMethod);//different sound
	test_unicode_collation(_L("\x75c5"),_L("\x7e94"),-1,3,&TheChineseMethod);//different sound
//same sound
	test_unicode_collation(_L("\x78e8"),_L("\x672b"),-1,3,&TheChineseMethod);//different tones
	test_unicode_collation(_L("\x5e9c"),_L("\x7236"),-1,3,&TheChineseMethod);//different tones
	test_unicode_collation(_L("\x7840"),_L("\x5904"),-1,3,&TheChineseMethod);//different tones
	test_unicode_collation(_L("\x5976"),_L("\x8010"),-1,3,&TheChineseMethod);//different tones
	test_unicode_collation(_L("\x7720"),_L("\x9762"),-1,3,&TheChineseMethod);//different tones
//same sound and stroke count
	test_unicode_collation(_L("\x5757"),_L("\x5feb"),-1,3,&TheChineseMethod);//different first stroke
	test_unicode_collation(_L("\x51b3"),_L("\x8bc0"),-1,3,&TheChineseMethod);//different first stroke
	test_unicode_collation(_L("\x5316"),_L("\x4ee4"),-1,3,&TheChineseMethod);//different first stroke
	test_unicode_collation(_L("\x62df"),_L("\x4f60"),1,3,&TheChineseMethod);//different radical
	test_unicode_collation(_L("\x6d1b"),_L("\x9a86"),-1,3,&TheChineseMethod);//different first stroke
//smae sound and same tones
	test_unicode_collation(_L("\x5757"),_L("\x4fa9"),-1,3,&TheChineseMethod);//different stroke count
	test_unicode_collation(_L("\x51e1"),_L("\x7e41"),-1,3,&TheChineseMethod);//different stroke count
	test_unicode_collation(_L("\x98ce"),_L("\x950b"),-1,3,&TheChineseMethod);//different stroke count
	test_unicode_collation(_L("\x5e01"),_L("\x5e87"),-1,3,&TheChineseMethod);//different stroke count
	test_unicode_collation(_L("\x529e"),_L("\x534a"),-1,3,&TheChineseMethod);//different stroke count
//compare the Chinese ideological characters to other characters which are not included in the Chinese collation table
	test_unicode_collation(_L("abc"),_L("\x4f4f"),-1,3,&TheChineseMethod);//compare alphabets to Chinese character
	test_unicode_collation(_L("zxc"),_L("\x4f4f"),-1,3,&TheChineseMethod);//compare alphabets to Chinese character
	test_unicode_collation(_L("\x3041"),_L("\x4f4f"),1,3,&TheChineseMethod);//compare a Hiragana to Chinese characters
	test_unicode_collation(_L("\x30f6"),_L("\x4f4f"),1,3,&TheChineseMethod);//compare a Katakana to Chinese characters
	test_unicode_collation(_L("\x3041"),_L("\x30f6"),-1,3,&TheChineseMethod);//compare a Hiragana to a Katakana
//compare two group of chinese characters
	test_unicode_collation(_L("\x8584\x554a"),_L("\x8584\x4f4f"),-1,3,&TheChineseMethod);//the first chinese character is the same
	test_unicode_collation(_L("\x5b89\x95ed"),_L("\x5b89\x6b65"),-1,3,&TheChineseMethod);//the first chinese character is the same
	test_unicode_collation(_L("\x60b2\x7c3f"),_L("\x60b2\x5e9c"),-1,3,&TheChineseMethod);//the first chinese character is the same
	test_unicode_collation(_L("\x8336\x5e01"),_L("\x8336\x5e87"),-1,3,&TheChineseMethod);//the first chinese character is the same
	test_unicode_collation(_L("\x4ea7\x529e"),_L("\x4ea7\x534a"),-1,3,&TheChineseMethod);//the first chinese character is the same

	test_unicode_collation(_L("\x3041\x554a"),_L("\x3041\x4f4f"),-1,3,&TheChineseMethod);//the first Hiragana is the same
	test_unicode_collation(_L("\x3063\x6bd4"),_L("\x3063\x660c"),-1,3,&TheChineseMethod);//the first Hiragana is the same
	test_unicode_collation(_L("\x30b7\x5999"),_L("\x30b7\x6728"),-1,3,&TheChineseMethod);//the first Katakana is the same
	test_unicode_collation(_L("\x3041\x534a"),_L("\x3041\x907e"),-1,3,&TheChineseMethod);//the first Katakana is the same

	test_unicode_collation(_L("\x3041\x554a"),_L("\x30f6\x4f4f"),-1,3,&TheChineseMethod);//the first symbole is different
	test_unicode_collation(_L("\x3041\x4fbc"),_L("\x3063\x5068"),-1,3,&TheChineseMethod);//the first symbole is different
	test_unicode_collation(_L("\x30b7\x6478"),_L("\x30c2\x51dd"),-1,3,&TheChineseMethod);//the first symbole is different
	test_unicode_collation(_L("\x3046\x5ff5"),_L("\x30ea\x5b81"),-1,3,&TheChineseMethod);//the first symbole is different
	test_unicode_collation(_L("\x3072\x7aed"),_L("\x254b\x670d"),1,3,&TheChineseMethod);//the first symbole is different
//compare full width and half width digits and letters
	test_unicode_collation(_L("\xff11"),_L("\x0031"),1,3,&TheChineseMethod);//full width "1" comes after half width "1"
	test_unicode_collation(_L("\x0032"),_L("\xff11"),1,3,&TheChineseMethod);//half width "2" comes after full width "1"
	test_unicode_collation(_L("\xff21"),_L("\x0061"),1,3,&TheChineseMethod);//full width "a" comes after half width "a"
	test_unicode_collation(_L("\x0062"),_L("\xff21"),1,3,&TheChineseMethod);//half width "b" comes after full width "a"
	

//----------------------------------------------------------------------------------------------

/**
   @SYMTestCaseID           TI18N-COLLATION-CIT-4006
   @SYMPREQ                 2471
   @SYMTestCaseDesc         Check collation mechanism works on new characters
   @SYMTestPriority         High 
   @SYMTestStatus           Implemented
   @SYMTestActions          1. Select one-byte character from GB2312-80, get its corresponding Unicode
                            2. Pass the Unicode to descriptor's collation member
                            3. Select one-byte character from GB2312-80, shape changed, get its corresponding Unicode
                            4. Pass the Unicode to descriptor's collation member
                            5. Select two-byte character from GB2312-80, get its corresponding Unicode
                            6. Pass the Unicode to descriptor's collation member
                            7. Select two-byte character from GBK, outside of GB2312-80, same with GBK, get its corresponding Unicode
                            8. Pass the Unicode to descriptor's collation member
                            9. Select two-byte character from GBK, outside of GB2312-80, different from GBK,  get its corresponding Unicode
                            10. Pass the Unicode to descriptor's collation member
                            11. Select four-byte character inside of BMP, get its corresponding Unicode
                            12. Pass the Unicode to descriptor's collation member
                            13. Select four-byte character outside of BMP, get its corresponding Unicode
                            14. Pass the Unicode to descriptor's collation member
   @SYMTestExpectedResults  Collation works according to Unicode Standard
 */
//----------------------------------------------------------------------------------------------
	//GB18030 Test from here. First rerun the GB2312 cases. Should be compatible
	test.Next(_L("GB18030 Chinese test starts"));
	test_unicode_collation(_L("\x554a"),_L("\x4f4f"),-1,3,&TheGB18030Method);//different sound
	test_unicode_collation(_L("\x54ed"),_L("\x5757"),-1,3,&TheGB18030Method);//different sound
	test_unicode_collation(_L("\x8d30"),_L("\x82ac"),-1,3,&TheGB18030Method);//different sound
	test_unicode_collation(_L("\x62bd"),_L("\x6577"),-1,3,&TheGB18030Method);//different sound
	test_unicode_collation(_L("\x75c5"),_L("\x7e94"),-1,3,&TheGB18030Method);//different sound
//same sound
	test_unicode_collation(_L("\x78e8"),_L("\x672b"),-1,3,&TheGB18030Method);//different tones
	test_unicode_collation(_L("\x5e9c"),_L("\x7236"),-1,3,&TheGB18030Method);//different tones
	test_unicode_collation(_L("\x7840"),_L("\x5904"),-1,3,&TheGB18030Method);//different tones
	test_unicode_collation(_L("\x5976"),_L("\x8010"),-1,3,&TheGB18030Method);//different tones
	test_unicode_collation(_L("\x7720"),_L("\x9762"),-1,3,&TheGB18030Method);//different tones
//same sound and stroke count
	test_unicode_collation(_L("\x5757"),_L("\x5feb"),-1,3,&TheGB18030Method);//different first stroke
	test_unicode_collation(_L("\x51b3"),_L("\x8bc0"),-1,3,&TheGB18030Method);//different first stroke
	test_unicode_collation(_L("\x5316"),_L("\x4ee4"),-1,3,&TheGB18030Method);//different first stroke
	test_unicode_collation(_L("\x62df"),_L("\x4f60"),1,3,&TheGB18030Method);//different radical
	test_unicode_collation(_L("\x6d1b"),_L("\x9a86"),-1,3,&TheGB18030Method);//different first stroke
//smae sound and same tones
	test_unicode_collation(_L("\x5757"),_L("\x4fa9"),-1,3,&TheGB18030Method);//different stroke count
	test_unicode_collation(_L("\x51e1"),_L("\x7e41"),-1,3,&TheGB18030Method);//different stroke count
	test_unicode_collation(_L("\x98ce"),_L("\x950b"),-1,3,&TheGB18030Method);//different stroke count
	test_unicode_collation(_L("\x5e01"),_L("\x5e87"),-1,3,&TheGB18030Method);//different stroke count
	test_unicode_collation(_L("\x529e"),_L("\x534a"),-1,3,&TheGB18030Method);//different stroke count
//compare the Chinese ideological characters to other characters which are not included in the Chinese collation table
	test_unicode_collation(_L("abc"),_L("\x4f4f"),-1,3,&TheGB18030Method);//compare alphabets to Chinese character
	test_unicode_collation(_L("zxc"),_L("\x4f4f"),-1,3,&TheGB18030Method);//compare alphabets to Chinese character
	test_unicode_collation(_L("\x3041"),_L("\x4f4f"),1,3,&TheGB18030Method);//compare a Hiragana to Chinese characters
	test_unicode_collation(_L("\x30f6"),_L("\x4f4f"),1,3,&TheGB18030Method);//compare a Katakana to Chinese characters
	test_unicode_collation(_L("\x3041"),_L("\x30f6"),-1,3,&TheGB18030Method);//compare a Hiragana to a Katakana
//compare two group of chinese characters
	test_unicode_collation(_L("\x8584\x554a"),_L("\x8584\x4f4f"),-1,3,&TheGB18030Method);//the first chinese character is the same
	test_unicode_collation(_L("\x5b89\x95ed"),_L("\x5b89\x6b65"),-1,3,&TheGB18030Method);//the first chinese character is the same
	test_unicode_collation(_L("\x60b2\x7c3f"),_L("\x60b2\x5e9c"),-1,3,&TheGB18030Method);//the first chinese character is the same
	test_unicode_collation(_L("\x8336\x5e01"),_L("\x8336\x5e87"),-1,3,&TheGB18030Method);//the first chinese character is the same
	test_unicode_collation(_L("\x4ea7\x529e"),_L("\x4ea7\x534a"),-1,3,&TheGB18030Method);//the first chinese character is the same

	test_unicode_collation(_L("\x3041\x554a"),_L("\x3041\x4f4f"),-1,3,&TheGB18030Method);//the first Hiragana is the same
	test_unicode_collation(_L("\x3063\x6bd4"),_L("\x3063\x660c"),-1,3,&TheGB18030Method);//the first Hiragana is the same
	test_unicode_collation(_L("\x30b7\x5999"),_L("\x30b7\x6728"),-1,3,&TheGB18030Method);//the first Katakana is the same
	test_unicode_collation(_L("\x3041\x534a"),_L("\x3041\x907e"),-1,3,&TheGB18030Method);//the first Katakana is the same

	test_unicode_collation(_L("\x3041\x554a"),_L("\x30f6\x4f4f"),-1,3,&TheGB18030Method);//the first symbole is different
	test_unicode_collation(_L("\x3041\x4fbc"),_L("\x3063\x5068"),-1,3,&TheGB18030Method);//the first symbole is different
	test_unicode_collation(_L("\x30b7\x6478"),_L("\x30c2\x51dd"),-1,3,&TheGB18030Method);//the first symbole is different
	test_unicode_collation(_L("\x3046\x5ff5"),_L("\x30ea\x5b81"),-1,3,&TheGB18030Method);//the first symbole is different
	test_unicode_collation(_L("\x3072\x7aed"),_L("\x254b\x670d"),1,3,&TheGB18030Method);//the first symbole is different
//compare full width and half width digits and letters
	test_unicode_collation(_L("\xff11"),_L("\x0031"),1,3,&TheGB18030Method);//full width "1" comes after half width "1"
	test_unicode_collation(_L("\x0032"),_L("\xff11"),1,3,&TheGB18030Method);//half width "2" comes after full width "1"
	test_unicode_collation(_L("\xff21"),_L("\x0061"),1,3,&TheGB18030Method);//full width "a" comes after half width "a"
	test_unicode_collation(_L("\x0062"),_L("\xff21"),1,3,&TheGB18030Method);//half width "b" comes after full width "a"

//compare the GB2312 ideological characters to GBK Character in CJK Unified
	test_unicode_collation(_L("\x4f4f"),_L("\x4e02"),-1,3,&TheGB18030Method);
	test_unicode_collation(_L("\x814b"),_L("\x696d"),-1,3,&TheGB18030Method);
//compare CJK Unified Both in GBK	
	test_unicode_collation(_L("\x696d"),_L("\x4e02"),1,3,&TheGB18030Method);
//Compare CJK Unified, Ext A and B
	test_unicode_collation(_L("\x3400"),_L("\x4f4f"),1,3,&TheGB18030Method);
	test_unicode_collation(_L("\x3400"),_L("\x4e02"),1,3,&TheGB18030Method);
	test_unicode_collation(_L("\x3530"),_L("\x4e02"),1,3,&TheGB18030Method);
	test_unicode_collation(_L("\x9fbb"),_L("\x3530"),-1,3,&TheGB18030Method);
	test_unicode_collation(_L("\xd840\xdc00"),_L("\x3530"),1,3,&TheGB18030Method);
	test_unicode_collation(_L("\xd840\xdc00"),_L("\xd869\xded6"),-1,3,&TheGB18030Method);
	
	//Compare String
	test_unicode_collation(_L("\x3041\x554a\x3400"),_L("\x30f6\x4f4f\xd840\xdc00"),-1,3,&TheGB18030Method);
	test_unicode_collation(_L("\xd840\xdc00\x3400"),_L("\xd840\xdc00\x4e02"),1,3,&TheGB18030Method);
	test_unicode_collation(_L("abc\x3041\x554a\x3400"),_L("ab\xd840\xdc00"),-1,3,&TheGB18030Method);
	test_unicode_collation(_L("\x3041\x554a"),_L("\x3041\xd840\xdc00"),-1,3,&TheGB18030Method);

	}

_LIT(KFoo1, "foo");
_LIT(KFoo2, "FOO");
_LIT(KFoo3, "Foo");
_LIT(KNotFoo1, "fo\x300o");
_LIT(KNotFoo2, "foo ");
_LIT(KNotFoo3, "foo\x300");
_LIT(KNotFoo4, "fo,o");
_LIT(KSigma1, "\x3C3");
_LIT(KSigma2, "\x3C2");
_LIT(KSigma3, "\x3A3");
_LIT(KPeach, "pe\x302\x63he");
_LIT(KFish, "pe\x301\x63he\x301");

TInt CompareIWrapper(const TDesC& a, const TDesC& b)
	{
#if 0
	return a.CompareIdentifiers(b);
#else
	TCollationMethod method;
	method.iMainTable = 0;
	method.iOverrideTable = 0;
	method.iFlags = TCollationMethod::EIgnoreNone | TCollationMethod::EFoldCase;
	return a.CompareC(b, 2, &method);
#endif
	}

void test_identifiers()
	{
	// test that comparison is as expected
	test(0 == CompareIWrapper(KFoo1, KFoo1));
	test(0 == CompareIWrapper(KFoo1, KFoo3));
	test(0 == CompareIWrapper(KFoo2, KFoo3));
	test(0 == CompareIWrapper(KFoo2, KFoo1));
	test(0 == CompareIWrapper(KFoo3, KFoo1));
	test(0 == CompareIWrapper(KFoo3, KFoo2));
	test(0 != CompareIWrapper(KFoo1, KNotFoo1));
	test(0 != CompareIWrapper(KNotFoo2, KFoo1));
	test(0 != CompareIWrapper(KFoo1, KNotFoo3));
	test(0 != CompareIWrapper(KNotFoo4, KFoo1));
	test(0 == CompareIWrapper(KSigma1, KSigma2));
	test(0 == CompareIWrapper(KSigma1, KSigma3));
	test(0 != CompareIWrapper(KPeach, KFish));
	}

// This whole test should first use the japanese collation lookup table and if not found what it was 
// looking for then it will search the basic standard table ... 


GLDEF_C TInt E32Main(void)
    {

	test.Title();
	test.Start(_L("TCollate Testing"));

	test_unicode_collations();
	test_identifiers();

	test.End();
	return(KErrNone);
    }

#pragma warning ( default :4700)
#pragma warning ( default :4710) 
